      SUBROUTINE MACHAR(IBETA,IT,IRND,NGRD,MACHEP,NEGEP,IEXP,MINEXP,
     *                  MAXEXP,EPS,EPSNEG,XMIN,XMAX)
C-----------------------------------------------------------------------
C     This Fortran 77 subroutine is intended to determine the parameters
C     of the floating-point arithmetic system specified below.  The
C     determination of the first three uses an extension of an algorithm
C    due to M. Malcolm, CACM 15 (1972), pp. 949-951, incorporating some,
C     but not all, of the improvements suggested by M. Gentleman and S.
C     Marovich, CACM 17 (1974), pp. 276-277.  An earlier version of this
C     program was published in the book Software Manual for the
C     Elementary Functions by W. J. Cody and W. Waite, Prentice-Hall,
C     Englewood Cliffs, NJ, 1980.  The present version is documented in
C     W. J. Cody, "MACHAR: A subroutine to dynamically determine machine
C     parameters," TOMS 14, December, 1988.
C
C     Parameter values reported are as follows:
C
C       IBETA   - the radix for the floating-point representation
C       IT      - the number of base IBETA digits in the floating-point
C                 significand
C       IRND    - 0 if floating-point addition chops
C                 1 if floating-point addition rounds, but not in the
C                   IEEE style
C                 2 if floating-point addition rounds in the IEEE style
C                 3 if floating-point addition chops, and there is
C                   partial underflow
C                 4 if floating-point addition rounds, but not in the
C                   IEEE style, and there is partial underflow
C                 5 if floating-point addition rounds in the IEEE style,
C                   and there is partial underflow
C       NGRD    - the number of guard digits for multiplication with
C                 truncating arithmetic.  It is
C                 0 if floating-point arithmetic rounds, or if it
C                   truncates and only  IT  base  IBETA digits
C                   participate in the post-normalization shift of the
C                   floating-point significand in multiplication;
C                 1 if floating-point arithmetic truncates and more
C                   than  IT  base  IBETA  digits participate in the
C                   post-normalization shift of the floating-point
C                   significand in multiplication.
C       MACHEP  - the largest negative integer such that
C                 1.0+FLOAT(IBETA)**MACHEP .NE. 1.0, except that
C                 MACHEP is bounded below by  -(IT+3)
C       NEGEPS  - the largest negative integer such that
C                 1.0-FLOAT(IBETA)**NEGEPS .NE. 1.0, except that
C                 NEGEPS is bounded below by  -(IT+3)
C       IEXP    - the number of bits (decimal places if IBETA = 10)
C                 reserved for the representation of the exponent
C                 (including the bias or sign) of a floating-point
C                 number
C       MINEXP  - the largest in magnitude negative integer such that
C                 FLOAT(IBETA)**MINEXP is positive and normalized
C       MAXEXP  - the smallest positive power of  BETA  that overflows
C       EPS     - the smallest positive floating-point number such
C                 that  1.0+EPS .NE. 1.0. In particular, if either
C                 IBETA = 2  or  IRND = 0, EPS = FLOAT(IBETA)**MACHEP.
C                 Otherwise,  EPS = (FLOAT(IBETA)**MACHEP)/2
C       EPSNEG  - A small positive floating-point number such that
C                 1.0-EPSNEG .NE. 1.0. In particular, if IBETA = 2
C                 or  IRND = 0, EPSNEG = FLOAT(IBETA)**NEGEPS.
C                 Otherwise,  EPSNEG = (IBETA**NEGEPS)/2.  Because
C                 NEGEPS is bounded below by -(IT+3), EPSNEG may not
C                 be the smallest number that can alter 1.0 by
C                 subtraction.
C       XMIN    - the smallest non-vanishing normalized floating-point
C                 power of the radix, i.e.,  XMIN = FLOAT(IBETA)**MINEXP
C       XMAX    - the largest finite floating-point number.  In
C                 particular  XMAX = (1.0-EPSNEG)*FLOAT(IBETA)**MAXEXP
C                 Note - on some machines  XMAX  will be only the
C                 second, or perhaps third, largest number, being
C                 too small by 1 or 2 units in the last digit of
C                 the significand.
C
C     Latest revision - December 4, 1987
C
C     Author - W. J. Cody
C              Argonne National Laboratory
C
C-----------------------------------------------------------------------
C     .. Scalar Arguments ..
      REAL EPS, EPSNEG, XMAX, XMIN
      INTEGER           IBETA, IEXP, IRND, IT, MACHEP, MAXEXP, MINEXP,
     *                  NEGEP, NGRD
C     .. Local Scalars ..
      REAL A, B, BETA, BETAH, BETAIN, ONE, T, TEMP, TEMP1,
     *                  TEMPA, TWO, Y, Z, ZERO
      INTEGER           I, ITEMP, IZ, J, K, MX, NXRES
C     .. Intrinsic Functions ..
      INTRINSIC         ABS, REAL, INT
C     .. Statement Functions ..
      REAL CONV
C     .. Statement Function definitions ..
C-----------------------------------------------------------------------
      CONV(I) = REAL(I)
C     .. Executable Statements ..
      ONE = CONV(1)
      TWO = ONE + ONE
      ZERO = ONE - ONE
C-----------------------------------------------------------------------
C     Determine IBETA, BETA ala Malcolm.
C-----------------------------------------------------------------------
      A = ONE
   20 A = A + A
      TEMP = A + ONE
      TEMP1 = TEMP - A
      IF (TEMP1-ONE.EQ.ZERO) GO TO 20
      B = ONE
   40 B = B + B
      TEMP = A + B
      ITEMP = INT(TEMP-A)
      IF (ITEMP.EQ.0) GO TO 40
      IBETA = ITEMP
      BETA = CONV(IBETA)
C-----------------------------------------------------------------------
C     Determine IT, IRND.
C-----------------------------------------------------------------------
      IT = 0
      B = ONE
   60 IT = IT + 1
      B = B*BETA
      TEMP = B + ONE
      TEMP1 = TEMP - B
      IF (TEMP1-ONE.EQ.ZERO) GO TO 60
      IRND = 0
      BETAH = BETA/TWO
      TEMP = A + BETAH
      IF (TEMP-A.NE.ZERO) IRND = 1
      TEMPA = A + BETA
      TEMP = TEMPA + BETAH
      IF ((IRND.EQ.0) .AND. (TEMP-TEMPA.NE.ZERO)) IRND = 2
C-----------------------------------------------------------------------
C     Determine NEGEP, EPSNEG.
C-----------------------------------------------------------------------
      NEGEP = IT + 3
      BETAIN = ONE/BETA
      A = ONE
      DO 80 I = 1, NEGEP
         A = A*BETAIN
   80 CONTINUE
      B = A
  100 TEMP = ONE - A
      IF (TEMP-ONE.NE.ZERO) GO TO 120
      A = A*BETA
      NEGEP = NEGEP - 1
      GO TO 100
  120 NEGEP = -NEGEP
      EPSNEG = A
C-----------------------------------------------------------------------
C     Determine MACHEP, EPS.
C-----------------------------------------------------------------------
      MACHEP = -IT - 3
      A = B
  140 TEMP = ONE + A
      IF (TEMP-ONE.NE.ZERO) GO TO 160
      A = A*BETA
      MACHEP = MACHEP + 1
      GO TO 140
  160 EPS = A
C-----------------------------------------------------------------------
C     Determine NGRD.
C-----------------------------------------------------------------------
      NGRD = 0
      TEMP = ONE + EPS
      IF ((IRND.EQ.0) .AND. (TEMP*ONE-ONE.NE.ZERO)) NGRD = 1
C-----------------------------------------------------------------------
C     Determine IEXP, MINEXP, XMIN.
C
C     Loop to determine largest I and K = 2**I such that
C         (1/BETA) ** (2**(I))
C     does not underflow.
C     Exit from loop is signaled by an underflow.
C-----------------------------------------------------------------------
      I = 0
      K = 1
      Z = BETAIN
      T = ONE + EPS
      NXRES = 0
  180 Y = Z
      Z = Y*Y
C-----------------------------------------------------------------------
C     Check for underflow here.
C-----------------------------------------------------------------------
      A = Z*ONE
      TEMP = Z*T
      IF ((A+A.EQ.ZERO) .OR. (ABS(Z).GE.Y)) GO TO 200
      TEMP1 = TEMP*BETAIN
      IF (TEMP1*BETA.EQ.Z) GO TO 200
      I = I + 1
      K = K + K
      GO TO 180
  200 IF (IBETA.EQ.10) GO TO 220
      IEXP = I + 1
      MX = K + K
      GO TO 280
C-----------------------------------------------------------------------
C     This segment is for decimal machines only.
C-----------------------------------------------------------------------
  220 IEXP = 2
      IZ = IBETA
  240 IF (K.LT.IZ) GO TO 260
      IZ = IZ*IBETA
      IEXP = IEXP + 1
      GO TO 240
  260 MX = IZ + IZ - 1
C-----------------------------------------------------------------------
C     Loop to determine MINEXP, XMIN.
C     Exit from loop is signaled by an underflow.
C-----------------------------------------------------------------------
  280 XMIN = Y
      Y = Y*BETAIN
C-----------------------------------------------------------------------
C     Check for underflow here.
C-----------------------------------------------------------------------
      A = Y*ONE
      TEMP = Y*T
      IF (((A+A).EQ.ZERO) .OR. (ABS(Y).GE.XMIN)) GO TO 300
      K = K + 1
      TEMP1 = TEMP*BETAIN
      IF ((TEMP1*BETA.NE.Y) .OR. (TEMP.EQ.Y)) THEN
         GO TO 280
      ELSE
         NXRES = 3
         XMIN = Y
      END IF
  300 MINEXP = -K
C-----------------------------------------------------------------------
C     Determine MAXEXP, XMAX.
C-----------------------------------------------------------------------
      IF ((MX.GT.K+K-3) .OR. (IBETA.EQ.10)) GO TO 320
      MX = MX + MX
      IEXP = IEXP + 1
  320 MAXEXP = MX + MINEXP
C     -----------------------------------------------------------------
C     Adjust IRND to reflect partial underflow.
C     -----------------------------------------------------------------
      IRND = IRND + NXRES
C     -----------------------------------------------------------------
C     Adjust for IEEE-style machines.
C     -----------------------------------------------------------------
      IF (IRND.GE.2) MAXEXP = MAXEXP - 2
C     -----------------------------------------------------------------
C     Adjust for machines with implicit leading bit in binary
C     significand, and machines with radix point at extreme
C     right of significand.
C     -----------------------------------------------------------------
      I = MAXEXP + MINEXP
      IF ((IBETA.EQ.2) .AND. (I.EQ.0)) MAXEXP = MAXEXP - 1
      IF (I.GT.20) MAXEXP = MAXEXP - 1
      IF (A.NE.Y) MAXEXP = MAXEXP - 2
      XMAX = ONE - EPSNEG
      IF (XMAX*ONE.NE.XMAX) XMAX = ONE - BETA*EPSNEG
      XMAX = XMAX/(BETA*BETA*BETA*XMIN)
      I = MAXEXP + MINEXP + 3
      IF (I.LE.0) GO TO 360
      DO 340 J = 1, I
         IF (IBETA.EQ.2) XMAX = XMAX + XMAX
         IF (IBETA.NE.2) XMAX = XMAX*BETA
  340 CONTINUE
  360 RETURN
C     ---------- LAST CARD OF MACHAR ----------
      END
