!*********************** SUBROUTINE SUBLAYER *************************

   SUBROUTINE SUBLAYER(pz1,pz2,pz3,px1,px2,ldgradps, &
                &       pt1,pt2,pw1,pw2,pzs1,pzs2,pzps)
!     
!-----------------------------------------------------------------------
!     
!     This software was developed and tested by Y.J. Rochon
!     with contributions from L. Garand, D.S. Turner, and S. Polavarapu
!     of the Atmospheric Science and Technology Directorate,
!     Environment Canada (EC).
!                              
!     Copyright 2006, Environment Canada, All Rights Reserved.

!     Please send any improvements to any one of the above authors
!     at firstname.lastname@ec.gc.ca (e.g yves.rochon@ec.gc.ca) and provide
!     appropriate aknowledgement through referencing to the authors or
!     the corresponding journal article (see below).
!     
!     Description:
!       
!     Determine weight coefficient contributions to assign to NWP/model 
!     variables at px1 and px2. Weights are determined from integration over
!     the range (zy1,zy2), which is within the ranges of the
!     NWP/model layer (px1,px2) and of the RTM layer (pz1,pz2). Intergrals
!     are approximated via the trapezoidal rule:

!         integral of f(x) from zy1 to zy2 = (f(zy1)+f(zy2))/2*abs(zy1-zy2)

!     This is synonomous to having an integrand linear in x.

!     Method:
!     - Piecewise weighted interpolation in ln(P).
!     - Journal reference:
!       Rochon, Y.J., L. Garand, D.S. Turner, and S. Polavarapu. 
!       Jacobian mapping between coordinate systems in data assimilation,
!       Q. J. of the Royal Met. Soc., Accepted pending revisions on 25
!       Sept. 2006.

!     History:

!     Version   Date        Comment
!     -------   ----        -------
!     1         10/2005     Original F77 code by Y.J. Rochon with
!                           contributions from S. Polavarapu. (EC)
!     2         10/2006     Adaptation for F90 and for improved consistency
!                           with RTTOV code format (Y.J. Rochon, EC).
!                           Argument names of original code were not changed.   
!     
!     Arguments:

!     Input

!     pz1.......................Outer boundary of RTM level (above or below pz2)
!     pz2.......................Inner boundary of RTM level
!                              (position of reference RTM level)
!     pz3.......................Second outer boundary
!     px1.......................Upper boundary of NWP layer (px1<px2)
!     px2.......................Lower boundary of NWP layer
!     pt1.......................Variable value at upper NWP level.
!     pt2.......................Variable value at lower NWP level.
!     ldgradps..................Flag indicating if gradient w.r.t. Ps is
!                              required. True for yes.
!     dzs1.....................dlnP/dPs = dx1/dPs (required when ldgradps=.true.)
!     dzs2.....................dlnP/dPs = dx2/dPs (required when ldgradps=.true.)
!     pzps.....................Current gradient contribution for 
!                              weights*variable w.r.t Ps
!                              (required when ldgradps=.true.)
!     pzps2....................Current gradient contribution for 
!                              weights w.r.t Ps
!                              (required when ldgradps=.true.)

!     Output

!     pw1.......................Weight assigned to variable at upper NWP level
!     pw2.......................Weight assigned to variable at lower NWP level
!     pzps.....................Updated gradient contribution for 
!                              weights*variable w.r.t Ps
!                              (provided when ldgradps=.true.)
!     pzps2....................Updated gradient contribution for  
!                              weights w.r.t Ps 
!                              (required when ldgradps=.true.) 

!     Other

!     ztot.....................Evaluated integral
!     zg1......................Gradient of weights*variables w.r.t. px1
!     zg2......................Gradient of weights*variables w.r.t. px2
!     zy1......................Upper boundary of integral range (zy1<zy2)
!     zy2......................Lower boundary of integral range
!                              
!     External functions and subroutines: None

!     Related routines: INTAVGAD, INTAVG, INTAVGTL, INTAVGK, LAYERAVG
!     
!     Assumptions:
!     
!     1) px1<px2

!     Comments: 

!--------------------------------------------------------------------
!     
      USE PARKIND1  ,ONLY : JPIM     ,JPRB
      USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

      IMPLICIT NONE
!     
! --- Subroutine arguments
!                              
      LOGICAL, INTENT(in) :: ldgradps
      REAL(KIND=JPRB), INTENT(in)    :: pz1,pz2,pz3,px1,px2,pt1,pt2,pzs1,pzs2
      REAL(KIND=JPRB), INTENT(out)    :: pw1,pw2,pzps
!     
! --- Local scalars and arrays
!               
      REAL(KIND=JPRB)    :: zy1,zy2,ztot,zd,zd2,zw10,zw20,zdz,zdx,zdy,zdzd,zdxd,zg1,zg2
      REAL(KIND=JPRB)    :: za1,za2,zaa1
      INTEGER(KIND=JPIM) :: ibot,itop
      REAL(KIND=JPRB)    :: ZHOOK_HANDLE

#include "abor1.intfb.h"
!-----------------------------------------------------------------------
      IF (LHOOK) CALL DR_HOOK('SUBLAYER',0,ZHOOK_HANDLE)

! --- Identify and set upper and lower boundaries of
!     integration/averaging layers (zy1 and zy2)

      itop=0
      ibot=0
      if (pz1 < pz3) then
         zy1=pz1
         if (px1 > pz1) then
            zy1=px1
            itop=1
         endif
         zy2=pz2
         if (px2 < pz2) then
            zy2=px2
            ibot=1
         endif
      else
         zy1=pz2
         if (px1 > pz2) then
            zy1=px1
            itop=1
         endif
         zy2=pz1
         if (px2 < pz1) then
            zy2=px2
            ibot=1
         endif
      endif

! --- Set weights for forward interpolator and linear model contribution to TLM

      zdy=zy2-zy1
      zdz=pz1-pz2
      if (abs(zdz) < 1E-15) then
         print*,'SUBLAYER: ERROR: zdz is <=0. zdz = ',zdz
         print*,'pz1,pz2,pz3 = ',pz1,pz2,pz3
         print*,'px1,px2    = ',px1,px2
         print*,'pt1,pt2    = ',pt1,pt2
         CALL ABOR1('SUBLAYER: zdz is <= 0.')
      else
         zdzd=1.0/zdz
      endif
      pw1=(pz1-zy1)*zdzd*zdy
      pw2=(pz1-zy2)*zdzd*zdy
      zw10=pw1
      zw20=pw2
      zdx=(px2-px1)
      if (abs(zdx) < 1E-10) then
         print*,'SUBLAYER: ERROR: zdx is <=0. zdx = ',zdx
         print*,'pz1,pz2,pz3 = ',pz1,pz2,pz3
         print*,'px1,px2    = ',px1,px2
         print*,'pt1,pt2    = ',pt1,pt2
         CALL ABOR1('SUBLAYER: zdx is <= 0.')
      else
         zdxd=1.0/zdx
      endif

      zd=(px2-pz2)*zdxd
      if (pz1 < pz3.and.ibot == 0) then
         pw1=pw1+pw2*zd
         pw2=pw2*(1.0-zd)
      elseif (pz1 > pz3.and.itop == 0) then
         pw2=pw2+pw1*(1.0-zd)
         pw1=pw1*zd
      endif
      ztot=pt1*pw1+pt2*pw2
!     
! --- Provide linearized contribution of non-linear model component 
!     to TLM (gradients w.r.t. Ps)

!     Note: The gradient calc w.r.t Ps does not account for possible
!     re-assigment of z* in LAYERAVG and of the bottom RTM level in INTAVG*
!     to the surface model level when the original values are below
!     the model surface. This will result in differences from gradients
!     calculated using finite differences.

      IF (ldgradps) THEN

!        Determine gradient of 'ztot' w.r.t. px1

         if (itop == 1) then
            za1=-(zdy+(pz1-zy1))*zdzd 
            za2=-(pz1-zy2)*zdzd
         else
            za1=0.0
            za2=0.0
         endif

         zd2=1.0-zd
         if (pz1 < pz3.and.ibot == 0) then
            zaa1=zw20*zd*zdxd
            za1=za1+za2*zd+zaa1
            za2=za2*zd2-zaa1
         elseif (pz1 > pz3.and.itop == 0) then
            zaa1=zw10*zd*zdxd
            za2=-zaa1
            za1=zaa1
         endif

         zg1=za1*pt1+za2*pt2
!                      
!        Determine gradient of 'ztot' w.r.t. px2

         if (ibot == 1) then
            za1=zdzd*(pz1-zy1)
            za2=((pz1-zy2)-zdy)*zdzd
         else
            za1=0.0
            za2=0.0
         endif

         if (pz1 < pz3.and.ibot == 0) then
            zaa1=zw20*zd2*zdxd
            za1=zaa1
            za2=-zaa1
         elseif (pz1 > pz3.and.itop == 0) then
            zaa1=zw10*zd2*zdxd
            za2=za2+za1*zd2-zaa1
            za1=za1*zd+zaa1
         endif

         zg2=za1*pt1+za2*pt2

!        Accumulate for gradient w.r.t. Ps

         pzps=pzps+zg1*pzs1+zg2*pzs2

      ENDIF

   IF (LHOOK) CALL DR_HOOK('SUBLAYER',1,ZHOOK_HANDLE)
   END SUBROUTINE SUBLAYER
