MODULE CONTROL_VECTORS_BASE_MIX

!   Purpose.
!   --------
!     This module contains the definition of the CONTROL_VECTOR type
!     and basic routines (allocate, deallocate).
!
!   Author.
!   -------
!     Y. Tremolet
!
!   Modifications.
!   -------------- 
!     Original   25-Jul-2004 Split from CONTROL_VECTORS
!     A. Deckmyn 01-Oct-2008 LAM-wavelet Jb
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE SPECTRAL_FIELDS_MOD, ONLY: SPECTRAL_FIELD, INQ_SPEC, DEALLOCATE_SPEC
USE CONTROL_VECTORS_DATA_MIX, ONLY: &
     & linitcv, ltovscv, nmoderr, lelam, lwaveletjb, &
     & nsmax, nmsmax, nserr, nflevl, nflevg, &
     & nctl3, nctl2, nctl1, nve3d, nve2d, nparam, nscales, &
     & nproma, nlentovscv, nlentovscvg, nvatovv, nblocksl, ngptotg, &
     & mytsoff, nobntvg, nobntv, ltscv, nblocksg

IMPLICIT NONE
PRIVATE
PUBLIC control_vector, allocate_ctlvec, deallocate_ctlvec, &
     & OPERATOR (.EQV.), OPERATOR (.NEQV.), check

! ------------------------------------------------------------------

TYPE CONTROL_VECTOR
  REAL(KIND=JPRB), POINTER :: data(:) => NULL()
  LOGICAL :: linitcv, ltovscv, lam1d, lwaveletjb, ltscv
  INTEGER(KIND=JPIM) :: nsmax, nmsmax, nmoderr, nserr, npara
  INTEGER(KIND=JPIM) :: ns3d, ns2d, ns1d, nve3d, nve2d
  INTEGER(KIND=JPIM) :: nsizel, nsizeg, nscales
  INTEGER(KIND=JPIM), POINTER :: ibgn_scale(:) => NULL()
  REAL(KIND=JPRB), POINTER :: initcv(:) => NULL()
  REAL(KIND=JPRB), POINTER :: moderr(:) => NULL()
  REAL(KIND=JPRB), POINTER :: tovscv(:) => NULL()
  REAL(KIND=JPRB), POINTER :: lamcv (:) => NULL()
  REAL(KIND=JPRB), POINTER :: params(:) => NULL()
  REAL(KIND=JPRB), POINTER :: tscv(:) => NULL()
  LOGICAL :: lalloc = .false.
END TYPE CONTROL_VECTOR

! ------------------------------------------------------------------

INTERFACE allocate_ctlvec
MODULE PROCEDURE allocate_cv, allocate_cv_cv
END INTERFACE

INTERFACE check
MODULE PROCEDURE check_cv, check_cv_cv
END INTERFACE

INTERFACE OPERATOR (.EQV.)
MODULE PROCEDURE equiv_cv
END INTERFACE

INTERFACE OPERATOR (.NEQV.)
MODULE PROCEDURE nequiv_cv
END INTERFACE

! ------------------------------------------------------------------
CONTAINS
! ------------------------------------------------------------------

SUBROUTINE allocate_cv(ydcv,ksmax,kmsmax,kmerr,kserr,kparam,ldinit,ldtovs, &
                     & ldlam,ks3d,ks2d,ks1d,ke3d,ke2d,knscales,ldts)

TYPE(control_vector), INTENT(OUT) :: ydcv
INTEGER(KIND=JPIM), OPTIONAL, INTENT(IN) :: ksmax,kmsmax,kmerr,kserr,kparam
LOGICAL, OPTIONAL, INTENT(IN) :: ldinit,ldtovs,ldlam, ldts
INTEGER(KIND=JPIM), OPTIONAL, INTENT(IN) :: ks3d,ks2d,ks1d,ke3d,ke2d,knscales

INTEGER(KIND=JPIM) :: ibgn_ic,iend_ic,ibgn_me,iend_me,ibgn_tv,iend_tv, &
                    & ibgn_la,iend_la,ibgn_pa,iend_pa,ibgn,ilen,jj, &
                    & ibgn_ts,iend_ts
TYPE(SPECTRAL_FIELD) :: ylsp

! Default values
ydcv%linitcv   = linitcv
ydcv%nsmax     = nsmax
ydcv%nmsmax    = nmsmax
ydcv%nmoderr   = nmoderr
ydcv%nserr     = nserr
ydcv%ltovscv   = ltovscv
ydcv%lam1d     = lelam
ydcv%ns3d      = nctl3
ydcv%ns2d      = nctl2
ydcv%ns1d      = nctl1
ydcv%nve3d     = nve3d
ydcv%nve2d     = nve2d
ydcv%ns1d      = nctl1
ydcv%npara     = nparam
ydcv%lwaveletjb= lwaveletjb
if (lwaveletjb) then
  ydcv%nscales = nscales
endif
ydcv%ltscv=ltscv

! Check for optional arguments
IF (PRESENT(ldinit))   ydcv%linitcv = ldinit
IF (PRESENT(ksmax))    ydcv%nsmax   = ksmax
IF (PRESENT(kmsmax))   ydcv%nmsmax  = kmsmax
IF (PRESENT(kmerr))    ydcv%nmoderr = kmerr
IF (PRESENT(kserr))    ydcv%nserr   = kserr
IF (PRESENT(ldtovs))   ydcv%ltovscv = ldtovs
IF (PRESENT(ldlam ))   ydcv%lam1d   = ldlam
IF (PRESENT(ks3d))     ydcv%ns3d    = ks3d
IF (PRESENT(ks2d))     ydcv%ns2d    = ks2d
IF (PRESENT(ks1d))     ydcv%ns1d    = ks1d
IF (PRESENT(ke3d))     ydcv%nve3d   = ke3d
IF (PRESENT(ke2d))     ydcv%nve2d   = ke2d
IF (PRESENT(kparam))   ydcv%npara   = kparam
IF (PRESENT(knscales)) ydcv%nscales = knscales
IF (PRESENT(ldts)) ydcv%ltscv=ldts

ydcv%lwaveletjb = ydcv%lwaveletjb .AND. ydcv%linitcv
IF (PRESENT(knscales)) THEN
  IF (knscales >NSCALES) THEN
    write(*,*) 'knscales too large: knscales=',knscales,' > NSCALES=',NSCALES
    CALL ABOR1('knscales>NSCALES in allocate_cv')
  ENDIF
ENDIF

!cf To avoid stupid array bound violation
IF (ydcv%ns1d==0) ydcv%lam1d=.false.
ydcv%nsizel=0
ydcv%nsizeg=0

IF (ydcv%linitcv) THEN
  IF (.not.ydcv%lwaveletjb) THEN
    CALL inq_spec(ylsp,ydcv%nsmax,ydcv%nmsmax,ydcv%ns3d,ydcv%ns2d,ydcv%ns1d)
    ilen=ylsp%nspsizel
    ibgn_ic=1
    iend_ic=ilen
    ydcv%nsizel=ilen
    ilen=ylsp%nspsizeg
    ydcv%nsizeg=ilen
    CALL deallocate_spec(ylsp)
  ELSEIF (.NOT. lelam) THEN
    if (ydcv%nscales==NSCALES) then
      ilen=nproma*(nflevg*ydcv%ns3d+ydcv%ns2d)*sum(nblocksl(:))
    else
      ilen = nproma*(nflevg*ydcv%ns3d+ydcv%ns2d) &
          & *(2*nblocksl(2+NSCALES-ydcv%nscales) &
          &   +sum(nblocksl(3+NSCALES-ydcv%nscales:)) )
    endif
    ibgn_ic=1
    iend_ic=ilen
    ydcv%nsizel=ilen
    if (ydcv%nscales==NSCALES) then
      ydcv%nsizeg=(nflevg*ydcv%ns3d+ydcv%ns2d)*sum(ngptotg(:))
    else
      ydcv%nsizeg = (nflevg*ydcv%ns3d+ydcv%ns2d) &
               & *(2*ngptotg(2+NSCALES-ydcv%nscales) &
               &   +sum(ngptotg(3+NSCALES-ydcv%nscales:)) )
    endif

    ALLOCATE(ydcv%ibgn_scale(ydcv%nscales))
    ibgn=ibgn_ic
    DO jj=1,ydcv%nscales
      ydcv%ibgn_scale(jj) = ibgn
      if (ydcv%nscales==NSCALES) then
        ilen = nproma*(nflevg*ydcv%ns3d+ydcv%ns2d)*nblocksl(jj)
      else
        ilen = nproma*(nflevg*ydcv%ns3d+ydcv%ns2d) &
                   & *nblocksl(max(2,jj)+NSCALES-ydcv%nscales)
      endif
      ibgn = ibgn + ilen
    ENDDO
    if (ibgn /= 1+iend_ic) CALL ABOR1 ('Error defining indices for initcv')
  ELSE ! LAM wavelets
! ilen_l =4*NWAVLATS*NWAVDIMY*(4*NFLEVG+1)
    ilen = 4*(nctl3*nflevg+nctl2)*nblocksl(1)*nblocksl(2)
    ibgn_ic=1
    iend_ic=ilen
    ydcv%nsizel=ilen
! ilen_g =4*NWAVDIMX*NWAVDIMY*(4*NFLEVG+1)
    ydcv%nsizeg = 4*(nctl3*nflevg+nctl2)*nblocksg(1)*nblocksg(2)
  ENDIF
ENDIF

IF (ydcv%nmoderr>0) THEN
  CALL inq_spec(ylsp,ydcv%nserr,ydcv%nserr,ydcv%nve3d,ydcv%nve2d,0)
  ilen = ydcv%nmoderr * ylsp%nspsizel
  ibgn_me = ydcv%nsizel + 1
  iend_me = ydcv%nsizel + ilen
  ydcv%nsizel = ydcv%nsizel + ilen
  ilen = ydcv%nmoderr * ylsp%nspsizeg
  ydcv%nsizeg = ydcv%nsizeg + ilen
  CALL deallocate_spec(ylsp)
ENDIF

IF (ydcv%ltovscv) THEN
  ilen = nlentovscv*nvatovv
  ibgn_tv = ydcv%nsizel + 1
  iend_tv = ydcv%nsizel + ilen
  ydcv%nsizel = ydcv%nsizel + ilen
  ydcv%nsizeg = ydcv%nsizeg + nlentovscvg*nvatovv
ENDIF

IF (ydcv%ltscv) THEN
   ilen = nobntv
   ibgn_ts= ydcv%nsizel + 1
   iend_ts= ydcv%nsizel + ilen
   ydcv%nsizel = ydcv%nsizel + ilen
   ydcv%nsizeg = ydcv%nsizeg + nobntvg
ENDIF


IF (ydcv%lam1d) THEN
  ilen=nflevl*ydcv%ns1d
  ibgn_la = ydcv%nsizel + 1
  iend_la = ydcv%nsizel + ilen
  ydcv%nsizel = ydcv%nsizel + ilen
  ydcv%nsizeg = ydcv%nsizeg + nflevg*ydcv%ns1d
ENDIF

IF (ydcv%npara>0) THEN
  ibgn_pa = ydcv%nsizel + 1
  iend_pa = ydcv%nsizel + ydcv%npara
  ydcv%nsizel = ydcv%nsizel + ydcv%npara
  ydcv%nsizeg = ydcv%nsizeg + ydcv%npara
ENDIF

ALLOCATE(ydcv%data(1:ydcv%nsizel))
ydcv%lalloc = .true.

IF (ydcv%linitcv  ) ydcv%initcv => ydcv%data(ibgn_ic:iend_ic)
IF (ydcv%nmoderr>0) ydcv%moderr => ydcv%data(ibgn_me:iend_me)
IF (ydcv%ltovscv  ) ydcv%tovscv => ydcv%data(ibgn_tv:iend_tv)
IF (ydcv%ltscv    ) ydcv%tscv   => ydcv%data(ibgn_ts:iend_ts)
IF (ydcv%lam1d    ) ydcv%lamcv  => ydcv%data(ibgn_la:iend_la)
IF (ydcv%npara>0  ) ydcv%params => ydcv%data(ibgn_pa:iend_pa)


END SUBROUTINE allocate_cv

! ------------------------------------------------------------------

SUBROUTINE allocate_cv_cv(ydcv,ydin)
TYPE(control_vector), INTENT(OUT) :: ydcv
TYPE(control_vector), INTENT(IN)  :: ydin
LOGICAL :: llinit,lltovs,lllam,llts

llinit=ydin%linitcv
lltovs=ydin%ltovscv
lllam =ydin%lam1d
llts=ydin%ltscv

CALL allocate_cv(ydcv, ksmax=ydin%nsmax, kmsmax=ydin%nmsmax, &
               & kmerr=ydin%nmoderr, kserr=ydin%nserr,kparam=ydin%npara, &
               & ldinit=llinit, ldtovs=lltovs, ldlam=lllam, &
               & ks3d=ydin%ns3d, ks2d=ydin%ns2d, ks1d=ydin%ns1d, &
               & ke3d=ydin%nve3d, ke2d=ydin%nve2d, ldts=llts)

END SUBROUTINE allocate_cv_cv

! ------------------------------------------------------------------

SUBROUTINE deallocate_ctlvec(ydcv)
TYPE(control_vector), INTENT(INOUT) :: ydcv

IF (ASSOCIATED(ydcv%initcv)) NULLIFY(ydcv%initcv)
IF (ASSOCIATED(ydcv%moderr)) NULLIFY(ydcv%moderr)
IF (ASSOCIATED(ydcv%tovscv)) NULLIFY(ydcv%tovscv)
IF (ASSOCIATED(ydcv%lamcv )) NULLIFY(ydcv%lamcv )
IF (ASSOCIATED(ydcv%params)) NULLIFY(ydcv%params)
IF (ASSOCIATED(ydcv%data)) DEALLOCATE(ydcv%data)
IF (ASSOCIATED(ydcv%ibgn_scale)) DEALLOCATE(ydcv%ibgn_scale)
ydcv%lalloc = .false.
IF (ASSOCIATED(ydcv%tscv)) NULLIFY(ydcv%tscv)

END SUBROUTINE deallocate_ctlvec

! ------------------------------------------------------------------

SUBROUTINE check_cv(ydcv)
TYPE(control_vector), INTENT(IN) :: ydcv

IF (.NOT.(ydcv%lalloc)) THEN
  CALL ABOR1('Control vector not allocated')
ENDIF
!yt the following is temporary, lalloc can be removed after
!yt this has been tested.
IF (ASSOCIATED(ydcv%data) .AND. .NOT.(ydcv%lalloc)) THEN
  CALL ABOR1('Control vector associated but not lalloc')
ENDIF
IF (.NOT.ASSOCIATED(ydcv%data) .AND. ydcv%lalloc) THEN
  CALL ABOR1('Control vector lalloc but not associated')
ENDIF

END SUBROUTINE check_cv

! ------------------------------------------------------------------

SUBROUTINE check_cv_cv(ydcv1,ydcv2)
TYPE(control_vector), INTENT(IN) :: ydcv1, ydcv2

IF (.NOT.(ydcv1%lalloc .AND. ydcv2%lalloc)) THEN
  CALL ABOR1('Control vectors not allocated')
ENDIF
IF (ydcv1.NEQV.ydcv2) THEN
  CALL ABOR1('Control vectors are incompatible')
ENDIF

IF ((ydcv1%nsmax/=ydcv2%nsmax) .OR. (ydcv1%nserr/=ydcv2%nserr) .OR. &
 &((ydcv1%nmsmax/=ydcv2%nmsmax).AND.(LELAM))) THEN
  CALL ABOR1('Control vectors have different resolution')
ENDIF

END SUBROUTINE check_cv_cv

! ------------------------------------------------------------------

LOGICAL FUNCTION equiv_cv(ydcv1,ydcv2)
TYPE(control_vector), INTENT(IN) :: ydcv1
TYPE(control_vector), INTENT(IN) :: ydcv2
LOGICAL :: ll

ll = .true.
ll = ll .AND. (ydcv1%linitcv.EQV.ydcv2%linitcv)
ll = ll .AND. (ydcv1%ltovscv.EQV.ydcv2%ltovscv)
ll = ll .AND. (ydcv1%lam1d  .EQV.ydcv2%lam1d  )

ll = ll .AND. (ydcv1%nmoderr== ydcv2%nmoderr)
ll = ll .AND. (ydcv1%ns3d   == ydcv2%ns3d)
ll = ll .AND. (ydcv1%ns2d   == ydcv2%ns2d)
ll = ll .AND. (ydcv1%ns1d   == ydcv2%ns1d)
ll = ll .AND. (ydcv1%nve3d  == ydcv2%nve3d)
ll = ll .AND. (ydcv1%nve2d  == ydcv2%nve2d)
ll = ll .AND. (ydcv1%npara  == ydcv2%npara)

equiv_cv=ll
END FUNCTION equiv_cv
  
! ------------------------------------------------------------------
  
LOGICAL FUNCTION nequiv_cv(ydcv1,ydcv2)
TYPE(control_vector), INTENT(IN) :: ydcv1
TYPE(control_vector), INTENT(IN) :: ydcv2
  
nequiv_cv = .NOT. equiv_cv(ydcv1,ydcv2)
  
END FUNCTION nequiv_cv

! ------------------------------------------------------------------

END MODULE CONTROL_VECTORS_BASE_MIX
