MODULE CONTROL_VECTORS_PARA_MOD

!   Purpose.
!   --------
!     This module contains the distributed routines associated
!     to the CONTROL_VECTOR type.
!
!   Author.
!   -------
!     Y. Tremolet
!
!   Modifications.
!   -------------- 
!     Original   25-Jul-2004 Split from CONTROL_VECTORS
!     M.Hamrud   25-Aug-2009 Remove use of DOT_PRODUCT_CTLVEC for most cases,
!                use simple dot-product or reproducible sum instead
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPRB, JPIM
USE MPL_MODULE, ONLY: MPL_ALLREDUCE,MPL_MYRANK,MPL_NPROC,MPL_ABORT
USE ORDER_INDEPENDENT_SUMMATION_MOD, ONLY  : ORDER_INDEP_DOT_PRODUCT
USE CONTROL_VECTORS_BASE_MIX, ONLY: CONTROL_VECTOR, &
                              & ALLOCATE_CTLVEC, DEALLOCATE_CTLVEC, CHECK
USE CONTROL_VECTORS_DATA_MIX, ONLY: LREPRO,myproc,mysetw,nproc,nprtrw,&
 & nflevl,nflevg
IMPLICIT NONE
PRIVATE
PUBLIC DOT_PRODUCT, MAXVAL, MINVAL, SUM, CTLVEC_SQNORM, CTLVEC_NORM

! ------------------------------------------------------------------

INTERFACE DOT_PRODUCT
MODULE PROCEDURE dot_product_cv_cv, dot_product_weight,dot_product_cvs_cv
END INTERFACE

INTERFACE MAXVAL
MODULE PROCEDURE maxval_cv
END INTERFACE

INTERFACE MINVAL
MODULE PROCEDURE minval_cv
END INTERFACE

INTERFACE SUM
MODULE PROCEDURE sum_cv
END INTERFACE

INTERFACE CTLVEC_SQNORM
MODULE PROCEDURE cv_sqnorm1, cv_sqnorm4
END INTERFACE

! ------------------------------------------------------------------
CONTAINS
! ------------------------------------------------------------------

REAL(KIND=JPRB) FUNCTION dot_product_cv_cv(ydcv1,ydcv2)
TYPE (control_vector), INTENT(IN) :: ydcv1,ydcv2
REAL(KIND=JPRB) :: zz
INTEGER(KIND=JPIM) :: jj,ilen,ileng

CALL check(ydcv1,ydcv2)

CALL DPLENS(ydcv1,ilen,ileng)
IF(lrepro) THEN
  dot_product_cv_cv= ORDER_INDEP_DOT_PRODUCT(ydcv1%data(1:ilen),ydcv2%data(1:ilen),KNG=ileng)
ELSE
  call gstats(1962,0)
  zz=0.0_jprb
  do jj=1,ilen
    zz=zz+ydcv1%data(jj)*ydcv2%data(jj)
  enddo
  call gstats(1962,1)
  call gstats_barrier(746)
  call gstats(657,0)
  CALL MPL_ALLREDUCE(zz,'SUM',LDREPROD=.TRUE.,CDSTRING='DOT_PRODUCT_CV_CV')
  call gstats(657,1)
  call gstats_barrier2(746)
  dot_product_cv_cv=zz
ENDIF

END FUNCTION dot_product_cv_cv

! ------------------------------------------------------------------
FUNCTION dot_product_cvs_cv(ydcv1,ydcv2)
TYPE (control_vector), INTENT(IN) :: ydcv1(:),ydcv2
REAL(KIND=JPRB) :: dot_product_cvs_cv(size(ydcv1))
REAL(KIND=JPRB) :: zz(size(ydcv1))
INTEGER(KIND=JPIM) :: jj,ilen,ileng,isize1,ja

CALL check(ydcv1(1),ydcv2)
isize1=size(ydcv1)
CALL DPLENS(ydcv2,ilen,ileng)
IF(lrepro) THEN
  DO JA=1,isize1
    dot_product_cvs_cv(JA)= ORDER_INDEP_DOT_PRODUCT(ydcv1(JA)%data(1:ilen),ydcv2%data(1:ilen),KNG=ileng)
  ENDDO
ELSE
  call gstats(1234,0)
  zz(:)=0.0_jprb
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JA,JJ)
  DO JA=1,isize1
    do jj=1,ilen
      zz(JA)=zz(JA)+ydcv1(JA)%data(jj)*ydcv2%data(jj)
    enddo
  enddo
!$OMP END PARALLEL DO 
  call gstats(1234,1)
  call gstats_barrier(746)
  call gstats(657,0)
  CALL MPL_ALLREDUCE(zz,'SUM',LDREPROD=.TRUE.,CDSTRING='DOT_PRODUCT_CV_CV')
  call gstats(657,1)
  call gstats_barrier2(746)
  dot_product_cvs_cv(:)=zz(:)
ENDIF

END FUNCTION dot_product_cvs_cv

! ------------------------------------------------------------------

REAL(KIND=JPRB) FUNCTION dot_product_weight(ydcv1,ydcv2,ydw)
TYPE (control_vector), INTENT(IN) :: ydcv1,ydcv2,ydw
REAL(KIND=JPRB) :: zz
INTEGER(KIND=JPIM) :: jj,ilen,ileng

CALL check(ydcv1,ydcv2)
CALL DPLENS(ydcv1,ilen,ileng)
IF(lrepro) THEN
  dot_product_weight = &
   & ORDER_INDEP_DOT_PRODUCT(ydcv1%data(1:ilen),ydcv2%data(1:ilen),pw=ydw%data(1:ilen),KNG=ileng)
ELSE
  call gstats(1962,0)
  zz=0.0_jprb
  do jj=1,ilen
    zz=zz+ydcv1%data(jj)*ydcv2%data(jj)*ydw%data(jj)
  enddo
  call gstats(1962,1)
  call gstats_barrier(746)
  call gstats(657,0)
  CALL MPL_ALLREDUCE(zz,'SUM',LDREPROD=.TRUE.,CDSTRING='DOT_PRODUCT_CV_CV')
  call gstats(657,1)
  call gstats_barrier2(746)
  dot_product_weight=zz
ENDIF
END FUNCTION dot_product_weight

! ------------------------------------------------------------------

SUBROUTINE cv_sqnorm1(ydcv,pnorm)
TYPE (control_vector), INTENT(IN) :: ydcv
REAL(KIND=JPRB), INTENT(OUT) :: pnorm
REAL(KIND=JPRB) :: zz1,zz2,zz3,zz4,zz5,zz6

CALL check(ydcv)
CALL dot_product_ctlvec(ydcv,ydcv,zz1,zz2,zz3,zz4,zz5,zz6,.true.)
pnorm=zz1+zz2+zz3+zz4+zz5+zz6


END SUBROUTINE cv_sqnorm1

! ------------------------------------------------------------------

SUBROUTINE cv_sqnorm4(ydcv,pnorm1,pnorm2,pnorm3,pnorm4,pnorm5,pnorm6)
TYPE (control_vector), INTENT(IN) :: ydcv
REAL(KIND=JPRB), INTENT(OUT) :: pnorm1,pnorm2,pnorm3,pnorm4,pnorm5,pnorm6

CALL check(ydcv)
CALL dot_product_ctlvec(ydcv,ydcv,pnorm1,pnorm2,pnorm3,pnorm4,pnorm5,pnorm6,.true.)

END SUBROUTINE cv_sqnorm4

! ------------------------------------------------------------------

REAL(KIND=JPRB) FUNCTION ctlvec_norm(ydcv)
TYPE (control_vector), INTENT(IN) :: ydcv
REAL(KIND=JPRB) :: zz

CALL cv_sqnorm1(ydcv,zz)
ctlvec_norm=SQRT(zz)

END FUNCTION ctlvec_norm

! ------------------------------------------------------------------

REAL(KIND=JPRB) FUNCTION maxval_cv(ydcv)
TYPE (control_vector), INTENT(IN) :: ydcv
REAL(KIND=JPRB) :: zz

CALL GSTATS(256,0)
zz = MAXVAL(ydcv%data)
CALL MPL_ALLREDUCE(zz,'MAX',CDSTRING='CONTROL_VECTORS_PARA_MOD:MAXVAL_CV')
maxval_cv=zz
CALL GSTATS(256,1)

END FUNCTION maxval_cv

! ------------------------------------------------------------------

REAL(KIND=JPRB) FUNCTION minval_cv(ydcv)
TYPE (control_vector), INTENT(IN) :: ydcv
REAL(KIND=JPRB) :: zz

CALL GSTATS(256,0)
zz = MINVAL(ydcv%data)
CALL MPL_ALLREDUCE(zz,'MIN',CDSTRING='CONTROL_VECTORS_PARA_MOD:MINVAL_CV')
minval_cv=zz
CALL GSTATS(256,1)

END FUNCTION minval_cv

! ------------------------------------------------------------------

REAL(KIND=JPRB) FUNCTION sum_cv(ydcv)
TYPE (control_vector), INTENT(IN) :: ydcv
REAL(KIND=JPRB) :: zz

! At the moment, this function is only used to print diagnostics in
! m1qn3 if impres>=5 which is not the case in the IFS by default.
! It is NOT reproducible.

CALL GSTATS(256,0)
zz=SUM(ydcv%data(:))
CALL MPL_ALLREDUCE(zz,'SUM',LDREPROD=.false., &
                 & CDSTRING='CONTROL_VECTORS_PARA_MOD:SUM_CV')
sum_cv = zz
CALL GSTATS(256,1)

END FUNCTION sum_cv

SUBROUTINE dplens(ydcv,klen,kleng)

! Adjusts length of control variable used in dot-product to avoid 
! double counting for the parts of the contol variable that is replicated
! on different mpi tasks (currently VarBC part and in a more complex way
! the mean winds for Aladin)

TYPE (control_vector), INTENT(IN) :: ydcv ! Control variable
INTEGER(KIND=JPIM),INTENT(OUT) :: klen    ! Adjusted local lenght of ctv
INTEGER(KIND=JPIM),INTENT(OUT) :: kleng   ! Adjusted global lenght of ctv

! Avoid counting VarBC ctl var more than once
if(myproc == 1) then
  klen=ydcv%nsizel
else
  klen=ydcv%nsizel-ydcv%npara
endif
IF (ydcv%lam1d) THEN
  CALL MPL_ABORT('NEEDS ALADIN CHECKING /MH')
  !Avoid counting mean winds more than once (not tested)
  if(mysetw /= 1) then
    klen=klen-nflevl*ydcv%ns1d
  endif
ENDIF
IF(lrepro) THEN
!Adjustments to global length should reflect those above for local lenghts
  kleng=ydcv%nsizeg-(nproc-1)*ydcv%npara  
  IF (ydcv%lam1d)  kleng=kleng-(nprtrw-1)*nflevg*ydcv%ns1d
ENDIF
END SUBROUTINE dplens
! ------------------------------------------------------------------

END MODULE CONTROL_VECTORS_PARA_MOD
