MODULE distributed_vectors_mix

!     ------------------------------------------------------------------

!*    Distributed storage for one-dimensional arrays

!     This module provides the following:

!       - The derived type "distributed_vector".
!       - Subroutines "allocate_vector" and "deallocate_vector".
!       - Access to the part of the vector held on the local PE.
!       - Overloaded assignment and arithmetic operators which
!         implicitly translate between normal 1d arrays and
!         distributed_vectors.
!       - A distributed version of DOT_PRODUCT.

!         A "distributed_vector" acts like a pointer to a real
!         one-dimensional array. Unlike an array pointer, you cannot
!         access individual elements.

!         It is vital that any operation involving a
!         "distributed_vector" is performed by all PEs. (In general
!         routines which use "distributed_vector"s will execute
!         identical code on all PEs. All message passing is done
!         implicitly via the module suroutines below.)

!  The following routines must be used to allocate and deallocate
!  space on each PE for each distributed_vector.

!         allocate_vector (handle,klen) allocates storage for the
!                                       distributed_vector "handle"
!                                       which can be used to store a
!                                       on dimensional array of "klen"
!                                       elements.

!         deallocate_vector (handle)    deallocates the space associated
!                                       with "handle".

! The following operations are defined, where handle, handle1 and
! handle2 represent distributed vectors (allocted with identical
! values for "klen"); pvec, pvec1 and pvec2 are 1d arrays with
! SHAPE "1:klen" or with SIZE "1+handle%local_end-handle%local_end";
! OP is one of +,-,*,/,** or =; and scalar is a real scalar.

!         handle  OP pvec(:)
!         pvec(:) OP handle
!         handle1 OP handle2
!         handle  OP scalar

!         scalar = DOT_PRODUCT (handle,pvec)
!         scalar = DOT_PRODUCT (handle1,handle2)
!         scalar = DOT_PRODUCT (pvec,handle2)

!   Externals.
!   ----------
!       MPL_SEND
!       MPL_RECV
!       ABOR1

!   Reference.
!   ----------
!       None yet!

!   Author.
!   -------
!       Mike Fisher *ECMWF*

!   Modifications.
!   --------------
!       Original   97-11-26
!       D.Dent     00-09-01  MPE to MPL conversion

!     ------------------------------------------------------------------



USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE MPL_MODULE

IMPLICIT NONE

PRIVATE
PUBLIC  distributed_vector, allocate_vector, deallocate_vector,&
        &ndv_chunk_size, ASSIGNMENT(=), OPERATOR(+), OPERATOR(-),&
        &OPERATOR(*), OPERATOR(/), OPERATOR(**), DOT_PRODUCT,&
        &dvsection,&
        &SUM,MAXVAL,SQRT,setup_distvec,scatter_vector,gather_vector

INTEGER(KIND=JPIM) :: ndv_chunk_size
INTEGER(KIND=JPIM) :: nproc,myproc,mintet,mrealt,MT_distributed_vector
INTEGER(KIND=JPIM) ,ALLOCATABLE :: nprcids(:)
INTEGER(KIND=JPIM),PARAMETER :: JPSETUP=77777

TYPE distributed_vector
REAL(KIND=JPRB), POINTER :: local(:)
INTEGER(KIND=JPIM)       :: global_length,local_start,local_end,nchnks
INTEGER(KIND=JPIM)       :: msetup
END TYPE distributed_vector

INTERFACE ASSIGNMENT (=)
MODULE PROCEDURE assign_ar_dv, assign_dv_ar,assign_dv_dv, assign_scalar_dv
END INTERFACE

INTERFACE OPERATOR (*)
MODULE PROCEDURE multiply_ar_dv, multiply_dv_ar,&
                   &multiply_dv_dv, multiply_scalar_dv,&
                   &multiply_dv_scalar
END INTERFACE

INTERFACE OPERATOR (+)
MODULE PROCEDURE add_ar_dv, add_dv_ar,&
                   &add_dv_dv, add_scalar_dv,&
                   &add_dv_scalar
END INTERFACE

INTERFACE OPERATOR (-)
MODULE PROCEDURE subtract_ar_dv, subtract_dv_ar,&
                   &subtract_dv_dv, subtract_scalar_dv,&
                   &subtract_dv_scalar, negate_dv
END INTERFACE

INTERFACE OPERATOR (/)
MODULE PROCEDURE divide_ar_dv, divide_dv_ar,&
                   &divide_dv_dv, divide_scalar_dv,&
                   &divide_dv_scalar
END INTERFACE

INTERFACE OPERATOR (**)
MODULE PROCEDURE power_ar_dv, power_dv_ar,&
                   &power_dv_dv, power_scalar_dv,&
                   &power_dv_scalar
END INTERFACE

INTERFACE DOT_PRODUCT
MODULE PROCEDURE dot_product_dv_dv, dot_product_dv_ar,dot_product_ar_dv
END INTERFACE

INTERFACE SCATTER_VECTOR
MODULE PROCEDURE scatter_vector_dv, scatter_vector_ar
END INTERFACE

INTERFACE SUM
MODULE PROCEDURE sum_dv
END INTERFACE

INTERFACE MAXVAL
MODULE PROCEDURE maxval_dv
END INTERFACE

INTERFACE SQRT
MODULE PROCEDURE sqrt_dv
END INTERFACE

! module procedures:

CONTAINS
SUBROUTINE setup_distvec(kproc,kyproc,kintet,krealt,ktag,&
   &kdv_chunk_size,kprcids)
INTEGER(KIND=JPIM) :: kproc,kyproc,kintet,krealt,ktag,kdv_chunk_size
INTEGER(KIND=JPIM) :: kprcids(kproc)
ALLOCATE(nprcids(kproc))
nproc=kproc
myproc=kyproc
mintet=kintet
mrealt=krealt
MT_distributed_vector=ktag
ndv_chunk_size=kdv_chunk_size
nprcids(:)=kprcids(:)

end SUBROUTINE setup_distvec

INTEGER(KIND=JPIM) FUNCTION nchunks (klen)
INTEGER(KIND=JPIM), INTENT(IN) :: klen
nchunks = (klen+ndv_chunk_size-1)/ndv_chunk_size

END FUNCTION nchunks

INTEGER(KIND=JPIM) FUNCTION jend (kproc,klen)
INTEGER(KIND=JPIM), INTENT(IN) :: kproc,klen
jend = MIN(  ndv_chunk_size &
                &*NINT( REAL(kproc)&
                      &*(REAL(nchunks(klen))/REAL(NPROC)))&
               &,klen)

END FUNCTION jend

INTEGER(KIND=JPIM) FUNCTION jstart (kproc,klen)
INTEGER(KIND=JPIM), INTENT(IN) :: kproc,klen
jstart = 1+jend(kproc-1,klen)

END FUNCTION jstart

SUBROUTINE allocate_vector (handle,klen)

!         allocate storage for the distributed vector

TYPE (distributed_vector), INTENT(OUT) :: handle
INTEGER(KIND=JPIM), INTENT(IN)  :: klen

IF(nchunks(klen) < nproc) CALL ABOR1&
 & ('ALLOCATE_VECTOR : NCHUNKS<NPROC, REDUCE CHUNK SIZE!')
  
handle%global_length = klen
handle%local_start   = jstart(MYPROC,klen)
handle%local_end     = jend  (MYPROC,klen)
handle%nchnks  = nchunks(klen)
handle%msetup = jpsetup

ALLOCATE (handle%local(handle%local_start:handle%local_end))

END SUBROUTINE allocate_vector

SUBROUTINE deallocate_vector (handle)

!         deallocate storage for the distributed array

TYPE (distributed_vector), INTENT(OUT) :: handle
DEALLOCATE (handle%local)
handle%msetup = 0

END SUBROUTINE deallocate_vector

SUBROUTINE assign_ar_dv (handle,pvec)

!         copy array to the distributed_vector

REAL(KIND=JPRB),                      INTENT(IN)    :: pvec(:)
TYPE (distributed_vector), INTENT(INOUT) :: handle

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: store to unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  handle%local(:) = pvec(:)
else if (SIZE(pvec) == handle%global_length) then
  handle%local(:) = pvec(handle%local_start:handle%local_end)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END SUBROUTINE assign_ar_dv

FUNCTION multiply_ar_dv (pvec,handle)

!         multiply array and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::multiply_ar_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: multiply unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  multiply_ar_dv = pvec(:) * handle%local(:)
else if (SIZE(pvec) == handle%global_length) then
  multiply_ar_dv =&
   &pvec(handle%local_start:handle%local_end)*handle%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION multiply_ar_dv

FUNCTION divide_ar_dv (pvec,handle)

!         divide array and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::divide_ar_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: divide unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  divide_ar_dv = pvec(:) / handle%local(:)
else if (SIZE(pvec) == handle%global_length) then
  divide_ar_dv =pvec(handle%local_start:handle%local_end)/handle%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION divide_ar_dv

FUNCTION power_ar_dv (pvec,handle)

!         power array and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::power_ar_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: power unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  power_ar_dv = pvec(:) ** handle%local(:)
else if (SIZE(pvec) == handle%global_length) then
  power_ar_dv =pvec(handle%local_start:handle%local_end)**handle%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION power_ar_dv

FUNCTION add_ar_dv (pvec,handle)

!         add array and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::add_ar_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: add unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  add_ar_dv = pvec(:) + handle%local(:)
else if (SIZE(pvec) == handle%global_length) then
  add_ar_dv =pvec(handle%local_start:handle%local_end)+handle%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION add_ar_dv

FUNCTION subtract_ar_dv (pvec,handle)

!         subtract array and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::subtract_ar_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: subtract unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  subtract_ar_dv = pvec(:) - handle%local(:)
else if (SIZE(pvec) == handle%global_length) then
  subtract_ar_dv =&
   &pvec(handle%local_start:handle%local_end)-handle%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION subtract_ar_dv

FUNCTION negate_dv (handle)

!         negate a distributed_vector

TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::negate_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: negate unallocated distributed_vector')

negate_dv =  -handle%local(:)

END FUNCTION negate_dv

SUBROUTINE assign_dv_dv (handle1,handle2)

!         copy one distributed_vector to another

TYPE (distributed_vector), INTENT(IN)    :: handle2
TYPE (distributed_vector), INTENT(INOUT) :: handle1

if (handle1%msetup /= jpsetup)&
     &call ABOR1 ('error: store to unallocated distributed_vector')

if (handle1%msetup /= jpsetup)&
     &call ABOR1('error: copy from unallocated distributed_vector')

if (handle1%global_length == handle2%global_length) then
  handle1%local(:) = handle2%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END SUBROUTINE assign_dv_dv

FUNCTION multiply_dv_dv (handle1,handle2)

!         multiply two distributed_vectors

TYPE (distributed_vector), INTENT(IN) :: handle2
TYPE (distributed_vector), INTENT(IN) :: handle1
REAL(KIND=JPRB), DIMENSION(handle1%local_start:handle1%local_end) ::multiply_dv_dv

if (handle1%msetup /= jpsetup .or. handle2%msetup /= jpsetup     )&
     &call ABOR1 ('error: multiply unallocated distributed_vector')

if (handle1%global_length == handle2%global_length) then
  multiply_dv_dv = handle1%local(:) * handle2%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION multiply_dv_dv

FUNCTION divide_dv_dv (handle1,handle2)

!         divide two distributed_vectors

TYPE (distributed_vector), INTENT(IN) :: handle2
TYPE (distributed_vector), INTENT(IN) :: handle1
REAL(KIND=JPRB), DIMENSION(handle1%local_start:handle1%local_end) ::divide_dv_dv

if (handle1%msetup /= jpsetup .or. handle2%msetup /= jpsetup  )&
     &call ABOR1 ('error: divide unallocated distributed_vector')

if (handle1%global_length == handle2%global_length) then
  divide_dv_dv = handle1%local(:) / handle2%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION divide_dv_dv

FUNCTION power_dv_dv (handle1,handle2)

!         power two distributed_vectors

TYPE (distributed_vector), INTENT(IN) :: handle2
TYPE (distributed_vector), INTENT(IN) :: handle1
REAL(KIND=JPRB), DIMENSION(handle1%local_start:handle1%local_end) ::power_dv_dv

if (handle1%msetup /= jpsetup .or. handle2%msetup /= jpsetup     )&
     &call ABOR1 ('error: power unallocated distributed_vector')

if (handle1%global_length == handle2%global_length) then
  power_dv_dv = handle1%local(:) ** handle2%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION power_dv_dv

FUNCTION add_dv_dv (handle1,handle2)

!         add two distributed_vectors

TYPE (distributed_vector), INTENT(IN) :: handle2
TYPE (distributed_vector), INTENT(IN) :: handle1
REAL(KIND=JPRB), DIMENSION(handle1%local_start:handle1%local_end) ::add_dv_dv

if (handle1%msetup /= jpsetup .or. handle2%msetup /= jpsetup   )&
     &call ABOR1 ('error: add unallocated distributed_vector')

if (handle1%global_length == handle2%global_length) then
  add_dv_dv = handle1%local(:) + handle2%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION add_dv_dv

FUNCTION subtract_dv_dv (handle1,handle2)

!         subtract two distributed_vectors

TYPE (distributed_vector), INTENT(IN) :: handle2
TYPE (distributed_vector), INTENT(IN) :: handle1
REAL(KIND=JPRB), DIMENSION(handle1%local_start:handle1%local_end) ::subtract_dv_dv

if (handle1%msetup /= jpsetup .or. handle2%msetup /= jpsetup    )&
     &call ABOR1 ('error: subtract unallocated distributed_vector')

if (handle1%global_length == handle2%global_length) then
  subtract_dv_dv = handle1%local(:) - handle2%local(:)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION subtract_dv_dv

SUBROUTINE assign_scalar_dv (handle,scalar)

!         copy scalar to distributed_vector

REAL(KIND=JPRB),                      INTENT(IN)    :: scalar
TYPE (distributed_vector), INTENT(INOUT) :: handle

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: store to unallocated distributed_vector')

handle%local(:) = scalar

END SUBROUTINE assign_scalar_dv

FUNCTION multiply_dv_scalar (handle,scalar)

!         multiply scalar and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: scalar
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::&
      &multiply_dv_scalar

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: multiply unallocated distributed_vector')

multiply_dv_scalar = handle%local(:) * scalar

END FUNCTION multiply_dv_scalar

FUNCTION divide_dv_scalar (handle,scalar)

!         divide scalar and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: scalar
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::divide_dv_scalar

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: divide unallocated distributed_vector')

divide_dv_scalar = handle%local(:) / scalar

END FUNCTION divide_dv_scalar

FUNCTION power_dv_scalar (handle,scalar)

!         power scalar and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: scalar
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::power_dv_scalar

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: power unallocated distributed_vector')

power_dv_scalar = handle%local(:) ** scalar

END FUNCTION power_dv_scalar

FUNCTION add_dv_scalar (handle,scalar)

!         add scalar and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: scalar
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::add_dv_scalar

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: add unallocated distributed_vector')

add_dv_scalar = handle%local(:) + scalar

END FUNCTION add_dv_scalar

FUNCTION subtract_dv_scalar (handle,scalar)

!         subtract scalar and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: scalar
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::&
      &subtract_dv_scalar

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: subtract unallocated distributed_vector')

subtract_dv_scalar = handle%local(:) - scalar

END FUNCTION subtract_dv_scalar

FUNCTION multiply_scalar_dv (scalar,handle)

!         multiply scalar and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: scalar
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::&
      &multiply_scalar_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: multiply unallocated distributed_vector')

multiply_scalar_dv = scalar * handle%local(:)

END FUNCTION multiply_scalar_dv

FUNCTION divide_scalar_dv (scalar,handle)

!         divide scalar and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: scalar
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::divide_scalar_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: divide unallocated distributed_vector')

divide_scalar_dv = scalar / handle%local(:)

END FUNCTION divide_scalar_dv

FUNCTION power_scalar_dv (scalar,handle)

!         power scalar and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: scalar
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::power_scalar_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: power unallocated distributed_vector')

power_scalar_dv = scalar ** handle%local(:)

END FUNCTION power_scalar_dv

FUNCTION add_scalar_dv (scalar,handle)

!         add scalar and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: scalar
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::add_scalar_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: add unallocated distributed_vector')

add_scalar_dv = scalar + handle%local(:)

END FUNCTION add_scalar_dv

FUNCTION subtract_scalar_dv (scalar,handle)

!         subtract scalar and distributed_vector

REAL(KIND=JPRB),                      INTENT(IN) :: scalar
TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::&
      &subtract_scalar_dv

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: subtract unallocated distributed_vector')

subtract_scalar_dv = scalar - handle%local(:)

END FUNCTION subtract_scalar_dv

SUBROUTINE assign_dv_ar (pvec,handle)
TYPE (distributed_vector), INTENT(IN)    :: handle
REAL(KIND=JPRB),                      INTENT(OUT)   :: pvec(:)

!         copy a distributed_vector to array

if (handle%msetup /= jpsetup)&
     &call ABOR1('error: copy from unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  pvec(:) = handle%local(:)
else if (SIZE(pvec) == handle%global_length) then
  pvec(:) = dvsection (handle,1,handle%global_length)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END SUBROUTINE assign_dv_ar

FUNCTION dvsection (handle,kstart,kend)

!         copy a section of a distributed_vector to array

INTEGER(KIND=JPIM),                 INTENT(IN)    :: kstart,kend
TYPE (distributed_vector), INTENT(IN)    :: handle
REAL(KIND=JPRB), DIMENSION(kstart:kend)           :: dvsection

INTEGER(KIND=JPIM) :: irecv_counts(nproc)
INTEGER(KIND=JPIM) :: istart,iend,jroc

if (handle%msetup /= jpsetup)&
     &call ABOR1('error: copy from unallocated distributed_vector')

!         work out which bits of the vector to recieve from whom

DO jroc=1,NPROC
  istart = MAX(kstart,jstart(jroc,handle%global_length))
  iend   = MIN(kend  ,jend  (jroc,handle%global_length))
  irecv_counts(jroc) = max(0,iend+1-istart)
ENDDO

istart = MAX(kstart,handle%local_start)
iend   = MIN(kend  ,handle%local_end  )

!         send and recieve bits of the vector and concatenate into dvsection

call gstats(651,0)
call MPL_ALLGATHERV(handle%local(istart:iend),dvsection,irecv_counts,&
 &CDSTRING='Y_D_V dvsection:')
call gstats(651,1)

END FUNCTION dvsection

FUNCTION multiply_dv_ar (handle,pvec)

!         multiply distributed_vector and array

TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::multiply_dv_ar

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: multiply unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  multiply_dv_ar = handle%local(:) * pvec(:)
else if (SIZE(pvec) == handle%global_length) then
  multiply_dv_ar =&
   &handle%local(:)*pvec(handle%local_start:handle%local_end)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION multiply_dv_ar

FUNCTION divide_dv_ar (handle,pvec)

!         divide distributed_vector and array

TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::divide_dv_ar

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: divide unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  divide_dv_ar = handle%local(:) / pvec(:)
else if (SIZE(pvec) == handle%global_length) then
  divide_dv_ar =handle%local(:)/pvec(handle%local_start:handle%local_end)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION divide_dv_ar

FUNCTION power_dv_ar (handle,pvec)

!         power distributed_vector and array

TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::power_dv_ar

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: power unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  power_dv_ar = handle%local(:) ** pvec(:)
else if (SIZE(pvec) == handle%global_length) then
  power_dv_ar =handle%local(:)**pvec(handle%local_start:handle%local_end)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION power_dv_ar

FUNCTION add_dv_ar (handle,pvec)

!         add distributed_vector and array

TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::add_dv_ar

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: add unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  add_dv_ar = handle%local(:) + pvec(:)
else if (SIZE(pvec) == handle%global_length) then
  add_dv_ar =handle%local(:)+pvec(handle%local_start:handle%local_end)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION add_dv_ar

FUNCTION subtract_dv_ar (handle,pvec)

!         subtract distributed_vector and array

TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) ::subtract_dv_ar

if (handle%msetup /= jpsetup)&
     &call ABOR1 ('error: subtract unallocated distributed_vector')

if (SIZE(pvec) == SIZE(handle%local)) then
  subtract_dv_ar = handle%local(:) - pvec(:)
else if (SIZE(pvec) == handle%global_length) then
  subtract_dv_ar =&
   &handle%local(:)-pvec(handle%local_start:handle%local_end)
else
  call ABOR1 ('error: distributed_vector size mismatch')
endif

END FUNCTION subtract_dv_ar

REAL(KIND=JPRB) FUNCTION dot_product_helper (pvec1,pvec2,mystart,myend,klen,nchnks)
REAL(KIND=JPRB),    INTENT(IN), DIMENSION(:) :: pvec1,pvec2
INTEGER(KIND=JPIM), INTENT(IN)  :: mystart,myend,klen,nchnks

INTEGER(KIND=JPIM) :: j,istart,iend,jroc,ierr,imsglen,&
            &ichnk1,ichnk2,i2,iprocs,jj,ichnk
REAL(KIND=JPRB), DIMENSION(nchnks) :: partial,zbuf
INTEGER(KIND=JPIM) :: IRECVCOUNTS(NPROC)


!   calculate the partial dot products for the local chunks


call gstats(1170,0)
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(j,ichnk,i2,jj)
DO j=1,myend-mystart+1,ndv_chunk_size
  ichnk=(j-1)/ndv_chunk_size+1
  i2=MIN(j-1+ndv_chunk_size,myend-mystart+1)
  zbuf(ichnk) = 0.0_JPRB
  do jj=j,i2
    zbuf(ichnk)=zbuf(ichnk)+pvec1(jj)*pvec2(jj)
  enddo
ENDDO
!$OMP END PARALLEL DO
call gstats(1170,1)


IF(NPROC > 1) THEN  
  DO jroc=1,NPROC 
    istart = jstart (JROC,klen)
    iend   = jend   (JROC,klen)
    ichnk1 = 1+(istart-1)/ndv_chunk_size
    ichnk2 = (iend-1+ndv_chunk_size)/ndv_chunk_size
    IRECVCOUNTS(JROC) = ichnk2-ichnk1+1
  ENDDO
  ichnk=IRECVCOUNTS(myproc)
  call gstats(652,0)
  CALL MPL_ALLGATHERV(zbuf(1:ichnk),partial,IRECVCOUNTS,&
   & CDSTRING='DISTRIBUTED VECTOR(DOT_PRODUCT):')
  call gstats(652,1)
ELSE
  partial(:) = zbuf(:)
ENDIF

!   calculate the full dot product

dot_product_helper = SUM (partial)

END FUNCTION dot_product_helper

REAL(KIND=JPRB) FUNCTION dot_product_dv_dv (handle1,handle2)
TYPE (distributed_vector), INTENT(IN) :: handle1,handle2

IF (handle1%msetup /= jpsetup .or. handle2%msetup /= jpsetup     )&
 & call ABOR1('error: dot_product with unallocated distributed_vector')

IF (handle1%global_length  /=  handle2%global_length)&
 &call ABOR1 ('error: dot_product different length distributed_vectors')

dot_product_dv_dv =&
      &dot_product_helper (handle1%local,handle2%local,&
                          &handle1%local_start,handle1%local_end,&
                          &handle1%global_length,handle1%nchnks)
END FUNCTION  dot_product_dv_dv

REAL(KIND=JPRB) FUNCTION dot_product_ar_dv (pvec,handle2)
REAL(KIND=JPRB),                      INTENT(IN) :: pvec(:)
TYPE (distributed_vector), INTENT(IN) :: handle2

IF (handle2%msetup /= jpsetup     ) call ABOR1 &
 &('error: dot_product with unallocated distributed_vector')

if (SIZE(pvec)  ==  SIZE(handle2%local)) then
  dot_product_ar_dv =&
   &dot_product_helper (pvec,handle2%local,&
   &handle2%local_start,handle2%local_end,&
   &handle2%global_length,handle2%nchnks)
elseif (SIZE(pvec)  ==  handle2%global_length) then
  dot_product_ar_dv =&
   &dot_product_helper (pvec(handle2%local_start:&
   &handle2%local_end),handle2%local,&
   &handle2%local_start,handle2%local_end,&
   &handle2%global_length,handle2%nchnks)
else
  call ABOR1('error: dot_product different length distributed_vectors')
endif

END FUNCTION dot_product_ar_dv

REAL(KIND=JPRB) FUNCTION dot_product_dv_ar (handle1,pvec)
TYPE (distributed_vector), INTENT(IN) :: handle1
REAL(KIND=JPRB),                    INTENT(IN) :: pvec(:)

IF (handle1%msetup /= jpsetup     ) call ABOR1 &
 &('error: dot_product with unallocated distributed_vector')

if (SIZE(handle1%local)  ==  SIZE(pvec)) then
  dot_product_dv_ar =&
   &dot_product_helper (handle1%local,pvec,&
   &handle1%local_start,handle1%local_end,&
   &handle1%global_length,handle1%nchnks)
else if (handle1%global_length  ==  SIZE(pvec)) then
  dot_product_dv_ar =&
   &dot_product_helper (handle1%local,&
   &pvec(handle1%local_start:&
   &handle1%local_end),&
   &handle1%local_start,handle1%local_end,&
   &handle1%global_length,handle1%nchnks)
else
  call ABOR1('error: dot_product different length  &
   &distributed_vectors')
endif

END FUNCTION dot_product_dv_ar

REAL(KIND=JPRB) FUNCTION sum_helper (pvec,mystart,myend,klen,nchnks)
REAL(KIND=JPRB),    INTENT(IN), DIMENSION(:) :: pvec
INTEGER(KIND=JPIM), INTENT(IN)  :: mystart,myend,klen,nchnks

INTEGER(KIND=JPIM) :: j,istart,iend,itag,jroc,ierr,&
 &ichnk1,ichnk2,i2,iprocs,iroc,jj,isendr
REAL(KIND=JPRB), DIMENSION(nchnks) :: partial,zbuf

itag    = mt_distributed_vector

!   calculate the partial sums for the local chunks

IF (myend >= mystart) THEN
  ichnk1 = 1+(mystart-1)/ndv_chunk_size
  ichnk2 = ichnk1-1
  
  DO j=1,myend-mystart+1,ndv_chunk_size
    i2=MIN(j-1+ndv_chunk_size,myend-mystart+1)
    ichnk2 = ichnk2+1
    !              partial(ichnk2)= SUM (pvec(j:i2))
    partial(ichnk2)=0.0_JPRB
    do jj=j,i2
      partial(ichnk2)=partial(ichnk2)+pvec(jj)
    enddo
    
  ENDDO

!   broadcast the partial dot products for the local chunks

  DO jroc=0,NPROC-2
    iroc=mod(myproc+jroc,nproc)+1
    CALL MPL_SEND (partial(ichnk1:ichnk2),KDEST=NPRCIDS(iroc), &
     &   KTAG=itag,CDSTRING='DISTRIBUTED VECTOR:')
  ENDDO
ENDIF

!   gather the partial dot products from the other PEs


IF (myend >= mystart) THEN
  iprocs=min(nchnks,nproc)-1
else
  iprocs=min(nchnks,nproc)
endif

DO jroc=1,iprocs
  CALL MPL_RECV(zbuf(1:nchnks),KFROM=isendr, &
   & KTAG=itag,CDSTRING='DISTRIBUTED VECTOR:')
  istart = jstart (isendr,klen)
  iend   = jend   (isendr,klen)
  ichnk1 = 1+(istart-1)/ndv_chunk_size
  ichnk2 = (iend-1+ndv_chunk_size)/ndv_chunk_size
  partial(ichnk1:ichnk2)=zbuf(1:ichnk2+1-ichnk1)
ENDDO

!   calculate the full sum

sum_helper = SUM (partial)

!   synchronize
CALL MPL_BARRIER (CDSTRING='DISTRIBUTED VECTOR SUM:')

END FUNCTION sum_helper

REAL(KIND=JPRB) FUNCTION sum_dv (handle)
TYPE (distributed_vector), INTENT(IN) :: handle

IF (handle%msetup /= jpsetup) call ABOR1 &
 &('error: sum with unallocated distributed_vector')


sum_dv =&
 &sum_helper (handle%local,&
 &handle%local_start,handle%local_end,&
 &handle%global_length,handle%nchnks)

END FUNCTION  sum_dv

REAL(KIND=JPRB) FUNCTION maxval_helper (pvec,mystart,myend,klen,nchnks)
REAL(KIND=JPRB),    INTENT(IN), DIMENSION(:) :: pvec
INTEGER(KIND=JPIM), INTENT(IN)  :: mystart,myend,klen,nchnks

INTEGER(KIND=JPIM) :: j,istart,iend,itag,jroc,ierr,imsglen,isendr,itagr,&
 &ichnk1,ichnk2,i2,iprocs,iroc,jj
REAL(KIND=JPRB), DIMENSION(nchnks) :: partial,zbuf

itag    = mt_distributed_vector

!   calculate the partial maxvals for the local chunks

IF (myend >= mystart) THEN
  ichnk1 = 1+(mystart-1)/ndv_chunk_size
  ichnk2 = ichnk1-1

  DO j=1,myend-mystart+1,ndv_chunk_size
    i2=MIN(j-1+ndv_chunk_size,myend-mystart+1)
    ichnk2 = ichnk2+1
    partial(ichnk2)= MAXVAL (pvec(j:i2))
  ENDDO

!   broadcast the partial max values for the local chunks


  DO jroc=0,NPROC-2
    iroc=mod(myproc+jroc,nproc)+1
    CALL MPL_SEND (partial(ichnk1:ichnk2),KDEST=NPRCIDS(iroc), &
     &   KTAG=itag,CDSTRING='DISTRIBUTED VECTOR:')
  ENDDO
ENDIF

!   gather the partial max values from the other PEs


IF (myend >= mystart) THEN
  iprocs=min(nchnks,nproc)-1
else
  iprocs=min(nchnks,nproc)
endif

DO jroc=1,iprocs
  CALL MPL_RECV(zbuf(1:nchnks),KFROM=isendr, &
   & KTAG=itag,CDSTRING='DISTRIBUTED VECTOR:')
  istart = jstart (isendr,klen)
  iend   = jend   (isendr,klen)
  ichnk1 = 1+(istart-1)/ndv_chunk_size
  ichnk2 = (iend-1+ndv_chunk_size)/ndv_chunk_size
  partial(ichnk1:ichnk2)=zbuf(1:ichnk2+1-ichnk1)
ENDDO

!   calculate the full max value

maxval_helper = MAXVAL (partial)

!   synchronize
CALL MPL_BARRIER (CDSTRING='MAXVAL:')

END FUNCTION maxval_helper

REAL(KIND=JPRB) FUNCTION maxval_dv (handle)
TYPE (distributed_vector), INTENT(IN) :: handle

IF (handle%msetup /= jpsetup) call ABOR1 &
 &('error: maxval with unallocated distributed_vector')


maxval_dv =&
 &maxval_helper (handle%local,&
 &handle%local_start,handle%local_end,&
 &handle%global_length,handle%nchnks)

END FUNCTION  maxval_dv

FUNCTION sqrt_dv (handle)

!         add two distributed_vectors

TYPE (distributed_vector), INTENT(IN) :: handle
REAL(KIND=JPRB), DIMENSION(handle%local_start:handle%local_end) :: sqrt_dv

if (handle%msetup /= jpsetup .or. handle%msetup /= jpsetup   )&
     &call ABOR1 ('error: sqrt unallocated distributed_vector')

sqrt_dv = SQRT(handle%local(:))

END FUNCTION sqrt_dv


SUBROUTINE scatter_vector_dv (handle,kroot,pvec)

!         distribute vector

TYPE (distributed_vector), INTENT(OUT)  :: handle
INTEGER(KIND=JPIM), INTENT(IN)                   :: kroot
REAL(KIND=JPRB),OPTIONAL,           INTENT(IN)   :: pvec(:)
INTEGER(KIND=JPIM) :: itag,jroc,istart,iend,ierr,imsglen,isendr,itagr

if (handle%msetup /= jpsetup)&
 &call ABOR1 &
 &('scatter_vector: copy to unallocated distributed_vector')

itag = mt_distributed_vector

if(kroot == MYPROC) then 
  if(.NOT. PRESENT(pvec)) &
   & call ABOR1 ('scatter_vector: pvec missing from kroot')
  if (SIZE(pvec) /= handle%global_length)&
   &call ABOR1 &
   &('scatter_vector: unequal size')

!         broadcast the local part to the other PEs


  DO jroc=1,NPROC
    IF (jroc /= MYPROC) THEN
      istart=jstart(jroc,handle%global_length)
      iend=jend(jroc,handle%global_length)
      if (iend >= istart) then
        CALL MPL_SEND (pvec(istart:iend),KDEST=NPRCIDS(jroc), &
         &   KTAG=itag,CDSTRING='DISTRIBUTED VECTOR:')
      ENDIF
    ENDIF
  ENDDO

!         copy the local part from pvec

  handle%local(:)=pvec(handle%local_start:handle%local_end)
ELSE

!         collect the local parts from the distributing PE
  
  istart = jstart (myproc,handle%global_length)
  iend   = jend   (myproc,handle%global_length)
  IF (iend >= istart) THEN
    CALL MPL_RECV(handle%local(istart:iend),KOUNT=imsglen, &
     & KTAG=itag,CDSTRING='DISTRIBUTED VECTOR:')
    IF (imsglen /= iend+1-istart)&
     &CALL ABOR1('SCATTER_VECTOR:WRONG LENGTH IN MPL_RECV')
  ENDIF
ENDIF

CALL MPL_BARRIER (CDSTRING='SCATTER_VECTOR:')

END SUBROUTINE scatter_vector_dv

SUBROUTINE scatter_vector_ar (parray,kroot,pvec)

!         distribute vector

REAL(KIND=JPRB), INTENT(OUT)             :: parray(:)
INTEGER(KIND=JPIM), INTENT(IN)           :: kroot
REAL(KIND=JPRB),OPTIONAL, INTENT(IN)     :: pvec(:)
INTEGER(KIND=JPIM) :: itag,ilen,ierr


itag = mt_distributed_vector
ilen = size(parray)

if(kroot == MYPROC) then 
  if(.NOT. PRESENT(pvec))& 
   & call ABOR1 ('scatter_vector: pvec missing from kroot')
  if (SIZE(pvec) /= ilen) call ABOR1('scatter_vector: unequal size')

!         copy  from pvec

  parray(:)=pvec(:)

ENDIF

call gstats(653,0)
CALL MPL_BROADCAST(parray,itag,KROOT=kroot,&
 & CDSTRING='DISTRIBUTED VECTOR (SCATTER_VECTOR):')
call gstats(653,1)

END SUBROUTINE scatter_vector_ar

SUBROUTINE gather_vector (handle,kroot,pvec)

!         distribute vector

TYPE (distributed_vector), INTENT(IN)   :: handle
INTEGER(KIND=JPIM), INTENT(IN)                   :: kroot
REAL(KIND=JPRB),OPTIONAL,           INTENT(OUT)  :: pvec(:)
INTEGER(KIND=JPIM) :: itag,jroc,istart,iend,ist,ien
INTEGER(KIND=JPIM) :: IRECVCOUNTS(NPROC)

if (handle%msetup /= jpsetup)&
 &call ABOR1('gather_vector: copy to unallocated distributed_vector')

DO jroc=1,NPROC
  ist=jstart(jroc,handle%global_length)
  ien=jend(jroc,handle%global_length)
  irecvcounts(jroc) = ien-ist+1
ENDDO

call gstats(654,0)
if(kroot == MYPROC) then 
  if(.NOT. PRESENT(pvec)) &
   & call ABOR1('gather_vector: pvec missing from kroot')
  if (SIZE(pvec) /= handle%global_length)&
   &call ABOR1('gather_vector: unequal size')
  
!         collect the local parts from all other PEs
  
  call MPL_GATHERV(handle%local,kroot=kroot,&
   & KRECVCOUNTS=irecvcounts,PRECVBUF=pvec,&
   & CDSTRING='DISTRIBUTED VECTOR(gather_VECTOR):')
  
ELSE
  call MPL_GATHERV(handle%local,kroot=kroot,&
   & KRECVCOUNTS=irecvcounts,&
   & CDSTRING='DISTRIBUTED VECTOR(gather_VECTOR):')
ENDIF
call gstats(654,1)

END SUBROUTINE gather_vector

END MODULE distributed_vectors_mix
