MODULE SPECTRAL_FIELDS_PARA_MOD

!   Purpose.
!   --------
!     This module contains the distributed routines associated
!     to the SPECTRAL_FIELD type.
!
!   Author.
!   -------
!     Yannick Tremolet
!
!   Modifications.
!   --------------
!     Original   12-Mar-2004
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE SPECTRAL_FIELDS_DATA, ONLY: SPECTRAL_FIELD, NPRTRV, NPRTRW, MYMS, &
                              & LELAM, NULOUT
USE MPL_MODULE

IMPLICIT NONE
PRIVATE
PUBLIC DOT_PRODUCT, SPECTRAL_NORM, SPECTRAL_NORM_LEVS, SPECTRAL_MAX_LEVS

INTERFACE DOT_PRODUCT
MODULE PROCEDURE DOT_PRODUCT_SPEC
END INTERFACE

INTERFACE SPECTRAL_NORM
MODULE PROCEDURE SPECNORM_PRT, SPECNORM_ARRAY, SPECNORM_ARRPRT
END INTERFACE

INTERFACE SPECTRAL_NORM_LEVS
MODULE PROCEDURE SPECNORM_PRT_LEV, SPECNORM_ARRAY_LEV
END INTERFACE

INTERFACE SPECTRAL_MAX_LEVS
MODULE PROCEDURE SPECMAX_PRT_LEV, SPECMAX_ARRAY_LEV
END INTERFACE

! ------------------------------------------------------------------
CONTAINS
! ------------------------------------------------------------------

REAL(KIND=JPRB) FUNCTION dot_product_spec(ydsp1,ydsp2,ldweight)

!   Purpose.
!   --------
!     Scalar product for distributed spectral variable
!                lweight : .T. => spectral inner product (weighted with 2's and 1's)
!                lweight : .F. (or missing) => simple, unweighted dot product
!
!   Author.
!   -------
!     Yannick Tremolet
!
!   Modifications.
!   --------------
!     Original   02-03-19
!     C.Fischer  03-03-05 Aladin spectral treatment added.
!     C. Soci    04-10-07 CY29 cleaning
! ------------------------------------------------------------------

IMPLICIT NONE

TYPE(SPECTRAL_FIELD), INTENT(IN) :: ydsp1, ydsp2
LOGICAL, OPTIONAL, INTENT(IN)    :: ldweight

INTEGER(KIND=JPIM) :: ii,jb,jl,il,ifldl,ifldg,inumlev(NPRTRV),jj
REAL(KIND=JPRB), ALLOCATABLE :: zbufl(:),zbufg(:),zprod3(:)
REAL(KIND=JPRB) :: zprod2, zzprod
LOGICAL :: llweight

! ------------------------------------------------------------------

CALL GSTATS(671,0)

IF (PRESENT(ldweight)) THEN
  llweight=ldweight
ELSE
  llweight=.FALSE.
ENDIF

!yt IF (ydsp1.NEQV.ydsp2) CALL ABOR1('Spectral fields are not compatible')
IF ((ydsp1%nsmax/=ydsp2%nsmax).OR.((ydsp1%nmsmax/=ydsp2%nmsmax).AND.LELAM)) &
    & CALL ABOR1('Scalar product requires fields at the same resolution')

ifldl=ydsp1%nflevl
IF (ydsp1%ns2d>0) ifldl=ifldl+1
ALLOCATE(zbufl(ifldl))
ifldg=ydsp1%nflevg
IF (ydsp1%ns2g>0) ifldg=ifldg+1
ALLOCATE(zbufg(ifldg))
ALLOCATE(zprod3(ydsp1%nflevg))
inumlev(:)=ydsp1%numll(:)
WHERE (ydsp1%ns2l(:)>0) inumlev(:)=inumlev(:)+1

! Scalar product by level
! -----------------------
CALL dot_prod_waves(ydsp1,ydsp2,zbufl,llweight)

! Gather values for all levels
! ----------------------------
CALL MPL_ALLGATHERV(PSENDBUF=zbufl(1:ifldl),PRECVBUF=zbufg,&
                  & KRECVCOUNTS=inumlev,KCOMM=MPL_ALL_LEVS_COMM, &
                  & CDSTRING='SPECTRAL_FIELDS_PARA_MOD:DOT_PRODUCT_SPEC')

! Reorder levels for reproducibility
! ----------------------------------
ii=0
DO jb=1,NPRTRV
  DO jl=1,ydsp1%numll(jb)
    il=ydsp1%nptrll(jb)+jl-1
    ii=ii+1
    zprod3(il)=zbufg(ii)
  ENDDO
  IF (ydsp1%ns2l(jb)>0) THEN
    ii=ii+1
    zprod2=zbufg(ii)
  ENDIF
ENDDO

! Sum all levels for 3D fields
! ----------------------------
zzprod=0.0_JPRB
DO jl=1,ydsp1%nflevg
  zzprod=zzprod+zprod3(jl)
ENDDO
IF (ydsp1%ns2g>0) THEN
  zzprod=zzprod+zprod2
ENDIF

DEALLOCATE(zbufl)
DEALLOCATE(zbufg)
DEALLOCATE(zprod3)

dot_product_spec = zzprod

CALL GSTATS(671,1)
END FUNCTION dot_product_spec

! ------------------------------------------------------------------

SUBROUTINE dot_prod_waves(ydsp1,ydsp2,prodsp,ldweight)

TYPE(SPECTRAL_FIELD), INTENT(IN) :: ydsp1, ydsp2
REAL(KIND=JPRB), INTENT(OUT) :: prodsp(:)
LOGICAL, INTENT(IN) :: ldweight

INTEGER(KIND=JPIM) :: jf, jm, jn, jl, ja, js, iflds, im, ilen, ilens, ileng, jj, j3d
INTEGER(KIND=JPIM) :: ilenr(NPRTRW), iiismax
REAL(KIND=JPRB), ALLOCATABLE :: zbufs(:), zbufr(:)
REAL(KIND=JPRB), ALLOCATABLE :: zallms(:,:)

! ------------------------------------------------------------------

iflds=ydsp1%nflevl
IF (ydsp1%ns2d>0) iflds=iflds+1
IF (SIZE(prodsp)/=iflds) CALL ABOR1('prodsp wrong size')

ilens=iflds*ydsp1%NUMP
ilenr(:)=iflds*ydsp1%numpp(:)
ileng=SUM(ilenr)
ALLOCATE(zbufs(ilens))
ALLOCATE(zbufr(ileng))
IF (LELAM) THEN
  iiismax=ydsp1%nmsmax
ELSE
  iiismax=ydsp1%nsmax
ENDIF
ALLOCATE(zallms(iflds,0:iiismax))
zbufs(:)=0.0_JPRB

! Partial products for local Ms in send buffer
! --------------------------------------------
jl=ydsp1%nflevl
IF (.not.LELAM) THEN
  DO jf=1,ydsp1%ns3d
    jj=0
    DO jm=1,ydsp1%NUMP
      im=MYMS(jm)
      js=ydsp1%NASM0(im)-1
      ilen=2*(ydsp1%NSMAX-im+1)
      DO jn=1,ilen
        zbufs(jj+1:jj+jl) = zbufs(jj+1:jj+jl) &
            & + ydsp1%sp3d(:,js+jn,jf) * ydsp2%sp3d(:,js+jn,jf)
      ENDDO
      jj=jj+jl
    ENDDO
  ENDDO
ELSE
  DO jf=1,ydsp1%ns3d
    jj=0
    DO jm=1,ydsp1%NUMP
      im=MYMS(jm)
      js=ydsp1%NESM0(im)-1
      ilen=ydsp1%NCPL4M(im)
      DO jn=1,ilen
        zbufs(jj+1:jj+jl) = zbufs(jj+1:jj+jl) &
            & + ydsp1%sp3d(:,js+jn,jf) * ydsp2%sp3d(:,js+jn,jf)
      ENDDO
      jj=jj+jl
    ENDDO
  ENDDO
ENDIF

j3d=ydsp1%nflevl*ydsp1%NUMP
IF (.not.LELAM) THEN
  DO jf=1,ydsp1%ns2d
    DO jm=1,ydsp1%NUMP
      im=MYMS(jm)
      js=ydsp1%NASM0(im)-1
      ilen=2*(ydsp1%NSMAX-im+1)
      DO jn=1,ilen
        zbufs(j3d+jm) = zbufs(j3d+jm) &
            & + ydsp1%sp2d(js+jn,jf) * ydsp2%sp2d(js+jn,jf)
      ENDDO
    ENDDO
  ENDDO
ELSE
  DO jf=1,ydsp1%ns2d
    DO jm=1,ydsp1%NUMP
      im=MYMS(jm)
      js=ydsp1%NESM0(im)-1
      ilen=ydsp1%NCPL4M(im)
      DO jn=1,ilen
        zbufs(j3d+jm) = zbufs(j3d+jm) &
            & + ydsp1%sp2d(js+jn,jf) * ydsp2%sp2d(js+jn,jf)
      ENDDO
    ENDDO
  ENDDO
ENDIF
! Communicate to get all the Ms
! -----------------------------
CALL MPL_ALLGATHERV(PSENDBUF=zbufs(1:ilens),PRECVBUF=zbufr,&
                  & KRECVCOUNTS=ilenr,KCOMM=MPL_ALL_MS_COMM, &
                  & CDSTRING='SPECTRAL_FIELDS_PARA_MOD:DOT_PROD_WAVES')

! Reorder Ms for reproducibility
! ------------------------------
jj=0
DO ja=1,NPRTRW
  DO jm=1,ydsp1%numpp(ja)
    im=ydsp1%ms_proc(jm,ja)
    DO jl=1,ydsp1%nflevl
      jj=jj+1
      zallms(jl,im)=zbufr(jj)
    ENDDO
  ENDDO
  IF (ydsp1%ns2d>0) THEN
    jl=ydsp1%nflevl+1
    DO jm=1,ydsp1%numpp(ja)
      im=ydsp1%ms_proc(jm,ja)
      jj=jj+1
      zallms(jl,im)=zbufr(jj)
    ENDDO
  ENDIF
ENDDO

! Total scalar products
! ---------------------

prodsp(:) = 0.0_JPRB

IF (ldweight) then
  DO jm=1,iiismax
    prodsp(:) = prodsp(:) + zallms(:,jm)
  ENDDO
  
  prodsp(:) = 2.0_JPRB*prodsp(:) + zallms(:,0)
ELSE
  DO jm=0,iiismax
    prodsp(:) = prodsp(:) + zallms(:,jm)
  ENDDO
ENDIF

DEALLOCATE(zbufs)
DEALLOCATE(zbufr)
DEALLOCATE(zallms)

END SUBROUTINE dot_prod_waves

! ------------------------------------------------------------------
SUBROUTINE SPECNORM_PRT(ydspec,cdgrep)
! ------------------------------------------------------------------
! Computes norms of distributed spectral fields and prints
! one value per field (3D or 2D). Adds string to output for
! easier grep. Equivalent of SPNORM for type SPECTRAL_FIELD.
! ------------------------------------------------------------------

IMPLICIT NONE
TYPE(SPECTRAL_FIELD), INTENT(IN) :: ydspec
CHARACTER(LEN=*), INTENT(IN) :: cdgrep

REAL(KIND=JPRB), ALLOCATABLE :: znor2(:),znor3(:)
CHARACTER(LEN=32) :: clfmt

! ----------------------------------------------

ALLOCATE(znor3(ydspec%ns3d))
ALLOCATE(znor2(ydspec%ns2g))

CALL SPECNORM(ydspec,znor3,znor2)

clfmt='(2(X,A),5(X,ES18.12))'
IF (ydspec%ns2g>0) WRITE(NULOUT,clfmt)cdgrep,'SP NORMS 2D:',znor2
IF (ydspec%ns3d>0) WRITE(NULOUT,clfmt)cdgrep,'SP NORMS 3D:',znor3

DEALLOCATE(znor2)
DEALLOCATE(znor3)

END SUBROUTINE SPECNORM_PRT

! ------------------------------------------------------------------
SUBROUTINE SPECNORM_ARRAY(ydspec,pnorms)
! ------------------------------------------------------------------
! Computes norms of distributed spectral fields and returns
! one value per field (3D or 2D) in array pnorms.
! ------------------------------------------------------------------

IMPLICIT NONE
TYPE(SPECTRAL_FIELD), INTENT(IN) :: ydspec
REAL(KIND=JPRB), INTENT(OUT) :: pnorms(:)

IF (SIZE(pnorms)/=ydspec%ns3d+ydspec%ns2g) &
  & CALL ABOR1('SPECNORM_ARRAY: pnorms wrong size')

CALL SPECNORM(ydspec,pnorms(1:ydspec%ns3d), &
                  & pnorms(ydspec%ns3d+1:ydspec%ns3d+ydspec%ns2g))

END SUBROUTINE SPECNORM_ARRAY

! ------------------------------------------------------------------
SUBROUTINE SPECNORM_ARRPRT(ydspec,cdgrep,pnorms)
! ------------------------------------------------------------------
! Computes norms of distributed spectral fields and prints one
! value per field (3D or 2D) and returns values in array pnorms.
! ------------------------------------------------------------------

IMPLICIT NONE
TYPE(SPECTRAL_FIELD), INTENT(IN) :: ydspec
CHARACTER(LEN=*), INTENT(IN) :: cdgrep
REAL(KIND=JPRB), INTENT(OUT) :: pnorms(:)

CHARACTER(LEN=32) :: clfmt

IF (SIZE(pnorms)/=ydspec%ns3d+ydspec%ns2g) &
  & CALL ABOR1('SPECNORM_ARRPRT: pnorms wrong size')

CALL SPECNORM(ydspec,pnorms(1:ydspec%ns3d), &
                  & pnorms(ydspec%ns3d+1:ydspec%ns3d+ydspec%ns2g))

clfmt='(2(X,A),5(X,ES18.12))'
IF (ydspec%ns2g>0) WRITE(NULOUT,clfmt)cdgrep,'SP NORMS 2D:', &
                       & pnorms(ydspec%ns3d+1:ydspec%ns3d+ydspec%ns2g)
IF (ydspec%ns3d>0) WRITE(NULOUT,clfmt)cdgrep,'SP NORMS 3D:',pnorms(1:ydspec%ns3d)

END SUBROUTINE SPECNORM_ARRPRT

! ------------------------------------------------------------------
SUBROUTINE SPECNORM_PRT_LEV(ydspec,cdgrep)
! ------------------------------------------------------------------
! Computes norms of distributed spectral fields and prints
! one value per 2D field and one value per level for 3D fields.
! Adds string to output for easier grep.
! ------------------------------------------------------------------

IMPLICIT NONE
TYPE(SPECTRAL_FIELD), INTENT(IN) :: ydspec
CHARACTER(LEN=*), INTENT(IN) :: cdgrep

INTEGER(KIND=JPIM) :: jl
REAL(KIND=JPRB), ALLOCATABLE :: znor2(:),znor3(:,:)
CHARACTER(LEN=32) :: clfmt

! ----------------------------------------------

ALLOCATE(znor3(ydspec%nflevg,ydspec%ns3d))
ALLOCATE(znor2(ydspec%ns2g))

CALL SPECNORM_ARRAY_LEV(ydspec,znor3,znor2)

IF (ydspec%ns2g>0) THEN
  clfmt='(2(X,A),5(X,ES18.12))'
  WRITE(NULOUT,clfmt)cdgrep,'SP NORMS 2D:',znor2
ENDIF

IF (ydspec%ns3d>0) THEN
  clfmt='(2(X,A),X,I3,5(X,ES18.12))'
  DO jl=1,ydspec%nflevg
    WRITE(NULOUT,clfmt)cdgrep,'SP NORMS 3D:',jl,znor3(jl,:)
  ENDDO
ENDIF

DEALLOCATE(znor2)
DEALLOCATE(znor3)

END SUBROUTINE SPECNORM_PRT_LEV

! ------------------------------------------------------------------
SUBROUTINE SPECNORM(ydspec,prod3,prod2)
! ------------------------------------------------------------------
! Performs norms computation
! ------------------------------------------------------------------

IMPLICIT NONE

TYPE(SPECTRAL_FIELD), INTENT(IN) :: ydspec
REAL(KIND=JPRB), INTENT(OUT) :: prod2(:),prod3(:)

INTEGER(KIND=JPIM) :: jf,jl
REAL(KIND=JPRB), ALLOCATABLE :: zprod3(:,:)
REAL(KIND=JPRB) :: zlev

! ------------------------------------------------------------------

IF (SIZE(prod2)/=ydspec%ns2g) CALL ABOR1('SPECNORM: prod2 wrong size')
IF (SIZE(prod3)/=ydspec%ns3d) CALL ABOR1('SPECNORM: prod3 wrong size')

ALLOCATE(zprod3(ydspec%nflevg,ydspec%ns3d))

CALL SPECNORM_ARRAY_LEV(ydspec,zprod3,prod2)

! Average levels for 3D fields
! ----------------------------
zlev=REAL(ydspec%nflevg,JPRB)
DO jf=1,ydspec%ns3d
  prod3(jf)=0.0_JPRB
  DO jl=1,ydspec%nflevg
    prod3(jf)=prod3(jf)+zprod3(jl,jf)
  ENDDO
  prod3(jf)=prod3(jf)/zlev
ENDDO

DEALLOCATE(zprod3)

END SUBROUTINE SPECNORM

! ------------------------------------------------------------------
SUBROUTINE SPECNORM_ARRAY_LEV(ydspec,prod3,prod2)
! ------------------------------------------------------------------
!   Purpose.
!   --------
!     Actual computation of norms of distributed spectral fields.
!     Returns one value per field in prod2 for 2D fields, one value
!     per fields and per level in prod3 for 3D norms.
!
!   Author.
!   -------
!     Yannick Tremolet
!
!   Modifications.
!   --------------
!     Original   11-Mar-2004
!     C. Soci    07-Oct-2004 Aladin spectral treatment added 
! ------------------------------------------------------------------

IMPLICIT NONE

TYPE(SPECTRAL_FIELD), INTENT(IN) :: ydspec
REAL(KIND=JPRB), INTENT(OUT) :: prod2(:)
REAL(KIND=JPRB), INTENT(OUT) :: prod3(:,:)

INTEGER(KIND=JPIM) :: ja,jb,jf,jj,jl,jm,jn,js
INTEGER(KIND=JPIM) :: ii,il,im,ifldl,ifldg,ilen,ilens,ileng,iiismax
INTEGER(KIND=JPIM) :: inumlev(NPRTRV),ilenr(NPRTRW)
REAL(KIND=JPRB), ALLOCATABLE :: zbufl(:),zbufg(:), zbufs(:), zbufr(:)
REAL(KIND=JPRB), ALLOCATABLE :: zallms(:,:)

! ------------------------------------------------------------------

IF (SIZE(prod2)/=ydspec%ns2g) &
 & CALL ABOR1('SPECNORM_ARRAY_LEV: prod2 wrong size')
IF (SIZE(prod3,1)/=ydspec%nflevg) &
 & CALL ABOR1('SPECNORM_ARRAY_LEV: prod3 wrong size 1')
IF (SIZE(prod3,2)/=ydspec%ns3d)   &
 & CALL ABOR1('SPECNORM_ARRAY_LEV: prod3 wrong size 2')

ifldl=ydspec%nflevl*ydspec%ns3d+ydspec%ns2d

inumlev(:)=ydspec%numll(:)*ydspec%ns3d+ydspec%ns2l(:)

! -----------------------------
! Scalar product level by level
! -----------------------------

ilens=ifldl*ydspec%NUMP
ilenr(:)=ifldl*ydspec%numpp(:)
ileng=SUM(ilenr)
ALLOCATE(zbufs(ilens))
ALLOCATE(zbufr(ileng))
IF (LELAM) THEN
  iiismax=ydspec%nmsmax
ELSE
  iiismax=ydspec%nsmax
ENDIF
zbufs(:)=0.0_JPRB

! Partial products for local Ms in send buffer
! --------------------------------------------
jj=0
IF (.not.LELAM) THEN
  DO jm=1,ydspec%NUMP
    im=MYMS(jm)
    js=ydspec%NASM0(im)-1
    ilen=2*(ydspec%NSMAX-im+1)
    DO jf=1,ydspec%ns3d
      DO jn=1,ilen
        DO jl=1,ydspec%nflevl
          zbufs(jj+jl) = zbufs(jj+jl) &
              & + ydspec%sp3d(jl,js+jn,jf) * ydspec%sp3d(jl,js+jn,jf)
        ENDDO
      ENDDO
      jj=jj+ydspec%nflevl
    ENDDO
  ENDDO
ELSE
  DO jm=1,ydspec%NUMP
    im=MYMS(jm)
    js=ydspec%NESM0(im)-1
    ilen=ydspec%NCPL4M(im)
    DO jf=1,ydspec%ns3d
      DO jn=1,ilen 
        DO jl=1,ydspec%nflevl
          zbufs(jj+jl) = zbufs(jj+jl) &
              & + ydspec%sp3d(jl,js+jn,jf) * ydspec%sp3d(jl,js+jn,jf)      
        ENDDO 
      ENDDO
      jj=jj+ydspec%nflevl
    ENDDO   
  ENDDO
ENDIF

IF (.not.LELAM) THEN
  DO jm=1,ydspec%NUMP
    im=MYMS(jm)
    js=ydspec%NASM0(im)-1
    ilen=2*(ydspec%NSMAX-im+1)
    DO jf=1,ydspec%ns2d
      jj=jj+1
      DO jn=1,ilen
        zbufs(jj) = zbufs(jj) &
            & + ydspec%sp2d(js+jn,jf) * ydspec%sp2d(js+jn,jf)
      ENDDO
    ENDDO
  ENDDO
ELSE
  DO jm=1,ydspec%NUMP
    im=MYMS(jm) 
    js=ydspec%NESM0(im)-1
    ilen=ydspec%NCPL4M(im)
    DO jf=1,ydspec%ns2d
      jj=jj+1
      DO jn=1,ilen
        zbufs(jj) = zbufs(jj) &
            & + ydspec%sp2d(js+jn,jf) * ydspec%sp2d(js+jn,jf)
      ENDDO
    ENDDO
  ENDDO 
ENDIF

IF (jj/=ilens) CALL ABOR1('SPECNORM_ARRAY_LEV: wrong ilens')

! Communicate to get all the Ms
! -----------------------------
CALL MPL_ALLGATHERV(PSENDBUF=zbufs,PRECVBUF=zbufr,&
                  & KRECVCOUNTS=ilenr,KCOMM=MPL_ALL_MS_COMM, &
                  & CDSTRING='SPECTRAL_FIELDS_PARA_MOD:SPECNORM_ARRAY_LEV')

ALLOCATE(zallms(ifldl,0:iiismax))

! Reorder Ms for reproducibility
! ------------------------------
jj=0
DO ja=1,NPRTRW
  DO jm=1,ydspec%numpp(ja)
    im=ydspec%ms_proc(jm,ja)
    DO jl=1,ydspec%ns3d*ydspec%nflevl
      jj=jj+1
      zallms(jl,im)=zbufr(jj)
    ENDDO
  ENDDO
  DO jm=1,ydspec%numpp(ja)
    im=ydspec%ms_proc(jm,ja)
    DO jf=1,ydspec%ns2d
      jl=ydspec%ns3d*ydspec%nflevl+jf
      jj=jj+1
      zallms(jl,im)=zbufr(jj)
    ENDDO
  ENDDO
ENDDO

DEALLOCATE(zbufs)
DEALLOCATE(zbufr)

! Total scalar products for each level
! ------------------------------------
ALLOCATE(zbufl(ifldl))
zbufl(:) = 0.0_JPRB
DO jm=1,iiismax  ! Start from M=1
  zbufl(:) = zbufl(:) + zallms(:,jm)
ENDDO

! Apply weight for computing spectral norm
if (.not.LELAM) zbufl(:) = 2.0_JPRB * zbufl(:)

! Weights do not apply to M=0
zbufl(:) = zbufl(:) + zallms(:,0)

! Square root
zbufl(:) = SQRT( zbufl(:) )

DEALLOCATE(zallms)
! ------------------------------------------------------------------

! Gather values for all levels
! ----------------------------
ifldg=ydspec%nflevg*ydspec%ns3d+ydspec%ns2g
ALLOCATE(zbufg(ifldg))

CALL MPL_ALLGATHERV(PSENDBUF=zbufl,PRECVBUF=zbufg,&
                  & KRECVCOUNTS=inumlev,KCOMM=MPL_ALL_LEVS_COMM, &
                  & CDSTRING='SPECTRAL_FIELDS_PARA_MOD:SPECNORM_ARRAY_LEV')

! Reorder levels for reproducibility
! ----------------------------------
ii=0
DO jb=1,NPRTRV
  DO jf=1,ydspec%ns3d
    DO jl=1,ydspec%numll(jb)
      il=ydspec%nptrll(jb)+jl-1
      ii=ii+1
      prod3(il,jf)=zbufg(ii)
    ENDDO
  ENDDO
  DO jf=1,ydspec%ns2l(jb)
    ii=ii+1
    prod2(jf)=zbufg(ii)
  ENDDO
ENDDO

! Finish
! ------
DEALLOCATE(zbufl)
DEALLOCATE(zbufg)

END SUBROUTINE SPECNORM_ARRAY_LEV

! ------------------------------------------------------------------
SUBROUTINE SPECMAX_PRT_LEV(ydspec,cdgrep)
! ------------------------------------------------------------------

IMPLICIT NONE
TYPE(SPECTRAL_FIELD), INTENT(IN) :: ydspec
CHARACTER(LEN=*), INTENT(IN) :: cdgrep

INTEGER(KIND=JPIM) :: jl
REAL(KIND=JPRB), ALLOCATABLE :: znor2(:),znor3(:,:)
CHARACTER(LEN=32) :: clfmt

! ----------------------------------------------

ALLOCATE(znor3(ydspec%nflevg,ydspec%ns3d))
ALLOCATE(znor2(ydspec%ns2g))

CALL SPECMAX_ARRAY_LEV(ydspec,znor3,znor2)

IF (ydspec%ns2g>0) THEN
  clfmt='(2(X,A),5(X,ES18.12))'
  WRITE(NULOUT,clfmt)cdgrep,'SP MAX 2D:',znor2
ENDIF

IF (ydspec%ns3d>0) THEN
  clfmt='(2(X,A),X,I3,5(X,ES18.12))'
  DO jl=1,ydspec%nflevg
    WRITE(NULOUT,clfmt)cdgrep,'SP MAX 3D:',jl,znor3(jl,:)
  ENDDO
ENDIF

DEALLOCATE(znor2)
DEALLOCATE(znor3)

END SUBROUTINE SPECMAX_PRT_LEV

! ------------------------------------------------------------------
SUBROUTINE SPECMAX_ARRAY_LEV(ydspec,prod3,prod2)
! ------------------------------------------------------------------
!   Purpose.
!   --------
!     Actual computation of max of distributed spectral fields.
!     Returns one value per field in prod2 for 2D fields, one value
!     per fields and per level in prod3 for 3D norms.
!
!   Author.
!   -------
!     Yannick Tremolet
!
!   Modifications.
!   --------------
!     Original   02-Nov-2004
! ------------------------------------------------------------------

IMPLICIT NONE

TYPE(SPECTRAL_FIELD), INTENT(IN) :: ydspec
REAL(KIND=JPRB), INTENT(OUT) :: prod2(:)
REAL(KIND=JPRB), INTENT(OUT) :: prod3(:,:)

INTEGER(KIND=JPIM) :: jf,jj,jl,js,ja,jb,ii,il,ifldl,ifldg
INTEGER(KIND=JPIM) :: inumlev(NPRTRV)
REAL(KIND=JPRB), ALLOCATABLE :: zbufl(:),zbufg(:)

! ------------------------------------------------------------------

IF (SIZE(prod2)/=ydspec%ns2g) &
 & CALL ABOR1('SPECTRAL_FIELDS_PARA_MOD:SPECMAX_ARRAY_LEV: prod2 wrong size')
IF (SIZE(prod3,1)/=ydspec%nflevg) &
 & CALL ABOR1('SPECTRAL_FIELDS_PARA_MOD:SPECMAX_ARRAY_LEV: prod3 wrong size 1')
IF (SIZE(prod3,2)/=ydspec%ns3d)   &
 & CALL ABOR1('SPECTRAL_FIELDS_PARA_MOD:SPECMAX_ARRAY_LEV: prod3 wrong size 2')

! -----------------------------
! Max level by level
! -----------------------------
ifldl=ydspec%nflevl*ydspec%ns3d+ydspec%ns2d
ALLOCATE(zbufl(ifldl))
zbufl(:)=0.0_JPRB

jj=0
DO jf=1,ydspec%ns3d
  DO js=1,ydspec%nspec2
    DO jl=1,ydspec%nflevl
      zbufl(jj+jl) = MAX(zbufl(jj+jl), ABS(ydspec%sp3d(jl,js,jf)))
    ENDDO
  ENDDO
  jj=jj+ydspec%nflevl
ENDDO

DO jf=1,ydspec%ns2d
  jj=jj+1
  DO js=1,ydspec%nspec2
    zbufl(jj) = MAX(zbufl(jj), ABS(ydspec%sp2d(js,jf)))
  ENDDO
ENDDO

! Communicate to get global max
! -----------------------------
CALL MPL_ALLREDUCE(PSENDBUF=zbufl,CDOPER='MAX',KCOMM=MPL_ALL_MS_COMM, &
                 & CDSTRING='SPECTRAL_FIELDS_PARA_MOD:SPECMAX_ARRAY_LEV')

! Gather values for all levels
! ----------------------------
ifldg=ydspec%nflevg*ydspec%ns3d+ydspec%ns2g
ALLOCATE(zbufg(ifldg))
inumlev(:)=ydspec%numll(:)*ydspec%ns3d+ydspec%ns2l(:)

CALL MPL_ALLGATHERV(PSENDBUF=zbufl,PRECVBUF=zbufg,&
                  & KRECVCOUNTS=inumlev,KCOMM=MPL_ALL_LEVS_COMM, &
                  & CDSTRING='SPECTRAL_FIELDS_PARA_MOD:SPECMAX_ARRAY_LEV')

! Reorder levels
! --------------
ii=0
DO jb=1,NPRTRV
  DO jf=1,ydspec%ns3d
    DO jl=1,ydspec%numll(jb)
      il=ydspec%nptrll(jb)+jl-1
      ii=ii+1
      prod3(il,jf)=zbufg(ii)
    ENDDO
  ENDDO
  DO jf=1,ydspec%ns2l(jb)
    ii=ii+1
    prod2(jf)=zbufg(ii)
  ENDDO
ENDDO

! Finish
! ------
DEALLOCATE(zbufl)
DEALLOCATE(zbufg)

END SUBROUTINE SPECMAX_ARRAY_LEV
! ------------------------------------------------------------------

END MODULE SPECTRAL_FIELDS_PARA_MOD
