SUBROUTINE CPG_END( &
 !---------------------------------------------------------------------
 ! - INPUT .
 & CDCONF, &
 & KST,KEND,KSTC,KENDC,KBL,KSTGLO, &
 & LDLADNMI,LDLDIAB, &
 & PGM, &
 & PD3Y0,PRRED0,PGPHL,PGPHM,PWW0, &
 & PQS, &
 !---------------------------------------------------------------------
 ! - INPUT/OUTPUT .
 & PGFL,PGPAR,PGPPC,PGMV,PGMVS)

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE PTRGPPC  , ONLY : MGPPCO_Y ,MGPPCO_GL,MGPPCO_GM,MGPPCO_RR,MGPPCO_WW
USE YOMGPPCB , ONLY : GPPCBUF
USE YOEPHY   , ONLY : LAGPHY
USE YOMCT0   , ONLY : NSTOP    ,LSLAG    ,LTWOTL   ,LPC_OLD  , &
 &                    LPC_FULL ,LTENC
USE YOMCT3   , ONLY : NSTEP
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NPROMNH  ,NFGPPC
USE YOMDYN   , ONLY : NCURRENT_ITER
USE YOMGEM   , ONLY : RSTRET
USE YOMGPPB  , ONLY : GPARBUF
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMIOP   , ONLY : NG2DS5
USE YOMSIMPHL, ONLY : LTRAJPS
USE YOM_YGFL , ONLY : YGFL
USE YOMGMV   , ONLY : YT0,YT9,NDIMGMV,NDIMGMVS
USE YOMARAR  , ONLY : NGPAR
USE YOMARPHY , ONLY : LMSE
USE SURFACE_FIELDS_MIX  , ONLY :  GPPOPER ,&
 & SP_RR, YSP_RR, SP_SG, YSP_SG

!**** *CPG_END* - Grid point calculations: end of non lagged part.

!     Purpose.
!     --------
!           Grid point calculations: end of non lagged part.
!           - save quantities at t and t-dt in buffers.
!           - save soil fields in a buffer.
!           - divide p-order derivatives (p=1 or 2) by M**p.
!           - temporal filter 

!           Abbreviation "vwv" stands for "vertical wind variable".

!**   Interface.
!     ----------
!        *CALL* *CPG_END(...)*

!        Explicit arguments :
!        --------------------

!     INPUT:
!     ------
!        CDCONF    : configuration.
!        KST       : first element of work.
!        KEND      : last element of work.
!        KSTC,KENDC: the same as KST,KEND but including zone C for ALADIN.
!        KBL       : block number.
!        KSTGLO    : global offset.
!        LDLADNMI  : .T. if adiabatic NNMI iteration (CDCONF is 'E' or 'F').
!        LDLDIAB   : .T. if complete physics is activated, and predictor step.
!        PGM       : mapping factor.
!        PD3Y0     : term 'Y' at t.
!        PRRED0    : reduced gas constant by (1.+Prond) in NH at t.
!        PGPHL     : semi-reduced zonal component of "gz" gradient
!                    on interlayers.
!        PGPHM     : semi-reduced meridian component of "gz" gradient
!                    on interlayers.
!        PWW0      : weights to compute (U,V) on interlayers.
!        PQS       : specific humidity at surface level.

!     INPUT/OUTPUT:
!     -------------
!        PGFL      : GFL variables at time t-dt and t.
!        PGPAR     : surface fields for AROME.
!        PGPPC     : buffer containing some fields used in the PC schemes.
!        PGMV      : upper air GMV variables at time t and t-dt.
!        PGMVS     : surface GMV variables at time t and t-dt.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ARPEGE documentation vol 2 ch 1 and vol 3 ch 6

!     Author.
!     -------
!        K. YESSAD after the old part 9 of CPG. 

! Modifications
! -------------
!   Original : 01-08-14
!   04 JAN 2002: K.YESSAD - pruning; thin layer hyp relaxation for SL.
!   03-2002, J. Vivoda: PC schemes for NH dynamics (LPC_XXXX keys)
!   30-09-02 P.Smolikova : variable d4 in NH
!   12-10-2002, J. Masek : PC bugfix.
!   Modified 08-2002 C. Smith : use "w" as prognostic variable in the
!    semi-lag advection of vertical divergence in the NH model.
!   18 SEP 2002: C.MOUSSY - Surface pressure and its derivatives
!   31-01-03 P.Smolikova : variable d4 in 2tl PC NH
!   25-02-03 J.Vivoda : corrected test for PC-scheme
!   Modified 2003-07-17 C. Fischer - psvdauxt0* come from pgmv
!   01-Oct-2003 M. Hamrud   CY28 Cleaning
!   10-Jan-2004 M. Hamrud   CY28R1 Cleaning
!   Jan-2004 K. YESSAD   split GP_SPV into GP_SPV + GPMPFC_GMVS.
!   20-Jan-2004 JM. Audoin  reactive Tendency surface pressure
!   09-Jun-2004 J. Masek    NH cleaning (LFULLIMP)
!   01-Jul-2004 K. Yessad  Make clearer the tests for PC scheme.
!   2004-11-16 Y. Seity - save surface fields for AROME in GPARBUF 
!   M.Hamrud      01-Jul-2006 Revised surface fields
!   06-Mar-2006 R. Zaaboul  Add LMSE when saving PGPAR for AROME
!   17-Apr-2007 S.Ivatek-S: Over dimensioning of PGPAR to NGPAR+1 and PGPNH to
!                NFGPNH+1, boundary checking problem if NGPAR=0 or NFGPNH=0 bf
!   31-08-2007 Y.Seity : remove LMPA key for GPARBUF reading 
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   K. Yessad (Dec 2008): remove dummy CDLOCK
! End Modifications
!------------------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KENDC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBL
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
LOGICAL           ,INTENT(IN)    :: LDLADNMI 
LOGICAL           ,INTENT(IN)    :: LDLDIAB 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PD3Y0(NPROMNH,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRRED0(NPROMNH,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGPHL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGPHM(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWW0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGPAR(NPROMA,NGPAR+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGPPC(NPROMA,NFGPPC+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVS(NPROMA,NDIMGMVS)

!     ------------------------------------------------------------------

LOGICAL    :: LLSTR

INTEGER(KIND=JPIM) :: IFLAG, ISTEP, JLEV
REAL(KIND=JPRB) :: ZEPS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "gpmpfc.intfb.h"
#include "gpmpfc_gmvs.intfb.h"
#include "sc2wrg.intfb.h"
#include "wrphtrsf.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG_END',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*    1.    PRELIMINARY INITIALISATIONS.
!           ----------------------------

ZEPS=100.0_JPRB*TINY(1.0_JPRB)
LLSTR=(ABS(RSTRET-1.0_JPRB)>ZEPS)

!     ------------------------------------------------------------------

!*    2.    SURFACE FIELDS TIMESTEPPING.
!           ----------------------------

! NO TIME-STEPPING FOR SURFACE FIELDS IF ADIABATIC CALL OR NNMI ITERATION OR
! NHS ITERATION.
IF (NCURRENT_ITER == 0) THEN
  IF ((.NOT.LDLADNMI) &
     & .AND.(CDCONF /= 'M').AND.(CDCONF /= 'X').AND.LDLDIAB) THEN  
    IF (LTWOTL) THEN
      CALL GPPOPER('SET0TO1', KBL=KBL)
    ELSE
      CALL GPPOPER('PHTFILT', KBL=KBL)
    ENDIF
  ENDIF
ENDIF

IF ((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)) THEN
  IF ((LTRAJPS).AND.(NSTEP == (NSTOP-1)) .AND.(.NOT.LDLADNMI)) THEN  
    ISTEP=NSTEP+1
    CALL WRPHTRSF(ISTEP,KST,KEND,KSTGLO,NG2DS5,&
     & PQS,SP_RR(1,YSP_RR%YT%MP9,KBL),SP_SG(1,YSP_SG%YF%MP9,KBL))  
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*    3.    DIVIDE BY MAPPING FACTOR.
!           -------------------------

IF (LLSTR.AND.(.NOT.LAGPHY).AND.(.NOT.LDLADNMI)) THEN

  IFLAG=1

  CALL GPMPFC(&
   ! --- INPUT ---------------------------------------------------------
   & KST,KEND,IFLAG,PGM,&
   ! --- INPUT/OUTPUT --------------------------------------------------
   & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),PGMV(1,1,YT0%MDIV),&
   & PGMV(1,1,YT0%MTL),PGMV(1,1,YT0%MTM),&
   & PGMV(1,1,YT0%MSPDL),PGMV(1,1,YT0%MSPDM),&
   & PGMV(1,1,YT0%MSVDL),PGMV(1,1,YT0%MSVDM),PGMV(1,1,YT0%MNHXL),&
   & PGMV(1,1,YT0%MNHXM),&
   & PGMV(1,1,YT9%MU)   ,PGMV(1,1,YT9%MV)   ,PGMV(1,1,YT9%MDIV),&
   & PGMV(1,1,YT9%MTL)  ,PGMV(1,1,YT9%MTM)  ,PGMV(1,1,YT9%MSPDL),&
   & PGMV(1,1,YT9%MSPDM),PGMV(1,1,YT9%MSVDL),PGMV(1,1,YT9%MSVDM),&
   & PGMV(1,1,YT0%MUL),PGMV(1,1,YT0%MVL),PGMV(1,1,YT0%MVOR),&
   & PGMV(1,1,YT9%MUL),PGMV(1,1,YT9%MVL),PGMV(1,1,YT9%MVOR),&
   & PGFL)

  CALL GPMPFC_GMVS(&
   ! --- INPUT ---------------------------------------------------------
   & KST,KEND,IFLAG,PGM,&
   ! --- INPUT/OUTPUT --------------------------------------------------
   & PGMVS(1,YT0%MSPL),PGMVS(1,YT0%MSPM),&
   & PGMVS(1,YT9%MSPL),PGMVS(1,YT9%MSPM))

ENDIF

!     ------------------------------------------------------------------

!*    4.    SURFACE PRESSURE AND ITS DERIVATIVES.
!           -------------------------------------

IF (LTWOTL.AND.LTENC) THEN
! CALL ABOR1('CPG_END : THIS MAY NOT BE CORRECT SPT9 ARRAYS /GFL')
  PGMVS(KST:KEND,YT9%MSP)=PGMVS(KST:KEND,YT0%MSP)
  PGMVS(KST:KEND,YT9%MSPL)=PGMVS(KST:KEND,YT0%MSPL)
  PGMVS(KST:KEND,YT9%MSPM)=PGMVS(KST:KEND,YT0%MSPM)
ENDIF

!     ------------------------------------------------------------------

!*    5.    "WRITE" TO WORK FILES.
!           ----------------------

! save Arome surface fields in GPARBUF
IF(NCURRENT_ITER == 0)THEN
  IF (LMSE.AND.NGPAR/=0) THEN
    CALL SC2WRG(GPARBUF,KSTGLO,KSTC,KENDC,PGPAR(1:NPROMA,1:NGPAR))
  ENDIF
ENDIF

!*  Save the intermediate "strangers" for the corrective NHS
!   iteration, if needed.

IF (NCURRENT_ITER == 0 .AND. LPC_OLD) THEN
  IF(.NOT.LSLAG)THEN
    DO JLEV=1,NFLEVG
      PGPPC(KST:KEND,MGPPCO_Y+JLEV-1)=PD3Y0 (KST:KEND,JLEV  )
    ENDDO
  ENDIF
  DO JLEV=1,NFLEVG
    PGPPC(KST:KEND,MGPPCO_RR+JLEV-1)=PRRED0(KST:KEND,JLEV  )
    PGPPC(KST:KEND,MGPPCO_GL+JLEV-1)=PGPHL (KST:KEND,JLEV-1)
    PGPPC(KST:KEND,MGPPCO_GM+JLEV-1)=PGPHM (KST:KEND,JLEV-1)
    PGPPC(KST:KEND,MGPPCO_WW+JLEV-1)=PWW0  (KST:KEND,JLEV  )
  ENDDO
  CALL SC2WRG(GPPCBUF,KSTGLO,KSTC,KENDC,PGPPC(1:NPROMA,1:NFGPPC))
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG_END',1,ZHOOK_HANDLE)
END SUBROUTINE CPG_END
