#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE CPG_GPB_NHGEOGW( &
 !---------------------------------------------------------------------
 ! - INPUT .
 & KST,KEND,LDLDIAB,PRCORI, &
 & PRATATX,PRATATH,POROG,POROGL,POROGM,POROGLL,POROGMM,POROGLM, &
 !---------------------------------------------------------------------
 ! - INPUT/OUTPUT .
 & PGFL,PGMV, &
 !---------------------------------------------------------------------
 ! - OUTPUT .
 & PRE0,PRE0L,PRE0M,PRE0F,PNHPRE0F,PNHPRE0H,PDELP0,PRDELP0,PLNPR0,PALPH0, &
 & PHI0,PHIF0,PHI0FL,PHI0FM,PCP0,PR0,PRTGR0,PKAP0,PRRED0, &
 & PEVEL0,PVVEL0,PGWFT0,PSDVBC0,PKENE0, &
 & PSGRTL,PSGRTM,PGPHL,PGPHM,PSDIV0,PDIVP0,PSDFI0,PRT0L,PRT0M, &
 & PRE9,PRE9F,PNHPRE9F,PNHPRE9H,PDELP9,PRDELP9,PLNPR9,PALPH9, &
 & PHI9,PHIF9,PCP9,PR9,PGWFT9, &
 & PTNDPD,PTNDVD,P3DIVG,PDBBC,&
 & PNHXT0,PNHXT9,PQCHA0L,PQCHA0M)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NPROMNH  ,NPROMVC
USE YOMCT0   , ONLY : LTWOTL   ,LSPRT    ,LRUBC    ,LPC_FULL ,LRPLANE
USE YOMCST   , ONLY : RD       ,RV
USE YOMDYN   , ONLY : NCURRENT_ITER
USE YOMDYNA  , ONLY : NPDVAR   ,NVDVAR   ,LGWOPT1
USE YOMGEM   , ONLY : VBF      ,VBH      ,VAH      ,VC       ,VDELB
USE YOM_YGFL , ONLY : YGFL     ,YQ
USE YOMSIMPHL, ONLY : LSIMPH
USE YOMCVER  , ONLY : LVERTFE
USE YOMGMV   , ONLY : YT0      ,YT9      ,NDIMGMV

!**** *CPG_GPB_NHGEOGW* - Grid point calculations:
!                initial part of not lagged grid-point calculations.
!                cf. part 3 of CPG_GP but for the Phi-gw NH model only.

!     Purpose.
!     --------
!      See above.
!      Paragraph numbering follows what is done in part 3 of CPG_GP, so
!      do not be surprised if numbering skips some numbers. 

!**   Interface.
!     ----------
!        *CALL* *CPG_GPB_NHGEOGW(...)*

!        Explicit arguments :
!        --------------------

!     INPUT:
!     ------
!        KST       : first element of work.
!        KEND      : last element of work.
!        LDLDIAB   : .T. if complete physics is activated and predictor step.
!        PRCORI    : f = 2 Omega sin(theta).
!        PRATATX   : spherical geometry: zero
!                    plane geometry: 2 S (sin(theta) - K_L)/(a cos(theta))
!                    Used for curvature terms.
!        PRATATH   : spherical geometry: 2 tan(theta_bne)/a
!                    (theta_bne = tilted but not stretched sphere latitude)
!                    plane geometry: 2 C (sin(theta) - K_L)/(a cos(theta))
!                    Used for curvature terms.
!        POROG     : surface orography.
!        POROGL    : zonal component of "grad(surf orography)"
!        POROGM    : meridian component of "grad(surf orography)"
!        POROGLL   : second order zonal der. of the orography.
!        POROGMM   : second order merid der. of the orography.
!        POROGLM   : second order crossed der. of the orography.

!     INPUT/OUTPUT:
!     -------------
!        PGFL      : unified_treatment grid-point fields at t
!        PGMV      : upper air GMV variables at time t and t-dt.

!     OUTPUT:
!     -------
!        PRE0      : hydrostatic pressure "prehyd" at half levels at time t.
!        PRE0L     : zonal component of "grad prehyds" at t.
!        PRE0M     : meridian component of "grad prehyds" at t.
!        PRE0F     : hydrostatic pressure "prehyd" at full levels at time t.
!        PNHPRE0F  : "pre" at full levels (time t).
!        PNHPRE0H  : "pre" at half levels (time t).
!        PDELP0    : hydrostatic pressure depth of layers at t.
!        PRDELP0   : 1/(hydrostatic pressure depth of layers) at t.
!        PLNPR0    : "delta" at full levels at time t.
!        PALPH0    : "alpha" at full levels at time t.
!        PHI0      : geopotential height "gz" at t at half levels.
!        PHIF0     : geopotential height "gz" at t at full levels.
!        PHI0FL    : zonal component of "grad (gz)" at t at full levels.
!        PHI0FM    : meridian component of "grad (gz)" at t at full levels.
!        PCP0      : "cp" at t.
!        PR0       : air constant "R" at t.
!        PRTGR0    : scalar ratio of vectors "(grad prehyd)/prehyd" at t.
!        PKAP0     : "kappa = R/cp" at t.
!        PRRED0    : reduced gas constant by (1.+Prond) in NH at t.
!        PEVEL0    : "etadot d prehyd / d eta" at t.
!                    at half levels (resp. layers) if LVERTFE=F (resp. T).
!        PVVEL0    : "omega/prehyd" at full levels at t.
!        PGWFT0    : [gw] at full layers at t.
!        PSDVBC0   : vertical integral of divergence computed in "gpcty",
!                    including the "lrubc" and "delta m=1" contributions
!                    of "etadot d prehyd / d eta".
!        PKENE0    : kinetic energy at t.
!        PSGRTL    : zonal component of the pressure force grad.
!        PSGRTM    : merid. component of the pressure force grad.
!        PGPHL     : zonal component of "gz" gradient
!                    at half levels (cf. PHIHL in GPGRGEO (time t)).
!        PGPHM     : meridian component of "gz" gradient
!                    at half levels (cf. PHIHM in GPGRGEO (time t)).
!        PSDIV0    : vertical integral of divergence without
!                   "lrubc" and "delta m=1" contributions.
!        PDIVP0    : "grad( vec(V) * (Delta prehyd) )" at full levels
!        PSDFI0    : vertical integral of divergence with
!                    "lrubc" and with "delta m=1" contributions
!                    for "etadot d prehyd / d eta", and with the
!                    physics of "delta m=1".
!        PRT0L     : zonal component of "grad RT" at full levels.
!        PRT0M     : meridian component of "grad RT" at full levels.
!        PRE9      : hydrostatic pressure "prehyd" at half levels at t-dt.
!        PRE9F     : hydrostatic pressure "prehyd" at full levels at time t-dt.
!        PNHPRE9F  : "pre" at full levels (time t-dt).
!        PNHPRE9H  : "pre" at half levels (time t-dt).
!        PDELP9    : hydrostatic pressure depth of layers at t-dt.
!        PRDELP9   : 1/(hydrostatic pressure depth of layers) at t-dt.
!        PLNPR9    : "delta" at full levels at time t-dt.
!        PALPH9    : "alpha" at full levels at time t-dt.
!        PHI9      : geopotential height "gz" at t-dt at half levels.
!        PHIF9     : geopotential height "gz" at t-dt at full levels.
!        PCP9      : "cp" at t-dt.
!        PR9       : air constant "R" at t-dt.
!        PGWFT9    : [gw] at full layers at t-dt.
!        PTNDPD    : [D(pressure departure variable)/Dt]_adiab
!        PTNDVD    : [D(vertical divergence variable)/Dt]_adiab
!        P3DIVG    : D3 at t (full levels).
!        PDBBC     : [D(gw)_surf/Dt]_adiab.
!        PNHXT0    : term 'NHX' at t.
!        PNHXT9    : term 'NHX' at t-dt.
!        PQCHA0L   : zonal comp grad(log(pre/prehyd)).
!        PQCHA0M   : merid comp grad(log(pre/prehyd)).

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ARPEGE documentation

!     Author.
!     -------
!        K. YESSAD, after part 3 of CPG_GP. 

! Modifications
! -------------
! Original : April 2008
! K. Yessad (Sep 2008): prune enhanced diffusion (lfrein).
! K. Yessad (Dec 2008): remove useless dummy arguments.
! End Modifications
!------------------------------------------------------------------

IMPLICIT NONE

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
LOGICAL           ,INTENT(IN)    :: LDLDIAB 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRATATX(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRATATH(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGLL(NPROMNH)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGMM(NPROMNH)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGLM(NPROMNH)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE0L(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE0M(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE0F(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNHPRE0F(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNHPRE0H(NPROMNH,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDELP0(NPROMA,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRDELP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLNPR0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPH0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHI0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHIF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHI0FL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHI0FM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PR0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRTGR0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKAP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRRED0(NPROMNH,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEVEL0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVVEL0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGWFT0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSDVBC0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKENE0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSGRTL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSGRTM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGPHL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGPHM(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSDIV0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIVP0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSDFI0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRT0L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRT0M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE9(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE9F(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNHPRE9F(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNHPRE9H(NPROMNH,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDELP9(NPROMA,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRDELP9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLNPR9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPH9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHI9(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHIF9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCP9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PR9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGWFT9(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTNDPD(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTNDVD(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P3DIVG(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDBBC(NPROMNH)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNHXT0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNHXT9(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQCHA0L(NPROMNH,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQCHA0M(NPROMNH,NFLEVG)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV,JROF
LOGICAL :: LLDER,LLGWF0,LLGWF9,LLGDWI,LLVERCOR

! * Miscellaneous.
REAL(KIND=JPRB) :: ZALPH0L(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZALPH0M(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZALPHPL0L(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZALPHPL0M(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZLNPR0L(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZLNPR0M(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZNHPRE0L(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZNHPRE0M(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZRNHPPI0(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRNHPPI9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRPRE0(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRPRE0F(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRPRE9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRPP0(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRPP9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRTGR9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRT0(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRT9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZNHPPI0(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZNHPPI9(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZPDEP0(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZPDEP9(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZDELNHPRE0(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZDELNHPRE9(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZLNNHPREF0L(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZLNNHPREF0M(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZDVER0(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZDVER9(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZUS0(NPROMA)
REAL(KIND=JPRB) :: ZVS0(NPROMA)
REAL(KIND=JPRB) :: ZUS9(NPROMA)
REAL(KIND=JPRB) :: ZVS9(NPROMA)
REAL(KIND=JPRB) :: ZSGRTSL(NPROMNH)
REAL(KIND=JPRB) :: ZSGRTSM(NPROMNH)
REAL(KIND=JPRB) :: ZPHIHYD(NPROMNH,0:NFLEVG)
REAL(KIND=JPRB) :: ZPHIHYDF(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZPHIHYDFL(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZPHIHYDFM(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZTNDUS(NPROMNH)
REAL(KIND=JPRB) :: ZTNDVS(NPROMNH)
REAL(KIND=JPRB) :: ZVAL(NPROMNH)
REAL(KIND=JPRB) :: ZTNDGWF_LAP(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZTNDGWF_OTH(NPROMNH,NFLEVG)
REAL(KIND=JPRB) :: ZGMV(NPROMNH,NFLEVG)

REAL(KIND=JPRB) :: ZDUHO(NPROMNH,0:NFLEVG)   ! useless output dummy arg
REAL(KIND=JPRB) :: ZDUMI(NPROMNH)            ! useless input dummy arg
REAL(KIND=JPRB) :: ZDUNI(NPROMNH,NFLEVG)     ! useless input dummy arg
REAL(KIND=JPRB) :: ZDUHI(NPROMNH,0:NFLEVG)   ! useless input dummy arg

REAL(KIND=JPRB) :: ZONE(NPROMA,0:NFLEVG)     ! contains 1
REAL(KIND=JPRB) :: ZERO(NPROMA,0:NFLEVG)     ! contains 0
REAL(KIND=JPRB) :: ZW1F(NPROMNH,NFLEVG)      ! work array nr1 at full levels
REAL(KIND=JPRB) :: ZW2F(NPROMNH,NFLEVG)      ! work array nr2 at full levels

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "gnhd3.intfb.h"
#include "gnhgrpre.intfb.h"
#include "gnhgw2svd.intfb.h"
#include "gnhpre.intfb.h"
#include "gnhpreh.intfb.h"
#include "gnh_tndlagadiab_gw.intfb.h"
#include "gnh_tndlagadiab_spd.intfb.h"
#include "gnh_tndlagadiab_uvs.intfb.h"
#include "gpcty.intfb.h"
#include "gpgeo.intfb.h"
#include "gpgrgeo.intfb.h"
#include "gpgrp.intfb.h"
#include "gpgrxyb.intfb.h"
#include "gpgw.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gprt.intfb.h"
#include "gpuvs.intfb.h"
#include "gpxx.intfb.h"
#include "gpxyb.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG_GPB_NHGEOGW',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       0.    PRELIMINARY INITIALISATIONS.
!              ----------------------------

! Fill input useless arguments ZDUMI and ZDUNI with absurd values.
ZDUMI(KST:KEND)=-999999._JPRB
ZDUNI(KST:KEND,1:NFLEVG)=-999999._JPRB
ZDUHI(KST:KEND,0:NFLEVG)=-999999._JPRB

ZERO(KST:KEND,1:NFLEVG)=0.0_JPRB
ZONE(KST:KEND,0:NFLEVG)=1.0_JPRB

LLVERCOR=.FALSE.

!     ------------------------------------------------------------------

!*       3.    INITIALISE AUXILIARY VARIABLES AND TENDENCIES.
!              ----------------------------------------------

!---------------------------------------------------
!      3.1         TIME t0 CALCULATIONS
!---------------------------------------------------

!*     3.1.1 COMPUTE HALF-LAYER HYDROSTATIC PRESSURE.

CALL GPPREH(NPROMA,KST,KEND,NFLEVG,VAH,VBH,PRE0)

!*     3.1.2 COMPUTE AUXILIARY ARRAYS RELATED TO THE VERTICAL
!            AND FULL-LAYER HYDROSTATIC PRESSURE.

CALL GPXYB(NPROMA,KST,KEND,NFLEVG,VDELB,VC,&
 & PRE0,PDELP0(1,1),PRDELP0,PLNPR0,PALPH0,PRTGR0,ZRPRE0,ZRPP0)  
CALL GPPREF(NPROMA,KST,KEND,NFLEVG,PALPH0,PRE0,PRE0F)

IF ((NCURRENT_ITER == 0).OR.(NCURRENT_ITER > 0 .AND. LPC_FULL)) THEN

  IF(LVERTFE) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZRPRE0F(JROF,JLEV)=1.0_JPRB/PRE0F(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF
  CALL GPGRXYB(NPROMA,KST,KEND,NFLEVG,VC,VBH,&
   & PRE0L,PRE0M,PRDELP0,ZRPRE0,ZRPP0,PLNPR0,PALPH0,PRTGR0,&
   & ZLNPR0L,ZLNPR0M,ZALPH0L,ZALPH0M,ZALPHPL0L,ZALPHPL0M)

!*     3.1.3 COMPUTE "R", "cp" AND "kappa".

  CALL GPRCP(NPROMA,KST,KEND,NFLEVG,PGFL=PGFL,PCP=PCP0,PR=PR0,PKAP=PKAP0)  

!*     3.1.4 COMPUTES "RT" AND ITS GRADIENT.

  CALL GPRT(LSPRT,NPROMA,KST,KEND,NFLEVG,RD,RV,&
   & PR0,PGMV(1,1,YT0%MT),PGMV(1,1,YT0%MTL),PGMV(1,1,YT0%MTM),&
   & PGFL(1,1,YQ%MPL),PGFL(1,1,YQ%MPM),&
   & ZRT0,PRT0L,PRT0M)  

!*     3.1.6 COMPUTE QUANTITIES FOR NH MODEL.

  ! Compute Phihyd = hydrostatic part of Phi
  ZPHIHYD(KST:KEND,NFLEVG)=POROG(KST:KEND)
  CALL GPGEO(NPROMA,KST,KEND,NFLEVG,ZPHIHYD,ZPHIHYDF,PGMV(1,1,YT0%MT),&
   & PR0,PLNPR0,PALPH0)  
  ! Compute 'pre-prehyd' quantities from (Phi - Phihyd)
  CALL GNHPRE(NPROMA,NFLEVG,KST,KEND,PGMV(1,1,YT0%MSPD),PRE0F, &
   & PNHPRE0F,ZNHPPI0,ZRNHPPI0,ZPDEP0, &
   & PLNPR=PLNPR0,PRT=ZRT0,POROG=POROG,PHIHYD=ZPHIHYDF)

  DO JLEV=1,NFLEVG
    PRRED0(KST:KEND,JLEV)=PR0(KST:KEND,JLEV)/ZNHPPI0(KST:KEND,JLEV)
  ENDDO

  CALL GNHPREH(NPROMA,NFLEVG,KST,KEND,PGMV(1,1,YT0%MSPD),PRE0, &
   & PDELP0(1,1),PLNPR0,ZNHPPI0,PNHPRE0H,ZDELNHPRE0)

  ! Compute grad(Phihyd) = hydrostatic part of grad(Phi)
  CALL GPGRGEO(.FALSE.,&
   & NPROMA,KST,KEND,NFLEVG,&
   & ZONE(1,1),PGMV(1,1,YT0%MT),PR0,PRT0L,PRT0M,&
   & PLNPR0,ZLNPR0L,ZLNPR0M,PALPH0,ZALPH0L,ZALPH0M,&
   & ZDUNI,ZDUNI,&
   & POROGL,POROGM,&
   & ZPHIHYDFL,ZPHIHYDFM,ZDUHO,ZDUHO)
  ! Compute 'grad(pre)' quantities from (Phi-Phihyd) and grad(Phi-Phihyd)
  CALL GNHGRPRE(NPROMA,NFLEVG,KST,KEND,PRTGR0,PRE0L,PRE0M, &
   & PNHPRE0F,PGMV(1,1,YT0%MSPDL),PGMV(1,1,YT0%MSPDM), &
   & ZNHPRE0L,ZNHPRE0M,ZLNNHPREF0L,ZLNNHPREF0M,PQCHA0L,PQCHA0M, &
   & PSPD=PGMV(1,1,YT0%MSPD), & 
   & PLNPR=PLNPR0,PLNPRL=ZLNPR0L,PLNPRM=ZLNPR0M, &
   & PRT=ZRT0,PRTL=PRT0L,PRTM=PRT0M, &
   & POROG=POROG,POROGL=POROGL,POROGM=POROGM, &
   & PHIHYD=ZPHIHYDF,PHIHYDL=ZPHIHYDFL,PHIHYDM=ZPHIHYDFM)

!*     3.1.7c COMPUTATION OF SOME VERTICAL VELOCITIES.
!*           Solve continuity equation
!            Computation of the vertical velocities "etapt (d prehyd / d eta)"
!            and "omega/prehyd".

  CALL GPCTY(LLVERCOR,NPROMA,NPROMVC,KST,KEND,NFLEVG,VDELB,VBH,&
   & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),&
   & PGMV(1,1,YT0%MDIV),PGMV(1,1,YT0%MEDOT),LRUBC,&
   & PDELP0(1,1),PRDELP0,PLNPR0,PALPH0,PRTGR0,PRE0L,PRE0M,&
   & ZONE(1,1),ZONE(1,0),ZONE(1,1),ZONE(1,0),ZERO(1,1),ZERO(1,1),&
   & PEVEL0,PVVEL0,PSDIV0,PSDVBC0,PDIVP0,ZRPRE0F)
  IF(LVERTFE)THEN
    PSDFI0(KST:KEND,NFLEVG)=PSDVBC0(KST:KEND,NFLEVG)
  ELSE
    PSDFI0(KST:KEND,0:NFLEVG)=PSDVBC0(KST:KEND,0:NFLEVG)
  ENDIF 

!*     3.1.8 COMPUTES THE GEOPOTENTIAL HEIGHT "gz" AND ITS GRADIENT.

  ! * "gz".

  PHI0(KST:KEND,NFLEVG)=POROG(KST:KEND)

  IF (LVERTFE) THEN
    IF (NPDVAR == 3) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KEND
          PHIF0(JROF,JLEV)=PGMV(JROF,JLEV,YT0%MSPD)+VBF(JLEV)*POROG(JROF)
        ENDDO
      ENDDO
    ELSEIF (NPDVAR == 4) THEN
      PHIF0(KST:KEND,1:NFLEVG)=PGMV(KST:KEND,1:NFLEVG,YT0%MSPD)
    ENDIF
    ! Provisional way to compute half-level Phi (physics input).
    DO JLEV=1,NFLEVG-1
      DO JROF=KST,KEND
        PHI0(JROF,JLEV)=0.5_JPRB*(PHIF0(JROF,JLEV)+PHIF0(JROF,JLEV+1))
      ENDDO
    ENDDO
    DO JROF=KST,KEND
      PHI0(JROF,0)=2.0_JPRB*PHIF0(JROF,1)-PHI0(JROF,1)
    ENDDO
  ELSE
    CALL ABOR1('CPG_GPB_NHGEOGW: Phi-gw NH model with LVERTFE=F not coded')
  ENDIF

  ! * "grad(gz)".

  IF (LVERTFE) THEN
    IF (NPDVAR == 3) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KEND
          PHI0FL(JROF,JLEV)=PGMV(JROF,JLEV,YT0%MSPDL)+VBF(JLEV)*POROGL(JROF)
          PHI0FM(JROF,JLEV)=PGMV(JROF,JLEV,YT0%MSPDM)+VBF(JLEV)*POROGM(JROF)
        ENDDO
      ENDDO
    ELSEIF (NPDVAR == 4) THEN
      PHI0FL(KST:KEND,1:NFLEVG)=PGMV(KST:KEND,1:NFLEVG,YT0%MSPDL)
      PHI0FM(KST:KEND,1:NFLEVG)=PGMV(KST:KEND,1:NFLEVG,YT0%MSPDM)
    ENDIF
  ELSE
    CALL ABOR1('CPG_GPB_NHGEOGW: Phi-gw NH model with LVERTFE=F not coded')
  ENDIF

!*     3.1.9 COMPUTES KINETIC ENERGY.

  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      PKENE0(JROF,JLEV)=0.5_JPRB* &
       & (PGMV(JROF,JLEV,YT0%MU)**2+PGMV(JROF,JLEV,YT0%MV)**2)  
    ENDDO
  ENDDO

!*     3.1.10 COMPUTES SURFACE WINDS.

  LLDER=.FALSE.
  CALL GPUVS(NFLEVG,NPROMA,KST,KEND,LLDER, &
   & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),ZUS0,ZVS0)

!*     3.1.12 COMPUTES THE PRESSURE FORCE FOR THE RHS OF MOMENTUM EQN.

  CALL GPGRP(LLVERCOR,NPROMA,NPROMVC,NPROMNH,KST,KEND,NFLEVG, &
   & ZRT0,PRT0L,PRT0M,PRDELP0,PRE0L,PRE0M,PRTGR0, &
   & PLNPR0,ZLNPR0L,ZLNPR0M,PALPH0,ZALPHPL0L,ZALPHPL0M, &
   & ZDELNHPRE0,ZLNNHPREF0L,ZLNNHPREF0M,ZRNHPPI0,PQCHA0L,PQCHA0M, &
   & PGPHL,PGPHM,PHI0FL,PHI0FM, &
   & ZERO(1,1),ZERO(1,1),ZERO(1,1),ZERO(1,1),ZERO(1,1),ZERO(1,1),&
   & PSGRTL,PSGRTM,ZSGRTSL,ZSGRTSM)  

!*     3.1.13 COMPUTES TERM "NHX".

  CALL GPXX(NFLEVG,NPROMA,KST,KEND,PGPHL,PGPHM,PHI0FL,PHI0FM, &
   & PLNPR0,ZNHPPI0,PGMV(1,1,YT0%MT),PR0, &
   & PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),ZDUHI,ZDUHI,PNHXT0)

!*     3.1.14 COMPUTES dver AND grad(dver).

  IF (NVDVAR == 5) THEN
    ! Convert (gw - B*gw_s) into (gw)
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZGMV(JROF,JLEV)=PGMV(JROF,JLEV,YT0%MSVD) &
         & +VBF(JLEV)*(ZUS0(JROF)*POROGL(JROF)+ZVS0(JROF)*POROGM(JROF))
      ENDDO
    ENDDO
  ELSEIF (NVDVAR == 6) THEN
    ZGMV(KST:KEND,1:NFLEVG)=PGMV(KST:KEND,1:NFLEVG,YT0%MSVD)
  ENDIF
  IF (LGWOPT1) THEN
    ! * apply vertical derivatives to NH deviations
    ! dver_hyd=-(R/Rd)*(D+X+(cv/cp)(omega/prehyd))
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZW1F(JROF,JLEV)=-(PR0(JROF,JLEV)/RD)*(PGMV(JROF,JLEV,YT0%MDIV)&
         & +PNHXT0(JROF,JLEV)+(1.0_JPRB-PKAP0(JROF,JLEV))*PVVEL0(JROF,JLEV))
      ENDDO
    ENDDO
    ! gw_hyd integrated from dver_hyd
    LLGWF0=.TRUE.
    LLGDWI=.FALSE.
    CALL GPGW(NFLEVG,NPROMA,KST,KEND,LLGWF0,LLGDWI, &
     & POROGL,POROGM,PLNPR0,PALPH0,ZUS0,ZVS0, &
     & PGMV(1,1,YT0%MT),ZW1F,ZRNHPPI0,ZDUHO,ZW2F)
    ! gw-gw_hyd
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZW2F(JROF,JLEV)=ZGMV(JROF,JLEV)-ZW2F(JROF,JLEV)
      ENDDO
    ENDDO
    ! dver-dver_hyd
    ZVAL(KST:KEND)=0.0_JPRB
    CALL GNHGW2SVD(KST,KEND,NVDVAR,ZDUMI, &
     & PGMV(1,1,YT0%MT),ZDUNI,ZDVER0,PLNPR=PLNPR0, &
     & PDEP=ZPDEP0,PREF=PRE0F,PGWF=ZW2F,PGWS=ZVAL)
    ! dver
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZDVER0(JROF,JLEV)=ZDVER0(JROF,JLEV)+ZW1F(JROF,JLEV)
      ENDDO
    ENDDO
  ELSE
    ! * apply vertical derivatives to whole "gw" 
    DO JROF=KST,KEND
      ZVAL(JROF)=ZUS0(JROF)*POROGL(JROF)+ZVS0(JROF)*POROGM(JROF)
    ENDDO
    CALL GNHGW2SVD(KST,KEND,NVDVAR,ZDUMI, &
     & PGMV(1,1,YT0%MT),ZDUNI,ZDVER0,PLNPR=PLNPR0, &
     & PDEP=ZPDEP0,PREF=PRE0F,PGWF=ZGMV,PGWS=ZVAL)
  ENDIF

!*     3.1.16 COMPUTES term [gw].

  IF (LVERTFE) THEN
    IF (NVDVAR == 5) THEN
      ! Convert (gw - B*gw_s) into (gw)
      DO JLEV=1,NFLEVG
        DO JROF=KST,KEND
          PGWFT0(JROF,JLEV)=PGMV(JROF,JLEV,YT0%MSVD) &
           & +VBF(JLEV)*(ZUS0(JROF)*POROGL(JROF)+ZVS0(JROF)*POROGM(JROF))
        ENDDO
      ENDDO
    ELSEIF (NVDVAR == 6) THEN
      PGWFT0(KST:KEND,1:NFLEVG)=PGMV(KST:KEND,1:NFLEVG,YT0%MSVD)
    ENDIF
  ELSE
    CALL ABOR1('CPG_GPB_NHGEOGW: Phi-gw NH model with LVERTFE=F not coded')
  ENDIF

!*     3.1.18 COMPUTES D3.

  CALL GNHD3(NFLEVG,NPROMA,KST,KEND, &
   & PGMV(1,1,YT0%MDIV),PNHXT0,ZDVER0,PR0,P3DIVG)

!*     3.1.20 COMPUTES [D V_surf/Dt].

  IF (NVDVAR==5) THEN
    CALL GNH_TNDLAGADIAB_UVS(NPROMA,KST,KEND,LRPLANE, &
     & PRCORI,PRATATX,PRATATH,ZUS0,ZVS0,ZSGRTSL,ZSGRTSM,ZTNDUS,ZTNDVS)
  ENDIF

!*     3.1.21 COMPUTES [D(gw)/Dt].

  IF (NVDVAR==5) THEN
    CALL GNH_TNDLAGADIAB_GW(NFLEVG,NPROMA,KST,KEND, &
     & POROGL=POROGL,POROGM=POROGM, &
     & POROGLM=POROGLM,POROGLL=POROGLL,POROGMM=POROGMM, &
     & PUS=ZUS0,PVS=ZVS0,PTNDUS=ZTNDUS,PTNDVS=ZTNDVS, &
     & PDELP=PDELP0(1,1),PRDELP=PRDELP0,PDELNHPRE=ZDELNHPRE0,PEVEL=PEVEL0, &
     & PTNDGWF_LAP=ZTNDGWF_LAP,PTNDGWF_OTH=ZTNDGWF_OTH,PDBBC=PDBBC)
  ELSEIF (NVDVAR==6) THEN
    CALL GNH_TNDLAGADIAB_GW(NFLEVG,NPROMA,KST,KEND, &
     & PDELP=PDELP0(1,1),PDELNHPRE=ZDELNHPRE0, &
     & PTNDGWF_LAP=ZTNDGWF_LAP,PTNDGWF_OTH=ZTNDGWF_OTH,PDBBC=PDBBC)
  ENDIF

!*     3.1.22 COMPUTES [D(pressure departure variable)/Dt].

  CALL GNH_TNDLAGADIAB_SPD(NFLEVG,NPROMA,KST,KEND, &
   & PKAP0,P3DIVG,PVVEL0,PGWFT0, &
   & PEVEL0,PRDELP0,PGMV(1,1,YT0%MU),PGMV(1,1,YT0%MV),POROG,POROGL,POROGM, &
   & PTNDPD)

!*     3.1.23 COMPUTES [D(vertical divergence variable)/Dt].

  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      PTNDVD(JROF,JLEV)=ZTNDGWF_LAP(JROF,JLEV)+ZTNDGWF_OTH(JROF,JLEV)
    ENDDO
  ENDDO

ENDIF

!---------------------------------------------------
!      3.2         TIME t9 CALCULATIONS
!---------------------------------------------------

IF(.NOT.LTWOTL .AND. NCURRENT_ITER == 0) THEN

!*     3.2.1 COMPUTE HALF-LAYER HYDROSTATIC PRESSURE.

  CALL GPPREH(NPROMA,KST,KEND,NFLEVG,VAH,VBH,PRE9)

!*     3.2.2 COMPUTE AUXILIARY ARRAYS RELATED TO THE VERTICAL
!            AND FULL-LAYER HYDROSTATIC PRESSURE.

  CALL GPXYB(NPROMA,KST,KEND,NFLEVG,VDELB,VC,&
   & PRE9,PDELP9(1,1),PRDELP9,PLNPR9,PALPH9,ZRTGR9,ZRPRE9,ZRPP9)  
  CALL GPPREF(NPROMA,KST,KEND,NFLEVG,PALPH9,PRE9,PRE9F)

!*     3.2.3 COMPUTE "R", "cp" AND "kappa".

  CALL GPRCP(NPROMA,KST,KEND,NFLEVG,PGFL=PGFL,KGFLTYP=9,PCP=PCP9,PR=PR9)

!*     3.2.4 COMPUTE "RT".

  ZRT9(KST:KEND,1:NFLEVG)=PGMV(KST:KEND,1:NFLEVG,YT9%MT)*PR9(KST:KEND,1:NFLEVG)

!*     3.2.6 COMPUTE QUANTITIES FOR NH MODEL.

  ! Compute Phihyd = hydrostatic part of Phi
  ZPHIHYD(KST:KEND,NFLEVG)=POROG(KST:KEND)
  CALL GPGEO(NPROMA,KST,KEND,NFLEVG,ZPHIHYD,ZPHIHYDF,PGMV(1,1,YT9%MT),&
   & PR9,PLNPR9,PALPH9)  
  ! Compute 'pre-prehyd' quantities from (Phi - Phihyd)
  CALL GNHPRE(NPROMA,NFLEVG,KST,KEND,PGMV(1,1,YT9%MSPD),PRE9F, &
   & PNHPRE9F,ZNHPPI9,ZRNHPPI9,ZPDEP9, &
   & PLNPR=PLNPR9,PRT=ZRT9,POROG=POROG,PHIHYD=ZPHIHYDF)

  CALL GNHPREH(NPROMA,NFLEVG,KST,KEND,PGMV(1,1,YT9%MSPD),PRE9, &
   & PDELP9(1,1),PLNPR9,ZNHPPI9,PNHPRE9H,ZDELNHPRE9)

!*     3.2.8 COMPUTES THE GEOPOTENTIAL HEIGHT "gz" AND ITS GRADIENT.

  ! * "gz".

  IF((LDLDIAB.OR.LSIMPH)) THEN
    PHI9(KST:KEND,NFLEVG)=POROG(KST:KEND)
    IF (LVERTFE) THEN
      ! Actual Phi (including NH effects)
      IF (NPDVAR == 3) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KEND
            PHIF9(JROF,JLEV)=PGMV(JROF,JLEV,YT9%MSPD)+VBF(JLEV)*POROG(JROF)
          ENDDO
        ENDDO
      ELSEIF (NPDVAR == 4) THEN
        PHIF9(KST:KEND,1:NFLEVG)=PGMV(KST:KEND,1:NFLEVG,YT9%MSPD)
      ENDIF
      ! Provisional way to compute half-level Phi (physics input).
      DO JLEV=1,NFLEVG-1
        DO JROF=KST,KEND
          PHI9(JROF,JLEV)=0.5_JPRB*(PHIF9(JROF,JLEV)+PHIF9(JROF,JLEV+1))
        ENDDO
      ENDDO
      DO JROF=KST,KEND
        PHI9(JROF,0)=2.0_JPRB*PHIF9(JROF,1)-PHI9(JROF,1)
      ENDDO
    ELSE
      CALL ABOR1('CPG_GPB_NHGEOGW: Phi-gw NH model with LVERTFE=F not coded')
    ENDIF
  ENDIF

!*     3.2.10 COMPUTES SURFACE WINDS.

  LLDER=.FALSE.
  CALL GPUVS(NFLEVG,NPROMA,KST,KEND,LLDER, &
   & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),ZUS9,ZVS9)

!*     3.2.13 COMPUTES TERM "NHX".

  ! caution: must use "R" and "pre/prehyd" at time t.
  CALL GPXX(NFLEVG,NPROMA,KST,KEND,PGPHL,PGPHM,PHI0FL,PHI0FM, &
   & PLNPR9,ZNHPPI0,PGMV(1,1,YT9%MT),PR0, &
   & PGMV(1,1,YT9%MU),PGMV(1,1,YT9%MV),ZDUHI,ZDUHI,PNHXT9)

!*     3.2.14 COMPUTES dver.

  IF (NVDVAR == 5) THEN
    ! Convert (gw - B*gw_s) into (gw)
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZGMV(JROF,JLEV)=PGMV(JROF,JLEV,YT9%MSVD) &
         & +VBF(JLEV)*(ZUS9(JROF)*POROGL(JROF)+ZVS9(JROF)*POROGM(JROF))
      ENDDO
    ENDDO
  ELSEIF (NVDVAR == 6) THEN
    ZGMV(KST:KEND,1:NFLEVG)=PGMV(KST:KEND,1:NFLEVG,YT9%MSVD)
  ENDIF
  IF (LGWOPT1) THEN
    ! * apply vertical derivatives to NH deviations
    ! dver_hyd=-(R/Rd)*(D+X+(cv/cp)(omega/prehyd))
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZW1F(JROF,JLEV)=-(PR0(JROF,JLEV)/RD)*(PGMV(JROF,JLEV,YT9%MDIV)&
         & +PNHXT9(JROF,JLEV)+(1.0_JPRB-PKAP0(JROF,JLEV))*PVVEL0(JROF,JLEV))
      ENDDO
    ENDDO
    ! gw_hyd integrated from dver_hyd
    LLGWF9=.TRUE.
    LLGDWI=.FALSE.
    CALL GPGW(NFLEVG,NPROMA,KST,KEND,LLGWF9,LLGDWI, &
     & POROGL,POROGM,PLNPR9,PALPH9,ZUS9,ZVS9, &
     & PGMV(1,1,YT9%MT),ZW1F,ZRNHPPI9,ZDUHO,ZW2F)
    ! gw-gw_hyd
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZW2F(JROF,JLEV)=ZGMV(JROF,JLEV)-ZW2F(JROF,JLEV)
      ENDDO
    ENDDO
    ! dver-dver_hyd
    ZVAL(KST:KEND)=0.0_JPRB
    CALL GNHGW2SVD(KST,KEND,NVDVAR,ZDUMI, &
     & PGMV(1,1,YT9%MT),ZDUNI,ZDVER9,PLNPR=PLNPR9, &
     & PDEP=ZPDEP9,PREF=PRE9F,PGWF=ZW2F,PGWS=ZVAL)
    ! dver
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZDVER9(JROF,JLEV)=ZDVER9(JROF,JLEV)+ZW1F(JROF,JLEV)
      ENDDO
    ENDDO
  ELSE
    ! * apply vertical derivatives to whole "gw" 
    DO JROF=KST,KEND
      ZVAL(JROF)=ZUS9(JROF)*POROGL(JROF)+ZVS9(JROF)*POROGM(JROF)
    ENDDO
    CALL GNHGW2SVD(KST,KEND,NVDVAR,ZDUMI, &
     & PGMV(1,1,YT9%MT),ZDUNI,ZDVER9,PLNPR=PLNPR9, &
     & PDEP=ZPDEP9,PREF=PRE9F,PGWF=ZGMV,PGWS=ZVAL)
  ENDIF

!*     3.2.16 COMPUTES term [gw].

  IF (LVERTFE) THEN
    IF (NVDVAR == 5) THEN
      ! Convert (gw - B*gw_s) into (gw)
      DO JLEV=1,NFLEVG
        DO JROF=KST,KEND
          PGWFT9(JROF,JLEV)=PGMV(JROF,JLEV,YT9%MSVD) &
           & +VBF(JLEV)*(ZUS9(JROF)*POROGL(JROF)+ZVS9(JROF)*POROGM(JROF))
        ENDDO
      ENDDO
    ELSEIF (NVDVAR == 6) THEN
      PGWFT9(KST:KEND,1:NFLEVG)=PGMV(KST:KEND,1:NFLEVG,YT9%MSVD)
    ENDIF
  ELSE
    CALL ABOR1('CPG_GPB_NHGEOGW: Phi-gw NH model with LVERTFE=F not coded')
  ENDIF

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPG_GPB_NHGEOGW',1,ZHOOK_HANDLE)
END SUBROUTINE CPG_GPB_NHGEOGW
