SUBROUTINE CPPFTTCINV(KGL1,KGL2,KOFF,&
 & PBASCO,PBASSI,&
 & PFORAB3C,PFORAB4C,PFORAB6C,PFORAB3N,PFORAB4N,PFORAB6N,&
 & PFORAT1C,PFORAT2C,PFORAT3C,PFORAT4C,PFORAT5C,&
 & PFORAT1N,PFORAT2N,PFORAT3N,PFORAT4N,PFORAT5N,&
 & PFOCOR,PFOAC,&
 & PRAB3C,PRAB4C,PRAB6C,PRAB3N,PRAB4N,PRAB6N,&
 & PRAT1C,PRAT2C,PRAT3C,PRAT4C,PRAT5C,&
 & PRAT1N,PRAT2N,PRAT3N,PRAT4N,PRAT5N,&
 & PCOR,PAC)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LRPLANE  ,LRGPTCNORM
USE YOMDIM   , ONLY : NTCMAX   ,NPROMM   ,NFLEVG   ,NDLSUR   ,NDLON   ,NDGLG
USE YOMGEM   , ONLY : NSTAGP   ,NMENTC   ,NTSTAGP  ,NLOENG
USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : MYSETB   ,NONL     ,MYLATS   ,NPTRFLOFF

!**** *CPPFTTCINV - Inverse slow Fourier transforms for transmission
!                   coefficients.

!     Purpose.
!     --------

!     Inverse slow Fourier transforms for transmission coefficients.

!     Since the 'Fourier' fit can create overshoots and undershoots,
!     the natural bounds for the transmission coefficients
!     (0 and 1 excepted for a subset of fields of the variable 'AC'
!     which are between -1 and 0) can be not conserved and a reset
!     between 0 and 1 (resp. -1 and 0) for positive (resp. negative)
!     coefficients is generally necessary.

!**   Interface.
!     ----------
!        *CALL* *CPPFTTCINV(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KGL1      - lower DM-local latitude for this NPROMM-packet.
!          KGL2      - upper DM-local latitude for this NPROMM-packet.
!          KOFF      - index offset between NPROMM-arrays and NGPTOT-arrays.
!          PBASCO    - basis of cosinus.
!          PBASSI    - basis of sinus.

!          ==  PFORAB3C to PFOAC: Fourier coefficients values ==
!          PFORAB3C  - clear-sky parallel/diffuse reflectivity.
!          PFORAB4C  - clear-sky diffuse transmissivity.
!          PFORAB6C  - clear-sky diffuse reflectivity.
!          PFORAB3N  - cloudy parallel/diffuse reflectivity.
!          PFORAB4N  - cloudy diffuse transmissivity.
!          PFORAB6N  - cloudy diffuse reflectivity.
!          PFORAT1C  - clear-sky parallel transmissivity.
!          PFORAT2C  - clear-sky parallel/diffuse transmissivity.
!          PFORAT3C  - clear-sky parallel/diffuse reflectivity.
!          PFORAT4C  - clear-sky diffuse transmissivity.
!          PFORAT5C  - clear-sky diffuse reflectivity.
!          PFORAT1N  - cloudy parallel transmissivity.
!          PFORAT2N  - cloudy parallel/diffuse transmissivity.
!          PFORAT3N  - cloudy parallel/diffuse reflectivity.
!          PFORAT4N  - cloudy diffuse transmissivity.
!          PFORAT5N  - cloudy diffuse reflectivity.
!          PFOCOR    - correction for cloudiness for thermal radiation.
!          PFOAC     - Curtis matrix for thermal radiation for clear sky.

!        OUTPUT:
!          ==  PRAB3C to PAC: grid-point values ==
!          PRAB3C    - clear-sky parallel/diffuse reflectivity.
!          PRAB4C    - clear-sky diffuse transmissivity.
!          PRAB6C    - clear-sky diffuse reflectivity.
!          PRAB3N    - cloudy parallel/diffuse reflectivity.
!          PRAB4N    - cloudy diffuse transmissivity.
!          PRAB6N    - cloudy diffuse reflectivity.
!          PRAT1C    - clear-sky parallel transmissivity.
!          PRAT2C    - clear-sky parallel/diffuse transmissivity.
!          PRAT3C    - clear-sky parallel/diffuse reflectivity.
!          PRAT4C    - clear-sky diffuse transmissivity.
!          PRAT5C    - clear-sky diffuse reflectivity.
!          PRAT1N    - cloudy parallel transmissivity.
!          PRAT2N    - cloudy parallel/diffuse transmissivity.
!          PRAT3N    - cloudy parallel/diffuse reflectivity.
!          PRAT4N    - cloudy diffuse transmissivity.
!          PRAT5N    - cloudy diffuse reflectivity.
!          PCOR      - correction for cloudiness for thermal radiation.
!          PAC       - Curtis matrix for thermal radiation for clear sky.

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        See documentation about spectral transforms.

!     Externals.
!     ----------
!        MXMAOP,GPTCNORM

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP).

!     Modifications.
!     --------------
!        Original : FEB 1999
!        Modified by G> Radnoti 99/11/07  ALADIN geometry
!        27 Aug 2002 by K. YESSAD: bug correction.
!        23 Oct 2002 by K. YESSAD: bound coefficients between 0 and 1
!         (resp. -1 and 0) for positive (resp. negative) coefficients.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KGL1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOFF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBASCO(NTCMAX,NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBASSI(NTCMAX,NPROMM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAB3C(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAB4C(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAB6C(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAB3N(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAB4N(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAB6N(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAT1C(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAT2C(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAT3C(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAT4C(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAT5C(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAT1N(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAT2N(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAT3N(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAT4N(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFORAT5N(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFOCOR(NFLEVG,0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFOAC((NFLEVG+1)*(NFLEVG+1),0:2*NTCMAX,KGL1:KGL2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAB3C(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAB4C(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAB6C(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAB3N(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAB4N(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAB6N(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAT1C(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAT2C(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAT3C(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAT4C(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAT5C(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAT1N(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAT2N(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAT3N(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAT4N(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRAT5N(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOR(NPROMM,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAC(NPROMM,(NFLEVG+1)*(NFLEVG+1)) 

! ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IDEB
INTEGER(KIND=JPIM) :: IFIN
INTEGER(KIND=JPIM) :: IFLEVG
INTEGER(KIND=JPIM) :: IGLG
INTEGER(KIND=JPIM) :: IIROF0(NDGLG)
INTEGER(KIND=JPIM) :: ILONGG
INTEGER(KIND=JPIM) :: ILONGE(NDGLG)
INTEGER(KIND=JPIM) :: INUMF
INTEGER(KIND=JPIM) :: IROF0
INTEGER(KIND=JPIM) :: JGL
INTEGER(KIND=JPIM) :: JLEV
INTEGER(KIND=JPIM) :: JLON
INTEGER(KIND=JPIM) :: JM

LOGICAL :: LLB
LOGICAL :: LLGPTCNORM

REAL(KIND=JPRB) :: ZFOITC(17*NFLEVG+(NFLEVG+1)*(NFLEVG+1),NTCMAX)
REAL(KIND=JPRB) :: ZFORTC(17*NFLEVG+(NFLEVG+1)*(NFLEVG+1),NTCMAX)
REAL(KIND=JPRB) :: ZGPTC(NDLSUR,17*NFLEVG+(NFLEVG+1)*(NFLEVG+1))
REAL(KIND=JPRB) :: ZGPITC(NDLSUR,17*NFLEVG+(NFLEVG+1)*(NFLEVG+1))
REAL(KIND=JPRB) :: ZGPRTC(NDLSUR,17*NFLEVG+(NFLEVG+1)*(NFLEVG+1))
REAL(KIND=JPRB) :: ZMOTC(17*NFLEVG+(NFLEVG+1)*(NFLEVG+1))
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------

INTERFACE
#include "mxmaop.h"
END INTERFACE

#include "gptcnorm.intfb.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPPFTTCINV',0,ZHOOK_HANDLE)

! ------------------------------------------------------------------

!*  1. PRELIMINARY SET TO ZERO.
!   ---------------------------

PRAB3C=0.0_JPRB
PRAB4C=0.0_JPRB
PRAB6C=0.0_JPRB
PRAB3N=0.0_JPRB
PRAB4N=0.0_JPRB
PRAB6N=0.0_JPRB
PRAT1C=0.0_JPRB
PRAT2C=0.0_JPRB
PRAT3C=0.0_JPRB
PRAT4C=0.0_JPRB
PRAT5C=0.0_JPRB
PRAT1N=0.0_JPRB
PRAT2N=0.0_JPRB
PRAT3N=0.0_JPRB
PRAT4N=0.0_JPRB
PRAT5N=0.0_JPRB
PCOR=0.0_JPRB
PAC=0.0_JPRB

! ------------------------------------------------------------------

!*  2. FOR EACH LATITUDE: MEMORY TRANSFERS AND TRANSFORMS.
!   ------------------------------------------------------

LLGPTCNORM=LRGPTCNORM(1).OR.LRGPTCNORM(2).OR.LRGPTCNORM(3)

! * reset positive (resp. negative) fields between 0 and 1 (resp. -1 and 0).
LLB=(NTCMAX>0).AND.(NTCMAX <= NDLON/2)

DO JGL=KGL1,KGL2

  IGLG=MYLATS(JGL)
  IF (LRPLANE) THEN
    IROF0=NTSTAGP(JGL)-KOFF
  ELSE
    IROF0=NSTAGP(JGL)-KOFF
  ENDIF

! * for ALADIN: ILONGG includes the extension zone.
  ILONGG=NONL(NPTRFLOFF+JGL,MYSETB)

  IDEB=MAX(1,IROF0+1)
  IFIN=MIN(NPROMM,IROF0+ILONGG)
  IIROF0(IGLG)=IDEB-1
  ILONGE(IGLG)=IFIN-IDEB+1

  ZMOTC=0.0_JPRB
  ZFORTC=0.0_JPRB
  ZFOITC=0.0_JPRB

!*      2.1 MEMORY TRANSFERS FOR FOURIER COEFFICIENTS ARRAYS.

  INUMF=0 

! * clear-sky parallel/diffuse reflectivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAB3C(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB3C(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB3C(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse transmissivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAB4C(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB4C(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB4C(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse reflectivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAB6C(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB6C(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB6C(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse reflectivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAB3N(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB3N(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB3N(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * cloudy diffuse transmissivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAB4N(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB4N(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB4N(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * cloudy diffuse reflectivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAB6N(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB6N(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAB6N(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * clear-sky parallel transmissivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAT1C(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT1C(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT1C(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * clear-sky parallel/diffuse transmissivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAT2C(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT2C(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT2C(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * clear-sky parallel/diffuse reflectivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAT3C(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT3C(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT3C(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse transmissivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAT4C(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT4C(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT4C(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse reflectivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAT5C(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT5C(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT5C(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * cloudy parallel transmissivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAT1N(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT1N(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT1N(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse transmissivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAT2N(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT2N(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT2N(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse reflectivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAT3N(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT3N(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT3N(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * cloudy diffuse transmissivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAT4N(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT4N(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT4N(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * cloudy diffuse reflectivity.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFORAT5N(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT5N(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFORAT5N(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * correction for cloudiness for thermal radiation.
  ZMOTC(INUMF+1:INUMF+NFLEVG)=PFOCOR(1:NFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+NFLEVG,JM)=PFOCOR(1:NFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+NFLEVG,JM)=PFOCOR(1:NFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+NFLEVG

! * Curtis matrix for thermal radiation for clear sky.
  IFLEVG=(NFLEVG+1)*(NFLEVG+1)
  ZMOTC(INUMF+1:INUMF+IFLEVG)=PFOAC(1:IFLEVG,0,JGL)
!OCL NOVREC
  DO JM=1,NMENTC(IGLG)
    ZFORTC(INUMF+1:INUMF+IFLEVG,JM)=PFOAC(1:IFLEVG,2*JM-1,JGL)
    ZFOITC(INUMF+1:INUMF+IFLEVG,JM)=PFOAC(1:IFLEVG,2*JM  ,JGL)
  ENDDO
  INUMF=INUMF+(NFLEVG+1)*(NFLEVG+1)

!*      2.2 INVERSE FOURIER TRANSFORM.

! * Pre-multiply real part by factor 2 (= discretisation
!   of the average of 1/PBASCO**2 on a latitude) for 
!   zonal wave numbers between 1 and nloeng(iglg)/2-1.
!   For jm=nloeng(iglg)/2 the discretisation of the average of
!   1/PBASCO**2 on a latitude yields 1 so nothing to do.
  IF (NTCMAX >= 1) THEN
    DO JM=1,MIN(NMENTC(IGLG),NLOENG(IGLG)/2-1)
      ZFORTC(1:INUMF,JM)=2.0_JPRB*ZFORTC(1:INUMF,JM)
    ENDDO
  ENDIF

! * Pre-multiply imaginary part by factor 2 (= discretisation
!   of the average of 1/PBASSI**2 on a latitude) for 
!   zonal wave numbers between 1 and nloeng(iglg)/2-1.
!   For jm=nloeng(iglg)/2 the imaginary part is normally zero
!   and is reset to zero for caution.
  IF (NTCMAX >= 1) THEN
    DO JM=1,MIN(NMENTC(IGLG),NLOENG(IGLG)/2-1)
      ZFOITC(1:INUMF,JM)=2.0_JPRB*ZFOITC(1:INUMF,JM)
    ENDDO
    IF (NMENTC(IGLG) == NLOENG(IGLG)/2) ZFOITC(1:INUMF,NLOENG(IGLG)/2)=0.0_JPRB
  ENDIF

! * Real part (ZGPRTC = transp(PBASCO) times ZFORTC) for m > 1.
  ZGPRTC=0.0_JPRB
  IF (NTCMAX >= 1) THEN
    CALL MXMAOP(PBASCO(1,IDEB),NTCMAX,1,ZFORTC(1,1),INUMF,1,&
     & ZGPRTC(1,1),1,NDLSUR,ILONGE(IGLG),NMENTC(IGLG),INUMF)  
  ENDIF
! * Imaginary part (ZGPITC = - transp(PBASSI) times ZFOITC) for m > 1.
  ZGPITC=0.0_JPRB
  IF (NTCMAX >= 1) THEN
    CALL MXMAOP(PBASSI(1,IDEB),NTCMAX,1,ZFOITC(1,1),INUMF,1,&
     & ZGPITC(1,1),1,NDLSUR,ILONGE(IGLG),NMENTC(IGLG),INUMF)  
    ZGPITC=-ZGPITC
  ENDIF
! * Compute grid-point value.
  ZGPTC=0.0_JPRB
  DO JLON=1,ILONGE(IGLG)
    ZGPTC(JLON,1:INUMF)=ZMOTC(1:INUMF)+ZGPRTC(JLON,1:INUMF)+ZGPITC(JLON,1:INUMF)
  ENDDO

!*  2.3 MEMORY TRANSFERS FOR GRID-POINT ARRAYS.
!       NORMS IF REQUIRED.

  INUMF=0 

! * clear-sky parallel/diffuse reflectivity.
  DO JLEV=1,NFLEVG
    PRAB3C(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAB3C',PRAB3C)  
  ENDIF
  IF (LLB) THEN
    PRAB3C(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAB3C(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAB3C',PRAB3C)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse transmissivity.
  DO JLEV=1,NFLEVG
    PRAB4C(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAB4C',PRAB4C)  
  ENDIF
  IF (LLB) THEN
    PRAB4C(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAB4C(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAB4C',PRAB4C)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse reflectivity.
  DO JLEV=1,NFLEVG
    PRAB6C(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAB6C',PRAB6C)  
  ENDIF
  IF (LLB) THEN
    PRAB6C(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAB6C(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAB6C',PRAB6C)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse reflectivity.
  DO JLEV=1,NFLEVG
    PRAB3N(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAB3N',PRAB3N)  
  ENDIF
  IF (LLB) THEN
    PRAB3N(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAB3N(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAB3N',PRAB3N)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * cloudy diffuse transmissivity.
  DO JLEV=1,NFLEVG
    PRAB4N(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAB4N',PRAB4N)  
  ENDIF
  IF (LLB) THEN
    PRAB4N(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAB4N(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAB4N',PRAB4N)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * cloudy diffuse reflectivity.
  DO JLEV=1,NFLEVG
    PRAB6N(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAB6N',PRAB6N)  
  ENDIF
  IF (LLB) THEN
    PRAB6N(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAB6N(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAB6N',PRAB6N)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * clear-sky parallel transmissivity.
  DO JLEV=1,NFLEVG
    PRAT1C(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAT1C',PRAT1C)  
  ENDIF
  IF (LLB) THEN
    PRAT1C(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAT1C(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAT1C',PRAT1C)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * clear-sky parallel/diffuse transmissivity.
  DO JLEV=1,NFLEVG
    PRAT2C(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAT2C',PRAT2C)  
  ENDIF
  IF (LLB) THEN
    PRAT2C(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAT2C(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAT2C',PRAT2C)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * clear-sky parallel/diffuse reflectivity.
  DO JLEV=1,NFLEVG
    PRAT3C(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAT3C',PRAT3C)  
  ENDIF
  IF (LLB) THEN
    PRAT3C(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAT3C(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAT3C',PRAT3C)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse transmissivity.
  DO JLEV=1,NFLEVG
    PRAT4C(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAT4C',PRAT4C)  
  ENDIF
  IF (LLB) THEN
    PRAT4C(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAT4C(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAT4C',PRAT4C)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * clear-sky diffuse reflectivity.
  DO JLEV=1,NFLEVG
    PRAT5C(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAT5C',PRAT5C)  
  ENDIF
  IF (LLB) THEN
    PRAT5C(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAT5C(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAT5C',PRAT5C)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * cloudy parallel transmissivity.
  DO JLEV=1,NFLEVG
    PRAT1N(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAT1N',PRAT1N)  
  ENDIF
  IF (LLB) THEN
    PRAT1N(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAT1N(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAT1N',PRAT1N)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse transmissivity.
  DO JLEV=1,NFLEVG
    PRAT2N(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAT2N',PRAT2N)  
  ENDIF
  IF (LLB) THEN
    PRAT2N(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAT2N(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAT2N',PRAT2N)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * cloudy parallel/diffuse reflectivity.
  DO JLEV=1,NFLEVG
    PRAT3N(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAT3N',PRAT3N)  
  ENDIF
  IF (LLB) THEN
    PRAT3N(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAT3N(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAT3N',PRAT3N)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * cloudy diffuse transmissivity.
  DO JLEV=1,NFLEVG
    PRAT4N(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAT4N',PRAT4N)  
  ENDIF
  IF (LLB) THEN
    PRAT4N(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAT4N(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAT4N',PRAT4N)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * cloudy diffuse reflectivity.
  DO JLEV=1,NFLEVG
    PRAT5N(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  RAT5N',PRAT5N)  
  ENDIF
  IF (LLB) THEN
    PRAT5N(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRAT5N(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B RAT5N',PRAT5N)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * correction for cloudiness for thermal radiation.
  DO JLEV=1,NFLEVG
    PCOR(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  COR  ',PCOR)  
  ENDIF
  IF (LLB) THEN
    PCOR(IDEB:IFIN,1:NFLEVG)=MAX(0.0_JPRB,MIN(1.0_JPRB,PCOR(IDEB:IFIN,1:NFLEVG)))
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,NFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B COR  ',PCOR)  
    ENDIF
  ENDIF
  INUMF=INUMF+NFLEVG

! * Curtis matrix for thermal radiation for clear sky.
  IFLEVG=(NFLEVG+1)*(NFLEVG+1)
  DO JLEV=1,IFLEVG
    PAC(IDEB:IFIN,JLEV)=ZGPTC(1:ILONGE(IGLG),INUMF+JLEV)
  ENDDO
  IF (LLGPTCNORM) THEN
    CALL GPTCNORM(LRGPTCNORM,NULOUT,IFLEVG,IGLG,IGLG,NPROMM,&
     & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'  AC   ',PAC)  
  ENDIF
  IF (LLB) THEN
    DO JLEV=1,IFLEVG
      IF (ZMOTC(INUMF+JLEV) < 0._JPRB) THEN
        PAC(IDEB:IFIN,JLEV)=MAX(-1._JPRB,MIN(0.0_JPRB,PAC(IDEB:IFIN,JLEV)))
      ELSE
        PAC(IDEB:IFIN,JLEV)=MAX(0.0_JPRB,MIN(1.0_JPRB,PAC(IDEB:IFIN,JLEV)))
      ENDIF
    ENDDO
    IF (LLGPTCNORM) THEN
      CALL GPTCNORM(LRGPTCNORM,NULOUT,IFLEVG,IGLG,IGLG,NPROMM,&
       & NDLSUR,IIROF0(IGLG),ILONGE(IGLG),'B AC   ',PAC)  
    ENDIF
  ENDIF
  INUMF=INUMF+IFLEVG

ENDDO

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CPPFTTCINV',1,ZHOOK_HANDLE)
END SUBROUTINE CPPFTTCINV
