SUBROUTINE GNH_CONV_NHVAR_GEOGW( &
 ! ----- INPUT ---------------------------------------------------------------
 & LDMODEL_TO_FILE, &
 ! ----- INPUT-OUTPUT --------------------------------------------------------
 & PU,PV,PT,PTL,PTM,PSPD,PSVD,PSP,PQ,PQL,PQM,PL,PI,PR,PS,PG)

!* GNH_CONV_NHVAR_GEOGW - Conversion of NH variables
!                         (model to file and vice-versa)
!                         for the Phi-Gw NH model.
!                         Grid-point part.

! Purpose
! -------
!   Converts NH geopotential and "Gw" variables between model and file.
!   Files contain "pre-prehyd" and -G.dw.
!   Modifies spectral buffers:
!     SPSPD
!     SPSVD

!   Wood and Staniforth deep-layer formulation of the equations (LRWSDLR=T)
!    is not coded for the Phi-Gw NH model.

! Interface
! ---------
!  * INPUT:
!     LDMODEL_TO_FILE - switch for model to file / file to model conversion

!  * INPUT-OUTPUT:
!     PU     : U-wind
!     PV     : V-wind
!     PT     : temperature
!     PTL    : zonal comp of grad(temperature)
!     PTM    : merid comp of grad(temperature)
!     PSPD   : NH pressure departure variable "spd"
!     PSVD   : NH vertical divergence variable "svd"
!     PSP    : log(prehyd)
!     PQ     : moisture
!     PQL    : zonal comp of grad(moisture)
!     PQM    : merid comp of grad(moisture)
!     PL     : liquid water
!     PI     : ice
!     PR     : rain
!     PS     : snow
!     PG     : graupels

! Externals
! ---------

! Method
! ------
!   See documentation

! Reference
! ---------

! Author
! ------
!   17-Apr-2008 K. Yessad

! Modifications
! -------------
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMCST   , ONLY : RD       ,RV
USE YOMCT0   , ONLY : LSPRT
USE YOMCVER  , ONLY : LVERTFE  ,LVFE_DERIB
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMDYNA  , ONLY : NVDVAR   ,NPDVAR   ,LNH_GEOGW
USE YOMGC    , ONLY : GM       ,OROG     ,OROGL    ,OROGM
USE YOMGEM   , ONLY : VAH      ,VBH      ,VC       ,VDELB    ,NGPTOT   , &
 & VFE_RDETAH,VBF
USE YOM_YGFL , ONLY : YQ

!------------------------------------------------------------------------------

IMPLICIT NONE

LOGICAL,INTENT(IN)            :: LDMODEL_TO_FILE 
REAL(KIND=JPRB),INTENT(INOUT) :: PU   (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PV   (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PT   (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PTL  (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PTM  (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PSPD (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PSVD (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PSP  (NGPTOT)
REAL(KIND=JPRB),INTENT(INOUT) :: PQ   (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PQL  (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PQM  (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PL   (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PI   (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PR   (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PS   (NGPTOT,NFLEVG)
REAL(KIND=JPRB),INTENT(INOUT) :: PG   (NGPTOT,NFLEVG)

!------------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JIST,JLEV
INTEGER(KIND=JPIM) :: IST,IEND,IPROMA
LOGICAL            :: LLDER,LLGWF,LLGDWI

REAL(KIND=JPRB) :: ZOROGL(NGPTOT)
REAL(KIND=JPRB) :: ZOROGM(NGPTOT)
REAL(KIND=JPRB) :: ZPDEP (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZDELP (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZRDELP(NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZLNPR (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZALPH (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZRTGR (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZRPRE (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZRPP  (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZPREH (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZPREF (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZR    (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZRT   (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZRTL  (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZRTM  (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZNHPREF(NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZNHPPI(NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZRNHPPI(NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZSVD  (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZRRED (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZPHIH (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZPHIHYDH(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZPHIHYD(NPROMA, NFLEVG)
REAL(KIND=JPRB) :: ZUS   (NPROMA)
REAL(KIND=JPRB) :: ZVS   (NPROMA)
REAL(KIND=JPRB) :: ZGWH  (NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZGWF  (NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZIN   (NPROMA,0:NFLEVG+1)
REAL(KIND=JPRB) :: ZOU   (NPROMA,0:NFLEVG+1)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!------------------------------------------------------------------------------

#include "abor1.intfb.h"
#include "gnhpre.intfb.h"
#include "gpgeo.intfb.h"
#include "gpgw.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gprt.intfb.h"
#include "gpuvs.intfb.h"
#include "gpxyb.intfb.h"
#include "verder.intfb.h"

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNH_CONV_NHVAR_GEOGW',0,ZHOOK_HANDLE)

!------------------------------------------------------------------------------

IF (.NOT.LNH_GEOGW) THEN
  CALL ABOR1(' GNH_CONV_NHVAR_GEOGW: should not be called if LNH_GEOGW=F!')
ENDIF

!* 1. GRID POINT COMPUTATIONS
!     -----------------------

!* 1.0 MULTIPLICATION BY MAP FACTOR

! orography
ZOROGL(1:NGPTOT)=OROGL(1:NGPTOT)*GM(1:NGPTOT)
ZOROGM(1:NGPTOT)=OROGM(1:NGPTOT)*GM(1:NGPTOT)

! GMV:
DO JLEV = 1,NFLEVG
  PU   (1:NGPTOT,JLEV)=PU   (1:NGPTOT,JLEV)*GM(1:NGPTOT)
  PV   (1:NGPTOT,JLEV)=PV   (1:NGPTOT,JLEV)*GM(1:NGPTOT)
  PTL  (1:NGPTOT,JLEV)=PTL  (1:NGPTOT,JLEV)*GM(1:NGPTOT)
  PTM  (1:NGPTOT,JLEV)=PTM  (1:NGPTOT,JLEV)*GM(1:NGPTOT)
ENDDO ! JLEV

! GFL:
DO JLEV = 1,NFLEVG
  IF (YQ%LSP) THEN
    PQM(1:NGPTOT,JLEV)=PQM(1:NGPTOT,JLEV)*GM(1:NGPTOT)
    PQL(1:NGPTOT,JLEV)=PQL(1:NGPTOT,JLEV)*GM(1:NGPTOT)
  ENDIF
ENDDO ! JLEV


IF ( LDMODEL_TO_FILE ) THEN

  !* 1.1 TRANSFORM FROM MODEL TO FILE

  ! loop through sections of length NPROMA
  DO JIST=1,NGPTOT,NPROMA

    IST    = JIST                      ! first element
    IEND   = MIN(IST+NPROMA-1,NGPTOT)  ! last element
    IPROMA = IEND-IST+1                ! actual number of elements

    !* 1.1.1 COMPUTE PRESSURES

    ZPREH(1:IPROMA,NFLEVG)=EXP(PSP(IST:IEND))

    CALL GPPREH(IPROMA,1,IPROMA,NFLEVG,VAH,VBH,ZPREH(1:IPROMA,:))
    CALL GPXYB(IPROMA,1,IPROMA,NFLEVG,VDELB,VC, &
     & ZPREH(1:IPROMA,:),ZDELP(1:IPROMA,:),ZRDELP(1:IPROMA,:), &
     & ZLNPR(1:IPROMA,:),ZALPH(1:IPROMA,:),ZRTGR (1:IPROMA,:), &
     & ZRPRE(1:IPROMA,:),ZRPP (1:IPROMA,:))  
    CALL GPPREF(IPROMA,1,IPROMA,NFLEVG, &
     & ZALPH(1:IPROMA,:),ZPREH(1:IPROMA,:),ZPREF(1:IPROMA,:))  

    !* 1.1.2 COMPUTE "RT", "pre-prehyd"; fill PSPD.

    ! Compute RT:
    CALL GPRCP(IPROMA,1,IPROMA,NFLEVG, &
     & PQ=PQ(IST:IEND,:),PQI=PI(IST:IEND,:),PQL=PL(IST:IEND,:), &
     & PQR=PR(IST:IEND,:),PQS=PS(IST:IEND,:),PQG=PG(IST:IEND,:), &
     & PR=ZR(1:IPROMA,:))
    CALL GPRT(LSPRT,IPROMA,1,IPROMA,NFLEVG,RD,RV, &
     & ZR(1:IPROMA,:),PT(IST:IEND,:),PTL(IST:IEND,:),PTM(IST:IEND,:), &
     & PQL(IST:IEND,:),PQM(IST:IEND,:), &
     & ZRT(1:IPROMA,:),ZRTL(1:IPROMA,:),ZRTM(1:IPROMA,:))
    ! Compute the hydrostatic part of Phi:
    ZPHIHYDH(1:IPROMA,NFLEVG)=OROG(IST:IEND)
    CALL GPGEO(IPROMA,1,IPROMA,NFLEVG, &
     & ZPHIHYDH(1:IPROMA,:),ZPHIHYD(1:IPROMA,:),PT(IST:IEND,:),&
     & ZR(1:IPROMA,:),ZLNPR(1:IPROMA,:),ZALPH(1:IPROMA,:))
    ! Compute "pre-prehyd":
    CALL GNHPRE(IPROMA,NFLEVG,1,IPROMA,PSPD(IST:IEND,:),ZPREF(1:IPROMA,:),&
     & ZNHPREF(1:IPROMA,:),ZNHPPI(1:IPROMA,:),ZRNHPPI(1:IPROMA,:),&
     & ZPDEP(1:IPROMA,:),PLNPR=ZLNPR(1:IPROMA,:),PRT=ZRT(1:IPROMA,:),&
     & POROG=OROG(IST:IEND),PHIHYD=ZPHIHYD(1:IPROMA,:))
    PSPD(IST:IEND,1:NFLEVG)=ZPDEP(1:IPROMA,1:NFLEVG)

    !* 1.1.3 COMPUTE "Gw"; fill PSVD.

    ! Remark: the code computes (-G dw) from Gw, but the question to apply
    !  the derivative to the pseudo NH departure of 'Gw' (cf. option
    !  LGWOPT1=T in CPG_GP) is also relevant here (currently not coded).
    ! Compute surface wind, then Gw_s:
    LLDER=.FALSE.
    CALL GPUVS(NFLEVG,IPROMA,1,IPROMA,LLDER, &
     & PU(IST:IEND,:),PV(IST:IEND,:),ZUS(1:IPROMA),ZVS(1:IPROMA))
    ZGWH(1:IPROMA,NFLEVG)= &
     & ZUS(1:IPROMA)*ZOROGL(IST:IEND)+ZVS(1:IPROMA)*ZOROGM(IST:IEND)
    ! Compute -G dw from Gw:
    IF (LVERTFE) THEN
      IF (NVDVAR == 5) THEN
        PSVD(IST:IEND,JLEV)=PSVD(IST:IEND,JLEV)+VBF(JLEV)*ZGWH(1:IPROMA,NFLEVG)
      ENDIF
      ZIN(1:IPROMA,1:NFLEVG)=PSVD(IST:IEND,1:NFLEVG)
      ! Constant upper BC.
      ZIN(1:IPROMA,0)=ZIN(1:IPROMA,1)
      ! Lower BC: (Gw)_surf = Gw_s.
      ZIN(1:IPROMA,NFLEVG+1)=ZGWH(1:IPROMA,NFLEVG)
      IF (LVFE_DERIB) THEN
        CALL VERDER(IPROMA,1,IPROMA,NFLEVG+2,&
         & ZIN(1:IPROMA,0),ZOU(1:IPROMA,0),.TRUE.)
      ELSE
        CALL VERDER(IPROMA,1,IPROMA,NFLEVG,ZIN(1:IPROMA,1),ZOU(1:IPROMA,1))
      ENDIF
      ZSVD(1:IPROMA,JLEV)=-ZOU(1:IPROMA,JLEV)/VFE_RDETAH(JLEV) 
    ELSE
      CALL ABOR1('GNH_CONV_NHVAR_GEOGW: NVDVAR=5 or 6 is coded only for VFE!') 
    ENDIF
    PSVD(IST:IEND,1:NFLEVG)=ZSVD(1:IPROMA,1:NFLEVG)

  ENDDO  ! JIST = 1,NGPTOT,NPROMA

ELSE

  !* 1.2 TRANSFORM FROM FILE TO MODEL

  ! loop through sections of length NPROMA
  DO JIST=1,NGPTOT,NPROMA

    IST    = JIST                      ! first element
    IEND   = MIN(IST+NPROMA-1,NGPTOT)  ! last element
    IPROMA = IEND-IST+1                ! actual number of elements

    !* 1.2.1 COMPUTE PRESSURES

    ZPREH(1:IPROMA,NFLEVG)=EXP(PSP(IST:IEND))

    CALL GPPREH(IPROMA,1,IPROMA,NFLEVG,VAH,VBH,ZPREH(1:IPROMA,:))
    CALL GPXYB(IPROMA,1,IPROMA,NFLEVG,VDELB,VC, &
     & ZPREH(1:IPROMA,:),ZDELP(1:IPROMA,:),ZRDELP(1:IPROMA,:), &
     & ZLNPR(1:IPROMA,:),ZALPH(1:IPROMA,:),ZRTGR (1:IPROMA,:), &
     & ZRPRE(1:IPROMA,:),ZRPP (1:IPROMA,:))  
    CALL GPPREF(IPROMA,1,IPROMA,NFLEVG, &
     & ZALPH(1:IPROMA,:),ZPREH(1:IPROMA,:),ZPREF(1:IPROMA,:))  

    !* 1.2.2 COMPUTE Phi FROM "pre-prehyd"; fill PSPD.

    IF ( LNH_GEOGW .AND. (NPDVAR==3 .OR. NPDVAR==4)) THEN
      ! Compute R:
      CALL GPRCP(IPROMA,1,IPROMA,NFLEVG, &
       & PQ=PQ(IST:IEND,:),PQI=PI(IST:IEND,:),PQL=PL(IST:IEND,:), &
       & PQR=PR(IST:IEND,:),PQS=PS(IST:IEND,:),PQG=PG(IST:IEND,:), &
       & PR=ZR(1:IPROMA,:))
      ! Compute prehyd/pre and R*(prehyd/pre):
      DO JLEV=1,NFLEVG
        ZRNHPPI(1:IPROMA,JLEV)= &
         & ZPREF(1:IPROMA,JLEV)/(PSPD(IST:IEND,JLEV)+ZPREF(1:IPROMA,JLEV))
        ZRRED(1:IPROMA,JLEV)=ZR(1:IPROMA,JLEV)*ZRNHPPI(1:IPROMA,JLEV)
      ENDDO
      ! Compute Phi at full levels, put it in PSPD:
      ZPHIH(1:IPROMA,NFLEVG)=OROG(IST:IEND)
      CALL GPGEO(IPROMA,1,IPROMA,NFLEVG, &
       & ZPHIH(1:IPROMA,:),PSPD(IST:IEND,:),PT(IST:IEND,:),&
       & ZRRED(1:IPROMA,:),ZLNPR(1:IPROMA,:),ZALPH(1:IPROMA,:))
      IF (NPDVAR == 3) THEN
        ! Substract B*Phi_s for NPDVAR=3:
        IF (LVERTFE) THEN
          DO JLEV=1,NFLEVG
            PSPD(IST:IEND,JLEV)=PSPD(IST:IEND,JLEV)-VBF(JLEV)*OROG(IST:IEND)
          ENDDO
        ELSE
          CALL ABOR1('GNH_CONV_NHVAR_GEOGW: NPDVAR=3 is coded only for VFE!') 
        ENDIF
      ENDIF
    ENDIF

    !* 1.2.3 COMPUTE Gw FROM "-G dw"; fill PSVD.

    IF ( LNH_GEOGW .AND. (NVDVAR==5 .OR. NVDVAR==6)) THEN
      ! Compute surface wind:
      LLDER=.FALSE.
      CALL GPUVS(NFLEVG,IPROMA,1,IPROMA,LLDER, &
       & PU(IST:IEND,:),PV(IST:IEND,:),ZUS(1:IPROMA),ZVS(1:IPROMA))
      ! Compute Gw:
      LLGWF=.FALSE.
      LLGDWI=.TRUE.
      CALL GPGW(NFLEVG,IPROMA,1,IPROMA,LLGWF,LLGDWI, &
       & ZOROGL(IST:IEND),ZOROGM(IST:IEND), &
       & ZLNPR(1:IPROMA,:),ZALPH(1:IPROMA,:),&
       & ZUS(1:IPROMA),ZVS(1:IPROMA), &
       & PT(IST:IEND,:),PSVD(IST:IEND,:),ZRNHPPI(1:IPROMA,:), &
       & ZGWH(1:IPROMA,:),ZGWF(1:IPROMA,:))
      IF (LVERTFE) THEN
        IF (NVDVAR==5) THEN
          DO JLEV=1,NFLEVG
            PSVD(IST:IEND,JLEV)=ZGWF(1:IPROMA,JLEV) &
             & -VBF(JLEV)*ZGWH(1:IPROMA,NFLEVG)
          ENDDO
        ELSEIF (NVDVAR==6) THEN
          PSVD(IST:IEND,1:NFLEVG)=ZGWF(1:IPROMA,1:NFLEVG)
        ENDIF
      ELSE
        CALL ABOR1('GNH_CONV_NHVAR_GEOGW: NVDVAR=5 or 6 is coded only for VFE!')
      ENDIF
    ENDIF

  ENDDO ! JIST

ENDIF  ! LDMODEL_TO_FILE

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNH_CONV_NHVAR_GEOGW',1,ZHOOK_HANDLE)

END SUBROUTINE GNH_CONV_NHVAR_GEOGW

