!OCL  NOEVAL
SUBROUTINE GNH_TNDLAGADIAB_SVD( &
 ! --- INPUT -----------------------------------------------------------------
 & KFLEV,KPROMA,KST,KPROF, &
 & PUF,PVF,PUH,PVH,PRDPHI,PGWHL,PGWHM,PGWFL,PGWFM,PDVER,PDIV,P3DIVG, &
 & PTNDGWH_LAP,PTNDGWH_OTH,PTNDGWF_OTH, &
 & PDBBC,PDEP,PREH,PREF,PDELP,PLNPR, &
 ! --- OUTPUT ----------------------------------------------------------------
 & PTNDVD &
 & )  

!**** *GNH_TNDLAGADIAB_SVD* - Computation of the adiabatic contribution of
!                             [D (svd) / Dt] in the NH model.
!                             "svd" is the vertical divergence variable.

!     Purpose.
!     --------

!     Computation of the adiabatic contribution of [D (svd) / Dt] in the NH model
!     at full levels.
!     For svd=d4 (NVDVAR=4) the lagrangian tendency currently computed
!     is [D (dver) / Dt]_adiab (and not [D (dqua) / Dt]_adiab).

!     For svd=dver, expression of this tendency is:
!      [D (dver) / Dt]_adiab =
!       dver (D-D3)
!       - [pre/(Rd T)] [ d [ [D (gw) / Dt]_adiab ] / d prehyd ]
!       + [pre/(Rd T)] grad(gw).[ d V / d prehyd ]
!     which can be rewritten:
!      [D (dver) / Dt]_adiab =
!       dver (D-D3)
!       - [pre/(Rd T prehyd)] [ d [ [D (gw) / Dt]_adiab ] / d log(prehyd) ]
!       + [pre/(Rd T prehyd)] grad(gw).[ d V / d log(prehyd) ]

!     Calculation of the Laplacien term with VFE:
!     the Laplacian term:
!      d [ d (pre - prehyd) / d prehyd ]/ d eta
!     is rewritten as the sum of two terms:
!      (1/prehyd) * d[(pre - prehyd)/prehyd]/d eta * d[(d eta/d prehyd) prehyd**2]/d eta
!      + d eta/d(log(prehyd)) (d**2 [(pre - prehyd)/prehyd]/d eta**2)

!**   Interface.
!     ----------
!        *CALL* *GNH_TNDLAGADIAB_SVD(...)

!        Explicit arguments :
!        --------------------
!         * INPUT:
!           KFLEV     : number of levels.
!           KPROMA    : horizontal dimension.
!           KST       : start of work.
!           KPROF     : working length.
!           PUF       : U-wind at full levels.
!           PVF       : V-wind at full levels.
!           PUH       : U-wind at half levels.
!           PVH       : V-wind at half levels.
!           PRDPHI    : term "pre / (Rd T prehyd delta)" at full levels.
!           PGWHL     : zonal comp. of grad(gw) at half levels.
!           PGWHM     : merid comp. of grad(gw) at half levels.
!           PGWFL     : zonal comp. of grad(gw) at full levels.
!           PGWFM     : merid comp. of grad(gw) at full levels.
!           PDVER     : vertical divergence "dver" at full levels.
!           PDIV      : horizontal divergence at full levels.
!           P3DIVG    : total divergence D3 at full levels.
!           PTNDGWH_LAP: tendency [D (gw) / Dt]_adiab at half levels (Lapl term)
!           PTNDGWH_OTH: tendency [D (gw) / Dt]_adiab at half levels (other)
!           PTNDGWF_OTH: tendency [D (gw) / Dt]_adiab at full levels (other)
!           PDBBC     : [D (gw)_surf / Dt]_adiab.
!           PDEP      : (pre-prehyd) at full levels.
!           PREH      : prehyd at half levels.
!           PREF      : prehyd at full levels.
!           PDELP     : [Delta prehyd] at full levels.
!           PLNPR     : quantity "delta" at full levels.

!         * OUTPUT:
!           PTNDVD    : tendency [D (svd) / Dt]_adiab at full levels.

!         Remark: input data PUF to PTNDGWH are at time t.

!        Implicit arguments :   None.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.    None.
!     ----------

!     Reference.
!     ----------
!        See documentations Arpege about the NH model.

!     Author.
!     -------
!        K. YESSAD (after routine GNHPDVD).

!     Modifications.
!     --------------
!   Original : 08-Dec-2004
!   J. Vivoda and K. Yessad (Oct 2007): VFE implementation.
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

USE YOMCVER  , ONLY : LVERTFE  ,LVFE_LAPL,LVFE_Z_TERM,LVFE_LAPL_BC,LVFE_DERIB
USE YOMCST   , ONLY : RG
USE YOMGEM   , ONLY : VFE_RDETAH
USE YOMDYNA  , ONLY : NVDVAR
USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVF(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUH(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVH(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDPHI(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGWHL(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGWHM(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGWFL(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGWFM(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDVER(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIV(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: P3DIVG(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTNDGWH_LAP(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTNDGWH_OTH(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTNDGWF_OTH(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDBBC(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDEP(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREH(KPROMA,0:KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREF(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTNDVD(KPROMA,KFLEV)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF,JLEV
REAL(KIND=JPRB) :: ZRG2, ZDETA
REAL(KIND=JPRB) :: ZLVGW(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZOTH(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZLAPL(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZDX(KPROMA,0:KFLEV+1)
REAL(KIND=JPRB) :: ZDDX(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZPI2MDETA(KPROMA,0:KFLEV+1)
REAL(KIND=JPRB) :: ZIN(KPROMA,0:KFLEV+1)
REAL(KIND=JPRB) :: ZDUF(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZDVF(KPROMA,KFLEV)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "verder.intfb.h"
#include "verdder.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNH_TNDLAGADIAB_SVD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    LAPLACIAN CONTRIBUTION OF (D gw/D t)
!              ------------------------------------

ZLAPL=0.0_JPRB
ZRG2=RG*RG

IF (LVERTFE.AND.LVFE_LAPL) THEN

  DO JLEV=1,KFLEV
    DO JROF=KST,KPROF
      ZIN(JROF,JLEV)=PDEP(JROF,JLEV)/PREF(JROF,JLEV)
    ENDDO
  ENDDO
  IF (LVFE_DERIB) THEN
    ! These top and bottom conditions should remain consistent with
    !  the ones done in GNHPREH for "(pre-prehyd)/prehyd".
    ZIN(KST:KPROF,0)=0.0_JPRB
    ZIN(KST:KPROF,KFLEV+1)=ZIN(KST:KPROF,KFLEV)
    CALL VERDER(KPROMA,KST,KPROF,KFLEV+2,ZIN(1,0),ZDX(1,0),.TRUE.)
  ELSE
    CALL VERDER(KPROMA,KST,KPROF,KFLEV,ZIN(1,1),ZDX(1,1))
  ENDIF
  CALL VERDDER(KPROMA,KST,KPROF,KFLEV,ZIN(1,1),ZDDX)

  ! d/deta (pi^2/[d pi/d eta])
  DO JLEV=1,KFLEV
    DO JROF=KST,KPROF
      ZIN(JROF,JLEV)=(PREF(JROF,JLEV)**2)/(PDELP(JROF,JLEV)*VFE_RDETAH(JLEV))
    ENDDO
  ENDDO
  IF (LVFE_DERIB) THEN
    DO JROF=KST,KPROF
      ZIN(JROF,0)=PREH(JROF,0)*PREH(JROF,0)/(PDELP(JROF,1)*VFE_RDETAH(1))
      ZIN(JROF,KFLEV+1)=PREH(JROF,KFLEV)*PREH(JROF,KFLEV) &
       & /(PDELP(JROF,KFLEV)*VFE_RDETAH(KFLEV))
    ENDDO
    CALL VERDER(KPROMA,KST,KPROF,KFLEV+2,ZIN(1,0),ZPI2MDETA(1,0),.TRUE.)
  ELSE
    CALL VERDER(KPROMA,KST,KPROF,KFLEV,ZIN(1,1),ZPI2MDETA(1,1))
  ENDIF

  IF(LVFE_LAPL_BC)THEN

    DO JLEV=1,KFLEV
      ZDETA=1.0_JPRB/VFE_RDETAH(JLEV)
      DO JROF=KST,KPROF
        ZLAPL(JROF,JLEV) = -ZRG2*PRDPHI(JROF,JLEV)* ( &
         & ZPI2MDETA(JROF,JLEV)*ZDX(JROF,JLEV)*ZDETA/PREF(JROF,JLEV) &
         & + ZDDX(JROF,JLEV)*ZDETA*ZDETA/PLNPR(JROF,JLEV))
      ENDDO
    ENDDO

  ELSE

    DO JLEV=2,KFLEV-1
      ZDETA=1.0_JPRB/VFE_RDETAH(JLEV)
      DO JROF=KST,KPROF
        ZLAPL(JROF,JLEV) = -ZRG2*PRDPHI(JROF,JLEV)* ( &
         & ZPI2MDETA(JROF,JLEV)*ZDX(JROF,JLEV)*ZDETA/PREF(JROF,JLEV) &
         & + ZDDX(JROF,JLEV)*ZDETA*ZDETA/PLNPR(JROF,JLEV))
      ENDDO
    ENDDO

    DO JROF=KST,KPROF

      ! enforce FD TBC

      !! ! Von Neumann
      !! ZWC1 = 1.0_JPRB/(PREF(JROF,2)-PREF(JROF,1))
      !! ZWB1 =-ZWC1
      !! ! Dirichlet
      !! ZWB1 = ZWB1*PREF(JROF,2)/PREF(JROF,1)
      !! ZLAPL(JROF,1)=-ZRG2*(ZWB1*PDEP(JROF,1)+ZWC1*PDEP(JROF,2))*PRDPHI(JROF,1)

      ! ky: I would prefer this: simpler to read, equivalent to the previous
      !     one, and does not hid the hypothesis prehyd_top=0.
      ZLAPL(JROF,1)=-PRDPHI(JROF,1)* &
       & (PTNDGWH_LAP(JROF,1)-PTNDGWH_LAP(JROF,0))

      ! enforce FD BBC
      !! ZWSL = ZRG2/(PREF(JROF,KFLEV)-PREF(JROF,KFLEV-1))
      !! ZLAPL(JROF,KFLEV)= (ZWSL*(PDEP(JROF,KFLEV)-PDEP(JROF,KFLEV-1) ) &
      !!  & + PDBBC(JROF) )*PRDPHI(JROF,KFLEV)

      ! ky: I would prefer this: simpler to read, equivalent to the previous
      !     one.
      ZLAPL(JROF,KFLEV)=-PRDPHI(JROF,KFLEV)* &
       & (PTNDGWH_LAP(JROF,KFLEV)-PTNDGWH_LAP(JROF,KFLEV-1))

    ENDDO

  ENDIF

ELSE

  DO JLEV=1,KFLEV
    DO JROF=KST,KPROF
      ZLAPL(JROF,JLEV)=-PRDPHI(JROF,JLEV)* &
        & (PTNDGWH_LAP(JROF,JLEV)-PTNDGWH_LAP(JROF,JLEV-1))
    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

!*       2.    OTHER CONTRIBUTIONS OF (D gw/D t)
!              ---------------------------------

! * Provisionally, simply put ZOTH to 0, but finally, after implementation
!   of the deep-layer equations (where this term is non-zero), the code
!   will look like that:
!   IF (LVERTFE .AND. LVFE_GW) THEN
!     apply a first-order derivative via VERDER to PTNDGWF_OTH
!   ELSE
!     ZOTH(.,jlev)=-PRDPHI(.,jlev)*(PTNDGWH_OTH(.,jlev)-PTNDGWH_OTH(.,jlev-1))
!   ENDIF
!   An alternative will be to use a new key LVFE_OTH instead of LVFE_GW.

ZOTH(KST:KPROF,1:KFLEV)=0.0_JPRB

!     ------------------------------------------------------------------

!*       3.    Z-TERM
!              ------

IF (NVDVAR == 3 .OR. NVDVAR == 4) THEN
  IF ( LVERTFE .AND. LVFE_Z_TERM ) THEN
    CALL VERDER(KPROMA,KST,KPROF,KFLEV,PUF,ZDUF)
    CALL VERDER(KPROMA,KST,KPROF,KFLEV,PVF,ZDVF)
    DO JLEV=1,KFLEV
      DO JROF=KST,KPROF
        ZLVGW(JROF,JLEV)=PRDPHI(JROF,JLEV)/VFE_RDETAH(JLEV)* &
         & (PGWFL(JROF,JLEV)*ZDUF(JROF,JLEV)+PGWFM(JROF,JLEV)*ZDVF(JROF,JLEV))
      ENDDO
    ENDDO
  ELSE
    DO JLEV=1,KFLEV
      DO JROF=KST,KPROF
        ZLVGW(JROF,JLEV)=((PUH(JROF,JLEV)-PUF(JROF,JLEV))*PGWHL(JROF,JLEV)&
         & +(PVH(JROF,JLEV)-PVF(JROF,JLEV))*PGWHM(JROF,JLEV)&
         & +(PUF(JROF,JLEV)-PUH(JROF,JLEV-1))*PGWHL(JROF,JLEV-1)&
         & +(PVF(JROF,JLEV)-PVH(JROF,JLEV-1))*PGWHM(JROF,JLEV-1)&
         & )*PRDPHI(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       4.    COMPUTATION OF [D (svd) / Dt]_adiab: SUM OF ALL CONTRIBS
!              --------------------------------------------------------

IF (NVDVAR == 3 .OR. NVDVAR == 4) THEN
  DO JLEV=1,KFLEV
    DO JROF=KST,KPROF
      PTNDVD(JROF,JLEV)=PDVER(JROF,JLEV)*(PDIV(JROF,JLEV)-P3DIVG(JROF,JLEV)) &
        & +ZLAPL(JROF,JLEV)+ZOTH(JROF,JLEV)+ZLVGW(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNH_TNDLAGADIAB_SVD',1,ZHOOK_HANDLE)
END SUBROUTINE GNH_TNDLAGADIAB_SVD
