!OCL  NOEVAL
SUBROUTINE GNH_TNDLAGADIAB_UVS( &
 ! --- INPUT -----------------------------------------------------------------
 & KPROMA,KSTART,KPROF, &
 & LDPLANE, &
 & PRCORI,PRATATX,PRATATH,PUS,PVS,PSGRTSL,PSGRTSM,&
 ! --- OUTPUT ----------------------------------------------------------------
 & PTNDUS,PTNDVS &
 & )  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!**** *GNH_TNDLAGADIAB_UVS* - Computation of the adiabatic contribution of
!                             [D Vsurf / Dt] in the NH model.

!     Purpose.
!     --------

!     Computation of the adiabatic contribution of [D Vsurf / Dt] in the NH model.
!     Coriolis force term is assumed to be always explicit in this calculation,
!      even if ladvf=T or limpf=T.
!     The terms which are taken in account are:
!     - the Coriolis force.
!     - the pressure gradient force.

!     This routine must be called in the NH model only.
!     It must remain consistent with the calculation of upper air [D V/D t]
!      done in routine GNHDYN (EUL) or LATTEX (SL).

!**   Interface.
!     ----------
!        *CALL* *GNH_TNDLAGADIAB_UVS(...)

!        Explicit arguments :
!        --------------------
!         * INPUT:
!           KPROMA      : horizontal dimension.
!           KSTART      : start of work.
!           KPROF       : working length.
!           LDPLANE     : .T./.F.: plane/spherical geometry.
!           PRCORI      : Coriolis parameter "f = 2 Omega sin(theta)".
!           PRATATH     : spherical geometry: 2 tan(theta_bne)/a
!                         (theta_bne = tilted unstretched sphere latitude)
!                         plane geometry: 2 C (sin(theta) - K_L)/(a cos(theta))
!           PRATATX     : spherical geometry: zero
!                         plane geometry: 2 S (sin(theta) - K_L)/(a cos(theta))
!           PUS         : surface U wind.
!           PVS         : surface V wind.
!           PSGRTSL     : zonal comp. of surf pressure (semi-reduc) grad force
!           PSGRTSM     : merid comp. of surf pressure (semi-reduc) grad force

!         * OUTPUT:
!           PTNDUS      : U-component of [D Vsurf / Dt]_adiab.
!           PTNDVS      : V-component of [D Vsurf / Dt]_adiab.

!        Implicit arguments :   None.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.    None.
!     ----------

!     Reference.
!     ----------
!        See documentations Arpege about the NH model.

!     Author.
!     -------
!        K. YESSAD (after routine GNHPDVD).

!     Modifications.
!     --------------
!   Original : 08-Dec-2004
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LDPLANE
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRATATX(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRATATH(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUS(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVS(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTSL(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTSM(KPROMA)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTNDUS(KPROMA)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTNDVS(KPROMA)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF
REAL(KIND=JPRB) :: ZCUR

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNH_TNDLAGADIAB_UVS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    COMPUTATION OF [D Vsurf / Dt]_adiab
!              -----------------------------------

DO JROF=KSTART,KPROF
  PTNDUS(JROF)=PRCORI(JROF)*PVS(JROF)-PSGRTSL(JROF)
  PTNDVS(JROF)=-PRCORI(JROF)*PUS(JROF)-PSGRTSM(JROF)
ENDDO

! Useless for the time being (LRWSDLW not yet coded for options
! requiring (PTNDUS,PTNDVS)).
! IF (LRWSDLW) THEN
!   DO JROF=KSTART,KPROF
!     PTNDUS(JROF)=PTNDUS(JROF)-PGNORDM(JROF)*PRCORIC(JROF)*PWS(JROF)
!     PTNDVS(JROF)=PTNDVS(JROF)+PGNORDL(JROF)*PRCORIC(JROF)*PWS(JROF)
!   ENDDO
! ENDIF

! * Add some curvature terms in plane geometry.
IF (LDPLANE) THEN
  DO JROF=KSTART,KPROF
    ZCUR=0.5_JPRB*(PVS(JROF)*PRATATX(JROF)+PUS(JROF)*PRATATH(JROF))
    PTNDUS(JROF)=PTNDUS(JROF)+PVS(JROF)*ZCUR
    PTNDVS(JROF)=PTNDVS(JROF)+PUS(JROF)*ZCUR
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNH_TNDLAGADIAB_UVS',1,ZHOOK_HANDLE)
END SUBROUTINE GNH_TNDLAGADIAB_UVS
