SUBROUTINE GNHD3( &
 ! --- INPUT -----------------------------------------------------------------
 & KFLEV,KPROMA,KSTART,KEND, &
 & PDIV,PNHX,PDVER,PR, &
 ! --- OUTPUT ----------------------------------------------------------------
 & P3DIVG, &
 ! --- OPTIONAL INPUT --------------------------------------------------------
 & PRDLR &
 & )

! GNHD3 - Diagnose D3-term in the NH model.

! Purpose
! -------
!   Diagnose D3-term in the NH model from D, NHX, dver, R.
!    D3 = D + NHX + dver (Rd/R) = D + dqua + dver (Rd/R - 1)
!   Remember that (thin layer equations):
!    dver = - (g pre)/(Rd T) (d w/d prehyd)
!    dver (Rd/R) = - (g pre)/(RT) (d w/d prehyd)
!    dqua = dver + NHX

! Interface
! ---------
!   * INPUT:
!   KFLEV   - number of levels.
!   KPROMA  - length of work.
!   KSTART  - start of work.
!   KEND    - end of work.
!   PDIV    - Horizontal divergence D = grad V at full levels.
!   PNHX    - NHX-term at full levels.
!   PDVER   - vertical divergence at full levels.
!   PR      - R at full levels.

!   * OUTPUT:
!   P3DIVG  - D3-term at full levels.

!   * OPTIONAL INPUT:
!   PRDLR   - array containing "r/a", "a/r", "grad(r/a)".

! Externals
! ---------

! Method
! ------

! Reference
! ---------

! Author
! ------
!   06 Dec 2004 K. Yessad (after GNHPDVD).

! Modifications
! -------------
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
! End Modifications
!---------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCST   , ONLY : RD
USE YOMDYNA  , ONLY : LRWSDLR
USE PAR_RDLR , ONLY : JP_ASRF  ,JP_DIMR
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

! -----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIV(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNHX(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDVER(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P3DIVG(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)    :: PRDLR(KPROMA,0:KFLEV,JP_DIMR)

! -----------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZHOOK_HANDLE

! -----------------------------------------------------------------------------

#include "abor1.intfb.h"

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHD3',0,ZHOOK_HANDLE)

! -----------------------------------------------------------------------------

IF (LRWSDLR) THEN
  ! Take account to the fact that we need D=nabla.V (geographic div) and
  ! that PDIV contains (r/a)*nabla.V (semi-reduced div).
  IF (.NOT.PRESENT(PRDLR)) CALL ABOR1(' GNHD3: PRDLR should be present!')
  DO JLEV=1,KFLEV
    DO JROF=KSTART,KEND
      P3DIVG(JROF,JLEV)=PRDLR(JROF,JLEV,JP_ASRF)*PDIV(JROF,JLEV) &
       & +PNHX(JROF,JLEV)+PDVER(JROF,JLEV)*(RD/PR(JROF,JLEV))
    ENDDO
  ENDDO
ELSE
  DO JLEV=1,KFLEV
    DO JROF=KSTART,KEND
      P3DIVG(JROF,JLEV)=PDIV(JROF,JLEV) &
       & +PNHX(JROF,JLEV)+PDVER(JROF,JLEV)*(RD/PR(JROF,JLEV))
    ENDDO
  ENDDO
ENDIF

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHD3',1,ZHOOK_HANDLE)

END SUBROUTINE GNHD3

