SUBROUTINE GNHDLRA( &
 ! ----- INPUT ---------------------------------------------------------------
 & KPROMA,KDIMGFL,KST,KEND,KGFLTYP,LDSPDFI,LDGEO,POROG,PSP,PT,PSPD, &
 ! ----- OUTPUT --------------------------------------------------------------
 & PRDLR, &
 ! ----- OPTIONAL INPUT AND OUTPUT -------------------------------------------
 & PIIF,PQ,PL,PI,PR,PS,PG,PGFL,PDPI,PIF,PIH)

! -----
! REMARKS:
!  - The structure of this routine must remain consistent with the
!    one of CPG_GP and GNHX.
!  - PSP contains log(presim), where d presim/d prehyd = r**2/a**2.
! -----

! GNHDLRA - Diagnose "r" radius variables like "r/a", "a/r".
!           Hat routine for GNHDLR, computing also the intermediate quantities
!           necessary to compute "r" (like "presim", R, "pre-presim").

! Purpose
! -------
!   See above, and also comments in GNHDLR. 

! Interface
! ---------
! * INPUT:
!   KPROMA  - length of work
!   KDIMGFL - number of GFL fields 
!   KST     - start of work
!   KEND    - end of work
!   KGFLTYP - cf. KGFLTYP in GPRCP (ex: KGFLTYP=1 for t+dt variables).
!   LDSPDFI - T/F: content of SPD variable is the model one/progn var one.
!   LDGEO   - T/F: call GPGEO/call GNHDLR.
!   POROG   - surface geopotential
!   PSP     - ln(presim)
!   PT      - temperature T
!   PSPD    - NH pressure departure variable.

! * OUTPUT:
!   PRDLR   - array containing "r/a", "a/r". 

! * OPTIONAL INPUT:
!   PIIF    - input "presim" at full levels. 
!   PQ      - specific humidity q
!   PL      - specific mass of liquid ql
!   PI      - specific mass of ice qi
!   PR      - specific mass of rain qr
!   PS      - specific mass of snow qs
!   PG      - specific mass of graupel qg
!   PGFL    - GFL variables

! * OPTIONAL OUTPUT:
!   PDPI    - [Delta presim] at full levels. 
!   PIF     - "presim" at full levels. 
!   PIH     - "presim" at half levels. 

! Externals
! ---------

! Method
! ------

! Reference
! ---------
!        ECMWF Research Department documentation of the IFS
!        ARPEGE documentation (IDEUL)
!        Wood and Staniforth, 2003 (in QJRMS)

! Author
! ------
!   K. Yessad (METEO-FRANCE/CNRM/GMAP). March 2008 

! Modifications
! -------------
! End Modifications
!---------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCST   , ONLY : RA       ,RG 
USE YOMDYNA  , ONLY : LRWSDLR
USE YOMDIM   , ONLY : NFLEVG
USE YOMGEM   , ONLY : VAH      ,VBH      ,VDELB    ,VC
USE PAR_RDLR , ONLY : JP_DIMR  ,JP_RSAF  ,JP_RSAH  ,JP_ASRF  ,JP_ASRH

USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

! -----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)        :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)        :: KDIMGFL
INTEGER(KIND=JPIM),INTENT(IN)        :: KST 
INTEGER(KIND=JPIM),INTENT(IN)        :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)        :: KGFLTYP
LOGICAL,INTENT(IN)                   :: LDSPDFI
LOGICAL,INTENT(IN)                   :: LDGEO
REAL(KIND=JPRB)   ,INTENT(IN)        :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)        :: PSP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)        :: PT(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)        :: PSPD(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)       :: PRDLR(KPROMA,0:NFLEVG,JP_DIMR) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)  :: PIIF(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)  :: PQ(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)  :: PL(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)  :: PI(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)  :: PR(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)  :: PS(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)  :: PG(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)  :: PGFL(KPROMA,NFLEVG,KDIMGFL) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PDPI(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PIF(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PIH(KPROMA,0:NFLEVG) 

! -----------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF,JLEV

REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB) :: ZRAG,ZPRE
REAL(KIND=JPRB) :: ZALPH (KPROMA, NFLEVG)  ! alpha
REAL(KIND=JPRB) :: ZLNPR (KPROMA, NFLEVG)  ! "delta"=delta(presim)/presim
REAL(KIND=JPRB) :: ZDPI  (KPROMA, NFLEVG)  ! delta(presim)
REAL(KIND=JPRB) :: ZRDPI (KPROMA, NFLEVG)  ! 1/delta(presim)
REAL(KIND=JPRB) :: ZPIH  (KPROMA, 0:NFLEVG)! presim at half levels
REAL(KIND=JPRB) :: ZPIF  (KPROMA, NFLEVG)  ! presim at full levels
REAL(KIND=JPRB) :: ZRPIF (KPROMA, NFLEVG)  ! 1/presim at full levels
REAL(KIND=JPRB) :: ZRPP  (KPROMA, NFLEVG)  ! 1/presim**2 at full levels
REAL(KIND=JPRB) :: ZRTGR (KPROMA, NFLEVG)  ! 1/delta(presim)*(VDELB + VC/presim)
REAL(KIND=JPRB) :: ZR    (KPROMA, NFLEVG)  ! gas "constatnt" of air R
REAL(KIND=JPRB) :: ZNHPREF(KPROMA,NFLEVG)  ! "pre" at full levels.
REAL(KIND=JPRB) :: ZNHPPI (KPROMA,NFLEVG)  ! "pre/presim" at full levels.
REAL(KIND=JPRB) :: ZRNHPPI(KPROMA,NFLEVG)  ! "presim/pre" at full levels.
REAL(KIND=JPRB) :: ZPDEP  (KPROMA,NFLEVG)  ! "pre-presim" at full levels.
REAL(KIND=JPRB) :: ZPHIF(KPROMA,NFLEVG),ZPHIH(KPROMA,0:NFLEVG)

! -----------------------------------------------------------------------------

#include "abor1.intfb.h"
#include "gnhdlr.intfb.h"
#include "gnhpre.intfb.h"
#include "gpgeo.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gpxyb.intfb.h"

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHDLRA',0,ZHOOK_HANDLE)

! -----------------------------------------------------------------------------

IF (.NOT.LRWSDLR) CALL ABOR1(' GNHDLRA should not be called if LRWSDLR=F!')

ZRAG=1.0/(RA*RG)

! -----
! computation of pressures
! -----

! * compute "presims"
DO JROF=KST,KEND
  ZPIH(JROF,NFLEVG)=EXP(PSP(JROF))
ENDDO
! * compute "presim" at half levels:
CALL GPPREH(KPROMA,KST,KEND,NFLEVG,VAH,VBH,ZPIH)
IF (PRESENT(PIH)) PIH(KST:KEND,0:NFLEVG)=ZPIH(KST:KEND,0:NFLEVG)
! * compute "presim" at full levels:
CALL GPXYB(KPROMA,KST,KEND,NFLEVG,VDELB,VC,ZPIH,ZDPI,ZRDPI,ZLNPR, &
 & ZALPH,ZRTGR,ZRPIF,ZRPP)
IF (PRESENT(PDPI)) PDPI(KST:KEND,1:NFLEVG)=ZDPI(KST:KEND,1:NFLEVG)
CALL GPPREF(KPROMA,KST,KEND,NFLEVG,ZALPH,ZPIH,ZPIF)  
IF (PRESENT(PIF)) PIF(KST:KEND,1:NFLEVG)=ZPIF(KST:KEND,1:NFLEVG)

! -----
! computation of R
! -----
! * compute R:
IF (PRESENT(PGFL)) THEN
  ! use PGFL. 
  CALL GPRCP(KPROMA,KST,KEND,NFLEVG,PGFL=PGFL,PR=ZR,KGFLTYP=KGFLTYP)
ELSE
  ! use individual GFL: PQ,PI,PL...
  IF (.NOT.(PRESENT(PQ).AND.PRESENT(PL).AND.PRESENT(PI).AND.PRESENT(PR) &
   & .AND.PRESENT(PS).AND.PRESENT(PG))) &
   & CALL ABOR1(' GNHDLRA: missing optional input PQ to PG')
  CALL GPRCP(KPROMA,KST,KEND,NFLEVG,PQ=PQ,PQI=PI,PQL=PL,PQR=PR,PQS=PS, &
   & PQG=PG,PR=ZR)  
ENDIF

! * Compute 'pre-presim' quantities.
IF (LDSPDFI) THEN
  IF (PRESENT(PIIF)) THEN
    ! PSPD contains 'pre-(input presim)'.
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZPRE=MAX(0._JPRB,PSPD(JROF,JLEV)+PIIF(JROF,JLEV))
        ZRNHPPI(JROF,JLEV)=ZPIF(JROF,JLEV)/ZPRE
      ENDDO
    ENDDO
  ELSE
    ! PSPD contains 'pre-presim'.
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZPRE=MAX(0._JPRB,PSPD(JROF,JLEV)+ZPIF(JROF,JLEV))
        ZRNHPPI(JROF,JLEV)=ZPIF(JROF,JLEV)/ZPRE
      ENDDO
    ENDDO
  ENDIF
ELSE
  ! PSPD contains prognostic variable (depending on NPDVAR).
  CALL GNHPRE(KPROMA,NFLEVG,KST,KEND,PSPD,ZPIF,ZNHPREF,ZNHPPI,ZRNHPPI,ZPDEP)
ENDIF

! * compute "r/a".
IF (LDGEO) THEN
  ! Case where we compute a guess of "r" from "prehyd".
  ZPHIH(KST:KEND,NFLEVG)=POROG(KST:KEND)
  ZR(KST:KEND,1:NFLEVG)=ZR(KST:KEND,1:NFLEVG)*ZRNHPPI(KST:KEND,1:NFLEVG)
  CALL GPGEO(KPROMA,KST,KEND,NFLEVG,ZPHIH,ZPHIF,PT,ZR,ZLNPR,ZALPH)
  PRDLR(KST:KEND,1:NFLEVG,JP_RSAF)=ZPHIF(KST:KEND,1:NFLEVG)*ZRAG+1.0_JPRB
  PRDLR(KST:KEND,0:NFLEVG,JP_RSAH)=ZPHIH(KST:KEND,0:NFLEVG)*ZRAG+1.0_JPRB
  PRDLR(KST:KEND,1:NFLEVG,JP_ASRF)=1.0_JPRB/PRDLR(KST:KEND,1:NFLEVG,JP_RSAF)
  PRDLR(KST:KEND,0:NFLEVG,JP_ASRH)=1.0_JPRB/PRDLR(KST:KEND,0:NFLEVG,JP_RSAH)
ELSE
  CALL GNHDLR(KPROMA,NFLEVG,KST,KEND,PT,ZR,ZRNHPPI,ZLNPR,ZALPH,POROG,PRDLR)
ENDIF

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHDLRA',1,ZHOOK_HANDLE)

END SUBROUTINE GNHDLRA

