!OCL NOEVAL
SUBROUTINE GNHDLRB( &
 ! ----- INPUT ---------------------------------------------------------------
 & KPROMA,KST,KEND,LDCPI,POROG,PT,PR,PSPD, &
 ! ----- INOUT ---------------------------------------------------------------
 & PREFI,PREHI,PDELPI,PRDELPI,PLNPRI,PALPHI, &
 ! ----- OPTIONAL OUTPUT -----------------------------------------------------
 & PREFO,PREHO,PDELPO,PRDELPO,PLNPRO,PALPHO,PRPREHO,PRPPO,PASR2,PRSAH)

! GNHDLRB:
!  Retrieves true hydrostatic pressure 'prehyd' profiles from "mass integrated
!  coordinate 'presim'" profiles when LRWSDLR=T.
!   - Assumes that "presim", and quantities depending on "presim"
!     (computed in GPPREH+GPXYB+GPPREF) have been already computed
!     (input variables).
!   - Assumes that moist air constant R has been computed.
!   - Diagnose "r" radius variables like "r/a", "a/r".
!   - Computes the "true hydrostatic pressure" equivalents of the outputs
!     of GPPREH+GPXYB+GPPREF, by applying a correction.
!     These "true hydrostatic pressure" variables are necessary as input
!     to processes such as physics.
!   - We recall that d(prehyd)/d(presim)=(a**2/r**2)*(g/G).

! Purpose
! -------
!   See above, and also comments in GNHDLR. 

! Interface
! ---------
! * INPUT:
!   KPROMA  - length of work
!   KST     - start of work
!   KEND    - end of work
!   LDCPI   - T/F: copy "prehyd" arrays into "presim" vs "prehyd" arrays
!   POROG   - surface geopotential
!   PT      - temperature T
!   PR      - moist air constant R 
!   PSPD    - NH pressure departure variable.

! * INOUT:
!   PREFI   - "presim" at full levels
!   PREHI   - "presim" at half levels
!   PDELPI  - "Delta presim" at full levels
!   PRDELPI - "1/Delta presim" at full levels
!   PLNPRI  - "Delta log(presim)" at full levels
!   PALPHI  - "alpha" based on "presim" at full levels

! * OPTIONAL OUTPUT:
!   PREFO   - "prehyd" at full levels
!   PREHO   - "prehyd" at half levels
!   PDELPO  - "Delta prehyd" at full levels
!   PRDELPO - "1/Delta prehyd" at full levels
!   PLNPRO  - "Delta log(prehyd)" at full levels
!   PALPHO  - "alpha" based on "prehyd" at full levels
!   PRPREHO - "1/prehyd" at half levels
!   PRPPO   - cf. PRPP in GPXYB 
!   PASR2   - "(a**2/r**2)*(g/G)" at full levels
!   PRSAH   - "r/a" at half levels

! Externals
! ---------

! Method
! ------

! Reference
! ---------
!        ECMWF Research Department documentation of the IFS
!        ARPEGE documentation (IDEUL)
!        Wood and Staniforth, 2003 (in QJRMS)

! Author
! ------
!   K. Yessad (METEO-FRANCE/CNRM/GMAP). March 2008 

! Modifications
! -------------
!  K. Yessad (March 2009): fix a bug for LRWSDLG=T
! End Modifications
!---------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMDYNA  , ONLY : LRWSDLR  ,LRWSDLG
USE YOMDIM   , ONLY : NFLEVG
USE YOMGEM   , ONLY : TOPPRES
USE PAR_RDLR , ONLY : JP_DIMR  ,JP_ASRF  ,JP_RSAH

USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

! -----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)        :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)        :: KST 
INTEGER(KIND=JPIM),INTENT(IN)        :: KEND 
LOGICAL,INTENT(IN)                   :: LDCPI
REAL(KIND=JPRB)   ,INTENT(IN)        :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)        :: PT(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)        :: PR(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)        :: PSPD(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT)     :: PREFI(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT)     :: PREHI(KPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT)     :: PDELPI(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT)     :: PRDELPI(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT)     :: PLNPRI(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT)     :: PALPHI(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PREFO(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PREHO(KPROMA,0:NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PDELPO(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PRDELPO(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PLNPRO(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PALPHO(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PRPREHO(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PRPPO(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PASR2(KPROMA,NFLEVG) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PRSAH(KPROMA,0:NFLEVG) 

! -----------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF,JLEV

REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB) :: ZNHPREF(KPROMA,NFLEVG)  ! "pre" at full levels.
REAL(KIND=JPRB) :: ZNHPPI (KPROMA,NFLEVG)  ! "pre/presim" at full levels.
REAL(KIND=JPRB) :: ZRNHPPI(KPROMA,NFLEVG)  ! "presim/pre" at full levels.
REAL(KIND=JPRB) :: ZPDEP  (KPROMA,NFLEVG)  ! "pre-presim" at full levels.
REAL(KIND=JPRB) :: ZASR2  (KPROMA,NFLEVG)  ! "a**2/r**2" at full levels.
REAL(KIND=JPRB) :: ZRDLR(KPROMA,0:NFLEVG,JP_DIMR) 
REAL(KIND=JPRB) :: ZPREFO(KPROMA,NFLEVG) 
REAL(KIND=JPRB) :: ZPREHO(KPROMA,0:NFLEVG) 
REAL(KIND=JPRB) :: ZDELPO(KPROMA,NFLEVG) 
REAL(KIND=JPRB) :: ZRDELPO(KPROMA,NFLEVG) 
REAL(KIND=JPRB) :: ZLNPRO(KPROMA,NFLEVG) 
REAL(KIND=JPRB) :: ZALPHO(KPROMA,NFLEVG) 
REAL(KIND=JPRB) :: ZZPRETOPO

! -----------------------------------------------------------------------------

#include "abor1.intfb.h"
#include "gnhdlr.intfb.h"
#include "gnhpre.intfb.h"

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHDLRB',0,ZHOOK_HANDLE)

! -----------------------------------------------------------------------------

IF (.NOT.LRWSDLR) CALL ABOR1(' GNHDLRB should not be called if LRWSDLR=F!')

! * compute 'pre-presim' quantities.
CALL GNHPRE(KPROMA,NFLEVG,KST,KEND,PSPD,PREFI,ZNHPREF,ZNHPPI,ZRNHPPI,ZPDEP)

! * compute "r/a".
CALL GNHDLR(KPROMA,NFLEVG,KST,KEND,PT,PR,ZRNHPPI,PLNPRI,PALPHI,POROG,ZRDLR)
ZASR2(KST:KEND,1:NFLEVG)= &
 & ZRDLR(KST:KEND,1:NFLEVG,JP_ASRF)*ZRDLR(KST:KEND,1:NFLEVG,JP_ASRF)
IF (LRWSDLG) ZASR2(KST:KEND,1:NFLEVG)=ZASR2(KST:KEND,1:NFLEVG)* &
 & ZRDLR(KST:KEND,1:NFLEVG,JP_ASRF)*ZRDLR(KST:KEND,1:NFLEVG,JP_ASRF)

! * correct pressure depths.
DO JLEV=1,NFLEVG
  DO JROF=KST,KEND
    ZDELPO(JROF,JLEV)=ZASR2(JROF,JLEV)*PDELPI(JROF,JLEV)
    ZRDELPO(JROF,JLEV)=1.0_JPRB/ZDELPO(JROF,JLEV)
  ENDDO
ENDDO

! * correct half level pressure.
ZPREHO(KST:KEND,0)=PREHI(KST:KEND,0)
DO JLEV=1,NFLEVG
  DO JROF=KST,KEND
    ZPREHO(JROF,JLEV)=ZPREHO(JROF,JLEV-1)+ZDELPO(JROF,JLEV)
  ENDDO
ENDDO

! * correct full level pressure.
DO JLEV=1,NFLEVG
  DO JROF=KST,KEND
    ZPREFO(JROF,JLEV)=ZPREHO(JROF,JLEV)+ZASR2(JROF,JLEV)* &
     & (PREFI(JROF,JLEV)-PREHI(JROF,JLEV))
  ENDDO
ENDDO

! * correct "alpha" and "delta".
DO JLEV=1,NFLEVG
  DO JROF=KST,KEND
    ZLNPRO(JROF,JLEV)=PLNPRI(JROF,JLEV)*ZASR2(JROF,JLEV)* &
     & PREFI(JROF,JLEV)/ZPREFO(JROF,JLEV)
    ZALPHO(JROF,JLEV)=PALPHI(JROF,JLEV)*ZASR2(JROF,JLEV)* &
     & PREFI(JROF,JLEV)/ZPREFO(JROF,JLEV)
  ENDDO
ENDDO

! * memory transfer.
IF (LDCPI) THEN
  PREFI(KST:KEND,1:NFLEVG)=ZPREFO(KST:KEND,1:NFLEVG)
  PREHI(KST:KEND,0:NFLEVG)=ZPREHO(KST:KEND,0:NFLEVG)
  PDELPI(KST:KEND,1:NFLEVG)=ZDELPO(KST:KEND,1:NFLEVG)
  PRDELPI(KST:KEND,1:NFLEVG)=ZRDELPO(KST:KEND,1:NFLEVG)
  PLNPRI(KST:KEND,1:NFLEVG)=ZLNPRO(KST:KEND,1:NFLEVG)
  PALPHI(KST:KEND,1:NFLEVG)=ZALPHO(KST:KEND,1:NFLEVG)
ELSE
  IF (.NOT.(PRESENT(PREFO).AND.PRESENT(PREHO).AND.PRESENT(PDELPO) &
   & .AND.PRESENT(PRDELPO).AND.PRESENT(PLNPRO).AND.PRESENT(PALPHO))) &
   & CALL ABOR1(' GNHDLRB: missing arg PREFO to PALPHO')
  PREFO(KST:KEND,1:NFLEVG)=ZPREFO(KST:KEND,1:NFLEVG)
  PREHO(KST:KEND,0:NFLEVG)=ZPREHO(KST:KEND,0:NFLEVG)
  PDELPO(KST:KEND,1:NFLEVG)=ZDELPO(KST:KEND,1:NFLEVG)
  PRDELPO(KST:KEND,1:NFLEVG)=ZRDELPO(KST:KEND,1:NFLEVG)
  PLNPRO(KST:KEND,1:NFLEVG)=ZLNPRO(KST:KEND,1:NFLEVG)
  PALPHO(KST:KEND,1:NFLEVG)=ZALPHO(KST:KEND,1:NFLEVG)
ENDIF
IF (PRESENT(PASR2)) PASR2(KST:KEND,1:NFLEVG)=ZASR2(KST:KEND,1:NFLEVG)
IF (PRESENT(PRSAH)) PRSAH(KST:KEND,0:NFLEVG)=ZRDLR(KST:KEND,0:NFLEVG,JP_RSAH)

! * optional PRPREHO and PRPPO.
IF (PRESENT(PRPREHO)) &
 & PRPREHO(KST:KEND,1:NFLEVG)=1.0_JPRB/ZPREHO(KST:KEND,1:NFLEVG)

IF (PRESENT(PRPPO)) THEN
  DO JLEV=2,NFLEVG
    DO JROF=KST,KEND
      PRPPO(JROF,JLEV)=1.0_JPRB/(ZPREHO(JROF,JLEV)*ZPREHO(JROF,JLEV-1))
    ENDDO
  ENDDO
  DO JROF=KST,KEND
    ZZPRETOPO=MAX(ZPREHO(JROF,0),TOPPRES)
    PRPPO(JROF,1)=1.0_JPRB/(ZPREHO(JROF,1)*ZZPRETOPO)
  ENDDO
ENDIF

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHDLRB',1,ZHOOK_HANDLE)

END SUBROUTINE GNHDLRB

