!OCL  NOEVAL
SUBROUTINE GNHGRDLR(KPROMA,KFLEV,KD,KF,&
 & PT,PR,PRNHPPI,PRTL,PRTM,&
 & PLNPR,PLNPRL,PLNPRM,PALPH,PALPHL,PALPHM,&
 & PQCHAL,PQCHAM,POROGL,POROGM,PRDLR,&
 & PHIFL,PHIFM,PHIHL,PHIHM)

!**** *GNHGRDLR* - Computes the horizontal gradient of the radius 'r' for
!     the Wood and Staniforth formulation of the NH deep-layer equations,
!     when LRWSDLR=T.

!     Purpose.
!     --------

!      Computes the horizontal gradient of the radius 'r' 
!      (more exactly the gradient of (r/a))
!      the Wood and Staniforth formulation of the NH deep-layer equations.

!      This routine does a vertical integration (according to LVERTFE) and
!      has a structure close to the one of GPGRGEO.

!      The gradients used and computed in this routine are the semi-reduced
!      ones ( "(r/a) nabla" ).

!**   Interface.
!     ----------
!        *CALL* *GNHGRDLR(...)

!        Explicit arguments :
!        --------------------
!         KPROMA   : horizontal dimension                             (input)
!         KFLEV    : number of levels                                 (input)
!         KD       : start of work                                    (input)
!         KF       : working length                                   (input)
!         PT       : temperature at full levels                       (input)
!         PR       : "R" at full levels                               (input)
!         PRNHPPI  : "presim/pre" at full levels                      (input)
!         PRTL     : zonal component of "grad(RT)" at full levels     (input) 
!         PRTM     : merid component of "grad(RT)" at full levels     (input)
!         PLNPR    : term "delta" at full levels                      (input)
!         PLNPRL   : zonal component of "grad(delta)" at full levels  (input)
!         PLNPRM   : merid component of "grad(delta)" at full levels  (input)
!         PALPH    : term "alpha" at full levels                      (input)
!         PALPHL   : zonal component of "grad(alpha)" at full levels  (input)
!         PALPHM   : merid component of "grad(alpha)" at full levels  (input)
!         PQCHAL   : zonal comp of "grad(log(pre/presim))" at full lev (input)
!         PQCHAM   : merid comp of "grad(log(pre/presim))" at full lev (input)
!         POROGL   : zonal component of "grad(surf orography)"        (input)
!         POROGM   : merid component of "grad(surf orography)"        (input)
!         PRDLR    : array containing "r/a", "a/r", "grad(r/a)"       (inout)
!         PHIFL    : zonal component of "grad(Phi)" at full levels    (output)
!                    (zonal comp. of grad(Gr) if LRWSDLG=T)
!         PHIFM    : merid component of "grad(Phi)" at full levels    (output)
!                    (merid comp. of grad(Gr) if LRWSDLG=T)
!         PHIHL    : zonal component of "grad(Phi)" at half levels    (output)
!                    (zonal comp. of grad(Gr) if LRWSDLG=T)
!         PHIHM    : merid component of "grad(Phi)" at half levels    (output)
!                    (merid comp. of grad(Gr) if LRWSDLG=T)

!        Implicit arguments :   None.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.   None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        ARPEGE documentation (IDEUL)
!        Wood and Staniforth, 2003 (in QJRMS)

!     Author.
!     -------
!        K. Yessad (METEO-FRANCE/CNRM/GMAP), after routine GPGRGEO.

!     Modifications.
!     --------------
!      Original : March 2008 
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMGEM   , ONLY : VFE_RDETAH
USE YOMCVER  , ONLY : LVERTFE
USE YOMCST   , ONLY : RA       ,RG 
USE YOMDYNA  , ONLY : LRWSDLR
USE PAR_RDLR , ONLY : JP_RSAF  ,JP_RSAH  , &
 & JP_RSAFL ,JP_RSAFM ,JP_RSAHL ,JP_RSAHM ,JP_DIMR

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRNHPPI(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRTL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRTM(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPRL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPRM(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPHL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPHM(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQCHAL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQCHAM(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDLR(KPROMA,0:KFLEV,JP_DIMR)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHIFL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHIFM(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHIHL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHIHM(KPROMA,0:KFLEV) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZINL(KPROMA,KFLEV),ZINM(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZOUTL(KPROMA,0:KFLEV),ZOUTM(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZRDLR3FL(KPROMA,KFLEV),ZRDLR3HL(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZRDLR3FM(KPROMA,KFLEV),ZRDLR3HM(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZNH1L(KPROMA,KFLEV),ZNH1M(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZAG,ZRAG,ZFAC

INTEGER(KIND=JPIM) :: JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "verint.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHGRDLR',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*    1. Computation of term:
!        R[l] T[l] {(grad(presim)/presim)[l] - (grad(pre)/pre)[l]}

IF (.NOT.LRWSDLR) THEN
  CALL ABOR1(' GNHGRDLR: should not be called if LRWSDLR=F!')
ENDIF

ZAG=RG*RA
ZRAG=1.0_JPRB/ZAG

DO JLEV=1,KFLEV
  ZNH1L(KD:KF,JLEV)=-PR(KD:KF,JLEV)*PT(KD:KF,JLEV)*PQCHAL(KD:KF,JLEV)
  ZNH1M(KD:KF,JLEV)=-PR(KD:KF,JLEV)*PT(KD:KF,JLEV)*PQCHAM(KD:KF,JLEV)
ENDDO

!     ------------------------------------------------------------------

!*       2.    COMPUTES grad(r/a).

!*    2.1 Calculation of "grad(r/a)" at half levels.

! Surface
DO JROF=KD,KF
  ZFAC=1.0_JPRB/ &
   & (3.0_JPRB*PRDLR(JROF,KFLEV,JP_RSAH)*PRDLR(JROF,KFLEV,JP_RSAH))
  ZRDLR3HL(JROF,KFLEV)=3.0_JPRB*ZRAG &
   & *PRDLR(JROF,KFLEV,JP_RSAH)*PRDLR(JROF,KFLEV,JP_RSAH)*POROGL(JROF)
  PRDLR(JROF,KFLEV,JP_RSAHL)=ZRDLR3HL(JROF,KFLEV)*ZFAC
  ZRDLR3HM(JROF,KFLEV)=3.0_JPRB*ZRAG &
   & *PRDLR(JROF,KFLEV,JP_RSAH)*PRDLR(JROF,KFLEV,JP_RSAH)*POROGM(JROF)
  PRDLR(JROF,KFLEV,JP_RSAHM)=ZRDLR3HM(JROF,KFLEV)*ZFAC
ENDDO

DO JLEV=KFLEV,1,-1
  DO JROF=KD,KF
    ZFAC=1.0_JPRB/ &
     & (3.0_JPRB*PRDLR(JROF,JLEV-1,JP_RSAH)*PRDLR(JROF,JLEV-1,JP_RSAH))
    ZRDLR3HL(JROF,JLEV-1)= ZRDLR3HL(JROF,JLEV)+3.0_JPRB*ZRAG* &
     & ( PLNPR(JROF,JLEV)*PRNHPPI(JROF,JLEV)*&
     & (PRTL(JROF,JLEV)+ZNH1L(JROF,JLEV))&
     & +PLNPRL(JROF,JLEV)*PRNHPPI(JROF,JLEV)*PR(JROF,JLEV)*PT(JROF,JLEV) )
    PRDLR(JROF,JLEV-1,JP_RSAHL)=ZRDLR3HL(JROF,JLEV-1)*ZFAC
    ZRDLR3HM(JROF,JLEV-1)= ZRDLR3HM(JROF,JLEV)+3.0_JPRB*ZRAG* &
     & ( PLNPR(JROF,JLEV)*PRNHPPI(JROF,JLEV)*&
     & (PRTM(JROF,JLEV)+ZNH1M(JROF,JLEV))&
     & +PLNPRM(JROF,JLEV)*PRNHPPI(JROF,JLEV)*PR(JROF,JLEV)*PT(JROF,JLEV) )
    PRDLR(JROF,JLEV-1,JP_RSAHM)=ZRDLR3HM(JROF,JLEV-1)*ZFAC
  ENDDO
ENDDO

!*    2.2 Calculation of "grad(r/a)" at full levels.

IF(LVERTFE) THEN
  DO JLEV=1,KFLEV
    DO JROF=KD,KF
      ZINL(JROF,JLEV)=3.0_JPRB*ZRAG*(-PLNPR(JROF,JLEV)*PRNHPPI(JROF,JLEV)*&
       & (PRTL(JROF,JLEV)+ZNH1L(JROF,JLEV))&
       & -PLNPRL(JROF,JLEV)*PRNHPPI(JROF,JLEV)*PR(JROF,JLEV)*PT(JROF,JLEV))&
       & *VFE_RDETAH(JLEV)  
      ZINM(JROF,JLEV)=3.0_JPRB*ZRAG*(-PLNPR(JROF,JLEV)*PRNHPPI(JROF,JLEV)*&
       & (PRTM(JROF,JLEV)+ZNH1M(JROF,JLEV))&
       & -PLNPRM(JROF,JLEV)*PRNHPPI(JROF,JLEV)*PR(JROF,JLEV)*PT(JROF,JLEV))&
       & *VFE_RDETAH(JLEV)  
    ENDDO
  ENDDO
  CALL VERINT(KPROMA,KD,KF,KFLEV,ZINL,ZOUTL,1)
  CALL VERINT(KPROMA,KD,KF,KFLEV,ZINM,ZOUTM,1)
  DO JLEV=1,KFLEV
    DO JROF=KD,KF
      ZFAC=1.0_JPRB/ &
       & (3.0_JPRB*PRDLR(JROF,JLEV,JP_RSAF)*PRDLR(JROF,JLEV,JP_RSAF))
      ZRDLR3FL(JROF,JLEV)=ZOUTL(JROF,JLEV-1)+ZRDLR3HL(JROF,KFLEV)
      PRDLR(JROF,JLEV,JP_RSAFL)=ZRDLR3FL(JROF,JLEV)*ZFAC
      ZRDLR3FM(JROF,JLEV)=ZOUTM(JROF,JLEV-1)+ZRDLR3HM(JROF,KFLEV)
      PRDLR(JROF,JLEV,JP_RSAFM)=ZRDLR3FM(JROF,JLEV)*ZFAC
    ENDDO
  ENDDO  
ELSE
  DO JLEV=1,KFLEV
    DO JROF=KD,KF
      ZFAC=1.0_JPRB/ &
       & (3.0_JPRB*PRDLR(JROF,JLEV,JP_RSAF)*PRDLR(JROF,JLEV,JP_RSAF))
      ZRDLR3FL(JROF,JLEV)= ZRDLR3HL(JROF,JLEV)+3.0_JPRB*ZRAG* &
       & ( PALPH(JROF,JLEV)*PRNHPPI(JROF,JLEV)*&
       & (PRTL(JROF,JLEV)+ZNH1L(JROF,JLEV))&
       & +PALPHL(JROF,JLEV)*PRNHPPI(JROF,JLEV)*PR(JROF,JLEV)*PT(JROF,JLEV) )
      PRDLR(JROF,JLEV,JP_RSAFL)=ZRDLR3FL(JROF,JLEV)*ZFAC
      ZRDLR3FM(JROF,JLEV)= ZRDLR3HM(JROF,JLEV)+3.0_JPRB*ZRAG* &
       & ( PALPH(JROF,JLEV)*PRNHPPI(JROF,JLEV)*&
       & (PRTM(JROF,JLEV)+ZNH1M(JROF,JLEV))&
       & +PALPHM(JROF,JLEV)*PRNHPPI(JROF,JLEV)*PR(JROF,JLEV)*PT(JROF,JLEV) )
      PRDLR(JROF,JLEV,JP_RSAFM)=ZRDLR3FM(JROF,JLEV)*ZFAC
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*       3.    COMPUTES grad(Gr) (= grad(Phi) if LRWSDLG=F).

DO JROF=KD,KF
  PHIHL(JROF,KFLEV)=POROGL(JROF)
  PHIHM(JROF,KFLEV)=POROGM(JROF)
ENDDO

DO JLEV=1,KFLEV
  DO JROF=KD,KF
    PHIFL(JROF,JLEV)=ZAG*PRDLR(JROF,JLEV,JP_RSAFL)
    PHIFM(JROF,JLEV)=ZAG*PRDLR(JROF,JLEV,JP_RSAFM)
    PHIHL(JROF,JLEV-1)=ZAG*PRDLR(JROF,JLEV-1,JP_RSAHL)
    PHIHM(JROF,JLEV-1)=ZAG*PRDLR(JROF,JLEV-1,JP_RSAHM)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHGRDLR',1,ZHOOK_HANDLE)
END SUBROUTINE GNHGRDLR
