SUBROUTINE GNHGW2SVD( &
 ! ----- INPUT ---------------------------------------------------------------
 & KST,KEND,KVDVAR,PSP,PT,PSPD, &
 ! ----- OUTPUT --------------------------------------------------------------
 & PSVD, &
 ! ----- OPTIONAL INPUT ------------------------------------------------------
 & LDRWSDLR,PRDLR,PNHX,PLNPR,PDEP,PREF,PGWH,PGWF,PGWS,PGWRF,PGWRS,PGDWR &
 & )

!* GNHGW2SVD - Conversion Gw ---> svd.

! Purpose
! -------
!   Convert "G times vertical velocity w" to vertical divergence variable svd.

!   Remark for LDRWSDLR=T: (prehyd,prehyds) must be replaced by (presim,presims)
!    in comments below.

! Interface
! ---------
!  INPUT:
!   KST    - start of work
!   KEND   - end of work
!   KVDVAR - controls the prognostic variable for "svd" equation.
!   PSP    - ln(prehyds)
!   PT     - temperature at full levels
!   PSPD   - NH pressure departure variable at full levels

!  OUTPUT:
!   PSVD   - vertical divergence variable "svd" (d3 or d4) at full levels

!  OPTIONAL INPUT:
!   LDRWSDLR - key for metric terms in Wood and Staniforth DL eqns
!   PRDLR  - array containing "r/a" and "a/r".
!   PNHX   - X-term (used only for d4) at full levels
!   PLNPR  - delta = delta(log(prehyd)) at full levels
!   PDEP   - "pre - prehyd" at full levels
!   PREF   - "prehyd" at full levels
!   PGWH   - "Gw" at half levels 
!   PGWF   - "Gw" at full levels 
!   PGWS   - "Gw" at the surface 
!   PGWRF  - reference "Gw_ref" at full levels (to apply RDERI to Gw-Gw_ref).
!   PGWRS  - reference "Gw_surf_ref".
!   PGDWR  - reference "G dw".

! Externals
! ---------

! Method
! ------

! Reference
! ---------

! Author
! ------
!   11-Oct-2002 J. Masek (SHMI)

! Modifications
! -------------
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   03-Jun-2004 J. Masek   NH cleaning (NPDVAR, NVDVAR)
!   25-Jan-2005 K. Yessad  Cleanings, more modularisation.
!   09-Oct-2007 J. Vivoda and K. Yessad  VFE in NH model
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
! End Modifications
!------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCST   , ONLY : RD
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG
USE YOMDYNA  , ONLY : LNH_GEOGW,LRWSDLR2
USE YOMGEM   , ONLY : VAH      ,VBH      ,VC       ,VDELB    ,VFE_RDETAH
USE YOMCVER  , ONLY : LVERTFE  ,LVFE_GW  ,LVFE_DERIB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE PAR_RDLR , ONLY : JP_RSAH  ,JP_RSAF  ,JP_DIMR

! -----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND
INTEGER(KIND=JPIM),INTENT(IN)    :: KVDVAR
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP (NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT  (NPROMA,   NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPD(NPROMA,   NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSVD(NPROMA,   NFLEVG)
LOGICAL,OPTIONAL  ,INTENT(IN)    :: LDRWSDLR
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PRDLR(NPROMA,0:NFLEVG,JP_DIMR)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PNHX(NPROMA,NFLEVG)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PLNPR(NPROMA,NFLEVG)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PDEP(NPROMA,NFLEVG)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PREF(NPROMA,NFLEVG)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PGWH(NPROMA,0:NFLEVG)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PGWF(NPROMA,NFLEVG)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PGWS(NPROMA)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PGWRF(NPROMA,NFLEVG)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PGWRS(NPROMA)
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PGDWR(NPROMA,NFLEVG)

! -----------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF
LOGICAL :: LLINC
LOGICAL :: LLRWSDLR

REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB) :: ZDELP (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZRDELP(NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZLNPR (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZALPH (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZRTGR (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZRPRE (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZRPP  (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZPREF (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZNHPREF(NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZNHPPI(NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZRNHPPI(NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZPDEP (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZPREH (NPROMA, 0:NFLEVG)
REAL(KIND=JPRB) :: ZGWH  (NPROMA, 0:NFLEVG)
REAL(KIND=JPRB) :: ZGWF  (NPROMA, 0:NFLEVG+1)
REAL(KIND=JPRB) :: ZGDW  (NPROMA, 0:NFLEVG+1)

! -----------------------------------------------------------------------------

#include "abor1.intfb.h"
#include "gnhpre.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gpxyb.intfb.h"
#include "verder.intfb.h"

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHGW2SVD',0,ZHOOK_HANDLE)

! -----------------------------------------------------------------------------

! -----
! preliminary tests 
! -----

IF (PRESENT(LDRWSDLR)) THEN
  LLRWSDLR=LDRWSDLR
ELSE
  LLRWSDLR=.FALSE.
ENDIF
IF (LLRWSDLR.AND.(.NOT.PRESENT(PRDLR))) THEN
  CALL ABOR1('GNHGW2SVD: if LRWSDLR=T, PRDLR should be present')
ENDIF

IF (LRWSDLR2.AND.LVFE_GW) THEN
  CALL ABOR1('GNHGW2SVD: LRWSDLR2=T not yet coded for LVFE_GW')
ENDIF

! -----
! compute pressures
! -----

IF (PRESENT(PLNPR).AND.PRESENT(PDEP).AND.PRESENT(PREF)) THEN

  ! * Simply transfer dummy input arrays into local arrays:
  ZLNPR(KST:KEND,1:NFLEVG)=PLNPR(KST:KEND,1:NFLEVG)
  ZPDEP(KST:KEND,1:NFLEVG)=PDEP(KST:KEND,1:NFLEVG)
  ZPREF(KST:KEND,1:NFLEVG)=PREF(KST:KEND,1:NFLEVG)

ELSE

  ! * Do calculations for ZLNPR, ZPDEP, ZPREF:

  IF (LNH_GEOGW) THEN
    CALL ABOR1('GNHGW2SVD: Arg PLNPR,PDEP,PREF for Phi-gw NH model')
  ENDIF

  ! convert ln(prehyds) to prehyds
  DO JROF=KST,KEND
    ZPREH(JROF,NFLEVG)=EXP(PSP(JROF))
  ENDDO

  ! compute half and full level pressures "pre"
  CALL GPPREH(NPROMA,KST,KEND,NFLEVG,VAH,VBH,ZPREH)  
  CALL GPXYB(NPROMA,KST,KEND,NFLEVG,VDELB,VC,ZPREH,ZDELP,ZRDELP,ZLNPR, &
   & ZALPH,ZRTGR,ZRPRE,ZRPP)  
  CALL GPPREF(NPROMA,KST,KEND,NFLEVG,ZALPH,ZPREH,ZPREF)  

  ! transform NH pressure departure to (pre-prehyd)
  CALL GNHPRE(NPROMA,NFLEVG,KST,KEND,PSPD,ZPREF,ZNHPREF,ZNHPPI,ZRNHPPI,ZPDEP)

ENDIF

! -----
! transform Gw ---> svd
! -----

! transform Gw into -G.dw (-G.d((r**2/a**2)w) if LRWSDLR2=T)
IF( LVERTFE.AND.LVFE_GW )THEN
  IF (.NOT.PRESENT(PGWF)) CALL ABOR1('GNHGW2SVD: missing input PGWF')
  IF (PRESENT(PGWRF).AND.PRESENT(PGDWR)) THEN
    LLINC=.TRUE.
  ELSE
    ! in this case it is better to omit PGWRF, PGWRS and PGDWR.
    ! equivalent to have PGWRF=PGWRS=PGDWR=0.
    LLINC=.FALSE.
  ENDIF

  IF (LLINC) THEN
    ! * Compute derivative on increments -((Gw)-(Gw_ref)):
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZGWF(JROF,JLEV)=-(PGWF(JROF,JLEV)-PGWRF(JROF,JLEV))
      ENDDO
    ENDDO
    IF (LVFE_DERIB) THEN
      IF (.NOT.PRESENT(PGWS)) CALL ABOR1('GNHGW2SVD: missing input PGWS')
      IF (.NOT.PRESENT(PGWRS)) CALL ABOR1('GNHGW2SVD: missing input PGWRS')
      ! Top condition: -(Gw(top)-Gw_ref(top))=-(Gw(jlev=1)-Gw_ref(jlev=1))
      ZGWF(KST:KEND,0)=ZGWF(KST:KEND,1)
      ! Bottom condition: -(Gw(surf)-Gw_ref(surf))=-(PGWS-PGWRS)
      ZGWF(KST:KEND,NFLEVG+1)=-(PGWS(KST:KEND)-PGWRS(KST:KEND))
      CALL VERDER(NPROMA,KST,KEND,NFLEVG+2,ZGWF(1,0),ZGDW(1,0),.TRUE.)
    ELSE
      CALL VERDER(NPROMA,KST,KEND,NFLEVG,ZGWF(1,1),ZGDW(1,1))
    ENDIF
    ! compute -G(dw) from -G(dw/deta - dw_ref/deta) and -G(dw_ref).
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZGDW(JROF,JLEV)=ZGDW(JROF,JLEV)/VFE_RDETAH(JLEV)-PGDWR(JROF,JLEV)
      ENDDO
    ENDDO
  ELSE
    ! * Compute derivative on -(Gw):
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZGWF(JROF,JLEV)=-PGWF(JROF,JLEV)
      ENDDO
    ENDDO
    IF (LVFE_DERIB) THEN
      IF (.NOT.PRESENT(PGWS)) CALL ABOR1('GNHGW2SVD: missing input PGWS')
      ! Top condition: -Gw(top)=-Gw(jlev=1)
      ZGWF(KST:KEND,0)=ZGWF(KST:KEND,1)
      ! Bottom condition: -Gw(surf)=-PGWS
      ZGWF(KST:KEND,NFLEVG+1)=-PGWS(KST:KEND)
      CALL VERDER(NPROMA,KST,KEND,NFLEVG+2,ZGWF(1,0),ZGDW(1,0),.TRUE.)
    ELSE
      CALL VERDER(NPROMA,KST,KEND,NFLEVG,ZGWF(1,1),ZGDW(1,1))
    ENDIF
    DO JLEV=1,NFLEVG
      DO JROF=KST,KEND
        ZGDW(JROF,JLEV)=ZGDW(JROF,JLEV)/VFE_RDETAH(JLEV)
      ENDDO
    ENDDO
  ENDIF
ELSE
  IF (.NOT.PRESENT(PGWH)) CALL ABOR1('GNHGW2SVD: missing input PGWH')
  ! Gw -> G.(r**2/a**2)w if relevant.
  ZGWH(KST:KEND,0:NFLEVG)=PGWH(KST:KEND,0:NFLEVG)
  IF (LLRWSDLR.AND.LRWSDLR2) THEN
    DO JLEV=0,NFLEVG
      DO JROF=KST,KEND
        ZGWH(JROF,JLEV)=ZGWH(JROF,JLEV)* &
         & PRDLR(JROF,JLEV,JP_RSAH)*PRDLR(JROF,JLEV,JP_RSAH)
      ENDDO
    ENDDO
  ENDIF
  ! Gw (G.(r**2/a**2)w if LRWSDLR2=T) 
  !  -> "-G.dw" (-G.d((r**2/a**2)w) if LRWSDLR2=T)
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      ZGDW(JROF,JLEV)=ZGWH(JROF,JLEV-1)-ZGWH(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

! transform "-G.dw" (-G.d((r**2/a**2)w) if LRWSDLR2=T) into "dver"
DO JLEV=1,NFLEVG
  DO JROF=KST,KEND
    PSVD(JROF,JLEV)=ZGDW(JROF,JLEV)/ &
     & (RD*PT(JROF,JLEV)*ZLNPR(JROF,JLEV) )* &
     & (ZPDEP(JROF,JLEV)/ZPREF(JROF,JLEV) + 1.0_JPRB )
  ENDDO
ENDDO
IF (LLRWSDLR.AND. .NOT.LRWSDLR2) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      PSVD(JROF,JLEV)=PSVD(JROF,JLEV)* &
       & PRDLR(JROF,JLEV,JP_RSAF)*PRDLR(JROF,JLEV,JP_RSAF)
    ENDDO
  ENDDO
ENDIF

! transform "dver" into "svd" (currently for "d4" only)

IF ( KVDVAR == 4 ) THEN
  IF (.NOT.PRESENT(PNHX)) CALL ABOR1('GNHGW2SVD: missing input PNHX')
  PSVD(KST:KEND,1:NFLEVG)=PSVD(KST:KEND,1:NFLEVG)+PNHX(KST:KEND,1:NFLEVG)
ENDIF

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHGW2SVD',1,ZHOOK_HANDLE)

END SUBROUTINE GNHGW2SVD

