SUBROUTINE GNHSIM( &
 ! --- INPUT -----------------------------------------------------------------
 & KSTART,KPROF, &
 & POROGL,POROGM, &
 & PUT1,PVT1,PTT1, &
 & PLHU,PLHV,PLNPR, &
 & PE3Y,PERR,PEGHL,PEGHM, &
 ! --- OUTPUT ----------------------------------------------------------------
 & PGSPD1)   

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCT0   , ONLY : LSLAG    ,LTWOTL      
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMDYN   , ONLY : TDT      ,BETADT   ,VESL     ,XIDT
USE YOMDYNA  , ONLY : NVDVAR
USE YOMCVER  , ONLY : LVERTFE
USE YOMLUN   , ONLY : NULOUT
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

!**** *GNHSIM* - Prepare the "Y" non linear term of D3 in elastic dyn.

!     Purpose.
!     --------
!           The non-linear part of the total divergence D3 is
!           calculated and ready in the "Y" table for the next
!           iteration of the S.-I. solver.

!**   Interface.
!     ----------
!        *CALL* *GNHSIM(...)*

!        Explicit arguments :
!        --------------------
!        Input :

!        KSTART    : start of work in message passing version
!        KPROF     : end   of work in message passing version
!        POROGL    : x- component of orography gradient
!        POROGM    : y- component of orography gradient
!        PUT1      : x- component of wind "guess"
!        PVT1      : y- component of wind "guess"
!        PTT1      : temperature "guess"
!        PLHU      : u-half-level wind "guess"
!        PLHV      : v-half-level wind "guess"
!        PLNPR     : log press thickness "guess"
!        PE3Y      : stranger: "Y" (combined 2*Y(0) - Y(9))
!        PERR      : stranger: "reduced" gas constant level 0
!        PEGHL     : stranger:  half-level x- geopotential gradient level 0
!        PEGHM     : stranger:  half-level y- geopotential gradient level 0

!        Output:

!        PGSPD1    : result combination: Y(1) + Y(9) - 2* Y(0)
!                    multiplied by 0.5 * BETA * delta(t) decentralized
!                    if semi-Lagrangian scheme.
!                    pointer MSLB2PD1

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ALADIN documentation on non hydrostatic

!     Author.
!     -------
!        Radmila Bubnova    *CNRM/GMAP - stage *

! Modifications
! -------------
!   Original : 95-02-23
!   Modified : 95-06-09  F90-LD
!   Modified : 96-04-25  by S. Issara (introducing lagged part GNHSIMLAG)
!   Modified : 97-05-27  by R. Bubnova: phasing to AL06
!   Modified : 98-05-22  by D. Cemas  (Lower BC)
!   Modified : 99-02-01  M. Charron : non-hydrostatic
!   Modified : 99-04-08  by R. El Khatib GPGEOTOP
!   Modified : 00-10-02  C. Fischer : move call inside cpg
!   Modified : 01-03-30  P.Smolikova : term Y comp. for diff. variables 
!                        for pseudo vert. divergence (according to NVDVAR)
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   03-Jun-2004 J. Masek   NH cleaning (NPDVAR, NVDVAR)
!   20-Jan-2005 K. Yessad  Cleanings, use routine GNHXX to compute X.
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
! End Modifications
! ----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTT1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLHU(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLHV(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PE3Y(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PERR(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEGHL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEGHM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGSPD1(NPROMA,NFLEVG) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZGPHL(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZGPHM(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZGPFL(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZGPFM(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZYY(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZUNITE(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZNHX1(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZBDT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "gpxx.intfb.h"
#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHSIM',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF( LVERTFE )THEN
  WRITE(NULOUT,*) "(E) message from GNHSIM"
  WRITE(NULOUT,*) "(E) LVERTFE not coded in GNHSIM"
  WRITE(NULOUT,*) "(E) grad(phi) on full levels not available"
  CALL ABOR1("(E) ABORT IN GNHSIM: LVERTFE not yet coded")
ENDIF

!     ------------------------------------------------------------------

!*       1.    ZBDT:
!              -----

ZBDT=0.5_JPRB*TDT*BETADT
IF(LSLAG) THEN
  IF(LTWOTL) THEN
    ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT*(1.0_JPRB+XIDT)
  ELSE
    ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*          2.    Save grad(gz) at half levels in (ZGPHL,ZGPHM)
!                 ---------------------------------------------

DO JROF=KSTART,KPROF
  ZGPHL(JROF,NFLEVG)=POROGL(JROF)
  ZGPHM(JROF,NFLEVG)=POROGM(JROF)
ENDDO

DO JLEV=1,NFLEVG
  DO JROF=KSTART,KPROF 
    ZGPHL(JROF,JLEV-1)=PEGHL(JROF,JLEV)
    ZGPHM(JROF,JLEV-1)=PEGHM(JROF,JLEV)
  ENDDO
ENDDO

IF( LVERTFE )THEN
  ZGPFL = 0.0_JPRB
  ZGPFM = 0.0_JPRB
ENDIF

!     ------------------------------------------------------------------

!*          3.    X- term and Y- term
!                 --------------------

! * Computation of X at time 1:

ZUNITE(KSTART:KPROF,1:NFLEVG)=1.0_JPRB
CALL GPXX( &
 ! --- INPUT -------------------------------------------------------------
 & NFLEVG,NPROMA,KSTART,KPROF,ZGPHL,ZGPHM,ZGPFL,ZGPFM, &
 & PLNPR,ZUNITE,PTT1,PERR, &
 & PUT1,PVT1,PLHU,PLHV, &
 ! --- OUTPUT ------------------------------------------------------------
 & ZNHX1 &
 & )

! * Computation of the X part of Y at time 1:
IF (NVDVAR==3) THEN
  ZYY(KSTART:KPROF,1:NFLEVG)=ZNHX1(KSTART:KPROF,1:NFLEVG)
ELSEIF (NVDVAR==4) THEN
  ZYY(KSTART:KPROF,1:NFLEVG)=0.0_JPRB
ENDIF

! * Semi - implicit combination   Y(1) + Y(9) - 2* Y(0)
!   multiplied by 0.5 * beta * delta(t), decentralized
!   if semi-Lagrangian scheme
DO JLEV=1,NFLEVG
  IF(LSLAG) THEN
    DO JROF=KSTART,KPROF
      ZYY(JROF,JLEV)=ZYY(JROF,JLEV)*ZBDT-PE3Y(JROF,JLEV)
    ENDDO
  ELSE
    DO JROF=KSTART,KPROF
      ZYY(JROF,JLEV)=(ZYY(JROF,JLEV)-PE3Y(JROF,JLEV))*ZBDT
    ENDDO
  ENDIF
ENDDO

! * Copy ZYY in PGSPD1:
PGSPD1(KSTART:KPROF,1:NFLEVG)=ZYY(KSTART:KPROF,1:NFLEVG)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHSIM',1,ZHOOK_HANDLE)
END SUBROUTINE GNHSIM
