SUBROUTINE GNHSVD2GW( &
 ! ----- INPUT ---------------------------------------------------------------
 & KST,KEND,KVDVAR,PSP,PT,PSPD,POROGL,POROGM,PUS,PVS,PNHX,PSVD, &
 ! ----- OUTPUT --------------------------------------------------------------
 & PGW &
 & )

! GNHSVD2GW - Conversion svd ---> gw

! Purpose
! -------
!   Convert vertical divergence variable svd to "g times vertical velocity w".

! Interface
! ---------
!   KST    - start of work
!   KEND   - end of work
!   KVDVAR - controls the prognostic variable for "svd" equation.
!   PSP    - ln(prehyds)
!   PT     - temperature at full levels
!   PSPD   - NH pressure departure variable at full levels
!   POROGL - zonal comp of grad[Phi_surf]
!   POROGM - merid comp of grad[Phi_surf]
!   PUS    - surface U wind
!   PVS    - surface V wind
!   PNHX   - X-term (used only for d4) at full levels
!   PSVD   - vertical divergence variable svd (d3 or d4) at full levels
!   PGW    - g times vertical velocity w

! Externals
! ---------

! Method
! ------

! Reference
! ---------

! Author
! ------
!   J. Masek (SHMI)

! Modifications
! -------------
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   03-Jun-2004 J. Masek   NH cleaning (NPDVAR, NVDVAR)
!   25-Jan-2005 K. Yessad  Cleanings, more modularisation.
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG
USE YOMDYNA  , ONLY : LNH_GEOGW,LRWSDLR
USE YOMGEM   , ONLY : VAH      ,VBH      ,VC       ,VDELB
USE YOMCVER  , ONLY : LVERTFE  ,LVFE_GW
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

! -----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVDVAR
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP   (NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT    (NPROMA,   NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPD  (NPROMA,   NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUS   (NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVS   (NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNHX  (NPROMA,   NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSVD  (NPROMA,   NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGW   (NPROMA, 0:NFLEVG) 

! -----------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF

REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB) :: ZDELP (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZRDELP(NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZLNPR (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZALPH (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZRTGR (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZRPRE (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZRPP  (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZNHPREF(NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZNHPPI(NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZRNHPPI(NPROMA,  NFLEVG)
REAL(KIND=JPRB) :: ZPREF (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZPDEP (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZGWF  (NPROMA,   NFLEVG)
REAL(KIND=JPRB) :: ZGWH  (NPROMA, 0:NFLEVG)
REAL(KIND=JPRB) :: ZPREH (NPROMA, 0:NFLEVG)
REAL(KIND=JPRB) :: ZDVER (NPROMA,   NFLEVG) 

! -----------------------------------------------------------------------------

#include "abor1.intfb.h"
#include "gnhpre.intfb.h"
#include "gpgw.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gpxyb.intfb.h"

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHSVD2GW',0,ZHOOK_HANDLE)

! -----------------------------------------------------------------------------

IF (LNH_GEOGW) THEN
  CALL ABOR1('GNHSVD2GW: should not be called for Phi-gw NH model')
ENDIF
IF (LRWSDLR) THEN
  CALL ABOR1('GNHSVD2GW: LRWSDLR not yet coded')
ENDIF

! convert ln(prehyds) to prehyds
DO JROF = KST, KEND
  ZPREH(JROF, NFLEVG) = EXP(PSP(JROF))
ENDDO

! compute half and full level pressures "pre"
CALL GPPREH(NPROMA,KST,KEND,NFLEVG,VAH,VBH,ZPREH)
CALL GPXYB(NPROMA,KST,KEND,NFLEVG,VDELB,VC,ZPREH,ZDELP,ZRDELP,ZLNPR, &
 & ZALPH,ZRTGR,ZRPRE,ZRPP)
CALL GPPREF(NPROMA,KST,KEND,NFLEVG,ZALPH,ZPREH,ZPREF)

! transform NH pressure departure to (pre-prehyd)
CALL GNHPRE(NPROMA,NFLEVG,KST,KEND,PSPD,ZPREF,ZNHPREF,ZNHPPI,ZRNHPPI,ZPDEP)

! transform "svd" into "dver"
IF (KVDVAR == 3) THEN
  ZDVER(KST:KEND,1:NFLEVG)=PSVD(KST:KEND,1:NFLEVG)
ELSEIF (KVDVAR == 4) THEN
  ZDVER(KST:KEND,1:NFLEVG)=PSVD(KST:KEND,1:NFLEVG)-PNHX(KST:KEND,1:NFLEVG)
ENDIF

! transform "dver" into "gw"
CALL GPGW( &
 & NFLEVG,NPROMA,KST,KEND,.FALSE.,.FALSE., &
 & POROGL,POROGM,ZLNPR,ZALPH,PUS,PVS, &
 & PT,ZDVER,ZRNHPPI, &
 & ZGWH,ZGWF &
 & )

! save full level or half level "gw" according to "lvertfe"
IF (LVERTFE.AND.LVFE_GW) THEN
  PGW(KST:KEND,1:NFLEVG)=ZGWF(KST:KEND,1:NFLEVG)
ELSE
  PGW(KST:KEND,0:NFLEVG)=ZGWH(KST:KEND,0:NFLEVG)
ENDIF

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GNHSVD2GW',1,ZHOOK_HANDLE)

END SUBROUTINE GNHSVD2GW

