SUBROUTINE GP_KAPPAAD( &
 & KPROMA,KSTART,KPROF,KFLEV,&
 & PDTS2,&
 & PUT0L5,PVT0L5,PVORT05,PDIVT05,&
 & PUT0L,PVT0L,PVORT0,PDIVT0,&
 & PSLHDA,PSLHDD0,&
 & PKAPPA)

!**** *GP_KAPPAAD*         (adjoint model)
!                Computation of the kappa function ("coefficient
!                of diffusion") based on rescaled horizontal
!                deformation of the flow.
!                Used by SLHD scheme.
!                NOTE: In case of LSLHD_STATIC this routine is
!                      not called.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *GP_KAPPAAD(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA      - horizontal dimension.
!          KSTART      - first element of work.
!          KPROF       - depth of work.
!          KFLEV       - number of layers.
!          PDTS2       - 0.5*"pdt", where "pdt" =
!                        time step for the first time-integration step of
!                        a leap-frog scheme or all time-integration steps of
!                        a two-time level scheme; 2*time step for the following
!                        time-integration steps of a leap-frog scheme.
!   --------------------- trajectory variables -----------------------
!          PUT0L5      - zonal derivative of U-wind at time t.
!          PVT0L5      - zonal derivative of V-wind at time t.
!          PVORT05     - vorticity at time t.
!          PDIVT05     - divergence at t.
!   ------------------------ end of traj. variables -------------------
!          PSLHDA      - Scaling factor of the deformation in f(d) function
!                        (including the model resolution correction)
!          PSLHDD0     - Treshold for deformation tensor enhancement
!          PKAPPA     - Kappa function ("coefficient of SLHD")
!                       based on the rescaled horizontal deformation
!                       of the flow evaluated at instant "t".

!        INPUT/OUTPUT:
!          PUT0L       - zonal derivative of U-wind at time t.
!          PVT0L       - zonal derivative of V-wind at time t.
!          PVORT0      - vorticity at time t.
!          PDIVT0      - divergence at t.


!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by LATTE_KAPPAAD.

!     Reference.
!     ----------
!             Arpege documentation about semi-Lagrangian scheme.

!  Author.
!  -------
!    Original F. VANA : JANUARY 2009.

!  Modifications.
!  --------------

!     ------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

USE YOMDYN   , ONLY : SLHDB    ,SLHDDIV  ,SLHDRATDDIV

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT0L5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT0L5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVORT05(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIVT05(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT0L(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT0L(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVORT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLHDA(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLHDD0(KPROMA)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PKAPPA(KPROMA,KFLEV) 

!     -------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV,JROF
REAL(KIND=JPRB) :: ZABSDTS2
REAL(KIND=JPRB) :: ZKAP5, ZMAX, ZMAXE
REAL(KIND=JPRB) :: ZDT, ZDS, ZDT5, ZDS5
REAL(KIND=JPRB) :: ZKAPPA51(KPROMA,KFLEV), ZKAPPA52(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GP_KAPPAAD',0,ZHOOK_HANDLE)

!    --------------------------------------------------------


!*       1.   Kappa function computation

ZABSDTS2=ABS(PDTS2)

DO JLEV=1,KFLEV
  DO JROF=KSTART,KPROF

    ! trajectory
    ZDT5= 2._JPRB*PUT0L5(JROF,JLEV) - PDIVT05(JROF,JLEV)
    ZDS5= 2._JPRB*PVT0L5(JROF,JLEV) - PVORT05(JROF,JLEV)

    ZKAP5 = SQRT(ZDT5*ZDT5+ZDS5*ZDS5)
    ZKAPPA51(JROF,JLEV)=  (1.0_JPRB-SLHDDIV)* ZKAP5 &
     & + SLHDDIV*SLHDRATDDIV*ABS(PDIVT05(JROF,JLEV))

    ZMAX=MAX(1.0_JPRB,ZKAPPA51(JROF,JLEV)/PSLHDD0(JROF))
    ZMAXE=ZMAX**SLHDB
    ZKAPPA52(JROF,JLEV)=2.0_JPRB*ZABSDTS2*PSLHDA(JROF) &
     & *ZKAPPA51(JROF,JLEV)*ZMAXE

    !adjoint
    PKAPPA(JROF,JLEV)= PKAPPA(JROF,JLEV) &
     & /(ZKAPPA52(JROF,JLEV)+1.0_JPRB)**2._JPRB

    PKAPPA(JROF,JLEV)=2.0_JPRB*ZABSDTS2*PSLHDA(JROF)* ( &
     & PKAPPA(JROF,JLEV)*ZMAXE + ZKAPPA51(JROF,JLEV)*SLHDB*(ZMAXE/ZMAX) &
     & *(0.5_JPRB-SIGN(0.5_JPRB,1._JPRB-ZKAPPA51(JROF,JLEV)/PSLHDD0(JROF))) &
     & * PKAPPA(JROF,JLEV)/PSLHDD0(JROF)              )

    ZDT=(1.0_JPRB-SLHDDIV)*ZDT5*PKAPPA(JROF,JLEV)/ZKAP5
    ZDS=(1.0_JPRB-SLHDDIV)*ZDS5*PKAPPA(JROF,JLEV)/ZKAP5
    PDIVT0(JROF,JLEV)=PDIVT0(JROF,JLEV) + SLHDDIV*SLHDRATDDIV &
     & *SIGN(1.0_JPRB,PDIVT05(JROF,JLEV))*PKAPPA(JROF,JLEV)

    PUT0L(JROF,JLEV) =PUT0L(JROF,JLEV) + 2._JPRB*ZDT
    PDIVT0(JROF,JLEV)=PDIVT0(JROF,JLEV)-ZDT
    PVT0L(JROF,JLEV) =PVT0L(JROF,JLEV) + 2._JPRB*ZDS
    PVORT0(JROF,JLEV)=PVORT0(JROF,JLEV)-ZDS

  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GP_KAPPAAD',1,ZHOOK_HANDLE)
END SUBROUTINE GP_KAPPAAD
