!OCL  NOEVAL
SUBROUTINE GPCTY(LDVERCOR,KPROMA,KPROMVC,KD,KF,KFLEV,PVDELB,PVBH,&
 & PU,PV,PD,PEVT,LDRUBC,&
 & PDELP,PRDELP,PLNPR,PALPH,PRTGR,PSPL,PSPM,&
 & PVCRSSAF,PVCRSSAH,PVCASRSF,PVCASRSH,PVCRSSAFL,PVCRSSAFM,&
 & PEVEL,PVVEL,PSDIV,PSDVBC,PDIVDP,PRPREF,&
 & LDRWSDLR,PRDLR)  

!**** *GPCTY* - Computes vertical velocities.

!     Purpose.
!     --------

!      Computes the vertical velocities
!       "etadot (d prehyd / d eta)" on interlayers
!       "(omega / prehyd)" on layers

!      ----- The following discretisations are valid for lvertfe=.false. -----

!      Omitting the "delta m=1" flux precipitation terms, which will be later
!      added in routine "cpmvvps", the discretised expression of
!      "etadot (d prehyd / d eta)" on the interlayer number "lbar" is:

!       etadot (d prehyd / d eta) [lbar] =
!        B[lbar] * ( a**2/rssurf**2 ) * { sum[k=1 to L]
!        (Delta B)[k] * (rs[k]/a) * vec(V)[k] * (rssavnabla prehyds) }
!        + B[lbar] * ( a**2/rssurf**2 ) * { sum[k=1 to L]
!        (rs[k]/a) * (Delta prehyd)[k] * (rssavnabla vec(V)[k]) }
!        + B[lbar] * ( a**2/rssurf**2 ) * { sum[k=1 to L]
!        vec(V)[k] * (Delta prehyd)[k] * (rssavnabla (rs/a))[k] }
!        - ( a**2/rs**2 )[lbar] * { sum[k=1 to l]
!        (Delta B)[k] * (rs[k]/a) * vec(V)[k] * (rssavnabla prehyds) }
!        - ( a**2/rs**2 )[lbar] * { sum[k=1 to l]
!        (rs[k]/a) * (Delta prehyd)[k] * (rssavnabla vec(V)[k]) }
!        - ( a**2/rs**2 )[lbar] * { sum[k=1 to l]
!        vec(V)[k] * (Delta prehyd)[k] * (rssavnabla (rs/a))[k] }
!        + { ( a**2/rs**2 )[lbar] - B[lbar] ( a**2/rssurf**2 ) } *
!        ( rstop**2/a**2 ) (etadot (d prehyd / d eta))[top]

!      where:
!       - "rssurf" is the surface value of "rs".
!       - "rstop" is the value of "rs" at the top of the model.
!       - "vec(V)" is the horizontal wind.
!       - "prehyds" is the surface hydrostatic pressure.
!       - "rssavnabla" is the semi-reduced "horizontal gradient" operator:
!         rssavnabla X = (rs/a) vnabla X = M vnabla' X

!      Omitting the "delta m=1" flux precipitation terms, which will be later
!      added in routine "cpmvvps", the discretised expression of
!      "(omega / prehyd)" on the layer number "l" is:

!       ( rs**2/a**2 )[l] (omega / prehyd)[l] =
!        (rs/a)[l] vec(V)[l] (rssavnabla prehyd / prehyd)[l]
!        - delta[l]/(Delta prehyd)[l] * { sum[k=1 to l-1]
!        (Delta B)[k] * (rs[k]/a) * vec(V)[k] * (rssavnabla prehyds) }
!        - delta[l]/(Delta prehyd)[l] * { sum[k=1 to l-1]
!        (rs[k]/a) * (Delta prehyd)[k] * (rssavnabla vec(V)[k]) }
!        - delta[l]/(Delta prehyd)[l] * { sum[k=1 to l-1]
!        vec(V)[k] * (Delta prehyd)[k] * (rssavnabla (rs/a))[k] }
!        - alpha[l]/(Delta prehyd)[l] * (Delta B)[l] * (rs[l]/a) * vec(V)[l]
!          * (rssavnabla prehyds)
!        - alpha[l] * (rs[l]/a) * (rssavnabla vec(V)[l])
!        - alpha[l] * vec(V)[l] * (rssavnabla (rs/a))[l]
!        + delta[l]/(Delta prehyd)[l]
!          * ( rstop**2/a**2 ) * (etadot (d prehyd / d eta))[top]

!      This routine stores additional quantities:

!      * vertical integral of divergence without the "lrubc" contribution:
!        for the interlayer number "lbar" its discretised expression is:

!        psdiv[lbar] = sum[k=1 to l]
!        { (Delta B)[k] * (rs[k]/a) * vec(V)[k] * (rssavnabla prehyds)
!        + (rs[k]/a) * (Delta prehyd)[k] * (rssavnabla vec(V)[k])
!        + vec(V)[k] * (Delta prehyd)[k] * (rssavnabla (rs/a))[k] }

!      * vertical integral of divergence with the "lrubc" contribution:
!        for the interlayer number "lbar" its discretised expression is:

!        psdvbc[lbar] = sum[k=1 to l]
!        { (Delta B)[k] * (rs[k]/a) * vec(V)[k] * (rssavnabla prehyds)
!        + (rs[k]/a) * (Delta prehyd)[k] * (rssavnabla vec(V)[k])
!        + vec(V)[k] * (Delta prehyd)[k] * (rssavnabla (rs/a))[k] }
!        - ( rstop**2/a**2 ) (etadot (d prehyd / d eta))[top]

!      * divergence term
!        "pdivdp=rssavnabla( (rs/a) * vec(V) * (Delta prehyd) )"
!        on layers: for the layer number "l" its discretised expression is: 

!        pdivdp[l] = 
!        (Delta B)[l] * (rs[l]/a) * vec(V)[l] * (rssavnabla prehyds)
!        + (rs[l]/a) * (Delta prehyd)[l] * (rssavnabla vec(V)[l])
!        + vec(V)[l] * (Delta prehyd)[l] * (rssavnabla (rs/a))[l]

!      -----------------------------------------------------------------------

!**   Interface.
!     ----------
!        *CALL* *GPCTY(...)

!        Explicit arguments :
!        --------------------
!        * INPUT:
!          LDVERCOR     : .T./.F.: thin layer hypothesis relaxed/applied
!          KPROMA       : horizontal dimensioning
!          KPROMVC      : hor. dimension for arrays used only if "lvercor=T".
!          KD           : start of work
!          KF           : depth of work
!          KFLEV        : number of layers
!          PVDELB       : delta B's on layers
!          PVBH         : B's on interlayers
!          PU           : U component of the wind, on layers
!          PV           : V component of the wind, on layers
!          PD           : horizontal divergence of the wind, on layers
!                         (semi-reduced divergence (M**2 D')= rssavnabla vec(V))
!          PEVT         : "etadot (d prehyd / d eta)" at the top
!          LDRUBC       : upper boundary condition switch
!          PDELP        : pressure difference across layers
!                         ( [delta prehyd](l) = prehyd(lbar) - prehyd(lbar-1) )
!          PRDELP       : inverse of PDELP
!          PLNPR        : term "delta" on layers
!                         (= logarithm of ratio of pressure if "ndlnpr=0")
!          PALPH        : term "alpha" on layers
!          PRTGR        : pressure gradient term:
!                         scalar ratio of vectors "(rssavnabla prehyd)/prehyd"
!                         and "rssavnabla prehyds" (computed in GPXYB)
!          PSPL         : surface hydrostatic pressure zonal derivative
!                         (zonal component of "rssavnabla prehyds")
!          PSPM         : surface hydrostatic pressure meridian derivative
!                         (meridian component of "rssavnabla prehyds")
!          PVCRSSAF     : "rs/a" on layers.
!          PVCRSSAH     : "rs/a" on interlayers 0 to kflev.
!          PVCASRSF     : "a/rs" on layers.
!          PVCASRSH     : "a/rs" on interlayers 0 to kflev.
!          PVCRSSAFL    : zonal component of "rssavnabla (rs/a)" on layers.
!          PVCRSSAFM    : meridian component of "rssavnabla (rs/a)" on layers.
!          PRPREF       : inverse of full level pressures.

!        * OUTPUT:
!          PEVEL        : "etadot (d prehyd / d eta)" on interlayers
!          PVVEL        : "(omega / prehyd)" on layers
!          PSDIV        : vertical integral of divergence
!                         without the "lrubc" contribution
!          PSDVBC       : vertical integral of divergence
!                         with the "lrubc" contribution
!          PDIVDP       : at full levels:
!                         thin layer eqns: "grad(vec(V) * (Delta prehyd))"
!                         lvercor=T: "rssavnabla((rs/a)*vec(V)*(Delta prehyd))"
!                         lrwsdlr=T: "(a/r) grad((a/r)*vec(V)*(Delta presim))"
!                          "grad" is the semi-reduced gradient ((r/a)nabla).

!        * OPTIONAL INPUT:
!          LDRWSDLR     : key for metric terms in Wood and Staniforth DL eqns
!          PRDLR        : array containing "r/a", "a/r", "grad(r/a)".

!        Implicit arguments :   None.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.    None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 88-02-04
!        Modification : 89-11 (Yves Bouteloup *EERM/CRMD*)
!                       Variable mesh
!        Modification by K. YESSAD (NOV 1999):
!         - additional dummy argument 'PSDVBC'.
!         - cleaning in comments and declarations.
!        Modification by K. YESSAD (AUGUST 2000):
!         - relaxation of thin layer hypothesis.
!         - correction of some false or ambiguous comments, add comments.
!         - debugging and simplifications for "lrubc" code.
!        00-12-18 C. Fischer : merge karim's ldvercor with ecmwf's lvertfe
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Jan 2008): complete (LVERCOR,LVERTFE)=(T,T).
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   K. Yessad (Dec 2008): remove dummy CDLOCK
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCVER  , ONLY : LVERTFE
USE YOMGEM   , ONLY : VBF      ,VFE_RDETAH
USE PAR_RDLR , ONLY : JP_ASRF  ,JP_RSAFL ,JP_RSAFM ,JP_DIMR

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMVC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
LOGICAL           ,INTENT(IN)    :: LDVERCOR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVDELB(KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVBH(0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PD(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVT(KPROMA) 
LOGICAL           ,INTENT(IN)    :: LDRUBC 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRTGR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCRSSAF(KPROMVC,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCRSSAH(KPROMVC,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCASRSF(KPROMVC,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCASRSH(KPROMVC,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCRSSAFL(KPROMVC,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCRSSAFM(KPROMVC,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEVEL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVVEL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSDIV(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSDVBC(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIVDP(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPREF(KPROMA,KFLEV) 
LOGICAL,OPTIONAL  ,INTENT(IN)    :: LDRWSDLR
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PRDLR(KPROMA,0:KFLEV,JP_DIMR)

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZSDIV(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZPSDIV(KPROMA,KFLEV+1)
REAL(KIND=JPRB) :: ZVP(KPROMA,KFLEV)

INTEGER(KIND=JPIM) :: JROF,JLEV
LOGICAL :: LLRWSDLR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "verint.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPCTY',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!     check for non-supported configurations

IF (LDRUBC.AND.LVERTFE) CALL ABOR1('GPCTY: BAD OPTIONS')

IF (PRESENT(LDRWSDLR)) THEN
  LLRWSDLR=LDRWSDLR
ELSE
  LLRWSDLR=.FALSE.
ENDIF
IF (LLRWSDLR.AND.(.NOT.PRESENT(PRDLR))) THEN
  CALL ABOR1('GPCTY: if LRWSDLR=T, PRDLR should be present')
ENDIF

!     ------------------------------------------------------------------

!*    1. Computes "vec(V) * rssavnabla prehyds" on layers.
!     ----------------------------------------------------

DO JLEV=1,KFLEV
  ZVP(KD:KF,JLEV)=PU(KD:KF,JLEV)*PSPL(KD:KF)+PV(KD:KF,JLEV)*PSPM(KD:KF)
ENDDO

!     ------------------------------------------------------------------

!*    2. Sum divergence.
!     ------------------

DO JLEV=1,KFLEV
  PDIVDP(KD:KF,JLEV)=&
   & PD(KD:KF,JLEV)*PDELP(KD:KF,JLEV)+PVDELB(JLEV)*ZVP(KD:KF,JLEV)  
  IF (LDVERCOR) THEN
    PDIVDP(KD:KF,JLEV)=PDIVDP(KD:KF,JLEV)*PVCRSSAF(KD:KF,JLEV)
    PDIVDP(KD:KF,JLEV)=&
     & PDIVDP(KD:KF,JLEV)+PDELP(KD:KF,JLEV)*&
     & (PU(KD:KF,JLEV)*PVCRSSAFL(KD:KF,JLEV)&
     & +PV(KD:KF,JLEV)*PVCRSSAFM(KD:KF,JLEV))  
  ENDIF
  IF (LLRWSDLR) THEN
    DO JROF=KD,KF
      PDIVDP(JROF,JLEV)=PDIVDP(JROF,JLEV) &
       & *PRDLR(JROF,JLEV,JP_ASRF)*PRDLR(JROF,JLEV,JP_ASRF)
      PDIVDP(JROF,JLEV)=PDIVDP(JROF,JLEV) &
       & -(PRDLR(JROF,JLEV,JP_ASRF)**3.0_JPRB)*PDELP(JROF,JLEV)* &
       & (PU(JROF,JLEV)*PRDLR(JROF,JLEV,JP_RSAFL) &
       & +PV(JROF,JLEV)*PRDLR(JROF,JLEV,JP_RSAFM))
    ENDDO
  ENDIF
ENDDO

PSDIV(KD:KF,0)=0.0_JPRB

IF(LVERTFE) THEN
  DO JLEV=1,KFLEV
    ZSDIV(KD:KF,JLEV)=PDIVDP(KD:KF,JLEV)*VFE_RDETAH(JLEV)
  ENDDO
  CALL VERINT(KPROMA,KD,KF,KFLEV,ZSDIV,ZPSDIV,0)
  DO JLEV=1,KFLEV
    PSDIV(KD:KF,JLEV)=ZPSDIV(KD:KF,JLEV)
  ENDDO
ELSE
  DO JLEV=1,KFLEV
    PSDIV(KD:KF,JLEV)=PSDIV(KD:KF,JLEV-1)+PDIVDP(KD:KF,JLEV)
  ENDDO
ENDIF

IF (LDRUBC) THEN
  IF(LVERTFE) THEN
    ! ky: not validated.
    IF (LDVERCOR) THEN
      PSDVBC(KD:KF,KFLEV)=ZPSDIV(KD:KF,KFLEV+1)&
       & -PVCRSSAH(KD:KF,0)*PVCRSSAH(KD:KF,0)*PEVT(KD:KF)  
    ELSE
      PSDVBC(KD:KF,KFLEV)=ZPSDIV(KD:KF,KFLEV+1)-PEVT(KD:KF)
    ENDIF
  ELSE
    IF (LDVERCOR) THEN
      DO JLEV=0,KFLEV
        PSDVBC(KD:KF,JLEV)=PSDIV(KD:KF,JLEV)&
         & -PVCRSSAH(KD:KF,0)*PVCRSSAH(KD:KF,0)*PEVT(KD:KF)  
      ENDDO
    ELSE
      DO JLEV=0,KFLEV
        PSDVBC(KD:KF,JLEV)=PSDIV(KD:KF,JLEV)-PEVT(KD:KF)
      ENDDO
    ENDIF
  ENDIF
ELSE
  IF(LVERTFE) THEN
    PSDVBC(KD:KF,KFLEV)=ZPSDIV(KD:KF,KFLEV+1)
  ELSE
    PSDVBC(KD:KF,0:KFLEV)=PSDIV(KD:KF,0:KFLEV)
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*    3. Computes "etadot (d prehyd / d eta)".
!     ----------------------------------------

! "etadot (d prehyd / d eta)" is computed at full levels if LVERTFE=T,
!  at half levels otherwise.

IF (LDVERCOR) THEN
  IF(LVERTFE) THEN
    DO JLEV=1,KFLEV
      PEVEL(KD:KF,JLEV)=VBF(JLEV)*PVCASRSH(KD:KF,KFLEV)*PVCASRSH(KD:KF,KFLEV)&
       & *ZPSDIV(KD:KF,KFLEV+1)&
       & -PVCASRSF(KD:KF,JLEV)*PVCASRSF(KD:KF,JLEV)*PSDIV(KD:KF,JLEV)  
    ENDDO
  ELSE
    DO JLEV=1,KFLEV-1
      PEVEL(KD:KF,JLEV)=&
       & PVBH(JLEV)*PVCASRSH(KD:KF,KFLEV)*PVCASRSH(KD:KF,KFLEV)&
       & *PSDIV(KD:KF,KFLEV)&
       & -PVCASRSH(KD:KF,JLEV)*PVCASRSH(KD:KF,JLEV)*PSDIV(KD:KF,JLEV)  
    ENDDO
  ENDIF
ELSE
  IF(LVERTFE) THEN
    DO JLEV=1,KFLEV
      PEVEL(KD:KF,JLEV)=VBF(JLEV)*ZPSDIV(KD:KF,KFLEV+1)&
       & -PSDIV(KD:KF,JLEV)  
    ENDDO
  ELSE
    DO JLEV=1,KFLEV-1
      PEVEL(KD:KF,JLEV)=PVBH(JLEV)*PSDIV(KD:KF,KFLEV)-PSDIV(KD:KF,JLEV)
    ENDDO
  ENDIF
ENDIF

PEVEL(KD:KF,0    )=0.0_JPRB
IF(.NOT.LVERTFE) PEVEL(KD:KF,KFLEV)=0.0_JPRB

IF (LDRUBC) THEN
  IF (LDVERCOR) THEN
    IF (LVERTFE) THEN
      CALL ABOR1(' GPCTY: VFE not coded for LRUBC.')
    ELSE
      DO JLEV=0,KFLEV
        PEVEL(KD:KF,JLEV)=PEVEL(KD:KF,JLEV)&
         & +(PVCASRSH(KD:KF,JLEV)*PVCASRSH(KD:KF,JLEV)&
         & -PVBH(JLEV)*PVCASRSH(KD:KF,KFLEV)*PVCASRSH(KD:KF,KFLEV))&
         & *PVCRSSAH(KD:KF,0)*PVCRSSAH(KD:KF,0)*PEVT(KD:KF)  
      ENDDO
    ENDIF
  ELSE
    IF (LVERTFE) THEN
      CALL ABOR1(' GPCTY: VFE not coded for LRUBC.')
    ELSE
      DO JLEV=0,KFLEV
        PEVEL(KD:KF,JLEV)=PEVEL(KD:KF,JLEV)+(1.0_JPRB-PVBH(JLEV))*PEVT(KD:KF)
      ENDDO
    ENDIF
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*    4. Computes "(omega/prehyd)" on layers.
!     ---------------------------------------

! * 4.1: computes "(rs/a)**2 (omega/prehyd)" if "ldvercor=T",
!        "(omega/prehyd)" otherwise.

IF(LVERTFE) THEN

  IF (LDVERCOR) THEN
    DO JLEV=1,KFLEV
      PVVEL(KD:KF,JLEV)=(-PSDIV(KD:KF,JLEV) &
       & +VBF(JLEV)*PVCRSSAF(KD:KF,JLEV)*ZVP(KD:KF,JLEV))*PRPREF(KD:KF,JLEV)
    ENDDO
  ELSE
    DO JLEV=1,KFLEV
      PVVEL(KD:KF,JLEV)=(-PSDIV(KD:KF,JLEV)+VBF(JLEV)*ZVP(KD:KF,JLEV))&
       & *PRPREF(KD:KF,JLEV)
    ENDDO
  ENDIF

ELSE

  DO JLEV=1,KFLEV
    PVVEL(KD:KF,JLEV)=+PRTGR(KD:KF,JLEV)*ZVP(KD:KF,JLEV)&
     & -PRDELP(KD:KF,JLEV)*PALPH(KD:KF,JLEV)*PVDELB(JLEV)*ZVP(KD:KF,JLEV)&
     & -PALPH(KD:KF,JLEV)*PD(KD:KF,JLEV)  
  ENDDO
  IF (LDVERCOR) THEN
    DO JLEV=1,KFLEV
      PVVEL(KD:KF,JLEV)=PVVEL(KD:KF,JLEV)*PVCRSSAF(KD:KF,JLEV)
      PVVEL(KD:KF,JLEV)=PVVEL(KD:KF,JLEV)&
       & -PALPH(KD:KF,JLEV)*(PU(KD:KF,JLEV)*PVCRSSAFL(KD:KF,JLEV)&
       & +PV(KD:KF,JLEV)*PVCRSSAFM(KD:KF,JLEV))  
    ENDDO
  ENDIF
  DO JLEV=1,KFLEV
    PVVEL(KD:KF,JLEV)=PVVEL(KD:KF,JLEV)&
     & -PRDELP(KD:KF,JLEV)*PSDVBC(KD:KF,JLEV-1)*PLNPR(KD:KF,JLEV)  
  ENDDO

ENDIF

! * 4.2: multiply by (a/rs)**2 to recover "(omega/prehyd)" if "ldvercor=T".

IF (LDVERCOR) THEN
  PVVEL(KD:KF,1:KFLEV)=PVVEL(KD:KF,1:KFLEV)&
   & *PVCASRSF(KD:KF,1:KFLEV)*PVCASRSF(KD:KF,1:KFLEV)  
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPCTY',1,ZHOOK_HANDLE)
END SUBROUTINE GPCTY

