SUBROUTINE GPGRVCRS(KPROMA,KD,KF,KFLEV,&
 & PRTGR,PREHYDF,PREHYDSL,PREHYDSM,&
 & PVCRSSAFL,PVCRSSAFM)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RA       ,RD       ,RCPD     ,RG
USE YOMDYN   , ONLY : VCAK     ,VCTR     ,VCPR

!**** *GPGRVCRS* - Computes the quantity "rssavnabla (rs/a)" on model layers.

!**   Purpose.
!     --------

!      When no thin layer hypothesis is applied, the model needs
!      a pseudo-radius "rs" which is not equal to the mean radius "a"
!      except for a reference pressure "prehydref". 
!      "rs" depends on the hydrostatic pressure "prehyd" and its expression is

!                  -- prehyd'=prehydref 
!                  |                       Rd Ts
!       rs = a +   |                     ----------  d prehyd'
!                  |                      g prehyd'
!                 --  prehyd'=prehyd

!      Reference pressure "prehydref" is assumed to be equal to the 
!      standard atmosphere surface temperature "prehydRST_surf".

!      Its expression involves a reference temperature Ts which only depends
!      on the hydrostatic pressure "prehyd". On assumes that this reference
!      temperature is computed in an analytic manner:

!       log(Ts/T1) = alpha_K (Rd/cpd) log(prehyd/prehydRST_surf)

!      where:
!      - T1 is a parametrable reference temperature.
!      - alpha_K is a parametrable constant.
!        For alpha_K=0 the profile of Ts is vertically constant: Ts=T1.
!        For alpha_K=1 the profile of Ts is a dry adiabatic one.
!      - Rd is the dry air constant.
!      - cpd is the constant pressure calorific capacity of dry air.

!      Combination of the previous equations for "Ts" and "rs" and application
!      of the operator semi-reduced horizontal gradient "rssavnabla"
!      ( rssavnabla X = [rs/a] vnabla X ) leads to
!      the following analytic expression for " rssavnabla (rs/a) " to be coded:

!                   rs      Rd T1       prehyd**vcpow      rssavnabla prehyd
!        rssavnabla -- = - -------  ---------------------  -----------------
!                    a       a g    prehydRST_surf**vcpow       prehyd

!        where: vcpow = alpha_K (Rd/cpd)

!**   Interface.
!     ----------
!        *CALL* *GPGRVCRS( ... )

!        Explicit arguments :
!        --------------------
!        INPUT :
!           KPROMA    - horizontal dimensioning.
!           KD        - start of work.
!           KF        - depth of work.
!           KFLEV     - vertical dimensioning (number of layers).
!           PRTGR     - scalar ratio of vectors "(rssavnabla prehyd)/prehyd"
!                       and "rssavnabla prehyds" (computed in GPXYB).
!           PREHYDF   - hydrostatic pressure on layers.
!           PREHYDSL  - zonal component of "rssavnabla prehyds"
!                       ("prehyds" is surface hydrostatic pressure).
!           PREHYDSM  - meridian component of "rssavnabla prehyds".

!        OUTPUT :
!           PVCRSSAFL - zonal component of "rssavnabla (rs/a)" on layers.
!           PVCRSSAFM - meridian component of "rssavnabla (rs/a)" on layers.

!        Implicit arguments :
!        --------------------
!        none.

!     Method.
!     -------

!     Externals.
!     ----------
!        none.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (METEO-FRANCE/CNRM/GMAP).

!     Modifications.
!     --------------
!        Original: AUGUST 2000.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Dec 2008): remove dummy CDLOCK
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRTGR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREHYDF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREHYDSL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREHYDSM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVCRSSAFL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVCRSSAFM(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV

REAL(KIND=JPRB) :: ZVCPOW
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPGRVCRS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

! * Preliminary initialisations:

ZVCPOW=VCAK*RD/RCPD

! ------------------------------------------------------------------

! * Computation of "rssavnabla (rs/a)" on layers:

DO JLEV=1,KFLEV
  PVCRSSAFL(KD:KF,JLEV)= &
   & - ((RD*VCTR)/(RA*RG)) * ( PREHYDF(KD:KF,JLEV)/VCPR )**ZVCPOW * &
   & PRTGR(KD:KF,JLEV) * PREHYDSL(KD:KF)  
  PVCRSSAFM(KD:KF,JLEV)= &
   & - ((RD*VCTR)/(RA*RG)) * ( PREHYDF(KD:KF,JLEV)/VCPR )**ZVCPOW * &
   & PRTGR(KD:KF,JLEV) * PREHYDSM(KD:KF)  
ENDDO

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPGRVCRS',1,ZHOOK_HANDLE)
END SUBROUTINE GPGRVCRS
