SUBROUTINE GPGRXYB(KPROMA,KD,KF,KFLEV,PVC,PVBH,&
 & PREL,PREM,PRDELP,PRPRE,PRPP,PLNPR,PALPH,PRTGR,&
 & PLNPRL,PLNPRM,PALPHL,PALPHM,PALPHPLL,PALPHPLM,&
 & PCOEFD,PCOEFA,PCOEFAPL &
 & )  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDYN   , ONLY : NDLNPR
USE YOMCVER  , ONLY : LVERTFE
USE YOMGEM   , ONLY : VDELB

!**** *GPGRXYB* - Complement to routine "GPXYB".
!                 Computation of the horizontal gradient of quantities
!                 "alpha" and "delta" on model layers.

!     Purpose.
!     --------

!     "alpha" and "delta" are computed on model layers in routine "GPXYB",
!     but not their horizontal gradient. So this routine provides the
!     horizontal gradients on layers. Quantity
!     "(rssavnabla(alpha)) + (rssavnabla(prehyd)/prehyd)"
!     is also provided separately (for case LVERTFE=.F.
!     its "NDLNPR=0" expression is simpler than the expressions
!     of "rssavnabla(alpha)" and "rssavnabla(prehyd)/prehyd").
!     Discretisation depends on variables "NDLNPR" and "LVERTFE".

!     For LVERTFE=.F., NDLNPR=0, discretisations are:

!      (rssavnabla(delta))[l] =
!      - (A[lbar]*B[lbar-1]-A[lbar-1]*B[lbar])/(prehyd[lbar]*prehyd[lbar-1])
!      * (rssavnabla prehyds)

!      (rssavnabla(alpha))[l] + (rssavnabla(prehyd)/prehyd)[l] =
!      B[lbar]/prehyd[lbar] * (rssavnabla prehyds)

!      Quantity "(rssavnabla(alpha))[l]" is computed by substracting
!      "(rssavnabla(prehyd)/prehyd)[l]" from
!      "(rssavnabla(alpha))[l] + (rssavnabla(prehyd)/prehyd)[l]"

!     For LVERTFE=.F., NDLNPR=1, discretisations are:

!      (rssavnabla(delta))[l] =
!      - delta[l] * (A[lbar]*B[lbar-1]-A[lbar-1]*B[lbar])
!      * (1/sqrt(prehyd[lbar]*prehyd[lbar-1])) * (1/(delta prehyd[l]))
!      * (rssavnabla prehyds)

!      (rssavnabla(alpha))[l] =
!      - alpha[l] * (A[lbar]*B[lbar-1]-A[lbar-1]*B[lbar])
!      * (1/sqrt(prehyd[lbar]*prehyd[lbar-1])) * (1/(delta prehyd[l]))
!      * (rssavnabla prehyds)

!      (rssavnabla(prehyd)/prehyd)[l] = prtgr[l] * (rssavnabla prehyds)
!      where "prtgr[l]" is computed in routine "gpxyb" as:
!      prtgr[l] = { (delta B)[l]
!      + delta[l] * (A[lbar]*B[lbar-1]-A[lbar-1]*B[lbar])/(delta prehyd[l]) }
!      * { 1/(delta prehyd[l]) }

!      In this case "(rssavnabla(alpha))[l]" is computed prior to
!      "(rssavnabla(alpha))[l] + (rssavnabla(prehyd)/prehyd)[l]"

!     For LVERTFE=.T., NDLNPR=0, discretisations are:

!      (rssavnabla(delta))[l] =
!      delta[l] * ((Delta B)[l]/(Delta prehyd)[l] - B[l]/prehyd[l])
!      * (rssavnabla prehyds)

!      rssavnabla(alpha) is useless in this case.

!     Notations:
!      - "rssavnabla" is the semi-reduced horizontal gradient operator.
!        (rssavnabla X = (rs/a) vnabla X = M vnabla' X)
!      - "prehyd" is the hydrostatic pressure.
!      - "prehyds" is the surface hydrostatic pressure.

!**   Interface.
!     ----------
!        *CALL* *GPGRXYB(...)

!        Explicit arguments :
!        --------------------
!         * INPUT:
!           KPROMA       : horizontal dimension
!           KD           : start of work
!           KF           : working length
!           KFLEV        : number of levels
!           PVC          : A[lbar]*B[lbar-1] - A[lbar-1]*B[lbar]
!           PVBH         : B[lbar] on interlayers
!           PREL         : zonal component of "rssavnabla prehyds"
!           PREM         : meridian component of "rssavnabla prehyds"
!           PRDELP       : 1./( prehyd[lbar] - prehyd[lbar-1] )
!           PRPRE        : 1./(prehyd[lbar]) (on interlayers) 
!           PRPP         : 1./((prehyd[lbar])*(prehyd[lbar-1]))
!           PLNPR        : "delta" on layers (is equal to
!                          "log(prehyd[lbar]/prehyd[lbar-1])" if "ndlnpr=0")
!           PALPH        : "alpha" on layers
!           PRTGR        : scalar ratio of vectors "(rssavnabla prehyd)/prehyd"
!                          and "rssavnabla prehyds" (computed in GPXYB).

!         * OUTPUT:
!           PLNPRL       : zonal component of "rssavnabla(delta)" on layers
!           PLNPRM       : meridian component of "rssavnabla(delta)" on layers
!           PALPHL       : zonal component of "rssavnabla(alpha)" on layers
!           PALPHM       : meridian component of "rssavnabla(alpha)" on layers
!           PALPHPLL     : zonal component of "rssavnabla (alpha + log prehyd)"
!                          on layers
!           PALPHPLM     : meridian component "rssavnabla (alpha + log prehyd)"
!                          on layers

!         * OUTPUT OPTIONAL:
!           PCOEFD       : coefficient to compute (PLNPRL,PLNPRM)
!           PCOEFA       : coefficient to compute (PALPHL,PALPHM)
!           PCOEFAPL     : coefficient to compute (PALPHPLL,PALPHPLM)

!        Implicit arguments :   None.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.    None.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD

!     Modifications.
!     --------------
!        Original : 00-08-11
!        K. YESSAD: May 2002: bugs and comments correction for LVERTFE=T
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. YESSAD: Dec 2004: bug correction for NDLNPR=1 + cleanings.
!        K. YESSAD: Mar 2006: Save some quantities in optional arg for TL/AD.
!        K. Yessad (Dec 2008): remove dummy CDLOCK
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVC(KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVBH(0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPRE(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPP(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRTGR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLNPRL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLNPRM(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPHL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPHM(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPHPLL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PALPHPLM(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL ,INTENT(OUT) :: PCOEFD(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL ,INTENT(OUT) :: PCOEFA(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL ,INTENT(OUT) :: PCOEFAPL(KPROMA,KFLEV)

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZCOEFA(KPROMA), ZCOEFAPL(KPROMA), ZCOEFD(KPROMA)

INTEGER(KIND=JPIM) :: JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPGRXYB',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*    1/ Calculation of "rssavnabla delta" on layers.

IF(LVERTFE) THEN
  DO JLEV=1,KFLEV
    DO JROF=KD,KF
      ZCOEFD(JROF)=(VDELB(JLEV)*PRDELP(JROF,JLEV)-PRTGR(JROF,JLEV))&
       & *PLNPR(JROF,JLEV)  
      PLNPRL(JROF,JLEV)=ZCOEFD(JROF)*PREL(JROF)
      PLNPRM(JROF,JLEV)=ZCOEFD(JROF)*PREM(JROF)
    ENDDO
    IF (PRESENT(PCOEFD)) PCOEFD(KD:KF,JLEV)=ZCOEFD(KD:KF)
  ENDDO
ELSE
  DO JLEV=1,KFLEV
    DO JROF=KD,KF
      ZCOEFD(JROF)=-PVC(JLEV)*PRPP(JROF,JLEV)
      PLNPRL(JROF,JLEV)=ZCOEFD(JROF)*PREL(JROF)
      PLNPRM(JROF,JLEV)=ZCOEFD(JROF)*PREM(JROF)
    ENDDO
    IF (PRESENT(PCOEFD)) PCOEFD(KD:KF,JLEV)=ZCOEFD(KD:KF)
  ENDDO
ENDIF

!*    2/ Calculation of "rssavnabla (alpha + log prehyd)" on layers
!        discretised as "rssavnabla alpha + (rssavnabla prehyd) / prehyd ",
!        and calculation of "rssavnabla alpha" on layers.

IF(.NOT.LVERTFE) THEN
  IF(NDLNPR == 0) THEN
    DO JLEV=1,KFLEV
      DO JROF=KD,KF
        ZCOEFAPL(JROF)=PVBH(JLEV)*PRPRE(JROF,JLEV)
        ZCOEFA(JROF)=ZCOEFAPL(JROF)-PRTGR(JROF,JLEV)
        PALPHPLL(JROF,JLEV)=ZCOEFAPL(JROF)*PREL(JROF)
        PALPHPLM(JROF,JLEV)=ZCOEFAPL(JROF)*PREM(JROF)
        PALPHL(JROF,JLEV)=ZCOEFA(JROF)*PREL(JROF)
        PALPHM(JROF,JLEV)=ZCOEFA(JROF)*PREM(JROF)
      ENDDO
      IF (PRESENT(PCOEFA)) PCOEFA(KD:KF,JLEV)=ZCOEFA(KD:KF)
      IF (PRESENT(PCOEFAPL)) PCOEFAPL(KD:KF,JLEV)=ZCOEFAPL(KD:KF)
    ENDDO
  ELSEIF(NDLNPR == 1) THEN
    DO JLEV=1,KFLEV
      DO JROF=KD,KF
        ZCOEFA(JROF)=-PVC(JLEV)*PRPP(JROF,JLEV)*PALPH(JROF,JLEV) &
         & /PLNPR(JROF,JLEV)
        ZCOEFAPL(JROF)=ZCOEFA(JROF)+PRTGR(JROF,JLEV)
        PALPHPLL(JROF,JLEV)=ZCOEFAPL(JROF)*PREL(JROF)
        PALPHPLM(JROF,JLEV)=ZCOEFAPL(JROF)*PREM(JROF)
        PALPHL(JROF,JLEV)=ZCOEFA(JROF)*PREL(JROF)
        PALPHM(JROF,JLEV)=ZCOEFA(JROF)*PREM(JROF)
      ENDDO
      IF (PRESENT(PCOEFA)) PCOEFA(KD:KF,JLEV)=ZCOEFA(KD:KF)
      IF (PRESENT(PCOEFAPL)) PCOEFAPL(KD:KF,JLEV)=ZCOEFAPL(KD:KF)
    ENDDO
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPGRXYB',1,ZHOOK_HANDLE)
END SUBROUTINE GPGRXYB
