SUBROUTINE GPGW( &
 ! --- INPUT -----------------------------------------------------------------
 & KFLEV,KPROMA,KSTART,KEND,LDGWF,LDGDWI, &
 & POROGL,POROGM,PLNPR,PALPH,PUS,PVS, &
 & PT,PDVER,PRNHPPI, &
 ! --- OUTPUT ----------------------------------------------------------------
 & PGWH,PGWF, &
 ! --- OPTIONAL INPUT --------------------------------------------------------
 & LDVFE,LDRWSDLR,LDRWSDLR2,PRDLR, &
 ! --- OPTIONAL OUTPUT -------------------------------------------------------
 & PGDW &
 & )

! GPGW - Diagnoses "Gw" from the vertical divergence "dver" or from "-G dw".
!        If LRWSDLG, this is "Gw" which is diagnosed (the vertically dependent
!        "g" is not used in this routine).

! Purpose
! -------
!   Diagnoses "Gw" from the vertical divergence "dver" if LDGDWI=F,
!    from "-G dw" if LDGDWI=T.
!   For the finite element vertical discretization (lvertfe=T + lvfe_gw=T),
!    "Gw" is computed only at full levels.
!   For the finite difference vertical discretization, "Gw" is
!    computed at both half and full levels.
!   Calculation is done by vertical integration of the formula:
!    lrwsdlr=F: dver = - (G pre)/(Rd T prehyd) (d w / d log(prehyd))
!    lrwsdlr=T,lrwsdlr2=F:
!     dver = - (G pre)/(Rd T presim) (r**2/a**2) (d w / d log(presim))
!    lrwsdlr=T,lrwsdlr2=T:
!     dver = - (G pre)/(Rd T presim) (d (r**2/a**2) w / d log(presim))
!   with the following bottom condition:
!    w_surf = V_surf grad[Phi_s].

!   This routine can be used in a hydrostatic model: in this case
!   the ratio (prehyd/pre) is equal to 1.

! Interface
! ---------
!   * INPUT:
!   KFLEV   - number of levels.
!   KPROMA  - horizontal dimension.
!   KSTART  - start of work.
!   KEND    - end of work.
!   LDGWF   - calculation of "Gw" at full layers asked for
!             if finite difference vertical discretization.
!   LDGDWI  - T vs F: input content of PDVER is "-G dw" vs "dver".
!   POROGL  - zonal component of grad[Phi_s].
!   POROGM  - meridian component of grad[Phi_s].
!   PLNPR   - "delta" at full layers (computed in GPXYB).
!   PALPH   - "alpha" at full layers (computed in GPXYB).
!   PUS     - surface U wind.
!   PVS     - surface V wind.
!   PT      - temperature at full layers.
!   PDVER   - vertical divergence "dver" at full layers.
!   PRNHPPI - (prehyd/pre) at full layers
!             (presim/pre if ldrwsdlr=T).

!   * OUTPUT:
!   PGWH    - G times vertical velocity w at half layers.
!             (computed only if lvertfe=F)
!   PGWF    - G times vertical velocity w at full layers.

!   * OPTIONAL INPUT:
!   LDVFE   - T if VFE discretisation is used in this routine.
!   LDRWSDLR - key for metric terms in Wood and Staniforth DL eqns
!   LDRWSDLR2- takes account of the metric terms for conversions between
!              "dver" and "Gw".
!   PRDLR    - array containing "r/a", "a/r", "grad(r/a)".

!   * OPTIONAL OUTPUT:
!   PGDW    - contains 'G (dw)'.

! Externals
! ---------

! Method
! ------

! Reference
! ---------

! Author
! ------
!   K. Yessad, Dec 2004 (after GNHSVD2GW)

! Modifications
! -------------
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCT0   , ONLY : LNHDYN
USE YOMCST   , ONLY : RD     
USE YOMDYNA  , ONLY : NVDVAR   ,LRWSDLR2
USE YOMCVER  , ONLY : LVERTFE  ,LVFE_GW
USE YOMGEM   , ONLY : VFE_RDETAH
USE PAR_RDLR , ONLY : JP_ASRH  ,JP_ASRF  ,JP_RSAH  ,JP_DIMR
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

! -----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
LOGICAL           ,INTENT(IN)    :: LDGWF
LOGICAL           ,INTENT(IN)    :: LDGDWI
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDVER(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRNHPPI(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGWH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGWF(KPROMA,KFLEV) 
LOGICAL, OPTIONAL ,INTENT(IN)    :: LDVFE
LOGICAL,OPTIONAL  ,INTENT(IN)    :: LDRWSDLR
LOGICAL,OPTIONAL  ,INTENT(IN)    :: LDRWSDLR2
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PRDLR(KPROMA,0:KFLEV,JP_DIMR)
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PGDW(KPROMA,KFLEV) 

! -----------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF
LOGICAL            :: LLVFE,LLRWSDLR,LLRWSDLR2
REAL(KIND=JPRB) :: ZGDW(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZGWH(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZGWS(KPROMA)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

! -----------------------------------------------------------------------------

#include "abor1.intfb.h"
#include "verint.intfb.h"

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPGW',0,ZHOOK_HANDLE)

! -----------------------------------------------------------------------------

!*      0. PRELIMINARY CALCULATIONS
!       ---------------------------

IF( PRESENT(LDVFE) ) THEN
  LLVFE=LDVFE
ELSE
  IF (LNHDYN) THEN
    IF (NVDVAR==3 .OR. NVDVAR==4) THEN
      LLVFE=LVERTFE.AND.LVFE_GW
    ELSEIF (NVDVAR==5 .OR. NVDVAR==6) THEN
      LLVFE=LVERTFE
    ENDIF
  ELSE
    LLVFE=LVERTFE
  ENDIF
ENDIF

IF (PRESENT(LDRWSDLR)) THEN
  LLRWSDLR=LDRWSDLR
ELSE
  LLRWSDLR=.FALSE.
ENDIF
IF (LLRWSDLR.AND.(.NOT.PRESENT(PRDLR))) THEN
  CALL ABOR1('GPGW: if LRWSDLR=T, PRDLR should be present')
ENDIF

IF (PRESENT(LDRWSDLR2)) THEN
  LLRWSDLR2=LDRWSDLR2
ELSE
  LLRWSDLR2=.FALSE.
ENDIF
IF (LLRWSDLR2.AND.(.NOT.LLRWSDLR)) THEN
  CALL ABOR1('GPGW: (LLRWSDLR,LLRWSDLR2)=(F,T) not possible')
ENDIF

IF (LLRWSDLR2.AND.LLVFE) THEN
  CALL ABOR1('GPGW: not yet coded for (LRWSDLR2,LVFE_GW)=(T,T)')
ENDIF

! -----------------------------------------------------------------------------

!*      1. Case LDGDWI=F: computes "Gw" from "dver"
!       -------------------------------------------

! The code takes account of LDRWSDLR2, and the output variable is always "Gw",
! never "G(r**2/a**2)w". 

IF (.NOT.LDGDWI) THEN

  ! * Transform "dver" into "-G.dw" (or -G.(d(r**2/a**2)w) if LLRWSDLR2=T).
  DO JLEV=1,KFLEV
    DO JROF=KSTART,KEND
      ZGDW(JROF,JLEV)=PDVER(JROF,JLEV)* &
       & RD*PT(JROF,JLEV)*PLNPR(JROF,JLEV)*PRNHPPI(JROF,JLEV)
    ENDDO
  ENDDO
  IF (LLRWSDLR.AND. .NOT.LLRWSDLR2) THEN
    DO JLEV=1,KFLEV
      DO JROF=KSTART,KEND
        ZGDW(JROF,JLEV)=ZGDW(JROF,JLEV)* &
         & PRDLR(JROF,JLEV,JP_ASRF)*PRDLR(JROF,JLEV,JP_ASRF)
      ENDDO
    ENDDO
  ENDIF

  ! * Compute "Gw" (or "G(r**2/a**2)w" if LLRWSDLR2=T) at the surface
  !   (free slip boundary condition)
  DO JROF=KSTART,KEND
    ZGWS(JROF)=PUS(JROF)*POROGL(JROF)+PVS(JROF)*POROGM(JROF)
    PGWH(JROF,KFLEV)=ZGWS(JROF)
  ENDDO
  IF(LLRWSDLR) THEN
    ! Take account to the fact that POROG(L,M) contain (r_surf/a)*nabla(Phi_s)
    ! and that ZGWS=V_surf.nabla(Phi_s): additional multiplication by (a/r_surf).
    DO JROF=KSTART,KEND
      ZGWS(JROF)=ZGWS(JROF)*PRDLR(JROF,KFLEV,JP_ASRH)
      PGWH(JROF,KFLEV)=ZGWS(JROF)
    ENDDO
  ENDIF
  IF(LLRWSDLR.AND.LLRWSDLR2) THEN
    ! "Gw" -> G(r**2/a**2)w
    ! Vertical integration provides G(r**2/a**2)w, and this is
    ! "G(r**2/a**2)w" at the surface which must be computed.
    DO JROF=KSTART,KEND
      ZGWS(JROF)=ZGWS(JROF)*PRDLR(JROF,KFLEV,JP_RSAH)*PRDLR(JROF,KFLEV,JP_RSAH)
      PGWH(JROF,KFLEV)=ZGWS(JROF)
    ENDDO
  ENDIF

  IF (LLVFE) THEN

    ! * Store "G dw" at full levels.
    IF (PRESENT(PGDW)) PGDW(KSTART:KEND,1:KFLEV)=-ZGDW(KSTART:KEND,1:KFLEV)

    ! * Compute "Gw" at full levels.

    DO JLEV=1,KFLEV
      DO JROF=KSTART,KEND
        ZGDW(JROF,JLEV)=-ZGDW(JROF,JLEV)*VFE_RDETAH(JLEV)
      ENDDO
    ENDDO

    CALL VERINT(KPROMA,KSTART,KEND,KFLEV,ZGDW,ZGWH,1)

    DO JLEV=1,KFLEV
      DO JROF=KSTART,KEND
        PGWF(JROF,JLEV)=ZGWH(JROF,JLEV-1)+ZGWS(JROF)
      ENDDO
    ENDDO

  ELSE

    ! * Compute "Gw" (G(r**2/a**2)w if LLRWSDLR2=T) at half levels.

    ! transform -G.dw into Gw (-G.(d(r**2/a**2)w) -> G(r**2/a**2)w if LLRWSDLR2=T)
    DO JLEV=KFLEV,1,-1
      DO JROF=KSTART,KEND
        PGWH(JROF,JLEV-1)=PGWH(JROF,JLEV)+ZGDW(JROF,JLEV)
      ENDDO
    ENDDO

    ! * Compute "Gw" (G(r**2/a**2)w if LLRWSDLR2=T) at full levels.

    IF (LDGWF) THEN
      ! k.y.: formula pgwf(jlev)=pgwh(jlev)(1-palph(jlev)/plnpr(jlev))
      ! +pgwh(jlev-1)(palph(jlev)/plnpr(jlev)) must be equivalent.
      IF (LLRWSDLR.AND. .NOT.LLRWSDLR2) THEN
        DO JLEV=1,KFLEV
          DO JROF=KSTART,KEND
            PGWF(JROF,JLEV)=PGWH(JROF,JLEV)+PDVER(JROF,JLEV)* &
             & RD*PT(JROF,JLEV)*PALPH(JROF,JLEV)*PRNHPPI(JROF,JLEV)* &
             & PRDLR(JROF,JLEV,JP_ASRF)*PRDLR(JROF,JLEV,JP_ASRF)
          ENDDO
        ENDDO
      ELSE
        DO JLEV=1,KFLEV
          DO JROF=KSTART,KEND
            PGWF(JROF,JLEV)=PGWH(JROF,JLEV)+PDVER(JROF,JLEV)* &
             & RD*PT(JROF,JLEV)*PALPH(JROF,JLEV)*PRNHPPI(JROF,JLEV)
          ENDDO
        ENDDO
      ENDIF
    ENDIF

  ENDIF ! test on LLVFE

  ! Convert G(r**2/a**2)w into Gw if relevant.
  IF(LLRWSDLR.AND.LLRWSDLR2) THEN
    IF (LLVFE) THEN
      ! not yet coded, at least multiply PGWF by (a**2/r**2).
    ELSE
      DO JLEV=0,KFLEV
        DO JROF=KSTART,KEND
          PGWH(JROF,JLEV)=PGWH(JROF,JLEV)* &
           & PRDLR(JROF,JLEV,JP_ASRH)*PRDLR(JROF,JLEV,JP_ASRH)
        ENDDO
      ENDDO
      IF (LDGWF) THEN
        DO JLEV=1,KFLEV
          DO JROF=KSTART,KEND
            PGWF(JROF,JLEV)=PGWF(JROF,JLEV)* &
             & PRDLR(JROF,JLEV,JP_ASRF)*PRDLR(JROF,JLEV,JP_ASRF)
          ENDDO
        ENDDO
      ENDIF
    ENDIF
  ENDIF

ENDIF

! -----------------------------------------------------------------------------

!*      2. Case LDGDWI=T: computes "Gw" from -G.dw 
!       ------------------------------------------

! If LLRWSDLR2=T, the input variable is -G.d((r**2/a**2)w) and the output
!  variable is "G(r**2/a**2)w".
! If LLRWSDLR2=F, the input variable is -G.dw and the output variable is "Gw".
! In a LRWSDLR2=T job:
! * call this part with LDRWSDLR2=F: -G.dw -> Gw
! * call this part with LDRWSDLR2=T: -G.d((r**2/a**2)w) -> G(r**2/a**2)w

IF (LDGDWI) THEN

  ! * Copy "-G.dw" (-G.d((r**2/a**2)w) if LLRWSDLR2=T).
  ZGDW(KSTART:KEND,1:KFLEV)=PDVER(KSTART:KEND,1:KFLEV)

  ! * Compute "Gw" (or "G(r**2/a**2)w" if LLRWSDLR2=T) at the surface
  !   (free slip boundary condition)
  DO JROF=KSTART,KEND
    ZGWS(JROF)=PUS(JROF)*POROGL(JROF)+PVS(JROF)*POROGM(JROF)
    PGWH(JROF,KFLEV)=ZGWS(JROF)
  ENDDO
  IF(LLRWSDLR) THEN
    ! Take account to the fact that POROG(L,M) contain (r_surf/a)*nabla(Phi_s)
    ! and that ZGWS=V_surf.nabla(Phi_s): additional multiplication by (a/r_surf).
    DO JROF=KSTART,KEND
      ZGWS(JROF)=ZGWS(JROF)*PRDLR(JROF,KFLEV,JP_ASRH)
      PGWH(JROF,KFLEV)=ZGWS(JROF)
    ENDDO
  ENDIF
  IF(LLRWSDLR.AND.LLRWSDLR2) THEN
    ! "Gw" -> G(r**2/a**2)w
    ! Vertical integration provides G(r**2/a**2)w, and this is
    ! "G(r**2/a**2)w" at the surface which must be computed.
    DO JROF=KSTART,KEND
      ZGWS(JROF)=ZGWS(JROF)*PRDLR(JROF,KFLEV,JP_RSAH)*PRDLR(JROF,KFLEV,JP_RSAH)
      PGWH(JROF,KFLEV)=ZGWS(JROF)
    ENDDO
  ENDIF

  IF (LLVFE) THEN

    ! * Store "G dw" at full levels.
    IF (PRESENT(PGDW)) PGDW(KSTART:KEND,1:KFLEV)=-ZGDW(KSTART:KEND,1:KFLEV)

    ! * Compute "Gw" at full levels.

    DO JLEV=1,KFLEV
      DO JROF=KSTART,KEND
        ZGDW(JROF,JLEV)=-ZGDW(JROF,JLEV)*VFE_RDETAH(JLEV)
      ENDDO
    ENDDO

    CALL VERINT(KPROMA,KSTART,KEND,KFLEV,ZGDW,ZGWH,1)

    DO JLEV=1,KFLEV
      DO JROF=KSTART,KEND
        PGWF(JROF,JLEV)=ZGWH(JROF,JLEV-1)+ZGWS(JROF)
      ENDDO
    ENDDO

  ELSE

    ! * Compute "Gw" (G(r**2/a**2)w if LLRWSDLR2=T) at half levels.

    ! transform -G.dw into Gw (-G.(d(r**2/a**2)w) -> G(r**2/a**2)w if LLRWSDLR2=T)
    DO JLEV=KFLEV,1,-1
      DO JROF=KSTART,KEND
        PGWH(JROF,JLEV-1)=PGWH(JROF,JLEV)+ZGDW(JROF,JLEV)
      ENDDO
    ENDDO

    ! * Compute "Gw" (G(r**2/a**2)w if LLRWSDLR2=T) at full levels.

    IF (LDGWF) THEN
      ! k.y.: formula pgwf(jlev)=pgwh(jlev)(1-palph(jlev)/plnpr(jlev))
      ! +pgwh(jlev-1)(palph(jlev)/plnpr(jlev)) must be valid.
      CALL ABOR1(' GPGW: compute "Gw" at full levels: case not coded')
    ENDIF

  ENDIF ! test on LLVFE

ENDIF

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPGW',1,ZHOOK_HANDLE)

END SUBROUTINE GPGW

