SUBROUTINE GPHLUVTL(&
 ! --- INPUT -----------------------------------------------------------------
 & KPROMA,KSTART,KPROF,PUT5,PVT5,PWW5,PWW0,PUT0,PVT0,&
 ! --- OUTPUT ----------------------------------------------------------------
 & PLHU0,PLHV0)

!**** *GPHLUVTL* - wind components calculation in half-levels (tangent linear)

!     Purpose.
!     --------
!           Compute wind components in half-levels
!           for non-hydrostatic dynamics in tangent linear model

!**   Interface.
!     ----------
!        *CALL* *GPHLUVTL()

!        Explicit arguments :
!        --------------------

!         See GPHLUV for comments.

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        Interpolation from full-levels using logaritmic pressure profile
!        Then modify values on the top and bottom using boundary condition
!        (at the bottom free-slip condition)
!        Store also the weights of vertical interpolation

!     Externals.
!     ----------
!     Reference.
!     ----------
!        ARPEGE/ALADIN documentation

!     Author.
!     -------
!        Martin Charron,  CNRM/GMAP - visiteur

! Modifications
! -------------
!   Original : September 1998
!   Modified 02-07-02 by C. Fischer : remove computation of trajectory
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   08-Jun-2004 J. Masek   NH cleaning (LVSLWBC)
!   21-Mar-2006 K. Yessad  debugging, rewrite according to the direct code.
!   07-Mar-2007 K. Yessad  Remove LVERAVE_HLUV.
!   K. Yessad (Dec 2008): remove dummy CDLOCK
! End Modifications
!---------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCVER  , ONLY : LVERTFE
USE YOMDIM   , ONLY : NFLEVG
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT5(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT5(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWW5(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWW0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLHU0(KPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLHV0(KPROMA,0:NFLEVG) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPHLUVTL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF (LVERTFE) THEN
  ! * case LVERTFE not yet coded.
  !   For lvertfe=true no half level wind must be computed, the code must use
  !    only full level winds and the proper discretisation of "dV/d eta"
  !    has to be coded.
  CALL ABOR1(' GPHLUVTL: case LVERTFE not yet coded!')
ENDIF

!     ------------------------------------------------------------------

!*    1. General case:

DO JLEV=1, NFLEVG - 1
  DO JROF=KSTART,KPROF
    PLHU0(JROF,JLEV) = PWW5(JROF,JLEV)*PUT0(JROF,JLEV)+&
     & (1.0_JPRB-PWW5(JROF,JLEV))*PUT0(JROF,JLEV+1)&
     & +(PUT5(JROF,JLEV)-PUT5(JROF,JLEV+1))*PWW0(JROF,JLEV)
    PLHV0(JROF,JLEV) = PWW5(JROF,JLEV)*PVT0(JROF,JLEV)+&
     & (1.0_JPRB-PWW5(JROF,JLEV))*PVT0(JROF,JLEV+1)&
     & +(PVT5(JROF,JLEV)-PVT5(JROF,JLEV+1))*PWW0(JROF,JLEV)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*    2. Top.

PLHU0(KSTART:KPROF,0) = PUT0(KSTART:KPROF,1)
PLHV0(KSTART:KPROF,0) = PVT0(KSTART:KPROF,1)

!     ------------------------------------------------------------------

!*    3. Bottom.

PLHU0(KSTART:KPROF,NFLEVG) = PUT0(KSTART:KPROF,NFLEVG)
PLHV0(KSTART:KPROF,NFLEVG) = PVT0(KSTART:KPROF,NFLEVG)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPHLUVTL',1,ZHOOK_HANDLE)
END SUBROUTINE GPHLUVTL
