SUBROUTINE GPHLWI(KPROMA,KSTART,KPROF,PLNPR,PALPH,PWW)

!**** *GPHLWI* - to half-levels interpolation weights

!     Purpose.
!     --------
!           Compute weights for interpolation of winds to
!           half levels (in non-hydrostatic dynamics)

!**   Interface.
!     ----------
!        *CALL* *GPHLWI()

!        Explicit arguments :
!        --------------------
!         INPUT:
!          KPROMA  - dimensioning.
!          KSTART  - start of work.
!          KPROF   - depth of work.
!          PLNPR   - "delta" (log(prehyd) depth) at full levels.
!          PALPH   - "alpha" at full levels.

!         OUTPUT:
!          PWW     - vertical weight.

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        Interpolation from full-levels using logaritmic pressure profile
!        Then modify values on the top and bottom using boundary condition
!        (at the bottom free-slip condition)
!        Store the weights of vertical interpolation

!     Externals.
!     ----------
!     Reference.
!     ----------
!        ARPEGE/ALADIN documentation

!     Author.
!     -------
!        Radmila Bubnova,  CNRM/GMAP/EXT

!     Modifications.
!     --------------
!        Original : November 1997
!        Modified 02-07-02 by C. Fischer : remove pww5 computation + intents
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.Yessad      20-Mar-2006 Cleaning + optimisation
!        K. Yessad (Dec 2008): remove dummy CDLOCK
!     ----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWW(KPROMA,NFLEVG) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPHLWI',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

DO JLEV=1, NFLEVG - 1
  DO JROF=KSTART,KPROF
    PWW(JROF,JLEV)=(PLNPR(JROF,JLEV+1)-PALPH(JROF,JLEV+1))&
     & /(PLNPR(JROF,JLEV+1)-PALPH(JROF,JLEV+1)+PALPH(JROF,JLEV))  
  ENDDO
ENDDO

!     ------------------------------------------------------------------

DO JROF=KSTART,KPROF
  !* Bottom half level: 
  PWW(JROF,NFLEVG) = 0.0_JPRB
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPHLWI',1,ZHOOK_HANDLE)
END SUBROUTINE GPHLWI
