SUBROUTINE GPNOX(KPROMA,KSTART,KPROF,KFLEV,PNO2,PT,PO3,PNOX,PSOL,PJNO2,PRXP)

!**** *GPNOX* - Computes NO2 on model levels

!     Purpose.
!     --------
!           Computes NO2 on model levels

!**   Interface.
!     ----------
!        *CALL* *GPNOX(KPROMA,KSTART,KPROF,KFLEV,PNO2,PT,PO3,PNOX,PSOL,PJNO2,PRXP)

!        Explicit arguments :
!        --------------------
!        KPROMA                     - HORIZ. DIMENSIONING      (INPUT)
!        KSTART                     - START OF WORK            (INPUT)
!        KPROF                      - DEPTH OF WORK            (INPUT)
!        KFLEV                      - NUMBER OF LEVELS         (INPUT)
!        PNO2 (KPROMA,KFLEV)       - NO2                      (OUTPUT)
!        PT  (KPROMA,KFLEV)         - TEMPERATURE              (INPUT)
!        PO3 (KPROMA,KFLEV)         - OZONE                    (INPUT)
!        PNOX(KPROMA,KFLEV)         - NOX                      (INPUT)
!        PSOL (KPROMA)              - solar elevation          (INPUT)
!        PJNO2(KPROMA,KFLEV)        - NO2 PHOTOLYSIS FREQ      (INPUT)
!        PRXP(KPROMA,0:KFLEV,NPPM)  - HALF,FULL AND LN HALF LEVEL
!                                     PRESSURES (SEE PPINIT)   (INPUT)

!        Implicit arguments :    None.
!        --------------------

!     Method.
!     -------
!        Calculate NO2 from NOx by using:
!        [NO2]/[NO] = k* [O3]/J(NO2)
!        k=1.7*10-12exp(1310/T)cm3s-1
!        JNO2 calculated using program from TM5

!     Externals.   None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Erik Andersson, based on PP-routines by
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-06-08
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Dethof      04-Jan-2007 Adapt for NOx operator
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
USE YOMCST   , ONLY : RG, RD, RNAVO,RPI
USE YOMANCS  , ONLY : RMDI

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNO2(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNOX(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOL(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PJNO2(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP(KPROMA,0:KFLEV,NPPM) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JL, JLEV

!antje original values REAL(KIND=JPRB) , PARAMETER :: ZC1=1.7E-12_JPRB,ZC2=-1310.0_JPRB
REAL(KIND=JPRB) , PARAMETER :: ZC1=3.0E-12_JPRB,ZC2=-1500.0_JPRB   !From Sander et al. (2003)
REAL(KIND=JPRB) , PARAMETER :: ZC3=4.5E19_JPRB
REAL(KIND=JPRB) :: ZRATIO,ZC4,ZRHO
REAL(KIND=JPRB) :: ZSZA(KPROMA),ZSZA_H(KPROMA),ZSOL(KPROMA)
REAL(KIND=JPRB) :: ZZN,ZZK,ZO3mol,ZZB
REAL(KIND=JPRB) :: ZRGI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPNOX',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    COMPUTES HALF LEVEL NO2 columns
!              --------------------------------

!*       1.1   Convert solar elevation to SZA
ZSZA(:)=0._JPRB
DO JL=KSTART,KPROF
  ZSOL(JL) = PSOL(JL)
  IF (ZSOL(JL) == RMDI) ZSOL(JL)=45.0_JPRB  !Set missing SOE values to 45 deg
  ZSZA_H(JL)=COS(RPI/180.0_JPRB*(90._JPRB-ZSOL(JL)))
  ZSZA(JL) = MAX(0.01_JPRB,ZSZA_H(JL))
ENDDO

ZRGI=1.0_JPRB/RG

ZC4= RNAVO / (1000000.0_JPRB * 0.048_JPRB)

DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF
    IF (ZSZA(JL) > 0.01_JPRB) THEN

! Determine ZRATIO=NO2/NOx
      ZZK=ZC1*EXP(ZC2/PT(JL,JLEV))
      ZZN=PJNO2(JL,JLEV)/ZZK
      ZRHO = PRXP(JL,JLEV,2)/(RD*PT(JL,JLEV))   !full level press
!      ZO3mol=PO3(JL,JLEV)*ZRHO*ZC4
! ZO3mol include param of effect of peroxyradicals 
       ZO3mol = (PO3(JL,JLEV) +0.5 * ZSZA(JL) * 1.3E-7 )* ZRHO * ZC4
      ZZB=1.0_JPRB + ZZN/ZO3mol
      ZRATIO=1.0_JPRB/ZZB

      PNO2(JL,JLEV)=PNOX(JL,JLEV)*ZRATIO

    ELSE
      PNO2(JL,JLEV)=PNOX(JL,JLEV)
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPNOX',1,ZHOOK_HANDLE)
END SUBROUTINE GPNOX
