SUBROUTINE GPNOXTL(KPROMA,KSTART,KPROF,KFLEV,PNO2,PT,PO3,PNOX,PSOL,PJNO2,PRXP,PT5,PO35,PNOX5,PJNO25,PRXP5)

!**** *GPNOXTL* - Computes half level NO2 columns.

!     Purpose.
!     --------
!           Computes half level NO2 columns, tangent linear.

!**   Interface.
!     ----------
!        *CALL* *GPNOXTL(KPROMA,KSTART,KPROF,KFLEV,PNO2,PT,PO3,PNOX,PJNO2,PRXP
!                                              ,PT5,PO35,PNOX5,PJNO25,PRXP5)

!        Explicit arguments :
!        --------------------
!        KPROMA                      - HORIZ. DIMENSIONING
!        KSTART                      - START OF WORK
!        KPROF                       - DEPTH OF WORK
!        KFLEV                       - NUMBER OF LEVELS
!        PNO2 (KPROMA,KFLEV)        - NO2 PERTURBATION          (OUTPUT)
!        PT  (KPROMA,KFLEV)          - TEMPERATURE PERTURBATION  (INPUT)
!        PO3 (KPROMA,KFLEV)          - OZONE PERTURBATION        (INPUT)
!        PNOX  (KPROMA,KFLEV)        - NOX PERTURBATION          (INPUT)
!        PSOL (KPROMA)               - solar elevation           (INPUT)
!        PJNO2(KPROMA,KFLEV)         - NO2 PHOTOLYSIS FREQ PERT  (INPUT)
!        PRXP(KPROMA,0:KFLEV,NPPM)   - HALF,FULL AND LN HALF LEVEL
!                                      PRESSURES (SEE PPINIT)    (INPUT)
!        PT5 (KPROMA,KFLEV)          - TEMPERATURE               (INPUT)
!        PO35 (KPROMA,KFLEV)         - OZONE                     (INPUT)
!        PNOX5 (KPROMA,KFLEV)        - NOX                       (INPUT)
!        PJNO25(KPROMA,KFLEV)        - NO2 PHOTOLYSIS FREQ       (INPUT)
!        PRXP5(KPROMA,0:KFLEV,NPPM)  - HALF,FULL AND LN HALF LEVEL
!                                     PRESSURES (SEE PPINIT)   (INPUT)

!        Implicit arguments :    None.
!        --------------------

!     Method.
!     -------
!        See documentation.

!     Externals.   None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Erik Andersson, based on PP-routines by
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-06-08
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Dethof      04-Jan-2007 Adapt for NOx operator
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
USE YOMCST   , ONLY : RG, RD, RNAVO,RPI
USE YOMANCS  , ONLY : RMDI

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNO2(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO3(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNOX(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOL(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PJNO2(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PO35(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNOX5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PJNO25(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP5(KPROMA,0:KFLEV,NPPM) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JL, JLEV

REAL(KIND=JPRB)  :: ZNO25(KPROMA,KFLEV) 
REAL(KIND=JPRB) , PARAMETER :: ZC1=1.7E-12_JPRB,ZC2=-1310.0_JPRB
REAL(KIND=JPRB) , PARAMETER :: ZC3=4.5E19_JPRB

REAL(KIND=JPRB) :: ZZK,ZZK5,ZRATIO,ZRATIO5,ZZB,ZZB5
REAL(KIND=JPRB) :: ZSZA(KPROMA),ZSZA_H(KPROMA),ZSOL(KPROMA)
REAL(KIND=JPRB) :: ZC4,ZRHO,ZRHO5
REAL(KIND=JPRB) :: ZRGI,ZO3mol,ZO3mol5,ZZN,ZZN5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPNOXTL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    COMPUTES HALF LEVEL NO2 columns
!              --------------------------------

!*       1.1   Convert solar elevation to SZA
DO JL=KSTART,KPROF
  ZSOL(JL) = PSOL(JL)
  IF (ZSOL(JL) == RMDI) ZSOL(JL)=45.0_JPRB  !Set missing SOE values to 45 deg
  ZSZA_H(JL)=COS(RPI/180.0_JPRB*(90._JPRB-ZSOL(JL)))
  ZSZA(JL) = MAX(0.01_JPRB,ZSZA_H(JL))
ENDDO

ZRGI=1.0_JPRB/RG
ZC4= RNAVO / (1000000.0_JPRB * 0.048_JPRB)

DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF
    IF (ZSZA(JL) > 0.01_JPRB) THEN

      ZZK5=ZC1*EXP(ZC2/PT5(JL,JLEV))
      ZZK=-ZC2*PT(JL,JLEV)/PT5(JL,JLEV)**2*ZC1*EXP(ZC2/PT5(JL,JLEV))

      ZZN5=PJNO25(JL,JLEV)/ZZK5
      ZZN=(PJNO2(JL,JLEV)*ZZK5-ZZK*PJNO25(JL,JLEV))/ZZK5**2

      ZRHO5 = PRXP5(JL,JLEV,2)/(RD*PT5(JL,JLEV)) !full level press
      ZRHO = (PRXP(JL,JLEV,2)*PT5(JL,JLEV) - &
        &      PRXP5(JL,JLEV,2)*PT(JL,JLEV))/PT5(JL,JLEV)**2/RD   

!      ZO3mol5=PO35(JL,JLEV)*ZRHO5*ZC4
!      ZO3mol=(PO3(JL,JLEV)*ZRHO5 + PO35(JL,JLEV)*ZRHO) * ZC4
      ZO3mol5=(PO35(JL,JLEV)+0.5 * ZSZA(JL) * 1.3E-7 )*ZRHO5*ZC4
      ZO3mol=PO3(JL,JLEV)*ZRHO5*ZC4 + (PO35(JL,JLEV)+0.5*ZSZA(JL)*1.3E-7)*ZRHO*ZC4

      ZZB5=1.0_JPRB + ZZN5/ZO3mol5
      ZZB= (ZZN*ZO3mol5 - ZZN5*ZO3mol)/ZO3mol5**2

      ZRATIO5=1.0_JPRB/ZZB5
      ZRATIO=-ZZB/ZZB5**2

      ZNO25(JL,JLEV)=PNOX5(JL,JLEV)*ZRATIO5
      PNO2(JL,JLEV)=PNOX5(JL,JLEV)*ZRATIO+PNOX(JL,JLEV)*ZRATIO5
    
    ELSE
      ZNO25(JL,JLEV)=PNOX5(JL,JLEV)
      PNO2(JL,JLEV)=PNOX(JL,JLEV)
  
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPNOXTL',1,ZHOOK_HANDLE)
END SUBROUTINE GPNOXTL
