SUBROUTINE GPPREAD(KPROMA,KSTART,KPROF,KFLEV,PVBH,&
 & PRESH,PRESF,&
 & PRESH5)  

!**** *GPPREAD* - Computes half and full level pressure, adjoint

!     Purpose.
!     --------
!           Computes pressures at half and full model levels - adjoint.

!**   Interface.
!     ----------
!        *CALL* *GPPREAD(KPROMA,KSTART,KPROF,KFLEV,PVBH
!                       ,PRESH,PRESF
!                       ,PRESH5)

!        Explicit arguments :
!        --------------------
!                              KPROMA    : DIMENSIONING
!                              KSTART    : START OF THE PACKET.
!                              KPROF     : DEPTH OF WORK
!                              KFLEV     : VERT. DIMENSIONING
!                              PRESH(KPROMA,0:KFLEV) - HALF LEVEL PRESSURE
!                              PRESF(KPROMA,KFLEV)   - FULL LEVEL PRESSURE
!                              PRESH(KPROF,0:KFLEV) - HALF LEVEL PRESSURE
!                              PRESF(KPROF,KFLEV)   - FULL LEVEL PRESSURE

!                              PRESH5(KPROMA,0:KFLEV)- HALF LEVEL PRESSURE

!        Implicit arguments :  None.
!        --------------------

!     Method.
!     -------
!        See documentation
!     Half level P:     PHk = Ak + Bk * Ps

!                                PHk*ln(PHk) - PHk-1*ln(PHk-1)
!     Full level P: ln(PFk) = [ ------------------------------- - 1. ]
!                                        PHk - PHk-1

!     Externals.  None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Jean-Noel THEPAUT METEO FRANCE/EERM/CRMD

!     Modifications.
!     --------------
!        Original : 89-12-12
!        J.N Thepaut: 92-04-08 . loop indexes (IO scheme)
!     Erik Andersson  920326: Altered computation of full level pressure
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LAPRXPK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVBH(0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KPROMA,0:KFLEV) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZLNPH(KPROMA,0:KFLEV)

REAL(KIND=JPRB) :: ZLNPH5(KPROMA,0:KFLEV)

INTEGER(KIND=JPIM) :: JLEV, JLON

REAL(KIND=JPRB) :: ZLNPF, ZLNPF5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPPREAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF (.NOT.LAPRXPK) THEN
  DO JLEV=0,KFLEV
    DO JLON=KSTART,KPROF
      ZLNPH5(JLON,JLEV)=LOG(MAX(PRESH5(JLON,JLEV),0.1_JPRB))
    ENDDO
  ENDDO

  DO JLEV=0,KFLEV
    DO JLON=KSTART,KPROF
      ZLNPH(JLON,JLEV)=0.0_JPRB
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*       2.    COMPUTES FULL LEVEL PRESSURES.
!              ------------------------------

IF (.NOT.LAPRXPK) THEN
  DO JLEV=KFLEV,1,-1
    DO JLON=KSTART,KPROF
      ZLNPF5=(PRESH5(JLON,JLEV)*ZLNPH5(JLON,JLEV)&
       & -PRESH5(JLON,JLEV-1)*ZLNPH5(JLON,JLEV-1))&
       & /(PRESH5(JLON,JLEV)-PRESH5(JLON,JLEV-1))&
       & -1.0_JPRB  

      ZLNPF=0.0_JPRB
      ZLNPF=ZLNPF+EXP(ZLNPF5)*PRESF(JLON,JLEV)

      PRESH(JLON,JLEV)=PRESH(JLON,JLEV)&
       & +(ZLNPH5(JLON,JLEV)&
       & /(PRESH5(JLON,JLEV)-PRESH5(JLON,JLEV-1))&
       & -(PRESH5(JLON,JLEV)*ZLNPH5(JLON,JLEV)&
       & -PRESH5(JLON,JLEV-1)*ZLNPH5(JLON,JLEV-1))&
       & /(PRESH5(JLON,JLEV)-PRESH5(JLON,JLEV-1))**2)*ZLNPF  

      ZLNPH(JLON,JLEV)=ZLNPH(JLON,JLEV)&
       & +PRESH5(JLON,JLEV)&
       & /(PRESH5(JLON,JLEV)-PRESH5(JLON,JLEV-1))*ZLNPF  

      PRESH(JLON,JLEV-1)=PRESH(JLON,JLEV-1)&
       & +(-ZLNPH5(JLON,JLEV-1)&
       & /(PRESH5(JLON,JLEV)-PRESH5(JLON,JLEV-1))&
       & +(PRESH5(JLON,JLEV)*ZLNPH5(JLON,JLEV)&
       & -PRESH5(JLON,JLEV-1)*ZLNPH5(JLON,JLEV-1))&
       & /(PRESH5(JLON,JLEV)-PRESH5(JLON,JLEV-1))**2)*ZLNPF  

      ZLNPH(JLON,JLEV-1)=ZLNPH(JLON,JLEV-1)&
       & -PRESH5(JLON,JLEV-1)&
       & /(PRESH5(JLON,JLEV)-PRESH5(JLON,JLEV-1))*ZLNPF  

      PRESF(JLON,JLEV)=0.0_JPRB
    ENDDO
  ENDDO

  DO JLEV=0,KFLEV
    DO JLON=KSTART,KPROF
      IF(PRESH5(JLON,JLEV) >= 0.1_JPRB) THEN
        PRESH(JLON,JLEV)=PRESH(JLON,JLEV)+ZLNPH(JLON,JLEV)/PRESH5(JLON,JLEV)
      ELSE
        PRESH(JLON,JLEV)=PRESH(JLON,JLEV)+0.0_JPRB
      ENDIF
    ENDDO
  ENDDO

ELSE
  DO JLEV=KFLEV,1,-1
    DO JLON=KSTART,KPROF
      PRESH(JLON,JLEV-1)=PRESH(JLON,JLEV-1)+0.5_JPRB*PRESF(JLON,JLEV)
      PRESH(JLON,JLEV)=PRESH(JLON,JLEV)+0.5_JPRB*PRESF(JLON,JLEV)
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*       1.    COMPUTES HALF LEVEL PRESSURES.
!              ------------------------------

DO JLEV=0,KFLEV-1
  DO JLON=KSTART,KPROF
    PRESH(JLON,KFLEV)=PRESH(JLON,KFLEV)+PVBH(JLEV)*PRESH(JLON,JLEV)
    PRESH(JLON,JLEV)=0.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPPREAD',1,ZHOOK_HANDLE)
END SUBROUTINE GPPREAD
