!OCL  NOEVAL
SUBROUTINE GPPVO(KPROMA,KPROMVC,KSTART,KPROF,KFLEV,&
 & PRESF,PRDELP,PKAP,PRCORI,&
 & PVOR,PU,PV,PT,PTM,PTL,PSPM,PSPL,PVO,PTETA,&
 & LDVERCOR,LDRWSDLW,LDRWSDLR,PRCORIC,PVCRSSAF,PVCASRSF)  

!**** *GPPVO* - COMPUTES POTENTIAL VORTICITY AND POTENTIAL TEMPERATURE.

!     PURPOSE.
!     --------

!      Computes potential temperature and potential vorticity at model
!      full levels.
!      Computes the horizontal derivatives of the potential temperature,
!      and the absolute vorticity as intermediate variables.

!      Potential temperature "pteta" is defined by:

!       pteta = T (prehyd / 100000Pa)**(R/cp)

!      where:
!      - "T" is the temperature
!      - "prehyd" is the hydrostatic pressure
!      - "R" is the air constant (including moisture effects)
!      - "cp" is the constant pressure calorific capacity for moist air.

!      Discretisation at full level "l" of "pteta" follows:

!       pteta[l] = T[l] ( prehyd[l] / 100000Pa )**( R[l] / cp[l] )

!      Discretisation at full level "l" of "rssavnabla pteta" follows
!      (horizontal derivatives of (R/cp) are assumed to be negligible):

!       (rssavnabla pteta)[l] = pteta[l] * 
!       { (rssavnabla T)[l] / T[l]
!       + ( R[l] / cp[l] ) ( rssavnabla prehyd / prehyd )[l] }

!      Remark: the current code for ( rssavnabla prehyd / prehyd )[l] in part 2
!      gives a discretisation which is not consistent with the one used
!      elsewhere in the code (gradient pressure term, omega/prehyd)
!      which has normally to use the array 'prtgr' computed in 'gpxyb'
!      ( prtgr*(pspl;pspm) yields the good discretisation of vector
!      ( rssavnabla prehyd / prehyd ) on layers ). This has to changed in
!      the future.

!      Absolute vorticity "zeta_abs" is defined by:
!       zeta_abs = zeta + f
!      where "zeta" is the (geographic) relative vorticity and "f" is the
!      Coriolis parameter.
!      When relaxing the thin layer hypothesis this routine actually
!      enters the semi-reduced relative vorticity
!       (rs/a) zeta = M**2 zeta'
!      and the code computes the geographic quantity "zeta_abs" on layers,
!      which has be to coded as:
!       zeta_abs[l] = (a/rs)[l] { ((rs/a) zeta)[l] } + f
!      using the input data "(a/rs)[l]", "((rs/a) zeta)[l]" and "f"
!      ("rs" is the pseudo-radius and "a" is the mean Earth radius).

!      Potential vorticity "PV" is defined by:

!       PV = (a/rs)**2 g ( d ((rs/a)V) / d prehyd ) (rssavnabla pteta).vec(i)
!          - (a/rs)**2 g ( d ((rs/a)U) / d prehyd ) (rssavnabla pteta).vec(j)
!          + 2 Omega cos(theta) (a/rs) (rssavnabla pteta).vec(j)
!          - g zeta_abs ( d pteta / d prehyd )

!      where:
!       - "g" is the gravitation constant.
!       - "U,V" are the components of the horizontal wind. 
!       - "(vec(i),vec(j))" are the unit vectors of the local repere.
!       - "Omega" is the angular velocity of the Earth rotation.
!       - "theta" is the geographic latitude.

!      When the thin layer hypothesis is applied, (a/rs) and (rs/a) become 1,
!      and the "2 Omega cos(theta)" term disappears.

!      "PV" is discretised on layers.
!      Vertical derivatives ( d X / d prehyd ) are discretised at full level "l"
!      as follows (X = (rs/a)U; (rs/a)V or "pteta"):
!       ( d X / d prehyd )[l] = 0.5 (X[l+1]-X[l-1])/(Delta prehyd)[l]
!       ( d X / d prehyd )[l=1] = (X[l=2]-X[l=1])/(Delta prehyd)[l=1]
!       ( d X / d prehyd )[l=L] = (X[l=L]-X[l=L-1])/(Delta prehyd)[l=L]
!      This discretisation is always done in finite differences.
!      For X = (rs/a)U and (rs/a)V the choice which has been done is to
!      directly discretise vertically them, and not to compute separately
!      vertical derivatives for "U", "V" and "rs", using the relation
!       (d rs / d prehyd)= (Rd Ts)/(g prehyd)
!      (see routine "gpvcrs" for more explanation) and giving another
!      possible but less simple method to implement.
!      The other quantities involved in the RHS of "PV" expression are 
!      available as "layer" quantities (input data or intermediate data
!      computed in the parts 1 and 2 of this subroutine).

!**   INTERFACE.
!     ----------
!        *CALL* *GPPVO(...)

!        EXPLICIT ARGUMENTS :
!        --------------------
!        * INPUT:
!          KPROMA          : horizontal dimensioning
!          KPROMVC         : hor. dimension for arrays used only if "lvercor=T".
!          KSTART to KPROF : depth of work
!          KFLEV           : number of levels
!          PRESF           : hydrostatic pressure "prehyd" at full levels
!          PRDELP          : 1/[Delta prehyd] at full levels
!          PKAP            : "kappa=R/cp" at full levels
!                            (previously computed by GPRCP)
!          PRCORI          : Coriolis parameter "f=2 Omega sin(theta)"
!          PVOR            : semi-reduced relative vorticity
!                            "(rs/a) zeta = (M**2 zeta')" at full levels
!          PU              : U-component of the horizontal wind, at full levels
!          PV              : V-component of the horizontal wind, at full levels
!          PT              : temperature at full levels
!          PTM             : merid grad of temperature "(rssavnabla T).vec(j)"
!                            at full levels
!          PTL             : zonal grad of temperature "(rssavnabla T).vec(i)"
!                            at full levels
!          PSPM            : meridian gradient of surface hydrostatic pressure
!                            "(rssavnabla prehyds).vec(j)"
!          PSPL            : zonal gradient of surface hydrostatic pressure
!                            "(rssavnabla prehyds).vec(i)"

!        * OUTPUT:
!          PVO             : geographic potential vorticity "PV" at full levels
!          PTETA           : potential temperature "pteta" at full levels

!        * OPTIONAL INPUT:
!          LDVERCOR        : key for White and Bromley deep-layer eqns.
!          LDRWSDLW        : key for Wood and Staniforth deep-layer eqns (w)
!          LDRWSDLR        : key for Wood and Staniforth deep-layer eqns (metr)
!          PRCORIC         : "2 Omega cos(theta)"
!          PVCRSSAF        : "rs/a" at full levels (LVERCOR=T)
!          PVCASRSF        : "a/rs" at full levels (LVERCOR=T)

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST,YOMGEM.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!       None.
!       Called by CPG and POS.

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!   ORIGINAL : 88-02-04
!   Modified by K. YESSAD (METEO-FRANCE CNRM/GMAP) 03/03/92.
!   Modification by K. YESSAD (AUGUST 2000):
!    - relaxation of thin layer hypothesis.
!    - correction of some false or ambiguous comments, add comments.
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   K. Yessad (Dec 2008): remove useless dummy arguments
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RATM
USE YOMGEM   , ONLY : VBH

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMVC 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAP(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVO(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTETA(KPROMA,KFLEV) 
LOGICAL,OPTIONAL  ,INTENT(IN)    :: LDVERCOR 
LOGICAL,OPTIONAL  ,INTENT(IN)    :: LDRWSDLW
LOGICAL,OPTIONAL  ,INTENT(IN)    :: LDRWSDLR
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PRCORIC(KPROMA) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PVCRSSAF(KPROMVC,KFLEV) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PVCASRSF(KPROMVC,KFLEV) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZTETAL(KPROMA,KFLEV),ZTETAM(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZPRESL(KPROMA,KFLEV),ZPRESM(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZUZ(KPROMA,KFLEV),ZVZ(KPROMA,KFLEV),ZTETAZ(KPROMA,KFLEV)

INTEGER(KIND=JPIM) :: JLEV, JROF
LOGICAL :: LLVERCOR,LLRWSDLW,LLRWSDLR

REAL(KIND=JPRB) :: ZGS2, ZGS2DP, ZGSDPB, ZGSDPT, ZKSPRES, ZUSPT, ZUSRATM, ZVBF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPPVO',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY CALCULATIONS.
!              -------------------------

LLVERCOR=.FALSE.
LLRWSDLW=.FALSE.
LLRWSDLR=.FALSE.
IF (PRESENT(LDVERCOR)) LLVERCOR=LDVERCOR
IF (PRESENT(LDRWSDLW)) LLRWSDLW=LDRWSDLW
IF (PRESENT(LDRWSDLR)) LLRWSDLR=LDRWSDLR

IF ((LLVERCOR.OR.LLRWSDLW).AND.(.NOT.PRESENT(PRCORIC))) THEN
  CALL ABOR1(' GPPVO 1a: Missing input optional PRCORIC')
ENDIF
IF (LLVERCOR.AND.(.NOT.PRESENT(PVCRSSAF))) THEN
  CALL ABOR1(' GPPVO 1b: Missing input optional PVCRSSAF')
ENDIF
IF (LLVERCOR.AND.(.NOT.PRESENT(PVCASRSF))) THEN
  CALL ABOR1(' GPPVO 1c: Missing input optional PVCASRSF')
ENDIF
IF (LLRWSDLR) THEN
  ! ky: for the time being GPPVO is called only in POS, in a "thin layer"
  !     piece of code.
  CALL ABOR1(' GPPVO 1d: LRWSDLR code not yet provided')
ENDIF

!     ------------------------------------------------------------------

!*       2.    COMPUTES PRESSURE W-E AND N-S DERIVATIVES.
!              ------------------------------------------

DO JLEV=1,KFLEV
  ZVBF=(VBH(JLEV-1)+VBH(JLEV))*0.5_JPRB
  DO JROF=KSTART,KPROF
    ZPRESM(JROF,JLEV) = ZVBF*PSPM(JROF)
    ZPRESL(JROF,JLEV) = ZVBF*PSPL(JROF)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*       3. COMPUTES POTENTIAL TEMPERATURE AND ITS W-E AND N-S DERIVATIVE.
!           --------------------------------------------------------------

ZUSRATM=1.0_JPRB/RATM
DO JLEV=1,KFLEV
  DO JROF=KSTART,KPROF
    ZUSPT  = 1.0_JPRB/PT(JROF,JLEV)
    ZKSPRES= PKAP(JROF,JLEV)/PRESF(JROF,JLEV)
    PTETA(JROF,JLEV)=&
     & +PT(JROF,JLEV)*(PRESF(JROF,JLEV)*ZUSRATM)**(-PKAP(JROF,JLEV))  
    ZTETAM(JROF,JLEV)=PTETA(JROF,JLEV)*&
     & (PTM(JROF,JLEV)*ZUSPT-ZKSPRES*ZPRESM(JROF,JLEV))  
    ZTETAL(JROF,JLEV)=PTETA(JROF,JLEV)*&
     & (PTL(JROF,JLEV)*ZUSPT-ZKSPRES*ZPRESL(JROF,JLEV))  
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*       4.    COMPUTES VERTICAL GRADIENTS.
!              ----------------------------

!     --- LEVELS 2 TO KFLEV-1 ----

ZGS2 = RG*0.5_JPRB

IF (LLVERCOR) THEN
  DO JLEV=2,KFLEV-1
    DO JROF=KSTART,KPROF
      ZGS2DP=ZGS2*PRDELP(JROF,JLEV)
      ZUZ(JROF,JLEV)=(PVCRSSAF(JROF,JLEV+1)*PU(JROF,JLEV+1)&
       & -PVCRSSAF(JROF,JLEV-1)*PU(JROF,JLEV-1))*ZGS2DP  
      ZVZ(JROF,JLEV)=(PVCRSSAF(JROF,JLEV+1)*PV(JROF,JLEV+1)&
       & -PVCRSSAF(JROF,JLEV-1)*PV(JROF,JLEV-1))*ZGS2DP  
      ZTETAZ(JROF,JLEV)=(PTETA(JROF,JLEV+1)-PTETA(JROF,JLEV-1))*ZGS2DP
    ENDDO
  ENDDO
ELSE
  DO JLEV=2,KFLEV-1
    DO JROF=KSTART,KPROF
      ZGS2DP=ZGS2*PRDELP(JROF,JLEV)
      ZUZ   (JROF,JLEV)=(PU   (JROF,JLEV+1)-PU   (JROF,JLEV-1))*ZGS2DP
      ZVZ   (JROF,JLEV)=(PV   (JROF,JLEV+1)-PV   (JROF,JLEV-1))*ZGS2DP
      ZTETAZ(JROF,JLEV)=(PTETA(JROF,JLEV+1)-PTETA(JROF,JLEV-1))*ZGS2DP
    ENDDO
  ENDDO
ENDIF

!     --- LEVELS 1 AND KFLEV ----

IF (LLVERCOR) THEN
  DO JROF=KSTART,KPROF
    ZGSDPT=RG*PRDELP(JROF,1)
    ZUZ(JROF,1)=(PVCRSSAF(JROF,2)*PU(JROF,2)&
     & -PVCRSSAF(JROF,1)*PU(JROF,1))*ZGSDPT  
    ZVZ(JROF,1)=(PVCRSSAF(JROF,2)*PV(JROF,2)&
     & -PVCRSSAF(JROF,1)*PV(JROF,1))*ZGSDPT  
    ZTETAZ(JROF,1)=(PTETA(JROF,2)-PTETA(JROF,1))*ZGSDPT
    ZGSDPB=RG*PRDELP(JROF,KFLEV)
    ZUZ(JROF,KFLEV)=(PVCRSSAF(JROF,KFLEV)*PU(JROF,KFLEV)&
     & -PVCRSSAF(JROF,KFLEV-1)*PU(JROF,KFLEV-1))*ZGSDPB  
    ZVZ(JROF,KFLEV)=(PVCRSSAF(JROF,KFLEV)*PV(JROF,KFLEV)&
     & -PVCRSSAF(JROF,KFLEV-1)*PV(JROF,KFLEV-1))*ZGSDPB  
    ZTETAZ(JROF,KFLEV)=(PTETA(JROF,KFLEV)-PTETA(JROF,KFLEV-1))*ZGSDPB
  ENDDO
ELSE
  DO JROF=KSTART,KPROF
    ZGSDPT=RG*PRDELP(JROF,1)
    ZUZ   (JROF,1)=(PU   (JROF,2)-PU   (JROF,1))*ZGSDPT
    ZVZ   (JROF,1)=(PV   (JROF,2)-PV   (JROF,1))*ZGSDPT
    ZTETAZ(JROF,1)=(PTETA(JROF,2)-PTETA(JROF,1))*ZGSDPT
    ZGSDPB=RG*PRDELP(JROF,KFLEV)
    ZUZ   (JROF,KFLEV)=(PU   (JROF,KFLEV)-PU   (JROF,KFLEV-1))*ZGSDPB
    ZVZ   (JROF,KFLEV)=(PV   (JROF,KFLEV)-PV   (JROF,KFLEV-1))*ZGSDPB
    ZTETAZ(JROF,KFLEV)=(PTETA(JROF,KFLEV)-PTETA(JROF,KFLEV-1))*ZGSDPB
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*       5.    COMPUTES POTENTIAL VORTICITY.
!              -----------------------------

IF (LLVERCOR) THEN
  DO JLEV=1,KFLEV
    DO JROF=KSTART,KPROF
      PVO(JROF,JLEV)=&
       & -(PVCASRSF(JROF,JLEV)*PVOR(JROF,JLEV)+PRCORI(JROF))*ZTETAZ(JROF,JLEV)&
       & +PVCASRSF(JROF,JLEV)*PVCASRSF(JROF,JLEV)&
       & *ZVZ(JROF,JLEV)*ZTETAL(JROF,JLEV)&
       & -PVCASRSF(JROF,JLEV)*PVCASRSF(JROF,JLEV)&
       & *ZUZ(JROF,JLEV)*ZTETAM(JROF,JLEV)
    ENDDO
  ENDDO
ELSE
  DO JLEV=1,KFLEV
    DO JROF=KSTART,KPROF
      PVO(JROF,JLEV)=&
       & -(PVOR(JROF,JLEV)+PRCORI(JROF))*ZTETAZ(JROF,JLEV)&
       & +ZVZ(JROF,JLEV)*ZTETAL(JROF,JLEV)&
       & -ZUZ(JROF,JLEV)*ZTETAM(JROF,JLEV)  
    ENDDO
  ENDDO
ENDIF

! * Add 'RCORIC' term when relevant:
IF (LLVERCOR) THEN
  DO JLEV=1,KFLEV
    DO JROF=KSTART,KPROF
      PVO(JROF,JLEV)=PVO(JROF,JLEV) &
       & +PRCORIC(JROF)*PVCASRSF(JROF,JLEV)*ZTETAM(JROF,JLEV)
    ENDDO
  ENDDO
ELSEIF (LLRWSDLW.AND.(.NOT.LLRWSDLR)) THEN
  DO JLEV=1,KFLEV
    DO JROF=KSTART,KPROF
      PVO(JROF,JLEV)=PVO(JROF,JLEV)+PRCORIC(JROF)*ZTETAM(JROF,JLEV)
    ENDDO
  ENDDO
ELSEIF (LLRWSDLW.AND.LLRWSDLR) THEN
  ! Not yet coded.
  ! PVO=PVO+PRCORIC*(a/r)*ZTETAM
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPPVO',1,ZHOOK_HANDLE)
END SUBROUTINE GPPVO
