SUBROUTINE GPPWCAD(KPROMA,KSTART,KPROF,KFLEV,PWCH,PQ,PRXP,PQ5,PRXP5)

!**** *GPPWCAD* - Computes half level PWC.

!     Purpose.
!     --------
!           Computes half level PWC, adjoint.

!**   Interface.
!     ----------
!        *CALL* *GPPWCAD(KPROMA,KSTART,KPROF,KFLEV,PWCH,PQ,PRXP
!                                              ,PQ5,PRXP5)

!        Explicit arguments :
!        --------------------
!        KPROMA                      - HORIZ. DIMENSIONING        (INPUT-C)
!        KSTART                      - START OF WORK              (INPUT-C)
!        KPROF                       - DEPTH OF WORK              (INPUT-C)
!        KFLEV                       - NUMBER OF LEVELS           (INPUT-C)
!        PWCH(KPROMA,0:KFLEV)        - HALF LEVEL PWC             (OUTPUT)
!        PQ  (KPROMA,KFLEV)          - HUMIDITY                   (INPUT)
!        PRXP(KPROMA,0:KFLEV,NPPM)  - HALF,FULL AND LN HALF LEVEL
!                                     PRESSURES (SEE PPINIT)   (INPUT)
!        PQ5 (KPROMA,KFLEV)          - HUMIDITY                   (INPUT-T)
!        PRXP5(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF LEVEL
!                                     PRESSURES (SEE PPINIT)   (INPUT)

!        Implicit arguments :    None.
!        --------------------

!     Method.
!     -------
!        See documentation.

!     Externals.   None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Erik Andersson, based on PP-routines by
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-06-08
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
USE YOMCST   , ONLY : RG

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWCH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP5(KPROMA,0:KFLEV,NPPM) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JL, JLEV

REAL(KIND=JPRB) :: ZDELP, ZDELP5, ZRGI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPPWCAD',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    COMPUTES HALF LEVEL PWC
!              -----------------------

ZRGI=1.0_JPRB/RG
DO JLEV=KFLEV,1,-1
  DO JL=KSTART,KPROF
    ZDELP5=PRXP5(JL,JLEV,1)-PRXP5(JL,JLEV-1,1)

    PQ(JL,JLEV)=PQ(JL,JLEV)+ZRGI*ZDELP5*PWCH(JL,JLEV)
    ZDELP=ZRGI*PQ5(JL,JLEV)*PWCH(JL,JLEV)
    PWCH(JL,JLEV-1)=PWCH(JL,JLEV-1)+PWCH(JL,JLEV)
    PWCH(JL,JLEV)=0.0_JPRB

    PRXP(JL,JLEV,1)=PRXP(JL,JLEV,1)+ZDELP
    PRXP(JL,JLEV-1,1)=PRXP(JL,JLEV-1,1)-ZDELP
  ENDDO
ENDDO
DO JL=KSTART,KPROF
  PWCH(JL,0)=0.0_JPRB
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPPWCAD',1,ZHOOK_HANDLE)
END SUBROUTINE GPPWCAD
