SUBROUTINE GPRH(LDWMORH,KPROMA,KSTART,KPROF,KFLEV,&
 & PRHMAX,PRHMIN,PQ,PT,PRESF,PES,PRH)  

!**** *GPRH* - COMPUTES ES AND RH FROM T AND Q

!     PURPOSE.  COMPUTES SATURATION VAPOUR PRESSURE AND RELATIVE
!     --------  HUMIDITY FROM TEMPERATURE AND SPECIFIC HUMIDITY.

!**   INTERFACE.
!     ----------
!        *CALL* *GPRH(..)*

!        EXPLICIT ARGUMENTS
!        --------------------

!        LDWMORH              - WMO CONSIDER WATER PHASE ONLY          (INPUT)
!        KPROMA               - HORIZONTAL DIMENSIONS.                 (INPUT)
!        KSTART               - START OF WORK                          (INPUT)
!        KPROF                - DEPTH OF WORK                          (INPUT)
!        KFLEV                - NUMBER OF MODEL LEVELS                 (INPUT)
!        PRHMAX               - MAXIMUM RELATIVE HUMIDITY              (INPUT)
!        PRHMIN               - MINIMUM RELATIVE HUMIDITY              (INPUT)
!        PQ(KPROMA,KFLEV)     - SPECIFIC HUMIDITY ON FULL MODEL LEVELS (INPUT)
!        PT(KPROMA,KFLEV)     - TEMPERATURE ON FULL MODEL LEVELS       (INPUT)
!        PRESF(KPROMA,KFLEV)  - MODEL FULL LEVEL PRESSURES             (INPUT)

!        PES(KPROMA,KFLEV)    - SATURATION PRESSURE                    (OUTPUT)
!        PRH(KPROMA,KFLEV)    - RELATIVE HUMIDITY                      (OUTPUT)

!        IMPLICIT ARGUMENTS :   PHYSICAL CONSTANTS FROM COMMONS YOMCST
!        --------------------   AND YOMPHY, FUNCTIONS FROM FCTTRM.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.     NONE.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 88-02-04
!     Per Unden : 1997-04-29  Use Tetens formula with new coefficients
!     E. Andersson  99-08-17  and Pedro, WMO consider water phase only.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RV       ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD  
USE YOMPHY   , ONLY : LNEIGE
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE   ,RTICECU,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOMCT0B  , ONLY : LECMWF
USE YOMDYNCORE, ONLY: LDYNCORE, LAQUA

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
LOGICAL           ,INTENT(IN)    :: LDWMORH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHMAX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHMIN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PES(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRH(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JL, JLEV

REAL(KIND=JPRB) :: ZDELTA, ZRH
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "fcttre.h"
#include "fcttrm.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPRH',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    COMPUTES ES AND RH.
!              -------------------

DO JLEV=1,KFLEV
  IF(LECMWF) THEN
    IF(LDWMORH) THEN
      DO JL=KSTART,KPROF
        PES(JL,JLEV)=(RETV+1.0_JPRB)*FOEEWMO(PT(JL,JLEV))
      ENDDO
    ELSE
      DO JL=KSTART,KPROF
        PES(JL,JLEV)=(RETV+1.0_JPRB)*FOEEWM(PT(JL,JLEV))
      ENDDO
    ENDIF
  ELSE
    IF(LNEIGE) THEN
      DO JL=KSTART,KPROF
        ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PT(JL,JLEV)))
        PES(JL,JLEV)=FOEW(PT(JL,JLEV),ZDELTA)
      ENDDO
    ELSE
      DO JL=KSTART,KPROF
        ZDELTA=0.0_JPRB
        PES(JL,JLEV)=FOEW(PT(JL,JLEV),ZDELTA)
      ENDDO
    ENDIF
  ENDIF

  IF( LDYNCORE.AND..NOT.LAQUA ) THEN
    DO JL=KSTART,KPROF
      PRH(JL,JLEV)=0.0_JPRB
    ENDDO
  ELSE
    DO JL=KSTART,KPROF
      ZRH=(PRESF(JL,JLEV)*PQ(JL,JLEV)*(RETV+1.0_JPRB))&
       & /((1.0_JPRB+RETV*PQ(JL,JLEV))*PES(JL,JLEV))  
      PRH(JL,JLEV)=MAX(PRHMIN,MIN(ZRH,PRHMAX))
    ENDDO
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPRH',1,ZHOOK_HANDLE)
END SUBROUTINE GPRH
