SUBROUTINE GPRHAD(LDWMORH,KPROMA,KSTART,KPROF,KFLEV,&
 & PRHMAX,PRHMIN,PQ,PT,PRESF,PES,PRH,&
 & PES5,PRESF5,PQ5,PT5)  

!**** *GPRHAD* - COMPUTES ES AND RH FROM T AND Q - ADJOINT

!     PURPOSE.  COMPUTES SATURATION VAPOUR PRESSURE AND RELATIVE
!     --------  HUMIDITY FROM TEMPERATURE AND SPECIFIC HUMIDITY.

!**   INTERFACE.
!     ----------
!        *CALL* *GPRHAD(...

!        EXPLICIT ARGUMENTS
!        --------------------

!        LDWMORH              - WMO CONSIDER WATER PHASE ONLY          (INPUT)
!        KPROMA               - HORIZONTAL DIMENSIONS.                 (INPUT-C)
!        KSTART               - START OF WORK.                         (INPUT-C)
!        KPROF                - DEPTH OF WORK                          (INPUT-C)
!        KFLEV                - NUMBER OF MODEL LEVELS                 (INPUT-C)
!        PRHMAX               - MAXIMUM RELATIVE HUMIDITY              (INPUT-C)
!        PRHMIN               - MINIMUM RELATIVE HUMIDITY              (INPUT-C)
!        PQ(KPROMA,KFLEV)     - SPECIFIC HUMIDITY ON FULL MODEL LEVELS (INPUT)
!        PT(KPROMA,KFLEV)     - TEMPERATURE ON FULL MODEL LEVELS       (INPUT)
!        PRESF(KPROMA,KFLEV)  - MODEL FULL LEVEL PRESSURES             (INPUT)

!        PES(KPROMA,KFLEV)    - SATURATION PRESSURE                    (OUTPUT)
!        PRH(KPROMA,KFLEV)    - RELATIVE HUMIDITY                      (OUTPUT)
!        PES5(KPROMA,KFLEV)    - SATURATION PRESSURE                  (INPUT-T)
!        PQ5(KPROMA,KFLEV)     - SPECIFIC HUMIDITY ON FULL MODEL LEVELS(INPUT-T)
!        PT5(KPROMA,KFLEV)     - TEMPERATURE ON FULL MODEL LEVELS      (INPUT-T)
!        PRESF5(KPROMA,KFLEV)  - MODEL FULL LEVEL PRESSURES            (INPUT-T)

!        IMPLICIT ARGUMENTS :   PHYSICAL CONSTANTS FROM COMMONS YOMCST
!        --------------------   AND YOMPHY, FUNCTIONS FROM FCTTRM,FCTTRMTL.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.     NONE.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-01-10
!     Per Unden : 1997-04-29  Use Tetens formula with new coefficients
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RV       ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD  
USE YOMPHY   , ONLY : LNEIGE
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE   ,RTICECU    ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOMCT0B  , ONLY : LECMWF

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
LOGICAL           ,INTENT(IN)    :: LDWMORH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHMAX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHMIN 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PES(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRH(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PES5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JL, JLEV

REAL(KIND=JPRB) :: ZALFAAD, ZDAL, ZDELTA, ZRH, ZRH5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "fcttrm.h"
#include "fcttrmad.h"
#include "fcttre.h"
#include "fcttread.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPRHAD',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    COMPUTES ES AND RH.
!              -------------------

DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF
    ZRH5=(PRESF5(JL,JLEV)*PQ5(JL,JLEV)*(RETV+1.0_JPRB))&
     & /((1.0_JPRB+RETV*PQ5(JL,JLEV))*PES5(JL,JLEV))  
    IF(ZRH5 > PRHMIN .AND. ZRH5 < PRHMAX) THEN
      ZRH=PRH(JL,JLEV)
    ELSE
      ZRH=0.0_JPRB
    ENDIF
    PRH(JL,JLEV)=0.0_JPRB
    PRESF(JL,JLEV)=PRESF(JL,JLEV)+PQ5(JL,JLEV)*(RETV+1.0_JPRB)&
     & /((1.0_JPRB+RETV*PQ5(JL,JLEV))*PES5(JL,JLEV))*ZRH  
    PQ(JL,JLEV)=PQ(JL,JLEV)+(PRESF5(JL,JLEV)*(RETV+1.0_JPRB))&
     & /((1.0_JPRB+RETV*PQ5(JL,JLEV))*PES5(JL,JLEV))*ZRH  
    PQ(JL,JLEV)=PQ(JL,JLEV)-(PRESF5(JL,JLEV)*PQ5(JL,JLEV)*&
     & (RETV+1.0_JPRB))/&
     & ((1.0_JPRB+RETV*PQ5(JL,JLEV))*PES5(JL,JLEV))**2*RETV*&
     & PES5(JL,JLEV)*ZRH  
    PES(JL,JLEV)=PES(JL,JLEV)-&
     & (PRESF5(JL,JLEV)*PQ5(JL,JLEV)*(RETV+1.0_JPRB))/&
     & ((1.0_JPRB+RETV*PQ5(JL,JLEV))*PES5(JL,JLEV))**2*&
     & (1.0_JPRB+RETV*PQ5(JL,JLEV))*ZRH  
  ENDDO
  IF(LECMWF) THEN
    IF(LDWMORH) THEN
      DO JL=KSTART,KPROF
        PT(JL,JLEV)=PT(JL,JLEV)+&
         & (RETV+1.0_JPRB)*FOEEWMOAD(PES(JL,JLEV),PT5(JL,JLEV))  
      ENDDO
    ELSE
      DO JL=KSTART,KPROF
        ZDAL=PES(JL,JLEV)
        IF(PT5(JL,JLEV) <= RTICE) ZDAL=0.0_JPRB
        IF(PT5(JL,JLEV) >= RTWAT) ZDAL=0.0_JPRB
        IF( ( (PT5(JL,JLEV)-RTICE)*RTWAT_RTICE_R )**2  >=  1.0_JPRB ) THEN
          ZALFAAD=0.0_JPRB
        ELSE
          ZALFAAD=2.0_JPRB*(PT5(JL,JLEV)-RTICE)*RTWAT_RTICE_R**2*ZDAL
        ENDIF
        PT(JL,JLEV)=PT(JL,JLEV)+&
         & (RETV+1.0_JPRB)*FOEEWMAD(PES(JL,JLEV),PT5(JL,JLEV),&
         & ZALFAAD)  
      ENDDO
    ENDIF
  ELSE
    IF(LNEIGE) THEN
      DO JL=KSTART,KPROF
        ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PT5(JL,JLEV)))
        PT(JL,JLEV)=PT(JL,JLEV)+FOEWAD(PES(JL,JLEV),PT5(JL,JLEV),ZDELTA)
      ENDDO
    ELSE
      DO JL=KSTART,KPROF
        ZDELTA=0.0_JPRB
        PT(JL,JLEV)=PT(JL,JLEV)+FOEWAD(PES(JL,JLEV),PT5(JL,JLEV),ZDELTA)
      ENDDO
    ENDIF
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPRHAD',1,ZHOOK_HANDLE)
END SUBROUTINE GPRHAD
