SUBROUTINE GPRHTL(LDWMORH,KPROMA,KSTART,KPROF,KFLEV,&
 & PRHMAX,PRHMIN,PQ,PT,PRESF,PES,PRH,&
 & PES5,PRESF5,PQ5,PT5)  

!**** *GPRHTL* - COMPUTES ES AND RH FROM T AND Q - TANGENT LINEAR

!     PURPOSE.  COMPUTES SATURATION VAPOUR PRESSURE AND RELATIVE
!     --------  HUMIDITY FROM TEMPERATURE AND SPECIFIC HUMIDITY.

!**   INTERFACE.
!     ----------
!        *CALL* *GPRHTL(...

!        EXPLICIT ARGUMENTS
!        --------------------

!        LDWMORH              - WMO CONSIDER WATER PHASE ONLY          (INPUT-C)
!        KPROMA               - HORIZONTAL DIMENSIONS.                 (INPUT-C)
!        KSTART               - START OF WORK.                         (INPUT-C)
!        KPROF                - DEPTH OF WORK                          (INPUT-C)
!        KFLEV                - NUMBER OF MODEL LEVELS                 (INPUT-C)
!        PRHMAX               - MAXIMUM RELATIVE HUMIDITY              (INPUT-C)
!        PRHMIN               - MINIMUM RELATIVE HUMIDITY              (INPUT-C)
!        PQ(KPROMA,KFLEV)     - SPECIFIC HUMIDITY ON FULL MODEL LEVELS (INPUT)
!        PT(KPROMA,KFLEV)     - TEMPERATURE ON FULL MODEL LEVELS       (INPUT)
!        PRESF(KPROMA,KFLEV)  - MODEL FULL LEVEL PRESSURES             (INPUT)

!        PES(KPROMA,KFLEV)    - SATURATION PRESSURE                    (OUTPUT)
!        PRH(KPROMA,KFLEV)    - RELATIVE HUMIDITY                      (OUTPUT)

!        PES5(KPROMA,KFLEV)    - SATURATION PRESSURE                   (INPUT-T)
!        PQ5(KPROMA,KFLEV)     - SPECIFIC HUMIDITY ON FULL MODEL LEVELS(INPUT-T)
!        PT5(KPROMA,KFLEV)     - TEMPERATURE ON FULL MODEL LEVELS      (INPUT-T)
!        PRESF5(KPROMA,KFLEV)  - MODEL FULL LEVEL PRESSURES            (INPUT-T)

!        IMPLICIT ARGUMENTS :   PHYSICAL CONSTANTS FROM COMMONS YOMCST
!        --------------------   AND YOMPHY, FUNCTIONS FROM FCTTRM,FCTTRMTL.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.     NONE.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD *ECMWF*
!     Per Unden : 1997-04-29  Use Tetens formula with new coefficients

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-01-10
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RV       ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD  
USE YOMPHY   , ONLY : LNEIGE
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE    ,RTICECU ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOMCT0B  , ONLY : LECMWF

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
LOGICAL           ,INTENT(IN)    :: LDWMORH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHMAX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHMIN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PES(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRH(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PES5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JL, JLEV

REAL(KIND=JPRB) :: ZALFATL, ZDELTA, ZDPT, ZRH, ZRH5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "fcttrm.h"
#include "fcttrmtl.h"
#include "fcttre.h"
#include "fcttretl.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPRHTL',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    COMPUTES ES AND RH.
!              -------------------

DO JLEV=1,KFLEV
  IF(LECMWF) THEN
    IF(LDWMORH) THEN
      DO JL=KSTART,KPROF
        PES(JL,JLEV)=(RETV+1.0_JPRB)*FOEEWMOTL(PT(JL,JLEV),PT5(JL,JLEV))
      ENDDO
    ELSE
      DO JL=KSTART,KPROF
        ZDPT=PT(JL,JLEV)
        IF(PT5(JL,JLEV) >= RTWAT) ZDPT=0.0_JPRB
        IF(PT5(JL,JLEV) <= RTICE) ZDPT=0.0_JPRB
        IF( ( (PT5(JL,JLEV)-RTICE)*RTWAT_RTICE_R )**2  >=  1.0_JPRB ) THEN
          ZALFATL=0.0_JPRB
        ELSE
          ZALFATL=2.0_JPRB*(PT5(JL,JLEV)-RTICE)*RTWAT_RTICE_R**2*ZDPT
        ENDIF
        PES(JL,JLEV)=(RETV+1.0_JPRB)*FOEEWMTL(PT(JL,JLEV),PT5(JL,JLEV),ZALFATL)
      ENDDO
    ENDIF
  ELSE
    IF(LNEIGE) THEN
      DO JL=KSTART,KPROF
        ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PT5(JL,JLEV)))
        PES(JL,JLEV)=FOEWTL(PT(JL,JLEV),PT5(JL,JLEV),ZDELTA)
      ENDDO
    ELSE
      DO JL=KSTART,KPROF
        ZDELTA=0.0_JPRB
        PES(JL,JLEV)=FOEWTL(PT(JL,JLEV),PT5(JL,JLEV),ZDELTA)
      ENDDO
    ENDIF
  ENDIF
  DO JL=KSTART,KPROF
    ZRH5=(PRESF5(JL,JLEV)*PQ5(JL,JLEV)*(RETV+1.0_JPRB))&
     & /((1.0_JPRB+RETV*PQ5(JL,JLEV))*PES5(JL,JLEV))  
    ZRH=PRESF(JL,JLEV)*PQ5(JL,JLEV)*(RETV+1.0_JPRB)&
     & /((1.0_JPRB+RETV*PQ5(JL,JLEV))*PES5(JL,JLEV))&
     & +(PRESF5(JL,JLEV)*PQ(JL,JLEV)*(RETV+1.0_JPRB))&
     & /((1.0_JPRB+RETV*PQ5(JL,JLEV))*PES5(JL,JLEV))&
     & -(PRESF5(JL,JLEV)*PQ5(JL,JLEV)*(RETV+1.0_JPRB))&
     & /((1.0_JPRB+RETV*PQ5(JL,JLEV))*PES5(JL,JLEV))**2*RETV*&
     & PES5(JL,JLEV)*PQ(JL,JLEV)&
     & -(PRESF5(JL,JLEV)*PQ5(JL,JLEV)*(RETV+1.0_JPRB))&
     & /((1.0_JPRB+RETV*PQ5(JL,JLEV))*PES5(JL,JLEV))**2 &
     & *(1.0_JPRB+RETV*PQ5(JL,JLEV))*PES(JL,JLEV)  
    IF(ZRH5 > PRHMIN .AND. ZRH5 < PRHMAX) THEN
      PRH(JL,JLEV)=ZRH
    ELSE
      PRH(JL,JLEV)=0.0_JPRB
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPRHTL',1,ZHOOK_HANDLE)
END SUBROUTINE GPRHTL

