SUBROUTINE GPTF1PC(&
 ! --- INPUT ---------------------------------------------------------
 & KST,KEN,&
 & PCUT9 ,PCVT9  ,PCTT9 ,PCSPDT9 ,PCSVDT9 ,&
 & PCSPT9,PCNHXT9,&
 & PUT0  ,PVT0   ,PTT0  ,PSPDT0  ,PSVDT0  ,&
 & PSPT0 ,PNHXT0    ,&
 ! --- INPUT/OUTPUT --------------------------------------------------
 & PUT9  ,PVT9   ,PTT9  ,PSPDT9  ,PSVDT9  ,&
 & PSPT9 ,PNHXT9)

!**** *GPTF1PC* - Timefilter part 1

!     Purpose.
!     --------
!           Performs part 1 of the time-filtering for the t-dt array.
!           Routine called in 3TL PC schemes.
           
!           - leap-frog: px9=eps1*pcx9+(1-eps1-eps2)*px0
!           - sl2tl    : no action

!**   Interface.
!     ----------
!        *CALL* *GPTF1PC(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!         KST          : start of work
!         KEN          : depth of work
!         PC(U,V,T,SPD,SVD,SP)T9 : quantities at time t-dt
!         P(U,V,T,SPD,SVD,SP)T0  : quantities at time t                        
!         PCNHXT9      : Xterm at time t-dt
!         PNHXT0       : Xterm at time t

!        INPUT/OUTPUT:
!         P(U,V,T,SPD,SVD,SP)T9  : pre-filtered quantities at time t
!         PNHXT9                 : pre-filtered Xterm at time t

!        Implicit arguments :  None
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.  None.
!     ----------

!     Reference.
!     ----------
!        LACE internal documentation

!     Author.
!     -------
!        Jozef Vivoda (based on GPTF1)  *ECMWF*

! Modifications
! -------------
!   Original : 21-Feb-2005
!   K. Yessad (Dec 2008): remove dummy CDLOCK
! End Modifications
!------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCT0   , ONLY : LTWOTL   ,LNHDYN   ,LPC_FULL
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA 
USE YOMDYN   , ONLY : REPS1    ,REPS2    ,NCURRENT_ITER  
USE YOMDYNA  , ONLY : NVDVAR   
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

USE YOMGMV

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEN 
REAL(KIND=JPRB)   ,INTENT(IN) :: PCUT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PCVT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PCTT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PCSPDT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PCSVDT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PCSPT9(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PCNHXT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PUT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PVT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PTT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PSPDT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PSVDT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PSPT0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN) :: PNHXT0(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT):: PUT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PVT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PTT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PSPDT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PSVDT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PSPT9(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT):: PNHXT9(NPROMA,NFLEVG) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZREST
INTEGER(KIND=JPIM) :: JL,JK
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPTF1PC',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1. PERFORM TIME FILTER (PART 1)
!           ----------------------------

IF (.NOT.LTWOTL.AND.NCURRENT_ITER==0) THEN

  ZREST  = 1.0_JPRB-(REPS1+REPS2)

  IF(LPC_FULL)THEN

    DO JL=1,NFLEVG
      DO JK=KST,KEN
        PUT9(JK,JL)=REPS1*PCUT9(JK,JL)+ZREST*PUT0(JK,JL)
        PVT9(JK,JL)=REPS1*PCVT9(JK,JL)+ZREST*PVT0(JK,JL)
        PTT9(JK,JL)=REPS1*PCTT9(JK,JL)+ZREST*PTT0(JK,JL)
      ENDDO
    ENDDO
    IF(LNHDYN)THEN
      DO JL=1,NFLEVG
        DO JK=KST,KEN
          PSPDT9(JK,JL)=REPS1*PCSPDT9(JK,JL)+ZREST*PSPDT0(JK,JL)
          PSVDT9(JK,JL)=REPS1*PCSVDT9(JK,JL)+ZREST*PSVDT0(JK,JL)
        ENDDO
      ENDDO
    ENDIF

    DO JK=KST,KEN
      PSPT9(JK) = REPS1*PCSPT9(JK)+ZREST*PSPT0(JK)
    ENDDO

  ENDIF

  IF(NVDVAR==4)THEN
    DO JL=1,NFLEVG
      DO JK=KST,KEN
        PNHXT9(JK,JL) = REPS1*PCNHXT9(JK,JL)+ZREST*PNHXT0(JK,JL)
      ENDDO
    ENDDO
  ENDIF
  
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPTF1PC',1,ZHOOK_HANDLE)
END SUBROUTINE GPTF1PC
