SUBROUTINE GPUVSAD( &
 ! --- INPUT -----------------------------------------------------------------
 & KFLEV,KPROMA,KSTART,KEND,LDER, &
 ! --- INPUT IN TL -----------------------------------------------------------
 & PUF,PVF, &
 ! --- OUTPUT IN TL ----------------------------------------------------------
 & PUS,PVS, &
 ! --- OPTIONAL INPUT IN TL --------------------------------------------------
 & PDIVF,PVORF,PUFL,PVFL, &
 ! --- OPTIONAL OUTPUT IN TL -------------------------------------------------
 & PUS_L,PVS_L,PUS_M,PVS_M &
 & )

! GPUVSAD - Diagnoses "V_surf" and "grad(V_surf)". Adjoint.

! Purpose
! -------
!   Diagnoses "V_surf" and "grad(V_surf)" (surface wind).
!   The current assumptions which are done are:
!   * V_surf=V(l=KFLEV)
!   * grad(V_surf)=grad(V(l=KFLEV))
!   The consistency must be kept with the content of routine GPHLUV.

! Interface
! ---------
!   * INPUT:
!   KFLEV   - number of levels.
!   KPROMA  - horizontal dimension.
!   KSTART  - start of work.
!   KEND    - end of work.
!   LDER    - T: treatment of the derivatives.

!   * INPUT IN TL:
!   PUF     - upper air U-wind at full levels.
!   PVF     - upper air V-wind at full levels.

!   * OUTPUT IN TL:
!   PUS     - surface U wind.
!   PVS     - surface V wind.

!   * OPTIONAL INPUT IN TL:
!   PDIVF   - upper air divergence at full levels.
!   PVORF   - upper air vorticity at full levels.
!   PUFL    - upper air zonal component of grad(U-wind) at full levels.
!   PVFL    - upper air zonal component of grad(V-wind) at full levels.

!   * OPTIONAL OUTPUT IN TL:
!   PUS_L   - zonal component of grad(surface U wind).
!   PVS_L   - zonal component of grad(surface V wind).
!   PUS_M   - meridian component of grad(surface U wind).
!   PVS_M   - meridian component of grad(surface V wind).

! Externals
! ---------

! Method
! ------

! Reference
! ---------

! Author
! ------
!   K. Yessad, Mar 2006

! Modifications
! -------------
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

! -----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM)      ,INTENT(IN)    :: KFLEV
INTEGER(KIND=JPIM)      ,INTENT(IN)    :: KPROMA
INTEGER(KIND=JPIM)      ,INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM)      ,INTENT(IN)    :: KEND 
LOGICAL                 ,INTENT(IN)    :: LDER
REAL(KIND=JPRB)         ,INTENT(INOUT) :: PUF(KPROMA,KFLEV)
REAL(KIND=JPRB)         ,INTENT(INOUT) :: PVF(KPROMA,KFLEV)
REAL(KIND=JPRB)         ,INTENT(INOUT) :: PUS(KPROMA) 
REAL(KIND=JPRB)         ,INTENT(INOUT) :: PVS(KPROMA) 
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PDIVF(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PVORF(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PUFL(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PVFL(KPROMA,KFLEV)
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PUS_L(KPROMA) 
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PVS_L(KPROMA) 
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PUS_M(KPROMA) 
REAL(KIND=JPRB),OPTIONAL,INTENT(INOUT) :: PVS_M(KPROMA) 

! -----------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF

REAL(KIND=JPRB) :: ZHOOK_HANDLE

! -----------------------------------------------------------------------------

#include "abor1.intfb.h"

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPUVSAD',0,ZHOOK_HANDLE)

! -----------------------------------------------------------------------------

!*      1. COMPUTES "V_surf" and "grad(V_surf)".
!       ----------------------------------------

DO JROF=KSTART,KEND
  PUF(JROF,KFLEV)=PUF(JROF,KFLEV)+PUS(JROF)
  PVF(JROF,KFLEV)=PVF(JROF,KFLEV)+PVS(JROF)
ENDDO

IF (LDER) THEN
  IF (.NOT.(PRESENT(PDIVF).AND.PRESENT(PVORF).AND.PRESENT(PUFL) &
   & .AND.PRESENT(PVFL).AND.PRESENT(PUS_L).AND.PRESENT(PVS_L) &
   & .AND.PRESENT(PUS_M).AND.PRESENT(PVS_M))) &
   & CALL ABOR1(' GPUVSAD: LDER=T => PDIVF to PVS_M should be present!')
  DO JROF=KSTART,KEND
    PUFL(JROF,KFLEV)=PUFL(JROF,KFLEV)+PUS_L(JROF)-PVS_M(JROF)
    PVFL(JROF,KFLEV)=PVFL(JROF,KFLEV)+PVS_L(JROF)+PUS_M(JROF)
    PVORF(JROF,KFLEV)=PVORF(JROF,KFLEV)-PUS_M(JROF)
    PDIVF(JROF,KFLEV)=PDIVF(JROF,KFLEV)+PVS_M(JROF)
  ENDDO
ENDIF

! -----------------------------------------------------------------------------

!*      2. FINAL RESET TO ZERO.
!       -----------------------

DO JROF=KSTART,KEND
  PUS(JROF)=0.0_JPRB
  PVS(JROF)=0.0_JPRB
ENDDO

IF (LDER) THEN
  DO JROF=KSTART,KEND
    PUS_L(JROF)=0.0_JPRB
    PVS_L(JROF)=0.0_JPRB
    PUS_M(JROF)=0.0_JPRB
    PVS_M(JROF)=0.0_JPRB
  ENDDO
ENDIF

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPUVSAD',1,ZHOOK_HANDLE)

END SUBROUTINE GPUVSAD
