SUBROUTINE GPVCRS(KPROMA,KD,KF,KFLEV,&
 & PREHYDF,PREHYDH,PVCRSSAF,PVCRSSAH,PVCASRSF,PVCASRSH)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RA       ,RD       ,RCPD     ,RG
USE YOMDYN   , ONLY : VCAK     ,VCTR     ,VCPR

!**** *GPVCRS* - Computes the ratio "rs/a" and its inverse on model layers
!                and interlayers.

!**   Purpose.
!     --------

!      When no thin layer hypothesis is applied, the model needs
!      a pseudo-radius "rs" which is not equal to the mean radius "a"
!      except for a reference pressure "prehydref". 
!      "rs" depends on the hydrostatic pressure "prehyd" and its expression is

!                  -- prehyd'=prehydref 
!                  |                       Rd Ts
!       rs = a +   |                     ----------  d prehyd'
!                  |                      g prehyd'
!                 --  prehyd'=prehyd

!      Reference pressure "prehydref" is assumed to be equal to the 
!      standard atmosphere surface temperature "prehydRST_surf".

!      Its expression involves a reference temperature Ts which only depends
!      on the hydrostatic pressure "prehyd". On assumes that this reference
!      temperature is computed in an analytic manner:

!       log(Ts/T1) = alpha_K (Rd/cpd) log(prehyd/prehydRST_surf)

!      where:
!      - T1 is a parametrable reference temperature.
!      - alpha_K is a parametrable constant.
!        For alpha_K=0 the profile of Ts is vertically constant: Ts=T1.
!        For alpha_K=1 the profile of Ts is a dry adiabatic one.
!      - Rd is the dry air constant.
!      - cpd is the constant pressure calorific capacity of dry air.

!      Combination of the previous equations for "Ts" and "rs" leads to
!       the following analytic expression for "rs" to be coded:

!       a/ if alpha_K = 0 (alpha_K < zeps in the code):

!                    Rd T1
!          rs = a - ------- log (prehyd/prehydRST_surf)
!                      g

!       b/ if alpha_K > 0 (alpha_K >= zeps in the code):

!                      cpd T1   
!          rs = a + ----------- ( 1 - (prehyd/prehydRST_surf)**vcpow )
!                    g alpha_K  

!          where: vcpow = alpha_K (Rd/cpd)

!       rs(top) is set to rs(layer=1) for consistency with some choices
!       made in the non-hydrostatic dynamics.

!**   Interface.
!     ----------
!        *CALL* *GPVCRS( ... )

!        Explicit arguments :
!        --------------------
!        INPUT :
!           KPROMA    - horizontal dimensioning.
!           KD        - start of work.
!           KF        - depth of work.
!           KFLEV     - vertical dimensioning (number of layers).
!           PREHYDF   - hydrostatic pressure on layers.
!           PREHYDH   - hydrostatic pressure on interlayers 1 to kflev.

!        OUTPUT :
!           PVCRSSAF  - "rs/a" on layers.
!           PVCRSSAH  - "rs/a" on interlayers 0 to kflev.
!           PVCASRSF  - "a/rs" on layers.
!           PVCASRSH  - "a/rs" on interlayers 0 to kflev.

!        Implicit arguments :
!        --------------------
!        none.

!     Method.
!     -------

!     Externals.
!     ----------
!        none.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (METEO-FRANCE/CNRM/GMAP).

!     Modifications.
!     --------------
!        Original: AUGUST 2000.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREHYDF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREHYDH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVCRSSAF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVCRSSAH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVCASRSF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVCASRSH(KPROMA,0:KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV

REAL(KIND=JPRB) :: ZEPS, ZVCPOW
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPVCRS',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

! * Preliminary initialisations:

ZVCPOW=VCAK*RD/RCPD
ZEPS=1.E-12

! ------------------------------------------------------------------

! * Computation of "rs/a" and "a/rs" on layers:

IF (VCAK >= ZEPS) THEN
  DO JLEV=1,KFLEV
    PVCRSSAF(KD:KF,JLEV)= 1.0_JPRB + ((RCPD*VCTR)/(RA*RG*VCAK)) * &
     & ( 1.0_JPRB - ( PREHYDF(KD:KF,JLEV)/VCPR )**ZVCPOW )  
  ENDDO
ELSE
  DO JLEV=1,KFLEV
    PVCRSSAF(KD:KF,JLEV)= 1.0_JPRB - ((RD*VCTR)/(RA*RG)) * &
     & ( LOG( PREHYDF(KD:KF,JLEV)/VCPR ) )  
  ENDDO
ENDIF

DO JLEV=1,KFLEV
  PVCASRSF(KD:KF,JLEV)= 1.0_JPRB / PVCRSSAF(KD:KF,JLEV)
ENDDO

! ------------------------------------------------------------------

! * Computation of "rs/a" and "a/rs" on interlayers 1 to kflev:

IF (VCAK >= ZEPS) THEN
  DO JLEV=1,KFLEV
    PVCRSSAH(KD:KF,JLEV)= 1.0_JPRB + ((RCPD*VCTR)/(RA*RG*VCAK)) * &
     & ( 1.0_JPRB - ( PREHYDH(KD:KF,JLEV)/VCPR )**ZVCPOW )  
  ENDDO
ELSE
  DO JLEV=1,KFLEV
    PVCRSSAH(KD:KF,JLEV)= 1.0_JPRB - ((RD*VCTR)/(RA*RG)) * &
     & ( LOG( PREHYDH(KD:KF,JLEV)/VCPR ) )  
  ENDDO
ENDIF

DO JLEV=1,KFLEV
  PVCASRSH(KD:KF,JLEV)= 1.0_JPRB / PVCRSSAH(KD:KF,JLEV)
ENDDO

! ------------------------------------------------------------------

! * Computation of "rs/a" and "a/rs" at the top of the model:

PVCRSSAH(KD:KF,0)=PVCRSSAF(KD:KF,1)
PVCASRSH(KD:KF,0)= 1.0_JPRB / PVCRSSAH(KD:KF,0)

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPVCRS',1,ZHOOK_HANDLE)
END SUBROUTINE GPVCRS
