SUBROUTINE GPVCTS(KPROMA,KD,KF,KFLEV,&
 & PRTGR,PREHYDF,PREHYDSL,PREHYDSM,&
 & PVCTSF,PVCTSFL,PVCTSFM)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RD       ,RCPD
USE YOMDYN   , ONLY : VCAK     ,VCTR     ,VCPR

!**** *GPVCTS* - Computes the reference profile of temperature Ts
!                 used in the definition of the pseudo-radius
!                 and its "semi-reduced" horizontal gradient on model layers.

!**   Purpose.
!     --------

!      When no thin layer hypothesis is applied, the model needs
!      a pseudo-radius "rs" which is not equal to the mean radius "a"
!      except for a reference pressure "prehydref". 
!      "rs" depends on the hydrostatic pressure "prehyd" and its expression is

!                  -- prehyd'=prehydref 
!                  |                       Rd Ts
!       rs = a +   |                     ----------  d prehyd'
!                  |                      g prehyd'
!                 --  prehyd'=prehyd

!      Reference pressure "prehydref" is assumed to be equal to the 
!      standard atmosphere surface temperature "prehydRST_surf".

!      Its expression involves a reference temperature Ts which only depends
!      on the hydrostatic pressure "prehyd". On assumes that this reference
!      temperature is computed in an analytic manner:

!       log(Ts/T1) = alpha_K (Rd/cpd) log(prehyd/prehydRST_surf)

!      where:
!      - T1 is a parametrable reference temperature.
!      - alpha_K is a parametrable constant.
!        For alpha_K=0 the profile of Ts is vertically constant: Ts=T1.
!        For alpha_K=1 the profile of Ts is a dry adiabatic one.
!      - Rd is the dry air constant.
!      - cpd is the constant pressure calorific capacity of dry air.

!      The semi-reduced horizontal gradient ("rssavnabla") of Ts is computed
!      as follows:

!       rssavnabla Ts = alpha_K (Rd Ts)/cpd (rssavnabla prehyd)/prehyd

!**   Interface.
!     ----------
!        *CALL* *GPVCTS( ... )

!        Explicit arguments :
!        --------------------
!        INPUT :
!           KPROMA    - horizontal dimensioning.
!           KD        - start of work.
!           KF        - depth of work.
!           KFLEV     - vertical dimensioning (number of layers).
!           PRTGR     - scalar ratio of vectors "(rssavnabla prehyd)/prehyd"
!                       and "rssavnabla prehyds" (computed in GPXYB).
!           PREHYDF   - hydrostatic pressure on layers.
!           PREHYDSL  - zonal component of "rssavnabla prehyds"
!                       ("prehyds" is surface hydrostatic pressure).
!           PREHYDSM  - meridian component of "rssavnabla prehyds".

!        OUTPUT :
!           PVCTSF    - reference temperature Ts on layers.
!           PVCTSFL   - zonal component of "rssavnabla Ts" on layers.
!           PVCTSFM   - meridian component of "rssavnabla Ts" on layers.

!        Implicit arguments :
!        --------------------
!        none.

!     Method.
!     -------

!     Externals.
!     ----------
!        none.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (METEO-FRANCE/CNRM/GMAP).

!     Modifications.
!     --------------
!        Original: AUGUST 2000.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRTGR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREHYDF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREHYDSL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREHYDSM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVCTSF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVCTSFL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVCTSFM(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV

REAL(KIND=JPRB) :: ZVCPOW
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPVCTS',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

! * Preliminary initialisations:

ZVCPOW=VCAK*RD/RCPD

! ------------------------------------------------------------------

! * Computation of "Ts" on layers:

DO JLEV=1,KFLEV
  PVCTSF(KD:KF,JLEV)= VCTR * ( PREHYDF(KD:KF,JLEV)/VCPR )**ZVCPOW
ENDDO

! ------------------------------------------------------------------

! * Computation of "rssavnabla Ts" on layers:

DO JLEV=1,KFLEV
  PVCTSFL(KD:KF,JLEV)= &
   & ZVCPOW * PVCTSF(KD:KF,JLEV) * PRTGR(KD:KF,JLEV) * PREHYDSL(KD:KF)  
  PVCTSFM(KD:KF,JLEV)= &
   & ZVCPOW * PVCTSF(KD:KF,JLEV) * PRTGR(KD:KF,JLEV) * PREHYDSM(KD:KF)  
ENDDO

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPVCTS',1,ZHOOK_HANDLE)
END SUBROUTINE GPVCTS
