SUBROUTINE GPVERDIA(&
 ! ----- INPUT ---------------------------------------------------------------
 & KPROMA,KSTART,KPROF,&
 & PUF,PVF,PDIV,PVVEL,PGPHL,PGPHM,PGPFL,PGPFM,&
 & PLNPR,PALPH,PRT,PKAP,&
 ! ----- OUTPUT --------------------------------------------------------------
 & PDVERV)

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCVER  , ONLY : LVERTFE  ,LVFE_X_TERM
USE YOMDIM   , ONLY : NFLEVG
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

!**** *GPVERDIA* - vertical divergence diagnostics in hydrostatic dynamics

!     Purpose.
!     --------
!      This routine computes the virtual vertical divergence "dverv"
!       dverv = (Rd/R) dver
!      where "dver" is the vertical divergence, in an hydrostatic model.
!      In an hydrostatic model:
!       dverv = -D -X +D3hyd
!       and
!       D3hyd=-(cv/cp)(omega/prehyd)

!      Currently not properly coded for the finite element vertical
!       discretization (LVERTFE=T).

!**   Interface.
!     ----------
!        *CALL* *GPVERDIA(...)

!        Explicit arguments :
!        --------------------

!        * INPUT:
!          KPROMA   : Horizontal dimension.
!          KSTART   : Start of work.
!          KPROF    : End of work.
!          PUF      : U-wind at full levels.
!          PVF      : V-wind at full levels.
!          PDIV     : Divergence at full levels.
!          PVVEL    : (omega/prehyd) at full levels.
!          PGPHL    : zonal component of grad(gz) at half levels.
!          PGPHM    : merid component of grad(gz) at half levels.
!          PGPFL    : zonal component of grad(gz) at full levels.
!          PGPFM    : merid component of grad(gz) at full levels.
!          PLNPR    : "delta" at full levels.
!          PALPH    : "alpha" at full levels.
!          PRT      : R*T at full levels.
!          PKAP     : kappa=R/cp at full levels.

!        * OUTPUT:
!          PDVERV   : virtual vertical divergence "dverv" at full levels.

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!     Reference.
!     ----------
!        ARPEGE/ALADIN documentation

!     Author.
!     -------
!        Radmila Bubnova *GMAP/COMPAS - stage MRE*

! Modifications
! -------------
!   Original : 94-01-28
!   Modified:  97-06-24 J.Boutahar (FULL-POS NH)
!   Modified:  98-12-04 L. Gerard: unused args for GPHLUV
!        requires to check LVSLWBC
!   01-Oct-2003 M. Hamrud   CY28 Cleaning
!   12-Dec-2003 R. Zaaboul  CY28 phasing to call GPHLUV
!   09-Jun-2004 J. Masek    NH cleaning (LVSLWBC)
!   24-Jan-2005 K. Yessad   renaming EVERDIA into GPVERDIA; cleanings.
!   09-Oct-2007 J. Vivoda and K. Yessad   VFE for NH.
! End Modifications
!------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVF(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIV(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVVEL(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGPHL(KPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGPHM(KPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGPFL(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGPFM(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRT(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAP(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDVERV(KPROMA,NFLEVG)

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZX(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZD3HYD(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZLHU(KPROMA,0:NFLEVG),ZLHV(KPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZWWW(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZUNITE(KPROMA,NFLEVG)

INTEGER(KIND=JPIM) :: JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "gphluv.intfb.h"
#include "gphlwi.intfb.h"
#include "gpxx.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPVERDIA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    Computes the horizontal wind at half levels.

IF (.NOT.(LVERTFE .AND. LVFE_X_TERM)) THEN
  ! compute interpolation weights
  CALL GPHLWI(KPROMA,KSTART,KPROF,PLNPR,PALPH,ZWWW)
  ! interpolate wind into half levels
  CALL GPHLUV(KPROMA,KSTART,KPROF,ZWWW,PUF,PVF,ZLHU,ZLHV)  
ELSE
  ! ZUH and ZVH are useless, set to 0 for safety.
  ZLHU(KSTART:KPROF,0:NFLEVG)=0.0_JPRB
  ZLHV(KSTART:KPROF,0:NFLEVG)=0.0_JPRB
ENDIF

!*       2.    X - Term

ZUNITE(KSTART:KPROF,1:NFLEVG)=1.0_JPRB
CALL GPXX( &
 ! --- INPUT -------------------------------------------------------------
 & NFLEVG,KPROMA,KSTART,KPROF,PGPHL,PGPHM,PGPFL,PGPFM, &
 & PLNPR,ZUNITE,ZUNITE,PRT, &
 & PUF,PVF,ZLHU,ZLHV, &
 ! --- OUTPUT ------------------------------------------------------------
 & ZX &
 & )

!*       3.    Computes D3hyd=-(cv/cp)(omega/prehyd)=(kappa-1)(omega/prehyd)

DO JLEV=1,NFLEVG
  DO JROF=KSTART,KPROF
    ZD3HYD(JROF,JLEV)=(PKAP(JROF,JLEV)-1.0_JPRB)*PVVEL(JROF,JLEV)
  ENDDO
ENDDO

!*       4.    Computes "dverv" and "dver".

DO JLEV=1,NFLEVG
  DO JROF=KSTART,KPROF
    PDVERV(JROF,JLEV)=-PDIV(JROF,JLEV)-ZX(JROF,JLEV)+ZD3HYD(JROF,JLEV)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPVERDIA',1,ZHOOK_HANDLE)
END SUBROUTINE GPVERDIA
