SUBROUTINE GPXX( &
 ! --- INPUT -----------------------------------------------------------------
 & KFLEV,KPROMA,KSTART,KEND,PHIHL,PHIHM,PHIFL,PHIFM, &
 & PLNPR,PNHPPI,PT,PR, &
 & PUF,PVF,PUH,PVH, &
 ! --- OUTPUT ----------------------------------------------------------------
 & PX, &
 ! --- OPTIONAL INPUT --------------------------------------------------------
 & LDRWSDLR,PRDLR &
 & )

! GPXX - Diagnose NHX-term

! Purpose
! -------
!   Diagnose NHX-term 
!    NHX = (pre/(RT)) grad[gz] (d V / d prehyd)
!   It is better to rewrite NHX as follows for the discretisation:
!    NHX = (pre/prehyd) (1/(RT)) grad[gz] ( d V / d (log(prehyd)) )

!   NHX is discretised as follows, at full levels, for FD discretisation:
!   [X]_[l] = [pre/prehyd]_[l] * [ 1/(R_[l] T_[l] delta_[l]) ] *
!             [ grad[gz]_[lbar-1] (V_[l] - V[lbar-1])
!             + grad[gz]_[lbar] (V_[lbar] - V_[l]) ]
!   VFE discretisation is different.

!   For Wood and Staniforth NH deep layer equations (LRWSDLR=T),
!    quantities based on hydrostatic pressure "prehyd" are now based on
!    "presim", where d(presim)/d(prehyd)=(r**2/a**2)*(G/g), and NHX writes:
!    NHX = (r**2/a**2) (pre/presim) (1/(RT)) geograd[Gr] ( d V / d (log(presim)) )
!    or:
!    NHX = (r/a) (pre/presim) (1/(RT)) grad[Gr] ( d V / d (log(presim)) )
!    ("grad" is the semi-reduced gradient, grad=(r/a)geograd).

!   This routine can also be used in a hydrostatic model: in this case
!   the ratio (pre/prehyd) is equal to 1.

! Interface
! ---------
!   * INPUT:
!   KFLEV   - number of levels.
!   KPROMA  - length of work
!   KSTART  - start of work
!   KEND    - end of work
!   PHIHL   - zonal component of "grad[gz]" at half levels.
!   PHIHM   - meridian component of "grad[gz]" at half levels.
!   PHIFL   - zonal component of "grad[gz]" at full levels.
!   PHIFM   - meridian component of "grad[gz]" at full levels.
!   PLNPR   - "delta" at full levels.
!   PNHPPI  - [pre/prehyd] at full levels.
!   PT      - Temperature at full levels.
!   PR      - R at full levels.
!   PUF     - U-wind at full levels.
!   PVF     - V-wind at full levels.
!   PUH     - U-wind at half levels.
!   PVH     - V-wind at half levels.

!   * OUTPUT:
!   PX      - NHX-term at full levels.

!   * OPTIONAL INPUT:
!   LDRWSDLR - key for metric terms in Wood and Staniforth DL eqns
!   PRDLR    - array containing "r/a", "a/r", "grad(r/a)".

! Externals
! ---------

! Method
! ------

! Reference
! ---------

! Author
! ------
!   06 Dec 2004 K. Yessad (after GNHX).

! Modifications
! -------------
!   J. Vivoda and K. Yessad (Oct 2007): VFE for NH model.
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   K. Yessad (March 2009): correct false comments for LRWSDLG=T
! End Modifications
!---------------------------------------------------------------------

USE YOMCVER  , ONLY : LVERTFE  ,LVFE_X_TERM
USE YOMGEM   , ONLY : VFE_RDETAH
USE PAR_RDLR , ONLY : JP_RSAF  ,JP_DIMR

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

! -----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHIHL(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHIHM(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHIFL(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHIFM(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNHPPI(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PX(KPROMA,KFLEV) 
LOGICAL,OPTIONAL  ,INTENT(IN)    :: LDRWSDLR
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PRDLR(KPROMA,0:KFLEV,JP_DIMR)

! -----------------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF
REAL(KIND=JPRB) :: ZDUF(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZDVF(KPROMA,KFLEV)
LOGICAL :: LLRWSDLR

REAL(KIND=JPRB) :: ZHOOK_HANDLE

! -----------------------------------------------------------------------------

#include "verder.intfb.h"

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPXX',0,ZHOOK_HANDLE)

! -----------------------------------------------------------------------------

IF (PRESENT(LDRWSDLR)) THEN
  LLRWSDLR=LDRWSDLR
ELSE
  LLRWSDLR=.FALSE.
ENDIF
IF (LLRWSDLR.AND.(.NOT.PRESENT(PRDLR))) THEN
  CALL ABOR1('GPXX: if LRWSDLR=T, PRDLR should be present')
ENDIF

IF( LVERTFE .AND. LVFE_X_TERM )THEN

  ! vertical derivatives of wind
  CALL VERDER(KPROMA,KSTART,KEND,KFLEV,PUF,ZDUF)
  CALL VERDER(KPROMA,KSTART,KEND,KFLEV,PVF,ZDVF)

  DO JLEV=1,KFLEV
    DO JROF=KSTART,KEND
      PX(JROF,JLEV) = &
       & PNHPPI(JROF,JLEV)/(PT(JROF,JLEV)*PR(JROF,JLEV)*PLNPR(JROF,JLEV))*&
       & (PHIFL(JROF,JLEV)*ZDUF(JROF,JLEV)+PHIFM(JROF,JLEV)*ZDVF(JROF,JLEV)) &
       & /VFE_RDETAH(JLEV)
    ENDDO
  ENDDO

ELSE

  DO JLEV=1,KFLEV
    DO JROF=KSTART,KEND
      PX(JROF,JLEV)=( &
       & (PUH(JROF,JLEV)-PUF(JROF,JLEV))*PHIHL(JROF,JLEV) + &
       & (PVH(JROF,JLEV)-PVF(JROF,JLEV))*PHIHM(JROF,JLEV) + &
       & (PUF(JROF,JLEV)-PUH(JROF,JLEV-1))*PHIHL(JROF,JLEV-1) + &
       & (PVF(JROF,JLEV)-PVH(JROF,JLEV-1))*PHIHM(JROF,JLEV-1) &
       & )*PNHPPI(JROF,JLEV)/(PT(JROF,JLEV)*PR(JROF,JLEV)*PLNPR(JROF,JLEV))
    ENDDO
  ENDDO

ENDIF

IF (LLRWSDLR) THEN
  DO JLEV=1,KFLEV
    DO JROF=KSTART,KEND
      PX(JROF,JLEV)=PX(JROF,JLEV)*PRDLR(JROF,JLEV,JP_RSAF)
    ENDDO
  ENDDO
ENDIF

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GPXX',1,ZHOOK_HANDLE)

END SUBROUTINE GPXX

