SUBROUTINE LAIDDIAD(KPROMA,KPROMB,KSTART,KPROF,KFLEV,&
 & KFLDN,KFLDX,KBUF,&
 & PCLA,PDLO,KL0,&
 & PXSL,PXF)  

!**** *LAIDDIAD-  semi-LAgrangian scheme:
!                 BI-dimensional 12 points interpolations (adjoint)

!     Purpose.
!     --------
!       Performs bi-dimensional 12 points interpolations.

!       The current code is valid only if the interpolation point and
!        the weights are constants (contrary to LAIDDITLAD which assumes
!        that these quantities temporally vary).

!**   Interface.
!     ----------
!        *CALL* *LAIDDIAD(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension for grid-point quantities.
!          KPROMB  - horizontal dimension for interpolation point
!                    quantities.
!          KSTART  - first element of arrays where
!                    computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KFLDN   - number of the first field.
!          KFLDX   - number of the last field.
!          KBUF    - number of buffers.
!          PCLA    - weights (distances) for horizontal cubic interpolation
!                    on a same latitude.
!          PDLO    - weights (distances) for horizontal cubic interpolation
!                    on a same longitude.
!          KL0     - indices of the four western points
!                    of the 16 points interpolation grid.
!          PXSL    - quantity to be interpolated.

!        OUTPUT:
!          PXF     - interpolated variable.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.

!     Reference.
!     ----------

!     Author.
!     -------
!        D. Vasiljevic

!     Modifications.
!     --------------
!        Original : 20/05/92
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Sep 2008): update comments + cleanings.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBUF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA(KPROMB,KFLEV,2:4) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO(KPROMB,KFLEV,0:3) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMB,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXSL(KPROMA*(KFLDX-KFLDN+1)*KBUF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXF(KPROMB,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: PD, ZCLA2, ZCLA3, ZCLA4, ZDLO, ZDLO1, ZDLO2,&
 & ZDLO3, ZVALLO, ZVALLO1, ZVALLO2, ZVALLO3  

REAL(KIND=JPRB) :: FCLO1
REAL(KIND=JPRB) :: FCLO2
REAL(KIND=JPRB) :: FCLO3
REAL(KIND=JPRB) :: FCLO4

REAL(KIND=JPRB),PARAMETER :: Z6_R=1.0_JPRB/6._JPRB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!     FUNCTIONS:
FCLO1(PD)=-PD*(PD-1.0_JPRB)*(PD-2.0_JPRB)*Z6_R
FCLO2(PD)= 0.5_JPRB*(PD+1.0_JPRB)*(PD-2.0_JPRB)*(PD-1.0_JPRB)
FCLO3(PD)=-0.5_JPRB*(PD+1.0_JPRB)*(PD-2.0_JPRB)* PD
FCLO4(PD)=PD*(PD-1.0_JPRB)*(PD+1.0_JPRB)*Z6_R

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAIDDIAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INTERPOLATIONS.
!              ---------------

! k.y.: this (currently unused) piece of code has not been updated
!   relatively to ne new data flow of the SL interpolation routines.
!   It is planned in the future to put the content of LAIDDITLAD
!   in this deck.
CALL ABOR1(' LAIDDIAD: code provisionally broken')

DO JROF=KSTART,KPROF
!DIR$ IVDEP
!OCL NOVREC
  DO JLEV=1,KFLEV

    ZCLA2 =PCLA(JROF,JLEV,2)
    ZCLA3 =PCLA(JROF,JLEV,3)
    ZCLA4 =PCLA(JROF,JLEV,4)

    ZDLO  =PDLO(JROF,JLEV,0)
    ZDLO1 =PDLO(JROF,JLEV,1)
    ZDLO2 =PDLO(JROF,JLEV,2)
    ZDLO3 =PDLO(JROF,JLEV,3)

!     interpolation.

    ZVALLO  = PXF(JROF,JLEV)*(1-ZCLA2-ZCLA3-ZCLA4)
    ZVALLO1 = PXF(JROF,JLEV)*ZCLA2
    ZVALLO2 = PXF(JROF,JLEV)*ZCLA3
    ZVALLO3 = PXF(JROF,JLEV)*ZCLA4

    PXSL(KL0(JROF,JLEV,3)+1) = PXSL(KL0(JROF,JLEV,3)+1) +ZVALLO3*(1-ZDLO3)
    PXSL(KL0(JROF,JLEV,3)+2) = PXSL(KL0(JROF,JLEV,3)+2) +ZVALLO3*ZDLO3

    PXSL(KL0(JROF,JLEV,2)  ) = PXSL(KL0(JROF,JLEV,2)  ) +&
     & ZVALLO2*(1-FCLO2(ZDLO2)&
     & -FCLO3(ZDLO2)&
     & -FCLO4(ZDLO2))  
    PXSL(KL0(JROF,JLEV,2)+1) = PXSL(KL0(JROF,JLEV,2)+1) +ZVALLO2*FCLO2(ZDLO2)
    PXSL(KL0(JROF,JLEV,2)+2) = PXSL(KL0(JROF,JLEV,2)+2) +ZVALLO2*FCLO3(ZDLO2)
    PXSL(KL0(JROF,JLEV,2)+3) = PXSL(KL0(JROF,JLEV,2)+3) +ZVALLO2*FCLO4(ZDLO2)

    PXSL(KL0(JROF,JLEV,1)  ) = PXSL(KL0(JROF,JLEV,1)  ) +&
     & ZVALLO1*(1-FCLO2(ZDLO1)&
     & -FCLO3(ZDLO1)&
     & -FCLO4(ZDLO1))  
    PXSL(KL0(JROF,JLEV,1)+1) = PXSL(KL0(JROF,JLEV,1)+1) +ZVALLO1*FCLO2(ZDLO1)
    PXSL(KL0(JROF,JLEV,1)+2) = PXSL(KL0(JROF,JLEV,1)+2) +ZVALLO1*FCLO3(ZDLO1)
    PXSL(KL0(JROF,JLEV,1)+3) = PXSL(KL0(JROF,JLEV,1)+3) +ZVALLO1*FCLO4(ZDLO1)

    PXSL(KL0(JROF,JLEV,0)+1) = PXSL(KL0(JROF,JLEV,0)+1) +ZVALLO*(1-ZDLO)
    PXSL(KL0(JROF,JLEV,0)+2) = PXSL(KL0(JROF,JLEV,0)+2) +ZVALLO*ZDLO

  ENDDO
ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LAIDDIAD',1,ZHOOK_HANDLE)
END SUBROUTINE LAIDDIAD

