SUBROUTINE LAIDDITL(KPROMA,KPROMB,KSTART,KPROF,KFLEV,&
 & KFLDN,KFLDX,&
 & PCLA,PDLO,PCLO,PCLA5,PDLO5,PCLO5,KL0,&
 & PXSL,PXSL5,PXF,PXF5)  

!**** *LAIDDITL - semi-LAgrangian scheme:    (tangent-linear version)
!                 BI-dimensional 12 points interpolations.

!     Purpose.
!     --------
!       Performs bi-dimensional 12 points interpolations (in horizontal).
!       High order interpolator fully controlled by weights, low order
!       interpolator always linear.

!**   Interface.
!     ----------
!        *CALL* *LAIDDITL(KPROMA,KPROMB,KSTART,KPROF,KFLEV
!                      ,KFLDN,KFLDX
!                      ,PCLA,PDLO,PCLO,PCLA5,PDLO5,PCLO5,KL0
!                      ,PXSL,PXSL5,PXF,PXF5)  

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension for grid-point quantities.
!          KPROMB  - horizontal dimension for interpolation point
!                    quantities.
!          KSTART  - first element of arrays where
!                    computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KFLDN   - number of the first field.
!          KFLDX   - number of the last field.
!          PCLA    - weights for horizontal cubic interpolation in latitude
!          PDLO    - distances for horizontal linear interpolations 
!                    in longitude (latitude rows 0, 3)
!          PCLO    - weights for horizontal cubic interpolations in longitude
!                    (latitude rows 1, 2)
! ----------------------- trajectory variables ----------------------------
!          PCLA5   - weights for horizontal cubic interpolation in latitude
!          PDLO5   - distances for horizontal linear interpolations 
!                    in longitude (latitude rows 0, 3)
!          PCLO5   - weights for horizontal cubic interpolations in longitude
!                    (latitude rows 1, 2)
! -------------------------------------------------------------------------
!          KL0     - indices of the four western points
!                    of the 16 points interpolation grid.
!          PXSL    - semi-lagrangian variable.
!          PXSL5   - semi-lagrangian variable.   (trajectory)

!        OUTPUT:
!          PXF     - interpolated variable.
!          PXF5    - interpolated variable.    (trajectory)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.
!        Called by LARCINTL.

!     Reference.
!     ----------

!     Author.
!     -------
!        C. Temperton (ECMWF)

!     Modifications.
!     --------------
!        Original : 98/06/13.
!        Modified 00-03-01 by C. Temperton: precompute functions (PFUN5).
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        F. Vana  08-Jun-20: Weight driven high order interpolation

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM    ,JPRB
USE YOMHOOK   ,ONLY : LHOOK   ,DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA(KPROMB,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO(KPROMB,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO(KPROMB,KFLEV,3,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA5(KPROMB,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO5(KPROMB,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO5(KPROMB,KFLEV,3,2) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMB,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL(KPROMA*(KFLDX-KFLDN+1)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL5(KPROMA*(KFLDX-KFLDN+1)) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXF(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXF5(KPROMB,KFLEV) 
!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZF11, ZF115, ZF12, ZF125, ZF21, ZF215, ZF22,&
 & ZF225, Z0, Z1, Z2, Z3, Z05, Z15, Z25, Z35

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAIDDITL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INTERPOLATIONS.
!              ---------------

!CDIR OUTERUNROLL=8
DO JLEV=1,KFLEV

  DO JROF=KSTART,KPROF

    ZF115=PXSL5(KL0(JROF,JLEV,1)+1)
    ZF215=PXSL5(KL0(JROF,JLEV,1)+2)
    ZF125=PXSL5(KL0(JROF,JLEV,2)+1)
    ZF225=PXSL5(KL0(JROF,JLEV,2)+2)

    ZF11=PXSL(KL0(JROF,JLEV,1)+1)
    ZF21=PXSL(KL0(JROF,JLEV,1)+2)
    ZF12=PXSL(KL0(JROF,JLEV,2)+1)
    ZF22=PXSL(KL0(JROF,JLEV,2)+2)

    ! longitudial interpolation.
    Z05=PXSL5(KL0(JROF,JLEV,0)+1)+PDLO5(JROF,JLEV,0) &
     & *(PXSL5(KL0(JROF,JLEV,0)+2)-PXSL5(KL0(JROF,JLEV,0)+1))
    Z0=PXSL(KL0(JROF,JLEV,0)+1)+PDLO(JROF,JLEV,0) &
     & *(PXSL5(KL0(JROF,JLEV,0)+2)-PXSL5(KL0(JROF,JLEV,0)+1)) &
     & +PDLO5(JROF,JLEV,0) &
     & *(PXSL(KL0(JROF,JLEV,0)+2)-PXSL(KL0(JROF,JLEV,0)+1))

    Z15=PXSL5(KL0(JROF,JLEV,1)) &
     & +PCLO5(JROF,JLEV,1,1)*(ZF115-PXSL5(KL0(JROF,JLEV,1))) &
     & +PCLO5(JROF,JLEV,2,1)*(ZF215-PXSL5(KL0(JROF,JLEV,1))) &
     & +PCLO5(JROF,JLEV,3,1)*(PXSL5(KL0(JROF,JLEV,1)+3)-PXSL5(KL0(JROF,JLEV,1)))
    Z1=PXSL(KL0(JROF,JLEV,1)) &
     & +PCLO(JROF,JLEV,1,1)*(ZF115-PXSL5(KL0(JROF,JLEV,1))) &
     & +PCLO(JROF,JLEV,2,1)*(ZF215-PXSL5(KL0(JROF,JLEV,1))) &
     & +PCLO(JROF,JLEV,3,1) &
     & *  (PXSL5(KL0(JROF,JLEV,1)+3)-PXSL5(KL0(JROF,JLEV,1))) &
     & +PCLO5(JROF,JLEV,1,1)*(ZF11-PXSL(KL0(JROF,JLEV,1))) &
     & +PCLO5(JROF,JLEV,2,1)*(ZF21-PXSL(KL0(JROF,JLEV,1))) &
     & +PCLO5(JROF,JLEV,3,1) &
     & *  (PXSL(KL0(JROF,JLEV,1)+3)-PXSL(KL0(JROF,JLEV,1)))

     Z25=PXSL5(KL0(JROF,JLEV,2)) &  
     & +PCLO5(JROF,JLEV,1,2)*(ZF125-PXSL5(KL0(JROF,JLEV,2))) &
     & +PCLO5(JROF,JLEV,2,2)*(ZF225-PXSL5(KL0(JROF,JLEV,2))) &
     & +PCLO5(JROF,JLEV,3,2)*(PXSL5(KL0(JROF,JLEV,2)+3)-PXSL5(KL0(JROF,JLEV,2)))
     Z2=PXSL(KL0(JROF,JLEV,2)) &  
     & +PCLO(JROF,JLEV,1,2)*(ZF125-PXSL5(KL0(JROF,JLEV,2))) &
     & +PCLO(JROF,JLEV,2,2)*(ZF225-PXSL5(KL0(JROF,JLEV,2))) &
     & +PCLO(JROF,JLEV,3,2) &
     & *  (PXSL5(KL0(JROF,JLEV,2)+3)-PXSL5(KL0(JROF,JLEV,2))) &
     & +PCLO5(JROF,JLEV,1,2)*(ZF12-PXSL(KL0(JROF,JLEV,2))) &
     & +PCLO5(JROF,JLEV,2,2)*(ZF22-PXSL(KL0(JROF,JLEV,2))) &
     & +PCLO5(JROF,JLEV,3,2) &
     & *  (PXSL(KL0(JROF,JLEV,2)+3)-PXSL(KL0(JROF,JLEV,2)))

     Z35=PXSL5(KL0(JROF,JLEV,3)+1)+PDLO5(JROF,JLEV,3) &
     & *(PXSL5(KL0(JROF,JLEV,3)+2)-PXSL5(KL0(JROF,JLEV,3)+1))
     Z3=PXSL(KL0(JROF,JLEV,3)+1)+PDLO(JROF,JLEV,3) &
     & *(PXSL5(KL0(JROF,JLEV,3)+2)-PXSL5(KL0(JROF,JLEV,3)+1)) &
     & +PDLO5(JROF,JLEV,3) &
     & *(PXSL(KL0(JROF,JLEV,3)+2)-PXSL(KL0(JROF,JLEV,3)+1))

    ! final interpolation in latitude
    PXF5(JROF,JLEV)= Z05 &
     & +PCLA5(JROF,JLEV,1)*(Z15-Z05) &
     & +PCLA5(JROF,JLEV,2)*(Z25-Z05) &
     & +PCLA5(JROF,JLEV,3)*(Z35-Z05)
    PXF(JROF,JLEV)= Z0 &
     & +PCLA(JROF,JLEV,1)*(Z15-Z05) &
     & +PCLA(JROF,JLEV,2)*(Z25-Z05) &
     & +PCLA(JROF,JLEV,3)*(Z35-Z05) &
     & +PCLA5(JROF,JLEV,1)*(Z1-Z0) &
     & +PCLA5(JROF,JLEV,2)*(Z2-Z0) &
     & +PCLA5(JROF,JLEV,3)*(Z3-Z0)

  ENDDO

ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAIDDITL',1,ZHOOK_HANDLE)
END SUBROUTINE LAIDDITL
