SUBROUTINE LAITLITL(KPROMA,KPROMB,KSTART,KPROF,KFLEV,&
 & KFLDN,KFLDX,&
 & PDLAT,PDLO,KL0,PDVER,PDLAT5,PDLO5,PDVER5,&
 & PXSL,PXSL5,PXF,PXF5)  

!**** *LAITLITL - semi-LAgrangian scheme: (tangent-linear version)
!                 Trilinear interpolations for one variable.

!     Purpose.
!     --------
!       Performs trilinear interpolations for one variable.

!**   Interface.
!     ----------
!        *CALL* *LAITLITL(KPROMA,KPROMB,KSTART,KPROF,KFLEV
!                      ,KFLDN,KFLDX
!                      ,PDLAT,PDLO,KL0,PDVER,PDLAT5,PDLO5,PDVER5
!                      ,PXSL,PXSL5,PXF,PXF5)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension for grid-point quantities.
!          KPROMB  - horizontal dimension for interpolation point
!                    quantities.
!          KSTART  - first element of arrays where
!                    computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KFLDN   - number of the first field.
!          KFLDX   - number of the last field.
!          PDLAT   - weight (distance) for horizontal linear interpolation
!                    on a same latitude.
!          PDLO    - weights (distances) for horizontal linear interpolation
!                    on a same longitude.
!          KL0     - indices of the four western points
!                    of the 16 points interpolation grid.
!          PDVER   - weights (distances) for vertical linear interpolation
!                    on a same vertical.
!          PXSL    - semi-lagrangian variable.
! ------------------------trajectory variables --------------------------
!          PDLAT5  - weight (distance) for horizontal linear interpolation
!                    on a same latitude.
!          PDLO5   - weights (distances) for horizontal linear interpolation
!                    on a same longitude.
!          PDVER5  - weights (distances) for vertical linear interpolation
!                    on a same vertical.
!          PXSL5   - semi-lagrangian variable.
! -----------------------------------------------------------------------

!        OUTPUT:
!          PXF     - interpolated variable.
!          PXF5    - interpolated variable (TRAJECTORY).

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.
!        Called by LARCINTL.

!     Reference.
!     ----------

!     Author.
!     -------
!        C. Temperton (ECMWF)                   

!     Modifications.
!     --------------
!        Original : 99/07/06
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        F. Vana       26-Aug-2008 optimization for NEC
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO(KPROMB,KFLEV,1:2) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMB,KFLEV,1:2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDVER(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT5(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO5(KPROMB,KFLEV,1:2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDVER5(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL(KPROMA*(KFLDX-KFLDN+1)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL5(KPROMA*(KFLDX-KFLDN+1)) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXF(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXF5(KPROMB,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IV1L1, IV1L2, IV2L1, IV2L2, JLEV, JROF

REAL(KIND=JPRB) :: ZDLAT, ZDLO1, ZDLO2, ZDVER, ZINF, ZINFLO1,&
 & ZINFLO2, ZSUP, ZSUPLO1, ZSUPLO2,&
 & ZDLAT5, ZDLO15, ZDLO25, ZDVER5, ZINF5, ZINFLO15,&
 & ZINFLO25, ZSUP5, ZSUPLO15, ZSUPLO25  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITLITL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INTERPOLATIONS.
!              ---------------

IV1L1=1+KPROMA
IV1L2=2+KPROMA
IV2L1=IV1L1+KPROMA
IV2L2=IV1L2+KPROMA

!CDIR OUTERUNROLL=8
DO JLEV=1,KFLEV
  DO JROF=KSTART,KPROF

!     Computation of coordinates and distances.

    ZDLAT5=PDLAT5(JROF,JLEV)
    ZDLAT =PDLAT(JROF,JLEV)

    ZDLO15=PDLO5(JROF,JLEV,1)
    ZDLO1 =PDLO(JROF,JLEV,1)
    ZDLO25=PDLO5(JROF,JLEV,2)
    ZDLO2 =PDLO(JROF,JLEV,2)

    ZDVER5=PDVER5(JROF,JLEV)
    ZDVER =PDVER(JROF,JLEV)

!     Interpolation.

    ZSUPLO15=PXSL5(KL0(JROF,JLEV,1)+IV1L1) + ZDLO15*&
     & ( PXSL5(KL0(JROF,JLEV,1)+IV1L2)-PXSL5(KL0(JROF,JLEV,1)+IV1L1) )  
    ZSUPLO1=PXSL(KL0(JROF,JLEV,1)+IV1L1) + ZDLO15*&
     & ( PXSL(KL0(JROF,JLEV,1)+IV1L2)-PXSL(KL0(JROF,JLEV,1)+IV1L1) )&
     & +ZDLO1*&
     & ( PXSL5(KL0(JROF,JLEV,1)+IV1L2)-PXSL5(KL0(JROF,JLEV,1)+IV1L1) )  
    ZSUPLO25=PXSL5(KL0(JROF,JLEV,2)+IV1L1) + ZDLO25*&
     & ( PXSL5(KL0(JROF,JLEV,2)+IV1L2)-PXSL5(KL0(JROF,JLEV,2)+IV1L1) )  
    ZSUPLO2=PXSL(KL0(JROF,JLEV,2)+IV1L1) + ZDLO25*&
     & ( PXSL(KL0(JROF,JLEV,2)+IV1L2)-PXSL(KL0(JROF,JLEV,2)+IV1L1) )&
     & +ZDLO2*&
     & ( PXSL5(KL0(JROF,JLEV,2)+IV1L2)-PXSL5(KL0(JROF,JLEV,2)+IV1L1) )  
    ZSUP5  =  ZSUPLO15+ ZDLAT5*(ZSUPLO25-ZSUPLO15)
    ZSUP   =  ZSUPLO1 + ZDLAT5*(ZSUPLO2-ZSUPLO1) &
     & + ZDLAT*(ZSUPLO25-ZSUPLO15)  

    ZINFLO15=PXSL5(KL0(JROF,JLEV,1)+IV2L1) + ZDLO15*&
     & ( PXSL5(KL0(JROF,JLEV,1)+IV2L2)-PXSL5(KL0(JROF,JLEV,1)+IV2L1) )  
    ZINFLO1=PXSL(KL0(JROF,JLEV,1)+IV2L1) + ZDLO15*&
     & ( PXSL(KL0(JROF,JLEV,1)+IV2L2)-PXSL(KL0(JROF,JLEV,1)+IV2L1) )&
     & +ZDLO1*&
     & ( PXSL5(KL0(JROF,JLEV,1)+IV2L2)-PXSL5(KL0(JROF,JLEV,1)+IV2L1) )  
    ZINFLO25=PXSL5(KL0(JROF,JLEV,2)+IV2L1) + ZDLO25*&
     & ( PXSL5(KL0(JROF,JLEV,2)+IV2L2)-PXSL5(KL0(JROF,JLEV,2)+IV2L1) )  
    ZINFLO2=PXSL(KL0(JROF,JLEV,2)+IV2L1) + ZDLO25*&
     & ( PXSL(KL0(JROF,JLEV,2)+IV2L2)-PXSL(KL0(JROF,JLEV,2)+IV2L1) )&
     & +ZDLO2*&
     & ( PXSL5(KL0(JROF,JLEV,2)+IV2L2)-PXSL5(KL0(JROF,JLEV,2)+IV2L1) )  
    ZINF5  =  ZINFLO15+ ZDLAT5*(ZINFLO25-ZINFLO15)
    ZINF   =  ZINFLO1 + ZDLAT5*(ZINFLO2-ZINFLO1) &
     & + ZDLAT*(ZINFLO25-ZINFLO15)  

    PXF5(JROF,JLEV)= ZSUP5 + ZDVER5*(ZINF5-ZSUP5)
    PXF(JROF,JLEV)= ZSUP + ZDVER5*(ZINF-ZSUP) +ZDVER*(ZINF5-ZSUP5)

  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITLITL',1,ZHOOK_HANDLE)
END SUBROUTINE LAITLITL

