SUBROUTINE LAITRE_GMV( &
 ! --- INPUT ---------------------------------------------------------
 & KASLB1,KPROMA,KSTART,KPROF,KFLEV,KFLDN,KFLDX, &
 & LDSLHD,LDQM,LDQMH, &
 & KL0,PDLO,PCLO,PCLOSLD, &
 & PDLAT,PCLA,PCLASLD,PVINTW,PVINTWSLD, &
 & PXSL, &
 ! --- OUTPUT --------------------------------------------------------
 & POUT)

! Purpose :
! -------
!   LAITRE_GMV - semi-Lagrangian scheme: origin point interpolations 
!   for GMV and some miscellaneous variables.

! Interface :
! ---------
!   INPUT:
!     KASLB1     - horizontal dimension of SLBUF1
!     KPROMA     - horizontal dimension
!     KSTART     - first element of arrays where computations are performed
!     KPROF      - depth of work
!     KFLEV      - vertical dimension
!     KFLDN      - number of the first field
!     KFLDX      - number of the last field
!     LDSLHD     - switch: .T. = diffusive interpolation (SLHD)
!                          .F. = normal interpolation
!     LDQM       - use three-dimensional quasi-monotone interpolation
!     LDQMH      - use quasi-monotone interpolation in the horizontal
!     KL0        - indices of the four western points of the 16 point 
!                  interpolation grid
!     PDLO       - distances for horizontal linear interpolations
!                  in longitude (latitude rows 0, 1, 2, 3)
!     PCLO       - weights for horizontal cubic interpolations in longitude
!                  (latitude rows 1, 2)
!     PCLOSLD    - weights for horizontal cubic interpolations in longitude,
!                  SLHD case (latitude rows 1, 2)
!     PDLAT      - distance for horizontal linear interpolations in latitude
!     PCLA       - weights for horizontal cubic interpolations in latitude
!     PCLASLD    - weights for horizontal cubic interpolations in latitude,
!                  SLHD case
!     PVINTW     - weights for cubic vertical interpolation
!     PVINTWSLD  - weights for cubic vertical interpolation, SLHD case
!     PXSL       - quantity to interpolate at O
!   OUTPUT:
!     POUT       - interpolated quantity at O

! Externals :
! ---------
!   LAITRI

! Method :
! ------
!   See documentation.

! Reference :
! ---------

! Author :
! ------
!   ??-Nov-2004 F. Vana (CHMI) and K. Yessad (Meteo-France)

! Modifications :
! -------------
!    28-Aug-2007  F. Vana  cleaning out the 4-points splines
!    30-06-2008 J. Masek   Dataflow for new SLHD interpolators.
!    K. Yessad Nov 2008: rationalisation of dummy argument interfaces
!    K. Yessad Nov 2008: interpolation routines: merge QM with not-QM version.
! End Modifications
!    ----------------------------------------------------------------------

USE PARKIND1  , ONLY : JPIM     ,JPRB
USE YOMHOOK   , ONLY : LHOOK    ,DR_HOOK

!    ----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KASLB1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LDSLHD
LOGICAL           ,INTENT(IN)    :: LDQM
LOGICAL           ,INTENT(IN)    :: LDQMH
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMA,KFLEV,0:3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO(KPROMA,KFLEV,0:3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO(KPROMA,KFLEV,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOSLD(KPROMA,KFLEV,3,2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLASLD(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTW(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTWSLD(KPROMA,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL(KASLB1,KFLDN:KFLDX) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POUT(KPROMA,KFLEV) 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!    ----------------------------------------------------------------------

#include "laitri.intfb.h"

!-------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITRE_GMV',0,ZHOOK_HANDLE)

!    ----------------------------------------------------------------------

! 1. Origin point interpolations for GMV and some miscellaneous variables
!    --------------------------------------------------------------------

IF (LDSLHD) THEN
  IF (LDQM) THEN
    CALL LAITRI(KASLB1,KPROMA,KSTART,KPROF,KFLEV,KFLDN,KFLDX,2, &
     & PDLAT,PCLASLD,PDLO,PCLOSLD,KL0(1,1,0),PVINTWSLD, &
     & PXSL,POUT)
  ELSEIF (LDQMH) THEN
    CALL LAITRI(KASLB1,KPROMA,KSTART,KPROF,KFLEV,KFLDN,KFLDX,1, &
     & PDLAT,PCLASLD,PDLO,PCLOSLD,KL0(1,1,0),PVINTWSLD, &
     & PXSL,POUT)
  ELSE
    CALL LAITRI(KASLB1,KPROMA,KSTART,KPROF,KFLEV,KFLDN,KFLDX,0, &
     & PDLAT,PCLASLD,PDLO,PCLOSLD,KL0(1,1,0),PVINTWSLD, &
     & PXSL,POUT)  
  ENDIF
ELSE
  IF (LDQM) THEN
    CALL LAITRI(KASLB1,KPROMA,KSTART,KPROF,KFLEV,KFLDN,KFLDX,2, &
     & PDLAT,PCLA,PDLO,PCLO,KL0(1,1,0),PVINTW, &
     & PXSL,POUT)
  ELSEIF (LDQMH) THEN
    CALL LAITRI(KASLB1,KPROMA,KSTART,KPROF,KFLEV,KFLDN,KFLDX,1, &
     & PDLAT,PCLA,PDLO,PCLO,KL0(1,1,0),PVINTW, &
     & PXSL,POUT)
  ELSE
    CALL LAITRI(KASLB1,KPROMA,KSTART,KPROF,KFLEV,KFLDN,KFLDX,0, &
     & PDLAT,PCLA,PDLO,PCLO,KL0(1,1,0),PVINTW, &
     & PXSL,POUT)
  ENDIF
ENDIF

!    ----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITRE_GMV',1,ZHOOK_HANDLE)
END SUBROUTINE LAITRE_GMV
