SUBROUTINE LAITRE_GMV_AD(KASLB1,KPROMA,KSTART,KPROF,KFLEV,&
 & KFLDN,KFLDX,KFLD,KSEP,KMAP,PSIGN,&
 & KDEP,LDVECADIN,LDPLANE,LDSLHD,LDADV55,&
 & PDLAT,PCLA,PCLASLD,PDLO,PCLO,PCLOSLD,&
 & PDLAT5,PCLA5,PCLASLD5,PDLO5,PCLO5,PCLOSLD5,&
 & KL0,PVINTW,PVINTWSLD,PVINTW5,PVINTWSLD5,&
 & KINC,PZINC,KDIM,PXSL,PXSL5,PXF) 

!**** *LAITRE_GMV_AD - semi-LAgrangian scheme:   (adjoint version)
!                      wrapper for tri-dimensional 32 points interpolations.
! Purpose :
! -------
!   LAITRE_GMV_AD - semi-Lagrangian scheme: origin point interpolations
!                   for all variables.

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KASLB1  - horizontal dimension for grid-point quantities.
!          KPROMA  - horizontal dimension for interpolation point
!                    quantities.
!          KSTART  - first element of arrays where
!                    computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KFLDN   - number of the first field.
!          KFLDX   - number of the last field.
!          KFLD    - field offset in semi lagrangian buffer
!          KSEP    - vertical separation for updates.
!          KMAP    - mapping array for bit reproducibility
!          PSIGN   - sign (+ or -) to be applied to point (symmetric/
!                    antisymmetric properties at polar latitudes)
!          KDEP    - indication of the interpolation stencil lat. dep. (LAM)
!          LDVECADIN - switch for scalar adjoint interpolation.
!          LDPLANE - switch: .T. = plane geometry; .F. = spherical geometry.
!          LDSLHD  - switch activating SLHD scheme.
!          LDADV55  - field advected without wind increments
!---------------------- trajectory variables --------------------------
!          PDLAT5  - weight (distance) for horizontal linear interpolation
!                    on a same latitude.
!          PCLA5   - weights (distances) for horizontal cubic interpolation
!                    on a same latitude.
!          PCLASLD5- weights (distances) for horizontal cubic interpolation
!                    on a same latitude, SLHD case.
!          PDLO5   - weights (distances) for horizontal cubic interpolation
!                    on a same longitude.
!          PCLO5   - weights for horizontal cubic interpolations in longitude 
!                    (latitude rows 1, 2)
!          PCLOSLD5- weights for horizontal cubic interpolations in longitude 
!                    (latitude rows 1, 2), SLHD case.
!----------------------------------------------------------------------
!          KL0     - index of the four western points
!                    of the 16 points interpolation grid.
!---------------------- trajectory variables --------------------------
!          PVINTW5 - weights for cubic vertical interpolation
!          PVINTWSLD5- weights for cubic vertical interpolation, SLHD case
!----------------------------------------------------------------------
!          KDIM    - first dimension of KINC and PZINC
!---------------------- trajectory variables --------------------------
!          PXSL5   - semi-lagrangian variable.   (trajectory)
!----------------------------------------------------------------------
!          PXF     - interpolated variable.

!        OUTPUT:
!          KINC    - addresses for increments.
!          PZINC   - increments (saved for later).

!        INPUT/OUTPUT:
!          PDLAT   - weight (distance) for horizontal linear interpolation
!                    on a same latitude.
!          PCLA    - weights (distances) for horizontal cubic interpolation
!                    on a same latitude.
!          PCLASLD - weights (distances) for horizontal cubic interpolation
!                    on a same latitude, SLHD case.
!          PDLO    - weights (distances) for horizontal cubic interpolation
!                    on a same longitude.
!          PCLO    - weights for horizontal cubic interpolations in longitude 
!                    (latitude rows 1, 2).
!          PCLOSLD - weights for horizontal cubic interpolations in longitude 
!                    (latitude rows 1, 2), SLHD case.
!          PVINTW  - weights for cubic vertical interpolation
!          PVINTWSLD - weights for cubic vertical interpolation, SLHD case
!          PXSL    - semi-lagrangian variable.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.
!        Called by LARCINBAD.

!     Reference.
!     ----------

!     Author.
!     -------
!        F. Vana (CHMI/LACE)

!     Modifications.
!     --------------
!      Original : 13-Aug-2008
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KASLB1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMAP(KPROMA,32,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSIGN(KPROMA,32,KFLEV) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDEP(KPROMA,KFLEV)
LOGICAL           ,INTENT(IN)    :: LDVECADIN 
LOGICAL           ,INTENT(IN)    :: LDPLANE
LOGICAL           ,INTENT(IN)    :: LDSLHD
LOGICAL           ,INTENT(IN)    :: LDADV55
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLAT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLA(KPROMA,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLASLD(KPROMA,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLO(KPROMA,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLO(KPROMA,KFLEV,3,2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLOSLD(KPROMA,KFLEV,3,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA5(KPROMA,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLASLD5(KPROMA,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO5(KPROMA,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO5(KPROMA,KFLEV,3,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOSLD5(KPROMA,KFLEV,3,2) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMA,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTW(KPROMA,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTWSLD(KPROMA,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTW5(KPROMA,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTWSLD5(KPROMA,KFLEV,3) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KINC(KDIM*KFLEV+1,KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZINC(KDIM*KFLEV+1,KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXSL(KASLB1,KFLDN:KFLDX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL5(KASLB1,KFLDN:KFLDX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXF(KPROMA,KFLEV) 

!-------------------------------------------------------------------------------

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-------------------------------------------------------------------------------

#include "laitritlad.intfb.h"

!-------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITRE_GMV_AD',0,ZHOOK_HANDLE)

!-------------------------------------------------------------------------------

! 1. Origin point interpolations
!    ---------------------------

IF (LDSLHD) THEN
  CALL LAITRITLAD(KASLB1,KPROMA,KSTART,KPROF,KFLEV, &
   & KFLDN,KFLDX,KFLD,KSEP,KMAP,PSIGN,KDEP,LDVECADIN,LDADV55,LDPLANE,&
   & PDLAT,PCLASLD,PDLO,PCLOSLD,PDLAT5,PCLASLD5,PDLO5,PCLOSLD5, &
   & KL0(1,1,0),PVINTWSLD,PVINTWSLD5, &
   & KINC,PZINC,KDIM,PXSL,PXSL5,PXF)
ELSE
  CALL LAITRITLAD(KASLB1,KPROMA,KSTART,KPROF,KFLEV, &
   & KFLDN,KFLDX,KFLD,KSEP,KMAP,PSIGN,KDEP,LDVECADIN,LDADV55,LDPLANE,&
   & PDLAT,PCLA,PDLO,PCLO,PDLAT5,PCLA5,PDLO5,PCLO5, &
   & KL0(1,1,0),PVINTW,PVINTW5, &
   & KINC,PZINC,KDIM,PXSL,PXSL5,PXF)
ENDIF

!-------------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LAITRE_GMV_AD',1,ZHOOK_HANDLE)
END SUBROUTINE LAITRE_GMV_AD
