SUBROUTINE LAITRI(KPROMA,KPROMB,KST,KPROF,KFLEV, &
 & KFLDN,KFLDX,KQM, &
 & PDLAT,PCLA,PDLO,PCLO,KL0,PVINTW, &
 & PXSL,PXF)

! Purpose :
! -------
!   LAITRI - semi-Lagrangian scheme: tri-dimensional 32-point
!   interpolations (with optional quasi-monotonic treatment). Type of high
!   order interpolator fully controlled by weights, low order
!   interpolator always linear.

! Interface :
! ---------
!   INPUT:
!     KPROMA  - horizontal dimension for grid-point quantities
!     KPROMB  - horizontal dimension for interpolation point quantities
!     KST     - first element of arrays where computations are performed
!     KPROF   - depth of work
!     KFLEV   - vertical dimension
!     KFLDN   - number of the first field
!     KFLDX   - number of the last field
!     KQM     - index of monotonicity
!               0: not monotonous interpolation
!               1: horizontally quasi-monotonous interpolation
!               2: quasi-monotonous interpolation
!     PDLAT   - distance for horizontal linear interpolations in latitude
!     PCLA    - weights for horizontal cubic interpolations in latitude
!     PDLO    - distances for horizontal linear interpolations
!               in longitude (latitude rows 0, 1, 2, 3)
!     PCLO    - weights for horizontal cubic interpolations in longitude 
!               (latitude rows 1, 2)
!     KL0     - indices of the four western points of the 16 point
!               interpolation grid
!     PVINTW  - weights for cubic vertical interpolation
!     PXSL    - quantity to be interpolated
!   OUTPUT:
!     PXF     - interpolated variable

! Externals :
! ---------
!   None.

! Method :
! ------
!   See documentation.

! Reference :
! ---------

! Author :
! ------
!   ??-Feb-1992 K. Yessad, after the subroutine LAGINT3 written by
!   Maurice Imbard, Alain Craplet and Michel Rochas,
!   Meteo-France, CNRM/GMAP.

! Modifications :
! -------------
!   04-Nov-1992 M. Hortal (ECMWF) and K. Yessad (CNRM/GMAP)
!   01-Oct-2003 M. Hamrud  CY28 Cleaning.
!   30-Jun-2008 J. Masek   High order interpolator fully driven by weights,
!     computation of all weights moved to (E)LASCAW.
!   03-Sep-2008 K. Yessad  Merge QM with not-QM version.
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB     ,JPIA
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMPLDSW , ONLY : LOPT_SCALAR

!------------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KQM
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT(KPROMB,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA(KPROMB,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO(KPROMB,KFLEV,0:3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO(KPROMB,KFLEV,3,2)
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMB,KFLEV,0:3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTW(KPROMB,KFLEV,3)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL(KPROMA*(KFLDX-KFLDN+1))
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXF(KPROMB,KFLEV)

!------------------------------------------------------------------------------

INTEGER(KIND=JPIA) :: IV0L1, IV0L2, IV1L0, IV1L1, IV1L2, IV1L3, &
 & IV2L0, IV2L1, IV2L2, IV2L3, IV3L1, IV3L2
INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZF111, ZF112, ZF121, ZF122
REAL(KIND=JPRB) :: ZF211, ZF212, ZF221, ZF222
REAL(KIND=JPRB) :: Z0, Z1, Z2, Z3
REAL(KIND=JPRB) :: ZMIN, ZMAX
REAL(KIND=JPRB) :: Z01(KPROMB), Z02(KPROMB)
REAL(KIND=JPRB) :: Z10(KPROMB), Z11(KPROMB), Z12(KPROMB), Z13(KPROMB)
REAL(KIND=JPRB) :: Z20(KPROMB), Z21(KPROMB), Z22(KPROMB), Z23(KPROMB)
REAL(KIND=JPRB) :: Z31(KPROMB), Z32(KPROMB)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!------------------------------------------------------------------------------

REAL(KIND=JPRB) :: X, Y, MAXJ, MINJ
#ifdef RS6K
MAXJ(X,Y)=FSEL(X-Y,X,Y)
MINJ(X,Y)=FSEL(X-Y,Y,X)
#else
MAXJ(X,Y)=MAX(X,Y)
MINJ(X,Y)=MIN(X,Y)
#endif

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITRI',0,ZHOOK_HANDLE)

!------------------------------------------------------------------------------

! 1. Interpolations
!    --------------

! offsets for getting values on interpolation stencil
IV0L1=1
IV0L2=2
IV1L0=  KPROMA
IV1L1=1+KPROMA
IV1L2=2+KPROMA
IV1L3=3+KPROMA
IV2L0=IV1L0+KPROMA
IV2L1=IV1L1+KPROMA
IV2L2=IV1L2+KPROMA
IV2L3=IV1L3+KPROMA
IV3L1=IV2L1+KPROMA
IV3L2=IV2L2+KPROMA

! 32-point interpolations
!CDIR OUTERUNROLL=4
DO JLEV=1,KFLEV

  ! interpolations in longitude
  IF(LOPT_SCALAR) THEN
!CDIR NODEP
    DO JROF=KST,KPROF
      ! interpolations in longitude, stencil level 0
      Z10(JROF)=PXSL(KL0(JROF,JLEV,1)+IV0L1)+PDLO(JROF,JLEV,1) &
       & *(PXSL(KL0(JROF,JLEV,1)+IV0L2)-PXSL(KL0(JROF,JLEV,1)+IV0L1))
      Z20(JROF)=PXSL(KL0(JROF,JLEV,2)+IV0L1)+PDLO(JROF,JLEV,2) &
       & *(PXSL(KL0(JROF,JLEV,2)+IV0L2)-PXSL(KL0(JROF,JLEV,2)+IV0L1))
      ! interpolations in longitude, stencil level 1
      Z01(JROF)=PXSL(KL0(JROF,JLEV,0)+IV1L1)+PDLO(JROF,JLEV,0) &
       & *(PXSL(KL0(JROF,JLEV,0)+IV1L2)-PXSL(KL0(JROF,JLEV,0)+IV1L1))
    ENDDO
    DO JROF=KST,KPROF
      ZF111=PXSL(KL0(JROF,JLEV,1)+IV1L1)
      ZF211=PXSL(KL0(JROF,JLEV,1)+IV1L2)
      Z11(JROF)=PXSL(KL0(JROF,JLEV,1)+IV1L0) &
       & +PCLO(JROF,JLEV,1,1)*(ZF111-PXSL(KL0(JROF,JLEV,1)+IV1L0)) &
       & +PCLO(JROF,JLEV,2,1)*(ZF211-PXSL(KL0(JROF,JLEV,1)+IV1L0)) &
       & +PCLO(JROF,JLEV,3,1)* &
       & (PXSL(KL0(JROF,JLEV,1)+IV1L3)-PXSL(KL0(JROF,JLEV,1)+IV1L0))
      IF ((KQM /= 0)) THEN
        ! bound Z11:
        ZMIN=MINJ(ZF111,ZF211)
        ZMAX=MAXJ(ZF111,ZF211)
        Z11(JROF)=MAXJ(ZMIN,MINJ(ZMAX,Z11(JROF)))
      ENDIF
    ENDDO
    DO JROF=KST,KPROF
      ZF121=PXSL(KL0(JROF,JLEV,2)+IV1L1)
      ZF221=PXSL(KL0(JROF,JLEV,2)+IV1L2)
      Z21(JROF)=PXSL(KL0(JROF,JLEV,2)+IV1L0) &
       & +PCLO(JROF,JLEV,1,2)*(ZF121-PXSL(KL0(JROF,JLEV,2)+IV1L0)) &
       & +PCLO(JROF,JLEV,2,2)*(ZF221-PXSL(KL0(JROF,JLEV,2)+IV1L0)) &
       & +PCLO(JROF,JLEV,3,2)* &
       & (PXSL(KL0(JROF,JLEV,2)+IV1L3)-PXSL(KL0(JROF,JLEV,2)+IV1L0))
      IF ((KQM /= 0)) THEN
        ! bound Z21:
        ZMIN=MINJ(ZF121,ZF221)
        ZMAX=MAXJ(ZF121,ZF221)
        Z21(JROF)=MAXJ(ZMIN,MINJ(ZMAX,Z21(JROF)))
      ENDIF
    ENDDO
    DO JROF=KST,KPROF
      Z31(JROF)=PXSL(KL0(JROF,JLEV,3)+IV1L1)+PDLO(JROF,JLEV,3) &
       & *(PXSL(KL0(JROF,JLEV,3)+IV1L2)-PXSL(KL0(JROF,JLEV,3)+IV1L1))
      ! interpolations in longitude, stencil level 2
      Z02(JROF)=PXSL(KL0(JROF,JLEV,0)+IV2L1)+PDLO(JROF,JLEV,0) &
       & *(PXSL(KL0(JROF,JLEV,0)+IV2L2)-PXSL(KL0(JROF,JLEV,0)+IV2L1))
    ENDDO
    DO JROF=KST,KPROF
      ZF112=PXSL(KL0(JROF,JLEV,1)+IV2L1)
      ZF212=PXSL(KL0(JROF,JLEV,1)+IV2L2)
      Z12(JROF)=PXSL(KL0(JROF,JLEV,1)+IV2L0) &
       & +PCLO(JROF,JLEV,1,1)*(ZF112-PXSL(KL0(JROF,JLEV,1)+IV2L0)) &
       & +PCLO(JROF,JLEV,2,1)*(ZF212-PXSL(KL0(JROF,JLEV,1)+IV2L0)) &
       & +PCLO(JROF,JLEV,3,1)* &
       & (PXSL(KL0(JROF,JLEV,1)+IV2L3)-PXSL(KL0(JROF,JLEV,1)+IV2L0))
      IF ((KQM /= 0)) THEN
        ! bound Z12:
        ZMIN=MINJ(ZF112,ZF212)
        ZMAX=MAXJ(ZF112,ZF212)
        Z12(JROF)=MAXJ(ZMIN,MINJ(ZMAX,Z12(JROF)))
      ENDIF
    ENDDO
    DO JROF=KST,KPROF
      ZF122=PXSL(KL0(JROF,JLEV,2)+IV2L1)
      ZF222=PXSL(KL0(JROF,JLEV,2)+IV2L2)
      Z22(JROF)=PXSL(KL0(JROF,JLEV,2)+IV2L0) &
       & +PCLO(JROF,JLEV,1,2)*(ZF122-PXSL(KL0(JROF,JLEV,2)+IV2L0)) &
       & +PCLO(JROF,JLEV,2,2)*(ZF222-PXSL(KL0(JROF,JLEV,2)+IV2L0)) &
       & +PCLO(JROF,JLEV,3,2)* &
       & (PXSL(KL0(JROF,JLEV,2)+IV2L3)-PXSL(KL0(JROF,JLEV,2)+IV2L0))
      IF ((KQM /= 0)) THEN
        ! bound Z22:
        ZMIN=MINJ(ZF122,ZF222)
        ZMAX=MAXJ(ZF122,ZF222)
        Z22(JROF)=MAXJ(ZMIN,MINJ(ZMAX,Z22(JROF)))
      ENDIF
    ENDDO
    DO JROF=KST,KPROF
      Z32(JROF)=PXSL(KL0(JROF,JLEV,3)+IV2L1)+PDLO(JROF,JLEV,3) &
       & *(PXSL(KL0(JROF,JLEV,3)+IV2L2)-PXSL(KL0(JROF,JLEV,3)+IV2L1))
      ! interpolations in longitude, stencil level 3
      Z13(JROF)=PXSL(KL0(JROF,JLEV,1)+IV3L1)+PDLO(JROF,JLEV,1) &
       & *(PXSL(KL0(JROF,JLEV,1)+IV3L2)-PXSL(KL0(JROF,JLEV,1)+IV3L1))
      Z23(JROF)=PXSL(KL0(JROF,JLEV,2)+IV3L1)+PDLO(JROF,JLEV,2) &
       & *(PXSL(KL0(JROF,JLEV,2)+IV3L2)-PXSL(KL0(JROF,JLEV,2)+IV3L1))
    ENDDO

  ELSE ! Optimised code for NEC

!CDIR NODEP
    DO JROF=KST,KPROF
      ! interpolations in longitude, stencil level 0
      Z10(JROF)=PXSL(KL0(JROF,JLEV,1)+IV0L1)+PDLO(JROF,JLEV,1) &
       & *(PXSL(KL0(JROF,JLEV,1)+IV0L2)-PXSL(KL0(JROF,JLEV,1)+IV0L1))
      Z20(JROF)=PXSL(KL0(JROF,JLEV,2)+IV0L1)+PDLO(JROF,JLEV,2) &
       & *(PXSL(KL0(JROF,JLEV,2)+IV0L2)-PXSL(KL0(JROF,JLEV,2)+IV0L1))
      ! interpolations in longitude, stencil level 1
      Z01(JROF)=PXSL(KL0(JROF,JLEV,0)+IV1L1)+PDLO(JROF,JLEV,0) &
       & *(PXSL(KL0(JROF,JLEV,0)+IV1L2)-PXSL(KL0(JROF,JLEV,0)+IV1L1))
      ZF111      =PXSL(KL0(JROF,JLEV,1)+IV1L1)
      ZF211      =PXSL(KL0(JROF,JLEV,1)+IV1L2)
      Z11(JROF)=PXSL(KL0(JROF,JLEV,1)+IV1L0) &
       & +PCLO(JROF,JLEV,1,1)*(ZF111      -PXSL(KL0(JROF,JLEV,1)+IV1L0)) &
       & +PCLO(JROF,JLEV,2,1)*(ZF211      -PXSL(KL0(JROF,JLEV,1)+IV1L0)) &
       & +PCLO(JROF,JLEV,3,1)* &
       & (PXSL(KL0(JROF,JLEV,1)+IV1L3)-PXSL(KL0(JROF,JLEV,1)+IV1L0))
      ZF121      =PXSL(KL0(JROF,JLEV,2)+IV1L1)
      ZF221      =PXSL(KL0(JROF,JLEV,2)+IV1L2)
      Z21(JROF)=PXSL(KL0(JROF,JLEV,2)+IV1L0) &
       & +PCLO(JROF,JLEV,1,2)*(ZF121      -PXSL(KL0(JROF,JLEV,2)+IV1L0)) &
       & +PCLO(JROF,JLEV,2,2)*(ZF221      -PXSL(KL0(JROF,JLEV,2)+IV1L0)) &
       & +PCLO(JROF,JLEV,3,2)* &
       & (PXSL(KL0(JROF,JLEV,2)+IV1L3)-PXSL(KL0(JROF,JLEV,2)+IV1L0))
      Z31(JROF)=PXSL(KL0(JROF,JLEV,3)+IV1L1)+PDLO(JROF,JLEV,3) &
       & *(PXSL(KL0(JROF,JLEV,3)+IV1L2)-PXSL(KL0(JROF,JLEV,3)+IV1L1))
      ! interpolations in longitude, stencil level 2
      Z02(JROF)=PXSL(KL0(JROF,JLEV,0)+IV2L1)+PDLO(JROF,JLEV,0) &
       & *(PXSL(KL0(JROF,JLEV,0)+IV2L2)-PXSL(KL0(JROF,JLEV,0)+IV2L1))
      ZF112      =PXSL(KL0(JROF,JLEV,1)+IV2L1)
      ZF212      =PXSL(KL0(JROF,JLEV,1)+IV2L2)
      Z12(JROF)=PXSL(KL0(JROF,JLEV,1)+IV2L0) &
       & +PCLO(JROF,JLEV,1,1)*(ZF112      -PXSL(KL0(JROF,JLEV,1)+IV2L0)) &
       & +PCLO(JROF,JLEV,2,1)*(ZF212      -PXSL(KL0(JROF,JLEV,1)+IV2L0)) &
       & +PCLO(JROF,JLEV,3,1)* &
       & (PXSL(KL0(JROF,JLEV,1)+IV2L3)-PXSL(KL0(JROF,JLEV,1)+IV2L0))
      ZF122      =PXSL(KL0(JROF,JLEV,2)+IV2L1)
      ZF222      =PXSL(KL0(JROF,JLEV,2)+IV2L2)
      Z22(JROF)=PXSL(KL0(JROF,JLEV,2)+IV2L0) &
       & +PCLO(JROF,JLEV,1,2)*(ZF122      -PXSL(KL0(JROF,JLEV,2)+IV2L0)) &
       & +PCLO(JROF,JLEV,2,2)*(ZF222      -PXSL(KL0(JROF,JLEV,2)+IV2L0)) &
       & +PCLO(JROF,JLEV,3,2)* &
       & (PXSL(KL0(JROF,JLEV,2)+IV2L3)-PXSL(KL0(JROF,JLEV,2)+IV2L0))
      Z32(JROF)=PXSL(KL0(JROF,JLEV,3)+IV2L1)+PDLO(JROF,JLEV,3) &
       & *(PXSL(KL0(JROF,JLEV,3)+IV2L2)-PXSL(KL0(JROF,JLEV,3)+IV2L1))
      ! interpolations in longitude, stencil level 3
      Z13(JROF)=PXSL(KL0(JROF,JLEV,1)+IV3L1)+PDLO(JROF,JLEV,1) &
       & *(PXSL(KL0(JROF,JLEV,1)+IV3L2)-PXSL(KL0(JROF,JLEV,1)+IV3L1))
      Z23(JROF)=PXSL(KL0(JROF,JLEV,2)+IV3L1)+PDLO(JROF,JLEV,2) &
       & *(PXSL(KL0(JROF,JLEV,2)+IV3L2)-PXSL(KL0(JROF,JLEV,2)+IV3L1))
  
      IF ((KQM == 1).OR.(KQM == 2)) THEN
        ! bound Z11:
        ZMIN=MINJ(ZF111      ,ZF211      )
        ZMAX=MAXJ(ZF111      ,ZF211      )
        Z11(JROF)=MAXJ(ZMIN,MINJ(ZMAX,Z11(JROF)))
        ! bound Z21:
        ZMIN=MINJ(ZF121      ,ZF221      )
        ZMAX=MAXJ(ZF121      ,ZF221      )
        Z21(JROF)=MAXJ(ZMIN,MINJ(ZMAX,Z21(JROF)))
        ! bound Z12:
        ZMIN=MINJ(ZF112      ,ZF212      )
        ZMAX=MAXJ(ZF112      ,ZF212      )
        Z12(JROF)=MAXJ(ZMIN,MINJ(ZMAX,Z12(JROF)))
        ! bound Z22:
        ZMIN=MINJ(ZF122      ,ZF222      )
        ZMAX=MAXJ(ZF122      ,ZF222      )
        Z22(JROF)=MAXJ(ZMIN,MINJ(ZMAX,Z22(JROF)))
      ENDIF
    ENDDO

  ENDIF

  DO JROF=KST,KPROF
    ! interpolations in latitude, stencil levels 0, 1, 2, 3
    Z0=Z10(JROF)+PDLAT(JROF,JLEV)*(Z20(JROF)-Z10(JROF))
    Z1=Z01(JROF)+PCLA(JROF,JLEV,1)*(Z11(JROF)-Z01(JROF)) &
     & +PCLA(JROF,JLEV,2)*(Z21(JROF)-Z01(JROF)) &
     & +PCLA(JROF,JLEV,3)*(Z31(JROF)-Z01(JROF))
    Z2=Z02(JROF)+PCLA(JROF,JLEV,1)*(Z12(JROF)-Z02(JROF)) &
     & +PCLA(JROF,JLEV,2)*(Z22(JROF)-Z02(JROF)) &
     & +PCLA(JROF,JLEV,3)*(Z32(JROF)-Z02(JROF))
    Z3=Z13(JROF)+PDLAT(JROF,JLEV)*(Z23(JROF)-Z13(JROF))
    IF ((KQM /= 0)) THEN
      ! bound Z1:
      ZMIN=MINJ(Z11(JROF),Z21(JROF))
      ZMAX=MAXJ(Z11(JROF),Z21(JROF))
      Z1=MAXJ(ZMIN,MINJ(ZMAX,Z1))
      ! bound Z2:
      ZMIN=MINJ(Z12(JROF),Z22(JROF))
      ZMAX=MAXJ(Z12(JROF),Z22(JROF))
      Z2=MAXJ(ZMIN,MINJ(ZMAX,Z2))
    ENDIF
    ! final interpolation in vertical
    PXF(JROF,JLEV)=Z0 &
     & +PVINTW(JROF,JLEV,1)*(Z1-Z0) &
     & +PVINTW(JROF,JLEV,2)*(Z2-Z0) &
     & +PVINTW(JROF,JLEV,3)*(Z3-Z0)  
    IF (KQM == 2) THEN
      ! bound PXF:
      ZMIN=MINJ(Z1,Z2)
      ZMAX=MAXJ(Z1,Z2)
      PXF(JROF,JLEV)=MAXJ(ZMIN,MINJ(ZMAX,PXF(JROF,JLEV)))
    ENDIF
  ENDDO

ENDDO
!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITRI',1,ZHOOK_HANDLE)
END SUBROUTINE LAITRI

