SUBROUTINE LAITRITLAD(KPROMA,KPROMB,KSTART,KPROF,KFLEV,&
 & KFLDN,KFLDX,KFLD,KSEP,KMAP,PSIGN,KDEP,LDVECADIN,LDADV55,LDPLANE,&
 & PDLAT,PCLA,PDLO,PCLO,PDLAT5,PCLA5,PDLO5,PCLO5,KL0,&
 & PVINTW,PVINTW5,&
 & KINC,PZINC,KDIM,PXSL,PXSL5,PXF)  

!**** *LAITRITLAD - semi-LAgrangian scheme:   (adjoint version)
!                 TRI-dimensional 32 points interpolations.

!     Purpose.
!     --------
!       Performs tri-dimensional 32 points interpolations.

!**   Interface.
!     ----------
!        *CALL* *LAITRITLAD(KPROMA,KPROMB,KSTART,KPROF,KFLEV
!                      ,KFLDN,KFLDX,KFLD,KSEP,KMAP,PSIGN,KDEP
!                      ,LDVECADIN,LDADV55,LDPLANE
!                      ,PDLAT,PCLA,PDLO,PCLO,PDLAT5,PCLA5,PDLO5,PCLO5,KL0
!                      ,PVINTW,PVINTW5
!                      ,KINC,PZINC,KDIM,PXSL,PXSL5,PXF)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension for grid-point quantities.
!          KPROMB  - horizontal dimension for interpolation point
!                    quantities.
!          KSTART  - first element of arrays where
!                    computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KFLDN   - number of the first field.
!          KFLDX   - number of the last field.
!          KFLD    - field offset in semi lagrangian buffer
!          KSEP    - vertical separation for updates.
!          KMAP    - mapping array for bit reproducibility
!          PSIGN   - sign (+ or -) to be applied to point (symmetric/
!                    antisymmetric properties at polar latitudes)
!          KDEP    - indication of the interpolation stencil lat. dep. (LAM)
!          LDVECADIN - switch for scalar adjoint interpolation.
!          LDADV55  - field advected without wind increments
!          LDPLANE - switch: .T. = plane geometry; .F. = spherical geometry.
!---------------------- trajectory variables --------------------------
!          PDLAT5  - weight (distance) for horizontal linear interpolation
!                    on a same latitude.
!          PCLA5   - weights (distances) for horizontal cubic interpolation
!                    on a same latitude.
!          PDLO5   - weights (distances) for horizontal cubic interpolation
!                    on a same longitude.
!          PCLO5   - weights for horizontal cubic interpolations in longitude 
!                    (latitude rows 1, 2)
!----------------------------------------------------------------------
!          KL0     - index of the four western points
!                    of the 16 points interpolation grid.
!---------------------- trajectory variables --------------------------
!          PVINTW5 - weights for cubic vertical interpolation
!----------------------------------------------------------------------
!          KDIM    - first dimension of KINC and PZINC
!---------------------- trajectory variables --------------------------
!          PXSL5   - semi-lagrangian variable.   (trajectory)
!----------------------------------------------------------------------
!          PXF     - interpolated variable.

!        OUTPUT:
!          KINC    - addresses for increments.
!          PZINC   - increments (saved for later).

!        INPUT/OUTPUT:
!          PDLAT   - weight (distance) for horizontal linear interpolation
!                    on a same latitude.
!          PCLA    - weights (distances) for horizontal cubic interpolation
!                    on a same latitude.
!          PDLO    - weights (distances) for horizontal cubic interpolation
!                    on a same longitude.
!          PCLO    - weights for horizontal cubic interpolations in longitude 
!                    (latitude rows 1, 2)
!          PVINTW  - weights for cubic vertical interpolation
!          PXSL    - semi-lagrangian variable.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.
!        Called by LARCINBAD.

!     Reference.
!     ----------

!     Author.
!     -------
!        C. Temperton (ECMWF)

!     Modifications.
!     --------------
!      Original : 99/09/22
!      Modified 00-02-04 by C. Temperton: KINC array transposed.
!      Modified 00-02-09 by C. Temperton: KSEP introduced.     
!      Modified 00-02-17 by C. Temperton: more optimization....
!      Modified 00-03-01 by C. Temperton: precompute functions (PFUN5).
!      Modified 00-07-11 by G. Mozdzynski: bit reproducibility support
!      Modified 01-02-27 by C. Temperton: LVECADIN option.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!      Modified 05-02-18 by C. Temperton: Removed redundant computation
!                         and output of interpolated trajectory variable.
!      Modified 07-06-27 by E. Holm     : Field advected without wind 
!                                         increments (LDADV55)
!      Modified 08-Jul-21 by F. Vana: - weights driven interpolation
!                            - integrating LAM vector version (ELAITRITLAD)
!                            - optimization (mainly LDADV55 option)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMSLREP  ,ONLY : LSLADREP
USE YOMHOOK   ,ONLY : LHOOK   ,DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMAP(KPROMB,32,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSIGN(KPROMB,32,KFLEV) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDEP(KPROMB,KFLEV)
LOGICAL           ,INTENT(IN)    :: LDVECADIN 
LOGICAL           ,INTENT(IN)    :: LDADV55
LOGICAL           ,INTENT(IN)    :: LDPLANE
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLAT(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLA(KPROMB,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLO(KPROMB,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLO(KPROMB,KFLEV,3,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT5(KPROMB,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA5(KPROMB,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO5(KPROMB,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO5(KPROMB,KFLEV,3,2) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMB,KFLEV,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTW(KPROMB,KFLEV,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVINTW5(KPROMB,KFLEV,3) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KINC(KDIM*KFLEV+1,KPROMB) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZINC(KDIM*KFLEV+1,KPROMB) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXSL(KPROMA*(KFLDX-KFLDN+1)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL5(KPROMA*(KFLDX-KFLDN+1)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXF(KPROMB,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IV0L1, IV0L2, IV1L0, IV1L1, IV1L2, IV1L3,&
 & IV2L0, IV2L1, IV2L2, IV2L3, IV3L1, IV3L2, &
 & IB, ILEV, JJLEV, JLEV, JROF, IBASE  

REAL(KIND=JPRB) :: ZF1115, ZF2115, ZF1215, ZF2215, ZF1125, ZF2125, &
 & ZF1225, ZF2225, ZDLO05, ZDLO15, ZDLO25, ZDLO35, ZCLO115, ZCLO215, &
 & ZCLO315, ZCLO125, ZCLO225, ZCLO325, ZDLAT5, ZCLA15, ZCLA25, ZCLA35, &
 & Z10, Z20, Z01, Z11, Z21, Z31, Z02, Z12, Z22, Z32, Z13, Z23, &
 & Z105, Z205, Z015, Z115, Z215, Z315, Z025, Z125, Z225, Z325, Z135, Z235, &
 & Z0, Z1, Z2, Z3, Z05, Z15, Z25, Z35

REAL(KIND=JPRB) :: ZSUM(KPROMB,32,KFLEV)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITRITLAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INTERPOLATIONS.
!              ---------------

IBASE=(KFLD-1)*KPROMA

IV0L1=1
IV0L2=2
IV1L0=  KPROMA
IV1L1=1+KPROMA
IV1L2=2+KPROMA
IV1L3=3+KPROMA
IV2L0=IV1L0+KPROMA
IV2L1=IV1L1+KPROMA
IV2L2=IV1L2+KPROMA
IV2L3=IV1L3+KPROMA
IV3L1=IV2L1+KPROMA
IV3L2=IV2L2+KPROMA

ILEV=1

IF( LSLADREP )THEN

  IF (.NOT.LDVECADIN) THEN

    DO JLEV=1,KFLEV

      DO JROF=KSTART,KPROF

       !   Interpolation (trajectory)

        ZF1115=PXSL5(KL0(JROF,JLEV,1)+IV1L1)
        ZF2115=PXSL5(KL0(JROF,JLEV,1)+IV1L2)
        ZF1215=PXSL5(KL0(JROF,JLEV,2)+IV1L1)
        ZF2215=PXSL5(KL0(JROF,JLEV,2)+IV1L2)
        ZF1125=PXSL5(KL0(JROF,JLEV,1)+IV2L1)
        ZF2125=PXSL5(KL0(JROF,JLEV,1)+IV2L2)
        ZF1225=PXSL5(KL0(JROF,JLEV,2)+IV2L1)
        ZF2225=PXSL5(KL0(JROF,JLEV,2)+IV2L2)

        ! interpolation distances in longitude
        ZDLO05=PDLO5(JROF,JLEV,0)
        ZDLO15=PDLO5(JROF,JLEV,1)
        ZDLO25=PDLO5(JROF,JLEV,2)
        ZDLO35=PDLO5(JROF,JLEV,3)

        ! interpolation weights in longitude (latitude rows 1, 2)
        ZCLO115=PCLO5(JROF,JLEV,1,1)
        ZCLO215=PCLO5(JROF,JLEV,2,1)
        ZCLO315=PCLO5(JROF,JLEV,3,1)
        ZCLO125=PCLO5(JROF,JLEV,1,2)
        ZCLO225=PCLO5(JROF,JLEV,2,2)
        ZCLO325=PCLO5(JROF,JLEV,3,2)

        ! interpolation distance in latitude
        ZDLAT5=PDLAT5(JROF,JLEV)

        ! interpolation weights in latitude
        ZCLA15=PCLA5(JROF,JLEV,1)
        ZCLA25=PCLA5(JROF,JLEV,2)
        ZCLA35=PCLA5(JROF,JLEV,3)

        ! interpolations in longitude, stencil level 0
        Z105=PXSL5(KL0(JROF,JLEV,1)+IV0L1)+ZDLO15 &
         & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
        Z205=PXSL5(KL0(JROF,JLEV,2)+IV0L1)+ZDLO25 &
         & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))

        ! interpolations in longitude, stencil level 1
        Z015=PXSL5(KL0(JROF,JLEV,0)+IV1L1)+ZDLO05 &
         & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
        Z115=PXSL5(KL0(JROF,JLEV,1)+IV1L0) &
         & +ZCLO115*(ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
         & +ZCLO215*(ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
         & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
        Z215=PXSL5(KL0(JROF,JLEV,2)+IV1L0) &
         & +ZCLO125*(ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
         & +ZCLO225*(ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
         & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))
        Z315=PXSL5(KL0(JROF,JLEV,3)+IV1L1)+ZDLO35 &
         & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))

        ! interpolations in longitude, stencil level 2
        Z025=PXSL5(KL0(JROF,JLEV,0)+IV2L1)+ZDLO05 &
         & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
        Z125=PXSL5(KL0(JROF,JLEV,1)+IV2L0) &
         & +ZCLO115*(ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
         & +ZCLO215*(ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
         & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
        Z225=PXSL5(KL0(JROF,JLEV,2)+IV2L0) &
         & +ZCLO125*(ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
         & +ZCLO225*(ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
         & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))
        Z325=PXSL5(KL0(JROF,JLEV,3)+IV2L1)+ZDLO35 &
         & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))

        ! interpolations in longitude, stencil level 3
        Z135=PXSL5(KL0(JROF,JLEV,1)+IV3L1)+ZDLO15 &
         & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
        Z235=PXSL5(KL0(JROF,JLEV,2)+IV3L1)+ZDLO25 &
         & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

        ! interpolations in latitude, stencil levels 0, 1, 2, 3
        Z05=Z105+ZDLAT5*(Z205-Z105)
        Z15=Z015+ZCLA15*(Z115-Z015)+ZCLA25*(Z215-Z015)+ZCLA35*(Z315-Z015)
        Z25=Z025+ZCLA15*(Z125-Z025)+ZCLA25*(Z225-Z025)+ZCLA35*(Z325-Z025)
        Z35=Z135+ZDLAT5*(Z235-Z135)

        !   Interpolation (adjoint)

        ! final interpolation in vertical
        Z1=PVINTW5(JROF,JLEV,1)*PXF(JROF,JLEV)
        Z2=PVINTW5(JROF,JLEV,2)*PXF(JROF,JLEV)
        Z3=PVINTW5(JROF,JLEV,3)*PXF(JROF,JLEV)
        Z0=PXF(JROF,JLEV)-Z1-Z2-Z3

        ! interpolations in latitude, stencil levels 0, 1, 2, 3
        Z10=(1._JPRB-ZDLAT5)*Z0
        Z20=ZDLAT5*Z0

        Z11=ZCLA15*Z1
        Z21=ZCLA25*Z1
        Z31=ZCLA35*Z1
        Z01=Z1-Z11-Z21-Z31

        Z12=ZCLA15*Z2
        Z22=ZCLA25*Z2
        Z32=ZCLA35*Z2
        Z02=Z2-Z12-Z22-Z32

        Z13=(1._JPRB-ZDLAT5)*Z3
        Z23=ZDLAT5*Z3

        ! interpolations in longitude, stencil level 3
        ZSUM(JROF,15,JLEV)=PSIGN(JROF,15,JLEV)*(1.0_JPRB-ZDLO15)*Z13
        ZSUM(JROF,16,JLEV)=PSIGN(JROF,16,JLEV)*ZDLO15*Z13
        ZSUM(JROF,27,JLEV)=PSIGN(JROF,27,JLEV)*(1.0_JPRB-ZDLO25)*Z23
        ZSUM(JROF,28,JLEV)=PSIGN(JROF,28,JLEV)*ZDLO25*Z23

        ! interpolations in longitude, stencil level 2
        ZSUM(JROF,3,JLEV)=PSIGN(JROF,3,JLEV)*(1.0_JPRB-ZDLO05)*Z02
        ZSUM(JROF,4,JLEV)=PSIGN(JROF,4,JLEV)*ZDLO05*Z02
        ZSUM(JROF,11,JLEV)=PSIGN(JROF,11,JLEV) &
         & *(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z12
        ZSUM(JROF,12,JLEV)=PSIGN(JROF,12,JLEV)*ZCLO115*Z12
        ZSUM(JROF,13,JLEV)=PSIGN(JROF,13,JLEV)*ZCLO215*Z12
        ZSUM(JROF,14,JLEV)=PSIGN(JROF,14,JLEV)*ZCLO315*Z12
        ZSUM(JROF,23,JLEV)=PSIGN(JROF,23,JLEV) &
         & *(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z22
        ZSUM(JROF,24,JLEV)=PSIGN(JROF,24,JLEV)*ZCLO125*Z22
        ZSUM(JROF,25,JLEV)=PSIGN(JROF,25,JLEV)*ZCLO225*Z22
        ZSUM(JROF,26,JLEV)=PSIGN(JROF,26,JLEV)*ZCLO325*Z22
        ZSUM(JROF,31,JLEV)=PSIGN(JROF,31,JLEV)*(1.0_JPRB-ZDLO35)*Z32
        ZSUM(JROF,32,JLEV)=PSIGN(JROF,32,JLEV)*ZDLO35*Z32

        ! interpolations in longitude, stencil level 1
        ZSUM(JROF,1,JLEV)=PSIGN(JROF,1,JLEV)*(1.0_JPRB-ZDLO05)*Z01
        ZSUM(JROF,2,JLEV)=PSIGN(JROF,2,JLEV)*ZDLO05*Z01
        ZSUM(JROF,7,JLEV)=PSIGN(JROF,7,JLEV) &
         & *(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z11
        ZSUM(JROF,8,JLEV)=PSIGN(JROF,8,JLEV)*ZCLO115*Z11
        ZSUM(JROF,9,JLEV)=PSIGN(JROF,9,JLEV)*ZCLO215*Z11
        ZSUM(JROF,10,JLEV)=PSIGN(JROF,10,JLEV)*ZCLO315*Z11
        ZSUM(JROF,19,JLEV)=PSIGN(JROF,19,JLEV) &
         & *(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z21
        ZSUM(JROF,20,JLEV)=PSIGN(JROF,20,JLEV)*ZCLO125*Z21
        ZSUM(JROF,21,JLEV)=PSIGN(JROF,21,JLEV)*ZCLO225*Z21
        ZSUM(JROF,22,JLEV)=PSIGN(JROF,22,JLEV)*ZCLO325*Z21
        ZSUM(JROF,29,JLEV)=PSIGN(JROF,29,JLEV)*(1.0_JPRB-ZDLO35)*Z31
        ZSUM(JROF,30,JLEV)=PSIGN(JROF,30,JLEV)*ZDLO35*Z31

        ! interpolations in longitude, stencil level 0
        ZSUM(JROF,5,JLEV)=PSIGN(JROF,5,JLEV)*(1.0_JPRB-ZDLO15)*Z10
        ZSUM(JROF,6,JLEV)=PSIGN(JROF,6,JLEV)*ZDLO15*Z10
        ZSUM(JROF,17,JLEV)=PSIGN(JROF,17,JLEV)*(1.0_JPRB-ZDLO25)*Z20
        ZSUM(JROF,18,JLEV)=PSIGN(JROF,18,JLEV)*ZDLO25*Z20

        IF (.NOT. LDADV55) THEN
          ! final interpolation in vertical
          PVINTW(JROF,JLEV,1)=PVINTW(JROF,JLEV,1)+(Z15-Z05)*PXF(JROF,JLEV)
          PVINTW(JROF,JLEV,2)=PVINTW(JROF,JLEV,2)+(Z25-Z05)*PXF(JROF,JLEV)
          PVINTW(JROF,JLEV,3)=PVINTW(JROF,JLEV,3)+(Z35-Z05)*PXF(JROF,JLEV)

          ! interpolations in latitude, stencil levels 0, 1, 2, 3
          PDLAT(JROF,JLEV) =PDLAT(JROF,JLEV) +(Z205-Z105)*Z0+(Z235-Z135)*Z3
          PCLA(JROF,JLEV,1)=PCLA(JROF,JLEV,1)+(Z115-Z015)*Z1+(Z125-Z025)*Z2
          PCLA(JROF,JLEV,2)=PCLA(JROF,JLEV,2)+(Z215-Z015)*Z1+(Z225-Z025)*Z2
          PCLA(JROF,JLEV,3)=PCLA(JROF,JLEV,3)+(Z315-Z015)*Z1+(Z325-Z025)*Z2
  
          ! interpolations in longitude, stencil level 3
          PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z13 &
           & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
          PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z23 &
           & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

          ! interpolations in longitude, stencil level 2
          PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z02 &
           & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
          PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z32 &
           & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))
          PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
           & + (ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
          PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
           & + (ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
          PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z12 &
           & *(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
          PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
           & + (ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
          PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
           & + (ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
          PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z22 &
           & *(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))

          ! interpolations in longitude, stencil level 1
          PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z01 &
           & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
          PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z31 &
           & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))
          PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
           & + (ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
          PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
           & + (ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
          PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z11 &
           & *(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
          PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
           & + (ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
          PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
           & + (ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
          PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z21 &
           & *(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))

          ! interpolations in longitude, stencil level 0
          PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z10 &
           & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
          PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z20 &
           & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))
        ENDIF
 
      ENDDO
    ENDDO

    DO JROF=KSTART,KPROF
      DO JLEV=1,KFLEV
        PXSL(KMAP(JROF,1,JLEV))=PXSL(KMAP(JROF,1,JLEV))+ZSUM(JROF,1,JLEV)
        PXSL(KMAP(JROF,2,JLEV))=PXSL(KMAP(JROF,2,JLEV))+ZSUM(JROF,2,JLEV)
        PXSL(KMAP(JROF,3,JLEV))=PXSL(KMAP(JROF,3,JLEV))+ZSUM(JROF,3,JLEV)
        PXSL(KMAP(JROF,4,JLEV))=PXSL(KMAP(JROF,4,JLEV))+ZSUM(JROF,4,JLEV)
        PXSL(KMAP(JROF,5,JLEV))=PXSL(KMAP(JROF,5,JLEV))+ZSUM(JROF,5,JLEV)
        PXSL(KMAP(JROF,6,JLEV))=PXSL(KMAP(JROF,6,JLEV))+ZSUM(JROF,6,JLEV)
        PXSL(KMAP(JROF,7,JLEV))=PXSL(KMAP(JROF,7,JLEV))+ZSUM(JROF,7,JLEV)
        PXSL(KMAP(JROF,8,JLEV))=PXSL(KMAP(JROF,8,JLEV))+ZSUM(JROF,8,JLEV)
        PXSL(KMAP(JROF,9,JLEV))=PXSL(KMAP(JROF,9,JLEV))+ZSUM(JROF,9,JLEV)
        PXSL(KMAP(JROF,10,JLEV))=PXSL(KMAP(JROF,10,JLEV))+ZSUM(JROF,10,JLEV)
        PXSL(KMAP(JROF,11,JLEV))=PXSL(KMAP(JROF,11,JLEV))+ZSUM(JROF,11,JLEV)
        PXSL(KMAP(JROF,12,JLEV))=PXSL(KMAP(JROF,12,JLEV))+ZSUM(JROF,12,JLEV)
        PXSL(KMAP(JROF,13,JLEV))=PXSL(KMAP(JROF,13,JLEV))+ZSUM(JROF,13,JLEV)
        PXSL(KMAP(JROF,14,JLEV))=PXSL(KMAP(JROF,14,JLEV))+ZSUM(JROF,14,JLEV)
        PXSL(KMAP(JROF,15,JLEV))=PXSL(KMAP(JROF,15,JLEV))+ZSUM(JROF,15,JLEV)
        PXSL(KMAP(JROF,16,JLEV))=PXSL(KMAP(JROF,16,JLEV))+ZSUM(JROF,16,JLEV)
        PXSL(KMAP(JROF,17,JLEV))=PXSL(KMAP(JROF,17,JLEV))+ZSUM(JROF,17,JLEV)
        PXSL(KMAP(JROF,18,JLEV))=PXSL(KMAP(JROF,18,JLEV))+ZSUM(JROF,18,JLEV)
        PXSL(KMAP(JROF,19,JLEV))=PXSL(KMAP(JROF,19,JLEV))+ZSUM(JROF,19,JLEV)
        PXSL(KMAP(JROF,20,JLEV))=PXSL(KMAP(JROF,20,JLEV))+ZSUM(JROF,20,JLEV)
        PXSL(KMAP(JROF,21,JLEV))=PXSL(KMAP(JROF,21,JLEV))+ZSUM(JROF,21,JLEV)
        PXSL(KMAP(JROF,22,JLEV))=PXSL(KMAP(JROF,22,JLEV))+ZSUM(JROF,22,JLEV)
        PXSL(KMAP(JROF,23,JLEV))=PXSL(KMAP(JROF,23,JLEV))+ZSUM(JROF,23,JLEV)
        PXSL(KMAP(JROF,24,JLEV))=PXSL(KMAP(JROF,24,JLEV))+ZSUM(JROF,24,JLEV)
        PXSL(KMAP(JROF,25,JLEV))=PXSL(KMAP(JROF,25,JLEV))+ZSUM(JROF,25,JLEV)
        PXSL(KMAP(JROF,26,JLEV))=PXSL(KMAP(JROF,26,JLEV))+ZSUM(JROF,26,JLEV)
        PXSL(KMAP(JROF,27,JLEV))=PXSL(KMAP(JROF,27,JLEV))+ZSUM(JROF,27,JLEV)
        PXSL(KMAP(JROF,28,JLEV))=PXSL(KMAP(JROF,28,JLEV))+ZSUM(JROF,28,JLEV)
        PXSL(KMAP(JROF,29,JLEV))=PXSL(KMAP(JROF,29,JLEV))+ZSUM(JROF,29,JLEV)
        PXSL(KMAP(JROF,30,JLEV))=PXSL(KMAP(JROF,30,JLEV))+ZSUM(JROF,30,JLEV)
        PXSL(KMAP(JROF,31,JLEV))=PXSL(KMAP(JROF,31,JLEV))+ZSUM(JROF,31,JLEV)
        PXSL(KMAP(JROF,32,JLEV))=PXSL(KMAP(JROF,32,JLEV))+ZSUM(JROF,32,JLEV)
      ENDDO
    ENDDO

  ELSE

    IF (LDPLANE) THEN

      DO JJLEV=1,KSEP 
        DO JLEV=JJLEV,KFLEV,KSEP

          IB=(ILEV-1)*KDIM

!CDIR NODEP
          DO JROF=KSTART,KPROF

           !   Interpolation (trajectory)

            ZF1115=PXSL5(KL0(JROF,JLEV,1)+IV1L1)
            ZF2115=PXSL5(KL0(JROF,JLEV,1)+IV1L2)
            ZF1215=PXSL5(KL0(JROF,JLEV,2)+IV1L1)
            ZF2215=PXSL5(KL0(JROF,JLEV,2)+IV1L2)
            ZF1125=PXSL5(KL0(JROF,JLEV,1)+IV2L1)
            ZF2125=PXSL5(KL0(JROF,JLEV,1)+IV2L2)
            ZF1225=PXSL5(KL0(JROF,JLEV,2)+IV2L1)
            ZF2225=PXSL5(KL0(JROF,JLEV,2)+IV2L2)

            ! interpolation distances in longitude
            ZDLO05=PDLO5(JROF,JLEV,0)
            ZDLO15=PDLO5(JROF,JLEV,1)
            ZDLO25=PDLO5(JROF,JLEV,2)
            ZDLO35=PDLO5(JROF,JLEV,3)

            ! interpolation weights in longitude (latitude rows 1, 2)
            ZCLO115=PCLO5(JROF,JLEV,1,1)
            ZCLO215=PCLO5(JROF,JLEV,2,1)
            ZCLO315=PCLO5(JROF,JLEV,3,1)
            ZCLO125=PCLO5(JROF,JLEV,1,2)
            ZCLO225=PCLO5(JROF,JLEV,2,2)
            ZCLO325=PCLO5(JROF,JLEV,3,2)

            ! interpolation distance in latitude
            ZDLAT5=PDLAT5(JROF,JLEV)

            ! interpolation weights in latitude
            ZCLA15=PCLA5(JROF,JLEV,1)
            ZCLA25=PCLA5(JROF,JLEV,2)
            ZCLA35=PCLA5(JROF,JLEV,3)

            ! interpolations in longitude, stencil level 0
            Z105=PXSL5(KL0(JROF,JLEV,1)+IV0L1)+ZDLO15 &
             & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
            Z205=PXSL5(KL0(JROF,JLEV,2)+IV0L1)+ZDLO25 &
             & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))

            ! interpolations in longitude, stencil level 1
            Z015=PXSL5(KL0(JROF,JLEV,0)+IV1L1)+ZDLO05 &
             & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
            Z115=PXSL5(KL0(JROF,JLEV,1)+IV1L0) &
             & +ZCLO115*(ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
             & +ZCLO215*(ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
             & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
            Z215=PXSL5(KL0(JROF,JLEV,2)+IV1L0) &
             & +ZCLO125*(ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
             & +ZCLO225*(ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
             & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))
            Z315=PXSL5(KL0(JROF,JLEV,3)+IV1L1)+ZDLO35 &
             & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))

            ! interpolations in longitude, stencil level 2
            Z025=PXSL5(KL0(JROF,JLEV,0)+IV2L1)+ZDLO05 &
             & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
            Z125=PXSL5(KL0(JROF,JLEV,1)+IV2L0) &
             & +ZCLO115*(ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
             & +ZCLO215*(ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
             & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
            Z225=PXSL5(KL0(JROF,JLEV,2)+IV2L0) &
             & +ZCLO125*(ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
             & +ZCLO225*(ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
             & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))
            Z325=PXSL5(KL0(JROF,JLEV,3)+IV2L1)+ZDLO35 &
             & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))

            ! interpolations in longitude, stencil level 3
            Z135=PXSL5(KL0(JROF,JLEV,1)+IV3L1)+ZDLO15 &
             & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
            Z235=PXSL5(KL0(JROF,JLEV,2)+IV3L1)+ZDLO25 &
             & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

            ! interpolations in latitude, stencil levels 0, 1, 2, 3
            Z05=Z105+ZDLAT5*(Z205-Z105)
            Z15=Z015+ZCLA15*(Z115-Z015)+ZCLA25*(Z215-Z015)+ZCLA35*(Z315-Z015)
            Z25=Z025+ZCLA15*(Z125-Z025)+ZCLA25*(Z225-Z025)+ZCLA35*(Z325-Z025)
            Z35=Z135+ZDLAT5*(Z235-Z135)

            !   Interpolation (adjoint)

            KINC(IB+1,JROF)=IBASE+KMAP(JROF,1,JLEV)
            KINC(IB+2,JROF)=IBASE+KMAP(JROF,2,JLEV)
            KINC(IB+3,JROF)=IBASE+KMAP(JROF,3,JLEV)
            KINC(IB+4,JROF)=IBASE+KMAP(JROF,4,JLEV)
            KINC(IB+5,JROF)=IBASE+KMAP(JROF,5,JLEV)
            KINC(IB+6,JROF)=IBASE+KMAP(JROF,6,JLEV)
            KINC(IB+7,JROF)=IBASE+KMAP(JROF,7,JLEV)
            KINC(IB+8,JROF)=IBASE+KMAP(JROF,8,JLEV)
            KINC(IB+9,JROF)=IBASE+KMAP(JROF,9,JLEV)
            KINC(IB+10,JROF)=IBASE+KMAP(JROF,10,JLEV)
            KINC(IB+11,JROF)=IBASE+KMAP(JROF,11,JLEV)
            KINC(IB+12,JROF)=IBASE+KMAP(JROF,12,JLEV)
            KINC(IB+13,JROF)=IBASE+KMAP(JROF,13,JLEV)
            KINC(IB+14,JROF)=IBASE+KMAP(JROF,14,JLEV)
            KINC(IB+15,JROF)=IBASE+KMAP(JROF,15,JLEV)
            KINC(IB+16,JROF)=IBASE+KMAP(JROF,16,JLEV)
            KINC(IB+17,JROF)=IBASE+KMAP(JROF,17,JLEV)
            KINC(IB+18,JROF)=IBASE+KMAP(JROF,18,JLEV)
            KINC(IB+19,JROF)=IBASE+KMAP(JROF,19,JLEV)
            KINC(IB+20,JROF)=IBASE+KMAP(JROF,20,JLEV)
            KINC(IB+21,JROF)=IBASE+KMAP(JROF,21,JLEV)
            KINC(IB+22,JROF)=IBASE+KMAP(JROF,22,JLEV)
            KINC(IB+23,JROF)=IBASE+KMAP(JROF,23,JLEV)
            KINC(IB+24,JROF)=IBASE+KMAP(JROF,24,JLEV)
            KINC(IB+25,JROF)=IBASE+KMAP(JROF,25,JLEV)
            KINC(IB+26,JROF)=IBASE+KMAP(JROF,26,JLEV)
            KINC(IB+27,JROF)=IBASE+KMAP(JROF,27,JLEV)
            KINC(IB+28,JROF)=IBASE+KMAP(JROF,28,JLEV)
            KINC(IB+29,JROF)=IBASE+KMAP(JROF,29,JLEV)
            KINC(IB+30,JROF)=IBASE+KMAP(JROF,30,JLEV)
            KINC(IB+31,JROF)=IBASE+KMAP(JROF,31,JLEV)
            KINC(IB+32,JROF)=IBASE+KMAP(JROF,32,JLEV)

            ! final interpolation in vertical
            Z1=PVINTW5(JROF,JLEV,1)*PXF(JROF,JLEV)
            Z2=PVINTW5(JROF,JLEV,2)*PXF(JROF,JLEV)
            Z3=PVINTW5(JROF,JLEV,3)*PXF(JROF,JLEV)
            Z0=PXF(JROF,JLEV)-Z1-Z2-Z3

            ! interpolations in latitude, stencil levels 0, 1, 2, 3
            Z10=(1._JPRB-ZDLAT5)*Z0
            Z20=ZDLAT5*Z0

            Z11=ZCLA15*Z1
            Z21=ZCLA25*Z1
            Z31=ZCLA35*Z1
            Z01=Z1-Z11-Z21-Z31

            Z12=ZCLA15*Z2
            Z22=ZCLA25*Z2
            Z32=ZCLA35*Z2
            Z02=Z2-Z12-Z22-Z32

            Z13=(1._JPRB-ZDLAT5)*Z3
            Z23=ZDLAT5*Z3

            ! interpolations in longitude, stencil level 3
            PZINC(IB+15,JROF)=PSIGN(JROF,15,JLEV)*(1.0_JPRB-ZDLO15)*Z13
            PZINC(IB+16,JROF)=PSIGN(JROF,16,JLEV)*ZDLO15*Z13
            PZINC(IB+27,JROF)=PSIGN(JROF,27,JLEV)*(1.0_JPRB-ZDLO25)*Z23
            PZINC(IB+28,JROF)=PSIGN(JROF,28,JLEV)*ZDLO25*Z23

            ! interpolations in longitude, stencil level 2
            PZINC(IB+3,JROF)=PSIGN(JROF,3,JLEV)*(1.0_JPRB-ZDLO05)*Z02
            PZINC(IB+4,JROF)=PSIGN(JROF,4,JLEV)*ZDLO05*Z02
            PZINC(IB+11,JROF)=PSIGN(JROF,11,JLEV) &
             & *(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z12
            PZINC(IB+12,JROF)=PSIGN(JROF,12,JLEV)*ZCLO115*Z12
            PZINC(IB+13,JROF)=PSIGN(JROF,13,JLEV)*ZCLO215*Z12
            PZINC(IB+14,JROF)=PSIGN(JROF,14,JLEV)*ZCLO315*Z12
            PZINC(IB+23,JROF)=PSIGN(JROF,23,JLEV) &
             & *(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z22
            PZINC(IB+24,JROF)=PSIGN(JROF,24,JLEV)*ZCLO125*Z22
            PZINC(IB+25,JROF)=PSIGN(JROF,25,JLEV)*ZCLO225*Z22
            PZINC(IB+26,JROF)=PSIGN(JROF,26,JLEV)*ZCLO325*Z22
            PZINC(IB+31,JROF)=PSIGN(JROF,31,JLEV)*(1.0_JPRB-ZDLO35)*Z32
            PZINC(IB+32,JROF)=PSIGN(JROF,32,JLEV)*ZDLO35*Z32

            ! interpolations in longitude, stencil level 1
            PZINC(IB+1,JROF)=PSIGN(JROF,1,JLEV)*(1.0_JPRB-ZDLO05)*Z01
            PZINC(IB+2,JROF)=PSIGN(JROF,2,JLEV)*ZDLO05*Z01
            PZINC(IB+7,JROF)=PSIGN(JROF,7,JLEV) &
             & *(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z11
            PZINC(IB+8,JROF)=PSIGN(JROF,8,JLEV)*ZCLO115*Z11
            PZINC(IB+9,JROF)=PSIGN(JROF,9,JLEV)*ZCLO215*Z11
            PZINC(IB+10,JROF)=PSIGN(JROF,10,JLEV)*ZCLO315*Z11
            PZINC(IB+19,JROF)=PSIGN(JROF,19,JLEV) &
             & *(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z21
            PZINC(IB+20,JROF)=PSIGN(JROF,20,JLEV)*ZCLO125*Z21
            PZINC(IB+21,JROF)=PSIGN(JROF,21,JLEV)*ZCLO225*Z21
            PZINC(IB+22,JROF)=PSIGN(JROF,22,JLEV)*ZCLO325*Z21
            PZINC(IB+29,JROF)=PSIGN(JROF,29,JLEV)*(1.0_JPRB-ZDLO35)*Z31
            PZINC(IB+30,JROF)=PSIGN(JROF,30,JLEV)*ZDLO35*Z31

            ! interpolations in longitude, stencil level 0
            PZINC(IB+5,JROF)=PSIGN(JROF,5,JLEV)*(1.0_JPRB-ZDLO15)*Z10
            PZINC(IB+6,JROF)=PSIGN(JROF,6,JLEV)*ZDLO15*Z10
            PZINC(IB+17,JROF)=PSIGN(JROF,17,JLEV)*(1.0_JPRB-ZDLO25)*Z20
            PZINC(IB+18,JROF)=PSIGN(JROF,18,JLEV)*ZDLO25*Z20

            IF (.NOT. LDADV55) THEN
              ! final interpolation in vertical
              PVINTW(JROF,JLEV,1)=PVINTW(JROF,JLEV,1)+(Z15-Z05)*PXF(JROF,JLEV)
              PVINTW(JROF,JLEV,2)=PVINTW(JROF,JLEV,2)+(Z25-Z05)*PXF(JROF,JLEV)
              PVINTW(JROF,JLEV,3)=PVINTW(JROF,JLEV,3)+(Z35-Z05)*PXF(JROF,JLEV)

              ! interpolations in latitude, stencil levels 0, 1, 2, 3
              PDLAT(JROF,JLEV) =PDLAT(JROF,JLEV) +(Z205-Z105)*Z0+(Z235-Z135)*Z3
              PCLA(JROF,JLEV,1)=PCLA(JROF,JLEV,1)+(Z115-Z015)*Z1+(Z125-Z025)*Z2
              PCLA(JROF,JLEV,2)=PCLA(JROF,JLEV,2)+(Z215-Z015)*Z1+(Z225-Z025)*Z2
              PCLA(JROF,JLEV,3)=PCLA(JROF,JLEV,3)+(Z315-Z015)*Z1+(Z325-Z025)*Z2
  
              ! interpolations in longitude, stencil level 3
              PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z13 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
              PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z23 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

              ! interpolations in longitude, stencil level 2
              PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z02 &
               & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
              PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z32 &
               & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))
              PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
               & + (ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
              PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
               & + (ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
              PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z12 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
              PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
               & + (ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
              PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
               & + (ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
              PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z22 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))

              ! interpolations in longitude, stencil level 1
              PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z01 &
               & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
              PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z31 &
               & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))
              PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
               & + (ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
              PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
               & + (ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
              PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z11 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
              PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
               & + (ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
              PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
               & + (ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
              PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z21 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))

              ! interpolations in longitude, stencil level 0
              PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z10 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
              PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z20 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))
            ENDIF
 
            ! correction of dependency for mirrored points
            IF (KDEP(JROF,JLEV) == 1) THEN
              KINC(IB+1,JROF)=KINC(IB+29,JROF)-1
              KINC(IB+2,JROF)=KINC(IB+30,JROF)+1
              KINC(IB+3,JROF)=KINC(IB+31,JROF)-1
              KINC(IB+4,JROF)=KINC(IB+32,JROF)+1
              !KINC(IB+1:IB+4,JROF)=0
              PZINC(IB+8,JROF)=PZINC(IB+8,JROF)+PZINC(IB+1,JROF)
              PZINC(IB+9,JROF)=PZINC(IB+9,JROF)+PZINC(IB+2,JROF)
              PZINC(IB+12,JROF)=PZINC(IB+12,JROF)+PZINC(IB+3,JROF)
              PZINC(IB+13,JROF)=PZINC(IB+13,JROF)+PZINC(IB+4,JROF)
              PZINC(IB+1,JROF)=0.0_JPRB
              PZINC(IB+2,JROF)=0.0_JPRB
              PZINC(IB+3,JROF)=0.0_JPRB
              PZINC(IB+4,JROF)=0.0_JPRB
            ELSEIF (KDEP(JROF,JLEV) == 4) THEN
              KINC(IB+29,JROF)=KINC(IB+1,JROF)-1
              KINC(IB+30,JROF)=KINC(IB+2,JROF)+1
              KINC(IB+31,JROF)=KINC(IB+3,JROF)-1
              KINC(IB+32,JROF)=KINC(IB+4,JROF)+1
              !KINC(IB+29:IB+32,JROF)=0
              PZINC(IB+20,JROF)=PZINC(IB+20,JROF)+PZINC(IB+29,JROF)
              PZINC(IB+21,JROF)=PZINC(IB+21,JROF)+PZINC(IB+30,JROF)
              PZINC(IB+24,JROF)=PZINC(IB+24,JROF)+PZINC(IB+31,JROF)
              PZINC(IB+25,JROF)=PZINC(IB+25,JROF)+PZINC(IB+32,JROF)
              PZINC(IB+29,JROF)=0.0_JPRB
              PZINC(IB+30,JROF)=0.0_JPRB
              PZINC(IB+31,JROF)=0.0_JPRB
              PZINC(IB+32,JROF)=0.0_JPRB
            ENDIF

          ENDDO
          ILEV=ILEV+1
        ENDDO
      ENDDO

    ELSE  ! sphere (no dependency to correct)

      DO JJLEV=1,KSEP 
        DO JLEV=JJLEV,KFLEV,KSEP

          IB=(ILEV-1)*KDIM

!CDIR NODEP
          DO JROF=KSTART,KPROF

           !   Interpolation (trajectory)

            ZF1115=PXSL5(KL0(JROF,JLEV,1)+IV1L1)
            ZF2115=PXSL5(KL0(JROF,JLEV,1)+IV1L2)
            ZF1215=PXSL5(KL0(JROF,JLEV,2)+IV1L1)
            ZF2215=PXSL5(KL0(JROF,JLEV,2)+IV1L2)
            ZF1125=PXSL5(KL0(JROF,JLEV,1)+IV2L1)
            ZF2125=PXSL5(KL0(JROF,JLEV,1)+IV2L2)
            ZF1225=PXSL5(KL0(JROF,JLEV,2)+IV2L1)
            ZF2225=PXSL5(KL0(JROF,JLEV,2)+IV2L2)

            ! interpolation distances in longitude
            ZDLO05=PDLO5(JROF,JLEV,0)
            ZDLO15=PDLO5(JROF,JLEV,1)
            ZDLO25=PDLO5(JROF,JLEV,2)
            ZDLO35=PDLO5(JROF,JLEV,3)

            ! interpolation weights in longitude (latitude rows 1, 2)
            ZCLO115=PCLO5(JROF,JLEV,1,1)
            ZCLO215=PCLO5(JROF,JLEV,2,1)
            ZCLO315=PCLO5(JROF,JLEV,3,1)
            ZCLO125=PCLO5(JROF,JLEV,1,2)
            ZCLO225=PCLO5(JROF,JLEV,2,2)
            ZCLO325=PCLO5(JROF,JLEV,3,2)

            ! interpolation distance in latitude
            ZDLAT5=PDLAT5(JROF,JLEV)

            ! interpolation weights in latitude
            ZCLA15=PCLA5(JROF,JLEV,1)
            ZCLA25=PCLA5(JROF,JLEV,2)
            ZCLA35=PCLA5(JROF,JLEV,3)

            ! interpolations in longitude, stencil level 0
            Z105=PXSL5(KL0(JROF,JLEV,1)+IV0L1)+ZDLO15 &
             & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
            Z205=PXSL5(KL0(JROF,JLEV,2)+IV0L1)+ZDLO25 &
             & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))

            ! interpolations in longitude, stencil level 1
            Z015=PXSL5(KL0(JROF,JLEV,0)+IV1L1)+ZDLO05 &
             & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
            Z115=PXSL5(KL0(JROF,JLEV,1)+IV1L0) &
             & +ZCLO115*(ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
             & +ZCLO215*(ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
             & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
            Z215=PXSL5(KL0(JROF,JLEV,2)+IV1L0) &
             & +ZCLO125*(ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
             & +ZCLO225*(ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
             & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))
            Z315=PXSL5(KL0(JROF,JLEV,3)+IV1L1)+ZDLO35 &
             & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))

            ! interpolations in longitude, stencil level 2
            Z025=PXSL5(KL0(JROF,JLEV,0)+IV2L1)+ZDLO05 &
             & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
            Z125=PXSL5(KL0(JROF,JLEV,1)+IV2L0) &
             & +ZCLO115*(ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
             & +ZCLO215*(ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
             & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
            Z225=PXSL5(KL0(JROF,JLEV,2)+IV2L0) &
             & +ZCLO125*(ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
             & +ZCLO225*(ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
             & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))
            Z325=PXSL5(KL0(JROF,JLEV,3)+IV2L1)+ZDLO35 &
             & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))

            ! interpolations in longitude, stencil level 3
            Z135=PXSL5(KL0(JROF,JLEV,1)+IV3L1)+ZDLO15 &
             & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
            Z235=PXSL5(KL0(JROF,JLEV,2)+IV3L1)+ZDLO25 &
             & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

            ! interpolations in latitude, stencil levels 0, 1, 2, 3
            Z05=Z105+ZDLAT5*(Z205-Z105)
            Z15=Z015+ZCLA15*(Z115-Z015)+ZCLA25*(Z215-Z015)+ZCLA35*(Z315-Z015)
            Z25=Z025+ZCLA15*(Z125-Z025)+ZCLA25*(Z225-Z025)+ZCLA35*(Z325-Z025)
            Z35=Z135+ZDLAT5*(Z235-Z135)

            !   Interpolation (adjoint)

            KINC(IB+1,JROF)=IBASE+KMAP(JROF,1,JLEV)
            KINC(IB+2,JROF)=IBASE+KMAP(JROF,2,JLEV)
            KINC(IB+3,JROF)=IBASE+KMAP(JROF,3,JLEV)
            KINC(IB+4,JROF)=IBASE+KMAP(JROF,4,JLEV)
            KINC(IB+5,JROF)=IBASE+KMAP(JROF,5,JLEV)
            KINC(IB+6,JROF)=IBASE+KMAP(JROF,6,JLEV)
            KINC(IB+7,JROF)=IBASE+KMAP(JROF,7,JLEV)
            KINC(IB+8,JROF)=IBASE+KMAP(JROF,8,JLEV)
            KINC(IB+9,JROF)=IBASE+KMAP(JROF,9,JLEV)
            KINC(IB+10,JROF)=IBASE+KMAP(JROF,10,JLEV)
            KINC(IB+11,JROF)=IBASE+KMAP(JROF,11,JLEV)
            KINC(IB+12,JROF)=IBASE+KMAP(JROF,12,JLEV)
            KINC(IB+13,JROF)=IBASE+KMAP(JROF,13,JLEV)
            KINC(IB+14,JROF)=IBASE+KMAP(JROF,14,JLEV)
            KINC(IB+15,JROF)=IBASE+KMAP(JROF,15,JLEV)
            KINC(IB+16,JROF)=IBASE+KMAP(JROF,16,JLEV)
            KINC(IB+17,JROF)=IBASE+KMAP(JROF,17,JLEV)
            KINC(IB+18,JROF)=IBASE+KMAP(JROF,18,JLEV)
            KINC(IB+19,JROF)=IBASE+KMAP(JROF,19,JLEV)
            KINC(IB+20,JROF)=IBASE+KMAP(JROF,20,JLEV)
            KINC(IB+21,JROF)=IBASE+KMAP(JROF,21,JLEV)
            KINC(IB+22,JROF)=IBASE+KMAP(JROF,22,JLEV)
            KINC(IB+23,JROF)=IBASE+KMAP(JROF,23,JLEV)
            KINC(IB+24,JROF)=IBASE+KMAP(JROF,24,JLEV)
            KINC(IB+25,JROF)=IBASE+KMAP(JROF,25,JLEV)
            KINC(IB+26,JROF)=IBASE+KMAP(JROF,26,JLEV)
            KINC(IB+27,JROF)=IBASE+KMAP(JROF,27,JLEV)
            KINC(IB+28,JROF)=IBASE+KMAP(JROF,28,JLEV)
            KINC(IB+29,JROF)=IBASE+KMAP(JROF,29,JLEV)
            KINC(IB+30,JROF)=IBASE+KMAP(JROF,30,JLEV)
            KINC(IB+31,JROF)=IBASE+KMAP(JROF,31,JLEV)
            KINC(IB+32,JROF)=IBASE+KMAP(JROF,32,JLEV)

            ! final interpolation in vertical
            Z1=PVINTW5(JROF,JLEV,1)*PXF(JROF,JLEV)
            Z2=PVINTW5(JROF,JLEV,2)*PXF(JROF,JLEV)
            Z3=PVINTW5(JROF,JLEV,3)*PXF(JROF,JLEV)
            Z0=PXF(JROF,JLEV)-Z1-Z2-Z3

            ! interpolations in latitude, stencil levels 0, 1, 2, 3
            Z10=(1._JPRB-ZDLAT5)*Z0
            Z20=ZDLAT5*Z0

            Z11=ZCLA15*Z1
            Z21=ZCLA25*Z1
            Z31=ZCLA35*Z1
            Z01=Z1-Z11-Z21-Z31

            Z12=ZCLA15*Z2
            Z22=ZCLA25*Z2
            Z32=ZCLA35*Z2
            Z02=Z2-Z12-Z22-Z32

            Z13=(1._JPRB-ZDLAT5)*Z3
            Z23=ZDLAT5*Z3

            ! interpolations in longitude, stencil level 3
            PZINC(IB+15,JROF)=PSIGN(JROF,15,JLEV)*(1.0_JPRB-ZDLO15)*Z13
            PZINC(IB+16,JROF)=PSIGN(JROF,16,JLEV)*ZDLO15*Z13
            PZINC(IB+27,JROF)=PSIGN(JROF,27,JLEV)*(1.0_JPRB-ZDLO25)*Z23
            PZINC(IB+28,JROF)=PSIGN(JROF,28,JLEV)*ZDLO25*Z23

            ! interpolations in longitude, stencil level 2
            PZINC(IB+3,JROF)=PSIGN(JROF,3,JLEV)*(1.0_JPRB-ZDLO05)*Z02
            PZINC(IB+4,JROF)=PSIGN(JROF,4,JLEV)*ZDLO05*Z02
            PZINC(IB+11,JROF)=PSIGN(JROF,11,JLEV) &
             & *(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z12
            PZINC(IB+12,JROF)=PSIGN(JROF,12,JLEV)*ZCLO115*Z12
            PZINC(IB+13,JROF)=PSIGN(JROF,13,JLEV)*ZCLO215*Z12
            PZINC(IB+14,JROF)=PSIGN(JROF,14,JLEV)*ZCLO315*Z12
            PZINC(IB+23,JROF)=PSIGN(JROF,23,JLEV) &
             & *(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z22
            PZINC(IB+24,JROF)=PSIGN(JROF,24,JLEV)*ZCLO125*Z22
            PZINC(IB+25,JROF)=PSIGN(JROF,25,JLEV)*ZCLO225*Z22
            PZINC(IB+26,JROF)=PSIGN(JROF,26,JLEV)*ZCLO325*Z22
            PZINC(IB+31,JROF)=PSIGN(JROF,31,JLEV)*(1.0_JPRB-ZDLO35)*Z32
            PZINC(IB+32,JROF)=PSIGN(JROF,32,JLEV)*ZDLO35*Z32

            ! interpolations in longitude, stencil level 1
            PZINC(IB+1,JROF)=PSIGN(JROF,1,JLEV)*(1.0_JPRB-ZDLO05)*Z01
            PZINC(IB+2,JROF)=PSIGN(JROF,2,JLEV)*ZDLO05*Z01
            PZINC(IB+7,JROF)=PSIGN(JROF,7,JLEV) &
             & *(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z11
            PZINC(IB+8,JROF)=PSIGN(JROF,8,JLEV)*ZCLO115*Z11
            PZINC(IB+9,JROF)=PSIGN(JROF,9,JLEV)*ZCLO215*Z11
            PZINC(IB+10,JROF)=PSIGN(JROF,10,JLEV)*ZCLO315*Z11
            PZINC(IB+19,JROF)=PSIGN(JROF,19,JLEV) &
             & *(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z21
            PZINC(IB+20,JROF)=PSIGN(JROF,20,JLEV)*ZCLO125*Z21
            PZINC(IB+21,JROF)=PSIGN(JROF,21,JLEV)*ZCLO225*Z21
            PZINC(IB+22,JROF)=PSIGN(JROF,22,JLEV)*ZCLO325*Z21
            PZINC(IB+29,JROF)=PSIGN(JROF,29,JLEV)*(1.0_JPRB-ZDLO35)*Z31
            PZINC(IB+30,JROF)=PSIGN(JROF,30,JLEV)*ZDLO35*Z31

            ! interpolations in longitude, stencil level 0
            PZINC(IB+5,JROF)=PSIGN(JROF,5,JLEV)*(1.0_JPRB-ZDLO15)*Z10
            PZINC(IB+6,JROF)=PSIGN(JROF,6,JLEV)*ZDLO15*Z10
            PZINC(IB+17,JROF)=PSIGN(JROF,17,JLEV)*(1.0_JPRB-ZDLO25)*Z20
            PZINC(IB+18,JROF)=PSIGN(JROF,18,JLEV)*ZDLO25*Z20

            IF (.NOT. LDADV55) THEN
              ! final interpolation in vertical
              PVINTW(JROF,JLEV,1)=PVINTW(JROF,JLEV,1)+(Z15-Z05)*PXF(JROF,JLEV)
              PVINTW(JROF,JLEV,2)=PVINTW(JROF,JLEV,2)+(Z25-Z05)*PXF(JROF,JLEV)
              PVINTW(JROF,JLEV,3)=PVINTW(JROF,JLEV,3)+(Z35-Z05)*PXF(JROF,JLEV)

              ! interpolations in latitude, stencil levels 0, 1, 2, 3
              PDLAT(JROF,JLEV) =PDLAT(JROF,JLEV) +(Z205-Z105)*Z0+(Z235-Z135)*Z3
              PCLA(JROF,JLEV,1)=PCLA(JROF,JLEV,1)+(Z115-Z015)*Z1+(Z125-Z025)*Z2
              PCLA(JROF,JLEV,2)=PCLA(JROF,JLEV,2)+(Z215-Z015)*Z1+(Z225-Z025)*Z2
              PCLA(JROF,JLEV,3)=PCLA(JROF,JLEV,3)+(Z315-Z015)*Z1+(Z325-Z025)*Z2
  
              ! interpolations in longitude, stencil level 3
              PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z13 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
              PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z23 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

              ! interpolations in longitude, stencil level 2
              PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z02 &
               & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
              PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z32 &
               & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))
              PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
               & + (ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
              PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
               & + (ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
              PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z12 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
              PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
               & + (ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
              PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
               & + (ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
              PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z22 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))

              ! interpolations in longitude, stencil level 1
              PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z01 &
               & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
              PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z31 &
               & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))
              PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
               & + (ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
              PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
               & + (ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
              PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z11 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
              PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
               & + (ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
              PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
               & + (ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
              PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z21 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))

              ! interpolations in longitude, stencil level 0
              PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z10 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
              PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z20 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))
            ENDIF
 
          ENDDO
          ILEV=ILEV+1
        ENDDO
      ENDDO

    ENDIF

  ENDIF

ELSE         !(non-reproducible)   

  IF (.NOT.LDVECADIN) THEN

    DO JLEV=1,KFLEV

      DO JROF=KSTART,KPROF

       !   Interpolation (trajectory)

        ZF1115=PXSL5(KL0(JROF,JLEV,1)+IV1L1)
        ZF2115=PXSL5(KL0(JROF,JLEV,1)+IV1L2)
        ZF1215=PXSL5(KL0(JROF,JLEV,2)+IV1L1)
        ZF2215=PXSL5(KL0(JROF,JLEV,2)+IV1L2)
        ZF1125=PXSL5(KL0(JROF,JLEV,1)+IV2L1)
        ZF2125=PXSL5(KL0(JROF,JLEV,1)+IV2L2)
        ZF1225=PXSL5(KL0(JROF,JLEV,2)+IV2L1)
        ZF2225=PXSL5(KL0(JROF,JLEV,2)+IV2L2)

        ! interpolation distances in longitude
        ZDLO05=PDLO5(JROF,JLEV,0)
        ZDLO15=PDLO5(JROF,JLEV,1)
        ZDLO25=PDLO5(JROF,JLEV,2)
        ZDLO35=PDLO5(JROF,JLEV,3)

        ! interpolation weights in longitude (latitude rows 1, 2)
        ZCLO115=PCLO5(JROF,JLEV,1,1)
        ZCLO215=PCLO5(JROF,JLEV,2,1)
        ZCLO315=PCLO5(JROF,JLEV,3,1)
        ZCLO125=PCLO5(JROF,JLEV,1,2)
        ZCLO225=PCLO5(JROF,JLEV,2,2)
        ZCLO325=PCLO5(JROF,JLEV,3,2)

        ! interpolation distance in latitude
        ZDLAT5=PDLAT5(JROF,JLEV)

        ! interpolation weights in latitude
        ZCLA15=PCLA5(JROF,JLEV,1)
        ZCLA25=PCLA5(JROF,JLEV,2)
        ZCLA35=PCLA5(JROF,JLEV,3)

        ! interpolations in longitude, stencil level 0
        Z105=PXSL5(KL0(JROF,JLEV,1)+IV0L1)+ZDLO15 &
         & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
        Z205=PXSL5(KL0(JROF,JLEV,2)+IV0L1)+ZDLO25 &
         & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))

        ! interpolations in longitude, stencil level 1
        Z015=PXSL5(KL0(JROF,JLEV,0)+IV1L1)+ZDLO05 &
         & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
        Z115=PXSL5(KL0(JROF,JLEV,1)+IV1L0) &
         & +ZCLO115*(ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
         & +ZCLO215*(ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
         & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
        Z215=PXSL5(KL0(JROF,JLEV,2)+IV1L0) &
         & +ZCLO125*(ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
         & +ZCLO225*(ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
         & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))
        Z315=PXSL5(KL0(JROF,JLEV,3)+IV1L1)+ZDLO35 &
         & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))

        ! interpolations in longitude, stencil level 2
        Z025=PXSL5(KL0(JROF,JLEV,0)+IV2L1)+ZDLO05 &
         & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
        Z125=PXSL5(KL0(JROF,JLEV,1)+IV2L0) &
         & +ZCLO115*(ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
         & +ZCLO215*(ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
         & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
        Z225=PXSL5(KL0(JROF,JLEV,2)+IV2L0) &
         & +ZCLO125*(ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
         & +ZCLO225*(ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
         & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))
        Z325=PXSL5(KL0(JROF,JLEV,3)+IV2L1)+ZDLO35 &
         & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))

        ! interpolations in longitude, stencil level 3
        Z135=PXSL5(KL0(JROF,JLEV,1)+IV3L1)+ZDLO15 &
         & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
        Z235=PXSL5(KL0(JROF,JLEV,2)+IV3L1)+ZDLO25 &
         & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

        ! interpolations in latitude, stencil levels 0, 1, 2, 3
        Z05=Z105+ZDLAT5*(Z205-Z105)
        Z15=Z015+ZCLA15*(Z115-Z015)+ZCLA25*(Z215-Z015)+ZCLA35*(Z315-Z015)
        Z25=Z025+ZCLA15*(Z125-Z025)+ZCLA25*(Z225-Z025)+ZCLA35*(Z325-Z025)
        Z35=Z135+ZDLAT5*(Z235-Z135)

        !   Interpolation (adjoint)

        ! final interpolation in vertical
        Z1=PVINTW5(JROF,JLEV,1)*PXF(JROF,JLEV)
        Z2=PVINTW5(JROF,JLEV,2)*PXF(JROF,JLEV)
        Z3=PVINTW5(JROF,JLEV,3)*PXF(JROF,JLEV)
        Z0=PXF(JROF,JLEV)-Z1-Z2-Z3

        ! interpolations in latitude, stencil levels 0, 1, 2, 3
        Z10=(1._JPRB-ZDLAT5)*Z0
        Z20=ZDLAT5*Z0

        Z11=ZCLA15*Z1
        Z21=ZCLA25*Z1
        Z31=ZCLA35*Z1
        Z01=Z1-Z11-Z21-Z31

        Z12=ZCLA15*Z2
        Z22=ZCLA25*Z2
        Z32=ZCLA35*Z2
        Z02=Z2-Z12-Z22-Z32

        Z13=(1._JPRB-ZDLAT5)*Z3
        Z23=ZDLAT5*Z3

        ! interpolations in longitude, stencil level 3
        PXSL(KL0(JROF,JLEV,1)+IV3L1)=PXSL(KL0(JROF,JLEV,1)+IV3L1) &
         & +(1.0_JPRB-ZDLO15)*Z13
        PXSL(KL0(JROF,JLEV,1)+IV3L2)=PXSL(KL0(JROF,JLEV,1)+IV3L2) &
         & +ZDLO15*Z13
        PXSL(KL0(JROF,JLEV,2)+IV3L1)=PXSL(KL0(JROF,JLEV,2)+IV3L1) &
         & +(1.0_JPRB-ZDLO25)*Z23
        PXSL(KL0(JROF,JLEV,2)+IV3L2)=PXSL(KL0(JROF,JLEV,2)+IV3L2) &
         & +ZDLO25*Z23

        ! interpolations in longitude, stencil level 2
        PXSL(KL0(JROF,JLEV,0)+IV2L1)=PXSL(KL0(JROF,JLEV,0)+IV2L1) &
         & +(1.0_JPRB-ZDLO05)*Z02
        PXSL(KL0(JROF,JLEV,0)+IV2L2)=PXSL(KL0(JROF,JLEV,0)+IV2L2) &
         & +ZDLO05*Z02
        PXSL(KL0(JROF,JLEV,1)+IV2L0)=PXSL(KL0(JROF,JLEV,1)+IV2L0) &
         & +(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z12
        PXSL(KL0(JROF,JLEV,1)+IV2L1)=PXSL(KL0(JROF,JLEV,1)+IV2L1) &
         & +ZCLO115*Z12
        PXSL(KL0(JROF,JLEV,1)+IV2L2)=PXSL(KL0(JROF,JLEV,1)+IV2L2) &
         & +ZCLO215*Z12
        PXSL(KL0(JROF,JLEV,1)+IV2L3)=PXSL(KL0(JROF,JLEV,1)+IV2L3) &
         & +ZCLO315*Z12
        PXSL(KL0(JROF,JLEV,2)+IV2L0)=PXSL(KL0(JROF,JLEV,2)+IV2L0) &
         & +(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z22
        PXSL(KL0(JROF,JLEV,2)+IV2L1)=PXSL(KL0(JROF,JLEV,2)+IV2L1) &
         & +ZCLO125*Z22
        PXSL(KL0(JROF,JLEV,2)+IV2L2)=PXSL(KL0(JROF,JLEV,2)+IV2L2) &
         & +ZCLO225*Z22
        PXSL(KL0(JROF,JLEV,2)+IV2L3)=PXSL(KL0(JROF,JLEV,2)+IV2L3) &
         & +ZCLO325*Z22
        PXSL(KL0(JROF,JLEV,3)+IV2L1)=PXSL(KL0(JROF,JLEV,3)+IV2L1) &
         & +(1.0_JPRB-ZDLO35)*Z32
        PXSL(KL0(JROF,JLEV,3)+IV2L2)=PXSL(KL0(JROF,JLEV,3)+IV2L2) &
         & +ZDLO35*Z32

        ! interpolations in longitude, stencil level 1
        PXSL(KL0(JROF,JLEV,0)+IV1L1)=PXSL(KL0(JROF,JLEV,0)+IV1L1) &
         & +(1.0_JPRB-ZDLO05)*Z01
        PXSL(KL0(JROF,JLEV,0)+IV1L2)=PXSL(KL0(JROF,JLEV,0)+IV1L2) &
         & +ZDLO05*Z01
        PXSL(KL0(JROF,JLEV,1)+IV1L0)=PXSL(KL0(JROF,JLEV,1)+IV1L0) &
         & +(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z11
        PXSL(KL0(JROF,JLEV,1)+IV1L1)=PXSL(KL0(JROF,JLEV,1)+IV1L1) &
         & +ZCLO115*Z11
        PXSL(KL0(JROF,JLEV,1)+IV1L2)=PXSL(KL0(JROF,JLEV,1)+IV1L2) &
         & +ZCLO215*Z11
        PXSL(KL0(JROF,JLEV,1)+IV1L3)=PXSL(KL0(JROF,JLEV,1)+IV1L3) &
         & +ZCLO315*Z11
        PXSL(KL0(JROF,JLEV,2)+IV1L0)=PXSL(KL0(JROF,JLEV,2)+IV1L0) &
         & +(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z21
        PXSL(KL0(JROF,JLEV,2)+IV1L1)=PXSL(KL0(JROF,JLEV,2)+IV1L1) &
         & +ZCLO125*Z21
        PXSL(KL0(JROF,JLEV,2)+IV1L2)=PXSL(KL0(JROF,JLEV,2)+IV1L2) &
         & +ZCLO225*Z21
        PXSL(KL0(JROF,JLEV,2)+IV1L3)=PXSL(KL0(JROF,JLEV,2)+IV1L3) &
         & +ZCLO325*Z21
        PXSL(KL0(JROF,JLEV,3)+IV1L1)=PXSL(KL0(JROF,JLEV,3)+IV1L1) &
         & +(1.0_JPRB-ZDLO35)*Z31
        PXSL(KL0(JROF,JLEV,3)+IV1L2)=PXSL(KL0(JROF,JLEV,3)+IV1L2) &
         & +ZDLO35*Z31

        ! interpolations in longitude, stencil level 0
        PXSL(KL0(JROF,JLEV,1)+IV0L1)=PXSL(KL0(JROF,JLEV,1)+IV0L1) &
         & +(1.0_JPRB-ZDLO15)*Z10
        PXSL(KL0(JROF,JLEV,1)+IV0L2)=PXSL(KL0(JROF,JLEV,1)+IV0L2) &
         & +ZDLO15*Z10
        PXSL(KL0(JROF,JLEV,2)+IV0L1)=PXSL(KL0(JROF,JLEV,2)+IV0L1) &
         & +(1.0_JPRB-ZDLO25)*Z20
        PXSL(KL0(JROF,JLEV,2)+IV0L2)=PXSL(KL0(JROF,JLEV,2)+IV0L2) &
         & +ZDLO25*Z20

        IF (.NOT. LDADV55) THEN
          ! final interpolation in vertical
          PVINTW(JROF,JLEV,1)=PVINTW(JROF,JLEV,1)+(Z15-Z05)*PXF(JROF,JLEV)
          PVINTW(JROF,JLEV,2)=PVINTW(JROF,JLEV,2)+(Z25-Z05)*PXF(JROF,JLEV)
          PVINTW(JROF,JLEV,3)=PVINTW(JROF,JLEV,3)+(Z35-Z05)*PXF(JROF,JLEV)

          ! interpolations in latitude, stencil levels 0, 1, 2, 3
          PDLAT(JROF,JLEV) =PDLAT(JROF,JLEV) +(Z205-Z105)*Z0+(Z235-Z135)*Z3
          PCLA(JROF,JLEV,1)=PCLA(JROF,JLEV,1)+(Z115-Z015)*Z1+(Z125-Z025)*Z2
          PCLA(JROF,JLEV,2)=PCLA(JROF,JLEV,2)+(Z215-Z015)*Z1+(Z225-Z025)*Z2
          PCLA(JROF,JLEV,3)=PCLA(JROF,JLEV,3)+(Z315-Z015)*Z1+(Z325-Z025)*Z2
  
          ! interpolations in longitude, stencil level 3
          PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z13 &
           & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
          PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z23 &
           & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

          ! interpolations in longitude, stencil level 2
          PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z02 &
           & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
          PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z32 &
           & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))
          PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
           & + (ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
          PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
           & + (ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
          PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z12 &
           & *(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
          PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
           & + (ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
          PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
           & + (ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
          PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z22 &
           & *(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))

          ! interpolations in longitude, stencil level 1
          PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z01 &
           & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
          PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z31 &
           & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))
          PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
           & + (ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
          PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
           & + (ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
          PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z11 &
           & *(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
          PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
           & + (ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
          PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
           & + (ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
          PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z21 &
           & *(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))

          ! interpolations in longitude, stencil level 0
          PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z10 &
           & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
          PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z20 &
           & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))
        ENDIF
 
      ENDDO
    ENDDO

  ELSE

    IF (LDPLANE) THEN

      DO JJLEV=1,KSEP 
        DO JLEV=JJLEV,KFLEV,KSEP

          IB=(ILEV-1)*KDIM

!CDIR NODEP
          DO JROF=KSTART,KPROF

           !   Interpolation (trajectory)

            ZF1115=PXSL5(KL0(JROF,JLEV,1)+IV1L1)
            ZF2115=PXSL5(KL0(JROF,JLEV,1)+IV1L2)
            ZF1215=PXSL5(KL0(JROF,JLEV,2)+IV1L1)
            ZF2215=PXSL5(KL0(JROF,JLEV,2)+IV1L2)
            ZF1125=PXSL5(KL0(JROF,JLEV,1)+IV2L1)
            ZF2125=PXSL5(KL0(JROF,JLEV,1)+IV2L2)
            ZF1225=PXSL5(KL0(JROF,JLEV,2)+IV2L1)
            ZF2225=PXSL5(KL0(JROF,JLEV,2)+IV2L2)

            ! interpolation distances in longitude
            ZDLO05=PDLO5(JROF,JLEV,0)
            ZDLO15=PDLO5(JROF,JLEV,1)
            ZDLO25=PDLO5(JROF,JLEV,2)
            ZDLO35=PDLO5(JROF,JLEV,3)

            ! interpolation weights in longitude (latitude rows 1, 2)
            ZCLO115=PCLO5(JROF,JLEV,1,1)
            ZCLO215=PCLO5(JROF,JLEV,2,1)
            ZCLO315=PCLO5(JROF,JLEV,3,1)
            ZCLO125=PCLO5(JROF,JLEV,1,2)
            ZCLO225=PCLO5(JROF,JLEV,2,2)
            ZCLO325=PCLO5(JROF,JLEV,3,2)

            ! interpolation distance in latitude
            ZDLAT5=PDLAT5(JROF,JLEV)

            ! interpolation weights in latitude
            ZCLA15=PCLA5(JROF,JLEV,1)
            ZCLA25=PCLA5(JROF,JLEV,2)
            ZCLA35=PCLA5(JROF,JLEV,3)

            ! interpolations in longitude, stencil level 0
            Z105=PXSL5(KL0(JROF,JLEV,1)+IV0L1)+ZDLO15 &
             & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
            Z205=PXSL5(KL0(JROF,JLEV,2)+IV0L1)+ZDLO25 &
             & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))

            ! interpolations in longitude, stencil level 1
            Z015=PXSL5(KL0(JROF,JLEV,0)+IV1L1)+ZDLO05 &
             & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
            Z115=PXSL5(KL0(JROF,JLEV,1)+IV1L0) &
             & +ZCLO115*(ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
             & +ZCLO215*(ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
             & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
            Z215=PXSL5(KL0(JROF,JLEV,2)+IV1L0) &
             & +ZCLO125*(ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
             & +ZCLO225*(ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
             & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))
            Z315=PXSL5(KL0(JROF,JLEV,3)+IV1L1)+ZDLO35 &
             & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))

            ! interpolations in longitude, stencil level 2
            Z025=PXSL5(KL0(JROF,JLEV,0)+IV2L1)+ZDLO05 &
             & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
            Z125=PXSL5(KL0(JROF,JLEV,1)+IV2L0) &
             & +ZCLO115*(ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
             & +ZCLO215*(ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
             & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
            Z225=PXSL5(KL0(JROF,JLEV,2)+IV2L0) &
             & +ZCLO125*(ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
             & +ZCLO225*(ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
             & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))
            Z325=PXSL5(KL0(JROF,JLEV,3)+IV2L1)+ZDLO35 &
             & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))

            ! interpolations in longitude, stencil level 3
            Z135=PXSL5(KL0(JROF,JLEV,1)+IV3L1)+ZDLO15 &
             & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
            Z235=PXSL5(KL0(JROF,JLEV,2)+IV3L1)+ZDLO25 &
             & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

            ! interpolations in latitude, stencil levels 0, 1, 2, 3
            Z05=Z105+ZDLAT5*(Z205-Z105)
            Z15=Z015+ZCLA15*(Z115-Z015)+ZCLA25*(Z215-Z015)+ZCLA35*(Z315-Z015)
            Z25=Z025+ZCLA15*(Z125-Z025)+ZCLA25*(Z225-Z025)+ZCLA35*(Z325-Z025)
            Z35=Z135+ZDLAT5*(Z235-Z135)

            !   Interpolation (adjoint)

            KINC(IB+1,JROF)=IBASE+KL0(JROF,JLEV,0)+IV1L1
            KINC(IB+2,JROF)=IBASE+KL0(JROF,JLEV,0)+IV1L2
            KINC(IB+3,JROF)=IBASE+KL0(JROF,JLEV,0)+IV2L1
            KINC(IB+4,JROF)=IBASE+KL0(JROF,JLEV,0)+IV2L2
            KINC(IB+5,JROF)=IBASE+KL0(JROF,JLEV,1)+IV0L1
            KINC(IB+6,JROF)=IBASE+KL0(JROF,JLEV,1)+IV0L2
            KINC(IB+7,JROF)=IBASE+KL0(JROF,JLEV,1)+IV1L0
            KINC(IB+8,JROF)=IBASE+KL0(JROF,JLEV,1)+IV1L1
            KINC(IB+9,JROF)=IBASE+KL0(JROF,JLEV,1)+IV1L2
            KINC(IB+10,JROF)=IBASE+KL0(JROF,JLEV,1)+IV1L3
            KINC(IB+11,JROF)=IBASE+KL0(JROF,JLEV,1)+IV2L0
            KINC(IB+12,JROF)=IBASE+KL0(JROF,JLEV,1)+IV2L1
            KINC(IB+13,JROF)=IBASE+KL0(JROF,JLEV,1)+IV2L2
            KINC(IB+14,JROF)=IBASE+KL0(JROF,JLEV,1)+IV2L3
            KINC(IB+15,JROF)=IBASE+KL0(JROF,JLEV,1)+IV3L1
            KINC(IB+16,JROF)=IBASE+KL0(JROF,JLEV,1)+IV3L2
            KINC(IB+17,JROF)=IBASE+KL0(JROF,JLEV,2)+IV0L1
            KINC(IB+18,JROF)=IBASE+KL0(JROF,JLEV,2)+IV0L2
            KINC(IB+19,JROF)=IBASE+KL0(JROF,JLEV,2)+IV1L0
            KINC(IB+20,JROF)=IBASE+KL0(JROF,JLEV,2)+IV1L1
            KINC(IB+21,JROF)=IBASE+KL0(JROF,JLEV,2)+IV1L2
            KINC(IB+22,JROF)=IBASE+KL0(JROF,JLEV,2)+IV1L3
            KINC(IB+23,JROF)=IBASE+KL0(JROF,JLEV,2)+IV2L0
            KINC(IB+24,JROF)=IBASE+KL0(JROF,JLEV,2)+IV2L1
            KINC(IB+25,JROF)=IBASE+KL0(JROF,JLEV,2)+IV2L2
            KINC(IB+26,JROF)=IBASE+KL0(JROF,JLEV,2)+IV2L3
            KINC(IB+27,JROF)=IBASE+KL0(JROF,JLEV,2)+IV3L1
            KINC(IB+28,JROF)=IBASE+KL0(JROF,JLEV,2)+IV3L2
            KINC(IB+29,JROF)=IBASE+KL0(JROF,JLEV,3)+IV1L1
            KINC(IB+30,JROF)=IBASE+KL0(JROF,JLEV,3)+IV1L2
            KINC(IB+31,JROF)=IBASE+KL0(JROF,JLEV,3)+IV2L1
            KINC(IB+32,JROF)=IBASE+KL0(JROF,JLEV,3)+IV2L2

            ! final interpolation in vertical
            Z1=PVINTW5(JROF,JLEV,1)*PXF(JROF,JLEV)
            Z2=PVINTW5(JROF,JLEV,2)*PXF(JROF,JLEV)
            Z3=PVINTW5(JROF,JLEV,3)*PXF(JROF,JLEV)
            Z0=PXF(JROF,JLEV)-Z1-Z2-Z3

            ! interpolations in latitude, stencil levels 0, 1, 2, 3
            Z10=(1._JPRB-ZDLAT5)*Z0
            Z20=ZDLAT5*Z0

            Z11=ZCLA15*Z1
            Z21=ZCLA25*Z1
            Z31=ZCLA35*Z1
            Z01=Z1-Z11-Z21-Z31

            Z12=ZCLA15*Z2
            Z22=ZCLA25*Z2
            Z32=ZCLA35*Z2
            Z02=Z2-Z12-Z22-Z32

            Z13=(1._JPRB-ZDLAT5)*Z3
            Z23=ZDLAT5*Z3

            ! interpolations in longitude, stencil level 3
            PZINC(IB+15,JROF)=(1.0_JPRB-ZDLO15)*Z13
            PZINC(IB+16,JROF)=ZDLO15*Z13
            PZINC(IB+27,JROF)=(1.0_JPRB-ZDLO25)*Z23
            PZINC(IB+28,JROF)=ZDLO25*Z23

            ! interpolations in longitude, stencil level 2
            PZINC(IB+3,JROF)=(1.0_JPRB-ZDLO05)*Z02
            PZINC(IB+4,JROF)=ZDLO05*Z02
            PZINC(IB+11,JROF)=(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z12
            PZINC(IB+12,JROF)=ZCLO115*Z12
            PZINC(IB+13,JROF)=ZCLO215*Z12
            PZINC(IB+14,JROF)=ZCLO315*Z12
            PZINC(IB+23,JROF)=(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z22
            PZINC(IB+24,JROF)=ZCLO125*Z22
            PZINC(IB+25,JROF)=ZCLO225*Z22
            PZINC(IB+26,JROF)=ZCLO325*Z22
            PZINC(IB+31,JROF)=(1.0_JPRB-ZDLO35)*Z32
            PZINC(IB+32,JROF)=ZDLO35*Z32

            ! interpolations in longitude, stencil level 1
            PZINC(IB+1,JROF)=(1.0_JPRB-ZDLO05)*Z01
            PZINC(IB+2,JROF)=ZDLO05*Z01
            PZINC(IB+7,JROF)=(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z11
            PZINC(IB+8,JROF)=ZCLO115*Z11
            PZINC(IB+9,JROF)=ZCLO215*Z11
            PZINC(IB+10,JROF)=ZCLO315*Z11
            PZINC(IB+19,JROF)=(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z21
            PZINC(IB+20,JROF)=ZCLO125*Z21
            PZINC(IB+21,JROF)=ZCLO225*Z21
            PZINC(IB+22,JROF)=ZCLO325*Z21
            PZINC(IB+29,JROF)=(1.0_JPRB-ZDLO35)*Z31
            PZINC(IB+30,JROF)=ZDLO35*Z31

            ! interpolations in longitude, stencil level 0
            PZINC(IB+5,JROF)=(1.0_JPRB-ZDLO15)*Z10
            PZINC(IB+6,JROF)=ZDLO15*Z10
            PZINC(IB+17,JROF)=(1.0_JPRB-ZDLO25)*Z20
            PZINC(IB+18,JROF)=ZDLO25*Z20

            IF (.NOT. LDADV55) THEN
              ! final interpolation in vertical
              PVINTW(JROF,JLEV,1)=PVINTW(JROF,JLEV,1)+(Z15-Z05)*PXF(JROF,JLEV)
              PVINTW(JROF,JLEV,2)=PVINTW(JROF,JLEV,2)+(Z25-Z05)*PXF(JROF,JLEV)
              PVINTW(JROF,JLEV,3)=PVINTW(JROF,JLEV,3)+(Z35-Z05)*PXF(JROF,JLEV)

              ! interpolations in latitude, stencil levels 0, 1, 2, 3
              PDLAT(JROF,JLEV) =PDLAT(JROF,JLEV) +(Z205-Z105)*Z0+(Z235-Z135)*Z3
              PCLA(JROF,JLEV,1)=PCLA(JROF,JLEV,1)+(Z115-Z015)*Z1+(Z125-Z025)*Z2
              PCLA(JROF,JLEV,2)=PCLA(JROF,JLEV,2)+(Z215-Z015)*Z1+(Z225-Z025)*Z2
              PCLA(JROF,JLEV,3)=PCLA(JROF,JLEV,3)+(Z315-Z015)*Z1+(Z325-Z025)*Z2

              ! interpolations in longitude, stencil level 3
              PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z13 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
              PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z23 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

              ! interpolations in longitude, stencil level 2
              PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z02 &
               & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
              PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z32 &
               & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))
              PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
               & + (ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
              PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
               & + (ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
              PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z12 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
              PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
               & + (ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
              PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
               & + (ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
              PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z22 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))

              ! interpolations in longitude, stencil level 1
              PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z01 &
               & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
              PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z31 &
               & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))
              PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
               & + (ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
              PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
               & + (ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
              PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z11 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
              PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
               & + (ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
              PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
               & + (ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
              PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z21 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))

              ! interpolations in longitude, stencil level 0
              PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z10 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
              PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z20 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))
            ENDIF
 
            ! correction of dependency for mirrored points
            IF (KDEP(JROF,JLEV) == 1) THEN
              KINC(IB+1,JROF)=KINC(IB+29,JROF)-1
              KINC(IB+2,JROF)=KINC(IB+30,JROF)+1
              KINC(IB+3,JROF)=KINC(IB+31,JROF)-1
              KINC(IB+4,JROF)=KINC(IB+32,JROF)+1
              !KINC(IB+1:IB+4,JROF)=0  ! a bit tricky alternative of previous
              PZINC(IB+8,JROF)=PZINC(IB+8,JROF)+PZINC(IB+1,JROF)
              PZINC(IB+9,JROF)=PZINC(IB+9,JROF)+PZINC(IB+2,JROF)
              PZINC(IB+12,JROF)=PZINC(IB+12,JROF)+PZINC(IB+3,JROF)
              PZINC(IB+13,JROF)=PZINC(IB+13,JROF)+PZINC(IB+4,JROF)
              PZINC(IB+1,JROF)=0.0_JPRB
              PZINC(IB+2,JROF)=0.0_JPRB
              PZINC(IB+3,JROF)=0.0_JPRB
              PZINC(IB+4,JROF)=0.0_JPRB
            ELSEIF (KDEP(JROF,JLEV) == 4) THEN
              KINC(IB+29,JROF)=KINC(IB+1,JROF)-1
              KINC(IB+30,JROF)=KINC(IB+2,JROF)+1
              KINC(IB+31,JROF)=KINC(IB+3,JROF)-1
              KINC(IB+32,JROF)=KINC(IB+4,JROF)+1
              !KINC(IB+29:IB+32,JROF)=0 
              PZINC(IB+20,JROF)=PZINC(IB+20,JROF)+PZINC(IB+29,JROF)
              PZINC(IB+21,JROF)=PZINC(IB+21,JROF)+PZINC(IB+30,JROF)
              PZINC(IB+24,JROF)=PZINC(IB+24,JROF)+PZINC(IB+31,JROF)
              PZINC(IB+25,JROF)=PZINC(IB+25,JROF)+PZINC(IB+32,JROF)
              PZINC(IB+29,JROF)=0.0_JPRB
              PZINC(IB+30,JROF)=0.0_JPRB
              PZINC(IB+31,JROF)=0.0_JPRB
              PZINC(IB+32,JROF)=0.0_JPRB
            ENDIF

          ENDDO
          ILEV=ILEV+1
        ENDDO
      ENDDO

    ELSE ! sphere (no dependency to correct)

      DO JJLEV=1,KSEP 
        DO JLEV=JJLEV,KFLEV,KSEP

          IB=(ILEV-1)*KDIM

!CDIR NODEP
          DO JROF=KSTART,KPROF

           !   Interpolation (trajectory)

            ZF1115=PXSL5(KL0(JROF,JLEV,1)+IV1L1)
            ZF2115=PXSL5(KL0(JROF,JLEV,1)+IV1L2)
            ZF1215=PXSL5(KL0(JROF,JLEV,2)+IV1L1)
            ZF2215=PXSL5(KL0(JROF,JLEV,2)+IV1L2)
            ZF1125=PXSL5(KL0(JROF,JLEV,1)+IV2L1)
            ZF2125=PXSL5(KL0(JROF,JLEV,1)+IV2L2)
            ZF1225=PXSL5(KL0(JROF,JLEV,2)+IV2L1)
            ZF2225=PXSL5(KL0(JROF,JLEV,2)+IV2L2)

            ! interpolation distances in longitude
            ZDLO05=PDLO5(JROF,JLEV,0)
            ZDLO15=PDLO5(JROF,JLEV,1)
            ZDLO25=PDLO5(JROF,JLEV,2)
            ZDLO35=PDLO5(JROF,JLEV,3)

            ! interpolation weights in longitude (latitude rows 1, 2)
            ZCLO115=PCLO5(JROF,JLEV,1,1)
            ZCLO215=PCLO5(JROF,JLEV,2,1)
            ZCLO315=PCLO5(JROF,JLEV,3,1)
            ZCLO125=PCLO5(JROF,JLEV,1,2)
            ZCLO225=PCLO5(JROF,JLEV,2,2)
            ZCLO325=PCLO5(JROF,JLEV,3,2)

            ! interpolation distance in latitude
            ZDLAT5=PDLAT5(JROF,JLEV)

            ! interpolation weights in latitude
            ZCLA15=PCLA5(JROF,JLEV,1)
            ZCLA25=PCLA5(JROF,JLEV,2)
            ZCLA35=PCLA5(JROF,JLEV,3)

            ! interpolations in longitude, stencil level 0
            Z105=PXSL5(KL0(JROF,JLEV,1)+IV0L1)+ZDLO15 &
             & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
            Z205=PXSL5(KL0(JROF,JLEV,2)+IV0L1)+ZDLO25 &
             & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))

            ! interpolations in longitude, stencil level 1
            Z015=PXSL5(KL0(JROF,JLEV,0)+IV1L1)+ZDLO05 &
             & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
            Z115=PXSL5(KL0(JROF,JLEV,1)+IV1L0) &
             & +ZCLO115*(ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
             & +ZCLO215*(ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0)) &
             & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
            Z215=PXSL5(KL0(JROF,JLEV,2)+IV1L0) &
             & +ZCLO125*(ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
             & +ZCLO225*(ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0)) &
             & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))
            Z315=PXSL5(KL0(JROF,JLEV,3)+IV1L1)+ZDLO35 &
             & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))

            ! interpolations in longitude, stencil level 2
            Z025=PXSL5(KL0(JROF,JLEV,0)+IV2L1)+ZDLO05 &
             & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
            Z125=PXSL5(KL0(JROF,JLEV,1)+IV2L0) &
             & +ZCLO115*(ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
             & +ZCLO215*(ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0)) &
             & +ZCLO315*(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
            Z225=PXSL5(KL0(JROF,JLEV,2)+IV2L0) &
             & +ZCLO125*(ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
             & +ZCLO225*(ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0)) &
             & +ZCLO325*(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))
            Z325=PXSL5(KL0(JROF,JLEV,3)+IV2L1)+ZDLO35 &
             & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))

            ! interpolations in longitude, stencil level 3
            Z135=PXSL5(KL0(JROF,JLEV,1)+IV3L1)+ZDLO15 &
             & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
            Z235=PXSL5(KL0(JROF,JLEV,2)+IV3L1)+ZDLO25 &
             & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

            ! interpolations in latitude, stencil levels 0, 1, 2, 3
            Z05=Z105+ZDLAT5*(Z205-Z105)
            Z15=Z015+ZCLA15*(Z115-Z015)+ZCLA25*(Z215-Z015)+ZCLA35*(Z315-Z015)
            Z25=Z025+ZCLA15*(Z125-Z025)+ZCLA25*(Z225-Z025)+ZCLA35*(Z325-Z025)
            Z35=Z135+ZDLAT5*(Z235-Z135)

            !   Interpolation (adjoint)

            KINC(IB+1,JROF)=IBASE+KL0(JROF,JLEV,0)+IV1L1
            KINC(IB+2,JROF)=IBASE+KL0(JROF,JLEV,0)+IV1L2
            KINC(IB+3,JROF)=IBASE+KL0(JROF,JLEV,0)+IV2L1
            KINC(IB+4,JROF)=IBASE+KL0(JROF,JLEV,0)+IV2L2
            KINC(IB+5,JROF)=IBASE+KL0(JROF,JLEV,1)+IV0L1
            KINC(IB+6,JROF)=IBASE+KL0(JROF,JLEV,1)+IV0L2
            KINC(IB+7,JROF)=IBASE+KL0(JROF,JLEV,1)+IV1L0
            KINC(IB+8,JROF)=IBASE+KL0(JROF,JLEV,1)+IV1L1
            KINC(IB+9,JROF)=IBASE+KL0(JROF,JLEV,1)+IV1L2
            KINC(IB+10,JROF)=IBASE+KL0(JROF,JLEV,1)+IV1L3
            KINC(IB+11,JROF)=IBASE+KL0(JROF,JLEV,1)+IV2L0
            KINC(IB+12,JROF)=IBASE+KL0(JROF,JLEV,1)+IV2L1
            KINC(IB+13,JROF)=IBASE+KL0(JROF,JLEV,1)+IV2L2
            KINC(IB+14,JROF)=IBASE+KL0(JROF,JLEV,1)+IV2L3
            KINC(IB+15,JROF)=IBASE+KL0(JROF,JLEV,1)+IV3L1
            KINC(IB+16,JROF)=IBASE+KL0(JROF,JLEV,1)+IV3L2
            KINC(IB+17,JROF)=IBASE+KL0(JROF,JLEV,2)+IV0L1
            KINC(IB+18,JROF)=IBASE+KL0(JROF,JLEV,2)+IV0L2
            KINC(IB+19,JROF)=IBASE+KL0(JROF,JLEV,2)+IV1L0
            KINC(IB+20,JROF)=IBASE+KL0(JROF,JLEV,2)+IV1L1
            KINC(IB+21,JROF)=IBASE+KL0(JROF,JLEV,2)+IV1L2
            KINC(IB+22,JROF)=IBASE+KL0(JROF,JLEV,2)+IV1L3
            KINC(IB+23,JROF)=IBASE+KL0(JROF,JLEV,2)+IV2L0
            KINC(IB+24,JROF)=IBASE+KL0(JROF,JLEV,2)+IV2L1
            KINC(IB+25,JROF)=IBASE+KL0(JROF,JLEV,2)+IV2L2
            KINC(IB+26,JROF)=IBASE+KL0(JROF,JLEV,2)+IV2L3
            KINC(IB+27,JROF)=IBASE+KL0(JROF,JLEV,2)+IV3L1
            KINC(IB+28,JROF)=IBASE+KL0(JROF,JLEV,2)+IV3L2
            KINC(IB+29,JROF)=IBASE+KL0(JROF,JLEV,3)+IV1L1
            KINC(IB+30,JROF)=IBASE+KL0(JROF,JLEV,3)+IV1L2
            KINC(IB+31,JROF)=IBASE+KL0(JROF,JLEV,3)+IV2L1
            KINC(IB+32,JROF)=IBASE+KL0(JROF,JLEV,3)+IV2L2

            ! final interpolation in vertical
            Z1=PVINTW5(JROF,JLEV,1)*PXF(JROF,JLEV)
            Z2=PVINTW5(JROF,JLEV,2)*PXF(JROF,JLEV)
            Z3=PVINTW5(JROF,JLEV,3)*PXF(JROF,JLEV)
            Z0=PXF(JROF,JLEV)-Z1-Z2-Z3

            ! interpolations in latitude, stencil levels 0, 1, 2, 3
            Z10=(1._JPRB-ZDLAT5)*Z0
            Z20=ZDLAT5*Z0

            Z11=ZCLA15*Z1
            Z21=ZCLA25*Z1
            Z31=ZCLA35*Z1
            Z01=Z1-Z11-Z21-Z31

            Z12=ZCLA15*Z2
            Z22=ZCLA25*Z2
            Z32=ZCLA35*Z2
            Z02=Z2-Z12-Z22-Z32

            Z13=(1._JPRB-ZDLAT5)*Z3
            Z23=ZDLAT5*Z3

            ! interpolations in longitude, stencil level 3
            PZINC(IB+15,JROF)=(1.0_JPRB-ZDLO15)*Z13
            PZINC(IB+16,JROF)=ZDLO15*Z13
            PZINC(IB+27,JROF)=(1.0_JPRB-ZDLO25)*Z23
            PZINC(IB+28,JROF)=ZDLO25*Z23

            ! interpolations in longitude, stencil level 2
            PZINC(IB+3,JROF)=(1.0_JPRB-ZDLO05)*Z02
            PZINC(IB+4,JROF)=ZDLO05*Z02
            PZINC(IB+11,JROF)=(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z12
            PZINC(IB+12,JROF)=ZCLO115*Z12
            PZINC(IB+13,JROF)=ZCLO215*Z12
            PZINC(IB+14,JROF)=ZCLO315*Z12
            PZINC(IB+23,JROF)=(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z22
            PZINC(IB+24,JROF)=ZCLO125*Z22
            PZINC(IB+25,JROF)=ZCLO225*Z22
            PZINC(IB+26,JROF)=ZCLO325*Z22
            PZINC(IB+31,JROF)=(1.0_JPRB-ZDLO35)*Z32
            PZINC(IB+32,JROF)=ZDLO35*Z32

            ! interpolations in longitude, stencil level 1
            PZINC(IB+1,JROF)=(1.0_JPRB-ZDLO05)*Z01
            PZINC(IB+2,JROF)=ZDLO05*Z01
            PZINC(IB+7,JROF)=(1.0_JPRB-ZCLO115-ZCLO215-ZCLO315)*Z11
            PZINC(IB+8,JROF)=ZCLO115*Z11
            PZINC(IB+9,JROF)=ZCLO215*Z11
            PZINC(IB+10,JROF)=ZCLO315*Z11
            PZINC(IB+19,JROF)=(1.0_JPRB-ZCLO125-ZCLO225-ZCLO325)*Z21
            PZINC(IB+20,JROF)=ZCLO125*Z21
            PZINC(IB+21,JROF)=ZCLO225*Z21
            PZINC(IB+22,JROF)=ZCLO325*Z21
            PZINC(IB+29,JROF)=(1.0_JPRB-ZDLO35)*Z31
            PZINC(IB+30,JROF)=ZDLO35*Z31

            ! interpolations in longitude, stencil level 0
            PZINC(IB+5,JROF)=(1.0_JPRB-ZDLO15)*Z10
            PZINC(IB+6,JROF)=ZDLO15*Z10
            PZINC(IB+17,JROF)=(1.0_JPRB-ZDLO25)*Z20
            PZINC(IB+18,JROF)=ZDLO25*Z20

            IF (.NOT. LDADV55) THEN
              ! final interpolation in vertical
              PVINTW(JROF,JLEV,1)=PVINTW(JROF,JLEV,1)+(Z15-Z05)*PXF(JROF,JLEV)
              PVINTW(JROF,JLEV,2)=PVINTW(JROF,JLEV,2)+(Z25-Z05)*PXF(JROF,JLEV)
              PVINTW(JROF,JLEV,3)=PVINTW(JROF,JLEV,3)+(Z35-Z05)*PXF(JROF,JLEV)

              ! interpolations in latitude, stencil levels 0, 1, 2, 3
              PDLAT(JROF,JLEV) =PDLAT(JROF,JLEV) +(Z205-Z105)*Z0+(Z235-Z135)*Z3
              PCLA(JROF,JLEV,1)=PCLA(JROF,JLEV,1)+(Z115-Z015)*Z1+(Z125-Z025)*Z2
              PCLA(JROF,JLEV,2)=PCLA(JROF,JLEV,2)+(Z215-Z015)*Z1+(Z225-Z025)*Z2
              PCLA(JROF,JLEV,3)=PCLA(JROF,JLEV,3)+(Z315-Z015)*Z1+(Z325-Z025)*Z2

              ! interpolations in longitude, stencil level 3
              PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z13 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV3L2)-PXSL5(KL0(JROF,JLEV,1)+IV3L1))
              PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z23 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV3L2)-PXSL5(KL0(JROF,JLEV,2)+IV3L1))

              ! interpolations in longitude, stencil level 2
              PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z02 &
               & *(PXSL5(KL0(JROF,JLEV,0)+IV2L2)-PXSL5(KL0(JROF,JLEV,0)+IV2L1))
              PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z32 &
               & *(PXSL5(KL0(JROF,JLEV,3)+IV2L2)-PXSL5(KL0(JROF,JLEV,3)+IV2L1))
              PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
               & + (ZF1125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
              PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
               & + (ZF2125-PXSL5(KL0(JROF,JLEV,1)+IV2L0))*Z12
              PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z12 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV2L3)-PXSL5(KL0(JROF,JLEV,1)+IV2L0))
              PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
               & + (ZF1225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
              PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
               & + (ZF2225-PXSL5(KL0(JROF,JLEV,2)+IV2L0))*Z22
              PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z22 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV2L3)-PXSL5(KL0(JROF,JLEV,2)+IV2L0))

              ! interpolations in longitude, stencil level 1
              PDLO(JROF,JLEV,0)=PDLO(JROF,JLEV,0) + Z01 &
               & *(PXSL5(KL0(JROF,JLEV,0)+IV1L2)-PXSL5(KL0(JROF,JLEV,0)+IV1L1))
              PDLO(JROF,JLEV,3)=PDLO(JROF,JLEV,3) + Z31 &
               & *(PXSL5(KL0(JROF,JLEV,3)+IV1L2)-PXSL5(KL0(JROF,JLEV,3)+IV1L1))
              PCLO(JROF,JLEV,1,1)=PCLO(JROF,JLEV,1,1) &
               & + (ZF1115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
              PCLO(JROF,JLEV,2,1)=PCLO(JROF,JLEV,2,1) &
               & + (ZF2115-PXSL5(KL0(JROF,JLEV,1)+IV1L0))*Z11
              PCLO(JROF,JLEV,3,1)=PCLO(JROF,JLEV,3,1) + Z11 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV1L3)-PXSL5(KL0(JROF,JLEV,1)+IV1L0))
              PCLO(JROF,JLEV,1,2)=PCLO(JROF,JLEV,1,2) &
               & + (ZF1215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
              PCLO(JROF,JLEV,2,2)=PCLO(JROF,JLEV,2,2) &
               & + (ZF2215-PXSL5(KL0(JROF,JLEV,2)+IV1L0))*Z21
              PCLO(JROF,JLEV,3,2)=PCLO(JROF,JLEV,3,2) + Z21 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV1L3)-PXSL5(KL0(JROF,JLEV,2)+IV1L0))

              ! interpolations in longitude, stencil level 0
              PDLO(JROF,JLEV,1)=PDLO(JROF,JLEV,1) + Z10 &
               & *(PXSL5(KL0(JROF,JLEV,1)+IV0L2)-PXSL5(KL0(JROF,JLEV,1)+IV0L1))
              PDLO(JROF,JLEV,2)=PDLO(JROF,JLEV,2) + Z20 &
               & *(PXSL5(KL0(JROF,JLEV,2)+IV0L2)-PXSL5(KL0(JROF,JLEV,2)+IV0L1))
            ENDIF
 
          ENDDO
          ILEV=ILEV+1
        ENDDO
      ENDDO

    ENDIF

  ENDIF

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAITRITLAD',1,ZHOOK_HANDLE)
END SUBROUTINE LAITRITLAD
