SUBROUTINE LAPINEA5(KST,KPROF,LD_AD,&
 & PCOLON,PSILON,PGM,PGEMU,PGSQM2,PGECLO,PGESLO,PGNORDL,PGNORDM,PINDX,PINDY,&
 & PB15,PB2,&
 & PLON5,PLAT5,PR5,PS5,&
 & PUF5,PVF5,&
 & KL0,KLH0,PDLO5,PCLO5,PCLOSLD5,&
 & PDLAT5,PCLA5,PCLASLD5,PDVER5,PVINTW5,PVINTWSLD5,&
 & PCOSCO5,PSINCO5,PCOPHI5,PSINLA5,&
 & PLEV5,POUT5,KDEP)

!**** *LAPINEA5* - semi-LAgrangian scheme:(Trajectory)
!                Interface subroutine for interpolations and lagrangian
!                trends. (Programme INterface d'Ensemble).

!     Purpose.
!     --------
!           Computes the semi-Lagrangian trajectory.
!           Computes the weights ready for interpolations at origin point.
!           Trajectory version of LAPINEA.

!**   Interface.
!     ----------
!        *CALL* *LAPINEA5(.....)

!        Explicit arguments :
!        --------------------
!        INPUT:
!          KST     - first element of arrays where computations are performed.
!          KPROF   - depth of work.
!          LD_AD   - T: LAPINEA5 is called in the adjoint model.
!                    F: LAPINEA5 is called in the linear tangent model.
!          PCOLON  - cos(computational sphere longitude).
!          PSILON  - sin(computational sphere longitude).
!          PGM     - mapping factor.
!          PGEMU   - sin(geographical latitude).
!          PGSQM2  - cos(geographical latitude).
!          PGECLO  - cos(geographical longitude).
!          PGESLO  - sin(geographical longitude).
!          PGNORDL - zonal component of the vector pointed towards the
!                    true North pole.
!          PGNORDM - meridian component of the vector pointed towards the
!                    true North pole.
!          PINDX   - x coordinate on the plane.
!          PINDY   - y coordinate on the plane.
!          PB15    - SLBUF1-buf traj for interpolations
!          PB2     - SLBUF2-buf to communicate info from non lag. to lag. dyn.

!        OUTPUT:
!          PLON5   - computational sphere longitude of interpolation point.
!          PLAT5   - computational sphere latitude of interpolation point.
!          PR5     - first element of the wind displacement matrix for M.
!          PS5     - second element of the wind displacement matrix for M.
!          PUF5    - U-comp of "(a/rs)*wind" at the medium point necessary to
!                    find the positions of the medium and origin points,
!                    in a local repere linked to computational sphere.
!          PVF5    - V-comp of "(a/rs)*wind" at the medium point necessary to
!                    find the positions of the medium and origin points,
!                    in a local repere linked to computational sphere.
!          KL0     - index of the four western points
!                    of the 16 points interpolation grid.
!          KLH0    - second value of index of the four western points
!                    of the 16 points interpolation grid if needed.
!          PDLO5   - distances for horizontal linear interpolations
!                    in longitude (latitude rows 0, 1, 2, 3)
!          PCLO5   - weights for horizontal cubic interpolations in
!                    longitude (latitude rows 1, 2)
!          PCLOSLD5  - idem for SLHD
!          PDLAT5  - distance for horizontal linear interpolation in latitude
!          PCLA5   - weights for horizontal cubic interpolation
!                    in latitude
!          PCLASLD5  - idem for SLHD
!          PDVER5  - distance for vertical linear interpolation
!          PVINTW5 - weights for vertical cubic interpolation
!          PVINTWSLD5 - idem for SLHD
!          PCOSCO5 - cos(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude).
!                    ! if LDPLANE: x - coordinate (fractional system).
!          PSINCO5 - sin(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude).
!                    ! if LDPLANE: y - coordinate (fractional system).
!          PCOPHI5 - cosine of the geographical angle between the
!                    interpolation point and the grid-point.
!          PSINLA5 - sine of the interpolation point geographical latitude.
!          PLEV5   - vertical coordinate of the interpolation point.
!          POUT5   - equal to 0 if O lays inside the computational area,
!                    otherwise contains sum of increments for each
!                    direction for which it is out of domain.
!          KDEP    - indication of the interpolation stencil latitudial
!                    dependences (used for LVECADIN=.T. option in adjoint)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------
!       See includes below.
!       Called by CALL_SL_AD and CALL_SL_TL.

!     Reference.
!     ----------
!        ARPEGE documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        C. Temperton (ECMWF)

!     Modifications.
!     --------------
!        Original : 99/10/15
!        Modified 00-02-07 by C. Temperton: removed redundant use of PDP0
!                                           when NVLAG=3 (contained zero!)
!        Modified 00-02-09 by C. Temperton: added PCL0 and PCL05 in call to
!                        LAINORAD (for improved vectorization in LARCINAD)
!                                           - also added NVSEP.
!        Modified 00-06-15 by C. Temperton: NVSEP -> NVSEPC,NVSEPL. 
!        Modified August 2000 by Deborah Salmond to split into _traj and _int
!        Modified 00-11-10 by C. Temperton: streamlining.
!        Modified 00-11-23 by C. Temperton: renamed _TRAJ and _INT routines.
!        Modified 01-06-26 by K. YESSAD: update dummy arguments of LAIDEP.
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified 05-01-11 by C. Temperton: streamlining.
!        D.Salmond     08-Feb-2005 SLADJTR replaced by LARMES5
!        F. Vana       Nov-2004  one more dummy to LAIDEP
!        K. Yessad     07-Sep-2005 Now calls directly LARCINA.
!        F. Vana       01-Jun-2006 LRPLANE=.T.
!        F. Vana       08-Jan-2007  fix + vectorization in LAM adjoint 
!        F. Vana       Nov-2004  one less dummy to LARCINA
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!        30-Jun-2008 J. Masek New arg PCLO5, updated arguments of LARCINA.
!        F. Vana       14-Aug-2008 Weights driven interpolation + KDEP optim.
!        K. Yessad Nov 2008: rationalisation of dummy argument interfaces
!        F. Vana       13-Jan-2009: TL/AD of SLHD
! End Modifications
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGSAG   ,NDGSAL   ,NDGENG   ,NDGENL   ,NDGUNG   ,&
 &                    NDGUXG   ,NDLUNG   ,NDLUXG   ,NFLEVG   ,NDGSAH   ,&
 &                    NDGENH   ,NRLEVX   ,NPROMA   ,NFLSA    ,NFLEN
USE YOMCT0   , ONLY : LRPLANE
USE YOMDYNA  , ONLY : LSLHD    ,LSLHDQUAD
USE YOMDYN   , ONLY : NITMP    ,VETAON   ,VETAOX   ,LSETTLST ,LELTRA   ,LADVF
USE YOMCST   , ONLY : RPI
USE YOMGEM   , ONLY : NLOENG
USE YEMGEO   , ONLY : EDELX    ,EDELY
USE YOMRIP   , ONLY : RTDT
USE YOMSC2   , ONLY : NSLWIDE  ,NFLDSLB15,NFLDSLB2
USE YOMMP    , ONLY : NSLOFF   ,NFRSTLOFF,NASLB1
USE PTRSLB2  , ONLY : MSLB2KAPPA5

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LD_AD
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINDX(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINDY(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB15(NASLB1,NFLDSLB15)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLON5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLAT5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PR5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PS5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUF5(NPROMA,NFLEVG,NITMP-1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVF5(NPROMA,NFLEVG,NITMP-1) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KL0(NPROMA,NFLEVG,0:3,NITMP) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLH0(NPROMA,NFLEVG,0:3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLO5(NPROMA,NFLEVG,0:3,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLO5(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLOSLD5(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLAT5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLA5(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLASLD5(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDVER5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVINTW5(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVINTWSLD5(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOSCO5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSINCO5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOPHI5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSINLA5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLEV5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POUT5(NPROMA,NFLEVG,NITMP)
INTEGER(KIND=JPIM),INTENT(OUT)   :: KDEP(NPROMA,NFLEVG)

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZGMDTX(NPROMA)
REAL(KIND=JPRB) :: ZGMDTY(NPROMA)
REAL(KIND=JPRB) :: ZLSDEPI(NDGSAH:NDGENH)
INTEGER(KIND=JPIM) :: ILEV(NPROMA,NFLEVG)

INTEGER(KIND=JPIM) :: ISTALAT(NDGSAH:NDGENH)

INTEGER(KIND=JPIM) :: IGLGLO, IHVI, IVSEPC, IVSEPL, JGL
INTEGER(KIND=JPIM) :: IROT
INTEGER(KIND=JPIM) :: ITIP
LOGICAL :: LLSETTLST, LLVSEP
LOGICAL :: LLINTV

REAL(KIND=JPRB) :: ZUF0(1), ZURL0(1), ZVF0(1)
REAL(KIND=JPRB) :: ZVRL0(1), ZWF0(1), ZWFSM(1), ZWRL0(1)

REAL(KIND=JPRB) :: ZDEPI

REAL(KIND=JPRB), ALLOCATABLE :: ZUN_VDERW(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZUN_HVW(:,:,:)
REAL(KIND=JPRB) :: ZVINTWS5(NPROMA,NFLEVG,1:4)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "larcina.intfb.h"
#include "elarmes5.intfb.h"
#include "larmes5.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAPINEA5',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY INITIALISATIONS:

ZDEPI=2.0_JPRB*RPI
DO JGL=MAX(NDGSAG,NDGSAL+NFRSTLOFF-NSLWIDE)-NFRSTLOFF,&
   & MIN(NDGENG,NDGENL+NFRSTLOFF+NSLWIDE)-NFRSTLOFF  
  IGLGLO=JGL+NFRSTLOFF
  ZLSDEPI(JGL)=REAL(NLOENG(IGLGLO),JPRB)/ZDEPI
ENDDO

IF(LRPLANE) THEN
  ZGMDTX (KST:KPROF)=0.5_JPRB*PGM(KST:KPROF)*RTDT/EDELX
  ZGMDTY (KST:KPROF)=0.5_JPRB*PGM(KST:KPROF)*RTDT/EDELY
ENDIF

DO JGL=MAX(NDGSAH,LBOUND(NSLOFF,1)),MIN(NDGENH,UBOUND(NSLOFF,1))
  ISTALAT(JGL)=NSLOFF(JGL)
ENDDO

!     ------------------------------------------------------------------

!*       2.    COMPUTATION OF THE SL TRAJECTORY.
!              ---------------------------------

LLSETTLST=LSETTLST.AND.(.NOT.LELTRA)
POUT5(KST:KPROF,1:NFLEVG,1:NITMP)=0.0_JPRB

IF(LRPLANE) THEN

  CALL ELARMES5(KST,KPROF,ISTALAT,LLSETTLST,PINDX,PINDY,ZGMDTX,ZGMDTY,PB15,PB2,&
   & ZLSDEPI,PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,&
   & PLON5,PLAT5,PCOSCO5,PSINCO5,PSINLA5,PCOPHI5,PLEV5,PR5,PS5,PUF5,PVF5,&
   & KL0,KLH0,PDLO5,PDLAT5,ILEV,PDVER5,POUT5)  

ELSE

  CALL LARMES5(KST,KPROF,ISTALAT,LLSETTLST,PB15,PB2,&
   & ZLSDEPI,PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,PGNORDL,PGNORDM,&
   & PLON5,PLAT5,PCOSCO5,PSINCO5,PSINLA5,PCOPHI5,PLEV5,PR5,PS5,PUF5,PVF5,&
   & KL0,KLH0,PDLO5,PDLAT5,ILEV,PDVER5)  

ENDIF

!     ------------------------------------------------------------------

!*       3.    COMPUTATION OF WEIGHTS AND INDICES ARRAYS FOR INTERPOLATIONS.
!              -------------------------------------------------------------

!*       3.1   Full-level Quantities.
!              ---------------------

IHVI=0
LLVSEP=.FALSE.

IF(LD_AD)THEN
  ALLOCATE(ZUN_VDERW(NPROMA,NFLEVG,2*IHVI,2*IHVI))
  ALLOCATE(ZUN_HVW(NPROMA,NFLEVG,4*IHVI))

  IF (LRPLANE.AND.LADVF) THEN
  !*Use PSINLA and PCOPHI to transfer origin point coordinates to LAPINEAAD
  ! before they are recalculated by "2 Omega vectorial a k" recalculation.
    PSINLA5(KST:KPROF,1:NFLEVG,NITMP)=PLON5(KST:KPROF,1:NFLEVG,NITMP)
    PCOPHI5(KST:KPROF,1:NFLEVG,NITMP)=PLAT5(KST:KPROF,1:NFLEVG,NITMP)
  ENDIF

  IROT=1
  ITIP=3

  LLINTV=.FALSE.

  !  Warning: ZURL0,ZVRL0,ZWRL0,ZUF0,ZVF0,ZWF0,ZWFSM are not
  !           used in this call to LARCINA.

  CALL LARCINA(KST,KPROF,IHVI,ISTALAT,LLVSEP,LSLHD,LSLHDQUAD,LLINTV,&
   & ITIP,IROT,LRPLANE,ZLSDEPI,&
   & PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,&
   & PCOSCO5(1,1,NITMP),PSINCO5(1,1,NITMP),&
   & PSINLA5(1,1,NITMP),PCOPHI5(1,1,NITMP),PLEV5(1,1,NITMP),&
   & PB2(1,MSLB2KAPPA5),ZURL0,ZVRL0,ZWRL0,&
   & IVSEPC,IVSEPL,&
   & PLON5(1,1,NITMP),PLAT5(1,1,NITMP),PR5(1,1,NITMP),PS5(1,1,NITMP),&
   & ZUF0,ZVF0,ZWF0,ZWFSM,&
   & KL0(1,1,0,NITMP),KLH0(1,1,0),PDLO5(1,1,0,NITMP),PCLO5,PCLOSLD5,&
   & PDLAT5(1,1,NITMP),PCLA5,PCLASLD5,ILEV,&
   & PDVER5(1,1,NITMP),PVINTW5,PVINTWSLD5,ZVINTWS5,&
   & ZUN_VDERW,ZUN_HVW,KDEP)

  DEALLOCATE(ZUN_VDERW)
  DEALLOCATE(ZUN_HVW)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAPINEA5',1,ZHOOK_HANDLE)

END SUBROUTINE LAPINEA5
