SUBROUTINE LARCHE(KPROMA,KSTART,KPROF,KFLEV,&
 & KSTTYP,PDSTRET,PC2M1,PC2P1,PI,PDEPI,&
 & PLOCEN,PMUCEN,PCOLON,PSILON,&
 & PGEMU,PGSQM2,PGECLO,PGESLO,&
 & PCOSCO,PSINCO,PSINLA,PCOPHI,&
 & KROT,PLON,PLAT,PO,PQ)  

!**** *LARCHE  -  semi-LAgrangian scheme:
!                 Research of the Coordinates (of the medium or origin
!                 point).

!     Purpose.
!     --------
!       Computes the longitude and latitude of the interpolation
!       point from its cartesian coordinates.
!       Then computes the vector displacement matrix
!                        I po pq I
!                        I       I
!                        I-pq po I
!       from the interpolation point to the grid point.

!**   Interface.
!     ----------
!        *CALL* *LARCHE(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension.
!          KSTART  - first element of arrays where computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KSTTYP  - 1: Not tilted pole;  2: Tilted pole.
!          PDSTRET - 2*c (where c is the stretching factor).
!          PC2M1   - c*c-1.
!          PC2P1   - c*c+1.
!          PI      - number PI
!          PDEPI   - 2 * PI
!          PLOCEN  - geographic longitude of the stretching pole.
!          PMUCEN  - sinus of the geographic latitude of the stretching pole.
!          PCOLON  - cosinus of the g.p. longitude on the computational sphere.
!          PSILON  - sinus of the g.p. longitude on the computational sphere.
!          PGEMU   - sinus of the grid-point geographic latitude.
!          PGSQM2  - cosinus of the grid-point geographic latitude.
!          PGECLO  - cosinus of the grid-point geographic longitude.
!          PGESLO  - sinus of the grid-point geographic longitude.
!          PCOSCO  - cos(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographic lon and lat).
!          PSINCO  - sin(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographic lon and lat).
!          PSINLA  - sinus of the interpolation point geographic latitude.
!          PCOPHI  - cosinus of the geographic angle between the
!                    interpolation point and the grid-point.
!          KROT    - KROT=1: computation of the elements po and pq
!                    of the wind displacement matrix.
!                    KROT=0: no computation.

!        OUTPUT:
!          PLON    - longitude of the interpolation point in the
!                    computational sphere.
!          PLAT    - latitude of the interpolation point in the
!                    computational sphere.
!          PO      - first element of the wind displacement matrix.
!          PQ      - second element of the wind displacement matrix.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.
!        Called by LARCINA.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD, after the subroutine LAGINT0 written by Maurice IMBARD
!        Alain CRAPLET and Michel ROCHAS  METEO FRANCE/EERM/CRMD

!     Modifications.
!     --------------
!        Original : FEBRUARY 1992.
!        J. HAGUE : JANUARY  2003.   MASS Vector Functions       
!        D.SALMOND : JUNE  2003.    Remove vdiv for adjoint test
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.Yessad : Aug 2005 : Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMJFH, ONLY : N_VMASS

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTTYP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDSTRET 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PC2M1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PC2P1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDEPI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLOCEN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMUCEN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOSCO(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINCO(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINLA(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOPHI(KPROMA,KFLEV) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KROT 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLON(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLAT(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PO(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZTMP1(KPROMA)
REAL(KIND=JPRB) :: ZTMP2(KPROMA)
REAL(KIND=JPRB) :: ZTMP4(KPROMA)
REAL(KIND=JPRB) :: ZTMP5(KPROMA)
REAL(KIND=JPRB) :: ZTMP6(KPROMA)
REAL(KIND=JPRB) :: ZTMP7(KPROMA)

INTEGER(KIND=JPIM) :: JLEV, JROF, ILEN

REAL(KIND=JPRB) :: Z1ST, Z1STT, ZCA, ZCOSCO, ZCOSL2, ZCOSLA,&
 & ZCOSLR, ZDECO, ZDI, ZEPS, ZFACT, ZMU, ZP, &
 & ZPCLAP, ZPCLOP, ZPSLAP, ZPSLOP, ZQ, ZSA, &
 & ZSINCO, ZSINLA, ZXX, ZYY, ZZP, ZZW, &
 & ZZX, ZZY, ZZZ  

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARCHE',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    COMPUTATION OF LAT LON OF THE INTERPOLATION POINT.
!              IF KROT=1 COMPUTATION OF THE WIND DISPLACEMENT MATRIX
!              FROM THE INTERPOLATION POINT TO THE FINAL POINT.
!              ( T FOR LATITUDE THETA, L FOR LONGITUDE LAMBDA).
!              PO = ( 1 / (1+cos(PHI)) )
!                  *( cos(TG)*cos(T) + (1+sin(TG)*sin(T))*cos(L-LG) )
!              PQ = (-1 / (1+cos(PHI)) )
!                  *( sin(TG)+sin(T) ) * sin(L-LG)
!     ------------------------------------------------------------------

ZEPS=1.E-14_JPRB

IF(KSTTYP == 1) THEN

!        POLE STRETCHING, NO POLE TILTING.

  IF(N_VMASS <= 0) THEN

!   ---- Not using MASS library vector routines

    DO JLEV=1,KFLEV
      DO JROF=KSTART,KPROF
        ZCOSLR=SQRT(PCOSCO(JROF,JLEV)*PCOSCO(JROF,JLEV)&
         & +PSINCO(JROF,JLEV)*PSINCO(JROF,JLEV))  
        ZCOSLR=MAX(ZEPS,ZCOSLR)
        ZFACT = PDSTRET/(PC2P1-PC2M1*PSINLA(JROF,JLEV))
        ZCOSCO= PCOSCO(JROF,JLEV)*ZFACT
        ZSINCO= PSINCO(JROF,JLEV)*ZFACT
        ZCOSLA= ZCOSLR*ZFACT
        ZSINLA= (PC2P1*PSINLA(JROF,JLEV)-PC2M1)/(PC2P1-PC2M1*PSINLA(JROF,JLEV))
        ZXX   = PCOLON(JROF)*ZCOSCO-PSILON(JROF)*ZSINCO
        ZYY   = PSILON(JROF)*ZCOSCO+PCOLON(JROF)*ZSINCO
        ZMU   = MAX(-1.0_JPRB,MIN(1.0_JPRB,ZSINLA))
        ZZW   = MAX(-1.0_JPRB,MIN(1.0_JPRB,ZXX/ZCOSLA))
        PLAT(JROF,JLEV)=ASIN(ZMU)
        PLON(JROF,JLEV)=MOD(PI+SIGN(1.0_JPRB,ZYY)*(ACOS(ZZW)-PI),PDEPI)
        IF (KROT == 1) THEN
          ZDECO =1.0_JPRB/(1.0_JPRB+PCOPHI(JROF,JLEV))
          Z1ST  =1.0_JPRB/ZCOSLR
          PO(JROF,JLEV)=(PGSQM2(JROF)*ZCOSLR &
           & +(1.0_JPRB+PGEMU(JROF)*PSINLA(JROF,JLEV))&
           & *PCOSCO(JROF,JLEV)*Z1ST )*ZDECO  
          PQ(JROF,JLEV)=-(PGEMU(JROF)+PSINLA(JROF,JLEV))&
           & *(PSINCO(JROF,JLEV)*Z1ST)*ZDECO  
        ENDIF
      ENDDO
    ENDDO

  ELSE

!   ---- Using MASS library vector routines

    ILEN=KPROF-KSTART+1
    DO JLEV=1,KFLEV
      DO JROF=KSTART,KPROF
        ZTMP1(JROF-KSTART+1)=PC2P1-PC2M1*PSINLA(JROF,JLEV)
      ENDDO

      CALL VREC(ZTMP2,ZTMP1,ILEN)

      DO JROF=KSTART,KPROF
        ZCOSLR=SQRT(PCOSCO(JROF,JLEV)*PCOSCO(JROF,JLEV)&
         & +PSINCO(JROF,JLEV)*PSINCO(JROF,JLEV))  
        ZCOSLR=MAX(ZEPS,ZCOSLR)
        ZFACT = PDSTRET*ZTMP2(JROF-KSTART+1)
        ZCOSCO= PCOSCO(JROF,JLEV)*ZFACT
        ZSINCO= PSINCO(JROF,JLEV)*ZFACT
        ZCOSLA= ZCOSLR*ZFACT
        ZSINLA= (PC2P1*PSINLA(JROF,JLEV)-PC2M1)*ZTMP2(JROF-KSTART+1)
        ZXX   = PCOLON(JROF)*ZCOSCO-PSILON(JROF)*ZSINCO
        ZYY   = PSILON(JROF)*ZCOSCO+PCOLON(JROF)*ZSINCO
        PLON(JROF,JLEV)=SIGN(1.0_JPRB,ZYY)
        ZTMP2(JROF-KSTART+1)   =ZSINLA 
        ZTMP7(JROF-KSTART+1)  = ZXX/ZCOSLA
! Note: replacing ZXX/ZCOSLA by vdiv not good for adjoint test
        IF(KROT == 1) THEN
          ZTMP4(JROF-KSTART+1) = 1.0_JPRB+PCOPHI(JROF,JLEV)
          ZTMP5(JROF-KSTART+1) = ZCOSLR
        ENDIF
      ENDDO

      CALL JFH_VIF2(ZTMP2,KPROF-KSTART+1)
      CALL VASIN(ZTMP6,ZTMP2,ILEN)
      CALL JFH_VIF2(ZTMP2,ILEN)
      CALL VASIN(PLAT(KSTART,JLEV),ZTMP2,ILEN)
      CALL JFH_VIF2(ZTMP7,ILEN)
      CALL VACOS(ZTMP1,ZTMP7,ILEN)

      DO JROF=KSTART,KPROF
        PLON(JROF,JLEV)=PI+PLON(JROF,JLEV)*(ZTMP1(JROF-KSTART+1)-PI)
      ENDDO

      CALL JFH_VMOD(PLON(KSTART,JLEV),PDEPI,KPROF-KSTART+1)

      IF (KROT == 1) THEN

        CALL VREC(ZTMP1,ZTMP4,ILEN)
        CALL VREC(ZTMP2,ZTMP5,ILEN)

        DO JROF=KSTART,KPROF
          ZCOSLR=ZTMP5(JROF-KSTART+1)
          ZDECO =ZTMP1(JROF-KSTART+1)
          Z1ST  =ZTMP2(JROF-KSTART+1)
          PO(JROF,JLEV)=(PGSQM2(JROF)*ZCOSLR &
           & +(1.0_JPRB+PGEMU(JROF)*PSINLA(JROF,JLEV))&
           & *PCOSCO(JROF,JLEV)*Z1ST )*ZDECO  
          PQ(JROF,JLEV)=-(PGEMU(JROF)+PSINLA(JROF,JLEV))&
           & *(PSINCO(JROF,JLEV)*Z1ST)*ZDECO  

        ENDDO
      ENDIF
    ENDDO
  ENDIF !End using MASS library vector routines

ELSEIF(KSTTYP == 2) THEN

!        POLE STRETCHING AND TILTING.

  IF(N_VMASS > 0) THEN
    CALL ABOR1(' LARCHE: VMASS lib not yet coded with NSTTYP=2')
  ENDIF

  ZPSLAP = PMUCEN
  ZPCLAP = SQRT(1.0_JPRB-PMUCEN*PMUCEN)
  ZPSLOP = SIN(PLOCEN)
  ZPCLOP = COS(PLOCEN)

  DO JLEV=1,KFLEV
    DO JROF=KSTART,KPROF

      ZZY   = PGESLO(JROF)*PCOSCO(JROF,JLEV)+PGECLO(JROF)*PSINCO(JROF,JLEV)
      ZZX   = PGECLO(JROF)*PCOSCO(JROF,JLEV)-PGESLO(JROF)*PSINCO(JROF,JLEV)
      ZCOSL2= ZZY*ZZY+ZZX*ZZX
      ZCOSLR= SQRT(ZCOSL2)
      ZCOSLR= MAX(ZEPS,ZCOSLR)
      ZZZ   = ZZX*ZPCLOP+ZPSLOP*ZZY
      ZZP   = ZPSLAP*PSINLA(JROF,JLEV)+ZPCLAP*ZZZ
      ZDI= 1.0_JPRB/(PC2P1-PC2M1*ZZP)
      ZYY   = PDSTRET*ZDI*(-ZPCLOP*ZZY+ZPSLOP*ZZX)
      ZXX   = PDSTRET*ZDI*(ZPCLAP*PSINLA(JROF,JLEV)-ZPSLAP*ZZZ)
      ZCOSLA= SQRT(ZXX*ZXX+ZYY*ZYY)
      ZCOSLA= MAX(ZEPS,ZCOSLA)
      ZSINLA=-(PC2M1-PC2P1*ZZP)*ZDI
      ZMU   = MAX(-1.0_JPRB,MIN(1.0_JPRB,ZSINLA))
      ZZW   = MAX(-1.0_JPRB,MIN(1.0_JPRB,ZXX/ZCOSLA))

      PLAT(JROF,JLEV)=ASIN(ZMU)
      PLON(JROF,JLEV)=MOD(PI+SIGN(1.0_JPRB,ZYY)*(ACOS(ZZW)-PI),PDEPI)

      IF (KROT == 1) THEN

        ZDECO =1.0_JPRB/(1.0_JPRB+PCOPHI(JROF,JLEV))
        Z1ST  =1.0_JPRB/ZCOSLR
        Z1STT =Z1ST/ZCOSLA

        ZP = (PGSQM2(JROF)*ZCOSLR &
         & +(1.0_JPRB+PGEMU(JROF)*PSINLA(JROF,JLEV))&
         & *PCOSCO(JROF,JLEV)*Z1ST )*ZDECO  
        ZQ =-(PGEMU(JROF)+PSINLA(JROF,JLEV))*(PSINCO(JROF,JLEV)*Z1ST )*ZDECO
        ZCA=PDSTRET*ZDI*(ZPSLAP*ZCOSL2-ZPCLAP*PSINLA(JROF,JLEV)*ZZZ)*Z1STT
        ZSA=-ZYY*ZPCLAP*Z1STT
        PO(JROF,JLEV)=ZP*ZCA+ZQ*ZSA
        PQ(JROF,JLEV)=ZQ*ZCA-ZP*ZSA

      ENDIF
    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARCHE',1,ZHOOK_HANDLE)

END SUBROUTINE LARCHE

