! This directive is necessary to stop pre-computing the reciprocal of PCOSLA5
! leads to bad test of adjoint /MH
!OCL NOEVAL
SUBROUTINE LARCHE5(KPROMA,KSTART,KPROF,KFLEV,KLEV,&
 & KSTTYP,PDSTRET,PC2M1,PC2P1,PI,PDEPI,&
 & PLOCEN,PMUCEN,PCOLON,PSILON,&
 & PGEMU,PGSQM2,PGECLO,PGESLO,&
 & PCOSCO5,PSINCO5,PSINLA5,PCOPHI5,&
 & KROT,PLON5,PLAT5,PO5,PQ5,&
 & PCOSLR5,PCOSLA5,PCOSL25,PXX5,PYY5,PZW5,PZA5,PDECO5,P1ST5,&
 & PFACT5,PZSINLA5,PZCOSLA5,PZSILON5,&
 & PZY5,PZX5,PDI5,PZZ5,PZP5,P1STT5,PRP5,PRQ5,PCA5,PSA5)  

!**** *LARCHE5 -  semi-LAgrangian scheme:
!                 Research of the Coordinates (of the medium or origin
!                 point).
!                 Version of LARCHE for computation of trajectory.

!     Purpose.
!     --------
!       Computes the longitude and latitude of the interpolation
!       point from its cartesian coordinates.
!       Then computes the vector displacement matrix
!                        I po pq I
!                        I       I
!                        I-pq po I
!       from the interpolation point to the grid point.

!       The differences with LARCHE are the following ones:
!       - treatment of one level (KLEV).
!       - Some variables which are local in LARCHE become
!         output dummy arguments in LARCHE5: 
!         PCOSLR5,PCOSLA5,PCOSL25,PXX5,PYY5,PZW5,PZA5,PDECO5,P1ST5,
!         PFACT5,PZSINLA5,PZCOSLA5,PZSILON5,
!         PZY5,PZX5,PDI5,PZZ5,PZP5,P1STT5,PRP5,PRQ5,PCA5,PSA5.
!       - Separation of the cases "no tilting, c=1" and
!         "no tilting, c>1" only for optimisations (this
!         type of optimisation could be done also in LARCHE).

!**   Interface.
!     ----------
!        *CALL* *LARCHE5(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension.
!          KSTART  - first element of arrays where computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KLEV    - current level.
!          KSTTYP  - 1: Not tilted pole;  2: Tilted pole.
!          PDSTRET - 2*c (where c is the stretching factor).
!          PC2M1   - c*c-1.
!          PC2P1   - c*c+1.
!          PI      - number PI
!          PDEPI   - 2 * PI
!          PLOCEN  - geographic longitude of the stretching pole.
!          PMUCEN  - sinus of the geographic latitude of the stretching pole.
!          PCOLON  - cosinus of the g.p. longitude on the computational sphere.
!          PSILON  - sinus of the g.p. longitude on the computational sphere.
!          PGEMU   - sinus of the grid-point geographic latitude.
!          PGSQM2  - cosinus of the grid-point geographic latitude.
!          PGECLO  - cosinus of the grid-point geographic longitude.
!          PGESLO  - sinus of the grid-point geographic longitude.
!          PCOSCO5 - cos(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographic lon and lat).
!          PSINCO5 - sin(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographic lon and lat).
!          PSINLA5 - sinus of the interpolation point geographic latitude.
!          PCOPHI5 - cosinus of the geographic angle between the
!                    interpolation point and the grid-point.
!          KROT    - KROT=1: computation of the elements po and pq
!                    of the wind displacement matrix.
!                    KROT=0: no computation.

!        OUTPUT (trajectory):
!          PLON5   - longitude of the interpolation point in the
!                    computational sphere.
!          PLAT5   - latitude of the interpolation point in the
!                    computational sphere.
!          PO5     - first element of the wind displacement matrix.
!          PQ5     - second element of the wind displacement matrix.
!          And some intermediate quantities which are local variables
!          (or not computed at all) in LARCHE.
!          PCOSLR5 -
!          PCOSLA5 -
!          PCOSL25 -
!          PXX5    -
!          PYY5    -
!          PZW5    -
!          PZA5    -
!          PDECO5  -
!          P1ST5   -
!          PFACT5  -
!          PZSINLA5- sin(PLAT5) in all cases (including tilting/stretching)
!                    (sinus of the interpol point lat in the comput. sphere)
!          PZCOSLA5- cos(PLAT5) in all cases (including tilting/stretching)
!                    (cosinus of the interpol point lat in the comput. sphere)
!          PZSILON5- sin(PLON5) in all cases (including tilting/stretching)
!                    (sinus of the interpol point lon in the comput. sphere)
!          PZY5    -
!          PZX5    -
!          PDI5    -
!          PZZ5    -
!          PZP5    -
!          P1STT5  -
!          PRP5    -
!          PRQ5    -
!          PCA5    -
!          PSA5    -

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.
!        Called by LARCHETL and LARCHEAD.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD, after the subroutine LAGINT0 written by Maurice IMBARD
!        Alain CRAPLET and Michel ROCHAS  METEO FRANCE/EERM/CRMD

!     Modifications.
!     --------------
!        Original : FEBRUARY 1992.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad: Aug 2005 : code AD+TL for variable mesh.
!        K. Yessad (Sep 2008): update comments + cleanings.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMJFH, ONLY : N_VMASS

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTTYP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDSTRET
REAL(KIND=JPRB)   ,INTENT(IN)    :: PC2M1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PC2P1
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDEPI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLOCEN
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMUCEN
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOSCO5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINCO5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINLA5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOPHI5(KPROMA,KFLEV) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KROT 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLON5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLAT5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PO5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ5(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOSLR5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOSLA5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOSL25(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXX5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PYY5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZW5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZA5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDECO5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P1ST5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFACT5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZSINLA5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZCOSLA5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZSILON5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZY5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZX5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZZ5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZP5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P1STT5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRP5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRQ5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCA5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSA5(KPROMA) 

!     ------------------------------------------------------------------

! Temporaries for fast path
REAL(KIND=JPRB) :: ZTMP1(KPROF-KSTART+1)
REAL(KIND=JPRB) :: ZTMP2(KPROF-KSTART+1)
REAL(KIND=JPRB) :: ZTMP3(KPROF-KSTART+1)
REAL(KIND=JPRB) :: ZTMP4(KPROF-KSTART+1)

INTEGER(KIND=JPIM) :: JROF, JLEN

REAL(KIND=JPRB) :: ZCOSCO,ZSINCO,ZPSLAP,ZPCLAP,ZPSLOP,ZPCLOP
REAL(KIND=JPRB) :: ZEPS1,ZEPS,ZMU5

LOGICAL         :: LLSTR

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARCHE5',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    COMPUTATION OF LAT LON OF THE INTERPOLATION POINT.
!              IF KROT=1 COMPUTATION OF THE WIND DISPLACEMENT MATRIX
!              FROM THE INTERPOLATION POINT TO THE FINAL POINT.
!              ( T FOR LATITUDE THETA, L FOR LONGITUDE LAMBDA).
!              PO = ( 1 / (1+cos(PHI)) )
!                  *( cos(TG)*cos(T) + (1+sin(TG)*sin(T))*cos(L-LG) )
!              PQ = (-1 / (1+cos(PHI)) )
!                  *( sin(TG)+sin(T) ) * sin(L-LG)
!     ------------------------------------------------------------------

! K.Y.: definition of LLSTR (c>1) must be the same everywhere in the code.
ZEPS1=100.0_JPRB*TINY(1.0_JPRB)
LLSTR=(ABS(0.5_JPRB*PDSTRET-1.0_JPRB)>ZEPS1)

ZEPS=1.E-14_JPRB

IF(KSTTYP == 1.AND.(.NOT.LLSTR)) THEN

!        NO POLE STRETCHING, NO POLE TILTING.

  IF(N_VMASS <= 0) THEN  ! Not using VMASS lib

    DO JROF=KSTART,KPROF
      PCOSLR5(JROF)=SQRT(PCOSCO5(JROF,KLEV)*PCOSCO5(JROF,KLEV)&
       & +PSINCO5(JROF,KLEV)*PSINCO5(JROF,KLEV))  
      PCOSLR5(JROF)=MAX(PCOSLR5(JROF),ZEPS)
      PCOSLA5(JROF)=PCOSLR5(JROF)
      PXX5(JROF)=PCOLON(JROF)*PCOSCO5(JROF,KLEV)&
       & -PSILON(JROF)*PSINCO5(JROF,KLEV)  
      PYY5(JROF)=PSILON(JROF)*PCOSCO5(JROF,KLEV)&
       & +PCOLON(JROF)*PSINCO5(JROF,KLEV)  
      ZMU5=PSINLA5(JROF,KLEV)
      IF (ABS(ZMU5) >= 1.0_JPRB) THEN
        ZMU5=SIGN(1.0_JPRB,ZMU5)
      ENDIF
      PLAT5(JROF,KLEV)=ASIN(ZMU5)
      PZW5(JROF)=PXX5(JROF)/PCOSLA5(JROF)
      IF (ABS(PZW5(JROF)) >= 1.0_JPRB) THEN
        PZW5(JROF) = SIGN(1.0_JPRB,PZW5(JROF))
      ENDIF
      
      PLON5(JROF,KLEV)=MOD(PI+SIGN(1.0_JPRB,PYY5(JROF))*&
       & (ACOS(PZW5(JROF))-PI),PDEPI)  
      
      PZA5(JROF)=PYY5(JROF)/PCOSLA5(JROF)
      IF (ABS(PZA5(JROF)) >= 1.0_JPRB) THEN
        PZA5(JROF) = SIGN(1.0_JPRB,PZA5(JROF))
      ENDIF
    ENDDO
    
  ELSE  ! Using VMASS lib

    JLEN=KPROF-KSTART+1

    DO JROF=KSTART,KPROF
      PCOSLR5(JROF)=SQRT(PCOSCO5(JROF,KLEV)*PCOSCO5(JROF,KLEV)&
       & +PSINCO5(JROF,KLEV)*PSINCO5(JROF,KLEV))  
      PCOSLR5(JROF)=MAX(PCOSLR5(JROF),ZEPS)
      PCOSLA5(JROF)=PCOSLR5(JROF)
!     ZTMP1(JROF-KSTART+1)=PCOSLR5(JROF)
    ENDDO
!   CALL VREC(ZTMP0,ZTMP1,JLEN) ! Not used in order to get bit reproducibility
    DO JROF=KSTART,KPROF
      PXX5(JROF)=PCOLON(JROF)*PCOSCO5(JROF,KLEV)&
       & -PSILON(JROF)*PSINCO5(JROF,KLEV)  
      PYY5(JROF)=PSILON(JROF)*PCOSCO5(JROF,KLEV)&
       & +PCOLON(JROF)*PSINCO5(JROF,KLEV)  
      ZMU5=PSINLA5(JROF,KLEV)
      IF (ABS(ZMU5) >= 1.0_JPRB) THEN
        ZMU5=SIGN(1.0_JPRB,ZMU5)
      ENDIF
      ZTMP1(JROF-KSTART+1)=ZMU5
      PZW5(JROF)=PXX5(JROF)/PCOSLA5(JROF)
!     PZW5(JROF)=PXX5(JROF)*ZTMP0(JROF-KSTART+1)
      IF (ABS(PZW5(JROF)) >= 1.0_JPRB) THEN
        PZW5(JROF) = SIGN(1.0_JPRB,PZW5(JROF))
      ENDIF
      ZTMP2(JROF-KSTART+1)=PZW5(JROF)
    ENDDO
    
    CALL VASIN(ZTMP3,ZTMP1,JLEN)
    CALL VACOS(ZTMP4,ZTMP2,JLEN)
!   DO JROF=KSTART,KPROF
!     ZTMP3(JROF)=(PI/_TWO_)-ACOS(ZTMP1(JROF))
!     ZTMP4(JROF)=ACOS(ZTMP2(JROF))
!   ENDDO
  
    DO JROF=KSTART,KPROF
      PLAT5(JROF,KLEV)=ZTMP3(JROF-KSTART+1)
      PLON5(JROF,KLEV)=PI+SIGN(1.0_JPRB,PYY5(JROF))*(ZTMP4(JROF-KSTART+1)-PI)
      PZA5(JROF)=PYY5(JROF)/PCOSLA5(JROF)
!     PZA5(JROF)=PYY5(JROF)*ZTMP0(JROF-KSTART+1)
      IF (ABS(PZA5(JROF)) >= 1.0_JPRB) THEN
        PZA5(JROF) = SIGN(1.0_JPRB,PZA5(JROF))
      ENDIF
    ENDDO
  
    CALL JFH_VMOD(PLON5(KSTART,KLEV),PDEPI,KPROF-KSTART+1)
!   DO JROF=KSTART,KPROF
!     PLON5(JROF,KLEV)=MOD(PLON5(JROF,KLEV),PDEPI)
!   ENDDO

  ENDIF
      
  IF (KROT == 1) THEN
    DO JROF=KSTART,KPROF
      PDECO5(JROF)=1.0_JPRB/(1.0_JPRB+PCOPHI5(JROF,KLEV))
      P1ST5(JROF) = 1.0_JPRB/PCOSLA5(JROF)
      PO5(JROF,KLEV)=(PGSQM2(JROF)*PCOSLA5(JROF) &
       & +(1.0_JPRB+PGEMU(JROF)*PSINLA5(JROF,KLEV))&
       & *PCOSCO5(JROF,KLEV)*P1ST5(JROF))*PDECO5(JROF)  
      PQ5(JROF,KLEV)=-(PGEMU(JROF)+PSINLA5(JROF,KLEV))&
       & *PSINCO5(JROF,KLEV)*P1ST5(JROF)*PDECO5(JROF)  
    ENDDO
  ENDIF

ELSEIF(KSTTYP == 1.AND.LLSTR) THEN

  !        STRETCHING, NO POLE TILTING.

  ! Remark: this code is also valid for the case ksttyp=1, c=1.

  IF(N_VMASS <= 0) THEN  ! Not using VMASS lib

    DO JROF=KSTART,KPROF
      PCOSLR5(JROF)=SQRT(PCOSCO5(JROF,KLEV)*PCOSCO5(JROF,KLEV)&
       & +PSINCO5(JROF,KLEV)*PSINCO5(JROF,KLEV))  
      PCOSLR5(JROF)=MAX(PCOSLR5(JROF),ZEPS)

      PFACT5(JROF)=PDSTRET/(PC2P1-PC2M1*PSINLA5(JROF,KLEV))
      ZCOSCO=PCOSCO5(JROF,KLEV)*PFACT5(JROF)
      ZSINCO=PSINCO5(JROF,KLEV)*PFACT5(JROF)

      PCOSLA5(JROF)=PCOSLR5(JROF)*PFACT5(JROF)
      PZSINLA5(JROF)= &
       & (PC2P1*PSINLA5(JROF,KLEV)-PC2M1)/(PC2P1-PC2M1*PSINLA5(JROF,KLEV))
      PXX5(JROF)=PCOLON(JROF)*ZCOSCO-PSILON(JROF)*ZSINCO  
      PYY5(JROF)=PSILON(JROF)*ZCOSCO+PCOLON(JROF)*ZSINCO
      ZMU5=PZSINLA5(JROF)
      IF (ABS(ZMU5) >= 1.0_JPRB) THEN
        ! * bound PZSINLA5 and ZMU5 in [-1,1].
        ZMU5=SIGN(1.0_JPRB,ZMU5)
        PZSINLA5(JROF)=ZMU5
      ENDIF
      PLAT5(JROF,KLEV)=ASIN(ZMU5)
      ! * Provides cos(plat5) in [0,1], we then prevent to have 0
      !   and we keep cos(plat5) in [zeps,1] to avoid divisions by zero.
      PZCOSLA5(JROF)=MAX(COS(PLAT5(JROF,KLEV)),ZEPS)

      PZW5(JROF)=PXX5(JROF)/PCOSLA5(JROF)
      IF (ABS(PZW5(JROF)) >= 1.0_JPRB) THEN
        PZW5(JROF) = SIGN(1.0_JPRB,PZW5(JROF))
      ENDIF

      ! * this is equivalent to the expression
      !   " plon5=mod(pi+sign(1,pyy5)*(acos(pzw5)-pi),2*pi) "
      !   but more understandable.
      !   Provides sin(plon5) in [-1,1]; we then prevent to have 0
      !   and we keep abs(sin(plon5)) in [zeps,1] to avoid divisions by zero.
      IF (PYY5(JROF) >= 0) THEN
        ! * in this case plon5 is in [0,pi] and sin(plon5) is in [0,1].
        PLON5(JROF,KLEV)=ACOS(PZW5(JROF))
        PZSILON5(JROF)=MAX(ZEPS,SIN(PLON5(JROF,KLEV)))
      ELSE
        ! * in this case plon5 is either 0 or in ]pi,2*pi[
        !   and sin(plon5) is in [-1,0].
        PLON5(JROF,KLEV)=MOD(PDEPI-ACOS(PZW5(JROF)),PDEPI)
        PZSILON5(JROF)=-MAX(ZEPS,-SIN(PLON5(JROF,KLEV)))
      ENDIF
    ENDDO
      
  ELSE  ! Using VMASS lib

    ! Not yet coded.
    CALL ABOR1(' LARCHE5: VMASS lib not yet coded with c>1')

  ENDIF
      
  IF (KROT == 1) THEN
    DO JROF=KSTART,KPROF
      PDECO5(JROF)=1.0_JPRB/(1.0_JPRB+PCOPHI5(JROF,KLEV))
      P1ST5(JROF) = 1.0_JPRB/PCOSLR5(JROF)
      PO5(JROF,KLEV)=(PGSQM2(JROF)*PCOSLR5(JROF) &
       & +(1.0_JPRB+PGEMU(JROF)*PSINLA5(JROF,KLEV))&
       & *PCOSCO5(JROF,KLEV)*P1ST5(JROF))*PDECO5(JROF)
      PQ5(JROF,KLEV)=-(PGEMU(JROF)+PSINLA5(JROF,KLEV))&
       & *PSINCO5(JROF,KLEV)*P1ST5(JROF)*PDECO5(JROF)
    ENDDO
  ENDIF

ELSEIF(KSTTYP == 2) THEN

  !        TILTING

  ZPSLAP = PMUCEN
  ZPCLAP = SQRT(1.0_JPRB-PMUCEN*PMUCEN)
  ZPSLOP = SIN(PLOCEN)
  ZPCLOP = COS(PLOCEN)

  IF(N_VMASS <= 0) THEN  ! Not using VMASS lib

    DO JROF=KSTART,KPROF

      PZY5(JROF)=PGESLO(JROF)*PCOSCO5(JROF,KLEV)+PGECLO(JROF)*PSINCO5(JROF,KLEV)
      PZX5(JROF)=PGECLO(JROF)*PCOSCO5(JROF,KLEV)-PGESLO(JROF)*PSINCO5(JROF,KLEV)
      PCOSL25(JROF)=PZY5(JROF)*PZY5(JROF)+PZX5(JROF)*PZX5(JROF)

      PCOSLR5(JROF)=SQRT(PCOSL25(JROF))
      PCOSLR5(JROF)=MAX(PCOSLR5(JROF),ZEPS)

      PZZ5(JROF)=ZPCLOP*PZX5(JROF)+ZPSLOP*PZY5(JROF)
      PZP5(JROF)=ZPSLAP*PSINLA5(JROF,KLEV)+ZPCLAP*PZZ5(JROF)
      PDI5(JROF)=1.0_JPRB/(PC2P1-PC2M1*PZP5(JROF))

      PYY5(JROF)=PDSTRET*PDI5(JROF)*(-ZPCLOP*PZY5(JROF)+ZPSLOP*PZX5(JROF))
      PXX5(JROF)=PDSTRET*PDI5(JROF) &
       & *(ZPCLAP*PSINLA5(JROF,KLEV)-ZPSLAP*PZZ5(JROF))

      PCOSLA5(JROF)=SQRT(PXX5(JROF)*PXX5(JROF)+PYY5(JROF)*PYY5(JROF))
      PCOSLA5(JROF)=MAX(PCOSLA5(JROF),ZEPS)

      PZSINLA5(JROF)=-(PC2M1-PC2P1*PZP5(JROF))*PDI5(JROF)
      ZMU5=PZSINLA5(JROF)
      IF (ABS(ZMU5) >= 1.0_JPRB) THEN
        ! * bound PZSINLA5 and ZMU5 in [-1,1].
        ZMU5=SIGN(1.0_JPRB,ZMU5)
        PZSINLA5(JROF)=ZMU5
      ENDIF
      PLAT5(JROF,KLEV)=ASIN(ZMU5)
      ! * Provides cos(plat5) in [0,1], we then prevent to have 0
      !   and we keep cos(plat5) in [zeps,1] to avoid divisions by zero.
      PZCOSLA5(JROF)=MAX(COS(PLAT5(JROF,KLEV)),ZEPS)

      PZW5(JROF)=PXX5(JROF)/PCOSLA5(JROF)
      IF (ABS(PZW5(JROF)) >= 1.0_JPRB) THEN
        PZW5(JROF) = SIGN(1.0_JPRB,PZW5(JROF))
      ENDIF
      
      ! * this is equivalent to the expression
      !   " plon5=mod(pi+sign(1,pyy5)*(acos(pzw5)-pi),2*pi) "
      !   but more understandable.
      !   Provides sin(plon5) in [-1,1]; we then prevent to have 0
      !   and we keep abs(sin(plon5)) in [zeps,1] to avoid divisions by zero.
      IF (PYY5(JROF) >= 0) THEN
        ! * in this case plon5 is in [0,pi] and sin(plon5) is in [0,1].
        PLON5(JROF,KLEV)=ACOS(PZW5(JROF))
        PZSILON5(JROF)=MAX(ZEPS,SIN(PLON5(JROF,KLEV)))
      ELSE
        ! * in this case plon5 is either 0 or in ]pi,2*pi[
        !   and sin(plon5) is in [-1,0].
        PLON5(JROF,KLEV)=MOD(PDEPI-ACOS(PZW5(JROF)),PDEPI)
        PZSILON5(JROF)=-MAX(ZEPS,-SIN(PLON5(JROF,KLEV)))
      ENDIF
    ENDDO
      
  ELSE  ! Using VMASS lib

    ! Not yet coded.
    CALL ABOR1(' LARCHE5: VMASS lib not yet coded with NSTTYP=2')

  ENDIF

  IF (KROT == 1) THEN
    DO JROF=KSTART,KPROF
      PDECO5(JROF)=1.0_JPRB/(1.0_JPRB+PCOPHI5(JROF,KLEV))
      P1ST5(JROF) = 1.0_JPRB/PCOSLR5(JROF)

      PRP5(JROF)=(PGSQM2(JROF)*PCOSLR5(JROF) &
       & +(1.0_JPRB+PGEMU(JROF)*PSINLA5(JROF,KLEV))&
       & *PCOSCO5(JROF,KLEV)*P1ST5(JROF) )*PDECO5(JROF)
      PRQ5(JROF)=-(PGEMU(JROF)+PSINLA5(JROF,KLEV))&
       &*(PSINCO5(JROF,KLEV)*P1ST5(JROF) )*PDECO5(JROF)

      P1STT5(JROF)=P1ST5(JROF)/PCOSLA5(JROF)
      PCA5(JROF)=PDSTRET*PDI5(JROF)*(ZPSLAP*PCOSL25(JROF)&
       &-ZPCLAP*PSINLA5(JROF,KLEV)*PZZ5(JROF))*P1STT5(JROF)
      PSA5(JROF)=-PYY5(JROF)*ZPCLAP*P1STT5(JROF)

      PO5(JROF,KLEV)=PRP5(JROF)*PCA5(JROF)+PRQ5(JROF)*PSA5(JROF)
      PQ5(JROF,KLEV)=PRQ5(JROF)*PCA5(JROF)-PRP5(JROF)*PSA5(JROF)
    ENDDO
  ENDIF

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARCHE5',1,ZHOOK_HANDLE)
END SUBROUTINE LARCHE5

