#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE LARCINA( &
 ! --- INPUT ---------------------------------------------------------
 & KST,KPROF,KHVI,KSTABUF,LDFINDVSEP,LDSLHD,LDSLHDQUAD,LDINTV,&
 & KTIP,KROT,LDPLANE,PLSDEPI,&
 & PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,&
 & PCOSCO,PSINCO,PSINLA,PCOPHI,PLEV,&
 & PKAPPA,PURL0,PVRL0,PWRL0,&
 ! --- INPUT/OUTPUT --------------------------------------------------
 & KVSEPC,KVSEPL,&
 ! --- OUTPUT --------------------------------------------------------
 & PLON,PLAT,PO,PQ,&
 & PUF0,PVF0,PWF0,PWFSM,&
 & KL0,KLH0,PDLO,PCLO,PCLOSLD,PDLAT,PCLA,PCLASLD,&
 & KLEV,PDVER,PVINTW,PVINTWSLD,PVINTWS,&
 & PVDERW,PHVW,KDEP)  

!**** *LARCINA  -  semi-LAgrangian scheme:(Trajectory)
!                 Research of the Coordinates (of the medium or origin
!                 point) and INterpolations.

!     Purpose.
!     --------
!       Computes the longitude and latitude of the interpolation
!       point from its cartesian coordinates.
!       Then computes the vector displacement matrix
!                        I po pq I
!                        I       I
!                        I-pq po I
!       from the interpolation point to the grid point.
!       At last determines the interpolation grid:
!       - computation of the latitude and the longitude of the
!         point situated at the upper left corner of the 16 points
!         square, and of the interpolation point.
!       - optionally, interpolates "(a/rs)*wind" field for next iteration 
!         of the algorithm to find the medium/departure point.

!**   Interface.
!     ----------
!        *CALL* *LARCINA(......)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of arrays where computations are performed.
!          KPROF   - depth of work.
!          KHVI    - 1/0: Cubic Hermite vertical interpolations are needed/not
!                         needed.
!          KSTABUF - for a latitude IGL, KSTABUF(IGL) is the
!                    address of the element corresponding to
!                    (ILON=1,IGL) in the NPROMA arrays.
!          LDFINDVSEP - compute KVSEPC, KVSEPL if .TRUE.
!          LDSLHD  - key activating SLHD weights precomputation
!          LDSLHDQUAD - key activating quadratic weights precomputation
!          LDINTV  - if .TRUE., interpolate "(a/rs)*wind" components.
!          KTIP    - 1: interpolation grid for wind used in the traj research.
!                    2: interpolation grid for origin point (RHS of eqns).
!                    3: interpolation grid for origin point, U and V only
!                       (used in adjoint of semi-Lagrangian scheme). 
!          KROT    - KROT=1: computation of the elements po and pq
!                    of the wind displacement matrix.
!                    KROT=0: no computation.
!          LDPLANE - switch: .T. = plane geometry; .F. = spherical geometry.
!          PLSDEPI - (Number of points by latitude) / (2 * PI) .
!          PCOLON  - cosine of the grid-point longitude on the
!                    computational sphere.
!          PSILON  - sine of the grid-point longitude on the
!                    computational sphere.
!          PGEMU   - sine of the grid-point geographical latitude.
!          PGSQM2  - cosine of the grid-point geographical latitude.
!          PGECLO  - cosine of the grid-point geographical longitude.
!          PGESLO  - sine of the grid-point geographical longitude.
!          PCOSCO  - cos(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude).
!                    ! if LDPLANE: x - coordinate (fractional system).
!          PSINCO  - sin(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude).
!                    ! if LDPLANE: y - coordinate (fractional system).
!          PSINLA  - sine of the interpolation point geographical latitude.
!          PCOPHI  - cosine of the geographical angle between the
!                    interpolation point and the grid-point.
!          PLEV    - vertical coordinate of the interpolation point.
!          PKAPPA  - kappa function ("coefficient of SLHD") based on the
!                    rescaled horizontal deformation of the flow evaluated
!                    at instant "t" for the final point F
!          PURL0   - U-component of "(a/rs)*wind".
!          PVRL0   - V-component of "(a/rs)*wind".
!          PWRL0   - "etadot".

!        INPUT/OUTPUT:
!          KVSEPC  - vertical separation (used in S/L adjoint, cubic interp.)
!          KVSEPL  - vertical separation (used in S/L adjoint, linear interp.)

!        OUTPUT:
!          PLON      - computational sphere longitude of interpolation point.
!          PLAT      - computational sphere latitude of interpolation point.
!          PO        - first element of the wind displacement matrix.
!          PQ        - second element of the wind displacement matrix.
!          PUF0      - Interpolated U-"(a/rs)*wind".
!          PVF0      - Interpolated V-"(a/rs)*wind".
!          PWF0      - Interpolated "etadot".
!          PWFSM     - Smoothly interpolated "etadot".
!          KL0       - index of the four western points
!                      of the 16 points interpolation grid.
!          KLH0      - second value of index of the four western points
!                      of the 16 points interpolation grid if needed.
!          PDLO      - distances for horizontal linear interpolations
!                      in longitude (latitude rows 0, 1, 2, 3)
!          PCLO      - weights for horizontal cubic interpolations in
!                      longitude (latitude rows 1, 2)
!          PCLOSLD   - weights for horizontal cubic interpolations in
!                      longitude, SLHD case (latitude rows 1, 2)
!          PDLAT     - distance for horizontal linear interpolations
!                      in latitude
!          PCLA      - weights for horizontal cubic interpolations
!                      in latitude
!          PCLASLD   - weights for horizontal cubic interpolations
!                      in latitude, SLHD case
!          KLEV      - lower level of the vertical interpolation
!                      grid needed for vertical interpolations.
!          PDVER     - distance for vertical linear interpolation
!          PVINTW    - vertical cubic interpolation weights
!          PVINTWSLD - vertical cubic interpolation weights, SLHD case
!          PVINTWS   - Vertical spline interpolation weights.
!          PVDERW    - weights to compute vertical derivatives necessary for
!                      Hermite cubic vertical interpolation.
!          PHVW      - Hermite vertical cubic interpolation weights.
!          KDEP      - indication of the interpolation stencil latitudial
!                      dependences (used for LVECADIN=.T. option in adjoint)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        Called by LARMES, ELARMES, LARMES5, ELARMES5, LAPINEA and LAPINEA5.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD, after the subroutine LAGINT0 written by Maurice IMBARD
!        Alain CRAPLET and Michel ROCHAS  METEO FRANCE/EERM/CRMD

!     Modifications.
!     --------------
!        Original : JUNE 1991.
!        Modifications:
!        K. YESSAD (METEO-FRANCE CNRM/GMAP): 21/11/1991.
!        K. YESSAD: February 1992 (optimisation, consistency with IO scheme).
!        M. HORTAL (ECMWF) and K. YESSAD (CNRM/GMAP): 04/11/1992.
!               (possible use of an irregularly spaced vertical coordinate).
!        K. YESSAD: December 1992 (non interpolating vertical scheme).
!        Modified 93-12-14 by K. YESSAD: interpolations at medium point for
!                                      semi-lagrangian temperature equation.
!        Modified 93-12-21 by C. Temperton: interpolations at medium point for
!                                      semi-lagrangian moisture equation.
!        Modified 93-02-15 by K. YESSAD: replacing switch KWIS by KWISA and
!                                      KXLAG.
!        Modified 94-03-23 by M. Hortal: interpolations at medium point for
!                                      semi-lagrangian momentum equations
!        Modified 94-09-30 by M. Hortal: Quasi-monotone interpolations
!        Modified 94-10-17 by M. Tolstykh: CUD-5 interpolation
!        Modified JAN 1995 by M. Tolstykh and K. YESSAD:
!               - CUD-5 interpolation for vertical interpolating scheme.
!               - CUD-5 interpolation for liquid water, ice and cloudiness.
!               - CUD-5 interpolation for options NVLAG=3, NWLAG=3, NTLAG=3.
!               - Merge of routines LAICDN and LAICDSW into LAICDD for
!                 horizontal CUD-5 interpolations.
!               - Harmonisation of conventions (notations, numbering of
!                 latitudes) in CUD-5 interpolations with 12 or 32 points
!                 interpolations routines.
!               - Introduction of array PCUDWM for optimisations of CUD-5
!                 meridian derivatives.
!               - Simplification of treatment of NVLAG=2 formulation of
!                 2D shallow-water continuity equation.
!               - Remove of useless quantity KBFLDOP.
!        Modified 95-01-17 by R. Bubnova:
!               - Plane geometry after developments of M. Janousek.
!        Modified by K. YESSAD (AUGUST 1995):
!               - Accurate two-time level SL scheme (after Tolstykh/Janousek).
!               - Second-order decentering in two-time level SL scheme.
!               - DO loops updated according to F90 norms.
!        Modified 95-10-01 by MPP Group *ECMWF* Distributed memory version
!        K. YESSAD (MARCH 1996):
!             - ZLON, ZLAT become dummy arguments PLON, PLAT.
!        Modified 96-06-03 by M. Janousek and R. Bubnova:
!               - non-hydrostatic dynamics and LCORAN option in plane geom.
!        Modified 96-03 by A. Untch : grid-point prognostic ozone
!        Modified 96-04-17 by C. Temperton:
!               - Refined treatment of Coriolis term in 2TL scheme
!        Modified by K. YESSAD (AUGUST 1996):
!               - Hermite cubic vertical interpolations for passive scalars.
!                 Not yet coded for plane geometry.
!                 Not yet coded for CUD-5 scheme.
!        Modified 97-05-14 by C. Temperton:
!               - Options NVLAG=3 and NWLAG=3 added to SWE model
!        Modified 97-05-19 by C. Temperton:
!               - Strange quasi-monotone options removed from SWE model
!               - and add switch for linear/cubic interp. to find trajectory.
!        Modified 98-01-19 by C. Temperton:
!                 2D (SWE) options removed and transferred to LARCIN2
!        Modified 98-09-01 by N. Wedi:
!                 split physics in semi-lagrangian mode
!        Modified 98-03-20 by K. YESSAD: bug corrections.
!        Modified 98-05-07 by K. YESSAD: adaptation of L2TLFF to LAGPHY=.F..
!        Modified July 1998 by K. YESSAD: cleaning
!               - removal of options NODEC=2, L2TLAW=T, L2TLAT=T
!               - complete declaration of dummy arguments.
!               - correction of false comments.
!               - improvement of code readability (indentation and numbering
!                 of comments for ex.).
!        Modified 98-05-27 by E. Holm: 
!               - Changed order of ozone in call and inside routine.
!        Modified 99-11-16 by C. Temperton: added KWISA(2)=4 option.
!        Modified 99-03-31 by P. SMOLIKOVA: option LRDAORG (double averaging).
!        Modified 99-09-21 by D.SALMOND: On Demand Semi-Lagrangian Comms
!        Modified 00-02-07 by C. Temperton: removed redundant use of PX0
!                                           when NVLAG=3.
!        Modified 00-02-11 by C. Temperton: added computation of KVSEP
!        Modified 00-06-15 by C. Temperton: KVSEP -> KVSEPC,KVSEPL.
!                                           (and logic corrected!)
!        Modified NOV 1999 by K. YESSAD:
!         - in 'lacudw', compute PDLAT and ILA as in 'lascaw' =>
!           'krlatx', 'kindlat' and 'pdllat' become useless.
!        K. YESSAD (Nov 1999): cleaning
!         - recode SL3TL like SL2TL and eliminate useless interpolations
!           in continuity equation.
!        Modified 99-12-22 by C. Fischer: clean out call to elacudw
!        Modified April 2000 by Malgorzata Szczech:
!         - bug corrections for thermodynamical ozon.
!         - add option "lhvo3" for thermodynamical ozon.
!        Modified August 2000 by Deborah Salmond to split into _traj and _int
!        Modified 00-11-07 by C. Temperton: streamlining.
!        Modified 00-11-23 by C. Temperton: renamed _TRAJ and _INT routines.
!        Modified 01-06-26 by K. YESSAD: cleanings (remove useless variables),
!         update or correct some comments, especially in the context of the
!         introduction of thin layer hypothesis relaxation ("(a/rs)*winds"
!         are interpolated and no longer winds if lvercor=true).
!         Restore options with Hermite vertical interpolations.
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        Modified 03-05-01 by A. Bogatchev: add arg in call elascaw
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hortal      01-Dec-2003 Vertical trajectory smoothing
!        Modified 05-01-11 by C. Temperton: streamlining.
!        F. Vana   2004-Nov: new arguments PVINTDS for (E)LASCAW
!        F. Vana 08-Jan-2007 new argument KDEP for LAM adjoint
!        F. Vana 28-Aug-2007 cleaning of PVINTDS for (E)LASCAW
!  30-Jun-2008 J. Masek   Dataflow for new SLHD interpolators.
!  K. Yessad Nov 2008: rationalisation of dummy argument interfaces
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCVER  , ONLY : LVSPLIP  ,LSVTSM
USE YOMCST   , ONLY : RPI
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NFLSA    ,NFLEN    ,NDGSAH   ,&
 & NDGENH   ,NDLUNG   ,NDLUXG   ,NDGUNG   ,NDGUXG   ,NRLEVX
USE YOMLEG   , ONLY : RLATI    ,RIPI0    ,RIPI1    ,RIPI2    ,RSLD1    ,&
 & RSLD2    ,RSLD3    ,RSLDW
USE YOMGEM   , ONLY : VETAH    ,VETAF    ,VCUICO   ,VSLD     ,VSLDW    ,&
 & NVAUTF   ,NSTTYP   ,RMUCEN   ,RLOCEN   ,RSTRET   ,R4JP     ,RC2P1    ,&
 & RC2M1    ,VRLEVX
USE YOMMP    , ONLY : NASLB1

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KHVI 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTABUF(NDGSAH:NDGENH) 
LOGICAL           ,INTENT(IN)    :: LDFINDVSEP 
LOGICAL           ,INTENT(IN)    :: LDSLHD
LOGICAL           ,INTENT(IN)    :: LDSLHDQUAD
LOGICAL           ,INTENT(IN)    :: LDINTV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTIP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KROT 
LOGICAL           ,INTENT(IN)    :: LDPLANE 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSDEPI(NDGSAH:NDGENH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOSCO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINCO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINLA(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOPHI(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLEV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAPPA(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PURL0(NASLB1,NFLSA:NFLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVRL0(NASLB1,NFLSA:NFLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWRL0(NASLB1,NFLSA:NFLEN) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KVSEPC 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KVSEPL 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLON(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLAT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWFSM(NPROMA,NFLEVG)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KL0(NPROMA,NFLEVG,0:3) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLH0(NPROMA,NFLEVG,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLO(NPROMA,NFLEVG,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLO(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLOSLD(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLAT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLA(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLASLD(NPROMA,NFLEVG,3)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLEV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDVER(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTW(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTWSLD(NPROMA,NFLEVG,3)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTWS(NPROMA,NFLEVG,1:4) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVDERW(NPROMA,NFLEVG,2*KHVI,2*KHVI) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHVW(NPROMA,NFLEVG,4*KHVI) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KDEP(NPROMA,NFLEVG)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: ISEP(NPROMA)
INTEGER(KIND=JPIM) :: IHOR, IMINSEP, IVSEP, IWIS, JLEV, JROF
LOGICAL :: LLDONE
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZDSTRET,ZDEPI,ZPIS2

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "elarche.intfb.h"
#include "elascaw.intfb.h"
#include "laitli.intfb.h"
#include "larche.intfb.h"
#include "lascaw.intfb.h"
#include "laismoo.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARCINA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY INITIALISATIONS.
!              ----------------------------

ZDSTRET=2.0_JPRB*RSTRET
ZDEPI=2.0_JPRB*RPI
ZPIS2=0.5_JPRB*RPI

! * Input variable IWIS for LASCAW.
IF (KTIP == 1) THEN
! * trajectory research.
  IWIS=101
  IHOR=1
ELSEIF (KTIP == 2) THEN
! * origin point interpolations.
  IWIS=103+KHVI
  IF (LVSPLIP) IWIS=105
  IHOR=1
ELSEIF (KTIP == 3) THEN
  IWIS=103
  IHOR=0    ! Used by adjoint - I wish I could remember why ! CT
ENDIF

!     ------------------------------------------------------------------

!*       2.    COMPUTATION OF LAT LON OF THE INTERPOLATION POINT.
!              IF KROT=1 COMPUTATION OF THE WIND DISPLACEMENT MATRIX
!              FROM THE INTERPOLATION POINT TO THE FINAL POINT.
!              ( T FOR LATITUDE THETA, L FOR LONGITUDE LAMBDA).
!              PO = ( 1 / (1+cos(PHI)) )
!                  *( cos(TG)*cos(T) + (1+sin(TG)*sin(T))*cos(L-LG) )
!              PQ = (-1 / (1+cos(PHI)) )
!                  *( sin(TG)+sin(T) ) * sin(L-LG)
!     ------------------------------------------------------------------

IF (LDPLANE) THEN

  IF (KROT == 1) THEN
    CALL ELARCHE(NPROMA,KST,KPROF,NFLEVG,&
     & PGEMU,PGSQM2,PGECLO,PGESLO,&
     & PCOSCO,PSINCO,PO,PQ,PLON,PLAT)  
  ENDIF

  CALL ELASCAW( &
   ! --- INPUT ----------------------------------------------------------------
   & NASLB1,NPROMA,KST,KPROF,1,NFLEVG,NDGSAH,NDGENH,&
   & NDLUNG,NDLUXG,NDGUNG,NDGUXG,&
   & NFLSA,NFLEN,KSTABUF,IWIS,IHOR,KHVI,&
   & LDSLHD,LDSLHDQUAD,&
   & PCOSCO,PSINCO,PLEV,&
   & VETAF,VETAH,NVAUTF,&
   & VCUICO,VSLD,VSLDW,NRLEVX,VRLEVX,PKAPPA,&
   ! --- OUTPUT ---------------------------------------------------------------
   & PDLAT,PCLA(1,1,1),PCLASLD(1,1,1),&
   & PDLO(1,1,0),PCLO(1,1,1,1),PCLOSLD(1,1,1,1),&
   & KL0(1,1,0),KLH0(1,1,0),KLEV,&
   & PDVER,PVINTW,PVINTWSLD,PVINTWS,&
   & PVDERW,PHVW,KDEP)

ELSE

  CALL LARCHE(NPROMA,KST,KPROF,NFLEVG,&
   & NSTTYP,ZDSTRET,RC2M1,RC2P1,RPI,ZDEPI,&
   & RLOCEN,RMUCEN,PCOLON,PSILON,&
   & PGEMU,PGSQM2,PGECLO,PGESLO,&
   & PCOSCO,PSINCO,PSINLA,PCOPHI,&
   & KROT,PLON,PLAT,PO,PQ)  

  CALL LASCAW(NASLB1,NPROMA,KST,KPROF,1,NFLEVG,NDGSAH,NDGENH,&
   & NFLSA,KSTABUF,IWIS,IHOR,KHVI, &
   & LDSLHD,LDSLHDQUAD,R4JP,ZPIS2,PLSDEPI,RLATI(NDGSAH), &
   & RIPI0(NDGSAH),RIPI1(NDGSAH),RIPI2(NDGSAH), &
   & RSLD1(NDGSAH),RSLD2(NDGSAH),RSLD3(NDGSAH),RSLDW(1,1,NDGSAH),&
   & PLON,PLAT,PLEV, &
   & VETAF,VETAH,NVAUTF, &
   & VCUICO,VSLD,VSLDW,NRLEVX,VRLEVX,PKAPPA, &
   & PDLAT,PCLA(1,1,1),PCLASLD(1,1,1), &
   & PDLO(1,1,0),PCLO(1,1,1,1),PCLOSLD(1,1,1,1), &
   & KL0(1,1,0),KLH0(1,1,0),KLEV, &
   & PDVER,PVINTW,PVINTWSLD,PVINTWS, &
   & PVDERW,PHVW)

ENDIF

IF (LDFINDVSEP) THEN

  IVSEP=KVSEPC-1
  LLDONE=.FALSE.
  DO WHILE (.NOT.LLDONE)
    IVSEP=IVSEP+1
    DO JROF=KST,KPROF
      ISEP(JROF)=KLEV(JROF,IVSEP+1)-KLEV(JROF,1)
    ENDDO
    DO JLEV=2,NFLEVG-IVSEP
      DO JROF=KST,KPROF
        ISEP(JROF)=MIN(ISEP(JROF),(KLEV(JROF,JLEV+IVSEP)-KLEV(JROF,JLEV)))
      ENDDO
    ENDDO
    IMINSEP=NFLEVG
    DO JROF=KST,KPROF
      IMINSEP=MIN(IMINSEP,ISEP(JROF))
    ENDDO
    LLDONE = IMINSEP >= 4 .OR. IVSEP == NFLEVG-1
  ENDDO
  IF (IMINSEP < 4) THEN
    CALL ABOR1('LARCINA: UNABLE TO COMPUTE NVSEPC')
  ENDIF
  KVSEPC=MAX(KVSEPC,IVSEP)

  IVSEP=KVSEPL-1
  LLDONE=.FALSE.
  DO WHILE (.NOT.LLDONE)
    IVSEP=IVSEP+1
    DO JROF=KST,KPROF
      ISEP(JROF)=KLEV(JROF,IVSEP+1)-KLEV(JROF,1)
    ENDDO
    DO JLEV=2,NFLEVG-IVSEP
      DO JROF=KST,KPROF
        ISEP(JROF)=MIN(ISEP(JROF),(KLEV(JROF,JLEV+IVSEP)-KLEV(JROF,JLEV)))
      ENDDO
    ENDDO
    IMINSEP=NFLEVG
    DO JROF=KST,KPROF
      IMINSEP=MIN(IMINSEP,ISEP(JROF))
    ENDDO
    LLDONE = IMINSEP >= 2 .OR. IVSEP == NFLEVG-1
  ENDDO
  IF (IMINSEP < 2) THEN
    CALL ABOR1('LARCINA: UNABLE TO COMPUTE NVSEPL')
  ENDIF
  KVSEPL=MAX(KVSEPL,IVSEP)

ENDIF

!     ------------------------------------------------------------------

!*       3.    INTERPOLATIONS OF ((a/rs)*U;(a/rs)*V;etadot)
!              FOR TRAJECTORY RESEARCH.
!              --------------------------------------------

!*      Trilinear interpolations.

IF (LDINTV) THEN
  CALL LAITLI(NASLB1,NPROMA,KST,KPROF,NFLEVG,NFLSA,NFLEN,&
   & PDLAT,PDLO(1,1,1),KL0(1,1,1),PDVER,PURL0,PUF0)  
  CALL LAITLI(NASLB1,NPROMA,KST,KPROF,NFLEVG,NFLSA,NFLEN,&
   & PDLAT,PDLO(1,1,1),KL0(1,1,1),PDVER,PVRL0,PVF0)  
  CALL LAITLI(NASLB1,NPROMA,KST,KPROF,NFLEVG,NFLSA,NFLEN,&
   & PDLAT,PDLO(1,1,1),KL0(1,1,1),PDVER,PWRL0,PWF0)  
  IF(LSVTSM) THEN
    CALL LAISMOO(NASLB1,NPROMA,KST,KPROF,NFLEVG,NFLSA,NFLEN,&
     & PDLAT,PDLO,KL0,PDVER,PWRL0,PWFSM)  
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARCINA',1,ZHOOK_HANDLE)
END SUBROUTINE LARCINA
