#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE LARCINATL( &
 ! --- INPUT ---------------------------------------------------------
 & KST,KPROF,KSTABUF,LDSLHD,LDSLHDQUAD,KTIP,KROT,LDPLANE,PLSDEPI,&
 & PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,&
 & PCOSCO,PSINCO,PSINLA,PCOPHI,PLEV,&
 & PCOSCO5,PSINCO5,PSINLA5,PCOPHI5,PLEV5,&
 & PKAPPA,PKAPPA5,PURL0,PVRL0,PWRL0,PURL05,PVRL05,PWRL05,&
 ! --- OUTPUT --------------------------------------------------------
 & PLON,PLAT,PO,PQ,&
 & PUF0,PVF0,PWF0,&
 & PLON5,PLAT5,PO5,PQ5,&
 & KL0,KLH0,PDLO,PCLO,PCLOSLD,PDLAT,PCLA,PCLASLD,&
 & KLEV,PDVER,PVINTW,PVINTWSLD,&
 & PDLO5,PCLO5,PCLOSLD5,PDLAT5,PCLA5,PCLASLD5,PDVER5,PVINTW5,PVINTWSLD5)  

!**** *LARCINATL - semi-LAgrangian scheme:(Trajectory)
!                                        (tangent-linear version)
!                 Research of the Coordinates (of the medium or origin
!                 point) and INterpolations.

!     Purpose.
!     --------
!       Computes the longitude and latitude of the interpolation
!       point from its cartesian coordinates.
!       Then computes the vector displacement matrix
!                        I po pq I
!                        I       I
!                        I-pq po I
!       from the interpolation point to the grid point.
!       At last determines the interpolation grid:
!       - computation of the latitude and the longitude of the
!         point situated at the upper left corner of the 16 points
!         square, and of the interpolation point.
!       - optionally, interpolates wind field for next iteration
!         of the algorithm to find the medium/departure point.

!**   Interface.
!     ----------
!        *CALL* *LARCINATL(......)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of arrays where computations are performed.
!          KPROF   - depth of work.
!          KSTABUF - for a latitude IGL, KSTABUF(IGL) is the
!                    address of the element corresponding to
!                    (ILON=1,IGL) in the NPROMA arrays.
!          LDSLHD  - key activating SLHD weights precomputation
!          LDSLHDQUAD - key activating quadratic weights precomputation
!          KTIP    - KTIP=1: interpolations for wind to find trajectory.
!                    KTIP=3: (E)LASCAWTL for origin point.
!          KROT    - KROT=1: computation of the elements po and pq
!                    of the wind displacement matrix.
!                    KROT=0: no computation.
!          LDPLANE - switch: .T. = plane geometry; .F. = spherical geometry.
!          PLSDEPI - (Number of points by latitude) / (2 * PI) .
!          PCOLON  - cosine of the grid-point longitude on the
!                    computational sphere.
!          PSILON  - sine of the grid-point longitude on the
!                    computational sphere.
!          PGEMU   - sine of the grid-point geographical latitude.
!          PGSQM2  - cosine of the grid-point geographical latitude.
!          PGECLO  - cosine of the grid-point geographical longitude.
!          PGESLO  - sine of the grid-point geographical longitude.
!          PCOSCO  - cos(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude).
!                    ! if LDPLANE: x - coordinate (fractional system).
!          PSINCO  - sin(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude).
!                    ! if LDPLANE: y - coordinate (fractional system).
!          PSINLA  - sine of the interpolation point geographical latitude.
!          PCOPHI  - cosine of the geographical angle between the
!                    interpolation point and the grid-point.
!          PLEV    - vertical coordinate of the interpolation point.
!          PCOSCO5 - cf. PCOSCO (trajectory)
!          PSINCO5 - cf. PSINCO (trajectory)
!          PSINLA5 - cf. PSINLA (trajectory)
!          PCOPHI5 - cf. PCOPHI (trajectory)
!          PLEV5   - cf. PLEV (trajectory)
!          PKAPPA  - kappa function ("coefficient of SLHD") based on the
!                    rescaled horizontal deformation of the flow evaluated
!                    at instant "t" for the final point F
!          PKAPPA5 - cf. PKAPPA (trajectory)
!          PURL0   - U-component of the wind.
!          PVRL0   - V-component of the wind.
!          PWRL0   - "etadot"-component of the wind.
!          PURL05  - cf. PURL0 (trajectory)
!          PVRL05  - cf. PVRL0 (trajectory)
!          PWRL05  - cf. PWRL0 (trajectory)

!        OUTPUT:
!          PLON    - computational sphere longitude of interpolation point.
!          PLAT    - computational sphere latitude of interpolation point.
!          PO      - first element of the wind displacement matrix.
!          PQ      - second element of the wind displacement matrix.
!          PUF0    - Interpolated U-wind at the medium point.
!          PVF0    - Interpolated V-wind at the medium point.
!          PWF0    - Interpolated "etadot"-wind at the medium point.
!          PLON5   - cf. PLON (trajectory)
!          PLAT5   - cf. PLAT (trajectory)
!          PO5     - cf. PO (trajectory)
!          PQ5     - cf. PQ (trajectory)
!          KL0     - index of the four western points
!                    of the 16 points interpolation grid.
!          KLH0    - second value of index of the four western points
!                    of the 16 points interpolation grid if needed.
!          PDLO    - distances for horizontal linear interpolations
!                    in longitude (latitude rows 0, 1, 2, 3)
!          PCLO    - weights for horizontal cubic interpolations in
!                    longitude (latitude rows 1, 2)
!          PCLOSLD - weights for horizontal cubic interpolations in
!                    longitude, SLHD case (latitude rows 1, 2)
!          PDLAT   - distance for horizontal linear interpolations
!                    in latitude
!          PCLA    - weights for horizontal cubic interpolations
!                    in latitude
!          PCLASLD - weights for horizontal cubic interpolations
!                    in latitude, SLHD case
!          KLEV    - lower level of the vertical interpolation
!                    grid needed for vertical interpolations.
!          PDVER   - distances for vertical linear interpolation
!                    on a same vertical.
!          PVINTW  - Vertical cubic interpolation weights.
!          PVINTWSLD - vertical cubic interpolation weights, SLHD case
!          PDLO5   - cf. PDLO (trajectory)
!          PCLO5   - cf. PCLO (trajectory)
!          PCLOSLD5- cf. PCLOSLD (trajectory)
!          PDLAT5  - cf. PDLAT (trajectory)
!          PCLA5   - cf. PCLA (trajectory)
!          PCLASLD5- cf. PCLASLD (trajectory)
!          PDVER5  - cf. PDVER (trajectory)
!          PVINTW5 - cf. PVINTW (trajectory)
!          PVINTWSLD5- cf. PVINTWSLD (trajectory)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        Called by LARMESTL, ELARMESTL and LAPINEATL.

!     Reference.
!     ----------

!     Author.
!     -------
!        C. Temperton (ECMWF)

!     Modifications.
!     --------------
!        Original: 99/07/12
!        Modified 00-02-07 by C. Temperton: removed redundant use of PX0
!                                           when NVLAG=3.
!        Modified 00-03-01 by C. Temperton: precompute functions for    
!                                           cubic interpolation.
!        Modified August 2000 by Deborah Salmond to split into _traj and _int
!        Modified 00-11-09 by C. Temperton: streamlining.    
!        Modified 00-11-23 by C. Temperton: renamed _TRAJ and _INT routines.
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified 05-01-11 by C. Temperton: streamlining.    
!        Modified 06-06-13 by F. Vana: LDPLANE=.T.
!        Modified 08-Jun-27 by F. Vana: weights driven interpolation
!        K. Yessad Nov 2008: rationalisation of dummy argument interfaces
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST    ,ONLY : RPI
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NFLSA    ,NFLEN    ,NDGSAH   ,&
 & NDGENH   ,NDLUNG   ,NDLUXG   ,NDGUXG   ,NRLEVX
USE YOMLEG   , ONLY : RLATI    ,RIPI0    ,RIPI1    ,RIPI2    ,RSLD1    ,&
 & RSLD2    ,RSLD3    ,RSLDW
USE YOMGEM   , ONLY : VETAF    ,VCUICO   ,VSLD     ,VSLDW    ,&
 & NVAUTF   ,NSTTYP   ,RMUCEN   ,RLOCEN   ,RSTRET   ,R4JP     ,RC2P1    ,&
 & RC2M1    ,VRLEVX
USE YOMMP    , ONLY : NASLB1
USE YOMDYNA  , ONLY : LSLHD

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTABUF(NDGSAH:NDGENH) 
LOGICAL           ,INTENT(IN)    :: LDSLHD
LOGICAL           ,INTENT(IN)    :: LDSLHDQUAD
INTEGER(KIND=JPIM),INTENT(IN)    :: KTIP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KROT 
LOGICAL           ,INTENT(IN)    :: LDPLANE 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSDEPI(NDGSAH:NDGENH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOSCO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINCO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINLA(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOPHI(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLEV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOSCO5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINCO5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINLA5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOPHI5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLEV5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAPPA(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAPPA5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PURL0(NASLB1,NFLSA:NFLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVRL0(NASLB1,NFLSA:NFLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWRL0(NASLB1,NFLSA:NFLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PURL05(NASLB1,NFLSA:NFLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVRL05(NASLB1,NFLSA:NFLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWRL05(NASLB1,NFLSA:NFLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLON(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLAT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWF0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLON5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLAT5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PO5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ5(NPROMA,NFLEVG) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KL0(NPROMA,NFLEVG,0:3) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLH0(NPROMA,NFLEVG,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLO(NPROMA,NFLEVG,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLO(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLOSLD(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLAT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLA(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLASLD(NPROMA,NFLEVG,3) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLEV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDVER(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTW(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTWSLD(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLO5(NPROMA,NFLEVG,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLO5(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLOSLD5(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLAT5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLA5(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLASLD5(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDVER5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTW5(NPROMA,NFLEVG,3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTWSLD5(NPROMA,NFLEVG,3) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IHOR, IWIS
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZDSTRET, ZDEPI, ZPIS2
REAL(KIND=JPRB) :: ZDUM5(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZUN_KAPPA(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZUN_KAPPA5(NPROMA,NFLEVG)

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "laitlitl.intfb.h"
#include "elarchetl.intfb.h"
#include "larchetl.intfb.h"
#include "elascawtl.intfb.h"
#include "lascawtl.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARCINATL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY INITIALISATIONS.
!              ----------------------------

ZDSTRET=2.0_JPRB*RSTRET
ZDEPI=2.0_JPRB*RPI
ZPIS2=0.5_JPRB*RPI
IHOR=1

! * Input variable IWIS for LASCAW.
IF (KTIP == 1) THEN
  ! * trajectory research.
  IWIS=101
ELSEIF (KTIP == 3) THEN
  ! * origin point interpolations.
  IWIS=103
ELSE
  IWIS=-999
  CALL ABOR1('LARCINATL: WRONG VALUE FOR IWIS')
ENDIF

!     ------------------------------------------------------------------

!*       2.    COMPUTATION OF LAT LON OF THE INTERPOLATION POINT.
!              IF KROT=1 COMPUTATION OF THE WIND DISPLACEMENT MATRIX
!              FROM THE INTERPOLATION POINT TO THE FINAL POINT.
!              ( T FOR LATITUDE THETA, L FOR LONGITUDE LAMBDA).
!              PO = ( 1 / (1+cos(PHI)) )
!                  *( cos(TG)*cos(T) + (1+sin(TG)*sin(T))*cos(L-LG) )
!              PQ = (-1 / (1+cos(PHI)) )
!                  *( sin(TG)+sin(T) ) * sin(L-LG)
!     ------------------------------------------------------------------

IF (LDPLANE) THEN

  IF (KROT == 1) THEN
    CALL ELARCHETL(NPROMA,KST,KPROF,NFLEVG,&
     & PGEMU,PGSQM2,PGECLO,PGESLO,&
     & PCOSCO,PSINCO,PO,PQ,PLON,PLAT,&
     & PCOSCO5,PSINCO5,PO5,PQ5,PLON5,PLAT5)
  ENDIF

  ! Unlike in global model, here we don't need (yet) to use
  ! the trick used for LASCAWTL.
  CALL ELASCAWTL(NASLB1,NPROMA,KST,KPROF,NFLEVG,NDGSAH,NDGENH,&
     & NDLUNG,NDLUXG,NDGUXG,&
     & NFLSA,KSTABUF,IWIS,IHOR,&
     & LDSLHD,LDSLHDQUAD,&
     & PCOSCO,PSINCO,PLEV,&
     & PCOSCO5,PSINCO5,PLEV5,&
     & VETAF,NVAUTF,&
     & VCUICO,VSLD,VSLDW,NRLEVX,VRLEVX,PKAPPA,PKAPPA5,&
     & PDLAT,PCLA,PCLASLD,PDLO,PCLO,PCLOSLD,&
     & KL0,KLH0,KLEV,PDVER,PVINTW,PVINTWSLD,&
     & PDLAT5,PCLA5,PCLASLD5,PDLO5,PCLO5,PCLOSLD5,&
     & PDVER5,PVINTW5,PVINTWSLD5)

ELSE

  CALL LARCHETL(NPROMA,KST,KPROF,NFLEVG,&
   & NSTTYP,ZDSTRET,RC2M1,RC2P1,RPI,ZDEPI,&
   & RLOCEN,RMUCEN,PCOLON,PSILON,&
   & PGEMU,PGSQM2,PGECLO,PGESLO,&
   & PCOSCO,PSINCO,PSINLA,PCOPHI,&
   & PCOSCO5,PSINCO5,PSINLA5,PCOPHI5,&
   & KROT,PLON,PLAT,PO,PQ,&
   & PLON5,PLAT5,PO5,PQ5)  

  ! This strange duplication has the only meaning to avoid
  ! memory problems when PKAPPA and PKAPPA5 are not defined.
  ! Seems that some F90 compilers has problem with such
  ! construct...
  IF (LSLHD) THEN
    CALL LASCAWTL(NASLB1,NPROMA,KST,KPROF,NFLEVG,NDGSAH,NDGENH,&
     & NFLSA,KSTABUF,IWIS,IHOR,&
     & LDSLHD,LDSLHDQUAD,&
     & R4JP,ZPIS2,PLSDEPI,RLATI(NDGSAH),&
     & RIPI0(NDGSAH),RIPI1(NDGSAH),RIPI2(NDGSAH),&
     & RSLD1(NDGSAH),RSLD2(NDGSAH),RSLD3(NDGSAH),RSLDW(1,1,NDGSAH),&
     & PLON,PLAT,PLEV,&
     & PLON5,PLAT5,PLEV5,&
     & VETAF,NVAUTF,&
     & VCUICO,VSLD,VSLDW,NRLEVX,VRLEVX,PKAPPA,PKAPPA5,&
     & PDLAT,PCLA,PCLASLD,PDLO,PCLO,PCLOSLD,KL0,KLH0,KLEV,&
     & PDVER,PVINTW,PVINTWSLD,&
     & PDLAT5,PCLA5,PCLASLD5,PDLO5,PCLO5,PCLOSLD5,&
     & PDVER5,PVINTW5,PVINTWSLD5 )
  ELSE
    CALL LASCAWTL(NASLB1,NPROMA,KST,KPROF,NFLEVG,NDGSAH,NDGENH,&
     & NFLSA,KSTABUF,IWIS,IHOR,&
     & LDSLHD,LDSLHDQUAD,&
     & R4JP,ZPIS2,PLSDEPI,RLATI(NDGSAH),&
     & RIPI0(NDGSAH),RIPI1(NDGSAH),RIPI2(NDGSAH),&
     & RSLD1(NDGSAH),RSLD2(NDGSAH),RSLD3(NDGSAH),RSLDW(1,1,NDGSAH),&
     & PLON,PLAT,PLEV,&
     & PLON5,PLAT5,PLEV5,&
     & VETAF,NVAUTF,&
     & VCUICO,VSLD,VSLDW,NRLEVX,VRLEVX,ZUN_KAPPA,ZUN_KAPPA5,&
     & PDLAT,PCLA,PCLASLD,PDLO,PCLO,PCLOSLD,KL0,KLH0,KLEV,&
     & PDVER,PVINTW,PVINTWSLD,&
     & PDLAT5,PCLA5,PCLASLD5,PDLO5,PCLO5,PCLOSLD5,&
     & PDVER5,PVINTW5,PVINTWSLD5 )
  ENDIF

ENDIF

!     ------------------------------------------------------------------

!*       3.    INTERPOLATIONS OF WIND FOR TRAJECTORY RESEARCH.
!              -----------------------------------------------

IF (KTIP == 1) THEN

  !*      Trilinear interpolations.

  CALL LAITLITL(NASLB1,NPROMA,KST,KPROF,NFLEVG,NFLSA,NFLEN,&
   & PDLAT,PDLO(1,1,1),KL0(1,1,1),PDVER,PDLAT5,PDLO5(1,1,1),PDVER5,&
   & PURL0,PURL05,PUF0,ZDUM5)  
  CALL LAITLITL(NASLB1,NPROMA,KST,KPROF,NFLEVG,NFLSA,NFLEN,&
   & PDLAT,PDLO(1,1,1),KL0(1,1,1),PDVER,PDLAT5,PDLO5(1,1,1),PDVER5,&
   & PVRL0,PVRL05,PVF0,ZDUM5)  
  CALL LAITLITL(NASLB1,NPROMA,KST,KPROF,NFLEVG,NFLSA,NFLEN,&
   & PDLAT,PDLO(1,1,1),KL0(1,1,1),PDVER,PDLAT5,PDLO5(1,1,1),PDVER5,&
   & PWRL0,PWRL05,PWF0,ZDUM5)  

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARCINATL',1,ZHOOK_HANDLE)
END SUBROUTINE LARCINATL
