SUBROUTINE LARCINHA( &
 ! --- INPUT ---------------------------------------------------------
 & KST,KPROF,KHVI,KSTABUF,KTIP,KROT,PLSDEPI,&
 & PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,&
 & PCOSCO,PSINCO,PSINLA,PCOPHI,PLEV,PKAPPA,&
 ! --- OUTPUT --------------------------------------------------------
 & KL0H,PDLOH,PCLOH,PCLOHSLD,&
 & PDLATH,PCLAH,PCLAHSLD,PDVERH,PVINTWH,PVINTWHSLD)  

!**** *LARCINHA  -  semi-LAgrangian scheme:(Trajectory)
!                 Research of the Coordinates (of the medium or origin
!                 point) and INterpolations.

!     Purpose.
!     --------
!       Computes the longitude and latitude of the interpolation
!       point from its cartesian coordinates.
!       Then computes the vector displacement matrix
!                        I po pq I
!                        I       I
!                        I-pq po I
!       from the interpolation point to the grid point.
!       At last determines the interpolation grid:
!       - computation of the latitude and the longitude of the
!         point situated at the upper left corner of the 16 points
!         square, and of the interpolation point.

!       Version of LARCINA used for half-level quantities.

!**   Interface.
!     ----------
!        *CALL* *LARCINHA(......)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of arrays where computations are performed.
!          KPROF   - depth of work.
!          KHVI    - 1/0: Cubic Hermite vertical interpolations are needed/not
!                         needed.
!          KSTABUF - for a latitude IGL, KSTABUF(IGL) is the
!                    address of the element corresponding to
!                    (ILON=1,IGL) in the NPROMA arrays.
!          KTIP    - 1: interpolation grid for wind used in the traj research.
!                    2: interpolation grid for origin point (RHS of eqns).
!                    3: interpolation grid for origin point, U and V only
!                       (used in adjoint of semi-Lagrangian scheme). 
!          KROT    - KROT=1: computation of the elements po and pq
!                    of the wind displacement matrix.
!                    KROT=0: no computation.
!          PLSDEPI - (Number of points by latitude) / (2 * PI) .
!          PCOLON  - cosine of the grid-point longitude on the
!                    computational sphere.
!          PSILON  - sine of the grid-point longitude on the
!                    computational sphere.
!          PGEMU   - sine of the grid-point geographical latitude.
!          PGSQM2  - cosine of the grid-point geographical latitude.
!          PGECLO  - cosine of the grid-point geographical longitude.
!          PGESLO  - sine of the grid-point geographical longitude.
!          PCOSCO  - cos(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude) at full levels.
!                    ! if LRPLANE: x - coordinate (fractional system).
!          PSINCO  - sin(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude) at full levels.
!                    ! if LRPLANE: y - coordinate (fractional system).
!          PSINLA  - sine of the interpolation point geographical latitude.
!          PCOPHI  - cosine of the geographical angle between the
!                    interpolation point and the grid-point.
!          PLEV    - vertical coordinate of the interpolation point
!                    at full levels.
!          PKAPPA  - kappa function ("coefficient of SLHD") based on the
!                    rescaled horizontal deformation of the flow evaluated
!                    at instant "t" for the final point F

!        OUTPUT:
!          KL0H      - index of the four western points
!                      of the 16 points interpolation grid.
!          PDLOH     - distances for horizontal linear interpolations
!                      in longitude (latitude rows 0, 1, 2, 3)
!          PCLOH     - weights for horizontal cubic interpolations in
!                      longitude (latitude rows 1, 2)
!          PCLOHSLD  - weights for horizontal cubic interpolations in
!                      longitude, SLHD case (latitude rows 1, 2)
!          PDLATH    - distance for horizontal linear interpolations
!                      in latitude
!          PCLAH     - weights for horizontal cubic interpolations
!                      in latitude
!          PCLAHSLD  - weights for horizontal cubic interpolations
!                      in latitude, SLHD case
!          PDVERH    - distance for vertical linear interpolation
!          PVINTWH   - vertical cubic interpolation weights
!          PVINTWHSLD- vertical cubic interpolation weights, SLHD case

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        Called by LAIDEPH.
!        Calls (E)LASCAW.

!     Reference.
!     ----------

!     Author.
!     -------
!        C. SMITH, mirrors the LAIDEP -> ELASCAW calling sequence.
!        K. YESSAD, after the subroutine LAGINT0 written by Maurice IMBARD
!        Alain CRAPLET and Michel ROCHAS  METEO FRANCE/EERM/CRMD

!     Modifications.
!     --------------
!  Original : AUGUST 2002.
!  Modifications:
!   Modified 03-05-01 by A. Bogatchev: add arg in call elascaw
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   F. Vana  2004-Nov: new argument PVINTDSH
!   K. Yessad and J. Vivoda (Mar 2005): move calculation of
!    ZSINCOH, ZCOSCOH, ZLEVH from LAIDEPH, and adapt the code
!    for spherical geometry also.
!   F.Vana  09-Jan-2007 new argument to ELASCAW
!   K. Yessad 07-03-2007: Remove useless (gw)_surf interpol. in NH+LGWADV.
!   F. Vana 28-Aug-2007 removing argument PVINTDSH
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   30-Jun-2008 J. Masek   Dataflow for new SLHD interpolators.
!   K. Yessad Nov 2008: rationalisation of dummy argument interfaces
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST    , ONLY : RPI
USE YOMCT0    , ONLY : LRPLANE
USE YOMDIM    , ONLY : NFLEVG   ,NPROMA   ,NFLSA    ,NFLEN    ,NDGSAH   ,&
 & NDGENH   ,NDLUNG   ,NDLUXG   ,NDGUNG   ,NDGUXG   ,NRLEVX
USE YOMDYNA   , ONLY : LSLHD    ,LSLHDQUAD
USE YOMLEG    , ONLY : RLATI    ,RIPI0    ,RIPI1    ,RIPI2    ,RSLD1    ,&
 & RSLD2    ,RSLD3    ,RSLDW
USE YOMGEM    , ONLY : VETAH    ,VETAF    ,VCUICOH  ,VSLDH    ,VSLDWH   ,&
 & NVAUTH   ,NSTTYP   ,RMUCEN   ,RLOCEN   ,RSTRET   ,R4JP     ,RC2P1    ,&
 & RC2M1    ,VRLEVX
USE YOMLUN    , ONLY : NULOUT
USE YOMMP     , ONLY : NASLB1

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KHVI 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTABUF(NDGSAH:NDGENH) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTIP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KROT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSDEPI(NDGSAH:NDGENH)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOSCO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINCO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSINLA(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOPHI(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLEV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAPPA(NPROMA,NFLEVG)
INTEGER(KIND=JPIM),INTENT(OUT)   :: KL0H(NPROMA,0:NFLEVG-1,0:3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLOH(NPROMA,0:NFLEVG-1,0:3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLOH(NPROMA,0:NFLEVG-1,3,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLOHSLD(NPROMA,0:NFLEVG-1,3,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLATH(NPROMA,0:NFLEVG-1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLAH(NPROMA,0:NFLEVG-1,3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLAHSLD(NPROMA,0:NFLEVG-1,3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDVERH(NPROMA,0:NFLEVG-1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVINTWH(NPROMA,0:NFLEVG-1,3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVINTWHSLD(NPROMA,0:NFLEVG-1,3)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: ILEVH(NPROMA,0:NFLEVG-1)
INTEGER(KIND=JPIM) :: ILH0H(NPROMA,0:NFLEVG-1,0:3)
INTEGER(KIND=JPIM) :: IDEP(NPROMA,0:NFLEVG-1)

INTEGER(KIND=JPIM) :: IHOR, IWIS
INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB)    :: ZVDERWH(NPROMA,0:NFLEVG-1,2*KHVI,2*KHVI)
REAL(KIND=JPRB)    :: ZPHVWH(NPROMA,0:NFLEVG-1,4*KHVI)
REAL(KIND=JPRB)    :: ZVINTWS(NPROMA,0:NFLEVG-1,1:4)
REAL(KIND=JPRB)    :: ZCOSCOH(NPROMA,0:NFLEVG-1) 
REAL(KIND=JPRB)    :: ZSINCOH(NPROMA,0:NFLEVG-1) 
REAL(KIND=JPRB)    :: ZLONH(NPROMA,0:NFLEVG-1) 
REAL(KIND=JPRB)    :: ZLATH(NPROMA,0:NFLEVG-1) 
REAL(KIND=JPRB)    :: ZLEVH(NPROMA,0:NFLEVG-1) 
REAL(KIND=JPRB)    :: ZWEI(NFLEVG) 
REAL(KIND=JPRB)    :: ZVETAON,ZVETAOX,ZDEL_LON
REAL(KIND=JPRB)    :: ZDSTRET,ZDEPI,ZPIS2

REAL(KIND=JPRB)    :: ZLON(NPROMA,NFLEVG) 
REAL(KIND=JPRB)    :: ZLAT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)    :: ZP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)    :: ZQ(NPROMA,NFLEVG) 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "elascaw.intfb.h"
#include "larche.intfb.h"
#include "lascaw.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARCINHA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY INITIALISATIONS.
!              ----------------------------

IHOR=1

ZDSTRET=2.0_JPRB*RSTRET
ZDEPI=2.0_JPRB*RPI
ZPIS2=0.5_JPRB*RPI

! * Input variable IWIS for LASCAW.
IF (KTIP == 1) THEN
! * trajectory research.
  CALL ABOR1('LARCINHA: interpolation for trajectory search not coded.')
  IWIS=101
ELSEIF (KTIP == 2) THEN
! * origin point interpolations.
  IWIS=103+KHVI
ELSEIF (KTIP == 3) THEN
  IWIS=103
ENDIF

IF ( IWIS /= 103 ) THEN
!  CALL ABOR1('LARCINHA: only cubic Lagrange interpolation is coded.')
  WRITE(NULOUT,*) 'IWIS = ', IWIS
  IWIS = 103
ENDIF

ZVETAON=VETAH(0)
ZVETAOX=VETAH(NFLEVG)

!     ------------------------------------------------------------------

!*       2.    COMPUTE INTERPOLATION WEIGHTS AND GRID.
!              ---------------------------------------

IF (LRPLANE) THEN

  IF (KROT == 1) THEN
    CALL ABOR1('LARCINHA: ELARCHE not coded for LGWADV=.T.')
    ! CALL ELARCHE(......)
  ENDIF

  ! * compute half level ZCOSCOH, ZSINCOH, ZLEVH:

  ! Obtain the departure points for half-level grid-points from the coordinates
  ! of full-level departure points.
  ! Notes:
  ! a) On the upper and lower boundaries there is no vertical displacement:
  !    that is, a particle initially on the upper or lower boundary remains
  !    within that boundary for all time.
  !    The horizontal displacement within these boundary surfaces is 
  !    obtained by vertical extrapolation of displacements at full-levels.
  !    The safest extrapolation is used: constant extrapolation.

  ! b) For interior half-levels, the departure point coordinates are
  !    obtained by averaging of the departure point coordinates on the
  !    two neighbouring full-levels. This averaging could be done using
  !    linear interpolation in eta or any other valid vertical coordinate.
  !    Logarithmic pressure thickness may offer desirable conservation
  !    properties if used for this purpose.
  !    Currently this average is done using a linear interpolation in eta.
  DO JLEV=1,NFLEVG-1
    ZWEI(JLEV)=(VETAH(JLEV)-VETAF(JLEV))/(VETAF(JLEV+1)-VETAF(JLEV))
  ENDDO
  !! ZWEI(1:NFLEVG)=0.5_JPRB

  DO JROF=KST,KPROF
    ! Horizontal coordinates of departure points for grid-points on upper
    ! and lower boundaries.
    ZSINCOH(JROF,0) = PSINCO(JROF,1)
    ZCOSCOH(JROF,0) = PCOSCO(JROF,1)
    ! Vertical coordinates reflecting zero normal displacement at boundaries
    ZLEVH(JROF,0) = 0.0_JPRB
  ENDDO

  DO JLEV=1,NFLEVG-1
    DO JROF=KST,KPROF

      ! * computations on horizontal plans.
      ZSINCOH(JROF,JLEV) = PSINCO(JROF,JLEV) &
       & + ZWEI(JLEV) * ( PSINCO(JROF,JLEV+1) - PSINCO(JROF,JLEV) )  
      ZCOSCOH(JROF,JLEV) = PCOSCO(JROF,JLEV) &
       & + ZWEI(JLEV) * ( PCOSCO(JROF,JLEV+1) - PCOSCO(JROF,JLEV) )  

      ! * computations on a vertical.
      ZLEVH(JROF,JLEV) = PLEV(JROF,JLEV) &
       & + ZWEI(JLEV) * ( PLEV(JROF,JLEV+1) - PLEV(JROF,JLEV) )
      ZLEVH(JROF,JLEV)=MAX(ZVETAON,MIN(ZVETAOX,ZLEVH(JROF,JLEV)))

    ENDDO
  ENDDO

  CALL ELASCAW( &
   ! --- INPUT ----------------------------------------------------------------
   & NASLB1,NPROMA,KST,KPROF,0,NFLEVG,NDGSAH,NDGENH,&
   & NDLUNG,NDLUXG,NDGUNG,NDGUXG,&
   & NFLSA,NFLEN,KSTABUF,IWIS,IHOR,KHVI,&
   & LSLHD,LSLHDQUAD,&
   & ZCOSCOH,ZSINCOH,ZLEVH,&
   & VETAF,VETAH,NVAUTH,&
   & VCUICOH,VSLDH,VSLDWH,NRLEVX,VRLEVX,PKAPPA,&
   ! --- OUTPUT ---------------------------------------------------------------
   & PDLATH,PCLAH(1,0,1),PCLAHSLD(1,0,1),&
   & PDLOH(1,0,0),PCLOH(1,0,1,1),PCLOHSLD(1,0,1,1),&
   & KL0H(1,0,0),ILH0H(1,0,0),ILEVH,&
   & PDVERH,PVINTWH,PVINTWHSLD,ZVINTWS,&
   & ZVDERWH,ZPHVWH,&
   & IDEP)

ELSE

  CALL LARCHE(NPROMA,KST,KPROF,NFLEVG,&
   & NSTTYP,ZDSTRET,RC2M1,RC2P1,RPI,ZDEPI,&
   & RLOCEN,RMUCEN,PCOLON,PSILON,&
   & PGEMU,PGSQM2,PGECLO,PGESLO,&
   & PCOSCO,PSINCO,PSINLA,PCOPHI,&
   & KROT,ZLON,ZLAT,ZP,ZQ)

  ! * compute half level ZLONH, ZLATH, ZLEVH:

  ! Obtain the departure points for half-level grid-points from the coordinates
  ! of full-level departure points.
  ! Notes:
  ! a) On the upper and lower boundaries there is no vertical displacement:
  !    that is, a particle initially on the upper or lower boundary remains
  !    within that boundary for all time.
  !    The horizontal displacement within these boundary surfaces is 
  !    obtained by vertical extrapolation of displacements at full-levels.
  !    The safest extrapolation is used: constant extrapolation.

  ! b) For interior half-levels, the departure point coordinates are
  !    obtained by averaging of the departure point coordinates on the
  !    two neighbouring full-levels. This averaging could be done using
  !    linear interpolation in eta or any other valid vertical coordinate.
  !    Logarithmic pressure thickness may offer desirable conservation
  !    properties if used for this purpose.
  !    Currently this average is done using a linear interpolation in eta.
  !    Note that, for spherical geometry, the simplest way to do the
  !    average is to do it on the computational sphere (and this is also
  !    more consistent with the fact that SL interpolations are done
  !    on the computational sphere).
    
  DO JLEV=1,NFLEVG-1
    ZWEI(JLEV)=(VETAH(JLEV)-VETAF(JLEV))/(VETAF(JLEV+1)-VETAF(JLEV))
  ENDDO
  !! ZWEI(1:NFLEVG)=0.5_JPRB

  DO JROF=KST,KPROF
    ! Horizontal coordinates of departure points for grid-points on upper
    ! and lower boundaries.
    ZLONH(JROF,0) = ZLON(JROF,1)
    ZLATH(JROF,0) = ZLAT(JROF,1)
    ! Vertical coordinates reflecting zero normal displacement at boundaries
    ZLEVH(JROF,0) = 0.0_JPRB
  ENDDO

  DO JLEV=1,NFLEVG-1
    DO JROF=KST,KPROF
      ! * computations on horizontal plans.
      IF ((ZLON(JROF,JLEV+1) - ZLON(JROF,JLEV)) > RPI) THEN
        ZDEL_LON=ZLON(JROF,JLEV+1)-ZLON(JROF,JLEV)-ZDEPI
      ELSEIF ((ZLON(JROF,JLEV+1) - ZLON(JROF,JLEV)) < -RPI) THEN
        ZDEL_LON=ZLON(JROF,JLEV+1)-ZLON(JROF,JLEV)+ZDEPI
      ELSE
        ZDEL_LON=ZLON(JROF,JLEV+1)-ZLON(JROF,JLEV)
      ENDIF
      ZLONH(JROF,JLEV)=ZLON(JROF,JLEV)+ZWEI(JLEV)*ZDEL_LON
    ENDDO
  ENDDO

  DO JLEV=1,NFLEVG-1
    DO JROF=KST,KPROF
      ! * computations on horizontal plans (latitude).
      ZLATH(JROF,JLEV) = ZLAT(JROF,JLEV) &
       & + ZWEI(JLEV) * ( ZLAT(JROF,JLEV+1) - ZLAT(JROF,JLEV) )  
      ! * computations on a vertical.
      ZLEVH(JROF,JLEV) = PLEV(JROF,JLEV) &
       & + ZWEI(JLEV) * ( PLEV(JROF,JLEV+1) - PLEV(JROF,JLEV) )
      ZLEVH(JROF,JLEV)=MAX(ZVETAON,MIN(ZVETAOX,ZLEVH(JROF,JLEV)))
    ENDDO
  ENDDO

  CALL LASCAW(&
   ! --- INPUT ----------------------------------------------------------------
   & NASLB1,NPROMA,KST,KPROF,0,NFLEVG,NDGSAH,NDGENH,&
   & NFLSA,KSTABUF,IWIS,IHOR,KHVI,&
   & LSLHD,LSLHDQUAD,&
   & R4JP,ZPIS2,PLSDEPI,RLATI(NDGSAH),&
   & RIPI0(NDGSAH),RIPI1(NDGSAH),RIPI2(NDGSAH),&
   & RSLD1(NDGSAH),RSLD2(NDGSAH),RSLD3(NDGSAH),RSLDW(1,1,NDGSAH),&
   & ZLONH,ZLATH,ZLEVH,&
   & VETAF,VETAH,NVAUTH,&
   & VCUICOH,VSLDH,VSLDWH,NRLEVX,VRLEVX,PKAPPA,&
   ! --- OUTPUT ---------------------------------------------------------------
   & PDLATH,PCLAH(1,0,1),PCLAHSLD(1,0,1),&
   & PDLOH(1,0,0),PCLOH(1,0,1,1),PCLOHSLD(1,0,1,1),&
   & KL0H(1,0,0),ILH0H(1,0,0),ILEVH,&
   & PDVERH,PVINTWH,PVINTWHSLD,ZVINTWS,&
   & ZVDERWH,ZPHVWH)

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARCINHA',1,ZHOOK_HANDLE)
END SUBROUTINE LARCINHA

