SUBROUTINE LARMES(KST,KPROF,KSTABUF,LDSETTLS,PB1,PB2,&
 & PLSDEPI,PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,PNORDL,PNORDM,&
 & KVSEPC,KVSEPL,&
 & PLON,PLAT,PCOSCO,PSINCO,PSINLA,PCOPHI,PLEV,PR,PS,PUF,PVF,PWF,&
 & KL0,KLH0,PDLO,PCLO,PDLAT,PCLA,KLEV,PDVER,PVINTW,PVINTWS)  

!**** *LARMES - semi-LAgrangian scheme:
!                Research of the MEdium point on the Sphere.
!                (and also the origin point).

!     Purpose.
!     --------

!      The computation of the location of the medium point "M" of
!     the lagrangian trajectory is performed by an iterative
!     method described by Robert and adapted to the sphere by M. Rochas.
!     Trajectories are great circles.
!     Finally we find the departure (origin) point "O".

!**   Interface.
!     ----------
!        *CALL* *LARMES(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of arrays where computations are performed.
!          KPROF   - depth of work.
!          KSTABUF - for a latitude IGL, KSTABUF(IGL) is the
!                    address of the element corresponding to
!                    (ILON=1,IGL) in the NPROMA arrays.
!          LDSETTLS- .T./.F.: Stable/Conventional algorithm of trajectory
!                    research in the 2TLSL scheme.
!          PB1     - SLBUF1-buffer for interpolations.
!          PB2     - SLBUF2-buf to communicate info from non lag. to lag. dyn.
!          PLSDEPI - (Number of points by latitude) / (2 * PI) .
!          PCOLON  - cosine of the grid-point longitude on the
!                    computational sphere.
!          PSILON  - sine of the grid-point longitude on the
!                    computational sphere.
!          PGEMU   - sine of the grid-point geographical latitude.
!          PGSQM2  - cosine of the grid-point geographical latitude.
!          PGECLO  - cosine of the grid-point geographical longitude.
!          PGESLO  - sine of the grid-point geographical longitude.
!          PNORDL  - zonal component of the normalized vector directed
!                    towards the geographical north pole.
!          PNORDM  - meridian component of the normalized vector directed
!                    towards the geographical north pole.

!        INPUT/OUTPUT:
!          KVSEPC  - vertical separation (used in S/L adjoint, cubic interp.)
!          KVSEPL  - vertical separation (used in S/L adjoint, linear interp.)

!        OUTPUT:
!          PLON    - computational sphere longitude of interpolation point.
!          PLAT    - computational sphere latitude of interpolation point.
!          PCOSCO  - cos(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude).
!          PSINCO  - sin(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude).
!          PSINLA  - sine of the interpolation point geographical latitude.
!          PCOPHI  - cosine of the geographical angle between the
!                    interpolation point and the grid-point.
!          PLEV    - vertical coordinate of the interpolation point.
!          PR      - first element of the wind displacement matrix.
!          PS      - second element of the wind displacement matrix.
!          PUF     - U-comp of "(a/rs)*wind" at the medium point necessary to
!                    find the positions of the medium and origin points,
!                    in a local repere linked to computational sphere.
!          PVF     - V-comp of "(a/rs)*wind" at the medium point necessary to
!                    find the positions of the medium and origin points,
!                    in a local repere linked to computational sphere.
!          PWF     - interpolated etadot used to find the vertical displacement.
!          KL0     - index of the four western points
!                    of the 16 points interpolation grid.
!          KLH0    - second value of index of the four western points
!                    of the 16 points interpolation grid if needed.
!          PDLO    - distances for horizontal linear interpolations
!                    in longitude (latitude rows 0, 1, 2, 3)
!          PCLO    - weights for horizontal cubic interpolations in
!                    longitude (latitude rows 1, 2)
!          PDLAT   - distance for horizontal linear interpolations
!                    in latitude
!          PCLA    - weights for horizontal cubic interpolations
!                    in latitude
!          KLEV    - lower level of the vertical interpolation
!                    grid needed for vertical interpolations.
!          PDVER   - distance for vertical linear interpolation
!          PVINTW  - Vertical cubic interpolation weights.
!          PVINTWS - Vertical spline interpolation weights.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation about semi-Lagrangian scheme.

!     Externals.
!     ----------
!        Calls  LARCINA.
!        Is called by LAPINEA (3D model) 

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD after LAGRMIL and a part of CPLGDY2 written by
!        Maurice IMBARD     METEO FRANCE/EERM/CRMD

!     Modifications.
!     --------------
!        Original : JUNE 1991
!        Modifications:
!        K. YESSAD (METEO-FRANCE CNRM/GMAP): 21/11/1991.
!        K. YESSAD: February 1992 (optimisation, consistency with IO scheme).
!        M. HORTAL (ECMWF) and K. YESSAD (CNRM/GMAP): 04/11/1992.
!               (possible use of an irregularly spaced vertical coordinate).
!        K. YESSAD: December 1992 (non interpolating vertical scheme).
!        M. HORTAL and K. YESSAD: March 1993 (in the research of the
!           trajectory, origin points are put on a layer at each iteration
!           in case of non interpolating vertical scheme).
!        K. YESSAD: NOVEMBER 1993. Optimisation of loop 215.
!        Modified 93-12-14 by K. YESSAD: interpolations at medium point for
!                                      semi-lagrangian temperature equation.
!                                      + optimisation of loop 215.
!        Modified 93-12-21 by C. Temperton: interpolations at medium point for
!                                      semi-lagrangian moisture equation.
!        Modified 94-03-23 by M. Hortal: interpolations at medium point for
!                                      semi-lagrangian momentum equations.
!        Modified 93-02-15 by K. YESSAD: replacing switch IWIS by IWISA and
!                                      IXLAG.
!        Modified 94-10-17 by M. Tolstykh: CUD-5 interpolation
!        K. YESSAD (JAN 1995):
!             - Introduction of array PCUDWM for optimisations of CUD-5
!               meridian derivatives.
!             - Simplification of treatment of NVLAG=2 formulation of
!               2D shallow-water continuity equation.
!             - Remove of useless quantity KBFLDOP.
!        R. Bubnova (25 JAN 1995):
!             - plane geometry after developments of Martin Janousek.
!        K. YESSAD (AUGUST 1995): 
!             - update number of dummy arguments in CALL LARCIN for
!               accurate version and second-order decentering in the
!               two-time level semi-lagrangian scheme.
!             - DO loops updated according to F90 norms.
!        Modified 95-10-01 by MPP Group *ECMWF*:  Distributed memory version
!        K. YESSAD (MARCH 1996):
!             - ZLON, ZLAT become dummy arguments PLON, PLAT.
!        M. Janousek and R. Bubnova (3 JUN 1996):
!             - non-hydrostatic dynamics.
!        Modified 96-03 by A. Untch : grid-point prognostic ozone (EC)
!        Modified by K. YESSAD (AUGUST 1996):
!               - Hermite cubic vertical interpolations for passive scalars.
!                 Not yet coded for plane geometry.
!                 Not yet coded for CUD-5 scheme.
!        Modified 96-10-23 by M. Hortal:
!           wind averaged along trajectory to find midpoint in 2TL scheme.
!        Modified 97-05-19 by C. Temperton: LDCUTRA switch added (for SWE)
!        Modified 97-09-11 by K. YESSAD: bug corrections.
!        Modified 98-01-19 by C. Temperton:
!                 2D (SWE) options removed and transferred to LARMES2
!        Modified 98-03-04 by C. Temperton: remove redundant ILOC function
!        Modified 98-09-01 by N. Wedi:add new dummy variables to larcin
!        Modified July 1998 by K. YESSAD: cleaning
!             - removal of options NODEC=2, L2TLAW=T, L2TLAT=T
!             - removal of useless dummy arguments.
!             - complete declaration of dummy arguments.
!             - correction of false comments.
!        Modified 98-05-27 by E. Holm: Order of ozone changed in call and
!               in call to LARCIN.
!        Modified 98-12-04 by K. YESSAD: 'zuf,zvf' become 'puf,pvf'
!        Modified 98-12_18 by K. YESSAD and I. GOSPODINOV:
!             - option LDELTRA.
!             - simplification and optimisation of code to compute PLEV.
!        Modified 99-03-31 by P. SMOLIKOVA: option LRDAORG (double averaging).
!        Modified 99-09-21 by D. SALMOND : On demand Semi-Lagrangian comms
!        Modified 00-02-11 by C. Temperton: added KVSEP, LDFINDVSEP
!        Modified 00-06-15 by C. Temperton: NVSEP -> NVSEPC,NVSEPL.
!        Modified 99-11-16 by K. YESSAD:
!             - in 'lacudw', compute PDLAT and ILA as in 'lascaw' =>
!               'krlatx', 'kindlat' and 'pdllat' become useless.
!        Modified April 2000 by Malgorzata Szczech:
!             - bug corrections for thermodynamical ozon.
!             - add option "lhvo3" for thermodynamical ozon.
!        Renamed August 2000 by Deborah Salmond for split into _traj and _int
!        Modified 00-11-07 by C. Temperton: streamlining.
!        Modified 00-11-23 by C. Temperton: renamed _TRAJ and _INT routines.
!        Modified 01-06-26 by K. YESSAD:
!         - cleanings (remove useless variables),
!           update or correct some comments, especially in the context of the
!           introduction of thin layer hypothesis relaxation ("(a/rs)*winds"
!           are interpolated and no longer winds if lvercor=true).
!         - relaxation of thin layer hypothesis.
!         - Restore options with Hermite vertical interpolations.
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hortal      01-Dec-2003 Extra-fields from the dynamics
!        Modified 05-01-11 by C. Temperton: streamlining.
!        Modified 05-01-14 by C. Temperton: transfer computations from
!                                start of LAIDEP to end of LARMES.
!        F. Vana  2004-Nov: New argument to larcina
!        O. Spaniel  05-2005: phasing 2005
!        M.Hortal    22-Nov-2005  Write to NULERR if Max wind too big
!        F.Vana   09-Jan-2007 New argument to LARCINA
!        F. Vana  28-Aug-2007: One less dummy array to LARCINA
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!        30-Jun-2008 J. Masek    Dataflow for new SLHD interpolators.
!        K. Yessad Nov 2008: rationalisation of dummy argument interfaces
! End Modifications
!     ------------------------------------------------------------------

USE PARKIND1  , ONLY : JPIM     ,JPRB
USE YOMHOOK   , ONLY : LHOOK,   DR_HOOK

USE YOMCST    , ONLY : RPI
USE YOMCT0    , ONLY : NCONF    ,LPC_NESCT,LVERCOR
USE YOMCVER   , ONLY : LSVTSM
USE YOMDIM    , ONLY : NFLEVG   ,NPROMA   ,NFLSA    ,NFLEN    ,NDGSAH   ,&
 & NDGENH   ,NDLUNG   ,NDLUXG   ,NDGUNG   ,NDGUXG   ,NRLEVX
USE YOMDYNA   , ONLY : LRWSDLR  ,LSLDIA
USE YOMDYN    , ONLY : NITMP    ,VMAX1    ,VMAX2    ,LELTRA   ,NCURRENT_ITER,&
 & VETAON   ,VETAOX
USE YOMGEM    , ONLY : VBH      ,VETAH    ,VETAF
USE YOMLUN    , ONLY : NULERR
USE YOMMP     , ONLY : NASLB1
USE YOMRIP    , ONLY : RDTSA    ,RDTS62   ,RDTS22   ,RTDT
USE YOMSC2    , ONLY : NFLDSLB1 ,NFLDSLB2
USE YOMSEP    , ONLY : LFINDVSEP
USE PTRSLB1   , ONLY : MSLB1UR0 ,MSLB1VR0 ,MSLB1WR0 ,MSLB1WRA
USE PTRSLB2   , ONLY : MSLB2URL ,MSLB2VRL ,MSLB2WRL

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTABUF(NDGSAH:NDGENH) 
LOGICAL           ,INTENT(IN)    :: LDSETTLS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB1(NASLB1,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSDEPI(NDGSAH:NDGENH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNORDL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNORDM(NPROMA) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KVSEPC 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KVSEPL 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLON(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLAT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOSCO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSINCO(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSINLA(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOPHI(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLEV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PR(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PS(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUF(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVF(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWF(NPROMA,NFLEVG) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KL0(NPROMA,NFLEVG,0:3) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLH0(NPROMA,NFLEVG,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLO(NPROMA,NFLEVG,0:3) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLO(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDLAT(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLA(NPROMA,NFLEVG,3)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLEV(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDVER(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTW(NPROMA,NFLEVG,3)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVINTWS(NPROMA,NFLEVG,1:4) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IHVI
INTEGER(KIND=JPIM) :: ILEVEXP
INTEGER(KIND=JPIM) :: IROFEXP
INTEGER(KIND=JPIM) :: IROT
INTEGER(KIND=JPIM) :: ISTESB(NPROMA)
INTEGER(KIND=JPIM) :: ISTEST(NPROMA)
INTEGER(KIND=JPIM) :: ISTEST0(NPROMA)
INTEGER(KIND=JPIM) :: ITIP
INTEGER(KIND=JPIM) :: JITER
INTEGER(KIND=JPIM) :: JLEV
INTEGER(KIND=JPIM) :: JROF
INTEGER(KIND=JPIM) :: IL0A (NPROMA,NFLEVG,0:3)
INTEGER(KIND=JPIM) :: IDEP (NPROMA,NFLEVG)

LOGICAL :: LLINTV
LOGICAL :: LLO
LOGICAL :: LLSLHD, LLSLHDQUAD

REAL(KIND=JPRB) :: ZDTS22, ZDTS62, ZDTSA
REAL(KIND=JPRB) :: ZEW, ZEWX
REAL(KIND=JPRB) :: ZFAC
REAL(KIND=JPRB) :: ZINT
REAL(KIND=JPRB) :: ZLEVB, ZLEVO, ZLEVT
REAL(KIND=JPRB) :: ZNOR, ZNOR2, ZNORX
REAL(KIND=JPRB) :: ZPU, ZPV
REAL(KIND=JPRB) :: ZSPHSV
REAL(KIND=JPRB) :: ZVMAX1, ZVMAX2
REAL(KIND=JPRB) :: ZEPS
REAL(KIND=JPRB) :: ZVETAON, ZVETAOX

REAL(KIND=JPRB) :: ZWF(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZWFSM(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZWFASM(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSINLA(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZCOSCO(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSINCO(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZCOPHI(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZPLEV(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! unused arguments in call to LARCINA:
! a) input arrays - will not be used since LSLHD was set to .FALSE.
!    in LAPINEA before call to LARMES/ELARMES => dimensions can be
!    contracted
REAL(KIND=JPRB) :: ZUN_KAPPA(1)
! b) output arrays - dimensions kept for safety
REAL(KIND=JPRB) :: ZUN_CLOSLD(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB) :: ZUN_CLASLD(NPROMA,NFLEVG,3)
REAL(KIND=JPRB) :: ZUN_VINTWSLD(NPROMA,NFLEVG,3)
REAL(KIND=JPRB) :: ZUN_VDERW(NPROMA,NFLEVG,0,0)  ! case KHVI=0
REAL(KIND=JPRB) :: ZUN_HVW(NPROMA,NFLEVG,0)      ! case KHVI=0

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "larcina.intfb.h"
#include "laismoa.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARMES',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY INITIALISATIONS AND TESTS.
!              --------------------------------------

!*       1.1   Test that wind is not too strong.

ZNOR=0.0_JPRB
ZEW=0.0_JPRB
ZVMAX1=VMAX1*VMAX1
ZVMAX2=VMAX2*VMAX2
ZEPS=1.E-6_JPRB

DO JLEV=1,NFLEVG
  DO JROF=KST,KPROF
    ZNOR=MAX(PB2(JROF,MSLB2VRL+JLEV-1)*PB2(JROF,MSLB2VRL+JLEV-1),ZNOR)
    ZEW =MAX(PB2(JROF,MSLB2URL+JLEV-1)*PB2(JROF,MSLB2URL+JLEV-1),ZEW )
  ENDDO
ENDDO

IF (ZNOR > ZVMAX1) THEN
  IF (LVERCOR) THEN
    WRITE(NULERR,*) ' MAX (a/rs)*V WIND=',SQRT(ZNOR)
  ELSEIF (LRWSDLR) THEN
    WRITE(NULERR,*) ' MAX (a/r)*V WIND=',SQRT(ZNOR)
  ELSE
    WRITE(NULERR,*) ' MAX V WIND=',SQRT(ZNOR)
  ENDIF
ENDIF
IF (ZEW > ZVMAX1) THEN
  IF (LVERCOR) THEN
    WRITE(NULERR,*) ' MAX (a/rs)*U WIND=',SQRT(ZEW)
  ELSEIF (LRWSDLR) THEN
    WRITE(NULERR,*) ' MAX (a/r)*U WIND=',SQRT(ZEW)
  ELSE
    WRITE(NULERR,*) ' MAX U WIND=',SQRT(ZEW)
  ENDIF
ENDIF
IF (ZNOR > ZVMAX2) THEN
  ZNOR=0.0_JPRB
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZNORX=PB2(JROF,MSLB2VRL+JLEV-1)*PB2(JROF,MSLB2VRL+JLEV-1)
      ZNOR=MAX(ZNORX,ZNOR)
      IF(ZNOR == ZNORX) THEN
        ILEVEXP=JLEV
        IROFEXP=JROF
      ENDIF
    ENDDO
  ENDDO
  IF (LVERCOR) THEN
    WRITE(NULERR,*) ' (a/rs)*V WIND =',SQRT(ZNOR),' IS TOO STRONG, EXPLOSION.'
  ELSEIF (LRWSDLR) THEN
    WRITE(NULERR,*) ' (a/r)*V WIND =',SQRT(ZNOR),' IS TOO STRONG, EXPLOSION.'
  ELSE
    WRITE(NULERR,*) ' V WIND =',SQRT(ZNOR),' IS TOO STRONG, EXPLOSION.'
  ENDIF
  WRITE(NULERR,*) ' LEVEL= ',ILEVEXP,' POINT= ',IROFEXP
  WRITE(NULERR,*) ' LON  = ',ACOS(PCOLON(IROFEXP))*180._JPRB/RPI
  WRITE(NULERR,*) ' LAT  = ',ASIN(PGEMU(IROFEXP))*180._JPRB/RPI
  IF (LVERCOR) THEN
    CALL ABOR1(' !(a/rs)*V WIND TOO STRONG, EXPLOSION!!!')
  ELSEIF (LRWSDLR) THEN
    CALL ABOR1(' !(a/r)*V WIND TOO STRONG, EXPLOSION!!!')
  ELSE
    CALL ABOR1(' !V WIND TOO STRONG, EXPLOSION!!!')
  ENDIF
ENDIF
IF (ZEW > ZVMAX2) THEN
  ZEW=0.0_JPRB
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZEWX=PB2(JROF,MSLB2URL+JLEV-1)*PB2(JROF,MSLB2URL+JLEV-1)
      ZEW=MAX(ZEWX,ZEW)
      IF(ZEW == ZEWX) THEN
        ILEVEXP=JLEV
        IROFEXP=JROF
      ENDIF
    ENDDO
  ENDDO
  IF (LVERCOR) THEN
    WRITE(NULERR,*) ' (a/rs)*U WIND =',SQRT(ZEW),' IS TOO STRONG, EXPLOSION.'
  ELSEIF (LRWSDLR) THEN
    WRITE(NULERR,*) ' (a/r)*U WIND =',SQRT(ZEW),' IS TOO STRONG, EXPLOSION.'
  ELSE
    WRITE(NULERR,*) ' U WIND =',SQRT(ZEW),' IS TOO STRONG, EXPLOSION.'
  ENDIF
  WRITE(NULERR,*) ' LEVEL= ',ILEVEXP,' POINT= ',IROFEXP
  WRITE(NULERR,*) ' LON  = ',ACOS(PCOLON(IROFEXP))*180._JPRB/RPI
  WRITE(NULERR,*) ' LAT  = ',ASIN(PGEMU(IROFEXP))*180._JPRB/RPI
  IF (LVERCOR) THEN
    CALL ABOR1(' !(a/rs)*U WIND TOO STRONG, EXPLOSION!!!')
  ELSEIF (LRWSDLR) THEN
    CALL ABOR1(' !(a/r)*U WIND TOO STRONG, EXPLOSION!!!')
  ELSE
    CALL ABOR1(' !U WIND TOO STRONG, EXPLOSION!!!')
  ENDIF
ENDIF

!*       1.2   Miscellaneous preliminary initialisations.

LLO=LDSETTLS.OR.LPC_NESCT.OR.(NCURRENT_ITER>0)

IF(LLO.OR.LELTRA) THEN
  ZDTS22=RDTS22*4._JPRB
  ZDTSA=RDTSA*2._JPRB
  ZDTS62=RDTS62*4._JPRB
ELSE
  ZDTS22=RDTS22
  ZDTSA=RDTSA
  ZDTS62=RDTS62
ENDIF

! deactivate computation of SLHD weights
LLSLHD     =.FALSE.
LLSLHDQUAD =.FALSE.

!*       1.3   Computation of weights for smooth interpolation at arrival point.

IF(LSVTSM) THEN
  !*     SMOOTH INTERPOLATION AT ARRIVAL POINT
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZSINLA(JROF,JLEV)= PGEMU(JROF)-ZEPS
      ZCOSCO(JROF,JLEV)= PGSQM2(JROF)
      ZSINCO(JROF,JLEV)=ZEPS
      ZCOPHI(JROF,JLEV)=1.0_JPRB-ZEPS
      ZPLEV(JROF,JLEV)=VETAF(JLEV)
    ENDDO
  ENDDO
  IHVI=0
  IROT=0
  ITIP=1
  LLINTV=.FALSE.
  CALL LARCINA(KST,KPROF,IHVI,KSTABUF,LFINDVSEP,LLSLHD,LLSLHDQUAD,LLINTV,&
   & ITIP,IROT,.FALSE.,PLSDEPI,&
   & PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,&
   & ZCOSCO,ZSINCO,ZSINLA,ZCOPHI,ZPLEV,&
   & ZUN_KAPPA,PB1(1,MSLB1UR0),PB1(1,MSLB1VR0),PB1(1,MSLB1WRA),&
   & KVSEPC,KVSEPL,&
   & PLON,PLAT,PR,PS,&
   & PUF,PVF,ZWF,ZWFASM,&
   & IL0A,KLH0,PDLO,PCLO,ZUN_CLOSLD,PDLAT,PCLA,ZUN_CLASLD,&
   & KLEV,PDVER,PVINTW,ZUN_VINTWSLD,PVINTWS,&
   & ZUN_VDERW,ZUN_HVW,IDEP)  
  CALL LAISMOA(NASLB1,NPROMA,KST,KPROF,NFLEVG,NFLSA,&
   & NFLEN,PDLO,IL0A,PDVER,PB1(1,MSLB1WRA),ZWFASM)  
ENDIF

!     ------------------------------------------------------------------

!*       2.    ITERATIONS.
!              -----------

ISTEST0(:)=0
DO JITER=1,NITMP

  !*       2.1   DETERMINATION OF THE MEDIUM POINT "M" OR THE ORIGIN POINT "O".

  ! Computation of the norm of the real "(a/rs)*wind" vector.
  ! Computation of the angle (PHI=DT . NORM OF V ON RADIUS)**2
  ! then computation of the coordinates of the medium point "M".
  ! If (LLO=T or LELTRA=T) the origin point "O" is computed
  ! instead of the medium point "M".

  ZLEVT=VETAF(0)
  ZLEVB=VETAF(NFLEVG+1)
  ZVETAON=(1.0_JPRB-VETAON)*VETAH(0)+VETAON*VETAF(1)
  ZVETAOX=(1.0_JPRB-VETAOX)*VETAH(NFLEVG)+VETAOX*VETAF(NFLEVG)
  ZFAC=REAL(NRLEVX,JPRB)/(VETAF(NFLEVG+1)-VETAF(0))
  ISTEST(:)=0
  ISTESB(:)=0

  IF (JITER == 1) THEN

    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF

        ! * computations on horizontal plans.

        PUF(JROF,JLEV)=PB2(JROF,MSLB2URL+JLEV-1)
        PVF(JROF,JLEV)=PB2(JROF,MSLB2VRL+JLEV-1)
        ZNOR2=( PB2(JROF,MSLB2URL+JLEV-1)*PB2(JROF,MSLB2URL+JLEV-1)&
         & +PB2(JROF,MSLB2VRL+JLEV-1)*PB2(JROF,MSLB2VRL+JLEV-1) )  
        PCOPHI(JROF,JLEV)=1.0_JPRB-ZDTS22*ZNOR2
        ZSPHSV=ZDTSA*(1.0_JPRB-ZDTS62*ZNOR2)
        ZINT=( PB2(JROF,MSLB2URL+JLEV-1)*PNORDL(JROF)&
         & +PB2(JROF,MSLB2VRL+JLEV-1)*PNORDM(JROF))*ZSPHSV  
        PSINLA(JROF,JLEV)= PGEMU(JROF)*PCOPHI(JROF,JLEV)-ZINT*PGSQM2(JROF)
        PCOSCO(JROF,JLEV)= PGSQM2(JROF)*PCOPHI(JROF,JLEV)+ZINT* PGEMU(JROF)
        PSINCO(JROF,JLEV)=-( PB2(JROF,MSLB2URL+JLEV-1)*PNORDM(JROF)&
         & -PB2(JROF,MSLB2VRL+JLEV-1)*PNORDL(JROF) )*ZSPHSV  

        ! * computations on a vertical.

        IF(VBH(JLEV) > 1.E-5_JPRB .OR. (NCONF /= 1 .AND. NCONF /= 302) &
         & .OR..NOT.LSVTSM) THEN
          ZWF(JROF,JLEV)=PB2(JROF,MSLB2WRL+JLEV-1)
        ELSE
          ZWF(JROF,JLEV)=ZWFASM(JROF,JLEV)
        ENDIF
        ZLEVO=VETAF(JLEV)-RTDT*ZWF(JROF,JLEV)
        ISTEST(JROF)=ISTEST(JROF)-MIN(0,MAX(-1,NINT(ZLEVO-ZLEVT-0.5_JPRB)))
        ISTESB(JROF)=ISTESB(JROF)-MIN(0,MAX(-1,NINT(ZLEVB-ZLEVO-0.5_JPRB)))
        ZLEVO=MIN(ZVETAOX,MAX(ZVETAON,ZLEVO))
        IF(LLO.OR.LELTRA) THEN
          PLEV(JROF,JLEV)=ZLEVO
        ELSE
          PLEV(JROF,JLEV)=0.5_JPRB*(ZLEVO+VETAF(JLEV))
        ENDIF

      ENDDO
    ENDDO

  ELSE

    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF

        ! * ZPU,ZPV are the coordinates of VM in the local repere
        !   related to F.

        IF(LLO) THEN
          ZPU= 0.5_JPRB*(PR(JROF,JLEV)*PUF(JROF,JLEV)&
           & +PS(JROF,JLEV)*PVF(JROF,JLEV)&
           & +PB2(JROF,MSLB2URL+JLEV-1)*PNORDM(JROF)&
           & -PB2(JROF,MSLB2VRL+JLEV-1)*PNORDL(JROF))  
          ZPV=0.5_JPRB*(-PS(JROF,JLEV)*PUF(JROF,JLEV)&
           & +PR(JROF,JLEV)*PVF(JROF,JLEV)&
           & +PB2(JROF,MSLB2URL+JLEV-1)*PNORDL(JROF)&
           & +PB2(JROF,MSLB2VRL+JLEV-1)*PNORDM(JROF))  
          IF(VBH(JLEV) > 1.E-5_JPRB .OR. (NCONF /= 1 .AND. NCONF /= 302) &
           & .OR..NOT.LSVTSM) THEN
            ZWF(JROF,JLEV)=0.5_JPRB*(ZWF(JROF,JLEV)+PB2(JROF,MSLB2WRL+JLEV-1))
          ELSE
            ZWF(JROF,JLEV)=0.5_JPRB*(ZWFSM(JROF,JLEV)+ZWFASM(JROF,JLEV))
          ENDIF
        ELSE
          ZPU= PR(JROF,JLEV)*PUF(JROF,JLEV)+PS(JROF,JLEV)*PVF(JROF,JLEV)
          ZPV=-PS(JROF,JLEV)*PUF(JROF,JLEV)+PR(JROF,JLEV)*PVF(JROF,JLEV)
        ENDIF
        ZNOR2             =(ZPU*ZPU+ZPV*ZPV)
        IF(JITER == NITMP) THEN
          ! * save (zpu,zpv) in (puf,pvf) with a rotation.
          PUF(JROF,JLEV)= ZPU*PNORDM(JROF)+ZPV*PNORDL(JROF)
          PVF(JROF,JLEV)=-ZPU*PNORDL(JROF)+ZPV*PNORDM(JROF)
        ENDIF
        PCOPHI(JROF,JLEV) =1.0_JPRB-ZDTS22*ZNOR2
        ZSPHSV            =ZDTSA*(1.0_JPRB-ZDTS62*ZNOR2)
        ZINT              =ZPV*ZSPHSV
        PSINLA(JROF,JLEV) = PGEMU(JROF)*PCOPHI(JROF,JLEV)-ZINT*PGSQM2(JROF)
        PCOSCO(JROF,JLEV) = PGSQM2(JROF)*PCOPHI(JROF,JLEV)+ZINT* PGEMU(JROF)
        PSINCO(JROF,JLEV) =-ZPU*ZSPHSV

        ! * computations on a vertical.

        ZLEVO=VETAF(JLEV)-RTDT*ZWF(JROF,JLEV)
        ISTEST(JROF)=ISTEST(JROF)-MIN(0,MAX(-1,NINT(ZLEVO-ZLEVT-0.5_JPRB)))
        ISTESB(JROF)=ISTESB(JROF)-MIN(0,MAX(-1,NINT(ZLEVB-ZLEVO-0.5_JPRB)))
        ZLEVO=MIN(ZVETAOX,MAX(ZVETAON,ZLEVO))
        IF(LLO.OR.LELTRA) THEN
          PLEV(JROF,JLEV)=ZLEVO
        ELSE
          PLEV(JROF,JLEV)=0.5_JPRB*(ZLEVO+VETAF(JLEV))
        ENDIF

        IF( LSLDIA.AND.(JITER==NITMP) ) THEN
          ! trajectory vertical velocity
          PWF(JROF,JLEV)=ZWF(JROF,JLEV)
        ENDIF

      ENDDO
    ENDDO

  ENDIF

  IF( ANY( ISTEST /= ISTEST0 ) .AND. JITER == NITMP) THEN
    WRITE(NULERR,'(A,I6,A)') ' MAX SMILAG TRAJECTORY OUT OF ATM ',&
     & MAXVAL(ISTEST),' TIMES.'
    ! print statistics
    DO JROF=KST,KPROF
      IF( ISTEST(JROF) /= 0 ) THEN
        WRITE(NULERR,*) ' POINT= ',JROF,' MAX VERTICAL VEL.= ',&
         & MAXVAL(ZWF(JROF,:))
        WRITE(NULERR,*) ' LON  = ',ACOS(PCOLON(JROF))*180._JPRB/RPI
        WRITE(NULERR,*) ' LAT  = ',ASIN(PGEMU(JROF))*180._JPRB/RPI
      ENDIF
    ENDDO
  ENDIF
  IF( ANY( ISTESB /= ISTEST0 ) .AND. JITER == NITMP) THEN
    WRITE(NULERR,'(A,I6,A)') ' MAX SMILAG TRAJECTORY UNDERGROUND ',&
     & MAXVAL(ISTESB),' TIMES.'
    ! print statistics
    DO JROF=KST,KPROF
      IF(ISTESB(JROF) /= 0 ) THEN
        WRITE(NULERR,*) ' POINT= ',JROF,' MAX VERTICAL VEL.= ', & 
         & MAXVAL(ZWF(JROF,:))
        WRITE(NULERR,*) ' LON  = ',ACOS(PCOLON(JROF))*180._JPRB/RPI
        WRITE(NULERR,*) ' LAT  = ',ASIN(PGEMU(JROF))*180._JPRB/RPI  
      ENDIF
    ENDDO
  ENDIF

  !*       2.2   DETERMINATION OF THE "(a/rs)*wind" AT "M" OR "O".

  IF(JITER /= NITMP) THEN

    ! If (LLO=T or LELTRA=T)
    ! and JITER < NITMP "(a/rs)*wind" is interpolated at
    ! the origin point "O" instead of at the medium point "M".
    ! In the other cases the wind is interpolated at "M".

    IHVI=0
    IROT=1
    ITIP=1
    LLINTV=.TRUE.

    CALL LARCINA(KST,KPROF,IHVI,KSTABUF,LFINDVSEP,LLSLHD,LLSLHDQUAD,LLINTV,&
     & ITIP,IROT,.FALSE.,PLSDEPI,&
     & PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,&
     & PCOSCO,PSINCO,PSINLA,PCOPHI,PLEV,&
     & ZUN_KAPPA,PB1(1,MSLB1UR0),PB1(1,MSLB1VR0),PB1(1,MSLB1WR0),&
     & KVSEPC,KVSEPL,&
     & PLON,PLAT,PR,PS,&
     & PUF,PVF,ZWF,ZWFSM,&
     & KL0,KLH0,PDLO,PCLO,ZUN_CLOSLD,PDLAT,PCLA,ZUN_CLASLD,&
     & KLEV,PDVER,PVINTW,ZUN_VINTWSLD,PVINTWS,&
     & ZUN_VDERW,ZUN_HVW,IDEP)  

  ENDIF

ENDDO

!     ------------------------------------------------------------------

!*       3.    COMPUTATION OF THE ORIGIN POINT "O" COORDINATES.
!              ------------------------------------------------

IF (.NOT.(LLO.OR.LELTRA)) THEN

  ZVETAON=(1.0_JPRB-VETAON)*VETAH(0)+VETAON*VETAF(1)
  ZVETAOX=(1.0_JPRB-VETAOX)*VETAH(NFLEVG)+VETAOX*VETAF(NFLEVG)

  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF

      ! computations on horizontal plans.

      PSINLA(JROF,JLEV)=2.0_JPRB*PCOPHI(JROF,JLEV)*PSINLA(JROF,JLEV)&
       & -PGEMU (JROF)
      PSINCO(JROF,JLEV)=2.0_JPRB*PCOPHI(JROF,JLEV)*PSINCO(JROF,JLEV)
      PCOSCO(JROF,JLEV)=2.0_JPRB*PCOPHI(JROF,JLEV)*PCOSCO(JROF,JLEV)&
       & -PGSQM2(JROF)

      ! computations on a vertical.

      PLEV(JROF,JLEV)=2.0_JPRB*PLEV(JROF,JLEV)-VETAF(JLEV)
      PLEV(JROF,JLEV)=MAX(ZVETAON,MIN(ZVETAOX,PLEV(JROF,JLEV)))

      ! Computation of the angle <0,G>.

      PCOPHI(JROF,JLEV)=2.0_JPRB*PCOPHI(JROF,JLEV)*PCOPHI(JROF,JLEV)-1.0_JPRB

    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARMES',1,ZHOOK_HANDLE)
END SUBROUTINE LARMES
