SUBROUTINE LARMES5(KST,KPROF,KSTABUF,LDSETTLS,PB15,PB2,&
 & PLSDEPI,PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,PNORDL,PNORDM,&
 & PLON5,PLAT5,PCOSCO5,PSINCO5,PSINLA5,PCOPHI5,PLEV5,PR5,PS5,PUF5,PVF5,&
 & KL0,KLH0,PDLO5,PDLAT5,KLEV,PDVER5)  

!**** *LARMES5 - semi-LAgrangian scheme:
!                Research of the MEdium point on the Sphere.
!                (and also the origin point).
!                Trajectory computation for TL/AD models.

!     Purpose.
!     --------

!      The computation of the location of the medium point "M" of
!     the lagrangian trajectory is performed by an iterative
!     method described by Robert and adapted to the sphere by M. Rochas.
!     Trajectories are great circles.
!     (Repeat of computations in LARMES, but useful results are saved
!     at each iteration - used by adjoint of semi-Lagrangian scheme)

!**   Interface.
!     ----------
!        *CALL* *LARMES5(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of arrays where computations are performed.
!          KPROF   - depth of work.
!          KSTABUF - for a latitude IGL, KSTABUF(IGL) is the
!                    address of the element corresponding to
!                    (ILON=1,IGL) in the NPROMA arrays.
!          LDSETTLS- .T./.F.: Stable/Conventional algorithm of trajectory
!                    research in the 2TLSL scheme.
!          PB15    - SLBUF1-buffer for interpolations (trajectory).
!          PB2     - SLBUF2-buf to communicate info from non lag. to lag. dyn.
!          PLSDEPI - (Number of points by latitude) / (2 * PI) .
!          PCOLON  - cosine of the grid-point longitude on the
!                    computational sphere.
!          PSILON  - sine of the grid-point longitude on the
!                    computational sphere.
!          PGEMU   - sine of the grid-point geographical latitude.
!          PGSQM2  - cosine of the grid-point geographical latitude.
!          PGECLO  - cosine of the grid-point geographical longitude.
!          PGESLO  - sine of the grid-point geographical longitude.
!          PNORDL  - zonal component of the normalized vector directed
!                    towards the geographical north pole.
!          PNORDM  - meridian component of the normalized vector directed
!                    towards the geographical north pole.

!        OUTPUT:
!          PLON5   - computational sphere longitude of interpolation point.
!          PLAT5   - computational sphere latitude of interpolation point.
!          PCOSCO5 - cos(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude).
!          PSINCO5 - sin(Longitude-Longitude(grid-point))*cos(Latitude)
!                    of the interpolation point (geographical longitude
!                    and latitude).
!          PSINLA5 - sine of the interpolation point geographical latitude.
!          PCOPHI5 - cosine of the geographical angle between the
!                    interpolation point and the grid-point.
!          PLEV5   - vertical coordinate of the interpolation point.
!          PR5     - first element of the wind displacement matrix.
!          PS5     - second element of the wind displacement matrix.
!          PUF5    - U-comp of "(a/rs)*wind" at the medium point necessary to
!                    find the positions of the medium and origin points,
!                    in a local repere linked to computational sphere.
!          PVF5    - V-comp of "(a/rs)*wind" at the medium point necessary to
!                    find the positions of the medium and origin points,
!                    in a local repere linked to computational sphere.
!          KL0     - index of the four western points
!                    of the 16 points interpolation grid.
!          KLH0    - second value of index of the four western points
!                    of the 16 points interpolation grid if needed.
!          PDLO5   - distances for horizontal linear interpolations
!                    in longitude (latitude rows 0, 1, 2, 3)
!          PDLAT5  - distance for horizontal linear interpolations
!                    in latitude
!          KLEV    - lower level of the vertical interpolation
!                    grid needed for vertical interpolations.
!          PDVER5  - distance for vertical linear interpolation.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation about semi-Lagrangian scheme.

!     Externals.
!     ----------
!        Calls  LARCINA.
!        Is called by LAPINEA5 (3D model) 

!     Reference.
!     ----------

!     Author.
!     -------
!        C. Temperton (ECMWF)

!     Modifications. (list from SLADJTR)
!     --------------
!        Original : 99/10/09
!        Modifications:
!        C. Temperton 00-02-11: changed call to LARCIN 
!        C. Temperton 00-06-16: changed call to LARCIN again.... 
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!   01-Oct-2003 M. Hamrud     CY28 Cleaning
!   14-Jan-2005 C. Temperton  transfer computations from start of LAIDEP
!                             to end of LARMES.
!   08-Feb-2005 D. Salmond    SLADJTR replaced by LARMES5
!        F.Vana  09-Jan-2007 update of arguments for LARCINA
!        F.Vana  28-Aug-2007 update of arguments for LARCINA
!   30-Jun-2008 J. Masek      Updated arguments of LARCINA.
!   K. Yessad Nov 2008: rationalisation of dummy argument interfaces
! End Modifications
!-------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM    , ONLY : NFLEVG   ,NPROMA   ,NFLSA    ,NFLEN    ,NDGSAH   ,&
 & NDGENH   ,NDLUNG   ,NDLUXG   ,NDGUNG   ,NDGUXG   ,NRLEVX
USE YOMDYN    , ONLY : NITMP    ,LELTRA   ,VETAON   ,VETAOX
USE YOMGEM    , ONLY : VETAH    ,VETAF
USE YOMMP     , ONLY : NASLB1
USE YOMRIP    , ONLY : RDTSA    ,RDTS62   ,RDTS22   ,RTDT
USE YOMSC2    , ONLY : NFLDSLB15,NFLDSLB2
USE PTRSLB15  , ONLY : MSLB1UR05,MSLB1VR05,MSLB1WR05
USE PTRSLB2   , ONLY : MSLB2URL5,MSLB2VRL5,MSLB2WRL5

!-------------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTABUF(NDGSAH:NDGENH) 
LOGICAL           ,INTENT(IN)    :: LDSETTLS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB15(NASLB1,NFLDSLB15)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSDEPI(NDGSAH:NDGENH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOLON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSILON(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGSQM2(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGECLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGESLO(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNORDL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNORDM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLON5(NPROMA,NFLEVG,NITMP)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLAT5(NPROMA,NFLEVG,NITMP)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOSCO5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSINCO5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSINLA5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOPHI5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLEV5(NPROMA,NFLEVG,NITMP) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PR5(NPROMA,NFLEVG,NITMP-1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PS5(NPROMA,NFLEVG,NITMP-1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUF5(NPROMA,NFLEVG,NITMP-1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVF5(NPROMA,NFLEVG,NITMP-1) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KL0(NPROMA,NFLEVG,0:3,NITMP)
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLH0(NPROMA,NFLEVG,0:3)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLO5(NPROMA,NFLEVG,0:3,NITMP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLAT5(NPROMA,NFLEVG,NITMP) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLEV(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDVER5(NPROMA,NFLEVG,NITMP) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: ITIP
INTEGER(KIND=JPIM) :: IDEP(NPROMA,NFLEVG)

INTEGER(KIND=JPIM) :: IROT, JITER, JLEV, JROF, IHVI

REAL(KIND=JPRB)    :: ZPU5,ZLEVO5,ZINT5,ZPW5,ZPV5,ZSPHSV5, &
 & ZDTS62,ZDTSA,ZDTS22,ZNOR25

LOGICAL :: LLFINDVSEP
LOGICAL :: LLINTV
LOGICAL :: LLO
LOGICAL :: LLSLHD, LLSLHDQUAD

REAL(KIND=JPRB) :: ZVETAON, ZVETAOX

REAL(KIND=JPRB) :: ZWF5(NPROMA,NFLEVG,NITMP-1)

! unused arguments in call to LARCINA:
! a) input arrays - will not be used since LSLHD was set to .FALSE.
!    in LAPINEA before call to LARMES/ELARMES => dimensions can be
!    contracted
REAL(KIND=JPRB) :: ZUN_KAPPA(1)
! b) input/output
INTEGER(KIND=JPIM) :: IUN_VSEPC
INTEGER(KIND=JPIM) :: IUN_VSEPL
! c) output arrays - dimensions kept for safety
REAL(KIND=JPRB) :: ZUN_WFSM(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZUN_CLO(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB) :: ZUN_CLOSLD(NPROMA,NFLEVG,3,2)
REAL(KIND=JPRB) :: ZUN_CLA(NPROMA,NFLEVG,3)
REAL(KIND=JPRB) :: ZUN_CLASLD(NPROMA,NFLEVG,3)
REAL(KIND=JPRB) :: ZUN_VINTW(NPROMA,NFLEVG,3)
REAL(KIND=JPRB) :: ZUN_VINTWSLD(NPROMA,NFLEVG,3)
REAL(KIND=JPRB) :: ZUN_VINTWS(NPROMA,NFLEVG,1:4)
REAL(KIND=JPRB) :: ZUN_VDERW(NPROMA,NFLEVG,0,0)  ! case KHVI=0
REAL(KIND=JPRB) :: ZUN_HVW(NPROMA,NFLEVG,0)      ! case KHVI=0

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "larcina.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARMES5',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY INITIALISATIONS AND TESTS.
!              --------------------------------------

!*       1.2   Miscellaneous preliminary initialisations.

LLO=LDSETTLS

IF(LLO.OR.LELTRA) THEN
  ZDTS22=RDTS22*4
  ZDTSA=RDTSA*2
  ZDTS62=RDTS62*4
ELSE
  ZDTS22=RDTS22
  ZDTSA=RDTSA
  ZDTS62=RDTS62
ENDIF

! deactivate computation of SLHD weights
LLSLHD     =.FALSE.
LLSLHDQUAD =.FALSE.

!     ------------------------------------------------------------------

!*       2.    ITERATIONS.
!              -----------

DO JITER=1,NITMP

  !*       2.1   DETERMINATION OF THE MEDIUM POINT "M" OR THE ORIGIN POINT "O".

  ! Computation of the norm of the real wind vector.
  ! Computation of the angle (PHI=DT . NORM OF V ON RADIUS)**2
  ! then computation of the coordinates of the medium point "M".
  ! If (LLO=T or LELTRA=T) the origin point "O" is computed
  ! instead of the medium point "M".

  ZVETAON=(1.0_JPRB-VETAON)*VETAH(0)+VETAON*VETAF(1)
  ZVETAOX=(1.0_JPRB-VETAOX)*VETAH(NFLEVG)+VETAOX*VETAF(NFLEVG)

  IF (JITER == 1) THEN

    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF

        ! * computations on horizontal plans.

        ! ky: probably missing here the following lines
        ! PUF5(JROF,JLEV,1)=PB2(JROF,MSLB2URL5+JLEV-1)
        ! PVF5(JROF,JLEV,1)=PB2(JROF,MSLB2VRL5+JLEV-1)

        ZNOR25=( PB2(JROF,MSLB2URL5+JLEV-1)*PB2(JROF,MSLB2URL5+JLEV-1)&
         & +PB2(JROF,MSLB2VRL5+JLEV-1)*PB2(JROF,MSLB2VRL5+JLEV-1) )  
        PCOPHI5(JROF,JLEV,1)=1.0_JPRB-ZDTS22*ZNOR25
        ZSPHSV5=ZDTSA*(1.0_JPRB-ZDTS62*ZNOR25)
        ZINT5=( PB2(JROF,MSLB2URL5+JLEV-1)*PNORDL(JROF)&
         & +PB2(JROF,MSLB2VRL5+JLEV-1)*PNORDM(JROF))*ZSPHSV5  
        PSINLA5(JROF,JLEV,1)=&
         & PGEMU(JROF)*PCOPHI5(JROF,JLEV,1)-ZINT5*PGSQM2(JROF)  
        PCOSCO5(JROF,JLEV,1)=&
         & PGSQM2(JROF)*PCOPHI5(JROF,JLEV,1)+ZINT5*PGEMU(JROF)  
        PSINCO5(JROF,JLEV,1)=-( PB2(JROF,MSLB2URL5+JLEV-1)*PNORDM(JROF)&
         & -PB2(JROF,MSLB2VRL5+JLEV-1)*PNORDL(JROF) )*ZSPHSV5  

        ! * computations on a vertical.

        ZLEVO5=VETAF(JLEV)-RTDT*PB2(JROF,MSLB2WRL5+JLEV-1)
        ZLEVO5=MIN(ZVETAOX,MAX(ZVETAON,ZLEVO5))
        IF(LLO.OR.LELTRA) THEN
          PLEV5(JROF,JLEV,1)=ZLEVO5
        ELSE
          PLEV5(JROF,JLEV,1)=0.5_JPRB*(ZLEVO5+VETAF(JLEV))
        ENDIF

      ENDDO
    ENDDO

  ELSE

    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF

        ! * ZPU,ZPV are the coordinates of VM in the local repere
        !   related to F.

        IF(LLO) THEN
          ZPU5= 0.5_JPRB*(PR5(JROF,JLEV,JITER-1)*PUF5(JROF,JLEV,JITER-1)&
           & +PS5(JROF,JLEV,JITER-1)*PVF5(JROF,JLEV,JITER-1)&
           & +PB2(JROF,MSLB2URL5+JLEV-1)*PNORDM(JROF)&
           & -PB2(JROF,MSLB2VRL5+JLEV-1)*PNORDL(JROF))  
          ZPV5=0.5_JPRB*(-PS5(JROF,JLEV,JITER-1)*PUF5(JROF,JLEV,JITER-1)&
           & +PR5(JROF,JLEV,JITER-1)*PVF5(JROF,JLEV,JITER-1)&
           & +PB2(JROF,MSLB2URL5+JLEV-1)*PNORDL(JROF)&
           & +PB2(JROF,MSLB2VRL5+JLEV-1)*PNORDM(JROF))  
          ZPW5=0.5_JPRB*(ZWF5(JROF,JLEV,JITER-1)+PB2(JROF,MSLB2WRL5+JLEV-1))
        ELSE
          ZPU5=PR5(JROF,JLEV,JITER-1)*PUF5(JROF,JLEV,JITER-1)&
           & +PS5(JROF,JLEV,JITER-1)*PVF5(JROF,JLEV,JITER-1)  
          ZPV5=-PS5(JROF,JLEV,JITER-1)*PUF5(JROF,JLEV,JITER-1)&
           & +PR5(JROF,JLEV,JITER-1)*PVF5(JROF,JLEV,JITER-1)  
          ZPW5=ZWF5(JROF,JLEV,JITER-1)
        ENDIF
        ZNOR25            =(ZPU5*ZPU5+ZPV5*ZPV5)
        PCOPHI5(JROF,JLEV,JITER) =1.0_JPRB-ZDTS22*ZNOR25
        ZSPHSV5            =ZDTSA*(1.0_JPRB-ZDTS62*ZNOR25)
        ZINT5             =ZPV5*ZSPHSV5
        PSINLA5(JROF,JLEV,JITER) = &
         & PGEMU(JROF)*PCOPHI5(JROF,JLEV,JITER)-ZINT5*PGSQM2(JROF)  
        PCOSCO5(JROF,JLEV,JITER) = &
         & PGSQM2(JROF)*PCOPHI5(JROF,JLEV,JITER)+ZINT5*PGEMU(JROF)  
        PSINCO5(JROF,JLEV,JITER) =-ZPU5*ZSPHSV5

        ! * computations on a vertical.

        ZLEVO5=VETAF(JLEV)-RTDT*ZPW5
        ZLEVO5=MIN(ZVETAOX,MAX(ZVETAON,ZLEVO5))
        IF(LLO.OR.LELTRA) THEN
          PLEV5(JROF,JLEV,JITER)=ZLEVO5
        ELSE
          PLEV5(JROF,JLEV,JITER)=0.5_JPRB*(ZLEVO5+VETAF(JLEV))
        ENDIF

      ENDDO
    ENDDO

  ENDIF

  !*       2.2   DETERMINATION OF THE "(a/rs)*wind" AT "M" OR "O".

  IF(JITER /= NITMP) THEN

    !   If (LLO=T or LELTRA=T)
    !   and JITER < NITMP the wind is interpolated at
    !   the origin point "O" instead of at the medium point "M".
    !   In the other cases the wind is interpolated at "M".

    IHVI=0
    IROT=1
    ITIP=1
    LLFINDVSEP=.FALSE.
    LLINTV=.TRUE.

    CALL LARCINA(&
     & KST,KPROF,IHVI,KSTABUF,LLFINDVSEP,LLSLHD,LLSLHDQUAD,LLINTV,&
     & ITIP,IROT,.FALSE.,PLSDEPI,&
     & PCOLON,PSILON,PGEMU,PGSQM2,PGECLO,PGESLO,&
     & PCOSCO5(1,1,JITER),PSINCO5(1,1,JITER),&
     & PSINLA5(1,1,JITER),PCOPHI5(1,1,JITER),&
     & PLEV5(1,1,JITER),&
     & ZUN_KAPPA,PB15(1,MSLB1UR05),PB15(1,MSLB1VR05),PB15(1,MSLB1WR05),&
     & IUN_VSEPC,IUN_VSEPL,&
     & PLON5(1,1,JITER),PLAT5(1,1,JITER),PR5(1,1,JITER),PS5(1,1,JITER),&
     & PUF5(1,1,JITER),PVF5(1,1,JITER),ZWF5(1,1,JITER),ZUN_WFSM,&
     & KL0(1,1,0,JITER),KLH0(1,1,0),PDLO5(1,1,0,JITER),ZUN_CLO,ZUN_CLOSLD,&
     & PDLAT5(1,1,JITER),ZUN_CLA,ZUN_CLASLD,&
     & KLEV,PDVER5(1,1,JITER),ZUN_VINTW,ZUN_VINTWSLD,ZUN_VINTWS,&
     & ZUN_VDERW,ZUN_HVW,IDEP)

  ENDIF

ENDDO

!     ------------------------------------------------------------------

!*       3.    COMPUTATION OF THE ORIGIN POINT "O" COORDINATES.
!              ------------------------------------------------

IF (.NOT.(LLO.OR.LELTRA)) THEN

  ZVETAON=(1.0_JPRB-VETAON)*VETAH(0)+VETAON*VETAF(1)
  ZVETAOX=(1.0_JPRB-VETAOX)*VETAH(NFLEVG)+VETAOX*VETAF(NFLEVG)

  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF

      ! computations on horizontal plans.

      PSINLA5(JROF,JLEV,NITMP)=&
       & 2.0_JPRB*PCOPHI5(JROF,JLEV,NITMP)*PSINLA5(JROF,JLEV,NITMP)&
       & -PGEMU (JROF)
      PSINCO5(JROF,JLEV,NITMP)=&
       & 2.0_JPRB*PCOPHI5(JROF,JLEV,NITMP)*PSINCO5(JROF,JLEV,NITMP)
      PCOSCO5(JROF,JLEV,NITMP)=&
       & 2.0_JPRB*PCOPHI5(JROF,JLEV,NITMP)*PCOSCO5(JROF,JLEV,NITMP)&
       & -PGSQM2(JROF)

      ! computations on a vertical.

      PLEV5(JROF,JLEV,NITMP)=2.0_JPRB*PLEV5(JROF,JLEV,NITMP)-VETAF(JLEV)
      PLEV5(JROF,JLEV,NITMP)=MAX(ZVETAON,MIN(ZVETAOX,PLEV5(JROF,JLEV,NITMP)))

      ! Computation of the angle <0,G>.

      PCOPHI5(JROF,JLEV,NITMP)=&
       & 2.0_JPRB*PCOPHI5(JROF,JLEV,NITMP)*PCOPHI5(JROF,JLEV,NITMP)-1.0_JPRB

    ENDDO
  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LARMES5',1,ZHOOK_HANDLE)
END SUBROUTINE LARMES5
