#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE LASCAW(&
 ! --- INPUT -------------------------------------------------
 & KPROMA,KPROMB,KST,KPROF,KSLEV,KFLEV,KGLN,KGLX,&
 & KFLDN,KSTABUF,KWIS,KHOR,KHVI,&
 & LDSLHD,LDSLHDQUAD,&
 & P4JP,PIS2,PLSDEPI,PLATI,&
 & PIPI0,PIPI1,PIPI2,PSLD1,PSLD2,PSLD3,PSLDW,&
 & PLON,PLAT,PLEV,&
 & PVETAF,PVETAH,KVAUT,&
 & PVCUICO,PVSLD,PVSLDW,KRLEVX,PVRLEVX,PKAPPA,&
 ! --- OUTPUT ------------------------------------------------
 & PDLAT,PCLA,PCLASLD,PDLO,PCLO,PCLOSLD,&
 & KL0,KLH0,KLEV,&
 & PDVER,PVINTW,PVINTWSLD,PVINTWS,&
 & PVDERW,PHVW&
 & )  

!     ------------------------------------------------------------------

!**** *LASCAW  -  semi-LAgrangian scheme:
!                 Storage of Coordinates And Weights.

!     Purpose.
!     --------
!       Determines the interpolation grid:
!       - computation of the latitude and the longitude of the
!         point situated at the upper left corner of the 16 points
!         square, and of the interpolation point.
!       - computation of weights.
!       Storage of coordinates and weights.

!**   Interface.
!     ----------
!        *CALL* *LASCAW( ... )

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension for grid-point quantities.
!          KPROMB  - horizontal dimension for interpolation point quantities.
!          KST     - first element of arrays where computations are performed.
!          KPROF   - depth of work.
!          KSLEV   - start vertical dimension (0 or 1).
!                    if 0, routine is used for half level (0 to kflev-1) data.
!                    if 1, routine is used for full level (1 to kflev) data.
!          KFLEV   - vertical dimension.
!          KGLN    - lower dimension for latitude.
!          KGLX    - upper dimension for latitude.
!          KFLDN   - number of the first field.
!          KSTABUF - for a latitude IGL, KSTABUF(IGL) is the
!                    address of the element corresponding to
!                    (ILON=0,IGL) in the KPROMB arrays.
!          KWIS    - kind of interpolation.
!          KHOR    - 0: Horizontal interpolation for each level
!                       of a 3D variable.
!                    1: Interpolation for all origin points corresponding
!                       to a final point of a 2D variable.
!          KHVI    - 1/0: filling weights arrays PVDERW and PHVW is
!                    necessary/not necessary.
!          LDSLHD  - key activating SLHD weights precomputation
!          LDSLHDQUAD - key activating quadratic weights precomputation
!          P4JP    - Approximative inverse of the differences of latitudes.
!          PIS2    - PI / 2
!          PLSDEPI - (Number of points by latitude) / (2 * PI) .
!          PLATI   - latitude on the computational sphere.
!          PIPI0   - coefficients for the bicubic interpolations.
!          PIPI1   - coefficients for the bicubic interpolations.
!          PIPI2   - coefficients for the bicubic interpolations.
!          PSLD1   - auxiliary quantity for SLHD interpolation in latitude
!          PSLD2   - auxiliary quantity for SLHD interpolation in latitude
!          PSLD3   - auxiliary quantity for SLHD interpolation in latitude
!          PSLDW   - weights for SLHD Laplacian smoother in latitude
!          PLON    - Interpolation point longitude on the computational sphere.
!          PLAT    - Interpolation point latitude on the computational sphere.
!          PLEV    - vertical coordinate of the interpolation point.
!          PVETAF  - Values of ETA at full levels.
!          PVETAH  - Values of ETA at half levels.
!          KVAUT   - Help table for vertical box search: gives the number
!                    of the layer immediately above eta.
!          PVCUICO - Denominators of the vertical cubic interpolation
!                    coefficients
!          PVSLD   - auxiliary quantities for vertical SLHD interpolation
!          PVSLDW  - weights for SLHD vertical Laplacian smoother
!          KRLEVX  - Dimension of KVAUT
!          PVRLEVX - REAL(KRLEVX).
!          PKAPPA  - kappa function ("coefficient of SLHD") based on the
!                    rescaled horizontal deformation of the flow evaluated
!                    at instant "t" for the final point F

!        OUTPUT:
!          PDLAT     - distance for horizontal linear interpolations
!                      in latitude
!          PCLA      - weights for horizontal cubic interpolations
!                      in latitude
!          PCLASLD   - weights for horizontal cubic interpolations
!                      in latitude, SLHD case
!          PDLO      - distances for horizontal linear interpolations
!                      in longitude (latitude rows 0, 1, 2, 3)
!          PCLO      - weights for horizontal cubic interpolations in
!                      longitude (latitude rows 1, 2)
!          PCLOSLD   - weights for horizontal cubic interpolations in
!                      longitude, SLHD case (latitude rows 1, 2)
!          KL0       - index of the four western points
!                      of the 16 points interpolation grid.
!          KLH0      - second value of index of the four western points
!                      of the 16 points interpolation grid if needed.
!          KLEV      - lower level of the vertical interpolation
!                      grid needed for vertical interpolations.
!          PDVER     - distance for vertical linear interpolation
!          PVINTW    - vertical cubic interpolation weights
!          PVINTWSLD - vertical cubic interpolation weights, SLHD case
!          PVINTWS   - Vertical spline interpolation weights.
!          PVDERW    - weights to compute vertical derivatives necessary for
!                      Hermite cubic vertical interpolation.
!          PHVW      - Hermite vertical cubic interpolation weights.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation about semi-Lagrangian scheme.

!     Externals.
!     ----------

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD, after the subroutine LAGINT1
!        written by Maurice IMBARD, Alain CRAPLET and Michel ROCHAS
!        METEO-FRANCE, CNRM/GMAP.

!     Modifications.
!     --------------
!        Original : MARCH 1992.
!        Modifications:
!        M. HORTAL (ECMWF) and K. YESSAD (CNRM/GMAP): 04/11/1992.
!               (possible use of an irregularly spaced vertical coordinate).
!        K. YESSAD: December 1992 (non interpolating vertical scheme).
!        Modified 93-12-14 by K. YESSAD and C. TEMPERTON:
!               interpolations at medium point for semi-lagrangian
!               temperature and moisture equations, changing values
!               for switch KWIS.
!        K. YESSAD (AUGUST 1996): Hermite vertical interpolations.
!        K. YESSAD (JULY 1998): cleaning
!          - removal of the useless sequence 2.02 (KWIS=202)
!          - removal of the useless dummy argument PIL.
!          - correction of false comments.
!          - complete declaration of dummy arguments.
!        M. Hamrud & D. Salmond  (Sept 1999): 
!          - Mask for Semi Lagrangian Communications
!        C. Temperton 00-02-11: always output  ILEV array 
!                      (used to compute NVSEP for S/L adjoint)
!        A. Untch (Sept 2001): vertical cubic spline interpolation
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hortal      01-Dec-2003 Info on 16 points even for trajectory computation
!        F. Vana (Nov 2004): 4-points cubic spline interpolation
!        J.Hague       22-Nov-2005 Optimisations
!        Modified 28-07-2006 R. El Khatib + NEC support team : optimization
!        for vector platforms (translation of VPP list_copy directives)
!        F. Vana 28-Aug-2007  removed 4-points cubic spline interpolation
!        07-Nov-2007 J. Masek   New weights for SLHD interpolators.
!        F. Vana 26-Aug-2008  vectorization support
!        K. Yessad (Dec 2008): remove useless dummy arguments
!        K. Yessad (Feb 2009): split loops, rewrite in a shorter way.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK
USE YOMDYN   , ONLY : NSITER   ,NCURRENT_ITER 
USE YOMDYNA  , ONLY : LSLHD_OLD
USE YOMCT0   , ONLY : NPROC
USE YOMMP    , ONLY : NSLEXT   ,NFRSTLOFF 
USE YOMMASK  , ONLY : MASK_SL2 ,LSLONDEM_ACTIVE, MASK_SL2A, LNEW_MASK_SL
USE YOMCVER  , ONLY : RFAA ,RFBB ,RFCC ,RFDD
USE YOMLUN   , ONLY : NULOUT
USE YOMLASCAW, ONLY : KPDUP

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGLN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGLX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTABUF(KGLN:KGLX) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KWIS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KHOR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KHVI 
LOGICAL           ,INTENT(IN)    :: LDSLHD
LOGICAL           ,INTENT(IN)    :: LDSLHDQUAD
INTEGER(KIND=JPIM),INTENT(IN)    :: KRLEVX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P4JP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSDEPI(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLATI(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIPI0(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIPI1(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIPI2(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLD1(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLD2(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLD3(KGLN:KGLX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLDW(3,3,KGLN:KGLX)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON(KPROMB,KSLEV:KFLEV+KSLEV-1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT(KPROMB,KSLEV:KFLEV+KSLEV-1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLEV(KPROMB,KSLEV:KFLEV+KSLEV-1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVETAF(0:KFLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVETAH(0:KFLEV) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVAUT(0:KRLEVX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCUICO(4,0:KFLEV-KSLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVSLD(3,0:KFLEV-KSLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVSLDW(3,3,0:KFLEV-KSLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVRLEVX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAPPA(KPROMB,KFLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLAT(KPROMB,KSLEV:KFLEV+KSLEV-1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLA(KPROMB,KSLEV:KFLEV+KSLEV-1,3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLASLD(KPROMB,KSLEV:KFLEV+KSLEV-1,3)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLO(KPROMB,KSLEV:KFLEV+KSLEV-1,0:3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLO(KPROMB,KSLEV:KFLEV+KSLEV-1,3,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCLOSLD(KPROMB,KSLEV:KFLEV+KSLEV-1,3,2)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KL0(KPROMB,KSLEV:KFLEV+KSLEV-1,0:3) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLH0(KPROMB,KSLEV:KFLEV+KSLEV-1,0:3) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLEV(KPROMB,KSLEV:KFLEV+KSLEV-1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDVER(KPROMB,KSLEV:KFLEV+KSLEV-1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVINTW(KPROMB,KSLEV:KFLEV+KSLEV-1,3) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVINTWSLD(KPROMB,KSLEV:KFLEV+KSLEV-1,3)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVINTWS(KPROMB,KSLEV:KFLEV+KSLEV-1,1:4) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVDERW(KPROMB,KSLEV:KFLEV+KSLEV-1,2*KHVI,2*KHVI) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHVW(KPROMB,KSLEV:KFLEV+KSLEV-1,4*KHVI) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IADDR(KGLN:KGLX)

INTEGER(KIND=JPIM) :: IFLVM2,ILA,ILEV,ILEVV,IZLAT,JLAT,JLEV,JROF,IDIF,IJ_,J_
INTEGER(KIND=JPIM) :: ILO(KPROMB), ILO1(KPROMB), ILO2(KPROMB), ILO3(KPROMB)
INTEGER(KIND=JPIM) :: IILA(KPROMB)

REAL(KIND=JPRB) :: PD, ZD2, ZDA, ZDB, ZDC, ZDD,&
 & ZDEN1, ZDEN2, ZDVER, ZFAC, ZLO, ZLO1, ZLO2, ZLO3, ZNUM

REAL(KIND=JPRB) :: ZVETA(0:KFLEV+KSLEV)
REAL(KIND=JPRB) :: ZKAPPA(KPROMB,0:KFLEV)

LOGICAL :: LLT_SLHD(3),LLSLHD,LLSLHDQUAD,LLSLHD_OLD

INTEGER(KIND=JPIM) :: IADDR_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZVETAF_(KPDUP,0:KFLEV+1)
REAL(KIND=JPRB) :: ZVETAH_(KPDUP,0:KFLEV)
REAL(KIND=JPRB) :: ZVCUICO_(KPDUP,4,0:KFLEV-KSLEV)
REAL(KIND=JPRB) :: ZVSLD_(KPDUP,3,0:KFLEV-KSLEV)
REAL(KIND=JPRB) :: ZVSLDW_(KPDUP,3,3,0:KFLEV-KSLEV)
REAL(KIND=JPRB) :: ZLSDEPI_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZRLATI_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZRIPI0_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZRIPI1_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZRIPI2_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZSLD1_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZSLD2_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZSLD3_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB) :: ZSLDW_(KPDUP,3,3,KGLN:KGLX)
REAL(KIND=JPRB) :: ZZWH(KPROMB,3,KSLEV:KFLEV+KSLEV-1)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
! functions

REAL(KIND=JPRB) :: FHLO1, FHLO2, FHLO3, FHLO4

! auxiliary functions for Hermite cubic interpolation
FHLO1(PD)= (1.0_JPRB-PD)*(1.0_JPRB-PD)*(1.0_JPRB+2.0_JPRB*PD)
FHLO2(PD)= PD*PD*(3._JPRB-2.0_JPRB*PD)
FHLO3(PD)= PD*(1.0_JPRB-PD)*(1.0_JPRB-PD)
FHLO4(PD)=-PD*PD*(1.0_JPRB-PD)

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "lascaw_cla.intfb.h"
#include "lascaw_clo.intfb.h"
#include "lascaw_vintw.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LASCAW',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       0.    PRELIMINARY INITIALISATIONS.
!              ----------------------------

! cases relevant for SLHD scheme (switches LDSLHD, LDSLHDQUAD are
! deactivated during computation of medium points in LAPINEA)
LLSLHD=LDSLHD.AND. &
 & (KWIS==103.OR.KWIS==104.OR.KWIS==105.OR.KWIS==203)
LLSLHDQUAD=LDSLHDQUAD.AND. &
 & (KWIS==103.OR.KWIS==104.OR.KWIS==105.OR.KWIS==203)

! iterative schemes: SLHD is active only in the last iteration
IF (NSITER > 0) THEN
  LLSLHD    =LLSLHD    .AND.(NCURRENT_ITER==NSITER)
  LLSLHDQUAD=LLSLHDQUAD.AND.(NCURRENT_ITER==NSITER)
ENDIF

! switch for old SLHD scheme
LLSLHD_OLD=LLSLHD.AND.LSLHD_OLD

LLT_SLHD(1)=LLSLHD
LLT_SLHD(2)=LLSLHDQUAD
LLT_SLHD(3)=LLSLHD_OLD

DO JLAT=KGLN,KGLX
  IADDR(JLAT)=KSTABUF(JLAT)+KPROMA*(0-KFLDN)+KPROMA*(1-KSLEV)
ENDDO

DO J_ = 1, KPDUP
  IADDR_(J_,KGLN:KGLX)=IADDR(KGLN:KGLX)
ENDDO
DO J_ = 1, KPDUP
  ZVETAF_ (J_,  0:KFLEV+1)=PVETAF (0:KFLEV+1)
  ZVETAH_ (J_,  0:KFLEV)  =PVETAH (0:KFLEV)
  ZLSDEPI_(J_,KGLN:KGLX)=PLSDEPI(KGLN:KGLX)
  ZRLATI_(J_,KGLN:KGLX)=PLATI(KGLN:KGLX)
  ZRIPI0_(J_,KGLN:KGLX)=PIPI0(KGLN:KGLX)
  ZRIPI1_(J_,KGLN:KGLX)=PIPI1(KGLN:KGLX)
  ZRIPI2_(J_,KGLN:KGLX)=PIPI2(KGLN:KGLX)
    ZVCUICO_(J_,1:4,0:KFLEV-KSLEV)=PVCUICO(1:4,0:KFLEV-KSLEV)
ENDDO
IF (LLSLHDQUAD) THEN
  DO J_ = 1, KPDUP
    ZSLD1_(J_,KGLN:KGLX)=PSLD1(KGLN:KGLX)
    ZSLD2_(J_,KGLN:KGLX)=PSLD2(KGLN:KGLX)
    ZSLD3_(J_,KGLN:KGLX)=PSLD3(KGLN:KGLX)
      ZVSLD_(J_,1:3,0:KFLEV-KSLEV)=PVSLD(1:3,0:KFLEV-KSLEV)
  ENDDO
ENDIF
IF ( LLSLHD ) THEN
  DO J_ = 1, KPDUP
    ! this line is not collapsible so it remains same for all platforms
    ZSLDW_(J_,1:3,1:3,KGLN:KGLX)=PSLDW(1:3,1:3,KGLN:KGLX)
      ZVSLDW_(J_,1:3,1:3,0:KFLEV-KSLEV)=PVSLDW(1:3,1:3,0:KFLEV-KSLEV)
  ENDDO
  IF (KSLEV==0) THEN
    ! determine half level kappa
    ZKAPPA(KST:KPROF,0)=PKAPPA(KST:KPROF,1)
    ZKAPPA(KST:KPROF,KFLEV)=PKAPPA(KST:KPROF,KFLEV)
    DO JLEV=1,KFLEV-1
      DO JROF=KST,KPROF
        ZKAPPA(JROF,JLEV)=0.5_JPRB*(PKAPPA(JROF,JLEV)+PKAPPA(JROF,JLEV+1))
      ENDDO
    ENDDO
  ELSEIF (KSLEV==1) THEN
    ! copy full level kappa
    ZKAPPA(KST:KPROF,1:KFLEV)=PKAPPA(KST:KPROF,1:KFLEV)
  ENDIF
ENDIF

IFLVM2=KFLEV-2

IF (KSLEV==1) ZVETA(0:KFLEV+KSLEV)=PVETAF(0:KFLEV+KSLEV)
IF (KSLEV==0) ZVETA(0:KFLEV+KSLEV)=PVETAH(0:KFLEV+KSLEV)
IDIF=KPROMA*(1-KSLEV)

!     ------------------------------------------------------------------

!*       1.    3D MODEL.
!              ---------

!        1.01  Coordinates and weights for trilinear interpolations.

IF (KWIS == 101) THEN

  ZFAC=PVRLEVX/(PVETAF(KFLEV+1)-PVETAF(0))

  DO JLEV=KSLEV,KFLEV+KSLEV-1

    ! * Calculation of linear weights, KL0.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT(JROF,JLEV))-1.5_JPRB)
      PDLAT(JROF,JLEV)=(PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1))&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))  

      ZLO   =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA  )
      ILO(JROF)   =INT(ZLO )
      PDLO(JROF,JLEV,0)=ZLO -REAL(ILO(JROF) ,JPRB)
      ZLO1  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+1)
      ILO1(JROF)  =INT(ZLO1)
      PDLO(JROF,JLEV,1)=ZLO1-REAL(ILO1(JROF),JPRB)
      ZLO2  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+2)
      ILO2(JROF)  =INT(ZLO2)
      PDLO(JROF,JLEV,2)=ZLO2-REAL(ILO2(JROF),JPRB)
      ZLO3  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+3)
      ILO3(JROF)  =INT(ZLO3)
      PDLO(JROF,JLEV,3)=ZLO3-REAL(ILO3(JROF),JPRB)

      ILEV  =KVAUT(INT(PLEV(JROF,JLEV)*ZFAC))-1
      IF(ILEV < IFLVM2.AND.&
       & (PLEV(JROF,JLEV)-ZVETA(ILEV+2)) > 0.0_JPRB) ILEV=ILEV+1  

      KLEV(JROF,JLEV)=ILEV

      PDVER(JROF,JLEV)=(PLEV(JROF,JLEV)-ZVETA(ILEV+1))/&
       & (ZVETA(ILEV+2)-ZVETA(ILEV+1))  

      ILO(JROF)=IADDR_(IJ_,ILA  )+NSLEXT(ILO(JROF) ,ILA  )
      ILO1(JROF)=IADDR_(IJ_,ILA+1)+NSLEXT(ILO1(JROF),ILA+1)
      ILO2(JROF)=IADDR_(IJ_,ILA+2)+NSLEXT(ILO2(JROF),ILA+2)
      ILO3(JROF)=IADDR_(IJ_,ILA+3)+NSLEXT(ILO3(JROF),ILA+3)

      KL0(JROF,JLEV,0)=ILO(JROF)+KPROMA*ILEV
      KL0(JROF,JLEV,1)=ILO1(JROF)+KPROMA*ILEV
      KL0(JROF,JLEV,2)=ILO2(JROF)+KPROMA*ILEV
      KL0(JROF,JLEV,3)=ILO3(JROF)+KPROMA*ILEV

    ENDDO

    ! * Mask calculation for on-demand communications:
    IF(NPROC > 1.AND.LSLONDEM_ACTIVE)THEN
!CDIR NODEP
      DO JROF=KST,KPROF
        MASK_SL2(ILO(JROF)-IDIF)=4
        MASK_SL2(ILO1(JROF)-IDIF)=4
        MASK_SL2(ILO2(JROF)-IDIF)=4
        MASK_SL2(ILO3(JROF)-IDIF)=4
      ENDDO
    ENDIF

  ENDDO

ENDIF

!        1.02  Coordinates and weights for trilinear interpolations
!              + 4 points interpolations.

IF (KWIS == 102) THEN

  ZFAC=PVRLEVX/(PVETAF(KFLEV+1)-PVETAF(0))

  DO JLEV=KSLEV,KFLEV+KSLEV-1
    IF (KHOR == 0) ILEV=JLEV
    IF (KHOR == 1) ILEV=KFLDN

    ! * Calculation of linear weights, KL0, KLH0.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT(JROF,JLEV))-1.5_JPRB)
      PDLAT(JROF,JLEV)=(PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1))&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))  

      ZLO1  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+1)
      ILO1(JROF)  =INT(ZLO1)
      PDLO(JROF,JLEV,1)=ZLO1-REAL(ILO1(JROF),JPRB)
      ZLO2  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+2)
      ILO2(JROF)  =INT(ZLO2)
      PDLO(JROF,JLEV,2)=ZLO2-REAL(ILO2(JROF),JPRB)

      ILEVV =KVAUT(INT(PLEV(JROF,JLEV)*ZFAC))-1
      IF(ILEVV < IFLVM2.AND.&
       & (PLEV(JROF,JLEV)-ZVETA(ILEVV+2)) > 0.0_JPRB) ILEVV=ILEVV+1  

      KLEV(JROF,JLEV)=ILEVV

      PDVER(JROF,JLEV)=(PLEV(JROF,JLEV)-ZVETA(ILEVV+1))/&
       & (ZVETA(ILEVV+2)-ZVETA(ILEVV+1))  

      ILO1(JROF)=IADDR_(IJ_,ILA+1)+NSLEXT(ILO1(JROF),ILA+1)
      ILO2(JROF)=IADDR_(IJ_,ILA+2)+NSLEXT(ILO2(JROF),ILA+2)

      KL0(JROF,JLEV,1)=ILO1(JROF)+KPROMA*ILEVV
      KL0(JROF,JLEV,2)=ILO2(JROF)+KPROMA*ILEVV

      KLH0(JROF,JLEV,1)=ILO1(JROF)+KPROMA*ILEV
      KLH0(JROF,JLEV,2)=ILO2(JROF)+KPROMA*ILEV

    ENDDO

    ! * Mask calculation for on-demand communications:
    IF(NPROC > 1.AND.LSLONDEM_ACTIVE)THEN
      IF(LNEW_MASK_SL)THEN
!CDIR NODEP
        DO JROF=KST,KPROF
          MASK_SL2A(ILO1(JROF)-IDIF)=2
          MASK_SL2A(ILO2(JROF)-IDIF)=2
        ENDDO
      ELSE
!CDIR NODEP
        DO JROF=KST,KPROF
          MASK_SL2(ILO1(JROF)-IDIF)=4
          MASK_SL2(ILO2(JROF)-IDIF)=4
        ENDDO
      ENDIF
    ENDIF

  ENDDO

ENDIF

!        1.03  Coordinates and weights for ( horizontal 12 points
!              + vertical cubic + 32 points interpolations ) or
!              ( horizontal 12 points + 32 points interpolations ).
!              For vertical interpolations only 4 points cubic
!              interpolations are used.

IF (KWIS == 103) THEN

  ZFAC=PVRLEVX/(PVETAF(KFLEV+1)-PVETAF(0))

  DO JLEV=KSLEV,KFLEV+KSLEV-1
    IF (KHOR == 0) ILEV=JLEV
    IF (KHOR == 1) ILEV=KFLDN

    ! * Calculation of linear weights, KL0, KLH0.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT(JROF,JLEV))-1.5_JPRB)
      PDLAT(JROF,JLEV)=(PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1))&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))  

      ZDA   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA)
      ZDB   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1)
      ZDC   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+2)
      ZDD   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+3)

      ! for input to LASCAW_CLA.
      IILA(JROF)=ILA
      ZZWH(JROF,1,JLEV)=(ZDA*ZDC)*ZDD*ZRIPI0_(IJ_,ILA+1)
      ZZWH(JROF,2,JLEV)=(ZDA*ZDB)*ZDD*ZRIPI1_(IJ_,ILA+1)
      ZZWH(JROF,3,JLEV)=(ZDA*ZDB)*ZDC*ZRIPI2_(IJ_,ILA+1)

      ZLO   =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA  )
      ILO(JROF)   =INT(ZLO )
      PDLO(JROF,JLEV,0)=ZLO -REAL(ILO(JROF) ,JPRB)
      ZLO1  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+1)
      ILO1(JROF)  =INT(ZLO1)
      PDLO(JROF,JLEV,1)=ZLO1-REAL(ILO1(JROF),JPRB)
      ZLO2  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+2)
      ILO2(JROF)  =INT(ZLO2)
      PDLO(JROF,JLEV,2)=ZLO2-REAL(ILO2(JROF),JPRB)
      ZLO3  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+3)
      ILO3(JROF)  =INT(ZLO3)
      PDLO(JROF,JLEV,3)=ZLO3-REAL(ILO3(JROF),JPRB)

      ILEVV=KVAUT(INT(PLEV(JROF,JLEV)*ZFAC))-1
      IF(ILEVV < IFLVM2.AND.&
       & (PLEV(JROF,JLEV)-ZVETA(ILEVV+2)) > 0.0_JPRB) ILEVV=ILEVV+1  

      KLEV(JROF,JLEV)=ILEVV
      PDVER(JROF,JLEV)=(PLEV(JROF,JLEV)-ZVETA(ILEVV+1))/&
       & (ZVETA(ILEVV+2)-ZVETA(ILEVV+1))  

      ILO(JROF)=IADDR_(IJ_,ILA  )+NSLEXT(ILO(JROF) ,ILA  )
      ILO1(JROF)=IADDR_(IJ_,ILA+1)+NSLEXT(ILO1(JROF),ILA+1)
      ILO2(JROF)=IADDR_(IJ_,ILA+2)+NSLEXT(ILO2(JROF),ILA+2)
      ILO3(JROF)=IADDR_(IJ_,ILA+3)+NSLEXT(ILO3(JROF),ILA+3)

      KL0(JROF,JLEV,0)=ILO(JROF)+KPROMA*ILEVV
      KL0(JROF,JLEV,1)=ILO1(JROF)+KPROMA*ILEVV
      KL0(JROF,JLEV,2)=ILO2(JROF)+KPROMA*ILEVV
      KL0(JROF,JLEV,3)=ILO3(JROF)+KPROMA*ILEVV

      KLH0(JROF,JLEV,0)=ILO(JROF)+KPROMA*ILEV
      KLH0(JROF,JLEV,1)=ILO1(JROF)+KPROMA*ILEV
      KLH0(JROF,JLEV,2)=ILO2(JROF)+KPROMA*ILEV
      KLH0(JROF,JLEV,3)=ILO3(JROF)+KPROMA*ILEV

    ENDDO

    ! * Mask calculation for on-demand communications:
    IF(NPROC > 1.AND.LSLONDEM_ACTIVE)THEN
      IF(LNEW_MASK_SL)THEN
!CDIR NODEP
        DO JROF=KST,KPROF
          MASK_SL2A(ILO(JROF)-IDIF)=2
          MASK_SL2(ILO1(JROF)-IDIF)=4
          MASK_SL2(ILO2(JROF)-IDIF)=4
          MASK_SL2A(ILO3(JROF)-IDIF)=2
        ENDDO
      ELSE
!CDIR NODEP
        DO JROF=KST,KPROF
          MASK_SL2(ILO(JROF)-IDIF)=4
          MASK_SL2(ILO1(JROF)-IDIF)=4
          MASK_SL2(ILO2(JROF)-IDIF)=4
          MASK_SL2(ILO3(JROF)-IDIF)=4
        ENDDO
      ENDIF
    ENDIF

  ENDDO

  ! * Calculation of PCLA and PCLASLD:
  CALL LASCAW_CLA(KFLEV,KSLEV,KFLEV+KSLEV-1,&
   & KPROMB,KGLN,KGLX,KST,KPROF,LLT_SLHD,&
   & IILA,ZZWH,PDLAT,ZKAPPA,ZSLD1_,ZSLD2_,ZSLD3_,ZSLDW_,&
   & PCLA,PCLASLD)

  ! * Calculation of PCLO and PCLOSLD:
  CALL LASCAW_CLO(KFLEV,KSLEV,KFLEV+KSLEV-1,&
   & KPROMB,KST,KPROF,LLT_SLHD,PDLO(:,:,1),ZKAPPA,&
   & PCLO(:,:,:,1),PCLOSLD(:,:,:,1))
  CALL LASCAW_CLO(KFLEV,KSLEV,KFLEV+KSLEV-1,&
   & KPROMB,KST,KPROF,LLT_SLHD,PDLO(:,:,2),ZKAPPA,&
   & PCLO(:,:,:,2),PCLOSLD(:,:,:,2))

  ! * Calculation of PVINTW and PVINTWSLD:
  CALL LASCAW_VINTW(KSLEV,KFLEV+KSLEV-1,&
   & KPROMB,KFLEV,KSLEV,KST,KPROF,LLT_SLHD,KLEV,&
   & PLEV,PDVER,ZKAPPA,ZVETA,ZVCUICO_,ZVSLD_,ZVSLDW_,&
   & PVINTW,PVINTWSLD)

ENDIF

!        1.04  Coordinates and weights for ( horizontal 12 points
!              + vertical cubic + 32 points interpolations ) or
!              ( horizontal 12 points + 32 points interpolations ).
!              For vertical interpolations 4 points cubic
!              interpolations and Hermite cubic interpolations are used.

IF (KWIS == 104) THEN

  ! * !!! case KWIS=104 valid only for KSLEV=1 !!!

  IF (KSLEV == 0) THEN
    WRITE(NULOUT,*) ' LASCAW: OPTION KWIS=104 IS NOT YET CODED'
    WRITE(NULOUT,*) ' FOR INTERPOLATIONS AT HALF-LEVEL DATA.'
    CALL ABOR1(' LASCAW: ABOR1')
  ENDIF

  ZFAC=PVRLEVX/(PVETAF(KFLEV+1)-PVETAF(0))

  DO JLEV=1,KFLEV

    IF (KHOR == 0) ILEV=JLEV
    IF (KHOR == 1) ILEV=KFLDN

    ! * Calculation of linear weights, KL0, KLH0.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT(JROF,JLEV))-1.5_JPRB)
      PDLAT(JROF,JLEV)=(PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1))&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))  

      ZDA   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA)
      ZDB   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1)
      ZDC   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+2)
      ZDD   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+3)

      ! for input to LASCAW_CLA.
      IILA(JROF)=ILA
      ZZWH(JROF,1,JLEV)=(ZDA*ZDC)*ZDD*ZRIPI0_(IJ_,ILA+1)
      ZZWH(JROF,2,JLEV)=(ZDA*ZDB)*ZDD*ZRIPI1_(IJ_,ILA+1)
      ZZWH(JROF,3,JLEV)=(ZDA*ZDB)*ZDC*ZRIPI2_(IJ_,ILA+1)

      ZLO   =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA  )
      ILO(JROF)   =INT(ZLO )
      PDLO(JROF,JLEV,0)=ZLO -REAL(ILO(JROF) ,JPRB)
      ZLO1  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+1)
      ILO1(JROF)  =INT(ZLO1)
      PDLO(JROF,JLEV,1)=ZLO1-REAL(ILO1(JROF),JPRB)
      ZLO2  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+2)
      ILO2(JROF)  =INT(ZLO2)
      PDLO(JROF,JLEV,2)=ZLO2-REAL(ILO2(JROF),JPRB)
      ZLO3  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+3)
      ILO3(JROF)  =INT(ZLO3)
      PDLO(JROF,JLEV,3)=ZLO3-REAL(ILO3(JROF),JPRB)

      ILEVV=KVAUT(INT(PLEV(JROF,JLEV)*ZFAC))-1
      IF(ILEVV < IFLVM2.AND.&
       & (PLEV(JROF,JLEV)-ZVETAF_(IJ_,ILEVV+2)) > 0.0_JPRB) ILEVV=ILEVV+1  

      KLEV(JROF,JLEV)=ILEVV
      PDVER(JROF,JLEV)=(PLEV(JROF,JLEV)-ZVETAF_(IJ_,ILEVV+1))/&
       & (ZVETAF_(IJ_,ILEVV+2)-ZVETAF_(IJ_,ILEVV+1))  

      ZDVER=PDVER(JROF,JLEV)
      PHVW(JROF,JLEV,1)=FHLO1(ZDVER)
      PHVW(JROF,JLEV,2)=FHLO2(ZDVER)
      PHVW(JROF,JLEV,3)=FHLO3(ZDVER)
      PHVW(JROF,JLEV,4)=FHLO4(ZDVER)

      ILO(JROF)=IADDR_(IJ_,ILA  )+NSLEXT(ILO(JROF) ,ILA  )
      ILO1(JROF)=IADDR_(IJ_,ILA+1)+NSLEXT(ILO1(JROF),ILA+1)
      ILO2(JROF)=IADDR_(IJ_,ILA+2)+NSLEXT(ILO2(JROF),ILA+2)
      ILO3(JROF)=IADDR_(IJ_,ILA+3)+NSLEXT(ILO3(JROF),ILA+3)

      KL0(JROF,JLEV,0)=ILO(JROF)+KPROMA*ILEVV
      KL0(JROF,JLEV,1)=ILO1(JROF)+KPROMA*ILEVV
      KL0(JROF,JLEV,2)=ILO2(JROF)+KPROMA*ILEVV
      KL0(JROF,JLEV,3)=ILO3(JROF)+KPROMA*ILEVV

      KLH0(JROF,JLEV,0)=ILO(JROF) +KPROMA*ILEV
      KLH0(JROF,JLEV,1)=ILO1(JROF)+KPROMA*ILEV
      KLH0(JROF,JLEV,2)=ILO2(JROF)+KPROMA*ILEV
      KLH0(JROF,JLEV,3)=ILO3(JROF)+KPROMA*ILEV

    ENDDO

    ! * Mask calculation for on-demand communications:
    IF(NPROC > 1.AND.LSLONDEM_ACTIVE)THEN
!CDIR NODEP
      DO JROF=KST,KPROF
        MASK_SL2(ILO(JROF))=4
        MASK_SL2(ILO1(JROF))=4
        MASK_SL2(ILO2(JROF))=4
        MASK_SL2(ILO3(JROF))=4
      ENDDO
    ENDIF

  ENDDO

  ! * Calculation of PCLA and PCLASLD:
  CALL LASCAW_CLA(KFLEV,1,KFLEV,&
   & KPROMB,KGLN,KGLX,KST,KPROF,LLT_SLHD,&
   & IILA,ZZWH,PDLAT,ZKAPPA,ZSLD1_,ZSLD2_,ZSLD3_,ZSLDW_,&
   & PCLA,PCLASLD)

  ! * Calculation of PCLO and PCLOSLD:
  CALL LASCAW_CLO(KFLEV,1,KFLEV,&
   & KPROMB,KST,KPROF,LLT_SLHD,PDLO(:,:,1),ZKAPPA,&
   & PCLO(:,:,:,1),PCLOSLD(:,:,:,1))
  CALL LASCAW_CLO(KFLEV,1,KFLEV,&
   & KPROMB,KST,KPROF,LLT_SLHD,PDLO(:,:,2),ZKAPPA,&
   & PCLO(:,:,:,2),PCLOSLD(:,:,:,2))

  ! * Calculation of PVINTW and PVINTWSLD:
  CALL LASCAW_VINTW(1,KFLEV,&
  & KPROMB,KFLEV,KSLEV,KST,KPROF,LLT_SLHD,KLEV,&
  & PLEV,PDVER,ZKAPPA,ZVETA,ZVCUICO_,ZVSLD_,ZVSLDW_,&
  & PVINTW,PVINTWSLD)

  DO JLEV=1,KFLEV

    ! * Calculation of PVDERW (this piece of code is coded for KSLEV=1 only):
    DO JROF=KST,KPROF
      IJ_ = MOD(JROF+1-KST,KPDUP)+1
      ILEVV=KLEV(JROF,JLEV)
      ZNUM=ZVETAF_(IJ_,ILEVV+2)-ZVETAF_(IJ_,ILEVV+1)
      ZDEN1=ZVETAH_(IJ_,ILEVV+1)-ZVETAH_(IJ_,ILEVV)
      ZDEN2=ZVETAH_(IJ_,ILEVV+2)-ZVETAH_(IJ_,ILEVV+1)
      IF(ILEVV >= 1.AND.ILEVV <= KFLEV-3) THEN
        PVDERW(JROF,JLEV,1,1)=0.5_JPRB*ZNUM/ZDEN1
        PVDERW(JROF,JLEV,2,1)=0.5_JPRB*ZNUM/ZDEN1
        PVDERW(JROF,JLEV,1,2)=0.5_JPRB*ZNUM/ZDEN2
        PVDERW(JROF,JLEV,2,2)=0.5_JPRB*ZNUM/ZDEN2
      ELSEIF (ILEVV == 0) THEN
        PVDERW(JROF,JLEV,1,1)=0.0_JPRB
        PVDERW(JROF,JLEV,2,1)=ZNUM/ZDEN1
        PVDERW(JROF,JLEV,1,2)=0.5_JPRB*ZNUM/ZDEN2
        PVDERW(JROF,JLEV,2,2)=0.5_JPRB*ZNUM/ZDEN2
      ELSEIF (ILEVV == KFLEV-2) THEN
        PVDERW(JROF,JLEV,1,1)=0.5_JPRB*ZNUM/ZDEN1
        PVDERW(JROF,JLEV,2,1)=0.5_JPRB*ZNUM/ZDEN1
        PVDERW(JROF,JLEV,1,2)=ZNUM/ZDEN2
        PVDERW(JROF,JLEV,2,2)=0.0_JPRB
      ENDIF
    ENDDO

  ENDDO

ENDIF

!        1.05  Coordinates and weights for ( horizontal 12 points
!              + vertical cubic + 32 points interpolations ) or
!              ( horizontal 12 points + 32 points interpolations ).
!              For vertical interpolations cubic B-spline
!              interpolations are used.

IF (KWIS == 105) THEN

  ZFAC=PVRLEVX/(PVETAF(KFLEV+1)-PVETAF(0))

  DO JLEV=KSLEV,KFLEV+KSLEV-1
    IF (KHOR == 0) ILEV=JLEV
    IF (KHOR == 1) ILEV=KFLDN

    ! * Calculation of linear weights, KL0, KLH0.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT(JROF,JLEV))-1.5_JPRB)
      PDLAT(JROF,JLEV)=(PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1))&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))  

      ZDA   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA)
      ZDB   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1)
      ZDC   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+2)
      ZDD   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+3)

      ! for input to LASCAW_CLA.
      IILA(JROF)=ILA
      ZZWH(JROF,1,JLEV)=(ZDA*ZDC)*ZDD*ZRIPI0_(IJ_,ILA+1)
      ZZWH(JROF,2,JLEV)=(ZDA*ZDB)*ZDD*ZRIPI1_(IJ_,ILA+1)
      ZZWH(JROF,3,JLEV)=(ZDA*ZDB)*ZDC*ZRIPI2_(IJ_,ILA+1)

      ZLO   =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA  )
      ILO(JROF)   =INT(ZLO )
      PDLO(JROF,JLEV,0)=ZLO -REAL(ILO(JROF) ,JPRB)
      ZLO1  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+1)
      ILO1(JROF)  =INT(ZLO1)
      PDLO(JROF,JLEV,1)=ZLO1-REAL(ILO1(JROF),JPRB)
      ZLO2  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+2)
      ILO2(JROF)  =INT(ZLO2)
      PDLO(JROF,JLEV,2)=ZLO2-REAL(ILO2(JROF),JPRB)
      ZLO3  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+3)
      ILO3(JROF)  =INT(ZLO3)
      PDLO(JROF,JLEV,3)=ZLO3-REAL(ILO3(JROF),JPRB)

      ILEVV=KVAUT(INT(PLEV(JROF,JLEV)*ZFAC))-1
      IF(ILEVV < IFLVM2.AND.&
       & (PLEV(JROF,JLEV)-ZVETA(ILEVV+2)) > 0.0_JPRB) ILEVV=ILEVV+1  

      KLEV(JROF,JLEV)=ILEVV
      PDVER(JROF,JLEV)=(PLEV(JROF,JLEV)-ZVETA(ILEVV+1))/&
       & (ZVETA(ILEVV+2)-ZVETA(ILEVV+1))  

      ILO(JROF)=IADDR_(IJ_,ILA  )+NSLEXT(ILO(JROF) ,ILA  )
      ILO1(JROF)=IADDR_(IJ_,ILA+1)+NSLEXT(ILO1(JROF),ILA+1)
      ILO2(JROF)=IADDR_(IJ_,ILA+2)+NSLEXT(ILO2(JROF),ILA+2)
      ILO3(JROF)=IADDR_(IJ_,ILA+3)+NSLEXT(ILO3(JROF),ILA+3)

      KL0(JROF,JLEV,0)=ILO(JROF)+KPROMA*ILEVV
      KL0(JROF,JLEV,1)=ILO1(JROF)+KPROMA*ILEVV
      KL0(JROF,JLEV,2)=ILO2(JROF)+KPROMA*ILEVV
      KL0(JROF,JLEV,3)=ILO3(JROF)+KPROMA*ILEVV

      KLH0(JROF,JLEV,0)=ILO(JROF)+KPROMA*ILEV
      KLH0(JROF,JLEV,1)=ILO1(JROF)+KPROMA*ILEV
      KLH0(JROF,JLEV,2)=ILO2(JROF)+KPROMA*ILEV
      KLH0(JROF,JLEV,3)=ILO3(JROF)+KPROMA*ILEV

    ENDDO

    ! * Mask calculation for on-demand communications:
    IF(NPROC > 1.AND.LSLONDEM_ACTIVE)THEN
!CDIR NODEP
      DO JROF=KST,KPROF
        MASK_SL2(ILO(JROF))=4
        MASK_SL2(ILO1(JROF))=4
        MASK_SL2(ILO2(JROF))=4
        MASK_SL2(ILO3(JROF))=4
      ENDDO
    ENDIF
  ENDDO

  ! * Calculation of PCLA and PCLASLD:
  CALL LASCAW_CLA(KFLEV,KSLEV,KFLEV+KSLEV-1,&
   & KPROMB,KGLN,KGLX,KST,KPROF,LLT_SLHD,&
   & IILA,ZZWH,PDLAT,ZKAPPA,ZSLD1_,ZSLD2_,ZSLD3_,ZSLDW_,&
   & PCLA,PCLASLD)

  ! * Calculation of PCLO and PCLOSLD:
  CALL LASCAW_CLO(KFLEV,KSLEV,KFLEV+KSLEV-1,&
   & KPROMB,KST,KPROF,LLT_SLHD,PDLO(:,:,1),ZKAPPA,&
   & PCLO(:,:,:,1),PCLOSLD(:,:,:,1))
  CALL LASCAW_CLO(KFLEV,KSLEV,KFLEV+KSLEV-1,&
   & KPROMB,KST,KPROF,LLT_SLHD,PDLO(:,:,2),ZKAPPA,&
   & PCLO(:,:,:,2),PCLOSLD(:,:,:,2))

  ! * Calculation of PVINTW and PVINTWSLD:
  CALL LASCAW_VINTW(KSLEV,KFLEV+KSLEV-1,&
  & KPROMB,KFLEV,KSLEV,KST,KPROF,LLT_SLHD,KLEV,&
  & PLEV,PDVER,ZKAPPA,ZVETA,ZVCUICO_,ZVSLD_,ZVSLDW_,&
  & PVINTW,PVINTWSLD)

  DO JLEV=KSLEV,KFLEV+KSLEV-1

    ! * Calculation of PVINTWS (weights for cubic spline interpolation)
    DO JROF=KST,KPROF
      IJ_ = MOD(JROF+1-KST,KPDUP)+1
      ILEVV=KLEV(JROF,JLEV)
      ZD2=PLEV(JROF,JLEV)-ZVETAF_(IJ_,ILEVV+1)
      PVINTWS(JROF,JLEV,1)=RFAA(4,ILEVV  )+ZD2*( RFBB(4,ILEVV  ) + &
       & ZD2*(RFCC(4,ILEVV   ) + ZD2*RFDD(4,ILEVV  ) ) )  
      PVINTWS(JROF,JLEV,2)=RFAA(3,ILEVV+1)+ZD2*( RFBB(3,ILEVV+1) + &
       & ZD2*( RFCC(3,ILEVV+1) + ZD2*RFDD(3,ILEVV+1) ) )  
      PVINTWS(JROF,JLEV,3)=RFAA(2,ILEVV+2)+ZD2*( RFBB(2,ILEVV+2) + &
       & ZD2*( RFCC(2,ILEVV+2) + ZD2*RFDD(2,ILEVV+2) ) )  
      PVINTWS(JROF,JLEV,4)=RFAA(1,ILEVV+3)+ZD2*( RFBB(1,ILEVV+3) + &
       & ZD2*( RFCC(1,ILEVV+3) + ZD2*RFDD(1,ILEVV+3) ) )  
    ENDDO

  ENDDO

ENDIF

!     ----------------------------------------------------------------

!*       2.    2D MODEL AND CASES IN THE 3D MODEL WHERE ONLY
!              2D INTERPOLATIONS ARE NEEDED.
!              ---------------------------------------------

!        2.01  Coordinates and weights for bilinear interpolations.

IF (KWIS == 201) THEN

  DO JLEV=KSLEV,KFLEV+KSLEV-1
    IF (KHOR == 0) ILEV=JLEV
    IF (KHOR == 1) ILEV=KFLDN

    ! * Calculation of linear weights, KL0.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT(JROF,JLEV))-1.5_JPRB)
      PDLAT(JROF,JLEV)=(PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1))&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))  

      ZLO1  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+1)
      ILO1(JROF)  =INT(ZLO1)
      PDLO(JROF,JLEV,1)=ZLO1-REAL(ILO1(JROF),JPRB)
      ZLO2  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+2)
      ILO2(JROF)  =INT(ZLO2)
      PDLO(JROF,JLEV,2)=ZLO2-REAL(ILO2(JROF),JPRB)

      KL0(JROF,JLEV,1)=IADDR_(IJ_,ILA+1)+NSLEXT(ILO1(JROF),ILA+1)+KPROMA*ILEV
      KL0(JROF,JLEV,2)=IADDR_(IJ_,ILA+2)+NSLEXT(ILO2(JROF),ILA+2)+KPROMA*ILEV

    ENDDO

    ! * Mask calculation for on-demand communications:
    IF(NPROC > 1.AND.LSLONDEM_ACTIVE)THEN
!CDIR NODEP
      DO JROF=KST,KPROF
        MASK_SL2(KL0(JROF,JLEV,1))=4
        MASK_SL2(KL0(JROF,JLEV,2))=4
      ENDDO
    ENDIF

  ENDDO

ENDIF

!        2.03  Coordinates and weights for 12 points interpolations.

IF (KWIS == 203) THEN

  DO JLEV=KSLEV,KFLEV+KSLEV-1
    IF (KHOR == 0) ILEV=JLEV
    IF (KHOR == 1) ILEV=KFLDN

    ! * Calculation of linear weights, KL0.
!CDIR NODEP
    DO JROF=KST,KPROF

      IJ_ = MOD(JROF+1-KST,KPDUP)+1

      IZLAT =INT(P4JP*(PIS2-PLAT(JROF,JLEV))+0.75_JPRB)-NFRSTLOFF
      ILA   =IZLAT+NINT(SIGN(0.5_JPRB,ZRLATI_(IJ_,IZLAT)-PLAT(JROF,JLEV))-1.5_JPRB)
      PDLAT(JROF,JLEV)=(PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1))&
       & /(ZRLATI_(IJ_,ILA+2)-ZRLATI_(IJ_,ILA+1))  

      ZDA   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA)
      ZDB   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+1)
      ZDC   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+2)
      ZDD   =PLAT(JROF,JLEV)-ZRLATI_(IJ_,ILA+3)

      ! for input to LASCAW_CLA.
      IILA(JROF)=ILA
      ZZWH(JROF,1,JLEV)=(ZDA*ZDC)*ZDD*ZRIPI0_(IJ_,ILA+1)
      ZZWH(JROF,2,JLEV)=(ZDA*ZDB)*ZDD*ZRIPI1_(IJ_,ILA+1)
      ZZWH(JROF,3,JLEV)=(ZDA*ZDB)*ZDC*ZRIPI2_(IJ_,ILA+1)

      ZLO   =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA  )
      ILO(JROF)   =INT(ZLO )
      PDLO(JROF,JLEV,0)=ZLO -REAL(ILO(JROF) ,JPRB)
      ZLO1  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+1)
      ILO1(JROF)  =INT(ZLO1)
      PDLO(JROF,JLEV,1)=ZLO1-REAL(ILO1(JROF),JPRB)
      ZLO2  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+2)
      ILO2(JROF)  =INT(ZLO2)
      PDLO(JROF,JLEV,2)=ZLO2-REAL(ILO2(JROF),JPRB)
      ZLO3  =PLON(JROF,JLEV)*ZLSDEPI_(IJ_,ILA+3)
      ILO3(JROF)  =INT(ZLO3)
      PDLO(JROF,JLEV,3)=ZLO3-REAL(ILO3(JROF),JPRB)

      ILO(JROF)=IADDR_(IJ_,ILA  )+NSLEXT(ILO(JROF) ,ILA  )
      ILO1(JROF)=IADDR_(IJ_,ILA+1)+NSLEXT(ILO1(JROF),ILA+1)
      ILO2(JROF)=IADDR_(IJ_,ILA+2)+NSLEXT(ILO2(JROF),ILA+2)
      ILO3(JROF)=IADDR_(IJ_,ILA+3)+NSLEXT(ILO3(JROF),ILA+3)

      KL0(JROF,JLEV,0)=ILO(JROF)+KPROMA*ILEV
      KL0(JROF,JLEV,1)=ILO1(JROF)+KPROMA*ILEV
      KL0(JROF,JLEV,2)=ILO2(JROF)+KPROMA*ILEV
      KL0(JROF,JLEV,3)=ILO3(JROF)+KPROMA*ILEV

    ENDDO

    ! * Mask calculation for on-demand communications:
    IF(NPROC > 1.AND.LSLONDEM_ACTIVE)THEN
!CDIR NODEP
      DO JROF=KST,KPROF
        MASK_SL2(KL0(JROF,JLEV,0))=4
        MASK_SL2(KL0(JROF,JLEV,1))=4
        MASK_SL2(KL0(JROF,JLEV,2))=4
        MASK_SL2(KL0(JROF,JLEV,3))=4
      ENDDO
    ENDIF

  ENDDO

  ! * Calculation of PCLA and PCLASLD:
  CALL LASCAW_CLA(KFLEV,KSLEV,KFLEV+KSLEV-1,&
   & KPROMB,KGLN,KGLX,KST,KPROF,LLT_SLHD,&
   & IILA,ZZWH,PDLAT,ZKAPPA,ZSLD1_,ZSLD2_,ZSLD3_,ZSLDW_,&
   & PCLA,PCLASLD)

  ! * Calculation of PCLO and PCLOSLD:
  CALL LASCAW_CLO(KFLEV,KSLEV,KFLEV+KSLEV-1,&
   & KPROMB,KST,KPROF,LLT_SLHD,PDLO(:,:,1),ZKAPPA,&
   & PCLO(:,:,:,1),PCLOSLD(:,:,:,1))
  CALL LASCAW_CLO(KFLEV,KSLEV,KFLEV+KSLEV-1,&
   & KPROMB,KST,KPROF,LLT_SLHD,PDLO(:,:,2),ZKAPPA,&
   & PCLO(:,:,:,2),PCLOSLD(:,:,:,2))

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LASCAW',1,ZHOOK_HANDLE)
END SUBROUTINE LASCAW
