SUBROUTINE LASCAW_CLA_AD(KFLEV,KPROM,KGLN,KGLX,KST,KPROF,LDT_SLHD,&
 & KILA,PWH,PDLAT,PKAPPA,PSLD1_,PSLD2_,PSLD3_,PSLDW_,&
 & PWH5,PDLAT5,PKAPPA5,&
 & PCLA,PCLASLD)

!     ------------------------------------------------------------------

!**** *LASCAW_CLA_AD  -  Weights for semi-LAgrangian interpolator:
!                        Computes PCLA and PCLASLD for one layer
!                        (high-order zonal weights)
!                        AD + trajectory code
!      Spherical geometry only.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *LASCAW_CLA_AD( ... )

!        Explicit arguments :
!        --------------------

!          KFLEV    - Vertical dimension
!          KPROM    - horizontal dimension.
!          KGLN     - lower dimension for latitude.
!          KGLX     - upper dimension for latitude.
!          KST      - first element of arrays where computations are performed.
!          KPROF    - depth of work.
!          LDT_SLHD - keys for SLHD.
!          KILA     - cf. ILA in LASCAW.
!          PWH      - cf. ZZWH in LASCAW.
!          PDLAT    - distance for horizontal linear interpolations in latitude
!          PKAPPA   - kappa function ("coefficient of SLHD").
!          PSLD1_   - auxiliary quantity for SLHD interpolation in latitude
!          PSLD2_   - auxiliary quantity for SLHD interpolation in latitude
!          PSLD3_   - auxiliary quantity for SLHD interpolation in latitude
!          PSLDW_   - weights for SLHD Laplacian smoother in latitude
!          PWH5     - cf. PWH (trajectory).
!          PDLAT5   - cf. PDLAT (trajectory).
!          PKAPPA5  - cf. PKAPPA (trajectory).

!          PCLA    - weights for horizontal cubic interpolations in latitude.
!          PCLASLD - cf. PCLA, SLHD case.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation about semi-Lagrangian scheme.

!     Externals.
!     ----------

!        No external.
!        Called by LASCAWAD.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD and F. VANA, after former LASCAWAD code (JAN 2009).
!        METEO-FRANCE, CNRM/GMAP.

!     Modifications.
!     --------------
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK 
USE YOMDYNA  , ONLY : SLHDKMIN ,SLHDKMAX
USE YOMLASCAW, ONLY : KPDUP

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN)    :: KFLEV
INTEGER(KIND=JPIM), INTENT(IN)    :: KPROM
INTEGER(KIND=JPIM), INTENT(IN)    :: KGLN
INTEGER(KIND=JPIM), INTENT(IN)    :: KGLX
INTEGER(KIND=JPIM), INTENT(IN)    :: KST
INTEGER(KIND=JPIM), INTENT(IN)    :: KPROF
LOGICAL           , INTENT(IN)    :: LDT_SLHD(3)
INTEGER(KIND=JPIM), INTENT(IN)    :: KILA(KPROM,KFLEV)
REAL(KIND=JPRB)   , INTENT(OUT)   :: PWH(KPROM,3,KFLEV)
REAL(KIND=JPRB)   , INTENT(INOUT) :: PDLAT(KPROM,KFLEV)
REAL(KIND=JPRB)   , INTENT(INOUT) :: PKAPPA(KPROM,KFLEV)
REAL(KIND=JPRB)   , INTENT(IN)    :: PSLD1_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB)   , INTENT(IN)    :: PSLD2_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB)   , INTENT(IN)    :: PSLD3_(KPDUP,KGLN:KGLX)
REAL(KIND=JPRB)   , INTENT(IN)    :: PSLDW_(KPDUP,3,3,KGLN:KGLX)
REAL(KIND=JPRB)   , INTENT(IN)    :: PWH5(KPROM,3,KFLEV)
REAL(KIND=JPRB)   , INTENT(IN)    :: PDLAT5(KPROM,KFLEV)
REAL(KIND=JPRB)   , INTENT(IN)    :: PKAPPA5(KPROM,KFLEV)
REAL(KIND=JPRB)   , INTENT(IN)    :: PCLA(KPROM,KFLEV,3)
REAL(KIND=JPRB)   , INTENT(IN)    :: PCLASLD(KPROM,KFLEV,3)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF,IJ_,ILA,JLEV
REAL(KIND=JPRB) :: ZWA1,ZWA2,ZWA3,ZWD1,ZWD2,ZWD3
REAL(KIND=JPRB) :: ZWDS1,ZWDS2,ZWDS3,ZWL1,ZWL2,ZWL3
REAL(KIND=JPRB) :: ZWA15,ZWA25,ZWA35,ZWD15,ZWD25,ZWD35
REAL(KIND=JPRB) :: ZWDS15,ZWDS25,ZWDS35,ZWL15,ZWL25,ZWL35
LOGICAL :: LLSLHD,LLSLHDQUAD,LLSLHD_OLD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LASCAW_CLA_AD',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

LLSLHD=LDT_SLHD(1)
LLSLHDQUAD=LDT_SLHD(2)
LLSLHD_OLD=LDT_SLHD(3)

! * AD of calculation of PCLA and PCLASLD:

IF (LLSLHD) THEN ! LLSLHD

  DO JLEV=1,KFLEV
!CDIR NODEP
  DO JROF=KST,KPROF

    ! a/ Preliminary calculations:

    IJ_ = MOD(JROF+1-KST,KPDUP)+1
    ILA=KILA(JROF,JLEV)

    ! b/ Trajectory code:

    IF (LLSLHDQUAD) THEN
      ZWL25=PDLAT5(JROF,JLEV)
      ZWL15=1.0_JPRB-ZWL25
      ZWL35=PSLD3_(IJ_,ILA+1)*ZWL15*ZWL25
      ZWL15=ZWL15+PSLD1_(IJ_,ILA+1)*ZWL35
      ZWL25=ZWL25+PSLD2_(IJ_,ILA+1)*ZWL35
    ELSEIF (LLSLHD_OLD) THEN
      ZWL25=PDLAT5(JROF,JLEV)
      ZWL15=1.0_JPRB-ZWL25
      ZWL35=0.0_JPRB
    ENDIF

    ZWA15=PWH5(JROF,1,JLEV)+SLHDKMIN*(ZWL15-PWH5(JROF,1,JLEV))
    ZWA25=PWH5(JROF,2,JLEV)+SLHDKMIN*(ZWL25-PWH5(JROF,2,JLEV))
    ZWA35=PWH5(JROF,3,JLEV)+SLHDKMIN*(ZWL35-PWH5(JROF,3,JLEV))
    ZWD15=PWH5(JROF,1,JLEV)+SLHDKMAX*(ZWL15-PWH5(JROF,1,JLEV))
    ZWD25=PWH5(JROF,2,JLEV)+SLHDKMAX*(ZWL25-PWH5(JROF,2,JLEV))
    ZWD35=PWH5(JROF,3,JLEV)+SLHDKMAX*(ZWL35-PWH5(JROF,3,JLEV))
    ZWDS15=PSLDW_(IJ_,1,1,ILA+1)*ZWD15+PSLDW_(IJ_,1,2,ILA+1)*ZWD25+ &
     & PSLDW_(IJ_,1,3,ILA+1)*ZWD35
    ZWDS25=PSLDW_(IJ_,2,1,ILA+1)*ZWD15+PSLDW_(IJ_,2,2,ILA+1)*ZWD25+ &
     & PSLDW_(IJ_,2,3,ILA+1)*ZWD35
    ZWDS35=PSLDW_(IJ_,3,1,ILA+1)*ZWD15+PSLDW_(IJ_,3,2,ILA+1)*ZWD25+ &
     & PSLDW_(IJ_,3,3,ILA+1)*ZWD35

    ! c/ AD code:

    PKAPPA(JROF,JLEV)=PKAPPA(JROF,JLEV) &
     & + (ZWDS15-ZWA15)*PCLASLD(JROF,JLEV,1) &
     & + (ZWDS25-ZWA25)*PCLASLD(JROF,JLEV,2) &
     & + (ZWDS35-ZWA35)*PCLASLD(JROF,JLEV,3)
    ZWA1=(1._JPRB-PKAPPA5(JROF,JLEV))*PCLASLD(JROF,JLEV,1)+PCLA(JROF,JLEV,1)
    ZWA2=(1._JPRB-PKAPPA5(JROF,JLEV))*PCLASLD(JROF,JLEV,2)+PCLA(JROF,JLEV,2)
    ZWA3=(1._JPRB-PKAPPA5(JROF,JLEV))*PCLASLD(JROF,JLEV,3)+PCLA(JROF,JLEV,3)
    ZWDS1=PKAPPA5(JROF,JLEV)*PCLASLD(JROF,JLEV,1)
    ZWDS2=PKAPPA5(JROF,JLEV)*PCLASLD(JROF,JLEV,2)
    ZWDS3=PKAPPA5(JROF,JLEV)*PCLASLD(JROF,JLEV,3)
    ZWD1=PSLDW_(IJ_,1,1,ILA+1)*ZWDS1+PSLDW_(IJ_,2,1,ILA+1)*ZWDS2 &
     & +PSLDW_(IJ_,3,1,ILA+1)*ZWDS3
    ZWD2=PSLDW_(IJ_,1,2,ILA+1)*ZWDS1+PSLDW_(IJ_,2,2,ILA+1)*ZWDS2 &
     & +PSLDW_(IJ_,3,2,ILA+1)*ZWDS3
    ZWD3=PSLDW_(IJ_,1,3,ILA+1)*ZWDS1+PSLDW_(IJ_,2,3,ILA+1)*ZWDS2 &
     & +PSLDW_(IJ_,3,3,ILA+1)*ZWDS3
    PWH(JROF,1,JLEV)=(1._JPRB-SLHDKMIN)*ZWA1+(1._JPRB-SLHDKMAX)*ZWD1
    PWH(JROF,2,JLEV)=(1._JPRB-SLHDKMIN)*ZWA2+(1._JPRB-SLHDKMAX)*ZWD2
    PWH(JROF,3,JLEV)=(1._JPRB-SLHDKMIN)*ZWA3+(1._JPRB-SLHDKMAX)*ZWD3
    ZWL1=SLHDKMIN*ZWA1+SLHDKMAX*ZWD1
    ZWL2=SLHDKMIN*ZWA2+SLHDKMAX*ZWD2
    ZWL3=SLHDKMIN*ZWA3+SLHDKMAX*ZWD3

    IF (LLSLHDQUAD) THEN
      ZWL3=ZWL3+PSLD1_(IJ_,ILA+1)*ZWL1+PSLD2_(IJ_,ILA+1)*ZWL2
      ZWL1=ZWL1+PSLD3_(IJ_,ILA+1)*PDLAT5(JROF,JLEV)*ZWL3
      ZWL2=ZWL2+PSLD3_(IJ_,ILA+1)*(1._JPRB-PDLAT5(JROF,JLEV))*ZWL3
      PDLAT(JROF,JLEV)=PDLAT(JROF,JLEV)-ZWL1+ZWL2
    ELSEIF (LLSLHD_OLD) THEN
      PDLAT(JROF,JLEV)=PDLAT(JROF,JLEV)-ZWL1+ZWL2
    ENDIF

  ENDDO
  ENDDO

ELSE ! .NOT.LLSLHD

  DO JLEV=1,KFLEV
!CDIR NODEP
  DO JROF=KST,KPROF

    ! a/ Preliminary calculations:
    
    IJ_ = MOD(JROF+1-KST,KPDUP)+1
    ILA=KILA(JROF,JLEV)

    ! c/ AD code:

    IF (LLSLHDQUAD) THEN
      ZWA1=PCLA(JROF,JLEV,1)
      ZWA2=PCLA(JROF,JLEV,2)
      ZWA3=PCLA(JROF,JLEV,3)
      PWH(JROF,1,JLEV)=(1._JPRB-SLHDKMIN)*ZWA1
      PWH(JROF,2,JLEV)=(1._JPRB-SLHDKMIN)*ZWA2
      PWH(JROF,3,JLEV)=(1._JPRB-SLHDKMIN)*ZWA3
      ZWL1=SLHDKMIN*ZWA1
      ZWL2=SLHDKMIN*ZWA2
      ZWL3=SLHDKMIN*ZWA3
      ZWL3=ZWL3+PSLD1_(IJ_,ILA+1)*ZWL1+PSLD2_(IJ_,ILA+1)*ZWL2
      ZWL1=ZWL1+PSLD3_(IJ_,ILA+1)*PDLAT5(JROF,JLEV)*ZWL3
      ZWL2=ZWL2+PSLD3_(IJ_,ILA+1)*(1._JPRB-PDLAT5(JROF,JLEV))*ZWL3
      PDLAT(JROF,JLEV)=PDLAT(JROF,JLEV)-ZWL1+ZWL2
    ELSE
      PWH(JROF,1,JLEV)=PCLA(JROF,JLEV,1)
      PWH(JROF,2,JLEV)=PCLA(JROF,JLEV,2)
      PWH(JROF,3,JLEV)=PCLA(JROF,JLEV,3)
    ENDIF

  ENDDO
  ENDDO

ENDIF ! Test on LLSLHD

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LASCAW_CLA_AD',1,ZHOOK_HANDLE)
END SUBROUTINE LASCAW_CLA_AD
