SUBROUTINE LASCAW_CLO(KFLEV,KLEV1,KLEV2,&
 & KPROM,KST,KPROF,LDT_SLHD,PDLO,PKAPPA,PCLO,PCLOSLD)

!     ------------------------------------------------------------------

!**** *LASCAW_CLO  -  Weights for semi-LAgrangian interpolator:
!                     Computes PCLO and PCLOSLD for one layer
!                     (high-order meridian weights)

!      Can be also called for zonal high-order weights if plane geometry
!       (no need to code a specific ELASCAW_CLA).

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *LASCAW_CLO( ... )

!        Explicit arguments :
!        --------------------

!        * INPUT:
!        KFLEV    - Vertical dimension
!        KFLEV1   - Start level
!        KFLEV2   - End level
!        KPROM    - horizontal dimension.
!        KST      - first element of arrays where computations are performed.
!        KPROF    - depth of work.
!        LDT_SLHD - keys for SLHD.
!        PDLO     - distances for horizontal linear interpolations in longitude.
!        PKAPPA   - kappa function ("coefficient of SLHD").

!        * OUTPUT:
!        PCLO     - weights for horizontal cubic interpolations in longitude.
!        PCLOSLD  - cf. PCLO, SLHD case.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation about semi-Lagrangian scheme.

!     Externals.
!     ----------

!        No external.
!        Called by some (E)LASCAW.. routines.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD, after former LASCAW code (JAN 2009).
!        METEO-FRANCE, CNRM/GMAP.

!     Modifications.
!     --------------
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK 
USE YOMDYNA  , ONLY : SLHDKMIN ,SLHDKMAX ,SLHDEPSH

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN)  :: KFLEV
INTEGER(KIND=JPIM), INTENT(IN)  :: KLEV1
INTEGER(KIND=JPIM), INTENT(IN)  :: KLEV2
INTEGER(KIND=JPIM), INTENT(IN)  :: KPROM
INTEGER(KIND=JPIM), INTENT(IN)  :: KST
INTEGER(KIND=JPIM), INTENT(IN)  :: KPROF
LOGICAL           , INTENT(IN)  :: LDT_SLHD(3)
REAL(KIND=JPRB)   , INTENT(IN)  :: PDLO(KPROM,KLEV1:KLEV2)
REAL(KIND=JPRB)   , INTENT(IN)  :: PKAPPA(KPROM,0:KFLEV)
REAL(KIND=JPRB)   , INTENT(OUT) :: PCLO(KPROM,KLEV1:KLEV2,3)
REAL(KIND=JPRB)   , INTENT(OUT) :: PCLOSLD(KPROM,KLEV1:KLEV2,3)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF,JLEV
REAL(KIND=JPRB) :: ZWA1,ZWA2,ZWA3,ZWD1,ZWD2,ZWD3,ZWH1,ZWH2,ZWH3
REAL(KIND=JPRB) :: ZWDS1,ZWDS2,ZWDS3,ZWL1,ZWL2,ZWL3
REAL(KIND=JPRB) :: Z1M2EPSH
LOGICAL :: LLSLHD,LLSLHDQUAD,LLSLHD_OLD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

REAL(KIND=JPRB),PARAMETER :: PP6_R=1.0_JPRB/6.0_JPRB
REAL(KIND=JPRB) :: PD
REAL(KIND=JPRB) :: FLAG1, FLAG2, FLAG3
REAL(KIND=JPRB) :: FQUAD1, FQUAD2, FQUAD3

! weights for cubic Lagrange interpolation (regular nodes)
FLAG1(PD)= 0.5_JPRB*(PD+1.0_JPRB)   *(PD-1.0_JPRB)*(PD-2.0_JPRB)
FLAG2(PD)=-0.5_JPRB*(PD+1.0_JPRB)*PD              *(PD-2.0_JPRB)
FLAG3(PD)= PP6_R   *(PD+1.0_JPRB)*PD*(PD-1.0_JPRB)

! weights for quadratic SLHD interpolation (regular nodes)
FQUAD1(PD)=(1.0_JPRB-PD)*(1.0_JPRB+0.25_JPRB*PD)
FQUAD2(PD)=PD*(1.25_JPRB-0.25_JPRB*PD)
FQUAD3(PD)=0.25_JPRB*PD*(PD-1.0_JPRB)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LASCAW_CLO',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

LLSLHD=LDT_SLHD(1)
LLSLHDQUAD=LDT_SLHD(2)
LLSLHD_OLD=LDT_SLHD(3)

! * Auxiliary quantity for Laplacian smoother:
Z1M2EPSH=1.0_JPRB-2.0_JPRB*SLHDEPSH

! * Calculation of PCLO and PCLOSLD:
DO JLEV=KLEV1,KLEV2
!CDIR NODEP
DO JROF=KST,KPROF

  ZWH1=FLAG1(PDLO(JROF,JLEV))
  ZWH2=FLAG2(PDLO(JROF,JLEV))
  ZWH3=FLAG3(PDLO(JROF,JLEV))
  IF (LLSLHDQUAD) THEN
    ZWL1=FQUAD1(PDLO(JROF,JLEV))
    ZWL2=FQUAD2(PDLO(JROF,JLEV))
    ZWL3=FQUAD3(PDLO(JROF,JLEV))
  ELSEIF (LLSLHD_OLD) THEN
    ZWL2=PDLO(JROF,JLEV)
    ZWL1=1.0_JPRB-ZWL2
    ZWL3=0.0_JPRB
  ENDIF

  IF (LLSLHD) THEN
    ZWA1=ZWH1+SLHDKMIN*(ZWL1-ZWH1)
    ZWA2=ZWH2+SLHDKMIN*(ZWL2-ZWH2)
    ZWA3=ZWH3+SLHDKMIN*(ZWL3-ZWH3)
    ZWD1=ZWH1+SLHDKMAX*(ZWL1-ZWH1)
    ZWD2=ZWH2+SLHDKMAX*(ZWL2-ZWH2)
    ZWD3=ZWH3+SLHDKMAX*(ZWL3-ZWH3)
    ZWDS1=Z1M2EPSH*ZWD1+SLHDEPSH*ZWD2
    ZWDS2=SLHDEPSH*ZWD1+Z1M2EPSH*ZWD2
    ZWDS3=SLHDEPSH*ZWD2+ZWD3
    PCLO(JROF,JLEV,1)=ZWA1
    PCLO(JROF,JLEV,2)=ZWA2
    PCLO(JROF,JLEV,3)=ZWA3
    PCLOSLD(JROF,JLEV,1)=ZWA1+PKAPPA(JROF,JLEV)*(ZWDS1-ZWA1)
    PCLOSLD(JROF,JLEV,2)=ZWA2+PKAPPA(JROF,JLEV)*(ZWDS2-ZWA2)
    PCLOSLD(JROF,JLEV,3)=ZWA3+PKAPPA(JROF,JLEV)*(ZWDS3-ZWA3)
  ELSEIF (LLSLHDQUAD) THEN
    ZWA1=ZWH1+SLHDKMIN*(ZWL1-ZWH1)
    ZWA2=ZWH2+SLHDKMIN*(ZWL2-ZWH2)
    ZWA3=ZWH3+SLHDKMIN*(ZWL3-ZWH3)
    PCLO(JROF,JLEV,1)=ZWA1
    PCLO(JROF,JLEV,2)=ZWA2
    PCLO(JROF,JLEV,3)=ZWA3
    PCLOSLD(JROF,JLEV,1)=ZWA1
    PCLOSLD(JROF,JLEV,2)=ZWA2
    PCLOSLD(JROF,JLEV,3)=ZWA3
  ELSE
    PCLO(JROF,JLEV,1)=ZWH1
    PCLO(JROF,JLEV,2)=ZWH2
    PCLO(JROF,JLEV,3)=ZWH3
    PCLOSLD(JROF,JLEV,1)=ZWH1
    PCLOSLD(JROF,JLEV,2)=ZWH2
    PCLOSLD(JROF,JLEV,3)=ZWH3
  ENDIF

ENDDO
ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LASCAW_CLO',1,ZHOOK_HANDLE)
END SUBROUTINE LASCAW_CLO
