SUBROUTINE LASCAW_VINTW(KLEV1,KLEV2,KPROM,KFLEV,KSLEV,KST,KPROF,LDT_SLHD,&
 & KLEV,PLEV,PDVER,PKAPPA,PVETA,PVCUICO_,PVSLD_,PVSLDW_,PVINTW,PVINTWSLD)

!     ------------------------------------------------------------------

!**** *LASCAW_VINTW  -  Weights for semi-LAgrangian interpolator:
!                       Computes PVINTW and PVINTWSLD for one layer
!                       (high-order vertical weights)

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *LASCAW_VINTW( ... )

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KFLEV1   - Start level
!          KFLEV2   - End level
!          KPROM    - horizontal dimension.
!          KFLEV    - vertical dimension.
!          KSLEV    - start vertical dimension (0 or 1).
!                     if zero, the routine is used for half level data.
!                     if one, the routine is used for full level data.
!          KST      - first element of arrays where computations are performed.
!          KPROF    - depth of work.
!          LDT_SLHD - keys for SLHD.
!          KLEV     - lower level of the vertical interpolation
!                     grid needed for vertical interpolations.
!          PLEV     - vertical coordinate of the interpolation point.
!          PDVER    - distance for vertical linear interpolation.
!          PKAPPA   - kappa function ("coefficient of SLHD").
!          PVETA    - Values of ETA (full or half level according to KSLEV).
!          PVCUICO_ - Denominators of the vertical cubic interpolation coef.
!          PVSLD_   - auxiliary quantities for vertical SLHD interpolation.
!          PVSLDW_  - weights for SLHD vertical Laplacian smoother.

!        OUTPUT:
!          PVINTW    - vertical cubic interpolation weights.
!          PVINTWSLD - vertical cubic interpolation weights, SLHD case.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation about semi-Lagrangian scheme.

!     Externals.
!     ----------

!        No external.
!        Called by some (E)LASCAW.. routines.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD, after former LASCAW code (JAN 2009).
!        METEO-FRANCE, CNRM/GMAP.

!     Modifications.
!     --------------
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK 
USE YOMDYNA  , ONLY : SLHDKMIN ,SLHDKMAX
USE YOMLASCAW, ONLY : KPDUP

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN)  :: KLEV1
INTEGER(KIND=JPIM), INTENT(IN)  :: KLEV2
INTEGER(KIND=JPIM), INTENT(IN)  :: KPROM
INTEGER(KIND=JPIM), INTENT(IN)  :: KFLEV
INTEGER(KIND=JPIM), INTENT(IN)  :: KSLEV
INTEGER(KIND=JPIM), INTENT(IN)  :: KST
INTEGER(KIND=JPIM), INTENT(IN)  :: KPROF
LOGICAL           , INTENT(IN)  :: LDT_SLHD(3)
INTEGER(KIND=JPIM), INTENT(IN)  :: KLEV(KPROM,KSLEV:KFLEV+KSLEV-1)
REAL(KIND=JPRB)   , INTENT(IN)  :: PLEV(KPROM,KSLEV:KFLEV+KSLEV-1)
REAL(KIND=JPRB)   , INTENT(IN)  :: PDVER(KPROM,KSLEV:KFLEV+KSLEV-1)
REAL(KIND=JPRB)   , INTENT(IN)  :: PKAPPA(KPROM,0:KFLEV)
REAL(KIND=JPRB)   , INTENT(IN)  :: PVETA(0:KFLEV+KSLEV)
REAL(KIND=JPRB)   , INTENT(IN)  :: PVCUICO_(KPDUP,4,0:KFLEV-KSLEV)
REAL(KIND=JPRB)   , INTENT(IN)  :: PVSLD_(KPDUP,3,0:KFLEV-KSLEV)
REAL(KIND=JPRB)   , INTENT(IN)  :: PVSLDW_(KPDUP,3,3,0:KFLEV-KSLEV)
REAL(KIND=JPRB)   , INTENT(OUT) :: PVINTW(KPROM,KSLEV:KFLEV+KSLEV-1,3)
REAL(KIND=JPRB)   , INTENT(OUT) :: PVINTWSLD(KPROM,KSLEV:KFLEV+KSLEV-1,3)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF,IJ_,ILEVV,JLEV
REAL(KIND=JPRB) :: ZWA1,ZWA2,ZWA3,ZWD1,ZWD2,ZWD3,ZWH1,ZWH2,ZWH3
REAL(KIND=JPRB) :: ZWDS1,ZWDS2,ZWDS3,ZWL1,ZWL2,ZWL3
REAL(KIND=JPRB) :: ZD0,ZD1,ZD2,ZD3
LOGICAL :: LLSLHD,LLSLHDQUAD,LLSLHD_OLD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LASCAW_VINTW',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

LLSLHD=LDT_SLHD(1)
LLSLHDQUAD=LDT_SLHD(2)
LLSLHD_OLD=LDT_SLHD(3)

! * Calculation of PVINTW and PVINTWSLD:
DO JLEV=KLEV1,KLEV2
!CDIR NODEP
DO JROF=KST,KPROF

  IJ_ = MOD(JROF+1-KST,KPDUP)+1
  ILEVV=KLEV(JROF,JLEV)
  IF (ILEVV >= KSLEV.AND.ILEVV <= KFLEV-3) THEN
    ZD0=PLEV(JROF,JLEV)-PVETA(ILEVV  )
    ZD1=PLEV(JROF,JLEV)-PVETA(ILEVV+1)
    ZD2=PLEV(JROF,JLEV)-PVETA(ILEVV+2)
    ZD3=PLEV(JROF,JLEV)-PVETA(ILEVV+3)
    ZWH1=ZD0    *ZD2*ZD3*PVCUICO_(IJ_,2,ILEVV)
    ZWH2=ZD0*ZD1    *ZD3*PVCUICO_(IJ_,3,ILEVV)
    ZWH3=ZD0*ZD1*ZD2    *PVCUICO_(IJ_,4,ILEVV)
    IF (LLSLHDQUAD) THEN
      ZWL2=PDVER(JROF,JLEV)
      ZWL1=1.0_JPRB-ZWL2
      ZWL3=PVSLD_(IJ_,3,ILEVV)*ZWL1*ZWL2
      ZWL1=ZWL1+PVSLD_(IJ_,1,ILEVV)*ZWL3
      ZWL2=ZWL2+PVSLD_(IJ_,2,ILEVV)*ZWL3
    ELSEIF (LLSLHD_OLD) THEN
      ZWL2=PDVER(JROF,JLEV)
      ZWL1=1.0_JPRB-ZWL2
      ZWL3=0.0_JPRB
    ENDIF
    IF (LLSLHD) THEN
      ZWA1=ZWH1+SLHDKMIN*(ZWL1-ZWH1)
      ZWA2=ZWH2+SLHDKMIN*(ZWL2-ZWH2)
      ZWA3=ZWH3+SLHDKMIN*(ZWL3-ZWH3)
      ZWD1=ZWH1+SLHDKMAX*(ZWL1-ZWH1)
      ZWD2=ZWH2+SLHDKMAX*(ZWL2-ZWH2)
      ZWD3=ZWH3+SLHDKMAX*(ZWL3-ZWH3)
      ZWDS1=PVSLDW_(IJ_,1,1,ILEVV)*ZWD1+PVSLDW_(IJ_,1,2,ILEVV)*ZWD2+ &
       &    PVSLDW_(IJ_,1,3,ILEVV)*ZWD3
      ZWDS2=PVSLDW_(IJ_,2,1,ILEVV)*ZWD1+PVSLDW_(IJ_,2,2,ILEVV)*ZWD2+ &
       &    PVSLDW_(IJ_,2,3,ILEVV)*ZWD3
      ZWDS3=PVSLDW_(IJ_,3,1,ILEVV)*ZWD1+PVSLDW_(IJ_,3,2,ILEVV)*ZWD2+ &
       &    PVSLDW_(IJ_,3,3,ILEVV)*ZWD3
      PVINTW(JROF,JLEV,1)=ZWA1
      PVINTW(JROF,JLEV,2)=ZWA2
      PVINTW(JROF,JLEV,3)=ZWA3
      PVINTWSLD(JROF,JLEV,1)=ZWA1+PKAPPA(JROF,JLEV)*(ZWDS1-ZWA1)
      PVINTWSLD(JROF,JLEV,2)=ZWA2+PKAPPA(JROF,JLEV)*(ZWDS2-ZWA2)
      PVINTWSLD(JROF,JLEV,3)=ZWA3+PKAPPA(JROF,JLEV)*(ZWDS3-ZWA3)
    ELSEIF (LLSLHDQUAD) THEN
      ZWA1=ZWH1+SLHDKMIN*(ZWL1-ZWH1)
      ZWA2=ZWH2+SLHDKMIN*(ZWL2-ZWH2)
      ZWA3=ZWH3+SLHDKMIN*(ZWL3-ZWH3)
      PVINTW(JROF,JLEV,1)=ZWA1
      PVINTW(JROF,JLEV,2)=ZWA2
      PVINTW(JROF,JLEV,3)=ZWA3
      PVINTWSLD(JROF,JLEV,1)=ZWA1
      PVINTWSLD(JROF,JLEV,2)=ZWA2
      PVINTWSLD(JROF,JLEV,3)=ZWA3
    ELSE
      PVINTW(JROF,JLEV,1)=ZWH1
      PVINTW(JROF,JLEV,2)=ZWH2
      PVINTW(JROF,JLEV,3)=ZWH3
      PVINTWSLD(JROF,JLEV,1)=ZWH1
      PVINTWSLD(JROF,JLEV,2)=ZWH2
      PVINTWSLD(JROF,JLEV,3)=ZWH3
    ENDIF
  ELSE
    PVINTW(JROF,JLEV,1)=1.0_JPRB-PDVER(JROF,JLEV)
    PVINTW(JROF,JLEV,2)=PDVER(JROF,JLEV)
    PVINTW(JROF,JLEV,3)=0.0_JPRB
    PVINTWSLD(JROF,JLEV,1)=PVINTW(JROF,JLEV,1)
    PVINTWSLD(JROF,JLEV,2)=PVINTW(JROF,JLEV,2)
    PVINTWSLD(JROF,JLEV,3)=PVINTW(JROF,JLEV,3)
  ENDIF

ENDDO
ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LASCAW_VINTW',1,ZHOOK_HANDLE)
END SUBROUTINE LASCAW_VINTW
