SUBROUTINE LASURE(&
 ! ----- INPUT ---------------------------------------------------------------
 & CDCONF,KSTART,KPROF,PDT,PGM,&
 ! ----- OUTPUT --------------------------------------------------------------
 & PDTS2,PBETA,PBT,LD2TLFF1,PBDT,PREDIV,PESGP,PESGM )  

!**** *LASURE*   Semi-Lagrangian scheme.
!                Set-up for other subroutines called by LACDYN.

!     Purpose.
!     --------
!       Computes some intermediate quantities necessary in the other
!       subroutines called by LACDYN.

!**   Interface.
!     ----------
!        *CALL* *LASURE(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          CDCONF  : configuration.
!          KSTART  : first element of work.
!          KPROF   : depth of work.
!          PDT     : time step for the first time-integration step of
!                    a leap-frog scheme or all time-integration steps of
!                    a two-time level scheme; 2*time step for the following
!                    time-integration steps of a leap-frog scheme.
!          PGM     : mapping factor.

!        OUTPUT:
!          PDTS2   : 0.5*PDT.
!          PBETA   : BETADT or zero according to the value of CDCONF.
!          PBT     : PDTS2*PBETA.
!          LD2TLFF1: .T./.F.: Refined treatement of (2*Omega Vec r) at
!                    the origin point when there is t-dt (or t in SL2TL)
!                    physics / Other cases.
!          PBDT    : PBT or PBT*(c**2/GM**2) according to LSIDG.
!          PREDIV  : 1. or c**2/GM**2 according to LSIDG.
!          PESGP   : (1 + uncentering factor).
!          PESGM   : (1 - uncentering factor).

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation about semi-lagrangian scheme.

!     Externals.
!     ----------
!           none. 
!           Called by LACDYN, LACDYNTL and LACDYNAD.

!     Reference.
!     ----------
!             Arpege documentation about semi-lagrangian scheme.

!     Author.
!     -------
!        K. YESSAD (METEO FRANCE/CNRM/GMAP) after old part one of LACDYN. 
!        Loops are rewritten according to F90 norms.

!     Modifications.
!     --------------
!        Original : JULY 1995.
!        Clive Temperton 96-03-06 : Time-filtering removed from 2TL scheme
!        Modified OCT 1996 by K. YESSAD: removal of variable uncentering.
!        K. YESSAD (July 1998): cleaning
!         - removal of options NODEC=2, L2TLAW=T, L2TLAT=T
!         - merge of VESLA and VESL into VESL
!         - declare all dummy arguments
!        Clive Temperton 99-08-16 : remove redundant LVADL
!        K. YESSAD 99-11-18: remove useless dummy arguments.
!        Modified Apr 2000 by M. Szczech: computation of "ldadvo3".
!        Modified Oct 2000 by M. Szczech: no LDADVAMV test for ozon.
!        Clive Temperton 00-11-06 : bugfix for ozone advection
!        R. El Khatib : 01-08-07 Pruning options
!        J.Vivoda 03-2002 PC schemes for NH dynamics (LPC_XXXX keys)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad Aug 2008: rationalisation of dummy argument interfaces
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEPHY    ,ONLY : LEPHYS   ,LAGPHY
USE YOMPHY    ,ONLY : LMPHYS
USE YOMDIM    ,ONLY : NPROMA
USE YOMDYN    ,ONLY : BETADT   ,LSIDG    ,VESL     ,L2TLFF
USE YOMGEM    ,ONLY : RSTRET

!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDTS2 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBETA 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBT 
LOGICAL           ,INTENT(OUT)   :: LD2TLFF1 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBDT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PREDIV(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PESGP 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PESGM 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) ::  JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LASURE',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY INITIALISATIONS:
!              ----------------------------

!     * Time step.

PDTS2 = 0.5_JPRB*PDT
PBETA = 0.0_JPRB
IF (CDCONF == 'A'.OR.CDCONF == 'B'.OR.CDCONF == 'S') THEN
  PBETA = BETADT
ELSEIF (CDCONF == 'E'.OR.CDCONF == 'F'.OR.CDCONF == 'M') THEN
  PBETA = 0.0_JPRB
ENDIF
PBT = PDTS2*PBETA

IF (LSIDG) THEN
  DO JROF=KSTART,KPROF
    PBDT(JROF)  = PBT
    PREDIV(JROF)= 1.0_JPRB
  ENDDO
ELSE
  DO JROF=KSTART,KPROF
    PBDT(JROF)  = PBT*RSTRET*RSTRET/(PGM(JROF)*PGM(JROF))
    PREDIV(JROF)= RSTRET*RSTRET/(PGM(JROF)*PGM(JROF))
  ENDDO
ENDIF

!     * ( 1 + uncentering factor ) and other quantities linked with
!       uncentering factor.

PESGP = 1.0_JPRB + VESL
PESGM = 1.0_JPRB - VESL

!     * Switch to activate two additional interpolations for refined
!       computation of Coriolis term.

LD2TLFF1=L2TLFF.AND.(LMPHYS.OR.LEPHYS).AND.(.NOT.LAGPHY)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LASURE',1,ZHOOK_HANDLE)
END SUBROUTINE LASURE
