SUBROUTINE LATTE_KAPPA(KST,KPROF,PDTS2,PSLHDA,PSLHDD0,PGMV,PGMVS, &
 & PLHU0,PLHV0,PRES0,PRDELP,PB2)

!**** *LATTE_KAPPA* input for SLHD diffusion.
!                   Computation of the kappa function ("coefficient
!                   of diffusion") based on rescaled horizontal
!                   deformation of the flow to be evaluated at F, and put
!                   it in PB2.
!                   Used by SLHD scheme.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *LATTE_KAPPA(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of work.
!          KPROF   - depth of work.
!          PDTS2   - 0.5*"pdt", where "pdt" =
!                    time step for the first time-integration step of
!                    a leap-frog scheme or all time-integration steps of
!                    a two-time level scheme; 2*time step for the following
!                    time-integration steps of a leap-frog scheme.
!          PSLHDA  - Scaling factor of the deformation in f(d) function
!                    (including the model resolution correction)
!          PSLHDD0 - Treshold for deformation tensor enhancement
!          PGMV    - GMV variables at t-dt and t.
!          PGMVS   - GMVS variables at t-dt and t.
!          PLHU0   - zonal wind time t at half levels.
!          PLHV0   - meridian wind time t at half levels.
!          PRES0   - prehyds at t.
!          PRDELP  - 1/(pressure depth of layers) at t.

!        INPUT/OUTPUT:
!          PB2     - "SLBUF2" buffer.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        Called by LACDYN.

!     Reference.
!     ----------
!        Arpege documentation about semi-Lagrangian scheme.
!        Arpege documentation about horizontal diffusion.

!     Author.
!     -------
!        Original (F. VANA and K. YESSAD): AUGUST 2003.

!     Modifications.
!     --------------
!        Modified 03-08-19 by F. Vana : horizontal flow deformation for SLHD
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        F. Vana  04-10-25 : New content of output + DFI fix -> renamed
!        F. Vana  06-06-08 : SLHDA,SLHDD0 changed to arays P...
!        K. Yessad Aug 2008: rationalisation of dummy argument interfaces
!         + put calculation of Kappa in GP_KAPPA.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPROMA   ,NFLEVG
USE YOMGMV   , ONLY : NDIMGMV  ,NDIMGMVS ,YT0
USE YOMSC2   , ONLY : NFLDSLB2
USE PTRSLB2  , ONLY : MSLB2KAPPA

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLHDA(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLHDD0(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVS(NPROMA,NDIMGMVS)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLHU0(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLHV0(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES0(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2)

!     -------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV
REAL(KIND=JPRB) :: ZKAPPA(NPROMA,NFLEVG) 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -------------------------------------------------------

#include "gp_kappa.intfb.h"

!     -------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTE_KAPPA',0,ZHOOK_HANDLE)

!    --------------------------------------------------------

!*       1.   Kappa function computation

CALL GP_KAPPA(NPROMA,KST,KPROF,NFLEVG,PDTS2,&
 & PGMV(1,1,YT0%MUL),PGMV(1,1,YT0%MVL),PGMV(1,1,YT0%MVOR),PGMV(1,1,YT0%MDIV),&
 & PLHU0,PLHV0,PRES0,PGMVS(1,YT0%MSPL),PGMVS(1,YT0%MSPM),PRDELP,&
 & PSLHDA,PSLHDD0,ZKAPPA)

!*       2.   Fill PB2 with Kappa

DO JLEV=1,NFLEVG
  PB2(KST:KPROF,MSLB2KAPPA+JLEV-1)=ZKAPPA(KST:KPROF,JLEV)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTE_KAPPA',1,ZHOOK_HANDLE)
END SUBROUTINE LATTE_KAPPA

