SUBROUTINE LATTESAD(KST,KPROF,PDTS2,PBDT,PESGP,PESGM,&
 & POROGL,POROGM,PSDIV0,PEVEL,PDELP0,PRES0,PGMVS,&
 & PGMV,PGMVT1S,PB1,PB2, &
 & PEVEL5,PDELP5,PRES5,PGMV5,PGMV5S)

!**** *LATTESAD* Semi-Lagrangian scheme.  (adjoint version)
!                Computation of the t and t-dt useful quantities
!                 at grid-points. Equations for bi-dimensional
!                 variables (continuity equation).

!     Purpose.
!     --------
!        * This subroutine computes the equation quantities to be
!          interpolated at each grid-point of the colocation grid.
!          The terms considered here are the explicit terms and
!          the explicit part of the semi-implicit scheme (terms
!          previously computed in LASSIE or LANHSI).
!          Equations considered here are equations for bi-dimensional
!          variables: continuity equation.
!        * Remark 1: when an alternate averaging is used for linear terms
!          in the 2TL SL scheme, the first timestep is treated differently
!          (first order uncentering), no uncentering is applied to the
!          total term ("cursive A") and the term saved in P[X]NLT9
!          is [ (Delta t/2) ("cursive A" - (1 + xidt) beta "cursive B") ]
!          instead of [ (Delta t/2) ("cursive A" - beta "cursive B") ].
!        * Remark 2: for lsettls=true, uncentering is applied to
!          the 'stable' extrapolation if vesl > 0 to avoid instability
!          in the momentum equation.
!        * Remark 3: for computation of ZSPTB and ZSPTB5, update matching
!          the direct code (use PSDVBC and no recomputation of the
!          divergence integral term here) has to be done in the future.

!        * Additional remark: some options coded in the TL are not yet coded
!          in the AD, but the corresponding dummy
!          arguments have been kept even if they are currently useless.

!**   Interface.
!     ----------
!        *CALL* *LATTESAD(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of work.
!          KPROF   - depth of work.
!          PDTS2   - 0.5*"pdt", where "pdt"=
!                    time step for the first time-integration step of
!                    a leap-frog scheme or all time-integration steps of
!                    a two-time level scheme; 2*time step for the following
!                    time-integration steps of a leap-frog scheme.
!          PBDT    - zbt if semi-implicit scheme with unreduced
!                    divergence, zbt*(c**2/GM**2) if semi-implicit
!                    scheme with reduced divergence, where zbt
!                    is equal to PDTS2*BETADT (BETADT is in YOMDYN) or zero
!                    according to the value of CDCONF.
!          PESGP   - (1 + uncentering factor).
!          PESGM   - (1 - uncentering factor).
!          POROGL  - Zonal derivative of surface orography.
!          POROGM  - Meridian derivative of surface orography.

!        INPUT/OUTPUT:
!          PSDIV0  - SI term at time t for continuity equation (Nu*D).
!          PEVEL   - "etadot * d prehyd/d eta" at t.
!          PDELP0  - pressure depths (Delta prehyd) at full levels at t.
!          PRES0   - hydrostatic pressure at half levels at t.
!          PGMVS   - GMVS variables at t-dt and t.
!          PGMV    - GMV variables at t-dt and t.
!          PGMVT1S - GMVS variables at t+dt.
!          PB1     - "SLBUF1" buffer for interpolations.
!          PB2     - "SLBUF2" buffer.

!        INPUT-TRAJECTORY:
!          PEVEL5  - "etadot * d prehyd/d eta" at t.
!          PDELP5  - pressure depths (Delta prehyd) at full levels at t.
!          PRES5   - hydrostatic pressure at half levels at t.
!          PGMV5,PGMV5S: trajectory for PGMV,PGMVS.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by LACDYNAD.

!     Reference.
!     ----------
!             Arpege documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        C. Temperton (ECMWF)

!     Modifications.
!     --------------
!        Original : 99/10/12.
!        A. Untch 2000-12: modifications for finite element scheme 
!                          in the vertical (LVERTFE)
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad Aug 2008: rationalisation of dummy argument interfaces
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RD
USE YOMCT0   , ONLY : LTWOTL
USE YOMCT3   , ONLY : NSTEP
USE YOMCVER  , ONLY : LVERTFE
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG   ,NFLSA
USE YOMDYN   , ONLY : NVLAG    ,XIDT     ,RCMSLP0  ,LSETTLS
USE YOMGEM   , ONLY : VFE_RDETAH  ,VDELB
USE YOMGMV   , ONLY : NDIMGMV  ,NDIMGMVS ,YT0      ,YT1      ,YT9      ,YT5
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2
USE YOMSTA   , ONLY : RTSUR
USE PTRSLB1  , ONLY : MSLB1SP9 ,MSLB1C9
USE PTRSLB2  , ONLY : MSLB2SPSI

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBDT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSDIV0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVEL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDELP0(NPROMA,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRES0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVS(NPROMA,NDIMGMVS)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1S(NPROMA,YT1%NDIMS)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(NPROMA,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV5(NPROMA,NFLEVG,YT5%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV5S(NPROMA,YT5%NDIMS)

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZMOY1SP(NPROMA)
REAL(KIND=JPRB) :: ZSPNLT0(NPROMA)
REAL(KIND=JPRB) :: ZSPNLT0FE(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSPT1(NPROMA,NFLEVG+1)
REAL(KIND=JPRB) :: ZMOYSPNL(NPROMA)
REAL(KIND=JPRB) :: ZSPTB(NPROMA)
REAL(KIND=JPRB) :: ZSPTBFE(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSPTBOUT(NPROMA,NFLEVG+1)
REAL(KIND=JPRB) :: ZUSPS0(NPROMA)
REAL(KIND=JPRB) :: ZUSPS5(NPROMA)

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZCMSLP, ZXIGM, ZXIGP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "verintad.intfb.h"

!     ------------------------------------------------------------------

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('LATTESAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       *   Preliminary initialisations.

ZSPTB(KST:KPROF)=0.0_JPRB
ZUSPS0(KST:KPROF)=0.0_JPRB

DO JROF=KST,KPROF
  ZUSPS5(JROF)=1.0_JPRB/PRES5(JROF,NFLEVG)
ENDDO
ZCMSLP=RCMSLP0/(RD*RTSUR)

IF (NSTEP <= 0) THEN
  ZXIGM=(1.0_JPRB-XIDT)
  ZXIGP=(1.0_JPRB+XIDT)
ELSE
  ZXIGM=(1.0_JPRB+XIDT)
  ZXIGP=(1.0_JPRB+XIDT)
ENDIF

!     ------------------------------------------------------------------

!*       *   Continuity equation.

IF (LTWOTL) THEN

! * Remark: the computation of "ZSPTB" matches with an old
!   version of the direct code which has been optimised since a couple of
!   cycles (use input array "PSDVBC" in the direct code).

  IF (NVLAG == 2) THEN   !!! TL/AD NOT DONE !!!

  ELSEIF (NVLAG == 3) THEN

    ! * compute perturbation terms

    IF (XIDT > 0.0_JPRB) THEN
      DO JROF=KST,KPROF
        PB2(JROF,MSLB2SPSI)=ZXIGP*PB2(JROF,MSLB2SPSI)
        PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)-ZXIGP*PGMVT1S(JROF,YT1%MSP)
        PGMVS(JROF,YT0%MSP)=PGMVS(JROF,YT0%MSP)+PB1(JROF,MSLB1SP9)
        PB1(JROF,MSLB1SP9)=0.0_JPRB
      ENDDO
    ELSE
      DO JROF=KST,KPROF
        PB2(JROF,MSLB2SPSI)=PESGP*PB2(JROF,MSLB2SPSI)
        PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)-PESGP*PGMVT1S(JROF,YT1%MSP)
        PGMVS(JROF,YT0%MSP)=PGMVS(JROF,YT0%MSP)+PB1(JROF,MSLB1SP9)
        PB1(JROF,MSLB1SP9)=0.0_JPRB
      ENDDO
    ENDIF

    IF(LVERTFE) THEN
      ZSPT1(:,:)=0.0_JPRB
      ZSPT1(KST:KPROF,NFLEVG+1)=PGMVT1S(KST:KPROF,YT1%MSP)
      PGMVT1S(KST:KPROF,YT1%MSP)=0.0_JPRB
      CALL VERINTAD(NPROMA,KST,KPROF,NFLEVG,ZSPNLT0FE,ZSPT1,0)
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB2(JROF,MSLB2SPSI)=&
           & PB2(JROF,MSLB2SPSI)-PESGM*PB1(JROF,MSLB1C9+JLEV-NFLSA)  
          ZMOYSPNL(JROF)=PB1(JROF,MSLB1C9+JLEV-NFLSA)
          PB2(JROF,MSLB2SPSI)=&
           & PB2(JROF,MSLB2SPSI)+PGMV(JROF,JLEV,YT9%MSPNL)  
          ZMOY1SP(JROF)=PGMV(JROF,JLEV,YT9%MSPNL)
          PGMV(JROF,JLEV,YT9%MSPNL)=0.0_JPRB
          PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)&
           & +ZSPNLT0FE(JROF,JLEV)*PESGP*VDELB(JLEV)*VFE_RDETAH(JLEV)  
          ZMOY1SP(JROF)=ZMOY1SP(JROF)&
           & +ZSPNLT0FE(JROF,JLEV)*PESGP*VDELB(JLEV)*VFE_RDETAH(JLEV)  
        ENDDO
        IF (NSTEP <= 0) THEN
          DO JROF=KST,KPROF
            PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)+ZMOYSPNL(JROF)  
            ZMOY1SP(JROF)=ZMOY1SP(JROF)+ZMOYSPNL(JROF)  
          ENDDO
        ELSE
          DO JROF=KST,KPROF
            PGMV(JROF,JLEV,YT9%MSPNL)=PGMV(JROF,JLEV,YT9%MSPNL)-ZMOYSPNL(JROF)  
            PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)&
             & +(1.0_JPRB+PESGM)*ZMOYSPNL(JROF)  
            ZMOY1SP(JROF)=ZMOY1SP(JROF)+(1.0_JPRB+PESGM)*ZMOYSPNL(JROF)  
          ENDDO
        ENDIF
        DO JROF=KST,KPROF
          ZSPTB(JROF)=ZSPTB(JROF)+ZMOY1SP(JROF)
          PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU)+PDTS2*&
           & (PGMV5S(JROF,YT5%MSPL)+ZCMSLP*POROGL(JROF))*ZMOY1SP(JROF)  
          PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV)+PDTS2*&
           & (PGMV5S(JROF,YT5%MSPM)+ZCMSLP*POROGM(JROF))*ZMOY1SP(JROF)  
          PGMVS(JROF,YT0%MSPL)=PGMVS(JROF,YT0%MSPL)+PDTS2*&
           & PGMV5(JROF,JLEV,YT5%MU)*ZMOY1SP(JROF)  
          PGMVS(JROF,YT0%MSPM)=PGMVS(JROF,YT0%MSPM)+PDTS2*&
           & PGMV5(JROF,JLEV,YT5%MV)*ZMOY1SP(JROF)  
        ENDDO
      ENDDO

    ELSE

      DO JLEV=1,NFLEVG

        IF (XIDT > 0.0_JPRB) THEN
          IF (LSETTLS) THEN
            DO JROF=KST,KPROF
              ZSPNLT0(JROF)=VDELB(JLEV)*PGMVT1S(JROF,YT1%MSP)
              PB2(JROF,MSLB2SPSI)=&
               & PB2(JROF,MSLB2SPSI)-ZXIGM*PB1(JROF,MSLB1C9+JLEV-NFLSA)  
              ZMOYSPNL(JROF)=PB1(JROF,MSLB1C9+JLEV-NFLSA)
            ENDDO
          ELSE
            DO JROF=KST,KPROF
              ZSPNLT0(JROF)=0.0_JPRB
              ZMOYSPNL(JROF)=VDELB(JLEV)*PGMVT1S(JROF,YT1%MSP)
              PB2(JROF,MSLB2SPSI)=&
               & PB2(JROF,MSLB2SPSI)-ZXIGM*PB1(JROF,MSLB1C9+JLEV-NFLSA)  
              ZMOYSPNL(JROF)=ZMOYSPNL(JROF)+PB1(JROF,MSLB1C9+JLEV-NFLSA)  
            ENDDO
          ENDIF
          DO JROF=KST,KPROF
            PB2(JROF,MSLB2SPSI)=&
             & PB2(JROF,MSLB2SPSI)+(1.0_JPRB+XIDT)*PGMV(JROF,JLEV,YT9%MSPNL)  
            ZMOY1SP(JROF)=PGMV(JROF,JLEV,YT9%MSPNL)
            PGMV(JROF,JLEV,YT9%MSPNL)=0.0_JPRB
          ENDDO
        ELSE
          IF (LSETTLS) THEN
            DO JROF=KST,KPROF
              ZSPNLT0(JROF)=PESGP*VDELB(JLEV)*PGMVT1S(JROF,YT1%MSP)  
              PB2(JROF,MSLB2SPSI)=&
               & PB2(JROF,MSLB2SPSI)-PESGM*PB1(JROF,MSLB1C9+JLEV-NFLSA)  
              ZMOYSPNL(JROF)=PB1(JROF,MSLB1C9+JLEV-NFLSA)
            ENDDO
          ELSE
            DO JROF=KST,KPROF
              ZSPNLT0(JROF)=0.0_JPRB
              ZMOYSPNL(JROF)=PESGP*VDELB(JLEV)*PGMVT1S(JROF,YT1%MSP)  
              PB2(JROF,MSLB2SPSI)=&
               & PB2(JROF,MSLB2SPSI)-PESGM*PB1(JROF,MSLB1C9+JLEV-NFLSA)  
              ZMOYSPNL(JROF)=ZMOYSPNL(JROF)+PESGM*PB1(JROF,MSLB1C9+JLEV-NFLSA)
            ENDDO
          ENDIF
          DO JROF=KST,KPROF
            PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)+PGMV(JROF,JLEV,YT9%MSPNL)
            ZMOY1SP(JROF)=PGMV(JROF,JLEV,YT9%MSPNL)
            PGMV(JROF,JLEV,YT9%MSPNL)=0.0_JPRB
          ENDDO
        ENDIF

        IF (NSTEP <= 0) THEN
          DO JROF=KST,KPROF
            PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)&
             & +ZSPNLT0(JROF)+ZMOYSPNL(JROF)  
            ZMOY1SP(JROF)=ZMOY1SP(JROF)+ZSPNLT0(JROF)+ZMOYSPNL(JROF)  
          ENDDO
        ELSE
          IF (XIDT > 0.0_JPRB) THEN
            IF (LSETTLS) THEN
              DO JROF=KST,KPROF
                PB2(JROF,MSLB2SPSI)=&
                 & PB2(JROF,MSLB2SPSI)+(1.0_JPRB+XIDT)*ZSPNLT0(JROF)  
                ZMOY1SP(JROF)=ZMOY1SP(JROF)+ZSPNLT0(JROF)  
                PGMV(JROF,JLEV,YT9%MSPNL)=&
                 & PGMV(JROF,JLEV,YT9%MSPNL)-ZMOYSPNL(JROF)  
                PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)&
                 & +2.0_JPRB*(1.0_JPRB+XIDT)*ZMOYSPNL(JROF)  
                ZMOY1SP(JROF)=ZMOY1SP(JROF)+2.0_JPRB*ZMOYSPNL(JROF)  
              ENDDO
            ELSE
              DO JROF=KST,KPROF
                PGMV(JROF,JLEV,YT9%MSPNL)=&
                 & PGMV(JROF,JLEV,YT9%MSPNL)-0.5_JPRB*ZMOYSPNL(JROF)  
                PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)&
                 & +1.5_JPRB*(1.0_JPRB+XIDT)*ZMOYSPNL(JROF)  
                ZMOY1SP(JROF)=ZMOY1SP(JROF)+1.5_JPRB*ZMOYSPNL(JROF)  
              ENDDO
            ENDIF
          ELSE
            IF (LSETTLS) THEN
              DO JROF=KST,KPROF
                PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)+ZSPNLT0(JROF)  
                ZMOY1SP(JROF)=ZMOY1SP(JROF)+ZSPNLT0(JROF)  
                PGMV(JROF,JLEV,YT9%MSPNL)=&
                 & PGMV(JROF,JLEV,YT9%MSPNL)-ZMOYSPNL(JROF)  
                PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)&
                 & +(1.0_JPRB+PESGM)*ZMOYSPNL(JROF)  
                ZMOY1SP(JROF)=ZMOY1SP(JROF)+(1.0_JPRB+PESGM)*ZMOYSPNL(JROF)  
              ENDDO
            ELSE
              DO JROF=KST,KPROF
                PGMV(JROF,JLEV,YT9%MSPNL)=&
                 & PGMV(JROF,JLEV,YT9%MSPNL)-0.5_JPRB*ZMOYSPNL(JROF)  
                PB2(JROF,MSLB2SPSI)=PB2(JROF,MSLB2SPSI)&
                 & +1.5_JPRB*ZMOYSPNL(JROF)  
                ZMOY1SP(JROF)=ZMOY1SP(JROF)+1.5_JPRB*ZMOYSPNL(JROF)  
              ENDDO
            ENDIF
          ENDIF
        ENDIF

        DO JROF=KST,KPROF
          ZSPTB(JROF)=ZSPTB(JROF)+ZMOY1SP(JROF)
          PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU)+PDTS2*&
           & (PGMV5S(JROF,YT5%MSPL)+ZCMSLP*POROGL(JROF))*ZMOY1SP(JROF)  
          PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV)+PDTS2*&
           & (PGMV5S(JROF,YT5%MSPM)+ZCMSLP*POROGM(JROF))*ZMOY1SP(JROF)  
          PGMVS(JROF,YT0%MSPL)=PGMVS(JROF,YT0%MSPL)+PDTS2*&
           & PGMV5(JROF,JLEV,YT5%MU)*ZMOY1SP(JROF)  
          PGMVS(JROF,YT0%MSPM)=PGMVS(JROF,YT0%MSPM)+PDTS2*&
           & PGMV5(JROF,JLEV,YT5%MV)*ZMOY1SP(JROF)  
        ENDDO

      ENDDO

    ENDIF

  ENDIF

  IF(LVERTFE) THEN
    DO JROF=KST,KPROF
      PSDIV0(JROF)=PSDIV0(JROF)+PBDT(JROF)*PB2(JROF,MSLB2SPSI)
    ENDDO
    ZSPTBOUT(KST:KPROF,:)=0.0_JPRB
    ZSPTBOUT(KST:KPROF,NFLEVG+1)=ZSPTB(KST:KPROF)
    CALL VERINTAD(NPROMA,KST,KPROF,NFLEVG,ZSPTBFE,ZSPTBOUT,0)
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PGMVS(JROF,YT0%MSPM)=PGMVS(JROF,YT0%MSPM)-PDTS2*VFE_RDETAH(JLEV)*&
         & PGMV5(JROF,JLEV,YT5%MV)*VDELB(JLEV)*ZSPTBFE(JROF,JLEV)  
        PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV)-PDTS2*VFE_RDETAH(JLEV)*&
         & ZSPTBFE(JROF,JLEV)*VDELB(JLEV)*PGMV5S(JROF,YT5%MSPM)  
        PGMVS(JROF,YT0%MSPL)=PGMVS(JROF,YT0%MSPL)-PDTS2*VFE_RDETAH(JLEV)*&
         & PGMV5(JROF,JLEV,YT5%MU)*VDELB(JLEV)*ZSPTBFE(JROF,JLEV)  
        PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU)-PDTS2*VFE_RDETAH(JLEV)*&
         & ZSPTBFE(JROF,JLEV)*VDELB(JLEV)*PGMV5S(JROF,YT5%MSPL)  
        ZUSPS0(JROF)=ZUSPS0(JROF)-PDTS2*VFE_RDETAH(JLEV)*&
         & PDELP5(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MDIV)*ZSPTBFE(JROF,JLEV)  
        PGMV(JROF,JLEV,YT0%MDIV)=PGMV(JROF,JLEV,YT0%MDIV)-PDTS2*VFE_RDETAH(JLEV)*&
         & PDELP5(JROF,JLEV)*ZSPTBFE(JROF,JLEV)*ZUSPS5(JROF)  
        PDELP0(JROF,JLEV)=PDELP0(JROF,JLEV)-PDTS2*VFE_RDETAH(JLEV)*&
         & ZSPTBFE(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MDIV)*ZUSPS5(JROF)  
      ENDDO
    ENDDO

  ELSE

    DO JROF=KST,KPROF
      PSDIV0(JROF)=PSDIV0(JROF)+PBDT(JROF)*PB2(JROF,MSLB2SPSI)
      ZUSPS0(JROF)=ZUSPS0(JROF)-PDTS2*PEVEL5(JROF,NFLEVG)*ZSPTB(JROF)
      PEVEL(JROF,NFLEVG)=PEVEL(JROF,NFLEVG)-PDTS2*ZUSPS5(JROF)*ZSPTB(JROF)
      ZUSPS0(JROF)=ZUSPS0(JROF)+PDTS2*PEVEL5(JROF,0)*ZSPTB(JROF)
      PEVEL(JROF,0)=PEVEL(JROF,0)+PDTS2*ZUSPS5(JROF)*ZSPTB(JROF)
    ENDDO

    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PDELP0(JROF,JLEV)=PDELP0(JROF,JLEV)-PDTS2*&
         & PGMV5(JROF,JLEV,YT5%MDIV)*ZUSPS5(JROF)*ZSPTB(JROF)  
        PGMV(JROF,JLEV,YT0%MDIV)=PGMV(JROF,JLEV,YT0%MDIV)-PDTS2*&
         & PDELP5(JROF,JLEV)*ZUSPS5(JROF)*ZSPTB(JROF)  
        ZUSPS0(JROF)=ZUSPS0(JROF)-PDTS2*&
         & PDELP5(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MDIV)*ZSPTB(JROF)  
        PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU)-PDTS2*&
         & VDELB(JLEV)*PGMV5S(JROF,YT5%MSPL)*ZSPTB(JROF)  
        PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV)-PDTS2*&
         & VDELB(JLEV)*PGMV5S(JROF,YT5%MSPM)*ZSPTB(JROF)  
        PGMVS(JROF,YT0%MSPL)=PGMVS(JROF,YT0%MSPL)-PDTS2*&
         & PGMV5(JROF,JLEV,YT5%MU)*VDELB(JLEV)*ZSPTB(JROF)  
        PGMVS(JROF,YT0%MSPM)=PGMVS(JROF,YT0%MSPM)-PDTS2*&
         & PGMV5(JROF,JLEV,YT5%MV)*VDELB(JLEV)*ZSPTB(JROF)  
      ENDDO
    ENDDO

  ENDIF

ELSE         !!! TL/AD NOT DONE FOR THREE-TIME-LEVEL SCHEME !!!

  CALL ABOR1(' LATTES, TL/AD NOT DONE FOR THREE-TIME-LEVEL SCHEME')

ENDIF

DO JROF=KST,KPROF
  PRES0(JROF,NFLEVG)=PRES0(JROF,NFLEVG)-ZUSPS0(JROF)/(PRES5(JROF,NFLEVG)**2)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTESAD',1,ZHOOK_HANDLE)
END SUBROUTINE LATTESAD

