SUBROUTINE LATTESTL(KST,KPROF,PDTS2,PBDT,PESGP,PESGM,&
 & POROG,POROGL,POROGM,PSDIV0,PEVEL,PDELP0,PRES0,PGMVS, &
 & PGMV,PGMVT1S,PB1,PB2, &
 & PSDIV5,PEVEL5,PDELP5,PRES5,PGMV5,PGMV5S,PSPNLT95,PB15)

!**** *LATTESTL* Semi-Lagrangian scheme.  (tangent-linear version)
!                Computation of the t and t-dt useful quantities
!                 at grid-points. Equations for bi-dimensional
!                 variables (continuity equation).

!     Purpose.
!     --------
!        * This subroutine computes the equation quantities to be
!          interpolated at each grid-point of the colocation grid.
!          The terms considered here are the explicit terms and
!          the explicit part of the semi-implicit scheme (terms
!          previously computed in LASSIE or LANHSI).
!          Equations considered here are equations for bi-dimensional
!          variables: continuity equation.
!        * Remark 1: when an alternate averaging is used for linear terms
!          in the 2TL SL scheme, the first timestep is treated differently
!          (first order uncentering), no uncentering is applied to the
!          total term ("cursive A") and the term saved in P[X]NLT9
!          is [ (Delta t/2) ("cursive A" - (1 + xidt) beta "cursive B") ]
!          instead of [ (Delta t/2) ("cursive A" - beta "cursive B") ].
!        * Remark 2: for lsettls=true, uncentering is applied to
!          the 'stable' extrapolation if vesl > 0 to avoid instability
!          in the momentum equation.
!        * Remark 3: for computation of ZSPTB and ZSPTB5, update matching
!          the direct code (use PSDVBC and no recomputation of the
!          divergence integral term here) has to be done in the future.

!**   Interface.
!     ----------
!        *CALL* *LATTESTL(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of work.
!          KPROF   - depth of work.
!          PDTS2   - 0.5*"pdt", where "pdt"=
!                    time step for the first time-integration step of
!                    a leap-frog scheme or all time-integration steps of
!                    a two-time level scheme; 2*time step for the following
!                    time-integration steps of a leap-frog scheme.
!          PBDT    - zbt if semi-implicit scheme with unreduced
!                    divergence, zbt*(c**2/GM**2) if semi-implicit
!                    scheme with reduced divergence, where zbt
!                    is equal to PDTS2*BETADT (BETADT is in YOMDYN) or zero
!                    according to the value of CDCONF.
!          PESGP   - (1 + uncentering factor).
!          PESGM   - (1 - uncentering factor).
!          POROG   - surface orography.
!          POROGL  - Zonal derivative of orography.
!          POROGM  - Meridian derivative of orography.
!          PSDIV0  - SI term at time t for continuity equation (Nu*D).
!          PEVEL   - "etadot * d prehyd/d eta" at t.
!          PDELP0  - pressure depths (Delta prehyd) at full levels at t.
!          PRES0   - hydrostatic pressure at half levels at t.
!          PGMVS   - GMVS variables at t-dt and t.

!        INPUT/OUTPUT:
!          PGMV    - GMV variables at t-dt and t.
!          PGMVT1S - GMVS variables at t+dt.
!          PB1     - "SLBUF1" buffer for interpolations.
!          PB2     - "SLBUF2" buffer.

!        INPUT-TRAJECTORY:
!          PSDIV5  - SI term at time t for continuity equation (Nu*D).
!          PEVEL5  - "etadot * d prehyd/d eta" at t.
!          PDELP5  - pressure depths (Delta prehyd) at full levels at t.
!          PRES5   - hydrostatic pressure at half levels at t.
!          PGMV5,PGMV5S: trajectory for PGMV,PGMVS.

!        INPUT/OUTPUT-TRAJECTORY:
!          PSPNLT95- nonlinear term in continuity-equn at t-dt.
!          PB15    - trajectory for PB1.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by LACDYNTL.

!     Reference.
!     ----------
!             Arpege documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        C. Temperton (ECMWF)

!     Modifications.
!     --------------
!        Original : 99/08/09.
!        C. Temperton 00-02-07: removed redundant use of PX0 when NVLAG=3
!        A. Untch 2000-11: modifications for finite element scheme 
!                          in the vertical (LVERTFE)
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad Aug 2008: rationalisation of dummy argument interfaces
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RD
USE YOMCT0   , ONLY : LTWOTL
USE YOMCT3   , ONLY : NSTEP
USE YOMCVER  , ONLY : LVERTFE
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG   ,NFLSA
USE YOMDYN   , ONLY : NVLAG    ,XIDT     ,RCMSLP0  ,LSETTLS
USE YOMGEM   , ONLY : VFE_RDETAH,VDELB
USE YOMGMV   , ONLY : NDIMGMV  ,NDIMGMVS ,YT0      ,YT1      ,YT9      ,YT5
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2 ,NFLDSLB15
USE YOMSTA   , ONLY : RTSUR
USE YOMTRSL  , ONLY : NTRSLTYPE
USE PTRSLB1  , ONLY : MSLB1SP9 ,MSLB1C9
USE PTRSLB15 , ONLY : MSLB1SP95,MSLB1C95
USE PTRSLB2  , ONLY : MSLB2SPSI

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBDT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSDIV0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP0(NPROMA,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES0(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVS(NPROMA,NDIMGMVS)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1S(NPROMA,YT1%NDIMS)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSDIV5(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(NPROMA,0:NFLEVG+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV5(NPROMA,NFLEVG,YT5%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV5S(NPROMA,YT5%NDIMS)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPNLT95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB15(NPROMA,NFLDSLB15)

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZMOY1SP(NPROMA)
REAL(KIND=JPRB) :: ZSPNLT0(NPROMA)
REAL(KIND=JPRB) :: ZSPNLT0FE(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSPT1(NPROMA,NFLEVG+1)
REAL(KIND=JPRB) :: ZMOYSPNL(NPROMA)
REAL(KIND=JPRB) :: ZSPTB(NPROMA)
REAL(KIND=JPRB) :: ZSPTB5(NPROMA)
REAL(KIND=JPRB) :: ZSPTBFE(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSPTB5FE(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSPTBOUT(NPROMA,NFLEVG+1)
REAL(KIND=JPRB) :: ZUSPS0(NPROMA)
REAL(KIND=JPRB) :: ZUSPS5(NPROMA)
REAL(KIND=JPRB) :: ZSPSI5(NPROMA)

INTEGER(KIND=JPIM) :: JLEV, JROF

REAL(KIND=JPRB) :: ZCMSLP, ZXIGM, ZXIGP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "verint.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTESTL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       *   Preliminary initialisations.

ZSPTB(KST:KPROF)=0.0_JPRB
ZSPTB5(KST:KPROF)=0.0_JPRB

DO JROF=KST,KPROF
  ZUSPS5(JROF)=1.0_JPRB/PRES5(JROF,NFLEVG)
  ZUSPS0(JROF)=-PRES0(JROF,NFLEVG)/(PRES5(JROF,NFLEVG)**2)
ENDDO
ZCMSLP=RCMSLP0/(RD*RTSUR)

IF (NSTEP <= 0) THEN
  ZXIGM=(1.0_JPRB-XIDT)
  ZXIGP=(1.0_JPRB+XIDT)
ELSE
  ZXIGM=(1.0_JPRB+XIDT)
  ZXIGP=(1.0_JPRB+XIDT)
ENDIF

!     ------------------------------------------------------------------

!*       *   Continuity equation.

IF (LTWOTL) THEN

! * Remark: the computation of "ZSPTB" and "ZSPTB5" matches with an old
!   version of the direct code which has been optimised since a couple of
!   cycles (use input array "PSDVBC" in the direct code).
 
  IF(LVERTFE) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ZSPTB5FE(JROF,JLEV)=-PDTS2*&
         & ( PDELP5(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MDIV)*ZUSPS5(JROF)&
         & +PGMV5(JROF,JLEV,YT5%MU)*VDELB(JLEV)*PGMV5S(JROF,YT5%MSPL)&
         & +PGMV5(JROF,JLEV,YT5%MV)*VDELB(JLEV)*PGMV5S(JROF,YT5%MSPM) )&
         & *VFE_RDETAH(JLEV)  
        ZSPTBFE(JROF,JLEV)=-PDTS2*&
         & ( PDELP0(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MDIV)*ZUSPS5(JROF)&
         & + PDELP5(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MDIV)*ZUSPS5(JROF)&
         & + PDELP5(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MDIV)*ZUSPS0(JROF)&
         & +PGMV(JROF,JLEV,YT0%MU)*VDELB(JLEV)*PGMV5S(JROF,YT5%MSPL)&
         & +PGMV5(JROF,JLEV,YT5%MU)*VDELB(JLEV)*PGMVS(JROF,YT0%MSPL)&
         & +PGMV(JROF,JLEV,YT0%MV)*VDELB(JLEV)*PGMV5S(JROF,YT5%MSPM)&
         & +PGMV5(JROF,JLEV,YT5%MV)*VDELB(JLEV)*PGMVS(JROF,YT0%MSPM) )&
         & *VFE_RDETAH(JLEV)  
      ENDDO
    ENDDO
    CALL VERINT(NPROMA,KST,KPROF,NFLEVG,ZSPTB5FE,ZSPTBOUT,0)
    ZSPTB5(KST:KPROF)=ZSPTBOUT(KST:KPROF,NFLEVG+1)
    CALL VERINT(NPROMA,KST,KPROF,NFLEVG,ZSPTBFE,ZSPTBOUT,0)
    ZSPTB(KST:KPROF)=ZSPTBOUT(KST:KPROF,NFLEVG+1)    

    DO JROF=KST,KPROF
      ZSPSI5(JROF)=PBDT(JROF)*PSDIV5(JROF)
      PB2(JROF,MSLB2SPSI)=PBDT(JROF)*PSDIV0(JROF)
    ENDDO

  ELSE
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ZSPTB5(JROF)=ZSPTB5(JROF)-PDTS2*&
         & ( PDELP5(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MDIV)*ZUSPS5(JROF)&
         & +PGMV5(JROF,JLEV,YT5%MU)*VDELB(JLEV)*PGMV5S(JROF,YT5%MSPL)&
         & +PGMV5(JROF,JLEV,YT5%MV)*VDELB(JLEV)*PGMV5S(JROF,YT5%MSPM) )  
        ZSPTB(JROF)=ZSPTB(JROF)-PDTS2*&
         & ( PDELP0(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MDIV)*ZUSPS5(JROF)&
         & + PDELP5(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MDIV)*ZUSPS5(JROF)&
         & + PDELP5(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MDIV)*ZUSPS0(JROF)&
         & +PGMV(JROF,JLEV,YT0%MU)*VDELB(JLEV)*PGMV5S(JROF,YT5%MSPL)&
         & +PGMV5(JROF,JLEV,YT5%MU)*VDELB(JLEV)*PGMVS(JROF,YT0%MSPL)&
         & +PGMV(JROF,JLEV,YT0%MV)*VDELB(JLEV)*PGMV5S(JROF,YT5%MSPM)&
         & +PGMV5(JROF,JLEV,YT5%MV)*VDELB(JLEV)*PGMVS(JROF,YT0%MSPM) )  
      ENDDO
    ENDDO

    DO JROF=KST,KPROF
      ZSPTB5(JROF)=ZSPTB5(JROF)&
       & +PDTS2*PEVEL5(JROF,0)*ZUSPS5(JROF)&
       & -PDTS2*PEVEL5(JROF,NFLEVG)*ZUSPS5(JROF)  
      ZSPSI5(JROF)=PBDT(JROF)*PSDIV5(JROF)
      ZSPTB(JROF)=ZSPTB(JROF)&
       & +PDTS2*PEVEL(JROF,0)*ZUSPS5(JROF)&
       & +PDTS2*PEVEL5(JROF,0)*ZUSPS0(JROF)&
       & -PDTS2*PEVEL(JROF,NFLEVG)*ZUSPS5(JROF) &
       & -PDTS2*PEVEL5(JROF,NFLEVG)*ZUSPS0(JROF)   
      PB2(JROF,MSLB2SPSI)=PBDT(JROF)*PSDIV0(JROF)
    ENDDO

  ENDIF

  IF (NVLAG == 2) THEN   !!! T/L NOT YET DONE !!!

  ELSEIF (NVLAG == 3) THEN

    IF (NTRSLTYPE == 1) THEN

      ! * recompute trajectory 
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          ZMOY1SP(JROF)=PDTS2*&
           & (PGMV5(JROF,JLEV,YT5%MU)&
           & *(PGMV5S(JROF,YT5%MSPL)+ZCMSLP*POROGL(JROF))&
           & +PGMV5(JROF,JLEV,YT5%MV)&
           & *(PGMV5S(JROF,YT5%MSPM)+ZCMSLP*POROGM(JROF)))&
           & +ZSPTB5(JROF)  
        ENDDO
        IF (NSTEP <= 0) THEN
          ZMOYSPNL(KST:KPROF)=ZMOY1SP(KST:KPROF)+ZSPSI5(KST:KPROF)
          ZSPNLT0(KST:KPROF)=ZMOY1SP(KST:KPROF)+ZSPSI5(KST:KPROF)
        ELSE
          IF (XIDT > 0.0_JPRB) THEN
            IF (LSETTLS) THEN
              DO JROF=KST,KPROF
                ZMOYSPNL(JROF)=2.0_JPRB*&
                 & (ZMOY1SP(JROF)+(1.0_JPRB+XIDT)*ZSPSI5(JROF))&
                 & -PSPNLT95(JROF,JLEV)  
                ZSPNLT0(JROF)=ZMOY1SP(JROF)+(1.0_JPRB+XIDT)*ZSPSI5(JROF)  
              ENDDO
            ELSE
              DO JROF=KST,KPROF
                ZMOYSPNL(JROF)=1.5_JPRB*&
                 & (ZMOY1SP(JROF)+(1.0_JPRB+XIDT)*ZSPSI5(JROF))&
                 & -0.5_JPRB*PSPNLT95(JROF,JLEV)  
              ENDDO
            ENDIF
          ELSE
            IF (LSETTLS) THEN
              DO JROF=KST,KPROF
                ZMOYSPNL(JROF)=(1.0_JPRB+PESGM)*&
                 & (ZMOY1SP(JROF)+ZSPSI5(JROF))-PSPNLT95(JROF,JLEV)  
                ZSPNLT0(JROF)=ZMOY1SP(JROF)+ZSPSI5(JROF)
              ENDDO
            ELSE
              DO JROF=KST,KPROF
                ZMOYSPNL(JROF)=1.5_JPRB*&
                 & (ZMOY1SP(JROF)+ZSPSI5(JROF))-0.5_JPRB*PSPNLT95(JROF,JLEV)  
              ENDDO
            ENDIF
          ENDIF
        ENDIF
        IF (XIDT > 0.0_JPRB) THEN
          DO JROF=KST,KPROF
            PSPNLT95(JROF,JLEV)=ZMOY1SP(JROF)+(1.0_JPRB+XIDT)*ZSPSI5(JROF)  
          ENDDO
          IF (LSETTLS) THEN
            DO JROF=KST,KPROF
              PB15(JROF,MSLB1C95+JLEV-NFLSA)=PB15(JROF,MSLB1C95+JLEV-NFLSA) &
               & +ZMOYSPNL(JROF)-ZXIGM*ZSPSI5(JROF)  
            ENDDO
          ELSE
            DO JROF=KST,KPROF
              PB15(JROF,MSLB1C95+JLEV-NFLSA)=PB15(JROF,MSLB1C95+JLEV-NFLSA) &
               & +ZMOYSPNL(JROF)-ZXIGM*ZSPSI5(JROF)  
            ENDDO
          ENDIF
        ELSE
          PSPNLT95(KST:KPROF,JLEV)=ZMOY1SP(KST:KPROF)+ZSPSI5(KST:KPROF)
          IF (LSETTLS) THEN
            DO JROF=KST,KPROF
              PB15(JROF,MSLB1C95+JLEV-NFLSA)=PB15(JROF,MSLB1C95+JLEV-NFLSA) &
               & +ZMOYSPNL(JROF)-PESGM*ZSPSI5(JROF)  
            ENDDO
          ELSE
            DO JROF=KST,KPROF
              PB15(JROF,MSLB1C95+JLEV-NFLSA)=PB15(JROF,MSLB1C95+JLEV-NFLSA) &
               & +PESGM*ZMOYSPNL(JROF)-PESGM*ZSPSI5(JROF)  
            ENDDO
          ENDIF
        ENDIF
      ENDDO

      DO JROF=KST,KPROF
        PB15(JROF,MSLB1SP95)=PGMV5S(JROF,YT5%MSP)+ZCMSLP*POROG(JROF)
      ENDDO

    ENDIF

    ! * compute perturbation terms
    IF(LVERTFE) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          ZMOY1SP(JROF)=PDTS2*&
           & (PGMV(JROF,JLEV,YT0%MU)&
           & *(PGMV5S(JROF,YT5%MSPL)+ZCMSLP*POROGL(JROF))&
           & +PGMV5(JROF,JLEV,YT5%MU)*PGMVS(JROF,YT0%MSPL)&
           & +PGMV(JROF,JLEV,YT0%MV)&
           & *(PGMV5S(JROF,YT5%MSPM)+ZCMSLP*POROGM(JROF))&
           & +PGMV5(JROF,JLEV,YT5%MV)*PGMVS(JROF,YT0%MSPM))&
           & +ZSPTB(JROF)  
        ENDDO
        IF (NSTEP <= 0) THEN
          ZMOYSPNL(KST:KPROF)=ZMOY1SP(KST:KPROF)+PB2(KST:KPROF,MSLB2SPSI)
        ELSE
          DO JROF=KST,KPROF
            ZMOYSPNL(JROF)=(1.0_JPRB+PESGM)*&
             & (ZMOY1SP(JROF)+PB2(JROF,MSLB2SPSI))-PGMV(JROF,JLEV,YT9%MSPNL)
          ENDDO
        ENDIF
        DO JROF=KST,KPROF
          ZSPNLT0FE(JROF,JLEV)=(ZMOY1SP(JROF)&
           & +PB2(JROF,MSLB2SPSI))*PESGP*VDELB(JLEV)*VFE_RDETAH(JLEV)  
          PGMV(JROF,JLEV,YT9%MSPNL)=ZMOY1SP(JROF)+PB2(JROF,MSLB2SPSI)
          PB1(JROF,MSLB1C9+JLEV-NFLSA)=PB1(JROF,MSLB1C9+JLEV-NFLSA)&
           & +ZMOYSPNL(JROF)-PESGM*PB2(JROF,MSLB2SPSI)  
        ENDDO
      ENDDO
      CALL VERINT(NPROMA,KST,KPROF,NFLEVG,ZSPNLT0FE,ZSPT1,0)
      DO JROF=KST,KPROF
        PGMVT1S(JROF,YT1%MSP)=PGMVT1S(JROF,YT1%MSP)+ZSPT1(JROF,NFLEVG+1)  
      ENDDO
    ELSE

      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          ZMOY1SP(JROF)=PDTS2*&
           & (PGMV(JROF,JLEV,YT0%MU)&
           & *(PGMV5S(JROF,YT5%MSPL)+ZCMSLP*POROGL(JROF))&
           & +PGMV5(JROF,JLEV,YT5%MU)*PGMVS(JROF,YT0%MSPL)&
           & +PGMV(JROF,JLEV,YT0%MV)&
           & *(PGMV5S(JROF,YT5%MSPM)+ZCMSLP*POROGM(JROF))&
           & +PGMV5(JROF,JLEV,YT5%MV)*PGMVS(JROF,YT0%MSPM))&
           & +ZSPTB(JROF)  
        ENDDO
        IF (NSTEP <= 0) THEN
          DO JROF=KST,KPROF
            ZMOYSPNL(JROF)=ZMOY1SP(JROF)+PB2(JROF,MSLB2SPSI)
            ZSPNLT0(JROF)=ZMOY1SP(JROF)+PB2(JROF,MSLB2SPSI)
          ENDDO
        ELSE
          IF (XIDT > 0.0_JPRB) THEN
            IF (LSETTLS) THEN
              DO JROF=KST,KPROF
                ZMOYSPNL(JROF)=2.0_JPRB*&
                 & (ZMOY1SP(JROF)+(1.0_JPRB+XIDT)*PB2(JROF,MSLB2SPSI))&
                 & -PGMV(JROF,JLEV,YT9%MSPNL)  
                ZSPNLT0(JROF)=ZMOY1SP(JROF)+(1.0_JPRB+XIDT)*PB2(JROF,MSLB2SPSI)
              ENDDO
            ELSE
              DO JROF=KST,KPROF
                ZMOYSPNL(JROF)=1.5_JPRB*&
                 & (ZMOY1SP(JROF)+(1.0_JPRB+XIDT)*PB2(JROF,MSLB2SPSI))&
                 & -0.5_JPRB*PGMV(JROF,JLEV,YT9%MSPNL)  
              ENDDO
            ENDIF
          ELSE
            IF (LSETTLS) THEN
              DO JROF=KST,KPROF
                ZMOYSPNL(JROF)=(1.0_JPRB+PESGM)*&
                 & (ZMOY1SP(JROF)+PB2(JROF,MSLB2SPSI))&
                 & -PGMV(JROF,JLEV,YT9%MSPNL)  
                ZSPNLT0(JROF)=ZMOY1SP(JROF)+PB2(JROF,MSLB2SPSI)
              ENDDO
            ELSE
              DO JROF=KST,KPROF
                ZMOYSPNL(JROF)=1.5_JPRB*(ZMOY1SP(JROF)+PB2(JROF,MSLB2SPSI))&
                 & -0.5_JPRB*PGMV(JROF,JLEV,YT9%MSPNL)  
              ENDDO
            ENDIF
          ENDIF
        ENDIF
        IF (XIDT > 0.0_JPRB) THEN
          DO JROF=KST,KPROF
            PGMV(JROF,JLEV,YT9%MSPNL)=ZMOY1SP(JROF)&
             & +(1.0_JPRB+XIDT)*PB2(JROF,MSLB2SPSI)  
          ENDDO
          IF (LSETTLS) THEN
            DO JROF=KST,KPROF
              PB1(JROF,MSLB1C9+JLEV-NFLSA)=PB1(JROF,MSLB1C9+JLEV-NFLSA)&
               & +ZMOYSPNL(JROF)-ZXIGM*PB2(JROF,MSLB2SPSI)  
              PGMVT1S(JROF,YT1%MSP)=PGMVT1S(JROF,YT1%MSP)&
               & +VDELB(JLEV)*ZSPNLT0(JROF)  
            ENDDO
          ELSE
            DO JROF=KST,KPROF
              PB1(JROF,MSLB1C9+JLEV-NFLSA)=PB1(JROF,MSLB1C9+JLEV-NFLSA)&
               & +ZMOYSPNL(JROF)-ZXIGM*PB2(JROF,MSLB2SPSI)  
              PGMVT1S(JROF,YT1%MSP)=PGMVT1S(JROF,YT1%MSP)&
               & +VDELB(JLEV)*ZMOYSPNL(JROF)  
            ENDDO
          ENDIF
        ELSE
          DO JROF=KST,KPROF
            PGMV(JROF,JLEV,YT9%MSPNL)=ZMOY1SP(JROF)+PB2(JROF,MSLB2SPSI)
          ENDDO
          IF (LSETTLS) THEN
            DO JROF=KST,KPROF
              PB1(JROF,MSLB1C9+JLEV-NFLSA)=PB1(JROF,MSLB1C9+JLEV-NFLSA)&
               & +ZMOYSPNL(JROF)-PESGM*PB2(JROF,MSLB2SPSI)  
              PGMVT1S(JROF,YT1%MSP)=PGMVT1S(JROF,YT1%MSP)&
               & +PESGP*VDELB(JLEV)*ZSPNLT0(JROF)  
            ENDDO
          ELSE
            DO JROF=KST,KPROF
              PB1(JROF,MSLB1C9+JLEV-NFLSA)=PB1(JROF,MSLB1C9+JLEV-NFLSA)&
               & +PESGM*ZMOYSPNL(JROF)-PESGM*PB2(JROF,MSLB2SPSI)  
              PGMVT1S(JROF,YT1%MSP)=PGMVT1S(JROF,YT1%MSP)&
               & +PESGP*VDELB(JLEV)*ZMOYSPNL(JROF)  
            ENDDO
          ENDIF
        ENDIF
      ENDDO

    ENDIF

    IF (XIDT > 0.0_JPRB) THEN
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1SP9)=PGMVS(JROF,YT0%MSP)
        PGMVT1S(JROF,YT1%MSP)=PGMVT1S(JROF,YT1%MSP)-ZXIGP*PB2(JROF,MSLB2SPSI)
        PB2(JROF,MSLB2SPSI)=ZXIGP*PB2(JROF,MSLB2SPSI)
      ENDDO
    ELSE
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1SP9)=PGMVS(JROF,YT0%MSP)
        PGMVT1S(JROF,YT1%MSP)=PGMVT1S(JROF,YT1%MSP)-PESGP*PB2(JROF,MSLB2SPSI)
        PB2(JROF,MSLB2SPSI)=PESGP*PB2(JROF,MSLB2SPSI)
      ENDDO
    ENDIF

  ENDIF

ELSE         !!! T/L NOT DONE FOR THREE-TIME-LEVEL SCHEME !!!

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTESTL',1,ZHOOK_HANDLE)
END SUBROUTINE LATTESTL

