SUBROUTINE LATTEX_DNT_AD(KST,KPROF,LDSETTLS,KXLAG,PESGP,PESGM,PXT0,PMOY1X,&
 & PXSI,PXNLT9,PXT1,PXL0,PXL9)

!------------------------------------------------------------------------------
! LATTEX_DNT_AD - Semi-Lagrangian scheme.
!               Computation of the t and t-dt useful quantities
!               at grid-points. Equations for tri-dimensional
!               variables for 2TL scheme.
!               Adjoint code.

! Purpose
! -------
!       Adjoint code of LATTEX_DNT: see LATTEX_DNT for more information.
!       Some options present in LATTEX_DNT are not coded in LATTEX_DNT_AD
!        (for ex: LPC_FULL, KXLAG=2), and the code, coming from LATTEXAD,
!        has a presentation which does not completely match with the one of
!        LATTEX_DNT.

! Interface
! ---------
!   CALL LATTEX_DNT_AD(..)

! Explicit arguments :
! --------------------

!          KST     - first element of work.
!          KPROF   - depth of work.
!          LDSETTLS- .T./.F.: Stable/Conventional extrapolations for SL2TL.
!          KXLAG   - type of SL discretisation 
!          PESGP   - (1 + uncentering factor).
!          PESGM   - (1 - uncentering factor).
!          PXT0    - prognostic variable time t (predictor or SI),
!                    preliminary t+dt (corrector)
!          PMOY1X  - full nonlinear model at time t [(Delta t/2) "cursive" A]
!          PXSI    - semi-implicit linear model at time t
!                    [- (Delta t/2) "cursive" B]
!          PXNLT9  - buffer used during predictor resp. SI time step
!          PXT1    - t+dt term and other final point terms 
!          PXL0    - second SL quantity to be interpolated (linearly) at O
!                    if KXLAG=3
!          PXL9    - SL quantity to be interpolated at O 

! Externals
! ---------
!   none
!   Called by LATTEXAD.

! Method
! ------

! Reference
! ---------
!   Arpege documentation about semi-Lagrangian scheme.

! Author
! ------
!      K. Yessad (March 2007) after LATTEXAD.

! Modifications
! -------------
!  K. Yessad Aug 2008: rationalisation of dummy argument interfaces
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

USE YOMCT3   , ONLY : NSTEP
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG   ,NFLSA    ,NFLEN
USE YOMDYN   , ONLY : XIDT

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LDSETTLS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KXLAG 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGM 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMOY1X(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXSI(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXNLT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXT1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXL0(NPROMA,NFLSA:NFLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXL9(NPROMA,NFLSA:NFLEN) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV,JROF

REAL(KIND=JPRB) :: ZMOYXNL(NPROMA)
REAL(KIND=JPRB) :: ZXNLT0(NPROMA)
REAL(KIND=JPRB) :: ZXIGM, ZXIGP

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEX_DNT_AD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*      1. AUXILIARY VARIABLES.
!       -----------------------

IF (NSTEP <= 0) THEN
  ZXIGM=(1.0_JPRB-XIDT)
  ZXIGP=(1.0_JPRB+XIDT)
ELSE
  ZXIGM=(1.0_JPRB+XIDT)
  ZXIGP=(1.0_JPRB+XIDT)
ENDIF

!     ------------------------------------------------------------------

!*      2. MAIN CALCULATIONS.
!       ---------------------

DO JLEV=1,NFLEVG

  IF (KXLAG == 2) THEN     !! TL/AD NOT YET DONE !!

  ELSEIF (KXLAG == 3) THEN

    DO JROF=KST,KPROF
      PXT0(JROF,JLEV)=PXT0(JROF,JLEV)+PXL9(JROF,JLEV)
    ENDDO
    IF (XIDT > 0.0_JPRB) THEN
      DO JROF=KST,KPROF
        PXSI(JROF,JLEV)=ZXIGP*PXSI(JROF,JLEV)
      ENDDO
      IF (LDSETTLS) THEN
        DO JROF=KST,KPROF
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)-ZXIGM*PXL0(JROF,JLEV)
          ZMOYXNL(JROF)=PXL0(JROF,JLEV)
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)-ZXIGP*PXT1(JROF,JLEV)
          ZXNLT0(JROF)=PXT1(JROF,JLEV)
        ENDDO
      ELSE
        DO JROF=KST,KPROF
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)-ZXIGM*PXL0(JROF,JLEV)
          ZMOYXNL(JROF)=PXL0(JROF,JLEV)
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)-ZXIGP*PXT1(JROF,JLEV)
          ZMOYXNL(JROF)=ZMOYXNL(JROF)+PXT1(JROF,JLEV)
          ZXNLT0(JROF)=0.0_JPRB
        ENDDO
      ENDIF
    ELSE
      DO JROF=KST,KPROF
        PXSI(JROF,JLEV)=PESGP*PXSI(JROF,JLEV)
      ENDDO
      IF (LDSETTLS) THEN
        DO JROF=KST,KPROF
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)-PESGM*PXL0(JROF,JLEV)
          ZMOYXNL(JROF)=PXL0(JROF,JLEV)
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)-PESGP*PXT1(JROF,JLEV)
          ZXNLT0(JROF)=PESGP*PXT1(JROF,JLEV)
        ENDDO
      ELSE
        DO JROF=KST,KPROF
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)-PESGM*PXL0(JROF,JLEV)
          ZMOYXNL(JROF)=PESGM*PXL0(JROF,JLEV)
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)-PESGP*PXT1(JROF,JLEV)
          ZMOYXNL(JROF)=ZMOYXNL(JROF)+PESGP*PXT1(JROF,JLEV)
          ZXNLT0(JROF)=0.0_JPRB
        ENDDO
      ENDIF
    ENDIF

  ENDIF ! KXLAG

  IF (XIDT > 0.0_JPRB) THEN
    DO JROF=KST,KPROF
      PXSI(JROF,JLEV)=PXSI(JROF,JLEV)+(1.0_JPRB+XIDT)*PXNLT9(JROF,JLEV)
      PMOY1X(JROF,JLEV)=PXNLT9(JROF,JLEV)
      PXNLT9(JROF,JLEV)=0.0_JPRB
    ENDDO
  ELSE
    DO JROF=KST,KPROF
      PXSI(JROF,JLEV)=PXSI(JROF,JLEV)+PXNLT9(JROF,JLEV)
      PMOY1X(JROF,JLEV)=PXNLT9(JROF,JLEV)
      PXNLT9(JROF,JLEV)=0.0_JPRB
    ENDDO
  ENDIF ! XIDT

  IF (NSTEP <= 0) THEN
    DO JROF=KST,KPROF
      PXSI(JROF,JLEV)=PXSI(JROF,JLEV)+ZMOYXNL(JROF)+ZXNLT0(JROF)
      PMOY1X(JROF,JLEV)=PMOY1X(JROF,JLEV)+ZMOYXNL(JROF)+ZXNLT0(JROF)
    ENDDO
  ELSE
    IF (XIDT > 0.0_JPRB) THEN
      IF (LDSETTLS) THEN
        DO JROF=KST,KPROF
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)+(1.0_JPRB+XIDT)*ZXNLT0(JROF)
          PMOY1X(JROF,JLEV)=PMOY1X(JROF,JLEV)+ZXNLT0(JROF)
          PXNLT9(JROF,JLEV)=PXNLT9(JROF,JLEV)-ZMOYXNL(JROF)
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)+2.0_JPRB*(1.0_JPRB+XIDT)*ZMOYXNL(JROF)
          PMOY1X(JROF,JLEV)=PMOY1X(JROF,JLEV)+2.0_JPRB*ZMOYXNL(JROF)
        ENDDO
      ELSE
        DO JROF=KST,KPROF
          PXNLT9(JROF,JLEV)=PXNLT9(JROF,JLEV)-0.5_JPRB*ZMOYXNL(JROF)
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)+1.5_JPRB*(1.0_JPRB+XIDT)*ZMOYXNL(JROF)
          PMOY1X(JROF,JLEV)=PMOY1X(JROF,JLEV)+1.5_JPRB*ZMOYXNL(JROF)
        ENDDO
      ENDIF
    ELSE
      IF (LDSETTLS) THEN
        DO JROF=KST,KPROF
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)+ZXNLT0(JROF)
          PMOY1X(JROF,JLEV)=PMOY1X(JROF,JLEV)+ZXNLT0(JROF)
          PXNLT9(JROF,JLEV)=PXNLT9(JROF,JLEV)-ZMOYXNL(JROF)
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)+(1.0_JPRB+PESGM)*ZMOYXNL(JROF)
          PMOY1X(JROF,JLEV)=PMOY1X(JROF,JLEV)+(1.0_JPRB+PESGM)*ZMOYXNL(JROF)
        ENDDO
      ELSE
        DO JROF=KST,KPROF
          PXNLT9(JROF,JLEV)=PXNLT9(JROF,JLEV)-0.5_JPRB*ZMOYXNL(JROF)
          PXSI(JROF,JLEV)=PXSI(JROF,JLEV)+1.5_JPRB*ZMOYXNL(JROF)
          PMOY1X(JROF,JLEV)=PMOY1X(JROF,JLEV)+1.5_JPRB*ZMOYXNL(JROF)
        ENDDO
      ENDIF
    ENDIF
  ENDIF ! NSTEP

ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEX_DNT_AD',1,ZHOOK_HANDLE)
END SUBROUTINE LATTEX_DNT_AD

