SUBROUTINE LATTEX_TNT(KST,KPROF,KXLAG,PESGP,PESGM,PXT9,PMOY1X,PXSI9,&
 & PXSI0,PXT1,PXL0,PXL9)  

!**** *LATTEX_TNT*   Semi-Lagrangian scheme.
!                Computation of the t and t-dt useful quantities
!                 at grid-points. Equations for tri-dimensional
!                 variables for 3TL scheme

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *LATTEX_TNT(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of work.
!          KPROF   - depth of work.
!          KXLAG   - type of SL discretisation.
!          PESGP   - (1 + uncentering factor).
!          PESGM   - (1 - uncentering factor).
!          PXT9    - prognostic variable time t-dt
!          PMOY1X  - full nonlinear model at time t [(Delta t/2) "cursive" A]
!          PXSI9   - semi-implicit linear model at time t-dt
!                    [- (Delta t/2) "cursive" B]

!        INPUT/OUTPUT:
!          PXSI0   - semi-implicit linear model at time t
!                    [- (Delta t/2) "cursive" B]
!          PXT1    - t+dt term and other final point terms 
!          PXL0    - second SL quantity to be interpolated (linearly) at O
!                    if KXLAG=3
!          PXL9    - SL quantity to be interpolated at O 

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by LATTEX.

!     Reference.
!     ----------
!             Arpege documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        J. VIVODA (SHMI/CHMI/LACE) - rationalization of LATTEX

!     Modifications.
!     --------------
!     Original : March 2002
!     M.Hamrud      01-Oct-2003 CY28 Cleaning
!     K. Yessad Aug 2008: rationalisation of dummy arg interfaces + cleanings
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG   ,NFLSA    ,NFLEN

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KXLAG 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXT9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMOY1X(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSI9(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXSI0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXT1(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXL0(NPROMA,NFLSA:NFLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PXL9(NPROMA,NFLSA:NFLEN) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB)    :: ZMOYXNL(NPROMA)

INTEGER(KIND=JPIM) :: JLEV,JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEX_TNT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

DO JLEV=1,NFLEVG

  DO JROF=KST,KPROF
    ZMOYXNL(JROF)=PMOY1X(JROF,JLEV)+PXSI0(JROF,JLEV)
  ENDDO
  IF (KXLAG == 2) THEN
    DO JROF=KST,KPROF
      PXT1(JROF,JLEV)=PXT1(JROF,JLEV)&
       & +PESGP*ZMOYXNL(JROF)-PESGP*PXSI0(JROF,JLEV)
      PXL9(JROF,JLEV)=PXL9(JROF,JLEV)+PXT9(JROF,JLEV)&
       & +PESGM*ZMOYXNL(JROF)-PESGM*PXSI9(JROF,JLEV)
    ENDDO
  ELSEIF(KXLAG == 3) THEN
    DO JROF=KST,KPROF
      PXL0(JROF,JLEV)=PXL0(JROF,JLEV)&
       & +PESGM*ZMOYXNL(JROF)-PESGM*PXSI9(JROF,JLEV)  
      PXT1(JROF,JLEV)=PXT1(JROF,JLEV)&
       & +PESGP*ZMOYXNL(JROF)-PESGP*PXSI0(JROF,JLEV)  
      PXL9(JROF,JLEV)=PXL9(JROF,JLEV)+PXT9(JROF,JLEV)
    ENDDO
  ENDIF
  PXSI0(KST:KPROF,JLEV)=PESGP*PXSI0(KST:KPROF,JLEV)

ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEX_TNT',1,ZHOOK_HANDLE)
END SUBROUTINE LATTEX_TNT

