SUBROUTINE LATTEXAD(KST,KPROF,PDTS2,PBT,PBDT,PESGP,PESGM,&
 & PRCORI,POROG,POROGL,POROGM,&
 & PGAGT0L,PGAGT0M,PTOD0,PSGRTL,PSGRTM,PKAP,PVVEL,PRDELP,PEVEL,&
 & PGFL,PGMV,PGMVT1,PB1,PB2,&
 & PKAP5,PVVEL5,PRDELP5,PEVEL5,PGMV5)  

!**** *LATTEXAD* Semi-Lagrangian scheme.  (adjoint version)
!                Computation of the t and t-dt useful quantities
!                 at grid-points. Equations for tri-dimensional
!                 variables.

!     Purpose.
!     --------
!       AD of LATTEX: see LATTEX for more information.
!       Remarks:
!        - AD of SL3TL not yet coded.
!        - AD of SL2TL: only K[X]LAG = 3 is coded.

!**   Interface.
!     ----------
!        *CALL* *LATTEXAD(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of work.
!          KPROF   - depth of work.
!          PDTS2   - 0.5*time step for the first time-integration step of
!                    a leap-frog scheme or all time-integration steps of
!                    a two-time level scheme; time step for the following
!                    time-integration steps of a leap-frog scheme.
!          PBT     - PDTS2*BETADT (BETADT is in YOMDYN) or zero 
!                    according to the value of CDCONF.
!          PBDT    - PBT if semi-implicit scheme with unreduced
!                    divergence, PBT*(c**2/GM**2) if semi-implicit
!                    scheme with reduced divergence.
!          PESGP   - (1 + uncentering factor).
!          PESGM   - (1 - uncentering factor).
!          PRCORI  - Coriolis parameter "f = 2 Omega sin(theta)".
!          POROG   - surface orography.
!          POROGL  - zonal component of the surface orography gradient.
!          POROGM  - meridian component of the surface orography gradient.

!        INPUT/OUTPUT or INPUT (match with INOUT in TL):
!          PGAGT0L - semi-implicit term at time t for U-wind equation.
!          PGAGT0M - semi-implicit term at time t for V-wind equation.
!          PTOD0   - semi-implicit term at time t for temperature equation.
!          PSGRTL  - zonal component of the pressure force term
!                    (grad(phi)+RT grad(p)).
!          PSGRTM  - meridian component of the pressure force term.
!          PKAP    - "R/cp" at full levels at t.
!          PVVEL   - "omega/prehyd" at full levels a t.
!          PRDELP  - "1/(pressure depth of layers)" at t.
!          PEVEL   - "etadot (d prehyd/d eta)" at half levels at t.
!          PGFL    - GFL at full levels at t.
!          PGMV    - GMV variables at t-dt and t.
!          PGMVT1  - GMV variables at t+dt.
!          PB1     - "SLBUF1" buffer for interpolations (IN).
!          PB2     - "SLBUF2" buffer.

!        TRAJECTORY INPUT:
!          PKAP5   - "R/cp" at full levels at t.
!          PVVEL5  - "omega/prehyd" at full levels a t.
!          PRDELP5 - "1/(pressure depth of layers)" at t.
!          PEVEL5  - "etadot (d prehyd/d eta)" at half levels at t.
!          PGMV5   - trajectory for PGMV.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by LACDYNAD.

!     Reference.
!     ----------
!             Arpege documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        C. Temperton (ECMWF) 

!     Modifications.
!     --------------
!        Original : 99/10/12.
!        Modified 2000-11 by A. Untch: vertical finite element scheme
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Untch  June-2005: AD of Rayleigh friction
!        K.Yessad  Mar-2006: minor reorganisation to match better with LATTEX.
!        K.Yessad  Mar-2007: make the code more modular, cf. the direct code.
!        K. Yessad Aug 2008: rationalisation of dummy argument interfaces
!        P. Bechtold+A. Untch 26-10-2008: add LEGWWMS switch for non-orogr. GWD
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RD
USE YOMCT0   , ONLY : LRFRIC  ,LTWOTL
USE YOMCVER  , ONLY : LVERTFE
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG   ,NFLSA    ,NFLEN
USE YOMDYN   , ONLY : NWLAG    ,NTLAG    ,RKRF     ,LSETTLS  ,LADVF    , &
 & RCMSLP0  ,RCORDIH  ,RCORDIF
USE YOMGMV   , ONLY : NDIMGMV  ,YT0      ,YT1      ,YT9      ,YT5
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2
USE YOMSTA   , ONLY : RTSUR
USE YOM_YGFL , ONLY : YGFL     ,YGFLC
USE YOEPHY   , ONLY : LEGWWMS
USE PTRSLB1  , ONLY : MSLB1U0  ,MSLB1V0  ,MSLB1T0  ,MSLB1U9  ,MSLB1V9  , &
 & MSLB1T9  ,MSLB1GFL9
USE PTRSLB2  , ONLY : MSLB2USI ,MSLB2VSI ,MSLB2TSI

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBDT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGAGT0L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGAGT0M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTOD0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSGRTL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSGRTM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PKAP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVVEL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVEL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB1(NPROMA,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAP5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVVEL5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV5(NPROMA,NFLEVG,YT5%NDIM)

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZMOY1U(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZMOY1V(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZMOY1T(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZWT0(NPROMA)
REAL(KIND=JPRB) :: ZWT5(NPROMA)

INTEGER(KIND=JPIM) :: IPX
INTEGER(KIND=JPIM) :: JLEV,JGFL,JROF
LOGICAL :: LLSETTLSW

REAL(KIND=JPRB) :: ZCMSLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "lattex_dnt_ad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEXAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*      1.  PRELIMINARY INITIALISATIONS.
!       --------------------------------

ZCMSLP=RCMSLP0/(RD*RTSUR)

!     ------------------------------------------------------------------

!*      2.  TREATMENT OF GMV VARIABLES.
!       -------------------------------

!*       2.1   Momentum equation.

! * LSETTLS is replaced by LLSETTLSW=.FALSE. for wind-eqn if VESL>0 because
!   stable extrapolation deteriorates scores without improving stability.
LLSETTLSW=LSETTLS.AND.(.NOT.(PESGP > PESGM))

IF (LTWOTL) THEN

  CALL LATTEX_DNT_AD(KST,KPROF,LLSETTLSW,NWLAG,PESGP,PESGM,&
   & PGMV(1,1,YT0%MU),ZMOY1U,PB2(1,MSLB2USI),&
   & PGMV(1,1,YT9%MUNL),PGMVT1(1,1,YT1%MU),&
   & PB1(1,MSLB1U0),PB1(1,MSLB1U9))

  CALL LATTEX_DNT_AD(KST,KPROF,LLSETTLSW,NWLAG,PESGP,PESGM,&
   & PGMV(1,1,YT0%MV),ZMOY1V,PB2(1,MSLB2VSI),&
   & PGMV(1,1,YT9%MVNL),PGMVT1(1,1,YT1%MV),&
   & PB1(1,MSLB1V0),PB1(1,MSLB1V9))

ELSE         !!! TL/AD OF THREE-TIME-LEVEL SCHEME NOT DONE !!!

  ! not coded (call to LATTEX_TNT_AD)

ENDIF

DO JLEV=1,NFLEVG

  ! * U
  IF (LRFRIC .AND. .NOT.LEGWWMS) THEN
    DO JROF=KST,KPROF
      PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU) &
       & -PDTS2*RKRF(JLEV)*ZMOY1U(JROF,JLEV)
    ENDDO
  ENDIF
  IF (LADVF) THEN
    DO JROF=KST,KPROF
      PSGRTL(JROF,JLEV)=PSGRTL(JROF,JLEV)-PDTS2*ZMOY1U(JROF,JLEV)  
    ENDDO
  ELSE
    DO JROF=KST,KPROF
      PSGRTL(JROF,JLEV)=PSGRTL(JROF,JLEV)-PDTS2*ZMOY1U(JROF,JLEV)  
      PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV) &
       & +PDTS2*PRCORI(JROF)*ZMOY1U(JROF,JLEV)  
    ENDDO
  ENDIF

  DO JROF=KST,KPROF
    PGAGT0L(JROF,JLEV)=PGAGT0L(JROF,JLEV)+PBT*PB2(JROF,MSLB2USI+JLEV-1)  
  ENDDO

  ! * V
  IF (LADVF) THEN
    DO JROF=KST,KPROF
      PSGRTM(JROF,JLEV)=PSGRTM(JROF,JLEV)-PDTS2*ZMOY1V(JROF,JLEV)  
    ENDDO
  ELSE
    DO JROF=KST,KPROF
      PSGRTM(JROF,JLEV)=PSGRTM(JROF,JLEV)-PDTS2*ZMOY1V(JROF,JLEV)  
      PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU) &
       & -PDTS2*PRCORI(JROF)*ZMOY1V(JROF,JLEV)  
    ENDDO
  ENDIF

  DO JROF=KST,KPROF
    PGAGT0M(JROF,JLEV)=PGAGT0M(JROF,JLEV)+PBT*PB2(JROF,MSLB2VSI+JLEV-1)  
  ENDDO

ENDDO

!*       2.2   Temperature equation.

IF (LTWOTL) THEN

  CALL LATTEX_DNT_AD(KST,KPROF,LSETTLS,NTLAG,PESGP,PESGM,&
   & PGMV(1,1,YT0%MT),ZMOY1T,PB2(1,MSLB2TSI),&
   & PGMV(1,1,YT9%MTNL),PGMVT1(1,1,YT1%MT), &
   & PB1(1,MSLB1T0),PB1(1,MSLB1T9))

ELSE         !!! T/L OF THREE-TIME-LEVEL SCHEME NOT DONE !!!

  ! not coded (call to LATTEX_TNT_AD)

ENDIF

DO JLEV=1,NFLEVG

  IF(LVERTFE) THEN
    ZWT5(KST:KPROF)=PEVEL5(KST:KPROF,JLEV)
  ELSE
    DO JROF=KST,KPROF
      ZWT5(JROF)=0.5_JPRB*(PEVEL5(JROF,JLEV)+PEVEL5(JROF,JLEV-1))  
    ENDDO
  ENDIF

  DO JROF=KST,KPROF
    PKAP(JROF,JLEV)=PKAP(JROF,JLEV)&
     & +PDTS2*PGMV5(JROF,JLEV,YT5%MT)*PVVEL5(JROF,JLEV)*ZMOY1T(JROF,JLEV)  
    PGMV(JROF,JLEV,YT0%MT)=PGMV(JROF,JLEV,YT0%MT)&
     & +PDTS2*PKAP5(JROF,JLEV)*PVVEL5(JROF,JLEV)*ZMOY1T(JROF,JLEV)  
    PVVEL(JROF,JLEV)=PVVEL(JROF,JLEV)&
     & +PDTS2*PKAP5(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MT)*ZMOY1T(JROF,JLEV)  
    PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU)&
     & +PDTS2*ZCMSLP*RCORDIF(JLEV)*POROGL(JROF)*ZMOY1T(JROF,JLEV)  
    PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV)&
     & +PDTS2*ZCMSLP*RCORDIF(JLEV)*POROGM(JROF)*ZMOY1T(JROF,JLEV)  
    ZWT0(JROF)=PDTS2*ZCMSLP*(RCORDIH(JLEV)-RCORDIH(JLEV-1))&
     & *PRDELP5(JROF,JLEV)*POROG(JROF)*ZMOY1T(JROF,JLEV)  
    PRDELP(JROF,JLEV)=PRDELP(JROF,JLEV)&
     & +PDTS2*ZCMSLP*(RCORDIH(JLEV)-RCORDIH(JLEV-1))&
     & *ZWT5(JROF)*POROG(JROF)*ZMOY1T(JROF,JLEV)  
  ENDDO

  IF(LVERTFE) THEN
    DO JROF=KST,KPROF
      PEVEL(JROF,JLEV)=PEVEL(JROF,JLEV)+ZWT0(JROF)  
    ENDDO
  ELSE
    DO JROF=KST,KPROF
      PEVEL(JROF,JLEV)=PEVEL(JROF,JLEV)+0.5_JPRB*ZWT0(JROF)  
      PEVEL(JROF,JLEV-1)=PEVEL(JROF,JLEV-1)+0.5_JPRB*ZWT0(JROF)  
    ENDDO
  ENDIF

  DO JROF=KST,KPROF
    PTOD0(JROF,JLEV)=PTOD0(JROF,JLEV)+PBDT(JROF)*PB2(JROF,MSLB2TSI+JLEV-1)  
  ENDDO

ENDDO

!*       2.3   Anhydrostatic variables equations: "pressure departure".

!NYC-NH IF (LNHDYN) THEN
!NYC-NH   AD not yet coded.
!NYC-NH ENDIF

!*       2.4a  Anhydrostatic variables equations: "vertical divergence".

!NYC-NH IF (LNHDYN.AND.(.NOT.LGWADV)) THEN
!NYC-NH   AD not yet coded.
!NYC-NH ENDIF

!*       2.4b  Anhydrostatic variables equations: "gw".

!NYC-NH IF (LNHDYN.AND.LGWADV) THEN
!NYC-NH   AD not yet coded.
!NYC-NH ENDIF

!     ------------------------------------------------------------------

!*      3.  TREATMENT OF GFL VARIABLES.
!       --------------------------------

IF (LTWOTL) THEN

  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LADV) THEN
      IPX=(YGFLC(JGFL)%MP_SL1-1)*(NFLEN-NFLSA+1)
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PGFL(JROF,JLEV,YGFLC(JGFL)%MP)=PGFL(JROF,JLEV,YGFLC(JGFL)%MP) &
           & +PB1(JROF,MSLB1GFL9+IPX+JLEV-NFLSA)
        ENDDO
      ENDDO
    ENDIF
  ENDDO

ELSE    !!! TL/AD OF THREE-TIME-LEVEL SCHEME NOT DONE !!!

ENDIF

!     ------------------------------------------------------------------

!*      4.  DEALLOCATIONS.
!       ------------------

! Empty for the time being.

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEXAD',1,ZHOOK_HANDLE)
END SUBROUTINE LATTEXAD

