SUBROUTINE LATTEXTL(KST,KPROF,PDTS2,PBT,PBDT,PESGP,PESGM,&
 & PRCORI,POMVRL,POMVRM,POROG,POROGL,POROGM,&
 & PGAGT0L,PGAGT0M,PTOD0,PSGRTL,PSGRTM,PKAP,PVVEL,PRDELP,PEVEL,&
 & PGFL,PGMV,PGMVT1,PB1,PB2,&
 & PGAGT5L,PGAGT5M,PTOD5,PSGRT5L,PSGRT5M,PKAP5,PVVEL5,PRDELP5,PEVEL5,&
 & PGMV5,PGFL5,PUNLT95,PVNLT95,PTNLT95,PB15)

!**** *LATTEXTL* Semi-Lagrangian scheme.  (tangent-linear version)
!                Computation of the t and t-dt useful quantities
!                 at grid-points. Equations for tri-dimensional
!                 variables.

!     Purpose.
!     --------
!       TL of LATTEX: see LATTEX for more information.
!       Remarks:
!        - TL of SL3TL not yet coded.
!        - TL of SL2TL: only K[X]LAG = 3 is coded.
!        - LATTEX_DNT is its own tangent linear code.

!**   Interface.
!     ----------
!        *CALL* *LATTEXTL(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of work.
!          KPROF   - depth of work.
!          PDTS2   - 0.5*time step for the first time-integration step of
!                    a leap-frog scheme or all time-integration steps of
!                    a two-time level scheme; time step for the following
!                    time-integration steps of a leap-frog scheme.
!          PBT     - PDTS2*BETADT (BETADT is in YOMDYN) or zero 
!                    according to the value of CDCONF.
!          PBDT    - PBT if semi-implicit scheme with unreduced
!                    divergence, PBT*(c**2/GM**2) if semi-implicit
!                    scheme with reduced divergence.
!          PESGP   - (1 + uncentering factor).
!          PESGM   - (1 - uncentering factor).
!          PRCORI  - Coriolis parameter "f = 2 Omega sin(theta)".
!          POMVRL  - zonal component of "2 Omega vec a k".
!          POMVRM  - meridian component of "2 Omega vec a k".
!          POROG   - surface orography.
!          POROGL  - zonal component of the surface orography gradient.
!          POROGM  - meridian component of the surface orography gradient.
!          PGAGT0L - semi-implicit term at time t for U-wind equation.
!          PGAGT0M - semi-implicit term at time t for V-wind equation.
!          PTOD0   - semi-implicit term at time t for temperature equation.
!          PSGRTL  - zonal component of the pressure force term
!                    (grad(phi)+RT grad(p)).
!          PSGRTM  - meridian component of the pressure force term.
!          PKAP    - "R/cp" at full levels at t.
!          PVVEL   - "omega/prehyd" at full levels a t.
!          PRDELP  - "1/(pressure depth of layers)" at t.
!          PEVEL   - "etadot (d prehyd/d eta)" at half levels at t.
!          PGFL    - GFL at full levels at t.

!        INPUT/OUTPUT:
!          PGMV    - GMV variables at t-dt and t.
!          PGMVT1  - GMV variables at t+dt.
!          PB1     - "SLBUF1" buffer for interpolations.
!          PB2     - "SLBUF2" buffer.

!        TRAJECTORY INPUT:
!          PGAGT5L - semi-implicit term at time t for U-wind equation.
!          PGAGT5M - semi-implicit term at time t for V-wind equation.
!          PTOD5   - semi-implicit term at time t for temperature equation.
!          PSGRT5L - zonal component of the pressure force term.
!          PSGRT5M - meridian component of the pressure force term.
!          PKAP5   - "R/cp" at full levels at t.
!          PVVEL5  - "omega/prehyd" at full levels a t.
!          PRDELP5 - "1/(pressure depth of layers)" at t.
!          PEVEL5  - "etadot (d prehyd/d eta)" at half levels at t.
!          PGMV5   - trajectory for PGMV.
!          PGFL5   - GFL at full levels at t.

!        TRAJECTORY INPUT/OUTPUT:
!          PUNLT95 - nonlinear term in U-equn at t-dt.
!          PVNLT95 - nonlinear term in V-equn at t-dt.
!          PTNLT95 - nonlinear term in T-equn at t-dt.
!          PB15    - trajectory for PB1.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by LACDYNTL.

!     Reference.
!     ----------
!             Arpege documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        C. Temperton (ECMWF) 

!     Modifications.
!     --------------
!        Original : 99/08/10.
!        Modified 2000-11 by A. Untch: vertical finite element scheme
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Untch  June-2005: TL of Rayleigh friction
!        K.Yessad  Mar-2006: minor reorganisation to match better with LATTEX.
!        K.Yessad  Mar-2007: make the code more modular, cf. the direct code.
!        K. Yessad Aug 2008: rationalisation of dummy argument interfaces
!        K. Yessad Nov 2008: rationalisation of dummy argument interfaces
!        P. Bechtold+A. Untch 26-10-2008: add LEGWWMS switch for non-orogr. GWD
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RD
USE YOMCT0   , ONLY : LRFRIC  ,LTWOTL
USE YOMCT3   , ONLY : NSTEP
USE YOMCVER  , ONLY : LVERTFE
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG   ,NFLSA    ,NFLEN
USE YOMDYN   , ONLY : NWLAG    ,NTLAG    ,RKRF     ,XIDT     ,LSETTLS  , &
 & LADVF    ,L2TLFF   ,RCMSLP0  ,RCORDIH  ,RCORDIF
USE YOMGMV   , ONLY : NDIMGMV  ,YT0      ,YT1      ,YT9      ,YT5
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB15,NFLDSLB2
USE YOMSTA   , ONLY : RTSUR
USE YOMTRSL  , ONLY : NTRSLTYPE
USE YOM_YGFL , ONLY : YGFL     ,YGFLC
USE YOEPHY   , ONLY : LEGWWMS
USE PTRSLB1  , ONLY : MSLB1U0  ,MSLB1V0  ,MSLB1T0  ,MSLB1U9  ,MSLB1V9  , &
 & MSLB1T9  ,MSLB1GFL9
USE PTRSLB15 , ONLY : MSLB1U05 ,MSLB1V05 ,MSLB1T05 ,MSLB1U95 ,MSLB1V95 , &
 & MSLB1T95 ,MSLB1GFL95
USE PTRSLB2  , ONLY : MSLB2USI ,MSLB2VSI ,MSLB2TSI ,MSLB2U15 ,MSLB2V15

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBDT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMVRL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMVRM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGAGT0L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGAGT0M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTOD0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVVEL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL(NPROMA,NFLEVG,YGFL%NDIM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMVT1(NPROMA,NFLEVG,YT1%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGAGT5L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGAGT5M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTOD5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRT5L(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRT5M(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKAP5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVVEL5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL5(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV5(NPROMA,NFLEVG,YT5%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL5(NPROMA,NFLEVG,YGFL%NDIM5) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUNLT95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVNLT95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTNLT95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB15(NPROMA,NFLDSLB15)

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZMOY1U(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZMOY1V(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZMOY1T(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZWT0(NPROMA)
REAL(KIND=JPRB) :: ZWT5(NPROMA)
REAL(KIND=JPRB) :: ZXT9_DUM(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZCXNLT9_DUM(NPROMA,NFLEVG)

INTEGER(KIND=JPIM) :: IPX
INTEGER(KIND=JPIM) :: JLEV,JGFL,JROF
LOGICAL :: LLSETTLSW

REAL(KIND=JPRB) :: ZCMSLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "lattex5.intfb.h"
#include "lattex_dnt.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEXTL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*      1.  PRELIMINARY INITIALISATIONS AND TRAJECTORY.
!       -----------------------------------------------

!       1.1 Preliminary initialisations.

ZCMSLP=RCMSLP0/(RD*RTSUR)

!       1.2 Trajectory.

IF (NTRSLTYPE == 1) THEN
  ! k.y.: I have put all the NTRSLTYPE=1 code in LATTEX5
  !       but this option has never worked nor been validated
  !       (missing code and aborts in some other routines).
  !       ==> code not validated, for this useless option which
  !        is probably a good candidate for pruning.
  CALL LATTEX5(&
   ! --- INPUT --------------------------------------
   & NPROMA,KST,KPROF,NFLEVG,NFLSA,NFLEN,NWLAG,NTLAG,&
   & PDTS2,PBT,PBDT,XIDT,NSTEP,LTWOTL,LSETTLS,LADVF,L2TLFF,&
   & PESGP,PESGM,PRCORI,POMVRL,POMVRM,RCMSLP0,&
   & POROG,POROGL,POROGM,RTSUR,RD,RCORDIH,RCORDIF,&
   & PGAGT5L,PGAGT5M,PTOD5,PSGRT5L,PSGRT5M,&
   & PKAP5,PVVEL5,PRDELP5,PEVEL5,&
   & PGMV5(1,1,YT5%MU),PGMV5(1,1,YT5%MV),PGMV5(1,1,YT5%MT),PGFL5,&
   ! --- INPUT-OUTPUT AND OUTPUT --------------------
   & PUNLT95,PVNLT95,PTNLT95,&
   & PB15(1,MSLB1U05),PB15(1,MSLB1V05),PB15(1,MSLB1T05),&
   & PB15(1,MSLB1U95),PB15(1,MSLB1V95),PB15(1,MSLB1T95),PB15(1,MSLB1GFL95),&
   & PB2(1,MSLB2U15),PB2(1,MSLB2V15))
ENDIF

!     ------------------------------------------------------------------

!*      2.  TREATMENT OF GMV VARIABLES.
!       -------------------------------

!*       2.1   Momentum equation.

! * LSETTLS is replaced by LLSETTLSW=.FALSE. for wind-eqn if VESL>0 because
!   stable extrapolation deteriorates scores without improving stability.
LLSETTLSW=LSETTLS.AND.(.NOT.(PESGP > PESGM))

DO JLEV=1,NFLEVG

  ! * U
  DO JROF=KST,KPROF
    PB2(JROF,MSLB2USI+JLEV-1)=PBT*PGAGT0L(JROF,JLEV)
  ENDDO
  IF (LADVF) THEN
    DO JROF=KST,KPROF
      ZMOY1U(JROF,JLEV) = -PDTS2*PSGRTL(JROF,JLEV)
    ENDDO
  ELSE
    DO JROF=KST,KPROF
      ZMOY1U(JROF,JLEV)=PDTS2* &
       & (PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MV)-PSGRTL(JROF,JLEV))
    ENDDO
  ENDIF
  IF (LRFRIC .AND. .NOT.LEGWWMS) THEN
    DO JROF=KST,KPROF
      ZMOY1U(JROF,JLEV)=ZMOY1U(JROF,JLEV) &
       & -PDTS2*RKRF(JLEV)*PGMV(JROF,JLEV,YT0%MU)
    ENDDO
  ENDIF

  ! * V
  DO JROF=KST,KPROF
    PB2(JROF,MSLB2VSI+JLEV-1)=PBT*PGAGT0M(JROF,JLEV)
  ENDDO
  IF (LADVF) THEN
    DO JROF=KST,KPROF
      ZMOY1V(JROF,JLEV) = -PDTS2*PSGRTM(JROF,JLEV)
    ENDDO
  ELSE
    DO JROF=KST,KPROF
      ZMOY1V(JROF,JLEV)=-PDTS2* &
       & (PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MU)+PSGRTM(JROF,JLEV))
    ENDDO
  ENDIF

ENDDO

IF (LTWOTL) THEN

  CALL LATTEX_DNT(KST,KPROF,LLSETTLSW,NWLAG,PESGP,PESGM,&
   & PGMV(1,1,YT0%MU),ZXT9_DUM,ZMOY1U,&
   & PB2(1,MSLB2USI),PGMV(1,1,YT9%MUNL),PGMVT1(1,1,YT1%MU),&
   & PB1(1,MSLB1U0),PB1(1,MSLB1U9),ZCXNLT9_DUM)

  CALL LATTEX_DNT(KST,KPROF,LLSETTLSW,NWLAG,PESGP,PESGM,&
   & PGMV(1,1,YT0%MV),ZXT9_DUM,ZMOY1V,&
   & PB2(1,MSLB2VSI),PGMV(1,1,YT9%MVNL),PGMVT1(1,1,YT1%MV),&
   & PB1(1,MSLB1V0),PB1(1,MSLB1V9),ZCXNLT9_DUM)

ELSE         !!! T/L OF THREE-TIME-LEVEL SCHEME NOT DONE !!!

  ! not coded (call to LATTEX_TNT)

ENDIF

!*       2.2   Temperature equation.

DO JLEV=1,NFLEVG
  DO JROF=KST,KPROF
    PB2(JROF,MSLB2TSI+JLEV-1)=PBDT(JROF)*PTOD0(JROF,JLEV)
  ENDDO
  IF(LVERTFE) THEN
    DO JROF=KST,KPROF
      ZWT5(JROF)=PEVEL5(JROF,JLEV)
      ZWT0(JROF)=PEVEL(JROF,JLEV)
    ENDDO
  ELSE
    DO JROF=KST,KPROF
      ZWT5(JROF)=0.5_JPRB*(PEVEL5(JROF,JLEV)+PEVEL5(JROF,JLEV-1))  
      ZWT0(JROF)=0.5_JPRB*(PEVEL(JROF,JLEV)+PEVEL(JROF,JLEV-1))  
    ENDDO
  ENDIF
  DO JROF=KST,KPROF
    ZMOY1T(JROF,JLEV)=&
     & PDTS2*(PKAP(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MT)*PVVEL5(JROF,JLEV)&
     & +PKAP5(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MT)*PVVEL5(JROF,JLEV)&
     & +PKAP5(JROF,JLEV)*PGMV5(JROF,JLEV,YT5%MT)*PVVEL(JROF,JLEV))&
     & +PDTS2*ZCMSLP*RCORDIF(JLEV)*PGMV(JROF,JLEV,YT0%MU)*POROGL(JROF)&
     & +PDTS2*ZCMSLP*RCORDIF(JLEV)*PGMV(JROF,JLEV,YT0%MV)*POROGM(JROF)&
     & +PDTS2*ZCMSLP*(RCORDIH(JLEV)-RCORDIH(JLEV-1))&
     & *( ZWT0(JROF)*PRDELP5(JROF,JLEV)+ZWT5(JROF)*PRDELP (JROF,JLEV) )&
     & *POROG(JROF)  
  ENDDO
ENDDO

IF (LTWOTL) THEN

  CALL LATTEX_DNT(KST,KPROF,LSETTLS,NTLAG,PESGP,PESGM,&
   & PGMV(1,1,YT0%MT),ZXT9_DUM,ZMOY1T,&
   & PB2(1,MSLB2TSI),PGMV(1,1,YT9%MTNL),PGMVT1(1,1,YT1%MT),&
   & PB1(1,MSLB1T0),PB1(1,MSLB1T9),ZCXNLT9_DUM)

ELSE         !!! T/L OF THREE-TIME-LEVEL SCHEME NOT DONE !!!

  ! not coded (call to LATTEX_TNT)

ENDIF

!*       2.3   Anhydrostatic variables equations: "pressure departure".

!NYC-NH IF (LNHDYN) THEN
!NYC-NH   TL not yet coded.
!NYC-NH ENDIF

!*       2.4a  Anhydrostatic variables equations: "vertical divergence".

!NYC-NH IF (LNHDYN.AND.(.NOT.LGWADV)) THEN
!NYC-NH   TL not yet coded.
!NYC-NH ENDIF

!*       2.4b  Anhydrostatic variables equations: "gw".

!NYC-NH IF (LNHDYN.AND.LGWADV) THEN
!NYC-NH   TL not yet coded.
!NYC-NH ENDIF

!     ------------------------------------------------------------------

!*      3.  TREATMENT OF GFL VARIABLES.
!       --------------------------------

IF (LTWOTL) THEN

  DO JGFL=1,YGFL%NUMFLDS
    IF(YGFLC(JGFL)%LADV) THEN
      IPX=(YGFLC(JGFL)%MP_SL1-1)*(NFLEN-NFLSA+1)
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1GFL9+IPX+JLEV-NFLSA) = &
           & PB1(JROF,MSLB1GFL9+IPX+JLEV-NFLSA)&
           & +PGFL(JROF,JLEV,YGFLC(JGFL)%MP)
        ENDDO
      ENDDO
    ENDIF
  ENDDO

ELSE    !!! T/L OF THREE-TIME-LEVEL SCHEME NOT DONE !!!

ENDIF

!     ------------------------------------------------------------------

!*      4.  DEALLOCATIONS.
!       ------------------

! Empty for the time being.

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEXTL',1,ZHOOK_HANDLE)
END SUBROUTINE LATTEXTL

