SUBROUTINE LATTEY(KPROMA,KSTART,KPROF,KFLEV,KFLSA,KFLEN,KYLAG,&
 & PBT,KSTEP,LDTWOTL,LDSETTLS,PXI,&
 & PESGP,PESGM,&
 & PD3Y0,PD3Y9,PYNLT9,PYL9,PYT1)  

!**** *LATTEY*   Semi-Lagrangian scheme.
!                Computation of the t and t-dt useful quantities
!                 at grid-points. Equations for tri-dimensional
!                 variable Y for non hydrostatic scheme (non-linear
!                 part of the 3D divergence used in an iterative
!                 algorithm to stabilize the semi-implicit scheme).

!     Purpose.
!     --------
!          This subroutine computes the quantities to be
!          interpolated at each grid-point 
!          of the colocation grid (Gauss grid at all levels). 
!          The quantity here considered is the non-linear part
!          of the 3D divergence used in an iterative
!          algorithm to stabilize the semi-implicit scheme,
!          in the non-hydrostatic scheme.

!          That leads to a way of coding which seems to treat Y like a
!          prognostic variable, like in routine LATTEX, with a t-dt value 
!          (for 3TL SL scheme) or t value (for 2TL SL scheme) equal to zero, 
!          but with the following simplifications and differences:
!          - Explicit adiabatic contribution (term ZMOY1Y) is zero.
!          - There is no physics, so all SI terms are added to PYT1 in
!            this routine and ZYSI is not saved in an array.
!          - For KYLAG=3, buffer PYL9 is used instead of PYL0 to store
!            terms to be tri-linearly interpolated because PYT0 (resp. PYT9)
!            for 2TL SL scheme (resp. for 3TL SL scheme) and physics are zero.
!          - For a 3TL SL scheme, code is identical for KYLAG=2 or 3.
!          - For a 2TL SL scheme, code is identical for KYLAG=2 or 3.

!          The following quantities (after interpolations for quantities,
!          the name of which starts by "PYL") will be necessary in routine
!          GNHSIM to achieve the iterative part of the semi-implicit scheme
!          concerning the non-linear part of the 3D divergence.

!          Called if LPC_OLD=T only.

!**   Interface.
!     ----------
!        *CALL* *LATTEY(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension.
!          KSTART  - first element of work.
!          KPROF   - depth of work.
!          KFLEV   - number of layers.
!          KFLSA   - lower number of layer for arrays with extra-layers.
!          KFLEN   - upper number of layer for arrays with extra-layers.
!          KYLAG   - type of SL discretisation for "Y" equation.
!          PBT     - ZDTS2*BETADT (BETADT is in YOMDYN) or zero 
!                    according to the value of CDCONF (in LACDYN), 
!                    where ZDTS2 is:
!                    0.5*time step for the first time-integration step of
!                    a leap-frog scheme or all time-integration steps of
!                    a two-time level scheme; time step for the following
!                    time-integration steps of a leap-frog scheme.
!          KSTEP   - Number of the current time-step.
!          LDTWOTL - .T./.F.: Two-time level/Three-time level SL scheme.
!          LDSETTLS- .T./.F.: Stable/Conventional extrapolations for SL2TL
!          PXI     - coeff. for "alternative averaging" in 2TL scheme      
!          PESGP   - (1 + uncentering factor).
!          PESGM   - (1 - uncentering factor).
!          PD3Y0   - semi-implicit term at time t for Y equation.
!          PD3Y9   - semi-implicit term at time t-dt for Y equation.

!        INPUT/OUTPUT:
!          PYNLT9  - nonlinear term in Y-equn at t-dt.

!        OUTPUT:
!          PYL9    - semi-lagrangian quantity to be interpolated at the
!                    origin point for the Y equation.
!          PYT1    - t+dt term and other final point terms
!                    for the Y equation.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by LACDYN.

!     Reference.
!     ----------
!             Arpege documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        K. YESSAD (METEO FRANCE/CNRM/GMAP) after LATTEX and code
!        introduced by M. JANOUSEK and R. BUBNOVA concerning NH scheme. 

!     Modifications.
!     --------------
!        Original : APRIL 1996.
!        Modified OCT 1996 by K. YESSAD: removal of variable uncentering,
!          arrays PESGP, PESGM, PESGA become scalar.
!        Modified JUL 1997 by R. BUBNOVA: introducing XIDT, new treatment
!          for 2TL scheme.
!        Modified June 1998 by Martin Charron: Introduction of 
!              LSETTLS=.T. and correction of an error on the variable PYT1.
!        K. YESSAD (July 1998): cleaning
!         - removal of option NODEC=2
!         - clearer writing of options LSETTLS=T/F
!         - clearer writing of options XIDT>0/VESL>0
!         - add uncentering in the stable extrapolation (under LDSETTLS)
!           when VESL>0 to avoid instability in the momentum equation.
!         - activation of KYLAG=1
!         - rewrite code in a more vectorisable way.
!        Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Sep 2008): update comments + cleanings.
!        K. Yessad Nov 2008: rationalisation of dummy argument interfaces
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLSA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KYLAG 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
LOGICAL           ,INTENT(IN)    :: LDTWOTL 
LOGICAL           ,INTENT(IN)    :: LDSETTLS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXI 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PESGM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PD3Y0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PD3Y9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PYNLT9(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PYL9(KPROMA,KFLSA:KFLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PYT1(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZYSI(KPROMA)
REAL(KIND=JPRB) :: ZMOYYNL(KPROMA)
REAL(KIND=JPRB) :: ZYNLT0(KPROMA)

INTEGER(KIND=JPIM) :: JLEV

REAL(KIND=JPRB) :: ZXIGM
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEY',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF (KSTEP <= 0) THEN
  ZXIGM=(1.0_JPRB-PXI)
ELSE
  ZXIGM=(1.0_JPRB+PXI)
ENDIF

!*       1.   Strange variable "Y" equation.

IF (LDTWOTL) THEN

  DO JLEV=1,KFLEV

    ZYSI(KSTART:KPROF)=PBT*PD3Y0(KSTART:KPROF,JLEV)

    IF (KSTEP <= 0) THEN
      ZMOYYNL(KSTART:KPROF)=ZYSI(KSTART:KPROF)
      ZYNLT0(KSTART:KPROF)=ZYSI(KSTART:KPROF)
    ELSE
      IF (PXI > 0.0_JPRB) THEN
        IF (LDSETTLS) THEN
          ZMOYYNL(KSTART:KPROF)=2.0_JPRB*(1.0_JPRB+PXI)*ZYSI(KSTART:KPROF)&
           & -PYNLT9(KSTART:KPROF,JLEV)  
          ZYNLT0(KSTART:KPROF)=(1.0_JPRB+PXI)*ZYSI(KSTART:KPROF)
        ELSE
          ZMOYYNL(KSTART:KPROF)=1.5_JPRB*(1.0_JPRB+PXI)*ZYSI(KSTART:KPROF)&
           & -0.5_JPRB*PYNLT9(KSTART:KPROF,JLEV)  
        ENDIF
      ELSE
        IF (LDSETTLS) THEN
          ZMOYYNL(KSTART:KPROF)=(1.0_JPRB+PESGM)*ZYSI(KSTART:KPROF)&
           & -PYNLT9(KSTART:KPROF,JLEV)  
          ZYNLT0(KSTART:KPROF)=ZYSI(KSTART:KPROF)
        ELSE
          ZMOYYNL(KSTART:KPROF)=1.5_JPRB*ZYSI(KSTART:KPROF)&
           & -0.5_JPRB*PYNLT9(KSTART:KPROF,JLEV)  
        ENDIF
      ENDIF
    ENDIF
    IF (PXI > 0.0_JPRB) THEN
      PYNLT9(KSTART:KPROF,JLEV)=(1.0_JPRB+PXI)*ZYSI(KSTART:KPROF)
    ELSE
      PYNLT9(KSTART:KPROF,JLEV)=ZYSI(KSTART:KPROF)
    ENDIF
    IF (KYLAG == 2.OR.KYLAG == 3) THEN
      IF (PXI > 0.0_JPRB) THEN
        IF (LDSETTLS) THEN
          PYL9(KSTART:KPROF,JLEV)=ZMOYYNL(KSTART:KPROF)&
           & -ZXIGM*ZYSI(KSTART:KPROF)  
          PYT1(KSTART:KPROF,JLEV)=ZYNLT0(KSTART:KPROF)
        ELSE
          PYL9(KSTART:KPROF,JLEV)=ZMOYYNL(KSTART:KPROF)&
           & -ZXIGM*ZYSI(KSTART:KPROF)  
          PYT1(KSTART:KPROF,JLEV)=ZMOYYNL(KSTART:KPROF)
        ENDIF
      ELSE
        IF (LDSETTLS) THEN
          PYL9(KSTART:KPROF,JLEV)=ZMOYYNL(KSTART:KPROF)&
           & -PESGM*ZYSI(KSTART:KPROF)  
          PYT1(KSTART:KPROF,JLEV)=PESGP*ZYNLT0(KSTART:KPROF)
        ELSE
          PYL9(KSTART:KPROF,JLEV)=PESGM*(ZMOYYNL(KSTART:KPROF)&
           & -ZYSI(KSTART:KPROF))  
          PYT1(KSTART:KPROF,JLEV)=PESGP*ZMOYYNL(KSTART:KPROF)
        ENDIF
      ENDIF
    ENDIF

  ENDDO

ELSE

  DO JLEV=1,KFLEV

    PYL9(KSTART:KPROF,JLEV)=PESGM*PBT &
     & *(PD3Y0(KSTART:KPROF,JLEV)-PD3Y9(KSTART:KPROF,JLEV))  
    PYT1(KSTART:KPROF,JLEV)=PESGP*PBT      *PD3Y0(KSTART:KPROF,JLEV)

  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LATTEY',1,ZHOOK_HANDLE)
END SUBROUTINE LATTEY

