!OCL  NOEVAL
SUBROUTINE LAVENT(KST,KPROF,LD2TLFF1,&
 & PVCASRS0F,PRDLR0,PSGRTL,PSGRTM,PDTS2,PRCORI,PRCORIC,PGNORDL,PGNORDM,&
 & PRDELP,PEVEL,PVCW0F,PWDLW0F,PGMV,PB1,PB2)

!**** *LAVENT*   Semi-Lagrangian scheme.
!                Computation of wind components necessary 
!                to find the SL trajectory.

!     Purpose.
!     --------
!          This subroutine computes the wind components at t
!          at each grid-point of the colocation grid (Gauss
!          grid and poles at all levels). These quantities will
!          be used in routine LARMES to find the semi-Lagrangian
!          trajectory.
!          LVERCOR deep-layer eqns: "(a/rs)*wind" is used
!           instead of wind, and extrapolations are done on "(a/rs)*wind".
!          LRWSDLR deep-layer eqns: "(a/r)*wind" is used
!           instead of wind, and extrapolations are done on "(a/r)*wind".

!**   Interface.
!     ----------
!        *CALL* *LAVENT(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST       - first element of work.
!          KPROF     - depth of work.
!          LD2TLFF1  - .T./.F.: Refined treatement of (2*Omega Vec r) at
!                      the origin point when there is t-dt (or t in SL2TL)
!                      physics / Other cases.
!          PVCASRS0F - "a/rs" at full levels at t.
!          PRDLR0    - array containing "r/a", "a/r", "grad(r/a)" at t.
!          PSGRTL    - semi-reduced zonal component of the pressure force grad.
!          PSGRTM    - semi-reduced merid component of the pressure force grad.
!          PDTS2     - 0.5*(time step) for the first time-integration step of
!                      a leap-frog scheme or all time-integration steps of
!                      a two-time level scheme; time step for the following
!                      time-integration steps of a leap-frog scheme.
!          PRCORI    - Coriolis parameter "f = 2 Omega sin(theta)".
!          PRCORIC   - 2 Omega cos(theta).
!          PGNORDL   - zonal component ("Gnordl") of the unit vector
!                      directed towards the true North pole.
!          PGNORDM   - meridian component ("Gnordm") of the unit vector
!                      directed towards the true North pole.
!          PRDELP    - "1/(pressure depth of layers)" at t.
!          PEVEL     - "etadot (d prehyd/d eta)" at t.
!          PVCW0F    - pseudo-vertical velocity "W = Drs/Dt" at full lev at t.
!          PWDLW0F   - vertical velocity " w = Dr/Dt " at full levels at t.

!        INPUT/OUTPUT:
!          PGMV      - GMV variables at t-dt and t.
!          PB1       - "SLBUF1" buffer for interpolations.

!        OUTPUT:
!          PB2       - "SLBUF2" buffer.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by LACDYN.

!     Reference.
!     ----------
!             Arpege documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        K. YESSAD (METEO FRANCE/CNRM/GMAP) after old loops 3202 and
!        3212 of LACDYN.
!        DO loops have been updated according to F90 norms.

! Modifications
! -------------
!   Original : AUGUST 1995.
!   Clive Temperton 96-03-06 : time-filtering removed from 2TL scheme
!   Clive Temperton 98-03-04 : repair 3TL semi-Lagrangian scheme
!   K. YESSAD 98-05-06: new quantities PURL9 and PVRL9 if LD2TLFF1=.T..
!   I. GOSPODINOV and K. YESSAD 98-12-18: option LELTRA.
!   I. GOSPODINOV and K. YESSAD 99-06-07: physics in option LELTRA.
!   K. YESSAD 01-06-20:
!    - relaxation of thin layer hypothesis.
!    - correction of false comments.
!    - some cleanings, especially for better vectorization.
!   Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!      Mar-2002 J. Vivoda  PC schemes for NH dynamics (LPC_XXXX keys)
!   12-Oct-2002 J. Masek   PC bugfix
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   M.Hortal 01-Dec-2003 Extra-fields from the dynamics
!   M.Hortal 01-Dec-2003 Vertical trajectory smoothing
!   09-Jun-2004 J. Masek   NH cleaning (LDFULLIMP)
!   01-Jul-2004 K. Yessad  Make clearer the tests for PC scheme.
!   15-Sep-2004 Y. Seity : Add LSQUALL case for AROME academic 2D squall line
!   26-Nov-2007 N. Wedi  : bug correction
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   K. Yessad Aug 2008: rationalisation of dummy argument interfaces + optimis.
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YOMCST   , ONLY : RA
USE YOMCT0   , ONLY : LAROME   ,LPC_FULL ,LPC_NESCT,LPC_CHEAP,LVERCOR  ,&
 & LTWOTL
USE YOMCT3   , ONLY : NSTEP
USE YOMCVER  , ONLY : LVERTFE  ,LSVTSM
USE YOMDIM   , ONLY : NPROMA   ,NPROMVC  ,NPROMDLW ,NPROMDLR ,NFLEVG   ,NFLSA
USE YOMDYNA  , ONLY : LRWSDLW  ,LRWSDLR 
USE YOMDYN   , ONLY : LELTRA   ,LSETTLST ,NCURRENT_ITER
USE YOMGEM   , ONLY : VETAH
USE YOMGMV   , ONLY : NDIMGMV  ,YT0      ,YT9
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2
USE YOMARAR  , ONLY : LSQUALL  ,VSQUALL
USE PTRSLB1  , ONLY : MSLB1U9  ,MSLB1V9  ,MSLB1UR0 ,MSLB1VR0 ,MSLB1WR0 ,&
 & MSLB1WRA ,MSLB1UR9 ,MSLB1VR9
USE PTRSLB2  , ONLY : MSLB2URL ,MSLB2VRL ,MSLB2WRL
USE PAR_RDLR , ONLY : JP_ASRF  ,JP_DIMR0

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LD2TLFF1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCASRS0F(NPROMVC,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDLR0(NPROMDLR,0:NFLEVG,JP_DIMR0)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTL(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTM(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORIC(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDL(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNORDM(NPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCW0F(NPROMVC,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWDLW0F(NPROMDLW,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PB2(NPROMA,NFLDSLB2)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) ::JROF, JLEV
REAL(KIND=JPRB) :: ZWRL0(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZUSA
LOGICAL :: LLCT, LLSETTLST, LLPC_NESCT

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAVENT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

ZUSA=1.0_JPRB/RA

LLCT = (LPC_FULL .AND. NCURRENT_ITER > 0)

IF (LPC_CHEAP.AND.LPC_NESCT.AND.(NCURRENT_ITER==0)) THEN
  ! Not extrapolating scheme is replaced by SETTLS-extrapolating scheme.
  LLSETTLST=.TRUE.
  LLPC_NESCT=.FALSE.
ELSE
  LLSETTLST=LSETTLST.AND.(.NOT.LELTRA)
  LLPC_NESCT=LPC_NESCT
ENDIF

!     * Computation of "etadot" at full levels.

IF(LVERTFE) THEN
  ! PEVEL is at full levels.
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZWRL0(JROF,JLEV)=PEVEL(JROF,JLEV)&
       & *(VETAH(JLEV)-VETAH(JLEV-1))*PRDELP(JROF,JLEV)
    ENDDO
  ENDDO
ELSE
  ! PEVEL is at half levels.
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZWRL0(JROF,JLEV)=&
       & 0.5_JPRB*(PEVEL(JROF,JLEV)+PEVEL(JROF,JLEV-1))&
       & *(VETAH(JLEV)-VETAH(JLEV-1))*PRDELP(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

!     * Computation of wind components necessary to find the SL trajectory.

IF(LTWOTL.AND.LSVTSM) THEN
  DO JLEV=1,NFLEVG
    PB1(KST:KPROF,MSLB1WRA+JLEV-NFLSA)=ZWRL0(KST:KPROF,JLEV)
  ENDDO
ENDIF


IF (LTWOTL.AND.LELTRA) THEN

  ! k.y.: some identical wind eqn RHS intermediate calculations are done also
  !       in CPEULDYN, LATTEX, CPDYDDH: in the future it would be desirable to
  !       do them once, in a new routine GP_TNDLAGADIAB_UV, called by CPG_GP.

  ! Use the RHS of momentum equation.
  IF (LVERCOR) THEN
    ! * Eqn of (a/rs)*wind with additional W-terms. 
    ! a/ DV/Dt:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU)-PDTS2*&
         & (-PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MV)&
         & +PGNORDM(JROF)*PRCORIC(JROF)*PVCW0F(JROF,JLEV)&
         & +ZUSA*PVCASRS0F(JROF,JLEV)*PVCW0F(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MU)&
         & +PVCASRS0F(JROF,JLEV)*PSGRTL(JROF,JLEV))&
         & +PB1(JROF,MSLB1U9+JLEV-NFLSA)*0.5_JPRB
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV)-PDTS2*&
         & (PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MU)&
         & -PGNORDL(JROF)*PRCORIC(JROF)*PVCW0F(JROF,JLEV)&
         & +ZUSA*PVCASRS0F(JROF,JLEV)*PVCW0F(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MV)&
         & +PVCASRS0F(JROF,JLEV)*PSGRTM(JROF,JLEV))&
         & +PB1(JROF,MSLB1V9+JLEV-NFLSA)*0.5_JPRB
      ENDDO
    ENDDO
    ! b/ (rs/a) * D((a/rs)*V)/Dt:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PB1(JROF,MSLB1UR0+JLEV-NFLSA)-PDTS2*ZUSA&
         & *PVCASRS0F(JROF,JLEV)*PVCW0F(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MU)
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PB1(JROF,MSLB1VR0+JLEV-NFLSA)-PDTS2*ZUSA&
         & *PVCASRS0F(JROF,JLEV)*PVCW0F(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MV)
      ENDDO
    ENDDO
    ! c/ D((a/rs)*V)/Dt:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PB1(JROF,MSLB1UR0+JLEV-NFLSA) &
         & *PVCASRS0F(JROF,JLEV)
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PB1(JROF,MSLB1VR0+JLEV-NFLSA) &
         & *PVCASRS0F(JROF,JLEV)
      ENDDO
    ENDDO
  ELSEIF (LRWSDLW.AND.(.NOT.LRWSDLR)) THEN
    ! * Eqn of wind with additional W-terms. 
    ! a/ DV/Dt:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU)-PDTS2*&
         & (-PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MV)&
         & +PGNORDM(JROF)*PRCORIC(JROF)*PWDLW0F(JROF,JLEV)&
         & +ZUSA*PWDLW0F(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MU)&
         & +PSGRTL(JROF,JLEV))&
         & +PB1(JROF,MSLB1U9+JLEV-NFLSA)*0.5_JPRB
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV)-PDTS2*&
         & (PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MU)&
         & -PGNORDL(JROF)*PRCORIC(JROF)*PWDLW0F(JROF,JLEV)&
         & +ZUSA*PWDLW0F(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MV)&
         & +PSGRTM(JROF,JLEV))&
         & +PB1(JROF,MSLB1V9+JLEV-NFLSA)*0.5_JPRB
      ENDDO
    ENDDO
  ELSEIF (LRWSDLW.AND.LRWSDLR) THEN
    ! * Eqn of (a/r)*wind with additional W-terms. 
    ! a/ DV/Dt:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU)-PDTS2*&
         & (-PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MV)&
         & +PGNORDM(JROF)*PRCORIC(JROF)*PWDLW0F(JROF,JLEV)+ZUSA* &
         & PRDLR0(JROF,JLEV,JP_ASRF)*PWDLW0F(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MU)&
         & +PRDLR0(JROF,JLEV,JP_ASRF)*PSGRTL(JROF,JLEV))&
         & +PB1(JROF,MSLB1U9+JLEV-NFLSA)*0.5_JPRB 
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV)-PDTS2*&
         & (PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MU)&
         & -PGNORDL(JROF)*PRCORIC(JROF)*PWDLW0F(JROF,JLEV)+ZUSA* &
         & PRDLR0(JROF,JLEV,JP_ASRF)*PWDLW0F(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MV)&
         & +PRDLR0(JROF,JLEV,JP_ASRF)*PSGRTM(JROF,JLEV))&
         & +PB1(JROF,MSLB1V9+JLEV-NFLSA)*0.5_JPRB
      ENDDO
    ENDDO
    ! b/ (r/a) * D((a/r)*V)/Dt:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PB1(JROF,MSLB1UR0+JLEV-NFLSA)-PDTS2*ZUSA*&
         & PRDLR0(JROF,JLEV,JP_ASRF)*PWDLW0F(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MU)
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PB1(JROF,MSLB1VR0+JLEV-NFLSA)-PDTS2*ZUSA*&
         & PRDLR0(JROF,JLEV,JP_ASRF)*PWDLW0F(JROF,JLEV)*PGMV(JROF,JLEV,YT0%MV)
      ENDDO
    ENDDO
    ! c/ D((a/r)*V)/Dt:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PB1(JROF,MSLB1UR0+JLEV-NFLSA) &
         & *PRDLR0(JROF,JLEV,JP_ASRF)
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PB1(JROF,MSLB1VR0+JLEV-NFLSA) &
         & *PRDLR0(JROF,JLEV,JP_ASRF)
      ENDDO
    ENDDO
  ELSEIF ((.NOT.LRWSDLW).AND.LRWSDLR) THEN
    ! * Eqn of (a/r)*wind, but without W-terms. 
    ! a/ DV/Dt:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU)-PDTS2*&
         & (-PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MV)&
         & +PRDLR0(JROF,JLEV,JP_ASRF)*PSGRTL(JROF,JLEV))&
         & +PB1(JROF,MSLB1U9+JLEV-NFLSA)*0.5_JPRB 
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV)-PDTS2*&
         & (PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MU)&
         & +PRDLR0(JROF,JLEV,JP_ASRF)*PSGRTM(JROF,JLEV))&
         & +PB1(JROF,MSLB1V9+JLEV-NFLSA)*0.5_JPRB
      ENDDO
    ENDDO
    ! b/ D((a/r)*V)/Dt:
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PB1(JROF,MSLB1UR0+JLEV-NFLSA) &
         & *PRDLR0(JROF,JLEV,JP_ASRF)
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PB1(JROF,MSLB1VR0+JLEV-NFLSA) &
         & *PRDLR0(JROF,JLEV,JP_ASRF)
      ENDDO
    ENDDO
  ELSE
    ! * Eqn of wind without W-terms.
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU)-PDTS2*&
         & (-PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MV)&
         & +PSGRTL(JROF,JLEV))+PB1(JROF,MSLB1U9+JLEV-NFLSA)*0.5  
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV)-PDTS2*&
         & (PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MU)&
         & +PSGRTM(JROF,JLEV))+PB1(JROF,MSLB1V9+JLEV-NFLSA)*0.5  
      ENDDO
    ENDDO
  ENDIF

  DO JLEV=1,NFLEVG
    PB1(KST:KPROF,MSLB1WR0+JLEV-NFLSA)=ZWRL0(KST:KPROF,JLEV)
    PB2(KST:KPROF,MSLB2URL+JLEV-1)=PB1(KST:KPROF,MSLB1UR0+JLEV-NFLSA)
    PB2(KST:KPROF,MSLB2VRL+JLEV-1)=PB1(KST:KPROF,MSLB1VR0+JLEV-NFLSA)
    PB2(KST:KPROF,MSLB2WRL+JLEV-1)=PB1(KST:KPROF,MSLB1WR0+JLEV-NFLSA)
  ENDDO

  IF (LD2TLFF1) THEN
    DO JLEV=1,NFLEVG
      PB1(KST:KPROF,MSLB1UR9+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT0%MU)
      PB1(KST:KPROF,MSLB1VR9+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT0%MV)
    ENDDO
    IF (LVERCOR) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR9+JLEV-NFLSA)=PB1(JROF,MSLB1UR9+JLEV-NFLSA) &
           & *PVCASRS0F(JROF,JLEV)
          PB1(JROF,MSLB1VR9+JLEV-NFLSA)=PB1(JROF,MSLB1VR9+JLEV-NFLSA) &
           & *PVCASRS0F(JROF,JLEV)
        ENDDO
      ENDDO
    ELSEIF (LRWSDLR) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR9+JLEV-NFLSA)=PB1(JROF,MSLB1UR9+JLEV-NFLSA) &
           & *PRDLR0(JROF,JLEV,JP_ASRF)
          PB1(JROF,MSLB1VR9+JLEV-NFLSA)=PB1(JROF,MSLB1VR9+JLEV-NFLSA) &
           & *PRDLR0(JROF,JLEV,JP_ASRF)
        ENDDO
      ENDDO
    ENDIF
  ENDIF

ELSEIF (LTWOTL.AND.(.NOT.LELTRA)) THEN

  IF (.NOT.LLCT) THEN

    ! * Cases:
    !   NSITER=0
    !   Predictor step of LPC_OLD
    !   Predictor step of LPC_FULL

    IF ( NSTEP <= 0 .OR. LLPC_NESCT ) THEN

      ! * non-extrapolating scheme X(t+dt) = X(t) 
      IF (LVERCOR) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU) &
             & *PVCASRS0F(JROF,JLEV)
            PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV) &
             & *PVCASRS0F(JROF,JLEV)
          ENDDO
        ENDDO
      ELSEIF (LRWSDLR) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU) &
             & *PRDLR0(JROF,JLEV,JP_ASRF)
            PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV) &
             & *PRDLR0(JROF,JLEV,JP_ASRF)
          ENDDO
        ENDDO
      ELSE
        DO JLEV=1,NFLEVG
          PB1(KST:KPROF,MSLB1UR0+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT0%MU)
          PB1(KST:KPROF,MSLB1VR0+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT0%MV)
        ENDDO
      ENDIF
      DO JLEV=1,NFLEVG
        PB1(KST:KPROF,MSLB1WR0+JLEV-NFLSA)=ZWRL0(KST:KPROF,JLEV)
        PB2(KST:KPROF,MSLB2URL+JLEV-1)=PB1(KST:KPROF,MSLB1UR0+JLEV-NFLSA)
        PB2(KST:KPROF,MSLB2VRL+JLEV-1)=PB1(KST:KPROF,MSLB1VR0+JLEV-NFLSA)
        PB2(KST:KPROF,MSLB2WRL+JLEV-1)=PB1(KST:KPROF,MSLB1WR0+JLEV-NFLSA)
      ENDDO

    ELSEIF (LLSETTLST) THEN

      ! * SETTLS-extrapolating schemes X(t+dt) = f( X(t),X(t-dt) )
      IF (LVERCOR) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB1(JROF,MSLB1UR0+JLEV-NFLSA)=&
             & 2.0_JPRB*PGMV(JROF,JLEV,YT0%MU)*PVCASRS0F(JROF,JLEV)&
             & -PGMV(JROF,JLEV,YT9%MU)*PGMV(JROF,JLEV,YT9%MVCASRS)
            PB1(JROF,MSLB1VR0+JLEV-NFLSA)=&
             & 2.0_JPRB*PGMV(JROF,JLEV,YT0%MV)*PVCASRS0F(JROF,JLEV)&
             & -PGMV(JROF,JLEV,YT9%MV)*PGMV(JROF,JLEV,YT9%MVCASRS)
          ENDDO
        ENDDO
      ELSEIF (LRWSDLR) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB1(JROF,MSLB1UR0+JLEV-NFLSA)=&
             & 2.0_JPRB*PGMV(JROF,JLEV,YT0%MU)*PRDLR0(JROF,JLEV,JP_ASRF)&
             & -PGMV(JROF,JLEV,YT9%MU)*PGMV(JROF,JLEV,YT9%MRDLASR)
            PB1(JROF,MSLB1VR0+JLEV-NFLSA)=&
             & 2.0_JPRB*PGMV(JROF,JLEV,YT0%MV)*PRDLR0(JROF,JLEV,JP_ASRF)&
             & -PGMV(JROF,JLEV,YT9%MV)*PGMV(JROF,JLEV,YT9%MRDLASR)
          ENDDO
        ENDDO
      ELSE
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB1(JROF,MSLB1UR0+JLEV-NFLSA)=2.0_JPRB*PGMV(JROF,JLEV,YT0%MU)&
             & -PGMV(JROF,JLEV,YT9%MU)
            PB1(JROF,MSLB1VR0+JLEV-NFLSA)=2.0_JPRB*PGMV(JROF,JLEV,YT0%MV)&
             & -PGMV(JROF,JLEV,YT9%MV)
          ENDDO
        ENDDO
      ENDIF
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1WR0+JLEV-NFLSA)=2.0_JPRB*ZWRL0(JROF,JLEV)&
           & -PGMV(JROF,JLEV,YT9%MEDOT)
        ENDDO
      ENDDO
      IF (LVERCOR) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB2(JROF,MSLB2URL+JLEV-1)=PGMV(JROF,JLEV,YT0%MU) &
             & *PVCASRS0F(JROF,JLEV)
            PB2(JROF,MSLB2VRL+JLEV-1)=PGMV(JROF,JLEV,YT0%MV) &
             & *PVCASRS0F(JROF,JLEV)
          ENDDO
        ENDDO
      ELSEIF (LRWSDLR) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB2(JROF,MSLB2URL+JLEV-1)=PGMV(JROF,JLEV,YT0%MU) &
             & *PRDLR0(JROF,JLEV,JP_ASRF)
            PB2(JROF,MSLB2VRL+JLEV-1)=PGMV(JROF,JLEV,YT0%MV) &
             & *PRDLR0(JROF,JLEV,JP_ASRF)
          ENDDO
        ENDDO
      ELSE
        DO JLEV=1,NFLEVG
          PB2(KST:KPROF,MSLB2URL+JLEV-1)=PGMV(KST:KPROF,JLEV,YT0%MU)
          PB2(KST:KPROF,MSLB2VRL+JLEV-1)=PGMV(KST:KPROF,JLEV,YT0%MV)
        ENDDO
      ENDIF
      PB2(KST:KPROF,MSLB2WRL:MSLB2WRL+NFLEVG-1)=ZWRL0(KST:KPROF,1:NFLEVG)

    ELSE

      ! * non-SETTLS-extrapolating schemes X(t+dt) = f( X(t),X(t-dt) )
      IF (LVERCOR) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB1(JROF,MSLB1UR0+JLEV-NFLSA)=&
             & 1.5_JPRB*PGMV(JROF,JLEV,YT0%MU)*PVCASRS0F(JROF,JLEV)&
             & -0.5_JPRB*PGMV(JROF,JLEV,YT9%MU)*PGMV(JROF,JLEV,YT9%MVCASRS)
            PB1(JROF,MSLB1VR0+JLEV-NFLSA)=&
             & 1.5_JPRB*PGMV(JROF,JLEV,YT0%MV)*PVCASRS0F(JROF,JLEV)&
             & -0.5_JPRB*PGMV(JROF,JLEV,YT9%MV)*PGMV(JROF,JLEV,YT9%MVCASRS)
          ENDDO
        ENDDO
      ELSEIF (LRWSDLR) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB1(JROF,MSLB1UR0+JLEV-NFLSA)=&
             & 1.5_JPRB*PGMV(JROF,JLEV,YT0%MU)*PRDLR0(JROF,JLEV,JP_ASRF)&
             & -0.5_JPRB*PGMV(JROF,JLEV,YT9%MU)*PGMV(JROF,JLEV,YT9%MRDLASR)
            PB1(JROF,MSLB1VR0+JLEV-NFLSA)=&
             & 1.5_JPRB*PGMV(JROF,JLEV,YT0%MV)*PRDLR0(JROF,JLEV,JP_ASRF)&
             & -0.5_JPRB*PGMV(JROF,JLEV,YT9%MV)*PGMV(JROF,JLEV,YT9%MRDLASR)
          ENDDO
        ENDDO
      ELSE
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB1(JROF,MSLB1UR0+JLEV-NFLSA)=1.5_JPRB*PGMV(JROF,JLEV,YT0%MU)&
             & -0.5_JPRB*PGMV(JROF,JLEV,YT9%MU)
            PB1(JROF,MSLB1VR0+JLEV-NFLSA)=1.5_JPRB*PGMV(JROF,JLEV,YT0%MV)&
             & -0.5_JPRB*PGMV(JROF,JLEV,YT9%MV)
          ENDDO
        ENDDO
      ENDIF
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1WR0+JLEV-NFLSA)=1.5_JPRB*ZWRL0(JROF,JLEV)&
           & -0.5_JPRB*PGMV(JROF,JLEV,YT9%MEDOT)
        ENDDO
      ENDDO
      DO JLEV=1,NFLEVG
        PB2(KST:KPROF,MSLB2URL+JLEV-1)=PB1(KST:KPROF,MSLB1UR0+JLEV-NFLSA)
        PB2(KST:KPROF,MSLB2VRL+JLEV-1)=PB1(KST:KPROF,MSLB1VR0+JLEV-NFLSA)
        PB2(KST:KPROF,MSLB2WRL+JLEV-1)=PB1(KST:KPROF,MSLB1WR0+JLEV-NFLSA)
      ENDDO

    ENDIF

    PGMV(KST:KPROF,1:NFLEVG,YT9%MEDOT)=ZWRL0(KST:KPROF,1:NFLEVG)
    IF (LVERCOR) THEN
      PGMV(KST:KPROF,1:NFLEVG,YT9%MVCASRS)=PVCASRS0F(KST:KPROF,1:NFLEVG)
    ELSEIF (LRWSDLR) THEN
      PGMV(KST:KPROF,1:NFLEVG,YT9%MRDLASR)=PRDLR0(KST:KPROF,1:NFLEVG,JP_ASRF)
    ENDIF

  ELSE 

    ! * Corrector step of LPC_FULL:

    IF (LVERCOR) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT9%MU) &
           & *PGMV(JROF,JLEV,YT9%MVCASRS)
          PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT9%MV) &
           & *PGMV(JROF,JLEV,YT9%MVCASRS)
        ENDDO
      ENDDO
    ELSEIF (LRWSDLR) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT9%MU) &
           & *PGMV(JROF,JLEV,YT9%MRDLASR)
          PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT9%MV) &
           & *PGMV(JROF,JLEV,YT9%MRDLASR)
        ENDDO
      ENDDO
    ELSE
      DO JLEV=1,NFLEVG
        PB1(KST:KPROF,MSLB1UR0+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT9%MU)
        PB1(KST:KPROF,MSLB1VR0+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT9%MV)
      ENDDO
    ENDIF
    DO JLEV=1,NFLEVG
      PB1(KST:KPROF,MSLB1WR0+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT9%MEDOT)
    ENDDO

    IF (LVERCOR) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB2(JROF,MSLB2URL+JLEV-1)=PGMV(JROF,JLEV,YT0%MU)*PVCASRS0F(JROF,JLEV)
          PB2(JROF,MSLB2VRL+JLEV-1)=PGMV(JROF,JLEV,YT0%MV)*PVCASRS0F(JROF,JLEV)
        ENDDO
      ENDDO
    ELSEIF (LRWSDLR) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB2(JROF,MSLB2URL+JLEV-1)=PGMV(JROF,JLEV,YT0%MU) &
           & *PRDLR0(JROF,JLEV,JP_ASRF)
          PB2(JROF,MSLB2VRL+JLEV-1)=PGMV(JROF,JLEV,YT0%MV) &
           & *PRDLR0(JROF,JLEV,JP_ASRF)
        ENDDO
      ENDDO
    ELSE
      DO JLEV=1,NFLEVG
        PB2(KST:KPROF,MSLB2URL+JLEV-1)=PGMV(KST:KPROF,JLEV,YT0%MU)
        PB2(KST:KPROF,MSLB2VRL+JLEV-1)=PGMV(KST:KPROF,JLEV,YT0%MV)
      ENDDO
    ENDIF
    DO JLEV=1,NFLEVG
      PB2(KST:KPROF,MSLB2WRL+JLEV-1)=ZWRL0(KST:KPROF,JLEV)
    ENDDO

  ENDIF

  IF (LD2TLFF1) THEN
    DO JLEV=1,NFLEVG
      PB1(KST:KPROF,MSLB1UR9+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT0%MU)
      PB1(KST:KPROF,MSLB1VR9+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT0%MV)
    ENDDO
    IF (LVERCOR) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR9+JLEV-NFLSA)=PB1(JROF,MSLB1UR9+JLEV-NFLSA) &
           & *PVCASRS0F(JROF,JLEV)
          PB1(JROF,MSLB1VR9+JLEV-NFLSA)=PB1(JROF,MSLB1VR9+JLEV-NFLSA) &
           & *PVCASRS0F(JROF,JLEV)
        ENDDO
      ENDDO
    ELSEIF (LRWSDLR) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR9+JLEV-NFLSA)=PB1(JROF,MSLB1UR9+JLEV-NFLSA) &
           & *PRDLR0(JROF,JLEV,JP_ASRF)
          PB1(JROF,MSLB1VR9+JLEV-NFLSA)=PB1(JROF,MSLB1VR9+JLEV-NFLSA) &
           & *PRDLR0(JROF,JLEV,JP_ASRF)
        ENDDO
      ENDDO
    ENDIF
  ENDIF

ELSE

  ! SL3TL:

  IF (LVERCOR) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU) &
         & *PVCASRS0F(JROF,JLEV)
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV) &
         & *PVCASRS0F(JROF,JLEV)
      ENDDO
    ENDDO
  ELSEIF (LRWSDLR) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU) &
         & *PRDLR0(JROF,JLEV,JP_ASRF)
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV) &
         & *PRDLR0(JROF,JLEV,JP_ASRF)
      ENDDO
    ENDDO
  ELSE
    DO JLEV=1,NFLEVG
      PB1(KST:KPROF,MSLB1UR0+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT0%MU)
      PB1(KST:KPROF,MSLB1VR0+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT0%MV)
    ENDDO
  ENDIF
  DO JLEV=1,NFLEVG
    PB1(KST:KPROF,MSLB1WR0+JLEV-NFLSA)=ZWRL0(KST:KPROF,JLEV)
    PB2(KST:KPROF,MSLB2URL+JLEV-1)=PB1(KST:KPROF,MSLB1UR0+JLEV-NFLSA)
    PB2(KST:KPROF,MSLB2VRL+JLEV-1)=PB1(KST:KPROF,MSLB1VR0+JLEV-NFLSA)
    PB2(KST:KPROF,MSLB2WRL+JLEV-1)=PB1(KST:KPROF,MSLB1WR0+JLEV-NFLSA)
  ENDDO

  IF (LD2TLFF1) THEN
    DO JLEV=1,NFLEVG
      PB1(KST:KPROF,MSLB1UR9+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT9%MU)
      PB1(KST:KPROF,MSLB1VR9+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT9%MV)
    ENDDO
    IF (LVERCOR) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR9+JLEV-NFLSA)=PB1(JROF,MSLB1UR9+JLEV-NFLSA) &
           & *PGMV(JROF,JLEV,YT9%MVCASRS)
          PB1(JROF,MSLB1VR9+JLEV-NFLSA)=PB1(JROF,MSLB1VR9+JLEV-NFLSA) &
           & *PGMV(JROF,JLEV,YT9%MVCASRS)
        ENDDO
      ENDDO
    ELSEIF (LRWSDLR) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR9+JLEV-NFLSA)=PB1(JROF,MSLB1UR9+JLEV-NFLSA) &
           & *PGMV(JROF,JLEV,YT9%MRDLASR)
          PB1(JROF,MSLB1VR9+JLEV-NFLSA)=PB1(JROF,MSLB1VR9+JLEV-NFLSA) &
           & *PGMV(JROF,JLEV,YT9%MRDLASR)
        ENDDO
      ENDDO
    ENDIF
  ENDIF

ENDIF

IF (LAROME.AND.LSQUALL) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PB1(JROF,MSLB1VR0+JLEV-NFLSA)+VSQUALL
      PB2(JROF,MSLB2VRL+JLEV-1)=PB2(JROF,MSLB2VRL+JLEV-1)+VSQUALL
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAVENT',1,ZHOOK_HANDLE)
END SUBROUTINE LAVENT

