SUBROUTINE LAVENTAD(KST,KPROF,LD2TLFF1,PSGRTL,PSGRTM,PDTS2,PRCORI,&
 & PRDELP,PEVEL,PGMV,PB1,PB2,PRDELP5,PEVEL5)

!**** *LAVENTAD* Semi-Lagrangian scheme.  (adjoint version)
!                Computation of wind components necessary 
!                to find the SL trajectory.

!     Purpose.
!     --------
!          AD code of LAVENT; see comments in LAVENT.
!          The AD code does not match exactly the structure of routine LAVENT
!           because some direct code options have not been yet implemented
!           in the AD code.

!**   Interface.
!     ----------
!        *CALL* *LAVENTAD(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of work.
!          KPROF   - depth of work.
!          LD2TLFF1- .T./.F.: Refined treatement of (2*Omega Vec r) at
!                    the origin point when there is t-dt (or t in SL2TL)
!                    physics / Other cases.
!          PSGRTL  - zonal component of the pressure force
!                    (grad(phi)+RT grad(p)).
!          PSGRTM  - meridian component of the pressure force.
!          PDTS2   - 0.5*(time step) for the first time-integration step of
!                    a leap-frog scheme or all time-integration steps of
!                    a two-time level scheme; time step for the following
!                    time-integration steps of a leap-frog scheme.
!          PRCORI  - Coriolis parameter.

!        INPUT/OUTPUT:
!          PRDELP  - 1/(delta prehyd) at full levels.
!          PEVEL   - "etadot (d prehyd/d eta)" at t.
!          PGMV    - GMV variables at t-dt and t.
!          PB1     - "SLBUF1" buffer for interpolations.

!        INPUT:
!          PB2     - "SLBUF2" buffer.

!        TRAJECTORY INPUT:
!          PRDELP5 - trajectory version of PRDELP.
!          PEVEL5  - trajectory version of PEVEL.


!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by LACDYNAD.

!     Reference.
!     ----------
!             Arpege documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        C. Temperton (ECMWF)

!     Modifications.
!     --------------
!        Original : 99/10/04
!        Modified 2000-12 by A. Untch: vertical finite element scheme
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.Yessad (Jan 2004): unlagged physics for leltra=true.
!        K. Yessad Aug 2008: rationalisation of dummy argument interfaces
!          + loop optimisation and other cleanings.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LTWOTL
USE YOMCT3   , ONLY : NSTEP
USE YOMCVER  , ONLY : LVERTFE
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG   ,NFLSA
USE YOMDYN   , ONLY : LELTRA   ,LSETTLST
USE YOMGEM   , ONLY : VETAH
USE YOMGMV   , ONLY : NDIMGMV  ,YT0      ,YT9
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2
USE PTRSLB1  , ONLY : MSLB1U9  ,MSLB1V9  ,MSLB1UR0 ,MSLB1VR0 ,MSLB1WR0 ,&
 & MSLB1UR9 ,MSLB1VR9
USE PTRSLB2  , ONLY : MSLB2URL ,MSLB2VRL ,MSLB2WRL

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LD2TLFF1 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSGRTL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSGRTM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRDELP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEVEL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL5(NPROMA,0:NFLEVG)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF
LOGICAL :: LLSETTLST

REAL(KIND=JPRB) :: ZWRL0(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAVENTAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF (.NOT.LTWOTL) THEN
  CALL ABOR1(' LAVENTAD CALLED WITH LTWOTL=.F.')
ENDIF

LLSETTLST=LSETTLST.AND.(.NOT.LELTRA)

! * AD code of increments calculation.

IF (LD2TLFF1) THEN
  DO JLEV=1,NFLEVG
    PGMV(KST:KPROF,JLEV,YT0%MU)=PGMV(KST:KPROF,JLEV,YT0%MU)&
     & +PB1(KST:KPROF,MSLB1UR9+JLEV-NFLSA)  
    PGMV(KST:KPROF,JLEV,YT0%MV)=PGMV(KST:KPROF,JLEV,YT0%MV)&
     & +PB1(KST:KPROF,MSLB1VR9+JLEV-NFLSA)  
  ENDDO
ENDIF

IF (LELTRA) THEN

  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PB1(JROF,MSLB1UR0+JLEV-NFLSA) &
       & +PB2(JROF,MSLB2URL+JLEV-1)
      PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PB1(JROF,MSLB1VR0+JLEV-NFLSA) &
       & +PB2(JROF,MSLB2VRL+JLEV-1)
      PB1(JROF,MSLB1WR0+JLEV-NFLSA)=PB1(JROF,MSLB1WR0+JLEV-NFLSA) &
       & +PB2(JROF,MSLB2WRL+JLEV-1)
      ZWRL0(JROF,JLEV)=PB1(JROF,MSLB1WR0+JLEV-NFLSA)
      PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU) &
       & +PB1(JROF,MSLB1UR0+JLEV-NFLSA)&
       & -PDTS2*PRCORI(JROF)*PB1(JROF,MSLB1VR0+JLEV-NFLSA)  
      PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV) &
       & +PB1(JROF,MSLB1VR0+JLEV-NFLSA)&
       & +PDTS2*PRCORI(JROF)*PB1(JROF,MSLB1VR0+JLEV-NFLSA)  
      PSGRTL(JROF,JLEV)=PSGRTL(JROF,JLEV)-PDTS2*PB1(JROF,MSLB1UR0+JLEV-NFLSA)
      PSGRTM(JROF,JLEV)=PSGRTM(JROF,JLEV)-PDTS2*PB1(JROF,MSLB1VR0+JLEV-NFLSA)
      PB1(JROF,MSLB1U9+JLEV-NFLSA)=PB1(JROF,MSLB1U9+JLEV-NFLSA) &
       & +0.5_JPRB*PB1(JROF,MSLB1UR0+JLEV-NFLSA)
      PB1(JROF,MSLB1V9+JLEV-NFLSA)=PB1(JROF,MSLB1V9+JLEV-NFLSA) &
       & +0.5_JPRB*PB1(JROF,MSLB1VR0+JLEV-NFLSA)
    ENDDO
  ENDDO

ELSE

  ZWRL0(KST:KPROF,1:NFLEVG)=PGMV(KST:KPROF,1:NFLEVG,YT9%MEDOT)
  PGMV(KST:KPROF,1:NFLEVG,YT9%MEDOT)=0.0_JPRB
  IF (NSTEP <= 0) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PB1(JROF,MSLB1UR0+JLEV-NFLSA) &
         & +PB2(JROF,MSLB2URL+JLEV-1)
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PB1(JROF,MSLB1VR0+JLEV-NFLSA) &
         & +PB2(JROF,MSLB2VRL+JLEV-1)
        PB1(JROF,MSLB1WR0+JLEV-NFLSA)=PB1(JROF,MSLB1WR0+JLEV-NFLSA) &
         & +PB2(JROF,MSLB2WRL+JLEV-1)
        PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU) &
         & +PB1(JROF,MSLB1UR0+JLEV-NFLSA)
        PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV) &
         & +PB1(JROF,MSLB1VR0+JLEV-NFLSA)
        ZWRL0(JROF,JLEV)=ZWRL0(JROF,JLEV)+PB1(JROF,MSLB1WR0+JLEV-NFLSA)
      ENDDO
    ENDDO
  ELSE
    IF(LLSETTLST) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU) &
           & +PB2(JROF,MSLB2URL+JLEV-1)
          PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV) &
           & +PB2(JROF,MSLB2VRL+JLEV-1)
          ZWRL0(JROF,JLEV)=ZWRL0(JROF,JLEV)+PB2(JROF,MSLB2WRL+JLEV-1)
          PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU) &
           & +2.0_JPRB*PB1(JROF,MSLB1UR0+JLEV-NFLSA)
          PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV) &
           & +2.0_JPRB*PB1(JROF,MSLB1VR0+JLEV-NFLSA)
          PGMV(JROF,JLEV,YT9%MU)=PGMV(JROF,JLEV,YT9%MU) &
           & -PB1(JROF,MSLB1UR0+JLEV-NFLSA)
          PGMV(JROF,JLEV,YT9%MV)=PGMV(JROF,JLEV,YT9%MV) &
           & -PB1(JROF,MSLB1VR0+JLEV-NFLSA)
          ZWRL0(JROF,JLEV)=ZWRL0(JROF,JLEV) &
           & +2.0_JPRB*PB1(JROF,MSLB1WR0+JLEV-NFLSA)
          PGMV(JROF,JLEV,YT9%MEDOT)=PGMV(JROF,JLEV,YT9%MEDOT) &
           & -PB1(JROF,MSLB1WR0+JLEV-NFLSA)
        ENDDO
      ENDDO
    ELSE
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PB1(JROF,MSLB1UR0+JLEV-NFLSA) &
           & +PB2(JROF,MSLB2URL+JLEV-1)
          PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PB1(JROF,MSLB1VR0+JLEV-NFLSA) &
           & +PB2(JROF,MSLB2VRL+JLEV-1)
          PB1(JROF,MSLB1WR0+JLEV-NFLSA)=PB1(JROF,MSLB1WR0+JLEV-NFLSA) &
           & +PB2(JROF,MSLB2WRL+JLEV-1)
          PGMV(JROF,JLEV,YT0%MU)=PGMV(JROF,JLEV,YT0%MU) &
           & +1.5_JPRB*PB1(JROF,MSLB1UR0+JLEV-NFLSA)
          PGMV(JROF,JLEV,YT0%MV)=PGMV(JROF,JLEV,YT0%MV) &
           & +1.5_JPRB*PB1(JROF,MSLB1VR0+JLEV-NFLSA)
          PGMV(JROF,JLEV,YT9%MU)=PGMV(JROF,JLEV,YT9%MU) &
           & -0.5_JPRB*PB1(JROF,MSLB1UR0+JLEV-NFLSA)
          PGMV(JROF,JLEV,YT9%MV)=PGMV(JROF,JLEV,YT9%MV) &
           & -0.5_JPRB*PB1(JROF,MSLB1VR0+JLEV-NFLSA)
          ZWRL0(JROF,JLEV)=ZWRL0(JROF,JLEV) &
           & +1.5_JPRB*PB1(JROF,MSLB1WR0+JLEV-NFLSA)
          PGMV(JROF,JLEV,YT9%MEDOT)=PGMV(JROF,JLEV,YT9%MEDOT) &
           & -0.5_JPRB*PB1(JROF,MSLB1WR0+JLEV-NFLSA)
        ENDDO
      ENDDO
    ENDIF
  ENDIF

ENDIF

! * AD code of calculation of ZWRL0 at full levels.

IF(LVERTFE) THEN
  ! PEVEL5 and PEVEL are at full levels.
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      PEVEL(JROF,JLEV)=PEVEL(JROF,JLEV)&
       & +(VETAH(JLEV)-VETAH(JLEV-1))*PRDELP5(JROF,JLEV)*ZWRL0(JROF,JLEV)
      PRDELP(JROF,JLEV)=PRDELP(JROF,JLEV)&
       & +(VETAH(JLEV)-VETAH(JLEV-1))*PEVEL5(JROF,JLEV)*ZWRL0(JROF,JLEV)
    ENDDO
  ENDDO
ELSE
  ! PEVEL5 and PEVEL are at half levels.
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      PEVEL(JROF,JLEV)=PEVEL(JROF,JLEV)+PRDELP5(JROF,JLEV)*&
       & 0.5_JPRB*(VETAH(JLEV)-VETAH(JLEV-1))*ZWRL0(JROF,JLEV)
      PEVEL(JROF,JLEV-1)=PEVEL(JROF,JLEV-1)+PRDELP5(JROF,JLEV)*&
       & 0.5_JPRB*(VETAH(JLEV)-VETAH(JLEV-1))*ZWRL0(JROF,JLEV)
      PRDELP(JROF,JLEV)=PRDELP(JROF,JLEV)+&
       & 0.5_JPRB*(VETAH(JLEV)-VETAH(JLEV-1))*&
       & (PEVEL5(JROF,JLEV)+PEVEL5(JROF,JLEV-1))*ZWRL0(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAVENTAD',1,ZHOOK_HANDLE)
END SUBROUTINE LAVENTAD

