SUBROUTINE LAVENTTL(KST,KPROF,LD2TLFF1,PSGRTL,PSGRTM,PDTS2,PRCORI, &
 & PRDELP,PEVEL,PGMV,PB1,PB2, &
 & PSGRT5L,PSGRT5M,PRDELP5,PEVEL5,PGMV5,PUT95,PVT95,PEDOTT95,PB15)

!**** *LAVENTTL* Semi-Lagrangian scheme.  (tangent-linear version)
!                Computation of wind components necessary 
!                to find the SL trajectory.

!     Purpose.
!     --------
!          TL code of LAVENT; see comments in LAVENT.
!          The TL code does not match exactly the structure of routine LAVENT
!           because some direct code options have not been yet implemented
!           in the TL code.

!**   Interface.
!     ----------
!        *CALL* *LAVENTTL(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KST     - first element of work.
!          KPROF   - depth of work.
!          LD2TLFF1- .T./.F.: Refined treatement of (2*Omega Vec r) at
!                    the origin point when there is t-dt (or t in SL2TL)
!                    physics / Other cases.
!          PSGRTL  - zonal component of the pressure force
!                    (grad(phi)+RT grad(p)).
!          PSGRTM  - meridian component of the pressure force.
!          PDTS2   - 0.5*(time step) for the first time-integration step of
!                    a leap-frog scheme or all time-integration steps of
!                    a two-time level scheme; time step for the following
!                    time-integration steps of a leap-frog scheme.
!          PRCORI  - Coriolis parameter.
!          PRDELP  - 1/(delta prehyd) at full levels.
!          PEVEL   - "etadot (d prehyd/d eta)" at t.

!        INPUT/OUTPUT:
!          PGMV    - GMV variables at t-dt and t.
!          PB1     - "SLBUF1" buffer for interpolations.

!        OUTPUT:
!          PB2     - "SLBUF2" buffer.

!        TRAJECTORY INPUT:
!          PSGRT5L - trajectory version of PSGRTL.
!          PSGRT5M - trajectory version of PSGRTM.
!          PRDELP5 - trajectory version of PRDELP.
!          PEVEL5  - trajectory version of PEVEL.
!          PGMV5   - trajectory for GMV variables (trajectory).
!          PUT95   - U-component of the wind at t-dt (trajectory).
!          PVT95   - V-component of the wind at t-dt (trajectory).

!        TRAJECTORY INPUT/OUTPUT:
!          PEDOTT95- etadot at t-dt (trajectory).
!          PB15    - trajectory for PB1.


!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by LACDYNTL.

!     Reference.
!     ----------
!             Arpege documentation about semi-Lagrangian scheme.

!     Author.
!     -------
!        C. Temperton (ECMWF)

!     Modifications.
!     --------------
!        Original : 99/07/20
!        Modified 2000-11 by A. Untch: vertical finite element scheme
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.Yessad (Jan 2004): unlagged physics for leltra=true.
!        K. Yessad Aug 2008: rationalisation of dummy argument interfaces
!          + loop optimisation and other cleanings.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LTWOTL
USE YOMCT3   , ONLY : NSTEP
USE YOMCVER  , ONLY : LVERTFE
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG   ,NFLSA
USE YOMDYN   , ONLY : LELTRA   ,LSETTLST
USE YOMGEM   , ONLY : VETAH
USE YOMGMV   , ONLY : NDIMGMV  ,YT0      ,YT9      ,YT5
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2 ,NFLDSLB15
USE YOMTRSL  , ONLY : NTRSLTYPE
USE PTRSLB1  , ONLY : MSLB1U9  ,MSLB1V9  ,MSLB1UR0 ,MSLB1VR0 ,MSLB1WR0 ,&
 & MSLB1UR9 ,MSLB1VR9
USE PTRSLB15 , ONLY : MSLB1UR95,MSLB1VR95,MSLB1UR05,MSLB1VR05,MSLB1WR05
USE PTRSLB2  , ONLY : MSLB2URL ,MSLB2VRL ,MSLB2WRL ,MSLB2URL5,MSLB2VRL5,&
 & MSLB2WRL5

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
LOGICAL           ,INTENT(IN)    :: LD2TLFF1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTL(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRTM(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDTS2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL(NPROMA,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGMV(NPROMA,NFLEVG,NDIMGMV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1(NPROMA,NFLDSLB1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PB2(NPROMA,NFLDSLB2)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRT5L(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSGRT5M(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVEL5(NPROMA,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV5(NPROMA,NFLEVG,YT5%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT95(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT95(NPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEDOTT95(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB15(NPROMA,NFLDSLB15)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JROF
LOGICAL :: LLSETTLST

REAL(KIND=JPRB) :: ZWRL0(NPROMA,NFLEVG),ZWRL05(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAVENTTL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF (.NOT.LTWOTL) THEN
  CALL ABOR1(' LAVENTTL CALLED WITH LTWOTL=.F.')
ENDIF

LLSETTLST=LSETTLST.AND.(.NOT.LELTRA)

!     * TRAJECTORY.

IF (NTRSLTYPE <= 1) THEN   !! Recomputing trajectory values !!

  ! * Compute ZWRL05 (trajectory of "etadot") at full levels.
  IF(LVERTFE) THEN
    ! PEVEL5 is at full levels.
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ZWRL05(JROF,JLEV)=PEVEL5(JROF,JLEV)&
         & *(VETAH(JLEV)-VETAH(JLEV-1))*PRDELP5(JROF,JLEV)
      ENDDO
    ENDDO
  ELSE
    ! PEVEL5 is at half levels.
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ZWRL05(JROF,JLEV)=0.5_JPRB*(PEVEL5(JROF,JLEV)+PEVEL5(JROF,JLEV-1))&
         & *(VETAH(JLEV)-VETAH(JLEV-1))*PRDELP5(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF

  IF (LELTRA) THEN

    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        ! * currently missing the unlagged physics (term 0.5*PUL95)
        PB15(JROF,MSLB1UR05+JLEV-NFLSA)=PGMV5(JROF,JLEV,YT5%MU)-PDTS2*&
         & (-PRCORI(JROF)*PGMV5(JROF,JLEV,YT5%MV)+PSGRT5L(JROF,JLEV))  
        ! * currently missing the unlagged physics (term 0.5*PVL95)
        PB15(JROF,MSLB1VR05+JLEV-NFLSA)=PGMV5(JROF,JLEV,YT5%MV)-PDTS2*&
         & (PRCORI(JROF)*PGMV5(JROF,JLEV,YT5%MU)+PSGRT5M(JROF,JLEV))  
        PB15(JROF,MSLB1WR05+JLEV-NFLSA)=ZWRL05(JROF,JLEV)
        PB2(JROF,MSLB2URL5+JLEV-1)=PB15(JROF,MSLB1UR05+JLEV-NFLSA)
        PB2(JROF,MSLB2VRL5+JLEV-1)=PB15(JROF,MSLB1VR05+JLEV-NFLSA)
        PB2(JROF,MSLB2WRL5+JLEV-1)=PB15(JROF,MSLB1WR05+JLEV-NFLSA)
      ENDDO
    ENDDO

  ELSE

    IF (NSTEP <= 0) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB15(JROF,MSLB1UR05+JLEV-NFLSA)=PGMV5(JROF,JLEV,YT5%MU)
          PB15(JROF,MSLB1VR05+JLEV-NFLSA)=PGMV5(JROF,JLEV,YT5%MV)
          PB15(JROF,MSLB1WR05+JLEV-NFLSA)=ZWRL05(JROF,JLEV)
          PB2(JROF,MSLB2URL5+JLEV-1)=PB15(JROF,MSLB1UR05+JLEV-NFLSA)
          PB2(JROF,MSLB2VRL5+JLEV-1)=PB15(JROF,MSLB1VR05+JLEV-NFLSA)
          PB2(JROF,MSLB2WRL5+JLEV-1)=PB15(JROF,MSLB1WR05+JLEV-NFLSA)
        ENDDO
      ENDDO
    ELSE
      IF(LLSETTLST) THEN
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB15(JROF,MSLB1UR05+JLEV-NFLSA)= &
             & 2.0_JPRB*PGMV5(JROF,JLEV,YT5%MU)-PUT95(JROF,JLEV)
            PB15(JROF,MSLB1VR05+JLEV-NFLSA)= &
             & 2.0_JPRB*PGMV5(JROF,JLEV,YT5%MV)-PVT95(JROF,JLEV)
            PB15(JROF,MSLB1WR05+JLEV-NFLSA)= &
             & 2.0_JPRB*ZWRL05(JROF,JLEV)-PEDOTT95(JROF,JLEV)
            PB2(JROF,MSLB2URL5+JLEV-1)=PGMV5(JROF,JLEV,YT5%MU)
            PB2(JROF,MSLB2VRL5+JLEV-1)=PGMV5(JROF,JLEV,YT5%MV)
            PB2(JROF,MSLB2WRL5+JLEV-1)=ZWRL05(JROF,JLEV)
          ENDDO
        ENDDO
      ELSE
        DO JLEV=1,NFLEVG
          DO JROF=KST,KPROF
            PB15(JROF,MSLB1UR05+JLEV-NFLSA)=&
             & 1.5_JPRB*PGMV5(JROF,JLEV,YT5%MU)-0.5_JPRB*PUT95(JROF,JLEV)  
            PB15(JROF,MSLB1VR05+JLEV-NFLSA)=&
             & 1.5_JPRB*PGMV5(JROF,JLEV,YT5%MV)-0.5_JPRB*PVT95(JROF,JLEV)  
            PB15(JROF,MSLB1WR05+JLEV-NFLSA)=&
             & 1.5_JPRB*ZWRL05(JROF,JLEV)-0.5_JPRB*PEDOTT95(JROF,JLEV)  
            PB2(JROF,MSLB2URL5+JLEV-1)=PB15(JROF,MSLB1UR05+JLEV-NFLSA)
            PB2(JROF,MSLB2VRL5+JLEV-1)=PB15(JROF,MSLB1VR05+JLEV-NFLSA)
            PB2(JROF,MSLB2WRL5+JLEV-1)=PB15(JROF,MSLB1WR05+JLEV-NFLSA)
          ENDDO
        ENDDO
      ENDIF
    ENDIF
    PEDOTT95(KST:KPROF,1:NFLEVG)=ZWRL05(KST:KPROF,1:NFLEVG)

  ENDIF

  IF (LD2TLFF1) THEN
    DO JLEV=1,NFLEVG
      PB15(KST:KPROF,MSLB1UR95+JLEV-NFLSA)=PGMV5(KST:KPROF,JLEV,YT5%MU)
      PB15(KST:KPROF,MSLB1VR95+JLEV-NFLSA)=PGMV5(KST:KPROF,JLEV,YT5%MV)
    ENDDO
  ENDIF

ENDIF

!     * INCREMENTS.

! * Compute ZWRL0 (increment of "etadot") at full levels.
IF(LVERTFE) THEN
  ! PEVEL5 and PEVEL are at full levels.
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZWRL0(JROF,JLEV)=(VETAH(JLEV)-VETAH(JLEV-1))&
       & *( PEVEL (JROF,JLEV)*PRDELP5(JROF,JLEV)&
       & +PEVEL5(JROF,JLEV)*PRDELP (JROF,JLEV) )
    ENDDO
  ENDDO
ELSE
  ! PEVEL5 and PEVEL are at half levels.
  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      ZWRL0(JROF,JLEV)=0.5_JPRB*(VETAH(JLEV)-VETAH(JLEV-1))*&
       & ((PEVEL(JROF,JLEV)+PEVEL(JROF,JLEV-1))*PRDELP5(JROF,JLEV)+&
       & (PEVEL5(JROF,JLEV)+PEVEL5(JROF,JLEV-1))*PRDELP(JROF,JLEV))
    ENDDO
  ENDDO
ENDIF

IF (LELTRA) THEN

  DO JLEV=1,NFLEVG
    DO JROF=KST,KPROF
      PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU)-PDTS2*&
       & (-PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MV)+PSGRTL(JROF,JLEV)) &
       & + 0.5_JPRB*PB1(JROF,MSLB1U9+JLEV-NFLSA)
      PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV)-PDTS2*&
       & (PRCORI(JROF)*PGMV(JROF,JLEV,YT0%MU)+PSGRTM(JROF,JLEV)) &
       & + 0.5_JPRB*PB1(JROF,MSLB1V9+JLEV-NFLSA)
      PB1(JROF,MSLB1WR0+JLEV-NFLSA)=ZWRL0(JROF,JLEV)
      PB2(JROF,MSLB2URL+JLEV-1)=PB1(JROF,MSLB1UR0+JLEV-NFLSA)
      PB2(JROF,MSLB2VRL+JLEV-1)=PB1(JROF,MSLB1VR0+JLEV-NFLSA)
      PB2(JROF,MSLB2WRL+JLEV-1)=PB1(JROF,MSLB1WR0+JLEV-NFLSA)
    ENDDO
  ENDDO

ELSE

  IF (NSTEP <= 0) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KPROF
        PB1(JROF,MSLB1UR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MU)
        PB1(JROF,MSLB1VR0+JLEV-NFLSA)=PGMV(JROF,JLEV,YT0%MV)
        PB1(JROF,MSLB1WR0+JLEV-NFLSA)=ZWRL0(JROF,JLEV)
        PB2(JROF,MSLB2URL+JLEV-1)=PB1(JROF,MSLB1UR0+JLEV-NFLSA)
        PB2(JROF,MSLB2VRL+JLEV-1)=PB1(JROF,MSLB1VR0+JLEV-NFLSA)
        PB2(JROF,MSLB2WRL+JLEV-1)=PB1(JROF,MSLB1WR0+JLEV-NFLSA)
      ENDDO
    ENDDO
  ELSE
    IF(LLSETTLST) THEN
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR0+JLEV-NFLSA)= &
           & 2.0_JPRB*PGMV(JROF,JLEV,YT0%MU)-PGMV(JROF,JLEV,YT9%MU)
          PB1(JROF,MSLB1VR0+JLEV-NFLSA)= &
           & 2.0_JPRB*PGMV(JROF,JLEV,YT0%MV)-PGMV(JROF,JLEV,YT9%MV)
          PB1(JROF,MSLB1WR0+JLEV-NFLSA)= &
           & 2.0_JPRB*ZWRL0(JROF,JLEV)-PGMV(JROF,JLEV,YT9%MEDOT)
          PB2(JROF,MSLB2URL+JLEV-1)=PGMV(JROF,JLEV,YT0%MU)
          PB2(JROF,MSLB2VRL+JLEV-1)=PGMV(JROF,JLEV,YT0%MV)
          PB2(JROF,MSLB2WRL+JLEV-1)=ZWRL0(JROF,JLEV)
        ENDDO
      ENDDO
    ELSE
      DO JLEV=1,NFLEVG
        DO JROF=KST,KPROF
          PB1(JROF,MSLB1UR0+JLEV-NFLSA)= &
           & 1.5_JPRB*PGMV(JROF,JLEV,YT0%MU)-0.5_JPRB*PGMV(JROF,JLEV,YT9%MU)
          PB1(JROF,MSLB1VR0+JLEV-NFLSA)= &
           & 1.5_JPRB*PGMV(JROF,JLEV,YT0%MV)-0.5_JPRB*PGMV(JROF,JLEV,YT9%MV)
          PB1(JROF,MSLB1WR0+JLEV-NFLSA)= &
           & 1.5_JPRB*ZWRL0(JROF,JLEV)-0.5_JPRB*PGMV(JROF,JLEV,YT9%MEDOT)
          PB2(JROF,MSLB2URL+JLEV-1)=PB1(JROF,MSLB1UR0+JLEV-NFLSA)
          PB2(JROF,MSLB2VRL+JLEV-1)=PB1(JROF,MSLB1VR0+JLEV-NFLSA)
          PB2(JROF,MSLB2WRL+JLEV-1)=PB1(JROF,MSLB1WR0+JLEV-NFLSA)
        ENDDO
      ENDDO
    ENDIF
  ENDIF
  PGMV(KST:KPROF,1:NFLEVG,YT9%MEDOT)=ZWRL0(KST:KPROF,1:NFLEVG)

ENDIF

IF (LD2TLFF1) THEN
  DO JLEV=1,NFLEVG
    PB1(KST:KPROF,MSLB1UR9+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT0%MU)
    PB1(KST:KPROF,MSLB1VR9+JLEV-NFLSA)=PGMV(KST:KPROF,JLEV,YT0%MV)
  ENDDO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAVENTTL',1,ZHOOK_HANDLE)
END SUBROUTINE LAVENTTL
