SUBROUTINE SI_CCCOR(KLEV,KLON,KNLON,PIN,POU)

!**** *SI_CCCOR* - Semi-implicit scheme in the NH model.
!                  Does a multiplication by C**2 * COR when the constraint
!                  C1 is not matched.

!     Purpose.
!     --------
!       This routine is called in the SI scheme (NH model)
!       It computes C**2 * COR where:
!        C = sqrt( R_dry Trsi Cp_dry/Cv_dry )
!       and:
!        COR = (Cv_dry/(R_d**2 Trsi)) Gamma Tau
!              - (Cv_dry/(R_dry*Cp_dry)) Gamma
!              - (Cv_dry/(R_dry*Trsi)) Tau
!              + (Cv_dry/Cv_dry) Nu
!       This is equivalent to write:
!        C**2 * COR = (Cp_dry/R_dry) Gamma Tau - Trsi Gamma
!         - Cp_dry Tau + (R_d*Trsi) Nu

!       It does a multiplication by C**2 * COR.

!       Remark: COR=0 in the continuous equations and in the discretisations
!        matching the constraint C1 (ex: NDLNPR=1 in finite differences).

!**   Interface.
!     ----------
!        *CALL* *SI_CCCOR(...)

!        Explicit arguments :
!        --------------------

!         KLEV     :  distance in memory between values of PIN      (input)
!                     at the same vertical.
!         KLON:    :  distance in memory between values of PIN      (input)
!                     at the same level.
!         For (KLEV,KLON), typical values are:
!                     NDLSUR,1 for grid-point array.
!                     1,NFLSUR for spectral array.
!         KNLON    :  number of vertical columns treated            (input)
!         PIN      :  known input vector.                           (input)
!         POU      :  unknown output vector.                        (output)

!        Implicit arguments :
!        --------------------
!        none.

!     Method.
!     -------
!        See documentation about semi-implicite scheme

!     Externals.
!     ----------
!        None.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        ARPEGE documentation about semi-implicit scheme

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP)

!     Modifications.
!     --------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST    ,ONLY : RCPD     ,RD
USE YOMDYN    ,ONLY : SITR
USE YOMDIM    ,ONLY : NFLEVG

!      ----------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON
INTEGER(KIND=JPIM),INTENT(IN)    :: KNLON
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIN(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POU(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 

!      ----------------------------------------------------------------

INTEGER(KIND=JPIM) :: IDT,JLEV,JLON

REAL(KIND=JPRB) :: ZGAM(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON)
REAL(KIND=JPRB) :: ZTAU(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON)
REAL(KIND=JPRB) :: ZGAMTAU(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON)
REAL(KIND=JPRB) :: ZNU(KNLON)
REAL(KIND=JPRB) :: ZSP(KNLON)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!      ----------------------------------------------------------------

#include "sigam.intfb.h"
#include "sitnu.intfb.h"

!      ----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SI_CCCOR',0,ZHOOK_HANDLE)

!      ----------------------------------------------------------------

!*       1.    COMPUTE C2*COR AND DO MULTIPLICATION
!              ------------------------------------

! * Computes ZTAU = Tau * PIN and ZNU = Nu * PIN
CALL SITNU(KLEV,KLON,PIN,ZTAU,ZNU,KNLON)

! * Computes ZGAM = Gamma * PIN
ZSP(1:KNLON)=0.0_JPRB
CALL SIGAM(KLEV,KLON,ZGAM,PIN,ZSP,KNLON)

! * Computes ZGAMTAU = Gamma * (Tau * PIN) = Gamma * ZTAU
CALL SIGAM(KLEV,KLON,ZGAMTAU,ZTAU,ZSP,KNLON)

! * Computes POU:
DO JLEV=1,NFLEVG
  DO JLON=1,KNLON
    IDT=1+(JLEV-1)*KLEV+(JLON-1)*KLON
    POU(IDT)=(RCPD/RD)*ZGAMTAU(IDT)-SITR*ZGAM(IDT)-RCPD*ZTAU(IDT) &
     & + RD*SITR*ZNU(JLON)
  ENDDO
ENDDO

!       ----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SI_CCCOR',1,ZHOOK_HANDLE)
END SUBROUTINE SI_CCCOR

