SUBROUTINE SIGAMAD(KLEV,KLON,PD,PT,PSP,KNLON)

!**** *SIGAMAD* - Solve hydrostatic operator in semi-implicit,ADJOINT

!     Purpose.
!     --------
!           Operator gamma to compute p.

!**   Interface.
!     ----------
!        *CALL* *SIGAMAD(KLEV,KLON,PD,PT,PSP,KNLON)

!        Explicit arguments :
!        --------------------
!        KLEV   : DISTANCE IN MEMORY BETWEEN VALUES OF THE DIVERGENCE
!                OR TEMPERATURE AT THE SAME LEVEL
!        KLON   : DISTANCE IN MEMORY BETWEEN VALUES OF THE DIVERGENCE
!                OR TEMPERATURE AT THE VERTICAL

!           TYPICAL VALUES ARE  NDLSUR,1  FOR GRID POINT ARRAY
!                               1,NFLSUR  FOR SPECTRAL ARRAY

!        PD    : DIVERGENCE       (input/output)
!        PT    : TEMPERATURE      (input/output)
!        PSP   : SURFACE PRESSURE (input/output)
!        KNLON : NUMBER OF VERTICAL COLUMNS TREATED

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Jean-Noel THEPAUT METEO FRANCE/EERM/CRMD

!     Modifications.
!     --------------
!        Original : 89-12-12
!        Modified 95-10-01 Adapted for Message Passing
!     E.Andersson 96-07-10 Fujitsu vectorisation
!     A. Untch    2000-12  Vertical finite element scheme
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     S. Ivatek-S. 10-Apr-2007 Initialisation NEC
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCST   , ONLY : RD
USE YOMDYN   , ONLY : SIALPH   ,SILNPR   ,SIRPRG
USE YOMGEM   , ONLY : VFE_RDETAH
USE YOMCVER  , ONLY : LVERTFE

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNLON 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PD(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSP(KNLON) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZSPHI(KNLON,0:NFLEVG), ZOUT(KNLON,0:NFLEVG)

INTEGER(KIND=JPIM) :: IDT, JLEV, JLON
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "verintad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SIGAMAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    SUM GEOPOTENTIAL, COMPUTES P AND PUT IT IN PD.
!              ----------------------------------------------

ZSPHI(:,:)=0.0_JPRB
IF(LVERTFE) THEN
  ZOUT(:,:)=0.0_JPRB
!cdir outerunroll=8
!$OMP PARALLEL PRIVATE(JLEV,JLON,IDT)
  DO JLEV=1,NFLEVG
!OCL NOVREC
!$OMP DO
    DO JLON=1,KNLON
      IDT=1+(JLEV-1)*KLEV+(JLON-1)*KLON
      PSP(JLON)=PSP(JLON)+SIRPRG*PD(IDT)
      ZOUT(JLON,JLEV-1)=PD(IDT)
      PD(IDT)=0.0_JPRB
    ENDDO
!$OMP END DO
  ENDDO
!$OMP END PARALLEL
  CALL VERINTAD(KNLON,1,KNLON,NFLEVG,ZSPHI(1,1),ZOUT,1)
!cdir outerunroll=8
!$OMP PARALLEL PRIVATE(JLEV,JLON,IDT)
  DO JLEV=1,NFLEVG
!OCL NOVREC
!$OMP DO
    DO JLON=1,KNLON
      IDT=1+(JLEV-1)*KLEV+(JLON-1)*KLON
      PT(IDT)=PT(IDT)-RD*ZSPHI(JLON,JLEV)*SILNPR(JLEV)*VFE_RDETAH(JLEV)
    ENDDO
!$OMP END DO
  ENDDO
!$OMP END PARALLEL

ELSE

  ZSPHI(:,0)=0.0_JPRB

!cdir outerunroll=8
!$OMP PARALLEL PRIVATE(JLEV,JLON,IDT)
  DO JLEV=1,NFLEVG
!OCL NOVREC
!$OMP DO
    DO JLON=1,KNLON
      IDT=1+(JLEV-1)*KLEV+(JLON-1)*KLON
      PSP(JLON)=PSP(JLON)+SIRPRG*PD(IDT)
      PT(IDT)=PT(IDT)+RD*SIALPH(JLEV)*PD(IDT)&
       & +RD*SILNPR(JLEV)*ZSPHI(JLON,JLEV-1)  
      ZSPHI(JLON,JLEV)=PD(IDT)+ZSPHI(JLON,JLEV-1)
      PD(IDT)=0.0_JPRB
    ENDDO
!$OMP END DO
  ENDDO
!$OMP END PARALLEL
ENDIF

!      -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SIGAMAD',1,ZHOOK_HANDLE)
END SUBROUTINE SIGAMAD
