SUBROUTINE SIPTPAD(KLEV,KLON,PDH,PDV,PRNH,PT,PSP,KNLON)

!**** *SIPTPAD* - Counterpart of SITNUAD in nonhydrostatic adjoint

!     Purpose.
!     --------
!           Provide corrections of temperature, surface
!           pressure and nonhydrostatic perturbation from
!           3D divergence in semiimplicit

!**   Interface.
!     ----------
!        *CALL* *SIPTPAD(KLEV,KLON,PDH,PDV,PRNH,PT,PSP,KNLON)

!        Explicit arguments :
!        --------------------
!        KLEV   : DISTANCE IN MEMORY BETWEEN VALUES OF THE DIVERGENCE
!                OR TEMPERATURE AT THE VERTICAL
!        KLON   : DISTANCE IN MEMORY BETWEEN VALUES OF THE DIVERGENCE
!                OR TEMPERATURE AT THE SAME LEVEL

!           TYPICAL VALUES ARE  NDLSUR,1  FOR GRID POINT ARRAY
!                               1,NFLSUR  FOR SPECTRAL ARRAY

!        PDH   : HORIZONTAL DIVERGENCE                       - input
!        PDV   : VERTICAL DIVERGENCE VARIABLE                - input
!        PRNH  : NONHYDROSTATIC PRESSURE DEPARTURE VARIABLE  - output
!        PT    : TEMPERATURE                                 - output
!        PSP   : LOG(SURFACE PRESSURE)                       - output
!        KNLON : NUMBER OF VERTICAL COLUMNS TREATED

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Arpege/Aladin Documentation

!     Author.
!     -------
!        Martin Charron  CNRM/GMAP - visiteur

!     Modifications.
!     --------------
!        Original : November 1998
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Sep 2008): update comments + cleanings.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCST   , ONLY : RD       ,RCPD     ,RCVD
USE YOMDYN   , ONLY : SITR

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNLON 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDH(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDV(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRNH(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSP(KNLON) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IAD, ILEV, ILON, INLON, JLEV, JLON

REAL(KIND=JPRB) :: Z3DIVC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "sitnuad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SIPTPAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

! ky: caution: this piece of code may not be properly updated
!     for NPDVAR=2 and NVDVAR=3 or 4.

ILEV = KLEV
ILON = KLON
INLON= KNLON

Z3DIVC=0.0_JPRB

DO JLEV = NFLEVG,1,-1
  DO JLON = INLON,1,-1
    IAD = 1 + (JLEV-1)*ILEV + (JLON-1)*ILON

    Z3DIVC=Z3DIVC+RCPD*PRNH(IAD)
    PRNH(IAD)=-RCPD*PRNH(IAD)/(RD*SITR)
    Z3DIVC=Z3DIVC+RD*SITR*PT(IAD)
    PT(IAD)=0.0_JPRB
    PDH(IAD)=PDH(IAD)+1.0_JPRB/RCVD*Z3DIVC
    PDV(IAD)=PDV(IAD)+1.0_JPRB/RCVD*Z3DIVC
    Z3DIVC=0.0_JPRB
  ENDDO
ENDDO

CALL SITNUAD(ILEV,ILON,PDH,PRNH,PSP,INLON)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SIPTPAD',1,ZHOOK_HANDLE)
END SUBROUTINE SIPTPAD
