SUBROUTINE SISEVEAD(KLEV,KLON,PV1,PV2,KNLON)

!**** *SISEVEAD* - SEcond VErtical derivative operator in Semi - Implicit

!     Purpose.
!     --------
!           Operator L to compute vertical Laplacian in non-hydrostatic
!           dynamics: formulation for semi - implicit adjoint model

!**   Interface.
!     ----------
!        *CALL* *SISEVEAD(KLEV,KLON,PV1,PV2,KNLON)

!        Explicit arguments :
!        --------------------
!        KLEV   : DISTANCE IN MEMORY BETWEEN VALUES OF THE VARIABLES
!                PV1 OR PV2 AT THE VERTICAL
!        KLON   : DISTANCE IN MEMORY BETWEEN VALUES OF THE VARIABLES
!                PV1 OR PV2 AT THE SAME LEVEL

!           TYPICAL VALUES ARE  NDLSUR,1  FOR GRID POINT ARRAY
!                               1,NFLSUR  FOR SPECTRAL ARRAY

!        PV1   : INPUT VARIABLE
!        PV2   : DERIVED VARIABLE BY VERTICAL LAPLACIAN
!        KNLON : NUMBER OF VERTICAL COLUMNS TREATED

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Arpege/Aladin documentation

!     Author.
!     -------
!        Martin Charron CNRM/GMAP - visiteur

!     Modifications.
!     --------------
!        Original : November 1998
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad (Sep 2008): update comments + cleanings.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMDYN   , ONLY : SILNPR   ,SIDELP

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNLON 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV1(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV2(1+(NFLEVG-1)*KLEV+(KNLON-1)*KLON) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZA (NFLEVG), ZC(NFLEVG)
REAL(KIND=JPRB) :: ZPRF(NFLEVG)

INTEGER(KIND=JPIM) :: IBOT1, IBOT2, ILEV1, ILEV2, ILEV3, ITOP1, ITOP2, JLEV, JLON

REAL(KIND=JPRB) :: ZB1, ZBN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SISEVEAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INITIATE COEFFICIENTS OF OPERATOR L
!              -----------------------------------

! ky: caution: this piece of code is maybe not properly coded for
!     NPDVAR=2, NVDVAR=3 or 4, and not properly updated relatively
!     to the last version of SISEVE.

DO JLEV = 1, NFLEVG
  ZPRF(JLEV) = SIDELP(JLEV)/SILNPR(JLEV)
ENDDO

ZB1 = -ZPRF(2)/(SILNPR(1)*(ZPRF(2)-ZPRF(1)))

ZBN = -ZPRF(NFLEVG)/(SILNPR(NFLEVG)*(ZPRF(NFLEVG) -ZPRF(NFLEVG-1)))

DO JLEV = 2, NFLEVG-1
  ZA(JLEV) = ZPRF(JLEV-1)/(SILNPR(JLEV)*(ZPRF(JLEV)-ZPRF(JLEV-1)))
  ZC(JLEV) = ZPRF(JLEV+1)/(SILNPR(JLEV)*(ZPRF(JLEV+1)-ZPRF(JLEV)))
ENDDO

DO JLEV = NFLEVG-1 , 2 , -1
  DO JLON = KNLON , 1 , -1
    ILEV1 = 1 + (JLEV-2)*KLEV + (JLON-1)*KLON
    ILEV2 = ILEV1 + KLEV
    ILEV3 = ILEV2 + KLEV

    PV1(ILEV1)=PV1(ILEV1)+ZA(JLEV)*PV2(ILEV2)
    PV1(ILEV2)=PV1(ILEV2)-(ZA(JLEV)+ZC(JLEV))*PV2(ILEV2)
    PV1(ILEV3)=PV1(ILEV3)+ZC(JLEV)*PV2(ILEV2)
    PV2(ILEV2)=0.0_JPRB
  ENDDO
ENDDO

DO JLON =  KNLON,1,-1
  ITOP1 = 1 + (JLON-1)*KLON
  ITOP2 = ITOP1 + KLEV
  IBOT1 = 1 + (NFLEVG-2)*KLEV + (JLON-1)*KLON
  IBOT2 = IBOT1 + KLEV

  PV1(IBOT1)=PV1(IBOT1)-ZBN*ZPRF(NFLEVG-1)/ZPRF(NFLEVG)*PV2(IBOT2)
  PV1(IBOT2)=PV1(IBOT2)+ZBN*PV2(IBOT2)
  PV2(IBOT2)=0.0_JPRB
  PV1(ITOP1)=PV1(ITOP1)+ZB1*PV2(ITOP1)
  PV1(ITOP2)=PV1(ITOP2)-ZB1*PV2(ITOP1)
  PV2(ITOP1)=0.0_JPRB
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SISEVEAD',1,ZHOOK_HANDLE)
END SUBROUTINE SISEVEAD
