SUBROUTINE SIVDERI(KNLON,KTYP,&
 & LDVDERIB,LDBCS,LDBCT,PGPIN,PSPIN,PGPOUT,PSPOUT)

!**** *SIVDERI* - first order vertical derivative operator in Semi - Implicit
!                 for Phi-gw NH equations.

!     Purpose.
!     --------
!       Operator 'sivderi' to compute the first-order vertical derivative
!       operator in the semi-implicit scheme of the Phi-gw NH equations.
!        sivderi X = d X / d log(prehyd_star) = dX/deta deta/d log(prehyd_star)
!       If KTYP=1, this is (sivderi+1)(X) which is computed.
!       If KTYP=2, this is sivderi((sivderi+1)(X)) which is computed.

!**   Interface.
!     ----------
!        *CALL* *SIVDERI(***)

!        Explicit arguments :
!        --------------------
!        * INPUT:
!        KNLON   : Number of vertical columns treated
!        KTYP    : 0: apply 'sivderi'
!                  1: apply 'sivderi+1'
!                  2: apply 'sivderi(sivderi+1)' (Laplacian)

!        * OPTIONAL INPUT:
!        LDVDERIB: T= use RDERI taking account of top and bottom values.
!                  F= use RDERI not taking account of top and bottom values.
!                  Not present=F
!                  (case T not yet coded)
!        LDBCS   : Type of bottom condition if LDVDERIB=F:
!                  F: not taken into account; T: taken into account.
!                  Not present=F
!        LDBCT   : Type of top condition LDVDERIB=F:
!                  F: not taken into account; T: taken into account.
!                  Not present=F
!        PSPIN   : Input variable in spectral space
!        PGPIN   : Input variable in grid-point space

!        * OPTIONAL OUTPUT:
!        PSPOUT  : Output variable in spectral space (derivative)
!        PGPOUT  : Output variable in grid-point space (derivative)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        Arpege/Aladin documentation

!     Author.
!     -------
!        Karim YESSAD (CNRM/GMAP)

!     Modifications.
!     --------------
!   Original : Nov 2007
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NFLEVG
USE YOMDYN   , ONLY : SILNPR
USE YOMGEM   , ONLY : VFE_RDETAH
USE YOMCVER  , ONLY : LVERTFE

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)        :: KNLON 
INTEGER(KIND=JPIM),INTENT(IN)        :: KTYP
LOGICAL,OPTIONAL  ,INTENT(IN)        :: LDVDERIB
LOGICAL,OPTIONAL  ,INTENT(IN)        :: LDBCS
LOGICAL,OPTIONAL  ,INTENT(IN)        :: LDBCT
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)  :: PSPIN(0:NFLEVG+1,KNLON) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN)  :: PGPIN(KNLON,0:NFLEVG+1) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PSPOUT(0:NFLEVG+1,KNLON) 
REAL(KIND=JPRB),OPTIONAL,INTENT(OUT) :: PGPOUT(KNLON,0:NFLEVG+1) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JLEV, JLON

REAL(KIND=JPRB) :: ZIN(KNLON,0:NFLEVG+1)
REAL(KIND=JPRB) :: ZINDETA(KNLON,0:NFLEVG+1)
REAL(KIND=JPRB) :: ZVAL
LOGICAL :: LLVDERIB,LLBCS,LLBCT

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "verder.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SIVDERI',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PRELIMINARY TESTS AND CALCULATIONS
!              ----------------------------------

IF (PRESENT(LDVDERIB)) THEN
  LLVDERIB=LDVDERIB
ELSE
  LLVDERIB=.FALSE.
ENDIF
IF (PRESENT(LDBCS)) THEN
  LLBCS=LDBCS
ELSE
  LLBCS=.FALSE.
ENDIF
IF (PRESENT(LDBCT)) THEN
  LLBCT=LDBCT
ELSE
  LLBCT=.FALSE.
ENDIF

IF ((.NOT.PRESENT(PSPIN)).AND.(.NOT.PRESENT(PGPIN))) THEN
  CALL ABOR1(' SIVDERI: PSPIN or PGPIN should be present')
ENDIF
IF (PRESENT(PSPIN).AND.PRESENT(PGPIN)) THEN
  CALL ABOR1(' SIVDERI: PSPIN or PGPIN should be present but not both')
ENDIF
IF (PRESENT(PSPIN).AND.(.NOT.PRESENT(PSPOUT))) THEN
  CALL ABOR1(' SIVDERI: sp space: PSPIN present => PSPOUT should be present')
ENDIF
IF (PRESENT(PGPIN).AND.(.NOT.PRESENT(PGPOUT))) THEN
  CALL ABOR1(' SIVDERI: gp space: PGPIN present => PGPOUT should be present')
ENDIF

!     ------------------------------------------------------------------

!*       2.    BOUNDARY CONDITIONS FOR FD
!              --------------------------

IF(.NOT.LVERTFE)THEN

  ! --- Finite differences ---

  WRITE(NULOUT,*) ' SIVDERI: Phi-gw NH eqns not coded for FD vert disc'
  CALL ABOR1(' SIVDERI: Phi-gw NH eqns not coded for FD vert disc')

ENDIF

!     ------------------------------------------------------------------

!*       3.    INNER LAYERS (AND BOUNDARY CONDITIONS FOR VFE)
!              ----------------------------------------------

IF(LVERTFE) THEN

  ! --- transform PSPIN or PGPIN into form appropriate for VERDER ---

  IF (PRESENT(PSPIN)) THEN
    DO JLEV=0,NFLEVG+1
      DO JLON=1,KNLON
        ZIN(JLON,JLEV)=PSPIN(JLEV,JLON)
      ENDDO
    ENDDO
  ELSEIF (PRESENT(PGPIN)) THEN
    ZIN(1:KNLON,0:NFLEVG+1)=PGPIN(1:KNLON,0:NFLEVG+1)
  ENDIF

  ! --- sivderi(PIN) or (sivderi+1)(PIN) ---

  IF (LLVDERIB) THEN

    ! * dX/d eta
    !   Assumes that dX/d eta converges towards 0 at the top and bottom.
    CALL VERDER(KNLON,1,KNLON,NFLEVG+2,ZIN(1:KNLON,0:NFLEVG+1),&
     & ZINDETA(1:KNLON,0:NFLEVG+1),.TRUE.)

  ELSE 

    ! * dX/d eta
    !   Assumes that dX/d eta converges towards 0 at the top and bottom.
    CALL VERDER(KNLON,1,KNLON,NFLEVG,ZIN(1:KNLON,1:NFLEVG),&
     & ZINDETA(1:KNLON,1:NFLEVG))
    ZINDETA(1:KNLON,0)=0.0_JPRB
    ZINDETA(1:KNLON,NFLEVG+1)=0.0_JPRB

    ! Correct derivative(l=nflevg) using ZIN(.,NFLEVG+1) (pseudo FD treatment).
    IF (LLBCS) THEN
      DO JLON=1,KNLON
        ! ZVAL=estimated value at half level NFLEVG-1:
        ZVAL=ZIN(JLON,NFLEVG)-0.5_JPRB*ZINDETA(JLON,NFLEVG)/VFE_RDETAH(NFLEVG)
        ZINDETA(JLON,NFLEVG)=(ZIN(JLON,NFLEVG+1)-ZVAL)*VFE_RDETAH(NFLEVG)
      ENDDO
    ENDIF

    ! Correct derivative(l=1) using ZIN(.,0) (pseudo FD treatment).
    IF (LLBCT) THEN
      DO JLON=1,KNLON
        ! ZVAL=estimated value at half level 1:
        ZVAL=ZIN(JLON,1)+0.5_JPRB*ZINDETA(JLON,1)/VFE_RDETAH(1)
        ZINDETA(JLON,1)=(ZVAL-ZIN(JLON,0))*VFE_RDETAH(1)
      ENDDO
    ENDIF

  ENDIF

  ! * dX/d(log(prehyd_star)) = dX/d eta * d eta/d(log(prehyd_star))
  DO JLEV=1,NFLEVG
    DO JLON=1,KNLON
      ZINDETA(JLON,JLEV)=ZINDETA(JLON,JLEV)/(SILNPR(JLEV)*VFE_RDETAH(JLEV))
    ENDDO
  ENDDO

  IF (KTYP == 1 .OR. KTYP == 2) THEN
    DO JLEV=0,NFLEVG+1
      DO JLON=1,KNLON
        ZINDETA(JLON,JLEV)=ZINDETA(JLON,JLEV)+ZIN(JLON,JLEV)
      ENDDO
    ENDDO
  ENDIF

  ! --- sivderi((sivderi+1)(PIN)) ---

  IF (KTYP == 2) THEN

    ZIN(1:KNLON,0:NFLEVG+1)=ZINDETA(1:KNLON,0:NFLEVG+1)

    IF (LLVDERIB) THEN

      ! * dX/d eta
      !   Assumes that dX/d eta converges towards 0 at the top and bottom.
      CALL VERDER(KNLON,1,KNLON,NFLEVG+2,ZIN(1:KNLON,0:NFLEVG+1),&
       & ZINDETA(1:KNLON,0:NFLEVG+1),.TRUE.)

    ELSE 

      ! * dX/d eta
      !   Assumes that dX/d eta converges towards 0 at the top and bottom.
      CALL VERDER(KNLON,1,KNLON,NFLEVG,ZIN(1:KNLON,1:NFLEVG),&
       & ZINDETA(1:KNLON,1:NFLEVG))
      ZINDETA(1:KNLON,0)=0.0_JPRB
      ZINDETA(1:KNLON,NFLEVG+1)=0.0_JPRB

      ! Correct derivative(l=nflevg) using ZIN(.,NFLEVG+1) (pseudo FD treatment)
      IF (LLBCS) THEN
        DO JLON=1,KNLON
          ! ZVAL=estimated value at half level NFLEVG-1:
          ZVAL=ZIN(JLON,NFLEVG)-0.5_JPRB*ZINDETA(JLON,NFLEVG)/VFE_RDETAH(NFLEVG)
          ZINDETA(JLON,NFLEVG)=(ZIN(JLON,NFLEVG+1)-ZVAL)*VFE_RDETAH(NFLEVG)
        ENDDO
      ENDIF

      ! Correct derivative(l=1) using ZIN(.,0) (pseudo FD treatment).
      IF (LLBCT) THEN
        DO JLON=1,KNLON
          ! ZVAL=estimated value at half level 1:
          ZVAL=ZIN(JLON,1)+0.5_JPRB*ZINDETA(JLON,1)/VFE_RDETAH(1)
          ZINDETA(JLON,1)=(ZVAL-ZIN(JLON,0))*VFE_RDETAH(1)
        ENDDO
      ENDIF

    ENDIF

    ! * dX/d(log(prehyd_star)) = dX/d eta * d eta/d(log(prehyd_star))
    DO JLEV=1,NFLEVG
      DO JLON=1,KNLON
        ZINDETA(JLON,JLEV)=ZINDETA(JLON,JLEV)/(SILNPR(JLEV)*VFE_RDETAH(JLEV))
      ENDDO
    ENDDO

  ENDIF

  ! --- Memory transfer into PSPOUT or PGPOUT ---

  IF (PRESENT(PSPOUT)) THEN
    DO JLEV=0,NFLEVG+1
      DO JLON=1,KNLON
        PSPOUT(JLEV,JLON)=ZINDETA(JLON,JLEV)
      ENDDO
    ENDDO
  ELSEIF (PRESENT(PGPOUT)) THEN
    PGPOUT(1:KNLON,0:NFLEVG+1)=ZINDETA(1:KNLON,0:NFLEVG+1)
  ENDIF

ELSE

  ! --- Finite differences ---

  WRITE(NULOUT,*) ' SIVDERI: Phi-gw NH eqns not coded for FD vert disc'
  CALL ABOR1(' SIVDERI: Phi-gw NH eqns not coded for FD vert disc')

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SIVDERI',1,ZHOOK_HANDLE)
END SUBROUTINE SIVDERI
