SUBROUTINE SPCSI(&
 ! --- INPUT -----------------------------------------------------------------
 & CDCONF,KM,KMLOC,KSTA,KEND,LDONEM,PSPAUXG, &
 ! --- INOUT -----------------------------------------------------------------
 & PSPVORG,PSPDIVG,PSPTG,PSPSPG)  

USE PARKIND1  ,ONLY : JPIM, JPRB
USE YOMHOOK   ,ONLY : LHOOK, DR_HOOK

USE YOMMP    , ONLY : MYSETV, MYSETW, NPROCM, NPTRSV, NPTRSVF, NSPEC2V, NSPEC2VF 
USE YOMDIM   , ONLY : NFLEVG, NSMAX
USE YOMCT0   , ONLY : NCONF, LSLAG, LTWOTL, LSITRIC
USE YOMDYN   , ONLY : SCGMAP, SIRDEL, SIMO, SIMI, SIVP, SIHEG, SIHEG2, &
 & SITRICA, SITRICB, SITRICC, TDT, BETADT, VESL, LSIDG, XIDT, LIMPF  
USE YOMLAP   , ONLY : NVALUE, NSE0L, RLAPDI, RLAPIN
USE YOMGEM   , ONLY : RSTRET

!**** *SPCSI* - SPECTRAL SPACE SEMI-IMPLICIT COMPUTATIONS FOR HYD MODEL.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SPCSI(..)

!        Explicit arguments :  CDCONF  - configuration (see doc.)
!        --------------------  KM      - Zonal wavenumber 
!                              KMLOC   - Zonal wavenumber (DM-local numbering)
!                              KSTA    - First column processed
!                              KEND    - Last column processed
!                              LDONEM  - T if only one m if processed
!                              PSPVORG - Vorticity columns
!                              PSPDIVG - Divergence columns
!                              PSPTG   - Temperature columns
!                              PSPSPG  - Surface Pressure

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-11-24 (before 1997 spcsi.F was part of spc.F)
!        Modified 91-05-21 by A.Lasserre-Bigorry (scalar variables)
!        Modified DEC 1992 by K. YESSAD: decentering factor in semi-lag scheme.
!        Modified JAN 1994 by D GIARD: CDCONF='I'.
!        Modified JAN 1994 by K. YESSAD: semi-implicit computations
!           with not reduced divergence + new formulation of variable
!           mesh horizontal diffusion + multitasking on m instead of n.
!        Modified 95-02-06 by C. Temperton: semi-implicit Coriolis term,
!           real/imag parts stored in conventional order in work arrays
!        K. YESSAD (MARCH 1995):
!            - Optimisation of semi-implicit scheme if variable decentering
!              and remove of IMSL routines.
!            - Use new array NSE0 (of YOMLAP) to define IS0.
!        Modified 95-06-06 by L.Isaksen : Reordering of spectral arrays
!        Modified 96-04-16 by C. Temperton: "alternative averaging" in  
!                                           two-time-level scheme
!        Modified 96-05-16 by C. Temperton: new semi-implicit solver
!        Modified OCT 1996 by K. YESSAD: removal of variable uncentering.
!        Modified 12/12/96 by L.Isaksen: Merged with message passing
!                                        version. Renamed and recoded.
!        C. Temperton 97-02-24: modified first timestep when XIDT>0
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        N.Wedi        08-Mar-2005 remove mass correction      
!        K.Yessad 09-Dec-2004: move mass correction in SPCMASCOR + cleanings.
!        K. Yessad 15-May-2006: memory optimisations for stretched geometry
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!        T.Wilhelmsson 09-09-25: Remove LFULLM requirement for LIMPF
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMLOC
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
LOGICAL           ,INTENT(IN)    :: LDONEM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPAUXG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPVORG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPDIVG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPTG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPSPG(:) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB), ALLOCATABLE :: ZSDIVP (:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPDIVP(:,:)

REAL(KIND=JPRB) :: ZSDIV  (NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZHELP  (NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZST    (NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSP    (       KSTA:KEND)

REAL(KIND=JPRB) :: ZSDIVPL (NFLEVG,KM:NSMAX,2)
REAL(KIND=JPRB) :: ZSPDIVPL(NFLEVG,KM:NSMAX,2)


INTEGER(KIND=JPIM) :: II, IN, IOFF, IS0, IS02,&
 & ISE, ISPCOL, ISTART, JLEV, JN, JSP  

LOGICAL :: LL3D,LLDOSI

REAL(KIND=JPRB) :: ZBDT, ZBDT2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "mxmaop.h"
END INTERFACE

#include "mxptma.h"
#include "mxture.h"
#include "mxturs.h"
#include "sigam.intfb.h"
#include "spcimpfsolve.intfb.h"
#include "sitnu.intfb.h"
#include "tricsi.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCSI',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

LL3D=(NCONF == 1.OR.NCONF == 131.OR.NCONF == 302.OR.NCONF == 401 &
 & .OR.NCONF == 501.OR.NCONF == 601.OR.NCONF == 801)
LLDOSI=LL3D.AND.(CDCONF == 'A'.OR.CDCONF == 'I'.OR. &
 & CDCONF == 'S' .OR. CDCONF == 'T')

!     ------------------------------------------------------------------

!*       2.    SEMI-IMPLICIT SPECTRAL COMPUTATIONS.
!              ------------------------------------

IF (LLDOSI) THEN

  ALLOCATE(ZSDIVP(NFLEVG,MAX(NSPEC2V,NSPEC2VF)))
  ALLOCATE(ZSPDIVP(NFLEVG,MAX(NSPEC2V,NSPEC2VF)))

  !*        2.1  Preliminary initialisations.

  IF (LDONEM) THEN
    IOFF=NPTRSVF(MYSETV)-1
  ELSE
    IOFF=NPTRSV(MYSETV)-1
  ENDIF
  ISPCOL=KEND-KSTA+1

  IF (LSLAG) THEN
    IF (LTWOTL) THEN
      ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT*(1.0_JPRB+XIDT)
    ELSE
      ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT
    ENDIF
  ELSE
    ZBDT=0.5_JPRB*TDT*BETADT
  ENDIF
  ZBDT2=(ZBDT*RSTRET)**2

  !*        2.3  Computes right-hand side of Helmholtz equation.

  IF( .NOT.LDONEM ) CALL GSTATS(1655,0)
  CALL SIGAM(1,NFLEVG,ZSDIV,PSPTG(:,KSTA:KEND),PSPSPG(KSTA:KEND),ISPCOL)
  IF( .NOT.LDONEM ) CALL GSTATS(1655,1)

  IF( .NOT.LDONEM ) CALL GSTATS(1656,0)
  IF (LSIDG) THEN
    IF (KM > 0) THEN
!$OMP PARALLEL DO PRIVATE(JSP,JLEV,IN)
      DO JSP=KSTA,KEND
        DO JLEV=1,NFLEVG
          IN=NVALUE(JSP+IOFF)
          ZSDIV(JLEV,JSP)=RLAPIN(IN)*PSPDIVG(JLEV,JSP)&
           & -ZBDT*ZSDIV(JLEV,JSP)      
        ENDDO
      ENDDO
!$OMP END PARALLEL DO
    ELSE
!$OMP PARALLEL DO PRIVATE(JSP,JLEV,IN)
      DO JSP=KSTA,KEND
        DO JLEV=1,NFLEVG
          IN=NVALUE(JSP+IOFF)
          ZSDIV(JLEV,JSP)=PSPDIVG(JLEV,JSP)&
           & -ZBDT*RLAPDI(IN)*ZSDIV(JLEV,JSP)  
        ENDDO
      ENDDO
!$OMP END PARALLEL DO
    ENDIF
  ELSE

    ! Case of No Stretching
!$OMP PARALLEL DO PRIVATE(JSP,JLEV,IN)
    DO JSP=KSTA,KEND
      DO JLEV=1,NFLEVG
        IN=NVALUE(JSP+IOFF)
        ZSDIV(JLEV,JSP)=PSPDIVG(JLEV,JSP)-ZBDT*RLAPDI(IN)*ZSDIV(JLEV,JSP)
      ENDDO
    ENDDO
!$OMP END PARALLEL DO
  ENDIF

  !        Add [F] * result to rhs of Helmholtz equation

  IF (LIMPF) THEN
!$OMP PARALLEL DO PRIVATE(JSP,JLEV)
    DO JSP=KSTA,KEND
      DO JLEV=1,NFLEVG
        ZSDIV(JLEV,JSP)=ZSDIV(JLEV,JSP) + PSPAUXG(JLEV,JSP)
      ENDDO
    ENDDO
!$OMP END PARALLEL DO
  ENDIF
  IF( .NOT.LDONEM ) CALL GSTATS(1656,1)

  !*        2.4  Solve Helmholtz equation

  IF (LSITRIC) THEN

    ! Tricky: The PE that owns wavenumber 0 starts at (m,n)=(1,1), so istart=3

    IF ((KM == 0.AND.LDONEM).OR.(MYSETW == NPROCM(0).AND..NOT.LDONEM)) THEN
      ISTART=KSTA+2
    ELSE
      ISTART=KSTA
    ENDIF
    CALL TRICSI(PSPDIVG(:,ISTART:KEND),ZSDIV(:,ISTART:KEND),&
     & SITRICA,SITRICB,SITRICC,SIRDEL,ISTART,KEND)  
  ELSE

    !           Current space --> vertical eigenmodes space.

    IF( .NOT.LDONEM ) CALL GSTATS(2020,0)
    CALL MXMAOP(SIMI,1,NFLEVG,ZSDIV,1,NFLEVG,ZSDIVP(:,KSTA:KEND),1,NFLEVG,&
     & NFLEVG,NFLEVG,ISPCOL)  
    IF( .NOT.LDONEM ) CALL GSTATS(2020,1)

    IF (LSIDG) THEN

      !             Inversion of two tridiagonal systems (Helmholtz equation)
      !                --> (SIMI*DIVprim(t+dt)).

      !             Reorganisation of divergence

      IS0=NSE0L(KMLOC)
      IS02=0
      II=MIN(KM,1)+1
      ZSDIVPL(:,:,:)=0.0_JPRB
      ZSPDIVPL(:,:,:)=0.0_JPRB

      DO JN=KM,NSMAX
        ISE=KSTA+2*(JN-KM)
        ZSDIVPL(:,JN,1:2)=ZSDIVP(:,ISE:ISE+1)
      ENDDO
      IF (KM > 0) THEN

        !               Inversion of a symmetric matrix.

        CALL MXTURS(NSMAX+1-KM,NFLEVG,NFLEVG,II,&
         & SIHEG(1,IS0+1,1),SIHEG(1,IS0+1,2),SIHEG(1,IS0+1,3),&
         & ZSDIVPL,ZSPDIVPL)  
      ELSE

        !               Inversion of a non-symmetric matrix.

        CALL MXTURE(NSMAX+1-KM,NFLEVG,NFLEVG,II,-2,.TRUE.,&
         & SIHEG(1,IS0+1,1),SIHEG(1,IS0+1,2),SIHEG(1,IS0+1,3),&
         & ZSDIVPL,ZSPDIVPL)  
        CALL MXTURE(NSMAX+1-KM,NFLEVG,NFLEVG,II,3,.FALSE.,&
         & SIHEG(1,IS0+1,1),SIHEG2(1,IS02+1,2),&
         & SIHEG2(1,IS02+1,3),ZSDIVPL,ZSPDIVPL)  
      ENDIF

      DO JN=KM,NSMAX
        ISE=KSTA+2*(JN-KM)
        ZSPDIVP(:,ISE:ISE+1)=ZSPDIVPL(:,JN,1:2)
      ENDDO
    ELSE

      !             Case with NO Stretching :

      IF (LIMPF) THEN

        !               Solve complex pentadiagonal system

        CALL SPCIMPFSOLVE(LDONEM,ZSDIVP,ZSPDIVP)

      ELSE

        !                 Inversion of a diagonal system (Helmholtz equation)
        !                 --> (SIMI*DIVprim(t+dt)).

        DO JSP=KSTA,KEND
          DO JLEV=1,NFLEVG
            ZSPDIVP(JLEV,JSP)=ZSDIVP(JLEV,JSP)&
             & /(1.0_JPRB-ZBDT2*SIVP(JLEV)*RLAPDI(NVALUE(JSP+IOFF)))  
          ENDDO
        ENDDO
      ENDIF
    ENDIF

    !           Vertical eigenmodes space --> current space.

    IF( .NOT.LDONEM ) CALL GSTATS(2020,0)
    CALL MXMAOP(SIMO,1,NFLEVG,ZSPDIVP(:,KSTA:KEND),1,NFLEVG,PSPDIVG(:,KSTA:KEND),1,&
     & NFLEVG,NFLEVG,NFLEVG,ISPCOL)  
    IF( .NOT.LDONEM ) CALL GSTATS(2020,1)
  ENDIF
  IF (LSIDG) THEN

    !           ZSPDIV=(DIVprim(t+dt)) --> ZSPDIVG=(GM**2 * DIVprim(t+dt)) .

    ZSDIVPL(:,:,:)=0.0_JPRB
    ZSPDIVPL(:,:,:)=0.0_JPRB

    !           Reorganisation of ZSDIVP (Back to the USSR)

    DO JN=KM,NSMAX
      ISE=KSTA+2*(JN-KM)
      ZSDIVPL(:,JN,1:2)=PSPDIVG(:,ISE:ISE+1)
    ENDDO

    !        ZSPDIV=(DIVprim(t+dt)) --> ZPSPDIVG=(GMBAR**2 * DIVprim(t+dt)).

    CALL MXPTMA(NSMAX+1-KM,NFLEVG,NFLEVG,II,SCGMAP(IS0+1,1),&
     & SCGMAP(IS0+1,2),SCGMAP(IS0+1,3),&
     & SCGMAP(IS0+1,2),SCGMAP(IS0+1,3),&
     & ZSDIVPL,ZSPDIVPL)  

    !           Reorganisation of ZSPDIVPL

    DO JN=KM,NSMAX
      ISE=KSTA+2*(JN-KM)
      ZHELP(:,ISE:ISE+1)=ZSPDIVPL(:,JN,1:2)
    ENDDO
  ELSE

    !       ZSPDIV=(DIVprim(t+dt)) --> ZSPDIVG=(GMBAR**2 * DIVprim(t+dt)) .

    IF( .NOT.LDONEM ) CALL GSTATS(1656,0)
!$OMP PARALLEL DO PRIVATE(JSP,JLEV)
    DO JSP=KSTA,KEND
      DO JLEV=1,NFLEVG
        ZHELP(JLEV,JSP)=PSPDIVG(JLEV,JSP)*RSTRET*RSTRET
      ENDDO
    ENDDO
!$OMP END PARALLEL DO
    IF( .NOT.LDONEM ) CALL GSTATS(1656,1)

  ENDIF

  !       If LSIDG:
  !         (GM**2 * DIVprim(t+dt)) --> [ tau * (GM**2 * DIVprim(t+dt)) ]
  !                                 and [  nu * (GM**2 * DIVprim(t+dt)) ]
  !       or if not LSIDG:
  !         (GMBAR**2 * DIVprim(t+dt)) --> [ tau * (GMBAR**2 * DIVprim(t+dt)) ]
  !                                    and [  nu * (GMBAR**2 * DIVprim(t+dt)) ]

  IF( .NOT.LDONEM ) CALL GSTATS(1657,0)
  CALL SITNU(1,NFLEVG,ZHELP,ZST,ZSP,ISPCOL)
  IF( .NOT.LDONEM ) CALL GSTATS(1657,1)

  !*       2.5  Increment Temperature and surface pressure

  IF( .NOT.LDONEM ) CALL GSTATS(1656,0)
!$OMP PARALLEL DO PRIVATE(JSP,JLEV)
  DO JSP=KSTA,KEND
    DO JLEV=1,NFLEVG
      PSPTG(JLEV,JSP)=PSPTG(JLEV,JSP)-ZBDT*ZST(JLEV,JSP)
    ENDDO
    PSPSPG(JSP)=PSPSPG(JSP)-ZBDT*ZSP(JSP)
  ENDDO
!$OMP END PARALLEL DO
  IF( .NOT.LDONEM ) CALL GSTATS(1656,1)

  DEALLOCATE(ZSDIVP)
  DEALLOCATE(ZSPDIVP)

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCSI',1,ZHOOK_HANDLE)
END SUBROUTINE SPCSI
