SUBROUTINE SPCSIAD(CDCONF,KM,KMLOC,KSTA,KEND,LDONEM,PSPAUXG, &
 & PSPVORG,PSPDIVG,PSPTG,PSPSPG)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : MYSETV   ,NPTRSV   ,NPTRSVF
USE YOMDIM   , ONLY : NFLEVG   ,NSMAX
USE YOMCT0   , ONLY : NCONF    ,LSLAG    ,LTWOTL
USE YOMDYN   , ONLY : SCGMAP   ,SIMO     ,SIMI     ,SIVP     ,&
 & SIHEG    ,SIHEG2   ,TDT      ,BETADT   ,VESL     ,LSIDG    ,&
 & XIDT     ,LIMPF  
USE YOMLAP   , ONLY : NVALUE   ,NSE0L    ,RLAPDI   ,RLAPIN
USE YOMGEM   , ONLY : RSTRET


!**** *SPCSIAD* - SPECTRAL SPACE SEMI-IMPLICIT COMPUTATIONS - ADJOINT

!               Caution! Only works for a triangular truncation, i.e.
!                        NASRN(n)=n.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!        *CALL* *SPCSIAD(..)

!        Explicit arguments :  CDCONF  - configuration (see doc.)
!        --------------------  KM      - Zonal wavenumber 
!                              KMLOC   - Zonal wavenumber (DM-local numbering)
!                              KSTA    - First column processed
!                              KEND    - Number of columns processed
!                              LDONEM  - T if only one m if processed
!                              PSPVORG - Vorticity columns
!                              PSPDIVG - Divergence columns
!                              PSPTG   - Temperature columns
!                              PSPSPG  - Surface Pressure

!     Method.
!     -------

!     Externals.
!     ----------
!        Calls SITNUAD,SIGAMAD,MXMAOP,MXTURE,MXTURS,MXPTMA.
!        Called by SPCMAD.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud, Jean-Noel Thepaut and Philippe Courtier (ECMWF)

!     Modifications.
!     --------------
!        Original : 87-11-24 (in the good old days spcsiad.F was part 
!                             of spcad.F)
!        Adjoint of the semi-implicit part: 89-12-12
!        K. YESSAD (03-1994): multitasking on zonal wave number.
!                             semi-implicit scheme with not reduced divergence.
!                             different coefficients for T and q diffusions.
!                             unified horizontal diffusions.
!                             CDCONF='I'.
!                             content of SPHDGAD put into SPCAD.
!        K. YESSAD (APRIL 1994): Unified horizontal diffusion scheme.
!        K. YESSAD (MARCH 1995):
!            - Real/imag parts stored in conventional order in work arrays
!              as it is already made in SPC.
!            - Use new array NSE0 (of YOMLAP) to define IS0.
!        L. Isaksen 95-06-06  Reordering of spectral arrays
!        K. YESSAD (NOV 1995): removal of HDC horizontal diffusion scheme.
!        L. Isaksen 96-05-05  NFLEV --> NFLEVG for consistency only
!        M.Tolstykh 96-10-14 Fourier space horizontal diffusion
!        L.Isaksen 96-12-12 Merged with message passing version.
!                           Renamed and recoded.
!        C. Temperton 97-02-24: modified first timestep when XIDT>0
!        C. Temperton 99-09-20: LIMPF option added.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad 15-May-2006: memory optimisations for stretched geometry
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!        T.Wilhelmsson 09-09-30: Remove LFULLM requirement for LIMPF
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMLOC
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
LOGICAL           ,INTENT(IN)    :: LDONEM 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPAUXG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPVORG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPDIVG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPTG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPSPG(:) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZSDIV  (NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZHELP  (NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZST    (NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSP    (       KSTA:KEND)
REAL(KIND=JPRB) :: ZSDIVP (NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSPDIVP(NFLEVG,KSTA:KEND)

REAL(KIND=JPRB) :: ZSDIVPL (NFLEVG,KM:NSMAX,2)
REAL(KIND=JPRB) :: ZSPDIVPL(NFLEVG,KM:NSMAX,2)

INTEGER(KIND=JPIM) :: II, IN, IOFF, IS0, IS02, &
 & ISE, ISPCOL, JLEV, JN, JSP

LOGICAL :: LL3D,LLDOSI

REAL(KIND=JPRB) :: ZBDT, ZBDT2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "mxmaop.h"
END INTERFACE

#include "mxptma.h"
#include "mxture.h"
#include "mxturs.h"
#include "sigamad.intfb.h"
#include "spcimpfsolvead.intfb.h"
#include "sitnuad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCSIAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

LL3D=(NCONF == 1.OR.NCONF == 131.OR.NCONF == 302.OR.NCONF == 401 &
   & .OR.NCONF == 501.OR.NCONF == 601.OR.NCONF == 801)

LLDOSI=LL3D.AND.(CDCONF == 'A'.OR.CDCONF == 'I'.OR. &
 & CDCONF == 'S' .OR. CDCONF == 'T')

IF (LLDOSI) THEN

!*       1.    SEMI-IMPLICIT SPECTRAL COMPUTATIONS.
!              ------------------------------------

!*        1.1  Preliminary initialisations.

  IF (LDONEM) THEN
    IOFF=NPTRSVF(MYSETV)-1
  ELSE
    IOFF=NPTRSV(MYSETV)-1
  ENDIF
  ISPCOL=KEND-KSTA+1

  IF (LSLAG) THEN
    IF (LTWOTL) THEN
      ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT*(1.0_JPRB+XIDT)
    ELSE
      ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT
    ENDIF
  ELSE
    ZBDT=0.5_JPRB*TDT*BETADT
  ENDIF
  ZBDT2=(ZBDT*RSTRET)**2

  ZHELP(:,:)=0.0_JPRB
  ZST(:,:)=0.0_JPRB
  ZSP(:)=0.0_JPRB


!*       1.4  Increment Temperature and surface pressure

  IF( .NOT.LDONEM ) CALL GSTATS(1659,0)
!$OMP PARALLEL DO PRIVATE(JSP,JLEV)
  DO JSP=KSTA,KEND
    DO JLEV=1,NFLEVG
      ZST(JLEV,JSP)=ZST(JLEV,JSP)-ZBDT*PSPTG(JLEV,JSP)
    ENDDO
    ZSP(JSP)=ZSP(JSP)-ZBDT*PSPSPG(JSP)
  ENDDO
!$OMP END PARALLEL DO
  IF( .NOT.LDONEM ) CALL GSTATS(1659,1)

!*       1.3  Solve Helmholtz equation

!        If LSIDG, adjoint of following operation:
!          (GM**2 * DIVprim(t+dt)) --> [ tau * (GM**2 * DIVprim(t+dt)) ]
!                                  and [  nu * (GM**2 * DIVprim(t+dt)) ]
!        or if not LSIDG, adjoint of following operation:
!          (GMBAR**2 * DIVprim(t+dt)) --> [ tau * (GMBAR**2 * DIVprim(t+dt)) ]
!                                     and [  nu * (GMBAR**2 * DIVprim(t+dt)) ]
  IF( .NOT.LDONEM ) CALL GSTATS(2022,0)
  CALL SITNUAD(1,NFLEVG,ZHELP,ZST,ZSP,ISPCOL)
  IF( .NOT.LDONEM ) CALL GSTATS(2022,1)
  IF (LSIDG) THEN

    ZSPDIVPL(:,:,:)=0.0_JPRB
    ZSDIVPL(:,:,:)=0.0_JPRB

    DO JN=KM,NSMAX
      ISE=KSTA+2*(JN-KM)
      ZSPDIVPL(:,JN,1:2)=ZHELP(:,ISE:ISE+1)
    ENDDO
    
    IS0=NSE0L(KMLOC)
    IS02=0
    II=MIN(KM,1)+1
    
!           Adjoint of:
!           ZSPDIV=(DIVprim(t+dt)) --> ZHELP=(GM**2 * DIVprim(t+dt)) .
    CALL MXPTMA(NSMAX+1-KM,NFLEVG,NFLEVG,II,SCGMAP(IS0+1,1),&
      & SCGMAP(IS0+1,2),SCGMAP(IS0+1,3),&
      & SCGMAP(IS0+1,2),SCGMAP(IS0+1,3),&
      & ZSPDIVPL,ZSDIVPL)  

    DO JN=KM,NSMAX
      ISE=KSTA+2*(JN-KM)
      ZHELP(:,ISE:ISE+1)=ZSDIVPL(:,JN,1:2)
    ENDDO

    DO JSP=KSTA,KEND
      DO JLEV=1,NFLEVG
        PSPDIVG(JLEV,JSP)=PSPDIVG(JLEV,JSP)+ZHELP(JLEV,JSP)
      ENDDO
    ENDDO

  ELSE

!           Adjoint of:
!        ZSPDIV=(DIVprim(t+dt)) --> ZPSPDIVG=(GMBAR**2 * DIVprim(t+dt)).

    IF( .NOT.LDONEM ) CALL GSTATS(1659,0)
!$OMP PARALLEL DO PRIVATE(JSP,JLEV)
    DO JSP=KSTA,KEND
      DO JLEV=1,NFLEVG
        PSPDIVG(JLEV,JSP)=PSPDIVG(JLEV,JSP)+ZHELP(JLEV,JSP)*RSTRET*RSTRET
      ENDDO
    ENDDO
!$OMP END PARALLEL DO
    IF( .NOT.LDONEM ) CALL GSTATS(1659,1)
  ENDIF

!         Vertical eigenmodes space --> current space.

  IF( .NOT.LDONEM ) CALL GSTATS(2021,0)
  CALL MXMAOP(SIMO,NFLEVG,1,PSPDIVG(:,KSTA:KEND),1,NFLEVG,ZSPDIVP,1,NFLEVG,&
    & NFLEVG,NFLEVG,ISPCOL)  
  IF( .NOT.LDONEM ) CALL GSTATS(2021,1)

  IF (LSIDG) THEN

!             Adjoint of: inversion of two tridiagonal systems
!                (Helmholtz equation) --> (SIMI*DIVprim(t+dt)).

    DO JN=KM,NSMAX
      ISE=KSTA+2*(JN-KM)
      ZSPDIVPL(:,JN,1:2)=ZSPDIVP(:,ISE:ISE+1)
    ENDDO
    
    IF (KM > 0) THEN
!             Inversion of a symmetric matrix: we keep the same
!              LU decomposition as in SPC.
      CALL MXTURS(NSMAX+1-KM,NFLEVG,NFLEVG,II,&
        & SIHEG(1,IS0+1,1),SIHEG(1,IS0+1,2),SIHEG(1,IS0+1,3),&
        & ZSPDIVPL,ZSDIVPL)  
    ELSEIF (KM == 0) THEN
!             Inversion of a non symmetric matrix: the decomposition
!              LU in the direct code gives here a decomposition
!              transp(U)*transp(L).
      CALL MXTURE(NSMAX+1-KM,NFLEVG,NFLEVG,II,-3,.TRUE.,&
        & SIHEG(1,IS0+1,1),SIHEG2(1,IS02+1,2),&
        & SIHEG2(1,IS02+1,3),ZSPDIVPL,ZSDIVPL)  
      CALL MXTURE(NSMAX+1-KM,NFLEVG,NFLEVG,II,2,.FALSE.,&
        & SIHEG(1,IS0+1,1),SIHEG(1,IS0+1,2),SIHEG(1,IS0+1,3),&
        & ZSPDIVPL,ZSDIVPL)  
    ENDIF
      
    DO JN=KM,NSMAX
      ISE=KSTA+2*(JN-KM)
      ZSDIVP(:,ISE:ISE+1)=ZSDIVPL(:,JN,1:2)
    ENDDO
    
  ELSEIF (LIMPF) THEN
      
!               Solve complex pentadiagonal system

    CALL SPCIMPFSOLVEAD(LDONEM,ZSPDIVP,ZSDIVP)  
    
  ELSE

!           Inversion of a diagonal system (Helmholtz equation)
!            --> (SIMI*DIVprim(t+dt)).

    DO JSP=KSTA,KEND
      DO JLEV=1,NFLEVG
        ZSDIVP(JLEV,JSP)=ZSPDIVP(JLEV,JSP)&
          & /(1.0_JPRB-ZBDT2*SIVP(JLEV)*RLAPDI(NVALUE(JSP+IOFF)))  
      ENDDO
    ENDDO
  ENDIF

!         Current space --> vertical eigenmodes space.

  IF( .NOT.LDONEM ) CALL GSTATS(2021,0)
  CALL MXMAOP(SIMI,NFLEVG,1,ZSDIVP,1,NFLEVG,ZSDIV,1,NFLEVG,&
    & NFLEVG,NFLEVG,ISPCOL)
  IF( .NOT.LDONEM ) CALL GSTATS(2021,1)


!        Add [F] * result to rhs of Helmholtz equation

  IF (LIMPF) THEN
    DO JSP=KSTA,KEND
      DO JLEV=1,NFLEVG
        PSPAUXG(JLEV,JSP) = ZSDIV(JLEV,JSP)
      ENDDO
    ENDDO
  ENDIF

!*      2.2  Computes right-hand side of Helmholtz equation.

  IF( .NOT.LDONEM ) CALL GSTATS(1659,0)
  IF (LSIDG) THEN
    IF (KM > 0) THEN
!$OMP PARALLEL DO PRIVATE(JSP,JLEV,IN)
      DO JSP=KSTA,KEND
        DO JLEV=1,NFLEVG
          IN=NVALUE(JSP+IOFF)
          PSPDIVG(JLEV,JSP)=RLAPIN(IN)*ZSDIV(JLEV,JSP)
          ZSDIV(JLEV,JSP)=-ZBDT*ZSDIV(JLEV,JSP)
        ENDDO
      ENDDO
!$OMP END PARALLEL DO
    ELSE
!$OMP PARALLEL DO PRIVATE(JSP,JLEV,IN)
      DO JSP=KSTA,KEND
        DO JLEV=1,NFLEVG
          IN=NVALUE(JSP+IOFF)
          PSPDIVG(JLEV,JSP)=ZSDIV(JLEV,JSP)
          ZSDIV(JLEV,JSP)=-ZBDT*RLAPDI(IN)*ZSDIV(JLEV,JSP)
        ENDDO
      ENDDO
!$OMP END PARALLEL DO
    ENDIF
  ELSE
!$OMP PARALLEL DO PRIVATE(JSP,JLEV,IN)
    DO JSP=KSTA,KEND
      DO JLEV=1,NFLEVG
        IN=NVALUE(JSP+IOFF)
        PSPDIVG(JLEV,JSP)=ZSDIV(JLEV,JSP)
        ZSDIV(JLEV,JSP)=-ZBDT*RLAPDI(IN)*ZSDIV(JLEV,JSP)
      ENDDO
    ENDDO
!$OMP END PARALLEL DO
  ENDIF
  IF( .NOT.LDONEM ) CALL GSTATS(1659,1)

  IF( .NOT.LDONEM ) CALL GSTATS(1658,0)
  CALL SIGAMAD(1,NFLEVG,ZSDIV,PSPTG(:,KSTA:KEND),PSPSPG(KSTA:KEND),ISPCOL)
  IF( .NOT.LDONEM ) CALL GSTATS(1658,1)
 
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCSIAD',1,ZHOOK_HANDLE)
END SUBROUTINE SPCSIAD
