SUBROUTINE SPNH_CONV_NHVAR(LDMODEL_TO_FILE)

!* SPNH_CONV_NHVAR - Conversion of NH variables (model to file and vice-versa)

! Purpose
! -------
!   Converts NH pressure departure and pseudovertical divergence
!   between model and file. Variables are scaled in model,
!   unscaled in file. Modifies spectral buffers:
!     SPSPD
!     SPSVD
!     SPNHX (only for d4 + file ---> model conversion)

! Interface
! ---------
!   LDMODEL_TO_FILE - switch for model to file / file to model conversion

! Externals
! ---------

! Method
! ------
!   See documentation

! Reference
! ---------

! Author
! ------
!   25-Jun-2001 P. Smolikova, J. Masek

! Modifications
! -------------
!   31-Mar-2002 P. Smolikova  option NVDVAR=4
!   31-Jan-2003 P. Smolikova  cleaning, bf in GNHX calls
!   19-Sep-2003 G. Hello      cleaning and use of pointers
!   01-Oct-2003 M. Hamrud     CY28 Cleaning
!   23-Mar-2004 G. Hello      Bug correction in file to model part
!   03-Jun-2004 J. Masek      NH cleaning (NPDVAR, NVDVAR)
!   25-Nov-2004 Y. Seity      new arguments to GNHX for R and CP calc.
!   27-Jan-2005 K. Yessad     GNHPDVDCONV-> SPNH_CONV_NHVAR + modularisation
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

USE YOMCT0   , ONLY : LNHDYN   ,LELAM
USE YOMDIM   , ONLY : NFLEVG   ,LSPT 
USE YOMDYNA  , ONLY : NVDVAR   ,LNH_PDVD ,LNH_GEOGW
USE YOMGEM   , ONLY : NGPTOT
USE YOMLUN   , ONLY : NULOUT
USE YOM_YGFL , ONLY : YQ       ,YL       ,YI

!------------------------------------------------------------------------------

IMPLICIT NONE

LOGICAL,INTENT(IN) :: LDMODEL_TO_FILE 

!------------------------------------------------------------------------------

REAL(KIND=JPRB) :: ZU     (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZV     (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZT     (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZTL    (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZTM    (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZPD    (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZPDL   (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZPDM   (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZVD    (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZNHX   (NGPTOT,NFLEVG)

REAL(KIND=JPRB) :: ZSP    (NGPTOT)
REAL(KIND=JPRB) :: ZSPL   (NGPTOT)
REAL(KIND=JPRB) :: ZSPM   (NGPTOT)

REAL(KIND=JPRB) :: ZQ     (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZQL    (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZQM    (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZL     (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZLL    (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZLM    (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZI     (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZIL    (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZIM    (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZR     (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZS     (NGPTOT,NFLEVG)
REAL(KIND=JPRB) :: ZG     (NGPTOT,NFLEVG)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!------------------------------------------------------------------------------

#include "abor1.intfb.h"
#include "etransdir_nhconv.intfb.h"
#include "etransinv_nhconv.intfb.h"
#include "gnh_conv_nhvar.intfb.h"
#include "gnh_conv_nhvar_geogw.intfb.h"
#include "transdir_nhconv.intfb.h"
#include "transinv_nhconv.intfb.h"

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPNH_CONV_NHVAR',0,ZHOOK_HANDLE)

!------------------------------------------------------------------------------

!* 1. CHECK CONSISTENCY
!     -----------------

IF ( .NOT.LNHDYN ) THEN
  WRITE(NULOUT,*) 'LNHDYN=.F. => SPNH_CONV_NHVAR SHOULD NOT BE CALLED!'
  CALL ABOR1('SPNH_CONV_NHVAR: ABOR1 CALLED!')
ELSEIF ( .NOT.LSPT ) THEN
  WRITE(NULOUT,*) 'LSPT=.F. => CANNOT TRANSFORM NH VARIABLES!'
  CALL ABOR1('SPNH_CONV_NHVAR: ABOR1 CALLED!')
ELSEIF ( (YQ%LGP.OR.YL%LGP.OR.YI%LGP) .AND. NVDVAR == 4 ) THEN
!  WRITE(NULOUT,*) &
!   & 'CANNOT TRANSFORM NH VARIABLES (GP specific humidity and NVDVAR=4)'  
!  CALL ABOR1('SPNH_CONV_NHVAR: ABOR1 CALLED!')
ENDIF

!------------------------------------------------------------------------------

!* 2. TRANSFORM SPECTRAL FIELDS TO GRID POINT SPACE
!     ---------------------------------------------

IF (LELAM) THEN
  CALL ETRANSINV_NHCONV(&
   & ZU,ZV,ZT,ZTL,ZTM,ZPD,ZPDL,ZPDM,ZVD,ZSP,ZSPL,ZSPM, &
   & ZQ,ZQL,ZQM,ZL,ZLL,ZLM,ZI,ZIL,ZIM,ZR,ZS,ZG &
   &)
ELSE
  CALL TRANSINV_NHCONV(&
   & ZU,ZV,ZT,ZTL,ZTM,ZPD,ZPDL,ZPDM,ZVD,ZSP,ZSPL,ZSPM, &
   & ZQ,ZQL,ZQM,ZL,ZLL,ZLM,ZI,ZIL,ZIM,ZR,ZS,ZG &
   &)
ENDIF

!------------------------------------------------------------------------------

!* 3. GRID POINT COMPUTATIONS
!     -----------------------

IF (LNH_PDVD) THEN
  CALL GNH_CONV_NHVAR(LDMODEL_TO_FILE, &
   & ZU,ZV,ZT,ZTL,ZTM,ZPD,ZPDL,ZPDM,ZVD,ZSP,ZSPL,ZSPM, &
   & ZQ,ZQL,ZQM,ZL,ZLL,ZLM,ZI,ZIL,ZIM,ZR,ZS,ZG,ZNHX)
ELSEIF (LNH_GEOGW) THEN
  CALL GNH_CONV_NHVAR_GEOGW(LDMODEL_TO_FILE, &
   & ZU,ZV,ZT,ZTL,ZTM,ZPD,ZVD,ZSP,ZQ,ZQL,ZQM,ZL,ZI,ZR,ZS,ZG)
ENDIF

!------------------------------------------------------------------------------

!* 4. TRANSFORM NH FIELDS BACK TO SPECTRAL SPACE
!     ------------------------------------------

IF (LELAM) THEN
  CALL ETRANSDIR_NHCONV(LDMODEL_TO_FILE,ZPD,ZVD,ZNHX)
ELSE
  CALL TRANSDIR_NHCONV(LDMODEL_TO_FILE,ZPD,ZVD,ZNHX)
ENDIF

!------------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPNH_CONV_NHVAR',1,ZHOOK_HANDLE)

END SUBROUTINE SPNH_CONV_NHVAR

