SUBROUTINE SPNHSI(&
 ! --- INPUT -----------------------------------------------------------------
 & CDCONF,KM,KMLOC,KSTA,KEND,LDONEM, &
 ! --- INOUT -----------------------------------------------------------------
 & PSPVORG,PSPDIVG,PSPTG,PSPSPG,PSPSPDG,PSPSVDG)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RD       ,RCPD     ,RCVD     ,RG       ,ROMEGA
USE YOMMP    , ONLY : MYSETV   ,NPTRSV   ,NPTRSVF  
USE YOMDIM   , ONLY : NFLEVG   ,NSMAX
USE YOMCT0   , ONLY : NCONF    ,LSLAG    ,LTWOTL   ,LSITRIC  
USE YOMDYN   , ONLY : SCGMAP   ,SIMO     ,SIMI     ,SIFACI   ,&
 & SITR     ,SITRA    ,SIVP     ,SIHEG    ,SIHEG2   ,SIHEGB   ,SIHEGB2  ,&
 & TDT      ,BETADT   ,VESL     ,LSIDG    ,XIDT     ,NITERHELM,&
 & LIMPF    ,NDLNPR
USE YOMLAP   , ONLY : NVALUE   ,NSE0L    ,RLAPDI   ,RLAPIN
USE YOMGEM   , ONLY : RSTRET
USE YOMCVER  , ONLY : LRNHC1

!**** *SPNHSI* - SPECTRAL SPACE SEMI-IMPLICIT COMPUTATIONS FOR NH MODEL.

!     Purpose.
!     --------

!     Semi-implicit scheme in the NH model with (Qcha, dver or d4) as
!     NH prognostic variables.

!     When the constraint C1 is matched, I_NITERHELM=0; if not matched
!     I_NITERHELM>0.
!     The predictor has dver(t+dt) (or d4(t+dt)), D'(t+dt), zeta'(t+dt),
!     Qcha(t+dt), T(t+dt), log(prehyds)(t+dt) as unknown.
!     The corrector steps (JITER>0) work in an incremental manner,
!     the unknowns are (delta X) = X(jiter) - X(iter=0), where
!     X=dver(t+dt), D'(t+dt), etc...
!     The incremental method makes the calculation of the RHS simpler
!     in the corrector steps; this is equivalent to replace:
!      * zeta_star by 0.
!      * Dcha_star_star by 0.
!      * Dprim_star_star by
!        beta**2 (Delta t)**2 vnabla'**2 * C**2 * COR * Mbar**2 D'(t+dt,jiter-1)


!**   Interface.
!     ----------
!        *CALL* *SPNHSI(..)

!        Explicit arguments :
!        -------------------- 
!        * INPUT:
!          CDCONF      - configuration (see doc.)
!          KM          - Zonal wavenumber 
!          KMLOC       - Zonal wavenumber (DM-local numbering)
!          KSTA        - First column processed
!          KEND        - Last column processed
!          LDONEM      - T if only one m if processed
!        * INPUT/OUTPUT:
!          PSPVORG     - Vorticity columns
!          PSPDIVG     - Divergence columns
!          PSPTG       - Temperature columns
!          PSPSPG      - Surface Pressure
!          PSPSPDG     - Pressure departure variable columns
!          PSPSVDG     - Vertical divergence variable columns

!     Method.
!     -------

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. Yessad (after SPCSI and ESPC).

!     Modifications.
!     --------------
!   Original : 09-Dec-2004
!   K. Yessad 15-May-2006: memory optimisations for stretched geometry
!   K. Yessad Aug-2007: iterative algorithm to relax the C1 constraint
!   K. Yessad Aug-2007: LIMPF in NH model.
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMLOC
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
LOGICAL           ,INTENT(IN)    :: LDONEM 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPVORG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPDIVG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPTG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPSPG(:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPSPDG(:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPSVDG(:,:) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZZSPVORG(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZZSPDIVG(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZZSPTG(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZZSPSPG(KSTA:KEND)
REAL(KIND=JPRB) :: ZZSPSPDG(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZZSPSVDG(NFLEVG,KSTA:KEND)

REAL(KIND=JPRB),ALLOCATABLE :: ZZSPGDIVG(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZZSPDIV0G(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZZSPGDIV0G(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZZSPVOR0G(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZZSPSVD0G(:,:)

REAL(KIND=JPRB) :: ZSDIV(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZST(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSP(KSTA:KEND)
REAL(KIND=JPRB) :: ZSDIVP(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSPDIVP(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSVED(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZR1D(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZR1DPRIM(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZR2D(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZR3D(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZR4D(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSRHS(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZWORK(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSRHS2(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSRHS2_INC(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSNHP(NFLEVG,KSTA:KEND)
REAL(KIND=JPRB) :: ZSDIVPL(NFLEVG,KM:NSMAX,2)
REAL(KIND=JPRB) :: ZSPDIVPL(NFLEVG,KM:NSMAX,2)

REAL(KIND=JPRB) :: ZALPHA (KM:NSMAX+1)
REAL(KIND=JPRB) :: ZDENIM (KM:NSMAX+1)
REAL(KIND=JPRB) :: ZEPSI  (KM:NSMAX)
REAL(KIND=JPRB) :: ZFPLUS (KM:NSMAX+1)
REAL(KIND=JPRB) :: ZFMINUS(KM:NSMAX+1)

REAL(KIND=JPRB) :: ZSIVP2(NFLEVG)

INTEGER(KIND=JPIM) :: II, IN, IOFF, IS0, IS02, ISE, ISPCOL, JLEV, JSP, JN
INTEGER(KIND=JPIM) :: ILO, JL, IRSP, IMSP
INTEGER(KIND=JPIM) :: I_NITERHELM, JITER

REAL(KIND=JPRB) :: ZBDT, ZBDT2, ZRRT, ZCC, ZBTCM, ZBTCMH
REAL(KIND=JPRB) :: ZEM, ZAL, ZEN, ZF, ZTEMP, ZCT

LOGICAL :: LL3D, LLDOSI
LOGICAL :: LLNH_NOC1
LOGICAL :: LLVECHOR

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "mxmaop.h"
END INTERFACE

#include "mxptma.h"
#include "mxture.h"
#include "mxturs.h"
#include "siseve.intfb.h"
#include "si_cccor.intfb.h"
#include "sidd.intfb.h"
#include "sigam.intfb.h"
#include "simplico.h"
#include "si_mxptco.h"
#include "siptp.intfb.h"
#include "sitnu.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPNHSI',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

LL3D=(NCONF == 1.OR.NCONF == 131.OR.NCONF == 302.OR.NCONF == 401 &
 & .OR.NCONF == 501.OR.NCONF == 601.OR.NCONF == 801)
LLDOSI=LL3D.AND.(CDCONF == 'A'.OR.CDCONF == 'I'.OR.&
 & CDCONF == 'S' .OR. CDCONF == 'T')

! * Case where the C1 constraint is not matched.
LLNH_NOC1=(.NOT.(NDLNPR == 1)).AND.(.NOT.LRNHC1)
IF (LLNH_NOC1) THEN
  I_NITERHELM=NITERHELM
ELSE
  I_NITERHELM=0
ENDIF

!     ------------------------------------------------------------------

!*       1.    MEMORY TRANSFER.
!              ----------------

IF (LL3D) THEN
  IF (LIMPF) ZZSPVORG(1:NFLEVG,KSTA:KEND)=&
   & PSPVORG(1:NFLEVG,KSTA:KEND)  
  ZZSPDIVG(1:NFLEVG,KSTA:KEND)=PSPDIVG(1:NFLEVG,KSTA:KEND)
  ZZSPTG  (1:NFLEVG,KSTA:KEND)=PSPTG  (1:NFLEVG,KSTA:KEND)
  ZZSPSPG (         KSTA:KEND)=PSPSPG (         KSTA:KEND)
  ZZSPSPDG(1:NFLEVG,KSTA:KEND)=PSPSPDG(1:NFLEVG,KSTA:KEND)
  ZZSPSVDG(1:NFLEVG,KSTA:KEND)=PSPSVDG(1:NFLEVG,KSTA:KEND)
ENDIF

!     ------------------------------------------------------------------

!*       2.    SEMI-IMPLICIT SPECTRAL COMPUTATIONS.
!              ------------------------------------

IF (LLDOSI) THEN

  !*        2.1  Preliminary initialisations.

  IF (LDONEM) THEN
    IOFF=NPTRSVF(MYSETV)-1
  ELSE
    IOFF=NPTRSV(MYSETV)-1
  ENDIF
  ISPCOL=KEND-KSTA+1

  IF (LSLAG) THEN
    IF (LTWOTL) THEN
      ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT*(1.0_JPRB+XIDT)
    ELSE
      ZBDT=(1.0_JPRB+VESL)*0.5_JPRB*TDT*BETADT
    ENDIF
  ELSE
    ZBDT=0.5_JPRB*TDT*BETADT
  ENDIF
  ZBDT2=(ZBDT*RSTRET)**2
  ZRRT =  1.0_JPRB/(RD*SITR)
  ZCC  =  RCPD/RCVD
  ZBTCM=  ZBDT*ZBDT*RD*SITR*ZCC*RSTRET*RSTRET
  ZBTCMH= ZBTCM*RG*RG*ZRRT*ZRRT
  ZSIVP2(1:NFLEVG)=RD*SITR*(RCPD/RCVD)

  IF (LSIDG) THEN
    II=MIN(KM,1)+1
    IS0=NSE0L(KMLOC)
    IS02=0
  ENDIF

  IF (I_NITERHELM > 0) THEN
    ALLOCATE(ZZSPGDIVG(1:NFLEVG,KSTA:KEND))
    ALLOCATE(ZZSPDIV0G(1:NFLEVG,KSTA:KEND))
    ALLOCATE(ZZSPGDIV0G(1:NFLEVG,KSTA:KEND))
    ALLOCATE(ZZSPVOR0G(1:NFLEVG,KSTA:KEND))
    ALLOCATE(ZZSPSVD0G(1:NFLEVG,KSTA:KEND))
  ENDIF

  !*        2.2  Set up helper arrays for implicit Coriolis case.
  !              (identical to the part 2.2 of SPCSI).

  IF (LIMPF) THEN
    ZEM=REAL(KM,JPRB)
    ZAL=2.0_JPRB*ZBDT*ROMEGA*ZEM
    ILO=KM
    IF (KM == 0) THEN
      ZALPHA(0)=0.0_JPRB
      ZDENIM(0)=0.0_JPRB
      ZEPSI(0)=0.0_JPRB
      ILO=1
    ENDIF
    DO JL=ILO,NSMAX
      ZEN=REAL(JL,JPRB)
      ZALPHA(JL)=ZAL/(ZEN*(ZEN+1.0_JPRB))
      ZDENIM(JL)=1.0_JPRB/(1.0_JPRB+ZALPHA(JL)**2)
      ZEPSI(JL)=SQRT((ZEN*ZEN-ZEM*ZEM)/(4.0_JPRB*ZEN*ZEN-1.0_JPRB))
    ENDDO
    ZALPHA(NSMAX+1)=0.0_JPRB
    ZDENIM(NSMAX+1)=0.0_JPRB

    IF (KM == 0) THEN
      ZFPLUS(0)=0.0_JPRB
      ZFMINUS(0)=0.0_JPRB
    ENDIF
    ZF=2.0_JPRB*ZBDT*ROMEGA
    DO JL=ILO,NSMAX-1
      ZEN=REAL(JL,JPRB)
      ZFPLUS(JL)=ZF*ZEN*ZEPSI(JL+1)/(ZEN+1.0_JPRB)
      ZFMINUS(JL)=ZF*(ZEN+1.0_JPRB)*ZEPSI(JL)/ZEN
    ENDDO
    ZEN=REAL(NSMAX,JPRB)
    ZFPLUS(NSMAX)=0.0_JPRB
    ZFMINUS(NSMAX)=ZF*(ZEN+1.0_JPRB)*ZEPSI(NSMAX)/ZEN
    ZFPLUS(NSMAX+1)=0.0_JPRB
    ZFMINUS(NSMAX+1)=0.0_JPRB
  ENDIF

  DO JITER=0,I_NITERHELM

    !*        2.3  Computes right-hand side of Helmholtz equation.

    IF (JITER == 0) THEN

      ! * Provides:
      !   a/ - [ gamma [SITR Qcha_rhs - T_rhs] - (Rd SITR)/(SIPR) logprehyd_rhs ]
      !      in array ZSDIV.
      !   b/ (g TRSI)/(H TARSI) LLstar Qcha_rhs in array ZSVED.
      CALL SIDD(1,NFLEVG,ZSDIV,ZSVED,ZZSPSPDG,ZZSPTG,ZZSPSPG,ISPCOL)

      ! * Provides Dprim_star_star in ZSDIV.
      DO JLEV=1,NFLEVG
        DO JSP=KSTA,KEND
          IN=NVALUE(JSP+IOFF)
          ZSDIV(JLEV,JSP)=ZZSPDIVG(JLEV,JSP)-ZBDT*RLAPDI(IN)*ZSDIV(JLEV,JSP)
        ENDDO
      ENDDO

    ELSE

      ! * Compute C**2 * COR * Mbar**2 D'(t+dt,jiter-1)
      CALL SI_CCCOR(1,NFLEVG,ISPCOL,ZZSPGDIVG,ZSDIV)

      ! * Multiply by beta**2 (Delta t)**2 vnabla'**2
      DO JLEV=1,NFLEVG
        DO JSP=KSTA,KEND
          IN=NVALUE(JSP+IOFF)
          ZSDIV(JLEV,JSP)=ZBDT*ZBDT*RLAPDI(IN)*ZSDIV(JLEV,JSP)
        ENDDO
      ENDDO

    ENDIF

    ! * Provides Dprim_star_star_star (in ZSDIV) from Dprim_star_star (in ZSDIV)
    !   (the quantity added is the same one as in the end of part 2.3 of SPCSI).
    IF (LIMPF) THEN
      IF (JITER == 0) THEN

        IF (KM > 0) THEN
          DO JLEV=1,NFLEVG
            DO JN=KM,NSMAX
              IRSP=KSTA+(JN-KM)*2
              IMSP=IRSP+1
              ZTEMP=ZDENIM(JN)*(ZZSPVORG(JLEV,IMSP)&
               & +ZALPHA(JN)*ZZSPVORG(JLEV,IRSP))
              ZZSPVORG(JLEV,IRSP)=ZDENIM(JN)*(ZZSPVORG(JLEV,IRSP)&
               & -ZALPHA(JN)*ZZSPVORG(JLEV,IMSP))
              ZZSPVORG(JLEV,IMSP)=ZTEMP
            ENDDO
          ENDDO
        ENDIF

        !        Add [F] * result to rhs of Helmholtz equation

        DO JLEV=1,NFLEVG
          DO JN=KM+1,NSMAX
            IRSP=KSTA+(JN-KM)*2
            IMSP=IRSP+1
            ZSDIV(JLEV,IRSP)=ZSDIV(JLEV,IRSP)&
             & + ZFMINUS(JN)*ZZSPVORG(JLEV,IRSP-2)
            ZSDIV(JLEV,IMSP)=ZSDIV(JLEV,IMSP)&
             & + ZFMINUS(JN)*ZZSPVORG(JLEV,IMSP-2)
          ENDDO
        ENDDO

        DO JLEV=1,NFLEVG
          DO JN=KM,NSMAX-1
            IRSP=KSTA+(JN-KM)*2
            IMSP=IRSP+1
            ZSDIV(JLEV,IRSP)=ZSDIV(JLEV,IRSP)+ ZFPLUS(JN)*ZZSPVORG(JLEV,IRSP+2)
            ZSDIV(JLEV,IMSP)=ZSDIV(JLEV,IMSP)+ ZFPLUS(JN)*ZZSPVORG(JLEV,IMSP+2)
          ENDDO
        ENDDO

      ELSE

        ! zeta_star is replaced by zero.
        ZZSPVORG(1:NFLEVG,KSTA:KEND)=0.0_JPRB

      ENDIF
    ENDIF

    ! * Provides Dprim_star_star (resp. Dprim_star_star_star) in ZR1D and
    !   ZR1DPRIM if LIMPF=F (resp. LIMPF=T); and Dcha_star_star in ZR2D.
    IF (JITER == 0) THEN
      DO JLEV=1,NFLEVG
        DO JSP=KSTA,KEND
          ZR2D(JLEV,JSP)=ZZSPSVDG(JLEV,JSP)-ZBDT*ZSVED(JLEV,JSP)
        ENDDO
      ENDDO
    ELSE
      ! Dcha_star_star is replaced by zero.
      ZR2D(1:NFLEVG,KSTA:KEND)=0.0_JPRB
    ENDIF
    ZR1D(1:NFLEVG,KSTA:KEND)=ZSDIV(1:NFLEVG,KSTA:KEND)
    ZR1DPRIM(1:NFLEVG,KSTA:KEND)=ZSDIV(1:NFLEVG,KSTA:KEND)
    
    IF (LSIDG) THEN
      ! Multiply ZR1D by M**2 (CALL MXPTMA using SCGMAP),
      !  then divide it by RSTRET**2.
      !  Use ZSDIVPL and ZSPDIVPL as intermediate work arrays.
      ZSDIVPL(:,:,:)=0.0_JPRB
      ZSPDIVPL(:,:,:)=0.0_JPRB
      DO JN=KM,NSMAX
        ISE=KSTA+2*(JN-KM)
        ZSDIVPL(:,JN,1:2)=ZR1D(:,ISE:ISE+1)
      ENDDO
      CALL MXPTMA(NSMAX+1-KM,NFLEVG,NFLEVG,II,SCGMAP(IS0+1,1),&
       & SCGMAP(IS0+1,2),SCGMAP(IS0+1,3),&
       & SCGMAP(IS0+1,2),SCGMAP(IS0+1,3),&
       & ZSDIVPL,ZSPDIVPL)
      DO JN=KM,NSMAX
        ISE=KSTA+2*(JN-KM)
        ZR1D(:,ISE:ISE+1)=ZSPDIVPL(:,JN,1:2)/(RSTRET*RSTRET)
      ENDDO
    ENDIF

    ! * Provides "LLstar Dprim_star_star" if LIMPF=F, or
    !   "LLstar Dprim_star_star_star" if LIMPF=T
    !   (mult by a constant coefficient) in ZSRHS.
    CALL SISEVE(1,NFLEVG,ZR1D,ZSRHS,ISPCOL)

    ! * Provides "Tau Dprim_star_star" if LIMPF=F, or
    !   "Tau Dprim_star_star_star" if LIMPF=T (mult by a constant coefficient)
    !   in ZST.
    CALL SITNU(1,NFLEVG,ZR1D,ZST,ZSP,ISPCOL)

    ! * Provides "LLstar Tau Dprim_star_star" if LIMPF=F, or
    !   "LLstar Tau Dprim_star_star_star" if LIMPF=T (mult by a constant
    !   coefficient) in ZWORK.
    CALL SISEVE(1,NFLEVG,ZST,ZWORK,ISPCOL)
    DO JLEV=1,NFLEVG
      DO JSP=KSTA,KEND
        IN=NVALUE(JSP+IOFF)
        ZWORK(JLEV,JSP)=ZWORK(JLEV,JSP)*RCVD*ZRRT
      ENDDO
    ENDDO

    ! * Provides
    !   "[I - beta**2 (Delta t)**2 C**2 Mbar**2 vnabla'**2] Dcha_star_star"
    !   in ZSVED.
    IF (JITER == 0) THEN
      IF (LSIDG) THEN
        ! First compute ZR3D=ZR2D*RLAPDI.
        ! Multiply ZR3D by M**2 (CALL MXPTMA using SCGMAP),
        !  put the result in a separate array ZR4D,
        !  use ZSDIVPL and ZSPDIVPL as intermediate work arrays.
        ! Compute ZSVED using ZR2D and ZR4D:
        !  ZSVED=ZR2D-(ZBTCM/(RSTRET*RSTRET))*ZR4D.
        DO JLEV=1,NFLEVG
          DO JSP=KSTA,KEND
            IN=NVALUE(JSP+IOFF)
            ZR3D(JLEV,JSP)=RLAPDI(IN)*ZR2D(JLEV,JSP)
          ENDDO
        ENDDO
        ZSDIVPL(:,:,:)=0.0_JPRB
        ZSPDIVPL(:,:,:)=0.0_JPRB
        DO JN=KM,NSMAX
          ISE=KSTA+2*(JN-KM)
          ZSDIVPL(:,JN,1:2)=ZR3D(:,ISE:ISE+1)
        ENDDO
        CALL MXPTMA(NSMAX+1-KM,NFLEVG,NFLEVG,II,SCGMAP(IS0+1,1),&
         & SCGMAP(IS0+1,2),SCGMAP(IS0+1,3),&
         & SCGMAP(IS0+1,2),SCGMAP(IS0+1,3),&
         & ZSDIVPL,ZSPDIVPL)
        DO JN=KM,NSMAX
          ISE=KSTA+2*(JN-KM)
          ZR4D(:,ISE:ISE+1)=ZSPDIVPL(:,JN,1:2)
        ENDDO
        DO JLEV=1,NFLEVG
          DO JSP=KSTA,KEND
            ZSVED(JLEV,JSP)=ZR2D(JLEV,JSP)-(ZBTCM/(RSTRET*RSTRET))*ZR4D(JLEV,JSP)
          ENDDO
        ENDDO
      ELSE
        DO JLEV=1,NFLEVG
          DO JSP=KSTA,KEND
            IN=NVALUE(JSP+IOFF)
            ZSVED(JLEV,JSP)=ZR2D(JLEV,JSP)-ZBTCM*ZR2D(JLEV,JSP)*RLAPDI(IN)
          ENDDO
        ENDDO
      ENDIF
    ELSE
      ! ZSVED is simply 0 in this case.
      ZSVED(1:NFLEVG,KSTA:KEND)=0.0_JPRB
    ENDIF

    ! * Provides "SIFAC * RHS of the Helmholtz eqn" in ZSRHS2.
    DO JLEV=1,NFLEVG
      DO JSP=KSTA,KEND
        ZSRHS2(JLEV,JSP)=ZBTCMH*(SITR/SITRA)*&
         & (ZSRHS(JLEV,JSP)-ZWORK(JLEV,JSP))+&
         & ZSVED(JLEV,JSP)  
      ENDDO
    ENDDO

    ! * Case LIMPF: add terms containing Dcha_star_star(m,n),
    !   Dcha_star_star(m,n-2), Dcha_star_star(m,n+2) in the RHS.

    IF (LIMPF .AND. (JITER == 0)) THEN
      LLVECHOR=.FALSE.
      CALL SI_MXPTCO(KM,NSMAX,NFLEVG,NFLEVG,LLVECHOR,ZF,ZALPHA(KM),&
       & ZDENIM(KM),ZEPSI(KM),ZR2D,ZSRHS2_INC)
      ZSRHS2(1:NFLEVG,KSTA:KEND)=ZSRHS2(1:NFLEVG,KSTA:KEND) &
       & +ZSRHS2_INC(1:NFLEVG,KSTA:KEND)
    ENDIF

    ! * Multiply by array SIFACI to obtain the RHS of the Helmholtz eqn,
    !   stored in ZSRHS.
    CALL MXMAOP(SIFACI,1,NFLEVG,ZSRHS2,1,NFLEVG,ZSRHS,1,NFLEVG,&
     & NFLEVG,NFLEVG,ISPCOL)

    !*        2.4  Solve Helmholtz equation (compute dver(t+dt))

    IF (LSITRIC) THEN
      ! LSITRIC not yet coded in the NH model.
    ELSE

      ! Current space --> vertical eigenmodes space
      ! (multiply by matrix Q stored in SIMI).
      CALL MXMAOP(SIMI,1,NFLEVG,ZSRHS,1,NFLEVG,ZSDIVP,1,NFLEVG,&
       & NFLEVG,NFLEVG,ISPCOL)  

      ! Inversion of Helmholtz equation itself.

      IF (LSIDG) THEN

        ! Case designed for stretching :
        ! Use ZSDIVPL and ZSPDIVPL as intermediate work arrays.

        ZSDIVPL(:,:,:)=0.0_JPRB
        ZSPDIVPL(:,:,:)=0.0_JPRB
        DO JN=KM,NSMAX
          ISE=KSTA+2*(JN-KM)
          ZSDIVPL(:,JN,1:2)=ZSDIVP(:,ISE:ISE+1)
        ENDDO
        IF (KM > 0) THEN
          ! Inversion of a symmetric penta-diagonal matrix,
          !  provides grad'**2*Q*dver(t+dt).
          CALL MXTURS(NSMAX+1-KM,NFLEVG,NFLEVG,II,&
           & SIHEG(1,IS0+1,1),SIHEG(1,IS0+1,2),SIHEG(1,IS0+1,3),&
           & ZSDIVPL,ZSPDIVPL)  
          ! Multiply by grad'**(-2) and put the result in ZSPDIVP.
          DO JN=KM,NSMAX
            ISE=KSTA+2*(JN-KM)
            ZSPDIVP(:,ISE:ISE+1)=ZSPDIVPL(:,JN,1:2)*RLAPIN(JN)
          ENDDO
        ELSE
          ! Inversion of a non-symmetric penta-diagonal matrix,
          !  provides Q*dver(t+dt).
          CALL MXTURE(NSMAX+1-KM,NFLEVG,NFLEVG,II,-2,.TRUE.,&
           & SIHEG(1,IS0+1,1),SIHEG(1,IS0+1,2),SIHEG(1,IS0+1,3),&
           & ZSDIVPL,ZSPDIVPL)  
          CALL MXTURE(NSMAX+1-KM,NFLEVG,NFLEVG,II,3,.FALSE.,&
           & SIHEG(1,IS0+1,1),SIHEG2(1,IS02+1,2),&
           & SIHEG2(1,IS02+1,3),ZSDIVPL,ZSPDIVPL)  
          ! Put the result in ZSPDIVP.
          DO JN=KM,NSMAX
            ISE=KSTA+2*(JN-KM)
            ZSPDIVP(:,ISE:ISE+1)=ZSPDIVPL(:,JN,1:2)
          ENDDO
        ENDIF

      ELSE

        ! Case designed when no stretching :

        IF (LIMPF) THEN
          ! Solve complex pentadiagonal system
          CALL SIMPLICO(KM,NSMAX,NFLEVG,NFLEVG,ZALPHA(KM),&
           & ZDENIM(KM),ZFPLUS(KM),ZFMINUS(KM),SIVP,RLAPDI(0),&
           & ZBDT2,ZSDIVP,ZSPDIVP)
        ELSE
          ! Inversion of a diagonal matrix, provides Q*dver(t+dt).
          DO JLEV=1,NFLEVG
            DO JSP=KSTA,KEND
              ZSPDIVP(JLEV,JSP)=ZSDIVP(JLEV,JSP)&
               & /(1.0_JPRB-ZBDT2*SIVP(JLEV)*RLAPDI(NVALUE(JSP+IOFF)))  
            ENDDO
          ENDDO
        ENDIF

      ENDIF

      ! Vertical eigenmodes space --> current space.
      ! (multiply by matrix Q**(-1) stored in SIMO).
      ! Provides dver(t+dt).
      CALL MXMAOP(SIMO,1,NFLEVG,ZSPDIVP,1,NFLEVG,ZZSPSVDG,1,&
       & NFLEVG,NFLEVG,NFLEVG,ISPCOL)  

    ENDIF

    !*        2.5  Recover the other prognostic variables
    !              (successively D, T, "spd" and log(prehyds)).

    DO JSP=KSTA,KEND
      ZSP(JSP) = 0.0_JPRB
    ENDDO

    ! * Provides "Gamma d(t+dt)" in array ZWORK.
    CALL SIGAM(1,NFLEVG,ZWORK,ZZSPSVDG,ZSP,ISPCOL)

    ! * Provides
    !   " beta**2 (Delta t)**2 vnabla'**2 (- SITR Gamma + C**2) d(t+dt)
    !   + Dprim_star_star " if LIMPF=F, or
    !   " beta**2 (Delta t)**2 vnabla'**2 (- SITR Gamma + C**2) d(t+dt)
    !   + Dprim_star_star_star " if LIMPF=T
    !   in array ZSRHS.
    DO JLEV=1,NFLEVG
      DO JSP=KSTA,KEND
        IN=NVALUE(JSP+IOFF)
        ZSRHS(JLEV,JSP)=(ZZSPSVDG(JLEV,JSP)*RD*ZCC &
         & -ZWORK(JLEV,JSP))*SITR*ZBDT*ZBDT*&
         & RLAPDI(IN)+ZR1DPRIM(JLEV,JSP)  
      ENDDO
    ENDDO

    ! * Provides D'(t+dt).
    IF (LSIDG) THEN

      ! * Divide by the pentadiagonal operator
      !   [I - beta**2 (Delta t)**2 vnabla'**2 C**2 M**2]
      !   For KM>0 one works with the symmetric
      !   operator [vnabla'**(-2) - beta**2 (Delta t)**2 C**2 M**2]
      !   after having multiplied the RHS "ZSRHS" by "RLAPIN".
      !   Use ZSDIVPL and ZSPDIVPL as intermediate work arrays.

      ZSDIVPL(:,:,:)=0.0_JPRB
      ZSPDIVPL(:,:,:)=0.0_JPRB
      IF (KM > 0) THEN
        ! Multiply the RHS by grad'**(-2).
        DO JN=KM,NSMAX
          ISE=KSTA+2*(JN-KM)
          ZSDIVPL(:,JN,1:2)=ZSRHS(:,ISE:ISE+1)*RLAPIN(JN)
        ENDDO
        ! Inversion of a symmetric penta-diagonal matrix.
        CALL MXTURS(NSMAX+1-KM,NFLEVG,NFLEVG,II,&
         & SIHEGB(1,IS0+1,1),SIHEGB(1,IS0+1,2),SIHEGB(1,IS0+1,3),&
         & ZSDIVPL,ZSPDIVPL)  
      ELSE
        DO JN=KM,NSMAX
          ISE=KSTA+2*(JN-KM)
          ZSDIVPL(:,JN,1:2)=ZSRHS(:,ISE:ISE+1)
        ENDDO
        ! Inversion of a non-symmetric penta-diagonal matrix.
        CALL MXTURE(NSMAX+1-KM,NFLEVG,NFLEVG,II,-2,.TRUE.,&
         & SIHEGB(1,IS0+1,1),SIHEGB(1,IS0+1,2),SIHEGB(1,IS0+1,3),&
         & ZSDIVPL,ZSPDIVPL)  
        CALL MXTURE(NSMAX+1-KM,NFLEVG,NFLEVG,II,3,.FALSE.,&
         & SIHEGB(1,IS0+1,1),SIHEGB2(1,IS02+1,2),&
         & SIHEGB2(1,IS02+1,3),ZSDIVPL,ZSPDIVPL)  
      ENDIF
      ! Put the result in ZZSPDIVG.
      DO JN=KM,NSMAX
        ISE=KSTA+2*(JN-KM)
        ZZSPDIVG(:,ISE:ISE+1)=ZSPDIVPL(:,JN,1:2)
      ENDDO

    ELSE

      ! * Retrieve D'(t+dt).
      IF (LIMPF) THEN
        ! * Solve complex pentadiagonal system.
        CALL SIMPLICO(KM,NSMAX,NFLEVG,NFLEVG,ZALPHA(KM),&
         & ZDENIM(KM),ZFPLUS(KM),ZFMINUS(KM),ZSIVP2,RLAPDI(0),&
         & ZBDT2,ZSRHS,ZZSPDIVG)
      ELSE
        ! * Division by the diagonal operator
        !   [I - beta**2 (Delta t)**2 vnabla'**2 C**2 RSTRET**2].
        DO JLEV=1,NFLEVG
          DO JSP=KSTA,KEND
            IN=NVALUE(JSP+IOFF)
            ZZSPDIVG(JLEV,JSP)=ZSRHS(JLEV,JSP)/(1.0_JPRB-ZBTCM*RLAPDI(IN))
          ENDDO
        ENDDO
      ENDIF

    ENDIF

    ! * Provides Mbar**2 D'(t+dt).
    IF (LSIDG) THEN
      ! Multiply ZZSPDIVG by M**2 (penta-diagonal operator).
      ! Use ZSDIVPL and ZSPDIVPL as intermediate work arrays.
      ZSDIVPL(:,:,:)=0.0_JPRB
      ZSPDIVPL(:,:,:)=0.0_JPRB
      DO JN=KM,NSMAX
        ISE=KSTA+2*(JN-KM)
        ZSDIVPL(:,JN,1:2)=ZZSPDIVG(:,ISE:ISE+1)
      ENDDO
      CALL MXPTMA(NSMAX+1-KM,NFLEVG,NFLEVG,II,SCGMAP(IS0+1,1),&
       & SCGMAP(IS0+1,2),SCGMAP(IS0+1,3),&
       & SCGMAP(IS0+1,2),SCGMAP(IS0+1,3),&
       & ZSDIVPL,ZSPDIVPL)
      DO JN=KM,NSMAX
        ISE=KSTA+2*(JN-KM)
        ZWORK(:,ISE:ISE+1)=ZSPDIVPL(:,JN,1:2)
      ENDDO
    ELSE
      DO JLEV=1,NFLEVG
        DO JSP=KSTA,KEND
          ZWORK(JLEV,JSP)=ZZSPDIVG(JLEV,JSP)*RSTRET*RSTRET
        ENDDO
      ENDDO
    ENDIF

    !*       2.6  Increment vorticity for case LIMPF=T
    !             (identical to part 2.6 of SPCSI).

    IF (LIMPF) THEN
      DO JLEV=1,NFLEVG
        IF (KM == 0) THEN
          DO JN=2,NSMAX
            IRSP=KSTA+(JN-KM)*2
            ZZSPVORG(JLEV,IRSP)=ZZSPVORG(JLEV,IRSP)&
             & -ZDENIM(JN)*ZFMINUS(JN)*ZZSPDIVG(JLEV,IRSP-2)
          ENDDO
          DO JN=1,NSMAX-1
            IRSP=KSTA+(JN-KM)*2
            ZZSPVORG(JLEV,IRSP)=ZZSPVORG(JLEV,IRSP)&
             & -ZDENIM(JN)*ZFPLUS(JN)*ZZSPDIVG(JLEV,IRSP+2)
          ENDDO
        ELSE
          DO JN=KM+1,NSMAX
            IRSP=KSTA+(JN-KM)*2
            IMSP=IRSP+1
            ZZSPVORG(JLEV,IRSP)=ZZSPVORG(JLEV,IRSP)&
             & -ZDENIM(JN)*ZFMINUS(JN)*(ZZSPDIVG(JLEV,IRSP-2)&
             & -ZALPHA(JN)*ZZSPDIVG(JLEV,IMSP-2))
            ZZSPVORG(JLEV,IMSP)=ZZSPVORG(JLEV,IMSP)&
             & -ZDENIM(JN)*ZFMINUS(JN)*(ZZSPDIVG(JLEV,IMSP-2)&
             & +ZALPHA(JN)*ZZSPDIVG(JLEV,IRSP-2))
          ENDDO
          DO JN=KM,NSMAX-1
            IRSP=KSTA+(JN-KM)*2
            IMSP=IRSP+1
            ZZSPVORG(JLEV,IRSP)=ZZSPVORG(JLEV,IRSP)&
             & -ZDENIM(JN)*ZFPLUS(JN)*(ZZSPDIVG(JLEV,IRSP+2)&
             & -ZALPHA(JN)*ZZSPDIVG(JLEV,IMSP+2))
            ZZSPVORG(JLEV,IMSP)=ZZSPVORG(JLEV,IMSP)&
             & -ZDENIM(JN)*ZFPLUS(JN)*(ZZSPDIVG(JLEV,IMSP+2)&
             & +ZALPHA(JN)*ZZSPDIVG(JLEV,IRSP+2))
          ENDDO
        ENDIF
      ENDDO
    ENDIF

    !*       2.7  Savings.

    IF (I_NITERHELM > 0) THEN
      IF (JITER == 0) THEN
        ! Save D'(t+dt,iter=0), Mbar**2 D'(t+dt,iter=0),
        !  zeta'(t+dt,iter=0), dver(t+dt,iter=0).
        ZZSPDIV0G(1:NFLEVG,KSTA:KEND)=ZZSPDIVG(1:NFLEVG,KSTA:KEND)
        ZZSPGDIV0G(1:NFLEVG,KSTA:KEND)=ZWORK(1:NFLEVG,KSTA:KEND)
        IF (LIMPF) ZZSPVOR0G(1:NFLEVG,KSTA:KEND)=ZZSPVORG(1:NFLEVG,KSTA:KEND)
        ZZSPSVD0G(1:NFLEVG,KSTA:KEND)=ZZSPSVDG(1:NFLEVG,KSTA:KEND)
      ENDIF

      ! Save Mbar**2 D'(t+dt,jiter) in ZZSPGDIVG
      IF (JITER == 0) THEN
        ZZSPGDIVG(1:NFLEVG,KSTA:KEND)=ZWORK(1:NFLEVG,KSTA:KEND)
      ELSE
        DO JLEV=1,NFLEVG
          DO JSP=KSTA,KEND
            ZZSPGDIVG(JLEV,JSP)=ZWORK(JLEV,JSP)+ZZSPGDIV0G(JLEV,JSP)
          ENDDO
        ENDDO
      ENDIF
    ENDIF

  ENDDO ! End loop on JITER

  !*       2.8  Add the increment and recover X(t+dt)
  !             for X=D',Mbar**2 D',zeta',dver, and deallocate.

  IF (I_NITERHELM > 0) THEN

    DO JLEV=1,NFLEVG
      DO JSP=KSTA,KEND
        ZZSPDIVG(JLEV,JSP)=ZZSPDIVG(JLEV,JSP)+ZZSPDIV0G(JLEV,JSP)
        ZWORK(JLEV,JSP)=ZWORK(JLEV,JSP)+ZZSPGDIV0G(JLEV,JSP)
        ZZSPSVDG(JLEV,JSP)=ZZSPSVDG(JLEV,JSP)+ZZSPSVD0G(JLEV,JSP)
      ENDDO
    ENDDO
    IF (LIMPF) THEN
      DO JLEV=1,NFLEVG
        DO JSP=KSTA,KEND
          ZZSPVORG(JLEV,JSP)=ZZSPVORG(JLEV,JSP)+ZZSPVOR0G(JLEV,JSP)
        ENDDO
      ENDDO
    ENDIF

    DEALLOCATE(ZZSPGDIVG)
    DEALLOCATE(ZZSPDIV0G)
    DEALLOCATE(ZZSPGDIV0G)
    DEALLOCATE(ZZSPVOR0G)
    DEALLOCATE(ZZSPSVD0G)

  ENDIF

  !*       2.9  Increment T, log(prehyds) and spd.

  ! * Provides some intermediate quantities allowing to compute
  !   T(t+dt) (in ZST), log(prehyds)(t+dt) (in ZSP), spd(t+dt) (in ZSNHP).
  CALL SIPTP(1,NFLEVG,ZWORK,ZZSPSVDG,ZSNHP,ZST,ZSP,ISPCOL)

  ! * Provides T(t+dt) (in ZZSPTG) and spd(t+dt) (in ZZSPSPDG).
  DO JLEV=1,NFLEVG
    DO JSP=KSTA,KEND
      ZZSPTG(JLEV,JSP)=ZZSPTG(JLEV,JSP)-ZBDT*ZST(JLEV,JSP)
      ZZSPSPDG(JLEV,JSP)=ZZSPSPDG(JLEV,JSP)-ZBDT*ZSNHP(JLEV,JSP)
    ENDDO
  ENDDO

  ! * Provides log(prehyds)(t+dt) (in ZZSPSPG).
  DO JSP=KSTA,KEND
    ZZSPSPG(JSP)=ZZSPSPG(JSP)-ZBDT*ZSP(JSP)
  ENDDO

ENDIF

!     ------------------------------------------------------------------

!*       3.    MEMORY TRANSFER.
!              ----------------

IF (LL3D) THEN
  IF (LIMPF) PSPVORG(1:NFLEVG,KSTA:KEND)=&
   & ZZSPVORG(1:NFLEVG,KSTA:KEND)  
  PSPDIVG(1:NFLEVG,KSTA:KEND)=ZZSPDIVG(1:NFLEVG,KSTA:KEND)
  PSPTG  (1:NFLEVG,KSTA:KEND)=ZZSPTG  (1:NFLEVG,KSTA:KEND)
  PSPSPG (         KSTA:KEND)=ZZSPSPG (         KSTA:KEND)
  PSPSPDG(1:NFLEVG,KSTA:KEND)=ZZSPSPDG(1:NFLEVG,KSTA:KEND)
  PSPSVDG(1:NFLEVG,KSTA:KEND)=ZZSPSVDG(1:NFLEVG,KSTA:KEND)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPNHSI',1,ZHOOK_HANDLE)
END SUBROUTINE SPNHSI
