SUBROUTINE CSEAICE

!**** *CSEAICE*  - Computes the sea ice concentration field for ARPEGE

!     Purpose.
!     -------
!       Interpolates the sea ice concentration field from satellite data to
!       the ARPEGE grid (defined by file if command line, or by namelist).

!**   Interface.
!     ---------
!       CALL CSEAICE

!     Method.
!     ------
!       After definition of the output ARPEGE file, the satellite data are
!       read and controlled.
!       They are interpolated on the model gaussian grid, which has been defined
!       by the frame of the climatological file.
!       A quality control, by comparison with a reference field (the previous
!       day field in general), is applied on the computed field (maximum daily
!       variation authorized).
!       Where there is no observation, the field is set to the climatological
!       value (according to the SST climatology).
!       At the end, the sea ice field is written on the output file.

!     Externals.
!     ---------
!          INTICE
!          UPDCAL   POSNAM + FA and LFI routines

!     Author.
!     ------
!        Francoise TAILLEFER  *METEO-FRANCE*

!     Modifications.
!     -------------
!        ORIGINAL :  2003-08-10
!        S. Petitcol : 2003-09-15 seaice.F90 renamed in cseaice.F90
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULNAM   ,NULCL1   ,NULCL2   , &
 & NINISH  
USE YOMDIM   , ONLY : NDGLG    ,NDLON    ,NSMAX    , &
 & NFLEVG   ,NDGSAG   ,NDGENG  
USE YOMGEM   , ONLY :    NGPTOTG  ,NLOENG   ,NMENG    , &
 & VALH     ,VBH      ,NSTTYP   ,NHTYP    ,RMUCEN   , &
 & RLOCEN   ,RSTRET   ,VP00  
USE YOMGC    , ONLY : GELAT
USE YOMCT0   , ONLY : CNMEXP   ,NQUAD    ,LELAM    
USE YOMCST   , ONLY : RPI
USE YOMRIP   , ONLY : NINDAT
USE YOMOP    , ONLY : CNMCA
USE YOMMP    , ONLY : MYPROC
USE YOMPHY1  , ONLY : TMERGL
USE YOMICE   , ONLY : RIPORT   ,RIALPHA  ,NREAOBS  ,RSATLON  ,RSATLAT  , &
 & RSATICE  ,RFICE    ,ROBMASK  

IMPLICIT NONE

REAL(KIND=JPRB) :: ZGAUSS(NDGSAG:NDGENG)
REAL(KIND=JPRB) :: ZCQ(NGPTOTG),ZCLI(NGPTOTG),ZOLDM(NGPTOTG),ZDATO(3)
REAL(KIND=JPRB) :: ZICE(NGPTOTG),ZNEWM(NGPTOTG)
REAL(KIND=JPRB) :: ZRAD, ZV, ZBID, ZLAT, ZLON, ZLIMS, ZLIMI
INTEGER(KIND=JPIM) :: IDATF(11), ILMO(12), IDALIM1, IDAOOBS, IREAOBS
INTEGER(KIND=JPIM) :: IMASK(NGPTOTG), IABER(NGPTOTG)
INTEGER(KIND=JPIM) :: IAA, IMM, IJJ, IA1, IM1, IJ1, ICOD, INC, JGL, IA2, IM2, IJ2
INTEGER(KIND=JPIM) :: INIMES, INBARP, INBARI, IREP, IUNITO, IDAT, IRESO, IBID, IINF
CHARACTER (LEN = 16) ::  CLIDEN,CLNOMF, CL_CBID*8
LOGICAL :: LLF

!  namelist NAMICE for this configuration only (932):
!  -------------------------------------------
!    NDAYDAT   date of the analysis
!    NMXOBSAT  max. number of satellite observations (for arrays size)
!    QCSLIM    ] seuils de SST pour detecter les [ --> eau libre  (conc. = 0%)
!    QCILIM    ]   valeurs aberrantes (en C)    [ --> glace  (conc. = 100%)
!    QCSEU     max. variation of the concentration field from day to day
!               (no control if negative)
!    LHIST     if .TRUE. and no previous sea ice file --> ABORT
!    NLIMDAT   max. date interval between analysis and observations (number
!                of days)  (clim otherwise)
!    RIPORT    length scale for the weight function            ]
!    RIALPHA   coef. of the "stretched length scale" function  ] module YOMICE

REAL(KIND=JPRB) :: QCSEU,QCSLIM,QCILIM
INTEGER(KIND=JPIM) :: NMXOBSAT,NDAYDAT,NLIMDAT
LOGICAL :: LHIST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

NAMELIST / NAMICE / NMXOBSAT,NDAYDAT,NLIMDAT,QCSEU,LHIST,RIPORT,RIALPHA, &
 & QCSLIM,QCILIM  

#include "abor1.intfb.h"
#include "cchien.intfb.h"
#include "chien.intfb.h"
#include "intice.intfb.h"
#include "posnam.intfb.h"
#include "updcal.intfb.h"

#include "fcttim.h"

!**---------------------------------------------------------------------
!**      1.    START OF ROUTINE.
!**            ----------------

IF (LHOOK) CALL DR_HOOK('CSEAICE',0,ZHOOK_HANDLE)
WRITE(NULOUT,FMT='(/,''ENTRANCE IN CSEAICE  ... '')')

!**      1.1  Sets implicit default values

NMXOBSAT=500000
NLIMDAT=30
LHIST=.FALSE.
QCSLIM=1.0_JPRB
QCILIM=-12.0_JPRB
QCSEU=20.0_JPRB
RIPORT=200000._JPRB
RIALPHA=0.08_JPRB

NDAYDAT=NINDAT

!**      1.2  Modifies default values --> namelist

CALL POSNAM(NULNAM,'NAMICE')
READ(NULNAM,NAMICE)

WRITE(NULOUT,FMT='(/,'' namelist NAMICE :'',/,''    NMXOBSAT '',I7, &
 & '' / NLIMDAT '',I2,'' / QCSEU '',F5.1,'' / LHIST '',L2, &
 & '' / RIPORT'',F8.0,'' / RIALPHA '', F4.2,/)') &
 & NMXOBSAT,NLIMDAT,QCSEU,LHIST,RIPORT,RIALPHA  
WRITE(NULOUT,FMT='(''  QCSLIM '',F5.1,'' / QCILIM '',F5.1,/)') QCSLIM,QCILIM

ZLIMS=273.15_JPRB+QCSLIM
ZLIMI=273.15_JPRB+QCILIM

NINDAT=NDAYDAT

!**      1.3  Various initialisations

INIMES=1
IREAOBS=0
IUNITO=16
ICOD=1
ZRAD=RPI/180._JPRB

!**      1.4  Presumed data date computation (J-1/J ARPEGE)

IAA=NCCAA(NINDAT)
IMM=NMM(NINDAT)
IJJ=NDD(NINDAT)
INC=-1
CALL UPDCAL(IJJ,IMM,IAA,INC,IJ1,IM1,IA1,ILMO,NULOUT)
IDALIM1=IA1*10000+IM1*100+IJ1
WRITE(NULOUT,'(/,''  date of the analysis : '',I8)') NINDAT
WRITE(NULOUT,'(''  expected date of the data : '',I8)') IDALIM1

!**      1.5  Arrays definition

WRITE(NULOUT,*) '   '
ALLOCATE(RSATLON(NMXOBSAT))
WRITE(NULOUT,2990) ' RSATLON  ',SIZE(RSATLON),SHAPE(RSATLON)
ALLOCATE(RSATLAT(NMXOBSAT))
WRITE(NULOUT,2990) ' RSATLAT  ',SIZE(RSATLAT),SHAPE(RSATLAT)
ALLOCATE(RSATICE(NMXOBSAT))
WRITE(NULOUT,2990) ' RSATICE  ',SIZE(RSATICE),SHAPE(RSATICE)
WRITE(NULOUT,*) '   '

!**---------------------------------------------------------------------
!**      2.    READING THE CLIMATOLOGICAL FILE.
!**            -------------------------------

ALLOCATE(RFICE(NGPTOTG))
CLNOMF='ICMSH'//CNMEXP(1:4)//'INIT'
CALL FAITOU(IREP,NINISH,.TRUE.,CLNOMF,'UNKNOWN',.TRUE., &
 & .TRUE.,1,1,INBARI,'CADRE LECTURE   ')  
CALL LFIMST(IREP,NINISH,.FALSE.)
CALL FACILE(IREP,NINISH,'SURF',0,'TEMPERATURE ',ZCQ,.FALSE.)
CALL FACILE(IREP,NINISH,'SURF',0,'IND.TERREMER',RFICE,.FALSE.)
CALL FAIRME(IREP,NINISH,'UNKNOWN')

ZBID=273.15_JPRB-1.8_JPRB

! ZCLI is the "climatological" ice field
! IABER est le masque de test des valeurs aberrantes
IABER(:)=0
DO JGL = 1,NGPTOTG
  IF (RFICE(JGL) > 0.5_JPRB) THEN
    ZCLI(JGL)=0.0_JPRB
  ELSE
    IF (ZCQ(JGL) <= TMERGL) THEN
      ZCLI(JGL)=100._JPRB
    ELSEIF (ZCQ(JGL) > ZBID) THEN
      ZCLI(JGL)=0.0_JPRB
    ELSE
      ZCLI(JGL)=60._JPRB
    ENDIF
    IF (ZCQ(JGL) >= ZLIMS) THEN
      IABER(JGL)=1
    ELSEIF (ZCQ(JGL) <= ZLIMI .AND. GELAT(JGL) < 0.0_JPRB) THEN
      IABER(JGL)=2
    ENDIF
  ENDIF
ENDDO

!**---------------------------------------------------------------------
!**      3.    READING THE PREVIOUS DAY FILE.
!**            -----------------------------

CLNOMF='ICMSH'//CNMEXP(1:4)//'SEAICE'
INQUIRE(FILE=CLNOMF,EXIST=LLF)
IF (.NOT.LLF) THEN
  WRITE(NULOUT,'('' No input ice field file found ! '')')
  IF (LHIST) THEN
    CALL ABOR1('LHIST set to TRUE so ABORT')
  ELSE
    WRITE(NULOUT,'(''    --> no control will be applied on the data ... '')')
    IDAOOBS=-1
  ENDIF
ELSE
  WRITE(NULOUT,'('' Opening previous day file ... '')')
  CALL FAITOU(IREP,NULCL1,.TRUE.,CLNOMF,'UNKNOWN',.TRUE., &
   & .TRUE.,INIMES,INBARP,INBARI,'CADRE REFERENCE ')  
  CALL LFIMST(IREP,NULCL1,.FALSE.)
  IF (.NOT.LELAM) THEN
    CALL CHIEN('CADRE REFERENCE ',NSTTYP,RMUCEN,RLOCEN,RSTRET, &
     & NSMAX,NDGLG,NDLON,NLOENG,NMENG,ZGAUSS,NHTYP,NFLEVG, &
     & VP00,VALH,VBH,NQUAD,0,NDGSAG,NDGENG, &
     & 1.0E-7_JPRB,LLF,NULOUT)  
  ELSE
    IINF=-1
    CALL CCHIEN('CADRE REFERENCE ',NULCL1,IINF)
  ENDIF
  CALL FADIES(IREP,NULCL1,IDATF)
  WRITE(NULOUT,*) '  date of the input field : ',IDATF(1),IDATF(2),IDATF(3)
  CALL FALAIS(IREP,NULCL1,'DATA.DATE',ZDATO,3)
  IDAOOBS=NINT(ZDATO(1))*10000+NINT(ZDATO(2))*100+NINT(ZDATO(3))
  WRITE(NULOUT,*) '  date of the last used observations : ',IDAOOBS
  CALL FACILE(IREP,NULCL1,'SURF',0,'SEA.ICE.CONC',ZCQ,.FALSE.)
  CALL FACILE(IREP,NULCL1,'SURF',0,'ICE.OBS.DATE',ZOLDM,.FALSE.)
  CALL FAIRME(IREP,NULCL1,'KEEP')
ENDIF

!**---------------------------------------------------------------------
!**      4.    READING SATELLITE DATA.
!**            ----------------------

INQUIRE(FILE='ice_concent',EXIST=LLF)
IF (.NOT.LLF) GOTO 401
OPEN(UNIT=IUNITO,FILE='ice_concent',FORM='FORMATTED')
WRITE(NULOUT,'(/,'' Reading the satellite data ... '')')

!**      4.1  Date control

READ(IUNITO,*) IDAT,IRESO

IF (IDAT > IDALIM1) THEN
  WRITE(NULOUT,*) '   strange data date : ',IDAT,' previous day date : ',IDALIM1
  CLOSE(IUNITO)
  GOTO 605
ENDIF
IF (IDAOOBS > 0) THEN
  IF (IDAT <= IDAOOBS) THEN
    WRITE(NULOUT,'(''  the satellite data are obsolete ! '', &
     & ''    --> no modification of the previous sea ice field with obs'')')  
    CLOSE(IUNITO)
    GOTO 605
  ENDIF
ENDIF

!**      4.2  Data arrays filling

DO
  READ(IUNITO,*,END=402) IBID,IBID,IBID,ZLAT,ZLON,&
   & CL_CBID,IBID,IBID,ZBID,IBID,IBID,IBID,&
   & IBID,ZBID,ZBID,ZV,IBID  
  IREAOBS=IREAOBS+1
  IF ( ZLON < 0.0_JPRB ) ZLON=ZLON+360.0_JPRB
  RSATLON(IREAOBS)=ZLON*ZRAD
  RSATLAT(IREAOBS)=ZLAT*ZRAD
  RSATICE(IREAOBS)=ZV
ENDDO
402 CONTINUE
CLOSE(IUNITO)
NREAOBS=IREAOBS

IF (IREAOBS > NMXOBSAT) CALL ABOR1(' too much satellite data : IREAOBS > NMXOBSAT ')
401 CONTINUE
IF (IREAOBS == 0) THEN
  WRITE(NULOUT,'(''  no satellite data available '', &
   & ''    --> no modification of the previous sea ice field with obs'')')  
  GOTO 605
ENDIF

ICOD=2
WRITE(NULOUT,'(''      '',I6,'' data found '')') NREAOBS

!**---------------------------------------------------------------------
!**      5.    DATA INTERPOLATION.
!**            ------------------

CALL INTICE

!**---------------------------------------------------------------------
!**      6.    FINAL FIELD ESTIMATION.
!**            ----------------------

605 CONTINUE

IF (MYPROC == 1) THEN
  WRITE(NULOUT,'(/,'' Final ice field computation '')')
  IF (IDAOOBS > 0) THEN
    INC=-NLIMDAT
    CALL UPDCAL(IJ1,IM1,IA1,INC,IJ2,IM2,IA2,ILMO,NULOUT)
    IDALIM1=IA2*10000+IM2*100+IJ2
    IMASK(:)=NINT(ZOLDM(:))
    DO JGL = 1,NGPTOTG
      IF (IMASK(JGL) < IDALIM1) THEN
        IMASK(JGL)=0
        ZCQ(JGL)=ZCLI(JGL)
      ENDIF
    ENDDO
    IF (ICOD == 1) THEN
      DO JGL = 1,NGPTOTG
        ZICE(JGL)=ZCQ(JGL)
        ZNEWM(JGL)=REAL(IMASK(JGL),JPRB)
      ENDDO
    ELSE
      DO JGL = 1,NGPTOTG
! controle des valeurs aberrantes
        IF (IABER(JGL) == 1) THEN
          ZICE(JGL)=0.0_JPRB
          ZNEWM(JGL)=0.0_JPRB
        ELSEIF (IABER(JGL) == 2) THEN
          ZICE(JGL)=100._JPRB
          ZNEWM(JGL)=0.0_JPRB
        ELSE
          IF (ROBMASK(JGL) > 0.5_JPRB) THEN
            IF (IMASK(JGL) > 0) THEN
              ZV=RFICE(JGL)-ZCQ(JGL)
              IF (QCSEU > 0 .AND. ABS(ZV) > QCSEU) THEN
                ZICE(JGL)=ZCQ(JGL)+SIGN(QCSEU,ZV)
              ELSE
                ZICE(JGL)=RFICE(JGL)
              ENDIF
            ELSE
              ZICE(JGL)=RFICE(JGL)
            ENDIF
            ZNEWM(JGL)=REAL(IDAT,JPRB)
          ELSE
            ZICE(JGL)=ZCQ(JGL)
            ZNEWM(JGL)=REAL(IMASK(JGL),JPRB)
          ENDIF
        ENDIF
      ENDDO
    ENDIF
  ELSE
    ZICE(:)=ZCLI(:)
    ZNEWM(:)=0.0_JPRB
    IF (ICOD == 2) THEN
      DO JGL = 1,NGPTOTG
! controle des valeurs aberrantes
        IF (IABER(JGL) == 1) THEN
          ZICE(JGL)=0.0_JPRB
        ELSEIF (IABER(JGL) == 2) THEN
          ZICE(JGL)=100._JPRB
        ELSE
          IF (ROBMASK(JGL) > 0.5_JPRB) THEN
            ZICE(JGL)=RFICE(JGL)
            ZNEWM(JGL)=REAL(IDAT,JPRB)
          ENDIF
        ENDIF
      ENDDO
    ENDIF
  ENDIF
ENDIF

!**---------------------------------------------------------------------
!**      7.    OUTPUT FILE WRITING.
!**            -------------------

WRITE(NULOUT,'(/,'' Creating new SEA ICE file '')')

IF (MYPROC == 1) THEN
  CLIDEN='SEA ICE CONCENTR'
  CLNOMF='ICMSH'//CNMEXP(1:4)//'SEAICEF'
  INBARP=20
  CALL FAITOU(IREP,NULCL2,.TRUE.,CLNOMF,'NEW',.TRUE.,.TRUE., &
   & INIMES,INBARP,INBARI,CNMCA)  
  CALL LFIMST(IREP,NULCL2,.FALSE.)
  CALL FAUTIF(IREP,NULCL2,CLIDEN)
  IDATF(1)=IAA
  IDATF(2)=IMM
  IDATF(3)=IJJ
  IDATF(4)=0
  IDATF(5)=0
  IDATF(6)=1
  IDATF(7)=0
  IDATF(8)=0
  IDATF(9)=0
  IDATF(10)=0
  IDATF(11)=0
  IF (ICOD == 2) THEN
    ZDATO(1)=NCCAA(IDAT)
    ZDATO(2)=NMM(IDAT)
    ZDATO(3)=NDD(IDAT)
  ELSE
    ZV=MAXVAL(ZNEWM)
    IF (ZV < 1.0_JPRB) ZDATO(:)=0
  ENDIF
  CALL FANDAR(IREP,NULCL2,IDATF)
  CALL FAISAN(IREP,NULCL2,'DATA.DATE',ZDATO,3)
  CALL FAIENC(IREP,NULCL2,'SURF',0,'SEA.ICE.CONC',ZICE,.FALSE.)
  CALL FAIENC(IREP,NULCL2,'SURF',0,'ICE.OBS.DATE',ZNEWM,.FALSE.)
  CALL FAIRME(IREP,NULCL2,'KEEP')
ENDIF

DEALLOCATE(RFICE)
IF (ALLOCATED(ROBMASK)) DEALLOCATE(ROBMASK)

!**---------------------------------------------------------------------
WRITE(NULOUT,FMT='(/,''END OF CSEAICE  ! '',/)')
2990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
IF (LHOOK) CALL DR_HOOK('CSEAICE',1,ZHOOK_HANDLE)
END SUBROUTINE CSEAICE
