SUBROUTINE GANISO(KPDG,P1,P2,P3,PGAMA,PTHETA)

!     PURPOSE
!     -------

!     This routine calculates the anisotropy coefficient and direction of the
!     principal axis of topography in GAUSS grid square,which are used in
!     parametrisation of the gravity wave drag.

!**   INTERFACE

!     CALL GANISO(KPDG,P1,P2,P3,PGAMA,PTHETA)

!         KPDG-Number of points of grid Gauss
!         P1,P2,P3-Input field with the components of topographic gradient
!                 correlation tensor
!         PGAMA-Output field-the anisotropy coefficient of orography in GAUSS
!               grid square(square value)
!         PTHETA-Output field-angle from x-axis to the principal axis of oro-
!                graphy in GAUSS grid square(radians)

!     METHOD
!     ------

!     Using the total topographic gradient correlation tensor(its eigen
!     values and eigen vectors )  the anisotropy coefficient and the direction
!     of principal axis of topography are calculated .
!     When the tensor is isotropic, the anisotropy is 1 and the direction is 0.
!     isotropy.

!     EXTERNAL
!     --------

!     NONE

!     AUTHORS
!     -------

!     E. Cordoneanu and M. Deque 92-02-15

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     D. Giard   00-09-15  message passing

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RPI
USE YOMDIM   , ONLY : NDGLG
USE YOMGEM   , ONLY : NLOENG   ,NSTAGP   ,NSTTYP
USE YOMGC    , ONLY : GNORDL   ,GNORDM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPDG 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P1(KPDG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P2(KPDG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P3(KPDG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGAMA(KPDG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTHETA(KPDG) 
INTEGER(KIND=JPIM) :: IADR, ID, IPT, J, JI, JJ

REAL(KIND=JPRB) :: Z, ZGAMA, ZPHI, ZZ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GANISO',0,ZHOOK_HANDLE)
ID=1

DO J=1,KPDG
  Z=P1(J)-P2(J)
  ZZ=SQRT(Z**2+4*P3(J)**2)
  IF (P3(J) == 0.0_JPRB) THEN
    IF (Z >= 0.0_JPRB) THEN
      PTHETA(J)=0.0_JPRB
    ELSE
      PTHETA(J)=RPI*0.5_JPRB
    ENDIF
  ELSE
    PTHETA(J)=ATAN((-Z+ZZ)/P3(J)*0.5_JPRB)
  ENDIF

  ZGAMA=P1(J)+P2(J)+ZZ
  IF (ZGAMA == 0.0_JPRB) THEN
    PGAMA(J)=1.0_JPRB
  ELSE
    PGAMA(J)=MIN(1.0_JPRB,MAX(0.0_JPRB,(P1(J)+P2(J)-ZZ)/ZGAMA))
  ENDIF
ENDDO

IF (NSTTYP == 2) THEN

!     Orientation with respect to the model grid

  IPT=0
  DO JJ=1,NDGLG
    DO JI=1,NLOENG(JJ)
      IPT=IPT+1
      IADR=JI+NSTAGP(JJ)-ID
      IF (GNORDM(IADR) == 0.0_JPRB) THEN
        ZPHI=RPI*0.5_JPRB
      ELSE
        ZPHI=-ATAN(GNORDL(IADR)/GNORDM(IADR))
      ENDIF
      PTHETA(IPT)=PTHETA(IPT)+ZPHI
    ENDDO
  ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('GANISO',1,ZHOOK_HANDLE)
END SUBROUTINE GANISO
