SUBROUTINE INCLIB

!**** *INCLIB*

!     PURPOSE.
!     --------

!      Interpolation of the fields used in the linear photochemical
!      parameterization of ozone

!**   INTERFACE.
!     ----------

!     CALL INCLIB

!     METHOD.
!     -------

!      The fields are read on units 11,12,...,22 (one unit per month).
!      They are interpolated on the gaussian grid.
!      Then they are interpolated linearly in pressure
!      Finally they are written on Arpege file units 41,42,...,52

!     EXTERNALS.
!     ----------

!     GEO923
!     INTER8
!     INTRV2
!     CHIEN
!     ABOR1
!     FA-LFI package (FAITOU,FAIENC,LFILAF,FAIRME)

!     AUTHORS.
!     --------

!         M. DEQUE  JAN 91 .

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!        modified by M. Deque 91-06-01 (mixing ratio in mass instead of volume)
!        Modified by R. EL Khatib : 94-03-08 comdeck YOMCLB
!        modified by M. Deque 95-10-30 (new input fields)
!        modified by M. Deque 96-05-02 (new input fields)
!        modified by M. Deque 98-02-09 (close fields)
!        modified by D. Giard 98-09-17 (option for spectral fields)
!        Modified 98-08-10 by K. YESSAD: partial removal of LRPOLE option.
!         - LRPOLE has been removed from YOMCT0 and 'CALL CHIEN' 
!           (assumed to be F).
!         - Call to POLADD and write poles on FA files is still made.
!        modified by D. Giard 98-12-08 (YOMCLI instead of PARAMETER)
!        modified by D. Giard 00-09-12 storing as spectral fields suppressed
!        modified by D. Giard 00-09-15 message passing

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCLI   , ONLY : NPINT    ,NDATX    ,NDATY
USE YOMDIM   , ONLY : NDGLG    ,NDGSAG   ,NDGENG   ,NDLON    ,&
 & NFLEVG   ,NSMAX  
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NQUAD
USE YOMLEG   , ONLY : RMU
USE YOMGEM   , ONLY : NLOENG   ,NMENG    ,VALH     ,VBH      ,&
 & NSTTYP   ,NHTYP    ,RMUCEN   ,RLOCEN   ,RSTRET   ,VP00  
USE YOMSTA   , ONLY : STPRE
USE YOMCST   , ONLY : RNAVO    ,RMD      ,RMO3

IMPLICIT NONE

!*    Source grid description
!       NDATY/IPLAT : Number of latitudes (must be even)
!       NDATX/IPLON : Number of longitudes
!       JPNIV/IPNIV : Number of pressure levels
!       JPNCH/IPNCH : Number of fields
!*    Successive versions :
!                NDATY=31,NDATX=2,JPNIV=32,JPNCH=7
!                NDATY=36,NDATX=2,JPNIV=40,JPNCH=7
!                NDATY=64,NDATX=2,JPNIV=26,JPNCH=7

INTEGER(KIND=JPIM) :: JPBX
INTEGER(KIND=JPIM) :: JPBY
INTEGER(KIND=JPIM) :: JPLEV
INTEGER(KIND=JPIM) :: JPNCH
INTEGER(KIND=JPIM) :: JPNIV
INTEGER(KIND=JPIM) :: JPNMOI

PARAMETER(JPNIV=26,JPNCH=7,JPBX=2,JPBY=2,JPNMOI=12,JPLEV=100)

REAL(KIND=JPRB),ALLOCATABLE :: ZFLD (:,:,:),ZPD(:,:,:),ZAUX(:,:)
REAL(KIND=JPRB) :: ZC(NDGLG*NDLON,JPNIV),ZP(NDGLG*NDLON,JPNIV),ZS(NDGLG*NDLON,NFLEVG)
REAL(KIND=JPRB) :: ZBO(NDGLG+1),ZLONG(NDGLG),ZMU(NDGLG*NDLON),ZSLA(NDGLG*NPINT)&
 & ,ZSLO(NDLON*NPINT,NDGLG),ZCLO(NDLON*NPINT,NDGLG)  
REAL(KIND=JPRB) :: ZSCALE(JPNCH)

CHARACTER :: CLSUFF(JPNCH)*20,CLNOMC*16,CLNOMF*14

INTEGER(KIND=JPIM) :: IARI, IARP, ICH, ICO, IDATX, IDATY, IINF, &
 & ILEND, ILENE, ILENT, IMES, &
 & INIQ, INJQ, INSOUR, INUM, IPLAT, IPLON, &
 & IPNCH, IPNIV, IREP, IZ, JC, JI, JJ, JL, JMOIS  

LOGICAL :: LLCOSP, LLIMST, LLPOLE

REAL(KIND=JPRB) :: ZEPS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "chien.intfb.h"
#include "geo923.intfb.h"
#include "inter8.intfb.h"
#include "intrv2.intfb.h"

!     ------------------------------------------------------------------
!*
!     1. SET INITIAL VALUES.
!        -------------------

IF (LHOOK) CALL DR_HOOK('INCLIB',0,ZHOOK_HANDLE)
IF (NFLEVG > JPLEV) CALL ABOR1(' INCLIB : NFLEVG TOO HIGH !')
ZEPS=1.E-6_JPRB

!     Grids.

ICO=NPINT*NPINT
INJQ=NDGLG*NPINT
INIQ=NDLON*NPINT
ILENT=NDGLG*NDLON
IDATX=NDATX+2*JPBX
IDATY=NDATY+2*JPBY
ILEND=NDATX*NDATY

ILENE=0
CALL GEO923(NPINT,ILENE,ZBO,ZLONG,ZMU,ZSLA,ZSLO,ZCLO)

!     Arpege fields identifiers and scaling.

DO JC=1,JPNCH
  WRITE(UNIT=CLSUFF(JC),FMT='(''OZONE.A'',I1)') JC
  ZSCALE(JC)=1.0_JPRB
  IF (JC == 1.OR.JC == 2.OR.JC == 5) ZSCALE(JC)=RMO3/RMD
  IF (JC == 6)                       ZSCALE(JC)=RMO3*10._JPRB/RNAVO
  IF (JC == 7)                       ZSCALE(JC)=.1_JPRB*RNAVO/RMD
ENDDO

LLCOSP=.FALSE.

!     ARPEGE files.

INUM=3
IREP=0
IARI=0
IARP=NFLEVG*JPNCH
CLNOMC='Const.Clim.Surfa'
LLIMST=.TRUE.
IMES=1

!     Loop on the months.

DO JMOIS=1,JPNMOI

!     ------------------------------------------------------------------
!*
!     2. OPEN FILES.
!        -----------

!     Open Arpege file.
  WRITE (UNIT=CLNOMF,FMT='(''Const.Clim.'',I2.2)') JMOIS
  CALL FAITOU(IREP,INUM,.TRUE.,CLNOMF,'OLD',.TRUE.,LLIMST,&
   & IMES,IARP,IARI,CLNOMC)  

!     Check geometry.
  IINF=0
  LLPOLE=.TRUE.
  CALL CHIEN(CLNOMC,NSTTYP,RMUCEN,RLOCEN,RSTRET,NSMAX,&
   & NDGLG,NDLON,NLOENG,NMENG,RMU,NHTYP,NFLEVG,VP00,VALH,VBH,&
   & NQUAD,IINF,NDGSAG,NDGENG,ZEPS,LLPOLE,NULOUT)  
  IF (LLPOLE) CALL ABOR1(' CLIM. FILES MUST NOT HAVE POLES !')

!     Open and check Source file.

  INSOUR=10+JMOIS
  OPEN(INSOUR,FORM='UNFORMATTED')
  READ(INSOUR) IPNCH
  WRITE(NULOUT,FMT='('' IPNCH '',I3)') IPNCH
  IF (IPNCH > JPNCH) CALL ABOR1('INCLIB: TOO MANY FIELDS !')

!     Loop and the fields.

  DO JC=1,IPNCH

!     ------------------------------------------------------------------
!*
!     3. READ FIELDS.
!        ------------

!     Read field characteristics.
    READ(INSOUR) ICH,IPNIV,IPLAT,IPLON

    WRITE(NULOUT,FMT=&
     & '('' ICH'',I3,'' IPNIV'',I3,'' IPLAT'',I4,'' IPLON'',I4)')&
     & ICH,IPNIV,IPLAT,IPLON  

    IF (IPNIV /= JPNIV.OR.IPLAT /= NDATY.OR.IPLON /= NDATX) THEN
      WRITE(NULOUT,FMT=&
       & '('' ICH'',I3,'' JPNIV'',I3,'' NDATY'',I4,'' NDATX'',I4)')&
       & ICH,JPNIV,NDATY,NDATX  
      CALL ABOR1('INCLIB: BAD SIZE OF INPUT FIELDS !')
    ENDIF

!     Read pressure levels (the same for all fields and months).
    ALLOCATE ( ZAUX (NDATY,JPNIV) )
    READ (INSOUR) ZAUX

    IF (JC == 1.AND.JMOIS == 1) THEN

      WRITE(NULOUT,FMT='('' PRESSURE LEVELS'')')
      WRITE(NULOUT,FMT='(10F8.0)') (ZAUX(1,JL),JL=1,JPNIV)

      ALLOCATE (  ZPD (IDATX,IDATY,JPNIV) )

      DO JL=1,JPNIV
        DO JJ=1+JPBY,NDATY+JPBY
          DO JI=1+JPBX,NDATX+JPBX
            IZ=JJ-JPBY
            ZPD(JI,JJ,JL)=ZAUX(IZ,JL)
          ENDDO
        ENDDO
      ENDDO

    ENDIF

    DEALLOCATE ( ZAUX )

!     Read field at each pressure level.
    ALLOCATE ( ZFLD (IDATX,IDATY,JPNIV) )
    ALLOCATE ( ZAUX (ILEND,1))

    DO JL=1,JPNIV

      READ (INSOUR) ZAUX

      DO JI=1+JPBX,NDATX+JPBX
        DO JJ=1+JPBY,NDATY+JPBY
          IZ=(JI-JPBX)+(JJ-1-JPBY)*NDATX
          ZFLD(JI,JJ,JL)=ZAUX(IZ,1)*ZSCALE(JC)
        ENDDO
      ENDDO

    ENDDO

    DEALLOCATE ( ZAUX )

!     ------------------------------------------------------------------
!*
!     4. INTERPOLATIONS.
!        ---------------

!     Interpolation of pressure levels (only once).
    IF (JC == 1.AND.JMOIS == 1) THEN
      CALL INTER8(NDGLG,NDLON,IDATY,IDATX,IPNIV,NPINT,ILENE,&
       & ICO,INJQ,INIQ,NLOENG(1),ILENT,ZP,ZPD,&
       & ZSLA,ZSLO,ZCLO)  
    ENDIF

!     Horizontal interpolation (4 points).

    CALL INTER8(NDGLG,NDLON,IDATY,IDATX,IPNIV,NPINT,ILENE,&
     & ICO,INJQ,INIQ,NLOENG(1),ILENT,ZC,ZFLD,&
     & ZSLA,ZSLO,ZCLO)  

!     Vertical interpolation (linear in pressure).

    CALL INTRV2(ZC,ZS,ILENE,IPNIV,NFLEVG,ZP,STPRE,ILENT,ILENT)

!     ------------------------------------------------------------------
!*
!     5. WRITE FIELDS.
!        -------------

!     Write on Arpege file.
    DO JL=1,NFLEVG
      CALL FAIENC(IREP,INUM,'S',JL,CLSUFF(JC),ZS(1,JL),LLCOSP)
    ENDDO

  ENDDO
!     End of loop over fields.

!     Close files.
  CLOSE(INSOUR)

  IF (JMOIS == 1) THEN
    CALL LFILAF(IREP,INUM,.TRUE.)
    LLIMST=.FALSE.
  ENDIF
  CALL FAIRME(IREP,INUM,'KEEP')

ENDDO

IF (ALLOCATED(ZFLD)) DEALLOCATE(ZFLD)
IF (ALLOCATED(ZPD)) DEALLOCATE(ZPD)

!     End of loop over months.

IF (LHOOK) CALL DR_HOOK('INCLIB',1,ZHOOK_HANDLE)
END SUBROUTINE INCLIB
