SUBROUTINE INCLIR

!**** *INCLIR*

!     PURPOSE.
!     --------

!      Interpolation or computation of aerosol fields

!**   INTERFACE.
!     ----------

!     CALL INCLIR

!     METHOD.
!     -------

!      The data are read on unit 11, if available, then interpolated.
!      Else, fields are derived from surface characteristics.
!      Finally they are written on ARPEGE clim file (unit 3).

!     EXTERNALS.
!     ----------

!     GEO923
!     INTER2
!     LISLAP
!     FA-LFI package (FAITOU,FACILE,FAIENC,LFILAF,FAIRME)
!     CHIEN

!     AUTHORS.
!     --------

!         M. DEQUE  AUG 94 .

!     MODIFICATIONS.
!     --------------
!        Modified 95-06-06 by L.Isaksen : Reordering of spectral arrays
!        Modified 98-08-10 by K. YESSAD: partial removal of LRPOLE option.
!         - LRPOLE has been removed from YOMCT0 and 'CALL CHIEN'
!           (assumed to be F).
!         - Call to POLADD and write poles on FA files is still made.
!        Modified 00-09-12 by D. Giard : adapted to new 923, one month, DM
!        Modified 00-10-19 by D. Giard : modifications from R. Mladek (97)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCLI   , ONLY : NPINT    ,NDATX    ,NDATY    ,LIEEE    ,&
 & NTPMER   ,NTPLAC   ,NTPGLA   ,NTPDES   ,SALBB    ,&
 & SVEG     ,SMASK      
USE YOMDIL   , ONLY : FACDI 
USE YOMDIM   , ONLY : NDGLG     ,NDGSAG    ,NDGENG    ,NDLON    ,NFLEVG   ,NSMAX
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NQUAD
USE YOMLEG   , ONLY : RMU
USE YOMGEM   , ONLY : NLOENG   ,NMENG    ,VALH     ,VBH      ,&
 & NSTTYP   ,NHTYP    ,RMUCEN   ,RLOCEN   ,RSTRET   ,VP00  

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JPBX
INTEGER(KIND=JPIM) :: JPBY
INTEGER(KIND=JPIM) :: JPNCH

PARAMETER (JPBX=2,JPBY=2,JPNCH=4)

!     JPBY : Extra-latitudes 
!          2 when INTER2 or INTER3 are used (even NDATY)
!          1 when INTER4 or INTER5 are used (odd  NDATY)
!     JPNCH: Number of fields to be processed (caution if not 4)

REAL(KIND=JPRB) :: ZS(NDGLG*NDLON,JPNCH)
REAL(KIND=JPRB) :: ZBO(NDGLG+1),ZLONG(NDGLG),ZMU(NDGLG*NDLON),ZSLA(NDGLG*NPINT)&
 & ,ZSLO(NDLON*NPINT,NDGLG),ZCLO(NDLON*NPINT,NDGLG),ZPORT(JPNCH)  
REAL(KIND=JPRB),ALLOCATABLE :: ZFLD(:,:), ZZLA(:), ZZLO(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZDUM(:),ZURB(:),ZVGX(:),ZITP(:),ZALB(:)

INTEGER(KIND=JPIM) :: ILISSE(JPNCH)
CHARACTER :: CLNOMC*16, CLNOMF*10, CLFORM*12
CHARACTER :: CLPRE(JPNCH)*8, CLSUF(JPNCH)*12

INTEGER(KIND=JPIM) :: IARI, IARP, ICO, IDATX, IDATY, IFLD, IINF, ILENE, ILENT,&
 & IMES, INIQ, INIV, INJQ, INUM, IOS, IREP, ITP, JF, J, JJ  

LOGICAL :: LLCOSP, LLIMST, LLPOLE, LLDATA, LLOPEN

REAL(KIND=JPRB) :: ZEPS, ZSCALE, ZURBN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "chien.intfb.h"
#include "geo923.intfb.h"
#include "inter2.intfb.h"
#include "lislap.intfb.h"

!     ------------------------------------------------------------------
!*
!     1. SET INITIAL VALUES.
!        -------------------

IF (LHOOK) CALL DR_HOOK('INCLIR',0,ZHOOK_HANDLE)
ZEPS=1.E-10_JPRB
ZSCALE=100._JPRB
ZURBN=5.E-3_JPRB

!     Grids.

ICO=NPINT*NPINT
INJQ=NDGLG*NPINT
INIQ=NDLON*NPINT
ILENT=NDGLG*NDLON
IDATX=NDATX+2*JPBX
IDATY=NDATY+2*JPBY

ILENE=0
CALL GEO923(NPINT,ILENE,ZBO,ZLONG,ZMU,ZSLA,ZSLO,ZCLO)

!     Arpege fields

INIV=0
LLCOSP=.FALSE.
DO JF=1,4
  CLPRE(JF)='SURF    '
ENDDO
CLSUF(1)='AEROS.SEA   '
CLSUF(2)='AEROS.LAND  '
CLSUF(3)='AEROS.URBAN '
CLSUF(4)='AEROS.DESERT'
DO JF=1,JPNCH
  DO J=1,NDLON*NDGLG
    ZS(J,JF)=0.0_JPRB
  ENDDO
ENDDO

!     Smoothing of aerosols (to be tuned)

ILISSE(1)=30
ILISSE(2)=40
ILISSE(3)=10
ILISSE(4)=30
ZPORT(1)=1.E+6_JPRB
ZPORT(2)=1.E+6_JPRB
ZPORT(3)=1.E+6_JPRB
ZPORT(4)=1.E+6_JPRB

!     Arpege file identifiers

INUM=3
IREP=0
IARI=0
IARP=1
CLNOMF='Const.Clim'
CLNOMC='Const.Clim.Surfa'
LLIMST=.TRUE.
IMES=1

IINF=0
LLPOLE=.TRUE.

!     Data type

IF (LIEEE) THEN
  CLFORM='UNFORMATTED'
  CALL ABOR1('INCLIR : INPUT FILES NOT YET READY !')
ELSE
  CLFORM='FORMATTED'
ENDIF

!     ------------------------------------------------------------------
!*
!     2. OPEN FILES.
!        -----------

!     Arpege file

CALL FAITOU(IREP,INUM,.TRUE.,CLNOMF,'OLD',.TRUE.,LLIMST,&
 & IMES,IARP,IARI,CLNOMC)  
CALL CHIEN(CLNOMC,NSTTYP,RMUCEN,RLOCEN,RSTRET,NSMAX,&
 & NDGLG,NDLON,NLOENG,NMENG,RMU,NHTYP,NFLEVG,VP00,VALH,VBH,&
 & NQUAD,IINF,NDGSAG,NDGENG,ZEPS,LLPOLE,NULOUT)  
IF (LLPOLE) CALL ABOR1(' CLIM. FILES MUST NOT HAVE POLES !')

!     External input data (if any)

LLDATA=.FALSE.
LLOPEN=.FALSE.
IOS= 0
INQUIRE(FILE='aero_GL',IOSTAT=IOS,EXIST=LLDATA,OPENED=LLOPEN)
LLDATA= LLDATA .AND. (IOS == 0) .AND. .NOT.LLOPEN

IF (LLDATA) THEN
  OPEN(UNIT=11,FILE='aero_GL',FORM=CLFORM)
  GO TO 300
ELSE
  GO TO 400
ENDIF

!     ------------------------------------------------------------------
!*
!     3. READ AND INTERPOLATE DATA.
!        --------------------------

300   CONTINUE
WRITE(NULOUT,'('' INCLIR : AEROSOL FIELDS INTERPOLATED'')')

!     Allocate temporary space

ALLOCATE ( ZFLD(IDATX,IDATY) )
ALLOCATE ( ZZLA(NDATY) )
ALLOCATE ( ZZLO(NDATX) )

!     Read file header

READ(11,9901) ZZLA
READ(11,9901) ZZLO
9901 FORMAT(5F15.4)

!     Loop on fields

DO JF=1,JPNCH

!     Read and scale data
  ZFLD(:,:)=0.0_JPRB
  DO JJ=JPBY+1,JPBY+NDATY
    READ (11,9902) (ZFLD(J,JJ),J=JPBX+1,JPBX+NDATX)
  ENDDO
  ZFLD(:,:)=ZFLD(:,:)/ZSCALE

!     Horizontal interpolation (12 points)
  IFLD=1
  CALL INTER2(NDGLG,NDLON,IDATY,IDATX,IFLD,NPINT,ILENE,ICO,INJQ,INIQ,&
   & NLOENG(1),ILENT,ZS(1,JF),ZFLD,ZSLA,ZSLO,ZCLO)  

ENDDO
9902 FORMAT(12F6.2)

!     Deallocate temporary space

DEALLOCATE ( ZFLD )
DEALLOCATE ( ZZLA )
DEALLOCATE ( ZZLO )

!     Close input file

CLOSE(11)
GO TO 500

!     ------------------------------------------------------------------
!*
!     4. COMPUTE FIELDS.
!        ---------------

400   CONTINUE
WRITE(NULOUT,'('' INCLIR : AEROSOL FIELDS COMPUTED'')')

!     Allocate temporary space

ALLOCATE ( ZDUM(NDGLG*NDLON) )
ALLOCATE ( ZURB(NDGLG*NDLON) )
ALLOCATE ( ZVGX(NDGLG*NDLON) )
ALLOCATE ( ZITP(NDGLG*NDLON) )
ALLOCATE ( ZALB(NDGLG*NDLON) )

!     Read surface characteristics

CALL FACILE(IREP,INUM,'SURF',INIV,'PROP.URBANIS',ZURB,LLCOSP)
CALL FACILE(IREP,INUM,'SURF',INIV,'PROP.VEG.MAX',ZVGX,LLCOSP)
CALL FACILE(IREP,INUM,'SURF',INIV,'IND.VEG.DOMI',ZITP,LLCOSP)
CALL FACILE(IREP,INUM,'SURF',INIV,'ALBEDO      ',ZALB,LLCOSP)

!     Compute and smooth aerosols distribution

DO J=1,ILENE
  ZDUM(J)=0.
  ITP=NINT(ZITP(J))
  IF (ZURB(J) > ZURBN) &
   & ZS(J,3)=1.0_JPRB  
  IF ((ITP == NTPMER .OR. ITP == NTPLAC) .AND. ZALB(J) < SALBB) &
   & ZS(J,1)= 1.0_JPRB  
  IF (ITP /= NTPMER .AND. ITP /= NTPLAC .AND. ITP /= NTPGLA) &
   & ZS(J,2)= 1.0_JPRB  
  IF (ITP == NTPDES .AND. ZVGX(J) < SVEG ) &
   & ZS(J,4)= 1.0_JPRB   
ENDDO

DO JF=1,JPNCH
  DO J=1,ILISSE(JF)
    CALL LISLAP(ZS(1,JF),NLOENG(1),NDGLG,NDLON,ILENE,ZLONG,ZBO,FACDI,&
     & ZDUM,SMASK,ZPORT(JF))  
  ENDDO
ENDDO     

!     Deallocate temporary space

DEALLOCATE ( ZDUM )
DEALLOCATE ( ZURB )
DEALLOCATE ( ZVGX )
DEALLOCATE ( ZITP )
DEALLOCATE ( ZALB )

!     ------------------------------------------------------------------
!*
!     5. WRITE FIELDS.
!        -------------

500  CONTINUE

!     Write fields

DO JF=1,JPNCH
  CALL FAIENC (IREP,INUM,CLPRE(JF),INIV,CLSUF(JF),ZS(1,JF),LLCOSP)
ENDDO

!     Close Arpege file

CALL LFILAF (IREP,INUM,.TRUE.)
CALL FAIRME (IREP,INUM,'KEEP')

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('INCLIR',1,ZHOOK_HANDLE)
END SUBROUTINE INCLIR
