SUBROUTINE INIPZ(POCE,PSPCM,KPDG)

!**** *INIPZ*

!     PURPOSE.
!     --------

!       Calculates the weights of the grid points

!**   INTERFACE.
!     ----------

!     *CALL* *INIPZ*(POCE,PSPCM,KPDG)

!       *KPDG*        - Number of grid points
!       *POCE*        - Fraction of ocean
!       *PSPCM*       - Threshold used to define ocean on the gaussian grid

!     METHOD.
!     -------

!       The weights are the ratio of the local dilatation factor over a sphere
!     with an enhanced stretching factor by the local dilatation factor over the
!     working sphere. The new stretching factor is calculated so that the weight
!     at the pole of interest is *FACE*.
!       The weights are increased over the oceans.
!       This routine works for the Operational Model (*OM*) or for the Climate
!     Version (*CV*).

!     EXTERNALS.
!     ----------

!     AUTHOR.
!     -------

!         Y. BOUTELOUP and M. DEQUE   1 FEB 91.

! Modified by Y.Bouteloup 08/93 (New cost function)
! Modified by R. EL Khatib : 94-03-08 comdecks YOMCLA and PTRSPOR
! Modified by P. Le Moigne : 95-01-18 (New cost function)
! Modified by Y. Bouteloup : 95-08-16 (Bug correction and tuning  ...
!                            IF LNEWORO .AND..NOT. LELAM)
! Modified : 95-12-22 M.Hamrud    Change of XREV,XPDS layout
! Modified : 00-09-15 D. Giard    message passing
! Modified : 01-03-08 S. Alexandru management of the extension zone
!                                  introduction of sea fraction
!                                  unified notations for cost functions
! Modified : 01-11-19 D. Giard  memory saving (allocatable arrays)
! Modified : 02-11-25 D. Giard  new formulation for extension zone
! Modified : 04-01-07 D. Giard  XPDS set to 0 by default
! Modified : 05-04-04 D. Giard  old ARPEGE cost function removed
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NDLON    ,NDGUXG   ,NDLUXG
USE PTRSPOR  , ONLY : XREV     ,XPDS     ,XEXT
USE YOMLEG   , ONLY : RMU
USE YOMGEM   , ONLY : NLOENG   ,NSTAGP   ,NTSTAGP  ,RSTRET
USE YOMCLA   , ONLY : QMAX     ,QMIN     ,HMIN     ,XINCOC   ,SCEXT    ,&
 & LNEWORO  ,LNEWORO2  
USE YOMCT0   , ONLY : LELAM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPDG 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POCE(KPDG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPCM 

REAL(KIND=JPRB) :: ZOCE(NDLON,NDGLG)
REAL(KIND=JPRB) :: ZC2, ZC2M1, ZC2P1, ZFAK, ZMU, ZEPS, ZLA, ZLO, ZDI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTEGER(KIND=JPIM) :: IADL(NDGLG)
INTEGER(KIND=JPIM) :: IADR, INDC, INDK, JI, JL

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('INIPZ',0,ZHOOK_HANDLE)

!*    1. SET INITIAL VALUES.
!        -------------------

XPDS(:)= 0.0_JPRB

!*    1.1. MANAGEMENT OF ADDRESSES

IF (LELAM) THEN
  IADL(1:NDGLG)=NTSTAGP(1:NDGLG)-1
ELSE
  IADL(1:NDGLG)=NSTAGP(1:NDGLG)-1
ENDIF  

!*    1.2. MANAGEMENT OF SEA-FRACTION

ZOCE(:,:)= 0.0_JPRB

IF (LELAM) THEN
  DO JL=1,NDGUXG
    DO JI=1,NDLUXG
      ZOCE(JI,JL)= POCE((JL-1)*NDLUXG+JI)
    ENDDO
  ENDDO
ELSE
  INDK= 0
  DO JL=1,NDGLG
    DO JI=1,NLOENG(JL)
      INDC= INDK+JI
      ZOCE(JI,JL)= POCE(INDC)
    ENDDO
    INDK= INDK+NLOENG(JL)
  ENDDO
ENDIF

!*    1.3. MANAGEMENT OF EXTENSION ZONE

XEXT(:,:)= 1.0_JPRB

IF (LELAM .AND. SCEXT > 0.0_JPRB) THEN
  ZEPS=1.E-1_JPRB
  ZLA=MAX(ZEPS,0.5_JPRB*(NDGLG-NDGUXG))**2
  ZLO=MAX(ZEPS,0.5_JPRB*(NDLON-NDLUXG))**2
  DO JL=1,NDGLG
    DO JI=1,NDLON
      ZDI=MAX(REAL((JL-NDGUXG)*(NDGLG-JL),JPRB)/ZLA,&
             &REAL((JI-NDLUXG)*(NDLON-JI),JPRB)/ZLO,0.0_JPRB)
      XEXT(JI,JL)= 1.0_JPRB/(1.0_JPRB+SCEXT*ZDI)
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*    2. NEW COST FUNCTION : BOUTELOUP.
!        ------------------------------

IF (LNEWORO) THEN

  ZC2P1= RSTRET*RSTRET+1.0_JPRB
  ZC2M1= RSTRET*RSTRET-1.0_JPRB
  ZC2  = ZC2M1/ZC2P1
  DO JL=1,NDGLG
    IF (LELAM) THEN
      ZFAK= 1.0_JPRB
    ELSE
      ZMU= (RMU(JL)-1.0_JPRB)/(ZC2P1+RMU(JL)*ZC2M1)
      ZFAK= 1.0_JPRB+ZC2*ZMU
    ENDIF
    DO JI=1,NLOENG(JL)
      IADR= JI+IADL(JL)
      XPDS(IADR)= QMIN +(QMAX-QMIN)*ZFAK*EXP(-ABS(XREV(IADR))/HMIN)&
       & *(1.0_JPRB+XINCOC*ZOCE(JI,JL))  
    ENDDO
  ENDDO

!     ------------------------------------------------------------------

!*    3. NEW COST FUNCTION : JERCZYNSKI.
!        -------------------------------

ELSEIF (LNEWORO2) THEN

  ZC2= (RSTRET*RSTRET-1.0_JPRB)/(RSTRET*RSTRET+1.0_JPRB)
  DO JL=1,NDGLG
    IF (LELAM) THEN
      ZFAK= 1.0_JPRB
    ELSE
      ZFAK= 1.0_JPRB+ZC2*((RMU(JL)-1.0_JPRB)/2.0_JPRB)
    ENDIF
    DO JI=1,NLOENG(JL)
      IADR= JI+IADL(JL)
      XPDS(IADR)= (QMIN+(QMAX-QMIN)*ZFAK*EXP(-ABS(XREV(IADR))/HMIN))&
       & *(1.0_JPRB+XINCOC*ZOCE(JI,JL))  
    ENDDO
  ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('INIPZ',1,ZHOOK_HANDLE)
END SUBROUTINE INIPZ
