SUBROUTINE INTER1 (KLAT,KLON,KLAT0,KLON0,KNCH,KPP,KNPT,KNPT0,&
 & KNCO,KNJQ,KNIQ,KNIP,KLEN,PZI,PZ0,PSLAC,PSLOC,PCLOC)  

!     PURPOSE.
!     --------

!       Iterpolation of a field from a finer grid.

!**   INTERFACE.
!     ----------

!     CALL INTER1 (KLAT,KLON,KLAT0,KLON0,KNCH,KPP,KNPT,KNPT0
!    $,KNCO,KNJQ,KNIQ,KNIP,KLEN,PZI,PZ0,PSLAC,PSLOC,PCLOC)
!        KLAT  = number of latitudes of the final grid
!        KLON  = number of longitudes of the final grid
!        KLAT0 = number of latitudes of the initial grid
!        KLON0 = number of longitudes of the initial grid
!        KPP   = size of the box (odd and at least KLAT0/KLAT)
!        KNCH  = number of field to interpolate
!        KNPT  = number of points of the final grid (KLAT*KLON if no reduction)
!        KLEN  = First length of the output field (may be different from KNPT)
!        KNCO  = KPP*KPP
!        KNPT0 = KLAT0*KLON0
!        KNJQ  = KLAT*KPP
!        KNIQ  = KLON*KPP
!        KNIP  = number of longitudes on a latitude circle (final grid)
!        PZI   = interpolated field (output)
!        PZ0   = initial field
!        PSLAC = sine of the latitude in the subgrid
!        PSLOC = sine of the longitude in the subgrid
!        PCLOC = cosine of the longitude in the subgrid

!     METHOD.
!     -------

!       The field is first calculated on a subgrid of the final grid.
!       Each value is taken at the nearest point of the initial grid.
!       Then the values of the box (subgrid) are averaged

!     EXTERNALS.
!     ----------

!        TRACARE

!     AUTHORS.
!     --------

!         J.F. GELEYN  M. DEQUE  1 FEB 91.

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIL   , ONLY : NPOTYP   ,SLAPO    ,GLAPO    ,SLOPO    ,GLOPO    ,FACDI
USE YOMCST   , ONLY : RPI

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNCH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNPT0 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNJQ 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNIQ 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM)               :: KLON ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(IN)    :: KLAT0 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON0 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNPT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNCO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNIP(KLAT) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZI(KLEN,KNCH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ0(KNPT0,KNCH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLAC(KNJQ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLOC(KNIQ,KLAT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOC(KNIQ,KLAT) 
REAL(KIND=JPRB) ::ZSLAC(KNCO),ZSLOC(KNCO),ZCLOC(KNCO)&
 & ,ZSLAR(KNCO),ZSLOR(KNCO),ZCLOR(KNCO)  
INTEGER(KIND=JPIM) :: IXY(KNCO)

INTEGER(KIND=JPIM) :: ICOMPT, INDICE, IX, IY, J, JCH, JI, JJ, JK, JL, JM

REAL(KIND=JPRB) :: ZCONO1, ZSI, ZX
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "tracare.intfb.h"

!     ------------------------------------------------------------------
!*
!     1. PROCESSING.
!        -----------

!     1.1 Setting initial values to zero.

IF (LHOOK) CALL DR_HOOK('INTER1',0,ZHOOK_HANDLE)
DO JCH=1,KNCH
  DO J=1,KNPT
    PZI(J,JCH)=0.0_JPRB
  ENDDO
ENDDO

!     1.2 Bulk summation.

!     Calculation of the coordinates of the points in the box

ICOMPT=0
DO JJ=1,KLAT

  DO JL=1,KPP
    DO JK=1,KPP
      ZSLAC((JL-1)*KPP+JK)=PSLAC((JJ-1)*KPP+JL)
    ENDDO
  ENDDO

  DO JI=1,KNIP(JJ)
    INDICE=ICOMPT+JI

    DO JL=1,KPP
      DO JK=1,KPP
        JM=(JL-1)*KPP+JK
        ZSLOC((JL-1)*KPP+JK)=PSLOC((JI-1)*KPP+JK,JJ)
        ZCLOC((JL-1)*KPP+JK)=PCLOC((JI-1)*KPP+JK,JJ)
      ENDDO
    ENDDO

!     Coordinate transform.

    CALL TRACARE (ZSLAC,ZSLOC,ZCLOC,ZSLAR,ZSLOR,ZCLOR,KNCO,&
     & SLAPO,GLAPO,SLOPO,GLOPO,FACDI,NPOTYP)  

    DO JM=1,KNCO

!     Loop in the box : the nearest value is taken.

      ZSLAR(JM)=MIN(1.0_JPRB,MAX(-1.0_JPRB,ZSLAR(JM)))
      IY=INT(1.0_JPRB+REAL(KLAT0,JPRB)*ACOS(ZSLAR(JM))/RPI)
      IY=MIN(IY,KLAT0)
      ZSI=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZSLOR(JM)))
      ZCLOR(JM)=MIN(1.0_JPRB,MAX(-1.0_JPRB,ZCLOR(JM)))
      ZX=(ZSI*ACOS(ZCLOR(JM))+(1.0_JPRB-ZSI)*(2.0_JPRB*RPI-ACOS(ZCLOR(&
       & JM))))*REAL(KLON0,JPRB)/RPI/2.0_JPRB  
      IX=1+MOD(INT(ZX),KLON0)
      IXY(JM)=(IY-1)*KLON0+IX
    ENDDO
    DO JCH=1,KNCH
      DO JM=1,KNCO
        PZI(INDICE,JCH)=PZI(INDICE,JCH)+PZ0(IXY(JM),JCH)
      ENDDO
    ENDDO

  ENDDO

  ICOMPT=ICOMPT+KNIP(JJ)
ENDDO

!     1.3 Normalisation.

ZCONO1=1.0_JPRB/REAL(KNCO,JPRB)

DO JCH=1,KNCH
  DO J=1,KNPT
    PZI(J,JCH)=PZI(J,JCH)*ZCONO1
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('INTER1',1,ZHOOK_HANDLE)
END SUBROUTINE INTER1
