SUBROUTINE INTER6 (KLAT,KLON,KLAT0,KLON0,KNCH,KPP,KNPT,KNCO,&
 & KNJQ,KNIQ,KNIP,KLEN,PZI,PZ0,PSLAC,PSLOC,PCLOC,PSEA0,PSEA,PSPCM)  

!     PURPOSE.
!     --------

!       Interpolation of a field from a finer grid :
!        fields with sea/land contrast (using 12 or 4 neighbouring points)

!**   INTERFACE.
!     ----------

!     CALL INTER6(KLAT,KLON,KLAT0,KLON0,KNCH,KPP,KNPT,KNCO,
!    &  KNJQ,KNIQ,KNIP,KLEN,PZI,PZ0,PSLAC,PSLOC,PCLOC,PSEA0,PSEA,PSPCM)
!        KLAT  = number of latitudes of the final grid
!        KLON  = number of longitudes of the final grid
!        KLAT0 = number of latitudes of the initial grid
!                  2  latitudes are added beyond each pole
!        KLON0 = number of longitudes of the initial grid
!                  4 longitudes are duplicated : 2 on each side
!        KNCH  = number of field to interpolate
!        KPP   = size of the box (odd and at least KLAT0/KLAT)
!        KNPT  = number of points of the final grid (KLAT*KLON if no reduction)
!        KNCO  = KPP*KPP
!        KNJQ  = KLAT*KPP
!        KNIQ  = KLON*KPP
!        KNIP  = number of longitudes on a latitude circle (final grid)
!        KLEN  = first length of the output field (may be different from KNPT)
!        PZI   = interpolated field (output)
!        PZ0   = initial field
!        PSLAC = sine of the latitude in the subgrid
!        PSLOC = sine of the longitude in the subgrid
!        PCLOC = cosine of the longitude in the subgrid
!        PSEA0 = fraction of sea on the initial grid
!        PSEA  = fraction of sea on the final grid (reference value)
!        PSPCM = threshold to define continents from the above reference

!     METHOD.
!     -------

!       The field is first calculated on a subgrid of the final grid.
!       Each value is interpolated from the 12 nearest points of the initial 
!       grid.
!       These points are weighted by the percent of area covered by the same
!       category (land or sea) as the final point.
!       In the case of an insufficient number of neighbours of the same
!       category, a 4 point interpolation is carried out to avoid overshoot.
!       If there are no neighbours of the same category, the interpolation is 
!       performed with the opposite category values.
!       Then the values of the box (subgrid) are averaged.

!     EXTERNALS.
!     ----------

!        TRACARE

!     AUTHORS.
!     --------

!         J.F. GELEYN  M. DEQUE  20 MAY 92.

!     MODIFICATIONS.
!     --------------
!     M. DEQUE 13 MAY 97: no more use of PSEA1 (interpolated fraction of sea)
!                         but recalculation (safer)
!     D. GIARD 22 OCT.97  PSEA1 no longer as argument
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIL   , ONLY : NPOTYP   ,SLAPO    ,GLAPO    ,SLOPO    ,GLOPO    ,FACDI
USE YOMCST   , ONLY : RPI

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLAT0 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON0 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNCH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNPT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNJQ 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNIQ 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM)               :: KLON ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(IN)    :: KPP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNCO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNIP(KLAT) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZI(KLEN,KNCH) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZ0(KLON0,KLAT0,KNCH) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLAC(KNJQ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLOC(KNIQ,KLAT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLOC(KNIQ,KLAT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSEA0(KLON0,KLAT0) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSEA(KNPT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPCM 
REAL(KIND=JPRB) :: ZSLAC(KNCO),ZSLOC(KNCO),ZCLOC(KNCO),&
 & ZSLAR(KNCO),ZSLOR(KNCO),ZCLOR(KNCO),&
 & ZXX(KNCO),ZYY(KNCO),ZWGT12(KNCO,12)  
INTEGER(KIND=JPIM) :: IXX(KNCO),IYY(KNCO)

INTEGER(KIND=JPIM) :: ICOMPT, INDICE, IPAUT, IXM, IYM, J, JCH, JFL, JI, JJ, JK, JL, JM

REAL(KIND=JPRB) :: ZCONO2, ZCX, ZCY, ZEPS12, ZEPS4, ZSI, ZWCO1,&
 & ZWCO2, ZWGT, ZX, ZXM, ZY, ZYM  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "tracare.intfb.h"

!     ------------------------------------------------------------------
!*
!     1. PRE-PROCESSING.
!     ------------------

!     ZEPS12 : CRITICAL PERCENT FOR THE 12 PT INTERPOLATION
!     ZEPS4 : CRITICAL PERCENT FOR THE 4 PT INTERPOLATION
IF (LHOOK) CALL DR_HOOK('INTER6',0,ZHOOK_HANDLE)
ZEPS12=.25_JPRB
ZEPS4=1.E-8_JPRB

!     1.1 Setting initial values to zero and extending the initial grid.

DO JCH=1,KNCH
  DO J=1,KNPT
    PZI(J,JCH)=0.0_JPRB
  ENDDO
ENDDO
IPAUT=(KLON0-4)/2
DO JFL=1,KNCH
  DO JI=3,IPAUT+2
    PZ0(JI      ,1      ,JFL)=PZ0(JI+IPAUT,4      ,JFL)
    PZ0(JI+IPAUT,1      ,JFL)=PZ0(JI      ,4      ,JFL)
    PZ0(JI      ,2      ,JFL)=PZ0(JI+IPAUT,3      ,JFL)
    PZ0(JI+IPAUT,2      ,JFL)=PZ0(JI      ,3      ,JFL)
    PZ0(JI      ,KLAT0  ,JFL)=PZ0(JI+IPAUT,KLAT0-3,JFL)
    PZ0(JI+IPAUT,KLAT0  ,JFL)=PZ0(JI      ,KLAT0-3,JFL)
    PZ0(JI      ,KLAT0-1,JFL)=PZ0(JI+IPAUT,KLAT0-2,JFL)
    PZ0(JI+IPAUT,KLAT0-1,JFL)=PZ0(JI      ,KLAT0-2,JFL)
  ENDDO

  DO JJ=1,KLAT0
    PZ0(1      ,JJ,JFL)=PZ0(KLON0-3,JJ,JFL)
    PZ0(2      ,JJ,JFL)=PZ0(KLON0-2,JJ,JFL)
    PZ0(KLON0-1,JJ,JFL)=PZ0(3      ,JJ,JFL)
    PZ0(KLON0  ,JJ,JFL)=PZ0(4      ,JJ,JFL)
  ENDDO
ENDDO
DO JI=3,IPAUT+2
  PSEA0(JI      ,1      )=PSEA0(JI+IPAUT,4      )
  PSEA0(JI+IPAUT,1      )=PSEA0(JI      ,4      )
  PSEA0(JI      ,2      )=PSEA0(JI+IPAUT,3      )
  PSEA0(JI+IPAUT,2      )=PSEA0(JI      ,3      )
  PSEA0(JI      ,KLAT0  )=PSEA0(JI+IPAUT,KLAT0-3)
  PSEA0(JI+IPAUT,KLAT0  )=PSEA0(JI      ,KLAT0-3)
  PSEA0(JI      ,KLAT0-1)=PSEA0(JI+IPAUT,KLAT0-2)
  PSEA0(JI+IPAUT,KLAT0-1)=PSEA0(JI      ,KLAT0-2)
ENDDO

DO JJ=1,KLAT0
  PSEA0(1      ,JJ)=PSEA0(KLON0-3,JJ)
  PSEA0(2      ,JJ)=PSEA0(KLON0-2,JJ)
  PSEA0(KLON0-1,JJ)=PSEA0(3      ,JJ)
  PSEA0(KLON0  ,JJ)=PSEA0(4      ,JJ)
ENDDO

!     ------------------------------------------------------------------
!*
!     2. LOOP ON THE FINAL GRID.
!     --------------------------

ICOMPT=0
DO JJ=1,KLAT

!     2.1 Calculation of the coordinates of the points in the box

  DO JL=1,KPP
    DO JK=1,KPP
      ZSLAC((JL-1)*KPP+JK)=PSLAC((JJ-1)*KPP+JL)
    ENDDO
  ENDDO

  DO JI=1,KNIP(JJ)
    INDICE=ICOMPT+JI

    DO JL=1,KPP
      DO JK=1,KPP
        JM=(JL-1)*KPP+JK
        ZSLOC((JL-1)*KPP+JK)=PSLOC((JI-1)*KPP+JK,JJ)
        ZCLOC((JL-1)*KPP+JK)=PCLOC((JI-1)*KPP+JK,JJ)
      ENDDO
    ENDDO

    CALL TRACARE (ZSLAC,ZSLOC,ZCLOC,ZSLAR,ZSLOR,ZCLOR,KNCO,&
     & SLAPO,GLAPO,SLOPO,GLOPO,FACDI,NPOTYP)  

!     2.2 Calculation of the weigths

    DO JM=1,KNCO
      ZSLAR(JM)=MIN(1.0_JPRB,MAX(-1.0_JPRB,ZSLAR(JM)))
      ZSI=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZSLOR(JM)))
      ZCLOR(JM)=MIN(1.0_JPRB,MAX(-1.0_JPRB,ZCLOR(JM)))
      ZX=(ZSI*ACOS(ZCLOR(JM))+(1.0_JPRB-ZSI)*(2.0_JPRB*RPI-ACOS(ZCLOR(JM))))&
       & *REAL(KLON0-4,JPRB)/RPI/2.0_JPRB  
      ZX=MOD(ZX,REAL(KLON0-4,JPRB))
      IXX(JM)=INT(2.5_JPRB+ZX)
      ZXX(JM)=2.5_JPRB+ZX-IXX(JM)
      ZY=REAL(KLAT0-4,JPRB)*ACOS(ZSLAR(JM))/RPI
      IYY(JM)=INT(2.5_JPRB+ZY)
      ZYY(JM)=2.5_JPRB+ZY-IYY(JM)
    ENDDO
    IF(PSEA(INDICE) <= PSPCM)THEN
!           land
      ZWCO1=1.0_JPRB
      ZWCO2=-1.0_JPRB
    ELSE
!           sea
      ZWCO1=0.0_JPRB
      ZWCO2=1.0_JPRB
    ENDIF

!          Loop in the box : the linear 12 pt interpolation weights are set
!            (multiplied by 24 to save time).

    DO JM=1,KNCO
      ZXM=ZXX(JM)
      ZYM=ZYY(JM)
      IXM=IXX(JM)
      IYM=IYY(JM)
      ZCX=1.0_JPRB-ZXM
      ZCY=1.0_JPRB-ZYM
      ZWGT12(JM,1)=3._JPRB*(8._JPRB*ZXM*ZYM+ZXM*ZCX*(ZXM-ZCX+2.0_JPRB*ZYM)&
       & +ZYM*ZCY*(ZYM-ZCY+2.0_JPRB*ZXM))*(ZWCO1+ZWCO2*PSEA0(IXM+1,IYM+1))  
      ZWGT12(JM,2)=3._JPRB*(8._JPRB*ZCX*ZYM+ZCX*ZXM*(ZCX-ZXM+2.0_JPRB*ZYM)&
       & +ZYM*ZCY*(ZYM-ZCY+2.0_JPRB*ZCX))*(ZWCO1+ZWCO2*PSEA0(IXM  ,IYM+1))  
      ZWGT12(JM,3)=3._JPRB*(8._JPRB*ZCX*ZCY+ZCX*ZXM*(ZCX-ZXM+2.0_JPRB*ZCY)&
       & +ZCY*ZYM*(ZCY-ZYM+2.0_JPRB*ZCX))*(ZWCO1+ZWCO2*PSEA0(IXM  ,IYM  ))  
      ZWGT12(JM,4)=3._JPRB*(8._JPRB*ZXM*ZCY+ZXM*ZCX*(ZXM-ZCX+2.0_JPRB*ZCY)&
       & +ZCY*ZYM*(ZCY-ZYM+2.0_JPRB*ZXM))*(ZWCO1+ZWCO2*PSEA0(IXM+1,IYM  ))  
      ZWGT12(JM, 5)=ZXM*ZCX*(ZCX-ZXM-6._JPRB*ZYM)&
       & *(ZWCO1+ZWCO2*PSEA0(IXM+2,IYM+1))  
      ZWGT12(JM, 6)=ZYM*ZCY*(ZCY-ZYM-6._JPRB*ZXM)&
       & *(ZWCO1+ZWCO2*PSEA0(IXM+1,IYM+2))  
      ZWGT12(JM, 7)=ZYM*ZCY*(ZCY-ZYM-6._JPRB*ZCX)&
       & *(ZWCO1+ZWCO2*PSEA0(IXM  ,IYM+2))  
      ZWGT12(JM, 8)=ZCX*ZXM*(ZXM-ZCX-6._JPRB*ZYM)&
       & *(ZWCO1+ZWCO2*PSEA0(IXM-1,IYM+1))  
      ZWGT12(JM, 9)=ZCX*ZXM*(ZXM-ZCX-6._JPRB*ZCY)&
       & *(ZWCO1+ZWCO2*PSEA0(IXM-1,IYM  ))  
      ZWGT12(JM,10)=ZCY*ZYM*(ZYM-ZCY-6._JPRB*ZCX)&
       & *(ZWCO1+ZWCO2*PSEA0(IXM  ,IYM-1))  
      ZWGT12(JM,11)=ZCY*ZYM*(ZYM-ZCY-6._JPRB*ZXM)&
       & *(ZWCO1+ZWCO2*PSEA0(IXM+1,IYM-1))  
      ZWGT12(JM,12)=ZXM*ZCX*(ZCX-ZXM-6._JPRB*ZCY)&
       & *(ZWCO1+ZWCO2*PSEA0(IXM+2,IYM  ))  
    ENDDO
    ZWGT=0.0_JPRB
    DO JM=1,KNCO
      DO JK=1,12
        ZWGT=ZWGT+ZWGT12(JM,JK)
      ENDDO
    ENDDO
    ZWGT=ZWGT/REAL(24*KNCO,JPRB)
    IF(ZWGT >= ZEPS12)THEN
      ZCONO2=1.0_JPRB/REAL(24*KNCO,JPRB)/ZWGT
    ELSE

!          Loop in the box : the linear 4 pt interpolation weights are set

      DO JM=1,KNCO
        ZXM=ZXX(JM)
        ZYM=ZYY(JM)
        IXM=IXX(JM)
        IYM=IYY(JM)
        ZCX=1.0_JPRB-ZXM
        ZCY=1.0_JPRB-ZYM
        ZWGT12(JM,1)=ZXM*ZYM*(ZWCO1+ZWCO2*PSEA0(IXM+1,IYM+1))
        ZWGT12(JM,2)=ZCX*ZYM*(ZWCO1+ZWCO2*PSEA0(IXM  ,IYM+1))
        ZWGT12(JM,3)=ZCX*ZCY*(ZWCO1+ZWCO2*PSEA0(IXM  ,IYM  ))
        ZWGT12(JM,4)=ZXM*ZCY*(ZWCO1+ZWCO2*PSEA0(IXM+1,IYM  ))
        ZWGT12(JM,5)=0.0_JPRB
        ZWGT12(JM,6)=0.0_JPRB
        ZWGT12(JM,7)=0.0_JPRB
        ZWGT12(JM,8)=0.0_JPRB
        ZWGT12(JM,9)=0.0_JPRB
        ZWGT12(JM,10)=0.0_JPRB
        ZWGT12(JM,11)=0.0_JPRB
        ZWGT12(JM,12)=0.0_JPRB
      ENDDO
      ZWGT=0.0_JPRB
      DO JM=1,KNCO
        DO JK=1,4
          ZWGT=ZWGT+ZWGT12(JM,JK)
        ENDDO
      ENDDO
      ZWGT=ZWGT/REAL(KNCO,JPRB)
      IF(ZWGT < ZEPS4)THEN

!          Loop in the box : the linear 4 pt interpolation weights are
!          recalculated by inverting the mask

        ZWGT=1.0_JPRB-ZWGT
        DO JM=1,KNCO
          ZXM=ZXX(JM)
          ZYM=ZYY(JM)
          ZCX=1.0_JPRB-ZXM
          ZCY=1.0_JPRB-ZYM
          ZWGT12(JM,1)=ZXM*ZYM-ZWGT12(JM,1)
          ZWGT12(JM,2)=ZCX*ZYM-ZWGT12(JM,2)
          ZWGT12(JM,3)=ZCX*ZCY-ZWGT12(JM,3)
          ZWGT12(JM,4)=ZXM*ZCY-ZWGT12(JM,4)
        ENDDO
      ENDIF
      ZCONO2=1.0_JPRB/REAL(KNCO,JPRB)/ZWGT
    ENDIF

!     2.3 Interpolation

    DO JM=1,KNCO
      DO JFL=1,KNCH
        IXM=IXX(JM)
        IYM=IYY(JM)
        PZI(INDICE,JFL)=PZI(INDICE,JFL)&
         & +ZWGT12(JM, 1)*PZ0(IXM+1,IYM+1,JFL)&
         & +ZWGT12(JM, 2)*PZ0(IXM  ,IYM+1,JFL)&
         & +ZWGT12(JM, 3)*PZ0(IXM  ,IYM  ,JFL)&
         & +ZWGT12(JM, 4)*PZ0(IXM+1,IYM  ,JFL)&
         & +ZWGT12(JM, 5)*PZ0(IXM+2,IYM+1,JFL)&
         & +ZWGT12(JM, 6)*PZ0(IXM+1,IYM+2,JFL)&
         & +ZWGT12(JM, 7)*PZ0(IXM  ,IYM+2,JFL)&
         & +ZWGT12(JM, 8)*PZ0(IXM-1,IYM+1,JFL)&
         & +ZWGT12(JM, 9)*PZ0(IXM-1,IYM  ,JFL)&
         & +ZWGT12(JM,10)*PZ0(IXM  ,IYM-1,JFL)&
         & +ZWGT12(JM,11)*PZ0(IXM+1,IYM-1,JFL)&
         & +ZWGT12(JM,12)*PZ0(IXM+2,IYM  ,JFL)  
      ENDDO
    ENDDO
    DO JFL=1,KNCH
      PZI(INDICE,JFL)=PZI(INDICE,JFL)*ZCONO2
    ENDDO

  ENDDO
  ICOMPT=ICOMPT+KNIP(JJ)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('INTER6',1,ZHOOK_HANDLE)
END SUBROUTINE INTER6
