SUBROUTINE INTRV2(P1,P2,KNPT,KL1,KL2,PR1,PR2,KLEN1,KLEN2)

!**** *INTRV2*

!     PURPOSE.
!     --------

!     Vertical interpolation linear in pressure for constant output levels

!**   INTERFACE.
!     ----------

!     CALL INTRV2(P1,P2,KNPT,KL1,KL2,PR1,PR2,KLEN1,KLEN2)
!          P1    : Input field
!          P2    : Output field
!          KNPT  : Number of horizontal grid points
!          KL1   : Number of vertical levels (input)
!          KL2   : Number of vertical levels (output)
!          PR1   : Pressure levels (input)
!          PR2   : Pressure levels (output)
!          KLEN1 : First dimension of input
!          KLEN2 : First dimension of output

!     METHOD.
!     -------

!       Interpolates as a linear function of pressure. The distribution of
!    input pressure may overlap the output pressure. In this case, the field
!    is kept constant above or below. The levels are arranged from top to
!    bottom. The distribution of output pressure is uniform.

!     EXTERNALS.
!     ----------

!     NONE

!     AUTHORS.
!     --------

!         M. DEQUE  JUL 91 .

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KL1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P1(KLEN1,KL1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P2(KLEN2,KL2) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNPT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR1(KLEN1,KL1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR2(KL2) 
INTEGER(KIND=JPIM) :: I1, IMAX, IMIN, J1, J2, JNPT

REAL(KIND=JPRB) :: ZUP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('INTRV2',0,ZHOOK_HANDLE)
DO JNPT=1,KNPT

!*
!     1. CHECK FOR EXTRAPOLATION.
!        ------------------------

  IMIN=1
  IF(PR1(JNPT,1) > PR2(1))THEN
!        Needs to extrapolate above
    DO J2=1,KL2
      IF(PR1(JNPT,1) > PR2(J2)) GO TO 101
      IMIN=J2
      GO TO 102
      101 continue
    ENDDO
    102 CONTINUE
  ENDIF
  IMAX=KL2
  IF(PR1(JNPT,KL1) < PR2(KL2))THEN
!        Needs to extrapolate below
    DO J2=KL2,1,-1
      IF(PR1(JNPT,KL1) < PR2(J2)) GO TO 103
      IMAX=J2
      GO TO 104
      103 continue
    ENDDO
    104 CONTINUE
  ENDIF
!*
!     2. INTERPOLATION .
!        ---------------

  I1=0
  DO J2=IMIN,IMAX
    DO J1=2,KL1
      IF(PR1(JNPT,J1) < PR2(J2))GO TO 202
      I1=J1
      GO TO 203
      202 continue
    ENDDO
    203 CONTINUE
    ZUP=1.0_JPRB/(PR1(JNPT,I1)-PR1(JNPT,I1-1))
    P2(JNPT,J2)=(P1(JNPT,I1-1)*(PR1(JNPT,I1)-PR2(J2))&
     & +P1(JNPT,I1)*(PR2(J2)-PR1(JNPT,I1-1)))*ZUP  
  ENDDO
!*
!     3. EXTRAPOLATION .
!        ---------------

  DO J2=1,IMIN-1
    P2(JNPT,J2)=P1(JNPT,1)
  ENDDO
  DO J2=IMAX+1,KL2
    P2(JNPT,J2)=P1(JNPT,KL1)
  ENDDO

ENDDO

IF (LHOOK) CALL DR_HOOK('INTRV2',1,ZHOOK_HANDLE)
END SUBROUTINE INTRV2

