SUBROUTINE LISLAP(P,KINIP,KLATG,KLONG,KPDG,PCOS,PLAT,PFAC,POCE,PSPCM,&
 & PORTEE)  

!**** *LISLAP*

!     PURPOSE.
!     --------

!       Spatial smoothing of a field by adding the laplacian times a constant
!     factor. This enables to deal with the inhomogeneity of the grid point
!     density. This routine may be called several times to increase smoothing.
!     In the initial version, the logarithm of the field was smoothed.

!**   INTERFACE.
!     ----------

!     *CALL* *LISLAP* (...)

!       *P*           - Field to be smoothed (input/output)
!       *KINIP*       - Number of longitudes per latitude
!       *KLONG*       - Number of longitudes of the gaussian grid
!       *KLATG*       - Number of latitudes of the gaussian grid
!       *KPDG*        - Number of grid points
!       *PCOS*        - Cosine of the latitude
!       *PLAT*        - Latitude (Radian)
!       *PFAC*        - Stretching factor
!       *POCE*        - Fraction of ocean
!       *PSPCM*       - Threshold used to define ocean on the gaussian grid
!       *PORTEE*      - Characteristic length of smoothing

!     METHOD.
!     -------

!       The laplacian is calculated with 7 grid-points (5 grid-points when  no
!     reduction of the number of longitudes). The local stretching factor allows
!     to take into account the actual density. The values are not modified
!     over ocean or sea ice (POCE(.).GT.PSPCM)

!     EXTERNALS.
!     ----------

!           NONE

!     AUTHOR.
!     -------

!         M. DEQUE   1 FEB 91.

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!        Michel Deque 15 JUL 91 :suppression of the last rotation and
!                    security on the weights (limitation to 0.25)
!        Ryad el Khatib 26 Mars 93 : value of ZPCO increased to 15000.
!        L. Gerard       5/06/1997 : value of ZPC0 decreased to 4000 for LAM
!        D. Giard       19/10/2000 : LOG conversion outside of LISLAP 
!                                    characteristic length as argument

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RPI      ,RA

!     PORTEE = Characteristic length of the smoothing : if the mesh size is DX,
!     the smoothing consists approximately of weighting the grid-point value
!     by 1-4*(PORTEE/DX)**2 , and the average of the neighbouring points by
!     4*(PORTEE/DX)**2 . The choice of PORTEE must lead to positive values of the
!     former weight , its minimum value being about .25, so that in the high
!     density area the smoothing acts as the previous 9-point smoothing.
!     However the weights are maintained to positive values even when DX becomes
!     small.

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLATG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPDG 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P(KPDG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KINIP(KLATG) 
INTEGER(KIND=JPIM)               :: KLONG ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOS(KLATG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT(KLATG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFAC 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POCE(KPDG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPCM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PORTEE 
REAL(KIND=JPRB) :: ZWORK(KPDG)
INTEGER(KIND=JPIM) :: IIM1, IINN, IINNP1, IINS, IINSP1, IIP1, IK1,&
 & IK2, IK3, IKM1, IKP1, ILONN, ILONS, IND11, &
 & IND12, IND21, IND22, IND23, IND31, IND32, JI, JK  

REAL(KIND=JPRB) :: ZCOSR2, ZDELTA, ZDX, ZDYN, ZDYS, ZSF,&
 & ZW1, ZW11, ZW12, ZW2, ZW21, ZW22, ZW23, ZW3, &
 & ZW31, ZW32, ZWMAX, ZXIN, ZXIS, ZXN, ZXS  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
!*
!     1.  SET INITIAL VALUES
!         ------------------

IF (LHOOK) CALL DR_HOOK('LISLAP',0,ZHOOK_HANDLE)
ZCOSR2=(PORTEE/RA)**2

DO JK=1,KPDG
  ZWORK(JK)=P(JK)
ENDDO

IK1=0
IK2=0
IK3=0

!     ------------------------------------------------------------------
!*
!     2. CALCULATION OF THE LAPLACIAN
!        ----------------------------

DO JK=1,KLATG

!     2.1 Latitude dependent terms

  ILONN=0
  ILONS=0
  IKP1=JK+1
  IKM1=JK-1
  IF(JK == 1)THEN
    ILONN=KINIP(1)/2
    IKM1=1
    IK3=KINIP(1)
  ENDIF
  IF(JK == 2)THEN
    IK2=KINIP(1)
    IK3=IK3+KINIP(2)
  ENDIF
  IF(JK > 2.AND.JK < KLATG)THEN
    IK1=IK1+KINIP(JK-2)
    IK2=IK2+KINIP(JK-1)
    IK3=IK3+KINIP(JK)
  ENDIF
  IF(JK == KLATG)THEN
    ILONS=KINIP(KLATG)/2
    IKP1=KLATG
    IK1=IK1+KINIP(JK-2)
    IK2=IK2+KINIP(JK-1)
  ENDIF

!     Latitude and longitude increments
  ZDX=2.0_JPRB*RPI*PCOS(JK)/KINIP(JK)
  ZDYN=PLAT(IKM1)-PLAT(JK)
  ZDYS=PLAT(JK)-PLAT(IKP1)
  IF(JK == 1)THEN
    ZDYN=RPI-2.0_JPRB*PLAT(1)
  ENDIF
  IF(JK == KLATG)THEN
    ZDYS=2.0_JPRB*PLAT(KLATG)+RPI
  ENDIF

!     Configuration of the neighbouring grid points

!             P11      P13                      N
!         P21       P22       P23             W . E
!               P31                P32          S

!     Weights used to calculate the Laplacian
  ZW1=2.0_JPRB/ZDYN/(ZDYN+ZDYS)
  ZW2=1.0_JPRB/ZDX/ZDX
  ZW3=2.0_JPRB/ZDYS/(ZDYN+ZDYS)

!     Local stretching factor
  ZSF=(((PFAC**2+1)+(PFAC**2-1)*SIN(PLAT(JK)))*0.5_JPRB/PFAC)**2

!     Security on the weights
  ZWMAX=0.25_JPRB/(ZSF*ZCOSR2)
  ZW1=MIN(ZW1,ZWMAX)
  ZW2=MIN(ZW2,ZWMAX)
  ZW3=MIN(ZW3,ZWMAX)

!     2.2 Loop over the longitudes

  DO JI=1,KINIP(JK)

!     Indices of the western and eastern point
    IIP1=1+MOD(JI,KINIP(JK))
    IIM1=1+MOD(JI-2+KINIP(JK),KINIP(JK))
    IND21=IK2+IIM1
    IND22=IK2+JI
    IND23=IK2+IIP1

!     Indices of the northern and southern points.
!     An interpolation of the field on the latitude circles is necessary
!     when the number of longitudes per latitude varies
    ZXN=REAL(JI-1,JPRB)*REAL(KINIP(IKM1),JPRB)/REAL(KINIP(JK),JPRB)
    ZXS=REAL(JI-1,JPRB)*REAL(KINIP(IKP1),JPRB)/REAL(KINIP(JK),JPRB)
    IINN=INT(ZXN)+1
    IINS=INT(ZXS)+1
    ZXIN=REAL(IINN,JPRB)-ZXN
    ZXIS=REAL(IINS,JPRB)-ZXS
    IINN=1+MOD(IINN+ILONN-1,KINIP(IKM1))
    IINS=1+MOD(IINS+ILONS-1,KINIP(IKP1))
    IINNP1=1+MOD(IINN,KINIP(IKM1))
    IINSP1=1+MOD(IINS,KINIP(IKP1))
    IND11=IK1+IINN
    IND12=IK1+IINNP1
    IND31=IK3+IINS
    IND32=IK3+IINSP1

!      Calculation of the weights of the neighbouring points
!      Only the points over continent are taken
    ZW11=ZW1*ZXIN*MAX(0.0_JPRB,SIGN(1.0_JPRB,PSPCM-POCE(IND11)))
    ZW12=ZW1*(1.0_JPRB-ZXIN)*MAX(0.0_JPRB,SIGN(1.0_JPRB,PSPCM-POCE(IND12)))
    ZW21=ZW2*MAX(0.0_JPRB,SIGN(1.0_JPRB,PSPCM-POCE(IND21)))
    ZW23=ZW2*MAX(0.0_JPRB,SIGN(1.0_JPRB,PSPCM-POCE(IND23)))
    ZW31=ZW3*ZXIS*MAX(0.0_JPRB,SIGN(1.0_JPRB,PSPCM-POCE(IND31)))
    ZW32=ZW3*(1.0_JPRB-ZXIS)*MAX(0.0_JPRB,SIGN(1.0_JPRB,PSPCM-POCE(IND32)))
    ZW22=-(ZW11+ZW12+ZW21+ZW23+ZW31+ZW32)

!     Laplacian
    ZDELTA=ZSF*(ZW11*ZWORK(IND11)+ZW12*ZWORK(IND12)&
     & +ZW21*ZWORK(IND21)+ZW22*ZWORK(IND22)+ZW23*ZWORK(IND23)&
     & +ZW31*ZWORK(IND31)+ZW32*ZWORK(IND32))  

!     Smoothing (only over continent)
    P(IND22)=ZWORK(IND22)+ZCOSR2*ZDELTA &
     & *MAX(0.0_JPRB,SIGN(1.0_JPRB,PSPCM-POCE(IND22)))  

  ENDDO

ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LISLAP',1,ZHOOK_HANDLE)
END SUBROUTINE LISLAP
