SUBROUTINE RELNEW(LDLSM,LDFIT,KPDG,PZR,PZM)

!**** *RELNEW*

!     PURPOSE.
!     --------

!     Reads a new orography ,checks the *CADRE* , and makes a spectral
!     truncation. May also read a new land-sea mask, according to LDLSM.

!**   INTERFACE.
!     ----------

!     CALL RELNEW(LDLSM,KPDG,PZR,PZM)
!         LDLSM : Option for reading a new land-sea mask
!         LDFIT : Option for making spectral fit of orography
!         KPDG  : Size of the field
!         PZR   : Orography
!         PZM   : Land-sea mask
!     Output in PTRSPOR : XREV, XRNM (in m)  

!     METHOD.
!     -------

!     The fields are read on unit 'Neworog'.
!     The grid point values are returned in PZR/XREV and PZM.
!     For ARPEGE, the spectral coefficients for orography are stored in XRNM.

!     EXTERNALS.
!     ----------

!     FAITOU , FACILE , FACIES , FANION , FAIRME
!     CHIEN  , CCHIEN 
!     REESPE , SPEREE , EREESPE , SPEREEE , SPREORD
!     LFINFO

!     AUTHORS.
!     --------

!         M. DEQUE  92-02-15

!     MODIFICATIONS.
!     --------------
!     Modified by R. EL Khatib : 94-03-08 comdeck PTRSPOR
!     Modified by L.Isaksen    : 95-06-06 Reordering of spectral arrays
!     Modified by M.Hamrud     : 95-12-22 Change of XREV,XPDS layout
!     Modified by D.Giard      : 97-03-13 Options LDLSM and LELAM 
!     Modified by P.Le Moigne  : 98-05-26 Read spectral orog. if pdg orog.
!                                         doesn't exist in input file.
!     Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!      - LRPOLE removed from YOMCT0, 'CALL CHIEN', 'CALL SPEREESM'.
!      - LLPOLE renamed into LLFICP.
!      - for spherical geometry, check that the output arrays PZM and PZR have
!        no polar values and have their non polar values in elements
!        1 to NGPTOTG.
!     Modified by D. Giard : 99-06-11 Cleaning, land-sea mask conversion
!     Modified by D. Giard : 00-09-15 message passing
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     Modified by D. Paradis & R. El Khatib : 04-07-22 GRIBEX
!     Modified by D. Giard : 02-12-16 Importing an orography with a reduced 
!                                     spectral resolution (from R. EL Khatib)
!     Modified by D. Giard : 05-04-04 Update and cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NDGSAG   ,NDGENG   ,NDGNH    ,&
 & NDLON    ,NDGUNG   ,NDGUXG   ,NDLUNG   ,NDLUXG   ,NFLEVG   ,&
 & NSMAX    ,NMSMAX   ,NSEFRE   ,NSPEC2  
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NQUAD    ,LELAM
USE YOMCST   , ONLY : RG
USE YOMGEM   , ONLY : NLOENG   ,NMENG    ,NSTAGP   ,NTSTAGP  ,&
 & VALH     ,VBH      ,NSTTYP   ,NHTYP    ,RMUCEN   ,RLOCEN   ,&
 & RSTRET   ,VP00  
USE YOMLEG   , ONLY : RMU
USE YOMCLA   , ONLY : NLISSP
USE PTRSPOR  , ONLY : XRNM     ,XREV

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPDG 
LOGICAL           ,INTENT(IN)    :: LDLSM 
LOGICAL           ,INTENT(INOUT) :: LDFIT 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZR(KPDG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZM(KPDG) 

REAL(KIND=JPRB),ALLOCATABLE :: ZSINLA(:),ZAHYBR(:),ZBHYBR(:)
REAL(KIND=JPRB) :: ZRNM(NSPEC2),ZW(NDLON*(NDGLG+2))
REAL(KIND=JPRB) :: ZEPS,ZSLAPO,ZCLOPO,ZSLOPO,ZCODIL,ZPREF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTEGER(KIND=JPIM),ALLOCATABLE :: INLOPA(:),INOZPA(:),IMENG(:)
INTEGER(KIND=JPIM) :: IADL(NDGLG)
INTEGER(KIND=JPIM) :: IADR, IARI, IARP, IDEB, IINF, ILONG, IMES, &
 & IMSMAX, INBITS, INDEX1, INDEX2, INDIC, INGRIB, INIVER, INLATI, &
 & INUM, INXLON, IPOSEX, IPUILA, IREP, IREPS, ISMAX, ISTRON, ITRONC, &
 & ITYPTR, J, JJ, JX, JY

CHARACTER (LEN = 16) ::  CLNOMC

LOGICAL :: LLFICP, LLNOTF, LLKEEP, LLEXIST, LLCOSP

#include "abor1.intfb.h"
#include "cchien.intfb.h"
#include "chien.intfb.h"
#include "ereespe.intfb.h"
#include "esperee.intfb.h"
#include "reespe.intfb.h"
#include "speree.intfb.h"
#include "spreord.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RELNEW',0,ZHOOK_HANDLE)

IF (LELAM) THEN
  IADL(1:NDGLG)=NTSTAGP(1:NDGLG)-1
ELSE
  IADL(1:NDGLG)=NSTAGP(1:NDGLG)-1
ENDIF  

IREP= 0
INUM= 7
IMES= 1
IARP= 2
IARI= 0
IDEB= 0
CLNOMC='Cadre.orog      '
ZEPS=1.E-6_JPRB
LLFICP=.FALSE.

!     1. OPEN THE FILE AND CHECK GEOMETRY.
!        ---------------------------------

! Open the file with the new orography
CALL FAITOU(IREP,INUM,.TRUE.,'Neworog','OLD',.TRUE.,.TRUE.,IMES,&
 & IARP,IARI,CLNOMC)  

! Check the spectral resolution if required and limit controls
IF (NLISSP == 1) THEN
  IF (LELAM) THEN
    ALLOCATE ( INLOPA (8) )
    ALLOCATE ( INOZPA (2*NSMAX+4) )
    ALLOCATE ( ZSINLA (18) )
  ELSE
    ALLOCATE ( INLOPA (NDGNH) )
    ALLOCATE ( INOZPA (NDGNH) )
    ALLOCATE ( IMENG  (NDGNH) )
    ALLOCATE ( ZSINLA (NDGNH) )
  ENDIF
  ALLOCATE ( ZAHYBR (0:100) )
  ALLOCATE ( ZBHYBR (0:100) )
  CALL FACIES(CLNOMC,ITYPTR,ZSLAPO,ZCLOPO,ZSLOPO,ZCODIL,ITRONC,&
   & INLATI,INXLON,INLOPA,INOZPA,ZSINLA,INIVER,ZPREF,&
   & ZAHYBR,ZBHYBR,LLKEEP)

  WRITE(NULOUT,&
       &'('' IMPORTING OROGRAPHY WITH REDUCED SPECTRAL RESOLUTION '')')
  IF (LELAM) THEN
    ISMAX = NSMAX
    IMSMAX= NMSMAX
    NSMAX = ITRONC 
    NMSMAX= -ITYPTR
    WRITE(NULOUT,'('' NSMAX NMSMAX = '',2I6,'' / '',2I6)')&
         & NSMAX,NMSMAX,ISMAX,IMSMAX
  ELSE
    ISMAX = NSMAX
    NSMAX = ITRONC 
    IMENG(:)=NMENG(1:NDGNH)
    NMENG(1:NDGNH)=INOZPA(:)    
    WRITE(NULOUT,'('' NSMAX = '',I6,'' / '',I6)') NSMAX,ISMAX
  ENDIF

  DEALLOCATE ( INLOPA )
  DEALLOCATE ( INOZPA )
  DEALLOCATE ( ZSINLA )
  DEALLOCATE ( ZAHYBR )
  DEALLOCATE ( ZBHYBR )
ENDIF

! Check geometry
IF (LELAM) THEN
! ALADIN
  IINF=-2
  CALL CCHIEN(CLNOMC,INUM,IINF)
  IF (NLISSP == 1 .AND. IINF == -3) THEN
    CALL ABOR1('RELNEW : EXTENSION ZONES CANNOT DIFFER HERE ')
  ENDIF
ELSE
! ARPEGE
  IINF=-1
  CALL CHIEN(CLNOMC,NSTTYP,RMUCEN,RLOCEN,RSTRET,NSMAX,NDGLG,NDLON,&
   & NLOENG,NMENG,RMU,NHTYP,NFLEVG,VP00,VALH,VBH,&
   & NQUAD,IINF,NDGSAG,NDGENG,ZEPS,LLFICP,NULOUT)  
  IF (LLFICP) IDEB= NLOENG(1)
ENDIF

! Restore spectral resolution if required
IF (NLISSP == 1) THEN
  IF (LELAM) THEN
    NSMAX = ISMAX
    NMSMAX= IMSMAX
  ELSE
    NSMAX = ISMAX
    NMENG(1:NDGNH)=IMENG(:)
    DEALLOCATE ( IMENG )
  ENDIF
ENDIF

!     2. READ AND TRANSFORM OROGRAPHY.
!        -----------------------------

IF (LELAM) THEN
! ALADIN

! Check gridpoint orography
  CALL LFINFO(IREP,INUM,'SURFGEOPOTENTIEL',ILONG,IPOSEX)
  LLNOTF=IREP == 0.AND.ILONG == 0.AND.IPOSEX == 0
  IF (LLNOTF) THEN
!   Gridpoint orography not available : use spectral one if possible
    IF (NLISSP == 1) THEN
      CALL ABOR1('RELNEW: GRIDPOINT OROGRAPHY REQUIRED IF SMOOTHING ')
    ENDIF
!   Check spectral orography and GRIB coding
    CALL FANION(IREPS,INUM,'SPECSURF',1,'GEOPOTENTIEL',LLEXIST,LLCOSP,&
     & INGRIB,INBITS,ISTRON,IPUILA)
    IF (LLEXIST) THEN
      LDFIT=.FALSE.
!     Old spectral ordering in file
      IF (.NOT.(INGRIB == -1 .OR. INGRIB == 3)) THEN
        CALL FACILE(IREP,INUM,'SPECSURF',1,'GEOPOTENTIEL',XRNM,.TRUE.)
        XRNM(:)= XRNM(:)/RG
        CALL SPREORD(1,XRNM,ZRNM,.TRUE.)
!     New spectral ordering in file
      ELSE
        CALL FACILE(IREP,INUM,'SPECSURF',1,'GEOPOTENTIEL',ZRNM,.TRUE.)
        ZRNM(:)=ZRNM(:)/RG
        CALL SPREORD(1,XRNM,ZRNM,.FALSE.)
      ENDIF
!     Compute gridpoint orography
      CALL ESPEREE(1,1,ZRNM,XREV)
      INDIC= 0
      DO JY=NDGUNG,NDGUXG
        DO JX=NDLUNG,NDLUXG
          INDIC= INDIC+1
          IADR= JX+IADL(JY)
          PZR(INDIC)= XREV(IADR)
        ENDDO
      ENDDO
    ELSE
      CALL ABOR1('RELNEW: NO OROGRAPHY AVAILABLE ')
    ENDIF
  ELSE
!   Gridpoint orography available
    CALL FACILE(IREP,INUM,'SURF',1,'GEOPOTENTIEL',ZW,.FALSE.)
!   Reading a smoother orography : no further spectral optimization  
    IF (NLISSP == 1) THEN
      DO JY=1,NDGLG
      DO JX=1,NDLON
        IADR=JX+IADL(JY)
        XREV(IADR)=ZW((JY-1)*NDLON+JX)/RG
      ENDDO
      ENDDO
      LDFIT=.FALSE.
      CALL EREESPE(1,1,ZRNM,XREV)
      CALL SPREORD(1,XRNM,ZRNM,.FALSE.)
      CALL ESPEREE(1,1,ZRNM,XREV)
!   Standard case : spectral fit and computation of XREV outside
    ELSE
      LDFIT=.TRUE.
    ENDIF 
    IF (IINF == -3) THEN
      PZR(1:KPDG)= ZW(1:KPDG)/RG
    ELSE
      INDEX1= 0
      INDEX2= (NDGUNG-1)*NDLON+NDLUNG-1
      DO JY=1,NDGUXG-NDGUNG+1
        DO JX=1,NDLUXG-NDLUNG+1
          PZR(JX+INDEX1)= ZW(JX+INDEX2)/RG
        ENDDO
        INDEX1= INDEX1+NDLUXG-NDLUNG+1
        INDEX2= INDEX2+NDLON
      ENDDO
    ENDIF
  ENDIF

ELSE
! ARPEGE

! Check gridpoint orography
  CALL LFINFO(IREP,INUM,'SURFGEOPOTENTIEL',ILONG,IPOSEX)
  LLNOTF=IREP == 0.AND.ILONG == 0.AND.IPOSEX == 0
  IF (LLNOTF) THEN
!   Gridpoint orography not available : use spectral one if possible
    IF (NLISSP == 1) THEN
      CALL ABOR1('RELNEW: GRIDPOINT OROGRAPHY REQUIRED IF SMOOTHING ')
    ENDIF
!   Check spectral orography and GRIB coding
    CALL FANION(IREP,INUM,'SPECSURF',1,'GEOPOTENTIEL',LLEXIST,LLCOSP,&
     & INGRIB,INBITS,ISTRON,IPUILA)
    IF (LLEXIST) THEN
!     Old spectral ordering in file
      IF (.NOT.(INGRIB == -1 .OR. INGRIB == 3)) THEN
        CALL FACILE(IREP,INUM,'SPECSURF',1,'GEOPOTENTIEL',XRNM,.TRUE.)
        XRNM(:)= XRNM(:)/RG
        CALL SPREORD(1,XRNM,ZRNM,.TRUE.)
!     New spectral ordering in file
      ELSE
        CALL FACILE(IREP,INUM,'SPECSURF',1,'GEOPOTENTIEL',ZRNM,.TRUE.)
        ZRNM(:)=ZRNM(:)/RG
        CALL SPREORD(1,XRNM,ZRNM,.FALSE.)
      ENDIF
!     Compute gridpoint orography
      CALL SPEREE(1,1,ZRNM,XREV)
      INDIC=0
      DO JJ=1,NDGLG
        DO J=1,NLOENG(JJ)
          INDIC=INDIC+1
          IADR= J+IADL(JJ)
          PZR(INDIC)=XREV(IADR)
        ENDDO
      ENDDO
    ELSE
      CALL ABOR1('RELNEW: NO OROGRAPHY AVAILABLE ')
    ENDIF
  ELSE
!   Gridpoint orography available
    CALL FACILE(IREP,INUM,'SURF',1,'GEOPOTENTIEL',ZW,.FALSE.)
    PZR(1:KPDG)= ZW(1+IDEB:KPDG+IDEB)/RG
    INDIC= 0
    DO JJ=1,NDGLG
      DO J=1,NLOENG(JJ)
        INDIC= INDIC+1
        IADR= J+IADL(JJ)
        XREV(IADR)= PZR(INDIC)
      ENDDO
    ENDDO
    CALL REESPE(1,1,ZRNM,XREV)
    CALL SPREORD(1,XRNM,ZRNM,.FALSE.)
    CALL SPEREE(1,1,ZRNM,XREV)
    INDIC=0
    DO JJ=1,NDGLG
      DO J=1,NLOENG(JJ)
        INDIC= INDIC+1
        IADR= J+IADL(JJ)
        PZR(INDIC)= XREV(IADR)
      ENDDO
    ENDDO
  ENDIF
! No further spectral fit for ARPEGE
  LDFIT=.FALSE.

ENDIF

!     3. READ AND TRANSFORM LAND-SEA MASK.
!        ---------------------------------

IF (LDLSM) THEN

! ALADIN
  IF (LELAM) THEN
    CALL FACILE(IREP,INUM,'SURF',1,'IND.TERREMER',ZW,.FALSE.)
    IF (IINF == -3) THEN
      PZM(1:KPDG)= 1.0_JPRB-ZW(1:KPDG)
    ELSE
      INDEX1= 0
      INDEX2= (NDGUNG-1)*NDLON+NDLUNG-1
      DO JY=1,NDGUXG-NDGUNG+1
        DO JX=1,NDLUXG-NDLUNG+1
          PZM(JX+INDEX1)= 1.0_JPRB-ZW(JX+INDEX2)
        ENDDO
        INDEX1= INDEX1+NDLUXG-NDLUNG+1
        INDEX2= INDEX2+NDLON
      ENDDO
    ENDIF
! ARPEGE
  ELSE
    CALL FACILE(IREP,INUM,'SURF',1,'IND.TERREMER',ZW,.FALSE.)
    PZM(1:KPDG)= 1.0_JPRB-ZW(1+IDEB:KPDG+IDEB)
  ENDIF

ENDIF

CALL FAIRME(IREP,INUM,'KEEP')

IF (LHOOK) CALL DR_HOOK('RELNEW',1,ZHOOK_HANDLE)
END SUBROUTINE RELNEW
