SUBROUTINE CABIYO
!--------------------------------------------------------------------
!   CABIYO : Initialisation des ecart-types d'erreurs 
!   ------   d'observation
!  ----------------------------------------------------------------
!  But : initialisation des tableaux contenant les ecart-types
!  ---   d'erreurs d'observation pour chaque type et sous-types
!        de messages, cela pour le geopotentiel, la temperature,
!        le vent, l'humidite relative et les epaisseurs de couche.

!   Tableau generique             : ECTERO( ww , xx , yy , zz )

!     avec  ww : type d'observation ( NSYNOP, NAIREP, etc... )
!           xx : sous-type d'observation ( SYNOP, SYNOP de montagne
!                creation sol, etc... )
!           yy : parametre mesure (le 05.03.91 vaut 4,
!                                  modifie le 27.04.94, vaut 5)
!                   1  geopotentiel        en J/kg
!                   2  temperature         en degres
!                   3  vent                en m/s
!                   4  humidite relative   de 0 a 1
!                   5  SST (temperature de surface de la mer)   en degres
!lg
!                   6  quantite de neige   en kg/m**2 
!           zz : nombre de niveaux pression standard pour lesquels
!                les ecart-types d'erreur d'obs. sont disponibles.
!   Precisions :
!   ----------
!   type      code           sous-types
!   d'obs.     BDM
!  ----------------------------------------------------------------
!   NSYNOP
!     xx = 2  1  SYNOP et SYNOR
!             2  SHIP
!     yy -->  Z - T - u/v - Hu - SST   (1-2-3-4-5)
!lg
!     yy -->  Z - T - u/v - Hu - SST - SN  (1-2-3-4-5-6)
!  ----------------------------------------------------------------
!   NAIREP
!     xx = 1  1  AIREP, AMDAR et ACAR
!     yy -->  u/v   (3)
!  ----------------------------------------------------------------
!   NSATOB
!     xx = 6  1  satellite de la CEE (METEOSAT)
!             2  satellite des Etats-Unis d'Amerique (GOES)
!             3  satellite japonais
!             4  satellite indien
!             5  autres satellites geostationnaire
!     yy -->  u/v   (3)
!  ----------------------------------------------------------------
!   NDRIBU
!     xx = 1  1  BUOY, BATHY et TESAC 
!     yy -->  Z - T - u/v - SST   (1-2-3-5)
!  ----------------------------------------------------------------
!   NTEMP
!     xx = 1  1  tout TEMP 
!     yy -->  Z - T - u/v - Hu   (1-2-3-4)
!  ----------------------------------------------------------------
!   NPILOT
!     xx = 1  1  tout PILOT 
!     yy -->  u/v   (3)
!  ----------------------------------------------------------------
!   NSATEM
!                93   SATEM 500 km
!               110   tous satellites haute resolution
!               111   SATEM 250 km
!               112   SATEM 120 km
!  ----------------------------------------------------------------
!   NPAOB
!     aucune initialisation
!  ----------------------------------------------------------------
!   NSCATT
!     aucune initialisation
!  ----------------------------------------------------------------
!   NRAWRA
!     aucune initialisation
!  ----------------------------------------------------------------
!  Sequence d'appel     : CALL CABIYO
!  ----------------
!  Sous-programme amont : CAMELO
!  --------------------
!  COMMON utilises      : QAPASS - QAREF  - QAETEO  -
!  ---------------        YOMLUN - YOMCST - PARDIMO - YOMOCTP
!--------------------------------------------------------------------
!   Auteur :      Ph.   Caille  04/90, V.Casse 04/94, 11/95
!   Modif  :      J.-N. Thepaut 10/97, recalibrage des sigma o.
!   Mofif  :      LG            03/00, rms error for snow initialized 
!--------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE QAPASS   , ONLY : JPNVS
USE QAREF    , ONLY : REF_STAT
USE QAETEO   , ONLY : ECTERO   ,ETEOST   ,ETEOHU
USE YOMCST   , ONLY : RG
USE YOMCOCTP , ONLY : NSYNOP   ,NAIREP   ,NSATOB   ,NDRIBU   ,&
 & NTEMP    ,NPILOT 

IMPLICIT NONE

REAL(KIND=JPRB) ::  ZCOEF(5), ZEPST

INTEGER(KIND=JPIM) :: IJ, J1, JB, JH, JI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "qastat.h"

!--------------------------------------------------------------------
!  - 1 -  Initialisation par defaut
!         -------------------------

IF (LHOOK) CALL DR_HOOK('CABIYO',0,ZHOOK_HANDLE)
ECTERO = 9999.99_JPRB
ETEOST = 9999.99_JPRB

!--------------------------------------------------------------------
!  - 2 -  Alimentation tableau ECTERO
!         ---------------------------

!  1 = geopotentiel
!  2 = temperature
!  3 = composante du vent
!  4 = humidite relative
!  5 = SST
!  6 = quantite de neige

!  SYNOP
!  -----
ECTERO(NSYNOP,1,1,1) = 8.0_JPRB * RG
ECTERO(NSYNOP,1,2,1) = 1.4_JPRB
ECTERO(NSYNOP,1,3,1) = 2.0_JPRB
ECTERO(NSYNOP,1,4,1) = 0.10_JPRB
ECTERO(NSYNOP,1,5,1) = 1.5_JPRB
ECTERO(NSYNOP,1,6,1) = 5.0_JPRB

!  SHIP
!  ----
ECTERO(NSYNOP,2,1,1) = 8.0_JPRB * RG
ECTERO(NSYNOP,2,2,1) = 1.4_JPRB
ECTERO(NSYNOP,2,3,1) = 3.0_JPRB
ECTERO(NSYNOP,2,4,1) = 0.10_JPRB
ECTERO(NSYNOP,2,5,1) = 1.5_JPRB

!  DRIBU
!  -----
ECTERO(NDRIBU,1,1,1) = 8.0_JPRB * RG
ECTERO(NDRIBU,1,2,1) = 1.4_JPRB
ECTERO(NDRIBU,1,3,1) = 3.0_JPRB
ECTERO(NDRIBU,1,5,1) = 1.5_JPRB

!  1 METEOSAT
!  2 GOES
!  3 satellite japonais
!  4 satellite indien
!  5 autre

ZCOEF(1) = 1.1_JPRB
ZCOEF(2) = 1.1_JPRB
ZCOEF(3) = 1.3_JPRB
ZCOEF(4) = 1.4_JPRB
ZCOEF(5) = 1.5_JPRB

! AIREP
! ----

!           Vent

ECTERO(NAIREP,1,3,1)=2.5_JPRB
ECTERO(NAIREP,1,3,2)=2.5_JPRB
ECTERO(NAIREP,1,3,3)=2.5_JPRB
ECTERO(NAIREP,1,3,4)=2.6_JPRB
ECTERO(NAIREP,1,3,5)=2.7_JPRB
ECTERO(NAIREP,1,3,6)=2.8_JPRB
ECTERO(NAIREP,1,3,7)=3.2_JPRB
ECTERO(NAIREP,1,3,8)=3.4_JPRB
ECTERO(NAIREP,1,3,9)=3.8_JPRB
ECTERO(NAIREP,1,3,10)=4.0_JPRB
ECTERO(NAIREP,1,3,11)=4.1_JPRB
ECTERO(NAIREP,1,3,12)=4.2_JPRB
ECTERO(NAIREP,1,3,13)=4.2_JPRB
ECTERO(NAIREP,1,3,14)=4.2_JPRB
ECTERO(NAIREP,1,3,15)=4.2_JPRB
ECTERO(NAIREP,1,3,16)=4.3_JPRB
ECTERO(NAIREP,1,3,17)=4.6_JPRB
ECTERO(NAIREP,1,3,18)=4.8_JPRB
ECTERO(NAIREP,1,3,19)=5.0_JPRB

!           temperature

ECTERO(NAIREP,1,2,1)=1.90_JPRB
ECTERO(NAIREP,1,2,2)=1.80_JPRB
ECTERO(NAIREP,1,2,3)=1.70_JPRB
ECTERO(NAIREP,1,2,4)=1.60_JPRB
ECTERO(NAIREP,1,2,5)=1.50_JPRB
ECTERO(NAIREP,1,2,6)=1.40_JPRB
ECTERO(NAIREP,1,2,7)=1.40_JPRB
ECTERO(NAIREP,1,2,8)=1.40_JPRB
ECTERO(NAIREP,1,2,9)=1.40_JPRB
ECTERO(NAIREP,1,2,10)=1.50_JPRB
ECTERO(NAIREP,1,2,11)=1.60_JPRB
ECTERO(NAIREP,1,2,12)=1.70_JPRB
ECTERO(NAIREP,1,2,13)=1.80_JPRB
ECTERO(NAIREP,1,2,14)=1.90_JPRB
ECTERO(NAIREP,1,2,15)=2.00_JPRB
ECTERO(NAIREP,1,2,16)=2.10_JPRB
ECTERO(NAIREP,1,2,17)=2.20_JPRB
ECTERO(NAIREP,1,2,18)=2.30_JPRB
ECTERO(NAIREP,1,2,19)=2.40_JPRB

! TEMP
! ----

!           Vent

ECTERO(NTEMP,1,3,1)=2.3_JPRB
ECTERO(NTEMP,1,3,2)=2.3_JPRB
ECTERO(NTEMP,1,3,3)=2.3_JPRB
ECTERO(NTEMP,1,3,4)=2.4_JPRB
ECTERO(NTEMP,1,3,5)=2.5_JPRB
ECTERO(NTEMP,1,3,6)=2.5_JPRB
ECTERO(NTEMP,1,3,7)=2.8_JPRB
ECTERO(NTEMP,1,3,8)=3.0_JPRB
ECTERO(NTEMP,1,3,9)=3.3_JPRB
ECTERO(NTEMP,1,3,10)=3.6_JPRB
ECTERO(NTEMP,1,3,11)=3.7_JPRB
ECTERO(NTEMP,1,3,12)=3.8_JPRB
ECTERO(NTEMP,1,3,13)=3.8_JPRB
ECTERO(NTEMP,1,3,14)=3.8_JPRB
ECTERO(NTEMP,1,3,15)=3.8_JPRB
ECTERO(NTEMP,1,3,16)=3.9_JPRB
ECTERO(NTEMP,1,3,17)=4.1_JPRB
ECTERO(NTEMP,1,3,18)=4.3_JPRB
ECTERO(NTEMP,1,3,19)=4.5_JPRB

!          Temperature

ECTERO(NTEMP,1,2,1)=1.7_JPRB
ECTERO(NTEMP,1,2,2)=1.6_JPRB
ECTERO(NTEMP,1,2,3)=1.5_JPRB
ECTERO(NTEMP,1,2,4)=1.4_JPRB
ECTERO(NTEMP,1,2,5)=1.4_JPRB
ECTERO(NTEMP,1,2,6)=1.3_JPRB
ECTERO(NTEMP,1,2,7)=1.3_JPRB
ECTERO(NTEMP,1,2,8)=1.3_JPRB
ECTERO(NTEMP,1,2,9)=1.3_JPRB
ECTERO(NTEMP,1,2,10)=1.4_JPRB
ECTERO(NTEMP,1,2,11)=1.5_JPRB
ECTERO(NTEMP,1,2,12)=1.5_JPRB
ECTERO(NTEMP,1,2,13)=1.6_JPRB
ECTERO(NTEMP,1,2,14)=1.6_JPRB
ECTERO(NTEMP,1,2,15)=1.6_JPRB
ECTERO(NTEMP,1,2,16)=1.7_JPRB
ECTERO(NTEMP,1,2,17)=1.8_JPRB
ECTERO(NTEMP,1,2,18)=1.9_JPRB
ECTERO(NTEMP,1,2,19)=2.0_JPRB

!         Geopotentiel

ECTERO(NTEMP,1,1,1)=8.0_JPRB * RG
ECTERO(NTEMP,1,1,2)=8.2_JPRB * RG
ECTERO(NTEMP,1,1,3)=8.6_JPRB * RG
ECTERO(NTEMP,1,1,4)=9.0_JPRB * RG
ECTERO(NTEMP,1,1,5)=9.4_JPRB * RG
ECTERO(NTEMP,1,1,6)=9.9_JPRB * RG
ECTERO(NTEMP,1,1,7)=11.4_JPRB * RG
ECTERO(NTEMP,1,1,8)=12.7_JPRB * RG
ECTERO(NTEMP,1,1,9)=14.0_JPRB * RG
ECTERO(NTEMP,1,1,10)=16.0_JPRB * RG
ECTERO(NTEMP,1,1,11)=15.7_JPRB * RG
ECTERO(NTEMP,1,1,12)=17.2_JPRB * RG
ECTERO(NTEMP,1,1,13)=20.1_JPRB * RG
ECTERO(NTEMP,1,1,14)=22.0_JPRB * RG
ECTERO(NTEMP,1,1,15)=24.4_JPRB * RG
ECTERO(NTEMP,1,1,16)=27.0_JPRB * RG
ECTERO(NTEMP,1,1,17)=30.0_JPRB * RG
ECTERO(NTEMP,1,1,18)=31.5_JPRB * RG
ECTERO(NTEMP,1,1,19)=36.0_JPRB * RG

DO J1 = 1 , JPNVS
  DO JI=1,5
    ECTERO(NSATOB,JI,3,J1) = REF_STAT(J1,4) * ZCOEF(JI)
  ENDDO
  ECTERO(NTEMP,1,4,J1) = 0.12_JPRB
  ECTERO(NPILOT,1,3,J1) = ECTERO(NTEMP,1,3,J1)
ENDDO

!--------------------------------------------------------------------
!  - 3 -  Alimentation tableaux ETEOST et ETEOHU
!         --------------------------------------

ZEPST = 1.5_JPRB
IJ    = 0

DO JB = 1 , JPNVS-1
  DO JH = JB+1 , JPNVS
    IJ = IJ + 1
    ETEOST(IJ) = ZEPST * FSIGDZ(REF_STAT(JB,2),REF_STAT(JH,2),&
     & LOG(REF_STAT(JB,1)),LOG(REF_STAT(JH,1)),REF_STAT(JH,6))  
  ENDDO
ENDDO

ETEOHU(1) = 0.20_JPRB
ETEOHU(2) = 0.17_JPRB
ETEOHU(3) = 0.14_JPRB

ETEOST(1) = 69._JPRB * RG
ETEOST(2) = 32._JPRB * RG
ETEOST(3) = 40._JPRB * RG
ETEOST(4) = 49._JPRB * RG
ETEOST(5) = 23._JPRB * RG
ETEOST(6) = 16._JPRB * RG
ETEOST(7) = 18._JPRB * RG

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CABIYO',1,ZHOOK_HANDLE)
END SUBROUTINE CABIYO
