SUBROUTINE CALINA ( PMAT, PBBB, POIDS, KDIMAT, KDIBBB, KDIPOI, &
 & KNPRM, KNV, KITER)  

!****-------------------------------------------------------------------
!****    CALINA : Resolution du systeme lineaire  de l'interpolation
!****    ------   optimale.
!****
!****   Auteurs : MCP 02/90; PB 06/91; PC 12/91; VC 08/92
!****-------------------------------------------------------------------
!***
!*** But : resolution du systeme lineaire de l'interpolation optimale
!*** ---   Utilisation  du  sous-programme  MINV. Eventuellement,  on
!***       augmente l'effet de pepite de la matrice pour faciliter la
!***       convergence lorsque le systeme est mal conditionne.
!***
!*** Sequence d'appel     : CALL CALINA ( PMAT   , PBBB   , POIDS  ,
!*** ----------------                     KDIMAT , KDIBBB , KDIPOI ,
!***                                      KNPRM  , KNV    , KITER  )
!***
!*** Arguments d'entree   :
!*** ------------------
!***   PMAT   : matrice carree (premier membre du systeme lineaire)
!***   PBBB   : matrice rectangulaire (second membre du systeme lineaire)
!***   KDIMAT : espacement entre deux colonnes consecutives de PMAT
!***   KDIBBB : espacement entre deux colonnes consecutives de PBBB
!***   KDIPOI : espacement entre deux colonnes consecutives de POIDS
!***   KNPRM  : nombre total de predicteurs
!***            = rang de la sous-matrice utile de PMAT
!***            = nombre de lignes utiles de PBBB (et de POIDS)
!***   KNV    : nombre total de predictants
!***            = nombre de colonnes de PBBB (et de POIDS)
!***
!*** Arguments de sortie  :
!*** -------------------
!***   POIDS  : coefficients, solutions du systeme lineaire
!***   KITER  : nombre d'iterations necessaires pour la resolution du
!***            systeme lineaire (negatif si divergence de l'algorithme)
!***
!*** Sous-programme amont : CANADA
!*** --------------------
!***
!*** Sous-programmes aval : MINV
!*** --------------------
!***
!*** COMMON utilises      : 
!*** ---------------
!***
!*** Algorithme : (idem EMERAUDE)
!*** ----------
!***
!*** Verification du bon conditionnement du systeme lineaire :
!*** -------------------------------------------------------
!***   On fait l'hypothese que  les valeurs propres v1,v2,..,vN de la
!***   matrice du systeme sont en progression geometrique: v, k.v,..,
!***   k**(N-1).v . Ceci est  legitime puisque la matrice en question
!***   est grosso-modo une matrice de correlation donc diagonalisable
!***   et que  la matrice diagonalisee  represente la matrice du sys-
!***   teme  que l'on  aurait a  resoudre  en  presence  du fameux et
!***   mythique  "reseau  equivalent  d'observations  independantes":
!***   dans ce cas, l'influence propre de chaque station independante
!***   decroit rapidement et l'hypothese de  decroissance geometrique
!***   est raisonnable.
!***   Le critere de bon conditionnement est choisi egal a:
!***                C = ( DET / Trace**N ) ** 1/(N-1)
!***   Dans le cas de valeurs propres a progression geometrique, C ne
!***   depend que de k et de N, ce qui permet de le tabuler. Des lors
!***   on  introduit  alpha ( alpha = k**(N-1) ), rapport  entre  les
!***   valeurs  propres extremes  (ex:10-5), et on  tabule pour cette
!***   valeur (d'ou le tableau ZSTABL).
!***
!***   Utilisation pratique: on calcule C pour  la matrice du systeme
!***   lineaire a resoudre. C nous fournit  indirectement une estima-
!***   tion de alpha  (si l'hypothese de decroissance geometrique est
!***   verifiee).  La matrice  est donc dite  bien conditionnee  si C
!***   calcule est plus grand C tabule pour la  valeur N du nombre de
!***   predicteurs.
!****-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDIMAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIBBB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIPOI 
INTEGER(KIND=JPIM)               :: KNV! UNDETERMINED INTENT 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMAT(KDIMAT,KDIMAT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBBB(KDIBBB,KNV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POIDS(KDIPOI,KNV) 
INTEGER(KIND=JPIM)               :: KNPRM ! UNDETERMINED INTENT
INTEGER(KIND=JPIM),INTENT(OUT)   :: KITER 
REAL(KIND=JPRB) :: ZMAT(KNPRM,KNPRM+KNV), ZSCRA(KNPRM,3)

INTEGER(KIND=JPIM) :: JPITER
INTEGER(KIND=JPIM) :: JPMODE

REAL(KIND=JPRB) :: PPEPSI

!  nombre maximum de tentatives de resolution du systeme lineaire
PARAMETER ( JPITER = 3 )

!  mode de fonctionnement de MINV
PARAMETER ( JPMODE = 0 )

!  critere de convergence pour MINV (valeurs minimales pour le
!  determinant et le critere de convergence)
PARAMETER ( PPEPSI = 1.E-306_JPRB  )

!  tableau local contenant le critere d'arret pour des matrices de taille =<100

REAL(KIND=JPRB) , DIMENSION(100)  :: ZSTABL=(/&
 & 1.000000000000000_JPRB,  .000009999800003_JPRB,  .000176985104618_JPRB,&
 & .000450873787167_JPRB,  .000697558505210_JPRB,  .000881234583606_JPRB,&
 & .001006712169895_JPRB,  .001087389012060_JPRB,  .001135672976177_JPRB,&
 & .001161105240316_JPRB,  .001170588136043_JPRB,  .001169009292691_JPRB,&
 & .001159810100111_JPRB,  .001145415945484_JPRB,  .001127542193561_JPRB,&
 & .001107406974641_JPRB,  .001085878307432_JPRB,  .001063576016744_JPRB,&
 & .001040942735858_JPRB,  .001018293737582_JPRB,  .000995852189159_JPRB,&
 & .000973774297967_JPRB,  .000952167388122_JPRB,  .000931102991627_JPRB,&
 & .000910626393716_JPRB,  .000890763635619_JPRB,  .000871526679891_JPRB,&
 & .000852917238133_JPRB,  .000834929618318_JPRB,  .000817552849022_JPRB,&
 & .000800772267260_JPRB,  .000784570706371_JPRB,  .000768929384300_JPRB,&
 & .000753828566573_JPRB,  .000739248059233_JPRB,  .000725167573121_JPRB,&
 & .000711566990600_JPRB,  .000698426558208_JPRB,  .000685727023059_JPRB,&
 & .000673449726536_JPRB,  .000661576665611_JPRB,  .000650090529719_JPRB,&
 & .000638974719226_JPRB,  .000628213350180_JPRB,  .000617791248884_JPRB,&
 & .000607693939084_JPRB,  .000597907623848_JPRB,  .000588419163783_JPRB,&
 & .000579216052816_JPRB,  .000570286392487_JPRB,  .000561618865484_JPRB,&
 & .000553202708936_JPRB,  .000545027687891_JPRB,  .000537084069259_JPRB,&
 & .000529362596435_JPRB,  .000521854464760_JPRB,  .000514551297903_JPRB,&
 & .000507445125245_JPRB,  .000500528360282_JPRB,  .000493793780065_JPRB,&
 & .000487234505671_JPRB,  .000480843983688_JPRB,  .000474615968682_JPRB,&
 & .000468544506623_JPRB,  .000462623919222_JPRB,  .000456848789152_JPRB,&
 & .000451213946098_JPRB,  .000445714453611_JPRB,  .000440345596704_JPRB,&
 & .000435102870175_JPRB,  .000429981967594_JPRB,  .000424978770930_JPRB,&
 & .000420089340773_JPRB,  .000415309907129_JPRB,  .000410636860735_JPRB,&
 & .000406066744879_JPRB,  .000401596247689_JPRB,  .000397222194860_JPRB,&
 & .000392941542795_JPRB,  .000388751372136_JPRB,  .000384648881660_JPRB,&
 & .000380631382516_JPRB,  .000376696292785_JPRB,  .000372841132342_JPRB,&
 & .000369063517998_JPRB,  .000365361158918_JPRB,  .000361731852278_JPRB,&
 & .000358173479164_JPRB,  .000354684000690_JPRB,  .000351261454329_JPRB,&
 & .000347903950427_JPRB,  .000344609668916_JPRB,  .000341376856188_JPRB,&
 & .000338203822140_JPRB,  .000335088937369_JPRB,  .000332030630506_JPRB,&
 & .000329027385699_JPRB,  .000326077740213_JPRB,  .000323180282154_JPRB,&
 & .000320333648307_JPRB /)  

SAVE ZSTABL

INTEGER(KIND=JPIM) :: ILIMIT, J, JC, JJ, JL

REAL(KIND=JPRB) :: ZDET, ZTRA
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!**---------------------------------------------------------------------
!**  - 1 -  INITIALISATIONS DIVERSES
!**         ------------------------

IF (LHOOK) CALL DR_HOOK('CALINA',0,ZHOOK_HANDLE)
KITER = 1
ZDET  = 0.0_JPRB
ILIMIT = MIN( KNPRM , 100 )
ZTRA=SUM( (/ (PMAT(J,J),J=1,KNPRM) /) ) ** REAL(-KNPRM,JPRB)

!**---------------------------------------------------------------------
!**  - 2 -  RESOLUTION DU SYSTEME APRES TEST DE CONDITIONNEMENT
!**         ET "AMELIORATION" DU SYSTEME
!**         ---------------------------------------------------

200 CONTINUE

! initialisation du tableau ZMAT utilise par MINV en entree et en sortie
DO JL = 1,KNPRM
  DO JC = 1,KNPRM
    ZMAT(JL,JC) = PMAT(JL,JC)
  ENDDO
  DO JC = 1,KNV
    ZMAT(JL,KNPRM+JC) = PBBB(JL,JC)
  ENDDO
ENDDO

CALL MINV(ZMAT, KNPRM, KNPRM, ZSCRA, ZDET, PPEPSI, KNV, JPMODE)

! si la matrice est mal conditionnee, on augmente l'effet de pepite
! en accroissant les elements diagonaux

IF (KNPRM > 1) THEN
  IF ( ZDET <= PPEPSI .OR. &
     & ZTRA*ZDET < ZSTABL(ILIMIT)**REAL(KNPRM-1,JPRB) ) THEN  
    IF ( KITER == JPITER ) THEN
      KITER = -JPITER
      GOTO 300
    ENDIF
    KITER = KITER + 1
    ZTRA  = ZTRA * 1.1_JPRB ** REAL(-KNPRM,JPRB)
    DO JJ = 1,KNPRM
      PMAT(JJ,JJ) = PMAT(JJ,JJ) * 1.1_JPRB
    ENDDO
    GOTO 200
  ENDIF
ENDIF

!**---------------------------------------------------------------------
!**  - 3 -  ECRITURE DU RESULTAT
!**         --------------------

300 CONTINUE

DO JL = 1,KNPRM
  DO JC = 1,KNV
    POIDS(JL,JC) = ZMAT(JL,KNPRM+JC)
  ENDDO
ENDDO

!**---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CALINA',1,ZHOOK_HANDLE)
END SUBROUTINE CALINA
