SUBROUTINE CATRMA (KTASK, KASELS, KOSELS, KNSELS, KNOUT,PCOVAR, PRESID, KNIV)
!--------------------------------------------------------------------------
!   CATRMA : Construction de la matrice du systeme lineaire
!   ------
!--------------------------------------------------------------------------
!  terme generique de la matrice :   RG(I,J) + CO(I,J)/SIGG(I)/SIGG(J)

!  avec  RG   : correlations entre erreurs du guess aux points d'obs.
!        CO   : covariances  entre erreurs d'obs.
!        SIGG : ecart-types d'erreurs de guess aux points d'obs.

!  INTERFACE: CALL CATRMA (KTASK,  KASELS, KOSELS, KNSELS, KNOUT,
!  ---------               PCOVAR, PRESID, KNIV)

!  ENTREE : KTASK : numero de la tache executant CATRMA
!  ------   KASELS: tableau contenant, pour chaque type de predicteur
!                   de la table NVY, les adresses des predicteurs
!           KOSELS: adresse de l'observation contenant le predicteur
!                   situe a l'adresse KASELS
!           KNSELS: nombre effectif de predicteurs de chaque type
!           KNOUT : nombre total de predicteurs (somme des KNSELS)
!                   dimension utile de la matrice de covariances
!           KNIV  : numero du niveau vertical

!  SORTIE : PCOVAR: tableau des "covariances" calculees
!  ------   PRESID: tableau des residus observes normes par l'ecart-
!                   type d'erreur du guess

!  COMMON UTILISES: YOMCT0 YOMOBA QACOST YOMCST QALORI
!  ---------------

!  DESCRIPTION:     les covariances PCOVAR sont calculees dans l'ordre
!  -----------      Z,T,U,V,DZ,H,Q,SST,SN,T2,H2,U1,V1 des predicteurs.
!                   Cela correspond a l'ordre de NVX et NVY et au rangement
!                   des tableaux de selection.
!                   PCOVAR pourra etre utilise pour construire la matrice
!                   du systeme lineaire. On calcule les covariances entre
!                   erreurs du guess et les covariances entre erreurs
!                   d'observation dans le meme module pour eviter de
!                   doubler des calculs de distance.

!                   On prend garde de ne faire les calculs que pour
!                   le triangle inferieur de la matrice qui est symetrique.

!                   Par ailleurs, on calcule les residus normes pendant qu'on
!                   a sous la main les bons ecart-types d'erreur du guess

!  MODIFICATIONS:   LG 03/00  p.9 for snow added
!  -------------    LG 11/00  vert. part of the covar. for snow added
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!---------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE QAPABL   , ONLY : JPKZMA   ,JPNPM2
USE QAPAVU   , ONLY : JPNVY    ,JPPYZ    ,JPPYT    ,JPPYU    ,&
 & JPPYV    ,JPPYDZ   ,JPPYHU   ,JPPYST   ,JPPYU1   ,&
 & JPPYV1   ,JPPYT2   ,JPPYH2   ,JPPYSN  
USE YOMDB
USE YOMCOCTP , ONLY : NSATOB   ,NTEMP    ,NPILOT   ,NSATEM
USE YOMCT0   , ONLY : NTASKS
USE YOMCST   , ONLY : RA
USE QACOST   , ONLY : QPOHZ    ,QPOHUD   ,QPOSST   ,QRMU     ,&
 & QRNU     ,QPOHHU   ,QPOVZ    ,QPOVH    ,QPOHT2   ,&
 & QPOHH2   ,QPOVORBL ,QPODIVBL ,QNUBL    ,QKPBL    ,&
 & QPOHSN   ,QPOHPSN  
USE QALORI   , ONLY : LCORRF

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KTASK 
INTEGER(KIND=JPIM),INTENT(IN)    :: KASELS(JPNVY,JPKZMA) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOSELS(JPNVY,JPKZMA) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNSELS(JPNVY) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNOUT 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOVAR(JPNPM2,JPNPM2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRESID(JPNPM2) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNIV 
INTEGER(KIND=JPIM) :: IREPER(JPNPM2)
REAL(KIND=JPRB) ::    ZFG(JPNPM2), ZDIST2(JPNPM2,JPNPM2)
LOGICAL :: LLSATO, LLTEMP, LLPILO, LLSATM

INTEGER(KIND=JPIM) :: IAD, IADO, IADOX, IADX, IADY, IIV, INIV, INSAT, &
 & INSATX, &
 & IPRX, IPRY, ISAT1, ISAT2, ISATX1, &
 & ISATX2, IX, IX1, IX2, IY, J1SEL, J1VAR, J2SEL, &
 & J2VAR, JJPRY, JJRE, JSEL, JSEX, JSEY, JX, JY  

REAL(KIND=JPRB) :: ZCOSPX, ZCOSPY, ZH2DIV, ZH2HU, ZH2SST, ZH2Z,&
 & ZKDZO7, ZKH, ZKHO5, ZKUO5, ZKUO6, ZKZ, ZKZO5, &
 & ZLAT, ZLATX, ZLON, ZLONX, ZMU, ZNU2, ZOB, &
 & ZPDZO7, ZPRESB, ZPRESS, ZPRUSS, ZRE, ZSIGB, &
 & ZSIGS, ZSINPX, ZSINPY, ZSINUS  
REAL(KIND=JPRB) :: ZH2T2, ZH2H2, ZH2VORBL, ZH2DIVBL, ZNU2BL, ZKPBL, ZH2SN, ZDISTP2, ZV2P2SN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "qastat.h"
#include "qastob.h"
#include "qadits.h"
#include "qabitu.h"
#include "openmp_obs.h"

!---------------------------------------------------------------------------
!  - 1 - Preambule: mise a zero des tableaux et initialisation des
!        constantes pour les correlations entre erreurs de l'ebauche.
!        precalcul de la matrice des interdistances entre predicteurs.
!        ------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CATRMA',0,ZHOOK_HANDLE)
DO JX = 1 , KNOUT
  DO JY = 1 , KNOUT
    PCOVAR(JY,JX) = 0.0_JPRB
    ZDIST2(JY,JX) = 0.0_JPRB
  ENDDO
ENDDO
DO JX = 1 , KNOUT
  IREPER(JX) = 0
  PRESID(JX) = 0.0_JPRB
ENDDO

!  initialisation des constantes pour les fonctions d'autocorrelation

IF (KTASK < 1.OR. KTASK > NTASKS)  CALL ABOR1('CATRMA')
INIV = IABS(KNIV)
IIV  = ( INIV + KNIV ) * 0.5_JPRB
ZH2Z   = QPOHZ(IIV,KTASK)**2
ZH2DIV = QPOHUD(IIV,KTASK)**2
ZH2SST = QPOSST(KTASK)**2
ZH2HU  = QPOHHU(IIV,KTASK)**2
ZKZ    = QPOVZ(INIV,KTASK)
ZKH    = QPOVH(INIV,KTASK)
ZNU2   = QRNU(IIV,KTASK)**2
ZMU    = QRMU(IIV,KTASK)

ZH2T2    = QPOHT2(KTASK) ** 2
ZH2H2    = QPOHH2(KTASK) ** 2
ZH2SN    = QPOHSN(KTASK) ** 2
ZH2VORBL = QPOVORBL(KTASK) ** 2
ZH2DIVBL = QPODIVBL(KTASK) ** 2
ZNU2BL   = QNUBL(KTASK) ** 2
ZKPBL    = QKPBL(KTASK)
ZV2P2SN  = QPOHPSN(KTASK) ** 2

ZKZO5 = ZKZ * 2.0_JPRB
ZKHO5 = ZKH * 2.0_JPRB
ZKUO5 = ZKZ * 2.0_JPRB
ZKUO6 = ZKZ * 2.0_JPRB
ZKDZO7= ZKZ * 2.0_JPRB

ZPDZO7= ZH2Z * 0.64_JPRB

!  initialisation du tableau IREPER:
!   IREPER(IX2)=0   si IX2 est le numero d'un predicteur qui appartient a une
!                                        obs rencontree pour la premiere fois
!   IREPER(IX2)=IX1 si IX2 appartient a la meme obs que IX1
! en sortie IX1 est le nombre d'obs. differentes dont provient les predicteurs

IX1 = 0
DO J1VAR = 1 , JPNVY
  DO J1SEL = 1 , KNSELS(J1VAR)
    IX1 = IX1 + 1
    IF (IREPER(IX1) == 0) THEN
      IX2 = IX1
      DO J2SEL = J1SEL+1 , KNSELS(J1VAR)
        IX2 = IX2 + 1
        IF (KOSELS(J1VAR,J2SEL) == KOSELS(J1VAR,J1SEL))IREPER(IX2)=IX1
      ENDDO
      DO J2VAR = J1VAR+1 , JPNVY
        DO J2SEL = 1 , KNSELS(J2VAR)
          IX2 = IX2 + 1
          IF (KOSELS(J2VAR,J2SEL) == KOSELS(J1VAR,J1SEL))IREPER(IX2)=IX1
        ENDDO
      ENDDO
    ENDIF
  ENDDO
ENDDO

! precalcul de la matrice des interdistances (en evitant de multiplier
! les calculs grace au tableau IREPER).

! etape 1: calcul de la partie superieure de la matrice
!          (on ne calcule que si c'est la 1ere fois (IREPER=0))
IX = 0
DO J1VAR = 1 , JPNVY
  DO JSEX = 1 , KNSELS(J1VAR)
    IX = IX + 1
    IY = IX
    IADOX = KOSELS(J1VAR,JSEX)
    ZLATX = ROBHDR(IADOX,MDBLAT)
    ZLONX = ROBHDR(IADOX,MDBLON)
    DO JSEY = JSEX+1 , KNSELS(J1VAR)
      IY = IY + 1
      IF (IREPER(IY) == 0) THEN
        IADO = KOSELS(J1VAR,JSEY)
        ZLAT = ROBHDR(IADO,MDBLAT)
        ZLON = ROBHDR(IADO,MDBLON)
        ZDIST2(IY,IX) = CADIT2(ZLATX,ZLONX,ZLAT,ZLON)
      ENDIF
    ENDDO
    DO JJPRY = J1VAR+1 , JPNVY
      DO JSEY = 1 , KNSELS(JJPRY)
        IY = IY + 1
        IF (IREPER(IY) == 0) THEN
          IADO = KOSELS(JJPRY,JSEY)
          ZLAT = ROBHDR(IADO,MDBLAT)
          ZLON = ROBHDR(IADO,MDBLON)
          ZDIST2(IY,IX) = CADIT2(ZLATX,ZLONX,ZLAT,ZLON)
        ENDIF
      ENDDO
    ENDDO
  ENDDO
ENDDO

! etape 2: mise a jour pour IREPER different de 0 et
!          symetrisation de la matrice
IX = 0
DO J1VAR = 1 , JPNVY
  DO JSEX = 1 , KNSELS(J1VAR)
    IX = IX + 1
    IY = IX
!ocl novrec
!DIR$ IVDEP
    DO JSEY = JSEX+1 , KNSELS(J1VAR)
      IY = IY + 1
      IF (IREPER(IY) == 0) THEN
        ZDIST2(IX,IY) = ZDIST2(IY,IX)
      ELSE
        ZDIST2(IY,IX) = ZDIST2(IREPER(IY),IX)
        ZDIST2(IX,IY) = ZDIST2(IY,IX)
      ENDIF
    ENDDO
    DO JJPRY = J1VAR+1 , JPNVY
!ocl novrec
!DIR$ IVDEP
      DO JSEY = 1 , KNSELS(JJPRY)
        IY = IY + 1
        IF (IREPER(IY) == 0) THEN
          ZDIST2(IX,IY) = ZDIST2(IY,IX)
        ELSE
          ZDIST2(IY,IX) = ZDIST2(IREPER(IY),IX)
          ZDIST2(IX,IY) = ZDIST2(IY,IX)
        ENDIF
      ENDDO
    ENDDO
  ENDDO
ENDDO

!---------------------------------------------------------------------------
!  - 2 - Calcul des covariances entre les predicteurs Z
!        et les autres predicteurs.
!        ----------------------------------------------

!  IX et IY serviront a reperer les elements de pcovar au cours des calculs

IX = 0

IPRX = JPPYZ
DO JSEX = 1 , KNSELS(IPRX)
  IX      = IX + 1
  IY      = IX
  IADOX   = KOSELS(IPRX,JSEX)
  ZLATX   = ROBHDR(IADOX,MDBLAT)
  ZLONX   = ROBHDR(IADOX,MDBLON)
  LLTEMP  = ROBHDR(IADOX,MDBOTP) == NTEMP
  IADX    = KASELS(IPRX,JSEX)
  ZPRESS  = ROBODY(IADX,MDBRBPIO)
  ZFG(IX) = ROBODY(IADX,MDBFGE)

  IPRY = JPPYZ
  DO JSEY = JSEX+1 , KNSELS(IPRY)
    IY   = IY + 1
    IADO = KOSELS(IPRY,JSEY)
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHZZ(ZDIST2(IX,IY),ZH2Z)&
     & *FRVZZ(ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZ)  
    IF (LLTEMP .AND. IADO == IADOX) THEN
      PCOVAR(IY,IX) = PCOVAR(IY,IX)+FCZZO5(ROBODY(IADX,MDBOER),&
       & ROBODY(IADY,MDBOER),ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZO5)&
       & / (ZFG(IX)*ROBODY(IADY,MDBFGE))  
    ENDIF
  ENDDO

  IPRY = JPPYT
  DO JSEY = 1 , KNSELS(IPRY)
    IY   = IY + 1
    IADO = KOSELS(IPRY,JSEY)
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHZT(ZDIST2(IX,IY),ZH2Z)&
     & *FRVZT(ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZ)  
    IF (LLTEMP .AND. IADO == IADOX) THEN
      PCOVAR(IY,IX) = PCOVAR(IY,IX)+FCZTO5(ROBODY(IADX,MDBOER),&
       & ROBODY(IADY,MDBOER),ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZO5)&
       & / (ZFG(IX) * ROBODY(IADY,MDBFGE))  
    ENDIF
  ENDDO

  IPRY = JPPYU
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZSINPY = 0.0_JPRB
    ELSE
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
    ENDIF
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHZU(ZDIST2(IX,IY),ZH2Z,ZNU2,ZMU,ZSINPY)&
     & *FRVZU(ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZ)  
  ENDDO

  IPRY = JPPYV
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZCOSPY = 1.0_JPRB
    ELSE
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHZV(ZDIST2(IX,IY),ZH2Z,ZNU2,ZMU,ZCOSPY)&
     & *FRVZV(ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZ)  
  ENDDO

  IPRY = JPPYDZ
  DO JSEY = 1 , KNSELS(IPRY)
    IY   = IY + 1
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHZZ(ZDIST2(IX,IY),ZH2Z)&
     & *FRVZDZ(ZPRESS,ROBODY(IADY,MDBRBPIO),ROBODY(IADY,MDBPRL),&
     & ROBODY(IADY,MDBFGE),ROBODY(IADY,MDBRBOE),ZKZ)  
  ENDDO

  IY = IY + KNSELS(JPPYHU) + KNSELS(JPPYST) + KNSELS(JPPYSN)
  IPRY = JPPYT2
  DO JSEY = 1 , KNSELS(IPRY)
    IY = IY + 1
    PCOVAR(IY,IX) = FRHZT(ZDIST2(IX,IY),ZH2T2)&
     & *FRVZT(ZPRESS,ROBODY(IADX,MDBPRL),ZKPBL)  
  ENDDO 

  IY = IY + KNSELS(JPPYH2)
  IPRY = JPPYU1
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10) THEN
      ZSINPY = 0.0_JPRB
    ELSE
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHZU(ZDIST2(IX,IY),ZH2Z,ZNU2BL,ZMU,ZSINPY)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKPBL)  
  ENDDO
  IPRY = JPPYV1
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10) THEN
      ZCOSPY = 1.0_JPRB
    ELSE
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHZV(ZDIST2(IX,IY),ZH2Z,ZNU2BL,ZMU,ZCOSPY)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKZ)  
  ENDDO
ENDDO

!---------------------------------------------------------------------------
!  - 3 - Calcul des covariances entre les predicteurs T
!        et les autres predicteurs differents de Z.
!        ----------------------------------------------

IPRX = JPPYT
DO JSEX = 1 , KNSELS(IPRX)
  IX      = IX + 1
  IY      = IX
  IADOX   = KOSELS(IPRX,JSEX)
  ZLATX   = ROBHDR(IADOX,MDBLAT)
  ZLONX   = ROBHDR(IADOX,MDBLON)
  LLTEMP  = ROBHDR(IADOX,MDBOTP) == NTEMP
  IADX    = KASELS(IPRX,JSEX)
  ZPRESS  = ROBODY(IADX,MDBRBPIO)
  ZFG(IX) = ROBODY(IADX,MDBFGE)

  IPRY = JPPYT
  DO JSEY = JSEX+1 , KNSELS(IPRY)
    IY   = IY + 1
    IADO = KOSELS(IPRY,JSEY)
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHTT(ZDIST2(IX,IY),ZH2Z)&
     & *FRVTT(ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZ)  
    IF (LLTEMP .AND. IADO == IADOX)&
     & PCOVAR(IY,IX) = PCOVAR(IY,IX) + FCTTO5(ROBODY(IADX,MDBOER),&
     & ROBODY(IADY,MDBOER),ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZO5)&
     & / (ZFG(IX)*ROBODY(IADY,MDBFGE))  
  ENDDO

  IPRY = JPPYU
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZSINPY = 0.0_JPRB
    ELSE
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
    ENDIF
    IADY   = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHTU(ZDIST2(IX,IY),ZH2Z,ZNU2,ZMU,ZSINPY)&
     & *FRVTU(ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZ)  
  ENDDO

  IPRY = JPPYV
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZCOSPY = 1.0_JPRB
    ELSE
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHTV(ZDIST2(IX,IY),ZH2Z,ZNU2,ZMU,ZCOSPY)&
     & *FRVTV(ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZ)  
  ENDDO

  IPRY = JPPYDZ
  DO JSEY = 1 , KNSELS(IPRY)
    IY   = IY + 1
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHTZ(ZDIST2(IX,IY),ZH2Z)&
     & *FRVTDZ(ZPRESS,ROBODY(IADY,MDBRBPIO),ROBODY(IADY,MDBPRL),&
     & ROBODY(IADY,MDBFGE),ROBODY(IADY,MDBRBOE),ZKZ)  
  ENDDO

  IY = IY + KNSELS(JPPYHU) + KNSELS(JPPYST) + KNSELS(JPPYSN)
  IPRY = JPPYT2
  DO JSEY = 1 , KNSELS(IPRY)
    IY = IY + 1
    PCOVAR(IY,IX) = FRHT2T2(ZDIST2(IX,IY),ZH2T2)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKPBL)  
  ENDDO

  IY = IY + KNSELS(JPPYH2)
  IPRY = JPPYU1
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZSINPY = 0.0_JPRB
    ELSE
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHTU(ZDIST2(IX,IY),ZH2VORBL,ZNU2BL,ZMU,ZSINPY)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKZ)  
  ENDDO

  IPRY = JPPYV1
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZCOSPY = 1.0_JPRB
    ELSE
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHTV(ZDIST2(IX,IY),ZH2VORBL,ZNU2BL,ZMU,ZCOSPY)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKZ)  
  ENDDO
ENDDO

!---------------------------------------------------------------------------
!  - 4 - Calcul des covariances entre les predicteurs U
!        et les predicteurs autres que Z et T.
!        ----------------------------------------------

IPRX = JPPYU
DO JSEX = 1 , KNSELS(IPRX)
  IX      = IX + 1
  IY      = IX
  IADOX   = KOSELS(IPRX,JSEX)
  ZLATX   = ROBHDR(IADOX,MDBLAT)
  ZLONX   = ROBHDR(IADOX,MDBLON)
  LLTEMP  = ROBHDR(IADOX,MDBOTP) == NTEMP
  LLPILO  = ROBHDR(IADOX,MDBOTP) == NPILOT
  LLSATO  = ROBHDR(IADOX,MDBOTP) == NSATOB
  IADX    = KASELS(IPRX,JSEX)
  ZPRESS  = ROBODY(IADX,MDBRBPIO)
  ZFG(IX) = ROBODY(IADX,MDBFGE)

  IPRY = JPPYU
  DO JSEY = JSEX+1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZSINPX = 0.0_JPRB
      ZCOSPX = 1.0_JPRB
      ZSINPY = 0.0_JPRB
      ZCOSPY = 1.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    INSATX = ROBHDR(IADOX,MDB_INSTTYPE_AT_HDR)
    INSAT  = ROBHDR(IADO,MDB_INSTTYPE_AT_HDR)
    IADY   = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHUU(ZDIST2(IX,IY),ZH2Z,ZH2DIV,ZNU2,&
     & ZCOSPX,ZCOSPY,ZSINPX,ZSINPY)&
     & *FRVUU(ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZ)  
    IF ( INSATX == INSAT .AND. LLSATO ) THEN
      PCOVAR(IY,IX) = PCOVAR(IY,IX)+FCUUO3(ROBODY(IADX,MDBOER),&
       & ROBODY(IADY,MDBOER),ZDIST2(IY,IX),ZPRESS,ROBODY(IADY,MDBRBPIO))&
       & / (ZFG(IX)*ROBODY(IADY,MDBFGE))  
    ELSEIF ( LLTEMP .AND. IADO == IADOX ) THEN
      PCOVAR(IY,IX) = PCOVAR(IY,IX)+FCUUO5(ROBODY(IADX,MDBOER),&
       & ROBODY(IADY,MDBOER),ZPRESS,ROBODY(IADY,MDBRBPIO),ZKUO5)&
       & / (ZFG(IX)*ROBODY(IADY,MDBFGE))  
    ELSEIF ( LLPILO .AND. IADO == IADOX ) THEN
      PCOVAR(IY,IX) = PCOVAR(IY,IX)+FCUUO6(ROBODY(IADX,MDBOER),&
       & ROBODY(IADY,MDBOER),ZPRESS,ROBODY(IADY,MDBRBPIO),ZKUO6)&
       & / (ZFG(IX)*ROBODY(IADY,MDBFGE))  
    ENDIF
  ENDDO

  IPRY = JPPYV
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZSINPX = 0.0_JPRB
      ZCOSPX = 1.0_JPRB
      ZSINPY = 0.0_JPRB
      ZCOSPY = 1.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHUV(ZDIST2(IX,IY),ZH2Z,ZH2DIV,ZNU2,&
     & ZCOSPX,ZCOSPY,ZSINPX,ZSINPY)&
     & *FRVUV(ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZ)  
  ENDDO

  IPRY = JPPYDZ
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZSINPX = 0.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
    ENDIF
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHUZ(ZDIST2(IX,IY),ZH2Z,ZNU2,ZMU,ZSINPX)&
     & *FRVUDZ(ZPRESS,ROBODY(IADY,MDBRBPIO),&
     & ROBODY(IADY,MDBPRL),ROBODY(IADY,MDBFGE),&
     & ROBODY(IADY,MDBRBOE),ZKZ)  
  ENDDO

  IY = IY + KNSELS(JPPYHU) + KNSELS(JPPYST)
  IPRY = JPPYT2
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZSINPX = 0.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHUT(ZDIST2(IX,IY),ZH2T2,ZNU2BL,ZMU,ZSINPX)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKPBL)  
  ENDDO

  IY = IY + KNSELS(JPPYH2)
  IPRY = JPPYU1
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10) THEN
      ZSINPX = 0.0_JPRB
      ZCOSPX = 1.0_JPRB
      ZSINPY = 0.0_JPRB
      ZCOSPY = 1.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHU1U1(ZDIST2(IX,IY),ZH2VORBL,ZH2DIVBL,ZNU2BL,&
     & ZCOSPX,ZCOSPY,ZSINPX,ZSINPY)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKPBL)  
  ENDDO

  IPRY = JPPYV1
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10) THEN
      ZSINPX = 0.0_JPRB
      ZCOSPX = 1.0_JPRB
      ZSINPY = 0.0_JPRB
      ZCOSPY = 1.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHU1V1(ZDIST2(IX,IY),ZH2VORBL,ZH2DIVBL,ZNU2BL,&
     & ZCOSPX,ZCOSPY,ZSINPX,ZSINPY)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKPBL)  
  ENDDO
ENDDO

!---------------------------------------------------------------------------
!  - 5 - Calcul des covariances entre les predicteurs V
!        et les predicteurs autres que Z, T et U.
!        ----------------------------------------------

IPRX = JPPYV
DO JSEX = 1 , KNSELS(IPRX)
  IX      = IX + 1
  IY      = IX
  IADOX   = KOSELS(IPRX,JSEX)
  ZLATX   = ROBHDR(IADOX,MDBLAT)
  ZLONX   = ROBHDR(IADOX,MDBLON)
  LLTEMP  = ROBHDR(IADOX,MDBOTP) == NTEMP
  LLPILO  = ROBHDR(IADOX,MDBOTP) == NPILOT
  LLSATO  = ROBHDR(IADOX,MDBOTP) == NSATOB
  IADX    = KASELS(IPRX,JSEX)
  ZPRESS  = ROBODY(IADX,MDBRBPIO)
  ZFG(IX) = ROBODY(IADX,MDBFGE)

  IPRY = JPPYV
  DO JSEY = JSEX+1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZSINPX = 0.0_JPRB
      ZSINPY = 0.0_JPRB
      ZCOSPX = 1.0_JPRB
      ZCOSPY = 1.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    ISATX1 = ROBHDR(IADOX,MDB_INSTTYPE_AT_HDR)
    ISAT1  = ROBHDR(IADO,MDB_INSTTYPE_AT_HDR)
    IADY   = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHVV(ZDIST2(IX,IY),ZH2Z,ZH2DIV,ZNU2,&
     & ZCOSPX,ZCOSPY,ZSINPX,ZSINPY)&
     & *FRVVV(ZPRESS,ROBODY(IADY,MDBRBPIO),ZKZ)  
    IF ( ISATX1 == ISAT1.AND. LLSATO ) THEN
      PCOVAR(IY,IX) = PCOVAR(IY,IX) +&
       & FCUUO3(ROBODY(IADX,MDBOER),ROBODY(IADY,MDBOER),&
       & ZDIST2(IY,IX),ZPRESS,ROBODY(IADY,MDBRBPIO))&
       & / (ZFG(IX)*ROBODY(IADY,MDBFGE))  
    ELSEIF ( LLTEMP .AND. IADO == IADOX ) THEN
      PCOVAR(IY,IX) = PCOVAR(IY,IX) +&
       & FCUUO5(ROBODY(IADX,MDBOER),ROBODY(IADY,MDBOER),&
       & ZPRESS,ROBODY(IADY,MDBRBPIO),ZKUO5)&
       & / (ZFG(IX)*ROBODY(IADY,MDBFGE))  
    ELSEIF ( LLPILO .AND. IADO == IADOX ) THEN
      PCOVAR(IY,IX) = PCOVAR(IY,IX) +&
       & FCUUO6(ROBODY(IADX,MDBOER),ROBODY(IADY,MDBOER),&
       & ZPRESS,ROBODY(IADY,MDBRBPIO),ZKUO6)&
       & / (ZFG(IX)*ROBODY(IADY,MDBFGE))  
    ENDIF
  ENDDO

  IPRY = JPPYDZ
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZCOSPX = 1.0_JPRB
    ELSE
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    IADY = KASELS(IPRY,JSEY)
    PCOVAR(IY,IX) = FRHVZ(ZDIST2(IX,IY),ZH2Z,ZNU2,ZMU,ZCOSPX)&
     & *FRVVDZ(ZPRESS,ROBODY(IADY,MDBRBPIO),&
     & ROBODY(IADY,MDBPRL),&
     & ROBODY(IADY,MDBFGE),ROBODY(IADY,MDBRBOE),ZKZ)  
  ENDDO

  IY = IY + KNSELS(JPPYHU) + KNSELS(JPPYST)
  IPRY = JPPYT2
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10_JPRB) THEN
      ZCOSPX = 1.0_JPRB
    ELSE
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHVT(ZDIST2(IX,IY),ZH2T2,ZNU2BL,ZMU,ZCOSPX)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKPBL)  
  ENDDO

  IY = IY + KNSELS(JPPYH2)
  IPRY = JPPYU1
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10) THEN
      ZSINPX = 0.0_JPRB
      ZCOSPX = 1.0_JPRB
      ZSINPY = 0.0_JPRB
      ZCOSPY = 1.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHU1U1(ZDIST2(IX,IY),ZH2VORBL,ZH2DIVBL,ZNU2BL,&
     & ZCOSPX,ZCOSPY,ZSINPX,ZSINPY)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKPBL)  
  ENDDO

  IPRY = JPPYV1
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10) THEN
      ZSINPX = 0.0_JPRB
      ZCOSPX = 1.0_JPRB
      ZSINPY = 0.0_JPRB
      ZCOSPY = 1.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHU1V1(ZDIST2(IX,IY),ZH2VORBL,ZH2DIVBL,ZNU2BL,&
     & ZCOSPX,ZCOSPY,ZSINPX,ZSINPY)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKPBL)  
  ENDDO
ENDDO

!---------------------------------------------------------------------------
!  - 6 - Calcul des covariances entre les predicteurs DZ
!        et les predicteurs autres que Z, T, U et V.
!        -----------------------------------------------

IPRX = JPPYDZ
DO JSEX = 1 , KNSELS(IPRX)
  IX      = IX + 1
  IY      = IX
  IADOX   = KOSELS(IPRX,JSEX)
  LLSATM  = ROBHDR(IADOX,MDBOTP) == NSATEM
  IADX    = KASELS(IPRX,JSEX)
  ZPRESS  = ROBODY(IADX,MDBRBPIO)
  ZPRESB  = ROBODY(IADX,MDBPRL)
  ZSIGS   = ROBODY(IADX,MDBFGE)
  ZSIGB   = ROBODY(IADX,MDBRBOE)
  ZFG(IX) = FSIGDZ(ZSIGS,ZSIGB,ZPRESS,ZPRESB,ZKZ)

  IPRY = JPPYDZ
  DO JSEY = JSEX+1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    IADY   = KASELS(IPRY,JSEY)
    ISATX2 = ROBHDR(IADOX,MDB_INSTTYPE_AT_HDR)
    ISAT2  = ROBHDR(IADO,MDB_INSTTYPE_AT_HDR)
    PCOVAR(IY,IX) = FRHZZ(ZDIST2(IX,IY),ZH2Z)&
     & *FRVDZDZ(ZPRESS,ZPRESB,ROBODY(IADY,MDBRBPIO),&
     & ROBODY(IADY,MDBPRL),ZSIGS,ZSIGB,&
     & ROBODY(IADY,MDBFGE),&
     & ROBODY(IADY,MDBRBOE),ZKZ)  
    IF ( ISATX2 == ISAT2.AND. LLSATM ) THEN
      PCOVAR(IY,IX) = PCOVAR(IY,IX) +&
       & FCZZO7(ROBODY(IADX,MDBOER),ROBODY(IADY,MDBOER),&
       & ZDIST2(IY,IX),ZPRESS,ZPRESB,&
       & ROBODY(IADY,MDBRBPIO),ROBODY(IADY,MDBPRL),&
       & ZPDZO7,ZKDZO7) / ZFG(IX) / &
       & FSIGDZ(ROBODY(IADY,MDBFGE),ROBODY(IADY,MDBRBOE),&
       & ROBODY(IADY,MDBRBPIO),ROBODY(IADY,MDBPRL),ZKZ)  
    ENDIF
  ENDDO

ENDDO

!---------------------------------------------------------------------------
!  - 7 - Calcul des covariances entre les predicteurs H
!        et les predicteurs autres que Z, T, U, V et DZ.
!        ----------------------------------------------
! (il n'existe, au 08/92, que des humidites de niveau ou traitees comme telles).

IPRX = JPPYHU
DO JSEX = 1 , KNSELS(IPRX)
  IX      = IX + 1
  IY      = IX
  IADOX   = KOSELS(IPRX,JSEX)
  LLTEMP  = ROBHDR(IADOX,MDBOTP) == NTEMP
  IADX    = KASELS(IPRX,JSEX)
  ZPRESS  = ROBODY(IADX,MDBRBPIO)
  ZFG(IX) = ROBODY(IADX,MDBFGE)

  IPRY = JPPYHU
  DO JSEY = JSEX+1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    IADY   = KASELS(IPRY,JSEY)
    ZPRUSS = ROBODY(IADY,MDBRBPIO)
    PCOVAR(IY,IX) = FRHHH(ZDIST2(IX,IY),ZH2HU)*FRVHH(ZPRESS,ZPRUSS,ZKH)
    IF ( LLTEMP .AND. IADO == IADOX ) PCOVAR(IY,IX) = PCOVAR(IY,IX) + &
     & FCHHO5(ROBODY(IADX,MDBOER),ROBODY(IADY,MDBOER),&
     & ZPRESS,ROBODY(IADY,MDBRBPIO),ZKHO5)&
     & / ( ZFG(IX)*ROBODY(IADY,MDBFGE) )  
  ENDDO

  IY = IY + KNSELS(JPPYST) + KNSELS(JPPYSN) + KNSELS(JPPYT2)
  IPRY = JPPYH2
  DO JSEY = 1 , KNSELS(IPRY)
    IY = IY + 1
    PCOVAR(IY,IX) = FRHH2H2(ZDIST2(IX,IY),ZH2H2)&
     & *FRVBLBL(ZPRESS,ROBODY(IADX,MDBPRL),ZKPBL)  
  ENDDO
ENDDO

!---------------------------------------------------------------------------
!  - 8 - Calcul des covariances entre les predicteurs SST
!        et les predicteurs autres (SST).
!        ------------------------------------------------

IPRX = JPPYST
DO JSEX = 1 , KNSELS(IPRX)
  IX      = IX + 1
  IY      = IX
  IADX    = KASELS(IPRX,JSEX)
  ZFG(IX) = ROBODY(IADX,MDBFGE)

  IPRY = JPPYST
  DO JSEY = JSEX+1 , KNSELS(IPRY)
    IY = IY + 1
    PCOVAR(IY,IX) = FRHSTST(ZDIST2(IX,IY),ZH2SST)
  ENDDO

ENDDO

!---------------------------------------------------------------------------
!  - 9 - Calcul des covariances entre les predicteurs SN
!        et les predicteurs autres (SN).
!        -----------------------------------------------

IPRX = JPPYSN
DO JSEX = 1 , KNSELS(IPRX)
  IX      = IX + 1
  IY      = IX
  IADX    = KASELS(IPRX,JSEX)
  ZPRESS  = ROBODY(IADX,MDBRBPIO)
  ZFG(IX) = ROBODY(IADX,MDBFGE)

  IPRY = JPPYSN
  DO JSEY = JSEX+1 , KNSELS(IPRY)
    IY = IY + 1
    IADY   = KASELS(IPRY,JSEY)
    ZPRUSS = ROBODY(IADY,MDBRBPIO)
    ZDISTP2 = (ZPRESS-ZPRUSS) ** 2
    PCOVAR(IY,IX) = FRHSNSN(ZDIST2(IX,IY),ZH2SN) * FRHPSNSN(ZDISTP2,ZV2P2SN)
  ENDDO

ENDDO

!---------------------------------------------------------------------------
! - 10 - Calcul des covariances entre les predicteurs T2m
!        et les predicteurs autres (T2m).
!        ------------------------------------------------

IF ( .NOT. LCORRF ) THEN
  IPRX = JPPYT2
  DO JSEX = 1 , KNSELS(IPRX)
    IX      = IX + 1
    IY      = IX
    IADX    = KASELS(IPRX,JSEX)
    ZFG(IX) = ROBODY(IADX,MDBFGE)
 
    IPRY = JPPYT2
    DO JSEY = JSEX+1 , KNSELS(IPRY)
      IY = IY + 1
      PCOVAR(IY,IX) = FRHT2T2(ZDIST2(IX,IY),ZH2T2)
    ENDDO
  ENDDO
ELSE
  IPRX = JPPYT2
  DO JSEX = 1 , KNSELS(IPRX)
    IX      = IX + 1
    IY      = IX
    IADX    = KASELS(IPRX,JSEX)
    ZFG(IX) = ROBODY(IADX,MDBFGE)

    IPRY = JPPYT2
    DO JSEY = JSEX+1 , KNSELS(IPRY)
      IY = IY + 1
      PCOVAR(IY,IX) = FRHT2T2(SQRT(ZDIST2(IX,IY)),SQRT(ZH2T2))
    ENDDO
  ENDDO
ENDIF
!----------------------------------------------------------------------
!  - 11 - Calcul des covariances entre les predicteurs Hu2m
!         et les predicteurs autres (Hu2m).
!         -------------------------------------------------
IF ( .NOT. LCORRF ) THEN
  IPRX = JPPYH2
  DO JSEX = 1 , KNSELS(IPRX)
    IX      = IX + 1
    IY      = IX
    IADX    = KASELS(IPRX,JSEX)
    ZFG(IX) = ROBODY(IADX,MDBFGE)
 
    IPRY = JPPYH2
    DO JSEY = JSEX+1 , KNSELS(IPRY)
      IY = IY + 1
      PCOVAR(IY,IX) = FRHH2H2(ZDIST2(IX,IY),ZH2H2)
    ENDDO
  ENDDO
ELSE
  IPRX = JPPYH2
  DO JSEX = 1 , KNSELS(IPRX)
    IX      = IX + 1
    IY      = IX
    IADX    = KASELS(IPRX,JSEX)
    ZFG(IX) = ROBODY(IADX,MDBFGE)

    IPRY = JPPYH2
    DO JSEY = JSEX+1 , KNSELS(IPRY)
      IY = IY + 1
      PCOVAR(IY,IX) = FRHH2H2(SQRT(ZDIST2(IX,IY)),SQRT(ZH2H2))
    ENDDO
  ENDDO
ENDIF

!----------------------------------------------------------------------
!  - 12 - Calcul des covariances entre les predicteurs U10m
!         et les predicteurs autres (U10m,V10m).
!         -------------------------------------------------

IPRX = JPPYU1
DO JSEX = 1 , KNSELS(IPRX)
  IX      = IX + 1
  IY      = IX
  IADOX   = KOSELS(IPRX,JSEX)
  ZLATX   = ROBHDR(IADOX,MDBLAT)
  ZLONX   = ROBHDR(IADOX,MDBLON)
  IADX    = KASELS(IPRX,JSEX)
  ZFG(IX) = ROBODY(IADX,MDBFGE)

  IPRY = JPPYU1
  DO JSEY = JSEX+1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10) THEN
      ZSINPX = 0.0_JPRB
      ZCOSPX = 1.0_JPRB
      ZSINPY = 0.0_JPRB
      ZCOSPY = 1.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHU1U1(ZDIST2(IX,IY),ZH2VORBL,ZH2DIVBL,ZNU2BL,&
     & ZCOSPX,ZCOSPY,ZSINPX,ZSINPY)  
  ENDDO

  IPRY = JPPYV1
  DO JSEY = 1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10) THEN
      ZSINPX = 0.0_JPRB
      ZCOSPX = 1.0_JPRB
      ZSINPY = 0.0_JPRB
      ZCOSPY = 1.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHU1V1(ZDIST2(IX,IY),ZH2VORBL,ZH2DIVBL,ZNU2BL,&
     & ZCOSPX,ZCOSPY,ZSINPX,ZSINPY)  
  ENDDO
ENDDO

!----------------------------------------------------------------------
!  - 13 - Calcul des covariances entre les predicteurs V10m
!         et les predicteurs autres (V10m).
!         -------------------------------------------------

IPRX = JPPYV1
DO JSEX = 1 , KNSELS(IPRX)
  IX      = IX + 1
  IY      = IX
  IADOX   = KOSELS(IPRX,JSEX)
  ZLATX   = ROBHDR(IADOX,MDBLAT)
  ZLONX   = ROBHDR(IADOX,MDBLON)
  IADX    = KASELS(IPRX,JSEX)
  ZFG(IX) = ROBODY(IADX,MDBFGE)
 
  IPRY = JPPYV1
  DO JSEY = JSEX+1 , KNSELS(IPRY)
    IY     = IY + 1
    IADO   = KOSELS(IPRY,JSEY)
    ZLAT   = ROBHDR(IADO,MDBLAT)
    ZLON   = ROBHDR(IADO,MDBLON)
    ZSINUS = CASINU(ZLATX,ZLONX,ZLAT,ZLON)
    IF (ZSINUS < 1.E-10) THEN
      ZSINPX = 0.0_JPRB
      ZSINPY = 0.0_JPRB
      ZCOSPX = 1.0_JPRB
      ZCOSPY = 1.0_JPRB
    ELSE
      ZSINPX = CASI1N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZSINPY = CASI2N(ZLATX,ZLAT,COS(ZLON-ZLONX))/ZSINUS
      ZCOSPX = CACON(ZLAT,SIN(ZLON-ZLONX))/ZSINUS
      ZCOSPY = CACON(ZLATX,SIN(ZLON-ZLONX))/ZSINUS
    ENDIF
    PCOVAR(IY,IX) = FRHV1V1(ZDIST2(IX,IY),ZH2VORBL,ZH2DIVBL,ZNU2BL,&
     & ZCOSPX,ZCOSPY,ZSINPX,ZSINPY)  
  ENDDO
ENDDO

!---------------------------------------------------------------------------
!  - 14 - Calcul de la diagonale et symetrisation de la matrice.
!         -----------------------------------------------------

IY = 0
DO JJRE = 1 , JPNVY
  DO JSEL = 1 , KNSELS(JJRE)
    IY = IY+1
    IAD = KASELS(JJRE,JSEL)
    ZOB = ROBODY(IAD,MDBOER)
    ZRE = ROBODY(IAD,MDBRER)
    PCOVAR(IY,IY) = 1.0_JPRB + (ZOB*ZOB + ZRE*ZRE)/(ZFG(IY)*ZFG(IY))
    PRESID(IY) = ROBODY(IAD,MDBOMF) / ZFG(IY)
  ENDDO
ENDDO

! symetrisation de la matrice

DO JX = 2 , KNOUT
  DO JY = 1 , JX-1
    PCOVAR(JY,JX) = PCOVAR(JX,JY)
  ENDDO
ENDDO

!---------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CATRMA',1,ZHOOK_HANDLE)
END SUBROUTINE CATRMA
