SUBROUTINE UPDCLIE_CO2(PTSTEP)

!**** *UPDCLIE*

!     PURPOSE.
!     --------

!     Updates the terrestrial biosphere compound of CO2 surface fluxes
!     in order to describe a diurnal cycle for the exchange with the
!     natural biopshere

!     (ECMWF version, distributed or shared memory)

!     CO2 climatological fields (with fine time resolution) are read
!     from CO2CLIM file written in GRIB format in an ascendent date/time order.
!     CO2CLIM covers the duration of a model forecast

!**   INTERFACE.
!     ----------

!     CALL UPDCLIE_CO2(PTSTEP)

!        Explicit arguments :
!        --------------------
!        PTSTEP: TIME STEP

!     EXTERNALS.
!     ----------

!     UPDCALSEC

!     AUTHORS. of UPDCLIE (see updclie.F90)
!     --------

!     modified version for CO2 : S. Serrar
!        M.Hamrud      01-Jul-2006 Revised surface fields
!        S.Serrar      23-JUL-2007 routine updated in order to use different 
!                                  climatologies with different time frequencies 
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMGRB    ,ONLY : NGRBCO2NBF
USE YOMCST   , ONLY : RDAY
USE YOMCT0   , ONLY : NSTOP    ,CNMEXP   ,LTWOTL
USE YOMDIM   , ONLY : NDGSAG   ,NDGENG   ,NDGNH    ,&
 & NFLEVG   ,NPROMA
USE YOMCT3   , ONLY : NSTEP
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTMX ,&
 & NLOENG   ,NGPTOTG  
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMMCC   , ONLY : NPCO2_1      ,NPCO2_2      ,CLIMRCO2   
USE YOEPHY   , ONLY : NCO2CLIMYY2, NCO2CLIMYY1, NCO2CLIMN2, NCO2CLIMN1, NCO2CLIMFRQ
USE YOMMP    , ONLY : MYPROC
USE YOMMSC   , ONLY : NINTLEN
USE YOMRIP   , ONLY : NINDAT   ,NSSSSS   ,NSTADD   ,NSTASS
USE SURFACE_FIELDS_MIX, ONLY : SD_VF, YSD_VF

IMPLICIT NONE

#include "abor1.intfb.h"
#include "ircvgpf.intfb.h"
#include "isndgpf.intfb.h"
#include "updcal.intfb.h"
#include "updcalsec.intfb.h"

INTEGER(KIND=JPIM), PARAMETER :: JPCLIMCO2=1

REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTEP 
INTEGER(KIND=JPIM) :: ILMOIS(12)
INTEGER(KIND=JPIM) :: IGRIG(NGPTOTG)
INTEGER(KIND=JPIM) ::   ISEC0(2),ISEC1(60),ISEC2(22+(NDGENG-NDGSAG+1)),&
 & ISEC3(2),ISEC4(42)  
REAL(KIND=JPRB) ::      ZSEC2(10+2*(NFLEVG+1)),ZSEC3(2)
REAL(KIND=JPRB) :: ZBUF (NGPTOTG)
REAL(KIND=JPRB) :: ZFLD (NGPTOTMX)
CHARACTER :: CLNOMF*15

INTEGER(KIND=JPIM) :: IA0, IAE, IAN, ICCYY0, ICCYYE,&
 & IAT1,IAT2,IAE1,ICC,&
 & IHOUR,IH0,IHT1,IHT2,IHE,IHH0,IHE1,IHHE,&
 & IMIN,IMNT1,IMNT2,IMNE,IMN0,IMNE1,IMNE2,&
 & ISEC,IUMIN,IJE1,&
 & IYYM,IDHMN,IYYM0,IIDHMN0,&
 & IDIF, IDIFD, IEND, &
 & IERR, IFIRST, IIDHMNE , IYYME , &
 & IJ0, IJE, IJOUR, IDD0, IDDE, &
 & IJT1, IJT2, &
 & ILENBYT, ILENGG, ILEVTY, ILONG, &
 & IM0, IME, IMM0, IMME, IMOIS, IMT1, IMT2, IME1, &
 & IPAR, IPARAM, &
 & IBL, IPOI, IPOIN, &
 & IREPRT, IRET, IST, ISTADDE, ISTASSE,&
 & ITIME, IUNITCO2, IWORD, &
 & IZTE, J, JCL, JF, &
 & JROF, JSTGLO, JTIM, IYEARG,&
 & IADD,IHE2,IJE2,IME2,IAE2, &
 & IAT1R, IAT2R

LOGICAL :: LLFIRST, LLFOUND, LLREAD, LLSCAN , LLOPEN
INTEGER(KIND=JPIM) :: ILASTYYYYMM, ILASTDDHHMN, IHFOUND
INTEGER(KIND=JPIM),SAVE :: ILASTREFD,ILASTREFT,ITIM
! dimension IINFO - Be aware 10000*ptstep should cover a forecast period
INTEGER(KIND=JPIM),SAVE :: IINFO(10000,3,JPCLIMCO2)

REAL(KIND=JPRB) :: ZPOID1, ZPOID2, ZCO2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcttim.h"

DATA LLFIRST /.TRUE./
DATA LLSCAN/.TRUE./

!*
!     1. SETTING CONSTANT VALUES.
!     ---------------------------

IF (LHOOK) CALL DR_HOOK('UPDCLIE_CO2',0,ZHOOK_HANDLE)

LLOPEN=.FALSE.

!*    1.1 Calendar

IJ0=NDD(NINDAT)
IM0=NMM(NINDAT)
IA0=NCCAA(NINDAT)
IH0=NSSSSS/3600_JPIM
CALL UPDCALSEC(IH0,IJ0,IM0,IA0,NSTADD,NSTASS,IHOUR,IMIN,ISEC,IJOUR,IMOIS,IAN,ILMOIS,NULOUT)

! CO2 diurnal cycle described at NCO2CLIMFRQ time frequency (in minutes)

IUMIN=IHOUR*60_JPIM+IMIN
IMNT1=MOD(NCO2CLIMN1,60_JPIM)
IMNT2=IMNT1

! climatologies may be provided for one year or for a limited period (few years)
! to run a forecast outside climatologies time interval one may use data
! from the boundaries 

IF(IUMIN >= NCO2CLIMN1 .and. IUMIN < NCO2CLIMN2) THEN
  IHFOUND=(IUMIN-NCO2CLIMN1)/NCO2CLIMFRQ
  IHT1 = (NCO2CLIMN1 + IHFOUND * NCO2CLIMFRQ)/60_JPIM
  IHT2 = IHT1 + NCO2CLIMFRQ/60_JPIM
! CO2 climatology is provided for the period between NCO2CLIMYY1 and NCO2CLIMYY2 
  IAT1=MAX(NCO2CLIMYY1,IAN)
  IAT1=MIN(IAT1,NCO2CLIMYY2)
  IAT2=IAT1
  IMT1=IMOIS
  IJT1=IJOUR
  IJT2=IJT1
  IMT2=IMT1
ELSEIF(IUMIN < NCO2CLIMN1) THEN 
  IHT1 = NCO2CLIMN2/60_JPIM
  IHT2 = NCO2CLIMN1/60_JPIM
  CALL UPDCAL(IJOUR,IMOIS,IAN,-1,IJT1,IMT1,IAT1,ILMOIS,NULOUT)
  IAT1=MAX(NCO2CLIMYY1,IAT1)
  IAT1=MIN(IAT1,NCO2CLIMYY2)
  IJT2=IJOUR
  IMT2=IMOIS
  IAT2=IAN
  IAT2=MAX(NCO2CLIMYY1,IAT2)
  IAT2=MIN(IAT2,NCO2CLIMYY2)
ELSEIF(IUMIN >= NCO2CLIMN2) THEN
  IHT1 = NCO2CLIMN2/60_JPIM
  IHT2 = NCO2CLIMN1/60_JPIM
  CALL UPDCAL(IJOUR,IMOIS,IAN,1,IJT2,IMT2,IAT2,ILMOIS,NULOUT)
  IAT2=MAX(NCO2CLIMYY1,IAT2)
  IAT2=MIN(IAT2,NCO2CLIMYY2)
  IJT1=IJOUR
  IMT1=IMOIS
  IAT1=IAN
  IAT1=MAX(NCO2CLIMYY1,IAN)
  IAT1=MIN(IAT1,NCO2CLIMYY2)
ENDIF


! keep the real calendar years for the calculation of the weights

IAT1R=IAN
IAT2R=IAN
IF(IJT1 >  IJOUR .AND. IMOIS == 1) IAT1R=IAT1R-1
IF(IJOUR > IJT2 .AND. IMOIS == 12) IAT2R=IAT2R+1


ILASTYYYYMM=IMT2+IAT2*100
ILASTDDHHMN=IMNT2+IHT2*100+IJT2*10000


! A new record is read in the climate file only when needed
IF(ILASTYYYYMM /= ILASTREFD .OR. ILASTDDHHMN /= ILASTREFT) LLFIRST=.TRUE.


! the climate file is scanned once during the whole forecast time period
IF(NSTEP > 0 .AND. MYPROC == 1) LLSCAN=.FALSE.

IRET=0

!*
!     2. OPEN AND SCAN THE FILE, IF CLIMATE FIELDS REQUIRED
!     -----------------------------------------------------

!*    2.1 OPEN CLIMATE FILE

SCANIF:IF(LLFIRST) THEN

  MYPROCIF: IF (MYPROC == 1) THEN

! ONLY PROCESSOR 1 SHOULD OPEN 

    CLNOMF='CO2CLIM'//CNMEXP(1:4)//'INIT'
    WRITE(UNIT=NULOUT,FMT='(A)') ' READ CLIMATE FIELDS FROM GRIB '
    WRITE(UNIT=NULOUT,FMT='(A,A)')&
     & ' INITIAL DATA TO BE READ FROM FILE ',CLNOMF  
    CALL PBOPEN(IUNITCO2,CLNOMF,'r',IRET)
    LLOPEN=.TRUE.
    IF(IRET /= 0) THEN
      WRITE(NULERR,'(A,I2)')'UPDCLIE_CO2: PROBLEM IN PBOPEN, IRET=',IRET
      CALL ABOR1('UPDCLIE_CO2: PROBLEM IN PBOPEN')
    ENDIF

    IF(LLSCAN) THEN

!*    2.2 READ, DECODE HEADERS, CHECK THE FIELDS AND POSITIONS THE FILE

    ILENGG=NGPTOTG
    ILENBYT=NINTLEN*ILENGG
    IPOIN=0
    SCAN: DO JTIM=1,10000
      DO JCL=1,JPCLIMCO2
        IRET=0
        CALL PBGRIB(IUNITCO2,IGRIG,ILENBYT,ILONG,IRET)
        IF(IRET == 0) THEN
          ILONG=ILONG/NINTLEN
        ELSEIF(IRET == -1) THEN
          ITIM=JTIM-1
          EXIT SCAN
        ELSE
          WRITE(NULERR,'(A,I2)')'UPDCLIE_CO2: PROBLEM IN PBGRIB, IRET=',IRET
          CALL ABOR1('UPDCLIE_CO2: PROBLEM IN PBGRIB')
        ENDIF
        IPOI=IPOIN
        CALL PBTELL(IUNITCO2,IPOIN)
        IF (IPOIN == -2) THEN
          WRITE(NULERR,'(A,I2)')'UPDCLIE_CO2: PROBLEM IN PBTELL, IRET=',IPOIN
          CALL ABOR1('UPDCLIE_CO2: PROBLEM IN PBTELL')        
        ENDIF
        CALL GSTATS(1703,0)
        CALL GRIBEX(ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,&
         & ZBUF,ILENGG,IGRIG,ILONG,IWORD,'I',IERR)  
        CALL GSTATS(1703,1)
        IPARAM=ISEC1(6)
        IF(ISEC1(1) == 210) IPARAM=IPARAM+210000
! SKIPPING OVER NON NEEDED FIELDS
        IF( NGRBCO2NBF /= IPARAM ) CYCLE SCAN

        ICC=ISEC1(21)-1_JPIM
        IDHMN=ISEC1(14)+ISEC1(13)*100+ISEC1(12)*10000
        IYYM=ISEC1(11)+ISEC1(10)*100+ICC*10000
        ILEVTY=ISEC1(7)
        IREPRT=ISEC2(1)

        WRITE(NULERR,*)&
         & 'UPDCLIE_CO2: PARAMETER ',IPARAM, ' FOR DATE ',&
         & IYYM,IDHMN,' FOUND IN CLIMATE FILE'


!      check some parameters

        IF (IREPRT /= 4) THEN
          WRITE(NULERR,'(" UPDCLIE_CO2 :UNEXPECTED DATA ",&
           & "REPRESENTATION TYPE",I6)') IREPRT  
          CALL ABOR1(' UPDCLIE_CO2:INVALID DATA REPRESENTATION TYPE')
        ENDIF
        IF(ISEC2(10) /= NDGNH) THEN
          WRITE(NULERR,'(" UPDCLIE_CO2: RESOLUTION OF MODEL ",I6,&
           & " ,OF INITIAL DATA ",I6)') NDGNH,ISEC2(10)  
          CALL ABOR1(' UPDCLIE_CO2 : INVALID DATA RESOLUTION')
        ENDIF
        IF(ILEVTY /= 1.AND.ILEVTY /= 112) THEN
          WRITE(NULERR,'(" UDPCLIEDM :  UNEXPECTED LEVEL TYPE ",I6)') ILEVTY
          CALL ABOR1(' UPDCLIE_CO2 : INVALID LEVEL TYPE')
        ENDIF


!      record/communicate field index

        IINFO(JTIM,1,JCL)=IPOI
        IINFO(JTIM,2,JCL)=IYYM
        IINFO(JTIM,3,JCL)=IDHMN
      ENDDO
    ENDDO SCAN

! Check fields come in a consistent order and position file at the
!   right place

    ITIME=NINT(PTSTEP)
    IF (LTWOTL) THEN
      IZTE=NINT(PTSTEP*(REAL(NSTOP,JPRB)+0.5_JPRB))
    ELSE
      IZTE=ITIME*NSTOP
    ENDIF
    ISTADDE=IZTE/NINT(RDAY)
    ISTASSE=IZTE-ISTADDE*RDAY
    CALL UPDCALSEC(IH0,IJ0,IM0,IA0,ISTADDE,ISTASSE,IHE,IMNE,ISEC,IJE,IME,IAE,ILMOIS,NULOUT)
    ICCYY0=IAT1
    IMM0=IMT1
    IDD0=IJT1
    IHH0=IHT1
    IMN0=IMNT1

! position of the last file needed for the current forecast model run

    IUMIN=IHE*60_JPIM+IMNE
    IMNE1=MOD(NCO2CLIMN1,60_JPIM)
    IMNE2=IMNE1

    IF(IUMIN >= NCO2CLIMN1 .and. IUMIN < NCO2CLIMN2) THEN
      IHFOUND=(IUMIN-NCO2CLIMN1)/NCO2CLIMFRQ
      IHE1 = (NCO2CLIMN1 + IHFOUND * NCO2CLIMFRQ)/60_JPIM
      IHE2 = IHE1 + NCO2CLIMFRQ/60_JPIM
      IAE1=MAX(NCO2CLIMYY1,IAE)
      IAE1=MIN(IAE1,NCO2CLIMYY2)
      IAE2=IAE1
      IME1=IME
      IJE1=IJE
      IJE2=IJE1
      IME2=IME1
    ELSEIF(IUMIN < NCO2CLIMN1) THEN 
      IHE1 = NCO2CLIMN2/60_JPIM
      IHE2 = NCO2CLIMN1/60_JPIM
      CALL UPDCAL(IJE,IME,IAE,-1,IJE1,IME1,IAE1,ILMOIS,NULOUT)
      IAE1=MAX(NCO2CLIMYY1,IAE1) 
      IAE1=MIN(IAE1,NCO2CLIMYY2)
      IJE2=IJE
      IME2=IME
      IAE2=IAE
      IAE2=MAX(NCO2CLIMYY1,IAE2)
      IAE2=MIN(IAE2,NCO2CLIMYY2)
    ELSEIF(IUMIN >= NCO2CLIMN2) THEN
      IHE1 = NCO2CLIMN2/60_JPIM
      IHE2 = NCO2CLIMN1/60_JPIM
      CALL UPDCAL(IJE,IME,IAE,1,IJE2,IME2,IAE2,ILMOIS,NULOUT)
      IAE2=MAX(NCO2CLIMYY1,IAE2)
      IAE2=MIN(IAE2,NCO2CLIMYY2)
      IJE1=IJE
      IME1=IME
      IAE1=IAE
      IAE1=MAX(NCO2CLIMYY1,IAE)
      IAE1=MIN(IAE1,NCO2CLIMYY2)
    ENDIF

      
    IMME=IME2
    ICCYYE=IAE2
    IDDE=IJE2
    IHHE=IHE2
    IMNE=IMNE2


! Locates the first field needed for the current forecast model run

    IYYM0=IMM0+100*ICCYY0
    IIDHMN0=IMN0+IHH0*100+IDD0*10000
    LLFOUND=.FALSE.
    DO J=1,ITIM
      IF (IYYM0 == IINFO(J,2,1) .AND. IIDHMN0 == IINFO(J,3,1)) THEN
        IADD=IINFO(J,1,1)
        LLFOUND=.TRUE.
        EXIT
      ENDIF
    ENDDO
    IF (.NOT.LLFOUND) THEN
      CALL ABOR1(' UPDCLIE_CO2 : FIRST FILE NOT FOUND')
    ENDIF


! Locates the last field needed at the end of the model run

    IYYME=IMME+100*ICCYYE
    IIDHMNE=IMNE+IHHE*100+IDDE*10000
    IEND=0_JPIM
    DO J=1,ITIM
      IF (IYYME == IINFO(J,2,1) .AND. IIDHMNE == IINFO(J,3,1)) THEN
        IEND=J
        EXIT
      ENDIF
    ENDDO
    IF (IEND == 0_JPIM) THEN
      WRITE(NULERR,*) 'IEND not found ',IYYME,IIDHMNE
      CALL ABOR1(' UPDCLIE_CO2 : END FILE NOT FOUND')
    ENDIF


    ENDIF

! end of test on LLSCAN - the scan is done only once at the beginning of a forecast model run

! Positions the file in the first field needed

! Locates the first field needed

    IF(.NOT.LLSCAN) THEN

      ICCYY0=IAT1
      IMM0=IMT1
      IDD0=IJT1
      IHH0=IHT1
      IMN0=IMNT1

      IYYM0=IMM0+100*ICCYY0
      IIDHMN0=IMN0+IHH0*100+IDD0*10000
      LLFOUND=.FALSE.
      DO J=1,ITIM
        IF (IYYM0 == IINFO(J,2,1) .AND. IIDHMN0 == IINFO(J,3,1)) THEN
          IADD=IINFO(J,1,1)
          LLFOUND=.TRUE.
          EXIT
        ENDIF
      ENDDO
      IF (.NOT.LLFOUND) THEN
        CALL ABOR1(' UPDCLIE_CO2 : FIRST FILE NOT FOUND')
      ENDIF
    ENDIF

    IRET=0
    CALL PBSEEK(IUNITCO2,IADD,0,IRET)
    IF(IRET < 0) THEN
      WRITE(NULERR,'(A,I2)')'UPDCLIE_CO2: PROBLEM IN PBSEEK, IRET=',IRET
      CALL ABOR1('UPDCLIE_CO2: PROBLEM IN PBSEEK')
    ENDIF


  ENDIF MYPROCIF

ENDIF SCANIF

!*    3. READ THE FILE, IF NEEDED.
!     -----------------------------


READIF: IF(JPCLIMCO2 >= 1) THEN

!* Decide if reading time

  IF (LLFIRST) THEN
    ILASTREFD=0_JPIM
    ILASTREFT=0_JPIM
    LLREAD=.TRUE.
    IFIRST=2
  ELSE
    LLREAD=.FALSE.
  ENDIF


!*    3.1 READ, DECODE AND SELECT THE FIELDS

  ILENGG=NGPTOTG
  ILENBYT=NINTLEN*ILENGG
  IF (LLREAD) THEN
    FIRST: DO JF=1,IFIRST
      NPCO2_1=3-NPCO2_1
      NPCO2_2=3-NPCO2_2


      READ: DO JCL=1,JPCLIMCO2
        IPARAM=NGRBCO2NBF
        IF (MYPROC == 1) THEN
          IRET=0
          CALL PBGRIB(IUNITCO2,IGRIG,ILENBYT,ILONG,IRET)
          IF(IRET == 0) THEN
             ILONG=ILONG/NINTLEN
          ELSE
            WRITE(NULERR,'(A,I2)')'UPDCLIE_CO2: PROBLEM IN PBGRIB, IRET=',IRET
            CALL ABOR1('UPDCLIE_CO2: PROBLEM IN PBGRIB')
          ENDIF

          ZSEC3(2)=-9999._JPRB
          CALL GSTATS(1703,0)
          CALL GRIBEX(ISEC0,ISEC1,ISEC2,ZSEC2,ISEC3,ZSEC3,ISEC4,&
           & ZBUF,ILENGG,IGRIG,ILONG,IWORD,'D',IERR)  
          CALL GSTATS(1703,1)
          IPAR=ISEC1(6)
          IF(ISEC1(1) == 210) IPAR=IPAR+210000

          IYEARG=(ISEC1(21)-1_JPIM)*100+ISEC1(10)


          IYYM=IYEARG*100+ISEC1(11)
          IDHMN=ISEC1(14)+ISEC1(13)*100+ISEC1(12)*10000

!        distribute field to nodes

          CALL ISNDGPF(ZBUF,ZFLD)

        ENDIF

        

        ILASTREFD=IAT2*100+IMT2
        ILASTREFT=IMNT2+IHT2*100+IJT2*10000

        
! ALL PROCESSORS SHOULD NOW RECEIVE

        CALL IRCVGPF(CLIMRCO2(1,NPCO2_2,JCL),ZFLD,1)

      ENDDO READ
    ENDDO FIRST
  ENDIF

ENDIF READIF


!*
!     3. TIME INTERPOLATION AND WRITING OF CLIMATE FIELDS
!     ---------------------------------------------------

call hourdiff(IAT2R,IMT2,IJT2,IHT2,IAN,IMOIS,IJOUR,IHOUR,IDIF,iret)
call hourdiff(IAT2R,IMT2,IJT2,IHT2,IAT1R,IMT1,IJT1,IHT1,IDIFD,iret)

IDIF=IDIF*60_JPIM+(IMNT2-IMIN)

IDIFD=IDIFD*60_JPIM+(IMNT2-IMNT1)

ZPOID1=REAL(IDIF,JPRB)/REAL(IDIFD,JPRB)
ZPOID2=1.0_JPRB-ZPOID1


DO JSTGLO=1,NGPTOT,NPROMA
  IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
  IST=1
  IBL=(JSTGLO-1)/NPROMA+1

! Natural biosphere CO2 flux 
! this program reads only one field 

  JCL=1

  DO JROF =1,IEND
    IF(CLIMRCO2(JSTGLO+JROF-1,NPCO2_1,JCL) < -9998_JPRB .OR. CLIMRCO2(JSTGLO+JROF-1,NPCO2_2,JCL) < -9998_JPRB) THEN
      ZCO2=-9999._JPRB
    ELSE
      ZCO2=CLIMRCO2(JSTGLO+JROF-1,NPCO2_1,JCL)*ZPOID1 &
     & +CLIMRCO2(JSTGLO+JROF-1,NPCO2_2,JCL)*ZPOID2  
    ENDIF
    SD_VF(JROF,YSD_VF%YCO2B%MP,IBL)    =ZCO2
  ENDDO

ENDDO

LLFIRST=.FALSE.

IRET=0
IF( MYPROC == 1 .AND. LLOPEN) THEN
  CALL PBCLOSE(IUNITCO2,IRET)
  IF( IRET /= 0 )THEN
      CALL ABOR1(' UPDCLIE_CO2: ERROR ON PBCLOSE')
  ENDIF
ENDIF


CALL FLUSH(NULOUT)

IF (LHOOK) CALL DR_HOOK('UPDCLIE_CO2',1,ZHOOK_HANDLE)
END SUBROUTINE UPDCLIE_CO2
