SUBROUTINE UPDO3CH

!**** *UPDO3CH*

!     PURPOSE.
!     --------

!     Updates the climatological O3 data for the parametrization 
!     of ozone chemistry

!**   INTERFACE.
!     ----------

!     CALL UPDO3CH

!     METHOD.
!     -------

!     Reads the monthly values of the 7(8) ozone chemistry fields.
!     For a given day, the value is interpolated linearly between the 15th
!     of the nearest months. Values for all months are on unit NULCL1.

!     EXTERNALS.
!     ----------

!     UPDCAL

!     AUTHORS.
!     --------

!       A. UNTCH (ECMWF) APR 96. (based on routines UPDCLI by M. DEQUE
!                                 and UPDCLIE by P. VITERBO)
!     MODIFICATIONS.
!     --------------
!   A. UNTCH   15.10.96 :  Changed reading of chemistry data
!   L. ISAKSEN 18.10.96 :  Adapted for message passing
!   E. HOLM     7.11.00 :  Added fields for heterogeneous chemistry
!   D.Dent     00-09-01 : MPE to MPL conversion
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   D.Salmond     22-Nov-2005 Mods for coarser/finer physics
!   A. UNTCH  14.12.2006: Scaling of the equivalent chlorine content by
!                         2ppbv/2.5ppbv=0.8 (recommended by D. Cariolle)
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   A. Untch (Oct 2008): Mods for version 2.9
! ---------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NFLEVG  ,NSMAX , NDLON
USE YOMDYN   , ONLY : TSTEP
USE YOMLUN   , ONLY : NULOUT   ,NUO3CH1  ,NUO3CH2
USE YOMGEM   , ONLY : NSTTYP   ,NLOENG
USE YOMCST   , ONLY : RDAY
USE YOMDPHY  , ONLY : NTOZ2D   ,NTOZ3D, NVCLIS
USE YOMCT0   , ONLY : NPROC    ,LTWOTL ,LNF 
USE YOMCT2   , ONLY : NSTAR2
USE YOMRIP   , ONLY : NINDAT   ,NSTADD
USE YOMOZO   , ONLY : TOZ2DG   ,TEECL2
USE YOMMP    , ONLY :   MYPROC   
USE MPL_MODULE , ONLY :  MPL_BROADCAST
USE YOMCOAPHY   , ONLY : NPHYINT
USE YOM_PHYS_GRID ,ONLY : PHYS_GRID
USE YOEPHY   , ONLY : LO3CH_OLDVER

! ---------------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB),ALLOCATABLE :: ZFLD1(:,:), ZFLD2(:,:)
REAL(KIND=JPRB) :: ZEECL(1950:2100)
INTEGER(KIND=JPIM) :: ILMOIS(12)
CHARACTER :: CLNOMF*34,CLPREF*5,CLVERS*5

INTEGER(KIND=JPIM) :: IA0, IAN, ICH, ICHF1, ICHF2, IJ0, IJOUR,&
 & IJT1, IJT2, ILATF1, ILATF2, ILEVF1, ILEVF2, &
 & IM0, IMOIS, IMONTH1, IMONTH2, IMT1, &
 & IMT2, IOMASTER, IPNCH1, IPNCH2, ITAG1, &
 & ITIME, IZT, ISTADD, JCH, JGL, JJ, JLEV, IDGLG, IERR 

REAL(KIND=JPRB) :: ZPOID1, ZPOID2
REAL(KIND=JPRB) :: ZHOOK_HANDLE
LOGICAL :: LLGRID

DATA ZEECL / 681.60_JPRB, 690.98_JPRB, 700.36_JPRB, 709.74_JPRB, 719.12_JPRB,&
 & 728.50_JPRB, 741.92_JPRB, 755.34_JPRB, 768.76_JPRB, 782.18_JPRB,&
 & 795.60_JPRB, 822.52_JPRB, 849.44_JPRB, 876.36_JPRB, 903.28_JPRB,&
 & 930.20_JPRB, 988.44_JPRB,1046.68_JPRB,1104.92_JPRB,1163.16_JPRB,&
 & 1221.40_JPRB,1327.18_JPRB,1432.96_JPRB,1538.74_JPRB,1644.52_JPRB,&
 & 1750.30_JPRB,1867.36_JPRB,1984.42_JPRB,2101.48_JPRB,2218.54_JPRB,&
 & 2335.60_JPRB,2431.50_JPRB,2527.40_JPRB,2623.30_JPRB,2719.20_JPRB,&
 & 2815.10_JPRB,2925.90_JPRB,3036.70_JPRB,3147.50_JPRB,3258.30_JPRB,&
 & 3369.10_JPRB,3405.96_JPRB,3442.82_JPRB,3479.68_JPRB,3516.54_JPRB,&
 & 3553.40_JPRB,3522.72_JPRB,3492.04_JPRB,3461.36_JPRB,3430.68_JPRB,&
 & 3400.00_JPRB,3390.34_JPRB,3380.68_JPRB,3371.02_JPRB,3361.36_JPRB,&
 & 3351.70_JPRB,3338.02_JPRB,3324.34_JPRB,3310.66_JPRB,3296.98_JPRB,&
 & 3283.30_JPRB,3256.12_JPRB,3228.94_JPRB,3201.76_JPRB,3174.58_JPRB,&
 & 3147.40_JPRB,3109.38_JPRB,3071.36_JPRB,3033.34_JPRB,2995.32_JPRB,&
 & 2957.30_JPRB,2916.84_JPRB,2876.38_JPRB,2835.92_JPRB,2795.46_JPRB,&
 & 2755.00_JPRB,2718.30_JPRB,2681.60_JPRB,2644.90_JPRB,2608.20_JPRB,&
 & 2571.50_JPRB,2539.44_JPRB,2507.38_JPRB,2475.32_JPRB,2443.26_JPRB,&
 & 2411.20_JPRB,2383.30_JPRB,2355.40_JPRB,2327.50_JPRB,2299.60_JPRB,&
 & 2271.70_JPRB,2245.00_JPRB,2218.30_JPRB,2191.60_JPRB,2164.90_JPRB,&
 & 2138.20_JPRB,2112.82_JPRB,2087.44_JPRB,2062.06_JPRB,2036.68_JPRB,&
 & 2011.30_JPRB,1989.00_JPRB,1966.70_JPRB,1944.40_JPRB,1922.10_JPRB,&
 & 1899.80_JPRB,1880.02_JPRB,1860.24_JPRB,1840.46_JPRB,1820.68_JPRB,&
 & 1800.90_JPRB,1783.34_JPRB,1765.78_JPRB,1748.22_JPRB,1730.66_JPRB,&
 & 1713.10_JPRB,1697.36_JPRB,1681.62_JPRB,1665.88_JPRB,1650.14_JPRB,&
 & 1634.40_JPRB,1620.10_JPRB,1605.80_JPRB,1591.50_JPRB,1577.20_JPRB,&
 & 1562.90_JPRB,1549.82_JPRB,1536.74_JPRB,1523.66_JPRB,1510.58_JPRB,&
 & 1497.50_JPRB,1485.46_JPRB,1473.42_JPRB,1461.38_JPRB,1449.34_JPRB,&
 & 1437.30_JPRB,1426.24_JPRB,1415.18_JPRB,1404.12_JPRB,1393.06_JPRB,&
 & 1382.00_JPRB,1371.76_JPRB,1361.52_JPRB,1351.28_JPRB,1341.04_JPRB,&
 & 1330.80_JPRB,1321.10_JPRB,1311.40_JPRB,1301.70_JPRB,1292.00_JPRB,&
 & 1282.30_JPRB /  

! ---------------------------------------------------------------------------

#include "fcttim.h"

! ---------------------------------------------------------------------------

#include "abor1.intfb.h"
#include "updcal.intfb.h"

! ---------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('UPDO3CH',0,ZHOOK_HANDLE)

! ---------------------------------------------------------------------------

!*     1. SETTING CONSTANT VALUES.
!     ---------------------------

IERR=0

IF(.NOT.LNF.AND.NSTADD == 0) THEN
! IN CASE OF RESTART:
  ITIME=NINT(TSTEP)
  IF (LTWOTL) THEN
    IZT=NINT(TSTEP*(REAL(NSTAR2,JPRB)+0.5_JPRB))
  ELSE
    IZT=ITIME*NSTAR2
  ENDIF
  ISTADD=IZT/NINT(RDAY)
ELSE
  ISTADD=NSTADD
ENDIF

!*    1.1 Calendar

IJ0=NDD(NINDAT)
IM0=NMM(NINDAT)
IA0=NCCAA(NINDAT)
CALL UPDCAL(IJ0,IM0,IA0,ISTADD,IJOUR,IMOIS,IAN,ILMOIS,NULOUT)
IF(IJOUR >= 15)THEN
  IMT1=IMOIS
  IMT2=1+MOD(IMOIS,12)
  IJT1=15
  IJT2=15+ILMOIS(IMT1)
ELSE
  IMT1=1+MOD(IMOIS+10,12)
  IMT2=IMOIS
  IJT1=15-ILMOIS(IMT1)
  IJT2=15
ENDIF

WRITE(NULOUT,*) ' UPDO3CH: IJOUR=',IJOUR,' IMOIS=',IMOIS,' IAN=',IAN
WRITE(NULOUT,*) ' UPDO3CH: IMT1=',IMT1,' IMT2=',IMT2

! WEIGHTS FOR TIME INTERPOLATION

ZPOID1=REAL(IJT2-IJOUR,JPRB)/REAL(IJT2-IJT1,JPRB)
ZPOID2=1.0_JPRB-ZPOID1
WRITE(NULOUT,*) ' ZPOID1=',ZPOID1,' ZPOID2=',ZPOID2

!*    1.2 Set (equivalent chlorine content)**2 for the present year

IF(IAN < 1950) IAN=1950
IF(IAN > 2100) IAN=2100

IF(LO3CH_OLDVER) THEN
! Scaling with 2ppbv/2.5ppbv=0.8 only needed for version 2.3
  TEECL2 = (ZEECL(IAN)*0.8_JPRB)**2
ELSE
! For version 2.9 the scaling with 0.8 is incorporated 
! in the coefficients.
  TEECL2 = ZEECL(IAN)**2
ENDIF

!*
!       2. READING MONTHLY VALUES AND TIME INTERPOLATION 
!       ------------------------------------------------

IF(LO3CH_OLDVER) THEN
  CLVERS=''
ELSE
  CLVERS='_V2.9'
ENDIF

LLGRID=NSMAX > (NDLON+3)/3
IF(LLGRID) THEN
  CLPREF='l_2'
ELSEIF(NDLON == NLOENG(1)) THEN
  CLPREF='_full'
ELSE
  CLPREF='_2'
ENDIF
 IF(NPHYINT == 0) THEN
  IDGLG=NDGLG
 ELSE
   IDGLG=PHYS_GRID%NDGLG
   DEALLOCATE(TOZ2DG)
   ALLOCATE(TOZ2DG(NFLEVG*NVCLIS*NTOZ2D,PHYS_GRID%NDGSAG:PHYS_GRID%NDGENG))
 ENDIF
ALLOCATE(ZFLD1(IDGLG,NFLEVG))
ALLOCATE(ZFLD2(IDGLG,NFLEVG))
IOMASTER=MIN(1,NPROC)
IF(MYPROC == IOMASTER) THEN

!*      2.1 OPEN CLIMATE FILE MONTH 1

  WRITE(UNIT=NULOUT,FMT='(A)') ' READ O3 CHEM. FIELDS '
!   WRITE(UNIT=CLNOMF,FMT='(''O3CHEM'',I2.2)')IMT1
   IF(NPHYINT == 0) THEN
    IF(NSMAX < 100) THEN
      WRITE(UNIT=CLNOMF,FMT='(I2.2,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
        &NSMAX,TRIM(CLPREF),NFLEVG,IMT1,CLVERS
    ELSEIF(NSMAX < 1000) THEN
      WRITE(UNIT=CLNOMF,FMT='(I3.3,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
        &NSMAX,TRIM(CLPREF),NFLEVG,IMT1,CLVERS
    ELSE
      WRITE(UNIT=CLNOMF,FMT='(I4.4,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
        &NSMAX,TRIM(CLPREF),NFLEVG,IMT1,CLVERS
    ENDIF
   ELSE
     IF(PHYS_GRID%NSMAX < 100) THEN
       WRITE(UNIT=CLNOMF,FMT='(I2.2,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
         &PHYS_GRID%NSMAX,TRIM(CLPREF),NFLEVG,IMT1,CLVERS
     ELSEIF(PHYS_GRID%NSMAX < 1000) THEN
       WRITE(UNIT=CLNOMF,FMT='(I3.3,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
         &PHYS_GRID%NSMAX,TRIM(CLPREF),NFLEVG,IMT1,CLVERS
     ELSE
       WRITE(UNIT=CLNOMF,FMT='(I4.4,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
         &PHYS_GRID%NSMAX,TRIM(CLPREF),NFLEVG,IMT1,CLVERS
     ENDIF
   ENDIF
  WRITE(UNIT=NULOUT,FMT='(A,A)')&
     & ' INITIAL DATA FOR MONTH 1 TO BE READ FROM FILE ',CLNOMF  
  

  OPEN(NUO3CH1,FILE=CLNOMF,FORM='UNFORMATTED',STATUS='OLD',IOSTAT=IERR)
  IF(IERR > 0) CALL ABOR1(' UPDO3CH: PROBLEM OPENING O3CHEM FILE FOR FIRST MONTH' )
  READ(NUO3CH1) IMONTH1
  READ(NUO3CH1) IPNCH1

!*      2.2 OPEN CLIMATE FILE MONTH 2

!  WRITE(UNIT=CLNOMF,FMT='(''o3chem_l'',I2.2,''/O3CHEM'',I2.2)')NFLEVG,IMT2
  IF(NPHYINT == 0) THEN
    IF(NSMAX < 100) THEN
      WRITE(UNIT=CLNOMF,FMT='(I2.2,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
        &NSMAX,TRIM(CLPREF),NFLEVG,IMT2,CLVERS
    ELSEIF(NSMAX < 1000) THEN
      WRITE(UNIT=CLNOMF,FMT='(I3.3,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
        &NSMAX,TRIM(CLPREF),NFLEVG,IMT2,CLVERS
    ELSE
      WRITE(UNIT=CLNOMF,FMT='(I4.4,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
        &NSMAX,TRIM(CLPREF),NFLEVG,IMT2,CLVERS
    ENDIF
  ELSE
    IF(PHYS_GRID%NSMAX < 100) THEN
      WRITE(UNIT=CLNOMF,FMT='(I2.2,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
        &PHYS_GRID%NSMAX,TRIM(CLPREF),NFLEVG,IMT2,CLVERS
    ELSEIF(PHYS_GRID%NSMAX < 1000) THEN
      WRITE(UNIT=CLNOMF,FMT='(I3.3,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
        &PHYS_GRID%NSMAX,TRIM(CLPREF),NFLEVG,IMT2,CLVERS
    ELSE
      WRITE(UNIT=CLNOMF,FMT='(I4.4,A,''/o3chem_l'',I2.2,''/O3CHEM'',I2.2,A)') &
        &PHYS_GRID%NSMAX,TRIM(CLPREF),NFLEVG,IMT2,CLVERS
    ENDIF
  ENDIF
  WRITE(UNIT=NULOUT,FMT='(A,A)')&
   & ' INITIAL DATA FOR MONTH 2 TO BE READ FROM FILE ',CLNOMF  

  OPEN(NUO3CH2,FILE=CLNOMF,FORM='UNFORMATTED',STATUS='OLD',IOSTAT=IERR)
  IF(IERR > 0) CALL ABOR1(' UPDO3CH: PROBLEM OPENING O3CHEM FILE FOR SECOND MONTH' )
  READ(NUO3CH2) IMONTH2
  READ(NUO3CH2) IPNCH2

  WRITE(NULOUT,*) ' UPDO3CH: IMONTH1=',IMONTH1,' IMT1=',IMT1
  WRITE(NULOUT,*) ' UPDO3CH: IMONTH2=',IMONTH2,' IMT2=',IMT2
  IF(IPNCH2 /= IPNCH1) THEN
    CALL ABOR1(' UPDO3CH: IPNCH2 /= IPNCH1')
  ENDIF
  WRITE(NULOUT,*) ' UPDO3CH: IPNCH1=',IPNCH1,' NVCLIS=',NVCLIS

!*      2.3 READ THE FIELDS FOR MONTH 1 AND MONTH 2

!    LOOP OVER FIELDS
  DO JCH=1,NVCLIS

! MONTH 1

    READ(NUO3CH1)ICHF1,ILEVF1,ILATF1
    WRITE(NULOUT,FMT='('' ICHF1'',I3,'' ILEVF1'',I3,'' ILATF1'',I4 &
     & )')  ICHF1,ILEVF1,ILATF1  

    READ(NUO3CH1)

! MONTH 2

    READ(NUO3CH2)ICHF2,ILEVF2,ILATF2
    WRITE(NULOUT,FMT='('' ICHF2'',I3,'' ILEVF2'',I3,'' ILATF2'',I4 &
     & )')  ICHF2,ILEVF2,ILATF2  

    READ(NUO3CH2)

! CHECKING FOR CONSISTENCY
    IF(ICHF2 /= ICHF1.OR.ICHF1 /= JCH) THEN
      CALL ABOR1(' UPDO3CH: ICHF2 /= ICHF1.OR.ICHF1 /= JCH')
    ELSEIF (ILEVF2 /= ILEVF1.OR.ILEVF1 /= NFLEVG) THEN
      CALL ABOR1(' UPDO3CH: ILEVF2 /= ILEVF1.OR.ILEVF1 /= NFLEVG')
    ELSEIF (ILATF2 /= ILATF1.OR.ILATF1 /= IDGLG) THEN
      CALL ABOR1(' UPDO3CH: ILATF2 /= ILATF1.OR.ILATF1 /= NDGLG')
    ENDIF
    DO JLEV=1,ILEVF1
      READ (NUO3CH1)(ZFLD1(JJ,JLEV),JJ=1,ILATF1)
      READ (NUO3CH2)(ZFLD2(JJ,JLEV),JJ=1,ILATF2)

!  TIME INTERPOLATION AND WRITING TO COMMON YOMOZO

      ICH=(ICHF1-1)*ILEVF1+JLEV
      IF(NTOZ3D == 1)THEN
!  FULL 3D COMMON
! THIS VERSION IS NOT ADAPTED FOR MESSAGE PASSING
        CALL ABOR1(&
         & ' UPDO3CH: VERSION NTOZ3D == 1 NOT ADAPTED FOR MESSAGE PASSING')  
      ENDIF

      IF(NTOZ2D == 1)THEN
!  LATITUDExLEVEL COMMON 
!  ADAPTED FOR MESSAGE PASSING
        IF(NSTTYP /= 1) THEN
          WRITE(NULOUT,FMT='('' OZONE INCONSISTENCY:&
           & NTOZ2D='',I4,'' NSTTYP = '',I4)')NTOZ2D,NSTTYP  
          CALL ABOR1('UPDO3CH: ABOR1 CALLED')
        ENDIF
        DO JGL=1,IDGLG
          TOZ2DG(ICH,JGL)=ZFLD1(JGL,JLEV)*ZPOID1 + ZFLD2(JGL,JLEV)*ZPOID2
        ENDDO
      ENDIF

! END LOOP OVER LEVELS
    ENDDO
! END LOOP OVER FIELDS
  ENDDO

!*    2.4 CLOSE CLIMATE FILES

  CLOSE(NUO3CH1)
  CLOSE(NUO3CH2)
ENDIF

!*    2.5 DISTRIBUTION OF FIELDS FROM IOMASTER TO ALL OTHER PE'S

IF(NPROC > 1) THEN
  CALL MPL_BROADCAST(TOZ2DG,KROOT=IOMASTER,KTAG=ITAG1,CDSTRING='UPDO3CH:')
ENDIF

DEALLOCATE(ZFLD1)
DEALLOCATE(ZFLD2)

! ---------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('UPDO3CH',1,ZHOOK_HANDLE)
END SUBROUTINE UPDO3CH
