#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE GP_MODEL_TL(CDCONF)

! GP_MODEL_TL - Model (Tangent Linear)

! Purpose
! -------
!   Interface to Tangent linear grid-point model

! Interface
! ---------
!   CDCONF - configuration of work (see doc.)

! Externals
! ---------
!   See the "include" topic below.

! Method
! ------

! Reference
! ---------
!   ECMWF Research Department documentation of the IFS

! Author
! ------
!   Deborah Salmond (ECMWF)

! Modifications
! -------------
!   06-Mar-2001 C. Fischer  Merge LAM version into global code
!      Mar-2002 J. Vivoda   PC schemes for NH dynamics (LPC_XXXX keys)
!   01-Oct-2003 M. Hamrud   CY28 Cleaning
!   10-Jun-2004 J. Masek    NH cleaning (LPC_NOTR, LFULLIMP)
!   01-Jul-2004 K. Yessad   Make clearer the tests for PC scheme.
!      Jul-2004 K. Yessad   Modularization of CPGTL according to CPG.
!     C. Fischer   01-03-06: Merge LAM version into global code
!     J.Vivoda (03-2002) PC schemes for NH dynamics (LPC_XXXX keys) 
!     M.Hamrud     01-Oct-2003 CY28 Cleaning
!   03-03-2006  R.Brozkova set T1 arrays to zero for security (extension zone problem)
!   05-04-2002  M. Jidane  : CY31 Phasing
!   01-08-2006  D. Salmond : 4D-Var rain assimilation
!   04-Jun-2007 M. Jidane  : Fix NSLCORE bug
!   25-Apr-2008 A. Geer    : Multiple sensor 4D-Var rain assimilation
!   23-Sep-2008 A. Geer    : Allow ADTEST to work with 4D-Var rain assimilation
!   09-Mar-2009 W. Bell    : Add azimuth angle for windsat-like sensors
!   16-Dec-2009 F. Karbou  : Allow surface emissivity estimation
!   04-Dec-2009 H. Hersbach: Tag for surface field update
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK

USE YOEPHY   , ONLY : LEPHYS   ,LEMWAVE
USE YOMCT0   , ONLY : NCONF    ,LSLAG    ,LELAM
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NGPBLKS    
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB15,NFLDSLB2
USE YOMGC    , ONLY : RCORI    ,GEMU     ,GSQM2    ,GELAM    ,GECLO    , &
 &                    GESLO    ,GM       ,GNORDL   ,GNORDM   ,OROG     , &
 &                    OROGL    ,OROGM    ,GOMVRL   ,GOMVRM     
USE YOMGEM   , ONLY : RCOLON   ,RSILON   ,RATATH   ,RATATX   ,NGPTOT   , &
 &                    RINDX    ,RINDY    ,NGPTOT_CAP  
USE YOMGFL   , ONLY : GFL      ,GFL5     ,GFLT1
USE YOMGMV   , ONLY : GMV      ,GMVS     ,GMV5     ,GMV5S     ,GMVT1   , &
 &                    GMVT1S   ,YT0      ,YT1      ,YT9       ,YT5
USE YOMLCZ   , ONLY : GPFORCEU ,GPFORCEV ,GPFORCET ,GPFORCEQ   
USE YOMMP    , ONLY : NASLB1   ,NSLCORE  ,NSLPAD   
USE YOMVRTL  , ONLY : LOBSTL
USE SURFACE_FIELDS_MIX , ONLY : SP_SB,SP_SG,SP_RR,SD_VN,SD_VD,YDUPD
USE YOM_YGFL , ONLY : YGFL
USE YOPHNC   , ONLY : LETRAJP
USE PARMWAVE , ONLY : NMAX_CHAN, NMWAVE_P
USE YOMMWAVE , ONLY : LEMWAVE_ACTIVE, N_SATSENS
USE YOMTVRAD , ONLY : MNSERIES 
USE TESTVAR_MIX,   ONLY : LTESTADJM

!    -------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=9)  ,INTENT(IN)    :: CDCONF 

!    -------------------------------------------------------------------

REAL(KIND=JPRB), ALLOCATABLE :: ZSLBUF1AUX(:,:), ZSLBUF1(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSLBUF15AUX(:,:), ZSLBUF15(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSLBUF2(:,:,:)

INTEGER(KIND=JPIM) :: IBL, ICEND, ICENDE, IGPCOMP, &
 & IOFF, JKGLO, JROF, JFLD, JBL, J

LOGICAL :: LL2D

REAL(KIND=JPRB) :: ZZZ

REAL(KIND=JPRB), ALLOCATABLE :: ZMWAVE_TB_BUF_TL(:,:,:), &
  & ZMWAVE_OERR_BUF(:,:,:), ZMWAVE_ZEN_BUF(:,:),ZMWAVE_AZI_BUF(:,:),&
  & ZMWAVE_PERS1_BUF(:,:,:), ZMWAVE_P19_BUF(:,:,:), ZMWAVE_P37_BUF(:,:,:)
INTEGER(KIND=JPIM), ALLOCATABLE:: IMWAVE_BUF(:,:,:)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!    -------------------------------------------------------------------

#include "call_sl_tl.intfb.h"
#include "cpg2lagtl.intfb.h"
#include "cpg2tl.intfb.h"
#include "cpglagtl.intfb.h"
#include "cpgtl.intfb.h"
#include "ec_phys_tl.intfb.h"
#include "mwave_get_tl.intfb.h"
#include "mwave_put_tl.intfb.h"

!    -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GP_MODEL_TL',0,ZHOOK_HANDLE)

!    -------------------------------------------------------------------
!    -------------------------------------------------------------------
!*       1.    INITIALISATION
!    -------------------------------------------------------------------

IF(NCONF/100 == 2.OR.NCONF/10 == 42.OR.NCONF/10 == 52) THEN
  LL2D=.TRUE.
ELSE
  LL2D=.FALSE.
ENDIF

! Total number of grid points - ndglg*ndlon in global / ndguxg*ndlon in LAM
! used for model computations

! Changed NGPTOT into IGPCOMP in order to pass properly
! the scan of NSLCORE values for the LAM models
IF(.NOT.LELAM) THEN
  IGPCOMP=NGPTOT
ELSE
  IGPCOMP=MIN(NGPTOT,NGPTOT_CAP)
ENDIF

CALL GSTATS(1227,0)
IF(.NOT.ALLOCATED(GMVT1)) THEN
  ALLOCATE(GMVT1(NPROMA,NFLEVG,YT1%NDIM,NGPBLKS))
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
  DO J=1,NGPBLKS
!cdir collapse
    GMVT1(:,:,:,J)=0.0_JPRB
  ENDDO
!$OMP END PARALLEL DO
ENDIF
IF(.NOT.ALLOCATED(GMVT1S)) THEN
  ALLOCATE(GMVT1S(NPROMA,YT1%NDIMS,NGPBLKS))
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
  DO J=1,NGPBLKS
!cdir collapse
    GMVT1S(:,:,J)=0.0_JPRB
  ENDDO
!$OMP END PARALLEL DO
ENDIF
IF(.NOT.ALLOCATED(GFLT1)) THEN
  ALLOCATE(GFLT1(NPROMA,NFLEVG,YGFL%NDIM1,NGPBLKS))
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
  DO J=1,NGPBLKS
!cdir collapse
    GFLT1(:,:,:,J)=0.0_JPRB
  ENDDO
!$OMP END PARALLEL DO
ENDIF

IF (.NOT.ALLOCATED(ZSLBUF1)) ALLOCATE(ZSLBUF1(NASLB1,NFLDSLB1))
IF (NSLPAD > 0)THEN
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
  DO J=1,NFLDSLB1
    ZSLBUF1(:,J)=HUGE(ZZZ)
  ENDDO
!$OMP END PARALLEL DO
ENDIF
IF (.NOT.ALLOCATED(ZSLBUF15)) ALLOCATE(ZSLBUF15(NASLB1,NFLDSLB15))
IF (NSLPAD > 0)THEN
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
  DO J=1,NFLDSLB15
    ZSLBUF15(:,J)=HUGE(ZZZ)
  ENDDO
!$OMP END PARALLEL DO
ENDIF

IF (.NOT.ALLOCATED(ZSLBUF2)) ALLOCATE(ZSLBUF2(NPROMA,NFLDSLB2,NGPBLKS))
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
DO J=1,NGPBLKS
!cdir collapse
  ZSLBUF2(:,:,J)=0.0_JPRB
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1227,1)

IF(LEMWAVE .AND. CDCONF(6:6) == 'V' .AND. MNSERIES > 0 &
 & .AND. .NOT. LTESTADJM) THEN
  LEMWAVE_ACTIVE=.TRUE.
ELSE
  LEMWAVE_ACTIVE=.FALSE.
ENDIF 
ALLOCATE( ZMWAVE_TB_BUF_TL(NGPTOT,NMAX_CHAN,N_SATSENS) )
ALLOCATE( ZMWAVE_OERR_BUF(NGPTOT,NMAX_CHAN,N_SATSENS) )
ALLOCATE( ZMWAVE_P19_BUF(NGPTOT,NMWAVE_P,N_SATSENS) )
ALLOCATE( ZMWAVE_P37_BUF(NGPTOT,NMWAVE_P,N_SATSENS) )
ALLOCATE( ZMWAVE_PERS1_BUF(NGPTOT,NMAX_CHAN,N_SATSENS) )
ALLOCATE( ZMWAVE_ZEN_BUF(NGPTOT,N_SATSENS) )
ALLOCATE( ZMWAVE_AZI_BUF(NGPTOT,N_SATSENS) )
ALLOCATE( IMWAVE_BUF(NGPTOT,NMAX_CHAN,N_SATSENS) )

!    -------------------------------------------------------------------
!*       2.    DYNAMICS
!    -------------------------------------------------------------------

CALL GSTATS(1006,0)

IF(.NOT.LL2D) THEN

  CALL CPGTL(&
   & CDCONF(4:4),IGPCOMP,&
   & GMV,GMVS,&
   & ZSLBUF1,ZSLBUF2,GMVT1,GMVT1S,GFLT1, &
   & ZSLBUF15,GMV5,GMV5S)

  IF(CDCONF(6:6) == 'V' .AND. (.NOT.LOBSTL)) THEN
    DO JBL=1,NGPBLKS
      GMV(:,:,1:YT0%NDIM,JBL)=GMV(:,:,1:YT0%NDIM,JBL)+GMV5(:,:,1:YT5%NDIM,JBL)
      GMVS(:,1:YT0%NDIMS,JBL)=GMVS(:,1:YT0%NDIMS,JBL)+GMV5S(:,1:YT5%NDIMS,JBL)
      GFL(:,:,1:YGFL%NDIM0,JBL)=GFL(:,:,1:YGFL%NDIM0,JBL)+&
       & GFL5(:,:,1:YGFL%NDIM5,JBL)  
    ENDDO
  ENDIF

ELSE

!$OMP PARALLEL PRIVATE(JKGLO,ICEND,IBL,IOFF,&
!$OMP&ZSLBUF1AUX,ZSLBUF15AUX,JROF,JFLD,ICENDE)

  IF (.NOT.ALLOCATED(ZSLBUF1AUX))  ALLOCATE(ZSLBUF1AUX(NPROMA,NFLDSLB1))
  IF (.NOT.ALLOCATED(ZSLBUF15AUX)) ALLOCATE(ZSLBUF15AUX(NPROMA,NFLDSLB15))

!$OMP DO SCHEDULE(DYNAMIC,1)
  DO JKGLO=1,IGPCOMP,NPROMA
    ICEND=MIN(NPROMA,IGPCOMP-JKGLO+1)
    ICENDE=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    IOFF=JKGLO

    CALL CPG2TL(CDCONF(4:4),ICEND,JKGLO,&
     & ZSLBUF1AUX,ZSLBUF2(1,1,IBL),GMV(1,1,1,IBL),GMVS(1,1,IBL),&
     & RCORI(IOFF),GM(IOFF),RATATH(IOFF),RATATX(IOFF),&
     & GOMVRL(IOFF),GOMVRM(IOFF),&
     & OROG(IOFF),OROGL(IOFF),OROGM(IOFF),&
     & GMV(1,1,YT0%MU,IBL)  ,GMV(1,1,YT0%MV,IBL),&
     & GMV(1,1,YT0%MUL,IBL) ,GMV(1,1,YT0%MVL,IBL),&
     & GMV(1,1,YT0%MDIV,IBL),&
     & GMV(1,1,YT0%MVOR,IBL),GMVS(1,YT0%MSP,IBL),&
     & GMVS(1,YT0%MSPL,IBL),&
     & GMVS(1,YT0%MSPM,IBL),&
     & GMV(1,1,YT9%MU,IBL) ,GMV(1,1,YT9%MV,IBL),&
     & GMV(1,1,YT9%MDIV,IBL),&
     & GMVS(1,YT9%MSP,IBL),&
     & GMVS(1,YT9%MSPL,IBL),GMVS(1,YT9%MSPM,IBL),&
     & GMVT1(1,1,YT1%MU,IBL)  ,GMVT1(1,1,YT1%MV,IBL),&
     & GMVT1S(1,YT1%MSP,IBL),&
     & GMV5(1,1,YT5%MU,IBL)  ,GMV5(1,1,YT5%MV,IBL),&
     & GMV5(1,1,YT5%MUL,IBL) ,GMV5(1,1,YT5%MVL,IBL),&
     & GMV5(1,1,YT5%MDIV,IBL),&
     & GMV5(1,1,YT5%MVOR,IBL),GMV5S(1,YT5%MSP,IBL),&
     & GMV5S(1,YT5%MSPL,IBL),&
     & GMV5S(1,YT5%MSPM,IBL))  

    !          move data from blocked form to latitude (NASLB1) form
    IF (LSLAG) THEN
      DO JFLD=1,NFLDSLB1
        DO JROF=JKGLO,MIN(JKGLO-1+NPROMA,IGPCOMP)
          ZSLBUF1(NSLCORE(JROF),JFLD)=ZSLBUF1AUX(JROF-JKGLO+1,JFLD)
        ENDDO
      ENDDO
    ENDIF

    IF(CDCONF(6:6) == 'V' .AND. (.NOT.LOBSTL)) THEN
      GMV(:,:,1:YT0%NDIM,IBL)=GMV(:,:,1:YT0%NDIM,IBL)+GMV5(:,:,1:YT5%NDIM,IBL)
      GMVS(:,1:YT0%NDIMS,IBL)=GMVS(:,1:YT0%NDIMS,IBL)+GMV5S(:,1:YT5%NDIMS,IBL)
      GFL(:,:,1:YGFL%NDIM0,IBL)=GFL(:,:,1:YGFL%NDIM0,IBL)+&
       & GFL5(:,:,1:YGFL%NDIM5,IBL)  
    ENDIF
  ENDDO
!$OMP END DO
  IF (ALLOCATED(ZSLBUF1AUX)) DEALLOCATE(ZSLBUF1AUX)
  IF (ALLOCATED(ZSLBUF15AUX)) DEALLOCATE(ZSLBUF15AUX)

!$OMP END PARALLEL

ENDIF

CALL GSTATS(1006,1)

!    -------------------------------------------------------------------
!*       3.    SEMI_LAGRANGIAN INTERPOLATION
!    -------------------------------------------------------------------

IF(LSLAG.AND..NOT.LL2D)THEN

  CALL CALL_SL_TL(ZSLBUF1,ZSLBUF15,ZSLBUF2,GFLT1 )

  DEALLOCATE(ZSLBUF1)
  DEALLOCATE(ZSLBUF15)

ENDIF

IF (LEMWAVE_ACTIVE) THEN 
    CALL MWAVE_GET_TL(IMWAVE_BUF,ZMWAVE_ZEN_BUF,ZMWAVE_AZI_BUF, &
      & ZMWAVE_P19_BUF,ZMWAVE_P37_BUF,ZMWAVE_OERR_BUF,ZMWAVE_PERS1_BUF)
ENDIF

!    -------------------------------------------------------------------
!*       4.    ECMWF Physics (Tangent Linear)
!    -------------------------------------------------------------------


IF(LEPHYS .AND. LETRAJP)THEN

  CALL GSTATS(1007,0)
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) PRIVATE(JKGLO,ICEND,IBL,IOFF)
  DO JKGLO=1,NGPTOT,NPROMA
    ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    IOFF=JKGLO

    IF(.NOT.LL2D) THEN
      CALL EC_PHYS_TL(CDCONF(4:4),IBL,ICEND,JKGLO,&
       & ZSLBUF2(1,1,IBL),&
       & GMV(1,1,1,IBL),GMVS(1,1,IBL),GFL(1,1,1,IBL),&
       & GMVT1(1,1,1,IBL),GMVT1S(1,1,IBL),GFLT1(1,1,1,IBL),&
       & SP_SB(1,1,1,IBL),SP_SG(1,1,IBL),SP_RR(1,1,IBL),&
       & SD_VN(1,1,IBL),SD_VD(1,1,IBL),&
       & GEMU(IOFF),GELAM(IOFF),GESLO(IOFF),GECLO(IOFF),OROG(IOFF),&
       & GPFORCEU(1,1,IBL,0),GPFORCEV(1,1,IBL,0),&
       & GPFORCET(1,1,IBL,0),GPFORCEQ(1,1,IBL,0),&  
       & ZMWAVE_ZEN_BUF, ZMWAVE_AZI_BUF,ZMWAVE_TB_BUF_TL,ZMWAVE_OERR_BUF,&
       & ZMWAVE_PERS1_BUF,IMWAVE_BUF)
    ENDIF
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1007,1)

  CALL GSTATS_BARRIER(731)

! Tag that surface fields have been updated
  IF(.NOT.LL2D)YDUPD%L_OK=.TRUE.
ENDIF

IF(LEMWAVE_ACTIVE)THEN
  CALL MWAVE_PUT_TL(ZMWAVE_TB_BUF_TL,IMWAVE_BUF)
ENDIF

!    -------------------------------------------------------------------
!*       5.    TRANSFER TO T+DT ARRAYS
!    -------------------------------------------------------------------

CALL GSTATS(1008,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JKGLO,ICEND,IBL,IOFF,ICENDE)
DO JKGLO=1,IGPCOMP,NPROMA
  ICEND=MIN(NPROMA,IGPCOMP-JKGLO+1)
  ICENDE=MIN(NPROMA,NGPTOT-JKGLO+1)
  IBL=(JKGLO-1)/NPROMA+1
  IOFF=JKGLO

  IF(.NOT.LL2D) THEN
    CALL CPGLAGTL(ICEND, &
     & ZSLBUF2(1,1,IBL),GM(IOFF),&
     & GMV(1,1,1,IBL),GMVS(1,1,IBL),GFL(1,1,1,IBL),&
     & GMVT1(1,1,1,IBL),GMVT1S(1,1,IBL))  

  ELSE
    CALL CPG2LAGTL(ICEND,NASLB1,&
     & ZSLBUF1,ZSLBUF2(1,1,IBL),GMV(1,1,1,IBL),GMVS(1,1,IBL),&
     & GM(IOFF),GEMU(IOFF),GSQM2(IOFF),GECLO(IOFF),&
     & GESLO(IOFF),GNORDL(IOFF),&
     & GNORDM(IOFF),RCOLON(IOFF),RSILON(IOFF),&
     & RINDX(IOFF),RINDY(IOFF),&
     & GMVT1(1,1,YT1%MU,IBL)  ,GMVT1(1,1,YT1%MV,IBL),&
     & GMVT1S(1,YT1%MSP,IBL))  
  ENDIF
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1008,1)


!    -------------------------------------------------------------------
!*       6.    DEALLOCATIONS
!    -------------------------------------------------------------------

IF (ALLOCATED(ZSLBUF1)) DEALLOCATE(ZSLBUF1)
IF (ALLOCATED(ZSLBUF15)) DEALLOCATE(ZSLBUF15)
IF (ALLOCATED(ZSLBUF2)) DEALLOCATE(ZSLBUF2)

DEALLOCATE( ZMWAVE_TB_BUF_TL, ZMWAVE_OERR_BUF, ZMWAVE_ZEN_BUF, IMWAVE_BUF )
DEALLOCATE( ZMWAVE_AZI_BUF,ZMWAVE_PERS1_BUF, ZMWAVE_P19_BUF,ZMWAVE_P37_BUF)

!    -------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GP_MODEL_TL',1,ZHOOK_HANDLE)
END SUBROUTINE GP_MODEL_TL
