SUBROUTINE RERESF

!**** *RERESF*  - read  restart files

!     Purpose.
!     --------
!     Read  a set of restart files if they exist:
!       (1) spectral fields
!       (2) upper-air grid-point fields (t-dt)
!       (3) surface fields
!       (4) upper-air grid-point fields (t)
!       (5) cumulated fluxes
!       (6) instantaneous fluxes
!       (7) restart control file

!**   Interface.
!     ----------
!        *CALL* *RERESF

!        Explicit arguments :      None.
!        --------------------

!        Implicit arguments :      None.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        David Dent *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-04-06
!        Modified : 92-06-04 V. Ducrocq and R. El Khatib (restart with physics)
!        Modified : 92-11-19 R. El Khatib (restart on cumulated fluxes)
!        Modified : 93-04-13 L. Labbe (new interface for SUOROG)
!        Modified : 93-09-06 R. El Khatib (restart on instantaneous fluxes)
!        Modified : 93-07-05 M.Hamrud (DDH restart)
!        Modified : 94-05-10 M.Hamrud (Add NSTEPLPP to NAMRCF)
!        Modified : 94-10-27 M.Hamrud (Restart for upper-air t grid-point)
!        Modified : 95-06-06 L.Isaksen   Reordering of spectral arrays
!        Modified : 97-04-18 B.Hansen    Reset begin date for WAM.
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!        - removal of LRPOLE in YOMCT0 and code under LRPOLE.
!        Modified 00-09-01 by D.Dent : MPE to MPL conversion
!        Modified : 01-06-13 R. El Khatib (ISP restart)
!        R. El Khatib : 01-08-07 Pruning options
!        Modified : 02-06-10 by D.Dent - Additional file for GP fields
!                            LFASTRES option and improved DDH handling
!        Modified : 08-2002 C. Smith : use "w" as prognostic variable in the
!         semi-lag advection of vertical divergence in the NH model.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning

!        Modified : 02-07-17 by P. Marquet : add VCLIA for aerosol files
!        Modified : 04-08-12 R. El Khatib Bugfixes + COPY
!        JJMorcrette 20060721 PP clear-sky PAR and TOA incident solar radiation
!        M.Hamrud      01-Jul-2006  Revised surface fields
!        JJMorcrette 20070321 Prognostic aerosols in rad and clouds
!        M.Hamrud      13-Oct-2008 Use IOSTREAM, one file per task, 
!                                  only FASTRES supported
!        JJMorcrette 20091201 Total and clear-sky direct SW radiation flux at surface 
!        M. Steinheimer Oct-2009  improved for stochastic physics options LSTOPH
!                                 and LSTOPH_SPBS
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT   ,NULRCF   ,NPPPSH,   NULERR
USE YOMCT0   , ONLY : NSTOP    ,NPRTRW   ,NPRTRV   ,&
 & NPRGPNS   ,NPRGPEW  ,LNF  
USE YOMCT2   , ONLY : NSTAR2
USE YOMDIM   , ONLY : NFLEVG    ,NGPBLKS  , NPROMA   ,&
 & NSPEC2
USE YOMSP    , ONLY : SPA3     ,SPA2     ,SPOR
USE YOMDPHY  , ONLY : NTILES
USE YOMSPSDT , ONLY : LSPSDT, YSPSDT_AR1
USE YOMIOS   , ONLY : CIOSPRF  ,CFRCF    ,CIOISPRF
USE YOMDYN   , ONLY : TSTEP
USE YOMGEM   , ONLY : NGPTOT
USE YOMRES   , ONLY : CTIME    ,CSTEP    ,COPY
USE YOMCFU   , ONLY : NFDCFU   ,LCUMFU
USE YOMGFUB  , ONLY : GFUBUF
USE YOMLDDH  , ONLY : LSDDH
USE YOMTDDH  , ONLY : HDCVB0   ,HDCVB1   ,HDCS0    ,HDCS1
USE YOMXFU   , ONLY : NFDXFU   ,LXFU
USE YOMXFUB  , ONLY : XFUBUF
USE YOMGRB   , ONLY : NSTEPLPP
USE YOMGFL   , ONLY : GFL,GFLSLP
USE YOMMP    , ONLY : MYPROC
USE YOMMSC   , ONLY : NINTLEN
USE YOEWCOU  , ONLY : NDURAT   ,LWCOU    ,CBEGDAT
USE YOMPPC   , ONLY : LMOVPH
USE YOM_YGFL , ONLY : YGFL     ,YGFLC
USE YOMSLPHY , ONLY : SAVTEND  ,LSLPHY
USE STOPH_MIX , ONLY : LSTOPH,  LSTOPH_SPBS, &
 & SPSTREAM, SPSTREAM_FORC, GPTOTDISS, &
 & GPTOTDISS_SMOOTH, GPSTREAM, GPVORTGRAD, RSTOPHU, RSTOPHV, RSTOPHT, RSTOPHQ, &
 & LSTOPH_SPBS_T, SPTEMP, SPTEMP_FORC, GPTEMP !, LSTOPH_CASBS, LVORTCON
USE YOMRADF  , ONLY : EMTD     ,TRSW     ,EMTC     ,TRSC     ,TAUAER   ,&
 & SRSWD    ,SRSWDCS ,EDRO  , SRSWPAR, SRSWUVB, SRSWPARC, SRSWTINC, SRFDIR, SRCDIR
USE YOE_TILE_PROP , ONLY : RUSTRTI, RVSTRTI, RAHFSTI, REVAPTI, RTSKTI
USE SURFACE_FIELDS_MIX  , ONLY : GPOPER, NDIMSURFL
USE YOMOP    , ONLY : CFNDDH   ,LINC
USE YOEPHY   , ONLY : LEPHYS
USE YOEAERSRC, ONLY : LEPAERO
USE YOMGMV   , ONLY : GMV      ,GMVS     ,YT0      ,YT9
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_GET, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST,CLOSE_IOREQUEST
USE YOMMASS  , ONLY : GMASS0, GMASSI
USE YOMRANDOM_STREAMS, ONLY  : YD_RANDOM_STREAM_STOCHPHYS, YD_RANDOM_STREAM_STOCHPHYS_SPBS, &
                             & YD_RANDOM_STREAM_STOCHPHYS_RVP, YD_RANDOM_STREAM_STOPH_SPBS_T, &
                             & YD_RANDOM_STREAM_STOPH_RVP_T
USE YOMRIP   , ONLY : NYMLPPGP, NYMLPPSP
USE RANDOM_NUMBERS_MIX, ONLY : RANDOM_NUMBER_RESTARTFILE

!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER (LEN = 80)   ::  CLFN
CHARACTER (LEN = 5)    ::  CLMYPROC
CHARACTER (LEN = 80)   ::  CLFNISP
CHARACTER (LEN = 200)  ::  CLSHELL
CHARACTER (LEN = 14)   ::  CLNMFI
CHARACTER (LEN = 4)    ::  CLNAME
CHARACTER (LEN = 6000) ::  CLDATA

REAL(KIND=JPRB), ALLOCATABLE :: ZBUF(:),ZBUF2(:,:),ZBUF3(:,:,:)

REAL(KIND=JPRB) :: ZDATA

INTEGER(KIND=JPIM) :: IB1PAR(18)
INTEGER(KIND=JPIM),ALLOCATABLE :: IPSCOD(:)

INTEGER(KIND=JPIM) :: I, IEND, IPROCA, IPROCB, &
 & IRET, ISDDH, IT, IUNIT, IWSTEP, JLEV, &
 & JSTGLO, IST,  IBL, JGFL, ISTEP,II,IINC,IUNITIN,ILENBYT,IDATA, &
 & IBYTE,ILENPS,ILENCH,ILENFP,ILENIN,ILENLG,IRSERV,IBIT,IWORD,JBLK,&  
 & ISIZE,JFLD,JROF,IBUF(1),IFLDS,IOFF,IBLSZ1,IBLSZ2,IBLSZ3,IBLSZ
TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST

! * PARAMETER INTEGER SCALARS:
INTEGER(KIND=JPIM),PARAMETER :: JPSCOD=500000

LOGICAL :: LLEXIST, LLFSTEP,LLOPENED, LLDAT
REAL(KIND=JPRB) :: ZSPOR(NSPEC2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "namrcf.h"

#include "abor1.intfb.h"
#include "ddhrcv.intfb.h"
#include "gpnorm3.intfb.h"
#include "sc2wrg.intfb.h"
#include "suorog.intfb.h"
!#include "surand2.intfb.h"

!     ------------------------------------------------------------------

!*       1.    READ  RESTART CONTROL FILE
!              --------------------------

IF (LHOOK) CALL DR_HOOK('RERESF',0,ZHOOK_HANDLE)
CALL GSTATS(28,0)
INQUIRE(FILE=CFRCF,EXIST=LLEXIST)

IF (LLEXIST) THEN

  LNF = .FALSE.
  OPEN  (NULRCF,FILE=CFRCF,DELIM="QUOTE")
  READ  (NULRCF,NAMRCF)
  CLOSE (NULRCF,STATUS='KEEP')

!        1.1   PERFORM CROSS-CHECKS ON NAMELIST VARIABLES

  IF(NPRGPNS /= IPRGPNSRES.OR.&
     & NPRGPEW /= IPRGPEWRES.OR.&
     & NPRTRW /= IPRTRWRES.OR.&
     & NPRTRV /= IPRTRVRES&
     & ) THEN  
!     must restart with same parallel configuration
    WRITE(NULOUT,'(A/A,I4.4/A,I4.4/A,I4.4/A,I4.4/A,L1/)') &
     & ' restart with '//&
     & ' different parallel configuration',&
     & 'NPRGPNS=',IPRGPNSRES,&
     & 'NPRGPEW=',IPRGPEWRES,&
     & 'NPRTRW=',IPRTRWRES,&
     & 'NPRTRV=',IPRTRVRES
    
    CALL ABOR1('RERESF: PROBLEM IN PARALLEL RESTART')
  ENDIF

!        1.2   MODIFY CONTROL VARIABLES FROM CONTENTS OF RCF

  READ(CSTEP,'(I9)') NSTAR2
  WRITE(NULOUT,'(A,I9.9)') ' READING RESTART FILES - STEP=',NSTAR2
  IT=INDEX(CTIME,' ')-1

!    ------------------------------------------------------------------

!*       3.    READ  SPECTRAL DATA
!              -------------------

  I = INDEX(CIOSPRF,' ')-1

  WRITE(CLMYPROC,'(A1,I4.4)')'.',MYPROC

  CLFN=CIOSPRF(1:I)//CTIME(1:IT)//CLMYPROC
  CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFN),CDMODE='r')
  CALL SETUP_IOREQUEST(YL_IOREQUEST,'RAW')
    
  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
  IF(IBUF(1) /= 88801) THEN
    CALL ABOR1('WRONG CHECK ON READ 1')
  ENDIF
  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=SPA3)
  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR2=SPA2)

  IF (LSPSDT) THEN
    CALL IO_GET(YL_IOSTREAM, YL_IOREQUEST,PR2=YSPSDT_AR1%SF%SP2D)
  ENDIF

!        3.1   COMPUTE OROGRAPHY IN GRID SPACE

  DO II=1,NSPEC2
    ZSPOR(II)=SPOR(II)
  ENDDO

  CALL SUOROG(ZSPOR)

!     ------------------------------------------------------------------

!*       4.    READ UPPER-AIR AND SURFACE DATA
!              -------------------------------

!*       4.1   READ UPPER-AIR T-DT GRID-POINT DATA

  IF(NSTAR2 > 0) THEN
    LLFSTEP = .FALSE.
  ELSE
    LLFSTEP = .TRUE.
  ENDIF

  IF(.NOT.LLFSTEP) THEN

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88802) THEN
      CALL ABOR1('WRONG CHECK ON READ 2')
    ENDIF
    IBLSZ1=NPROMA*NFLEVG*YT9%NDIM
    IBLSZ2=NPROMA*YT9%NDIMS
    IBLSZ3=NPROMA*NFLEVG*YGFL%NDIM9
    IBLSZ =IBLSZ1+IBLSZ2+IBLSZ3
    ALLOCATE(ZBUF(NGPBLKS*IBLSZ))
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR1=ZBUF)

    DO JSTGLO=1,NGPTOT,NPROMA
      IBL=(JSTGLO-1)/NPROMA+1
      IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
      DO JFLD=1,YT9%NDIM
        DO JLEV=1,NFLEVG
          DO JROF=1,IEND
            II=(IBL-1)*IBLSZ &
             & +(JFLD-1)*NPROMA*NFLEVG+(JLEV-1)*NPROMA+JROF
            GMV(JROF,JLEV,YT0%NDIM+JFLD,IBL)= ZBUF(II)
          ENDDO
        ENDDO
      ENDDO
      DO JFLD=1,YT9%NDIMS
        DO JROF=1,IEND
          II=IBLSZ1+(IBL-1)*IBLSZ &
           & +(JFLD-1)*NPROMA+JROF
          GMVS(JROF,YT0%NDIMS+JFLD,IBL)=ZBUF(II)
        ENDDO
      ENDDO
      DO JFLD=1,YGFL%NDIM9
        DO JLEV=1,NFLEVG
          DO JROF=1,IEND
            II=IBLSZ1+IBLSZ2+(IBL-1)*IBLSZ &
             & +(JFLD-1)*NPROMA*NFLEVG+(JLEV-1)*NPROMA+JROF
            GFL(JROF,JLEV,YGFL%NDIM0+JFLD,IBL)=ZBUF(II)
          ENDDO
        ENDDO
      ENDDO

    ENDDO
    DEALLOCATE(ZBUF)
  ENDIF

!*       4.2   READ SURFACE GRID-POINT DATA



  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
  IF(IBUF(1) /= 88803) THEN
    CALL ABOR1('WRONG CHECK ON READ 3')
  ENDIF
  ALLOCATE(ZBUF3(NPROMA,NDIMSURFL,NGPBLKS))
  CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=ZBUF3)
  DO JBLK=1,NGPBLKS
    CALL GPOPER('PUTALLFLDS',KBL=JBLK,PFIELD=ZBUF3(:,:,JBLK))
  ENDDO
  DEALLOCATE(ZBUF3)
 

!*       4.3   READ UPPER-AIR T GRID-POINT DATA

  IF(YGFL%NUMGPFLDS > 0) THEN

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88804) THEN
      CALL ABOR1('WRONG CHECK ON READ 4')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR4=GFL(:,:,1:YGFL%NUMGPFLDS,:))

    WRITE(NULOUT,'(A)') ' RERESF : STATISTICS FOR ALL G.P. GFL FIELDS'
    DO JGFL=1,YGFL%NUMGPFLDS
      WRITE(NULOUT,'(A)') YGFLC(JGFL)%CNAME
      CALL GPNORM3 (GFL,YGFL%NDIM,JGFL)
    ENDDO
  ENDIF

!*       4.4   READ CUMULATED AND INSTANTANEOUS FLUXES GRID-POINT DATA

  IF (LCUMFU) THEN
    IF (NFDCFU > 0) THEN
      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
      IF(IBUF(1) /= 88805) THEN
        CALL ABOR1('WRONG CHECK ON READ 5')
      ENDIF
      ALLOCATE(ZBUF2(NPROMA,NFDCFU))
      DO JSTGLO=1,NGPTOT,NPROMA
        IST=1
        IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
        CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR2=ZBUF2)
        CALL SC2WRG (GFUBUF,JSTGLO,IST,IEND,ZBUF2)
      ENDDO
      DEALLOCATE(ZBUF2)
    ENDIF
  ENDIF


  IF (LXFU) THEN
    IF (NFDXFU > 0) THEN
      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
      IF(IBUF(1) /= 88806) THEN
        CALL ABOR1('WRONG CHECK ON READ 6')
      ENDIF
      ALLOCATE(ZBUF2(NPROMA,NFDXFU))
      DO JSTGLO=1,NGPTOT,NPROMA
        IST=1
        IEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
        CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR2=ZBUF2)
        CALL SC2WRG (XFUBUF,JSTGLO,IST,IEND,ZBUF2)
      ENDDO
      DEALLOCATE(ZBUF2)
    ENDIF
  ENDIF

!*       4.41  READ ADDITIONAL TENDENCIES AND RADIATION FIELD DATA

  IF (LEPHYS) THEN
    IF (LSLPHY) THEN
      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
      IF(IBUF(1) /= 88807) THEN
        CALL ABOR1('WRONG CHECK ON READ 7')
      ENDIF
      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR4=SAVTEND)
      
      IF(ALLOCATED(GFLSLP)) THEN
        IF(SIZE(GFLSLP) > 0) THEN
          CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR4=GFLSLP)
        ENDIF
      ENDIF
    ENDIF

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88808) THEN
      CALL ABOR1('WRONG CHECK ON READ 8')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=EMTD)
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=TRSW)
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=EMTC)
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=TRSC)

    IF (LEPAERO) THEN
      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR4=TAUAER)
    ENDIF

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88809) THEN
      CALL ABOR1('WRONG CHECK ON READ 9')
    ENDIF
    IFLDS=9
    ISIZE=NPROMA*NGPBLKS
    ALLOCATE(ZBUF(ISIZE*IFLDS))
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR1=ZBUF)
    IOFF=0
    SRSWDCS=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(SRSWDCS))
    IOFF=IOFF+ISIZE
    SRSWD=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(SRSWD))
    IOFF=IOFF+ISIZE
    EDRO=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(EDRO))
    IOFF=IOFF+ISIZE
    SRSWPAR=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(SRSWPAR))
    IOFF=IOFF+ISIZE
    SRSWUVB=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(SRSWUVB))
    IOFF=IOFF+ISIZE
    SRSWPARC=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(SRSWPARC))
    IOFF=IOFF+ISIZE
    SRSWTINC=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(SRSWTINC))
    IOFF=IOFF+ISIZE
    SRFDIR  =RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(SRFDIR))
    IOFF=IOFF+ISIZE
    SRCDIR  =RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(SRCDIR))
    DEALLOCATE(ZBUF)

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88810) THEN
      CALL ABOR1('WRONG CHECK ON READ 10')
    ENDIF
    IFLDS=5
    ISIZE=NPROMA*NGPBLKS*NTILES
    ALLOCATE(ZBUF(ISIZE*IFLDS))
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR1=ZBUF)
    IOFF=0
    RUSTRTI=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(RUSTRTI))
    IOFF=IOFF+ISIZE
    RVSTRTI=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(RVSTRTI))
    IOFF=IOFF+ISIZE
    RAHFSTI=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(RAHFSTI))
    IOFF=IOFF+ISIZE
    REVAPTI=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(REVAPTI))
    IOFF=IOFF+ISIZE
    RTSKTI=RESHAPE(ZBUF(IOFF+1:IOFF+ISIZE),SHAPE(RTSKTI))
    IOFF=IOFF+ISIZE
    DEALLOCATE(ZBUF)

  ENDIF

!*       4.5  READ  DDH RESTART DATA

  IF (LSDDH ) THEN 

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88811) THEN
      CALL ABOR1('WRONG CHECK ON READ 11')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IPROCA=IBUF(1)
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IPROCB=IBUF(1)
    
    ISDDH=1
    IF( IPROCA /= NPRTRW .OR. IPROCB /= NPRTRV )THEN
      WRITE(NULOUT,'("RERESF: NPRTRW/NPRTRV DIFFERENT ON ",&
       & "DDH RESTART FILE, DDH TURNED OFF")')  
      WRITE(NULERR,'("RERESF: NPRTRW/NPRTRV DIFFERENT ON ",&
       & "DDH RESTART FILE, DDH TURNED OFF")')  
      LSDDH=.FALSE.
      ISDDH=0
    ENDIF

    CALL DDHRCV(HDCVB0,SIZE(HDCVB0),YL_IOSTREAM,YL_IOREQUEST)
    CALL DDHRCV(HDCVB1,SIZE(HDCVB1),YL_IOSTREAM,YL_IOREQUEST)
    CALL DDHRCV(HDCS0 ,SIZE(HDCS0 ),YL_IOSTREAM,YL_IOREQUEST)
    CALL DDHRCV(HDCS1 ,SIZE(HDCS1 ),YL_IOSTREAM,YL_IOREQUEST)
    
  ENDIF

!*       4.51 READ DDH DATA FILE and POSITION CORRECTLY AT RESTART TIMESTEP

  IF ( LINC ) THEN
    IINC = NINT( REAL(NSTAR2,JPRB)*TSTEP/3600._JPRB )
  ELSE
    IINC = NSTAR2
  ENDIF
  WRITE (CLNMFI,'(A9,I4.4)') CFNDDH(1:9),IINC
  CLNMFI(4:5)='DL'
  IF(MYPROC == 1) INQUIRE(FILE=CLNMFI(1:9),EXIST=LLEXIST)
  IF (LLEXIST.AND.MYPROC == 1) THEN
!     Rename the DDH data file
    WRITE(CLSHELL,'(''mv '',A,'' '',A)') CLNMFI(1:9),CLNMFI
    CALL SYSTEM(CLSHELL) 
    CALL PBOPEN(IUNITIN,CLNMFI,'r',IRET)
    IF (IRET /= 0) THEN
      WRITE(*,*) ' PROBLEMS OPENING THE DDH DATA FILE',CLNMFI
      WRITE(*,*) ' IRET= ',IRET
      CALL ABOR1('PROBLEMS OPENING THE DDH DATA FILE')
    ENDIF 
    CALL PBOPEN(IUNIT,CLNMFI(1:9),'w',IRET)
    IF (IRET /= 0) THEN
      WRITE(*,*) ' PROBLEMS OPENING THE DDH NEW DATA FILE',CLNMFI(1:9)
      WRITE(*,*) ' IRET= ',IRET
      CALL ABOR1('PROBLEMS OPENING THE DDH NEW DATA FILE')
    ENDIF 
    ALLOCATE(IPSCOD(JPSCOD))
    READ_LOOP:DO
      IRET=0
      CALL PBPSEU(IUNITIN,IPSCOD,NINTLEN*JPSCOD,ILENBYT,IRET)
      if (IRET == -1) THEN
        EXIT READ_LOOP
      ELSEIF(IRET <= -1) THEN
        WRITE(*,*) ' PROBLEMS READING THE DDH DATA FILE',CLNMFI
        WRITE(*,*) ' IRET= ',IRET
        CALL ABOR1('PROBLEMS READING THE DDH DATA FILE')
      ELSE
        ILENPS=JPSCOD
        IBIT=32
        CALL DECOPS2(CLNAME,CLDATA,1,ZDATA,1,IDATA,1,&
         & IBYTE,LLDAT,1,IB1PAR,IPSCOD,ILENPS,&
         & ILENCH,ILENFP,ILENIN,ILENLG,IRSERV,IBIT,IWORD,Iret)  
        IF(IRET /= 0) THEN
          PRINT *,' ERROR IN DECOPS - IERR= ',IRET
          CALL ABOR1('ERROR IN DECOPS FOR DDH DATA FILE')
        ENDIF
        ISTEP=IB1PAR(15)
      ENDIF
      IF (ISTEP < NSTAR2 ) THEN
        CALL PBWRITE(IUNIT,IPSCOD,ILENBYT,IRET)
        IF(IRET < 0) THEN
          WRITE(NULOUT,*)'RERESF: ERROR IN PBWRITE FOR DDH- IERR= ',IRET
          CALL ABOR1('RERESF: ERROR IN PBWRITE FOR DDH')
        ENDIF
      ENDIF
    ENDDO READ_LOOP
    DEALLOCATE(IPSCOD)

    CALL PBCLOSE(IUNITIN,IRET)
    CALL PBCLOSE(IUNIT,IRET)

  ENDIF

  CALL CLOSE_IOREQUEST(YL_IOREQUEST)
  CALL CLOSE_IOSTREAM(YL_IOSTREAM)

!*       4.6 READ ISP DATA

  IF (LMOVPH) THEN
    IF (MYPROC == 1) THEN
      CLFN=TRIM(CIOISPRF)//TRIM(CTIME)
      INQUIRE(NPPPSH,OPENED=LLOPENED)
      IF (.NOT.LLOPENED) THEN
        OPEN(NPPPSH,FORM='UNFORMATTED')
      ENDIF
!     Get the file name
      INQUIRE(NPPPSH,NAME=CLFNISP)
      CLOSE(NPPPSH)
!     Copy the restart file on the local file
      WRITE(CLSHELL,'(A,'' '',A,'' '',A)') TRIM(COPY),TRIM(CLFN),TRIM(CLFNISP)
      CALL SYSTEM(CLSHELL) 
!     Re-open the local file to be ready to go on !!
      OPEN(NPPPSH,FORM='UNFORMATTED',POSITION='APPEND')
    ENDIF
  ENDIF

!*       4.7 READ OLD STOCHASTIC PHYSICS DATA
  IF (LSTOPH) THEN
    CLFN='../stoph_old' // CTIME(1:IT) //CLMYPROC
    WRITE(UNIT=NULOUT,FMT='('' READING RESTART FILE FOR OLD STOPH'',A)') CLFN
    CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFN),CDMODE='r')
    CALL SETUP_IOREQUEST(YL_IOREQUEST,'RAW')

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88801) THEN
       CALL ABOR1('WRONG CHECK ON OLD STOPH READ 1')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR1=RSTOPHU)

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88802) THEN
       CALL ABOR1('WRONG CHECK ON OLD STOPH READ 2')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR1=RSTOPHV)

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88803) THEN
       CALL ABOR1('WRONG CHECK ON OLD STOPH READ 3')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR1=RSTOPHT)

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88804) THEN
       CALL ABOR1('WRONG CHECK ON OLD STOPH READ 4')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR1=RSTOPHQ)

    CALL CLOSE_IOREQUEST(YL_IOREQUEST)
    CALL CLOSE_IOSTREAM(YL_IOSTREAM)
    !read state of the random number generator
    CLFN='../stoph_old_rand_nbrs' // CTIME(1:IT) //CLMYPROC
    WRITE(UNIT=NULOUT,FMT='('' READ RANDOM NUMBER STATE FOR OLD STOPH'',A)') CLFN
    CALL RANDOM_NUMBER_RESTARTFILE( CLFN, 'r',YD_RANDOM_STREAM_STOCHPHYS)
  ENDIF

!*       4.8 READ SPBS DATA
  IF (LSTOPH_SPBS) THEN
    CLFN='../stoph_spbs' // CTIME(1:IT) //CLMYPROC
    WRITE(UNIT=NULOUT,FMT='('' READING RESTART FILE FOR SPBS'',A)') CLFN
    CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFN),CDMODE='r')
    CALL SETUP_IOREQUEST(YL_IOREQUEST,'RAW')

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88801) THEN
       CALL ABOR1('WRONG CHECK ON SPBS READ 1')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR2=SPSTREAM)

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88802) THEN
       CALL ABOR1('WRONG CHECK ON SPBS READ 2')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR2=SPSTREAM_FORC)

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88803) THEN
       CALL ABOR1('WRONG CHECK ON SPBS READ 3')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=GPSTREAM)

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88804) THEN
       CALL ABOR1('WRONG CHECK ON SPBS READ 4')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=GPTOTDISS)

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88805) THEN
       CALL ABOR1('WRONG CHECK ON SPBS READ 5')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=GPTOTDISS_SMOOTH)

    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
    IF(IBUF(1) /= 88806) THEN
       CALL ABOR1('WRONG CHECK ON SPBS READ 6')
    ENDIF
    CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=GPVORTGRAD)

    IF (LSTOPH_SPBS_T) THEN
      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
      IF(IBUF(1) /= 88807) THEN
        CALL ABOR1('WRONG CHECK ON SPBS READ 7')
      ENDIF
      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR2=SPTEMP)

      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
      IF(IBUF(1) /= 88808) THEN
        CALL ABOR1('WRONG CHECK ON SPBS READ 8')
      ENDIF
      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR2=SPTEMP_FORC)

      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,KR1=IBUF)
      IF(IBUF(1) /= 88809) THEN
        CALL ABOR1('WRONG CHECK ON SPBS READ 9')
      ENDIF
      CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=GPTEMP)
    ENDIF
    CALL CLOSE_IOREQUEST(YL_IOREQUEST)
    CALL CLOSE_IOSTREAM(YL_IOSTREAM)
    !read state of the random number generator
    CLFN='../stoph_spbs_rand_nbrs' // CTIME(1:IT) //CLMYPROC
    WRITE(UNIT=NULOUT,FMT='('' READ RANDOM NUMBER STATE FOR SPBS'',A)') CLFN
    CALL RANDOM_NUMBER_RESTARTFILE( CLFN, 'r',YD_RANDOM_STREAM_STOCHPHYS_SPBS)
    CLFN='../stoph_rvp_rand_nbrs' // CTIME(1:IT) //CLMYPROC
    WRITE(UNIT=NULOUT,FMT='('' READ RANDOM NUMBER STATE FOR RVP'',A)') CLFN
    CALL RANDOM_NUMBER_RESTARTFILE( CLFN, 'r',YD_RANDOM_STREAM_STOCHPHYS_RVP)
    IF (LSTOPH_SPBS_T) THEN
      CLFN='../stoph_spbs_T_rand_nbrs' // CTIME(1:IT) //CLMYPROC
      WRITE(UNIT=NULOUT,FMT='('' READ RANDOM NUMBER STATE FOR SPBS_T'',A)') CLFN
      CALL RANDOM_NUMBER_RESTARTFILE( CLFN, 'r',YD_RANDOM_STREAM_STOPH_SPBS_T)
      CLFN='../stoph_rvp_T_rand_nbrs' // CTIME(1:IT) //CLMYPROC
      WRITE(UNIT=NULOUT,FMT='('' READ RANDOM NUMBER STATE FOR RVP_T'',A)') CLFN
      CALL RANDOM_NUMBER_RESTARTFILE( CLFN, 'r',YD_RANDOM_STREAM_STOPH_RVP_T)
    ENDIF
  ENDIF


!     ------------------------------------------------------------------

!*       5.    IF COUPLED WITH WAM CORRECT BEGINNDATE.
!              ---------------------------------------

  IF (LWCOU) THEN
    IWSTEP=NSTAR2-1
    CALL INCDATE(CBEGDAT, (IWSTEP)*NINT(TSTEP))
    NDURAT=REAL(NSTOP-IWSTEP,JPRB)*TSTEP/60._JPRB+0.01_JPRB
  ENDIF

!*       5.1   IF LSTOPH, CORRECT THE RANDOM NUMBERS TO ALLOW
!              FOR THE CALLS MADE IN THE ORIGINAL EXECUTION
!              ----------------------------------------------


! following loop is obsolete, because STOPH and SPBS fields are written out to and read from 
! restart files and also the states of the random number generators are now 
! written out/ read in.
! For LVORTCON this was not working anyway because surand2 does nothing vorticity
! confinement related (for correct restarts using vorticity confinement GPVORTGRAD 
! must be written out and read in -> at the moment only done if LSTOPH_SPBS=true).
! For LSTOPH_CASBS this implementation was not very practical because it is expensive
! to recalculate all previous steps of the CA -> possibility of exceeded CPU limit. If
! CASBS should be used again in the future writing out and reading in the CA state
! would be the clean solution (together with the dissipation rates and SPSTREAM_FORC)
!  DO ISTEP=0,NSTAR2-1
!     NB:  SURAND2 has already been called once by SURAND1
!    IF((LSTOPH.OR.LSTOPH_CASBS.OR.LSTOPH_SPBS.OR.LVORTCON) .AND. ISTEP >= 0)  THEN
!      CALL SURAND2(ISTEP)
!    ENDIF
!  ENDDO

!     ------------------------------------------------------------------

ELSE

!*       6.    RESTART CONTROL FILE DOES NOT EXIST
!              -----------------------------------

  LNF = .TRUE.

ENDIF

CALL GSTATS(28,1)
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RERESF',1,ZHOOK_HANDLE)
END SUBROUTINE RERESF
