SUBROUTINE SPCMAD(CDCONF)

!**** *SPCMAD* - Multi tasking interface to SPCAD

!     Purpose.
!     --------
!        Multi tasking interface (also calls SPCAD single-tasked if
!        multi-tasking not requested).

!**   Interface.
!     ----------
!        *CALL* *SPCMAD(CDCONF)

!        Explicit arguments :  CDCONF - configuration of work (see doc.)

!        Implicit arguments :
!        --------------------
!                            NONE.
!     Method.
!     -------

!     Externals.   SPCAD - Spectral space computations
!     ----------   
!                  Called by SPCHAD.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 87-11-24
!        Modified by K. YESSAD (MAR 1994): multitasking on zonal wave number.
!        L. Isaksen 96-02-20 Adapted for Message Passing
!        L. Isaksen 96-12-12 Merge Message Passing and SH-memory
!        E. Holm    : 97-08-20 Call to trmtos/trstom includes ozone
!        M. Szczech: 00-04-18 Add horizontal diffusion for thermodynamic ozon
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        08-Feb-2005 K. Yessad     NH in ARPEGE
!        K. Yessad 15-May-2006: memory optimisations for stretched geometry
!        T.Wilhelmsson 09-09-30: Remove LFULLM requirement for LIMPF
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NFLSUR   ,NSMAX    ,NSPEC2   ,NUMP
USE YOMCT0   , ONLY : NPRTRV
USE YOMLAP   , ONLY : MYMS     ,NASM0
USE YOMSP    , ONLY : SPVOR    ,SPDIV    ,SPT      ,SPSPD    ,SPSVD    ,&
 & SPGFL    ,SPSP
USE YOMMP    , ONLY : MYSETN   ,NPTRMF   ,NSPSTAF  ,NSPEC2V  ,NSPEC2VF  
USE YOMDYN   , ONLY : LIMPF    ,LSIDG

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IEND, IM, IMLOC, ISTA, JMLOC

LOGICAL :: LLONEM

REAL(KIND=JPRB),ALLOCATABLE ::  ZSPVORG(:,:)
REAL(KIND=JPRB),ALLOCATABLE ::  ZSPDIVG(:,:)
REAL(KIND=JPRB),ALLOCATABLE ::  ZSPTG  (:,:)
REAL(KIND=JPRB),ALLOCATABLE ::  ZSPSPDG(:,:)
REAL(KIND=JPRB),ALLOCATABLE ::  ZSPSVDG(:,:)
REAL(KIND=JPRB),ALLOCATABLE ::  ZSPSPG (:)
REAL(KIND=JPRB),ALLOCATABLE ::  ZSPTALLG(:,:)
REAL(KIND=JPRB),ALLOCATABLE ::  ZSPAUX (:,:)
REAL(KIND=JPRB),ALLOCATABLE ::  ZSPAUXG(:,:)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "brptob.intfb.h"
#include "spcsiad.intfb.h"
#include "spchorad.intfb.h"
#include "trmtos.intfb.h"
#include "trstom.intfb.h"
#include "spcimpfpostad.intfb.h"
#include "spcimpfinitad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCMAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF (LSIDG) THEN
  LLONEM=.TRUE.
ELSE
  LLONEM=.FALSE.
ENDIF

IF(CDCONF == 'A'.OR.CDCONF == 'I'.OR.CDCONF == 'P') THEN

  ALLOCATE(ZSPVORG(NFLEVG,MAX(NSPEC2V,NSPEC2VF)))
  ALLOCATE(ZSPDIVG(NFLEVG,MAX(NSPEC2V,NSPEC2VF)))
  ALLOCATE(ZSPTG  (NFLEVG,MAX(NSPEC2V,NSPEC2VF)))
  ALLOCATE(ZSPSPDG(NFLEVG,MAX(NSPEC2V,NSPEC2VF)))
  ALLOCATE(ZSPSVDG(NFLEVG,MAX(NSPEC2V,NSPEC2VF)))
  ALLOCATE(ZSPSPG (MAX(NSPEC2V,NSPEC2VF)))
  ALLOCATE(ZSPTALLG(NFLEVG,NSPEC2))

  IF (LIMPF) THEN
    ALLOCATE(ZSPAUX(NFLSUR,NSPEC2))
    ALLOCATE(ZSPAUXG(NFLEVG,MAX(NSPEC2V,NSPEC2VF)))
  ENDIF

  CALL GSTATS(36,0)

  IF (NSPEC2 > 0) THEN

!   * Copies of surface pressure and temperature are required 
!     on all PE's

    IF(NPRTRV > 1) CALL BRPTOB(SPSP,SPT,ZSPTALLG)
    CALL SPCHORAD(CDCONF,1,NSPEC2,SPVOR,SPDIV,SPT,SPGFL,SPSP,ZSPTALLG)
  ENDIF

  IF (LIMPF) THEN
    CALL GSTATS(1029,0)
!$OMP PARALLEL DO SCHEDULE(STATIC,1) PRIVATE(JMLOC,IM,ISTA,IEND)
    DO JMLOC=1,NUMP
      IM=MYMS(JMLOC)
      ISTA=NASM0(IM)
      IEND=ISTA+2*(NSMAX+1-IM)-1
      CALL SPCIMPFPOSTAD(CDCONF,IM,ISTA,IEND,SPVOR,SPDIV)  
    ENDDO
!$OMP END PARALLEL DO
    CALL GSTATS(1029,1)
  ENDIF

! * Transpose spectral data to vertical columns
!   Vorticity is only required if LIMPF=.T.

  CALL GSTATS(36,2)
  CALL GSTATS(86,0)
  CALL TRMTOS(PSPVOR=SPVOR,PSPDIV=SPDIV,PSPT=SPT,PSPSPD=SPSPD,&
   & PSPSVD=SPSVD,PSPSP=SPSP,&
   & PSPVORG=ZSPVORG,PSPDIVG=ZSPDIVG,PSPTG=ZSPTG,PSPSPDG=ZSPSPDG,&
   & PSPSVDG=ZSPSVDG,PSPSPG=ZSPSPG,&
   & LDFULLM=LLONEM)

  CALL GSTATS(86,2)
  CALL GSTATS(36,3)
  IF (LLONEM) THEN

!   * Each zonal wave number has to be treated separately

    CALL GSTATS(1030,0)
!$OMP PARALLEL DO SCHEDULE(STATIC,1) PRIVATE(JMLOC,IM,ISTA,IEND)
    DO JMLOC=NPTRMF(MYSETN),NPTRMF(MYSETN+1)-1
      IM=MYMS(JMLOC)
      ISTA=NSPSTAF(IM)
      IEND=ISTA+2*(NSMAX+1-IM)-1
      CALL SPCSIAD(CDCONF,IM,JMLOC,ISTA,IEND,LLONEM,&
       & ZSPAUXG,ZSPVORG,ZSPDIVG,ZSPTG,ZSPSPG)  
    ENDDO
!$OMP END PARALLEL DO
    CALL GSTATS(1030,1)

  ELSE

!   * All spectral colunms can be done in one go

    IF (NSPEC2V > 0) THEN
      IM=-999
      IMLOC=-999
      CALL SPCSIAD(CDCONF,IM,IMLOC,1,NSPEC2V,LLONEM,&
       & ZSPAUXG,ZSPVORG,ZSPDIVG,ZSPTG,ZSPSPG)  
    ENDIF
  ENDIF

! * Transpose spectral data back to level structure

  CALL GSTATS(36,1)
  CALL GSTATS(86,3)
  IF (LIMPF) THEN
    CALL TRSTOM(&
      & PSPVORG=ZSPVORG,PSPDIVG=ZSPDIVG,PSPTG=ZSPTG,PSPSPDG=ZSPSPDG,&
      & PSPSVDG=ZSPSVDG,PSPSPG=ZSPSPG,PSPAUXG=ZSPAUXG,&
      & PSPVOR=SPVOR,PSPDIV=SPDIV,PSPT=SPT,PSPSPD=SPSPD,&
      & PSPSVD=SPSVD,PSPSP=SPSP,PSPAUX=ZSPAUX,&
      & LDFULLM=LLONEM,LDNEEDPS=.FALSE.)
  ELSE
    CALL TRSTOM(&
      & PSPVORG=ZSPVORG,PSPDIVG=ZSPDIVG,PSPTG=ZSPTG,PSPSPDG=ZSPSPDG,&
      & PSPSVDG=ZSPSVDG,PSPSPG=ZSPSPG,&
      & PSPVOR=SPVOR,PSPDIV=SPDIV,PSPT=SPT,PSPSPD=SPSPD,&
      & PSPSVD=SPSVD,PSPSP=SPSP,&
      & LDFULLM=LLONEM,LDNEEDPS=.FALSE.)
  ENDIF
  CALL GSTATS(86,1)

  IF (LIMPF) THEN
    CALL GSTATS(1031,0)
!$OMP PARALLEL DO SCHEDULE(STATIC,1) PRIVATE(JMLOC,IM,ISTA,IEND)
    DO JMLOC=1,NUMP
      IM=MYMS(JMLOC)
      ISTA=NASM0(IM)
      IEND=ISTA+2*(NSMAX+1-IM)-1
      CALL SPCIMPFINITAD(CDCONF,IM,ISTA,IEND,SPVOR,ZSPAUX)  
    ENDDO
!$OMP END PARALLEL DO
    CALL GSTATS(1031,1)
  ENDIF

  DEALLOCATE(ZSPVORG)
  DEALLOCATE(ZSPDIVG)
  DEALLOCATE(ZSPTG)
  DEALLOCATE(ZSPSPDG)
  DEALLOCATE(ZSPSVDG)
  DEALLOCATE(ZSPSPG)
  DEALLOCATE(ZSPTALLG)
  IF (LIMPF) THEN
    DEALLOCATE(ZSPAUX)
    DEALLOCATE(ZSPAUXG)
  ENDIF

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPCMAD',1,ZHOOK_HANDLE)
END SUBROUTINE SPCMAD
