SUBROUTINE STEPO(CDCONF)

!**** *STEPO*  - Controls integration job at lowest level

!     Purpose.
!     --------
!        CONTROLS THE INTEGRATION

!**   Interface.
!     ----------
!        *CALL* *STEPO(CDCONF)

!        Explicit arguments :
!        --------------------
!     CHARACTER*9 CDCONF
!   1 : configuration of IOPACK     IO handling
!   2 : configuration of TRANSINVH  inverse Legendre transform
!   2 : configuration of ETRANSINVH inverse North/South Fourier tr. (LAM)
!   3 : configuration of TRANSINVH  inverse Fourier transform
!   3 : configuration of ETRANSINVH inverse East/West Fourier tr. (LAM)
!   4 : configuration of CPG        grid point computations   (under SCAN2M*)
!   5 : configuration of POS        post processing           (under SCAN2M*)
!   6 : configuration of OBS        comparison to observations(under SCAN2M*)
!   7 : configuration of TRANSDIRH  direct Fourier transform
!   7 : configuration of ETRANSDIRH direct East/West Fourier tr. (LAM)
!   8 : configuration of TRANSDIRH  direct Legendre transform
!   8 : configuration of ETRANSDIRH direct North/South Fourier tr. (LAM)
!   9 : configuration of SPC        spectral space computations
!   9 : configuration of ESPCH      spectral space computations (LAM)
!   9 : configuration of SPAY/SPAWR spectral space computations (NHS)
!   9 : configuration of ESPAY/ESPAWR spectral space computations (LAM/NHS)

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.    IOPACK    - IO handling
!     ----------    TRANSINVH - inverse spectral transform
!                   ETRANSINVH - inverse spectral transform (LAM)
!                   OBSM      - comparison to observations
!                   TRANSDIRH - direct spectral transform
!                   ETRANSDIRH - direct spectral transform (LAM)
!                   SPCH      - spectral space computations.
!                   ESPCH      - spectral space computations. (LAM)
!                   ESPAY/ESPAWR - spectral space computations. (LAM/NHS)
!                   SCAN2H    - interface grid-point computations
!                   SUALSPA   - allocate spectral array
!                   WRSPEC    - write out spectral arrays
!                   USER_CLOCK-timing

!        Called by CNT4 and:
!                   CACUMTND.
!                   CADAVR,CAHUCX,CAN1,CAPSCX,CAVTCX.
!                   CMACAD.
!                   CNT4AD,CNT4TL.
!                   COSJC,COSJG,COVOPOB.
!                   CVAR2,CVAR2AD,CVAR2IN,CVAR2INAD,CVARBL,CVARBLAD.
!                   DFI3.
!                   NMDIAG,NMIJC,NNMI2AD,NNMI2TL,NNMI3,NNMI3AD,NNMI3TL.
!                   OBSIO3D.
!                   SCALJGG,SIM4D,SUECGES,SUMDFCE.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

! Modifications
! -------------
!   Original : 87-10-15
!   BILL HECKLEY : 90-11-20 GRID POINT PROCESSING OF JG
!   Implantation sorties diagnostics DDH, A.Joly, 1/91
!   Mats Hamrud  : 91-04-01 IO scheme introduced
!   R. El Khatib : 92-09-16 calls to REMARK2 during integration moved
!                       to CNT4
!   J.N. Thepaut : 92-11-26 new buffer for observations.
!   D. Giard     : 94-01-24 no zonal diagnostics if nnmi iteration
!   M.Hamrud     : 93-11-18 Only one Fourier file
!   R. El Khatib : 94-04-06 Full-POS implementation
!   L. Isaksen   : 95-06-06 Reordering of spectral arrays
!   R. El Khatib : 95-08-28 Off-line reprod. for pp. on gridpoint upper air
!   MPP Group    : 95-10-01 Adapted for Message Passing
!   R. El Khatib : 95-11-21 Off-line reprod. for pp. on gridpoint upper air
!   M.Hamrud     : 95-10-30 New form for CDCONF
!   F. Bouttier  : 95-12-28 Clean calls to OBSV using config string
!   M. Fisher    : 96-05-10 use OBSV config 'AD' if L131TL=.T.
!   D. Giard     : 96-05-29 CDCONF(4)='X'
!   K. YESSAD    : 97-03    parallelisation of FULL-POS.
!   D. Dent      : 97-10    USER_CLOCK
!   R. El Khatib : 97-11-27 Full-POS cleanups.
!   R. El Khatib : 98-01-29 subroutine PRESPFPOS ; cleanup fpos in-line
!   R. El Khatib : 99-01-27 unconditional allocation of GDFPBUF
!   A. Horanyi   : 99-06-23 correcting conditions for calling OBSV in SM
!   M.Hamrud     : 00-10-25 implementation of transform package
!   R. El Khatib : 00-04-10 GDFPBUF allocated here only if SM
!   R. El Khatib : 00-07-31 reorganize fullpos conf. (5:5)
!   R. El Khatib : 00-12-22 cleaning conf. (9:9)
!   R. El Khatib : 01-04-10 More cleaning conf. (9:9)
!   C. Fischer   : 01-03-14 merge Aladin with Arp/IFS version
!   G. Radnoti   : 01-03-15 Aladin spectral transform package
!   R. El Khatib : 01-08-07 Pruning options
!   J.Vivoda     : 03-2002  PC schemes for NH dynamics (LPC_XXXX keys)
!   V.Guidard&C.Fischer: 02-09-30 3dfgat coupling switched on for Aladin
!   R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!   P.Termonia   : 03-04-15 call ecoupl1 out of transform package
!   R. El Khatib : 03-04-17 Fullpos improvemnts
!   R. El Khatib : 03-04-18 Phasing aladin with improved fullpos
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   10-Jun-2004 J. Masek   NH cleaning (LPC_NOTR, LFULLIMP)
!   08-Feb-2005 K. Yessad  NH model in ARPEGE 
!   16-Feb-2006 R. Zaaboul call to surfex diagnotstics
!   22-Nov-2005 D.Salmond & M.Hortal  Mods for coarser/finer physics
!   11-Jan-2008 Y.Seity Remove call to aro_surf_diagh for surfex 
!                      (replaced by CDCONF(1:1)='S'in iopack)
!   13-Jan-2009 F. Vana   remove useless LSPC_FROM_DI key
!   L. Batte IC3 : 04-2013 Option added to call relaxation (as L. Magnusson in CY38R1)
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDYN   , ONLY : NSITER   ,NCURRENT_ITER
USE YOMFPDIM , ONLY : NFPAUXB  ,NFPDYNB
USE YOMFPSC2 , ONLY : NFPROMA  ,NFPBLOCS
USE YOMLUN   , ONLY : NULOUT
USE YOMTIM   , ONLY : RSTART   ,RVSTART  ,RTIMEF
USE YOMCT0   , ONLY : NCONF    ,NSTART   ,NSTOP    ,LALLOPR  ,&
 & LFPOS    ,LOPDIS  ,LCANARI  ,LOBS     ,LRPLANE  ,&
 & LOBSC1   ,LOBSREF  ,LNHDYN   ,&
 & LELAM    ,LRUBC    ,LPC_OLD
USE YEMDIM   , ONLY : LECOBI
USE YEMCT0   , ONLY : NECRIPL  ,NECOTL
USE YOMCT3   , ONLY : NSTEP
USE YOMLDDH  , ONLY : LHDOUFG  ,LHDOUFZ  ,&
 & LHDOUFD  ,LHDOUP  
USE YOMECTAB , ONLY : NTSLOTNO
USE YOMDFPB  , ONLY : HFPBUF   ,NSTAHFPB ,HEFPBUF  ,NESTAHFP ,NELENHFP
USE YOMAFPB  , ONLY : NLENAFPBL,AFPBUF   ,NSTAAFPB
USE YOMAUXB  , ONLY : GAUXBUF
USE YOMVRTL  , ONLY : L131TL   ,LOBSTL
USE YOMIOS   , ONLY : NPCKFGX
USE GRIDPOINT_BUFFERS_MIX , ONLY : ALLOCATED_GRIDPOINT_BUFFER, &
 & ALLOCATE_GRIDPOINT_BUFFER,  &
 & DEALLOCATE_GRIDPOINT_BUFFER  
USE YOMFPC   , ONLY : CFPFMT
USE YOMFPG   , ONLY : LFPEZO
USE YOMFPEZO , ONLY : NFPROMEL ,NFPEBLKS
USE YOMCOAPHY   , ONLY : NPHYINT
USE YOMSPSDT ,            ONLY : LSPSDT, YSPSDT_AR1, YGPSDT, LWRITE_ARP,&
     & XCLIP_RATIO_SDT, SDEVTOT_SDT, LCLIP_GRID_SDT, &
     & LWRPATTRUN_SDT, COPATTRUN_SDT, COPATSP_SDT, COPATGP_SDT, &
     & NSEED_SDT, LRESETSEED_SDT, LUSESETRAN_SDT
USE YOMVAREPS, ONLY : LVAREPS,NST_TRUNC_INI
USE SPECTRAL_ARP_MOD ,    ONLY : EVOLVE_ARP, SUM_ARPS, SET_SEED_ARP
USE SPECTRAL_FIELDS_MOD , ONLY : SPECTRAL_NORM
USE GRIDPOINT_FIELDS_MIX, ONLY : GRIDPOINT_FIELD, &
     & CLIP_GRID, ASSIGNMENT(=)
USE YOMRLX   , ONLY : LRLXG

!      -----------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=9)  ,INTENT(IN)    :: CDCONF 

!      -----------------------------------------------------------

INTEGER(KIND=JPIM) :: IJUM
INTEGER(KIND=JPIM) :: ISEED, ISEEDX

LOGICAL :: LLFGAT, LLHFP, LLVPP, LLCPL, LLCALL_POSDDH
LOGICAL,SAVE :: LLRESP=.FALSE.

REAL(KIND=JPRB) :: ZCT, ZJUM, ZVT, ZWT
REAL(KIND=JPRB), DIMENSION(YGPSDT%NG2D) :: ZMIN2D, ZMAX2D

CHARACTER(LEN=1) :: CLCONF
REAL(KIND=JPRB) :: ZHOOK_HANDLE
CHARACTER(LEN=1)   :: CLCONF9
!      -----------------------------------------------------------

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "abor1.intfb.h"
#include "ecoupl1.intfb.h"
#include "espawr.intfb.h"
#include "espay.intfb.h"
#include "espch.intfb.h"
#include "espfp.intfb.h"
#include "etransdirh.intfb.h"
#include "etransinvh.intfb.h"
#include "fpezo2h.intfb.h"
#include "gridpoint_norm.intfb.h"
#include "iopack.intfb.h"
#include "obsv.intfb.h"
#include "obsvad.intfb.h"
#include "opdis.intfb.h"
#include "posddh.intfb.h"
#include "prespfpos.intfb.h"
#include "scan2h.intfb.h"
#include "setran.intfb.h"
#include "spawr.intfb.h"
#include "spay.intfb.h"
#include "spec2grid.intfb.h"
#include "spch.intfb.h"
#include "sufpsc2b.intfb.h"
#include "transdirh.intfb.h"
#include "transinvh.intfb.h"
!!#include "relaxgp.intfb.h"
#include "updobs.intfb.h"
#include "write_grid_grib.intfb.h"
#include "write_spec_grib.intfb.h"

!      -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('STEPO',0,ZHOOK_HANDLE)

!      -----------------------------------------------------------

!*       0.    VARIOUS INITIALIZATION
!              ----------------------

CALL USER_CLOCK(PELAPSED_TIME=ZWT,PTOTAL_CP=ZCT,PVECTOR_CP=ZVT)
ZCT=ZCT-RSTART
ZVT=ZVT-RVSTART
ZWT=ZWT-RTIMEF
RSTART=RSTART+ZCT
RVSTART=RVSTART+ZVT
RTIMEF=RTIMEF+ZWT
ZJUM=10._JPRB**(INT(LOG10(REAL(MAX(NSTOP-NSTART,1),JPRB)))+1)
IJUM=NINT(MAX(ZJUM/100._JPRB,1.0_JPRB))
IF(NSTEP-NSTART <= 10.OR.MOD(NSTEP,IJUM) == 0.OR.(NCONF == 1.OR.NCONF == 302))THEN
  WRITE(NULOUT,'('' NSTEP ='',I6,'' STEPO   '',A9)')&
   & NSTEP,CDCONF
ENDIF

IF (LOPDIS) THEN
  CALL OPDIS(CDCONF,'STEPO   ',ZCT,ZVT,ZWT,RSTART,RVSTART,RTIMEF)
ENDIF

LLVPP=(CDCONF(5:5) == 'B').OR.(CDCONF(5:5) == 'H')&
 & .OR.(CDCONF(5:5) == 'V').OR.(CDCONF(5:5) == 'T')&
 & .OR.(CDCONF(5:5) == 'S').OR.(CDCONF(5:5) == 'M')&
 & .OR.(CDCONF(5:5) == 'Z').OR.(CDCONF(5:5) == 'E')  
LLHFP=(CDCONF(5:5) == 'A').OR.(CDCONF(5:5) == 'I')
LLFGAT=(NCONF == 1.OR.NCONF == 302).AND.(NSTOP <= 1).AND.(NTSLOTNO > 1).AND.LOBSC1

!        0.1   Define configurations for coupling LBC in LAM

LLCPL = (LELAM.AND.LECOBI.AND.NECRIPL == 1.AND.&
 & .NOT.(LRUBC.AND.CDCONF(9:9) /= 'T')) .AND.&
 & ((CDCONF(3:3) == 'A'.OR.CDCONF(3:3) == 'B'.OR.&
 & CDCONF(6:6) == '1').AND.CDCONF(5:5) == '0') .AND.&
 & (NCONF/100 == 0.OR.NCONF/100 == 2.OR.NCONF == 302.OR.&
 & NCONF/100 == 7.OR.NCONF == 801.OR.&
 & ((NCONF/100==1).AND.(NSTOP>0)).OR.&
 & (NECOTL < 0.AND.(NCONF == 401.OR.&
 & NCONF == 501.OR.NCONF == 601)))  

!     ------------------------------------------------------------------

!*       1.    IO (TRAJECTORY,POST-PROC.,RESTART)
!              ----------------------------------

IF(CDCONF(1:1) /= '0')THEN
  CALL IOPACK(CDCONF(1:1))
ENDIF
IF (ALLOCATED(HFPBUF)) THEN
  DEALLOCATE(NSTAHFPB,HFPBUF)
  IF (LALLOPR) THEN
    WRITE(NULOUT,9991) 'NSTAHFPB '
    WRITE(NULOUT,9991) 'HFPBUF   '
  ENDIF
ENDIF
IF (ALLOCATED(HEFPBUF)) THEN
  DEALLOCATE(NESTAHFP,HEFPBUF)
  IF (LALLOPR) THEN
    WRITE(NULOUT,9991) 'NESTAHFP '
    WRITE(NULOUT,9991) 'HEFPBUF  '
  ENDIF
ENDIF

!     Prepare for post-processing
IF (LFPOS.OR.LLVPP.OR.LLRESP) THEN
  CALL PRESPFPOS(CDCONF(2:2),LLVPP,LLHFP,LLRESP)
ENDIF

!     ------------------------------------------------------------------

!*       2.    INVERSE TRANSFORMS.
!              -------------------

IF(CDCONF(2:2) /= '0') THEN
  IF (LELAM) THEN
    CALL ETRANSINVH(CDCONF)
  ELSE
    CALL TRANSINVH(CDCONF)
  ENDIF
  
  !--------
  ! Relaxation
  IF (LRLXG .AND. NSTEP > 1) THEN
    CALL RELAXGP(CDCONF)
  ENDIF  

  IF (LSPSDT.AND.(CDCONF(2:2)=='A')) THEN
    CALL SUM_ARPS(  YSPSDT_AR1 )
    CALL SPECTRAL_NORM(YSPSDT_AR1%SF,    '  STEPO, YSPSDT_AR1(1,...), ')
    CALL SPECTRAL_NORM(YSPSDT_AR1%SFSUM, '  STEPO, YSPSDT_AR1( sum ), ')
    CALL SPEC2GRID( YSPSDT_AR1%SFSUM, YGPSDT )
    CALL GRIDPOINT_NORM( YGPSDT, '  STEPO, YGPSDT  , ')
    !
    !    clip if required
    !
    IF (LCLIP_GRID_SDT) THEN
      ZMIN2D(:)= -XCLIP_RATIO_SDT * SDEVTOT_SDT 
      ZMAX2D(:)=  XCLIP_RATIO_SDT * SDEVTOT_SDT 
      CALL CLIP_GRID(YGPSDT, PMIN2D=ZMIN2D, PMAX2D=ZMAX2D)
      CALL GRIDPOINT_NORM( YGPSDT, '  STEPO, YGPSDT[clipped] , ')
    ENDIF
    !
    !    write multiplicative noise
    !
    IF (LWRITE_ARP) THEN
      CALL WRITE_GRID_GRIB( COPATGP_SDT ,YGPSDT,CDMODE='a')
    ENDIF
    IF (LVAREPS.AND.LWRPATTRUN_SDT.AND.(NSTEP==NST_TRUNC_INI)) THEN
      CALL WRITE_SPEC_GRIB(COPATTRUN_SDT, YSPSDT_AR1%SF,CDMODE="w")
    ENDIF
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       3.   GRIDPOINT COMPUTATIONS.
!             -----------------------

IF(CDCONF(3:7) /= '00000') THEN

  CALL GSTATS(1980,0)
  IF (LLVPP) THEN
    IF (ALLOCATED_GRIDPOINT_BUFFER(GAUXBUF)) THEN
      CALL DEALLOCATE_GRIDPOINT_BUFFER(GAUXBUF)
      IF (LALLOPR) WRITE(NULOUT,9991) 'GAUXBUF  '
    ENDIF
    IF (NFPAUXB > 0) THEN
      CALL ALLOCATE_GRIDPOINT_BUFFER('GAUXBUF',GAUXBUF,NFPAUXB,NPCKFGX)
    ENDIF
  ENDIF
  IF (CDCONF(5:5) == 'I') THEN
    IF (.NOT.ALLOCATED(AFPBUF)) THEN
      ALLOCATE(NSTAAFPB(NFPBLOCS))
      CALL SUFPSC2B(NFPDYNB,NSTAAFPB,NLENAFPBL,NFPROMA,NFPBLOCS)
      ALLOCATE(AFPBUF(NLENAFPBL))
      IF (LALLOPR) THEN
        WRITE(NULOUT,9990) 'NSTAAFPB  ',SIZE(NSTAAFPB),SHAPE(NSTAAFPB)
        WRITE(NULOUT,9990) 'AFPBUF    ',SIZE(AFPBUF),SHAPE(AFPBUF)
      ENDIF
    ENDIF
  ENDIF
  CALL GSTATS(1980,1)

  CALL SCAN2H(CDCONF)

  IF (LLHFP.AND.ALLOCATED_GRIDPOINT_BUFFER(GAUXBUF)) THEN
    CALL DEALLOCATE_GRIDPOINT_BUFFER(GAUXBUF)
    IF (LALLOPR) WRITE(NULOUT,9991) 'GAUXBUF  '
  ENDIF
  IF (CDCONF(5:5) == 'Z'.OR.CDCONF(5:5) == 'E') THEN
    IF (ALLOCATED(AFPBUF)) THEN
      DEALLOCATE(NSTAAFPB,AFPBUF)
      IF (LALLOPR) THEN 
        WRITE(NULOUT,9991) 'NSTAAFPB'
        WRITE(NULOUT,9991) 'AFPBUF  '
      ENDIF
    ENDIF
  ENDIF

ENDIF

!     ------------------------------------------------------------------

!*       4.    JO COMPUTATION
!              --------------

IF(LOBS.AND.(CDCONF(6:6) == 'C'.OR.CDCONF(6:6) == 'V')&
   & .AND..NOT.(LOBSREF.AND.L131TL.AND.LOBSTL)&
   & .AND..NOT.((LOBSC1.OR.((NCONF == 131).AND.(LOBSREF))))) THEN  

!       Decide on whether to do direct and/or adjoint Jo computations
  IF (LCANARI) THEN
    CALL OBSV('DI')
  ELSEIF (LOBSC1.OR.((NCONF == 131).AND.(LOBSREF))) THEN
    CALL OBSV('DI')
  ELSEIF ((NCONF == 131).AND..NOT.LOBSREF) THEN
    IF (L131TL) THEN
      CALL OBSVAD
    ELSE
      IF (.NOT.LOBSTL) THEN
        CALL OBSV('DA')
      ELSE
        CALL ABOR1('STEPO ERROR : TL OBS OPER NEEDS TL MODEL')
      ENDIF
    ENDIF
  ELSE
    CALL ABOR1('STEPO ERROR : UNKNOWN WAY TO USE OBSERVATIONS')
  ENDIF

  IF(.NOT.LCANARI) THEN
    CALL UPDOBS(NTSLOTNO)
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       5.    DIAGNOSTICS.
!              ------------

LLCALL_POSDDH=(NCONF == 1.OR.NCONF == 302)&
 &.AND.(CDCONF(4:4) /= '0').AND.(CDCONF(4:4) /= 'E')&
 &.AND.(CDCONF(4:4) /= 'M').AND.(CDCONF(4:4) /= 'X')&
 &.AND.(LHDOUFG.OR.LHDOUFZ.OR.LHDOUFD.OR.LHDOUP)&
 &.AND.(NCURRENT_ITER == 0 .AND. NPHYINT == 0)
IF (LLCALL_POSDDH) CALL POSDDH


!     ------------------------------------------------------------------

!*       8.    DIRECT TRANSFORMS.
!              ------------------

IF(CDCONF(8:8) /= '0')THEN
  IF (LLCPL) CALL ECOUPL1
  IF (LELAM) THEN
    CALL ETRANSDIRH(CDCONF)
  ELSE
    CALL TRANSDIRH(CDCONF)
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       9.    SPECTRAL COMPUTATIONS.
!              ----------------------

IF (CDCONF(9:9) /= '0') THEN

  IF (CDCONF(9:9) /= 'P') THEN
!*      iteration of S.-I. solver if required
!*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!* !!!! This condition could be potential problem for RUBC !!!
!* !!!! so RUBC need to be run with LPC_OLD=.TRUE.         !!!
!*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    IF (NSITER > 0.AND.LNHDYN.AND.LPC_OLD) THEN
      IF (NCURRENT_ITER == 0) THEN
        IF (LELAM) THEN
          CALL ESPAWR
        ELSE
          CALL SPAWR
        ENDIF
      ELSEIF (NCURRENT_ITER /= 0) THEN
        IF (LELAM) THEN
          CALL ESPAY(CDCONF(9:9))
        ELSE
          CALL SPAY(CDCONF(9:9))
        ENDIF
      ENDIF
    ENDIF
  ENDIF
!*      standard spectral computations
  IF (LELAM) THEN
    CALL ESPCH(CDCONF(9:9))
  ELSE
    CALL SPCH(CDCONF(9:9))
  ENDIF

  CLCONF9=CDCONF(9:9)
  IF (LSPSDT.AND.CLCONF9 == 'A') THEN
    IF (LRESETSEED_SDT) THEN
      ISEEDX=NSEED_SDT+NSTEP*256
      IF (LUSESETRAN_SDT) THEN
        CALL SETRAN( ISEEDX, KOUTSEED=ISEED)
      ELSE
        ISEED=ISEEDX
      ENDIF
      WRITE(NULOUT,'(''STEPO: Using seed '',I12,'' for YSPSDT_AR1 evolution in NSTEP='',I12)')  ISEED, NSTEP
      CALL SET_SEED_ARP(YSPSDT_AR1, ISEED)
    ENDIF
    CALL EVOLVE_ARP(YSPSDT_AR1)
    IF (LWRITE_ARP) THEN
      CALL WRITE_SPEC_GRIB(COPATSP_SDT, YSPSDT_AR1%SF,CDMODE='a')
    ENDIF
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       10.    FULLPOS RESIDUAL COMPUTATIONS
!               -----------------------------

IF (LFPOS .AND. CDCONF(5:5) /= '0') THEN
  IF (CFPFMT == 'MODEL') THEN
    IF (LRPLANE) THEN
      CALL ESPFP
    ENDIF
  ELSE
    SELECT CASE (CDCONF(5:5))
    CASE ('A','Z','E')
      IF (LFPEZO) THEN
        ALLOCATE(NESTAHFP(NFPEBLKS))
        CALL SUFPSC2B(NFPDYNB,NESTAHFP,NELENHFP,NFPROMEL,NFPEBLKS)
        ALLOCATE(HEFPBUF(NELENHFP))
        CLCONF='A'
        CALL FPEZO2H(CLCONF)
      ENDIF
    END SELECT
  ENDIF
ENDIF

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
9991 FORMAT(1X,'ARRAY ',A10,' DEALLOCATED ')

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('STEPO',1,ZHOOK_HANDLE)
END SUBROUTINE STEPO
