SUBROUTINE STEPOAD(CDCONF)

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YEMCT0   , ONLY : NECRIPL  ,NECOAD
USE YEMDIM   , ONLY : LECOBI
USE YOMCT0   , ONLY : NSTART   ,NSTOP    ,LOPDIS   ,LELAM
USE YOMCT3   , ONLY : NSTEP
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMTIM   , ONLY : RSTART   ,RVSTART  ,RTIMEF

!**** *STEPOAD*  - Controls adjoint integration at lowest level

!     Purpose.
!     --------
!     Controls the adjoint integration.

!**   Interface.
!     ----------
!        *CALL* *STEPOAD(CDCONF)

!        Explicit arguments :
!        --------------------
!     CHARACTER*9 CDCONF
!   1 : configuration of IOPACK      write or read the trajectory
!   2 : configuration of TRANSINVHAD  inverse Legendre transform
!   2 : configuration of ETRANSINVHAD inverse North/South Fourier tr. (LAM)
!   3 : configuration of TRANSINVHAD  inverse Fourier transform
!   3 : configuration of ETRANSINVHAD inverse East/West Fourier tr. (LAM)
!   4 : configuration of GPCAD      grid point computations
!  (5 : configuration of POS        post processing           )
!  (6 : configuration of OBS        grid point calculations for analysis)
!   7 : configuration of TRANSDIRHAD  direct Fourier transform
!   7 : configuration of ETRANSDIRHAD direct East/West Fourier tr. (LAM)
!   8 : configuration of TRANSDIRHAD  direct Legendre transform
!   8 : configuration of ETRANSDIRHAD direct North/South Fourier tr. (LAM)
!   9 : configuration of SPCAD      spectral space computations
!   9 : configuration of ESPCHAD      spectral space computations (LAM)
!   9 : configuration of ESPAYAD/ESPAWRAD spectral space computations (LAM/NHS)

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.    IOPACK       - input/output
!     ----------    TRANSINVHAD  - inverse spectral transform - adjoint
!                   ETRANSINVHAD - inverse spectral transform (LAM) - adjoint
!                   TRANSDIRHAD  - direct spectral transform - adjoint
!                   ETRANSDIRHAD - direct spectral transform (LAM) - adjoint
!                   SPCHAD       - spectral space computations - adjoint
!                   ESPCHAD      - spectral space comp. (LAM) - adjoint
!                   ESPAYAD/ESPAWRAD - spectral space comp. (LAM/NHS) - adjoint
!                   USER_CLOCK   - timing
!        Called by CNT4AD and:
!                   COVOPOB,CVAR2AD,CVAR2INAD,CVARBLAD.
!                   NMIJC,NNMI2AD,NNMI3AD,OBSIO3D.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

! Modifications
! -------------
!   Original : 87-10-15
!   BILL HECKLEY: 90-11-20  GRID POINT PROCESSING OF JG
!   Modified : 91-10-15 Bill Heckley and Mats Hamrud ECMWF
!                       Use of MIOLIB for IO on forecast error files
!   M.Hamrud : 95-10-30 (New form for CDCONF)
!   M.Hamrud : 97-02-18  cleaning
!   M.Hamrud : 00-10-25 implementation of transform package
!   C.Fischer: 01-03-14 merge Aladin with Arp/IFS version
!   G. Radnoti: 01-03-15 Aladin spectral transform package
!   J.Vivoda : 03-2002  PC schemes for NH dynamics (LPC_XXXX keys)
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   10-Jun-2004 J. Masek   NH cleaning (LPC_NOTR)
! End Modifications
!------------------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=9)  ,INTENT(IN)    :: CDCONF 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IJUM

LOGICAL :: LLCPL

REAL(KIND=JPRB) :: ZCT, ZJUM, ZVT, ZWT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "ecoupl1ad.intfb.h"
#include "espchad.intfb.h"
#include "etransdirhad.intfb.h"
#include "etransinvhad.intfb.h"
#include "iopack.intfb.h"
#include "opdis.intfb.h"
#include "scan2had.intfb.h"
#include "spchad.intfb.h"
#include "transdirhad.intfb.h"
#include "transinvhad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('STEPOAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       0.    VARIOUS INITIALIZATION
!              ----------------------

CALL USER_CLOCK(PELAPSED_TIME=ZWT,PVECTOR_CP=ZVT,PTOTAL_CP=ZCT)
ZCT=ZCT-RSTART
ZVT=ZVT-RVSTART
ZWT=ZWT-RTIMEF
RSTART=RSTART+ZCT
RVSTART=RVSTART+ZVT
RTIMEF=RTIMEF+ZWT
ZJUM=10._JPRB**(INT(LOG10(REAL(MAX(NSTOP-NSTART,1),JPRB)))+1)
IJUM=NINT(MAX(ZJUM/100._JPRB,1.0_JPRB))
IF(NSTEP-NSTART <= 10.OR.MOD(NSTEP,IJUM) == 0)THEN
  WRITE(NULOUT,'('' NSTEP ='',I6,'' STEPOAD '',A9)')&
   & NSTEP,CDCONF
ENDIF
IF (LOPDIS) THEN
  CALL OPDIS(CDCONF,'STEPOAD ',ZCT,ZVT,ZWT,RSTART,RVSTART,RTIMEF)
ENDIF

!        0.1   Define configurations for coupling LBC in LAM

LLCPL = (LELAM.AND.LECOBI.AND.NECRIPL == 1) .AND.&
 & ((CDCONF(3:3) == 'A'.OR.CDCONF(3:3) == 'B'.OR.&
 & CDCONF(6:6) == '1').AND.CDCONF(5:5) == '0') .AND.&
 & (NECOAD < 0)  

!     ------------------------------------------------------------------

!*       1.    READ TRAJECTORY (NOT NORMALLY DONE HERE ANY LONGER).
!              ----------------------------------------------------

IF(CDCONF(1:1) /= '0')THEN
  CALL IOPACK(CDCONF(1:1))
ENDIF
!     ------------------------------------------------------------------

!*       9.    SPECTRAL COMPUTATIONS.
!              ----------------------

IF(CDCONF(9:9) /= '0')THEN
  IF (.NOT.LELAM) THEN
    CALL SPCHAD(CDCONF(9:9))
  ELSE
    CALL ESPCHAD(CDCONF(9:9))
  ENDIF
ENDIF
!     ------------------------------------------------------------------

!*       8.    DIRECT TRANSFORMS.
!              ------------------

IF(CDCONF(8:8) /= '0')THEN
  IF (LELAM) THEN
    CALL ETRANSDIRHAD(CDCONF)
    IF (LLCPL) CALL ECOUPL1AD
  ELSE
    CALL TRANSDIRHAD(CDCONF)
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       4.    JO COMPUTATION
!              --------------
! Moved to CNT4AD
!     ------------------------------------------------------------------

!*       3.   GRIDPOINT COMPUTATIONS.
!             -----------------------

IF(CDCONF(3:7) /= '00000') THEN
  CALL SCAN2HAD(CDCONF)
ENDIF

!     ------------------------------------------------------------------
!*       2.    INVERSE TRANSFORMS.
!              -------------------

IF(CDCONF(2:2) /= '0')THEN
  IF (LELAM) THEN
    CALL ETRANSINVHAD(CDCONF)
  ELSE
    CALL TRANSINVHAD(CDCONF)
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('STEPOAD',1,ZHOOK_HANDLE)
END SUBROUTINE STEPOAD
