SUBROUTINE DOLFIL(PTAU,PDT,KSTEP,LDWIND,PHW,PHW0)

!**** *DOLFIL* -  Computing DFI weights for Dolph-Chebyshev filter

!     Purpose :  Computes filtering weights used during
!     -------    digital filter initialization : Dolph-Chebyshev filter

!** Interface :
!   ----------
!              *CALL* *DOLFIL(..)*

!              Explicit arguments :
!              -------------------
!  Input :
! PTAU    :  Period (seconds) of 
!   stop-band edge of the filter itself or
!   cut-off period of the main filter for a use as a window
! PDT     :  Time-step (seconds)
! KSTEP   :  Half-length of the filter
! LDWIND  :  Use of the filter as a window
!  Output :
! PHW :  Impulse Response / Weights : PHW(i)= weight of x(i-1) i=1,2*KSTEP+1
! PHW0 : weights

!              Implicit arguments :  None
!              -------------------

!     Method : see reference
!     ------

!     Externals : FCHEBY
!     ---------

!     Reference :     Documentation of ARPEGE/ALADIN
!     ---------       Peter Lynch (1996)

!     Author :        D. Giard  *CNRM/GMAP/MOD*
!     ------

!     Modifications :
!     -------------
!        Original : 96-01-18
!        P. Gauthier : 99-10-07 Option LDWIND
!        D. Giard    : 99-10-22 LDPRINT removed, corrections
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        P. Termonia : 15-Dec-08 Modifs for SSDFI
!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0    ,ONLY : LELAM
USE YOMDIM    ,ONLY : NSPEC2   ,NUMP
USE YEMLAP    ,ONLY : NCPL4M
USE YOMLAP    ,ONLY : MYMS
USE YOMDFI    ,ONLY : RDFIS
USE YEMGEO    ,ONLY : ELX      ,ELY

!----------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAU 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
LOGICAL           ,INTENT(IN)    :: LDWIND 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHW(NSPEC2,2*KSTEP+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PHW0(2*KSTEP+1) 

!----------------------------------------------------------------------

REAL(KIND=JPRB), EXTERNAL :: FCHEBY
REAL(KIND=JPRB) :: ZX(0:90),ZP(0:90)
INTEGER(KIND=JPIM) :: J, ICNT, JC, IM, JN, JMLOC, JST

REAL(KIND=JPRB) :: ZD, ZH, ZL, ZN, ZPI, ZR, ZS, ZTETAS, ZTETAC
REAL(KIND=JPRB) :: ZKAP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!----------------------------------------------------------------------

#include "abor1.intfb.h"

!----------------------------------------------------------------------


IF (LHOOK) CALL DR_HOOK('DOLFIL',0,ZHOOK_HANDLE)

!----------------------------------------------------------------------

! Everything reset to 0.

DO J=1,2*KSTEP+1
  PHW0(J)= 0.0_JPRB
ENDDO
DO J=0,90
  ZX(J)= 0.0_JPRB
  ZP(J)= 0.0_JPRB
ENDDO
! Constants
ZPI= 2.0_JPRB*ASIN(1.0_JPRB)
ZL= REAL(2*KSTEP+1,JPRB)
! Stop-band edge (half span if window, taus else)
IF (LDWIND) THEN
  ZTETAS= 2.0_JPRB*ZPI/REAL(KSTEP,JPRB)
ELSE
  ZTETAS= 2.0_JPRB*ZPI*PDT/PTAU
ENDIF
! Discretization of [0,pi/2]; computation of Chebyshev polynomials
ZD= ZPI/ZL
ZS= 1.0_JPRB/COS(ZTETAS/2.0_JPRB)
DO J=0,KSTEP
  ZX(J)= REAL(J,JPRB)*ZD
  ZP(J)= FCHEBY(ZS*COS(ZX(J)),2*KSTEP)
ENDDO
ZR= 2.0_JPRB/ZP(0)
! Computing weights : forward then backward time-steps
DO JST=0,KSTEP
  ZH= 1.0_JPRB
  DO J=1,KSTEP
    ZH= ZH + ZR*ZP(J)*COS(REAL(2*J,JPRB)*ZX(JST))
  ENDDO
  PHW0(KSTEP+1+JST)= ZH/ZL
ENDDO
DO JST=1,KSTEP
  PHW0(KSTEP+1-JST)= PHW0(KSTEP+1+JST)
ENDDO
! Combining with simple filter
IF (LDWIND) THEN
  ZTETAC = 2.0_JPRB*ZPI*PDT/PTAU
  ZS =0.0_JPRB
  DO JST=-KSTEP,KSTEP
    ZN = REAL(JST,JPRB)
    IF (JST == 0) THEN
      ZH = ZTETAC/ZPI
    ELSE
      ZH = SIN(ZN*ZTETAC)/(ZN*ZPI)
    ENDIF
    PHW0(KSTEP+1-JST)= PHW0(KSTEP+1-JST)*ZH
    ZS= ZS + PHW0(KSTEP+1-JST)
  ENDDO
  DO JST=-KSTEP,KSTEP
    PHW0(KSTEP+1-JST)= PHW0(KSTEP+1-JST)/ZS
  ENDDO
ENDIF

IF (LELAM) THEN

  ZPI=2.0_JPRB*ASIN(1.0_JPRB)
  ICNT=0
  DO JMLOC=1,NUMP
    IM=MYMS(JMLOC)
    DO JN=0,(NCPL4M(IM)/4)-1
      ZKAP=2._JPRB*ZPI*SQRT(REAL(IM,JPRB)**2/(ELX**2)+REAL(JN,JPRB)**2/(ELY**2))
      ZTETAC = 2.0_JPRB*ZPI*PDT/PTAU
      DO JC=1,4
        ICNT=ICNT+1
        DO J=1,2*KSTEP+1
          PHW(ICNT,J)= 0.0_JPRB
        ENDDO
        DO J=0,90
          ZX(J)= 0.0_JPRB
          ZP(J)= 0.0_JPRB
        ENDDO
        ! Constants
        ZPI= 2.0_JPRB*ASIN(1.0_JPRB)
        ZL= REAL(2*KSTEP+1,JPRB)
        ! Stop-band edge (half span if window, taus else)
        IF (LDWIND) THEN
          ZTETAS= 2.0_JPRB*ZPI/REAL(KSTEP,JPRB)
        ELSE
          ZTETAS= 2.0_JPRB*ZPI*PDT/PTAU
        ENDIF
        ! Discretization of [0,pi/2]; computation of Chebyshev polynomials
        ZD= ZPI/ZL
        ZS= 1.0_JPRB/COS(ZTETAS/2.0_JPRB)
        DO J=0,KSTEP
          ZX(J)= REAL(J,JPRB)*ZD
          ZP(J)= FCHEBY(ZS*COS(ZX(J)),2*KSTEP)
        ENDDO
        ZR= 2.0_JPRB/ZP(0)
        ! Computing weights : forward then backward time-steps
        DO JST=0,KSTEP
          ZH= 1.0_JPRB
          DO J=1,KSTEP
            ZH= ZH + ZR*ZP(J)*COS(REAL(2*J,JPRB)*ZX(JST))
          ENDDO
          PHW(ICNT,KSTEP+1+JST)= ZH/ZL
        ENDDO
        DO JST=1,KSTEP
          PHW(ICNT,KSTEP+1-JST)= PHW(ICNT,KSTEP+1+JST)
        ENDDO
        ! Combining with simple filter
        IF (LDWIND) THEN
          ZTETAC = 2.0_JPRB*ZPI*PDT/PTAU
          ZTETAC=MIN(ZTETAC+RDFIS*ZKAP*PDT,ZPI)
          ZS =0.0_JPRB
          DO JST=-KSTEP,KSTEP
            ZN = REAL(JST,JPRB)
            IF (JST == 0) THEN
              ZH = ZTETAC/ZPI
            ELSE
              ZH = SIN(ZN*ZTETAC)/(ZN*ZPI)
            ENDIF
            PHW(ICNT,KSTEP+1-JST)= PHW(ICNT,KSTEP+1-JST)*ZH
            ZS= ZS + PHW(ICNT,KSTEP+1-JST)
          ENDDO
          DO JST=-KSTEP,KSTEP
            PHW(ICNT,KSTEP+1-JST)= PHW(ICNT,KSTEP+1-JST)/ZS
          ENDDO
        ENDIF
      ENDDO
    ENDDO
  ENDDO
  IF (ICNT/=NSPEC2) CALL ABOR1(' DOLFIL: BUG ICNT NE NSPEC2')

ELSE

  ! Wavenumber dependent formulation not yet coded for spherical geometry.
  ! Provisional code to reproduce the old (not wavenumber dependent) code.
  DO J=1,2*KSTEP+1
    PHW(1:NSPEC2,J)=PHW0(J)
  ENDDO

ENDIF

!----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('DOLFIL',1,ZHOOK_HANDLE)
END SUBROUTINE DOLFIL
