SUBROUTINE OPTFIL(PTAUP,PTAUS,PDT,KLEN,LDPRINT,PHW)

!**** *OPTFIL* -  Computing DFI weights for optimal filter

!     Purpose :  Computes filtering weights used during
!     -------    digital filter initialization : optimal filter

!** Interface :
!   ----------
!              *CALL* *OPTFIL(..)*

!              Explicit arguments :
!              -------------------
!  Input :
! PTAUP   :  Period of pass-band edge (seconds)
! PTAUS   :  Period of stop-band edge (seconds)
! PDT     :  Time-step (seconds)
! KLEN    :  Half-length of the filter
! LDPRINT :  Logical switch for messages
!  Output :
! PHW :  Impulse Response / Weights : PHW(i)= weight of x(i-1) i=1,2*KLEN+1

!              Implicit arguments :  None
!              -------------------

!     Method : see reference
!     ------

!     Externals : OPTFILB , ABORT
!     ---------

!     Reference :     Documentation of ARPEGE/ALADIN
!     ---------       HIRLAM Technical Report nr 10

!     Author :        G. Radnoti  *CNRM/GMAP/MICECO*
!     ------

!     Modifications :
!     -------------
!        Original : 93-01-20
!        D. Giard : 95-04-07  splitting , doctor form
!        D. Giard : 96-01-18  deviations are no longer output arguments
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAUP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAUS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
LOGICAL           ,INTENT(IN)    :: LDPRINT 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHW(181) 
REAL(KIND=JPRB) :: ZF(20),ZFX(10),ZWTX(10),ZDEVIA(10)
INTEGER(KIND=JPIM) :: IBANDS, IGRID, ILEN, ITYPE, J

REAL(KIND=JPRB) :: ZCV, ZDF, ZDT, ZDTAU, ZFC, ZSUMH, ZTAUC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "optfilb.intfb.h"

!        1. Fill the control vectors for MCCPAR

! Length of filter
IF (LHOOK) CALL DR_HOOK('OPTFIL',0,ZHOOK_HANDLE)
ILEN = 2*KLEN+1
IF (ILEN < 3.OR. ILEN > 128)&
 & CALL ABOR1(' Check number of timesteps for OPTFIL')  
! The following three should always be the same :
ITYPE = 1
IBANDS = 2
IGRID = 16
! Calculate transition frequencies
! (f=Dt/TAU : 1-> f=0  2-> pass-band (TAUP)  3-> stop-band (TAUS)  4-> TAU=2*Dt)
ZDT = ABS(PDT)
ZF(1) = 0.0_JPRB
ZF(2) = ZDT/PTAUP
ZF(3) = ZDT/PTAUS
ZF(4) = 0.5_JPRB
IF(ZF(2) < 0.0_JPRB) CALL ABOR1(' TAUP too large in OPTFIL')
IF(ZF(3) > 0.5_JPRB) CALL ABOR1(' TAUS too small in OPTFIL')
! Slopes
ZFX(1) = 1.0_JPRB
ZFX(2) = 0.0_JPRB
! Relative weights in pass(1)- and stop(2)-bands.
ZWTX(1) = 1.0_JPRB
ZWTX(2) = 1.0_JPRB

!        2. Compute weights

CALL OPTFILB(ILEN,ITYPE,IBANDS,IGRID,LDPRINT,ZF,ZFX,ZWTX,ZDEVIA,PHW)
! Fill out the array PHW (only first half filled in MCCPAR).
DO J=1,KLEN
  PHW(ILEN+1-J) = PHW(J)
ENDDO
! Normalize the sums to be unity.
ZSUMH = 0.0_JPRB
DO J=1,ILEN
  ZSUMH = ZSUMH + PHW(J)
ENDDO
DO J=1,ILEN
  PHW(J) = PHW(J)/ZSUMH
ENDDO

!        2. Printings

!   TAUP: period of pass-band edge (s)  TAUS: period of stop-band edge (s)
!   TAUC: cut-off period (s)            DTAU: transition half-width (s)
!   FP = DT/TAUP  ,  FS = DT/TAUS  ,  FC = DT/TAUC  ,  DF = DT/DTAU
! (FP,FC,FS) is an arithmetic progression :
!   FP = FC - DF  ,  FS = FC + DF  ,  FC =(FS+FP)/2 ,  DF =(FS-FP)/2
! so (1/FS,1/FC,1/FP) is a harmonic one :
!   TAUP= 1/( (1/TAUC)-(1/DTAU) )       TAUS = 1/( (1/TAUC)+(1/DTAU) )
!   TAUC= 2/( (1/TAUS)+(1/TAUP) )       DTAU = 2/( (1/TAUS)-(1/TAUP) )

IF (LDPRINT) THEN
  ZTAUC = 2.0_JPRB/((1.0_JPRB/PTAUS)+(1.0_JPRB/PTAUP))
  ZDTAU = 2.0_JPRB/((1.0_JPRB/PTAUS)-(1.0_JPRB/PTAUP))
  ZFC = ZDT/ZTAUC
  ZDF = ZDT/ZDTAU
  ZCV = 3600._JPRB
  WRITE(6,'('' Optimal Filter '')')
  WRITE(6,'('' TAUs TAUp TAUc dTAU (h)'',4F8.3)')&
   & PTAUS/ZCV,PTAUP/ZCV,ZTAUC/ZCV,ZDTAU/ZCV  
  WRITE(6,'(''  Fs   Fp   Fc   dF     '',4F8.5)')ZF(3),ZF(2),ZFC,ZDF
  WRITE(6,'('' Weights in stop and pass bands    '',2F8.4)')ZWTX(2),ZWTX(1)
  WRITE(6,'('' Deviations in stop and pass bands '',2F8.4)')&
   & ZDEVIA(2),ZDEVIA(1)  
ENDIF

IF (LHOOK) CALL DR_HOOK('OPTFIL',1,ZHOOK_HANDLE)
END SUBROUTINE OPTFIL

