SUBROUTINE OPTFILB(KLEN,KTYPE,KBANDS,KGRID,LDPRINT,PF,PFX,PWTX,PDEVIA,PHW)

!**** *OPTFILB* -  Computing DFI weights for optimal filter

!     Purpose :  Program for the design of linear phase finite impulse
!     -------    response (FIR) filters using the Remez exchange algorithm
!                Three types of filters are included : Bandpass filters (1),
!                Differentiators (2) and Hilbert transform filters (3)

!** Interface :
!   ----------
!              *CALL* *OPTFILB(..)*

!              Explicit arguments :
!              -------------------
!  Input :
! KLEN    :  Length of the filter
! KTYPE   :  Type of the filter
! KBANDS  :  Number of frequency bands
! KGRID   :  Time-step (seconds)
! LDPRINT :  Logical switch for messages
! PF      :  Transition frequencies (defining bands)
! PFX     :  Slopes (for each band)
! PWTX    :  Relative weights (for each band)
!  Output :
! PDEVIA  :  Deviation (for each band)
! PHW     :  Weights (haIf array)

!              Implicit arguments :  None
!              -------------------

!     Method : see reference
!     ------

!     Externals : REMEZ , ABORT
!     ---------

!     Reference :  McClellan, J.H., T.W. Parks and L.R.Rabiner,1973 :
!     ---------    " A computer program for designing optimum FIR
!                  linear phase digital filters "
!                  IEEE Trans. on Audio and Electroacoustics,
!                  Vol AU-21, No. 6, 506-526.

!     Author :
!     ------

!     Modifications :
!     -------------
!        G. Radnoti : 93-01-20 copy and adaptation (MCCPAR)
!        D. Giard   : 95-04-07 splitting, doctor form, single precision
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMOPF   , ONLY : JPNLX    ,NEXT     ,ALPHO    ,GRID     ,&
 & FS       ,WT       ,R2PI     ,ADEV     ,NLEN     ,&
 & NGRID  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPE 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KBANDS 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KGRID 
LOGICAL           ,INTENT(IN)    :: LDPRINT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PF(20) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFX(10) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWTX(10) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDEVIA(10) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHW(181) 
REAL(KIND=JPRB) :: ZDEVIA(10)
CHARACTER (LEN = 1) ::  CLSIGN

INTEGER(KIND=JPIM) :: IBAND, IJ, IKP, IL, J, JK

LOGICAL :: LLCAS, LLODD, LLTP1, LLTP2

REAL(KIND=JPRB) :: ZCHANGE, ZDF, ZF, ZFUP, ZZ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "remez.intfb.h"

!       1. PROGRAM INPUT SECTION
!       ------------------------

! check arguments
IF (LHOOK) CALL DR_HOOK('OPTFILB',0,ZHOOK_HANDLE)
IF (KLEN > 2*JPNLX .OR. KLEN < 3) CALL ABOR1('OPTFILB: ABOR1 CALLED')
IF (KTYPE > 3.OR. KTYPE < 1) CALL ABOR1('OPTFILB: ABOR1 CALLED')
IF (KGRID <= 0) KGRID = 16
KBANDS = MIN(10,MAX(1,KBANDS))
! constants
R2PI = 4._JPRB*ASIN(1.0_JPRB)
LLTP1 = KTYPE == 1
LLTP2 = KTYPE == 2
LLODD= MOD(KLEN,2) == 1
NLEN = KLEN/2
IF (LLODD .AND. LLTP1) NLEN = NLEN+1

! frequency partition , ordering and description of bands
GRID(1) = PF(1)
ZDF = 0.5_JPRB/REAL(KGRID*NLEN,JPRB)
IF (.NOT.LLTP1) GRID(1) = MAX(GRID(1),ZDF)

IJ = 1
IL = 1
IBAND = 1
100 CONTINUE
ZFUP = PF(IL+1)

105 CONTINUE
ZF  = GRID(IJ)
FS(IJ) = PFX (IBAND)
WT(IJ) = PWTX(IBAND)
IF (LLTP2) THEN
  FS(IJ) = FS(IJ)*ZF
  IF (PFX(IBAND) >= 1.E-4_JPRB) WT(IJ) = WT(IJ)/ZF
ENDIF
IJ = IJ+1
GRID(IJ) = ZF+ZDF
IF (GRID(IJ) <= ZFUP) GOTO 105

GRID(IJ-1) = ZFUP
IF (LLTP2) THEN
  FS(IJ-1) = PFX (IBAND)*ZFUP
  IF (PFX(IBAND) >= 1.E-4_JPRB) WT(IJ-1) = PWTX(IBAND)/ZFUP
ENDIF
IBAND = IBAND+1
IL = IL+2
IF (IBAND <= KBANDS) THEN
  GRID(IJ) = PF(IL)
  GOTO 100
ENDIF

NGRID = IJ-1
IF ( ((LLTP1.AND..NOT.LLODD).OR.(.NOT.LLTP1.AND.LLODD))&
 & .AND. (GRID(NGRID) > (0.5_JPRB-ZDF)) ) NGRID = NGRID-1  

! correction of slopes and weights
IF (LLTP1) THEN
  IF (.NOT.LLODD) THEN
    DO J=1,NGRID
      ZCHANGE = COS(0.5_JPRB*R2PI*GRID(J))
      FS(J) = FS(J)/ZCHANGE
      WT(J) = WT(J)*ZCHANGE
    ENDDO
  ENDIF
ELSE
  IF (.NOT.LLODD) THEN
    DO J = 1,NGRID
      ZCHANGE = SIN(0.5_JPRB*R2PI*GRID(J))
      FS(J) = FS(J)/ZCHANGE
      WT(J) = WT(J)*ZCHANGE
    ENDDO
  ELSE
    DO J =1,NGRID
      ZCHANGE = SIN(R2PI*GRID(J))
      FS(J) = FS(J)/ZCHANGE
      WT(J) = WT(J)*ZCHANGE
    ENDDO
  ENDIF
ENDIF
! initial choice of extremal frequencies
ZZ = REAL(NGRID-1,JPRB)/REAL(NLEN,JPRB)
DO J=1,NLEN
  NEXT(J) = (J-1)*ZZ+1
ENDDO
NEXT(NLEN+1) = NGRID
NEXT(NLEN+2) = NGRID+1

!       2. CALCULATE THE IMPULSE RESPONSE
!       ---------------------------------

! call the Remez exchange algorithm to do the approximation problem
LLCAS = (PF(1) == 0.0_JPRB .AND. PF(2*KBANDS) == 0.5_JPRB) .OR. (NLEN <= 3)
CALL REMEZ(LLCAS)
! compute weights (half array)
IF (LLTP1) THEN
  IF (LLODD) THEN
    DO J=1,NLEN-1
      PHW(J) = 0.5_JPRB*ALPHO(NLEN+1-J)
    ENDDO
    PHW(NLEN) = ALPHO(1)
  ELSE
    PHW(1) = 0.25_JPRB*ALPHO(NLEN)
    DO J=2,NLEN-1
      PHW(J) = 0.25_JPRB*(ALPHO(NLEN+1-J)+ALPHO(NLEN+2-J))
    ENDDO
    PHW(NLEN) = 0.5_JPRB*ALPHO(1)+0.25_JPRB*ALPHO(2)
  ENDIF
ELSE
  IF (LLODD) THEN
    PHW(1) = 0.25_JPRB*ALPHO(NLEN)
    PHW(2) = 0.25_JPRB*ALPHO(NLEN-1)
    DO J=3,NLEN-1
      PHW(J) = 0.25_JPRB*(ALPHO(NLEN+1-J)-ALPHO(NLEN+3-J))
    ENDDO
    PHW(NLEN) = 0.5_JPRB*ALPHO(1)-0.25_JPRB*ALPHO(3)
    PHW(NLEN+1) = 0.0_JPRB
  ELSE
    PHW(1) = 0.25_JPRB*ALPHO(NLEN)
    DO J=2,NLEN-1
      PHW(J) = 0.25_JPRB*(ALPHO(NLEN+1-J)-ALPHO(NLEN+2-J))
    ENDDO
    PHW(NLEN) = 0.5_JPRB*ALPHO(1)-0.25_JPRB*ALPHO(2)
  ENDIF
ENDIF
! compute deviations
DO J=1,KBANDS
  PDEVIA(J) = ADEV/PWTX(J)
ENDDO

!       3. PROGRAM OUTPUT SECTION
!       -------------------------

IF (LDPRINT) THEN

  WRITE(6,'(/,80(''*''),//,5X,''FINITE IMPULSE RESPONSE (FIR)'',&
   & /,5X,''LINEAR PHASE DIGITAL FILTER DESIGN'',&
   & /,5X,''REMEZ EXCHANGE ALGORITHM'',/)')  
  IF (KTYPE == 1) WRITE(6,'(5X,''BANDPASS FILTER    '',/)')
  IF (KTYPE == 2) WRITE(6,'(5X,''DIFFERENTIATOR     '',/)')
  IF (KTYPE == 3) WRITE(6,'(5X,''HILBERT TRANSFORMER'',/)')
  WRITE(6,'(5X,''FILTER LENGTH ='',I3,/)') KLEN
  WRITE(6,'(5X,'' ***** IMPULSE RESPONSE ***** '')')

  IF (LLTP1) THEN
    CLSIGN=' '
  ELSE
    CLSIGN='-'
  ENDIF
  DO J=1,NLEN
    WRITE(6,'(5X,''H('',I3,'')='',E15.8,'' ='',A,''H('',&
     & I3,'')'')')&
     & J,PHW(J),CLSIGN,KLEN+1-J  
  ENDDO
  IF (LLODD.AND..NOT.LLTP1)WRITE(6,'(5X,''H('',I3,'')= 0.'')') NLEN+1

  DO JK=1,KBANDS,4
    IKP = MIN(JK+3,KBANDS)
    WRITE(6,'(/,24X,4(''BAND'',I3,8X))') (J,J=JK,IKP)
    WRITE(6,'(''  LOWER BAND EDGE'',5F15.8)') (PF(2*J-1),J=JK,IKP)
    WRITE(6,'(''  UPPER BAND EDGE'',5F15.8)') (PF(2*J),J=JK,IKP)
    IF (LLTP2) THEN
      WRITE(6,'(''  DESIRED SLOPE  '',5F15.8)') (PFX(J),J=JK,IKP)
    ELSE
      WRITE(6,'(''  DESIRED VALUE  '',5F15.8)') (PFX(J),J=JK,IKP)
    ENDIF
    WRITE(6,'(''  WEIGHTING      '',5F15.8)') (PWTX(J),J=JK,IKP)
    WRITE(6,'(''  DEVIATION      '',5F15.8)') (PDEVIA(J),J=JK,IKP)
    IF (LLTP1) THEN
      DO J=JK,IKP
        ZDEVIA(J) = 20._JPRB*LOG10(PDEVIA(J))
      ENDDO
      WRITE(6,'(''  DEVIATION IN DB'',5F15.8)') (ZDEVIA(J),J=JK,IKP)
    ENDIF
  ENDDO
  WRITE(6,'(/,''  EXTREMAL FREQUENCIES'',/,(2X,5F15.7))')&
   & (GRID(NEXT(J)),J=1,NLEN+1)  
  WRITE(6,'(/,80(''*''),/)')

ENDIF

IF (LHOOK) CALL DR_HOOK('OPTFILB',1,ZHOOK_HANDLE)
END SUBROUTINE OPTFILB
