SUBROUTINE REMEZ(LDCAS)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMOPF   , ONLY : JPNLX    ,NEXT     ,AD       ,AX       ,&
 & AY       ,ALPHO    ,GRID     ,FS       ,WT       ,&
 & R2PI     ,ADEV     ,NLEN     ,NGRID  

!**** *REMEZ* -  Remez exchange algorithm

!     Purpose : This subroutine implements the Remez exchange algorithm
!     -------   for the weighted Chebyshev approximation of a continuous
!               function with a sum of cosines. Inputs to the subroutine
!               are :
!                 a dense grid which replaces the frequency axis,
!                 the desired function on this grid,
!                 the weight function on the grid,
!                 the number of cosines,
!                 and the initial guess of the extremal frequencies.
!               The program minimizes the Chebyshev error by determining
!               the best location of the extremal frequencies (points of
!               maximum error) and then calculates the coefficients of
!               the best approximation.

!** Interface :
!   ----------
!              *CALL* *REMEZ(LDCAS)*

!              Explicit arguments :  LDCAS (Input) :  Special case
!              -------------------

!              Implicit arguments :  GRID,NEXT (I/O), FS,WT,NLEN (Input)
!              -------------------   ( common YOMOPF )

!     Externals : GEE
!     ---------

!     Reference :     Documentation of ARPEGE/ALADIN
!     ---------       HIRLAM Technical Report nr 10

!     Author :
!     ------

!     Modifications :
!     -------------
!        G. Radnoti : 93-01-20  copy and adaptation
!        D. Giard   : 95-04-07  splitting,doctor form,single precision
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!----------------------------------------------------------------------

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDCAS 
REAL(KIND=JPRB) :: ZA(JPNLX+3),ZP(JPNLX+3),ZQ(JPNLX+3)
REAL(KIND=JPRB), EXTERNAL :: GEE

INTEGER(KIND=JPIM) :: IDV, IK, IL, ILOW, ILUCK, IM1, IN, IN1, INZ,&
 & ITERMAX, IUP, IUT, IUT1, IZ, IZZ, J, JITER, &
 & JJ, JK  

LOGICAL :: LLCHANGE

REAL(KIND=JPRB) :: ZAA, ZBB, ZC1, ZCN, ZCOMP, ZCZ, ZDEV, ZDF,&
 & ZE, ZEPS, ZERR, ZFT, ZG1, ZN, ZSIGN, ZT, &
 & ZXE, ZXT  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!  THE PROGRAM ALLOWS A MAXIMUM NUMBER OF ITERATIONS OF 25

IF (LHOOK) CALL DR_HOOK('REMEZ',0,ZHOOK_HANDLE)
ITERMAX = 25
ZDEV = -1.0_JPRB
IZ = NLEN+1
IZZ = NLEN+2

DO JITER=1,ITERMAX

! COMPUTE AX,AY,AD,ADEV WITH THE NEW ORDERING / PARTITION OF FREQUENCIES

  NEXT(IZZ) = NGRID+1

  DO J=1,IZ
    AX(J) = COS( R2PI* GRID(NEXT(J)) )
  ENDDO

  DO J=1,IZ
    ZN = 1.0_JPRB
    ZT = AX(J)
    DO JJ=1,IZ
      IF (JJ /= J) ZN = 2.0_JPRB*ZN*(ZT-AX(JJ))
    ENDDO
    AD(J) = 1.0_JPRB/ZN
  ENDDO

  ZN = 0.0_JPRB
  ZE = 0.0_JPRB
  IK = 1
  DO J=1,IZ
    IL = NEXT(J)
    ZT = AD(J)
    ZN = ZN + ZT*FS(IL)
    ZE = ZE + ZT*IK/WT(IL)
    IK = -IK
  ENDDO
  ADEV = ZN/ZE
  IDV = SIGN(1.0_JPRB,-ADEV)
  ADEV = ABS(ADEV)

  IK = IDV
  DO J=1,IZ
    IL = NEXT(J)
    AY(J) = FS(IL)+ADEV*IK/WT(IL)
    IK = -IK
  ENDDO

  IF (ADEV < ZDEV) THEN
    WRITE(6,*) ' ******** FAILURE TO CONVERGE *********** '
    WRITE(6,*) ' PROBABLE CAUSE IS MACHINE ROUNDING ZERROR '
    WRITE(6,*) ' THE IMPULSE RESPONSE MAY BE CORRECT '
    WRITE(6,*) ' CHECK WITH A FREQUENCY RESPONSE '
    WRITE(6,*) ' **************************************** '
    GOTO 400
  ENDIF
  ZDEV = ADEV

  LLCHANGE = .FALSE.
  IN1 = NEXT(1)
  INZ = NEXT(IZ)
  ILOW = 0
  IUT = -IDV

!  SEARCH FOR THE EXTERNAL FREQUENCIES OF THE BEST APPROXIMATION.

  DO J=1,IZ

    IUP = NEXT(J+1)
    IL = NEXT(J)+1
    IUT = -IUT
    IF (J == 2) ZC1 = ZCOMP
    ZCOMP = ADEV
    IF (IL >= IUP) THEN
      ZT = 0.0_JPRB
    ELSE
      ZERR = (GEE(IL,IZ)-FS(IL))*WT(IL)
      ZT = IUT*ZERR-ZCOMP
    ENDIF
    IF (ZT > 0.0_JPRB) THEN
      ZCOMP = IUT*ZERR
    ELSE
      IL=IL-1

      210 CONTINUE
      IL=IL-1
      IF (IL > ILOW) THEN
        ZERR = (GEE(IL,IZ)-FS(IL))*WT(IL)
        ZT = IUT*ZERR-ZCOMP
        IF (ZT > 0.0_JPRB) THEN
          ZCOMP=IUT*ZERR
          220 CONTINUE
          IL=IL-1
          IF (IL <= ILOW) THEN
            ZT = 0.0_JPRB
          ELSE
            ZERR = (GEE(IL,IZ)-FS(IL))*WT(IL)
            ZT = IUT*ZERR-ZCOMP
          ENDIF
          IF (ZT > 0.0_JPRB) THEN
            ZCOMP = IUT*ZERR
            GOTO 220
          ENDIF
          ILOW = NEXT(J)
          NEXT(J) = IL+1
          LLCHANGE = .TRUE.
          GOTO 200
        ELSEIF (LLCHANGE) THEN
          ILOW = NEXT(J)
          GOTO 200
        ELSE
          GOTO 210
        ENDIF
      ENDIF
      IL = NEXT(J)+1
      IF (LLCHANGE) GOTO 260

      240 CONTINUE
      IL = IL+1
      IF (IL >= IUP) THEN
        ILOW = NEXT(J)
        GOTO 200
      ENDIF
      ZERR = (GEE(IL,IZ)-FS(IL))*WT(IL)
      ZT = IUT*ZERR-ZCOMP
      IF (ZT <= 0.0_JPRB) GOTO 240
      ZCOMP = IUT*ZERR

    ENDIF

    250 CONTINUE
    IL=IL+1
    IF (IL >= IUP) THEN
      ZT = 0.0_JPRB
    ELSE
      ZERR = (GEE(IL,IZ)-FS(IL))*WT(IL)
      ZT = IUT*ZERR-ZCOMP
    ENDIF
    IF (ZT > 0.0_JPRB) THEN
      ZCOMP = IUT*ZERR
      GOTO 250
    ENDIF

    260 CONTINUE
    NEXT(J) = IL-1
    ILOW = NEXT(J)
    LLCHANGE = .TRUE.

    200 continue
  ENDDO

  J = IZZ
  ZCZ = ZCOMP
  IN1 = MIN(NEXT(1),IN1)
  INZ = MAX(NEXT(IZ),INZ)
  IUT1 = IUT
  IUT = -IDV
  IL = 0
  IUP = IN1
  ZCOMP = ZCZ*(1.00001_JPRB)
  ILUCK = 1

  310 CONTINUE
  IL = IL+1
  IF (IL >= IUP) THEN
    ILUCK=2
    GOTO 325
  ENDIF
  ZERR = (GEE(IL,IZ)-FS(IL))*WT(IL)
  ZT = IUT*ZERR-ZCOMP
  IF (ZT <= 0.0_JPRB) GOTO 310
  ZCOMP = IUT*ZERR
  J = IZZ

  315 CONTINUE
  IL=IL+1
  IF (IL >= IUP) THEN
    ZT = 0.0_JPRB
  ELSE
    ZERR = (GEE(IL,IZ)-FS(IL))*WT(IL)
    ZT = IUT*ZERR-ZCOMP
  ENDIF
  IF (ZT > 0.0_JPRB) THEN
    ZCOMP = IUT*ZERR
    GOTO 315
  ENDIF
  NEXT(J) = IL-1
  ILOW = IL-1
  LLCHANGE = .TRUE.

  320 CONTINUE
  IF (ILUCK > 2) GOTO 360
  ZC1 = MAX(ZCOMP,ZC1)
  IN1 = NEXT(IZZ)

  325 CONTINUE
  IL = NGRID+1
  ILOW = INZ
  IUT = -IUT1
  ZCOMP = ZC1*(1.00001_JPRB)

  330 CONTINUE
  IL = IL-1
  IF (IL <= ILOW) GOTO 340
  ZERR = (GEE(IL,IZ)-FS(IL))*WT(IL)
  ZT = IUT*ZERR-ZCOMP
  IF (ZT <= 0.0_JPRB) GOTO 330
  J = IZZ
  ZCOMP = IUT*ZERR
  ILUCK = ILUCK+2

  335 CONTINUE
  IL=IL-1
  IF (IL <= ILOW) THEN
    ZT = 0.0_JPRB
  ELSE
    ZERR = (GEE(IL,IZ)-FS(IL))*WT(IL)
    ZT = IUT*ZERR-ZCOMP
  ENDIF
  IF (ZT > 0.0_JPRB) THEN
    ZCOMP = IUT*ZERR
    GOTO 335
  ENDIF
  ILOW = NEXT(J)
  NEXT(J) = IL+1
  LLCHANGE = .TRUE.
  GOTO 320

  340 CONTINUE
  IF (ILUCK == 2) GOTO 380
  DO J=1,NLEN
    NEXT(IZZ-J) = NEXT(IZ-J)
  ENDDO
  NEXT(1) = IN1
  IF ( JITER < ITERMAX) GOTO 100
  360 CONTINUE
  IN=NEXT(IZZ)
  DO J=1,NLEN
    NEXT(J) = NEXT(J+1)
  ENDDO
  NEXT(IZ) = IN
  IF (JITER < ITERMAX) GOTO 100
  380 CONTINUE
  IF (.NOT.LLCHANGE) GOTO 400

  100 continue
ENDDO

!  CALCULATION OF THE COEFFICIENTS OF THE BEST APPROXIMATION
!  USING THE INVERSE DISCRETE FOURIER TRANSFORM.

400 CONTINUE

IM1 = NLEN-1
ZEPS = 1.E-6_JPRB
ZG1 = GRID(1)
AX(IZZ) = -2.0_JPRB
ZCN = 2*NLEN-1
ZDF = 1.0_JPRB/ZCN
IL = 1
IF (.NOT.LDCAS) THEN
  ZT = COS(R2PI*GRID(1))
  ZN = COS(R2PI*GRID(NGRID))
  ZAA = 2.0_JPRB/(ZT-ZN)
  ZBB = -(ZT+ZN)/(ZT-ZN)
ENDIF

DO J=1,NLEN

  ZFT = (J-1)*ZDF
  ZXT = COS(R2PI*ZFT)
  IF (.NOT.LDCAS) THEN
    ZXT = (ZXT-ZBB)/ZAA
    ZFT = ACOS(ZXT)/R2PI
  ENDIF
  420 CONTINUE
  ZXE = AX(IL)
  ZSIGN = SIGN(1.0_JPRB,ZXT-ZXE)
  IF (ZSIGN*(ZXT-ZXE) < ZEPS) THEN
    ZA(J) = AY(IL)
  ELSEIF (ZXT > ZXE) THEN
    GRID(1) = ZFT
    ZA(J) = GEE(1,IZ)
  ELSE
    IL = IL+1
    GOTO 420
  ENDIF
  IF (IL > 1) IL = IL-1

ENDDO

GRID(1) = ZG1
ZE = R2PI/ZCN

! OUTPUT (USED TO COMPUTE WEIGHTS)

DO J=1,NLEN
  ZT = 0.0_JPRB
  ZN = (J-1)*ZE
  IF (IM1 >= 1) THEN
    DO JK=1,IM1
      ZT = ZT + ZA(JK+1)*COS(ZN*JK)
    ENDDO
  ENDIF
  ZT = 2.0_JPRB*ZT+ZA(1)
  ALPHO(J) = ZT
ENDDO

ALPHO(1) = ALPHO(1)/ZCN
DO J=2,NLEN
  ALPHO(J) = 2*ALPHO(J)/ZCN
ENDDO

IF (.NOT.LDCAS) THEN
  ZP(1) = 2.0_JPRB*ALPHO(NLEN)*ZBB+ALPHO(IM1)
  ZP(2) = 2.0_JPRB*ZAA*ALPHO(NLEN)
  ZQ(1) = ALPHO(NLEN-2)-ALPHO(NLEN)
  DO J=2,IM1
    IF (J == IM1) THEN
      ZAA = 0.5_JPRB*ZAA
      ZBB = 0.5_JPRB*ZBB
    ENDIF
    ZP(J+1) = 0.0_JPRB
    DO JK=1,J
      ZA(JK) = ZP(JK)
      ZP(JK) = 2.0_JPRB*ZBB*ZA(JK)
    ENDDO
    ZP(2) = ZP(2) + 2.0_JPRB*ZAA*ZA(1)
    DO JK=1,J-1
      ZP(JK) = ZP(JK) + ZQ(JK) + ZAA*ZA(JK+1)
    ENDDO
    DO JK=3,J+1
      ZP(JK) = ZP(JK) + ZAA*ZA(JK-1)
    ENDDO
    IF (J < IM1) THEN
      DO JK=1,J
        ZQ(JK) = -ZA(JK)
      ENDDO
      ZQ(1) = ZQ(1) + ALPHO(IM1-J)
    ENDIF
  ENDDO
  DO J=1,NLEN
    ALPHO(J) = ZP(J)
  ENDDO

ELSEIF(NLEN <= 3) THEN
  ALPHO(NLEN+1) = 0.0_JPRB
  ALPHO(NLEN+2) = 0.0_JPRB
ENDIF

IF (LHOOK) CALL DR_HOOK('REMEZ',1,ZHOOK_HANDLE)
END SUBROUTINE REMEZ
