SUBROUTINE CPVPOSPR(CDCONF)

!**** *CPVPOSPR*  - COMPUTE FIELD POINTERS FOR VERTICAL POST-PROCESSING

!     PURPOSE.
!     --------
!        Compute the pointers of the post-processing fields so that
!        the fields are stored one after the others in the arrays
!        - GT1 (fitted fields) 
!        - GAUX (not fitted fields).
!        - DYNFP (horizontally post-processed fields).
!        Fields not requested are pointing at adress 0
!         To fit the spectral transforms ordering, fields are ordered in GT1 as follows : 
!          all "U", then all "V", then all scalars

!**   INTERFACE.
!     ----------
!       *CALL* *CPVPOSPR*

!        EXPLICIT ARGUMENTS
!        --------------------
!           INPUT  :
!        CDCONF   : Configuration of work for vertical post-processing

!        IMPLICIT ARGUMENTS
!        -----------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified : 95-01-20 Thierry Bergot (spectral filtering for all
!                   3D fields if LDDER=.TRUE., except wind component and Psi/Khi)
!    R. El Khatib : 97-06-02  3 NH fields
!    R. El Khatib : 98-12-11 Cleanup pp of liquid water
!    D.Dent       : 99-05-06 Support for log surface pressure
!    A.Beljaars   : 00-03-06 Extra field 2 and 3 moved from spectral to gp
!    D. Dent      : 00-04-12 Support for computation of Montgomery geopotential
!    R. El Khatib : 99-12-01 Enable processing of spectral ozone
!    R. El Khatib : 00-09-26 substantial rewrite
!    R. El Khatib : 01-03-28 compute QFPTYPE%IGT1 in the proper order to fit the spectral transforms
!    R. El Khatib : 01-04-04 QFPTYPE%IGT1(0) after inverse spectral transform
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMVPOS  , ONLY : QFPTYPE ,NFPOSDYN ,NFPTRDYN
USE YOMFPDIM , ONLY : NFPVECG ,NFPIVECG

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM) :: IPTRGPX  ! last pointer for GAUX
INTEGER(KIND=JPIM) :: IPTRGFP  ! last pointer for DYNFP
INTEGER(KIND=JPIM) :: J, INC, IJ, JL, ISPREAD
INTEGER(KIND=JPIM) :: IVOR     ! last pointer of vorticity fields in GT1
INTEGER(KIND=JPIM) :: IDIV     ! last pointer of divergence fields in GT1
INTEGER(KIND=JPIM) :: ISCA1    ! last pointer of scalar fields in GT1
INTEGER(KIND=JPIM) :: IUMOM    ! last pointer of U momenta in GT0
INTEGER(KIND=JPIM) :: IVMOM    ! last pointer of V momenta in GT0
INTEGER(KIND=JPIM) :: ISCA0    ! last pointer of scalar fields in GT0
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    COMPUTE POINTERS
!              ----------------

IF (LHOOK) CALL DR_HOOK('CPVPOSPR',0,ZHOOK_HANDLE)
IPTRGPX=1
IPTRGFP=1

IF (CDCONF == 'B' .OR. CDCONF == 'V' .OR. CDCONF == 'T' .OR.&
   & CDCONF == 'H' .OR. CDCONF == 'S') THEN  

! Compute pointers for GT1/GT0 & GAUX ; reset pointers for DYNFP
  IVOR=1
  IDIV=NFPVECG+1
  ISCA1=2*NFPVECG+1
  IUMOM=1
  IVMOM=NFPIVECG+1
  ISCA0=2*NFPIVECG+1
  QFPTYPE%IGT1(:,:)=0
  QFPTYPE%IGPX(:)=0
  QFPTYPE%IGFP(:)=0
  DO J=1,NFPOSDYN
    IJ=NFPTRDYN(J)
    IF (QFPTYPE%ISF(IJ) >= 1) THEN
!     Pointers % transforms : 
      INC=QFPTYPE%ILEV(IJ)
      ISPREAD=0
      DO  JL=1, INC
        ISPREAD=ISPREAD+QFPTYPE%ISPD(JL,IJ)
      ENDDO
      SELECT CASE (QFPTYPE%ILED(IJ))
      CASE (0)
!         Scalar
        QFPTYPE%IGT1(IJ,1)=ISCA1
        ISCA1=ISCA1+INC
        QFPTYPE%IGT1(IJ,0)=ISCA0
        ISCA0=ISCA0+ISPREAD
      CASE (:-1)
!         V from (U,V)<=>(Vor,Div) : 
!         set Vor then Div for direct transforms
        QFPTYPE%IGT1(-QFPTYPE%ILED(IJ),1)=IVOR
        IVOR=IVOR+INC
        QFPTYPE%IGT1(IJ,1)=IDIV
        IDIV=IDIV+INC
!         set U then V for inverse transforms
        QFPTYPE%IGT1(-QFPTYPE%ILED(IJ),0)=IUMOM
        IUMOM=IUMOM+ISPREAD
        QFPTYPE%IGT1(IJ,0)=IVMOM
        IVMOM=IVMOM+ISPREAD
      CASE (2,3)
!         (U,V) for individual Vor or Div : set Vor then Div
        QFPTYPE%IGT1(IJ,1)=IVOR
        IVOR=IVOR+INC
        QFPTYPE%IGT1(IJ,2)=IDIV
        IDIV=IDIV+INC
!         set as scalar in inverse transforms because only 1 field selected
        QFPTYPE%IGT1(IJ,0)=ISCA0
        ISCA0=ISCA0+ISPREAD
      END SELECT
    ELSEIF(QFPTYPE%ISF(IJ) == 0) THEN 
      QFPTYPE%IGPX(IJ)=IPTRGPX
      IPTRGPX=IPTRGPX+QFPTYPE%ILEV(IJ)
    ENDIF
  ENDDO

ELSEIF (CDCONF == 'M') THEN

! Compute pointers for GAUX only ; reset pointers for GT1 & DYNFP
  QFPTYPE%IGT1(:,:)= 0
  QFPTYPE%IGPX(:)  = 0
  QFPTYPE%IGFP(:)  = 0
  DO J=1,NFPOSDYN  
    IJ=NFPTRDYN(J)
    QFPTYPE%IGPX(IJ)= IPTRGPX
    IPTRGPX         = IPTRGPX + QFPTYPE%ILEV(IJ)
  ENDDO

ELSEIF (CDCONF == 'Z' .OR. CDCONF == 'E') THEN

! Compute pointers for DYNFP ; keep pointers for GAUX ; reset pointers for GT1
! => needs consistancy with cpfpds for the time being
  QFPTYPE%IGT1(:,:)=0
  QFPTYPE%IGFP(:)=0
  DO J=1,NFPOSDYN  
    IJ=NFPTRDYN(J)
    QFPTYPE%IGFP(IJ)=IPTRGFP
    IPTRGFP=IPTRGFP+QFPTYPE%ILEV(IJ)
  ENDDO

ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CPVPOSPR',1,ZHOOK_HANDLE)
END SUBROUTINE CPVPOSPR
