SUBROUTINE CVLANISO(KST,KEND,KPROMA,LDCONV,LDGAMA,LDALFA,PGAMA,&
 & PALFA,PU,PV,PSDO)  

!**** *CVLANISO*  - CONVOLUTION/DECONVOLUTION of the vector 
!                 (sigma**2)*(1-gamma)/(1+gamma) * (cos(2*alpha),sin(2*alpha))
!                 where :
!                 sigma = standard deviation of orography    
!                 gamma = Anisotropy coefficient of topography    
!                 alpha = Direction of the principal axis of the topography

!     PURPOSE.
!     --------
!        Convolution/deconvolution of the vector 
!        (sigma**2)*(1-gamma)/(1+gamma) * (cos(2*alpha),sin(2*alpha))
!        which components can be interpolated/extrapolated in place of 
!        Anisotropy coefficient or Direction of the principal axis 
!        of the topography.

!**   INTERFACE.
!     ----------
!       *CALL* *CVLANISO*

!        EXPLICIT ARGUMENTS
!        --------------------

!        INPUT:
!         KST    : first point in row.
!         KEND   : last point in row.
!         KPROMA : length of the row.
!         LDCONV : .TRUE. <=> convolution ; .FALSE. <=> deconvolution
!         LDGAMA : .TRUE. <=> U component if convolution ; else anisotropy
!         LDALFA : .TRUE. <=> V component if convolution ; else direction
!         PSDO   : standart deviation of orography times g

!        INPUT/OUTPUT:
!         PGAMA  : Anisotropy coefficient of topography
!         PALFA  : Direction of the principal axis of the topography
!         PU,PV  : components of vector 
!                 (sigma**2)*(1-gamma)/(1+gamma) * (cos(2*alpha),sin(2*alpha))

!        IMPLICIT ARGUMENTS
!        --------------------
!          See module above

!     METHOD.
!     -------
!          See documentation.

!     EXTERNALS.
!     ----------
!      None

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        See documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 98-09-10 (from HPOS and FPINTPHY)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST, ONLY : RPI

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
LOGICAL           ,INTENT(IN)    :: LDCONV 
LOGICAL           ,INTENT(IN)    :: LDGAMA 
LOGICAL           ,INTENT(IN)    :: LDALFA 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGAMA(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALFA(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSDO(KPROMA) 
!     ZMOD : (sigma**2)*(1-gamma)/(1+gamma)
!     ZSDO2: PSDO**2

REAL(KIND=JPRB) :: ZMOD(KPROMA), ZSDO2(KPROMA)

INTEGER(KIND=JPIM) :: JI

LOGICAL :: LLU, LLV

REAL(KIND=JPRB) :: ZEPS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CVLANISO',0,ZHOOK_HANDLE)
IF (LDCONV) THEN

!*       1. CONVOLUTION
!           -----------

  DO JI=KST,KEND
    ZSDO2(JI)=PSDO(JI)**2
  ENDDO
  LLU=LDGAMA
  LLV=LDALFA

!       first component of vector 
  IF (LLU) THEN
    DO JI=KST,KEND
      PU(JI)=ZSDO2(JI)*((1.0_JPRB-PGAMA(JI))/(1.0_JPRB+PGAMA(JI)))*&
       & ( COS(PALFA(JI))**2 - SIN(PALFA(JI))**2 )  
    ENDDO
  ENDIF

!       Second component of vector 
  IF (LLV) THEN
    DO JI=KST,KEND
      PV(JI)=ZSDO2(JI)*((1.0_JPRB-PGAMA(JI))/(1.0_JPRB+PGAMA(JI)))*&
       & ( 2.0_JPRB*SIN(PALFA(JI))*COS(PALFA(JI)) )  
    ENDDO
  ENDIF

ELSE

!*       2. DECONVOLUTION
!           -------------

  ZEPS=EPSILON(1.0_JPRB)*10000._JPRB
  DO JI=KST,KEND
    ZMOD(JI)=SQRT( PU(JI)**2 + PV(JI)**2 )
  ENDDO

!       Anisotropy of orography :
  IF (LDGAMA) THEN
!         When the tensor is isotropic, the anisotropy is 1.
    PGAMA(KST:KEND)=1.0_JPRB
    ZSDO2(KST:KEND)=PSDO(KST:KEND)*PSDO(KST:KEND)
    DO JI=KST,KEND
      IF (ZMOD(JI) >= ZEPS) THEN
        PGAMA(JI)=MAX(0.0_JPRB, (ZSDO2(JI)-ZMOD(JI))&
         & /MAX(ZEPS,ZSDO2(JI)+ZMOD(JI)))  
      ENDIF
    ENDDO
  ENDIF

!       Direction of orography :
  IF (LDALFA) THEN
!         When the tensor is isotropic, the direction is 0.
    PALFA(KST:KEND)=0.0_JPRB
    DO JI=KST,KEND
      IF (ZMOD(JI) >= ZEPS) THEN
        IF (ABS(PU(JI)) <= ZEPS) THEN
          IF (PV(JI) > 0) THEN
            PALFA(JI)=RPI*0.25_JPRB
          ELSE
            PALFA(JI)=-RPI*0.25_JPRB
          ENDIF
        ELSE
          IF (PU(JI) > 0) THEN
            PALFA(JI)=0.5_JPRB*ATAN(PV(JI)/PU(JI))
          ELSEIF (PV(JI) > 0) THEN
            PALFA(JI)=0.5_JPRB*(RPI+ATAN(PV(JI)/PU(JI)))
          ELSE
            PALFA(JI)=0.5_JPRB*(ATAN(PV(JI)/PU(JI))-RPI)
          ENDIF
        ENDIF
      ENDIF
    ENDDO
!         Set final direction between -pi/2 and pi/2 :
    DO JI=KST,KEND
      PALFA(JI)=ATAN(TAN(PALFA(JI)))
    ENDDO
  ENDIF

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CVLANISO',1,ZHOOK_HANDLE)
END SUBROUTINE CVLANISO
