SUBROUTINE ENDVPOS(CDCONF)

!**** *ENDVPOS*  - END OF VERTICAL POST-PROCESSING

!     PURPOSE.
!     --------
!        TRANSFER OR COMPUTE POST-PROCESSED FIELDS ON HEIGT LEVELS OR ETA
!        LEVELS FROM THE HORIZONTALLY INTERPOLATED MODEL FIELDS

!**   INTERFACE.
!     ----------
!       *CALL* *ENDVPOS*

!        EXPLICIT ARGUMENTS
!        --------------------
!     CDCONF    : configuration of work

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION
!        pre-processed fields are the horizontally interpolated model fields

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified by K. YESSAD (MARCH 1997):
!          - parallelisation of FULL-POS.
!        97-06-02 R.El Khatib 3 NH fields
!        R. El Khatib : 98-10-07 Simplified scanning
!        D.Dent       : 99-05-06 Support for log surface pressure
!    R. El Khatib : 00-07-31 Now reads horizontally interpolated model fields
!        R. El Khatib : 01-05-04 HUn, HUx
!        R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-08-13 gfl
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!      R. Engelen      15-Mar-2005 Greenhouse Gases
!        A. Untch      15-Mar-2005 Aerosols
!        J. Flemming 11-Apr-2005 Aerosol replaced with GEMS GRG
!    Y. Bouteloup      28-Jan-2005 Introduction of precipitation
!    G. Hello          28-Jan-2005 Snow, graupel and rain too
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG
USE YOMFPDIM , ONLY : NFPDYNB  ,NAFPOS
USE YOMFPSC2 , ONLY : NFPBLOCS ,NFPROMA  ,NFPSTART ,NFPEND
USE PTRFPB2  , ONLY : NFPVT0   ,MAFPTR
USE YOMRFPDS , ONLY : NRFPOS   ,MFPGMO   ,MOFPTR
USE YOMAFPB  , ONLY : AFPBUF   ,NSTAAFPB
USE YOMDFPB  , ONLY : HFPBUF   ,NSTAHFPB
USE YOMRFPB  , ONLY : RFPBUF   ,NSTARFPB
USE YOMPFPB  , ONLY : GFPBUF   ,NSTAGFPB
USE YOMFPC   , ONLY : NFPMASK
USE YOMVPOS  , ONLY : NXFPLEV  ,XFPLEV   ,QFPTYPE
USE YOMAFN   , ONLY :    TFP_VW   ,TFP_WWS  ,TFP_LNSP ,TFP_FIS  , &
 & TFP_U    ,TFP_V    ,TFP_T    ,TFP_Q    ,TFP_L    ,TFP_I      , &
 & TFP_CLF  ,TFP_O3MX ,TFP_AERO ,TFP_GHG  ,TFP_GRG  ,TFP_GFL    , &
 & TFP_PD   ,TFP_SN   ,TFP_RR   , &
 & TFP_GR   ,TFP_HL   ,TFP_TKE  ,GFP_ST   ,GFP_RDST , &
 & GFP_INCTX,GFP_INCTN,GFP_SD   ,GFP_SSW  ,GFP_FSSW ,GFP_HV   , &
 & GFP_IDZ0 ,GFP_X2T  ,GFP_X2RH ,GFP_ITZ0 ,GFP_INCHX,GFP_INCHN, &
 & GFP_GFIS ,GFP_LSM  ,GFP_VEG  ,GFP_IVEG ,GFP_ARG  ,GFP_D2   , &
 & GFP_SAB  ,GFP_LAN  ,GFP_SST

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 

!     ==== CONTROL VARIABLES ===

!     IDFP    : pointer of GDFP (output data)

REAL(KIND=JPRB) :: ZDFP(NFPROMA,NFPDYNB)  ! post-processed fields
REAL(KIND=JPRB) :: ZGT2(NFPROMA,0:NAFPOS) ! pre-processed fields
REAL(KIND=JPRB) :: ZCLI(NFPROMA,0:NRFPOS) ! climatology fields
REAL(KIND=JPRB) :: ZAUX(NFPROMA,0:NFPVT0) ! auxilary physical fields

! NB : (.,0) = adress of non-requested fields

!     ==== WORKING ARRAYS ===

! pre-processed surface pressure
REAL(KIND=JPRB) :: ZPST2(NFPROMA)
! pre-processed half levels true vertical velocity
REAL(KIND=JPRB) :: ZWHT2(NFPROMA,0:NFLEVG)
! Mixed surface temperature : output and interpolated
REAL(KIND=JPRB) :: ZTS(NFPROMA)

!     LOCAL &INTEGER SCALARS

INTEGER(KIND=JPIM) :: IUT2   ! U
INTEGER(KIND=JPIM) :: IVT2   ! V
INTEGER(KIND=JPIM) :: ITT2   ! T
INTEGER(KIND=JPIM) :: IQT2   ! Q
INTEGER(KIND=JPIM) :: ILT2   ! Liquid water
INTEGER(KIND=JPIM) :: IIT2   ! Ice
INTEGER(KIND=JPIM) :: IAT2   ! Cloud fraction
INTEGER(KIND=JPIM) :: IO3T2  ! Ozone mixing ratio
INTEGER(KIND=JPIM) :: IGHGT2 ! Greenhouse gases => pointer on the first one
INTEGER(KIND=JPIM) :: IGRGT2 ! GEMS GRG  => pointer on the first one
INTEGER(KIND=JPIM) :: IAEROT2 ! Aerosols => pointer on the first one
INTEGER(KIND=JPIM) :: ISNT2  ! Snow
INTEGER(KIND=JPIM) :: IRRT2  ! Rain
INTEGER(KIND=JPIM) :: IGRT2  ! Graupel
INTEGER(KIND=JPIM) :: IHLT2  ! Hail
INTEGER(KIND=JPIM) :: ITKET2 ! Turbulent kinetic energy
INTEGER(KIND=JPIM) :: IWVE2  ! True vertical velocity ON UPPER HALF LEVELS
INTEGER(KIND=JPIM) :: IPDE2  ! NH Pressure departure
INTEGER(KIND=JPIM) :: ISVT2  ! Passive scalars => pointer on the first one
INTEGER(KIND=JPIM) :: ILNPT2 ! Ln(Ps)
INTEGER(KIND=JPIM) :: IFIST2 ! Surface geopotential (interpolated)
INTEGER(KIND=JPIM) :: IWWST2 ! True vertical velocity ON SURFACE

INTEGER(KIND=JPIM) :: IST, IEND, JBLOC, JLEV, JI

INTEGER(KIND=JPIM) :: IATS0, IATSI, IATXI, IATNI, IASD0, IASW0, IASF0, IAHV0
INTEGER(KIND=JPIM) :: IASR0, IAT2M, IAQ2M, IATR0, IAHXI, IAHNI, IASST
INTEGER(KIND=JPIM) :: ICFIS ,ICLSM ,ICVEG ,ICIVG ,ICARG ,ICD2 ,ICSAB, IALAN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "endpos.intfb.h"
#include "sc2rdgfp.intfb.h"
#include "sc2wrgfp.intfb.h"

!     ------------------------------------------------------------------

!*       0. INITIALIZE LOCAL POINTERS FOR INPUT FIELDS
!           ------------------------------------------

IF (LHOOK) CALL DR_HOOK('ENDVPOS',0,ZHOOK_HANDLE)

IATS0=MAFPTR(GFP_ST%ICOD)
IATSI=MAFPTR(GFP_RDST%ICOD)
IATXI=MAFPTR(GFP_INCTX%ICOD)
IATNI=MAFPTR(GFP_INCTN%ICOD)
IASD0=MAFPTR(GFP_SD%ICOD)
IASW0=MAFPTR(GFP_SSW%ICOD)
IASF0=MAFPTR(GFP_FSSW%ICOD)
IAHV0=MAFPTR(GFP_HV%ICOD)
IASR0=MAFPTR(GFP_IDZ0%ICOD)
IAT2M=MAFPTR(GFP_X2T%ICOD)
IAQ2M=MAFPTR(GFP_X2RH%ICOD)
IATR0=MAFPTR(GFP_ITZ0%ICOD)
IAHXI=MAFPTR(GFP_INCHX%ICOD)
IAHNI=MAFPTR(GFP_INCHN%ICOD)
IASST=MAFPTR(GFP_SST%ICOD)
IALAN=MAFPTR(GFP_LAN%ICOD)

ICFIS=MOFPTR(GFP_GFIS%ICOD)
ICLSM=MOFPTR(GFP_LSM%ICOD)
ICVEG=MOFPTR(GFP_VEG%ICOD)
ICIVG=MOFPTR(GFP_IVEG%ICOD)
ICARG=MOFPTR(GFP_ARG%ICOD)
ICD2 =MOFPTR(GFP_D2%ICOD)
ICSAB=MOFPTR(GFP_SAB%ICOD)

ILNPT2=QFPTYPE%IGPX(TFP_LNSP%ICOD)
IFIST2=QFPTYPE%IGPX(TFP_FIS%ICOD)
IWWST2=QFPTYPE%IGPX(TFP_WWS%ICOD)
IUT2  =QFPTYPE%IGPX(TFP_U%ICOD)
IVT2  =QFPTYPE%IGPX(TFP_V%ICOD)
ITT2  =QFPTYPE%IGPX(TFP_T%ICOD)
IQT2  =QFPTYPE%IGPX(TFP_Q%ICOD)
ILT2  =QFPTYPE%IGPX(TFP_L%ICOD)
IIT2  =QFPTYPE%IGPX(TFP_I%ICOD)
IAT2  =QFPTYPE%IGPX(TFP_CLF%ICOD)
IO3T2 =QFPTYPE%IGPX(TFP_O3MX%ICOD)
IGHGT2=QFPTYPE%IGPX(TFP_GHG(1)%ICOD)
IGRGT2 =QFPTYPE%IGPX(TFP_GRG(1)%ICOD)
IAEROT2 =QFPTYPE%IGPX(TFP_AERO(1)%ICOD)
ISNT2 =QFPTYPE%IGPX(TFP_SN%ICOD)
IRRT2 =QFPTYPE%IGPX(TFP_RR%ICOD)
IGRT2 =QFPTYPE%IGPX(TFP_GR%ICOD)
IHLT2 =QFPTYPE%IGPX(TFP_HL%ICOD)
ITKET2=QFPTYPE%IGPX(TFP_TKE%ICOD)
ISVT2 =QFPTYPE%IGPX(TFP_GFL(1)%ICOD)
IPDE2 =QFPTYPE%IGPX(TFP_PD%ICOD)
IWVE2 =QFPTYPE%IGPX(TFP_VW%ICOD)

!*       1. LOOP ON SUBROWS OF EACH PP. PACKET
!           ----------------------------------

DO JBLOC=1,NFPBLOCS

  IST =NFPSTART(JBLOC)
  IEND=NFPEND(JBLOC)

!        1.1 READ INPUT DATA

!        1.1.1 Horizontally interpolated model fields

  CALL SC2RDGFP(NFPROMA,NAFPOS,AFPBUF,NSTAAFPB(JBLOC),IST,IEND,ZGT2(1,1))

!        1.1.2 Climatology & geometry

  IF (NRFPOS > 0) THEN
    CALL SC2RDGFP(NFPROMA,NRFPOS,RFPBUF,NSTARFPB(JBLOC),IST,IEND,ZCLI(1,1))
  ENDIF

!        1.1.3 Auxilary physical fields

  IF (NFPVT0 > 0) THEN
    CALL SC2RDGFP(NFPROMA,NFPVT0,GFPBUF,NSTAGFPB(JBLOC),IST,IEND,&
     & ZAUX(1,1))  
  ENDIF

!        1.3 COMPUTE OUTPUT FIELDS

  IF (ILNPT2 > 0) THEN
    DO JI=IST,IEND
      ZPST2(JI)=EXP(ZGT2(JI,ILNPT2))
    ENDDO
  ENDIF
  IF (IWVE2 > 0) THEN
    DO JLEV=0,NFLEVG-1
      DO JI=IST,IEND
        ZWHT2(JI,JLEV)=ZGT2(JI,IWVE2+JLEV)
      ENDDO
    ENDDO
  ENDIF
  IF (IWWST2 > 0) THEN
    DO JI=IST,IEND
      ZWHT2(JI,NFLEVG)=ZGT2(JI,IWWST2)
    ENDDO
  ENDIF
  IF (NFPMASK <= 1) THEN
    DO JI=IST,IEND
      ZTS(JI)=ZAUX(JI,IATS0)
    ENDDO
  ELSE
!   ZAUX(.,IATSI) is already a combination of interpolated Ts and SST
    DO JI=IST,IEND
      ZTS(JI)=ZAUX(JI,IALAN)*ZAUX(JI,IATS0) &
       & +(1._JPRB-ZAUX(JI,IALAN))*ZAUX(JI,IASST)
    ENDDO
  ENDIF  
  ZDFP(:,:) = 0.0_JPRB
  CALL ENDPOS(NFPROMA,IST,IEND,NFPDYNB,CDCONF,NXFPLEV,XFPLEV, &
   & ZGT2(1,IUT2) ,ZGT2(1,IVT2)  ,ZGT2(1,ITT2) ,ZGT2(1,IQT2) ,ZGT2(1,ILT2) , &
   & ZGT2(1,IIT2) ,ZGT2(1,ISVT2),ZGT2(1,IAT2) ,ZGT2(1,IO3T2),ZGT2(1,IGHGT2), &
   & ZGT2(1,IGRGT2),ZGT2(1,IAEROT2),&
   & ZGT2(1,ISNT2),ZGT2(1,IRRT2) ,ZGT2(1,IGRT2),ZGT2(1,IHLT2),ZGT2(1,ITKET2),&
   & ZGT2(1,IPDE2),&
   & ZWHT2        ,ZGT2(1,IFIST2),ZPST2, &
   & ZTS          ,ZAUX(1,IATSI),ZAUX(1,IASD0),ZAUX(1,IATXI),ZAUX(1,IATNI), &
   & ZAUX(1,IASW0),ZAUX(1,IASF0),ZAUX(1,IAHV0),ZAUX(1,IASR0),ZAUX(1,IAT2M), &
   & ZAUX(1,IAQ2M),ZAUX(1,IATR0),ZAUX(1,IAHXI),ZAUX(1,IAHNI), &
   & ZCLI(1,ICFIS),ZCLI(1,ICLSM),ZCLI(1,ICVEG),ZCLI(1,ICIVG),ZCLI(1,ICARG), &
   & ZCLI(1,ICD2) ,ZCLI(1,ICSAB),ZCLI(1,MFPGMO), &
   & ZDFP)  

!        1.4 WRITE OUT OUTPUT FIELDS IN ARRAY/WORKFILE

  CALL SC2WRGFP(NFPROMA,NFPDYNB,HFPBUF,NSTAHFPB(JBLOC),IST,IEND,ZDFP)

ENDDO

IF (LHOOK) CALL DR_HOOK('ENDVPOS',1,ZHOOK_HANDLE)
END SUBROUTINE ENDVPOS
