SUBROUTINE FPACHMT(KPROMA,KSTART,KPROF,KFLEV,PFISO,PTT0,PQT0,PIT0,PLT0,PRRT0,&
 & PSNT0,PGRT0,PUT0,PVT0,PSPT0,PLSM,PTS,PGZ0F,PGZ0H,PVEG0,PARG,PD2,PIVEG,    &
 & PSAB,PSNS,PWS,PWSI,PHV,PQCLS,PRHCLS,PTCLS,PUCLS,PVCLS,PFCLS,PUGST,PVGST,  &
 & PFGST,PRPCLS,PRESH,PRESF,PLNPR,PALPH)  

!     PURPOSE.
!     --------
!        Full-POS interface to ACHMT, to compute pressure, humidity,
!        temperature and wind

!**   INTERFACE.
!     ----------
!       *CALL* *FPACHMT*

!        EXPLICIT ARGUMENTS
!        --------------------
!               INPUT :
!               ------
!        KPROMA    : horizontal dimension
!        KSTART    : start of work
!        KPROF     : depth of work
!        KFLEV     : numer of full levels
!        PFISO     : surface geopotential
!        PTT0      : temperature
!        PQT0      : specific moisture
!        PIT0      : ice
!        PLT0      : liquid water
!        PRRT0     : rain
!        PSNT0     : snow
!        PGRT0     : graupel
!        PUT0      : geographical wind - U momentum
!        PVT0      : geographical wind - V momentum
!        PSPT0     : surface pressure
!        PLSM      : land/sea mask
!        PTS       : surface temperature
!        PGZ0F     : gravity * roughness length
!        PGZ0H     : gravity * thermal roughness length
!        PVEG0     : fractional cover by vegetation
!        PARG      : silt percentage
!        PD2       : soil depth
!        PIVEG     : type of vegetation
!        PSAB      : sand percentage 
!        PSNS      : snow cover per unit surface
!        PWS       : surface water content
!        PWSI      : surface layer ice content
!        PHV       : resistance to evapotranspiration

!               OUTPUT:
!               ------
!        PQCLS     : specific moisture at 2m
!        PRHCLS    : relative moisture at 2m
!        PTCLS     : temperature at 2m
!        PUCLS     : U-component of wind at 10m
!        PVCLS     : V-component of wind at 10m
!        PFCLS     : Wind velocity of wind at 10m
!        PUGST     : U-component of gusts
!        PVGST     : V-component of gusts
!        PFGST     : gusts
!        PRPCLS    : pressure

!               OPTIONAL INPUT :
!               ----------------
!        PRESH     : half level hydrostatic pressure 
!        PRESF     : full level hydrostatic pressure 
!        PLNPR     : "delta" (see in GPXYB)
!        PALPH     : "alpha" (see in GPXYB)

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE* & WITOLD OWCARZ

!     MODIFICATIONS.
!     --------------
!        ORIGINAL   : 2000-03-08 by Witold Owcarz
!      R. El Khatib : 2000-10-05 Wind velocity instead of pressure in output
!      R. El Khatib : 2001-01-05 Gusts
!      J.M. Piriou / J.F Geleyn  : 2001-08-23 New definition and use of the
!                      thermal mixing length.
!      M. Bellus    : 2003-04-24 Change of arguments in ACHMT calling
!                                + removing LSOLV and some cleaning
!      M. Tudor  : 2003-10-21 Change args in ACHMT, reintroduce LSOLV
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!      A. Alias  : 2007-10-11 New Output in ACHMT (ZRTI=1/R*T)
!      C. Payan  : 2008/11 Neutral Wind Diagnostic (ACHMT)
!     K. Yessad (Dec 2008): remove dummy CDLOCK + cleanings
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMGEM   , ONLY : VBH      ,VAH      ,VC       ,VDELB
USE YOMCST   , ONLY : RG
USE YOMPHY   , ONLY : LSOLV    ,LSNV
USE YOMPHY2  , ONLY : HVCLS    ,HTCLS    ,LRAFTUR
USE SURFACE_FIELDS_MIX, ONLY : YSD_VVD

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFISO(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRRT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGRT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVT0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPT0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0F(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0H(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEG0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PARG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PD2(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIVEG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSAB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSNS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWSI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHV(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRHCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUGST(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVGST(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFGST(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRPCLS(KPROMA) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PRESH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PRESF(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PLNPR(KPROMA,KFLEV) 
REAL(KIND=JPRB),OPTIONAL,INTENT(IN) :: PALPH(KPROMA,KFLEV) 

!-----------------------------------------------------------------------

!     ZPRESH, ZPRESF : HALF LEVEL, FULL LEVEL PRESSURES OF MODEL LEVELS
!     ZGEOPH, ZGEOPF : HALF LEVEL, FULL LEVEL GEOPOTENTIALS
!     ZPXP : HALF,FULL AND LN HALF,FULL LEVEL GEOPOTENTIALS
!     ZPXPD: 1./D(Z) AND 1./D(LN(Z))
!     ZPREHA, ZPREFU : HALF LEVEL, FULL LEVEL PRESSURES OF OUTPUT ETA SYSTEM
!     ZPUCLN : U-component of neutral wind at 10m
!     ZPVCLN : V-component of neutral wind at 10m

REAL(KIND=JPRB) :: ZGZ0RLF(KPROMA), ZGZ0(KPROMA), ZGZ0H(KPROMA)
REAL(KIND=JPRB) :: ZCD(KPROMA), ZCDN(KPROMA), ZCH(KPROMA), ZCS(KPROMA)
REAL(KIND=JPRB) :: ZNEIJJ(KPROMA), ZVEG(KPROMA)
REAL(KIND=JPRB) :: ZRS(KPROMA), ZQS(KPROMA), ZQSATS(KPROMA)
REAL(KIND=JPRB) :: ZNBVNO(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZFPLSH(KPROMA,0:KFLEV), ZFPLCH(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZXDROV(KPROMA), ZXHROV(KPROMA)
REAL(KIND=JPRB) :: ZDPHIV(KPROMA), ZDPHIT(KPROMA)
REAL(KIND=JPRB) :: ZNEIJG(KPROMA), ZNEIJV(KPROMA), ZWFC(KPROMA)
REAL(KIND=JPRB) :: ZCDROV(KPROMA),ZCHROV(KPROMA),ZDQSTS(KPROMA),ZGWDCS(KPROMA)
REAL(KIND=JPRB) :: ZHQ(KPROMA),ZHU(KPROMA), ZSTAB(KPROMA)
REAL(KIND=JPRB) :: ZRTI(KPROMA)

REAL(KIND=JPRB) :: ZWPMX(KPROMA), ZWSMX(KPROMA), ZWWILT(KPROMA), ZWSAT(KPROMA)

REAL(KIND=JPRB) :: ZR0   (KPROMA,KFLEV), ZCP0  (KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZDELP (KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZRDELP(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZLNPR (KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZALPH (KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZRTGR (KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZRPRE (KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZRPP  (KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZKAP0 (KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZGEOPH(KPROMA,0:KFLEV), ZGEOPF(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZPRESH(KPROMA,0:KFLEV), ZPRESF(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZPUCLN(KPROMA), ZPVCLN(KPROMA) 

LOGICAL :: LLCLS, LLHMT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

#include "abor1.intfb.h"
#include "achmt.intfb.h"
#include "acsolw.intfb.h"
#include "gpgeo.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gpxyb.intfb.h"

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPACHMT',0,ZHOOK_HANDLE)

!-----------------------------------------------------------------------

!*       0.    RELIABITY OF INPUT ARGUMENTS
!              ----------------------------

IF (LSNV) THEN
  CALL ABOR1('FPACHMT : NOT READY FOR LSNV - ZNEIJG,ZNEIJV')
ENDIF

!-----------------------------------------------------------------------

!*       1.    COMPUTATION OF Cp, R AND R/Cp FROM Q
!              ------------------------------------
CALL GPRCP(KPROMA,KSTART,KPROF,KFLEV,PQ=PQT0,PQI=PIT0,PQL=PLT0,&
 & PQR=PRRT0,PQS=PSNT0,PQG=PGRT0,PCP=ZCP0,PR=ZR0,PKAP=ZKAP0)

!*       2.    COMPUTATION OF MODEL LEVELS PRESSURES
!              -------------------------------------

IF (PRESENT(PRESH).AND.PRESENT(PRESF).AND.PRESENT(PLNPR).AND.PRESENT(PALPH)) &
 & THEN
  ZPRESH(KSTART:KPROF,0:KFLEV)=PRESH(KSTART:KPROF,0:KFLEV)
  ZPRESF(KSTART:KPROF,1:KFLEV)=PRESF(KSTART:KPROF,1:KFLEV)
  ZLNPR(KSTART:KPROF,1:KFLEV)=PLNPR(KSTART:KPROF,1:KFLEV)
  ZALPH(KSTART:KPROF,1:KFLEV)=PALPH(KSTART:KPROF,1:KFLEV)
ELSE
  ! ky: these calculations are actually not useful because FPACHMT is always
  ! called from routines where these quantities have been already computed.
  ZPRESH(KSTART:KPROF,KFLEV)=PSPT0(KSTART:KPROF)
  CALL GPPREH(KPROMA,KSTART,KPROF,KFLEV,VAH,VBH,ZPRESH)
  CALL GPXYB(KPROMA,KSTART,KPROF,KFLEV,VDELB,VC,ZPRESH,ZDELP,&
   & ZRDELP,ZLNPR,ZALPH,ZRTGR,ZRPRE,ZRPP)  
  CALL GPPREF(KPROMA,KSTART,KPROF,KFLEV,ZALPH,ZPRESH,ZPRESF)
ENDIF

!*       3.    COMPUTATION OF MODEL LEVELS GEOPOTENTIALS
!              -----------------------------------------

ZGEOPH(KSTART:KPROF,KFLEV)=PFISO(KSTART:KPROF)
CALL GPGEO(KPROMA,KSTART,KPROF,KFLEV,ZGEOPH,ZGEOPF,PTT0,ZR0,ZLNPR,ZALPH)

!        4.    COMPUTATION OF T, Q, U AND V
!              ----------------------------

LLHMT=.TRUE.

IF (LSOLV) THEN
  CALL ACSOLW (KSTART,KPROF,KPROMA,PARG,PD2,PLSM,PIVEG,PSAB,LLHMT,ZWFC,ZWPMX,&
   & ZWSAT,ZWSMX,ZWWILT,'ACSOLW ')  
ENDIF

ZDPHIV(KSTART:KPROF)=RG*HVCLS
ZDPHIT(KSTART:KPROF)=RG*HTCLS
LLCLS=.FALSE.
 
CALL ACHMT(KSTART,KPROF,KPROMA,1,KFLEV,YSD_VVD%NUMFLDS,&
 & ZGEOPH,ZGEOPF,ZPRESH,ZPRESF,ZCP0,PQT0,ZR0,PTT0,PUT0,PVT0,&
 & ZFPLSH,ZFPLCH,ZDPHIT,ZDPHIV,PGZ0F, PGZ0H, ZGZ0RLF, PHV, PLSM,&
 & ZNEIJG,ZNEIJV, PSNS, PTS, PVEG0, ZWFC, PWS, PWSI,&
 & LLCLS, LLHMT,&
 & ZNBVNO,&
 & ZCD, ZCDN, ZCDROV, ZCH, ZCHROV, ZCS, ZDQSTS, ZGWDCS,&
 & ZGZ0, ZGZ0H, ZHQ, ZHU, ZNEIJJ, PQCLS, ZQS, ZQSATS, PRHCLS,&
 & ZRS, ZRTI, ZSTAB, PTCLS, PUCLS, PVCLS, ZPUCLN, ZPVCLN, PRPCLS, ZVEG,&
 & ZXDROV, ZXHROV, PUGST, PVGST, 'ACHMT  ')  

IF (.NOT.LRAFTUR) THEN 
  CALL ABOR1('FPACHMT:CALL TO ACCLPH BROKEN\MH')
  ! CALL ACCLPH (KSTART,KPROF,KPROMA,1,KFLEV,ZGEOPH,ZGEOPF,ZR0,PTT0,PUT0,PVT0,&
  !  &ZRS,PTS,ICLPH,ZCLPH,PUGST,PVGST,'ACCLPH ')
  ! else they are computed inside ACHMT
ENDIF

PFCLS(KSTART:KPROF)=SQRT((PUCLS(KSTART:KPROF)**2)+(PVCLS(KSTART:KPROF)**2))
PRHCLS(KSTART:KPROF)=MAX(0.0_JPRB,MIN(1.0_JPRB,PRHCLS(KSTART:KPROF)))
PFGST(KSTART:KPROF)=SQRT((PUGST(KSTART:KPROF)**2)+(PVGST(KSTART:KPROF)**2))

!-------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('FPACHMT',1,ZHOOK_HANDLE)
END SUBROUTINE FPACHMT
