SUBROUTINE FPCICA(KST,KEND,KPROMA,KLEV,KCAPETYPE,PTCLS,PRPCLS,PRHCLS,PT,PRP,&
 & PQV,PCAPE,PCIN,KLCL,KFCL,KEL)  

! --------------------------------------------------------------
! **** *FPCICA* COMPUTE CAPE AND CIN.
! --------------------------------------------------------------
! SUBJECT:
!    ROUTINE COMPUTING CAPE AND CIN FOR SELECTED "TYPE" (PARCEL)  

! INTERFACE:
!    *CALL* *FPCICA*

! --------------------------------------------------------------
! -   INPUT ARGUMENTS
!     ---------------

! - DIMENSIONING

! KST      : FIRST INDEX OF LOOPS
! KEND     : LAST INDEX OF LOOPS
! KPROMA   : DEPTH OF THE VECTORIZATION ARRAYS
! KLEV     : END OF VERTICAL LOOP AND VERTICAL DIMENSION

! - VARIABLES
! KCAPETYPE: TYPE OF CAPE COMPUTATION
! PTCLS    : CLS TEMPERATURE (K)
! PRPCLS   : CLS PRESSURE (PA)
! PRHCLS   : CLS RELATIVE HUMIDITY (NO DIM)
! PT       : TEMPERATURE (K)
! PRP      : PRESSURE (PA)
! PQV      : WATER VAPOUR SPECIFIC HUMIDITY (NO DIM)

! --------------------------------------------------------------
! -   OUTPUT ARGUMENTS
!     ---------------
! - VARIABLES
! PCAPE    : CAPE - CONVECTIVE AVAILABLE POTENTIAL ENERGY (J/KG)
!                   (POTENTIALLY AVAILABLE CONVECTIVE KINETIC ENERGY)
! PCIN     : CIN - CONVECTIVE INHIBITION (J/KG)

! --------------------------------------------------------------
! -   IMPLICITE ARGUMENTS
!     -------------------
! YOMCAPE
! --------------------------------------------------------------
! EXTERNALS:

! METHOD:
!   different types of CAPE and CIN are calculated:
!    - CAPE for the parcel at the lowest model level
!    - CAPE for the most unstable parcel
!            CAPEs are calculated for the parcels released from
!            all model levels in the layer (depth=GCAPEPSD) above
!            the ground.
!            The highest value of CAPE is taken.
!    - CAPE for the CLS parcel 

! AUTEUR/AUTHOR:   2001-03, N. PRISTOV

! MODIFICATIONS:
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        P. Marquet    16-Jan-2009 (CY33t1) : add the option
!                     (KCAPETYPE=5) = the merge of the options
!                      2 and 3. Correction for the computations
!                      of KLCL,KFCL,KEL for the options 2 and 4,
!                      where the value for the "maximum CAPE"
!                      ought to be retained (level=IMAX(1)).
!                  !! (KCAPETYPE=4) is equivalent to KCAPETYPE=3
!                      : see "endpos" or "phymfpos" => use of 
!                      KCAPETYPE=5 !!
! --------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : LNEIGE
USE YOMCST   , ONLY : RV       ,RCPV     ,RETV     ,&
 & RCW      ,RCS      ,RLVTT    ,RLSTT    ,RTT      ,&
 & RALPW    ,RBETW    ,RGAMW    ,RALPS    ,RBETS    ,&
 & RGAMS    ,RALPD    ,RBETD    ,RGAMD  
USE YOMCAPE  , ONLY : GCAPEPSD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCAPETYPE 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRP(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQV(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCAPE(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCIN(KPROMA) 
INTEGER(KIND=JPIM)  ,INTENT(OUT)   :: KLCL(KPROMA)
INTEGER(KIND=JPIM)  ,INTENT(OUT)   :: KFCL(KPROMA)
INTEGER(KIND=JPIM)  ,INTENT(OUT)   :: KEL(KPROMA)

INTEGER(KIND=JPIM):: JLON,JLEV,ILEV,IMAX(1),IL(KPROMA)

REAL(KIND=JPRB):: ZDELTA, ZEW
REAL(KIND=JPRB):: ZT(KPROMA,KLEV+1)
REAL(KIND=JPRB):: ZP(KPROMA,KLEV+1)
REAL(KIND=JPRB):: ZQV(KPROMA,KLEV+1)
REAL(KIND=JPRB):: ZCAPE(KPROMA,KLEV),ZCIN(KPROMA,KLEV)

REAL   (KIND=JPRB) :: Z2CAPE(KPROMA),    Z2CIN(KPROMA)
INTEGER(KIND=JPIM) :: ILCL(KPROMA,KLEV), I2LCL(KPROMA)
INTEGER(KIND=JPIM) :: IFCL(KPROMA,KLEV), I2FCL(KPROMA)
INTEGER(KIND=JPIM) :: IEL (KPROMA,KLEV), I2EL (KPROMA)
INTEGER(KIND=JPIM) :: IMX (KPROMA)
REAL   (KIND=JPRB) :: ZIND2(KPROMA)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fpcincape.intfb.h"
!---------

#include "fcttrm.h"

!-------------------------------------------------
! INITIALIZE TO ZERO.
!-------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPCICA',0,ZHOOK_HANDLE)
PCAPE(KST:KEND)=0.0_JPRB
PCIN (KST:KEND)=0.0_JPRB
ZCAPE(KST:KEND,1:KLEV)=0.0_JPRB
ZCIN (KST:KEND,1:KLEV)=0.0_JPRB
Z2CAPE(KST:KEND)=0.0_JPRB
Z2CIN (KST:KEND)=0.0_JPRB

!========================
 IF (KCAPETYPE == 1) THEN
!========================

!    - 1) CAPE for the parcel at the lowest model level

CALL FPCINCAPE(KST,KEND,KPROMA,KLEV,KLEV,PT,PRP,PQV,PCAPE,PCIN,KLCL,KFCL,KEL)

!============================
 ELSEIF (KCAPETYPE == 2) THEN
!============================

!    - 2) CAPE for the most unstable parcel

! for how many levels CAPE should be computed
  ILEV=1
  IL=1
  DO JLEV=KLEV,1,-1
    DO JLON=KST,KEND
      IL(JLON)=MAX(IL(JLON),JLEV*SIGN(1,-NINT(GCAPEPSD-PRP(JLON,KLEV)&
       & +PRP(JLON,JLEV))))  
    ENDDO
  ENDDO
   DO JLON=1,KPROMA
      IF (IL(JLON)==1) IL(JLON)=9999
   ENDDO
  ILEV=MINVAL(IL)+1
  DO JLEV=KLEV,ILEV,-1
    CALL FPCINCAPE(KST,KEND,KPROMA,KLEV,JLEV,PT,PRP,PQV,&
     & ZCAPE(:,JLEV),ZCIN(:,JLEV),&
     & ILCL(:,JLEV),IFCL(:,JLEV),IEL(:,JLEV))  
    DO JLON=KST,KEND
      ZCAPE(JLON,JLEV)=ZCAPE(JLON,JLEV)*MAX(0.0_JPRB,&
       & SIGN(1.0_JPRB,GCAPEPSD-PRP(JLON,KLEV)+PRP(JLON,JLEV)))  
    ENDDO
  ENDDO
  DO JLON=KST,KEND
    IMAX=MAXLOC(ZCAPE(JLON,ILEV:KLEV))+ILEV-1
    PCAPE(JLON)=ZCAPE(JLON,IMAX(1))
    PCIN (JLON)=ZCIN (JLON,IMAX(1))
    KLCL (JLON)=ILCL (JLON,IMAX(1))
    KFCL (JLON)=IFCL (JLON,IMAX(1))
    KEL  (JLON)=IEL  (JLON,IMAX(1))
  ENDDO

!============================
 ELSEIF (KCAPETYPE == 3) THEN
!============================

!    - 3) CAPE for the CLS parcel 

  ILEV=KLEV+1
  DO JLON=KST,KEND
    ZP(JLON,ILEV)=PRPCLS(JLON)
    ZT(JLON,ILEV)=PTCLS(JLON)
    IF (LNEIGE) THEN
      ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PTCLS(JLON)))
    ELSE
      ZDELTA=0.0_JPRB
    ENDIF
    ZEW= FOEW (PTCLS(JLON),ZDELTA)
    ZQV(JLON,ILEV)=ZEW*PRHCLS(JLON) &
     & /((RETV+1.0_JPRB)*PRPCLS(JLON)-RETV*ZEW*PRHCLS(JLON))  
  ENDDO
  DO JLEV=1,KLEV
    DO JLON=KST,KEND
      ZP(JLON,JLEV)=PRP(JLON,JLEV)
      ZT(JLON,JLEV)=PT(JLON,JLEV)
      ZQV(JLON,JLEV)=PQV(JLON,JLEV)
    ENDDO
  ENDDO
  CALL FPCINCAPE(KST,KEND,KPROMA,ILEV,ILEV,ZT,ZP,ZQV,PCAPE,PCIN,KLCL,KFCL,KEL)

!============================
 ELSEIF (KCAPETYPE == 5) THEN
!============================

!    - 5) CAPE for the most unstable parcel among a) the CLS parcel
!         and b) the first upper air levels located within
!         the layer of depth "GCAPEPSD Pa" above the groung level.

!    - 5a) CAPE for the CLS parcel.
!          -----------------------

  ILEV=KLEV+1
  DO JLON=KST,KEND
    ZP(JLON,ILEV)=PRPCLS(JLON)
    ZT(JLON,ILEV)=PTCLS(JLON)
    IF (LNEIGE) THEN
      ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PTCLS(JLON)))
    ELSE
      ZDELTA=0.0_JPRB
    ENDIF
    ZEW= FOEW (PTCLS(JLON),ZDELTA)
    ZQV(JLON,ILEV)=ZEW*PRHCLS(JLON) &
     & /((RETV+1.0_JPRB)*PRPCLS(JLON)-RETV*ZEW*PRHCLS(JLON))  
  ENDDO
  DO JLEV=1,KLEV
    DO JLON=KST,KEND
      ZP(JLON,JLEV)=PRP(JLON,JLEV)
      ZT(JLON,JLEV)=PT(JLON,JLEV)
      ZQV(JLON,JLEV)=PQV(JLON,JLEV)
    ENDDO
  ENDDO
  CALL FPCINCAPE(KST,KEND,KPROMA,ILEV,ILEV,ZT,ZP,ZQV,&
     &           Z2CAPE,Z2CIN,I2LCL,I2FCL,I2EL)

!    - 5b) CAPE for the most unstable parcel.
!          ---------------------------------

! for how many levels CAPE should be computed
  ILEV=1
  IL=1
  DO JLEV=KLEV,1,-1
    DO JLON=KST,KEND
      IL(JLON)=MAX(IL(JLON),JLEV*SIGN(1,-NINT(GCAPEPSD-PRP(JLON,KLEV)&
       & +PRP(JLON,JLEV))))  
    ENDDO
  ENDDO
   DO JLON=1,KPROMA
      IF (IL(JLON)==1) IL(JLON)=9999
   ENDDO

! the loop for the computation of the CAPE :
  ILEV=MINVAL(IL)+1
  DO JLEV=KLEV,ILEV,-1
    CALL FPCINCAPE(KST,KEND,KPROMA,KLEV,JLEV,PT,PRP,PQV,&
     & ZCAPE(:,JLEV),ZCIN(:,JLEV),&
     & ILCL(:,JLEV),IFCL(:,JLEV),IEL(:,JLEV))  
    DO JLON=KST,KEND
      ZCAPE(JLON,JLEV)=ZCAPE(JLON,JLEV)*MAX(0.0_JPRB,&
       & SIGN(1.0_JPRB,GCAPEPSD-PRP(JLON,KLEV)+PRP(JLON,JLEV)))  
    ENDDO
  ENDDO

!    - 5c) The true maximum value for the CAPE. 
!          -----------------------------------
!          i.e. either for the CLS value (Z2CAPE) 
!          or for the Max-Upper-Air value
!          (ZCAPE(IMX(JLON))) :

  DO JLON=KST,KEND
  ! IMAX = the index array for the Maximum
  !        value among the Upper-Air data :
    IMAX=MAXLOC(ZCAPE(JLON,ILEV:KLEV))+ILEV-1
    IMX(JLON)=IMAX(1)
  ENDDO
  DO JLON=KST,KEND
  ! ZIND2=1 if the CLS value is greater
  !         than the Max-Upper-Air value : 
    ZIND2(JLON)= MAX(0.0_JPRB, SIGN(1.0_JPRB, &
     &           Z2CAPE(JLON)-ZCAPE(JLON,IMX(JLON)) &
     &               )              )
  ENDDO
  DO JLON=KST,KEND
    PCAPE(JLON) = Z2CAPE(JLON)                   *ZIND2(JLON) &
     &          + ZCAPE (JLON,IMX(JLON))*(1._JPRB-ZIND2(JLON))
    PCIN (JLON) = Z2CIN (JLON)                   *ZIND2(JLON) &
     &          + ZCIN  (JLON,IMX(JLON))*(1._JPRB-ZIND2(JLON))
    KLCL (JLON) = I2LCL (JLON)                   *ZIND2(JLON) &
     &          + ILCL  (JLON,IMX(JLON))*(1._JPRB-ZIND2(JLON))
    KFCL (JLON) = I2FCL (JLON)                   *ZIND2(JLON) &
     &          + IFCL  (JLON,IMX(JLON))*(1._JPRB-ZIND2(JLON))
    KEL  (JLON) = I2EL  (JLON)                   *ZIND2(JLON) &
     &          + IEL   (JLON,IMX(JLON))*(1._JPRB-ZIND2(JLON))
  ENDDO

! End of the test on KCAPETYPE
!=====
 ENDIF
!=====

IF (LHOOK) CALL DR_HOOK('FPCICA',1,ZHOOK_HANDLE)
END SUBROUTINE FPCICA
