SUBROUTINE FPCLIPHY(KFLD,KST,KEND,KFPROMA,KFLDCLI,PCLI,PROW,LDCLI)

!**** *FPCLIPHY*  - FULL-POS writer of the output climatology

!     PURPOSE.
!     --------
!        To write the output climatology in the post-processing buffers
!        and initialize a logical climatology mask

!**   INTERFACE.
!     ----------
!       *CALL* *FPCLIPHY*

!        EXPLICIT ARGUMENTS
!        --------------------

!        INPUT:
!         KFLD   : number of fields in output row.
!         KST    : first output point in row.
!         KEND   : last output point in row.
!         KFPROMA: length of the output row.
!         KFLDCLI: total number of fields in climatology row

!        INPUT/OUTPUT:
!         PCLI   : output climatology row

!        OUTPUT:
!         PROW   : output fields row
!         LDCLI : logical mask : .TRUE. if the climatology
!                  field has been written

!        IMPLICIT ARGUMENTS
!        --------------------
!          See module above

!     METHOD.
!     -------
!          Scan the request ; if it is a climatology field
!          and the corresponding climatology is at disposal, 
!          then fill the row and set the logical mask to .TRUE.

!     EXTERNALS.
!     ----------
!      None

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        See documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 98-09-03 (partially from FPINTPHY)
!        R. El Khatib 00-04-05 Replace hard-coded names
!        D. Giard     00-10-25 Albedos of bare ground and vegetation
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad: 31-01-2007 Cleanings + optional calcul. of PCLI,PROW,LDCLI
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PTRFP4   , ONLY : YRQGFP
USE YOMRFPDS , ONLY : MOFPTR
USE YOMAFN   , ONLY : GFP_SDOG ,GFP_ACOT ,GFP_DPAT ,GFP_PADOU,GFP_PADOV, &
 & GFP_ST   ,GFP_DST  ,GFP_SD

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDCLI 
REAL(KIND=JPRB)   ,INTENT(INOUT), OPTIONAL :: PCLI(KFPROMA,KFLDCLI) 
REAL(KIND=JPRB)   ,INTENT(OUT)  , OPTIONAL :: PROW(KFPROMA,KFLD) 
LOGICAL           ,INTENT(OUT)  , OPTIONAL :: LDCLI(KFLD) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JF, ISDOG, IACOT,IDPAT, ICOD, IPTR

LOGICAL :: LL, LLC, LLR

REAL(KIND=JPRB) :: ZU(KFPROMA), ZV(KFPROMA)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "cvlaniso.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPCLIPHY',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1. Initializations
!           ---------------

! Compute LDCLI:
IF (PRESENT(LDCLI)) THEN
  LLC=.TRUE.
ELSE
  LLC=.FALSE.
ENDIF

! Compute PROW and use/modify PCLI:
IF (PRESENT(PCLI).AND.(.NOT.PRESENT(PROW))) THEN
  CALL ABOR1(' FPCLIPHY: PCLI present => PROW should be present ')
ELSEIF (PRESENT(PROW).AND.(.NOT.PRESENT(PCLI))) THEN
  CALL ABOR1(' FPCLIPHY: PROW present => PCLI should be present ')
ELSEIF (PRESENT(PROW).AND.PRESENT(PCLI)) THEN
  LLR=.TRUE.
ELSE
  LLR=.FALSE.
ENDIF

ISDOG=MOFPTR(GFP_SDOG%ICOD)
IDPAT=MOFPTR(GFP_DPAT%ICOD)
IACOT=MOFPTR(GFP_ACOT%ICOD)

!     ------------------------------------------------------------------

!*       2. Loop on fields
!           --------------

DO JF=1,KFLD

  ICOD=YRQGFP%ICOD(JF)
  IPTR=MOFPTR(ICOD)

  IF (IPTR > 0) THEN
    ! Climatology exists for this field :
    IF (ICOD == GFP_ST%ICOD .OR. ICOD == GFP_DST%ICOD .OR. &
     & ICOD == GFP_SD%ICOD) THEN  
      ! Actually Ts, Td, Snow depth are pronostic fields
      IF (LLC) LDCLI(JF)=.FALSE.
    ELSE
      ! Climatology exists, get it :
      IF (LLR) PROW(KST:KEND,JF)=PCLI(KST:KEND,IPTR)
      IF (LLC) LDCLI(JF)=.TRUE.
    ENDIF
  ELSE
    ! No actual climatology for this field :
    IF (IACOT*IDPAT*ISDOG > 0 .AND.  &
     & (ICOD == GFP_PADOU%ICOD .OR. ICOD == GFP_PADOV%ICOD)) THEN  
      ! Climatology exists indirectly for the vector of anisotropy,
      ! defined by its circular coordinates :
      ! module = (sigma**2)*(1-gamma)/(1+gamma) ; angle  = 2*alpha
      ! where sigma = standard deviation of orography
      ! gamma = Anisotropy coefficient of topography
      ! alpha = Direction of the principal axis of the topography
      IF (LLR) THEN
        LL=.TRUE.
        CALL CVLANISO(KST,KEND,KFPROMA,LL,LL,LL,PCLI(1,IACOT),PCLI(1,IDPAT), &
         & ZU,ZV,PCLI(1,ISDOG))  
        IF (ICOD == GFP_PADOU%ICOD) THEN
          PROW(KST:KEND,JF)=ZU(KST:KEND)
        ELSEIF (ICOD == GFP_PADOV%ICOD) THEN
          PROW(KST:KEND,JF)=ZV(KST:KEND)
        ENDIF
      ENDIF
      IF (LLC) LDCLI(JF)=.TRUE.
    ELSE
      ! No climatology
      IF (LLC) LDCLI(JF)=.FALSE.
    ENDIF
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPCLIPHY',1,ZHOOK_HANDLE)
END SUBROUTINE FPCLIPHY
