SUBROUTINE FPCORPHY(KBLOC,KST,KEND,KFPROMA,KFIELDS,PROW,KAUX,PAUX,KCLI,PCLI,&
 & LDUSE)  

!**** *FPCORPHY*  - FULL-POS corrector on physical fields after
!                   horizontal interpolations

!     PURPOSE.
!     --------
!        To correct physical fields after the horizontal interpolations or
!        biperiodicisations that may create overshoots,
!        or to recover the requested fields after relative interpolations or
!        biperiodicisations.

!**   INTERFACE.
!     ----------
!       *CALL* *FPCORPHY*

!        EXPLICIT ARGUMENTS
!        --------------------

!        INPUT:
!         KST    : first output point in row
!         KEND   : last output point in row
!         KFPROMA: length of the output row
!         KFIELDS: number of fields in output row
!         KAUX   : number of fields in auxilary row
!         PAUX   : auxilary fields row
!         KCLI   : number of fields in climatology row
!         PCLI   : climatology fields row
!         LDUSE  : .TRUE. to use the auxilary buffer

!        OUTPUT:
!         PROW  : output row for physics

!        IMPLICIT ARGUMENTS
!        --------------------
!          None

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!       FPHOR12 , ACSOLW

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        See documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 98-09-07 from FPINTPHY
!        Modified : 99-03-15 by D. Giard (Wdf,Hu,gZ0,Wsf,ACSOLW)
!        R. El Khatib : hierarchic corrections to preserve from strange
!                       data from climatology row
!        1999-04-19, J.M. Piriou: Moon radiation.
!      R. El Khatib 99-11-23 Fix after extrapolation of index of vegetation
!      R. El Khatib 00-01-28 Fix : fields from aux. row are already corrected
!                            because FPSURF calls FPCORPHY 
!      J. Boutahar: 2000-03-01 new treatment of the Lakes in Fullpos
!      R. El Khatib: 2000-03-28 Remove some hard-coded names
!      Vanda Costa  2000-10-23 add the corrections for inteception content and
!                              resistance to evapotranspiration
!        Modified : 00-10-25 by D. Giard (Als,Alv)
!      R. El Khatib: 00-08-04 Fix : clim snow depth to be corrected
!                             with clim surf. temperature
!      R. El Khatib : 00-07-24 Fix on Tx/Tn
!      R. El Khatib : 00-08-09 Search in aux row first.
!      R. El Khatib : 01-05-04 Duration of total precipitations,HUn,HUx
!      R. El Khatib : 01-06-14 LFPLAKE => NFPLAKE
!      R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!    R. El Khatib : 03-04-23 Options ICOR reorganized
!    R. El Khatib : 03-05-26 Fix for overshoot on soil wetness prop. (e923)
!    R. El Khatib : 03-08-04 Fix : .NOT.LFPOSHOR => IANO=0 (see HPOS)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!      P, Viterbo :  24-05-2004   Change surface units
!    R. El Khatib :  17-May-2005 SST for Meteo-France
!    Y. Seity :  06-March-2007 add Graupel and Hail precipitations 
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMAFN   , ONLY : GFP_LSM  ,GFP_ST   ,GFP_DST  ,GFP_SSW  , &
 & GFP_DSW  ,GFP_SD   ,GFP_SR   ,GFP_AL   ,GFP_EMIS , &
 & GFP_SDOG ,GFP_VEG  ,GFP_SWL1 ,GFP_SWL2 ,GFP_SWL3 ,GFP_SWL4 , &
 & GFP_SRC  ,GFP_CDUTP,GFP_CHAR ,GFP_CLSP ,GFP_CCP  ,GFP_CLSS , &
 & GFP_CLSG ,GFP_CCSG ,GFP_CLSH ,GFP_CCSH , &
 & GFP_IC   ,GFP_CCSF ,GFP_CSSR ,GFP_CSMR ,GFP_CTSR ,GFP_XTCC , &
 & GFP_X2T  ,GFP_X2RH ,GFP_XCCC ,GFP_XHCC ,GFP_XMCC ,GFP_XLCC , &
 & GFP_XX2T ,GFP_XN2T ,GFP_XLSP ,GFP_XCP  ,GFP_XLSS ,GFP_BAAL , &
 & GFP_XLSG ,GFP_XLSH ,GFP_XCSG ,GFP_XCSH, &
 & GFP_XCSF ,GFP_XSSR ,GFP_XTSR ,GFP_IVEG ,GFP_ALSN ,GFP_ALV  , &
 & GFP_SNDE ,GFP_FDSW ,GFP_SFIS ,GFP_ALS  ,GFP_SST  , &
 & GFP_DPAT ,GFP_D2   ,GFP_ARG  ,GFP_SAB  ,GFP_RSMIN,GFP_ACOT , &
 & GFP_LAN  ,GFP_CDSW ,GFP_CSSW ,GFP_RDSW ,GFP_BSR  ,GFP_FSSW , &
 & GFP_PSRHU,GFP_PADOU,GFP_PADOV,GFP_X10FF,GFP_D2MX , &
 & GFP_XCAPE,GFP_XGUST,GFP_XCLPH,GFP_HV   ,GFP_RDST ,GFP_XN2HU, &
 & GFP_Z0H  ,GFP_LAI  ,GFP_CI   ,GFP_XX2HU, &
 & GFP_ASN  ,GFP_RSN  ,GFP_CVL  ,GFP_CVH  ,GFP_IDZ0 , &
 & GFP_ITZ0 ,GFP_PVGMX,GFP_Z0V  ,GFP_PURB ,GFP_GFIS ,GFP_PHYDS  
USE YOMFPC   , ONLY : WSXO     ,WDXO     ,NFPCLI   ,NFPLAKE   , &
 & NFPMASK
USE YOMCST   , ONLY : RG       ,RTT
USE YOMSTA   , ONLY : RDTDZ1
USE YOMPHY1  , ONLY : ALBMAX   ,ALBMIN   ,RHOMAX   ,RHOMIN   ,GWPIMX    ,&
 & GWLMX  
USE YOMPHY   , ONLY : LMPHYS   ,LSOLV
USE YOEPHY   , ONLY : LEPHYS
USE YOMRFPDS , ONLY : MOFPTR
USE PTRFPB2  , ONLY : MAFPTR
USE PTRFP4   , ONLY : YRQGFP
USE YOEVDF   , ONLY : NVTYPES
USE YOMFPG   , ONLY : LFPOSHOR
USE YOMCLI   , ONLY : SMASK    ,SZZ0N    ,SALBN    ,SALBX    ,SEMIN    ,&
 & SEMIX    ,SDEPN    ,SDEPX    ,SARGN    ,SARGX    ,SSABN    ,SSABX    ,&
 & SRSMN    ,SRSMX    ,SZZ0M    ,STHER  
USE YOMARPHY , ONLY : LMSE

IMPLICIT NONE

INTERFACE
#include "surf_inq.h"
END INTERFACE

#include "abor1.intfb.h"
#include "acsolw.intfb.h"
#include "fphor12.intfb.h"
#include "fplake.intfb.h"

INTEGER(KIND=JPIM),INTENT(IN)    :: KBLOC
INTEGER(KIND=JPIM),INTENT(IN)    :: KFPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KAUX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCLI 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PROW(KFPROMA,KFIELDS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAUX(KFPROMA,KAUX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLI(KFPROMA,KCLI) 
LOGICAL           ,INTENT(IN)    :: LDUSE 
!     ZTT   : Threshold
!     ZSCAL : scaling factor
!     ZMIN  : minimum value of the field
!     ZMAX  : maximum value of the field
!     ZSEA  : field value over sea
!     ZLSM  : land-sea mask
!     ZLAN  : Land proportion
!     ZRHS  : surface relative moisture
!     ZTS   : surface temperature
!     ZTCLS : cls temperature
!     ZRHCLS: cls relative moisture
!     ZSTDV : standard deviation of orography
!     ZD2   : soil depth
!     ZARG  : Clay
!     ZCLI  : working array
!     ZALT  : altitude correction for temperature
!     ZCSN  : climatology snow depth
!     ZCSSW : climatology relative surface soil water content
!     ZCDSW : climatology relative deep soil water content
!     ZDTSGDZ: (1./g)*(dT/dz)
!     ZLAI  : climatology leaf area index
!     ZVEG  : climatology vegetation
!     ZTSA  : interpolated surface temperature + altitude correction

REAL(KIND=JPRB) :: ZTT (KFPROMA),ZSCAL(KFPROMA),ZMIN (KFPROMA),ZMAX  (KFPROMA)
REAL(KIND=JPRB) :: ZSEA(KFPROMA),ZLSM (KFPROMA),ZCLI (KFPROMA),ZALT  (KFPROMA)
REAL(KIND=JPRB) :: ZRHS(KFPROMA),ZTS  (KFPROMA),ZSTDV(KFPROMA),ZARG  (KFPROMA)
REAL(KIND=JPRB) :: ZSAB(KFPROMA),ZIVG (KFPROMA),ZD2  (KFPROMA),ZWSMX (KFPROMA)
REAL(KIND=JPRB) :: ZWFC(KFPROMA),ZWPMX(KFPROMA),ZWSAT(KFPROMA),ZWWILT(KFPROMA)
REAL(KIND=JPRB) :: ZCSN(KFPROMA),ZCSSW(KFPROMA),ZCDSW(KFPROMA),ZTCLS (KFPROMA)
REAL(KIND=JPRB) :: ZLAI(KFPROMA),ZVEG (KFPROMA),ZTSA (KFPROMA),ZRHCLS(KFPROMA)
REAL(KIND=JPRB) :: ZRVLAI(0:NVTYPES)
REAL(KIND=JPRB) :: ZTMP(KFPROMA),ZLAN (KFPROMA),ZSST (KFPROMA)
CHARACTER(LEN=7) :: CLLOCK

INTEGER(KIND=JPIM) :: IATS0, IAHS0, IAFIS, IALSM, IASDO, IAT2M, IAQ2M, IATSI
INTEGER(KIND=JPIM) :: IALAN
INTEGER(KIND=JPIM) :: IXTS0, IXHS0, IXFIS, IXLSM, IXSDO, IXT2M, IXQ2M, IXTSI
INTEGER(KIND=JPIM) :: IXSST, IASST, IXLAN

INTEGER(KIND=JPIM) :: IAARG, IAD2, IADSW, IAIVG, IASAB, IASSW, IAVEG, IALAI
INTEGER(KIND=JPIM) :: ICARG, ICD2, ICDSW, ICIVG, ICSAB, ICSSW, ICVEG, ICLAI

INTEGER(KIND=JPIM) :: ICFIS, ICLSM, ICSDO, ICSD, ICTS, ICDT, ICLAN

INTEGER(KIND=JPIM) :: ICOR, JF, JI, ICOD, IANO, IX, IA

LOGICAL :: LLALT, LLARG, LLCDSW, LLCSN, LLCSSW, LLD2, LLHMT, LLIVG, LLRHS, &
 & LLSAB, LLSM, LLSTDV, LLTS, LLDONE, LLVEG, LLLAI ,LLTCLS, LL, LLSST, &
 & LLRHCLS, LLTSA, LLAND 

REAL(KIND=JPRB) :: ZDTSGDZ
REAL(KIND=JPRB) :: ZRWSAT,ZRWLMAX,ZRALFMINSN
REAL(KIND=JPRB) :: ZRALFMAXSN,ZRHOMINSN,ZRHOMAXSN,ZRALBSEAD
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPCORPHY',0,ZHOOK_HANDLE)

!        0. POINTERS
!           --------

! Auxilary buffer:

IXTS0=MAFPTR(GFP_ST%ICOD)
IXSST=MAFPTR(GFP_SST%ICOD)
IXHS0=MAFPTR(GFP_PSRHU%ICOD)
IXFIS=MAFPTR(GFP_SFIS%ICOD)
IXLSM=MAFPTR(GFP_LSM%ICOD)
IXLAN=MAFPTR(GFP_LAN%ICOD)
IXSDO=MAFPTR(GFP_SDOG%ICOD)
IXT2M=MAFPTR(GFP_X2T%ICOD)
IXQ2M=MAFPTR(GFP_X2RH%ICOD)
IXTSI=MAFPTR(GFP_RDST%ICOD)

! Climatology buffer:

ICARG=MOFPTR(GFP_ARG%ICOD)
ICD2 =MOFPTR(GFP_D2%ICOD)
ICDSW=MOFPTR(GFP_CDSW%ICOD)
ICIVG=MOFPTR(GFP_IVEG%ICOD)
ICSAB=MOFPTR(GFP_SAB%ICOD)
ICSSW=MOFPTR(GFP_CSSW%ICOD)
ICVEG=MOFPTR(GFP_VEG%ICOD)
ICLAI=MOFPTR(GFP_LAI%ICOD)
ICLSM=MOFPTR(GFP_LSM%ICOD)
ICLAN=MOFPTR(GFP_LAN%ICOD)
ICFIS=MOFPTR(GFP_GFIS%ICOD)
ICSDO=MOFPTR(GFP_SDOG%ICOD)

ICTS =MOFPTR(GFP_ST%ICOD)
ICDT =MOFPTR(GFP_DST%ICOD)
ICSD =MOFPTR(GFP_SD%ICOD)


!*       1. PREPARATIONS
!           ------------

IF (NFPCLI > 1.AND.LEPHYS) THEN
  CALL ABOR1('FPCORPHY : NOT READY FOR NFPCLI > 1.AND.LEPHYS')
ENDIF

CALL SURF_INQ(PRWSAT=ZRWSAT,PRWLMAX=ZRWLMAX,&
 & PRALFMINSN=ZRALFMINSN,PRALFMAXSN=ZRALFMAXSN,&
 & PRHOMINSN=ZRHOMINSN,PRHOMAXSN=ZRHOMAXSN,PRALBSEAD=ZRALBSEAD,PRVLAI=ZRVLAI)

!     Search some fields in the working row itself :
IAFIS=-999
IALSM=-999
IALAN=-999
IASDO=-999
IATS0=-999
IASST=-999
IAHS0=-999
IAD2 =-999
IAARG=-999
IASAB=-999
IAIVG=-999
IASSW=-999
IADSW=-999
IALAI=-999
IAVEG=-999
IAT2M=-999
IAQ2M=-999
IATSI=-999
DO JF=1,KFIELDS
  ICOD=YRQGFP%ICOD(JF)
  IF (ICOD == GFP_SFIS%ICOD) THEN
!         Interpolated orography
    IAFIS=JF
  ELSEIF (ICOD == GFP_X2T%ICOD) THEN
!         Cls temperature
    IAT2M=JF
  ELSEIF (ICOD == GFP_X2RH%ICOD) THEN
!         Cls relative moisture
    IAQ2M=JF
  ELSEIF (ICOD == GFP_LSM%ICOD) THEN
!         Land-sea mask
    IALSM=JF
  ELSEIF (ICOD == GFP_LAN%ICOD) THEN
!         Land proportion
    IALAN=JF
  ELSEIF (ICOD == GFP_SDOG%ICOD) THEN
!         standardt deviation of orography
    IASDO=JF
  ELSEIF (ICOD == GFP_ST%ICOD) THEN
!         Surface temperature
    IATS0=JF
  ELSEIF (ICOD == GFP_SST%ICOD) THEN
!         SST
    IASST=JF
  ELSEIF (ICOD == GFP_PSRHU%ICOD) THEN
!         Surface relative moisture
    IAHS0=JF
  ELSEIF (ICOD == GFP_D2%ICOD) THEN
!         Soil depth
    IAD2 =JF
  ELSEIF (ICOD == GFP_ARG%ICOD) THEN
!         Clay
    IAARG=JF
  ELSEIF (ICOD == GFP_SAB%ICOD) THEN
!         Sand
    IASAB=JF
  ELSEIF (ICOD == GFP_IVEG%ICOD) THEN
!         Index of vegetation
    IAIVG=JF
  ELSEIF (ICOD == GFP_CSSW%ICOD) THEN
!         Proportion of surface water content
    IASSW=JF
  ELSEIF (ICOD == GFP_CDSW%ICOD) THEN
!         Proportion of deep soil water content
    IADSW=JF
  ELSEIF (ICOD == GFP_LAI%ICOD) THEN
!         Leaf area index
    IALAI=JF
  ELSEIF (ICOD == GFP_VEG%ICOD) THEN
!        Vegetation 
    IAVEG=JF
  ELSEIF (ICOD == GFP_RDST%ICOD) THEN
!        Interpolated surface temperature
    IATSI=JF
  ENDIF
ENDDO

!*       2. CORRECT MAIN CLIMATOLOGY CONSTANTS
!           ----------------------------------

!     search in clim row, then in working row and at last in auxilary row.

!     2.1 Land-sea mask

LLSM=.TRUE.
LLDONE=.FALSE.
IF (ICLSM > 0) THEN
  ZLSM(KST:KEND)=PCLI(KST:KEND,ICLSM)
ELSEIF(LDUSE.AND.IXLSM > 0) THEN
  ZLSM(KST:KEND)=PAUX(KST:KEND,IXLSM)
  LLDONE=.TRUE.
ELSEIF (IALSM > 0) THEN
  ZLSM(KST:KEND)=PROW(KST:KEND,IALSM)
ELSE
  LLSM=.FALSE.
ENDIF
IF (LLSM.AND..NOT.LLDONE) THEN
  IF (LMPHYS) THEN
    ICOR=5
    ZTT(KST:KEND)=SMASK
    CALL FPHOR12(1,KST,KEND,ZLSM,1,KFPROMA,ICOR,ZSEA,ZCLI,ZMIN,&
     & ZMAX,ZSCAL,ZTT)  
  ENDIF
ENDIF
IF (IALSM > 0) THEN
  PROW(KST:KEND,IALSM)=ZLSM(KST:KEND)
ENDIF

!     2.1bis Land proportion

LLAND=.TRUE.
LLDONE=.FALSE.
IF (ICLAN > 0) THEN
  ZLAN(KST:KEND)=PCLI(KST:KEND,ICLAN)
ELSEIF(LDUSE.AND.IXLAN > 0) THEN
  ZLAN(KST:KEND)=PAUX(KST:KEND,IXLAN)
  LLDONE=.TRUE.
ELSEIF (IALAN > 0) THEN
  ZLAN(KST:KEND)=PROW(KST:KEND,IALAN)
ELSE
  LLAND=.FALSE.
ENDIF
IF (LLAND.AND..NOT.LLDONE) THEN
  IF (LMPHYS) THEN
    ICOR=6
    ZMIN(KST:KEND)=0.0_JPRB
    ZMAX(KST:KEND)=1.0_JPRB
    CALL FPHOR12(1,KST,KEND,ZLAN,1,KFPROMA,ICOR,ZSEA,ZCLI,ZMIN,&
     & ZMAX,ZSCAL,ZTT)
  ENDIF
ENDIF
IF (IALAN > 0) THEN
  PROW(KST:KEND,IALAN)=ZLAN(KST:KEND)
ENDIF

!     2.2 Standard deviation of orography

LLSTDV=.TRUE.
LLDONE=.FALSE.
IF (ICSDO > 0) THEN
  ZSTDV(KST:KEND)=PCLI(KST:KEND,ICSDO)
ELSEIF(LDUSE.AND.IXSDO > 0) THEN
  ZSTDV(KST:KEND)=PAUX(KST:KEND,IXSDO)
  LLDONE=.TRUE.
ELSEIF (IASDO > 0) THEN
  ZSTDV(KST:KEND)=PROW(KST:KEND,IASDO)
ELSE
  LLSTDV=.FALSE.
ENDIF
IF (LLSTDV.AND..NOT.LLDONE) THEN
  IF (LMPHYS) THEN
    ICOR=4
    ZMIN(KST:KEND)=0.0_JPRB
    CALL FPHOR12(1,KST,KEND,ZSTDV,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,&
     & ZMAX,ZSCAL,ZTT)  
  ENDIF
ENDIF
IF (IASDO > 0) THEN
  PROW(KST:KEND,IASDO)=ZSTDV(KST:KEND)
ENDIF

!     2.3 Soil depth

LLD2=.TRUE.
IF (ICD2 > 0) THEN
  ZD2(KST:KEND)=PCLI(KST:KEND,ICD2)
ELSEIF (IAD2 > 0) THEN
  ZD2(KST:KEND)=PROW(KST:KEND,IAD2)
ELSE
  LLD2=.FALSE.
ENDIF
IF (LLD2) THEN
  ICOR=3
  ZMIN(KST:KEND)=SDEPN
  ZMAX(KST:KEND)=SDEPX
  ZSEA(KST:KEND)=SDEPX
  ZSCAL(KST:KEND)=1.0_JPRB
  IF (.NOT.LLSM) THEN
    CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
  ENDIF
  CALL FPHOR12(1,KST,KEND,ZD2,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,ZMAX,ZSCAL,ZTT)
ENDIF
IF (IAD2 > 0) THEN
  PROW(KST:KEND,IAD2)=ZD2(KST:KEND)
ENDIF

!     2.4 Clay

LLARG=.TRUE.
IF (ICARG > 0) THEN
  ZARG(KST:KEND)=PCLI(KST:KEND,ICARG)
ELSEIF (IAARG > 0) THEN
  ZARG(KST:KEND)=PROW(KST:KEND,IAARG)
ELSE
  LLARG=.FALSE.
ENDIF
IF (LLARG) THEN
  ICOR=3
  ZMIN(KST:KEND)=SARGN
  ZMAX(KST:KEND)=SARGX
  ZSEA(KST:KEND)=SARGN
  ZSCAL(KST:KEND)=1.0_JPRB
  IF (.NOT.LLSM) THEN
    CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
  ENDIF
  CALL FPHOR12(1,KST,KEND,ZARG,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,ZMAX,ZSCAL,ZTT)
ENDIF
IF (IAARG > 0) THEN
  PROW(KST:KEND,IAARG)=ZARG(KST:KEND)
ENDIF

!     2.5 Sand

LLSAB=.TRUE.
IF (ICSAB > 0) THEN
  ZSAB(KST:KEND)=PCLI(KST:KEND,ICSAB)
ELSEIF (IASAB > 0) THEN
  ZSAB(KST:KEND)=PROW(KST:KEND,IASAB)
ELSE
  LLSAB=.FALSE.
ENDIF
IF (LLSAB) THEN
  ICOR=3
  ZMIN(KST:KEND)=SSABN
  ZMAX(KST:KEND)=SSABX
  ZSEA(KST:KEND)=SSABN
  ZSCAL(KST:KEND)=1.0_JPRB
  IF (.NOT.LLSM) THEN
    CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
  ENDIF
  CALL FPHOR12(1,KST,KEND,ZSAB,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,ZMAX,ZSCAL,ZTT)
ENDIF
IF (IASAB > 0) THEN
  PROW(KST:KEND,IASAB)=ZSAB(KST:KEND)
ENDIF

!     2.6 Index of vegetation

LLIVG=.TRUE.
IF (ICIVG > 0) THEN
  ZIVG(KST:KEND)=PCLI(KST:KEND,ICIVG)
ELSEIF (IAIVG > 0) THEN
  ZIVG(KST:KEND)=PROW(KST:KEND,IAIVG)
ELSE
  LLIVG=.FALSE.
ENDIF
IF (LLIVG) THEN
!       Security to pseudo-interpolations/extrapolations :
  DO JI=KST,KEND
    ZIVG(JI)=MAX(1,MIN(5,NINT(ZIVG(JI))))
  ENDDO
ENDIF
IF (IAIVG > 0) THEN
  PROW(KST:KEND,IAIVG)=ZIVG(KST:KEND)
ENDIF

!     2.7 Vegetation 

LLVEG=.TRUE.
IF (ICVEG > 0) THEN
  ZVEG(KST:KEND)=PCLI(KST:KEND, ICVEG)  
ELSEIF (IAVEG > 0) THEN
  ZVEG(KST:KEND)=PROW(KST:KEND,IAVEG) 
ELSE
  LLVEG=.FALSE.
ENDIF
IF (LLVEG) THEN
  ICOR=3
  ZMIN(KST:KEND)=0.0_JPRB
  ZMAX(KST:KEND)=1.0_JPRB
  ZSEA(KST:KEND)=0.0_JPRB
  ZSCAL(KST:KEND)=1.0_JPRB
  IF (.NOT. LLSM) THEN
    CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK') 
  ENDIF
  CALL FPHOR12(1,KST,KEND,ZVEG,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,ZMAX,ZSCAL,ZTT)
  IF (IAVEG > 0) THEN
    PROW(KST:KEND, IAVEG)=ZVEG(KST:KEND)
  ENDIF
ENDIF
IF (IAVEG > 0) THEN
  PROW(KST:KEND,IAVEG)=ZVEG(KST:KEND)
ENDIF

!     2.8 Leaf area index 

LLLAI=.TRUE.
IF (ICLAI > 0) THEN
  ZLAI(KST:KEND)=PCLI(KST:KEND, ICLAI)  
ELSEIF (IALAI > 0) THEN
  ZLAI(KST:KEND)=PROW(KST:KEND,IALAI) 
ELSE
  LLLAI=.FALSE.
ENDIF
IF (LLLAI) THEN
  ICOR=1
  ZMIN(KST:KEND)=0.0_JPRB
  ZSEA(KST:KEND)=0.0_JPRB
  IF (.NOT. LLSM) THEN
    CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK') 
  ENDIF
  CALL FPHOR12(1,KST,KEND,ZLAI,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,ZMAX,ZSCAL,ZTT)
ENDIF
IF (IALAI > 0) THEN
  PROW(KST:KEND,IALAI)=ZLAI(KST:KEND)
ENDIF

!*       3. CORRECT AUXILARY QUANTITIES
!           ---------------------------

!     3.2 Relative surface moisture & cls moisture

LLRHS=.TRUE.
LLDONE=.FALSE.
IF(LDUSE.AND.IXHS0 > 0) THEN
  ZRHS(KST:KEND)=PAUX(KST:KEND,IXHS0)
  LLDONE=.TRUE.
ELSEIF (IAHS0 > 0) THEN
  ZRHS(KST:KEND)=PROW(KST:KEND,IAHS0)
ELSE
  LLRHS=.FALSE.
ENDIF
IF (LLRHS.AND..NOT.LLDONE) THEN
  ZMIN(KST:KEND)=0.0_JPRB
  ZMAX(KST:KEND)=1.0_JPRB
  ZSCAL(KST:KEND)=1.0_JPRB
  ZSEA(KST:KEND)=1.0_JPRB
  IF (LLSM) THEN
    ICOR=3
    CALL FPHOR12(1,KST,KEND,ZRHS,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,&
     & ZMAX,ZSCAL,ZTT)  
  ELSE
    CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
  ENDIF
ENDIF
IF (IAHS0 > 0) THEN
  PROW(KST:KEND,IAHS0)=ZRHS(KST:KEND)
ENDIF

LLRHCLS=.TRUE.
LLDONE=.FALSE.
IF(LDUSE.AND.IXQ2M > 0) THEN
  ZRHCLS(KST:KEND)=PAUX(KST:KEND,IXQ2M)
  LLDONE=.TRUE.
ELSEIF (IAQ2M > 0) THEN
  ZRHCLS(KST:KEND)=PROW(KST:KEND,IAQ2M)
ELSE
  LLRHCLS=.FALSE.
ENDIF
IF (LLRHCLS.AND..NOT.LLDONE) THEN
! Anomaly HU2m-HUs => HU2m
  IF (LLRHS) THEN
    IF (LFPOSHOR.AND.GFP_X2RH%IANO > 0) THEN
      ZRHCLS(KST:KEND)=ZRHCLS(KST:KEND)+ZRHS(KST:KEND)
    ENDIF
    IF (LMPHYS) THEN
      ICOR=6
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=1.0_JPRB
      CALL FPHOR12(1,KST,KEND,ZRHCLS,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,&
       & ZMAX,ZSCAL,ZTT)  
    ENDIF
  ELSEIF (GFP_X2RH%IANO > 0) THEN 
    CALL ABOR1('FPCORPHY : CANNOT FIND SURF. REL MOISTURE')
  ENDIF
ENDIF
IF (IAQ2M > 0) THEN
  PROW(KST:KEND,IAQ2M)=ZRHCLS(KST:KEND)
ENDIF

!     3.3 Altitude correction for interpolated fields

LLALT=.TRUE.
IF (ICFIS > 0) THEN
  ZDTSGDZ=RDTDZ1/RG
  IF (LDUSE.AND.IXFIS > 0) THEN
    DO JI=KST,KEND
      ZALT(JI)=ZDTSGDZ*(PCLI(JI,ICFIS)-PROW(JI,IXFIS))
    ENDDO
  ELSEIF (IAFIS > 0) THEN
    DO JI=KST,KEND
      ZALT(JI)=ZDTSGDZ*(PCLI(JI,ICFIS)-PROW(JI,IAFIS))
    ENDDO
  ELSE
    LLALT=.FALSE.
  ENDIF
ELSE
  LLALT=.FALSE.
ENDIF

!     3.4 SST (for the time being : it is an "interpolated" SST)

IX=IXSST
IA=IASST
IANO=GFP_SST%IANO

LL=.TRUE.
LLDONE=.FALSE.
IF(LDUSE .AND. IX > 0) THEN
  ZTMP(KST:KEND)=PAUX(KST:KEND,IX)
  LLDONE=.TRUE.
ELSEIF (IA > 0) THEN
  ZTMP(KST:KEND)=PROW(KST:KEND,IA)
ELSE
  LL=.FALSE.
ENDIF
IF (LL.AND..NOT.LLDONE) THEN
  IF ((.NOT.LFPOSHOR) .OR. IANO == 0 .OR. NFPCLI <= 1) THEN
    IF (NFPCLI == 1) THEN
!     Don't do altitude correction if ECMWF
      IF (LMPHYS) THEN
        IF (LLALT) THEN
          ZTMP(KST:KEND)=ZTMP(KST:KEND)+ZALT(KST:KEND)
        ELSE
          CALL ABOR1('FPCORPHY : CAN T FIND ALTITUDE CORRECTION')
        ENDIF
      ENDIF
    ENDIF
  ELSE
    CALL ABOR1('FPCORPHY: SST + NFPCLI >= 2 NOT YET CODED')
  ENDIF
  ZMIN(KST:KEND)=0.0_JPRB
  ICOR=4
  CALL FPHOR12(1,KST,KEND,ZTMP,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,ZMAX,ZSCAL,ZTT)
ENDIF
IF (IA > 0) THEN
  PROW(KST:KEND,IA)=ZTMP(KST:KEND)
ENDIF
LLSST=LL
IF (LLSST) THEN
  ZSST(KST:KEND)=ZTMP(KST:KEND)
ENDIF


!     3.4 Interpolated Surface temperature + altitude correction
!         (for internal use only)

LLTSA=.TRUE.
IF (LDUSE.AND.IXTSI > 0) THEN
  ZTSA(KST:KEND)=PAUX(KST:KEND,IXTSI)
ELSEIF (IATSI > 0) THEN
  ZTSA(KST:KEND)=PROW(KST:KEND,IATSI)
ELSE
  LLTSA=.FALSE.
ENDIF
IF (LLTSA) THEN
! If 2 masks are used, make a combination of interpolated Ts and SST :
  IF (NFPMASK >= 2) THEN
    IF (.NOT.LLSST) CALL ABOR1('FPCORPHY:CAN T FIND SST')
    IF (.NOT.LLAND) CALL ABOR1('FPCORPHY:CAN T FIND LAND PROPORTION')
    DO JI=KST,KEND
      ZTSA(JI)=ZLAN(JI)*ZTSA(JI)+(1._JPRB-ZLAN(JI))*ZSST(JI)
    ENDDO
  ENDIF
ENDIF
IF (IATSI > 0) THEN
  PROW(KST:KEND,IATSI)=ZTSA(KST:KEND)
ENDIF

!     3.4 Surface temperature

IX=IXTS0
IA=IATS0
IANO=GFP_ST%IANO

LL=.TRUE.
LLDONE=.FALSE.
IF(LDUSE .AND. IX > 0) THEN
  ZTMP(KST:KEND)=PAUX(KST:KEND,IX)
  LLDONE=.TRUE.
ELSEIF (IA > 0) THEN
  ZTMP(KST:KEND)=PROW(KST:KEND,IA)
ELSE
  LL=.FALSE.
ENDIF
IF (LL.AND..NOT.LLDONE) THEN
  IF ((.NOT.LFPOSHOR) .OR. IANO == 0 .OR. NFPCLI <= 1) THEN
    IF (NFPCLI == 1) THEN
!     Don't do altitude correction if ECMWF
      IF (LMPHYS) THEN
        IF (LLALT) THEN
          ZTMP(KST:KEND)=ZTMP(KST:KEND)+ZALT(KST:KEND)
        ELSE
          CALL ABOR1('FPCORPHY : CAN T FIND ALTITUDE CORRECTION')
        ENDIF
      ENDIF
    ENDIF
  ELSE
!   Anomaly Ts-T*  => Ts
    ZTMP(KST:KEND)=ZTMP(KST:KEND)+PCLI(KST:KEND,ICTS)
    IF (NFPLAKE == 1) THEN
!     correction for lakes & islands with climatology
      CALL FPLAKE(KBLOC,PCLI(1,ICTS),ZTMP)
    ELSEIF(NFPLAKE == -1) THEN
!     correction for lakes & islands with interpolated field
      CALL FPLAKE(KBLOC,ZTSA,ZTMP)
    ENDIF
  ENDIF
  ZMIN(KST:KEND)=0.0_JPRB
  ICOR=4
  CALL FPHOR12(1,KST,KEND,ZTMP,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,ZMAX,ZSCAL,ZTT)
ENDIF
IF (IA > 0) THEN
  PROW(KST:KEND,IA)=ZTMP(KST:KEND)
ENDIF

LLTS=LL
IF (LLTS) THEN
  ZTS(KST:KEND)=ZTMP(KST:KEND)
ENDIF

!     3.4 Climatology snow depth

IF (ICSD > 0) THEN
  LLCSN=.TRUE.
  IF (LLSM) THEN
    IF (LMPHYS) THEN
      ICOR=2
      ZSEA(KST:KEND)=PCLI(KST:KEND,ICTS)
      ZTT(KST:KEND)=RTT
      ZSCAL(KST:KEND)=1.0_JPRB
    ELSE
      ICOR=1
      ZSEA(KST:KEND)=0.0_JPRB
      ZMIN(KST:KEND)=0.0_JPRB
    ENDIF
    ZCSN(KST:KEND)=PCLI(KST:KEND,ICSD)
  ELSE
    CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
  ENDIF
  CALL FPHOR12(1,KST,KEND,ZCSN,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,ZMAX,ZSCAL,ZTT)
ELSE
  LLCSN=.FALSE.
ENDIF

!     3.5 Proportion of surface water content

LLCSSW=.TRUE.
IF (ICSSW > 0) THEN
  ZCSSW(KST:KEND)=PCLI(KST:KEND,ICSSW)
ELSEIF (IASSW > 0) THEN
  ZCSSW(KST:KEND)=PROW(KST:KEND,IASSW)
ELSE
  LLCSSW=.FALSE.
ENDIF
IF (LLCSSW) THEN
  IF (LMPHYS) THEN
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=1.0_JPRB
      ZSEA(KST:KEND)=1.0_JPRB
      ZSCAL(KST:KEND)=1.0_JPRB
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
    ENDIF
    CALL FPHOR12(1,KST,KEND,ZCSSW,1,KFPROMA,ICOR,ZSEA,ZLSM,&
     & ZMIN,ZMAX,ZSCAL,ZTT)  
  ENDIF
ENDIF
IF (IASSW > 0) THEN
  PROW(KST:KEND,IASSW)=ZCSSW(KST:KEND)
ENDIF

!     3.6 Proportion of deep soil water content

LLCDSW=.TRUE.
IF (ICDSW > 0) THEN
  ZCDSW(KST:KEND)=PCLI(KST:KEND,ICDSW)
ELSEIF (IADSW > 0) THEN
  ZCDSW(KST:KEND)=PROW(KST:KEND,IADSW)
ELSE
  LLCDSW=.FALSE.
ENDIF
IF (LLCDSW) THEN
  IF (LMPHYS) THEN
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=1.0_JPRB
      ZSEA(KST:KEND)=1.0_JPRB
      ZSCAL(KST:KEND)=1.0_JPRB
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
    ENDIF
    CALL FPHOR12(1,KST,KEND,ZCDSW,1,KFPROMA,ICOR,ZSEA,ZLSM,&
     & ZMIN,ZMAX,ZSCAL,ZTT)  
  ENDIF
ENDIF
IF (IADSW > 0) THEN
  PROW(KST:KEND,IADSW)=ZCDSW(KST:KEND)
ENDIF

!     3.3 Cls temperature

LLTCLS=.TRUE.
LLDONE=.FALSE.
IF (LDUSE.AND.IXT2M > 0) THEN
  ZTCLS(KST:KEND)=PAUX(KST:KEND,IXT2M)
  LLDONE=.TRUE.
ELSEIF (IAT2M > 0) THEN
  ZTCLS(KST:KEND)=PROW(KST:KEND,IAT2M)
ELSE
  LLTCLS=.FALSE.
ENDIF
IF (LLTCLS.AND..NOT.LLDONE) THEN
  IF (LFPOSHOR .AND. GFP_X2T%IANO > 0) THEN
!   Anomaly T2m-Ts => T2m
    IF (LLTS) THEN
      ZTCLS(KST:KEND)=ZTCLS(KST:KEND)+ZTS(KST:KEND)
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND SURFACE TEMPERATURE')
    ENDIF
  ENDIF
  ZMIN(KST:KEND)=0.0_JPRB
  ICOR=4
  CALL FPHOR12(1,KST,KEND,ZTCLS,1,KFPROMA,ICOR,ZSEA,ZLSM,ZMIN,ZMAX,ZSCAL,ZTT)
ENDIF
IF (IAT2M > 0) THEN
  PROW(KST:KEND,IAT2M)=ZTCLS(KST:KEND)
ENDIF

!*       4. CURRENT CORRECTIONS ON OTHER FIELDS
!           -----------------------------------

DO JF=1,KFIELDS

!       Security presets :
  ZSEA (KST:KEND)=-99999._JPRB
  ZMIN (KST:KEND)=+99999._JPRB
  ZMAX (KST:KEND)=-99999._JPRB
  ZSCAL(KST:KEND)=+10000._JPRB
  ZTT  (KST:KEND)=-99999._JPRB

  ICOD=YRQGFP%ICOD(JF)
  IF (LFPOSHOR) THEN
    IANO=GFP_PHYDS(ICOD)%IANO
  ELSE
!   No interpolation => only buffers transfers
    IANO=0
  ENDIF

!     4.1 Deep soil temperature

  IF (ICOD == GFP_DST%ICOD) THEN
    IF (LLSM.AND.LLTS) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=+99999._JPRB
      ZSEA(KST:KEND)=ZTS(KST:KEND)
      ZSCAL(KST:KEND)=1.0_JPRB
      IF (IANO > 0) THEN
        IF (NFPCLI <= 1) THEN
!         Anomaly Tp-Ts  => Tp
          PROW(KST:KEND,JF)=PROW(KST:KEND,JF)+ZTS(KST:KEND)
        ELSEIF (NFPCLI >= 2) THEN
!         Anomaly Tp-Tp*  => Tp
          DO JI=KST,KEND
            PROW(JI,JF)=PROW(JI,JF)+PCLI(JI,ICDT)
          ENDDO
          IF (NFPLAKE /= 0) THEN
!           correction for lakes & islands
            CALL FPLAKE(KBLOC,PCLI(1,ICDT),PROW(1,JF))
          ENDIF
        ENDIF
      ENDIF
    ELSE
      CALL ABOR1('FPCORPHY : CAN T COMPUTE DEEP SOIL TEMP.')
    ENDIF

!     4.2 Surface water content

  ELSEIF (ICOD == GFP_SSW%ICOD) THEN
    IF (LSOLV.OR.LMSE) THEN
!     Wsmax can be computed by ACSOLW (ISBA scheme)
!     even if climatology is not present (pre-interpolated fields)
      IF (LLARG .AND. LLD2 .AND. LLSM .AND. LLIVG .AND. LLSAB) THEN
        LLHMT=.FALSE.
        CLLOCK='ACSOLW '
        CALL ACSOLW(KST,KEND,KFPROMA,ZARG,ZD2,ZLSM,ZIVG,ZSAB,LLHMT,ZWFC, &
         & ZWPMX,ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
      ELSE
        CALL ABOR1('FPCORPHY : MISSING FIELDS TO CALL ACSOLW !')
      ENDIF
    ELSE
!     Wsmax is the max. value from input field
      ZWSMX(KST:KEND)=WSXO
    ENDIF
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      IF (IANO == 0) THEN
        ZMAX(KST:KEND)=ZWSMX(KST:KEND)
        ZSEA(KST:KEND)=ZWSMX(KST:KEND)
        ZSCAL(KST:KEND)=1.0_JPRB
      ELSE
        ZMAX(KST:KEND)=1.0_JPRB
        ZSEA(KST:KEND)=1.0_JPRB
        ZSCAL(KST:KEND)=ZWSMX(KST:KEND)
        IF (LLCSSW) THEN
!         (Ws/Wsmax - Wsr) => Ws/Wsmax
          PROW(KST:KEND,JF)=PROW(KST:KEND,JF)+ZCSSW(KST:KEND)
        ELSEIF (NFPCLI > 1) THEN
          CALL ABOR1('FPCORPHY : CLIM.PROP. OF SURF.WATER CONTENT MISSING')
        ENDIF
      ENDIF
      IF (NFPLAKE /= 0) THEN
!       Correction for lakes & islands
        IF (LLCSSW) THEN
          CALL FPLAKE(KBLOC,ZCSSW,PROW(1,JF))
        ELSE
          CALL ABOR1('FPCORPHY : CAN T APPLY LAKE/ISLAND CORRECTION')
        ENDIF
      ENDIF
    ELSE
      CALL ABOR1('FPCORPHY : CAN T COMPUTE WATER CONTENTS')
    ENDIF

!     4.3 Superficial soil ice content

  ELSEIF (ICOD == GFP_FSSW%ICOD) THEN
    IF (LLSM) THEN
      ZMIN(KST:KEND)=0.0_JPRB
      ZSEA(KST:KEND)=0.0_JPRB
      IF (IANO == 0) THEN
        ICOR=1
      ELSE
        IF (LSOLV.OR.LMSE) THEN
!         Wsmax can be computed by ACSOLW (ISBA scheme)
!         even if climatology is not present (pre-interpolated fields)
          IF (LLARG .AND. LLD2 .AND. LLSM .AND. LLIVG .AND. LLSAB) THEN
            LLHMT=.FALSE.
            CLLOCK='ACSOLW '
            CALL ACSOLW(KST,KEND,KFPROMA,ZARG,ZD2,ZLSM,ZIVG,ZSAB,LLHMT,ZWFC, &
             & ZWPMX,ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
          ELSE
            CALL ABOR1('FPCORPHY : MISSING FIELDS TO CALL ACSOLW !')
          ENDIF
        ELSE
!         Wsmax is the max. value from input field
          ZWSMX(KST:KEND)=WSXO
        ENDIF
        ICOR=3
        ZSCAL(KST:KEND)=ZWSMX(KST:KEND)
        ZMAX(KST:KEND)=1.0_JPRB
      ENDIF
    ELSE
      CALL ABOR1('FPCORPHY : CAN T COMPUTE WATER CONTENTS')
    ENDIF

!     4.4 Deep soil water content

  ELSEIF (ICOD == GFP_DSW%ICOD) THEN
    IF (LSOLV.OR.LMSE) THEN
!     Wsmax can be computed by ACSOLW (ISBA scheme)
!     even if climatology is not present (pre-interpolated fields)
      IF (LLARG .AND. LLD2 .AND. LLSM .AND. LLIVG .AND. LLSAB) THEN
        LLHMT=.FALSE.
        CLLOCK='ACSOLW '
        CALL ACSOLW(KST,KEND,KFPROMA,ZARG,ZD2,ZLSM,ZIVG,ZSAB,LLHMT,ZWFC, &
         & ZWPMX,ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
      ELSE
        CALL ABOR1('FPCORPHY : MISSING FIELDS TO CALL ACSOLW !')
      ENDIF
    ELSE
!     Wsmax is the max. value from input field
      ZWPMX(KST:KEND)=WDXO
    ENDIF
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      IF (IANO == 0) THEN
        ZMAX(KST:KEND)=ZWPMX(KST:KEND)
        ZSEA(KST:KEND)=ZWPMX(KST:KEND)
        ZSCAL(KST:KEND)=1.0_JPRB
      ELSE
        ZMAX(KST:KEND)=1.0_JPRB
        ZSEA(KST:KEND)=1.0_JPRB
        ZSCAL(KST:KEND)=ZWPMX(KST:KEND)
        IF (LLCDSW) THEN
!         (Wp/Wpmax - Wpr) => Wp/Wpmax
          PROW(KST:KEND,JF)=PROW(KST:KEND,JF)+ZCDSW(KST:KEND)
        ELSEIF (NFPCLI > 1) THEN
          CALL ABOR1('FPCORPHY : CLIM.PROP. OF DEEP SOIL.WATER MISSING')
        ENDIF
      ENDIF
      IF (NFPLAKE /= 0) THEN
!       Correction for lakes & islands
        IF (LLCDSW) THEN
          CALL FPLAKE(KBLOC,ZCDSW,PROW(1,JF))
        ELSE
          CALL ABOR1('FPCORPHY : CAN T APPLY LAKE/ISLAND CORRECTION')
        ENDIF
      ENDIF
    ELSE
      CALL ABOR1('FPCORPHY : CAN T COMPUTE WATER CONTENTS')
    ENDIF

!     4.5 Deep soil ice content

  ELSEIF (ICOD == GFP_FDSW%ICOD) THEN
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      ZSEA(KST:KEND)=0.0_JPRB
      IF (IANO == 0) THEN
        ZSCAL(KST:KEND)=1.0_JPRB
        ZMAX(KST:KEND)=GWPIMX
      ELSE
        IF (LSOLV.OR.LMSE) THEN
!         Wsmax can be computed by ACSOLW (ISBA scheme)
!         even if climatology is not present (pre-interpolated fields)
          IF (LLARG .AND. LLD2 .AND. LLSM .AND. LLIVG .AND. LLSAB) THEN
            LLHMT=.FALSE.
            CLLOCK='ACSOLW '
            CALL ACSOLW(KST,KEND,KFPROMA,ZARG,ZD2,ZLSM,ZIVG,ZSAB,LLHMT,ZWFC, &
             & ZWPMX,ZWSAT,ZWSMX,ZWWILT,CLLOCK)  
          ELSE
            CALL ABOR1('FPCORPHY : MISSING FIELDS TO CALL ACSOLW !')
          ENDIF
        ELSE
!         Wsmax is the max. value from input field
          ZWPMX(KST:KEND)=WDXO
        ENDIF
        ZSCAL(KST:KEND)=ZWPMX(KST:KEND)
        ZMAX(KST:KEND)=MIN(1.0_JPRB,GWPIMX/ZWPMX(KST:KEND))
      ENDIF
    ELSE
      CALL ABOR1('FPCORPHY : CAN T COMPUTE WATER CONTENTS')
    ENDIF

!     4.6 ECMWF extra fields : soil reservoir contents

  ELSEIF (ICOD == GFP_SRC%ICOD) THEN
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=MAXVAL(ZRVLAI(1:NVTYPES))*ZRWLMAX
      ZSEA(KST:KEND)=0.0_JPRB
      ZSCAL(KST:KEND)=1.0_JPRB
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
    ENDIF

!     4.7 ECMWF extra fields : Charnock parameter

  ELSEIF (ICOD == GFP_CHAR%ICOD) THEN
!         => use option #1 by reversing land and sea roles :
    IF (LLSM) THEN
      ICOR=1
      DO JI=KST,KEND
        ZCLI(JI)=1.0_JPRB-ZLSM(JI)
      ENDDO
      ZSEA(KST:KEND)=0.018_JPRB
      ZMIN(KST:KEND)=0.005_JPRB
      CALL FPHOR12(JF,KST,KEND,PROW,KFIELDS,KFPROMA,ICOR,ZSEA,&
       & ZCLI,ZMIN,ZMAX,ZSCAL,ZTT)  
      ICOR=0
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
    ENDIF

!     4.8 ECMWF extra fields SWL1

  ELSEIF (ICOD == GFP_SWL1%ICOD) THEN
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=ZRWSAT
      ZSEA(KST:KEND)=0.0_JPRB
      ZSCAL(KST:KEND)=1.0_JPRB
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
    ENDIF

!     4.9 ECMWF extra fields SWL2

  ELSEIF (ICOD == GFP_SWL2%ICOD) THEN
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=ZRWSAT
      ZSEA(KST:KEND)=0.0_JPRB
      ZSCAL(KST:KEND)=1.0_JPRB
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
    ENDIF

!     4.10 ECMWF extra fields SWL3

  ELSEIF (ICOD == GFP_SWL3%ICOD) THEN
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=ZRWSAT
      ZSEA(KST:KEND)=0.0_JPRB
      ZSCAL(KST:KEND)=1.0_JPRB
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
    ENDIF

!     4.11 ECMWF extra fields SWL4

  ELSEIF (ICOD == GFP_SWL4%ICOD) THEN
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=ZRWSAT
      ZSEA(KST:KEND)=0.0_JPRB
      ZSCAL(KST:KEND)=1.0_JPRB
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
    ENDIF

  ELSEIF (ICOD == GFP_ASN%ICOD) THEN
!         Snow albedo
    IF (LLSM) THEN
      ICOR=3
      ZMIN(KST:KEND)=ZRALFMINSN
      ZMAX(KST:KEND)=ZRALFMAXSN
      ZSEA(KST:KEND)=ZRALBSEAD
      ZSCAL(KST:KEND)=1.0_JPRB
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
    ENDIF
  ELSEIF (ICOD == GFP_CI%ICOD) THEN
!         Ice concentration
    ICOR=6
    ZMIN(KST:KEND)=0.0_JPRB
    ZMAX(KST:KEND)=1.0_JPRB
  ELSEIF (ICOD == GFP_RSN%ICOD) THEN
!         Snow density
    ICOR=6
    ZMIN(KST:KEND)=ZRHOMINSN
    ZMAX(KST:KEND)=ZRHOMAXSN
  ELSEIF (ICOD == GFP_CVH%ICOD) THEN
!         High vegetation fraction
    ICOR=6
    ZMIN(KST:KEND)=0.0_JPRB
    ZMAX(KST:KEND)=1.0_JPRB
  ELSEIF (ICOD == GFP_CVL%ICOD) THEN
!         Low vegetation fraction
    ICOR=6
    ZMIN(KST:KEND)=0.0_JPRB
    ZMAX(KST:KEND)=1.0_JPRB

!     4.13 Snow cover

  ELSEIF (ICOD == GFP_SD%ICOD) THEN
    IF (LLSM) THEN
      IF (LMPHYS) THEN
        IF (LLTS) THEN
          ICOR=2
          ZSEA(KST:KEND)=ZTS(KST:KEND)
          ZTT(KST:KEND)=RTT
          ZSCAL(KST:KEND)=1.0_JPRB
        ELSE
          CALL ABOR1('FPCORPHY : CANNOT FIND SURF.TEMPERATURE')
        ENDIF
      ELSE
        ICOR=1
        ZSEA(KST:KEND)=0.0_JPRB
        ZMIN(KST:KEND)=0.0_JPRB
      ENDIF
    ELSE
      CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
    ENDIF
    IF (IANO > 0 .AND. NFPCLI >= 2) THEN
!     Snow cover anomaly => Snow cover
      IF (LLCSN) THEN
        PROW(KST:KEND,JF)=PROW(KST:KEND,JF)+ZCSN(KST:KEND)
      ELSE
        CALL ABOR1('FPCORPHY : CANNOT FIND CLIM. SNOW DEPTH')
      ENDIF
    ENDIF

!     4.14 Albedo

  ELSEIF (ICOD == GFP_AL%ICOD  .OR. ICOD == GFP_BAAL%ICOD .OR. &
     & ICOD == GFP_ALS%ICOD .OR. ICOD == GFP_ALV%ICOD  ) THEN  
    IF (LMPHYS) THEN
      ICOR=6
      ZMIN(KST:KEND)=SALBN
      ZMAX(KST:KEND)=SALBX
    ELSE
      ICOR=0
    ENDIF

!     4.15 Emissivity

  ELSEIF (ICOD == GFP_EMIS%ICOD) THEN
    IF (LMPHYS) THEN
      ICOR=6
      ZMIN(KST:KEND)=SEMIN
      ZMAX(KST:KEND)=SEMIX
    ELSE
      ICOR=0
    ENDIF

!     4.16 Maximum proportion of vegetation

  ELSEIF (ICOD == GFP_Z0V%ICOD) THEN
    IF (LMPHYS) THEN
      IF (LLSM) THEN
        ICOR=3
        ZMIN(KST:KEND)=0.0_JPRB
        ZMAX(KST:KEND)=1.0_JPRB
        ZSEA(KST:KEND)=0.0_JPRB
        ZSCAL(KST:KEND)=1.0_JPRB 
      ELSE
        CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
      ENDIF
    ELSE
      ICOR=0
    ENDIF

!     4.17 Model Roughness lenth * g

  ELSEIF (ICOD == GFP_IDZ0%ICOD .OR. ICOD == GFP_ITZ0%ICOD) THEN
    IF (LMPHYS) THEN
      ICOR=4
      ZMIN(KST:KEND)=0.0_JPRB
    ELSE
      ICOR=0
    ENDIF

!     4.17 Climatology Roughness lenth * g

  ELSEIF (ICOD == GFP_SR%ICOD .OR. ICOD == GFP_BSR%ICOD) THEN
    IF (LMPHYS) THEN
      IF (LLSM) THEN
        ICOR=1
        ZMIN(KST:KEND)=SZZ0N*RG
        ZSEA(KST:KEND)=SZZ0M*RG
      ELSE
        CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
      ENDIF
    ELSE
      ICOR=0
    ENDIF

!     4.18 Climatology Thermal roughness lenth * g

  ELSEIF (ICOD == GFP_Z0H%ICOD) THEN
    IF (LMPHYS) THEN
      IF (LLSM) THEN
        ICOR=1
        ZMIN(KST:KEND)=STHER*SZZ0N*RG
        ZSEA(KST:KEND)=SZZ0M*RG
      ELSE
        CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
      ENDIF
    ELSE
      ICOR=0
    ENDIF

!     4.19 Leaf Area Index ; vegetation roughness lenth * g

  ELSEIF (ICOD == GFP_LAI%ICOD .OR. ICOD == GFP_PVGMX%ICOD) THEN
    IF (LMPHYS) THEN
      IF (LLSM) THEN
        ICOR=1
        ZMIN(KST:KEND)=0.0_JPRB
        ZSEA(KST:KEND)=0.0_JPRB
      ELSE
        CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
      ENDIF
    ELSE
      ICOR=0
    ENDIF

!     4.20 Stomatal Minimum resistance

  ELSEIF (ICOD == GFP_RSMIN%ICOD) THEN
    IF (LMPHYS) THEN
      IF (LLSM) THEN
        ICOR=3
        ZMIN(KST:KEND)=SRSMN
        ZMAX(KST:KEND)=SRSMX
        ZSEA(KST:KEND)=SRSMX
        ZSCAL(KST:KEND)=1.0_JPRB
      ELSE
        CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
      ENDIF
    ELSE
      ICOR=0
    ENDIF

!     4.21 Snow albedo

  ELSEIF (ICOD == GFP_ALSN%ICOD) THEN
    IF (LMPHYS) THEN
      ICOR=6
      ZMIN(KST:KEND)=ALBMIN
      ZMAX(KST:KEND)=ALBMAX
    ELSE
      ICOR=0
    ENDIF

!     4.22 Snow density

  ELSEIF (ICOD == GFP_SNDE%ICOD) THEN
    IF (LMPHYS) THEN
      ICOR=6
      ZMIN(KST:KEND)=RHOMIN
      ZMAX(KST:KEND)=RHOMAX
    ELSE
      ICOR=0
    ENDIF

!     4.23 Anisotropy coefficient of topography

  ELSEIF (ICOD == GFP_ACOT%ICOD) THEN
    IF (LMPHYS) THEN
      ICOR=4
      ZMIN(KST:KEND)=0.0_JPRB
    ELSE
      ICOR=0
    ENDIF

!     4.24 Direction of principal axis of topography

  ELSEIF (ICOD == GFP_DPAT%ICOD) THEN
    DO JI=KST,KEND
      PROW(JI,JF)=ATAN(TAN(PROW(JI,JF)))
    ENDDO
    ICOR=0

!     4.25 Percentage of urbanisation

  ELSEIF (ICOD == GFP_PURB%ICOD) THEN
    IF (LMPHYS) THEN
      ICOR=6
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=1.0_JPRB
    ELSE
      ICOR=0
    ENDIF

!     4.26 Maximum soil depth

  ELSEIF (ICOD == GFP_D2MX%ICOD) THEN
    IF (LMPHYS) THEN
      ICOR=3
      ZMIN(KST:KEND)=SDEPN
      ZMAX(KST:KEND)=SDEPX
      ZSEA(KST:KEND)=SDEPX
      ZSCAL(KST:KEND)=1.0_JPRB
      IF (.NOT.LLSM) THEN
        CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
      ENDIF
    ELSE
      ICOR=0
    ENDIF

!     4.27 Relaxation relative deep soil wetness

  ELSEIF (ICOD == GFP_RDSW%ICOD) THEN
    IF (LMPHYS) THEN
      IF (LLSM) THEN
        ICOR=3
        ZMIN(KST:KEND)=0.0_JPRB
        ZMAX(KST:KEND)=1.0_JPRB
        ZSEA(KST:KEND)=1.0_JPRB
        ZSCAL(KST:KEND)=1.0_JPRB
      ELSE
        CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
      ENDIF
    ELSE
      ICOR=0
    ENDIF

!     4.28 Components of vector anisotropy

  ELSEIF (ICOD == GFP_PADOU%ICOD .OR. ICOD == GFP_PADOV%ICOD) THEN
    IF (LLSTDV) THEN
      ICOR=3
      ZMIN(KST:KEND)=-ZSTDV(KST:KEND)**2
      ZMAX(KST:KEND)=+ZSTDV(KST:KEND)**2
      ZSEA(KST:KEND)=0.0_JPRB
      ZSCAL(KST:KEND)=1.0_JPRB
    ELSE
      CALL ABOR1('FPCORPHY : MISSING STD.DEV OF OROG.')
    ENDIF

!     4.29 Precipitations

  ELSEIF (ICOD == GFP_CLSP%ICOD  .OR. ICOD == GFP_CCP%ICOD  .OR. &
     & ICOD == GFP_CLSS%ICOD  .OR. ICOD == GFP_CCSF%ICOD .OR. &
     & ICOD == GFP_CLSG%ICOD  .OR. ICOD == GFP_CCSG%ICOD .OR. &
     & ICOD == GFP_CLSH%ICOD  .OR. ICOD == GFP_CCSH%ICOD .OR. &
     & ICOD == GFP_CDUTP%ICOD .OR. ICOD == GFP_XLSP%ICOD .OR. &
     & ICOD == GFP_XCP%ICOD   .OR. ICOD == GFP_XLSS%ICOD .OR. &
     & ICOD == GFP_XCSF%ICOD  .OR. ICOD == GFP_XLSG%ICOD .OR. &   
     & ICOD == GFP_XLSH%ICOD  .OR. ICOD == GFP_XCSG%ICOD .OR. &
     & ICOD == GFP_XCSH%ICOD ) THEN
   
    IF (LMPHYS) THEN
      ICOR=4
      ZMIN(KST:KEND)=0.0_JPRB
    ELSE
      ICOR=0
    ENDIF

!     4.30 Solar or lunar fluxes

  ELSEIF (ICOD == GFP_CSSR%ICOD .OR. ICOD == GFP_CTSR%ICOD .OR. &
     & ICOD == GFP_XSSR%ICOD .OR. ICOD == GFP_XTSR%ICOD .OR. &
     & ICOD == GFP_CSMR%ICOD) THEN  
    IF (LMPHYS) THEN
      ICOR=4
      ZMIN(KST:KEND)=0.0_JPRB
    ELSE
      ICOR=0
    ENDIF

!     4.31 Extrema of temperature at 2 meters

  ELSEIF (ICOD == GFP_XX2T%ICOD .OR. ICOD == GFP_XN2T%ICOD) THEN
    IF (IANO > 0) THEN
      IF (LLTCLS) THEN
        PROW(KST:KEND,JF)=PROW(KST:KEND,JF)+ZTCLS(KST:KEND)
      ELSE
        CALL ABOR1('FPCORPHY : CAN T FIND CLS TEMPERATURE')
      ENDIF
    ENDIF
    ZMIN(KST:KEND)=0.0_JPRB
    ICOR=4

!     4.32 Extrema of relative moisture at 2 meters

  ELSEIF (ICOD == GFP_XX2HU%ICOD .OR. ICOD == GFP_XN2HU%ICOD) THEN
    IF (IANO > 0) THEN
      IF (LLRHCLS) THEN
        PROW(KST:KEND,JF)=PROW(KST:KEND,JF)+ZRHCLS(KST:KEND)
      ELSE
        CALL ABOR1('FPCORPHY : CAN T FIND CLS RELATIVE MOISTURE')
      ENDIF
    ENDIF
    ICOR=6
    ZMIN(KST:KEND)=0.0_JPRB
    ZMAX(KST:KEND)=1.0_JPRB

!     4.34 Cloudiness

  ELSEIF (ICOD == GFP_XTCC%ICOD .OR. ICOD == GFP_XCCC%ICOD .OR. &
     & ICOD == GFP_XHCC%ICOD .OR. ICOD == GFP_XMCC%ICOD .OR. &
     & ICOD == GFP_XLCC%ICOD) THEN  
    IF (LMPHYS) THEN
      ICOR=6
      ZMIN(KST:KEND)=0.0_JPRB
      ZMAX(KST:KEND)=1.0_JPRB
    ELSE
      ICOR=0
    ENDIF

!     4.35 PBL wind velocity

  ELSEIF (ICOD == GFP_X10FF%ICOD .OR. ICOD == GFP_XGUST%ICOD) THEN
    ICOR=4
    ZMIN(KST:KEND)=0.0_JPRB

!     4.35 Model CAPE

  ELSEIF (ICOD == GFP_XCAPE%ICOD) THEN
    ICOR=4
    ZMIN(KST:KEND)=0.0_JPRB

!     4.35b Height of the PBL

  ELSEIF (ICOD == GFP_XCLPH%ICOD) THEN
    ICOR=4
    ZMIN(KST:KEND)=0.0_JPRB

!     4.36 Interception content 

  ELSEIF (ICOD == GFP_IC%ICOD) THEN
    IF (LMPHYS) THEN
      IF (LLSM) THEN
        IF (LLLAI) THEN
          IF (LLVEG) THEN
            ICOR=3
            ZMIN(KST:KEND)=0.0_JPRB
            ZMAX(KST:KEND)=GWLMX*ZLAI(KST:KEND)*ZVEG(KST:KEND)
            ZSEA(KST:KEND)=0.0_JPRB
            ZSCAL(KST:KEND)=1.0_JPRB
          ELSE
            CALL ABOR1('FPCORPHY : VEGETATION CLIMATOLOGY IS MISSING')
          ENDIF
        ELSE
          CALL ABOR1('FPCORPHY : LAI CLIMATOLOGY IS MISSING')
        ENDIF
      ELSE
        CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
      ENDIF
    ELSE
      ICOR=0
    ENDIF

!     4.37 Resistance to evapotranspiration

  ELSEIF (ICOD == GFP_HV%ICOD) THEN
    IF (LMPHYS) THEN
      IF (LLSM) THEN
        ICOR=3
        ZMIN(KST:KEND)=0.0_JPRB
        ZMAX(KST:KEND)=1.0_JPRB
        ZSEA(KST:KEND)=1.0_JPRB
        ZSCAL(KST:KEND)=1.0_JPRB
      ELSE
        CALL ABOR1('FPCORPHY : CANNOT FIND LAND-SEA MASK')
      ENDIF
    ELSE
      ICOR=0
    ENDIF

  ELSE

!    Else, nothing.

    ICOR=0

  ENDIF

  IF (ICOR /= 0) THEN
    CALL FPHOR12(JF,KST,KEND,PROW,KFIELDS,KFPROMA,ICOR,ZSEA,ZLSM,&
     & ZMIN,ZMAX,ZSCAL,ZTT)  
  ENDIF

ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPCORPHY',1,ZHOOK_HANDLE)
END SUBROUTINE FPCORPHY
