SUBROUTINE FPGEO(KST,KEND,KFPROMA,PGNXO,PGNYO,PGMO,KFIELDS,PROW,CDCONF)

!**** *FPGEO*  - Full- POS geometry of output

!     PURPOSE.
!     --------
!        Transform the post-processed fields into the output geometry

!**   INTERFACE.
!     ----------
!       *CALL* *FPGEO*

!        EXPLICIT ARGUMENTS
!        --------------------
!       INPUT : 
!     KST    : first output point in row
!     KEND   : last output point in row
!     KFPROMA: length of the output subrow
!     PGNXO  : output X-component of rotation matrix devided by map factor
!     PGNYO  : output Y-component of rotation matrix devided by map factor
!     PGMO   : output map factor **2
!     KFIELDS: number of fields in row
!     CDCONF : configuration of work
!       INPUT/OUTPUT : 
!     PROW   : modified output row

!        IMPLICIT ARGUMENTS
!        --------------------
!          None

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        R. El Khatib : 98-09-18 CDCONF rather than KFIELDS/KFIELDS
!        R. El Khatib : Support for gusts ; remove bottom level wind ; no hard-coded names
!        R. El Khatib : 00-08-07 conf 'G' for auxilary surface fields
!        R. El Khatib : 00-08-18 derived types
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMAFN   , ONLY : TFP_DYNDS,GFP_PHYDS
USE YOMVPOS  , ONLY : QFPTYPE  ,NFPOSDYN  ,NFPTRDYN
USE PTRFP4   , ONLY : YRQGFP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNXO(KFPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGNYO(KFPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMO(KFPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PROW(KFPROMA,KFIELDS) 
CHARACTER         ,INTENT(IN)    :: CDCONF 
REAL(KIND=JPRB) :: ZROW(KFPROMA,KFIELDS) ! provisional copy of PROW

INTEGER(KIND=JPIM) :: IORDER, IJ, IPTR, INC, IU, IV, JF, JL, JF2, IJ2, IPTR2, JI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPGEO',0,ZHOOK_HANDLE)
ZROW(KST:KEND,:)=PROW(KST:KEND,:)

SELECT CASE (CDCONF)

CASE('A','I') ! Dynamics

  IPTR=1
  DO JF=1,NFPOSDYN
    IJ=NFPTRDYN(JF)
    IORDER=TFP_DYNDS(IJ)%IORDR
    INC   =QFPTYPE%ILEV(IJ)
    SELECT CASE (IORDER)
    CASE (2)
!         2nd order derivative => multiply with square map factor
      DO JL=IPTR,IPTR+INC-1
        DO JI=KST,KEND
          PROW(JI,JL)=ZROW(JI,JL)*PGMO(JI)
        ENDDO
      ENDDO
    CASE (1)
!         Vector/U => apply compass & map factor for both momenta
!         Find second momentum :
      IPTR2=1   
      DO JF2=1,NFPOSDYN
        IJ2=NFPTRDYN(JF2)
        IF (QFPTYPE%ILED(IJ2) == -IJ) THEN
          EXIT
        ELSE
          IPTR2=IPTR2+QFPTYPE%ILEV(IJ2)
        ENDIF
      ENDDO
      IU=IPTR
      IV=IPTR2
      DO JL=0,INC-1
        DO JI=KST,KEND
          PROW(JI,IU+JL)=ZROW(JI,IU+JL)*PGNYO(JI)+ZROW(JI,IV+JL)*PGNXO(JI)
          PROW(JI,IV+JL)=ZROW(JI,IV+JL)*PGNYO(JI)-ZROW(JI,IU+JL)*PGNXO(JI)
        ENDDO
      ENDDO
    END SELECT
    IPTR=IPTR+INC
  ENDDO
  IF (IPTR-1 /= KFIELDS) CALL ABOR1('FPGEO : INTERNAL ERROR 1')

CASE('P','G') ! physics

  IPTR=1
  DO JF=1, KFIELDS
    IJ    =YRQGFP%ICOD(JF)
    IORDER=GFP_PHYDS(IJ)%IORDR
    INC   =YRQGFP%ILEV(JF)
    SELECT CASE (IORDER)
    CASE (2)
!         2nd order derivative => multiply with square map factor
      DO JL=IPTR,IPTR+INC-1
        DO JI=KST,KEND
          PROW(JI,JL)=ZROW(JI,JL)*PGMO(JI)
        ENDDO
      ENDDO
    CASE (1)
!         Vector/U => apply compass & map factor for both momenta
!         Find second momentum :
      IPTR2=1
      DO JF2=1, KFIELDS
        IF (YRQGFP%IVEC(JF2) == -JF) THEN
          EXIT
        ELSE
          IPTR2=IPTR2+YRQGFP%ILEV(JF2)
        ENDIF
      ENDDO
      IU=IPTR
      IV=IPTR2
      DO JL=0,INC-1
        DO JI=KST,KEND
          PROW(JI,IU+JL)=ZROW(JI,IU+JL)*PGNYO(JI)+ZROW(JI,IV+JL)*PGNXO(JI)
          PROW(JI,IV+JL)=ZROW(JI,IV+JL)*PGNYO(JI)-ZROW(JI,IU+JL)*PGNXO(JI)
        ENDDO
      ENDDO
    END SELECT
    IPTR=IPTR+INC
  ENDDO
  IF (IPTR-1 /= KFIELDS) CALL ABOR1('FPGEO : INTERNAL ERROR 2')

END SELECT

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPGEO',1,ZHOOK_HANDLE)
END SUBROUTINE FPGEO
