SUBROUTINE FPINTDYN(KFIELDS,KGPST,KGPEND,PROW,PBUF,KL0,KFPROMA, &
 & KFLDBUF,KBLOCK)

!**** *FPINTDYN*  - FULL-POS horizontal interpolations of dynamic fields

!     PURPOSE.
!     --------
!        Performs horizontal interpolations or buffer transfers.

!**   INTERFACE.
!     ----------
!       *CALL* *FPINTDYN*

!        EXPLICIT ARGUMENTS
!        --------------------

!        INPUT:
!         KFIELDS  : number of fields in row
!         KGPST    : first output point in row.
!         KGPEND   : last output point in row.
!         PBUF     : buffer which contain the fields to interpolate.
!         KL0      : indexes of the western points, from north to south :
!                    4 points (n13,7,9,15)
!         KFPROMA  : length of the output row.
!         KFLDBUF  : number of fields in input buffer
!         KBLOCK   : current NFPROMA_DEP-sized block
!        OUTPUT:
!         PROW     : interpolated fields.

!        IMPLICIT ARGUMENTS
!        --------------------
!          See #include below. 

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!      See below

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        See documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified by K. YESSAD (MARCH 1997):
!         - rewriting in a way more consistent with semi-Lagrangian code,
!           using array KL0, prior to parallelisation.
!         - parallelisation of FULL-POS.
!         - some cleaning.
!        Ryad El Khatib : 98-08-02 Interpolations only (no corrections)
!        J. Boutahar: 2000-03-01:
!             - new treatment of the Lakes in Fullpos
!             - Search of the nearst point, instead of the interpolation.
!    R. El Khatib : 00-08-18 derived types
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K. Yessad: 28-02-2007 DM-features optimisations in FULL-POS
!     -----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMFPC   , ONLY : NFPINDYN
USE YOMWFPDS , ONLY : NFPWIDE  ,LWSTD12   ,LWSTD04
USE YOMFPSC2 , ONLY : NFPBLOFF_DEP
USE YOMVPOS  , ONLY : QFPTYPE  ,NFPOSDYN  ,NFPTRDYN
USE YOMAFN   , ONLY : TFP_DYNDS
USE YOMMP    , ONLY : NAFPB1
USE YOMWFPB  , ONLY : WSTD12   ,WSTD04
USE YOMFPG   , ONLY : NFPNUMD_DEP

!     -----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDBUF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGPST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGPEND 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PROW(KFPROMA,KFIELDS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBUF(NAFPB1*KFLDBUF) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KFPROMA,NFPWIDE+1) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBLOCK

!     -----------------------------------------------------------------------
!     LLMONO : .TRUE. for monotonic interpolations
!     LLMNMX : .TRUE. to replace interpolations by the search of the 
!              nearest point
!     LLMASK : logical mask : .TRUE. => no interpolation
!     IDER   : Number of "boxes" for each field
!     IDMP   : subdomain number for each field in input buffer (if relevent)

LOGICAL :: LLMONO(KFIELDS), LLMNMX(KFIELDS), LLMASK(KFIELDS)

INTEGER(KIND=JPIM) :: JFLD, IJ, IPTR, INC, JL, JD, IDOM, IPTRBUF
INTEGER(KIND=JPIM) :: ISTB, IENB
INTEGER(KIND=JPIM) :: IDER(KFIELDS)
INTEGER(KIND=JPIM) :: IDMP(KFLDBUF)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -----------------------------------------------------------------------

#include "abor1.intfb.h"
#include "fpint12.intfb.h"
#include "fpint4.intfb.h"

!     -----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPINTDYN',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1. PREPARATIONS
!           ------------

!     Search for fields with missing points / "derivatives"
IDMP(:)=0
IPTR=1
IPTRBUF=1
DO JFLD=1,NFPOSDYN
  IJ=NFPTRDYN(JFLD)
  INC=QFPTYPE%ILEV(IJ)
  DO JL=1,INC
    IDER(IPTR+JL-1)=QFPTYPE%ISPD(JL,IJ)
    LLMNMX(IPTR+JL-1)=(NFPINDYN == 0).OR.(TFP_DYNDS(IJ)%INTER == 0)
  ENDDO
  DO JL=1,INC
    IDOM=QFPTYPE%ISPD(JL,IJ)
    IF (IDOM > 1) THEN
      DO JD=1, IDOM
        IDMP(IPTRBUF+JD-1)=QFPTYPE%IDMP(JD,JL,IJ)
      ENDDO
    ENDIF
    IPTRBUF=IPTRBUF+IDOM
  ENDDO
  IPTR=IPTR+INC
ENDDO
IF (IPTR-1 /= KFIELDS) CALL ABOR1('FPINTDYN : INTERNAL ERROR IPTR')
IF (IPTRBUF-1 /= KFLDBUF) CALL ABOR1('FPINTDYN : INTERNAL ERROR IPTRBUF')

LLMONO(:)=.FALSE.

!*       2. INTERPOLATIONS
!           --------------

!     Interpolate everything :
LLMASK(:)=.FALSE.
ISTB=NFPBLOFF_DEP(KBLOCK)+1
IENB=NFPBLOFF_DEP(KBLOCK)+KGPEND-KGPST+1
IF (NFPINDYN == 12 .OR. (NFPINDYN == 0 .AND. LWSTD12)) THEN
  CALL FPINT12(KFIELDS,KGPST,KGPEND,KFPROMA,KFLDBUF,IDER,IDMP,LLMNMX,LLMONO,&
   & NFPNUMD_DEP(ISTB:IENB),KL0,WSTD12(ISTB:IENB,1:12),LLMASK,PBUF,PROW)  
ELSEIF (LWSTD04) THEN
  CALL FPINT4(KFIELDS,KGPST,KGPEND,KFPROMA,KFLDBUF,IDER,IDMP,LLMNMX,&
   & NFPNUMD_DEP(ISTB:IENB),KL0,WSTD04(ISTB:IENB,1:4),LLMASK,PBUF,PROW)  
ELSE
  CALL ABOR1('FPINTDYN : NO INTERPOLATOR FOUND !')
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPINTDYN',1,ZHOOK_HANDLE)
END SUBROUTINE FPINTDYN
