SUBROUTINE FPINTPHY(KFIELDS,KGPST,KGPEND,PROW,PBUF,KL0, &
 & KFPROMA,KFLDBUF,LDCLI,LDNIL,LDONE,KBLOCK)

!**** *FPINTPHY*  - FULL-POS horizontal interpolations of physical fields

!     PURPOSE.
!     --------
!        Performs horizontal interpolations or buffer transfers.

!**   INTERFACE.
!     ----------
!       *CALL* *FPINTPHY*

!        EXPLICIT ARGUMENTS
!        --------------------
!        INPUT:
!         KFIELDS  : number of fields in row
!         KGPST    : first output point in row.
!         KGPEND   : last output point in row.
!         PBUF     : buffer which contain the fields to interpolate.
!         KL0      : indexes of the western points, from north to south :
!                    4 points (n13,7,9,15)
!         KFPROMA  : length of the output row.
!         KFLDBUF  : number of fields in input buffer
!         LDCLI    : .T. if the field is overwritten by the climatology
!         LDNIL    : .T. if the field is not interpolatable
!         KBLOCK   : current NFPROMA_DEP-sized block

!        OUTPUT:
!         LDONE    : .T. if the field has been interpolated
!         PROW     : interpolated fields.

!        IMPLICIT ARGUMENTS
!        --------------------
!          None

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!       See below

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        See documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified by K. YESSAD (MARCH 1997):
!         - rewriting in a way more consistent with semi-Lagrangian code,
!           using array KL0, prior to parallelisation.
!         - parallelisation of FULL-POS.
!         - some cleaning.
!        Modified : 97-07-23 by Cristina Madeira (ISBA Fields in FULL-POS)
!        Modified : 97-10-06 by Adam Dziedzic (ISBA schema Douville Fields
!                                              in FULL-POS)
!        Modified : 97-12-16 by Dominique Giard (frozen deep soil water)
!        R. El Khatib : 98-09-03 (split to FPINT12/FPCLIPHY/FPCORPHY)
!        Modified : 99-04-19 by J.M. Piriou (Moon radiation)
!        R. El Khatib  : 00-03-28 Convection indexes and PBL diagnostics
!        Modified :  2000-03-01 by J. Boutahar:
!             - new treatment of the Lakes in Fullpos
!             - Search of the nearst point, instead of the interpolation.
!       R. El Khatib : 00-12-06 Monotonic interpolations for resistance to 
!                               evapotranspiration
!       R. El Khatib : 00-12-18 Monotonic interpolations for interpolated
!                               roughness length
!       R. El Khatib : 01-05-04 Duration of total precipitations,HUn,HUx
!       R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        R. El Khatib  10-May-2005 %LLLSM replaced by mask index.
!        R. El Khatib  20-May-2005 NFPWIDE moved to YOMWFPDS
!        K. Yessad: 28-02-2007 DM-features optimisations in FULL-POS
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMAFN   , ONLY : GFP_PHYDS
USE PTRFP4   , ONLY : YRQGFP
USE YOMWFPDS , ONLY : NFPWIDE  ,LWSTD12  ,LWSTD04  ,LWLAN12  ,LWLAN04  , &
 & LWSEA12  ,LWSEA04
USE YOMFPC   , ONLY : NFPINPHY ,NFPMASK
USE YOMMP    , ONLY : NAFPB1
USE YOMWFPB  , ONLY : WSTD12   ,WSTD04   ,WLAN12   ,WLAN04   ,WSEA12   , &
 & WSEA04
USE YOMFPSC2 , ONLY : NFPBLOFF_DEP

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDBUF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGPST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGPEND 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PROW(KFPROMA,KFIELDS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBUF(NAFPB1*KFLDBUF) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KFPROMA,NFPWIDE+1) 
LOGICAL           ,INTENT(IN)    :: LDCLI(KFIELDS) 
LOGICAL           ,INTENT(IN)    :: LDNIL(KFIELDS) 
LOGICAL           ,INTENT(OUT)   :: LDONE(KFIELDS) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBLOCK

!     ------------------------------------------------------------------

!     IDER   : Number of "boxes"  for each field (for "horizontal derivative")
!     LLMONO : .TRUE. for monotonic interpolations
!     LLMNMX : .TRUE. to replace interpolations by the search of the
!              nearest point
!     IMASK  : mask for the interpolations :
!              0 = no mask
!              1 = land-sea mask
!              2 = sea mask
!     LLSKIP : logical mask for skipping interpolations.

LOGICAL :: LLSKIP(KFIELDS), LLINT(KFLDBUF)
LOGICAL :: LLMNMX(KFIELDS), LLMONO(KFIELDS)
LOGICAL :: LLDO(KFIELDS)
INTEGER(KIND=JPIM) :: IBOX(KFIELDS), IDER(KFIELDS), IDMP(KFLDBUF)
INTEGER(KIND=JPIM) :: IMASK(KFIELDS)
INTEGER(KIND=JPIM) :: ISTB, IENB

INTEGER(KIND=JPIM) :: JF, ICOD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "fpint12.intfb.h"
#include "fpint4.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPINTPHY',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1. PREPARATIONS
!           ------------

DO JF=1,KFIELDS
  ICOD=YRQGFP%ICOD(JF)
  LLINT (JF)=.NOT.(LDCLI(JF).OR.LDNIL(JF))
  LLMONO(JF)=GFP_PHYDS(ICOD)%LLMON
  LLMNMX(JF)=(GFP_PHYDS(ICOD)%INTER == 0).OR.(NFPINPHY == 0)
  IMASK (JF)=MIN(GFP_PHYDS(ICOD)%IMASK,NFPMASK) ! a security
  IDER  (JF)=1
  IBOX  (JF)=1
ENDDO

IDMP(:)=0

ISTB=NFPBLOFF_DEP(KBLOCK)+1
IENB=NFPBLOFF_DEP(KBLOCK)+KGPEND-KGPST+1


!*       2. INTERPOLATIONS WITHOUT MASK
!           ---------------------------

! Fields that will be computed now :
DO JF=1,KFIELDS
  LLDO(JF)=(IMASK(JF) == 0) .AND. LLINT(JF)
ENDDO
DO JF=1,KFIELDS
  LLSKIP(JF)=.NOT.LLDO(JF)
ENDDO
IF (ANY(LLDO)) THEN
  IF (NFPINPHY == 12 .OR. (NFPINPHY == 0 .AND. LWSTD12)) THEN
    CALL FPINT12(KFIELDS,KGPST,KGPEND,KFPROMA,KFLDBUF,IDER, &
     & IDMP,LLMNMX,LLMONO,IBOX,KL0,WSTD12(ISTB:IENB,1:12),LLSKIP,PBUF,PROW)
  ELSEIF (LWSTD04) THEN
    CALL FPINT4(KFIELDS,KGPST,KGPEND,KFPROMA,KFLDBUF,IDER, &
     & IDMP,LLMNMX,IBOX,KL0,WSTD04(ISTB:IENB,1:4),LLSKIP,PBUF,PROW)
  ELSE
    CALL ABOR1('FPINTPHY: NO INTERPOLATOR FOUND - 0 !')
  ENDIF
ENDIF
! Signal FIRST event
DO JF=1,KFIELDS
  LDONE(JF)=LLDO(JF)
ENDDO


!*       3. INTERPOLATIONS WITH MASK
!           ------------------------

!*       3.1 INTERPOLATIONS WITH LAND(-SEA) MASK

! Fields that will be computed now : 
DO JF=1,KFIELDS
  LLDO(JF)=(IMASK(JF) == 1) .AND. LLINT(JF)
ENDDO
DO JF=1,KFIELDS
  LLSKIP(JF)=.NOT.LLDO(JF)
ENDDO
IF (ANY(LLDO)) THEN
  IF (NFPINPHY == 12 .OR. (NFPINPHY == 0 .AND. LWLAN12)) THEN
    CALL FPINT12(KFIELDS,KGPST,KGPEND,KFPROMA,KFLDBUF,IDER, &
     & IDMP,LLMNMX,LLMONO,IBOX,KL0,WLAN12(ISTB:IENB,1:12),LLSKIP,PBUF,PROW)
  ELSEIF (LWLAN04) THEN
    CALL FPINT4(KFIELDS,KGPST,KGPEND,KFPROMA,KFLDBUF,IDER, &
     & IDMP,LLMNMX,IBOX,KL0,WLAN04(ISTB:IENB,1:4),LLSKIP,PBUF,PROW)  
  ELSE
    CALL ABOR1('FPINTPHY: NO INTERPOLATOR FOUND - 1 !')
  ENDIF
ENDIF
! Signal SUPPLEMENTARY event
DO JF=1,KFIELDS
  LDONE(JF)=LDONE(JF).OR.LLDO(JF)
ENDDO

!*       3.2 INTERPOLATIONS WITH MASK SEA(-LAND) MASK

! Fields that will be computed now :
DO JF=1,KFIELDS
  LLDO(JF)=(IMASK(JF) == 2) .AND. LLINT(JF)
ENDDO
DO JF=1,KFIELDS
  LLSKIP(JF)=.NOT.LLDO(JF)
ENDDO
IF (ANY(LLDO)) THEN
  IF (NFPINPHY == 12 .OR. (NFPINPHY == 0 .AND. LWSEA12)) THEN
    CALL FPINT12(KFIELDS,KGPST,KGPEND,KFPROMA,KFLDBUF,IDER, &
     & IDMP,LLMNMX,LLMONO,IBOX,KL0,WSEA12(ISTB:IENB,1:12),LLSKIP,PBUF,PROW)
  ELSEIF (LWSEA04) THEN
    CALL FPINT4(KFIELDS,KGPST,KGPEND,KFPROMA,KFLDBUF,IDER, &
     & IDMP,LLMNMX,IBOX,KL0,WSEA04(ISTB:IENB,1:4),LLSKIP,PBUF,PROW)
  ELSE
    CALL ABOR1('FPINTPHY: NO INTERPOLATOR FOUND - 1 !')
  ENDIF
ENDIF
! Signal SUPPLEMENTARY event
DO JF=1,KFIELDS
  LDONE(JF)=LDONE(JF).OR.LLDO(JF)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPINTPHY',1,ZHOOK_HANDLE)
END SUBROUTINE FPINTPHY
