SUBROUTINE FPOSHOR(PBUF1,CDCONF)

!**** *FPOSHOR*  - HORIZONTAL POST-PROCESSING

!     PURPOSE.
!     --------
!        PERFORM THE HORIZONTAL INTERPOLATIONS AND THE PBL DISPLACEMENT

!        Computations are DM-local if distributed memory.

!**   INTERFACE.
!     ----------
!       *CALL* *FPOSHOR*

!        EXPLICIT ARGUMENTS
!        --------------------

!     All dummy arguments are input ones.

!     PBUF1  : buffer 1 which contain the fields to interpolate
!     CDCONF : configuration of work
!              A : vertically interpolated fields
!              P : surface physical fields & fluxes
!              I : model fields
!              G : auxilary surface gridpoint fields

!        IMPLICIT ARGUMENTS
!        ------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        See documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified : 96-01-03 R. El Khatib (fullpos aladin)
!        Modified by K. YESSAD (MARCH 1997): parallelisation of FULL-POS.
!        Modified : 97-10-13 R. El Khatib (rotation matrix includes possibly
!                                   the map factor)
!        Modified : 97-07-23 by Cristina Madeira (ISBA Fields in FULL-POS)
!        Modified : 97-10-06 by Adam Dziedzic (ISBA schema Douville Fields)
!        Modified : 98-04-10 R. El Khatib (remove NITMFPP)
!        Modified : 98-03-18 R. El Khatib (interpolated orography)
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of LRPOLE in YOMCT0, LLPOLE and code under LRPOLE, LLPOLE.
!          - removal of NDGLG, LRPOLE, LLPOLE in 'CALL FPSCAW'.
!        Modified : 98-08-28 R. El Khatib (adiabatic pp.)
!        R. El Khatib : 98-09-04 modernised handling of interpolations
!        Modified : 98-09-26 G. Radnoti   (saving aux fields to buffe/workfile)
!        R. El Khatib : 98-10-07 Replace NFPROF by NFPSTART
!        D. Giard : 99-07-26 new management of bilinear interpolations
!        J. Boutahar: 2000-03-01:
!             - debugging bilinear interpolation
!             - new treatment of the Lakes in Fullpos
!             - Search of the nearst point, instead of the interpolation.
!        R. El Khatib : 00-07-28 Specific configuration 'G' for auxilary fields
!        R. El Khatib : 00-07-28 Vertical part moved in "lagged" part
!        R. El Khatib : 00-08-18 derived types
!        R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvemnts
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!   R. El Khatib  20-May-2005 NFPWIDE moved to YOMWFPDS
!        K. Yessad: 28-02-2007 DM-environment optimisations in FULL-POS.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMFPSC2 , ONLY : NFPBLOCS ,NFPROMA  ,NFPSTART ,NFPEND   ,&
 & NFPBLOCS_DEP,NFPROMA_DEP,NFPSTART_DEP,NFPEND_DEP,NFPSORT  ,NFPSORT_DEP
USE YOMWFPDS , ONLY : NFPWIDE
USE YOMRFPDS , ONLY : NRFPOS   ,MFPGMO   ,MFPGNXO  ,MFPGNYO
USE YOMFPC   , ONLY : NFPCLI   ,NFPINPHY
USE YOMMP    , ONLY : NAFPB1
USE PTRFPB2  , ONLY : NFPVT0
USE YOMFPG   , ONLY : LFPOSHOR ,LFPOSBUF ,LFPDISTRIB,NFPRGPL,NFPRGPL_DEP
USE YOMAFPB  , ONLY : AFPBUF   ,NSTAAFPB
USE YOMPFPB  , ONLY : GFPBUF   ,NSTAGFPB
USE YOMRFPB  , ONLY : RFPBUF   ,NSTARFPB
USE YOMDFPB  , ONLY : HFPBUF   ,NSTAHFPB
USE YOMFPDIM , ONLY : NFPBFIN  ,NFPBFOUT

!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PBUF1(NAFPB1*NFPBFIN) 
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 

!     ------------------------------------------------------------------

!     * output climatology and geometry:
REAL(KIND=JPRB) :: ZRFP   (NFPROMA,NRFPOS)
REAL(KIND=JPRB) :: ZNORDM(NFPROMA),ZNORDL(NFPROMA),Z1SGM2(NFPROMA)
!     * auxilary surface fields:
REAL(KIND=JPRB) :: ZAUX(NFPROMA,NFPVT0)
!     * output fields:
REAL(KIND=JPRB) :: ZROW(NFPROMA,NFPBFOUT)
REAL(KIND=JPRB) :: ZROW_DEP(NFPROMA_DEP,NFPBFOUT)
REAL(KIND=JPRB),ALLOCATABLE :: ZROW_ALLBLOCKS_DEP(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZROW_ALLBLOCKS(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZROW_SAVE(:,:,:)
!     * working arrays:
INTEGER(KIND=JPIM) :: IL1(NFPROMA_DEP,NFPWIDE+1)

!     Handling horizontal interpolations of physical fields : 
!     LLCLI=.T. if the field is overwritten by the climatology
!     LLNIL=.T. if the field cannot be "directly" interpolated
!     LLINT=.T. if the field is to be interpolated
!     IHINTS : number of times a field has been computed

LOGICAL :: LLCLI(NFPBFOUT), LLNIL(NFPBFOUT), LLINT(NFPBFOUT)
INTEGER(KIND=JPIM) :: IHINTS(NFPBFOUT)

INTEGER(KIND=JPIM) :: IEND, IST, JBLOC, JFLD, JFP, IERR  
INTEGER(KIND=JPIM) :: IC, IFLDS

LOGICAL :: LLAUX    ! auxilary row at disposal
LOGICAL :: LLINTER  ! .TRUE. if actual interpolation on physical fields

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "fpcliphy.intfb.h"
#include "fpcordyn.intfb.h"
#include "fpcorphy.intfb.h"
#include "fpgeo.intfb.h"
#include "fpintdyn.intfb.h"
#include "fpintphy.intfb.h"
#include "fpnilphy.intfb.h"
#include "fpsampl.intfb.h"
#include "fpscaw.intfb.h"
#include "fptrdtoa.intfb.h"
#include "sc2rdgfp.intfb.h"
#include "sc2wrgfp.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPOSHOR',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

CALL GSTATS(1911,0)

!     ------------------------------------------------------------------

!*       1. PREPARATIONS
!           ------------

LLAUX = CDCONF=='P' .AND. NFPVT0 > 0
LLINT(:)=.FALSE.

! * Compute LLCLI:
IF (CDCONF == 'A'.OR.CDCONF == 'I') THEN
  ! * LLCLI always .F. in this case.
  LLCLI(:)=.FALSE.
ELSE
  LLCLI(:)=.FALSE.
  IF (NFPCLI > 0) THEN
    CALL FPCLIPHY(NFPBFOUT,IST,IEND,NFPROMA,NRFPOS,LDCLI=LLCLI)
  ENDIF
ENDIF

! * Compute LLNIL:
IF (CDCONF == 'A'.OR.CDCONF == 'I') THEN
  ! * LLNIL always .F. in this case.
  LLNIL(:)=.FALSE.
ELSE
  LLNIL(:)=.FALSE.
  IF (LFPOSHOR) THEN
    LLINTER=NFPINPHY > 0
    CALL FPNILPHY(NFPBFOUT,IST,IEND,NFPROMA,NFPVT0,LLINTER,LDNIL=LLNIL)
  ENDIF
ENDIF

! * Conditional allocation of some (large) ZROW... arrays:
IF (LFPOSBUF.AND.LFPDISTRIB) THEN
  ALLOCATE(ZROW_ALLBLOCKS_DEP(NFPRGPL_DEP,NFPBFOUT))
  ALLOCATE(ZROW_ALLBLOCKS(NFPRGPL,NFPBFOUT))
  ZROW_ALLBLOCKS_DEP(:,:)=0.0_JPRB
ELSE
  ALLOCATE(ZROW_SAVE(NFPROMA,NFPBFOUT,NFPBLOCS))
ENDIF

!     ------------------------------------------------------------------

!*       2. CALCULATIONS DOING HORIZONTAL INTERPOLATIONS OR SAMPLING
!           --------------------------------------------------------

! These calculations must be done on the departure geometry DM-environment.

DO JBLOC=1,NFPBLOCS_DEP

  IST =NFPSTART_DEP(JBLOC)
  IEND=NFPEND_DEP(JBLOC)

  CALL FPSCAW(NFPROMA_DEP,IST,IEND,IL1,JBLOC)  

  IF (CDCONF == 'A'.OR.CDCONF == 'I') THEN

    IF (LFPOSHOR) THEN
      ! Interpolate fields :  
      CALL FPINTDYN(NFPBFOUT,IST,IEND,ZROW_DEP,PBUF1,IL1,NFPROMA_DEP,NFPBFIN,&
       & JBLOC)
    ELSE
      ! Sampling on all fields : 
      CALL FPSAMPL(NFPBFOUT,IST,IEND,NFPROMA_DEP,NFPBFIN,PBUF1,ZROW_DEP,IL1,&
       & LLCLI,LLINT,JBLOC)
    ENDIF

  ELSE

    ! Compute pronostic fields : 
    IF (LFPOSHOR) THEN
      ! Interpolate fields :  
      CALL FPINTPHY(NFPBFOUT,IST,IEND,ZROW_DEP,PBUF1,IL1,NFPROMA_DEP,NFPBFIN,&
       & LLCLI,LLNIL,LLINT,JBLOC)
    ELSE
      ! Sampling : 
      CALL FPSAMPL(NFPBFOUT,IST,IEND,NFPROMA_DEP,NFPBFIN,PBUF1,ZROW_DEP,IL1,&
       & LLCLI,LLINT,JBLOC)
    ENDIF

  ENDIF

  ! * Memory transfer between ZROW_DEP and ZROW_ALLBLOCKS_DEP:
  IF (LFPOSBUF.AND.LFPDISTRIB) THEN
    DO JFLD=1,NFPBFOUT
      DO JFP=IST,IEND
        ZROW_ALLBLOCKS_DEP(NFPSORT_DEP(JFP,JBLOC),JFLD)=ZROW_DEP(JFP,JFLD)
      ENDDO
    ENDDO
  ELSE
    ZROW_SAVE(IST:IEND,1:NFPBFOUT,JBLOC)=ZROW_DEP(IST:IEND,1:NFPBFOUT)
  ENDIF

ENDDO

!     ------------------------------------------------------------------

!*       3. SOME CHECKINGS ABOUT LLCLI, LLNIL, LLINT
!           ----------------------------------------

! * Check that all fields have been computed once and only once:
IF (.NOT.(CDCONF == 'A'.OR.CDCONF == 'I')) THEN
  IERR=0
  DO JFLD=1,NFPBFOUT
    IHINTS(JFLD)=0
    IF (LLCLI(JFLD)) IHINTS(JFLD)=IHINTS(JFLD)+1
    IF (LLNIL(JFLD)) IHINTS(JFLD)=IHINTS(JFLD)+1
    IF (LLINT(JFLD)) IHINTS(JFLD)=IHINTS(JFLD)+1
    IF (IHINTS(JFLD) > 1) IERR=1
  ENDDO
  IF (IERR == 1) CALL ABOR1('FPOSHOR : INTERNAL ERROR ON IHINTS')
ENDIF

!     ------------------------------------------------------------------

!*       4. TRANSPOSITION BETWEEN DEPART. GEOM AND ARRIVAL. GEOM DM-PARTITIONS
!           ------------------------------------------------------------------

IF (LFPOSBUF.AND.LFPDISTRIB) THEN
  IC=1
  IFLDS=NFPBFOUT
  CALL FPTRDTOA(IC,IFLDS,ZROW_ALLBLOCKS_DEP,ZROW_ALLBLOCKS)
ENDIF

!     ------------------------------------------------------------------

!*       5. CALCULATIONS WHICH DO NOT HORIZONTAL INTERPOLATIONS OR SAMPLING
!           ---------------------------------------------------------------

! These calculations must be done on the arrival geometry DM-environment.

DO JBLOC=1,NFPBLOCS

  IST =NFPSTART(JBLOC)
  IEND=NFPEND(JBLOC)

  !        5.1 MEMORY TRANSFER

  ! * Memory transfer between ZROW and ZROW_ALLBLOCKS:
  IF (LFPOSBUF.AND.LFPDISTRIB) THEN
    DO JFLD=1,NFPBFOUT
      DO JFP=IST,IEND
        ZROW(JFP,JFLD)=ZROW_ALLBLOCKS(NFPSORT(JFP,JBLOC),JFLD)
      ENDDO
    ENDDO
  ELSE
    ZROW(IST:IEND,1:NFPBFOUT)=ZROW_SAVE(IST:IEND,1:NFPBFOUT,JBLOC)
  ENDIF

  !        5.2 READ OUTPUT GEOMETRY AND CLIMATOLOGY

  IF (NRFPOS > 0) THEN

    CALL SC2RDGFP(NFPROMA,NRFPOS,RFPBUF,NSTARFPB(JBLOC),IST,IEND,ZRFP)

    IF (MFPGMO > 0) THEN
      DO JFP=IST,IEND
        Z1SGM2(JFP)=1.0_JPRB/(ZRFP(JFP,MFPGMO)**2)
      ENDDO
    ENDIF
    IF (MFPGNXO > 0.AND.MFPGNYO > 0) THEN
      DO JFP=IST,IEND
        ZNORDM(JFP)=ZRFP(JFP,MFPGNYO)
        ZNORDL(JFP)=ZRFP(JFP,MFPGNXO)
      ENDDO
    ENDIF

  ENDIF

  !        5.3 READ AUXILARY FIELDS

  IF (LLAUX) THEN
    CALL SC2RDGFP(NFPROMA,NFPVT0,GFPBUF,NSTAGFPB(JBLOC),IST,IEND,ZAUX)
  ENDIF

  !        5.4 NO-INTERPOLATION OR SAMPLING CALCULATIONS FOR ZROW.
  !            AND STORE DATA IN BUFFERS VIA SC2WRGFP.

  IF (CDCONF == 'A'.OR.CDCONF == 'I') THEN

    IF (LFPOSHOR) THEN
      ! Corrections afer interpolations : 
      CALL FPCORDYN(NFPBFOUT,IST,IEND,ZROW,NFPROMA)
      ! Go to output geometry : 
      CALL FPGEO(IST,IEND,NFPROMA,ZNORDL,ZNORDM,Z1SGM2,NFPBFOUT,ZROW,CDCONF)
    ENDIF

    SELECT CASE (CDCONF)
    CASE ('I')
      ! Store intermediate results
      CALL SC2WRGFP(NFPROMA,NFPBFOUT,AFPBUF,NSTAAFPB(JBLOC),IST,IEND,ZROW)
    CASE DEFAULT
      ! Store current results
      CALL SC2WRGFP(NFPROMA,NFPBFOUT,HFPBUF,NSTAHFPB(JBLOC),IST,IEND,ZROW)
    END SELECT

  ELSE

    ! Overwrite constant fields with climatology : 
    IF (NFPCLI > 0) THEN
      CALL FPCLIPHY(NFPBFOUT,IST,IEND,NFPROMA,NRFPOS,PCLI=ZRFP,PROW=ZROW)
    ENDIF

    ! Compute pronostic fields : 
    IF (LFPOSHOR) THEN
      ! Compute fields that cannot be interpolated : 
      LLINTER=NFPINPHY > 0
      CALL FPNILPHY(NFPBFOUT,IST,IEND,NFPROMA,NFPVT0,LLINTER,PAUX=ZAUX, &
       & PROW=ZROW)
    ENDIF

    ! Corrections after interpolations or climatology usage : 
    IF (LFPOSHOR.OR.NFPCLI > 0) THEN
      CALL FPCORPHY(JBLOC,IST,IEND,NFPROMA,NFPBFOUT,ZROW,NFPVT0,ZAUX,NRFPOS,&
       & ZRFP,LLAUX)  
    ENDIF

    ! Go to output geometry
    IF (LFPOSHOR) THEN
      CALL FPGEO(IST,IEND,NFPROMA,ZNORDL,ZNORDM,Z1SGM2,NFPBFOUT,ZROW,CDCONF)
    ENDIF

    SELECT CASE (CDCONF)
    CASE ('G')
      ! Store auxilary results
      CALL SC2WRGFP(NFPROMA,NFPBFOUT,GFPBUF,NSTAGFPB(JBLOC),IST,IEND,ZROW)
    CASE DEFAULT
      ! Store current results
      CALL SC2WRGFP(NFPROMA,NFPBFOUT,HFPBUF,NSTAHFPB(JBLOC),IST,IEND,ZROW)
    END SELECT 

  ENDIF

ENDDO

!     ------------------------------------------------------------------

!*       6. DEALLOCATIONS
!           -------------

! * Conditional deallocation of some (large) ZROW... arrays:
IF (LFPOSBUF.AND.LFPDISTRIB) THEN
  DEALLOCATE(ZROW_ALLBLOCKS_DEP)
  DEALLOCATE(ZROW_ALLBLOCKS)
ELSE
  DEALLOCATE(ZROW_SAVE)
ENDIF

CALL GSTATS(1911,1)

!-----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('FPOSHOR',1,ZHOOK_HANDLE)
END SUBROUTINE FPOSHOR
