SUBROUTINE SPACONVERT(LDMODEL_TO_FILE)

!**** *SPACONVERT*  - Spectral array conversion

!     Purpose.
!     --------
!        To convert model spectral fields into the fields actually written on file, 
!        and vice-versa.
!        Arpege/Ifs : Vor/Div in model is Psi/Khi in file
!        Aladin     : Vor/Div + mean wind in model is geographical wind in file
!        Both       : NH variables are scaled.
!        Both       : T-RT conversion

!**   Interface.
!     ----------
!        *CALL* *SPACONVERT(LDMODEL_TO_FILE)

!        Explicit arguments :
!        --------------------
!           LDMODEL_TO_FILE : .TRUE. to convert from model to file

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.    
!     ----------    

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        R. El Khatib  *METEO-FRANCE*

!     Modifications.
!     --------------
!   Original : 98-03-30
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!   R. El Khatib   08-Dec-2004 upgrade NH fields conversion
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   R. El Khatib   04-Sep-2008 T-RT conversion
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB     ,JPIM
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LELAM    ,LNHDYN   ,LSPRT
USE YOMDYNA  , ONLY : LRWSDLR

IMPLICIT NONE
LOGICAL,INTENT(IN)    :: LDMODEL_TO_FILE 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "espconvert.intfb.h"
#include "spconvert.intfb.h"
#include "spnh_conv_nhvar.intfb.h"
#include "spnh_conv_prhs.intfb.h"
#include "especrt.intfb.h"
#include "specrt.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPACONVERT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF (LDMODEL_TO_FILE) THEN
! From model to file :
  IF (LELAM) THEN
!   Convert RT to T if relevent
    IF (LSPRT) THEN
      CALL ESPECRT(1)
    ENDIF
!   Scale NH variable AFTER RT->T conversion because it uses T not RT
    IF (LNHDYN) THEN
      CALL SPNH_CONV_NHVAR(.TRUE.)
      IF (LRWSDLR) CALL SPNH_CONV_PRHS(.TRUE.)
    ENDIF
!   Convert (vor,div,mean-U,mean-V) to geographical (U,V) at the end
    CALL ESPCONVERT(.TRUE.)
  ELSE
!   Convert RT to T if relevent
    IF (LSPRT) THEN
      CALL SPECRT(1)
    ENDIF
!   Scale NH variable AFTER RT->T conversion because it uses T not RT
    IF (LNHDYN) THEN
      CALL SPNH_CONV_NHVAR(.TRUE.)
      IF (LRWSDLR) CALL SPNH_CONV_PRHS(.TRUE.)
    ENDIF
!   Convert (vor,div) to (psi,khi) at the end
    CALL SPCONVERT(.TRUE.)
  ENDIF

ELSE
! From file to model :
  IF (LELAM) THEN
!   Convert geographical (U,V) to (vor,div,mean-U,mean-V) first
    CALL ESPCONVERT(.FALSE.)
!   Unscale NH variable BEFORE T->RT conversion because it uses T not RT
    IF (LNHDYN) THEN
      IF (LRWSDLR) CALL SPNH_CONV_PRHS(.FALSE.)
      CALL SPNH_CONV_NHVAR(.FALSE.)
    ENDIF
!   Convert T to RT if relevent
    IF (LSPRT) THEN
      CALL ESPECRT(0)
    ENDIF
  ELSE
!   Convert (psi,khi) to (vor,div) first
    CALL SPCONVERT(.FALSE.)
!   Unscale NH variable BEFORE T->RT conversion because it uses T not RT
    IF (LNHDYN) THEN
      IF (LRWSDLR) CALL SPNH_CONV_PRHS(.FALSE.)
      CALL SPNH_CONV_NHVAR(.FALSE.)
    ENDIF
!   Convert T to RT if relevent
    IF (LSPRT) THEN
      CALL SPECRT(0)
    ENDIF
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPACONVERT',1,ZHOOK_HANDLE)
END SUBROUTINE SPACONVERT
