SUBROUTINE SPOS

!**** *SPOS*  - SPECTRAL POST-PROCESSING - FULL POS

!     PURPOSE.
!     --------
!        To modify the post-processed fields in the spectral space :
!        -Overtruncation of each field if required
!        -Horizontal filter for each derivative

!**   INTERFACE.
!     ----------
!       *CALL* *SPOS*

!        EXPLICIT ARGUMENTS
!        --------------------

!        IMPLICIT ARGUMENTS
!        --------------------
!          See #include below.

!     METHOD.
!     -------
!        See documentation about FULL-POS.

!     EXTERNALS.
!     ----------
!        Calls MXMAOP.
!        Is called by SPCM and ESPCM.

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!       Modified : 95-06-06 L. Isaksen - Reordering of spectral arrays
!       Modified : 97-06-09 K. YESSAD  - A-parallelisation, F90 rewriting.
!       R. El Khatib : 00-08-18 derived types
!       R. El Khatib : 01-04-10 Enable post-processing of filtered spectra + cleaning
!       R. El Khatib : 01-08-07 Pruning options
!       R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!       R. El Khatib : 03-04-17 Fullpos improvemnts
!       R. El Khatib : 03-08-26 LFPMAT (bugfix)
!       M.Hamrud     : 01-Oct-2003 CY28 Cleaning
!       F. Taillefer : 06-04-01 called by ALADIN too (instead of ESPOS)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMAFN   , ONLY : TFP_KHI  ,TFP_PSI,  TFP_DYNDS
USE YOMDIM   , ONLY : NSMAX    ,NUMP     ,NSPEC2
USE YOMLAP   , ONLY : NASM0    ,NVALUE   ,MYMS     ,RLAPIN
USE YOMFPSP  , ONLY : SPBFP
USE YOMFPDIM , ONLY : NFPSPD
USE YOMFPF   , ONLY : LFPFIL   ,RFPFIL   ,LFPMAT   ,RFPMAT   ,RFPEXPC
USE YOMVPOS  , ONLY : QFPTYPE  ,NFPOSDYN ,NFPTRDYN ,MYSFSET
USE YOMFPC   , ONLY : NFPDOM
USE YOMCT0   , ONLY : LELAM

IMPLICIT NONE

!     II    : =1 if IM=0 ; =2 if not
!     ZSPD  : ALL input derivatives for the working wavenumber
!     ZSPD  : ALL output derivatives of ONE subdomain for the working wavenumber
!     ZSPB  : output derivatives for ONE field for the working wavenumber
!     IDIM  : first dimension of ZSPD, ZSPB
!     ISM   : start adress of matrix in RFPMAT

REAL(KIND=JPRB),ALLOCATABLE :: ZSPD(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZSPB(:,:,:)

INTEGER(KIND=JPIM) :: IN, JDOM, JSE, J, IJ, JL, ISPB, IM, IDOM
INTEGER(KIND=JPIM) :: IDIM, II, ISE, ISM, JN, ISPD, JD, JMLOC, IFPSPB
INTEGER(KIND=JPIM) :: ISRANGE

LOGICAL :: LLPSIKHI

REAL(KIND=JPRB) :: ZB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "mxmaop.h"
END INTERFACE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------
!*       1.    COMPUTATIONS FOR NON-DILATED OR NON-STRETCHED FIELDS
!              ----------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SPOS',0,ZHOOK_HANDLE)
IFPSPB=SIZE(SPBFP,DIM=1)

ISPB=1
DO J=1,NFPOSDYN
  IJ=NFPTRDYN(J)
  SELECT CASE (QFPTYPE%ISF(IJ))
  CASE (1:)
    LLPSIKHI=(IJ == TFP_KHI%ICOD .OR. IJ == TFP_PSI%ICOD)
    SELECT CASE (QFPTYPE%ISF(IJ))
    CASE (2) 
      ZB=-0.5_JPRB*TFP_DYNDS(IJ)%ZFK
    CASE DEFAULT 
      ZB=0.0_JPRB 
    END SELECT
    DO JL=1,QFPTYPE%ILEV(IJ)
      IF (QFPTYPE%ISET(JL,IJ)==MYSFSET) THEN
        DO JDOM=1,QFPTYPE%ISPD(JL,IJ) 

!           1.1 Compute stream function & velocity potential from U & V

          IF (LLPSIKHI) THEN
            DO JSE=1,NSPEC2
              SPBFP(ISPB,JSE) = SPBFP(ISPB,JSE)*RLAPIN(NVALUE(JSE))
            ENDDO
          ENDIF

!           1.2 Spectral filters
  
          SELECT CASE (QFPTYPE%ISF(IJ))
          CASE (2)
            DO JSE=1,NSPEC2
              SPBFP(ISPB,JSE)=SPBFP(ISPB,JSE)*EXP(ZB*RFPEXPC(JSE))
            ENDDO
          CASE (3)
            IDOM=QFPTYPE%IDMP(JDOM,JL,IJ)
            IF ((.NOT.LFPMAT).AND.LFPFIL(IDOM)) THEN
              IF (LELAM) THEN
                SPBFP(ISPB,:)=SPBFP(ISPB,:)*RFPFIL(:,IDOM)
              ELSE
                DO JSE=1,NSPEC2
                  IN=NVALUE(JSE)
                  SPBFP(ISPB,JSE)=SPBFP(ISPB,JSE)*RFPFIL(IN,IDOM)
                ENDDO
              ENDIF
            ENDIF
          END SELECT

          ISPB=ISPB+1
        ENDDO
      ENDIF
    ENDDO
  END SELECT
ENDDO
IF (ISPB-1 /= IFPSPB) CALL ABOR1('SPOS:INTERNAL ERROR IN ISPB')

!*       2.    COMPUTATIONS FOR DILATED STRETCHED FIELDS
!              -----------------------------------------

IF (LFPMAT .AND. ANY(LFPFIL)) THEN

  ISM=1
  DO JMLOC=1,NUMP

    IM=MYMS(JMLOC)
    ISRANGE=NSMAX+1-IM

!*            2.1 Allocations

    II=1+MIN(1,IABS(IM))
    IDIM=2*(ISRANGE/2)+1
    ALLOCATE(ZSPD(IDIM,NFPSPD,2),ZSPB(IDIM,NFPSPD,2))

!*            2.2 Input memory transfer

    ISPB=1
    ISPD=1
    DO J=1, NFPOSDYN
      IJ=NFPTRDYN(J)
      IF (QFPTYPE%ISF(IJ) >= 1) THEN
        DO JL=1,QFPTYPE%ILEV(IJ)
          IF (QFPTYPE%ISET(JL,IJ)==MYSFSET) THEN
            IF (QFPTYPE%ISF(IJ)==3) THEN
              DO JN=IM,NSMAX
                ISE=NASM0(IM)+2*(JN-IM)
                ZSPD(JN+1-IM,ISPD,1)=SPBFP(ISPB,ISE  )
                ZSPD(JN+1-IM,ISPD,2)=SPBFP(ISPB,ISE+1)
              ENDDO
              ISPD=ISPD+1
            ENDIF
            ISPB=ISPB+QFPTYPE%ISPD(JL,IJ)
          ENDIF
        ENDDO 
      ENDIF
    ENDDO
    IF (ISPD-1 /= NFPSPD) CALL ABOR1('SPOS:INTERNAL ERROR IN TRANSFER - ISPD')
    IF (ISPB-1 /= IFPSPB) CALL ABOR1('SPOS:INTERNAL ERROR IN TRANSFER - ISPB')

!*             2.3 Filter then output memory transfer

!       Prevent unitialized values :
    ZSPB(:,:,2)=0.0_JPRB
    DO JDOM=1,NFPDOM
      IF (LFPFIL(JDOM)) THEN
        CALL MXMAOP(RFPMAT(ISM,JDOM),1,ISRANGE,ZSPD,1,IDIM,ZSPB,1,IDIM,&
         & ISRANGE,ISRANGE,II*NFPSPD)  
        ISPB=1
        ISPD=1
        DO J=1, NFPOSDYN
          IJ=NFPTRDYN(J)
          IF (QFPTYPE%ISF(IJ) >= 1) THEN
            DO JL=1,QFPTYPE%ILEV(IJ)
              IF (QFPTYPE%ISET(JL,IJ)==MYSFSET) THEN
                IF (QFPTYPE%ISF(IJ)==3) THEN
                  DO JD=1,QFPTYPE%IDOM(JL,IJ)
                    IF (QFPTYPE%IDMP(JD,JL,IJ) == JDOM) THEN
                      DO JN=IM,NSMAX
                        ISE=NASM0(IM)+2*(JN-IM)
                        SPBFP(ISPB+JD-1,ISE  )=ZSPB(JN+1-IM,ISPD,1)
                        SPBFP(ISPB+JD-1,ISE+1)=ZSPB(JN+1-IM,ISPD,2)
                      ENDDO
                      EXIT
                    ENDIF
                  ENDDO
                  ISPD=ISPD+1
                ENDIF
                ISPB=ISPB+QFPTYPE%ISPD(JL,IJ)
              ENDIF
            ENDDO
          ENDIF
        ENDDO
        IF (ISPD-1 /= NFPSPD) CALL ABOR1('SPOS:INTERNAL ERROR ON ISPD')
        IF (ISPB-1 /= IFPSPB) CALL ABOR1('SPOS:INTERNAL ERROR IN ISPB')
      ENDIF
    ENDDO

!*            2.4 Deallocations

    DEALLOCATE(ZSPD,ZSPB)

    ISM=ISM+ISRANGE**2

  ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('SPOS',1,ZHOOK_HANDLE)
END SUBROUTINE SPOS
