SUBROUTINE SUFPD

!**** *SUFPD*  - INITIALIZE FULL-POS HORIZONTAL SUBDOMAINS

!     PURPOSE.
!     --------
!        SETS DEFAULT VALUES, THEN READS NAMELIST NAMFPD AND CHECKS IT.
!        COMPUTES TOTAL NUMBER OF OUTPUT POINTS.

!**   INTERFACE.
!     ----------
!       *CALL* *SUFPD*

!        EXPLICIT ARGUMENTS
!        --------------------
!        NONE

!        IMPLICIT ARGUMENTS
!        --------------------
!        YOMLUN
!        PARDIM
!        PARFPOS
!        YOMFPD
!        NAMFPD

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        MODIFIED : 95-12-15 by P. Le Moigne: Aladin case included.
!        Modified by K. YESSAD (MARCH 1997): parallelisation of FULL-POS.
!        Modified : 97-31-10 R. El Khatib (Modify default values)
!        Modified : 98-06-09 R. El Khatib (Set default latlon domain inside
!                   aladin small enough to remain 2 rows within the model area
!                   in order not to degenerate horizontal interpolations.
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!         - remove obsolete comments about poles.
!         - remove poles from NFPD1 calculation for spherical geometry output.
!        Modified : 98-07-07  R. El Khatib (Move computation of NFPD1 to SUFPG)
!        Modified : 98-10-01  R. El Khatib (Move computation of NFPSIZEG to SUFPG1)
!        Modified : 98-10-01  R. El Khatib (Remove NFPGP)
!        Modified : 98-11-27 P. Le Moigne (bug correction)
!        Modified : 99-01-14  R. El Khatib (Remove tests out of object)
!        Modified : 01-03-26  R. El Khatib (More controls + modified default values for aladin)
!        Modified : 01-04-13  M. Janousek (New domain definition)
!        Modified : 01-05-25  J. Masek (RCORI_ACAD added to NEMGEO/YEMGEO)
!        R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 03-04-17 Fullpos improvments
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified : 04-07-08 S. Martinez (Introduce ATOUR05 domain)
!        J-D Gril : 04-11-29  calls modif. for Mercator Rotated/tilted
!        Modified : 05-11-09 S. Martinez (Introduce ATOURX05 domain)
!        J-D Gril : 06-01-31 Initialization at 0,0 of ELAT0,ELON0
!                   for latlon cases
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULNAM   ,NULERR
USE YOMDIM   , ONLY : NDGLG    ,NDLON    ,NDGUXG   ,NDLUXG   ,&
 & NDGUNG   ,NDLUNG
USE YOMFPC   , ONLY : CFPFMT   ,CFPDOM   ,NFPDOM   ,LTRACEFP 
USE YOMFPD   , ONLY : NLAT     ,NLON     ,RLATC    ,RLONC    ,&
 & RDELX    ,RDELY    ,NFPLUX   ,&
 & NFPGUX   ,&
 & RLONW    ,RLONE    ,RLATS    ,RLATN  
USE YEMGEO   , ONLY :       EDELX    ,EDELY    ,&
 & ELONC    ,ELATC    ,ELON0    ,ELAT0    ,RCORI_ACAD ,&
 & LMRT     ,LMAP , RLAT_ACAD, RLON_ACAD 
USE YOMCT0   , ONLY : LFPART2  ,LELAM  ,LRPLANE, LFPSPEC
USE YOMCST   , ONLY : RPI      ,RA
USE YOMARG   , ONLY : NARGS    ,UGEMU    

IMPLICIT NONE

CHARACTER(LEN=132)            :: CL

INTEGER(KIND=JPIM)            :: IERR, J, JI, IROTEQ, ISOTRP, IGIVO

REAL(KIND=JPRB)               :: ZDELX, ZDELY, ZEPS, ZLAMAX,ZLAMIN, ZLATC, ZLAT0,  &
 & ZLOMAX, ZLOMIN, ZLONC, ZLON0, ZLON1, ZLAT1, ZLON2, ZLAT2, ZRPK, ZRD2DG, ZDG2RD, &
 & ZLONR, ZLATR, ZBETA

REAL(KIND=JPRB) , ALLOCATABLE :: ZGLON(:,:),ZGLAT(:,:),ZNORX(:,:),ZNORY(:,:),ZGM(:,:)
REAL(KIND=JPRB)               :: ZHOOK_HANDLE
LOGICAL                       :: LLMRT

#include "abor1.intfb.h"
#include "eggx_n.intfb.h"
#include "posnam.intfb.h"

#include "namfpd.h"
#include "nemgeo.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPD',0,ZHOOK_HANDLE)
ZEPS=EPSILON(1.0_JPRB)*100._JPRB
ZRD2DG=180.0_JPRB/RPI
ZDG2RD=RPI/180.0_JPRB

!*       1. SET DEFAULT VALUES
!           -------------------

IF (.NOT.LELAM.AND.(CFPFMT == 'GAUSS'.OR.CFPFMT == 'MODEL')) THEN
!       Default values as the global model
  NLAT(1)=NDGLG
  NLON(1)=NDLON
  RLONC(1)=0.0_JPRB
  RLATC(1)=0.0_JPRB
  RDELX(1)=10000._JPRB
  RDELY(1)=10000._JPRB
  NFPLUX=0
  NFPGUX=0
ELSEIF (LELAM.AND.(CFPFMT == 'LELAM'.OR.CFPFMT == 'MODEL')) THEN
!       Read namelist or command line, waiting for a cleanup
  CALL POSNAM(NULNAM,'NEMGEO')
  READ(NULNAM,NEMGEO)
  IF (NARGS /= 0.OR.LFPART2) THEN
    ELATC=MOD(UGEMU(6)*ZRD2DG,360._JPRB)
    ELONC=MOD(UGEMU(5)*ZRD2DG,360._JPRB)
    IF (UGEMU(2) < 0.0_JPRB) THEN
      EDELX=MOD(UGEMU(7)*ZRD2DG,360._JPRB)
      EDELY=MOD(UGEMU(8)*ZRD2DG,360._JPRB)
    ELSE
      EDELX=UGEMU(7)
      EDELY=UGEMU(8)
    ENDIF
  ENDIF
!       Default values as the LAM model
  NFPLUX=NDLUXG
  NFPGUX=NDGUXG
  IF (CFPFMT == 'MODEL'.OR.LFPSPEC) THEN
    NLON(1)=NDLON
    NLAT(1)=NDGLG
  ELSE
    NLON(1)=NDLUXG
    NLAT(1)=NDGUXG
  ENDIF
  RLATC(1)=ELATC
  RLONC(1)=ELONC
  IF (RLONC(1) > 180._JPRB+ZEPS) THEN
    RLONC(1)=RLONC(1)-360._JPRB
  ENDIF
  RDELX(1)=EDELX
  RDELY(1)=EDELY
ELSEIF (.NOT.LELAM.AND.CFPFMT == 'LELAM') THEN
  IF     (CFPDOM(1) == 'BELG') THEN
    RLONC(1)= 4.90727841961041_JPRB
    RLATC(1)= 50.44595488554766_JPRB
    RDELX(1)= 12715.66669793552_JPRB
    RDELY(1)= 12715.66669793411_JPRB
    NFPLUX=61
    NFPGUX=61
  ELSEIF (CFPDOM(1) == 'SLOV') THEN
    RLONC(1)= 13.52668207859151_JPRB
    RLATC(1)= 46.05017943078632_JPRB
    RDELX(1)= 26271.55175829969_JPRB
    RDELY(1)= 26271.55175398597_JPRB
    NFPLUX=37
    NFPGUX=37
  ELSEIF (CFPDOM(1) == 'MARO') THEN
    RLONC(1)= -6.9999999966693_JPRB
    RLATC(1)= 31.56059436526966_JPRB
    RDELX(1)= 18808.17792427479_JPRB
    RDELY(1)= 18808.17793051683_JPRB
    NFPLUX=149
    NFPGUX=149
  ELSEIF (CFPDOM(1) == 'OPMA') THEN
    RLONC(1)= -7.00000000285346_JPRB
    RLATC(1)= 31.56059442218072_JPRB
    RDELX(1)= 31336.13988918715_JPRB
    RDELY(1)= 31336.13991686922_JPRB
    NFPLUX=97
    NFPGUX=97
  ELSEIF (CFPDOM(1) == 'LACE14') THEN
    RLONC(1)= 16.99999999944358_JPRB
    RLATC(1)= 46.24470063381371_JPRB
    RDELX(1)= 14734.913810093_JPRB
    RDELY(1)= 14734.91380550296_JPRB
    NFPGUX=181
    NFPLUX=205
  ELSEIF (CFPDOM(1) == 'LACE12') THEN
    RLONC(1)=17._JPRB
    RLATC(1)=46.2447006_JPRB
    RDELX(1)=12176.15632814141_JPRB
    RDELY(1)=12176.15632814142_JPRB
    NFPGUX=205
    NFPLUX=229
  ELSEIF (CFPDOM(1) == 'LACE20') THEN
    RLONC(1)=17.00000000009107_JPRB
    RLATC(1)=46.24470062058892_JPRB
    RDELX(1)=20678.82820101414_JPRB
    RDELY(1)=20678.82820101414_JPRB
    NFPLUX=139
    NFPGUX=124
  ELSEIF (CFPDOM(1) == 'ROUM') THEN
    RLONC(1)= 25.00000000483406_JPRB
    RLATC(1)= 44.77301981937139_JPRB
    RDELX(1)= 33102.62857952392_JPRB
    RDELY(1)= 33102.6285617361_JPRB
    NFPLUX=61
    NFPGUX=61
  ELSEIF (CFPDOM(1) == 'FRAN') THEN
    RLONC(1)= 1.27754303826285_JPRB
    RLATC(1)= 45.31788242335041_JPRB
    RDELX(1)= 12715.66779231173_JPRB
    RDELY(1)= 12715.67301977791_JPRB
    NFPLUX=189
    NFPGUX=189
  ELSE
!   Old domain ALADIN-FRANCE
    RLONC(1)= 2.57831063089259_JPRB
    RLATC(1)= 46.46884540633992_JPRB
    RDELX(1)= 12715.66736292664_JPRB
    RDELY(1)= 12715.6635946432_JPRB
    NFPLUX=169
    NFPGUX=169
  ENDIF
  IF (LFPSPEC) THEN
    NLON(1)=NFPLUX+11
    NLAT(1)=NFPGUX+11
  ELSE
    NLON(1)=NFPLUX
    NLAT(1)=NFPGUX
  ENDIF
ELSEIF (LELAM.AND.CFPFMT == 'GAUSS') THEN
  NLAT(1)=32
  NLON(1)=64
  RLONC(1)=0._JPRB
  RLATC(1)=0._JPRB
  RDELX(1)=180._JPRB/REAL(NLAT(1)-1,JPRB)
  RDELY(1)=360._JPRB/REAL(NLON(1)-1,JPRB)
  NFPLUX=0
  NFPGUX=0
ELSE
  DO J=1, NFPDOM
    IF     (CFPDOM(J) == 'HENORD') THEN
      NLAT(J)=60
      NLON(J)=180
      RLONC(J)=179._JPRB
      RLATC(J)=45._JPRB
      RDELX(J)=ABS(358._JPRB-0.0_JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(89.25_JPRB-0.75_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'HESUDC') THEN
      NLAT(J)=60
      NLON(J)=180
      RLONC(J)=179._JPRB
      RLATC(J)=-45._JPRB
      RDELX(J)=ABS(358._JPRB-0.0_JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(-89.25_JPRB+0.75_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'HESUDA') THEN
      NLAT(J)=30
      NLON(J)=90
      RLONC(J)=178._JPRB
      RLATC(J)=-45._JPRB
      RDELX(J)=ABS(356._JPRB-0.0_JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(-88.5_JPRB+1.5_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'ATLMED') THEN
      NLAT(J)=65
      NLON(J)=129
      RLONC(J)=-20._JPRB
      RLATC(J)=48.75_JPRB
      RDELX(J)=ABS(44._JPRB+84._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(72.75_JPRB-24.75_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'EURATL') THEN
      NLAT(J)=103
      NLON(J)=103
      RLONC(J)=2._JPRB
      RLATC(J)=45.75_JPRB
      RDELX(J)=ABS(36._JPRB+32._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(71.25_JPRB-20.25_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'ZONCOT') THEN
      NLAT(J)=81
      NLON(J)=81
      RLONC(J)=0.0_JPRB
      RLATC(J)=48.75_JPRB
      RDELX(J)=ABS(20._JPRB+20._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(63.75_JPRB-33.75_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'FRANCE') THEN
      NLAT(J)=61
      NLON(J)=61
      RLONC(J)=2._JPRB
      RLATC(J)=45.75_JPRB
      RDELX(J)=ABS(12._JPRB+8._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(53.25_JPRB-38.25_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'GLOB15') THEN
      NLAT(J)=121
      NLON(J)=240
      RLONC(J)=179.25_JPRB
      RLATC(J)=0.0_JPRB
      RDELX(J)=ABS(358.5_JPRB-0.0_JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(90._JPRB+90._JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'EURAT5') THEN
      NLAT(J)=105
      NLON(J)=149
      RLONC(J)=5._JPRB
      RLATC(J)=46._JPRB
      RDELX(J)=ABS(42._JPRB+32._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(72._JPRB-20._JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'ATOUR10') THEN
      NLAT(J)=81
      NLON(J)=166
      RLONC(J)=-17.5_JPRB
      RLATC(J)=40._JPRB
      RDELX(J)=ABS(65._JPRB+100._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(80._JPRB-0.0_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'EUROC25') THEN
      NLAT(J)=105
      NLON(J)=129
      RLONC(J)=1._JPRB
      RLATC(J)=48._JPRB
      RDELX(J)=ABS(17._JPRB+15._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(61._JPRB-35._JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'GLOB25') THEN
      NLAT(J)=73
      NLON(J)=144
      RLONC(J)=178.75_JPRB
      RLATC(J)=0.0_JPRB
      RDELX(J)=ABS(357.5_JPRB-0.0_JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(90._JPRB+90._JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'EURSUD') THEN
      NLAT(J)=41
      NLON(J)=54
      RLONC(J)=-19._JPRB/3._JPRB
      RLATC(J)=38.25_JPRB
      RDELX(J)=ABS(24._JPRB+34._JPRB/3._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(48.25_JPRB-28.25_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'EUREST') THEN
      NLAT(J)=39
      NLON(J)=73
      RLONC(J)=16._JPRB/3._JPRB
      RLATC(J)=50.75_JPRB
      RDELX(J)=ABS(88._JPRB/3._JPRB+56._JPRB/3._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(60.25_JPRB-41.25_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'GRID25') THEN
      NLAT(J)=21
      NLON(J)=41
      RLONC(J)=0.0_JPRB
      RLATC(J)=50._JPRB
      RDELX(J)=ABS(50._JPRB+50._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(75._JPRB-25._JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'MAROC') THEN
      NLAT(J)=158
      NLON(J)=171
      RLONC(J)=-6.975_JPRB
      RLATC(J)=31.05_JPRB
      RDELX(J)=ABS(5.85_JPRB+19.8_JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(42.9_JPRB-19.2_JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'OCINDIEN15') THEN
      NLAT(J)=67
      NLON(J)=89
      RLONC(J)=66._JPRB
      RLATC(J)=-16.5_JPRB
      RDELX(J)=ABS(132._JPRB-0.0_JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(33._JPRB+66._JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'REUNION05') THEN
      NLAT(J)=61
      NLON(J)=141
      RLONC(J)=65._JPRB
      RLATC(J)=-20._JPRB
      RDELX(J)=ABS(100._JPRB-30._JPRB)/REAL(NLON(J)-1,JPRB)
      RDELY(J)=ABS(-5._JPRB+35._JPRB)/REAL(NLAT(J)-1,JPRB)
    ELSEIF (CFPDOM(J) == 'GLOB05') THEN
      NLAT(J)=361
      NLON(J)=720
      RLONC(J)=179.75_JPRB
      RLATC(J)=0._JPRB
      RDELX(J)=0.5_JPRB
      RDELY(J)=0.5_JPRB
    ELSEIF (CFPDOM(J) == 'ATOUR05') THEN
      NLAT(J)=241
      NLON(J)=401
      RLONC(J)=0._JPRB
      RLATC(J)=20._JPRB
      RDELX(J)=0.5_JPRB
      RDELY(J)=0.5_JPRB
    ELSEIF (CFPDOM(J) == 'ATOURX05') THEN
      NLAT(J)=181
      NLON(J)=401
      RLONC(J)=0._JPRB
      RLATC(J)=35._JPRB
      RDELX(J)=0.5_JPRB
      RDELY(J)=0.5_JPRB
    ELSE
      IF (LELAM) THEN
        NLON(J)=NDLUXG
        NLAT(J)=NDGUXG
!       Find largest LALON domain included
        IF (NARGS /= 0.OR.LFPART2) THEN
          ZLON0=UGEMU(3)
          ZLAT0=UGEMU(4)
          ZLONC=UGEMU(5)
          ZLATC=UGEMU(6)
          ZDELX=UGEMU(7)
          ZDELY=UGEMU(8)
          LLMRT = (INT(UGEMU(1)) == -2)
        ELSE
          ELONC = 10._JPRB
          ELATC = 60._JPRB
          ELON0 = 10._JPRB
          ELAT0 = 60._JPRB
          EDELX = 10000._JPRB
          EDELY = 10000._JPRB
          LMRT  = .FALSE.
          CALL POSNAM(NULNAM,'NEMGEO')
          READ(NULNAM,NEMGEO)
          ZLON0 = ELON0*ZDG2RD
          ZLAT0 = ELAT0*ZDG2RD
          ZLONC = ELONC*ZDG2RD
          ZLATC = ELATC*ZDG2RD
          ZDELX = EDELX
          ZDELY = EDELY
          LLMRT = LMRT
        ENDIF
        IF (.NOT.LRPLANE) THEN
          ELON0 = 0._JPRB  ! forcing init at 0 (latlon), may be wrong values
          ELAT0 = 0._JPRB  ! when reading them in NEMGEO few lines above 
          RLONC(J)=ZLONC
          RLATC(J)=ZLATC
          RDELX(J)=ZDELX*ZRD2DG
          RDELY(J)=ZDELY*ZRD2DG         
        ELSE
          ALLOCATE(ZGLON(NDLUNG:NDLUXG,NDGUNG:NDGUXG))
          ALLOCATE(ZGLAT(NDLUNG:NDLUXG,NDGUNG:NDGUXG))
          ALLOCATE(ZGM  (NDLUNG:NDLUXG,NDGUNG:NDGUXG))
          ALLOCATE(ZNORX(NDLUNG:NDLUXG,NDGUNG:NDGUXG))
          ALLOCATE(ZNORY(NDLUNG:NDLUXG,NDGUNG:NDGUXG))
          IROTEQ=-1_JPIM
          IF (LLMRT) IROTEQ=IROTEQ-1_JPIM
          ZLONR=0._JPRB
          ZLATR=0._JPRB
          ZBETA=0._JPRB
          ISOTRP=0_JPIM
          IGIVO=0_JPIM

          WRITE(NULOUT,*) 'Call EGGX_N by SUFPD'

          CALL EGGX_N(RPI,RA,IROTEQ,ZLONR,ZLATR,ZBETA,ZLON1,ZLAT1,ZLON2,ZLAT2,&
           & ZLON0,ZLAT0,ZRPK,NULOUT,ISOTRP,IGIVO,&
           & ZGLON,ZGLAT,ZGM,ZNORX,ZNORY,NDLUNG,&
           & NDLUXG,NDGUNG,NDGUXG,NDLUNG,NDLUXG,NDGUNG,NDGUXG,&
           & ZDELX,ZDELY,ZLONC,ZLATC)  
          ZLAMAX= ZGLAT(2,NDGUXG-1)
          ZLAMIN= ZGLAT(2,2)
          ZLOMAX= ZGLON(NDLUXG-1,2)
          ZLOMIN= ZGLON(2,2)
          DO JI=2,NDLUXG-1
            ZLAMAX=MIN(ZGLAT(JI,NDGUXG-1),ZLAMAX)
            ZLAMIN=MAX(ZGLAT(JI,1+1),ZLAMIN)
          ENDDO
          DO JI=2,NDGUXG-1
            ZLOMAX=MIN(ZGLON(NDLUXG-1,JI),ZLOMAX)
            ZLOMIN=MAX(ZGLON(1+1,JI),ZLOMIN)
          ENDDO
          RLATN(J)=(ZLAMAX*ZRD2DG)-0.01_JPRB
          RLATS(J)=(ZLAMIN*ZRD2DG)+0.01_JPRB
          RLONW(J)=(ZLOMIN*ZRD2DG)+0.01_JPRB
          RLONE(J)=(ZLOMAX*ZRD2DG)-0.01_JPRB
          IF (RLONE(J) < RLONW(J)) THEN
            RLONW(J)=RLONW(J)-360._JPRB
          ENDIF
          RLONC(J)=0.5_JPRB*(RLONW(J)+RLONE(J))
          RLATC(J)=0.5_JPRB*(RLATN(J)+RLATS(J))
          RDELX(J)=(RLONE(J)-RLONW(J))/REAL(NLON(J)-1,JPRB)
          RDELY(J)=(RLATN(J)-RLATS(J))/REAL(NLAT(J)-1,JPRB)
          DEALLOCATE(ZGLON)
          DEALLOCATE(ZGLAT)
          DEALLOCATE(ZGM)
          DEALLOCATE(ZNORX)
          DEALLOCATE(ZNORY)
        ENDIF
      ELSE
        NLAT(J)=0
        NLON(J)=0
        RLATC(J)=0.0_JPRB
        RLONC(J)=0.0_JPRB
        RDELX(J)=0.0_JPRB
        RDELY(J)=0.0_JPRB
      ENDIF
    ENDIF
  ENDDO
  NFPLUX=0
  NFPGUX=0
ENDIF

!*       2. READ NAMELIST
!           -------------

IF (CFPFMT /= 'MODEL') THEN
  CALL POSNAM(NULNAM,'NAMFPD')
  READ(NULNAM,NAMFPD)
  IF (CFPFMT /= 'LELAM') THEN
    NFPLUX=0
    NFPGUX=0
  ENDIF
ENDIF

!     Calculate corners whenever appropriate
IF (CFPFMT == 'LALON') THEN
  DO J=1,NFPDOM
    RLATS(J)=RLATC(J)-RDELY(J)*0.5_JPRB*REAL(NLAT(J)-1,JPRB)
    RLATN(J)=RLATC(J)+RDELY(J)*0.5_JPRB*REAL(NLAT(J)-1,JPRB)
    RLONW(J)=MOD(RLONC(J)-RDELX(J)*0.5_JPRB*REAL(NLON(J)-1,JPRB),360._JPRB)
    RLONE(J)=MOD(RLONC(J)+RDELX(J)*0.5_JPRB*REAL(NLON(J)-1,JPRB),360._JPRB)
    IF (RLONW(J) >= 360._JPRB) RLONW(J)=RLONW(J)-360._JPRB
    IF (RLONE(J) >= 360._JPRB) RLONE(J)=RLONE(J)-360._JPRB
  ENDDO
ENDIF

!*       3. CHECK VALUES
!           ------------

IERR=0
CL=' '
DO J=1, NFPDOM
  IF ((RLATC(J) < (-90._JPRB-ZEPS)).OR.(RLATC(J) > (90._JPRB+ZEPS))) THEN
    IERR=IERR+1
    WRITE(UNIT=CL,FMT='('' RLATC('',I2,'') = '',F6.1,&
     & '' OUT OF [-90.,90.]'')') J, RLATC(J)  
    WRITE(NULERR,*) CL
  ENDIF
  IF ((RLONC(J) < (-360._JPRB-ZEPS)).OR.(RLONC(J) > (360._JPRB+ZEPS)))THEN
    IERR=IERR+1
    WRITE(UNIT=CL,FMT='('' RLONC('',I2,'') = '',F6.1,&
     & '' OUT OF [-360.,360.]'')') J, RLONC(J)  
    WRITE(NULERR,*) CL
  ENDIF
  IF (CFPFMT == 'LALON') THEN
    IF (RLATS(J) < (-90._JPRB-ZEPS) .OR. RLATS(J) > (90._JPRB+ZEPS)) THEN
      IERR=IERR+1
      WRITE(UNIT=CL,FMT='('' MINIMUM LATITUDE '',F6.1,'' OF DOMAIN '',I2,&
       & '' OUT OF [-90.,90.]'')') RLATS(J), J  
      WRITE(NULERR,*) CL
    ENDIF
    IF (RLATN(J) < (-90._JPRB-ZEPS) .OR. RLATN(J) > (90._JPRB+ZEPS)) THEN
      IERR=IERR+1
      WRITE(UNIT=CL,FMT='('' MAXIMUM LATITUDE '',F6.1,'' OF DOMAIN '',I2,&
       & '' OUT OF [-90.,90.]'')') RLATN(J), J  
      WRITE(NULERR,*) CL
    ENDIF
    IF (RLONW(J) < (-360._JPRB-ZEPS) .OR. RLONW(J) > (360._JPRB+ZEPS)) THEN
      IERR=IERR+1
      WRITE(UNIT=CL,FMT='('' MINIMUM LONGITUDE '',F6.1,'' OF DOMAIN '',I2,&
       & '' OUT OF [-360.,360.]'')') RLONW(J), J  
      WRITE(NULERR,*) CL
    ENDIF
    IF (RLONE(J) < (-360._JPRB-ZEPS) .OR. RLONE(J) > (360._JPRB+ZEPS)) THEN
      IERR=IERR+1
      WRITE(UNIT=CL,FMT='('' MAXIMUM LONGITUDE '',F6.1,'' OF DOMAIN '',I2,&
       & '' OUT OF [-360.,360.]'')') RLONE(J), J  
      WRITE(NULERR,*) CL
    ENDIF
  ENDIF
ENDDO

CL=' '
DO J=1, NFPDOM
  IF (NLAT(J) < 1) THEN
    IERR=IERR+1
    WRITE(UNIT=CL,FMT='('' NLAT('',I2,'') = '',I5,&
     & '' LESS THAN 1 !'')') J,NLAT(J)  
    WRITE(NULERR,*) CL
  ENDIF
  IF (NLON(J) < 1) THEN
    IERR=IERR+1
    WRITE(UNIT=CL,FMT='('' NLON('',I2,'') = '',I5,&
     & '' LESS THAN 1 !'')') J,NLON(J)  
    WRITE(NULERR,*) CL
  ENDIF
ENDDO

CL=' '
IF (CFPFMT == 'LELAM') THEN
  DO J=1, NFPDOM
    IF (NFPGUX > NLAT(J)) THEN
      IERR=IERR+1
      WRITE(UNIT=CL,FMT='(''ERROR : NFPGUX > NLAT('',I2,'')'')') J
      WRITE(NULERR,*) CL
    ENDIF
    IF (NFPLUX > NLON(J)) THEN
      IERR=IERR+1
      WRITE(UNIT=CL,FMT='(''ERROR : NFPLUX > NLON('',I2,'')'')') J
      WRITE(NULERR,*) CL
    ENDIF
  ENDDO
ENDIF

!*       4. PRINT OUT FINAL VALUES
!           ----------------------

IF(LTRACEFP) THEN
  WRITE(UNIT=NULOUT,FMT='('' COMMON YOMFPD'')')
  IF (NFPDOM > 0) THEN
    IF (CFPFMT == 'LALON') THEN
      WRITE(UNIT=NULOUT,FMT='('' &
       & ( J CFPDOM      RLATN   RLATS   RLONW   RLONE  NLAT NLON  RLONC  &
       & RLATC      RDELX     RDELY)'')')  
      DO J=1,NFPDOM
        WRITE(UNIT=NULOUT,FMT='('' ('',I2,1X,A9,4F8.2,2I5,2F8.2,2F10.2'')'')')&
         & J,CFPDOM(J),RLATN(J),RLATS(J),RLONW(J),RLONE(J),NLAT(J),NLON(J),&
         & RLONC(J),RLATC(J),RDELX(J),RDELY(J)  
      ENDDO
    ELSE
      WRITE(UNIT=NULOUT,FMT='('' &
       & ( J CFPDOM      NLAT NLON   RLONC   RLATC      RDELX     RDELY)'')')  
      WRITE(UNIT=NULOUT,FMT='('' ('',I2,1X,A9,2I5,2F8.2,2F10.2'')'')')&
       & (J,CFPDOM(J),NLAT(J),NLON(J),RLONC(J),RLATC(J),RDELX(J),RDELY(J),&
       & J=1,NFPDOM)  
    ENDIF
    WRITE(UNIT=NULOUT,FMT='('' NFPLUX = '',I4,'' NFPGUX = '',I4)') &
     & NFPLUX,NFPGUX  
  ENDIF
ENDIF

IF (IERR > 0) CALL ABOR1('SUFPD: ABOR1 CALLED')

IF (LHOOK) CALL DR_HOOK('SUFPD',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPD
