SUBROUTINE SUFPDYN(KTFPP,KTFPH,KTFPTH,KTFPPV,KTFPS,K2FPB,&
 & KOFPP,KOFPH,KOFPTH,KOFPPV,KOFPS,&
 & KAFPP,KAFPH,KAFPT,KAFPV,KAFPS,KA2FP,&
 & KCPFP,KCHFP,KCTFP,KCVFP,KCSFP,KC2FP,&
 & KPFP,KHFP,KTFP,KVFP,KSFP,&
 & KIPFP,KIHFP,KITFP,KIVFP,KISFP,KI2FP,&
 & LDNOFIL,CDYFILE)  

!**** *SUFPDYN*  - INITIALIZE REQUESTS ON DYNAMICS

!     PURPOSE.
!     --------
!        INITIALIZE DYNAMIC REQUESTS OF POST-PROCESSING AT A GIVEN TIME STEP.
!        IN THIS ROUTINE, WE CONSIDER THAT FOR A GIVEN LEVEL TYPE, ALL THE
!        FIELDS ARE POST-PROCESSED AT ALL LEVELS AND FOR ALL SUBDOMAINS.

!**   INTERFACE.
!     ----------
!       *CALL* *SUFPDYN*

!        EXPLICIT ARGUMENTS
!        --------------------
!           KTFPP  : total number of fields in P levels
!           KTFPH  : total number of fields in H levels
!           KTFPTH : total number of fields in THETA levels
!           KTFPPV : total number of fields in PV levels
!           KTFPS  : total number of fields in eta levels

!           K2FPB : total number of 2D fields (dynamics)

!           KOFPP  : total number of P levels for each field
!           KOFPH  : total number of H levels for each field
!           KOFPTH : total number of THETA levels for each field
!           KOFPPV : total number of PV levels for each field
!           KOFPS  : total number of eta levels for each field

!           KAFPP  : total number of subdomains for each P level of each field
!           KAFPH  : total number of subdomains for each H level of each field
!           KAFPT  : total number of subdomains for each TH level of each field
!           KAFPV  : total number of subdomains for each PV level of each field
!           KAFPS  : total number of subdomains for each eta level of each field

!           KA2FP : total number of subdomains for each 2D field

!           KCPFP : field pointers in P levels
!           KCHFP : field pointers in H levels
!           KCTFP : field pointers in THETA levels
!           KCVFP : field pointers in PV levels
!           KCSFP : field pointers in eta levels

!           KC2FP : 2-D field pointers

!           KPFP : P levels per field
!           KHFP : H levels per field
!           KTFP : THETA levels per field
!           KVFP : PV levels per field
!           KSFP : eta levels per field

!           KIPFP : subdomain index for each P level of each field
!           KIHFP : subdomain index for each H level of each field
!           KITFP : subdomain index for each THETA level of each field
!           KIVFP : subdomain index for each PV level of each field
!           KISFP : subdomain index for each eta level of each field

!           KI2FP : subdomain index for each 2D field

!           LDNOFIL : .TRUE. if no control file
!           CDYFILE : control file name

!        IMPLICIT ARGUMENTS
!        --------------------
!        NONE.

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        K. YESSAD (97-09-03): cleaning for cy18. Removal of variables
!          M1NAM3 and M1NAM2 (which disappear), replace them by TFP_DYNDS%IGRIB
!          and TFP_DYNDS%IGRIB. Consequence of the removal of the instructions
!          'EQUIVALENCE' in YOMAFN. 
!        D.Dent    (99-05-06): allow different field code lists
!        R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Bogatchev : 26-Oct-2007 removing of severe breaks of coding rools
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULNAM   ,NULERR
USE PARFPOS  , ONLY : JPOSDOM  ,JPOSLEN  ,JPOSLE   ,JPOSDYN  ,JPOS2DF  ,JPOS3DF
USE YOMAFN   , ONLY : TFP_DYNDS
USE YOMFPC   , ONLY : CFPDOM   ,CFP2DF   ,C1FP3DF  ,&
 & CFP3DFS  ,CFP3DFP  ,CFP3DFH  ,CFP3DFT  ,CFP3DFV  ,&
 & C1FP2DF  ,MFP2DF   ,NFPDOM   ,&
 & MFP3DFS  ,MFP3DFP  ,MFP3DFH  ,MFP3DFT  ,MFP3DFV  ,&
 & NFP3DFS  ,NFP3DFP  ,NFP3DFH  ,NFP3DFT  ,NFP3DFV  ,&
 & NFP2DF   ,NFP3P    ,NFP3H    ,NFP3TH   ,NFP3PV   ,&
 & NFP3S  
USE YOMCT0B  , ONLY : LECMWF

IMPLICIT NONE

!     EXTERNAL INTEGER FUNCTIONS
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTFPP 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTFPH 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTFPTH 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTFPPV 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTFPS 
INTEGER(KIND=JPIM),INTENT(OUT)   :: K2FPB 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KOFPP(NFP3DFP) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KOFPH(NFP3DFH) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KOFPTH(NFP3DFT) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KOFPPV(NFP3DFV) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KOFPS(NFP3DFS) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KAFPP(NFP3P,NFP3DFP) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KAFPH(NFP3H,NFP3DFH) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KAFPT(NFP3TH,NFP3DFT) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KAFPV(NFP3PV,NFP3DFV) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KAFPS(NFP3S,NFP3DFS) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KA2FP(NFP2DF) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCPFP(NFP3DFP) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCHFP(NFP3DFH) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCTFP(NFP3DFT) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCVFP(NFP3DFV) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCSFP(NFP3DFS) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KC2FP(NFP2DF) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KPFP(NFP3P,NFP3DFP) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KHFP(NFP3H,NFP3DFH) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTFP(NFP3TH,NFP3DFT) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KVFP(NFP3PV,NFP3DFV) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KSFP(NFP3S,NFP3DFS) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KIPFP(NFPDOM,NFP3P,NFP3DFP) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KIHFP(NFPDOM,NFP3H,NFP3DFH) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KITFP(NFPDOM,NFP3TH,NFP3DFT) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KIVFP(NFPDOM,NFP3PV,NFP3DFV) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KISFP(NFPDOM,NFP3S,NFP3DFS) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KI2FP(NFPDOM,NFP2DF) 
LOGICAL           ,INTENT(IN)    :: LDNOFIL 
CHARACTER(LEN=120),INTENT(IN)    :: CDYFILE 
INTEGER(KIND=JPIM), EXTERNAL :: ISRCHEQ

CHARACTER (LEN = 39) ::  CLNOTE
CHARACTER (LEN = 16) ::  CL2DF(JPOS2DF)
CHARACTER (LEN=(JPOSDOM*(JPOSLEN+1))-1) :: CLD2DF(JPOS2DF)
NAMELIST/NAMFPDY2/CL2DF,CLD2DF
CHARACTER (LEN = 12) ::  CL3DF(JPOS3DF)
INTEGER(KIND=JPIM) :: IL3DF(JPOSLE,JPOS3DF), ILEV3(JPOS3DF)
CHARACTER (LEN=(JPOSDOM*(JPOSLEN+1))-1) :: CLD3DF(JPOSLE,JPOS3DF)

INTEGER(KIND=JPIM) :: I, I2DF, I3DF, IDIM, IERR, IXD, J, JDOM,&
 & JF, JFLD, JL, JLEV, JSCAN  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "sufpdom.intfb.h"

#include "namfpdyp.h"
#include "namfpdyh.h"
#include "namfpdyt.h"
#include "namfpdyv.h"
#include "namfpdys.h"

!*       1. INITIALIZE REQUESTS IF NO POST-PROCESSING CONTROL FILE
!           ------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPDYN',0,ZHOOK_HANDLE)
IXD=JPOSDOM

IF (LECMWF) THEN
! Prepare internal codes list from grib numbers : 
  
ENDIF

!        Post-processing on P levels :
KTFPP=NFP3DFP
DO JFLD=1,KTFPP
  IF (LECMWF) THEN
    KCPFP(JFLD)=ISRCHEQ(JPOSDYN,TFP_DYNDS%IGRIB,1,MFP3DFP(JFLD))
  ELSE
    DO J=1,JPOSDYN
      IF (CFP3DFP(JFLD) == TFP_DYNDS(J)%CLNAME(1:12)) EXIT
    ENDDO
    IF (J > JPOSDYN) THEN
      CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
    ELSE
      KCPFP(JFLD)=J
    ENDIF
  ENDIF
  KOFPP(JFLD)=NFP3P
ENDDO
DO JFLD=1,KTFPP
  DO JLEV=1,KOFPP(JFLD)
    KPFP(JLEV,JFLD)=JLEV
    KAFPP(JLEV,JFLD)=NFPDOM
  ENDDO
ENDDO
DO JFLD=1,KTFPP
  DO JLEV=1,KOFPP(JFLD)
    DO JDOM=1,KAFPP(JLEV,JFLD)
      KIPFP(JDOM,JLEV,JFLD)=JDOM
    ENDDO
  ENDDO
ENDDO

!        Post-processing on H levels :
KTFPH=NFP3DFH
DO JFLD=1,KTFPH
  IF (LECMWF) THEN
    KCHFP(JFLD)=ISRCHEQ(JPOSDYN,TFP_DYNDS%IGRIB,1,MFP3DFH(JFLD))
  ELSE
    DO J=1,JPOSDYN
      IF (CFP3DFH(JFLD) == TFP_DYNDS(J)%CLNAME(1:12)) EXIT
    ENDDO
    IF (J > JPOSDYN) THEN
      CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
    ELSE
      KCHFP(JFLD)=J
    ENDIF
  ENDIF
  KOFPH(JFLD)=NFP3H
ENDDO
DO JFLD=1,KTFPH
  DO JLEV=1,KOFPH(JFLD)
    KHFP(JLEV,JFLD)=JLEV
    KAFPH(JLEV,JFLD)=NFPDOM
  ENDDO
ENDDO
DO JFLD=1,KTFPH
  DO JLEV=1,KOFPH(JFLD)
    DO JDOM=1,KAFPH(JLEV,JFLD)
      KIHFP(JDOM,JLEV,JFLD)=JDOM
    ENDDO
  ENDDO
ENDDO

!        Post-processing on TH levels :
KTFPTH=NFP3DFT
DO JFLD=1,KTFPTH
  IF (LECMWF) THEN
    KCTFP(JFLD)=ISRCHEQ(JPOSDYN,TFP_DYNDS%IGRIB,1,MFP3DFT(JFLD))
  ELSE
    DO J=1,JPOSDYN
      IF (CFP3DFT(JFLD) == TFP_DYNDS(J)%CLNAME(1:12)) EXIT
    ENDDO
    IF (J > JPOSDYN) THEN
      CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
    ELSE
      KCTFP(JFLD)=J
    ENDIF
  ENDIF
  KOFPTH(JFLD)=NFP3TH
ENDDO
DO JFLD=1,KTFPTH
  DO JLEV=1,KOFPTH(JFLD)
    KTFP(JLEV,JFLD)=JLEV
    KAFPT(JLEV,JFLD)=NFPDOM
  ENDDO
ENDDO
DO JFLD=1,KTFPTH
  DO JLEV=1,KOFPTH(JFLD)
    DO JDOM=1,KAFPT(JLEV,JFLD)
      KITFP(JDOM,JLEV,JFLD)=JDOM
    ENDDO
  ENDDO
ENDDO

!        Post-processing on PV levels :
KTFPPV=NFP3DFV
DO JFLD=1,KTFPPV
  IF (LECMWF) THEN
    KCVFP(JFLD)=ISRCHEQ(JPOSDYN,TFP_DYNDS%IGRIB,1,MFP3DFV(JFLD))
  ELSE
    DO J=1,JPOSDYN
      IF (CFP3DFV(JFLD) == TFP_DYNDS(J)%CLNAME(1:12)) EXIT
    ENDDO
    IF (J > JPOSDYN) THEN
      CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
    ELSE
      KCVFP(JFLD)=J
    ENDIF
  ENDIF
  KOFPPV(JFLD)=NFP3PV
ENDDO
DO JFLD=1,KTFPPV
  DO JLEV=1,KOFPPV(JFLD)
    KVFP(JLEV,JFLD)=JLEV
    KAFPV(JLEV,JFLD)=NFPDOM
  ENDDO
ENDDO
DO JFLD=1,KTFPPV
  DO JLEV=1,KOFPPV(JFLD)
    DO JDOM=1,KAFPV(JLEV,JFLD)
      KIVFP(JDOM,JLEV,JFLD)=JDOM
    ENDDO
  ENDDO
ENDDO

!        Post-processing on eta levels :
KTFPS=NFP3DFS
DO JFLD=1,KTFPS
  IF (LECMWF) THEN
    KCSFP(JFLD)=ISRCHEQ(JPOSDYN,TFP_DYNDS%IGRIB,1,MFP3DFS(JFLD))
  ELSE
    DO J=1,JPOSDYN
      IF (CFP3DFS(JFLD) == TFP_DYNDS(J)%CLNAME(1:12)) EXIT
    ENDDO
    IF (J > JPOSDYN) THEN
      CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
    ELSE
      KCSFP(JFLD)=J
    ENDIF
  ENDIF
  KOFPS(JFLD)=NFP3S
ENDDO
DO JFLD=1,KTFPS
  DO JLEV=1,KOFPS(JFLD)
    KSFP(JLEV,JFLD)=JLEV
    KAFPS(JLEV,JFLD)=NFPDOM
  ENDDO
ENDDO
DO JFLD=1,KTFPS
  DO JLEV=1,KOFPS(JFLD)
    DO JDOM=1,KAFPS(JLEV,JFLD)
      KISFP(JDOM,JLEV,JFLD)=JDOM
    ENDDO
  ENDDO
ENDDO

!        Post-processing on 2D fields :
K2FPB=NFP2DF
DO JFLD=1,K2FPB
  IF (LECMWF) THEN
    DO J=JPOSDYN,1,-1
      IF(TFP_DYNDS(J)%IGRIB == MFP2DF(JFLD)) EXIT
    ENDDO
    IF (J > JPOSDYN) THEN
      CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
    ELSE
      KC2FP(JFLD)=J
    ENDIF
  ELSE
    DO J=1,JPOSDYN
      IF (CFP2DF(JFLD) == TFP_DYNDS(J)%CLNAME(1:16)) EXIT
    ENDDO
    IF (J > JPOSDYN) THEN
      CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
    ELSE
      KC2FP(JFLD)=J
    ENDIF
  ENDIF
  KA2FP(JFLD)=NFPDOM
ENDDO
DO JFLD=1,K2FPB
  DO JDOM=1,KA2FP(JFLD)
    KI2FP(JDOM,JFLD)=JDOM
  ENDDO
ENDDO

!*       2. MODIFY REQUESTS IF CONTROL FILE EXISTS
!           --------------------------------------

IF (.NOT.LDNOFIL) THEN

  CLOSE(NULNAM)
  OPEN(NULNAM,FILE=CDYFILE)

!        Post-processing on 2D fields :

  DO J=1,JPOS2DF
    CL2DF(J)=' '
    CLD2DF(J)=' '
  ENDDO

  CALL POSNAM(NULNAM,'NAMFPDY2')
  READ(NULNAM,NAMFPDY2)

  DO J=1, JPOS2DF
    IF (CL2DF(J) == ' ') GO TO 213
  ENDDO
  213 CONTINUE
  I2DF=J-1

  IERR=0
  DO J=1, I2DF
    I=INDEX(C1FP2DF,CL2DF(J))
    IF (I == 0) THEN
      WRITE(UNIT=CLNOTE,FMT='('' UNKNOWN FIELD '',A16)') CL2DF(J)
      WRITE(NULERR,*) CLNOTE
      IERR=IERR+1
    ENDIF
  ENDDO
  IF (IERR > 0) CALL ABOR1('SUFPDYN: ABOR1 CALLED')

  IF(I2DF /= 0) K2FPB=I2DF
  DO JFLD=1,I2DF
    DO J=1,JPOSDYN
      IF (CL2DF(JFLD) == TFP_DYNDS(J)%CLNAME(1:16)) EXIT
    ENDDO
    IF (J > JPOSDYN) THEN
      CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
    ELSE
      KC2FP(JFLD)=J
    ENDIF
  ENDDO
  IF (I2DF > 0) THEN
    IDIM=JPOSDYN
    CALL SUFPDOM(CFPDOM,IXD,NFPDOM,CLD2DF,IDIM,I2DF,KA2FP,KI2FP)
  ENDIF

!        Post-processing on 3D fields :
  DO JSCAN=1,5

    DO J=1,JPOS3DF
      CL3DF(J)=' '
    ENDDO
    DO JF=1,JPOS3DF
      DO JL=1,JPOSLE
        IL3DF(JL,JF)=0
        CLD3DF(JL,JF)=' '
      ENDDO
    ENDDO

    IF (JSCAN == 1) THEN
      CALL POSNAM(NULNAM,'NAMFPDYP')
      READ(NULNAM,NAMFPDYP)
    ELSEIF (JSCAN == 2) THEN
      CALL POSNAM(NULNAM,'NAMFPDYH')
      READ(NULNAM,NAMFPDYH)
    ELSEIF (JSCAN == 3) THEN
      CALL POSNAM(NULNAM,'NAMFPDYT')
      READ(NULNAM,NAMFPDYT)
    ELSEIF (JSCAN == 4) THEN
      CALL POSNAM(NULNAM,'NAMFPDYV')
      READ(NULNAM,NAMFPDYV)
    ELSEIF (JSCAN == 5) THEN
      CALL POSNAM(NULNAM,'NAMFPDYS')
      READ(NULNAM,NAMFPDYS)
    ENDIF

    DO J=1, JPOS3DF
      IF (CL3DF(J) == ' ') GO TO 2232
    ENDDO
    2232 CONTINUE
    I3DF=J-1

    DO JF=1, I3DF
      DO JL=1, JPOSLE
        IF (IL3DF(JL,JF) <= 0) GO TO 2243
      ENDDO
      2243 CONTINUE
      ILEV3(JF)=JL-1
    ENDDO

    IERR=0
    DO J=1, I3DF

      I=INDEX(C1FP3DF,CL3DF(J))
      IF (I == 0) THEN
        WRITE(UNIT=CLNOTE,FMT='('' UNKNOWN FIELD '',A12)') CL3DF(J)
        WRITE(NULERR,*) CLNOTE
        IERR=IERR+1
      ENDIF
      IF (JSCAN == 1) THEN
        IF (ILEV3(J) > NFP3P) THEN
          WRITE(UNIT=CLNOTE,FMT='('' TOO MUCH PRESSURE LEVELS '')')
          WRITE(NULERR,*) CLNOTE
          IERR=IERR+1
        ENDIF
        DO JL=1,ILEV3(J)
          IF (IL3DF(JL,J) > NFP3P) THEN
            WRITE(UNIT=CLNOTE,FMT='(''PRESSURE INDEX RANGE ERROR'')')
            WRITE(NULERR,*) CLNOTE
            IERR=IERR+1
          ENDIF
        ENDDO
      ELSEIF (JSCAN == 2) THEN
        IF (ILEV3(J) > NFP3H) THEN
          WRITE(UNIT=CLNOTE,FMT='('' TOO MUCH HEIGHT LEVELS '')')
          WRITE(NULERR,*) CLNOTE
          IERR=IERR+1
        ENDIF
        DO JL=1,ILEV3(J)
          IF (IL3DF(JL,J) > NFP3H) THEN
            WRITE(UNIT=CLNOTE,FMT='('' HEIGHT INDEX RANGE ERROR '')')
            WRITE(NULERR,*) CLNOTE
            IERR=IERR+1
          ENDIF
        ENDDO
      ELSEIF (JSCAN == 3) THEN
        IF (ILEV3(J) > NFP3TH) THEN
          WRITE(UNIT=CLNOTE,FMT='('' TOO MUCH THETA LEVELS '')')
          WRITE(NULERR,*) CLNOTE
          IERR=IERR+1
        ENDIF
        DO JL=1,ILEV3(J)
          IF (IL3DF(JL,J) > NFP3TH) THEN
            WRITE(UNIT=CLNOTE,FMT='('' THETA INDEX RANGE ERROR '')')
            WRITE(NULERR,*) CLNOTE
            IERR=IERR+1
          ENDIF
        ENDDO
      ELSEIF (JSCAN == 4) THEN
        IF (ILEV3(J) > NFP3PV) THEN
          WRITE(UNIT=CLNOTE,FMT='('' TOO MUCH PV LEVELS '')')
          WRITE(NULERR,*) CLNOTE
          IERR=IERR+1
        ENDIF
        DO JL=1,ILEV3(J)
          IF (IL3DF(JL,J) > NFP3PV) THEN
            WRITE(UNIT=CLNOTE,FMT='('' PV INDEX RANGE ERROR '')')
            WRITE(NULERR,*) CLNOTE
            IERR=IERR+1
          ENDIF
        ENDDO
      ELSEIF (JSCAN == 5) THEN
        IF (ILEV3(J) > NFP3S) THEN
          WRITE(UNIT=CLNOTE,FMT='('' TOO MUCH ETA LEVELS '')')
          WRITE(NULERR,*) CLNOTE
          IERR=IERR+1
        ENDIF
        DO JL=1,ILEV3(J)
          IF (IL3DF(JL,J) > NFP3S) THEN
            WRITE(UNIT=CLNOTE,FMT='('' ETA INDEX RANGE ERROR '')')
            WRITE(NULERR,*) CLNOTE
            IERR=IERR+1
          ENDIF
        ENDDO
      ENDIF

    ENDDO
    IF (IERR > 0) CALL ABOR1('SUFPDYN: ABOR1 CALLED')

    IDIM=JPOSLE
    IF (JSCAN == 1) THEN
      IF(I3DF /= 0) KTFPP=I3DF
      DO JFLD=1,I3DF
        DO J=1,JPOSDYN
          IF (CL3DF(JFLD) == TFP_DYNDS(J)%CLNAME(1:12)) EXIT
        ENDDO
        IF (J > JPOSDYN) THEN
          CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
        ELSE
          KCPFP(JFLD)=J
        ENDIF
        KOFPP(JFLD)=ILEV3(JFLD)
        IF (KOFPP(JFLD) > 0) THEN
          CALL SUFPDOM(CFPDOM,IXD,NFPDOM,CLD3DF(1,JFLD),IDIM,&
           & KOFPP(JFLD),KAFPP(1,JFLD),KIPFP(1,1,JFLD))  
        ENDIF
        DO JLEV=1,KOFPP(JFLD)
          KPFP(JLEV,JFLD)=IL3DF(JLEV,JFLD)
        ENDDO
      ENDDO
    ELSEIF (JSCAN == 2) THEN
      IF(I3DF /= 0) KTFPH=I3DF
      DO JFLD=1,I3DF
        DO J=1,JPOSDYN
          IF (CL3DF(JFLD) == TFP_DYNDS(J)%CLNAME(1:12)) EXIT
        ENDDO
        IF (J > JPOSDYN) THEN
          CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
        ELSE
          KCHFP(JFLD)=J
        ENDIF
        KOFPH(JFLD)=ILEV3(JFLD)
        IF (KOFPH(JFLD) > 0) THEN
          CALL SUFPDOM(CFPDOM,IXD,NFPDOM,CLD3DF(1,JFLD),IDIM,&
           & KOFPH(JFLD),KAFPH(1,JFLD),KIHFP(1,1,JFLD))  
        ENDIF
        DO JLEV=1,KOFPH(JFLD)
          KHFP(JLEV,JFLD)=IL3DF(JLEV,JFLD)
        ENDDO
      ENDDO
    ELSEIF (JSCAN == 3) THEN
      IF(I3DF /= 0) KTFPTH=I3DF
      DO JFLD=1,I3DF
        DO J=1,JPOSDYN
          IF (CL3DF(JFLD) == TFP_DYNDS(J)%CLNAME(1:12)) EXIT
        ENDDO
        IF (J > JPOSDYN) THEN
          CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
        ELSE
          KCTFP(JFLD)=J
        ENDIF
        KOFPTH(JFLD)=ILEV3(JFLD)
        IF (KOFPTH(JFLD) > 0) THEN
          CALL SUFPDOM(CFPDOM,IXD,NFPDOM,CLD3DF(1,JFLD),IDIM,&
           & KOFPTH(JFLD),KAFPT(1,JFLD),KITFP(1,1,JFLD))  
        ENDIF
        DO JLEV=1,KOFPTH(JFLD)
          KTFP(JLEV,JFLD)=IL3DF(JLEV,JFLD)
        ENDDO
      ENDDO
    ELSEIF (JSCAN == 4) THEN
      IF(I3DF /= 0) KTFPPV=I3DF
      DO JFLD=1,I3DF
        DO J=1,JPOSDYN
          IF (CL3DF(JFLD) == TFP_DYNDS(J)%CLNAME(1:12)) EXIT
        ENDDO
        IF (J > JPOSDYN) THEN
          CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
        ELSE
          KCVFP(JFLD)=J
        ENDIF
        KOFPPV(JFLD)=ILEV3(JFLD)
        IF (KOFPPV(JFLD) > 0) THEN
          CALL SUFPDOM(CFPDOM,IXD,NFPDOM,CLD3DF(1,JFLD),IDIM,&
           & KOFPPV(JFLD),KAFPV(1,JFLD),KIVFP(1,1,JFLD))  
        ENDIF
        DO JLEV=1,KOFPPV(JFLD)
          KVFP(JLEV,JFLD)=IL3DF(JLEV,JFLD)
        ENDDO
      ENDDO
    ELSEIF (JSCAN == 5) THEN
      IF(I3DF /= 0) KTFPS=I3DF
      DO JFLD=1,I3DF
        DO J=1,JPOSDYN
          IF (CL3DF(JFLD) == TFP_DYNDS(J)%CLNAME(1:12)) EXIT
        ENDDO
        IF (J > JPOSDYN) THEN
          CALL ABOR1('SUFPDYN : UNKNOWN FIELD ')
        ELSE
          KCSFP(JFLD)=J
        ENDIF
        KOFPS(JFLD)=ILEV3(JFLD)
        IF (KOFPS(JFLD) > 0) THEN
          CALL SUFPDOM(CFPDOM,IXD,NFPDOM,CLD3DF(1,JFLD),IDIM,&
           & KOFPS(JFLD),KAFPS(1,JFLD),KISFP(1,1,JFLD))  
        ENDIF
        DO JLEV=1,KOFPS(JFLD)
          KSFP(JLEV,JFLD)=IL3DF(JLEV,JFLD)
        ENDDO
      ENDDO
    ENDIF

  ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('SUFPDYN',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPDYN
