SUBROUTINE SUFPSC2_DEP

!**** *SUFPSC2_DEP*  - FULL-POS HORIZONTAL SCANNING

!     PURPOSE.
!     --------
!        TO SET UP MODULE YOMFPSC2 WHICH CONTAINS THE CONTROL PARAMETERS AND
!        ARRAYS TO SCAN THE POST-PROCESSING BUFFERS PER ROW
!        DM-features linked to departure geometry.

!        The following variables (in YOMFPSC2) are computed:
!         NFPROMA_DEP
!         NFPBLOCS_DEP, NFPSTART_DEP, NFPEND_DEP, NFPSORT_DEP, NFPBLOFF_DEP

!**   INTERFACE.
!     ----------
!       *CALL* *SUFPSC2_DEP*

!        EXPLICIT ARGUMENTS
!        ------------------
!        NONE.

!        IMPLICIT ARGUMENTS
!        --------------------
!        See modules below.

!     METHOD.
!     -------
!        SEE DOCUMENTATION ABOUT FULL-POS

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Documentation about FULL-POS.

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE* (SUFPSC2)
!        K. Yessad (rename into SUFPSC2_DEP)

!     MODIFICATIONS.
!     --------------
!      G. Radnoti   : 95-01-23   full pos of aladin
!      R. El Khatib : 95-11-08   initialisation of NFPROMA
!      Modified by K. YESSAD (MARCH 1997): 
!          - parallelisation of FULL-POS.
!      Modified by N. WEDI (SUMMER 1997): corrections of bugs in the DM code.
!      Modified by V. SARAVANE (OCT 1997): some optimisations.
!      Modified : ??-??-?? J.Latour ?? tidied up by M.Hamrud
!      R. El Khatib : 98-07-29 Unification SM/DM ; communicate row length
!      R. El Khatib : 98-10-06 LFPOSBUF ; SUFPSORT ; reunification of scanning
!      R. El Khatib : 03-01-29 Remove SM aspects/cleanings
!      R. El Khatib : 03-04-17 Fullpos improvments
!      M.Hamrud     : 01-Oct-2003 CY28 Cleaning
!      R. El Khatib : 23-may-2005 NFPBLOFF
!      M. Jidane    : 19-04-2006  Correction of a bug in allocation of NFPBLOFF
!      K. Yessad    : 27-Feb-2007 Rename into SUFPSC2_DEP, adapt, clean.
!      R. El Khatib : 17-Sep-2007 Proper default value for NFPROMA_DEP if the default is negative
!-----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   ,NULNAM
USE YOMCT0   , ONLY : NPROC    ,NPRINTLEV
USE YOMDIM   , ONLY : NPROMA
USE YOMFPG   , ONLY : NFPRGPL_DEP,NFPRGPG  ,LFPOSBUF
USE YOMFPSC2 , ONLY : NFPBLOCS_DEP,NFPROMA_DEP,NFPSTART_DEP,NFPEND_DEP, &
 & NFPSORT_DEP,NFPBLOFF_DEP
USE YOMMP    , ONLY : MYPROC
USE YOMTAG   , ONLY : MTAGPART
USE YOMFPC   , ONLY : LTRACEFP
USE MPL_MODULE, ONLY : MPL_BROADCAST

!     ------------------------------------------------------------------

IMPLICIT NONE

! Rows lengths for each processor
INTEGER(KIND=JPIM) :: ILENROWS(NPROC)
! Number of active/idle proc. for pp. work
INTEGER(KIND=JPIM) :: IACTIVG(1), IIDLEG(1)
! Proc. of the smallest/biggest pp. row
INTEGER(KIND=JPIM) :: IPROCN(1), IPROCX(1)

! Size of the smallest/biggest pp. row 
INTEGER(KIND=JPIM) :: ISMALLG, IBIGG
! Mean size of active pp. rows
INTEGER(KIND=JPIM) :: IMEAN

INTEGER(KIND=JPIM) :: IBLOC, IPROMA, IRANK, ITAG, J, JROC, IPROC

REAL(KIND=JPRB) :: ZFILL(NPROC) ! Buffer fill-up ratio
REAL(KIND=JPRB) :: ZWORK(NPROC) ! Working ratio
REAL(KIND=JPRB) :: ZNUME(NPROC) ! For more statistics : numerator
REAL(KIND=JPRB) :: ZDENO(NPROC) ! For more statistics : denominator

REAL(KIND=JPRB) :: ZRFPRGPG     ! 1/NFPRGPG

CHARACTER(LEN=39) :: CLFMT='(1X,''ARRAY '',A13,'' ALLOCATED '',8I8)'
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "namfpsc2_dep.h"

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "posnam.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPSC2_DEP',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1. COMPUTES SEGMENTATION 
!           ---------------------

SELECT CASE (LFPOSBUF)

CASE (.TRUE.)

  !      1.1 Statistics on the lengths of non-empty output rows

  ! Preinitialisation :
  ILENROWS(:)=-999

  ! Fill my own row length :
  ILENROWS(MYPROC)=NFPRGPL_DEP

  ! Broadcast my row length / gather the other rows lengths :
  SELECT CASE (NPROC)
  CASE (2:)
    DO JROC=1,NPROC
      ITAG=MTAGPART+NPROC+JROC
      IPROC=JROC
      CALL MPL_BROADCAST(ILENROWS(IPROC:IPROC),KROOT=IPROC,KTAG=ITAG, &
       & CDSTRING='SUFPSC2_DEP')
    ENDDO
  CASE(1)
    CONTINUE
  CASE DEFAULT
    CALL ABOR1('SUFPSC2_DEP:ILLEGAL VALUE NPROC')
  END SELECT   

  ! Number of active/idle processors :
  IIDLEG(1)=COUNT(ILENROWS==0)
  IACTIVG(1)=COUNT(ILENROWS>0)
  IF (IIDLEG(1)+IACTIVG(1) /= NPROC) THEN
    CALL ABOR1('SUFPSC2_DEP:ILLEGAL VALUE ILENROWS')
  ENDIF

  ! Smallest and biggest rows :
  IPROCN=MINLOC(ILENROWS)
  IPROCX=MAXLOC(ILENROWS)
  ISMALLG=ILENROWS(IPROCN(1))
  IBIGG=ILENROWS(IPROCX(1))

  ! Mean value of active row lengths :
  IMEAN=NINT(REAL(NFPRGPG,JPRB)/REAL(IACTIVG(1),JPRB))

  IF (LTRACEFP) THEN
    WRITE(UNIT=NULOUT,FMT='(A)') &
     & ' SUFPSC2_DEP : GLOBAL STATISTICS FOR ALL POST-PROCESSED POINTS :'  
    WRITE(UNIT=NULOUT,FMT='('' MINIMUM SIZE OF ROWS = '',I6, &
     & '' PROC # '',I4)') ISMALLG,IPROCN  
    WRITE(UNIT=NULOUT,FMT='('' MAXIMUM SIZE OF ROWS = '',I6, &
     & '' PROC # '',I4)') IBIGG,IPROCX  
    WRITE(UNIT=NULOUT,FMT='('' MEAN SIZE OF ROWS '',I6)') IMEAN
    WRITE(UNIT=NULOUT,FMT='('' NUMBER OF IDLE PROCESSORS = '',I3)') IIDLEG(1)
  ENDIF

  !      1.2 Read namelist & control

  NFPROMA_DEP=IMEAN
  CALL POSNAM(NULNAM,'NAMFPSC2_DEP')
  READ(NULNAM,NAMFPSC2_DEP)

  IF (NFPROMA_DEP < 0) THEN
!   Keep the absolute value
    NFPROMA_DEP=ABS(NFPROMA_DEP)
  ELSEIF (NFPROMA_DEP > 0) THEN
!   Ensure that the value is odd. We could make it more clever later (a divider of the number of blocks).
    IPROMA=MIN(NFPROMA_DEP,NFPRGPL_DEP)
    NFPROMA_DEP=MAX(1,IPROMA+1-MOD(IPROMA,2))
  ELSE
!   What a strange value. Keep the default calculation as the mean value of blocks.
    IPROMA=MIN(IMEAN,NFPRGPL_DEP)
    NFPROMA_DEP=MAX(1,IPROMA+1-MOD(IPROMA,2))
  ENDIF

CASE DEFAULT

  ! Use model-shaped buffer segmentation :
  NFPROMA_DEP=NPROMA

END SELECT

!     ------------------------------------------------------------------

!*       2. COMPUTES NUMBER OF BLOCS AND RELATED CONTROL ARRAYS
!           ---------------------------------------------------

SELECT CASE (NFPRGPL_DEP)
CASE (1:)
  NFPBLOCS_DEP=(NFPRGPL_DEP-1)/NFPROMA_DEP +1
CASE (0)
  NFPBLOCS_DEP=0
CASE DEFAULT
  CALL ABOR1('SUFPSC2_DEP:ILLEGAL VALUE NFPRGPL_DEP')
END SELECT

IF (ALLOCATED(NFPSTART_DEP)) DEALLOCATE(NFPSTART_DEP)
ALLOCATE(NFPSTART_DEP(NFPBLOCS_DEP))
IF (ALLOCATED(NFPEND_DEP)) DEALLOCATE(NFPEND_DEP)
ALLOCATE(NFPEND_DEP(NFPBLOCS_DEP))
IF (ALLOCATED(NFPBLOFF_DEP)) DEALLOCATE(NFPBLOFF_DEP)
ALLOCATE(NFPBLOFF_DEP(NFPBLOCS_DEP))
IF (LTRACEFP) THEN
  WRITE(NULOUT,CLFMT) 'NFPSTART_DEP ',SIZE(NFPSTART_DEP),SHAPE(NFPSTART_DEP)
  WRITE(NULOUT,CLFMT) 'NFPEND_DEP   ',SIZE(NFPEND_DEP),SHAPE(NFPEND_DEP)
  WRITE(NULOUT,CLFMT) 'NFPBLOFF_DEP ',SIZE(NFPBLOFF_DEP),SHAPE(NFPBLOFF_DEP)
ENDIF
DO J=1,NFPBLOCS_DEP-1
  NFPSTART_DEP(J)=1
  NFPEND_DEP  (J)=NFPROMA_DEP
  NFPBLOFF_DEP(J)=NFPROMA_DEP*(J-1)
ENDDO
IF (NFPBLOCS_DEP > 0) THEN
  NFPSTART_DEP(NFPBLOCS_DEP)=1
  NFPEND_DEP  (NFPBLOCS_DEP)=NFPRGPL_DEP-NFPROMA_DEP*(NFPBLOCS_DEP-1)
  NFPBLOFF_DEP(NFPBLOCS_DEP)=NFPROMA_DEP*(NFPBLOCS_DEP-1)
ENDIF

IF (ALLOCATED(NFPSORT_DEP)) DEALLOCATE(NFPSORT_DEP)
ALLOCATE(NFPSORT_DEP(NFPROMA_DEP,NFPBLOCS_DEP))
IF (LTRACEFP) THEN
  WRITE(NULOUT,CLFMT) 'NFPSORT_DEP  ',SIZE(NFPSORT_DEP),SHAPE(NFPSORT_DEP)
ENDIF
DO J=1,NFPRGPL_DEP
  IBLOC=(J-1)/NFPROMA_DEP+1
  IRANK=J-(IBLOC-1)*NFPROMA_DEP
  NFPSORT_DEP(IRANK,IBLOC)=J
ENDDO

!     ------------------------------------------------------------------

!*       3.  PRINT OUT FINAL VALUES
!            ----------------------

IF (LTRACEFP) THEN

  IF (LFPOSBUF) THEN
    ! More statistics : buffers fill-up ratio :
    ZRFPRGPG=1.0_JPRB/REAL(NFPRGPG,JPRB)
    DO JROC=1,NPROC
      ZNUME(JROC)=100._JPRB*REAL(ILENROWS(JROC),JPRB)
      ZWORK(JROC)=ZNUME(JROC)*ZRFPRGPG
      ZDENO(JROC)=REAL(((ILENROWS(JROC)-1)/NFPROMA_DEP +1)*NFPROMA_DEP,JPRB)
    ENDDO
    DO JROC=1,NPROC
      SELECT CASE (ILENROWS(JROC))
      CASE (1:)
        ZFILL(JROC)=ZNUME(JROC)/ZDENO(JROC)
      CASE DEFAULT
        ZFILL(JROC)=-999._JPRB
      END SELECT
    ENDDO
    WRITE(NULOUT,*) ' (PROCESSOR  LENGHT  WORKING RATIO(%)  FILL-UP RATIO(%) )'
    WRITE(NULOUT,'('' # '',4I8)') &
     & (JROC,ILENROWS(JROC),NINT(ZWORK(JROC)),NINT(ZFILL(JROC)),JROC=1,NPROC)  
  ENDIF

  WRITE(NULOUT,'(A)') ' '
  WRITE(NULOUT,'('' MODULE YOMFPSC2 '')')
  WRITE(NULOUT,'('' NFPROMA_DEP = '',I6,'' NFPBLOCS_DEP = '',I6)') &
   & NFPROMA_DEP, NFPBLOCS_DEP
  IF (NPRINTLEV > 1) THEN
    WRITE(NULOUT,'('' (IBLOC NFPSTART_DEP NFPEND_DEP) '')')
    WRITE(NULOUT,'(4(1X,''('',I4,2I6,'')''))') &
    & (J,NFPSTART_DEP(J),NFPEND_DEP(J), J=1, NFPBLOCS_DEP)  
  ENDIF

ENDIF

! -------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUFPSC2_DEP',1,ZHOOK_HANDLE)
END SUBROUTINE SUFPSC2_DEP
