SUBROUTINE SUHOW1(KDGSA,KDGEN,KFRSTLOFF,P4JP,PI,KLOEN,PLATIN,&
 & KBINL,PLAT,PLON,KLEN,&
 & KLA,KLON,KLONN,KLOS,KLOSS,&
 & PDLAT,PDLO,PDLO1,PDLO2,PDLO3)  

!**** *SUHOW1* - Computes the coordinates of nearest points of an
!****            interpolation point for FULL-POS. 
!****

!     Purpose.
!     -------
!       Determines the interpolation grid for 12 points interpolations or
!       bilinear interpolations for FULL-POS fields.
!       Numbering of the points (I is the interpolation point):
!                     13       5       6      14

!                      7       1       2       8
!                                 (I)
!                      9       3       4      10

!                     15      11      12      16
!       Points number 1 to 12 are used if 12 points interpolations.
!       Points number 1 to 4 are used if bilinear interpolations.
!       - computation of the latitude and the longitude of the
!         points numbered 13, 7, 9, 15.
!       - computation of linear weights.
!       Storage of coordinates and weights.

!       For distributed memory SUHOW1 is executed locally on each processor
!       (iproca,iprocb).
!       Abbreviation DM-global means 'variable global for all processors'.
!       Abbreviation DM-local means 'variable local for each processor
!        iproca and iprocb'.

!**   Interface.
!**   ---------
!**   CALL SUHOW1(...)
!**
!        Explicit arguments :
!        --------------------

!     INPUT:
!      KDGSA     : Lower DM-locally numbered latitude 'jgl' involved.
!                  (DM-local).
!      KDGEN     : Upper DM-locally numbered latitude 'jgl' involved.
!                  (DM-local).
!      KFRSTLOFF : Difference 'DM-global numbering of latitudes' -
!                  'DM-local numbering of latitudes'
!                  KFRSTLOFF=0 if not distributed memory.
!                  (DM-local).
!      P4JP      : Approximative inverse of the differences of latitudes.
!                  (DM-global).
!      PI        : Constant PI.                                 
!                  (DM-global)
!      KLOEN     : KLOEN(jglg): number of longitudes for the DM-globally 
!                  numbered latitude 'jglg'.
!                  (DM-global, but only elements KDGSA+KFRSTLOFF to 
!                  KDGEN+KFRSTLOFF are provided here).
!      PLATIN    : Gaussian/Lobatto latitudes.
!                  (DM-global, but only elements KDGSA+KFRSTLOFF to
!                  KDGEN+KFRSTLOFF are provided here).
!      KBINL     : Options for choice of interpolations:
!                  = 0 ==> 12 points interpolation
!                  = 1 ==> bilinear interpolation
!                  (DM-global).
!      PLAT      : Latitudes of output grid (in radians).
!                  (DM-local).
!      PLON      : Longitudes of output grid (in radians).
!                  (DM-local).
!      KLEN      : Length of output vector, to which data are interpolated.
!                  (DM-local).

!     OUTPUT:
!      KLA       : Latitude index of the point numbered 1
!                  (DM-local).
!      KLON      : Longitude index of the point numbered 7
!                  (DM-local).
!      KLONN     : Longitude index of the point numbered 13
!                  (DM-local).
!      KLOS      : Longitude index of the point numbered 9
!                  (DM-local).
!      KLOSS     : Longitude index of the point numbered 15
!                  (DM-local).
!      PDLAT     : Meridian linear weight.
!                  (DM-local).
!      PDLO      : Zonal linear weight for the latitude of the point numbered 5.
!                  (DM-local).
!      PDLO1     : Zonal linear weight for the latitude of the point numbered 1.
!                  (DM-local).
!      PDLO2     : Zonal linear weight for the latitude of the point numbered 3.
!                  (DM-local).
!      PDLO3     : Zonal linear weight for the latitude of the point numbered 
!                  11.
!                  (DM-local).

!        Implicit arguments :
!        --------------------
!        none

!     Method.
!     -------
!        See documentation about FULL-POS.

!     Externals.
!     ----------

!        No external.
!        Called by CPCLIMO and SUFPWFPBUF.

!     Reference.
!     ----------

!     Author.
!     -------
!     Author : K. YESSAD after routines SUHOW and LASCAW.
!     Date   : MARCH 1997.

!     Modifications
!     -------------
!        K.YESSAD : (AUGUST 1997):
!          - cleaning for cy18: corrections or removal of false comments.
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!         -> dummy arguments LDPOLMODEL,LDPOLFPOS become always .false. and
!            disappear, dummy argument KDGLG becomes useless and disappears.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!--------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     INPUT:
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGSA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFRSTLOFF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P4JP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOEN(KDGSA+KFRSTLOFF:KDGEN+KFRSTLOFF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLATIN(KDGSA+KFRSTLOFF:KDGEN+KFRSTLOFF) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBINL 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT(KLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON(KLEN) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLA(KLEN) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLON(KLEN) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLONN(KLEN) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLOS(KLEN) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLOSS(KLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLAT(KLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLO(KLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLO1(KLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLO2(KLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDLO3(KLEN) 
!     OUTPUT:
REAL(KIND=JPRB) :: ZLSDEPI(KDGSA:KDGEN)

INTEGER(KIND=JPIM) :: IGLGLO, ILA, ILAG, ILO, ILO1, ILO2, ILO3,&
 & IZLAT, IZLATG, JGL, JLEN  

REAL(KIND=JPRB) :: ZDEPI, ZDLAT, ZDLO, ZDLO1, ZDLO2, ZDLO3,&
 & ZDY, ZLO, ZLO1, ZLO2, ZLO3, ZLOO  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ---------------------------------------------------------------------

!*       0. INITIAL SETTING.
!           ----------------

IF (LHOOK) CALL DR_HOOK('SUHOW1',0,ZHOOK_HANDLE)
ZDEPI=2.0_JPRB*PI

!     Numbering of latitudes is DM-local in ZLSDEPI.
DO JGL=KDGSA,KDGEN
  IGLGLO=JGL+KFRSTLOFF
  ZLSDEPI(JGL)=REAL(KLOEN(IGLGLO),JPRB)/ZDEPI
ENDDO

!     ---------------------------------------------------------------------

!*       1. COORDINATES.
!           ------------

!        Latitudes of points 5,1,3,11 are always contiguous.

DO JLEN = 1,KLEN

  IZLAT =INT(P4JP*(0.5_JPRB*PI-PLAT(JLEN))+0.75_JPRB)-KFRSTLOFF
  IZLATG=IZLAT+KFRSTLOFF
  ILA   =IZLAT+NINT(SIGN(0.5_JPRB,PLATIN(IZLATG)-PLAT(JLEN))-1.5_JPRB)
  ILAG  =ILA+KFRSTLOFF
  ZDY   =PLAT(JLEN)-PLATIN(ILAG+1)
  ZDLAT =ZDY/(PLATIN(ILAG+2)-PLATIN(ILAG+1))
  KLA(JLEN)=ILA+1
  PDLAT(JLEN)=ZDLAT

  ZLOO = MOD(PLON(JLEN),ZDEPI)

  ZLO1  =ZLOO*ZLSDEPI(ILA+1)
  ILO1  =INT(ZLO1)
  ZDLO1 =ZLO1-REAL(ILO1,JPRB)
  KLON(JLEN)=ILO1
  PDLO1(JLEN)=ZDLO1
  ZLO2  =ZLOO*ZLSDEPI(ILA+2)
  ILO2  =INT(ZLO2)
  ZDLO2 =ZLO2-REAL(ILO2,JPRB)
  KLOS(JLEN)=ILO2
  PDLO2(JLEN)=ZDLO2

  IF(KBINL == 0) THEN

    ZLO   =ZLOO*ZLSDEPI(ILA  )
    ILO   =INT(ZLO )
    ZDLO  =ZLO -REAL(ILO ,JPRB)
    KLONN(JLEN)=ILO
    PDLO(JLEN)=ZDLO
    ZLO3  =ZLOO*ZLSDEPI(ILA+3)
    ILO3  =INT(ZLO3)
    ZDLO3 =ZLO3-REAL(ILO3,JPRB)
    KLOSS(JLEN)=ILO3
    PDLO3(JLEN)=ZDLO3

  ENDIF

ENDDO

!     ---------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUHOW1',1,ZHOOK_HANDLE)
END SUBROUTINE SUHOW1

