SUBROUTINE WRMLFP(CDCONF)

!**** *WRMLFP*  - writes out the model level and sfc fields in GRIB

!     Purpose.
!     --------
!     Write out the model level and sfc fields in GRIB

!**   Interface.
!     ----------
!        *CALL* *WRMLFP(CDCONF)

!        Explicit arguments :     CDCONF - configuration of call
!        --------------------

!        Implicit arguments :      The state variables of the model
!        --------------------

!     Method.
!     -------
!        See documentation
!        - spectral part would only work if input-resol = output resol
!          for vertical and horizontal resolution
!        - surface fields pp. without any fluxes ...

!     Externals.   MODMSEC - modify GRIB headers, output resolution
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Nils Wedi *ECMWF*

!     Modifications.
!     --------------
!       Original : 97-02-20 adapted from WRMLPPG,WRHFPG
!       R. El Khatib (98-10-05) : calling sequence to extfpf
!       D.Dent       (99-05-05) : allow different field code lists
!       D.Dent       (99-05-05) : constrain physical range for cloud fields
!                                 enable budget values
!       R. El Khatib 99-02-03   : FPSPNORM
!       R. El Khatib 99-03-09   : Simplify fullpos control
!       J. Teixeira (99-03-26)  : Surface tiling
!       D.Dent       (00-01-10) : pass first dimension to FPSPNORM
!       T. Stockdale (00-02-16) : seasonal stream / local use 15
!       D. Dent       00-04-12  : Support for computation of 
!                                 potential temperature
!       R. El Khatib  99-12-28  : remove poles rows
!       D.Dent        00-09-01  : convert from MPE to MPL
!    R. El Khatib : 00-11-03 Optional communications in extgpf
!    R. El Khatib : 01-04-10 Enable post-processing of filtered spectra
!    R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!       D.Dent       (02-05-15) : remove reference to extra field 099
!    R. El Khatib : 03-04-17 Fullpos improvments
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!       A. Tompkins : 11-02-04: adding TCLW, TCIW
!       P. Viterbo  : 24-05-2004   Change surface units
!       S. Serrar   : 26-09-2005 total column CO2 and SF6 added for postprocessing
!        M.Hamrud      01-May-2006 Generalized IO scheme
!       A. Dethof   : 15-Dec-05  : Add total column GRG fields
!       R. Engelen  : 02-Jun-06  : CO2 and SF6 replaced by generic GHG
!       JJMorcrette : 20060630   : MODIS Albedo
!       S. Serrar    : 07-Sep-06  : post-process in lagged mode total columns
!                                   for a few tracers
!       JJMorcrette  20060925    DU, BC, OM, SU, VOL, SOA climatological fields 
!       G. Balsamo  : 20070115   : Soil Type (SOTY)
!       P. Bechtold : 20070829   : CAPE
!       H. Hersbach : 04-Dec-2009: 10m-neutral wind and friction velocity

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMMP    , ONLY : LSPLITOUT,MYPROC   ,MYSETV   
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NCONF    ,LFDBOP  ,LOBSC1, NPROC
USE YOMCT3   , ONLY : NSTEP    !KPP
USE YOMFPCT0 , ONLY : NFPCT0
USE YOMDIM   , ONLY : NPROMA   ,NSPEC2   
USE YOMPHY   , ONLY : LMPHYS
USE YOEPHY   , ONLY : LEPHYS
USE YOMOP    , ONLY : CFNPL    
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG ,NGPTOTMX !KPP
USE YOMVAR   , ONLY : LTWBGV   ,LTWCGL  
USE YOMGRB   , ONLY : &
 & NGRBBV   ,NGRBT    ,NGRBTCW  ,NGRBTCWV ,NGRBTCLW  ,NGRBTCIW,&
 & NGRBTCRW ,NGRBTCSW,&
 & NGRBSD   ,NGRBLSP  ,NGRBCP   ,NGRBTP   ,&
 & NGRBSF   ,NGRBMSL  ,NGRBSP   ,NGRBSLT ,&
 & NGRBSDOR ,NGRBISOR ,NGRBANOR ,NGRBSLOR ,NGRBTCC  ,&
 & NGRB10U  ,NGRB10V  ,NGRB2T   ,NGRB2D   ,&
 & NGRBLSM  ,NGRBSR   ,NGRBAL   ,NGRBE ,&
 & NGRBCCC  ,NGRBLCC  ,NGRBMCC  ,NGRBHCC  ,NGRBSRC  ,&
 & NGRBRO   ,NGRBSSRO   ,NGRBSRO   ,NGRBTCO3 ,&
 & NGRBTCGHG,NGRBTCTRAC,NGRBTCGRG, &
 & NGRBIEWS ,NGRBINSS ,NGRBISHF ,NGRBIE   ,NGRBLSRH ,&
 & NGRBCSF  ,NGRBLSF  ,NGRBFAL  ,NGRBFSR  ,&
 & NGRBFLSR ,NGRBCLWC ,NGRBCIWC ,NGRBCC   ,NGRBBLH  ,NGRBCAPE  ,&
 & NGRBCVL  ,NGRBCVH  ,NGRBTVL  ,NGRBTVH  ,NGRBSDFOR,& 
 & NGRBALUVP,NGRBALUVD,NGRBALNIP,NGRBALNID,&
 & NGRBAERDEP, NGRBAERLTS, NGRBAERSCC  ,&
 & NGRBAERLS01,NGRBAERLS02,NGRBAERLS03,NGRBAERLS04,NGRBAERLS05,NGRBAERLS06,&
 & NGRBAERLS07,NGRBAERLS08,NGRBAERLS09 ,NGRBAERLS10,NGRBAERLS11,NGRBAERLS12,&
 & NGRBDNDZN, NGRBDNDZA, NGRBDCTB, NGRBTPLB, NGRBTPLT,&
 & NGRB100U, NGRB100V,NGRB10NU,NGRB10NV,NGRBZUST,&
 & NGRBOCT  ,NGRBOCS  ,NGRBOCU  ,NGRBOCV  ,          &  !KPP
 & NGRBOCDEP,NGRBOCLDP,NGRBOCLZ ,NGRBADVT ,NGRBADVS ,&  !KPP
 & NGRBOCUC  ,NGRBOCVC  

USE YOMCST   , ONLY : RG
USE YOMAUXB  , ONLY : GAUXBUF
USE YOMAFN   , ONLY : TFP_DYNDS, GFP_PHYDS
USE YOMFPC   , ONLY : CFPFMT   ,MFP3DFS  ,MFPPHY   ,&
 & NFP3DFS  ,NFP2DF   ,NFPPHY   ,NFPCFU   ,NFPXFU     
USE YOMFPG   , ONLY : NFPMAX   ,NFPRGPL  ,NFPRGPG  ,NFPRGPLX
USE YOMDFPB  , ONLY : HFPBUF
USE YOMFP4L  , ONLY : NDYNFPL  ,NFLDFPL  ,NFPSPBG  ,LFPSRFL  ,RFPSLEVG ,&
 & LFPSSRFG ,NFPSFLDG ,NFPSPBL  ,NFPSSETG ,NFPSLOCG ,RLEVFPL  
USE PTRFP4   , ONLY : YRQGFP
USE YOMFPDIM , ONLY : NFPBFOUT
USE YOMFPSP  , ONLY : SPBFP
USE YOMFPSC2 , ONLY : NFPROMA
USE YOMVPOS  , ONLY : QFPTYPE
USE YOECLDP  , ONLY : RLMIN    ,RAMIN
USE YOMSKF   , ONLY : LTWSKFY  ,LTWSKFZ
USE YOETHF   , ONLY : RHOH2O
USE SURFACE_FIELDS_MIX, ONLY : SP_OM    ,YSP_OM    ,YSP_OMD      ,& !KPP
 & SD_V2    ,YSD_V2    ,YSD_V2D   ,SD_V3    ,YSD_V3    ,YSD_V3D     !KPP
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_PUT, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST, &
 & CLOSE_IOREQUEST
USE GRIDPOINT_BUFFERS_MIX
USE MPL_MODULE, ONLY : MPL_BARRIER

IMPLICIT NONE

!     EXTERNAL INTEGER FUNCTIONS
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM), EXTERNAL :: ISRCHEQ

CHARACTER :: CLFPFN2*30
CHARACTER (LEN = 1) ::  CLMODE,CLP
REAL(KIND=JPRB),ALLOCATABLE :: ZREAL(:,:)
REAL(KIND=JPRB) :: ZSD(NFPSPBG)
REAL(KIND=JPRB) :: ZSPFLD((NFPMAX(1)+1)*(NFPMAX(1)+2),1)
REAL(KIND=JPRB) :: ZGPFLD(NFPRGPG,1,1)

REAL(KIND=JPRB) ,ALLOCATABLE    :: ZPTRFLD(:,:)
INTEGER(KIND=JPIM),ALLOCATABLE  :: IFLDPTU(:),IGRIBU(:),ILEVELU(:)
INTEGER(KIND=JPIM),ALLOCATABLE  :: ISKIPU(:)
INTEGER(KIND=JPIM),ALLOCATABLE  :: IFLDGB(:),IFLDPT(:)
INTEGER(KIND=JPIM),ALLOCATABLE  :: IGRIBIOSH(:,:)
INTEGER(KIND=JPIM),ALLOCATABLE  :: IGRIBIOGG(:,:)


INTEGER(KIND=JPIM) :: IBSET, IEND, IFIELDST, IFLD, IGGMAX, IGRIBCD, II, IJUMP, &
 & ILEN, ILENF, ILENG, ILEVEL, ILOCLV, IMAXDYFLD, IMAXPTS, IPT,   &
 & ISKIP, ISPEC2, ISPECG, IST, IFF, IZZ, J,  &
 & JF, JFLD,  JNM, &
 & JLEV, JSTGLO, IBL !KPP   

LOGICAL :: LLDYN, LLPHY, LLSURF, LLUPPER

REAL(KIND=JPRB) :: ZMEAN, ZSCALE, ZZSCALE
REAL(KIND=JPRB) :: ZHOOK_HANDLE
TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST

#include "fcttim.h"

#include "abor1.intfb.h"
#include "fpspnorm.intfb.h"
#include "prepfdb.intfb.h"
#include "wroutgpgb.intfb.h"
#include "wroutspgb.intfb.h"

!     ------------------------------------------------------------------

!*       1.    PREPARATIONS.
!              --------------

IF (LHOOK) CALL DR_HOOK('WRMLFP',0,ZHOOK_HANDLE)

IF( NFPCT0 <= 0 ) LSPLITOUT=.FALSE.

IF (CDCONF == 'E') THEN
  CLP='P'
ELSEIF (CDCONF == 'U') THEN
  CLP='H'
ELSEIF (CDCONF == 'Y') THEN
  CLP='V'
ELSEIF (CDCONF == 'M') THEN
  CLP='T'
ELSEIF (CDCONF == 'Z') THEN
  CLP='S'
ENDIF

ISPECG=(NFPMAX(1)+1)*(NFPMAX(1)+2)/2
ISPEC2  = NSPEC2

ILENF =ISPECG*2
ILENG = NFPRGPG

!     *  COMPUTE FIELD POINTERS

LLDYN=CDCONF == 'E'.OR.CDCONF == 'U'.OR.CDCONF == 'Y'.OR.&
 & CDCONF == 'M'.OR.CDCONF == 'Z'  
LLPHY=CDCONF == 'I'

IMAXDYFLD=NDYNFPL

IF (LLDYN.AND.(NFP3DFS+NFP2DF) > 0) THEN
  ALLOCATE(IFLDPTU(IMAXDYFLD))
  ALLOCATE(IGRIBU(IMAXDYFLD))
  ALLOCATE(ILEVELU(IMAXDYFLD))
  ALLOCATE(ISKIPU(IMAXDYFLD))
  ALLOCATE(IGRIBIOSH(4,NFPSPBG+1))
ENDIF

IF (LLPHY.AND.(NFPPHY+NFPCFU+NFPXFU) > 0) THEN
  ALLOCATE(IFLDGB(NFPBFOUT))
  ALLOCATE(IFLDPT(NFPBFOUT))
ENDIF

!*       1.1   PREPARE FDB

IF(LFDBOP) THEN
  CALL PREPFDB
ENDIF

!      -----------------------------------------------------------

!*       2.    WRITE SPECTRAL DATA ON MODEL LEVELS.
!              ------------------------------------

!*       2.1   EXTRACT AND COPY SPECTRAL FIELDS
!              --------------------------------

IF (NFPSPBG > 0.AND.CFPFMT == 'MODEL'.AND.LLDYN) THEN

! Initialise igribiosh data structure
!   1 - grib code
!   2 - level 
!   3 - bset to send this field
!   4 - local level for sending bset

  CALL GSTATS(1952,0)
  DO JF=1,NFPSPBG
    IGRIBIOSH(1,JF)=TFP_DYNDS(NFPSFLDG(JF))%IGRIB
    IF(LFPSSRFG(JF)) THEN
      IGRIBIOSH(2,JF)=1  !Always level 1 for surface model fields
    ELSE
      IGRIBIOSH(2,JF)=NINT(RFPSLEVG(JF))
    ENDIF
    IGRIBIOSH(3,JF)=NFPSSETG(JF)
    IGRIBIOSH(4,JF)=NFPSLOCG(JF)
  ENDDO
  ALLOCATE(ZPTRFLD(ISPEC2,NFPSPBL))
  IFF=0
  DO JF=1,NFPSPBG
    IBSET  = IGRIBIOSH(3,JF)
    IF( IBSET == MYSETV )THEN
      IFF=IFF+1
      ILOCLV = IGRIBIOSH(4,JF)
      DO JNM=1,ISPEC2
        ZPTRFLD(JNM,IFF)=SPBFP(ILOCLV,JNM)
      ENDDO
    ENDIF
  ENDDO
  CALL GSTATS(1952,1)

! Compute and write out spectral norms
  WRITE(NULOUT,FMT='(''FULL-POS SPNORMS'')')
  CALL FPSPNORM(SPBFP,NFPSPBL,NFPSPBG,1,NFPSSETG,NFPSLOCG,ZSD)
  IF (MYPROC == 1) THEN
    DO JF=1,NFPSPBG
      IGRIBCD = IGRIBIOSH(1,JF)
      ILEVEL = IGRIBIOSH(2,JF)
      IBSET  = IGRIBIOSH(3,JF)
      ILOCLV = IGRIBIOSH(4,JF)
      ZMEAN=ZSD(JF)
      LLSURF=LFPSSRFG(JF)
      LLUPPER=.NOT.LLSURF
      IF (LLUPPER) THEN
        WRITE(NULOUT,FMT='(1X, A1,I5.5,I3.3,'' : '',E21.15)') CLP,ILEVEL,&
         & IGRIBCD,ZMEAN  
      ELSEIF(LLSURF) THEN
        WRITE(NULOUT,FMT='(1X, I3.3,'' : '',E21.15)')IGRIBCD,ZMEAN
      ENDIF
    ENDDO
  ENDIF
  
!  Gather global fields and write out

  CALL WROUTSPGB(ZPTRFLD,NFPSPBG,IGRIBIOSH,'m')
  DEALLOCATE(ZPTRFLD)

ENDIF

!      -----------------------------------------------------------
!*           write spectral gribheader in output-resolution 
!            in preparation of NFPCT0.EQ.0

IF( NFPCT0 < 0 ) THEN
  ILEN=INDEX(CFNPL,' ')-1
  CLFPFN2='SH'//CFNPL(3:ILEN)
  ZSPFLD(:,:) = 0.0_JPRB
  CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFPFN2),CDMODE='w',KIOMASTER=1)
  CALL SETUP_IOREQUEST(YL_IOREQUEST,'SPECTRAL_FIELDS',LDGRIB=.TRUE.,&
   & KGRIB2D=(/NGRBT/),KLEVS2D=(/1/),CDLEVTYPE='ML',LDISTRIBUTE=.FALSE.,&
   & LDFPDIST=.TRUE.)
  CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR2=ZSPFLD)
  CALL CLOSE_IOREQUEST(YL_IOREQUEST)
  CALL CLOSE_IOSTREAM(YL_IOSTREAM)
  WRITE(NULOUT,*) ' SH GRIBHEADER WRITTEN '
ENDIF

!      -----------------------------------------------------------
!*           write gridpoint gribheader in output-resolution 
!            in preparation of NFPCT0.EQ.0

IF( NFPCT0 < 0 ) THEN
  ILEN=INDEX(CFNPL,' ')-1
  CLFPFN2='GG'//CFNPL(3:ILEN)
  ZGPFLD(:,:,:) = 0.0_JPRB
  CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFPFN2),CDMODE='w',KIOMASTER=1)
  CALL SETUP_IOREQUEST(YL_IOREQUEST,'GRIDPOINT_FIELDS',LDGRIB=.TRUE.,&
   & KGRIB2D=(/NGRBT/),KLEVS2D=(/1/),CDLEVTYPE='ML',LDISTRIBUTE=.FALSE.,&
   & LDFPDIST=.TRUE.,KPROMA=NFPRGPG)
  CALL IO_PUT(YL_IOSTREAM,YL_IOREQUEST,PR3=ZGPFLD)
  CALL CLOSE_IOREQUEST(YL_IOREQUEST)
  CALL CLOSE_IOSTREAM(YL_IOSTREAM)
  WRITE(NULOUT,*) ' GG GRIBHEADER WRITTEN '
ENDIF


!      -----------------------------------------------------------

!*       4.0   WRITE OUT SURFACE GRID POINT FIELDS
!              -----------------------------------

IF ( LLPHY.AND.(.NOT.(LTWBGV.OR.LTWCGL.OR.LTWSKFY.OR.LTWSKFZ).AND.&
   & ((LMPHYS.OR.LEPHYS).OR.LOBSC1.OR.NCONF == 131)) ) THEN  

!*       4.1   COPY FROM NPROMA FORMAT TO FIELD FORMAT

  IPT     = 0
  IGGMAX=0
  ALLOCATE(IGRIBIOGG(2,NFPBFOUT))

  DO JFLD=1,NFPBFOUT

    IGRIBCD = GFP_PHYDS(YRQGFP%ICOD(JFLD))%IGRIB

    IF( NFPPHY /= 0 .AND. ISRCHEQ(NFPPHY,MFPPHY,1,IGRIBCD) <= NFPPHY ) THEN
! skip writing out as this is done in lagged mode ...
      IF ( (IGRIBCD == NGRBTCC ).OR.(IGRIBCD == NGRB10U ).OR.&
         & (IGRIBCD == NGRB10V ).OR.(IGRIBCD == NGRB2T  ).OR.&
         & (IGRIBCD == NGRB2D  ).OR.(IGRIBCD == NGRBCCC ).OR.&
         & (IGRIBCD == NGRBLCC ).OR.(IGRIBCD == NGRBMCC ).OR.&
         & (IGRIBCD == NGRBHCC ).OR.(IGRIBCD == NGRBBLH ).OR.&
         & (IGRIBCD == NGRBIEWS).OR.(IGRIBCD == NGRBINSS).OR.&
         & (IGRIBCD == NGRBISHF).OR.(IGRIBCD == NGRBIE  ).OR.&
         & (IGRIBCD == NGRBBV  ).OR.(IGRIBCD == NGRBFAL ).OR.&
         & (IGRIBCD == NGRBMSL ).OR.(IGRIBCD == NGRBSP  ).OR.&
         & (IGRIBCD == NGRBFSR ).OR.(IGRIBCD == NGRBFLSR).OR.&
         & (IGRIBCD == NGRBTCW ).OR.(IGRIBCD == NGRBTCWV).OR.&
         & (IGRIBCD == NGRBTCLW).OR.(IGRIBCD == NGRBTCIW).OR.&
         & (IGRIBCD == NGRBCAPE).OR.(IGRIBCD == NGRBTCO3).OR.&
         & (IGRIBCD == NGRBTCRW).OR.(IGRIBCD == NGRBTCSW).OR.&
         & (IGRIBCD == NGRB100U).OR.(IGRIBCD == NGRB100V).OR.&
         & (IGRIBCD == NGRBZUST).OR.&
         & (IGRIBCD == NGRB10NU).OR.(IGRIBCD == NGRB10NV).OR.&
         & (IGRIBCD == NGRBDNDZN).OR.(IGRIBCD == NGRBDNDZA).OR.&
         & (IGRIBCD == NGRBDCTB).OR.&
         & (IGRIBCD == NGRBTPLB).OR.(IGRIBCD == NGRBTPLT).OR.&
         & ANY(IGRIBCD == NGRBTCGHG(:)).OR.ANY(IGRIBCD == NGRBTCTRAC(:)).OR.&
         & ANY(IGRIBCD == NGRBTCGRG(:))&
         & ) THEN  
        CYCLE
      ENDIF

      IF( NFPCT0 < 0 ) THEN
! skip writing out of climate fields
        IF ( (IGRIBCD == NGRBSDOR).OR.(IGRIBCD == NGRBISOR).OR.&
           & (IGRIBCD == NGRBSLT).OR.&
           & (IGRIBCD == NGRBANOR).OR.(IGRIBCD == NGRBSLOR).OR.&
           & (IGRIBCD == NGRBLSM).OR.(IGRIBCD == NGRBSR).OR.&
           & (IGRIBCD == NGRBCVL).OR.(IGRIBCD == NGRBCVH).OR.&
           & (IGRIBCD == NGRBTVL).OR.(IGRIBCD == NGRBTVH).OR.&
           & (IGRIBCD == NGRBAL).OR.(IGRIBCD == NGRBLSRH).OR.&
           & (IGRIBCD == NGRBSDFOR).OR.&
           & (IGRIBCD == NGRBALUVP).OR.(IGRIBCD == NGRBALUVD).OR.&
           & (IGRIBCD == NGRBALNIP).OR.(IGRIBCD == NGRBALNID).OR.&
           & (IGRIBCD == NGRBAERDEP).OR.(IGRIBCD == NGRBAERLTS).OR.&
           & (IGRIBCD == NGRBAERSCC).OR.(IGRIBCD == NGRBAERLS09).OR.&
           & (IGRIBCD == NGRBAERLS01).OR.(IGRIBCD == NGRBAERLS02).OR.&
           & (IGRIBCD == NGRBAERLS03).OR.(IGRIBCD == NGRBAERLS04).OR.&
           & (IGRIBCD == NGRBAERLS05).OR.(IGRIBCD == NGRBAERLS06).OR.&
           & (IGRIBCD == NGRBAERLS07).OR.(IGRIBCD == NGRBAERLS08).OR.&
           & (IGRIBCD == NGRBAERLS10).OR.(IGRIBCD == NGRBAERLS11).OR.&
           & (IGRIBCD == NGRBAERLS12) &
           & ) THEN
          CYCLE
        ENDIF
      ENDIF
      IGGMAX = IGGMAX+1
      IGRIBIOGG(1,IGGMAX) = IGRIBCD
      IGRIBIOGG(2,IGGMAX) = 0
      IFLDPT(IGGMAX) = JFLD
      IFLDGB(IGGMAX) = IGRIBCD
    ELSE
      WRITE(NULOUT,'(A,I6.6)')&
       & ' FIELD IN BUFFER BUT PP NOT REQUESTED ...',IGRIBCD  
    ENDIF

  ENDDO

  IJUMP = MIN(NFPRGPL,NFPROMA)
  ALLOCATE(ZREAL(NFPRGPL,IGGMAX))

  IFIELDST = IPT

  DO JF=1,IGGMAX
    IGRIBCD = IGRIBIOGG(1,JF)

    II=0
    DO IST=1,NFPRGPL,IJUMP
      IEND  = MIN(IST+IJUMP-1,NFPRGPL)
      ISKIP = IEND-IST+1
      DO J=IST,IEND
        ZREAL(J,JF) = HFPBUF(II*IJUMP+(IFLDPT(JF)-1)*ISKIP+J-IST+1)
      ENDDO
      II = II+NFPBFOUT
    ENDDO

!*       4.2   SCALINGS FOR ECMWF MARS ARCHIVING

    IF    (IFLDGB(JF) == NGRBSD  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBLSP ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBCP  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBTP  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBSF  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBSR  ) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSEIF(IFLDGB(JF) == NGRBE   ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBSRC ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBRO  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBSSRO  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBSRO  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBCSF ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBLSF ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IFLDGB(JF) == NGRBFSR ) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSEIF(IFLDGB(JF) == NGRBSDOR) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSE
      ZSCALE = 1.0_JPRB
    ENDIF
    IF(ZSCALE /= 1.0_JPRB) THEN
      DO J=1,NFPRGPL
        ZREAL(J,JF) = ZREAL(J,JF)*ZSCALE
      ENDDO
    ENDIF
  ENDDO

!  Gather global fields and write out
  IF(IGGMAX > 0) THEN
    CLMODE='w'
    CALL WROUTGPGB(ZREAL,NFPRGPL,NFPRGPG,NFPRGPLX,IGGMAX,IGRIBIOGG,'s',CLMODE)
  ENDIF
  DEALLOCATE(ZREAL)
  DEALLOCATE(IGRIBIOGG)

ENDIF

!      -----------------------------------------------------------

!*       5.0   WRITE OUT UPPER AIR GRID POINT FIELDS
!              --------------------------------------
IF( LSPLITOUT.OR.NFPCT0 == 0 )THEN
  CLMODE = 'w'
ELSE
  CLMODE = 'a'
ENDIF

IF (LLDYN.AND.(NFP3DFS+NFP2DF) > 0) THEN

!*       5.1   COPY FROM NPROMA FORMAT TO FIELD FORMAT

  IFF = 0
  DO JFLD=1,NDYNFPL

    IFLD=NFLDFPL(JFLD)
    LLSURF=LFPSRFL(JFLD)
    LLUPPER=.NOT.LLSURF
    ILEVELU(JFLD)=NINT(RLEVFPL(JFLD))
    IGRIBU (JFLD) = TFP_DYNDS(IFLD)%IGRIB
    IFLDPTU(JFLD) = JFLD

    IGRIBCD=IGRIBU(JFLD)
    IF( LLUPPER ) THEN
      IF((NFP3DFS /= 0 .AND. ISRCHEQ(NFP3DFS,MFP3DFS,1,IGRIBCD) > NFP3DFS)&
         & .AND..NOT.( NFPCT0 < 0 )) THEN  
        WRITE(NULOUT,'(A,I3.3)')&
         & ' FIELD IN BUFFER BUT PP NOT REQUESTED ...',IGRIBCD  
        ISKIPU(JFLD) = 1
      ELSE
        IFF = IFF+1
        ISKIPU (JFLD) = 0
      ENDIF
    ELSE
      IF(QFPTYPE%ISF(IFLD)>=1.AND.NFPCT0 >= 0) THEN
        WRITE(NULOUT,'(A,I3.3)')&
         & ' FIELD CONSIDERED SPECTRAL ONLY ... SKIPPED PAR=',IGRIBCD  
        ISKIPU(JFLD) = 1
      ELSE
        IFF = IFF+1
        ISKIPU(JFLD) = 0
      ENDIF
    ENDIF

  ENDDO

  IFIELDST = NDYNFPL

  IF( CFPFMT == 'GAUSS' ) THEN
    IJUMP = MIN(NFPRGPL,NFPROMA)
    IMAXPTS=NFPRGPL
  ELSE
    IJUMP = MIN(NGPTOT,NPROMA)
    IMAXPTS=NGPTOT
    IF( .NOT. ALLOCATED_GRIDPOINT_BUFFER(GAUXBUF).AND.(IFIELDST>0)) THEN
      CALL ABOR1('WRMLFP:GAUXBUF NOT ALLOCATED')
    ENDIF
    IF( .NOT. GAUXBUF%LFILLED.AND.(IFIELDST>0)) THEN
      CALL ABOR1('WRMLFP:GAUXBUF NOT FILLED')
    ENDIF
  ENDIF

!     Loop over all fields

  ALLOCATE(ZREAL(NFPRGPL,IFF))
  ALLOCATE(IGRIBIOGG(2,IFF))

  IFF = 0
  CALL GSTATS(1953,0)
  DO JF=1,IFIELDST
    IGRIBCD = IGRIBU(JF)
    ILEVEL  = ILEVELU(JF)
    IF( ISKIPU(JF) /= 1 ) THEN
      IFF =IFF+1
      IGRIBIOGG(1,IFF) = IGRIBCD
      IGRIBIOGG(2,IFF) = ILEVEL
      II=0
      DO IST=1,IMAXPTS,IJUMP
        IEND=MIN(IST+IJUMP-1,IMAXPTS)
        ISKIP=IEND-IST+1
        IF( CFPFMT == 'GAUSS' ) THEN
          DO J=IST,IEND
            ZREAL(J,IFF) = HFPBUF(II*IJUMP+(IFLDPTU(JF)-1)*ISKIP+J-IST+1)
          ENDDO
        ELSE
          DO J=IST,IEND
!mf This needs changing. Gridpoint buffers should be accessed using SC2RDG.
            ZREAL(J,IFF) = GAUXBUF%GPBUF(II*IJUMP+(IFLDPTU(JF)-1)*ISKIP+J-IST+1)
          ENDDO
        ENDIF
        II=II+IMAXDYFLD
      ENDDO
 
!*             SCALE FIELDS
!               prevent small values falling outside physical range

      IF(IGRIBCD == NGRBCC) THEN
        ZSCALE = RAMIN
      ELSEIF(IGRIBCD == NGRBCLWC) THEN
        ZSCALE = RLMIN
      ELSEIF(IGRIBCD == NGRBCIWC) THEN
        ZSCALE = RLMIN
      ELSE
        ZSCALE = 1.0_JPRB
      ENDIF
      IF(ZSCALE /= 1.0_JPRB) THEN
        ZZSCALE = 1.0_JPRB/ZSCALE
        DO J=1,NFPRGPL
          IZZ = ZREAL(J,IFF)*ZZSCALE
          ZREAL(J,IFF) = IZZ*ZSCALE
        ENDDO
      ENDIF
    ENDIF
  ENDDO
  CALL GSTATS(1953,1)
  IGGMAX = IFF

!  Gather global fields and write out
  IF(IGGMAX > 0) THEN
    CALL WROUTGPGB(ZREAL,NFPRGPL,NFPRGPG,NFPRGPLX,IGGMAX,IGRIBIOGG,'m',CLMODE)
  ENDIF
  DEALLOCATE(ZREAL)
  DEALLOCATE(IGRIBIOGG)
ENDIF


!*       6.0   WRITE OUT OCEAN MIXED LAYER FIELDS
!              ----------------------------------

!*       6.1   COPY FROM NPROMA FORMAT TO FIELD FORMAT

WRITE(NULOUT,*) '-------- KPP OUTPUT KPP OUTPUT ',YSP_OMD%NUMFLDS 

IFLD = YSP_OMD%NLEVS * YSP_OMD%NUMFLDS   ! all prognostic variables, T,S,U,V
IFLD = IFLD + YSD_V3D%NLEVS * 4          ! PZO, PDO, clim_U & clim_V 
IFLD = IFLD + YSD_V2D%NUMFLDS            ! bathymetry, taux_clim, tauy_clim

IGGMAX=0
IF(IFLD > 0) THEN
  ALLOCATE (ZREAL(NGPTOT,IFLD))
  ALLOCATE (IGRIBIOGG(2,IFLD))
ENDIF

IF (YSP_OMD%NUMFLDS>0) THEN
  DO JF=1,YSP_OMD%NUMFLDS
    WRITE(NULOUT,'(A,I4,I6)') ' OML PROGNOSTIC FIELDS :',JF, YSP_OM%YOM(JF)%IGRBCODE(1)
!    CALL GPNORM3(SP_OM,YSP_OMD%NUMFLDS,JF,KLEVS=YSP_OMD%NLEVS,LDLEVELS=.TRUE.)
  ENDDO
ENDIF

IF( YSP_OMD%NUMFLDS > 0 ) THEN
  DO JF=1,YSP_OMD%NUMFLDS
    DO JLEV =1, YSP_OMD%NLEVS
      IGGMAX = IGGMAX+1
      IGRIBIOGG(1,IGGMAX) = YSP_OM%YOM(JF)%IGRBCODE(1)
      IGRIBIOGG(2,IGGMAX) = JLEV
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL   = (JSTGLO-1)/NPROMA+1
        DO J=1,IEND
          ZREAL(JSTGLO+J-1,IGGMAX) = SP_OM(J,JLEV,YSP_OM%YOM(JF)%MP,IBL)
        ENDDO
      ENDDO
    ENDDO
  ENDDO
ENDIF

IF( NSTEP == 0 ) THEN  

  IF (YSD_V2D%NUMFLDS>0) THEN
    WRITE(NULOUT,'(A)') ' OML 2D DIAG FIELDS '
!    CALL GPNORM2(YSD_V2D%NUMFLDS,1,SD_V2)
  ENDIF

  IF( YSD_V2D%NUMFLDS > 0 ) THEN
    DO JF=1,YSD_V2D%NUMFLDS
      WRITE(NULOUT,'(A,I4,I6)') ' OML 2D DIAGNOSTIC FIELDS :',JF, YSD_V2%YV2(JF)%IGRBCODE
      IGGMAX = IGGMAX+1
      IGRIBIOGG(1,IGGMAX) = YSD_V2%YV2(JF)%IGRBCODE
      IGRIBIOGG(2,IGGMAX) = 1
      DO JSTGLO=1,NGPTOT,NPROMA
        IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
        IBL   = (JSTGLO-1)/NPROMA+1
        DO J=1,IEND
          ZREAL(JSTGLO+J-1,IGGMAX) = SD_V2(J,YSD_V2%YV2(JF)%MP,IBL)
        ENDDO
      ENDDO
    ENDDO
  ENDIF

  IF( YSD_V3D%NUMFLDS > 0 ) THEN
    DO JF=1,YSD_V3D%NUMFLDS
      WRITE(NULOUT,'(A,I4,I6)') ' OML 3D DIAGNOSTIC FIELDS :',JF, YSD_V3%YV3(JF)%IGRBCODE(1)
      IF(YSD_V3%YV3(JF)%IGRBCODE(1) == NGRBOCLZ  .OR. &   !layer depth       
        &YSD_V3%YV3(JF)%IGRBCODE(1) == NGRBOCLDP .OR. &   !layer thickness
        &YSD_V3%YV3(JF)%IGRBCODE(1) == NGRBOCUC  .OR. &
        &YSD_V3%YV3(JF)%IGRBCODE(1) == NGRBOCVC ) THEN  
        DO JLEV =1, YSD_V3D%NLEVS
          IGGMAX = IGGMAX+1
          IGRIBIOGG(1,IGGMAX) = YSD_V3%YV3(JF)%IGRBCODE(1)
          IGRIBIOGG(2,IGGMAX) = JLEV
          DO JSTGLO=1,NGPTOT,NPROMA
            IEND = MIN(NPROMA,NGPTOT-JSTGLO+1)
            IBL   = (JSTGLO-1)/NPROMA+1
            DO J=1,IEND
              ZREAL(JSTGLO+J-1,IGGMAX) = SD_V3(J,JLEV,YSD_V3%YV3(JF)%MP,IBL)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
    ENDDO
  ENDIF

ENDIF

!*       6.2   WRITE FIELDS

IF(IGGMAX > 0) THEN
!  CALL WROUTGPGB(ZREAL,NGPTOT,NGPTOTG,NGPTOTMX,IGGMAX,IGRIBIOGG,'o',CLMODE)
!  To output the ocean mixed layer fields, more work to be done for GRIB encode,
!  the model level is used for a while...

  CALL WROUTGPGB(ZREAL,NGPTOT,NGPTOTG,NGPTOTMX,IGGMAX,IGRIBIOGG,'m',CLMODE)
ENDIF


IF(IFLD > 0) THEN
  DEALLOCATE(ZREAL) 
  DEALLOCATE(IGRIBIOGG) 
ENDIF


IF( NPROC > 1 )THEN
  CALL GSTATS(792,0)
  CALL MPL_BARRIER(CDSTRING='WRMLFP:')
  CALL GSTATS(792,1)
ENDIF

!     ---------------------------------------------------------------

IF (LLDYN.AND.(NFP3DFS+NFP2DF) > 0) THEN
  DEALLOCATE(IFLDPTU)
  DEALLOCATE(IGRIBU)
  DEALLOCATE(ILEVELU)
  DEALLOCATE(ISKIPU)
  DEALLOCATE(IGRIBIOSH)
  IF (ALLOCATED(SPBFP)) DEALLOCATE(SPBFP)
ENDIF

IF (LLPHY.AND.(NFPPHY+NFPCFU+NFPXFU) > 0) THEN
  DEALLOCATE(IFLDGB)
  DEALLOCATE(IFLDPT)
ENDIF
IF (LHOOK) CALL DR_HOOK('WRMLFP',1,ZHOOK_HANDLE)

!     ---------------------------------------------------------------

END SUBROUTINE WRMLFP

