SUBROUTINE WRMLFPL(CDCONF)

!**** *WRMLFPL*  - writes out the model level and sfc fields in GRIB
!                  lagged mode
!     Purpose.
!     --------
!     Write out the model level and sfc fields in GRIB

!**   Interface.
!     ----------
!        *CALL* *WRMLFP(CDCONF)

!        Explicit arguments :     CDCONF - configuration of call
!        --------------------

!        Implicit arguments :      The state variables of the model
!        --------------------

!     Method.
!     -------
!        See documentation
!        - spectral part would only work if input-resol = output resol
!          for vertical and horizontal resolution
!        - surface fields pp. without any fluxes ...

!     Externals.   MODMSEC - modify GRIB headers, output resolution
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Nils Wedi *ECMWF*

!     Modifications.
!     --------------
!       Original : 97-02-20 adapted from WRMLPPG,WRHFPG
!       R. El Khatib 98-10-05 calling sequence to extfpf
!       R. El Khatib : 99-03-09 Simplify fullpos control
!       Modified : 99-03-26  J Teixeira    Surface tiling
!       R. El Khatib : 99-12-28 remove poles rows
!       D.Dent       : 00-09-01 convert from MPE to MPL
!       R. El Khatib : 01-08-07 Pruning options
!    R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!    R. El Khatib : 03-04-17 Fullpos improvments
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!       A.TOmpkins: 11-02-04 add TCLW, TCIW
!       P. Viterbo  : 24-05-2004   Change surface units
!       S. Serrar   : 26-09-2005 total column CO2 and SF6 added for postprocessing
!       A. Dethof   : 15-Dec-05    Add total column GRG fields
!       R. Engelen   : 02-Jun-06  : CO2 replaced by generic GHG
!       S. Serrar    : 07-Sep-06  : post-process in lagged mode total columns
!                                   for a few tracers
!       JJMorcrette : 20060630   : MODIS Albedo
!       JJMorcrette : 20060715  Desert dust climatological fields
!       P. Bechtold : 20070829   : CAPE
!       H. Hersbach : 04-Dec-2009: 10m-neutral wind and friction velocity
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMMP    , ONLY : LSPLITOUT
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NCONF    ,LFDBOP   ,LOBSC1  
USE YOMFPCT0 , ONLY : NFPCT0
USE YOMPHY   , ONLY : LMPHYS
USE YOEPHY   , ONLY : LEPHYS
USE YOMVAR   , ONLY : LTWBGV   ,LTWCGL
USE YOMGRB   , ONLY :                 &
 & NGRBBV   ,&
 & NGRBTCW  ,NGRBTCWV ,NGRBTCLW ,NGRBTCIW ,NGRBSD   ,NGRBLSP  ,&
 & NGRBTCRW ,NGRBTCSW,&
 & NGRBCP   ,NGRBTP   ,NGRBSF   ,NGRBMSL  ,NGRBSP   ,&
 & NGRBSDOR ,NGRBTCC  ,NGRB10U  ,NGRB10V  ,NGRB2T   ,NGRB2D   ,&
 & NGRBSR   ,NGRBE    ,NGRBCCC  ,NGRBLCC  ,&
 & NGRBMCC  ,NGRBHCC  ,NGRBSRC  ,&
 & NGRBRO   ,NGRBSSRO   ,NGRBSRO   ,NGRBTCO3 ,&
 & NGRBTCGHG,NGRBTCTRAC,NGRBTCGRG,&
 & NGRBIEWS ,NGRBINSS ,NGRBISHF ,NGRBIE   ,&
 & NGRBCSF  ,NGRBLSF  ,NGRBFAL  ,NGRBFSR  ,NGRBFLSR ,&
 & NGRBBLH  ,NGRBCAPE ,&
 & NGRB100U, NGRB100V ,NGRBZUST, NGRB10NU, NGRB10NV ,&
 & NGRBDNDZN,NGRBDNDZA,NGRBDCTB ,NGRBTPLB ,NGRBTPLT  
USE YOMCST   , ONLY : RG
USE YOMAFN   , ONLY : GFP_PHYDS
USE YOMFPC   , ONLY : MFPPHY   ,NFPPHY
USE YOMFPG   , ONLY : NFPRGPL  ,NFPRGPG  ,NFPRGPLX
USE YOMDFPB  , ONLY : HFPBUF
USE PTRFP4   , ONLY : YRQGFP
USE YOMFPDIM , ONLY : NFPBFOUT
USE YOMFPSC2 , ONLY : NFPROMA
USE YOETHF   , ONLY : RHOH2O

IMPLICIT NONE

#include "prepfdb.intfb.h"
#include "wrbudg.intfb.h"
#include "wroutgpgb.intfb.h"

!     EXTERNAL INTEGER FUNCTIONS
CHARACTER(LEN=1),   INTENT(IN)   :: CDCONF
INTEGER(KIND=JPIM), EXTERNAL :: ISRCHEQ

CHARACTER (LEN = 1) ::  CLMODE

REAL(KIND=JPRB),ALLOCATABLE :: ZREAL(:,:)

INTEGER(KIND=JPIM),ALLOCATABLE  :: IFLDGB(:),IFLDPT(:)
INTEGER(KIND=JPIM),ALLOCATABLE  :: IGRIBIOGG(:,:)

INTEGER(KIND=JPIM) :: IEND, IGGMAX, IGRIBCD, II, IJUMP, IPOS, ISKIP, IST, J, JF, JFLD

LOGICAL :: LLPHY, LLBV

REAL(KIND=JPRB) :: ZSCALE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcttim.h"

!     ------------------------------------------------------------------

!*       1.    PREPARATIONS.
!              --------------

IF (LHOOK) CALL DR_HOOK('WRMLFPL',0,ZHOOK_HANDLE)

IF( NFPCT0 <= 0 ) LSPLITOUT=.FALSE.

CLMODE = 'a'

!     *  COMPUTE FIELD POINTERS

LLPHY=CDCONF == 'I'

IF (LLPHY.AND.(NFPPHY) > 0) THEN
  ALLOCATE(IFLDGB(NFPBFOUT))
  ALLOCATE(IFLDPT(NFPBFOUT))
  ALLOCATE(IGRIBIOGG(2,NFPBFOUT))
ENDIF

!*       1.1   PREPARE FDB

IF(LFDBOP) THEN
  CALL PREPFDB
ENDIF

!      -----------------------------------------------------------

!*       4.0   WRITE OUT SURFACE GRID POINT FIELDS
!              -----------------------------------

IF ( LLPHY.AND..NOT.(LTWBGV.OR.LTWCGL).AND.&
   & (LMPHYS.OR.LEPHYS).OR.LOBSC1.OR.NCONF == 131 ) THEN  

!*       4.1   COPY FROM NPROMA FORMAT TO FIELD FORMAT

  IGGMAX=0
  LLBV=.FALSE.
  DO JFLD=1,NFPBFOUT

    IGRIBCD = GFP_PHYDS(YRQGFP%ICOD(JFLD))%IGRIB
    IPOS  = ISRCHEQ(NFPPHY,MFPPHY,1,IGRIBCD)
    IF( IPOS <= NFPPHY ) THEN
! writing out only if one of the following
      IF((IGRIBCD == NGRBTCC ).OR.(IGRIBCD == NGRB10U ).OR.&
         & (IGRIBCD == NGRB10V ).OR.(IGRIBCD == NGRB2T  ).OR.&
         & (IGRIBCD == NGRB2D  ).OR.(IGRIBCD == NGRBCCC ).OR.&
         & (IGRIBCD == NGRBLCC ).OR.(IGRIBCD == NGRBMCC ).OR.&
         & (IGRIBCD == NGRBHCC ).OR.(IGRIBCD == NGRBBLH ).OR.&
         & (IGRIBCD == NGRBIEWS).OR.(IGRIBCD == NGRBINSS).OR.&
         & (IGRIBCD == NGRBISHF).OR.(IGRIBCD == NGRBIE  ).OR.&
         & (IGRIBCD == NGRBFAL ).OR.&
         & (IGRIBCD == NGRBMSL ).OR.(IGRIBCD == NGRBSP  ).OR.&
         & (IGRIBCD == NGRBFSR ).OR.(IGRIBCD == NGRBFLSR).OR.&
         & (IGRIBCD == NGRBTCW ).OR.(IGRIBCD == NGRBTCWV).OR.&
         & (IGRIBCD == NGRBTCLW).OR.(IGRIBCD == NGRBTCIW).OR.&
         & (IGRIBCD == NGRBCAPE).OR.(IGRIBCD == NGRBTCO3).OR.&
         & (IGRIBCD == NGRBTCRW).OR.(IGRIBCD == NGRBTCSW).OR.&
         & (IGRIBCD == NGRB100U).OR.(IGRIBCD == NGRB100V).OR.&
         & (IGRIBCD == NGRBZUST).OR.&
         & (IGRIBCD == NGRB10NU).OR.(IGRIBCD == NGRB10NV).OR.&
         & (IGRIBCD == NGRBDNDZN).OR.(IGRIBCD == NGRBDNDZA).OR.&
         & (IGRIBCD == NGRBDCTB).OR.&
         & (IGRIBCD == NGRBTPLB).OR.(IGRIBCD == NGRBTPLT).OR.&
         & ANY(IGRIBCD == NGRBTCGHG(:)).OR.ANY(IGRIBCD == NGRBTCTRAC(:)).OR.&
         & ANY(IGRIBCD == NGRBTCGRG(:))&
         & ) THEN  
        IGGMAX = IGGMAX+1
        IGRIBIOGG(1,IGGMAX) = IGRIBCD
        IGRIBIOGG(2,IGGMAX) = 0
        IFLDPT(IGGMAX) = JFLD
        IFLDGB(IGGMAX) = IGRIBCD
      ENDIF
      IF(IGRIBCD == NGRBBV ) LLBV=.TRUE.
    ELSE
      WRITE(NULOUT,'(A,I6.6)')&
       & ' FIELD IN BUFFER BUT PP NOT REQUESTED ...',IGRIBCD             
    ENDIF

  ENDDO

  IJUMP = MIN(NFPRGPL,NFPROMA)
  ALLOCATE(ZREAL(NFPRGPL,IGGMAX))

  DO JF=1,IGGMAX
    IGRIBCD = IGRIBIOGG(1,JF)

    II=0
    DO IST=1,NFPRGPL,IJUMP
      IEND  = MIN(IST+IJUMP-1,NFPRGPL)
      ISKIP = IEND-IST+1
      DO J=IST,IEND
        ZREAL(J,JF) = HFPBUF(II*IJUMP+(IFLDPT(JF)-1)*ISKIP+J-IST+1)
      ENDDO
      II = II+NFPBFOUT
    ENDDO

!*       4.2   SCALINGS FOR ECMWF MARS ARCHIVING

    IF    (IGRIBCD == NGRBSD  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBLSP ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBCP  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBTP  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBSF  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBSR  ) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSEIF(IGRIBCD == NGRBE   ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBSRC ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBRO  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBSSRO  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBSRO  ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBCSF ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBLSF ) THEN
      ZSCALE = 1.0_JPRB/RHOH2O
    ELSEIF(IGRIBCD == NGRBFSR ) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSEIF(IGRIBCD == NGRBSDOR) THEN
      ZSCALE = 1.0_JPRB/RG
    ELSE
      ZSCALE = 1.0_JPRB
    ENDIF
    IF(ZSCALE /= 1.0_JPRB) THEN
      DO J=1,NFPRGPL
        ZREAL(J,JF) = ZREAL(J,JF)*ZSCALE
      ENDDO
    ENDIF
    
  ENDDO

!  Gather global fields and write out
  IF(IGGMAX > 0) THEN
    CALL WROUTGPGB(ZREAL,NFPRGPL,NFPRGPG,NFPRGPLX,IGGMAX,IGRIBIOGG,'s',CLMODE)
  ENDIF

  DEALLOCATE(ZREAL)

! Write out "budget record"

  IF(LLBV) CALL WRBUDG

ENDIF

!     ---------------------------------------------------------------

IF (LLPHY.AND.(NFPPHY) > 0) THEN
  DEALLOCATE(IFLDGB)
  DEALLOCATE(IFLDPT)
  DEALLOCATE(IGRIBIOGG)
ENDIF
IF (LHOOK) CALL DR_HOOK('WRMLFPL',1,ZHOOK_HANDLE)

!     ---------------------------------------------------------------

END SUBROUTINE WRMLFPL

