MODULE MWIMAGER_MIX

! This module maintains a private table of radiative parameters for dealing
! with microwave imagers, and provides a function to return the appropriate 
! ones, given the satellite ID of a microwave imager. 

! The principal use of this module is in making a regression of liquid water 
! path, for which the parameters vary depending upon the instrument. See
! mwimager_lwp.F90 for more information.

! Changes
! -------

! 15-Sept-2007  Alan Geer      Initial version
! 17-Sept-2008  Alan Geer      Use a different set of coeffs for all-sky 4D-Var
! 05-Febr-2009  Bill Bell      Add parameters for windsat.
! 10-Marc-2009  Qifeng Lu      add for FY3A MWRI

USE PARKIND1, ONLY: JPIM, JPRB

IMPLICIT NONE
SAVE

PRIVATE
PUBLIC  :: RTBCONST, RDTB_LIMIT, YCOEFF, GET_LWPCOEFF

REAL(KIND=JPRB), PARAMETER :: RTBCONST   = 280.0_JPRB ! Max TB in regression
REAL(KIND=JPRB), PARAMETER :: RDTB_LIMIT = 40.0_JPRB  ! 37v-37h "rain" threshold

TYPE YCOEFF
  INTEGER (KIND=JPIM) :: I22V, I37V, I37H 
  REAL    (KIND=JPRB) :: ZCONST, ZA, ZB, ZLWP_LIMIT, ZABIAS, ZBBIAS
END TYPE YCOEFF

INTEGER(KIND=JPIM), PARAMETER :: NINST = 6 

TYPE(YCOEFF) :: YMCOEFF(NINST*2)

!  -- Used for switching between rainy 1D+4D-Var or direct 4D-Var clear sky
!  -- (Windsat values are copies of AMSR-E parameters initially)
!
!  Index of channel 22v 37v 37h   CONST         A         B    LWP   A-BIAS-B
DATA & 
& YMCOEFF(1) /YCOEFF( 3,  4,  5, 4.2993, 0.399635, 1.406920, 0.01,   0.0, 0.0)/, & ! SSMI
& YMCOEFF(2) /YCOEFF(14, 16, 15, 4.5367, 0.4768,   1.5442,   0.015, -1.8, 0.0)/, & ! SSMIS
& YMCOEFF(3) /YCOEFF( 5,  6,  7, 3.9291, 0.6112,   1.5461,   0.015,  1.8, 3.4)/, & ! TMI
& YMCOEFF(4) /YCOEFF( 7,  9, 10, 4.2405, 0.6027,   1.6150,   0.015,  0.2, 2.8)/, & ! AMSRE
& YMCOEFF(5) /YCOEFF( 5,  7,  8, 4.2405, 0.6027,   1.6150,   0.015,  0.2, 2.8)/, & ! MWRI
& YMCOEFF(6) /YCOEFF(11, 13, 14, 4.2405, 0.6027,   1.6150,   0.015,  0.2, 2.8)/, & ! Windsat

!  -- Used for diagnostics with all-sky 4D-Var 

!  SSMI and AMSR-E values have been derived using a new technique that avoids 
!  ad-hoc bias corrections, by A. Geer on 17-Sept-2008. The 0.05kg m-2 threshold
!  gives roughly half observations "clear" and half "cloudy".
!  SSMIS and TMI still to be re-derived without the ad-hoc bias corrections!
!  Windsat parameters are a copy of the AMSR-E coefficients for initial implementation.
!
!  Index of channel 22v 37v 37h   CONST         A         B    LWP   A-BIAS-B
& YMCOEFF(7) /YCOEFF( 3,  4,  5, 4.0170, 0.3458,   1.2796,   0.05,   0.0, 0.0)/, & ! SSMI
& YMCOEFF(8) /YCOEFF(14, 16, 15, 4.5367, 0.4768,   1.5442,   0.015, -1.8, 0.0)/, & ! SSMIS
& YMCOEFF(9) /YCOEFF( 5,  6,  7, 3.9291, 0.6112,   1.5461,   0.015,  1.8, 3.4)/, & ! TMI
& YMCOEFF(10)/YCOEFF( 7,  9, 10, 3.9250, 0.5354,   1.4569,   0.05,   0.0, 0.0)/, & ! AMSRE
& YMCOEFF(11)/YCOEFF( 5,  7,  8, 3.9250, 0.5354,   1.4569,   0.05,   0.0, 0.0)/, & ! MWRI
& YMCOEFF(12)/YCOEFF(11, 13, 14, 3.9250, 0.5354,   1.4569,   0.05,   0.0, 0.0)/    ! Windsat
CONTAINS


! Function to return the appropriate coefficients and channel
! indices given a particular sensor ID. The return value of
! the function is TRUE if the sensor was recognised and FALSE
! if not.

FUNCTION GET_LWPCOEFF( KSENS_ID, YDCOEFF, LD_MWAVE)

  USE YOMHOOK, ONLY  : LHOOK, DR_HOOK 
  USE YOMTVRAD, ONLY : MSENSOR_SSMI, MSENSOR_SSMIS, MSENSOR_AMSRE, MSENSOR_TMI, MSENSOR_MWRI, MSENSOR_WINDSAT

  INTEGER(KIND=JPIM), INTENT(IN) :: KSENS_ID     ! I: Sensor ID
  TYPE(YCOEFF), INTENT(OUT)      :: YDCOEFF      ! O: Coefficients
  LOGICAL, INTENT(IN)            :: LD_MWAVE     ! I: Select "all-sky" coeffs
  LOGICAL                        :: GET_LWPCOEFF ! O: Success = TRUE

  INTEGER (KIND=JPIM) :: I
  REAL (KIND=JPRB)    :: ZHOOK_HANDLE  

  IF (LHOOK) CALL DR_HOOK('MWIMAGER_MIX:GET_LWPCOEFF',0,ZHOOK_HANDLE) 

  GET_LWPCOEFF = .TRUE. 

  SELECT CASE(KSENS_ID)

    CASE(MSENSOR_SSMI)  ! SSM/I
      I=1
    CASE(MSENSOR_SSMIS) ! SSMIS
      I=2
    CASE(MSENSOR_TMI)   ! TMI
      I=3
    CASE(MSENSOR_AMSRE) ! AMSR-E
      I=4
    CASE(MSENSOR_MWRI ) ! MWRI
      I=5
    CASE(MSENSOR_WINDSAT) ! WINDSAT
      I=6
    CASE DEFAULT
      GET_LWPCOEFF = .FALSE.

  END SELECT

  IF (GET_LWPCOEFF) THEN
    IF(LD_MWAVE) I = I + NINST
    YDCOEFF = YMCOEFF(I)
  ENDIF

  IF (LHOOK) CALL DR_HOOK('MWIMAGER_MIX:GET_LWPCOEFF',1,ZHOOK_HANDLE)

END FUNCTION GET_LWPCOEFF

END MODULE MWIMAGER_MIX
