MODULE TYPE_FPRQDYNS

! Purpose :
! -------
!    To define the internal type "TYPE_FPRQDYN" for the control of 
!    post-processed dynamic fields for a given vertical post-processing level :
!      - %LL    : .TRUE. if the computation of the field is needed
!      - %ISF   : spectral computation :
!                 0 : no spectral fit
!                 1 : spectral fit only
!                 2 : spectral fit + gaussian filter in the model spectral space
!                 3 : spectral fit + gaussian filter in the spectral space of 
!                     homogenous resolution (stretched model only)
!        spectral space (1) or not (0)
!      - %ISKP  : the number of fields to be reserved for the computation in gridpoint space
!      - %ILED  : Information about tensor aspect of the field in spectral space :
!        -j : vector 2nd momentum corresponding to the field nr j
!         0 : "scalar" 
!         1 : vector 1st momentum
!         2 : Vorticity as (U,V)
!         3 : Divergence as (U,V)
!      - %ILEV  : the number of vertical levels
!      - %ILVP  : the vertical levels pointers
!      - %IDOM  : the number of subdomains for each level
!      - %IDMP  : the subdomains pointers for each level
!      - %IGPX  : the data array pointers for gridpoint fields (GAUX)
!      - %IGT1(:)  : the data array pointers for spectrally fitted fields : 
!         (0) : pointer after spectral fit (GT0)
!         (1) : main field pointer before spectral fit (GT1)
!         (2) : secondary field pointer (if any) before spectral fit (GT1)
!      - %IGFP : the data array pointers for horizontally interpolated fields (DYNFP)
!      - %ISET : the processor (V-)set number for each level of each field
!      - %LLXR : .TRUE. if the field is an auxilary one (not to be written out to file)
!      - %ILOC : the location of the field in the local V-set spectral array
!      - %ISPD : the number of arrays the field is spread over (for stretching only)

! Interface :
! ---------
!    Empty.

! External :
! --------
!    None.

! Method :
! ------
!    See Documentation.

! Reference :
! ---------
!    Fullpos technical & users guide.

! Author :
! ------
!    Ryad El Khatib *METEO-FRANCE* thanks to Mike Fisher *ECMWF*

! Modifications :
! -------------
! Original : 2000-08-18
! R. El Khatib : 01-03-28 Redefinition of %ILED
! R. El Khatib : 03-02-05 Split module into fullpos_mix + TYPE_FPRQDYN
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!-----------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE
SAVE

PRIVATE
PUBLIC TYPE_FPRQDYN      ,ALLOCATE_FPRQDYN, DEALLOCATE_FPRQDYN, &
 & ALLOCATED_FPRQDYN ,INQUIRE_FPRQDYN  

TYPE TYPE_FPRQDYN
LOGICAL,   POINTER :: LL(:)
LOGICAL,   POINTER :: LLXR(:)
INTEGER(KIND=JPIM), POINTER :: ISF(:)
INTEGER(KIND=JPIM), POINTER :: ISKP(:)
INTEGER(KIND=JPIM), POINTER :: ILED(:)
INTEGER(KIND=JPIM), POINTER :: ILEV(:)
INTEGER(KIND=JPIM), POINTER :: ILVP(:,:)
INTEGER(KIND=JPIM), POINTER :: IDOM(:,:)
INTEGER(KIND=JPIM), POINTER :: IDMP(:,:,:)
INTEGER(KIND=JPIM), POINTER :: IGPX(:)
INTEGER(KIND=JPIM), POINTER :: IGT1(:,:)
INTEGER(KIND=JPIM), POINTER :: IGFP(:)
INTEGER(KIND=JPIM), POINTER :: ISET(:,:)
INTEGER(KIND=JPIM), POINTER :: ILOC(:,:)
INTEGER(KIND=JPIM), POINTER :: ISPD(:,:)
END TYPE TYPE_FPRQDYN

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE ALLOCATE_FPRQDYN(YDTYPE,KDOM,KLEV,KDIM)

TYPE(TYPE_FPRQDYN), INTENT(INOUT) :: YDTYPE
INTEGER(KIND=JPIM), INTENT(IN)    :: KLEV, KDOM, KDIM 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

LOGICAL :: LL_FIRST_CALL=.TRUE.

! KDOM : number of subdomains
! KLEV : number of levels
! KDIM : number of fields

IF (LHOOK) CALL DR_HOOK('TYPE_FPRQDYNS:ALLOCATE_FPRQDYN',0,ZHOOK_HANDLE)

IF (LL_FIRST_CALL) THEN
  NULLIFY(YDTYPE%LL)
  NULLIFY(YDTYPE%LLXR)
  NULLIFY(YDTYPE%ISF)
  NULLIFY(YDTYPE%ISKP)
  NULLIFY(YDTYPE%ILED)
  NULLIFY(YDTYPE%ILEV)
  NULLIFY(YDTYPE%ILVP)
  NULLIFY(YDTYPE%IDOM)
  NULLIFY(YDTYPE%IDMP)
  NULLIFY(YDTYPE%IGPX)
  NULLIFY(YDTYPE%IGT1)
  NULLIFY(YDTYPE%IGFP)
  NULLIFY(YDTYPE%ISET)
  NULLIFY(YDTYPE%ILOC)
  NULLIFY(YDTYPE%ISPD)
  LL_FIRST_CALL=.FALSE.
ENDIF

CALL DEALLOCATE_FPRQDYN(YDTYPE)

ALLOCATE(YDTYPE%LL(KDIM))
ALLOCATE(YDTYPE%LLXR(KDIM))
ALLOCATE(YDTYPE%ISF(KDIM))
ALLOCATE(YDTYPE%ISKP(KDIM))
ALLOCATE(YDTYPE%ILED(KDIM))
ALLOCATE(YDTYPE%ILEV(KDIM))
ALLOCATE(YDTYPE%ILVP(KLEV,KDIM))
ALLOCATE(YDTYPE%IDOM(KLEV,KDIM))
ALLOCATE(YDTYPE%IDMP(KDOM,KLEV,KDIM))
ALLOCATE(YDTYPE%IGPX(KDIM))
ALLOCATE(YDTYPE%IGT1(KDIM,0:2))
ALLOCATE(YDTYPE%IGFP(KDIM))
ALLOCATE(YDTYPE%ISET(KLEV,KDIM))
ALLOCATE(YDTYPE%ILOC(KLEV,KDIM))
ALLOCATE(YDTYPE%ISPD(KLEV,KDIM))

IF (LHOOK) CALL DR_HOOK('TYPE_FPRQDYNS:ALLOCATE_FPRQDYN',1,ZHOOK_HANDLE)

END SUBROUTINE ALLOCATE_FPRQDYN

SUBROUTINE DEALLOCATE_FPRQDYN(YDTYPE)

TYPE(TYPE_FPRQDYN), INTENT(INOUT) :: YDTYPE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('TYPE_FPRQDYNS:DEALLOCATE_FPRQDYN',0,ZHOOK_HANDLE)
IF (ASSOCIATED(YDTYPE%LL))   DEALLOCATE(YDTYPE%LL)
IF (ASSOCIATED(YDTYPE%LLXR)) DEALLOCATE(YDTYPE%LLXR)
IF (ASSOCIATED(YDTYPE%ISF))  DEALLOCATE(YDTYPE%ISF)
IF (ASSOCIATED(YDTYPE%ISKP)) DEALLOCATE(YDTYPE%ISKP)
IF (ASSOCIATED(YDTYPE%ILED)) DEALLOCATE(YDTYPE%ILED)
IF (ASSOCIATED(YDTYPE%ILEV)) DEALLOCATE(YDTYPE%ILEV)
IF (ASSOCIATED(YDTYPE%ILVP)) DEALLOCATE(YDTYPE%ILVP)
IF (ASSOCIATED(YDTYPE%IDOM)) DEALLOCATE(YDTYPE%IDOM)
IF (ASSOCIATED(YDTYPE%IDMP)) DEALLOCATE(YDTYPE%IDMP)
IF (ASSOCIATED(YDTYPE%IGPX)) DEALLOCATE(YDTYPE%IGPX)
IF (ASSOCIATED(YDTYPE%IGT1)) DEALLOCATE(YDTYPE%IGT1)
IF (ASSOCIATED(YDTYPE%IGFP)) DEALLOCATE(YDTYPE%IGFP)
IF (ASSOCIATED(YDTYPE%ISET)) DEALLOCATE(YDTYPE%ISET)
IF (ASSOCIATED(YDTYPE%ILOC)) DEALLOCATE(YDTYPE%ILOC)
IF (ASSOCIATED(YDTYPE%ILOC)) DEALLOCATE(YDTYPE%ISPD)

IF (LHOOK) CALL DR_HOOK('TYPE_FPRQDYNS:DEALLOCATE_FPRQDYN',1,ZHOOK_HANDLE)

END SUBROUTINE DEALLOCATE_FPRQDYN

LOGICAL FUNCTION ALLOCATED_FPRQDYN(YDTYPE)

TYPE(TYPE_FPRQDYN), INTENT(IN) :: YDTYPE 

ALLOCATED_FPRQDYN = &
 & ASSOCIATED(YDTYPE%LL  ).AND. &
 & ASSOCIATED(YDTYPE%LLXR).AND. &
 & ASSOCIATED(YDTYPE%ISF ).AND. &
 & ASSOCIATED(YDTYPE%ISKP).AND. &
 & ASSOCIATED(YDTYPE%ILED).AND. &
 & ASSOCIATED(YDTYPE%ILEV).AND. &
 & ASSOCIATED(YDTYPE%ILVP).AND. &
 & ASSOCIATED(YDTYPE%IDOM).AND. &
 & ASSOCIATED(YDTYPE%IDMP).AND. &
 & ASSOCIATED(YDTYPE%IGPX).AND. &
 & ASSOCIATED(YDTYPE%IGT1).AND. &
 & ASSOCIATED(YDTYPE%IGFP).AND. &
 & ASSOCIATED(YDTYPE%ISET).AND. &
 & ASSOCIATED(YDTYPE%ILOC).AND. &
 & ASSOCIATED(YDTYPE%ISPD)  

END FUNCTION ALLOCATED_FPRQDYN

SUBROUTINE INQUIRE_FPRQDYN(YDTYPE,CDNAME,KULOUT)

! YDTYPE  : structure variable
! CDNAME : structure name
! KULOUT : output logical unit number

TYPE(TYPE_FPRQDYN), INTENT(IN) :: YDTYPE
CHARACTER(LEN=*),   INTENT(IN) :: CDNAME
INTEGER(KIND=JPIM),          INTENT(IN) :: KULOUT

! CLFMT  : format for output

CHARACTER(LEN=36) :: CLFMT
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('TYPE_FPRQDYNS:INQUIRE_FPRQDYN',0,ZHOOK_HANDLE)
CLFMT='(1X,''ARRAY '',A,A7,'' ALLOCATED '',8I8)'

WRITE(KULOUT,CLFMT) CDNAME,'%LL    ',SIZE(YDTYPE%LL  ),SHAPE(YDTYPE%LL  )
WRITE(KULOUT,CLFMT) CDNAME,'%LLXR  ',SIZE(YDTYPE%LLXR),SHAPE(YDTYPE%LLXR)
WRITE(KULOUT,CLFMT) CDNAME,'%ISF   ',SIZE(YDTYPE%ISF) ,SHAPE(YDTYPE%ISF )
WRITE(KULOUT,CLFMT) CDNAME,'%ISKP  ',SIZE(YDTYPE%ISKP),SHAPE(YDTYPE%ISKP)
WRITE(KULOUT,CLFMT) CDNAME,'%ILED  ',SIZE(YDTYPE%ILED),SHAPE(YDTYPE%ILED)
WRITE(KULOUT,CLFMT) CDNAME,'%ILEV  ',SIZE(YDTYPE%ILEV),SHAPE(YDTYPE%ILEV)
WRITE(KULOUT,CLFMT) CDNAME,'%ILVP  ',SIZE(YDTYPE%ILVP),SHAPE(YDTYPE%ILVP)
WRITE(KULOUT,CLFMT) CDNAME,'%IDOM  ',SIZE(YDTYPE%IDOM),SHAPE(YDTYPE%IDOM)
WRITE(KULOUT,CLFMT) CDNAME,'%IDMP  ',SIZE(YDTYPE%IDMP),SHAPE(YDTYPE%IDMP)
WRITE(KULOUT,CLFMT) CDNAME,'%IGPX  ',SIZE(YDTYPE%IGPX),SHAPE(YDTYPE%IGPX)
WRITE(KULOUT,CLFMT) CDNAME,'%IGT1  ',SIZE(YDTYPE%IGT1),SHAPE(YDTYPE%IGT1)
WRITE(KULOUT,CLFMT) CDNAME,'%IGFP  ',SIZE(YDTYPE%IGFP),SHAPE(YDTYPE%IGFP)
WRITE(KULOUT,CLFMT) CDNAME,'%ISET  ',SIZE(YDTYPE%ISET),SHAPE(YDTYPE%ISET)
WRITE(KULOUT,CLFMT) CDNAME,'%ILOC  ',SIZE(YDTYPE%ILOC),SHAPE(YDTYPE%ILOC)
WRITE(KULOUT,CLFMT) CDNAME,'%ISPD  ',SIZE(YDTYPE%ISPD),SHAPE(YDTYPE%ISPD)
IF (LHOOK) CALL DR_HOOK('TYPE_FPRQDYNS:INQUIRE_FPRQDYN',1,ZHOOK_HANDLE)

END SUBROUTINE INQUIRE_FPRQDYN

END MODULE TYPE_FPRQDYNS
