MODULE VARBC_eval
!-----------------------------------------------------------------------
!   VARBC_EVAL - Module for variational bias correction

!   Purpose. 
!   --------
!   Bias evaluation, including TL and AD

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   D. Dee    2008/10/01 : Original
!   D. Li     29-Jan-2009: Add switch of mode-based VarBC 
!   G. Mozdzynski  03-Jun-2009 VarBC reproducibility rework
!   T. Wilhelmsson 03-Mar-2010 More VarBC reproducibility rework
!------------------------------------------------------------------------

USE parkind1, ONLY : JPIM, JPRB
USE yomhook , ONLY : LHOOK, DR_HOOK
USE yomancs , ONLY : RMDI
USE yomdimo , ONLY : NOBTOT
USE yomlun  , ONLY : NULERR, nulout
USE yomvar  , ONLY : LREPRO4DVAR
USE oml_mod , ONLY : OML_MY_THREAD
USE yomdb

USE varbc_setup, ONLY : YVARBC, NHSTBINS, NHSTFGDEP_COMP, &
                      & APARAMS_COMP, NPARAMS_COMP, IMASKRS
USE varbc_rad  , ONLY : NCLD_AIRS                   
!------------------------------------------------------------------------

IMPLICIT NONE
SAVE

PRIVATE
PUBLIC varbc_bias, varbc_biastl, varbc_biasad

INTERFACE varbc_bias
  MODULE PROCEDURE eval_bias
END INTERFACE varbc_bias

INTERFACE varbc_biastl
  MODULE PROCEDURE eval_biastl
END INTERFACE varbc_biastl

INTERFACE varbc_biasad
  MODULE PROCEDURE eval_biasad
END INTERFACE varbc_biasad

CHARACTER(LEN=*), PARAMETER :: modname = 'VarBC_eval'

#include "abor1.intfb.h"

! ========
CONTAINS
! ========

SUBROUTINE eval_bias (kxvarbc, kmxcount, kcount, kdlen, klen, &
                            &  kpred, ppred, pomf, pbias)  
!-----------------------------------------------------------------------
!   EVAL_BIAS - Evaluate bias

!   Purpose. 
!   --------
!   Compute bias based on a linear predictor model.

!   Input arguments: 
!      kxvarbc  = VarBC group index
!      kmxcount = maximum number of data per obs
!      kcount   = actual number of data for each obs
!      kdlen    = first dimension: maximum number of obs
!      klen     = actual number of obs (profiles)
!      kpred    = max predictor index
!      ppred    = bias predictors
!      pomf     = uncorrected observation minus first-guess

!   Output arguments:
!      pbias    = bias estimates

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2004/03/19
!        P.Burton :  29-09-04 LVARBC cleanup
!        T.Auligne:  09-12-05 Included scan bias correction
!        D.Dee    : 2008/03/01 Moved to this module
!-----------------------------------------------------------------------

! Arguments

INTEGER(KIND=JPIM),INTENT(IN)  :: kmxcount 
INTEGER(KIND=JPIM),INTENT(IN)  :: kdlen 
INTEGER(KIND=JPIM),INTENT(IN)  :: kxvarbc(kdlen,kmxcount) 
INTEGER(KIND=JPIM),INTENT(IN)  :: kcount(kdlen) 
INTEGER(KIND=JPIM),INTENT(IN)  :: klen 
INTEGER(KIND=JPIM),INTENT(IN)  :: kpred 
REAL(KIND=JPRB)   ,INTENT(IN)  :: ppred(0:kpred,klen) 
REAL(KIND=JPRB)   ,INTENT(IN)  :: pomf(kdlen,kmxcount) 
REAL(KIND=JPRB)   ,INTENT(OUT) :: pbias(kdlen,kmxcount) 

INTEGER(KIND=JPIM) :: ipf, ibd, ix, ip, ib
INTEGER(KIND=JPIM) :: itid
REAL(KIND=JPRB)    :: zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (eval_bias)'

!------------------------------------------------------------------------

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

itid = OML_MY_THREAD()
pbias(1:kdlen,1:kmxcount) = 0.0_JPRB

! Bias correction according to VarBC group index (varbc_ix@body) 
! --------------------------------------------------------------
DO ibd = 1, kmxcount  
  DO ipf = 1, klen
!   write(nulout,'('' ibd, ipf= '',3(i10,x))') ibd, ipf, kcount(ipf)
!   call flush(nulout)
    IF (ibd>kcount(ipf)) CYCLE
    ix = kxvarbc(ipf,ibd)            
    IF (ix==0) CYCLE

    ! Bias prediction
    !----------------
!   write(nulout,'('' ix= '',i10)') ix
!   call flush(nulout)
    IF (ANY(YVARBC(ix)%APARAMS==RMDI)) THEN
!     write(nulout,'('' pbias set to RMDI'')')
!     call flush(nulout)
      pbias(ipf,ibd) = RMDI
    ELSE  
!     write(nulout,'('' pbias not set to RMDI'',i10)') YVARBC(ix)%NPARAM
!     call flush(nulout)
      DO ip = 1, YVARBC(ix)%NPARAM
!       write(nulout,'('' ip= '',i10,x,f15.4,x,i10,x,f15.4)') ip,YVARBC(ix)%APARAMS(ip),&
!                                     & YVARBC(ix)%NPREDCS(ip),ppred(YVARBC(ix)%NPREDCS(ip),ipf) 
!       call flush(nulout)
        pbias(ipf,ibd) = pbias(ipf,ibd) &
                     & + YVARBC(ix)%APARAMS(ip) * ppred(YVARBC(ix)%NPREDCS(ip),ipf)
      ENDDO
    ENDIF  

    ! Accumulate statistics for histogram (to be finalized in varbc_setup)
    ! --------------------------------------------------------------------
    IF (pomf(ipf,ibd)/=RMDI) THEN

      ib = NINT( NHSTBINS * ( 0.5 + pomf(ipf,ibd) / YVARBC(ix)%DFGDEP ) )
      IF ((ib>0).AND.(ib<=NHSTBINS)) NHSTFGDEP_COMP(ix,ib,itid) = &
                                   & NHSTFGDEP_COMP(ix,ib,itid) + 1
    ENDIF  
  ENDDO
ENDDO

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE eval_bias
!-----------------------------------------------------------------------------

SUBROUTINE eval_biastl (kxvarbc, kmxcount, kcount, kdlen, klen, &
                      & kpred, ppred, kvisnir, plat, plon, pomf, pbias)  
!-----------------------------------------------------------------------
!   EVAL_BIAS - Evaluate bias

!   Purpose. 
!   --------
!   Compute bias based on a linear predictor model.

!   Input arguments: 
!      kxvarbc  = VarBC group index
!      kmxcount = maximum number of data per obs
!      kcount   = actual number of data for each obs
!      kdlen    = first dimension: maximum number of obs
!      klen     = actual number of obs (profiles)
!      kpred    = max predictor index
!      ppred    = bias predictors
!      kvisnir  = AIRS Visible NearInfraRed cloud percentage
!      plat     = latitude in degrees
!      plon     = longitude in degrees
!      pomf     = uncorrected observation minus first-guess

!   Output arguments:
!      pbias    = bias estimates

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2004/03/19
!        T.Auligne :  01-04-04 Created TL from RADTRB
!        T.Auligne :  01-04-04 Added masking option
!        T.Auligne :  09-12-05 Included scan bias correction
!        T.Auligne :  09-12-05 Added Radiosonde mask option
!        T.Auligne :  09-12-14 Second metric for bias parameters update
!        D.Dee    : 2008/03/01 Moved to this module
!-----------------------------------------------------------------------

! Arguments

INTEGER(KIND=JPIM),INTENT(IN)  :: kmxcount 
INTEGER(KIND=JPIM),INTENT(IN)  :: kdlen 
INTEGER(KIND=JPIM),INTENT(IN)  :: kxvarbc(kdlen,kmxcount) 
INTEGER(KIND=JPIM),INTENT(IN)  :: kcount(kdlen) 
INTEGER(KIND=JPIM),INTENT(IN)  :: klen 
INTEGER(KIND=JPIM),INTENT(IN)  :: kpred 
REAL(KIND=JPRB)   ,INTENT(IN)  :: ppred(0:kpred,klen) 
INTEGER(KIND=JPIM),INTENT(IN)  :: kvisnir(kdlen)
REAL(KIND=JPRB)   ,INTENT(IN)  :: plat(kdlen)
REAL(KIND=JPRB)   ,INTENT(IN)  :: plon(kdlen)
REAL(KIND=JPRB)   ,INTENT(IN)  :: pomf(kdlen,kmxcount) 
REAL(KIND=JPRB)   ,INTENT(OUT) :: pbias(kdlen,kmxcount) 

INTEGER(KIND=JPIM) :: ipf, ibd, ix, ip
REAL(KIND=JPRB)    :: zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (eval_biastl)'

!------------------------------------------------------------------------

IF (lhook) CALL dr_hook(myname,0,zhook_handle)
pbias(1:kdlen,1:kmxcount) = 0.0_JPRB

! Bias correction according to VarBC group index (varbc_ix@body) 
! --------------------------------------------------------------
DO ibd = 1, kmxcount  
  DO ipf = 1, klen
    IF (ibd>kcount(ipf)) CYCLE
    ix = kxvarbc(ipf,ibd)            
    IF (ix==0) CYCLE

    ! Masking
    ! -------
    IF ((YVARBC(ix)%LLMASKCLD) .AND. & 
         & (kvisnir(ipf) > NCLD_AIRS)) CYCLE  !! AIRS VISNIR cloud cover

    IF ((YVARBC(ix)%LLMASKRS) .AND. & 
         & (IMASKRS(MOD(NINT(plon(ipf)),360)+1,NINT(90.0-plat(ipf))+1) == 0)) &
         &                             CYCLE  !! Not near radiosondes

    ! Bias evaluation
    !----------------
    DO ip = 1, YVARBC(ix)%NPARAM
      IF (.NOT. YVARBC(ix)%LLCONST(ip)) &
                          & pbias(ipf,ibd) = pbias(ipf,ibd) &
                          & + YVARBC(ix)%APARAMS(ip) * ppred(YVARBC(ix)%NPREDCS(ip),ipf)
    ENDDO

  ENDDO
ENDDO

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE eval_biastl
!-----------------------------------------------------------------------------

SUBROUTINE eval_biasad (kset, kxvarbc, kmxcount, kcount, kdlen, klen, &
                      & kpred, ppred, kvisnir, plat, plon, pomf, pbias)  
!-----------------------------------------------------------------------
!   EVAL_BIAS - Evaluate bias

!   Purpose. 
!   --------
!   Compute bias based on a linear predictor model.

!   Input arguments:
!      kset     = obs set number 
!      kxvarbc  = VarBC group index
!      kmxcount = maximum number of data per obs
!      kcount   = actual number of data for each obs
!      kdlen    = first dimension: maximum number of obs
!      klen     = actual number of obs (profiles)
!      kpred    = max predictor index
!      ppred    = bias predictors
!      kvisnir  = AIRS Visible NearInfraRed cloud percentage
!      plat     = latitude in degrees
!      plon     = longitude in degrees
!      pomf     = uncorrected observation minus first-guess
!      pbias    = bias estimates

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2004/03/19
!        P.Burton :  29-09-04 LVARBC cleanup
!        P.Burton    04/09/09 : Fix reproducibility of sat. bias correction
!        T.Auligne:  01-04-04 : Added masking option
!        T.Auligne:  09-12-05 : Included scan bias correction
!        T.Auligne   09-12-05 : Added Radiosonde mask option
!        T.Auligne   09-12-14 : Second metric for bias parameters update
!        D.Dee    : 2008/03/01 Moved to this module
!-----------------------------------------------------------------------

! Arguments

INTEGER(KIND=JPIM),INTENT(IN)  :: kset 
INTEGER(KIND=JPIM),INTENT(IN)  :: kmxcount 
INTEGER(KIND=JPIM),INTENT(IN)  :: kdlen 
INTEGER(KIND=JPIM),INTENT(IN)  :: kxvarbc(kdlen,kmxcount) 
INTEGER(KIND=JPIM),INTENT(IN)  :: kcount(kdlen) 
INTEGER(KIND=JPIM),INTENT(IN)  :: klen 
INTEGER(KIND=JPIM),INTENT(IN)  :: kpred 
REAL(KIND=JPRB)   ,INTENT(IN)  :: ppred(0:kpred,klen) 
INTEGER(KIND=JPIM),INTENT(IN)  :: kvisnir(kdlen)
REAL(KIND=JPRB)   ,INTENT(IN)  :: plat(kdlen)
REAL(KIND=JPRB)   ,INTENT(IN)  :: plon(kdlen)
REAL(KIND=JPRB)   ,INTENT(IN)  :: pomf(kdlen,kmxcount) 
REAL(KIND=JPRB)   ,INTENT(IN) :: pbias(kdlen,kmxcount) 

INTEGER(KIND=JPIM) :: ipf, ibd, ix, ip
REAL(KIND=JPRB)    :: zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (eval_biasad)'

!------------------------------------------------------------------------

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

! Bias correction according to VarBC group index (varbc_ix@body) 
! --------------------------------------------------------------
DO ibd = 1, kmxcount  
  DO ipf = 1, klen
    IF (ibd>kcount(ipf)) CYCLE
    ix = kxvarbc(ipf,ibd)            
    IF (ix==0) CYCLE

    IF ((YVARBC(ix)%LLMASKCLD) .AND. & 
         & (kvisnir(ipf) > NCLD_AIRS)) CYCLE  !! AIRS VISNIR cloud cover

    IF ((YVARBC(ix)%LLMASKRS) .AND. & 
         & (IMASKRS(MOD(NINT(plon(ipf)),360)+1,NINT(90.0-plat(ipf))+1) == 0)) &
         &                             CYCLE  !! Not near radiosondes

    ! Bias evaluation
    !----------------
    IF (.NOT. LREPRO4DVAR) THEN
      DO ip = 1, YVARBC(ix)%NPARAM
        IF (.NOT. YVARBC(ix)%LLCONST(ip)) THEN
          APARAMS_COMP(kset,ip,ix) = APARAMS_COMP(kset,ip,ix)   &
            & + pbias(ipf,ibd)*ppred(YVARBC(ix)%NPREDCS(ip),ipf)
        ENDIF
      ENDDO
    ELSE
!$OMP CRITICAL
      DO ip = 1, YVARBC(ix)%NPARAM
        IF (.NOT. YVARBC(ix)%LLCONST(ip)) THEN
          NPARAMS_COMP(ip,ix) = NPARAMS_COMP(ip,ix) + 1
          IF (NPARAMS_COMP(ip,ix) > NOBTOT) THEN
            WRITE(nulerr,*) myname//': group ', ix, ' parameter: ', ip, 'nobtot', nobtot
            CALL ABOR1(myname//': NPARAMS_COMP > NOBTOT')
          ENDIF
          APARAMS_COMP(NPARAMS_COMP(ip,ix),ip,ix) = &
            & pbias(ipf,ibd)*ppred(YVARBC(ix)%NPREDCS(ip),ipf)
        ENDIF
      ENDDO
!$OMP END CRITICAL
    ENDIF
  ENDDO
ENDDO

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE eval_biasad
!-----------------------------------------------------------------------------

END MODULE VARBC_eval
