MODULE varbc_rad
!-----------------------------------------------------------------------
!   VARBC_RAD - Module for variational bias correction (radiance data)

!   Purpose. 
!   --------
!   Data structures and controls for variational bias correction (radiance data)

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2008/10/01
!   Qifeng Lu   2009/Mar/11  Add FY3A IRAS, MWTS, MWHS and MWRI
!   11/03/2009   W. Bell  Windsat changes
!------------------------------------------------------------------------
USE parkind1, ONLY : JPIM, JPRB
USE parcma  , ONLY : JPMXTOCH 
USE yomhook , ONLY : LHOOK, DR_HOOK
USE yomlun  , ONLY : NULOUT, NULNAM
USE yomancs , ONLY : RMDI 
USE yomcosjo, ONLY : JOT
USE yomcoctp, ONLY : NSATEM, NGTHRB
USE yomct0b , ONLY : LECMWF
USE yomtvrad, ONLY : SATGRP_TABLE, NSATGRP, MXSENSOR, &
                   & msensor_HIRS, msensor_MSU, msensor_SSU, &
                   & msensor_IRAS, msensor_MWTS, msensor_MWHS, msensor_MWRI, &
                   & msensor_AMSUA, msensor_AMSUB, msensor_MHS, msensor_SSMI, &
                   & msensor_METEOSAT, msensor_GOESIMG, msensor_AIRS, &
                   & msensor_MTSATIMG, msensor_MSG, msensor_MSG_HR, msensor_VISSR, &
                   & msensor_SSMIS, msensor_AMSRE, msensor_TMI, &
                   & msensor_IASI, msensor_WINDSAT,  &
                   & AIRSCHAN, IASICHAN, M__NumIASICHANs
USE varbc_pred, ONLY: NPREDCS_RAD

USE yomdb

IMPLICIT NONE

SAVE

PRIVATE
PUBLIC lbc_rad
PUBLIC varbc_rad_config, varbc_rad_groups
PUBLIC varbc_rad_varbcix, varbc_rad_pred
PUBLIC varbc_rad_min, varbc_rad_groupdescr
PUBLIC lbc_amsua, lbc_amsub, lbc_mhs, lbc_ssmi, lbc_hirs
PUBLIC lbc_iras, lbc_mwts, lbc_mwhs, lbc_mwri

PUBLIC ncld_AIRS
PUBLIC yconfig, type_config

INTERFACE varbc_rad_config
  MODULE PROCEDURE config
END INTERFACE varbc_rad_config

INTERFACE varbc_rad_groups
  MODULE PROCEDURE find_groups
END INTERFACE varbc_rad_groups

INTERFACE varbc_rad_groupdescr
  MODULE PROCEDURE groupdescr
END INTERFACE varbc_rad_groupdescr

INTERFACE varbc_rad_pred
  MODULE PROCEDURE get_predictors
END INTERFACE varbc_rad_pred

INTERFACE varbc_rad_varbcix
  MODULE PROCEDURE set_varbcix
END INTERFACE varbc_rad_varbcix

INTERFACE varbc_rad_min
  MODULE PROCEDURE get_min
END INTERFACE varbc_rad_min

CHARACTER(LEN=*), PARAMETER :: modname = 'VarBC_RAD'

! Table for available satellite/sensor combinations 
! -------------------------------------------------
TYPE type_sensors
  INTEGER(KIND=JPIM) :: nbufrid 
  INTEGER(KIND=JPIM) :: nsensor 
END TYPE type_sensors

INTEGER(KIND=JPIM), PARAMETER :: jpmxns = 1000   ! max number of sat/sensor combinations
TYPE(type_sensors) :: ysensors(1:jpmxns)

INTEGER(KIND=JPIM), PARAMETER :: jpmxnp = SIZE(NPREDCS_RAD) ! max number of parameters per group

! Table for settings by sensor/channel
! ------------------------------------
TYPE type_config
  INTEGER(KIND=JPIM) :: nparam            ! number of parameters
  INTEGER(KIND=JPIM) :: npredcs(jpmxnp)   ! list of predictors
  LOGICAL            :: llconst(jpmxnp)   ! flags for constant parameters
  REAL(KIND=JPRB)    :: zparams(jpmxnp)   ! a priori parameter values
  REAL(KIND=JPRB)    :: dfgdep            ! histogram range, for estimating the mode
  INTEGER(KIND=JPIM) :: ncstart           ! cold start option
  INTEGER(KIND=JPIM) :: nbgstdv           ! eqv #obs for background constraint
  LOGICAL            :: lreset            ! flag to reset parameters
  LOGICAL            :: llmode            ! flag for mode correction
  LOGICAL            :: llmaskrs          ! flag for radiosonde masking
  LOGICAL            :: llmaskcld         ! flag for AIRS VISNIR cloud-cover masking
END TYPE type_config

!-----------------------------------------------------------------------
! Namelist parameters
!-----------------------------------------------------------------------
TYPE(type_config) :: yconfig(0:MXSENSOR,1:JPMXTOCH)
LOGICAL            :: lbc_rad
LOGICAL            :: lbc_HIRS,  lbc_MSU,      lbc_SSU,   lbc_AMSUA, lbc_AMSUB, &
                    & lbc_IRAS,  lbc_MWTS,     lbc_MWHS,  lbc_MWRI, &
                    & lbc_SSMI,  lbc_METEOSAT, lbc_AIRS,  lbc_IASI,  lbc_GOESIMG, &
                    & lbc_MSG,   lbc_MTSATIMG, lbc_VISSR, lbc_MHS,   lbc_SSMIS, &
                    & lbc_AMSRE, lbc_TMI,      lbc_MSG_HR, lbc_WINDSAT
INTEGER(KIND=JPIM) :: nbg_HIRS,  nbg_MSU,      nbg_SSU,   nbg_AMSUA, nbg_AMSUB, &
                    & nbg_IRAS,  nbg_MWTS,     nbg_MWHS,  nbg_MWRI, &
                    & nbg_SSMI,  nbg_METEOSAT, nbg_AIRS,  nbg_IASI,  nbg_GOESIMG, &
                    & nbg_MSG,   nbg_MTSATIMG, nbg_VISSR, nbg_MHS,   nbg_SSMIS, &
                    & nbg_AMSRE, nbg_TMI,      nbg_MSG_HR, nbg_WINDSAT
LOGICAL            :: lconst             ! Keep all VarBC parameters fixed
LOGICAL            :: lmode              ! Correct pdf mode rather than mean
LOGICAL            :: lmaskrs            ! Radiosonde vicinity masking
LOGICAL            :: lcld_AIRS          ! AIRS VISNIR cloud cover masking
INTEGER(KIND=JPIM) :: ncld_AIRS          ! AIRS VISNIR cloud cover percentage

NAMELIST /NAMVARBC_RAD/ yconfig, &
                      & lbc_rad, &
                      & lbc_HIRS,  lbc_MSU,      lbc_SSU,   lbc_AMSUA, lbc_AMSUB, &
                      & lbc_IRAS,  lbc_MWTS,     lbc_MWHS,  lbc_MWRI, &
                      & lbc_SSMI,  lbc_METEOSAT, lbc_AIRS,  lbc_IASI,  lbc_GOESIMG, &
                      & lbc_MSG,   lbc_MTSATIMG, lbc_VISSR, lbc_MHS,   lbc_SSMIS, &
                      & lbc_AMSRE, lbc_TMI,      lbc_MSG_HR, lbc_WINDSAT,  & 
                      & nbg_HIRS,  nbg_MSU,      nbg_SSU,   nbg_AMSUA, nbg_AMSUB, &
                      & nbg_IRAS,  nbg_MWTS,     nbg_MWHS,  nbg_MWRI, &
                      & nbg_SSMI,  nbg_METEOSAT, nbg_AIRS,  nbg_IASI,  nbg_GOESIMG, &
                      & nbg_MSG,   nbg_MTSATIMG, nbg_VISSR, nbg_MHS,   nbg_SSMIS, &
                      & nbg_AMSRE, nbg_TMI,      nbg_MSG_HR, nbg_WINDSAT, &
                      & lconst, lmode, lmaskrs, lcld_AIRS, ncld_AIRS
!-----------------------------------------------------------------------

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "abor1.intfb.h"
#include "posnam.intfb.h"
#include "read_iasichans.intfb.h"

! ========
CONTAINS
! ========

SUBROUTINE config

!-----------------------------------------------------------------------
!   config - Configure radiance VarBC

!   Interface.
!   ----------
!   CALL config

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2008/03/01
!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: i, is, ic, ierr
REAL(KIND=JPRB)    :: zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (config)'

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

WRITE(nulout,'(/,a)') myname//': Configuring radiance VarBC'


! default settings:
! ----------------
DO is = 0, MXSENSOR
  DO ic = 1, JPMXTOCH
    yconfig(is,ic)%nparam     = 0  
    yconfig(is,ic)%npredcs(:) = 0
    yconfig(is,ic)%zparams(:) = RMDI
    yconfig(is,ic)%llconst(:) = .false.
    yconfig(is,ic)%ncstart    = 2
    yconfig(is,ic)%dfgdep     = 20.0_JPRB
    yconfig(is,ic)%nbgstdv    = 0
    yconfig(is,ic)%lreset     = .false.
    yconfig(is,ic)%llmode     = .false. 
    yconfig(is,ic)%llmaskrs   = .false. 
    yconfig(is,ic)%llmaskcld  = .false.
  ENDDO
ENDDO

DO ic = 1, 15
  SELECT case (ic)
  case (1:7,9:13)
    yconfig(msensor_HIRS, ic)%nparam = 8
    yconfig(msensor_HIRS, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
  case (14,15)
    yconfig(msensor_HIRS, ic)%nparam = 9
    yconfig(msensor_HIRS, ic)%npredcs(1:9) = (/0,1,2,5,6,8,9,10,12/)
  case (8)
    yconfig(msensor_HIRS,  8)%nparam = 4
    yconfig(msensor_HIRS,  8)%npredcs(1:4) = (/0,8,9,10/)
  END SELECT 
ENDDO

DO ic = 1, 4
  yconfig(msensor_MSU, ic)%nparam = 8
  yconfig(msensor_MSU, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
ENDDO

DO ic = 1, 3
  yconfig(msensor_SSU, ic)%nparam = 8
  yconfig(msensor_SSU, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
ENDDO

DO ic = 1, 20
  SELECT case (ic)
  ! For 1:3 predictor 12 is used due to instrument biases.
  ! For the other channels it's RT biases.
  case (1:3,16:17,19,20)
    yconfig(msensor_IRAS, ic)%nparam = 9
    yconfig(msensor_IRAS, ic)%npredcs(1:9) = (/0,1,2,5,6,8,9,10,12/)
  case (4:8,10:15,18)
    yconfig(msensor_IRAS, ic)%nparam = 8
    yconfig(msensor_IRAS, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
  case (9)
  ! Qifeng used the 8-predictor model for channel 9
    yconfig(msensor_IRAS, ic)%nparam = 4
    yconfig(msensor_IRAS, ic)%npredcs(1:4) = (/0,8,9,10/)
  END SELECT
ENDDO

DO ic = 1, 4
  yconfig(msensor_MWTS, ic)%nparam = 8
  yconfig(msensor_MWTS, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
ENDDO

DO ic = 1, 5
  SELECT case (ic)
  case (1:2)
    yconfig(msensor_MWHS, ic)%nparam = 4
    yconfig(msensor_MWHS, ic)%npredcs(1:4) = (/0,8,9,10/)
  case (3:5)
    yconfig(msensor_MWHS, ic)%nparam = 8
    yconfig(msensor_MWHS, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
  END SELECT
ENDDO

DO ic = 1, 10
  yconfig(msensor_MWRI, ic)%nparam = 7
  yconfig(msensor_MWRI, ic)%npredcs(1:7) = (/0,3,4,7,8,9,10/)
ENDDO

DO ic = 3, 14
  SELECT case (ic)
  case (3)
    yconfig(msensor_AMSUA, ic)%nparam = 4
    yconfig(msensor_AMSUA, ic)%npredcs(1:4) = (/0,8,9,10/)
  case (4)
    yconfig(msensor_AMSUA, ic)%nparam = 8
    yconfig(msensor_AMSUA, ic)%npredcs(1:8) = (/0,8,9,10,15,16,17,18/)
  case (5)
    yconfig(msensor_AMSUA, ic)%nparam = 12
    yconfig(msensor_AMSUA, ic)%npredcs(1:12) = (/0,1,2,5,6,8,9,10,15,16,17,18/)
  case (6:14)
    yconfig(msensor_AMSUA, ic)%nparam = 8
    yconfig(msensor_AMSUA, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
  END SELECT 
ENDDO

DO ic = 2, 5
  SELECT case (ic)
  case (2)
    yconfig(msensor_AMSUB, ic)%nparam = 4
    yconfig(msensor_AMSUB, ic)%npredcs(1:4) = (/0,8,9,10/)
  case (3:5)
    yconfig(msensor_AMSUB, ic)%nparam = 8
    yconfig(msensor_AMSUB, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
  END SELECT 
ENDDO

DO ic = 2, 5
  SELECT case (ic)
  case (2)
    yconfig(msensor_MHS, ic)%nparam = 4
    yconfig(msensor_MHS, ic)%npredcs(1:4) = (/0,8,9,10/)
  case (3:5)
    yconfig(msensor_MHS, ic)%nparam = 8
    yconfig(msensor_MHS, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
  END SELECT 
ENDDO

DO ic = 1, 7
  yconfig(msensor_SSMI, ic)%nparam = 7
  yconfig(msensor_SSMI, ic)%npredcs(1:7) = (/0,3,4,7,8,9,10/)
ENDDO

yconfig(msensor_METEOSAT, 1)%nparam = 4
yconfig(msensor_METEOSAT, 1)%npredcs(1:4) = (/0,1,2,4/)
yconfig(msensor_METEOSAT, 2)%nparam = 1
yconfig(msensor_METEOSAT, 2)%npredcs(1:1) = (/0/)

DO ic = 2, 7
  SELECT case (ic)
  case (2:3)
    yconfig(msensor_MSG, ic)%nparam = 4
    yconfig(msensor_MSG, ic)%npredcs(1:4) = (/0,1,2,4/)
  case (6)
    yconfig(msensor_MSG, ic)%nparam = 1
    yconfig(msensor_MSG, ic)%npredcs(1:1) = (/0/)
  case (4,7)
    IF (.NOT. LECMWF) THEN
      yconfig(msensor_MSG, ic)%nparam = 1
      yconfig(msensor_MSG, ic)%npredcs(1:1) = (/0/)
    ENDIF
  END SELECT 
ENDDO

DO ic = 2, 7
  SELECT case (ic)
  case (2:4,6:7)
    yconfig(msensor_MSG_HR, ic)%nparam = 4
    yconfig(msensor_MSG_HR, ic)%npredcs(1:4) = (/0,1,2,4/)
  END SELECT 
ENDDO

yconfig(msensor_GOESIMG,  2)%nparam = 4
yconfig(msensor_GOESIMG,  2)%npredcs(1:4) = (/0,1,2,4/)
yconfig(msensor_GOESIMG,  3)%nparam = 1
yconfig(msensor_GOESIMG,  3)%npredcs(1:1) = (/0/)

yconfig(msensor_MTSATIMG, 1)%nparam = 1
yconfig(msensor_MTSATIMG, 1)%npredcs(1:1) = (/0/)
yconfig(msensor_MTSATIMG, 3)%nparam = 4
yconfig(msensor_MTSATIMG, 3)%npredcs(1:4) = (/0,1,2,4/)

yconfig(msensor_VISSR,    3)%nparam = 4
yconfig(msensor_VISSR,    3)%npredcs(1:4) = (/0,1,2,4/)

DO i = 1, SIZE(AIRSCHAN)
  ic = AIRSCHAN(i)
  SELECT case (ic)
  case (325:914)
    yconfig(msensor_AIRS, ic)%nparam = 4
    yconfig(msensor_AIRS, ic)%npredcs(1:4) = (/0,8,9,10/)
  case (1921:1928)
    yconfig(msensor_AIRS, ic)%nparam = 9
    yconfig(msensor_AIRS, ic)%npredcs(1:9) = (/0,1,2,5,6,8,9,10,12/)
  case default
    yconfig(msensor_AIRS, ic)%nparam = 8
    yconfig(msensor_AIRS, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
  END SELECT
ENDDO

IF (.NOT.(ALLOCATED(IASICHAN))) THEN
  CALL read_iasichans(ierr)        ! Get the IASI channels if required
  IF (ierr==1) CALL ABOR1(myname//': Problem reading IASICHAN')
ENDIF

IF (M__NumIASICHANs > 0) THEN
DO i = 1, SIZE(IASICHAN)
  ic = IASICHAN(i)
  SELECT case (ic)
  case (380:1180,1820:2200)
    yconfig(msensor_IASI, ic)%nparam = 4
    yconfig(msensor_IASI, ic)%npredcs(1:4) = (/0,8,9,10/)
  case default
    yconfig(msensor_IASI, ic)%nparam = 8
    yconfig(msensor_IASI, ic)%npredcs(1:8) = (/0,1,2,5,6,8,9,10/)
  END SELECT
ENDDO
ENDIF

DO ic = 1, 24
  SELECT case (ic)
  case (1,3:11,19:24)
    yconfig(msensor_SSMIS, ic)%nparam = 9
    yconfig(msensor_SSMIS, ic)%npredcs(1:9) = (/0,1,2,5,6,8,9,10,11/)
  case (2)
    yconfig(msensor_SSMIS, ic)%nparam = 4
    yconfig(msensor_SSMIS, ic)%npredcs(1:4) = (/0,8,9,10/)
  case (12:18)
    yconfig(msensor_SSMIS, ic)%nparam = 8
    yconfig(msensor_SSMIS, ic)%npredcs(1:8) = (/0,3,4,7,8,9,10,11/) 
  END SELECT 
ENDDO

DO ic = 1, 12
  yconfig(msensor_AMSRE, ic)%nparam = 8
  yconfig(msensor_AMSRE, ic)%npredcs(1:8) = (/0,3,4,7,8,9,10,11/) 
ENDDO

DO ic = 1, 9
  yconfig(msensor_TMI, ic)%nparam = 8
  yconfig(msensor_TMI, ic)%npredcs(1:8) = (/0,3,4,7,8,9,10,11/) 
ENDDO

DO ic = 1, 16
  yconfig(msensor_WINDSAT, ic)%nparam = 8
  yconfig(msensor_WINDSAT, ic)%npredcs(1:8) = (/0,3,4,7,8,9,10,11/) 
ENDDO

! Switches by sensor:
! ------------------
lbc_HIRS     = .true.    
lbc_MSU      = .true.
lbc_SSU      = .true.
lbc_IRAS     = .true.
lbc_MWTS     = .true.
lbc_MWHS     = .true.
lbc_MWRI     = .true.
lbc_AMSUA    = .true.
lbc_AMSUB    = .true.
lbc_MHS      = .true.
lbc_SSMI     = .true.
lbc_METEOSAT = .true.
lbc_GOESIMG  = .true.
lbc_MSG      = .true.
lbc_MSG_HR   = .true.
lbc_MTSATIMG = .true.
lbc_VISSR    = .true.
lbc_AIRS     = .true.
lbc_IASI     = .true.
lbc_SSMIS    = .true.
lbc_AMSRE    = .true.
lbc_TMI      = .true.
lbc_WINDSAT  = .true.
lbc_RAD      = .true.

! Background error parameters by sensor:
! -------------------------------------
nbg_HIRS     = 5000    
nbg_MSU      = 5000
nbg_SSU      = 5000
nbg_IRAS     = 5000
nbg_MWTS     = 5000
nbg_MWHS     = 5000
nbg_MWRI     = 5000
nbg_AMSUA    = 5000
nbg_AMSUB    = 5000
nbg_MHS      = 5000
nbg_SSMI     = 5000
nbg_METEOSAT = 5000
nbg_GOESIMG  = 5000
nbg_MSG      = 5000
nbg_MSG_HR   = 5000
nbg_MTSATIMG = 5000
nbg_VISSR    = 5000
nbg_AIRS     = 5000
nbg_IASI     = 5000
nbg_SSMIS    = 5000
nbg_AMSRE    = 5000
nbg_TMI      = 5000
nbg_WINDSAT  = 5000

! All parameters constant:
! -----------------------
lconst = .false.

! Correct mode rather than mean:
! -----------------------------
lmode = .false.

! Radiosonde vicinity masking:
! ---------------------------
lmaskrs = .false.

! AIRS VISNIR cloud cover masking:
! -------------------------------
lcld_AIRS = .false.

WRITE(nulout,'(a)') myname//': Reading namelist NAMVARBC_RAD, first iteration'
CALL posnam(nulnam,'NAMVARBC_RAD')
READ(nulnam,NAMVARBC_RAD)

IF (.NOT. lbc_HIRS    ) yconfig(msensor_HIRS    ,:)%nparam = 0
IF (.NOT. lbc_MSU     ) yconfig(msensor_MSU     ,:)%nparam = 0       
IF (.NOT. lbc_SSU     ) yconfig(msensor_SSU     ,:)%nparam = 0  
IF (.NOT. lbc_IRAS    ) yconfig(msensor_IRAS    ,:)%nparam = 0
IF (.NOT. lbc_MWTS    ) yconfig(msensor_MWTS    ,:)%nparam = 0
IF (.NOT. lbc_MWHS    ) yconfig(msensor_MWHS    ,:)%nparam = 0
IF (.NOT. lbc_MWRI    ) yconfig(msensor_MWRI    ,:)%nparam = 0
IF (.NOT. lbc_AMSUA   ) yconfig(msensor_AMSUA   ,:)%nparam = 0  
IF (.NOT. lbc_AMSUB   ) yconfig(msensor_AMSUB   ,:)%nparam = 0  
IF (.NOT. lbc_MHS     ) yconfig(msensor_MHS     ,:)%nparam = 0  
IF (.NOT. lbc_SSMI    ) yconfig(msensor_SSMI    ,:)%nparam = 0 
IF (.NOT. lbc_METEOSAT) yconfig(msensor_METEOSAT,:)%nparam = 0
IF (.NOT. lbc_GOESIMG ) yconfig(msensor_GOESIMG ,:)%nparam = 0
IF (.NOT. lbc_MSG     ) yconfig(msensor_MSG     ,:)%nparam = 0
IF (.NOT. lbc_MSG_HR  ) yconfig(msensor_MSG_HR  ,:)%nparam = 0
IF (.NOT. lbc_MTSATIMG) yconfig(msensor_MTSATIMG,:)%nparam = 0
IF (.NOT. lbc_VISSR   ) yconfig(msensor_VISSR   ,:)%nparam = 0
IF (.NOT. lbc_AIRS    ) yconfig(msensor_AIRS    ,:)%nparam = 0  
IF (.NOT. lbc_IASI    ) yconfig(msensor_IASI    ,:)%nparam = 0  
IF (.NOT. lbc_SSMIS   ) yconfig(msensor_SSMIS   ,:)%nparam = 0 
IF (.NOT. lbc_AMSRE   ) yconfig(msensor_AMSRE   ,:)%nparam = 0 
IF (.NOT. lbc_TMI     ) yconfig(msensor_TMI     ,:)%nparam = 0
IF (.NOT. lbc_WINDSAT ) yconfig(msensor_WINDSAT ,:)%nparam = 0

yconfig(msensor_HIRS    ,:)%nbgstdv = nbg_HIRS    
yconfig(msensor_MSU     ,:)%nbgstdv = nbg_MSU     
yconfig(msensor_SSU     ,:)%nbgstdv = nbg_SSU     
yconfig(msensor_IRAS    ,:)%nbgstdv = nbg_IRAS
yconfig(msensor_MWTS    ,:)%nbgstdv = nbg_MWTS
yconfig(msensor_MWHS    ,:)%nbgstdv = nbg_MWHS
yconfig(msensor_MWRI    ,:)%nbgstdv = nbg_MWRI
yconfig(msensor_AMSUA   ,:)%nbgstdv = nbg_AMSUA   
yconfig(msensor_AMSUB   ,:)%nbgstdv = nbg_AMSUB   
yconfig(msensor_MHS     ,:)%nbgstdv = nbg_MHS     
yconfig(msensor_SSMI    ,:)%nbgstdv = nbg_SSMI    
yconfig(msensor_METEOSAT,:)%nbgstdv = nbg_METEOSAT
yconfig(msensor_GOESIMG ,:)%nbgstdv = nbg_GOESIMG 
yconfig(msensor_MSG     ,:)%nbgstdv = nbg_MSG     
yconfig(msensor_MSG_HR  ,:)%nbgstdv = nbg_MSG_HR  
yconfig(msensor_MTSATIMG,:)%nbgstdv = nbg_MTSATIMG
yconfig(msensor_VISSR   ,:)%nbgstdv = nbg_VISSR   
yconfig(msensor_AIRS    ,:)%nbgstdv = nbg_AIRS    
yconfig(msensor_IASI    ,:)%nbgstdv = nbg_IASI    
yconfig(msensor_SSMIS   ,:)%nbgstdv = nbg_SSMIS   
yconfig(msensor_AMSRE   ,:)%nbgstdv = nbg_AMSRE   
yconfig(msensor_TMI     ,:)%nbgstdv = nbg_TMI     
yconfig(msensor_WINDSAT ,:)%nbgstdv = nbg_WINDSAT     

DO is = 0, MXSENSOR
  DO ic = 1, JPMXTOCH
    yconfig(is,ic)%llconst(:) = lconst
    yconfig(is,ic)%llmode     = lmode     
    yconfig(is,ic)%llmaskrs   = lmaskrs   
    yconfig(is,ic)%llmaskcld  = lcld_AIRS
  ENDDO
ENDDO

! Override settings for individual channels/predictors:
! ----------------------------------------------------
WRITE(nulout,'(a)') myname//': Reading namelist NAMVARBC_RAD, second iteration'
CALL posnam(nulnam,'NAMVARBC_RAD')
READ(nulnam,NAMVARBC_RAD)

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE config
!-----------------------------------------------------------------------

SUBROUTINE encode_groupkey (cdkey, kbufrid, ksensor, kchannl)

!-----------------------------------------------------------------------
! Encodes group description

! cdkey:  encoded string defining a group of radiance data

!-----------------------------------------------------------------------
CHARACTER(LEN=*)  , INTENT(OUT) :: cdkey
INTEGER(KIND=JPIM), INTENT(IN)  :: kbufrid, ksensor, kchannl

REAL(KIND=JPRB) :: zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (encode_groupkey)'

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

WRITE(cdkey,'(3(i0,x))') kbufrid, ksensor, kchannl

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE encode_groupkey
!-----------------------------------------------------------------------

SUBROUTINE decode_groupkey (cdkey, kbufrid, ksensor, kchannl)

!-----------------------------------------------------------------------
! Decodes group description

! cdkey:  encoded string defining a group of radiance data

!-----------------------------------------------------------------------
CHARACTER(LEN=*)  , INTENT(IN)  :: cdkey
INTEGER(KIND=JPIM), INTENT(OUT) :: kbufrid, ksensor, kchannl

REAL(KIND=JPRB) :: zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (decode_groupkey)'

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

READ(cdkey,*) kbufrid, ksensor, kchannl

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE decode_groupkey
!-----------------------------------------------------------------------

SUBROUTINE groupdescr (cdkey, cdescr)

!-----------------------------------------------------------------------
! Returns group description

! cdkey:      encoded string defining a group of radiance data
! cdescr:     string suitable for printing

!-----------------------------------------------------------------------
CHARACTER(LEN=*), INTENT(IN)  :: cdkey
CHARACTER(LEN=*), INTENT(OUT) :: cdescr

CHARACTER(LEN=80)  :: cldesc
INTEGER(KIND=JPIM) :: ibufrid, isensor, ichannl, j, ilen
REAL(KIND=JPRB)    :: zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (groupdescr)'

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

CALL decode_groupkey (cdkey, ibufrid, isensor, ichannl)

! Fallback string

WRITE(cldesc,'(a,4x,3(a,i0))') 'Tb', &
           & ' satid=', ibufrid, ' sensor=', isensor, ' channel=', ichannl

! Try to get string from SATGRP_TABLE

DO j = 1, NSATGRP
  IF (SATGRP_TABLE(j)%codetype/=ngthrb ) CYCLE  
  IF (SATGRP_TABLE(j)%bufrid  /=ibufrid) CYCLE  
  IF (SATGRP_TABLE(j)%sensor  /=isensor) CYCLE 
  WRITE(cldesc,'(a,4x,a,4x,a,i0)') 'Tb', &
           &  TRIM(JOT(NSATEM)%jost(j)%sname), 'channel=', ichannl 
ENDDO

ilen = LEN(cldesc)
IF (LEN(cdescr)<ilen) CALL ABOR1(myname//': Increase string length')

cdescr(1:ilen) = cldesc

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE groupdescr
!-----------------------------------------------------------------------

SUBROUTINE find_groups (cdkeys, kng)

!-----------------------------------------------------------------------
!   find_groups - Find data groups present in ODB requiring VarBC

!   Interface.
!   ----------
!   CALL find_groups (cdkeys, kng)

!       cdkeys(1:kng)    group descriptions

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2008/03/01
!-----------------------------------------------------------------------

CHARACTER(LEN=*)  , INTENT(OUT) :: cdkeys(:)
INTEGER(KIND=JPIM), INTENT(OUT) :: kng

INTEGER(KIND=JPIM) :: ibufrid, isensor, ichannl
INTEGER(KIND=JPIM) :: is, ins
REAL(KIND=JPRB)    :: zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (find_groups)'

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

WRITE(nulout,'(a)') myname//': Looking for radiance data groups..'

! Find all satellite/sensor combinations in the present data 
! ----------------------------------------------------------
CALL find_sensors (ins)

! Return group descriptions for VarBC
! -----------------------------------
kng = 0
IF (ins>0) THEN

  DO is = 1, ins  ! Look at each sensor

    ibufrid = ysensors(is)%nbufrid
    isensor = ysensors(is)%nsensor

    DO ichannl = 1, JPMXTOCH

      IF (yconfig(isensor,ichannl)%nparam==0) CYCLE

!     This group requires bias correction; add it to the list of keys

      kng = kng + 1
      IF (kng>size(cdkeys)) CALL ABOR1(myname//': Insufficient allocation.')

      CALL encode_groupkey (cdkeys(kng), ibufrid, isensor, ichannl)

    ENDDO

  ENDDO

  WRITE(nulout,'(a,i3,a)') myname//': ', kng, ' radiance data groups for VarBC'

ENDIF

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE find_groups
!-----------------------------------------------------------------------------

SUBROUTINE get_min ( cdkey, knparam, &
                   & ldconst, kbgstdv, ldmaskrs, ldmode, pdfgdep, ldmaskld )

!-----------------------------------------------------------------------
!   get_min - Return parameters used in the minimisation for a particular 
!             group of radiance data

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2008/03/01
!-----------------------------------------------------------------------

CHARACTER(LEN=*)  , INTENT(IN)  :: cdkey
INTEGER(KIND=JPIM), INTENT(IN)  :: knparam
INTEGER(KIND=JPIM), INTENT(OUT) :: kbgstdv
LOGICAL           , INTENT(OUT) :: ldconst(knparam), ldmaskrs, ldmode, ldmaskld
REAL(KIND=JPRB)   , INTENT(OUT) :: pdfgdep

INTEGER(KIND=JPIM) :: ibufrid, isensor, ichannl
REAL(KIND=JPRB)    :: zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (get_min)'

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

CALL decode_groupkey (cdkey, ibufrid, isensor, ichannl)

IF (knparam/=yconfig(isensor,ichannl)%nparam) THEN
  WRITE(nulout,'(a)'   ) myname//': Parameter mismatch!'
  WRITE(nulout,'(2a)'  ) ' groupkey=', cdkey
  WRITE(nulout,'(a,i0)') ' nparam=', knparam
  WRITE(nulout,'(a,i0)') ' yconfig(isensor,ichannl)%nparam=', yconfig(isensor,ichannl)%nparam  
  CALL ABOR1(myname//': Number of parameters does not match.')
ENDIF

ldconst(:) = yconfig(isensor,ichannl)%llconst(1:knparam)
kbgstdv    = yconfig(isensor,ichannl)%nbgstdv
ldmode     = yconfig(isensor,ichannl)%llmode
pdfgdep    = yconfig(isensor,ichannl)%dfgdep
ldmaskrs   = yconfig(isensor,ichannl)%llmaskrs
ldmaskld   = yconfig(isensor,ichannl)%llmaskcld

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE get_min
!-----------------------------------------------------------------------------

SUBROUTINE find_sensors (kns)

INTEGER(KIND=JPIM), INTENT(OUT) :: kns

! Finds all distinct bufrid/sensor combinations in ODB

INTEGER(KIND=JPIM) :: ibufrid, isensor
INTEGER(KIND=JPIM) :: iret, info(1), ic, j
REAL(KIND=JPRB)    :: zinfo(0)
REAL(KIND=JPRB)    :: zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (find_sensors)'


#include "openmp_obs.h"

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

WRITE(nulout,'(a)') &
 &   myname//': Checking ODB for available bufrid/sensor combinations'

CALL getdb('GETSATID', 0, iret, info, 0, zinfo, 0, &
          & -1, -1, -1, -1, -1, -1)  

! ROBSU(:,MDB_SATID_AT_SAT   ) contains bufrid
! ROBSU(:,MDBSSIA            ) contains sensor
! ROBSU(:,MDB_CODETYPE_AT_HDR) contains codetype

ic = 0
DO isensor = 0, MXSENSOR ! For each sensor, ..
  ibufrid = -1
  DO j = 1, NROWS_ROBSU ! .. find all distinct bufrid's
    IF (ROBSU(j,MDBSSIA            )/=isensor) CYCLE
    IF (ROBSU(j,MDB_CODETYPE_AT_HDR)/= NGTHRB) CYCLE
    IF (ROBSU(j,MDB_SATID_AT_SAT)   ==ibufrid) CYCLE ! duplicate
    ibufrid = ROBSU(j,MDB_SATID_AT_SAT)
    ic = ic + 1
    IF (ic>jpmxns) &
      & CALL ABOR1(myname//': Insufficient allocation - increase jpmxns')
    ysensors(ic)%nbufrid = ibufrid
    ysensors(ic)%nsensor = isensor
    WRITE(nulout,'(2(a,i0))') myname//': bufrid=', ibufrid, ' sensor=', isensor
  ENDDO
ENDDO
kns = ic

CALL putdb('GETSATID', 0, iret, info, 0, zinfo, 0)

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE find_sensors
!-----------------------------------------------------------------------------

SUBROUTINE get_predictors ( cdkey, knparam, &
                          & kpredcs, pparams, ldreset, ldmode, kcstart, pdfgdep)

! Return list of predictors, a-priori parameter values, ..
 
CHARACTER(LEN=*)  , INTENT(IN)  :: cdkey
INTEGER(KIND=JPIM), INTENT(OUT) :: knparam, kcstart
LOGICAL           , INTENT(OUT) :: ldreset, ldmode
INTEGER(KIND=JPIM), INTENT(OUT) :: kpredcs(jpmxnp)
REAL(KIND=JPRB)   , INTENT(OUT) :: pparams(jpmxnp)
REAL(KIND=JPRB)   , INTENT(OUT) :: pdfgdep

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (get_predictors)'

INTEGER(KIND=JPIM) :: ibufrid, isensor, ichannl
REAL(KIND=JPRB)    :: zhook_handle

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

CALL decode_groupkey (cdkey, ibufrid, isensor, ichannl)

IF ( (isensor>=0 .AND. isensor<=MXSENSOR) .AND. &
   & (ichannl>=1 .AND. ichannl<=JPMXTOCH) ) THEN

  knparam = yconfig(isensor,ichannl)%nparam
  IF (knparam>0) THEN
    kpredcs(1:knparam) = yconfig(isensor,ichannl)%npredcs(1:knparam)
    pparams(1:knparam) = yconfig(isensor,ichannl)%zparams(1:knparam)
  ENDIF 
  kcstart = yconfig(isensor,ichannl)%ncstart
  ldreset = yconfig(isensor,ichannl)%lreset
  ldmode  = yconfig(isensor,ichannl)%llmode
  pdfgdep = yconfig(isensor,ichannl)%dfgdep

ELSE

  WRITE(nulout,'(a)') myname//': Invalid key '//cdkey 
  knparam = 0

ENDIF

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE get_predictors
!-----------------------------------------------------------------------------

SUBROUTINE set_varbcix (cdkeys, kix, kng)

!-----------------------------------------------------------------------
!   set_varbcix - Set VarBC group indices in ODB

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2008/03/01
!-----------------------------------------------------------------------

INTEGER(KIND=JPIM), INTENT(IN) :: kix(kng), kng
CHARACTER(LEN=*)  , INTENT(IN) :: cdkeys(kng)

INTEGER(KIND=JPIM) :: iret, ilen, jg, info(1), jobs, ilev, jlev
INTEGER(KIND=JPIM) :: ibufrid, isensor, ichannl
REAL(KIND=JPRB)    :: zinfo(0), zhook_handle

CHARACTER(LEN=*), PARAMETER :: myname = modname//' (set_varbcix)'


#include "openmp_obs.h"

IF (lhook) CALL dr_hook(myname,0,zhook_handle)

WRITE (nulout,'(/,a)') myname//': Setting VarBC group indices (varbc_ix@body) in ODB'

CALL GETDB('VARBC_RAD',0,iret,info,0,zinfo,0, -1, -1, -1, -1, -1, -1)  

ilen = iret
DO jg = 1, kng

  CALL decode_groupkey (cdkeys(jg), ibufrid, isensor, ichannl)

  DO jobs = 1, ilen

    IF (ROBHDR(jobs,MDB_SATID_AT_SAT) == ibufrid .AND. &
      & ROBHDR(jobs,MDBSSIA         ) == isensor       ) THEN  

      DO jlev = MLNKH2B(jobs), MLNKH2B(jobs+1)-1
        ilev = NINT(ROBODY(jlev,MDBPPP))
        IF (ilev==ichannl) THEN
          ROBODY(jlev,MDB_VARBC_IX_AT_BODY) = kix(jg)
        ENDIF
      ENDDO

    ENDIF

  ENDDO
  
ENDDO

CALL PUTDB('VARBC_RAD',0,iret,info,0,zinfo,0)

IF (lhook) CALL dr_hook(myname,1,zhook_handle)

END SUBROUTINE set_varbcix
!-----------------------------------------------------------------------------

END MODULE VARBC_RAD
