MODULE YOMDYNCORE

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

! MAIN SWITCHES

! LDYNCORE=.T.    switches on idealized planet
! LAQUA=.T.       switches on idealized aqua planet
! LHELDSUAREZ=.T. switches on idealized Held-Suarez
! LVERTINI=.T.    allows to provide different set of A,Bs in namelist
!                 (!only works with idealized initial data!)

! LPPSTEPS=.T.    specify postprocessing/fdb output in steps (emulating hours)

LOGICAL         :: LDYNCORE
LOGICAL         :: LAQUA
LOGICAL         :: LHELDSUAREZ
LOGICAL         :: LVERTINI
LOGICAL         :: LPPSTEPS

! small planet factor
REAL(KIND=JPRB)     :: RPLRADI
! Coriolis acceleration factor
REAL(KIND=JPRB)     :: RCORIOI

! The following options are implemented 
! N3DINI == 2 == analysed initial data from file, (orog == 0., moisture depends
!                on LAQUA=T/F)
! N3DINI == 3 == idealized GATE initial profiles (after Grabowski (2001) JAS,
!                58, 978--997)
!                (moisture depends on LAQUA=T/F)
! N3DINI == 4 == no/zonal flow, neutral stratification (theta == constant)
! N3DINI == 5 == no/zonal flow, stratified isothermal atmosphere (T == constant) 

! NTESTCASE = 0 Williamson mountain, case 1;
!               Williamson et al. (1992) JCP, 102, 211--224
! NTESTCASE = 1 Williamson mountain, case 5;
!               Williamson et al. (1992) JCP, 102, 211--224
! NTESTCASE = 2 Klemp-Schaer mountain profile;
!               Klemp, Skamarock, Fuhrer (2003) MWR, 131, 1229-1239
! NTESTCASE = 3 simple mountain, low froude number flow
! NTESTCASE = 4 elliptic (quasi-2D) Witch of Agnesi setup for tropos-stratos NH,
!               T. Keller (1994) JAS, 51, 1915--1929
! NTESTCASE = 5 marginally resolved step mountain (like Himalaya)
! NTESTCASE = 6 profile and wind shear following
!               T. Keller (1994) JAS, 51, 1915--1929
! NTESTCASE = 7 critical flow LS2 following Grubisic and Smolarkiewicz (1997)
!               JAS, 1943--1960
! NTESTCASE = 8 critical flow LS5 following Grubisic and Smolarkiewicz (1997)
!               JAS, 1943--1960
! NTESTCASE = 9 acoustic wave (following Landau/Lifschitz)
! NTESTCASE = 10 sea breeze, to be done.
! NTESTCASE = 11 double bubble test following Robert (1993) JAS, 1865--1873
! NTESTCASE = 12 single rising bubble test following Robert (1993)
!                JAS, 1865--1873

! NOISEVOR : if 1 add initial noise in vorticity

INTEGER(KIND=JPIM) :: NTESTCASE

INTEGER(KIND=JPIM) :: NOISEVOR

!     PARAMETER USED FOR AQUAPLANET

! defined in GP_SSTAQUA()
! MSSTSCHEME: defines the prescribed SST distribution as described in
! Atmos. Sc. Letters (2001) Vol.1
! == 1 Control
! == 2 Peaked
! == 3 Flat
! == 4 Qobs
! == 5 Control5N
! == 6 1KEQ, chi=1.0_JPRB
! == 7 3KEQ, chi=3.0_JPRB
! == 8 3KW1, chi=3.0_JPRB
! == 9 constant SST everywhere

! RLAMBDA0       : Longitude of maximum SST anomaly
! RLAMBDAD, RPHID: Half width in long/lat of SST anomaly
! RCHI           : Maximum magnitude of SST anomaly 

INTEGER(KIND=JPIM)  :: MSSTSCHEME
REAL(KIND=JPRB)     :: RCHI
REAL(KIND=JPRB)     :: RLAMBDA0
REAL(KIND=JPRB)     :: RLAMBDAD
REAL(KIND=JPRB)     :: RPHID
!-----------------------------------------------------------------------------

!    INITIAL PROFILES

! RT00_DYN  -  Initial idealized temperature
! RP00_DYN  -  Initial idealized pressure
! RU00_DYN  -  Initial idealized zonal wind
! RST0_DYN  -  Initial idealized wind shear (du/dz)
REAL(KIND=JPRB)  :: RT00_DYN
REAL(KIND=JPRB)  :: RP00_DYN
REAL(KIND=JPRB)  :: RU00_DYN
REAL(KIND=JPRB)  :: RST0_DYN

!     PARAMETER USED FOR THE ORIGINAL HELD/SUAREZ FORCING

! RSIGMAB          - HS test: vertical extent of friction layer expressed in
!                    sigma (between 0..1)
! REFERENCE_LEVEL  - HS test: 1.0_JPRB-RSIGMAB
! RDELTA_T         - HS test: pole - equator temperature difference
! RDELTA_THETA     - HS test: tropical heating differential
! RPRESSURE_SCALE  - HS test: pressure scaling
! RHS_KF           - HS test: frictional relaxation time scale
! RHS_KA           - HS test: heating time scale (atmosphere) 0.025_JPRB*RHS_KF
! RHS_KS           - HS test: heating time scale (surface) 0.25_JPRB*RHS_KF
! RHS_KS_KA        - HS test: RHS_KS-RHS_KA
! RHS_KAPPA        - HS test: Kappa = 2/7 
REAL(KIND=JPRB) :: RSIGMAB
REAL(KIND=JPRB) :: REFERENCE_LEVEL
REAL(KIND=JPRB) :: RDELTA_T
REAL(KIND=JPRB) :: RDELTA_THETA
REAL(KIND=JPRB) :: RPRESSURE_SCALE
REAL(KIND=JPRB) :: RHS_KF
REAL(KIND=JPRB) :: RHS_KA
REAL(KIND=JPRB) :: RHS_KS
REAL(KIND=JPRB) :: RHS_KS_KA
REAL(KIND=JPRB) :: RHS_KAPPA

!     PARAMETER USED FOR THE MODIFIED FORCING IN UPPER LEVELS
!     (AFTER WILLIAMSON)

! L_HS_WILLIAMSON=.FALSE. - .T. use the Williamson extension of the Held-Suarez
!  test in the stratosphere (MWR 126:1001-1012)
LOGICAL         :: L_HS_WILLIAMSON=.FALSE.

! RGAMMA_D         - HSW extension: lapse rate active when pressure <= p_d,
!                    decreases temperature in stratospheric polar regions
! RGAMMA_I         - HSW extension: lapse rate active when pressure <= p_i,
!                    increases temperature in stratospheric equatorial regions
! RTHSW_0          - HSW extension: stratospheric reference temperature
! RPHSW_D          - HSW extension: pressure threshold p_d
! RPHSW_EQ         - HSW extension: equatorial stratospheric reference pressure
! RPHSW_PL         - HSW extension: vertical transition threshold in hPa
! RPHSW_EQ_PL_HALF - HSW extension: 0.5_JPRB*(RPHSW_EQ-RPHSW_PL)
! RPHSW_D_REV      - HSW extension: 1.0_JPRB/RPHSW_D
REAL(KIND=JPRB), PARAMETER :: RGAMMA_D = 2.e-3_JPRB
REAL(KIND=JPRB), PARAMETER :: RGAMMA_I = -3.345e-3_JPRB
REAL(KIND=JPRB) :: RTHSW_0
REAL(KIND=JPRB) :: RPHSW_D
REAL(KIND=JPRB) :: RPHSW_EQ
REAL(KIND=JPRB) :: RPHSW_PL
REAL(KIND=JPRB) :: RPHSW_EQ_PL_HALF
REAL(KIND=JPRB) :: RPHSW_D_REV

!     PARAMETER USED FOR SAMPLING STATISTICS

! RSAMPLING_START     -  start of time mean calculations
! RSAMPLING_INTERVAL  -  interval used in time mean calculations
REAL(KIND=JPRB), PARAMETER :: RSAMPLING_START=0.0_JPRB
REAL(KIND=JPRB), PARAMETER :: RSAMPLING_INTERVAL=12._JPRB*3600._JPRB

! DRAGX      -  mountain drag in zonal direction
! DRAGY      -  mountain drag in meridional direction
! DRAGNORM   -  asymptotic linear mountain drag
REAL(KIND=JPRB), ALLOCATABLE :: DRAGX(:)
REAL(KIND=JPRB), ALLOCATABLE :: DRAGY(:)
REAL(KIND=JPRB) :: DRAGNORM

! PARAMETER USED FOR SEMI-LAGRANGIAN DIAGNOSTICS

! RJACSLDIA == THRESHOLD FOR OUTPUT OF JACOBIAN
! RLIPSLDIA == THRESHOLD FOR OUTPUT OF LIPSCHITZ NUMBER

REAL(KIND=JPRB) :: RJACSLDIA
REAL(KIND=JPRB) :: RLIPSLDIA

END MODULE YOMDYNCORE
