MODULE YOMTRAJ

!     Purpose.
!     --------
!       Variables to control trajectory

!     Author.
!     -------
!        Y. Tremolet *ECMWF*

!     Modifications.
!     --------------
!        Original : 2002-12-03
!        Y.Tremolet: 14-01-04 Rename a few things and add comments
!        Y.Tremolet    18-Mar-2004 Add lprttraj option for diagnostics
!        K.Yessad  : 27-Jun-2007 Add missing comments
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB     ,JPRM
USE GRIB_HEADER_MIX
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE
SAVE
PUBLIC

LOGICAL :: LTRAJHR              ! Use interpolated trajectory
LOGICAL :: LTRAJHR_ALTI         ! Use interpolated trajectory (upper air flds)
LOGICAL :: LTRAJHR_SURF         ! Use interpolated trajectory (surface flds)
LOGICAL :: LTRAJGP              ! Store trajectory in grid-point space
LOGICAL :: LTRAJSAVE = .FALSE.  ! Saving trajectory
LOGICAL :: LTRAJSLAG = .FALSE.  ! Semi-Lagrangian trajectory is used
LOGICAL :: LTRAJPHYS = .FALSE.  ! Physics trajectory is used
LOGICAL :: LTRAJCST  = .FALSE.  ! Constants in trajectory (needs tidying-up)
LOGICAL :: LTRAJALLOC= .FALSE.  ! Trajectory storage is allocated
LOGICAL :: LPRTTRAJ  = .FALSE.  ! Prints more info from trajectory routines
LOGICAL :: LTRAJRESET= .FALSE.  ! Recomputing and changing trajectory
LOGICAL :: LREADGPTRAJ          ! Write/read main trajectory in grid-point space

REAL(KIND=JPRB) :: TSTEP_TRAJ       ! Time interval between saved trajectory steps
INTEGER(KIND=JPIM) :: NSMAX_TRAJ    ! Spectral truncation for trajectory

! * NSMAX_BACKGR00 to NSMAX_BACKGR03: Spectral truncation for background state
!   for the various inner loops.
INTEGER(KIND=JPIM) :: NSMAX_BACKGR00, NSMAX_BACKGR01, NSMAX_BACKGR02, &
 & NSMAX_BACKGR03

! * NSPTRAJ          : number of spectral fields stored in the trajectory
! * NGPTRAJ          : number of grid-point fields stored in the trajectory
! * NTRAJP           : number of fields in buffer TRAJ_PHYS
! * NGP5             : number of fields in TRAJ_SRFC
! * NTRAJ_CST        : number of fields in TRAJ_CST
! * NSTEPTRAJ        : number of timesteps where the trajectory must be stored
! * NSTEPTRAJ_SPEC   : ??? (useless)
! * MSTART           : MSTEPTRAJW(NSTART)
! * NTRAJSL3D        : number of 3D fields in TRAJ_SLAG
! * NTRAJSL2D        : number of 2D fields in TRAJ_SLAG
! * NTRAJSL          : total number of (equivalent-2D) fields in TRAJ_SLAG
! * MSTEPTRAJW       : numbering for timesteps where trajectory is written
! * MSTEPTRAJR       : numbering for timesteps where trajectory is read
! * MIOTRAJMAIN      : ???
INTEGER(KIND=JPIM) :: NSPTRAJ
INTEGER(KIND=JPIM) :: NGPTRAJ
INTEGER(KIND=JPIM) :: NTRAJP
INTEGER(KIND=JPIM) :: NGP5
INTEGER(KIND=JPIM) :: NTRAJ_CST
INTEGER(KIND=JPIM) :: NSTEPTRAJ
INTEGER(KIND=JPIM) :: NSTEPTRAJ_SPEC
INTEGER(KIND=JPIM) :: MSTART
INTEGER(KIND=JPIM) :: NTRAJSL3D
INTEGER(KIND=JPIM) :: NTRAJSL2D
INTEGER(KIND=JPIM) :: NTRAJSL
INTEGER(KIND=JPIM), ALLOCATABLE :: MSTEPTRAJW(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MSTEPTRAJR(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MIOTRAJMAIN(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MIOTRAJSURF(:)
INTEGER(KIND=JPIM), PARAMETER :: MKINDTRAJ = JPRM

! For no packing, replace previous line by
! INTEGER, PARAMETER :: MKINDTRAJ = JPRB
! For packing, replace previous line by
! INTEGER, PARAMETER :: MKINDTRAJ = JPRM

! * TRAJ_BUF : intermediate trajectory buffer (because GRIBEX cannot handle
!   more than one type of REAL and thus cannot accept MKINDTRAJ arrays).
REAL(KIND=JPRB), ALLOCATABLE :: TRAJ_BUF(:,:,:)

!-----------------------------------------------------------------------
! * These arrays contain the actual trajectory
!   TRAJ_[X] buffers for trajectory, BCKG_[X] buffers for background.
!   [X] = SPEC for spectral data, SRFC for surface data, CST for constants,
!    SLAG for semi-Lagrangian data, PHYS for physics data, ZERO (cf. PHYS
!    but for some particuliar instants), GMV for GMV variables, GMVS for GMVS
!    variables, GFL for GFL variables.
!   TRAJ_SPEC_TMP0 is a copy of TRAJ_SPEC at the initial instant.
TYPE(SPECTRAL_FIELD), ALLOCATABLE :: TRAJ_SPEC(:)
TYPE(SPECTRAL_FIELD), ALLOCATABLE :: BCKG_SPEC(:) 
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: TRAJ_SRFC(:,:,:,:)
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: TRAJ_CST(:,:,:)
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: TRAJ_SLAG(:,:,:,:)
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: TRAJ_PHYS(:,:,:,:)
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: TRAJ_ZERO(:,:,:)
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: TRAJ_GMV(:,:,:,:,:)
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: TRAJ_GMVS(:,:,:,:)
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: TRAJ_GFL(:,:,:,:,:)
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: BCKG_GMV(:,:,:,:)
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: BCKG_GMVS(:,:,:)
REAL(KIND=MKINDTRAJ), ALLOCATABLE :: BCKG_GFL(:,:,:,:)
TYPE(SPECTRAL_FIELD) :: TRAJ_SPEC_TMP0

!-----------------------------------------------------------------------

! * TRAJ_GRIB, MAIN_GRIB, BACKGR_GRIB: information for grib header
! * MTYPE_[X]_TRAJ: grib codes ([X] = SURF for surface, SLAG for semi-Lag data,
!   PHYS for physics data, MAIN3 for upper air 3D variables, MAIN2 for
!   2D variables).
INTEGER(KIND=JPIM) :: MTRAJ_GRIB
INTEGER(KIND=JPIM) :: MAIN_GRIB
INTEGER(KIND=JPIM), ALLOCATABLE :: MBACKGR_GRIB(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MTYPE_SURF_TRAJ(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MTYPE_SLAG_TRAJ(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MTYPE_PHYS_TRAJ(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MTYPE_MAIN3_TRAJ(:)
INTEGER(KIND=JPIM), ALLOCATABLE :: MTYPE_MAIN2_TRAJ(:)

!-----------------------------------------------------------------------

END MODULE YOMTRAJ
