MODULE YOMVARBC
!-----------------------------------------------------------------------
!   YOMVARBC - Module for variational bias correction

!   Purpose. 
!   --------
!   Define data structures and parameters to support
!   variational bias correction

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2004/03/19
!   Paul Burton 2004/09/09 : Added parcom to type_varbc
!   Matt Szyndel2005/03/23 : Added MSG, MTSAT, FY-2
!   T. Auligne  2005/12/09 : Added Nadir Viewing Angle (and powers) as
!                            predictors for the scan component of bias
!   Dick Dee    2005/12/16 : Slight adjustment of normalization of p5, p6
!------------------------------------------------------------------------

USE PARKIND1, ONLY : JPIM, JPRB, JPIB

USE PARCMA  , ONLY : JPMXTOCH 
USE YOMTVRAD, ONLY : MXSENSOR

IMPLICIT NONE

SAVE

! Table for administration of bias parameters
! -------------------------------------------
TYPE type_varbc
INTEGER(KIND=JPIM)          :: NPARAM       ! number of bias parameters
INTEGER(KIND=JPIM)          :: NBUFRID      ! buffer code for this satellite
INTEGER(KIND=JPIM)          :: NSENSOR      ! sensor id
INTEGER(KIND=JPIM)          :: NCHANNL      ! channel number
INTEGER(KIND=JPIM)          :: NCOUNT       ! active data count
INTEGER(KIND=JPIM)          :: NCODETYPE    ! codetype
REAL(KIND=JPRB)             :: OBSERR       ! rms obs error std dev
INTEGER(KIND=JPIM), POINTER :: NPREDCS(:)   ! list of predictors
REAL(KIND=JPRB), POINTER    :: APARAMS(:)   ! bias parameters
REAL(KIND=JPRB), POINTER    :: APARCOM(:,:) ! component of bias parameters
INTEGER(KIND=JPIB), POINTER :: IPARCOM(:,:) ! component of bias parameters
REAL(KIND=JPRB), POINTER    :: APARAM0(:)   ! background parameters
REAL(KIND=JPRB), POINTER    :: APARAM5(:)   ! trajectory parameters
REAL(KIND=JPRB), POINTER    :: BKGERR(:)    ! background error std dev
REAL(KIND=JPRB), POINTER    :: ACHGVAR(:,:) ! change-of-variable operator
REAL(KIND=JPRB), POINTER    :: ACVARIN(:,:) ! inverse change-of-variable
END TYPE type_varbc

TYPE(type_varbc), ALLOCATABLE :: YVARBC(:)

INTEGER(KIND=JPIM), PARAMETER :: JPMXNGROUP = 10000 ! maximum number of bias groups
INTEGER(KIND=JPIM), PARAMETER :: NBINS = 200  ! maximum number of bins for histograms
REAL(KIND=JPRB)   , PARAMETER :: AOMF  = 10.  ! maximum absolute value for FG departure 
REAL(KIND=JPRB)               :: ZBIN  = 2*AOMF / NBINS

INTEGER(KIND=JPIM) :: NOBGROUP  ! total number of bias parameter groups
INTEGER(KIND=JPIM) :: NOBPARAM  ! total number of bias parameters
REAL(KIND=JPRB)    :: AFJPCOST = 0.0_JPRB  ! background contribution to cost function

! Predictor information:
! ---------------------
!   JPMXNPRED     predictors are indexed from 0 to mxnpred
!   JPMXNPARAM    maximum number of bias parameters per group
!   CPREDDESC   predictor descriptions (for printing)
!   PPREDNORM   predictor normalization: global mean, stdv

INTEGER(KIND=JPIM), PARAMETER :: JPMXNPRED = 11
INTEGER(KIND=JPIM), PARAMETER :: JPMXNPARAM = JPMXNPRED + 1
CHARACTER(LEN=21),  PARAMETER :: CPREDDESC(0:JPMXNPRED) = &
                               & (/ '1 (constant)         ', &
                               &    '1000-300hPa thickness', &
                               &    '200-50hPa thickness  ', &
                               &    'T_skin               ', &
                               &    'total column water   ', &
                               &    '10-1hPa thickness    ', &
                               &    '50-5hPa thickness    ', &
                               &    'surface wind speed   ', &
                               &    'nadir viewing angle  ', &
                               &    'nadir view angle **2 ', &
                               &    'nadir view angle **3 ', &
                               &    'nadir view angle **4 ' /)
REAL(KIND=JPRB),    PARAMETER :: PPREDNORM(2,0:JPMXNPRED) = &
                               & RESHAPE(                         &
                               & (/     1.0_JPRB,    0.0_JPRB,    &
                               &     9207.0_JPRB,  446.0_JPRB,    &
                               &     8491.0_JPRB,  387.0_JPRB,    & 
                               &      285.0_JPRB,   20.5_JPRB,    &
                               &       25.0_JPRB,   17.8_JPRB,    &
                               &    16622.0_JPRB,  374.0_JPRB,    &
                               &    14975.0_JPRB,  570.0_JPRB,    &
                               &        6.0_JPRB,    3.6_JPRB,    &
                               &        5.5_JPRB,   28.7_JPRB,    &
                               &      853.0_JPRB,  744.0_JPRB,    &
                               &     9300.0_JPRB,46700.0_JPRB,    &
                               &  1540000.0_JPRB,2799000.0_JPRB /), &
                               & (/ 2, JPMXNPARAM /) )

!   arrays for computing predictor statistics

INTEGER(KIND=JPIM) :: MPREDXCNT(0:JPMXNPRED,0:JPMXNPRED)
INTEGER(KIND=JPIM), ALLOCATABLE :: MPREDXCNT_COMP(:,:,:) ! MPREDXCNT partial sum
REAL(KIND=JPRB)    :: APREDMEAN(0:JPMXNPRED)
REAL(KIND=JPRB)    :: APREDXCOV(0:JPMXNPRED,0:JPMXNPRED)
REAL(KIND=JPRB), ALLOCATABLE :: APREDMEAN_COMP(:,:) ! APREDMEAN partial sum
REAL(KIND=JPRB), ALLOCATABLE :: APREDXCOV_COMP(:,:,:) ! APREDXCOV ""    ""
INTEGER(KIND=JPIB), ALLOCATABLE :: IPREDMEAN_COMP(:,:) ! APREDMEAN partial sum
INTEGER(KIND=JPIB), ALLOCATABLE :: IPREDXCOV_COMP(:,:,:) ! APREDXCOV ""    ""
INTEGER(KIND=JPIM) :: MPREDXHST(JPMXNGROUP,NBINS)
INTEGER(KIND=JPIM), ALLOCATABLE :: MPREDXHST_COMP(:,:,:) ! MPREDXHST partial sum
! Group definitions
! -----------------
LOGICAL :: LBC_HIRS, LBC_MSU, LBC_SSU, LBC_AMSUA, LBC_AMSUB,LBC_MHS
LOGICAL :: LBC_SSMI, LBC_METEOSAT, LBC_GOESIMG, LBC_AIRS, LBC_IASI
LOGICAL :: LBC_FY2 , LBC_MTSATIMG, LBC_MSG, LBC_MSG_HR
LOGICAL :: LBC_SSMIS, LBC_AMSRE, LBC_TMI
LOGICAL :: LBC_RAD1C(0:MXSENSOR,JPMXTOCH)          
LOGICAL :: LBC_PRED_RAD1C(0:MXSENSOR,JPMXTOCH,0:JPMXNPRED)

! Initialisation options
! ----------------------
LOGICAL            :: LCOLDSTART  ! force a cold start on all parameters
LOGICAL            :: LCS_STRICT  ! do not allow choice of predictors to 
                                  ! change on the fly
LOGICAL            :: LCS(JPMXNGROUP) ! cold-start flag
INTEGER(KIND=JPIM) :: NCS_CONFIG  ! cold start configuration options:
                                  !   0: set cold-start parameters to zero
                                  !   1: use predictor coefficients
                                  !      from file when available
                                  !   2: set cold-start parameters to the 
                                  !      mode of the first-guess departures
! Background constraints
! ----------------------
LOGICAL            :: LBGCONSTR
INTEGER(KIND=JPIM) :: NBG_HIRS, NBG_MSU, NBG_SSU
INTEGER(KIND=JPIM) :: NBG_AMSUA, NBG_AMSUB, NBG_MHS, NBG_SSMI
INTEGER(KIND=JPIM) :: NBG_METEOSAT, NBG_GOESIMG, NBG_AIRS, NBG_IASI
INTEGER(KIND=JPIM) :: NBG_MSG     , NBG_MSG_HR , NBG_MTSATIMG, NBG_FY2
INTEGER(KIND=JPIM) :: NBG_AMSRE, NBG_SSMIS, NBG_TMI
INTEGER(KIND=JPIM) :: NBG_RAD1C(0:MXSENSOR,JPMXTOCH,0:JPMXNPRED)

! Cloud cover switch & maximum percentage 
! ------------------------------
LOGICAL            :: LCLDCOVER_AIRS(0:MXSENSOR,JPMXTOCH,0:JPMXNPRED) ! AIRS VISNIR cloud cover masking
LOGICAL            :: LCLDCOVER_AIRS_ALL 
INTEGER(KIND=JPIM) :: NCLDCOVER_AIRS ! AIRS VISNIR cloud cover maximum percentage

! Constant VARBC switch 
! --------------------
LOGICAL            :: LCONSTANT_VARBC(0:MXSENSOR,JPMXTOCH,0:JPMXNPRED) ! VARBC constant over time
LOGICAL            :: LCONSTANT_VARBC_ALL

! Radio-sonde mask
! ----------------
LOGICAL            :: LMASKRS(0:MXSENSOR,JPMXTOCH,0:JPMXNPRED)   ! update VARBC only near Radio-sondes
LOGICAL            :: LMASKRS_ALL
INTEGER(KIND=JPIM) :: IMASKRS(360,181) ! Radio-sonde Mask (0/1)

! Metric approximating the mode of the distribution 
! -------------------------------------------------
LOGICAL            :: LMODE(0:MXSENSOR,JPMXTOCH) ! Weight bias update with fg_depar distribution
LOGICAL            :: LMODE_ALL
END MODULE YOMVARBC
