MODULE YOMWAVELET

!     ------------------------------------------------------------------

!*     Wavelet transform

!         LJBWAVELET        =  Global switch to turn on wavelet Jb
!         N_WAVELET_CUTOFFS =  Largest wavenumber for each scale
!         N_WAVELET_SCALES  =  Number of scales in the wavelet transform
!         WAVELET_FILTER    =  Lowpass filter coefficients
!         NMAX_RESOL        =  Maximum number of spectral resoutions for
!                              which spherical transforms are required
!                              (Set in namelist NAMTRANS.)
!         NSMIN_WAVELET     =  Minimum spherical transform resolution
!                              (The transform package insists on having
!                               more wavenumbers than processors.)
!         SKYLINE_TOL    =  To reduce computational cost, small elements of the symmetric
!                           square roots of the vertical covariance matrices are replaced
!                           by zero. This is done by constructing a "skyline" for each matrix.
!                           For each row, M*_SKYLINE_LEFT and M*_SKYLINE_RIGHT give the first
!                           and last elements of each row that must be used, such that the
!                           absolute values of all the unused elements are less than
!                           SKYLINE_TOL times the diagonal element for the row.

!         WAVELET_VCORS  =  vertical covariance matrices

!         LJBWSTATS      =  .T. => calculate and output covariance
!                           matrices, rather than reading them. 
!         L_IN_WAVELET_STATS_CALC = .true. when under SUJBWAVSTATS, .false. otherwise
!         CINBGSTATES    =  file name prefix for input background states
!         N_BGDATES      =  number of states per member for the input analysis ensemble
!         N_BGMEMBERS    =  number of members in the input analysis ensemble

!      WAVELET_VCOR_STRUCT        = derived type for correlation matrices
!      ---------"---------%NLATSG = number of latitudes (all processors)
!      ---------"---------%NLATS  = number of latitudes (this processor)
!      ---------"---------%NLONSG = longitudes for each row (all PEs)
!      ---------"---------%NLONS  = longitudes for each row (this PE)
!      ---------"---------%NPTOTG = number of horizontal points (all PEs)
!      ---------"---------%NPTOT  = number of horizontal points (this PE)
!      ---------"---------%NLEVS  = first dimension of a matrix
!      ---------"---------%NLON_OFFSET offset to this PE's zeroth longitude
!      ---------"---------%NLAT_OFFSET = offset to this PE's zeroth latitude
!      ---------"---------%LON0G  = first longitude of each row (all PEs)
!      ---------"---------%DLON   = longitudinal gridpacing each row (this PE)
!      ---------"---------%LATSG  = latitudes of rows (all processors)
!      ---------"---------%LPELIST= flags showing which PEs update which
!                                   matrices
!      ---------"---------%MASTERPE = master PE for each matrix.
!      ---------"---------%INDEXG2L = converts global matrix number to
!                                     local matrix number
!      ---------"---------%MATS   = square roots of correlation matrices
!      ---------"---------%MATINVS  = inverse square roots of matrices
!      ---------"---------%NSAMPS  = sample size for covariance calc'n
!      ---------"---------%SAMPAV  = sample mean for covariance calc'n
!      ---------"---------%EVALS   = eigenvalues of the covariance matrices
!      ---------"---------%MATS_SKYLINE_LEFT  = matrix skyline for MATS (see SKYLINE_TOL above)
!      ---------"---------%MATS_SKYLINE_RIGHT = matrix skyline for MATS
!      ---------"---------%MATINVS_SKYLINE_LEFT  = matrix skyline for MATINVS
!      ---------"---------%MATINVS_SKYLINE_RIGHT = matrix skyline for MATINVS

!      WAVELET_GRID_STRUCT        = derived type for grid definitions
!      ---------"---------%NDGLG       = Number of Gaussian latitudes
!      ---------"---------%NLOENG      = Number of points in each row
!      ---------"---------%NRESOL_ID   = Resolution id for transform package
!      ---------"---------%NGPTOT      = Number of gridpoints on this PE
!      ---------"---------%NGPTOTG     = Global number of gridpoints
!      ---------"---------%NGPTOTL     = number of gridpoints on this and other PEs
!      ---------"---------%NGPTOTMX    = max(NGPTOT) over all PEs
!      ---------"---------%NSMAX       = Spectral truncation
!      ---------"---------%NSPEC2      = Number of spectral coeffs on this PE
!      ---------"---------%GELAM       = Longitudes of gridpoints on this PE
!      ---------"---------%GELAT       = Latitudes  of gridpoints on this PE
!      ---------"---------%GW          = Gaussian quadrature weights
!      ---------"---------%NSTA        = KSTA      array returned by TRANS_INQ
!      ---------"---------%NONL        = KONL      array returned by TRANS_INQ
!      ---------"---------%NPTRFRSTLAT = KPTRFRSTLAT returned by TRANS_INQ
!      ---------"---------%NFRSTLAT    = KFRSTLAT  array returned by TRANS_INQ
!      ---------"---------%NLSTLAT     = KLSTLAT   array returned by TRANS_INQ
!      ---------"---------%NPTRFLOFF   = KPTRFLOFF array returned by TRANS_INQ
!      ---------"---------%NUMP        = KUMP      Number of m's on this PE
!      ---------"---------%MYMS        = MYMS      List of m's on this PE
!      ---------"---------%NASM0       = KASM0     array returned by TRANS_INQ
!      ---------"---------%NBLOCKSL    = Number of NPROMA blocks on this PE
!      ---------"---------%NBLOCKSG    = Number of NPROMA blocks on all PEs

!  Modified:
!    M.Fisher   17-Mar-2004 : Wavelet Jb control vector in grid space(s)

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB

IMPLICIT NONE

SAVE

LOGICAL :: LJBWAVELET, LJBWSTATS, L_IN_WAVELET_STATS_CALC
INTEGER(KIND=JPIM) :: N_WAVELET_SCALES, N_BGMEMBERS, N_BGDATES, &
 & NMAX_RESOL, NSMIN_WAVELET  
REAL(KIND=JPRB) :: SKYLINE_TOL
INTEGER(KIND=JPIM), ALLOCATABLE :: N_WAVELET_CUTOFFS(:)
REAL(KIND=JPRB), ALLOCATABLE :: WAVELET_FILTERS(:,:)

CHARACTER (LEN = 16) ::  CINBGSTATES

TYPE WAVELET_VCOR_STRUCT
  INTEGER(KIND=JPIM)                            :: NLATSG, NLATS, NPTOTG, NPTOT, &
   & NLEVS, NLAT_OFFSET  
  INTEGER(KIND=JPIM), POINTER, DIMENSION(:)     :: NLONSG, NLONS, NSAMPS, INDEXG2L, &
   & MASTERPE, NLON_OFFSET
  INTEGER(KIND=JPIM), POINTER, DIMENSION(:,:)   :: MATS_SKYLINE_LEFT, MATS_SKYLINE_RIGHT, &
                                                 & MATINVS_SKYLINE_LEFT, MATINVS_SKYLINE_RIGHT
  LOGICAL,   POINTER, DIMENSION(:,:)   :: LPELIST
  REAL(KIND=JPRB),    POINTER, DIMENSION(:)     :: LON0G,  DLON,   LATSG
  REAL(KIND=JPRB),    POINTER, DIMENSION(:,:)   :: SAMPAV, EVALS
  REAL(KIND=JPRB),    POINTER, DIMENSION(:,:,:) :: MATS,  MATINVS
END TYPE WAVELET_VCOR_STRUCT

TYPE (WAVELET_VCOR_STRUCT), ALLOCATABLE, DIMENSION(:,:) :: WAVELET_VCORS

TYPE WAVELET_GRID_STRUCT
  INTEGER(KIND=JPIM)                            :: NDGLG, NRESOL_ID, NGPTOT, NGPTOTG, &
   & NGPTOTMX, NSPEC2, NSMAX, &
   & NPTRFLOFF, NUMP, NBLOCKSL, NBLOCKSG  
  INTEGER(KIND=JPIM), POINTER, DIMENSION(:)     :: NLOENG, NPTRFRSTLAT, NFRSTLAT, &
   & NLSTLAT, MYMS, NASM0  
  INTEGER(KIND=JPIM), POINTER, DIMENSION(:,:)   :: NSTA, NONL, NGPTOTL
  REAL(KIND=JPRB),    POINTER, DIMENSION(:)     :: GELAM, GELAT, GW
END TYPE WAVELET_GRID_STRUCT

TYPE (WAVELET_GRID_STRUCT), ALLOCATABLE, DIMENSION(:) :: GRID_DEFINITION

END MODULE YOMWAVELET
