subroutine mwave_obsop (kstart, kend, kproma, klev, kmaxchannels, &  ! in    : dimensions       
                      & yd_mwave_phys_fw, yd_mwave_rad)              ! in/out: 2d/3d fields, FW

!**** Observation operator for 
!     4D-Var in clouds/precipitation for microwave radiances 

!     P. Bauer   ECMWF    May 2005

!     1. Call RTTOV-interface of observation operator

!     MWAVE_OBSOP is called from CALLPAR (ifs/phys_ec)

!     In normal incremental 4D-Var, this will happen once in each of:
!       1) Screening trajectory (full resolution)
!       2) First minimisation (reduced resolution)
!       3) Second outer loop (full resolution)
!       4) Second minimsation (reduced resolution)
!       -) Possible additional outer loops
!       -) Possible additional minimsations
!       5) Analysis trajectory (full resolution)

!     See Tech. Memos. 618, 619 and 620 for documentation

!     Modifications:
!     September 2006, P. Bauer, Recoded to outsource phys operators to CALLPAR and 
!                               Use new F90   types
!     21-Nov-2007     A. Geer   RTTOV9 compatible
!     16-Apr-2008    Alan Geer  Extension to multiple sensors
!     23-Sep-2008    Alan Geer  Initialisations now done in ec_phys; code is now
!                               diffable with ad and tl versions.
!     09 Mar  2009    W. Bell   Add azimuth angles for windsat-like sensors
!     10-Jun-2009    Alan Geer  Per-channel status flag; normalised polarisation 
!                               difference calculated at 19 and 37 GHz.
!     16-Dec-2009     F. Karbou Surface emissivity estimation
!     07-Apr-2010    Alan Geer  Cloud / precip fraction consistent with physics 
!     15-Apr-2010    Alan Geer  Revised snow quality check based on P37  

!**** MODULES      

!* KIND     
use parkind1, only: jpim, jprb
use yomhook , only: lhook, dr_hook

!* IFS
use yommwave, only: mwave_phys_type, mwave_rad_type, mwave_id, n_satsens, &
                  & mwave_serious, mwave_negative_q, mwave_land, mwave_passive, &
                  & rlimit_obs_err, mwave_error_model, &
                  & mwave_excess_snow, mwave_cold_sector, ldmwave_land
use parmwave, only: rmissing_angle, nmwave_p, imwave_cfrac, & 
  & imwave_p_obs, imwave_p_sim, imwave_tcwv, imwave_tclw, imwave_tciw, &
  & imwave_swp, wn_snow_rt_problem, rmwave_no_emis 
use yomcst  , only: rpi
use yomct0  , only: lifstraj, lscreen
use yomtvrad, only : msensor_ssmis, msensor_windsat
use yomancs,  only: rmdi

implicit none
          
!* Common variables
integer (kind=jpim), intent    (in)    :: kstart, kend, kproma, klev, kmaxchannels

type (mwave_phys_type), intent (inout) :: yd_mwave_phys_fw (kproma)
type (mwave_rad_type) , intent (inout) :: yd_mwave_rad     (kproma, n_satsens)

!* OTHER
integer (kind=jpim) :: iprof, inprof, i, jsat, inchan, ich
integer (kind=jpim), dimension (kproma) :: jprof

real    (kind=jprb), dimension (kproma,klev+1) :: zphys_ph_fw
real    (kind=jprb), dimension (kproma,klev)   :: zphys_rfl_fw, zphys_sfl_fw
real    (kind=jprb), dimension (kproma,klev)   :: zphys_cwc_fw, zphys_cic_fw , zphys_cc_fw
real    (kind=jprb), dimension (kproma,klev)   :: zphys_p_fw  , zphys_t_fw   , zphys_q_fw
real    (kind=jprb), dimension (kproma)        :: zphys_u10_fw, zphys_v10_fw , zphys_t2m_fw , zphys_q2m_fw
real    (kind=jprb), dimension (kproma)        :: zphys_lsm, zphys_tsfc_fw, zphys_lat, zphys_elevation
real    (kind=jprb), dimension (kproma)        :: zrad_zen, zrad_azi

real    (kind=jprb), dimension (kproma,kmaxchannels) :: zrad_tb_fw
real    (kind=jprb), dimension (kproma,kmaxchannels) :: zrad_tb_clr
real    (kind=jprb), dimension (kproma,kmaxchannels) :: zrad_ee_fw

real    (kind=jprb), dimension (kproma)              :: zcfrac_fw
real    (kind=jprb) :: zd19_clr, zd37_clr
real    (kind=jprb) :: zmp37
real    (kind=jprb) :: ztotal_cloud, zcloud_water_fraction

logical :: llprocess

! adjustment variables
real (kind=jprb) :: zhook_handle
    
!* INTERFACES
!#include "mwave_obsop_rttov.intfb.h"
#include "mwave_diags.intfb.h"
#include "mwave_cpfrac.intfb.h"

if (lhook) call dr_hook ('MWAVE_OBSOP',0,zhook_handle)

!* Loop over all satellite/sensor combinations
do jsat = 1, n_satsens

  inchan = mwave_id (jsat) % nchannels 
  if (mwave_id(jsat) % sensid == msensor_ssmis) then

    ! Nasty hack for SSMIS: The last 4 channels on SSMIS, at 60.792GHz, are not 
    ! included in the RTTOV-9 Mie tables, so we must avoid using them. In future
    ! we need to regenerate the Mie tables with 24 channels. The TL and AD versions
    ! are protected against this because we flag these channels in mwave_screen.F90
    inchan = 20
      
  endif


  !* Extract active profiles
  inprof    = 0
  jprof (:) = 0

  do i = kstart, kend

    !* Re-check land-sea contamination (resolution dependent) and for negative q after screening
    IF (.not. ldmwave_land) THEN
      if (yd_mwave_phys_fw (i) % lsm /= 0.0_JPRB) &
        & yd_mwave_rad (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_land)
    ENDIF
    if (any (yd_mwave_phys_fw (i) % q (1:klev) < 0.0_JPRB)) &
      & yd_mwave_rad (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_negative_q)

    !* Process any profile with at least one channel with no serious errors
    llprocess = .false.
    do ich = 1, inchan
      if ( .not. mwave_serious(yd_mwave_rad (i,jsat) % screen(ich)) ) then 
        llprocess = .true.
        exit
      endif
    enddo
    if (llprocess) then 

      inprof = inprof + 1
      jprof (inprof) = i

      zphys_p_fw    (inprof,1:klev)   = yd_mwave_phys_fw (i) % p   (1:klev)
      zphys_ph_fw   (inprof,1:klev+1) = yd_mwave_phys_fw (i) % ph  (1:klev+1)
      zphys_t_fw    (inprof,1:klev)   = yd_mwave_phys_fw (i) % t   (1:klev) 
      zphys_q_fw    (inprof,1:klev)   = yd_mwave_phys_fw (i) % q   (1:klev) 

      zphys_rfl_fw  (inprof,1:klev)   = yd_mwave_phys_fw (i) % rfl (1:klev) &
                                    & + yd_mwave_phys_fw (i) % crfl(1:klev)
      zphys_sfl_fw  (inprof,1:klev)   = yd_mwave_phys_fw (i) % sfl (1:klev) &
                                    & + yd_mwave_phys_fw (i) % csfl(1:klev) 
      zphys_cwc_fw  (inprof,1:klev)   = yd_mwave_phys_fw (i) % cwc (1:klev) 
      zphys_cic_fw  (inprof,1:klev)   = yd_mwave_phys_fw (i) % cic (1:klev) 
      zphys_cc_fw   (inprof,1:klev)   = yd_mwave_phys_fw (i) % cc  (1:klev)

      zphys_t2m_fw  (inprof) = yd_mwave_phys_fw (i) % t2m
      zphys_q2m_fw  (inprof) = yd_mwave_phys_fw (i) % q2m
      zphys_u10_fw  (inprof) = yd_mwave_phys_fw (i) % u10
      zphys_v10_fw  (inprof) = yd_mwave_phys_fw (i) % v10
      zphys_tsfc_fw (inprof) = yd_mwave_phys_fw (i) % tsk 

      zphys_lsm     (inprof) = yd_mwave_phys_fw (i) % lsm
      zphys_lat     (inprof) = asin( yd_mwave_phys_fw (i) % slat ) * 180.0_JPRB / rpi
      zphys_elevation (inprof) = yd_mwave_phys_fw (i) % orog ! metres

      if ( yd_mwave_rad (i,jsat) % zenith /= rmissing_angle ) then 
        ! Zenith angle available from observation data
        zrad_zen (inprof) = yd_mwave_rad (i,jsat) % zenith
      else
        ! Fixed zenith angle (SSM/I, SSMIS, AMSR-E)
        zrad_zen (inprof) = mwave_id (jsat) % zenith
      endif

      if ( yd_mwave_rad (i,jsat) % azimuth /= rmissing_angle .and. &
        &  mwave_id(jsat) % sensid == msensor_windsat ) then 
        ! Windsat requires azimuth angle for emissivity calculation
        zrad_azi (inprof) = yd_mwave_rad (i,jsat) % azimuth
      else
        ! Set azimuth angle to RMDI to trigger FASTEM-2 behaviour, i.e.
        ! no azimuth dependence.
        zrad_azi (inprof) = rmdi 
      endif

      IF (ldmwave_land) then  
      ! get stored land surface emissivity       
        zrad_ee_fw (inprof,1:inchan) = yd_mwave_rad (i,jsat) % pers1 (1:inchan)
      else
        ! use RTTOV's emissivity model
        zrad_ee_fw (inprof,1:inchan) = rmwave_no_emis
      endif

      !* Calculate condensate paths 
      call mwave_diags ( &
       & klev,                       &  ! in  : dimensions   
       & jsat,            &  ! in : satellite ID (for mwave) 
       & yd_mwave_phys_fw (i),            &  ! in: 2d/3d fields, FW
       & yd_mwave_rad (i,jsat) % diag)  ! inout: diagnostics array

      !* Calculate cloud / precip fraction
      call mwave_cpfrac ( &
       & klev,                       &  ! in  : dimensions   
       & jsat,                       &  ! in : satellite ID (for mwave) 
       & yd_mwave_phys_fw (i),       &  ! in: 2d/3d fields, FW
       & zcfrac_fw(inprof))             ! out: cloud / precip fraction
      yd_mwave_rad (i,jsat) % diag (imwave_cfrac) = zcfrac_fw(inprof)

      ! Avoid cold sectors in winter - model just can't produce enough liquid cloud
      ! See Tech. Memo. 620, section 2.6
      ! Identified by (i)   TCWV < 15 kg m^-2 (ii) CWF < 0.5 
      !               (iii) Total hydrometeors > 0.01 kg m^-2
      ! Do only in screening, to avoid "contaminating" huge numbers of obs
      if (lscreen) then
 
        ztotal_cloud = sum (yd_mwave_rad (i,jsat) % diag (imwave_tclw:imwave_tciw))
        if ( ztotal_cloud > 1.0E-12_JPRB ) then
          zcloud_water_fraction = yd_mwave_rad (i,jsat) % diag (imwave_tclw) / ztotal_cloud
        else
          zcloud_water_fraction = 1.0_JPRB
        endif
        if ( yd_mwave_rad (i,jsat) % diag (imwave_tcwv) < 15 &
          & .and. sum (yd_mwave_rad (i,jsat) % diag (imwave_tclw:imwave_swp)) > 0.01 &
          & .and. (zcloud_water_fraction < 0.5 ) ) then

          yd_mwave_rad (i,jsat) % screen(:) = &
           & ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_cold_sector)

        endif

      endif
      
    endif
  enddo                          

  if (inprof > 0) then

    !* RTTOV-SCATT
    call mwave_obsop_rttov (klev, inprof, &  ! in: levels, profiles
      &  inchan                           , &  ! in: channels
      &  jsat                             , &  ! in: satellite/sensor index
      &  zphys_p_fw    (1:inprof,:)       , &  ! in: pressure
      &  zphys_ph_fw   (1:inprof,:)       , &  ! in: pressure half levels
      &  zphys_t_fw    (1:inprof,:)       , &  ! in: temperature
      &  zphys_q_fw    (1:inprof,:)       , &  ! in: spec. humidity
      &  zphys_rfl_fw  (1:inprof,:)       , &  ! in: rain flux 
      &  zphys_sfl_fw  (1:inprof,:)       , &  ! in: snow flux
      &  zphys_cwc_fw  (1:inprof,:)       , &  ! in: cloud water content 
      &  zphys_cic_fw  (1:inprof,:)       , &  ! in: cloud ice content 
      &  zphys_cc_fw   (1:inprof,:)       , &  ! in: cloud coverage    
      &  zphys_t2m_fw  (1:inprof)         , &  ! in: 2m-temperature
      &  zphys_q2m_fw  (1:inprof)         , &  ! in: 2m-spec. humdity                            
      &  zphys_u10_fw  (1:inprof)         , &  ! in: 10m-u windspeed
      &  zphys_v10_fw  (1:inprof)         , &  ! in: 10m-v windspeed
      &  zphys_tsfc_fw (1:inprof)         , &  ! in: skin temperature
      &  zcfrac_fw     (1:inprof)         , &  ! in: cloud frac 
      &  zphys_lsm     (1:inprof)         , &  ! in: land fraction
      &  zphys_lat     (1:inprof)         , &  ! in: latitude (degrees)
      &  zphys_elevation (1:inprof)       , &  ! in: surface elevation (m)
      &  zrad_zen      (1:inprof)         , &  ! in: satellite zenith angle
      &  zrad_azi      (1:inprof)         , &  ! in: satellite azimuth angle
      &  zrad_ee_fw    (1:inprof,:)       , &  ! inout: emissivity
      &  zrad_tb_fw    (1:inprof,:)       , &  ! out: TB's
      &  zrad_tb_clr   (1:inprof,:))           ! out: TB's assuming clear sky (forward only)

    !* Restore to kproma-array                     
    do iprof = 1, inprof
      i = jprof (iprof)

      yd_mwave_rad (i,jsat) % fw (1:inchan)       = zrad_tb_fw (iprof,1:inchan)
      yd_mwave_rad (i,jsat) % pers1 (1:inchan) = zrad_ee_fw    (iprof,1:inchan)

      ! Calculate normalised polarisation differences (e.g. Petty 1994, Meteor. Atm. Phy.)
      ! as AVERAGE between observed and modelled. Used later as cloud/rain indicator for 
      ! error estimation and bias correction. We can only guarantee to have all channels
      ! available in the FG trajectory. No bias correction is applied but because 
      ! (a) the signal (80K) is much larger than typical bias sizes (2K) and
      ! (b) we are differencing model to model and obs to obs, I think it's OK. 
      zd19_clr = zrad_tb_clr (iprof,mwave_id(jsat) % i19v) - zrad_tb_clr (iprof,mwave_id(jsat) % i19h)
      zd37_clr = zrad_tb_clr (iprof,mwave_id(jsat) % i37v) - zrad_tb_clr (iprof,mwave_id(jsat) % i37h)

      if ( zd19_clr > 0.0_JPRB .and. zd37_clr > 0.0_JPRB) then

        if(lscreen) then
          ! First trajectory - can calculate only here when all obs channels available
          yd_mwave_rad (i,jsat) % p19 (imwave_p_obs) = ( 1.0_JPRB / zd19_clr ) * ( &
            &   yd_mwave_rad (i,jsat) % obs (mwave_id(jsat) % i19v)  &
            & - yd_mwave_rad (i,jsat) % obs (mwave_id(jsat) % i19h)  )           
          yd_mwave_rad (i,jsat) % p37 (imwave_p_obs) = ( 1.0_JPRB / zd37_clr ) * ( &
            &   yd_mwave_rad (i,jsat) % obs (mwave_id(jsat) % i37v)  &
            & - yd_mwave_rad (i,jsat) % obs (mwave_id(jsat) % i37h)  )  
        endif
   
        if(lifstraj) then
          ! Outer loop trajectory, including first. 
          yd_mwave_rad (i,jsat) % p19 (imwave_p_sim) = ( 1.0_JPRB / zd19_clr ) * ( &
            &   yd_mwave_rad (i,jsat) % fw  (mwave_id(jsat) % i19v)  &
            & - yd_mwave_rad (i,jsat) % fw  (mwave_id(jsat) % i19h)  )           
          yd_mwave_rad (i,jsat) % p37 (imwave_p_sim) = ( 1.0_JPRB / zd37_clr ) * ( &
            &   yd_mwave_rad (i,jsat) % fw  (mwave_id(jsat) % i37v)  &
            & - yd_mwave_rad (i,jsat) % fw  (mwave_id(jsat) % i37h)  ) 
        else
          yd_mwave_rad (i,jsat) % p19 (imwave_p_sim) = 0.0_JPRB 
          yd_mwave_rad (i,jsat) % p37 (imwave_p_sim) = 0.0_JPRB 
        endif
      
      else
        yd_mwave_rad (i,jsat) % p19 (1:nmwave_p) = 1.0_JPRB ! Pretend it's clear.
        yd_mwave_rad (i,jsat) % p37 (1:nmwave_p) = 1.0_JPRB
      endif

      if(lifstraj) then
        ! Avoid excess snow in moist physics / RTTOV-SCATT excessive response to snow in
        ! heavy cloud & precip regions at higher frequencies. See Tech. Memo. 620, Sec. 2.6
        if ( min( yd_mwave_rad (i,jsat) % p37 (imwave_p_obs), &
                & yd_mwave_rad (i,jsat) % p37 (imwave_p_sim)) < 0.8_JPRB) then

          where( mwave_id(jsat) % rcwn(1:inchan) > wn_snow_rt_problem ) 
            yd_mwave_rad (i,jsat) % screen(1:inchan) = &
              & ibset (yd_mwave_rad (i,jsat) % screen(1:inchan), mwave_excess_snow)
          endwhere

        endif
      endif

        if (lscreen) then

          ! Calculate observation errors based on a new "P37" model. These errors
          ! will be set in the FG traj and not changed thereafter, except in the
          ! passive monitoring case (see below)

          ! Mean normalised 37GHz polarisation difference
          zmp37 = 0.5 * ( yd_mwave_rad (i,jsat) % p37 (imwave_p_obs) &
              &         + yd_mwave_rad (i,jsat) % p37 (imwave_p_sim) )

          do ich = 1, inchan
            ! Estimated obs error
            yd_mwave_rad (i,jsat) % oerr (ich) = &
             & mwave_error_model (zmp37, jsat, ich, ldobs = .true.)
          enddo

        endif

        ! Set high obs errors in the case of passive monitoring 
        ! Note: When in the FG trajectory we don't know whether the obs       
        ! is passive or not (that is only known after the blacklisting). So
        ! this will be done multiple times to ensure it's correctly set in both the
        ! minimisations and the final traj (thus getting the right value in the ECMA)
        ! It is perfectly safe, if inelegant, to do this.
        where( btest( yd_mwave_rad (i,jsat) % screen(1:inchan), mwave_passive ) &
         & .and. yd_mwave_rad (i,jsat) % oerr (1:inchan) < rlimit_obs_err )
          yd_mwave_rad (i,jsat) % oerr (1:inchan) = rlimit_obs_err
        endwhere

    enddo
  endif

enddo

if (lhook) call dr_hook ('MWAVE_OBSOP',1,zhook_handle) 

end subroutine mwave_obsop
