subroutine mwave_obsop_test (kstart, kend, kproma, klev, kmaxchannels, &       ! in: dimensions        
                          &  yd_mwave_phys_fw, yd_mwave_phys_tl, yd_mwave_rad) ! in: 2d/3d fields, FW/TL

!**** Linearity and adjoint tests of observation operator for 
!     4D-Var in clouds/precipitation for microwave radiances 

!     P. Bauer   ECMWF    December 2005

!     1. Call MWAVE_OBSOP
!     2. Call MWAVE_OBSOP_TL
!     3. Call MWAVE_OBSOP with different scaling factor for finite differences
!     4. Call MWAVE_OBSOP_TL with fixed (small) perturbation
!     5. Call MWAVE_OBSOP_AD with TL perturbation
!     6. Print results to log file

!     MWAVE_OBSOP_TEST is called from MWAVE_OBSOP_SCREEN 

!     Modifications:
!     September 2006, P. Bauer, Use new F90   types
!     07 May 2008     A. Geer   Multiple sensor version
!     10 June 2009    A. Geer   Passive mode enabled; per-channel status flag
!     16-Dec-2009     F. Karbou Surface emissivity estimation

!**** MODULES      

!* KIND     
use parkind1, only: jpim, jprb
use yomhook , only: lhook, dr_hook

use yomlun  , only: nulerr
!* IFS
use yommwave, only: mwave_phys_type, mwave_rad_type, n_satsens, mwave_id, &    
 & mwave_ok, mwave_ok_a1, mwave_ok_a3     
use yomcst,   only: rpi
implicit none
         
!* Common variables
integer (kind=jpim), intent    (in) :: kstart, kend, kproma, klev, kmaxchannels

type (mwave_phys_type), intent (in) :: yd_mwave_phys_fw (kproma), yd_mwave_phys_tl (kproma)
type (mwave_rad_type) , intent (in) :: yd_mwave_rad     (kproma, n_satsens)

!* Local variables
real (kind=jprb), target :: zphys_p_fw    (kproma,klev)
real (kind=jprb), target :: zphys_ph_fw   (kproma,klev+1)
real (kind=jprb), target :: zphys_t_fw    (kproma,klev)
real (kind=jprb), target :: zphys_q_fw    (kproma,klev)
real (kind=jprb), target :: zphys_cc_fw   (kproma,klev)
real (kind=jprb), target :: zphys_cwc_fw  (kproma,klev)
real (kind=jprb), target :: zphys_cic_fw  (kproma,klev)
real (kind=jprb), target :: zphys_rfl_fw  (kproma,klev)
real (kind=jprb), target :: zphys_sfl_fw  (kproma,klev)
real (kind=jprb), target :: zphys_crfl_fw (kproma,klev)
real (kind=jprb), target :: zphys_csfl_fw (kproma,klev)
real (kind=jprb), target :: zphys_pfrc_fw (kproma,klev)
    
real (kind=jprb), target :: zphys_p_tl    (kproma,klev)
real (kind=jprb), target :: zphys_ph_tl   (kproma,klev+1)
real (kind=jprb), target :: zphys_t_tl    (kproma,klev)
real (kind=jprb), target :: zphys_q_tl    (kproma,klev)
real (kind=jprb), target :: zphys_cc_tl   (kproma,klev)
real (kind=jprb), target :: zphys_cwc_tl  (kproma,klev)
real (kind=jprb), target :: zphys_cic_tl  (kproma,klev)
real (kind=jprb), target :: zphys_rfl_tl  (kproma,klev)
real (kind=jprb), target :: zphys_sfl_tl  (kproma,klev)
real (kind=jprb), target :: zphys_crfl_tl (kproma,klev)
real (kind=jprb), target :: zphys_csfl_tl (kproma,klev)
real (kind=jprb), target :: zphys_pfrc_tl (kproma,klev)
    
real (kind=jprb), target :: zphys_p_ad    (kproma,klev)
real (kind=jprb), target :: zphys_ph_ad   (kproma,klev+1)
real (kind=jprb), target :: zphys_t_ad    (kproma,klev)
real (kind=jprb), target :: zphys_q_ad    (kproma,klev)
real (kind=jprb), target :: zphys_cc_ad   (kproma,klev)
real (kind=jprb), target :: zphys_cwc_ad  (kproma,klev)
real (kind=jprb), target :: zphys_cic_ad  (kproma,klev)
real (kind=jprb), target :: zphys_rfl_ad  (kproma,klev)
real (kind=jprb), target :: zphys_sfl_ad  (kproma,klev)
real (kind=jprb), target :: zphys_crfl_ad (kproma,klev)
real (kind=jprb), target :: zphys_csfl_ad (kproma,klev)
real (kind=jprb), target :: zphys_pfrc_ad (kproma,klev)
              
type (mwave_phys_type) :: yl_phys_fw (kproma), yl_phys_tl (kproma), yl_phys_ad (kproma)
type (mwave_rad_type)  :: yl_zrad    (kproma, n_satsens)

real (kind=jprb), dimension (kproma,kmaxchannels,n_satsens) :: ztb_fw, ztb_fd, ztb_tl, ztb_lin
real (kind=jprb), dimension (kproma,kmaxchannels,n_satsens) :: zee_fw

real    (kind=jprb) :: zip_tl, zip_ad, zz, zmp, zdtor
integer (kind=jpim) :: i, ilev, ichannels, jsat, inchan

integer (kind=jpim), parameter :: jpks = kind (zz)


! adjustment variables
real (kind=jprb) :: zhook_handle
    
!* INTERFACES
#include "mwave_obsop.intfb.h"
#include "mwave_obsop_tl.intfb.h"
#include "mwave_obsop_ad.intfb.h"

if (lhook) call dr_hook ('MWAVE_OBSOP_TEST',0,zhook_handle)

zdtor = rpi / 180.0_JPRB 

!* Local allocation
do i = kstart, kend
  yl_phys_fw (i) % nlevels   = klev

  yl_phys_fw (i) % p   => zphys_p_fw   (i,:)
  yl_phys_fw (i) % ph  => zphys_ph_fw  (i,:)
  yl_phys_fw (i) % t   => zphys_t_fw   (i,:)
  yl_phys_fw (i) % q   => zphys_q_fw   (i,:)
  yl_phys_fw (i) % rfl => zphys_rfl_fw (i,:)
  yl_phys_fw (i) % sfl => zphys_sfl_fw (i,:)
  yl_phys_fw (i) % crfl => zphys_crfl_fw (i,:)
  yl_phys_fw (i) % csfl => zphys_csfl_fw (i,:)
  yl_phys_fw (i) % cwc => zphys_cwc_fw (i,:)
  yl_phys_fw (i) % cic => zphys_cic_fw (i,:)
  yl_phys_fw (i) % cc  => zphys_cc_fw  (i,:)
  yl_phys_fw (i) % pfrc => zphys_pfrc_fw (i,:)

  yl_phys_tl (i) % nlevels   = klev 

  yl_phys_tl (i) % p   => zphys_p_tl   (i,:)
  yl_phys_tl (i) % ph  => zphys_ph_tl  (i,:)
  yl_phys_tl (i) % t   => zphys_t_tl   (i,:)
  yl_phys_tl (i) % q   => zphys_q_tl   (i,:)
  yl_phys_tl (i) % rfl => zphys_rfl_tl (i,:)
  yl_phys_tl (i) % sfl => zphys_sfl_tl (i,:)
  yl_phys_tl (i) % crfl => zphys_crfl_tl (i,:)
  yl_phys_tl (i) % csfl => zphys_csfl_tl (i,:)
  yl_phys_tl (i) % cwc => zphys_cwc_tl (i,:)
  yl_phys_tl (i) % cic => zphys_cic_tl (i,:)
  yl_phys_tl (i) % cc  => zphys_cc_tl  (i,:)
  yl_phys_tl (i) % pfrc => zphys_pfrc_tl (i,:)

  yl_phys_ad (i) % nlevels   = klev    

  yl_phys_ad (i) % p   => zphys_p_ad   (i,:)
  yl_phys_ad (i) % ph  => zphys_ph_ad  (i,:)
  yl_phys_ad (i) % t   => zphys_t_ad   (i,:)
  yl_phys_ad (i) % q   => zphys_q_ad   (i,:)
  yl_phys_ad (i) % rfl => zphys_rfl_ad (i,:)
  yl_phys_ad (i) % sfl => zphys_sfl_ad (i,:)
  yl_phys_ad (i) % crfl => zphys_crfl_ad (i,:)
  yl_phys_ad (i) % csfl => zphys_csfl_ad (i,:)
  yl_phys_ad (i) % cwc => zphys_cwc_ad (i,:)
  yl_phys_ad (i) % cic => zphys_cic_ad (i,:)
  yl_phys_ad (i) % cc  => zphys_cc_ad  (i,:)
  yl_phys_ad (i) % pfrc => zphys_pfrc_ad (i,:)

enddo

do jsat = 1, n_satsens       
  do ichannels = 1, kmaxchannels           
    yl_zrad (kstart:kend,jsat) % screen(ichannels) = &
      &  yd_mwave_rad (kstart:kend,jsat) % screen(ichannels)
    yl_zrad (kstart:kend,jsat) % oerr(ichannels)   = &
      &  yd_mwave_rad (kstart:kend,jsat) % oerr(ichannels)
  enddo
enddo
yl_zrad (kstart:kend,:) % zenith = yd_mwave_rad (kstart:kend,:) % zenith
yl_zrad (kstart:kend,:) % azimuth = yd_mwave_rad (kstart:kend,:) % azimuth

!*** Linearity test
do i = kstart, kend
  yl_phys_fw (i) % p   (1:klev)   = yd_mwave_phys_fw (i) % p   (1:klev)  
  yl_phys_fw (i) % ph  (1:klev+1) = yd_mwave_phys_fw (i) % ph  (1:klev+1)
  yl_phys_fw (i) % t   (1:klev)   = yd_mwave_phys_fw (i) % t   (1:klev)  
  yl_phys_fw (i) % q   (1:klev)   = yd_mwave_phys_fw (i) % q   (1:klev)  

  yl_phys_fw (i) % rfl (1:klev)   = yd_mwave_phys_fw (i) % rfl (1:klev)  
  yl_phys_fw (i) % sfl (1:klev)   = yd_mwave_phys_fw (i) % sfl (1:klev)  
  yl_phys_fw (i) % crfl (1:klev)  = yd_mwave_phys_fw (i) % crfl (1:klev)  
  yl_phys_fw (i) % csfl (1:klev)  = yd_mwave_phys_fw (i) % csfl (1:klev) 
  yl_phys_fw (i) % cwc (1:klev)   = yd_mwave_phys_fw (i) % cwc (1:klev)  
  yl_phys_fw (i) % cic (1:klev)   = yd_mwave_phys_fw (i) % cic (1:klev)  
  yl_phys_fw (i) % cc  (1:klev)   = yd_mwave_phys_fw (i) % cc  (1:klev)  
  yl_phys_fw (i) % pfrc(1:klev)   = yd_mwave_phys_fw (i) % pfrc(1:klev)  

  yl_phys_fw (i) % t2m            = yd_mwave_phys_fw (i) % t2m           
  yl_phys_fw (i) % q2m            = yd_mwave_phys_fw (i) % q2m           
  yl_phys_fw (i) % u10            = yd_mwave_phys_fw (i) % u10           
  yl_phys_fw (i) % v10            = yd_mwave_phys_fw (i) % v10           
  yl_phys_fw (i) % tsk            = yd_mwave_phys_fw (i) % tsk           

  yl_phys_fw (i) % lsm            = yd_mwave_phys_fw (i) % lsm           
  yl_phys_fw (i) % sim            = yd_mwave_phys_fw (i) % sim      
  yl_phys_fw (i) % slat           = yd_mwave_phys_fw (i) % slat      
  yl_phys_fw (i) % slon           = yd_mwave_phys_fw (i) % slon         
  yl_phys_fw (i) % clon           = yd_mwave_phys_fw (i) % clon  
  yl_phys_fw (i) % orog           = yd_mwave_phys_fw (i) % orog  
 
  yl_phys_tl (i) % p   (1:klev)   = yd_mwave_phys_tl (i) % p   (1:klev)  
  yl_phys_tl (i) % ph  (1:klev+1) = yd_mwave_phys_tl (i) % ph  (1:klev+1)
  yl_phys_tl (i) % t   (1:klev)   = yd_mwave_phys_tl (i) % t   (1:klev)  
  yl_phys_tl (i) % q   (1:klev)   = yd_mwave_phys_tl (i) % q   (1:klev)  

  yl_phys_tl (i) % rfl (1:klev)   = yd_mwave_phys_tl (i) % rfl (1:klev)  
  yl_phys_tl (i) % sfl (1:klev)   = yd_mwave_phys_tl (i) % sfl (1:klev)  
  yl_phys_tl (i) % crfl (1:klev)  = yd_mwave_phys_tl (i) % crfl (1:klev)  
  yl_phys_tl (i) % csfl (1:klev)  = yd_mwave_phys_tl (i) % csfl (1:klev) 
  yl_phys_tl (i) % cwc (1:klev)   = yd_mwave_phys_tl (i) % cwc (1:klev)  
  yl_phys_tl (i) % cic (1:klev)   = yd_mwave_phys_tl (i) % cic (1:klev)  
  yl_phys_tl (i) % cc  (1:klev)   = yd_mwave_phys_tl (i) % cc  (1:klev)  
  yl_phys_tl (i) % pfrc(1:klev)   = yd_mwave_phys_tl (i) % pfrc(1:klev)  

  yl_phys_tl (i) % t2m            = yd_mwave_phys_tl (i) % t2m           
  yl_phys_tl (i) % q2m            = yd_mwave_phys_tl (i) % q2m           
  yl_phys_tl (i) % u10            = yd_mwave_phys_tl (i) % u10           
  yl_phys_tl (i) % v10            = yd_mwave_phys_tl (i) % v10                  
  yl_phys_tl (i) % tsk            = yd_mwave_phys_tl (i) % tsk        

  yl_phys_tl (i) % lsm            = 0.0_JPRB  
  yl_phys_tl (i) % sim            = 0.0_JPRB      
  yl_phys_tl (i) % slat           = 0.0_JPRB      
  yl_phys_tl (i) % slon           = 0.0_JPRB         
  yl_phys_tl (i) % clon           = 0.0_JPRB  
  yl_phys_tl (i) % orog           = 0.0_JPRB 
   
enddo

call mwave_obsop_tl (kstart, kend, kproma, klev, kmaxchannels, yl_phys_fw, yl_phys_tl, yl_zrad)
                     
do i = kstart, kend
  do jsat = 1, n_satsens 
    ztb_fw (i,1:kmaxchannels,jsat) = yl_zrad (i,jsat) % fw (1:kmaxchannels)
    ztb_tl (i,1:kmaxchannels,jsat) = yl_zrad (i,jsat) % tl (1:kmaxchannels)
    zee_fw (i,1:kmaxchannels,jsat) = yl_zrad (i,jsat) % pers1 (1:kmaxchannels)

  enddo

  yl_phys_fw (i) % p   (1:klev)   = yd_mwave_phys_fw (i) % p   (1:klev)   &
                                  & + yd_mwave_phys_tl (i) % p   (1:klev)   
  yl_phys_fw (i) % ph  (1:klev+1) = yd_mwave_phys_fw (i) % ph  (1:klev+1) &
                                  & + yd_mwave_phys_tl (i) % ph  (1:klev+1) 
  yl_phys_fw (i) % t   (1:klev)   = yd_mwave_phys_fw (i) % t   (1:klev)   &
                                  & + yd_mwave_phys_tl (i) % t   (1:klev)   
  yl_phys_fw (i) % q   (1:klev)   = yd_mwave_phys_fw (i) % q   (1:klev)   &
                                  & + yd_mwave_phys_tl (i) % q   (1:klev)   

  yl_phys_fw (i) % rfl (1:klev)   = yd_mwave_phys_fw (i) % rfl (1:klev)   &
                                  & + yd_mwave_phys_tl (i) % rfl (1:klev)   
  yl_phys_fw (i) % sfl (1:klev)   = yd_mwave_phys_fw (i) % sfl (1:klev)   &
                                  & + yd_mwave_phys_tl (i) % sfl (1:klev)   
  yl_phys_fw (i) % crfl (1:klev)   = yd_mwave_phys_fw (i) % crfl (1:klev)   &
                                  & + yd_mwave_phys_tl (i) % crfl (1:klev)   
  yl_phys_fw (i) % csfl (1:klev)   = yd_mwave_phys_fw (i) % csfl (1:klev)   &
                                  & + yd_mwave_phys_tl (i) % csfl (1:klev)   
  yl_phys_fw (i) % cwc (1:klev)   = yd_mwave_phys_fw (i) % cwc (1:klev)   &
                                  & + yd_mwave_phys_tl (i) % cwc (1:klev)   
  yl_phys_fw (i) % cic (1:klev)   = yd_mwave_phys_fw (i) % cic (1:klev)   &
                                  & + yd_mwave_phys_tl (i) % cic (1:klev)   
  yl_phys_fw (i) % cc  (1:klev)   = yd_mwave_phys_fw (i) % cc  (1:klev)   &
                                  & + yd_mwave_phys_tl (i) % cc  (1:klev)   
  yl_phys_fw (i) % pfrc(1:klev)   = yd_mwave_phys_fw (i) % pfrc(1:klev)   &
                                  & + yd_mwave_phys_tl (i) % pfrc(1:klev)   

  yl_phys_fw (i) % t2m = yd_mwave_phys_fw (i) % t2m + yd_mwave_phys_tl (i) % t2m            
  yl_phys_fw (i) % q2m = yd_mwave_phys_fw (i) % q2m + yd_mwave_phys_tl (i) % q2m            
  yl_phys_fw (i) % u10 = yd_mwave_phys_fw (i) % u10 + yd_mwave_phys_tl (i) % u10            
  yl_phys_fw (i) % v10 = yd_mwave_phys_fw (i) % v10 + yd_mwave_phys_tl (i) % v10         
  yl_phys_fw (i) % tsk = yd_mwave_phys_fw (i) % tsk + yd_mwave_phys_tl (i) % tsk            
enddo

call mwave_obsop (kstart, kend, kproma, klev, kmaxchannels, yl_phys_fw, yl_zrad)

do jsat = 1, n_satsens 
  inchan = mwave_id(jsat) % nchannels                   
  do i = kstart, kend 

    ! Sometimes adding FW to TL gives negative Q and a failure code, so ignore those
    ! by checking yl_zrad rather than yd_mwave_rad
    if (mwave_ok_a1(yl_zrad (i,jsat) % screen(1:inchan))) then   

      ztb_fd  (i,1:inchan,jsat) = yl_zrad (i,jsat) % fw (1:inchan)
      ztb_lin (i,1:inchan,jsat) = 999.9_jprb

      where (ztb_tl (i,1:inchan,jsat) /= 0.0_jprb) ztb_lin (i,1:inchan,jsat) = &
        & (ztb_fd (i,1:inchan,jsat) - ztb_fw  (i,1:inchan,jsat)) / ztb_tl (i,1:inchan,jsat)

      write (nulerr,"(a23,i5,1x,i5,1x,f10.5,1x,f10.5,30(1x,f10.5))") &
        & 'MWAVE_OBSOP_TEST TL: ', i, jsat, &
        & asin(yd_mwave_phys_fw (i) % slat)/zdtor, asin(yd_mwave_phys_fw (i) % slon)/zdtor, &
        & ztb_lin (i,1:inchan,jsat)

    endif
  enddo
enddo

!*** Adjoint test
!* Produce RTTOVSCATT_AD input: TB's, tendencies and surface precip
do i = kstart, kend 
  yl_phys_fw (i) % p   (1:klev)   = yd_mwave_phys_fw (i) % p   (1:klev)  
  yl_phys_fw (i) % ph  (1:klev+1) = yd_mwave_phys_fw (i) % ph  (1:klev+1)
  yl_phys_fw (i) % t   (1:klev)   = yd_mwave_phys_fw (i) % t   (1:klev)  
  yl_phys_fw (i) % q   (1:klev)   = yd_mwave_phys_fw (i) % q   (1:klev)  

  yl_phys_fw (i) % rfl (1:klev)   = yd_mwave_phys_fw (i) % rfl (1:klev)  
  yl_phys_fw (i) % sfl (1:klev)   = yd_mwave_phys_fw (i) % sfl (1:klev)  
  yl_phys_fw (i) % crfl (1:klev)  = yd_mwave_phys_fw (i) % crfl (1:klev)  
  yl_phys_fw (i) % csfl (1:klev)  = yd_mwave_phys_fw (i) % csfl (1:klev)  
  yl_phys_fw (i) % cwc (1:klev)   = yd_mwave_phys_fw (i) % cwc (1:klev)  
  yl_phys_fw (i) % cic (1:klev)   = yd_mwave_phys_fw (i) % cic (1:klev)  
  yl_phys_fw (i) % cc  (1:klev)   = yd_mwave_phys_fw (i) % cc  (1:klev)  
  yl_phys_fw (i) % pfrc(1:klev)   = yd_mwave_phys_fw (i) % pfrc(1:klev)

  yl_phys_fw (i) % t2m            = yd_mwave_phys_fw (i) % t2m           
  yl_phys_fw (i) % q2m            = yd_mwave_phys_fw (i) % q2m           
  yl_phys_fw (i) % u10            = yd_mwave_phys_fw (i) % u10           
  yl_phys_fw (i) % v10            = yd_mwave_phys_fw (i) % v10           
  yl_phys_fw (i) % tsk            = yd_mwave_phys_fw (i) % tsk           

  yl_phys_fw (i) % lsm            = yd_mwave_phys_fw (i) % lsm           
  yl_phys_fw (i) % sim            = yd_mwave_phys_fw (i) % sim      
  yl_phys_fw (i) % slat           = yd_mwave_phys_fw (i) % slat      
  yl_phys_fw (i) % slon           = yd_mwave_phys_fw (i) % slon         
  yl_phys_fw (i) % clon           = yd_mwave_phys_fw (i) % clon  
  yl_phys_fw (i) % orog           = yd_mwave_phys_fw (i) % orog  

  !* Initialize
  yl_phys_ad (i) % p   (1:klev)   = 0.0_jprb
  yl_phys_ad (i) % ph  (1:klev+1) = 0.0_jprb 
  yl_phys_ad (i) % t   (1:klev)   = 0.0_jprb 
  yl_phys_ad (i) % q   (1:klev)   = 0.0_jprb 

  yl_phys_ad (i) % rfl (1:klev)   = 0.0_jprb 
  yl_phys_ad (i) % sfl (1:klev)   = 0.0_jprb 
  yl_phys_ad (i) % crfl (1:klev)  = 0.0_jprb 
  yl_phys_ad (i) % csfl (1:klev)  = 0.0_jprb 
  yl_phys_ad (i) % cwc (1:klev)   = 0.0_jprb 
  yl_phys_ad (i) % cic (1:klev)   = 0.0_jprb 
  yl_phys_ad (i) % cc  (1:klev)   = 0.0_jprb 
  yl_phys_ad (i) % pfrc(1:klev)   = 0.0_jprb 

  yl_phys_ad (i) % t2m            = 0.0_jprb 
  yl_phys_ad (i) % q2m            = 0.0_jprb 
  yl_phys_ad (i) % u10            = 0.0_jprb 
  yl_phys_ad (i) % v10            = 0.0_jprb 
  yl_phys_ad (i) % tsk            = 0.0_jprb

  yl_phys_ad (i) % lsm            = 0.0_jprb 
  yl_phys_ad (i) % sim            = 0.0_JPRB      
  yl_phys_ad (i) % slat           = 0.0_JPRB      
  yl_phys_ad (i) % slon           = 0.0_JPRB         
  yl_phys_ad (i) % clon           = 0.0_JPRB  
  yl_phys_ad (i) % orog           = 0.0_JPRB 

  do jsat = 1, n_satsens 
    inchan = mwave_id(jsat) % nchannels                                       
    do ichannels = 1, inchan
      if (mwave_ok(yd_mwave_rad (i,jsat) % screen(ichannels))) then
        yl_zrad (i,jsat) % ad (ichannels) = ztb_tl (i,ichannels,jsat)
      else
        yl_zrad (i,jsat) % ad (ichannels) = 0.0_jprb
      endif
    enddo
    yl_zrad (i,jsat) % pers1 (1:kmaxchannels) = zee_fw (i,1:kmaxchannels,jsat)
    yl_zrad (i,jsat) % screen(:) = yd_mwave_rad (i,jsat) % screen(:)
    yl_zrad (i,jsat) % oerr(:)   = yd_mwave_rad (i,jsat) % oerr(:)
    yl_zrad (i,jsat) % zenith = yd_mwave_rad (i,jsat) % zenith
    yl_zrad (i,jsat) % azimuth   = yd_mwave_rad (i,jsat) % azimuth
  enddo
enddo
      
call mwave_obsop_ad (kstart, kend, kproma, klev, kmaxchannels, yl_phys_fw, yl_phys_ad, yl_zrad)

zip_tl = 0.0_jprb
zip_ad = 0.0_jprb

!* 1st scalar product: operator output   
do jsat = 1, n_satsens 
  inchan = mwave_id(jsat) % nchannels                   
 
  do i = kstart, kend                     
      do ichannels = 1, inchan
      if (mwave_ok(yd_mwave_rad (i,jsat) % screen(ichannels))) then
        zip_tl = zip_tl + ztb_tl (i,ichannels,jsat) * ztb_tl (i,ichannels,jsat)
      endif
      enddo
  enddo
enddo
         
!* 2nd scalar product: operator input   
do i = kstart, kend                     
  do ilev = 1, klev
    zip_ad = zip_ad + yl_phys_ad (i) % p   (ilev) * yl_phys_tl (i) % p   (ilev) 
    zip_ad = zip_ad + yl_phys_ad (i) % ph  (ilev) * yl_phys_tl (i) % ph  (ilev) 
    zip_ad = zip_ad + yl_phys_ad (i) % t   (ilev) * yl_phys_tl (i) % t   (ilev) 
    zip_ad = zip_ad + yl_phys_ad (i) % q   (ilev) * yl_phys_tl (i) % q   (ilev) 

    zip_ad = zip_ad + yl_phys_ad (i) % rfl (ilev) * yl_phys_tl (i) % rfl (ilev) 
    zip_ad = zip_ad + yl_phys_ad (i) % sfl (ilev) * yl_phys_tl (i) % sfl (ilev) 
    zip_ad = zip_ad + yl_phys_ad (i) % crfl(ilev) * yl_phys_tl (i) % crfl(ilev) 
    zip_ad = zip_ad + yl_phys_ad (i) % csfl(ilev) * yl_phys_tl (i) % csfl(ilev) 
    zip_ad = zip_ad + yl_phys_ad (i) % cwc (ilev) * yl_phys_tl (i) % cwc (ilev) 
    zip_ad = zip_ad + yl_phys_ad (i) % cic (ilev) * yl_phys_tl (i) % cic (ilev) 
    zip_ad = zip_ad + yl_phys_ad (i) % cc  (ilev) * yl_phys_tl (i) % cc  (ilev) 
    zip_ad = zip_ad + yl_phys_ad (i) % pfrc(ilev) * yl_phys_tl (i) % pfrc(ilev) 
  enddo

  zip_ad = zip_ad + yl_phys_ad (i) % ph (klev+1) * yl_phys_tl (i) % ph (klev+1) 

  zip_ad = zip_ad + yl_phys_ad (i) % t2m * yl_phys_tl (i) % t2m 
  zip_ad = zip_ad + yl_phys_ad (i) % q2m * yl_phys_tl (i) % q2m 
  zip_ad = zip_ad + yl_phys_ad (i) % u10 * yl_phys_tl (i) % u10 
  zip_ad = zip_ad + yl_phys_ad (i) % v10 * yl_phys_tl (i) % v10 
  zip_ad = zip_ad + yl_phys_ad (i) % tsk * yl_phys_tl (i) % tsk 

enddo

!* Evaluate vs. machine precision  
if (mwave_ok_a3(kproma,yd_mwave_rad)) then 

  if (zip_tl == 0.0) then
      zz = 1.0_jprb
  else
      zz = zip_tl
  endif
  zmp = abs(real(real(zip_tl,jpks)-real(zip_ad,jpks),jpks)/epsilon(zz)/zz)

  write (nulerr,"(a23,1x,F20.1,2(1x,e30.10))") 'MWAVE_OBSOP_TEST AD: ', zmp, zip_tl, zip_ad

endif

if (lhook) call dr_hook ('MWAVE_OBSOP_TEST',1,zhook_handle) 

end subroutine mwave_obsop_test
