subroutine mwave_screen (kstart, kend, kproma, klev, kmaxchannels, &   ! in    : dimensions        
 & yd_mwave_phys, yd_mwave_rad)                  ! in/out: 2d/3d fields


!**** Observation screening for 
!     4D-Var in clouds/precipitation for microwave radiances 

!     P. Bauer   ECMWF    May 2005

!     1. Pre-screening        

!     MWAVE_SCREEN is called from CALLPAR (ifs/phys_ec)

!     Modifications:
!     September 2006, P. Bauer, Recoded to outsource phys operators to CALLPAR
!                               Use new F90 types
!     February 2008 , P. Bauer, Added AD-based sensitivity check
!     16-Apr-2008     A. Geer   Extension to multiple sensors
!     01-Oct-2008     A. Geer   Tidied and now works properly with multiple sensors 
!     10-Jun-2009     A. Geer   Status flag is now per-channel; passive monitoring
!                               is possible; reject obs if important channels are missing.
!     16-Dec-2009     F. Karbou Surface emissivity estimation
!**** MODULES      

!* KIND     
use parkind1, only: jpim, jprb
use yomhook , only: lhook, dr_hook

!* IFS
use yommwave, only: mwave_id, rlimit_lat, mwave_phys_type, mwave_rad_type, &
 & ldmwave_sens, n_satsens, mwave_no_obs, mwave_missing, &
 & mwave_obs_tb,  mwave_negative_q, mwave_tsfc, mwave_land, &
 & mwave_sea_ice, mwave_latitude, mwave_fg_depar, mwave_active, &
 & mwave_ok, mwave_ok_a3, mwave_ok_a1, &
 & mwave_missing_tb, mwave_ssmis_hack, ldmwave_land   
use yomtvrad, only: msensor_ssmis
use yomancs,  only: rmdi

implicit none

!* Common variables
integer (kind=jpim), intent (in)       :: kstart, kend, kproma, klev, kmaxchannels

type (mwave_phys_type), intent (inout) :: yd_mwave_phys (kproma)
type (mwave_rad_type) , intent (inout) :: yd_mwave_rad  (kproma,n_satsens)

!* Local variables
real (kind=jprb), target :: zphys_p    (kproma,klev)
real (kind=jprb), target :: zphys_ph   (kproma,klev+1)
real (kind=jprb), target :: zphys_t    (kproma,klev)
real (kind=jprb), target :: zphys_q    (kproma,klev)
real (kind=jprb), target :: zphys_cc   (kproma,klev)
real (kind=jprb), target :: zphys_cwc  (kproma,klev)
real (kind=jprb), target :: zphys_cic  (kproma,klev)
real (kind=jprb), target :: zphys_rfl  (kproma,klev)
real (kind=jprb), target :: zphys_sfl  (kproma,klev)

type (mwave_phys_type) :: yl_mwave_phys_ad (kproma)

type (mwave_rad_type) :: yl_mwave_rad (kproma,n_satsens)

integer (kind=jpim) :: i, ichan, ilev, jsat

! Adjustment variables
real (kind=jprb) :: zhook_handle

!* INTERFACES
#include "mwave_obsop.intfb.h"
#include "mwave_obsop_ad.intfb.h"
#include "mwave_emis.intfb.h"

if (lhook) call dr_hook ('MWAVE_SCREEN',0,zhook_handle)

!* Loop over all satellite/sensor combinations
do jsat = 1, n_satsens

  !* Loop over active profiles
  do i = kstart, kend

    !* Skip if no active or passive observation
    if( .not. mwave_ok_a1( yd_mwave_rad (i,jsat) % screen(1:mwave_id(jsat) % nchannels) ) ) cycle

    !* TB limits
    do ichan = 1, mwave_id(jsat) % nchannels
      if (yd_mwave_rad (i,jsat) % obs (ichan) == rmdi ) then
        yd_mwave_rad (i,jsat) % screen (ichan) = ibset (yd_mwave_rad (i,jsat) % screen (ichan), mwave_missing_tb)
      elseif (yd_mwave_rad (i,jsat) % obs (ichan) < -10.0_JPRB .or. yd_mwave_rad (i,jsat) % obs (ichan) > 350.0_JPRB) then
        yd_mwave_rad (i,jsat) % screen (ichan) = ibset (yd_mwave_rad (i,jsat) % screen (ichan), mwave_obs_tb)
      endif
    enddo

    if (mwave_id(jsat) % sensid == msensor_ssmis) then

      ! The last 4 channels on SSMIS, at 60.792GHz, are not included in 
      ! the RTTOV-9 Mie tables, so we must avoid using them.
      yd_mwave_rad (i,jsat) % screen(21:24) = ibset (yd_mwave_rad (i,jsat) % screen(21:24), mwave_ssmis_hack)

    endif

    !* Check important channels and if data is missing or bad, reject whole obs
    if ( .not. mwave_ok( yd_mwave_rad (i,jsat) % screen (mwave_id(jsat) % i19v) )) &
     & yd_mwave_rad (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_missing)
    if ( .not. mwave_ok( yd_mwave_rad (i,jsat) % screen (mwave_id(jsat) % i19h) )) &
     & yd_mwave_rad (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_missing)
    if ( .not. mwave_ok( yd_mwave_rad (i,jsat) % screen (mwave_id(jsat) % i37v) )) &
     & yd_mwave_rad (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_missing)
    if ( .not. mwave_ok( yd_mwave_rad (i,jsat) % screen (mwave_id(jsat) % i37h) )) &
     & yd_mwave_rad (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_missing)

    !* Check for negative q
    if (any (yd_mwave_phys (i) % q (1:klev) < 0.0_JPRB)) &
     & yd_mwave_rad  (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_negative_q)

    !* Land/sea-ice contamination latitude limits
    if (yd_mwave_phys (i) % tsk < 274.0_JPRB) &
     & yd_mwave_rad (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_tsfc)

    IF (.not. ldmwave_land) THEN
      if (yd_mwave_phys (i) % lsm /=  0.0_JPRB) &
       & yd_mwave_rad (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_land)
    ENDIF
    if (yd_mwave_phys (i) % sim /=  0.0_JPRB) &
     & yd_mwave_rad (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_sea_ice)

    if (abs (yd_mwave_phys (i) % slat) > rlimit_lat) &
     & yd_mwave_rad (i,jsat) % screen(:) = ibset (yd_mwave_rad (i,jsat) % screen(:), mwave_latitude)

  enddo  ! i-loop

enddo

if ( mwave_ok_a3( kend-kstart+1, yd_mwave_rad (kstart:kend,:))) then

  if (ldmwave_land) then

    !* calculate the surface emissivity
    call mwave_emis (kstart, kend, kproma, klev, kmaxchannels, yd_mwave_phys, yd_mwave_rad)

  endif

  !* Call forward operator
  call mwave_obsop (kstart, kend, kproma, klev, kmaxchannels, yd_mwave_phys, yd_mwave_rad)

  !* Check if FG-departures within rlimit_fgc * observation error; observation error scaled as function of total water path
  do jsat = 1, n_satsens
    do i = kstart, kend
      do ichan = 1, mwave_id(jsat) % nchannels
        if (mwave_ok( yd_mwave_rad (i,jsat) % screen(ichan) )) then    
          if (abs (yd_mwave_rad (i,jsat) % obs (ichan) - yd_mwave_rad (i,jsat) % fw (ichan)) &
           & > yd_mwave_rad (i,jsat) % oerr (ichan) * mwave_id(jsat) % rlimit_fgc (ichan)) &
           &   yd_mwave_rad (i,jsat) % screen(ichan) = ibset (yd_mwave_rad (i,jsat) % screen(ichan), mwave_fg_depar)
        endif
      enddo
    enddo
  enddo

endif

!* Apply sensitivity test using gradient norm
if (ldmwave_sens .and.  mwave_ok_a3( kend-kstart+1, yd_mwave_rad (kstart:kend,:))) then  

  !* Allocate
  do i = kstart, kend
    yl_mwave_phys_ad (i) % nlevels   = klev

    yl_mwave_phys_ad (i) % p   => zphys_p   (i,:)
    yl_mwave_phys_ad (i) % ph  => zphys_ph  (i,:)
    yl_mwave_phys_ad (i) % t   => zphys_t   (i,:)
    yl_mwave_phys_ad (i) % q   => zphys_q   (i,:)
    yl_mwave_phys_ad (i) % rfl => zphys_rfl (i,:)
    yl_mwave_phys_ad (i) % sfl => zphys_sfl (i,:)
    yl_mwave_phys_ad (i) % cwc => zphys_cwc (i,:)
    yl_mwave_phys_ad (i) % cic => zphys_cic (i,:)
    yl_mwave_phys_ad (i) % cc  => zphys_cc  (i,:)

  enddo

  do jsat = 1, n_satsens
    do i = kstart, kend

      yl_mwave_rad (i,jsat) % screen(:) = 0

      !* Nominal channel's unified perturbation      
      yl_mwave_rad (i,jsat) % ad (1:kmaxchannels) = 0.0_JPRB       
      yl_mwave_rad (i,jsat) % ad (mwave_id(jsat) % nchan_sens) = 1.0_JPRB      
      yl_mwave_rad (i,jsat) % grad_norm (:)    = 0.0_JPRB    
      yl_mwave_rad (i,jsat) % screen(:)        = yd_mwave_rad (i,jsat) % screen(:)
      yl_mwave_rad (i,jsat) % pers1(:)         = yd_mwave_rad (i,jsat) % pers1(:)
      yl_mwave_rad (i,jsat) % zenith           = yd_mwave_rad (i,jsat) % zenith

      !* Initialization      
      yl_mwave_phys_ad (i) % u10 = 0.0_JPRB 
      yl_mwave_phys_ad (i) % v10 = 0.0_JPRB 
      yl_mwave_phys_ad (i) % t2m = 0.0_JPRB 
      yl_mwave_phys_ad (i) % q2m = 0.0_JPRB 
      yl_mwave_phys_ad (i) % tsk = 0.0_JPRB 
      yl_mwave_phys_ad (i) % lsm = 0.0_JPRB 

      yl_mwave_phys_ad (i) % p  (1:klev)   = 0.0_JPRB 
      yl_mwave_phys_ad (i) % ph (1:klev+1) = 0.0_JPRB 
      yl_mwave_phys_ad (i) % t  (1:klev)   = 0.0_JPRB 
      yl_mwave_phys_ad (i) % q  (1:klev)   = 0.0_JPRB 

      yl_mwave_phys_ad (i) % rfl (1:klev) = 0.0_JPRB 
      yl_mwave_phys_ad (i) % sfl (1:klev) = 0.0_JPRB 
      yl_mwave_phys_ad (i) % cwc (1:klev) = 0.0_JPRB 
      yl_mwave_phys_ad (i) % cic (1:klev) = 0.0_JPRB 
      yl_mwave_phys_ad (i) % cc  (1:klev) = 0.0_JPRB 

    enddo

    !* Call AD    
    call mwave_obsop_ad (kstart, kend, kproma, klev, kmaxchannels, yd_mwave_phys, yl_mwave_phys_ad, yl_mwave_rad) 

    !* Sum up norm of gradients scaled with state dependent 'variability'
    do i = kstart, kend
      if (mwave_ok_a1( yd_mwave_rad (i,jsat) % screen(1:mwave_id(jsat) % nchannels) )) then  

        !* Surface contribution
        yl_mwave_rad (i,jsat) % grad_norm (1) = yl_mwave_phys_ad (i) % u10 * yl_mwave_phys_ad (i) % u10 & 
         & * yd_mwave_phys    (i) % u10 * yd_mwave_phys    (i) % u10 * 0.04 &
         & + yl_mwave_phys_ad (i) % v10 * yl_mwave_phys_ad (i) % v10 &
         & * yd_mwave_phys    (i) % v10 * yd_mwave_phys    (i) % v10 * 0.04

        yl_mwave_rad (i,jsat) % grad_norm (2) = yl_mwave_phys_ad (i) % tsk * yl_mwave_phys_ad (i) % tsk &
         & * yd_mwave_phys    (i) % tsk * yd_mwave_phys    (i) % tsk

        !* Atmospheric contribution
        do ilev = 1, klev
          yl_mwave_rad (i,jsat) % grad_norm (3) = yl_mwave_rad (i,jsat) % grad_norm (3) &
           & + yl_mwave_phys_ad (i) % t   (ilev) * yl_mwave_phys_ad (i) % t   (ilev) &
           & * yd_mwave_phys    (i) % t   (ilev) * yd_mwave_phys    (i) % t   (ilev)    
          yl_mwave_rad (i,jsat) % grad_norm (4) = yl_mwave_rad (i,jsat) % grad_norm (4) &
           & + yl_mwave_phys_ad (i) % q   (ilev) * yl_mwave_phys_ad (i) % q   (ilev) &
           & * yd_mwave_phys    (i) % q   (ilev) * yd_mwave_phys    (i) % q   (ilev) * 0.010
          yl_mwave_rad (i,jsat) % grad_norm (5) = yl_mwave_rad (i,jsat) % grad_norm (5) &
           & + yl_mwave_phys_ad (i) % rfl (ilev) * yl_mwave_phys_ad (i) % rfl (ilev) &
           & * yd_mwave_phys    (i) % rfl (ilev) * yd_mwave_phys    (i) % rfl (ilev) * 0.025  
          yl_mwave_rad (i,jsat) % grad_norm (6) = yl_mwave_rad (i,jsat) % grad_norm (6) &
           & + yl_mwave_phys_ad (i) % sfl (ilev) * yl_mwave_phys_ad (i) % sfl (ilev) &
           & * yd_mwave_phys    (i) % sfl (ilev) * yd_mwave_phys    (i) % sfl (ilev) * 0.025  
          yl_mwave_rad (i,jsat) % grad_norm (7) = yl_mwave_rad (i,jsat) % grad_norm (7) &
           & + yl_mwave_phys_ad (i) % cwc (ilev) * yl_mwave_phys_ad (i) % cwc (ilev) &
           & * yd_mwave_phys    (i) % cwc (ilev) * yd_mwave_phys    (i) % cwc (ilev) * 0.025
          yl_mwave_rad (i,jsat) % grad_norm (8) = yl_mwave_rad (i,jsat) % grad_norm (8) &
           & + yl_mwave_phys_ad (i) % cic (ilev) * yl_mwave_phys_ad (i) % cic (ilev) &
           & * yd_mwave_phys    (i) % cic (ilev) * yd_mwave_phys    (i) % cic (ilev) * 0.025 
          yl_mwave_rad (i,jsat) % grad_norm (9) = yl_mwave_rad (i,jsat) % grad_norm (9) &
           & + yl_mwave_phys_ad (i) % cc  (ilev) * yl_mwave_phys_ad (i) % cc  (ilev) &
           & * yd_mwave_phys    (i) % cc  (ilev) * yd_mwave_phys    (i) % cc  (ilev) * 0.040
        enddo

        yd_mwave_rad (i,jsat) % grad_norm (:) = yl_mwave_rad (i,jsat) % grad_norm (:)
      endif
    enddo
  enddo
endif

if (lhook) call dr_hook ('MWAVE_SCREEN',1,zhook_handle)

end subroutine mwave_screen
