SUBROUTINE MOPRJAD(PVORI,PDIVI,PHGTI,PVORO,PDIVO,PHGTO,&
 & PCL,PBL,PUPL,PDIAGL,PLOWL,PFQ,PFP,&
 & KMO,KVMO,KM,KSPACE)  

!**** *MOPRJAD* - ROUTINE TO PROJECT ON ROSSBY OR GRAVITY MODES,
!                FOR A SINGLE VERTICAL MODE.ADJOINT

!     PURPOSE.   TO PROJECT VORTICITY,DIVERGENCE AND HEIGHT ON
!     --------   THE ROSSBY MODES OR THE GRAVITY MODES. KMO INDICATES
!                THE KIND OF PROJECTION REQUESTED IN HOUGH SPACE.
!                    KMO = 0 : NO PROJECTION REQUESTED
!                    KMO = 1 : PROJECT ON ROSSBY  MODES
!                    KMO = 2 : PROJECT ON GRAVITY MODES
!                    KMO = 3 : COMPUTE THE NMI INCREMENTS
!                              (PROJ. ON GRAV. MODES AND DIVIDE BY FREQ.)

!**   INTERFACE.
!     ----------
!        *CALL* *MOPRJAD(PVORI,PDIVI,PHGTI,PVORO,PDIVO,PHGTO,
!    1                   PCL,PBL,PUPL,PDIAGL,PLOWL,PFQ,PFP,
!    2                   KMO,KVMO,KM,KSPACE)

!        EXPLICIT ARGUMENTS :  PVORI- VORTICITY     \
!        --------------------  PDIVI- DIVERGENCE    \ - OUTPUT
!                              PHGTI- GEOPOTENTIAL  \
!                              PVORO- VORTICITY     \
!                              PDIVO- DIVERGENCE    \ - INPUT
!                              PHGTO- GEOPOTENTIAL  \
!                              PCL to - HELP ARRAYS FOR IMPLICIT NMI
!                               PFP
!                              KMO   - SWITCH FOR PROJECTION REQUESTED
!                              KVMO  - INDEX OF VERTICAL MODE
!                              KM    - INDEX OF WAVE NUMBER
!                              KSPACE- SWITCH FOR FINAL SPACE OF FIELDS
!                                 1 - Results in the spectral space
!                                     (results : PVOPO, PDIVO and PHGTO arrays)
!                                 2 - Results left in HOUGH space (diab. tend.)
!                                     (result  : the array FILT in YOMNMI)

!        IMPLICIT ARGUMENTS :  LRPIMP (YOMNMI) : .TRUE. if IMPLICIT NMI req.
!        --------------------                    (otherwise EXPLICIT)
!                              Other logical switches are in YOMNMI :

!                              NLEX   (YOMNMI) : IMPLICIT NMI above N=NLEX
!                              LTYPGR (YOMNMI) : .F. ==> ROSSBY, .T. ==> GRAVITY
!                              FREQ   (YOMNMI) : Frequencies of HOUGH modes
!                              FREQ2  (YOMNMI) : Frequencies of HOUGH modes
!                                 (LTYPxx, FILT, FREQ and FREQ2 are
!                                  arrays, see YOMNMI for the dimensions)

!     METHOD.   THE FIELDS ARE REORDERED AND NORMALIZED. WE GO INTO NORMAL
!     -------   MODE SPACE BY MULTIPLYING BY THE MATRIX CONSISTING OF THE
!               "STANDARD SPACE" EXPRESSED IN NORMAL MODES.THIS MATRIX HAS
!               BEEN PREVIOUSLY COMPUTED BY SUMODE. AFTER THAT A FILTER IS
!               APPLIED TO THE NORMAL MODES (EXPLICIT NMI CASE), OR IMPLICIT
!               MACHENHAUER INCREMENTS ARE COMPUTED (IMPLICIT NMI CASE).
!               FINALLY WE GO BACK TO " STANDARD SPACE" AND PUT THE FIELD
!               BACK IN "PVOR-PDIV-PHGT" FORM (WITH MIXED SCHEME IF NEEDED).

!     EXTERNALS.
!     ----------  SPEHOU3AD  - SPECTRAL TO HOUGH TRANSFORM.ADJOINT
!                 HOUSPE3AD  - HOUGH TO SPECTRAL TRANSFORM .ADJOINT
!                 SPEIMPAD   - SPECTRAL TO "IMPLICIT" TRANSFORM (AND REVERSE)
!                 MULTFAD    - MULTIPLY BY IMPLICIT OPERATOR "F"
!                 SGTSL      - CRAY ROUTINE TO SOLVE TRIDIAG. LINEAR SYSTEMS
!                              (A LINPACK SUBROUTINE)
!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        JEAN-NOEL THEPAUT *METEO FRANCE EERM/CRMD*
!        -- BASED ON THE ROUTINE SCMPRA WRITTEN BY
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL     : 88-05-25   (SCMPRA)
!        J.N THEPAUT  : 90-03-15   SCMPRA => MOPRA
!        F.  RABIER   : 91-09-03   Implicit NMI
!        L. ISAKSEN   : 95-01-11   Move MACROTASKING TO A HIGHER LEVEL
!        L. ISAKSEN   : 95-06-06   Change interface to PFQ and PFP
!        E.Andersson  : 96-07-08   Fujitsu vectorisation
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NCMAX    ,NCPEC2   ,NXMAX
USE YOMCST   , ONLY : ROMEGA
USE YOMCT0   , ONLY : NPRINTLEV
USE YOMLAP   , ONLY : NACM0
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMNMIA  , ONLY : NLEX     ,LRPIMP
USE YOMNMIB  , ONLY : LTYPGR   ,FREQ

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KM 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVORI(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVI(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHGTI(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVORO(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVO(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHGTO(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCL(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBL(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUPL(NCPEC2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIAGL(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLOWL(NCPEC2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFQ(KM:NCMAX) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFP(KM:NCMAX) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVMO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSPACE 
!***************************************************************
!**   LARGE IMPLICIT TRUNCATION ON THE REAL GEOGRAPHIC SPHERE **
!***************************************************************
REAL(KIND=JPRB) :: ZVORM(2*(NCMAX+1-KM)),ZDIVM(2*(NCMAX+1-KM)),ZHGTM(2*(NCMAX+1-KM))
REAL(KIND=JPRB) :: ZVOR(2*(NCMAX+1-KM)),ZDIV(2*(NCMAX+1-KM)),ZHGT(2*(NCMAX+1-KM))
REAL(KIND=JPRB) :: ZLOW(2*(NCMAX+1-KM)+1),ZDIAG(2*(NCMAX+1-KM)),&
 & ZUP(0:2*(NCMAX+1-KM)),ZW(2*(NCMAX+1-KM))  
!***************************************************************
!**   SMALL EXPLICIT TRUNCATION ON THE REAL GEOGRAPHIC SPHERE **
!***************************************************************
REAL(KIND=JPRB) :: ZVORO(2*(NCMAX+1-KM)),ZDIVO(2*(NCMAX+1-KM)),ZHGTO(2*(NCMAX+1-KM))
REAL(KIND=JPRB) :: ZF1(6*(NXMAX+1)),ZF2(6*(NXMAX+1))

INTEGER(KIND=JPIM) :: ICMVAL, ICPEC, IJ, IMPVAL, INFO, IREVAL,&
 & IS, IXMVAL, IXPEC, J, JJ, JN, JSPEC  

REAL(KIND=JPRB) :: ZOME2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "houspe3ad.intfb.h"
#include "multfad.intfb.h"
#include "spehou3ad.intfb.h"
#include "speimpad.intfb.h"

!     ------------------------------------------------------------------

!*       0.    RESET LOCAL ARRAYS TO ZERO.
!              ---------------------------

IF (LHOOK) CALL DR_HOOK('MOPRJAD',0,ZHOOK_HANDLE)
IJ=MIN(1,KM)
ICPEC=(NCMAX+1-KM)*(IJ+1)
ICMVAL=NACM0(KM)-1
IXPEC=NXMAX+1-KM

INFO=0
DO JSPEC=1,ICPEC
  ZVORO(JSPEC)=0.0_JPRB
  ZDIVO(JSPEC)=0.0_JPRB
  ZHGTO(JSPEC)=0.0_JPRB
ENDDO
DO JSPEC=1,6*IXPEC
  ZF1(JSPEC)=0.0_JPRB
  ZF2(JSPEC)=0.0_JPRB
ENDDO
DO JSPEC=1,ICPEC
  ZVORM(JSPEC)=0.0_JPRB
  ZDIVM(JSPEC)=0.0_JPRB
  ZHGTM(JSPEC)=0.0_JPRB
  ZW   (JSPEC)=0.0_JPRB
  ZVOR (JSPEC)=0.0_JPRB
  ZDIV (JSPEC)=0.0_JPRB
  ZHGT (JSPEC)=0.0_JPRB
ENDDO

!     ------------------------------------------------------------------

!*    3.  MIXING OF IMPLICIT (N > NLEX) AND  EXPLICIT (N = or < NLEX).
!         ------------------------------------------------------------

!     --------------------
IF(KSPACE /= 2) THEN
!     --------------------
  IF(LRPIMP) THEN
    IF(NLEX /= 0) THEN
!*          PVORO, PDIVO and PHGTO not changed for JJ>ILEFRE (i.e. for N>NLEX),
!*          but replaced by EXPLICIT increments for N<NLEX.
!           WRITE(UNIT=NULOUT,FMT='(''  IMPLICIT AND EXPLICIT PARTS'',
!    1      '' ARE MIXED, CUT-OFF='',I4,
!    2      '',   VERTICAL MODE NUMBER :'', I4 )') NLEX,KVMO

      IF (KM == 0) THEN
        DO JJ=1,NLEX+1
          ZHGTO(JJ) = ZHGTO(JJ) + PHGTO(JJ)
          PHGTO(JJ) = 0.0_JPRB
          ZDIVO(JJ) = ZDIVO(JJ) + PDIVO(JJ)
          PDIVO(JJ) = 0.0_JPRB
          ZVORO(JJ) = ZVORO(JJ) + PVORO(JJ)
          PVORO(JJ) = 0.0_JPRB
        ENDDO
      ELSE
!OCL NOVREC
        DO JJ=1,NLEX+1-KM
          IJ=JJ+NCMAX+1-KM
          ZHGTO(JJ) = ZHGTO(JJ) + PHGTO(JJ)
          PHGTO(JJ) = 0.0_JPRB
          ZHGTO(IJ) = ZHGTO(IJ) + PHGTO(IJ)
          PHGTO(IJ) = 0.0_JPRB
          ZDIVO(JJ) = ZDIVO(JJ) + PDIVO(JJ)
          PDIVO(JJ) = 0.0_JPRB
          ZDIVO(IJ) = ZDIVO(IJ) + PDIVO(IJ)
          PDIVO(IJ) = 0.0_JPRB
          ZVORO(JJ) = ZVORO(JJ) + PVORO(JJ)
          PVORO(JJ) = 0.0_JPRB
          ZVORO(IJ) = ZVORO(IJ) + PVORO(IJ)
          PVORO(IJ) = 0.0_JPRB
        ENDDO
      ENDIF

    ENDIF
!*       case NLEX=0 : PVORO, PDIVO and PHGTO not changed (pure IMPLICIT scheme)
  ELSE
!*       PVORO, PDIVO and PHGTO replaced by pure EXPLICIT increments.
    DO JJ=1,ICPEC
      ZHGTO(JJ) = ZHGTO(JJ) + PHGTO(JJ)
      PHGTO(JJ) = 0.0_JPRB
      ZDIVO(JJ) = ZDIVO(JJ) + PDIVO(JJ)
      PDIVO(JJ) = 0.0_JPRB
      ZVORO(JJ) = ZVORO(JJ) + PVORO(JJ)
      PVORO(JJ) = 0.0_JPRB
    ENDDO
  ENDIF

!     -----
ENDIF
!     -----

!     ------------------------------------------------------------------

!     2.  IMPLICIT PROJECTIONS.
!         ---------------------

!*    *** BEGINING OF THE IMPLICIT PART ***
IF(LRPIMP) THEN

  IJ=MIN(1,KM)
  IF (KM == 0) THEN
    IS=2
  ELSE
    IS=1
  ENDIF

!*       2.9   DIVISION BY "i" AND DIVISION BY "2*OMEGA"
!*       (except for JM=0 : divided by "+2*OMEGA" for VOR and HGT)
!*                          divided by "-2*OMEGA" for    DIV     )
!*       (in this section ZVORM, ZDIVM and ZHGTM are used as worked arrays)
  IF(KMO == 3) THEN
!*       computation of opposite of increments to
!*       update state variables in NNMI3 :
    ZOME2 = 1.0_JPRB/(2.0_JPRB*ROMEGA)
    DO JJ=1,ICPEC
      ZHGTM(JJ) = ZHGTM(JJ) - ZOME2*PHGTO(JJ)
      PHGTO(JJ) = 0.0_JPRB
      ZDIVM(JJ) = ZDIVM(JJ) - ZOME2*PDIVO(JJ)
      PDIVO(JJ) = 0.0_JPRB
      ZVORM(JJ) = ZVORM(JJ) - ZOME2*PVORO(JJ)
      PVORO(JJ) = 0.0_JPRB
    ENDDO
    IF (KM == 0) THEN
      DO JN=0,NCMAX
        IREVAL=JN+1
        PHGTO(IREVAL) = PHGTO(IREVAL) + ZHGTM(IREVAL)
        ZHGTM(IREVAL) = 0.0_JPRB
        PDIVO(IREVAL) = PDIVO(IREVAL) - ZDIVM(IREVAL)
        ZDIVM(IREVAL) = 0.0_JPRB
        PVORO(IREVAL) = PVORO(IREVAL) + ZVORM(IREVAL)
        ZVORM(IREVAL) = 0.0_JPRB
      ENDDO
    ELSE
!OCL NOVREC
      DO JN=KM,NCMAX
        IREVAL=JN+1-KM
        IMPVAL=NCMAX+1-KM+IREVAL
        PHGTO(IREVAL)= PHGTO(IREVAL) + ZHGTM(IMPVAL)
        ZHGTM(IMPVAL)= 0.0_JPRB
        PHGTO(IMPVAL)= PHGTO(IMPVAL) - ZHGTM(IREVAL)
        ZHGTM(IREVAL)= 0.0_JPRB
        PDIVO(IREVAL)= PDIVO(IREVAL) + ZDIVM(IMPVAL)
        ZDIVM(IMPVAL)= 0.0_JPRB
        PDIVO(IMPVAL)= PDIVO(IMPVAL) - ZDIVM(IREVAL)
        ZDIVM(IREVAL)= 0.0_JPRB
        PVORO(IREVAL)= PVORO(IREVAL) + ZVORM(IMPVAL)
        ZVORM(IMPVAL)= 0.0_JPRB
        PVORO(IMPVAL)= PVORO(IMPVAL) - ZVORM(IREVAL)
        ZVORM(IREVAL)= 0.0_JPRB
      ENDDO
    ENDIF
  ENDIF

!*    -- 2.8  TRANSFORM FROM "IMPLICIT NMI" SPACE TO SPECTRAL SPACE.
  CALL SPEIMPAD(PVORO,PDIVO,PHGTO,ZVOR,ZDIV,ZHGT,KVMO,KM,-1)
!*    -- 2.4   PROJECTION ON ROSSBY MODES
  IF (KMO == 1) THEN
!        WRITE(UNIT=NULOUT,FMT='('' PROJECTION ON ROSSBY MODES ''
!    1   ,'' (IMPLICIT CASE)''
!    2   ,'', VERTICAL MODE NUMBER :'', I4 )') KVMO
    DO JJ=1,ICPEC
      ZHGTM(JJ) = ZHGTM(JJ) + ZHGT(JJ)
      ZHGT (JJ) = -ZHGT(JJ)
      ZDIV(JJ) = 0.0_JPRB
      ZVORM(JJ) = ZVORM(JJ) + ZVOR(JJ)
      ZVOR (JJ) = -ZVOR(JJ)
    ENDDO

!*    -- 2.5   PROJECTION ON GRAVITY MODES
  ELSEIF (KMO == 2) THEN
!        WRITE(UNIT=NULOUT,FMT='('' PROJECTION ON GRAVITY MODES ''
!    1   ,'' (IMPLICIT CASE)''
!    2   ,'', VERTICAL MODE NUMBER :'', I4 )') KVMO
    IF (KM == 0) ZDIV(1) = 0.0_JPRB
    DO JJ=IS,ICPEC
!        ZVOR(JJ) : computed in loop 232
      ZDIVM(JJ) = ZDIVM(JJ) + ZDIV(JJ)
      ZDIV (JJ) = 0.0_JPRB
!        ZHGT(JJ) : computed in loop 232
    ENDDO

!*    -- 2.6   COMPUTATION OF THE WORK ARRAY " ZW = C-1(INC.HGT) " :
!*            " (F*F + C*C)(ZW) = - B(INC.DIV) - TEND.DIV "
  ELSEIF (KMO == 3) THEN
    IF (KM == 1.AND.NPRINTLEV >= 1) WRITE(UNIT=NULOUT,FMT='(&
     & '' PROJ. ON GRAVITY MODES, DIVISION BY -i*FREQUENCY'',&
     & '' (IMPLICIT CASE), VERTICAL MODE NUMBER'',I4)') KVMO  

!*    -- 2.7   COMPUTATION OF IMPLICIT INCREMENTS :
!*             " INC.VOR = F(ZW) "  and  " INC.HGT = C(ZW) ".
    DO JJ=1,ICPEC
!        ZDIV(JJ) : computed in loop 224
      ZW  (JJ) = ZW  (JJ) +PCL(JJ)*ZHGT(JJ)
      ZHGT(JJ) = 0.0_JPRB
    ENDDO
    CALL MULTFAD(ZW,PFP,PFQ,ZVOR,KM)
    IF (KM == 0) ZW(1) = 0.0_JPRB
    DO JJ=1,ICPEC
      ZUP  (JJ) = PUPL  (JJ+ICMVAL)
      ZDIAG(JJ) = PDIAGL(JJ)
      ZLOW (JJ) = PLOWL (JJ+ICMVAL)
    ENDDO
    IF (KM > 0) ZUP(0) = 0.0_JPRB
    ZLOW(ICPEC+1)=0.0_JPRB
    CALL SGTSL(ICPEC-IS+1,ZUP(IS-1),ZDIAG(IS),ZLOW(IS+1),ZW(IS),INFO)
!        (before SGTSL : ZW is the right hand side of tridiag. system)
!        (after  SGTSL : ZW is the worked array " C-1(INC.HGT) ")
    IF (INFO /= 0) THEN
      WRITE(UNIT=NULERR,FMT='(''ERROR IN SGTSL (TRIDIAG. SYSTEM 2)'')')
      WRITE(UNIT=NULERR,FMT='('' INFO ='',I6)') INFO
      CALL ABOR1('PBL SOLVE TRIDIAG. SYSTEM 2 (SGTSL)')
    ENDIF

    DO JJ=IS,ICPEC
      ZDIV (JJ) = ZDIV (JJ) - PBL(JJ)* ZW(JJ)
      ZDIVM(JJ) = ZDIVM(JJ) -ZW(JJ)
      ZW   (JJ) = 0.0_JPRB
    ENDDO
  ENDIF

!*    -- 2.3   COMPUTE IMPLICIT GRAVITY COMPONENTS OF TENDENCIES FOR :
!*                 VORTICITY     and     GEOPOTENTIAL
!*             " - F(INC.DIV) "  and   " - C(INC.DIV) "
  IF (KMO == 1.OR.KMO == 2) THEN
    DO JJ=1,ICPEC
      ZDIV(JJ) = ZDIV(JJ) - PCL(JJ)*ZHGT(JJ)
      ZHGT(JJ) = 0.0_JPRB
      ZVOR(JJ) = - ZVOR(JJ)
    ENDDO
    CALL MULTFAD(ZDIV,PFP,PFQ,ZVOR,KM)
  ENDIF

!*    -- 2.1   NO PROJECTION.
  IF(KMO == 0)THEN
    IF (KM == 1.AND.NPRINTLEV >= 1) WRITE(UNIT=NULOUT,FMT='(&
     & '' NO PROJECTION REQUESTED (IMPLICIT CASE)'',&
     & '', VERTICAL MODE NUMBER'',I4)') KVMO  
    DO JJ=1,ICPEC
      ZHGTM(JJ) = ZHGTM(JJ) + ZHGT(JJ)
      ZHGT (JJ) = 0.0_JPRB
      ZDIVM(JJ) = ZDIVM(JJ) + ZDIV(JJ)
      ZDIV (JJ) = 0.0_JPRB
      ZVORM(JJ) = ZVORM(JJ) + ZVOR(JJ)
      ZVOR (JJ) = 0.0_JPRB
    ENDDO
  ELSE

!*    -- 2.2   COMPUTE IMPLICIT INCREMENT OF DIVERGENCE "INC.DIV" :
!*            "(F*F + C*C)(INC.DIV) = - F(TEND.VOR) - C(TEND.HGT) "
!        (KMO = 1, 2 or 3)
    INFO=0
    DO JJ=1,ICPEC
      ZUP  (JJ) = PUPL  (JJ+ICMVAL)
      ZDIAG(JJ) = PDIAGL(JJ)
      ZLOW (JJ) = PLOWL (JJ+ICMVAL)
    ENDDO
    IF (KM > 0) ZUP(0)=0.0_JPRB
    ZLOW(ICPEC+1)=0.0_JPRB
    CALL SGTSL(ICPEC-IS+1,ZUP(IS-1),ZDIAG(IS),ZLOW(IS+1),ZDIV(IS),INFO)
!        (before SGTSL : ZDIV is the right hand side of tridiag. system)
!        (after  SGTSL : ZDIV is the array of increments of divergence)
    IF (INFO /= 0) THEN
      WRITE(UNIT=NULERR,FMT='(''ERROR IN SGTSL (TRIDIAG. SYSTEM 1)'')')
      WRITE(UNIT=NULERR,FMT='('' INFO ='',I6)') INFO
      CALL ABOR1('PBL SOLVE TRIDIAG. SYSTEM 1 (SGTSL)')
    ENDIF
    DO JJ=IS,ICPEC
      ZHGTM(JJ) =   ZHGTM(JJ) - PCL(JJ)*ZDIV(JJ)
      ZDIV (JJ) = - ZDIV(JJ)
    ENDDO
    CALL MULTFAD(ZVORM,PFP,PFQ,ZDIV,KM)

  ENDIF

  CALL SPEIMPAD(PVORI,PDIVI,PHGTI,ZVORM,ZDIVM,ZHGTM,KVMO,KM,+1)
!*    *** END OF IMPLICIT PART ***
ENDIF

!*    1.  EXPLICIT PROJECTIONS.
!         ---------------------

!*    *** BEGINING OF THE EXPLICIT PART ***
IF ( ((.NOT.LRPIMP) .OR. (LRPIMP.AND.NLEX /= 0)).AND.KM <= NXMAX) THEN

  IXMVAL=(KM*(NXMAX+1)-KM*(KM-1)/2)*6

!*    -- 1.7  TRANSFORM FROM "HOUGH" SPACE TO SPECTRAL SPACE.
  IF (KSPACE /= 2) THEN
    CALL HOUSPE3AD(ZF1,ZVORO,ZDIVO,ZHGTO,KVMO,KM)
  ENDIF

!*    -- 1.3   NO PROJECTION.
  IF(KMO == 0)THEN
    IF(KM == 1.AND.NPRINTLEV >= 1) WRITE(UNIT=NULOUT,FMT='(&
     & '' NO PROJECTION REQUESTED (EXPLICIT CASE)'',&
     & '', VERTICAL MODE NUMBER'',I4)') KVMO  
    DO J=1,6*IXPEC
      ZF2(J)=ZF2(J)+ZF1(J)
      ZF1(J)=0.0_JPRB
    ENDDO

!*    -- 1.4   PROJECTION ON ROSSBY MODES.
  ELSEIF(KMO == 1)THEN
!        WRITE(UNIT=NULOUT,FMT='('' PROJECTION ON ROSSBY MODES ''
!    1   ,'' (EXPLICIT CASE)''
!    2   ,'', VERTICAL MODE NUMBER :'', I4 )') KVMO
    DO J=1,6*IXPEC
      IF ( LTYPGR(IXMVAL+J,KVMO) ) THEN
        ZF1(J) = 0.0_JPRB
      ELSE
        ZF2(J)=ZF2(J)+ZF1(J)
        ZF1(J)=0.0_JPRB
      ENDIF
    ENDDO

!*    -- 1.5   PROJECTION ON GRAVITY MODES.

  ELSEIF(KMO == 2) THEN

!*       --> 1.5.3 PROJECTION ONTO THE GRAVITY MODES SUBSET OF THE TENDENCIES
!        WRITE(UNIT=NULOUT,FMT='('' PROJECTION ON GRAVITY MODES ''
!    1   ,'' (EXPLICIT CASE)''
!    2   ,'', VERTICAL MODE NUMBER :'', I4 )') KVMO
    DO J=1,6*IXPEC
      IF ( LTYPGR(IXMVAL+J,KVMO) ) THEN
        ZF2(J)=ZF2(J)+ZF1(J)
        ZF1(J)=0.0_JPRB
      ELSE
        ZF1(J) = 0.0_JPRB
      ENDIF
    ENDDO

!*    -- 1.6 PROJECTION ON GRAVITY MODES, DIVISION BY
!*           "2*OMEGA*FREQUENCY" AND MULTIPLICATION BY "i".
  ELSEIF(KMO == 3) THEN

!*       --> 1.6.3 RETAIN ONLY GRAVITY MODES AND DIVIDE BY "-i*FREQUENCY"
!*                 (Machenhauer step of the NMI sheme)
    IF (KM == 1.AND.NPRINTLEV >= 1) WRITE(UNIT=NULOUT,FMT='(&
     & '' PROJ. ON GRAVITY MODES, DIVISION BY -i*FREQUENCY'',&
     & '' (EXPLICIT CASE), VERTICAL MODE NUMBER'',I4)') KVMO  

    ZOME2=2.0_JPRB*ROMEGA
!DIR$ IVDEP
!OCL NOVREC
    DO J=1,3*IXPEC
      IF ( LTYPGR(IXMVAL+2*J,KVMO) ) THEN
        ZF2(2*(J-1)+1) = ZF2(2*(J-1)+1)&
         & + ZF1(2*(J-1)+2)/ZOME2/FREQ(J+IXMVAL/2,KVMO)  
        ZF1(2*(J-1)+2) = 0.0_JPRB
        ZF2(2*(J-1)+2) = ZF2(2*(J-1)+2)&
         & - ZF1(2*(J-1)+1)/ZOME2/FREQ(J+IXMVAL/2,KVMO)  
        ZF1(2*(J-1)+1) = 0.0_JPRB
      ELSE
        ZF1(2*(J-1)+1) = 0.0_JPRB
        ZF1(2*(J-1)+2) = 0.0_JPRB
      ENDIF
    ENDDO

!*    *** END OF "KMO=0,1,2 or 3" ***
  ENDIF

!*    -- 1.0  TRANSFORM FROM SPECTRAL SPACE TO "HOUGH" SPACE
  CALL SPEHOU3AD(PVORI,PDIVI,PHGTI,ZF2,KVMO,KM)

!*    *** END OF EXPLICIT PART ***
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('MOPRJAD',1,ZHOOK_HANDLE)
END SUBROUTINE MOPRJAD
