SUBROUTINE NNMI2AD(KTYPE)

!**** *NNMI2AD* - Adjoint of 2D NMI for S.W.M.

!     Purpose.   To perform the adjoint of non-linear normal mode
!     --------   initialization for a shallow water model.

!**   Interface.
!     ----------
!        *CALL* *NNMI2AD(KTYPE)

!        Explicit arguments :  KTYPE=1 : a 2D linear NMI is required
!        --------------------           (one step with  only MOPRJM(..,KMO=1,..))
!                              KTYPE=2 : a 2D non-linear NMI is required
!                                       (NITNMI step of NNMI sheme)

!        Implicit arguments :  The spectral arrays of the model
!        --------------------

!     Method.   The tendencies are computed by an appropiate call to STEPO.
!     -------   The tendencies are then changed into normal mode space
!               and projected onto the gravity modes divided by the
!               frequencies. We go back into spectral space and deduct the
!               projected fields from the state variables. The process is
!               repeated NITNMI times.

!     Externals.  STEPOAD  - compute the tendencies
!     ----------  MOPRJMAD - project on gravity modes
!                 TRAGEOAD - transformation between the two spheres
!                          (real to transformed, or reverse path)
!                 SPNORM   - compute norms in spectral space (one level)
!                 MVTENDAD - COPYING TENDENCIES TO ARGUMENTS OF MO3DPRJ
!                 SUALLTT  - ALLOCATE SPACE FOR SPECTRAL TENDENCIES
!                 DEALLTT  - DESALLOCATE SPACE FOR SPECTRAL TENDENCIES
!                 RDPINMI  - READ HELP ARRAYS FOR IMPLICIT NMI

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 88-05-25
!        F. Rabier  : 91-08-30 : SCMPRJ --> MOPRJ, and the new option KTYPE.
!        D. Giard   : 93-04-05 : NNMI with stretching (see NNMI3, MO3DPRJ)
!        D. Giard   : 93-04-08 : ISPORTS=NSTTYP-1
!        D. Giard   : 94-11-24 : previous opening of matrixes or not
!        L.Isaksen  : 95-02-01 : new interface to MOPRJ
!        L.Isaksen  : 95-06-06 : Reordering of spectral arrays
!        D. Giard   : 95-09-06 : corrections
!        M.Hamrud   : 95-10-30 : Changed CLCONF def.
!        C.Temperton: 98-03-11 : bugfix             
!        R. El Khatib : 01-08-07 Pruning options
!        G. Desroziers : 02-09-16 also treat Aladin mean wind trajectory
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!***************************************************************************

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSMAX    ,NSPEC2   ,NCMAX    ,NCPEC    ,&
 & NCPEC2   ,NTMAX    ,NTPEC2     
USE YOMSP    , ONLY : SPVOR    ,SPDIV    ,SPSP,SP5A3, SP5A2
USE YOMLAP   , ONLY : NASM0    ,NCVALUE  ,&
 & NTVALUE  ,NSTREL   ,RCAPDI   ,RCAPIN   ,&
 & RTAPDI   ,RTAPIN  
USE YOMNMIA  , ONLY : NLEX     ,NITNMI   ,NVMOD    ,NVMODPP  ,&
 & LRPIMP   ,LTRANS   ,LSPNDG  
USE YOMLUN   , ONLY : NULOUT   
USE YOMGEM   , ONLY : NSTTYP   
USE YOMCT0   , ONLY : NSTART
USE YOMCT1   , ONLY : N1SDI
USE YOMCT3   , ONLY : NSTEP
USE YOMMUL   , ONLY : N
USE YOMVAR   , ONLY : LTRREF
USE TRAJECTORY_MOD, ONLY: LTRAJGP, GET_TRAJ_SPEC

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPE 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZDUM(1,1)

!*******************************************************
!**  ARRAYS ON THE T"NSMAX" OF THE TRANSFORMED SPHERE **
!*******************************************************
REAL(KIND=JPRB) :: ZVOR(1,NSPEC2),ZDIV(1,NSPEC2),ZSP(NSPEC2)
!*******************************************************
!**  ARRAYS ON THE T"NTMAX" OF THE TRANSFORMED SPHERE **
!*******************************************************
REAL(KIND=JPRB) :: ZVORDI(1,NTPEC2),ZDIVDI(1,NTPEC2),ZSPDI(NTPEC2)
REAL(KIND=JPRB) :: ZVDTT(NTPEC2,1,3)
!***********************************************************
!**  ARRAYS ON THE T"NCMAX" OF THE REAL GEOGRAPHIC SPHERE **
!***********************************************************
REAL(KIND=JPRB) :: ZVDTR(NCPEC2,1,3)
REAL(KIND=JPRB) :: ZC(NCPEC2),ZDIAG(NCPEC2),ZFP(NCPEC),ZFQ(NCPEC)
REAL(KIND=JPRB) :: ZB(NCPEC2),ZLOW(NCPEC2),ZUP(NCPEC2)

CHARACTER (LEN = 9) ::  CLCONF,CLCONA

INTEGER(KIND=JPIM) :: I, IDIRTR, IFIELD, IFLDSUR, IK, IMO, IMODES,&
 & ISPACE, ISPORTS, ITERMAX, ITYPFLD, J, JITER, &
 & JV  

LOGICAL :: LLTRAGEO
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "moprjmad.intfb.h"
#include "mvtendad.intfb.h"
#include "rdpinmi.intfb.h"
#include "spnorm.intfb.h"
#include "stepo.intfb.h"
#include "stepoad.intfb.h"
#include "sualltt.intfb.h"
#include "dealltt.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('NNMI2AD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       0.    PREAMBLE.
!              ---------

!*      0.0    SET VERTICAL MODE INDEX
IK= 1

!*      0.1    WRITE THE KIND OF JOB REQUESTED

IF (KTYPE == 1) THEN
  WRITE(NULOUT,'(/,'' LINEAR 2D NORMAL MODE INITIALIZATION'')')
ELSEIF (KTYPE == 2) THEN
  WRITE(NULOUT,'(/,'' NONLINEAR 2D NORMAL MODE INITIALIZATION'')')
ELSE
  WRITE(NULOUT,'('' KTYPE MUST 1 OR 2.  KTYPE ='',I5)') KTYPE
  CALL ABOR1(' WRONG VALUE FOR KTYPE IN NNMI2 ')
ENDIF
IF (LRPIMP) THEN
  IF (NLEX == 0) THEN
    WRITE(NULOUT,'('' PURE IMPLICIT SCHEME  '')')
  ELSE
    WRITE(NULOUT,'('' MIXED (EXPL.-IMPL.) SCHEME  '')')
  ENDIF
ELSE
  WRITE(NULOUT,'('' PURE EXPLICIT SCHEME  '')')
ENDIF

!*       0.2   SET UP CONFIGURATION FOR STEPO AND STEPOAD
IF (KTYPE == 2) THEN
  CLCONA(1:9)='0AAE00A?0'
  IF (NTMAX == NSMAX) THEN
    CLCONA(8:8)='A'
  ELSE
    CLCONA(8:8)='T'
  ENDIF
ENDIF

!*      0.3    SPECTRAL DIAGNOSTICS.

IF ((N1SDI == 1) .AND. LSPNDG) THEN
  IF (KTYPE == 2) THEN
    WRITE(NULOUT,'('' SPECTRUM BEFORE NNMI ITERATION'')')
  ELSEIF (KTYPE == 1) THEN
    WRITE(NULOUT,'('' SPECTRUM BEFORE LINEAR NMI'')')
  ENDIF
  CALL SPNORM(0)
ENDIF

!*      0.4    READ OF HELP ARRAYS FOR IMPLICIT NMI SCHEME

IF (LRPIMP) THEN
  IMODES = 1
  CALL RDPINMI(IMODES,ZFP,ZFQ,ZB,ZLOW,ZUP,ZC,ZDIAG)
ENDIF

!*      0.5    INITIALIZE LOGICAL SWITCHES FOR HORIZONTAL
!              TRANSFORMATIONS BETWEEN THE TWO SPHERES :

! NSTTYP=1 ==> pole of stretching and pole of colloc. grid at the northern 
!              pole of the real sphere. (YOMGEM)
! LTRANS=.TRUE. if NMI required on the transformed sphere.(YOMNMI) 
! The subroutine TRAGEO is likely to be used if LLTRAGEO=TRUE :
! if NSMAX.NE.NCMAX (RSTRET>1.) or if NSTTYP.NE.1

LLTRAGEO = (NSMAX /= NCMAX) .OR. (NSTTYP /= 1)

!*      0.6    INITIALIZE THE REAL NUMBER OF ITERATION : ITERMAX

IF (KTYPE == 1) ITERMAX = 1
IF (KTYPE == 2) ITERMAX = NITNMI

!*      0.7    INITIALIZE LOCAL ARRAYS

DO J=1,NSPEC2
  ZVOR(IK,J)= 0.0_JPRB
  ZDIV(IK,J)= 0.0_JPRB
  ZSP (J)= 0.0_JPRB
ENDDO

DO J=1,NTPEC2
  ZVORDI(IK,J)= 0.0_JPRB
  ZDIVDI(IK,J)= 0.0_JPRB
  ZSPDI (J)   = 0.0_JPRB
  DO JV=1,3
    ZVDTT(J,IK,JV)= 0.0_JPRB
  ENDDO
ENDDO

DO JV=1,3
  DO J=1,NCPEC2
    ZVDTR(J,IK,JV)= 0.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!     *******************************
!     ** MAIN LOOP OVER ITERATIONS **

ITERATION_LOOP: DO JITER=ITERMAX,1,-1

!     ------------------------------------------------------------------

!*       5.    COMPUTE TRAJECTORY.
!              -------------------

  IF (KTYPE == 2) THEN
    NSTEP= NSTART-ITERMAX-1+JITER
    WRITE(NULOUT,'('' NON-LINEAR NMI, ITERATION='',I2)') JITER
    IF (.NOT.LTRAJGP) THEN
      IF (LTRREF) THEN
        CLCONF(1:9)='BE0000000'
      ELSE
        CALL GSTATS(15,0)
        CALL get_traj_spec(SP5A3,SP5A2,ZDUM,KSTEP=nstep)
        CALL GSTATS(15,1)
        CLCONF(1:9)='0E0000000'
      ENDIF
      CALL STEPO(CLCONF)
    ENDIF
  ENDIF

!     ------------------------------------------------------------------

!*       4.    UPDATE STATE VARIABLES.
!              -----------------------

  CALL SPNORM(0)
  SPVOR(IK,NASM0(0))= 0.0_JPRB
  SPDIV(IK,NASM0(0))= 0.0_JPRB
  IF (NTMAX > NSMAX) THEN
    DO J=1,NTPEC2
      ZVORDI(IK,J)= 0.0_JPRB
      ZDIVDI(IK,J)= 0.0_JPRB
      ZSPDI (J)   = 0.0_JPRB
    ENDDO
  ENDIF

!*      4.1   STATE VARIABLES ARE UPDATED WITH THE NNMI INCREMENTS:
!*            Zxx - ZxxDI --> SPxx  

  IF (KTYPE == 2) THEN
    IF (NSMAX < NTMAX) THEN
      DO J=1,NSPEC2
        I= NSTREL(J)
        ZVORDI(IK,I)= ZVORDI(IK,I) - SPVOR(IK,J)
        ZDIVDI(IK,I)= ZDIVDI(IK,I) - SPDIV(IK,J)
        ZSPDI (I)   = ZSPDI (I)    - SPSP (J)
        ZVOR(IK,J)= ZVOR(IK,J) + SPVOR(IK,J)
        ZDIV(IK,J)= ZDIV(IK,J) + SPDIV(IK,J)
        ZSP (J)   = ZSP (J)    + SPSP (J)
        SPVOR(IK,J)= 0.0_JPRB
        SPDIV(IK,J)= 0.0_JPRB
        SPSP (J)   = 0.0_JPRB
      ENDDO
    ELSE
      ZVORDI(IK,:)= ZVORDI(IK,:) - SPVOR(IK,:)
      ZDIVDI(IK,:)= ZDIVDI(IK,:) - SPDIV(IK,:)
      ZSPDI (:)   = ZSPDI (:)    - SPSP (:)
      ZVOR(IK,:)= ZVOR(IK,:) + SPVOR(IK,:)
      ZDIV(IK,:)= ZDIV(IK,:) + SPDIV(IK,:)
      ZSP (:)   = ZSP (:)    + SPSP (:)
      SPVOR(IK,:)= 0.0_JPRB
      SPDIV(IK,:)= 0.0_JPRB
      SPSP (:)   = 0.0_JPRB
    ENDIF
  ENDIF

!*      4.2   STATE VARIABLES ARE UPDATED WITH THE LNMI VALUES:
!*            ZxxDI --> SPxx        

  IF (KTYPE == 1) THEN
    IF (NSMAX < NTMAX) THEN
      DO J=1,NSPEC2
        I= NSTREL(J)
        ZVORDI(IK,I)= ZVORDI(IK,I) + SPVOR(IK,J)
        ZDIVDI(IK,I)= ZDIVDI(IK,I) + SPDIV(IK,J)
        ZSPDI (I)   = ZSPDI (I)    + SPSP (J)
        SPVOR (IK,J)=0.0_JPRB
        SPDIV (IK,J)=0.0_JPRB
        SPSP (J)=0.0_JPRB
      ENDDO
    ELSE
      ZVORDI(IK,:)= ZVORDI(IK,:) + SPVOR(IK,:)
      ZDIVDI(IK,:)= ZDIVDI(IK,:) + SPDIV(IK,:)
      ZSPDI (:)   = ZSPDI (:)    + SPSP (:)
      SPVOR (IK,:)=0.0_JPRB
      SPDIV (IK,:)=0.0_JPRB
      SPSP (:)=0.0_JPRB
    ENDIF
  ENDIF

!     ------------------------------------------------------------------

!*    3.    COME BACK TO THE CORRECT SPHERE.
!           --------------------------------

!*      3.2   FROM "GEOGRAPHIC" TO "TRANSFORMED" SPHERE:
!*            ZVDTR --> ZVDTT --> ZxxDI

  IF ((.NOT.LTRANS).AND.LLTRAGEO) THEN

! Goto the transformed sphere : ZVDTR --> ZVDTR --> ZxxDI

    WRITE(NULOUT,'('' COME BACK TO THE TRANSFORMED SPHERE :'' )')

! Vorticity and divergence (transformed sphere) are obtained using the
! stream function and the velocity potential :  ZVDTT --> ZxxDI

    WRITE(NULOUT,'(''   COMPUTE VORTICITY AND DIVERGENCE'',&
     & '' ON THE TRANSFORMED SPHERE'' )')  
    DO J=1,NTPEC2
      ZVDTT(J,IK,1)= ZVDTT(J,IK,1)+RTAPDI(NTVALUE(J))*ZVORDI(IK,J)
      ZVDTT(J,IK,2)= ZVDTT(J,IK,2)+RTAPDI(NTVALUE(J))*ZDIVDI(IK,J)
      ZVDTT(J,IK,3)= ZVDTT(J,IK,3)+                   ZSPDI (J)
      ZVORDI(IK,J)= 0.0_JPRB
      ZDIVDI(IK,J)= 0.0_JPRB
      ZSPDI (J)   = 0.0_JPRB
    ENDDO

! Goto the transformed sphere : ZVDTR --> ZVDTT

    WRITE(NULOUT,'(''   CALL TO TRAGEOAD, DIR = -1 '' )')
    IFIELD  = 3
    IFLDSUR = 3
    ISPORTS = NSTTYP-1
    ITYPFLD = 2
    IDIRTR = -1
    CALL ABOR1('NNMI2AD: LLTRAGEO DEAD')

!    CALL TRAGEOAD(NULROTC,NULCONT,NULOUT,&
!     &IFIELD,IFLDSUR,NTMAX,NATM0,NTPEC2,NUMTP,&
!     &NCMAX,NACM0,NCPEC2,NUMCP,MYMS,&
!     &ISPORTS,ITYPFLD,IDIRTR,RLOCEN,0.0_JPRB,ZVDTT,ZVDTR)

! Vorticity and divergence on the real sphere are transformed into
! stream function and velocity potential : ZVDTR --> ZVDTR

    WRITE(NULOUT,'(''   COMPUTE STREAM FONCTION AND VELOCITY'',&
     & '' POTENTIAL ON THE REAL SPHERE'' )')  
    DO J=1,NCPEC2
      ZVDTR(J,IK,1)= RCAPIN(NCVALUE(J))* ZVDTR(J,IK,1)
      ZVDTR(J,IK,2)= RCAPIN(NCVALUE(J))* ZVDTR(J,IK,2)
    ENDDO

  ELSE
    DO J=1,NTPEC2
      ZVDTR(J,IK,1)= ZVDTR(J,IK,1) + ZVORDI(IK,J)
      ZVDTR(J,IK,2)= ZVDTR(J,IK,2) + ZDIVDI(IK,J)
      ZVDTR(J,IK,3)= ZVDTR(J,IK,3) + ZSPDI (J)
      ZVORDI(IK,J)= 0.0_JPRB
      ZDIVDI(IK,J)= 0.0_JPRB
      ZSPDI (J)   = 0.0_JPRB
    ENDDO

  ENDIF

!     ------------------------------------------------------------------

!*    2.    PROJECT TENDENCIES OR STATE VARIABLES (ZVDTR(..,1 to 3))
!*          ONTO REQUESTED MODES (ACCORDING TO KTYPE).
!*          (we are on the correct sphere with a truncation NCMAX)
!           -------------------------------------------------------

  IF (KTYPE == 2) THEN
    IMO = 3
    WRITE(NULOUT,'('' *****   COMPUTE NNMI INCREMENTS    *****'')')
  ELSEIF (KTYPE == 1) THEN
    IMO = 1
    WRITE(NULOUT,'('' ***** PROJECTION ONTO ROSSBY MODES *****'')')
  ENDIF

  N= -1
  ISPACE= 1
  CALL MOPRJMAD(ZVDTR,ZC,ZB,ZUP,ZDIAG,ZLOW,ZFQ,ZFP,NVMOD,NVMODPP,IMO,ISPACE)

!     ------------------------------------------------------------------

!*    1.    GOTO THE CORRECT SPHERE.
!           ------------------------

!*      1.6   FROM "TRANSFORMED" TO "GEOGRAPHIC" SPHERE:
!*            ZxxDI --> ZVDTT --> ZVDTR

  IF ((.NOT.LTRANS).AND.LLTRAGEO) THEN

!*        goto the real geographic sphere : ZxxDI --> ZVDTT --> ZVDTR

    WRITE(NULOUT,'('' GOTO THE REAL GEOGRAPHIC SPHERE :'' )')

! Vorticity and divergence on the real sphere are obtained using the
! stream function and the velocity potential : ZVDTR --> ZVDTR

    WRITE(NULOUT,'(''   COMPUTE VORTICITY AND DIVERGENCE'',&
     & '' ON THE REAL SPHERE'' )')  
    DO J=1,NCPEC2
      ZVDTR(J,IK,1)= RCAPDI(NCVALUE(J))* ZVDTR(J,IK,1)
      ZVDTR(J,IK,2)= RCAPDI(NCVALUE(J))* ZVDTR(J,IK,2)
    ENDDO

! Goto the real geographic sphere : ZVDTT --> ZVDTR

    WRITE(NULOUT,'(''   CALL TO TRAGEOAD, IDIR = +1 '' )')
    IDIRTR  = 1

!    CALL TRAGEOAD(NULROTC,NULDILA,NULOUT,&
!     &IFIELD,IFLDSUR,NTMAX,NATM0,NTPEC2,NUMTP,&
!     &NCMAX,NACM0,NCPEC2,NUMCP,MYMS,&
!     &ISPORTS,ITYPFLD,IDIRTR,RLOCEN,0.0_JPRB,ZVDTT,ZVDTR)

! Vorticity and divergence (transformed sphere) are first transformed into
! stream function and velocity potential before the call to TRAGEO.

    WRITE(NULOUT,'(''   COMPUTE STREAM FONCTION AND VELOCITY'',&
     & '' POTENTIAL ON THE TRANSFORMED SPHERE'' )')  
    DO J=1,NTPEC2
      ZVORDI(IK,J)= ZVORDI(IK,J)+RTAPIN(NTVALUE(J))* ZVDTT(J,IK,1)
      ZDIVDI(IK,J)= ZDIVDI(IK,J)+RTAPIN(NTVALUE(J))* ZVDTT(J,IK,2)
      ZSPDI (J)   = ZSPDI (J)   +                    ZVDTT(J,IK,3)
      ZVDTT(J,IK,1)= 0.0_JPRB
      ZVDTT(J,IK,2)= 0.0_JPRB
      ZVDTT(J,IK,3)= 0.0_JPRB
    ENDDO

!*        1.5   TRANSFORM OF VARIABLES (KTYPE=1) OR TENDENCIES (KTYPE=2).

  ELSE
    DO J=1,NTPEC2
      ZVORDI(IK,J)= ZVORDI(IK,J) + ZVDTR(J,IK,1)
      ZDIVDI(IK,J)= ZDIVDI(IK,J) + ZVDTR(J,IK,2)
      ZSPDI (J)   = ZSPDI (J)    + ZVDTR(J,IK,3)
      ZVDTR(J,IK,1)= 0.0_JPRB
      ZVDTR(J,IK,2)= 0.0_JPRB
      ZVDTR(J,IK,3)= 0.0_JPRB
    ENDDO

  ENDIF

!*      1.4   COPY STATE VARIABLES (linear NMI). (SPxx --> ZxxDI)

  IF (KTYPE == 1) THEN
    IF (NSMAX < NTMAX) THEN
      DO J=1,NSPEC2
        I= NSTREL(J)
        SPVOR(IK,J)= SPVOR(IK,J)+ ZVORDI(IK,I)
        SPDIV(IK,J)= SPDIV(IK,J)+ ZDIVDI(IK,I)
        SPSP (J)   = SPSP (J)   + ZSPDI (I)
      ENDDO
    ELSE
      SPVOR(IK,:)= SPVOR(IK,:)+ ZVORDI(IK,:)
      SPDIV(IK,:)= SPDIV(IK,:)+ ZDIVDI(IK,:)
      SPSP (:)   = SPSP (:)   + ZSPDI (:)
      ZVORDI(IK,:)=0.0_JPRB
      ZDIVDI(IK,:)=0.0_JPRB
      ZSPDI(:)=0.0_JPRB
    ENDIF

    SPVOR(IK,NASM0(0))= 0.0_JPRB
    SPDIV(IK,NASM0(0))= 0.0_JPRB
  ENDIF

!*      1.3   COPY TENDENCIES (non-linear NMI). (SPxx --> ZxxDI)

  IF (KTYPE == 2) THEN
    IF (NTMAX == NSMAX) THEN
      DO J=1,NSPEC2
        SPVOR(IK,J)= SPVOR(IK,J) + ZVORDI(IK,J)
        SPDIV(IK,J)= SPDIV(IK,J) + ZDIVDI(IK,J)
        SPSP (J)   = SPSP (J)    + ZSPDI (J)
      ENDDO
      ZVORDI(IK,:)=0.0_JPRB
      ZDIVDI(IK,:)=0.0_JPRB
      ZSPDI(:)=0.0_JPRB

      SPVOR(IK,NASM0(0))= 0.0_JPRB
      SPDIV(IK,NASM0(0))= 0.0_JPRB
    ELSE
      CALL SUALLTT
      CALL MVTENDAD(ZVORDI,ZDIVDI,ZSPDI,ZSPDI)
    ENDIF

!*      1.2   COMPUTE TENDENCIES.

    NSTEP= NSTART-ITERMAX-1+JITER

!* TRAJECTORY

    CALL STEPOAD(CLCONA)
    IF (NTMAX > NSMAX) CALL DEALLTT

!*      1.1   SAVE STATE VARIABLES. (SPxx --> Zxx)

    DO J=1,NSPEC2
      SPVOR(IK,J)= SPVOR(IK,J) + ZVOR(IK,J)
      SPDIV(IK,J)= SPDIV(IK,J) + ZDIV(IK,J)
      SPSP (J)   = SPSP (J)    + ZSP (J)
      ZVOR(IK,J)= 0.0_JPRB
      ZDIV(IK,J)= 0.0_JPRB
      ZSP (J)   = 0.0_JPRB
    ENDDO
    SPVOR(IK,NASM0(0))= 0.0_JPRB
    SPDIV(IK,NASM0(0))= 0.0_JPRB

  ENDIF

!     ** END OF MAIN LOOP OVER ITERATIONS **
!     **************************************

ENDDO ITERATION_LOOP

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('NNMI2AD',1,ZHOOK_HANDLE)
END SUBROUTINE NNMI2AD
