SUBROUTINE NNMI3AD

!**** *NNMI3AD* - NON-LINEAR NORMAL MODE INITIALIZATION FOR 3-D P.E.M.
!                 ADJOINT.

!     PURPOSE.   TO PERFORM THE NON-LINEAR NORMAL MODE INITIALIZATION
!     --------   FOR A 3-D P. E. MODEL.( ADJOINT ).

!**   INTERFACE.
!     ----------
!        *CALL* *NNMI3AD

!        EXPLICIT ARGUMENTS :  NONE
!        --------------------

!        IMPLICIT ARGUMENTS :  THE SPECTRAL ARRAYS OF THE MODEL
!        --------------------

!     METHOD.   THE TENDENCIES ARE COMPUTED BY AN APPROPIATE CALL TO STEPO.
!     -------   THE TENDENCIES ARE THEN CHANGED INTO NORMAL MODE SPACE
!               AND PROJECTED ONTO THE GRAVITY MODES DIVIDED BY THE
!               FREQUENCIES. WE GO BACK INTO SPECTRAL SPACE AND DEDUCT THE
!               PROJECTED FIELDS FROM THE STATE VARIABLES. THE PROCESS IS
!               REPEATED NITNMI TIMES.

!     EXTERNALS.  STEPOAD  - COMPUTE THE TENDENCIES
!     ----------  MO3DPRJAD- PROJECT ON MODES
!                 SPNORM   - COMPUTE NORMS IN SPECTRAL SPACE FOR ALL LEVELS
!                 MVTENDAD - COPYING TENDENCIES TO ARGUMENTS OF MO3DPRJ
!                 SUALLTT  - ALLOCATE SPACE FOR SPECTRAL TENDENCIES
!                 DEALLTT  - DESALLOCATE SPACE FOR SPECTRAL TENDENCIES
!                 RDPINMI  - READ HELP ARRAYS FOR IMPLICIT NMI
!                 USER_CLOCK- timing

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        Jean-Noel THEPAUT *METEO FRANCE EERM/CRMD*
!        - BASED ON THE ROUTINE NNMI2A WRITTEN BY
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 90-03-15
!        D. GIARD : 92-11-09 : additionnal NNMI step to test convergence
!        D. GIARD : 92-11-26 : LRSPSL to restore Ps, final form
!        D. GIARD : 93-04-05 : NNMI with stretching, NTMAX
!        D. GIARD : 93-05-24 : cleaning
!        J.N. Thepaut   : 94-02-21 : tidying up of initialization of ps
!        L. Isaksen : 95-06-06 : Reordering of spectral arrays
!        M.Hamrud   : 95-10-30 : Changed CLCONF def.
!        D.Dent     : 97-10-03 : Changed SECOND to USER_CLOCK
!        C.Temperton: 98-04-06 : Correction of (benign?) bug  
!        C.Temperton: 01-01-24 : Add LEULNMI to adjoint code  
!        G. Desroziers : 02-09-16 also treat Aladin mean wind trajectory
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!***************************************************************************

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVL   ,NSMAX    ,NSPEC2   ,NCPEC    ,&
 & NCPEC2   ,NTMAX    ,NTPEC2  
USE YOMSP    , ONLY : SPVOR    ,SPDIV    ,SPT      ,SPQ      ,SPSP,&
 & SP5A3, SP5A2  
USE YOMLAP   , ONLY : NASM0    ,NSTREL
USE YOMDYN   , ONLY : TSTEP    ,TDT
USE YOMNMIA  , ONLY : NLEX     ,NITNMI   ,NVMODM   ,NVMOD    ,&
 & NVMODPP  ,LRPIMP   ,LSPNDG   ,LRSPSL  
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NSTART   ,LSLAG    ,LTWOTL
USE YOMCT1   , ONLY : N1SDI
USE YOMCT3   , ONLY : NSTEP
USE YOMMP    , ONLY : MYSETW   ,NPROCM   ,NUMVMO   ,NPSP
USE YOMVAR   , ONLY : LTRREF
USE TRAJECTORY_MOD, ONLY: LTRAJGP, GET_TRAJ_SPEC

IMPLICIT NONE

!     DUMMY REAL
INTEGER(KIND=JPIM) :: I, IMODE, IMODES, ISPACE, J, JITER, JK

LOGICAL :: LLRSPSL, LLSLAGX, LLTWOTLX

REAL(KIND=JPRB) :: ZT1, ZT2

!    DUMMY REAL
REAL(KIND=JPRB) :: ZDUM(1,1)

INTERFACE
#include "user_clock.h"
END INTERFACE

#include "mo3dprjad.intfb.h"
#include "mvtendad.intfb.h"
#include "rdpinmi.intfb.h"
#include "spnorm.intfb.h"
#include "stepo.intfb.h"
#include "stepoad.intfb.h"
#include "sualltt.intfb.h"
#include "dealltt.intfb.h"
#include "updtim.intfb.h"

!*******************************************************
!**  ARRAYS ON THE T"NSMAX" OF THE TRANSFORMED SPHERE **
!*******************************************************
REAL(KIND=JPRB) :: ZVOR(NFLEVL,NSPEC2),ZDIV(NFLEVL,NSPEC2),ZT(NFLEVL,NSPEC2)
REAL(KIND=JPRB) :: ZSP(NSPEC2),ZQ(NFLEVL,NSPEC2)
!*******************************************************
!**  ARRAYS ON THE T"NTMAX" OF THE TRANSFORMED SPHERE **
!*******************************************************
REAL(KIND=JPRB) :: ZVORDI(NFLEVL,NTPEC2),ZDIVDI(NFLEVL,NTPEC2)
REAL(KIND=JPRB) :: ZTDI(NFLEVL,NTPEC2),ZSPDI(NTPEC2)
!***********************************************************
!**  ARRAYS ON THE T"NCMAX" OF THE REAL GEOGRAPHIC SPHERE **
!***********************************************************
REAL(KIND=JPRB) :: ZC(NCPEC2,NVMODM),ZDIAG(NCPEC2,NVMODM)
REAL(KIND=JPRB) :: ZFP(NCPEC),ZFQ(NCPEC),ZB(NCPEC2),ZLOW(NCPEC2),ZUP(NCPEC2)

CHARACTER (LEN = 9) ::  CLCONF,CLCONA
CHARACTER (LEN = 39) ::  CLMARK,CLTIME
CHARACTER (LEN = 10) ::  CLTIMEOD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       0.    PREAMBLE.
!              ---------

!*       0.1   SET UP CONFIGURATION FOR STEPO AND STEPOAD.

IF (LHOOK) CALL DR_HOOK('NNMI3AD',0,ZHOOK_HANDLE)
CLCONA(1:9)='0AAE00A?0'
IF (NTMAX == NSMAX) THEN
  CLCONA(8:8)='A'
ELSE
  CLCONA(8:8)='T'
ENDIF

!*    0.2    DIAGNOSTICS.

WRITE(NULOUT,'('' NON LINEAR 3-D NORMAL MODE INITIALIZATION'')')
IF (LRPIMP) THEN
  IF (NLEX == 0) THEN
    WRITE(NULOUT,'('' PURE IMPLICIT SCHEME  '')')
  ELSE
    WRITE(NULOUT,'('' MIXED (EXPL.-IMPL.) SCHEME  '')')
  ENDIF
ELSE
  WRITE(NULOUT,'('' PURE EXPLICIT SCHEME  '')')
ENDIF
IF (N1SDI == 1.AND.LSPNDG) THEN
  WRITE(NULOUT,'('' SPECTRUM DIAGNOSTICS BEFORE NMI ITERATIONS'')')
  CALL SPNORM(0)
ENDIF

!*    0.3    READ OF HELP ARRAYS FOR IMPLICIT NMI SCHEME

IF (LRPIMP) THEN
  IMODES = NVMOD
  CALL RDPINMI(IMODES,ZFP,ZFQ,ZB,ZLOW,ZUP,ZC,ZDIAG)
ENDIF

!*    0.4    SET LOCAL VARIABLES TO ZERO.

ZVOR(:,:)= 0.0_JPRB
ZDIV(:,:)= 0.0_JPRB
ZT  (:,:)= 0.0_JPRB
ZQ  (:,:)= 0.0_JPRB
IF (NPSP == 1) ZSP(:) = 0.0_JPRB

ZVORDI(:,:)= 0.0_JPRB
ZDIVDI(:,:)= 0.0_JPRB
ZTDI  (:,:)= 0.0_JPRB
IF (NPSP == 1) ZSPDI(:)= 0.0_JPRB

!*    0.5    RESET SEMI-LAGRANGIAN SWITCH.

! Convert to Eulerian model for computation of tendencies,
! having first saved the value of the semi-Lagrangian switch.

LLSLAGX  = LSLAG
LSLAG  = .FALSE.
LLTWOTLX = LTWOTL
LTWOTL = .FALSE.

!     ------------------------------------------------------------------
!     *******************************
!     ** MAIN LOOP OVER ITERATIONS **
!     *******************************

ITERATION_LOOP: DO JITER=NITNMI,1,-1

  CALL USER_CLOCK(PTOTAL_CP=ZT1)

!       If LLRSPSL=.TRUE., Ps is not updated
!       LLRSPSL must be .FALSE. at the last iteration

  LLRSPSL=LRSPSL
  IF (JITER == NITNMI) LLRSPSL=.FALSE.

!     ------------------------------------------------------------------

!*       5.    COMPUTE TRAJECTORY.

  NSTEP=NSTART-NITNMI-1+JITER
  CALL UPDTIM(0,1.0_JPRB,1.0_JPRB,.FALSE.)

  IF (.NOT.LTRAJGP) THEN
    IF (LTRREF) THEN
      CLCONF(1:9) = 'BE0000000'
    ELSE
      CALL GSTATS(15,0)
      CALL get_traj_spec(SP5A3,SP5A2,ZDUM,KSTEP=nstep)
      CALL GSTATS(15,1)
      CLCONF(1:9)='0E0000000'
    ENDIF
    CALL STEPO(CLCONF)
  ENDIF

!     ------------------------------------------------------------------

!*       4.    UPDATE STATE VARIABLES.
!              -----------------------

  IF (MYSETW == NPROCM(0)) THEN
    SPVOR(1:NFLEVL,NASM0(0))= 0.0_JPRB
    SPDIV(1:NFLEVL,NASM0(0))= 0.0_JPRB
  ENDIF

  IF (NSMAX < NTMAX) THEN
    IF (NPSP == 1) THEN
      IF (LLRSPSL) THEN
        ZSP(:) = ZSP(:) + SPSP(:)
      ELSE
        DO J=1,NSPEC2
          I= NSTREL(J)
          ZSP(J)  = ZSP(J)   + SPSP(J)
          ZSPDI(I)= ZSPDI(I) - SPSP(J)
        ENDDO
      ENDIF
    ENDIF
    DO JK=1,NFLEVL
      DO J=1,NSPEC2
        I= NSTREL(J)
        ZVOR(JK,J)= ZVOR(JK,J) + SPVOR(JK,J)
        ZDIV(JK,J)= ZDIV(JK,J) + SPDIV(JK,J)
        ZT  (JK,J)= ZT  (JK,J) + SPT  (JK,J)
        ZQ  (JK,J)= ZQ  (JK,J) + SPQ  (JK,J)
        ZVORDI(JK,I)= ZVORDI(JK,I) - SPVOR(JK,J)
        ZDIVDI(JK,I)= ZDIVDI(JK,I) - SPDIV(JK,J)
        ZTDI  (JK,I)= ZTDI  (JK,I) - SPT  (JK,J)
      ENDDO
    ENDDO

  ELSE
    IF (NPSP == 1) THEN
      IF (LLRSPSL) THEN
        ZSP(:) = ZSP(:) + SPSP(:)
      ELSE
        ZSP(:)  = ZSP(:)   + SPSP(:)
        ZSPDI(:)= ZSPDI(:) - SPSP(:)
      ENDIF
    ENDIF
    ZVOR(1:NFLEVL,:)= ZVOR(1:NFLEVL,:) + SPVOR(1:NFLEVL,:)
    ZDIV(1:NFLEVL,:)= ZDIV(1:NFLEVL,:) + SPDIV(1:NFLEVL,:)
    ZT  (1:NFLEVL,:)= ZT  (1:NFLEVL,:) + SPT  (1:NFLEVL,:)
    ZQ  (1:NFLEVL,:)= ZQ  (1:NFLEVL,:) + SPQ  (1:NFLEVL,:)
    ZVORDI(1:NFLEVL,:)= ZVORDI(1:NFLEVL,:) - SPVOR(1:NFLEVL,:)
    ZDIVDI(1:NFLEVL,:)= ZDIVDI(1:NFLEVL,:) - SPDIV(1:NFLEVL,:)
    ZTDI  (1:NFLEVL,:)= ZTDI  (1:NFLEVL,:) - SPT  (1:NFLEVL,:)
  ENDIF
  SPSP(:) = 0.0_JPRB
  SPVOR(1:NFLEVL,:)= 0.0_JPRB
  SPDIV(1:NFLEVL,:)= 0.0_JPRB
  SPT  (1:NFLEVL,:)= 0.0_JPRB
  SPQ  (1:NFLEVL,:)= 0.0_JPRB

!     ------------------------------------------------------------------

!*       3.    PROJECT TENDENCIES ON GRAVITY MODES.
!              -------------------------------------

! IMODE=3: projection on grav. waves and division by freq.
! ISPACE=1: working space = spectral space

  IMODE=3
  ISPACE=1
  CALL MO3DPRJAD(IMODE,ISPACE,NVMOD,NVMODPP,NUMVMO,ZVORDI,ZDIVDI,&
   & ZTDI,ZSPDI,ZC,ZB,ZUP,ZDIAG,ZLOW,ZFQ,ZFP)  

!     ------------------------------------------------------------------

!*       2.    COMPUTE TENDENCIES.
!              -------------------

  IF (NTMAX == NSMAX) THEN
    IF (NPSP == 1) THEN
      SPSP(:) = SPSP(:) + ZSPDI(:)
      ZSPDI(:)= 0.0_JPRB
    ENDIF
    SPVOR(1:NFLEVL,:)= SPVOR(1:NFLEVL,:) + ZVORDI(1:NFLEVL,:)
    SPDIV(1:NFLEVL,:)= SPDIV(1:NFLEVL,:) + ZDIVDI(1:NFLEVL,:)
    SPT  (1:NFLEVL,:)= SPT  (1:NFLEVL,:) + ZTDI  (1:NFLEVL,:)
    ZVORDI(1:NFLEVL,:)= 0.0_JPRB
    ZDIVDI(1:NFLEVL,:)= 0.0_JPRB
    ZTDI  (1:NFLEVL,:)= 0.0_JPRB

    IF (MYSETW == NPROCM(0)) THEN
      SPVOR(1:NFLEVL,NASM0(0))= 0.0_JPRB
      SPDIV(1:NFLEVL,NASM0(0))= 0.0_JPRB
    ENDIF
  ELSE
    CALL SUALLTT
    CALL MVTENDAD(ZVORDI,ZDIVDI,ZTDI,ZSPDI)
  ENDIF

  CALL STEPOAD(CLCONA)
  IF (NTMAX /= NSMAX) CALL DEALLTT

!     ------------------------------------------------------------------

!*       1.    SAVE STATE VARIBLES.
!              ---------

  IF (NPSP == 1) THEN
    SPSP(:)= SPSP(:)+ZSP(:)
    ZSP(:) = 0.0_JPRB
  ENDIF
  SPVOR(1:NFLEVL,:)= SPVOR(1:NFLEVL,:) +ZVOR(1:NFLEVL,:)
  SPDIV(1:NFLEVL,:)= SPDIV(1:NFLEVL,:) +ZDIV(1:NFLEVL,:)
  SPT  (1:NFLEVL,:)= SPT  (1:NFLEVL,:) +ZT  (1:NFLEVL,:)
  SPQ  (1:NFLEVL,:)= SPQ  (1:NFLEVL,:) +ZQ  (1:NFLEVL,:)
  ZVOR(1:NFLEVL,:)= 0.0_JPRB
  ZDIV(1:NFLEVL,:)= 0.0_JPRB
  ZT  (1:NFLEVL,:)= 0.0_JPRB
  ZQ  (1:NFLEVL,:)= 0.0_JPRB

  IF (MYSETW == NPROCM(0)) THEN
    SPVOR(1:NFLEVL,NASM0(0))= 0.0_JPRB
    SPDIV(1:NFLEVL,NASM0(0))= 0.0_JPRB
  ENDIF

  WRITE(NULOUT,'('' NON-LINEAR NMI (ADJ), ITERATION='',I2)') JITER
  IF (N1SDI == 1.AND.LSPNDG) THEN
    WRITE(NULOUT,FMT=&
     & '('' SPECTRUM DIAGNOSTICS AFTER'',I2,'' NMI ITERATION(S)'')')&
     & JITER  
    CALL SPNORM(0)
  ENDIF
  CALL USER_CLOCK(PTOTAL_CP=ZT2)
  CALL DATE_AND_TIME(TIME=CLTIMEOD)
  WRITE (UNIT=CLTIME,FMT=997)&
   & CLTIMEOD(1:2),CLTIMEOD(3:4),CLTIMEOD(5:6),ZT2-ZT1  
  WRITE (UNIT=CLMARK,FMT=998) JITER,NITNMI
  WRITE(NULOUT,*) CLTIME
  WRITE(NULOUT,*) CLMARK

!     ** END OF MAIN LOOP OVER ITERATIONS **

ENDDO ITERATION_LOOP

CALL UPDTIM(0,TDT,TSTEP,.FALSE.)

!*    5.1   RESET SEMI-LAGRANGIAN SWITCH.

LSLAG  = LLSLAGX
LTWOTL = LLTWOTLX

997 FORMAT (1X,A,'':'',A,'':'',A,1X,F11.5,' CPU S IN ITERATION')
998 FORMAT (' INITIAL. , 3D NNMI : ITERATION ',I2,' / ',I2)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('NNMI3AD',1,ZHOOK_HANDLE)
END SUBROUTINE NNMI3AD
