SUBROUTINE REORDO3(PF,PVOR,PDIV,PHGT,KVMO,KM,KDIR)

!**** *REORDO3* - RE-ORDER FIELDS FOR NMI, FOR A GIVEN VERTICAL MODE.

!     PURPOSE.   TO REORDER THE FIELDS FOR NMI SO THAT THEY ARE
!     --------   ORDERED BY INCREASING M AND VORTICITY, DIVERGENCE
!                AND HEIGHT FOLLOW EACH OTHER. KDIR=1 MEANS SET
!                UP BEFORE NMI, KDIR= -1 MEANS PUT VALUES BACK
!                AFTER NMI. THE ROUTINE ALSO NON-DIMENSIONALIZES
!                THE VARIABLES (BUT NOT WITH RESPECT TO TIME).
!                (ALL ARRAYS ARE ON TRUNCATION : NXMAX)

!**   INTERFACE.
!     ----------
!        *CALL* *REORDO3(PF,PVOR,PDIV,PHGT,KVMO,KM,KDIR)*

!        EXPLICIT ARGUMENTS :  PF - FIELD SET UP FOR NMI
!        --------------------  PVOR - VORTICITY
!                              PDIV - DIVERGENCE
!                              PHGT - GEOPOTENTIAL
!                              KVMO - VERTICAL MODE INDEX
!                              KM   - WAVE NUMBER
!                              KDIR - +1: PF=OUTPUT  PVOR,PDIV,PHGT=INPUT
!                                     -1: PF=INPUT   PVOR,PDIV,PHGT=OUTPUT

!        IMPLICIT ARGUMENTS :  SIVP (YOMDYN)
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.   NONE.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!      P.COURTIER AND J.F. GELEYN: A GLOBAL MODEL WITH VARAIBLE
!      RESOLUTION. APPLICATION TO A SHALLOW-WATER MODEL.
!      QUA.J.ROI.MET.SOC. 1988.

!     AUTHOR.
!     -------
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 88-05-25
!        CLIVE TEMPERTON : 90-01-14 : REORDO => REORDO3
!        PASCAL MARQUET  : 91-02-15 : TRUNCATION "NXMAX"
!        D. Giard        : 91-12-16 : condition for NMI on the
!                                     transformed sphere (NSTTYP=1)
!        D. Giard        : 93-04-30 : LTRANS checked in SUGEM1 now
!        L. Isaksen      : 94-08-17 : Macro tasked over wave number
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : ROMEGA   ,RA
USE YOMDYN   , ONLY : SIVP
USE YOMGEM   , ONLY : RSTRET
USE YOMDIM   , ONLY : NCMAX    ,NXMAX
USE YOMNMIA  , ONLY : LTRANS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KM 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PF(6*(NXMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVOR(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIV(2*(NCMAX+1-KM)) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PHGT(2*(NCMAX+1-KM)) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KVMO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIR 
INTEGER(KIND=JPIM) :: IJ, IMPVAL, IREVAL, JN

REAL(KIND=JPRB) :: ZCP1SC, ZFP0, ZGEO, ZOME2, ZZGEO
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    COPY FROM SPECTRAL ARRAYS TO "NMI" ARRAY.
!              -----------------------------------------

IF (LHOOK) CALL DR_HOOK('REORDO3',0,ZHOOK_HANDLE)
IF (LTRANS) THEN
  ZCP1SC=RSTRET+1.0_JPRB/RSTRET
  ZFP0=ZCP1SC**2/4._JPRB
ELSE
  ZFP0= 1.0_JPRB
ENDIF

IF (KDIR == 1) THEN
  ZZGEO=1.0_JPRB/(2.0_JPRB*ROMEGA*RA*SQRT(ZFP0*SIVP(KVMO)))
  ZOME2=1.0_JPRB/(2.0_JPRB*ROMEGA)
  IJ=-5
  IF(KM /= 0) THEN
    DO JN=KM,NXMAX
      IJ=IJ+6
      IREVAL=JN+1-KM
      IMPVAL=NCMAX+1-KM+IREVAL
      ZGEO=ZZGEO*SQRT(REAL(MAX(JN*(JN+1),1),JPRB))
      PF(IJ)  = PVOR(IREVAL)*ZOME2
      PF(IJ+1)= PVOR(IMPVAL)*ZOME2
      PF(IJ+2)= PDIV(IMPVAL)*ZOME2
      PF(IJ+3)=-PDIV(IREVAL)*ZOME2
      PF(IJ+4)= PHGT(IREVAL)*ZGEO
      PF(IJ+5)= PHGT(IMPVAL)*ZGEO
    ENDDO
  ELSE
    DO JN=KM,NXMAX
      IJ=IJ+6
      IREVAL=JN+1-KM
      ZGEO=ZZGEO*SQRT(REAL(MAX(JN*(JN+1),1),JPRB))
      PF(IJ)  =PVOR(IREVAL)*ZOME2
      PF(IJ+1)=0.0_JPRB
      PF(IJ+2)=0.0_JPRB
      PF(IJ+3)=-PDIV(IREVAL)*ZOME2
      PF(IJ+4)= PHGT(IREVAL)*ZGEO
      PF(IJ+5)=0.0_JPRB
    ENDDO
  ENDIF

!     ------------------------------------------------------------------

!*          2.    COPY FROM "NMI" ARRAY TO SPECTRAL ARRAY.
!                 ----------------------------------------

ELSEIF (KDIR == -1) THEN
  ZZGEO=2.0_JPRB*ROMEGA*RA*SQRT(ZFP0*SIVP(KVMO))
  ZOME2=2.0_JPRB*ROMEGA
  IJ=-5
  IF(KM /= 0) THEN
    DO JN=KM,NXMAX
      IJ=IJ+6
      IREVAL=JN+1-KM
      IMPVAL=NCMAX+1-KM+IREVAL
      ZGEO=ZZGEO/SQRT(REAL(MAX(JN*(JN+1),1),JPRB))
      PVOR(IREVAL)= PF(IJ)  *ZOME2
      PVOR(IMPVAL)= PF(IJ+1)*ZOME2
      PDIV(IMPVAL)= PF(IJ+2)*ZOME2
      PDIV(IREVAL)=-PF(IJ+3)*ZOME2
      PHGT(IREVAL)= PF(IJ+4)*ZGEO
      PHGT(IMPVAL)= PF(IJ+5)*ZGEO
    ENDDO
  ELSE
    DO JN=KM,NXMAX
      IJ=IJ+6
      IREVAL=JN+1-KM
      ZGEO=ZZGEO/SQRT(REAL(MAX(JN*(JN+1),1),JPRB))
      PVOR(IREVAL)= PF(IJ)  *ZOME2
      PDIV(IREVAL)=-PF(IJ+3)*ZOME2
      PHGT(IREVAL)= PF(IJ+4)*ZGEO
    ENDDO
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('REORDO3',1,ZHOOK_HANDLE)
END SUBROUTINE REORDO3

