SUBROUTINE SUMODE3E(KVIMOD)

!**** *SUMODE3E* - COMPUTE NORMAL MODES FOR MULTI-LEVEL MODEL
!*                 CALLED IN CASE MATRICES AND FREQUENCIES MUST BE COMPUTED
!*

!     PURPOSE. - TO COMPUTE THE NORMAL MODES FOR A STRETCHED CO-
!     --------   ORDINATE MULTI-LEVEL MODEL ON THE SPHERE.
!              - STORING OF MATRICES OF FREQUENCIES        (EXPLICIT MODES)
!              - STORING OF "LTYPGR,LTYPDB,LTYPTD" ARRAYS  (EXPLICIT MODES)
!              - STORING OF HELP ARRAYS                    (IMPLICIT MODES)

!**   INTERFACE.
!     ----------
!        *CALL* *SUMODE3E(KVIMOD)*

!        EXPLICIT ARGUMENTS
!        ------------------
!                          KVIMOD : Number if vertical modes to be catered for.

!        IMPLICIT ARGUMENTS :  in YOMNMI : LTYPGR,... ; FREQ, ...
!        --------------------  in YOMDYN : (SIVP(JK),JK=1,KVIMOD)
!                              in YOMLAP : NACNO(JN),...
!                              in YOMGEM : RSTRET
!                              in YOMLUN : NSPHMO,NHOUMO,NPINMI,NFREQ2,NLTYPE
!                              in YOMCST : ROMEGA,RA,RPI
!                              in YOMGC  : RCOR0,RCOR1
!                                       RCOR0  - EXPANSION OF CORIOLIS PARAMETER
!                                       RCOR1  - EXPANSION OF CORIOLIS PARAMETER
!                        f = 2 * ROMEGA * [ RCOR0 + (SQRT(3) * RCOR1)  * RMU ]

!     METHOD.  SEE DOCUMENTATION.
!     -------

!     EXTERNALS.   SCINIM - SET UP MATRIX FOR SOLVING LIN. EQ.
!     ----------   EIGSOL - OBTAIN EIGENVALUES AND EIGENVECTORS
!                  SCORDO - ORDER FREQUENCIES AND MODES
!                  SCNORM - NORMALIZE MODES
!                  SGECO  - FACTOR A MATRIX BY GAUSSIAN ELIMINATION
!                  SGEDI  - OBTAIN INVERSE OF MATRIX

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS
!      P.COURTIER AND J.F. GELEYN: A GLOBAL MODEL WITH VARAIBLE
!      RESOLUTION. APPLICATION TO A SHALLOW-WATER MODEL.
!      QUA.J.ROI.MET.SOC. 1988.

!     AUTHOR.
!     -------
!        LARS ISAKSEN *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 97-05-05 (Split off from old SUMODE3)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ---------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : NCONF
USE YOMDIM   , ONLY : NXMAX    ,NXPEC    ,NXPECG
USE YOMDYN   , ONLY : SIVP
USE YOMLUN   , ONLY : NULOUT
USE YOMNMIA  , ONLY : LTRANS   ,LNMIPR   ,LNMIFF
USE YOMNMIB  , ONLY : FREQ
USE YOMGEM   , ONLY : RSTRET   ,RCOR0    ,RCOR1
USE YOMCST   , ONLY : ROMEGA   ,RA       ,RG
USE YOMSPHB  , ONLY : SPHBUF   ,NSTASPHB
USE YOMHOUB  , ONLY : HOUBUF   ,NSTAHOUB
USE YOMMP    , ONLY : MYSETW   ,NPROCM
USE YOMMSC   , ONLY : N_DEFAULT_REAL_KIND, N_DOUBLE_KIND

IMPLICIT NONE

!     EXTERNAL INTEGER FUNCTIONS
INTEGER(KIND=JPIM),INTENT(IN)    :: KVIMOD 
INTEGER(KIND=JPIM), EXTERNAL :: ISAMAX

!     EXTERNAL REAL FUNCTIONS
REAL(KIND=JPRB), EXTERNAL :: SNRM2,DNRM2

!     ------------------------------------------------------------------
REAL(KIND=JPRB) :: ZA(3*(NXMAX+1)*3*(NXMAX+1)),ZEPS(0:NXMAX)
REAL(KIND=JPRB) :: ZD(-1:NXMAX+1),ZK(0:NXMAX),ZP(0:NXMAX),ZQ(0:NXMAX)
REAL(KIND=JPRB) :: ZSCR(6*NXMAX+6)
REAL(KIND=JPRB) :: ZWSPH(((3*(NXMAX+1)*3*(NXMAX+1)-1)/512+1)*512,1)
REAL(KIND=JPRB) :: ZWHOU(((3*(NXMAX+1)*3*(NXMAX+1)-1)/512+1)*512,1)
REAL(KIND=JPRB) :: ZFREQR(3*(NXMAX+1)),ZFREQI(3*(NXMAX+1)),ZRWOR(3*(NXMAX+1))
REAL(KIND=JPRB) :: ZZFREQ(KVIMOD,3*NXPEC)
INTEGER(KIND=JPIM) :: IWOR(3*(NXMAX+1))

INTEGER(KIND=JPIM) :: IBUFHO, IBUFSH, ICFREQ, ICFREQG, ICFRQ, ICONF,&
 & IDEB, IDIA, IERR, IFLAG, IFREQ, IFREQG, IJ, &
 & IJMAX, ILEN, IN, INGE, INGW, INRO, &
 & IORDR, IPRGE, IPRGW, IPRR, IREC, IRLEN, IROW, &
 & J, JCOL, JJ, JK, JM, JN, INFO  

REAL(KIND=JPRB) :: ZCM1SC, ZCP1SC, ZERRL2, ZERRM, ZF0,&
 & ZF1, ZFP0, ZFP1, ZFP2, ZOM2, ZRA2, ZTES  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "eigsol.h"
#include "gatherfreq.intfb.h"
#include "scinim.intfb.h"
#include "scnorm.intfb.h"
#include "scordo.intfb.h"

!     ------------------------------------------------------------------

!*    1.1   PREAMBLE

IF (LHOOK) CALL DR_HOOK('SUMODE3E',0,ZHOOK_HANDLE)
IFREQ = 6*NXPEC
IFREQG = 6*NXPECG
ZRA2 = RA**2
ZOM2 = ROMEGA*2
IFLAG = 1

!*    1.2   COMPUTE MAPPING FACTOR AND CORIOLIS EXPANSIONS.
!           For c=1 and if NSTTYP=1 : RCOR0=0 and RCOR1*SQRT(3)=1

IF (LTRANS) THEN
  ZCP1SC = RSTRET+1.0_JPRB/RSTRET
  ZCM1SC = RSTRET-1.0_JPRB/RSTRET
  ZFP0 = ZCP1SC*ZCP1SC/4._JPRB
  ZFP1 = ZCP1SC*ZCM1SC/2.0_JPRB
  ZFP2 = ZCM1SC*ZCM1SC/4._JPRB
  ZF0  = RCOR0
  ZF1  = RCOR1*SQRT(3._JPRB)
ELSE
  ZFP0 = 1._JPRB
  ZFP1 = 0._JPRB
  ZFP2 = 0._JPRB
  ZF0  = 0._JPRB
  ZF1  = 1._JPRB
ENDIF

!     **************************************************
!     *** BEGINNING OF LOOP OVER JK (VERTICAL MODES) ***
!     ***                                            ***

MODES_LOOP: DO JK=1,KVIMOD

  ICFRQ =0
  ICFREQ=0

!       *********************************************
!       *** BEGINNING OF LOOP OVER WAVE NUMBERS M ***
!       ***                                       ***

  WAVE_LOOP: DO JM=0,NXMAX
    IF (MYSETW /= NPROCM(JM)) CYCLE
    IREC=(JK-1)*(NXMAX+1)+JM+1
    IBUFSH=1
    IBUFHO=1

!*        2.1   COMPUTE HELP ARRAYS FOR EXPLICIT NMI

    DO JN=MAX(JM,1),NXMAX
      ZEPS(JN) = SQRT(REAL(JN*(JN+1),JPRB)*SIVP(JK)*ZFP0)/(ZOM2*RA)
      ZD(JN)   = SQRT(REAL(JN**2-JM**2,JPRB)/REAL(4*JN**2-1,JPRB))
      ZK(JN)   = - REAL(JM,JPRB)/REAL(JN*(JN+1),JPRB)
      ZP(JN)   = ZD(JN)*REAL(JN+1,JPRB)/REAL(JN,JPRB)
      ZQ(JN)   = SQRT(REAL((JN+1)**2-JM**2,JPRB)/&
       & REAL(4*(JN+1)**2-1,JPRB))*REAL(JN,JPRB)/REAL(JN+1,JPRB)  
    ENDDO
    IN=NXMAX+1
    ZD(IN) = SQRT(REAL(IN*IN-JM*JM,JPRB)/REAL(4*IN*IN-1,JPRB))

    IJMAX=  MAX(JM,1)-1
    ZEPS(0:IJMAX) = 0.0_JPRB
    ZP(0:IJMAX)   = 0.0_JPRB
    ZK(0:IJMAX)   = 0.0_JPRB
    ZQ(0:IJMAX)   = 0.0_JPRB
    ZD(-1:IJMAX)   = 0.0_JPRB

    IF(JM == 0) THEN
      ZP(1) = 0._JPRB
    ENDIF

!         ------------------------------------------------------------

!*        3.    COMPUTE THE MATRIX DESCRIBING LINEARISED EQ. (ZA)
!               -------------------------------------------------

    ZA(1:3*(NXMAX+1)*3*(NXMAX+1))=0.0_JPRB

    CALL SCINIM(NXMAX,JM,ZA,ZD,ZK,ZP,ZQ,ZEPS,ZF0,ZF1,ZFP0,ZFP1,ZFP2)

!         ------------------------------------------------------------

!*        4.    SOLVE EIGEN-VALUE PROBLEM.
!               --------------------------

    IORDR=3*(NXMAX+1-JM)

!*        4.1   COMPUTE EIGENVALUES (ZFREQx) AND EIGENVECTORS (ZWHOU) OF "ZA".

    IERR=0
    CALL EIGSOL(IORDR,IORDR,ZA,ZFREQR,ZFREQI,1,ZWHOU(1,IBUFHO),&
     & IWOR,ZRWOR,IERR)  
    IF(IERR /= 0)THEN
      WRITE(NULOUT,'('' ERROR IN RG : IERR ='',I8)')IERR
      IFLAG=0
      CALL ABOR1('SUMODE3 : ERROR IN RG (EIGSOL) CALL')
    ENDIF

!*        4.2   ORDER AND NORMALIZE THE MODES.

    CALL SCORDO(IORDR,ZFREQR,ZFREQI,ZWHOU(1,IBUFHO),ZWSPH(1,IBUFSH))
    CALL SCNORM(JM,NXMAX,IORDR,ZWHOU(1,IBUFHO),ZOM2,SIVP(JK))

!*        4.3   FIX SIGNS OF EIGENVECTORS

!         FIRST WESTWARD GRAVITY MODES :
!              "DIAGONAL DIVERGENCE COMPONENT" SET POSITIVE
!         THEN  ROSSBY MODES :
!              "DIAGONAL VORTICITY COMPONENT" SET POSITIVE
!         THEN  EASTWARD GRAVITY MODES :
!              "DIAGONAL DIVERGENCE COMPONENT" SET POSITIVE

!         ROW = 1 : VORTICITY    I
!         ROW = 2 : DIVERGENCE   I  NORMALIZED AS IN COURTIER-GELEYN
!         ROW = 3 : GEOPOTENTIAL I

!         EX.:
!            . . .  . . *  . . .   VOR N=M
!            . . *  . . .  * . .   DIV N=M
!        I   . . .  . . .  . . .   PHI N=M
!        O   . . .  . * .  . . .   VOR N=M+1
!        R   . * .  . . .  . * .   DIV N=M+1           * SET POSITIVE
!        D   . . .  . . .  . . .   PHI N=M+1
!        R   . . .  * . .  . . .   VOR N=M+2=NXMAX
!            * . .  . . .  . . *   DIV N=M+2=NXMAX
!            . . .  . . .  . . .   PHI N=M+2=NXMAX
!            INGW  INRO  INGE

!         CAUTION : DEGENERATE CASE FOR M=0
!         EXTRA CAUTION : VARIATIONAL CONFS. RELY ON 1/3,1/3,1/3
!                         DIVISION, DO NOT CHANGE WITHOUT CONSULTING
!                         SCIENTISTS IN CHARGE OF JB FORMULATION

    ICONF=NCONF/100
    IF(ICONF /= 1) THEN
      IF (JM == 0.AND..NOT.LNMIFF) THEN
        INGW = NXMAX
        INRO = NXMAX+3
        INGE = NXMAX
      ELSEIF (JM == 0.AND.LNMIFF) THEN
        INGW = NXMAX-1
        INRO = NXMAX+3
        INGE = NXMAX+1
      ELSE
        INGW = IORDR/3
        INRO = INGW
        INGE = INGW
      ENDIF
    ELSE
      INGW = IORDR/3
      INRO = INGW
      INGE = INGW
    ENDIF

    IROW = 2
    DO JCOL = 1,INGW
      IDEB = (JCOL-1)*IORDR
      IDIA = IORDR + IROW - 3*JCOL
      IF ( ZWHOU(IDEB+IDIA,IBUFHO) < 0.0_JPRB ) THEN
        DO JJ = 1,IORDR
          ZWHOU(IDEB+JJ,IBUFHO) = -ZWHOU(IDEB+JJ,IBUFHO)
        ENDDO
      ENDIF
    ENDDO

    IROW = 1
    DO JCOL = 1,INRO
      IDEB = (INGW+JCOL-1)*IORDR
      IDIA = IORDR + IROW - 3*JCOL
      IF ( ZWHOU(IDEB+IDIA,IBUFHO) < 0.0_JPRB ) THEN
        DO JJ = 1,IORDR
          ZWHOU(IDEB+JJ,IBUFHO) = -ZWHOU(IDEB+JJ,IBUFHO)
        ENDDO
      ENDIF
    ENDDO

    IROW = 2
    DO JCOL = 1,INGE
      IDEB = (INGW+INRO+JCOL-1)*IORDR
      IDIA = IROW + 3*(JCOL-1)
      IF ( ZWHOU(IDEB+IDIA,IBUFHO) < 0.0_JPRB ) THEN
        DO JJ = 1,IORDR
          ZWHOU(IDEB+JJ,IBUFHO) = -ZWHOU(IDEB+JJ,IBUFHO)
        ENDDO
      ENDIF
    ENDDO

!*        4.4   PRINT SOME FREQUENCIES

    IF (LNMIPR) THEN
      WRITE(NULOUT,441)
      WRITE(NULOUT,442) JM
      WRITE(NULOUT,'('' REAL PART OF FREQUENCIES'')')
      WRITE(NULOUT,'(14(1X,E8.2))')(ZFREQR(J),J=1,IORDR)
      ZTES=ZFREQI(ISAMAX(IORDR,ZFREQI,1))
      IF(ZTES /= 0.0_JPRB)THEN
        WRITE(NULOUT,'('' IMAGINARY PART OF FREQUENCIES'')')
        WRITE(NULOUT,'(14(1X,E8.2))')(ZFREQI(J),J=1,IORDR)
      ENDIF
    ENDIF

    IF (JM <= 10.AND.LNMIPR) THEN
      IPRGW = 10
      IPRGE = 10
      IPRR  = 10
      IF (INGW < IPRGW) IPRGW = INGW
      IF (INGE < IPRGE) IPRGE = INGE
      IF (INRO < IPRR ) IPRR  = INRO

      WRITE(NULOUT,444) SIVP(JK)
      WRITE(NULOUT,445)

!           WESTWARD GRAVITY :

      WRITE(NULOUT,446) (ZFREQR(INGW     +1-J),J=1,IPRGW)

!           EASTWARD GRAVITY   (The first one is the Kelvin mode) :

      WRITE(NULOUT,447) (ZFREQR(INGW+INRO+ J ),J=1,IPRGE)

!           ROSSBY (The first one is the mixed Rossby/grav. mode) :

      WRITE(NULOUT,448) (ZFREQR(INGW     + J ),J=1,IPRR)

    ENDIF

    441 FORMAT(' ')
    442 FORMAT(' FREQUENCIES/(2*OMEGA) FOR ZONAL WAVENUMBER',I3)
    444 FORMAT(' PHIBAR=',G13.5,' (M/SEC)**2')
    445 FORMAT(' FIRST MERIDIONAL (l=n-m) MODES.',&
     & ' KELVIN: the first eastward component ;',&
     & ' MIXED ROSSBY-GRAVITY : the first Rossby component. ' )  
    446 FORMAT(' W. GRAV. : ',10G12.4)
    447 FORMAT(' E. GRAV. : ',10G12.4)
    448 FORMAT(' ROSSBY   : ',10G12.4)

!         --------------------------------------------------------------

!*        5.    INVERT THE MATRIX.
!               ------------------

    DO J=1,IORDR*IORDR
      ZWSPH(J,IBUFSH)=ZWHOU(J,IBUFHO)
    ENDDO

    IF (JPRB == N_DEFAULT_REAL_KIND) THEN

      CALL SGETRF (IORDR,IORDR,ZWSPH(1,IBUFSH),IORDR,IWOR,INFO)
      IF (INFO /= 0) THEN
        WRITE (NULOUT,*) 'SGETRF RETURNS NON-ZERO INFO: ',INFO
        CALL ABOR1 ('SUMODE3E :SGETRF RETURNS NON-ZERO INFO')
      ENDIF

      CALL SGETRI (IORDR,ZWSPH(1,IBUFSH),IORDR,IWOR,ZSCR,SIZE(ZSCR),INFO)
      IF (INFO /= 0) THEN
        WRITE (NULOUT,*) 'SGETRI RETURNS NON-ZERO INFO: ',INFO
        CALL ABOR1 ('SUMODE3E :SGETRI RETURNS NON-ZERO INFO')
      ENDIF

    ELSEIF (JPRB == N_DOUBLE_KIND) THEN

      CALL DGETRF (IORDR,IORDR,ZWSPH(1,IBUFSH),IORDR,IWOR,INFO)
      IF (INFO /= 0) THEN
        WRITE (NULOUT,*) 'DGETRF RETURNS NON-ZERO INFO: ',INFO
        CALL ABOR1 ('SUMODE3E :DGETRF RETURNS NON-ZERO INFO')
      ENDIF

      CALL DGETRI (IORDR,ZWSPH(1,IBUFSH),IORDR,IWOR,ZSCR,SIZE(ZSCR),INFO)
      IF (INFO /= 0) THEN
        WRITE (NULOUT,*) 'DGETRI RETURNS NON-ZERO INFO: ',INFO
        CALL ABOR1 ('SUMODE3E :DGETRI RETURNS NON-ZERO INFO')
      ENDIF

    ELSE
      call ABOR1 ('REAL(KIND=JPRB) is neither default real nor double precision')
    ENDIF

!         --------------------------------------------------------------

!*        6.    TESTING OF THE INVERSION.
!               -------------------------

    IF (JPRB == N_DEFAULT_REAL_KIND) THEN
      CALL SGEMM('N','N',IORDR,IORDR,IORDR,1.0_JPRB,ZWSPH(1,IBUFSH),&
       & IORDR,ZWHOU(1,IBUFHO),IORDR,1.0_JPRB,ZA,IORDR)  
    ELSEIF (JPRB == N_DOUBLE_KIND) THEN
      CALL DGEMM('N','N',IORDR,IORDR,IORDR,1.0_JPRB,ZWSPH(1,IBUFSH),&
       & IORDR,ZWHOU(1,IBUFHO),IORDR,1.0_JPRB,ZA,IORDR)  
    ELSE
      call ABOR1 ('REAL(KIND=JPRB) is neither default real nor double precision')
    ENDIF

    IJ=1
!OCL NOVREC
    DO J=1,IORDR
      ZA(IJ)=ZA(IJ)-1.0_JPRB
      IJ=IJ+IORDR+1
    ENDDO

!         *** MAXIMUM ERROR  ***

    ZERRM=ABS(ZA(ISAMAX(IORDR*IORDR,ZA,1)))

!         *** ERROR IN L2 NORM ***

    IF (JPRB == N_DEFAULT_REAL_KIND) THEN
      ZERRL2=SNRM2(IORDR*IORDR,ZA,1)/REAL(IORDR,JPRB)
    ELSEIF (JPRB == N_DOUBLE_KIND) THEN
      ZERRL2=DNRM2(IORDR*IORDR,ZA,1)/REAL(IORDR,JPRB)
    ELSE
      call ABOR1('SUMODE3E:REAL(KIND=JPRB) is neither dflt real nor double precision')
    ENDIF

    IF (LNMIPR) WRITE(NULOUT,&
     & '('' *** ERROR IN INVERSION OF MATRIX ***  '',&
     & '' MAX : '',E13.7,'' L2 : '',E13.7)') ZERRM,ZERRL2  

!         --------------------------------------------------------------

!*        7.    STORING OF MATRICES, FREQUENCIES AND LTYPxx ARRAYS.
!*              ---------------------------------------------------

!*        7.1   SET UP  "FREQ" AND "FREQ2" ARRAYS (YOMNMI) :
!*              "FREQ" AND "FREQ2" ARE STORED IN ROUTINE GATHERFREQ

    DO J=1,IORDR
      ZZFREQ (JK,ICFRQ+J)=ZFREQR(J)
    ENDDO

!*        7.2   WRITE (ZWSPH) and (ZWHOU) ON "NSPHMO" and "NHOUMO" :

    ILEN=IORDR*IORDR
    IRLEN=((IORDR*IORDR-1)/512+1)*512
    DO J=1,ILEN
      SPHBUF(NSTASPHB(IREC)+J)=ZWSPH(J,1)
      HOUBUF(NSTAHOUB(IREC)+J)=ZWHOU(J,1)
    ENDDO

!*        7.3   SET UP COUNTERS FOR NEXT ZONAL WAVENUMBER "JM" :

    ICFRQ  = ICFRQ  + IORDR
    ICFREQ = ICFREQ + IORDR*2

!         ** END OF MAIN LOOP OVER M **
!         **                         **
!         *****************************

  ENDDO WAVE_LOOP

!       *** END OF LOOP OVER VERTICAL MODES ***
!       ***                                 ***
!       ***************************************

ENDDO MODES_LOOP

IF (LNMIPR) WRITE(NULOUT,&
 & '('' LOCAL VALUES OF ICFREQ= '',I6,'' IFREQ= '',I6)') ICFREQ,&
 & IFREQ  

IF(ICFREQ /= IFREQ) CALL ABOR1('SUMODE3E: ICFREQ WRONG')
IF(IFLAG == 0) CALL ABOR1('SUMODE3E: ABOR1 CALLED')

!*    8.1  GATHER FREQ AND FREQ2 CONTRIBUTIONS FROM OTHER PE'S. ONLY THE LOCAL
!*         WAVE NUMBERS ARE CALCULATED ON EACH PE. GLOBAL VERSIONS ARE KEPT ON
!          EACH PE IN ORDER TO KEEP THE NMI CODE SIMPLER.

CALL GATHERFREQ(ZZFREQ,FREQ,KVIMOD,ICFRQ,ICFREQG)

IF (LNMIPR) WRITE(NULOUT,&
 & '('' GLOBAL VALUES OF ICFREQ= '',I6,'' IFREQ= '',I6)') ICFREQG,&
 & IFREQG  
IF(ICFREQG /= IFREQG) CALL ABOR1('SUMODE3E: ICFREQG WRONG')

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUMODE3E',1,ZHOOK_HANDLE)
END SUBROUTINE SUMODE3E
