SUBROUTINE BIASCOR (LD_LBC,K_NST,P_SPP,P_ST,P_STNBC,&
 & CD_CIDENT,K_IM,K_ID,K_IH,P_RLAT,P_RLONG,K_IRSTYP,&
 & K_IMISS,P_RMISS)  

!**** *BIASCOR*

!       PURPOSE.
!      ----------

!          BIAS CORRECTION FOR TEMPERATURES

!       INTERFACE.
!      ------------

!          CALL BIASCOR (LBC,NST,SPP,ST,STNBC,
!     X                  CIDENT,IM,ID,IH,RLAT,RLONG,IRSTYP,
!     X                  IMISS,RMISS)

!        INPUT
!         NST       NUMBER OF LEVELS
!         SPP       ARRAY WITH PRESSURE VALUES
!         ST        ARRAY WITH T VALUES
!         CIDENT    STATION IDENTIFIER
!         IM        MONTH
!         ID        DAY
!         IH        HOUR
!         RLAT      LATITUDE
!         RLONG     LONGITUDE
!         IRSTYP    RADIOSONDE TYPE (BUFR CODE TABLE 002011)
!         IMISS     MISSING VALUE FOR INTEGERS
!         RMISS     MISSING VALUE FOR REALS

!        OUTPUT
!         LBC       LOGICAL TO STATE IF BIAS COR. WAS SUCCESSFUL
!         STNBC     ARRAY WITH BIAS CORRECTED T VALUES

!       METHOD.
!      ---------

!        READ BIAS CORRECTION TABLES:
!            TABLE 1 : CROSS-REF STATION ID, SONDE NAME
!            TABLE 2 : ARRAY OF CORRECTIONS DEPENDING ON
!                      SOLAR ELEVATION AND PRESSURE LEVEL
!            TABLE 3 : CROSS-REF SONDE TYPE CODE, SONDE NAME
!        TRY FOR MATCH IN TABLE 3, IF NOT THEN TABLE 1
!        DETERMINE CORRECTION TO BE APPLIED FROM TABLE 2

!       EXTERNALS.
!      ------------

!        DIURNAL          CALCULATE SOLAR ELEVATION
!        PNTERP           INTERPOLATE TO PRESSURE LEVEL

!       REFERENCE.
!      ------------

!        RADIOSONDE BIAS CORRECTION
!        OD MEMORANDUM BY B. STRAUSS  22.12.92

!       AUTHOR.
!      ---------

!        B. NORRIS       JULY  1991   APPLY CORRECTIONS TO AOF

!       MODIFICATIONS.
!      ----------------

!        M. DRAGOSAVAC   APRIL 1993   IMPLEMENT IN PRE-PROCESSING
!        B. NORRIS       JULY  1998   INTERFACE TO DATA ASSIMILATION
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning (attempt)
!        S.Saarinen    16-Dec-2005 CY30R1+ OMP FLUSH(CL_CHECK)'s added

!-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMCT0B   ,ONLY : LECMWF
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN    ,ONLY : NULERR,  NULOUT
USE OML_MOD, ONLY : OML_SET_LOCK, OML_UNSET_LOCK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: K_NST 
LOGICAL           ,INTENT(OUT)   :: LD_LBC 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_SPP(K_NST) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_ST(K_NST) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_STNBC(K_NST) 
CHARACTER(LEN=*)  ,INTENT(IN)    :: CD_CIDENT 
INTEGER(KIND=JPIM),INTENT(IN)    :: K_IM 
INTEGER(KIND=JPIM),INTENT(IN)    :: K_ID 
INTEGER(KIND=JPIM),INTENT(IN)    :: K_IH 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_RLAT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_RLONG 
INTEGER(KIND=JPIM),INTENT(IN)    :: K_IRSTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: K_IMISS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_RMISS 
INTEGER(KIND=JPIM) :: I_NCORBC
INTEGER(KIND=JPIM) :: I_NLEVBC
INTEGER(KIND=JPIM) :: I_NSONBC
INTEGER(KIND=JPIM) :: I_NSSNBC
INTEGER(KIND=JPIM) :: I_NSTNBC

PARAMETER (I_NLEVBC=16,I_NCORBC=13)
PARAMETER (I_NSTNBC=2000,I_NSONBC=20,I_NSSNBC=300)

REAL(KIND=JPRB) ::Z_RCORBC(I_NCORBC,I_NLEVBC,I_NSONBC)
INTEGER(KIND=JPIM) :: ILEVBC(I_NLEVBC,I_NSONBC),ISSTYBC(I_NSSNBC)
CHARACTER (LEN = 5) ::   CL_YSTIDBC(I_NSTNBC),CL_YDUMMYBC
CHARACTER (LEN = 16) ::  CL_YSTYPBC(I_NSTNBC),CL_YSNAMBC(I_NSONBC),CL_YSSTYBC(I_NSSNBC)
CHARACTER (LEN = 16) ::  CL_YSNAMCT
CHARACTER (LEN = 16) ::   CL_CHECK
CHARACTER (LEN = 256) ::  CL_CF1,CL_CPPBASE
CHARACTER (LEN = 1) ::    CL_CFILE
CHARACTER (LEN = 8) ::    CLPUINT
LOGICAL ::LL_LNAMFND, LLPRINT
DATA CL_CHECK/'FILES NOT READ'/
!DATA cppbase/'/ws/home/mo/mom/public/bias_correction/table'/
DATA CL_CPPBASE/'table'/
SAVE CL_CHECK
SAVE I_NSTBC,CL_YSTIDBC,CL_YSTYPBC
SAVE I_NSOBC,CL_YSNAMBC,ILEVBC,Z_RCORBC
SAVE I_NSSBC,ISSTYBC,CL_YSSTYBC

REAL(KIND=JPRB) :: PP, Z_POS, Z_RADD, Z_ANGLEBC

INTEGER(KIND=JPIM) :: I, icf1, ICOL, icpp, IHGH, ILOW,&
 & IOS, ISON, IUNIT, J, I_K, I_KUNIT, I_NSOBC, &
 & I_NSSBC, I_NSTBC  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "diurnal.intfb.h"
#include "pnterp.intfb.h"

!  -------------------------------------------------------------------

!            1.  OPEN AND READ TABLE FILES
!           -------------------------------

IF (LHOOK) CALL DR_HOOK('BIASCOR',0,ZHOOK_HANDLE)

LD_LBC    = .FALSE.
LL_LNAMFND= .FALSE.
LLPRINT   = .TRUE.

IF(CL_CHECK /= 'FILES READ') THEN ! outer
!$ CALL OML_SET_LOCK()
!$OMP FLUSH(CL_CHECK)

 IF(CL_CHECK /= 'FILES READ') THEN
  icpp=index(CL_CPPBASE,' ')
  icpp=icpp-1

!                1.1  OPEN TABLES

  DO I_KUNIT=1,3
    IUNIT=I_KUNIT+64
    WRITE(CL_CFILE,'(I1)') I_KUNIT
    CL_CF1=' '
!           cf1=cppbase(1:icpp)//'/dat/biascor'//CFILE//'.tab'
!   cf1=cppbase(1:icpp)//'/table'//CFILE
    CL_CF1=CL_CPPBASE(1:icpp)//CL_CFILE
    icf1=index(CL_CF1,' ')
    icf1=icf1-1

    OPEN(UNIT=IUNIT, FILE=CL_CF1(1:icf1),IOSTAT=IOS,ERR=114,STATUS='OLD')
    GO TO 116
114 CONTINUE
    WRITE(NULERR,'(A/1X,A/A,I5)') ' BIAS CORRECTION ERROR: FILE: ',&
     & CL_CF1(1:icf1),&
     & ' OPEN FAILED, IOSTAT = ',IOS  
    CALL ABOR1('SUBROUTINE BIASCOR')
116 CONTINUE
  ENDDO

!              1.2  READ TABLE 1 : STATION LIST & SONDE TYPE

  I_NSTBC=0
  122 CONTINUE
  IF (I_NSTBC < I_NSTNBC) THEN
    I_NSTBC=I_NSTBC+1
    READ (65,'(A,1X,A)',END=124)CL_YSTIDBC(I_NSTBC),CL_YSTYPBC(I_NSTBC)
    GO TO 122
  ELSE
    WRITE (NULERR,'(1X,A,I5)')&
     & ' BIAS CORRECTION ERROR: DIMENSION NSTN TOO SMALL :',&
     & I_NSTNBC  
    CALL ABOR1('SUBROUTINE BIASCOR')
  ENDIF
  124 CONTINUE
  I_NSTBC=I_NSTBC-1
  WRITE (NULOUT,*)' BIAS CORRECTION: NUMBER OF STATIONS IN TABLE 1 ',I_NSTBC

!              1.3  READ TABLE 2 : SONDE CORRECTIONS

  I_NSOBC=0
  132 CONTINUE
  IF(I_NSOBC < I_NSONBC) THEN
    I_NSOBC=I_NSOBC+1
    READ (66,'(A)',END=138) CL_YSNAMBC(I_NSOBC)
    READ (66,'(A)',END=138) CL_YDUMMYBC
    DO J=1,I_NLEVBC
      READ (66,'(I5,8F8.2/5X,8F8.2)',END=138)&
       & ILEVBC(J,I_NSOBC),(Z_RCORBC(I_K,J,I_NSOBC),I_K=1,I_NCORBC)  
    ENDDO
    READ (66,'(A)',END=138) CL_YDUMMYBC
    GO TO 132
  ELSE
    WRITE (NULERR,'(1X,A,I5)')&
     & ' BIAS CORRECTION ERROR: DIMENSION NSON TOO SMALL :',&
     & I_NSONBC  
    CALL ABOR1('SUBROUTINE BIASCOR')
  ENDIF
  138 CONTINUE
  I_NSOBC=I_NSOBC-1
  WRITE (NULOUT,*)' BIAS CORRECTION: NUMBER OF SONDES IN TABLE 2 ',I_NSOBC

!              1.4  READ TABLE 3 : SONDE TYPE CODE & SONDE TYPE

  I_NSSBC=0
  142 CONTINUE
  IF (I_NSSBC < I_NSSNBC) THEN
    I_NSSBC=I_NSSBC+1
    READ (67,'(I3,1X,A)',END=144)ISSTYBC(I_NSSBC),CL_YSSTYBC(I_NSSBC)
    GO TO 142
  ELSE
    WRITE (NULERR,'(1X,A,I5)')&
     & ' BIAS CORRECTION ERROR: DIMENSION NSSN TOO SMALL :',&
     & I_NSSNBC  
    CALL ABOR1('SUBROUTINE BIASCOR')
  ENDIF
  144 CONTINUE
  I_NSSBC=I_NSSBC-1
  WRITE (NULOUT,*)' BIAS CORRECTION: NUMBER OF SONDES IN TABLE 3 ',I_NSSBC

!                CLOSE FILES

  CLOSE(65)
  CLOSE(66)
  CLOSE(67)
  CL_CHECK='FILES READ'
!$OMP FLUSH(CL_CHECK)
 ENDIF

!$ CALL OML_UNSET_LOCK()
ENDIF ! IF(CL_CHECK /= 'FILES READ') THEN ! outer

!            1.5  INITIALLY SET CORRECTED = ORIGINAL

DO I=1,K_NST
  P_STNBC(I)=P_ST(I)
ENDDO

!  ------------------------------------------------------------------

!                  2.  DETERMINE CORRECT ELEMENTS OF TABLES
!                 -------------------------------------------

!            2.1  DETERMINE SONDE TYPE FROM TABLE 3

IF(LLPRINT) THEN
  WRITE(NULOUT,'('' BIAS CORRECTION INPUT: '',A,1X,3(I5,1X),2(F9.2,1X),I11)') &
                & CD_CIDENT,K_IM,K_ID,K_IH,P_RLAT,P_RLONG,K_IRSTYP
  CALL FLUSH(NULOUT)
ENDIF

IF(ABS(K_IRSTYP) == K_IMISS)  THEN
  IF(LLPRINT) THEN
    WRITE(NULOUT,'('' SONDE TYPE MISSING TRY TABLE 1'')')
    CALL FLUSH(NULOUT)
  ENDIF
  GO TO 214
ENDIF
DO I=1,I_NSSBC
  IF(K_IRSTYP == ISSTYBC(I)) THEN
    CL_YSNAMCT=CL_YSSTYBC(I)
    IF(CL_YSNAMCT /= 'nobias') THEN
      LL_LNAMFND=.TRUE.
      IF(LLPRINT) THEN
        WRITE(NULOUT,'('' SONDE TYPE FROM TABLE 3 FOUND:'',I5,1X,A,1X,I11,1X,L1)') &
         & I,CL_YSNAMCT,K_IRSTYP,LL_LNAMFND
        CALL FLUSH(NULOUT)
      ENDIF
    ENDIF
    GO TO 224
  ENDIF
ENDDO
IF(LLPRINT) THEN
  WRITE(NULOUT,'('' SONDE TYPE NOT FOUND TRY TABLE 1'')')
  CALL FLUSH(NULOUT)
ENDIF
214 CONTINUE



!            2.2  DETERMINE SONDE TYPE FROM TABLE 1

IF(LECMWF) THEN
  DO I=1,I_NSTBC
    IF(CD_CIDENT == CL_YSTIDBC(I)) THEN
      CL_YSNAMCT=CL_YSTYPBC(I)
      IF(LLPRINT) THEN
        WRITE(NULOUT,'('' SONDE TYPE FROM TABLE 1 FOUND:'',I5,1X,A,1X,A  )') &
         & I,CL_YSNAMCT,CD_CIDENT
        CALL FLUSH(NULOUT)
      ENDIF
      LL_LNAMFND=.TRUE.
      GO TO 224
    ENDIF
  ENDDO
ELSE
  DO I=1,I_NSTBC
    IF(CD_CIDENT(1:5) == CL_YSTIDBC(I)(1:5)) THEN
      CL_YSNAMCT=CL_YSTYPBC(I)
      WRITE (NULOUT,*) ' SONDE NAME FROM TABLE 1 : ',CL_YSNAMCT,CD_CIDENT(1:5)
      LL_LNAMFND=.TRUE.
      EXIT
    ENDIF
  ENDDO
ENDIF

224 CONTINUE


IF(.NOT.LL_LNAMFND) THEN
  WRITE(NULOUT,'('' BIAS CORRECTION NOT DONE '')')
  GO TO 400
ENDIF

!           2.3  DETERMINE SECTION OF TABLE 2

DO I=1,I_NSOBC
  IF(CL_YSNAMCT == CL_YSNAMBC(I)) THEN
    ISON=I
    IF(LLPRINT) THEN
      WRITE(NULOUT,'('' SECTION OF TABLE 2 :'',I10)') ISON
      CALL FLUSH(NULOUT)
    ENDIF
    GO TO 234
  ENDIF
ENDDO
GO TO 400
234 CONTINUE

!           2.4  CALCULATE SOLAR ELEVATION

CALL DIURNAL ( K_IM, K_ID, K_IH, P_RLAT,P_RLONG, Z_ANGLEBC )
IF(LLPRINT) THEN
  WRITE(NULOUT,'('' SOLAR ELEVATION:'',F10.3)') Z_ANGLEBC
  CALL FLUSH(NULOUT)
ENDIF


!           2.5  CALCULATE CORRECT COLUMN OF TABLE 2

ICOL=NINT(Z_ANGLEBC/5.0_JPRB)+3
IF(ICOL <        1) ICOL = 1
IF(ICOL > I_NCORBC) ICOL = I_NCORBC
IF(LLPRINT) THEN
  WRITE(NULOUT,'('' TABLE 2 COLUMN: '',I10)') ICOL
  CALL FLUSH(NULOUT)
ENDIF

!           2.6  LOOP OVER LEVELS

DO I=1,K_NST
  IF(P_SPP(I) /= P_RMISS.AND.P_ST(I) /= P_RMISS) THEN

!           2.7  CALCULATE CORRECT ROW OF TABLE 2

    PP      =P_SPP(I) / 100.0_JPRB
    CLPUINT = 'hPa'
    CALL PNTERP (ILEVBC(1,ISON),I_NLEVBC,PP,Z_POS,CLPUINT)
    ILOW = Z_POS
    IHGH = ILOW + 1

!  -------------------------------------------------------------

!                3.  BIAS CORRECTION
!               ---------------------

    IF(LLPRINT) THEN
      WRITE(NULOUT,'('' BIAS CORRECTION IN:'',&
       & 4(F12.9,1X))') &
       & Z_RCORBC(ICOL,ILOW,ISON),(REAL(IHGH)-Z_POS),&
       & Z_RCORBC(ICOL,IHGH,ISON),(Z_POS-REAL(ILOW))
      WRITE(NULOUT,'('' BIAS CORRECTION IN:'',&
       & 2(F12.9,1X))') &
       & Z_RCORBC(ICOL,ILOW,ISON),Z_RCORBC(ICOL,IHGH,ISON)
      WRITE(NULOUT,'('' BIAS CORRECTION IN:'',&
       & 6(F12.9,1X))') &
       & REAL(IHGH),Z_POS,(REAL(IHGH)-Z_POS),&
       & REAL(ILOW),Z_POS,(Z_POS-REAL(ILOW))
      CALL FLUSH(NULOUT)
    ENDIF
    Z_RADD=Z_RCORBC(ICOL,ILOW,ISON)*(REAL(IHGH)-Z_POS)+&
     & Z_RCORBC(ICOL,IHGH,ISON)*(Z_POS-REAL(ILOW))  
    P_STNBC(I)=P_ST(I)+Z_RADD
    IF(LLPRINT) THEN
      WRITE(NULOUT,'('' BIAS CORRECTION RESULT:'',&
       & F10.2,1X,3(I5,1x),F12.9,1X,2(F10.2,1X))') &
       & PP*100.,ICOL,ILOW,IHGH,Z_RADD,P_ST(I),P_STNBC(I)
      CALL FLUSH(NULOUT)
    ENDIF
  ENDIF
ENDDO
LD_LBC=.TRUE.

!--------------------------------------------------------------------

!                   4.  EXIT
!                  -----------

400 CONTINUE
IF (LHOOK) CALL DR_HOOK('BIASCOR',1,ZHOOK_HANDLE)
END SUBROUTINE BIASCOR
