SUBROUTINE FIRST(KSET)

!**** SUBROUTINE FIRST - CONTROL FIRST GUESS CHECK DECISIONS

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        CONTROL FIRST GUESS CHECK DECISIONS

!**   INTERFACE
!     ---------

!     ***CALL*FIRST(KSET)
!     WHERE KSET = NUMBER OF THE OBSERVATION SET

!        FIRST IS CALLED BY SUBROUTINE DECIS

!*    METHOD
!     ------

!        FIRST GUESS CHECK IS PERFORMED DIFFERENTLY FOR WIND OBSERVATIONS
!     AND FOR ALL THE OTHER OBSERVED VARIABLES. OBSERVATION TYPES ARE
!     TREATED SOMEWHAT DIFFERENTLY INSIDE THE SUBROUTINES FGWND AND FGCHK.

!*    EXTERNALS
!     ---------

!        FGWND  - FIRST GUESS CHECK FOR U/V/U10/V10M
!        FGCHK  - FIRST GUESS CHECK FOR Z/PS/DZ/T/T2M/Q/RH/RH2M

!**   MODIFICATIONS
!     -------------
!     Erik Andersson 980612: BgQC for radiances
!     Erik Andersson 981204: BgQC for RTCWV and TCO3
!     Sean Healy     041027: BgQC for GPSRO  
!     Niels Bormann  041223: BgQC for limb radiances  
!     Antje Dethof   05-May-05: BgQC for GRG variables
!     Richard Engelen 20-Jun-06: BgQC for GHG variables
!     Sean Healy     23-Aug-06: Reintroduced FGCHECK for GPSRO (NVNUMB(76))
!     David Tan      070302: BgQC for Aeolus HLoS (Doppler wind lidar)

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARDIMO  , ONLY : JPNUMV
USE YOMLUN   , ONLY : NULOUT
USE YOMOBSET_THSAFE
USE YOMVNMB  , ONLY : NVNUMB
USE YOMSCC   , ONLY : LSCDPR
USE YOMDB
USE OML_MOD, ONLY : OML_MY_THREAD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
INTEGER(KIND=JPIM) :: IVRBLS(JPNUMV)

INTEGER(KIND=JPIM) :: ICMVNM,&
 & ILEV, IOBTYP, JNLV, JOBS, JV, ITID, IGRP

LOGICAL :: LLNOTFND
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fgchk.intfb.h"
#include "fgwnd.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------

!*          1.0   INFORM

IF (LHOOK) CALL DR_HOOK('FIRST',0,ZHOOK_HANDLE)
ITID = OML_MY_THREAD()
IF(LSCDPR) WRITE(NULOUT,'(''     FIRST GUESS CHECK '')')

!*          1.1   OBSERVATION TYPE

IOBTYP  = MOD(OBSET(ITID)%MTYPOB(KSET),100)
IGRP    = MOD(OBSET(ITID)%MTYPOB(KSET)/100,100)
LLNOTFND = .FALSE.
IVRBLS(:) = 0
!*
!     ------------------------------------------------------------------

!        2.       LOOP OVER THE SET, REPORT BY REPORT
!                 -----------------------------------

OBS_LOOP: DO JOBS = 1,OBSET(ITID)%MLNSET(KSET)


!*          2.1   LOOP OVER THE REPORT

  DAT_LOOP: DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
    ILEV = JNLV

!*          2.2   WHICH VARIABLE

    ICMVNM = ROBODY(ILEV,MDBVNM)
!*
!     ------------------------------------------------------------------

!        3.       F.G. CHECK FOR DIFFERENT VARIABLES
!                 ----------------------------------

!*          3.1   FIRST GUESS CHECK FOR U/V, U10M/V10M AND AMBIG U/V
!                 - BOTH COMPONENTS ARE CHECKED AT THE SAME TIME

    IF(ICMVNM == NVNUMB( 1) .OR. ICMVNM == NVNUMB(56) .OR.&
       & ICMVNM == NVNUMB(62)) THEN  
      CALL FGWND(KSET,JOBS,ILEV)
    ELSEIF(ICMVNM == NVNUMB( 2) .OR. ICMVNM == NVNUMB(57) .OR.&
       & ICMVNM == NVNUMB(61)) THEN  
      CONTINUE
!*
!           3.2   FIRST GUESS CHECK FOR Z/PS/DZ/T/T2M/Q/RH/RH2M/RAD/
!                                       GPSRO/LRA/GRG(JPGRG)/GHG/APD/RFL
!                                       /APD/HLoS/RAO/RFA/ODA/DOW

    ELSEIF(ANY(ICMVNM ==&
       & (/NVNUMB( 3),NVNUMB(47),NVNUMB( 4),NVNUMB( 8)&
       & ,NVNUMB(10),NVNUMB(63),NVNUMB( 5),NVNUMB( 7)&
       & ,NVNUMB(50),NVNUMB(68),NVNUMB( 6),NVNUMB(74),NVNUMB(75),NVNUMB(76),NVNUMB(77)&
       & ,NVNUMB(78),NVNUMB(79),NVNUMB(80),NVNUMB(82),NVNUMB(83)&
       & ,NVNUMB(84),NVNUMB(85),NVNUMB(86),NVNUMB(87),NVNUMB(88)&
       & ,NVNUMB(93),NVNUMB(94),NVNUMB(95)/))) THEN  

      CALL FGCHK(JOBS,ILEV,IGRP)
      CALL FGCHK(JOBS,ILEV,IGRP)
!*
!           3.3   VARIABLE MISSING FROM FIRST GUESS CHECK

    ELSE
      DO JV = 1, JPNUMV
        IF(NVNUMB(JV) == ICMVNM) THEN
          IVRBLS(JV) = 1
          LLNOTFND = .TRUE.
        ENDIF
      ENDDO
    ENDIF
!*
!     ------------------------------------------------------------------

!        4.       CLOSE THE LOOP ...
!                 ------------------

!*          4.1   ... OVER THE REPORT

  ENDDO DAT_LOOP

!*          4.2   ... OVER THE CMA

ENDDO OBS_LOOP

!*          4.3   PRINT LIST OF VARIABLES MISSING FROM F.G. CHECK

IF(LLNOTFND) THEN
  WRITE(NULOUT,*) ' IN FIRST: NO BGQC FOR VARIABLES: '
  DO JV = 1, JPNUMV
    IF(IVRBLS(JV) /= 0) WRITE(NULOUT,*) '       - ', NVNUMB(JV)
  ENDDO
ENDIF
!*
!     ------------------------------------------------------------------

!        5.       RETURN
!                 ------
IF (LHOOK) CALL DR_HOOK('FIRST',1,ZHOOK_HANDLE)
END SUBROUTINE FIRST
