SUBROUTINE PNTERP (KLEVLIST,KLEV,PP,POS,CDPUNIT)

!**** *PNTERP*

!       PURPOSE.
!      ----------

!           INTERPOLATE POSITION OF PRESSURE OR HEIGHT
!               WITHIN TABLE OF UPPER AIR LEVELS

!       INTERFACE.
!      ------------

!         CALL PNTERP (KLEVLIST,KLEV,PP,POS,CDPUNIT)

!        INPUT
!         KLEVLIST   - TABLE OF STANDARD PRESSURE OR HEIGHT LEVELS
!         KLEV       - DIMENSION OF KLEVLIST
!         PP         - PRESSURE LEVEL
!         CDPUNIT    - PRESSURE UNIT

!        OUTPUT
!         POS        - INTERPOLATED POSITION WITHIN TABLE

!       METHOD.
!      ---------

!           IF VALUES OF KLEVLIST INCREASE, HEIGHT ASSUMED, LINEAR IN Z
!           IF VALUES OF KLEVLIST DECREASE, PRESSURE ASSUMED, LINEAR IN LOGP

!       EXTERNALS.
!      -----------

!         NONE

!       REFERENCE.
!      ------------

!         NONE

!       AUTHOR.
!      ---------

!         B. NORRIS,  ECMWF,  JANUARY 1989.

!       MODIFICATIONS.
!      ----------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!         NONE

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVLIST(KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PP 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POS 
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDPUNIT 
REAL(KIND=JPRB) :: Z_EPS, ZP, ZPUNIT

INTEGER(KIND=JPIM) :: ILEV(KLEV)
LOGICAL :: LLDOWN
DATA Z_EPS/0.000001_JPRB/

INTEGER(KIND=JPIM) :: JLEV, JPOS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!    -------------------------------------------------------------------

!                       1.  SETUP
!                           -----

IF (LHOOK) CALL DR_HOOK('PNTERP',0,ZHOOK_HANDLE)
100 CONTINUE

!         1.0   PRESSURE unit

ZPUNIT  = 1.0_JPRB
IF(CDPUNIT == 'Pa') ZPUNIT = 100.0_JPRB

!         1.1   CHECK IF LEVEL VALUSE INCREASE/DECREASE

LLDOWN=.FALSE.
IF(KLEVLIST(1) > KLEVLIST(KLEV)) LLDOWN=.TRUE.

!         1.1.1 PRESSURE LEVELS

ZP      = PP          / ZPUNIT
ILEV(:) = KLEVLIST(:) / ZPUNIT


!    -------------------------------------------------------------------

!           2.  CHECK WHETHER VALUE IS OUTSIDE TABLE LIMITS
!               -------------------------------------------

200 CONTINUE
IF(LLDOWN) THEN
  IF(ZP >= REAL(ILEV(1))) THEN
    POS=1.0_JPRB+Z_EPS
    GO TO 400
  ENDIF
  IF(ZP <= REAL(ILEV(KLEV))) THEN
    POS=REAL(KLEV)-Z_EPS
    GO TO 400
  ENDIF
ELSE
  IF(ZP <= REAL(ILEV(1))) THEN
    POS=1.0_JPRB+Z_EPS
    GO TO 400
  ENDIF
  IF(ZP >= REAL(ILEV(KLEV))) THEN
    POS=REAL(KLEV)-Z_EPS
    GO TO 400
  ENDIF
ENDIF

!    ------------------------------------------------------------------

!          3.   VALUE IS BETWEEN MAX AND MIN TABLE VALUES
!               -----------------------------------------

300 CONTINUE

!         3.2   VALUE IS BETWEEN 2 TABLE VALUES

DO JLEV=1,KLEV-1
  IF(LLDOWN) THEN
    IF(ZP <= REAL(ILEV(JLEV)).AND.ZP >= REAL(ILEV(JLEV+1)))THEN
      JPOS=JLEV
      GO TO 324
    ENDIF
  ELSE
    IF(ZP >= REAL(ILEV(JLEV)).AND.ZP <= REAL(ILEV(JLEV+1)))THEN
      JPOS=JLEV
      GO TO 324
    ENDIF
  ENDIF
ENDDO
324 CONTINUE

!           3.3   INTERPOLATE

IF(LLDOWN) THEN
  POS=REAL(JPOS)+&
   & (LOG(REAL(ILEV(JPOS)))-LOG(ZP))/&
   & (LOG(REAL(ILEV(JPOS)))-LOG(REAL(ILEV(JPOS+1))))  
ELSE
  POS=REAL(JPOS)+&
   & (REAL(ILEV(JPOS))-ZP)/&
   & (REAL(ILEV(JPOS))-REAL(ILEV(JPOS+1)))  
ENDIF

!    ------------------------------------------------------------------

!                   4.   EXIT
!                        ----

400 CONTINUE
IF (LHOOK) CALL DR_HOOK('PNTERP',1,ZHOOK_HANDLE)
END SUBROUTINE PNTERP
