SUBROUTINE PRECH(KSET)

!**** SUBROUTINE PRECH - PRELIMINARY CHECKS OF DATUMS/REPORTS ARE PERFORMED

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        SEVERAL PRELIMINARY DATA CHECK ARE PERFORMED. REPORTS/DATUMS ARE
!     REJECTED ACCORDINGLY.
!        - CHECK THE REPORT DATA BASE FLAGS FOR REPORT/DATUM
!        - ASSIGN THE MODEL OROGRAPHY TO THE REPORT HEADER
!        - CHECK MISSING STATION ALTITUDE
!        - CHECK NUMBER OF BODY ENTRIES (IF EMPTY REPORT)
!        - CHECK THAT THERE IS A VERTICAL COORDINATE
!        - CHECK FOR CORRECT REPORTING PRACTICE (SYNOP/TEMP/PILOT)
!        - INCREASE OBSERVATION ERROR DUE TO EXTRAPOLATION (SYNOP
!          GEOPOTENTIAL AND SURFACE PRESSURE)

!**   INTERFACE
!     ---------

!     ***CALL*PRECH(KSET)
!     WHERE KSET = ECSET

!        PRECH IS CALLED BY SUBROUTINE DECIS

!*    METHOD
!     ------

!*    EXTERNALS
!     ---------

!        REPRA - CHECK REPORTING PRACTICE FOR SYNOP/TEMP/PILOT
!        ADDOER- INCRESE OBSERVATION ERROR FOR SYNOP GEOPOTENTIAL

!**   MODIFICATIONS
!     -------------

!     ALAN GEER     02/SEP/2008  UNIVERSALLY STORE LSM IN THE HEADER

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMCST   , ONLY : RG
USE YOMDIM   , ONLY : NFLEVG
USE YOMOBS   , ONLY : LMFSCRE
USE YOMOBSET_THSAFE
USE YOMCOCTP , ONLY : NSYNOP   ,NTEMP    ,NPILOT   ,NSHSCD   ,&
 & NATSHS  
USE YOMANCS  , ONLY : RMDI
USE YOMVNMB  , ONLY : NVNUMB
USE YOMNMEV  , ONLY : NR1EVENT ,ND1EVENT
USE YOMSCC   , ONLY : LSCDPR
USE GOMS_MIX     , ONLY : YGOMEC,YGOMS,YGOMUA
USE YOMDB
USE OML_MOD, ONLY : OML_MY_THREAD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
INTEGER(KIND=JPIM) :: ICDTYP, &
 & ILEV, INLV, IOBTYP, IPOSM, IVAR, &
 & JNLV, JOBS, ITID

REAL(KIND=JPRB) :: ZCMFOE, ZCMOER, ZCMOMF, ZCMPPP, ZCMVAR, ZSTALT
REAL(KIND=JPRB) :: ZTFG, ZMPS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "addoer.intfb.h"
#include "repra.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------

!*          1.0   OBSERVATION TYPE

IF (LHOOK) CALL DR_HOOK('PRECH',0,ZHOOK_HANDLE)
ITID = OML_MY_THREAD()
IOBTYP = MOD(OBSET(ITID)%MTYPOB(KSET),100)

!*          1.1   INFORM

IF(LSCDPR) WRITE(NULOUT,'(''   PRELIMINARY CHECK OF OBS TYPE:'',I10)') IOBTYP

!*
!     ------------------------------------------------------------------

!        2.       LOOP OVER THE SET, REPORT BY REPORT
!                 -----------------------------------

OBS_LOOP: DO JOBS = 1, OBSET(ITID)%MLNSET(KSET)

!*          2.1   FIND OUT OBSERVATION POINTER, ETC.

  IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
  ICDTYP = ROBHDR(JOBS,MDB_CODETYPE_AT_HDR)

!*
!           2.2   ASSIGN THE MODEL OROGRAPHY AND LAND/SEA MASK TO THE REPORT HEADER

  ROBHDR(JOBS,MDB_OROGRAPHY)  = YGOMEC(IPOSM)%ECOROG/RG
  ROBHDR(JOBS,MDB_LSM) = YGOMEC(IPOSM)%ECLSMA

!*
!           2.3   CHECK FOR STATION ALTITUDE IN SYNOP/TEMP/PILOT REPORTS

  ZSTALT = ROBHDR(JOBS,MDBALT)
  IF(ZSTALT == RMDI) THEN
    IF(IOBTYP == NSYNOP .OR.IOBTYP == NTEMP  .OR.IOBTYP == NPILOT     ) THEN
      IF ( IOBTYP == NSYNOP .AND. LMFSCRE(NSYNOP) .AND.&
         & ( ICDTYP == NSHSCD .OR. ICDTYP == NATSHS )   ) THEN  
!             no rejection
      ELSE
        ROBHDR(JOBS,MDBRST) = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
        ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT(13))
      ENDIF
    ENDIF
  ENDIF
!*
!           2.4   CHECK FOR NUMBER OF BODY ENTRIES (IF EMPTY REPORT)

  INLV = MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
  IF(INLV == 0) THEN
    ROBHDR(JOBS,MDBRST) = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
    ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT( 1))
    CYCLE OBS_LOOP
  ENDIF
!*
!     ------------------------------------------------------------------

!        3.       LOOP OVER THE REPORT, DATUM BY DATUM
!                 ------------------------------------

!*          3.1   START THE LOOP

  DATUM_LOOP: DO JNLV = MLNKH2B(JOBS),MLNKH2B(JOBS+1)-1
    ILEV = JNLV
!*
!           3.2   CHECK THAT THERE IS A VERTICAL COORDINATE

    ZCMPPP = ROBODY(ILEV,MDBPPP)
    IF(ZCMPPP == RMDI) THEN
      ROBODY(ILEV,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(ILEV,MDBDSTA))
      ROBODY(ILEV,MDBDEV1) = ZCHEV1( ROBODY(ILEV,MDBDEV1),ND1EVENT( 1))
    ENDIF
!*
!           3.3   CHECK THAT THERE IS AN OBSERVED VARIABLE

    ZCMVAR = ROBODY(ILEV,MDBVAR)
    IF(ZCMVAR == RMDI) THEN
      ROBODY(ILEV,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(ILEV,MDBDSTA))
      ROBODY(ILEV,MDBDEV1) = ZCHEV1( ROBODY(ILEV,MDBDEV1),ND1EVENT( 2))
    ENDIF
!*
!           3.4   CHECK THAT THERE IS A FIRST GUESS DEPARTURE

    ZCMOMF = ROBODY(ILEV,MDBOMF)
    IF(ZCMOMF == RMDI) THEN
      ROBODY(ILEV,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(ILEV,MDBDSTA))
      ROBODY(ILEV,MDBDEV1) = ZCHEV1( ROBODY(ILEV,MDBDEV1),ND1EVENT( 3))
    ENDIF
!*
!           3.5   CHECK THAT THERE IS AN OBSERVATION ERROR

    ZCMOER = ROBODY(ILEV,MDBOER)
    IF(ZCMOER == RMDI) THEN
      ROBODY(ILEV,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(ILEV,MDBDSTA))
      ROBODY(ILEV,MDBDEV1) = ZCHEV1( ROBODY(ILEV,MDBDEV1),ND1EVENT(12))
    ENDIF
!*
!           3.6   CHECK THAT THERE IS A FINAL OBSERVATION ERROR

    ZCMFOE = ROBODY(ILEV,MDBFOE)
    IF(ZCMFOE == RMDI) THEN
      ROBODY(ILEV,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(ILEV,MDBDSTA))
      ROBODY(ILEV,MDBDEV1) = ZCHEV1( ROBODY(ILEV,MDBDEV1),ND1EVENT(12))
    ENDIF
!*
!           3.7   CHECK REPORTING PRACTICE FOR SYNOP/TEMP/PILOT

    IF(IOBTYP == NSYNOP .OR. IOBTYP == NTEMP .OR.IOBTYP == NPILOT) THEN
      CALL REPRA(JOBS,ILEV,IPOSM)
    ENDIF
!*
!           3.8   INCREASE OBS.ERROR FOR SYNOP GEOPOTENTIALS

    IVAR = ROBODY(ILEV,MDBVNM)
!*AF 15/05/2010 always increase obs. error here. because we know the orography (MDB_OROGRAPHY)
!*AF    IF(.not.L_OBS_ERR_INCREASE_Hretr) THEN
      IF(IOBTYP == NSYNOP) THEN
        IF(IVAR == NVNUMB( 3) .OR. IVAR == NVNUMB(47)) THEN  
!*
!           3.8.1 ASSIGN LOWEST MODEL LEVEL T AND  MODEL SURFACE PRESSURE
 
          ZTFG = YGOMUA(IPOSM)%T(NFLEVG)
          ZMPS = EXP(YGOMS(IPOSM)%SP)
!*
!           3.8.2 INCREASE OBS.ERROR IF RIGHT CONFIG
 
          CALL ADDOER(JOBS,ILEV,ZTFG,ZMPS)
        ENDIF
      ENDIF
!*AF    ENDIF
!*
!     ------------------------------------------------------------------

!        4.       CLOSE THE LOOP OVER ...
!                 -----------------------
!*
!           4.1   ... THE REPORT

  ENDDO DATUM_LOOP
!*
!           4.2   ... THE SET OF OBSERVATIONS

ENDDO OBS_LOOP
!*
!     ------------------------------------------------------------------

!        5.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('PRECH',1,ZHOOK_HANDLE)
END SUBROUTINE PRECH
