SUBROUTINE REDML_NO_SQ(KLEN)

!**** SUBROUTINE REDML_NO_SQ - LEVEL SELECTION OF MULTILEVEL REPORTS

!          HEIKKI JARVINEN   ECMWF          15/AUG/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        PERFORM REDUNDANCY CHECKS FOR TEMP/PILOT OBSERVATIONS

!*    INTERFACE
!     ---------

!     ***CALL REDML_NO_SQ(KLEN)
!     WHERE KLEN   = DB LEN

!        REDML_NO_SQ IS CALLED BY SUBROUTINE REDUN

!*    METHOD
!     ------

!     * LEVEL SELECTION OF MULTILEVEL REPORTS TEMP/PILOT:
!       - CONSIDER TEMPS AND PILOTS FROM ONE TIME WINDOW (6h IN 3D-VAR,
!         1h IN 4D-VAR, OR ALTERNATIVELY WHOLE 6h DATA WINDOW ALSO IN
!         ALSO IN 4D-VAR)
!       - REJECT IN LAYERS AROUND STD-LEVELS ACCORDING TO PRIORITIES
!           1. MAXIMUM DATUM FLAG
!           2. TIME DIFFERENCE TO ANALYISIS TIME
!           3. DISTANCE TO THE STD-LEVEL
!           4. SIGNIFICANT LEVEL
!           5. STANDARD LEVEL
!           6. TEMP OVER PILOT

!**   EXTERNALS
!     ---------

!        NGERSTA - GET REPORT STATUS
!        REDMO   - REDUNDANT MASS OBSERVATION
!        RESET   - RESET DATUM COUNTERS AND POINTER TABLES
!        STORD   - STORE DATUM POINTERS INTO LEVEL TABLES
!        REDOR   - FIND REDUNDANT REPORTS OF OTHER OBS.TYPE
!        SELEC   - MAKE LEVEL SELECTION

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMSCC   , ONLY : LSCDPR
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMCOCTP , ONLY :    &
 & NTEMP    ,NPILOT     
USE YOMDB
USE YOMOBS
USE YOMANCS  , ONLY : RDEGREES
USE ecsort_mix   , ONLY : keysort

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
REAL(KIND=JPRB)   ,allocatable  :: ZGLMLR(:,:)
REAL(KIND=JPRB)   , ALLOCATABLE :: ZKEY  (:,:)

INTEGER(KIND=JPIM), PARAMETER   :: icolumns0=7,ipo=1,idt0=2,itm0=3,ilt=4,iln=5,icom=6,irn0=7

INTEGER(KIND=JPIM) :: IRSTA(4)
!INTEGER_M :: mkey(3), isize, iret
INTEGER(KIND=JPIM) :: I_MKEY(2), isize, iret

INTEGER(KIND=JPIM) :: ICDTYP, ICOUNT, IOBS, IOBTYP, &
 & JOBS, ILEN, IGLMLR, INGLMLR, INADA, ICMONM, JPR  

LOGICAL :: LLOCAL
LOGICAL :: LL_PRINT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "ngenada.intfb.h"
#include "ngersta.intfb.h"
#include "redor.intfb.h"
#include "reset.intfb.h"
#include "selec.intfb.h"
#include "stord.intfb.h"
#include "thin_red_presort.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        0.       INITIALIZE
!                 ----------

IF (LHOOK) CALL DR_HOOK('REDML_NO_SQ',0,ZHOOK_HANDLE)
IF(LSCDPR) THEN
  WRITE(NULOUT,'(''     TEMP/PILOT REDUNDANCY (LEVEL SELECTION) CHECK '')')
ENDIF
LL_PRINT=LTHINNER_PRINT

!*          0.1   LEN

ILEN = KLEN

!           0.2   PRESORT OBS PER DATE, TIME, LAT, LON AND ..
!                 TO AVOID USING OBS SEQUENCE NO.

ALLOCATE(ZKEY (ILEN,icolumns0))
CALL THIN_RED_PRESORT(ILEN,'REDML',ZKEY,icolumns0,&
 & IPO,IDT0,ITM0,ILT,ILN,ICOM,IRN0 )  

!*
!     ------------------------------------------------------------------

!        1.       STUDY OBSERVATIONS
!                 ------------------

!*
!           1.1   LOOP OVER OBSERVATIONS

IGLMLR = 0
OBS_LOOP1: DO JOBS = 1, ILEN
  IOBS      = ZKEY(JOBS,1)
  IOBTYP    = ROBHDR(IOBS,MDBOTP)
  IF(IOBTYP /= NTEMP .AND. IOBTYP /= NPILOT) CYCLE OBS_LOOP1

!*          1.2   ACCEPT ACTIVE REPORTS ONLY

  CALL NGERSTA(INT(ROBHDR(IOBS,MDBRST)),ICOUNT,IRSTA)
  IF(IRSTA(2) == 1.OR. IRSTA(3) == 1.OR. IRSTA(4) == 1) CYCLE OBS_LOOP1

!*          1.3   ACCEPT ONLY REPORTS CONTAINING ACTIVE DATA

  CALL NGENADA(IOBS,INADA)
  IF(INADA == 0) CYCLE OBS_LOOP1

!*          1.4   UPDATE COUNTER

  IGLMLR = IGLMLR + 1

!*          1.5   CLOSE THE LOOP

ENDDO OBS_LOOP1

!     ------------------------------------------------------------------

!        2.       ALLOCATE AND PRESET GLOBAL ARRAYS
!                 ---------------------------------

!*          2.0   INFORM

!IF(LSCDPR) THEN
WRITE(NULOUT,'('' *** NO. OF ACTIVE TEMP/PILOT REPORTS:'',I10)') IGLMLR
CALL FLUSH(NULOUT)
!ENDIF
INGLMLR = IGLMLR
IF(INGLMLR == 0)  GO TO 7

!*          2.4        POINTER/SORT ARRAY

!ALLOCATE(ZGLMLR(4,INGLMLR))
ALLOCATE(ZGLMLR(3,INGLMLR))
IF(LSCDPR) THEN
  WRITE(NULOUT,9990) 'ZGLMLR   ',SIZE(ZGLMLR   ),SHAPE(ZGLMLR   )
ENDIF
ZGLMLR(:,:) = 0.0_JPRB

!*          2.5        FORMAT

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

!*
!     ------------------------------------------------------------------

!        3.       FILL GLOBAL ARRAYS

IGLMLR = 0

!*
!           3.1   LOOP OVER OBSERVATIONS

OBS_LOOP2: DO JOBS = 1, ILEN
  IOBS      = ZKEY(JOBS,1)
  IOBTYP    = ROBHDR(IOBS,MDBOTP)
  IF(IOBTYP /= NTEMP .AND. IOBTYP /= NPILOT) CYCLE OBS_LOOP2
  ICDTYP    = ROBHDR(IOBS,MDB_CODETYPE_AT_HDR)
  ICMONM    = ROBHDR(IOBS,MDBONM)

!*          3.2   ACCEPT ACTIVE REPORTS ONLY

  CALL NGERSTA(INT(ROBHDR(IOBS,MDBRST)),ICOUNT,IRSTA)
  IF(IRSTA(2) == 1.OR. IRSTA(3) == 1.OR. IRSTA(4) == 1) CYCLE OBS_LOOP2

!*          3.3   ACCEPT ONLY REPORTS CONTAINING ACTIVE DATA

  CALL NGENADA(IOBS,INADA)
  IF(INADA == 0) CYCLE OBS_LOOP2

!*          3.4   UPDATE COUNTER

  IGLMLR = IGLMLR + 1

!           3.5   FILL ARRAY

  ZGLMLR( 1,IGLMLR) = IOBS
  ZGLMLR( 2,IGLMLR) = ROBHDR(IOBS,MDBSID)
  ZGLMLR( 2,IGLMLR) = ZKEY(JOBS,IRN0)
! ZGLMLR( 2,IGLMLR) = ICDTYP
! ZGLMLR( 3,IGLMLR) = ROBHDR(IOBS,MDBSID)
! ZGLMLR( 4,IGLMLR) = ZKEY(JOBS,IRN0)
! ZGLMLR( 3,IGLMLR) = ICMONM

!           3.6   CLOSE THE LOOP

ENDDO OBS_LOOP2

!           3.7   CROSS CHECK

IF(IGLMLR /= INGLMLR) THEN
  WRITE(NULERR,'('' *** ERROR IN REDGL (1): '')')
  WRITE(NULERR,'(''     IGLMLR = '',I10)') IGLMLR
  WRITE(NULERR,'(''     INGLMLR= '',I10)') INGLMLR
  CALL ABOR1('REDMN_NO_SQL: ERROR 1')
ENDIF
!*
!     ------------------------------------------------------------------

!        4.       SORT
!                 ----
IF(LL_PRINT) THEN
  WRITE(NULOUT,'('' REDML_NO_SQ; ZKEY'')')
  DO JPR=1,ILEN
    WRITE(NULOUT,'('' '',I6,1X,6(F15.2,1X))') JPR,&
     & ZKEY(JPR,2),ZKEY(JPR,3),&
     & ZKEY(JPR,4),ZKEY(JPR,5),&
     & ZKEY(JPR,6),&
     & ZKEY(JPR,7)  
  ENDDO
  CALL FLUSH(NULOUT)
ENDIF

I_MKEY(1) = 2
I_MKEY(2) = 3
!mkey(3) = 4
isize = INGLMLR
CALL KEYSORT(iret, ZGLMLR, isize,multikey=I_MKEY, transposed=.TRUE.)

IF(LL_PRINT) THEN
  WRITE(NULOUT,'('' REDML_NO_SQ; ZGLMLR '')')
  DO JPR=1,INGLMLR
    IOBS = ZGLMLR(1,JPR)
    WRITE(NULOUT,'('' '',I5,1X,2(F15.2,1x))') JPR,&
     & ROBHDR(IOBS,MDBLAT)*RDEGREES,ROBHDR(IOBS,MDBLON)*RDEGREES  
  ENDDO
  CALL FLUSH(NULOUT)
ENDIF

!*
!     ------------------------------------------------------------------

!        5.       LEVEL SELECTION OF MULTILEVEL REPORTS
!                 -------------------------------------

3 CONTINUE

!*          5.1   LOOP OVER ALL OBS

OBS_LOOP3: DO JOBS = 1, INGLMLR
  IOBS = ZGLMLR(1,JOBS)
  ICMONM    = ROBHDR(IOBS,MDBONM)

!*          5.2   OBSERVATION TYPE

  IOBTYP = ROBHDR(IOBS,MDBOTP)

!*          5.3   SELECT TEMP/PILOT REPORTS

  IF(IOBTYP /= NTEMP  .AND. IOBTYP /= NPILOT) CYCLE OBS_LOOP3

!*          5.5   FIND OUT HEADER INFO

  ICDTYP = ROBHDR(IOBS,MDB_CODETYPE_AT_HDR)
  
!*
!           5.6   GET REPORT STATUS

  CALL NGERSTA(INT(ROBHDR(IOBS,MDBRST)),ICOUNT,IRSTA)
!*
!           5.7   CHECK WHETHER THE REPORT STATUS IS PAS/REJ/BLA

  IF(IRSTA(2) == 1.OR. IRSTA(3) == 1.OR. IRSTA(4) == 1) THEN
    CYCLE OBS_LOOP3
  ENDIF

!*
!           5.9   RESET DATUM POINTERS FOR LEVEL SELECTION

  CALL RESET

!*
!           5.10  STORE DATUM POINTERS

  LLOCAL = .TRUE.
  CALL STORD(IOBS,LLOCAL)
!*
!           5.11  IF TEMP, TAKE ALSO COLOCATED TEMPS INTO LEVEL SELECTION

  IF(IOBTYP == NTEMP) THEN
    CALL REDOR(INGLMLR,ZGLMLR,JOBS,NTEMP)
  ENDIF
!*
!           5.12  IF TEMP, TAKE COLOCATED PILOTS INTO LEVEL SELECTION

  IF(IOBTYP == NTEMP) THEN
    CALL REDOR(INGLMLR,ZGLMLR,JOBS,NPILOT)
  ENDIF
!*
!           5.13  IF PILOT, TAKE ALSO COLOCATED TEMPS INTO LEVEL SELECTION

  IF(IOBTYP == NPILOT) THEN
    CALL REDOR(INGLMLR,ZGLMLR,JOBS,NTEMP)
  ENDIF
!*
!           5.14  IF PILOT, TAKE ALSO COLOCATED PILOTS INTO LEVEL SELECTION

  IF(IOBTYP == NPILOT) THEN
    CALL REDOR(INGLMLR,ZGLMLR,JOBS,NPILOT)
  ENDIF
!*
!           5.15  PERFORM LEVEL SELECTION: WITHIN A TIME WINDOW

  CALL SELEC

!*          5.17  CLOSE THE LOOP OVER OBSERVATIONS

ENDDO OBS_LOOP3

!*
!     ------------------------------------------------------------------

!        6.       DEALOC
!                 ------

DEALLOCATE(ZGLMLR)

!*
!     ------------------------------------------------------------------

!        7.      RETURN
!                ------
7 CONTINUE
DEALLOCATE(ZKEY )
IF (LHOOK) CALL DR_HOOK('REDML_NO_SQ',1,ZHOOK_HANDLE)
END SUBROUTINE REDML_NO_SQ
