SUBROUTINE REDUN(KTSLOT)

!**** SUBROUTINE REDUN - PERFORM ALL REDUNDANCY CHECKS OF OBSERVATIONS

!          HEIKKI JARVINEN   ECMWF          15/AUG/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        PERFORM ALL REDUNDANCY CHECKS OF OBSERVATIONS, AT THE MOMENT
!     FOR SYNOP/AIREP/DRIBU/TEMP/PILOT/PAOB REPORTS

!*    INTERFACE
!     ---------

!     ***CALL REDUN(KTSLOT)
!     WHERE KTSLOT = TIME SLOT

!        REDUN IS CALLED BY SUBROUTINE DECIS

!*    METHOD
!     ------

!        THE ORDER OF PROCESSING OF DIFFERENT OBSERVATION TYPES IS
!     AS FOLLOWS:
!        1) REDUNDANT SYNOP,DRIBU AND PAOB REPORTS ARE REMOVED
!        2) REDUNDANT SHIP-SYNOP AND AIREP REPORTS ARE REMOVED
!        3) LEVEL SELECTION FOR TEMP/PILOT AND TEMP/SYNOP REDUNDANCIES
!           ARE REMOVED

!        PERFORM THE FOLLOWING REDUNDANCY CHECKS OF OBSERVATIONS:

!     * REDUNDANCY CHECK FOR SYNOP/PAOB REPORTS:
!       - CO-LOCATED REPORTS WITH THE SAME STATION ID ARE SEARCHED FOR
!         ACTIVE DATA WITH IDENTICAL TIME AND INSTRUMENT TYPE. ONLY
!         ONE REPORT IS KEPT ACTIVE, OTHERS REJECTED.

!     * REDUNDANCY CHECK FOR SYNOP (LAND) REPORTS:
!       - CO-LOCATED REPORTS WITH THE SAME STATION ID ARE SEARCHED FOR
!       - CLOSEST ACTIVE REPORT TO THE ANALYSIS TIME IS RETAINED, OTHERS
!         ARE REJECTED
!       - FOR SIMULTANEOUS REPORTS, THE ONE WITH MORE ACTIVE DATA IS
!         RETAINED.  THE SAME IS DONE FOR REPORTS THAT HAVE EQUAL TIME
!         DIFFERENCE TO THE ANALYSIS TIME.
!       - IF Z AND PS PRESENT IN THE SAME REPORT, MAKE THE MASS
!         OBSERVATION THER REDUNDANT

!     * MOVING PLATFORM REDUNDANCY CHECK FOR SYNOP (SHIP) REPORTS:
!       - REPORTS WITH THE SAME STATION ID ARE SEARCHED FOR
!       - AS FOR LAND SYNOP REPORTS BUT A GLOBAL ARRAY OF ACTIVE SHIP
!         SYNOPS IS USED
!       - LIST OF AMBIGUOUS PLATFORMS IS CHECKED TO PREVENT MAKING ANY
!         FURTHER MOVING PLATFORM CHECKS

!     * MOVING PLATFORM REDUNDANCY CHECK FOR AIREPS:
!       - REPORTS WITH THE SAME STATION ID ARE SEARCHED FOR
!       - DUPLICATES: ALREADY REMOVED IN DUPRP (BY MISCE)
!       - GLOBAL AIREP ARRAY IS USED TO FIND REDUNDANCIES (THIS COULD BE
!         ACTUALLY CALLED AS THINNING OF DATA).  THE TEST IS CODED IN
!         SUCH A WAY THAT THE RESULT IS INSENSITIVE TO THE NUMBER OF
!         PROCESSORS.

!     * MOVING PLATFORM REDUNDANCY CHECK FOR DRIBU REPORTS:
!       - REPORTS WITH THE SAME STATION ID ARE SEARCHED FOR
!       - AS FOR SHIP SYNOP REPORTS BUT A GLOBAL ARRAY OF ACTIVE DRIBUS
!         IS USED

!     * LEVEL SELECTION OF MULTILEVEL REPORTS TEMP/PILOT:
!       - CONSIDER TEMPS AND PILOTS FROM ONE TIME WINDOW (6h IN 3D-VAR,
!         1h IN 4D-VAR, OR ALTERNATIVELY WHOLE 6h DATA WINDOW ALSO IN
!         ALSO IN 4D-VAR)
!       - REJECT IN LAYERS AROUND STD-LEVELS ACCORDING TO PRIORITIES
!           1. MAXIMUM DATUM FLAG
!           2. TIME DIFFERENCE TO ANALYISIS TIME
!           3. DISTANCE TO THE STD-LEVEL
!           4. SIGNIFICANT LEVEL
!           5. STANDARD LEVEL
!           6. TEMP OVER PILOT
!       - COLOCATED SYNOP IS REDUNDANT TO TEMP IN ONE TIME WINDOW

!     * SINGLE LEVEL REPORT SYNOP/DRIBU/PAOB:
!       - IF Z AND PS PRESENT IN THE SAME REPORT, MAKE THE OTHER REDUN-
!         DANT

!**   EXTERNALS
!     ---------

!        GETDB        - GET DATA BASE
!        NGERSTA      - GET REPORT STATUS
!        REDRP        - THINNING OF REDUNDANT REPORTS
!        REDRP_NO_SQ  - THINNING OF REDUNDANT REPORTS
!        REDRP1       - SEARCH FOR REDUNDANT REPORTS WITH DIFFERENT CODE TYPE
!        REDRP1_NO_SQ - SEARCH FOR REDUNDANT REPORTS WITH DIFFERENT CODE TYPE
!        REDGL        - SEARCH FOR REDUNDANT REPORTS FROM GLOBAL ARRAY
!        REDGL_NO_SQ  - SEARCH FOR REDUNDANT REPORTS FROM GLOBAL ARRAY
!        REDMO        - REDUNDANT MASS OBSERVATION
!        REDMO_NO_SQ  - REDUNDANT MASS OBSERVATION
!        MOVPL        - MOVING PLATFORM (AIREP) REDUNDANCY CHECK
!        MOVPL_NO_SQ  - MOVING PLATFORM (AIREP) REDUNDANCY CHECK
!        RESET        - RESET DATUM COUNTERS AND POINTER TABLES
!        STORD        - STORE DATUM POINTERS INTO LEVEL TABLES
!        REDOR        - FIND REDUNDANT REPORTS OF OTHER OBS.TYPE
!        SELEC        - MAKE LEVEL SELECTION
!        REDSM        - FIND SYNOP MASS OBSERVATIONS REDUNDANT TO TEMP
!        REDSM_NO_SQ  - FIND SYNOP MASS OBSERVATIONS REDUNDANT TO TEMP
!        PUTDB        - PUT DATA BASE AWAY
!        REDPROF - FIND REDUNDANT PROFILER REPORTS

!**   MODIFICATIONS
!     -------------
!     98082900 : J-N Thepaut : allow for the use of french SYNOR stations
!     27/11/2001 A. Deckmyn  : added profiler redundancy check (3.)

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT   
USE YOMDB
USE YOMOBS
USE YOMCT0B  , ONLY : LECMWF
USE OML_MOD, ONLY : OML_SET_LOCK, OML_UNSET_LOCK

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN)   :: KTSLOT
INTEGER(KIND=JPIM) :: INFO(1), IRET
INTEGER(KIND=JPIM) :: ILEN

LOGICAL :: LLSHIP
REAL(KIND=JPRB) :: ZINFO(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcobs.h"

INTERFACE
#include "getdb.h"
#include "putdb.h"
END INTERFACE

#include "movpl.intfb.h"
#include "movpl_no_sq.intfb.h"
#include "redgl.intfb.h"
#include "redgl_no_sq.intfb.h"
#include "redml.intfb.h"
#include "redml_no_sq.intfb.h"
#include "redmo.intfb.h"
#include "redprof.intfb.h"
#include "redrp.intfb.h"
#include "redrp1.intfb.h"
#include "redrp1_no_sq.intfb.h"
#include "redrp_no_sq.intfb.h"
#include "redsm.intfb.h"
#include "redsm_no_sq.intfb.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       INITIALIZE
!                 ----------

!*          1.0   INFORM

IF (LHOOK) CALL DR_HOOK('REDUN',0,ZHOOK_HANDLE)
WRITE(NULOUT,'('' *** REDUNDANCY CHECK '')')
!*
!     ------------------------------------------------------------------

!        2.       SEARCH FOR REDUNDANT REPORTS
!                 ----------------------------

!           2.1   REDUNDANCY CHECK FOR LAND SYNOP AND PAOB

WRITE(NULOUT,'(''     REDUN_1; LAND SYNOP AND PAOB REDUNDANCY '')')

IRET = 0
CALL GETDB('REDUN',1,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, -1, -1, -1, -1)  

IF(IRET /= 0) THEN
  ILEN = IRET
!$ CALL OML_SET_LOCK()
  IF(LSYNOPPAOB_RED_NO_SQ) THEN
    IF(LSYNOPPAOB_REDRP_NO_SQ) THEN
      CALL REDRP_NO_SQ (ILEN)
    ELSE
      CALL REDRP       (ILEN)
    ENDIF
    IF(LSYNOPPAOB_REDRP1_NO_SQ) THEN
      CALL REDRP1_NO_SQ(ILEN)
    ELSE
      CALL REDRP1      (ILEN)
    ENDIF
  ELSE
    CALL REDRP       (ILEN)
    CALL REDRP1      (ILEN)
  ENDIF
!$ CALL OML_UNSET_LOCK()
  CALL PUTDB('REDUN',1,IRET,INFO,0,ZINFO,0)
ENDIF

!           2.2   REDUNDANCY CHECK FOR SURFACE Z/PS OBSERVATIONS

WRITE(NULOUT,'(''     REDUN_2; Z/PS REDUNDANCY CHECK'')')

IRET = 0
CALL GETDB('REDUN',2,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, -1, -1, -1, -1)  

IF(IRET /= 0) THEN
  ILEN = IRET
  CALL REDMO      (ILEN)
  CALL PUTDB('REDUN',2,IRET,INFO,0,ZINFO,0)
ENDIF

!*          2.3   REDUNDANCY CHECK FOR MOVING PLATFORMS: SHIP SYNOPS

WRITE(NULOUT,'(''     REDUN_3; SHIP SYNOPS REDUNDANCY CHECK'')')

IRET = 0
CALL GETDB('REDUN',3,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, -1, -1, -1, -1)  

IF(IRET /= 0) THEN
  ILEN   = IRET
  LLSHIP = .TRUE.
  IF(LSHIP_NO_SQ) THEN
    CALL REDGL_NO_SQ(ILEN,LLSHIP)
  ELSE
    CALL REDGL      (ILEN,LLSHIP)
  ENDIF
  CALL PUTDB('REDUN',3,IRET,INFO,0,ZINFO,0)
ENDIF

!*          2.4   REDUNDANCY CHECK FOR MOVING PLATFORMS: AIREPS

WRITE(NULOUT,'(''     REDUN_4; AIREPS REDUNDANCY CHECK'')')

IRET = 0
CALL GETDB('REDUN',4,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, -1, -1, -1, -1)  

IF(IRET /= 0) THEN
  ILEN = IRET
  IF(LAIREPS_NO_SQ) THEN
    CALL MOVPL_NO_SQ(ILEN)
  ELSE
    CALL MOVPL      (ILEN)
  ENDIF
  CALL PUTDB('REDUN',4,IRET,INFO,0,ZINFO,0)
ENDIF

!*          2.5   REDUNDANCY CHECK FOR MOVING PLATFORMS: DRIBUS

WRITE(NULOUT,'(''     REDUN_5; DRIBUS REDUNDANCY CHECK'')')

IRET = 0
CALL GETDB('REDUN',5,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, -1, -1, -1, -1)  

IF(IRET /= 0) THEN
  ILEN   = IRET
  LLSHIP = .FALSE.
  IF(LDRIBUS_NO_SQ) THEN
    CALL REDGL_NO_SQ(ILEN,LLSHIP)
  ELSE
    CALL REDGL      (ILEN,LLSHIP)
  ENDIF
  CALL PUTDB('REDUN',5,IRET,INFO,0,ZINFO,0)
ENDIF
!*
!     ------------------------------------------------------------------

!        3.       LEVEL SELECTION OF MULTILEVEL REPORTS
!                 -------------------------------------

WRITE(NULOUT,'(''     REDUN_6; TEMP/PILOT REDUNDANCY CHECK'')')

!           3.0   INITIALIZE DB (GLOBAL; TEMP, PILOT)

IRET = 0
CALL GETDB('REDUN',6,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, -1, -1, -1, -1)  

IF(IRET /= 0) THEN
  IF (.NOT.LECMWF) THEN
    WRITE(NULOUT,'(''     REDUN_6a; PROFILER REDUNDANCY CHECK'')')
    ILEN = IRET
    CALL REDPROF(ILEN)
  ENDIF

  ILEN = IRET
!$ CALL OML_SET_LOCK()
  IF(LTEMPPILOT_NO_SQ) THEN
    CALL REDML_NO_SQ(ILEN)
  ELSE
    CALL REDML      (ILEN)
  ENDIF
!$ CALL OML_UNSET_LOCK()

  CALL PUTDB('REDUN',6,IRET,INFO,0,ZINFO,0)
ENDIF
!*
!     ------------------------------------------------------------------

!        4.      FIND SYNOP MASS OBSERVATIONS REDUNDANT TO TEMP
!                ----------------------------------------------

WRITE(NULOUT,'(''     REDUN_7; TEMP/SYNOP REDUNDANCY CHECK'')')

!           4.0   INITIALIZE DB (GLOBAL; TEMP, SYNOP)

IRET = 0
CALL GETDB('REDUN',7,IRET,INFO,0,ZINFO,0,&
 & -1, KTSLOT, -1, -1, -1, -1)  

IF(IRET /= 0) THEN
  ILEN = IRET
  IF(LTEMPSYNOP_NO_SQ) THEN
    CALL REDSM_NO_SQ(ILEN)
  ELSE
    CALL REDSM      (ILEN)
  ENDIF
  CALL PUTDB('REDUN',7,IRET,INFO,0,ZINFO,0)
ENDIF

IF (LHOOK) CALL DR_HOOK('REDUN',1,ZHOOK_HANDLE)
END SUBROUTINE REDUN
