SUBROUTINE RH2Q(PRH,PRHERR,PT,PTERR,PP,PPERR,PQ,PQERRFRH,PQERRFP,PQERRFT  )

!**** RH2Q - CONVERT RH TO Q

!     Purpose.
!     -------

!        CONVERT RELATIVE HUMIDITY TO SPECIFIC HUMIDITY

!**   Interface.
!     ---------

!         CALL RH2Q(PRH,PRHERR,PT,PTERR,PP,PPERR,
!                   PQ,PQERRFRH,PQERRFP,PQERRFT  )
!     WHERE: PRH     = RH 
!            PRHERR  = RH ERROR
!            PT      = T
!            PTERR   = T ERROR
!            PP      = P
!            PPERR   = P ERROR
!            PQ      = Q
!            PQERRFRH= Q ERROR (RH FUNCTION)
!            PQERRFP = Q ERROR (P FUNCTION)
!            PQERRFT = Q ERROR (T FUNCTION)

!**   METHOD
!     ------

!        Q IS CALCULATED:

!            Q = RH*Qs      

!     AND ITS ERROR:

!            QERR = Qs*RHERR + Qs*(1./Qs**2)*QsERR

!     Qs IS DEFINED:

!            Qs    = Rd/Rv*Es(T)/P / (1-(Rv/Rd-1)*Rd/Rv*Es(T)/P)

!            Es(T) = 610.78*exp(a*(T-T0)/(T-b)) 

!            T0    = 273.16

!                   | 17.269  if T.GE.T0
!            a     =|
!                   | 21.875  if T.LT.T0

!                   | 35.86   if T.GE.T0   
!            b     =|
!                   |  7.66   if T.LT.T0

!            Rv    = 461.51

!            Rd    = 287.05 

!     Author: D. VASILJEVIC
!     Date  : 94-02-16

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 

!--------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMMKODB , ONLY : LQERRFRH ,LQERRFT  ,LQERRFP
USE YOMANCS  , ONLY : RTT      ,R2ES     ,R3LES    ,&
 & R3IES    ,R4LES    ,R4IES    ,RTWAT    ,RTICE    ,&
 & RRD      ,RRV      ,RMDI     ,EPSMAC  

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: PRH 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHERR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTERR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPERR 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQERRFRH 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQERRFP 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQERRFT 

REAL(KIND=JPRB) :: ZDENOM, ZDENOM2,&
 & ZEPESRP, ZEPS, ZQERRFTL, ZQS, ZRETV  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "stmfun_ifs.h"
!*

!        1.       INITIATE
!                 --------

!*
!-----------------------------------------------------------------------

!        2.       CONVERT RH TO Q
!                 ---------------

!*          2.1    CALCULATE eps*Es(T)/p  AND qs

IF (LHOOK) CALL DR_HOOK('RH2Q',0,ZHOOK_HANDLE)
ZEPS=RRD/RRV
ZRETV=1.0_JPRB/ZEPS-1.0_JPRB
ZEPESRP=FOEEWMO(PT)/PP
ZEPESRP=MIN(0.5_JPRB,ZEPESRP)
ZQS=ZEPESRP/(1.0_JPRB-ZRETV*ZEPESRP)
ZDENOM=( 1.0_JPRB - PRH*ZRETV*ZEPESRP )

!*          2.3    CALCULATE Q

PQ     =  PRH * ZEPESRP / ZDENOM
!*
!-----------------------------------------------------------------------

!        3.        Q ERROR (DEPENDS ON RH, P AND T ERRORS)
!                  ---------------------------------------

!*          3.1    Q ERROR DEPENDENT ON RH ERROR

IF(LQERRFRH)   THEN
  PQERRFRH = PRHERR*ZEPESRP/ZDENOM**2
ELSE
  PQERRFRH = RMDI
ENDIF

!*          3.2    Qs ERROR (VIA WHICH Q ERROR IS A FUNCTION
!                  OF P AND T ERRORS)

IF(.NOT.LQERRFP.AND..NOT.LQERRFT)        THEN
  PQERRFP = RMDI
  PQERRFT = RMDI
ELSE
  ZDENOM2 =ZDENOM * ZDENOM

!*          3.2.1  P DEPENDECY

  IF(LQERRFP)                    THEN
    PQERRFP =-(ZEPESRP*(ZDENOM/PP+ ZRETV*ZEPESRP/PP)/&
     & ZDENOM**2                                     ) *&
     & PPERR   
    PQERRFP = PRH * PQERRFP
  ELSE
    PQERRFP = RMDI
  ENDIF

!*          3.2.2  T DEPENDECY

  IF(LQERRFT)                    THEN
    ZQERRFTL = (ZEPESRP*R3LES*(RTT-R4LES)/(PT-R4LES)**2*&
     & (ZDENOM+PRH*ZEPESRP*ZRETV)    /&
     & ZDENOM**2                               ) *&
     & PTERR  
    PQERRFT = PRH * ZQERRFTL
  ELSE
    PQERRFT = RMDI
  ENDIF
ENDIF
!*
!-----------------------------------------------------------------------

!        4.        RETURN
!                  ------

IF (LHOOK) CALL DR_HOOK('RH2Q',1,ZHOOK_HANDLE)
END SUBROUTINE RH2Q
