SUBROUTINE SATEMIS(KINS,KDPT,KSTID,KI3,KI4,KI2I2,KI1,KIS   )

!**** SUBROUTINE SATEMIS - CONSTRUCT SATEM INSTRUMENT SPECIFICATION WORD

!        D. VASILJEVIC     ECMWF     24/2/95

!     PURPOSE
!     -------

!        CONSTRUCT INSTRUMENT SPECIFICATION WORD FOR SATEM

!**   INTERFACE
!     ---------

!        CALL SATEMIS(KINS,KDPT,KSTID,
!                     KI3,KI4,KI2I2,KI1,KIS   )

!     WHERE:
!           KINS    = INPUT SAT. INSTRUMENT VALUE (BUFR TABLE 2021)
!           KDPT    = INPUT DATA PROCESSING TECHNIQUE (BUFR TABLE 2022)
!           KSTID   = INPUT STATION ID (BUFR TABLE 1007)
!           KI3     = OUTPUT I3 PARAM.
!           KI4     = OUTPUT I4 PARAM.
!           KI2I2   = OUTPUT I2I2 PARAM.
!           KIS     = OUTPUT INSTRUMENT TYPE

!        SATEMIS IS CALLED BY SUBROUTINE OBINSSP

!     METHOD
!     ------

!        THE SATEM INSTRUMENT SPECIFICATION WORD HAS THE FOLLOWING
!     FORMAT:

!        BITS 
!         0-23 - 77 777 777B
!        24-27 - I3, INSTRUMENT DATA USED IN PROCESSING
!                (SEE WMO MANUAL ON CODES, VOL II, SECTION II-4-E-8)
!        31-28 - I4, DATA PROCESSING TECHNIQUE, SEE SECTION II-4-E-9
!        38-32 - I2I2, SATELLITE NAME, SEE SECTION II-4-E-7
!        42-39 - I1, COUNTRY OPERATING SATELLITE, WMO CODE 1761
!        49-43 - IS, INSTRUMENT SPECIFICATION CODE, SEE RESEARCH
!                MANUAL 5, TABLE 7.5

!     EXTERNALS
!     ---------

!        NONE

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMANCS  , ONLY : NMDI     ,NBITS    ,RADIANS  ,RTT      ,&
 & R2ES     ,R3LES    ,R3IES    ,R4LES    ,R4IES    ,&
 & RTWAT    ,RTICE    ,EPSMAC  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KINS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTID 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KI3 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KI4 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KI2I2 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KI1 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KIS 
INTEGER(KIND=JPIM) :: I1, I2I2, I3, I4, ICLD, ICLR, IHIRS, IMSU,&
 & IPCLD, IS, ISSU

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

#include "fcobs.h"
#include "stmfun_ifs.h"
!*

!        1.             CHANNEL COMBINATIONS
!                       --------------------

IF (LHOOK) CALL DR_HOOK('SATEMIS',0,ZHOOK_HANDLE)
IHIRS = 0
IMSU  = 0
ISSU  = 0

!*          1.1         MISSING VALUE FOR INSTRUMENT WORD

IF(KINS == NBITS( 9).OR.KINS ==      NMDI    )         THEN
  I3 = 0
  GO TO 2
ENDIF

!*          1.2         IDENTIFY HIRS, MSU AND SSU

IHIRS = IBITS(KINS, 8,1)
IMSU  = IBITS(KINS, 7,1)
ISSU  = IBITS(KINS, 6,1)

!*          1.3         DERIVE I3 FROM CHANNEL COMBINATION

IF(IHIRS == 1.AND.IMSU  == 1.AND.ISSU  == 1     )               THEN
  I3 = 1
ELSEIF(IHIRS == 1.AND.IMSU  == 1     )          THEN
  I3 = 2
ELSEIF(IHIRS == 1.AND.ISSU  == 1     )          THEN
  I3 = 4
ELSEIF(IHIRS == 1     )          THEN
  I3 = 3
ELSEIF(IMSU  == 1.AND.ISSU  == 1     )          THEN
  I3 = 6
ELSEIF(IMSU  == 1     )          THEN
  I3 = 5
ELSEIF(ISSU  == 1     )          THEN
  I3 = 7
ELSE
  I3 = 0
ENDIF
!*
!-----------------------------------------------------------------------

!        2.             DATA PROCESSING TECHNIQUE
!                       -------------------------

2 CONTINUE

!*          2.1         MISSING VALUES FOR DATA PROCESSING TECHNIQUE
!                       AND INSTRUMENT SPECIFICATION WORD

IF(KDPT == NBITS( 8).OR.KDPT ==      NMDI    )         THEN
  IF(KINS == NBITS( 9).OR.KINS ==      NMDI    )         THEN
    I4 = 0

!*          2.2         DATA PROCESSING TECHNIQUE MISSING. USE
!                       INSTRUMENT SPECIFICATION WORD TO TRY TO DEDUCE
!                       DATA PROCESSING TECHNIQUE (CAN'T DISTINGUISH
!                       BETWEEN CLEAR AND PARTLY CLOUDY)

  ELSE
    IF(IHIRS == 1)                 THEN
      I4 = 2
    ELSE
      I4 = 3
    ENDIF
  ENDIF
  GO TO 3
ENDIF

!*          2.3         DATA PROCESSING TECHNIQUE SPECIFIED

ICLR  = IBITS(KDPT, 5,1)
IPCLD = IBITS(KDPT, 4,1)
ICLD  = IBITS(KDPT, 3,1)
IF(ICLR == 1)                  THEN
  I4 = 1
ELSEIF(IPCLD == 1)            THEN
  I4 = 2
ELSEIF(ICLD == 1)             THEN
  I4 = 3
ELSE
  I4 = 0
ENDIF
!*
!-----------------------------------------------------------------------

!        3.             SATELLITE IDENTIFIERS
!                       ---------------------

3 CONTINUE

!*          3.1         COUNTRY OF ORIGIN

I1 = KSTID/100

!*          3.2         SATELLITE NAME

I2I2 = MOD(KSTID,100)

!*          3.3         INSTRUMENT SPECIFICATION CODE

IS = 89
!*
!-----------------------------------------------------------------------

!        4.             INSTRUMENT SPECIFICATION/TYPE
!                       -----------------------------

!*          4.1         INSTRUMENT SPECIFICATION PARAMETERS

KI3   = I3
KI4   = I4
KI2I2 = I2I2
KI1   = I1

!*          4.2         INSTRUMENT TYPE

KIS   = IS
!*
!-----------------------------------------------------------------------

!        5.             RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('SATEMIS',1,ZHOOK_HANDLE)
END SUBROUTINE SATEMIS
