SUBROUTINE SCAQC(KLEN,CD_MODE)

!**** SUBROUTINE SCAQC - GLOBAL SCATTEROMETER QUALITY CONTROL

!          HEIKKI JARVINEN   ECMWF          15/SEP/1997
!          DRASKO VASILJEVIC ECMWF          25/OCT/1999

!*    PURPOSE
!     -------

!        CHECK THE "AVERAGE DISTANCE TO THE CONE" FOR THE SCATTEROMETER
!     DATA.

!*    INTERFACE
!     ---------

!     ***CALL SCAQC(KLEN,CD_MODE)
!     WHERE KLEN   = NUMBER OF DATA
!           CD_MODE= Modus operandii

!        SCAQC IS CALLED BY SUBROUTINE DECIS

!*    METHOD
!     ------

!*    EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     2005-05 C.Payan : Use of sea-ice cc field controlled by NUSE_ECCI
!     2006-11 Hans Hersbach : add ASCAT
!     2007-10 Hans Hersbach : Add option for use of NSCAWSOLMAX ambiguities
!     2007-04 C.Payan : Use of YGOMS%TS (NICESEAMASK=1 case)
!                 in the Non-Quikscat case
!     2007-04 C.Payan : rain QC under NSCAT5_RQC control /1,2/=/rainc,dist2cone/
!                         Qscatt flag is now in MDBSCPFL (like others Scatt data) 
!     2007-04 C.Payan : control on number of "active" amb winds (exactly 2 sol)
!                        plus v-comp just after u-comp, made for all Scatt data
!                        (not only Qscat data) 
!     2007-06 C.Payan : changes in Quality Flags. Check of L2 Ascat data from
!                        KNMI added
!     2009-09 H.Hersbach : Add Oceansat-2
!     2010-05 H.Hersbach : Add QC for celno out of range

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RPI
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMANCS  , ONLY : RMDI     ,RADIANS
USE YOMTHLIM , ONLY :   NSCAMIN  ,NSCATWVC  ,NASCAWVC,  NNSCAWVC  ,NQSCAWVC,&
 & NOSCAWVC, RSCAFAC  ,RSCATDIS  ,RASCADIS ,RNSCADIS  ,RQSCADIS   ,ROSCADIS 
USE YOMDB
USE YOMCOCTP , ONLY : NSCATT   ,NSCAT2   ,NSCAT3   ,NSCAT4   ,NSCAT5,  NSCAT6
USE YOMNMEV  , ONLY : NR2EVENT ,ND1EVENT,NR1EVENT
USE YOMSCC   , ONLY : NTHINSCA ,NICESEAMASK ,NSCAT5_RQC, LQSCATAZI,LOSCATAZI,NSCAWSOLMAX
USE YOMVNMB  , ONLY : NVNUMB
USE GOMS_MIX , ONLY : YGOMEC ,YGOMS  ,NUSE_ECCI
USE MPL_MODULE
USE LOCAL_TRAFOS

IMPLICIT NONE

INTERFACE
#include "surf_inq.h"
END INTERFACE

#include "abor1.intfb.h"
#include "cmoctmap.intfb.h"
#include "ngersta.intfb.h"

!     DUMMY SCALARS
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
CHARACTER(LEN=*)  ,INTENT(IN)    :: CD_MODE 
INTEGER(KIND=JPIM) :: J,JOBS,JBODY,IDBREV2
INTEGER(KIND=JPIM) :: ICELLN,INSTA,IPOSM,ICDTYPE(KLEN),INBINS,ISUM,ISUM_TOTAL,&
 & ICDSQNO,IOTSQNO,IAMB,IBODY,IRSTA(4),IUBODIES,IVBODIES,IUV,IPRODCD,&
 & IAMBMAX,ISCATQC,IWVC_QC,IPROD_KNMI  
LOGICAL :: LLREJ,LL_TYPE(KLEN),LLQSCAT,LLOSCAT
REAL(KIND=JPRB) :: ZAVD, ZDISTF, ZDIR(2) ,ZDIF,ZU,ZV
INTEGER(KIND=JPIM), ALLOCATABLE :: IGLNUM(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGLDIS(:), ZLIMIT(:)
REAL(KIND=JPRB) :: ZCI,ZRCIMIN,ZRTMELTSICE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcobs.h"

#include "openmp_obs.h"
!     ------------------------------------------------------------------

!*       0.       INITIALIZE
!                 ----------
!*
!           0.2   Initialize according to CD_MODE

IF (LHOOK) CALL DR_HOOK('SCAQC',0,ZHOOK_HANDLE)
CALL SURF_INQ(PRTMELTSICE=ZRTMELTSICE,PRCIMIN=ZRCIMIN)
LLQSCAT=.FALSE.
LLOSCAT=.FALSE.
IF(CD_MODE == 'ERS' )THEN
  INBINS=NSCATWVC
ELSEIF(CD_MODE == 'NSCAT' )THEN
  INBINS=NNSCAWVC
ELSEIF(CD_MODE == 'QSCAT' )THEN
  INBINS=NQSCAWVC
  LLQSCAT=.TRUE.
ELSEIF(CD_MODE == 'ASCAT' )THEN
  INBINS=NASCAWVC
ELSEIF(CD_MODE == 'OSCAT' )THEN
  INBINS=NOSCAWVC
  LLOSCAT=.TRUE.
ELSE
  WRITE(NULERR,'(''BAD CD_MODE IN SCAQC '',A5)') CD_MODE
  CALL ABOR1('SCAQC: BAD CD_MODE')
ENDIF

IPROD_KNMI=99
ALLOCATE(IGLNUM(INBINS))
ALLOCATE(ZGLDIS(INBINS))
ALLOCATE(ZLIMIT(INBINS))
IGLNUM(:) = 0_JPIM
ZGLDIS(:) = 0._JPRB
IF(CD_MODE == 'ERS')   ZLIMIT(:)=RSCAFAC*RSCATDIS(:)
IF(CD_MODE == 'ASCAT') ZLIMIT(:)=RSCAFAC*RASCADIS(:)
IF(CD_MODE == 'NSCAT') ZLIMIT(:)=RSCAFAC*RNSCADIS(:)
IF(CD_MODE == 'QSCAT') ZLIMIT(:)=RSCAFAC*RQSCADIS(:)
IF(CD_MODE == 'OSCAT') ZLIMIT(:)=RSCAFAC*ROSCADIS(:)

IF (KLEN == 0) GOTO 210

ICDTYPE(1:KLEN)   =  ROBHDR(1:KLEN,MDB_CODETYPE_AT_HDR)

IF(CD_MODE == 'ERS' )THEN
  LL_TYPE(1:KLEN) = ICDTYPE(1:KLEN) == NSCAT2
  CALL CMOCTMAP('CODETOSQ',NULOUT,NSCATT,NSCAT2,IOTSQNO,ICDSQNO)
ELSEIF(CD_MODE == 'ASCAT' )THEN
  LL_TYPE(1:KLEN) = ICDTYPE(1:KLEN) == NSCAT3
  CALL CMOCTMAP('CODETOSQ',NULOUT,NSCATT,NSCAT3,IOTSQNO,ICDSQNO)
ELSEIF(CD_MODE == 'NSCAT' )THEN
  LL_TYPE(1:KLEN) = ICDTYPE(1:KLEN) == NSCAT4
  CALL CMOCTMAP('CODETOSQ',NULOUT,NSCATT,NSCAT4,IOTSQNO,ICDSQNO)
ELSEIF(CD_MODE == 'QSCAT' )THEN
  LL_TYPE(1:KLEN) = ICDTYPE(1:KLEN) == NSCAT5
  CALL CMOCTMAP('CODETOSQ',NULOUT,NSCATT,NSCAT5,IOTSQNO,ICDSQNO)
ELSEIF(CD_MODE == 'OSCAT' )THEN
  LL_TYPE(1:KLEN) = ICDTYPE(1:KLEN) == NSCAT6
  CALL CMOCTMAP('CODETOSQ',NULOUT,NSCATT,NSCAT6,IOTSQNO,ICDSQNO)
ENDIF
WRITE(NULOUT,'('' *** PERFORM GLOBAL '',A5,'' QUALITY CONTROL '')') CD_MODE
WRITE(NULOUT,*) ' Total and type-restricted # of DATA: ',KLEN,COUNT(LL_TYPE)
CALL FLUSH(NULOUT)

!     ------------------------------------------------------------------

!*       1.       Calculate the distance locally
!                 ------------------------------

!           1.0   Loop the observations

OBS_LOOP1: DO JOBS = 1, KLEN
  IF(.NOT.LL_TYPE(JOBS)) CYCLE OBS_LOOP1

!*          1.0.1   Find out producer code, observation pointers, cell number,
!                   Home-Made ProductFlag and ProducerFlag
  IPRODCD= ROBHDR(JOBS,MDB_INSTTYPE_AT_HDR)
  IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
  ICELLN = ROBHDR(JOBS,MDBSCCNO)
!ABS function allows to resolve the RMDI=-NMDI case, with the rejection of data
  ISCATQC= ABS(NINT(ROBHDR(JOBS,MDBSCPFL)  ))
  IWVC_QC= ABS(NINT(ROBHDR(JOBS,MDB_WVC_QF)))

!*          1.1   Some generic QC

!*          1.1.0 Reject report if cellno is out of range
  IF (ICELLN<1 .OR. ICELLN>INBINS) THEN
     ROBHDR(JOBS,MDBRST)  = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
     ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT(3))
     CYCLE OBS_LOOP1
  ENDIF

!*          1.1.1 Assimilate up to first NSCAWSOLMAX solutions
!                 Note that lower limits may have been hard coded
!                 in the preceding SCATIN routines.
  IAMBMAX=MAX(2,NSCAWSOLMAX)
  IUBODIES=0_JPIM
  IVBODIES=0_JPIM
  BODIES_LOOP: DO IBODY = MLNKH2B(JOBS), MLNKH2B(JOBS+1)-1
     IF     (ROBODY(IBODY,MDBVNM)==NVNUMB(62)) THEN
       IUBODIES=IUBODIES+1
       IUV     =IUBODIES
     ELSEIF (ROBODY(IBODY,MDBVNM)==NVNUMB(61)) THEN
       IVBODIES=IVBODIES+1
       IUV     =IVBODIES
     ELSE
       IUV=0_JPIM
     ENDIF
     IF (IUV>IAMBMAX) THEN
          ROBODY(IBODY,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IBODY,MDBDSTA))
          ROBODY(IBODY,MDBDEV1) = ZCHEV1( ROBODY(IBODY,MDBDEV1),ND1EVENT(19))
     ENDIF
  ENDDO BODIES_LOOP

!*          1.1.2 Reject report if less than two ambiguous winds or one v-comp missing
  IF (IUBODIES<2 .OR. IVBODIES<2 .OR. IUBODIES/=IVBODIES) THEN
     ROBHDR(JOBS,MDBRST)  = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
     ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT(3))
     CYCLE OBS_LOOP1
  ENDIF


!*          1.2   QuikScat/Oceansat-2 section
  IF(LLQSCAT.OR.LLOSCAT) THEN

!*          1.2.1 Reject QuickScat data at the swath edges
!                 This could be taken care of by the BLACKLIST

! 1 2 3 4 5 6 7 8 91011121314151617181920212223242526272829303132333435363738 <-- QuikSCAT
! x x x x                                                             x x x x
! 1 2 3 4 5 6 7 8 9101112131415161718192021222324252627282930313233343536     <-- Oceansat-2
! x x x X                                                        X x x x

    IF( (LLQSCAT.AND.(ICELLN <= 4 .OR.  ICELLN >= 35)) .OR. &
      & (LLOSCAT.AND.(ICELLN <= 4 .OR.  ICELLN >= 33)) ) THEN
      ROBHDR(JOBS,MDBRST)  = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
      IDBREV2 = ROBHDR(JOBS,MDBREV2)
      CALL SRGEVENT('SET E',IDBREV2,NR2EVENT(3,IOTSQNO),1)
      ROBHDR(JOBS,MDBREV2) = IDBREV2
    ENDIF

!*          1.2.2 Reject QuikSCAT/Oceansat-2 winds if the directions of the wind
!*                solutions are not sufficiently diverse.
!*                This algorithm assumes that the first two wind solutions are 
!*                the most likely one, and its most ant-parallel one.
!*                Is taken care of in subroutines QSCATIN/OSCATIN.

    IF( (LLQSCAT.AND.LQSCATAZI) .OR.&
       &(LLOSCAT.AND.LOSCATAZI) ) THEN
      BODY_LOOP1: DO JBODY = 1, 4, 2
        IBODY = MLNKH2B(JOBS)+(JBODY-1)
        ZU   = ROBODY(IBODY  ,MDBVAR)
        ZV   = ROBODY(IBODY+1,MDBVAR)
        IAMB = (JBODY+1)/2
        IF(ZU /= RMDI .AND. ZV /= RMDI) THEN
          ZDIR(IAMB) = UV2DD(ZU, ZV)*RADIANS
        ELSE
          WRITE(NULERR,'(''  SCAQC:: ERROR CONDITION...'')')
          WRITE(NULERR,'(''         JBODY IAMB ZU ZV ='',&
           & 2I4,2I6,2E16.4)') JBODY,IAMB,ZU,ZV  
          CALL ABOR1('ERROR IN SCAQC.  CANNOT COMPUTE QSCAT/OSCAT WIND DIRECTIONS.')
        ENDIF
      ENDDO BODY_LOOP1

!           1.2.3 Reject report if wind direction difference is less
!                 than +/- 135 degrees

      ZDIF = RPI - ABS(RPI - ABS(ZDIR(1) - ZDIR(2)))
      IF(ZDIF <  3._JPRB*RPI/4._JPRB) THEN
        ROBHDR(JOBS,MDBRST)  = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
        IDBREV2 = ROBHDR(JOBS,MDBREV2)
        CALL SRGEVENT('SET E',IDBREV2,NR2EVENT(2,IOTSQNO),1)
        ROBHDR(JOBS,MDBREV2) = IDBREV2
      ENDIF
    ENDIF

!*          1.2.4 Reject QuikSCAT/OCEANSAT data
!*                when more than one 25km box is rain contaminated
!*                do not discriminate on number of 25-cells that
!*                were removed due to rain contamination

    LLREJ = .FALSE.
    IF(NSCAT5_RQC == 1) THEN
      IF(IBITS(ISCATQC,0,3) > 1_JPIM)  LLREJ=.TRUE.

!*          1.2.5 Reject QuikSCAT data when for one or more 25-cells
!*                the distance-to-cone flag is set.

    ELSEIF(NSCAT5_RQC == 2) THEN
      IF (IBITS(ISCATQC,3,3)/=0_JPIM) LLREJ=.TRUE.
    ELSE
      WRITE(NULOUT,'(''0WRONG NSCAT5_RQC specified'',I2)')NSCAT5_RQC      
      CALL ABOR1('ERROR IN SCATQC')      
    ENDIF
    IF(LLREJ) THEN
         ROBHDR(JOBS,MDBRST)  = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
         IDBREV2 = ROBHDR(JOBS,MDBREV2)
         CALL SRGEVENT('SET E',IDBREV2,NR2EVENT(NSCAT5_RQC+3,IOTSQNO),1)
         ROBHDR(JOBS,MDBREV2) = IDBREV2
    ENDIF

!*          1.2.6 Reject QuikSCAT/Oceansat-2 data
!*                when global instrument-performance flag is set

    IF (IBITS(ISCATQC,6,1)==1_JPIM) THEN
      ROBHDR(JOBS,MDBRST)  = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
      ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT(16))
    ENDIF

  ELSE
!*          1.3   Select the report - Non-QuikSCAT case

    IF ( NICESEAMASK == 1 ) THEN                  !TS mask
       ZCI = ZRTMELTSICE - 1._JPRB
       ZCI = YGOMS(IPOSM)%TS
       IF (ZCI < ZRTMELTSICE) CYCLE OBS_LOOP1
    ELSEIF (NICESEAMASK == 2 .OR. NICESEAMASK == 3) THEN  !ECCI mask 
       ZCI = ZRCIMIN + 1._JPRB
       IF (NUSE_ECCI == 1) ZCI = YGOMEC(IPOSM)%ECCI
       IF (ZCI > ZRCIMIN)  CYCLE OBS_LOOP1
    ENDIF
    CALL NGERSTA(INT(ROBHDR(JOBS,MDBRST)),INSTA,IRSTA)
    IF(IRSTA(2) == 1.OR.IRSTA(3) == 1.OR.IRSTA(4) == 1) CYCLE OBS_LOOP1
  ENDIF

!*          1.4  Across-wise Thinning

  IF(CD_MODE == 'ERS') THEN
! In case NTHINSCA=4:
! 1 2 3 4 5 6 7 8 910111213141516171819
!     x       x       x       x       x
    IF(MOD(ICELLN-3,NTHINSCA) /= 0) THEN
      ROBHDR(JOBS,MDBRST)  = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
      IDBREV2 = ROBHDR(JOBS,MDBREV2)
      CALL SRGEVENT('SET E',IDBREV2,NR2EVENT(1,IOTSQNO),1)
      ROBHDR(JOBS,MDBREV2) = IDBREV2
    ENDIF
  ENDIF


  IF(CD_MODE == 'ASCAT') THEN
! In case NTHINSCA=4:
! 1 2 3 4 5 6 7 8 9101112131415161718192021 222324252627282930313233343536373839404142
! x       x       x       x       x       x  x       x       x       x       x       x

    IF(MOD(MIN(ICELLN-1,INBINS-ICELLN),NTHINSCA) /= 0) THEN
      ROBHDR(JOBS,MDBRST)  = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
      IDBREV2 = ROBHDR(JOBS,MDBREV2)
      CALL SRGEVENT('SET E',IDBREV2,NR2EVENT(1,IOTSQNO),1)
      ROBHDR(JOBS,MDBREV2) = IDBREV2
    ENDIF

    LLREJ=.FALSE.
!   CHECK QUALITY FLAG DETERMINED BY IN-HOUSE WIND INVERSION (ASCATIF)
!   Determined in BATOR for the French part, without Eumetsat QF (not tuned yet)
!   To be possibly fine-tuned after more data has become available
!   Switch off for early data since EUMETSAT did not fill QC-flags yet
!   besides, all data looks o.k. anyway.
!   IF( ISCATQC /= 0._JPIM ) LLREJ=.TRUE.
!   CHECK QUALITY FLAG from KNMI (according to Ascat Wind Product Manual, Sea
!   Ice SAF, Version 1.0, March 2007) + Land presence (replace land test of
!   in-house QF (test recommended by ECMWF), made in ASCATIF (not amb winds produced)
!   and excluded with the KNMI data)
!   /18,17,16/=/Product Monitoring Flag, KNMI QC, Var QC/
    IF( IPRODCD == IPROD_KNMI ) THEN
       LLREJ=BTEST(IWVC_QC,18).OR.BTEST(IWVC_QC,17).OR.BTEST(IWVC_QC,16).OR.(ISCATQC /= 0_JPIM)
    ENDIF
    IF (LLREJ) THEN
       ROBHDR(JOBS,MDBRST)  = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
       ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT(16))
       CYCLE OBS_LOOP1
    ENDIF

  ENDIF


!*          1.5   Pick up the first residual 

  ZDISTF = ROBODY(MLNKH2B(JOBS),MDBSCRES)
  IF(ZDISTF == RMDI) CYCLE OBS_LOOP1

!*          1.6   Add the distance

! ZDISTF<=0 in ASCAT L2 format (as Quikscat)
  IF(CD_MODE == 'ERS' .OR. CD_MODE == 'ASCAT') ZDISTF = SQRT(ABS(ZDISTF))
  IGLNUM(ICELLN) = IGLNUM(ICELLN) + 1_JPIM
  ZGLDIS(ICELLN) = ZGLDIS(ICELLN) + ZDISTF

!*          1.6   Close the loop over observations

ENDDO OBS_LOOP1

!*
!     ------------------------------------------------------------------

!        2.       SUM GLOBALLY AND CHECK
!                 ----------------------

!*          2.1   SUM GLOBALLY, ACROSS PROCESSORS

210  CONTINUE

IF (CD_MODE == 'ERS' .OR. CD_MODE == 'ASCAT'  .OR. CD_MODE == 'NSCAT' ) THEN
  CALL MPL_ALLREDUCE(IGLNUM, 'SUM',CDSTRING='SCAQC:')

  ISUM_TOTAL=SUM(IGLNUM(1:INBINS))
  IF(ISUM_TOTAL == 0) GOTO 9
  LLREJ = .FALSE.

  CALL MPL_ALLREDUCE(ZGLDIS, 'SUM', LDREPROD=.TRUE.,CDSTRING='SCAQC:')

!*          2.2   TEST THE AVERAGE DISTANCE

  WRITE(NULOUT,'('' *** GLOBAL SCATT QUALITY CONTROL         '')')
  WRITE(NULOUT,'(''     CELL NO.  N.OBS.  AV.DIST.  MAX.DIST. QC.LIMIT '')')
  ISUM  = 0

  DO J = 1, INBINS
    IF(CD_MODE == 'ERS' .AND. MOD(J-3,NTHINSCA) /= 0) CYCLE
    IF(CD_MODE =='ASCAT'.AND. MOD(MIN(J-1,INBINS-J),NTHINSCA) /= 0) CYCLE

    ZAVD = 0.0_JPRB
    IF(IGLNUM(J) > 0) ZAVD = ZGLDIS(J)/IGLNUM(J)
    ISUM = ISUM + IGLNUM(J)
    IF(ZAVD > ZLIMIT(J) .AND. IGLNUM(J) > NSCAMIN) LLREJ=.TRUE.

    WRITE(NULOUT,'(4X,I8,1X,I7,2X,2(F8.2,3X),L2)') &
     & J,INT(IGLNUM(J)),ZAVD,ZLIMIT(J),LLREJ  
  ENDDO
  WRITE(NULOUT,'(''     TOTAL OF '',I10,1x,A5,1x,&
   & '' SCATT REPORTS USED IN THE Q.C. '')') ISUM,CD_MODE  
!*
!             2.3   IF DISTANCE IS EXCESSIVE, THEN REJECT ALL SCATT DATA
!                   OF THIS TYPE

  IF(.NOT.LLREJ) THEN
    WRITE(NULOUT,'(1x,a5,''  SCATT DATA PASSED GLOBAL CHECK '')') CD_MODE
  ELSE
    WRITE(NULOUT,'(1x,a5,''  SCATT DATA FAILED GLOBAL CHECK '')') CD_MODE

    IF (KLEN > 0) THEN
      DO JOBS = 1, KLEN
        IF(LL_TYPE(JOBS)) THEN
          ROBHDR(JOBS,MDBRST)  = ZCHSTAT_REJECT( ROBHDR(JOBS,MDBRST))
          ROBHDR(JOBS,MDBREV1) = ZCHEV1( ROBHDR(JOBS,MDBREV1),NR1EVENT(16))
        ENDIF
      ENDDO
    ENDIF

  ENDIF
ENDIF

!*
!           9.    FINISH
9 CONTINUE
DEALLOCATE(IGLNUM,ZGLDIS,ZLIMIT)

IF (LHOOK) CALL DR_HOOK('SCAQC',1,ZHOOK_HANDLE)
END SUBROUTINE SCAQC
