SUBROUTINE SELEC

!**** SUBROUTINE SELEC - SELECT THE BEST DATA AROUND A STD-LEVEL

!          HEIKKI JARVINEN   ECMWF          15/MAR/1995
!          DRASKO VASILJEVIC ECMWF          21/10/1999

!*    PURPOSE
!     -------

!        MAKE THE SELECTION OF BEST DATA AROUND A STD-PRESSURE LEVEL
!     USING THE COUNTERS AND POINTERS UPDATED DURING A SCAN OVER
!     TEMP/PILOT REPORTS.
!     PRIORITIES ARE AS FOLLOWS:

!        1. MAXIMUM DATUM FLAG
!        2. TIME DIFFERENCE TO THE ANALYSIS TIME
!        3. DISTANCE TO THE STD-LEVEL
!        4  SIGNIFICANT LEVEL
!        5. STANDARD LEVEL
!        6. TEMP OVER PILOT
!        7. LOWEST UNIQUE OBSERVATION SEQUENCE NUMBER

!*    INTERFACE
!     ---------

!        CALL SELEC

!        SELEC IS CALLED BY SUBROUTINE REDUN

!*    EXTERNALS
!     ---------

!        FLGDMX - GET THE MAXIMUM DATUM FLAG

!**   MODIFICATIONS
!     -------------
!     Per Unden       1997-10-23 : Use all non-duplicated levels
!     Heikki Jarvinen 1998-08-20 : Bugfix for using significant levels

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCMBDY , ONLY : NTLIDBP  ,NTDPABP  ,NTDPAOC  ,NTCPABP  ,&
 & NTCPAOC  ,NTBPABP  ,NTBPAOC  ,NTAPABP  ,&
 & NTAPAOC  ,NPLIDBP  ,NPDPABP  ,NPDPAOC  ,&
 & NPCPABP  ,NPCPAOC  ,NPBPABP  ,NPBPAOC  ,&
 & NPAPABP  ,NPAPAOC  
USE YOMLUN   , ONLY : NULOUT
USE YOMCOCTP , ONLY : NTEMP    ,NPILOT   ,NEWPPCD
USE YOMANA   , ONLY : NANDAT   ,NANTIM
USE YOMNMEV  , ONLY : ND1EVENT
USE YOMSCC   , ONLY : LSCDPR
USE YOMLEVSE , ONLY : NSTDPL   ,NLLNVA   ,NVCOUN   ,NVPOIN   ,&
 & GVDIST  
USE YOMVNMB  , ONLY : NVNUMB
USE YOMDB

IMPLICIT NONE


INTEGER(KIND=JPIM) :: IFLAGS(5)

INTEGER(KIND=JPIM) :: IAN, IAPART1, IAPART2, IBPART1, IBPART2,&
 & ICPART1, ICPART2, IOBS1, IOBS2, IDAT1, &
 & IDAT2, IDATPO1, IDATPO2, IDPART1, IDPART2, &
 & IETM1, IETM2, IFLMX1, IFLMX2, ILEVID, IOBTY1, &
 & IOBTY2, IOFF, IONM1, IONM2, IRF1, IRF2, &
 & ISEQ1, ISEQ2, IVNM, J1, J2, JLEV, &
 & ICDTY1, ICDTY2, &
 & JVAR  

LOGICAL :: LLSIGL1,LLSIGL2,LLSTDL1,LLSTDL2,LLSIGL12,LLSTDL12

REAL(KIND=JPRB) :: ZDIST1, ZDIST2, ZPP1, ZPP2, ZFAC
REAL(KIND=JPRB),PARAMETER :: ZPABSDIF=100._JPRB, ZPPERCDIF=0.01_JPRB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "flgdmx.intfb.h"
#include "reftim.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

!        1.       START MAKING SELECTION
!                 ----------------------
!*
!           1.1      LOOP OVER ALL THE STD-LEVELS

IF (LHOOK) CALL DR_HOOK('SELEC',0,ZHOOK_HANDLE)
LEVEL_LOOP: DO JLEV = 1, NSTDPL
!*
!           1.2      LOOP OVER ALL VARIABLES U/V/Z/T/RH

  VARIABLE_LOOP: DO JVAR = 1, NLLNVA
!*
!           1.3      LOOP OVER THE DATA

    DATUM1_LOOP: DO J1 = 1, NVCOUN(JLEV,JVAR)-1
!*
!           1.4      START SECONDARY LOOP

      DATUM2_LOOP: DO J2 = J1+1, NVCOUN(JLEV,JVAR)
!*
!              1.4.1   CHECK WHETHER DATUM IS ALREADY REJECTED

        IF(NVPOIN(JLEV,JVAR,J1,3) == 1) CYCLE DATUM1_LOOP
        IF(NVPOIN(JLEV,JVAR,J2,3) == 1) CYCLE DATUM2_LOOP
!*
!     ------------------------------------------------------------------

!        2.       EXTRACT NECESSARY INFORMATION FROM THE REPORTS
!                 ----------------------------------------------
!*
!           2.1   GET THE PROPER POINTERS AND WORDS FROM THE TWO REPORTS

!              2.1.1 FIRST REPORT

        IOBS1 = NVPOIN(JLEV,JVAR,J1,1)
        IDATPO1 = NVPOIN(JLEV,JVAR,J1,2)
        IF(NVPOIN(JLEV,JVAR,J1,4) == 1) THEN
          ISEQ1   = ROBODY(IDATPO1,MDBESQN)
          IVNM    = ROBODY(IDATPO1,MDBVNM )
          ZPP1    = ROBODY(IDATPO1,MDBPPP )
        ELSE
          CALL ABOR1(' SELEC: NVPOIN ARRAY CORRUPTED (1) ')
        ENDIF

!              2.1.2 SECOND REPORT

        IOBS2 = NVPOIN(JLEV,JVAR,J2,1)
        IDATPO2 = NVPOIN(JLEV,JVAR,J2,2)
        IF(NVPOIN(JLEV,JVAR,J2,4) == 1) THEN
          ISEQ2   = ROBODY(IDATPO2,MDBESQN)
          ZPP2    = ROBODY(IDATPO2,MDBPPP )
        ELSE
          CALL ABOR1(' SELEC: NVPOIN ARRAY CORRUPTED (2) ')
        ENDIF
!*
!              2.1.3 CHECK WHETHER THIS PAIR IS POTENTIALLY REDUNDANT
!                    IF BOTH ARE TEMP OBSERVATION ONLY IDENTICAL PRESSURE
!                    OBSERVATIONS ARE CONSIDERED POTENTIALLY REDUNDANT.
!                    IF AT LEAST ONE OF THE TWO OBSERVATIONS IS A PILOT
!                    DATA IS CONSIDERED REDUNDANT IF THE ABSOLUTE PRESSURE
!                    DIFFERENCE IS LESS THAN 1 HPA AND THE RELATIVE
!                    PRESSURE DIFFERENCE IS LESS THAN 1 %. THE PILOT WILL
!                    THEN BE MADE REDUNDANT.
!                    European Profilers a thinned to every 5 hPa / 5 %.

        IOBTY1 = ROBHDR(IOBS1,MDBOTP)
        IOBTY2 = ROBHDR(IOBS2,MDBOTP)
        ICDTY1 = ROBHDR(IOBS1,MDB_CODETYPE_AT_HDR)
        ICDTY2 = ROBHDR(IOBS2,MDB_CODETYPE_AT_HDR)

        IF(IVNM /= NVNUMB( 3)) THEN
          IF(IOBTY1 == NTEMP .AND. IOBTY2 == NTEMP) THEN
            IF(ZPP1 /= ZPP2) CYCLE
          ELSE
            ZFAC=1.0_JPRB
! European Profilers a thinned to every 5 hPa.
            IF(ICDTY1 == NEWPPCD .AND. ICDTY2 == NEWPPCD) ZFAC=5._JPRB
            IF(ABS(ZPP1-ZPP2) > ZPABSDIF*ZFAC .OR. &
             & ABS(ZPP1-ZPP2)/(0.5_JPRB*(ZPP1+ZPP2)) > ZPPERCDIF*ZFAC) CYCLE  
          ENDIF
        ENDIF
!*
!     ------------------------------------------------------------------

!        3.       COMPARE TWO DATA WITH PRIORITIES
!                 --------------------------------
!*
!           3.1   1. PRIORITY: MAXIMUM DATUM FLAG
!                 -------------------------------
!*
!              3.1.1   EXTRACT THE MAXIMUM DATUM FLAG

        CALL FLGDMX(IDATPO1,IFLMX1,IFLAGS)
        CALL FLGDMX(IDATPO2,IFLMX2,IFLAGS)
!*
!              3.1.2   MAKE THE DECISION

        IF(IFLMX1 > IFLMX2) GOTO 41
        IF(IFLMX1 < IFLMX2) GOTO 42
!*
!           3.2   2. PRIORITY: TIME DIFFERENCE TO THE ANALYSIS TIME
!                 -------------------------------------------------
!*
!              3.2.1   EXTRACT AND CALCULATE THE REFERECE TIMES

        IDAT1 = ROBHDR(IOBS1,MDBDAT)
        IETM1 = ROBHDR(IOBS1,MDBETM)
        IDAT2 = ROBHDR(IOBS2,MDBDAT)
        IETM2 = ROBHDR(IOBS2,MDBETM)
        CALL REFTIM(IDAT1,IETM1,IRF1)
        CALL REFTIM(IDAT2,IETM2,IRF2)
        CALL REFTIM(NANDAT,NANTIM,IAN)
!*
!              3.2.2   MAKE THE DECISION

        IF(ABS(IRF1-IAN)  >  ABS(IRF2-IAN)) GOTO 41
        IF(ABS(IRF1-IAN)  <  ABS(IRF2-IAN)) GOTO 42
!*
!           3.3   3. PRIORITY: DISTANCE TO THE STD-LEVEL, FOR HEIGHTS.
!             FOR OTHER VARIABLES,  REJECT ONLY IF ZDIST1=ZDIST2
!                 ---------------------------------------------------

        ZDIST1 = GVDIST(JLEV,JVAR,J1)
        ZDIST2 = GVDIST(JLEV,JVAR,J2)
        IF(IVNM == NVNUMB( 3) ) THEN
          IF(ZDIST1 > ZDIST2) GOTO 41
          IF(ZDIST1 < ZDIST2) GOTO 42
        ENDIF
!*
!           3.4   4. PRIORITY: SIGNIFICANT LEVEL
!                 ------------------------------
!*
!              3.4.1   EXTRACT THE LEVEL INDICATOR (REPORT 1)

        IOFF = IDATPO1
        IF(IOBTY1 == NTEMP) THEN
          ILEVID  = ROBODY(IDATPO1,MDBTELID)
          IAPART1 = IBITS(ILEVID,NTLIDBP+NTAPABP,NTAPAOC)
          ICPART1 = IBITS(ILEVID,NTLIDBP+NTCPABP,NTCPAOC)
          IBPART1 = IBITS(ILEVID,NTLIDBP+NTBPABP,NTBPAOC)
          IDPART1 = IBITS(ILEVID,NTLIDBP+NTDPABP,NTDPAOC)
        ELSEIF(IOBTY1 == NPILOT) THEN
          ILEVID  = ROBODY(IDATPO1,MDBPILID)
          IAPART1 = IBITS(ILEVID,NPLIDBP+NPAPABP,NPAPAOC)
          ICPART1 = IBITS(ILEVID,NPLIDBP+NPCPABP,NPCPAOC)
          IBPART1 = IBITS(ILEVID,NPLIDBP+NPBPABP,NPBPAOC)
          IDPART1 = IBITS(ILEVID,NPLIDBP+NPDPABP,NPDPAOC)
        ENDIF
!*
!              3.4.2   EXTRACT THE LEVEL INDICATOR (REPORT 2)

        IOFF = IDATPO2
        IF(IOBTY2 == NTEMP) THEN
          ILEVID  = ROBODY(IDATPO2,MDBTELID)
          IAPART2 = IBITS(ILEVID,NTLIDBP+NTAPABP,NTAPAOC)
          ICPART2 = IBITS(ILEVID,NTLIDBP+NTCPABP,NTCPAOC)
          IBPART2 = IBITS(ILEVID,NTLIDBP+NTBPABP,NTBPAOC)
          IDPART2 = IBITS(ILEVID,NTLIDBP+NTDPABP,NTDPAOC)
        ELSEIF(IOBTY2 == NPILOT) THEN
          ILEVID  = ROBODY(IDATPO2,MDBPILID)
          IAPART2 = IBITS(ILEVID,NPLIDBP+NPAPABP,NPAPAOC)
          ICPART2 = IBITS(ILEVID,NPLIDBP+NPCPABP,NPCPAOC)
          IBPART2 = IBITS(ILEVID,NPLIDBP+NPBPABP,NPBPAOC)
          IDPART2 = IBITS(ILEVID,NPLIDBP+NPDPABP,NPDPAOC)
        ENDIF
        IF(LSCDPR .AND.&
           & (IAPART1 /= IAPART2.OR. ICPART1 /= ICPART2.OR.&
           & IBPART1 /= IBPART2.OR. IDPART1 /= IDPART2)) THEN  
          WRITE(NULOUT,'('' IN LEVEL SELECTION: '')')
          WRITE(NULOUT,'('' A/C/B/D PARTS FOR 1 '',I1,''/'',&
           & I1,''/'',&
           & I1,''/'',I1,''/'')') IAPART1,ICPART1,IBPART1,IDPART1  
          WRITE(NULOUT,'('' A/C/B/D PARTS FOR 2 '',I1,''/'',&
           & I1,''/'',&
           & I1,''/'',I1,''/'')') IAPART2,ICPART2,IBPART2,IDPART2  
          WRITE(NULOUT,'('' '')')
        ENDIF
!*
!              3.4.3   PREPARE FOR DECISION TAKING

        LLSIGL1 = .FALSE.
        IF(IBPART1 == 1.OR.IDPART1 == 1) LLSIGL1 = .TRUE.
        LLSIGL2 = .FALSE.
        IF(IBPART2 == 1.OR.IDPART2 == 1) LLSIGL2 = .TRUE.
        LLSTDL1 = .FALSE.
        IF(IAPART1 == 1.OR.ICPART1 == 1) LLSTDL1 = .TRUE.
        LLSTDL2 = .FALSE.
        IF(IAPART2 == 1.OR.ICPART2 == 1) LLSTDL2 = .TRUE.
        LLSIGL12 = LLSIGL1 .AND. LLSIGL2
        LLSTDL12 = LLSTDL1 .AND. LLSTDL2

!*
!              3.4.3.1 BOTH SIGNIFICAT LEVELS (DO NOTHING FOR NOW)

        IF(LLSIGL12)   THEN
          GO TO 35
!*
!              3.4.3.2 EITHER BOTH OR ONE NON-SIGNIFICAT LEVELS

        ELSE
          IF(LLSIGL1) GO TO 42
          IF(LLSIGL2) GO TO 41
        ENDIF

!       IF(IBPART1 == 1.AND.(IBPART2 == 0.AND. IDPART2 == 0)) GOTO 42
!       IF(IDPART1 == 1.AND.(IBPART2 == 0.AND. IDPART2 == 0)) GOTO 42
!*
!           3.5   5. PRIORITY: STANDARD LEVEL
!                 ---------------------------

        35     CONTINUE
!*
!              3.5.1   BOTH STANDARD LEVELS
        LLSTDL2 = .FALSE.

        IF(LLSTDL12)     THEN
          GO TO 36
!*
!              3.5.2   NONE OR ONE STANDARD LEVELS

        ELSE
          IF(LLSTDL1) GO TO 42
          IF(LLSTDL2) GO TO 41
        ENDIF
!       IF(IAPART1 == 1.AND.(IAPART2 == 0.AND. ICPART2 == 0)) GOTO 42
!       IF(ICPART1 == 1.AND.(IAPART2 == 0.AND. ICPART2 == 0)) GOTO 42
!*
!           3.6   6. PRIORITY: TEMP OVER PILOT
!                 ----------------------------

        36     CONTINUE
        IF(IOBTY1 == NPILOT .AND. IOBTY2 == NTEMP ) GOTO 41
        IF(IOBTY1 == NTEMP  .AND. IOBTY2 == NPILOT) GOTO 42
!*
!           3.7   7. PRIORITY: LOWER UNIQUE OBSERVATION SEQUENCE NUMBER
!                 -----------------------------------------------------
!*
!              3.7.1   EXTRACT THE SEQUENCE NUMBERS

        IONM1 = ROBHDR(IOBS1,MDBONM)
        IONM2 = ROBHDR(IOBS2,MDBONM)
!*
!              3.7.1   MAKE THE DECISION

        IF(IONM1 > IONM2) GOTO 41
        IF(IONM1 < IONM2) GOTO 42
!*
!           3.8   8. PRIORITY: TAKE THE DATUM WITH LOWER SEQUANCE NUMBER
!                 ------------------------------------------------------
!*
!              3.8.1   MAKE THE DECISION

        IF(ISEQ1 > ISEQ2) GOTO 41
        IF(ISEQ1 < ISEQ2) GOTO 42
!*
!           3.9   9. PRIORITY: IF STILL EQUAL, SOMETHING IS BADLY WRONG
!                 -----------------------------------------------------

        WRITE(NULOUT,'('' LOGICAL ERROR IN SELEC '')')
        WRITE(NULOUT,'('' - OBS. SEQUENCE NO.'',2I8)') IONM1,IONM2
        WRITE(NULOUT,'('' - VAR. SEQUENCE NO.'',2I8)') ISEQ1,ISEQ2
        WRITE(NULOUT,'('' - OBS. POINTERS    '',2I8)') IOBS1,IOBS2
        WRITE(NULOUT,'('' - DAT. POINTERS    '',2I8)') IDATPO1,IDATPO2
        CALL ABOR1(' SELEC: LOGICAL ERROR ')
!*
!     ------------------------------------------------------------------

!        4.       REJECT DATUM AND PREVENT REPETITIVE REJECTION
!                 ---------------------------------------------
!*
!           4.1      REJECT THE FIRST ONE

        41 CONTINUE
        ROBODY(IDATPO1,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IDATPO1,MDBDSTA))
        ROBODY(IDATPO1,MDBDEV1) = ZCHEV1( ROBODY(IDATPO1,MDBDEV1),ND1EVENT(14))
        NVPOIN(JLEV,JVAR,J1,3) = 1
        CYCLE DATUM1_LOOP
!*
!           4.2      REJECT THE SECOND ONE

        42 CONTINUE
        ROBODY(IDATPO2,MDBDSTA) = ZCHSTAT_REJECT( ROBODY(IDATPO2,MDBDSTA))
        ROBODY(IDATPO2,MDBDEV1) = ZCHEV1( ROBODY(IDATPO2,MDBDEV1),ND1EVENT(14))
        NVPOIN(JLEV,JVAR,J2,3) = 1
        CYCLE DATUM2_LOOP
!*
!     ------------------------------------------------------------------

!        5.       CLOSE THE LOOP OVER THE ...
!                 ---------------------------

!           5.1      ... DATUM2

      ENDDO DATUM2_LOOP

!           5.2      ... DATUM1

    ENDDO DATUM1_LOOP

!           5.3      ... VARIABLES

  ENDDO VARIABLE_LOOP

!           5.4      ... STANDARD LEVELS

ENDDO LEVEL_LOOP

!*
!     ------------------------------------------------------------------

!        6.       RETURN
!                 ------

IF (LHOOK) CALL DR_HOOK('SELEC',1,ZHOOK_HANDLE)
END SUBROUTINE SELEC
