SUBROUTINE SONDE_COUNTRY_DB_MATCH(CDSTID,KTYPESOND,KCDTYP,PLAT,PLON,&
                                 &KSONDE_DB_ENTRY,KCOUNTRY_DB_ENTRY  )

!**** *BIASCOR_ODB*

!       PURPOSE.
!      ----------

!         BIAS CORRECTION FOR RS TEMPERATURES

!       INTERFACE.
!      ------------

!         CALL BIASCOR_ODB(LDBC   ,KNOPLE,PPRES,PT,PTBC,
!     X                    CDIDENT,KM    ,KD   ,KH,PLAT,PLON,KRSTYP,
!     X                    KMISS  ,PMISS                             )

!        INPUT
!         KNOPLEV   NUMBER OF LEVELS
!         PPRES     ARRAY WITH PRESSURE VALUES
!         PT        ARRAY WITH T VALUES
!         CDIDENT   STATION IDENTIFIER
!         KY        YEAR
!         KM        MONTH
!         KD        DAY
!         KH        HOUR
!         PLAT      LATITUDE
!         PLON      LONGITUD
!         PALT      ALTITUDE
!         KRSTYP    RADIOSONDE TYPE (BUFR CODE TABLE 002011)
!         KMISS     MISSING VALUE FOR INTEGERS
!         PMISS     MISSING VALUE FOR REALS

!        OUTPUT
!         LDBC      LOGICAL TO STATE IF BIAS CORRECTION WAS SUCCESSFUL
!         PTBC      ARRAY WITH BIAS CORRECTED T VALUES

!       METHOD.
!      ---------

!        READ BIAS CORRECTION TABLES:
!            TABLE 1 : CROSS-REF STATION ID, SONDE NAME
!            TABLE 2 : ARRAY OF CORRECTIONS DEPENDING ON
!                      SOLAR ELEVATION AND PRESSURE LEVEL
!            TABLE 3 : CROSS-REF SONDE TYPE CODE, SONDE NAME
!        TRY FOR MATCH IN TABLE 3, IF NOT THEN TABLE 1
!        DETERMINE CORRECTION TO BE APPLIED FROM TABLE 2

!       EXTERNALS.
!      ------------

!        DIURNAL          CALCULATE SOLAR ELEVATION
!        PNTERP           INTERPOLATE TO PRESSURE LEVEL

!       REFERENCE.
!      ------------

!        RADIOSONDE BIAS CORRECTION
!        OD MEMORANDUM BY B. STRAUSS  22.12.92

!       AUTHOR.
!      ---------

!        D. VASILJEVIC  21/10/03 (ORIGINAL B. NORRIS JULY  1991)

!       MODIFICATIONS.
!      ----------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Bogatchev : 26-Oct-2007 removing of severe breaks of coding rools

!-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMANCS  , ONLY : NMDI
USE YOMRSTRHBIAS
USE YOMCOCTP , ONLY : NLDTCD
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

!     DUMMY INTEGERs
INTEGER(KIND=JPIM),INTENT(IN) :: KTYPESOND
INTEGER(KIND=JPIM),INTENT(IN) :: KCDTYP
INTEGER(KIND=JPIM),INTENT(OUT) :: KSONDE_DB_ENTRY
INTEGER(KIND=JPIM),INTENT(OUT) :: KCOUNTRY_DB_ENTRY
INTEGER(KIND=JPIM) :: ISTID
INTEGER(KIND=JPIM) :: ILAT
INTEGER(KIND=JPIM) :: ILON
CHARACTER(LEN=*),INTENT(IN)   :: CDSTID 
REAL(KIND=JPRB),INTENT(IN)    :: PLAT 
REAL(KIND=JPRB),INTENT(IN)    :: PLON 

INTEGER(KIND=JPIM) :: JSONDE, JCOUNTRY
REAL(KIND=JPRB)    :: ZHOOK_HANDLE


!  -------------------------------------------------------------------

!                  1.  PRESET
!                      ------

IF (LHOOK) CALL DR_HOOK('SONDE_COUNTRY_DB_MATCH',0,ZHOOK_HANDLE)

WRITE(NULOUT,'('' SONDE_COUNTRY_DB_MATCH: '',A,X,I5,X,I5,2(F9.2,X))') CDSTID,KTYPESOND,KCDTYP,PLAT,PLON
CALL FLUSH(NULOUT)

KSONDE_DB_ENTRY   = NMDI
KCOUNTRY_DB_ENTRY = NMDI

!  ------------------------------------------------------------------

!                  2.  MATCH SONDE AND COUNTRY
!                      -----------------------


!        2.1           SONDE MATCH

IF(KTYPESOND /= NMDI) THEN

  SONDE_LOOP: DO JSONDE =  1 , NO_TRH_BIAS_SONDES
    IF(KTYPESOND == M_SONDE_CODE(JSONDE) ) THEN
      KSONDE_DB_ENTRY =JSONDE
      WRITE(NULOUT,'('' SONDE TYPE MATCHED '')')
      CALL FLUSH(NULOUT)
      GO TO 210
    ENDIF
  enddo SONDE_LOOP
  WRITE(NULOUT,'('' SONDE TYPE NOT MATCHED '')')
  CALL FLUSH(NULOUT)
  210 CONTINUE

endif
      
!        2.2           COUNTRY MATCH

IF(KCDTYP == NLDTCD) THEN

  if(CDSTID(4:4) == '0') then
    READ(CDSTID(1:8),'(4x,i4.4)') ISTID
  else
    READ(CDSTID(1:8),'(3x,i5.5)') ISTID
  endif

! Known new stations (Croat.)
  if(ISTID/1000 == 14) ISTID = 13*1000+(ISTID-(ISTID/1000)*1000)

  ILAT      = NINT(PLAT*100.)
  ILON      = NINT(PLON*100.)

  COUNTRY_LOOP: DO JCOUNTRY = 1,NO_TRH_BIAS_COUNTRIES

    IF(ISTID >= M_STID_FROM(JCOUNTRY) .and. ISTID <= M_STID_TO(JCOUNTRY)) THEN
      IF(ILAT <= M_LAT_TO(JCOUNTRY)*100 .and. ILAT >= M_LAT_FROM(JCOUNTRY)*100) then
        IF(ILON >= M_LON_FROM(JCOUNTRY)*100 .AND. ILON <= M_LON_TO(JCOUNTRY)*100) THEN
!         KCOUNTRY_DB_ENTRY = M_COUNTRY_P(JCOUNTRY)
          KCOUNTRY_DB_ENTRY = M_COUNTRY_SQN(JCOUNTRY)
          WRITE(NULOUT,'('' COUNTRY MATCHED '')')
          CALL FLUSH(NULOUT)
          GO TO 220
        ENDIF
      ENDIF
    ENDIF

  enddo COUNTRY_LOOP
  WRITE(NULOUT,'('' COUNTRY NOT MATCHED '')')
  CALL FLUSH(NULOUT)
  220 CONTINUE
else
  WRITE(NULOUT,'('' COUNTRY MATCH NOT DONE FOR NON LAND RS '')')
  CALL FLUSH(NULOUT)
ENDIF

!--------------------------------------------------------------------

!                   4.  RETURN
!                       ------

IF (LHOOK) CALL DR_HOOK('SONDE_COUNTRY_DB_MATCH',1,ZHOOK_HANDLE)
END SUBROUTINE SONDE_COUNTRY_DB_MATCH
