SUBROUTINE SSMIS_FLAGGING(KOBS)

!**** SUBROUTINE SSMIS_FLAGGING - FLAG CONTAMINATED SSMIS SPECTRA

!          NIELS BORMANN     ECMWF          23/AUG/2006

!*    PURPOSE
!     -------

!        FLAG SSMIS CLEAR-SKY SPECTRA THAT APPEAR INCORRECT

!**   INTERFACE
!     ---------

!     ***CALL*SSMIS_FLAGGING(KOBS)
!     WHERE KSET = NUMBER OF THE OBSERVATION RECORD

!        SSMIS_FLAGGING IS CALLED BY SUBROUTINE FIRST

!*    METHOD
!     ------

!     Records are flagged if the absolute value of the 
!     average departure in the AMSU-A like channels is
!     above a certain threshold. This is to remove data 
!     that suffers from poor corrections to the reflector
!     temperature or undetected solar intrusions.

!*    EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMNMEV  , ONLY : NR1EVENT
USE YOMVNMB  , ONLY : NVNUMB

USE YOMOBSET_THSAFE
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KOBS   ! Observation number

! SSMIS AMSU-A like channels considered
INTEGER(KIND=JPIM),DIMENSION(8)  :: JPSSMIS_AMSUCH= &
 &      (/ 3,         4,         5,         6,        7,&
 &        22,         23,        24 /)
! Typical Obs-FG standard deviations for these channels
REAL(KIND=JPRB), DIMENSION(8)    :: ZPSSMIS_OMF=   &
 &      (/ 0.18_JPRB, 0.14_JPRB, 0.20_JPRB, 0.22_JPRB, 0.22_JPRB,&
 &         0.85_JPRB, 0.60_JPRB, 0.60_JPRB /)

INTEGER(KIND=JPIM) :: JBODY
INTEGER(KIND=JPIM) :: ICHAN,INCHAN
INTEGER(KIND=JPIM) :: IBAD
INTEGER(KIND=JPIM),DIMENSION(1) :: IICHAN
REAL(KIND=JPRB)    :: ZOMF         

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcobs.h"

#include "openmp_obs.h"
!*
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SSMIS_FLAGGING',0,ZHOOK_HANDLE)

INCHAN=0
IBAD=0
ZOMF=0.0_JPRB

! Sum up departures (normalised by typical stdev(O-F)) of data not 
! currently rejected
! Also count data with too large departures, with special emphasis on ch 5
DO JBODY = MLNKH2B(KOBS),MLNKH2B(KOBS+1)-1
  IF(ROBODY(JBODY,MDBVNM) /= NVNUMB(50)) CYCLE
  IF(ROBODY(JBODY,MDBDSTA) == ZCHSTAT_REJECT( ROBODY(JBODY,MDBDSTA))) CYCLE

  ICHAN=NINT(ROBODY(JBODY,MDBPPP))

  IF(ANY(JPSSMIS_AMSUCH == ICHAN))THEN
    IICHAN = MAXLOC(JPSSMIS_AMSUCH, MASK=JPSSMIS_AMSUCH == ICHAN)
    ZOMF   = ZOMF + ROBODY(JBODY,MDBOMF)/ZPSSMIS_OMF(IICHAN(1))
    IF(ABS(ROBODY(JBODY,MDBOMF)) > ZPSSMIS_OMF(IICHAN(1)))THEN
      IF(ICHAN == 5 .AND. ABS(ROBODY(JBODY,MDBOMF)) > 0.25_JPRB)THEN
        IBAD = IBAD + 10
      ELSE
        IBAD = IBAD + 1
      ENDIF
    ENDIF
    INCHAN  = INCHAN+1
  ENDIF
ENDDO

IF(INCHAN > 0)THEN
  ! Reject records which have too large average departure or 
  ! too many large departures
  IF( ABS(ZOMF/INCHAN) >= 1.0_JPRB .OR. IBAD >= 9+0.33_JPRB*INCHAN )THEN
    ROBHDR(KOBS,MDBRST) =ZCHSTAT_REJECT(ROBHDR(KOBS,MDBRST))
    ROBHDR(KOBS,MDBREV1)=ZCHEV1(ROBHDR(KOBS,MDBREV1),NR1EVENT( 3))
  ENDIF
ENDIF


IF (LHOOK) CALL DR_HOOK('SSMIS_FLAGGING',1,ZHOOK_HANDLE)
END SUBROUTINE SSMIS_FLAGGING
