SUBROUTINE TOVSLRIS(KBCDTYP,KINS,KDPT,KA,KB,KC,KV,KW,KX,KY,KZ )

!**** SUBROUTINE TOVSLRIS - LOW RESOLUTION TOVS INSTRUMENT SPECIFICATION

!        D. VASILJEVIC   ECMWF     24/02/95

!     PURPOSE
!     -------

!        CONSTRUCT INSTRUMENT SPECIFICATION/TYPE FOR LOW RESOLUTION
!     TOVS

!**   INTERFACE
!     ---------

!        CALL TOVSLRIS(KBCDTYP,KINS,KDPT,
!                      KA,KB,KC,KV,KW,KX,KY,KZ )

!     WHERE:
!           KBCDTYP = BUFR CODE TYPE
!           KINS    = INPUT SATELLITE INSTRUMENT VALUE (BUFR TABLE 2021)
!           KDPT    = INPUT DATA PROCESSING TECHNIQUE (BUFR TABLE 2022)
!           KA      = OUTPUT A PARAM.
!           KB      = OUTPUT B PARAM.
!           KC      = OUTPUT C PARAM.
!           KV      = OUTPUT V PARAM.
!           KW      = OUTPUT W PARAM.
!           KX      = OUTPUT X PARAM.
!           KY      = OUTPUT Y PARAM.
!           KZ      = OUTPUT Z PARAM.

!        TOVSLRIS IS CALLED BY SUBROUTINE OBINSSP

!     METHOD
!     ------

!        SEE RESEARCH MANUAL 5, TABLE 7.13 FOR A DESCRIPTION OF THE TOVS
!     INSTRUMENT SPECIFICATION WORD.

!     EXTERNALS
!     ---------

!        NONE

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning 

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMANCS  , ONLY : NMDI     ,NBITS    ,RADIANS  ,RTT      ,&
 & R2ES     ,R3LES    ,R3IES    ,R4LES    ,R4IES    ,&
 & RTWAT    ,RTICE    ,EPSMAC  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KBCDTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KINS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPT 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KA 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KB 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KC 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KV 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KW 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KX 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KY 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KZ 
INTEGER(KIND=JPIM) :: IA, IB, IC, ICLD, ICLR, IHIRS, IMSU, IPCLD,&
 & IS, ISSU, IV, IW, IX, IY, IZ

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

#include "fcobs.h"
#include "stmfun_ifs.h"
!*

!        1.             CHANNEL COMBINATIONS
!                       --------------------

IF (LHOOK) CALL DR_HOOK('TOVSLRIS',0,ZHOOK_HANDLE)
IV    = 0
IW    = 0
IX    = 0
IY    = 0
IZ    = 0

IHIRS = 0
IMSU  = 0
ISSU  = 0

!*          1.1         MISSING VALUE FOR INSTRUMENT WORD

IF(KINS  == NBITS( 9).OR.KINS  ==      NMDI    )                    GO TO 2

IHIRS = IBITS(KINS, 8,1)
IMSU  = IBITS(KINS, 7,1)
ISSU  = IBITS(KINS, 6,1)

!*          1.2         V - CHANNEL COMBINATIONS USED TO OBTAIN LAYER
!                       PWC FOR SURFACE-700MB, 700-500MB AND 500-300MB

IF(KBCDTYP == 72)                  THEN
  IF(IHIRS == 1.AND.IMSU  == 1     )             THEN
    IV = 1
  ELSEIF(IHIRS == 1)             THEN
    IV = 2
  ENDIF
ENDIF

!*          1.3         Y - CHANNEL COMBINATIONS USED TO OBTAIN
!                       TROPOSPHERIC LAYER TEMPERATURES

IF(KBCDTYP == 71)                 THEN
  IF(IHIRS == 1.AND.IMSU  == 1     )               THEN
    IY = 1
  ELSEIF(IHIRS == 1)               THEN
    IY = 3
  ELSEIF(IMSU  == 1)               THEN
    IY = 4
  ENDIF
ENDIF

!*          1.4         Z - CHANNEL COMBINATIONS USED TO OBTAIN
!                       STRATOSPHERIC LAYER TEMPERATURES

IF(KBCDTYP == 73)                 THEN
  IF(IHIRS == 1.AND.ISSU  == 1.AND.IMSU  == 1     )               THEN
    IZ = 1
  ELSEIF(IHIRS == 1.AND.IMSU  == 1     )          THEN
    IZ = 2
  ELSEIF(ISSU  == 1.AND.IMSU  == 1     )          THEN
    IZ = 3
  ELSEIF(IHIRS == 1.AND.ISSU  == 1     )          THEN
    IZ = 4
  ELSEIF(IHIRS == 1     )          THEN
    IZ = 5
  ELSEIF(IMSU  == 1     )          THEN
    IZ = 6
  ENDIF
ENDIF
!*
!-----------------------------------------------------------------------

!        2.             DATA PROCESSING TECHNIQUE
!                       -------------------------

2 CONTINUE

!*          2.1         MISSING VALUES FOR DATA PROCESSING TECHNIQUE
!                       AND INSTRUMENT SPECIFICATION WORD

IF(KDPT == NBITS( 8).OR.KDPT ==      NMDI    )                  THEN
  IF(KINS == NBITS( 9).OR.KINS ==      NMDI    )                  THEN
    IA = 0
    IB = 0
    IC = 0

!*          2.2         DATA PROCESSING TECHNIQUE MISSING. USE
!                       INSTRUMENT SPECIFICATION WORD TO TRY TO DEDUCE
!                       DATA PROCESSING TECHNIQUE (CAN'T DISTINGUISH
!                       BETWEEN CLEAR AND PARTLY CLOUDY)

  ELSE
    IF(IHIRS == 1)                          THEN
      IA = 1
      IB = 1
      IC = 1
    ELSE
      IA = 0
      IB = 0
      IC = 0
    ENDIF
  ENDIF
  GO TO 3
ENDIF

!*          2.3         DATA PROCESSING TECHNIQUE SPECIFIED
!                       A - CLEAR RADIANCES
!                       B - CLOUDY RADIANCES

ICLR  = IBITS(KDPT, 5,1)
IPCLD = IBITS(KDPT, 4,1)
ICLD  = IBITS(KDPT, 3,1)
IF(ICLR == 1)                             THEN
  IA = 1
  IB = 1
ELSEIF(IPCLD == 1)                       THEN
  IA = 2
  IB = 1
ELSE
  IA = 0
  IB = 2
ENDIF

!*          2.4         C - STATISTICAL RETRIEVAL METHOD

IS = IBITS(KDPT, 6,1)
IF(IS == 1)                               THEN
  IC = 1
ELSE
  IC = 0
ENDIF
!*
!-----------------------------------------------------------------------

!        3.             INSTRUMENT SPECIFICATION/TYPE
!                       -----------------------------
3 CONTINUE

!*          3.1         INSTRUMENT SPECIFICATION PARAMETERS

KA = IA
KB = IB
KC = IC
KV = IV
KW = IW
KX = IX
KY = IY
KZ = IZ
!*
!-----------------------------------------------------------------------

!        4.             RETURN
!                       ------
IF (LHOOK) CALL DR_HOOK('TOVSLRIS',1,ZHOOK_HANDLE)
END SUBROUTINE TOVSLRIS
