subroutine onedvar_lintest (ksat, k1d_x, k1d_y, k1d_z, knumb, &
 &      pt         , &   ! in:  temperature
 &      pq         , &   ! in:  spec. humidity
 &      pten_t     , &   ! in:  temperature tendency 
 &      pten_q     , &   ! in:  spec. humidity tendency         
 &      pt_2m      , &   ! in:  temperature @ 2m
 &      pq_2m      , &   ! in:  spec. humidity @ 2m
 &      ptd_2m     , &   ! in:  dew-point temperature @ 2m
 &      pu_star    , &   ! in:  wind stress, u
 &      pv_star    , &   ! in:  wind stress, v                             
 &      pu_10m     , &   ! in:  wind speed, u @ 10m
 &      pv_10m     , &   ! in:  wind speed, v @ 10m
 &      psh_sfc    , &   ! in:  sensible heat flux @ sfc
 &      plh_sfc    , &   ! in:  latent heat flux @ sfc
 &      psea_ice   , &   ! in:  sea-ice matrix
 &      pland_sea  , &   ! in:  land-sea matrix
 &      p_sfc      , &   ! in:  pressure @ sfc
 &      pt_sfc     , &   ! in:  skin temperature @ sfc  
 &      pperturb   , &   ! in:  (x) perturbation 
 &      p1d_y_lin )      ! out: (y) linearity test
     
!**** 1D-Var linearity test in clouds/precipitation for SSM/I radiances
!     P. Bauer    May 2005

!     Runs TL vs. finite differences for testing observation operator linearity

!     ONEDVAR_LINTEST is called from ONEDVAR_RAINTB

!     Modifications:
!      A Geer 28/11/06 to run linearity test with a supplied perturbation and state
!                      Any reference to the FG/analysis is now irrelevant here.
!                      Also updated to use 10m u&v wind in the control vector

!**** MODULES

!* KIND     
use parkind1     , only: jpim, jprb
use yomhook      , only: lhook,   dr_hook

!* IFS
use yomdim       , only: nflevg 

implicit none 

!**** VARIABLES
!* IN/OUTPUT
integer (kind=jpim), intent (in) :: ksat, k1d_x, k1d_y, k1d_z, knumb

real (kind=jprb), intent (in), dimension (nflevg) :: &
 & pt       , &
 & pq       , &
 & pten_t   , &
 & pten_q   
         
real (kind=jprb), intent (in) :: &
 & pt_2m    , &
 & pq_2m    , &
 & ptd_2m   , &
 & pu_star  , &
 & pv_star  , &
 & pu_10m   , &
 & pv_10m   , &
 & psh_sfc  , &
 & plh_sfc  , &
 & psea_ice , &
 & pland_sea, &
 & p_sfc    , &
 & pt_sfc
 
real (kind=jprb), intent (in), dimension (k1d_x) :: pperturb 
 
real (kind=jprb), intent (out), dimension (k1d_y) :: p1d_y_lin 

!* OTHER
real (kind=jprb), dimension (k1d_x)       :: z1d_x   , z1d_x_fd   , z1d_x_tl   
real (kind=jprb), dimension (k1d_y)       :: z1d_y   , z1d_y_fd   , z1d_y_tl    

real (kind=jprb), dimension (k1d_z)       :: z1d_x_xt, z1d_x_xt_fd, z1d_x_xt_tl   
              
real (kind=jprb), dimension (knumb,k1d_x) :: zz1d_x   , zz1d_x_tl
real (kind=jprb), dimension (knumb,k1d_y) :: zz1d_y   , zz1d_y_tl
real (kind=jprb), dimension (knumb,k1d_z) :: zz1d_x_xt, zz1d_x_xt_tl

real (kind=jprb) :: zhook_handle

#include "onedvar_obsop.intfb.h"
#include "onedvar_obsop_tl.intfb.h"

if (lhook) call dr_hook ('ONEDVAR_LINTEST',0,zhook_handle)

!* Initialize
z1d_x    (:) = 0.0_JPRB
z1d_y    (:) = 0.0_JPRB
z1d_x_xt (:) = 0.0_JPRB

!** Linearisation state
zz1d_x    (:,:) = 0.0_JPRB
zz1d_y    (:,:) = 0.0_JPRB
zz1d_x_xt (:,:) = 0.0_JPRB

zz1d_x    (knumb,         1:  nflevg) = pt     (1:nflevg) 
zz1d_x    (knumb,  nflevg+1:2*nflevg) = pq     (1:nflevg) 
zz1d_x    (knumb,  2*nflevg+1       ) = pu_10m  
zz1d_x    (knumb,  2*nflevg+2       ) = pv_10m 
        
zz1d_x_xt (knumb,         1:  nflevg) = pten_t (1:nflevg)
zz1d_x_xt (knumb,  nflevg+1:2*nflevg) = pten_q (1:nflevg)
zz1d_x_xt (knumb,7*nflevg+1         ) = pt_2m     
zz1d_x_xt (knumb,7*nflevg+2         ) = pq_2m     
zz1d_x_xt (knumb,7*nflevg+3         ) = ptd_2m       
zz1d_x_xt (knumb,7*nflevg+4         ) = pu_star   
zz1d_x_xt (knumb,7*nflevg+5         ) = pv_star   
zz1d_x_xt (knumb,7*nflevg+6         ) = psh_sfc   
zz1d_x_xt (knumb,7*nflevg+7         ) = plh_sfc   
zz1d_x_xt (knumb,7*nflevg+8         ) = pland_sea 
zz1d_x_xt (knumb,7*nflevg+9         ) = p_sfc     
zz1d_x_xt (knumb,7*nflevg+10        ) = pt_sfc    
      
z1d_x    (:) = zz1d_x    (knumb,:)
z1d_x_xt (:) = zz1d_x_xt (knumb,:)

call onedvar_obsop (ksat, knumb, k1d_x, k1d_y, k1d_z, zz1d_x, zz1d_x_xt, zz1d_y, .true.)
 
z1d_y    (:) = zz1d_y (knumb,:)
z1d_x_xt (2*nflevg+1 :7*nflevg   ) = zz1d_x_xt (knumb,2*nflevg+1 :7*nflevg   )
z1d_x_xt (7*nflevg+11:7*nflevg+12) = zz1d_x_xt (knumb,7*nflevg+11:7*nflevg+12)

!** tangent linear
zz1d_x_tl    (:,:) = 0.0_JPRB
zz1d_y_tl    (:,:) = 0.0_JPRB
zz1d_x_xt_tl (:,:) = 0.0_JPRB

!* Perturbation
zz1d_x_tl (knumb,:) = pperturb(:)  

!* Reset input
zz1d_x    (knumb,:                  ) = z1d_x    (:)
zz1d_y    (:    ,:                  ) = 0.0_JPRB
zz1d_x_xt (knumb,:                  ) = z1d_x_xt (:)
zz1d_x_xt (knumb,2*nflevg+1:7*nflevg) = 0.0_JPRB

call onedvar_obsop_tl (ksat, knumb, k1d_x, k1d_y, k1d_z, zz1d_x, &
 & zz1d_x_xt, zz1d_y, zz1d_x_tl, zz1d_x_xt_tl, zz1d_y_tl)

z1d_x_tl    (:) = zz1d_x_tl    (knumb,:)
z1d_y_tl    (:) = zz1d_y_tl    (knumb,:)
z1d_x_xt_tl (:) = zz1d_x_xt_tl (knumb,:)

!** finite difference
zz1d_x    (knumb,:) = z1d_x(:) + pperturb(:) 
      
zz1d_x_xt (knumb,:                  ) = z1d_x_xt (:)
zz1d_x_xt (knumb,2*nflevg+1:7*nflevg) = 0.0_JPRB
zz1d_y    (:    ,:                  ) = 0.0_JPRB

call onedvar_obsop (ksat, knumb, k1d_x, k1d_y, k1d_z, zz1d_x, zz1d_x_xt, zz1d_y)
      
z1d_x_fd    (:) = zz1d_x    (knumb,:)
z1d_y_fd    (:) = zz1d_y    (knumb,:)
z1d_x_xt_fd (:) = zz1d_x_xt (knumb,:)

!* Linearity test 
p1d_y_lin (:) = 1.0E+10_JPRB
where (z1d_y_tl (:) /= 0.0_JPRB) &
 & p1d_y_lin (:) = (z1d_y_fd (:) - z1d_y (:)) / z1d_y_tl (:)
      
if (lhook) call dr_hook ('ONEDVAR_LINTEST',1,zhook_handle)

end subroutine onedvar_lintest

