subroutine onedvar_obsop (ksat, kproma, k1d_x, k1d_y, k1d_z, &      
                       &  p1d_x5, p1d_x_xt5, p1d_y, ld1d_physics, &
                       &  p1d_y_clear, pcfrac)

!**** Observation operator for 
!     1D-Var in clouds/precipitation for SSM/I radiances 

!     P. Bauer, P. Lopez, E. Moreau, D. Salmond   ECMWF    May 2004

!     1. Call simplified moist physics (by default or if optional
!        parameter ld1d_physics is true. If ld1d_physics is false,
!        cloud profiles may be supplied in p1d_x_xt5 - note, not in ad or tl)
!     2. Call RTTOV-interface of observation operator

!     ONEDVAR_OBSOP is called from ONEDVAR_RAINTB, ONEDVAR_SIMUL,
!                                  ONEDVAR_SCREEN, ONEDVAR_LINTEST

!     Modifications:

!     A. Geer     22-Mar-2006  Add 10m wind to control vector
!     A. Geer     05-Feb-2007  Add option to bypass the linear physics
!     C. O'Dell      Dec-2006  Slight changes necessary for multiple sensors
!     A. Geer     11-Nov-2007  RTTOV9 compatible

!**** MODULES      

!* KIND     
use parkind1, only: jpim, jprb
use yomhook , only: lhook,   dr_hook

!* IFS
use yomdim  , only: nflevg                      
use yomgem  , only: vah, vbh

implicit none
         
!**** VARIABLES
!* IN/OUTPUT
integer (kind=jpim), intent    (in)                           :: ksat, kproma, k1d_x, k1d_y, k1d_z
real    (kind=jprb), intent    (in), dimension (kproma,k1d_x) :: p1d_x5
real    (kind=jprb), intent (inout), dimension (kproma,k1d_z) :: p1d_x_xt5
real    (kind=jprb), intent   (out), dimension (kproma,k1d_y) :: p1d_y   
logical,             intent    (in), optional                 :: ld1d_physics
! Optional diagnostic variables
real    (kind=jprb), intent   (out), optional, dimension (kproma,k1d_y) :: p1d_y_clear  
real    (kind=jprb), intent   (out), optional, dimension (kproma)       :: pcfrac  
   
!* OTHER                      
integer (kind=jpim) :: i_lev, i_print, i_flevg, i_proma
integer (kind=jpim) :: ichannels
real    (kind=jprb), dimension (kproma,nflevg+1) :: zphys_ph5
real    (kind=jprb), dimension (kproma,nflevg)   :: zphys_rfl5, zphys_sfl5, zphys_tet5, zphys_teq5  
real    (kind=jprb), dimension (kproma,nflevg)   :: zphys_cwc5, zphys_cic5, zphys_cc5, zphys_t5
real    (kind=jprb), dimension (kproma,nflevg)   :: zphys_q5, zphys_p5  
real    (kind=jprb), dimension (kproma)          :: zphys_ps5 , zphys_sh5 , zphys_lh5, zphys_us5, zphys_vs5   
real    (kind=jprb), dimension (kproma)          :: zphys_crf5, zphys_srf5, z_sfc 
logical :: lluse_1d_physics
real    (kind=jprb), dimension (kproma,k1d_y) :: z1d_y_clear  
real    (kind=jprb), dimension (kproma)       :: zcfrac
           
! adjustment variables
real (kind=jprb) :: zhook_handle
    
!* INTERFACES
#include "phys_nl.intfb.h"
!#include "onedvar_obsop_rttov.intfb.h"

if (lhook) call dr_hook ('ONEDVAR_OBSOP',0,zhook_handle)

! Determine if we should use the linearised 1D moist physics 
if (present(ld1d_physics)) then 
  lluse_1d_physics = ld1d_physics
else
  ! Default: use 1D moist physics
  lluse_1d_physics = .true.
endif 

i_print   = 0  
i_flevg   = nflevg
  
!* Set up pressure at ML
do i_proma = 1, kproma
   z_sfc (i_proma) = p1d_x_xt5 (i_proma,7*nflevg+9)
   zphys_ph5 (i_proma,1:nflevg+1) = vbh (0:nflevg) * z_sfc (i_proma) + vah (0:nflevg)
enddo
    
do i_lev =  nflevg, 1, -1
  zphys_p5 (1:kproma,i_lev) = 0.5_JPRB * (zphys_ph5 (1:kproma,i_lev) + zphys_ph5 (1:kproma,i_lev+1))
enddo

if (lluse_1d_physics) then

  !* Control variable 
  zphys_t5  (1:kproma,1:nflevg)  = p1d_x5 (1:kproma,       1:  nflevg)
  zphys_q5  (1:kproma,1:nflevg)  = p1d_x5 (1:kproma,nflevg+1:2*nflevg)

  !* Others
  zphys_tet5 (1:kproma,1:nflevg) = p1d_x_xt5 (1:kproma,       1:  nflevg)
  zphys_teq5 (1:kproma,1:nflevg) = p1d_x_xt5 (1:kproma,nflevg+1:2*nflevg)

  zphys_ps5  (1:kproma) = p1d_x_xt5 (1:kproma,7*nflevg+9)
  zphys_sh5  (1:kproma) = p1d_x_xt5 (1:kproma,7*nflevg+6)
  zphys_lh5  (1:kproma) = p1d_x_xt5 (1:kproma,7*nflevg+7)
  zphys_us5  (1:kproma) = p1d_x_xt5 (1:kproma,7*nflevg+4)
  zphys_vs5  (1:kproma) = p1d_x_xt5 (1:kproma,7*nflevg+5)

  !* Initialize for safety
  zphys_rfl5 (:,:) = 0.0_JPRB
  zphys_sfl5 (:,:) = 0.0_JPRB
  zphys_cwc5 (:,:) = 0.0_JPRB
  zphys_cic5 (:,:) = 0.0_JPRB
  zphys_cc5  (:,:) = 0.0_JPRB
  zphys_crf5 (:)   = 0.0_JPRB
  zphys_srf5 (:)   = 0.0_JPRB

  !* Moist physics
  call phys_nl (i_flevg, kproma, &
   &   zphys_t5                  , &            ! in:  T
   &   zphys_q5                  , &            ! in:  q, rh'
   &   zphys_p5                  , &            ! in:  p
   &   zphys_ph5                 , &            ! in:  ph
   &   zphys_tet5                , &            ! in:  ten_t
   &   zphys_teq5                , &            ! in:  ten_q
   &   zphys_sh5                 , &            ! in:  sh_sfc
   &   zphys_lh5                 , &            ! in:  lh_sfc
   &   zphys_us5                 , &            ! in:  u_star
   &   zphys_vs5                 , &            ! in:  v_star
   &   zphys_rfl5                , &            ! out: rain flux
   &   zphys_sfl5                , &            ! out: snow flux
   &   zphys_cwc5                , &            ! out: cloud water
   &   zphys_cic5                , &            ! out: cloud ice
   &   zphys_cc5                 , &            ! out: cloud cover
   &   zphys_srf5                , &            ! out: strt. rain flux at sfc
   &   zphys_crf5)                              ! out: conv. rain flux at sfc      
            
  p1d_x_xt5 (1:kproma,2*nflevg+1:3*nflevg) = zphys_rfl5 (1:kproma,1:nflevg)
  p1d_x_xt5 (1:kproma,3*nflevg+1:4*nflevg) = zphys_sfl5 (1:kproma,1:nflevg)
  p1d_x_xt5 (1:kproma,4*nflevg+1:5*nflevg) = zphys_cwc5 (1:kproma,1:nflevg)
  p1d_x_xt5 (1:kproma,5*nflevg+1:6*nflevg) = zphys_cic5 (1:kproma,1:nflevg)
  p1d_x_xt5 (1:kproma,6*nflevg+1:7*nflevg) = zphys_cc5  (1:kproma,1:nflevg)

  p1d_x_xt5 (1:kproma,7*nflevg+11) = zphys_srf5 (1:kproma)                  
  p1d_x_xt5 (1:kproma,7*nflevg+12) = zphys_crf5 (1:kproma)

endif
                                            
ichannels = k1d_y * kproma

call onedvar_obsop_rttov (ksat, &
&  k1d_x, k1d_y, k1d_z, &       
&  ichannels,       &
&  i_flevg, kproma, &   ! in: levels, profiles
&  p1d_x5,          &   ! in: control variable
&  p1d_x_xt5,       &   ! in: extra fields
&  zphys_p5,        &   ! in: pressure
&  zphys_ph5,       &   ! in: pressure at i/f
&  p1d_y,           &   ! out: radiances 
&  z1d_y_clear,     &   ! out: clear sky radiances (diagnostics only)
&  zcfrac)              ! out: cloud fraction used in 2-IC (diagnostics only)

if ( present(p1d_y_clear) ) p1d_y_clear = z1d_y_clear
if ( present(pcfrac) )      pcfrac      = zcfrac

if (lhook) call dr_hook ('ONEDVAR_OBSOP',1,zhook_handle) 

end subroutine onedvar_obsop
