SUBROUTINE ONEDVAR_RAINTB_SET(PRAIN_2DCHAN, PSLAT_OBS, PSEA_ICE, PLAND_SEA, &
 & PT_SFC, KST, KEND, KSTJ, KENDJ, KHLPRDY, KSATID)

!****-------------------------------------------------------------------
!**** *ONEDVAR_RAINTB_SET* - ONEDVAR LOAD BALANCING
!****-------------------------------------------------------------------
!     Purpose.   Set Observation distribution to Helpers
!     --------

!**   Interface.
!     ----------

!        Explicit arguments :  PRAIN_2DCHAN, KST, KEND, KSTJ, KENDJ, KHLPRDY
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------
!       Called from EC_PHYS

!     Author.
!     -------
!        John Hague   *ECMWF*

!     Modifications.
!     --------------
!        J. Hague      22-Nov-2005  Load balancing for 1D-Var
!        A. Geer        1-Apr-2006  Ensure correct failure code generated for each obs
!        C. O'Dell     26-Oct-2006  Modified for consistency with new ONEDVAR_FSTSCRN
!        C. O'Dell     17-Jan-2006  Change fast-screening
!        A. Geer       27-Feb-2007  Multiple sensors
!     --------------------------------------------------------------------------------

USE PARKIND1, ONLY : JPIM, JPRB
USE YOMHOOK, ONLY : LHOOK, DR_HOOK
USE YOMDIM, ONLY : NPROMA
USE PARONEDVAR, ONLY : NMAX_CHANNELS
USE YOMRAIN_LB, ONLY : JPHLP
USE YOMONEDVAR, ONLY : ONEDVAR_ID, STREAM1D_NUM, STREAM1D_ACTIVE, &
  &  STREAM1D_MODE

IMPLICIT NONE

REAL(KIND=JPRB), INTENT(IN)  :: PRAIN_2DCHAN(NPROMA,STREAM1D_NUM,NMAX_CHANNELS)
REAL(KIND=JPRB), INTENT(IN), DIMENSION(NPROMA) :: PSLAT_OBS, PSEA_ICE, PLAND_SEA, PT_SFC
INTEGER(KIND=JPIM), INTENT(IN) :: KST, KEND
INTEGER(KIND=JPIM), INTENT(INOUT) :: KSTJ(JPHLP+1), KENDJ(JPHLP+1)
INTEGER(KIND=JPIM), INTENT(IN) :: KHLPRDY(JPHLP+1)
INTEGER(KIND=JPIM), INTENT(IN) :: KSATID(NPROMA,STREAM1D_NUM) ! SATELLITE BUFR ID ARRAY

INTEGER(KIND=JPIM) :: JJX(NPROMA), INOBSX(NPROMA)
INTEGER(KIND=JPIM) :: ICNT, JFOUNDX, JJ, ISTREAM1D, JPROC
INTEGER(KIND=JPIM) :: ISAT, IFAIL, INCHAN, INAPRROX, INALLOCATED, INWORKERS
REAL(KIND=JPRB)    :: ZHOOK_HANDLE

#include "onedvar_fstscrn.intfb.h"
#include "onedvar_find_satsens.intfb.h"

IF (LHOOK) CALL DR_HOOK('ONEDVAR_RAINTB_SET',0,ZHOOK_HANDLE)

INWORKERS = KHLPRDY(1)+1

! Count up observations that are likely to require full 
! (i.e. time consuming) 1D-var processing
ICNT=0
JJX(:)=0
INOBSX(:)=0
JFOUNDX=1

! Grid point loop
DO JJ=KST,KEND

  ! Multiple streams at each grid point
  DO ISTREAM1D=1,STREAM1D_NUM
   
    IF ( STREAM1D_MODE(ISTREAM1D) == STREAM1D_ACTIVE ) THEN
   
      ! Find index to the onedvar satellite lookup table    
      CALL ONEDVAR_FIND_SATSENS( KSATID(JJ,ISTREAM1D), ISAT)    
      IF (ISAT == 0) CYCLE ! Sensor not recognised. This will be flagged later.

      IFAIL = 0
      INCHAN = ONEDVAR_ID(ISAT) % NCHANNELS
      CALL ONEDVAR_FSTSCRN(ONEDVAR_ID(ISAT) % SENSID, &
         & INCHAN, &
         & PRAIN_2DCHAN(JJ,ISTREAM1D,1:INCHAN), &
         & PSLAT_OBS(JJ), PSEA_ICE(JJ), &
         & PLAND_SEA(JJ), PT_SFC(JJ), IFAIL)

      IF(IFAIL==0) THEN
      
        ! OK observation found
        JJX(JFOUNDX)=JJ ! grid point location of found obs        
        INOBSX(JFOUNDX)=INOBSX(JFOUNDX) + 1 ! Number of obs at this location  
           
      ENDIF

    ENDIF 
  ENDDO
  
  IF (INOBSX(JFOUNDX) > 0) THEN
    JFOUNDX=JFOUNDX+1    
    ICNT = ICNT+1      
  ENDIF
  
ENDDO

! Find the approximate number of these observations to be given
! to each available process
INAPRROX = 1 + ICNT/INWORKERS

! Divide into contiguous chunks of grid points, bearing in mind
! that one grid point may have more than one observation in the
! multiple stream case. 
JFOUNDX=0
KSTJ(:)=0
KENDJ(:)=0
KSTJ(1)=KST
DO JPROC=1,INWORKERS
 
  ! Find actual number of obs for each processor
  INALLOCATED = 0
  DO WHILE ((INALLOCATED < INAPRROX) .AND. (JFOUNDX < ICNT))
    JFOUNDX=JFOUNDX+1
    INALLOCATED = INALLOCATED + INOBSX(JFOUNDX)
  ENDDO

  ! Allocate the corresponding grid point start and end values,
  ! ultimately covering all grid points.
  IF (JFOUNDX < ICNT) THEN
    KENDJ(JPROC)=JJX(JFOUNDX)
    KSTJ(JPROC+1)=KENDJ(JPROC) + 1
  ELSE
    KENDJ(JPROC)=KEND
    EXIT
  ENDIF   

ENDDO 

IF (LHOOK) CALL DR_HOOK('ONEDVAR_RAINTB_SET',1,ZHOOK_HANDLE)

END SUBROUTINE ONEDVAR_RAINTB_SET
