SUBROUTINE AER_OPT_PROP &
 & ( KDLEN  , KLEN  , KMXCOUNT, & 
 &   PQF, PTF, PPRESH, PPRESF, &
 &   YDGEMSPROF, &  
 &   PAEREXT,PAEROM, PAERASY)

!*** *AER_OPT_PROP * Routine for aerosol optical properties calculations.

!**   INTERFACE.
!     ----------
!          *AER_OPT_PROP * IS CALLED FROM *AER_REFL_OP*.
!     CALL AER_OPT_PROP(KDLEN,ILEN,IMXCOUNT,ZQF, ZTF, ZPRESH, ZPRESF, ZGEMSPROF, PAEREXT, PAEROM, PAERASY)
!     WHERE KDLEN      = First dimension: Max number of observations, this set (INPUT)
!           KLEN       = Number of aerosol optical depth obs (INPUT)
!           IMXCOUNT   = Number of channels (INPUT)
!           ZQF       =  Specific humidity at observation points, model levels (INPUT)
!           ZTF       =  Temperature at observation points, model levels (INPUT)
!           ZPRESH,ZPRESF =   Half/Full level pressure values at obs points, model levels (INPUT)
!           ZGEMSPROF     = GEMS variables (INPUT)
!           PAEREXT       = Aerosol opt depth profile at different locations and wavelengths (OUTPUT)
!           PAEROM        = Aerosol ssa profile at different locations and wavelengths (OUTPUT)
!           PAERASY       = Aerosol asymmetri parameter profile at different locations and wavelengths (OUTPUT)


!**   AUTHOR.
!     -------
!        ANGELA BENEDETTI        *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2008-08-11 (adapted from aod_op.F90)

!     PURPOSE.
!     --------
!     - derive aerosol optical thickness, single scattering albedo and asymmetry parameter of aerosols
!         from profiles of mass mixing ratio 

! Optical properties in su_aerop.F90 avaliable at following wavelenghts
! NOTE that number of wavelengths has gone up from 16 to 17!
!         WVAL=   1 (340),  2 (380),  3 (400),  4 (440),  5 (469),  6 (500), 7 (532), 8 (555), 
!                 9 (645), 10 (670), 11 (800), 12 (858), 13 (865), 14 (1020), 15 (1240), 16 (1640), 17 (2130)
!     'MODIS Channels' (channels from other instruments can be added) 
!      -         470 nm, 550 nm, 660 nm, 870 nm, 1240 nm, 1630 nm, 2130 nm
! Corresponding IWAVL
!      IWAVL = [ 5, 8, 10, 13, 15, 16, 17] 
! **** DEPENDS ON J.-J.'s choice of numbering in su_aerop.F90 - please double check!!!!!!
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOM_YGFL , ONLY : NAERO, NACTAERO, JPAERO
USE YOMDIM, ONLY : NFLEVG
USE YOMDB

USE YOMCST    ,ONLY : RG
USE YOMLUN    ,ONLY : NULERR
USE YOEAEROP  ,ONLY : ALF_SU, ALF_OM, ALF_DD, ALF_SS, ALF_BC &
                  & , ASY_SU, ASY_OM, ASY_DD, ASY_SS, ASY_BC &
                  & , OMG_SU, OMG_OM, OMG_DD, OMG_SS, OMG_BC
USE YOEAERSNK ,ONLY : RRHTAB
USE YOEAERSRC ,ONLY : NMAXTAER, NTYPAER, NBIN, NTYP 

USE TYPE_GEMS_PROFILES

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN, KLEN, KMXCOUNT

REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH(KDLEN,0:NFLEVG) ! PRESSURE ON HALF LEVELS FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF(KDLEN,NFLEVG)   ! PRESSURE ON FULL LEVELS FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KDLEN,0:NFLEVG)    ! temperature FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF(KDLEN,0:NFLEVG)    ! specific humidity FROM TRAJ

TYPE (TYPE_GEMSPROF)   ,INTENT(IN) :: YDGEMSPROF(KDLEN,NGEMS)

REAL(KIND=JPRB)   ,INTENT(OUT) :: PAEREXT(KDLEN,NFLEVG,KMXCOUNT) ! AEROSOL EXTINCTION 
REAL(KIND=JPRB)   ,INTENT(OUT) :: PAEROM(KDLEN,NFLEVG,KMXCOUNT)  ! AEROSOL SINGLE SCATTERING ALBEDO 
REAL(KIND=JPRB)   ,INTENT(OUT) :: PAERASY(KDLEN,NFLEVG,KMXCOUNT) ! AEROSOL ASYMMETRY PARAMETER


!*       0.5   LOCAL VARIABLES
!              ---------------
INTEGER(KIND=JPIM), PARAMETER :: IWAVLMX=7 ! Max number of MODIS channels
INTEGER(KIND=JPIM), DIMENSION(IWAVLMX) :: IWAVL=(/5, 8, 10, 13, 14, 15, 16/) ! Index for MODIS channels
INTEGER(KIND=JPIM) :: IACTAEROP ! Number of optically active aerosol species (NACTAERO-1, for now)
INTEGER(KIND=JPIM) :: IAER, ITYP, IBIN, IFLAG, ISUM, IWAVLINDEX
INTEGER(KIND=JPIM) :: JAER, JK, JL, JJ, JF, JP, JLEN, JTAB
INTEGER(KIND=JPIM), PARAMETER  :: JTABMX=12
INTEGER(KIND=JPIM), ALLOCATABLE :: ITYPE(:) !index to determine aerosol species
INTEGER(KIND=JPIM), DIMENSION(KLEN,NFLEVG) :: IRH

REAL(KIND=JPRB), DIMENSION(KLEN,NFLEVG)   :: ZQSAT, ZT, ZPF
REAL(KIND=JPRB), ALLOCATABLE   :: ZAERTAU(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZAERF(:,:,:) ! aer mixing ratios from traj (interpolated to obs locations)
REAL(KIND=JPRB), DIMENSION(KLEN,NFLEVG) :: ZAERTMR ! total aer mixing ratios from traj 
REAL(KIND=JPRB), ALLOCATABLE :: ZALF(:), ZOMG(:), ZASY(:) ! optical properties of aerosol species
REAL(KIND=JPRB)    :: ZRHCL(KLEN,NFLEVG), ZDP
REAL(KIND=JPRB), ALLOCATABLE :: ZEXTTMP(:,:,:,:), ZOMTMP(:,:,:,:),  &
     &             ZASYTMP(:,:,:,:)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "satur.intfb.h"
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_OPT_PROP',0,ZHOOK_HANDLE)

!-----------------------------------------------------------------------
! Check that total number of aerosol bins is not larger than NACTAERO
ISUM=0
DO IAER=1,NMAXTAER
  ISUM=ISUM+NTYPAER(IAER)
ENDDO
IF (ISUM > NACTAERO) THEN
  CALL ABOR1('AOD_OP:TOTAL NUMBER OF REQUESTED BINS PER TYPE EXCEEDS NACTAERO')
ENDIF

! Remember to put in a check for maximum IBIN allowed per aerosol type

IACTAEROP=NACTAERO-1 ! Number of optically active aerosol species

ALLOCATE(ITYPE(IACTAEROP))
ALLOCATE(ZAERF(KLEN,NFLEVG,IACTAEROP))
ALLOCATE(ZALF(IACTAEROP))
ALLOCATE(ZASY(IACTAEROP))
ALLOCATE(ZOMG(IACTAEROP))
ALLOCATE(ZAERTAU(KLEN,IACTAEROP))

ALLOCATE(ZEXTTMP(KLEN,NFLEVG,IACTAEROP,17))
ALLOCATE(ZOMTMP(KLEN,NFLEVG,IACTAEROP,17))
ALLOCATE(ZASYTMP(KLEN,NFLEVG,IACTAEROP,17))

! Extract aerosol mixing ratios from GEMS profiles structure
ZAERF(:,:,:)=0.0
ITYPE(:)=0
IF(NAERO > 0) THEN
  DO JLEN=1,KLEN
    IAER=0
    DO JF=1,NGEMS
      JP=0
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210001 .OR.  &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210002 .OR.  &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210003) JP=1 !SS 3 bins
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210004 .OR.  &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210005 .OR.  &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210006 ) JP=2 !DD 3 bins
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210007 .OR. &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210008) JP=3 ! ORGANIC MATTER - 2 bins
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210009 .OR. &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210010) JP=4 ! Black Carbon - 2 bins
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210011) JP=5 ! Sulphates
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210013) JP=7 ! Stratospheric background
      
      IF(JP > 0) THEN
        IAER=IAER+1
        ZAERF(JLEN,:,IAER)=YDGEMSPROF(JLEN,JF)%MODELPROF(:)
        ITYPE(IAER)=JP
        IF(IAER > JPAERO) THEN
          CALL ABOR1('AOD_OP:NOT ENOUGH AEROSOL SPECIES ALLOCATED')
        ENDIF
      ENDIF
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210048) THEN ! Total aerosol mixing ratio
        ZAERTMR(JLEN,:)=YDGEMSPROF(JLEN,JF)%MODELPROF(:)
      ENDIF
    ENDDO
  ENDDO
ENDIF

! Compute relative humidity at observation points, on model levels

ZT(1:KLEN,:) = PTF(1:KLEN,1:NFLEVG)
ZPF(1:KLEN,:)= PPRESF(1:KLEN,:)
IFLAG=2
CALL SATUR (1 , KLEN , KLEN  , 1 , NFLEVG,&
  & ZPF, ZT , ZQSAT , IFLAG)

!-- define RH index from "clear-sky" relative humidity
IRH(:,:)=1
DO JK=1,NFLEVG
  DO JL=1,KLEN
    ZRHCL(JL,JK)=(PQF(JL,JK)/ZQSAT(JL,JK))*100._JPRB
    DO JTAB=1,JTABMX
      IF (ZRHCL(JL,JK) > RRHTAB(JTAB)) THEN
        IRH(JL,JK)=JTAB
      ENDIF
    ENDDO
  ENDDO
ENDDO

! BEN - initialize optical depth, ssa, and asymmetry parameter to non zero values

 PAEREXT(:,:,:) = 0.0000001_JPRB
 PAEROM(:,:,:)  = 0.0000001_JPRB
 PAERASY(:,:,:) = 0.0000001_JPRB

 ZEXTTMP(:,:,:,:) = 0.0_JPRB
 ZOMTMP(:,:,:,:) = 0.0_JPRB
 ZASYTMP(:,:,:,:) = 0.0_JPRB

  
IF(KMXCOUNT ==1) IWAVLINDEX=8   ! 550nm

DO JJ=1,KMXCOUNT

!   IWAVLINDEX is the wavelength index: for now the choice of channels is tailored to MODIS optical
!         depth retrievals (Remer et al. 2005 and Benedetti, 2005, ECMWF Tech. Memo
!                                                 "Requirements for AEROSOL-related observations").
! *** to be activated, for now assume AOD is only defined at 550nm , i.e. IWAVLINDEX=8 always
!BEN - let's try and see if it works for all channels
     IWAVLINDEX=IWAVL(JJ)

  DO JL=1,KLEN
   DO JAER=1,IACTAEROP

!-- ITYP is the aerosol type 1:SeaSalt,   2:DesertDust,   3:OrganicMatter, 4: BlackCarbon,
!                            5:SUlphates, 6:FlyingAshes
!   IBIN is the bin index: 1-3:SS, 1-3:DD, 1:OM, 1:BC, 1:SU, 1:FA
! This indeces are defined in su_aerw.F90 

         ITYP=NTYP(JAER)
         IBIN=NBIN(JAER)
         IF(ITYP /= 0 .AND. IBIN == 0) THEN
          WRITE(NULERR,*)'AOD_OP:AT LEAST ONE BIN NEEDS TO BE ALLOCATED FOR TYPE ', ITYP
          WRITE(NULERR,*) 'PROGRAM WILL STOP'
          CALL ABOR1('') 
         ENDIF
         IF(ITYP /= ITYPE(JAER)) THEN
          WRITE(NULERR,*)'AOD_OP: AEROSOL TYPE IS NOT MATCHING', ITYP
          WRITE(NULERR,*) 'PROGRAM WILL STOP'
          CALL ABOR1('')
         ENDIF

      ZAERTAU(JL,JAER)=0.0_JPRB
        
      DO JK=1,NFLEVG

! Always double check dimensioning of arrays ALF_** in yoeaerop.F90 !!!!!!!!!
! It has changed a number of times!!!!!!!!

         IF (ITYP == 1) THEN
          ZALF(JAER)=ALF_SS(IRH(JL,JK),IWAVLINDEX,IBIN)
          ZOMG(JAER)=OMG_SS(IRH(JL,JK),IWAVLINDEX,IBIN)
          ZASY(JAER)=ASY_SS(IRH(JL,JK),IWAVLINDEX,IBIN)
         ELSEIF (ITYP == 2) THEN
          ZALF(JAER)=ALF_DD(IBIN,IWAVLINDEX)
          ZOMG(JAER)=OMG_DD(IBIN,IWAVLINDEX)
          ZASY(JAER)=ASY_DD(IBIN,IWAVLINDEX)
         ELSEIF (ITYP == 3) THEN
          ZALF(JAER)=ALF_OM(IRH(JL,JK),IWAVLINDEX)
          ZOMG(JAER)=OMG_OM(IRH(JL,JK),IWAVLINDEX)
          ZASY(JAER)=ASY_OM(IRH(JL,JK),IWAVLINDEX)
         ELSEIF (ITYP == 4 .OR. ITYP == 6) THEN
          ZALF(JAER)=ALF_BC(IWAVLINDEX)
          ZOMG(JAER)=OMG_BC(IWAVLINDEX)
          ZASY(JAER)=ASY_BC(IWAVLINDEX)
         ELSEIF (ITYP == 5 .OR. ITYP == 7 .OR. ITYP == 8 .OR. ITYP == 9) THEN
          ZALF(JAER)=ALF_SU(IRH(JL,JK),IWAVLINDEX)
          ZOMG(JAER)=OMG_SU(IRH(JL,JK),IWAVLINDEX)
          ZASY(JAER)=ASY_SU(IRH(JL,JK),IWAVLINDEX)
         ENDIF
               
! convert units from m^2/g to m^2/kg

            ZALF(JAER)=ZALF(JAER)*1000._JPRB ! conversion from m^2/g to m^2/kg

            ZDP=PPRESH(JL,JK)-PPRESH(JL,JK-1)

            ZAERTAU(JL,JAER)=ZAERTAU(JL,JAER)+ ZAERF(JL,JK,JAER)*ZALF(JAER)*(ZDP/RG)

            ZEXTTMP(JL,JK,JAER,JJ) = ZAERTAU(JL,JAER)
            ZOMTMP(JL,JK,JAER,JJ) =  ZOMG(JAER) * ZAERTAU(JL,JAER) ! equals the scattering cross section
            ZASYTMP(JL,JK,JAER,JJ) = ZASY(JAER) * ZOMG(JAER) * ZAERTAU(JL,JAER)

      ENDDO ! loop over model vertical levels

! BEN - optical properties as sum over all individual species

         DO JK=1,NFLEVG
           PAEREXT(JL,JK,JJ) =   PAEREXT(JL,JK,JJ) + ZEXTTMP(JL,JK,JAER,JJ)
           PAEROM(JL,JK,JJ)  =   PAEROM(JL,JK,JJ)  + ZOMTMP(JL,JK,JAER,JJ)
           PAERASY(JL,JK,JJ) =   PAERASY(JL,JK,JJ) + ZASYTMP(JL,JK,JAER,JJ)
        ENDDO


    ENDDO ! loop over aerosol type

  ENDDO ! loop over horizontal observational points

! BEN - normalizations 

        DO JL=1,KLEN
         DO JK=1,NFLEVG
          IF(PAEREXT(JL,JK,JJ) > 0.0_JPRB) THEN
           PAEROM(JL,JK,JJ) = PAEROM(JL,JK,JJ)/PAEREXT(JL,JK,JJ)
          ENDIF
          IF(PAEREXT(JL,JK,JJ) > 0.0_JPRB .AND. PAEROM(JL,JK,JJ) > 0.0_JPRB) THEN
           PAERASY(JL,JK,JJ)= PAERASY(JL,JK,JJ)/ ( PAEROM(JL,JK,JJ)*PAEREXT(JL,JK,JJ) )
          ENDIF
         ENDDO
        ENDDO

ENDDO ! loop over MODIS channels

DEALLOCATE(ZAERF)
DEALLOCATE(ITYPE)
DEALLOCATE(ZALF)
DEALLOCATE(ZASY)
DEALLOCATE(ZOMG)
DEALLOCATE(ZAERTAU)

DEALLOCATE(ZEXTTMP)
DEALLOCATE(ZOMTMP)
DEALLOCATE(ZASYTMP)

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_OPT_PROP',1,ZHOOK_HANDLE)
END SUBROUTINE AER_OPT_PROP

