  SUBROUTINE AER_REFL_OP( KDLEN  , KLEN  , KMXCOUNT, &
 &   PQF, PTF, PPRESH, PPRESF, &
 &   YDGEMSPROF, &
 &   PSOLANG, PVIEANG, PDIFFANG, & 
 &   PAERREFL ) 

!**** AER_REFL_OP *** OBSERVATION OPERATOR FOR AEROSOL REFLECTANCE

!     INTERFACE
!     ---------
!           MAIN CALLING ROUTINE THAT COMPUTES AEROSOL OPTICAL PROPERTIES AND CALLS 6S (RT6S_GEMS)

!          *AER_REFL_OP* IS CALLED FROM *HOP*.

!     CALL AER_REFL_OP(KDLEN,ILEN,IMXCOUNT,ZQF, ZTF, ZPRESH, ZPRESF, ZGEMSPROF, ZDIFFANG, & 
!            &         ZSOLANG, ZVIEANG, ZXPP)
!     WHERE KDLEN      = First dimension: Max number of observations, this set (INPUT)
!           KLEN       = Number of aerosol reflectance obs (INPUT)
!           IMXCOUNT   = Number of channels (INPUT)
!           ZQF       =  Specific humidity at observation points, model levels (INPUT)
!           ZTF       =  Temperature at observation points, model levels (INPUT)
!           ZPRESH,ZPRESF =   Half/Full level pressure values at obs points, model levels (INPUT)
!           ZGEMSPROF = GEMS variables (INPUT)
!           ZSOLANG  = SOLAR ZENITH ANGLE (INPUT) - degrees
!           ZVIEANG  = VIEWING ZENTIH ANGLE (INOUT) - degrees
!           ZDIFFANG = difference in azimuthal angle (radiants) (viewing minus solar azimuthal angle).
!           ZXPP     = Aerosol reflectance at different wavelengths (OUTPUT)

!     EXTERNALS
!     ---------
!           AER_OPT_PROP - routine that computes and returns the aerosol optical properties (extinction,
!                          single scattering albedo and asymmetry parameter)
!           RT6S_GEMS      - performs radiative transfer calculations. For now, this is tuned to 
!                          the MODIS instrument but it can be made more general.   

!     AUTHOR 
!     -------------
!          A.BENEDETTI - ECMWF 11.08.2008

!     ----------------
!--------------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM, ONLY : NFLEVG
USE YOMLUN   , ONLY : NULOUT

USE RT6SVALUES

USE TYPE_GEMS_PROFILES

IMPLICIT NONE 

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN, KLEN  !NUMBER OF OBS POINTS
INTEGER(KIND=JPIM),INTENT(IN)    :: KMXCOUNT     !NUMBER OF CHANNELS (WAVELENGTHS) 

REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH(KDLEN,0:NFLEVG) ! PRESSURE ON HALF LEVELS FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF(KDLEN,NFLEVG)   ! PRESSURE ON FULL LEVELS FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KDLEN,0:NFLEVG)    ! temperature FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF(KDLEN,0:NFLEVG)    ! specific humidity FROM TRAJ

TYPE (TYPE_GEMSPROF)   ,INTENT(IN) :: YDGEMSPROF(KDLEN,NGEMS)

REAL(KIND=JPRB), INTENT(IN) :: PDIFFANG(KDLEN), PSOLANG(KDLEN), PVIEANG(KDLEN)

REAL(KIND=JPRB)   ,INTENT(OUT) :: PAERREFL(KDLEN,KMXCOUNT) ! AEROSOL REFLECTANCE 
                                                           !(FUNCTION OF HORIZONTAL POSITION AND CHANNEL #)

! LOCAL VARIABLES 
! Main driver

INTEGER(KIND=JPIM) :: JJ ! Max number of MODIS channels
INTEGER(KIND=JPIM), PARAMETER :: IWAVLMX=7 ! Max number of MODIS channels
REAL(KIND=JPRB), DIMENSION(IWAVLMX) :: ZWAVL=(/0.469e-06, .550e-06, 0.167e-06, 0.865e-06,  &
     &                                         1.240e-06, 1.640e-06, 2.130e-06 /) ! MODIS wavelengths

! For AER_OPT_PROP 

REAL(KIND=JPRB) :: ZAEREXT(KDLEN,NFLEVG,KMXCOUNT), ZAEROM(KDLEN,NFLEVG,KMXCOUNT), & 
     &             ZAERASY(KDLEN,NFLEVG,KMXCOUNT)

! For RT6S_GEMS 

INTEGER(KIND=JPIM) :: JLEV, JMU, JLEN
! THESE VALUES ARE NOW IN A MODULE (rt6svalues)
!!!!!!!!!!!!!!!!!!!!!! REMEMBER KLON and KWL must always be equal to 1 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!INTEGER(KIND=JPIM),PARAMETER :: KLON=1, KWL=1, KLEV=60, KNT=26
!INTEGER(KIND=JPIM),PARAMETER :: KMU=25, KMU2=48, KNP=49, KNANG=80, KNBMU=83

REAL(KIND=JPRB) :: ZPROC(KLON,KWL)           ! Auxiliary surface reflectance
REAL(KIND=JPRB) :: ZWL(KWL)                  ! Channel wavelength
REAL(KIND=JPRB) :: ZZDIFFANG(KLON), ZZSOLANG(KLON), ZZVIEANG(KLON) ! Satellite and solar viewing angles
REAL(KIND=JPRB) :: ZTRAY550_VERTIN(KLON,0:KLEV),ZTAER_VERTIN(KLON,0:KLEV,KWL)
REAL(KIND=JPRB) :: ZPHASEL_IN(KLON,0:KLEV,KWL,KNBMU),ZPIZ_IN(KLON,0:KLEV,KWL)
REAL(KIND=JPRB) :: ZREFLET(KLON,KWL)


REAL(KIND=JPRB) :: ZHEYGR, ZSTEP, ZANGLE

REAL(KIND=JPRB) :: ZHOOK_HANDLE

LOGICAL         :: LLAERREFL(KDLEN,KMXCOUNT)

#include "aer_opt_prop.intfb.h"
#include "rt6s_gems.intfb.h"

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_REFL_OP',0,ZHOOK_HANDLE)


! Call aerosol optical depth routine
        CALL AER_OPT_PROP( KDLEN  , KLEN  , KMXCOUNT, &
 &   PQF, PTF, PPRESH, PPRESF, &
 &   YDGEMSPROF, &
 &   ZAEREXT,ZAEROM, ZAERASY )

 LLAERREFL(:,:) = .TRUE.

 DO JJ=1,KMXCOUNT
  DO JLEN=1,KLEN
    IF(MAXVAL(ZAEREXT(JLEN, :, JJ)) == 0.0_JPRB) THEN
      LLAERREFL(JLEN, JJ) = .FALSE.
    ENDIF
  ENDDO
 ENDDO

!! Normalizations - should they go into CALL AER_OPT_PROP? Yes, see in there.

! DO JJ=1,KMXCOUNT
!  DO JLEN=1,KLEN
!   DO JLEV=1,NFLEVG
!    IF(ZAEREXT(JLEN, JLEV, JJ) > 0.0) THEN
!      ZAEROM(JLEN,JLEV, JJ) = ZAEROM(JLEN,JLEV, JJ)/ZAEREXT(JLEN, JLEV, JJ)
!    ENDIF
!    IF(ZAEREXT(JLEN, JLEV, JJ) > 0.0 .AND. ZAEROM(JLEN,JLEV, JJ) > 0.) THEN
!       ZAERASY(JLEN,JLEV, JJ) = ZAERASY(JLEN,JLEV, JJ)/ (ZAEROM(JLEN,JLEV, JJ) * ZAEREXT(JLEN, JLEV, JJ) )
!    ENDIF
!   ENDDO
!  ENDDO
! ENDDO

 DO JJ=1,KMXCOUNT

 ZWL(1)=ZWAVL(JJ) ! Select JJ=2 for 550 nm. 

 DO JLEN =1,KDLEN ! loop on number of observations 

    IF(LLAERREFL(JLEN,JJ)) THEN

    ZPROC(1,1)=0.5   ! Surface reflectance - for now constant, but should be a function of wavelenght and
                    ! horizontal position such this:  ZPROC(1,1)= ZSURFREFL(JJ,JLEN) - this could be read
                    ! from a file or calculated from other model inputs: ask JEAN-JACQUES!!!

! Solar and satellite viewing angles 

    ZZDIFFANG(1) = PDIFFANG(JLEN)  ! For convenience this could be fixed to PI/2. 
    ZZSOLANG(1)  = PSOLANG(JLEN)   ! For convenience this could be fixed to =0.2   
    ZZVIEANG(1)  = PVIEANG(JLEN)   ! For convenience this could be fixed to =0.3 as in the test code

! Profile of aerosol extinction from aer_opt_prop 
 ZTAER_VERTIN(1,1:KLEV,1) = ZAEREXT(JLEN, 1:KLEV, JJ) ! constant added for testing purposes
 ZTAER_VERTIN(1,0,1)      = ZAEREXT(JLEN, 1, JJ) ! level 0 value is set equal to value for level 1

! Rayleigh optical depth from Dutton et al. (1994) 
 DO JLEV= 0, NFLEVG
  ZTRAY550_VERTIN(1,JLEV) = (PPRESH(JLEN,JLEV)/101325._JPRB) * 0.00877_JPRB * (ZWL(1)*1.E06)** (-4.05_JPRB)
!  WRITE(NULOUT,*) 'RAYLEIGH OPT DEPTH', JLEV, ZTRAY550_VERTIN(1,JLEV) 
 ENDDO

 ZPIZ_IN(1,1:KLEV,1) = ZAEROM(JLEN, 1:KLEV, JJ)    ! For convenience this could be set to 0.98
 ZPIZ_IN(1,0,1)      = ZAEROM(JLEN, 1, JJ) ! level 0 value is set equal to value for level 1
 

! Phase function calculations using Heyney-Greenstein

 ZHEYGR = 0.0_JPRB
 ZSTEP = 0.037

 DO JLEV = 1,NFLEVG

! BEN - Heyney-Greenstein phase function based on the asymmetry parameter g (ZAERASY)
  ZANGLE = 3.034
  DO JMU = 2, KNBMU
   ZANGLE = ZANGLE - ZSTEP
   ZHEYGR = (1.-ZAERASY(JLEN,JLEV,JJ)**2) / (1. + ZAERASY(JLEN,JLEV,JJ)**2 - 2._JPRB * ZAERASY(JLEN,JLEV,JJ) * &
     &      COS(ZANGLE) )**(3./2.)

   ZPHASEL_IN(1,JLEV,1,JMU) = ZHEYGR
  ENDDO
   ZPHASEL_IN(1,JLEV,1,1)  = (1.-ZAERASY(JLEN,JLEV,JJ)**2)  / (1. + ZAERASY(JLEN,JLEV,JJ)**2 - 2._JPRB *  &
     &                         ZAERASY(JLEN,JLEV,JJ) * COS(ZPI) )**(3./2.)

 ENDDO

 ZPHASEL_IN (:,0,:,:) = ZPHASEL_IN (:,1,:,:) ! level 0 value is set equal to value for level 1
 
  WRITE(NULOUT,*)'CALLING RT6S_GEMS '
   CALL RT6S_GEMS(ZZDIFFANG,ZZSOLANG,ZZVIEANG,ZWL,ZPHASEL_IN,ZPIZ_IN,ZTRAY550_VERTIN,ZTAER_VERTIN,ZPROC, &
&                                                                                   ZREFLET)
  WRITE(NULOUT,*) 'END OF RT6S_GEMS'

! BEN - Save reflectance
  PAERREFL(JLEN,JJ) = ZREFLET(1,1) 

  WRITE(NULOUT,*) 'IN AER_REFL_OP AER REFL = ', JJ, JLEN, PAERREFL(JLEN,JJ) 

  ELSE

   PAERREFL(JLEN,JJ) = 0.0_JPRB
 
  ENDIF

 ENDDO ! LOOP ON HORIZONTAL POINTS (JLEN) 
ENDDO ! LOOP ON CHANNELS  (JJ)

  IF (LHOOK) CALL DR_HOOK('AER_REFL_OP',1,ZHOOK_HANDLE)
END SUBROUTINE AER_REFL_OP
