SUBROUTINE AOD_OP &
 & ( KDLEN  , KLEN  , KMXCOUNT, & 
 &   PQF, PTF, PPRESH, PPRESF, &
 &   YDGEMSPROF, &  
 &   PAEROD )

!*** *AOD_OP* Operator routine for aerosol optical depth observations.

!**   INTERFACE.
!     ----------
!          *AOD_OP* IS CALLED FROM *HOP*.
!     CALL AOD_OP(KDLEN,ILEN,IMXCOUNT,ZQF, ZTF, ZPRESH, ZPRESF, ZGEMSPROF, ZXPP)
!     WHERE KDLEN      = First dimension: Max number of observations, this set (INPUT)
!           KLEN       = Number of aerosol optical depth obs (INPUT)
!           IMXCOUNT   = Number of channels (INPUT)
!           ZQF       =  Specific humidity at observation points, model levels (INPUT)
!           ZTF       =  Temperature at observation points, model levels (INPUT)
!           ZPRESH,ZPRESF =   Half/Full level pressure values at obs points, model levels (INPUT)
!           ZGEMSPROF     = GEMS variables (INPUT)
!           ZXPP       = Aerosol optical depth at different wavelengths (OUTPUT)



!**   AUTHOR.
!     -------
!        ANGELA BENEDETTI        *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2005-06-17 (adapted from aer_bdgmass.F90 by J.-J. Morcrette)
!        A. Benedetti: 2009-02-25. Readaptation of code to allow for two sulphate variables
!                                   with SO2 not contributing to the total aerosol optical depth

!     PURPOSE.
!     --------
!     - derive aerosol optical thickness from profiles of mass mixing ratio 

! Optical properties in su_aerop.F90 avaliable at following wavelenghts
! (340)355(380)400 440 469 500 532 550 645 670 800 858 865(1020)1064 1240 1640 2130
! corresponding IWAVL 
!   1   2   3   4   5   6   7   8   9  10   11  12  13  14  15  16   17    18   19
!                      ---         ---     ---         ---           ---- ---- ---- MODIS
!              ***                             ***                                  SEVIRI
!  ---     ---     xxx     ---             xxx         xxx xxxx                     AERONET
!      000                     000                               000                EARLINET/CALIPSO
! 

!     'MODIS Channels' (channels from other instruments can be added) 
!      -         470 nm, 550 nm, 660 nm, 870 nm, 1240 nm, 1630 nm, 2130 nm
! Corresponding IWAVL
!      IWAVL = [ 6, 9, 11, 14, 17, 18, 19] 
! **** DEPENDS ON J.-J.'s choice of numbering in su_aerop.F90 !!!!!! CHECK PERIODICALLY !!!
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOM_YGFL , ONLY : NAERO, NACTAERO, JPAERO
USE YOMDIM, ONLY : NFLEVG
USE YOMDB

USE YOMCST    ,ONLY : RG
USE YOMLUN    ,ONLY : NULERR
USE YOEAEROP  ,ONLY : ALF_SU, ALF_OM, ALF_DD, ALF_SS, ALF_BC
USE YOEAERSNK ,ONLY : RRHTAB
USE YOEAERSRC ,ONLY : NMAXTAER, NTYPAER, NBIN, NTYP 

USE TYPE_GEMS_PROFILES

IMPLICIT NONE


!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN, KLEN, KMXCOUNT

REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH(KDLEN,0:NFLEVG) ! PRESSURE ON HALF LEVELS FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF(KDLEN,NFLEVG)   ! PRESSURE ON FULL LEVELS FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KDLEN,0:NFLEVG)    ! temperature FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF(KDLEN,0:NFLEVG)    ! specific humidity FROM TRAJ

TYPE (TYPE_GEMSPROF)   ,INTENT(IN) :: YDGEMSPROF(KDLEN,NGEMS)

REAL(KIND=JPRB)   ,INTENT(OUT) :: PAEROD(KDLEN,KMXCOUNT) ! AEROSOL OPT DEPTH (FUNCTION OF HORIZONTAL POSITION AND CHANNEL #)



!*       0.5   LOCAL VARIABLES
!              ---------------
!INTEGER(KIND=JPIM), PARAMETER :: IWAVLMX=7 ! Max number of MODIS channels
! IWAVL: Index for MODIS channels - based on latest su_aerop.F90
!INTEGER(KIND=JPIM), DIMENSION(IWAVLMX) :: IWAVL=(/6, 9, 11, 14, 17, 18, 19/) 
INTEGER(KIND=JPIM) :: IACTAEROP ! Number of optically active aerosol species (NACTAERO-1, for now) 
INTEGER(KIND=JPIM) :: IAER, ITYP, IBIN, IFLAG, ISUM, IWAVLINDEX
INTEGER(KIND=JPIM) :: JAER, JK, JL, JJ, JF, JP, JLEN, JTAB
INTEGER(KIND=JPIM), PARAMETER  :: JTABMX=12
INTEGER(KIND=JPIM), ALLOCATABLE :: ITYPE(:) !index to determine optically active aerosol species
INTEGER(KIND=JPIM), DIMENSION(KLEN,NFLEVG) :: IRH

REAL(KIND=JPRB), DIMENSION(KLEN,NFLEVG)   :: ZQSAT, ZT, ZPF
REAL(KIND=JPRB), ALLOCATABLE   :: ZAERTAU(:,:)
REAL(KIND=JPRB), ALLOCATABLE   :: ZAERF(:,:,:) ! aer mixing ratios from traj (interpolated to obs locations)
REAL(KIND=JPRB), DIMENSION(KLEN,NFLEVG) :: ZAERTMR ! total aer mixing ratios from traj 
REAL(KIND=JPRB), ALLOCATABLE :: ZALF(:)
REAL(KIND=JPRB)    :: ZRHCL(KLEN,NFLEVG), ZAEREXT, ZDP

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "satur.intfb.h"
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AOD_OP',0,ZHOOK_HANDLE)

!-----------------------------------------------------------------------
! Check that total number of aerosol bins is not larger than NACTAERO
ISUM=0
DO IAER=1,NMAXTAER
  ISUM=ISUM+NTYPAER(IAER)
ENDDO
IF (ISUM > NACTAERO) THEN
  CALL ABOR1('AOD_OP:TOTAL NUMBER OF REQUESTED BINS PER TYPE EXCEEDS NACTAERO')
ENDIF

! Remember to put in a check for maximum IBIN allowed per aerosol type


IACTAEROP=NACTAERO-1 ! Number of optically active aerosol species

ALLOCATE(ZAERF(KLEN,NFLEVG,IACTAEROP))
ALLOCATE(ITYPE(IACTAEROP))
ALLOCATE(ZAERTAU(KLEN,IACTAEROP))
ALLOCATE(ZALF(IACTAEROP))

ZAERF(:,:,:)=0.0
ITYPE(:)=0
! Extract aerosol mixing ratios from GEMS profiles structure
IF(NAERO > 0) THEN
  DO JLEN=1,KLEN
    IAER=0
    DO JF=1,NGEMS
      JP=0
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210001 .OR.  &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210002 .OR.  &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210003) JP=1 !SS 3 bins
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210004 .OR.  &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210005 .OR.  &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210006 ) JP=2 !DD 3 bins
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210007 .OR. &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210008) JP=3 ! ORGANIC MATTER - 2 bins
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210009 .OR. &
       &   YDGEMSPROF(JLEN,JF)%IGRIB == 210010) JP=4 ! Black Carbon - 2 bins
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210011)  JP=5 ! SO4 
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210013) JP=7 ! Stratospheric background
      
      IF(JP > 0) THEN
        IAER=IAER+1
        ZAERF(JLEN,:,IAER)=YDGEMSPROF(JLEN,JF)%MODELPROF(:)
        ITYPE(IAER)=JP
        IF(IAER > JPAERO) THEN
          CALL ABOR1('AOD_OP:NOT ENOUGH AEROSOL SPECIES ALLOCATED')
        ENDIF
      ENDIF
      IF(YDGEMSPROF(JLEN,JF)%IGRIB == 210048) THEN ! Total aerosol mixing ratio
        ZAERTMR(JLEN,:)=YDGEMSPROF(JLEN,JF)%MODELPROF(:)
      ENDIF
    ENDDO
  ENDDO
ENDIF

! initialise optical depth 
PAEROD(:,:) = 0.0_JPRB 

! Compute relative humidity at observation points, on model levels

ZT(1:KLEN,:) = PTF(1:KLEN,1:NFLEVG)
ZPF(1:KLEN,:)= PPRESF(1:KLEN,:)
IFLAG=2
CALL SATUR (1 , KLEN , KLEN  , 1 , NFLEVG,&
  & ZPF, ZT , ZQSAT , IFLAG)

!-- define RH index from "clear-sky" relative humidity
IRH(:,:)=1
DO JK=1,NFLEVG
  DO JL=1,KLEN
    ZRHCL(JL,JK)=(PQF(JL,JK)/ZQSAT(JL,JK))*100._JPRB
    DO JTAB=1,JTABMX
      IF (ZRHCL(JL,JK) > RRHTAB(JTAB)) THEN
        IRH(JL,JK)=JTAB
      ENDIF
    ENDDO
  ENDDO
ENDDO
  
IF(KMXCOUNT ==1) IWAVLINDEX=9   ! 550nm

DO JJ=1,KMXCOUNT

!   IWAVLINDEX is the wavelength index: for now the choice of channels is tailored to MODIS optical
!         depth retrievals (Remer et al. 2005  and Benedetti et al, 2009, ECMWF Tech. Memo 571)
! *** to be activated, for now assume AOD is only defined at 550nm , i.e. IWAVLINDEX=9 always
!     IWAVLINDEX=IWAVL(JJ)

  DO JL=1,KLEN
   DO JAER=1,IACTAEROP  ! SO2 does not contribute to aerosol optical depth

!-- ITYP is the aerosol type 1:SeaSalt,   2:DesertDust,   3:OrganicMatter, 4: BlackCarbon,
!                            5:Sulphates, 6:FlyingAshes
!   IBIN is the bin index: 1-3:SS, 1-3:DD, 1:OM, 1:BC, 2:SU, 1:FA 
!   Only one "bin" is used for sulphates (SO4) as SO2 does not contribute to the aerosol optical depth. 
! This indeces are defined in su_aerw.F90 

         ITYP=NTYP(JAER)
         IBIN=NBIN(JAER)
         IF(ITYP /= 0 .AND. IBIN == 0) THEN
          WRITE(NULERR,*)'AOD_OP:AT LEAST ONE BIN NEEDS TO BE ALLOCATED FOR TYPE ', ITYP
          WRITE(NULERR,*) 'PROGRAM WILL STOP'
          CALL ABOR1('') 
         ENDIF
         IF(ITYP /= ITYPE(JAER)) THEN
          WRITE(NULERR,*)'AOD_OP: AEROSOL TYPE IS NOT MATCHING', ITYP
          WRITE(NULERR,*) 'PROGRAM WILL STOP'
          CALL ABOR1('')
         ENDIF

      ZAERTAU(JL,JAER)=0.0_JPRB
        
      DO JK=1,NFLEVG

! Always double check dimensioning of arrays ALF_** in yoeaerop.F90 !!!!!!!!!
! It has changed a number of times!!!!!!!!
         IF (ITYP == 1) THEN
          ZALF(JAER)=ALF_SS(IRH(JL,JK),IWAVLINDEX,IBIN)
         ELSEIF (ITYP == 2) THEN
          ZALF(JAER)=ALF_DD(IBIN,IWAVLINDEX)
         ELSEIF (ITYP == 3) THEN
          ZALF(JAER)=ALF_OM(IRH(JL,JK),IWAVLINDEX)
         ELSEIF (ITYP == 4 .OR. ITYP == 6) THEN
          ZALF(JAER)=ALF_BC(IWAVLINDEX)
         ELSEIF (ITYP == 5 .OR. ITYP == 7 .OR. ITYP == 8 .OR. ITYP == 9) THEN
          ZALF(JAER)=ALF_SU(IRH(JL,JK),IWAVLINDEX)
         ENDIF
               
! convert units from m^2/g to m^2/kg
            ZALF(JAER)=ZALF(JAER)*1000._JPRB ! conversion from m^2/g to m^2/kg

            ZDP=PPRESH(JL,JK)-PPRESH(JL,JK-1)
            ZAEREXT=ZAERF(JL,JK,JAER)*ZALF(JAER)

            ZAERTAU(JL,JAER)=ZAERTAU(JL,JAER)+ ZAEREXT*(ZDP/RG)

      ENDDO ! loop over model vertical levels

! Sum over JAER to get total optical depth as function of horizontal position and channel
      PAEROD(JL,JJ)=PAEROD(JL,JJ)+ZAERTAU(JL,JAER)
    ENDDO ! loop over aerosol type

  ENDDO ! loop over horizontal observational points
ENDDO ! loop over MODIS channels

! Security check for negative optical depths
   DO  JJ = 1,KMXCOUNT
    DO JL=1, KLEN
     IF(PAEROD(JL,JJ) < 0.0_JPRB) THEN
      PAEROD(JL,JJ)= 0.0_JPRB
     ENDIF
    ENDDO
   ENDDO

DEALLOCATE(ZAERF)
DEALLOCATE(ITYPE)
DEALLOCATE(ZAERTAU)
DEALLOCATE(ZALF)

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AOD_OP',1,ZHOOK_HANDLE)
END SUBROUTINE AOD_OP

