SUBROUTINE CF_DIGITAL( &
&    K__Sensor,         &
&    K__Band,           &
&    K__NumChans,       &
&    P__DBT,            &
&    K__INDEX,          &
&    K__Chan_High,      &
&    K__Chan_Low,       &
&    K__Chan_Windows,   &
&    K__Window_Width,   &
&    K__Cloud_Flag,     &
&    K__Cloud_Level)  

!*** *CLOUD_DETECT* - CLOUD FLAGGING FOR AIRS USING DIGITAL FILTER
!                     OR CHI-SQUARED METHODS

!    AUTHOR: PHIL WATTS      DATE : 02-01-21

!    PURPOSE.
!    -------
!    FLAG THE PRESENCE OR OTHERWISE OF CLOUD CONTAMINATION IN AIRS
!    CHANNELS USING A RANK-SORTED/MODEL DIFFERENCE METHOD. METHODS
!    SUPPORTED ARE DIGITAL FILTER AND CHI-SQUARED FILTER.

!**  INTERFACE.
!    ---------
!**  *CALL* * CF_DIGITAL( )* (FROM CLOUD_DETECT)
!    WHERE K__SENSOR       : Satellite sensor (AIRS/IASI/CrIS)
!          K__Band         : Band number
!          P__DBT          : Input DBT signal
!          K__INDEX        : Ranking index for DBT 
!          K__Chan_High    : High channel considered for initial minimum search
!          K__Chan_Low     : Low channel considered for initial minimum search
!          K__Chan_Windows : Two channels defining longwave window
!          K__Window_Width : Window width for filter
!          K__Cloud_Flag   : Cloud flag by channel; 0=clear, 1=cloudy

!**  EXTERNALS
!    ---------
!    MOVINGA

!    MODIFICATIONS
!    -------------
!    NONE
!*   -----------------------------------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Collard     03-Feb-2006 Tidy up in preparation for IASI
!        A.Collard     03-May-2006 Band size is now passed in (allows
!                                  for missing channels).
!        A.Collard     04-May-2006 The index of the first cloudy channel is now
!                                  returned to allow cross-band cloud detection
!        A.Collard     16-Feb-2007 Change to the padding to allow the bottom 
!                                  channel to be flagged as clear in a 
!                                  non-quickstart situation.
!        A.Collard     16-Feb-2007 Hopefully temporary introduction of errors
!                                  to make bit-comparible with old version.
!        A.Collard     16-Jan-2009 Gradient check on quick exit 
!                                  Start channel for cold start moved to highest
!                                  channel where BT threshold exceeded
 
USE PARKIND1,  ONLY : JPIM     ,JPRB
USE YOMHOOK,   ONLY : LHOOK,   DR_HOOK

USE YOMIASI, ONLY : S__Cloud_Detect_Setup


IMPLICIT NONE

!* 0.1 Global arrays
!- subroutine arguments
INTEGER(KIND=JPIM),INTENT(IN) :: K__SENSOR              ! sensor
INTEGER(KIND=JPIM),INTENT(IN) :: K__Band                ! Band number 
INTEGER(KIND=JPIM),INTENT(IN) :: K__NumChans            ! Number of useable
                                                        ! channels in band
REAL(KIND=JPRB),   INTENT(IN) :: P__DBT(K__NumChans)    ! Input ranked dBT 
                                                        ! signal 
INTEGER(KIND=JPIM),INTENT(IN) :: K__INDEX(K__NumChans)  ! ranking index for dBT
INTEGER(KIND=JPIM),INTENT(IN) :: K__Window_Width        ! Smoothing width
INTEGER(KIND=JPIM),INTENT(IN) :: K__Chan_High           ! First channel clear 
                                                        ! of high stratospheric
                                                        ! model errors
INTEGER(KIND=JPIM),INTENT(IN) :: K__Chan_Low            ! Last channel clear 
                                                        ! of boundary layer
                                                        ! humidity errors
INTEGER(KIND=JPIM),INTENT(IN) :: K__Chan_Windows(2)     ! Two channels defining
                                                        ! longwave window
INTEGER(KIND=JPIM),INTENT(INOUT) :: K__Cloud_Flag(K__NumChans) ! cloud flags
INTEGER(KIND=JPIM),INTENT(OUT)   :: K__Cloud_Level      ! Index of first cloudy channel


! Local variables


REAL(KIND=JPRB),ALLOCATABLE :: Z__DBT_Smoothed(:) ! Smoothed-ranked DBT signal 
                                                  ! (extended to allow for 
                                                  ! detection algorithm).
INTEGER(KIND=JPIM) :: JCH,JMIN(1),JMAX(1)
INTEGER(KIND=JPIM) :: I__Buffer              ! Number of buffer channels  
INTEGER(KIND=JPIM) :: I__Start_Channel       ! channel to start filtering 
INTEGER(KIND=JPIM) :: I__Start_Channel_Surf  ! 2nd channel to start filtering 
INTEGER(KIND=JPIM) :: I__Max_Channel         ! Used in Quick Exit 
INTEGER(KIND=JPIM) :: I 

LOGICAL :: LLCOLD, LL__WINDOW_GRAD_CHECK, LL__StartChannelChanged
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! These carry the values in S__Cloud_Detect_Setup
INTEGER(KIND=JPIM) :: I__GradChkInterval     ! Used in calculating the gradient

REAL(KIND=JPRB)  :: Z__BT_Threshold   ! Solution contaminated threshold
REAL(KIND=JPRB)  :: Z__Grad_Threshold ! Gradient threshold at which 
                                      ! to stop filter procession
REAL(KIND=JPRB)  :: Z__Window_Grad_Threshold ! Gradient threshold for window
                                      ! check
#include "movinga.intfb.h"

!=============================================================================

IF (LHOOK) CALL DR_HOOK('CF_DIGITAL',0,ZHOOK_HANDLE)

I__GradChkInterval = &
&    S__Cloud_Detect_Setup(K__SENSOR) % N__GradChkInterval(K__Band)
Z__BT_Threshold    = &
&    S__Cloud_Detect_Setup(K__SENSOR) % R__BT_Threshold(K__Band) 
Z__Grad_Threshold =  &
&    S__Cloud_Detect_Setup(K__SENSOR) % R__Grad_Threshold(K__Band)
Z__Window_Grad_Threshold = &
&    S__Cloud_Detect_Setup(K__SENSOR) % R__Window_Grad_Threshold(K__Band)

K__Cloud_Flag(:)=1

!1. Smooth with boxcar (Moving Average) filter and pad averaged array

I__BUFFER = I__GradChkInterval
ALLOCATE(Z__DBT_Smoothed(-I__Buffer+1:K__NumChans+1))

CALL MOVINGA( &
&    P__DBT(K__INDEX(:)),     &
&    K__NumChans,            &
&    K__Window_Width,         &
&    Z__DBT_Smoothed(1:K__NumChans) )

Z__DBT_Smoothed(-I__BUFFER+1:0)  = Z__DBT_Smoothed(1)
Z__DBT_Smoothed(K__NumChans+1) = Z__DBT_Smoothed(K__NumChans)

!3.  Detection

!Find channels to start filtering process.
!If clouds are found starting from I__Start_Channel, we start from
!I__Start_Channel_Surf if it is lower.
JMIN=MINLOC(Z__DBT_Smoothed(K__Chan_High:K__NumChans))
I__Start_Channel_Surf = K__Chan_High+JMIN(1)-1

JMIN=MINLOC(Z__DBT_Smoothed(K__Chan_High:K__Chan_Low))
I__Start_Channel = K__Chan_High+JMIN(1)-1

! Look for highest channel with DBT<-BT_Threshold and move I__Start_Channel
! there if higher than current I__Start_Channel:
JCH = I__Start_Channel
StartChanLoop : DO I=K__Chan_High,K__NumChans
   IF (Z__DBT_Smoothed(I) < -Z__BT_Threshold .OR. I == I__Start_Channel) THEN
      JCH = I
      Exit StartChanLoop
   ENDIF
ENDDO StartChanLoop
I__Start_Channel = JCH

! Do the same with I__Start_Channel_Surf 
JCH = I__Start_Channel_Surf
StartChanLoop_Surf : DO I=K__Chan_High,K__NumChans
   IF (Z__DBT_Smoothed(I) < -Z__BT_Threshold .OR. I == I__Start_Channel_Surf) THEN
      JCH = I
      Exit StartChanLoop_Surf
   ENDIF
ENDDO StartChanLoop_Surf
I__Start_Channel_Surf = JCH

! Find the position of the equivalent maximum departure (for quick exit test)
JMAX=MAXLOC(Z__DBT_Smoothed(K__Chan_High:K__NumChans))
I__Max_Channel = K__Chan_High+JMAX(1)-1

LL__WINDOW_GRAD_CHECK=.TRUE.
IF (ALL(K__Chan_Windows > 0)) LL__WINDOW_GRAD_CHECK = &
&    (ABS(Z__DBT_Smoothed(K__INDEX(K__Chan_Windows(1))) - &
&    Z__DBT_Smoothed(K__INDEX(K__Chan_Windows(2)))) &
&    < Z__Window_Grad_Threshold)

IF (ABS(Z__DBT_Smoothed(I__Start_Channel_Surf)) < Z__BT_Threshold .AND. &
&    ABS(Z__DBT_Smoothed(I__Start_Channel)) < Z__BT_Threshold .AND. &
&    ABS(Z__DBT_Smoothed(I__Max_Channel)) < Z__BT_Threshold .AND. &
&    ABS(Z__DBT_Smoothed(K__NumChans)) < Z__BT_Threshold .AND. &
&    LL__WINDOW_GRAD_CHECK .AND. &
&    S__Cloud_Detect_Setup(K__SENSOR) % L__Do_Quick_Exit) THEN
   !Quick exit
   K__Cloud_Flag(K__INDEX(1:K__NumChans)) = 0
   K__CLoud_Level = 0               ! Special Indicator of Clear FOV
   IF (LHOOK) CALL DR_HOOK('CF_DIGITAL',1,ZHOOK_HANDLE)
   RETURN

ELSEIF (ABS(Z__DBT_Smoothed(I__Start_Channel)) < Z__BT_Threshold .AND. &
     &    Z__DBT_Smoothed(K__NumChans) > Z__BT_Threshold ) THEN
   !Warm cloud start at next-to-bottom channel (allowing one channel for 
   !gradient calculations).
   LLCOLD = .FALSE.
   I__Start_Channel = K__NumChans-1
ELSEIF (Z__DBT_Smoothed(I__Start_Channel) < -Z__BT_Threshold ) THEN
   LLCOLD = .TRUE. 
ELSEIF (Z__DBT_Smoothed(I__Start_Channel) > Z__BT_Threshold ) THEN
   LLCOLD = .FALSE.
ELSE
   LLCOLD = .TRUE.
ENDIF

JCH=I__Start_Channel

! If I__Start_Channel and I__Start_Channel_Surf differ, check that cloud is 
! detected at I__Start_Channel.  If not, set I__Start_Channel to 
! I__Start_Channel_Surf and redo warm/cold-start tests (but no quick exit)

IF (I__Start_Channel /= I__Start_Channel_Surf) THEN

   LL__StartChannelChanged  = .FALSE.
   IF (LLCOLD .AND. ( (Z__DBT_Smoothed(JCH-1)-Z__DBT_Smoothed(JCH+1)) < &
&       Z__Grad_Threshold .AND. &
&       Z__DBT_Smoothed(JCH-I__GradChkInterval)-Z__DBT_Smoothed(JCH+1) < &
&       Z__Grad_Threshold .AND. &
&       ABS(Z__DBT_Smoothed(JCH)) < Z__BT_Threshold)) THEN
           I__Start_Channel = I__Start_Channel_Surf
           LL__StartChannelChanged  = .TRUE.
   ELSEIF ( ((Z__DBT_Smoothed(JCH-1)-Z__DBT_Smoothed(JCH+1)) > &
&               -1.0_JPRB*Z__Grad_Threshold .AND. &
&        (Z__DBT_Smoothed(JCH-I__GradChkInterval)-Z__DBT_Smoothed(JCH+1)) >  &
&               -1.0_JPRB*Z__Grad_Threshold .AND. &
&        ABS(Z__DBT_Smoothed(JCH)) < Z__BT_Threshold) ) THEN
          I__Start_Channel = I__Start_Channel_Surf
          LL__StartChannelChanged  = .TRUE.
   ENDIF

   IF (LL__StartChannelChanged) THEN

      IF (ABS(Z__DBT_Smoothed(I__Start_Channel)) < Z__BT_Threshold .AND. &
&          Z__DBT_Smoothed(K__NumChans) > Z__BT_Threshold ) THEN
         !Warm cloud start at next-to-bottom channel (allowing one channel for 
         !gradient calculations).
         LLCOLD = .FALSE.
         I__Start_Channel = K__NumChans-1
      ELSEIF (Z__DBT_Smoothed(I__Start_Channel) < -Z__BT_Threshold ) THEN
         LLCOLD = .TRUE. 
      ELSEIF (Z__DBT_Smoothed(I__Start_Channel) > Z__BT_Threshold ) THEN
         LLCOLD = .FALSE.
      ELSE
         LLCOLD = .TRUE.
      ENDIF
      JCH = I__Start_Channel
      
   ENDIF
ENDIF


! 3.2a if Cold then Progress towards higher channels whilst -ve difference is 
!     decreasing  
IF (LLCOLD) THEN

   DO WHILE (( (Z__DBT_Smoothed(JCH-1)-Z__DBT_Smoothed(JCH+1)) > &
&               Z__Grad_Threshold .OR. &
&      (Z__DBT_Smoothed(JCH-I__GradChkInterval)-Z__DBT_Smoothed(JCH+1)) > &
&               Z__Grad_Threshold .OR. &
&      ABS(Z__DBT_Smoothed(JCH)) > Z__BT_Threshold) .AND. JCH > 1 )  
      JCH = JCH-1
   ENDDO
ELSE
! 3.2b if Warm then Progress towards higher channels whilst +ve difference is 
!     decreasing
 
   DO WHILE ( ((Z__DBT_Smoothed(JCH-1)-Z__DBT_Smoothed(JCH+1)) < &
&               -1.0_JPRB*Z__Grad_Threshold .OR. &
&        (Z__DBT_Smoothed(JCH-I__GradChkInterval)-Z__DBT_Smoothed(JCH+1)) <  &
&               -1.0_JPRB*Z__Grad_Threshold .OR. &
&        ABS(Z__DBT_Smoothed(JCH)) > Z__BT_Threshold) .AND. JCH > 1 )
      JCH = JCH-1 
   ENDDO
ENDIF
K__Cloud_Level = JCH

!3.4 Check final position is not at higher level than chan_high
! check removed for PSCs and TSCs ...
!          IF (K__Cloud_Level < K__Chan_High) K__Cloud_Level=K__Chan_High

!3.3 Check departure at final position is not outside threshold
IF (ABS(Z__DBT_Smoothed(JCH)) > Z__BT_Threshold) K__Cloud_Level=1 !K__Chan_High

!   K__Cloud_Flag(K__INDEX(K__Cloud_Level:K__NumChans))=1
IF (K__Cloud_Level /= 1) K__Cloud_Flag(K__INDEX(1:K__Cloud_Level-1))=0

! Output channel number for first cloudy level
K__Cloud_Level=K__INDEX(K__Cloud_Level)

IF (LHOOK) CALL DR_HOOK('CF_DIGITAL',1,ZHOOK_HANDLE)
END SUBROUTINE CF_DIGITAL
