SUBROUTINE CLOUD_DETECT( &
&    K__SENSOR,     &
&    K__NCHANS,     &
&    K__CHANID,     &
&    P__MODELBTs,   &
&    P__OBSBTs,     &
&    P__CHAN_LEVEL, &
&    K__CLOUD_FLAG, &
&    K__MINLEV,     &
&    K__MAXLEV)  

!*** *CLOUD_DETECT* - Cloud flagging for AIRS/IASI using digital filter 
!                     or other METHODS

!    AUTHOR: PHIL WATTS      DATE : 02-01-21

!    PURPOSE.
!    -------
!    Flag the presence or otherwise of cloud contamination in AIRS/IASI
!    channels using a rank-sorted/model difference method. Currently 
!    on a digital filter is supported.                     

!**  INTERFACE.
!    ---------
!**  *CALL* * CLOUD_DETECT( )* (FROM RADTRCLD)
!    WHERE K__SENSOR      : Satellite sensor (AIRS/IASI/CrIS)
!          K__NCHANS      : Number of channels
!          K__CHANID      : Channel indicies of input channels
!          P__MODELBTS    : Clear brightness temperatures from Model
!          P__OBSBTS      : Potentially cloudy observations
!          P__CHAN_LEVEL  : Pressure level assignment to channel (RADTRCLD)
!          K__CLOUD_FLAG  : Cloud flag by channel; 0=clear, 1=cloudy
!          K__MINLEV      : level corresponding to the top of the initial 
!                           cloud search (200hPa)
!          K__MAXLEV      : level corresponding to the lower extent of cloud
!                           search (surface)

!**  EXTERNALS
!    ---------
!    KEYSORT, CF_DIGITAL, CF_CHISQR, MOVINGA

!    MODIFICATIONS
!    -------------
!    NONE
!*   ------------------------------------------------------------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Collard     01-Feb-2006 Big rewrite to include IASI
!        A.Collard     03-May-2006 Allow for missing channels
!        A.Collard     04-May-2006 Allow cross-band cloud detection
!        A.Collard     15-Jan-2007 Initialise with automatic cross-band for 
!                                  all channels from band 1 for IASI.

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN    , ONLY : NULOUT   
USE ECSORT_MIX, ONLY : KEYSORT

USE YOMIASI, ONLY : S__Cloud_Detect_Setup, &
&                   JP__Digital_Filter,    &
&                   MSENSOR_IASI
IMPLICIT NONE

!* 0.1 Global arrays
INTEGER(KIND=JPIM),INTENT(IN)    :: K__NCHANS ! no. of channels 
INTEGER(KIND=JPIM),INTENT(IN)    :: K__SENSOR ! sensor
INTEGER(KIND=JPIM),INTENT(IN)    :: K__MINLEV ! level corresponding to top of
                                              ! initial cloud search (200hPa) 
INTEGER(KIND=JPIM),INTENT(IN)    :: K__MAXLEV ! level corresponding bottom 
                                              ! of cloud search (surface)
INTEGER(KIND=JPIM),INTENT(IN)    :: K__CHANID(K__NCHANS)  ! Channel IDs
REAL(KIND=JPRB)   ,INTENT(IN)    :: P__MODELBTS(K__NCHANS)! Model clear 
                                                          ! radiance estimates
REAL(KIND=JPRB)   ,INTENT(IN)    :: P__OBSBTS(K__NCHANS)  ! Observed radiances
REAL(KIND=JPRB)   ,INTENT(IN)    :: P__CHAN_LEVEL(K__NCHANS) ! channel level
INTEGER(KIND=JPIM),INTENT(OUT)   :: K__CLOUD_FLAG(K__NCHANS) ! cloud flags

!* 0.2 local variables
INTEGER(KIND=JPIM) :: IST,ICOUNT,J,I_K,JBAND,JBAND2
LOGICAL :: LL__AerosolPresent

!* 0.3 Local variables - band splitting details
INTEGER(KIND=JPIM), POINTER     :: I__Bands(:,:)     ! Channel detection bands
INTEGER(KIND=JPIM), POINTER     :: I__Band_Size(:)   ! # of chans per band
INTEGER(KIND=JPIM), POINTER     :: I__BandToUse(:)   ! Cross band definitions
INTEGER(KIND=JPIM)              :: I__Num_Bands      ! # of bands
INTEGER(KIND=JPIM)              :: I__NumFoundChans  ! # of useable chans
INTEGER(KIND=JPIM)       :: I__BandNumber(K__NChans) ! Channel band indicator
INTEGER(KIND=JPIM), ALLOCATABLE :: INDEX(:),IDCHAN(:),I__Cloud_Flag(:)
INTEGER(KIND=JPIM)       :: I__WindowBounds(2)  ! Boundary of window
INTEGER(KIND=JPIM)       :: I__Window_Chans(2)  ! Boundary of longwave window

LOGICAL  :: LL__Do_CrossBand

REAL(KIND=JPRB), ALLOCATABLE :: Z__DBT(:),Z__LEVEL(:)

!* 0.4 Local variables - digital filter parameters
INTEGER(KIND=JPIM) :: I__CHAN_HIGH  ! First high channel cloud-free a-priori
INTEGER(KIND=JPIM) :: I__CHAN_LOW   ! Lowest channel considered
INTEGER(KIND=JPIM) :: I__FirstCloudyChannel ! Lowest cloud affected channel
INTEGER(KIND=JPIM) :: I__METHOD     ! Cloud detection method
INTEGER(KIND=JPIM), POINTER :: I__Window_Width(:)

REAL(KIND=JPRB) :: Z__Cloud_Level  ! Pressure level of cloud
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cf_digital.intfb.h"
#include "aerosol_detect.intfb.h"


IF (LHOOK) CALL DR_HOOK('CLOUD_DETECT',0,ZHOOK_HANDLE)

! Get correct processing parameters for this sensor:
I__METHOD         =  S__Cloud_Detect_Setup(K__SENSOR) % N__Filter_Method
I__Num_Bands      =  S__Cloud_Detect_Setup(K__SENSOR) % N__Num_Bands
I__Band_Size      => S__Cloud_Detect_Setup(K__SENSOR) % N__Band_Size
I__Bands          => S__Cloud_Detect_Setup(K__SENSOR) % N__Bands
I__Window_Width   => S__Cloud_Detect_Setup(K__SENSOR) % N__Window_Width
I__BandToUse      => S__Cloud_Detect_Setup(K__SENSOR) % N__BandToUse
LL__Do_CrossBand  =  S__Cloud_Detect_Setup(K__SENSOR) % L__Do_CrossBand

! Initialise
K__CLOUD_FLAG(:)=1       ! intialise ALL channels to cloudy

! If using cross-band, set up an array indicating which channels correspond 
! to which bands in K__CHANID
IF (LL__Do_CrossBand) THEN
   I__BandNumber(:)=-1  ! Initialise     
   DO JBAND = 1, I__Num_Bands  
      DO I_K=1,K__NCHANS
         IF (ANY(I__BANDS(:,JBAND) == K__CHANID(I_K))) &
&              I__BandNumber(I_K)=JBand
      ENDDO
   ENDDO
ENDIF

!1 Loop over bands  
BAND_LOOP: DO JBAND = 1, I__Num_Bands         ! Loop over bands

  ! Don't bother doing the cloud detection if we're just going to use 
  ! the results from another band anyway:
  IF (LL__Do_CrossBand) THEN
     IF (.NOT.(ANY(I__BandToUse(:) == JBAND))) CYCLE
  ENDIF

  ALLOCATE (Z__DBT(I__Band_Size(JBAND)))
  Z__DBT(:)   = 0.0_JPRB

  ALLOCATE (Z__LEVEL(I__Band_Size(JBAND)))
  Z__LEVEL(:) = REAL(K__MAXLEV)

  ALLOCATE (I__Cloud_Flag(I__Band_Size(JBAND)))
  ALLOCATE (INDEX(I__Band_Size(JBAND)))

  ALLOCATE (IDCHAN(I__Band_Size(JBAND)))
  IDCHAN(:)   = 1

  I__WindowBounds(:)   = &
&      S__Cloud_Detect_Setup(K__SENSOR) % N__Window_Bounds(JBand,:)

!1.1 find channels within current band --------------------------------------
  I__NumFoundChans = 0
  I__Window_Chans(:) = -1

  DO J=1,I__Band_Size(JBAND)
    DO I_K=1,K__NCHANS
      IF (K__CHANID(I_K) == I__BANDS(J,JBAND)) THEN
        IF (P__OBSBTS(I_K) < 0. .OR. P__MODELBTS(I_K) < 0.) CYCLE
        I__NumFoundChans = I__NumFoundChans + 1
        IF(I__NumFoundChans > I__Band_Size(JBAND)) THEN
!DJS Temporary fix
!         write(0,*)'I__NumFoundChans too big - so reset it',I__NumFoundChans,I__Band_Size(JBAND),JBAND,K__SENSOR
          I__NumFoundChans=I__Band_Size(JBAND)
        ENDIF
        Z__DBT(I__NumFoundChans)=P__OBSBTS(I_K)-P__MODELBTS(I_K)
        Z__LEVEL(I__NumFoundChans)=P__CHAN_LEVEL(I_K)
        INDEX(I__NumFoundChans)=I__NumFoundChans
        IDCHAN(I__NumFoundChans)=I_K
        IF (K__CHANID(I_K) == I__WindowBounds(1)) &
&             I__Window_Chans(1) = I__NumFoundChans
        IF (K__CHANID(I_K) == I__WindowBounds(2)) &
&             I__Window_Chans(2) = I__NumFoundChans
      ENDIF
    ENDDO
  ENDDO
  IF ( I__NumFoundChans == 0 ) THEN
     WRITE(NULOUT,*) &
&          '**CLOUD_DETECT - WARNING: CHANNELS NOT FOUND CYCLING BAND: **', &
&          JBAND
     DEALLOCATE (Z__DBT,Z__LEVEL,I__Cloud_Flag,INDEX,IDCHAN)
     CYCLE BAND_LOOP
  ENDIF

!----------------------------------------------------------------------------
  IST=0
  ICOUNT=I__NumFoundChans
  I__Cloud_Flag(:)=1

!1. Sort according to level assignment
  CALL KEYSORT(IST,Z__LEVEL,ICOUNT,INDEX=INDEX,INIT=.TRUE.)

!1.1 Find I__CHAN_LOW - lowest channel considered based on level assignment
  J=1
  DO WHILE (J < I__NumFoundChans .AND. Z__LEVEL(INDEX(J)) < REAL(K__MAXLEV))
    J=J+1
  ENDDO

  IF (J == I__NumFoundChans) THEN
    I__CHAN_LOW = I__NumFoundChans-1 
  ELSE
    I__CHAN_LOW = J 
  ENDIF
  IF(I__CHAN_LOW <= 1)I__CHAN_LOW=1

!1.1a Find I__CHAN_HIGH - first high channel considered cloud free a priori  
  J=1
  DO WHILE (J < I__NumFoundChans .AND. Z__LEVEL(INDEX(J)) < REAL(K__MINLEV))
    J=J+1
  ENDDO
  I__CHAN_HIGH=J

!2. Call cloud detector

  IF (I__METHOD == JP__Digital_Filter) THEN
    I__Cloud_Flag(:) = K__CLOUD_FLAG(IDCHAN(:))
    CALL CF_DIGITAL( &
&        K__SENSOR,                  &
&        JBAND,                      &
&        I__NumFoundChans,           &
&        Z__DBT(1:I__NumFoundChans), &
&        INDEX(1:I__NumFoundChans),  &
&        I__CHAN_HIGH,               &
&        I__CHAN_LOW,                &
&        I__Window_Chans,            &
&        I__Window_Width(JBAND),     &
&        I__Cloud_Flag,              &
&        I__FirstCloudyChannel)  
    K__CLOUD_FLAG(IDCHAN(1:I__NumFoundChans)) = &
&                          I__Cloud_Flag(1:I__NumFoundChans)

    ! Set cloud level for cross-band:
    IF (I__FirstCloudyChannel == 0) THEN   ! FOV is completely clear
       Z__Cloud_Level = 1.e20   ! Large value
    ELSE 
       Z__Cloud_Level = P__Chan_Level(IDCHAN(I__FirstCloudyChannel))
    ENDIF

    ! Automatically do cross band cloud detection for all channels
    ! (whether assigned a band or not) if JBand == 1.  This can be
    ! over-ridden for the other bands.

    IF (K__SENSOR == MSENSOR_IASI .AND. JBand == 1) &
&        WHERE(P__Chan_Level(:) < Z__Cloud_Level) K__CLOUD_FLAG(:) = 0

    CrossBand : IF (LL__Do_CrossBand) THEN   
       ! Cross Band:
       ! Loop through bands applying cloud detection to those that take their 
       ! cloud detection information from the current band JBAND.
       DO JBand2 = 1, I__Num_Bands
          IF (I__BandToUse(JBand2) == JBand) THEN
             WHERE(P__Chan_Level(:) < Z__Cloud_Level .AND. &
&                    I__BandNumber == JBand2) K__CLOUD_FLAG(:) = 0
          ENDIF
       ENDDO
    ENDIF CrossBand

  ELSE
    ! No other methods implemented as yet
    WRITE(6,*) 'Only CF_DIGITAL (I__METHOD=1) is currently implemented'
  ENDIF
  
! Deallocate arrays
  DEALLOCATE (Z__DBT,Z__LEVEL,I__Cloud_Flag,INDEX,IDCHAN)

ENDDO BAND_LOOP                      ! Loop over band

! Do aerosol detection if required
IF (S__Cloud_Detect_Setup(K__SENSOR) % L__Do_AerosolDetection) THEN
   CALL Aerosol_Detect (&
&         K__SENSOR,     &
&         K__NCHANS,     &
&         K__CHANID,     &
&         P__MODELBTs,   &    
&         P__OBSBTs,     &
&         LL__AerosolPresent)

   IF (LL__AerosolPresent) K__CLOUD_FLAG(:) = -K__CLOUD_FLAG(:)
ENDIF

! Nullify pointers
NULLIFY(I__Band_Size, I__Bands, I__Window_Width, I__BandToUse)

IF (LHOOK) CALL DR_HOOK('CLOUD_DETECT',1,ZHOOK_HANDLE)

END SUBROUTINE CLOUD_DETECT
