SUBROUTINE CLOUD_ESTIMATE( &
&    K__SENSOR,     &
&    K__NCHANS,     &
&    K__CHANID,     &
&    K__LEVELS,     &
&    P__MODELBTs,   &
&    P__OBSBTs,     &
&    P__RADCLDLEV,  &
&    P__PRESF,      &
&    P__PTOP,       &
&    P__NE)                

!*** *CLOUD_ESTIMATE* - Cloud parameter estimation for HIRS/AIRS/IASI 

!    AUTHOR: Tony McNally      DATE : 02-03-2009

!    PURPOSE.
!    -------
!    calculate cloud parameters that fit observed radiances for use in cloud analysis

!**  INTERFACE.
!    ---------
!**  *CALL* * CLOUD_ESTIMATE( )* (FROM HRETR)
!    WHERE K__SENSOR      : Satellite sensor (AIRS/IASI/CrIS)
!          K__NCHANS      : Number of channels
!          K__CHANID      : Channel indicies of input channels
!          P__MODELBTS    : Clear brightness temperatures from Model
!          P__OBSBTS      : Potentially cloudy observations
!          P__RADCLDLEV   : overcast radiances at each level (from RTTOV)
!          P__PTOP        : cloud top pressure
!          P__NE          : cloud effective fraction

!**  EXTERNALS
!    ---------

!    MODIFICATIONS
!    -------------
!    NONE
!*   ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMTVRAD , ONLY : AIRSCHAN_BACK, IASICHAN_BACK, MSENSOR_HIRS, MSENSOR_AIRS,&
 &                    MSENSOR_IASI, MSENSOR_IRAS
USE YOMDIMO  , ONLY : NMXTCHU
USE YOMDIM   , ONLY : NFLEVG
USE YOMANCS  , ONLY : RMDI 
!USE YOMLUN   , ONLY : NULERR   

IMPLICIT NONE

!* 0.1 Global arrays
INTEGER(KIND=JPIM),INTENT(IN)    :: K__NCHANS ! no. of channels 
INTEGER(KIND=JPIM),INTENT(IN)    :: K__SENSOR ! sensor
INTEGER(KIND=JPIM),INTENT(IN)    :: K__CHANID(K__NCHANS)  ! Channel IDs
INTEGER(KIND=JPIM),INTENT(IN)    :: K__LEVELS  ! number of levels corresponding to overcast rads
REAL(KIND=JPRB)   ,INTENT(IN)    :: P__MODELBTS(K__NCHANS)! Model clear 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P__OBSBTS(K__NCHANS)  ! Observed radiances
REAL(KIND=JPRB)   ,INTENT(IN)    :: P__RADCLDLEV(K__NCHANS,K__LEVELS)  ! overcast rad at each level
REAL(KIND=JPRB)   ,INTENT(IN)    :: P__PRESF(NFLEVG)  ! pressures of model levelsl
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P__PTOP  ! cloud top pressure                     
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P__NE    ! cloud effective amount                 

!* 0.2 local variables
INTEGER(KIND=JPIM) :: ICH,ILEV,i_NPSAVE2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: Z__OBSRAD(NMXTCHU)  ! Observed radiances
REAL(KIND=JPRB) :: Z__MODRAD(NMXTCHU)  ! Observed radiances
REAL(KIND=JPRB) :: Z_RADCONVX(0:20,NMXTCHU),Z_RADCONVY(0:20,NMXTCHU),Z_D1,Z_D2,Z_D3
REAL(KIND=JPRB), ALLOCATABLE, DIMENSION(:)  :: ZNP,z_CLDCOST2
REAL(KIND=JPRB), ALLOCATABLE, DIMENSION(:)  :: Z_SUMDELTA1,Z_SUMDELTA2

IF (LHOOK) CALL DR_HOOK('CLOUD_ESTIMATE',0,ZHOOK_HANDLE)

!*    1.3  Brightness temperature to radiance 

Z_RADCONVX(:,:) = 999.0_JPRB
Z_RADCONVY(:,:) = 999.0_JPRB

IF (K__SENSOR == MSENSOR_HIRS) THEN
Z_RADCONVX(MSENSOR_HIRS,       5) = 1.19106590E-5_JPRB * 716.0_JPRB**3
Z_RADCONVY(MSENSOR_HIRS,       5) = 1.438833 * 716.0_JPRB
Z_RADCONVX(MSENSOR_HIRS,       6) = 1.19106590E-5_JPRB * 733.0_JPRB**3
Z_RADCONVY(MSENSOR_HIRS,       6) = 1.438833 * 733.0_JPRB
Z_RADCONVX(MSENSOR_HIRS,       7) = 1.19106590E-5_JPRB * 748.0_JPRB**3
Z_RADCONVY(MSENSOR_HIRS,       7) = 1.438833 * 748.0_JPRB
Z_RADCONVX(MSENSOR_HIRS,       8) = 1.19106590E-5_JPRB * 898.0_JPRB**3
Z_RADCONVY(MSENSOR_HIRS,       8) = 1.438833 * 898.0_JPRB
ENDIF

! This should be checked by Tony McNally
IF (K__SENSOR == MSENSOR_IRAS) THEN
Z_RADCONVX(MSENSOR_HIRS,       5) = 1.19106590E-5_JPRB * 715.0_JPRB**3
Z_RADCONVY(MSENSOR_HIRS,       5) = 1.438833 * 715.0_JPRB
Z_RADCONVX(MSENSOR_HIRS,       6) = 1.19106590E-5_JPRB * 732.0_JPRB**3
Z_RADCONVY(MSENSOR_HIRS,       6) = 1.438833 * 732.0_JPRB
Z_RADCONVX(MSENSOR_HIRS,       7) = 1.19106590E-5_JPRB * 748.0_JPRB**3
Z_RADCONVY(MSENSOR_HIRS,       7) = 1.438833 * 748.0_JPRB
Z_RADCONVX(MSENSOR_HIRS,       9) = 1.19106590E-5_JPRB * 901.0_JPRB**3
Z_RADCONVY(MSENSOR_HIRS,       9) = 1.438833 * 901.0_JPRB
ENDIF

IF (K__SENSOR == MSENSOR_AIRS) THEN
Z_RADCONVX(MSENSOR_AIRS,AIRSCHAN_BACK(239)) = 1.19106590E-5_JPRB * 717.0_JPRB**3
Z_RADCONVY(MSENSOR_AIRS,AIRSCHAN_BACK(239)) = 1.438833 * 717.0_JPRB
Z_RADCONVX(MSENSOR_AIRS,AIRSCHAN_BACK(252)) = 1.19106590E-5_JPRB * 721.0_JPRB**3
Z_RADCONVY(MSENSOR_AIRS,AIRSCHAN_BACK(252)) = 1.438833 * 721.0_JPRB
Z_RADCONVX(MSENSOR_AIRS,AIRSCHAN_BACK(308)) = 1.19106590E-5_JPRB * 738.0_JPRB**3
Z_RADCONVY(MSENSOR_AIRS,AIRSCHAN_BACK(308)) = 1.438833 * 738.0_JPRB
Z_RADCONVX(MSENSOR_AIRS,AIRSCHAN_BACK(355)) = 1.19106590E-5_JPRB * 753.0_JPRB**3
Z_RADCONVY(MSENSOR_AIRS,AIRSCHAN_BACK(355)) = 1.438833 * 753.0_JPRB
Z_RADCONVX(MSENSOR_AIRS,AIRSCHAN_BACK(787)) = 1.19106590E-5_JPRB * 917.0_JPRB**3
Z_RADCONVY(MSENSOR_AIRS,AIRSCHAN_BACK(787)) = 1.438833 * 917.0_JPRB
ENDIF

IF (K__SENSOR == MSENSOR_IASI) THEN
Z_RADCONVX(MSENSOR_IASI,IASICHAN_BACK(294)) = 1.19106590E-5_JPRB * 718.0_JPRB**3
Z_RADCONVY(MSENSOR_IASI,IASICHAN_BACK(294)) = 1.438833 * 718.0_JPRB
Z_RADCONVX(MSENSOR_IASI,IASICHAN_BACK(308)) = 1.19106590E-5_JPRB * 721.0_JPRB**3
Z_RADCONVY(MSENSOR_IASI,IASICHAN_BACK(308)) = 1.438833 * 721.0_JPRB
Z_RADCONVX(MSENSOR_IASI,IASICHAN_BACK(375)) = 1.19106590E-5_JPRB * 738.0_JPRB**3
Z_RADCONVY(MSENSOR_IASI,IASICHAN_BACK(375)) = 1.438833 * 738.0_JPRB
Z_RADCONVX(MSENSOR_IASI,IASICHAN_BACK(434)) = 1.19106590E-5_JPRB * 753.0_JPRB**3
Z_RADCONVY(MSENSOR_IASI,IASICHAN_BACK(434)) = 1.438833 * 753.0_JPRB
Z_RADCONVX(MSENSOR_IASI,IASICHAN_BACK(1090))= 1.19106590E-5_JPRB * 917.0_JPRB**3
Z_RADCONVY(MSENSOR_IASI,IASICHAN_BACK(1090))= 1.438833 * 917.0_JPRB
ENDIF

ALLOCATE(ZNP(NFLEVG))
ALLOCATE(z_CLDCOST2(NFLEVG))
ALLOCATE(Z_SUMDELTA1(NFLEVG))
ALLOCATE(Z_SUMDELTA2(NFLEVG))

Z_SUMDELTA1(:)=0.0_JPRB
Z_SUMDELTA2(:)=0.0_JPRB
z_CLDCOST2(:)=0.0_JPRB
Z_D1 = 0.0_JPRB
Z_D2 = 0.0_JPRB
Z_D3 = 0.0_JPRB
i_NPSAVE2 = NFLEVG

P__PTOP = 123.0_JPRB
P__NE   =   0.0_JPRB

  DO ICH = 1, K__NCHANS
    IF ( NINT( Z_RADCONVX(K__SENSOR,ICH) )  /= 999 ) THEN
      IF(P__OBSBTS(ICH)==RMDI .OR. P__MODELBTS(ICH)==RMDI) THEN
!        WRITE(NULERR,*)'** CLOUD_ESTIMATE EXITING WITH MISSING DATA **',K__SENSOR,ICH,P__OBSBTS(ICH),P__MODELBTS(ICH)
         GOTO 123
      ENDIF
      Z__OBSRAD(ICH) = Z_RADCONVX(K__SENSOR,ICH) / (EXP(Z_RADCONVY(K__SENSOR,ICH)/P__OBSBTS(ICH)) - 1.0_JPRB) 
      Z__MODRAD(ICH) = Z_RADCONVX(K__SENSOR,ICH) / (EXP(Z_RADCONVY(K__SENSOR,ICH)/P__MODELBTS(ICH)) - 1.0_JPRB) 
!     WRITE(NULERR,*)'KEY-CHAN-FOUND:',K__SENSOR,K__CHANID(ICH),P__MODELBTS(ICH),P__OBSBTS(ICH)
!     WRITE(NULERR,*)'RADIANCE SEARCH:'
      DO ILEV = 1, NFLEVG
        Z_D2 = P__RADCLDLEV(ICH,ILEV) - Z__MODRAD(ICH)
        Z_SUMDELTA2(ILEV) = Z_SUMDELTA2(ILEV) + (Z_D2*Z_D2)
        Z_D1 = (Z__OBSRAD(ICH) - Z__MODRAD(ICH)) * Z_D2
        Z_SUMDELTA1(ILEV) = Z_SUMDELTA1(ILEV) + Z_D1
      ENDDO
    ENDIF
  ENDDO

  DO ILEV = 1, NFLEVG
     IF (ABS(Z_SUMDELTA2(ILEV)) > 1.0E-10_JPRB) THEN
        ZNP(ILEV)=Z_SUMDELTA1(ILEV)/Z_SUMDELTA2(ILEV)
     ELSE
        ZNP(ILEV)=0.0_JPRB
     ENDIF
!       ZNP(ILEV)=MAX(MIN(ZNP(ILEV),1.0_JPRB),0.0_JPRB)
  ENDDO

  z_CLDCOST2(:)=0.0_JPRB
  DO ICH = 1, K__NCHANS
     IF ( NINT( Z_RADCONVX(K__SENSOR,ICH) )  /= 999 ) THEN
!       WRITE(NULERR,*)'EVALUATING CLOUD COST FOR CHANNEL:',K__SENSOR,ICH,K__CHANID(ICH)
        DO ILEV = 1, NFLEVG
           Z_D3 = ( Z__OBSRAD(ICH) - ( ZNP(ILEV)*P__RADCLDLEV(ICH,ILEV) + (1.0_JPRB - ZNP(ILEV))*Z__MODRAD(ICH) ) )**2
           z_CLDCOST2(ILEV) = z_CLDCOST2(ILEV) + Z_D3
        ENDDO
     ENDIF
  ENDDO

  Z_D1 = 1.0E10_JPRB
  DO ILEV = 1, NFLEVG
!    WRITE(NULERR,*)'FINDING MINIMUM OF ALTERNATIVE TOTAL CLOUD COST:',ILEV,ZNP(ILEV),ABS(z_CLDCOST2(ILEV))
     IF(ABS(z_CLDCOST2(ILEV)) < Z_D1 ) THEN
       Z_D1 = ABS(z_CLDCOST2(ILEV))
       i_NPSAVE2 = ILEV
     ENDIF
  ENDDO


  P__NE = ZNP(i_NPSAVE2)
  P__PTOP = P__PRESF(i_NPSAVE2)/100.0_JPRB
  

123 CONTINUE                                                             

 
DEALLOCATE(Z_SUMDELTA1,Z_SUMDELTA2,z_CLDCOST2,ZNP)

IF (LHOOK) CALL DR_HOOK('CLOUD_ESTIMATE',1,ZHOOK_HANDLE)

END SUBROUTINE CLOUD_ESTIMATE
