SUBROUTINE CO2_TCMR_TL &
 & ( KDLEN  , KLEN  , KDCOUNT, KCOUNT, & 
 &   PPRESH, PPRESF, &
 &   YDGEMSPROF, &  
 &   PPRESH5, PPRESF5, PSOLEL, &
 &   YDGEMSPROF5, &
 &   PCO2MEAN)

!*** *CO2_TCMR_TL* Tangent Linear Operator routine for aerosol optical depth observations.

!**   INTERFACE.
!     ----------
!          *CO2_TCMR_TL* IS CALLED FROM *HOPTL*.
!     CALL CO2_TCMR_TL(KDLEN,ILEN,IMXCOUNT,ZQF, ZTF, ZPRESH, ZPRESF, ZLS, ZGEMSPROF,
!                  ZQF5, ZTF5, ZPRESH5, ZPRESF5, ZLS5, ZGEMSPROF5, ZXPP)
!     WHERE KDLEN      = First dimension: Max number of observations, this set (INPUT)
!           KLEN       = Number of aerosol optical depth obs (INPUT)
!           IMXCOUNT   = Number of channels (INPUT)
!           ZQF       =  Perturb in specific humidity at observation points, model levels (INPUT)
!           ZTF       =  Perturb in temperature at observation points, model levels (INPUT)
!           ZPRESH,ZPRESF = Perturb in Half/Full level pressure values at obs points, model levels (INPUT)
!           ZGEMSPROF     =  Perturb in GEMS species, including aerosol mixing ratios (INPUT)
!           ZQF5       =  Specific humidity at observation points, model levels (INPUT)
!           ZTF5       =  Temperature at observation points, model levels (INPUT)
!           ZPRESH5,ZPRESF5 =   Half/Full level pressure values at obs points, model levels (INPUT)
!           ZGEMSPROF5     = GEMS species including aerosol mixing ratios (INPUT)

!           ZXPP       = Aerosol optical depth at different wavelengths (OUTPUT)



!**   AUTHOR.
!     -------
!        ANGELA BENEDETTI        *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2005-06-17

!     PURPOSE.
!     --------
!     - derive tangent linear aerosol optical thickness from profiles of mass mixing ratio 

! Optical properties in su_aerop.F90 avaliable at following wavelenghts
!         WVAL=   1 (340),  2 (380),  3 (400),  4 (440),  5 (469),  6 (500), 7 (532), 8 (555),
!                 9 (645), 10 (670), 11 (800), 12 (858), 13 (865), 14 (1240), 15 (1640), 16 (2130)
!     'MODIS Channels' (channels from other instruments can be added)
!      -         470 nm, 550 nm, 660 nm, 870 nm, 1240 nm, 1630 nm, 2130 nm
! Corresponding IWAVL
!      IWAVL = [5, 8, 10, 13, 14, 15, 16]
! **** DEPENDS ON J.-J.'s choice of numbering in su_aerop.F90 !!!!!!
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOM_YGFL , ONLY : NGHG
USE YOMDIM, ONLY : NFLEVG
USE YOMDB
USE YOMGRB

USE YOMCST    ,ONLY : RG, RMD, RMCO2

USE TYPE_GEMS_PROFILES

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN, KLEN, KDCOUNT
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 

REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH5(KDLEN,0:NFLEVG) ! PRESSURE ON HALF LEVELS FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF5(KDLEN,NFLEVG)   ! PRESSURE ON FULL LEVELS FROM TRAJ

REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH(KDLEN,0:NFLEVG) ! perturbation on PRESSURE ON HALF LEVELS
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF(KDLEN,NFLEVG) !  " "    PRESSURE ON FULL LEVELS
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOLEL(KDLEN)          ! SOLAR ELEVATION

TYPE (TYPE_GEMSPROF)   ,INTENT(INOUT) :: YDGEMSPROF(KDLEN,NGEMS)
TYPE (TYPE_GEMSPROF)   ,INTENT(IN)    :: YDGEMSPROF5(KDLEN,NGEMS)

REAL(KIND=JPRB)   ,INTENT(OUT) :: PCO2MEAN(KDLEN,KDCOUNT) ! AEROSOL OPT DEPTH PERTURBATION 

!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JK, JL, JJ, JF, JLEN, J, INDEX

REAL(KIND=JPRB), DIMENSION(KLEN,NFLEVG)   :: ZCO2MR5, ZCO2MR
REAL(KIND=JPRB), DIMENSION(KDLEN,KDCOUNT)   :: ZCO2MEAN5
REAL(KIND=JPRB)    :: ZDP5, ZDP, ZDPSUM, ZDPSUM5
REAL(KIND=JPRB)    :: ZMMR2VMR
REAL(KIND=JPRB), DIMENSION(60) :: ZAK_PRES
REAL(KIND=JPRB), DIMENSION(600) :: ZDUM
REAL(KIND=JPRB), DIMENSION(10,60) :: ZAK
REAL(KIND=JPRB), DIMENSION(10) :: ZAK_SELEL
REAL(KIND=JPRB), DIMENSION(NFLEVG) :: ZAK_MODEL

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "intavg.h"

DATA ZAK_PRES / &
 &     0.30,    0.38,    0.50,    0.65,    0.80,    1.03,    1.33, &
 &     1.72,    2.22,    2.87,    3.81,    5.06,    6.75,    9.00, &
 &    11.97,   14.15,   16.34,   19.04,   22.27,   25.49,   29.72, &
 &    34.67,   40.47,   47.29,   55.29,   64.67,   75.65,   88.50, &
 &   103.50,  121.10,  141.70,  165.80,  194.00,  227.00,  265.00, &
 &   308.00,  356.50,  411.00,  441.60,  472.20,  506.35,  540.50, &
 &   578.55,  616.60,  637.75,  658.90,  680.05,  701.20,  724.65, &
 &   748.10,  771.55,  795.00,  820.95,  846.90,  872.85,  898.80, &
 &   927.35,  955.90,  984.45, 1013.00 /
DATA ZAK_SELEL / &
 &  70.0, 60.0, 50.0, 40.0, 30.0, 25.0, 20.0, 15.0, 10.0, 5.0 /
DATA ZDUM / &
 &  0.058, 0.057, 0.056, 0.053, 0.050, 0.047, 0.043, 0.038, 0.030, 0.019, &
 &  0.061, 0.060, 0.058, 0.056, 0.052, 0.050, 0.046, 0.040, 0.032, 0.020, &
 &  0.062, 0.061, 0.060, 0.057, 0.054, 0.051, 0.047, 0.041, 0.033, 0.020, &
 &  0.063, 0.062, 0.061, 0.058, 0.055, 0.052, 0.048, 0.042, 0.034, 0.021, &
 &  0.068, 0.067, 0.065, 0.063, 0.059, 0.056, 0.051, 0.045, 0.036, 0.023, &
 &  0.068, 0.068, 0.066, 0.064, 0.060, 0.057, 0.052, 0.046, 0.038, 0.024, &
 &  0.070, 0.069, 0.067, 0.065, 0.060, 0.057, 0.053, 0.047, 0.038, 0.025, &
 &  0.071, 0.070, 0.069, 0.066, 0.062, 0.058, 0.054, 0.048, 0.039, 0.026, &
 &  0.074, 0.073, 0.071, 0.068, 0.063, 0.060, 0.055, 0.049, 0.040, 0.027, &
 &  0.078, 0.077, 0.075, 0.072, 0.067, 0.063, 0.058, 0.051, 0.041, 0.028, &
 &  0.082, 0.080, 0.078, 0.074, 0.069, 0.065, 0.059, 0.052, 0.042, 0.029, &
 &  0.087, 0.086, 0.083, 0.079, 0.072, 0.067, 0.061, 0.053, 0.043, 0.030, &
 &  0.095, 0.093, 0.090, 0.085, 0.078, 0.072, 0.065, 0.056, 0.044, 0.032, &
 &  0.108, 0.106, 0.102, 0.096, 0.086, 0.080, 0.071, 0.060, 0.048, 0.035, &
 &  0.124, 0.121, 0.117, 0.110, 0.099, 0.091, 0.082, 0.070, 0.056, 0.043, &
 &  0.129, 0.126, 0.122, 0.115, 0.104, 0.096, 0.087, 0.075, 0.062, 0.049, &
 &  0.139, 0.136, 0.132, 0.124, 0.113, 0.105, 0.095, 0.083, 0.069, 0.058, &
 &  0.150, 0.147, 0.142, 0.134, 0.122, 0.114, 0.104, 0.092, 0.078, 0.067, &
 &  0.162, 0.159, 0.153, 0.145, 0.133, 0.125, 0.114, 0.102, 0.088, 0.077, &
 &  0.180, 0.176, 0.171, 0.162, 0.149, 0.140, 0.129, 0.116, 0.102, 0.092, &
 &  0.198, 0.194, 0.188, 0.179, 0.165, 0.155, 0.144, 0.130, 0.116, 0.107, &
 &  0.219, 0.215, 0.208, 0.199, 0.184, 0.174, 0.161, 0.147, 0.133, 0.125, &
 &  0.244, 0.239, 0.232, 0.222, 0.206, 0.195, 0.182, 0.167, 0.152, 0.146, &
 &  0.273, 0.268, 0.261, 0.249, 0.232, 0.220, 0.206, 0.190, 0.175, 0.171, &
 &  0.308, 0.302, 0.294, 0.281, 0.262, 0.250, 0.235, 0.218, 0.203, 0.200, &
 &  0.349, 0.343, 0.334, 0.320, 0.299, 0.285, 0.269, 0.251, 0.235, 0.234, &
 &  0.397, 0.391, 0.381, 0.365, 0.342, 0.327, 0.309, 0.289, 0.273, 0.275, &
 &  0.454, 0.446, 0.435, 0.418, 0.392, 0.376, 0.356, 0.335, 0.318, 0.323, &
 &  0.510, 0.503, 0.490, 0.472, 0.444, 0.426, 0.405, 0.382, 0.365, 0.373, &
 &  0.542, 0.535, 0.523, 0.505, 0.478, 0.460, 0.440, 0.418, 0.403, 0.413, &
 &  0.580, 0.572, 0.561, 0.543, 0.517, 0.500, 0.481, 0.461, 0.447, 0.460, &
 &  0.623, 0.616, 0.605, 0.588, 0.563, 0.547, 0.529, 0.510, 0.499, 0.515, &
 &  0.674, 0.668, 0.657, 0.641, 0.617, 0.602, 0.585, 0.568, 0.560, 0.582, &
 &  0.734, 0.728, 0.718, 0.703, 0.681, 0.667, 0.651, 0.636, 0.631, 0.660, &
 &  0.809, 0.804, 0.795, 0.781, 0.761, 0.748, 0.735, 0.723, 0.723, 0.760, &
 &  0.888, 0.883, 0.874, 0.862, 0.845, 0.835, 0.823, 0.815, 0.821, 0.871, &
 &  0.941, 0.937, 0.931, 0.923, 0.910, 0.903, 0.896, 0.893, 0.907, 0.972, &
 &  1.000, 0.998, 0.994, 0.989, 0.981, 0.978, 0.975, 0.979, 1.000, 1.079, &
 &  1.030, 1.028, 1.026, 1.022, 1.018, 1.016, 1.016, 1.023, 1.048, 1.136, &
 &  1.065, 1.065, 1.063, 1.061, 1.060, 1.060, 1.062, 1.072, 1.102, 1.197, &
 &  1.095, 1.095, 1.095, 1.095, 1.096, 1.099, 1.104, 1.117, 1.151, 1.255, &
 &  1.122, 1.123, 1.124, 1.126, 1.131, 1.136, 1.144, 1.161, 1.201, 1.318, &
 &  1.147, 1.148, 1.151, 1.155, 1.163, 1.170, 1.182, 1.203, 1.250, 1.378, &
 &  1.176, 1.179, 1.183, 1.190, 1.201, 1.211, 1.225, 1.252, 1.304, 1.444, &
 &  1.189, 1.192, 1.197, 1.205, 1.218, 1.229, 1.245, 1.274, 1.328, 1.470, &
 &  1.204, 1.207, 1.213, 1.222, 1.237, 1.249, 1.267, 1.297, 1.354, 1.498, &
 &  1.220, 1.224, 1.230, 1.240, 1.257, 1.271, 1.290, 1.322, 1.381, 1.526, &
 &  1.237, 1.241, 1.249, 1.260, 1.279, 1.294, 1.315, 1.349, 1.409, 1.552, &
 &  1.248, 1.253, 1.261, 1.273, 1.294, 1.309, 1.332, 1.367, 1.429, 1.567, &
 &  1.261, 1.266, 1.275, 1.288, 1.310, 1.327, 1.350, 1.387, 1.449, 1.581, &
 &  1.276, 1.281, 1.291, 1.305, 1.329, 1.347, 1.372, 1.410, 1.471, 1.584, &
 &  1.293, 1.300, 1.310, 1.326, 1.352, 1.372, 1.398, 1.436, 1.489, 1.554, &
 &  1.309, 1.316, 1.327, 1.345, 1.373, 1.393, 1.420, 1.454, 1.489, 1.472, &
 &  1.327, 1.334, 1.346, 1.364, 1.393, 1.413, 1.438, 1.463, 1.471, 1.359, &
 &  1.345, 1.353, 1.365, 1.384, 1.412, 1.430, 1.450, 1.462, 1.437, 1.232, &
 &  1.365, 1.373, 1.385, 1.403, 1.429, 1.443, 1.455, 1.451, 1.388, 1.097, &
 &  1.379, 1.387, 1.398, 1.414, 1.435, 1.443, 1.445, 1.420, 1.316, 0.954, &
 &  1.394, 1.401, 1.410, 1.423, 1.435, 1.435, 1.423, 1.372, 1.225, 0.811, &
 &  1.408, 1.413, 1.419, 1.426, 1.425, 1.412, 1.380, 1.298, 1.108, 0.665, &
 &  1.422, 1.425, 1.430, 1.433, 1.424, 1.404, 1.361, 1.262, 1.050, 0.597 /
 
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CO2_TCMR_TL',0,ZHOOK_HANDLE)

ZAK=reshape(ZDUM,(/10,60/))

ZMMR2VMR=RMD/RMCO2

IF(NGHG > 0) THEN
  DO JLEN=1,KLEN
    DO JF=1,NGEMS
      IF(YDGEMSPROF5(JLEN,JF)%IGRIB == 210061) THEN ! CO2 mixing ratio and pert
        ZCO2MR5(JLEN,:)=YDGEMSPROF5(JLEN,JF)%MODELPROF(:)
        ZCO2MR(JLEN,:)=YDGEMSPROF(JLEN,JF)%MODELPROF(:)
      ENDIF
    ENDDO ! loop over GEMS species
  ENDDO
ENDIF



! initialise perturbations to zero 
PCO2MEAN(:,:) = 0.0_JPRB 
! initialise trajectory 
ZCO2MEAN5(:,:) = 0.0_JPRB 

DO JJ=1,KDCOUNT

  DO JL=1,KLEN

    ZDPSUM5=0.0_JPRB
    ZDPSUM =0.0_JPRB

    IF(KCOUNT(JL) == 0) THEN
      INDEX=5
    ELSE
      DO J=1,9
        IF(PSOLEL(JL) <= ZAK_SELEL(J) .AND. PSOLEL(JL) > ZAK_SELEL(J+1)) INDEX=J
      ENDDO
      IF(PSOLEL(JL) >  ZAK_SELEL(1))  INDEX=1
      IF(PSOLEL(JL) <= ZAK_SELEL(10)) INDEX=10
    ENDIF
    
    CALL INTAVG(ZAK_PRES,ZAK(INDEX,:),60,60,1,NFLEVG,PPRESF5(JL,:)/100.0,ZAK_MODEL)
    
    DO JK=1,NFLEVG

      ZDP5=PPRESH5(JL,JK)-PPRESH5(JL,JK-1)
      ZDP =PPRESH(JL,JK) -PPRESH(JL,JK-1)

      ZCO2MEAN5(JL,JJ)=ZCO2MEAN5(JL,JJ)+ (ZAK_MODEL(JK)*ZCO2MR5(JL,JK)*ZMMR2VMR+ &
            & (1.0_JPRB-ZAK_MODEL(JK))*370.0E-6_JPRB)*(ZDP5/RG)
      PCO2MEAN(JL,JJ) =PCO2MEAN(JL,JJ) + ZAK_MODEL(JK)*ZCO2MR(JL,JK)*(ZDP5/RG)*ZMMR2VMR + &
            & (ZAK_MODEL(JK)*ZCO2MR5(JL,JK)*ZMMR2VMR+(1.0_JPRB-ZAK_MODEL(JK))*370.0E-6_JPRB)*(ZDP/RG)

      ZDPSUM5=ZDPSUM5 + (ZDP5/RG)
      ZDPSUM =ZDPSUM  + (ZDP/RG)

    ENDDO ! loop over model vertical levels

    ZCO2MEAN5(JL,JJ)=ZCO2MEAN5(JL,JJ)/ZDPSUM5
    PCO2MEAN(JL,JJ) =PCO2MEAN(JL,JJ)/ZDPSUM5 - ZDPSUM * ZCO2MEAN5(JL,JJ)/ZDPSUM5**2
      
  ENDDO ! loop over horizontal observational points
ENDDO ! loop over channels

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CO2_TCMR_TL',1,ZHOOK_HANDLE)
END SUBROUTINE CO2_TCMR_TL
