SUBROUTINE COD_OP &
 & ( KDLEN  , KLEN  , KMXCOUNT, & 
 &   PQF, PTF, PPRESH, PPRESF, &
 &   PLSF, PLWCF, PIWCF, &  
 &   PCOD, PTAUC )

!*** *COD_OP* Operator routine for cloud optical depth observations at 550 nm. 

!**   INTERFACE.
!     ----------
!          *COD_OP* IS CALLED FROM *HOP*.
!     CALL COD_OP(KDLEN,ILEN,IMXCOUNT,ZQF5, ZTF5, ZPRESH5, ZPRESF5, ZLS5, ZCLWF5,ZCIWF5,ZXPP)
!     WHERE KDLEN      = First dimension: Max number of observations, this set (INPUT)
!           KLEN       = Number of aerosol optical depth obs (INPUT)
!           IMXCOUNT   = Number of channels (INPUT)
!           ZQF5       =  Specific humidity at observation points, model levels (INPUT)
!           ZTF5       =  Temperature at observation points, model levels (INPUT)
!           ZPRESH5,ZPRESF5 =   Half/Full level pressure values at obs points, model levels (INPUT)
!           ZLS5       = land-sea mask 
!           ZCLWF5     = cloud liquid water (INPUT)
!           ZCIWF5     = cloud ice water (INPUT)
!           ZXPP       = TOTAL Cloud optical depth at 550 nm (OUTPUT)

!**   INTERFACE.
!     ----------
!          *COD_OP* IS CALLED FROM *HOP*.

!     AUTHOR.
!     -------
!        ANGELA BENEDETTI  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        20060303 - adaptation of Jean-Jacques Morcrette's cloprop550.F90
!        20071114 - output of PTAUC for each layer, naj
!        20090428 - J. Flemming: Add default formulation NRADIP=3

!     PURPOSE.
!     --------
!     Process cloud optical thickness at 0.55 um from mass mixing ratio
!     of liquid and ice water

!-----------------------------------------------------------------------
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST    ,ONLY : RD, RG, RPI, RTT
USE YOECLOP550,ONLY : RSA55, RSB55, RFA55
USE YOERAD    ,ONLY : RCCNLND, RCCNSEA, RRe2De , RMINICE
USE YOERDU    ,ONLY : REPLOG, REPSCW
USE YOETHF   , ONLY : RTICE
USE YOMLUN    ,ONLY : NULERR
USE YOMDIM    ,ONLY : NFLEVG

IMPLICIT NONE
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN, KLEN, KMXCOUNT

REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH(KDLEN,0:NFLEVG) ! PRESSURE ON HALF LEVELS FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF(KDLEN,NFLEVG)   ! PRESSURE ON FULL LEVELS FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF(KDLEN,0:NFLEVG)    ! temperature FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF(KDLEN,0:NFLEVG)    ! specific humidity FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSF(KDLEN)             ! Land-sea mask 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLWCF(KDLEN,0:NFLEVG)  ! liquid water content FROM TRAJ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIWCF(KDLEN,0:NFLEVG)  ! ice water content FROM TRAJ


REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOD(KDLEN,KMXCOUNT)   ! CLOUD OPT DEPTH 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTAUC(KDLEN,NFLEVG,KMXCOUNT) ! CLOUD OPT DEPTH per layer

!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JK, JL, JJ

REAL(KIND=JPRB) :: ZDESR(KDLEN), ZIWC(KDLEN), ZLWC(KDLEN)
REAL(KIND=JPRB) :: ZRADIP(KDLEN), ZRADLP(KDLEN)
REAL(KIND=JPRB) :: ZD(KDLEN), ZNTOT(KDLEN), ZDEN(KDLEN)
REAL(KIND=JPRB) :: ZTAUC(KDLEN,NFLEVG)

REAL(KIND=JPRB) :: Z1RADI, ZALND, ZASEA, ZBETAI, ZDefRe, &
                  & ZDPOG, ZNUM,  &
                  & ZPODT, ZQIP, ZQLP, ZRefDe, ZTEMPC, ZTOI, ZTOL
REAL(KIND=JPRB) :: ZRGI, ZRDI
REAL(KIND=JPRB) :: ZTCELS, ZAIWC, ZBIWC, ZFSR 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('COD_OP',0,ZHOOK_HANDLE)

!-- units:
!   ------
! PLWCF  is the liquid water mass mixing ratio      kg kg-1
! PIWCF  is the ice water ,ass mixing ratio         kg kg-1

ZASEA=RCCNSEA
ZALND=RCCNLND
ZRefDe = RRe2De
ZDefRe = 1.0_JPRB / ZRefDe
ZRGI = 1.0_JPRB/RG
ZRDI = 1.0_JPRB/RD

PCOD(:,:) = 0._JPRB

IF(KMXCOUNT /= 1) THEN
 WRITE(NULERR,*) 'COD_OP: ROUTINE CANNOT HANDLE MORE THAN WAVELENGHT'
 WRITE(NULERR,*) 'PROGRAM WILL STOP'
 CALL ABOR1('')
ENDIF

DO JL=1,KLEN
!  IF (PLSF(JL) < 0.5_JPRB) THEN
    ZD(JL)=0.33_JPRB
    ZNTOT(JL)=-1.15E-03_JPRB*ZASEA*ZASEA+0.963_JPRB*ZASEA+5.30_JPRB
!  ELSE
!    ZD(JL)=0.43_JPRB
!    ZNTOT(JL)=-2.10E-04_JPRB*ZALND*ZALND+0.568_JPRB*ZALND-27.9_JPRB
!  ENDIF
  ZDEN(JL)=4.0_JPRB*RPI*ZNTOT(JL)*(1.0_JPRB+ZD(JL)*ZD(JL))**3
ENDDO

DO JJ=1,KMXCOUNT
  DO JK=1,NFLEVG
    DO JL=1,KLEN
      ZDPOG=ZRGI*(PPRESH(JL,JK)-PPRESH(JL,JK-1))

!-- cloud water path in g m-2
      ZQIP=MAX(0._JPRB,1.E+03_JPRB*ZDPOG*PIWCF(JL,JK))
      ZQLP=MAX(0._JPRB,1.E+03_JPRB*ZDPOG*PLWCF(JL,JK))
      ZPODT=ZRDI*PPRESF(JL,JK)/PTF(JL,JK)
     ZIWC(JL)=1.E+03_JPRB*PIWCF(JL,JK)*ZPODT
     ZLWC(JL)=1.E+03_JPRB*PLWCF(JL,JK)*ZPODT


!-- droplet effective radius

      ZNUM=3.0_JPRB*ZLWC(JL)*(1.0_JPRB+3.0_JPRB*ZD(JL)*ZD(JL))**2
      IF((ZNUM/ZDEN(JL)) > REPLOG)THEN
        ZRADLP(JL)=100.0_JPRB*EXP(0.333_JPRB*LOG(ZNUM/ZDEN(JL)))
        ZRADLP(JL)=MAX(ZRADLP(JL), 4.0_JPRB)
        ZRADLP(JL)=MIN(ZRADLP(JL),16.0_JPRB)
      ELSE
        ZRADLP(JL)=4.0_JPRB
      ENDIF

!-- ice particle effective size

      IF (PTF(JL,JK) < RTICE) THEN
        ZTEMPC=PTF(JL,JK)-RTT
      ELSE
        ZTEMPC=RTICE-RTT
      ENDIF
      ZRADIP(JL)=326.3_JPRB+ZTEMPC*(12.42_JPRB + ZTEMPC*(0.197_JPRB + ZTEMPC*&
       & 0.0012_JPRB))
      ZRADIP(JL)=MAX(ZRADIP(JL),30.0_JPRB)
      ZRADIP(JL)=MIN(ZRADIP(JL),60.0_JPRB)
      ZDESR(JL)= ZDefRe * ZRADIP(JL)

!- default formulation NRADIP=3
      ZTEMPC=PTF(JL,JK)-83.15_JPRB
      ZTCELS=PTF(JL,JK)-RTT
      ZFSR=1.2351_JPRB+0.0105_JPRB*ZTCELS
      ZAIWC=45.8966_JPRB*ZIWC(JL)**0.2214_JPRB
      ZBIWC=0.7957_JPRB *ZIWC(JL)**0.2535_JPRB
      ZDESR(JL)=ZFSR*(ZAIWC+ZBIWC*ZTEMPC)
      ZDESR(JL)=MIN(MAX(ZDESR(JL),RMINICE), 155.0_JPRB)

!-- cloud optical properties

      ZTOL=0.0_JPRB
      ZTOI=0.0_JPRB

!-- for liquid water clouds (Slingo, 1989)

      IF (ZQLP > REPSCW) THEN
        ZTOL = ZQLP *(RSA55 + RSB55 /ZRADLP(JL))
      ENDIF

!-- for ice water clouds (Fu, 1996)

      IF (ZQIP > REPSCW) THEN
        Z1RADI = 1.0_JPRB / ZDESR(JL)
        ZBETAI = RFA55(1) + Z1RADI * RFA55(2)
        ZTOI = ZQIP * ZBETAI
      ENDIF

      ZTAUC(JL,JK)=ZTOI+ZTOL
      PCOD(JL,JJ)=PCOD(JL,JJ)+ ZTAUC(JL,JK)
      PTAUC(JL,JK,JJ)=PCOD(JL,JJ)

   ENDDO
 ENDDO
ENDDO

! BEN - add check for optical depth above 100.
DO JJ=1,KMXCOUNT
  DO JL=1,KLEN
   IF(PCOD(JL,JJ) > 100._JPRB ) THEN
    PCOD(JL,JJ) = 100.0_JPRB
   ENDIF
  ENDDO
ENDDO



!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('COD_OP',1,ZHOOK_HANDLE)
END SUBROUTINE COD_OP

