SUBROUTINE COD_OPAD( KDLEN  , KLEN   , KMXCOUNT,& 
 &   PQF   , PTF   , PPRESH , PPRESF ,&
 &   PLWCF , PIWCF ,&
 &   PQF5  , PTF5  , PPRESH5, PPRESF5, PLSF5   ,& 
 &   PLWCF5, PIWCF5,& 
 &   PCOD,PTAUC)

!*** *COD_OPAD* Adjoint Operator routine for aerosol optical depth observations

!**   INTERFACE.
!     ----------
!          *COD_OPAD* IS CALLED FROM *HOPAD*.
!     CALL COD_OPAD(KDLEN,ILEN,IMXCOUNT,ZQF, ZTF, ZPRESH, ZPRESF,&
!      &  ZCLWF,ZCIWF, &
!      &  ZQF5, ZTF5, ZPRESH5, ZPRESF5, ZLS5, ZCLWF5, ZCIWF5, ZXPP)
!*
!    INPUT PARAMETERS (INTEGER):
!      KDLEN      = First dimension: Max number of observations, this set
!                       (INPUT)
!      KLEN       = Number of aerosol optical depth obs (INPUT)
!      IMXCOUNT   = Number of channels (INPUT)
!*
!    INPUT PARAMETERS (REAL):
!      Trajectory  Perturbation  Description
!      PQF5        PQF           Specific humidity at observation points,
!                                model levels 
!      PTF5        PTF           Temperature at obs points, model levels
!      PPRESH5     PPRESH        Half level pressure values at obs points
!      PPRESF5     PPRESF        Full level pressure values at obs points
!      PLS5        ---           Land-sea mask
!      PLWCF5      PLWCF         Liquid water content
!      PIWCF5      PIWCF         Ice water content
!*
!    OUTPUT PARAMETERS (REAL):
!      ---         PCOD          Aerosol optical depth at different 
!                                wavelengths 
!      ---         PTAUC         Cloud optical depth per layer

!     AUTHOR.
!     -------
!      Marta Janiskova       ECMWF     4 April 2006

!     MODIFICATIONS.
!     --------------
!      20090429  Antje Inness: Add default formulation NRADIP=3
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST    ,ONLY : RD, RG, RPI, RTT
USE YOECLOP550,ONLY : RSA55, RSB55, RFA55
USE YOERAD    ,ONLY : RCCNLND, RCCNSEA, RRe2De, RMINICE
USE YOERDU    ,ONLY : REPLOG, REPSCW
USE YOETHF   , ONLY : RTICE
USE YOMLUN    ,ONLY : NULERR
USE YOMDIM    ,ONLY : NFLEVG

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN, KLEN, KMXCOUNT

REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF5(KDLEN,NFLEVG)   
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSF5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLWCF5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIWCF5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPRESH(KDLEN,0:NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPRESF(KDLEN,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF(KDLEN,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLWCF(KDLEN,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PIWCF(KDLEN,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOD(KDLEN,KMXCOUNT)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTAUC(KDLEN,NFLEVG,KMXCOUNT)

!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JK, JL, JJ

REAL(KIND=JPRB) :: ZDESR(KDLEN), ZIWC(KDLEN), ZLWC(KDLEN)
REAL(KIND=JPRB) :: ZRADIP(KDLEN), ZRADLP(KDLEN)
REAL(KIND=JPRB) :: ZD(KDLEN), ZNTOT(KDLEN), ZDEN(KDLEN)
REAL(KIND=JPRB) :: ZTAUC(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZTAUC5(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZIWC5(KDLEN,NFLEVG), ZLWC5(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZRADLP5(KDLEN,NFLEVG), ZRADIP5(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZDESR5(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZRADLP55(KDLEN,NFLEVG), ZRADIP55(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZDPOG5(KDLEN,NFLEVG), ZPODT5(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZNUM5(KDLEN,NFLEVG), ZTEMPC5(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZTOI5(KDLEN,NFLEVG), ZTOL5(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: Z1RADI5(KDLEN,NFLEVG), ZBETAI5(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZQIP5(KDLEN,NFLEVG), ZQLP5(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZQIP15(KDLEN,NFLEVG), ZQLP15(KDLEN,NFLEVG)
REAL(KIND=JPRB) :: ZCOD5(KDLEN,KMXCOUNT)

REAL(KIND=JPRB) :: Z1RADI, ZALND, ZASEA, ZBETAI, ZDefRe, &
                  & ZDPOG, ZNUM, &
                  & ZPODT, ZQIP, ZQLP, ZRefDe, ZTEMPC, ZTOI, ZTOL
REAL(KIND=JPRB) :: ZQIP1, ZQLP1
REAL(KIND=JPRB) :: ZRGI, ZRDI
REAL(KIND=JPRB) :: ZTCELS, ZAIWC, ZBIWC, ZFSR
REAL(KIND=JPRB) :: ZTCELS5(KDLEN,NFLEVG),ZAIWC5(KDLEN,NFLEVG), &
                  & ZBIWC5(KDLEN,NFLEVG), ZFSR5(KDLEN,NFLEVG)


REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('COD_OPAD',0,ZHOOK_HANDLE)

!-- units:
!   ------
! PLWCF  is the liquid water mass mixing ratio      kg kg-1
! PIWCF  is the ice water ,ass mixing ratio         kg kg-1

ZASEA=RCCNSEA
ZALND=RCCNLND
ZRefDe = RRe2De
ZDefRe = 1.0_JPRB / ZRefDe
ZRGI = 1.0_JPRB/RG
ZRDI = 1.0_JPRB/RD

ZCOD5(:,:)= 0.0_JPRB

IF(KMXCOUNT /= 1) THEN
 WRITE(NULERR,*) 'COD_OPAD: ROUTINE CANNOT HANDLE MORE THAN WAVELENGHT'
 WRITE(NULERR,*) 'PROGRAM WILL STOP'
 CALL ABOR1('')
ENDIF

DO JL=1,KLEN
!  IF (PLSF5(JL) < 0.5_JPRB) THEN
    ZD(JL)=0.33_JPRB
    ZNTOT(JL)=-1.15E-03_JPRB*ZASEA*ZASEA+0.963_JPRB*ZASEA+5.30_JPRB
!  ELSE
!    ZD(JL)=0.43_JPRB
!    ZNTOT(JL)=-2.10E-04_JPRB*ZALND*ZALND+0.568_JPRB*ZALND-27.9_JPRB
!  ENDIF
  ZDEN(JL)=4.0_JPRB*RPI*ZNTOT(JL)*(1.0_JPRB+ZD(JL)*ZD(JL))**3
ENDDO

DO JJ=1,KMXCOUNT
  DO JK=1,NFLEVG
    DO JL=1,KLEN

      ZDPOG5(JL,JK) = ZRGI*(PPRESH5(JL,JK)-PPRESH5(JL,JK-1))

!-- cloud water path in g m-2

      ZQIP15(JL,JK) = 1.E+03_JPRB*ZDPOG5(JL,JK)*PIWCF5(JL,JK)
      IF (ZQIP15(JL,JK) > 0._JPRB) THEN
        ZQIP5(JL,JK) = ZQIP15(JL,JK)
      ELSE
        ZQIP5(JL,JK) = 0._JPRB
      ENDIF
      ZQLP15(JL,JK) = 1.E+03_JPRB*ZDPOG5(JL,JK)*PLWCF5(JL,JK)
      IF (ZQLP15(JL,JK) > 0._JPRB) THEN
        ZQLP5(JL,JK) = ZQLP15(JL,JK)
      ELSE
        ZQLP5(JL,JK) = 0._JPRB
      ENDIF
      ZPODT5(JL,JK) = ZRDI*PPRESF5(JL,JK)/PTF5(JL,JK)
      ZIWC5(JL,JK)=1.E+03_JPRB*PIWCF5(JL,JK)*ZPODT5(JL,JK)
      ZLWC5(JL,JK)=1.E+03_JPRB*PLWCF5(JL,JK)*ZPODT5(JL,JK)

!-- droplet effective radius

      ZNUM5(JL,JK) = 3.0_JPRB*ZLWC5(JL,JK) &
       & *(1.0_JPRB+3.0_JPRB*ZD(JL)*ZD(JL))**2
      IF ((ZNUM5(JL,JK)/ZDEN(JL)) > REPLOG)THEN
        ZRADLP5(JL,JK) = 100.0_JPRB &
         & *EXP(0.333_JPRB*LOG(ZNUM5(JL,JK)/ZDEN(JL)))
        ZRADLP55(JL,JK) = ZRADLP5(JL,JK)
        IF (ZRADLP5(JL,JK) < 4.0_JPRB) THEN
          ZRADLP5(JL,JK) = 4.0_JPRB
        ENDIF
        IF (ZRADLP5(JL,JK) > 16.0_JPRB) THEN
          ZRADLP5(JL,JK) = 16.0_JPRB
        ENDIF
      ELSE
        ZRADLP5(JL,JK) = 4.0_JPRB
      ENDIF

!-- ice particle effective size

      IF (PTF5(JL,JK) < RTICE) THEN
        ZTEMPC5(JL,JK) = PTF5(JL,JK)-RTT
      ELSE
        ZTEMPC5(JL,JK) = RTICE-RTT
      ENDIF

      ZRADIP5(JL,JK) = 326.3_JPRB+ZTEMPC5(JL,JK)*(12.42_JPRB+ZTEMPC5(JL,JK) &
       & *(0.197_JPRB + ZTEMPC5(JL,JK)*0.0012_JPRB))
      ZRADIP55(JL,JK) = ZRADIP5(JL,JK)
      IF (ZRADIP5(JL,JK) < 30.0_JPRB) THEN
        ZRADIP5(JL,JK) = 30.0_JPRB
      ENDIF
      IF (ZRADIP5(JL,JK) > 60.0_JPRB) THEN
        ZRADIP5(JL,JK) = 60.0_JPRB
      ENDIF
      ZDESR5(JL,JK) = ZDefRe * ZRADIP5(JL,JK)

!- default formulation NRADIP=3
      ZTEMPC5(JL,JK)=PTF5(JL,JK)-83.15_JPRB
      ZTCELS5(JL,JK)=PTF5(JL,JK)-RTT
      ZFSR5(JL,JK)=1.2351_JPRB+0.0105_JPRB*ZTCELS5(JL,JK)
      ZAIWC5(JL,JK)=45.8966_JPRB*ZIWC5(JL,JK)**0.2214_JPRB
      ZBIWC5(JL,JK)=0.7957_JPRB *ZIWC5(JL,JK)**0.2535_JPRB
      ZDESR5(JL,JK)=ZFSR5(JL,JK)*(ZAIWC5(JL,JK)+ZBIWC5(JL,JK)*ZTEMPC5(JL,JK))
      IF(ZDESR5(JL,JK) > RMINICE) THEN
        IF(ZDESR5(JL,JK) < 155.0_JPRB) THEN
          ZDESR5(JL,JK)=ZDESR5(JL,JK)
        ELSE
          ZDESR5(JL,JK)=155.0_JPRB
        ENDIF
      ELSE
        IF(RMINICE < 155.0_JPRB) THEN
          ZDESR5(JL,JK)=RMINICE
        ELSE
          ZDESR5(JL,JK)=155.0_JPRB
        ENDIF
      ENDIF

!-- cloud optical properties

      ZTOL5(JL,JK) = 0.0_JPRB
      ZTOI5(JL,JK) = 0.0_JPRB

!-- for liquid water clouds (Slingo, 1989)

      IF (ZQLP5(JL,JK) > REPSCW) THEN
        ZTOL5(JL,JK) = ZQLP5(JL,JK) *(RSA55 + RSB55 /ZRADLP5(JL,JK))
      ENDIF

!-- for ice water clouds (Fu, 1996)

      IF (ZQIP5(JL,JK) > REPSCW) THEN
        Z1RADI5(JL,JK) = 1.0_JPRB / ZDESR5(JL,JK)
        ZBETAI5(JL,JK) = RFA55(1) + Z1RADI5(JL,JK) * RFA55(2)
        ZTOI5(JL,JK) = ZQIP5(JL,JK)*ZBETAI5(JL,JK)
      ENDIF

       ZTAUC5(JL,JK) = ZTOI5(JL,JK)+ZTOL5(JL,JK)
       ZCOD5(JL,JJ) = ZCOD5(JL,JJ)+ZTAUC5(JL,JK)

    ENDDO
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!                 ----- ADJOINT COMPUTATION ------

!     ------------------------------------------------------------------

! Initialization

ZDESR (:) = 0.0_JPRB
ZIWC  (:) = 0.0_JPRB
ZLWC  (:) = 0.0_JPRB
ZRADIP(:) = 0.0_JPRB
ZRADLP(:) = 0.0_JPRB
ZTCELS = 0.0_JPRB
ZAIWC = 0.0_JPRB
ZBIWC = 0.0_JPRB
ZFSR = 0.0_JPRB

ZTAUC(:,:) = 0.0_JPRB

! BEN - add check for optical depth above 100.
DO JJ=1,KMXCOUNT
  DO JL=1,KLEN
   IF(ZCOD5(JL,JJ) > 100._JPRB ) THEN
    PCOD(JL,JJ) = 0.0_JPRB
   ENDIF
  ENDDO
ENDDO

DO JJ=1,KMXCOUNT
  DO JK=NFLEVG,1,-1
    DO JL=1,KLEN

      ZDPOG = 0.0_JPRB
      ZQIP1 = 0.0_JPRB
      ZQIP  = 0.0_JPRB
      ZQLP1 = 0.0_JPRB
      ZQLP  = 0.0_JPRB
      ZPODT = 0.0_JPRB
      ZNUM  = 0.0_JPRB
      ZTEMPC= 0.0_JPRB
      ZTOL  = 0.0_JPRB
      ZTOI  = 0.0_JPRB
      Z1RADI= 0.0_JPRB
      ZBETAI= 0.0_JPRB

      PCOD(JL,JJ)=PCOD(JL,JJ)+PTAUC(JL,JK,JJ)
      PTAUC(JL,JK,JJ)=0.0_JPRB   
    
      ZTAUC(JL,JK) = ZTAUC(JL,JK)+PCOD(JL,JJ)
      ZTOI = ZTOI+ZTAUC(JL,JK)
      ZTOL = ZTOL+ZTAUC(JL,JK)
      ZTAUC(JL,JK) = 0.0_JPRB

!-- cloud optical properties

!-- for ice water clouds (Fu, 1996)

      IF (ZQIP5(JL,JK) > REPSCW) THEN
        ZBETAI = ZBETAI+ZQIP5(JL,JK)*ZTOI
        ZQIP = ZQIP+ZBETAI5(JL,JK)*ZTOI
        Z1RADI = Z1RADI+RFA55(2)*ZBETAI
        ZDESR(JL) = ZDESR(JL)-Z1RADI*Z1RADI5(JL,JK)*Z1RADI5(JL,JK)
      ENDIF

!-- for liquid water clouds (Slingo, 1989)

      IF (ZQLP5(JL,JK) > REPSCW) THEN
        ZQLP = ZQLP+(RSA55 + RSB55 /ZRADLP5(JL,JK))*ZTOL
        ZRADLP(JL) = ZRADLP(JL)-RSB55*ZQLP5(JL,JK)*ZTOL/ZRADLP5(JL,JK)**2
      ENDIF

!- default formulation NRADIP=3
      IF(ZDESR5(JL,JK) < RMINICE .OR. ZDESR5(JL,JK) > 155.0_JPRB) THEN
        ZDESR(JL)=0.0_JPRB
      ENDIF
      ZFSR=ZFSR+ZDESR(JL)*(ZAIWC5(JL,JK)+ZBIWC5(JL,JK)*ZTEMPC5(JL,JK))
      ZAIWC=ZAIWC+ZDESR(JL)*ZFSR5(JL,JK)
      ZTEMPC=ZTEMPC+ZDESR(JL)*ZFSR5(JL,JK)*ZBIWC5(JL,JK)
      ZBIWC=ZBIWC+ZDESR(JL)*ZFSR5(JL,JK)*ZTEMPC5(JL,JK)
      ZDESR(JL)=0.0_JPRB
      ZIWC(JL)=ZIWC(JL)+ZBIWC*0.7957_JPRB*0.2535_JPRB*ZIWC5(JL,JK)**(0.2535_JPRB-1.0_JPRB)
      ZBIWC=0.0_JPRB
      ZIWC(JL)=ZIWC(JL)+ZAIWC*45.8966_JPRB*0.2214_JPRB*ZIWC5(JL,JK)**(0.2214_JPRB-1.0_JPRB)
      ZAIWC=0.0_JPRB
      ZTCELS=ZTCELS+0.0105_JPRB*ZFSR
      ZFSR=0.0_JPRB
      PTF(JL,JK)=PTF(JL,JK)+ZTCELS
      ZTCELS=0.0_JPRB
      PTF(JL,JK)=PTF(JL,JK)+ZTEMPC
      ZTEMPC=0
      
!-- ice particle effective size

      ZRADIP(JL) = ZRADIP(JL)+ZDefRe*ZDESR (JL)
      ZDESR (JL) = 0.0_JPRB
      IF (ZRADIP55(JL,JK)>60.0_JPRB .OR. ZRADIP55(JL,JK)<30.0_JPRB) THEN
        ZRADIP(JL) = 0.0_JPRB
      ENDIF
      ZTEMPC = ZTEMPC+ZRADIP(JL)*(12.42_JPRB+ZTEMPC5(JL,JK) &
       & *(2*0.197_JPRB+3*0.0012_JPRB*ZTEMPC5(JL,JK)))
      ZRADIP(JL) = 0.0_JPRB
      IF (PTF5(JL,JK) < RTICE) THEN
        PTF(JL,JK) = PTF(JL,JK)+ZTEMPC
      ENDIF

!-- droplet effective radius

      IF ((ZNUM5(JL,JK)/ZDEN(JL)) > REPLOG) THEN
        IF (ZRADLP55(JL,JK)>16.0_JPRB .OR. ZRADLP55(JL,JK)<4.0_JPRB) THEN
          ZRADLP (JL) = 0.0_JPRB
        ENDIF
        ZNUM = ZNUM+0.333_JPRB*ZRADLP55(JL,JK)*ZRADLP(JL)/ZNUM5(JL,JK)
      ENDIF
      ZRADLP(JL) = 0.0_JPRB
      ZLWC(JL) = ZLWC(JL) &
       & +3.0_JPRB*ZNUM*(1.0_JPRB+3.0_JPRB*ZD(JL)*ZD(JL))**2

!-- cloud water path in g m-2

      ZPODT=ZPODT+1.E+03_JPRB*PLWCF5(JL,JK)*ZLWC(JL)
      PLWCF(JL,JK)=PLWCF(JL,JK)+1.E+03_JPRB*ZPODT5(JL,JK)*ZLWC(JL)
      ZLWC(JL) = 0.0_JPRB
      ZPODT=ZPODT+1.E+03_JPRB*PIWCF5(JL,JK)*ZIWC(JL)
      PIWCF(JL,JK)=PIWCF(JL,JK)+1.E+03_JPRB*ZPODT5(JL,JK)*ZIWC(JL)
      ZIWC(JL) = 0.0_JPRB
      PPRESF(JL,JK) = PPRESF(JL,JK)+ZRDI*ZPODT/PTF5(JL,JK)
      PTF(JL,JK) = PTF(JL,JK)-ZRDI*PPRESF5(JL,JK)*ZPODT/PTF5(JL,JK)**2

      IF (ZQLP15(JL,JK) > 0._JPRB) THEN
        ZQLP1 = ZQLP1+ZQLP
      ENDIF
      PLWCF(JL,JK) = PLWCF(JL,JK)+1.E+03_JPRB*ZDPOG5(JL,JK)*ZQLP1
      ZDPOG = ZDPOG+1.E+03_JPRB*PLWCF5(JL,JK)*ZQLP1

      IF (ZQIP15(JL,JK) > 0._JPRB) THEN
        ZQIP1 = ZQIP1+ZQIP
      ENDIF
      PIWCF(JL,JK) = PIWCF(JL,JK)+1.E+03_JPRB*ZDPOG5(JL,JK)*ZQIP1
      ZDPOG = ZDPOG+1.E+03_JPRB*PIWCF5(JL,JK)*ZQIP1

      PPRESH(JL,JK) = PPRESH(JL,JK)+ZRGI*ZDPOG
      PPRESH(JL,JK-1) = PPRESH(JL,JK-1)-ZRGI*ZDPOG
    ENDDO
  ENDDO
ENDDO

PCOD(:,:) = 0.0_JPRB

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('COD_OPAD',1,ZHOOK_HANDLE)
END SUBROUTINE COD_OPAD
