!  Differentiation of discom_gems in reverse (adjoint) mode:
!   gradient, with respect to input variables: pomega ptaer phasef
!   of linear combination of output variables: pomega pdtdif proatm
!                pdtdir ptaer phasef psphal putdif putdir

SUBROUTINE DISCOM_GEMS_AD(phirad, pxmus, pxmuv, prm, pgb, prp, prmu, pga, &
&  ptaer, ptaerb, ptray, pomega, pomegab, phasef, phasefb, ppl, ppsl, &
&  proatm, proatmb, pdtdir, pdtdirb, pdtdif, pdtdifb, putdir, putdirb, &
&  putdif, putdifb, psphal, psphalb)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE rt6svalues

IMPLICIT NONE

!****DISCOM*** COMPUTES OPTICAL PROPERTIES OF THE ATMOSPHERE AT REQUESTED WAVELENGTHS


!     INTERFACE
!     ---------
! DISCOM IS CALLED BY MAIN ROUTINE 6S_GEMS

!     INPUTS
!     ------

! PHIRAD          : DIFFERENCE IN AZIMUTAL ANGLE [RAD]
! PXMUS           : COSINE OF SOLAR ZENITH ANGLE
! PXMUV           : COSINE OF VIEWING ZENITH ANGLE
! PRP(KNP)        : GAUSSIAN ANGLES FOR AZIMUTH
! PRMU(KNBMU)     : GAUSSIAN ANGLES FOR TRUNCATION OF PHASE FUNCTION 
! PGA(KNBMU)      : GAUSSIAN WEIGHTS FOR TRUNCATION OF PHASE FUNCTION
! PRM(-KMU:KMU)   : GAUSSIAN ANGLES FOR THE ZENITH
! PGB(-KMU:KMU)   : GAUSSIAN WEIGHTS FOR THE ZENITH
! PTAER(0:KNT)      : Aerosol optical thickness profile.
! PTRAY(0:KNT)      : Molecular optical thickness profile
! PHASEF(0:KNT,KMU) : Aerosol phase function profile.
! POMEGA(0:KNT)     : Aerosol single scattering albedo profile.
! PPL(KNBMU,-1:KNANG+1) : USED IN TRUNCATION OF PHASE FUNCTION
! PPSL(0:KNANG,-1:KNANG,-KMU:KMU) :

!     OUTPUTS
!     -------

! PROATM : ATMOSPHERIC REFLECTANCE
! PDTDIR : DOWNWARD DIRECT TRANSMITTANCE
! PDTDIF : DOWNWARD DIFFUSE TRANSMITTANCE
! PUTDIR : UPWARD DIRECT TRANSMITTANCE
! PUTDIF : UPWARD DIFFUSE TRANSMITTANCE
! PSPHAL : SPHERICAL ALBEDO

!     EXTERNALS.
!     ----------
!          TRUNCA, ATMREF, SCATRA

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007

!     SOURCE
!     ------
! ADAPTED FROM DISCOM.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------
!INTEGER(KIND=JPIM),PARAMETER :: KMU=25, KNP=49, KNANG=80, KNBMU=83, KNT=20
  REAL(KIND=JPRB), INTENT(IN) :: prp(knp), prmu(knbmu), pga(knbmu)
  REAL(KIND=JPRB), INTENT(INOUT) :: prm(-kmu:kmu), pgb(-kmu:kmu)
  REAL(KIND=JPRB), INTENT(IN) :: phirad, pxmus, pxmuv
  REAL(KIND=JPRB), INTENT(IN) :: ptaer(0:knt), ptray(0:knt)
  REAL(KIND=JPRB), INTENT(INOUT) :: ptaerb(0:knt)
  REAL(KIND=JPRB), INTENT(IN) :: ppl(knbmu, -1:knang+1)
  REAL(KIND=JPRB), INTENT(INOUT) :: phasef(0:knt, knbmu)
  REAL(KIND=JPRB), INTENT(IN) :: pomega(0:knt)
  REAL(KIND=JPRB), INTENT(INOUT) :: phasefb(0:knt, knbmu), pomegab(0:knt)
  REAL(KIND=JPRB), INTENT(IN) :: ppsl(0:knang, -1:knang, -kmu:kmu)
  REAL(KIND=JPRB), INTENT(IN) :: proatm, psphal
  REAL(KIND=JPRB), INTENT(INOUT) :: proatmb, psphalb
  REAL(KIND=JPRB), INTENT(IN) :: pdtdir, pdtdif
  REAL(KIND=JPRB), INTENT(INOUT) :: pdtdirb, pdtdifb
  REAL(KIND=JPRB), INTENT(IN) :: putdir, putdif
  REAL(KIND=JPRB), INTENT(INOUT) :: putdirb, putdifb
! LOCAL VARIABLES
! ---------------
  INTEGER(KIND=JPIM) :: jlev
  REAL(KIND=JPRB) :: z_pbetal(0:knt, 0:knang), z_pcoeff(0:knt)
  REAL(KIND=JPRB) :: z_pbetalb(0:knt, 0:knang), z_pcoeffb(0:knt)
  REAL(KIND=JPRB) :: z_promix
  REAL(KIND=JPRB) :: z_promixb
  REAL(KIND=JPRB) :: z_ptamoy(0:knt), z_pizmoy(0:knt)
  REAL(KIND=JPRB) :: z_ptamoyb(0:knt), z_pizmoyb(0:knt)
  REAL(KIND=JPRB) :: z_ptodaer, z_ptodray
  REAL(KIND=JPRB) :: z_ptodaerb
  REAL(KIND=JPRB) :: z_pddirtt, z_pddiftt, z_pudirtt, z_pudiftt, z_psphalbt
  REAL(KIND=JPRB) :: z_pddirttb, z_pddifttb, z_pudirttb, z_pudifttb, z_psphalbtb
  REAL(KIND=JPRB) :: z_temp
  REAL(KIND=JPRB) :: z_temp0b
  REAL(KIND=JPRB) :: z_tempb

  REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "atmref_gems_ad.intfb.h"
#include "scatra_gems_ad.intfb.h"
#include "trunca_gems.intfb.h"
#include "trunca_gems_ad.intfb.h"
!#include "pushreal8array.intfb.h"
!#include "poreal8array.intfb.h"

  IF (LHOOK) CALL DR_HOOK('DISCOM_GEMS_AD',0,ZHOOK_HANDLE)

  CALL PUSHREAL8ARRAY(phasef, 27*83)

! THE AEROSOL PHASE FUNCTION IS DECOMPOSED IN THE BETAL

!PRINT*, 'CALLING TRUNCA_GEMS'
  CALL TRUNCA_GEMS(phasef, prmu, pga, ppl, z_pbetal, z_pcoeff)
!PRINT*, 'END TRUNCA_GEMS'

! TRUNCATION OF THE MOLECULAR AND AEROSOL PHASE FUNCTION

  DO jlev=0,knt
    z_ptamoy(jlev) = ptaer(jlev)*(1.0_JPRB-pomega(jlev)*z_pcoeff(jlev))
    z_pizmoy(jlev) = pomega(jlev)*(1.0_JPRB-z_pcoeff(jlev))/(1.0_JPRB-pomega(jlev)&
&      *z_pcoeff(jlev))
  ENDDO

! AEROSOL AND MOLECULAR OPTICAL DEPTH

  z_ptodaer = 0.0_JPRB
  z_ptodray = 0.0_JPRB
  DO jlev=0,knt
! AEROSOL OPTICAL THICKNESS
    z_ptodaer = z_ptodaer + z_ptamoy(jlev)
! MOLECULAR OPTICAL THICKNESS
    z_ptodray = z_ptodray + ptray(jlev)
  ENDDO
  z_psphalbtb = psphalb
  z_pudifttb = putdifb
  z_pudirttb = putdirb
  z_pddifttb = pdtdifb
  z_pddirttb = pdtdirb
  z_promixb = proatmb
  CALL SCATRA_GEMS_AD(z_ptamoy, z_ptamoyb, ptray, z_ptodaer, z_ptodaerb, z_ptodray, &
&               z_pizmoy, z_pizmoyb, prm, pgb, pxmus, pxmuv, z_pbetal, &
&               z_pbetalb, ppsl, z_pddirtt, z_pddirttb, z_pddiftt, z_pddifttb, &
&               z_pudirtt, z_pudirttb, z_pudiftt, z_pudifttb, z_psphalbt, &
&               z_psphalbtb)
  CALL ATMREF_GEMS_AD(z_ptamoy, z_ptamoyb, ptray, z_pizmoy, z_pizmoyb, phirad, &
&               pxmus, pxmuv, prm, pgb, prp, z_pbetal, z_pbetalb, ppsl, &
&               z_ptodaer, z_ptodray, z_promix, z_promixb)
  DO jlev=knt,0,-1
    z_ptamoyb(jlev) = z_ptamoyb(jlev) + z_ptodaerb
  ENDDO
  z_pcoeffb(0:knt) = 0.0_JPRB
  DO jlev=knt,0,-1
    z_temp = -(pomega(jlev)*z_pcoeff(jlev)) + 1.0_JPRB
    z_temp0b = z_pizmoyb(jlev)/z_temp
    z_tempb = -(pomega(jlev)*(1.0_JPRB-z_pcoeff(jlev))*z_temp0b/z_temp)
    pomegab(jlev) = pomegab(jlev) + (1.0_JPRB-z_pcoeff(jlev))*z_temp0b - z_pcoeff&
&      (jlev)*z_tempb - ptaer(jlev)*z_pcoeff(jlev)*z_ptamoyb(jlev)
    z_pcoeffb(jlev) = z_pcoeffb(jlev) - ptaer(jlev)*pomega(jlev)*z_ptamoyb(&
&      jlev) - pomega(jlev)*z_tempb - pomega(jlev)*z_temp0b
    z_pizmoyb(jlev) = 0.0_JPRB
    ptaerb(jlev) = ptaerb(jlev) + (1.0_JPRB-pomega(jlev)*z_pcoeff(jlev))*&
&      z_ptamoyb(jlev)
    z_ptamoyb(jlev) = 0.0_JPRB
  ENDDO
  CALL POPREAL8ARRAY(phasef, 27*83)
  CALL TRUNCA_GEMS_AD(phasef, phasefb, prmu, pga, ppl, z_pbetal, z_pbetalb, &
&               z_pcoeff, z_pcoeffb)

  IF (LHOOK) CALL DR_HOOK('DISCOM_GEMS_AD',1,ZHOOK_HANDLE)
END SUBROUTINE DISCOM_GEMS_AD
