!  Differentiation of discom_gems in forward (tangent) mode:
!   variations  of output variables: pdtdif proatm pdtdir phasef
!                psphal putdif putdir
!   with respect to input variables: pomega ptaer phasef

SUBROUTINE DISCOM_GEMS_TL(phirad, pxmus, pxmuv, prm, pgb, prp, prmu, pga, &
&  ptaer, ptaerd, ptray, pomega, pomegad, phasef, phasefd, ppl, ppsl, &
&  proatm, proatmd, pdtdir, pdtdird, pdtdif, pdtdifd, putdir, putdird, &
&  putdif, putdifd, psphal, psphald)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE rt6svalues

IMPLICIT NONE
!****DISCOM*** COMPUTES OPTICAL PROPERTIES OF THE ATMOSPHERE AT REQUESTED WAVELENGTHS


!     INTERFACE
!     ---------
! DISCOM IS CALLED BY MAIN ROUTINE 6S_GEMS

!     INPUTS
!     ------

! PHIRAD          : DIFFERENCE IN AZIMUTAL ANGLE [RAD]
! PXMUS           : COSINE OF SOLAR ZENITH ANGLE
! PXMUV           : COSINE OF VIEWING ZENITH ANGLE
! PRP(KNP)        : GAUSSIAN ANGLES FOR AZIMUTH
! PRMU(KNBMU)     : GAUSSIAN ANGLES FOR TRUNCATION OF PHASE FUNCTION 
! PGA(KNBMU)      : GAUSSIAN WEIGHTS FOR TRUNCATION OF PHASE FUNCTION
! PRM(-KMU:KMU)   : GAUSSIAN ANGLES FOR THE ZENITH
! PGB(-KMU:KMU)   : GAUSSIAN WEIGHTS FOR THE ZENITH
! PTAER(0:KNT)      : Aerosol optical thickness profile.
! PTRAY(0:KNT)      : Molecular optical thickness profile
! PHASEF(0:KNT,KMU) : Aerosol phase function profile.
! POMEGA(0:KNT)     : Aerosol single scattering albedo profile.
! PPL(KNBMU,-1:KNANG+1) : USED IN TRUNCATION OF PHASE FUNCTION
! PPSL(0:KNANG,-1:KNANG,-KMU:KMU) :

!     OUTPUTS
!     -------

! PROATM : ATMOSPHERIC REFLECTANCE
! PDTDIR : DOWNWARD DIRECT TRANSMITTANCE
! PDTDIF : DOWNWARD DIFFUSE TRANSMITTANCE
! PUTDIR : UPWARD DIRECT TRANSMITTANCE
! PUTDIF : UPWARD DIFFUSE TRANSMITTANCE
! PSPHAL : SPHERICAL ALBEDO

!     EXTERNALS.
!     ----------
!          TRUNCA, ATMREF, SCATRA

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007

!     SOURCE
!     ------
! ADAPTED FROM DISCOM.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------
!INTEGER(KIND=JPIM),PARAMETER :: KMU=25, KNP=49, KNANG=80, KNBMU=83, KNT=20
  REAL(KIND=JPRB), INTENT(IN) :: prp(knp), prmu(knbmu), pga(knbmu)
  REAL(KIND=JPRB), INTENT(INOUT) :: prm(-kmu:kmu), pgb(-kmu:kmu)
  REAL(KIND=JPRB), INTENT(IN) :: phirad, pxmus, pxmuv
  REAL(KIND=JPRB), INTENT(IN) :: ptaer(0:knt), ptray(0:knt)
  REAL(KIND=JPRB), INTENT(IN) :: ptaerd(0:knt)
  REAL(KIND=JPRB), INTENT(IN) :: ppl(knbmu, -1:knang+1)
  REAL(KIND=JPRB), INTENT(INOUT) :: phasef(0:knt, knbmu), phasefd(0:knt, knbmu)
  REAL(KIND=JPRB), INTENT(IN) :: pomega(0:knt), pomegad(0:knt)
  REAL(KIND=JPRB), INTENT(IN) :: ppsl(0:knang, -1:knang, -kmu:kmu)
  REAL(KIND=JPRB), INTENT(OUT) :: proatm, psphal
  REAL(KIND=JPRB), INTENT(OUT) :: proatmd, psphald
  REAL(KIND=JPRB), INTENT(OUT) :: pdtdir, pdtdif
  REAL(KIND=JPRB), INTENT(OUT) :: pdtdird, pdtdifd
  REAL(KIND=JPRB), INTENT(OUT) :: putdir, putdif
  REAL(KIND=JPRB), INTENT(OUT) :: putdird, putdifd
! LOCAL VARIABLES
! ---------------
  INTEGER(KIND=JPIM) :: jlev
  REAL(KIND=JPRB) :: z_pbetal(0:knt, 0:knang), z_pcoeff(0:knt)
  REAL(KIND=JPRB) :: z_pbetald(0:knt, 0:knang), z_pcoeffd(0:knt)
  REAL(KIND=JPRB) :: z_promix
  REAL(KIND=JPRB) :: z_promixd
  REAL(KIND=JPRB) :: z_ptamoy(0:knt), z_pizmoy(0:knt)
  REAL(KIND=JPRB) :: z_ptamoyd(0:knt), z_pizmoyd(0:knt)
  REAL(KIND=JPRB) :: z_ptodaer, z_ptodray
  REAL(KIND=JPRB) :: z_ptodaerd
  REAL(KIND=JPRB) :: z_pddirtt, z_pddiftt, z_pudirtt, z_pudiftt, z_psphalbt
  REAL(KIND=JPRB) :: z_pddirttd, z_pddifttd, z_pudirttd, z_pudifttd, z_psphalbtd

  REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "atmref_gems_tl.intfb.h"
#include "scatra_gems_tl.intfb.h"
#include "trunca_gems_tl.intfb.h"

  IF (LHOOK) CALL DR_HOOK('DISCOM_GEMS_TL',0,ZHOOK_HANDLE)

! THE AEROSOL PHASE FUNCTION IS DECOMPOSED IN THE BETAL

  CALL TRUNCA_GEMS_TL(phasef, phasefd, prmu, pga, ppl, z_pbetal, z_pbetald, &
&               z_pcoeff, z_pcoeffd)
  z_pizmoyd(0:knt) = 0._JPRB
  z_ptamoyd(0:knt) = 0._JPRB

! TRUNCATION OF THE MOLECULAR AND AEROSOL PHASE FUNCTION

  DO jlev=0,knt
    z_ptamoyd(jlev) = ptaerd(jlev)*(1.0_JPRB-pomega(jlev)*z_pcoeff(jlev)) + &
&      ptaer(jlev)*(-(pomegad(jlev)*z_pcoeff(jlev))-pomega(jlev)*z_pcoeffd(&
&      jlev))
    z_ptamoy(jlev) = ptaer(jlev)*(1.0_JPRB-pomega(jlev)*z_pcoeff(jlev))
    z_pizmoyd(jlev) = ((pomegad(jlev)*(1.0_JPRB-z_pcoeff(jlev))-pomega(jlev)*&
&      z_pcoeffd(jlev))*(1.0_JPRB-pomega(jlev)*z_pcoeff(jlev))-pomega(jlev)*(&
&      1.0_JPRB-z_pcoeff(jlev))*(-(pomegad(jlev)*z_pcoeff(jlev))-pomega(jlev)*&
&      z_pcoeffd(jlev)))/(1.0_JPRB-pomega(jlev)*z_pcoeff(jlev))**2
    z_pizmoy(jlev) = pomega(jlev)*(1.0_JPRB-z_pcoeff(jlev))/(1.0_JPRB-pomega(jlev)&
&      *z_pcoeff(jlev))
  ENDDO

! AEROSOL AND MOLECULAR OPTICAL DEPTH

  z_ptodaer = 0.0_JPRB
  z_ptodray = 0.0_JPRB
  z_ptodaerd = 0._JPRB
  DO jlev=0,knt
! AEROSOL OPTICAL THICKNESS
    z_ptodaerd = z_ptodaerd + z_ptamoyd(jlev)
    z_ptodaer = z_ptodaer + z_ptamoy(jlev)
! MOLECULAR OPTICAL THICKNESS
    z_ptodray = z_ptodray + ptray(jlev)
  ENDDO

! COMPUTATION OF AEROSOL, RAYLEIGH AND MIXTURE REFLECTANCE

  CALL ATMREF_GEMS_TL(z_ptamoy, z_ptamoyd, ptray, z_pizmoy, z_pizmoyd, phirad, &
&               pxmus, pxmuv, prm, pgb, prp, z_pbetal, z_pbetald, ppsl, &
&               z_ptodaer, z_ptodray, z_promix, z_promixd)

! COMPUTATION OF THE TRANSMISSION FUNCTION

  CALL SCATRA_GEMS_TL(z_ptamoy, z_ptamoyd, ptray, z_ptodaer, z_ptodaerd, z_ptodray, &
&               z_pizmoy, z_pizmoyd, prm, pgb, pxmus, pxmuv, z_pbetal, &
&               z_pbetald, ppsl, z_pddirtt, z_pddirttd, z_pddiftt, z_pddifttd, &
&               z_pudirtt, z_pudirttd, z_pudiftt, z_pudifttd, z_psphalbt, &
&               z_psphalbtd)

  proatmd = z_promixd
  proatm = z_promix
  pdtdird = z_pddirttd
  pdtdir = z_pddirtt
  pdtdifd = z_pddifttd
  pdtdif = z_pddiftt
  putdird = z_pudirttd
  putdir = z_pudirtt
  putdifd = z_pudifttd
  putdif = z_pudiftt
  psphald = z_psphalbtd
  psphal = z_psphalbt

 IF (LHOOK) CALL DR_HOOK('DISCOM_GEMS_TL',1,ZHOOK_HANDLE)
END SUBROUTINE DISCOM_GEMS_TL
