SUBROUTINE EXCHCO(&
 & KPROMA,KIDIA,KFDIA,&
 & PQ,PT,PAPHI,PAPHIH,PR,PTS,&
 & PUB,PVB,PAZO,&
 & PWET,PQS,&
 & PRI,PBN,PBM,PBH)  

!**** *EXCHCO* - COMPUTATION OF THE MODIFIED ROUGHNESS LENGTHS

!     PURPOSE
!     -------
!          COMPUTE THE FUNCTION OF DRAG FOR MOMENTUM AND HEAT
!          AS A FUNCTION OF STABILITY FUNCTION.

!**   INTERFACE
!     ---------
!       *CALL* *EXCHCO(
!    C                 KPROMA,KIDIA,KFDIA,NFLEVG,
!    I                 ZQF5(1,NFLEVG),ZTF5(1,NFLEVG),
!    I                 ZAPHI5(1,NFLEVG),ZTS5,
!    I                 ZUF5(1,NFLEVG),ZVF5(1,NFLEVG),ZAZO5,
!    I                 ZWET5,ZQS5(1,NFLEVG),
!    O                 ZRI,ZBN,ZBM,ZBH)
!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA             - HORIZONTAL DIMENSION.                    (INPUT-C)
!        KIDIA              - INDEX OF START POINT                     (INPUT-C)
!        KFDIA              - INDEX OF END   POINT                     (INPUT-C)
!        PQ(KPROMA          - SPECIFIC HUMIDITY LOWER  LEVEL  Kg/Kg    (INPUT)
!        PT(KPROMA)         - TEMPERATURE ON THE LOWER          K      (INPUT)
!        PAPHI(KPROMA)      - GEOPOTENTIAL MODEL LEVELS     M**2/S**2  (INPUT)
!        PTS(KPROMA)        - SURFACE TEMPERATURE               K      (INPUT)
!        PUB(KPROMA)        - U-WIND COMPONENT                 M/S     (INPUT)
!        PVB(KPROMA)        - V-WIND COMPONENT                 M/S     (INPUT)
!        PAZO(KPROMA)       - ROUGHNESS DISTRIBUTION            M      (INPUT)
!        PWET(KPROMA)       - WETNESS                           M      (INPUT)
!        PQS(KPROMA)        - SURFACE SATURATION              Kg/Kg    (INPUT)
!        PRI(KPROMA)        - RICHARSON NUMBER                         (OUTPUT)
!        PBN(KPROMA)        - NEUTRAL COEFFICIENT                      (OUTPUT)
!        PBM(KPROMA)        - MOMENTUM COEFFICIENT                     (OUTPUT)
!        PBH(KPROMA)        - HEAT COEFFICIENT                         (OUTPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        CARLA CARDINALI *ECMWF*

!        ORIGINAL   : 92-05-22
!        L. Isaksen : 96-02-20  NFLEV --> NFLEVG
!        MODIFICATION : P. MOLL 18 juillet 1996  ACHMT transplantation      
!                       1999-06, J.M. Piriou: Richardson critical number variable in the vertical.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RD       ,RV       ,RCPD     ,RKAPPA   ,RETV
USE YOETHF   , ONLY : RVTMP2
USE YOEVDF   , ONLY : RKAP     ,REPDU2
USE YOMCT0B  , ONLY : LECMWF
USE YOMPHY0  , ONLY : USURIC   ,USURICL

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAZO(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWET(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBH(KPROMA) 
REAL(KIND=JPRB) :: ZRTI(KPROMA),ZRS(KPROMA)
REAL(KIND=JPRB) :: ZDFI(KPROMA)

INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZB, ZC, ZCDN, ZCDN1, ZCFH, ZCFH1, ZCFM, ZCFM1,&
 & ZCONS1, ZCONS2, ZCONS3, ZCONS4, ZD, ZDPHI, &
 & ZDU2, ZEPSEC, ZRVMD, ZSCF, ZTVD, ZTVS, ZUCF, &
 & ZUCF1  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     1.     DEFINITION OF THE CONSTANTS
!            ---------------------------

!*    1.1    PHISICAL CONSTANTS

IF (LHOOK) CALL DR_HOOK('EXCHCO',0,ZHOOK_HANDLE)
ZRVMD=RV-RD

!     TAKEN FROM COMMON DECKS

!*    1.2  *RKAP* IS THE VON KARMAN CONSTANT, *ZB,* *ZC* AND *ZD* ARE
!          CONSTANTS FOR FORMULAE FOR STABILITY DEPENDENCY
!          RESPECTIVELY   NEAR THE NEUTRAL, IN THE UNSTABLE AND IN THE
!          STABLE CASE.

ZB=5._JPRB
ZC=5._JPRB
ZD=5._JPRB

!*    1.3    SECURITY PARAMETER.
!            *REPDU2* IS A MINIMUM SQUARED WIND INCREMENT
!            TO AVOID DIVIDING BY ZERO IN THE *RICHARDSON NUMBER'S  A
!            CALCULATION.
!            *ZEPSEC* IS A MINIMUM VALUE FOR DRAG COEFFICIENTS.
ZEPSEC=1.E-02_JPRB

!*    1.4     COMPUTATIONAL CONSTANTS.

ZCONS1=3._JPRB*ZB*ZC
ZCONS2=2.0_JPRB*ZB
ZCONS3=3._JPRB*ZB
ZCONS4=1.0_JPRB/RKAP**2

!*         2.     COMPUTATION OF THE MODIFIED ROUGHNESS LENGTHS.
!                 ----------------------------------------------

!*         2.1     COMPUTATION OF THE ELEMENTS USED IN THE DRAG
!                  COEFFICIENT.
!                  FUNCTION OF DRAG FOR MOMENTUM AND HEAT

DO JL=KIDIA,KFDIA
  ZDU2=PUB(JL)**2+PVB(JL)**2
  IF(ZDU2 > REPDU2) THEN
    ZDU2=ZDU2
  ELSE
    ZDU2=REPDU2
  ENDIF
  ZDPHI=PAPHI(JL)/RCPD
  ZTVD=(PT(JL)+ZDPHI/(1.0_JPRB+RVTMP2*PQ(JL)))*(1.0_JPRB+RETV*PQ(JL))
  ZTVS=PTS(JL)*(1.0_JPRB+RETV*PWET(JL)*PQS(JL))
  PRI(JL)=PAPHI(JL)*(1.0_JPRB-ZTVS/ZTVD)/ZDU2

  IF(.NOT.LECMWF)THEN
! achmt Ri number
    ZRS(JL)=RD+ZRVMD*PQS(JL)
    ZDFI(JL)=PAPHI(JL)-PAPHIH(JL)
    ZRTI(JL)=2.0_JPRB/(PR(JL)*PT(JL)+RKAPPA*ZDFI(JL)+ZRS(JL)*PTS(JL))
    PRI(JL)=ZDFI(JL)*(PR(JL)*PT(JL)+RKAPPA &
     & *ZDFI(JL)-ZRS(JL)*PTS(JL))*ZRTI(JL)/ZDU2  
    PRI(JL)=PRI(JL)/(1.0_JPRB+MAX(0.0_JPRB,PRI(JL))*USURIC*USURICL)

  ENDIF

  ZCDN1=LOG(1.0_JPRB+PAPHI(JL)/(RG*PAZO(JL)))
  ZCDN=(RKAP/ZCDN1)**2
  IF(PRI(JL) >= 0.0_JPRB) THEN
    ZSCF=SQRT(1.0_JPRB+ZD*PRI(JL))
    ZCFM=ZCDN*(1.0_JPRB/(1.0_JPRB+ZCONS2*PRI(JL)/ZSCF))
    ZCFH=ZCDN*(1.0_JPRB/(1.0_JPRB+ZCONS3*PRI(JL)*ZSCF))
  ELSE
    ZUCF1=SQRT(-PRI(JL)*(1.0_JPRB+PAPHI(JL)/(RG*PAZO(JL))))
    ZUCF=1.0_JPRB/(1.0_JPRB+ZCONS1*ZCDN*ZUCF1)
    ZCFM=ZCDN*(1.0_JPRB-ZCONS2*PRI(JL)*ZUCF)
    ZCFH=ZCDN*(1.0_JPRB-ZCONS3*PRI(JL)*ZUCF)
  ENDIF
  ZCFM1=SQRT(ZCFM*ZCONS4)
  IF(ZCFM1 > ZEPSEC) THEN
    ZCFM1=ZCFM1
  ELSE
    ZCFM1=ZEPSEC
  ENDIF
  ZCFH1=ZCFH*ZCONS4/ZCFM1
  IF(ZCFH1 > ZEPSEC) THEN
    ZCFH1=ZCFH1
  ELSE
    ZCFH1=ZEPSEC
  ENDIF
  PBN(JL)=RKAP/SQRT(ZCDN)
  PBM(JL)=1.0_JPRB/ZCFM1
  PBH(JL)=1.0_JPRB/ZCFH1
ENDDO

!     --------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('EXCHCO',1,ZHOOK_HANDLE)
END SUBROUTINE EXCHCO

