SUBROUTINE EXCHCO_VDF(&
 & KPROMA,KIDIA,KFDIA,&
 & PUB5,PVB5,PUN5,PVN5,&
 & PAPHI5,PAZO5,&
 & PRI5,PBN5,PBM5)  

!**** *EXCHCO_VDF* - COMPUTATION OF THE MODIFIED ROUGHNESS LENGTHS

!     PURPOSE
!     -------
!          COMPUTE THE FUNCTION OF DRAG FOR MOMENTUM
!          BASED ON SURFACE ROUGNESS, LOWEST-LEVEL WIND AND 10M NEUTRAL WIND

!**   INTERFACE
!     ---------
!       *CALL* *EXCHCO_VDF(
!    C                 KPROMA,KIDIA,KFDIA,
!    I                 ZUF5(1,NFLEVG),ZVF5(1,NFLEVG),ZUN5(1,NFLEVG),ZVN5(1,NFLEVG),
!    I                 PAPHI5(1,NFLEVG),PAZO5(1,NFLEVG),
!    O                 ZBN5(1,NFLEVG),ZBM5(1,NFLEVG)
!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA             - HORIZONTAL DIMENSION.                      (INPUT-C)
!        KIDIA              - INDEX OF START POINT                       (INPUT-C)
!        KFDIA              - INDEX OF END   POINT                       (INPUT-C)
!        PAPHI5(KPROMA)     - GEOPOTENTIAL     AT LOWER LEVEL M**2/S**2  (INPUT)
!        PUB5(KPROMA)       - U-WIND COMPONENT AT LOWER LEVEL    M/S     (INPUT)
!        PVB5(KPROMA)       - V-WIND COMPONENT AT LOWER LEVEL    M/S     (INPUT)
!        PUN5(KPROMA)       - U-WIND NEUTRAL COMPONENT AT 10M    M/S     (INPUT)
!        PVN5(KPROMA)       - V-WIND NEUTRAL COMPONENT AT 10M    M/S     (INPUT)
!        PRI5(KPROMA)       - SIGN OF THE RICHARDSON NUMBER      +/-1    (OUTPUT)
!        PBN5(KPROMA)       - NEUTRAL COEFFICIENT                        (OUTPUT)
!        PBM5(KPROMA)       - MOMENTUM COEFFICIENT                       (OUTPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        HANS HERSBACH *ECMWF*

!        ORIGINAL   : 28-10-2009
!     ------------------------------------------------------------------

USE PARKIND1 ,ONLY : JPIM     ,JPRB
USE YOMHOOK ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG
USE YOEVDF   , ONLY : REPDU2

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUB5  (KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVB5  (KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUN5  (KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVN5  (KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAZO5 (KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRI5  (KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBN5  (KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBM5  (KPROMA) 

INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZH,ZL,ZBL,ZBH,ZDUS,ZDNS,ZDU,ZDN

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('EXCHCO_VDF',0,ZHOOK_HANDLE)

!*         1.     COMPUTATION OF THE MODIFIED ROUGHNESS LENGTH.
!                 ---------------------------------------------

ZH=10.0_JPRB
DO JL=KIDIA,KFDIA
  ZDUS    =MAX(PUB5(JL)**2+PVB5(JL)**2,REPDU2)
  ZDNS    =MAX(PUN5(JL)**2+PVN5(JL)**2,REPDU2)
  ZDU     =SQRT(ZDUS)
  ZDN     =SQRT(ZDNS)

  ZL      =PAPHI5(JL)/RG
  ZBL     =LOG(1.0_JPRB+ZL/PAZO5(JL))
  ZBH     =LOG(1.0_JPRB+ZH/PAZO5(JL))

  PBN5(JL)=ZBL
  PBM5(JL)=ZBH*ZDU/ZDN

  IF (PBN5(JL)<PBM5(JL)) THEN ! Stable
     PRI5(JL)= 1.0_JPRB
  ELSE
     PRI5(JL)=-1.0_JPRB
  ENDIF
ENDDO

!     --------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('EXCHCO_VDF',1,ZHOOK_HANDLE)
END SUBROUTINE EXCHCO_VDF

