SUBROUTINE EXCHCOTL(&
 & KPROMA,KIDIA,KFDIA,&
 & PQ5,PT5,PAPHI5,PAPHIH5,PR5,PTS5,&
 & PQ,PT,PAPHI,PAPHIH,PR,PTS,&
 & PUB5,PVB5,PAZO5,&
 & PUB,PVB,PAZO,&
 & PWET5,PQS5,&
 & PWET,PQS,&
 & PRI5,&
 & PBN,PBM,PBH)  

!**** *EXCHCOTL* - TANGENT LINEAR OF THE EXCHANGE COEFFICIENTS

!     PURPOSE
!     -------
!          COMPUTE THE TANGENT LINEAR OF DRAG FOR MOMENTUM AND HEAT
!          OF DRAG COEFFICIENT FOR MOMENTUM AND HEAT
!          AND OF THE RICHARDSON NUMBER

!**   INTERFACE
!     ---------
!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KIDIA                     - INITIAL POINT
!        KFDIA                     - FINAL POINT
!        PQ(KPROMA)                - SPECIFIC HUMIDITY ON THE LOWER
!                                    LEVEL.                            (INPUT)
!        PT(KPROMA)                - TEMPERATURE ON THE LOWER LEVEL    (INPUT)
!        PAPHI(KPROMA)             - GEOPOTENTIAL LOWER MODEL LEVEL    (INPUT)
!        PTS(KPROMA)               - SURFACE TEMPERATURE               (INPUT)
!        PUB(KPROMA)               - U-WIND COMPONENT                  (INPUT)
!        PVB(KPROMA)               - V-WIND COMPONENT                  (INPUT)
!        PAZO(KPROMA)              - ROUGHNESS DISTRIBUTION            (INPUT)
!        PWET(KPROMA)              - SOIL HUMIDITY                     (INPUT)
!        PQS(KPROMA)               - SURFACE SATURATION HUMIDITY       (INPUT)
!        PRI(KPROMA)               - RICHARSON NUMBER FUNCTION         (INPUT)
!        PBN(KPROMA)               - NEUTRAL COEFFICIENT               (OUTPUT)
!        PBM(KPROMA)               - MOMENTUM COEFFICIENT              (OUTPUT)
!        PBH(KPROMA)               - HEAT COEFFICIENT                  (OUTPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        CARLA CARDINALI *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 92-05-22
!        MODIFICATION : P. MOLL   18 juillet  1996 ACHMT transplantation
!        MODIFICATION : F. VEERSE 11 novembre 1997 Bug correction
!                       1999-06, J.M. Piriou: Richardson critical number variable in the vertical.
!                     2000-03-01, M. Janiskova: REPSEC changed to ZEPSEC since 
!                                               REPSEC should not be redefined
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RD       ,RV       ,&
 & RCPD     ,RKAPPA   ,RETV  
USE YOETHF   , ONLY : RVTMP2
USE YOEVDF   , ONLY : RKAP     ,REPDU2
USE YOMCT0B  , ONLY : LECMWF
USE YOMPHY0  , ONLY : USURIC   ,USURICL

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIH5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUB5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVB5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAZO5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAZO(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWET5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWET(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBH(KPROMA) 
REAL(KIND=JPRB) :: ZRTI(KPROMA),ZRS(KPROMA)
REAL(KIND=JPRB) :: ZRTI5(KPROMA),ZRS5(KPROMA)
REAL(KIND=JPRB) :: ZDFI(KPROMA),ZDFI5(KPROMA)

INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZB, ZC, ZCDN, ZCDN1, ZCDN15, ZCDN5, ZCFH,&
 & ZCFH1, ZCFH15, ZCFH5, ZCFM, ZCFM1, ZCFM15, &
 & ZCFM5, ZCONS1, ZCONS2, ZCONS3, ZCONS4, ZD, &
 & ZDPHI, ZDPHI5, ZDU2, ZDU25, ZRI, ZRI5, ZRIV, &
 & ZRIV5, ZRVMD, ZSCF, ZSCF5, ZTVD, ZTVD5, ZTVS, &
 & ZTVS5, ZUCF, ZUCF1, ZUCF15, ZUCF5, ZEPSEC  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     1.     DEFINITION OF THE CONSTANTS
!            ---------------------------

!*    1.1    PHYSICAL CONSTANTS

IF (LHOOK) CALL DR_HOOK('EXCHCOTL',0,ZHOOK_HANDLE)
ZRVMD=RV-RD

!     TAKEN FROM COMMON DECKS

!*    1.2  *RKAP* IS THE VON KARMAN CONSTANT, *ZB,* *ZC* AND *ZD* ARE
!          CONSTANTS FOR FORMULAE FOR STABILITY DEPENDENCY
!          RESPECTIVELY   NEAR THE NEUTRAL, IN THE UNSTABLE AND IN THE
!          STABLE CASE.

ZB=5._JPRB
ZC=5._JPRB
ZD=5._JPRB

!*    1.3    SECURITY PARAMETER.
!            *REPDU2* IS A MINIMUM SQUARED WIND INCREMENT
!            TO AVOID DIVIDING BY ZERO IN THE *RICHARDSON NUMBER'S  A
!            CALCULATION.
!            *ZEPSEC* IS A MINIMUM VALUE FOR DRAG COEFFICIENTS.
ZEPSEC=1.E-02_JPRB

!*    1.4     COMPUTATIONAL CONSTANTS.

ZCONS1=3._JPRB*ZB*ZC
ZCONS2=2.0_JPRB*ZB
ZCONS3=3._JPRB*ZB
ZCONS4=1.0_JPRB/RKAP**2

!*      2.  TANGENT LINEAR COMPUTATION

DO JL=KIDIA,KFDIA
  ZDU25=PUB5(JL)**2+PVB5(JL)**2
  IF(ZDU25 > REPDU2) THEN
    ZDU25=ZDU25
  ELSE
    ZDU25=REPDU2
  ENDIF
  ZDPHI5=PAPHI5(JL)/RCPD
  ZTVD5=(PT5(JL)+ZDPHI5/(1.0_JPRB+RVTMP2*PQ5(JL)))*(1.0_JPRB+RETV*PQ5(JL))
  ZTVS5=PTS5(JL)*(1.0_JPRB+RETV*PWET5(JL)*PQS5(JL))

  IF(.NOT.LECMWF)THEN
! achmt Ri number
    ZRS5(JL)=RD+ZRVMD*PQS5(JL)
    ZDFI5(JL)=PAPHI5(JL)-PAPHIH5(JL)
    ZRTI5(JL)=2.0_JPRB/(PR5(JL)*PT5(JL)+RKAPPA*ZDFI5(JL)+ZRS5(JL)*PTS5(JL))
    ZRIV5=ZDFI5(JL)*(PR5(JL)*PT5(JL)+RKAPPA &
     & *ZDFI5(JL)-ZRS5(JL)*PTS5(JL))*ZRTI5(JL)  
    ZRI5=ZRIV5/ZDU25
    ZRI5=ZRI5/(1.0_JPRB+MAX(0.0_JPRB,ZRI5)*USURIC*USURICL)

  ENDIF

  ZCDN15=LOG(1.0_JPRB+PAPHI5(JL)/(RG*PAZO5(JL)))
  ZCDN5=(RKAP/ZCDN15)**2
  IF(PRI5(JL) >= 0.0_JPRB) THEN
    ZSCF5=SQRT(1.0_JPRB+ZD*PRI5(JL))
    ZCFM5=ZCDN5*(1.0_JPRB/(1.0_JPRB+ZCONS2*PRI5(JL)/ZSCF5))
    ZCFH5=ZCDN5*(1.0_JPRB/(1.0_JPRB+ZCONS3*PRI5(JL)*ZSCF5))
  ELSE
    ZUCF15=SQRT(-PRI5(JL)*(1.0_JPRB+PAPHI5(JL)/(RG*PAZO5(JL))))
    ZUCF5=1.0_JPRB/(1.0_JPRB+ZCONS1*ZCDN5*ZUCF15)
    ZCFM5=ZCDN5*(1.0_JPRB-ZCONS2*PRI5(JL)*ZUCF5)
    ZCFH5=ZCDN5*(1.0_JPRB-ZCONS3*PRI5(JL)*ZUCF5)
  ENDIF
  ZCFM15=SQRT(ZCFM5*ZCONS4)
  IF(ZCFM15 > ZEPSEC) THEN
    ZCFM15=ZCFM15
  ELSE
    ZCFM15=ZEPSEC
  ENDIF
  ZCFH15=ZCFH5*ZCONS4/ZCFM15
  IF(ZCFH15 > ZEPSEC) THEN
    ZCFH15=ZCFH15
  ELSE
    ZCFH15=ZEPSEC
  ENDIF

  IF(ZDU25 > REPDU2)THEN
    ZDU2=2.0_JPRB*PUB5(JL)*PUB(JL)+2.0_JPRB*PVB5(JL)*PVB(JL)
  ELSE
    ZDU2=0.0_JPRB
  ENDIF
  ZDPHI=PAPHI(JL)/RCPD
  ZTVD=(1.0_JPRB+RETV*PQ5(JL))*PT(JL)&
   & +1.0_JPRB/(1.0_JPRB+RVTMP2*PQ5(JL))*(1.0_JPRB+RETV*PQ5(JL))*ZDPHI &
   & +( PT5(JL)*RETV-ZDPHI5*RVTMP2 &
   & *(1.0_JPRB+RETV*PQ5(JL))/(1.0_JPRB+RVTMP2*PQ5(JL))**2 &
   & +ZDPHI5/(1.0_JPRB+RVTMP2*PQ5(JL))*RETV )*PQ(JL)  
  ZTVS=(1.0_JPRB+RETV*PWET5(JL)*PQS5(JL))*PTS(JL)&
   & +PTS5(JL)*RETV*PQS5(JL)*PWET(JL)&
   & +PTS5(JL)*RETV*PWET5(JL)*PQS(JL)  
  ZRI=(1.0_JPRB-ZTVS5/ZTVD5)/ZDU25*PAPHI(JL)&
   & -PAPHI5(JL)/ZTVD5/ZDU25*ZTVS &
   & +PAPHI5(JL)*ZTVS5/ZTVD5**2/ZDU25*ZTVD &
   & -PAPHI5(JL)*(1.0_JPRB-ZTVS5/ZTVD5)/ZDU25**2*ZDU2  
  IF(.NOT.LECMWF)THEN
! achmt Ri number
    ZRS(JL)=ZRVMD*PQS(JL)
    ZDFI(JL)=PAPHI(JL)-PAPHIH(JL)
    ZRTI(JL)=-2.0_JPRB*(PR(JL)*PT5(JL)+PR5(JL)*PT(JL)&
     & +RKAPPA*ZDFI(JL)&
     & +ZRS(JL)*PTS5(JL)+ZRS5(JL)*PTS(JL))/&
     & (PR5(JL)*PT5(JL)+RKAPPA*ZDFI5(JL)&
     & +ZRS5(JL)*PTS5(JL))**2.0_JPRB  
    ZRIV=ZDFI(JL)*(PR5(JL)*PT5(JL)+RKAPPA &
     & *ZDFI5(JL)-ZRS5(JL)*PTS5(JL))*ZRTI5(JL) +&
     & ZDFI5(JL)*(PR5(JL)*PT5(JL)+RKAPPA &
     & *ZDFI5(JL)-ZRS5(JL)*PTS5(JL))*ZRTI(JL) +&
     & ZDFI5(JL)*(PR5(JL)*PT(JL)+PR(JL)*PT5(JL)+&
     & RKAPPA*ZDFI(JL)-&
     & ZRS(JL)*PTS5(JL)-ZRS5(JL)*PTS(JL))*ZRTI5(JL)  
    ZRI=(ZDU25*ZRIV-ZRIV5*ZDU2)/ZDU25**2
    IF(ZRI5 > 0.0_JPRB)THEN
      ZRI=ZRI/(1.0_JPRB+ZRI5*USURIC*USURICL)**2
    ENDIF
  ENDIF

  ZCDN1=1.0_JPRB/(RG*PAZO5(JL))/(1.0_JPRB+PAPHI5(JL)/(RG*PAZO5(JL)))*PAPHI(JL)&
   & -PAPHI5(JL)/(RG*PAZO5(JL)**2)&
   & /(1.0_JPRB+PAPHI5(JL)/(RG*PAZO5(JL)))*PAZO(JL)  
  ZCDN=-2.0_JPRB*(RKAP/ZCDN15)*RKAP/ZCDN15**2*ZCDN1
  IF(PRI5(JL) >= 0.0_JPRB) THEN
    ZSCF=ZD/(2.0_JPRB*ZSCF5)*ZRI
    ZCFM=(1.0_JPRB/(1.0_JPRB+ZCONS2*PRI5(JL)/ZSCF5))*ZCDN &
     & +ZCDN5/(1.0_JPRB+ZCONS2*PRI5(JL)/ZSCF5)**2 &
     & *( -ZCONS2*( ZRI/ZSCF5 - PRI5(JL)*ZSCF/ZSCF5**2 ) )  
    ZCFH=(1.0_JPRB/(1.0_JPRB+ZCONS3*PRI5(JL)*ZSCF5))*ZCDN &
     & +ZCDN5/(1.0_JPRB+ZCONS3*PRI5(JL)*ZSCF5)**2 &
     & *( -ZCONS3*( ZRI*ZSCF5 + PRI5(JL)*ZSCF ) )  
  ELSE
    ZUCF1=1.0_JPRB/(2.0_JPRB*ZUCF15)*( -(1.0_JPRB+PAPHI5(JL)/(RG*PAZO5(JL)))*ZRI &
     & -PRI5(JL)/(RG*PAZO5(JL))*PAPHI(JL)&
     & +PRI5(JL)*PAPHI5(JL)/(RG*PAZO5(JL)**2)*PAZO(JL) )  
    ZUCF=-ZUCF5**2*( ZCONS1*ZUCF15*ZCDN+ZCONS1*ZCDN5*ZUCF1 )
    ZCFM=(1.0_JPRB-ZCONS2*PRI5(JL)*ZUCF5)*ZCDN &
     & +ZCDN5*( -ZCONS2*( ZRI*ZUCF5 + PRI5(JL)*ZUCF ) )  
    ZCFH=(1.0_JPRB-ZCONS3*PRI5(JL)*ZUCF5)*ZCDN &
     & +ZCDN5*( -ZCONS3*( ZRI*ZUCF5 + PRI5(JL)*ZUCF ) )  
  ENDIF
  IF(ZCFM15 > ZEPSEC)THEN
    ZCFM1=ZCONS4/(2.0_JPRB*ZCFM15)*ZCFM
  ELSE
    ZCFM1=0.0_JPRB
  ENDIF
  IF(ZCFH15 > ZEPSEC)THEN
    ZCFH1=ZCONS4/ZCFM15*ZCFH-ZCFH5*ZCONS4/ZCFM15**2*ZCFM1
  ELSE
    ZCFH1=0.0_JPRB
  ENDIF
  PBN(JL)=-RKAP/(2.0_JPRB*SQRT(ZCDN5)*ZCDN5)*ZCDN
  PBM(JL)=-1.0_JPRB/ZCFM15**2*ZCFM1
  PBH(JL)=-1.0_JPRB/ZCFH15**2*ZCFH1
ENDDO
IF (LHOOK) CALL DR_HOOK('EXCHCOTL',1,ZHOOK_HANDLE)
END SUBROUTINE EXCHCOTL

