SUBROUTINE gpscalc_alpha2d(kobs,    & ! no.of observations
                         & klev,    & ! no. of vertical levels
                         & khoriz,  & ! no. of horizontal layers  ODD
                         & pdsep,   & ! the angular spacing 
                         & pa,      & ! impact parameter values
                         & prefrac, & ! refractivity
                         & pradius, & ! radius values
                         & pnr,     & ! nr product
                         & pa_path, & ! the impact params at end points
                         & palpha)    ! 2d bending angles. 

!**** *gpscalc_alpha2d* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2004/10/26
!     Based on a GRAS-SAF routine.   

!     PURPOSE
!     -------
!     It evaluates the bending angle integral for each impact parameter

!**   INTERFACE
!     ---------

!     CALL gpscalc_alpha2d(....)    

!     WHERE kobs   =  number of observed bending angles 
!           klev   =  number of vertical levels
!           khoriz =  number of horizontal locations
!           pdsep  =  angular spacing
!           pa     =  impact parameters
!           prefrac=  refractivity values
!           pradius=  radius values
!           pnr    =  nr product 
!           pa_path= impact parameter at end points of ray path
!           paloha  = bending angle values 


!     gpscalc_alpha2d is called from gpsro_op2d

!**   METHOD
!     ------
!     1) calculate the exponential decay of refractivity with nr between model levels
!     2) evaluates the bending angle integral for each impact parameter

!**   EXTERNALS
!     ---------
!     1) fcserf calculates the Gaussian error function

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS  , ONLY : RMDI 

USE YOMCST   , ONLY : RPI



IMPLICIT NONE


! subroutine args. 


INTEGER(KIND=JPIM), INTENT(IN)  :: kobs                 ! size of ob. vector
INTEGER(KIND=JPIM), INTENT(IN)  :: klev                 ! no. of refractivity levels
INTEGER(KIND=JPIM), INTENT(IN)  :: khoriz               ! no. of horizontal locations
REAL(KIND=JPRB),    INTENT(IN)  :: pdsep                ! angular spacing of grid
REAL(KIND=JPRB),    INTENT(IN)  :: pa(kobs)             ! impact parameter
REAL(KIND=JPRB),    INTENT(IN)  :: prefrac(klev,khoriz) ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(IN)  :: pradius(klev,khoriz) ! radius values
REAL(KIND=JPRB),    INTENT(IN)  :: pnr(klev,khoriz)     ! index * radius product
REAL(KIND=JPRB),    INTENT(OUT) :: pa_path(kobs,2)        
REAL(KIND=JPRB),    INTENT(OUT) :: palpha(kobs)         ! 2d bending angles 
       

! local variables


INTEGER(KIND=JPIM) :: i,in,ibot,jbot,ikbot,iside,ik,ikp1
INTEGER(KIND=JPIM) :: icen
REAL(KIND=JPRB) :: zkval
REAL(KIND=JPRB) :: zroot_halfpi
REAL(KIND=JPRB) :: fcserf,zdiff_erf
REAL(KIND=JPRB) :: zdalpha
REAL(KIND=JPRB) :: zdthi,zthi_up,zthi_low
REAL(KIND=JPRB) :: zrad,zds
REAL(KIND=JPRB) :: zsinthi,zrad_over_sinthi
REAL(KIND=JPRB) :: zavar
REAL(KIND=JPRB) :: zhwt1,zhwt2
REAL(KIND=JPRB) :: zdn_dtheta
REAL(KIND=JPRB) :: zalpha_half(2)
REAL(KIND=JPRB) :: ztheta,zdtheta,ztheta_tan,ztheta_min,ztheta_max
REAL(KIND=JPRB) :: zref_low,zref_up
REAL(KIND=JPRB) :: znr_low,znr_up
REAL(KIND=JPRB) :: ztup,ztlow
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GPSCALC_ALPHA2D',0,ZHOOK_HANDLE)


! the central profile icen


icen = khoriz/2 + 1

ztheta_tan = REAL(icen-1)*pdsep 
ztheta_min = -ztheta_tan
ztheta_max =  ztheta_tan


! cannot integrate through super refracting layer


jbot = 1

DO in = 1,khoriz

  DO

    IF ((prefrac(jbot,in) > 0.0_JPRB .AND. pnr(jbot,in) > 0.0_JPRB) .OR. &
    &jbot == klev) EXIT
  
    jbot = jbot + 1

  ENDDO

ENDDO

DO in = 1,khoriz

   ikbot = klev

   DO i=klev,jbot+1,-1

      IF (pnr(ikbot,in) < pnr(ikbot-1,in)) EXIT 

      ikbot = ikbot - 1

   ENDDO
 
   jbot = MAX(jbot,ikbot)

ENDDO


! set the outputs to missing


palpha(:)=RMDI
pa_path(:,:)= RMDI


! now calculate the bending angles


zRoot_halfPI = SQRT(0.5_JPRB*RPI)

DO in=1,kobs

   IF (pa(in) < pnr(jbot,icen) .OR. pa(in) > pnr(klev,icen)) CYCLE  

   ibot = jbot

   DO 

      IF (pa(in) < pnr(ibot+1,icen)) EXIT   !

      ibot=ibot+1

   ENDDO
 

!  initialise bending angle value   

   zalpha_half(:) = 0.0_JPRB

   DO iside = 1,2 
    
      zavar = pa(in) 
      ztheta = 0.0_JPRB
      zthi_low = 0.5_JPRB*RPI
      
      DO i = ibot, klev-1
     

         IF (i == ibot) THEN

            ik = icen  
          
            znr_low = zavar  
            znr_up  = pnr(i+1,ik)

! refractivity at tangent point   

            zkval=&
             & LOG(prefrac(i,ik)/prefrac(i+1,ik))/ &
             & MAX(1.0_JPRB,(pnr(i+1,ik)-pnr(i,ik)))
            zkval = MAX(1.0E-6_JPRB,zkval)
   
            zref_low = prefrac(i,ik)*EXP(-zkval*(zavar-pnr(i,ik)))       
            zref_up  = prefrac(i+1,ik)

            ztlow = 0.0_JPRB
            ztup = SQRT(zkval*(znr_up-zavar))


! calculate the horizontal gradient - for the impact parameter change

   
            IF (iside == 1) THEN  ! LEO side 
    
              zdn_dtheta = 0.5E-6_JPRB* &
               & (prefrac(i,ik+1) + prefrac(i+1,ik+1) - &
               & prefrac(i,ik)   - prefrac(i+1,ik))/pdsep 


            ELSE 
    
              zdn_dtheta = 0.5E-6_JPRB* &
               & (prefrac(i,ik)   + prefrac(i+1,ik) - &
               & prefrac(i,ik-1) - prefrac(i+1,ik-1))/pdsep 

            ENDIF 
 
     
         ELSE


! not the first level


            ik=INT((ztheta + ztheta_tan)/pdsep)+1
            ik = MIN(MAX(1,ik),khoriz-1)
            ikp1 = MIN(khoriz,ik+1)

! horizontal weighting factor       
       
            zhwt1 = (REAL(ik)*pdsep - (ztheta+ztheta_tan))/pdsep 
            zhwt1 = MAX(MIN(1.0_JPRB,zhwt1),0.0_JPRB) 
            zhwt2 = 1.0_JPRB - zhwt1
    

!  interpolated nr value
               
            znr_low = zhwt1*pnr(i,ik) + zhwt2*pnr(i,ikp1)
            znr_up  = zhwt1*pnr(i+1,ik) + zhwt2*pnr(i+1,ikp1)

! intepolated refractivity value
      
            zref_low = zhwt1*prefrac(i,ik) + zhwt2*prefrac(i,ikp1)
            zref_up  = zhwt1*prefrac(i+1,ik) + zhwt2*prefrac(i+1,ikp1)


            zkval = LOG(zref_low/zref_up)/MAX(1.0_JPRB,(znr_up-znr_low))
            zkval = MAX(1.0E-6_JPRB,zkval)

            ztlow = 0.0_JPRB
            IF (znr_low > zavar) &
             &  ztlow = SQRT(zkval*(znr_low-zavar)) 
    
            ztup = 0.0_JPRB
            IF (znr_up > zavar) &    
             & ztup = SQRT(zkval*(znr_up-zavar))
  

! calculate the horizontal gradient - for the impact parameter change


            zdn_dtheta = 0.0_JPRB
   
    
            IF (ztheta > ztheta_min .AND. ztheta < ztheta_max) &
             & zdn_dtheta = 0.5E-6*(prefrac(i,ikp1)  + prefrac(i+1,ikp1) - &
             & prefrac(i,ik)    - prefrac(i+1,ik))/pdsep    
         
         ENDIF


! the bending over the element of path 


         IF (i == klev-1) THEN 
 
! this will handle the extrapolation above the model top
 
           zdiff_erf = 1.0_JPRB - fcserf(ztlow)  
     
         ELSE    

           zdiff_erf = fcserf(ztup) - fcserf(ztlow)

         ENDIF 
 
 
 
         zdalpha = 1.0E-6_JPRB*zRoot_halfPI*SQRT(zavar*zkval)* &
          & zref_low*EXP(zkval*(znr_low-zavar))*zdiff_erf


! total bending angle on current side


         zalpha_half(iside) = zalpha_half(iside) + zdalpha


! change in angle made with local radius vector


         zthi_up  = ASIN(MIN(1.0_JPRB,zavar/znr_up))
         zdthi = zthi_up - zthi_low

! For next calculation
 
         zthi_low = zthi_up
 

! change in angular position in plane

        
         zdtheta = zdalpha - zdthi  ! (dthi is always -ve, )
 
! radius and sinthi value used for path-length calculation, ds = rad_over_sinthi*dtheta.

                           
         zrad    = 0.5_JPRB*(pradius(i,ik)+pradius(i+1,ik))
         zsinthi = 0.5_JPRB*zavar*(1.0_JPRB/pnr(i,ik) + 1.0_JPRB/pnr(i+1,ik))
         zrad_over_sinthi = zrad/zsinthi


! element of path

         zds = zrad_over_sinthi*zdtheta

         IF (iside == 1) THEN
    
           ztheta = ztheta + zdtheta    ! LEO satellite side

           ztheta = MIN(ztheta,ztheta_max)

           zavar = zavar + zdn_dtheta*zds

         ELSE
    
           ztheta = ztheta - zdtheta  ! back towards the GPS transmitter

             ztheta = MAX(ztheta,ztheta_min)

             zavar = zavar - zdn_dtheta*zds

         ENDIF 
 
      ENDDO  ! i the layers
 
      pa_path(in,iside) = zavar

   ENDDO ! iside
   

   palpha(in) = zalpha_half(1) + zalpha_half(2)


ENDDO ! all the observations


IF (LHOOK) CALL DR_HOOK('GPSCALC_ALPHA2D',1,ZHOOK_HANDLE)


END SUBROUTINE gpscalc_alpha2d
