SUBROUTINE gpscalc_alpha2dad(kobs,        & ! no.of observations
                          &  klev,        & ! no. of vertical levels
                          &  khoriz,      & ! no. of horizontal layers  ODD
                          &  pdsep,       & ! the angular spacing 
                          &  pa,          & ! impact parameter values
                          &  prefrac,     & ! refractivity
                          &  prefrac_hat, &
                          &  pradius,     & ! radius values
                          &  pradius_hat, &
                          &  pnr,         & ! nr product
                          &  pnr_hat,     &
                          &  pa_path_hat, & ! the impact params at end points
                          &  palpha_hat)    ! 2d bending angles. 

!**** *gpscalc_alpha2dad* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2004/10/26
!     Based on a EUMETSAT GRAS-SAF routine.  

!     PURPOSE
!     -------
!     It evaluates the bending angle integral for each impact parameter

!**   INTERFACE
!     ---------

!     CALL gpscalc_alpha2dad(...)    

!     WHERE 

!     gpscalc_alpha2dad is called from gpsro_2dad

!**   METHOD
!     ------
!     1) calculate the exponential decay of refractivity with nr between model levels
!     2) evaluates the bending angle integral for each impact parameter

!**   EXTERNALS
!     ---------
!     1) fcserf calculates the Gaussian error function

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RPI


IMPLICIT NONE


! subroutine args. 


INTEGER(KIND=JPIM), INTENT(IN)  :: kobs           ! size of ob. vector
INTEGER(KIND=JPIM), INTENT(IN)  :: klev           ! no. of refractivity levels
INTEGER(KIND=JPIM), INTENT(IN)  :: khoriz         ! no. of horizontal locations
REAL(KIND=JPRB),    INTENT(IN)  :: pdsep           ! angular spacing of grid
REAL(KIND=JPRB),    INTENT(IN)  :: pa(kobs)        ! impact parameter - NOW assumed to be on pressure levels
REAL(KIND=JPRB),    INTENT(IN)  :: prefrac(klev,khoriz)   ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(INOUT)  :: prefrac_hat(klev,khoriz)   ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(IN)  :: pradius(klev,khoriz)   ! radius values
REAL(KIND=JPRB),    INTENT(INOUT)  :: pradius_hat(klev,khoriz)   ! radius values
REAL(KIND=JPRB),    INTENT(IN)  :: pnr(klev,khoriz)       ! index * radius product
REAL(KIND=JPRB),    INTENT(INOUT)  :: pnr_hat(klev,khoriz)       ! index * radius product
REAL(KIND=JPRB),    INTENT(INOUT) :: pa_path_hat(kobs,2)        
REAL(KIND=JPRB),    INTENT(INOUT) :: palpha_hat(kobs) ! 2d bending angles 

       

! local variables


INTEGER(KIND=JPIM) :: i,in,ibot,jbot,ikbot,iside,ik,ikp1
INTEGER(KIND=JPIM) :: icen
REAL(KIND=JPRB) :: zkval(klev-1)
REAL(KIND=JPRB) :: zkval_hat
REAL(KIND=JPRB) :: zroot_halfpi
REAL(KIND=JPRB) :: ztup(klev-1),ztlow(klev-1)
REAL(KIND=JPRB) :: ztup_hat,ztlow_hat
REAL(KIND=JPRB) :: fcserf,zdiff_erf(klev-1)
REAL(KIND=JPRB) :: zdiff_erf_hat
REAL(KIND=JPRB) :: zdalpha(klev-1)
REAL(KIND=JPRB) :: zdalpha_hat
REAL(KIND=JPRB) :: zdthi,zthi_up(klev-1),zthi_low(klev)
REAL(KIND=JPRB) :: zdthi_hat,zthi_up_hat,zthi_low_hat
REAL(KIND=JPRB) :: zrad(klev-1)
REAL(KIND=JPRB) :: zrad_hat
REAL(KIND=JPRB) :: zsinthi(klev-1),zrad_over_sinthi(klev-1)
REAL(KIND=JPRB) :: zsinthi_hat,zrad_over_sinthi_hat
REAL(KIND=JPRB) :: zavar(klev)
REAL(KIND=JPRB) :: zavar_hat
REAL(KIND=JPRB) :: zhwt1(klev-1),zhwt2(klev-1)
REAL(KIND=JPRB) :: zhwt1_hat,zhwt2_hat
REAL(KIND=JPRB) :: zdn_dtheta(klev-1)
REAL(KIND=JPRB) :: zdn_dtheta_hat
REAL(KIND=JPRB) :: zalpha_half(2)
REAL(KIND=JPRB) :: zalpha_half_hat(2)
REAL(KIND=JPRB) :: ztheta(klev),zdtheta(klev-1),ztheta_tan,ztheta_min,ztheta_max
REAL(KIND=JPRB) :: ztheta_hat,zdtheta_hat
REAL(KIND=JPRB) :: zref_low(klev-1),zref_up(klev-1)
REAL(KIND=JPRB) :: zref_low_hat,zref_up_hat
REAL(KIND=JPRB) :: znr_low(klev-1),znr_up(klev-1) 
REAL(KIND=JPRB) :: znr_low_hat,znr_up_hat
REAL(KIND=JPRB) :: zalpha(kobs) ! 2d bending angles 
REAL(KIND=JPRB) :: zds(klev-1),zds_hat
REAL(KIND=JPRB) :: zerf_up_hat,zerf_low_hat
REAL(KIND=JPRB) :: ZHOOK_HANDLE


#include "fcserf_ad.intfb.h"

IF (LHOOK) CALL DR_HOOK('GPSCALC_ALPHA2DAD',0,ZHOOK_HANDLE)



! the central profile icen


zalpha(:) = 0.0_JPRB

icen = khoriz/2 + 1
ztheta_tan = REAL(icen-1)*pdsep 
ztheta_min = -ztheta_tan
ztheta_max =  ztheta_tan


! cannot integrate through super refracting layer



jbot = 1


DO in = 1,khoriz

  DO

    IF ((prefrac(jbot,in) > 0.0_JPRB .AND. pnr(jbot,in) > 0.0_JPRB) .OR. &
    &jbot == klev) EXIT
  
    jbot = jbot + 1

  ENDDO

ENDDO

DO in = 1,khoriz

   ikbot = klev

   DO i=klev,jbot+1,-1

      IF (pnr(ikbot,in) < pnr(ikbot-1,in)) EXIT 

      ikbot = ikbot - 1

   ENDDO
 
   jbot = MAX(jbot,ikbot)

ENDDO



! initialise local adjoint variables



zalpha_half_hat(:) = 0.0_JPRB
zdalpha_hat = 0.0_JPRB
zdtheta_hat = 0.0_JPRB
ztheta_hat = 0.0_JPRB
zref_low_hat = 0.0_JPRB
zref_up_hat = 0.0_JPRB
znr_up_hat = 0.0_JPRB
znr_low_hat = 0.0_JPRB
ztup_hat = 0.0_JPRB
ztlow_hat = 0.0_JPRB
zavar_hat = 0.0_JPRB
zkval_hat = 0.0_JPRB
zds_hat = 0.0_JPRB
zdn_dtheta_hat = 0.0_JPRB
zsinthi_hat = 0.0_JPRB
zrad_hat = 0.0_JPRB
zrad_over_sinthi_hat = 0.0_JPRB
zhwt1_hat = 0.0_JPRB
zhwt2_hat = 0.0_JPRB
zdiff_erf_hat = 0.0_JPRB
zthi_up_hat = 0.0_JPRB
zthi_low_hat = 0.0_JPRB
zerf_up_hat = 0.0_JPRB
zerf_low_hat = 0.0_JPRB



! now calculate the bending angles


zRoot_halfPI = SQRT(0.5_JPRB*RPI)

DO in=kobs,1,-1

   IF (pa(in) < pnr(jbot,icen) .OR. pa(in) > pnr(klev,icen)) CYCLE  

   ibot = jbot

   DO 

      IF (pa(in) < pnr(ibot+1,icen)) EXIT   !

      ibot=ibot+1

   ENDDO
 

!  initialise bending angle value   

   zalpha_half_hat(1) = zalpha_half_hat(1) + palpha_hat(in)
   zalpha_half_hat(2) = zalpha_half_hat(2) + palpha_hat(in)
   palpha_hat(in) = 0.0_JPRB
   

! Forwrad bit
  
  
   DO iside = 2,1,-1 
    
            
      zavar(ibot) = pa(in)      
      ztheta(ibot) = 0.0_JPRB
      zthi_low(ibot) = 0.5_JPRB*RPI
      zhwt1(:) = 0.0_JPRB
      zhwt2(:) = 0.0_JPRB
            
      DO i = ibot, klev-1
     
         IF (i == ibot) THEN

            ik=icen 

            znr_low(i) = zavar(i)
            znr_up(i)  = pnr(i+1,ik)

! refractivity at tangent point   

            zkval(i)=LOG(prefrac(i,ik)/prefrac(i+1,ik))/(pnr(i+1,ik)-pnr(i,ik))
            zkval(i) = MAX(1.0E-6_JPRB,zkval(i))

            zref_low(i) = prefrac(i,ik)*EXP(-zkval(i)*(zavar(i)-pnr(i,ik)))
    
            zref_up(i)  = prefrac(i+1,ik)

            ztlow(i) = 0.0_JPRB
            ztup(i) = SQRT(zkval(i)*(znr_up(i)-zavar(i)))


! calculate the horizontal gradient - for the impact parameter change

   
            IF (iside == 1) THEN  ! LEO side 
   
              zdn_dtheta(i) = 0.5E-6_JPRB*(prefrac(i,ik+1) + prefrac(i+1,ik+1) - &
               &  prefrac(i,ik)   - prefrac(i+1,ik))/pdsep
   
            ELSE 
  
              zdn_dtheta(i) = 0.5E-6_JPRB*(prefrac(i,ik)   + prefrac(i+1,ik) - &
               &  prefrac(i,ik-1) - prefrac(i+1,ik-1))/pdsep
 
            ENDIF 
 
   
         ELSE


! not the first level


  
        ik=INT((ztheta(i) + ztheta_tan)/pdsep)+1
        ik = MIN(MAX(1,ik),khoriz-1)
        ikp1 = ik+1
       
! horizontal weighting factor       
       
        zhwt1(i) = (REAL(ik)*pdsep - (ztheta(i)+ztheta_tan))/pdsep
        zhwt1(i) = MAX(MIN(1.0_JPRB,zhwt1(i)),0.0_JPRB)
        zhwt2(i) = 1.0_JPRB - zhwt1(i)
   
!  interpolated nr value
               
        znr_low(i) = zhwt1(i)*pnr(i,ik) + zhwt2(i)*pnr(i,ikp1)
        znr_up(i)  = zhwt1(i)*pnr(i+1,ik) + zhwt2(i)*pnr(i+1,ikp1)

! intepolated refractivity value
      
        zref_low(i) = zhwt1(i)*prefrac(i,ik) + zhwt2(i)*prefrac(i,ikp1)
        zref_up(i)  = zhwt1(i)*prefrac(i+1,ik) + zhwt2(i)*prefrac(i+1,ikp1)

        zkval(i) = LOG(zref_low(i)/zref_up(i))/MAX(1.0_JPRB,(znr_up(i)-znr_low(i)))
            zkval(i) = MAX(1.0E-6_JPRB,zkval(i))
 
            ztlow(i) = 0.0_JPRB
        IF (znr_low(i) > zavar(i)) ztlow(i) = SQRT(zkval(i)*(znr_low(i)-zavar(i)))

          ztup(i) = 0.0_JPRB
          IF (znr_up(i) > zavar(i)) &
           &  ztup(i) = SQRT(zkval(i)*(znr_up(i)-zavar(i)))



! calculate the horizontal gradient - for the impact parameter change

   
          zdn_dtheta(i) = 0.0_JPRB
    
    
          IF (ztheta(i) > ztheta_min .AND. ztheta(i) < ztheta_max) &
           &    zdn_dtheta(i) = 0.5E-6_JPRB*(prefrac(i,ikp1)  + prefrac(i+1,ikp1) - &
           &    prefrac(i,ik)    - prefrac(i+1,ik))/pdsep



         ENDIF

        

! the bending over the element of path 


         IF (i == klev-1) THEN 

! this will handle the extrapolation above the model top

           zdiff_erf(i) = 1.0_JPRB - fcserf(ztlow(i))  
     
         ELSE    

           zdiff_erf(i) = fcserf(ztup(i)) - fcserf(ztlow(i))
     
         ENDIF 

                   
         zdalpha(i) = 1.0E-6_JPRB*zRoot_halfPI*SQRT(zavar(i)*zkval(i))* &
          &  zref_low(i)*EXP(zkval(i)*(znr_low(i)-zavar(i)))*zdiff_erf(i)


! testing
         zalpha(in) = zalpha(in) + zdalpha(i)

! total bending angle on current side


         zalpha_half(iside) = zalpha_half(iside) + zdalpha(i)

! change in angle made with local radius vector


         zthi_up(i)  = ASIN(MIN(1.0_JPRB,zavar(i)/znr_up(i)))
    
         zdthi = zthi_up(i) - zthi_low(i)
 

! For next calculation
 
         zthi_low(i+1) = zthi_up(i)
 

! change in angular position in plane

        
         zdtheta(i) = zdalpha(i) - zdthi   ! (dthi is always -ve, )


! radius and sinthi value used for path-length calculation, ds = rad_over_sinthi*dtheta.

                          
         zrad(i)    = 0.5_JPRB*(pradius(i,ik)+pradius(i+1,ik))

         zsinthi(i) = 0.5_JPRB*zavar(i)*(1.0_JPRB/pnr(i,ik) + 1.0_JPRB/pnr(i+1,ik))
 
         zrad_over_sinthi(i) = zrad(i)/zsinthi(i)

! element of path

         zds(i) = zrad_over_sinthi(i)*zdtheta(i)
 
         IF (iside == 1) THEN
    
             ztheta(i+1) = ztheta(i) + zdtheta(i)    ! LEO satellite side

             ztheta(i+1) = MIN(ztheta(i+1),ztheta_max)
    
             zavar(i+1) = zavar(i) + zdn_dtheta(i)*zds(i)

         ELSE
    
             ztheta(i+1) = ztheta(i) - zdtheta(i)  ! back towards the GPS transmitter

             ztheta(i+1) = MAX(ztheta(i+1),ztheta_min)
    
             zavar(i+1) = zavar(i) - zdn_dtheta(i)*zds(i)
     
         ENDIF 
         

      ENDDO  ! i the layers
     
     
      zavar_hat = zavar_hat + pa_path_hat(in,iside)
      pa_path_hat(in,iside) = 0.0_JPRB    

      DO i = klev-1,ibot,-1

      
         IF (iside == 1) THEN
   

            IF (ztheta(i) < ztheta_max) THEN
    
              zdtheta_hat = zdtheta_hat + ztheta_hat

              ztheta_hat = ztheta_hat
 

            ELSE
 
              ztheta_hat = 0.0_JPRB
 
            ENDIF
     
     
            zdn_dtheta_hat = zdn_dtheta_hat + zds(i)*zavar_hat
            zds_hat = zds_hat + zdn_dtheta(i)*zavar_hat
            zavar_hat = zavar_hat
 
  
         ELSE
    
     
         IF (ztheta(i) > ztheta_min) THEN
     
            zdtheta_hat = zdtheta_hat - ztheta_hat

            ztheta_hat = ztheta_hat 
      

         ELSE
     
            ztheta_hat = 0.0_JPRB
 
         ENDIF 
 
         zdn_dtheta_hat = zdn_dtheta_hat - zds(i)*zavar_hat
         zds_hat = zds_hat - zdn_dtheta(i)*zavar_hat
         zavar_hat = zavar_hat
     
     
         ENDIF 

          
         zdtheta_hat = zdtheta_hat + zrad_over_sinthi(i)*zds_hat
         zrad_over_sinthi_hat = zrad_over_sinthi_hat + zdtheta(i)*zds_hat
         zds_hat = 0.0_JPRB  


         zrad_hat = zrad_hat + zrad_over_sinthi_hat/zsinthi(i)
         zsinthi_hat = zsinthi_hat - zrad_over_sinthi(i)/zsinthi(i)* &
          &  zrad_over_sinthi_hat   
         zrad_over_sinthi_hat = 0.0_JPRB

! need to know k
 
 
         IF (i == ibot) THEN
 
           ik = icen
    
         ELSE
 
           ik=INT((ztheta(i) + ztheta_tan)/pdsep)+1
           ik = MIN(MAX(1,ik),khoriz-1)
   
         ENDIF 
 
         ikp1 = ik+1

         zavar_hat = zavar_hat + zsinthi(i)/zavar(i)*zsinthi_hat
         pnr_hat(i,ik) = pnr_hat(i,ik) - 0.5_JPRB*zavar(i)/pnr(i,ik)**2*zsinthi_hat
         pnr_hat(i+1,ik) = pnr_hat(i+1,ik) - 0.5_JPRB*zavar(i)/pnr(i+1,ik)**2*zsinthi_hat
         zsinthi_hat = 0.0_JPRB
      
      
         pradius_hat(i,ik) = pradius_hat(i,ik) + 0.5_JPRB*zrad_hat
         pradius_hat(i+1,ik) = pradius_hat(i+1,ik) + 0.5_JPRB*zrad_hat
         zrad_hat = 0.0_JPRB  

         zdalpha_hat = zdalpha_hat + zdtheta_hat
         zdthi_hat = zdthi_hat - zdtheta_hat
         zdtheta_hat = 0.0_JPRB

         zthi_up_hat = zthi_up_hat + zthi_low_hat
         zthi_low_hat = 0.0_JPRB
 
         zthi_up_hat = zthi_up_hat + zdthi_hat
         zthi_low_hat = zthi_low_hat - zdthi_hat
         zdthi_hat = 0.0_JPRB 
 
         IF (zavar(i) < znr_up(i)) THEN
 
            zavar_hat = zavar_hat + &
             &  zthi_up_hat/SQRT(znr_up(i)**2 - zavar(i)**2)

            znr_up_hat = znr_up_hat - &
             &  zavar(i)/(znr_up(i)*SQRT(znr_up(i)**2 - zavar(i)**2))*zthi_up_hat

            zthi_up_hat = 0.0_JPRB


         ENDIF
         
         zthi_up_hat = 0.0_JPRB 
 
         zdalpha_hat = zdalpha_hat + zalpha_half_hat(iside)
         zalpha_half_hat(iside) = zalpha_half_hat(iside)
      
         zref_low_hat = zref_low_hat + &
          &  zdalpha(i)/MAX(1.0E-10_JPRB,zref_low(i))*zdalpha_hat
       
         zdiff_erf_hat = zdiff_erf_hat+ &
          &  zdalpha(i)/MAX(1.0E-10_JPRB,zdiff_erf(i))*zdalpha_hat

         zavar_hat = zavar_hat + &
          &  zdalpha(i)*(0.5_JPRB/zavar(i)-zkval(i))*zdalpha_hat

         zkval_hat = zkval_hat + &
          &  zdalpha(i)*(znr_low(i) -zavar(i) + 0.5_JPRB/zkval(i))*zdalpha_hat


         znr_low_hat = znr_low_hat + zdalpha(i)*zkval(i)*zdalpha_hat
 
         zdalpha_hat = 0.0_JPRB 
 
         IF (i == klev-1) THEN 
 
! this will handle the extrapolation above the model top
  
           zerf_low_hat = zerf_low_hat - zdiff_erf_hat
           zdiff_erf_hat = 0.0_JPRB 
     
           CALL fcserf_ad(ztlow(i),ztlow_hat,zerf_low_hat)    
 
         ELSE    

           zerf_up_hat  = zerf_up_hat + zdiff_erf_hat
           zerf_low_hat = zerf_low_hat - zdiff_erf_hat
           zdiff_erf_hat = 0.0_JPRB

           CALL fcserf_ad(ztup(i),ztup_hat,zerf_up_hat)
           CALL fcserf_ad(ztlow(i),ztlow_hat,zerf_low_hat) 
 

         ENDIF 


         IF (i == ibot) THEN

           IF (iside == 1) THEN  ! LEO side 
    
             prefrac_hat(i,ikp1) = prefrac_hat(i,ikp1) + &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep

             prefrac_hat(i+1,ikp1) = prefrac_hat(i+1,ikp1) + &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep
 
             prefrac_hat(i,ik) = prefrac_hat(i,ik) - &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep

             prefrac_hat(i+1,ik) = prefrac_hat(i+1,ik) - &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep

             zdn_dtheta_hat = 0.0_JPRB
    
 
           ELSE 
    
             prefrac_hat(i,ik) = prefrac_hat(i,ik) + &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep

             prefrac_hat(i+1,ik) = prefrac_hat(i+1,ik) + &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep
 
             prefrac_hat(i,ik-1) = prefrac_hat(i,ik-1) - &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep

             prefrac_hat(i+1,ik-1) = prefrac_hat(i+1,ik-1) - &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep

             zdn_dtheta_hat = 0.0_JPRB   
   
 
  
           ENDIF 

 
           zkval_hat = zkval_hat + &
            &  0.5_JPRB*(znr_up(i)-zavar(i))/ztup(i)*ztup_hat
      
           znr_up_hat = znr_up_hat + &
            &  0.5_JPRB*zkval(i)/ztup(i)*ztup_hat
  
           zavar_hat = zavar_hat - 0.5_JPRB*zkval(i)/ztup(i)*ztup_hat
       
           ztup_hat = 0.0_JPRB
       
           ztlow_hat = 0.0_JPRB
 
           prefrac_hat(i+1,ik) = prefrac_hat(i+1,ik) + zref_up_hat
           zref_up_hat = 0.0_JPRB
      
           prefrac_hat(i,ik) = prefrac_hat(i,ik) + &
            & zref_low(i)/prefrac(i,ik)*zref_low_hat

           zkval_hat = zkval_hat - zref_low(i)*(zavar(i)-pnr(i,ik))* &
            & zref_low_hat

           pnr_hat(i,ik) = pnr_hat(i,ik)+zref_low(i)*zkval(i)*zref_low_hat
   
           zavar_hat = zavar_hat - zref_low(i)*zkval(i)*zref_low_hat
           zref_low_hat = 0.0_JPRB
 

           IF (zkval(i) > 1.0E-6_JPRB)  THEN 
                  
             pnr_hat(i,ik) = pnr_hat(i,ik) + &
              &  zkval(i)/MAX(1.0_JPRB,(pnr(i+1,ik)-pnr(i,ik)))*zkval_hat
    
             pnr_hat(i+1,ik) = pnr_hat(i+1,ik) - &
              &  zkval(i)/MAX(1.0_JPRB,(pnr(i+1,ik)-pnr(i,ik)))*zkval_hat
 
             prefrac_hat(i,ik) = prefrac_hat(i,ik) + &
              &  zkval_hat/(prefrac(i,ik)*MAX(1.0_JPRB,(pnr(i+1,ik)-pnr(i,ik))))
     
             prefrac_hat(i+1,ik) = prefrac_hat(i+1,ik) - &
              &  zkval_hat/(prefrac(i+1,ik)*MAX(1.0_JPRB,(pnr(i+1,ik)-pnr(i,ik))))
     
             zkval_hat = 0.0_JPRB
              

           ENDIF
    
           zkval_hat = 0.0_JPRB

           pnr_hat(i+1,ik) = pnr_hat(i+1,ik) + znr_up_hat
           znr_up_hat = 0.0_JPRB
    
           zavar_hat = zavar_hat + znr_low_hat
           znr_low_hat = 0.0_JPRB
 
 
         ELSE
 
           IF (ztheta(i) > ztheta_min .AND. ztheta(i) < ztheta_max) THEN
   
             prefrac_hat(i,ikp1) = prefrac_hat(i,ikp1) + &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep
     
             prefrac_hat(i+1,ikp1) = prefrac_hat(i+1,ikp1) + &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep
     
             prefrac_hat(i,ik) = prefrac_hat(i,ik) - &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep
 
             prefrac_hat(i+1,ik) = prefrac_hat(i+1,ik) - &
              &  0.5E-6_JPRB*zdn_dtheta_hat/pdsep

             zdn_dtheta_hat = 0.0_JPRB

           ENDIF  

           zdn_dtheta_hat = 0.0_JPRB 
 
           IF (znr_up(i) > zavar(i)) THEN
    
             zkval_hat = zkval_hat + &
              &  0.5_JPRB*(znr_up(i)-zavar(i))/ztup(i)*ztup_hat
      
             znr_up_hat = znr_up_hat + &
              &  0.5_JPRB*zkval(i)/ztup(i)*ztup_hat
  
             zavar_hat = zavar_hat - 0.5_JPRB*zkval(i)/ztup(i)*ztup_hat
       
             ztup_hat = 0.0_JPRB
 
           ENDIF 
    
           ztup_hat = 0.0_JPRB
    
           IF (znr_low(i) > zavar(i) ) THEN
    
             zkval_hat = zkval_hat + &
              &  0.5_JPRB*(znr_low(i)-zavar(i))/ztlow(i)*ztlow_hat
      
             znr_low_hat = znr_low_hat + &
              &  0.5_JPRB*zkval(i)/ztlow(i)*ztlow_hat
  
             zavar_hat = zavar_hat - 0.5_JPRB*zkval(i)/ztlow(i)*ztlow_hat
       
             ztlow_hat = 0.0_JPRB
    

           ENDIF  
 
 
           ztlow_hat =0.0_JPRB 
 
           IF (zkval(i) > 1.0E-6_JPRB)  THEN
               
             znr_low_hat = znr_low_hat + &
              &  zkval(i)/MAX(1.0_JPRB,(znr_up(i)-znr_low(i)))*zkval_hat
     
             znr_up_hat = znr_up_hat - &
              &  zkval(i)/MAX(1.0_JPRB,(znr_up(i)-znr_low(i)))*zkval_hat
 
             zref_low_hat = zref_low_hat + &
              &  zkval_hat/(zref_low(i)*MAX(1.0_JPRB,(znr_up(i)-znr_low(i))))
     
             zref_up_hat = zref_up_hat - &
              &  zkval_hat/(zref_up(i)*MAX(1.0_JPRB,(znr_up(i)-znr_low(i))))
     
               zkval_hat = 0.0_JPRB
              

           ENDIF
    
           zkval_hat = 0.0_JPRB
 
           zhwt1_hat = zhwt1_hat + prefrac(i+1,ik)*zref_up_hat
           prefrac_hat(i+1,ik) = prefrac_hat(i+1,ik) + zhwt1(i)*zref_up_hat
           zhwt2_hat = zhwt2_hat + prefrac(i+1,ikp1)*zref_up_hat
           prefrac_hat(i+1,ikp1) = prefrac_hat(i+1,ikp1) + zhwt2(i)*zref_up_hat
           zref_up_hat=0.0_JPRB

 
           zhwt1_hat = zhwt1_hat + prefrac(i,ik)*zref_low_hat
           prefrac_hat(i,ik) = prefrac_hat(i,ik) + zhwt1(i)*zref_low_hat
           zhwt2_hat = zhwt2_hat + prefrac(i,ikp1)*zref_low_hat
           prefrac_hat(i,ikp1) = prefrac_hat(i,ikp1) + zhwt2(i)*zref_low_hat
           zref_low_hat=0.0_JPRB

            
           zhwt1_hat = zhwt1_hat + pnr(i+1,ik)*znr_up_hat
           pnr_hat(i+1,ik) = pnr_hat(i+1,ik) + zhwt1(i)*znr_up_hat
           zhwt2_hat = zhwt2_hat + pnr(i+1,ikp1)*znr_up_hat
           pnr_hat(i+1,ikp1) = pnr_hat(i+1,ikp1) +zhwt2(i)*znr_up_hat
           znr_up_hat=0.0_JPRB


           zhwt1_hat = zhwt1_hat + pnr(i,ik)*znr_low_hat
           pnr_hat(i,ik) = pnr_hat(i,ik) + zhwt1(i)*znr_low_hat
           zhwt2_hat = zhwt2_hat + pnr(i,ikp1)*znr_low_hat
           pnr_hat(i,ikp1) = pnr_hat(i,ikp1) +zhwt2(i)*znr_low_hat
           znr_low_hat=0.0_JPRB


           zhwt1_hat = zhwt1_hat - zhwt2_hat
           zhwt2_hat = 0.0_JPRB
    
           IF (zhwt1(i) > 0.0_JPRB .AND. zhwt1(i) < 1.0_JPRB) THEN
    
             ztheta_hat = ztheta_hat - zhwt1_hat/pdsep
             zhwt1_hat = 0.0_JPRB
    
           ELSE
    
             zhwt1_hat = 0.0_JPRB
       
           ENDIF        
  
         ENDIF
      
      ENDDO  ! layers

      ztheta_hat = 0.0_JPRB
      zthi_low_hat = 0.0_JPRB      
      zavar_hat = 0.0_JPRB
      
   ENDDO  ! sides
   
   zalpha_half_hat(:) = 0.0_JPRB
   
ENDDO ! all the observations

palpha_hat(:) = 0.0_JPRB
pa_path_hat(:,:) = 0.0_JPRB


IF (LHOOK) CALL DR_HOOK('GPSCALC_ALPHA2DAD',1,ZHOOK_HANDLE)


END SUBROUTINE gpscalc_alpha2dad
