SUBROUTINE gpscalc_alpha2dtl(kobs,   & ! no.of observations
                           & klev,   & ! no. of vertical levels
                           & khoriz, & ! no. of horizontal layers  ODD
                           & pdsep,   & ! the angular spacing 
                           & pa,      & ! impact parameter values
                           & prefrac, & ! refractivity
                           & prefrac_prime, &
                           & pradius, & ! radius values
                           & pradius_prime, &
                           & pnr,     & ! nr product
                           & pnr_prime, &
                           & pa_path_prime, & ! the impact params at end points
                           & palpha_prime)    ! 2d bending angles. 


!**** *gpscalc_alpha2dtl* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2004/10/26
!     Based on a EUMETSAT GRAS-SAF routine that was Dr'd automatically by the CY27 merge.  

!     PURPOSE
!     -------
!     It evaluates the bending angle integral for each impact parameter

!**   INTERFACE
!     ---------

!     CALL gpscalc_alpha2dtl()    

!     WHERE 

!     gpscalc_alpha2dtl is called from gpsro_2dtl

!**   METHOD
!     ------
!     1) calculate the exponential decay of refractivity with nr between model levels
!     2) evaluates the bending angle integral for each impact parameter

!**   EXTERNALS
!     ---------
!     1) fcserf calculates the Gaussian error function

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS  , ONLY : RMDI 
USE YOMCST   , ONLY : RPI

IMPLICIT NONE

! subroutine args. 

INTEGER(KIND=JPIM), INTENT(IN)  :: kobs           ! size of ob. vector
INTEGER(KIND=JPIM), INTENT(IN)  :: klev           ! no. of refractivity levels
INTEGER(KIND=JPIM), INTENT(IN)  :: khoriz         ! no. of horizontal locations
REAL(KIND=JPRB),    INTENT(IN)  :: pdsep           ! angular spacing of grid
REAL(KIND=JPRB),    INTENT(IN)  :: pa(kobs)        ! impact parameter - NOW assumed to be on pressure levels
REAL(KIND=JPRB),    INTENT(IN)  :: prefrac(klev,khoriz)   ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(IN)  :: prefrac_prime(klev,khoriz)   ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(IN)  :: pradius(klev,khoriz)   ! radius values
REAL(KIND=JPRB),    INTENT(IN)  :: pradius_prime(klev,khoriz)   ! radius values
REAL(KIND=JPRB),    INTENT(IN)  :: pnr(klev,khoriz)       ! index * radius product
REAL(KIND=JPRB),    INTENT(IN)  :: pnr_prime(klev,khoriz)       ! index * radius product
REAL(KIND=JPRB),    INTENT(OUT) :: pa_path_prime(kobs,2)        
REAL(KIND=JPRB),    INTENT(OUT) :: palpha_prime(kobs) ! 2d bending angles 

       

! local variables


INTEGER(KIND=JPIM) :: i,in,ibot,jbot,ikbot,iside,ik,ikp1
INTEGER(KIND=JPIM) :: icen
REAL(KIND=JPRB) :: zkval
REAL(KIND=JPRB) :: zkval_prime
REAL(KIND=JPRB) :: zroot_halfpi
REAL(KIND=JPRB) :: ztup,ztlow
REAL(KIND=JPRB) :: ztup_prime,ztlow_prime
REAL(KIND=JPRB) :: fcserf,zdiff_erf
REAL(KIND=JPRB) :: zdiff_erf_prime
REAL(KIND=JPRB) :: zdalpha
REAL(KIND=JPRB) :: zdalpha_prime
REAL(KIND=JPRB) :: zdthi,zthi_up,zthi_low
REAL(KIND=JPRB) :: zdthi_prime,zthi_up_prime,zthi_low_prime
REAL(KIND=JPRB) :: zrad
REAL(KIND=JPRB) :: zrad_prime
REAL(KIND=JPRB) :: zsinthi,zrad_over_sinthi
REAL(KIND=JPRB) :: zsinthi_prime,zrad_over_sinthi_prime
REAL(KIND=JPRB) :: zavar
REAL(KIND=JPRB) :: zavar_prime
REAL(KIND=JPRB) :: zhwt1,zhwt2
REAL(KIND=JPRB) :: zhwt1_prime,zhwt2_prime
REAL(KIND=JPRB) :: zdn_dtheta
REAL(KIND=JPRB) :: zdn_dtheta_prime
REAL(KIND=JPRB) :: zalpha_half(2)
REAL(KIND=JPRB) :: zalpha_half_prime(2)
REAL(KIND=JPRB) :: ztheta,zdtheta,ztheta_tan,ztheta_min,ztheta_max
REAL(KIND=JPRB) :: ztheta_prime,zdtheta_prime
REAL(KIND=JPRB) :: zref_low,zref_up
REAL(KIND=JPRB) :: zref_low_prime,zref_up_prime
REAL(KIND=JPRB) :: znr_low,znr_up 
REAL(KIND=JPRB) :: znr_low_prime,znr_up_prime
REAL(KIND=JPRB) :: za_path(kobs,2)        
REAL(KIND=JPRB) :: zalpha(kobs) ! 2d bending angles 
REAL(KIND=JPRB) :: zds,zds_prime
REAL(KIND=JPRB) :: zerf_up_prime,zerf_low_prime
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcserf_tl.intfb.h"

IF (LHOOK) CALL DR_HOOK('GPSCALC_ALPHA2DTL',0,ZHOOK_HANDLE)



! the central profile kcen


icen = khoriz/2 + 1
ztheta_tan = REAL(icen-1)*pdsep 
ztheta_min = -ztheta_tan
ztheta_max =  ztheta_tan


! cannot integrate through super refracting layer



jbot = 1

DO in = 1,khoriz

  DO

    IF ((prefrac(jbot,in) > 0.0_JPRB .AND. pnr(jbot,in) > 0.0_JPRB) .OR. &
    &jbot == klev) EXIT
  
    jbot = jbot + 1

  ENDDO

ENDDO

DO in = 1,khoriz

   ikbot = klev

   DO i=klev,jbot+1,-1

      IF (pnr(ikbot,in) < pnr(ikbot-1,in)) EXIT 

      ikbot = ikbot - 1

   ENDDO
 
   jbot = MAX(jbot,ikbot)

ENDDO


! set the outputs to missing


zalpha(:)=RMDI
palpha_prime(:) = 0.0_JPRB
za_path(:,:)= RMDI
pa_path_prime(:,:)=0.0_JPRB


! now calculate the bending angles


zRoot_halfPI = SQRT(0.5_JPRB*RPI)

DO in=1,kobs

   IF (pa(in) < pnr(jbot,icen) .OR. pa(in) > pnr(klev,icen)) CYCLE  

   ibot = jbot

   DO 

      IF (pa(in) < pnr(ibot+1,icen)) EXIT   !

      ibot=ibot+1

   ENDDO
  

!  initialise bending angle value   

   zalpha_half(:) = 0.0_JPRB
   zalpha_half_prime(:) = 0.0_JPRB

   DO iside = 1,2 
    
      zavar = pa(in) 
      zavar_prime = 0.0_JPRB
      
      ztheta = 0.0_JPRB
      ztheta_prime = 0.0_JPRB
      
      zthi_low = 0.5_JPRB*RPI
      zthi_low_prime = 0.0_JPRB
      
      
      DO i = ibot, klev-1
      
         IF (i == ibot) THEN

            ik=icen 

            znr_low = zavar
            znr_low_prime = zavar_prime
     
            znr_up  = pnr(i+1,ik)
            znr_up_prime = pnr_prime(i+1,ik)

! refractivity at tangent point   

            zkval=LOG(prefrac(i,ik)/prefrac(i+1,ik))/ &
             &  MAX(1.0_JPRB,(pnr(i+1,ik)-pnr(i,ik)))
            zkval = MAX(1.0E-6_JPRB,zkval)
            zkval_prime = 0.0_JPRB
            IF (zkval > 1.0E-6_JPRB) &
             &  zkval_prime = ((zkval*(pnr_prime(i,ik)-pnr_prime(i+1,ik))) + &
             &  (prefrac_prime(i,ik)/prefrac(i,ik)-              &
             &  prefrac_prime(i+1,ik)/prefrac(i+1,ik)))/        &
             &  MAX(1.0_JPRB,(pnr(i+1,ik)-pnr(i,ik)))


            zref_low = prefrac(i,ik)*EXP(-zkval*(zavar-pnr(i,ik)))
    
            zref_low_prime = zref_low*               &
             &  (prefrac_prime(i,ik)/prefrac(i,ik) -  &
             &  zkval_prime*(zavar-pnr(i,ik)) + &
             &  zkval*(pnr_prime(i,ik)-zavar_prime))     ! new bit

    
 
            zref_up  = prefrac(i+1,ik)
            zref_up_prime = prefrac_prime(i+1,ik)

            ztlow = 0.0_JPRB
            ztlow_prime = 0.0_JPRB

            ztup = SQRT(zkval*(znr_up-zavar))
            ztup_prime = 0.5_JPRB*(zkval_prime*(znr_up -zavar) + &
             &  zkval*(znr_up_prime-zavar_prime))/ztup   ! new bit


! calculate the horizontal gradient - for the impact parameter change

   
            IF (iside == 1) THEN  ! LEO side 
    
              zdn_dtheta = 0.5E-6_JPRB*(prefrac(i,ik+1) + prefrac(i+1,ik+1) - &
               &  prefrac(i,ik)   - prefrac(i+1,ik))/pdsep
   
              zdn_dtheta_prime = &
               &  0.5E-6_JPRB*(prefrac_prime(i,ik+1) + prefrac_prime(i+1,ik+1) - &
               &  prefrac_prime(i,ik)   - prefrac_prime(i+1,ik))/pdsep
 
 
            ELSE 
   
              zdn_dtheta = 0.5E-6_JPRB*(prefrac(i,ik)   + prefrac(i+1,ik) - &
               &  prefrac(i,ik-1) - prefrac(i+1,ik-1))/pdsep
 
              zdn_dtheta_prime = &
               &  0.5E-6_JPRB*(prefrac_prime(i,ik)   + prefrac_prime(i+1,ik) - &
               &  prefrac_prime(i,ik-1) - prefrac_prime(i+1,ik-1))/pdsep
 

            ENDIF 
    
      
         ELSE


! not the first level

 
 
            ik=INT((ztheta + ztheta_tan)/pdsep)+1
            ik = MIN(MAX(1,ik),khoriz-1)
            ikp1 = ik+1
       
! horizontal weighting factor      
       
            zhwt1 = (REAL(ik)*pdsep - (ztheta+ztheta_tan))/pdsep
            zhwt1 = MAX(MIN(1.0_JPRB,zhwt1),0.0_JPRB)
   
            IF (zhwt1 > 0.0_JPRB .AND. zhwt1 < 1.0_JPRB) THEN
  
               zhwt1_prime = - ztheta_prime/pdsep

            ELSE
    
               zhwt1_prime = 0.0_JPRB
       
            ENDIF        
       
        
            zhwt2 = 1.0_JPRB - zhwt1
            zhwt2_prime = - zhwt1_prime
   
   
!  interpolated nr value
               
            znr_low = zhwt1*pnr(i,ik) + zhwt2*pnr(i,ikp1)
            znr_low_prime = zhwt1_prime*pnr(i,ik) + &
             &  zhwt1*pnr_prime(i,ik) + &
             &  zhwt2_prime*pnr(i,ikp1) + &
             &  zhwt2*pnr_prime(i,ikp1) 
    
    
            znr_up  = zhwt1*pnr(i+1,ik) + zhwt2*pnr(i+1,ikp1)
            znr_up_prime = zhwt1_prime*pnr(i+1,ik) + &
             &  zhwt1*pnr_prime(i+1,ik) + &
             &  zhwt2_prime*pnr(i+1,ikp1) + &
             &  zhwt2*pnr_prime(i+1,ikp1)
 

! intepolated refractivity value
       
            zref_low = zhwt1*prefrac(i,ik) + zhwt2*prefrac(i,ikp1)
            zref_low_prime = zhwt1_prime*prefrac(i,ik) + &
             &  zhwt1*prefrac_prime(i,ik) + &
             &  zhwt2_prime*prefrac(i,ikp1) + &
             &  zhwt2*prefrac_prime(i,ikp1)
    
    
    
            zref_up  = zhwt1*prefrac(i+1,ik) + zhwt2*prefrac(i+1,ikp1)
            zref_up_prime = zhwt1_prime*prefrac(i+1,ik) + &
             &  zhwt1*prefrac_prime(i+1,ik) + &
             &  zhwt2_prime*prefrac(i+1,ikp1) + &
             &  zhwt2*prefrac_prime(i+1,ikp1)


            zkval = LOG(zref_low/zref_up)/MAX(1.0_JPRB,(znr_up-znr_low))
            zkval = MAX(1.0E-6_JPRB,zkval)
    
    
            zkval_prime = 0.0_JPRB
            IF (zkval > 1.0E-6_JPRB) &
             & zkval_prime = ((zkval*(znr_low_prime-znr_up_prime)) + &
             &  (zref_low_prime/zref_low-              &
             &   zref_up_prime/zref_up))/        &
             &   MAX(1.0_JPRB,(znr_up-znr_low))
   
            ztlow = 0.0_JPRB
            ztlow_prime = 0.0_JPRB
     
            IF (znr_low > zavar) ztlow = SQRT(zkval*(znr_low-zavar))
            IF (znr_low > zavar ) &
             &  ztlow_prime = 0.5_JPRB*(zkval_prime*(znr_low-zavar) + &
             &  zkval*(znr_low_prime-zavar_prime))/ztlow     
    
            ztup = 0.0_JPRB
            ztup_prime = 0.0_JPRB
 
            IF (znr_up > zavar) ztup = SQRT(zkval*(znr_up-zavar))
            IF (znr_up > zavar) &
             &  ztup_prime = 0.5_JPRB*(zkval_prime*(znr_up -zavar) + &
             &  zkval*(znr_up_prime-zavar_prime))/ztup   ! new bit


! calculate the horizontal gradient - for the impact parameter change

    
            zdn_dtheta = 0.0_JPRB
            zdn_dtheta_prime = 0.0_JPRB
    
            IF (ztheta > ztheta_min .AND. ztheta < ztheta_max) THEN
    
               zdn_dtheta = 0.5E-6_JPRB*(prefrac(i,ikp1)  + prefrac(i+1,ikp1) - &
                &  prefrac(i,ik)    - prefrac(i+1,ik))/pdsep

               zdn_dtheta_prime = 0.5E-6_JPRB*&
                &  (prefrac_prime(i,ikp1)  + prefrac_prime(i+1,ikp1) - &
                &  prefrac_prime(i,ik)    - prefrac_prime(i+1,ik))/pdsep
 

            ENDIF  


         ENDIF

       

! the bending over the element of path 


         IF (i == klev-1) THEN 

! this will handle the extrapolation above the model top

           zdiff_erf = 1.0_JPRB - fcserf(ztlow)  

           CALL fcserf_tl(ztlow,ztlow_prime,zerf_low_prime)

           zdiff_erf_prime = - zerf_low_prime 

         ELSE    

           zdiff_erf = fcserf(ztup) - fcserf(ztlow)
     
           CALL fcserf_tl(ztup,ztup_prime,zerf_up_prime) 
           CALL fcserf_tl(ztlow,ztlow_prime,zerf_low_prime)
           zdiff_erf_prime = zerf_up_prime - zerf_low_prime
    
         ENDIF 
 
 
         zdalpha = 1.0E-6_JPRB*zRoot_halfPI*SQRT(zavar*zkval)* &
          &  zref_low*EXP(zkval*(znr_low-zavar))*zdiff_erf
  
         zdalpha_prime = zdalpha*(  &
          &  zref_low_prime/MAX(1.0E-10_JPRB,zref_low)  + &
          &  zdiff_erf_prime/MAX(1.0E-10_JPRB,zdiff_erf) + &
          &  (0.5_JPRB/zavar - zkval)*zavar_prime + &
          &  (znr_low -zavar + 0.5_JPRB/zkval)*zkval_prime + &
          &  zkval*znr_low_prime ) 



! total bending angle on current side


         zalpha_half(iside) = zalpha_half(iside) + zdalpha

         zalpha_half_prime(iside) = &
          &  zalpha_half_prime(iside) + zdalpha_prime


! change in angle made with local radius vector


   
         zthi_up  = ASIN(MIN(1.0_JPRB,zavar/znr_up))
         zthi_up_prime = 0.0_JPRB
 
         IF (zavar < znr_up) zthi_up_prime = &
          &  zavar_prime/SQRT(znr_up**2 - zavar**2) -  &                ! new line
          &  zavar/(znr_up*SQRT(znr_up**2 - zavar**2))*znr_up_prime

    
         zdthi = zthi_up - zthi_low
 
         zdthi_prime = zthi_up_prime - zthi_low_prime

! For next calculation 
 
         zthi_low = zthi_up
         zthi_low_prime = zthi_up_prime
 

! change in angular position in plane

        
         zdtheta = zdalpha - zdthi ! (dthi is always -ve, )
         zdtheta_prime = zdalpha_prime - zdthi_prime
 
! radius and sinthi value used for path-length calculation, ds = rad_over_sinthi*dtheta.

                           
         zrad    = 0.5_JPRB*(pradius(i,ik)+pradius(i+1,ik))
         zrad_prime = 0.5_JPRB*(pradius_prime(i,ik)+pradius_prime(i+1,ik))

         zsinthi = 0.5_JPRB*zavar*(1.0_JPRB/pnr(i,ik) + 1.0_JPRB/pnr(i+1,ik)) 
         zsinthi_prime = zsinthi/zavar*zavar_prime - &
          &  0.5_JPRB*zavar*(1.0_JPRB/pnr(i,ik)**2*pnr_prime(i,ik) + &
          &  1.0_JPRB/pnr(i+1,ik)**2*pnr_prime(i+1,ik))
 
         zrad_over_sinthi = zrad/zsinthi
 
         zrad_over_sinthi_prime = zrad_prime/zsinthi - &
          &  zrad_over_sinthi/zsinthi * zsinthi_prime

! element of path 
 
         zds = zrad_over_sinthi*zdtheta
          

         zds_prime = zrad_over_sinthi*zdtheta_prime + &
          &  zdtheta*zrad_over_sinthi_prime
        


         IF (iside == 1) THEN
    
           ztheta = ztheta + zdtheta    ! LEO satellite side

           ztheta = MIN(ztheta,ztheta_max)

           IF (ztheta < ztheta_max) THEN
     
             ztheta_prime = ztheta_prime + zdtheta_prime

           ELSE
     
             ztheta_prime = 0.0_JPRB
 
           ENDIF 
     
     
           zavar = zavar + zdn_dtheta*zds

           zavar_prime = zavar_prime + &
            &  zdn_dtheta_prime*zds + &
            &  zdn_dtheta*zds_prime
  
         ELSE
    
           ztheta = ztheta - zdtheta ! back towards the GPS transmitter

           ztheta = MAX(ztheta,ztheta_min)
     
           IF (ztheta > ztheta_min) THEN
     
             ztheta_prime = ztheta_prime - zdtheta_prime

           ELSE
     
             ztheta_prime = 0.0_JPRB
 
           ENDIF 
         

           zavar = zavar - zdn_dtheta*zds
     
           zavar_prime = zavar_prime - &
            &  zdn_dtheta_prime*zds - &
            &  zdn_dtheta*zds_prime
  
     
         ENDIF 

      ENDDO  ! i the layers
 
      za_path(in,iside) = zavar

      pa_path_prime(in,iside) = zavar_prime
         

   ENDDO ! iside

! total bending angle

   zalpha(in) = zalpha_half(1) + zalpha_half(2)   
   
! perturbed b.a.    
   
   palpha_prime(in) = zalpha_half_prime(1) + zalpha_half_prime(2)
    

ENDDO ! all the observations


IF (LHOOK) CALL DR_HOOK('GPSCALC_ALPHA2DTL',1,ZHOOK_HANDLE)


END SUBROUTINE gpscalc_alpha2dtl
