!**************************************************************
! calculate the bending angle "alpha" for impact parameters a

SUBROUTINE gpscalc_alphaRKm2tl(kobs,    & ! no.of observations
                             & klev,    & ! no. of vertical levels
                             & khoriz,  & ! no. of horizontal layers  ODD
                             & ksplit,  &
                             & pdsep,   & ! the angular spacing 
                             & pa,      & ! impact parameter values
                             & prefrac, & ! refractivity
                             & prefrac_prime, &
                             & pradius, & ! radius values
                             & pradius_prime, &
                             & pnr,     &
                             & pnr_prime, &
                             & proc, &
                             & pz_2d, &
                             & pa_path_prime, &
                             & palpha_prime) ! partial path length along rays


!**** *gpscalc_alpharkm2* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2007/12/11
!     Based on a GRAS-SAF routine.   

!     PURPOSE
!     -------
!     It evaluates the bending angle integral for each impact parameter

!**   INTERFACE
!     ---------

!     CALL gpscalc_alpharkm2(....)    

!     WHERE kobs   =  number of observed bending angles 
!           klev   =  number of vertical levels
!           khoriz =  number of horizontal locations
!           ksplit = splitting of model levels
!           pdsep  =  angular spacing
!           pa     =  impact parameters
!           prefrac=  refractivity values
!           pradius=  radius values
!           pnr    =  nr product 
!           pa_path= impact parameter at end points of ray path
!           palpha  = bending angle values 


!     gpscalc_alphark is called from gpsro_op2d

!**   METHOD
!     ------
!     1) calculate the ray bending angle for each impact param - using Runge-Kutta 
!    

!**   EXTERNALS
!     ---------
!     1) gpspderivs evaluates the derivs for the Runge-Kutta

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2005/08/01 : Original
!     -------------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS  , ONLY : RMDI 

USE YOMCST   , ONLY : RPI


IMPLICIT NONE


! subroutine args. 


INTEGER(KIND=JPIM), INTENT(IN)  :: kobs           ! size of ob. vector
INTEGER(KIND=JPIM), INTENT(IN)  :: klev           ! no. of refractivity levels
INTEGER(KIND=JPIM), INTENT(IN)  :: khoriz         ! no. of horizontal locations
INTEGER(KIND=JPIM), INTENT(IN)  :: ksplit
REAL(KIND=JPRB),    INTENT(IN)  :: pdsep           ! angular spacing of grid
REAL(KIND=JPRB),    INTENT(IN)  :: pa(kobs)        ! impact parameter - NOW assumed to be on pressure levels
REAL(KIND=JPRB),    INTENT(IN)  :: prefrac(klev,khoriz)   ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(IN)  :: prefrac_prime(klev,khoriz)   ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(IN)  :: pradius(klev,khoriz)   ! radius values
REAL(KIND=JPRB),    INTENT(IN)  :: pradius_prime(klev,khoriz)   ! radius values
REAL(KIND=JPRB),    INTENT(IN)  :: pnr(klev,khoriz)
REAL(KIND=JPRB),    INTENT(IN)  :: pnr_prime(klev,khoriz)
REAL(KIND=JPRB),    INTENT(IN)  :: proc
REAL(KIND=JPRB),    INTENT(IN)  :: pz_2d
REAL(KIND=JPRB),    INTENT(OUT) :: pa_path_prime(kobs,2)        
REAL(KIND=JPRB),    INTENT(OUT) :: palpha_prime(kobs)   ! path length
                       

! local variables


INTEGER(KIND=JPIM) :: i,j,in,im,ibot,jbot,ikbot,iside,ik,ikp1,in_2d
INTEGER(KIND=JPIM) :: ikcen
REAL(KIND=JPRB), PARAMETER :: zhmax = 3.0E4_JPRB
REAL(KIND=JPRB), PARAMETER :: zhmin = 1.0E2_JPRB
REAL(KIND=JPRB) :: zrad,zdndr
REAL(KIND=JPRB) :: zrad_prime,zdndr_prime
REAL(KIND=JPRB) :: zhwt1,zhwt2
REAL(KIND=JPRB) :: zhwt1_prime,zhwt2_prime
REAL(KIND=JPRB) :: zamult
REAL(KIND=JPRB) :: zh,zh2,zhuse,zhnew,zh_up
REAL(KIND=JPRB) :: zh_prime,zh2_prime,zhuse_prime,zhnew_prime,zh_up_prime
REAL(KIND=JPRB) :: zy(4),zyt(4)
REAL(KIND=JPRB) :: zy_prime(4),zyt_prime(4)
REAL(KIND=JPRB) :: zdydh(4),zdydht(4,4)
REAL(KIND=JPRB) :: zdydh_prime(4),zdydht_prime(4,4)
REAL(KIND=JPRB) :: ztheta_tan,ztheta_min,ztheta_max
REAL(KIND=JPRB) :: zdr_max,zdr_dtheta,zrtan,zdr
REAL(KIND=JPRB) :: zdr_max_prime,zdr_dtheta_prime,zrtan_prime,zdr_prime
REAL(KIND=JPRB) :: zalpha_half(2)
REAL(KIND=JPRB) :: zalpha_half_prime(2)
REAL(KIND=JPRB) :: zkval(klev-1,khoriz)
REAL(KIND=JPRB) :: zkval_prime(klev-1,khoriz)
REAL(KIND=JPRB) :: ztlow,ztup,zdalpha,zRoot_halfPI
REAL(KIND=JPRB) :: ztlow_prime,ztup_prime,zdalpha_prime
REAL(KIND=JPRB) :: zerf_up,zerf_low,zt,zdiff_erf,znr_low,zref_low,zaval
REAL(KIND=JPRB) :: zerf_up_prime,zerf_low_prime,zt_prime,zdiff_erf_prime,znr_low_prime
REAL(KIND=JPRB) :: zref_low_prime,zaval_prime
INTEGER(KIND=JPIM) :: ikdum
LOGICAL :: llfirst_1d,lleaving,llone_d_calc         
REAL(KIND=JPRB) :: zalpha(kobs),za_path(kobs,2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gpspderivstl.intfb.h"

IF (LHOOK) CALL DR_HOOK('GPSCALC_ALPHARKM2TL',0,ZHOOK_HANDLE)


! the central profile kcen


ikcen = khoriz/2 + 1
ztheta_tan = REAL(ikcen-1)*pdsep 
ztheta_min = -ztheta_tan
ztheta_max =  ztheta_tan


DO i = 1,klev-1

   DO j = 1, khoriz
   
      zkval(i,j) = LOG(prefrac(i,j)/prefrac(i+1,j))/MAX((pnr(i+1,j) - pnr(i,j)),1.0_JPRB)
      
      IF (zkval(i,j) > 1.0E-6_JPRB) THEN
      
        zkval_prime(i,j) = ((zkval(i,j)*(pnr_prime(i,j)-pnr_prime(i+1,j))) + &
                  & (prefrac_prime(i,j)/prefrac(i,j)-              &
                  &  prefrac_prime(i+1,j)/prefrac(i+1,j)))/        &
                  &  MAX(1.0_JPRB,(pnr(i+1,j)-pnr(i,j)))
                    
      ELSE
      
        zkval(i,j)=1.0E-6_JPRB
        zkval_prime(i,j) = 0.0_JPRB
            
      ENDIF        
      
  ENDDO
   
ENDDO   



! set n_2d level. For levels below n_2d we do 2D ray bending calculation
! above n_2d we do the 1D calculation


in_2d = 0 

DO WHILE ((pnr(in_2d+1,ikcen)-proc < pz_2d) .AND. (in_2d < klev - 1)) 

    in_2d = in_2d + 1
    
ENDDO    

jbot = 1

DO

  IF (prefrac(jbot,ikcen) > 0.0_JPRB .AND. pnr(jbot,ikcen) > 0.0_JPRB) EXIT
  
  jbot = jbot + 1

ENDDO

ikbot = klev

DO i=klev,jbot+1,-1

   IF (pnr(ikbot,ikcen) < pnr(ikbot-1,ikcen)) EXIT 

   ikbot = ikbot - 1

ENDDO
 
jbot = MAX(jbot,ikbot)



! set the outputs to missing


zalpha(:)=RMDI
palpha_prime(:) = 0.0_JPRB
za_path(:,:)= RMDI
pa_path_prime(:,:) = 0.0_JPRB


zRoot_halfPI = SQRT(0.5_JPRB*RPI)

OBLOOP: DO in=1,kobs
   
   IF (pa(in) < pnr(jbot,ikcen) .OR. pa(in) > pnr(klev,ikcen)) CYCLE  
      
   ibot = jbot

   DO 

      IF (pnr(ibot+1,ikcen) - pa(in) > 1.0_JPRB) EXIT   ! assuming "a" is on one of the pressure levels

      ibot=ibot+1

   ENDDO


! calculate the radius at tangent point   

   zrad = 0.5_JPRB*(pradius(ibot,ikcen)+pradius(ibot+1,ikcen))   
   zrad_prime = 0.5_JPRB*(pradius_prime(ibot,ikcen)+pradius_prime(ibot+1,ikcen))
   
   zdndr = 1.0E-6_JPRB*(prefrac(ibot+1,ikcen)-prefrac(ibot,ikcen))/ &
                & (pradius(ibot+1,ikcen)-pradius(ibot,ikcen)) 
  
   zdndr_prime = (1.0E-6_JPRB*(prefrac_prime(ibot+1,ikcen)-prefrac_prime(ibot,ikcen)) -  &
 & zdndr*(pradius_prime(ibot+1,ikcen)-pradius_prime(ibot,ikcen)))/(pradius(ibot+1,ikcen)-pradius(ibot,ikcen))


 
   IF ( zrad*zdndr > -1.0_JPRB) THEN
  
       zrtan = pradius(ibot,ikcen) + &
           &  (pa(in)-pnr(ibot,ikcen))/(1.0_JPRB + zrad*zdndr)


       zrtan_prime = pradius_prime(ibot,ikcen) - &
                 &   (pnr_prime(ibot,ikcen) + &
                 &   (pa(in)-pnr(ibot,ikcen))/ &
                 &    (1.0_JPRB + zrad*zdndr)*(zrad*zdndr_prime + zdndr*zrad_prime))/&  
                 &   (1.0_JPRB + zrad*zdndr)   

   ELSE
   
       zrtan = zrad   ! probably in a super-refracting layer
       
       zrtan_prime = zrad_prime
       
       
   ENDIF                

! if zrtan is within a 1 m of upper level set to upper level
   
   IF ((zrtan - pradius(ibot+1,ikcen)) > -1.0_JPRB) THEN
   
       ibot = ibot + 1
   
       zrtan = pradius(ibot,ikcen)
       
       zrtan_prime = pradius_prime(ibot,ikcen) 
              
   ENDIF     



!  set bending angle value  

   zalpha_half(:) = 0.0_JPRB   
   zalpha_half_prime(:) = 0.0_JPRB

   DO iside = 1,2 
   
 
      za_path(in,iside) = pa(in)
      pa_path_prime(in,iside) = 0.0_JPRB
      llfirst_1d = .TRUE.
      llone_d_calc = .FALSE.        
      zamult = 1.0_JPRB
      IF (iside == 2) zamult  = -1.0_JPRB
      

! initialise vector


      zy(1) = 0.0_JPRB           ! height above tangent point           
      zy(2) = 0.0_JPRB           ! theta
      zy(3) = ASIN(1.0_JPRB)     ! thi
      zy(4) = 0.0_JPRB           ! bending angle 
      
      
      zy_prime(:) = 0.0_JPRB
      
        
      DO i = ibot,klev-1
        
        
         IF ( i < MIN(in_2d,klev-1)) THEN
        
           IF (i == ibot) THEN
         
              ik = ikcen
              zdr_max = (pradius(i+1,ik)- zrtan)/ksplit                        
              zdr_max_prime = (pradius_prime(i+1,ik)-zrtan_prime)/REAL(ksplit)
              zh = SQRT(2.0_JPRB*pradius(ibot,ik)*zdr_max)                    
              zh_prime = &
             &(pradius(ibot,ik)*zdr_max_prime + zdr_max*pradius_prime(ibot,ik))/zh
                        
           ELSE

              ik = INT((zy(2) + ztheta_tan)/pdsep)+1
              ik = MIN(MAX(1,ik),khoriz)
                   zdr_max = (pradius(i+1,ik)-pradius(i,ik))/REAL(ksplit)
              zdr_max_prime = (pradius_prime(i+1,ik)-pradius_prime(i,ik))/REAL(ksplit)
            
              if (zdr_max < 0.0_JPRB .or. pradius(i,ik) < 0.0_jprb) then
              
                  write (*,*) 'gpsro -neg error',i,ibot,ik,ksplit,zdr_max
                  write (*,*) 'gpsto -neg2',pradius(i+1,ik),pradius(i,ik),proc
            
              endif 
            
            
              zh_up = SQRT(2.0_JPRB*pradius(i,ik)*zdr_max)
              zh_up_prime = &
             &(pradius(i,ik)*zdr_max_prime + zdr_max*pradius_prime(i,ik))/zh_up
                          
              zh = zdr_max/MAX(COS(zy(3)),1.0E-10_JPRB)
            

              IF (COS(zy(3)) < 1.0E-10_JPRB) THEN
            
                 zh_prime = 1.0E10_JPRB*zdr_max_prime
                
              ELSE 
            
                 zh_prime = zh/zdr_max*zdr_max_prime + zh*TAN(zy(3))*zy_prime(3)           
               
              ENDIF 

! use zh_up when cos(phi) cose to 0.0
              
              IF (zh > zh_up) THEN
              
                  zh = zh_up
                  zh_prime = zh_up_prime
                  
              ENDIF  
               

           ENDIF

! limit the step-length

           IF (zh > zhmax) THEN
         
             zh = zhmax
             zh_prime = 0.0_JPRB
            
          ELSEIF (zh < zhmin) THEN
         
             zh = zhmin    
             zh_prime = 0.0_JPRB
         
          ENDIF 


           zh2 = 0.5_JPRB*zh         
           zh2_prime = 0.5_JPRB*zh_prime
         

! now calculate the path-length with a RUNGE-KUTTA


           LLEAVING=.FALSE.
         
           DO j = 1,ksplit          
            
              ikdum = ikdum + 1 
            
              zyt(:) = zy(:)            
              zyt_prime(:) = zy_prime(:)  
                      
            
! first calculation of derivs

              CALL gpspderivstl(klev,khoriz,i,pdsep,ztheta_min,ztheta_max,ztheta_tan,&
             &zrtan,zrtan_prime,zamult,prefrac,prefrac_prime,pradius,pradius_prime,zyt, &
             &zyt_prime,zdydht(:,1),zdydht_prime(:,1)) 
        
        
              zyt(:) = zy(:) + zdydht(:,1)*zh2            
              zyt_prime(:) = zy_prime(:) + zdydht(:,1)*zh2_prime + zdydht_prime(:,1)*zh2
            
                      
! second call at new yt
            
              CALL gpspderivstl(klev,khoriz,i,pdsep,ztheta_min,ztheta_max,ztheta_tan,&
             & zrtan,zrtan_prime,zamult,prefrac,prefrac_prime,pradius,pradius_prime,zyt, &
             & zyt_prime,zdydht(:,2),zdydht_prime(:,2)) 
 
              zyt(:) = zy(:) + zdydht(:,2)*zh2
              zyt_prime(:) = zy_prime(:) + zdydht(:,2)*zh2_prime + zdydht_prime(:,2)*zh2

                                    
! third call at new yt
            
              CALL gpspderivstl(klev,khoriz,i,pdsep,ztheta_min,ztheta_max,ztheta_tan,&
             & zrtan,zrtan_prime,zamult,prefrac,prefrac_prime,pradius,pradius_prime,zyt, &
             & zyt_prime,zdydht(:,3),zdydht_prime(:,3)) 

              zyt(:) = zy(:) + zdydht(:,3)*zh
              zyt_prime(:) = zy_prime(:) + zdydht(:,3)*zh_prime + zdydht_prime(:,3)*zh
            
                        
! fourth last call

              CALL gpspderivstl(klev,khoriz,i,pdsep,ztheta_min,ztheta_max,ztheta_tan,&
             & zrtan,zrtan_prime,zamult,prefrac,prefrac_prime,pradius,pradius_prime,zyt, &
             & zyt_prime,zdydht(:,4),zdydht_prime(:,4)) 
                                  
              zdydh(:) = (zdydht(:,1)+zdydht(:,4)+2.0_JPRB*(zdydht(:,2)+zdydht(:,3)))/6.0_JPRB

              zdydh_prime(:) = &
             & (zdydht_prime(:,1)+zdydht_prime(:,4)+ &
             & 2.0_JPRB*(zdydht_prime(:,2)+zdydht_prime(:,3)))/6.0_JPRB
           
              
              zyt(:) = zy(:) + zdydh(:)*zh            
              zyt_prime(:) = zy_prime(:) + zdydh(:)*zh_prime + zdydh_prime(:)*zh
            
            
!              yt_prime = 0.0
            

! check the radius - have we exited the level
            
              ik = INT((zyt(2) + ztheta_tan)/pdsep)+1
              ik = MIN(MAX(1,ik),khoriz-1)
              ikp1 = ik+1
         
! horizontal weighting factor               
            
              IF ( zyt(2) < ztheta_max .AND. zyt(2) > ztheta_min) THEN
                           
                 zhwt1 = (REAL(ik)*pdsep - (zyt(2)+ztheta_tan))/pdsep    
                 zhwt2 = 1.0_JPRB - zhwt1
               
                 zhwt1_prime = -zyt_prime(2)/pdsep
                 zhwt2_prime = - zhwt1_prime 
            
              ELSEIF (zyt(2) < ztheta_min) THEN            
            
                 zhwt1 = 1.0_JPRB
                 zhwt2 = 0.0_JPRB
               
                 zhwt1_prime = 0.0_JPRB
                 zhwt2_prime = 0.0_JPRB
               
              ELSEIF (zyt(2) > ztheta_max) THEN
            
                 zhwt1 = 0.0_JPRB
                 zhwt2 = 1.0_JPRB
               
                 zhwt1_prime = 0.0_JPRB
                 zhwt2_prime = 0.0_JPRB
               
              ENDIF    

         
! radius of pressure level
             
              zrad = zhwt1*pradius(i+1,ik)+zhwt2*pradius(i+1,ikp1)
            
              zrad_prime = zhwt1*pradius_prime(i+1,ik)+zhwt2*pradius_prime(i+1,ikp1) + &
             & zhwt1_prime*pradius(i+1,ik)+zhwt2_prime*pradius(i+1,ikp1)
               
            
! if gone over the boundary scale h
            
              IF ( j == ksplit .OR. (zy(1)+zrtan - zrad) > 0.0_JPRB) THEN
            
            
                 LLEAVING = .TRUE. 
                  
                 zdr_dtheta = 0.0_JPRB
                 zdr_dtheta_prime = 0.0_JPRB 
            
                 IF (zyt(2) < ztheta_max .AND. zyt(2) > ztheta_min) THEN
                
                    zdr_dtheta = (pradius(i+1,ikp1)-pradius(i+1,ik))/pdsep
               
                    zdr_dtheta_prime = &
                   & (pradius_prime(i+1,ikp1)-pradius_prime(i+1,ik))/pdsep
        
                 ENDIF                
                            
                 zhuse = zh - (zyt(1)+zrtan-zrad)/(zdydh(1) - zdr_dtheta*zdydh(2))
                        
                 zhuse_prime = zh_prime - &
                & (zyt_prime(1) + zrtan_prime - zrad_prime)/(zdydh(1) - zdr_dtheta*zdydh(2))  &
                & + (zyt(1)+zrtan-zrad)/(zdydh(1) - zdr_dtheta*zdydh(2))**2* &
                & (zdydh_prime(1) - zdr_dtheta_prime*zdydh(2) - zdr_dtheta*zdydh_prime(2))              

! limit the step

               
               IF (zhuse > zhmax) THEN
               
                   zhuse = zhmax
            
                   zhuse_prime = 0.0_JPRB
            
               ELSEIF (zhuse < zhmin) THEN
               
                   zhuse = zhmin
                   
                   zhuse_prime = 0.0_JPRB
            
               ENDIF

                
                
              ELSE
                
                 zhuse = zh
                 zhuse_prime = zh_prime                             
                     
              ENDIF 

! update the position vector


                     
              zy(:) = zy(:) + zdydh(:)*zhuse 
              zy_prime(:) = zy_prime(:) + zdydh(:)*zhuse_prime + zdydh_prime(:)*zhuse


              zalpha_half(iside) = zy(4)
              zalpha_half_prime(iside) = zy_prime(4)
         
              IF (LLEAVING) EXIT
                             
! try to maintain roughly the same radial increment by adjusting h
        
              IF (j < ksplit) THEN
            
        
                 zdr = (zrad-zy(1)-zrtan)/REAL(MAX(ksplit-j,1))
                 zdr_prime = (zrad_prime-zy_prime(1)-zrtan_prime)/ &
               &        REAL(ksplit-j) 
                
                 zhnew = MIN(zh,zdr/MAX(1.0E-10_JPRB,COS(zy(3))))
               
            
                 IF (zhnew < zh) THEN
            
                    zhnew_prime = zhnew/zdr*zdr_prime + zhnew*TAN(zy(3))*zy_prime(3)
               
                 ELSE
            
                    zhnew_prime = zh_prime    

                 ENDIF  
                
                   zh = zhnew
                 zh_prime = zhnew_prime


! set minimum step-length
            
                 IF (zh > zhmax) THEN
               
                    zh = zhmax
            
                    zh_prime = 0.0_JPRB
            
                 ELSEIF (zh < zhmin) THEN
               
                    zh = zhmin
                    
                    zh_prime = 0.0_JPRB
            
                 ENDIF

            
                 zh2 = 0.5_JPRB*zh
                 zh2_prime = 0.5_JPRB*zh_prime

              ENDIF 


              ENDDO  ! complete path thru ith layer

          
      ELSE


! DO 1D calculation


         IF (llfirst_1d) THEN
         
         
            ik = NINT((zy(2) + ztheta_tan)/pdsep)+1
            ik = MIN(MAX(1,ik),khoriz-1)
            
            za_path(in,iside) = (1.0_JPRB+1.0E-6_JPRB*prefrac(i,ik))*((zy(1)+zrtan)*SIN(zy(3)))

            pa_path_prime(in,iside) = za_path(in,iside)* &
          &  (1.0E-6_JPRB/(1.0_JPRB+1.0E-6_JPRB*prefrac(i,ik))*prefrac_prime(i,ik) + &
          &  (zy_prime(1)+zrtan_prime)/(zy(1)+zrtan) + &
          &  COS(zy(3))/SIN(zy(3))*zy_prime(3) ) 
                        
             zaval = za_path(in,iside)
            zaval_prime = pa_path_prime(in,iside)            
                       
                        
            zalpha_half(iside) = zy(4)
            zalpha_half_prime(iside) = zy_prime(4)
            
            
            
            llfirst_1d = .FALSE.
            
          ENDIF     

! Continue with 1D bending angle calculation

         
         IF ( i == ibot) THEN 
      
! we are doing a 1d calc for entire ray path
            
            llone_d_calc = .TRUE.             
            zref_low = prefrac(i,ik)*EXP(-zkval(i,ik)*(pa(in)-pnr(i,ik)))
            
            zref_low_prime = zref_low*               &
                 & (prefrac_prime(i,ik)/prefrac(i,ik) -  &
                 &  zkval_prime(i,ik)*(pa(in)-pnr(i,ik)) + &
                 &  zkval(i,ik)*pnr_prime(i,ik))  

            za_path(in,iside) = pa(in)
            pa_path_prime(in,iside) = 0.0_JPRB

            zaval = pa(in)
            zaval_prime = 0.0_JPRB
                         
            znr_low = pa(in)
            znr_low_prime = 0.0_JPRB


         ELSE 
      
            zref_low = prefrac(i,ik)
            zref_low_prime = prefrac_prime(i,ik)
             
            znr_low  = pnr(i,ik) 
            znr_low_prime = pnr_prime(i,ik)
                  
         ENDIF


         
         IF (i > ibot) THEN
         
            ztlow = SQRT(MAX(zkval(i,ik)*(pnr(i,ik) - zaval),1.0E-10_JPRB))
            
            IF (zkval(i,ik)*(pnr(i,ik) - zaval) > 1.0E-10_JPRB) THEN
            
               ztlow_prime = 0.5_JPRB*(zkval_prime(i,ik)*(pnr(i,ik)-zaval) + &
             &  zkval(i,ik)*(pnr_prime(i,ik)-zaval_prime))/ztlow
               
            ELSE
            
               ztlow_prime = 0.0_JPRB                
                
            ENDIF                  

         ENDIF 

         ztup = SQRT(MAX(zkval(i,ik)*(pnr(i+1,ik)-zaval),1.0E-10_JPRB))
 
         IF (zkval(i,ik)*(pnr(i+1,ik)-zaval) > 1.0E-10_JPRB) THEN 
 
            ztup_prime = 0.5_JPRB*(zkval_prime(i,ik)*(pnr(i+1,ik)-zaval) + &
         &   zkval(i,ik)*(pnr_prime(i+1,ik)-zaval_prime))/ztup

         ELSE
            
            ztup_prime = 0.0_JPRB
            
         ENDIF   


! calculate the error functions within this routine rather than an external function call.



         IF (i == ibot) THEN
                                   
            zerf_low = 0.0_JPRB
            
            zerf_low_prime = 0.0_JPRB
                                      
            zt = 1.0_JPRB/(1.0_JPRB+0.47047_JPRB*ztup)
            
            zt_prime = - zt/(1.0_JPRB+0.47047_JPRB*ztup)*0.47047_JPRB*ztup_prime
                        
            zerf_up= &
            &1.0_JPRB-(0.3480242_JPRB-(0.0958798_JPRB-0.7478556_JPRB*zt)*zt)*zt*EXP(-(ztup*ztup))
            
            zerf_up_prime = &
           &-(0.3480242_JPRB-(0.1917596_JPRB-2.2435668_JPRB*zt)*zt)*EXP(-(ztup*ztup))*zt_prime + &
           & (0.3480242_JPRB-(0.0958798_JPRB-0.7478556_JPRB*zt)*zt)*zt*EXP(-(ztup*ztup))&
           &*2.0_JPRB*ztup*ztup_prime
            
         ELSEIF (i > ibot .AND. i < klev-1) THEN
                    
! lower
            zt = 1.0_JPRB/(1.0_JPRB+0.47047_JPRB*ztlow)            
            zt_prime = - zt/(1.0_JPRB+0.47047_JPRB*ztlow)*0.47047_JPRB*ztlow_prime
            
            zerf_low = -(0.3480242_JPRB-(0.0958798_JPRB-0.7478556_JPRB*zt)*zt)*zt*EXP(-(ztlow*ztlow))
             
            zerf_low_prime = &
          &  -(0.3480242_JPRB-(0.1917596_JPRB-2.2435668_JPRB*zt)*zt)*EXP(-(ztlow*ztlow))*zt_prime + &
          &  (0.3480242_JPRB-(0.0958798_JPRB-0.7478556_JPRB*zt)*zt)* &
          &  zt*EXP(-(ztlow*ztlow))*2.0_JPRB*ztlow*ztlow_prime

! upper

            zt = 1.0_JPRB/(1.0_JPRB+0.47047_JPRB*ztup)
            zt_prime = - zt/(1.0_JPRB+0.47047_JPRB*ztup)*0.47047_JPRB*ztup_prime

            
            zerf_up= -(0.3480242_JPRB-(0.0958798_JPRB-0.7478556_JPRB*zt)*zt)*zt*EXP(-(ztup*ztup))            
            
            zerf_up_prime = &
          &  -(0.3480242_JPRB-(0.1917596_JPRB-2.2435668_JPRB*zt)*zt)*&
          &   EXP(-(ztup*ztup))*zt_prime + &
          &  (0.3480242_JPRB-(0.0958798_JPRB-0.7478556_JPRB*zt)*zt)*zt*EXP(-(ztup*ztup))* &
          &   2.0_JPRB*ztup*ztup_prime
                        
        ELSE
         
            zerf_up = 0.0_JPRB
            zerf_up_prime = 0.0_JPRB 
         
            zt = 1.0_JPRB/(1.0_JPRB+0.47047_JPRB*ztlow)            
            zt_prime = - zt/(1.0_JPRB+0.47047_JPRB*ztlow)*0.47047_JPRB*ztlow_prime
            
            zerf_low = -(0.3480242_JPRB-(0.0958798_JPRB-0.7478556_JPRB*zt)*zt)*zt*EXP(-(ztlow*ztlow))
             
            zerf_low_prime = &
           &-(0.3480242_JPRB-(0.1917596_JPRB-2.2435668_JPRB*zt)*zt)*&
           &EXP(-(ztlow*ztlow))*zt_prime + &
           &(0.3480242_JPRB-(0.0958798_JPRB-0.7478556_JPRB*zt)*zt)*zt*EXP(-(ztlow*ztlow)) &
           &*2.0_JPRB*ztlow*ztlow_prime
                  
                                 
         ENDIF     
          
         
          
         zdiff_erf = zerf_up - zerf_low 
         zdiff_erf_prime = zerf_up_prime - zerf_low_prime
         


! bending angle          
         
         
         zdalpha    =  &
        & + 1.0E-6_JPRB * zRoot_halfPI* SQRT(zaval*zkval(i,ik)) & 
        & * zref_low*EXP(zkval(i,ik)*(znr_low-zaval))*zdiff_erf 
 
 
         zdalpha_prime = zdalpha*(  &
                      & zref_low_prime/MAX(1.0E-10_JPRB,zref_low)  + &
                      &        zdiff_erf_prime/MAX(1.0E-10_JPRB,zdiff_erf) + &
                      &        (0.5_JPRB/zaval - zkval(i,ik))*zaval_prime + &
                      &        (znr_low -zaval + 0.5_JPRB/zkval(i,ik))*zkval_prime(i,ik) + &
                      &        zkval(i,ik)*znr_low_prime ) 



        zalpha_half(iside) = zalpha_half(iside) + zdalpha 
        zalpha_half_prime(iside) = zalpha_half_prime(iside) + zdalpha_prime  


      ENDIF 
         
         
      ENDDO  ! i the layers


! if we performed a 1d calculation don't evaluate iside = 2

     
      IF (llone_d_calc) THEN
      
         zalpha_half(2) = zalpha_half(1)

         zalpha_half_prime(2) = zalpha_half_prime(1)
          
         EXIT  ! exiting the iside loop
     
     ENDIF      


   ENDDO ! iside


! the total bending angle adding both sides

    
       
   zalpha(in) = zalpha_half(1) + zalpha_half(2) 
   
   palpha_prime(in) = zalpha_half_prime(1) + zalpha_half_prime(2)
   
ENDDO OBLOOP


! we're not using the impact variation

pa_path_prime = 0.0_JPRB

IF (LHOOK) CALL DR_HOOK('GPSCALC_ALPHARKM2TL',1,ZHOOK_HANDLE)

END SUBROUTINE gpscalc_alphaRKm2tl
