SUBROUTINE gpscalc_compress(KLEV,PRES,PTEMP,PQ,PZCOMP_MIX,PZCOMP_DRY_INV, &
&PZCOMP_WET_INV)

!**** *gpscalc_compress* - Operator routine for gps radio occultation observations.

!     S. Healy             ECMWF          2009/03/04
!     
!     PURPOSE
!     -------
!     Calculates the compressibility of air

!**   INTERFACE
!     ---------

!SUBROUTINE gpscalc_compress(KLEV,PRES,PTEMP,PQ,PZCOMP_MIX,PZCOMP_DRY_INV, &
!&PZCOMP_WET_INT)
!      KLEV   = number of model levels
!      PRES   = pressure on model levels
!      PTEMP  = temp on model levels
!      PQ     = specific humidity on model levs 
!      PZCOMP_MIX = compressibility of moist air
!      PZCOMP_DRY_INV = inverse of compressibility of dry air
!      PZCOMP_WET_INV = inverse of compressibility of moisture

!     gpscalc_compress is called from gpsro_op

!**   METHOD
!     ------
!    
!     1) See R.S. Davis, 1992, Metrologia, vol 29, pp 67-70.


!**   EXTERNALS
!     ---------
!
!**   MODIFICATIONS
!     -------------
!     Sean Healy      2009/03/04 : Original
!     -------------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCST, ONLY : rmv, rmd   

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN) :: KLEV 
REAL(KIND=JPRB),INTENT(IN)  :: PRES(KLEV)     ! pressure values
REAL(KIND=JPRB),INTENT(IN)  :: PTEMP(KLEV)    ! temperature values
REAL(KIND=JPRB),INTENT(IN)  :: PQ(KLEV)       ! specific humidity values
REAL(KIND=JPRB),INTENT(OUT) :: PZCOMP_MIX(KLEV)      ! comp of mixture
REAL(KIND=JPRB),INTENT(OUT) :: PZCOMP_DRY_INV(KLEV)  ! inverse dry air comp.
REAL(KIND=JPRB),INTENT(OUT) :: PZCOMP_WET_INV(KLEV)  ! inverse water vapour comp.

! local variables

INTEGER(KIND=JPIM) :: i
REAL(KIND=JPRB) :: zcomp1(KLEV),zcomp2(KLEV),zcomp3(KLEV)
REAL(KIND=JPRB) :: zpd,zpwet,ztc,zx,zpot
REAL(KIND=JPRB) :: zterm1,zterm2,zterm3
REAL(KIND=JPRB) :: z_eps
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! parameters from the Davis paper

REAL(KIND=JPRB), PARAMETER :: za0=1.58123E-6_JPRB
REAL(KIND=JPRB), PARAMETER :: za1=-2.91331E-8_JPRB
REAL(KIND=JPRB), PARAMETER :: za2=1.1043E-10_JPRB
REAL(KIND=JPRB), PARAMETER :: zb0=5.707E-6_JPRB
REAL(KIND=JPRB), PARAMETER :: zb1=-2.051E-8_JPRB
REAL(KIND=JPRB), PARAMETER :: zc0=1.9898E-4_JPRB
REAL(KIND=JPRB), PARAMETER :: zc1=-2.376E-6_JPRB
REAL(KIND=JPRB), PARAMETER :: zd=1.83E-11_JPRB
REAL(KIND=JPRB), PARAMETER :: ze=-0.765E-8_JPRB



IF (LHOOK) CALL DR_HOOK('GPSCALC_COMPRESS',0,ZHOOK_HANDLE)

z_eps = rmv/rmd

DO i = 1,KLEV

! Temp in celcius
   
   ztc = PTEMP(i) - 273.15_JPRB

! calculate the water vapour and dry pressure

   zpwet = PRES(i) * PQ(i) / (z_eps + (1.0_JPRB - z_eps)*PQ(i))
 
! dry pressure

   zpd = PRES(i) - zpwet

   zterm1 = za0+ztc*(za1+ztc*za2)

   zterm2 = zb0+zb1*ztc
   
   zterm3 = zc0+zc1*ztc
   
! compressibility of moist air
  
   zpot = PRES(i)/PTEMP(i)
  
   zx = zpwet/PRES(i)
 
   zcomp1(i) = 1.0_JPRB - zpot*(zterm1 + zx*(zterm2 + zx*zterm3)) 
   
   zcomp1(i) = zcomp1(i) + zpot**2*(zd + zx**2*ze)
 	
! compressibility for dry air
   
   zpot = zpd/PTEMP(i)
   
   zcomp2(i) = 1.0_JPRB - zpot*(zterm1 - zpot*zd)
   
! compressibility of water vapour

   zpot = zpwet/PTEMP(i)
   
   zcomp3(i) = 1.0_JPRB - zpot*(zterm1 + zterm2 + zterm3 - zpot*(zd+ze))
   
ENDDO   

! 
    
DO i = 1, KLEV

   IF (i == 1) THEN
   
      PZCOMP_MIX(i) = zcomp1(i)
   
   ELSE
  
! use the mean value of the i and (i-1) levels
   
      PZCOMP_MIX(i) = 0.5_JPRB*(zcomp1(i)+zcomp1(i-1))
      
   ENDIF    
   
! return the inverse of the compresibility for dry air + vapour   
   
   PZCOMP_DRY_INV(i) = 1.0_JPRB/zcomp2(i)
   
   PZCOMP_WET_INV(i) = 1.0_JPRB/zcomp3(i)


ENDDO   




IF (LHOOK) CALL DR_HOOK('GPSCALC_COMPRESS',1,ZHOOK_HANDLE)



END SUBROUTINE GPSCALC_COMPRESS
