SUBROUTINE gpscalc_compressad(KLEV,PRES,PRES_HAT,PTEMP,PTEMP_HAT,PQ,PQ_HAT,&
&PZCOMP_MIX_HAT,PZCOMP_DRY_INV_HAT,PZCOMP_WET_INV_HAT)

!**** *gpscalc_compress* - Operator routine for gps radio occultation observations.

!     S. Healy             ECMWF          2009/03/04
!     
!     PURPOSE
!     -------
!     Calculates the compressibility of air

!**   INTERFACE
!     ---------

!SUBROUTINE gpscalc_compress(KLEV,PRES,PTEMP,PQ,PZCOMP_MIX,PZCOMP_DRY_INV, &
!&PZCOMP_WET_INT)
!      KLEV   = number of model levels
!      PRES   = pressure on model levels
!      PTEMP  = temp on model levels
!      PQ     = specific humidity on model levs 
!      PZCOMP_MIX = compressibility of moist air
!      PZCOMP_DRY_INV = inverse of compressibility of dry air
!      PZCOMP_WET_INV = inverse of compressibility of moisture

!     gpscalc_compress is called from gpsro_op

!**   METHOD
!     ------
!    
!     1) See R.S. Davis, 1992, Metrologia, vol 29, pp 67-70.


!**   EXTERNALS
!     ---------
!
!**   MODIFICATIONS
!     -------------
!     Sean Healy      2009/03/04 : Original
!     -------------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCST, ONLY : rmv, rmd   

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KLEV 
REAL(KIND=JPRB),INTENT(IN)    :: PRES(KLEV)     ! pressure values
REAL(KIND=JPRB),INTENT(INOUT) :: PRES_HAT(KLEV)     ! pressure values
REAL(KIND=JPRB),INTENT(IN)    :: PTEMP(KLEV)     ! pressure values
REAL(KIND=JPRB),INTENT(INOUT) :: PTEMP_HAT(KLEV)     ! pressure values
REAL(KIND=JPRB),INTENT(IN)    :: PQ(KLEV)       ! specific humidity values
REAL(KIND=JPRB),INTENT(INOUT) :: PQ_HAT(KLEV)       ! specific humidity values
REAL(KIND=JPRB),INTENT(INOUT) :: PZCOMP_MIX_HAT(KLEV)      ! comp of mixture
REAL(KIND=JPRB),INTENT(INOUT) :: PZCOMP_DRY_INV_HAT(KLEV)  ! inverse dry air comp.
REAL(KIND=JPRB),INTENT(INOUT) :: PZCOMP_WET_INV_HAT(KLEV)  ! inverse water vapour comp.

! local variables

INTEGER(KIND=JPIM) :: i
REAL(KIND=JPRB) :: zcomp1(KLEV),zcomp2(KLEV),zcomp3(KLEV)
REAL(KIND=JPRB) :: zcomp1_hat(KLEV),zcomp2_hat(KLEV),zcomp3_hat(KLEV)
REAL(KIND=JPRB) :: zpd(KLEV),zpwet(KLEV),ztc,zx,zpot
REAL(KIND=JPRB) :: zpd_hat,zpwet_hat,ztc_hat,zx_hat,zpot_hat
REAL(KIND=JPRB) :: zterm1(KLEV),zterm2(KLEV),zterm3(KLEV)
REAL(KIND=JPRB) :: zterm1_hat,zterm2_hat,zterm3_hat
REAL(KIND=JPRB) :: z_eps
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! parameters from the Davis paper

REAL(KIND=JPRB), PARAMETER :: za0=1.58123E-6_JPRB
REAL(KIND=JPRB), PARAMETER :: za1=-2.91331E-8_JPRB
REAL(KIND=JPRB), PARAMETER :: za2=1.1043E-10_JPRB
REAL(KIND=JPRB), PARAMETER :: zb0=5.707E-6_JPRB
REAL(KIND=JPRB), PARAMETER :: zb1=-2.051E-8_JPRB
REAL(KIND=JPRB), PARAMETER :: zc0=1.9898E-4_JPRB
REAL(KIND=JPRB), PARAMETER :: zc1=-2.376E-6_JPRB
REAL(KIND=JPRB), PARAMETER :: zd=1.83E-11_JPRB
REAL(KIND=JPRB), PARAMETER :: ze=-0.765E-8_JPRB



IF (LHOOK) CALL DR_HOOK('GPSCALC_COMPRESSAD',0,ZHOOK_HANDLE)

! initialise adjoint variables

zcomp1_hat(:) = 0.0_JPRB
zcomp2_hat(:) = 0.0_JPRB
zcomp3_hat(:) = 0.0_JPRB
zpd_hat   = 0.0_JPRB
zpwet_hat = 0.0_JPRB
ztc_hat   = 0.0_JPRB
zx_hat    = 0.0_JPRB
zpot_hat  = 0.0_JPRB
zterm1_hat= 0.0_JPRB
zterm2_hat= 0.0_JPRB
zterm3_hat= 0.0_JPRB

z_eps = rmv/rmd

DO i = 1,KLEV

! Temp in celcius
   
   ztc = PTEMP(i) - 273.15_JPRB   

! calculate the water vapour and dry pressure

   zpwet(i) = PRES(i) * PQ(i) / (z_eps + (1.0_JPRB - z_eps)*PQ(i))

! dry pressure

   zpd(i) = PRES(i) - zpwet(i)

   zterm1(i) = za0+ztc*(za1+ztc*za2)

   zterm2(i) = zb0+zb1*ztc
   
   zterm3(i) = zc0+zc1*ztc
   
! compressibility of moist air
  
   zpot = PRES(i)/PTEMP(i)
  
   zx = zpwet(i)/PRES(i)
 
   zcomp1(i) = 1.0_JPRB - zpot*(zterm1(i) + zx*(zterm2(i) + zx*zterm3(i))) 
   
   zcomp1(i) = zcomp1(i) + zpot**2*(zd + zx**2*ze)
   
! compressibility for dry air
   
   zpot = zpd(i)/PTEMP(i)
   
   zcomp2(i) = 1.0_JPRB - zpot*(zterm1(i) - zpot*zd)
   
! compressibility of water vapour

   zpot = zpwet(i)/PTEMP(i)
   
   zcomp3(i) = 1.0_JPRB - zpot*(zterm1(i) + zterm2(i) + zterm3(i) - zpot*(zd+ze))
   
ENDDO   


! adjoint bit

DO i = KLEV, 1, -1


!!   PZCOMP_WET_INV_PRIME(i) = -1.0_JPRB/zcomp3(i)**2*zcomp3_prime(i)

   zcomp3_hat(i) = zcomp3_hat(i) -1.0_JPRB/zcomp3(i)**2*PZCOMP_WET_INV_HAT(i)
   PZCOMP_WET_INV_HAT(i) = 0.0_JPRB
   
!!   PZCOMP_DRY_INV_PRIME(i) = - 1.0_JPRB/zcomp2(i)**2*zcomp2_prime(i)
   
   zcomp2_hat(i) = zcomp2_hat(i) -1.0_JPRB/zcomp2(i)**2*PZCOMP_DRY_INV_HAT(i)
   PZCOMP_DRY_INV_HAT(i) = 0.0_JPRB


   IF (i == 1) THEN
   
!!      PZCOMP_MIX_PRIME(i) = zcomp1_prime(i)
      
      zcomp1_hat(i) = zcomp1_hat(i) + PZCOMP_MIX_HAT(i)
      PZCOMP_MIX_HAT(i) = 0.0_JPRB
      
              
   ELSE
  
! use the mean value of the i and (i-1) levels
   
!!      PZCOMP_MIX_PRIME(i) = 0.5_JPRB*(zcomp1_prime(i)+zcomp1_prime(i-1))
  
      zcomp1_hat(i) = zcomp1_hat(i)     + 0.5_JPRB*PZCOMP_MIX_HAT(i) 
      zcomp1_hat(i-1) = zcomp1_hat(i-1) + 0.5_JPRB*PZCOMP_MIX_HAT(i) 
      PZCOMP_MIX_HAT(i) = 0.0_JPRB
  
      
   ENDIF    

   
ENDDO   



DO i = KLEV, 1, -1

! vapour

   zpot = zpwet(i)/PTEMP(i)

!!   zcomp3_prime(i) = -(zterm1 + zterm2 + zterm3 - 2.0_JPRB*zpot*(zd+ze))*zpot_prime &
!!                   & -zpot*(zterm1_prime + zterm2_prime + zterm3_prime)     

   zpot_hat = zpot_hat - &
  &(zterm1(i) + zterm2(i) + zterm3(i) - 2.0_JPRB*zpot*(zd+ze))*zcomp3_hat(i)
   zterm1_hat = zterm1_hat - zpot*zcomp3_hat(i)
   zterm2_hat = zterm2_hat - zpot*zcomp3_hat(i)
   zterm3_hat = zterm3_hat - zpot*zcomp3_hat(i)
   zcomp3_hat(i) = 0.0_JPRB
   
   zpwet_hat = zpwet_hat + zpot_hat/PTEMP(i)
   PTEMP_HAT(i) = PTEMP_HAT(i) - zpot/PTEMP(i)*zpot_hat
   zpot_hat = 0.0_JPRB
   
  
! dry air

   zpot = zpd(i)/PTEMP(i)

!!   zcomp2_prime(i) = - (zterm1 - 2.0_JPRB*zpot*zd)*zpot_prime  & 
!!                   & - zpot*zterm1_prime     
   
   zpot_hat = zpot_hat - (zterm1(i) - 2.0_JPRB*zpot*zd)*zcomp2_hat(i)
   zterm1_hat = zterm1_hat - zpot*zcomp2_hat(i)
   zcomp2_hat(i) = 0.0_JPRB   

!!   zpot_prime = zpot/zpd*zpd_prime - zpot/PTEMP(i)*PTEMP_PRIME(i)

   zpd_hat = zpd_hat + zpot/zpd(i)*zpot_hat
   PTEMP_HAT(i) = PTEMP_HAT(i) - zpot/PTEMP(i)*zpot_hat
   zpot_hat = 0.0_JPRB


! mixture

   zpot = PRES(i)/PTEMP(i)
   zx = zpwet(i)/PRES(i)

!!   zcomp1_prime(i) = zcomp1_prime(i) + 2.0_JPRB*zpot*(zd + zx**2*ze)*zpot_prime + &
!!                   & 2.0_JPRB*zpot**2*zx*ze*zx_prime   

   zpot_hat = zpot_hat + 2.0_JPRB*zpot*(zd + zx**2*ze)*zcomp1_hat(i)
   zx_hat = zx_hat + 2.0_JPRB*zpot**2*zx*ze*zcomp1_hat(i)
   zcomp1_hat(i) = zcomp1_hat(i)

!!   zcomp1_prime(i) = -(zterm1 + zx*(zterm2 + zx*zterm3))*zpot_prime  &
!!                &   -zpot*(zterm1_prime + (zterm2 + 2.0_JPRB*zx*zterm3)*zx_prime  &
!!                &   +zx*(zterm2_prime + zx*zterm3_prime))		   
   

   zpot_hat = zpot_hat - (zterm1(i) + zx*(zterm2(i) + zx*zterm3(i)))*zcomp1_hat(i)
   zterm1_hat = zterm1_hat - zpot*zcomp1_hat(i)
   zterm2_hat = zterm2_hat - zpot*zx*zcomp1_hat(i)
   zterm3_hat = zterm3_hat - zpot*zx*zx*zcomp1_hat(i)
   zx_hat     = zx_hat - zpot*(zterm2(i) + 2.0_JPRB*zx*zterm3(i))*zcomp1_hat(i)
   zcomp1_hat(i) = 0.0_JPRB
   
!!   zx_prime = 1.0_JPRB/PRES(i)*zpwet_prime - zx/PRES(i)*PRES_PRIME(i) 
      
   zpwet_hat = zpwet_hat + zx_hat/PRES(i)
   PRES_HAT(i) = PRES_HAT(i) - zx/PRES(i)*zx_hat
   zx_hat = 0.0_JPRB
   
!!   zpot_prime = zpot/PRES(i)*PRES_PRIME(i) - zpot/PTEMP(i)*PTEMP_PRIME(i)

   PRES_HAT(i) = PRES_HAT(i) + zpot/PRES(i)*zpot_hat
   PTEMP_HAT(i)= PTEMP_HAT(i) - zpot/PTEMP(i)*zpot_hat
   zpot_hat = 0.0_JPRB

!!   zterm3_prime = zc1*ztc_prime
   
   ztc_hat = ztc_hat + zc1*zterm3_hat
   zterm3_hat = 0.0_JPRB
   
!!   zterm2_prime = zb1*ztc_prime

   ztc_hat = ztc_hat + zb1*zterm2_hat 
   zterm2_hat = 0.0_JPRB
   
!!   zterm1_prime = (za1+2.0_JPRB*ztc*za2)*ztc_prime
  
   ztc = PTEMP(i) - 273.15_JPRB
   ztc_hat = ztc_hat + (za1+2.0_JPRB*ztc*za2)*zterm1_hat
   zterm1_hat = 0.0_JPRB       
  
!!   zpd_prime = PRES_PRIME(i) - zpwet_prime 

   PRES_HAT(i) = PRES_HAT(i) + zpd_hat
   zpwet_hat = zpwet_hat - zpd_hat
   zpd_hat = 0.0_JPRB   

!!   zpwet_prime = zpwet/PRES(i)*PRES_PRIME(i) &
!!             & + PRES(i)*z_eps/(z_eps + (1.0_JPRB - z_eps)*PQ(i))**2*PQ_PRIME(i)   

   PRES_HAT(i) = PRES_HAT(i) + zpwet(i)/PRES(i)*zpwet_hat
   PQ_HAT(i) = PQ_HAT(i) + PRES(i)*z_eps/(z_eps + (1.0_JPRB - z_eps)*PQ(i))**2*zpwet_hat
   zpwet_hat = 0.0_JPRB

!!   ztc_prime = PTEMP_PRIME(i)
     
   PTEMP_HAT(i) = PTEMP_HAT(i) + ztc_hat
   ztc_hat = 0.0_JPRB


ENDDO

IF (LHOOK) CALL DR_HOOK('GPSCALC_COMPRESSAD',1,ZHOOK_HANDLE)



END SUBROUTINE GPSCALC_COMPRESSAD
