SUBROUTINE gpscalc_nr2dad(KLEV,KHORIZ,&
&P_ROC,P_LAT,P_ZG,P_ZG_HAT,P_REFRAC,P_REFRAC_HAT, &
&P_RAD_HAT,P_NR_HAT)  
 
!**** *gpscalc_nr* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2004/10/26
!     Based on a EUMETSAT GRAS-SAF routine that was Dr'd automatically by the CY27 merge.  

!     PURPOSE
!     -------
!     It calculates the refractive index * radius on the model levels :

!**   INTERFACE
!     ---------

!     CALL gpscalc_nr(KLEV,KVERT,KHORIZ,P_ROC,P_LAT,P_ZG,P_REFRAC,P_NR)  

!     WHERE KLEV   =  number of full model levels
!           KHORIZ   =  no. of horizontal locations 
!           P_ROC    =  radius of curvature
!           P_LAT    =  pressure on the full model levels
!           P_ZG     =  geopotential height on the full model levels
!           P_REFRAC =  refractivity on full model levels
!           P_REFRAC2=   
!           P_NR     =  refractive index - radius product 


!     gpscalc_2dnr is called from gpsro_2dop and gpsro_2dad

!**   METHOD
!     ------

!     1) calculate the geometric and radius of full model levels
!     2) calculate the refractive index - radius product  
!    

!**   EXTERNALS
!     ---------
!     1) fcseradi calculates the radius used to convert geopotential to geometric height
!     2) variation of gravity with latitude

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS  , ONLY : RMDI 

USE YOMCST   , ONLY : RG           

IMPLICIT NONE

! subroutine args. 

INTEGER(KIND=JPIM),INTENT(IN)      :: KLEV! no. of p levels in state vec. 
INTEGER(KIND=JPIM),INTENT(IN)      :: KHORIZ
REAL(KIND=JPRB)   ,INTENT(IN)      :: P_ROC 
REAL(KIND=JPRB)   ,INTENT(IN)      :: P_LAT 
REAL(KIND=JPRB)   ,INTENT(IN)      :: P_ZG(KLEV,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: P_ZG_HAT(KLEV,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)      :: P_REFRAC(KLEV,KHORIZ)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: P_REFRAC_HAT(KLEV,KHORIZ)
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: P_RAD_HAT(KLEV,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(INOUT)   :: P_NR_HAT(KLEV,KHORIZ) 

! local variables

INTEGER(KIND=JPIM) :: i,j,ibot
REAL(KIND=JPRB) :: zed(KLEV)          ! geometric height
REAL(KIND=JPRB) :: zed_hat(KLEV)      ! geometric height
REAL(KIND=JPRB) :: Z_GRAT               ! ratio of g(lat)/g0
REAL(KIND=JPRB) :: Z_RADIUS             ! effective radius used by List
REAL(KIND=JPRB) :: fcseradi              ! function to calculate effective radius
REAL(KIND=JPRB) :: fcsglat              ! g varying with latitude
REAL(KIND=JPRB) :: Z_RAD(KLEV,KHORIZ)
REAL(KIND=JPRB) :: Z_NR(KLEV,KHORIZ)
REAL(KIND=JPRB) :: ZHOOK_HANDLE   

IF (LHOOK) CALL DR_HOOK('GPSCALC_NR2DAD',0,ZHOOK_HANDLE)

! calculate the radius and g values used in the geopotential/geometric height
! conversion.

Z_GRAT   = fcsglat(P_LAT)/RG     ! RG is gravity value 9.80...   
Z_RADIUS = fcseradi(P_LAT)

! calculate the geometric heights

Z_RAD(:,:) = RMDI
Z_NR(:,:)  = RMDI
zed(:) = 0.0_JPRB

zed_hat(:) = 0.0_JPRB
 

! calculate the geometric height of the model levels and interpolate


DO j = 1,KHORIZ 

  ibot = 1
  
  DO i=1,KLEV

    IF (P_REFRAC(i,j) > 0.0_JPRB ) THEN

      zed(i)=P_ZG(i,j)/(Z_GRAT - P_ZG(i,j)/Z_RADIUS) ! geometric heights

    ELSE
      
      zed(i) = RMDI
      
      ibot = i+1 
 
    ENDIF


  ENDDO

  DO i = ibot, KLEV
    
       Z_RAD(i,j) = P_ROC + zed(i)

       Z_NR(i,j) = Z_RAD(i,j)* &
     & (1.0_JPRB + 1.0E-6_JPRB*P_REFRAC(i,j))

       P_RAD_HAT(i,j) = P_RAD_HAT(i,j) + &
      &(1.0_JPRB + 1.0E-6_JPRB*P_REFRAC(i,j))*P_NR_HAT(i,j)

       P_REFRAC_HAT(i,j) = P_REFRAC_HAT(i,j) + &
      &1.0E-6_JPRB*Z_RAD(i,j)*P_NR_HAT(i,j)
       P_NR_HAT(i,j) = 0.0_JPRB
       
       zed_hat(i) = zed_hat(i) + P_RAD_HAT(i,j) 
       P_RAD_HAT(i,j) = 0.0_JPRB
       
              
  ENDDO       


  DO i=1,KLEV

    IF (P_REFRAC(i,j) > 0.0_JPRB ) THEN
      
      P_ZG_HAT(i,j) = P_ZG_HAT(i,j) + &
     &Z_GRAT/(Z_GRAT - P_ZG(i,j)/Z_RADIUS)**2*zed_hat(i)
      zed_hat(i) = 0.0_JPRB 
      
    ELSE
    
      zed_hat(i) = 0.0_JPRB 
             
    ENDIF


  ENDDO
  

ENDDO

P_NR_HAT(:,:) = 0.0_JPRB
P_RAD_HAT(:,:) = 0.0_JPRB


IF (LHOOK) CALL DR_HOOK('GPSCALC_NR2DAD',1,ZHOOK_HANDLE)

END SUBROUTINE gpscalc_nr2dad
