SUBROUTINE gpscalc_refrac2dAD(KLEV,        &
 & KHORIZ,     &
 & pres,       &
 & pres_hat, &
 & P_TEMP,       &
 & P_TEMP_HAT,  &
 & P_Q,          &
 & P_Q_HAT,    &
 & P_REFRAC_HAT)  
 
!**** *gpscalc_refrac2dAD* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2004/10/26
!     Based on a EUMETSAT GRAS-SAF routine that was Dr'd automatically by the CY27 merge.  

!     PURPOSE
!     -------
!     It calculates the refractivity values on the model levels :

!**   INTERFACE
!     ---------

!     CALL gpscalc_refrac2dAD
!     (KLEV,pres,pres_prime,P_TEMP,P_TEMP_HAT,P_Q,P_Q_HAT,P_REFRAC_HAT)  
!     WHERE KLEV         = number of full model levels
!           pres           = pressure on the full model levels
!           pres_hat       = Adjoint pressure on the full model levels
!           P_TEMP         = temperature on the full model levels
!           P_TEMP_HAT     = Adjoint temperature on the full model levels
!           P_Q            = specific humidity on full model levels
!           P_Q_HAT        = Adjoint  specific humidity on full model levels
!           P_REFRAC_HAT   = Adjoint refractivity on full model levels 

!     gpscalc_refracAD is called from gpsro_ad

!**   METHOD
!     ------
!     1) calculate the dry and wet refractivity terms and add them. 

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------

               
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST, ONLY : rmv,   &
 & rmd       

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV                 ! number of model levels
INTEGER(KIND=JPIM),INTENT(IN)    :: KHORIZ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KLEV,KHORIZ)           ! pressure on model levels 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRES_HAT(KLEV,KHORIZ)       ! Adjoint pressure on model levels 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_TEMP(KLEV,KHORIZ)         ! temperataure on model levels
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P_TEMP_HAT(KLEV,KHORIZ)     ! Adjoint temperataure on model levels 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_Q(KLEV,KHORIZ)            ! specific humidity on model levels 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P_Q_HAT(KLEV,KHORIZ)        ! Adjoint specific humidity values on model levels
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P_REFRAC_HAT(KLEV,KHORIZ)   ! Adjoint refracivity values on model levels
! local variables

REAL(KIND=JPRB), PARAMETER :: aval = 0.776_JPRB
REAL(KIND=JPRB), PARAMETER :: bval = 3.73E3_JPRB
INTEGER(KIND=JPIM) :: i,j                    ! counters
REAL(KIND=JPRB)    :: Z_NDRY,Z_NWET          ! dry and wet refractivity
REAL(KIND=JPRB)    :: Z_NDRY_HAT,Z_NWET_HAT  ! Adjoint "
REAL(KIND=JPRB)    :: Z_EPS                  ! ratio of vapour/dry molecular weights   
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! initialise local adjoint variables

IF (LHOOK) CALL DR_HOOK('GPSCALC_REFRAC2DAD',0,ZHOOK_HANDLE)

! set the ratio of molecular weights (vapour/dry)

Z_EPS = rmv/rmd

Z_NDRY_HAT = 0.0_JPRB
Z_NWET_HAT = 0.0_JPRB

DO j = KHORIZ,1,-1

   DO i = KLEV,1,-1  

! forward variables used in adjoint code 

     Z_NDRY = aval*pres(i,j)/P_TEMP(i,j)
   
     Z_NWET = bval*pres(i,j)*P_Q(i,j)/ &
   & (P_TEMP(i,j)*P_TEMP(i,j)*(Z_EPS+(1.0_JPRB - Z_EPS)*P_Q(i,j)))  

! adjoint code
            
     Z_NDRY_HAT = Z_NDRY_HAT + P_REFRAC_HAT(i,j)
     Z_NWET_HAT = Z_NWET_HAT + P_REFRAC_HAT(i,j)
     P_REFRAC_HAT(i,j) = 0.0_JPRB

     pres_hat(i,j) = pres_hat(i,j) + Z_NWET/pres(i,j)         *Z_NWET_HAT
     P_TEMP_HAT(i,j) = P_TEMP_HAT(i,j) - 2.0_JPRB*Z_NWET/P_TEMP(i,j)*Z_NWET_HAT
     P_Q_HAT(i,j) = P_Q_HAT(i,j) + bval*pres(i,j)*Z_EPS/ &
   & (P_TEMP(i,j)*(Z_EPS+(1.0_JPRB - Z_EPS)*P_Q(i,j)))**2 *Z_NWET_HAT         
     Z_NWET_HAT = 0.0_JPRB
   
     pres_hat(i,j) = pres_hat(i,j) + Z_NDRY/pres(i,j) * Z_NDRY_HAT
     P_TEMP_HAT(i,j) = P_TEMP_HAT(i,j) - Z_NDRY/P_TEMP(i,j) * Z_NDRY_HAT
     Z_NDRY_HAT = 0.0_JPRB
                  
   ENDDO

ENDDO

! should already be set to zero - but put in.

P_REFRAC_HAT(:,:) = 0.0_JPRB 

IF (LHOOK) CALL DR_HOOK('GPSCALC_REFRAC2DAD',1,ZHOOK_HANDLE)

END SUBROUTINE gpscalc_refrac2dad
