SUBROUTINE gpscalc_refrac2dTL(KLEV,        &
 & KHORIZ, &
 & pres,       &
 & pres_prime, &
 & P_TEMP,       &
 & P_TEMP_PRIME,  &
 & P_Q,          &
 & P_Q_PRIME,    &
 & P_REFRAC,     &
 & P_REFRAC_PRIME)  

!**** *gpscalc_refracTL* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2004/10/26
!     Based on a EUMETSAT GRAS-SAF routine that was Dr'd automatically by the CY27 merge.  

!     PURPOSE
!     -------
!     It calculates the refractivity values on the model levels :

!**   INTERFACE
!     ---------

!     CALL gpscalc_refrac2dTL
!     (KLEV,pres,pres_prime,P_TEMP,P_TEMP_PRIME,P_Q,P_Q_PRIME,P_REFRAC,P_REFRAC_PRIME)  
!     WHERE KLEV         = number of full model levels
!           pres           = pressure on the full model levels
!           pres_prime     = perturbed pressure on the full model levels
!           P_TEMP         = temperature on the full model levels
!           P_TEMP_PRIME   = perturbed temperature on the full model levels
!           P_Q            = specific humidity on full model levels
!           P_Q_PRIME      = perturbed  specific humidity on full model levels
!           P_REFRAC       = refractivity on full model levels 
!           P_REFRAC_PRIME = perturbed refractivity on full model levels 


!     gpscalc_refracTL is called from gpsro_tl

!**   METHOD
!     ------
!     1) calculate the dry and wey refractivity terms and add them. 

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------
               
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS, ONLY : RMDI

USE YOMCST, ONLY : rmv, &
 & rmd   

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KHORIZ
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KLEV,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES_PRIME(KLEV,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_TEMP(KLEV,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_TEMP_PRIME(KLEV,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_Q(KLEV,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_Q_PRIME(KLEV,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_REFRAC(KLEV,KHORIZ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_REFRAC_PRIME(KLEV,KHORIZ) 

! local variables

REAL(KIND=JPRB), PARAMETER :: aval = 0.776_JPRB
REAL(KIND=JPRB), PARAMETER :: bval = 3.73E3_JPRB
INTEGER(KIND=JPIM) :: i,j                        ! counter
REAL(KIND=JPRB)    :: Z_NDRY,Z_NWET              ! wet and dry refractivity
REAL(KIND=JPRB)    :: Z_NDRY_PRIME,Z_NWET_PRIME  ! " perturbed
REAL(KIND=JPRB)    :: Z_EPS                      ! ratio of vapour/dry molecular weights
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! set the ratio of molecular weights (vapour/dry)

IF (LHOOK) CALL DR_HOOK('GPSCALC_REFRAC2DTL',0,ZHOOK_HANDLE)
Z_EPS = rmv/rmd

! initialise the refractivity to missing

P_REFRAC(:,:)  = RMDI
P_REFRAC_PRIME(:,:) = 0.0_JPRB

Z_NDRY = 0.0_JPRB
Z_NWET = 0.0_JPRB

! calculate the refrac values

DO j = 1, KHORIZ

   DO i = 1, KLEV

     Z_NDRY = aval*pres(i,j)/P_TEMP(i,j)
   
     Z_NDRY_PRIME = Z_NDRY/pres(i,j)*pres_prime(i,j) - &
   & Z_NDRY/P_TEMP(i,j)*P_TEMP_PRIME(i,j)  
                    
     Z_NWET = bval*pres(i,j)*P_Q(i,j)/ &
   & (P_TEMP(i,j)*P_TEMP(i,j)*(Z_EPS+(1.0_JPRB - Z_EPS)*P_Q(i,j)))  
   
     Z_NWET_PRIME = Z_NWET/pres(i,j) * pres_prime(i,j) - &
   & 2.0_JPRB*Z_NWET/P_TEMP(i,j)* P_TEMP_PRIME(i,j) + &
   & bval*pres(i,j)*Z_EPS/ &
   & (P_TEMP(i,j)*(Z_EPS+(1.0_JPRB - Z_EPS)*P_Q(i,j)))**2 *P_Q_PRIME(i,j)    
      
     P_REFRAC(i,j) = Z_NDRY + Z_NWET
   
     P_REFRAC_PRIME(i,j) = Z_NDRY_PRIME + Z_NWET_PRIME

   
   ENDDO
   
ENDDO

IF (LHOOK) CALL DR_HOOK('GPSCALC_REFRAC2DTL',1,ZHOOK_HANDLE)

END SUBROUTINE gpscalc_refrac2dTL
