SUBROUTINE gpscalc_refracAD(K_NLEV,        &
 & pres,       &
 & pres_hat, &
 & P_TEMP,       &
 & P_TEMP_HAT,  &
 & P_Q,          &
 & P_Q_HAT,    &
 & PZ_DRY,    &
 & PZ_DRY_HAT,    &
 & PZ_WET,   &  
 & PZ_WET_HAT,   &  
 & P_REFRAC_HAT)  
 
!**** *gpscalc_refracAD* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2004/10/26
!     Based on a EUMETSAT GRAS-SAF routine that was Dr'd automatically by the CY27 merge.  

!     PURPOSE
!     -------
!     It calculates the refractivity values on the model levels :

!**   INTERFACE
!     ---------

!     CALL gpscalc_refracAD
!     (K_NLEV,pres,pres_prime,P_TEMP,P_TEMP_HAT,P_Q,P_Q_HAT,P_REFRAC_HAT)  
!     WHERE K_NLEV         = number of full model levels
!           pres           = pressure on the full model levels
!           pres_hat       = Adjoint pressure on the full model levels
!           P_TEMP         = temperature on the full model levels
!           P_TEMP_HAT     = Adjoint temperature on the full model levels
!           P_Q            = specific humidity on full model levels
!           P_Q_HAT        = Adjoint  specific humidity on full model levels
!           P_REFRAC_HAT   = Adjoint refractivity on full model levels 


!     gpscalc_refracAD is called from gpsro_ad

!**   METHOD
!     ------
!    
!     1) calculate the dry and wet refractivity terms and add them. 
!    

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------

               
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST, ONLY : rmv,   &
 & rmd       

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: K_NLEV                 ! number of model levels
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(K_NLEV)           ! pressure on model levels 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRES_HAT(K_NLEV)       ! Adjoint pressure on model levels 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_TEMP(K_NLEV)         ! temperataure on model levels
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P_TEMP_HAT(K_NLEV)     ! Adjoint temperataure on model levels 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_Q(K_NLEV)            ! specific humidity on model levels 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P_Q_HAT(K_NLEV)        ! Adjoint specific humidity values on model levels
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ_DRY(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZ_DRY_HAT(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ_WET(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZ_WET_HAT(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P_REFRAC_HAT(K_NLEV)   ! Adjoint refracivity values on model levels
! local variables

REAL(KIND=JPRB), PARAMETER :: z_aval = 0.77643_JPRB
REAL(KIND=JPRB), PARAMETER :: z_bval = 3.75463E3_JPRB
REAL(KIND=JPRB), PARAMETER :: z_cval = 0.712952_JPRB

INTEGER(KIND=JPIM) :: i                      ! counter
REAL(KIND=JPRB)    :: Z_NDRY,Z_NWET          ! dry and wet refractivity
REAL(KIND=JPRB)    :: Z_NDRY_HAT,Z_NWET_HAT  ! Adjoint "
REAL(KIND=JPRB)    :: Z_PW                   ! water vapour pressure  
REAL(KIND=JPRB)    :: Z_PW_HAT               ! water vapour pressure  
REAL(KIND=JPRB)    :: Z_PD                   ! dry air pressure  
REAL(KIND=JPRB)    :: Z_PD_HAT               ! dry air pressure  
REAL(KIND=JPRB)    :: Z_EPS                  ! ratio of vapour/dry molecular weights   
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! initialise local adjoint variables

IF (LHOOK) CALL DR_HOOK('GPSCALC_REFRACAD',0,ZHOOK_HANDLE)
Z_NDRY_HAT = 0.0_JPRB
Z_NWET_HAT = 0.0_JPRB
Z_PW_HAT   = 0.0_JPRB
Z_PD_HAT   = 0.0_JPRB

! set the ratio of molecular weights (vapour/dry)

Z_EPS = rmv/rmd

DO i = K_NLEV,1,-1  

  Z_PW = pres(i)*P_Q(i)/(Z_EPS+(1.0_JPRB-Z_EPS)*P_Q(i))

  Z_PD = pres(i) - Z_PW 
 
! forward variables used in adjoint code 

  Z_NDRY = z_aval*Z_PD/P_TEMP(i)
   
  Z_NWET = Z_PW/P_TEMP(i)*(z_bval/P_TEMP(i)+ z_cval)  

! adjoint code
            
  Z_NDRY_HAT = Z_NDRY_HAT + PZ_DRY(i)*P_REFRAC_HAT(i)
  Z_NWET_HAT = Z_NWET_HAT + PZ_WET(i)*P_REFRAC_HAT(i)
  PZ_DRY_HAT(i) = PZ_DRY_HAT(i) + Z_NDRY*P_REFRAC_HAT(i)
  PZ_WET_HAT(i) = PZ_WET_HAT(i) + Z_NWET*P_REFRAC_HAT(i)
  P_REFRAC_HAT(i) = 0.0_JPRB

! nwet bit

  P_TEMP_HAT(i) = P_TEMP_HAT(i) - &
  &Z_PW*(2.0_JPRB*z_bval/P_TEMP(i)+z_cval)/P_TEMP(i)**2*Z_NWET_HAT  
  Z_PW_HAT = Z_PW_HAT + (z_bval/P_TEMP(i)+z_cval)/P_TEMP(i)*Z_NWET_HAT
  Z_NWET_HAT = 0.0_JPRB

  
! ndry bit
  
  Z_PD_HAT = Z_PD_HAT + Z_NDRY/Z_PD * Z_NDRY_HAT
  P_TEMP_HAT(i) = P_TEMP_HAT(i) - Z_NDRY/P_TEMP(i) * Z_NDRY_HAT
  Z_NDRY_HAT = 0.0_JPRB

! pd_prime = pres_prime - pw_prime

  pres_hat(i) = pres_hat(i) + Z_PD_HAT
  Z_PW_HAT = Z_PW_HAT - Z_PD_HAT
  Z_PD_HAT = 0.0_JPRB


! pw bit

  pres_hat(i) = pres_hat(i) + Z_PW/pres(i)*Z_PW_HAT   
  P_Q_HAT(i) = P_Q_HAT(i) + Z_EPS*pres(i)/(Z_EPS+(1.0_JPRB-Z_EPS)*P_Q(i))**2*Z_PW_HAT 
  Z_PW_HAT = 0.0_JPRB 

                  
ENDDO

! should already be set to zero - but put in.

P_REFRAC_HAT(:) = 0.0_JPRB 

IF (LHOOK) CALL DR_HOOK('GPSCALC_REFRACAD',1,ZHOOK_HANDLE)

END SUBROUTINE gpscalc_refracAD
