SUBROUTINE gpscalc_refracTL(K_NLEV,        &
 & pres,       &
 & pres_prime, &
 & P_TEMP,       &
 & P_TEMP_PRIME,  &
 & P_Q,          &
 & P_Q_PRIME,    &
 & PZ_DRY,    &
 & PZ_DRY_PRIME,    &
 & PZ_WET,   &  
 & PZ_WET_PRIME,   &  
 & P_REFRAC,     &
 & P_REFRAC_PRIME)  

!**** *gpscalc_refracTL* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2004/10/26
!     Based on a EUMETSAT GRAS-SAF routine that was Dr'd automatically by the CY27 merge.  

!     PURPOSE
!     -------
!     It calculates the refractivity values on the model levels :

!**   INTERFACE
!     ---------

!     CALL gpscalc_refracTL
!     (K_NLEV,pres,pres_prime,P_TEMP,P_TEMP_PRIME,P_Q,P_Q_PRIME,P_REFRAC,P_REFRAC_PRIME)  
!     WHERE K_NLEV         = number of full model levels
!           pres           = pressure on the full model levels
!           pres_prime     = perturbed pressure on the full model levels
!           P_TEMP         = temperature on the full model levels
!           P_TEMP_PRIME   = perturbed temperature on the full model levels
!           P_Q            = specific humidity on full model levels
!           P_Q_PRIME      = perturbed  specific humidity on full model levels
!           P_REFRAC       = refractivity on full model levels 
!           P_REFRAC_PRIME = perturbed refractivity on full model levels 


!     gpscalc_refracTL is called from gpsro_tl

!**   METHOD
!     ------
!     1) calculate the dry and wey refractivity terms and add them. 

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------
               
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS, ONLY : RMDI
USE YOMCST, ONLY  : rmv, rmd   

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: K_NLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES_PRIME(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_TEMP(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_TEMP_PRIME(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_Q(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_Q_PRIME(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ_DRY(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ_DRY_PRIME(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ_WET(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ_WET_PRIME(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_REFRAC(K_NLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_REFRAC_PRIME(K_NLEV) 

! local variables

REAL(KIND=JPRB), PARAMETER :: z_aval = 0.77643_JPRB
REAL(KIND=JPRB), PARAMETER :: z_bval = 3.75463E3_JPRB
REAL(KIND=JPRB), PARAMETER :: z_cval = 0.712952_JPRB


INTEGER(KIND=JPIM) :: i                          ! counter
REAL(KIND=JPRB)    :: Z_NDRY,Z_NWET              ! wet and dry refractivity
REAL(KIND=JPRB)    :: Z_NDRY_PRIME,Z_NWET_PRIME  ! " perturbed
REAL(KIND=JPRB)    :: Z_PW                       ! water vapour pressure  
REAL(KIND=JPRB)    :: Z_PW_PRIME                 ! water vapour pressure  
REAL(KIND=JPRB)    :: Z_PD 
REAL(KIND=JPRB)    :: Z_PD_PRIME 
REAL(KIND=JPRB)    :: Z_EPS                      ! ratio of vapour/dry molecular weights
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! set the ratio of molecular weights (vapour/dry)

IF (LHOOK) CALL DR_HOOK('GPSCALC_REFRACTL',0,ZHOOK_HANDLE)
Z_EPS = rmv/rmd

! initialise the refractivity to missing

P_REFRAC(:)  = RMDI
P_REFRAC_PRIME(:) = 0.0_JPRB

DO i = 1, K_NLEV

! vapour pressure

  Z_PW = pres(i)*P_Q(i)/(Z_EPS+(1.0_JPRB-Z_EPS)*P_Q(i))
  
  Z_PW_PRIME = Z_PW/pres(i)*pres_prime(i) + &
  &         Z_EPS*pres(i)/(Z_EPS+(1.0_JPRB-Z_EPS)*P_Q(i))**2*P_Q_PRIME(i)        

! dry pressure

  Z_PD = pres(i) - Z_PW
  Z_PD_PRIME = pres_prime(i) - Z_PW_PRIME
  

! dry refrac

  Z_NDRY = z_aval*Z_PD/P_TEMP(i)   

  Z_NDRY_PRIME = Z_NDRY/Z_PD*Z_PD_PRIME - &
   & Z_NDRY/P_TEMP(i)*P_TEMP_PRIME(i)  

                    
  Z_NWET = Z_PW/P_TEMP(i)*(z_bval/P_TEMP(i)+ z_cval)  
   
  Z_NWET_PRIME = (z_bval/P_TEMP(i)+z_cval)/P_TEMP(i)*Z_PW_PRIME - &
  &               Z_PW*(2.0_JPRB*z_bval/P_TEMP(i)+z_cval)/P_TEMP(i)**2*P_TEMP_PRIME(i)
       
         
  P_REFRAC(i) = Z_NDRY*PZ_DRY(i) + Z_NWET*PZ_WET(i)
  
  P_REFRAC_PRIME(i) = Z_NDRY_PRIME*PZ_DRY(i) + Z_NWET_PRIME*PZ_WET(i) + &
 &                    Z_NDRY*PZ_DRY_PRIME(i) + Z_NWET*PZ_WET_PRIME(i)   
   
ENDDO

IF (LHOOK) CALL DR_HOOK('GPSCALC_REFRACTL',1,ZHOOK_HANDLE)

END SUBROUTINE gpscalc_refracTL
