SUBROUTINE gpspderivsad(klev,   & ! no.of observations
                      & khoriz, & ! no. of horizontal layers  ODD
                      & kk, &
                      & pdsep, &
                      & ptheta_min, & 
                      & ptheta_max, & 
                      & ptheta_tan, & 
                      & prtan, &
                      & prtan_hat, &
                      & pamult, &
                      & prefrac, &
                      & prefrac_hat, &
                      & pradius, &
                      & pradius_hat,&
                      & py, &
                      & py_hat, &
                      & pdydh_hat)
                   
!**** *gpspderivsad* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2005/08/01
!     Based on a GRAS-SAF routine.   

!     PURPOSE
!     -------
!     It evaluates the derivs for the Runge-Kutta routines

!**   INTERFACE
!     ---------

!     CALL gpspderivstl(....)    

!     WHERE klev   =  number of vertical levels
!           khoriz =  number of horizontal locations
!           kk      = in the kkth box in the horizontal
!           pdsep  =  angular spacing
!           ptheta_min = minium theta 
!           ptheta_max = maximum theta 
!           ptheta_tan = theta at tangent point & 
!           prtan = radius of tangent point
!           pamult = 
!           prefrac = refractivity
!           prefrac_hat
!           pradius = radius value
!           pradius_hat
!           py = current location
!           py_hat
!           pdydh = derivs used in the RK calculation
!           pdydh_hat

!     gpspderivsad is called from gpscalc_alpharkad

!**   METHOD
!     ------
!     1) calculate the derivs. used in the Runge-Kutta 

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2005/08/01 : Original
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE
                   
INTEGER(KIND=JPIM), INTENT(IN)  :: klev           ! no. of refractivity levels
INTEGER(KIND=JPIM), INTENT(IN)  :: khoriz         ! no. of horizontal locations
INTEGER(KIND=JPIM), INTENT(IN)  :: kk 
REAL(KIND=JPRB),    INTENT(IN)  :: pdsep           ! angular spacing of grid
REAL(KIND=JPRB),    INTENT(IN)  :: ptheta_min
REAL(KIND=JPRB),    INTENT(IN)  :: ptheta_max
REAL(KIND=JPRB),    INTENT(IN)  :: ptheta_tan
REAL(KIND=JPRB),    INTENT(IN)  :: prtan
REAL(KIND=JPRB),    INTENT(INOUT) :: prtan_hat    
REAL(KIND=JPRB),    INTENT(IN)    :: pamult          !
REAL(KIND=JPRB),    INTENT(IN)    :: prefrac(klev,khoriz)   ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(INOUT) :: prefrac_hat(klev,khoriz)   ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(IN)    :: pradius(klev,khoriz)   ! radius values
REAL(KIND=JPRB),    INTENT(INOUT) :: pradius_hat(klev,khoriz)   ! radius values
REAL(KIND=JPRB),    INTENT(IN)    :: py(4)   !  current location
REAL(KIND=JPRB),    INTENT(INOUT) :: py_hat(4)   !  current location
REAL(KIND=JPRB),    INTENT(INOUT) :: pdydh_hat(4)

! local

INTEGER(KIND=JPIM) :: ik,ikp1
REAL(KIND=JPRB) :: zhwt1,zhwt2
REAL(KIND=JPRB) :: zhwt1_hat,zhwt2_hat
REAL(KIND=JPRB) :: zref_up,zref_low
REAL(KIND=JPRB) :: zrad_up,zrad_low
REAL(KIND=JPRB) :: zref_up_hat,zref_low_hat
REAL(KIND=JPRB) :: zrad_up_hat,zrad_low_hat

REAL(KIND=JPRB) :: zkval,zdndr,zdndr2
REAL(KIND=JPRB) :: zkval_hat,zdndr_hat,zdndr2_hat
REAL(KIND=JPRB) :: zdydh(4)
REAL(KIND=JPRB) :: zed,zed_hat,zed_dum
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GPSPDERIVSAD',0,ZHOOK_HANDLE)


! local adjoint variables

zref_up_hat = 0.0_JPRB
zref_low_hat = 0.0_JPRB
zrad_up_hat = 0.0_JPRB
zrad_low_hat = 0.0_JPRB
zhwt1_hat = 0.0_JPRB
zhwt2_hat = 0.0_JPRB
zkval_hat = 0.0_JPRB
zdndr_hat = 0.0_JPRB
zdndr2_hat = 0.0_JPRB
zed_hat = 0.0_JPRB

! the easy bits

IF (COS(py(3)) < 1.0E-10_JPRB) THEN

   zdydh(1) = 1.0E-10_JPRB

ELSE

   zdydh(1) = COS(py(3))

ENDIF 

zdydh(2) = pamult*SIN(py(3))/(py(1)+prtan)

IF ( py(2) >= ptheta_min .AND. py(2) <= ptheta_max) THEN

   ik = INT((py(2) + ptheta_tan)/pdsep)+1
   ik = MIN(MAX(1,ik),khoriz)
   ikp1 = MIN(khoriz,ik+1)        

! horizontal weighting factor               
               
   zhwt1 = (REAL(ik)*pdsep - (py(2)+ptheta_tan))/pdsep       
   zhwt2 = 1.0_JPRB - zhwt1
   
ELSEIF (py(2) < ptheta_min ) THEN

   ik = 1
   ikp1 = 2
   zhwt1 = 1.0_JPRB
   zhwt2 = 0.0_JPRB
   
ELSEIF  (py(2) > ptheta_max) THEN

   ik = khoriz -1 
   ikp1 = khoriz
   zhwt1 = 0.0_JPRB
   zhwt2 = 1.0_JPRB
   
ENDIF    

zref_up  = zhwt1*prefrac(kk+1,ik)+zhwt2*prefrac(kk+1,ikp1)

zref_low = zhwt1*prefrac(kk,ik) + zhwt2*prefrac(kk,ikp1)

zrad_up = zhwt1*pradius(kk+1,ik)+zhwt2*pradius(kk+1,ikp1)

zrad_low = zhwt1*pradius(kk,ik) + zhwt2*pradius(kk,ikp1)

! radial gradient of refractivity

zkval = LOG(zref_low/zref_up)/(zrad_up-zrad_low)

zed = MAX(0.0_JPRB,(py(1)+prtan-zrad_low))
zed_dum = py(1)+prtan-zrad_low

!!!zdndr = - 1.0E-6_JPRB*zkval*zref_low*EXP(-zkval*(py(1)+prtan-zrad_low))

zdndr = - 1.0E-6_JPRB*zkval*zref_low*EXP(-zkval*zed)

zdndr2 = MAX(-1.5E-7_JPRB,zdndr)

zdydh(3) = -SIN(py(3))*(1.0_JPRB/(py(1)+prtan) + zdndr2)

zdydh(4) = - SIN(py(3))*zdndr


py_hat(3) = py_hat(3) - COS(py(3))*zdndr*pdydh_hat(4)
zdndr_hat = zdndr_hat - SIN(py(3))*pdydh_hat(4)
pdydh_hat(4) = 0.0_JPRB


py_hat(3) = py_hat(3) - COS(py(3))*(1.0_JPRB/(py(1)+prtan) + zdndr2)*pdydh_hat(3)
zdndr2_hat = zdndr2_hat - SIN(py(3))*pdydh_hat(3)
py_hat(1) = py_hat(1) + SIN(py(3))/(py(1)+prtan)**2*pdydh_hat(3)
prtan_hat = prtan_hat + SIN(py(3))/(py(1)+prtan)**2*pdydh_hat(3)
pdydh_hat(3) = 0.0_JPRB

IF (zdndr > -1.5E-7_JPRB) THEN

   zdndr_hat = zdndr_hat + zdndr2_hat
   zdndr2_hat = 0.0_JPRB

ELSE

   zdndr2_hat = 0.0_JPRB

ENDIF


zkval_hat = zkval_hat + zdndr*(1.0_JPRB/zkval - zed)*zdndr_hat
zref_low_hat = zref_low_hat + zdndr/zref_low*zdndr_hat
zed_hat = zed_hat - zdndr*zkval*zdndr_hat 
zdndr_hat = 0.0_JPRB

IF (zed_dum < 0.0_JPRB) THEN

   zed_hat = 0.0_JPRB
   
ELSE

   py_hat(1) = py_hat(1) + zed_hat
   prtan_hat = prtan_hat + zed_hat
   zrad_low_hat = zrad_low_hat - zed_hat
   zed_hat = 0.0_JPRB    

ENDIF


zref_low_hat = zref_low_hat + zkval_hat/(zref_low*(zrad_up-zrad_low))
zref_up_hat = zref_up_hat - zkval_hat/(zref_up*(zrad_up-zrad_low))
zrad_low_hat = zrad_low_hat + zkval/(zrad_up-zrad_low)*zkval_hat
zrad_up_hat = zrad_up_hat - zkval/(zrad_up-zrad_low)*zkval_hat
zkval_hat = 0.0_JPRB

pradius_hat(kk,ik) = pradius_hat(kk,ik) + zhwt1*zrad_low_hat
zhwt1_hat = zhwt1_hat + pradius(kk,ik)*zrad_low_hat
pradius_hat(kk,ikp1) = pradius_hat(kk,ikp1) + zhwt2*zrad_low_hat
zhwt2_hat = zhwt2_hat + pradius(kk,ikp1)*zrad_low_hat
zrad_low_hat = 0.0_JPRB

pradius_hat(kk+1,ik) = pradius_hat(kk+1,ik) + zhwt1*zrad_up_hat
zhwt1_hat = zhwt1_hat + pradius(kk+1,ik)*zrad_up_hat
pradius_hat(kk+1,ikp1) = pradius_hat(kk+1,ikp1) + zhwt2*zrad_up_hat
zhwt2_hat = zhwt2_hat + pradius(kk+1,ikp1)*zrad_up_hat
zrad_up_hat = 0.0_JPRB


prefrac_hat(kk,ik) = prefrac_hat(kk,ik) + zhwt1*zref_low_hat
zhwt1_hat = zhwt1_hat + prefrac(kk,ik)*zref_low_hat
prefrac_hat(kk,ikp1) = prefrac_hat(kk,ikp1) + zhwt2*zref_low_hat
zhwt2_hat = zhwt2_hat + prefrac(kk,ikp1)*zref_low_hat 
zref_low_hat = 0.0_JPRB
 
prefrac_hat(kk+1,ik) = prefrac_hat(kk+1,ik) + zhwt1*zref_up_hat
zhwt1_hat = zhwt1_hat + prefrac(kk+1,ik)*zref_up_hat
prefrac_hat(kk+1,ikp1) = prefrac_hat(kk+1,ikp1) + zhwt2*zref_up_hat
zhwt2_hat = zhwt2_hat + prefrac(kk+1,ikp1)*zref_up_hat 
zref_up_hat = 0.0_JPRB


IF ( py(2) >= ptheta_min .AND. py(2) <= ptheta_max) THEN

   zhwt1_hat = zhwt1_hat - zhwt2_hat
   zhwt2_hat = 0.0_JPRB
   
   py_hat(2) = py_hat(2) - zhwt1_hat/pdsep
   zhwt1_hat = 0.0_JPRB  
   
ELSEIF (py(2) < ptheta_min ) THEN

   zhwt1_hat = 0.0_JPRB
   zhwt2_hat = 0.0_JPRB
   
ELSEIF (py(2) > ptheta_max) THEN

   zhwt1_hat = 0.0_JPRB
   zhwt2_hat = 0.0_JPRB
   
ENDIF    

py_hat(3) = py_hat(3) + (pamult*COS(py(3)))/(py(1)+prtan)*pdydh_hat(2)
py_hat(1) = py_hat(1) - zdydh(2)/(py(1)+prtan)*pdydh_hat(2)
prtan_hat = prtan_hat - zdydh(2)/(py(1)+prtan)*pdydh_hat(2)
pdydh_hat(2) = 0.0_JPRB


IF (COS(py(3)) < 1.0E-10_JPRB) THEN

   pdydh_hat(1) = 0.0_JPRB

ELSE

   py_hat(3) = py_hat(3) - SIN(py(3))*pdydh_hat(1)
   pdydh_hat(1) = 0.0_JPRB

ENDIF 

IF (LHOOK) CALL DR_HOOK('GPSPDERIVSAD',1,ZHOOK_HANDLE)

END SUBROUTINE gpspderivsad



