SUBROUTINE gpspderivstl(klev,   & ! no.of observations
                      & khoriz, & ! no. of horizontal layers  ODD
                      & kk, &
                      & pdsep, &
                      & ptheta_min, & 
                      & ptheta_max, & 
                      & ptheta_tan, & 
                      & prtan, &
                      & prtan_prime, &
                      & pamult, &
                      & prefrac, &
                      & prefrac_prime, &
                      & pradius, &
                      & pradius_prime,&
                      & py, &
                      & py_prime, &
                      & pdydh, &
                      & pdydh_prime)

!**** *gpspderivstl* - part of the GPSRO observation operator.

!     S. Healy             ECMWF          2005/08/01
!     Based on a GRAS-SAF routine.   

!     PURPOSE
!     -------
!     It evaluates the derivs for the Runge-Kutta routines

!**   INTERFACE
!     ---------

!     CALL gpspderivstl(....)    

!     WHERE klev   =  number of vertical levels
!           khoriz =  number of horizontal locations
!           kk      = in the kkth box in the horizontal
!           pdsep  =  angular spacing
!            ptheta_min = minium theta 
!           ptheta_max = maximum theta 
!            ptheta_tan = theta at tangent point & 
!            prtan = radius of tangent point
!           pamult = 
!           prefrac = refractivity
!           prefrac_prime
!           pradius = radius value
!           pradius_prime
!           py = current location
!           py_prime
!           pdydh = derivs used in the RK calculation
!           pdydh_prime

!     gpspderivstl is called from gpscalc_alpharktl

!**   METHOD
!     ------
!     1) calculate the derivs. used in the Runge-Kutta 

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2005/08/01 : Original
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

                   
IMPLICIT NONE

                   
INTEGER(KIND=JPIM), INTENT(IN)  :: klev           ! no. of refractivity levels
INTEGER(KIND=JPIM), INTENT(IN)  :: khoriz         ! no. of horizontal locations
INTEGER(KIND=JPIM), INTENT(IN)  :: kk 
REAL(KIND=JPRB),    INTENT(IN)  :: pdsep           ! angular spacing of grid
REAL(KIND=JPRB),    INTENT(IN)  :: ptheta_min
REAL(KIND=JPRB),    INTENT(IN)  :: ptheta_max
REAL(KIND=JPRB),    INTENT(IN)  :: ptheta_tan
REAL(KIND=JPRB),    INTENT(IN)  :: prtan
REAL(KIND=JPRB),    INTENT(IN)  :: prtan_prime    
REAL(KIND=JPRB),    INTENT(IN)  :: pamult          !
REAL(KIND=JPRB),    INTENT(IN)  :: prefrac(klev,khoriz)   ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(IN)  :: prefrac_prime(klev,khoriz)   ! refractivity values on levels
REAL(KIND=JPRB),    INTENT(IN)  :: pradius(klev,khoriz)   ! radius values
REAL(KIND=JPRB),    INTENT(IN)  :: pradius_prime(klev,khoriz)   ! radius values
REAL(KIND=JPRB),    INTENT(IN)  :: py(4)   !  current location
REAL(KIND=JPRB),    INTENT(IN)  :: py_prime(4)   !  current location
REAL(KIND=JPRB),    INTENT(OUT) :: pdydh(4)
REAL(KIND=JPRB),    INTENT(OUT) :: pdydh_prime(4)

! local

INTEGER(KIND=JPIM) :: ik,ikp1
REAL(KIND=JPRB) :: zhwt1,zhwt2
REAL(KIND=JPRB) :: zhwt1_prime,zhwt2_prime
REAL(KIND=JPRB) :: zref_up,zref_low
REAL(KIND=JPRB) :: zrad_up,zrad_low
REAL(KIND=JPRB) :: zref_up_prime,zref_low_prime
REAL(KIND=JPRB) :: zrad_up_prime,zrad_low_prime

REAL(KIND=JPRB) :: zkval,zdndr,zdndr2
REAL(KIND=JPRB) :: zkval_prime,zdndr_prime,zdndr2_prime
REAL(KIND=JPRB) :: zed,zed_prime
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GPSPDERIVSTL',0,ZHOOK_HANDLE)


! the easy bits


IF (COS(py(3)) < 1.0E-10_JPRB) THEN

   pdydh(1) = 1.0E-10_JPRB
   pdydh_prime(1) = 0.0_JPRB

ELSE

   pdydh(1) = COS(py(3))
   pdydh_prime(1) = - SIN(py(3))*py_prime(3)


ENDIF 


pdydh(2) = pamult*SIN(py(3))/(py(1)+prtan)
pdydh_prime(2) = (pamult*COS(py(3))*py_prime(3)-pdydh(2)* &
& (py_prime(1) + prtan_prime))/(py(1)+prtan)

IF ( py(2) >= ptheta_min .AND. py(2) <= ptheta_max) THEN

   ik = INT((py(2) + ptheta_tan)/pdsep)+1
   ik = MIN(MAX(1,ik),khoriz)
   ikp1 = MIN(khoriz,ik+1)        

! horizontal weighting factor               
               
   zhwt1 = (REAL(ik)*pdsep - (py(2)+ptheta_tan))/pdsep 
   zhwt1_prime = - py_prime(2)/pdsep
      
   zhwt2 = 1.0_JPRB - zhwt1
   zhwt2_prime = - zhwt1_prime
   
   
ELSEIF (py(2) < ptheta_min ) THEN

   ik = 1
   ikp1 = 2
   zhwt1 = 1.0_JPRB
   zhwt1_prime = 0.0_JPRB
   zhwt2 = 0.0_JPRB
   zhwt2_prime = 0.0_JPRB
   
ELSEIF  (py(2) > ptheta_max) THEN

   ik = khoriz -1 
   ikp1 = khoriz
   zhwt1 = 0.0_JPRB
   zhwt1_prime = 0.0_JPRB
   zhwt2 = 1.0_JPRB
   zhwt2_prime = 0.0_JPRB
   
ENDIF    


zref_up  = zhwt1*prefrac(kk+1,ik)+zhwt2*prefrac(kk+1,ikp1)

zref_up_prime = zhwt1*prefrac_prime(kk+1,ik) + &
              & prefrac(kk+1,ik)*zhwt1_prime + &
              & zhwt2*prefrac_prime(kk+1,ikp1) + &
              & prefrac(kk+1,ikp1)*zhwt2_prime 


zref_low = zhwt1*prefrac(kk,ik) + zhwt2*prefrac(kk,ikp1)

zref_low_prime = zhwt1*prefrac_prime(kk,ik) + &
               & prefrac(kk,ik)*zhwt1_prime + &
               & zhwt2*prefrac_prime(kk,ikp1) + &
               & prefrac(kk,ikp1)*zhwt2_prime 


zrad_up = zhwt1*pradius(kk+1,ik)+zhwt2*pradius(kk+1,ikp1)

zrad_up_prime = zhwt1*pradius_prime(kk+1,ik) + &
              & pradius(kk+1,ik)*zhwt1_prime + &
              & zhwt2*pradius_prime(kk+1,ikp1) + &
              & pradius(kk+1,ikp1)*zhwt2_prime 

zrad_low = zhwt1*pradius(kk,ik) + zhwt2*pradius(kk,ikp1)

zrad_low_prime = zhwt1*pradius_prime(kk,ik) + &
               & pradius(kk,ik)*zhwt1_prime + &
               & zhwt2*pradius_prime(kk,ikp1) + &
               & pradius(kk,ikp1)*zhwt2_prime 


! radial gradient of refractivity

zkval = LOG(zref_low/zref_up)/(zrad_up-zrad_low)


zkval_prime = (zref_low_prime/zref_low - zref_up_prime/zref_up + &
            &  zkval*(zrad_low_prime - zrad_up_prime))/(zrad_up-zrad_low)


zed = MAX((py(1)+prtan-zrad_low),0.0_JPRB)

IF ((py(1)+prtan-zrad_low) >  0.0_JPRB) THEN

   zed_prime = py_prime(1) + prtan_prime - zrad_low_prime
   
ELSE   

   zed_prime = 0.0_JPRB

ENDIF 

!!zdndr = - 1.0E-6_JPRB*zkval*zref_low*EXP(-zkval*(py(1)+prtan-zrad_low))

zdndr = - 1.0E-6_JPRB*zkval*zref_low*EXP(-zkval*zed)

zdndr_prime = zdndr*((1.0_JPRB/zkval - zed)*zkval_prime + &
            &        zref_low_prime/zref_low - &
            &        zkval*zed_prime)

zdndr2 = MAX(-1.5E-7_JPRB,zdndr)

IF (zdndr > -1.5E-7_JPRB) THEN

   zdndr2_prime = zdndr_prime

ELSE

   zdndr2_prime = 0.0_JPRB

ENDIF


pdydh(3) = -SIN(py(3))*(1.0_JPRB/(py(1)+prtan) + zdndr2)
   
pdydh_prime(3)  = - COS(py(3))*(1.0_JPRB/(py(1)+prtan) + zdndr2)*py_prime(3) - &
                &   SIN(py(3))*(zdndr2_prime - 1.0_JPRB/(py(1)+prtan)**2* &
                &   (py_prime(1) + prtan_prime))


pdydh(4) = - SIN(py(3))*zdndr

pdydh_prime(4)  = - COS(py(3))*zdndr*py_prime(3) - SIN(py(3))*zdndr_prime


IF (LHOOK) CALL DR_HOOK('GPSPDERIVSTL',1,ZHOOK_HANDLE)

END SUBROUTINE gpspderivstl



