 SUBROUTINE gpsro_2dop(kdlen,kdbdy,KHORIZ,K_IMXCOUNT,klen,K_IVNMRQ,presf5,   &
                  & P_TF5,P_QF5,P_GEOPF5,P_ZXPP)  

!**** *gpsro_op* - Operator routine for gps radio occultation observations.

!     S. Healy             ECMWF          2004/10/26
!     Based on a EUMETSAT GRAS-SAF routine that was Dr'd automatically by the CY27 merge.  

!     PURPOSE
!     -------
!     It simulates GPSRO bending angles with a 1D operator :

!**   INTERFACE
!     ---------

!    CALL gpsro_op(kdlen,kdbdy,K_IMXCOUNT,klen,K_IVNMRQ,presf5,P_TF5,P_QF5,P_GEOPF5,P_ZXPP)  
!     WHERE KDLEN      = First dimension: Max number of observations, this set (INPUT)
!           KDBDY      = Second dimension: Max number of bodies, this set (INPUT)
!           K_IMXCOUNT = Maximum number of bending angles per body (INPUT)
!           klen       = Number of bodies (number of bending angle profiles) (INPUT)
!           K_IVNMRQ   = Identifier of variable type (INPUT)
!           presf5     = Pressure on full levels (INPUT)
!           P_TF5      = Temperature on full levels (INPUT)
!           P_QF5      = Specific humidity on full levels (INPUT)
!           P_GEOPF5   = Geopotential on full levels (INPUT)
!           P_ZXPP     = Bending angle values (OUTPUT)  

!     gpsro_op is called from HOP

!**   METHOD
!     ------
!     1) Extract the impact parameter values from ROBODY
!     2) Copy the temperature, humidity and pressure data into temporary arrays
!     3) Calculate the refractivity on the model (full) levels
!     4) Calculate the "nr" product on full levels
!     5) Evaluate the bending angles    

!**   EXTERNALS
!     ---------
!        gpscalc_refrac2d - Calculate the refractivity on model (full) levels
!        gpscalc_nr2d     - Calculate the refractive index - radius product on full levels
!        gpscalc_alpha2d  - Calculate the bending angles

!**   MODIFICATIONS
!     -------------
!     Sean Healy      2004/10/26 : Original
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS  , ONLY : rdegrees,RMDI
USE YOMDIM, ONLY : nflevg
USE YOMDB
USE YOMVNMB  , ONLY : NVNUMB
USE YOMCST   , ONLY : RG
USE YOMLIMB, ONLY : dtheta


IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDBDY
INTEGER(KIND=JPIM),INTENT(IN)    :: KHORIZ 
INTEGER(KIND=JPIM),INTENT(IN)    :: K_IMXCOUNT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN
 
INTEGER(KIND=JPIM),INTENT(IN)    :: K_IVNMRQ(KDLEN,KDBDY) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG,KHORIZ)   ! pressure on **full levels**
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_TF5(KDLEN,0:NFLEVG,KHORIZ)  ! Z_TEMP. values     "
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_QF5(KDLEN,0:NFLEVG,KHORIZ)  ! spec.humid values "
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_GEOPF5(KDLEN,NFLEVG,KHORIZ) ! geopotential heights "
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_ZXPP(KDLEN,K_IMXCOUNT)      ! bending angles          

! local variables           


INTEGER(KIND=JPIM) :: i,j,I_K,I_L,isplit 
REAL(KIND=JPRB) :: Z_LAT                     ! latitude 
REAL(KIND=JPRB) :: Z_ROC                     ! radius of curvature
REAL(KIND=JPRB) :: Z_PRES(nflevg,KHORIZ)     ! pressure values
REAL(KIND=JPRB) :: Z_TEMP(nflevg,KHORIZ)     ! temperature values
REAL(KIND=JPRB) :: Z_Q(nflevg,KHORIZ)        ! specific humidity values
REAL(KIND=JPRB) :: zg(nflevg,KHORIZ)         ! geopotential heights
REAL(KIND=JPRB) :: Z_REFRAC(nflevg,KHORIZ)   ! refractivity values
REAL(KIND=JPRB) :: Z_RAD(nflevg,KHORIZ)      ! radius value
REAL(KIND=JPRB) :: Z_NR(nflevg,KHORIZ)       ! radius - refractive index product  
REAL(KIND=JPRB) :: Z_A(K_IMXCOUNT)           ! impact parameter values
REAL(KIND=JPRB) :: Z_ALPHA(K_IMXCOUNT)       ! bending angles
REAL(KIND=JPRB) :: Z_APATH(K_IMXCOUNT,2)     ! impact parameter calculated at ray endpoints
REAL(KIND=JPRB) :: Z_2D
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "gpscalc_nr2d.intfb.h"
#include "gpscalc_refrac2d.intfb.h"
#include "gpscalc_alpharkm2.intfb.h"

#include "openmp_obs.h"


IF (LHOOK) CALL DR_HOOK('GPSRO_2DOP',0,ZHOOK_HANDLE)

! initialise zxpp

P_ZXPP(:,:) = RMDI

! set the splitting and the 2d calculation height limit

isplit = 4
Z_2D = 1.0E4_JPRB

! loop through each ob

DO i = 1, klen

! lat and radius of curvature at each location

  Z_LAT = ROBHDR(i,MDBLAT)*rdegrees
  Z_ROC = ROBHDR(i,MDBRADCURV)+ ROBHDR(i,MDB_UNDULATION)
  
! extract the impact parameters from ROBODY

  Z_A(:) = RMDI
   
  j=1
  I_L=1
       
  DO I_K = MLNKH2B(i),MLNKH2B(i+1) - 1
     
    IF (K_IVNMRQ(i,I_L) == nvnumb(76)) THEN

! copy the impact parameter value
      
      Z_A(j) = ROBODY(I_K,MDBPPP) 

      j=j+1

    ENDIF 
      
    I_L = I_L + 1
      
    IF (j > K_IMXCOUNT) EXIT
            
  ENDDO
  
! copy pressure, temp, humidity and geopotential height in temporary arrays 
! and reverse ordering

  I_K = nflevg
   
  DO j = 1,nflevg
   
    Z_PRES(I_K,:) = presf5(i,j,:)
    Z_TEMP(I_K,:) = P_TF5(i,j,:) 
    Z_Q(I_K,:)    = P_QF5(i,j,:)
    zg(I_K,:)     = P_GEOPF5(i,j,:)/RG       ! geopotential height in m 
    
    I_K = I_K-1
      
  ENDDO        
 
! calculate the refractivity on model levels

  CALL gpscalc_refrac2d(nflevg, &
   & KHORIZ,  &
   & Z_PRES,  &
   & Z_TEMP,  &
   & Z_Q,     &
   & Z_REFRAC)  ! output   


! calculate the nr product
 
  CALL gpscalc_nr2d(nflevg, &
   & KHORIZ,    &
   & Z_ROC,     &
   & Z_LAT,     &
   & zg,        &
   & Z_REFRAC,  &
   & Z_RAD,     &
   & Z_NR)  
   
! calculate the bending angles
 
  CALL gpscalc_alpharkm2(K_IMXCOUNT, &
   & nflevg,     &
   & KHORIZ,     &
   & isplit,     &
   & dtheta,     &
   & Z_A,        &
   & Z_REFRAC,   &
   & Z_RAD,      &
   & Z_NR,       &
   & Z_ROC,      &
   & Z_2D,       &
   & Z_APATH,    &
   & Z_ALPHA)  
   

! Now store the bending angles

  P_ZXPP(i,1:K_IMXCOUNT) = Z_ALPHA(1:K_IMXCOUNT)           
   
ENDDO

IF (LHOOK) CALL DR_HOOK('GPSRO_2DOP',1,ZHOOK_HANDLE)

END SUBROUTINE gpsro_2dop
