SUBROUTINE GPSZEN_DELAY(KDLEN,KLEN,KDCOUNT,KCOUNT,&
 &                      PRESH5,PGEOH5,PTF5,PQF5,&
 &                      POROGOB,PRESOB,&
 &                      KDPP,PZEN_DELAY, &
 &                      PZHD_DELAY,PZWD_DELAY)

! Observation operator to calculate Atmospheric Path Delay

! INPUT: KDLEN,KLEN,KDCOUNT,KCOUNT,KDPP
!        PRESH5,PGEOH5,PTF5,PQF5,POROGOB,PRESOB
! OUTPUT: PZEN_DELAY             : ZTD in meters
!         PZHD_DELAY (OPTIONAL)  : ZHD in meters
!         PZWD_DELAY (OPTIONAL)  : ZWD in meters

!  xx/xx/2004  S.B. Healy      Original code
!  23/11/2004  P. Poli         Removed ODB calls
!  20/01/2006  P. Poli         Added ZHD and ZWD as optional outputs

USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCST   , ONLY : RG, RMV, RMD
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN) :: KDLEN, KLEN,KDCOUNT,KCOUNT(KDLEN),KDPP
REAL(KIND=JPRB), INTENT(IN) :: PRESH5(KDLEN,0:NFLEVG),PGEOH5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB), INTENT(IN) :: PTF5(KDLEN,0:NFLEVG),PQF5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB), INTENT(IN) :: POROGOB(KDLEN),PRESOB(KDLEN,KDCOUNT)
REAL(KIND=JPRB), INTENT(OUT) :: PZEN_DELAY(KDLEN,KDCOUNT,KDPP)
REAL(KIND=JPRB), INTENT(OUT), OPTIONAL :: PZHD_DELAY(KDLEN,KDCOUNT,KDPP)
REAL(KIND=JPRB), INTENT(OUT), OPTIONAL :: PZWD_DELAY(KDLEN,KDCOUNT,KDPP)

! local variables

INTEGER(KIND=JPIM) :: JOBS,JDATA,JLEV,ILEVP1
REAL(KIND=JPRB) :: ZREFRACT,ZREFRACTA,ZREFRACTB,ZHUPPER,ZHLOWER
REAL(KIND=JPRB) :: ZPUPPER,ZPLOWER,ZPLOWERB,ZKAPPA,ZEPSILON
REAL(KIND=JPRB), PARAMETER :: AVAL = 7.76E-7_JPRB
REAL(KIND=JPRB), PARAMETER :: BVAL = 3.73E-3_JPRB
LOGICAL :: LL_FLAG_ZHD, LL_FLAG_ZWD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "openmp_obs.h"

!-------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPSZEN_DELAY',0,ZHOOK_HANDLE)

! 1. initialise the variables 
! ratio of water-vapour mass/dry-air mass

ZEPSILON = RMV/RMD
PZEN_DELAY(:,:,:) = 0.0_JPRB 

LL_FLAG_ZHD = .FALSE.
IF (PRESENT(PZHD_DELAY)) THEN
   PZHD_DELAY(:,:,:) = 0.0_JPRB
   LL_FLAG_ZHD = .TRUE.
ENDIF
LL_FLAG_ZWD = .FALSE.
IF (PRESENT(PZWD_DELAY)) THEN
   PZWD_DELAY(:,:,:) = 0.0_JPRB
   LL_FLAG_ZWD = .TRUE.
ENDIF

!-------------------------------------------------------------------------


! assume that the refractivity varies exponentially between 
! the half-levels, as a function of geopotential height. Also assume
! the vertical integral of refrac.(N) w.r.t geopotential height ~= 
! integral w.r.t. geometric height.     

DO JDATA = 1,KDCOUNT
  DO JLEV = 0,NFLEVG-1
    ILEVP1 = JLEV+1

    DO JOBS = 1, KLEN
      IF(JDATA > KCOUNT(JOBS)) CYCLE
      
! assume temp and humidity are constant between half-levels and
! calculate their contribution to the refractivity term
      ZREFRACTA = AVAL/PTF5(JOBS,ILEVP1)
      ZREFRACTB = BVAL*PQF5(JOBS,ILEVP1)/ &
    &           (PTF5(JOBS,ILEVP1)**2 * &
    &            (ZEPSILON+(1.0_JPRB-ZEPSILON)*PQF5(JOBS,ILEVP1)))
      ZREFRACT  = ZREFRACTA + ZREFRACTB
 
! skip the top 0.01 Pa, to avoid division by zero later
! note, PGEOH5(JOBS,0)/RG is roughly 75,000 m in the L60 model.
      ZPUPPER = MAX(PRESH5(JOBS,JLEV),0.01_JPRB) 
      ZPLOWER = PRESH5(JOBS,ILEVP1)
      ZHUPPER = PGEOH5(JOBS,JLEV)/RG
      ZHLOWER = PGEOH5(JOBS,ILEVP1)/RG
      ZKAPPA = LOG(ZPLOWER/ZPUPPER)/(ZHUPPER-ZHLOWER)

! this will extrapolate/interpolate pressure to the station pressure      
      IF(ZPUPPER >= PRESOB(JOBS,JDATA)) CYCLE
      IF(ILEVP1 == NFLEVG .OR. &
     &   PRESH5(JOBS,ILEVP1) > PRESOB(JOBS,JDATA)) THEN
         ZPLOWERB = PRESOB(JOBS,JDATA)
      ELSE
         ZPLOWERB = PRESH5(JOBS,ILEVP1)
      ENDIF
      
! add to total zenith delay integration, station by station
! assume pressure varies exponentially with geopotential height
      PZEN_DELAY(JOBS,JDATA,1) = PZEN_DELAY(JOBS,JDATA,1) + &
     &                   ZREFRACT*(ZPLOWERB-ZPUPPER)/ZKAPPA

      IF (LL_FLAG_ZHD) &
     & PZHD_DELAY(JOBS,JDATA,1) = PZHD_DELAY(JOBS,JDATA,1) + &
     &                 ZREFRACTA*(ZPLOWERB-ZPUPPER)/ZKAPPA

      IF (LL_FLAG_ZWD) &
     & PZWD_DELAY(JOBS,JDATA,1) = PZWD_DELAY(JOBS,JDATA,1) + &
     &                 ZREFRACTB*(ZPLOWERB-ZPUPPER)/ZKAPPA

    ENDDO

  ENDDO
ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GPSZEN_DELAY',1,ZHOOK_HANDLE)
END SUBROUTINE GPSZEN_DELAY
