SUBROUTINE GRG_FPARAM(PO3COL,PO3TCOL,PCC_COL,PTAUI,PTAUS,PPRES,PSURF,PALB,PSOL,PCRS_O3,&
 &                    PFACT)

!**** *GRG_FPARAM* - Approximates the actinic flux for a number of wavelengths

!     Purpose.
!     --------
!     Approximates the actinic flux for a number of wavelengths
!     Result is passed in "PFACT(1:7)

!**   Interface.
!     ----------
!     *CALL* *GRG_FPARAM(PO3COL,PO3TCOL,PCC_COL,PTAUI,PTAUS,PPRES,PSURF,PALB,PSOL,PCRS_O3,&
!                        & PFACT)

!        Explicit arguments :
!        --------------------
!        PO3COL  : HALF LEVEL O3 partial column      (INPUT)
!        PO3TCOL : Ozone total column in mol/cm2     (INPUT)
!        PCC_COL : overhead cloud cover              (INPUT)
!        PTAUI    : cloud optical thickness per level (INPUT)
!        PTAUS    : total cloud optical thickness     (INPUT)
!        PPRES   : pressure and full levels          (INPUT)
!        PSURF   : surface pressure                  (INPUT)
!        PALB    : surface albedo                    (INPUT)
!        PSOL    : solar zenith angle                (INPUT)
!        PCRS_O3  : ozone absorption cross section    (INPUT)
!        PFACT    : approximated actinic flux         (OUTPUT)

!        Implicit arguments :    None.
!        --------------------

!     Externals.   None.
!     ----------

!     Author.
!     -------
!        Antje Dethof based on TM5 routine provided by Michiel van Weene

!     Modifications.
!     --------------
!        Original : 21-May-2007

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCST   , ONLY : RPI
USE GRG_PHOTOLYSIS_MIX , ONLY :FINT

IMPLICIT NONE

REAL(KIND=JPRB),intent(in)                  :: PO3COL,PO3TCOL
REAL(KIND=JPRB),intent(in)                  :: PCC_COL,PTAUI,PTAUS
REAL(KIND=JPRB),intent(in)                  :: PPRES,PSURF,PALB,PSOL
REAL(KIND=JPRB),intent(in),DIMENSION(1:7)   :: PCRS_O3
REAL(KIND=JPRB),intent(out),DIMENSION(1:7)  :: PFACT

REAL(KIND=JPRB) :: ZTU,ZTUC,ZTL,ZTL2,ZTLC  !optical thicknesses lower/upper
REAL(KIND=JPRB) :: ZETU,ZETUC,ZETL,ZETLC  !exp(tau/mu)
REAL(KIND=JPRB) :: ZETO3               !for ozon or tau
REAL(KIND=JPRB) :: ZALA0,ZALAD
REAL(KIND=JPRB) :: ZALA0C,ZALADC,ZAL0,ZALD,ZAL0C,ZALDC
REAL(KIND=JPRB) :: ZETO3U
REAL(KIND=JPRB) :: ZAU,ZAU0,ZAU0C,ZAUC
REAL(KIND=JPRB) :: ZF0,ZF0C,ZFS,ZFSC
REAL(KIND=JPRB) :: ZFU,ZFUC,ZFD,ZFDC
REAL(KIND=JPRB) :: ZCREF
REAL(KIND=JPRB),PARAMETER      :: one_g = 1._JPRB-0.87_JPRB
REAL(KIND=JPRB),PARAMETER      :: one_g2 = 1._JPRB-0.87_JPRB*0.87_JPRB
INTEGER(KIND=JPIM)  :: IRD=2

INTEGER(KIND=JPIM),DIMENSION(5),PARAMETER  ::IBAND = (/ 3,4,5,6,7 /)
REAL(KIND=JPRB),DIMENSION(3:7),PARAMETER   ::ZTAUR = &
 & (/ 1.177e-5, 1.067e-5, 0.919e-5, &
 & 0.498e-5, 0.0782e-5 /)

INTEGER(KIND=JPIM) :: j,ip,ith ,IK
REAL(KIND=JPRB)    :: ZMU0,pp,ZDP,ZDTH,ZTTH
REAL(KIND=JPRB)    :: ZHOOK_HANDLE
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GRG_FPARAM',0,ZHOOK_HANDLE)

ZMU0 = cos(PSOL*RPI/180.)


!*       1.   Band 2:
!             actinic flux only depends on ozone absorption

j=2

pp = MAX(101._JPRB-1.e-3_JPRB*PPRES,0.0_JPRB)
ip = int(pp)
ZDP = pp-ip
ZTTH = MIN(PSOL,89.9999_JPRB)*0.2_JPRB
ith = int(ZTTH)
ZDTH = ZTTH-ith
ZCREF =( (1-ZDP)*((1-ZDTH)*fint(ip  ,ith  ,j)      &
          &       + ZDTH *fint(ip  ,ith+1,j))     &
          &       +  ZDP *((1-ZDTH)*fint(ip+1,ith  ,j)      &
          &       + ZDTH *fint(ip+1,ith+1,j)) )

ZETO3 = exp(-PCRS_O3(j)*PO3COL/ZMU0)
PFACT(j) =  ZCREF*ZETO3


!*       2.   Other Bands

DO IK=1,5   ! over wavelength intervals
   j = IBAND(IK)

   ZTU    = ZTAUR(j)*PPRES        !optical thickness upper atmosphere clear
   ZTUC   = ZTU + one_g*PTAUI
   ZTL    = ZTAUR(j)*(PSURF - PPRES) !lower atmosphere
   ZTL2   = MAX(0._JPRB,PTAUS-PTAUI)  !mvw  prevent negatives
   ZTLC   = ZTL + one_g*ZTL2     !clouds included
   ZETU   = exp(-ZTU/ZMU0)
   ZETUC  = ZETU*exp(-PTAUI*one_g2/ZMU0)     !includes cloud effects
   ZETL   = exp(-ZTL/ZMU0)
   ZETLC  = ZETL*exp(-ZTL2*one_g2/ZMU0)      !includes cloud effects
   ZALA0  = 1._JPRB-(2._JPRB+3._JPRB*ZMU0+(2._JPRB-3._JPRB*ZMU0)*ZETL)/(4._JPRB+3._JPRB*ZTL)
   ZALAD  = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ZTL)
   ZALA0C = 1._JPRB-(2._JPRB+3._JPRB*ZMU0+(2._JPRB-3._JPRB*ZMU0)*ZETLC)/(4._JPRB+3._JPRB*ZTLC)
   ZALADC = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ZTLC)
   ZAL0   = ZALA0   + (1._JPRB-ZALA0) * PALB * (1._JPRB-ZALAD)  / (1._JPRB-PALB*ZALAD)
   ZALD   = ZALAD   + (1._JPRB-ZALAD) * PALB * (1._JPRB-ZALAD)  / (1._JPRB-PALB*ZALAD)
   ZAL0C  = ZALA0C  + (1._JPRB-ZALA0C)* PALB * (1._JPRB-ZALADC) / (1._JPRB-PALB*ZALADC)
   ZALDC  = ZALADC  + (1._JPRB-ZALADC)* PALB * (1._JPRB-ZALADC) / (1._JPRB-PALB*ZALADC)
   ZAU    = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ZTU)
   ZAU0   = 1._JPRB-(2._JPRB+3._JPRB*ZMU0+(2._JPRB-3._JPRB*ZMU0)*ZETU) /(4._JPRB+3._JPRB*ZTU)
   ZAU0C  = 1._JPRB-(2._JPRB+3._JPRB*ZMU0+(2._JPRB-3._JPRB*ZMU0)*ZETUC)/(4._JPRB+3._JPRB*ZTUC)
   ZAUC   = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ZTUC)

   IF (j /= 6) THEN 
! actinic flux does depend on ozone absorption
     ZETO3  = exp(-PCRS_O3(j)*PO3COL/ZMU0)
     ZETO3U = exp(-PCRS_O3(j)*(PO3TCOL-PO3COL)*2.5)
! Direct radiation
     ZF0    = ZETO3*ZETU
     ZF0C   = ZETO3*ZETUC
! Downward diffuse, only from events above p(i)
     ZFS    = IRD *ZMU0*(1._JPRB-ZAU0-ZETU)  *ZETO3
     ZFSC   = IRD *ZMU0*(1._JPRB-ZAU0C-ZETUC)*ZETO3
!correct lower albedo for ozone absorbtion
     ZAL0   = ZAL0     *ZETO3U      
     ZALD   = ZALD     *ZETO3U
     ZAL0C  = ZAL0C    *ZETO3U
     ZALDC  = ZALDC    *ZETO3U

   ELSEIF (j==6) THEN
! actinic flux does not depend on ozone absorption
! Direct radiation
     ZF0    = ZETU   
     ZF0C   = ZETUC  
! Downward diffuse, only from events above p(i)
     ZFS    = IRD *ZMU0*(1._JPRB-ZAU0-ZETU)  
     ZFSC   = IRD *ZMU0*(1._JPRB-ZAU0C-ZETUC)

   ENDIF

! Upward diffuse radiation 

   ZFU   = (IRD *ZMU0*ZF0 *ZAL0  + ZALD *ZFS ) / (1._JPRB-ZALD *ZAU)
   ZFUC  = (IRD *ZMU0*ZF0C*ZAL0C + ZALDC*ZFSC) / (1._JPRB-ZALDC*ZAUC)

! Downward direct + diffuse radiation

   ZFD   = ZF0     + ZFS     + ZAU  *ZFU
   ZFDC  = ZF0C    + ZFSC    + ZAUC *ZFUC

!  Interpolate actinic flux Table

   pp = MAX(101._JPRB-1.e-3_JPRB*PPRES,0.0_JPRB)
   ip = int(pp)
   ZDP = pp-ip
   ZTTH = MIN(PSOL,89.9999_JPRB)*0.2_JPRB
   ith = int(ZTTH)
   ZDTH = ZTTH-ith
   ZCREF =( (1-ZDP)*((1-ZDTH)*fint(ip  ,ith  ,j)      &
            &        + ZDTH *fint(ip  ,ith+1,j))     &
            &        +  ZDP *((1-ZDTH)*fint(ip+1,ith  ,j)      &
            &        + ZDTH *fint(ip+1,ith+1,j)) )

! Calculate actinic flux, weighted with the cloud fraction

  PFACT(j) = (     PCC_COL  * (ZFDC + ZFUC) + &
            &  (1.-PCC_COL) * (ZFU + ZFD) ) * ZCREF
ENDDO

IF (LHOOK) CALL DR_HOOK('GRG_FPARAM',1,ZHOOK_HANDLE)
END SUBROUTINE GRG_FPARAM
