SUBROUTINE GRG_FPARAMAD(PO3COL,PO3TCOL,PCC_COL5,ptaui5,ptaus5,PPRES,PSURF,PALB,PSOL,pcrs_o3,&
 &                      pfact,PO3COL5,PO3TCOL5,PPRES5,PSURF5)

!**** *GRG_FPARAMAD* - Approximates the actinic flux for a number of wavelengths
!                      Adjoint

!     Purpose.
!     --------
!     Approximates the actinic flux for a number of wavelengths
!     Result is passed in "pfact(1:7)

!**   Interface.
!     ----------
!     *CALL* *GRG_FPARAMAD(PO3COL,PO3TCOL,PCC_COL5,ptaui5,ptaus5,PPRES,PSURF,PALB,PSOL,pcrs_o3,&
!                          pfact,PO3COL5,PO3TCOL5,PPRES5,PSURF5)

!        Explicit arguments :
!        --------------------
!        PO3COL  : HALF LEVEL O3 partial column perturbation   (INOUT)
!        PO3TCOL : Ozone total column in mol/cm2 perturbation  (INOUT)
!        PCC_COL5 : overhead cloud cover                        (INPUT)
!        ptaui5    : cloud optical thickness per level           (INPUT)
!        ptaus5    : total cloud optical thickness               (INPUT)
!        PPRES   : pressure and full levels  perturbation      (INOUT)
!        PSURF   : surface pressure perturbation               (INOUT)
!        PALB    : surface albedo                              (INPUT)
!        PSOL    : solar zenith angle                          (INPUT)
!        pcrs_o3  : ozone absorption cross section              (INPUT)
!        pfact    : approximated actinic flux                   (INOUT)
!        PO3COL5 : HALF LEVEL O3 partial column trajectory     (INPUT)
!        PO3TCOL5: Ozone total column in mol/cm2 trajectory    (INPUT)
!        PPRES5  : pressure and full levels trajectory         (INPUT)
!        PSURF5  : surface pressure trajectory                 (INPUT)

!        Implicit arguments :    None.
!        --------------------

!     Externals.   None.
!     ----------

!     Author.
!     -------
!        Antje Dethof based on TM5 routine provided by Michiel van Weene

!     Modifications.
!     --------------
!        Original : 21-May-2007

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCST   , ONLY : RPI
USE GRG_PHOTOLYSIS_MIX , ONLY :FINT

  IMPLICIT NONE

REAL(KIND=JPRB),intent(inout)                 :: PO3COL,PO3TCOL
REAL(KIND=JPRB),intent(in)                    :: PCC_COL5,ptaui5,ptaus5
REAL(KIND=JPRB),intent(in)                    :: PALB,PSOL
REAL(KIND=JPRB),intent(inout)                 :: PSURF,PPRES
REAL(KIND=JPRB),intent(in),DIMENSION(1:7)     :: pcrs_o3
REAL(KIND=JPRB),intent(inout),DIMENSION(1:7)  :: pfact
REAL(KIND=JPRB),intent(in)                    :: PO3COL5,PO3TCOL5
REAL(KIND=JPRB),intent(in)                    :: PPRES5,PSURF5

REAL(KIND=JPRB) :: ztu,ztuc,ztl,ztlc      !optical thicknesses lower/upper
REAL(KIND=JPRB) :: ztu5,ztuc5,ztl5,ztl25,ztlc5 !optical thicknesses lower/upper
REAL(KIND=JPRB) :: zetu,zetuc,zetl,zetlc      !exp(tau/mu)
REAL(KIND=JPRB) :: zetu5,zetuc5,zetl5,zetlc5  !exp(tau/mu)
REAL(KIND=JPRB) :: zeto3,zeto35             !for ozon or tau
REAL(KIND=JPRB) :: zala0,zalad
REAL(KIND=JPRB) :: zala05,zalad5
REAL(KIND=JPRB) :: zala0c,zaladc,zal0,zald,zal0c,zaldc
REAL(KIND=JPRB) :: zal0_h,zald_h,zal0c_h,zaldc_h
REAL(KIND=JPRB) :: zala0c5,zaladc5,zal05,zald5,zal0c5,zaldc5
REAL(KIND=JPRB) :: zal0_h5,zald_h5,zal0c_h5,zaldc_h5
REAL(KIND=JPRB) :: zeto3u,zeto3u5
REAL(KIND=JPRB) :: zau,zau0,zau0c,zauc
REAL(KIND=JPRB) :: zau5,zau05,zau0c5,zauc5
REAL(KIND=JPRB) :: zf0,zf0c,zfs,zfsc
REAL(KIND=JPRB) :: zf05,zf0c5,zfs5,zfsc5
REAL(KIND=JPRB) :: zfu,zfuc,zfd,zfdc
REAL(KIND=JPRB) :: zfu5,zfuc5,zfd5,zfdc5
REAL(KIND=JPRB) :: zcref,zcref5
REAL(KIND=JPRB),PARAMETER      :: zone_g = 1._JPRB-0.87_JPRB
REAL(KIND=JPRB),PARAMETER      :: zone_g2 = 1._JPRB-0.87_JPRB*0.87_JPRB
INTEGER(KIND=JPIM)  :: ird=2

INTEGER(KIND=JPIM),DIMENSION(5),PARAMETER  ::iband = (/ 3,4,5,6,7 /)
REAL(KIND=JPRB),DIMENSION(3:7),PARAMETER   ::ztaur = &
 & (/ 1.177e-5, 1.067e-5, 0.919e-5, &
 & 0.498e-5, 0.0782e-5 /)

INTEGER(KIND=JPIM) :: j,ip,ik,ip5,ith5
REAL(KIND=JPRB)    :: zmu0,zpp,zdp
REAL(KIND=JPRB)    :: zpp5,zdp5,zdth5,ztth5
REAL(KIND=JPRB)    :: ZHOOK_HANDLE
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GRG_FPARAMAD',0,ZHOOK_HANDLE)
  zmu0 = cos(PSOL*RPI/180.)

!*       0.   COMPUTE GENERALIZED TRAJECTORY
!-----------------------------------------------------------
  j=2


!  Interpolate actinic flux Table

zpp5 = MAX(101._JPRB-1.e-3_JPRB*PPRES5,0.0_JPRB)
ip5 = int(zpp5)
zdp5 = zpp5-ip5

ztth5 = MIN(PSOL,89.9999_JPRB)*0.2_JPRB
ith5 = int(ztth5)
zdth5 = ztth5-ith5

zcref5 =( (1-zdp5)*((1-zdth5)*fint(ip5  ,ith5  ,j)      &
 &                + zdth5 *fint(ip5  ,ith5+1,j))     &
 &       +  zdp5 *((1-zdth5)*fint(ip5+1,ith5  ,j)      &
 &                + zdth5 *fint(ip5+1,ith5+1,j)) )


zeto35 = exp(-pcrs_o3(j)*PO3COL5/zmu0)


!*       2.   Other Bands

DO ik=1,5   ! over wavelength intervals
  j = iband(ik)

  ztu5  = ztaur(j)*PPRES5       !optical thickness upper atmosphere clear
  ztuc5 = ztu5 + zone_g*ptaui5
  ztl5  = ztaur(j)*(PSURF5 - PPRES5) !lower atmosphere
  ztl25  = MAX(0._JPRB,ptaus5-ptaui5)  !mvw  prevent negatives
  ztlc5 = ztl5 + zone_g*ztl25     !clouds included

  zetu5  = exp(-ztu5/zmu0)
  zetuc5 = zetu5*exp(-ptaui5*zone_g2/zmu0)     !includes cloud effects
  zetl5  = exp(-ztl5/zmu0)
  zetlc5 = zetl5*exp(-ztl25*zone_g2/zmu0)      !includes cloud effects

  zala05 = 1._JPRB-(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetl5)/&
   &       (4._JPRB+3._JPRB*ztl5)

  zalad5 = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ztl5)

  zala0c5= 1._JPRB-(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetlc5)/&
   &       (4._JPRB+3._JPRB*ztlc5)

  zaladc5 = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ztlc5)

  zal05   = zala05   + (1._JPRB-zala05) * PALB * (1._JPRB-zalad5)/&
   &       (1._JPRB-PALB*zalad5)

  zald5  = zalad5  + (1._JPRB-zalad5) * PALB * (1._JPRB-zalad5)  /&
   &      (1._JPRB-PALB*zalad5)

  zal0c5  = zala0c5  + (1._JPRB-zala0c5)* PALB * (1._JPRB-zaladc5) / &
   &       (1._JPRB-PALB*zaladc5)

  zaldc5  = zaladc5  + (1._JPRB-zaladc5)* PALB * (1._JPRB-zaladc5) / &
   &       (1._JPRB-PALB*zaladc5)

  zau5   = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ztu5)

  zau05   = 1._JPRB-(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetu5) /&
   &       (4._JPRB+3._JPRB*ztu5)

  zau0c5  = 1._JPRB-(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetuc5)/&
   &       (4._JPRB+3._JPRB*ztuc5)

  zauc5   = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ztuc5)

  IF (j /= 6) THEN
! actinic flux does depend on ozone absorption
    zeto35  = exp(-pcrs_o3(j)*PO3COL5/zmu0)
    zeto3u5 = exp(-pcrs_o3(j)*(PO3TCOL5-PO3COL5)*2.5)

! Direct radiation

    zf05   = zeto35*zetu5
    zf0c5  = zeto35*zetuc5

! Downward diffuse, only from events above p(i)

    zfs5   = ird *zmu0*(1._JPRB-zau05-zetu5)  *zeto35
    zfsc5  = ird *zmu0*(1._JPRB-zau0c5-zetuc5)*zeto35

!correct lower albedo for ozone absorbtion

    zal0_h5  = zal05 * zeto3u5
    zald_h5  = zald5 * zeto3u5
    zal0c_h5 = zal0c5 * zeto3u5
    zaldc_h5 = zaldc5* zeto3u5
    zal05=zal0_h5
    zald5=zald_h5
    zal0c5=zal0c_h5
    zaldc5=zaldc_h5

  ELSEIF (j==6) THEN
! actinic flux does not depend on ozone absorption
! Direct radiation
    zf05   = zetu5
    zf0c5  = zetuc5
! Downward diffuse, only from events above p(i)
    zfs5   = ird *zmu0*(1._JPRB-zau05-zetu5)
    zfsc5  = ird *zmu0*(1._JPRB-zau0c5-zetuc5)

  ENDIF

! Upward diffuse radiation

  zfu5  = (ird*zmu0*zf05*zal05+zald5*zfs5)/(1._JPRB-zald5*zau5)
  zfuc5 = (ird*zmu0*zf0c5*zal0c5+zaldc5*zfsc5)/(1._JPRB-zaldc5*zauc5)

! Downward direct + diffuse radiation

  zfd5  = zf05     + zfs5     + zau5  *zfu5
  zfdc5 = zf0c5   + zfsc5   + zauc5*zfuc5

!  Interpolate actinic flux Table

  zpp5 = MAX(101._JPRB-1.e-3_JPRB*PPRES5,0.0_JPRB)
  ip5 = int(zpp5)
  zdp5 = zpp5-ip5

  ztth5 = MIN(PSOL,89.9999_JPRB)*0.2_JPRB
  ith5 = int(ztth5)
  zdth5 = ztth5-ith5

  zcref5 =( (1-zdp5)*((1-zdth5)*fint(ip5  ,ith5  ,j)      &
   &                + zdth5 *fint(ip5  ,ith5+1,j))     &
   &       +  zdp5 *((1-zdth5)*fint(ip5+1,ith5  ,j)      &
   &                + zdth5 *fint(ip5+1,ith5+1,j)) )
ENDDO
!---------------------end trajectory calculations

!Initialize variables
zfdc=0.0_JPRB
zfu=0.0_JPRB
zfu=0.0_JPRB
zfd=0.0_JPRB
zcref=0.0_JPRB
zdp=0.0_JPRB
ip=0_JPIM
zpp=0.0_JPRB
zf0c=0.0_JPRB
zfsc=0.0_JPRB
zauc=0.0_JPRB
zfu=0.0_JPRB
zf0=0.0_JPRB
zfs=0.0_JPRB
zau=0.0_JPRB
zfu=0.0_JPRB
zal0c=0.0_JPRB
zaldc=0.0_JPRB
zal0=0.0_JPRB
zald=0.0_JPRB
zau0c=0.0_JPRB
zetuc=0.0_JPRB
zeto3=0.0_JPRB
zau0=0.0_JPRB
zetu=0.0_JPRB
ztuc=0.0_JPRB
ztu=0.0_JPRB
zaladc=0.0_JPRB
zala0c=0.0_JPRB
zalad=0.0_JPRB
zala0=0.0_JPRB
ztlc=0.0_JPRB
ztl=0.0_JPRB
zetl=0.0_JPRB

!End Initialize variables


!*       2.   Other Bands

DO ik=1,5   ! over wavelength intervals
  j = iband(ik)
 

! Calculate actinic flux, weighted with the cloud fraction


  zfdc=zfdc+PCC_COL5*pfact(j)*zcref5
  zfu=zfuc+PCC_COL5*pfact(j)*zcref5
  zfu=zfu+(1.-PCC_COL5)*pfact(j)*zcref5
  zfd=zfd+(1.-PCC_COL5)*pfact(j)*zcref5
  zcref=zcref+(PCC_COL5*(zfdc5+ zfuc5)+(1.-PCC_COL5)*(zfu5+zfd5))*pfact(j)
  pfact(j)=0.0_JPRB
  

!  Interpolate actinic flux Table


  zdp=zdp-zcref*((1-zdth5)*fint(ip5  ,ith5  ,j)    &    !rm the th bits that are 0
   &            + zdth5 *fint(ip5  ,ith5+1,j))
  zdp=zdp+zcref*((1-zdth5)*fint(ip5+1,ith5  ,j)    &
   &            + zdth5 *fint(ip5+1,ith5+1,j))
  zcref=0.0_JPRB

  zpp=zpp+zdp
  ip=ip-zdp
  zdp=0.0_JPRB
  zpp=zpp+INT(ip)
  ip=0.0_JPRB
  IF(101._JPRB-1.e-3_JPRB*PPRES5>0.0_JPRB) THEN
    PPRES=PPRES+MAX(-1.e-3_JPRB*zpp,0.0_JPRB)
    zpp=0.0_JPRB
  ELSE
    zpp = 0.0_JPRB
  ENDIF

! Downward direct + diffuse radiation

  zf0c=zf0c+zfdc
  zfsc=zfsc+zfdc
  zauc=zauc+zfdc*zfuc5
  zfu=zfuc+zauc5 *zfdc
  zfdc=0.0_JPRB

  zf0=zf0+zfd
  zfs=zfs+zfd
  zau=zau+zfd*zfu5
  zfu=zfu+zau5*zfd
  zfd=0.0_JPRB


! Upward diffuse radiation

  zf0c=zf0c+ird*zmu0*zfu*zal0c5*(1._JPRB-zaldc5*zauc5)/(1._JPRB-zaldc5*zauc5)**2
  zal0c=zal0c+ird*zmu0*zf0c5*zfu*(1._JPRB-zaldc5*zauc5)/(1._JPRB-zaldc5*zauc5)**2
  zaldc=zaldc+zfu*zfsc5*(1._JPRB-zaldc5*zauc5)/(1._JPRB-zaldc5*zauc5)**2
  zfsc=zfsc+zaldc5*zfu*(1._JPRB-zaldc5*zauc5)/(1._JPRB-zaldc5*zauc5)**2
  zaldc=zaldc-(ird*zmu0*zf0c5*zal0c5+zaldc5*zfsc5)*zfu*zauc5/(1._JPRB-zaldc5*zauc5)**2
  zauc=zauc-(ird*zmu0*zf0c5*zal0c5+zaldc5*zfsc5)*zaldc5*zfu/(1._JPRB-zaldc5*zauc5)**2
  zfu=0.0_JPRB

  zf0=zf0+ird*zmu0*zfu*zal05*(1._JPRB-zald5 *zau5)/(1._JPRB-zald5*zau5)**2
  zal0=zal0+ird*zmu0*zf05*zfu*(1._JPRB-zald5 *zau5)/(1._JPRB-zald5*zau5)**2
  zald=zald+zfu*zfs5*(1._JPRB-zald5 *zau5)/(1._JPRB-zald5*zau5)**2
  zfs=zfs+zald5*zfu*(1._JPRB-zald5 *zau5)/(1._JPRB-zald5*zau5)**2
  zald=zald-(ird*zmu0*zf05*zal05+zald5*zfs5)*zfu*zau5/(1._JPRB-zald5*zau5)**2
  zau=zau-(ird*zmu0*zf05*zal05+zald5*zfs5)*zald5*zfu/(1._JPRB-zald5*zau5)**2
  zfu=0.0_JPRB

  IF (j /= 6) THEN
!correct lower albedo for ozone absorbtion

!    zaldc_h=0.0_JPRB
!    zal0c_h=0.0_JPRB
!    zald_h=0.0_JPRB
!    zal0_h=0.0_JPRB
!    zaldc_h=aldc_h+aldc
!    zal0c_h=al0c_h+al0c
!    zald_h=ald_h+ald
!    zal0_h=al0_h+al0

    zaldc_h=zaldc
    zal0c_h=zal0c
    zald_h=zald
    zal0_h=zal0
    zaldc=0.0_JPRB
    zal0c=0.0_JPRB
    zald=0.0_JPRB
    zal0=0.0_JPRB

    zaldc=zaldc+zaldc_h*zeto3u5
    zeto3u=zeto3u+zaldc5*zaldc_h
    zaldc_h=0.0_JPRB

    zal0c=zal0c+zal0c_h*zeto3u5
    zeto3u=zeto3u+zal0c5 * zal0c_h
    zal0c_h=0.0_JPRB

    zald=zald+zald_h* zeto3u5
    zeto3u=zeto3u+zald5*zald_h
    zald_h=0.0_JPRB

    zal0=zal0+zal0_h*zeto3u5
    zeto3u=zeto3u+zal05 *zal0_h
    zal0_h=0.0_JPRB


! Downward diffuse, only from events above p(i)

    zau0c=zau0c-ird *zmu0*zfsc*zeto35
    zetuc=zetuc-ird *zmu0*zfsc*zeto35
    zeto3=zeto3+ird *zmu0*(1._JPRB-zau0c5-zetuc5)*zfsc
    zfsc=0.0_JPRB

    zau0=zau0-ird *zmu0*zfs*zeto35
    zetu=zetu-ird *zmu0*zfs*zeto35
    zeto3=zeto3+ird *zmu0*(1._JPRB-zau05-zetu5)*zfs
    zfs=0.0_JPRB

! Direct radiation

    zeto3=zeto3+zf0c*zetuc5
    zetuc=zetuc+zeto35*zf0c
    zf0c=0.0_JPRB

    zeto3=zeto3+zf0*zetu5
    zetu=zetu+zeto35*zf0
    zf0=0.0_JPRB

! actinic flux does depend on ozone absorption
    PO3TCOL=PO3TCOL-pcrs_o3(j)*zeto3u*2.5*&
     &      exp(-pcrs_o3(j)*(PO3TCOL5-PO3COL5)*2.5)
    PO3COL=PO3COL+pcrs_o3(j)*zeto3u*2.5*&
     &      exp(-pcrs_o3(j)*(PO3TCOL5-PO3COL5)*2.5)
    zeto3u=0.0_JPRB
  
    PO3COL=PO3COL-pcrs_o3(j)*zeto3/zmu0*exp(-pcrs_o3(j)*PO3COL5/zmu0)
    zeto3=0.0_JPRB

  ELSEIF (j==6) THEN
! actinic flux does not depend on ozone absorption
! Downward diffuse, only from events above p(i)
    zau0c=zau0c-ird *zmu0*zfsc
    zetuc=zetuc-ird *zmu0*zfsc
    zfsc=0.0_JPRB
 
    zau0=zau0-ird *zmu0*zfs
    zetu=zetu-ird *zmu0*zfs
    zfs=0.0_JPRB

! Direct radiation
    zetuc=zetuc+zf0c
    zf0c=0.0_JPRB
    zetu=zetu+zf0
    zf0=0.0_JPRB

  ENDIF

  ztuc=ztuc+3._JPRB*zauc*(-4._JPRB)/(4._JPRB+3._JPRB*ztuc5)
  zauc=0.0_JPRB

  zetuc=zetuc-(2._JPRB-3._JPRB*zmu0)*zau0c*(4._JPRB+3._JPRB*ztuc5)/ &
   &        (4._JPRB+3._JPRB*ztuc5)**2
  ztuc=ztuc+(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetuc5)*3._JPRB*zau0c/ &
   &        (4._JPRB+3._JPRB*ztuc5)**2
  zau0c  =0.0_JPRB

  zetu=zetu-(2._JPRB-3._JPRB*zmu0)*zau0*(4._JPRB+3._JPRB*ztu5) /&
   &       (4._JPRB+3._JPRB*ztu5)**2
  ztu=ztu+(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetu5)*3._JPRB*zau0/&
   &       (4._JPRB+3._JPRB*ztu5)**2
  zau0  =0.0_JPRB

  ztu=ztu+3._JPRB*zau*(-4._JPRB)/(4._JPRB+3._JPRB*ztu5)
  zau=0.0_JPRB

  zaladc=zaladc+zaldc 
  zaladc=zaladc-PALB*zaldc*(1._JPRB-zaladc5)*(1._JPRB-PALB*zaladc5)/ &
   &       (1._JPRB-PALB*zaladc5)**2
  zaladc=zaladc-PALB*(1._JPRB-zaladc5)*zaldc*&
   &       (1._JPRB-PALB*zaladc5)/ &
   &       (1._JPRB-PALB*zaladc5)**2
  zaladc=zaladc+(1._JPRB-zaladc5) * PALB * (1._JPRB-zaladc5) *PALB*zaldc/ &
   &       (1._JPRB-PALB*zaladc5)**2
  zaldc=0.0_JPRB

  zala0c=zala0c+zal0c
  zala0c=zala0c-PALB*zal0c*(1._JPRB-zaladc5)* &
   &       (1._JPRB-PALB*zaladc5)/ &
   &       (1._JPRB-PALB*zaladc5)**2
  zaladc=zaladc-PALB*(1._JPRB-zala0c5)*zal0c* &
   &       (1._JPRB-PALB*zaladc5)/ &
   &       (1._JPRB-PALB*zaladc5)**2
  zaladc=zaladc+(1._JPRB-zala0c5) * PALB * (1._JPRB-zaladc5) *PALB*zal0c/ &
   &       (1._JPRB-PALB*zaladc5)**2
  zal0c=0.0_JPRB
 
  zalad=zalad+zald
  zalad=zalad-PALB*zald*(1._JPRB-zalad5)*&
   &       (1._JPRB-PALB*zalad5)/ &
   &       (1._JPRB-PALB*zalad5)**2
  zalad=zalad-PALB*(1._JPRB-zalad5)*zald*&
   &       (1._JPRB-PALB*zalad5) / &
   &       (1._JPRB-PALB*zalad5)**2
  zalad=zalad+(1._JPRB-zalad5) * PALB * (1._JPRB-zalad5) *PALB*zald/ &
   &       (1._JPRB-PALB*zalad5)**2
  zald=0.0_JPRB

  zala0=zala0+zal0
  zala0=zala0-PALB*zal0*(1._JPRB-zalad5)* &
   &       (1._JPRB-PALB*zalad5)/ &
   &       (1._JPRB-PALB*zalad5)**2
  zalad=zalad-PALB*(1._JPRB-zala05)*zal0* &
   &       (1._JPRB-PALB*zalad5)/ &
   &       (1._JPRB-PALB*zalad5)**2
  zalad=zalad+(1._JPRB-zala05)*PALB*(1._JPRB-zalad5)*PALB*zal0/ &
   &       (1._JPRB-PALB*zalad5)**2
  zal0=0.0_JPRB

  ztlc=ztlc+3._JPRB*zaladc*(-4._JPRB)/(4._JPRB+3._JPRB*ztlc5)**2
  zaladc=0.0_JPRB

  zetlc=0.0_JPRB
  zetlc=zetlc-(2._JPRB-3._JPRB*zmu0)*zala0c*(4._JPRB+3._JPRB*ztlc5)/&
   &       (4._JPRB+3._JPRB*ztlc5)**2
  ztlc=ztlc+(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetlc5)*3._JPRB*zala0c/&
   &       (4._JPRB+3._JPRB*ztlc5)**2
  zala0c=0.0_JPRB

  ztl=ztl+3._JPRB*zalad*(-4._JPRB)/(4._JPRB+3._JPRB*ztl5)**2
  zalad=0.0_JPRB

  zetl=zetl-(2._JPRB-3._JPRB*zmu0)*zala0*(4._JPRB+3._JPRB*ztl5)/  &
   &       (4._JPRB+3._JPRB*ztl5)**2
  ztl=ztl+(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetl5)*3._JPRB*zala0/  &
   &       (4._JPRB+3._JPRB*ztl5)**2
  zala0=0.0_JPRB

  zetl=zetl+zetlc*exp(-ztl25*zone_g2/zmu0) 
  zetlc=0.0_JPRB
  
  ztl=ztl-zetl/zmu0*exp(-ztl5/zmu0)
  zetl=0.0_JPRB

  zetu=zetu+zetuc*exp(-ptaui5*zone_g2/zmu0) 
  zetuc=0.0_JPRB

  ztu=ztu-zetu/zmu0*exp(-ztu5/zmu0)
  zetu=0.0_JPRB

  ztl=ztl+ztlc
  ztlc=0.0_JPRB

  PSURF=PSURF+ztaur(j)*ztl
  PPRES=PPRES-ztaur(j)*ztl
  ztl=0.0_JPRB 

  ztu=ztu+ztuc
  ztuc=0.0_JPRB

  PPRES=PPRES+ztaur(j)*ztu
  ztu=0.0_JPRB

ENDDO

!*       1.   Band 2:
!             actinic flux only depends on ozone absorption

j=2

zcref=zcref+pfact(j)*zeto35
zeto3=zeto3+pfact(j)*zcref5
pfact(j)=0.0_JPRB

PO3COL=PO3COL-pcrs_o3(j)*zeto3/zmu0*exp(-pcrs_o3(j)*PO3COL5/zmu0)
zeto3=0.0_JPRB


zdp=zdp-zcref*((1-zdth5)*fint(ip5  ,ith5  ,j)    &  
 &            + zdth5 *fint(ip5  ,ith5+1,j))
zdp=zdp+zcref*((1-zdth5)*fint(ip5+1,ith5  ,j)    &
 &            + zdth5 *fint(ip5+1,ith5+1,j))
zcref=0.0_JPRB

zpp=zpp+zdp
ip=ip-zdp
zdp=0.0_JPRB
zpp=zpp+INT(ip)
ip=0.0_JPRB
IF(101._JPRB-1.e-3_JPRB*PPRES5>0.0_JPRB) THEN
  PPRES=PPRES+MAX(-1.e-3_JPRB*zpp,0.0_JPRB)
  zpp=0.0_JPRB
ELSE
  zpp = 0.0_JPRB
ENDIF


IF (LHOOK) CALL DR_HOOK('GRG_FPARAMAD',1,ZHOOK_HANDLE)
END SUBROUTINE GRG_FPARAMAD
