SUBROUTINE GRG_FPARAMTL(PO3COL,PO3TCOL,PCC_COL5,ptaui5,ptaus5,PPRES,PSURF,PALB,PSOL,pcrs_o3,&
 &                      pfact,PO3COL5,PO3TCOL5,PPRES5,PSURF5)

!**** *GRG_FPARAMTL* - Approximates the actinic flux for a number of wavelengths
!                      Tangent linear.

!     Purpose.
!     --------
!     Approximates the actinic flux for a number of wavelengths
!     Result is passed in "pfact(1:7)

!     Assume perturbations=0 of PCC_COL,taui,taus so that there is no feedback
!     on these fields in the adjoint

!**   Interface.
!     ----------
!     *CALL* *GRG_FPARAMTL(PO3COL,PO3TCOL,PCC_COL5,ptaui5,ptaus5,PPRES,PSURF,PALB,PSOL,pcrs_o3,&
!                          pfact,PO3COL5,PO3TCOL5,PPRES5,PSURF5)

!        Explicit arguments :
!        --------------------
!        PO3COL  : HALF LEVEL O3 partial column perturbation   (INPUT)
!        PO3TCOL : Ozone total column in mol/cm2 perturbation  (INPUT)
!        PCC_COL5: overhead cloud cover                        (INPUT)
!        ptaui5   : cloud optical thickness per level           (INPUT)
!        ptaus5   : total cloud optical thickness               (INPUT)
!        PPRES   : pressure and full levels  perturbation      (INPUT)
!        PSURF   : surface pressure perturbation               (INPUT)
!        PALB    : surface albedo                              (INPUT)
!        PSOL    : solar zenith angle                          (INPUT)
!        pcrs_o3  : ozone absorption cross section              (INPUT)
!        pfact    : approximated actinic flux                   (OUTPUT)
!        PO3COL5 : HALF LEVEL O3 partial column trajectory     (INPUT)
!        PO3TCOL5: Ozone total column in mol/cm2 trajectory    (INPUT)
!        PPRES5  : pressure and full levels trajectory         (INPUT)
!        PSURF5  : surface pressure trajectory                 (INPUT)

!        Implicit arguments :    None.
!        --------------------

!     Externals.   None.
!     ----------

!     Author.
!     -------
!        Antje Dethof based on TM5 routine provided by Michiel van Weene

!     Modifications.
!     --------------
!        Original : 21-May-2007
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCST   , ONLY : RPI
USE GRG_PHOTOLYSIS_MIX , ONLY :FINT

  IMPLICIT NONE

REAL(KIND=JPRB),intent(in)                            :: PO3COL,PO3TCOL
REAL(KIND=JPRB),intent(in)                            :: PCC_COL5,ptaui5,ptaus5
REAL(KIND=JPRB),intent(in)                            :: PPRES,PSURF,PALB,PSOL
REAL(KIND=JPRB),intent(in),DIMENSION(1:7)             :: pcrs_o3
REAL(KIND=JPRB),intent(out),DIMENSION(1:7)            :: pfact
REAL(KIND=JPRB),intent(in)                            :: PO3COL5,PO3TCOL5
REAL(KIND=JPRB),intent(in)                            :: PPRES5,PSURF5

REAL(KIND=JPRB) :: ztu,ztuc,ztl,ztlc      !optical thicknesses lower/upper
REAL(KIND=JPRB) :: ztu5,ztuc5,ztl5,ztl25,ztlc5 !optical thicknesses lower/upper
REAL(KIND=JPRB) :: zetu,zetuc,zetl,zetlc      !exp(tau/mu)
REAL(KIND=JPRB) :: zetu5,zetuc5,zetl5,zetlc5  !exp(tau/mu)
REAL(KIND=JPRB) :: zeto3,zeto35             !for ozon or tau
REAL(KIND=JPRB) :: zala0,zalad
REAL(KIND=JPRB) :: zala05,zalad5
REAL(KIND=JPRB) :: zala0c,zaladc,zal0,zald,zal0c,zaldc
REAL(KIND=JPRB) :: zal0_h,zald_h,zal0c_h,zaldc_h
REAL(KIND=JPRB) :: zala0c5,zaladc5,zal05,zald5,zal0c5,zaldc5
REAL(KIND=JPRB) :: zal0_h5,zald_h5,zal0c_h5,zaldc_h5
REAL(KIND=JPRB) :: zeto3u,zeto3u5
REAL(KIND=JPRB) :: zau,zau0,zau0c,zauc
REAL(KIND=JPRB) :: zau5,zau05,zau0c5,zauc5
REAL(KIND=JPRB) :: zf0,zf0c,zfs,zfsc
REAL(KIND=JPRB) :: zf05,zf0c5,zfs5,zfsc5
REAL(KIND=JPRB) :: zfu,zfuc,zfd,zfdc
REAL(KIND=JPRB) :: zfu5,zfuc5,zfd5,zfdc5
REAL(KIND=JPRB) :: zcref,zcref5
REAL(KIND=JPRB),PARAMETER      :: zone_g = 1._JPRB-0.87_JPRB
REAL(KIND=JPRB),PARAMETER      :: zone_g2 = 1._JPRB-0.87_JPRB*0.87_JPRB
INTEGER(KIND=JPIM)  :: ird=2

INTEGER(KIND=JPIM),DIMENSION(5),PARAMETER  ::iband = (/ 3,4,5,6,7 /)
REAL(KIND=JPRB),DIMENSION(3:7),PARAMETER   ::ztaur = &
 & (/ 1.177e-5, 1.067e-5, 0.919e-5, &
 & 0.498e-5, 0.0782e-5 /)

INTEGER(KIND=JPIM) :: j,ip,ik,ip5,ith5
REAL(KIND=JPRB)    :: zmu0,zpp,zdp
REAL(KIND=JPRB)    :: zpp5,zdp5,zdth5,ztth5
REAL(KIND=JPRB)    :: ZHOOK_HANDLE
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GRG_FPARAMTL',0,ZHOOK_HANDLE)
  zmu0 = cos(PSOL*RPI/180.)

!*       1.   Band 2:
!             actinic flux only depends on ozone absorption

j=2


!  Interpolate actinic flux Table

zpp5 = MAX(101._JPRB-1.e-3_JPRB*PPRES5,0.0_JPRB)
IF(101._JPRB-1.e-3_JPRB*PPRES5>0.0_JPRB) THEN
  zpp = -1.e-3_JPRB*PPRES
ELSE
  zpp = 0.0_JPRB
ENDIF
ip5 = int(zpp5)
ip = int(zpp)
zdp5 = zpp5-ip5
zdp = zpp-ip

ztth5 = MIN(PSOL,89.9999_JPRB)*0.2_JPRB
ith5 = int(ztth5)
zdth5 = ztth5-ith5

zcref5 =( (1-zdp5)*((1-zdth5)*fint(ip5  ,ith5  ,j)      &
 &                + zdth5 *fint(ip5  ,ith5+1,j))     &
 &       +  zdp5 *((1-zdth5)*fint(ip5+1,ith5  ,j)      &
 &                + zdth5 *fint(ip5+1,ith5+1,j)) )

zcref =( -zdp*((1-zdth5)*fint(ip5  ,ith5  ,j)    &    !rm the th bits that are 0
 &            + zdth5 *fint(ip5  ,ith5+1,j))   &
 &      +zdp*((1-zdth5)*fint(ip5+1,ith5  ,j)    &
 &            + zdth5 *fint(ip5+1,ith5+1,j)) )

zeto35 = exp(-pcrs_o3(j)*PO3COL5/zmu0)
zeto3 = -pcrs_o3(j)*PO3COL/zmu0*exp(-pcrs_o3(j)*PO3COL5/zmu0)

pfact(j) =  zcref*zeto35+zcref5*zeto3

!*       2.   Other Bands

DO ik=1,5   ! over wavelength intervals
  j = iband(ik)

  ztu5  = ztaur(j)*PPRES5       !optical thickness upper atmosphere clear
  ztu   = ztaur(j)*PPRES        !optical thickness upper atmosphere clear
  ztuc5 = ztu5 + zone_g*ptaui5
  ztuc  = ztu 
  ztl5  = ztaur(j)*(PSURF5 - PPRES5) !lower atmosphere
  ztl   = ztaur(j)*(PSURF - PPRES) !lower atmosphere
  ztl25 = MAX(0._JPRB,ptaus5-ptaui5)  !mvw  prevent negatives
  ztlc5= ztl5 + zone_g*ztl25    !clouds included ztlc  = ztl      !clouds included
  ztlc= ztl     !clouds included ztlc  = ztl      !clouds included

  zetu5  = exp(-ztu5/zmu0)
  zetu   = -ztu/zmu0*exp(-ztu5/zmu0)
  zetuc5 = zetu5*exp(-ptaui5*zone_g2/zmu0)     !includes cloud effects
  zetuc  = zetu*exp(-ptaui5*zone_g2/zmu0)     !includes cloud effects
  zetl5  = exp(-ztl5/zmu0)
  zetl   = -ztl/zmu0*exp(-ztl5/zmu0)
  zetlc5 = zetl5*exp(-ztl25*zone_g2/zmu0)      !includes cloud effects
  zetlc  = zetl*exp(-ztl25*zone_g2/zmu0)      !includes cloud effects

  zala05 = 1._JPRB-(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetl5)/&
   &       (4._JPRB+3._JPRB*ztl5)
  zala0  = -((2._JPRB-3._JPRB*zmu0)*zetl*(4._JPRB+3._JPRB*ztl5) - &
   &       (2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetl5)*3._JPRB*ztl)/  &
   &       (4._JPRB+3._JPRB*ztl5)**2

  zalad5 = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ztl5)
  zalad  = 3._JPRB*ztl*(-4._JPRB)/(4._JPRB+3._JPRB*ztl5)**2

  zala0c5= 1._JPRB-(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetlc5)/&
   &       (4._JPRB+3._JPRB*ztlc5)
  zala0c = -((2._JPRB-3._JPRB*zmu0)*zetlc*(4._JPRB+3._JPRB*ztlc5) - &
   &       (2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetlc5)*3._JPRB*ztlc) /&
   &       (4._JPRB+3._JPRB*ztlc5)**2

  zaladc5 = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ztlc5)
  zaladc = 3._JPRB*ztlc*(-4._JPRB)/(4._JPRB+3._JPRB*ztlc5)**2

  zal05   = zala05   + (1._JPRB-zala05) * PALB * (1._JPRB-zalad5)/&
   &       (1._JPRB-PALB*zalad5)
  zal0   = zala0 + (PALB*(-zala0*(1._JPRB-zalad5)-(1._JPRB-zala05)*zalad) * &
   &       (1._JPRB-PALB*zalad5) - &
   &       (1._JPRB-zala05)*PALB*(1._JPRB-zalad5)*(-PALB*zalad)) / &
   &       (1._JPRB-PALB*zalad5)**2

  zald5  = zalad5  + (1._JPRB-zalad5) * PALB * (1._JPRB-zalad5)  /&
   &      (1._JPRB-PALB*zalad5)
  zald   = zalad + (PALB*(-zalad*(1._JPRB-zalad5)-(1._JPRB-zalad5)*zalad) *&
   &       (1._JPRB-PALB*zalad5)  - &
   &       (1._JPRB-zalad5) * PALB * (1._JPRB-zalad5) * (-PALB*zalad)) / &
   &       (1._JPRB-PALB*zalad5)**2

  zal0c5  = zala0c5  + (1._JPRB-zala0c5)* PALB * (1._JPRB-zaladc5) / &
   &       (1._JPRB-PALB*zaladc5)
  zal0c   = zala0c + (PALB*(-zala0c*(1._JPRB-zaladc5)-(1._JPRB-zala0c5)*zaladc) * &
   &       (1._JPRB-PALB*zaladc5) - &
   &       (1._JPRB-zala0c5) * PALB * (1._JPRB-zaladc5) * (-PALB*zaladc)) / &
   &       (1._JPRB-PALB*zaladc5)**2

  zaldc5  = zaladc5  + (1._JPRB-zaladc5)* PALB * (1._JPRB-zaladc5) / &
   &       (1._JPRB-PALB*zaladc5)
  zaldc   = zaladc + (PALB*(-zaladc*(1._JPRB-zaladc5)-(1._JPRB-zaladc5)*zaladc) *&
   &       (1._JPRB-PALB*zaladc5) - &
   &       (1._JPRB-zaladc5) * PALB * (1._JPRB-zaladc5) * (-PALB*zaladc)) / &
   &       (1._JPRB-PALB*zaladc5)**2

  zau5   = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ztu5)
  zau    = 3._JPRB*ztu*(-4._JPRB)/(4._JPRB+3._JPRB*ztu5)

  zau05   = 1._JPRB-(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetu5) /&
   &       (4._JPRB+3._JPRB*ztu5)
  zau0  = -((2._JPRB-3._JPRB*zmu0)*zetu*(4._JPRB+3._JPRB*ztu5)  -&
   &       (2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetu5)*3._JPRB*ztu)  /&
   &       (4._JPRB+3._JPRB*ztu5)**2

  zau0c5  = 1._JPRB-(2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetuc5)/&
   &       (4._JPRB+3._JPRB*ztuc5)
  zau0c  = -((2._JPRB-3._JPRB*zmu0)*zetuc*(4._JPRB+3._JPRB*ztuc5)-  &
   &        (2._JPRB+3._JPRB*zmu0+(2._JPRB-3._JPRB*zmu0)*zetuc5)*3._JPRB*ztuc) / &
   &        (4._JPRB+3._JPRB*ztuc5)**2

  zauc5   = 1._JPRB-4._JPRB/(4._JPRB+3._JPRB*ztuc5)
  zauc   = 3._JPRB*ztuc*(-4._JPRB)/(4._JPRB+3._JPRB*ztuc5)

  IF (j /= 6) THEN
! actinic flux does depend on ozone absorption
    zeto35  = exp(-pcrs_o3(j)*PO3COL5/zmu0)
    zeto3 = -pcrs_o3(j)*PO3COL/zmu0*exp(-pcrs_o3(j)*PO3COL5/zmu0)
    zeto3u5 = exp(-pcrs_o3(j)*(PO3TCOL5-PO3COL5)*2.5)
    zeto3u = -pcrs_o3(j)*(PO3TCOL-PO3COL)*2.5*&
     &      exp(-pcrs_o3(j)*(PO3TCOL5-PO3COL5)*2.5)

! Direct radiation

    zf05   = zeto35*zetu5
    zf0    = zeto3*zetu5+zeto35*zetu
    zf0c5  = zeto35*zetuc5
    zf0c   = zeto3*zetuc5+zeto35*zetuc

! Downward diffuse, only from events above p(i)

    zfs5   = ird *zmu0*(1._JPRB-zau05-zetu5)  *zeto35
    zfs    = ird *zmu0*((-zau0-zetu)*zeto35+(1._JPRB-zau05-zetu5))*zeto3
    zfsc5  = ird *zmu0*(1._JPRB-zau0c5-zetuc5)*zeto35
    zfsc   = ird *zmu0*((-zau0c-zetuc)*zeto35+(1._JPRB-zau0c5-zetuc5))*zeto3

!correct lower albedo for ozone absorbtion

!    zal05  = zal05 * zeto3u5     
!    zal0   = zal0  * zeto3u5 + zal05 * zeto3u      
!    zald5  = zald5 * zeto3u5
!    zald   = zald  * zeto3u5 +zald5  * zeto3u
!    zal0c5 = zal0c5 * zeto3u5
!    zal0c  = zal0c * zeto3u5 +zal0c5 * zeto3u
!    zaldc5 = zaldc5* zeto3u5
!    zaldc  = zaldc * zeto3u5 + zaldc5* zeto3u

    zal0_h5  = zal05 * zeto3u5     
    zal0_h   = zal0  * zeto3u5 + zal05 * zeto3u      
    zald_h5  = zald5 * zeto3u5
    zald_h   = zald  * zeto3u5 +zald5  * zeto3u
    zal0c_h5 = zal0c5 * zeto3u5
    zal0c_h  = zal0c * zeto3u5 +zal0c5 * zeto3u
    zaldc_h5 = zaldc5* zeto3u5
    zaldc_h  = zaldc * zeto3u5 + zaldc5* zeto3u

    zal05  =  zal0_h5
    zal0   =  zal0_h
    zald5  =  zald_h5
    zald   =  zald_h
    zal0c5 =  zal0c_h5
    zal0c  =  zal0c_h
    zaldc5 = zaldc_h5
    zaldc  = zaldc_h
  ELSEIF (j==6) THEN
! actinic flux does not depend on ozone absorption
! Direct radiation
    zf05   = zetu5
    zf0    = zetu
    zf0c5  = zetuc5
    zf0c   = zetuc
! Downward diffuse, only from events above p(i)
    zfs5   = ird *zmu0*(1._JPRB-zau05-zetu5)
    zfs    = ird *zmu0*(-zau0-zetu)
    zfsc5  = ird *zmu0*(1._JPRB-zau0c5-zetuc5)
    zfsc   = ird *zmu0*(-zau0c-zetuc)

  ENDIF

! Upward diffuse radiation 

  zfu5  = (ird*zmu0*zf05*zal05+zald5*zfs5)/(1._JPRB-zald5*zau5)
  zfu   = ((ird*zmu0*(zf0*zal05+zf05*zal0)+(zald*zfs5+zald5*zfs))*&
   &     (1._JPRB-zald5 *zau5) - &
   &     (ird*zmu0*zf05*zal05+zald5*zfs5)*(-zald*zau5-zald5*zau))/ &
   &     (1._JPRB-zald5*zau5)**2

  zfuc5 = (ird*zmu0*zf0c5*zal0c5+zaldc5*zfsc5)/(1._JPRB-zaldc5*zauc5)
  zfuc  = ((ird*zmu0*(zf0c*zal0c5+zf0c5*zal0c)+(zaldc*zfsc5+zaldc5*zfsc))*&
   &     (1._JPRB-zaldc5*zauc5)- &
   &     (ird*zmu0*zf0c5*zal0c5+zaldc5*zfsc5)*(-zaldc*zauc5-zaldc5*zauc))/ &
   &     (1._JPRB-zaldc5*zauc5)**2

! Downward direct + diffuse radiation

  zfd5  = zf05    + zfs5    + zau5 *zfu5
  zfd   = zf0     + zfs     + zau  *zfu5 + zau5 *zfu
  zfdc5 = zf0c5   + zfsc5   + zauc5*zfuc5
  zfdc  = zf0c    + zfsc    + zauc *zfuc5 +zauc5 *zfuc

!  Interpolate actinic flux Table

  zpp5 = MAX(101._JPRB-1.e-3_JPRB*PPRES5,0.0_JPRB)
  IF(101._JPRB-1.e-3_JPRB*PPRES5>0.0_JPRB) THEN
    zpp = -1.e-3_JPRB*PPRES
  ELSE
    zpp = 0.0_JPRB
  ENDIF
  ip5 = int(zpp5)
  ip = int(zpp)
  zdp5 = zpp5-ip5
  zdp = zpp-ip

  ztth5 = MIN(PSOL,89.9999_JPRB)*0.2_JPRB
  ith5 = int(ztth5)
  zdth5 = ztth5-ith5

  zcref5 =( (1-zdp5)*((1-zdth5)*fint(ip5  ,ith5  ,j)      &
   &                + zdth5 *fint(ip5  ,ith5+1,j))     &
   &       +  zdp5 *((1-zdth5)*fint(ip5+1,ith5  ,j)      &
   &                + zdth5 *fint(ip5+1,ith5+1,j)) )

  zcref =( -zdp*((1-zdth5)*fint(ip5  ,ith5  ,j)    &    !rm the th bits that are 0
   &            + zdth5 *fint(ip5  ,ith5+1,j))   &
   &      +zdp*((1-zdth5)*fint(ip5+1,ith5  ,j)    &
   &            + zdth5 *fint(ip5+1,ith5+1,j)) )

! Calculate actinic flux, weighted with the cloud fraction

  pfact(j) = (PCC_COL5*(zfdc+zfuc)+(1.-PCC_COL5)*(zfu+zfd))*zcref5 + &
   &        (PCC_COL5*(zfdc5+ zfuc5)+(1.-PCC_COL5)*(zfu5+zfd5))*zcref
ENDDO

IF (LHOOK) CALL DR_HOOK('GRG_FPARAMTL',1,ZHOOK_HANDLE)
END SUBROUTINE GRG_FPARAMTL
