SUBROUTINE GRG_JNO2(KPROMA,KSTART,KPROF,KFLEV,PPCO3,PT,PSOL,PAL,PJNO2,PRXP)

!**** *GRG_JNO2* - Computes NO2 photolysis frequency JNO2

!     Purpose.
!     --------
!           Computes NO2 photolysis frequency JNO2

!**   Interface.
!     ----------
!        *CALL* *GRG_JNO2(KPROMA,KSTART,KPROF,KFLEV,PPCO3,PT,PSOL,PAL,PJNO2,PRXP)

!        Explicit arguments :
!        --------------------
!        KPROMA               - HORIZ. DIMENSIONING      (INPUT)
!        KSTART               - START OF WORK            (INPUT)
!        KPROF                - DEPTH OF WORK            (INPUT)
!        KFLEV                - NUMBER OF LEVELS         (INPUT)
!        PPCO3 (KPROMA,KFLEV) - HALF LEVEL O3 partial column (INPUT)
!        PT  (KPROMA,KFLEV)   - TEMPERATURE              (INPUT)
!        PSOL (KPROMA)        - solar elevation          (INPUT)
!        PAL  (KPROMA)        - albedo                   (INPUT)
!        PJNO2 (KPROMA,KFLEV) - JNO2 photolysis freq.    (OUTPUT)
!        PRXP(KPROMA,0:KFLEV,NPPM)  - HALF,FULL AND LN HALF LEVEL
!                                     PRESSURES (SEE PPINIT)   (INPUT)

!        Implicit arguments :    None.
!        --------------------

!     Method.
!     -------
!        See documentation.

!     Externals.   None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Antje Dethof based on TM5 routine *ECMWF*

!     Modifications.
!     --------------
!        Original : 09-Oct-2006

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NPPM
USE YOMCST   , ONLY : RPI
USE GRG_PHOTOLYSIS_MIX
USE YOMANCS  , ONLY : RMDI

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)  :: KPROMA
INTEGER(KIND=JPIM),INTENT(IN)  :: KSTART
INTEGER(KIND=JPIM),INTENT(IN)  :: KPROF
INTEGER(KIND=JPIM),INTENT(IN)  :: KFLEV
REAL(KIND=JPRB)   ,INTENT(IN)  :: PPCO3(KPROMA,KFLEV) !overhead ozone (kg/m2)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PT(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PSOL(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PAL(KPROMA)
REAL(KIND=JPRB)   ,INTENT(OUT) :: PJNO2(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PRXP(KPROMA,0:KFLEV,NPPM)

!mvw added declarations
REAL(KIND=JPRB) :: zTAUC(KPROMA,KFLEV) ! cloud optical thickness per level
REAL(KIND=JPRB) :: zTAUS(KPROMA)       ! total cloud optical thickness
REAL(KIND=JPRB) :: ZALB(KPROMA)       ! surface albedo
REAL(KIND=JPRB) :: zPS(KPROMA)         ! surface pressure (for now I just take: PMID(KFLEV)

!LOCAL PARAMETERS
INTEGER(KIND=JPIM),PARAMETER :: inbands=7
REAL(KIND=JPRB), PARAMETER   :: todu = 3.767e-17    ! O3 from molecules/cm2 --> du
!antje REAL(KIND=JPRB), PARAMETER   :: todu = 47296 ! O3 from kg/m2 --> du
REAL(KIND=JPRB), PARAMETER   :: tomol = 1.255e+21   ! O3 from kg/m2 --> mol/cm2
REAL(KIND=JPRB), PARAMETER   :: zf0(inbands) = &   ! f0: bruhl wavelength works with fintwb.data
 &   (/73.44,4.977,16.13,8.017,27.9,19.00,48.12/)
REAL(KIND=JPRB), PARAMETER :: zcrs_o3(inbands) = &
 &   (/3.6448e-19,1.7115e-18,2.8218e-19,1.1091e-19, &
 &     2.8145e-20,1.9000e-23,4.5500e-21/)
REAL(KIND=JPRB), PARAMETER :: zfe(inbands) = &
 &   (/2.20560e+12,2.22730e+14,7.52530e+13,1.01010e+14, &
 &     1.19010e+14,1.07760e+15,2.67400e+15/)

!LOCAL VARIABLES
INTEGER(KIND=JPIM) :: JL, JLEV
INTEGER(KIND=JPIM) :: i2,i3,ind         !indices lookup tables
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZPCO3(KPROMA,KFLEV)  !overhead ozone slant column (molecules/cm2)
REAL(KIND=JPRB) :: ZO3COL(KPROMA,KFLEV) !overhead ozone (molecules/cm2)
REAL(KIND=JPRB) :: ZO3TCOL(KPROMA)      !overhead total ozone (molecules/cm2)
REAL(KIND=JPRB) :: ZCC_COL(KPROMA)      !overhead cloud cover
REAL(KIND=JPRB) :: ztscale
REAL(KIND=JPRB) :: zv3_du(KPROMA,KFLEV), zv3_du1(KPROMA,KFLEV), zv3_du2(KPROMA,KFLEV)
REAL(KIND=JPRB) :: zv3s1(KPROMA,KFLEV),zv3s2(KPROMA,KFLEV)      ! ozone column variables
REAL(KIND=JPRB) :: ztau_2,ztau_3,ztau_4,ztau_5                    ! optical depth
REAL(KIND=JPRB) :: zfint_2,zfint_3,zfint_4, zfint_5,zfint_6 ! integrated actinic fluxes
REAL(KIND=JPRB) :: zsig_2,zsig_3,zsig_4,zsig_5,zsig_6        ! effective cross-sections
REAL(KIND=JPRB) :: zfact(inbands)                               ! cloud scaling stuff
real(KIND=JPRB) :: ZPRES(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZSZA(KPROMA),ZSZA_H(KPROMA),ZSOl(KPROMA)

#include "grg_fparam.intfb.h"
!     ------------------------------------------------------------------

!*       1.    COMPUTES NO2 photolysis frequency PJNO2
!              ---------------------------------------

  ! ===========================================================
  
IF (LHOOK) CALL DR_HOOK('GRG_JNO2',0,ZHOOK_HANDLE)
    

!*       1.   Convert solar elevation to SZA
DO JL=KSTART,KPROF
  IF (ZSOL(JL) == RMDI) ZSOL(JL)=45.0_JPRB  !Set missing SOE values to 45 deg
  ZSOL(JL) = 90._JPRB-PSOL(JL)
  ZSZA_H(JL)=COS(RPI/180.0_JPRB*(ZSOL(JL)))
  ZSZA(JL) = MAX(0.01_JPRB,ZSZA_H(JL))
ENDDO

!*       2.   COMPUTE J(NO2)

DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF

    IF (ZSZA(JL) > 0.01_JPRB) THEN
!*       2.1   Convert ozone columns to DU
      ZPCO3(JL,JLEV)  = PPCO3(JL,JLEV)*tomol / ZSZA(JL)   ! o3 overhead slant column (molec/cm2)
      zv3_du(JL,JLEV) = ZPCO3(JL,JLEV) * todu  ! DU

!*       2.2   Interval 1 + 2
      if (zv3_du(JL,JLEV) >= 300._JPRB) then
        zv3_du1(JL,JLEV) = 300._JPRB
        zv3s1(JL,JLEV)   = 8.161e+18_JPRB
      else
        zv3_du1(JL,JLEV) = zv3_du(JL,JLEV)
        zv3s1(JL,JLEV)   = ZPCO3(JL,JLEV)
      endif

!*       2.3   Interval 3 - 7
      if ( zv3_du(JL,JLEV) >= 3000.0_JPRB ) then
        zv3_du2(JL,JLEV)   = 3000.0_JPRB
        zv3s2(JL,JLEV)     = 8.161e+19_JPRB
      else
        zv3_du2(JL,JLEV)   = zv3_du(JL,JLEV)
        zv3s2(JL,JLEV)     = ZPCO3(JL,JLEV)
      endif

!*       2.4   Calculate fact using fparam routine
!              first set clouds to zero and set surface albedo, no clouds
      ZCC_COL(JL)   = 0._JPRB   ! overhead cloud cover
      zTAUC(JL,JLEV) = 0._JPRB   ! cloud optical thickness per level
      zTAUS(JL) = 0._JPRB        ! total cloud optical thickness
      ZALB(JL) = 0.05_JPRB      ! surface albedo =0.05 for tests

      ZPRES(JL,JLEV) = PRXP(JL,JLEV,2)  ! full level pressures
      zPS(JL) = PRXP(JL,KFLEV,1)  ! half level pressure of lowest level, surface pressure
      ZO3COL(JL,JLEV) =PPCO3(JL,JLEV)*tomol
      ZO3TCOL(JL) =PPCO3(JL,KFLEV)*tomol

! calculate fact and include overhead ozone corrections
      CALL GRG_FPARAM(ZO3COL(JL,JLEV),ZO3TCOL(JL), &
       & ZCC_COL(JL),zTAUC(JL,JLEV),zTAUS(JL),ZPRES(JL,JLEV),zPS(JL),PAL(JL), &
       & ZSOL(JL),zCRS_O3,zFACT)

!*       2.5   Scaled temperature
      ztscale = (PT(JL,JLEV)-250.0_JPRB)/250.0_JPRB

!*       2.6   get indices for lookup-tables
      ind = nint(min(max(1.0_JPRB,(zv3_du(JL,JLEV)-0.5_JPRB)/2.5_JPRB+1.0_JPRB),115.0_JPRB))
      i2  = ifil(ind)
      ind = nint(min(max(1.0_JPRB,(zv3_du(JL,JLEV)-5.0_JPRB)/25.0_JPRB+1.0_JPRB),115.0_JPRB))
      i3  = ifil(ind)

!*       2.7   calculate optical depth due to ozone absorbtion

      ztau_2 = taub2(i2)+taua2(i2)*zv3_du1(JL,JLEV) 
      ztau_3 = taub3(i3)+taua3(i3)*zv3_du2(JL,JLEV) 
      ztau_4 = c4_tau(1) +c4_tau(2)*zv3_du2(JL,JLEV) + &
       &      c4_tau(3)*zv3_du2(JL,JLEV)**2 + &
       &      c4_tau(4)*zv3_du2(JL,JLEV)**3
      ztau_5 = c5_tau(1)+c5_tau(2)*zv3_du2(JL,JLEV) + &
       &      c5_tau(3)*zv3_du2(JL,JLEV)**2

!*       2.8   effective actinic fluxes in units quanta/s/cm2

      zfint_2 = exp(-ztau_2+zcrs_o3(2)*zv3s1(JL,JLEV))*zf0(2)*zfact(2)*zfe(2) 
      zfint_3 = exp(-ztau_3+zcrs_o3(3)*zv3s2(JL,JLEV))*zf0(3)*zfact(3)*zfe(3)
      zfint_4 = exp(-ztau_4+zcrs_o3(4)*zv3s2(JL,JLEV))*zf0(4)*zfact(4)*zfe(4)
      zfint_5 = exp(-ztau_5+zcrs_o3(5)*zv3s2(JL,JLEV))*zf0(5)*zfact(5)*zfe(5)
      zfint_6 =                                     zf0(6)*zfact(6)*zfe(6)

!*       2.9   NO2 photolysis frequency
  
      zsig_2 = (tj_no2(1,2)+tj_no2(2,2)*ztscale) * &
       &      (b2_no2(i2)+a2_no2(i2)*zv3_du1(JL,JLEV))

      zsig_3 = (tj_no2(1,3)+tj_no2(2,3)*ztscale) * &
       &      (b3_no2(i3)+a3_no2(i3)*zv3_du2(JL,JLEV))

      zsig_4 = (tj_no2(1,4)+tj_no2(2,4)*ztscale) * &
       &      (c4_no2(1) +c4_no2(2)*zv3_du2(JL,JLEV)+c4_no2(3)*zv3_du2(JL,JLEV)**2)

      zsig_5 = (tj_no2(1,5)+tj_no2(2,5)*ztscale) * &
       &      (c5_no2(1) +c5_no2(2)*zv3_du2(JL,JLEV)+c5_no2(3)*zv3_du2(JL,JLEV)**2)

      zsig_6 = (tj_no2(1,6)+tj_no2(2,6)*ztscale) * &
       &      (c6_no2(1) +c6_no2(2)*zv3_du2(JL,JLEV))

      PJNO2(JL,JLEV) =  zsig_2*zfint_2 + &
       &                zsig_3*zfint_3 + &
       &                zsig_4*zfint_4 + &
       &                zsig_5*zfint_5 + &
       &                zsig_6*zfint_6 

    ELSE

      PJNO2(JL,JLEV) = 0.0_JPRB

    ENDIF

  ENDDO
ENDDO

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GRG_JNO2',1,ZHOOK_HANDLE)

END SUBROUTINE GRG_JNO2
