SUBROUTINE GRG_JNO2_CLOUDAD(KPROMA,KSTART,KPROF,KFLEV,PPCO3,PT,PSOL,PAL,PJNO2,PRXP,PPCO35,PT5,PCC5,PTAUC5,PTCTAUC5,PRXP5)

!**** *GRG_JNO2_CLOUDAD* - Computes NO2 photolysis frequency JNO2, adjoint

!     Purpose.
!     --------
!           Computes NO2 photolysis frequency JNO2, adjoint

!**   Interface.
!     ----------
!        *CALL* *GRG_JNO2_CLOUDAD(KPROMA,KSTART,KPROF,KFLEV,PPCO3,PT,PSOL,PAL,PJNO2,PRXP,PPCO35,PT5,PCC5,PTAUC5,PTCTAUC5,PRXP5)

!        Explicit arguments :
!        --------------------
!        KPROMA                - HORIZ. DIMENSIONING          (INPUT)
!        KSTART                - START OF WORK                (INPUT)
!        KPROF                 - DEPTH OF WORK                (INPUT)
!        KFLEV                 - NUMBER OF LEVELS             (INPUT)
!        PPCO3 (KPROMA,KFLEV)  - HALF LEVEL O3 partial column PERT (INOUT)
!        PT  (KPROMA,KFLEV)    - TEMPERATURE PERT             (INOUT)
!        PSOL (KPROMA)         - COS(SZA)                     (INPUT)
!        PAL  (KPROMA)         - Albedo                       (INPUT)
!        PJNO2 (KPROMA,KFLEV)  - JNO2 photolysis freq. PERT   (INOUT)
!        PRXP(KPROMA,0:KFLEV,NPPM)  - HALF,FULL AND LN HALF LEVEL
!                                     PRESSURES (SEE PPINIT)   (INPUT)
!        PPCO35 (KPROMA,KFLEV) - HALF LEVEL O3 partial column (INPUT)
!        PT5  (KPROMA,KFLEV)   - TEMPERATURE                  (INPUT)
!        PCC5 (KPROMA,KFLEV)   - CLOUD COVER ABOVE          (INPUT)
!        PTAUC5 (KPROMA,KFLEV) - CLOUD OPTICAL THICKNESS  (INPUT)
!        PTCTAUC5 (KPROMA) - TOTAL CLOUD OPTICAL THICKNESS  (INPUT)
!        PRXP5(KPROMA,0:KFLEV,NPPM)  - HALF,FULL AND LN HALF LEVEL
!                                     PRESSURES (SEE PPINIT)   (INPUT)


!        Implicit arguments :    None.
!        --------------------

!     Method.
!     -------
!        See documentation.

!     Externals.   None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Antje Dethof based on TM5 routine *ECMWF*

!     Modifications.
!     --------------
!        Original : 09-Oct-2006

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NPPM
USE YOMCST   , ONLY : RPI
USE GRG_PHOTOLYSIS_MIX
USE YOMANCS  , ONLY : RMDI

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)  :: KPROMA
INTEGER(KIND=JPIM),INTENT(IN)  :: KSTART
INTEGER(KIND=JPIM),INTENT(IN)  :: KPROF
INTEGER(KIND=JPIM),INTENT(IN)  :: KFLEV
REAL(KIND=JPRB)   ,INTENT(INOUT):: PPCO3(KPROMA,KFLEV) !overhead ozone (kg/m2)
REAL(KIND=JPRB)   ,INTENT(INOUT):: PT(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PSOL(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PAL(KPROMA)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PJNO2(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT)  :: PRXP(KPROMA,0:KFLEV,NPPM)
! Trajectory input fields
REAL(KIND=JPRB)   ,INTENT(IN)  :: PPCO35(KPROMA,KFLEV) !overhead ozone (kg/m2)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PT5(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PCC5(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PTAUC5(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PTCTAUC5(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PRXP5(KPROMA,0:KFLEV,NPPM)

!mvw added declarations
!REAL(KIND=JPRB) :: TAUC(KPROMA,KFLEV) ! cloud optical thickness per level
!REAL(KIND=JPRB) :: TAUS(KPROMA)       ! total cloud optical thickness
!REAL(KIND=JPRB) :: ZALB(KPROMA)       ! surface albedo
REAL(KIND=JPRB) :: ZPS(KPROMA)         ! surface pressure (for now I just take: PMID(KFLEV)
REAL(KIND=JPRB) :: ZPS5(KPROMA)         ! surface pressure (for now I just take: PMID(KFLEV)

!LOCAL PARAMETERS
INTEGER(KIND=JPIM),PARAMETER :: inbands=7
REAL(KIND=JPRB), PARAMETER   :: todu = 3.767e-17    ! O3 from molecules/cm2 --> du
!antje REAL(KIND=JPRB), PARAMETER   :: todu = 47296 ! O3 from kg/m2 --> du
REAL(KIND=JPRB), PARAMETER   :: tomol = 1.255e+21   ! O3 from kg/m2 --> mol/cm2
REAL(KIND=JPRB), PARAMETER   :: zf0(inbands) = &      ! f0: bruhl wavelength works with fintwb.data
 &   (/73.44,4.977,16.13,8.017,27.9,19.00,48.12/)
REAL(KIND=JPRB), PARAMETER   :: zcrs_o3(inbands) = &
 &   (/3.6448e-19,1.7115e-18,2.8218e-19,1.1091e-19, &
 &     2.8145e-20,1.9000e-23,4.5500e-21/)
REAL(KIND=JPRB), PARAMETER   :: zfe(inbands) = &
 &   (/2.20560e+12,2.22730e+14,7.52530e+13,1.01010e+14, &
 &     1.19010e+14,1.07760e+15,2.67400e+15/)

!LOCAL VARIABLES
INTEGER(KIND=JPIM):: JL, JLEV
INTEGER(KIND=JPIM):: i2,i3,ind !indices lookup tables

REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZPCO3(KPROMA,KFLEV)   !overhead ozone (molecules/cm2)
REAL(KIND=JPRB) :: ZPCO35(KPROMA,KFLEV)  !overhead ozone (molecules/cm2)
REAL(KIND=JPRB) :: ZO3COL(KPROMA,KFLEV)  !overhead ozone (molecules/cm2)
REAL(KIND=JPRB) :: ZO3COL5(KPROMA,KFLEV) !overhead ozone (molecules/cm2)
REAL(KIND=JPRB) :: ZO3TCOL(KPROMA)       !overhead total ozone (molecules/cm2)
REAL(KIND=JPRB) :: ZO3TCOL5(KPROMA)      !overhead total ozone (molecules/cm2)

REAL(KIND=JPRB) :: ztscale
REAL(KIND=JPRB) :: ztscale5(KPROMA,KFLEV)
REAL(KIND=JPRB) :: zv3_du(KPROMA,KFLEV),zv3_du1(KPROMA,KFLEV),zv3_du2(KPROMA,KFLEV)
REAL(KIND=JPRB) :: zv3s1(KPROMA,KFLEV),zv3s2(KPROMA,KFLEV)        ! ozone column variables
REAL(KIND=JPRB) :: zv3_du5(KPROMA,KFLEV), zv3_du15(KPROMA,KFLEV), zv3_du25(KPROMA,KFLEV)
REAL(KIND=JPRB) :: zv3s15(KPROMA,KFLEV),zv3s25(KPROMA,KFLEV)      ! ozone column variables
REAL(KIND=JPRB) :: ztau_2,ztau_3,ztau_4,ztau_5                      ! optical depth
REAL(KIND=JPRB) :: ztau_25(KPROMA,KFLEV),ztau_35(KPROMA,KFLEV),&
 &                 ztau_45(KPROMA,KFLEV),ztau_55(KPROMA,KFLEV)  
REAL(KIND=JPRB) :: zfint_2,zfint_3,zfint_4, zfint_5,zfint_6   ! integrated actinic fluxes
REAL(KIND=JPRB) :: zfint_25(KPROMA,KFLEV),zfint_35(KPROMA,KFLEV),&
 &                 zfint_45(KPROMA,KFLEV),zfint_55(KPROMA,KFLEV),&
 &                 zfint_65(KPROMA,KFLEV)  ! integrated actinic fluxes
REAL(KIND=JPRB) :: zsig_2,zsig_3,zsig_4,zsig_5,zsig_6          ! effective cross-sections
REAL(KIND=JPRB) :: zsig_25(KPROMA,KFLEV),zsig_35(KPROMA,KFLEV), &
 &                 zsig_45(KPROMA,KFLEV),zsig_55(KPROMA,KFLEV), &  
 &                 zsig_65(KPROMA,KFLEV)    ! effective cross-sections
REAL(KIND=JPRB) :: zfact(7)
REAL(KIND=JPRB) :: zfact5(7),zfact_h5(KPROMA,KFLEV,7)           ! cloud scaling stuff
real(KIND=JPRB) :: ZPRES(KPROMA,KFLEV),ZPRES5(KPROMA,KFLEV)
REAL(KIND=JPRB) :: ZSZA(KPROMA),ZSZA_H(KPROMA),ZSOl(KPROMA)

!     ------------------------------------------------------------------


#include "grg_fparam.intfb.h"
#include "grg_fparamad.intfb.h"

!*       1.    COMPUTES NO2 photolysis frequency PJNO2, tangent linear
!              -------------------------------------------------------

  ! ===========================================================
  
IF (LHOOK) CALL DR_HOOK('GRG_JNO2_CLOUDAD',0,ZHOOK_HANDLE)
    
!*       1.   No clouds yet, fact=1
!fact = 1.0

!*       1.1   Convert solar elevation to SZA
ZSZA(:)=0._JPRB
DO JL=KSTART,KPROF
  IF (PSOL(JL) == RMDI) ZSOL(JL)=45.0_JPRB  !Set missing SOE values to 45 deg
  ZSOL(JL) = 90._JPRB-PSOL(JL)
  ZSZA_H(JL)=COS(RPI/180.0_JPRB*(ZSOL(JL)))
  ZSZA(JL) = MAX(0.01_JPRB,ZSZA_H(JL))
ENDDO

!*       0.   COMPUTE GENERALIZED TRAJECTORY
!-----------------------------------------------------------

!*       0.1   Convert ozone columns to DU
zv3_du5(:,:)=0._JPRB
DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF
    IF (ZSZA(JL) > 0.01_JPRB) THEN
      ZPCO35(JL,JLEV) = PPCO35(JL,JLEV)*tomol/ZSZA(JL) ! o3 overhead slant column (molec/cm2) 
      zv3_du5(JL,JLEV) = ZPCO35(JL,JLEV)*todu     ! DU
!*       0.2   Interval 1 + 2, TRAJ
      if (zv3_du5(JL,JLEV) >= 300._JPRB) then
        zv3_du15(JL,JLEV) = 300._JPRB
        zv3s15(JL,JLEV)   = 8.161e+18_JPRB
      else
        zv3_du15(JL,JLEV) = zv3_du5(JL,JLEV)
        zv3s15(JL,JLEV)   = ZPCO35(JL,JLEV)
      endif

!*       0.3   Interval 3 - 7, TRAJ
      if ( zv3_du5(JL,JLEV) >= 3000.0_JPRB ) then
        zv3_du25(JL,JLEV)   = 3000.0_JPRB
        zv3s25(JL,JLEV)     = 8.161e+19_JPRB
      else
        zv3_du25(JL,JLEV)   = zv3_du5(JL,JLEV)
        zv3s25(JL,JLEV)     = ZPCO35(JL,JLEV)
      endif

!*       0.4   Calculate fact using fparam routine
!              first set clouds to zero and set surface albedo, no clouds
!      ZCC_COL(JL)   = 0._JPRB   ! overhead cloud cover
!      zTAUC(JL,JLEV) = 0._JPRB   ! cloud optical thickness per level
!      zTAUS(JL) = 0._JPRB        ! total cloud optical thickness
!      ZALB(JL) = 0.05_JPRB      ! surface albedo =0.05 for tests

      ZPRES5(JL,JLEV) = PRXP5(JL,JLEV,2)  ! full level pressures
      ZPS5(JL) = PRXP5(JL,KFLEV,1)  ! half level pressure of lowest level, surface pressure
      ZO3COL5(JL,JLEV) =PPCO35(JL,JLEV)*tomol
      ZO3TCOL5(JL) =PPCO35(JL,KFLEV)*tomol

! calculate fact and include overhead ozone corrections
      CALL GRG_FPARAM(ZO3COL5(JL,JLEV),ZO3TCOL5(JL), &
       & PCC5(JL,JLEV),PTAUC5(JL,JLEV),PTCTAUC5(JL),ZPRES5(JL,JLEV),ZPS5(JL),PAL(JL), &
       & ZSOL(JL),ZCRS_O3,ZFACT5)
       zFACT_H5(JL,JLEV,:)=zFACT5(:)


!*       0.5   Scaled temperature
      ztscale5(JL,JLEV) = (PT5(JL,JLEV)-250.0_JPRB)/250.0_JPRB

!*       0.6   get indices for lookup-tables
      ind = nint(min(max(1.0_JPRB,(zv3_du5(JL,JLEV)-0.5_JPRB)/2.5_JPRB+1.0_JPRB),115.0_JPRB))
      i2 = ifil(ind)
      ind = nint(min(max(1.0_JPRB,(zv3_du5(JL,JLEV)-5.0_JPRB)/25.0_JPRB+1.0_JPRB),115.0_JPRB))
      i3 = ifil(ind)

!*       0.7   calculate optical depth due to ozone absorbtion

      ztau_25(JL,JLEV) = taub2(i2)+taua2(i2)*zv3_du15(JL,JLEV)
      ztau_35(JL,JLEV) = taub3(i3)+taua3(i3)*zv3_du25(JL,JLEV)
      ztau_45(JL,JLEV) = c4_tau(1) +c4_tau(2)*zv3_du25(JL,JLEV) + &
       &       c4_tau(3)*zv3_du25(JL,JLEV)**2 + &
       &       c4_tau(4)*zv3_du25(JL,JLEV)**3
      ztau_55(JL,JLEV) = c5_tau(1)+c5_tau(2)*zv3_du25(JL,JLEV)+ &
       &       c5_tau(3)*zv3_du25(JL,JLEV)**2

!*       0.8   effective actinic fluxes in units quanta/s/cm2
      zfint_25(JL,JLEV) = exp(-ztau_25(JL,JLEV)+zcrs_o3(2)*zv3s15(JL,JLEV))*zf0(2)*zfact_h5(JL,JLEV,2)*zfe(2)
      zfint_35(JL,JLEV) = exp(-ztau_35(JL,JLEV)+zcrs_o3(3)*zv3s25(JL,JLEV))*zf0(3)*zfact_h5(JL,JLEV,3)*zfe(3)
      zfint_45(JL,JLEV) = exp(-ztau_45(JL,JLEV)+zcrs_o3(4)*zv3s25(JL,JLEV))*zf0(4)*zfact_h5(JL,JLEV,4)*zfe(4)
      zfint_55(JL,JLEV) = exp(-ztau_55(JL,JLEV)+zcrs_o3(5)*zv3s25(JL,JLEV))*zf0(5)*zfact_h5(JL,JLEV,5)*zfe(5)
      zfint_65(JL,JLEV) = zf0(6)*zfact_h5(JL,JLEV,6)*zfe(6)

!*       0.9  NO2, photolosis rate

      zsig_25(JL,JLEV) = (tj_no2(1,2)+tj_no2(2,2)*ztscale5(JL,JLEV)) * &
       &       (b2_no2(i2)+a2_no2(i2)*zv3_du15(JL,JLEV))

      zsig_35(JL,JLEV) = (tj_no2(1,3)+tj_no2(2,3)*ztscale5(JL,JLEV)) * &
       &       (b3_no2(i3)+a3_no2(i3)*zv3_du25(JL,JLEV))

      zsig_45(JL,JLEV) = (tj_no2(1,4)+tj_no2(2,4)*ztscale5(JL,JLEV)) * &
       &       (c4_no2(1) +c4_no2(2)*zv3_du25(JL,JLEV)+c4_no2(3)*zv3_du25(JL,JLEV)**2)

      zsig_55(JL,JLEV) = (tj_no2(1,5)+tj_no2(2,5)*ztscale5(JL,JLEV)) * &
       &       (c5_no2(1) +c5_no2(2)*zv3_du25(JL,JLEV)+c5_no2(3)*zv3_du25(JL,JLEV)**2)

      zsig_65(JL,JLEV) = (tj_no2(1,6)+tj_no2(2,6)*ztscale5(JL,JLEV)) * &
       &       (c6_no2(1) +c6_no2(2)*zv3_du25(JL,JLEV))

    ENDIF
  ENDDO
ENDDO

!end trajectory calculations
!-----------------------------------------------------------

!*       2.   COMPUTE J(NO2)

zfint_2=0.0_JPRB
zfint_3=0.0_JPRB
zfint_4=0.0_JPRB
zfint_5=0.0_JPRB
zfint_6=0.0_JPRB
zsig_2=0.0_JPRB
zsig_3=0.0_JPRB
zsig_4=0.0_JPRB
zsig_5=0.0_JPRB
zsig_6=0.0_JPRB
ztscale=0.0_JPRB
zfact(:)=0.0_JPRB
zv3_du2(:,:)= 0.0_JPRB
zv3_du1(:,:)= 0.0_JPRB
zv3s2(:,:)= 0.0_JPRB
zv3s1(:,:)= 0.0_JPRB
zv3_du(:,:)= 0.0_JPRB
ztau_2=0.0_JPRB
ztau_3=0.0_JPRB
ztau_4=0.0_JPRB
ztau_5=0.0_JPRB
ZPCO3(:,:)=0.0_JPRB
ZPRES(:,:) =0.0_JPRB
ZPS(:) =0.0_JPRB
ZO3COL(:,:) =0.0_JPRB
ZO3TCOL(:) =0.0_JPRB

DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF

    IF (ZSZA(JL) > 0.01_JPRB) THEN

!*       2.8  NO2, photolosis rate
      zfint_2 = zfint_2 + PJNO2(JL,JLEV)*zsig_25(JL,JLEV)
      zsig_2  = zsig_2 + PJNO2(JL,JLEV)*zfint_25(JL,JLEV)
      zfint_3 = zfint_3 + PJNO2(JL,JLEV)*zsig_35(JL,JLEV)
      zsig_3  = zsig_3 + PJNO2(JL,JLEV)*zfint_35(JL,JLEV)
      zfint_4 = zfint_4 + PJNO2(JL,JLEV)*zsig_45(JL,JLEV)
      zsig_4  = zsig_4 + PJNO2(JL,JLEV)*zfint_45(JL,JLEV)
      zfint_5 = zfint_5 + PJNO2(JL,JLEV)*zsig_55(JL,JLEV)
      zsig_5  = zsig_5 + PJNO2(JL,JLEV)*zfint_55(JL,JLEV)
      zfint_6 = zfint_6 + PJNO2(JL,JLEV)*zsig_65(JL,JLEV)
      zsig_6  = zsig_6 + PJNO2(JL,JLEV)*zfint_65(JL,JLEV)
      PJNO2(JL,JLEV) = 0.0_JPRB

      ztscale = ztscale +  tj_no2(2,6)*zsig_6* &
       &       (c6_no2(1) +c6_no2(2)*zv3_du25(JL,JLEV))
      zv3_du2(JL,JLEV) = zv3_du2(JL,JLEV) + &
       &       (tj_no2(1,6)+tj_no2(2,6)*ztscale5(JL,JLEV))*c6_no2(2)*zsig_6
      zsig_6  = 0.0_JPRB

      ztscale = ztscale + tj_no2(2,5)*zsig_5* &
       &       (c5_no2(1) +c5_no2(2)*zv3_du25(JL,JLEV)+&
       &        c5_no2(3)*zv3_du25(JL,JLEV)**2)
      zv3_du2(JL,JLEV) = zv3_du2(JL,JLEV) +&
       &       (tj_no2(1,5)+tj_no2(2,5)*ztscale5(JL,JLEV))* &
       &       (c5_no2(2)*zsig_5 +2.0_JPRB*c5_no2(3)*zsig_5*zv3_du25(JL,JLEV))
      zsig_5  = 0.0_JPRB

      ztscale = ztscale + tj_no2(2,4)*zsig_4*&
       &       (c4_no2(1) +c4_no2(2)*zv3_du25(JL,JLEV)+ &
       &        c4_no2(3)*zv3_du25(JL,JLEV)**2)
      zv3_du2(JL,JLEV) = zv3_du2(JL,JLEV) +&
       &       (tj_no2(1,4)+tj_no2(2,4)*ztscale5(JL,JLEV))*&
       &       (c4_no2(2)*zsig_4 +2.0_JPRB*c4_no2(3)*zsig_4*zv3_du25(JL,JLEV))
      zsig_4  = 0.0_JPRB

      ztscale = ztscale +tj_no2(2,3)*zsig_3* &
       &       (b3_no2(i3)+a3_no2(i3)*zv3_du25(JL,JLEV))
      zv3_du2(JL,JLEV) = zv3_du2(JL,JLEV) + &
       &       (tj_no2(1,3)+tj_no2(2,3)*ztscale5(JL,JLEV))*a3_no2(i3)*zsig_3
      zsig_3  = 0.0_JPRB

      ztscale = ztscale +tj_no2(2,2)*zsig_2* &
       &       (b2_no2(i2)+a2_no2(i2)*zv3_du15(JL,JLEV))
      zv3_du1(JL,JLEV) = zv3_du1(JL,JLEV) + &
       &       (tj_no2(1,2)+tj_no2(2,2)*ztscale5(JL,JLEV))*a2_no2(i2)*zsig_2
      zsig_2  = 0.0_JPRB
  
!*       2.8   effective actinic fluxes in units quanta/s/cm2

      zfact(6)= zfact(6)+zf0(6)*zfint_6*zfe(6)
      zfint_6 = 0.0_JPRB                                       
  
      ztau_5 = ztau_5 - &
       &      zfint_5*exp(-ztau_55(JL,JLEV)+zcrs_o3(5)*zv3s25(JL,JLEV))* &
       &      zf0(5)*zfact_h5(JL,JLEV,5)*zfe(5)
      zv3s2(JL,JLEV) = zv3s2(JL,JLEV) + &
       &      zcrs_o3(5)*zfint_5*exp(-ztau_55(JL,JLEV)+zcrs_o3(5)*zv3s25(JL,JLEV))*&
       &      zf0(5)*zfact_h5(JL,JLEV,5)*zfe(5)
      zfact(5)=zfact(5)+ exp(-ztau_55(JL,JLEV)+zcrs_o3(5)*zv3s25(JL,JLEV))*zf0(5)*zfint_5*zfe(5)
      
      zfint_5= 0.0_JPRB

      ztau_4 = ztau_4 - &
       &      zfint_4*exp(-ztau_45(JL,JLEV)+zcrs_o3(4)*zv3s25(JL,JLEV))*&
       &      zf0(4)*zfact_h5(JL,JLEV,4)*zfe(4)
      zv3s2(JL,JLEV) = zv3s2(JL,JLEV) + &
       &      zcrs_o3(4)*zfint_4*exp(-ztau_45(JL,JLEV)+zcrs_o3(4)*zv3s25(JL,JLEV))*&
       &      zf0(4)*zfact_h5(JL,JLEV,4)*zfe(4)
      zfact(4)=zfact(4)+exp(-ztau_45(JL,JLEV)+zcrs_o3(4)*zv3s25(JL,JLEV))*zf0(4)*zfint_4*zfe(4)
      zfint_4= 0.0_JPRB

      ztau_3 = ztau_3 - &
       &      zfint_3*exp(-ztau_35(JL,JLEV)+zcrs_o3(3)*zv3s25(JL,JLEV))*&
       &      zf0(3)*zfact_h5(JL,JLEV,3)*zfe(3)
      zv3s2(JL,JLEV) = zv3s2(JL,JLEV) + &
       &      zcrs_o3(3)*zfint_3*exp(-ztau_35(JL,JLEV)+zcrs_o3(3)*zv3s25(JL,JLEV))*&
       &      zf0(3)*zfact_h5(JL,JLEV,3)*zfe(3)
      zfact(3)=zfact(3)+exp(-ztau_35(JL,JLEV)+zcrs_o3(3)*zv3s25(JL,JLEV))*zf0(3)*zfint_3*zfe(3)
      zfint_3= 0.0_JPRB

      ztau_2 = ztau_2 - &
       &      zfint_2*exp(-ztau_25(JL,JLEV)+zcrs_o3(2)*zv3s15(JL,JLEV))*&
       &      zf0(2)*zfact_h5(JL,JLEV,2)*zfe(2)
      zv3s1(JL,JLEV) = zv3s1(JL,JLEV) + &
       &      zcrs_o3(2)*zfint_2*exp(-ztau_25(JL,JLEV)+zcrs_o3(2)*zv3s15(JL,JLEV))*&
       &      zf0(2)*zfact_h5(JL,JLEV,2)*zfe(2)
      zfact(2)=zfact(2)+exp(-ztau_25(JL,JLEV)+zcrs_o3(2)*zv3s15(JL,JLEV))*zf0(2)*zfint_2*zfe(2)
      zfint_2= 0.0_JPRB

!*       2.7   calculate optical depth due to ozone absorbtion

      zv3_du2(JL,JLEV) = zv3_du2(JL,JLEV)+c5_tau(2)*ztau_5 + &
       &      2.0_JPRB*c5_tau(3)*ztau_5*zv3_du25(JL,JLEV)
      ztau_5 = 0.0_JPRB

      zv3_du2(JL,JLEV) = zv3_du2(JL,JLEV) +c4_tau(2)*ztau_4 + &
       &      2.0_JPRB*c4_tau(3)*ztau_4*zv3_du25(JL,JLEV) + &
       &      3.0_JPRB*c4_tau(4)*ztau_4*zv3_du25(JL,JLEV)**2
      ztau_4 = 0.0_JPRB
   
      zv3_du2(JL,JLEV)= zv3_du2(JL,JLEV) +taua3(i3)*ztau_3
      ztau_3 = 0.0_JPRB

      zv3_du1(JL,JLEV) = zv3_du1(JL,JLEV) +taua2(i2)*ztau_2
      ztau_2 = 0.0_JPRB

!*       2.6   get indices for lookup-tables

!*       2.5   Scaled temperature
      PT(JL,JLEV) = PT(JL,JLEV) + ztscale/250.0_JPRB
      ztscale = 0.0_JPRB

!*       2.4   Calculate fact using fparam routine
!              first set clouds to zero and set surface albedo, no clouds
! calculate fact and include overhead ozone corrections
      CALL GRG_FPARAMAD(ZO3COL(JL,JLEV),ZO3TCOL(JL), &
       & PCC5(JL,JLEV),PTAUC5(JL,JLEV),PTCTAUC5(JL),ZPRES(JL,JLEV),ZPS(JL),PAL(JL), &
       & ZSOL(JL),zCRS_O3,zFACT,&
       & ZO3COL5(JL,JLEV),ZO3TCOL5(JL),ZPRES5(JL,JLEV),ZPS5(JL))

      PPCO3(JL,KFLEV)=PPCO3(JL,KFLEV)+ZO3TCOL(JL)*tomol
      ZO3TCOL(JL)=0.0_JPRB

      PPCO3(JL,JLEV)=PPCO3(JL,JLEV)+ZO3COL(JL,JLEV)*tomol
      ZO3COL(JL,JLEV)=0.0_JPRB

      PRXP(JL,KFLEV,1)=PRXP(JL,KFLEV,1)+ZPS(JL)
      ZPS(JL)=0.0_JPRB
    
      PRXP(JL,JLEV,2)=PRXP(JL,JLEV,2)+ZPRES(JL,JLEV)
      ZPRES(JL,JLEV)=0.0_JPRB


!*       2.3   Interval 3 - 7, PERT
      if ( zv3_du5(JL,JLEV) >= 3000.0_JPRB ) then
        zv3_du2(JL,JLEV)   = 0.0_JPRB
        zv3s2(JL,JLEV)     = 0.0_JPRB
      else
        ZPCO3(JL,JLEV)   = ZPCO3(JL,JLEV) + zv3s2(JL,JLEV)
        zv3s2(JL,JLEV)    = 0.0_JPRB
        zv3_du(JL,JLEV)   = zv3_du(JL,JLEV) + zv3_du2(JL,JLEV)
        zv3_du2(JL,JLEV)  = 0.0_JPRB
      endif

!*       2.2   Interval 1 + 2, PERT
      if (zv3_du5(JL,JLEV) >= 300._JPRB) then
        zv3_du1(JL,JLEV) = 0.0_JPRB
        zv3s1(JL,JLEV)   = 0.0_JPRB
      else
        ZPCO3(JL,JLEV)   = ZPCO3(JL,JLEV) + zv3s1(JL,JLEV)
        zv3s1(JL,JLEV)    = 0.0_JPRB
        zv3_du(JL,JLEV)   = zv3_du(JL,JLEV) + zv3_du1(JL,JLEV)
        zv3_du1(JL,JLEV)  = 0.0_JPRB
      endif

!*       2.1   Convert ozone columns to DU
      ZPCO3(JL,JLEV)  = ZPCO3(JL,JLEV) + zv3_du(JL,JLEV)*todu
      zv3_du(JL,JLEV)  = 0.0_JPRB

      PPCO3(JL,JLEV)  = PPCO3(JL,JLEV) + ZPCO3(JL,JLEV)*tomol/ZSZA(JL)
      ZPCO3(JL,JLEV)  = 0.0_JPRB 

    ELSE

      PJNO2(JL,JLEV)  = 0.0_JPRB

    ENDIF
  ENDDO
ENDDO

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GRG_JNO2_CLOUDAD',1,ZHOOK_HANDLE)

END SUBROUTINE GRG_JNO2_CLOUDAD
