SUBROUTINE HOP(KDLEN,KDBDY,KSET,KHORIZ)

!**** *HOP* - Operator routine for all types of observations.

!     E. ANDERSSON            ECMWF          01/04/99
!     Based on sections of TOVCLR, UPPERAIR, SURFACEO and SATEM

!     PURPOSE
!     -------
!        Treats one set of observations, performing the following tasks:
!     1. Prepare for interpolation in vertical
!     2. Extract observed pressures and variable numbers
!     3. Perform post-processing for the required parameters
!     4. Compute departures
!     5. Calculate the cost function incorporating variational quality control

!     This routine can be used by FRENCH and ECMWF VAR.

!**   INTERFACE
!     ---------
!        *CALL* *HOP(KDLEN,KDBDY,KSET)*
!     WHERE KDLEN = First dimension: Max number of observations, this set
!           KDBDY = Second dimension: Max number of bodies, this set
!           KSET  = Number (numero) of the obs set to be treated (INPUT)
!                   task (INPUT/OUTPUT).
!           KHORIZ= Number of horizontal locations for 2D upper-air operators 

!     Also the contents of YOMMVO, YOMOBA, YOMOBSET are implicit inputs.

!     HOP is called from TASKOB

!**   METHOD
!     ------
!        The method employed in this routine is to let the content of the
!     observation file determine which observation operators to call. The
!     'variable numbers' and the 'vertcal postions' of all data belonging
!     to the current set (KSET) are extracted from the observation array.
!     A 'request' is built, consisting of IVNMRQ (variable numbers) and
!     ZVERTP, both dimensioned (1:KDLEN,1:KDBDY), where KDLEN is <= NMXLEN
!     (=511) and KDBDY <= MMXBDY(KSET) (maximum number of CMA bodies in
!     this set, determined in ecset.F90 and smtov.F90). Invalid requests
!     may be nullified by setting IVNMRQ(jobs,JBODY)=0.
!        All loops vectorize over the first dimension, 1:ILEN.
!        The PREINT routines are used to extract model information at
!     observation points, mostly from GOM-arrays.
!        The lists of known variable numbers, and the available obs-operators
!     are matched (in hvnmtlt.F90). There is a loop over all known variable
!     numbers (currently 79 of them). If the loop-variable-number has any
!     matching ocurrences in IVNMRQ, then the relevant vertical positions
!     are extracted (and contracted) from ZVERTP into ZVPOBS and the
!     appropriate PPOBSA-routine is called.
!        Some special treatment is necessary for two-level quantities (PWC,
!     DZ, layer-mean RH and integrated O3), for winds (U,V, U10,V10 and FF)
!     and radiances. The latter require interpolation of T, q and O3 to NLSAT
!     fix pressure levels.
!        The results from the PPOBSA-routines are accumulated in ZHOFX(:,:),
!     i.e. H(x). ZHOFX has the same layout as IVNMRQ(:,:) and ZVERTP(:,:).
!        The routine hdepart.F90 computes the departures H(x)-obs and stores
!     them in the observation array. It overwrites ZHOFX with the normalized
!     departures, [obs-H(x)]/sigma_o.
!        The routine hjo.F90 computes jo for each datum, and applies VarQC.
!     Any observation-error correlation will be taken care of inside hjo.F90,
!     when coded (not yet).

!**   EXTERNALS
!     ---------
!        HVNMTLT - Translate varaible numbers tables
!        PREINT  - Prepare interpolation
!        PREINTS - Prepare interpolation, surface
!        PREINTR - Prepare interpolation, radiances
!        QNEGLIM - Ensure positive humidity
!        HINTH   - Get interpolation height
!        GRG_AK_OP- Equivalents of observed values using averaging kernels
!        PPOBSA  - Equivalents of observed values
!        PPOBSAS - Equivalents of observed values, surface
!        PPOBSAZ - Equivalents of observed values, surface pressure
!        HRADP   - Radiance preparations
!        HRADP_ML - Radiance preparations (for RTTOV-based interpolation)
!        RADTR   - Interface to radiative transfer code.
!        RADTR_ML - Interface to radiative transfer code (RT on model levels)
!        HDEPART - Obs. departures, vectorized version
!        HJO     - Cost function calculation, with VarQC
!        MEANUV_WEIGHTS - Calculation of weights for AMV/SATOB obs operator
!        MEANUV_AVERAGE - Layer mean for AMV/SATOB obs operator

!**   MODIFICATIONS
!     -------------
!     Erik Andersson  99/04/01 : Original
!     E. Gerard       99/05/05 : Assimilation of SSM/I 10-m wind speed
!     Pedro Viterbo   99/09/01 : Sea-ice fraction
!     P. Moll         99/10/15 : Put Apache and ACHMT in cy22 (for MF)
!     F.Bouttier      00/03/27 : Add incremental obs filter
!     Xu Li           00/05/23 : (1) Add offset for VTPR1 and VTPR 2
!                                (2) Transfer Satellite sensor array to HJO    
!     N. Bormann      01/09/21 : Layer-mean operator for AMV/SATOBs  
!     L.Gaytandjieva  00/11/01 : Add ZTS5 as input param. in PPOBSN
!     A. Dethof       02/07/02 : Disable layer check for PWC from ozone bufr
!     D. Dee          03/10/22 : Variational bias correction
!     E. Andersson    04/11/02 : Introduce ground-based GPS
!     S. Healy        04/10/27 : Add call for GPSRO operator 
!     C. Payan        04/06/23 : use of MDBALT for fixing land/sea flag for AMV
!     P.Burton        14/10/04 : Added KSET arg to radtrb
!     A. Benedetti    05/03/31 : Add AEROSOL OPTICAL DEPTH dummy operator  
!     N. Bormann      04/12/23 : Call RTLIMB for limb rad.; RARAD becomes LIMB 
!        Y.Tremolet    18-Feb-2005 Jo for active data only in twin experiment
!     A. Benedetti    05/08/15 : AEROSOL OPTICAL DEPTH operator
!     C. Payan        05/02/22 : land/sea value in MDBALT for AMV and SCATT
!     Y. Bouteloup    05/01/13 : Introduction of cloud condensate liquid and ice
!                                  in the call to ppobsap (like ppobsa)
!     A. Dethof       20-Feb-06: USe of averaging kernels for reo3 data
!     T. Auligne      05/12/09 : Added nadir viewing angle in VarBC and special
!                                  handling of RMDI
!     S. Healy        05/12/08 : 2d operator for GPSRO
!     N. Bormann      05/12/08 : 2d operator for limb radiances
!     T. Auligne      10/01/07 : old scan correction no longer added to VarBC
!     R. El Khatib & NEC 01-Aug-2006 Vectorization
!     D. Tan          07/03/02 : Add HLoS wind, azimuth retrieved from Aux(1)@body
!     P. Poli         07/03/07 : Extra outputs for GPSRO obs operator at MF
!     K. Scipal       08/04/10 : introduced scat soil moisture (#4.5.12)
!     P. Bauer        08/01/03 : Add Envisat MERIS TCWV
!     N. Bormann      07/11/15 : RTTOV-9
!     V. Guidard      07/08/16 : Modify Call RADTR (after A. Collard)
!     R. Engelen      08/10/22 : Add prior profile to reo3 averaging kernel and 
!                                  generalize GRG AK to GEMS
!     A. Geer         08/09/01 : Rainy radiance assimilation  
!     Y.Tremolet     29-May-2008 Print Jo table by bins
!     H. Hersbach     04/07/08 : Add ocean current and wave Charnock
!     D. Dee          10-Oct-2008 generalise VarBC
!     D. Dee          10-Oct-2008 generalise VarBC
!     P. Bauer        23/01/09 : Add VarBC for TCWV
!     D. Dee        10-Oct-2008 generalise VarBC
!     A. Benedetti    09/03/11 : Add AEROSOL REFLECTANCE OPERATOR (AER_REFL_OP)
!     D. Li         05-Dec-2008: Add mode-based VarBC in control variable 
!     A. Inness     19-Aug-2009: Add Varbc for GRG
!     A.Inness    13-Oct-2009: Include apriori profile for GRG
!     A.Inness    18-Nov-2009: Averaging kernel operator for MOPITT v4.
!     A. Geer       03-Dec-2008 Solar hour VarBC predictor for TMI
!     V. Guidard    2009/02/24 : CO2-slicing for AIRS  
!     A. Geer       10-Jun-2009 VarBC statistics by channel; All-sky flags change
!     H. Hersbach     04/12/09 : 10-m neutral wind
!     N. Bormann     09/10/09: Boxcar average for AMVs
!     A. Fouilloux   31/03/10: use new codetype for MERIS data
!     J. Munoz Sabater Apr-2010: Add SMOS to store bg departures
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARCMA   , ONLY : JPMXVAR  ,JPMX_AK
USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMDIMO  , ONLY : NLSAT, NPNAV
USE YOMVAR   , ONLY : NUPTRA   ,LVARBC, LTSCV, MUPTRA
USE VARBC_SETUP,ONLY : YVARBC, NHSTBINS
USE VARBC_EVAL, ONLY : VARBC_BIAS
USE VARBC_PRED, ONLY : JPMXNPRED, VARBC_PRED_RAD, VARBC_PRED_TO3, VARBC_PRED_AIREP
USE VARBC_AIREP, ONLY : LBC_AIREP
USE YOMVRTL  , ONLY : LINCV
USE YOMCMA   , ONLY : NRESUPD
USE YOMCT0   , ONLY : LCANARI  ,LOBSC1   ,NCONF, LSCREEN, LIFSTRAJ
USE YOMCST   , ONLY : RG
USE YOMOBS   , ONLY : LCAPACH  ,LMFSCREEN,LCACHMT
USE YOMVNMB  , ONLY : NVNUMB
USE YOMOBSET , ONLY : MTYPOB   ,MLNSET   ,MTSLSET
USE YOMCOSJO , ONLY : LQSCATT  ,NOTVAR, NPRTBINS
USE YOMCOCTP , ONLY : NAIREP, NSATOB   ,NTEMP    ,NPILOT   ,&
 & NSSMI    ,NGTHRB   ,NTCWC  , &
 & NSCATT   ,NSYNOP   ,NDRIBU   ,NPAOB    ,&
 & NSTRO3   ,NSATEM   ,NLIMB, NRADAR, NSMOS
USE YOMLUN   , ONLY : NULERR   , NULOUT
USE YOMANCS  , ONLY : RMDI     ,NMDI     ,RDEGREES
USE YOMCT0B  , ONLY : LECMWF
USE YOMTVRAD , ONLY : RADPRE   ,&
 & SATGRP_TABLE,MSENSOR_AIRS,MSENSOR_HIRS,SATOBGRP_TABLE,&
 & MXSENSOR, LRTTOV_INTERPOL, NSENSOR_MERIS, NSENSOR_MOPITT,&
 & LCO2_DIAG_AIRS
USE YOMLIMB,   ONLY : Y_LIMBGRP_TABLE
USE YOMCMBDY , ONLY : NCMDACBP ,NCMDACOC
USE YOMDB
USE YOMNNE   , ONLY : LNNWRI   ,LNNHC    ,BETA
USE ENKF_MIX , ONLY : LENKF
USE YOMGRB   , ONLY : NENSFNB
USE GOMS_MIX     , ONLY : YGOMEC
USE TESTVAR_MIX  , ONLY : LTWINEXP
USE TYPE_GEMS_PROFILES
USE YOM_YGFL , ONLY : NGHG,NGRG,YGHG_NL,YGRG_NL
USE YOMTS    , ONLY : TSCVX5
USE YOEPHY   , ONLY : LERAIN, LEMWAVE, LESMOS
USE YOMMWAVE , ONLY : CLDBIT_SHIFT, MWAVE_SERIOUS, MWAVE_OK
USE YOMMKODB , ONLY : NTBMAR, NTFMAR
USE YOMLUN   , ONLY : NULOUT

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN
INTEGER(KIND=JPIM),INTENT(IN)    :: KDBDY
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET
INTEGER(KIND=JPIM),INTENT(IN)    :: KHORIZ
!                   ------------

!   ZPRESH5,ZPRESF5 :   Half/Full level pressure values at obs points
!   ZGEOPH5,ZGEOPF5 :   Half/Full level geopotential values at obs points
REAL(KIND=JPRB), DIMENSION(KDLEN,0:NFLEVG) :: ZPRESH5,ZGEOPH5
REAL(KIND=JPRB), DIMENSION(KDLEN,NFLEVG)   :: ZPRESF5,ZGEOPF5

!   ZPRESH5_2D,ZPRESF5_2D :   Half/Full level pressure values at obs points
!   ZGEOPH5_2D,ZGEOPF5_2D :   Half/Full level geopotential values at obs points
REAL(KIND=JPRB), DIMENSION(KDLEN,0:NFLEVG,KHORIZ) :: ZPRESH5_2D,ZGEOPH5_2D
REAL(KIND=JPRB), DIMENSION(KDLEN,NFLEVG,KHORIZ)   :: ZPRESF5_2D,ZGEOPF5_2D


!   ZPXP5,ZPXPD5 :   Intermediate variables (see PPINIT)
!   ZPXZ5,ZPXZD5 :   Intermediate variables (see PPINITZ)
REAL(KIND=JPRB), DIMENSION(KDLEN,0:NFLEVG,NPPM) ::ZPXP5,ZPXPD5,ZPXZ5,ZPXZD5

!   ZPXP5_2D,ZPXPD5_2D :   Intermediate variables (see PPINIT)
!   ZPXZ5_2D,ZPXZD5_2D :   Intermediate variables (see PPINITZ)
REAL(KIND=JPRB), DIMENSION(KDLEN,0:NFLEVG,NPPM,KHORIZ) ::ZPXP5_2D,ZPXPD5_2D,ZPXZ5_2D,ZPXZD5_2D

!   ZALPH5  :   Coefficients of the hydrostatics (see GPXYB)
!   ZLNPR5  :   LOG. of the ratio of pressure (see GPXYB)
!   ZDELP5  :   Pressure diff. across layers (see GPXYB)
!   ZRDELP5 :   Inverse of the above
!   ZRTGR5  :   For press. grad. term and ene. conv. (see GPXYB) -
!   ZRPRE5  :   Inverse of pressure
!   ZRPP5   :   Inverse of P(j)*P(j+1)
REAL(KIND=JPRB), DIMENSION(KDLEN,NFLEVG) ::&
 & ZALPH5,ZLNPR5,ZDELP5,ZRDELP5,ZRTGR5,ZRPRE5,ZRPP5  

!   ZALPH5_2D  :   Coefficients of the hydrostatics (see GPXYB)
!   ZLNPR5_2D  :   LOG. of the ratio of pressure (see GPXYB)
!   ZDELP5_2D  :   Pressure diff. across layers (see GPXYB)
!   ZRDELP5_2D :   Inverse of the above
!   ZRTGR5_2D  :   For press. grad. term and ene. conv. (see GPXYB) -
!   ZRPRE5_2D  :   Inverse of pressure
!   ZRPP5_2D   :   Inverse of P(j)*P(j+1)
REAL(KIND=JPRB), DIMENSION(KDLEN,NFLEVG,KHORIZ) ::&
 & ZALPH5_2D,ZLNPR5_2D,ZDELP5_2D,ZRDELP5_2D,ZRTGR5_2D,ZRPRE5_2D,ZRPP5_2D  

!   ZT05    :   Surface temperature (see CTSTAR)
!   ZTSTAR5 :   Surface temperature (see CTSTAR)
!   ZTS5    :   Model's surface temperature.
!   ZSST5   :   Model's surface temperature over sea (duplicated obs).
!   ZOROG5  :   Model orography at obs points
!   ZLS5    :   Land/sea
!   ZCI5    :   Sea-ice fraction
!   ZAL5    :   Albedo
!   ZU05    :   U-comp ocean current
!   ZV05    :   V-comp ocean current
!   ZU10N5  :   U-comp 10m neutral wind
!   ZV10N5  :   V-comp 10m neutral wind
!   ZCHN5   :   Wave Charnock
REAL(KIND=JPRB), DIMENSION(KDLEN) :: ZT05,ZTSTAR5,ZTS5,ZOROG5,ZLS5,ZCI5,&
 &                                   ZU05,ZV05,ZU10N5,ZV10N5,ZCHN5,&
 &                                   ZAL5,ZSST5,ZTCLS5,ZHUCLS5

!   ZT05_2D    :   Surface temperature (see CTSTAR)
!   ZTSTAR5_2D :   Surface temperature (see CTSTAR)
!   ZOROG5_2D  :   Model orography at obs points
REAL(KIND=JPRB), DIMENSION(KDLEN,KHORIZ) :: &
 & ZT05_2D,ZTSTAR5_2D,ZOROG5_2D


!   Model surface fields, at observation points
REAL(KIND=JPRB), DIMENSION(KDLEN) :: ZSLM5,ZWS5,ZVEG5,ZWL5,ZSN5,ZZ05,ZZ0SCAT

!   ZINTH   :   Interpolation height
!   ZINTHUV :   Interpolation height, for surface wind data
!   ZOROGOB :   Elevation*RG of observation
!   ZNVANG,ZZEANG,ZAZIANG :   Nadir viewing, zenith and azimuth angle
!   ZSOLZENANG,ZSOLAZIANG : Solar zenith and azimuth angle
REAL(KIND=JPRB), DIMENSION(KDLEN) :: ZINTH,ZINTHUV,ZOROGOB
REAL(KIND=JPRB), DIMENSION(KDLEN) :: ZNVANG,ZZEANG, ZAZIANG
REAL(KIND=JPRB), DIMENSION(KDLEN) :: ZSOLZENANG,ZSOLAZIANG,ZSOLHOUR

!   Variables for the surface layer observation operators
REAL(KIND=JPRB), DIMENSION(KDLEN) :: ZCPTGZ5,ZWET5,ZQS5,ZCPTS5,ZRI5,ZBN5,ZBM5,ZBH5
REAL(KIND=JPRB), DIMENSION(KDLEN,2) :: ZES5
REAL(KIND=JPRB), DIMENSION(KDLEN,NFLEVG) :: ZAPHI5

!   Model U/V/T/Q/O3/CLW/CIW/CC/CRW/CSW/CGW at observation points, model levels
REAL(KIND=JPRB), DIMENSION(KDLEN,0:NFLEVG) ::&
 & ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,ZCLWF5,ZCIWF5,ZCCF5,ZCSWF5,ZCRWF5,ZCGWF5,ZQN5  

!   Model U/V/T/Q/O3/CLW/CIW/CC at observation points, model levels
REAL(KIND=JPRB), DIMENSION(KDLEN,0:NFLEVG,KHORIZ) ::&
 & ZUF5_2D,ZVF5_2D,ZTF5_2D,ZQF5_2D,ZO3F5_2D,ZCLWF5_2D,ZCIWF5_2D,ZCCF5_2D,&
 & ZCSWF5_2D, ZCRWF5_2D, ZCGWF5_2D


!   Cp  R and  Kappa, at observation points
REAL(KIND=JPRB), DIMENSION(KDLEN,NFLEVG) :: ZCP5,ZRF5,ZKAP5

!   Cp  R and  Kappa, at observation points
REAL(KIND=JPRB), DIMENSION(KDLEN,NFLEVG,KHORIZ) :: ZCP5_2D,ZRF5_2D,ZKAP5_2D


!   ZVERTP  : Vertical position of obs datum (Pressure/Height/Channel)
!   ZVERTP_L: Vertical position of obs datum (Lower level)
!   ZHOFX   : Observation equivalents, H(x)
REAL(KIND=JPRB), DIMENSION(KDLEN,KDBDY) :: ZVERTP,ZVERTP_L,ZHOFX
INTEGER(KIND=JPIM), DIMENSION(KDLEN)    :: MLNKH2B_SAV

!   ZPRESS_AK : Pressure levels for averaging kernel calculations
!   ZWEIGHT_AK: Weights for averaging kernel calculations
!   ZPRIOR_AK: Prior profile for averaging kernel calculations
REAL(KIND=JPRB), DIMENSION(KDLEN,KDBDY,JPMX_AK) :: ZPRES_AK,ZWEIGHT_AK,ZPRIOR_AK

!   IVNM    : Variable number from observation array
!   IVAR    : Variable number from NVAR table
!   IPNLV   : Pointer to locations in ZHOFX
!   INAK     : Number of averaging kernel elements
INTEGER(KIND=JPIM), DIMENSION(KDLEN,KDBDY) :: IVNMRQ,IVAR,IPNLV,INAK

!   ICDTYP  :  Observation code type from observation array
!   ICMBDY  :  Number of data entries in each report in this KSET
!   ICOUNT  :  Number of data per obs, for call to PP-routines
INTEGER(KIND=JPIM), DIMENSION(KDLEN) :: ICDTYP,ICMBDY,ICOUNT,ICOUNTX,INDXTIME

!   IXVARBC  :  VarBC group index for bias correction
INTEGER(KIND=JPIM), ALLOCATABLE :: IXVARBC(:,:)
REAL(KIND=JPRB), DIMENSION(NHSTBINS) :: ZWPD_MODE

! For ozone bias correction
REAL(KIND=JPRB), ALLOCATABLE :: ZSOLAR(:)
REAL(KIND=JPRB)    :: ZZLON, ZZLAT
INTEGER(KIND=JPIM) :: IOBDATE, IOBTIME, IYEAR, IMONTH, IDAY, IHOUR, ISCAN

!   ZXPP    : Post-processed values, i.e. outputs from obs operators.
!   ZXPRED  : Bias predictor values
!   ZXPPE1  : Post-processed values (extra output 1 from obs operators)
!   ZXPPE2  : Post-processed values (extra output 2 from obs operators)
!   ZXPPE3  : Post-processed values (extra output 3 from obs operators)
!   IXPPE1  : Post-processed values (integer extra output from obs operators)
!   LLXPPE1 : Post-processed values (logical extra output 1 from obs operators)
!   LLXPPE2 : Post-processed values (logical extra output 2 from obs operators)
!   ZXPPB   : Bias estimates for post-processed values
!   ZVPOBS  : List of pressures/heights to interpolate to.
!   ZVPOBS_L: Second list of (lower) pressures/heights to interpolate to.
!   ZPAK    : List of pressures for averaging kernel calculations
!   ZWAK    : List of weights for averaging kernel calculations
!   ZAPWAK  : List of prior values for averaging kernel calculations
REAL(KIND=JPRB), ALLOCATABLE :: ZXPP(:,:,:),ZXPPB(:,:,:),ZXPRED(:,:),ZVPOBS(:,:),ZVPOBS_L(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZXPPC(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZPAK(:,:,:),ZWAK(:,:,:),ZAPAK(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCH4COR(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZCOD(:),ZTAUC(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZXPPE1(:,:,:),ZXPPE2(:,:,:),ZXPPE3(:,:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IXPPE1(:,:,:)
LOGICAL(KIND=JPIM), ALLOCATABLE :: LLXPPE1(:,:,:), LLXPPE2(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZXPP_2D(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZEM5(:,:),ZPS5(:),ZT2M5(:),ZQ2M5(:),ZCTOP5(:),ZCAMT5(:), &
 & ZTQOPP5(:,:,:),ZPRES(:,:),ZTS5_REF(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZTBTOT(:,:)  ! Model total brightness temps
REAL(KIND=JPRB), ALLOCATABLE :: ZOMF(:,:), ZBIAS_WK(:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: ICLOUDFLAG(:)
!   ZORDIF  : Difference (orography model) minus (orography observation)
REAL(KIND=JPRB), ALLOCATABLE :: ZORDIF(:)
REAL(KIND=JPRB)              :: ZOBALT


!   Solar elevation for ozone data
REAL(KIND=JPRB), ALLOCATABLE :: ZSOLEL(:)
!   Sensor for reo3 data
INTEGER(KIND=JPIM), ALLOCATABLE :: IO3SENSOR(:)
!   ZUV  : U and V on model or RTTOV pressure levels
!   ZW   : Weights used for mean layer winds
!   ZTAU : Transmittances on RTTOV pressure levels
REAL(KIND=JPRB), ALLOCATABLE :: ZUV(:,:,:), ZW(:,:), ztau(:,:,:),ZRADCLDLEV(:,:,:)

!   IAK    : number of elements for averaging kernel calculations
INTEGER(KIND=JPIM), ALLOCATABLE :: IAK(:,:)

CHARACTER (LEN = 50)::  CL_HPRIOR_RETRIEVAL
CHARACTER (LEN = 3) ::  CLV(JPMXVAR)
INTEGER(KIND=JPIM) :: IVARL(JPMXVAR)
INTEGER(KIND=JPIM) :: ILEN,IOBSTYPE,ICDTYP_TOVS,IGRP,IMXBDY,&
 & IIVNM,IBODY,INFO(1),IRET,&
 & IMXCOUNT,ISLCT,IBDY,IDF,IDFS,IPOSM,&
 & ICOEF,IDSTA_ACTIVE,ISENSOR,ITSLOT,IRETRTYPE,&
 & ITMO100, itime
INTEGER(KIND=JPIM) :: ISENSOR_GETDB
INTEGER(KIND=JPIM) :: JOBS,JBODY,JVNM,JNLV,JLEV,JGRG, JGHG
INTEGER(KIND=JPIM) :: IJB, JB1, JB2, JB3
REAL(KIND=JPRB) :: Z500HPA,ZLAYL(JPMXVAR,2),ZCMALT,ZSPEED5
REAL(KIND=JPRB) :: ZHLS,ZAZIM,Zsin_azim,Zcos_azim,ZTIME,ZLEFTTM,ZRIGHTM
LOGICAL :: LLRAD1C,LLAPACH,LL_RAIN,LL_MWAVE,LL_MWAVE_DIAG,LL_LAYER,LL_WIND&
 & ,LLEXTLI,LL_SFC_OP,LL_RAD_OP,LL_SPEED,&
 &LL_MEAN_UV,LLREO3, LL_HLS, LL_TCWV, LL_AIREP, LL_AK,LLNO2, LL_MODE, LL_MOPITT, LL_SMOS
LOGICAL, DIMENSION(NGRG) :: LLGRG
LOGICAL, DIMENSION(NGHG) :: LLGHG
REAL(KIND=JPRB) :: ZINFO(1)

! For AIRS radiances
REAL(KIND=JPRB), ALLOCATABLE :: ZOBS(:,:), ZFIT(:,:), ZNNBIAS(:,:)
REAL(KIND=JPRB), DIMENSION(KDLEN) :: ZLAT,ZLON

!BEN Extra variables for aerosol reflectance operator
REAL(KIND=JPRB), DIMENSION(KDLEN) ::    ZSOLANG5, ZVIEANG5, ZDIFFANG5

REAL(KIND=JPRB) :: ZHOOK_HANDLE

TYPE (TYPE_GEMSPROF), DIMENSION(KDLEN,NGEMS) :: YLGEMSPROF5
REAL(KIND=JPRB), TARGET :: Z_MODELPROF(KDLEN,NGEMS,NFLEVG)
REAL(KIND=JPRB), TARGET :: Z_RTTOVPROF(KDLEN,NGEMS,NLSAT)
REAL(KIND=JPRB), ALLOCATABLE :: ZNO25(:,:)
INTEGER(KIND=JPIM) :: JLEN,JGEMS
INTEGER(KIND=JPIM) :: ix, ib, ibin_max, imax_loc, ihist_sum, imax_dist
INTEGER(KIND=JPIM) :: JHIST, I_nhist_avg_r, I_nhist_avg_l, ibinx, ibd, ICC
REAL(KIND=JPRB)    :: zwpd

INTERFACE
#include "getdb.h"
#include "putdb.h"
#include "append_num.h"
#include "abortdb.h"
END INTERFACE

#include "abor1.intfb.h"
!#include "aer_refl_op.intfb.h"
#include "aod_op.intfb.h"
#include "calver.intfb.h"
#include "ch4_tcmr.intfb.h"
#include "co2_tcmr.intfb.h"
#include "cod_op.intfb.h"
#include "diurnal.intfb.h"
#include "gpsro_op.intfb.h"
#include "ghg_ak_op.intfb.h"
#include "grg_ak_op.intfb.h"
#include "gpsro_2dop.intfb.h"
#include "gpszen_delay.intfb.h"
#include "hdepart.intfb.h"
#include "hinth.intfb.h"
#include "hjo.intfb.h"
#include "hqscatt.intfb.h"
#include "hradp.intfb.h"
#include "hradp_ml.intfb.h"
#include "hvnmtlt.intfb.h"
#include "incvfilt.intfb.h"
#include "meanuv_average.intfb.h"
#include "meanuv_weights.intfb.h"
#include "mopitt_ak_op.intfb.h"
#include "mwave_postproc.intfb.h"
#include "nox2no2.intfb.h"
#include "ppobsa.intfb.h"
#include "ppobsac.intfb.h"
#include "ppobsap.intfb.h"
#include "ppobsas.intfb.h"
#include "ppobsaz.intfb.h"
#include "ppobsn.intfb.h"
#include "preint.intfb.h"
#include "preint2d.intfb.h"
#include "preintr.intfb.h"
#include "preints.intfb.h"
#include "qneglim.intfb.h"
#include "rad1cnne.intfb.h"
#include "radtr.intfb.h"
#include "radtr_ml.intfb.h"
#include "pppmer.intfb.h"
#include "reflsim.intfb.h"
#include "dopplsim.intfb.h"
#include "reflsim_2dop.intfb.h"
#include "rtl_hop_1d.intfb.h"
#include "rtl_hop_2d.intfb.h"
#include "writenn.intfb.h"
#include "timdif.intfb.h"
#include "openmp_obs.h"

!     ------------------------------------------------------------------
!*       1.       Introduction
!                 ------------

!*          1.1   Set length (=number of observations)

IF (LHOOK) CALL DR_HOOK('HOP',0,ZHOOK_HANDLE)
!write(nulout,'('' ENTER HOP '')')
!call flush(nulout)
ICC=0
ILEN   = MLNSET(KSET)
ITSLOT = MTSLSET(KSET)
IF(ILEN > KDLEN) THEN
  WRITE(NULERR,*) 'HOP:ILEN > KDLEN ',ILEN,KDLEN,KSET
  CALL ABOR1('HOP:INCONSISTENCY 1')
ENDIF

DO JGEMS=1,NGEMS
  DO JLEN=1,KDLEN
    YLGEMSPROF5(JLEN,JGEMS)%MODELPROF => Z_MODELPROF(JLEN,JGEMS,:)
    YLGEMSPROF5(JLEN,JGEMS)%RTTOVPROF => Z_RTTOVPROF(JLEN,JGEMS,:)
  ENDDO
ENDDO

!      -------------------------------------------------
!*          0.0  inicialization of new fields for radar
!                should be removed when new GOMS_MIX are ready: removed now

!ZCSWF5(:,:)=0.0_JPRB
!ZCGWF5(:,:)=0.0_JPRB
!      -------------------------------------------------

!*          1.2   Observation type and code type for TOVS data

IOBSTYPE = MOD(MTYPOB(KSET),100)
IGRP =MOD(MTYPOB(KSET)/100,100)
LL_MEAN_UV = .FALSE.
LL_MODE = .FALSE.
ISENSOR_GETDB = -1
IF(.NOT. LCANARI .AND. (IOBSTYPE == NSATEM)) THEN
  ICDTYP_TOVS=SATGRP_TABLE(IGRP)%codetype
  ISENSOR=SATGRP_TABLE(IGRP)%sensor
  IF (ICDTYP_TOVS == NSTRO3) ISENSOR=1
  IF (ICDTYP_TOVS == NGTHRB) ISENSOR_GETDB=ISENSOR
  IF (ICDTYP_TOVS == NSSMI ) ISENSOR_GETDB=ISENSOR
  IF (ICDTYP_TOVS == NTCWC ) ISENSOR_GETDB=ISENSOR

  IF (ISENSOR == NSENSOR_MERIS) ISENSOR = 1

ELSEIF(IOBSTYPE == NSATOB)THEN
  LL_MEAN_UV  = TRIM(SATOBGRP_TABLE(IGRP)%obs_oper) == 'Gauss average' &
   & .OR.TRIM(SATOBGRP_TABLE(IGRP)%obs_oper) == 'Boxcar average' &
   & .OR.TRIM(SATOBGRP_TABLE(IGRP)%obs_oper) == 'RT average' &
   & .OR.TRIM(SATOBGRP_TABLE(IGRP)%obs_oper) == 'RT median Gauss average' &
   & .OR.TRIM(SATOBGRP_TABLE(IGRP)%obs_oper) == 'RT median RT average' &
   & .OR.TRIM(SATOBGRP_TABLE(IGRP)%obs_oper) == 'RT max Gauss average' &
   & .OR.TRIM(SATOBGRP_TABLE(IGRP)%obs_oper) == 'RT max RT average' 
  ICDTYP_TOVS = SATOBGRP_TABLE(IGRP)%codetype
  ISENSOR     = 1
ELSEIF(IOBSTYPE == NLIMB)THEN
  ICDTYP_TOVS = Y_LIMBGRP_TABLE(IGRP)%codetype  ! For ODB
  ISENSOR     = 1 
ELSE
  ICDTYP_TOVS=-1
  ISENSOR=1
ENDIF

IF (ISENSOR < 0 .OR. ISENSOR > MXSENSOR) THEN
  WRITE(NULERR,*)'*** ERROR IN HOP: ISENSOR=',ISENSOR,&
   & ' OUT OF RANGE FOR KSET,IOBSTYPE,IGRP,ITSLOT=',&
   & KSET,IOBSTYPE,IGRP,ITSLOT
  CALL ABORTDB('HOP', 'ISENSOR OUT OF RANGE', ISENSOR)
ENDIF

!*          1.3   Initialize ODB

IF (LCANARI) THEN
  CALL GETDB('HOP_CA',0,IRET, INFO, 0, ZINFO, 0, &
   & KSET, -1, -1, -1, -1, -1)
ELSE
  CALL GETDB('HOP', NUPTRA, IRET, INFO, 0, ZINFO, 0, &
   & KSET, ITSLOT, -1, IOBSTYPE, ICDTYP_TOVS, ISENSOR_GETDB)
ENDIF

IF (IRET /= ILEN) THEN
  WRITE(NULERR,'('' ** INCONSISTENT NO. OF OBSERVATIONS FROM GETDB'')')
  WRITE(NULERR,*)' Expecting ',ILEN,', but got =',IRET
  CALL ABORTDB('HOP', 'INCONSISTENCIES FROM GETDB', IRET)
ENDIF

!*          1.4   Other

Z500HPA=50000._JPRB
ZZ0SCAT(1:ILEN)=0.0_JPRB

!*          1.5   Switches

!*          1.5.1 Apach or not?

LLAPACH=LCAPACH .AND. (IOBSTYPE == NTEMP .OR. IOBSTYPE == NPILOT)

!*          1.5.2 Types of TOVS/RAD1C and SSMI/METEOSAT processing
!                 LLRAD1C=Calibrated 'raw' radiances, no 1D-Var
!                 METEOSAT and SSMI radiances are LLRAD1C

LLRAD1C  = ICDTYP_TOVS == NGTHRB
LL_RAIN  = ICDTYP_TOVS == NSSMI .AND. LERAIN
LL_MWAVE = ICDTYP_TOVS == NSSMI .AND. LEMWAVE
LL_MWAVE_DIAG = LL_MWAVE .AND. (LSCREEN .OR. NUPTRA == MUPTRA)

LLREO3    = ICDTYP_TOVS == NSTRO3
LL_AK     = .FALSE.
LL_TCWV  = ICDTYP_TOVS == NTCWC
LL_AIREP = IOBSTYPE    == NAIREP

!*          1.5.3 Are surface/cloud/radiance operators required or not

LL_SFC_OP=ANY(IOBSTYPE == (/NSYNOP,NDRIBU,NTEMP,NPILOT,NPAOB,NSCATT/))
LL_RAD_OP=LLRAD1C .OR. LL_MWAVE

!           1.5.4 SMOS L1C brightness temperatures (L-band)
LL_SMOS = ICDTYP_TOVS == NSMOS .AND. LESMOS

!*          1.5.5 Make connection between CMA-variable number,
!*                NVAR and PPOBSA-variable codes.

CALL HVNMTLT(JPMXVAR,CLV,IVARL,ZLAYL)

!     ------------------------------------------------------------------
!*       2.       Interpolation preparation
!                 -------------------------

!*          2.1   Upper-air, always required

IF (KHORIZ > 1) THEN 

  ! call the 2d planar observation routine (eg, GPSRO or RTLIMB)

  CALL PREINT2D(KDLEN,ILEN,KSET,KHORIZ,ZOROG5_2D,&
   & ZPRESH5_2D,ZPRESF5_2D,ZGEOPH5_2D,ZGEOPF5_2D,&
   & ZPXP5_2D,ZPXPD5_2D,ZPXZ5_2D,ZPXZD5_2D,ZCP5_2D,ZRF5_2D,ZKAP5_2D,&
   & ZUF5_2D,ZVF5_2D,ZTF5_2D,ZQF5_2D,ZO3F5_2D,ZCLWF5_2D,ZCIWF5_2D,ZCCF5_2D,&
   & ZCSWF5_2D, ZCRWF5_2D, ZCGWF5_2D,&
   & ZTSTAR5_2D,ZT05_2D,&
   & ZDELP5_2D,ZRDELP5_2D,ZLNPR5_2D,ZALPH5_2D,ZRTGR5_2D,&
   & ZRPRE5_2D,ZRPP5_2D)  

ELSE

  ! standard vertical profile routine

  CALL PREINT(KDLEN,ILEN,KSET,ZOROG5,&
   & ZPRESH5,ZPRESF5,ZGEOPH5,ZGEOPF5,&
   & ZPXP5,ZPXPD5,ZPXZ5,ZPXZD5,ZCP5,ZRF5,ZKAP5,&
   & ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,ZCLWF5,ZCIWF5,ZCCF5,&
   & ZCSWF5,ZCRWF5,ZCGWF5,&
   & ZTSTAR5,ZT05,&
   & ZDELP5,ZRDELP5,ZLNPR5,ZALPH5,ZRTGR5,&
   & ZRPRE5,ZRPP5,YDGEMSPROF5=YLGEMSPROF5,PAL=ZAL5)  

ENDIF

!*          2.2   Surface

IF(LL_SFC_OP) CALL PREINTS(KDLEN,ILEN,KSET,&
 & ZPRESH5,&
 & ZUF5,ZVF5,ZTF5,ZQF5,ZQN5,&
 & ZTS5,ZSST5,ZU05,ZV05,ZU10N5,ZV10N5,ZCHN5,&
 & ZRF5,&
 & ZWS5,ZVEG5,ZCI5,ZWL5,&
 & ZSN5,ZSLM5,ZZ05,&
 & ZES5,ZCPTGZ5,ZWET5,ZQS5,ZCPTS5,ZAPHI5,&
 & ZRI5,ZBN5,ZBM5,ZBH5,&
 & ZLNPR5,ZALPH5,ZTCLS5,ZHUCLS5)

!       fill in the TSCVX5 arrays (contains the TS at observation points)
IF (.NOT.LSCREEN) THEN
  IF (LTSCV) THEN
    DO JOBS = 1,ILEN
      IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
      TSCVX5(IPOSM)= ZTS5(JOBS)
    ENDDO
  ENDIF
ENDIF

!*          2.3   Radiance, and make q positive.

IF(LL_RAD_OP) CALL PREINTR(KDLEN,ILEN,KSET,ZLS5,ZCI5,ZAL5,ZTS5,ZWS5,ZSN5)
IF(LL_RAD_OP) CALL QNEGLIM(ZQF5(1,1),1,KDLEN,1,NFLEVG,1,ILEN,1,NFLEVG)

!*          2.4 Fill in solar elevation and sensor for ozone/GEMS data 
LL_MOPITT=.false.
IF (LLREO3) THEN
  ALLOCATE(ZSOLEL(KDLEN))
  ALLOCATE(IO3SENSOR(KDLEN))
  DO JOBS = 1,ILEN
    ZSOLEL(JOBS)  = SATHDR(JOBS,MDBRO3SOE)
    IO3SENSOR(JOBS)= SATHDR(JOBS,MDBSSIA)
!   only MOPITT data should be in this set sorted by ecset.F90
    IF (.NOT.LL_MOPITT .AND. (IO3SENSOR(JOBS)==NSENSOR_MOPITT)) LL_MOPITT=.TRUE.
  ENDDO
ENDIF

!     ------------------------------------------------------------------
!*       3.       Look in CMA to see what data is there
!                 -------------------------------------

!*          3.1   Find out number of bodies (ICMBDY)
!                 code-type (ICDTYP), 
!                 obs orography (ZOROGOB) from obs array.

ZLEFTTM = REAL(NTBMAR,JPRB)
ZRIGHTM = REAL(NTFMAR,JPRB)

DO JOBS = 1,ILEN
  ICMBDY(JOBS)=MLNKH2B(JOBS+1)-MLNKH2B(JOBS)
  ICDTYP(JOBS)=ROBHDR(JOBS,MDB_CODETYPE_AT_HDR)
  ZCMALT=ROBHDR(JOBS,MDBALT)
  IF(ZCMALT == RMDI .OR. ABS(ZCMALT) >  10000._JPRB) THEN
    ZOROGOB(JOBS) = RMDI
  ELSE
    ZOROGOB(JOBS) = ZCMALT*RG
  ENDIF

  IF ( LMFSCREEN ) THEN
    IF ( IOBSTYPE == NSATOB .OR. IOBSTYPE == NSCATT ) THEN
      IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
      ROBHDR(JOBS,MDBALT) = YGOMEC(IPOSM)%ECLSMA
    ENDIF
  ENDIF
ENDDO
IMXBDY=MAXVAL(ICMBDY(1:ILEN))

IF(IMXBDY > KDBDY) THEN
  WRITE(NULERR,*) 'HOP:IMXBDY > KDBDY',IMXBDY,KDBDY,KSET
  CALL ABOR1('HOP:INCONSISTENCY 2')
ENDIF

!*          3.2   Extract vertical positions i.e. pressure/height/channel
!                 (ZVERTP) and variable numbers (IVNMRQ). These will
!                 constitude the 'request' for PP-calculations, later.
!                 For reo3 data type extract averaging kernels, which will
!                 be used in the observation operator

IVNMRQ(1:ILEN,1:IMXBDY)=0
IPNLV(1:ILEN,1:IMXBDY)=0
ZVERTP(1:ILEN,1:IMXBDY)=RMDI
ZVERTP_L(1:ILEN,1:IMXBDY)=RMDI
IF(LLREO3) THEN
  INAK(1:ILEN,1:IMXBDY)=0
  ZPRES_AK(1:ILEN,1:IMXBDY,1:JPMX_AK)=RMDI
  ZWEIGHT_AK(1:ILEN,1:IMXBDY,1:JPMX_AK)=0.0_JPRB
  ZPRIOR_AK(1:ILEN,1:IMXBDY,1:JPMX_AK)=0.0_JPRB
ENDIF

BODY_LOOP: DO JBODY=1,IMXBDY
  CMA_LOOP: DO JOBS = 1,ILEN
    IF(JBODY > ICMBDY(JOBS)) CYCLE CMA_LOOP
    IBODY = MLNKH2B(JOBS)+(JBODY-1)

    IF(ROBODY(IBODY,MDBVAR) /= RMDI) &
     & IVNMRQ(JOBS,JBODY) = ROBODY(IBODY,MDBVNM)
    ZVERTP(JOBS,JBODY)  = ROBODY(IBODY,MDBPPP)
    ZVERTP_L(JOBS,JBODY)= ROBODY(IBODY,MDBPRL)

    IF(LLREO3) THEN
      INAK(JOBS,JBODY)  = SATBODY(IBODY,MDB_NAK_AT_REO3_BODY)
      IF(INAK(JOBS,JBODY) > 0 .AND. ABS(INAK(JOBS,JBODY)) < 1000) THEN
        IF (.NOT.LL_AK) LL_AK=.TRUE.
        ZPRES_AK(JOBS,JBODY,1:INAK(JOBS,JBODY))  = &
         & SATBODY(IBODY,MDB_PAK_AT_REO3_BODY(1:INAK(JOBS,JBODY)))
        ZWEIGHT_AK(JOBS,JBODY,1:INAK(JOBS,JBODY))  = &
         & SATBODY(IBODY,MDB_WAK_AT_REO3_BODY(1:INAK(JOBS,JBODY)))
        ZPRIOR_AK(JOBS,JBODY,1:INAK(JOBS,JBODY))  = &
         & SATBODY(IBODY,MDB_APAK_AT_REO3_BODY(1:INAK(JOBS,JBODY)))
      ENDIF
    ENDIF

  ENDDO CMA_LOOP
ENDDO BODY_LOOP

!*          3.3   Nullify invalid requests, by setting IVNMRQ=0

DO JBODY=1,IMXBDY
  DO JOBS = 1,ILEN
    IF(JBODY > ICMBDY(JOBS)) CYCLE

    !*          3.3.0 Surface analysis variables:
    !                 1. SNOW DEPTH (NVNUMB( 25)=71)
    !                 2. SNOWFALL (NVNUMB( 46)=92)
    !                 3. RAIN (NVNUMB( 34)=80)
    !                 4. Normalised soil moisture (NVNUMB(89)=179)

    IF(IVNMRQ(JOBS,JBODY) == NVNUMB(25) .OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(46) .OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(34) .OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(89) ) THEN
      IVNMRQ(JOBS,JBODY)=0
    ENDIF

    !*          3.3.1 Missing vertical position

    IF(ZVERTP(JOBS,JBODY) == RMDI) IVNMRQ(JOBS,JBODY)=0
    IF(ZVERTP(JOBS,JBODY) <= 0. .AND. IVNMRQ(JOBS,JBODY) /= NVNUMB(47)) &
     & IVNMRQ(JOBS,JBODY)=0

    !*          3.3.2 Missing lower vertical position

    IF(IVNMRQ(JOBS,JBODY) == NVNUMB( 4) .OR.&
     !     & IVNMRQ(JOBS,JBODY) == NVNUMB( 6) .OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(58) .OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(68).OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(80).OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(81).OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(82).OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(83).OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(84).OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(89).OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(94).OR.&
     & IVNMRQ(JOBS,JBODY) == NVNUMB(95) ) THEN
      IF(ZVERTP_L(JOBS,JBODY) == RMDI .OR. ZVERTP_L(JOBS,JBODY) <= 0.) &
       & IVNMRQ(JOBS,JBODY)=0
    ENDIF

    !*          3.3.3 Discard thickness data above model's top

    IF(IVNMRQ(JOBS,JBODY) == NVNUMB( 4) .AND.&
     & ZVERTP(JOBS,JBODY) < ZPRESF5(JOBS,1)) IVNMRQ(JOBS,JBODY)=0
  ENDDO

  !*          3.3.5 For SCAT require at least two ambiguous winds
  IF(IOBSTYPE == NSCATT) &
   & WHERE(ICMBDY(1:ILEN) < 4) IVNMRQ(1:ILEN,JBODY)=0

  !*          3.3.6a Only consider TCWV amongst SSMI 1D-Var
  !                 body entries (skipping radiances, for example).

  IF(LL_RAIN .OR. LL_TCWV) &
   & WHERE(IVNMRQ(1:ILEN,JBODY) /= NVNUMB(6))  IVNMRQ(1:ILEN,JBODY)=0  

  !*          3.3.6b Only consider radiances for MWAVE 4D-Var

  IF(LL_MWAVE) &
   & WHERE(IVNMRQ(1:ILEN,JBODY) /= NVNUMB(50)) IVNMRQ(1:ILEN,JBODY)=0  

  !*          3.3.7 To reproduce current operations we must not compute
  !*                R/S T2m departures, not to affect 'Combined flagging'

  IF (LECMWF) THEN
    IF(IOBSTYPE == NTEMP) &
     & WHERE(IVNMRQ(1:ILEN,JBODY) == NVNUMB(10)) IVNMRQ(1:ILEN,JBODY)=0
  ELSEIF(NCONF == 131) THEN
    IF(IOBSTYPE == NTEMP .OR. IOBSTYPE == NPILOT) &
     & WHERE(IVNMRQ(1:ILEN,JBODY) == NVNUMB( 7) .OR. &
     & IVNMRQ(1:ILEN,JBODY) == NVNUMB(10)) IVNMRQ(1:ILEN,JBODY)=0
  ENDIF

  !*          3.3.8 Discard radiances and ambiguous winds, in CANARI

  IF (LCANARI) THEN
    WHERE(IVNMRQ(1:ILEN,JBODY) == NVNUMB(50) .OR.&
     & IVNMRQ(1:ILEN,JBODY) == NVNUMB(61) .OR.&
     & IVNMRQ(1:ILEN,JBODY) == NVNUMB(62)) IVNMRQ(1:ILEN,JBODY)=0
  ENDIF

 !*          3.3.9 Only consider radiances for SMOS

  IF(LL_SMOS) &
   & WHERE(IVNMRQ(1:ILEN,JBODY) /= NVNUMB(97)) IVNMRQ(1:ILEN,JBODY)=0 

ENDDO

!*          3.5   Surface wind data

IF(LL_SFC_OP) CALL HINTH(KDLEN,ILEN,KDBDY,IMXBDY,ICMBDY,IOBSTYPE,&
 & ZAPHI5(1,NFLEVG),ZINTHUV,IVNMRQ)

!*          3.6   Quadratic SCAT cost function requires that one of the two
!                 ambiguous winds is chosen a priori.
!                 Use the one closest to the high resolution trajectory.

IF(IOBSTYPE == NSCATT .AND. LQSCATT .AND. NUPTRA /= NRESUPD) &
 & CALL HQSCATT(KDLEN,ILEN,KDBDY,IMXBDY,ICMBDY,IOBSTYPE,IVNMRQ)

!*          3.7   Discard HIRS channel 20 (no RT coefficients available)

IF(LL_RAD_OP) THEN
  IF (ISENSOR == MSENSOR_HIRS) THEN
    DO JBODY=1,IMXBDY
      WHERE(NINT(ZVERTP(1:ILEN,JBODY)) == 20) IVNMRQ(1:ILEN,JBODY)=0
    ENDDO
  ENDIF
ENDIF

!     ------------------------------------------------------------------
!*       4.       Postprocessing of the model variables to
!                 obtain the equivalent of observed quantities
!                 --------------------------------------------

!*          4.0   Preset to missing

ZHOFX(1:ILEN,:)=RMDI
IVAR(1:ILEN,:) =NMDI

!*          4.1   Loop variables, skip empty ones

VNM_LOOP: DO JVNM=1,JPMXVAR

  IIVNM=NVNUMB(JVNM)
  IF(CLV(JVNM) == 'XXX') CYCLE VNM_LOOP
  IF(CLV(JVNM) == 'V' .OR. CLV(JVNM) == 'V10') CYCLE VNM_LOOP
  IF(IVARL(JVNM) == 0) CYCLE VNM_LOOP
  IF(NOTVAR(IVARL(JVNM),IOBSTYPE) == -1) CYCLE VNM_LOOP
  ! SCAT and SSMI wind speeds require 10m surface-version of wind-speed
  IF((LL_RAIN .OR. IOBSTYPE == NSCATT) &
   & .AND. CLV(JVNM) == 'FF') CLV(JVNM) = 'FFS'

  !*          4.2   Extract positions in the volume JOBS*JBODY which
  !                 contain IIVNM. Positions are in IPNLV and counts in ICOUNT.
  !*          3.2   Extract positions in the volume JOBS*JBDY which

  ICOUNT(:)=0
  DO JBODY=1,IMXBDY
    DO JOBS = 1,ILEN
      IF(IVNMRQ(JOBS,JBODY) == IIVNM) THEN
        ICOUNT(JOBS)=ICOUNT(JOBS)+1
        IPNLV(JOBS,ICOUNT(JOBS))=JBODY
        IVAR(JOBS,JBODY)=IVARL(JVNM)
      ENDIF
    ENDDO
  ENDDO
  IMXCOUNT=MAXVAL(ICOUNT(1:ILEN))
  IF(IMXCOUNT == 0 ) CYCLE VNM_LOOP

  !*          4.3.1 Are GHG and GRG operators required or not
  LLGHG=.false.
  DO JGHG=1,NGHG
    LLGHG(JGHG)=CLV(JVNM) == TRIM(YGHG_NL(JGHG)%cname)
  ENDDO

  LLGRG=.false.
  DO JGRG=1,NGRG
    LLGRG(JGRG)=CLV(JVNM) == TRIM(YGRG_NL(JGRG)%cname)
  ENDDO

  LLNO2=CLV(JVNM) == 'NOX'
  IF(LLNO2) THEN
    ALLOCATE(ZNO25(KDLEN,0:NFLEVG))
  ENDIF

  !*          4.4   Layer quantities and winds need special treatment

  LL_LAYER=.NOT.(ZLAYL(JVNM,1) == 1.0_JPRB .AND. ZLAYL(JVNM,2) == 0.0_JPRB) &
   & .AND. (.NOT.LL_AK)
  LL_WIND =CLV(JVNM) == 'U'   .OR. CLV(JVNM) == 'U10' .OR. CLV(JVNM) == 'AUV'
  LL_SPEED=(CLV(JVNM) == 'FF' .OR. CLV(JVNM) == 'FFS')
  LL_HLS  =(CLV(JVNM) == 'HLS')

  !*          4.5   Now that we know the number of data, of this variable,
  !                 allocate the array to hold PP-values (ZXPP) and
  !                 contracted lists of vertical positions (ZVPOBS, ZVPOBS_L)
  !                 TOVS may need emissivities (ZEM5) from the obs array.

  IDF=2
  ! adding allocation for statistic inversion of reflectivity
  ALLOCATE(ZXPP_2D(KDLEN,IMXCOUNT,IDF,KHORIZ))
  ALLOCATE(ZXPP(KDLEN,IMXCOUNT,IDF))
  ALLOCATE(ZVPOBS(KDLEN,IMXCOUNT))
  ALLOCATE(ZCH4COR(KDLEN,IMXCOUNT))
  ZVPOBS(:,:)=RMDI
  ZVPOBS(1:ILEN,:)=Z500HPA
  ALLOCATE(ZORDIF(KDLEN))
  ALLOCATE(ZVPOBS_L(KDLEN,IMXCOUNT))
  ZVPOBS_L(:,:)=RMDI
  ZVPOBS_L(1:ILEN,:)=Z500HPA
  ALLOCATE(ZTAUC(KDLEN,NFLEVG))
  IF(LL_RAD_OP) THEN
    ALLOCATE(ZEM5(KDLEN,IMXCOUNT))
    ZEM5(1:ILEN,:)=1.0_JPRB
  ENDIF
  IF(LLREO3) THEN
    ALLOCATE(IAK(KDLEN,IMXCOUNT))
    IAK(:,:)=0.0_JPRB
    ALLOCATE(ZPAK(KDLEN,IMXCOUNT,JPMX_AK))
    ZPAK(:,:,:)=Z500HPA
    ALLOCATE(ZWAK(KDLEN,IMXCOUNT,JPMX_AK))
    ZWAK(:,:,:)=0.0_JPRB
    ALLOCATE(ZAPAK(KDLEN,IMXCOUNT,JPMX_AK))
    ZAPAK(:,:,:)=0.0_JPRB
  ENDIF

  IF(ANY(LLGRG)) THEN
    ALLOCATE(ZCOD(KDLEN))
  ENDIF
  IF (LMFSCREEN) THEN
    ALLOCATE(ZXPPE1(KDLEN,IMXCOUNT,IDF))
    ALLOCATE(ZXPPE2(KDLEN,IMXCOUNT,IDF))
    ALLOCATE(ZXPPE3(KDLEN,IMXCOUNT,IDF))
    ALLOCATE(IXPPE1(KDLEN,IMXCOUNT,IDF))
    ALLOCATE(LLXPPE1(KDLEN,IMXCOUNT,IDF))
    ALLOCATE(LLXPPE2(KDLEN,IMXCOUNT,IDF))
  ENDIF

  DO JNLV=1,IMXCOUNT
    DO JOBS=1,ILEN
      IF(JNLV <= ICOUNT(JOBS)) THEN
        IBDY=IPNLV(JOBS,JNLV)
        ZVPOBS(JOBS,JNLV)=ZVERTP(JOBS,IBDY)
        ZVPOBS_L(JOBS,JNLV)=ZVERTP_L(JOBS,IBDY)
        IF(LLREO3) THEN
          IAK(JOBS,JNLV)=INAK(JOBS,IBDY)
          IF(IAK(JOBS,JNLV) > 0) THEN
            ZPAK(JOBS,JNLV,1:IAK(JOBS,JNLV)) = &
             &   ZPRES_AK(JOBS,IBDY,1:INAK(JOBS,IBDY))
            ZWAK(JOBS,JNLV,1:IAK(JOBS,JNLV)) = &
             &   ZWEIGHT_AK(JOBS,IBDY,1:INAK(JOBS,IBDY))
            ZAPAK(JOBS,JNLV,1:IAK(JOBS,JNLV)) = &
             &   ZPRIOR_AK(JOBS,IBDY,1:INAK(JOBS,IBDY))
          ENDIF
        ENDIF
      ENDIF
    ENDDO
  ENDDO

  IF(LLRAD1C) THEN
    DO JNLV=1,IMXCOUNT
      DO IJB = 1, ILEN
        IF( JNLV <= ICOUNT(IJB) )THEN
          JB1 = MLNKH2B(IJB)
          JB2 = IPNLV(IJB,JNLV)
          JB3 = JB1 + JB2 - 1
          ZEM5(IJB,JNLV)=ROBODY(JB3,MDBPER)
          !* to avoid mismatch between preset emissivities and land/sea flag
          IF( ZEM5(IJB,JNLV) >  1.00_JPRB .AND. &
           &  ZLS5(IJB)      >= 0.01_JPRB         )THEN
            ZEM5(IJB,JNLV)  =   1.00_JPRB
            ROBODY(JB3,MDBFOE) = 999.00_JPRB
          ENDIF
        ENDIF
      ENDDO
    ENDDO
  ENDIF

  !*          4.5   Call the appropriate PPOBSA routine

  !*          4.5.1 Upper air operators

  IF( (CLV(JVNM) == 'U' .AND. .NOT.LL_MEAN_UV)  .OR.                       &
   & CLV(JVNM) == 'HLS'.OR.&
   & CLV(JVNM) == 'T'  .OR. CLV(JVNM) == 'H'  .OR. CLV(JVNM) == 'FF' .OR.&
   & CLV(JVNM) == 'LH' .OR. CLV(JVNM) == 'PWC'.OR. CLV(JVNM) == 'Q'  .OR.&
   & CLV(JVNM) == 'Z'  .OR. CLV(JVNM) == 'DZ' .OR. CLV(JVNM) == 'TO3'.OR.&
   & CLV(JVNM) == 'CLW'.OR. CLV(JVNM) == 'CIW'.OR. CLV(JVNM) == 'CC' ) THEN

    ! fill 1D-arrays with GOM_2D in radar case for relative humidity
    IF (CLV(JVNM) == 'H' .AND. IOBSTYPE == NRADAR .AND. KHORIZ > 1 ) THEN
      ZOROG5(:) = ZOROG5_2D(:,1)
      ZPRESH5(:,:) = ZPRESH5_2D(:,:,1)
      ZPRESF5(:,:) = ZPRESF5_2D(:,:,1)
      ZGEOPH5(:,:) = ZGEOPH5_2D(:,:,1)
      ZGEOPF5(:,:) = ZGEOPF5_2D(:,:,1)
      ZPXP5(:,:,:) = ZPXP5_2D(:,:,:,1)
      ZPXPD5(:,:,:) = ZPXPD5_2D(:,:,:,1)
      ZPXZ5(:,:,:) = ZPXZ5_2D(:,:,:,1)
      ZPXZD5(:,:,:) = ZPXZD5_2D(:,:,:,1)
      ZUF5(:,:) = ZUF5_2D(:,:,1)
      ZVF5(:,:) = ZVF5_2D(:,:,1)
      ZTF5(:,:) = ZTF5_2D(:,:,1)
      ZQF5(:,:) = ZQF5_2D(:,:,1)
      ZO3F5(:,:) = ZO3F5_2D(:,:,1)
      ZCLWF5(:,:) = ZCLWF5_2D(:,:,1)
      ZCIWF5(:,:) = ZCIWF5_2D(:,:,1)
      ZCCF5(:,:) = ZCCF5_2D(:,:,1)
      ZTSTAR5(:) = ZTSTAR5_2D(:,1)
      ZT05(:) = ZT05_2D(:,1)
      ZDELP5(:,:) =  ZDELP5_2D(:,:,1)
      ZRDELP5(:,:) = ZRDELP5_2D(:,:,1)
      ZLNPR5(:,:) = ZLNPR5_2D(:,:,1)
      ZALPH5(:,:) = ZALPH5_2D(:,:,1)
      ZRTGR5(:,:) = ZRTGR5_2D(:,:,1)
    ENDIF
    ! end of filling the 1D arrays fields to compute fg of relative humidity

    IF (.NOT.LLAPACH) THEN
      CALL PPOBSA(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),IOBSTYPE,ICOUNT,ZVPOBS,ZOROG5,&
       & ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,&
       & ZCLWF5,ZCIWF5,ZCCF5,ZTSTAR5,ZT05,ZLNPR5,ZALPH5,IDF,ZXPP)

      !*          4.5.1b Upper air operators, lower level

      IF(LL_LAYER) THEN
        CALL PPOBSA(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),IOBSTYPE,&
         & ICOUNT,ZVPOBS_L,ZOROG5,&
         & ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,&
         & ZCLWF5,ZCIWF5,ZCCF5,ZTSTAR5,ZT05,ZLNPR5,ZALPH5,&
         & IDF-1,ZXPP(1,1,2))
      ENDIF
    ELSE
      CALL PPOBSAP(KDLEN,KDBDY,ILEN,IMXCOUNT,IPNLV,CLV(JVNM),&
       & ICOUNT,ZVPOBS,ZTF5,ZQF5,ZCLWF5,ZCIWF5,ZUF5,ZVF5,ZTS5,ZPRESH5,ZOROGOB,ZOROG5,&
       & IDF,ZXPP)
    ENDIF

    !*          4.5.1c Variational bias correction for O3

    IF (LVARBC .AND. CLV(JVNM) == 'TO3') THEN
      ALLOCATE(ZXPPB(KDLEN,IMXCOUNT,1),ZXPRED(0:JPMXNPRED,ILEN), &
       & ZSOLAR(ILEN), &
       & ZOMF(KDLEN,IMXCOUNT),IXVARBC(KDLEN,IMXCOUNT))
      ZOMF(:,:)=RMDI
      DO JNLV=1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV > ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          IF (ZXPP(JOBS,JNLV,1) /= RMDI) &
           & ZOMF(JOBS,JNLV) = ROBODY(IBODY,MDBVAR) &         
           &          - (ZLAYL(JVNM,1)*ZXPP(JOBS,JNLV,1)+ZLAYL(JVNM,2)*ZXPP(JOBS,JNLV,2))
          IXVARBC(JOBS,JNLV) = ROBODY(IBODY,MDB_VARBC_IX_AT_BODY)
        ENDDO
      ENDDO
      DO JOBS= 1, ILEN
        ZZLAT   = ROBHDR(JOBS,MDBLAT) * RDEGREES
        ZZLON   = ROBHDR(JOBS,MDBLON) * RDEGREES
        IOBDATE = ROBHDR(JOBS,MDBDAT)
        IYEAR   = IOBDATE/10000
        IMONTH  = IOBDATE/100 - IYEAR*100
        IDAY    = IOBDATE - (IYEAR*10000+IMONTH*100)
        IOBTIME = ROBHDR(JOBS,MDBETM)
        IHOUR   = IOBTIME/10000
        CALL DIURNAL ( IMONTH, IDAY, IHOUR, ZZLAT, ZZLON, ZSOLAR(JOBS))
      ENDDO

      CALL VARBC_PRED_TO3 (ILEN, IMXCOUNT, ICOUNT(1:ILEN), IXVARBC(1:ILEN,:), &
        & ZSOLAR, JPMXNPRED, ZXPRED, LSTATS=LIFSTRAJ)

      CALL VARBC_BIAS (IXVARBC, IMXCOUNT, ICOUNT, KDLEN, ILEN, &
       & JPMXNPRED, ZXPRED, ZOMF, ZXPPB(:,:,1))

      DO JNLV = 1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV>ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          ROBODY(IBODY,MDBTORB) = ZXPPB(JOBS,JNLV,1)
        ENDDO
      ENDDO
      DEALLOCATE(ZXPPB, ZXPRED, ZSOLAR, ZOMF, IXVARBC)
    ENDIF


    !*          4.5.1d Variational bias correction for TCWV

    IF (LVARBC .AND. LL_TCWV .AND. CLV(JVNM) == 'PWC') THEN
      DO JOBS = 1,ILEN
        ZNVANG     (JOBS) = RMDI
        ZSOLZENANG (JOBS) = RMDI
        ZSOLHOUR   (JOBS) = 0.0_JPRB ! check this if ever solar hour is required
      ENDDO

      ALLOCATE(ZXPPB(KDLEN,IMXCOUNT,1),ZXPRED(0:JPMXNPRED,ILEN), &
       & ZOMF(KDLEN,IMXCOUNT),IXVARBC(KDLEN,IMXCOUNT))

      ZOMF(:,:)=RMDI
      DO JNLV=1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV > ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          IF (ZXPP(JOBS,JNLV,1) /= RMDI) &
           & ZOMF(JOBS,JNLV) = ROBODY(IBODY,MDBVAR) &         
           &          - (ZLAYL(JVNM,1)*ZXPP(JOBS,JNLV,1)+ZLAYL(JVNM,2)*ZXPP(JOBS,JNLV,2))
          IXVARBC(JOBS,JNLV) = ROBODY(IBODY,MDB_VARBC_IX_AT_BODY)
        ENDDO
      ENDDO

      CALL VARBC_PRED_RAD (KDLEN, ILEN, IMXCOUNT, ICOUNT(1:ILEN), IXVARBC(1:ILEN,:), &
       & ZPRESF5, ZGEOPF5(:,1:NFLEVG), ZQF5(:,1:NFLEVG), ZLS5(:), ZCI5(:), ZTF5(:,NFLEVG) , &
       & ZUF5(:,NFLEVG), ZVF5(:,NFLEVG), ZNVANG, ZSOLZENANG, ZSOLHOUR, &
       & NFLEVG, JPMXNPRED, ZXPRED, LSTATS=LIFSTRAJ)

      CALL VARBC_BIAS (IXVARBC, IMXCOUNT, ICOUNT, KDLEN, ILEN, &
       & JPMXNPRED, ZXPRED, ZOMF, ZXPPB(:,:,1))

      DO JNLV = 1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV>ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          ROBODY(IBODY,MDBTORB) = ZXPPB(JOBS,JNLV,1)
        ENDDO
      ENDDO

      DEALLOCATE(ZXPPB, ZXPRED, ZOMF, IXVARBC)
    ENDIF

    !*          4.5.1e Variational bias correction for AIREP T

    IF (LBC_AIREP .AND. LVARBC .AND. LL_AIREP .AND. CLV(JVNM) == 'T') THEN

      ALLOCATE(ZXPPB(KDLEN,IMXCOUNT,1),ZXPRED(0:JPMXNPRED,ILEN), &
       & ZOMF(KDLEN,IMXCOUNT),IXVARBC(KDLEN,IMXCOUNT))

      ZOMF(:,:)=RMDI
      DO JNLV=1,IMXCOUNT
       
        DO JOBS = 1,ILEN
          IF (JNLV > ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          IF (ZXPP(JOBS,JNLV,1) /= RMDI) THEN
            ZOMF(JOBS,JNLV) = ROBODY(IBODY,MDBVAR) - ZXPP(JOBS,JNLV,1)
          endif
          IXVARBC(JOBS,JNLV) = ROBODY(IBODY,MDB_VARBC_IX_AT_BODY)
        ENDDO
      ENDDO

      CALL VARBC_PRED_AIREP (ILEN, IMXCOUNT, ICOUNT(1:ILEN), IXVARBC(1:ILEN,:), &
        & JPMXNPRED, ZXPRED, LSTATS=LIFSTRAJ)

      CALL VARBC_BIAS (IXVARBC, IMXCOUNT, ICOUNT, KDLEN, ILEN, &
       & JPMXNPRED, ZXPRED, ZOMF, ZXPPB(:,:,1))
 
      DO JNLV = 1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV>ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          ROBODY(IBODY,MDBTORB) = ZXPPB(JOBS,JNLV,1)
        ENDDO
      ENDDO
 
      DEALLOCATE(ZXPPB, ZXPRED, ZOMF, IXVARBC)
    ENDIF

    !*          4.5.2 Surface operators

  ELSEIF(CLV(JVNM) == 'U10' .OR. CLV(JVNM) == 'T2' .OR. &
   &   CLV(JVNM) == 'H2'  .OR. CLV(JVNM) == 'AUV'.OR. &
   &   CLV(JVNM) == 'FFS' .OR. CLV(JVNM) == 'TS' ) THEN  
    IF(LL_WIND .OR. LL_SPEED) THEN
      ZINTH(1:ILEN)=ZINTHUV(1:ILEN)
    ELSE
      ZINTH(1:ILEN)=2.0_JPRB*RG
    ENDIF
    IF(.NOT.LL_SFC_OP) CALL ABOR1('HOP: PPOBSAS requires LL_SFC_OP=.T.')
    IF(.NOT.LCACHMT) THEN
      CALL PPOBSAS(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),ZINTH,IOBSTYPE,ICOUNT,ZZ0SCAT,&
       & ZPRESF5,ZUF5,ZVF5,ZTF5,ZQN5,ZU05,ZV05,&
       & ZCPTGZ5,ZCPTS5,ZAPHI5,ZES5,ZRI5,ZBN5,ZBM5,ZBH5,IDF,ZXPP)
    ELSE
      CALL PPOBSAC(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),ZINTH,IOBSTYPE,ICOUNT,&
       & ZOROGOB,ZOROG5,ZPRESH5,ZPRESF5,ZGEOPH5,ZGEOPF5,&
       & ZCP5,ZRF5,ZUF5,ZVF5,ZTF5,ZQF5,&
       & ZTS5,ZSST5,ZWS5,ZVEG5,ZSN5,ZSLM5,ZZ05,IDF,ZTCLS5,ZHUCLS5)
    ENDIF

    !*          4.5.2.2 Surface operators CANARI

  ELSEIF(CLV(JVNM) == 'RR' .OR. CLV(JVNM) == 'SN' ) THEN

    CALL PPOBSN(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),ICOUNT,&
     & ZOROGOB,ZOROG5,ZPRESF5,ZTF5,ZTS5,IDF,ZXPP)

    !*          4.5.3 Surface pressure, with height as vertical coordinate
    !                 ZVPOBS is height

  ELSEIF(CLV(JVNM) == 'PS') THEN
    LLEXTLI=.FALSE.
    ISLCT=1
    CALL PPOBSAZ(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),IOBSTYPE,ICOUNT,ZVPOBS,&
     & ZOROG5,ISLCT,LLEXTLI,&
     & ZPXZ5,ZPXZD5,ZGEOPH5,ZGEOPF5,ZPRESH5,ZPRESF5,ZTSTAR5,IDF,ZXPP)

    !*          4.5.5 Radiances

  ELSEIF(CLV(JVNM) == 'RAD') THEN

    !*          4.5.5a Allocates and fill lat/lon, angles

    IF(.NOT.LL_RAD_OP) CALL ABOR1('HOP: RADTR requires LL_RAD_OP=.T.')
    ALLOCATE(ZPS5(KDLEN))
    ALLOCATE(ZT2M5(KDLEN))
    ALLOCATE(ZQ2M5(KDLEN))
    ALLOCATE(ZCTOP5(KDLEN))
    ALLOCATE(ZCAMT5(KDLEN))
    ALLOCATE(ZTS5_REF(KDLEN))
    ALLOCATE(ZTBTOT(KDLEN,IMXCOUNT))
    ZTS5_REF(:)=ZTS5(:)
    DO JOBS = 1,ILEN
      ZLAT(JOBS) = ROBHDR(JOBS,MDBLAT) * RDEGREES 
      ZLON(JOBS) = ROBHDR(JOBS,MDBLON) * RDEGREES
      IF (ZLON(JOBS)< 0.0_JPRB) ZLON(JOBS) =ZLON(JOBS) + 360.0_JPRB
      IF(.NOT. LL_MWAVE)THEN
        ZAZIANG(JOBS)    = SATHDR(JOBS,MDB_AZIMUTH_AT_SAT)
        ZSOLZENANG(JOBS) = SATHDR(JOBS,MDB_SOLAR_ZENITH_AT_SAT)
        ZSOLAZIANG(JOBS) = SATHDR(JOBS,MDB_SOLAR_AZIMUTH_AT_SAT)
      ELSE
        ZSOLZENANG(JOBS) = RMDI ! initialise for VarBC
      ENDIF
      IOBTIME = ROBHDR(JOBS,MDBETM)
      ZSOLHOUR(JOBS) = MOD(IOBTIME/10000 + MOD(IOBTIME/100, 100)/60.0_JPRB &
       &             + ZLON(JOBS)/15.0_JPRB + 24.0_JPRB, 24.0_JPRB) 
    ENDDO

    !*          4.5.5b Preparations if RTTOV-internal interpolation is used
    IF( LRTTOV_INTERPOL )THEN
      CALL HRADP_ML(2,KDLEN,ILEN,KSET,ZNVANG,ZZEANG,ICOEF,&
       & ZPRESH5,ZTF5,ZQF5,&
       & ZLS5,ZCI5,ZPS5,ZTS5,ZT2M5,ZQ2M5,ZCTOP5,ZCAMT5)  
      ALLOCATE(ZTAU(KDLEN,IMXCOUNT,NFLEVG+1))
      ALLOCATE(ZRADCLDLEV(KDLEN,IMXCOUNT,NFLEVG+1))
    ELSE

      !*          4.5.5c Interpolate T, q and O3 to fix pressure levels

      ICOUNTX(1:ILEN)=NLSAT

      IDFS=NPNAV+2
      ALLOCATE(ZPRES(KDLEN,NLSAT))
      ALLOCATE(ZTQOPP5(KDLEN,NLSAT,IDFS))
      ALLOCATE(ZTAU(KDLEN,IMXCOUNT,NLSAT))
      ALLOCATE(ZRADCLDLEV(KDLEN,IMXCOUNT,NLSAT))

      !     List of fix pressure levels to interpolate to
      DO JLEV=1,NLSAT
        ZPRES(1:ILEN,JLEV) = RADPRE(JLEV)
      ENDDO

      CALL PPOBSA(KDLEN,ILEN,NLSAT,'TV1',IOBSTYPE,ICOUNTX,ZPRES,ZOROG5,&
       & ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,&
       & ZCLWF5,ZCIWF5,ZCCF5,ZTSTAR5,ZT05,ZLNPR5,ZALPH5,&
       & IDFS,ZTQOPP5,YDGEMSPROF5=YLGEMSPROF5)

      !*          4.5.5d Preparations if IFS-based interpolation is used

      CALL HRADP(2,KDLEN,ILEN,IDFS,KSET,ZOROG5,ZOROGOB,&
       & ZNVANG,ZZEANG,ICOEF,&
       & ZPRESH5,ZPRESF5,ZTF5,ZTSTAR5,ZT05,ZTQOPP5,&
       & ZLS5,ZCI5,ZPS5,ZTS5,ZT2M5,ZQ2M5,ZCTOP5,ZCAMT5)

    ENDIF

    IF ((ISENSOR == MSENSOR_AIRS).AND.LCO2_DIAG_AIRS) THEN
      ZCTOP5(:)=500.0_JPRB
      ZCAMT5(:)=0.0_JPRB
      DO JOBS=1, ILEN
        ZCTOP5(JOBS)=SATHDR(JOBS,MDB1BCP15A)
        ZCAMT5(JOBS)=SATHDR(JOBS,MDB1BCP16A)
      ENDDO
    ENDIF

    !*          4.5.5d Call radiative transfer model, RTTOV

    IF(LL_MWAVE)THEN
      DO JNLV=1,IMXCOUNT
        DO JOBS=1,ILEN
          IF (JNLV>ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS)+IPNLV(JOBS,JNLV)-1
          ! If observation is OK, and also, in screening or final traj, 
          ! just if no serious errors, calculate a departure.
          IF( (LL_MWAVE_DIAG .AND. .NOT. MWAVE_SERIOUS(INT(SATBODY(IBODY,MDB_DATUM_TBFLAG),JPIM))) &
            & .OR. MWAVE_OK(INT(SATBODY(IBODY,MDB_DATUM_TBFLAG),JPIM)))THEN
            ZXPP(JOBS,JNLV,1)=SATBODY(IBODY,MDB_TBVALUE)
          ELSE
            ZXPP(JOBS,JNLV,1)=RMDI
          ENDIF
        ENDDO
      ENDDO
    ELSE
      IF( LRTTOV_INTERPOL )THEN
        ZTBTOT=RMDI
        CALL RADTR_ML(ICOEF,ISENSOR,KDLEN,ILEN,IMXCOUNT,ICOUNT,ZVPOBS,&
         & SATGRP_TABLE(IGRP)%cld_rtcalc_assim,&
         & ZZEANG,ZAZIANG,ZSOLZENANG,ZSOLAZIANG,ZLAT,ZOROG5,&
         & ZPRESF5,ZTF5,ZQF5,ZO3F5,ZCLWF5,ZCIWF5,ZCCF5,YLGEMSPROF5,&
         & ZPS5,ZTS5,ZLS5,ZT2M5,ZQ2M5,ZCTOP5,ZCAMT5 ,&
         & ZUF5(1,NFLEVG),ZVF5(1,NFLEVG),ZEM5,&
         & ZXPP, ZTBTOT, ZTAU,ZRADCLDLEV)
        IF(SATGRP_TABLE(IGRP)%cld_rtcalc_assim) ZXPP(:,:,1) = ZTBTOT
        ZXPP(:,:,1) = ZTBTOT
      ELSE
        CALL RADTR(ICOEF,ISENSOR,KDLEN,ILEN,IDFS,IMXCOUNT,ICOUNT,ZVPOBS,&
         & .FALSE.,&
         & ZZEANG,ZAZIANG,ZSOLZENANG,ZSOLAZIANG,ZLAT,ZOROG5,&
         & ZTQOPP5,YLGEMSPROF5,ZPS5,ZTS5,ZLS5,ZT2M5,ZQ2M5,ZCTOP5,ZCAMT5 ,&
         & ZUF5(1,NFLEVG),ZVF5(1,NFLEVG),ZEM5,ZXPP,ZTBTOT,ZTAU,ZRADCLDLEV)
      ENDIF
    ENDIF

    !*          4.5.5e AIRS 1DVar and NN Bias Correction

    IF((LNNWRI .OR. LNNHC) .AND. ISENSOR == MSENSOR_AIRS) THEN
      ALLOCATE(ZOBS(ILEN,IMXCOUNT))
      ALLOCATE(ZFIT(ILEN,IMXCOUNT))
      ALLOCATE(ZNNBIAS(ILEN,IMXCOUNT))
      ZOBS(:,:)=RMDI
      ZFIT(:,:)=RMDI

      DO JOBS=1,ILEN
        DO JNLV=1,ICOUNT(JOBS)
          ZOBS(JOBS,JNLV)=ROBODY(MLNKH2B(JOBS)+IPNLV(JOBS,JNLV)-1,MDBVAR)
          IF( LSCREEN ) THEN
            ZFIT(JOBS,JNLV)=ZXPP(JOBS,JNLV,1)
          ELSE
            ZFIT(JOBS,JNLV)=ZXPP(JOBS,JNLV,1) - BETA * &
             & (ZOBS(JOBS,JNLV)-ZXPP(JOBS,JNLV,1) &
             &  -ROBODY(MLNKH2B(JOBS)+IPNLV(JOBS,JNLV)-1,MDBOMF))
          ENDIF
        ENDDO
      ENDDO

      IF (LNNWRI)  CALL WRITENN(ILEN,IMXCOUNT,ZVPOBS(1:ILEN,:),&
       & ZOBS(1:ILEN,:),ZFIT(1:ILEN,:),ZPS5(1:ILEN),ZTS5(1:ILEN),&
       & ZLS5(1:ILEN),ZZEANG(1:ILEN),ZLAT(1:ILEN),ZLON(1:ILEN),&
       & ZEM5(1:ILEN,:))

      IF (LNNHC) THEN
        CALL RAD1CNNE(ILEN,IMXCOUNT,ZVPOBS(1:ILEN,:),ZNNBIAS)

        DO JNLV=1,IMXCOUNT
          WHERE(JNLV <= ICOUNT(1:ILEN)) &
           & ROBODY(MLNKH2B(1:ILEN)+IPNLV(1:ILEN,JNLV)-1,MDBTORB)= ZNNBIAS(1:ILEN,JNLV)
        ENDDO

      ENDIF

      DEALLOCATE(ZOBS,ZFIT,ZNNBIAS)
    ENDIF

    !*          4.5.5f Variational bias correction

    IF (LVARBC) THEN
      ALLOCATE(ZXPPB(KDLEN,IMXCOUNT,1),ZXPRED(0:JPMXNPRED,ILEN), &
       & ZOMF(KDLEN,IMXCOUNT),IXVARBC(KDLEN,IMXCOUNT))
      ALLOCATE(ZXPPC(KDLEN,IMXCOUNT,1))
      ZOMF(:,:)=RMDI
      ZXPPC(:,:,1)=RMDI
      DO JNLV=1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV > ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          IF (ZXPP(JOBS,JNLV,1) /= RMDI) THEN
            ZOMF(JOBS,JNLV) = ROBODY(IBODY,MDBVAR) - ZXPP(JOBS,JNLV,1)
            ZXPPC(JOBS,JNLV,1) = ZXPP(JOBS,JNLV,1)
          ENDIF
          IXVARBC(JOBS,JNLV) = ROBODY(IBODY,MDB_VARBC_IX_AT_BODY)
          ix = IXVARBC(JOBS,JNLV)
          IF (ix /= 0) LL_MODE = (LL_MODE .OR. YVARBC(ix)%llmode)
        ENDDO
      ENDDO

      CALL VARBC_PRED_RAD (KDLEN, ILEN, IMXCOUNT, ICOUNT(1:ILEN), IXVARBC(1:ILEN,:), &
       & ZPRESF5, ZGEOPF5(:,1:NFLEVG), ZQF5(:,1:NFLEVG), ZLS5(:), ZCI5(:), ZTS5_REF(:), &
       & ZUF5(:,NFLEVG), ZVF5(:,NFLEVG), ZNVANG, ZSOLZENANG, ZSOLHOUR, &
       & NFLEVG, JPMXNPRED, ZXPRED, LSTATS=LIFSTRAJ)

      CALL VARBC_BIAS (IXVARBC, IMXCOUNT, ICOUNT, KDLEN, ILEN, &
       & JPMXNPRED, ZXPRED, ZOMF, ZXPPB(:,:,1))

      DO JNLV = 1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV>ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          ROBODY(IBODY,MDBTORB) = ZXPPB(JOBS,JNLV,1)
        ENDDO
      ENDDO

      !*         4.5.5g Rainy radiance diagnostic postprocessing (after bias estimate)

      IF(LL_MWAVE .AND. LL_MWAVE_DIAG)THEN
        ALLOCATE(ICLOUDFLAG(KDLEN))
        CALL MWAVE_POSTPROC(KDLEN, IMXCOUNT, ILEN, ICOUNT, IGRP, &
         & ZVPOBS, ZOMF, ZXPP(:,:,1), ZXPPB(:,:,1), ICLOUDFLAG) 
        DO JOBS=1,ILEN
          IF (LSCREEN)          SATHDR(JOBS,MDB_REPORT_TBCLOUD) = ICLOUDFLAG(JOBS)
          IF (NUPTRA == MUPTRA) SATHDR(JOBS,MDB_REPORT_TBCLOUD) = &
           & SATHDR(JOBS,MDB_REPORT_TBCLOUD) + ISHFT(ICLOUDFLAG(JOBS), CLDBIT_SHIFT) 
        ENDDO
        DEALLOCATE(ICLOUDFLAG)
      ENDIF

      IF ( .NOT. LL_MODE ) DEALLOCATE(ZXPPC)
      DEALLOCATE(ZXPPB, ZXPRED, ZOMF, IXVARBC)
    ENDIF

    !*          4.5.5h Deallocate radiance arrays

    DEALLOCATE(ZPS5,ZT2M5,ZQ2M5,ZCTOP5,ZCAMT5,ZTAU,ZRADCLDLEV,ZTS5_REF,ZTBTOT )
    IF( .NOT. LRTTOV_INTERPOL )THEN
      DEALLOCATE(ZPRES,ZTQOPP5)
    ENDIF

    !*          4.5.5i SMOS Level-1C brightness temperatures
  ELSEIF(CLV(JVNM) == 'SFR' .AND. LL_SMOS) THEN  

      DO JNLV=1,IMXCOUNT
        DO JOBS=1,ILEN
          IF (JNLV>ICOUNT(JOBS)) CYCLE
          IF(SATHDR(JOBS,MDB_TB_FLAG_SMOS) == 1)THEN
            !ZXPP(JOBS,JNLV,1)=(MLNKH2B(JOBS)+IPNLV(JOBS,JNLV)-1,MDB_TBVALUE_SMOS)
            ZXPP(JOBS,JNLV,1)=SATHDR(JOBS,MDB_TBVALUE_SMOS)
          ELSE
            ZXPP(JOBS,JNLV,1)=RMDI
          ENDIF
        ENDDO
      ENDDO

    !*          4.5.6 Mean layer wind (for SATOB/AMVs)
  ELSEIF(CLV(JVNM) == 'U' .AND. LL_MEAN_UV) THEN
    SELECT CASE (TRIM(SATOBGRP_TABLE(IGRP)%obs_oper))
      !*           4.5.6a Function averaging
    CASE ('Gauss average','Boxcar average','RT median Gauss average','RT max Gauss average')
      ALLOCATE(ZUV(KDLEN,NFLEVG,2))
      ALLOCATE(ZW(KDLEN,NFLEVG))
      ALLOCATE(ZTAU(KDLEN,IMXCOUNT,NFLEVG))
      CALL MEANUV_WEIGHTS(KDLEN,ILEN,IMXCOUNT,NFLEVG,&
       & SATOBGRP_TABLE(IGRP)%obs_oper,ZTAU,ZVPOBS,&
       & ZPRESF5,SATOBGRP_TABLE(IGRP)%SIGMA,ZW)
      ZUV(1:ILEN,1:NFLEVG,1) = ZUF5(1:ILEN,1:NFLEVG)
      ZUV(1:ILEN,1:NFLEVG,2) = ZVF5(1:ILEN,1:NFLEVG)
      CALL MEANUV_AVERAGE(KDLEN,ILEN,IMXCOUNT,NFLEVG,ZUV,&
       & ZW,ZXPP)
      DEALLOCATE(ZUV,ZW,ZTAU)

      !*           4.5.6b Averaging using weighting function from radiative transfer
    CASE ('RT average','RT median RT average','RT max RT average')
      ! Allocations
      ALLOCATE(ZPRES(KDLEN,NLSAT))
      ALLOCATE(ZUV(KDLEN,NLSAT,2))
      ALLOCATE(ZW(KDLEN,NLSAT))

      ! RTTOV pressure levels and stored weights
      ICOUNTX(1:ILEN)=NLSAT
      DO JLEV=1,NLSAT
        ZPRES(1:ILEN,JLEV) = RADPRE(JLEV)
        ZW(1:ILEN,JLEV)    = SATHDR(1:ILEN,MDB_WEIGHT_AT_SATOB(JLEV))
      ENDDO

      !       Interpolate U and V to fixed pressure levels of RTTOV
      CALL PPOBSA(KDLEN,ILEN,NLSAT,'U',IOBSTYPE,ICOUNTX,ZPRES,ZOROG5,&
       & ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,&
       & ZCLWF5,ZCIWF5,ZCCF5,ZTSTAR5,ZT05,ZLNPR5,ZALPH5,&
       & 2,ZUV)
      !       Calculate weighted layer mean
      CALL MEANUV_AVERAGE(KDLEN,ILEN,IMXCOUNT,NLSAT,ZUV,&
       & ZW,ZXPP)

      DEALLOCATE(ZPRES,ZUV,ZW)
    END SELECT

    !*           4.5.7 Radar reflectivity

  ELSEIF(CLV(JVNM) == 'RFL') THEN

    IF (KHORIZ == 1) THEN

      CALL REFLSIM(KDLEN,KDBDY,ILEN,IMXCOUNT,ICOUNT,IPNLV,ZTF5,ZQF5,ZCLWF5,&
       & ZCIWF5,ZCSWF5,ZCRWF5,ZCGWF5,ZGEOPF5,IDF,ZXPP)

    ELSE

      ! call to the 2d_operator in order to inverse reflectivities

      ZXPP(:,:,:)=0.0_JPRB
      CALL REFLSIM_2DOP(KDLEN,KDBDY,ILEN,KHORIZ,IMXCOUNT,ICOUNT,&
       &  IPNLV,ZTF5_2D,ZQF5_2D,&
       &  ZCLWF5_2D, ZCIWF5_2D,ZCSWF5_2D,ZCRWF5_2D,ZCGWF5_2D,&
       &  ZGEOPF5_2D,IDF,ZXPP_2D)

      ZXPP(:,:,:)=ZXPP_2D(:,:,:,1)
    ENDIF

    !*           4.5.7 bis Radar doppler wind

  ELSEIF(CLV(JVNM) == 'DOW') THEN
    IF (IOBSTYPE == NRADAR .AND. KHORIZ > 1 ) THEN
      ZOROG5(:) = ZOROG5_2D(:,1)
      ZPRESH5(:,:) = ZPRESH5_2D(:,:,1)
      ZPRESF5(:,:) = ZPRESF5_2D(:,:,1)
      ZGEOPH5(:,:) = ZGEOPH5_2D(:,:,1)
      ZGEOPF5(:,:) = ZGEOPF5_2D(:,:,1)
      ZPXP5(:,:,:) = ZPXP5_2D(:,:,:,1)
      ZPXPD5(:,:,:) = ZPXPD5_2D(:,:,:,1)
      ZPXZ5(:,:,:) = ZPXZ5_2D(:,:,:,1)
      ZPXZD5(:,:,:) = ZPXZD5_2D(:,:,:,1)
      ZUF5(:,:) = ZUF5_2D(:,:,1)
      ZVF5(:,:) = ZVF5_2D(:,:,1)
      ZTF5(:,:) = ZTF5_2D(:,:,1)
      ZQF5(:,:) = ZQF5_2D(:,:,1)
      ZO3F5(:,:) = ZO3F5_2D(:,:,1)
      ZCLWF5(:,:) = ZCLWF5_2D(:,:,1)
      ZCIWF5(:,:) = ZCIWF5_2D(:,:,1)
      ZCCF5(:,:) = ZCCF5_2D(:,:,1)
      ZTSTAR5(:) = ZTSTAR5_2D(:,1)
      ZT05(:) = ZT05_2D(:,1)
      ZDELP5(:,:) =  ZDELP5_2D(:,:,1)
      ZRDELP5(:,:) = ZRDELP5_2D(:,:,1)
      ZLNPR5(:,:) = ZLNPR5_2D(:,:,1)
      ZALPH5(:,:) = ZALPH5_2D(:,:,1)
      ZRTGR5(:,:) = ZRTGR5_2D(:,:,1)
    ENDIF
    CALL DOPPLSIM(KDLEN,KDBDY,ILEN,IMXCOUNT,ICOUNT,IPNLV,ZUF5,ZVF5,&
     &   ZGEOPH5,IDF,ZXPP)

    !*          4.5.8 GPS Atmospheric Path Delay
  ELSEIF(CLV(JVNM) == 'APD') THEN
    DO JOBS = 1,ILEN
      ZOBALT=ROBHDR(JOBS,MDBALT)
      IF(ZOBALT == RMDI .OR. ABS(ZOBALT) >  10000._JPRB) THEN
        ZORDIF(JOBS) = 0.0_JPRB
      ELSE
        ZORDIF(JOBS) = ZOROG5(JOBS)-ZOBALT*RG
      ENDIF
    ENDDO
    CALL PPPMER(KDLEN,1,ILEN,ZPRESH5(1,NFLEVG),ZORDIF,ZTSTAR5,ZT05,ZVPOBS)
    IF (LMFSCREEN) THEN
      CALL GPSZEN_DELAY(KDLEN,ILEN,IMXCOUNT,ICOUNT,&
       &                 ZPRESH5,ZGEOPH5,ZTF5,ZQF5,ZOROGOB,ZVPOBS,&
       &                 IDF,ZXPP,PZHD_DELAY=ZXPPE1,PZWD_DELAY=ZXPPE2)
      DO JNLV = 1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV>ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          ROBODY(IBODY,MDB_CSR_PCLEAR_AT_BODY) = ZXPPE1(JOBS,JNLV,1)
          ROBODY(IBODY,MDB_CSR_PCLOUDY_AT_BODY) = ZXPPE2(JOBS,JNLV,1)
          ROBODY(IBODY,MDB_SURFEMISS_AT_BODY) = ZORDIF(JOBS)/RG
        ENDDO
      ENDDO
    ELSE
      CALL GPSZEN_DELAY(KDLEN,ILEN,IMXCOUNT,ICOUNT,&
       &                 ZPRESH5,ZGEOPH5,ZTF5,ZQF5,ZOROGOB,ZVPOBS,&
       &                 IDF,ZXPP)
    ENDIF

    !*          4.5.9 Radio occultation
  ELSEIF(CLV(JVNM) =='RO ') THEN


    IF (KHORIZ == 1) THEN

      IF (LMFSCREEN) THEN
        CALL GPSRO_OP(KDLEN,KDBDY,IMXCOUNT,ILEN,IVNMRQ,&
         &             ZPRESF5,ZTF5,ZQF5,ZGEOPF5,ZXPP, &
         &             P_ZXPP_N=ZXPPE1,P_ZXPP_DNDZ=ZXPPE2,P_ZXPP_T=ZXPPE3, &
         &             K_LEVM=IXPPE1,LD_QC_DNDZ=LLXPPE1,LD_QC_D2NDZ2=LLXPPE2)
        DO JNLV = 1,IMXCOUNT
          DO JOBS = 1,ILEN
            IF (JNLV>ICOUNT(JOBS)) CYCLE
            IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
            ROBODY(IBODY,MDB_BG_REFRACTIVITY_AT_ROBODY) = ZXPPE1(JOBS,JNLV,1)
            ROBODY(IBODY,MDB_BG_DNDZ_AT_ROBODY) = ZXPPE2(JOBS,JNLV,1)
            ROBODY(IBODY,MDB_BG_LAYERNO_AT_ROBODY) = REAL(IXPPE1(JOBS,JNLV,1))
            IRETRTYPE=ROBHDR(JOBS,MDB_RETRTYPE_AT_HDR)
            IF (.NOT.LLXPPE1(JOBS,JNLV,1)) IRETRTYPE=IBSET(IRETRTYPE,19)
            IF (.NOT.LLXPPE2(JOBS,JNLV,1)) IRETRTYPE=IBSET(IRETRTYPE,20)
            ROBHDR(JOBS,MDB_RETRTYPE_AT_HDR)=IRETRTYPE
            ROBODY(IBODY,MDB_BG_TVALUE_AT_ROBODY) = ZXPPE3(JOBS,JNLV,1)
          ENDDO
        ENDDO
      ELSE
        CALL GPSRO_OP(KDLEN,KDBDY,IMXCOUNT,ILEN,IVNMRQ,&
         &             ZPRESF5,ZTF5,ZQF5,ZGEOPF5,ZXPP)  
      ENDIF

    ELSE

      ! call the 2d routine

      CALL &
       & GPSRO_2DOP(KDLEN,KDBDY,KHORIZ,IMXCOUNT,ILEN,IVNMRQ, &
       & ZPRESF5_2D,ZTF5_2D,ZQF5_2D,ZGEOPF5_2D,ZXPP)  

    ENDIF

    !*          4.5.9 Aerosol optical depth 
  ELSEIF(CLV(JVNM) =='AOD') THEN
    CALL AOD_OP(KDLEN,ILEN,IMXCOUNT,ZQF5, ZTF5, ZPRESH5, ZPRESF5, YLGEMSPROF5, ZXPP)

!*          4.5.9a Variational bias correction for AOD
!
!BEN - Add variational bias correction for aerosol. Use TO3 template, but without solar elevation 
! as bias predictor (set it to RMDI). For now bias model for aerosol consists only of a global constant.

    IF (LVARBC) THEN
      ALLOCATE(ZXPPB(KDLEN,IMXCOUNT,1),ZXPRED(0:JPMXNPRED,ILEN), &
       & ZSOLAR(ILEN), &
       & ZOMF(KDLEN,IMXCOUNT),IXVARBC(KDLEN,IMXCOUNT))
      ZOMF(:,:)=RMDI
      DO JNLV=1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV > ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          IF (ZXPP(JOBS,JNLV,1) /= RMDI) &
           & ZOMF(JOBS,JNLV) = ROBODY(IBODY,MDBVAR) - ZXPP(JOBS,JNLV,1)
          IXVARBC(JOBS,JNLV) = ROBODY(IBODY,MDB_VARBC_IX_AT_BODY)
        ENDDO
      ENDDO

! BEN - For now this calculation is not needed
!      DO JOBS= 1, ILEN
!        ZZLAT   = ROBHDR(JOBS,MDBLAT) * RDEGREES
!        ZZLON   = ROBHDR(JOBS,MDBLON) * RDEGREES
!        IOBDATE = ROBHDR(JOBS,MDBDAT)
!        IYEAR   = IOBDATE/10000
!        IMONTH  = IOBDATE/100 - IYEAR*100
!        IDAY    = IOBDATE - (IYEAR*10000+IMONTH*100)
!        IOBTIME = ROBHDR(JOBS,MDBETM)
!        IHOUR   = IOBTIME/10000
!        CALL DIURNAL ( IMONTH, IDAY, IHOUR, ZZLAT, ZZLON, ZSOLAR(JOBS))
!      ENDDO

      ZSOLAR(:) =RMDI

      CALL VARBC_PRED_TO3 (ILEN, IMXCOUNT, ICOUNT(1:ILEN), IXVARBC(1:ILEN,:), &
        & ZSOLAR, JPMXNPRED, ZXPRED, LSTATS=LIFSTRAJ)

      CALL VARBC_BIAS (IXVARBC, IMXCOUNT, ICOUNT, KDLEN, ILEN, &
       & JPMXNPRED, ZXPRED, ZOMF, ZXPPB(:,:,1))

      DO JNLV = 1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV>ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          ROBODY(IBODY,MDBTORB) = ZXPPB(JOBS,JNLV,1)
        ENDDO
      ENDDO
      DEALLOCATE(ZXPPB, ZXPRED, ZSOLAR, ZOMF, IXVARBC)
    ENDIF

    !*          4.5.9.b Cloud optical depth
  ELSEIF(CLV(JVNM) =='COD') THEN
    CALL COD_OP(KDLEN,ILEN,IMXCOUNT,ZQF5, ZTF5, ZPRESH5, ZPRESF5, ZLS5,ZCLWF5,ZCIWF5,ZXPP,ZTAUC)

    !*          4.5.9.c Ratio fine mode to total aerosol optical depth at 0.55 microns
    ! BEN - for now no operator
  ELSEIF(CLV(JVNM) =='RAO') THEN
    ZXPP(:,:,1) = 0.0_JPRB

    !*          4.5.9.d Aerosol reflectance (multi-channel)
  ELSEIF(CLV(JVNM) =='RFA') THEN
    ! BEN - for now do not call the operator
    ZXPP(:,:,1) = 0.0_JPRB

!    ! For now define ZSOLANG5, ZVIEANG5 and ZDIFFANG5 here - in the future they have to come from ODB!!!
!
!    ZSOLANG5(:) = 0.2
!    ZVIEANG5(:) = 0.3
!    ZDIFFANG5(:) = ACOS(-1.0)/2.
!
!    CALL AER_REFL_OP( KDLEN  , ILEN  , IMXCOUNT, ZQF5, ZTF5, ZPRESH5, ZPRESF5, YLGEMSPROF5, &
!     &   ZSOLANG5, ZVIEANG5, ZDIFFANG5, &
!     &   ZXPP )

    !*          4.5.9.e Aerosol optical depth (multi-channel)
  ELSEIF(CLV(JVNM) =='ODA') THEN
    ZXPP(:,:,1) = 0.0_JPRB



    !*           4.5.10  Limb radiances
  ELSEIF (CLV(JVNM) =='LRA') THEN

    IF (KHORIZ == 1) THEN

      CALL RTL_HOP_1D(KDLEN,ILEN,IMXCOUNT,ICOUNT,IOBSTYPE,         &
       & Y_LIMBGRP_TABLE(IGRP),ZVPOBS,ZVPOBS_L,ZOROG5,             &
       & ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,   &
       & ZCLWF5,ZCIWF5,ZCCF5,ZTSTAR5,ZT05,ZLNPR5,ZALPH5,ZXPP)

    ELSE

      CALL RTL_HOP_2D(KDLEN,ILEN,KHORIZ,IMXCOUNT,ICOUNT,IOBSTYPE,  &
       & Y_LIMBGRP_TABLE(IGRP),ZVPOBS,ZVPOBS_L,ZOROG5_2D,          &
       & ZPRESH5_2D,ZPRESF5_2D,ZPXP5_2D,ZPXPD5_2D,ZUF5_2D,ZVF5_2D, &
       & ZTF5_2D,ZQF5_2D,ZO3F5_2D,ZCLWF5_2D,ZCIWF5_2D,ZCCF5_2D,    &
       & ZTSTAR5_2D,ZT05_2D,ZLNPR5_2D,ZALPH5_2D,ZXPP)

    ENDIF

    !*           4.5.11  Greenhouse gases (GHG) operator
  ELSEIF(ANY(LLGHG)) THEN

    !*          4.5.11b GEMS operators using averaging kernels
    IF (LL_AK) THEN

      CALL GHG_AK_OP(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),ICOUNT,&
       & IAK,ZWAK,ZAPAK,ZPAK,ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,&
       & YLGEMSPROF5,IDF,ZXPP)

      !*          4.5.11c GHG operators without averaging kernels
    ELSE

      !*          Mean Total Column CO2 Mixing Ratio
      IF(CLV(JVNM) =='CO2') THEN

        ZSOLEL(1:ILEN)  = SATHDR(1:ILEN,MDBRO3SOE)
        CALL CO2_TCMR(KDLEN,ILEN,IMXCOUNT,ICOUNT,ZPRESH5, ZPRESF5, ZSOLEL, YLGEMSPROF5, ZXPP)

      ENDIF

      !*          Mean Total Column CH4 Mixing Ratio
      IF(CLV(JVNM) =='CH4') THEN

        ZSOLEL(1:ILEN)  = SATHDR(1:ILEN,MDBRO3SOE)
        CALL CH4_TCMR(KDLEN,ILEN,IMXCOUNT,ICOUNT,ZPRESH5, ZPRESF5, ZSOLEL, YLGEMSPROF5, ZXPP, ZCH4COR)
        SATHDR(1:ILEN,MDB_CH4COR_AT_REO3) = ZCH4COR(1:ILEN,1)

      ENDIF

    ENDIF

    !*           4.5.12  Reactive gases (GRG) operator
  ELSEIF(ANY(LLGRG)) THEN

    !*          4.5.12a Convert model NOX to NO2
    IF (LLNO2) THEN

      ! calc optical thicknes of clouds
      CALL COD_OP(KDLEN,ILEN,1,ZQF5, ZTF5, ZPRESH5, ZPRESF5, ZLS5,ZCLWF5,ZCIWF5,ZCOD,ZTAUC)

      CALL NOX2NO2(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),IOBSTYPE,ICOUNT,ZVPOBS,&
       & ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,&
       & ZTF5,ZSOLEL,ZAL5,ZCOD,ZTAUC,ZCCF5,YLGEMSPROF5,&
       & ZNO25)

    ENDIF

    !*          4.5.12b GEMS operators using averaging kernels
    IF (LL_AK) THEN

      IF (LLNO2) THEN
        CALL GRG_AK_OP(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),ICOUNT,&
         & IAK,ZWAK,ZAPAK,ZPAK,ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,&
         & YLGEMSPROF5,IDF,ZXPP,PNO25=ZNO25)
      ELSE
        IF(LL_MOPITT) THEN
          CALL MOPITT_AK_OP(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),IOBSTYPE,ICOUNT,&
           & IAK,ZWAK,ZAPAK,ZPAK,ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,&
           & YLGEMSPROF5,IDF,ZXPP)
        ELSE
          CALL GRG_AK_OP(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),ICOUNT,&
           & IAK,ZWAK,ZAPAK,ZPAK,ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,&
           & YLGEMSPROF5,IDF,ZXPP)
        ENDIF
      ENDIF

      !*          4.5.12c GRG operators without averaging kernels
    ELSE

      IF (LLNO2) THEN
        CALL PPOBSA(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),IOBSTYPE,ICOUNT,ZVPOBS,&
         & ZOROG5,ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,&
         & ZCLWF5,ZCIWF5,ZCCF5,ZTSTAR5,ZT05,ZLNPR5,ZALPH5,IDF,ZXPP,&
         & YDGEMSPROF5=YLGEMSPROF5,PNO25=ZNO25)

        IF(LL_LAYER) THEN
          CALL PPOBSA(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),IOBSTYPE,&
           & ICOUNT,ZVPOBS_L,ZOROG5,&
           & ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,&
           & ZCLWF5,ZCIWF5,ZCCF5,ZTSTAR5,ZT05,ZLNPR5,ZALPH5,&
           & IDF-1,ZXPP(1,1,2),YDGEMSPROF5=YLGEMSPROF5,PNO25=ZNO25)
        ENDIF
      ELSE
        CALL PPOBSA(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),IOBSTYPE,ICOUNT,ZVPOBS,&
         & ZOROG5,ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,&
         & ZCLWF5,ZCIWF5,ZCCF5,ZTSTAR5,ZT05,ZLNPR5,ZALPH5,IDF,ZXPP,&
         & YDGEMSPROF5=YLGEMSPROF5)

        IF(LL_LAYER) THEN
          CALL PPOBSA(KDLEN,ILEN,IMXCOUNT,CLV(JVNM),IOBSTYPE,&
           & ICOUNT,ZVPOBS_L,ZOROG5,&
           & ZPRESH5,ZPRESF5,ZPXP5,ZPXPD5,ZUF5,ZVF5,ZTF5,ZQF5,ZO3F5,&
           & ZCLWF5,ZCIWF5,ZCCF5,ZTSTAR5,ZT05,ZLNPR5,ZALPH5,&
           & IDF-1,ZXPP(1,1,2),YDGEMSPROF5=YLGEMSPROF5)
        ENDIF
      ENDIF

    ENDIF

    !*          4.5.12d Variational bias correction for GRG

    IF (LVARBC) THEN
      ALLOCATE(ZXPPB(KDLEN,IMXCOUNT,1),ZXPRED(0:JPMXNPRED,ILEN), &
       & ZSOLAR(ILEN), &
       & ZOMF(KDLEN,IMXCOUNT),IXVARBC(KDLEN,IMXCOUNT))
      ZOMF(:,:)=RMDI
      DO JNLV=1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV > ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          IF (ZXPP(JOBS,JNLV,1) /= RMDI) THEN
            IF (LL_AK) THEN
              ZOMF(JOBS,JNLV) = ROBODY(IBODY,MDBVAR) - ZXPP(JOBS,JNLV,1)
            ELSE
              ZOMF(JOBS,JNLV) = ROBODY(IBODY,MDBVAR) &
             &          - (ZLAYL(JVNM,1)*ZXPP(JOBS,JNLV,1)+ZLAYL(JVNM,2)*ZXPP(JOBS,JNLV,2))
            ENDIF
          ENDIF
          IXVARBC(JOBS,JNLV) = ROBODY(IBODY,MDB_VARBC_IX_AT_BODY)
        ENDDO
      ENDDO
      DO JOBS= 1, ILEN
        ZZLAT   = ROBHDR(JOBS,MDBLAT) * RDEGREES
        ZZLON   = ROBHDR(JOBS,MDBLON) * RDEGREES
        IOBDATE = ROBHDR(JOBS,MDBDAT)
        IYEAR   = IOBDATE/10000
        IMONTH  = IOBDATE/100 - IYEAR*100
        IDAY    = IOBDATE - (IYEAR*10000+IMONTH*100)
        IOBTIME = ROBHDR(JOBS,MDBETM)
        IHOUR   = IOBTIME/10000
        CALL DIURNAL ( IMONTH, IDAY, IHOUR, ZZLAT, ZZLON, ZSOLAR(JOBS))
      ENDDO

      CALL VARBC_PRED_TO3 (ILEN, IMXCOUNT, ICOUNT(1:ILEN), IXVARBC(1:ILEN,:), &
        & ZSOLAR, JPMXNPRED, ZXPRED, LSTATS=LIFSTRAJ)

      CALL VARBC_BIAS (IXVARBC, IMXCOUNT, ICOUNT, KDLEN, ILEN, &
       & JPMXNPRED, ZXPRED, ZOMF, ZXPPB(:,:,1))

      DO JNLV = 1,IMXCOUNT
        DO JOBS = 1,ILEN
          IF (JNLV>ICOUNT(JOBS)) CYCLE
          IBODY = MLNKH2B(JOBS) + (IPNLV(JOBS,JNLV)-1)
          ROBODY(IBODY,MDBTORB) = ZXPPB(JOBS,JNLV,1)
        ENDDO
      ENDDO
      DEALLOCATE(ZXPPB, ZXPRED, ZSOLAR, ZOMF, IXVARBC)
    ENDIF

    !*           4.5.13  ASCAT Soil Moisture
  ELSEIF(CLV(JVNM) == 'SM ') THEN
    DO JNLV=1,IMXCOUNT
      DO JOBS=1,ILEN
        IF(JNLV > ICOUNT(JOBS)) CYCLE
        ! nullify soil moisture if Temperature is below 0 or snow is present
        ! under these conditions soil moisture is not correct
        IF ((ZTF5(JOBS,1) > 273.15).AND.(ZSN5(JOBS) < 0.1)) then
          ZXPP(JOBS,JNLV,1)=ZWS5(JOBS)
        ELSE
          ZXPP(JOBS,JNLV,1)=RMDI
        ENDIF
      ENDDO
    ENDDO
  ENDIF
  IF (LVARBC .AND. .NOT.(CLV(JVNM) == 'TO3') &
          &    .AND. .NOT.(LL_TCWV .AND. CLV(JVNM) == 'PWC') &
          &    .AND. .NOT.(CLV(JVNM) == 'RAD')) THEN
!$OMP ORDERED
      ICC=ICC+1
!$OMP END ORDERED

  ENDIF

  !*          4.6   Transfer PP-values (i.e. the model equivalents of
  !*                the observations) to their location in ZHOFX

  DO JNLV=1,IMXCOUNT
    DO JOBS=1,ILEN
      IF(JNLV > ICOUNT(JOBS)) CYCLE
      IBDY=IPNLV(JOBS,JNLV)

      IF(LL_WIND) THEN
        ZHOFX(JOBS,IBDY  )=ZXPP(JOBS,JNLV,1)
        ZHOFX(JOBS,IBDY+1)=ZXPP(JOBS,JNLV,2)
        IVAR( JOBS,IBDY+1)=IVAR(JOBS,IBDY)
      ELSEIF(LL_SPEED) THEN
        ZSPEED5=SQRT(ZXPP(JOBS,JNLV,1)**2+ZXPP(JOBS,JNLV,2)**2)
        ZHOFX(JOBS,IBDY)=MAX(ZSPEED5,1.E-10_JPRB)
      ELSEIF(LL_HLS) THEN
        IBODY = MLNKH2B(JOBS)+(IBDY-1)
        ZAZIM=ROBODY(IBODY,MDB_AUX_AT_BODY(1))
        Zsin_azim=SIN(ZAZIM)
        Zcos_azim=COS(ZAZIM)
        ZHLS=-ZXPP(JOBS,JNLV,1)*Zsin_azim-ZXPP(JOBS,JNLV,2)*Zcos_azim
        ZHOFX(JOBS,IBDY)=ZHLS
      ELSEIF(LL_LAYER) THEN
        ZHOFX(JOBS,IBDY)=ZLAYL(JVNM,1)*ZXPP(JOBS,JNLV,1) &
         & +ZLAYL(JVNM,2)*ZXPP(JOBS,JNLV,2)
      ELSE
        ZHOFX(JOBS,IBDY)=ZXPP(JOBS,JNLV,1)
      ENDIF

    ENDDO
  ENDDO

  !*          4.7   CANARI CALLS CALVER

  IF(LCANARI) THEN
    IF(CLV(JVNM) == 'U'  .OR. CLV(JVNM) == 'T'  .OR. CLV(JVNM) == 'H'  .OR.&
     & CLV(JVNM) == 'U10'.OR. CLV(JVNM) == 'T2' .OR. CLV(JVNM) == 'H2' .OR.&
     & CLV(JVNM) == 'Z'  .OR. CLV(JVNM) == 'TS' .OR. CLV(JVNM) == 'SN' .OR.&
     & CLV(JVNM) == 'RR' .OR. CLV(JVNM) == 'Q'  .OR. CLV(JVNM) == 'PWC' ) THEN

      CALL CALVER(KDLEN,KDBDY,KSET,IMXCOUNT,IPNLV,ZVPOBS,&
       & ICOUNT,ZPRESF5,MDBFGE)

    ENDIF
  ENDIF

  IF ( LCANARI .OR. LMFSCREEN ) THEN
    IF(CLV(JVNM) == 'DZ' ) THEN
      CALL CALVER(KDLEN,KDBDY,KSET,IMXCOUNT,IPNLV,ZVPOBS,&
       & ICOUNT,ZPRESF5,MDBFGE)
      CALL CALVER(KDLEN,KDBDY,KSET,IMXCOUNT,IPNLV,ZVPOBS_L,&
       & ICOUNT,ZPRESF5,MDBRBOE)
    ENDIF
  ENDIF

  !*          4.8   DEALLOCATE

  DEALLOCATE(ZXPP_2D,ZXPP,ZVPOBS,ZVPOBS_L,ZORDIF,ZTAUC)
  DEALLOCATE(ZCH4COR)
  IF(LL_RAD_OP) DEALLOCATE(ZEM5)
  IF(LLREO3) DEALLOCATE(IAK,ZPAK,ZWAK,ZAPAK)
  IF(ANY(LLGRG)) DEALLOCATE(ZCOD)
  IF(LLNO2) DEALLOCATE(ZNO25)
  IF(LMFSCREEN) DEALLOCATE(ZXPPE1,ZXPPE2,ZXPPE3,IXPPE1,LLXPPE1,LLXPPE2)

ENDDO VNM_LOOP

IF(LLREO3) DEALLOCATE(ZSOLEL,IO3SENSOR)

!     ------------------------------------------------------------------
!*       5.       Departures (obs-model)
!                 ----------------------

IF(IOBSTYPE == NAIREP) THEN
  CALL HDEPART(KDLEN,ILEN,KDBDY,IMXBDY,IOBSTYPE,&
 & ICMBDY,IVNMRQ,ZHOFX)
ELSE
  CALL HDEPART(KDLEN,ILEN,KDBDY,IMXBDY,ICDTYP_TOVS,&
 & ICMBDY,IVNMRQ,ZHOFX)
ENDIF

!*          5.1   In verification mode vi only want Jo for active data

IF (NUPTRA==NRESUPD.OR.LTWINEXP) THEN
  DO JBODY=1,IMXBDY
    DO JOBS=1,ILEN
      IF(JBODY > ICMBDY(JOBS)) CYCLE

      IBODY = MLNKH2B(JOBS)+(JBODY-1)
      IDSTA_ACTIVE=IBITS(INT(ROBODY(IBODY,MDBDSTA)), NCMDACBP, NCMDACOC)
      IF(IDSTA_ACTIVE /= 1) ZHOFX(JOBS,JBODY)=RMDI
    ENDDO
  ENDDO
ENDIF

!*           5.2  Mode based var bias correction

IF ( LL_MODE .AND. (.NOT. LSCREEN) ) THEN
  ICOUNT(:)=0
  DO JBODY=1,IMXBDY
    DO JOBS = 1,ILEN
      IF(IVNMRQ(JOBS,JBODY) == NVNUMB(50)) THEN  !BT count
        ICOUNT(JOBS)=ICOUNT(JOBS)+1
      ENDIF
    ENDDO
  ENDDO
  IMXCOUNT=MAXVAL(ICOUNT(1:ILEN))

  ALLOCATE( ZOMF(KDLEN,IMXBDY),IXVARBC(KDLEN,IMXBDY))
  ALLOCATE( ZBIAS_WK(KDLEN,IMXBDY) )
  ZOMF(:,:)=RMDI
  ZBIAS_WK(:,:)=RMDI
  DO JBODY=1,IMXBDY
    DO JOBS = 1,ILEN
      IIVNM=IVNMRQ(JOBS,JBODY)
      IF( IIVNM /= NVNUMB(50) ) CYCLE
      IF(JBODY > ICMBDY(JOBS) .OR. JBODY > ICOUNT(JOBS) ) CYCLE
      IBODY = MLNKH2B(JOBS)+(JBODY-1)
      IF (ZXPPC(JOBS,JBODY,1) == RMDI .OR. ROBODY(IBODY,MDBVAR) == RMDI  &
       & .OR. ZXPPC(JOBS,JBODY,1) <= 0.5 ) THEN
        ZOMF(JOBS,JBODY) = RMDI
      ELSE
        ZOMF(JOBS,JBODY) = ROBODY(IBODY,MDBVAR) - ZXPPC(JOBS,JBODY,1)         
        ZBIAS_WK(JOBS,JBODY)=ROBODY(IBODY,MDBTORB)
      ENDIF
      IXVARBC(JOBS,JBODY) = ROBODY(IBODY,MDB_VARBC_IX_AT_BODY)
    ENDDO
  ENDDO
  ZWPD_MODE(:) = 0.0_JPRB
  DO JBODY=1,IMXBDY
    DO JOBS = 1,ILEN
      IIVNM=IVNMRQ(JOBS,JBODY)
      IF( IIVNM /= NVNUMB(50)) CYCLE
      IF(JBODY > ICMBDY(JOBS) .OR. JBODY > ICOUNT(JOBS) ) CYCLE
      IF (ZHOFX(JOBS,JBODY) == RMDI .OR. ZOMF(JOBS,JBODY) == RMDI ) CYCLE 
      ix = IXVARBC(JOBS,JBODY)
      IF (ix == 0) CYCLE
      IF( YVARBC(ix)%llmode ) THEN 
        IF ( SUM(yvarbc(ix)%nhstfgdep(:)) /= 0 ) THEN
          ibin_max = maxval( yvarbc(ix)%nhstfgdep(:) )
          imax_loc = maxloc( yvarbc(ix)%nhstfgdep(:), dim=1 )
          imax_dist = imax_loc
          if (imax_dist > NHSTBINS/2)imax_dist = NHSTBINS - imax_dist
          ibinx=imax_dist - 1
          if (ibinx < 0) ibinx = 0
          ihist_sum=SUM(yvarbc(ix)%nhstfgdep(imax_loc-ibinx:imax_loc+ibinx))
          if ( imax_dist > 2 .AND. ihist_sum /= 0  ) then
            ZWPD_MODE(imax_loc) = REAL(ibin_max)/REAL(ihist_sum)
            DO JHIST = 1, imax_dist - 1
              if(JHIST <= ibinx )then
                 I_nhist_avg_r = yvarbc(ix)%nhstfgdep(imax_loc + JHIST)
                 I_nhist_avg_l = yvarbc(ix)%nhstfgdep(imax_loc - JHIST)
              else
                I_nhist_avg_r = 0
                I_nhist_avg_l = 0
              endif
              ZWPD_MODE(imax_loc + JHIST) = REAL(I_nhist_avg_l)/REAL(ihist_sum)
              ZWPD_MODE(imax_loc - JHIST) = REAL(I_nhist_avg_r)/REAL(ihist_sum)
            ENDDO
          endif
        ENDIF
        IF ( ABS(ZOMF(JOBS,JBODY)) < YVARBC(ix)%DFGDEP) THEN
          ib = NINT(REAL(NHSTBINS)*(0.5 + ZOMF(JOBS,JBODY)/YVARBC(ix)%DFGDEP))
          ibd = NINT(REAL(NHSTBINS)*(ZBIAS_WK(JOBS,JBODY)/     &
          &                               YVARBC(ix)%DFGDEP))
        ELSE
          ib=0
        ENDIF
        IF ( ((ib>1).AND.(ib<NHSTBINS)) .AND.   &
        &    ((ib+ibd)>(imax_loc-ibinx).AND.(ib+ibd)<(imax_loc+ibinx)) .AND. &
        &     ZWPD_MODE(ib+ibd) >= 0.0_JPRB ) THEN
           zwpd = ZWPD_MODE(ib + ibd) * REAL(yvarbc(ix)%ncount)
           zwpd = SQRT(zwpd)
         ELSE
           zwpd =  0.0_JPRB
         ENDIF
         ZHOFX(JOBS,JBODY) = ZHOFX(JOBS,JBODY) * zwpd
      ENDIF
    ENDDO
  ENDDO
  DEALLOCATE(ZOMF, ZBIAS_WK, IXVARBC, ZXPPC)
ENDIF

IF ( .NOT.LCANARI ) THEN
  DO JOBS = 1,ILEN
    ZTIME = 0.0_JPRB
    IOBDATE= ROBHDR(JOBS,MDBDAT)
    IOBTIME= ROBHDR(JOBS,MDBETM)
    ITMO100 = IOBTIME / 100
    CALL TIMDIF(IOBDATE,ITMO100,ZTIME)
    IF (ZTIME<ZLEFTTM .OR. ZTIME>ZRIGHTM) THEN
      write(nulerr,*)'HOP: Time out of range'
      write(nulerr,*)'HOP: ztime,zleft,zright=',ZTIME,ZLEFTTM,ZRIGHTM
      write(nulerr,*)'HOP: IOBDATE,IOBTIME=',IOBDATE,IOBTIME
!       Cannot abort here because observations outside the window do
!       get to this point!!!
      INDXTIME(JOBS)=-999
      ZHOFX(JOBS,:)=RMDI
    ELSE
      ZTIME=(ZTIME-ZLEFTTM)/(ZRIGHTM-ZLEFTTM)
      ZTIME=ZTIME * REAL(NPRTBINS,JPRB)
      ITIME=INT(ZTIME)+1
      IF (ITIME<1 .OR. ITIME>NPRTBINS+1) THEN
        write(nulerr,*)'HOP: Error itime=',ITIME,ZTIME,ZLEFTTM,ZRIGHTM
        write(nulerr,*)'HOP: IOBDATE,IOBTIME=',IOBDATE,IOBTIME
        CALL ABOR1('HOP: Error itime')
      ENDIF
      INDXTIME(JOBS)=MIN(NPRTBINS,ITIME)
    ENDIF
  ENDDO
ENDIF

!     ------------------------------------------------------------------
!*       6.       Cost function
!                 -------------

!*           6.1  Cost-function, with or without VarQC.
ISCAN=1
IF(.NOT.LCANARI) CALL HJO(KDLEN,KDBDY,KSET,ICMBDY,IOBSTYPE,ICDTYP,&
 & IVAR,IVNMRQ,ZHOFX,ISENSOR,INDXTIME,ISCAN)

!*           6.2  Incremental convergence filter, for subsequent minimizations

IF (LINCV .AND. LOBSC1 ) &
 & CALL INCVFILT(KDLEN,ILEN,KDBDY,IMXBDY,ICMBDY,IVNMRQ)

!     ------------------------------------------------------------------
!*       7.       Update ODB
!                 ----------

IF (LENKF .AND. NUPTRA == 1) THEN
  MLNKH2B_SAV(1:ILEN) = MLNKH2B(1:ILEN)
ENDIF
IF (LCANARI) THEN
  CALL PUTDB('HOP_CA', 0, IRET, INFO, 0, ZINFO, 0)
ELSE
  CALL PUTDB('HOP', NUPTRA, IRET, INFO, 0, ZINFO, 0)
ENDIF

IF (LENKF .AND. NUPTRA == 1) THEN
  CALL APPEND_NUM(CL_HPRIOR_RETRIEVAL, 'UPDATE_HPRIOR_', NENSFNB)
  write(0,*) 'HOP HPRIOR KSET = ', KSET, ' IOBSTYPE = ', IOBSTYPE, 'CODETYPE = ', &
   & ICDTYP_TOVS, ' SENSOR = ', ISENSOR_GETDB
  
  CALL GETDB(CL_HPRIOR_RETRIEVAL, 0, IRET, INFO, 0, ZINFO, 0, &
   & KSET, ITSLOT, -1, IOBSTYPE, ICDTYP_TOVS, ISENSOR_GETDB)
  write(0,*) 'HOP HPRIOR GETDB ', trim(CL_HPRIOR_RETRIEVAL),  ' number of rows ', &
  &            IRET, 'MDB POINTER= ', MDB_HPRIOR_AT_ENKF(NENSFNB)
  DO JOBS=1, IRET
    ROBODY(JOBS,MDB_HPRIOR_AT_ENKF(NENSFNB))= ROBODY(JOBS, MDBVAR) - ROBODY(JOBS, MDBOMN) 
  ENDDO
  CALL PUTDB(CL_HPRIOR_RETRIEVAL, 0, IRET, INFO, 0, ZINFO, 0)
ENDIF
!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('HOP',1,ZHOOK_HANDLE)
END SUBROUTINE HOP
