SUBROUTINE HRADP_ML(KIND,KDLEN,KLEN,KSET,&
 & PNVANG,PZEANG,KCOEF,&
 & PPRESH5,PTF5,PQF5,&
 & PLS5,PCI5,PPS5,PTS5,PT2M5,PQ2M5,PCTOP5,PCAMT5)  

!****  HRADP_ML  - PREPARATION FOR THE RADIANCE INTERPOLATION

!     ERIK ANDERSSON          01/04/99       ECMWF

!     PURPOSE
!     -------

!        HRADP_ML PREPARES INTERPOLATION FOR ONE SET OF OBSERVATIONS
!        IF RT IS DONE ON MODEL LEVELS.

!**   INTERFACE
!     ---------

!        CALL HRADP_ML(KIND,KDLEN,KLEN,KSET,&
!    &PNVANG,PZEANG,KCOEF,&
!    &PPRESH5,PTF5,PQF5,&
!    &PLS5,PPS5,PTS5,PT2M5,PQ2M5)

!        Where KIND    = Mode of call, 1=SCREEN, 2=FWD, 3=TL, 4=AD
!              KDLEN   = First dimension (Number of observations)
!              KLEN    = Actual number of observations
!              KSET    = Number (numero) of the obs set to be treated
!              PNVANG  = Nadir viewing angle
!              PZEANG  = Zenith angle
!              KCOEF   = Location of rttov transmittance coeffs, for this sat
!              PPRESH5 = Half-level pressures
!              PTF5    = Model level temperature
!              PQF5    = Model level humidity
!              PLS5    = Land/Sea Mask
!              PCI5    = Sea-ice fraction
!              PPS5    = Surface pressure at TOVS locations
!              PTS5    = Surface skin temperature
!              PT2M5   = Two-metre temperature, for rttov
!              PQ2M5   = Two-metre spec. humidity, for rttov

!        HRADP_ML IS CALLED FROM HOP

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     E.Andersson 990104 : Original
!     P.Viterbo   990901 : Sea-ice fraction
!     Xu Li       000523 : Add FOV information for VTPR
!     N. Bormann  010622 : Altered call to HSATANG
!     P.Viterbo   28.05.2004 surf library and modified surface units
!     N. Bormann  20071215 : Renamed and revamped for RTTOV-9
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LOBSREF  ,LOBSC1
USE YOMCMA   , ONLY : NRESUPD
USE GOMS_MIX     , ONLY : LGOMUA,YGOMV
USE YOMDIM   , ONLY : NFLEVG
USE YOMTVRAD , ONLY : SATGRP_TABLE,TOVSCVX  ,&
 & TOVSCVX5 ,TVTSBGE_LAND,TVTSBGE_SEA,TVTSBGE_ICE,&
 & SATOBGRP_TABLE
USE YOMVAR   , ONLY : LTOVSCV  ,NUPTRA, LCO2, LCLDSINK , CTOPBGE , CAMTBGE
USE YOMCOCTP , ONLY : NGTHRB   , NSATOB
USE YOMOBSET , ONLY : MTYPOB
USE YOMVRTL  , ONLY : LTLINT   ,LOBSTL
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KIND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNVANG(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZEANG(KDLEN) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCOEF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCI5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT2M5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQ2M5(KDLEN),PCTOP5(KDLEN),PCAMT5(KDLEN) 

INTEGER(KIND=JPIM) :: IOBSTYPE,ICDTYP_TOVS,IGRP,ISERIES
INTEGER(KIND=JPIM) :: IPOSM
INTEGER(KIND=JPIM) :: JOBS
REAL(KIND=JPRB) :: ZTSER,ZRCIMIN
LOGICAL :: LLRAD1C,LLSATOB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "surf_inq.h"
END INTERFACE

#include "abor1.intfb.h"
#include "hsatang.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"

! ---------------------------------------------------------------------
!*          1.0   Initiate, set switches

IF (LHOOK) CALL DR_HOOK('HRADP_ML',0,ZHOOK_HANDLE)
CALL SURF_INQ(PRCIMIN=ZRCIMIN)
IOBSTYPE = MOD(MTYPOB(KSET),100)
IGRP = MOD(MTYPOB(KSET)/100,100)

LLSATOB = IOBSTYPE == NSATOB

IF(LLSATOB) THEN
  ICDTYP_TOVS = SATOBGRP_TABLE(IGRP)%codetype
  ISERIES     = SATOBGRP_TABLE(IGRP)%rtseries
  KCOEF       = SATOBGRP_TABLE(IGRP)%rtcoef_pos
ELSE
  ICDTYP_TOVS = SATGRP_TABLE(IGRP)%codetype
  ISERIES     = SATGRP_TABLE(IGRP)%rtseries
  KCOEF       = SATGRP_TABLE(IGRP)%rtcoef_pos
ENDIF

LLRAD1C = ICDTYP_TOVS == NGTHRB 

!*          1.1   Compute satellite nadir and viewing angles.
CALL HSATANG(KDLEN,KLEN,IOBSTYPE,ISERIES,ICDTYP_TOVS,IGRP,PNVANG,PZEANG)

!*          1.2   TOVS control variable, Skin temperature.
IF(LTOVSCV .AND. LLRAD1C) THEN
!OCL NOVREC
  DO JOBS = 1,KLEN
    IF(LOBSC1) THEN
      IF(NUPTRA == 0) THEN
        IF(PCI5(JOBS) > ZRCIMIN) THEN
          ZTSER=TVTSBGE_SEA+PCI5(JOBS)*(TVTSBGE_ICE-TVTSBGE_SEA)
        ELSE
          ZTSER=TVTSBGE_SEA+PLS5(JOBS)*(TVTSBGE_LAND-TVTSBGE_SEA)
        ENDIF
        SATHDR(JOBS,MDB1BCP13A)=ZTSER
      ELSE
        PTS5(JOBS) = SATHDR(JOBS,MDB1BCP14A)
      ENDIF
      IF(NUPTRA /= NRESUPD) &
       & SATHDR(JOBS,MDB1BCPTS1(NUPTRA+1))=PTS5(JOBS)  
    ELSEIF(LOBSREF) THEN
      PTS5(JOBS) = SATHDR(JOBS,MDB1BCPTS1(NUPTRA+1))
    ELSE
      IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
      IF((LTLINT .AND. LOBSTL) .OR. KIND == 4) THEN
        PTS5(JOBS) = TOVSCVX5(IPOSM,1)
      ELSE
        PTS5(JOBS) = TOVSCVX(IPOSM,1)
      ENDIF
    ENDIF
    IF(KIND /= 4) SATHDR(JOBS,MDB1BCP14A)=PTS5(JOBS)
  ENDDO
ENDIF

!*          1.2a   TOVS control variable, cloud sink variables

IF(LTOVSCV .AND. LLRAD1C .AND. LCLDSINK) THEN
!OCL NOVREC
  DO JOBS = 1,KLEN
    IF(LOBSC1) THEN
      IF(NUPTRA == 0) THEN
       IF(KIND == 1) THEN  ! when called by HRETR   
        ROBSU(JOBS,MDB_CTOPBG_AT_CLOUD_SINK)=1050.0_JPRB !no active estimates available yet   
        ROBSU(JOBS,MDB_CAMTBG_AT_CLOUD_SINK)= 0.0_JPRB !no active estimates available yet   
        ROBSU(JOBS,MDB_CTOPER_AT_CLOUD_SINK)=CTOPBGE 
        ROBSU(JOBS,MDB_CAMTER_AT_CLOUD_SINK)=CAMTBGE 
        PCTOP5(JOBS)=  ROBSU(JOBS,MDB_CTOPBG_AT_CLOUD_SINK)
        PCAMT5(JOBS)=  ROBSU(JOBS,MDB_CAMTBG_AT_CLOUD_SINK)
       ELSE                 ! when called by HOP
        IF(SATHDR(JOBS,MDB_CLDNE2_AT_ATOVS)>=0.0_JPRB .AND. &
         & SATHDR(JOBS,MDB_CLDNE2_AT_ATOVS)<=0.99_JPRB) THEN
          ! active value but not overcast use clear system
          ROBSU(JOBS,MDB_CTOPBG_AT_CLOUD_SINK)= 555.0_JPRB                             
          ROBSU(JOBS,MDB_CAMTBG_AT_CLOUD_SINK)= 0.0_JPRB                                
          ROBSU(JOBS,MDB_CTOPER_AT_CLOUD_SINK)=0.000001_JPRB  !disable cloud analysis for non-overcast
          ROBSU(JOBS,MDB_CAMTER_AT_CLOUD_SINK)=0.000001_JPRB  !disable cloud analysis for non-overcast
          PCTOP5(JOBS)=  ROBSU(JOBS,MDB_CTOPBG_AT_CLOUD_SINK) 
          PCAMT5(JOBS)=  ROBSU(JOBS,MDB_CAMTBG_AT_CLOUD_SINK)    
        ELSEIF (SATHDR(JOBS,MDB_CLDNE2_AT_ATOVS)>0.99_JPRB .AND. &
         &      SATHDR(JOBS,MDB_CLDNE2_AT_ATOVS)<=1.01_JPRB) THEN
          ! active value overcast use cloud sink system
          ROBSU(JOBS,MDB_CTOPBG_AT_CLOUD_SINK)= SATHDR(JOBS,MDB_CLDPTOP2_AT_ATOVS)
          ROBSU(JOBS,MDB_CAMTBG_AT_CLOUD_SINK)= SATHDR(JOBS,MDB_CLDNE2_AT_ATOVS)
          ROBSU(JOBS,MDB_CTOPER_AT_CLOUD_SINK)=CTOPBGE       
          ROBSU(JOBS,MDB_CAMTER_AT_CLOUD_SINK)=CAMTBGE 
          PCTOP5(JOBS)=  ROBSU(JOBS,MDB_CTOPBG_AT_CLOUD_SINK)
          PCAMT5(JOBS)=  ROBSU(JOBS,MDB_CAMTBG_AT_CLOUD_SINK)    
          PCTOP5(JOBS)= MIN(MAX(PCTOP5(JOBS),51.0_JPRB),1012.0_JPRB)     ! trap cloud top pressure     
        ELSE 
          ! no active value use clear system   
          ROBSU(JOBS,MDB_CTOPBG_AT_CLOUD_SINK)= 555.0_JPRB
          ROBSU(JOBS,MDB_CAMTBG_AT_CLOUD_SINK)= 0.0_JPRB
          ROBSU(JOBS,MDB_CTOPER_AT_CLOUD_SINK)=0.000001_JPRB  !disable cloud analysis for non-overcast
          ROBSU(JOBS,MDB_CAMTER_AT_CLOUD_SINK)=0.000001_JPRB  !disable cloud analysis for non-overcast
          PCTOP5(JOBS)=  ROBSU(JOBS,MDB_CTOPBG_AT_CLOUD_SINK)
          PCAMT5(JOBS)=  ROBSU(JOBS,MDB_CAMTBG_AT_CLOUD_SINK) 
        ENDIF
       ENDIF
      ELSE
        PCTOP5(JOBS)=  ROBSU(JOBS,MDB_CTOPINC_AT_CLOUD_SINK)
        PCAMT5(JOBS)=  ROBSU(JOBS,MDB_CAMTINC_AT_CLOUD_SINK)
        PCTOP5(JOBS)=  MIN(MAX(PCTOP5(JOBS),51.0_JPRB),1012.0_JPRB)     ! trap cloud top pressure
      ENDIF
      IF(NUPTRA /= NRESUPD) THEN
        ROBSU(JOBS,MDB_CTOP_AT_CLOUD_SINK(NUPTRA+1))=PCTOP5(JOBS)
        ROBSU(JOBS,MDB_CAMT_AT_CLOUD_SINK(NUPTRA+1))=PCAMT5(JOBS)
      ENDIF
    ELSE
      IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
        PCTOP5(JOBS)=TOVSCVX5(IPOSM,2)
        PCAMT5(JOBS)=TOVSCVX5(IPOSM,3)
    ENDIF
    IF(KIND /= 4) THEN
      ROBSU(JOBS,MDB_CTOPINC_AT_CLOUD_SINK)=PCTOP5(JOBS)                
      ROBSU(JOBS,MDB_CAMTINC_AT_CLOUD_SINK)=PCAMT5(JOBS)                   
    ENDIF
  ENDDO
ENDIF


!*          1.2a   TOVS control variable, CO2.

IF(LTOVSCV .AND. LLRAD1C .AND. LCO2) THEN

  CALL ABOR1('HRADP_ML: CO2 as sink variable and internal RTTOV interpolation not implemented.')

ENDIF

!*          1.3   Replace ozone with climate, if not avalable as model field.

IF(.NOT.LGOMUA(IOBSTYPE,YGOMV%MO3)) THEN
  CALL ABOR1('HRADP_ML: No ozone and internal RTTOV interpolation not implemented')
ENDIF


!*          1.6   Perform the post-processing of 2M temperature and humidity
!                 and surface pressure
!                 A)  T2M is taken equal to T(NFLEVG): Lowest model level,
!                 B) HU2M is taken equal to HU(NFLEVG): Lowest model level.

PT2M5(1:KLEN)=PTF5(1:KLEN,NFLEVG)
PQ2M5(1:KLEN)=PQF5(1:KLEN,NFLEVG)
PPS5(1:KLEN)=PPRESH5(1:KLEN,NFLEVG)


IF (LHOOK) CALL DR_HOOK('HRADP_ML',1,ZHOOK_HANDLE)
END SUBROUTINE HRADP_ML
