SUBROUTINE HRADP_ML_AD(KDLEN,KLEN,KSET,&
 & PPRESH ,PTF ,PQF,&
 & PPS ,PTS ,PT2M ,PQ2M ,PCTOP , PCAMT ,&
 & PTS5,PTS5_REF)  

!****  HRADP_ML_AD  - PREPARATION FOR THE RADIANCE INTERPOLATION

!     ERIK ANDERSSON          01/04/99       ECMWF

!     PURPOSE
!     -------

!        HRADP_ML_AD PREPARES INTERPOLATION FOR ONE SET OF OBSERVATIONS
!        IF THE RT INPUT IS ON MODEL LEVELS.
!**   INTERFACE
!     ---------

!        CALL HRADP_ML_AD(KDLEN,KLEN,KSET,&
!    &PPRESH ,PTF ,PQF ,&
!    &PPS ,PTS ,PT2M ,PQ2M ,&
!    &PTS5,PTS5_REF)

!        Where KDLEN   = First dimension (Number of observations)
!              KLEN    = Actual number of observations
!              KSET    = Number (numero) of the obs set to be treated
!              PTF5    = Model level temperature
!              PTS5    = Surface skin temperature
!              PTS5_REF= Surface skin temperature-reference value

!        HRADP_ML_AD IS CALLED FROM HOPAD

!**   METHOD
!     ------

!**   EXTERNALS
!     ---------

!**   MODIFICATIONS
!     -------------
!     E.Andersson 990104 : Original
!     N. Bormann  20071215 : Renamed and revamped for RTTOV-9
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE GOMS_MIX     , ONLY : LGOMUA,YGOMV
USE YOMDIM   , ONLY : NFLEVG
USE YOMTVRAD , ONLY : SATGRP_TABLE, TOVSCVX
USE YOMVAR   , ONLY : LTOVSCV, LCO2, LCLDSINK
USE YOMCOCTP , ONLY : NGTHRB   
USE YOMOBSET , ONLY : MTYPOB
USE YOMDB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSET 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPRESH(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPS(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS(KDLEN),PCTOP(KDLEN),PCAMT(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT2M(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ2M(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5_REF(KDLEN) 

INTEGER(KIND=JPIM) :: ICDTYP_TOVS,IGRP,IOBSTYPE
INTEGER(KIND=JPIM) :: JOBS
INTEGER(KIND=JPIM) :: IPOSM
LOGICAL :: LLFAIL,LLRAD1C
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "fcobs.h"

#include "openmp_obs.h"

! ---------------------------------------------------------------------
!*          1.1   Initiate, set switches

IF (LHOOK) CALL DR_HOOK('HRADP_ML_AD',0,ZHOOK_HANDLE)
IOBSTYPE = MOD(MTYPOB(KSET),100)
IGRP =MOD(MTYPOB(KSET)/100,100)
ICDTYP_TOVS=SATGRP_TABLE(IGRP)%codetype

LLRAD1C= ICDTYP_TOVS == NGTHRB
LLFAIL=.FALSE.


!*          1.6   Perform the post-processing of 2M temperature and humidity
!                 and surface pressure

PTF(1:KLEN,NFLEVG) = PTF(1:KLEN,NFLEVG) + PT2M(1:KLEN)
PQF(1:KLEN,NFLEVG) = PQF(1:KLEN,NFLEVG) + PQ2M(1:KLEN)
PPRESH(1:KLEN,NFLEVG) = PPRESH(1:KLEN,NFLEVG) + PPS(1:KLEN)

IF(.NOT.(LTOVSCV.AND.LLRAD1C)) THEN
  PTS5(1:KLEN)=PTS5_REF(1:KLEN)
  PTS (1:KLEN)=0.0_JPRB
ENDIF

!*          1.3   Replace ozone with climatology, if not avalable as
!                 model field.

IF(.NOT.LGOMUA(IOBSTYPE,YGOMV%MO3)) THEN
  CALL ABOR1('HRADP_ML_AD: No ozone and internal RTTOV interpolation not implemented')
ENDIF

IF(KSET < 0) GOTO 4

!*          1.2   Fill the TOVSCVX array with the grad with respect to Ts

IF(LTOVSCV .AND. LLRAD1C) THEN
  DO JOBS =1,KLEN
    IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
    TOVSCVX(IPOSM,1)=PTS(JOBS)
    PTS(JOBS)=0.0_JPRB
  ENDDO
ENDIF

!*          1.2a   Fill the TOVSCVX array with the grad with respect to cloud sink

IF(LTOVSCV .AND. LLRAD1C .AND. LCLDSINK) THEN

  DO JOBS =1,KLEN
    IPOSM  = ROBHDR(JOBS,MDB_MAPOMM_AT_INDEX)
      TOVSCVX(IPOSM,2)=PCTOP(JOBS)
      TOVSCVX(IPOSM,3)=PCAMT(JOBS)
      PCTOP(JOBS)=0.0_JPRB
      PCAMT(JOBS)=0.0_JPRB
  ENDDO
ENDIF


!*          1.2a   Fill the TOVSCVX array with the grad with respect to CO2

IF(LTOVSCV .AND. LLRAD1C .AND. LCO2) THEN
  CALL ABOR1('HRADP_ML_AD: CO2 as sink variable and internal RTTOV interpolation not implemented.')
ENDIF

4 CONTINUE


IF (LHOOK) CALL DR_HOOK('HRADP_ML_AD',1,ZHOOK_HANDLE)
END SUBROUTINE HRADP_ML_AD
