SUBROUTINE ISO_GEMS(PTAMOY,PTRAY,PIZMOY,PBETAL,PRM,PGB,PPSL,PTODAER,PTODRAY,   &
&                                                                     PXF)

!****ISO*** COMPUTES ATMOSPHERIC TRANSMISSION


!     INTERFACE
!     ---------
! ISO IS CALLED BY SCATRA

!     INPUTS
!     ------

! PRM(-KMU:KMU)       : GAUSSIAN ANGLES FOR THE ZENITH
! PGB(-KMU:KMU)       : GAUSSIAN WEIGHTS FOR THE ZENITH
! PTAMOY(KNT)         : TRUNCATED AEROSOL OPTICAL THICKNESS PROFILE.
! PTRAY(KNT)          : MOLECULAR OPTICAL THICKNESS PROFILE.
! PIZMOY(KNT)         : TRUNCATED AEROSOL SINGLE SCATTERING ALBEDO PROFILE.
! PBETAL(KNT,KNANG+1) : COEFFICIENTS OF LEGENDRE POLYNOM
! PPSL(0:KNANG,-1:KNANG,-KMU:KMU)
! PTODAER             : TOTAL AEROSOL OPTICAL DEPTH
! PTODRAY             : TOTAL MOLECULAR OPTICAL DEPTH

!     OUTPUTS
!     -------

! PXF(-1,1) : ATMOSPHERIC TRANSMISSION 

!     EXTERNALS.
!     ----------
!          KERNEL_PBP

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007
!          A.BENEDETTI - ECMWF - 06.08.2008

!     SOURCE
!     ------
! ADAPTED FROM ISO.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE RT6SVALUES

IMPLICIT NONE

REAL(KIND=JPRB),INTENT(IN) :: PRM(-KMU:KMU)   ! GAUSSIAN ANGLES FOR THE ZENITH
REAL(KIND=JPRB),INTENT(IN) :: PGB(-KMU:KMU)   ! GAUSSIAN WEIGHTS FOR THE ZENITH
REAL(KIND=JPRB),INTENT(IN) :: PTAMOY(0:KNT)     ! TRUNCATED AEROSOL OPTICAL THICKNESS PROFILE.
REAL(KIND=JPRB),INTENT(IN) :: PTRAY(0:KNT)      ! MOLECULAR OPTICAL THICKNESS PROFILE.
REAL(KIND=JPRB),INTENT(IN) :: PIZMOY(0:KNT)     ! TRUNCATED AEROSOL SINGLE SCATTERING ALBEDO PROFILE.
REAL(KIND=JPRB),INTENT(IN) :: PBETAL(0:KNT,0:KNANG), PTODAER, PTODRAY
REAL(KIND=JPRB),INTENT(IN) :: PPSL(0:KNANG,-1:KNANG,-KMU:KMU)
REAL(KIND=JPRB),INTENT(OUT) :: PXF(-1:1)

! LOCAL VARIABLES
! ---------------
!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KNP=49, KNANG=80, KNT=20
INTEGER(KIND=JPIM) :: JLEV, JMU, JMU2, IG
REAL(KIND=JPRB) :: ZH(0:KNT), ZYDEL(0:KNT), ZXDEL(0:KNT)
REAL(KIND=JPRB) :: ZAAA, ZRON, ZBETA0, ZBETA2
REAL(KIND=JPRB) :: ZI1(0:KNT,-KMU:KMU), ZI2(0:KNT,-KMU:KMU)
REAL(KIND=JPRB) :: ZI3(-KMU:KMU), ZIN(-KMU:KMU)
REAL(KIND=JPRB) :: ZINM1(-KMU:KMU), ZINM2(-KMU:KMU)
REAL(KIND=JPRB) :: ZII1, ZII2, ZBPJK, ZBPJMK, ZXI1, ZXI2
REAL(KIND=JPRB) :: Z, ZX, ZXX, ZXPK, ZY, ZYY
REAL(KIND=JPRB) :: ZA1, ZD1, ZG1, ZY1, ZA, ZB, ZC, ZD, ZF, ZZI1
REAL(KIND=JPRB) :: ZYPK, ZXPJ, ZRO, ZRO0, ZRO1, ZRO2, ZAUX
! OUTPUT OF KERNEL
REAL(KIND=JPRB) :: z_PXPL(-KMU:KMU)
REAL(KIND=JPRB) :: z_PBP(0:KNT,0:KMU,-KMU:KMU)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "kernel_pbp.intfb.h"

IF (LHOOK) CALL DR_HOOK('ISO_GEMS',0,ZHOOK_HANDLE)

! VARIABLE INITIALIZATION 
ZIN(:)=0.0_JPRB
ZI3(:)=0.0_JPRB
ZI1(:,:)=0.0_JPRB
ZI2(:,:)=0.0_JPRB
ZINM1(:)=0.0_JPRB
ZINM2(:)=0.0_JPRB
PXF(:)=0.0_JPRB


! CASE 1: PURE RAYLEIGH
! -------

IF ((PTODAER <= ACCU2).AND.(PTODRAY > PTODAER)) THEN
  ZAUX=0.0_JPRB
  DO JLEV=0,KNT ! JLEV=0 CORRESPONDS TO TOP OF THE ATMOSPHERE
    ZAUX=ZAUX+PTRAY(JLEV)
    ZH(JLEV)=ZAUX
    ZYDEL(JLEV)=1.0_JPRB
    ZXDEL(JLEV)=0.0_JPRB
  ENDDO
ENDIF

! CASE 2: PURE AEROSOL
! -------

IF ((PTODRAY <= ACCU2).AND.(PTODAER > PTODRAY)) THEN
  ZAUX=0.0_JPRB
  DO JLEV=0,KNT
    ZAUX=ZAUX+PTAMOY(JLEV)
    ZH(JLEV)=ZAUX
    ZYDEL(JLEV)=0.0_JPRB
    ZXDEL(JLEV)=PIZMOY(JLEV)
  ENDDO
ENDIF

! CASE 3: MIXTURE
! -------

IF ((PTODRAY > ACCU2).AND.(PTODAER > ACCU2)) THEN
  ZAUX=0.0_JPRB
  DO JLEV=0,KNT
    ZAUX=ZAUX+PTAMOY(JLEV)+PTRAY(JLEV)
    ZH(JLEV)=ZAUX
    ZXDEL(JLEV)=PIZMOY(JLEV)*PTAMOY(JLEV)/ZH(JLEV)
    ZYDEL(JLEV)=PTRAY(JLEV)/ZH(JLEV)
  ENDDO
ENDIF

! INCIDENT ANGLE MU_S

ZAAA=DELTA/(2.0_JPRB-DELTA)
ZRON=(1.0_JPRB-ZAAA)/(1.0_JPRB+2.0_JPRB*ZAAA)

! RAYLEIGH PHASE FUNCTION

ZBETA0=1.0_JPRB
ZBETA2=0.5_JPRB*ZRON


! PRIMARY SCATTERING

IG=1
ZRO2=0.0_JPRB
ZRO1=0.0_JPRB
ZRO0=0.0_JPRB
ZRO=0.0_JPRB
ZI3(:)=0.0_JPRB

! KERNEL COMPUTATIONS

  CALL KERNEL_PBP(0,PPSL,PBETAL,z_PXPL,z_PBP)

ZI2(:,:)=0.0_JPRB



! VERTICAL INTEGRATION, PRIMARY UPWARD RADIATION

DO JMU=1,KMU
  ZI1(KNT,JMU)=1.0_JPRB
  ZZI1=ZI1(KNT,JMU)
  ZYY=PRM(JMU)
  DO JLEV=KNT-1,0,-1
    ZI1(JLEV,JMU)=EXP(-(PTODAER+PTODRAY-ZH(JLEV))/ZYY)
    IF(ZI1(JLEV,JMU) < 1.E-14) ZI1(JLEV,JMU) = 0.0_JPRB
  ENDDO
ENDDO


! VERTICAL INTEGRATION, PRIMARY DOWNWARD RADIATION


DO JMU=-KMU,-1
  DO JLEV=0,KNT
    ZI1(JLEV,JMU)=0.0_JPRB
  ENDDO
ENDDO

! ZINM2 IS INITIALIZED WITH SCATTERING COMPUTED AT N-2
! ZI3 IS INITIALIZED WITH PRIMARY SCATTERING


DO JMU=-KMU,-1
  ZINM1(JMU)=ZI1(KNT,JMU)
  ZINM2(JMU)=ZI1(KNT,JMU)
  ZI3(JMU)=ZI1(KNT,JMU)
ENDDO
DO JMU=1,KMU
  ZINM1(JMU)=ZI1(0,JMU)
  ZINM2(JMU)=ZI1(0,JMU)
  ZI3(JMU)=ZI1(0,JMU)
ENDDO
ZRO2=ZI1(0,KMU)
ZRO=ZI1(0,KMU)


! LOOP ON SUCCESSIVE ORDER

DO IG=2,NORD 

! SUCCESSIVE ORDERS

! MULTIPLE SCATTERING SOURCE FUNCTION AT EVERY LEVEL WITHIN THE LAYER

  DO JMU=1,KMU
    ZXPK=z_PXPL(JMU)
    ZYPK=z_PXPL(-JMU)
    DO JLEV=0,KNT
      ZII1=0.0_JPRB
      ZII2=0.0_JPRB
      ZX=ZXDEL(JLEV)
      ZY=ZYDEL(JLEV)
      DO JMU2=1,KMU
        ZXPJ=z_PXPL(JMU2)
        Z=PGB(JMU2)
        ZXI1=ZI1(JLEV,JMU2)
        ZXI2=ZI1(JLEV,-JMU2)
        ZBPJK=z_PBP(JLEV,JMU2,JMU)*ZX+ZY*(ZBETA0+ZBETA2*ZXPJ*ZXPK)
        ZBPJMK=z_PBP(JLEV,JMU2,-JMU)*ZX+ZY*(ZBETA0+ZBETA2*ZXPJ*ZYPK)
        ZII2=ZII2+Z*(ZXI1*ZBPJK+ZXI2*ZBPJMK)
        ZII1=ZII1+Z*(ZXI1*ZBPJMK+ZXI2*ZBPJK)
      ENDDO
      ZI2(JLEV,JMU)=ZII2
      ZI2(JLEV,-JMU)=ZII1      
    ENDDO
  ENDDO

! VERTICAL INTEGRATION, UPWARD RADIATION

  DO JMU=1,KMU
    ZI1(KNT,JMU)=0.0_JPRB
    ZZI1=ZI1(KNT,JMU)
    ZYY=PRM(JMU)
    DO JLEV=KNT-1,0,-1
      ZF=ZH(JLEV+1)-ZH(JLEV)
      ZA=(ZI2(JLEV+1,JMU)-ZI2(JLEV,JMU))/ZF
      ZB=ZI2(JLEV,JMU)-ZA*ZH(JLEV)
      ZC=EXP(-ZF/ZYY)
      ZD=1.D+00-ZC
      ZXX=ZH(JLEV)-ZH(JLEV+1)*ZC
      ZZI1=ZC*ZZI1+(ZD*(ZB+ZA*ZYY)+ZA*ZXX)*0.5_JPRB
      ZI1(JLEV,JMU)=ZZI1
    ENDDO
  ENDDO

! VERTICAL INTEGRATION, DOWNWARD RADIATION

  DO JMU=-KMU,-1
    ZI1(0,JMU)=0.0_JPRB
    ZZI1=ZI1(0,JMU)
    ZYY=PRM(JMU)
    DO JLEV=1,KNT
      ZF=ZH(JLEV)-ZH(JLEV-1)
      ZC=EXP(ZF/ZYY)
      ZD=1.D+00-ZC
      ZA=(ZI2(JLEV,JMU)-ZI2(JLEV-1,JMU))/ZF
      ZB=ZI2(JLEV,JMU)-ZA*ZH(JLEV)
      ZXX=ZH(JLEV)-ZH(JLEV-1)*ZC
      ZZI1=ZC*ZZI1+(ZD*(ZB+ZA*ZYY)+ZA*ZXX)*0.5_JPRB
      ZI1(JLEV,JMU)=ZZI1
    ENDDO
  ENDDO

! IN IS THE NIEME SCATTERING ORDER

  DO JMU=-KMU,-1
    ZIN(JMU)=ZI1(KNT,JMU)
  ENDDO
  DO JMU=1,KMU
    ZIN(JMU)=ZI1(0,JMU)
  ENDDO
  ZRO0=ZI1(0,KMU)

! CONVERGENCE TEST (GEOMETRICAL SERIE)

  IF (IG > 2) THEN
    Z=0.0_JPRB
    ZA1=ZRO2
    ZD1=ZRO1
    ZG1=ZRO0
    IF ((ZA1 >= ACCU).AND.(ZD1 >= ACCU).AND.(ZRO >= ACCU)) THEN
      ZY=((ZG1/ZD1-ZD1/ZA1)/((1.-ZG1/ZD1)**2)*(ZG1/ZRO))
      ZY=ABS(ZY)
      Z=MAX(Z,ZY)
    ENDIF
    DO JMU=-KMU,-1
      ZA1=ZINM2(JMU)
      ZD1=ZINM1(JMU)
      ZG1=ZIN(JMU)
      IF ((ZA1 /= 0.0_JPRB).AND.(ZD1 /= 0.0_JPRB).AND.(ZI3(JMU) /= 0.0_JPRB)) THEN
        ZY=((ZG1/ZD1-ZD1/ZA1)/((1.-ZG1/ZD1)**2)*(ZG1/ZI3(JMU)))
        ZY=ABS(ZY)
        Z=MAX(ZY,Z)
      ENDIF
    ENDDO
    DO JMU=1,KMU
      ZA1=ZINM2(JMU)
      ZD1=ZINM1(JMU)
      ZG1=ZIN(JMU)
      IF ((ZA1 /= 0.0_JPRB).AND.(ZD1 /= 0.0_JPRB).AND.(ZI3(JMU) /= 0.0_JPRB)) THEN
        ZY=((ZG1/ZD1-ZD1/ZA1)/((1.-ZG1/ZD1)**2)*(ZG1/ZI3(JMU)))
        ZY=ABS(ZY)
        Z=MAX(ZY,Z)
      ENDIF
    ENDDO
    IF (Z < ACCU4) THEN

! SUCCESSFUL TEST (GEOMETRICAL SERIE)
   
      DO JMU=-KMU,-1
        ZY1=1.0_JPRB
        ZD1=ZINM1(JMU)
        ZG1=ZIN(JMU)
        IF (ZD1 /= 0.0_JPRB) THEN
          ZY1=1.0_JPRB-ZG1/ZD1
          ZG1=ZG1/ZY1
          ZI3(JMU)=ZI3(JMU)+ZG1
        ENDIF
      ENDDO
      DO JMU=1,KMU
        ZY1=1.0_JPRB
        ZD1=ZINM1(JMU)
        ZG1=ZIN(JMU)
        IF (ZD1 /= 0.0_JPRB) THEN
          ZY1=1.0_JPRB-ZG1/ZD1
          ZG1=ZG1/ZY1
          ZI3(JMU)=ZI3(JMU)+ZG1
        ENDIF
      ENDDO
      ZD1=ZRO1
      ZG1=ZRO0
      ZY=1.0_JPRB
      IF (ZD1 >= ACCU) THEN
        IF (ABS(ZG1-ZD1) >= ACCU) THEN
          ZY1=1.0_JPRB-ZG1/ZD1
          ZG1=ZG1/ZY1
        ENDIF
        ZRO=ZRO+ZG1
      ENDIF
!      EXIT 
    ELSE

! ZINM2 IS THE (N-2)ieme SCATTERING ORDER

     DO JMU=-KMU,KMU
        ZINM2(JMU)=ZINM1(JMU)
      ENDDO
      ZRO2=ZRO1
    ENDIF
  ENDIF
  IF ((IG <= 2).OR.(Z >= ACCU4)) THEN

! ZINM1 IS THE (N-1)ieme SCATTERING ORDER

    DO JMU=-KMU,KMU
      ZINM1(JMU)=ZIN(JMU)
    ENDDO
    ZRO1=ZRO0
! SUM OF THE N-1 ORDERS

    DO JMU=-KMU,KMU
      ZI3(JMU)=ZI3(JMU)+ZIN(JMU)
    ENDDO
    ZRO=ZRO+ZRO0
!!
!! STOP IF ORDER N IS LESS THAN 1% OF THE SUM
!!
!    Z=0.0_JPRB
!    DO JMU=-KMU,KMU
!      IF (ZI3(JMU) /= 0.0_JPRB) THEN
!        ZY=ABS(ZIN(JMU)/ZI3(JMU))
!        Z=MAX(Z,ZY)
!      ENDIF
!    ENDDO
!    IF ((Z < ACCU5).OR.(IG > SCATORD)) THEN
!      EXIT
!    ENDIF
  ENDIF
ENDDO  ! END LOOP ON SUCCESSIVE ORDERS 
PXF(1)=PXF(1)+ZI3(KMU)
PXF(-1)=ZRO
DO JMU=1,KMU
  PXF(0)=PXF(0)+PRM(JMU)*PGB(JMU)*ZI3(-JMU)
ENDDO

IF (LHOOK) CALL DR_HOOK('ISO_GEMS',1,ZHOOK_HANDLE)

END SUBROUTINE ISO_GEMS

