SUBROUTINE LAIDDIOBS(KPROMA,KPROMB,KSTART,KPROF,KLEV,KFLEV,KFLDN,KFLDX,&
 & PCLA,PDLO,PCLO,KL0,PXSL,PXF)  

!**** *LAIDDIOBS-  Observation interpolator:
!                  BI-dimensional 12 points interpolations (adjoint)

!     Purpose.
!     --------
!       Performs bi-dimensional 12 points interpolations.
!       Version designed for observation interpolator.
!       Differences with LAIDDI are the following ones:
!        - one level treated only.
!        - the position of the interpolation point and the weights
!          are considered as constants.

!**   Interface.
!     ----------    
!        *CALL* *LAIDDIOBS(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension for grid-point quantities.
!          KPROMB  - horizontal dimension for interpolation point
!                    quantities.
!          KSTART  - first element of arrays where
!                    computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KFLDN   - number of the first field.
!          KFLDX   - number of the last field.
!          PDLAT   - latitudinal distance
!          PCLA    - weights (distances) for horizontal cubic interpolation
!                    on a same latitude.
!          PDLO    - distances for horizontal linear interpolations 
!                    in longitude (latitude rows 0, 3)
!          PCLO    - weights for horizontal cubic interpolations in longitude
!                    (latitude rows 1, 2)
!          KL0     - indices of the four western points
!                    of the 16 points interpolation grid.
!          PXSL    - quantity to be interpolated.

!        OUTPUT:
!          PXF     - interpolated variable.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.
!        Called by SLINT.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD, after the subroutine LAGINT2
!        written by Maurice IMBARD, Alain CRAPLET and Michel ROCHAS
!        METEO-FRANCE, CNRM/GMAP.

!     Modifications.
!     --------------
!        Original : FEBRUARY 1992.
!        Modification : JANV 1993: K. YESSAD
!                       (set up interpolated term to 0 if useless).
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        F. Vana    20-Aug-2008 adapted to weights driven interp. data-flow
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLA(KPROMB,3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO(KPROMB,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLO(KPROMB,3,2)
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMB,0:3,0:3) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL(KPROMA*(KFLDX-KFLDN+1)) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXF(KPROMB,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IP01, IP02, IP10, IP11, IP12, IP13, IP20,&
 & IP21, IP22, IP23, IP31, IP32, JROF  

REAL(KIND=JPRB) :: Z0, Z1, Z2, Z3

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAIDDIOBS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INTERPOLATIONS.
!              ---------------

DO JROF=KSTART,KPROF

  IP31 = KL0(JROF,3,1)+(KLEV-1)*KPROMA
  IP32 = KL0(JROF,3,2)+(KLEV-1)*KPROMA
  IP20 = KL0(JROF,2,0)+(KLEV-1)*KPROMA
  IP21 = KL0(JROF,2,1)+(KLEV-1)*KPROMA
  IP22 = KL0(JROF,2,2)+(KLEV-1)*KPROMA
  IP23 = KL0(JROF,2,3)+(KLEV-1)*KPROMA
  IP10 = KL0(JROF,1,0)+(KLEV-1)*KPROMA
  IP11 = KL0(JROF,1,1)+(KLEV-1)*KPROMA
  IP12 = KL0(JROF,1,2)+(KLEV-1)*KPROMA
  IP13 = KL0(JROF,1,3)+(KLEV-1)*KPROMA
  IP01 = KL0(JROF,0,1)+(KLEV-1)*KPROMA
  IP02 = KL0(JROF,0,2)+(KLEV-1)*KPROMA

  ! interpolations in longitude
  Z0=PXSL(IP01) + PDLO(JROF,0) * ( PXSL(IP02)-PXSL(IP01) )
  Z1=PXSL(IP10)&
   & +PCLO(JROF,1,1)*( PXSL(IP11)-PXSL(IP10) )&
   & +PCLO(JROF,2,1)*( PXSL(IP12)-PXSL(IP10) )&
   & +PCLO(JROF,3,1)*( PXSL(IP13)-PXSL(IP10) )  
  Z2=PXSL(IP20)&
   & +PCLO(JROF,1,2)*( PXSL(IP21)-PXSL(IP20) )&
   & +PCLO(JROF,2,2)*( PXSL(IP22)-PXSL(IP20) )&
   & +PCLO(JROF,3,2)*( PXSL(IP23)-PXSL(IP20) )  
  Z3=PXSL(IP31) + PDLO(JROF,3) * ( PXSL(IP32)-PXSL(IP31) )

  ! final interpolation in latitude
  PXF(JROF,KLEV)= Z0 &
     & +PCLA(JROF,1)*(Z1-Z0) &
     & +PCLA(JROF,2)*(Z2-Z0) &
     & +PCLA(JROF,3)*(Z3-Z0)

ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAIDDIOBS',1,ZHOOK_HANDLE)
END SUBROUTINE LAIDDIOBS
