SUBROUTINE LAIDLIOBS(KPROMA,KPROMB,KSTART,KPROF,KLEV,KFLEV,KFLDN,KFLDX,&
 & PDLAT,PDLO,KL0,PXSL,PXF)  

!**** *LAIDLIOBS- Observation interpolator: 
!                 Bilinear horizontal interpolations for one variable.

!     Purpose.
!     --------
!       Performs bilinear horizontal interpolations for one variable.
!       Version designed for observation interpolator.
!       Differences with LAIDLI are the following ones:
!        - one level treated only.
!        - the position of the interpolation point and the weights
!          are considered as constants.

!**   Interface.
!     ----------
!        *CALL* *LAIDLIOBS(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension for grid-point quantities.
!          KPROMB  - horizontal dimension for interpolation point
!                    quantities.
!          KSTART  - first element of arrays where
!                    computations are performed.
!          KPROF   - depth of work.
!          KFLEV   - vertical dimension.
!          KFLDN   - number of the first field.
!          KFLDX   - number of the last field.
!          PDLAT   - weight (distance) for horizontal linear interpolation
!                    on a same latitude.
!          PDLO    - weights (distances) for horizontal linear interpolation
!                    on a same longitude.
!          KL0     - indices of the four western points
!                    of the 16 points interpolation grid.
!          PXSL    - quantity to be interpolated.

!        OUTPUT:
!          PXF     - interpolated variable.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        No external.
!        Called by SLINT.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD, after the subroutines LAGINL3
!        written by Maurice IMBARD, Alain CRAPLET and Michel ROCHAS
!        METEO-FRANCE, CNRM/GMAP.

!     Modifications.
!     --------------
!        Original : FEBRUARY 1992.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.Yessad (Sept 2008): correct false comments + cleanings.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLAT(KPROMB) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLO(KPROMB,1:2) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KL0(KPROMB,2,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXSL(KPROMA*(KFLDX-KFLDN+1)) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXF(KPROMB,KFLEV) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IP11, IP12, IP21, IP22, JROF

REAL(KIND=JPRB) :: ZDLAT, ZDLO1, ZDLO2, ZVALLO1, ZVALLO2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LAIDLIOBS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INTERPOLATIONS.
!              ---------------

DO JROF=KSTART,KPROF

!     Computation of coordinates and distances.

  ZDLAT =PDLAT(JROF)

  ZDLO1 =PDLO(JROF,1)
  ZDLO2 =PDLO(JROF,2)

  IP11 = KL0(JROF,1,1)+(KLEV-1)*KPROMA
  IP12 = KL0(JROF,1,2)+(KLEV-1)*KPROMA
  IP21 = KL0(JROF,2,1)+(KLEV-1)*KPROMA
  IP22 = KL0(JROF,2,2)+(KLEV-1)*KPROMA

!     Interpolation.

  ZVALLO1=PXSL(IP11) + ZDLO1*( PXSL(IP12)-PXSL(IP11) )
  ZVALLO2=PXSL(IP21) + ZDLO2*( PXSL(IP22)-PXSL(IP21) )
  PXF(JROF,KLEV)= ZVALLO1 + ZDLAT*(ZVALLO2-ZVALLO1)

ENDDO

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('LAIDLIOBS',1,ZHOOK_HANDLE)
END SUBROUTINE LAIDLIOBS
