SUBROUTINE MEANUV_WEIGHTS(KDLEN,KLEN,KDCOUNT,KLEV,CD_COBS_OPER,PTAU,PVPOBS,&
 & PPRESF5,PSIGMA,PW5)  

!**** *MEANUV_WEIGHTS*   Calculate weights for SATOB/AMV obs operator

!  N. Bormann     19/02/2001    ECMWF

!     PURPOSE
!     -------

!     Calculates the weights used in the layer-mean observation 
!     operator H for SATOB/AMVs. The observation operator calculates the
!     integral of f * v, with v the wind profile, and f a suitable weighting
!     function. The weights PW5 are calculated so that H can be written
!     as H(V) = DOT_PRODUCT(PW5*V) in discretised form. 

!**   INTERFACE
!     ---------
!     *CALL* *MEANUV_WEIGHTS
!     WHERE KDLEN  = First dimension: Max number of members in set
!           KLEN   = Actual number of members
!           KDCOUNT = Second dimension (Data per observations)
!           KLEV   = Number of pressure levels
!           COBS_OPER= Method used for weight calculation
!           PTAU   = Transmittances on RTTOV levels
!           PVPOBS = SATOB/AMV pressure
!           PPRESF5  = Pressure on model or RTTOV levels
!           PSIGMA = Sigma used for Gaussian
!           PW5     = Weights (output)

!     MEANUV_WEIGHTS is called from HOP, HOPTL, HOPAD

!**   METHOD
!     ------
!     Gauss average:
!     Use a Gaussian function to calculate weights. Reduce weight
!     linearly to zero between 200 and 100 hPa, as there won't be
!     contribution above 100 hPa in SATOB/AMVs.
!
!     Boxcar average:
!     Uses a boxcar weighting function that is a constant between 
!     ZPAMVTOP and ZPAMVBOT, and zero elsewhere.
!
!     RT average:
!     Calculate weights corresponding to numerical approximation of
!     radiance integral (integral( B(T(tau)) * dtau )), from 
!     transmittances tau.

!**   EXTERNALS
!     --------- 

!**   MODIFICATIONS
!     -------------
!     Niels Bormann   2001/02/19: Original
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     N. Bormann      2009/10/09: Boxcar average for AMVs
!----------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE yomlun   , ONLY : NULERR

IMPLICIT none

! Arguments
INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN ! 1st dimension, Max number of members in set
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN ! 1st dimension, number of obs
INTEGER(KIND=JPIM),INTENT(IN)    :: KDCOUNT ! Data per observations
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV ! Number of pressure levels
CHARACTER(LEN=*)  ,INTENT(IN)    :: CD_COBS_OPER ! Method used for weight calculation
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAU(KDLEN,KDCOUNT,KLEV) ! Transmittances
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVPOBS(KDLEN,KDCOUNT) ! Pressure of obs
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPRESF5(KDLEN,KLEV) ! Pres. of model/RTTOV levels
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSIGMA ! Sigma of Gaussian [Pa]
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PW5(KDLEN,KLEV) ! Weights
! Local parameters 
REAL(KIND=JPRB)       :: ZMXNSIGMA = 3._JPRB

! Local variables
REAL(KIND=JPRB)       :: ZSIGMA
REAL(KIND=JPRB)       :: ZPDELTA5
REAL(KIND=JPRB)       :: ZPAMVTOP, ZPAMVBOT
INTEGER(KIND=JPIM)    :: JOBS
INTEGER(KIND=JPIM)    :: ILEV
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
!==================================================================
IF (LHOOK) CALL DR_HOOK('MEANUV_WEIGHTS',0,ZHOOK_HANDLE)
IF(KDCOUNT > 1)&
 & CALL ABOR1('MEANUV_WEIGHTS: MULTISPECTRAL WINDS NOT YET IMPLEMENTED')  

PW5 = 0.0_JPRB

SELECT CASE (TRIM(CD_COBS_OPER))

!    1      Calculate weights for simple Gauss averaging
!           ============================================

CASE ('Gauss average')
  DO JOBS=1, KLEN
      ! Adjust sigma (for lower levels)
    ZSIGMA = MAX(MIN(PSIGMA,(PPRESF5(JOBS,KLEV)-PVPOBS(JOBS,1))/ZMXNSIGMA), &
     & 1000._JPRB)  
      ! Calculate Gaussian
    PW5(JOBS,:) = EXP(-0.5_JPRB * ((PPRESF5(JOBS,:)-PVPOBS(JOBS,1))/ZSIGMA)**2)
      ! Reduce weights to 0 above 200 hPa
    PW5(JOBS,:) = PW5(JOBS,:)                    &
     & * MAX(0._JPRB,MIN(1._JPRB,0.0001_JPRB*(PPRESF5(JOBS,:)-10000._JPRB)))  
      ! Set weights to 0 for pressure levels outside ZMXNSIGMA ZSIGMAs 
      ! around p
    IF(ANY(ABS(PPRESF5(JOBS,:)-PVPOBS(JOBS,1)) <= ZMXNSIGMA*ZSIGMA))THEN
      WHERE(ABS(PPRESF5(JOBS,:)-PVPOBS(JOBS,1)) > ZMXNSIGMA*ZSIGMA) &
       & PW5(JOBS,:) = 0.0_JPRB  
    ENDIF
      ! Calculate integral over dp
    PW5(JOBS,2:KLEV-1) = PW5(JOBS,2:KLEV-1)     &
     & * ( PPRESF5(JOBS,3:KLEV) - PPRESF5(JOBS,1:KLEV-2) )  
    PW5(JOBS,1)    = PW5(JOBS,1) * ( PPRESF5(JOBS,2) - PPRESF5(JOBS,1) )
    PW5(JOBS,KLEV) = PW5(JOBS,KLEV) * ( PPRESF5(JOBS,KLEV) - PPRESF5(JOBS,KLEV-1) )
      ! Normalise weight
    IF(SUM(PW5(JOBS,:))> 0.0_JPRB)THEN
      PW5(JOBS,:) = PW5(JOBS,:) / SUM(PW5(JOBS,:))
    ELSE
      WRITE(nulerr,*) 'WARNING meanuv_weigths: Detected zero weights.'
      WRITE(nulerr,*) PW5(JOBS,:)
    ENDIF   
  ENDDO

!    2      Calculate weights for box averaging
!           ===================================

CASE ('Boxcar average')

  ! Uses a boxcar weighting function that is a constant between 
  ! ZPAMVTOP and ZPAMVBOT, and zero elsewhere.

  DO JOBS=1, KLEN

    ZPAMVTOP = PVPOBS(JOBS,1)
    ZPAMVBOT = PVPOBS(JOBS,1) + PSIGMA

    ! Search for pressure level just below ZPAMVTOP
    DO ILEV = 2, KLEV
      IF ( PPRESF5(JOBS,ILEV) > ZPAMVTOP )EXIT
    ENDDO 

    IF ( ILEV > KLEV .AND. PPRESF5(JOBS,KLEV) < ZPAMVTOP ) THEN

      ! ZPAMVTOP below lowest pressure level; just use value of lowest level

      PW5(JOBS,KLEV) = 1.0_JPRB

    ELSE

      ! Assign weights. To evaluate the integral, the wind is assumed to 
      ! vary linearly in pressure.

      ! First: Layer between ZPAMVTOP and model pressure just below:
      PW5(JOBS,ILEV-1) = (PPRESF5(JOBS,ILEV) - ZPAMVTOP)**2   &
       &               /(PPRESF5(JOBS,ILEV) - PPRESF5(JOBS,ILEV-1))
      PW5(JOBS,ILEV) = (PPRESF5(JOBS,ILEV)**2                       &
       &               - 2.0_JPRB*PPRESF5(JOBS,ILEV)*PPRESF5(JOBS,ILEV-1)   &
       &               + 2.0_JPRB*ZPAMVTOP*PPRESF5(JOBS,ILEV-1)           &
       &               - ZPAMVTOP**2)                            & 
       &              /(PPRESF5(JOBS,ILEV) - PPRESF5(JOBS,ILEV-1))
      ILEV = ILEV + 1
      IF( ILEV <= KLEV )THEN
        DO ILEV = ILEV, KLEV
          IF ( PPRESF5(JOBS,ILEV-1) < ZPAMVBOT ) THEN
            ! Layers between model levels
            PW5(JOBS,ILEV-1) = PW5(JOBS,ILEV-1) + PPRESF5(JOBS,ILEV) - PPRESF5(JOBS,ILEV-1)
            PW5(JOBS,ILEV)   = PPRESF5(JOBS,ILEV) - PPRESF5(JOBS,ILEV-1)
          ELSE
            EXIT
          ENDIF
        ENDDO
      ENDIF

      ILEV = ILEV - 1

      IF ( PPRESF5(JOBS,ILEV) > ZPAMVBOT ) THEN
        ! Last: Subtract layer between ZPAMVBOT and model level just below
        PW5(JOBS,ILEV-1) = PW5(JOBS,ILEV-1)                    &
         &      -  (PPRESF5(JOBS,ILEV) - ZPAMVBOT)**2        &
         &        /(PPRESF5(JOBS,ILEV) - PPRESF5(JOBS,ILEV-1))
!        PW5(JOBS,ILEV) = PW5(JOBS,ILEV)                        &
!         &      -  (PPRESF5(JOBS,ILEV)**2                      &
!         &          - 2.0_JPRB*PPRESF5(JOBS,ILEV)*PPRESF5(JOBS,ILEV-1)  &
!         &          + 2.0_JPRB*ZPAMVBOT*PPRESF5(JOBS,ILEV-1)            &
!         &          - ZPAMVBOT**2)                             &
!         &        /(PPRESF5(JOBS,ILEV) - PPRESF5(JOBS,ILEV-1))
! IGBG:: I.Genkova and N.Bormann November 2009
        PW5(JOBS,ILEV) = PW5(JOBS,ILEV)                                  &
         &      -  (   PPRESF5(JOBS,ILEV) -  PPRESF5(JOBS,ILEV-1)       &
         &      - ( PPRESF5(JOBS,ILEV-1) - ZPAMVBOT )**2               &
         &      /(PPRESF5(JOBS,ILEV) - PPRESF5(JOBS,ILEV-1)) ) 
! IGBG:: end of changes needed to avoid negative weight for numberical reasons

        ZPDELTA5 = PSIGMA
      ELSE
        ZPDELTA5 = PPRESF5(JOBS,ILEV)-ZPAMVTOP
      ENDIF

      IF( ZPDELTA5 > 0.0_JPRB)THEN
        PW5(JOBS,:) = PW5(JOBS,:) / (2.0_JPRB * ZPDELTA5)
      ELSE
        WRITE(nulerr,*) 'WARNING meanuv_weigths: Detected zero weights for box average.'
        WRITE(nulerr,*) PW5(JOBS,:)
      ENDIF   
    ENDIF
  ENDDO

!    3      Calculate weights from transmittances
!           (Note: PW5 are calculated so that we can write
!                  H(v-profile) = < PW5 , v-profile >)
!           ===================================================

CASE ('RT average','RT median','RT max','RT median RT average',&
     & 'RT max RT average','RT median Gauss average','RT max Gauss average')

  DO JOBS = 1,KLEN
    PW5(JOBS,2:KLEV-1) = PTAU(JOBS,1,1:KLEV-2) - PTAU(JOBS,1,3:KLEV)
    PW5(JOBS,1)    = PTAU(JOBS,1,1) - PTAU(JOBS,1,2)
    PW5(JOBS,KLEV) = PTAU(JOBS,1,KLEV-1) - PTAU(JOBS,1,KLEV)
    PW5(JOBS,:) = PW5(JOBS,:) / SUM(PW5(JOBS,:))
  ENDDO

END SELECT

IF(ANY(PW5 < 0.0_JPRB))THEN
  WRITE(NULERR,*) 'MEANUV_WEIGHTS: Negative weight:'
  DO JOBS = 1,KLEN
    IF(ANY(PW5(JOBS,:) < 0.0_JPRB))THEN
!      WRITE(NULERR,*) jobs, PW5(JOBS,:)
! IGBG
      WRITE(NULERR,*) jobs
      WRITE(NULERR,*) 'weights',PW5(JOBS,:)
      WRITE(NULERR,*) 'pamv',PVPOBS(JOBS,1)
      WRITE(NULERR,*) 'pamv+50',PVPOBS(JOBS,1) + PSIGMA
      WRITE(NULERR,*) 'pmod',PPRESF5(JOBS,:)
    ENDIF
  ENDDO
  CALL ABOR1('MEANUV_WEIGHTS: Negative weights for some AMVs')
ENDIF

IF (LHOOK) CALL DR_HOOK('MEANUV_WEIGHTS',1,ZHOOK_HANDLE)
END SUBROUTINE MEANUV_WEIGHTS
