FUNCTION MWIMAGER_LWP( K1D_Y, PTB_OBS, KSENS_ID, LD_MWAVE)

! Estimate cloud liquid water path (over oceans only), based on a simple 
! regression using the 22v and 37v channels, or nearest equivalents, 
! on a microwave imager.

! The form of this regression, and the parameters for SSMI, come from 
! Karstens, Simmer and Ruprecht (1994) in Meteorol. Atmos. Phys, 54, 
! p.157-171. The parameters for TMI, SSMIS and AMSRE were fitted by
! Niels Bormann using RTTOV simulated radiances from first guess fields. 
! As a result of the relatively large biases between first guess and
! observations, observations need to be given an ad-hoc bias correction
! before they can be used in a regression that has been tuned using the 
! first guess. 

! Changes
! -------

! 15-Sept-2007  Alan Geer      Initial version
! 02-Sept-2008  Alan Geer      Add test for RMDI
! 17-Sept-2008  Alan Geer      Allow use of different coefficients with
!                              all-sky 4D-Var of mw imagers

USE PARKIND1    , ONLY: JPIM, JPRB
USE YOMHOOK     , ONLY: LHOOK, DR_HOOK
USE YOMANCS     , ONLY: RMDI 
USE MWIMAGER_MIX , ONLY: RTBCONST, YCOEFF, GET_LWPCOEFF 
 
IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN) :: KSENS_ID ! The sensor ID number (SSMI=6 TMI=17 etc.)
INTEGER(KIND=JPIM), INTENT(IN) :: K1D_Y    ! Number of channels
REAL(KIND=JPRB),    INTENT(IN) :: PTB_OBS(K1D_Y) ! TB obs for a single profile. [K]
LOGICAL,            INTENT(IN) :: LD_MWAVE ! Flag to select mwave-specific coeffs
REAL(KIND=JPRB)                :: MWIMAGER_LWP   ! Liquid water path [kg m^-2]

TYPE(YCOEFF)    :: YLCOEFF
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZTB22V, ZTB37V

IF (LHOOK) CALL DR_HOOK('MWIMAGER_LWP',0,ZHOOK_HANDLE)

MWIMAGER_LWP = 0.0_JPRB
 
IF ( GET_LWPCOEFF(KSENS_ID, YLCOEFF, LD_MWAVE) ) THEN

  IF( PTB_OBS (YLCOEFF%I22V) /= RMDI .AND. PTB_OBS (YLCOEFF%I37V) /= RMDI ) THEN

    ZTB22V = PTB_OBS (YLCOEFF%I22V) + YLCOEFF%ZABIAS
    ZTB37V = PTB_OBS (YLCOEFF%I37V) + YLCOEFF%ZBBIAS

    IF (ZTB22V < RTBCONST .AND. ZTB37V < RTBCONST) THEN

      MWIMAGER_LWP = YLCOEFF%ZCONST + &
        & YLCOEFF%ZA * LOG(RTBCONST - ZTB22V) - &
        & YLCOEFF%ZB * LOG(RTBCONST - ZTB37V)

    ELSE

      ! Regression will fail given excessively high Tb. This might
      ! still indicate cloud or rain, however.
      MWIMAGER_LWP = RMDI

    ENDIF
  ELSE
    MWIMAGER_LWP = RMDI
  ENDIF
ELSE
  
  CALL ABOR1(' MWIMAGER_LWP: Unknown instrument') 

ENDIF

IF (LHOOK) CALL DR_HOOK('MWIMAGER_LWP',1,ZHOOK_HANDLE)

END FUNCTION MWIMAGER_LWP
