SUBROUTINE NOX2NO2(KDLEN,KLEN,KDCOUNT,CDPAR,KOBSTYPE,KCOUNT,PRESO,&
 & PRESH5,PRESF5,PXP5,PXPD5,&
 & PTF5,PSOE,PAL,PCOD5,PTAUC5,PCCF5,YDGEMSPROF5,&
 & PNO25)

!**** NOX2NO2 - POST-PROCESSING

!     D. VASILJEVIC          ECMWF           91-01-03

!     PURPOSE
!     -------

!        FIND OUT AN EQUIVALENT MODEL PARAMETER FOR A GIVEN OBSERVED.

!**   INTERFACE
!     ---------

!        CALL NOX2NO2(KDLEN,KLEN,KDCOUNT,CDPAR,KOBSTYPE,KCOUNT,PRESO,
!    I               PRESH5,PRESF5,PXP5,PXPD5,
!    I               PSOE,PAL,PCOD5,PTAUC5,PCCINT5,YDGEMSPROF5,
!    I               PNO25)
!     WHERE KDLEN   : FIRST DIMENSION (Observations)
!           KLEN    : ACTUAL NUMBER OF OBSERVATIONS
!           KDCOUNT : SECOND DIMENSION (Data)
!           CDPAR   : PARAMETER FOR POST-PROCESSING
!           KOBSTYPE: OBS. TYPE
!           KCOUNT  : COUNTER FOR THE PARTICULAR OBS. PARAMETER
!           PRESO   : OBSERVED PRESSURES
!           PRESH5  : HALF LEVEL PRESSURES
!           PRESF5  : FULL LEVEL PRESSURES
!           PXP5    : INTERMEDIATE ARRAY
!           PXPD5   : ..................
!           PTF5    : Model T  
!           PSOE    : SOLAR ELEVATION 
!           PAL     : ALBEDO
!           PCOD5    : Total cloud optical depth
!           PTAUC5   : Cloud optical depth per layer
!           PCCF5   : cloud cover
!           YDGEMSPROF5   : GEMS PROFILES


!        NOX2NO2 IS CALLED FROM HOP

!**   METHOD
!     ------

!        CONVERT MODEL NOX to NO2

!**   EXTERNALS
!     ---------

!        PPFLEV  - FIND PRESSURE LEVEL UNDER SPECIFIED PRESSURES

!**   MODIFICATIONS
!     -------------
!     910103 D.VASILJEVIC ORIGINAL
!     E.Andersson 92-06-22 : Introduce PWC
!     VC          96-01    : Cleaning
!     E.Andersson 96-02-07 : Remove 'V'-option
!     E.Andersson 97-11-14 : GOM-variables more flexible
!     E.Holm      98-05-08 : Introduce integrated ozone option (TO3)
!     E.Andersson 98-06-11 : Pass dimension as argument
!     F.Chevallier00-03    : Install cloud switch 'TVC'
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     J.M. Piriou 04-02    : Install cloud switch 'TQL'
!     N.Bormann   04-12-23 : New CDPAR-flavour 'RTL' for limb radiances
!     A.Dethof  04-Jan-2007  Add NOx operator for GRG
!     D.Tan       06-03-13 : Introduce HLoS (horizontal line-of-sight wind component)
!     A.Dethof  24-Jan-2008  new routine nox2no2 based on ppobsa

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPPM
USE YOMLUN   , ONLY : NULERR
USE TYPE_GEMS_PROFILES
USE YOM_YGFL , ONLY : NGRG,YGRG_NL
USE YOECLD   , ONLY : REPSEC
USE YOERDI   , ONLY : REPCLC

!     ------------------------------------------------------------------
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDCOUNT
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDPAR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOBSTYPE
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESO(KDLEN,KDCOUNT)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXP5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXPD5(KDLEN,0:NFLEVG,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOE(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAL(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCOD5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAUC5(KDLEN,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PNO25(KDLEN,0:NFLEVG) 
TYPE (TYPE_GEMSPROF), INTENT(IN), OPTIONAL :: YDGEMSPROF5(KDLEN,NGEMS)

!     ------------------------------------------------------------------

REAL(KIND=JPRB)   :: ZNOX5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   :: ZGO35(KDLEN,0:NFLEVG) 
INTEGER(KIND=JPIM), ALLOCATABLE :: ILEVB (:,:,:)
LOGICAL, ALLOCATABLE :: LLBELO(:,:),LLBLOW(:),LLBELS(:,:),LLBLES(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZLNPRESO(:,:)
INTEGER(KIND=JPIM) :: ILEN,IMXCNT,JLEN,JLEVP,JLEV,JGRG
INTEGER(KIND=JPIM) :: JGEMS,IND_GO3,IND_NOX
REAL(KIND=JPRB) :: ZCOL5(KDLEN,0:NFLEVG)
REAL(KIND=JPRB) :: ZJNO25(KDLEN,NFLEVG)


REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB), ALLOCATABLE :: ZCCINT5(:,:)

#include "abor1.intfb.h"
#include "gpnox.intfb.h"
#include "gppwc.intfb.h"
#include "grg_jno2_cloud.intfb.h"
#include "ppflev.intfb.h"
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('NOX2NO2',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------
!*    1.  PREPARATION

!*    1.1 MAX. NO. OF LEVELS
ILEN   = KLEN
IMXCNT = MAXVAL(KCOUNT(1:ILEN)) !number of layers, IMXCNT=1: total column data
IF(IMXCNT == 0) GOTO 9
IF(IMXCNT > 1) THEN
  WRITE(NULERR,*) 'NOX2NO2: ROUTINE DOES NOT WORK FOR IMXCNT>1. IMXCNT=',IMXCNT
  CALL ABOR1('NOX2NO2: ROUTINE DOES NOT WORK FOR IMXCNT>1.')
ENDIF

!*    1.2 FIND OUT PRESSURE LEVEL BELOW SPECIFIED (OBSERVED) PRESSURES

ALLOCATE(ILEVB(KDLEN,IMXCNT,NPPM))
ALLOCATE(LLBELO(KDLEN,IMXCNT))
ALLOCATE(LLBELS(KDLEN,IMXCNT))
ALLOCATE(LLBLOW(IMXCNT))
ALLOCATE(LLBLES(IMXCNT))
CALL PPFLEV(KDLEN,1,ILEN,NFLEVG,IMXCNT,&
 & PRESO,PRESH5,PRESF5,&
 & ILEVB,&
 & LLBELO,LLBELS,LLBLOW,LLBLES)  

!*    1.3 LN(P)

ALLOCATE(ZLNPRESO(KDLEN,IMXCNT))
DO JLEVP=1,IMXCNT
  ZLNPRESO(1:ILEN,JLEVP) = LOG(PRESO(1:ILEN,JLEVP))
ENDDO


!*    1.4 TOTAL CLOUD COVER

!*    1.4.1 Locate NOx and GO3 in GRG array
IND_NOX=-1
DO JGRG=1,NGRG
  IF(TRIM(YGRG_NL(JGRG)%cname) =='NOX') IND_NOX=JGRG
ENDDO
IF (IND_NOX==-1) THEN
  WRITE(NULERR,*) 'NOX2NO2: PROBLEM WITH NOX OPERATOR. NOX FIELD NEEDED BUT DOES NOT EXIST.'
  CALL ABOR1('NOX2NO2HOP: PROBLEM WITH NOX OPERATOR. NOX FIELD DOES NOT EXIST')
ENDIF
IND_GO3=-1
DO JGRG=1,NGRG
  IF(TRIM(YGRG_NL(JGRG)%cname) =='GO3') IND_GO3=JGRG
ENDDO
IF (IND_GO3==-1) THEN
  WRITE(NULERR,*) 'NOX2NO2HOP: PROBLEM WITH NOX OPERATOR. GO3 FIELD NEEDED BUT DOES NOT EXIST.'
  CALL ABOR1('NOX2NO2HOP: PROBLEM WITH NOX OPERATOR. GO3 FIELD DOES NOT EXIST')
ENDIF

JGRG=0
DO JGEMS=1,NGEMS
  IF(YDGEMSPROF5(1,JGEMS)%CSPECIES=='GRG') THEN
    JGRG=JGRG+1
    IF(JGRG==IND_NOX) THEN
      DO JLEN=1,ILEN
        ZNOX5(JLEN,1:NFLEVG)= &
         & YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1:NFLEVG)
        ZNOX5(JLEN,0)=YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1)
      ENDDO
    ENDIF
    IF(JGRG==IND_GO3) THEN
      DO JLEN=1,ILEN
        ZGO35(JLEN,1:NFLEVG)= &
         & YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1:NFLEVG)
        ZGO35(JLEN,0)=YDGEMSPROF5(JLEN,JGEMS)%MODELPROF(1)
      ENDDO
    ENDIF
  ENDIF
ENDDO

!*    1.4.2 Calculate integrated cloud cover above level - adapted from cldpp.f90
ALLOCATE(ZCCINT5(KDLEN,NFLEVG))
ZCCINT5(:,:)=0._JPRB
DO JLEN=1,ILEN
  ZCCINT5(JLEN,1)=1.0_JPRB-MIN(MAX(PCCF5(JLEN,1),REPCLC),1.0_JPRB-REPCLC)
ENDDO

DO JLEV=2,NFLEVG
  DO JLEN=1,ILEN
    IF(PCCF5(JLEN,JLEV-1) < 1.0_JPRB-REPSEC) THEN
      ZCCINT5(JLEN,JLEV)=ZCCINT5(JLEN,JLEV-1)* &
       & (1.0_JPRB-MAX(PCCF5(JLEN,JLEV),PCCF5(JLEN,JLEV-1)))&
       & /(1.0_JPRB-PCCF5(JLEN,JLEV-1))
    ENDIF
  ENDDO
ENDDO

DO JLEV=1,NFLEVG
  DO JLEN=1,ILEN
    ZCCINT5(JLEN,JLEV)=1.0_JPRB-ZCCINT5(JLEN,JLEV)
  ENDDO
ENDDO

!*    2. Convert model level NOx to NO2

!*    2.1 Compute layer O3 ON MODEL HALF LEVELS for use in GRG_JNO2. ZCOL5=0 AT TOP OF MODEL.

ZCOL5(:,:)=0._JPRB
CALL GPPWC(KDLEN,1,ILEN,NFLEVG,ZCOL5,ZGO35(1,1),PXP5)

!*    2.2 COMPUTE photolysis frequency for NO2

ZJNO25(:,:)=0._JPRB
CALL GRG_JNO2_CLOUD(KDLEN,1,ILEN,NFLEVG,ZCOL5(1,1),PTF5(1,1),PSOE,PAL,&
 &ZCCINT5,PTAUC5,PCOD5,ZJNO25,PXP5)

!*    2.3 Model level NOX to NO2 

CALL GPNOX(KDLEN,1,ILEN,NFLEVG,PNO25(1,1),PTF5(1,1),ZGO35(1,1),ZNOX5(1,1),PSOE,ZJNO25,PXP5)

DO JLEN=1,ILEN
  PNO25(JLEN,0)=PNO25(JLEN,1)
ENDDO

!*    3.  FINALLY, DEALLOCATE AND RETURN

DEALLOCATE(ILEVB,LLBELO,LLBELS,LLBLOW,LLBLES)
IF(ALLOCATED(ZLNPRESO)) DEALLOCATE(ZLNPRESO)

9 CONTINUE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('NOX2NO2',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

CONTAINS
SUBROUTINE ABORX(CDERROR)
CHARACTER (LEN = *),INTENT(IN) ::  CDERROR
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('NOX2NO2:ABORX',0,ZHOOK_HANDLE)
WRITE(NULERR,*) 'KOBSTYPE = ',KOBSTYPE,' CDPAR=',CDPAR
CALL ABOR1(CDERROR)
IF (LHOOK) CALL DR_HOOK('NOX2NO2:ABORX',1,ZHOOK_HANDLE)
END SUBROUTINE ABORX

!     ------------------------------------------------------------------
END SUBROUTINE NOX2NO2
