 SUBROUTINE OS_GEMS(PTAMOY,PTRAY,PIZMOY,PHIRAD,PBETAL,PRM,PGB,PRP,PPSL,PTODAER,PTODRAY,   &
&                                                     PXL)

!****OS*** COMPUTES ATMOSPHERIC INTRINSIC REFLECTANCE


!     INTERFACE
!     ---------
! OS IS CALLED BY ATMREF

!     INPUTS
!     ------

! PHIRAD              : DIFFERENCE IN AZIMUTAL ANGLE [RAD]
! PRP(KNP)            : GAUSSIAN ANGLES FOR AZIMUTH
! PRM(-KMU:KMU)       : GAUSSIAN ANGLES FOR THE ZENITH
! PGB(-KMU:KMU)       : GAUSSIAN WEIGHTS FOR THE ZENITH
! PTAMOY(KNT)         : TRUNCATED AEROSOL OPTICAL THICKNESS PROFILE.
! PTRAY(KNT)          : MOLECULAR OPTICAL THICKNESS PROFILE.
! PIZMOY(KNT)         : TRUNCATED AEROSOL SINGLE SCATTERING ALBEDO PROFILE.
! PBETAL(KNT,KNANG+1) : COEFFICIENTS OF LEGENDRE POLYNOM
! PPSL(0:KNANG,-1:KNANG,-KMU:KMU)
! PTODAER               : TOTAL AEROSOL OPTICAL THICKNESS
! PTODRAY               : TOTAL MOLECULAR OPTICAL THICKNESS

!     OUTPUTS
!     -------

! PXL(-KMU:KMU,KNP) : ATMOSPHERIC REFLECTANCE OF MIXTURE

!     EXTERNALS.
!     ----------
!          KERNEL

!     MODIFICATIONS.
!     -------------
!     N.HUNEUUS
!     A.BENEDETTI - ECMWF 06.08.2008

!     SOURCE
!     ------
! ADAPTED FROM OS.f FROM ORIGINAL 6S CODE, SEE VERMOTTE ET AL. (1997)

!     REMARKS
!     -------
! KNT=0 CORRESPONDS TO TOP OF THE ATMOSPHERE
! VARIABLES WITH DIMENSION FROM -KMU TO KMU, THE ELEMENTES CORRESPONDING TO NEGATIVE 
! INDEX CORRESPOND TO THE DOWNWARD DIRECTION AND WITH POSITIVE INDEX TO THE UPWARD
! DIRECTION. EXCEPT FOR THE ELEMENTS WITH INDEX -KMU, KMU AND 0. ELEMENTS WITH THE 
! INDEX -KMU & KMU CORRESPOND TO THE SATELLITE CASE. THEY SHOULD HAVE THE SAME VALUE 
!--------------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE RT6SVALUES

IMPLICIT NONE

REAL(KIND=JPRB),INTENT(IN) :: PHIRAD          ! DIFFERENCE IN AZIMUTAL ANGLE [RAD]
REAL(KIND=JPRB),INTENT(IN) :: PRP(KNP)        ! GAUSSIAN ANGLES FOR AZIMUTH
REAL(KIND=JPRB),INTENT(IN) :: PRM(-KMU:KMU)   ! GAUSSIAN ANGLES FOR THE ZENITH
REAL(KIND=JPRB),INTENT(IN) :: PGB(-KMU:KMU)   ! GAUSSIAN WEIGHTS FOR THE ZENITH
REAL(KIND=JPRB),INTENT(IN) :: PTAMOY(0:KNT)     ! TRUNCATED AEROSOL OPTICAL THICKNESS PROFILE.
REAL(KIND=JPRB),INTENT(IN) :: PTRAY(0:KNT)      ! MOLECULAR OPTICAL THICKNESS PROFILE.
REAL(KIND=JPRB),INTENT(IN) :: PIZMOY(0:KNT)     ! TRUNCATED AEROSOL SINGLE SCATTERING ALBEDO PROFILE.
REAL(KIND=JPRB),INTENT(IN) :: PBETAL(0:KNT,0:KNANG), PTODAER, PTODRAY
REAL(KIND=JPRB),INTENT(IN) :: PPSL(0:KNANG,-1:KNANG,-KMU:KMU)
REAL(KIND=JPRB),INTENT(OUT) :: PXL(-KMU:KMU,KNP)

! LOCAL VARIABLES
! ---------------

!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KMU2=48, KNP=49, KNANG=80, KNBMU=83, KNT=20

INTEGER(KIND=JPIM) :: JLEV, JPLEG, JMU, IPLEG, JMU2, JMUM1, IG, JNP, J
REAL(KIND=JPRB) :: ZH(0:KNT), ZCH(0:KNT), ZYDEL(0:KNT), ZXDEL(0:KNT)
REAL(KIND=JPRB) :: ZXMUS, ZAAA, ZRON, ZBETA0, ZBETA2
REAL(KIND=JPRB) :: ZSA2(0:KNT)                                    ! AEROSOL PHASE FUNCTION
REAL(KIND=JPRB) :: ZI1(0:KNT,-KMU:KMU), ZI2(0:KNT,-KMU:KMU)
REAL(KIND=JPRB) :: ZI3(-KMU:KMU), ZI4(-KMU:KMU)
REAL(KIND=JPRB) :: ZINM1(-KMU:KMU), ZINM2(-KMU:KMU)
REAL(KIND=JPRB) :: ZIN(-KMU:KMU)
REAL(KIND=JPRB) :: ZII1, ZII2, ZBPJK, ZBPJMK, ZXI1, ZXI2, ZXDB
REAL(KIND=JPRB) :: ZSPL, Z, ZX, ZXX, ZSA1, ZXPK, ZY, ZYY
REAL(KIND=JPRB) :: ZA1, ZD1, ZG1,ZY1, z_DELTA0S, ZA, ZB, ZC, ZD, ZF, ZZI1
REAL(KIND=JPRB) :: ZYPK, ZXPJ, ZPHI, ZRO, ZRO0, ZRO1, ZRO2, ZAUX, ZAUX2
! OUTPUT OF KERNEL
REAL(KIND=JPRB) :: z_PXPL(-KMU:KMU)
REAL(KIND=JPRB) :: z_PBP(0:KNT,0:KMU,-KMU:KMU)

REAL(KIND=JPRB) :: z_AUX_AER, z_AUX_RAY

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "kernel_pbp.intfb.h"

IF (LHOOK) CALL DR_HOOK('OS_GEMS',0,ZHOOK_HANDLE)

! VARIABLE INITIALIZATION 
ZIN(:)=0.0_JPRB
ZI4(:)=0.0_JPRB
ZI3(:)=0.0_JPRB
ZI1(:,:)=0.0_JPRB
ZI2(:,:)=0.0_JPRB
ZINM1(:)=0.0_JPRB
ZINM2(:)=0.0_JPRB
PXL(:,:)=0.0_JPRB
ZXMUS=-PRM(0)


! COMPUTATION OF TRANSMISSION (CH), TOTAL OPTICAL THICKNESS (ZH) AND PROFILE OF SINGLE
! SCATTERING ALBEDO OF MOLECULES (YDEL) AND AEROSOLS (XDEL) FOR THREE CASES


! CASE 1: PURE RAYLEIGH
! -------


IF ((PTODAER <= ACCU2).AND.(PTODRAY >  PTODAER)) THEN
  ZAUX=0.0_JPRB
  DO JLEV=0,KNT ! JLEV=0 CORRESPONDS TO TOP OF THE ATMOSPHERE
    ZAUX=ZAUX+PTRAY(JLEV)
    ZH(JLEV)=ZAUX
    ZCH(JLEV)=EXP(-ZH(JLEV)/ZXMUS)/2.0_JPRB
    ZYDEL(JLEV)=1.0_JPRB
    ZXDEL(JLEV)=0.0_JPRB
  ENDDO
ENDIF

! CASE 2: PURE AEROSOL
! -------

IF ((PTODRAY <= ACCU2).AND.(PTODAER >  PTODRAY)) THEN
  ZAUX=0.0_JPRB
  DO JLEV=0,KNT
    ZAUX=ZAUX+PTAMOY(JLEV)
    ZH(JLEV)=ZAUX
    ZCH(JLEV)=EXP(-ZH(JLEV)/ZXMUS)/2.0_JPRB
    ZYDEL(JLEV)=0.0_JPRB
    ZXDEL(JLEV)=PIZMOY(JLEV)
  ENDDO
ENDIF

! CASE 3: MIXTURE
! -------

IF ((PTODRAY >  ACCU2).AND.(PTODAER >  ACCU2)) THEN
  ZAUX=0.0_JPRB
  z_AUX_AER=0.0_JPRB
  z_AUX_RAY=0.0_JPRB
  DO JLEV=0,KNT
    ZAUX=ZAUX+PTAMOY(JLEV)+PTRAY(JLEV)
    z_AUX_AER=z_AUX_AER+PTAMOY(JLEV)
    z_AUX_RAY=z_AUX_RAY+PTRAY(JLEV)
    ZH(JLEV)=ZAUX
    ZCH(JLEV)=EXP(-ZH(JLEV)/ZXMUS)/2.0_JPRB
    ZXDEL(JLEV)=PIZMOY(JLEV)*PTAMOY(JLEV)/(PTAMOY(JLEV)+PTRAY(JLEV))
    ZYDEL(JLEV)=PTRAY(JLEV)/(PTAMOY(JLEV)+PTRAY(JLEV))
  ENDDO
ENDIF


! INCIDENT ANGLE MU_S

ZAAA=DELTA/(2.0_JPRB-DELTA)
ZRON=(1.0_JPRB-ZAAA)/(1.0_JPRB+2.0_JPRB*ZAAA)

! RAYLEIGH PHASE FUNCTION

ZBETA0=1.0_JPRB
ZBETA2=0.5_JPRB*ZRON


! FOURIER DECOMPOSITION

IF (ABS(ZXMUS-1.0_JPRB) <  ACCU3) THEN
  IPLEG=0          
ELSE
  IPLEG=KNANG
ENDIF
JPLEG=0

DO JPLEG=0,NFOUR    ! LOOP ON S, TERM IN THE FOURIER SERIES

! PRIMARY SCATTERING

  ZRO2=0.0_JPRB
  ZRO1=0.0_JPRB
  ZRO0=0.0_JPRB
  ZRO=0.0_JPRB
  ZI3(:)=0.0_JPRB

! KERNEL COMPUTATIONS

  CALL KERNEL_PBP(JPLEG,PPSL,PBETAL,z_PXPL,z_PBP)
  
  IF (JPLEG >  0) ZBETA0=0.0_JPRB

  DO JMU=-KMU,KMU
    IF ((JPLEG-2) <= 0) THEN
      ZSPL=z_PXPL(0)
      ZSA1=ZBETA0+ZBETA2*z_PXPL(JMU)*ZSPL   ! RAYLEIGH PHASE FUNCTION
    ELSE
      ZSA1=0.0_JPRB
    ENDIF
    DO JLEV=0,KNT
      ZSA2(JLEV)=z_PBP(JLEV,0,JMU)
      ZI2(JLEV,JMU)=ZCH(JLEV)*(ZSA2(JLEV)*ZXDEL(JLEV)+ZSA1*ZYDEL(JLEV))
    ENDDO
  ENDDO

! VERTICAL INTEGRATION, PRIMARY UPWARD RADIATION

  DO JMU=1,KMU
    ZI1(KNT,JMU)=0.0_JPRB
    ZZI1=ZI1(KNT,JMU)
    ZY=PRM(JMU)
    DO JLEV=KNT-1,0,-1
      ZF=ZH(JLEV+1)-ZH(JLEV)
      ZA=(ZI2(JLEV+1,JMU)-ZI2(JLEV,JMU))/ZF
      ZB=ZI2(JLEV,JMU)-ZA*ZH(JLEV)
      ZC=EXP(-ZF/ZY)
      ZD=1.0_JPRB-ZC
      ZX=ZH(JLEV)-ZH(JLEV+1)*ZC
      ZZI1=ZC*ZZI1+(ZD*(ZB+ZA*ZY)+ZA*ZX)*0.5_JPRB
      ZI1(JLEV,JMU)=ZZI1
    ENDDO
  ENDDO

! VERTICAL INTEGRATION, PRIMARY DOWNWARD RADIATION

  DO JMU=-KMU,-1
    ZI1(0,JMU)=0.0_JPRB
    ZZI1=ZI1(0,JMU)
    ZY=PRM(JMU)
    DO JLEV=1,KNT
      ZF=ZH(JLEV)-ZH(JLEV-1)
      ZC=EXP(ZF/ZY)
      ZD=1.0_JPRB-ZC
      ZA=(ZI2(JLEV,JMU)-ZI2(JLEV-1,JMU))/ZF
      ZB=ZI2(JLEV,JMU)-ZA*ZH(JLEV)
      ZX=ZH(JLEV)-ZH(JLEV-1)*ZC
      ZZI1=ZC*ZZI1+(ZD*(ZB+ZA*ZY)+ZA*ZX)*0.5_JPRB
      ZI1(JLEV,JMU)=ZZI1
    ENDDO
  ENDDO


! ZINM2 IS INITIALIZED WITH SCATTERING COMPUTED AT N-2
! I3 IS INITIALIZED WITH PRIMARY SCATTERING

  DO JMU=-KMU,-1
    ZINM1(JMU)=ZI1(KNT,JMU)
    ZINM2(JMU)=ZI1(KNT,JMU)
    ZI3(JMU)=ZI1(KNT,JMU)
  ENDDO
  DO JMU=1,KMU
    ZINM1(JMU)=ZI1(0,JMU)
    ZINM2(JMU)=ZI1(0,JMU)
    ZI3(JMU)=ZI1(0,JMU)
  ENDDO
  ZRO2=ZI1(0,KMU)
  ZRO=ZI1(0,KMU)

! LOOP ON SUCCESSIVE ORDER

  IG=1 ! PRIMARY SCATTERING
  DO IG=2,NORD

! SUCCESSIVE ORDERS

! MULTIPLE SCATTERING SOURCE FUNCTION AT EVERY LEVEL WITHIN THE LAYER

! IF JPLEG <= 2 KERNELS ARE MIXING OF AEROSOLS AND MOLECULES
! IF JPLEG >  2 AEROSOL KERNEL ONLY

    IF ((JPLEG-2) <= 0) THEN
      DO JMU=1,KMU
        ZXPK=z_PXPL(JMU)
        ZYPK=z_PXPL(-JMU)
        DO JLEV=0,KNT
          ZII1=0.0_JPRB
          ZII2=0.0_JPRB
          ZX=ZXDEL(JLEV)
          ZY=ZYDEL(JLEV)
          DO JMU2=1,KMU
            ZXPJ=z_PXPL(JMU2)    
            ZAUX2=PGB(JMU2)
            ZXI1=ZI1(JLEV,JMU2)
            ZXI2=ZI1(JLEV,-JMU2)
            ZBPJK=z_PBP(JLEV,JMU2,JMU)*ZX+ZY*(ZBETA0+ZBETA2*ZXPJ*ZXPK)
            ZBPJMK=z_PBP(JLEV,JMU2,-JMU)*ZX+ZY*(ZBETA0+ZBETA2*ZXPJ*ZYPK)
            ZXDB=ZAUX2*(ZXI1*ZBPJK+ZXI2*ZBPJMK)
            ZII2=ZII2+ZXDB
            ZXDB=ZAUX2*(ZXI1*ZBPJMK+ZXI2*ZBPJK)
            ZII1=ZII1+ZXDB
          ENDDO
          ZI2(JLEV,JMU)=ZII2
          ZI2(JLEV,-JMU)=ZII1
        ENDDO
      ENDDO
      WHERE (ZI2(:,:) <  ZEPS) ZI2(:,:)=0.0_JPRB
    ELSEIF ((JPLEG-2) >  0) THEN 
      DO JMU=1,KMU
        DO JLEV=0,KNT
          ZII1=0.0_JPRB
          ZII2=0.0_JPRB
          ZX=ZXDEL(JLEV)
          DO JMU2=1,KMU
            ZAUX2=PGB(JMU2)
            ZXI1=ZI1(JLEV,JMU2)
            ZXI2=ZI1(JLEV,-JMU2)
            ZBPJK=z_PBP(JLEV,JMU2,JMU)*ZX
            ZBPJMK=z_PBP(JLEV,JMU2,-JMU)*ZX
            ZXDB=ZAUX2*(ZXI1*ZBPJK+ZXI2*ZBPJMK)
            ZII2=ZII2+ZXDB
            ZXDB=ZAUX2*(ZXI1*ZBPJMK+ZXI2*ZBPJK)
            ZII1=ZII1+ZXDB
          ENDDO
          ZI2(JLEV,JMU)=ZII2
          ZI2(JLEV,-JMU)=ZII1
        ENDDO
      ENDDO
      WHERE (ZI2(:,:) <  ZEPS) ZI2(:,:)=0.0_JPRB
    ENDIF

! VERTICAL INTEGRATION, UPWARD RADIATION

    DO JMU=1,KMU
      ZI1(KNT,JMU)=0.0_JPRB
      ZZI1=ZI1(KNT,JMU)
      ZYY=PRM(JMU)
      DO JLEV=KNT-1,0,-1
        ZF=ZH(JLEV+1)-ZH(JLEV)
        ZA=(ZI2(JLEV+1,JMU)-ZI2(JLEV,JMU))/ZF
        ZB=ZI2(JLEV,JMU)-ZA*ZH(JLEV)
        ZC=EXP(-ZF/ZYY)
        ZD=1.D+00-ZC
        ZXX=ZH(JLEV)-ZH(JLEV+1)*ZC
        ZZI1=ZC*ZZI1+(ZD*(ZB+ZA*ZYY)+ZA*ZXX)*0.5_JPRB
        IF (ABS(ZZI1) <  ACCU) ZZI1=0.0_JPRB
        ZI1(JLEV,JMU)=ZZI1
      ENDDO
    ENDDO

! VERTICAL INTEGRATION, DOWNWARD RADIATION

    DO JMU=-KMU,-1
      ZI1(0,JMU)=0.0_JPRB
      ZZI1=ZI1(0,JMU)
      ZYY=PRM(JMU)
      DO JLEV=1,KNT
        ZF=ZH(JLEV)-ZH(JLEV-1)
        ZC=EXP(ZF/ZYY)
        ZD=1.D+00-ZC
        ZA=(ZI2(JLEV,JMU)-ZI2(JLEV-1,JMU))/ZF
        ZB=ZI2(JLEV,JMU)-ZA*ZH(JLEV)
        ZXX=ZH(JLEV)-ZH(JLEV-1)*ZC
        ZZI1=ZC*ZZI1+(ZD*(ZB+ZA*ZYY)+ZA*ZXX)*0.5_JPRB
        IF (ABS(ZZI1) <  ACCU) ZZI1=0.0_JPRB
        ZI1(JLEV,JMU)=ZZI1
      ENDDO
    ENDDO

! IN IS THE Nieme SCATTERING ORDER

    DO JMU=-KMU,-1
      ZIN(JMU)=ZI1(KNT,JMU)
    ENDDO
    DO JMU=1,KMU
      ZIN(JMU)=ZI1(0,JMU)
    ENDDO
    ZRO0=ZI1(0,KMU)

! CONVERGENCE TEST (GEOMETRICAL SERIE)

    IF (IG >  2) THEN
      Z=0.0_JPRB
      ZA1=ZRO2
      ZD1=ZRO1
      ZG1=ZRO0
      IF ((ZA1 >= ACCU).AND.(ZD1 >= ACCU).AND.(ZRO >= ACCU)) THEN
        ZY=((ZG1/ZD1-ZD1/ZA1)/((1.0_JPRB-ZG1/ZD1)**2)*(ZG1/ZRO))
        ZY=ABS(ZY)
        Z=MAX(Z,ZY)
      ENDIF
      DO JMU=-KMU,-1
        ZA1=ZINM2(JMU)
        ZD1=ZINM1(JMU)
        ZG1=ZIN(JMU)
        IF ((ZA1 >  ACCU).AND.(ZD1 >  ACCU).AND.(ZI3(JMU) >  ACCU)) THEN
          ZY=((ZG1/ZD1-ZD1/ZA1)/((1.0_JPRB-ZG1/ZD1)**2)*(ZG1/ZI3(JMU)))
          ZY=ABS(ZY)
          Z=MAX(ZY,Z)
        ENDIF
      ENDDO
      DO JMU=1,KMU
        ZA1=ZINM2(JMU)
        ZD1=ZINM1(JMU)
        ZG1=ZIN(JMU)
        IF ((ZA1 >  ACCU).AND.(ZD1 >  ACCU).AND.(ZI3(JMU) >  ACCU)) THEN
          ZY=((ZG1/ZD1-ZD1/ZA1)/((1.0_JPRB-ZG1/ZD1)**2)*(ZG1/ZI3(JMU)))
          ZY=ABS(ZY)
          Z=MAX(ZY,Z)
        ENDIF
      ENDDO

      IF (Z <  ACCU4) THEN

! SUCCESSFUL TEST (GEOMETRICAL SERIE)

        DO JMU=-KMU,KMU
          ZY1=1.0_JPRB
          ZD1=ZINM1(JMU)
          ZG1=ZIN(JMU)
          IF (ZD1 >  ACCU) THEN
            ZY1=1.0_JPRB-ZG1/ZD1
            IF (ABS(ZG1-ZD1) >  ACCU) THEN
              ZG1=ZG1/ZY1
              ZI3(JMU)=ZI3(JMU)+ZG1
            ENDIF
          ENDIF
        ENDDO
        ZD1=ZRO1
        ZG1=ZRO0
        ZY=1.0_JPRB
        IF (ZD1 >= ACCU) THEN
          IF (ABS(ZG1-ZD1) >= ACCU) THEN
            ZY1=1.0_JPRB-ZG1/ZD1
            ZG1=ZG1/ZY1
          ENDIF
          ZRO=ZRO+ZG1
        ENDIF
!        EXIT
      ELSE

! ZINM2 IS THE (N-2)ieme SCATTERING ORDER

        DO JMU=-KMU,KMU
          ZINM2(JMU)=ZINM1(JMU)
        ENDDO
        ZRO2=ZRO1
      ENDIF
    ENDIF
    IF ((IG <= 2).OR.(Z >= 0.0001_JPRB)) THEN

! ZINM1 IS THE (N-1)ieme SCATTERING ORDER

      DO JMU=-KMU,KMU
        ZINM1(JMU)=ZIN(JMU)
      ENDDO
      ZRO1=ZRO0

! SUM OF THE N-1 ORDERS

      DO JMU=-KMU,KMU
        ZI3(JMU)=ZI3(JMU)+ZIN(JMU)
      ENDDO
      ZRO=ZRO+ZRO0

! STOP IF ORDER N IS LESS THAN 1% OF THE SUM

      Z=0.0_JPRB
      DO JMU=-KMU,KMU
        IF (ABS(ZI3(JMU)) >= ACCU) THEN
          ZY=ABS(ZIN(JMU)/ZI3(JMU))
          Z=MAX(Z,ZY)
        ENDIF
      ENDDO
!      IF ((Z <  0.00001_JPRB).OR.(IG >  20)) THEN
!        EXIT
!      ENDIF
    ENDIF
  ENDDO  ! END LOOP ON SUCCESSIVE ORDERS 

! SUM OF THE FOURIER COMPONENT S

  z_DELTA0S=1.0_JPRB
  IF (JPLEG /= 0) THEN
    z_DELTA0S=2.0_JPRB
  ENDIF
  DO JMU=-KMU,KMU
    ZI4(JMU)=ZI4(JMU)+z_DELTA0S*ZI3(JMU)
  ENDDO

! STOP OF THE FOURIER DECOMPOSITION

  JMUM1=KMU-1
  DO JNP=1,KNP
    ZPHI=PRP(JNP)
    DO J=-JMUM1,JMUM1
      IF (J >  0) THEN
        PXL(J,JNP)=PXL(J,JNP)+z_DELTA0S*ZI3(J)*COS(DBLE(JPLEG)*(ZPHI+ZPI))
      ELSE
        PXL(J,JNP)=PXL(J,JNP)+z_DELTA0S*ZI3(J)*COS(DBLE(JPLEG)*ZPHI)
      ENDIF
    ENDDO
  ENDDO
  IF (JPLEG == 0) THEN 
    DO J=1,JMUM1
      PXL(0,1)=PXL(0,1)+PRM(J)*PGB(J)*ZI3(-J)
    ENDDO
  ENDIF
  PXL(KMU,1)=PXL(KMU,1)+z_DELTA0S*ZI3(KMU)*COS(DBLE(JPLEG)*(PHIRAD+ZPI))
  PXL(-KMU,1)=PXL(-KMU,1)+z_DELTA0S*ZI3(-KMU)*COS(DBLE(JPLEG)*(PHIRAD+ZPI)) ! DOWNWARD FLUX
  Z=0.0
  DO JMU=-KMU,KMU
    IF (ABS(ZI4(JMU)) >= ACCU) THEN
      ZX=ABS(ZI3(JMU)/ZI4(JMU))
      Z=MAX(Z,ZX)
    ENDIF
  ENDDO
!  IF ((Z <= 0.001_JPRB).OR.(JPLEG >= IPLEG)) THEN
!    EXIT
!  ENDIF
!  JPLEG=JPLEG+1
ENDDO

IF (LHOOK) CALL DR_HOOK('OS_GEMS',1,ZHOOK_HANDLE)

END SUBROUTINE OS_GEMS
