SUBROUTINE RADTRAD(KSAT,KSENSOR,KDPROF,KPROF,KMXCH,KNCH,PRCH, &
 & PZENANG,PAZIANG,PSOLZENANG,PSOLAZIANG,PLAT,POROG,&
 & PTQOSAT,YDGEMSPROF,PPSSAT,PTSSAT,PT2SAT,PH2SAT,PCTOP,PCAMT  ,&
 & PUSSAT,PVSSAT,PEMIS,                                  &
 & PTQOSAT5,YDGEMSPROF5,PPSSAT5,PTSSAT5,PLSSAT5,PT2SAT5,&
 & PH2SAT5 ,PCTOP5,PCAMT5,PUSSAT5,PVSSAT5,PEMIS5, PTB)  

!**** *RADTRAD* - INTERFACE TO TOVS RAD. TRANSFER CODE - ADJOINT

!     AUTHOR: TONY MCNALLY     DATE  : 90-11-05

!     PURPOSE.
!     -------
!     SET UP REASONABLE CHUNKS OF WORK FOR THE TOVS RADIANCE
!     CALCULATION.

!**   INTERFACE.
!**   ---------
!**   *CALL* * RADTRAD( )*  (FROM TOVCLR)
!     WHERE PZENANG: Satellite zenith Angle
!           PAZIANG: Satellite azimuth angle
!           PSOLZENANG: Solar zenith angle
!           PSOLAZIANG: Solar azimuth angle
!           PLAT:    Latitude [deg]
!           POROG:   Orography [m^2 s^-2]
!           KSAT   : NOAA-sateliite id
!           KSENSOR: Sensor id
!           KMXCH  : Maximum number of channels
!           KNCH   : Number of channels to be computed
!           PRCH   : List of channels to be computed
!           KDPROF : First dimension of arrays
!           KPROF  : Actual number of TOVS points
!           PTQOSAT: Model values at TOVS locations, T/Q/O3  (AD)
!           YDGEMSPROF : GEMS variables  (AD)
!           PPSSAT : Surface pressure  (AD)
!           PTSSAT : Surface temperature  (AD)
!           PT2SAT : 2m temperature  (AD)
!           PH2SAT : 2m humidity  (AD)
!           PUSSAT : Surface wind u  (AD)
!           PVSSAT : Surface wind v  (AD)
!           PEMIS  : Surface emissivity for each channel  (AD)
!           ...5   : Trajectory
!           PLSSAT5 : MODEL SURFACE TYPE
!           PTB    : Brightness temperatures  (input)  (AD)

!**   EXTERNALS
!     ---------
!        RTTOVAD - RT-CODE, ADJOINT

!     MODIFICATIONS
!     -------------
!     Erik Andersson 920304 : DOCTORize and multi-task.
!     Erik Andersson 920818 : New version of RT-code
!     Erik Andersson 921216 : Remove regression extrapolation
!     Erik Andersson 970220 : F90, simplify code.
!     Roger Saunders 970811 : Include ozone, emissivity & cloud as 
!                               variables
!     Erik Andersson 971117 : Add ozone profile input
!     Rose Munro     980108 : Add satellite view angle
!     Rose Munro     980108 : Alter tests on max/min regression prof for new RT
!     Roger Saunders 980821 : Changes for RTTOV5 interfACE
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        O.Spaniel     Oct-2004 cleaning AL29
!     Niels Bromann  : 071003 - Updated for RTTOV-9
!     Alan Geer      : 080718   Fix units for orography (IFS uses m^2 s^-2)
!        V.Guidard     23-Feb-2007 Add arg KSENSOR (after A.Collard)
!     NEC         11-Dec-2007 Optimizations
!     Richard Engelen  090210 : More flexible interface between tracers and RTTOV input
!     V. Guidard  24-02-2009  CO2-slicing for AIRS
!     -----------------------------------------------------------------------------



USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMVAR   , ONLY : LTOVSCV  ,LCO2, LCLDSINK 
USE YOMDIMO  , ONLY :     NLSAT    ,NMXTCHU  ,&
 & NPNAV    ,NPTNAV    ,NPNSAV   ,NPNSSV   ,NPNCV   
USE YOMTVRAD , ONLY : NJPPF    ,RADPRE    ,MSENSOR_AIRS, MSENSOR_IASI, &
 & AIRSCHAN_BACK, IASICHAN_BACK, LCO2_DIAG_AIRS
USE YOMANCS  , ONLY : NMDI
USE YOM_YGFL , ONLY : NGHG
USE YOMCST   , ONLY : RG
USE TYPE_GEMS_PROFILES

IMPLICIT NONE

!*    0.1  Global arrays

INTEGER(KIND=JPIM),INTENT(IN)    :: KDPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF
INTEGER(KIND=JPIM),INTENT(IN)    :: KSAT
INTEGER(KIND=JPIM),INTENT(IN)    :: KSENSOR
INTEGER(KIND=JPIM),INTENT(IN)    :: KMXCH 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNCH(KDPROF)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZENANG(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAZIANG(KDPROF)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOLZENANG(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOLAZIANG(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCH(KDPROF,KMXCH) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTQOSAT(KDPROF,NLSAT,NPNAV-1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPSSAT(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSSAT(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT2SAT(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PH2SAT(KDPROF),PCTOP(KDPROF),PCAMT(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUSSAT(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVSSAT(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIS(KDPROF,KMXCH) 
TYPE (TYPE_GEMSPROF)   ,INTENT(INOUT) :: YDGEMSPROF(KDPROF,NGEMS)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTQOSAT5(KDPROF,NLSAT,NPNAV-1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPSSAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSSAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSSAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT2SAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PH2SAT5(KDPROF),PCTOP5(KDPROF),PCAMT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUSSAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVSSAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS5(KDPROF,KMXCH) 
TYPE (TYPE_GEMSPROF)   ,INTENT(IN) :: YDGEMSPROF5(KDPROF,NGEMS)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTB(KDPROF,KMXCH) 

INTEGER(KIND=JPIM), DIMENSION(NJPPF*NMXTCHU) :: ICHID,IPFID
INTEGER(KIND=JPIM), DIMENSION(NJPPF)         :: ISURF
INTEGER(KIND=JPIM), DIMENSION(NJPPF)         :: INCH
INTEGER(KIND=JPIM), DIMENSION(NJPPF)         :: IFAIL
REAL(KIND=JPRB), DIMENSION(NJPPF*NMXTCHU):: ZTB ! Brightness temperatures (AD)
REAL(KIND=JPRB), DIMENSION(NJPPF*NMXTCHU):: ZTB5 ! Clear brightness temperatures

REAL(KIND=JPRB), DIMENSION(NJPPF*NMXTCHU)    :: ZEM
REAL(KIND=JPRB), DIMENSION(NLSAT,NJPPF)      :: ZPPRES
REAL(KIND=JPRB), DIMENSION(NLSAT,NPTNAV+2,NJPPF):: ZPROF
REAL(KIND=JPRB), DIMENSION(NPNSAV,NJPPF)     :: ZPROFS
REAL(KIND=JPRB), DIMENSION(NPNSSV,NJPPF)     :: ZPROFSS
REAL(KIND=JPRB), DIMENSION(NPNCV,NJPPF)      :: ZPROFCV
REAL(KIND=JPRB), DIMENSION(NJPPF*NMXTCHU)    :: ZEM5
REAL(KIND=JPRB), DIMENSION(NLSAT,NJPPF)      :: ZPPRES5
REAL(KIND=JPRB), DIMENSION(NLSAT,NPTNAV+2,NJPPF):: ZPROF5
REAL(KIND=JPRB), DIMENSION(NPNSAV,NJPPF)     :: ZPROFS5
REAL(KIND=JPRB), DIMENSION(NPNSSV,NJPPF)     :: ZPROFSS5
REAL(KIND=JPRB), DIMENSION(NPNCV,NJPPF)      :: ZPROFCV5
REAL(KIND=JPRB), DIMENSION(NJPPF)            :: ZLAT,ZOROG
REAL(KIND=JPRB), DIMENSION(NJPPF)            :: ZZENANG,ZAZIANG
REAL(KIND=JPRB), DIMENSION(NJPPF)            :: ZSOLZENANG,ZSOLAZIANG

INTEGER(KIND=JPIM) :: IPACK,IFIN,IEND,IDONE,ICHPF,IMXCHPF
INTEGER(KIND=JPIM) :: JJPACK,JJPF,JCH,JLEV,JF, JP
LOGICAL :: LL_LCLOUD, LL_LINTERP
LOGICAL, DIMENSION(4) :: LLTRACER
CHARACTER(LEN=80) :: CLERROR

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!========================================================================

IF (LHOOK) CALL DR_HOOK('RADTRAD',0,ZHOOK_HANDLE)

LL_LCLOUD  = .FALSE.
LL_LINTERP = .FALSE.

!*    1.0  Number of packets
!*    ------------------------------------------------------------------

IPACK=KPROF/NJPPF+1
IFIN=MOD(KPROF,NJPPF)
IF(IFIN == 0) THEN
  IPACK=IPACK-1
  IFIN=NJPPF
ENDIF

!*    1.1  Pressure Pa--> hPa

DO JJPF=1,NJPPF
  ZPPRES5(:,JJPF)=RADPRE(:)/100._JPRB
ENDDO

!*    1.2  Allocate RTTOV temporary arrays

IMXCHPF=KMXCH*NJPPF
CALL RTTOV_EC_ALLOC_AD(IFAIL, KSAT, IMXCHPF, NJPPF, NLSAT, &
 &    LL_LCLOUD, LL_LINTERP,                               &
 &    1_JPIM)


!*    2.0  Make packets, Call RT-code.
!*    ------------------------------------------------------------------

IDONE=0
PACKET_LOOP: DO JJPACK=1,IPACK
  IFAIL(:)=0

  IF(JJPACK == IPACK) THEN
    IEND=IFIN
  ELSE
    IEND=NJPPF
  ENDIF

!*    2.1  Profile variables, surface, T2m, q2m, Ts and Ps.
!*         Ozon, cloud pressure and amount, emissivity
!*         Surface indicators and angles

  DO JJPF=1,IEND
    ZPROF5(1:NLSAT,1,JJPF)=PTQOSAT5(JJPF+IDONE,1:NLSAT,1)  ! T
    ZPROF5(1:NLSAT,2,JJPF)=PTQOSAT5(JJPF+IDONE,1:NLSAT,2)  ! Q 
    ZPROF5(1:NLSAT,3,JJPF)=PTQOSAT5(JJPF+IDONE,1:NLSAT,3)  ! O3

    ZPROFS5(1,JJPF)=PT2SAT5(JJPF+IDONE)
    ZPROFS5(2,JJPF)=PH2SAT5(JJPF+IDONE)
    ZPROFS5(3,JJPF)=PPSSAT5(JJPF+IDONE)/100._JPRB
    ZPROFS5(4,JJPF)=PUSSAT5(JJPF+IDONE)
    ZPROFS5(5,JJPF)=PVSSAT5(JJPF+IDONE)
    ZPROFSS5(1,JJPF)=PTSSAT5(JJPF+IDONE)
    ZPROFCV5(2,JJPF)=0.0_JPRB      ! cloud amount

  IF(LTOVSCV .AND. LCLDSINK) THEN
    ZPROFCV5(1,JJPF)=MIN(MAX(PCTOP5(JJPF+IDONE),51.0_JPRB),1012.0_JPRB)     ! cloud top pressure
    ZPROFCV5(2,JJPF)=MIN(MAX(PCAMT5(JJPF+IDONE),0.0_JPRB),1.0_JPRB)     ! cloud amount
  ELSEIF ((KSENSOR == MSENSOR_AIRS).AND.LCO2_DIAG_AIRS) THEN
    ZPROFCV5(1,JJPF)=PCTOP5(JJPF+IDONE)
    ZPROFCV5(2,JJPF)=PCAMT5(JJPF+IDONE)
  ELSE
    ZPROFCV5(1,JJPF)=500.0_JPRB                                                                          
    ZPROFCV5(2,JJPF)=0.0_JPRB                                                                
  ENDIF

    ZZENANG(JJPF)=PZENANG(JJPF+IDONE)
    ZAZIANG(JJPF)=PAZIANG(JJPF+IDONE)
    ZSOLZENANG(JJPF)=PSOLZENANG(JJPF+IDONE)
    ZSOLAZIANG(JJPF)=PSOLAZIANG(JJPF+IDONE)
    ZLAT(JJPF)  =PLAT(JJPF+IDONE)
    ZOROG(JJPF) =POROG(JJPF+IDONE)/RG ! convert to m
    IF (PLSSAT5(JJPF+IDONE)  <=  0.5_JPRB)THEN
      ISURF(JJPF)=1
    ELSE
      ISURF(JJPF)=0
    ENDIF
  ENDDO
  ZPROF5(1:NLSAT,4:6,1:IEND)=0.0_JPRB

!*    2.2  Additional gases

  LLTRACER(:)=.FALSE.
  IF(NGHG > 0) THEN
    DO JF=1,NGEMS
      DO JJPF=1,IEND
        IF(YDGEMSPROF5(JJPF+IDONE,JF)%LRTTOV) THEN
          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210061) JP=7 !CO2
          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210063) JP=8 !N2O
          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210062) JP=9 !CH4
          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210061) LLTRACER(1)=.TRUE. !CO2
          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210063) LLTRACER(2)=.TRUE. !N2O
          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210062) LLTRACER(3)=.TRUE. !CH4
          ZPROF5(:,JP,JJPF)=YDGEMSPROF5(JJPF+IDONE,JF)%RTTOVPROF(:)
        ENDIF
      ENDDO
    ENDDO
  ENDIF
! The following bit has not been tested yet
!  IF(NGRG > 0) THEN
!    DO JF=1,NGEMS
!      DO JJPF=1,IEND
!        IF(YDGEMSPROF5(JJPF+IDONE,JF)%LRTTOV) THEN
!          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210123) JP=10 !CO
!          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210123) LLTRACER(4)=.TRUE. !CO
!          ZPROF(:,JP,JJPF)=YDGEMSPROF(JJPF+IDONE,JF)%RTTOVPROF(:)
!          ZPROF5(:,JP,JJPF)=YDGEMSPROF5(JJPF+IDONE,JF)%RTTOVPROF(:)
!        ENDIF
!      ENDDO
!    ENDDO
!  ENDIF

  IF(LCO2) THEN
    DO JJPF=1,IEND
      ZPROF5(JLEV,7,JJPF)=PTQOSAT5(JJPF+IDONE,JLEV,4)
    ENDDO
  ENDIF


!*    2.3  Set indices of TOVS channels
!*         Perform radtiative transfer calculations

  ICHPF=0
  PF_LOOP_RI: DO JJPF=1,IEND
    INCH(JJPF)=KNCH(JJPF+IDONE)
    DO JCH=1,KNCH(JJPF+IDONE)
      ICHPF=ICHPF+1
      ICHID(ICHPF)=NINT(PRCH(JJPF+IDONE,JCH))
      IPFID(ICHPF)=JJPF
      ZEM5(ICHPF)=PEMIS5(JJPF+IDONE,JCH)
      ZTB(ICHPF)=PTB(JJPF+IDONE,JCH)
    ENDDO
  ENDDO PF_LOOP_RI
   
  ! Reset channel numbers to the values in the reduced set initialised in 
  ! RTTVI
  SELECT CASE (KSENSOR)
     CASE (MSENSOR_AIRS)
        ICHID(1:ICHPF) = AIRSCHAN_BACK(ICHID(1:ICHPF))
     CASE (MSENSOR_IASI)
        ICHID(1:ICHPF) = IASICHAN_BACK(ICHID(1:ICHPF))
  END SELECT
  IF (ANY(ICHID(1:ICHPF) == -NMDI)) &
       & CALL ABOR1('Error in channel subsetting for RTTOV')

  CALL RTTOV_EC_AD(IFAIL, KSAT, ICHPF, IEND, NLSAT,           &
   &    ICHID, IPFID,                                         &
   &    ZPPRES5,ZPROF5,ZPROFS5,ZPROFSS5,ISURF,ZEM5,ZPROFCV5,  &
   &    ZPPRES, ZPROF, ZPROFS, ZPROFSS,       ZEM, ZPROFCV,   &
   &    ZZENANG, ZAZIANG, ZSOLZENANG, ZSOLAZIANG, ZLAT, ZOROG,&
   &    LL_LCLOUD, LL_LINTERP,LLTRACER,                        &
   &    ZTB5, ZTB)

  IF(ANY(IFAIL(:) == 2))THEN
    WRITE(CLERROR, *) 'Problems in RTTOV call for satellite ',ksat
    CALL ABOR1(CLERROR)
  ENDIF

!  Adjoint part
!+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

!*    2.5.AD  Emissivity

  ICHPF=0
  PF_LOOP_EG: DO JJPF=1,IEND
    IF(IFAIL(JJPF) == 0)THEN
      DO JCH=1,KNCH(JJPF+IDONE)
        ICHPF=ICHPF+1
        PEMIS(JJPF+IDONE,JCH) = PEMIS(JJPF+IDONE,JCH) + ZEM(ICHPF)
      ENDDO
    ENDIF
  ENDDO PF_LOOP_EG


!*   2.2.AD Additional gases

  IF(NGHG > 0) THEN
    DO JF=1,NGEMS
      DO JJPF=1,IEND
        IF(IFAIL(JJPF) == 0)THEN
          IF(YDGEMSPROF5(JJPF+IDONE,JF)%LRTTOV) THEN
            IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210061) JP=7 !CO2
            IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210063) JP=8 !N2O
            IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210062) JP=9 !CH4
            YDGEMSPROF(JJPF+IDONE,JF)%RTTOVPROF(:) = &
               & YDGEMSPROF(JJPF+IDONE,JF)%RTTOVPROF(:) &
               & + ZPROF(:,JP,JJPF)
          ENDIF
        ENDIF
      ENDDO
    ENDDO
  ENDIF
! The following bit has not been tested yet
!  IF(NGRG > 0) THEN
!    DO JF=1,NGEMS
!      DO JJPF=1,IEND
!        IF(IFAIL(JJPF) == 0)THEN
!          IF(YDGEMSPROF5(JJPF+IDONE,JF)%LRTTOV) THEN
!            IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210123) JP=10 !CO
!            YDGEMSPROF(JJPF+IDONE,JF)%RTTOVPROF(:) = &
!               & YDGEMSPROF(JJPF+IDONE,JF)%RTTOVPROF(:) &
!               & + ZPROF(:,JP,JJPF)
!          ENDIF
!        ENDIF
!      ENDDO
!    ENDDO
!  ENDIF

  IF(LCO2) THEN
    DO JJPF=1,IEND
      IF(IFAIL(JJPF) == 0)THEN
        PTQOSAT(JJPF+IDONE,:,4) = PTQOSAT(JJPF+IDONE,:,4) &
         &                         + ZPROF(:,7,JJPF)
      ENDIF
    ENDDO
  ENDIF

!*    2.1.AD  Profile variables, surface, T2m, q2m, Ts and Ps.
!*         Ozon, cloud pressure and amount, emissivity
!*         Surface indicators and angles

  DO JJPF=1,IEND
    IF(IFAIL(JJPF) == 0)THEN
      PTQOSAT(JJPF+IDONE,:,1) = PTQOSAT(JJPF+IDONE,:,1) &
        & + ZPROF(:,1,JJPF)
      PTQOSAT(JJPF+IDONE,:,2) = PTQOSAT(JJPF+IDONE,:,2) &
        & + ZPROF(:,2,JJPF)
      PTQOSAT(JJPF+IDONE,:,3) = PTQOSAT(JJPF+IDONE,:,3) &
        & + ZPROF(:,3,JJPF)

      PT2SAT(JJPF+IDONE) = PT2SAT(JJPF+IDONE) + ZPROFS(1,JJPF)
      PH2SAT(JJPF+IDONE) = PH2SAT(JJPF+IDONE) + ZPROFS(2,JJPF)
      PPSSAT(JJPF+IDONE) = PPSSAT(JJPF+IDONE) + ZPROFS(3,JJPF)/100._JPRB
      PUSSAT(JJPF+IDONE) = PUSSAT(JJPF+IDONE) + ZPROFS(4,JJPF)
      PVSSAT(JJPF+IDONE) = PVSSAT(JJPF+IDONE) + ZPROFS(5,JJPF)
      PTSSAT(JJPF+IDONE) = PTSSAT(JJPF+IDONE) + ZPROFSS(1,JJPF)
      IF(LTOVSCV .AND. LCLDSINK) THEN
        PCTOP(JJPF+IDONE)=PCTOP(JJPF+IDONE)+ZPROFCV(1,JJPF) 
        PCAMT(JJPF+IDONE)=PCAMT(JJPF+IDONE)+ZPROFCV(2,JJPF)
      ELSE
        PCTOP(JJPF+IDONE)=0.0_JPRB                              
        PCAMT(JJPF+IDONE)=0.0_JPRB                              
      ENDIF
    ENDIF
  ENDDO
  
  IDONE=IDONE+IEND

ENDDO PACKET_LOOP

!*    3.0  Deallocate RTTOV temporary arrays

CALL RTTOV_EC_ALLOC_AD(IFAIL, KSAT, IMXCHPF, NJPPF, NLSAT, &
 &    LL_LCLOUD, LL_LINTERP,                               &
 &    0_JPIM)

IF (LHOOK) CALL DR_HOOK('RADTRAD',1,ZHOOK_HANDLE)
END SUBROUTINE RADTRAD
