
SUBROUTINE RADTRK(KSAT,KSENSOR,KDPROF,KPROF,KMXCH,KNCH,PRCH, &
 & PZENANG,PAZIANG,PSOLZENANG,PSOLAZIANG,PLAT,POROG,&
 & PTQOSAT5,YDGEMSPROF5,PPSSAT5,PTSSAT5,PLSSAT5,PT2SAT5,&
 & PH2SAT5 ,PUSSAT5,PVSSAT5,PEMIS5)  

!**** *RADTRTL* - INTERFACE TO K TOVS RAD. TRANSFER CODE

!     AUTHOR: Mike Fisher (based on RADTR)     DATE  : 95-05-23

!     PURPOSE.
!     -------
!     SET UP REASONABLE CHUNKS OF WORK FOR THE TOVS RADIANCE
!     CALCULATION.
!     EXTRAPOLATE PROFILES ABOVE TOP OF MODEL IF REQUIRED.
!     CALL THE RT-CODE.

!**   INTERFACE.
!**   ---------
!**   *CALL* * RADTRK( )*  (FROM TOVCLR)
!     WHERE PZENANG: Satellite zenith Angle
!           PAZIANG: Satellite azimuth angle
!           PSOLZENANG: Solar zenith angle
!           PSOLAZIANG: Solar azimuth angle
!           PLAT:    Latitude [deg]
!           POROG:   Orography [m^2 s^-2]
!           KSAT   : NOAA-sateliite id
!           KSENSOR: Sensor id
!           KMXCH  : Maximum number of channels
!           KNCH   : Number of channels to be computed
!           PRCH   : List of channels to be computed
!           KDPROF : First dimension of arrays
!           KPROF  : Actual number of TOVS points
!           PTQOSAT: Model values at TOVS locations, T/Q/O3  (TL)
!           YDGEMSPROF5 : GEMS variables  (TL)
!           PPSSAT5 : Surface pressure  (TL)
!           PTSSAT5 : Surface temperature  (TL)
!           PT2SAT5 : 2m temperature  (TL)
!           PH2SAT5 : 2m humidity  (TL)
!           PUSSAT5 : Surface wind u  (TL)
!           PVSSAT5 : Surface wind v  (TL)
!           PEMIS5  : Surface emissivity for each channel  (TL)
!           PLSSAT5 : MODEL SURFACE TYPE
!**   EXTERNALS
!     ---------
!        RTTOVK - RT-CODE

!     MODIFICATIONS
!     -------------
!     Erik Andersson : 970220 - F90, simplify code.
!     Roger Saunders : 970811 - Include ozone, emissivity & cloud as 
!                               variables
!     Rose Munro     : 980108 - Include satellite view angle
!     Rose Munro     : 980620 - Alter tests on max/min regr. prof. for new RT
!     Roger Saunders : 980821 - Changes for RTTOV5 interface
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     Niels Bormann  : 071023 - Changes for RTTOV-9 interface (routine never tested)
!     Alan Geer      : 080718   Fix units for orography (IFS uses m^2 s^-2)
!        V.Guidard     23-Feb-2007 Add arg KSENSOR & add IASI
!*    ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMVAR, ONLY : LCO2
USE YOMCT0B  , ONLY : LECMWF
USE YOMDIMO  , ONLY :     NLSAT    ,NMXTCHU  ,&
 & NPNAV    ,NPNSAV   ,NPNSSV   ,NPNCV        
USE YOMTVRAD , ONLY : NJPPF ,RADPRE,&
 & CO2BGE_T , MSENSOR_AIRS, MSENSOR_IASI, AIRSCHAN_BACK, IASICHAN_BACK 
USE YOMANCS  , ONLY : NMDI
USE YOM_YGFL , ONLY : NGHG
USE YOMCST   , ONLY : RG
USE TYPE_GEMS_PROFILES
USE YOMDB
IMPLICIT NONE

!*    0.1  Global arrays

REAL(KIND=JPRB)   ,INTENT(IN)    :: PZENANG(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAZIANG(KDPROF)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOLZENANG(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOLAZIANG(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAT(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KDPROF) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSENSOR
INTEGER(KIND=JPIM),INTENT(IN)    :: KMXCH
INTEGER(KIND=JPIM),INTENT(IN)    :: KNCH(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCH(KDPROF,KMXCH) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPROF
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTQOSAT5(KDPROF,NLSAT,NPNAV-1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPSSAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSSAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSSAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT2SAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PH2SAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUSSAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVSSAT5(KDPROF) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS5(KDPROF,KMXCH) 
TYPE (TYPE_GEMSPROF)   ,INTENT(IN) :: YDGEMSPROF5(KDPROF,NGEMS)

INTEGER(KIND=JPIM), DIMENSION(NJPPF*NMXTCHU) :: ICHID,IPFID
INTEGER(KIND=JPIM), DIMENSION(NJPPF)         :: ISURF
INTEGER(KIND=JPIM), DIMENSION(NJPPF)         :: INCH
INTEGER(KIND=JPIM), DIMENSION(NJPPF)         :: IFAIL
REAL(KIND=JPRB), DIMENSION(NJPPF*NMXTCHU):: ZTBCLR ! Clear brightness temperatures (AD)
REAL(KIND=JPRB), DIMENSION(NJPPF*NMXTCHU):: ZTBCLR5 ! Clear brightness temperatures

REAL(KIND=JPRB), DIMENSION(NLSAT)            :: ZPPRES
REAL(KIND=JPRB), DIMENSION(NLSAT,NPNAV,NJPPF):: ZPROF5
REAL(KIND=JPRB), DIMENSION(NPNSAV,NJPPF)     :: ZPROFS5
REAL(KIND=JPRB), DIMENSION(NPNSSV,NJPPF)     :: ZPROFSS5
REAL(KIND=JPRB), DIMENSION(NPNCV,NJPPF)      :: ZPROFCV5
REAL(KIND=JPRB), DIMENSION(NJPPF)            :: ZLAT,ZOROG
REAL(KIND=JPRB), DIMENSION(NJPPF)            :: ZZENANG,ZAZIANG
REAL(KIND=JPRB), DIMENSION(NJPPF)            :: ZSOLZENANG,ZSOLAZIANG
!**
REAL(KIND=JPRB), DIMENSION(NLSAT,NPNAV,NJPPF*NMXTCHU):: ZPROF
REAL(KIND=JPRB), DIMENSION(NPNSAV,NJPPF*NMXTCHU)     :: ZPROFS
REAL(KIND=JPRB), DIMENSION(NPNSSV,NJPPF*NMXTCHU)     :: ZPROFSS
REAL(KIND=JPRB), DIMENSION(NPNCV,NJPPF*NMXTCHU)      :: ZPROFCV
!**
REAL(KIND=JPRB), DIMENSION(NJPPF*NMXTCHU)    :: ZEM5
REAL(KIND=JPRB), DIMENSION(NJPPF*NMXTCHU)    :: ZEM

INTEGER(KIND=JPIM) :: IPACK,IFIN,IEND,IDONE,ICHPF,IMXCHPF
INTEGER(KIND=JPIM) :: JJPACK,JJPF,JCH,JF,JLEV,JP
INTEGER(KIND=JPIM), DIMENSION(KDPROF) :: ILTROP
REAL(KIND=JPRB) :: ZERROR_T
REAL(KIND=JPRB), DIMENSION(KMXCH) :: ZK_T
LOGICAL :: LL_LCLOUD
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "tropopause.intfb.h"

#include "openmp_obs.h"

IF (LHOOK) CALL DR_HOOK('RADTRK',0,ZHOOK_HANDLE)

LL_LCLOUD = .FALSE.

!*    1.0  Number of packets
!*    ------------------------------------------------------------------

IPACK=KPROF/NJPPF+1
IFIN=MOD(KPROF,NJPPF)
IF(IFIN == 0) THEN
  IPACK=IPACK-1
  IFIN=NJPPF
ENDIF

CALL TROPOPAUSE(KDPROF,KPROF,PTQOSAT5,ILTROP)

!*    1.1  Pressure Pa--> hPa

ZPPRES(:)=RADPRE(:)/100._JPRB

!*    1.2  Allocate RTTOV temporary arrays

IMXCHPF=KMXCH*NJPPF
CALL RTTOV_EC_ALLOC_K(IFAIL, KSAT, IMXCHPF, NJPPF, NLSAT, &
 &    LL_LCLOUD,                                          &
 &    1_JPIM)

!*    2.0  Make packets, Call RT-code.
!*    ------------------------------------------------------------------

IDONE=0
PACKET_LOOP: DO JJPACK=1,IPACK
  IFAIL(:)=0

  IF(JJPACK == IPACK) THEN
    IEND=IFIN
  ELSE
    IEND=NJPPF
  ENDIF


!*    2.1  Temperature=1, Humidity=2, Ozone=3, CLW=4

  DO JJPF=1,IEND
    ZPROF5(1:NLSAT,1,JJPF)=PTQOSAT5(JJPF+IDONE,1:NLSAT,1)  ! T
    ZPROF5(1:NLSAT,2,JJPF)=PTQOSAT5(JJPF+IDONE,1:NLSAT,2)  ! Q 
    ZPROF5(1:NLSAT,3,JJPF)=PTQOSAT5(JJPF+IDONE,1:NLSAT,3)  ! O3
  ENDDO
  ZPROF5(1:NLSAT,4:6,1:IEND)=0.0_JPRB

!*   2.2 GEMS greenhouse gases

  IF(NGHG > 0) THEN
    DO JF=1,NGEMS
      DO JJPF=1,IEND
        IF(YDGEMSPROF5(JJPF+IDONE,JF)%LRTTOV) THEN
          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210061) JP=7 !CO2
          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210063) JP=8 !N2O
          IF(YDGEMSPROF5(JJPF+IDONE,JF)%IGRIB == 210062) JP=9 !CH4
          ZPROF5(:,JP,JJPF)=YDGEMSPROF5(JJPF+IDONE,JF)%RTTOVPROF(:)
        ENDIF
      ENDDO
    ENDDO
  ENDIF

!*   2.3 CO2

  IF(LCO2) THEN
    DO JJPF=1,IEND
      ZPROF5(JLEV,7,JJPF)=PTQOSAT5(JJPF+IDONE,JLEV,4)
    ENDDO
  ENDIF
!*    2.4  Surface, T2m, q2m, Ts and Ps.
!*         Ozon, cloud pressure and amount, emissivity
!*         Surface indicators and angles

  PF_LOOP_S: DO JJPF=1,IEND

    ZPROFS5(1,JJPF)=PT2SAT5(JJPF+IDONE)
    ZPROFS5(2,JJPF)=PH2SAT5(JJPF+IDONE)
    ZPROFS5(3,JJPF)=PPSSAT5(JJPF+IDONE)/100._JPRB
    ZPROFS5(4,JJPF)=PUSSAT5(JJPF+IDONE)
    ZPROFS5(5,JJPF)=PVSSAT5(JJPF+IDONE)
    ZPROFSS5(1,JJPF)=PTSSAT5(JJPF+IDONE)
    ZPROFCV5(1,JJPF)=500._JPRB     ! cloud top pressure
    ZPROFCV5(2,JJPF)=0.0_JPRB      ! cloud amount

    ZZENANG(JJPF)=PZENANG(JJPF+IDONE)
    ZAZIANG(JJPF)=PAZIANG(JJPF+IDONE)
    ZSOLZENANG(JJPF)=PSOLZENANG(JJPF+IDONE)
    ZSOLAZIANG(JJPF)=PSOLAZIANG(JJPF+IDONE)
    ZLAT(JJPF)  =PLAT(JJPF+IDONE)
    ZOROG(JJPF) =POROG(JJPF+IDONE)/RG ! convert to m
    IF(PLSSAT5(JJPF+IDONE)  <=  0.5_JPRB) THEN
      ISURF(JJPF)=1
    ELSE
      ISURF(JJPF)=0
    ENDIF
  ENDDO PF_LOOP_S

!*    2.5  Set indices of TOVS channels
!*         Perform radtiative transfer calculations

  ICHPF=0
  PF_LOOP_RI: DO JJPF=1,IEND
    INCH(JJPF)=KNCH(JJPF+IDONE)
    DO JCH=1,KNCH(JJPF+IDONE)
      ICHPF=ICHPF+1
      ICHID(ICHPF)=NINT(PRCH(JJPF+IDONE,JCH))
      IPFID(ICHPF)=JJPF
      ZEM5(ICHPF)=PEMIS5(JJPF+IDONE,JCH)
    ENDDO
  ENDDO PF_LOOP_RI

  ! Reset channel numbers to the values in the reduced set initialised in 
  ! RTTVI
  IF(LECMWF)THEN
    SELECT CASE (KSENSOR)
      CASE (MSENSOR_AIRS)
        ICHID(1:ICHPF) = AIRSCHAN_BACK(ICHID(1:ICHPF))
      CASE (MSENSOR_IASI)
        ICHID(1:ICHPF) = IASICHAN_BACK(ICHID(1:ICHPF))
    END SELECT
  ENDIF
  IF (ANY(ICHID(1:ICHPF) == -NMDI)) &
       & CALL ABOR1('Error in channel subsetting for RTTOV')

  CALL RTTOV_EC_K(IFAIL, KSAT, ICHPF, IEND, NLSAT,           &
   &    ICHID, IPFID,                                         &
   &    ZPPRES, ZPROF5,ZPROFS5,ZPROFSS5,ISURF,ZEM5,ZPROFCV5,  &
   &            ZPROF, ZPROFS, ZPROFSS,       ZEM, ZPROFCV,   &
   &    ZZENANG, ZAZIANG, ZSOLZENANG, ZSOLAZIANG, ZLAT, ZOROG,&
   &    LL_LCLOUD,                                            &
   &    ZTBCLR5, ZTBCLR)

!*    2.5  Construct output array containing model radiances

  ICHPF=0
  PF_LOOP_RO: DO JJPF=1,IEND
    DO JCH=1,KNCH(JJPF+IDONE)
      ICHPF=ICHPF+1
      ZK_T(JCH)=SUM(ZPROF(ILTROP(JJPF+IDONE):NLSAT,5,ICHPF))/ROBODY(MLNKH2B(JJPF+IDONE)-1+JCH,MDBFOE)
    ENDDO
    ZERROR_T=sqrt(1.0_JPRB/(CO2BGE_T**(-2)+ &
     & dot_product(ZK_T(1:KNCH(JJPF+IDONE)),ZK_T(1:KNCH(JJPF+IDONE)))))
     if (LCO2) then
      ROBDDR(JJPF+IDONE,MDB_CO2ANER_S_AT_CO2_SINK)=ZERROR_T
     endif
  ENDDO PF_LOOP_RO

  IDONE=IDONE+IEND


ENDDO PACKET_LOOP

!*    3.0  Deallocate RTTOV temporary arrays

CALL RTTOV_EC_ALLOC_K(IFAIL, KSAT, IMXCHPF, NJPPF, NLSAT, &
 &    LL_LCLOUD,                                          &
 &    0_JPIM)

IF (LHOOK) CALL DR_HOOK('RADTRK',1,ZHOOK_HANDLE)
END SUBROUTINE RADTRK
