SUBROUTINE REO3BCOR(KOBS,KSAT,KSENS,P_ZSOE)

!**** *REO3BCOR - Calculates bias correction for ozone data

!     PURPOSE
!     -------
!     Routine called by hrert.F90 to estimate bias for reo3 ozone layers
!     and load bias into odb for later use by analysis.

!     INPUT
!     -----
!     kobs      - report pointer
!     ksat      - satellite id                            
!     ksens     - observation sensor id                            
!     zsoe      - solar elevation (used as predictor)

!     OUTPUT
!     ------
!     NONE

!     EXTERNALS
!     ---------
!     NONE

!     AUTHOR
!     ------
!     Antje Dethof 28/04/2003

!     MODIFICATIONS
!     -------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!---------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDB
USE yom_reo3_bcor, ONLY : o3bc_info,nbc_info
USE YOMLUN   , ONLY : NULOUT   
USE YOMANCS  , ONLY : RMDI
USE YOMVNMB  , ONLY : NVNUMB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KOBS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSENS 
REAL(KIND=JPRB)   ,INTENT(IN)    :: P_ZSOE 
! Local variables
INTEGER(KIND=JPIM) :: jc,jcor,jpred,jbody
REAL(KIND=JPRB)    :: zbias
REAL(KIND=JPRB), ALLOCATABLE    :: zpreds(:)    ! Predictors needed
LOGICAL, ALLOCATABLE :: llgood_preds(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcobs.h"

#include "openmp_obs.h"

!====================================================

!*    1    Preparations
!         ------------

IF (LHOOK) CALL DR_HOOK('REO3BCOR',0,ZHOOK_HANDLE)
IF(nbc_info <= 0 .AND. LHOOK) CALL DR_HOOK('REO3BCOR',1,ZHOOK_HANDLE)
IF(nbc_info <= 0) RETURN

jcor=0
DO jc=1,nbc_info
  IF(o3bc_info(jc)%sat_id == ksat   &
   & .AND. o3bc_info(jc)%sensor == ksens) jcor = jc  
ENDDO

!IF (jcor <= 0) THEN
!  WRITE(nulout,*) 'INFO REO3BCOR: No bias correction information available!'
!  WRITE(nulout,*) '  Satid,sensor:',ksat,ksens
!  WRITE(nulout,*) '  No bias correction calculated.'
! RETURN
!ENDIF

!     2   Compute bias from model input
!         -----------------------------

IF(o3bc_info(jcor)%npreds > 0)THEN

  ALLOCATE(zpreds(o3bc_info(jcor)%npreds))
  ALLOCATE(llgood_preds(o3bc_info(jcor)%npreds))
  llgood_preds=.false.

  DO jpred = 1, o3bc_info(jcor)%npreds
    SELECT CASE (TRIM(o3bc_info(jcor)%pred_name(jpred)))
    CASE ('soe')
      IF( P_ZSOE /= rmdi )THEN
        zpreds(jpred) = P_ZSOE
      ELSE
        WRITE(nulout,*) 'WARNING REO3BCOR: soe not avail.'
        WRITE(nulout,*) '  satid,sensor',o3bc_info(jcor)%sat_id,&
         & o3bc_info(jcor)%sensor  
        WRITE(nulout,*) '  No bias correction calculated.'
        DEALLOCATE(zpreds)
        IF (LHOOK) CALL DR_HOOK('REO3BCOR',1,ZHOOK_HANDLE)
        RETURN
      ENDIF
    CASE DEFAULT
      WRITE(nulout,*) 'WARNING REO3BCOR: Unimplemented predictor name: '//&
       & TRIM(o3bc_info(jcor)%pred_name(jpred))  
      WRITE(nulout,*) '  No bias correction calculated.'
      DEALLOCATE(zpreds)
      IF (LHOOK) CALL DR_HOOK('REO3BCOR',1,ZHOOK_HANDLE)
      RETURN
    END SELECT
  ENDDO

!     3   Apply correction
!         ----------------

  zbias=0.0_JPRB
  WHERE(zpreds >= o3bc_info(jcor)%pred_min .AND. &
     & zpreds <= o3bc_info(jcor)%pred_max)  
    llgood_preds = .true.
  ELSEWHERE
    llgood_preds = .false.
  ENDWHERE

  IF (ANY(llgood_preds)) THEN
    zbias = o3bc_info(jcor)%coef(0)     &
     & + SUM(zpreds*o3bc_info(jcor)%coef(1:o3bc_info(jcor)%npreds), &
     & MASK = llgood_preds)  
  ELSE
    WRITE(nulout,*) 'INFO REO3BCOR: Outside valid range of predictors.'
    WRITE(nulout,*) '  No bias correction calculated.'
  ENDIF

!     4   Load biases into ODB
!         --------------------

  DO JBODY= MLNKH2B(KOBS),MLNKH2B(KOBS+1)-1      ! loop body entries 
    IF(ROBODY(JBODY,MDBVNM) == NVNUMB(68)) THEN  ! Found reo3 datum
      IF(ROBODY(JBODY,MDBVAR) /= RMDI) THEN 
        ROBODY(JBODY,MDBTORB)= ROBODY(JBODY,MDBVAR)*zbias !Bias estimate
      ELSE
        ROBODY(JBODY,MDBTORB)=RMDI
      ENDIF
    ENDIF
  ENDDO

!*    5   Tidy up
!         -------

  DEALLOCATE(zpreds)
  DEALLOCATE(llgood_preds)
!ELSE
!  WRITE(nulout,*) 'REO3BCOR: No bias correction applied'
!  WRITE(nulout,*) '  satid,sensor',o3bc_info(jcor)%sat_id,&
!    &                o3bc_info(jcor)%sensor
ENDIF

IF (LHOOK) CALL DR_HOOK('REO3BCOR',1,ZHOOK_HANDLE)
END SUBROUTINE REO3BCOR
