!  Differentiation of scatra_gems in reverse (adjoint) mode:
!   gradient, with respect to input variables: pizmoy pbetal ptodaer
!                ptamoy
!   of linear combination of output variables: pudiftt pudirtt
!                pddiftt psphalbt pddirtt

SUBROUTINE SCATRA_GEMS_AD(ptamoy, ptamoyb, ptray, ptodaer, ptodaerb, &
&  ptodray, pizmoy, pizmoyb, prm, pgb, pxmus, pxmuv, pbetal, pbetalb, &
&  ppsl, pddirtt, pddirttb, pddiftt, pddifttb, pudirtt, pudirttb, &
&  pudiftt, pudifttb, psphalbt, psphalbtb)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE rt6svalues

IMPLICIT NONE

!****SCATRA*** COMPUTES SCATTERING TRANSMISSION FUNCTIONS FOR THE THREE ATMOPHERIC MODELS,
!              RAYLEIGH, AEROSOL AND MIXTURE


!     INTERFACE
!     ---------
! SCATRA IS CALLED BY DISCOM

!     INPUTS
!     ------

! PXMUS                     : COSINE OF SOLAR ZENITH ANGLE
! PXMUV                     : COSINE OF VIEWING ZENITH ANGLE
! PRM(-KMU:KMU)             : GAUSSIAN ANGLES FOR THE ZENITH
! PGB(-KMU:KMU)             : GAUSSIAN WEIGHTS FOR THE ZENITH
! PIZMOY(KNT)               : TRUNCATED AEROSOL SINGLE SCATTERING ALBEDO PROFILE.
! PTODAER                   : TOTAL AEROSOL OPTICAL DEPTH
! PTODRAY                   : TOTAL MOLECULAR OPTICAL DEPTH
! PTAMOY(0:KNT)             : TRUNCATED AEROSOL OPTICAL THICKNESS PROFILE.
! PTRAY(0:KNT)              : MOLECULAR OPTICAL THICKNESS PROFILE.
! PBETAL(KNT,KNANG+1)       : COEFFICIENTS OF LEGENDRE POLYNOM
! PPSL(0:KNANG,-1:KNANG,-KMU:KMU)

!     OUTPUTS
!     -------

! PDDIRTT  : TOTAL DIRECT DOWNWARD TRANSMITTANCE
! PDDIFTT  : TOTAL DIFUSE DOWNWARD TRANSMITTANCE
! PUDIRTT  : TOTAL DIRECT UPWARD TRANSMITTANCE
! PUDIFTT  : TOTAL DIFUSE UPWARD TRANSMITTANCE
! PSPHALBT : TOTAL SPHERICAL ALBEDO

!     EXTERNALS.
!     ----------
!          ISO & CSALBR

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007

!     SOURCE
!     ------
! ADAPTED FROM SCATRA.f FROM ORIGINAL 6S CODE, SEE VERMOTTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------

  REAL(KIND=JPRB), INTENT(IN) :: pxmus, pxmuv
  REAL(KIND=JPRB), INTENT(IN) :: ptodaer, ptodray
  REAL(KIND=JPRB), INTENT(INOUT) :: ptodaerb
  REAL(KIND=JPRB), INTENT(IN) :: ptamoy(0:knt), ptray(0:knt)
  REAL(KIND=JPRB), INTENT(INOUT) :: ptamoyb(0:knt)
  REAL(KIND=JPRB), INTENT(IN) :: pgb(-kmu:kmu)
  REAL(KIND=JPRB), INTENT(IN) :: pizmoy(0:knt)
  REAL(KIND=JPRB), INTENT(INOUT) :: pizmoyb(0:knt)
  REAL(KIND=JPRB), INTENT(IN) :: pbetal(0:knt, 0:knang)
  REAL(KIND=JPRB), INTENT(INOUT) :: pbetalb(0:knt, 0:knang)
  REAL(KIND=JPRB), INTENT(IN) :: ppsl(0:knang, -1:knang, -kmu:kmu)
  REAL(KIND=JPRB), INTENT(INOUT) :: prm(-kmu:kmu)
  REAL(KIND=JPRB), INTENT(IN) :: pddirtt
  REAL(KIND=JPRB), INTENT(INOUT) :: pddirttb
  REAL(KIND=JPRB), INTENT(IN) :: pddiftt
  REAL(KIND=JPRB), INTENT(INOUT) :: pddifttb
  REAL(KIND=JPRB), INTENT(IN) :: pudirtt
  REAL(KIND=JPRB), INTENT(INOUT) :: pudirttb
  REAL(KIND=JPRB), INTENT(IN) :: pudiftt
  REAL(KIND=JPRB), INTENT(INOUT) :: pudifttb
  REAL(KIND=JPRB), INTENT(IN) :: psphalbt
  REAL(KIND=JPRB), INTENT(INOUT) :: psphalbtb
! LOCAL VARIABLES
! ---------------
!INTEGER(KIND=JPIM),PARAMETER  :: KMU=25, KMU2=48, KNP=49, KNANG=80, KNBMU=83, KNT=20
  REAL(KIND=JPRB) :: z_ptrans(-1:1)
  REAL(KIND=JPRB) :: z_ptransb(-1:1)
  INTRINSIC EXP

  REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "iso_gems_ad.intfb.h"
#include "popreal8.intfb.h"
#include "pushreal8.intfb.h"

  IF (LHOOK) CALL DR_HOOK('SCATRA_GEMS_AD',0,ZHOOK_HANDLE)

  CALL PUSHREAL8(prm(-kmu))
! INITIALIZING VARIABLES
! ----------------------
! UPWARD TRANSMITTANCE
  prm(-kmu) = -pxmuv
  CALL PUSHREAL8(prm(kmu))
  prm(kmu) = pxmuv
  CALL PUSHREAL8(prm(0))
  prm(0) = pxmus
  CALL PUSHREAL8(prm(-kmu))
! DOWNWARD TRANSMITTANCE 
  prm(-kmu) = -pxmus
  CALL PUSHREAL8(prm(kmu))
  prm(kmu) = pxmus
  CALL PUSHREAL8(prm(0))
  prm(0) = pxmus
  z_ptransb(-1:1) = 0._JPRB
  z_ptransb(0) = 2.0_JPRB*psphalbtb
  z_ptransb(1) = z_ptransb(1) + pddifttb
  pddirttb = pddirttb - pddifttb
  ptodaerb = -(EXP(-((ptodray+ptodaer)/pxmus))*pddirttb/pxmus)
  pbetalb(:, :) = 0._JPRB
  pizmoyb(:) = 0._JPRB
  ptamoyb(:) = 0._JPRB
  CALL ISO_GEMS_AD(ptamoy, ptamoyb, ptray, pizmoy, pizmoyb, pbetal, &
&            pbetalb, prm, pgb, ppsl, ptodaer, ptodaerb, ptodray, z_ptrans, &
&            z_ptransb)
  z_ptransb = 0._JPRB
  CALL POPREAL8(prm(0))
  CALL POPREAL8(prm(kmu))
  CALL POPREAL8(prm(-kmu))
  z_ptransb(-1) = z_ptransb(-1) + pudifttb
  pudirttb = pudirttb - pudifttb
  ptodaerb = ptodaerb - EXP(-((ptodray+ptodaer)/pxmuv))*pudirttb/pxmuv
  CALL ISO_GEMS_AD(ptamoy, ptamoyb, ptray, pizmoy, pizmoyb, pbetal, &
&            pbetalb, prm, pgb, ppsl, ptodaer, ptodaerb, ptodray, z_ptrans, &
&            z_ptransb)
  CALL POPREAL8(prm(0))
  CALL POPREAL8(prm(kmu))
  CALL POPREAL8(prm(-kmu))

  IF (LHOOK) CALL DR_HOOK('SCATRA_GEMS_AD',1,ZHOOK_HANDLE)
END SUBROUTINE SCATRA_GEMS_AD
