SUBROUTINE SURBOUND(&
 & KPROMA,KIDIA,KFDIA,&
 & PAPHI,PQ,PT,PTS,PAPH,&
 & PWS,PVGRAT,PWL,PSN,PSLM,&
 & PES,PCPTGZ,PWET,PQS,PCPTS)  

!**** *SURBOUND* - NEW THERMODYNAMIC VARIABLE AND BOUNDARY CONDITION

!     PURPOSE
!     -------
!          COMPUTE THE NEW THERMODYNAMIC VARIABLE ,THE STATIC
!          ENERGY, TO USE INSTEAD OF THE TEMPERATURE VARIABLE
!          AND COMPUTE THE BOUNDARY CONDITIONS

!**   INTERFACE
!     ---------
!    *CALL* *SURBOUND(
!    C                 KPROMA,KIDIA,KFDIA,NFLEVG,
!    I                 ZAPHI5(1,NFLEVG),ZQF5(1,NFLEVG),ZTF5(1,NFLEVG),
!    I                 ZT05,ZPRESH5(1,NFLEVG),
!    I                 ZWS5,ZVGRAT5,ZWL5,
!    I                 ZSN5,ZSLM5,
!    O                 ZCPTGZ5,ZWET5,ZQS5(1,NFLEVG+1),ZCPTS5)

!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA            - HORIZONTAL DIMENSION.                  (INPUT-C)
!        KIDIA             - INDEX OF START POINT                   (INPUT-C)
!        KFDIA             - INDEX OF END   POINT                   (INPUT-C)
!        PAPHI (KPROMA)    - GEOPOTENTIAL MODEL LEVELS    M**2/S**2 (INPUT)
!        PQ (KPROMA)       - SPECIFIC HUMIDITY ON THE LOWER
!                            LEVEL.                         Kg/Kg   (INPUT)
!        PT (KPROMA)       - TEMPERATURE ON THE LOWER LEVEL   K     (INPUT)
!        PTS (KPROMA)      - TEMPERATURE AT THE SURFACE       K     (INPUT)
!        PAPH (KPROMA)     - HALF PRESSURE  ON THE LOWER
!                            LEVEL                            Pa    (INPUT)
!        PWS (KPROMA)      - SOIL MOISTURE                    M     (INPUT)
!        PVGRAT (KPROMA)   - VEGETATION RATIO                       (INPUT)
!        PWL (KPROMA)      - SKIN RESERVOIR WATER CONTENT     M     (INPUT)
!        PSN (KPROMA)      - SNOW (EQUIVALENT WATER DEPTH)    M     (INPUT)
!        PSLM (KPROMA)     - SEA/LAND MASK                          (INPUT)
!        PCPTGZ (KPROMA)   - STATIC ENERGY (LOWER LEVEL)  M**2/S**2 (OUTPUT)
!        PQS (KPROMA)      - SATURATION HUMIDITY            Kg/Kg   (OUTPUT)
!        PWET (KPROMA)     - WETNESS                          M     (OUTPUT)
!        PCPTS (KPROMA)    - STATIC ENERGY AT THE SURFACE M**2/S**2 (OUTPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        *GPRH*

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        CARLA CARDINALI *ECMWF*

!        ORIGINAL   : 92-05-22
!        L. Isaksen : 96-02-20  NFLEV --> NFLEVG
!        MODIFICATIONS:
!        P. MOLL 18 juillet 1996  ACHMT transplantation
!        P. Viterbo    28-May-2004 surf library and new surface units.
!        M. Ko"hler    03-Dec-2004 delete unsafe redefinition of RVTMP2
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0B  , ONLY : LECMWF
USE YOMCST   , ONLY : RPI      ,RV       ,RCPD     ,RCPV     ,&
 & RETV     ,RCW      ,RCS      ,RLVTT    ,RLSTT    ,&
 & RTT      ,RALPW    ,RBETW    ,RGAMW    ,RALPS    ,&
 & RBETS    ,RGAMS    ,RALPD    ,RBETD    ,RGAMD  
USE YOETHF   , ONLY : RVTMP2
USE YOMPHY1  , ONLY : WCRIN    ,WSMX

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVGRAT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PES(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCPTGZ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWET(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCPTS(KPROMA) 
!C

REAL(KIND=JPRB) :: ZRH(KPROMA)
REAL(KIND=JPRB) :: ZHU(KPROMA),ZQSATS(KPROMA),ZHQ(KPROMA),ZESP(KPROMA)
REAL(KIND=JPRB) :: ZDELT(KPROMA),ZNEIJ(KPROMA)

INTEGER(KIND=JPIM) :: JL, JLON

REAL(KIND=JPRB) :: ZCPVMD, ZDEW, ZEW, ZQWSBCR, ZRHMAX, ZRHMIN,&
 & ZSN, ZVGRAT, ZVLT, ZWET1, ZWET2, ZWL, &
 & ZWLMXI, ZWS
REAL(KIND=JPRB) :: ZRQSNCR,ZRWLMAX
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "surf_inq.h"
END INTERFACE

#include "gprh.intfb.h"

#include "fcttrm.h"

!     1.     DEFINITION OF THE CONSTANTS
!            ---------------------------

IF (LHOOK) CALL DR_HOOK('SURBOUND',0,ZHOOK_HANDLE)
CALL SURF_INQ(PRQSNCR=ZRQSNCR,PRWLMAX=ZRWLMAX,PRQWSBCR=ZQWSBCR)

!*    1.1    PHISICAL CONSTANTS

ZVLT=4._JPRB
ZCPVMD=RCPV-RCPD

!*         2.     NEW THERMODYNAMIC VARIABLE AND BOUNDARY CONDITIONS.
!                 ---------------------------------------------------

!*         2.1     REPLACE T BY CP(Q)*T+GZ IN THE ATMOSPHERE.

DO JL=KIDIA,KFDIA
  PCPTGZ(JL)=PAPHI(JL)+PT(JL)*RCPD*(1.0_JPRB+RVTMP2*PQ(JL))
ENDDO

!*         2.2     COMPUTE THE SATURATION HUMIDITY AT THE SURFACE.

ZRHMAX=1._JPRB
ZRHMIN=0._JPRB
CALL GPRH(.FALSE.,KPROMA,KIDIA,KFDIA, 1 ,ZRHMAX,ZRHMIN,PQ,PTS,PAPH,PES,ZRH)
DO JL=KIDIA,KFDIA
  PQS(JL)=PES(JL)/(PAPH(JL)*(RETV+1.0_JPRB)-RETV*PES(JL))
ENDDO

!*         2.3     SOIL MOISTURE PARAMETERS AND CP(Q)*T AT SURFACE.

DO JL=KIDIA,KFDIA
  IF(PSLM(JL) <= 0.5_JPRB.OR.PQ(JL) > PQS(JL)) THEN
    PWET(JL)=1.0_JPRB
  ELSE
    ZVGRAT=PVGRAT(JL)*ZVLT
    ZWLMXI=1.0_JPRB/(ZRWLMAX*(1.0_JPRB-PVGRAT(JL)+ZVGRAT))
    ZWS=PWS(JL)*ZQWSBCR
    ZWL=PWL(JL)*ZWLMXI
    ZSN=PSN(JL)*ZRQSNCR
    ZWET1=0.5_JPRB*(1.0_JPRB-COS(ZWS))
    ZWET2=ZWL+(1.0_JPRB-ZWL)*ZWET1
    IF(ZSN > 1.0_JPRB)THEN
      ZSN=1._JPRB
    ENDIF
    PWET(JL)=ZSN+(1.0_JPRB-ZSN)*ZWET2
  ENDIF
  PCPTS(JL)=PTS(JL)*RCPD*(1.0_JPRB+RVTMP2*PWET(JL)*PQS(JL))
ENDDO

IF(.NOT.LECMWF)THEN
! prepare use of achmt Ri number
  DO JLON=KIDIA,KFDIA

    ZNEIJ(JLON)=PSLM(JLON)*PSN(JLON)/(PSN(JLON)+WCRIN)
    ZDELT(JLON)=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PTS(JLON)))

    ZEW= FOEW (PTS(JLON),ZDELT(JLON))
    ZESP(JLON)=ZEW/PAPH(JLON)
    ZQSATS(JLON)= FOQS (ZESP(JLON))

    ZHU(JLON)=1.0_JPRB+PSLM(JLON)*(1.0_JPRB-ZNEIJ(JLON))*(0.5_JPRB*(1.0_JPRB-COS(RPI &
     & *PWS(JLON)/WSMX))-1.0_JPRB)  
    ZHQ(JLON)=(1.0_JPRB-ZHU(JLON))*PVGRAT(JLON)
    ZDEW=MAX(0.0_JPRB,SIGN(1.0_JPRB,PQ(JLON)-ZQSATS(JLON)))
    ZHU(JLON)=ZHU(JLON)+ZDEW*ZHQ(JLON)
    ZHQ(JLON)=ZHQ(JLON)-ZDEW*ZHQ(JLON)

    PQS(JLON)=ZHU(JLON)*ZQSATS(JLON)+ZHQ(JLON)*PQ(JLON)
    PCPTS(JLON)=RCPD+ZCPVMD*PQS(JLON)

  ENDDO

ENDIF

! ---------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SURBOUND',1,ZHOOK_HANDLE)
END SUBROUTINE SURBOUND
