!  Differentiation of trunca_gems in reverse (adjoint) mode:
!   gradient, with respect to input variables: phasef
!   of linear combination of output variables: pcoeff pbetal phasef

SUBROUTINE TRUNCA_GEMS_AD(phasef, phasefb, prmu, pga, ppl, pbetal, &
&  pbetalb, pcoeff, pcoeffb)

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE rt6svalues

IMPLICIT NONE

!****TRUNCA****DECOMPOSITION OF THE AEROSOL PHASE FUNCTION IN SERIES OF LEGENDRE POLYNOMIAL 
!              USED IN OS.f AND ISO.f

! THE TRUNCATUR METHOD CONSISTS IN DEFINING THE PHASE FUNCTION CORRESPONDING TO A COSENUS OF GAUSSIAN
! ANGLE OF 0.8 & 0.94. FROM THIS TWO POINTS THE PHASE FUNCTION OF MU=0 IS EXTRAPOLATED. THE TRUNCATUR
! COEFFICIENT (COEFF) IS DETERMINED ACCORDING TO THE PHASE FUNCTION THAT IS NOT ABOVE OR HIGHER THAN THE 
! EXTRAPOLATED VALUE.

!     INTERFACE
!     ---------
! TRUNCA IS CALLED BY DISCOM  

!     INPUTS
!     ------

! PHASEF(KNT,KNBMU)     : Aerosol phase function profile.
! PRMU(KNANG)           : GAUSSIAN ANGLES FOR TRUNCATION OF PHASE FUNCTION 
! PGA(KNANG)            : GAUSSIAN WEIGHTS FOR TRUNCATION OF PHASE FUNCTION
! PPL(KNBMU,-1:KNANG+1) : USED IN TRUNCATION OF PHASE FUNCTION

!     OUTPUTS
!     -------

! PCOEFF(KNT)         : TRUNCATION COEFFICIENT
! PBETAL(KNT,KNANG+1) : COEFFICIENTS OF LEGENDRE POLYNOM

!     EXTERNALS.
!     ----------

!     MODIFICATIONS.
!     -------------
!          N.HUNEEUS 02.05.2007

!     SOURCE
!     ------
! ADAPTED FROM TRUNCA.f FROM ORIGINAL 6S CODE, SEE VERMOTE ET AL. (1997)

!     MODEL PARAMETERS
!     ----------------
!--------------------------------------------------------------------------------------
!INTEGER(KIND=JPIM), PARAMETER :: KNT=20, KNBMU=83, KNANG=80
  REAL(KIND=JPRB), INTENT(INOUT) :: phasef(0:knt, knbmu)
  REAL(KIND=JPRB), INTENT(INOUT) :: phasefb(0:knt, knbmu)
  REAL(KIND=JPRB), INTENT(IN) :: prmu(knbmu), pga(knbmu)
  REAL(KIND=JPRB), INTENT(IN) :: ppl(knbmu, -1:knang+1)
  REAL(KIND=JPRB), INTENT(INOUT) :: pbetal(0:knt, 0:knang)
  REAL(KIND=JPRB), INTENT(IN) :: pcoeff(0:knt)
  REAL(KIND=JPRB), INTENT(INOUT) :: pbetalb(0:knt, 0:knang), pcoeffb(0:knt)
! LOCAL VARIABLE
! --------------
  INTEGER(KIND=JPIM) :: ind(1), j, j1, jk, jkk, jang, jlev
  REAL(KIND=JPRB) :: zaa, z1, z2, za, zz1, z
  REAL(KIND=JPRB) :: zaab, z1b, zab, zz1b, zb
  REAL(KIND=JPRB) :: zphase(knbmu), zaux1(knbmu), zaux2(knbmu)
  REAL(KIND=JPRB) :: zphaseb(knbmu)
  LOGICAL, DIMENSION(knbmu) :: ll_mask
  LOGICAL, DIMENSION(knbmu) :: ll_mask0
  INTEGER(KIND=JPIM) :: i_ad_from
  INTRINSIC MAXLOC
  REAL(KIND=JPRB) :: z_tempb
  INTRINSIC DBLE
  INTRINSIC DLOG10
  INTRINSIC ACOS

  REAL(KIND=JPRB) :: ZHOOK_HANDLE  

#include "popinteger4.intfb.h"
#include "popreal8.intfb.h"
!#include "popbooleanarray.intfb.h"
#include "pushinteger4.intfb.h"
#include "pushreal8.intfb.h"
!#include "pushbooleanarray.intfb.h"

  IF (LHOOK) CALL DR_HOOK('TRUNCA_GEMS_AD',0,ZHOOK_HANDLE)

  zaux1(:) = 0.0_JPRB
  zaux2(:) = 0.0_JPRB
  pbetal(:, :) = 0.0_JPRB
  DO jlev=0,knt
    DO jang=1,knbmu
      zphase(jang) = phasef(jlev, jang)
    ENDDO
    CALL PUSHBOOLEANARRAY(ll_mask, 83)
! FINDING FIRST POINT FOR THE EXTRAPOLATION
    ll_mask = prmu  >   0.8_JPRB
    WHERE (ll_mask) zaux1 = 1.0_JPRB
    ind = MAXLOC(zaux1)
    CALL PUSHINTEGER4(jk)
    jk = ind(1) - 2
    CALL PUSHBOOLEANARRAY(ll_mask0, 83)
! FINDING SECOND POINT FOR THE EXTRAPOLATION
    ll_mask0 = prmu  >   0.94
    WHERE (ll_mask0) zaux2 = 1.0_JPRB
    ind = MAXLOC(zaux2)
    CALL PUSHINTEGER4(jkk)
    jkk = ind(1) - 2
! DEFINING THE SLOPE
    zaa = (LOG10(phasef(jlev, jkk))-LOG10(phasef(jlev, jk)))/(ACOS(&
&      prmu(jkk))-ACOS(prmu(jk)))
    z1 = LOG10(phasef(jlev, jkk))
    CALL PUSHREAL8(z2)
    z2 = ACOS(prmu(jkk))
    i_ad_from = jkk + 1
! RECALCULATING THE PHASE FUNCTION FOR ANGLES CLOSE TO 0 OR WITH A COSENUS BIGGER THAN 0.94  
    DO j=i_ad_from,knbmu
      CALL PUSHREAL8(za)
      za = z1 + zaa*(ACOS(prmu(j))-z2)
      zphase(j) = 10.0_JPRB**za
    ENDDO
    CALL PUSHINTEGER4(i_ad_from)
    DO j=1,knbmu
      CALL PUSHREAL8(phasef(jlev, j))
      phasef(jlev, j) = zphase(j)
    ENDDO
! INITIALIZING BETAL
    DO j=0,knang
      pbetal(jlev, j) = 0.0_JPRB
    ENDDO
! CALCULATING BETAL
    DO j=1,knbmu
      z = phasef(jlev, j)*pga(j)
      DO j1=0,knang
        pbetal(jlev, j1) = pbetal(jlev, j1) + z*ppl(j, j1)
      ENDDO
    ENDDO
    DO j=0,knang
      pbetal(jlev, j) = (2.0_JPRB*DBLE(j)+1.0_JPRB)*0.5_JPRB*pbetal(jlev, j)
    ENDDO
    CALL PUSHREAL8(zz1)
    zz1 = pbetal(jlev, 0)
  ENDDO
  zphaseb(1:knbmu) = 0._JPRB
  DO jlev=knt,0,-1
    zz1b = 0._JPRB
    DO j=knang,0,-1
      zz1b = zz1b - pbetal(jlev, j)*pbetalb(jlev, j)/zz1**2
      pbetalb(jlev, j) = pbetalb(jlev, j)/zz1
    ENDDO
    zz1b = zz1b - pcoeffb(jlev)
    pcoeffb(jlev) = 0._JPRB
    CALL POPREAL8(zz1)
    pbetalb(jlev, 0) = pbetalb(jlev, 0) + zz1b
    DO j=knang,0,-1
      pbetalb(jlev, j) = (DBLE(j)*2.0_JPRB+1.0_JPRB)*0.5_JPRB*pbetalb(jlev, j)
    ENDDO
    DO j=knbmu,1,-1
      zb = 0._JPRB
      DO j1=knang,0,-1
        zb = zb + ppl(j, j1)*pbetalb(jlev, j1)
      ENDDO
      phasefb(jlev, j) = phasefb(jlev, j) + pga(j)*zb
    ENDDO
    DO j=knang,0,-1
      pbetalb(jlev, j) = 0._JPRB
    ENDDO
    DO j=knbmu,1,-1
      CALL POPREAL8(phasef(jlev, j))
      zphaseb(j) = zphaseb(j) + phasefb(jlev, j)
      phasefb(jlev, j) = 0._JPRB
    ENDDO
    z1b = 0._JPRB
    zaab = 0._JPRB
    CALL POPINTEGER4(i_ad_from)
    DO j=knbmu,i_ad_from,-1
      zab = 10.0_JPRB**za*LOG(10.0_JPRB)*zphaseb(j)
      zphaseb(j) = 0._JPRB
      CALL POPREAL8(za)
      z1b = z1b + zab
      zaab = zaab + (ACOS(prmu(j))-z2)*zab
    ENDDO
    CALL POPREAL8(z2)
    z_tempb = zaab/(ACOS(prmu(jkk))-ACOS(prmu(jk)))
    phasefb(jlev, jkk) = phasefb(jlev, jkk) + z_tempb/(phasef(jlev, jkk)*&
&      LOG(10._JPRB)) + z1b/(phasef(jlev, jkk)*LOG(10._JPRB))
    phasefb(jlev, jk) = phasefb(jlev, jk) - z_tempb/(phasef(jlev, jk)*LOG&
&      (10._JPRB))
    CALL POPINTEGER4(jkk)
    CALL POPBOOLEANARRAY(ll_mask0, 83)
    CALL POPINTEGER4(jk)
    CALL POPBOOLEANARRAY(ll_mask, 83)
    DO jang=knbmu,1,-1
      phasefb(jlev, jang) = phasefb(jlev, jang) + zphaseb(jang)
      zphaseb(jang) = 0._JPRB
    ENDDO
  ENDDO

  IF (LHOOK) CALL DR_HOOK('TRUNCA_GEMS_AD',1,ZHOOK_HANDLE)
END SUBROUTINE TRUNCA_GEMS_AD
