SUBROUTINE Z0SEA(KPROMA,KIDIA,KFDIA,PAPHI,PUB,PVB,PSLM,PCHAR,PAZO)

!**** *Z0SEA* - COMPUTATION OF THE ROUGHNESS LENGTH OVER SEA

!     PURPOSE
!     -------
!          GIVEN WIND+HEIGHT AT LOWEST LEVEL AND CHARNOCK,
!          COMPUTES THE ROUGHNESS LENGTH OVER SEA
!          BASED ON MOLECULAR VISCOSITY PLUS CHARNOCK.

!          ASSUMES THAT THE INPUT WIND IS EQUIVALENT NEUTRAL.
!          HOWEVER USAGE OF REAL WIND ONLY INTRODUCES A SMALL SHIFT.


!**   INTERFACE
!     ---------
!     *CALL* Z0SEA(KPROMA,KIDIA,KFDIA,PAPHI5(1,NFLEVG),
!    I             PUF5(1,NFLEVG),PVF5(1,NFLEVG),PSLM5,PCHAR,
!    O             PZ05)

!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA             - HORIZONTAL DIMENSION.                    (INPUT-C)
!        KIDIA              - INDEX OF START POINT                     (INPUT-C)
!        KFDIA              - INDEX OF END   POINT                     (INPUT-C)
!        PAPHI(KPROMA)      - GEOPOTENTIAL MODEL LEVELS     M**2/S**2  (INPUT)
!        PUB(KPROMA)        - U-WIND COMPONENT                 M/S     (INPUT)
!        PVB(KPROMA)        - V-WIND COMPONENT                 M/S     (INPUT)
!        PSLM(KPROMA)       - SEA LAND MASK                            (INPUT)
!        PCHAR(KPROMA)      - CHARNOCK PARAMETER                       (INPUT)
!        PAZO(KPROMA)       - ROUGHNESS LENGTH                  M      (INPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!       SOLVES Z0=Z0(UN) FOR THE FOLLOWING IMPLICIT EQUATION
!       Z0 = MOLEC.VISC + CHARNOCK
!       MOLEC.VISC = RNUM/USTAR
!       CHARNOCK   = (ALPHA/RG)*USTAR**2
!       USTAR=(RKAP/BN)*UN
!       BN=LOG(1+ZN/Z0)

!       PROVIDES PARAMETRIZED SOLUTIONS OF DIMENSIONLESS EQUATION FOR BN
!       IN CASE EITHER THE MOLEC.VISC OR CHARNOCK TERM IS NEGLECTED.
!       THEN MERGES BOTH SOLUTIONS IN A PARAMETRIZED MANNER.

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        HANS HERSBACH *ECMWF*

!        ORIGINAL   : 08-07-02
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG
USE YOEVDF   , ONLY : RKAP


IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCHAR(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PAZO(KPROMA) 

INTEGER(KIND=JPIM) :: JL
REAL(KIND=JPRB) ,PARAMETER :: ZRN=0.11*1.5E-5
REAL(KIND=JPRB) :: ZUMIN,ZPOW,ZCR(2),ZCC(3)
REAL(KIND=JPRB) :: ZUK,ZNLEV,ZULOGR,ZULOGC,ZBNR,ZBNC,ZBN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     1.     DEFINITION OF THE CONSTANTS
!            ---------------------------

IF (LHOOK) CALL DR_HOOK('Z0SEA',0,ZHOOK_HANDLE)

ZUMIN=0.1_JPRB
ZPOW=-12.0_JPRB

ZCR = (/-1.47_JPRB, 0.93_JPRB            /)
ZCC = (/ 2.65_JPRB,-1.44_JPRB,-0.015_JPRB/)

!*         2.     COMPUTATION OF THE MODIFIED ROUGHNESS LENGTHS.
!                 ----------------------------------------------

DO JL=KIDIA,KFDIA
  IF(PSLM(JL) < 0.5_JPRB) THEN

    ZUK=RKAP*MAX(ZUMIN,SQRT(PUB(JL)**2+PVB(JL)**2))
    ZNLEV=PAPHI(JL)/RG

!   MOLECULAR VISCOSITY
    ZULOGR=LOG(ZNLEV*ZUK/ZRN)
    ZBNR=ZCR(1) + ZCR(2)*ZULOGR

!   CHARNOCK
    ZULOGC=LOG(ZUK*ZUK*PCHAR(JL)/PAPHI(JL))
    ZBNC=ZCC(1) + ZCC(2)*ZULOGC + ZCC(3)*ZULOGC*ZULOGC

!   COMBINE TOGETHER
    ZBN=(ZBNR**ZPOW +ZBNC**ZPOW)**(1.0_JPRB/ZPOW)
    PAZO(JL)=ZNLEV/(EXP(ZBN)-1.0_JPRB)


  ENDIF
ENDDO

!     --------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('Z0SEA',1,ZHOOK_HANDLE)
END SUBROUTINE Z0SEA
