SUBROUTINE BRPTOB(PPSP,PSPT,PSPTALLG)

!***** *BRPTOB*  ****  BRoadcast Pressure TO Bset of processors

!     Purpose.
!     --------
!               For the adjoint Horizontal Diffusion computations, the 
!               pressure at ground level is needed on all processors.
!               However, after the trgtol transpose only the  processors
!               with NPSP.eq.1 have this 2D field in their own memory.
!               Temperatures for all NFLEVG levels are also required on
!               all PE's. Both pressure and temperature fields are only
!               required for the local spectral coefficients (NSPEC2). 

!**   Interface.
!     ----------
!        *CALL* *BRPTOB(PPSP,PSPT,PSPTALLG)

!        Explicit arguments : PPSP - distributed surface pressure
!        -------------------- PSPT - distributed temperature
!                             PSPTALLG - temperatures for NFLEVG levels

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        MPL_SEND, MPL_RECV  - communication interface

!     Reference.
!     ----------

!     Author.
!     -------
!        MPP Group *METEO FRANCE*

!     Modifications.
!     --------------
!        Original   : 96-10-10 
!        L. Isaksen : 96-12-12  Temperature field added
!        M.Hamrud   : 00-02-02  Removal of MYSENDV etc.
!        D.Dent     : 00-09-01  MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM, JPRB
USE YOMHOOK   ,ONLY : LHOOK, DR_HOOK

USE YOMDIM   , ONLY : NFLEVL, NSPEC2
USE YOMCT0   , ONLY : NPRTRV
USE YOMMP    , ONLY : NPRCIDS, MYSETV, NPTRLL, NPSP, MYSETW
USE YOMTAG   , ONLY : MTAGBRPR
USE YOMLUN   , ONLY : NULOUT
USE MPL_MODULE

IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPSP(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPT(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPTALLG(:,:) 
! === END OF INTERFACE BLOCK ===
REAL(KIND=JPRB),    ALLOCATABLE :: ZCOMBUF(:,:)

INTEGER(KIND=JPIM) :: IA, IB, IBSET, IBUFLEN, IFRSTLEV, ILASTLEV,&
 & ILEVELS, ILRECV, IRECV, IRECVID, &
 & ISEND, IW, JROCB,ISENDSET,IRECVSET ,ISENDREQ(NPRTRV) 

!    INTEGER FUNCTIONS
INTEGER(KIND=JPIM) :: MYSENDSET,MYRECVSET
INTEGER(KIND=JPIM) :: JSP,JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "pe2set.intfb.h"
#include "set2pe.intfb.h"
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('BRPTOB',0,ZHOOK_HANDLE)


!   The PE that owns spsp sends it to other PE's in its B-set

CALL GSTATS_BARRIER(780)
CALL GSTATS(602,0)
IF(NPSP == 1) THEN

  DO JROCB=1,NPRTRV-1

!*   Send directly the SPSP array

    ISENDSET=MYSENDSET(NPRTRV,MYSETV,JROCB)
    CALL SET2PE(ISEND,0,0,MYSETW,ISENDSET)

    CALL MPL_SEND(PPSP(1:NSPEC2),KDEST=NPRCIDS(ISEND),KTAG=MTAGBRPR, &
     & KMP_TYPE=JP_NON_BLOCKING_STANDARD, KREQUEST=ISENDREQ(JROCB),&
     & CDSTRING='BRPTOB:')  

  ENDDO

ELSE

!   when NPSP .eq. 0 the processor does NOT have the pressure field

!   The pressure field is in the "highest" (having mysetv=NPRTRV)
!   processor in the "B-set"  (see sudim)

  IRECVSET=NPRTRV
  CALL SET2PE(IRECV,0,0,MYSETW,IRECVSET)
  ILRECV=0

  CALL MPL_RECV(PPSP(1:NSPEC2),KSOURCE=NPRCIDS(IRECV),KTAG=MTAGBRPR, &
   & KMP_TYPE=JP_BLOCKING_STANDARD,KOUNT=ILRECV,CDSTRING='BRPTOB:')  

  IF (ILRECV /= NSPEC2) THEN
    WRITE(NULOUT,*) MYSETV,'receiving',ILRECV,'expected',NSPEC2
    CALL ABOR1('BRPTOB: RECEIVED MESSAGE NOT CORRECT LENGTH')
  ENDIF

ENDIF
IF(NPSP == 1) THEN

  DO JROCB=1,NPRTRV-1

!*   Wait for send to complete

    ISENDSET=MYSENDSET(NPRTRV,MYSETV,JROCB)
    CALL SET2PE(ISEND,0,0,MYSETW,ISENDSET)

    CALL MPL_WAIT(PPSP(1:NSPEC2), KREQUEST=ISENDREQ(JROCB), &
     & CDSTRING='BRPTOB' )
  ENDDO
ENDIF
CALL GSTATS(602,1)

!     The temperature field dimensioned NFLEVG*NSPEC2 is build up
!     from the local temperature fields SPT(NFLSUR,NSPEC2). They
!     are distributed among processors in a B-set.

ILEVELS=0
DO JROCB=1,NPRTRV-1
  IRECVSET=MYRECVSET(NPRTRV,MYSETV,JROCB)
  CALL SET2PE(IRECV,0,0,MYSETW,IRECVSET)
  IRECVID=NPRCIDS(IRECV)
  CALL PE2SET(IRECVID,IA,IB,IW,IBSET)
  IFRSTLEV=NPTRLL(IBSET)
  ILASTLEV=NPTRLL(IBSET+1)-1
  ILEVELS=MAX(ILASTLEV-IFRSTLEV+1,ILEVELS)
ENDDO
IBUFLEN=ILEVELS*NSPEC2
ALLOCATE(ZCOMBUF(IBUFLEN,NPRTRV))
  
CALL GSTATS(602,0)
DO JROCB=1,NPRTRV-1
  
!*      Send SPT to other members of the B-set
  
  ISENDSET=MYSENDSET(NPRTRV,MYSETV,JROCB)
  CALL SET2PE(ISEND,0,0,MYSETW,ISENDSET)
  
  CALL MPL_SEND(PSPT(1:NFLEVL,1:NSPEC2),KDEST=NPRCIDS(ISEND),KTAG=MTAGBRPR, &
   & KMP_TYPE=JP_NON_BLOCKING_STANDARD,KREQUEST=ISENDREQ(JROCB), &
   & CDSTRING='BRPTOB:') 
ENDDO 
  
DO JROCB=1,NPRTRV-1
  IRECVSET=MYRECVSET(NPRTRV,MYSETV,JROCB)
  CALL SET2PE(IRECV,0,0,MYSETW,IRECVSET)
  ILRECV=0
  
  CALL MPL_RECV(ZCOMBUF(1:IBUFLEN,JROCB),KSOURCE=NPRCIDS(IRECV),KTAG=MTAGBRPR, &
   & KOUNT=ILRECV,CDSTRING='BRPTOB:')  
  IRECVID=NPRCIDS(IRECV)
  CALL PE2SET(IRECVID,IA,IB,IW,IBSET)
  IFRSTLEV=NPTRLL(IBSET)
  ILASTLEV=NPTRLL(IBSET+1)-1
  
  ILEVELS=ILASTLEV-IFRSTLEV+1
  IF (ILRECV /= ILEVELS*NSPEC2) THEN
    WRITE(NULOUT,*) MYSETV,'receiving',ILRECV,'expected',ILEVELS*NSPEC2
    CALL ABOR1('BRPTOB: RECEIVED MESSAGE NOT CORRECT LENGTH')
  ENDIF
ENDDO

DO JROCB=1,NPRTRV-1
  CALL MPL_WAIT(ZCOMBUF(1:IBUFLEN,JROCB),KREQUEST=ISENDREQ(JROCB), &
   & CDSTRING='BRPTOB: WAIT FOR SEND')  
ENDDO
CALL GSTATS(602,1)
CALL GSTATS_BARRIER2(780)
!       Determine level range based on id of receiver (IRECVID)
  
  
!       Store received contribution in the right part of PSPTALLG       
  
CALL GSTATS(1890,0)
DO JROCB=1,NPRTRV-1
  IRECVSET=MYRECVSET(NPRTRV,MYSETV,JROCB)
  CALL SET2PE(IRECV,0,0,MYSETW,IRECVSET)
  IRECVID=NPRCIDS(IRECV)
  CALL PE2SET(IRECVID,IA,IB,IW,IBSET)
  IFRSTLEV=NPTRLL(IBSET)
  ILASTLEV=NPTRLL(IBSET+1)-1
  ILEVELS=ILASTLEV-IFRSTLEV+1
  DO JSP=1,NSPEC2
    DO JL=1,ILEVELS
      PSPTALLG(IFRSTLEV+JL-1,JSP)=ZCOMBUF((JSP-1)*ILEVELS+JL,JROCB)
    ENDDO
  ENDDO
ENDDO
CALL GSTATS(1890,1)
  
!     Copy local contribution into PSPTALLG
  
IFRSTLEV=NPTRLL(MYSETV)
CALL GSTATS(1891,0)
DO JSP=1,NSPEC2
  DO JL=1,NFLEVL
    PSPTALLG(IFRSTLEV+JL-1,JSP)=PSPT(JL,JSP)
  ENDDO
ENDDO
CALL GSTATS(1891,1)

DEALLOCATE (ZCOMBUF)

IF (LHOOK) CALL DR_HOOK('BRPTOB',1,ZHOOK_HANDLE)
END SUBROUTINE BRPTOB
