SUBROUTINE DIWRFOU(KNMAX,KDGLL,KDGLG,KDGLMX,KNUMF,KIND,&
 & KMTAGDISTFO,KNUM,KIOPROC,KDIM,PFO)  

!**** *DIWRFOU*  - SEND OR RECEIVE FOURIER SPACE DATA (OUTPUT)

!     Purpose.
!     --------
!       Collects DM-local Fourier space data of truncation KNMAX
!        and DM-local latitudes 1 to KDGLL. The DM-global array
!        to be written has truncation KNMAX and latitudes 1 to KDGLG.
!        All wave numbers 0 to KNMAX are communicated even if zero
!        (reduced grid is currently not taken in account for simplification).
!       Only works for LSPLIT=.FALSE., LAPPLE=.TRUE. 
!       One call of MPL_SEND and MPL_RECV communicates the KNUMF fields.

!**   Interface.
!     ----------
!        *CALL* *DIWRFOU(.....)*

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KNMAX        - truncation. 
!          KDGLL        - DM-local number of latitudes which are treated
!                         in slow Fourier transforms by the current proc.
!          KDGLG        - DM-global number of latitudes.
!          KDGLMX       - is above or equal to the max of KDGLL on all procs. 
!          KNUMF        - number of fields.
!          KIND         - 0: send data; 1: receive data.
!          KMTAGDISTFO  - 'tag' number.
!          KNUM         - index of field.
!          KIOPROC      - processor which gathers data.
!          KDIM         - dimension of PFO: is > or = KDGLG*(2*KNMAX+1)*KNUMF

!        INPUT/OUTPUT:
!          PFO          - field to be communicated.
!                         as input: the DM-local field of the current proc.
!                         as output: the DM-global field.

!        Implicit arguments :
!        --------------------

!     Externals.
!     ----------
!       Calls MPL_SEND, MPL_RECV. 

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP), written after DIWRGRID. 

!     Modifications.
!     --------------
!        Original : FEB 1999
!        Modified by G> Radnoti 99/11/07  ALADIN geometry
!        Modified: 00-09-01 by D.Dent : MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : NPRINTLEV ,N_REGIONS ,N_REGIONS_NS
USE YOMMP    , ONLY : MYPROC   ,NFRSTLAT ,NLSTLAT  ,NPRCIDS ,NGPSET2PE
USE YOMDIM   , ONLY : NDGUNG   ,NDGUXG
USE MPL_MODULE

IMPLICIT NONE

! * DUMMY:
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNMAX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGLL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGLG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGLMX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMTAGDISTFO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUM 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIOPROC 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFO(KDIM) 
INTEGER(KIND=JPIM) :: IFOTOT
INTEGER(KIND=JPIM) :: IFOTOTG
INTEGER(KIND=JPIM) :: IFOTOTMX
INTEGER(KIND=JPIM) :: IGL1
INTEGER(KIND=JPIM) :: IGL2
INTEGER(KIND=JPIM) :: ILEN
INTEGER(KIND=JPIM) :: ILENB
INTEGER(KIND=JPIM) :: IOFF
INTEGER(KIND=JPIM) :: IRCV
INTEGER(KIND=JPIM) :: ISENDER
INTEGER(KIND=JPIM) :: ITAG
INTEGER(KIND=JPIM) :: ITAGR
INTEGER(KIND=JPIM) :: JA
INTEGER(KIND=JPIM) :: JB
REAL(KIND=JPRB) :: ZFLD(KDGLMX*(2*KNMAX+1)*KNUMF)
REAL(KIND=JPRB) :: ZFOTOT(KDGLG*(2*KNMAX+1)*KNUMF)
REAL(KIND=JPRB) :: ZHOOK_HANDLE


! ------------------------------------------------------------------

!*  0. PRELIMINARY INITIALISATIONS.
!   -------------------------------

IF (LHOOK) CALL DR_HOOK('DIWRFOU',0,ZHOOK_HANDLE)
IFOTOT=KDGLL*(2*KNMAX+1)
IFOTOTG=KDGLG*(2*KNMAX+1)
IFOTOTMX=KDGLMX*(2*KNMAX+1)

! ------------------------------------------------------------------

!*  1. SENDS DM-LOCAL FIELDS TO THE OUTPUT NODE 'KIOPROC'.
!   ------------------------------------------------------

IF (KIND == 0) THEN

  IF (IFOTOT > 0) THEN
    ILEN=IFOTOT*KNUMF
    ITAG=KMTAGDISTFO
    IF(KIOPROC /= MYPROC)THEN
      CALL MPL_SEND(PFO(1:ILEN),KDEST=NPRCIDS(KIOPROC),KTAG=ITAG, &
       & CDSTRING='DIWRFOU:')  
      IF (NPRINTLEV == 2) THEN
        WRITE(NULOUT,*) 'DIWRFOU : send Fourier space field', &
         & ' num ',KNUM,' from ',MYPROC,' to ',KIOPROC,&
         & ' with tag ',ITAG  
      ENDIF
    ENDIF
  ENDIF

ENDIF

! ------------------------------------------------------------------

!*  2. RECEIVES DM-LOCAL FIELDS FROM OTHER PROCESSORS.
!   --------------------------------------------------

IF (KIND == 1) THEN

  DO JA=1,N_REGIONS_NS
    DO JB=1,N_REGIONS(JA)

      IRCV=NGPSET2PE(JA,JB)
      ILENB=IFOTOTMX*KNUMF
      ITAG=KMTAGDISTFO
      IF (IRCV /= KIOPROC) THEN
        CALL MPL_RECV(ZFLD(1:ILENB),KSOURCE=NPRCIDS(IRCV),KTAG=ITAG, &
         & KFROM=ISENDER,KRECVTAG=ITAGR,CDSTRING='DIWRFOU:')  
        IF (NPRINTLEV == 2) THEN
          WRITE(NULOUT,*) 'DIWRFOU : recv Fourier space field', &
           & ' num ',KNUM,' from ',ISENDER,' to ',KIOPROC, &
           & ' with tag ',ITAGR  
        ENDIF
      ELSE
        ZFLD(1:IFOTOT*KNUMF)=PFO(1:IFOTOT*KNUMF)
      ENDIF

      IGL1=MAX(NFRSTLAT(JA),NDGUNG)
      IGL2=MIN(NLSTLAT(JA),NDGUXG)
      IF (JA > 1) THEN
        IOFF=(NLSTLAT(JA-1)+1-NFRSTLAT(1))*(1+2*KNMAX)*KNUMF
      ELSE
        IOFF=0
      ENDIF
      ILEN=(IGL2+1-IGL1)*(1+2*KNMAX)*KNUMF
      ZFOTOT(IOFF+1:IOFF+ILEN)=ZFLD(1:ILEN)

    ENDDO
  ENDDO

  PFO(1:IFOTOTG*KNUMF)=ZFOTOT(1:IFOTOTG*KNUMF)

ENDIF

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('DIWRFOU',1,ZHOOK_HANDLE)
END SUBROUTINE DIWRFOU
