!OPTION! -pvctl noouterunroll -pvctl noouterstrip
SUBROUTINE GATHERFREQ(PLFREQ,PFREQ,KVIMOD,KLCFRQ,KCHECK)

!**** *GATHERFREQ*  - Gather partitioned frequency arrays to global form

!     Purpose.
!     --------
!           Gather global versions of FREQ and arrays on each PE

!**   Interface.
!     ----------
!        *CALL* *GATHERFREQ(PLFREQ,PFREQ,KVIMOD,KLCFRQ,KCHECK)

!        Explicit arguments :
!        --------------------
!        PLFREQ          : Local frequency array  (input)
!        PFREQ           : Global frequency arrays (output)
!        KCHECK          : Global check sum

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        MPL_SEND, MPL_RECV  -  communication interface

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Lars Isaksen *ECMWF*

!     Modifications.
!     --------------
!        Original  : 97-12-12
!        M.Hamrud   : 00-02-02  Removal of MYSENDA etc.
!        D.Dent     : 00-09-01  MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!      E. Sevault 18-May-2009 : SX9 compiler workaround : optimisations
!            "outerunroll" and "outerstrip" manually switched off to avoid
!            erroneous numerical variations. Force two loops (probably short for
!            a vector machine) to be left unvectorised, as the compiler re-write
!            it in a wrong way.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NUMXP    ,NXMAX    ,NXPECG
USE YOMCT0   , ONLY : NPRTRW
USE YOMLAP   , ONLY : MYMS
USE YOMMP    , ONLY : NPRCIDS  ,NUMXPP   ,&
 & NPTRMS   ,NALLMS,MYSETV,MYSETW  
USE YOMTAG   , ONLY : MTAGFREQ
USE MPL_MODULE

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KVIMOD 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLCFRQ 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLFREQ(KVIMOD,KLCFRQ) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFREQ(NXPECG*3,KVIMOD) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCHECK 
REAL(KIND=JPRB),ALLOCATABLE :: ZRECVBUF(:,:)

INTEGER(KIND=JPIM) :: IA, IB, IBUFLEN, ICFREQ, ICFRQ, ICFRQTOT, ILCFRQ, IM, IORDR, IRCFRQ, IRECV, &
 & IRECVID, IRECVNUMXP, IRECVSETA, ISEND, ISENDLEN, &
 & ITAG, IV, J, JK, JMLOC, JROCA,ISENDSETA  

!    INTEGER FUNCTIONS
INTEGER(KIND=JPIM) :: MYSENDSET,MYRECVSET
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "pe2set.intfb.h"
#include "set2pe.intfb.h"

!     ------------------------------------------------------------------

!     1.    Store local frequencies in global arrays
!           ----------------------------------------

IF (LHOOK) CALL DR_HOOK('GATHERFREQ',0,ZHOOK_HANDLE)
ILCFRQ =0
DO JMLOC=1,NUMXP
  IM=MYMS(JMLOC)
  ICFRQ=(IM*(NXMAX+1)-IM*(IM-1)/2)*3
  ICFREQ=2*ICFRQ
  IORDR=3*(NXMAX+1-IM)
  DO JK=1,KVIMOD
!CDIR NODEP
    DO J=1,IORDR
      PFREQ (ICFRQ+J,JK)=PLFREQ(JK,ILCFRQ+J)
    ENDDO
  ENDDO

  ILCFRQ=ILCFRQ+IORDR
ENDDO
ICFRQTOT=ILCFRQ

!     ------------------------------------------------------------------

!     2.    Send local spectral arrays to all PE's in the A-set
!           ---------------------------------------------------

ALLOCATE (ZRECVBUF(KVIMOD,NXPECG*3))
IBUFLEN=SIZE(ZRECVBUF)
ITAG=MTAGFREQ
ISENDLEN=KVIMOD*ILCFRQ
DO JROCA=1,NPRTRW-1

  ISENDSETA=MYSENDSET(NPRTRW,MYSETW,JROCA)
  CALL SET2PE(ISEND,0,0,ISENDSETA,MYSETV)
  CALL MPL_SEND(PLFREQ(1:ISENDLEN,1),KDEST=NPRCIDS(ISEND),KTAG=ITAG, &
   & CDSTRING='GATHERFREQ:')  

  IRECVSETA=MYRECVSET(NPRTRW,MYSETW,JROCA)
  CALL SET2PE(IRECV,0,0,IRECVSETA,MYSETV)
  CALL MPL_RECV(ZRECVBUF(1:IBUFLEN,1),KSOURCE=NPRCIDS(IRECV),KTAG=ITAG, &
   & KFROM=IRECVID,CDSTRING='GATHERFREQ:')  

  CALL PE2SET(IRECVID,IA,IB,IRECVSETA,IV)
  IRECVNUMXP=NUMXPP(IRECVSETA)

  IRCFRQ =0
!CDIR NOVECTOR
  DO JMLOC=1,IRECVNUMXP
    IM=NALLMS(NPTRMS(IRECVSETA)-1+JMLOC)
    ICFRQ=(IM*(NXMAX+1)-IM*(IM-1)/2)*3
    ICFREQ=2*ICFRQ
    IORDR=3*(NXMAX+1-IM)
!CDIR NOVECTOR
    DO JK=1,KVIMOD
!CDIR NODEP
      DO J=1,IORDR
        PFREQ (ICFRQ+J         ,JK)=ZRECVBUF(JK,IRCFRQ+J)
      ENDDO
    ENDDO
    IRCFRQ=IRCFRQ+IORDR
  ENDDO
  ICFRQTOT=ICFRQTOT+IRCFRQ
ENDDO

KCHECK=ICFRQTOT*2

IF (ALLOCATED(ZRECVBUF)) DEALLOCATE(ZRECVBUF)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GATHERFREQ',1,ZHOOK_HANDLE)
END SUBROUTINE GATHERFREQ
