SUBROUTINE GATHERJCVERT(PLOCAL,PGLOBAL,KFIELDS,PSURF,KSURF)

!**** *GATHERJCVERT*  - Gather three arrays partitioned over levels into
!                       global arrays.

!     Purpose.
!     --------
!           Simple help routine needed to keep JC reproducible.

!**   Interface.
!     ----------
!        *CALL* *GATHERJCVERT(KFIELDS,PLOCAL,PGLOBAL,PSURF,KSURF)

!        Explicit arguments :
!        --------------------
!        PLOCAL(KFIELDS,NUMP,NFLEVL)  : Local part of cost function contribution
!        PGLOBAL(KFIELDS,NUMP,NFLEVG) : Global version of cost function contribution
!        KFIELDS                      : Number of 3D fields
!        PSURF(KSURF,NUMP)            : Surface field cost function contribution
!        KSURF                        : Number of surface fields

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        MPL_SEND, MPL_RECV  -  Communication interface

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Lars Isaksen *ECMWF*

!     Modifications.
!     --------------
!        Original   : 98-08-08
!        G.Mozdzynski : 99-09-28 Add communication for surface field(s) 
!        D.Dent     : 00-09-01  MPE to MPL conversion
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVL   ,NFLEVG   ,NUMP
USE YOMCT0   , ONLY : NPRTRV
USE YOMMP    , ONLY : NPRCIDS  ,NPTRLL   ,MYLEVS   ,&
 & NUMLL   ,NPSP   ,MYSETW, MYSETV  
USE YOMTAG   , ONLY : MTAGCOST
USE MPL_MODULE

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSURF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLOCAL(NFLEVL,NUMP,KFIELDS) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGLOBAL(NFLEVG,NUMP,KFIELDS) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSURF(KSURF,NUMP) 
REAL(KIND=JPRB) :: ZCOMRECV(NFLEVG*NUMP*KFIELDS)

INTEGER(KIND=JPIM) ::  IFRSTLEV, II, ILASTLEV, ILRECV, IRECV,&
 & IRECVLEN, IRECVLEV, IRECVSET, ISEND, &
 & ISENDLEN, ITAG, JLEV, JLEVRECV, JMLOC, &
 & JROCB, JTYPE,ISENDSET  
!     INTEGER FUNCTIONS
INTEGER(KIND=JPIM) :: MYSENDSET,MYRECVSET
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "set2pe.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('GATHERJCVERT',0,ZHOOK_HANDLE)
IRECVLEN=SIZE(ZCOMRECV)

!     1.    Plain copy of data if NPRTRV is 1.

IF (NPRTRV == 1) THEN
  PGLOBAL(:,:,:)=PLOCAL(:,:,:)
ELSE

!       2. If NPRTRV>1 communication within B-sets are required.

!       2.1 Store this PE's local part in global output array.

  DO JLEV=1,NFLEVL
    PGLOBAL(MYLEVS(JLEV),:,:)=PLOCAL(JLEV,:,:)
  ENDDO

  ISENDLEN=SIZE(PLOCAL)

!       2.2   Send local arrays to all other PE's in this PE's B-set

  ITAG=MTAGCOST+4

  DO JROCB=1,NPRTRV-1
    ISENDSET = MYSENDSET(NPRTRV,MYSETV,JROCB)
    CALL SET2PE(ISEND,0,0,MYSETW,ISENDSET)

    CALL MPL_SEND(PLOCAL(1:ISENDLEN,1,1),KDEST=NPRCIDS(ISEND),KTAG=ITAG, &
     & CDSTRING='GATHERJCVERT:')  
  ENDDO

!       2.3 Receive and store data in the proper place in global arrays.

  DO JROCB=1,NPRTRV-1
    IRECVSET = MYRECVSET(NPRTRV,MYSETV,JROCB)
    CALL SET2PE(IRECV,0,0,MYSETW,IRECVSET)

    CALL MPL_RECV(ZCOMRECV(1:IRECVLEN),KSOURCE=NPRCIDS(IRECV),KTAG=ITAG, &
     & KOUNT=ILRECV,CDSTRING='GATHERJCVERT:')  

    IRECVLEV=NUMLL(IRECVSET)
    IF (ILRECV /= IRECVLEV*NUMP*KFIELDS)&
     & CALL ABOR1(' GATHERJCVERT : RECEIVED MESSAGE WRONG LENGTH')  

    IFRSTLEV=NPTRLL(IRECVSET)
    ILASTLEV=NPTRLL(IRECVSET+1)-1

!         The received data is packed in the order: level waves type

    II=0
    DO JTYPE=1,KFIELDS
      DO JMLOC=1,NUMP
        DO JLEVRECV=IFRSTLEV,ILASTLEV
          II=II+1
          PGLOBAL(JLEVRECV,JMLOC,JTYPE)=ZCOMRECV(II)
        ENDDO
      ENDDO
    ENDDO
  ENDDO

!       2.4 Send/Receive surface field contributions

  ISENDLEN=SIZE(PSURF)
  IRECVLEN=SIZE(PSURF)

  IF (NPSP == 1) THEN

    DO JROCB=1,NPRTRV-1
      ISENDSET = MYSENDSET(NPRTRV,MYSETV,JROCB)
      CALL SET2PE(ISEND,0,0,MYSETW,ISENDSET)

      CALL MPL_SEND(PSURF(1:ISENDLEN,1),KDEST=NPRCIDS(ISEND),KTAG=ITAG, &
       & CDSTRING='GATHERJCVERT:')  
    ENDDO

  ELSE

    CALL SET2PE(IRECV,0,0,MYSETW,NPRTRV)
    CALL MPL_RECV(PSURF(1:IRECVLEN,1),KSOURCE=NPRCIDS(IRECV),KTAG=ITAG, &
     & KOUNT=ILRECV,CDSTRING='GATHERJCVERT:')  
    IF (ILRECV /= IRECVLEN)&
     & CALL ABOR1(' GATHERJCVERT : RECEIVED MESSAGE WRONG LENGTH SURF')  

  ENDIF

ENDIF

IF (LHOOK) CALL DR_HOOK('GATHERJCVERT',1,ZHOOK_HANDLE)
END SUBROUTINE GATHERJCVERT
