SUBROUTINE PE2SET(KPE,KPRGPNS,KPRGPEW,KPRTRW,KPRTRV)

!**** *PE2SET* - Convert from PE number to set numbers

!     Purpose.
!     --------
!        Convert from PE number to set numbers in both
!                  grid-point space and spectral space

!**   Interface.
!     ----------
!        *CALL* *PE2SET(KPE,KPRGPNS,KPRGPEW,KPRTRW,KPRTRV)

!        Explicit arguments :  
!        --------------------
!                  input:   KPE     - integer processor number 
!                                     in the range 1 .. NPROC
!                  output:  KPRGPNS - integer A set number in grid space
!                                     in the range 1 .. NPRGPNS
!                           KPRGPEW - integer B set number in grid space
!                                     in the range 1 .. NPRGPEW
!                           KPRTRW  - integer A set number in spectral space
!                                     in the range 1 .. NPRTRW 
!                           KPRTRV  - integer B set number in spectral space
!                                     in the range 1 .. NPRTRV 

!        Implicit arguments :  YOMMP parameters
!                              NPRGPNS,NPRGPEW,NPRTRW,NPRTRV,NPROC

!        --------------------
!     Method.
!     -------

!        PE allocation order is row oriented (e.g. NPRGPNS or NPRTRW = 4):

!                1  2  3  4 
!                5  6  7  8 
!                9 10 11 12 
!               13 14 15 16 
!                .  .  .  .

!     Externals.
!     ----------
!         NONE

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        David Dent *ECMWF*

!     Modifications.
!     --------------
!        Original : 98-08-19
!        Revision : 98-10-13 row ordering
!        Y.Tremolet: 02-03-13 use groups
!        R. El Khatib : 03-01-23 Case LMPOFF
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY :   NPRGPEW  ,NPRTRV   ,NPROC    ,LMPOFF ,&
               & N_REGIONS_NS, N_REGIONS
USE YOMMP    , ONLY : LEQ_REGIONS
USE YOMLUN   , ONLY : NULERR
USE MPL_MODULE

IMPLICIT NONE
INTEGER(KIND=JPIM),INTENT(IN)    :: KPE 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KPRGPNS 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KPRGPEW 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KPRTRW 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KPRTRV 
INTEGER(KIND=JPIM) :: IPRTRW,IPRTRV,IPE,JA
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

!*       1.    Check input argument for validity 
!              ---------------------------------

IF (LHOOK) CALL DR_HOOK('PE2SET',0,ZHOOK_HANDLE)

IF(KPE <= 0.OR.KPE > NPROC) THEN

  WRITE(*,'(A,2I8)') ' PE2SET INVALID ARGUMENT ',KPE,NPROC
  CALL ABOR1(' PE2SET INVALID ARGUMENT ')

ELSE

!*       2.    Compute output parameters
!              -------------------------

  IF( LEQ_REGIONS )THEN
    KPRGPNS=1
    IPE=KPE
    DO JA=1,N_REGIONS_NS
      IF( IPE > N_REGIONS(JA) )THEN
        IPE=IPE-N_REGIONS(JA)
        KPRGPNS=KPRGPNS+1
        CYCLE
      ENDIF
      KPRGPEW=IPE
      EXIT
    ENDDO
  ELSE
    KPRGPEW=MOD(KPE-1,NPRGPEW)+1
    KPRGPNS=(KPE-1)/NPRGPEW+1
  ENDIF

  SELECT CASE (LMPOFF)
  CASE (.TRUE.)
    KPRTRW=(KPE-1)/NPRTRV+1
    KPRTRV=MOD(KPE-1,NPRTRV)+1
  CASE (.FALSE.)
    CALL MPL_CART_COORDS(KPE,KPRTRW,KPRTRV)

!     Just checking for now...
    IPRTRV =MOD(KPE-1,NPRTRV)+1
    IPRTRW =(KPE-1)/NPRTRV+1
    IF (IPRTRV/=KPRTRV .OR. IPRTRW/=KPRTRW) THEN
      write(nulerr,*)'PE2SET IPRTRV,KPRTRV,IPRTRW,KPRTRW=', &
       & IPRTRV,KPRTRV,IPRTRW,KPRTRW  
      CALL ABOR1('PE2SET wrong group values')
    ENDIF
  END SELECT

ENDIF

IF (LHOOK) CALL DR_HOOK('PE2SET',1,ZHOOK_HANDLE)

END SUBROUTINE PE2SET
