SUBROUTINE RDPXFA(PFIELDS,CDPREF,KLEV,CDSUFF,KFIELDS,KFILE)

!**** *RDPXFA*  - Read pre-processed fields from intermediate file PX... (Fullpos)
!                     Part of DM counterpart of SPECFITASM.

!     Purpose.
!     --------
!        To Read and distribute the horizontally pre-processed gridpoint fields 
!        that should be modified : after fit of orography or before being read by 
!        SUSPECADM (fit).
!        File will be closed at end of WRFPPX.

!**   Interface.
!     ----------
!        *CALL* *RDPXFA*

!        Explicit arguments :
!        --------------------

!            PFIELDS : gridpoint fields buffer
!            CDPREF  : prefix of field names to read
!            KLEV    : level  of field names to read
!            CDSUFF  : suffix of field names to read
!            KFIELDS : number of fields in buffer
!            KFILE   : identificator of the file to use

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation
!        Remark : while reading, the field may contain the poles ; 
!        the usual length of a pole row is NLOENG(1), but NDLON is a 
!        safer majorant. 

!     Externals.
!     ----------
!           OPENFA
!        Processor communication routines
!           DISGRID
!        DM environment setup routine

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!         RYAD EL KHATIB *METEO-FRANCE*

!     Modifications.
!     --------------
!        ORIGINAL : 97-12-29
!        R. El Khatib : 98-02-25 (simplification since LRPOLE=.F. in DM)
!        R. El Khatib : 98-12-16 (break away from SUARPIO)
!        R. El Khatib : 99-12-28 proper overdimensionning for ZREALG
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     Apr 2008  K. Yessad: use DISGRID instead of DISGRID_C + cleanings
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NULOUT
USE YOMGEM   , ONLY : NGPTOT   ,NGPTOTG
USE YOMMP    , ONLY : MYPROC   ,NSTRIN
USE YOMDIM   , ONLY : NDLON
USE YOMCT1   , ONLY : LRFILAF
USE YOMCT0   , ONLY : LOUTPUT

!     ------------------------------------------------------------------

IMPLICIT NONE
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDS 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFIELDS(NGPTOT,KFIELDS) 
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDPREF(KFIELDS) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV(KFIELDS) 
CHARACTER(LEN=*)  ,INTENT(IN)    :: CDSUFF(KFIELDS) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFILE 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IDATEF(11)
REAL(KIND=JPRB) :: ZREALG(NGPTOTG+2*NDLON)
INTEGER(KIND=JPIM) :: IASPIO(KFIELDS)

INTEGER(KIND=JPIM) :: IDEB, IFIELDS, IFIN, IINDEX, IREP, IUNTIN, JF, JNUM
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "disgrid.intfb.h"
#include "openfa.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('RDPXFA',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*    1.    PREPARATIONS
!           ------------

DO JF=1,KFIELDS
  IASPIO(JF)=MOD(JF-1,NSTRIN)+1
ENDDO
IFIELDS=0
DO JF=1,KFIELDS
  IF (IASPIO(JF) == MYPROC) THEN
    IFIELDS=IFIELDS+1
  ENDIF
ENDDO
IF (LOUTPUT) THEN
  WRITE(NULOUT,*) 'RDPXFA : IFIELDS = ',IFIELDS, ' IASPIO = '
  WRITE(NULOUT,'(15(1X,I4))') IASPIO
ENDIF

!*    2.    OPEN AND CHECK FILE.
!           --------------------

IF (MYPROC <= NSTRIN) THEN
  CALL OPENFA(KFILE,IUNTIN,IDEB,IFIN,IINDEX,IDATEF)
ENDIF

!     3.    READ & SEND FIELDS
!           ------------------

DO JNUM=1,KFIELDS
  IF (IASPIO(JNUM) == MYPROC) THEN
    CALL FACILE(IREP,IUNTIN,CDPREF(JNUM),KLEV(JNUM),CDSUFF(JNUM),&
     & ZREALG,.FALSE.)  
    CALL DISGRID(JNUM,0,NGPTOTG,ZREALG(IINDEX+1),LDC=.TRUE.,KIOPROC=IASPIO(JNUM))
    PFIELDS(1:NGPTOT,JNUM)=ZREALG(IINDEX+1:IINDEX+NGPTOT)
  ENDIF
ENDDO

!     4.    RECIEVE FIELDS
!           --------------

DO JNUM=1,KFIELDS
  IF (IASPIO(JNUM) /= MYPROC) THEN
    CALL DISGRID(JNUM,1,NGPTOT,PFIELDS(1,JNUM),LDC=.TRUE.,KIOPROC=IASPIO(JNUM))
  ENDIF
ENDDO

!*    5.    CLOSE FILE
!           ----------

IF (MYPROC <= NSTRIN) THEN
  IF (LRFILAF) THEN
    CALL LFILAF(IREP,IUNTIN,.FALSE.)
  ENDIF
  CALL FAIRME(IREP,IUNTIN,'UNKNOWN')
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RDPXFA',1,ZHOOK_HANDLE)
END SUBROUTINE RDPXFA
