SUBROUTINE SLEXTPOLAD(KSLWIDE,&
 & KASLB1,KDGSAG,KDGENG,KDGSAL,KDGENL,KFLDSLB,KSLSTA,KSLONL,KSLOFF,&
 & KDGLG,KLOENG,KFRSTLOFF,KYFRSTACTLAT,KYLSTACTLAT,&
 & PARSLB,PB1A,KMASK1)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMASK  , ONLY : LSLONDEM

!****-------------------------------------------------------------------
!**** *SLEXTPOLAD* - Communication for horizontal interpolations: 
!                  Adjoint of SLEXTPOL.
!                  Special Pole treatment.

!                  This routine is called only if distributed memory.
!                  Computations are DM-local.
!                  This routine assumes that there are exactly three 
!                  extra-longitudes: jlon=0, jlon=ndlon+1, jlon=ndlon+2.
!****-------------------------------------------------------------------
!     Purpose.
!     --------
!**   Interface.
!     ----------
!        *CALL* *SLEXTPOLAD(...)*

!        Explicit arguments :
!        --------------------
!        - KSLWIDE    : number of rows the model lagged part runs behind.
!        - KASLB1     : local inner dimension of interpolation buffer.
!        - KDGSAG     : Global version of KDGSA
!        - KDGENG     : Global version of KDGEN
!        - KDGSAL     : Local version of KDGSA , always 1
!        - KDGENL     : No. of latitude rows for which this process has grid points
!        - KFLDSLB    : number of fields.
!        - KSLSTA     : interpolation buffer start position of grid columns
!                       on latitudes.
!        - KSLONL     : interpolation buffer number of grid column on latitudes.
!        - KSLOFF     : interpolation buffer offset to start of each row in
!                       the interpolation buffer.
!        - KDGLG      : number of latitude rows
!        - KLOENG     : Number of grid points per latitude
!        - KFRSTLOFF  : offset for first lat of own a-set in grid-point space,
!                        i.e. kfrstloff=kfrstlat(my_region_ns)-1
!        - KYFRSTACTLAT: first actual lat on this PE in grid-point space,
!                        it is kfrstlat(my_region_ns)
!        - KYLSTACTLAT : last actual lat on this PE in grid-point space,
!                        it is klstlat(my_region_ns)
!        - PARSLB     : parity for extra-polar calculations.
!        - PB1A       : fields to be computed at the extra-polar latitudes.
!        - KMASK1     : Mask1 for on demand-comms (updated)

!        Implicit arguments :
!        --------------------

!     Externals.
!     ----------
!      None.
!      Is called by SCAN2MAD and CALL_SL_AD.

!     Method.
!     -------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Documentation about distributed memory.

!     Author.
!     -------
!       Lars ISAKSEN (ECMWF), OCTOBER 1995.

!     Modifications.
!     --------------
!       K. YESSAD (JAN 1998): 
!        - adapt to new content of SLEXTPOL. 
!       Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!       99-06-21  Remove !OCL SCALAR  D.SALMOND
!       02-10-01  G.Mozdzynski     support for radiation on-demand comms
!       M.Hamrud      01-Oct-2003 CY28 Cleaning
!       K.Yessad (Dec 2008) Cleanings
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSLWIDE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KASLB1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGSAG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGENG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGSAL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGENL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDSLB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSLSTA(KDGSAL-KSLWIDE:KDGENL+KSLWIDE) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSLONL(KDGSAL-KSLWIDE:KDGENL+KSLWIDE) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSLOFF(KDGSAL-KSLWIDE:KDGENL+KSLWIDE) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGLG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOENG(KDGSAG:KDGENG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFRSTLOFF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KYFRSTACTLAT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KYLSTACTLAT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PARSLB(KFLDSLB) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PB1A(KASLB1,KFLDSLB,1) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KMASK1(KASLB1+4) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IEND, IGLGLO, IGLLOC, IGLSGLO, IGLSLOC, ILFB,&
 & ILPT, ILSB, IPT, IPTF, IST, JFLD, JGLLOC, JLP  

LOGICAL :: LLNOR, LLSUD
INTEGER(KIND=JPIM) :: ILEN, JOFF
INTEGER(KIND=JPIM),ALLOCATABLE :: IPTA(:), IPTFA(:), I_LPTA(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SLEXTPOLAD',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

ILEN=1

!     ** Search through all latitudes within halo region.

ILSB=MAX(KDGSAG,KYFRSTACTLAT-KSLWIDE)-KFRSTLOFF
ILFB=MIN(KDGENG,KYLSTACTLAT +KSLWIDE)-KFRSTLOFF

!     ** If complete latitude present we correct for additional points
!        required at start/finish for interpolation star.

CALL GSTATS(1815,0)
!$OMP  PARALLEL DO SCHEDULE(STATIC)&
!$OMP& PRIVATE(JGLLOC,IST,IEND,JFLD)
DO JFLD=1,KFLDSLB
  DO JGLLOC=ILSB,ILFB
    IF (KLOENG(JGLLOC+KFRSTLOFF) == KSLONL(JGLLOC)) THEN
      IST =KSLOFF(JGLLOC)-1
      IEND=KSLOFF(JGLLOC)+KLOENG(JGLLOC+KFRSTLOFF)
      PB1A(IST+3,JFLD,1)=PB1A(IST+3,JFLD,1)+PB1A(IEND+2,JFLD,1)
      PB1A(IST+2,JFLD,1)=PB1A(IST+2,JFLD,1)+PB1A(IEND+1,JFLD,1)
      PB1A(IEND ,JFLD,1)=PB1A(IEND ,JFLD,1)+PB1A(IST +1,JFLD,1)
    ENDIF
  ENDDO
ENDDO
!$OMP END PARALLEL DO

DO JGLLOC=ILSB,ILFB
  IF (KLOENG(JGLLOC+KFRSTLOFF) == KSLONL(JGLLOC)) THEN
    IST =KSLOFF(JGLLOC)-1
    IEND=KSLOFF(JGLLOC)+KLOENG(JGLLOC+KFRSTLOFF)
    IF(LSLONDEM)THEN
      KMASK1(IST+3)=1
      KMASK1(IST+2)=1
      KMASK1(IEND)=1
    ENDIF
  ENDIF
  IGLGLO=JGLLOC+KFRSTLOFF
  LLNOR=(IGLGLO >= KDGSAG).AND.(IGLGLO <= 0)
  LLSUD=(IGLGLO >= KDGLG+1).AND.(IGLGLO <= KDGENG)
  IF (LLNOR.OR.LLSUD) THEN
    IGLLOC=IGLGLO-KFRSTLOFF
    ILEN=MAX(ILEN,KSLONL(IGLLOC))
  ENDIF
ENDDO

CALL GSTATS(1815,1)

ALLOCATE(IPTA(ILEN))
ALLOCATE(I_LPTA(ILEN))
ALLOCATE(IPTFA(ILEN))

!     ** Fix extra data required that has not been communicated:
!        1) pole mimics from core data.
!        2) end of latitude star copies (extmerb equivalent operations).

CALL GSTATS(1132,0)

DO JGLLOC=ILSB,ILFB

  IGLGLO=JGLLOC+KFRSTLOFF
  LLNOR=(IGLGLO >= KDGSAG).AND.(IGLGLO <= 0)
  LLSUD=(IGLGLO >= KDGLG+1).AND.(IGLGLO <= KDGENG)

  IF (LLNOR.OR.LLSUD) THEN

    IF (LLNOR) IGLSGLO=1-IGLGLO
    IF (LLSUD) IGLSGLO=2*KDGLG+1-IGLGLO

!         * Local view also required.

    IGLLOC=IGLGLO-KFRSTLOFF
    IGLSLOC=IGLSGLO-KFRSTLOFF

!         * Determine which mimic points come from this processors core region.
!           The mirrored latitudes close to the poles require correction for
!           symmetric and antisymmetric properties (PARSLB).

    DO JLP=KSLSTA(IGLLOC),KSLONL(IGLLOC)+KSLSTA(IGLLOC)-1

      ILPT=MOD(JLP-1+KLOENG(IGLGLO),KLOENG(IGLGLO))+1
      IPT =KSLOFF(IGLLOC)+&
       & MOD(ILPT-KSLSTA(IGLLOC)+KLOENG(IGLGLO),KLOENG(IGLGLO))  
      IPTF=KSLOFF(IGLSLOC)+&
       & MOD(ILPT-KSLSTA(IGLSLOC)+KLOENG(IGLSGLO),KLOENG(IGLSGLO))  
      JOFF=JLP-KSLSTA(IGLLOC)+1
      IPTA (JOFF)=IPT
      IPTFA(JOFF)=IPTF
      I_LPTA(JOFF)=0
      IF(LSLONDEM) KMASK1(IPT+1)=1

!           update only if the polar mimic points are required on this processor

      IF ( (KSLOFF(IGLSLOC) <= IPTF).AND.&
         & (IPTF <= KSLOFF(IGLSLOC)+KSLONL(IGLSLOC)-1) ) THEN  
        I_LPTA(JOFF)=1
        IF(LSLONDEM)THEN
          KMASK1(IPTF+1)=1
          KMASK1(IPT+1)=1
        ENDIF
      ENDIF

    ENDDO

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JFLD,JLP,IPT,IPTF)
    DO JFLD=1,KFLDSLB
      DO JLP=1,KSLONL(IGLLOC)
        IPT=IPTA(JLP)
        PB1A(IPT+1,JFLD,1)=PARSLB(JFLD)*PB1A(IPT+1,JFLD,1)
      ENDDO
      DO JLP=1,KSLONL(IGLLOC)
        IPT =IPTA(JLP)
        IPTF=IPTFA(JLP)
        IF(I_LPTA(JLP) == 1) THEN
          PB1A(IPTF+1,JFLD,1)=PB1A(IPTF+1,JFLD,1)+PB1A(IPT+1,JFLD,1)
!           zero mimic points to avoid adding contribution twice in slcomm
          PB1A(IPT+1,JFLD,1)=0.0_JPRB
        ENDIF
      ENDDO
    ENDDO
!$OMP END PARALLEL DO

  ENDIF

ENDDO

CALL GSTATS(1132,1)
DEALLOCATE(IPTA)
DEALLOCATE(I_LPTA)
DEALLOCATE(IPTFA)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SLEXTPOLAD',1,ZHOOK_HANDLE)
END SUBROUTINE SLEXTPOLAD
