!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACCONVTL ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPHIF5,PAPRS5,PAPRSF5,PCVGQ5,&
 & PDELP5,PQ5,PRDELP5,PT5,PU5,PV5,&
 & PAPHIF,PAPRS,PAPRSF,PCVGQ,&
 & PDELP,PQ,PRDELP,PT,PU,PV,&
 ! - INPUT  1D .
 & PTS,&
 ! - OUTPUT 2D .
 & PDIFCQ5,PDIFCS5,PFPLCL5,PFPLCN5,PSTRCU5,&
 & PSTRCV5,&
 & PDIFCQ,PDIFCS,PFPLCL,PFPLCN,PSTRCU,PSTRCV,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *ACCONVTL * - TANGENT LINEAR COMPUTATION OF MASS-FLUX DEEP
!                   CONVECTION SCHEME.

!     Subject.
!     -------
!     Tangent linear computation of:
!     - MASS-FLUX DEEP CONVECTION SCHEME, COMPUTING CONVECTIVE
!       PRECIPITATIONS AND "SUB-GRID SCALE" ASSOCIATED FLUXES .
!     - COMPUTATION OF ENTHALPY FLUXES LINKED TO CONVECT. PRECIPITATIONS
!              (WATER AND SNOW) .

!**   Interface.
!     ----------
!        *CALL* *ACCONVTL*

! -   ARGUMENTS FOR INPUT.
!     --------------------

! - DIMENSIONAL PARAMETERS OF PHYSICS.

! KIDIA      : START OF HORIZONTAL LOOP (IST in CPG).
! KFDIA      : END OF HORIZONTAL LOOP (IEND in CPG).
! KLON       : HORIZONTAL DIMENSIONAL.
! KTDIA      : START OF THE VERTICAL LOOP IN THE PHYSIC (1 IN GENERAL).
! KLEV       : END OF VERTICAL LOOP AND VERTICAL DIMENSION "FULL LEVEL".
!              (NFLEVG in CPG).

! - THE NAME OF THE PHYSICAL VARIABLES

! - 2D (0:KLEV) .

! PAPRS      : PRESSURE ON HALF LEVELS.

! The same but for trajectory
! ---------------------------
! PAPRS5     : PRESSURE ON HALF LEVELS.

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIAL ON FULL LEVELS.
! PAPRSF     : PRESSURE ON FULL LEVELS.
! PCVGQ      : CONVERGENCE OF THE HUMIDITY ("KUO" CONDITION).
! PDELP      : LAYER THICKNESS IN PRESSURE UNITS.
! PQ         : SPECIFIC HUMIDITY OF WATER VAPOUR.
! PRDELP     : INVERSE OF LAYER THICKNESS IN PRESSURE UNITS.
! PT         : TEMPERATURE.
! PU         : X COMPONENT OF WIND.
! PV         : Y COMPONENT OF WIND.

! The same but for trajectory
! ---------------------------
! PAPHIF5    : GEOPOTENTIAL ON FULL LEVELS.
! PAPRSF5    : PRESSURE ON FULL LEVELS.
! PCVGQ5     : CONVERGENCE OF THE HUMIDITY ("KUO" CONDITION).
! PDELP5     : LAYER THICKNESS IN PRESSURE UNITS.
! PQ5        : SPECIFIC HUMIDITY OF WATER VAPOUR.
! PRDELP5    : INVERSE OF LAYER THICKNESS IN PRESSURE UNITS.
! PT5        : TEMPERATURE.
! PU5        : X COMPONENT OF WIND.
! PV5        : Y COMPONENT OF WIND.

! - 1D

! PTS        : SURFACE TEMPERATURE.

!-----------------------------------------------------------------------

! -   ARGUMENTS FOR OUTPUT.
!     ---------------------

! - 2D (0:KLEV) .

! PDIFCQ     : CONVECTIVE FLUX OF SPECIFIC HUMIDITY (NOT RAIN/SNOW).
! PDIFCS     : CONVECTIVE FLUX OF ENTHALPY (NOT RAIN/SNOW).
! PFPLCL     : CONVECTIVE PRECIPITATION AS RAIN.
! PFPLCN     : CONVECTIVE PRECIPITATION AS SNOW.
! PSTRCU     : CONVECTIVE FLUX OF MOMENTUM "U".
! PSTRCV     : CONVECTIVE FLUX OF MOMENTUM "V".

! The same but for trajectory
! ---------------------------
! PDIFCQ5    : CONVECTIVE FLUX OF SPECIFIC HUMIDITY (NOT RAIN/SNOW).
! PDIFCS5    : CONVECTIVE FLUX OF ENTHALPY (NOT RAIN/SNOW).
! PFPLCL5    : CONVECTIVE PRECIPITATION AS RAIN.
! PFPLCN5    : CONVECTIVE PRECIPITATION AS SNOW.
! PSTRCU5    : CONVECTIVE FLUX OF MOMENTUM "U".
! PSTRCV5    : CONVECTIVE FLUX OF MOMENTUM "V".

!-----------------------------------------------------------------------

! -   IMPLICIT ARGUMENTS.
!     ---------------------

! COMMON/YOMPHY /
! COMMON/YOMCST /
! COMMON/YOMPHY0/
! COMMON/YOMPHY2/
! COMMON/FCTTRM /
! COMMON/FCTTRMTL/

!-----------------------------------------------------------------------

!     Externals.
!     ----------

!     Method.
!     -------

!     Author
!     -------
!        97-02-15   -   M. Janiskova
!     Modifications.
!     --------------
!        97-01, modifications in stability criterium
!               and Newton loop - M. Janiskova.
!        97-08, introduce CDLOCK - J.M. Piriou.
!        99-12, modification in the condition related to negative
!               precipitations at the surface -  F. Bouyssel.
!        99-12, perturbations of mass, entrainment and detrainment fluxes
!               are set to zero (if LSMOOTHB=.T.) - F. Bouyssel.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : NBITER   ,LNEIGE
USE YOMCST   , ONLY : RG       ,RD       ,RV       ,RCPD     ,&
 & RCPV     ,RETV     ,RCW      ,RCS      ,RLVTT    ,&
 & RLSTT    ,RTT      ,RALPW    ,RBETW    ,RGAMW    ,&
 & RALPS    ,RBETS    ,RGAMS    ,RALPD    ,RBETD    ,&
 & RGAMD  
USE YOMPHY0  , ONLY : ECMNP    ,SCO      ,USDMLT
USE YOMPHY2  , ONLY : TSPHY
USE YOMSIMPHL, ONLY : LSMOOTHB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVGQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVGQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFCQ5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFCS5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLCL5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLCN5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRCU5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRCV5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFCQ(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDIFCS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLCL(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLCN(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRCU(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSTRCV(KLON,0:KLEV) 
CHARACTER(LEN=7)  ,INTENT(IN)    :: CDLOCK 
INTEGER(KIND=JPIM) :: I_KNLAB(KLON,KLEV),I_KNND(KLON)

REAL(KIND=JPRB) :: ZSNP(KLON,0:KLEV),ZLHN(KLON,0:KLEV),ZLHL(KLON,0:KLEV)&
 & ,ZFORM(KLON,0:KLEV),ZZLHP(KLON,0:KLEV)  
REAL(KIND=JPRB) :: ZCP(KLON,KLEV),ZLHE(KLON,KLEV),ZDSE(KLON,KLEV)&
 & ,ZQW(KLON,KLEV),ZTW(KLON,KLEV),ZMSE(KLON,KLEV)&
 & ,ZMSEU(KLON,KLEV),ZF(KLON,KLEV),ZMFC(KLON,KLEV)&
 & ,ZCORU(KLON,KLEV),ZCORV(KLON,KLEV),ZCORQ(KLON,KLEV)&
 & ,ZCORS(KLON,KLEV),ZFQSC(KLON,KLEV)&
 & ,ZFUSC(KLON,KLEV),ZFVSC(KLON,KLEV),ZFSSC(KLON,KLEV)&
 & ,ZQDN(KLON,KLEV),ZPOID(KLON,KLEV),ZIPOI(KLON,KLEV)&
 & ,ZMSEC(KLON,KLEV),ZPHI(KLON,KLEV)  
REAL(KIND=JPRB) :: ZLHSB(KLON),ZTN(KLON),ZQN(KLON),ZS1(KLON),ZS2(KLON)&
 & ,ZPOII(KLON),ZALF(KLON),ZLN(KLON),ZUN(KLON),ZVN(KLON)&
 & ,ZMSEN(KLON),ZDFMDQ(KLON),ZDFMDU(KLON),ZDFMDV(KLON)&
 & ,ZDFMDS(KLON),ZDSEN(KLON),ZENTR(KLON),ZDETR(KLON)&
 & ,ZPOIL(KLON)  

REAL(KIND=JPRB) :: ZSNP5(KLON,0:KLEV),ZLHN5(KLON,0:KLEV),ZLHL5(KLON,0:KLEV)&
 & ,ZFORM5(KLON,0:KLEV),ZZLHP5(KLON,0:KLEV)  
REAL(KIND=JPRB) :: ZCP5(KLON,KLEV),ZLHE5(KLON,KLEV),ZDSE5(KLON,KLEV)&
 & ,ZQW5(KLON,KLEV),ZTW5(KLON,KLEV),ZMSE5(KLON,KLEV)&
 & ,ZMSEU5(KLON,KLEV),ZF5(KLON,KLEV),ZMFC5(KLON,KLEV)&
 & ,ZCORU5(KLON,KLEV),ZCORV5(KLON,KLEV),ZCORQ5(KLON,KLEV)&
 & ,ZCORS5(KLON,KLEV),ZFQSC5(KLON,KLEV)&
 & ,ZFUSC5(KLON,KLEV),ZFVSC5(KLON,KLEV),ZFSSC5(KLON,KLEV)&
 & ,ZQDN5(KLON,KLEV),ZPOID5(KLON,KLEV),ZIPOI5(KLON,KLEV)&
 & ,ZMSEC5(KLON,KLEV),ZPHI5(KLON,KLEV)  
REAL(KIND=JPRB) :: ZLHSB5(KLON),ZTN5(KLON),ZQN5(KLON),ZS15(KLON),ZS25(KLON)&
 & ,ZPOII5(KLON),ZALF5(KLON),ZLN5(KLON),ZUN5(KLON)&
 & ,ZVN5(KLON),ZMSEN5(KLON),ZDFMDQ5(KLON),ZDFMDU5(KLON)&
 & ,ZDFMDV5(KLON),ZDFMDS5(KLON),ZDSEN5(KLON),ZENTR5(KLON)&
 & ,ZDETR5(KLON),ZPOIL5(KLON)  

INTEGER(KIND=JPIM) :: ISUM, ITOP, JIT, JLEV, JLON

LOGICAL :: LL_LCLOSN

REAL(KIND=JPRB) :: ZCORIN, ZCPSMD, ZCPVMS, ZCPVMW, ZCPWMD, ZCVMCD,&
 & ZDELQ, ZDELQ5, ZDELT, ZDELT5, ZDELTA, ZDELTA5, &
 & ZDETC, ZDLEW, ZDLEW5, ZDQW, ZDQW5, ZEPS1, &
 & ZEPS2, ZEPS3, ZEPS4, ZEPS5, ZEPS6, ZEPS7, &
 & ZESP, ZESP5, ZEW, ZEW5, ZFCORQ, ZFCORQ5, &
 & ZFMDQ, ZFMDQ5, ZFMDS, ZFMDS5, ZFMDU, ZFMDU5, &
 & ZFMDV, ZFMDV5, ZFTOTQ, ZFTOTQ5, ZFTOTS, ZFTOTS5, &
 & ZGDT, ZGDTI, ZINSTA, ZIV, ZIV1, ZIV15, ZIV5, &
 & ZMSEP, ZMSEP5, ZMSEPR, ZMSEPR5, ZRVMD, ZTI, &
 & ZTI5, ZTQ, ZTQ5, ZZAL1, ZZAL15, ZZAL2, ZZAL25, &
 & ZZFO, ZZFO5, ZZM5, ZZPO1, ZZPO15, ZZPO2, &
 & ZZPO25, ZZPO5, ZZQW, ZZQW5, ZZSN, ZZSN1, &
 & ZZSN15, ZZSN2, ZZSN25, ZZSN5, ZZZL, ZZZL5, &
 & ZZZS, ZZVAL  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "fcttrm.h"
#include "fcttrmtl.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACCONVTL',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'CONVTL ') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

!*
!     ------------------------------------------------------------------
!     I - AUXILIARY CONSTANTS.

LL_LCLOSN=.FALSE.

ZRVMD=RV-RD
ZCPVMW=RCPV-RCW
ZCPVMS=RCPV-RCS
ZCPWMD=RCW-RCPD
ZCPSMD=RCS-RCPD
ZCVMCD=RCPV-RCPD

!*
!     ------------------------------------------------------------------
!     II - COMPUTATION OF DERIVED PARAMETERS, SECURITY CONSTANTS
!     AND INVERSE OF A CRITICAL ARGUMENT (TO AVOID "UNDERFLOW" IN
!     LIQUID (OR ICE) WATER CALCULATIONS FOR SUSTENTATION IN CLOUDY
!     ASCENTS).

ZGDT=RG*TSPHY
ZGDTI=1.0_JPRB/ZGDT

ZEPS1=1.E-01_JPRB
ZEPS2=1.E+01_JPRB
ZEPS3=1.E-01_JPRB
ZEPS4=1.E-04_JPRB

ZEPS5=1.E-06_JPRB
ZEPS6=1.E-07_JPRB
ZEPS7=1.E-05_JPRB

ZDETC=0.0_JPRB
ZCORIN=0.0_JPRB

!*
!     ------------------------------------------------------------------
!     III - COMPUTATION OF VALUE OF ATMOSPHERIC Cp,
!     SETTING ALL FLUXES TO ZERO (FOR THE CASE WITHOUT CONVECTION)
!     AND PRELIMINARY CALCULATIONS OF DRY STATIC ENERGY, OF EFFECTIVE
!     LATENT HEAT (DEPENDING UPON THE OPTION WHETHER SURFACE PRESSURE IS
!     VARYING OR NOT WITH THE SURFACE WATER FLUXES), OF LEVEL PRESSURE
!     THICKNESSES DIVIDED BY G*DT AND OF THEIR INVERSES AT ALL LEVELS AS
!     WELL AS OF IMPOSED PROPORTIONS OF SNOW PRECIPITATION IN FUNCTION
!     OF TEMPERATURE AND PRESSURE.

! - TEMPORARY(S) 2D (1:KLEV) .

! ZCP        : VALUE OF ATMOSPHERIC Cp
! ZPOID     : DP/(RG*DT) FOR A GIVEN LEVEL AND A GIVEN TIME STEP.
! ZIPOI     : INVERSE OF ZPOID.
! ZDSE      : LOCAL ARRAY FOR DRY STATIC ENERGY.
! ZMSE      : LOCAL ARRAY FOR MOIST STATIC ENERGY.
! ZLHE      : LOCAL ARRAY FOR EFFECTIVE LATENT HEATS.

! The same but for trajectory
! ---------------------------
! ZCP5      : VALUE OF ATMOSPHERIC Cp
! ZPOID5    : DP/(RG*DT) FOR A GIVEN LEVEL AND A GIVEN TIME STEP.
! ZIPOI5    : INVERSE OF ZPOID.
! ZDSE5     : LOCAL ARRAY FOR DRY STATIC ENERGY.
! ZMSE5     : LOCAL ARRAY FOR MOIST STATIC ENERGY.
! ZLHE5     : LOCAL ARRAY FOR EFFECTIVE LATENT HEATS.

! - TEMPORARY(S) 2D (0:KLEV) .

! ZSNP      : PROPORTION OF SNOW PRECIPITATION.
! ZLHL      : MINUS THE LATENT HEAT OF CONSERVATION OF LIQUID ENTHALPY.
! ZLHN      : MINUS THE LATENT HEAT OF CONSERVATION OF SOLID ENTHALPY.

! The same but for trajectory
! ---------------------------
! ZSNP5     : PROPORTION OF SNOW PRECIPITATION.
! ZLHL5     : MINUS THE LATENT HEAT OF CONSERVATION OF LIQUID ENTHALPY.
! ZLHN5     : MINUS THE LATENT HEAT OF CONSERVATION OF SOLID ENTHALPY.

!     COMPUTATION OF VALUE OF ATMOSPHERIC Cp

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZCP5(JLON,JLEV)=RCPD*(1.0_JPRB-PQ5(JLON,JLEV))+RCPV*PQ5(JLON,JLEV)
    ZCP(JLON,JLEV)=RCPD*(-PQ(JLON,JLEV))+RCPV*PQ(JLON,JLEV)
  ENDDO
ENDDO

!     SNOW OPTION DEPENDENT CALCULATIONS.

DO JLON=KIDIA,KFDIA

  IF (LNEIGE) THEN
    ZSNP5(JLON,KTDIA-1)=1.0_JPRB-MIN(1.0_JPRB,USDMLT*MAX(0.0_JPRB,PT5(JLON,KTDIA)&
     & -RTT)**2/MAX(1.E-03_JPRB,PAPRS5(JLON,KTDIA-1)))  
    IF (PAPRS5(JLON,KTDIA-1) > 1.E-03_JPRB) THEN
      ZZSN15=PAPRS5(JLON,KTDIA-1)
      ZZSN1 =PAPRS(JLON,KTDIA-1)
    ELSE
      ZZSN15=1.E-03_JPRB
      ZZSN1 =0.0_JPRB
    ENDIF
    IF ((PT5(JLON,KTDIA)-RTT) > 0.0_JPRB) THEN
      ZZSN25=PT5(JLON,KTDIA)-RTT
      ZZSN2 =PT(JLON,KTDIA)
    ELSE
      ZZSN25=0.0_JPRB
      ZZSN2 =0.0_JPRB
    ENDIF
    IF ((USDMLT*ZZSN25**2/ZZSN15) < 1.0_JPRB) THEN
      ZSNP(JLON,KTDIA-1)=-(2*USDMLT*ZZSN25/ZZSN15)*ZZSN2 &
       & +(USDMLT*ZZSN25**2/ZZSN15**2)*ZZSN1  
    ELSE
      ZSNP(JLON,KTDIA-1)=0.0_JPRB
    ENDIF
  ELSE
    ZSNP5(JLON,KTDIA-1)=0.0_JPRB
    ZSNP(JLON,KTDIA-1)=0.0_JPRB
  ENDIF

ENDDO

!     SNOW OPTION DEPENDENT CALCULATIONS.

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA

    IF (LNEIGE) THEN
      ZSNP5(JLON,JLEV)=1.0_JPRB-MIN(1.0_JPRB,USDMLT*MAX(0.0_JPRB,0.5_JPRB*(PT5(JLON, &
       & JLEV)&
       & +PT5(JLON,JLEV+1))-RTT)**2/PAPRS5(JLON,JLEV))  
      IF ((0.5_JPRB*(PT5(JLON,JLEV)+PT5(JLON,JLEV+1))-RTT) > 0.0_JPRB) THEN
        ZZSN5=0.5_JPRB*(PT5(JLON,JLEV)+PT5(JLON,JLEV+1))-RTT
        ZZSN =0.5_JPRB*PT(JLON,JLEV)+0.5_JPRB*PT(JLON,JLEV+1)
      ELSE
        ZZSN5=0.0_JPRB
        ZZSN =0.0_JPRB
      ENDIF
      IF ((USDMLT*ZZSN5**2/PAPRS5(JLON,JLEV)) < 1.0_JPRB) THEN
        ZSNP (JLON,JLEV)=-(2*USDMLT*ZZSN5/PAPRS5(JLON,JLEV))&
         & *ZZSN &
         & +(USDMLT*ZZSN5**2/PAPRS5(JLON,JLEV)**2)&
         & *PAPRS(JLON,JLEV)  
      ELSE
        ZSNP (JLON,JLEV)=0.0_JPRB
      ENDIF
    ELSE
      ZSNP5(JLON,JLEV)=0.0_JPRB
      ZSNP (JLON,JLEV)=0.0_JPRB
    ENDIF

  ENDDO
ENDDO

!     SNOW OPTION DEPENDENT CALCULATIONS.

DO JLON=KIDIA,KFDIA

  IF (LNEIGE) THEN
    ZSNP5(JLON,KLEV)=1.0_JPRB-MIN(1.0_JPRB,USDMLT*MAX(0.0_JPRB,PTS(JLON)-RTT)**2 &
     & /PAPRS5(JLON,KLEV))  
    ZZZS=USDMLT*MAX(0.0_JPRB,PTS(JLON)-RTT)**2
    IF ((ZZZS/PAPRS5(JLON,KLEV)) < 1.0_JPRB) THEN
      ZSNP (JLON,KLEV)=(ZZZS/PAPRS5(JLON,KLEV)**2)*PAPRS(JLON,KLEV)
    ELSE
      ZSNP (JLON,KLEV)=0.0_JPRB
    ENDIF
  ELSE
    ZSNP5(JLON,KLEV)=0.0_JPRB
    ZSNP (JLON,KLEV)=0.0_JPRB
  ENDIF

ENDDO

!     "FLUX LATENT HEAT" ARRAYS INITIALIZATION.

! --- COMPUTATION AT THE TOP
!     ----------------------
DO JLON=KIDIA,KFDIA
  ZTQ5=ZCVMCD*PT5(JLON,KTDIA)
  ZTQ=ZCVMCD*PT(JLON,KTDIA)

  ZLHL5(JLON,0)=-(FOLH(PT5(JLON,KTDIA),0.0_JPRB)-ZTQ5)
  ZLHL(JLON,0)=-(FOLHTL(PT(JLON,KTDIA),0.0_JPRB)-ZTQ)

  ZLHN5(JLON,0)=-(FOLH(PT5(JLON,KTDIA),1.0_JPRB)-ZTQ5)
  ZLHN(JLON,0)=-(FOLHTL(PT(JLON,KTDIA),1.0_JPRB)-ZTQ)
ENDDO

! --- COMPUTATION FOR LEVELS
!     ----------------------
DO JLEV = KTDIA , KLEV-1
  DO JLON = KIDIA , KFDIA
    ZTI5=0.5_JPRB*(PT5(JLON,JLEV)+PT5(JLON,JLEV+1))
    ZTI=0.5_JPRB*(PT(JLON,JLEV)+PT(JLON,JLEV+1))

    ZTQ5=ZCVMCD*ZTI5
    ZTQ=ZCVMCD*ZTI

    ZLHL5(JLON,JLEV)=-(FOLH(ZTI5,0.0_JPRB)-ZTQ5)
    ZLHL(JLON,JLEV)=-(FOLHTL(ZTI,0.0_JPRB)-ZTQ)

    ZLHN5(JLON,JLEV)=-(FOLH(ZTI5,1.0_JPRB)-ZTQ5)
    ZLHN(JLON,JLEV)=-(FOLHTL(ZTI,1.0_JPRB)-ZTQ)
  ENDDO
ENDDO

! --- SURFACE COMPUTATION
!     --------------------
DO JLON=KIDIA,KFDIA
  ZTQ5=ZCVMCD*PTS(JLON)
  ZLHL5(JLON,KLEV)=-(FOLH(PTS(JLON),0.0_JPRB)-ZTQ5)
  ZLHL(JLON,KLEV)=0.0_JPRB

  ZLHN5(JLON,KLEV)=-(FOLH(PTS(JLON),1.0_JPRB)-ZTQ5)
  ZLHN(JLON,KLEV)=0.0_JPRB
ENDDO

DO JLON=KIDIA,KFDIA

!     CALL TO THE DEFINED FUNCTION TO COMPUTE THE REFERENCE OF EFFECTIVE
!     LATENT HEAT AT THE SURFACE.

! - TEMPORARY(S) 1D .

! ZLHSB     : BUDGET LATENT HEAT AS COMPUTED AT THE SURFACE.

! The same but for trajectory
! ---------------------------
! ZLHSB5    : BUDGET LATENT HEAT AS COMPUTED AT THE SURFACE.

  ZLHSB5(JLON)=-ZLHL5(JLON,KLEV)-ZSNP5(JLON,KLEV)&
   & *(ZLHN5(JLON,KLEV)-ZLHL5(JLON,KLEV))  
  ZLHSB(JLON)=-ZLHL(JLON,KLEV)&
   & -(ZLHN5(JLON,KLEV)-ZLHL5(JLON,KLEV))*ZSNP(JLON,KLEV)&
   & -ZSNP5(JLON,KLEV)*ZLHN(JLON,KLEV)&
   & +ZSNP5(JLON,KLEV)*ZLHL(JLON,KLEV)  

!     SETTING TO ZERO FLUXES AT THE TOP.

  PFPLCL5(JLON,KTDIA-1)=0.0_JPRB
  PFPLCL(JLON,KTDIA-1)=0.0_JPRB
  PFPLCN5(JLON,KTDIA-1)=0.0_JPRB
  PFPLCN(JLON,KTDIA-1)=0.0_JPRB
  PDIFCQ5(JLON,KTDIA-1)=0.0_JPRB
  PDIFCQ(JLON,KTDIA-1)=0.0_JPRB
  PDIFCS5(JLON,KTDIA-1)=0.0_JPRB
  PDIFCS(JLON,KTDIA-1)=0.0_JPRB
  PSTRCU5(JLON,KTDIA-1)=0.0_JPRB
  PSTRCU(JLON,KTDIA-1)=0.0_JPRB
  PSTRCV5(JLON,KTDIA-1)=0.0_JPRB
  PSTRCV(JLON,KTDIA-1)=0.0_JPRB
ENDDO

!     COMPUTATIONS AT ALL LEVELS. BEWARE, THE DEFINED FUNCTION FOLH WILL
!     BE CALLED WITH VALUES THAT ARE NOT NECESSARILY 0 OR 1 FOR ITS
!     SECOND ARGUMENT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZPOID5(JLON,JLEV)=PDELP5(JLON,JLEV)*ZGDTI
    ZPOID(JLON,JLEV)=PDELP(JLON,JLEV)*ZGDTI

    ZIPOI5(JLON,JLEV)=PRDELP5(JLON,JLEV)*ZGDT
    ZIPOI(JLON,JLEV)=PRDELP(JLON,JLEV)*ZGDT

    ZDSE5(JLON,JLEV)=ZCP5(JLON,JLEV)*PT5(JLON,JLEV)+PAPHIF5(JLON,JLEV)
    ZDSE(JLON,JLEV)=ZCP5(JLON,JLEV)*PT(JLON,JLEV)&
     & +PT5(JLON,JLEV)*ZCP(JLON,JLEV)&
     & +PAPHIF(JLON,JLEV)  

    ZDELTA5=0.5_JPRB*(ZSNP5(JLON,JLEV-1)+ZSNP5(JLON,JLEV))
    ZDELTA=0.5_JPRB*(ZSNP(JLON,JLEV-1)+ZSNP(JLON,JLEV))

    ZLHE5(JLON,JLEV)=FOLH(PT5(JLON,JLEV),ZDELTA5)+(ZCPWMD+ZDELTA5 &
     & *(ZCPSMD-ZCPWMD))*(PT5(JLON,JLEV)&
     & -PTS(JLON))  
    ZLHE(JLON,JLEV)=RV*RBETD*ZDELTA &
     & -RV*(RGAMW+ZDELTA5*RGAMD)*PT(JLON,JLEV)&
     & -RV*RGAMD*PT5(JLON,JLEV)*ZDELTA &
     & +(ZCPWMD+ZDELTA5*(ZCPSMD-ZCPWMD))*PT(JLON,JLEV)&
     & +(PT5(JLON,JLEV)-PTS(JLON))*(ZCPSMD-ZCPWMD)&
     & *ZDELTA  

    ZMSE5(JLON,JLEV)=ZCP5(JLON,JLEV)*PT5(JLON,JLEV)&
     & +ZLHE5(JLON,JLEV)*PQ5(JLON,JLEV)&
     & +PAPHIF5(JLON,JLEV)  
    ZMSE(JLON,JLEV)=ZCP5(JLON,JLEV)*PT(JLON,JLEV)&
     & +PT5(JLON,JLEV)*ZCP(JLON,JLEV)&
     & +ZLHE5(JLON,JLEV)*PQ(JLON,JLEV)&
     & +PQ5(JLON,JLEV)*ZLHE(JLON,JLEV)&
     & +PAPHIF(JLON,JLEV)  

    ZMSEC5(JLON,JLEV)=ZCP5(JLON,JLEV)*PT5(JLON,JLEV)&
     & +ZLHSB5(JLON)*PQ5(JLON,JLEV)&
     & +PAPHIF5(JLON,JLEV)  
    ZMSEC(JLON,JLEV)=ZCP5(JLON,JLEV)*PT(JLON,JLEV)&
     & +PT5(JLON,JLEV)*ZCP(JLON,JLEV)&
     & +ZLHSB5(JLON)*PQ(JLON,JLEV)&
     & +PQ5(JLON,JLEV)*ZLHSB(JLON)&
     & +PAPHIF(JLON,JLEV)  

!     SETTING FLUXES TO ZERO.

    PFPLCL5(JLON,JLEV)=0.0_JPRB
    PFPLCL(JLON,JLEV)=0.0_JPRB
    PFPLCN5(JLON,JLEV)=0.0_JPRB
    PFPLCN(JLON,JLEV)=0.0_JPRB
    PDIFCQ5(JLON,JLEV)=0.0_JPRB
    PDIFCQ(JLON,JLEV)=0.0_JPRB
    PDIFCS5(JLON,JLEV)=0.0_JPRB
    PDIFCS(JLON,JLEV)=0.0_JPRB
    PSTRCU5(JLON,JLEV)=0.0_JPRB
    PSTRCU(JLON,JLEV)=0.0_JPRB
    PSTRCV5(JLON,JLEV)=0.0_JPRB
    PSTRCV(JLON,JLEV)=0.0_JPRB
  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     IV - COMPUTATION OF PROFILE OF MASS FLUX AND NORMALIZATION
!     FACTOR FOR THIS PROFILE CONNECTED WITH THE "KUO-TYPE" CLOSURE

!     ACTIVE CHARACTERISTICS OF THE CLOUD AT THE BOTTOM

! - TEMPORARY(S) 2D (0:KLEV) .

! ZFORM     : PROFILE OF MASS FLUX AND LATER MASS FLUX TIME DT.

! The same but for trajectory
! ---------------------------
! ZFORM5    : PROFILE OF MASS FLUX AND LATER MASS FLUX TIME DT.

! - TEMPORARY(S) 1D .

! ZS1       : INTEGRAL OF HUMIDITY CONVERGENCE
! ZS2       : INTEGRAL OF BUOYANCY

! The same but for trajectory
! ---------------------------
! ZS15      : INTEGRAL OF HUMIDITY CONVERGENCE
! ZS25      : INTEGRAL OF BUOYANCY

IF (.NOT.LL_LCLOSN) THEN
  DO JLON=KIDIA,KFDIA
    ZMSEU5(JLON,KLEV)=ZMSE5(JLON,KLEV)
    ZMSEU(JLON,KLEV)=ZMSE(JLON,KLEV)

    ZFORM5(JLON,KLEV)=0.0_JPRB
    ZFORM(JLON,KLEV)=0.0_JPRB

    ZS15(JLON)=0.0_JPRB
    ZS1(JLON)=0.0_JPRB
    ZS25(JLON)=0.0_JPRB
    ZS2(JLON)=0.0_JPRB
  ENDDO

!       COMPUTATION FOR LEVELS

  DO JLEV=KLEV-1,KTDIA,-1
    DO JLON=KIDIA,KFDIA
      ZINSTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZMSEU5(JLON,JLEV+1)-ZMSE5(JLON,JLEV)&
       & -ZCORIN))  
      ZMSEU5(JLON,JLEV)=ZINSTA*ZMSEU5(JLON,JLEV+1)&
       & +(1.0_JPRB-ZINSTA)*ZMSE5(JLON,JLEV)  
      ZMSEU(JLON,JLEV)=ZINSTA*ZMSEU(JLON,JLEV+1)&
       & +(1.0_JPRB-ZINSTA)*ZMSE(JLON,JLEV)  

      ZFORM5(JLON,JLEV)=ZINSTA*SQRT(MAX(0.0_JPRB,ZMSEU5(JLON,JLEV)&
       & +ZMSEU5(JLON,JLEV+1)&
       & -ZMSE5(JLON,JLEV)-ZMSE5(JLON,JLEV+1))&
       & *PAPRS5(JLON,JLEV)&
       & /(PT5(JLON,JLEV)+PT5(JLON,JLEV+1)))  
      ZZFO5=ZMSEU5(JLON,JLEV)+ZMSEU5(JLON,JLEV+1)&
       & -ZMSE5(JLON,JLEV)-ZMSE5(JLON,JLEV+1)  
      ZZFO=ZMSEU(JLON,JLEV)+ZMSEU(JLON,JLEV+1)&
       & -ZMSE(JLON,JLEV)-ZMSE(JLON,JLEV+1)  
      IF (ZZFO5 > 0.0_JPRB) THEN
        ZFORM(JLON,JLEV)=(0.5_JPRB*ZINSTA*ZZFO5 &
         & /(SQRT(ZZFO5*PAPRS5(JLON,JLEV)&
         & *(PT5(JLON,JLEV)+PT5(JLON,JLEV+1)))))&
         & *PAPRS(JLON,JLEV)&
         & +(0.5_JPRB*ZINSTA*PAPRS5(JLON,JLEV)&
         & /(SQRT(ZZFO5*PAPRS5(JLON,JLEV)&
         & *(PT5(JLON,JLEV)+PT5(JLON,JLEV+1)))))*ZZFO &
         & -(0.5_JPRB*ZINSTA*SQRT(ZZFO5*PAPRS5(JLON,JLEV)&
         & *(PT5(JLON,JLEV)+PT5(JLON,JLEV+1)))&
         & /(PT5(JLON,JLEV)+PT5(JLON,JLEV+1))**2)&
         & *(PT(JLON,JLEV)+PT(JLON,JLEV+1))  
      ELSE
        ZFORM(JLON,JLEV)=0.0_JPRB
      ENDIF

      ZPOII5(JLON)=MIN(1.0_JPRB,MAX(0.0_JPRB,(1.0_JPRB+ZDETC*PDELP5(JLON,JLEV+1))&
       & *ZFORM5(JLON,JLEV)-ZFORM5(JLON,JLEV+1))&
       & /MAX(ZEPS7,ZFORM5(JLON,JLEV)))  
      ZZPO5=(1.0_JPRB+ZDETC*PDELP5(JLON,JLEV+1))*ZFORM5(JLON,JLEV)&
       & -ZFORM5(JLON,JLEV+1)  
      IF (ZZPO5 > 0.0_JPRB) THEN
        ZZPO15=ZZPO5
        ZZPO1=(1.0_JPRB+ZDETC*PDELP5(JLON,JLEV+1))*ZFORM(JLON,JLEV)&
         & +ZDETC*ZFORM5(JLON,JLEV)*PDELP(JLON,JLEV+1)&
         & -ZFORM(JLON,JLEV+1)  
      ELSE
        ZZPO15=0.0_JPRB
        ZZPO1=0.0_JPRB
      ENDIF
      IF (ZFORM5(JLON,JLEV) > ZEPS7) THEN
        ZZPO25=ZFORM5(JLON,JLEV)
        ZZPO2=ZFORM(JLON,JLEV)
      ELSE
        ZZPO25=ZEPS7
        ZZPO2=0.0_JPRB
      ENDIF
      IF ((ZZPO15/ZZPO25) < 1.0_JPRB) THEN
        ZPOII(JLON)=ZZPO1/ZZPO25-(ZZPO15/ZZPO25**2)*ZZPO2
      ELSE
        ZPOII(JLON)=0.0_JPRB
      ENDIF

      ZMSEP5=ZPOII5(JLON)*ZMSE5(JLON,JLEV+1)&
       & +(1.0_JPRB-ZPOII5(JLON))*ZMSEU5(JLON,JLEV+1)  
      ZMSEP=ZPOII5(JLON)*ZMSE(JLON,JLEV+1)&
       & +ZMSE5(JLON,JLEV+1)*ZPOII(JLON)&
       & +(1.0_JPRB-ZPOII5(JLON))*ZMSEU(JLON,JLEV+1)&
       & -ZMSEU5(JLON,JLEV+1)*ZPOII(JLON)  

      ZINSTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZMSEP5-ZMSE5(JLON,JLEV)-ZCORIN))
      ZMSEU5(JLON,JLEV)=ZINSTA*ZMSEP5+(1.0_JPRB-ZINSTA)*ZMSE5(JLON,JLEV)
      ZMSEU(JLON,JLEV)=ZINSTA*ZMSEP+(1.0_JPRB-ZINSTA)*ZMSE(JLON,JLEV)

      ZFORM5(JLON,JLEV)=ZINSTA*SQRT(MAX(0.0_JPRB,ZMSEU5(JLON,JLEV)&
       & +ZMSEU5(JLON,JLEV+1)&
       & -ZMSE5(JLON,JLEV)-ZMSE5(JLON,JLEV+1))&
       & *PAPRS5(JLON,JLEV)&
       & /(PT5(JLON,JLEV)+PT5(JLON,JLEV+1)))  
      ZZFO5=ZMSEU5(JLON,JLEV)+ZMSEU5(JLON,JLEV+1)&
       & -ZMSE5(JLON,JLEV)-ZMSE5(JLON,JLEV+1)  
      ZZFO=ZMSEU(JLON,JLEV)+ZMSEU(JLON,JLEV+1)&
       & -ZMSE(JLON,JLEV)-ZMSE(JLON,JLEV+1)  
      IF (ZZFO5 > 0.0_JPRB) THEN
        ZFORM(JLON,JLEV)=(0.5_JPRB*ZINSTA*ZZFO5 &
         & /(SQRT(ZZFO5*PAPRS5(JLON,JLEV)&
         & *(PT5(JLON,JLEV)+PT5(JLON,JLEV+1)))))&
         & *PAPRS(JLON,JLEV)&
         & +(0.5_JPRB*ZINSTA*PAPRS5(JLON,JLEV)&
         & /(SQRT(ZZFO5*PAPRS5(JLON,JLEV)&
         & *(PT5(JLON,JLEV)+PT5(JLON,JLEV+1)))))*ZZFO &
         & -(0.5_JPRB*ZINSTA*SQRT(ZZFO5*PAPRS5(JLON,JLEV)&
         & *(PT5(JLON,JLEV)+PT5(JLON,JLEV+1)))&
         & /(PT5(JLON,JLEV)+PT5(JLON,JLEV+1))**2)&
         & *(PT(JLON,JLEV)+PT(JLON,JLEV+1))  
      ELSE
        ZFORM(JLON,JLEV)=0.0_JPRB
      ENDIF

      ZS15(JLON)=ZS15(JLON)+PCVGQ5(JLON,JLEV+1)*PDELP5(JLON,JLEV+1)*ZINSTA
      ZS1(JLON)=ZS1(JLON)&
       & +ZINSTA*PCVGQ5(JLON,JLEV+1)*PDELP(JLON,JLEV+1)&
       & +ZINSTA*PDELP5(JLON,JLEV+1)*PCVGQ(JLON,JLEV+1)  

      ZS25(JLON)=ZS25(JLON)+(PQ5(JLON,JLEV+1)-PQ5(JLON,JLEV))&
       & *ZFORM5(JLON,JLEV)  
      ZS2(JLON)=ZS2(JLON)&
       & +(PQ5(JLON,JLEV+1)-PQ5(JLON,JLEV))*ZFORM(JLON,JLEV)&
       & +ZFORM5(JLON,JLEV)*PQ(JLON,JLEV+1)&
       & -ZFORM5(JLON,JLEV)*PQ(JLON,JLEV)  
    ENDDO
  ENDDO

  DO JLON=KIDIA,KFDIA
    ZALF5(JLON)=(MAX(0.0_JPRB,ZS15(JLON))/MAX(ZEPS6,ZS25(JLON)))&
     & *MAX(0.0_JPRB,SIGN(1.0_JPRB,ZS25(JLON)-ZEPS6))  
    IF (ZS15(JLON) > 0.0_JPRB) THEN
      ZZAL15=ZS15(JLON)
      ZZAL1=ZS1(JLON)
    ELSE
      ZZAL15=0.0_JPRB
      ZZAL1=0.0_JPRB
    ENDIF
    IF (ZS25(JLON) > ZEPS6) THEN
      ZZAL25=ZS25(JLON)
      ZZAL2=ZS2(JLON)
    ELSE
      ZZAL25=ZEPS6
      ZZAL2=0.0_JPRB
    ENDIF
    ZALF(JLON)=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZS25(JLON)-ZEPS6))&
     & *(ZZAL1/ZZAL25-(ZZAL15/ZZAL25**2)*ZZAL2)  
  ENDDO
ENDIF

!*
!     ------------------------------------------------------------------
!     V - BOTTOM INITIALISATION. THE CLOUD PROFILE WILL EVENTUALY BE
!     CHARACTERISED BY ITS TOTAL HUMIDITY AND ITS DETRAINING DRY STATIC
!     ENERGY AS WELL AS BY A STABILITY INDEX (ONE AT BOTH ENDS OF ANY
!     CONVECTIVE SLAB (FROM MODEL LAYER TO THE ADJACENT ONE) AND ZERO
!     ELSEWHERE).

DO JLON=KIDIA,KFDIA

!     INITIALIZATION OF THE VARIABLES IN THE NEWTON LOOP,
!     THE ITERATIVE TW AND QW HAVING TO EVOLVE.

! - TEMPORARY 2D (1:KLEV)

! ZQW        : SPECIFIC WET BULB HUMIDITY.
! ZTW        : WET BULB TEMPERATURE.

! The same but for trajectory
! ---------------------------
! ZQW5       : SPECIFIC WET BULB HUMIDITY.
! ZTW5       : WET BULB TEMPERATURE.

  ZQW5(JLON,KLEV)=PQ5(JLON,KLEV)
  ZQW(JLON,KLEV)=PQ(JLON,KLEV)

  ZTW5(JLON,KLEV)=PT5(JLON,KLEV)
  ZTW(JLON,KLEV)=PT(JLON,KLEV)
ENDDO

!     NEWTON LOOP FOR THE "WET BULB" POINT.

DO JIT=1,NBITER
  DO JLON=KIDIA,KFDIA

!     SNOW OPTION DEPENDENT COMPUTATIONS.

    IF (LNEIGE) THEN
      ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-ZTW5(JLON,KLEV)))
    ELSE
      ZDELTA=0.0_JPRB
    ENDIF

!     SATURATION CALCULATIONS USING THE DEFINED FUNCTIONS

    ZEW5= FOEW (ZTW5(JLON,KLEV),ZDELTA)
    ZEW= FOEWTL (ZTW(JLON,KLEV),ZTW5(JLON,KLEV),ZDELTA)

    ZESP5=ZEW5/PAPRSF5(JLON,KLEV)
    ZESP=ZEW/PAPRSF5(JLON,KLEV)&
     & -(ZEW5/PAPRSF5(JLON,KLEV)**2)*PAPRSF(JLON,KLEV)  

    ZZQW5= FOQS (ZESP5)
    IF ((1.0_JPRB-ZESP5) > 0.0_JPRB) THEN
      ZZQW= FOQSTL1 (ZESP,ZESP5)
    ELSE
      ZZQW= FOQSTL2 (ZESP,ZESP5)
    ENDIF

    ZDLEW5=FODLEW (ZTW5(JLON,KLEV),ZDELTA)
    ZDLEW=FODLEWTL (ZTW(JLON,KLEV),ZTW5(JLON,KLEV),ZDELTA)
    ZDQW5= FODQS (ZZQW5,ZESP5,ZDLEW5)
    ZDQW= FODQSTL (ZZQW,ZZQW5,ZESP,ZESP5,ZDLEW,ZDLEW5)

!     INCREMENTATIONS.

    ZDELQ5=(ZZQW5-ZQW5(JLON,KLEV))*ZCP5(JLON,KLEV)&
     & /(ZCP5(JLON,KLEV)+ZLHSB5(JLON)*ZDQW5)  
    ZDELQ=ZCP5(JLON,KLEV)*ZZQW &
     & /(ZCP5(JLON,KLEV)+ZLHSB5(JLON)*ZDQW5)&
     & -ZCP5(JLON,KLEV)*ZQW(JLON,KLEV)&
     & /(ZCP5(JLON,KLEV)+ZLHSB5(JLON)*ZDQW5)&
     & +(ZZQW5-ZQW5(JLON,KLEV))*ZCP(JLON,KLEV)&
     & /(ZCP5(JLON,KLEV)+ZLHSB5(JLON)*ZDQW5)&
     & -(ZZQW5-ZQW5(JLON,KLEV))*ZCP5(JLON,KLEV)&
     & *ZCP(JLON,KLEV)&
     & /(ZCP5(JLON,KLEV)+ZLHSB5(JLON)*ZDQW5)**2 &
     & -(ZZQW5-ZQW5(JLON,KLEV))*ZCP5(JLON,KLEV)*ZDQW5*ZLHSB(JLON)&
     & /(ZCP5(JLON,KLEV)+ZLHSB5(JLON)*ZDQW5)**2 &
     & -(ZZQW5-ZQW5(JLON,KLEV))*ZCP5(JLON,KLEV)*ZLHSB5(JLON)*ZDQW &
     & /(ZCP5(JLON,KLEV)+ZLHSB5(JLON)*ZDQW5)**2  

    ZDELT5=-ZDELQ5*ZLHSB5(JLON)/ZCP5(JLON,KLEV)
    ZDELT=-(ZLHSB5(JLON)/ZCP5(JLON,KLEV))*ZDELQ &
     & -(ZDELQ5/ZCP5(JLON,KLEV))*ZLHSB(JLON)&
     & +(ZDELQ5*ZLHSB5(JLON)/ZCP5(JLON,KLEV)**2)*ZCP(JLON,KLEV)  

    ZQW5(JLON,KLEV)=ZQW5(JLON,KLEV)+ZDELQ5
    ZQW(JLON,KLEV)=ZQW(JLON,KLEV)+ZDELQ

    ZTW5(JLON,KLEV)=ZTW5(JLON,KLEV)+ZDELT5
    ZTW(JLON,KLEV)=ZTW(JLON,KLEV)+ZDELT
  ENDDO

ENDDO

DO JLON=KIDIA,KFDIA

!     THERMODYNAMIC CHARACTERISTICS OF THE CLOUD.

! - TEMPORARY(S) 1D .

! ZTN       : TEMPERATURE OF THE CLOUDY ASCENT.
! ZQN       : WATER VAPOUR SPECIFIC HUMIDITY OF THE CLOUDY ASCENT.
! ZLN       : CONDENSED SPECIFIC HUMIDITY OF THE CLOUDY ASCENT.
! ZUN       : U-WIND COMPONENT OF THE CLOUDY ASCENT.
! ZVN       : V-WIND COMPONENT OF THE CLOUDY ASCENT.
! ZMSEN     : MOIST STATIC ENERGY OF THE CLOUDY ASCENT.
! ZDSEN     : DRY STATIC ENERGY OF THE CLOUDY ASCENT.

! The same but for trajectory
! ---------------------------
! ZTN5      : TEMPERATURE OF THE CLOUDY ASCENT.
! ZQN5      : WATER VAPOUR SPECIFIC HUMIDITY OF THE CLOUDY ASCENT.
! ZLN5      : CONDENSED SPECIFIC HUMIDITY OF THE CLOUDY ASCENT.
! ZUN5      : U-WIND COMPONENT OF THE CLOUDY ASCENT.
! ZVN5      : V-WIND COMPONENT OF THE CLOUDY ASCENT.
! ZMSEN5    : MOIST STATIC ENERGY OF THE CLOUDY ASCENT.
! ZDSEN5    : DRY STATIC ENERGY OF THE CLOUDY ASCENT.

  ZTN5(JLON)=ZTW5(JLON,KLEV)
  ZTN(JLON)=ZTW(JLON,KLEV)

  ZQN5(JLON)=ZQW5(JLON,KLEV)
  ZQN(JLON)=ZQW(JLON,KLEV)

  ZUN5(JLON)=PU5(JLON,KLEV)
  ZUN(JLON)=PU(JLON,KLEV)

  ZVN5(JLON)=PV5(JLON,KLEV)
  ZVN(JLON)=PV(JLON,KLEV)

  ZMSEN5(JLON)=ZCP5(JLON,KLEV)*ZTN5(JLON)&
   & +ZLHSB5(JLON)*ZQN5(JLON)+PAPHIF5(JLON,KLEV)  
  ZMSEN(JLON)=ZCP5(JLON,KLEV)*ZTN(JLON)&
   & +ZTN5(JLON)*ZCP(JLON,KLEV)&
   & +ZLHSB5(JLON)*ZQN(JLON)+ZQN5(JLON)*ZLHSB(JLON)&
   & +PAPHIF(JLON,KLEV)  

  ZDSEN5(JLON)=ZCP5(JLON,KLEV)*ZTN5(JLON)+PAPHIF5(JLON,KLEV)
  ZDSEN(JLON)=ZCP5(JLON,KLEV)*ZTN(JLON)&
   & +ZTN5(JLON)*ZCP(JLON,KLEV)+PAPHIF(JLON,KLEV)  

  ZLN5(JLON)=0.0_JPRB
  ZLN(JLON)=0.0_JPRB

!     ACTIVE OTHER CHARACTERISTICS OF THE CLOUD.

! - TEMPORARY(S) 2D (1:KLEV)

! ZMCF      : MASS FLUX IN CLOUD
! ZQDN      : TOTAL CLOUD HUMIDITY

! The same but for trajectory
! ---------------------------
! ZMCF5     : MASS FLUX IN CLOUD
! ZQDN5     : TOTAL CLOUD HUMIDITY

  ZMFC5(JLON,KLEV)=0.0_JPRB
  ZMFC(JLON,KLEV)=0.0_JPRB

  ZQDN5(JLON,KLEV)=ZQN5(JLON)
  ZQDN(JLON,KLEV)=ZQN(JLON)

  I_KNLAB(JLON,KLEV)=0
  I_KNND(JLON)=0

ENDDO

!*
!     ------------------------------------------------------------------
!     VI - FIRST VERTICAL LOOP (UPWARDS) INCLUDING THE SATURATED
!     ADIABATIC TYPE CALCULATION OF THE CLOUD PROFILE.

!     INITIALIZATION OF THE PARAMETER THAT SHALL EVENTUALLY HELP
!     AVOIDING UNNECESSARY COMPUTATIONS IN THE UPPER PART OF THE
!     ATMOSPHERE AND START OF THE VERTICAL LOOP.

ITOP=KLEV+1

!     OPTIONAL COMPUTATION ACCORDING CHOSEN CLOSURE ASSUMPTION
!     COMPUTATION OF VALUES OF DETRAINMENT AND ENTRAINMENT, TEST
!     OF STABILITY, COMPUTATION OF THE SECOND TERM OF THE BASIC EQUATION,
!     COMPUTATION OF VALUES u,v,q,h IN CLOUD

! - TEMPORARY(S) 2D (1:KLEV)

!   ZCORU   : THE SECOND TERM OF THE BASIC EQUATION FOR U-WIND COMP.
!   ZCORV   : THE SECOND TERM OF THE BASIC EQUATION FOR V-WIND COMP.
!   ZCORQ   : THE SECOND TERM OF THE BASIC EQUATION FOR SPEC.HUMIDITY
!   ZCORS   : THE SECOND TERM OF THE BASIC EQUATION FOR DRY STATIC
!             ENERGY

! The same but for trajectory
! ---------------------------
!   ZCORU5  : THE SECOND TERM OF THE BASIC EQUATION FOR U-WIND COMP.
!   ZCORV5  : THE SECOND TERM OF THE BASIC EQUATION FOR V-WIND COMP.
!   ZCORQ5  : THE SECOND TERM OF THE BASIC EQUATION FOR SPEC.HUMIDITY
!   ZCORS5  : THE SECOND TERM OF THE BASIC EQUATION FOR DRY STATIC
!             ENERGY

DO JLEV=KLEV-1,KTDIA,-1

  DO JLON=KIDIA,KFDIA

!     OPTIONAL COMPUTATION ACCORDING CHOSEN CLOSURE ASSUMPTION
!     IF LCLOSN=TRUE NEW CLOSURE ASSUMPTION OF SIMPLIFIED PHYSICS
!     WILL BE USED

    IF (LL_LCLOSN) THEN
      ZF5(JLON,JLEV+1)=PCVGQ5(JLON,JLEV+1)/MAX(ZEPS5,PQ5(JLON,JLEV+1))
      IF (PQ5(JLON,JLEV+1) > ZEPS5) THEN
        ZF(JLON,JLEV+1)=PCVGQ(JLON,JLEV+1)/PQ5(JLON,JLEV+1)&
         & -(PCVGQ5(JLON,JLEV+1)/PQ5(JLON,JLEV+1)**2)&
         & *PQ(JLON,JLEV+1)  
      ELSE
        ZF(JLON,JLEV+1)=PCVGQ(JLON,JLEV+1)/ZEPS5
      ENDIF
    ELSE
      ZF5(JLON,JLEV+1)=ZALF5(JLON)*(ZFORM5(JLON,JLEV)&
       & -ZFORM5(JLON,JLEV+1))/PDELP5(JLON,JLEV+1)  
      ZF(JLON,JLEV+1)=((ZFORM5(JLON,JLEV)-ZFORM5(JLON,JLEV+1))&
       & /PDELP5(JLON,JLEV+1))*ZALF(JLON)&
       & +(ZALF5(JLON)/PDELP5(JLON,JLEV+1))*ZFORM(JLON,JLEV)&
       & -(ZALF5(JLON)/PDELP5(JLON,JLEV+1))*ZFORM(JLON,JLEV+1)&
       & -(ZALF5(JLON)*(ZFORM5(JLON,JLEV)&
       & -ZFORM5(JLON,JLEV+1))/PDELP5(JLON,JLEV+1)**2)&
       & *PDELP(JLON,JLEV+1)  
    ENDIF

    IF (ZF5(JLON,JLEV+1) > 0.0_JPRB) THEN
      ZF(JLON,JLEV+1)=ZF(JLON,JLEV+1)/(1.0_JPRB+TSPHY*ZF5(JLON,JLEV+1))**2
    ELSE
      ZF(JLON,JLEV+1)=ZF(JLON,JLEV+1)
    ENDIF
    ZF5(JLON,JLEV+1)=ZF5(JLON,JLEV+1)&
     & /(1.0_JPRB+MAX(0.0_JPRB,ZF5(JLON,JLEV+1))*TSPHY)  

    ZMFC5(JLON,JLEV)=MAX(0.0_JPRB,ZMFC5(JLON,JLEV+1)&
     & +ZF5(JLON,JLEV+1)*PDELP5(JLON,JLEV+1))  
    ZZM5=ZMFC5(JLON,JLEV+1)+ZF5(JLON,JLEV+1)*PDELP5(JLON,JLEV+1)
    IF (ZZM5 > 0.0_JPRB) THEN
      ZMFC(JLON,JLEV)=ZMFC(JLON,JLEV+1)&
       & +ZF5(JLON,JLEV+1)*PDELP(JLON,JLEV+1)&
       & +PDELP5(JLON,JLEV+1)*ZF(JLON,JLEV+1)  
    ELSE
      ZMFC(JLON,JLEV)=0.0_JPRB
    ENDIF

!     COMPUTATION OF DETRAINMENT AND ENTRAINMENT

    ZENTR5(JLON)=(MAX(0.0_JPRB,ZF5(JLON,JLEV+1)+ZDETC*ZMFC5(JLON,JLEV)))&
     & *PDELP5(JLON,JLEV+1)  
    IF ((ZF5(JLON,JLEV+1)+ZDETC*ZMFC5(JLON,JLEV)) > 0.0_JPRB) THEN
      ZENTR(JLON)=(ZF5(JLON,JLEV+1)+ZDETC*ZMFC5(JLON,JLEV))&
       & *PDELP(JLON,JLEV+1)&
       & +PDELP5(JLON,JLEV+1)*ZF(JLON,JLEV+1)&
       & +ZDETC*PDELP5(JLON,JLEV+1)*ZMFC(JLON,JLEV)  
    ELSE
      ZENTR(JLON)=0.0_JPRB
    ENDIF

    IF (ZENTR5(JLON) < ZMFC5(JLON,JLEV)) THEN
      ZENTR(JLON)=ZENTR(JLON)
    ELSE
      ZENTR(JLON)=ZMFC(JLON,JLEV)
    ENDIF
    ZENTR5(JLON)=MIN(ZENTR5(JLON),ZMFC5(JLON,JLEV))

    ZDETR5(JLON)=ZENTR5(JLON)+ZMFC5(JLON,JLEV+1)-ZMFC5(JLON,JLEV)
    ZDETR(JLON)=ZENTR(JLON)+ZMFC(JLON,JLEV+1)-ZMFC(JLON,JLEV)

    IF (LSMOOTHB) THEN
      ZMFC(JLON,JLEV)=0.0_JPRB
      ZENTR(JLON)=0.0_JPRB
      ZDETR(JLON)=0.0_JPRB
    ENDIF

    ZPOIL5(JLON)=ZENTR5(JLON)/MAX(ZEPS4,ZMFC5(JLON,JLEV))
    IF (ZMFC5(JLON,JLEV) > ZEPS4) THEN
      ZPOIL(JLON)=ZENTR(JLON)/ZMFC5(JLON,JLEV)&
       & -(ZENTR5(JLON)/ZMFC5(JLON,JLEV)**2)*ZMFC(JLON,JLEV)  
    ELSE
      ZPOIL(JLON)=ZENTR(JLON)/ZEPS4
    ENDIF

!     STABILTY TEST

    ZMSEPR5=ZPOIL5(JLON)*ZMSEC5(JLON,JLEV+1)&
     & +(1.0_JPRB-ZPOIL5(JLON))*ZMSEN5(JLON)  
    ZMSEPR=ZPOIL5(JLON)*ZMSEC(JLON,JLEV+1)&
     & +(ZMSEC5(JLON,JLEV+1)-ZMSEN5(JLON))*ZPOIL(JLON)&
     & +(1.0_JPRB-ZPOIL5(JLON))*ZMSEN(JLON)  

    I_KNLAB(JLON,JLEV)=NINT(MAX(0.0_JPRB,-SIGN(1.0_JPRB,&
     & -(ZMSEPR5-ZMSEC5(JLON,JLEV))))&
     & *MAX(MAX(0.0_JPRB,SIGN(1.0_JPRB,&
     & ZDSE5(JLON,JLEV)-ZDSE5(JLON,JLEV+1))),&
     & MAX(0.0_JPRB,-SIGN(1.0_JPRB,0.0_JPRB-ZMFC5(JLON,JLEV+1)))))  

    I_KNLAB(JLON,JLEV+1)=MAX(I_KNLAB(JLON,JLEV+1),I_KNLAB(JLON,JLEV))
    I_KNND(JLON)=MAX(I_KNND(JLON),I_KNLAB(JLON,JLEV))

    ZMFC5(JLON,JLEV)=I_KNLAB(JLON,JLEV)*ZMFC5(JLON,JLEV)
    ZMFC(JLON,JLEV)=I_KNLAB(JLON,JLEV)*ZMFC(JLON,JLEV)

    ZDETR5(JLON)=I_KNLAB(JLON,JLEV)*ZDETR5(JLON)&
     & +(1-I_KNLAB(JLON,JLEV))*ZMFC5(JLON,JLEV+1)  
    ZDETR(JLON)=I_KNLAB(JLON,JLEV)*ZDETR(JLON)&
     & +(1-I_KNLAB(JLON,JLEV))*ZMFC(JLON,JLEV+1)  

    ZENTR5(JLON)=I_KNLAB(JLON,JLEV)*ZENTR5(JLON)
    ZENTR(JLON)=I_KNLAB(JLON,JLEV)*ZENTR(JLON)

!     COMPUTATION OF THE SECOND TERM OF THE BASIC EQUATION

    ZCORU5(JLON,JLEV+1)=ZENTR5(JLON)*PU5(JLON,JLEV+1)-ZDETR5(JLON)*ZUN5(JLON)
    ZCORU(JLON,JLEV+1)=ZENTR5(JLON)*PU(JLON,JLEV+1)&
     & +PU5(JLON,JLEV+1)*ZENTR(JLON)&
     & -ZDETR5(JLON)*ZUN(JLON)-ZUN5(JLON)*ZDETR(JLON)  

    ZCORV5(JLON,JLEV+1)=ZENTR5(JLON)*PV5(JLON,JLEV+1)-ZDETR5(JLON)*ZVN5(JLON)
    ZCORV(JLON,JLEV+1)=ZENTR5(JLON)*PV(JLON,JLEV+1)&
     & +PV5(JLON,JLEV+1)*ZENTR(JLON)&
     & -ZDETR5(JLON)*ZVN(JLON)-ZVN5(JLON)*ZDETR(JLON)  

    ZCORQ5(JLON,JLEV+1)=ZENTR5(JLON)*PQ5(JLON,JLEV+1)&
     & -ZDETR5(JLON)*(ZQN5(JLON)+ZLN5(JLON))  
    ZCORQ(JLON,JLEV+1)=ZENTR5(JLON)*PQ(JLON,JLEV+1)&
     & +PQ5(JLON,JLEV+1)*ZENTR(JLON)&
     & -(ZQN5(JLON)+ZLN5(JLON))*ZDETR(JLON)&
     & -ZDETR5(JLON)*ZQN(JLON)-ZDETR5(JLON)*ZLN(JLON)  

    ZCORS5(JLON,JLEV+1)=ZENTR5(JLON)*ZDSE5(JLON,JLEV+1)&
     & -ZDETR5(JLON)&
     & *(ZDSEN5(JLON)-ZLHSB5(JLON)*ZLN5(JLON))  
    ZCORS(JLON,JLEV+1)=ZENTR5(JLON)*ZDSE(JLON,JLEV+1)&
     & +ZDSE5(JLON,JLEV+1)*ZENTR(JLON)&
     & -(ZDSEN5(JLON)-ZLHSB5(JLON)*ZLN5(JLON))*ZDETR(JLON)&
     & -ZDETR5(JLON)*ZDSEN(JLON)&
     & +ZDETR5(JLON)*ZLHSB5(JLON)*ZLN(JLON)&
     & +ZDETR5(JLON)*ZLN5(JLON)*ZLHSB(JLON)  

!     COMPUTATION OF VALUES u,v,h,q,T IN THE CLOUD

    ZUN(JLON)=I_KNLAB(JLON,JLEV)*(ZPOIL5(JLON)*PU(JLON,JLEV+1)&
     & +(PU5(JLON,JLEV+1)-ZUN5(JLON))*ZPOIL(JLON)&
     & +(1.0_JPRB-ZPOIL5(JLON))*ZUN(JLON))&
     & +(1-I_KNLAB(JLON,JLEV))*PU(JLON,JLEV)  
    ZUN5(JLON)=I_KNLAB(JLON,JLEV)*(ZPOIL5(JLON)*PU5(JLON,JLEV+1)&
     & +(1.0_JPRB-ZPOIL5(JLON))*ZUN5(JLON))&
     & +(1-I_KNLAB(JLON,JLEV))*PU5(JLON,JLEV)  

    ZVN(JLON)=I_KNLAB(JLON,JLEV)*(ZPOIL5(JLON)*PV(JLON,JLEV+1)&
     & +(PV5(JLON,JLEV+1)-ZVN5(JLON))*ZPOIL(JLON)&
     & +(1.0_JPRB-ZPOIL5(JLON))*ZVN(JLON))&
     & +(1-I_KNLAB(JLON,JLEV))*PV(JLON,JLEV)  
    ZVN5(JLON)=I_KNLAB(JLON,JLEV)*(ZPOIL5(JLON)*PV5(JLON,JLEV+1)&
     & +(1.0_JPRB-ZPOIL5(JLON))*ZVN5(JLON))&
     & +(1-I_KNLAB(JLON,JLEV))*PV5(JLON,JLEV)  

    ZQW(JLON,JLEV)=I_KNLAB(JLON,JLEV)&
     & *(ZPOIL5(JLON)*PQ(JLON,JLEV+1)&
     & +(PQ5(JLON,JLEV+1)-ZQN5(JLON))*ZPOIL(JLON)&
     & +(1.0_JPRB-ZPOIL5(JLON))*ZQN(JLON))&
     & +(1-I_KNLAB(JLON,JLEV))*PQ(JLON,JLEV)  
    ZQW5(JLON,JLEV)=I_KNLAB(JLON,JLEV)*(ZPOIL5(JLON)&
     & *PQ5(JLON,JLEV+1)&
     & +(1.0_JPRB-ZPOIL5(JLON))*ZQN5(JLON))&
     & +(1-I_KNLAB(JLON,JLEV))*PQ5(JLON,JLEV)  

    ZDSEN(JLON)=I_KNLAB(JLON,JLEV)*(ZPOIL5(JLON)*ZDSE(JLON,JLEV+1)&
     & +(ZDSE5(JLON,JLEV+1)-ZDSEN5(JLON))*ZPOIL(JLON)&
     & +(1.0_JPRB-ZPOIL5(JLON))*ZDSEN(JLON))&
     & +(1-I_KNLAB(JLON,JLEV))*ZDSE(JLON,JLEV)  
    ZDSEN5(JLON)=I_KNLAB(JLON,JLEV)*(ZPOIL5(JLON)&
     & *ZDSE5(JLON,JLEV+1)&
     & +(1.0_JPRB-ZPOIL5(JLON))*ZDSEN5(JLON))&
     & +(1-I_KNLAB(JLON,JLEV))*ZDSE5(JLON,JLEV)  

    ZMSEN5(JLON)=I_KNLAB(JLON,JLEV)*ZMSEPR5 &
     & +(1-I_KNLAB(JLON,JLEV))*ZMSEC5(JLON,JLEV)  
    ZMSEN(JLON)=I_KNLAB(JLON,JLEV)*ZMSEPR+(1-I_KNLAB(JLON,JLEV))*ZMSEC(JLON,JLEV)

!     INITIALIZATION OF THE VARIABLES IN THE NEWTON LOOP,
!     THE ITERATIVE TW AND QW HAVING TO EVOLVE.

    ZTW5(JLON,JLEV)=(ZDSEN5(JLON)-PAPHIF5(JLON,JLEV))/ZCP5(JLON,JLEV)
    ZTW(JLON,JLEV)=ZDSEN(JLON)/ZCP5(JLON,JLEV)&
     & -PAPHIF(JLON,JLEV)/ZCP5(JLON,JLEV)&
     & -((ZDSEN5(JLON)-PAPHIF5(JLON,JLEV))&
     & /ZCP5(JLON,JLEV)**2)*ZCP(JLON,JLEV)  
  ENDDO

!     NEWTON LOOP FOR THE "WET BULB" POINT.

  DO JIT=1,NBITER
    DO JLON=KIDIA,KFDIA

!     SNOW OPTION DEPENDENT COMPUTATIONS.

      IF (LNEIGE) THEN
        ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-ZTW5(JLON,JLEV)))
      ELSE
        ZDELTA=0.0_JPRB
      ENDIF

!     SATURATION CALCULATIONS USING THE DEFINED FUNCTIONS AND TEMPORARY

! - TEMPORARY 1D .

      ZEW5= FOEW (ZTW5(JLON,JLEV),ZDELTA)
      ZEW= FOEWTL (ZTW(JLON,JLEV),ZTW5(JLON,JLEV),ZDELTA)

      ZESP5=ZEW5/PAPRSF5(JLON,JLEV)
      ZESP=ZEW/PAPRSF5(JLON,JLEV)&
       & -(ZEW5/PAPRSF5(JLON,JLEV)**2)*PAPRSF(JLON,JLEV)  

      ZZQW5= FOQS (ZESP5)
      IF ((1.0_JPRB-ZESP5) > 0.0_JPRB) THEN
        ZZQW= FOQSTL1 (ZESP,ZESP5)
      ELSE
        ZZQW= FOQSTL2 (ZESP,ZESP5)
      ENDIF

      ZDLEW5=FODLEW (ZTW5(JLON,JLEV),ZDELTA)
      ZDLEW=FODLEWTL (ZTW(JLON,JLEV),ZTW5(JLON,JLEV),ZDELTA)
      ZDQW5= FODQS (ZZQW5,ZESP5,ZDLEW5)
      ZDQW= FODQSTL (ZZQW,ZZQW5,ZESP,ZESP5,ZDLEW,ZDLEW5)

!     INCREMENTATIONS.

      ZDELQ5=(ZZQW5-ZQW5(JLON,JLEV))*ZCP5(JLON,JLEV)&
       & /(ZCP5(JLON,JLEV)+ZLHSB5(JLON)*ZDQW5)  
      ZDELQ=ZCP5(JLON,JLEV)*ZZQW &
       & /(ZCP5(JLON,JLEV)+ZLHSB5(JLON)*ZDQW5)&
       & -ZCP5(JLON,JLEV)*ZQW(JLON,JLEV)&
       & /(ZCP5(JLON,JLEV)+ZLHSB5(JLON)*ZDQW5)&
       & +(ZZQW5-ZQW5(JLON,JLEV))*ZCP(JLON,JLEV)&
       & /(ZCP5(JLON,JLEV)+ZLHSB5(JLON)*ZDQW5)&
       & -(ZZQW5-ZQW5(JLON,JLEV))*ZCP5(JLON,JLEV)*ZCP(JLON,JLEV)&
       & /(ZCP5(JLON,JLEV)+ZLHSB5(JLON)*ZDQW5)**2 &
       & -(ZZQW5-ZQW5(JLON,JLEV))*ZCP5(JLON,JLEV)*ZDQW5*ZLHSB(JLON)&
       & /(ZCP5(JLON,JLEV)+ZLHSB5(JLON)*ZDQW5)**2 &
       & -(ZZQW5-ZQW5(JLON,JLEV))*ZCP5(JLON,JLEV)*ZLHSB5(JLON)*ZDQW &
       & /(ZCP5(JLON,JLEV)+ZLHSB5(JLON)*ZDQW5)**2  

      ZDELT5=-ZDELQ5*ZLHSB5(JLON)/ZCP5(JLON,JLEV)
      ZDELT=-(ZLHSB5(JLON)/ZCP5(JLON,JLEV))*ZDELQ &
       & -(ZDELQ5/ZCP5(JLON,JLEV))*ZLHSB(JLON)&
       & +(ZDELQ5*ZLHSB5(JLON)/ZCP5(JLON,JLEV)**2)*ZCP(JLON,JLEV)  

      ZQW5(JLON,JLEV)=ZQW5(JLON,JLEV)+ZDELQ5
      ZQW(JLON,JLEV)=ZQW(JLON,JLEV)+ZDELQ

      ZTW5(JLON,JLEV)=ZTW5(JLON,JLEV)+ZDELT5
      ZTW(JLON,JLEV)=ZTW(JLON,JLEV)+ZDELT
    ENDDO

  ENDDO

  DO JLON=KIDIA,KFDIA
    ZPHI5(JLON,JLEV)=PAPHIF5(JLON,JLEV)-PAPHIF5(JLON,JLEV+1)
    ZPHI(JLON,JLEV)=PAPHIF(JLON,JLEV)-PAPHIF(JLON,JLEV+1)

    ZZZL5=ZLN5(JLON)*(1.0_JPRB-ZPOIL5(JLON)/(1.0_JPRB+ZPOIL5(JLON)))&
     & *(1.0_JPRB-ZPHI5(JLON,JLEV)/ECMNP)&
     & -(ZQW5(JLON,JLEV)-ZQN5(JLON)&
     & -(ZPOIL5(JLON)/(1.0_JPRB+ZPOIL5(JLON)))&
     & *(PQ5(JLON,JLEV+1)-ZQN5(JLON)))  
    ZZZL=(1.0_JPRB-ZPOIL5(JLON)/(1.0_JPRB+ZPOIL5(JLON)))&
     & *(1.0_JPRB-ZPHI5(JLON,JLEV)/ECMNP)*ZLN(JLON)&
     & -ZLN5(JLON)*(1.0_JPRB-ZPOIL5(JLON)/(1.0_JPRB+ZPOIL5(JLON)))&
     & *ZPHI(JLON,JLEV)/ECMNP &
     & -(ZLN5(JLON)*(1.0_JPRB-ZPHI5(JLON,JLEV)/ECMNP)&
     & /(1.0_JPRB+ZPOIL5(JLON))**2)*ZPOIL(JLON)&
     & -ZQW(JLON,JLEV)+ZQN(JLON)&
     & +(ZPOIL5(JLON)/(1.0_JPRB+ZPOIL5(JLON)))&
     & *(PQ(JLON,JLEV+1)-ZQN(JLON))&
     & +((PQ5(JLON,JLEV+1)-ZQN5(JLON))/(1.0_JPRB+ZPOIL5(JLON))**2)&
     & *ZPOIL(JLON)  
    IF (ZZZL5 > 0.0_JPRB) THEN
      ZLN5(JLON)=ZZZL5*I_KNLAB(JLON,JLEV)
      ZLN(JLON)=ZZZL*I_KNLAB(JLON,JLEV)
    ELSE
      ZLN5(JLON)=0.0_JPRB
      ZLN(JLON)=0.0_JPRB
    ENDIF

    ZQN5(JLON)=ZQW5(JLON,JLEV)
    ZQN(JLON)=ZQW(JLON,JLEV)

    ZQDN5(JLON,JLEV)=ZQN5(JLON)+ZLN5(JLON)
    ZQDN(JLON,JLEV)=ZQN(JLON)+ZLN(JLON)

    ZTN5(JLON)=ZTW5(JLON,JLEV)
    ZTN(JLON)=ZTW(JLON,JLEV)
  ENDDO

!     VERIFICATION OF THE PRESENCE OF AT LEAST ONE CLOUD ALONG THE
!     "JLON" VECTOR AND MODIFICATION OF ITOP IF NECESSARY.

  ISUM=0
  DO JLON=KIDIA,KFDIA
    ISUM=ISUM+I_KNLAB(JLON,JLEV)+I_KNLAB(JLON,JLEV+1)
  ENDDO

  IF (ISUM /= 0) THEN
    ITOP=JLEV
  ENDIF

ENDDO

!     CALCULATION AT THE TOP OF THE ATMOSPHERE

DO JLON=KIDIA,KFDIA

  ZCORU5(JLON,KTDIA)=-ZMFC5(JLON,KTDIA)*ZUN5(JLON)
  ZCORU(JLON,KTDIA)=-ZMFC5(JLON,KTDIA)*ZUN(JLON)-ZUN5(JLON)*ZMFC(JLON,KTDIA)

  ZCORV5(JLON,KTDIA)=-ZMFC5(JLON,KTDIA)*ZVN5(JLON)
  ZCORV(JLON,KTDIA)=-ZMFC5(JLON,KTDIA)*ZVN(JLON)-ZVN5(JLON)*ZMFC(JLON,KTDIA)

  ZCORQ5(JLON,KTDIA)=-ZMFC5(JLON,KTDIA)*(ZQN5(JLON)+ZLN5(JLON))
  ZCORQ(JLON,KTDIA)=-(ZQN5(JLON)+ZLN5(JLON))*ZMFC(JLON,KTDIA)&
   & -ZMFC5(JLON,KTDIA)*ZQN(JLON)-ZMFC5(JLON,KTDIA)*ZLN(JLON)  

  ZCORS5(JLON,KTDIA)=-ZMFC5(JLON,KTDIA)&
   & *(ZDSEN5(JLON)-ZLHSB5(JLON)*ZLN5(JLON))  

  ZCORS(JLON,KTDIA)=-(ZDSEN5(JLON)-ZLHSB5(JLON)*ZLN5(JLON))&
   & *ZMFC(JLON,KTDIA)&
   & -ZMFC5(JLON,KTDIA)*ZDSEN(JLON)&
   & +ZMFC5(JLON,KTDIA)*ZLHSB5(JLON)*ZLN(JLON)&
   & +ZMFC5(JLON,KTDIA)*ZLN5(JLON)*ZLHSB(JLON)  
ENDDO

! ----------------------------------------------------------------------
!     QUITTING IN CASE OF NO CONVECTION EVERYWHERE. THE "IF THEN /
!     ENDIF" ENCOMPASSED SEQUENCE WILL NOT BE INDENTED GIVEN ITS LENGTH
!     AND ITS SPECIAL CHARACTER (PSEUDO "RETURN" STATEMENT).

IF (ITOP < KLEV+1) THEN
! ----------------------------------------------------------------------

!*
!     ------------------------------------------------------------------
!     VII - SECOND VERTICAL LOOP (DOWNWARDS) TO SOLVE THE IMPLICIT
!     ALGORITHM FOR COMPENSATING SUBSIDENCE ADVECTION. ONE WILL AT THE
!     SAME TIME STORE PROVISIONAL FINAL VALUES FOR HUMIDITY, MOIST
!     STATIC ENERGY AND FOR THE TWO WIND COMPONENTS.
!     STRAIGHTFORWARD ELIMINATION-TYPE SOLUTION OF THE LINEAR SYSTEM
!     (BI-DIAGONAL MATRIX).

! - TEMPORARY(S) 2D (1:KLEV) .

! ZFQSC     : FINAL Q VALUE (AT FIRST ONLY COMPENSATING SUBSIDENCE).
! ZFSSC     : FINAL S VALUE (AT FIRST ONLY COMPENSATING SUBSIDENCE).
! ZFUSC     : FINAL U VALUE (AT FIRST ONLY COMPENSATING SUBSIDENCE).
! ZFVSC     : FINAL V VALUE (AT FIRST ONLY COMPENSATING SUBSIDENCE).

! The same but for trajectory
! ---------------------------
! ZFQSC5    : FINAL Q VALUE (AT FIRST ONLY COMPENSATING SUBSIDENCE).
! ZFSSC5    : FINAL S VALUE (AT FIRST ONLY COMPENSATING SUBSIDENCE).
! ZFUSC5    : FINAL U VALUE (AT FIRST ONLY COMPENSATING SUBSIDENCE).
! ZFVSC5    : FINAL V VALUE (AT FIRST ONLY COMPENSATING SUBSIDENCE).

! - TEMPORARY(S) 1D .

! ZDFMDS   :  HALF THE MASS FLUX TIME THE JUMP IN S.
! ZDFMDU   :  HALF THE MASS FLUX TIME THE JUMP IN U.
! ZDFMDV   :  HALF THE MASS FLUX TIME THE JUMP IN V.
! ZDFMDQ   :  HALF THE MASS FLUX TIME THE JUMP IN Q.

! The same but for trajectory
! ---------------------------
! ZDFMDS5  :  HALF THE MASS FLUX TIME THE JUMP IN S.
! ZDFMDU5  :  HALF THE MASS FLUX TIME THE JUMP IN U.
! ZDFMDV5  :  HALF THE MASS FLUX TIME THE JUMP IN V.
! ZDFMDQ5  :  HALF THE MASS FLUX TIME THE JUMP IN Q.

!     SPECIAL CASE OF THE FIRST LEVEL.

  DO JLON=KIDIA,KFDIA

    ZIV15=PDELP5(JLON,ITOP)/TSPHY
    ZIV1=PDELP(JLON,ITOP)/TSPHY

    ZIV5=1.0_JPRB/(ZIV15+ZMFC5(JLON,ITOP))
    ZIV=-ZIV1/(ZIV15+ZMFC5(JLON,ITOP))**2 &
     & -ZMFC(JLON,ITOP)/(ZIV15+ZMFC5(JLON,ITOP))**2  

    ZDFMDU5(JLON)=0.5_JPRB*ZMFC5(JLON,ITOP)*(PU5(JLON,ITOP+1)-PU5(JLON,ITOP))
    ZDFMDU(JLON)=0.5_JPRB*(PU5(JLON,ITOP+1)-PU5(JLON,ITOP))&
     & *ZMFC(JLON,ITOP)&
     & +0.5_JPRB*ZMFC5(JLON,ITOP)*PU(JLON,ITOP+1)&
     & -0.5_JPRB*ZMFC5(JLON,ITOP)*PU(JLON,ITOP)  

    ZDFMDV5(JLON)=0.5_JPRB*ZMFC5(JLON,ITOP)*(PV5(JLON,ITOP+1)-PV5(JLON,ITOP))
    ZDFMDV(JLON)=0.5_JPRB*(PV5(JLON,ITOP+1)-PV5(JLON,ITOP))&
     & *ZMFC(JLON,ITOP)&
     & +0.5_JPRB*ZMFC5(JLON,ITOP)*PV(JLON,ITOP+1)&
     & -0.5_JPRB*ZMFC5(JLON,ITOP)*PV(JLON,ITOP)  

    ZDFMDQ5(JLON)=0.5_JPRB*ZMFC5(JLON,ITOP)*(PQ5(JLON,ITOP+1)-PQ5(JLON,ITOP))
    ZDFMDQ(JLON)=0.5_JPRB*(PQ5(JLON,ITOP+1)-PQ5(JLON,ITOP))&
     & *ZMFC(JLON,ITOP)&
     & +0.5_JPRB*ZMFC5(JLON,ITOP)*PQ(JLON,ITOP+1)&
     & -0.5_JPRB*ZMFC5(JLON,ITOP)*PQ(JLON,ITOP)  

    ZDFMDS5(JLON)=0.5_JPRB*ZMFC5(JLON,ITOP)*(ZDSE5(JLON,ITOP+1)&
     & -ZDSE5(JLON,ITOP))  
    ZDFMDS(JLON)=0.5_JPRB*(ZDSE5(JLON,ITOP+1)-ZDSE5(JLON,ITOP))&
     & *ZMFC(JLON,ITOP)&
     & +0.5_JPRB*ZMFC5(JLON,ITOP)*ZDSE(JLON,ITOP+1)&
     & -0.5_JPRB*ZMFC5(JLON,ITOP)*ZDSE(JLON,ITOP)  

    ZFUSC5(JLON,ITOP)=(PU5(JLON,ITOP)*ZIV15-ZCORU5(JLON,ITOP)&
     & -ZDFMDU5(JLON))*ZIV5  
    ZFUSC(JLON,ITOP)=(PU5(JLON,ITOP)*ZIV15-ZCORU5(JLON,ITOP)&
     & -ZDFMDU5(JLON))*ZIV &
     & +ZIV5*PU5(JLON,ITOP)*ZIV1+ZIV5*ZIV15*PU(JLON,ITOP)&
     & -ZIV5*ZCORU(JLON,ITOP)-ZIV5*ZDFMDU(JLON)  

    ZFVSC5(JLON,ITOP)=(PV5(JLON,ITOP)*ZIV15-ZCORV5(JLON,ITOP)&
     & -ZDFMDV5(JLON))*ZIV5  
    ZFVSC(JLON,ITOP)=(PV5(JLON,ITOP)*ZIV15-ZCORV5(JLON,ITOP)&
     & -ZDFMDV5(JLON))*ZIV &
     & +ZIV5*PV5(JLON,ITOP)*ZIV1+ZIV5*ZIV15*PV(JLON,ITOP)&
     & -ZIV5*ZCORV(JLON,ITOP)-ZIV5*ZDFMDV(JLON)  

    ZFQSC5(JLON,ITOP)=(PQ5(JLON,ITOP)*ZIV15-ZCORQ5(JLON,ITOP)&
     & -ZDFMDQ5(JLON))*ZIV5  
    ZFQSC(JLON,ITOP)=(PQ5(JLON,ITOP)*ZIV15-ZCORQ5(JLON,ITOP)&
     & -ZDFMDQ5(JLON))*ZIV &
     & +ZIV5*PQ5(JLON,ITOP)*ZIV1+ZIV5*ZIV15*PQ(JLON,ITOP)&
     & -ZIV5*ZCORQ(JLON,ITOP)-ZIV5*ZDFMDQ(JLON)  

    ZFSSC5(JLON,ITOP)=(ZDSE5(JLON,ITOP)*ZIV15-ZCORS5(JLON,ITOP)&
     & -ZDFMDS5(JLON))*ZIV5  
    ZFSSC(JLON,ITOP)=(ZDSE5(JLON,ITOP)*ZIV15-ZCORS5(JLON,ITOP)&
     & -ZDFMDS5(JLON))*ZIV &
     & +ZIV5*ZDSE5(JLON,ITOP)*ZIV1+ZIV5*ZIV15*ZDSE(JLON,ITOP)&
     & -ZIV5*ZCORS(JLON,ITOP)-ZIV5*ZDFMDS(JLON)  
  ENDDO

!     VERTICAL LOOP WITH USING THE RESULTS FROM PREVIOUS UPPER LEVEL.

  DO JLEV=ITOP+1,KLEV-1
    DO JLON=KIDIA,KFDIA

      ZIV15=PDELP5(JLON,JLEV)/TSPHY
      ZIV1=PDELP(JLON,JLEV)/TSPHY

      ZIV5=1.0_JPRB/(ZIV15+ZMFC5(JLON,JLEV))
      ZIV=-ZIV1/(ZIV15+ZMFC5(JLON,JLEV))**2 &
       & -ZMFC(JLON,JLEV)/(ZIV15+ZMFC5(JLON,JLEV))**2  

      ZFMDU5=0.5_JPRB*ZMFC5(JLON,JLEV)*(PU5(JLON,JLEV+1)-PU5(JLON,JLEV))
      ZFMDU=0.5_JPRB*(PU5(JLON,JLEV+1)-PU5(JLON,JLEV))&
       & *ZMFC(JLON,JLEV)&
       & +0.5_JPRB*ZMFC5(JLON,JLEV)*PU(JLON,JLEV+1)&
       & -0.5_JPRB*ZMFC5(JLON,JLEV)*PU(JLON,JLEV)  

      ZFMDV5=0.5_JPRB*ZMFC5(JLON,JLEV)*(PV5(JLON,JLEV+1)-PV5(JLON,JLEV))
      ZFMDV=0.5_JPRB*(PV5(JLON,JLEV+1)-PV5(JLON,JLEV))&
       & *ZMFC(JLON,JLEV)&
       & +0.5_JPRB*ZMFC5(JLON,JLEV)*PV(JLON,JLEV+1)&
       & -0.5_JPRB*ZMFC5(JLON,JLEV)*PV(JLON,JLEV)  

      ZFMDQ5=0.5_JPRB*ZMFC5(JLON,JLEV)*(PQ5(JLON,JLEV+1)-PQ5(JLON,JLEV))
      ZFMDQ=0.5_JPRB*(PQ5(JLON,JLEV+1)-PQ5(JLON,JLEV))&
       & *ZMFC(JLON,JLEV)&
       & +0.5_JPRB*ZMFC5(JLON,JLEV)*PQ(JLON,JLEV+1)&
       & -0.5_JPRB*ZMFC5(JLON,JLEV)*PQ(JLON,JLEV)  

      ZFMDS5=0.5_JPRB*ZMFC5(JLON,JLEV)*(ZDSE5(JLON,JLEV+1)-ZDSE5(JLON,JLEV))
      ZFMDS=0.5_JPRB*(ZDSE5(JLON,JLEV+1)-ZDSE5(JLON,JLEV))&
       & *ZMFC(JLON,JLEV)&
       & +0.5_JPRB*ZMFC5(JLON,JLEV)*ZDSE(JLON,JLEV+1)&
       & -0.5_JPRB*ZMFC5(JLON,JLEV)*ZDSE(JLON,JLEV)  

      ZFUSC5(JLON,JLEV)=(PU5(JLON,JLEV)*ZIV15-ZCORU5(JLON,JLEV)&
       & +ZMFC5(JLON,JLEV-1)*ZFUSC5(JLON,JLEV-1)&
       & -(ZFMDU5-ZDFMDU5(JLON)))*ZIV5  
      ZFUSC(JLON,JLEV)=(PU5(JLON,JLEV)*ZIV15-ZCORU5(JLON,JLEV)&
       & +ZMFC5(JLON,JLEV-1)*ZFUSC5(JLON,JLEV-1)&
       & -(ZFMDU5-ZDFMDU5(JLON)))*ZIV &
       & +ZIV5*PU5(JLON,JLEV)*ZIV1 &
       & +ZIV5*ZIV15*PU(JLON,JLEV)-ZIV5*ZCORU(JLON,JLEV)&
       & +ZIV5*ZMFC5(JLON,JLEV-1)*ZFUSC(JLON,JLEV-1)&
       & +ZIV5*ZFUSC5(JLON,JLEV-1)*ZMFC(JLON,JLEV-1)&
       & -ZIV5*ZFMDU+ZIV5*ZDFMDU(JLON)  

      ZFVSC5(JLON,JLEV)=(PV5(JLON,JLEV)*ZIV15-ZCORV5(JLON,JLEV)&
       & +ZMFC5(JLON,JLEV-1)*ZFVSC5(JLON,JLEV-1)&
       & -(ZFMDV5-ZDFMDV5(JLON)))*ZIV5  
      ZFVSC(JLON,JLEV)=(PV5(JLON,JLEV)*ZIV15-ZCORV5(JLON,JLEV)&
       & +ZMFC5(JLON,JLEV-1)*ZFVSC5(JLON,JLEV-1)&
       & -(ZFMDV5-ZDFMDV5(JLON)))*ZIV &
       & +ZIV5*PV5(JLON,JLEV)*ZIV1 &
       & +ZIV5*ZIV15*PV(JLON,JLEV)-ZIV5*ZCORV(JLON,JLEV)&
       & +ZIV5*ZMFC5(JLON,JLEV-1)*ZFVSC(JLON,JLEV-1)&
       & +ZIV5*ZFVSC5(JLON,JLEV-1)*ZMFC(JLON,JLEV-1)&
       & -ZIV5*ZFMDV+ZIV5*ZDFMDV(JLON)  

      ZFQSC5(JLON,JLEV)=(PQ5(JLON,JLEV)*ZIV15-ZCORQ5(JLON,JLEV)&
       & +ZMFC5(JLON,JLEV-1)*ZFQSC5(JLON,JLEV-1)&
       & -(ZFMDQ5-ZDFMDQ5(JLON)))*ZIV5  
      ZFQSC(JLON,JLEV)=(PQ5(JLON,JLEV)*ZIV15-ZCORQ5(JLON,JLEV)&
       & +ZMFC5(JLON,JLEV-1)*ZFQSC5(JLON,JLEV-1)&
       & -(ZFMDQ5-ZDFMDQ5(JLON)))*ZIV &
       & +ZIV5*PQ5(JLON,JLEV)*ZIV1 &
       & +ZIV5*ZIV15*PQ(JLON,JLEV)-ZIV5*ZCORQ(JLON,JLEV)&
       & +ZIV5*ZMFC5(JLON,JLEV-1)*ZFQSC(JLON,JLEV-1)&
       & +ZIV5*ZFQSC5(JLON,JLEV-1)*ZMFC(JLON,JLEV-1)&
       & -ZIV5*ZFMDQ+ZIV5*ZDFMDQ(JLON)  

      ZFSSC5(JLON,JLEV)=(ZDSE5(JLON,JLEV)*ZIV15-ZCORS5(JLON,JLEV)&
       & +ZMFC5(JLON,JLEV-1)*ZFSSC5(JLON,JLEV-1)&
       & -(ZFMDS5-ZDFMDS5(JLON)))*ZIV5  
      ZFSSC(JLON,JLEV)=(ZDSE5(JLON,JLEV)*ZIV15-ZCORS5(JLON,JLEV)&
       & +ZMFC5(JLON,JLEV-1)*ZFSSC5(JLON,JLEV-1)&
       & -(ZFMDS5-ZDFMDS5(JLON)))*ZIV &
       & +ZIV5*ZDSE5(JLON,JLEV)*ZIV1 &
       & +ZIV5*ZIV15*ZDSE(JLON,JLEV)-ZIV5*ZCORS(JLON,JLEV)&
       & +ZIV5*ZMFC5(JLON,JLEV-1)*ZFSSC(JLON,JLEV-1)&
       & +ZIV5*ZFSSC5(JLON,JLEV-1)*ZMFC(JLON,JLEV-1)&
       & -ZIV5*ZFMDS+ZIV5*ZDFMDS(JLON)  

      ZDFMDU5(JLON)=ZFMDU5
      ZDFMDU(JLON)=ZFMDU
      ZDFMDV5(JLON)=ZFMDV5
      ZDFMDV(JLON)=ZFMDV
      ZDFMDQ5(JLON)=ZFMDQ5
      ZDFMDQ(JLON)=ZFMDQ
      ZDFMDS5(JLON)=ZFMDS5
      ZDFMDS(JLON)=ZFMDS

    ENDDO
  ENDDO

!     SPECIAL CASE OF THE LAST LEVEL.

  IF (ITOP /= KLEV) THEN
    DO JLON=KIDIA,KFDIA
      ZIV5=1.0_JPRB/(PDELP5(JLON,KLEV)/TSPHY)
      ZIV=-(TSPHY/PDELP5(JLON,KLEV)**2)*PDELP(JLON,KLEV)

      ZFUSC5(JLON,KLEV)=PU5(JLON,KLEV)+(ZMFC5(JLON,KLEV-1)&
       & *ZFUSC5(JLON,KLEV-1)-ZCORU5(JLON,KLEV)&
       & +ZDFMDU5(JLON))*ZIV5  
      ZFUSC(JLON,KLEV)=PU(JLON,KLEV)+(ZMFC5(JLON,KLEV-1)&
       & *ZFUSC5(JLON,KLEV-1)-ZCORU5(JLON,KLEV)&
       & +ZDFMDU5(JLON))*ZIV &
       & +ZIV5*ZMFC5(JLON,KLEV-1)*ZFUSC(JLON,KLEV-1)&
       & +ZIV5*ZFUSC5(JLON,KLEV-1)*ZMFC(JLON,KLEV-1)&
       & -ZIV5*ZCORU(JLON,KLEV)+ZIV5*ZDFMDU(JLON)  

      ZFVSC5(JLON,KLEV)=PV5(JLON,KLEV)+(ZMFC5(JLON,KLEV-1)&
       & *ZFVSC5(JLON,KLEV-1)-ZCORV5(JLON,KLEV)&
       & +ZDFMDV5(JLON))*ZIV5  
      ZFVSC(JLON,KLEV)=PV(JLON,KLEV)+(ZMFC5(JLON,KLEV-1)&
       & *ZFVSC5(JLON,KLEV-1)-ZCORV5(JLON,KLEV)&
       & +ZDFMDV5(JLON))*ZIV &
       & +ZIV5*ZMFC5(JLON,KLEV-1)*ZFVSC(JLON,KLEV-1)&
       & +ZIV5*ZFVSC5(JLON,KLEV-1)*ZMFC(JLON,KLEV-1)&
       & -ZIV5*ZCORV(JLON,KLEV)+ZIV5*ZDFMDV(JLON)  

      ZFQSC5(JLON,KLEV)=PQ5(JLON,KLEV)+(ZMFC5(JLON,KLEV-1)&
       & *ZFQSC5(JLON,KLEV-1)-ZCORQ5(JLON,KLEV)&
       & +ZDFMDQ5(JLON))*ZIV5  
      ZFQSC(JLON,KLEV)=PQ(JLON,KLEV)+(ZMFC5(JLON,KLEV-1)&
       & *ZFQSC5(JLON,KLEV-1)-ZCORQ5(JLON,KLEV)&
       & +ZDFMDQ5(JLON))*ZIV &
       & +ZIV5*ZMFC5(JLON,KLEV-1)*ZFQSC(JLON,KLEV-1)&
       & +ZIV5*ZFQSC5(JLON,KLEV-1)*ZMFC(JLON,KLEV-1)&
       & -ZIV5*ZCORQ(JLON,KLEV)+ZIV5*ZDFMDQ(JLON)  

      ZFSSC5(JLON,KLEV)=ZDSE5(JLON,KLEV)+(ZMFC5(JLON,KLEV-1)&
       & *ZFSSC5(JLON,KLEV-1)-ZCORS5(JLON,KLEV)&
       & +ZDFMDS5(JLON))*ZIV5  
      ZFSSC(JLON,KLEV)=ZDSE(JLON,KLEV)+(ZMFC5(JLON,KLEV-1)&
       & *ZFSSC5(JLON,KLEV-1)-ZCORS5(JLON,KLEV)&
       & +ZDFMDS5(JLON))*ZIV &
       & +ZIV5*ZMFC5(JLON,KLEV-1)*ZFSSC(JLON,KLEV-1)&
       & +ZIV5*ZFSSC5(JLON,KLEV-1)*ZMFC(JLON,KLEV-1)&
       & -ZIV5*ZCORS(JLON,KLEV)+ZIV5*ZDFMDS(JLON)  
    ENDDO
  ENDIF

!*
!     ------------------------------------------------------------------
!     VIII - FLUX COMPUTATION

!     DIFFUSIVE HUMIDITY FLUX AND INTERPOLATED LATENT HEAT OF FLUXES.

! - TEMPORARY(S) 2D (0:KLEV) .

! ZZLHP     : FICTITIOUS PRECIPITATION FLUX LATENT HEAT.

! The same but for trajectory
! ---------------------------
! ZZLHP5    : FICTITIOUS PRECIPITATION FLUX LATENT HEAT.

  DO JLON=KIDIA,KFDIA
    ZZLHP5(JLON,ITOP-1)=-ZLHL5(JLON,ITOP-1)-ZSNP5(JLON,ITOP-1)&
     & *(ZLHN5(JLON,ITOP-1)-ZLHL5(JLON,ITOP-1))  

    ZZLHP(JLON,ITOP-1)=(ZSNP5(JLON,ITOP-1)-1.0_JPRB)*ZLHL(JLON,ITOP-1)&
     & -(ZLHN5(JLON,ITOP-1)-ZLHL5(JLON,ITOP-1))*ZSNP(JLON,ITOP-1)&
     & -ZSNP5(JLON,ITOP-1)*ZLHN(JLON,ITOP-1)  
  ENDDO
  DO JLEV=ITOP,KLEV-1
    DO JLON=KIDIA,KFDIA
      PDIFCQ5(JLON,JLEV)=ZMFC5(JLON,JLEV)&
       & *0.5_JPRB*(ZFQSC5(JLON,JLEV)&
       & +ZFQSC5(JLON,JLEV+1)-ZQDN5(JLON,JLEV)&
       & -ZQDN5(JLON,JLEV+1))/RG  
      PDIFCQ(JLON,JLEV)=(0.5_JPRB/RG)&
       & *(ZFQSC5(JLON,JLEV)+ZFQSC5(JLON,JLEV+1)&
       & -ZQDN5(JLON,JLEV)-ZQDN5(JLON,JLEV+1))*ZMFC(JLON,JLEV)&
       & +(0.5_JPRB/RG)*ZMFC5(JLON,JLEV)&
       & *(ZFQSC(JLON,JLEV)+ZFQSC(JLON,JLEV+1)&
       & -ZQDN(JLON,JLEV)-ZQDN(JLON,JLEV+1))  

      ZZLHP5(JLON,JLEV)=-ZLHL5(JLON,JLEV)-ZSNP5(JLON,JLEV)&
       & *(ZLHN5(JLON,JLEV)-ZLHL5(JLON,JLEV))  
      ZZLHP(JLON,JLEV)=(ZSNP5(JLON,JLEV)-1.0_JPRB)*ZLHL(JLON,JLEV)&
       & -(ZLHN5(JLON,JLEV)-ZLHL5(JLON,JLEV))*ZSNP(JLON,JLEV)&
       & -ZSNP5(JLON,JLEV)*ZLHN(JLON,JLEV)  
    ENDDO
  ENDDO

  DO JLON=KIDIA,KFDIA
    ZZLHP5(JLON,KLEV)=ZLHSB5(JLON)
    ZZLHP(JLON,KLEV)=ZLHSB(JLON)
  ENDDO

!     FINAL COMPUTATION OF FLUXES WITH SECURITY AGAINST NEGATIVE
!     PRECIPITATION.

  DO JLEV=ITOP,KLEV
    DO JLON=KIDIA,KFDIA

!     DIFFUSIVE MOMENTUM FLUXES.

      PSTRCU5(JLON,JLEV)=PSTRCU5(JLON,JLEV-1)&
       & -ZPOID5(JLON,JLEV)*(ZFUSC5(JLON,JLEV)&
       & -PU5(JLON,JLEV))  
      PSTRCU(JLON,JLEV)=PSTRCU(JLON,JLEV-1)&
       & -(ZFUSC5(JLON,JLEV)-PU5(JLON,JLEV))*ZPOID(JLON,JLEV)&
       & -ZPOID5(JLON,JLEV)*ZFUSC(JLON,JLEV)&
       & +ZPOID5(JLON,JLEV)*PU(JLON,JLEV)  

      PSTRCV5(JLON,JLEV)=PSTRCV5(JLON,JLEV-1)&
       & -ZPOID5(JLON,JLEV)*(ZFVSC5(JLON,JLEV)&
       & -PV5(JLON,JLEV))  
      PSTRCV(JLON,JLEV)=PSTRCV(JLON,JLEV-1)&
       & -(ZFVSC5(JLON,JLEV)-PV5(JLON,JLEV))*ZPOID(JLON,JLEV)&
       & -ZPOID5(JLON,JLEV)*ZFVSC(JLON,JLEV)&
       & +ZPOID5(JLON,JLEV)*PV(JLON,JLEV)  

!     HUMIDITY FLUXES (TOTAL AND PRECIPITATING).

      ZFTOTQ5=PDIFCQ5(JLON,JLEV-1)+PFPLCL5(JLON,JLEV-1)&
       & +PFPLCN5(JLON,JLEV-1)&
       & -(ZPOID5(JLON,JLEV)*(ZFQSC5(JLON,JLEV)&
       & -PQ5(JLON,JLEV)) )  
      ZFTOTQ=PDIFCQ(JLON,JLEV-1)+PFPLCL(JLON,JLEV-1)&
       & +PFPLCN(JLON,JLEV-1)&
       & -(ZFQSC5(JLON,JLEV)-PQ5(JLON,JLEV))*ZPOID(JLON,JLEV)&
       & -ZPOID5(JLON,JLEV)*ZFQSC(JLON,JLEV)&
       & +ZPOID5(JLON,JLEV)*PQ(JLON,JLEV)  

      ZFCORQ5=MAX(0.0_JPRB,PDIFCQ5(JLON,JLEV)-ZFTOTQ5)
      IF ((PDIFCQ5(JLON,JLEV)-ZFTOTQ5) > 0.0_JPRB) THEN
        ZFCORQ=PDIFCQ(JLON,JLEV)-ZFTOTQ
      ELSE
        ZFCORQ=0.0_JPRB
      ENDIF

      PFPLCL5(JLON,JLEV)=(ZFCORQ5-(PDIFCQ5(JLON,JLEV)-ZFTOTQ5))&
       & *(1.0_JPRB-ZSNP5(JLON,JLEV))  
      PFPLCL(JLON,JLEV)=-(ZFCORQ5-PDIFCQ5(JLON,JLEV)+ZFTOTQ5)&
       & *ZSNP(JLON,JLEV)+(1.0_JPRB-ZSNP5(JLON,JLEV))&
       & *(ZFCORQ-PDIFCQ(JLON,JLEV)+ZFTOTQ)  

      PFPLCN5(JLON,JLEV)=(ZFCORQ5-(PDIFCQ5(JLON,JLEV)-ZFTOTQ5))&
       & *ZSNP5(JLON,JLEV)  
      PFPLCN(JLON,JLEV)=(ZFCORQ5-PDIFCQ5(JLON,JLEV)+ZFTOTQ5)&
       & *ZSNP(JLON,JLEV)+ZSNP5(JLON,JLEV)&
       & *(ZFCORQ-PDIFCQ(JLON,JLEV)+ZFTOTQ)  

!     ENTHALPY FLUXES (TOTAL AND DIFFUSIVE).

      ZFTOTS5=PDIFCS5(JLON,JLEV-1)-ZZLHP5(JLON,JLEV-1)&
       & *(PFPLCL5(JLON,JLEV-1)+PFPLCN5(JLON,JLEV-1))&
       & -ZPOID5(JLON,JLEV)*(ZFSSC5(JLON,JLEV)-ZDSE5(JLON,JLEV))  
      ZFTOTS=PDIFCS(JLON,JLEV-1)&
       & -(PFPLCL5(JLON,JLEV-1)+PFPLCN5(JLON,JLEV-1))&
       & *ZZLHP(JLON,JLEV-1)&
       & -ZZLHP5(JLON,JLEV-1)*PFPLCL(JLON,JLEV-1)&
       & -ZZLHP5(JLON,JLEV-1)*PFPLCN(JLON,JLEV-1)&
       & -(ZFSSC5(JLON,JLEV)-ZDSE5(JLON,JLEV))*ZPOID(JLON,JLEV)&
       & -ZPOID5(JLON,JLEV)*ZFSSC(JLON,JLEV)&
       & +ZPOID5(JLON,JLEV)*ZDSE(JLON,JLEV)  

      PDIFCS5(JLON,JLEV)=ZFTOTS5-ZLHSB5(JLON)*ZFCORQ5 &
       & +ZZLHP5(JLON,JLEV)&
       & *(PFPLCL5(JLON,JLEV)+PFPLCN5(JLON,JLEV))  
      PDIFCS(JLON,JLEV)=ZFTOTS-ZLHSB5(JLON)*ZFCORQ &
       & -ZFCORQ5*ZLHSB(JLON)&
       & +(PFPLCL5(JLON,JLEV)+PFPLCN5(JLON,JLEV))&
       & *ZZLHP(JLON,JLEV)&
       & +ZZLHP5(JLON,JLEV)*PFPLCL(JLON,JLEV)&
       & +ZZLHP5(JLON,JLEV)*PFPLCN(JLON,JLEV)  
    ENDDO
  ENDDO

!     SETTING ALL RESULTS TO ZERO IN CASE OF NEGATIVE PRECIPITATIONS AT
!     THE SURFACE .

  DO JLON=KIDIA,KFDIA
    ZZVAL=PFPLCL5(JLON,KLEV)+PFPLCN5(JLON,KLEV)-SCO
    I_KNND(JLON)=I_KNND(JLON)*MAX(0.0_JPRB,-SIGN(1.0_JPRB,0.0_JPRB-ZZVAL))
  ENDDO
  DO JLEV=ITOP,KLEV
    DO JLON=KIDIA,KFDIA
      PFPLCL5(JLON,JLEV)=I_KNND(JLON)*PFPLCL5(JLON,JLEV)
      PFPLCL(JLON,JLEV)=I_KNND(JLON)*PFPLCL(JLON,JLEV)

      PFPLCN5(JLON,JLEV)=I_KNND(JLON)*PFPLCN5(JLON,JLEV)
      PFPLCN(JLON,JLEV)=I_KNND(JLON)*PFPLCN(JLON,JLEV)

      PDIFCQ5(JLON,JLEV)=I_KNND(JLON)*PDIFCQ5(JLON,JLEV)
      PDIFCQ(JLON,JLEV)=I_KNND(JLON)*PDIFCQ(JLON,JLEV)

      PDIFCS5(JLON,JLEV)=I_KNND(JLON)*PDIFCS5(JLON,JLEV)
      PDIFCS(JLON,JLEV)=I_KNND(JLON)*PDIFCS(JLON,JLEV)

      PSTRCU5(JLON,JLEV)=I_KNND(JLON)*PSTRCU5(JLON,JLEV)
      PSTRCU(JLON,JLEV)=I_KNND(JLON)*PSTRCU(JLON,JLEV)

      PSTRCV5(JLON,JLEV)=I_KNND(JLON)*PSTRCV5(JLON,JLEV)
      PSTRCV(JLON,JLEV)=I_KNND(JLON)*PSTRCV(JLON,JLEV)
    ENDDO
  ENDDO

!*
!     RETURN FROM THE PSEUDO-RETURN SITUATED AFTER LOOP 670.

! -------------
ENDIF
! -------------

IF (LHOOK) CALL DR_HOOK('ACCONVTL',1,ZHOOK_HANDLE)
END SUBROUTINE ACCONVTL
