!OPTIONS XOPT(NOEVAL)
SUBROUTINE ACTSECAD ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 !-----------------------------------------------------------------------
 ! - INPUT  2D .
 & PAPHI5,PAPHIF5,PAPRS5,PAPRSF5,PQ5,PT5,PU5,PV5,&
 & PAPHI,PAPHIF,PAPRS,PAPRSF,PQ,PT,PU,PV,&
 ! - INPUT  1D .
 & PQS,PTS,PGZ0,&
 ! - OUTPUT 2D .
 & PKTROV5,PKUROV5,PXTROV5,PXUROV5,&
 & PKTROV,PKUROV,PXTROV,PXUROV,&
 ! - OUTPUT 1D .
 & PCDROV5,PCHROV5,PXDROV5,PXHROV5,&
 & PCDROV,PCHROV,PXDROV,PXHROV,&
 ! - LOCK ARGUMENTS .
 & CDLOCK)  

!**** *ACTSECAD* - ADJOINT COMPUTATION OF VERTICAL TURBULENT
!                  AND SURFACE EXCHANGE COEFFICIENTS.

!     Subject.
!     --------
!     Adjoint computation of:
!     - COMPUTATION OF VERTICAL TURBULENT EXCHANGE COEFFICIENTS
!       (DIMENSION (DP/(G*DT)) AND OF STATIC STABILITY (DIMENSION
!       (U/DP)**2), AS WELL AS OF THE "ANTI-FIBRILLATION" AMPLIFICATION
!       FACTORS OF THE FORMERS .
!     - COMPUTATION OF SURFACE EXCHANGE COEFFICIENTS PLUS THEIR
!       AMPLIFICATION FACTOR IF "ANTI-FIBRILLATION" IS ACTIVATED .

!**   Interface.
!     ----------
!        *CALL* *ACTSECAD*

!-----------------------------------------------------------------------

! -   ARGUMENTS FOR INPUT
!     -------------------

! - DIMENSIONAL PARAMETERS OF PHYSICS.

! KIDIA      : START OF HORIZONTAL LOOP (IST in CPG)
! KFDIA      : END OF HORIZONTAL LOOP.
! KLON       : HORIZONTAL DIMENSION.
! KTDIA      : START OF THE VERTICAL LOOP IN THE PHYSICS (1 IN GENERAL)
! KLEV       : END OF VERTICAL LOOP IN THE PHYSICS (1 IN GENERAL)

! - THE NAME OF THE PHYSICAL VARIABLES.

! PAPHI      : GEOPOTENTIAL ON HALF LEVELS.
! PAPRS      : PRESSURE ON HALF LEVELS.

! The same but for trajectory
! ---------------------------
! PAPHI5     : GEOPOTENTIAL ON HALF LEVELS.
! PAPRS5     : PRESSURE ON HALF LEVELS.

! - 2D (1:KLEV) .

! PAPHIF     : GEOPOTENTIAL ON FULL LEVELS.
! PQ         : SPECIFIC HUMIDITY OF WATER VAPOR.
! PT         : TEMPERATURE.
! PU         : X - COMPONENT OF WIND.
! PV         : Y - COMPONENT OF WIND.

! The same but for trajectory
! ---------------------------
! PAPHIF5    : GEOPOTENTIAL ON FULL LEVELS.
! PQ5        : SPECIFIC HUMIDITY OF WATER VAPOR.
! PT5        : TEMPERATURE.
! PU5        : X - COMPONENT OF WIND.
! PV5        : Y - COMPONENT OF WIND.

! - 1D (PROGNOSTIC) .

! PTP        : DEEP LAYER TEMPERATURE.
!     (KCSS)
! PTS        : SURFACE TEMPERATURE.
! PQS        : SURFACE SPECIFIC HUMIDITY

! - 1D (GEOGRAPHIC) .

! PGZ0       : GRAVITY * ROUGHNESS LENGTH.

!-----------------------------------------------------------------------

! -   ARGUMENTS FOR OUTPUT.
!     ---------------------

! - 2D (0:KLEV) .

! PKTROV     : VERTICAL EXCHANGE COEFFICIENT OF T AND Q IN KG/(M*M*S).
! PKUROV     : VERTICAL EXCHANGE COEFFICIENT OF U AND V IN KG/(M*M*S).
! PXTROV     : "ANTI-FIBRILLATION" MULTIPLICATOR OF PKTROV.
! PXUROV     : "ANTI-FIBRILLATION" MULTIPLICATOR OF PKUROV.

! The same but for trajectory
! ---------------------------
! PKTROV5    : VERTICAL EXCHANGE COEFFICIENT OF T AND Q IN KG/(M*M*S).
! PKUROV5    : VERTICAL EXCHANGE COEFFICIENT OF U AND V IN KG/(M*M*S).
! PXTROV5    : "ANTI-FIBRILLATION" MULTIPLICATOR OF PKTROV.
! PXUROV5    : "ANTI-FIBRILLATION" MULTIPLICATOR OF PKUROV.

! - 1D (DIAGNOSTIC) .

! PCDROV     : PCD RENORME EN DENSITE FOIS VITESSE.
! PCHROV     : PCH RENORME EN DENSITE FOIS VITESSE.
! PXDROV     : "ANTI-FIBRILLATION" MULTIPLICATOR OF PCDROV.
! PXHROV     : "ANTI-FIBRILLATION" MULTIPLICATOR OF PCHROV.

! The same but for trajectory
! ---------------------------
! PCDROV5    : PCD RENORME EN DENSITE FOIS VITESSE.
! PCHROV5    : PCH RENORME EN DENSITE FOIS VITESSE.
! PXDROV5    : "ANTI-FIBRILLATION" MULTIPLICATOR OF PCDROV.
! PXHROV5    : "ANTI-FIBRILLATION" MULTIPLICATOR OF PCHROV.

!-----------------------------------------------------------------------

! -   IMPLICIT ARGUMENTS.
!     -------------------

! COMMON/YOMPHY /
! COMMON/YOMCST /
! COMMON/YOMPHY0/
! COMMON/YOMPHY1/
! COMMON/FCTTRM /
! COMMON/FCTTRMAD/

!-----------------------------------------------------------------------

!     Externals.
!     ----------

!     Methode.
!     --------

!     Authors
!     -------
!        95-10, M.Janiskova

!     Modifications.
!     --------------
!        97-08, introduce CDLOCK - J.M. Piriou.
!        97-11, modified smoothing - M. Janiskova
!        98-12, adopted modifications for anti-fibrilation
!               and shallow convection - M. Janiskova
!        99-01, code optimization - M. Janiskova
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        03-12, introduction of USURID,USURIDE,USURICE et USURICL - C Loo
!        03-12, remove of "Girard-Delage" anti-fibrillation - C Loo
!        04-06, introduction of EDK for Fm and Fh in stable case - F Bouyssel
!        07-01, optimisation liee a LSMOOTHA - F Bouyssel - C Loo
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY   , ONLY : LCVPP    ,LNEIGE
USE YOMCST   , ONLY : RG       ,RD       ,RV       ,RCPD     ,&
 & RCPV     ,RKAPPA   ,RETV     ,RCW      ,RCS      ,&
 & RLVTT    ,RLSTT    ,RTT      ,RALPW    ,RBETW    ,&
 & RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,RALPD    ,&
 & RBETD    ,RGAMD  
USE YOMPHY0  , ONLY : ALMAV    ,BEDIFV   ,EDB      ,EDC      ,&
 & EDD      ,UHDIFV   ,USURIC   ,VKARMN   ,GRCVPP   ,&
 & USURICL  ,USURICE  ,USURIDE  ,USURID   ,EDK
USE YOMSIMPHL, ONLY : LSMOOTHA

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHIF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHI(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGZ0(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKTROV5(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKUROV5(KLON,0:KLEV) 
REAL(KIND=JPRB)                  :: PXTROV5(KLON,0:KLEV) ! Argument NOT used
REAL(KIND=JPRB)                  :: PXUROV5(KLON,0:KLEV) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PKTROV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PKUROV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXTROV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXUROV(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCDROV5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCHROV5(KLON) 
REAL(KIND=JPRB)                  :: PXDROV5(KLON) ! Argument NOT used
REAL(KIND=JPRB)                  :: PXHROV5(KLON) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCDROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCHROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXDROV(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXHROV(KLON) 
CHARACTER(LEN=7)  ,INTENT(IN)    :: CDLOCK 
REAL(KIND=JPRB) :: ZLSCPE(KLON,KLEV),ZQSAT(KLON,KLEV),ZLH(KLON,KLEV),ZR(KLON,KLEV)
REAL(KIND=JPRB) :: ZCDNH(KLON),ZDPHI(KLON),ZRTI(KLON),ZSTA(KLON)&
 & ,ZU(KLON),ZRS(KLON),ZSTAB(KLON)&
 & ,ZCP(KLON),ZPCD(KLON),ZPCDN(KLON),ZPCH(KLON),ZSTAC(KLON)&
 & ,ZSTAH(KLON),ZSTACH(KLON)  

REAL(KIND=JPRB) :: ZLOS5(KLON,KLEV,2),ZLOI5(KLON,KLEV,2)
REAL(KIND=JPRB) :: ZLSCPE5(KLON,KLEV),ZQSAT5(KLON,KLEV),ZLH5(KLON,KLEV)&
 & ,ZR5(KLON,KLEV),ZEW15(KLON,KLEV),ZESP15(KLON,KLEV)&
 & ,ZCP5(KLON,KLEV),ZDPHI5(KLON,KLEV),ZRTI5(KLON,KLEV)&
 & ,ZSTA5(KLON,KLEV),ZU5(KLON,KLEV),ZCIS5(KLON,KLEV)&
 & ,ZDS5(KLON,KLEV)&
 & ,ZZ5(KLON,KLEV),ZPFD5(KLON,KLEV),ZGLMU25(KLON,KLEV)&
 & ,ZGLMT25(KLON,KLEV),ZCK5(KLON,KLEV),ZDTETA5(KLON,KLEV)&
 & ,ZAUX5(KLON,KLEV),ZZC5(KLON,KLEV),ZZD5(KLON,KLEV)&
 & ,ZSTAC5(KLON,KLEV),ZSTAO5(KLON,KLEV),ZIXP5(KLON,KLEV)&
 & ,ZUSURID5(KLON,KLEV),ZUSURIC5(KLON,KLEV),ZPFH5(KLON,KLEV)&
 & ,ZZT5(KLON,KLEV),ZHS5(KLON,KLEV)  
REAL(KIND=JPRB) :: ZCDNH5(KLON),ZSTAB5(KLON),ZPCD5(KLON),ZPCDN5(KLON)&
 & ,ZPCH5(KLON),ZRZD5(KLON),ZRZH5(KLON),ZCD5(KLON)&
 & ,ZCH5(KLON),ZDID5(KLON),ZDIH5(KLON)&
 & ,ZDIDP5(KLON),ZDIHP5(KLON),ZSTAH5(KLON,KLEV),ZSTACH5(KLON,KLEV)  

INTEGER(KIND=JPIM) :: JLEV, JLON

REAL(KIND=JPRB) :: Z2B, Z3B, Z3BC, Z3BCF, ZAUX, ZCD, ZCH, ZCIS, ZCK, &
 & ZDELTA, ZDID, ZDIDP, ZDIFV, ZDIH, ZDIHP, ZDS, ZDTETA, &
 & ZEPS1, ZESP1, ZEW1, ZGLMT2, ZGLMU2, ZGLT, ZGLU, ZIS, &
 & ZLOI, ZLOS, ZPFD, ZRCVPP, ZRLTLU, ZRVMD, ZRZD, ZRZH, &
 & ZXX, ZZ, ZZB, ZZC, ZZD, ZZZPF, &
 & ZEDIFV, ZDIFT, ZZUSURIC, ZZIXP, ZHS, ZZT, &
 & ZPFH, ZUSURIC, ZUSURID, ZIXP, ZZZPFH, &
 & ZTERM1, ZTERM2, ZTERM3, ZTERM4, ZTERM5, ZTERM6, &
 & ZTERM7, ZTERM8, ZTERM9   
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

#include "fcttrm.h"
#include "fcttrmad.h"

!     CHECK RELIABILITY OF INPUT ARGUMENTS.

IF (LHOOK) CALL DR_HOOK('ACTSECAD',0,ZHOOK_HANDLE)
IF(CDLOCK /= 'TSECAD ') CALL ABOR1('BAD NUMBER OF ARGUMENTS!')

!*
!     ----------------------------------------------------------------
IF (LSMOOTHA) THEN
!     ----------------------------------------------------------------

DO JLEV=KTDIA,KLEV-1
  DO JLON=KIDIA,KFDIA
    PKTROV(JLON,JLEV)=0.0_JPRB
    PKUROV(JLON,JLEV)=0.0_JPRB
    PXTROV(JLON,JLEV)=0.0_JPRB
    PXUROV(JLON,JLEV)=0.0_JPRB
  ENDDO
ENDDO

DO JLON=KIDIA,KFDIA
  PCHROV(JLON)=0.0_JPRB
  PCDROV(JLON)=0.0_JPRB
  PXHROV(JLON)=0.0_JPRB
  PXDROV(JLON)=0.0_JPRB
ENDDO

!     ----------------------------------------------------------------
ELSE
!     ----------------------------------------------------------------

!     ------------------------------------------------------------------
!     I - AUXILIARY CONSTANTS.

ZRVMD=RV-RD

!*
!     ------------------------------------------------------------------
!     II - COMPUTATION OF DERIVED PARAMETERS AND SECURITY CONSTANT (FOR
!     THE SQUARE OF THE WIND SHEAR).

Z2B=2.0_JPRB*EDB
Z3B=3._JPRB*EDB
Z3BC=3._JPRB*EDB*EDC
Z3BCF=EDB*EDC*VKARMN**2/SQRT(3._JPRB)
ZEDIFV=BEDIFV/SQRT(3._JPRB)
ZRLTLU=1.5_JPRB*EDD
ZDIFV=UHDIFV/(RG*VKARMN)
ZDIFT=ZDIFV/SQRT(3._JPRB)
ZRCVPP=1.0_JPRB/GRCVPP
ZZUSURIC=USURIC*USURICL

IF(USURID == 0.0_JPRB) THEN
  ZEDIFV=BEDIFV
  ZZIXP=1.0_JPRB
ELSE
  ZZIXP=3._JPRB
ENDIF

ZEPS1=1.E-04_JPRB

!*
!     ----------------------------------------------------------------
!     0. 3 - Computation of specific humidity of saturation and
!     effective ratio of L and Cp for condensation/evaporation
!     Computation of gass constant for air

! For trajectory
! ---------------
! - TEMPORARY(S) 2D (1:KLEV) .

! ZLH5      : LATENT HEAT
! ZQSAT5    : SPECIFIC HUMIDITY OF SATURATION
! ZLSCPE5   : EFFECTIVE RATIO OF L AND Cp FOR CONDENSATION/EVAPORATION
! ZR5      : GASS CONSTANT FOR AIR

! - TEMPORARY 2D

! ZCP5      : VALUE OF ATMOSPHERIC Cp

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA

    ZCP5(JLON,JLEV)=RCPD*(1.0_JPRB-PQ5(JLON,JLEV))+RCPV*PQ5(JLON,JLEV)
    ZR5(JLON,JLEV)=RD*(1.0_JPRB-PQ5(JLON,JLEV))+RV*PQ5(JLON,JLEV)

!     SNOW OPTION DEPENDENT COMPUTATIONS.

    IF (LNEIGE) THEN
      ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PT5(JLON,JLEV)))
    ELSE
      ZDELTA=0.0_JPRB
    ENDIF

!     Call to the defined function to compute Lh.

    ZLH5(JLON,JLEV)= FOLH (PT5(JLON,JLEV),ZDELTA)

!     Computation of effective ratio of L and Cp for cond./evap.

    ZLSCPE5(JLON,JLEV)=ZLH5(JLON,JLEV)/ZCP5(JLON,JLEV)

!     Computation of Qsat(PT)

    ZEW15(JLON,JLEV)= FOEW (PT5(JLON,JLEV),ZDELTA)
    ZESP15(JLON,JLEV)=ZEW15(JLON,JLEV)/PAPRSF5(JLON,JLEV)
    ZQSAT5(JLON,JLEV)= FOQS (ZESP15(JLON,JLEV))

  ENDDO
ENDDO

!*
!     ------------------------------------------------------------------
!     0. 4 - Computation of surface exchange coefficients plus their
!     amplification factor if "anti-fibrillation" is activated .

!*
!          Computation of the thickness of the SBL (in geopotential).

! For trajectory
! --------------
! - TEMPORARY(S) 2D .

! ZDPHI5    : GEOPOTENTIAL THICKNESS OF THE SURFACE LEVEL.

DO JLON=KIDIA,KFDIA
  ZDPHI5(JLON,KLEV)=PAPHIF5(JLON,KLEV)-PAPHI5(JLON,KLEV)
ENDDO

!*
!          Effective calculations of the surface characteristics.

! - TEMPORARY(S) 1D .

! ZRS       : GASS CONSTANT FOR AIR AT THE SURFACE.

! For the trajectory
! -------------------
! ZPCDN5    : EXCHANGE COEF. AT SURFACE LEVEL IN NEUTRAL CONDITION.

DO JLON=KIDIA,KFDIA

  ZRS(JLON)=RD+ZRVMD*PQS(JLON)

!     Geometric calculations.

  ZRZD5(JLON)=1.0_JPRB+ZDPHI5(JLON,KLEV)/PGZ0(JLON)
  ZPCDN5(JLON)=(VKARMN/LOG(ZRZD5(JLON)))**2

!     Preliminary computations necessary for the determination of
!     exchange coefficients Cd and Ch

! For trajectory
! --------------
! ZCDNH5    : NEUTRAL SURFACE THERMAL EXCHANGE COEFFICIENT.

  ZRZH5(JLON)=ZRZD5(JLON)
  ZCDNH5(JLON)=ZPCDN5(JLON)

  ZCD5(JLON)=1.0_JPRB
  ZCH5(JLON)=1.0_JPRB

!     Wind shear.

! For trajectory
! ---------------

! - TEMPORARY(S) 1D .
! ZPCD5     : EXCHANGE COEFFICIENT AT SURFACE LEVEL FOR U AND V.

! - TEMPORARY 2D
! ZU5    : SURFACE WIND SPEED.

  ZCIS5(JLON,KLEV)=MAX(ZEPS1,PU5(JLON,KLEV)**2+PV5(JLON,KLEV)**2)
  ZU5(JLON,KLEV)=SQRT(ZCIS5(JLON,KLEV))

!     Stability computation.

! For trajectory
! ---------------
! - TEMPORARY(S) 2D.

! ZRTI5    : INVERSE OF R*T.
! ZSTA5     : APPROXIMATION OF DELTA(PHI)*DELTA(LN(THETA)).

  ZRTI5(JLON,KLEV)=2.0_JPRB/(ZR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON,KLEV)+ZRS(JLON)*PTS(JLON))  

  ZSTAO5(JLON,KLEV)=ZDPHI5(JLON,KLEV)*(ZR5(JLON,KLEV)&
   & *PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON,KLEV)-ZRS(JLON)*PTS(JLON))&
   & *ZRTI5(JLON,KLEV)  
  ZSTAH5(JLON,KLEV)=ZSTAO5(JLON,KLEV)/(1.0_JPRB+ZZIXP*ZZUSURIC&
   & *MAX(0.0_JPRB,ZSTAO5(JLON,KLEV))/ZCIS5(JLON,KLEV))**(1.0_JPRB/ZZIXP)  
  ZSTA5(JLON,KLEV)=ZSTAO5(JLON,KLEV)/(1.0_JPRB+MAX(0.0_JPRB,ZSTAO5(JLON,KLEV))&
   & *ZZUSURIC/ZCIS5(JLON,KLEV))  
  ZSTAB5(JLON)=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZSTA5(JLON,KLEV)))

  ZSTAC5(JLON,KLEV)=ABS(ZSTA5(JLON,KLEV))
  ZSTACH5(JLON,KLEV)=ABS(ZSTAH5(JLON,KLEV))

!     Common computations for momentum and energy.

  ZDS5(JLON,KLEV)=SQRT(ZCIS5(JLON,KLEV)+(EDD/EDK)*ZSTAC5(JLON,KLEV))
  ZHS5(JLON,KLEV)=SQRT(ZCIS5(JLON,KLEV)+EDD*EDK*ZSTACH5(JLON,KLEV))

  ZDIDP5(JLON)=ZU5(JLON,KLEV)+ZCD5(JLON)*Z3BC*ZPCDN5(JLON)&
   & *SQRT(ZSTAC5(JLON,KLEV)*ZRZD5(JLON))  
  ZDID5(JLON)=1.0_JPRB/ZDIDP5(JLON)

  ZDIHP5(JLON)=ZU5(JLON,KLEV)+ZCH5(JLON)*Z3BC*ZCDNH5(JLON)&
   & *SQRT(ZSTAC5(JLON,KLEV)*ZRZH5(JLON))  
  ZDIH5(JLON)=1.0_JPRB/ZDIHP5(JLON)

!     Computations for momentum of Cd and of its product by density
!     time wind speed.

  ZLOS5(JLON,KLEV,1)=ZCIS5(JLON,KLEV)*ZDS5(JLON,KLEV)&
   & /(ZU5(JLON,KLEV)*ZDS5(JLON,KLEV)&
   & +Z2B*ZSTAC5(JLON,KLEV))  
  ZLOI5(JLON,KLEV,1)=ZU5(JLON,KLEV)-Z2B*ZSTA5(JLON,KLEV)*ZDID5(JLON)

  ZPCD5(JLON)=(ZLOI5(JLON,KLEV,1)&
   & +ZSTAB5(JLON)*(ZLOS5(JLON,KLEV,1)-ZLOI5(JLON,KLEV,1)))&
   & *ZPCDN5(JLON)/ZU5(JLON,KLEV)  

!     The same for tempereture and humidity.

  ZLOS5(JLON,KLEV,2)=ZCIS5(JLON,KLEV)**2/(ZU5(JLON,KLEV)&
   & *ZCIS5(JLON,KLEV)&
   & +Z3B*ZSTACH5(JLON,KLEV)*ZHS5(JLON,KLEV))  
  ZLOI5(JLON,KLEV,2)=ZU5(JLON,KLEV)-Z3B*ZSTA5(JLON,KLEV)*ZDIH5(JLON)

  ZPCH5(JLON)=(ZLOI5(JLON,KLEV,2)&
   & +ZSTAB5(JLON)*(ZLOS5(JLON,KLEV,2)-ZLOI5(JLON,KLEV,2)))&
   & *ZCDNH5(JLON)/ZU5(JLON,KLEV)  

ENDDO

!     Computation of the exchange coefficients multiplied by density
!     and wind speed.

DO JLON=KIDIA,KFDIA
  PCDROV5(JLON)=ZPCD5(JLON)*ZU5(JLON,KLEV)*PAPRS5(JLON,KLEV)*ZRTI5(JLON,KLEV)
  PCHROV5(JLON)=ZPCH5(JLON)*ZU5(JLON,KLEV)*PAPRS5(JLON,KLEV)*ZRTI5(JLON,KLEV)
ENDDO

!*
!     ------------------------------------------------------------------
!     0. 5 - Computation of vertical exchange coefficients,
!     as well as of the "anti-fibrillation" amplification
!     factors of the formers .

!*
!          Going to geopotential for asymptotic mixing lengths.

ZGLU=RG*ALMAV
ZGLT=ZGLU*ZRLTLU

!*
!           Passive loop on vertical levels.

DO JLEV=KTDIA,KLEV-1

!*
!          Effective calculations.

  DO JLON=KIDIA,KFDIA

!     Geometric computations, the multiplying factor ZPFD (potentially
!     different of 1) for the diffusion's vertical profile depends upon
!     the choice of UHDIFV and BEDIFV.

    ZZ5(JLON,JLEV)=VKARMN*(0.5_JPRB*(PAPHIF5(JLON,JLEV)&
     & +PAPHIF5(JLON,JLEV+1))-PAPHI5(JLON,KLEV)+PGZ0(JLON))  
    ZZT5(JLON,JLEV)=VKARMN*(0.5_JPRB*(PAPHIF5(JLON,JLEV)&
     & +PAPHIF5(JLON,JLEV+1))-PAPHI5(JLON,KLEV)+PGZ0(JLON))  
    ZPFD5(JLON,JLEV)=MAX(0.0_JPRB,BEDIFV+(1.0_JPRB-BEDIFV)&
     & /(1.0_JPRB+(ZDIFV*ZZ5(JLON,JLEV))**2))  
    ZPFH5(JLON,JLEV)=MAX(0.0_JPRB,ZEDIFV+(1.0_JPRB-ZEDIFV)&
     & /(1.0_JPRB+(ZDIFT*ZZT5(JLON,JLEV))**2))  

    ZGLMU25(JLON,JLEV)=(ZZ5(JLON,JLEV)*ZGLU*ZPFD5(JLON,JLEV)&
     & /(ZGLU+ZZ5(JLON,JLEV)))**2  
    ZGLMT25(JLON,JLEV)=(ZZT5(JLON,JLEV)*ZGLT*ZPFH5(JLON,JLEV)&
     & /(ZGLT+ZZT5(JLON,JLEV)))&
     & *(ZZ5(JLON,JLEV)*ZGLU*ZPFD5(JLON,JLEV)/(ZGLU+ZZ5(JLON,JLEV)))  

    ZCK5(JLON,JLEV)=Z3BCF*(ZGLT*ZPFH5(JLON,JLEV)/(ZGLT+ZZT5(JLON,JLEV)))&
     & *(ZGLU*ZPFD5(JLON,JLEV)/(ZGLU+ZZ5(JLON,JLEV)))  
    ZDPHI5(JLON,JLEV)=PAPHIF5(JLON,JLEV)-PAPHIF5(JLON,JLEV+1)

!     Wind shear.

    ZCIS5(JLON,JLEV)=MAX(ZEPS1,&
     & (PU5(JLON,JLEV)-PU5(JLON,JLEV+1))**2 &
     & +(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))**2)  
    ZU5(JLON,JLEV)=SQRT(ZCIS5(JLON,JLEV))

!     Preliminary stability computation.

    ZDTETA5(JLON,JLEV)=ZR5(JLON,JLEV)*PT5(JLON,JLEV)&
     & -ZR5(JLON,JLEV+1)*PT5(JLON,JLEV+1)&
     & +RKAPPA*ZDPHI5(JLON,JLEV)  

!     Correction for shallow convection (optional).

    IF (LCVPP) THEN
      ZAUX5(JLON,JLEV)=ZRCVPP*MIN(0.0_JPRB,PQ5(JLON,JLEV)&
       & -ZQSAT5(JLON,JLEV)-PQ5(JLON,JLEV+1)&
       & +ZQSAT5(JLON,JLEV+1))*(0.5_JPRB*(ZLSCPE5(JLON,JLEV)&
       & +ZLSCPE5(JLON,JLEV+1)))*MAX(0.0_JPRB,SIGN(1.0_JPRB,RD &
       & *(PQ5(JLON,JLEV+1)-PQ5(JLON,JLEV))*(0.5_JPRB*(ZLSCPE5(JLON,JLEV)&
       & +ZLSCPE5(JLON,JLEV+1)))-ZDTETA5(JLON,JLEV)))  

      ZZC5(JLON,JLEV)=PQ5(JLON,JLEV)-ZQSAT5(JLON,JLEV)&
       & -PQ5(JLON,JLEV+1)+ZQSAT5(JLON,JLEV+1)  
      IF (ZZC5(JLON,JLEV) < 0) THEN
        ZZD5(JLON,JLEV)=ZZC5(JLON,JLEV)
      ELSE
        ZZD5(JLON,JLEV)=0.0_JPRB
      ENDIF

    ELSE
      ZAUX5(JLON,JLEV)=0.0_JPRB
    ENDIF

!     Stability computation.

    ZRTI5(JLON,JLEV)=2.0_JPRB/(ZR5(JLON,JLEV)*PT5(JLON,JLEV)&
     & +ZR5(JLON,JLEV+1)*PT5(JLON,JLEV+1))  
    ZSTAO5(JLON,JLEV)=ZDPHI5(JLON,JLEV)&
     & *(ZDTETA5(JLON,JLEV)+RD*ZAUX5(JLON,JLEV))*ZRTI5(JLON,JLEV)  

    ZUSURIC5(JLON,JLEV)=USURIC*(1.0_JPRB+(USURICL-1.0_JPRB)&
     & *((ZGLT*ZPFH5(JLON,JLEV))/(ZGLT+ZZT5(JLON,JLEV)))**USURICE)  
    ZUSURID5(JLON,JLEV)=USURID*(1.0_JPRB+1.0_JPRB&
     & /((ZZT5(JLON,JLEV)*ZDIFT)*(ZZ5(JLON,JLEV)*ZDIFV)))**USURIDE  
    ZIXP5(JLON,JLEV)=(1.0_JPRB+3._JPRB*ZUSURID5(JLON,JLEV)&
     & *MAX(0.0_JPRB,ZSTAO5(JLON,JLEV))/ZCIS5(JLON,JLEV))&
     & /(1.0_JPRB+ZUSURID5(JLON,JLEV)*MAX(0.0_JPRB,ZSTAO5(JLON,JLEV))&
     & /ZCIS5(JLON,JLEV))  
    ZSTAH5(JLON,JLEV)=ZSTAO5(JLON,JLEV)&
     & /(1.0_JPRB+ZIXP5(JLON,JLEV)*ZUSURIC5(JLON,JLEV)&
     & *MAX(0.0_JPRB,ZSTAO5(JLON,JLEV))&
     & /ZCIS5(JLON,JLEV))**(1.0_JPRB/ZIXP5(JLON,JLEV))  

    ZSTA5(JLON,JLEV)=ZSTAO5(JLON,JLEV)/(1.0_JPRB+MAX(0.0_JPRB &
     & ,ZSTAO5(JLON,JLEV))*ZUSURIC5(JLON,JLEV)/ZCIS5(JLON,JLEV))  

    ZIS=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZSTA5(JLON,JLEV)))
    ZSTAC5(JLON,JLEV)=ABS(ZSTA5(JLON,JLEV))
    ZSTACH5(JLON,JLEV)=ABS(ZSTAH5(JLON,JLEV))

!     Common computations for momentum and energy.

    ZDS5(JLON,JLEV)=SQRT(ZCIS5(JLON,JLEV)+(EDD/EDK)*ZSTAC5(JLON,JLEV))
    ZHS5(JLON,JLEV)=SQRT(ZCIS5(JLON,JLEV)+EDD*EDK*ZSTACH5(JLON,JLEV))

!     Computations for the wind components.

    ZLOS5(JLON,JLEV,1)=ZCIS5(JLON,JLEV)*ZDS5(JLON,JLEV)&
     & /(ZU5(JLON,JLEV)*ZDS5(JLON,JLEV)&
     & +Z2B*ZSTAC5(JLON,JLEV))  
    ZLOI5(JLON,JLEV,1)=ZU5(JLON,JLEV)-Z2B*ZSTA5(JLON,JLEV)&
     & /(ZU5(JLON,JLEV)+ZCK5(JLON,JLEV)*SQRT(ZSTAC5(JLON,JLEV)))  

    PKUROV5(JLON,JLEV)=(ZLOI5(JLON,JLEV,1)&
     & +ZIS*(ZLOS5(JLON,JLEV,1)-ZLOI5(JLON,JLEV,1)))&
     & *ZGLMU25(JLON,JLEV)*PAPRS5(JLON,JLEV)&
     & *ZRTI5(JLON,JLEV)/ZDPHI5(JLON,JLEV)**2  

!     Computations for temperature and humidity.

    ZLOS5(JLON,JLEV,2)=ZCIS5(JLON,JLEV)**2 &
     & /(ZU5(JLON,JLEV)*ZCIS5(JLON,JLEV)&
     & +Z3B*ZSTACH5(JLON,JLEV)*ZHS5(JLON,JLEV))  
    ZLOI5(JLON,JLEV,2)=ZU5(JLON,JLEV)-Z3B*ZSTA5(JLON,JLEV)&
     & /(ZU5(JLON,JLEV)+ZCK5(JLON,JLEV)*SQRT(ZSTAC5(JLON,JLEV)))  

    PKTROV5(JLON,JLEV)=(ZLOI5(JLON,JLEV,2)&
     & +ZIS*(ZLOS5(JLON,JLEV,2)-ZLOI5(JLON,JLEV,2)))&
     & *ZGLMT25(JLON,JLEV)*PAPRS5(JLON,JLEV)&
     & *ZRTI5(JLON,JLEV)/ZDPHI5(JLON,JLEV)**2  

  ENDDO
ENDDO
!*

!     INITIALIZATION OF THE LOCAL VARIABLES

ZCDNH(:)=0.0_JPRB
ZDPHI(:)=0.0_JPRB
ZRTI(:)=0.0_JPRB
ZSTA(:)=0.0_JPRB
ZSTAH(:)=0.0_JPRB
ZU(:)=0.0_JPRB
ZRS(:)=0.0_JPRB
ZSTAB(:)=0.0_JPRB
ZCP(:)=0.0_JPRB
ZPCD(:)=0.0_JPRB
ZPCDN(:)=0.0_JPRB
ZPCH(:)=0.0_JPRB
ZSTAC(:)=0.0_JPRB
ZSTACH(:)=0.0_JPRB

ZLSCPE(:,:)=0.0_JPRB
ZQSAT(:,:)=0.0_JPRB
ZLH(:,:)=0.0_JPRB
ZR(:,:)=0.0_JPRB

!*
!     ------------------------------------------------------------------
!     V - COMPUTATION OF VERTICAL EXCHANGE COEFFICIENTS,
!     AS WELL AS OF THE "ANTI-FIBRILLATION" AMPLIFICATION
!     FACTORS OF THE FORMERS .

!*
!          GOING TO GEOPOTENTIAL FOR ASYMPTOTIC MIXING LENGTHS.

ZGLU=RG*ALMAV
ZGLT=ZGLU*ZRLTLU

!*
!           PASSIVE LOOP ON VERTICAL LEVELS.

DO JLEV=KLEV-1,KTDIA,-1

!*
!          EFFECTIVE CALCULATIONS.

  DO JLON=KIDIA,KFDIA
    ZCK=0.0_JPRB
    ZLOI=0.0_JPRB
    ZLOS=0.0_JPRB
    ZGLMT2=0.0_JPRB
    ZCIS=0.0_JPRB
    ZDS=0.0_JPRB
    ZHS=0.0_JPRB
    ZGLMU2=0.0_JPRB
    ZDTETA=0.0_JPRB
    ZAUX=0.0_JPRB
    ZZD=0.0_JPRB
    ZZC=0.0_JPRB
    ZPFD=0.0_JPRB
    ZPFH=0.0_JPRB
    ZZ=0.0_JPRB
    ZZT=0.0_JPRB
    ZUSURIC=0.0_JPRB
    ZUSURID=0.0_JPRB
    ZIXP=0.0_JPRB

    ZIS=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZSTA5(JLON,JLEV)))

!     NO ANTI-FIBRILLATION.

    PXTROV(JLON,JLEV)=0.0_JPRB
    PXUROV(JLON,JLEV)=0.0_JPRB

!     COMPUTATIONS FOR TEMPERATURE AND HUMIDITY.

    ZLOI=ZLOI+((1.0_JPRB-ZIS)*ZGLMT25(JLON,JLEV)*PAPRS5(JLON,JLEV)&
     & *ZRTI5(JLON,JLEV)/ZDPHI5(JLON,JLEV)**2)*PKTROV(JLON,JLEV)  
    ZLOS=ZLOS+(ZIS*ZGLMT25(JLON,JLEV)*PAPRS5(JLON,JLEV)&
     & *ZRTI5(JLON,JLEV)/ZDPHI5(JLON,JLEV)**2)*PKTROV(JLON,JLEV)  
    ZGLMT2=ZGLMT2+((ZLOI5(JLON,JLEV,2)&
     & +ZIS*(ZLOS5(JLON,JLEV,2)-ZLOI5(JLON,JLEV,2)))&
     & *PAPRS5(JLON,JLEV)*ZRTI5(JLON,JLEV)/ZDPHI5(JLON,JLEV)**2)&
     & *PKTROV(JLON,JLEV)  
    ZRTI(JLON)=ZRTI(JLON)+((ZLOI5(JLON,JLEV,2)&
     & +ZIS*(ZLOS5(JLON,JLEV,2)-ZLOI5(JLON,JLEV,2)))&
     & *ZGLMT25(JLON,JLEV)&
     & *PAPRS5(JLON,JLEV)/ZDPHI5(JLON,JLEV)**2)*PKTROV(JLON,JLEV)  
    PAPRS(JLON,JLEV)=PAPRS(JLON,JLEV)+((ZLOI5(JLON,JLEV,2)&
     & +ZIS*(ZLOS5(JLON,JLEV,2)-ZLOI5(JLON,JLEV,2)))&
     & *ZGLMT25(JLON,JLEV)*ZRTI5(JLON,JLEV)&
     & /ZDPHI5(JLON,JLEV)**2)*PKTROV(JLON,JLEV)  
    ZDPHI(JLON)=ZDPHI(JLON)-(2.0_JPRB*(ZLOI5(JLON,JLEV,2)+ZIS &
     & *(ZLOS5(JLON,JLEV,2)-ZLOI5(JLON,JLEV,2)))*ZGLMT25(JLON,JLEV)&
     & *PAPRS5(JLON,JLEV)*ZRTI5(JLON,JLEV)&
     & /ZDPHI5(JLON,JLEV)**3)*PKTROV(JLON,JLEV)  
    PKTROV(JLON,JLEV)=0.0_JPRB

    ZU(JLON)=ZU(JLON)+(1.0_JPRB+Z3B*ZSTA5(JLON,JLEV)&
     & /(ZU5(JLON,JLEV)&
     & +ZCK5(JLON,JLEV)*SQRT(ZSTAC5(JLON,JLEV)))**2)*ZLOI  
    ZSTA(JLON)=ZSTA(JLON)&
     & -(Z3B/(ZU5(JLON,JLEV)&
     & +ZCK5(JLON,JLEV)*SQRT(ZSTAC5(JLON,JLEV))))*ZLOI  
    ZCK=ZCK+(Z3B*ZSTA5(JLON,JLEV)*SQRT(ZSTAC5(JLON,JLEV))&
     & /(ZU5(JLON,JLEV)&
     & +ZCK5(JLON,JLEV)*SQRT(ZSTAC5(JLON,JLEV)))**2)*ZLOI  
    ZSTAC(JLON)=ZSTAC(JLON)&
     & +(0.5_JPRB*Z3B*ZSTA5(JLON,JLEV)*ZCK5(JLON,JLEV)&
     & /(SQRT(ZSTAC5(JLON,JLEV))&
     & *(ZU5(JLON,JLEV)+ZCK5(JLON,JLEV)&
     & *SQRT(ZSTAC5(JLON,JLEV)))**2))*ZLOI  
    ZLOI=0.0_JPRB

    ZCIS=ZCIS+(ZCIS5(JLON,JLEV)*(ZCIS5(JLON,JLEV)*ZU5(JLON,JLEV)&
     & +2.0_JPRB*Z3B*ZSTACH5(JLON,JLEV)*ZHS5(JLON,JLEV))&
     & /(ZU5(JLON,JLEV)*ZCIS5(JLON,JLEV)&
     & +Z3B*ZSTACH5(JLON,JLEV)*ZHS5(JLON,JLEV))**2)*ZLOS  
    ZU(JLON)=ZU(JLON)-(ZCIS5(JLON,JLEV)**3 &
     & /(ZU5(JLON,JLEV)*ZCIS5(JLON,JLEV)&
     & +Z3B*ZSTACH5(JLON,JLEV)*ZHS5(JLON,JLEV))**2)*ZLOS  
    ZSTACH(JLON)=ZSTACH(JLON)-(Z3B*ZCIS5(JLON,JLEV)**2 &
     & *ZHS5(JLON,JLEV)&
     & /(ZU5(JLON,JLEV)*ZCIS5(JLON,JLEV)&
     & +Z3B*ZSTACH5(JLON,JLEV)*ZHS5(JLON,JLEV))**2)*ZLOS  
    ZHS=ZHS-(Z3B*ZCIS5(JLON,JLEV)**2*ZSTACH5(JLON,JLEV)&
     & /(ZU5(JLON,JLEV)*ZCIS5(JLON,JLEV)&
     & +Z3B*ZSTACH5(JLON,JLEV)*ZHS5(JLON,JLEV))**2)*ZLOS  
    ZLOS=0.0_JPRB

!     COMPUTATIONS FOR THE WIND COMPONENTS.

    ZLOI=ZLOI+((1.0_JPRB-ZIS)*ZGLMU25(JLON,JLEV)*PAPRS5(JLON,JLEV)&
     & *ZRTI5(JLON,JLEV)/ZDPHI5(JLON,JLEV)**2)*PKUROV(JLON,JLEV)  
    ZLOS=ZLOS+(ZIS*ZGLMU25(JLON,JLEV)*PAPRS5(JLON,JLEV)&
     & *ZRTI5(JLON,JLEV)/ZDPHI5(JLON,JLEV)**2)*PKUROV(JLON,JLEV)  
    ZGLMU2=ZGLMU2+((ZLOI5(JLON,JLEV,1)&
     & +ZIS*(ZLOS5(JLON,JLEV,1)-ZLOI5(JLON,JLEV,1)))&
     & *PAPRS5(JLON,JLEV)*ZRTI5(JLON,JLEV)/ZDPHI5(JLON,JLEV)**2)&
     & *PKUROV(JLON,JLEV)  
    ZRTI(JLON)=ZRTI(JLON)+((ZLOI5(JLON,JLEV,1)&
     & +ZIS*(ZLOS5(JLON,JLEV,1)-ZLOI5(JLON,JLEV,1)))&
     & *ZGLMU25(JLON,JLEV)&
     & *PAPRS5(JLON,JLEV)/ZDPHI5(JLON,JLEV)**2)*PKUROV(JLON,JLEV)  
    PAPRS(JLON,JLEV)=PAPRS(JLON,JLEV)+((ZLOI5(JLON,JLEV,1)&
     & +ZIS*(ZLOS5(JLON,JLEV,1)-ZLOI5(JLON,JLEV,1)))&
     & *ZGLMU25(JLON,JLEV)*ZRTI5(JLON,JLEV)&
     & /ZDPHI5(JLON,JLEV)**2)*PKUROV(JLON,JLEV)  
    ZDPHI(JLON)=ZDPHI(JLON)-(2.0_JPRB*(ZLOI5(JLON,JLEV,1)+ZIS &
     & *(ZLOS5(JLON,JLEV,1)-ZLOI5(JLON,JLEV,1)))*ZGLMU25(JLON,JLEV)&
     & *PAPRS5(JLON,JLEV)*ZRTI5(JLON,JLEV)&
     & /ZDPHI5(JLON,JLEV)**3)*PKUROV(JLON,JLEV)  
    PKUROV(JLON,JLEV)=0.0_JPRB

    ZU(JLON)=ZU(JLON)+(1.0_JPRB+Z2B*ZSTA5(JLON,JLEV)&
     & /(ZU5(JLON,JLEV)&
     & +ZCK5(JLON,JLEV)*SQRT(ZSTAC5(JLON,JLEV)))**2)*ZLOI  
    ZSTA(JLON)=ZSTA(JLON)&
     & -(Z2B/(ZU5(JLON,JLEV)&
     & +ZCK5(JLON,JLEV)*SQRT(ZSTAC5(JLON,JLEV))))*ZLOI  
    ZCK=ZCK+(Z2B*ZSTA5(JLON,JLEV)*SQRT(ZSTAC5(JLON,JLEV))&
     & /(ZU5(JLON,JLEV)&
     & +ZCK5(JLON,JLEV)*SQRT(ZSTAC5(JLON,JLEV)))**2)*ZLOI  
    ZSTAC(JLON)=ZSTAC(JLON)&
     & +(0.5_JPRB*Z2B*ZSTA5(JLON,JLEV)*ZCK5(JLON,JLEV)&
     & /(SQRT(ZSTAC5(JLON,JLEV))&
     & *(ZU5(JLON,JLEV)+ZCK5(JLON,JLEV)&
     & *SQRT(ZSTAC5(JLON,JLEV)))**2))*ZLOI  

    ZCIS=ZCIS+(ZDS5(JLON,JLEV)/(ZU5(JLON,JLEV)*ZDS5(JLON,JLEV)&
     & +Z2B*ZSTAC5(JLON,JLEV)))*ZLOS  
    ZDS=ZDS+(Z2B*ZSTAC5(JLON,JLEV)*ZCIS5(JLON,JLEV)&
     & /(ZU5(JLON,JLEV)*ZDS5(JLON,JLEV)+Z2B*ZSTAC5(JLON,JLEV))**2)&
     & *ZLOS  
    ZU(JLON)=ZU(JLON)-(ZCIS5(JLON,JLEV)*ZDS5(JLON,JLEV)**2 &
     & /(ZU5(JLON,JLEV)*ZDS5(JLON,JLEV)+Z2B*ZSTAC5(JLON,JLEV))**2)&
     & *ZLOS  
    ZSTAC(JLON)=ZSTAC(JLON)-(Z2B*ZCIS5(JLON,JLEV)*ZDS5(JLON,JLEV)&
     & /(ZU5(JLON,JLEV)*ZDS5(JLON,JLEV)+Z2B*ZSTAC5(JLON,JLEV))**2)&
     & *ZLOS  

!     COMMON COMPUTATIONS FOR MOMENTUM AND ENERGY.

    ZCIS=ZCIS+(0.5_JPRB/SQRT(ZCIS5(JLON,JLEV)+EDD*EDK*ZSTACH5(JLON,JLEV)))*ZHS
    ZSTACH(JLON)=ZSTACH(JLON)+(0.5_JPRB*EDD*EDK/SQRT(ZCIS5(JLON,JLEV)&
     & +EDD*EDK*ZSTACH5(JLON,JLEV)))*ZHS  

    ZCIS=ZCIS+(0.5_JPRB/SQRT(ZCIS5(JLON,JLEV)+(EDD/EDK)*ZSTAC5(JLON,JLEV)))*ZDS
    ZSTAC(JLON)=ZSTAC(JLON)+(0.5_JPRB*(EDD/EDK)/SQRT(ZCIS5(JLON,JLEV)&
     & +(EDD/EDK)*ZSTAC5(JLON,JLEV)))*ZDS  

!     STABILITY COMPUTATION

    IF (ZSTAH5(JLON,JLEV) > 0.0_JPRB) THEN
      ZSTAH(JLON)=ZSTAH(JLON)+ZSTACH(JLON)
      ZSTACH(JLON)=0.0_JPRB
    ELSE
      ZSTAH(JLON)=ZSTAH(JLON)-ZSTACH(JLON)
      ZSTACH(JLON)=0.0_JPRB
    ENDIF

    IF (ZSTA5(JLON,JLEV) > 0.0_JPRB) THEN
      ZSTA(JLON)=ZSTA(JLON)+ZSTAC(JLON)
      ZSTAC(JLON)=0.0_JPRB
    ELSE
      ZSTA(JLON)=ZSTA(JLON)-ZSTAC(JLON)
      ZSTAC(JLON)=0.0_JPRB
    ENDIF

    IF (ZSTAO5(JLON,JLEV) > 0.0_JPRB) THEN
      ZUSURIC=ZUSURIC &
       & -ZSTA(JLON)*ZCIS5(JLON,JLEV)*ZSTAO5(JLON,JLEV)**2 &
       & /(ZCIS5(JLON,JLEV)+ZSTAO5(JLON,JLEV)*ZUSURIC5(JLON,JLEV))**2  
      ZCIS=ZCIS &
       & +ZSTA(JLON)*ZUSURIC5(JLON,JLEV)*ZSTAO5(JLON,JLEV)**2 &
       & /(ZCIS5(JLON,JLEV)+ZSTAO5(JLON,JLEV)*ZUSURIC5(JLON,JLEV))**2  
      ZSTA(JLON)=ZSTA(JLON)*ZCIS5(JLON,JLEV)**2 &
       & /(ZCIS5(JLON,JLEV)+ZSTAO5(JLON,JLEV)*ZUSURIC5(JLON,JLEV))**2  
    ENDIF

    IF (ZSTAO5(JLON,JLEV) > 0.0_JPRB) THEN
      ZTERM1=(1.0_JPRB+ZIXP5(JLON,JLEV)*ZUSURIC5(JLON,JLEV)*ZSTAO5(JLON,JLEV) &
       & /ZCIS5(JLON,JLEV))**(-1.0_JPRB/ZIXP5(JLON,JLEV))  
      ZSTA(JLON)=ZSTA(JLON)+ZSTAH(JLON)*ZTERM1
      ZIXP=ZIXP+ZSTAH(JLON)*ZTERM1*ZSTAO5(JLON,JLEV)/ZIXP5(JLON,JLEV)**2 &
       & *LOG(1.0_JPRB+ZIXP5(JLON,JLEV)*ZUSURIC5(JLON,JLEV)*ZSTAO5(JLON,JLEV) &
       & /ZCIS5(JLON,JLEV))  
      ZIXP=ZIXP-ZSTAH(JLON)*ZTERM1*ZUSURIC5(JLON,JLEV) &
       & *ZSTAO5(JLON,JLEV)**2/ZIXP5(JLON,JLEV) &
       & /(ZCIS5(JLON,JLEV)+ZIXP5(JLON,JLEV)*ZUSURIC5(JLON,JLEV) &
       & *ZSTAO5(JLON,JLEV))  
      ZUSURIC=ZUSURIC-ZSTAH(JLON)*ZTERM1*ZSTAO5(JLON,JLEV)**2 &
       & /(ZCIS5(JLON,JLEV)+ZIXP5(JLON,JLEV)*ZUSURIC5(JLON,JLEV) &
       & *ZSTAO5(JLON,JLEV))  
      ZSTA(JLON)=ZSTA(JLON)-ZSTAH(JLON)*ZTERM1*ZSTAO5(JLON,JLEV) &
       & *ZUSURIC5(JLON,JLEV) &
       & /(ZCIS5(JLON,JLEV)+ZIXP5(JLON,JLEV)*ZUSURIC5(JLON,JLEV) &
       & *ZSTAO5(JLON,JLEV))  
      ZCIS=ZCIS+ZSTAH(JLON)*ZTERM1*ZUSURIC5(JLON,JLEV)*ZSTAO5(JLON,JLEV)**2 &
       & /(ZCIS5(JLON,JLEV)**2+ZIXP5(JLON,JLEV)*ZUSURIC5(JLON,JLEV) &
       & *ZSTAO5(JLON,JLEV)*ZCIS5(JLON,JLEV))  
    ELSE
      ZSTA(JLON)=ZSTA(JLON)+ZSTAH(JLON)
    ENDIF
    ZSTAH(JLON)=0.0_JPRB

    IF (ZSTAO5(JLON,JLEV) > 0.0_JPRB) THEN
      ZUSURID=ZUSURID+2.0_JPRB*ZIXP*ZSTAO5(JLON,JLEV)*ZCIS5(JLON,JLEV) &
       & /(ZCIS5(JLON,JLEV)+ZUSURID5(JLON,JLEV)*ZSTAO5(JLON,JLEV))**2  
      ZSTA(JLON)=ZSTA(JLON)+2.0_JPRB*ZIXP*ZUSURID5(JLON,JLEV)*ZCIS5(JLON,JLEV) &
       & /(ZCIS5(JLON,JLEV)+ZUSURID5(JLON,JLEV)*ZSTAO5(JLON,JLEV))**2  
      ZCIS=ZCIS-2.0_JPRB*ZIXP*ZUSURID5(JLON,JLEV)*ZSTAO5(JLON,JLEV) &
       & /(ZCIS5(JLON,JLEV)+ZUSURID5(JLON,JLEV)*ZSTAO5(JLON,JLEV))**2  
    ENDIF
    ZIXP=0.0_JPRB

    ZTERM2=(1.0_JPRB+1.0_JPRB/((ZZT5(JLON,JLEV)*ZDIFT)*(ZZ5(JLON,JLEV)*ZDIFV))) &
     & **(USURIDE-1.0_JPRB)  
    ZZT=ZZT-ZUSURID*USURID*USURIDE*ZTERM2*ZZ5(JLON,JLEV)/ &
     & ((ZDIFT*ZDIFV)*(ZZT5(JLON,JLEV)*ZZ5(JLON,JLEV))**2)  
    ZZ=ZZ-ZUSURID*USURID*USURIDE*ZTERM2*ZZT5(JLON,JLEV)/ &
     & ((ZDIFT*ZDIFV)*(ZZ5(JLON,JLEV)*ZZT5(JLON,JLEV))**2)  
    ZUSURID=0.0_JPRB

    ZTERM3=USURIC*(USURICL-1.0_JPRB)*USURICE*ZGLT/(ZGLT+ZZT5(JLON,JLEV))**2 &
     & *((ZGLT*ZPFH5(JLON,JLEV))/(ZGLT+ZZT5(JLON,JLEV)))**(USURICE-1.0_JPRB)  
    ZPFH=ZPFH+ZUSURIC*ZTERM3*(ZGLT+ZZT5(JLON,JLEV))
    ZZT=ZZT-ZUSURIC*ZTERM3*ZPFH5(JLON,JLEV)

    ZDPHI(JLON)=ZDPHI(JLON)+(ZDTETA5(JLON,JLEV)&
     & +RD*ZAUX5(JLON,JLEV))*ZRTI5(JLON,JLEV)*ZSTA(JLON)  
    ZDTETA=ZDTETA+ZDPHI5(JLON,JLEV)*ZRTI5(JLON,JLEV)*ZSTA(JLON)
    ZAUX=ZAUX+RD*ZDPHI5(JLON,JLEV)*ZRTI5(JLON,JLEV)*ZSTA(JLON)
    ZRTI(JLON)=ZRTI(JLON)+(ZDTETA5(JLON,JLEV)+RD*ZAUX5(JLON,JLEV))&
     & *ZDPHI5(JLON,JLEV)*ZSTA(JLON)  
    ZSTA(JLON)=0.0_JPRB

    PT(JLON,JLEV)=PT(JLON,JLEV)-2.0_JPRB*ZR5(JLON,JLEV)*ZRTI(JLON)&
     & /(ZR5(JLON,JLEV)*PT5(JLON,JLEV)+ZR5(JLON,JLEV+1)&
     & *PT5(JLON,JLEV+1))**2  
    ZR(JLON,JLEV)=ZR(JLON,JLEV)-2.0_JPRB*PT5(JLON,JLEV)*ZRTI(JLON)&
     & /(ZR5(JLON,JLEV)*PT5(JLON,JLEV)+ZR5(JLON,JLEV+1)&
     & *PT5(JLON,JLEV+1))**2  
    PT(JLON,JLEV+1)=PT(JLON,JLEV+1)-2.0_JPRB*ZR5(JLON,JLEV+1)*ZRTI(JLON)&
     & /(ZR5(JLON,JLEV)*PT5(JLON,JLEV)+ZR5(JLON,JLEV+1)&
     & *PT5(JLON,JLEV+1))**2  
    ZR(JLON,JLEV+1)=ZR(JLON,JLEV+1)-2.0_JPRB*PT5(JLON,JLEV+1)*ZRTI(JLON)&
     & /(ZR5(JLON,JLEV)*PT5(JLON,JLEV)+ZR5(JLON,JLEV+1)&
     & *PT5(JLON,JLEV+1))**2  
    ZRTI(JLON)=0.0_JPRB

!     CORRECTION FOR SHALLOW CONVECTION (OPTIONAL).

    IF (LCVPP) THEN
      ZZB=MAX(0.0_JPRB,SIGN(1.0_JPRB,RD*(PQ5(JLON,JLEV+1)-PQ5(JLON,JLEV))&
       & *(0.5_JPRB*(ZLSCPE5(JLON,JLEV)&
       & +ZLSCPE5(JLON,JLEV+1)))-ZDTETA5(JLON,JLEV)))  

      ZLSCPE(JLON,JLEV)=ZLSCPE(JLON,JLEV)&
       & +ZZD5(JLON,JLEV)*0.5_JPRB*ZRCVPP*ZZB*ZAUX  
      ZLSCPE(JLON,JLEV+1)=ZLSCPE(JLON,JLEV+1)&
       & +ZZD5(JLON,JLEV)*0.5_JPRB*ZRCVPP*ZZB*ZAUX  
      ZZD=ZZD+0.5_JPRB*(ZLSCPE5(JLON,JLEV)+ZLSCPE5(JLON,JLEV+1))&
       & *ZRCVPP*ZZB*ZAUX  

      IF (ZZC5(JLON,JLEV) < 0.0_JPRB) THEN
        ZZC=ZZC+ZZD
      ENDIF

      PQ(JLON,JLEV)=PQ(JLON,JLEV)+ZZC
      ZQSAT(JLON,JLEV)=ZQSAT(JLON,JLEV)-ZZC
      PQ(JLON,JLEV+1)=PQ(JLON,JLEV+1)-ZZC
      ZQSAT(JLON,JLEV+1)=ZQSAT(JLON,JLEV+1)+ZZC
    ENDIF

!     PRELIMINARY STABILITY COMPUTATION.

    PT(JLON,JLEV)=PT(JLON,JLEV)+ZR5(JLON,JLEV)*ZDTETA
    ZR(JLON,JLEV)=ZR(JLON,JLEV)+PT5(JLON,JLEV)*ZDTETA
    PT(JLON,JLEV+1)=PT(JLON,JLEV+1)-ZR5(JLON,JLEV+1)*ZDTETA
    ZR(JLON,JLEV+1)=ZR(JLON,JLEV+1)-PT5(JLON,JLEV+1)*ZDTETA
    ZDPHI(JLON)=ZDPHI(JLON)+RKAPPA*ZDTETA

!     WIND SHEAR.

    ZCIS=ZCIS+ZU(JLON)/(2.0_JPRB*SQRT(ZCIS5(JLON,JLEV)))
    ZU(JLON)=0.0_JPRB

    ZXX=(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))**2 &
     & +(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))**2  
    IF (ZXX > ZEPS1) THEN
      PU(JLON,JLEV)=PU(JLON,JLEV)&
       & +2.0_JPRB*(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))*ZCIS  
      PU(JLON,JLEV+1)=PU(JLON,JLEV+1)&
       & -2.0_JPRB*(PU5(JLON,JLEV)-PU5(JLON,JLEV+1))*ZCIS  
      PV(JLON,JLEV)=PV(JLON,JLEV)&
       & +2.0_JPRB*(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))*ZCIS  
      PV(JLON,JLEV+1)=PV(JLON,JLEV+1)&
       & -2.0_JPRB*(PV5(JLON,JLEV)-PV5(JLON,JLEV+1))*ZCIS  
    ENDIF

!     GEOMETRIC COMPUTATIONS, THE MULTIPLYING FACTOR ZPFD (POTENTIALLY
!     DIFFERENT OF 1) FOR THE DIFFUSION'S VERTICAL PROFILE DEPENDS UPON
!     THE CHOICE OF UHDIFV AND BEDIFV.

    PAPHIF(JLON,JLEV)=PAPHIF(JLON,JLEV)+ZDPHI(JLON)
    PAPHIF(JLON,JLEV+1)=PAPHIF(JLON,JLEV+1)-ZDPHI(JLON)
    ZDPHI(JLON)=0.0_JPRB

    ZTERM4=Z3BCF*ZGLT*ZGLU*ZPFD5(JLON,JLEV)/(ZGLU+ZZ5(JLON,JLEV)) &
     & /(ZGLT+ZZT5(JLON,JLEV))**2  
    ZTERM5=Z3BCF*ZGLT*ZGLU*ZPFH5(JLON,JLEV)/(ZGLT+ZZT5(JLON,JLEV)) &
     & /(ZGLU+ZZ5(JLON,JLEV))**2  
    ZPFH=ZPFH+ZTERM4*(ZGLT+ZZT5(JLON,JLEV))*ZCK
    ZZT=ZZT-ZTERM4*ZPFH5(JLON,JLEV)*ZCK
    ZPFD=ZPFD+ZTERM5*(ZGLU+ZZ5(JLON,JLEV))*ZCK
    ZZ=ZZ-ZTERM5*ZPFD5(JLON,JLEV)*ZCK
    
    ZTERM6=ZGLT*ZZ5(JLON,JLEV)*ZGLU*ZPFD5(JLON,JLEV) &
     & /(ZGLU+ZZ5(JLON,JLEV))/(ZGLT+ZZT5(JLON,JLEV))**2  
    ZTERM7=ZGLU*ZZT5(JLON,JLEV)*ZGLT*ZPFH5(JLON,JLEV) &
     & /(ZGLT+ZZT5(JLON,JLEV))/(ZGLU+ZZ5(JLON,JLEV))**2  
    ZZT=ZZT+ZTERM6*ZGLT*ZPFH5(JLON,JLEV)*ZGLMT2
    ZPFH=ZPFH+ZTERM6*(ZGLT*ZZT5(JLON,JLEV)+ZZT5(JLON,JLEV)**2)*ZGLMT2
    ZZ=ZZ+ZTERM7*ZGLU*ZPFD5(JLON,JLEV)*ZGLMT2
    ZPFD=ZPFD+ZTERM7*(ZGLU*ZZ5(JLON,JLEV)+ZZ5(JLON,JLEV)**2)*ZGLMT2

    ZZ=ZZ+(2.0_JPRB*ZGLU**3*ZZ5(JLON,JLEV)*ZPFD5(JLON,JLEV)**2 &
     & /(ZGLU+ZZ5(JLON,JLEV))**3)*ZGLMU2  
    ZPFD=ZPFD+(2.0_JPRB*ZGLU**2*ZZ5(JLON,JLEV)**2*ZPFD5(JLON,JLEV)&
     & /(ZGLU+ZZ5(JLON,JLEV))**2)*ZGLMU2  

    ZZZPFH=ZEDIFV+(1.0_JPRB-ZEDIFV)/(1.0_JPRB+(ZDIFT*ZZT5(JLON,JLEV))**2)
    IF (ZZZPFH > 0.0_JPRB) THEN
      ZZT=ZZT+(-2.0_JPRB*(1.0_JPRB-ZEDIFV)*ZDIFT**2.0_JPRB*ZZT5(JLON,JLEV) &
       & /(1.0_JPRB+(ZDIFT*ZZT5(JLON,JLEV))**2)**2)*ZPFH  
    ENDIF
    
    ZZZPF=BEDIFV+(1.0_JPRB-BEDIFV)/(1.0_JPRB+(ZDIFV*ZZ5(JLON,JLEV))**2)
    IF (ZZZPF > 0.0_JPRB) THEN
      ZZ=ZZ-(2.0_JPRB*(1.0_JPRB-BEDIFV)*ZDIFV**2.0_JPRB*ZZ5(JLON,JLEV)&
       & /(1.0_JPRB+(ZDIFV*ZZ5(JLON,JLEV))**2)**2)*ZPFD  
    ENDIF

    PAPHIF(JLON,JLEV)=PAPHIF(JLON,JLEV)+VKARMN*0.5_JPRB*ZZT
    PAPHIF(JLON,JLEV+1)=PAPHIF(JLON,JLEV+1)+VKARMN*0.5_JPRB*ZZT
    PAPHI(JLON,KLEV)=PAPHI(JLON,KLEV)-VKARMN*ZZT

    PAPHIF(JLON,JLEV)=PAPHIF(JLON,JLEV)+VKARMN*0.5_JPRB*ZZ
    PAPHIF(JLON,JLEV+1)=PAPHIF(JLON,JLEV+1)+VKARMN*0.5_JPRB*ZZ
    PAPHI(JLON,KLEV)=PAPHI(JLON,KLEV)-VKARMN*ZZ

  ENDDO

ENDDO

!*
!     ------------------------------------------------------------------
!     IV - COMPUTATION OF SURFACE EXCHANGE COEFFICIENTS PLUS THEIR
!     AMPLIFICATION  FACTOR IF "ANTI-FIBRILLATION" IS ACTIVATED .

!*
DO JLON=KIDIA,KFDIA

!     COMPUTATION OF THE EXCHANGE COEFFICIENTS MULTIPLIED BY DENSITY
!     AND WIND SPEED.

  ZPCH(JLON)=ZPCH(JLON)+ZU5(JLON,KLEV)*PAPRS5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*PCHROV(JLON)  
  ZU(JLON)=ZU(JLON)+ZPCH5(JLON)*PAPRS5(JLON,KLEV)*ZRTI5(JLON,KLEV)&
   & *PCHROV(JLON)  
  ZRTI(JLON)=ZRTI(JLON)+ZPCH5(JLON)*ZU5(JLON,KLEV)&
   & *PAPRS5(JLON,KLEV)*PCHROV(JLON)  
  PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)+ZPCH5(JLON)*ZU5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*PCHROV(JLON)  
  PCHROV(JLON)=0.0_JPRB

  ZPCD(JLON)=ZPCD(JLON)+ZU5(JLON,KLEV)*PAPRS5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*PCDROV(JLON)  
  ZU(JLON)=ZU(JLON)+ZPCD5(JLON)*PAPRS5(JLON,KLEV)*ZRTI5(JLON,KLEV)&
   & *PCDROV(JLON)  
  ZRTI(JLON)=ZRTI(JLON)+ZPCD5(JLON)*ZU5(JLON,KLEV)&
   & *PAPRS5(JLON,KLEV)*PCDROV(JLON)  
  PAPRS(JLON,KLEV)=PAPRS(JLON,KLEV)+ZPCD5(JLON)*ZU5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*PCDROV(JLON)  
  PCDROV(JLON)=0.0_JPRB
ENDDO

!*
!          EFFECTIVE CALCULATIONS OF THE SURFACE CHARACTERISTICS.

! - TEMPORARY(S) 1D .

! ZPCDN     : EXCHANGE COEF. AT SURFACE LEVEL IN NEUTRAL CONDITION.
! ZRS       : GASS CONSTANT FOR AIR AT THE SURFACE.

DO JLON=KIDIA,KFDIA
  ZCH=0.0_JPRB
  ZRZH=0.0_JPRB
  ZCIS=0.0_JPRB
  ZLOI=0.0_JPRB
  ZLOS=0.0_JPRB
  ZDIH=0.0_JPRB
  ZDS=0.0_JPRB
  ZHS=0.0_JPRB
  ZDID=0.0_JPRB
  ZDIHP=0.0_JPRB
  ZDIDP=0.0_JPRB
  ZCD=0.0_JPRB
  ZRZD=0.0_JPRB

  ZRS(JLON)=RD+ZRVMD*PQS(JLON)

!     NO ANTI-FIBRILLATION.

  PXHROV(JLON)=0.0_JPRB
  PXDROV(JLON)=0.0_JPRB

!     COMPUTATION FOR TEMPERATURE AND HUMIDITY.

  ZLOI=ZLOI+((1.0_JPRB-ZSTAB5(JLON))*ZCDNH5(JLON)/ZU5(JLON,KLEV))*ZPCH(JLON)
  ZSTAB(JLON)=ZSTAB(JLON)+((ZLOS5(JLON,KLEV,2)-ZLOI5(JLON,KLEV,2))&
   & *ZCDNH5(JLON)/ZU5(JLON,KLEV))*ZPCH(JLON)  
  ZLOS=ZLOS+(ZSTAB5(JLON)*ZCDNH5(JLON)/ZU5(JLON,KLEV))*ZPCH(JLON)
  ZCDNH(JLON)=ZCDNH(JLON)+((ZLOI5(JLON,KLEV,2)+ZSTAB5(JLON)&
   & *(ZLOS5(JLON,KLEV,2)-ZLOI5(JLON,KLEV,2)))&
   & /ZU5(JLON,KLEV))*ZPCH(JLON)  
  ZU(JLON)=ZU(JLON)-((ZLOI5(JLON,KLEV,2)+ZSTAB5(JLON)&
   & *(ZLOS5(JLON,KLEV,2)-ZLOI5(JLON,KLEV,2)))*ZCDNH5(JLON)&
   & /ZU5(JLON,KLEV)**2)*ZPCH(JLON)  
  ZPCH(JLON)=0.0_JPRB

  ZU(JLON)=ZU(JLON)+ZLOI
  ZDIH=ZDIH-Z3B*ZSTA5(JLON,KLEV)*ZLOI
  ZSTA(JLON)=ZSTA(JLON)-Z3B*ZDIH5(JLON)*ZLOI
  ZLOI=0.0_JPRB

  ZCIS=ZCIS+(ZCIS5(JLON,KLEV)*(ZCIS5(JLON,KLEV)*ZU5(JLON,KLEV)&
   & +2.0_JPRB*Z3B*ZSTACH5(JLON,KLEV)*ZHS5(JLON,KLEV))&
   & /(ZU5(JLON,KLEV)*ZCIS5(JLON,KLEV)&
   & +Z3B*ZSTACH5(JLON,KLEV)*ZHS5(JLON,KLEV))**2)*ZLOS  
  ZU(JLON)=ZU(JLON)-(ZCIS5(JLON,KLEV)**3 &
   & /(ZU5(JLON,KLEV)*ZCIS5(JLON,KLEV)&
   & +Z3B*ZSTACH5(JLON,KLEV)*ZHS5(JLON,KLEV))**2)*ZLOS  
  ZSTACH(JLON)=ZSTACH(JLON)-(Z3B*ZCIS5(JLON,KLEV)**2*ZHS5(JLON,KLEV)&
   & /(ZU5(JLON,KLEV)*ZCIS5(JLON,KLEV)&
   & +Z3B*ZSTACH5(JLON,KLEV)*ZHS5(JLON,KLEV))**2)*ZLOS  
  ZHS=ZHS-(Z3B*ZCIS5(JLON,KLEV)**2*ZSTACH5(JLON,KLEV)&
   & /(ZU5(JLON,KLEV)*ZCIS5(JLON,KLEV)&
   & +Z3B*ZSTACH5(JLON,KLEV)*ZHS5(JLON,KLEV))**2)*ZLOS  
  ZLOS=0.0_JPRB

!     COMPUTATIONS FOR MOMENTUM OF CD AND OF ITS PRODUCT BY DENSITY
!     TIME WIND SPEED.

  ZLOI=ZLOI+((1.0_JPRB-ZSTAB5(JLON))*ZPCDN5(JLON)/ZU5(JLON,KLEV))*ZPCD(JLON)
  ZSTAB(JLON)=ZSTAB(JLON)+((ZLOS5(JLON,KLEV,1)-ZLOI5(JLON,KLEV,1))&
   & *ZPCDN5(JLON)/ZU5(JLON,KLEV))*ZPCD(JLON)  
  ZLOS=ZLOS+(ZSTAB5(JLON)*ZPCDN5(JLON)/ZU5(JLON,KLEV))*ZPCD(JLON)
  ZPCDN(JLON)=ZPCDN(JLON)+((ZLOI5(JLON,KLEV,1)+ZSTAB5(JLON)&
   & *(ZLOS5(JLON,KLEV,1)-ZLOI5(JLON,KLEV,1)))&
   & /ZU5(JLON,KLEV))*ZPCD(JLON)  
  ZU(JLON)=ZU(JLON)-((ZLOI5(JLON,KLEV,1)+ZSTAB5(JLON)&
   & *(ZLOS5(JLON,KLEV,1)-ZLOI5(JLON,KLEV,1)))*ZPCDN5(JLON)&
   & /ZU5(JLON,KLEV)**2)*ZPCD(JLON)  
  ZPCD(JLON)=0.0_JPRB

  ZU(JLON)=ZU(JLON)+ZLOI
  ZDID=ZDID-Z2B*ZSTA5(JLON,KLEV)*ZLOI
  ZSTA(JLON)=ZSTA(JLON)-Z2B*ZDID5(JLON)*ZLOI

  ZCIS=ZCIS+(ZDS5(JLON,KLEV)/(ZU5(JLON,KLEV)*ZDS5(JLON,KLEV)&
   & +Z2B*ZSTAC5(JLON,KLEV)))*ZLOS  
  ZDS=ZDS+(Z2B*ZSTAC5(JLON,KLEV)*ZCIS5(JLON,KLEV)&
   & /(ZU5(JLON,KLEV)*ZDS5(JLON,KLEV)+Z2B*ZSTAC5(JLON,KLEV))**2)&
   & *ZLOS  
  ZU(JLON)=ZU(JLON)-(ZCIS5(JLON,KLEV)*ZDS5(JLON,KLEV)**2 &
   & /(ZU5(JLON,KLEV)*ZDS5(JLON,KLEV)+Z2B*ZSTAC5(JLON,KLEV))**2)&
   & *ZLOS  
  ZSTAC(JLON)=ZSTAC(JLON)-(Z2B*ZCIS5(JLON,KLEV)*ZDS5(JLON,KLEV)&
   & /(ZU5(JLON,KLEV)*ZDS5(JLON,KLEV)+Z2B*ZSTAC5(JLON,KLEV))**2)&
   & *ZLOS  

!     COMMON COMPUTATIONS FOR MOMENTUM AND ENERGY.

  ZDIHP=ZDIHP-ZDIH/ZDIHP5(JLON)**2

  ZU(JLON)=ZU(JLON)+ZDIHP
  ZCH=ZCH+Z3BC*ZCDNH5(JLON)*SQRT(ZSTAC5(JLON,KLEV)*ZRZH5(JLON))*ZDIHP
  ZCDNH(JLON)=ZCDNH(JLON)+Z3BC*ZCH5(JLON)&
   & *SQRT(ZSTAC5(JLON,KLEV)*ZRZH5(JLON))*ZDIHP  
  ZRZH=ZRZH+(0.5_JPRB*Z3BC*ZCH5(JLON)*ZCDNH5(JLON)*ZSTAC5(JLON,KLEV)&
   & /SQRT(ZSTAC5(JLON,KLEV)*ZRZH5(JLON)))*ZDIHP  
  ZSTAC(JLON)=ZSTAC(JLON)+(0.5_JPRB*Z3BC*ZCH5(JLON)*ZCDNH5(JLON)&
   & *ZRZH5(JLON)/SQRT(ZSTAC5(JLON,KLEV)*ZRZH5(JLON)))*ZDIHP  

  ZDIDP=ZDIDP-ZDID/ZDIDP5(JLON)**2

  ZU(JLON)= ZU(JLON)+ZDIDP
  ZCD=ZCD+Z3BC*ZPCDN5(JLON)*SQRT(ZSTAC5(JLON,KLEV)*ZRZD5(JLON))*ZDIDP
  ZPCDN(JLON)=ZPCDN(JLON)+Z3BC*ZCD5(JLON)&
   & *SQRT(ZSTAC5(JLON,KLEV)*ZRZD5(JLON))*ZDIDP  
  ZRZD=ZRZD+(0.5_JPRB*Z3BC*ZCD5(JLON)*ZPCDN5(JLON)*ZSTAC5(JLON,KLEV)&
   & /SQRT(ZSTAC5(JLON,KLEV)*ZRZD5(JLON)))*ZDIDP  
  ZSTAC(JLON)=ZSTAC(JLON)+(0.5_JPRB*Z3BC*ZCD5(JLON)*ZPCDN5(JLON)&
   & *ZRZD5(JLON)/SQRT(ZSTAC5(JLON,KLEV)*ZRZD5(JLON)))*ZDIDP  

  ZCIS=ZCIS+(0.5_JPRB/SQRT(ZCIS5(JLON,KLEV)+EDD*EDK*ZSTACH5(JLON,KLEV)))*ZHS
  ZSTACH(JLON)=ZSTACH(JLON)+(0.5_JPRB*EDD*EDK/SQRT(ZCIS5(JLON,KLEV)&
   & +EDD*EDK*ZSTACH5(JLON,KLEV)))*ZHS  

  ZCIS=ZCIS+(0.5_JPRB/SQRT(ZCIS5(JLON,KLEV)+(EDD/EDK)*ZSTAC5(JLON,KLEV)))*ZDS
  ZSTAC(JLON)=ZSTAC(JLON)+(0.5_JPRB*(EDD/EDK)/SQRT(ZCIS5(JLON,KLEV)&
   & +(EDD/EDK)*ZSTAC5(JLON,KLEV)))*ZDS  

!     STABILITY COMPUTATION.

! - TEMPORARY(S) 1D .

! ZRTI      : INVERSE OF R*T.
! ZSTA      : APPROXIMATION OF DELTA(PHI)*DELTA(LN(THETA)).

  IF (ZSTAH5(JLON,KLEV) > 0.0_JPRB) THEN
    ZSTAH(JLON)=ZSTAH(JLON)+ZSTACH(JLON)
    ZSTACH(JLON)=0.0_JPRB
  ELSE
    ZSTAH(JLON)=ZSTAH(JLON)-ZSTACH(JLON)
    ZSTACH(JLON)=0.0_JPRB
  ENDIF

  IF (ZSTA5(JLON,KLEV) > 0.0_JPRB) THEN
    ZSTA(JLON)=ZSTA(JLON)+ZSTAC(JLON)
    ZSTAC(JLON)=0.0_JPRB
  ELSE
    ZSTA(JLON)=ZSTA(JLON)-ZSTAC(JLON)
    ZSTAC(JLON)=0.0_JPRB
  ENDIF

  ZSTAB(JLON)=0.0_JPRB

  IF (ZSTAO5(JLON,KLEV) > 0.0_JPRB) THEN
    ZCIS=ZCIS+((ZSTAO5(JLON,KLEV)/(ZCIS5(JLON,KLEV)&
     & +ZZUSURIC*ZSTAO5(JLON,KLEV)))&
     & -(ZSTAO5(JLON,KLEV)*ZCIS5(JLON,KLEV)&
     & /(ZCIS5(JLON,KLEV)+ZZUSURIC*ZSTAO5(JLON,KLEV))**2))&
     & *ZSTA(JLON)  
    ZSTA(JLON)=((ZCIS5(JLON,KLEV)/(ZCIS5(JLON,KLEV)&
     & +ZZUSURIC*ZSTAO5(JLON,KLEV)))&
     & -(ZZUSURIC*ZSTAO5(JLON,KLEV)*ZCIS5(JLON,KLEV)&
     & /(ZCIS5(JLON,KLEV)+ZZUSURIC*ZSTAO5(JLON,KLEV))**2))&
     & *ZSTA(JLON)  
  ENDIF

  ZTERM8=(1.0_JPRB+ZZIXP*ZZUSURIC*MAX(0.0_JPRB,ZSTAO5(JLON,KLEV))/ZCIS5(JLON,KLEV))
  ZTERM9=ZSTAO5(JLON,KLEV)*ZZUSURIC/ZTERM8**(1.0_JPRB+1.0_JPRB/ZZIXP) &
   & /ZCIS5(JLON,KLEV)**2  
  IF (ZSTAO5(JLON,KLEV) > 0.0_JPRB) THEN
    ZSTA(JLON)=ZSTA(JLON) &
     & +(1.0_JPRB/ZTERM8**(1.0_JPRB/ZZIXP)-ZTERM9*ZCIS5(JLON,KLEV))*ZSTAH(JLON)  
    ZCIS=ZCIS+ZTERM9*ZSTAO5(JLON,KLEV)*ZSTAH(JLON)
  ELSE
    ZSTA(JLON)=ZSTA(JLON)+ZSTAH(JLON)
  ENDIF
  ZSTAH(JLON)=0.0_JPRB

  ZDPHI(JLON)=ZDPHI(JLON)+(ZR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON,KLEV)-ZRS(JLON)*PTS(JLON))&
   & *ZRTI5(JLON,KLEV)*ZSTA(JLON)  
  PT(JLON,KLEV)=PT(JLON,KLEV)+ZDPHI5(JLON,KLEV)*ZR5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*ZSTA(JLON)  
  ZR(JLON,KLEV)=ZR(JLON,KLEV)+ZDPHI5(JLON,KLEV)*PT5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*ZSTA(JLON)  
  ZDPHI(JLON)=ZDPHI(JLON)+RKAPPA*ZDPHI5(JLON,KLEV)&
   & *ZRTI5(JLON,KLEV)*ZSTA(JLON)  
  ZRTI(JLON)=ZRTI(JLON)+(ZR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON,KLEV)-ZRS(JLON)*PTS(JLON))&
   & *ZDPHI5(JLON,KLEV)*ZSTA(JLON)  
  ZSTA(JLON)=0.0_JPRB

  PT(JLON,KLEV)=PT(JLON,KLEV)-(2.0_JPRB*ZR5(JLON,KLEV)&
   & /(ZR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON,KLEV)+ZRS(JLON)*PTS(JLON))**2)&
   & *ZRTI(JLON)  
  ZR(JLON,KLEV)=ZR(JLON,KLEV)-(2.0_JPRB*PT5(JLON,KLEV)&
   & /(ZR5(JLON,KLEV)*PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON,KLEV)+ZRS(JLON)*PTS(JLON))**2)&
   & *ZRTI(JLON)  
  ZDPHI(JLON)=ZDPHI(JLON)-(2.0_JPRB*RKAPPA/(ZR5(JLON,KLEV)&
   & *PT5(JLON,KLEV)&
   & +RKAPPA*ZDPHI5(JLON,KLEV)+ZRS(JLON)*PTS(JLON))**2)&
   & *ZRTI(JLON)  
  ZRTI(JLON)=0.0_JPRB

!     WIND SHEAR.

! - TEMPORARY(S) 1D .

! ZU        : SURFACE WIND SPEED.
! ZPCD      : EXCHANGE COEFFICIENT AT SURFACE LEVEL FOR U AND V.

  ZCIS=ZCIS+ZU(JLON)/(2.0_JPRB*SQRT(ZCIS5(JLON,KLEV)))
  ZU(JLON)=0.0_JPRB

  IF ((PU5(JLON,KLEV)**2+PV5(JLON,KLEV)**2) > ZEPS1) THEN
    PU(JLON,KLEV)=PU(JLON,KLEV)+2.0_JPRB*PU5(JLON,KLEV)*ZCIS
    PV(JLON,KLEV)=PV(JLON,KLEV)+2.0_JPRB*PV5(JLON,KLEV)*ZCIS
  ENDIF

!     PRELIMINARY COMPUTATIONS NECESSARY FOR THE DETERMINATION OF
!     EXCHANGE COEFFICIENTS CD AND CH

! - TEMPORARY(S) 1D .

! ZCDNH     : NEUTRAL SURFACE THERMAL EXCHANGE COEFFICIENT.

  ZCH=0.0_JPRB
  ZCD=0.0_JPRB

  ZPCDN(JLON)=ZPCDN(JLON)+ZCDNH(JLON)
  ZCDNH(JLON)=0.0_JPRB

  ZRZD=ZRZD+ZRZH

!     GEOMETRIC CALCULATIONS.

  ZRZD=ZRZD-(2.0_JPRB*VKARMN**2/(ZRZD5(JLON)*LOG(ZRZD5(JLON))**3))*ZPCDN(JLON)
  ZPCDN(JLON)=0.0_JPRB

  ZDPHI(JLON)=ZDPHI(JLON)+ZRZD/PGZ0(JLON)

ENDDO

!*
!          COMPUTATION OF THE THICKNESS OF THE SBL (IN GEOPOTENTIAL).

! - TEMPORARY(S) 2D .

! ZDPHI     : GEOPOTENTIAL THICKNESS OF THE SURFACE LEVEL.

DO JLON=KIDIA,KFDIA
  PAPHIF(JLON,KLEV)=PAPHIF(JLON,KLEV)+ZDPHI(JLON)
  PAPHI(JLON,KLEV)=PAPHI(JLON,KLEV)-ZDPHI(JLON)
  ZDPHI(JLON)=0.0_JPRB
ENDDO

!*
!     ----------------------------------------------------------------
!     III - COMPUTATION OF SPECIFIC HUMIDITY OF SATURATION AND
!     EFFECTIVE RATIO OF L AND Cp FOR CONDENSATION/EVAPORATION
!     COMPUTATION OF GASS CONSTANT FOR AIR

! - TEMPORARY(S) 2D (1:KLEV) .

! ZLH       : LATENT HEAT
! ZQSAT     : SPECIFIC HUMIDITY OF SATURATION
! ZLSCPE    : EFFECTIVE RATIO OF L AND Cp FOR CONDENSATION/EVAPORATION
! ZR       : GASS CONSTANT FOR AIR

! - TEMPORARY 1D

! ZCP       : VALUE OF ATMOSPHERIC Cp

DO JLEV=KTDIA,KLEV
  DO JLON=KIDIA,KFDIA
    ZEW1=0.0_JPRB
    ZESP1=0.0_JPRB

!     SNOW OPTION DEPENDENT COMPUTATIONS.

    IF (LNEIGE) THEN
      ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PT5(JLON,JLEV)))
    ELSE
      ZDELTA=0.0_JPRB
    ENDIF

!     COMPUTATION OF Qsat(PT)

    IF ((1.0_JPRB-ZESP15(JLON,JLEV)) > 0.0_JPRB) THEN
      ZESP1=ZESP1+FOQSAD1(ZQSAT(JLON,JLEV),ZESP15(JLON,JLEV))
    ELSE
      ZESP1=ZESP1+FOQSAD2(ZQSAT(JLON,JLEV),ZESP15(JLON,JLEV))
    ENDIF
    ZEW1=ZEW1+ZESP1/PAPRSF5(JLON,JLEV)
    PAPRSF(JLON,JLEV)=PAPRSF(JLON,JLEV)&
     & -(ZEW15(JLON,JLEV)/PAPRSF5(JLON,JLEV)**2)*ZESP1  

    PT(JLON,JLEV)=PT(JLON,JLEV)+FOEWAD (ZEW1,PT5(JLON,JLEV),ZDELTA)

!     COMPUTATION OF EFFECTIVE RATIO OF L AND Cp FOR COND./EVAP.

    ZLH(JLON,JLEV)=ZLH(JLON,JLEV)+ZLSCPE(JLON,JLEV)/ZCP5(JLON,JLEV)
    ZCP(JLON)=ZCP(JLON)-(ZLH5(JLON,JLEV)/ZCP5(JLON,JLEV)**2)&
     & *ZLSCPE(JLON,JLEV)  
    ZLSCPE(JLON,JLEV)=0.0_JPRB

!     CALL TO THE DEFINED FUNCTION TO COMPUTE LH.

    PT(JLON,JLEV)=PT(JLON,JLEV)+FOLHAD (ZLH(JLON,JLEV),ZDELTA)
    ZLH(JLON,JLEV)=0.0_JPRB

    PQ(JLON,JLEV)=PQ(JLON,JLEV)-RD*ZR(JLON,JLEV)
    PQ(JLON,JLEV)=PQ(JLON,JLEV)+RV*ZR(JLON,JLEV)
    ZR(JLON,JLEV)=0.0_JPRB

    PQ(JLON,JLEV)=PQ(JLON,JLEV)-RCPD*ZCP(JLON)
    PQ(JLON,JLEV)=PQ(JLON,JLEV)+RCPV*ZCP(JLON)
    ZCP(JLON)=0.0_JPRB

  ENDDO
ENDDO

!     ----------------------------------------------------------------
ENDIF
!     ----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('ACTSECAD',1,ZHOOK_HANDLE)
END SUBROUTINE ACTSECAD
