! Library:phys $Id: hlaconds.F90 5126 2007-05-11 16:21:54Z towil $

!options xopt(noeval)
subroutine hlaconds(                             &
!     scalars and logicals 
     &              ksvar,    khor,    klev,     &
     &              kstart,   kstop,   kstep,    &
     &              pdtime,   pconacc, pdtheta,  & 
     &              ldstat,   kcall,             &  
!     prognostic arrays
     &              pu,       pv,      pt,       & 
     &              pq,       pcw,     psvar,    &
     &              pqampli,                     &
!     diagnostic arrays   
     &              ptotcov,  pomf,    pfrland,  &
     &              pdqdtin,  pdcwdin, pdsvdin,  &
     &              pdtdtin,  pf,      phalf,    &
     &              psdin,                       &
     &              ptendt,   ptendq,  ptendql,  &
     &              ptendsv,                     &

     &              pcov2d,   pcwpath, pdraindt, &
     &              pdsnowdt, preta,   prcpst,   &
     &              prcpcu,   pstsnow, pcusnow,  &
     &              pahyb,    pbhyb,   pstcon,   &
     &              pstdcon,  pstcov,  pstdcov,  &
     &              pstccov,  pstscov, pstcw,    &
     &              pstcpnt,  psttpnt, pstscal   )
 
!     ======================================================
!     PURPOSE:

!     Subroutine to compute fluxes or tendencies 
!     due to condensation and evaporation/sublimation 
!     processes in the atmosphere. 
!     STRACO (Soft TRAnsition COndenstion)
!     implies that soft transitions between convective
!     and stratiform type of conditions are treated 
!     by the scheme.

!     METHOD: 
!     ------------------------------------------------------
!     The present interface routine is used in the context 
!     of HIRLAM, but features from the ALADIN implementation
!     has been retained in order to keep differences small.

!     The implementation of STRACO in ARPEGE/ALADIN  
!     environment imposes changes compared to the 
!     original implementation in HIRLAM. This is due to 

!     i)   application of fluxes instead of tendencies
!     ii)  use of enthalpy instead of temperature
!     iii) the order of physical parameterization calls
!          is different between HIRLAM and ARPEGE. 
!          As a consequence the heating effects 
!          (enthalpy fluxes) are involved in the  
!          convection scheme of ARPEGE. To be consistent,
!          the output enthalpy flux PDIFCS from ACONDS
!          (the STRACO scheme) will include enthalpy 
!          flux.
          
!     Remark:

!     A new pseudo-prognostic array `qampli( , )' is 
!     stored at the field location `pqva' ("QVA"-pointer)

!     ------------------------------------------------------
!     AUTHOR: Bent H. Sass, DMI.
 
!     MODIFICATIONS: 
!     Original version 2004-09-02.

!     ======================================================

!     1. DECLARATIONS.
 
!     1.1 MODULES USED:

use parkind1  ,only : jpim, jprb
use yhlconst  ,only : cpair, ccpq
use yhloption ,only : lhlorig 
use yhlcond   ,only : ht273,  prbice, dttabl
use yomhook   ,only : lhook, dr_hook

!     1.2 OTHER DECLARATIONS:

implicit none

! Scalars

integer(kind=jpim),intent(in)    ::  ksvar    ! no. of extra variabl.
integer(kind=jpim),intent(in)    ::  kcall    ! mode of call 
integer(kind=jpim),intent(in)    ::  khor     ! hor. loop dimension
integer(kind=jpim),intent(in)    ::  klev     ! number of vert. lev.
integer(kind=jpim),intent(in)    ::  kstart   ! start index of loops
integer(kind=jpim),intent(in)    ::  kstop    ! end index of loops 
integer(kind=jpim),intent(in)    ::  kstep    ! step no. in forecast
real(kind=jprb),   intent(in)    ::  pdtime   ! time step (s)
logical,           intent(in)    ::  ldstat   ! statistics switch


real(kind=jprb),  intent(inout)  ::   pconacc
real(kind=jprb),  intent(inout)  ::   pdtheta

! Prognostic variables

real(kind=jprb),   intent(in)    ::  pu(khor,klev)    
!                                    first horiz. wind component  (m/s) 
real(kind=jprb),   intent(in)    ::  pv(khor,klev)
!                                    second horiz. wind component (m/s)  
real(kind=jprb),   intent(in)    ::  pt(khor,klev)
!                                    temperature (K)
real(kind=jprb),   intent(in)    ::  pq(khor,klev)
!                                    specific humidity    (kg/kg)
real(kind=jprb),   intent(inout) ::  pcw(khor,klev)
!                                    specific cloud water (kg/kg) 
real(kind=jprb),   intent(in)    ::  psvar(khor,klev,ksvar)
!                                    specific cloud ice   (kg/kg)
real(kind=jprb),   intent(inout) ::  pqampli(khor,klev)
!                                    amplitude of hum. fluctuations
! Diagnostic variables 

real(kind=jprb),   intent(inout) ::  ptotcov(khor,klev)
!                                    fractional cloud cover
real(kind=jprb),   intent(in)    ::  pomf(khor,klev)
!                                    vertical velocity (Pa/s)
real(kind=jprb),   intent(in)    ::  pfrland(khor)
!                                    fraction of land
real(kind=jprb),   intent(inout) ::  pdqdtin(khor,klev)
!                                    specific humidity tendency (1/s) 
real(kind=jprb),   intent(inout) ::  pdcwdin(khor,klev)
!                                    specific cloud water tendency (1/s)  
real(kind=jprb),   intent(inout) ::  pdsvdin(khor,klev,ksvar)
!                                    specific cloud ice tendency (1/s) 
real(kind=jprb),   intent(inout) ::  pdtdtin(khor,klev)
!                                    temperature tendency (K/s) 
real(kind=jprb),   intent(in)    ::  pf(khor,klev)
!                                    full level pressure (Pa) 
real(kind=jprb),   intent(in)    ::  phalf(khor,0:klev)
!                                    half level pressure (Pa)

real(kind=jprb),   intent(in)    ::  psdin(khor) 
!                                    surface pressure tendency (Pa/s)
!                                    (currently set to zero)

real(kind=jprb),   intent(inout) ::  ptendt(khor,klev)
!                                    accumulated temperature ph-tendency
real(kind=jprb),   intent(inout) ::  ptendq(khor,klev)
!                                    accumulated spec. hum. ph-tendency 
real(kind=jprb),   intent(inout) ::  ptendql(khor,klev)
!                                    accumulated spec. cloud water ph-tendency
real(kind=jprb),   intent(inout) ::  ptendsv(khor,klev,ksvar)
!                                    accumulated spec. cloud ice ph-tendency

!  Old HIRLAM diagnostic parameters to be computed   

real(kind=jprb),  intent(inout) ::   pcov2d  (khor)
real(kind=jprb),  intent(inout) ::   pcwpath (khor)
real(kind=jprb),  intent(inout) ::   pdraindt(khor)
real(kind=jprb),  intent(inout) ::   pdsnowdt(khor)
real(kind=jprb),  intent(inout) ::   preta   (khor,klev)
real(kind=jprb),  intent(inout) ::   prcpst  (khor) 
real(kind=jprb),  intent(inout) ::   prcpcu  (khor)
real(kind=jprb),  intent(inout) ::   pstsnow (khor)
real(kind=jprb),  intent(inout) ::   pcusnow (khor)
real(kind=jprb),  intent(inout) ::   pahyb   (klev+1) 
real(kind=jprb),  intent(inout) ::   pbhyb   (klev+1)
real(kind=jprb),  intent(inout) ::   pstcon  (klev)
real(kind=jprb),  intent(inout) ::   pstdcon (klev)
real(kind=jprb),  intent(inout) ::   pstcov  (klev)
real(kind=jprb),  intent(inout) ::   pstdcov (klev)
real(kind=jprb),  intent(inout) ::   pstccov (klev)
real(kind=jprb),  intent(inout) ::   pstscov (klev) 
real(kind=jprb),  intent(inout) ::   pstcw   (klev)
real(kind=jprb),  intent(inout) ::   pstcpnt (klev)
real(kind=jprb),  intent(inout) ::   psttpnt (klev)
real(kind=jprb),  intent(inout) ::   pstscal (klev)

! Fluxes (used for ALADIN implementation)

  real(kind=jprb)    ::  zdiftq(khor,0:klev)
!                                    turbulence flux of spec. hum.
  real(kind=jprb)    ::  zdiftql(khor,0:klev)
!                                   turbulence liquid water flux 
  real(kind=jprb)    ::  zdiftqn(khor,0:klev)
!                                    turbulence snow/ice flux     
  real(kind=jprb)    ::  zdifts(khor,0:klev)
!                                    turbulence flux of enthalpy
  real(kind=jprb)    ::  zdifcq(khor,0:klev)
!                                    convective flux of spec. hum.  
  real(kind=jprb)    ::  zdifcql(khor,0:klev)
!                                    convective flux of liquid water
  real(kind=jprb)    ::  zdifcqn(khor,0:klev) 
!                                    convective flux of ice
  real(kind=jprb)    ::  zdifcs(khor,0:klev)
!                                    convective flux of enthalpy
  real(kind=jprb)    ::  zfccql(khor,0:klev)
!                                    convective liquid condensation flux
  real(kind=jprb)    ::  zfccqn(khor,0:klev)
!                                    convective ice condensation flux
  real(kind=jprb)    ::  zfcsql(khor,0:klev)
!                                    stratiform liquid condensation flux
  real(kind=jprb)    ::  zfcsqn(khor,0:klev)
!                                    stratiform ice condensation flux
  real(kind=jprb)    ::  zfplcl(khor,0:klev)
!                                    convective liquid precipitation flux
  real(kind=jprb)    ::  zfplcn(khor,0:klev)
!                                    convective ice condensation flux
  real(kind=jprb)    ::  zfplsl(khor,0:klev)
!                                    stratiform liquid precipitation flux
  real(kind=jprb)    ::  zfplsn(khor,0:klev)  
!                                    stratiform ice precipitation flux 

! Other declarations  

logical              ::   llpupd

! Temporary interface variables

real(kind=jprb)      ::   zcw(khor,klev),zcice(khor,klev)
real(kind=jprb)      ::   ztke(khor,klev)
real(kind=jprb)      ::   zdcwdin(khor,klev),zdicedin(khor,klev)

real(kind=jprb)      ::   zahyb(khor,klev+1),zbhyb(khor,klev+1)
real(kind=jprb)      ::   zdtdt(khor,klev),zdcwdt(khor,klev)
real(kind=jprb)      ::   zdicedt(khor,klev)
real(kind=jprb)      ::   zdqdt(khor,klev)
real(kind=jprb)      ::   zps(khor), zdpsdin(khor)
real(kind=jprb)      ::   zdpf(khor,klev),zpf(khor,klev)
real(kind=jprb)      ::   zhook_handle

integer(kind=jpim)   ::   jl,  jk, jintt

#include "hlcondfc.intfb.h"
#include "hlconds.intfb.h"
#include "hltend2flx.intfb.h"

      if (lhook) call dr_hook('hlaconds',0,zhook_handle)

!  ======================================================================
!     1) Preparations to provide proper variables to condensation scheme  
  
      if( ksvar <= 1 ) then
        do jk=1,klev
          do jl=kstart,kstop
            jintt = max(1, 1 + int((ht273 -pt(jl,jk))/dttabl))
            jintt = min(jintt, 1750)
            zcice(jl,jk)    = prbice(jintt)*pcw(jl,jk)
            ztke(jl,jk)     = psvar(jl,jk,1)
            zcw(jl,jk)      = pcw(jl,jk) -zcice(jl,jk)
            zdicedin(jl,jk) = pdcwdin(jl,jk)*prbice(jintt)
            zdcwdin(jl,jk)  = pdcwdin(jl,jk) -zdicedin(jl,jk)
          enddo
        enddo
      endif

      if( ksvar == 2 ) then
        do jk=1,klev
          do jl=kstart,kstop
            zcice(jl,jk)    = psvar(jl,jk,2 )
            ztke(jl,jk)     = psvar(jl,jk,1 )    
            zcw(jl,jk)      = pcw(jl,jk)
            zdcwdin(jl,jk)  = pdcwdin(jl,jk)
            zdicedin(jl,jk) = pdsvdin(jl,jk,2)
          enddo
        enddo
      endif

      do jk=1,klev
        do jl=kstart,kstop  
          if( kstep <= 1 ) pqampli(jl,jk)=0.05_JPRB 
          pqampli(jl,jk)=max(min( pqampli(jl,jk),0.5_JPRB ), 0.005_JPRB )
          ptotcov(jl,jk)=max(min( ptotcov(jl,jk),1.0_JPRB ), 0.0_JPRB )
          zcw(jl,jk)=max( zcw(jl,jk), 0.0_JPRB )
          zcice(jl,jk)=max( zcice(jl,jk), 0.0_JPRB )
        enddo 
      enddo 
    
!     ===================================================================
!     2.  Transformation of input parameters to further treatment
!         by subroutines CONDS.

!     2.1  Basic parameters. 

!     Logical switch "llpupd" defining if stratiform condensation 
!     is to be done on old (current variables' state (llpupd is false) 
!     or on preliminary updated state (llpupd is true)  

      llpupd=.true. 
 
!     Equivalent model resolution in deg. `pdtheta' 
!     used for parameterizing scale dependent processes
!     in the STRACO scheme.

!     Surface pressure and tendency
!     NB : question mark on zdpsdin =0  

      do jl=kstart,kstop
        zps(jl)=phalf(jl,klev) 
        zdpsdin(jl)=0._JPRB
      enddo      

!     Pseudo hybrid values `ahyb',`bhyb':
!     ( note that ahyb,bhyb are defined from 1:nlev,
!     but phalf is defined from 0:nlev.

      do jk=1,klev+1
        do jl=kstart,kstop
          zahyb(jl,jk)=0._JPRB
          zbhyb(jl,jk)=phalf(jl,jk-1)/phalf(jl,klev)
        enddo 
      enddo 

!     Initialize local arrays. 

      do jk=1,klev
        do jl=kstart,kstop
          zdtdt(jl,jk)  =0.0_jprb
          zdqdt(jl,jk)  =0.0_jprb
          zdcwdt(jl,jk) =0.0_jprb
          zdicedt(jl,jk)=0.0_jprb
          zpf(jl,jk)=pf(jl,jk)
          zdpf(jl,jk)=phalf(jl,jk) -phalf(jl,jk-1) 
        enddo
      enddo

      if( lhlorig ) then  
!     ***************************** 
        do jk=0,klev
          do jl=kstart,kstop
             zdiftql(jl,jk)=0.0_jprb
             zdiftqn(jl,jk)=0.0_jprb
             zdiftq(jl,jk) =0.0_jprb
             zdifts(jl,jk) =0.0_jprb
          enddo
        enddo
      endif
!     *****************************

!     -------------------------------------------------------
!     2.2    Compute forcing from dynamics and turbulence
!     In the case of ALADIN implementation fluxes of humidity 
!     variables and enthalpy are used to produce temperature
!     and humidity tendency forcings).
!     -------------------------------------------------------

      call hlcondfc (                                      &
!     integers    
     &             khor,         klev,          kstart,    &
     &             kstop,        kstep,                    &
!     prognostic variables   
     &             pt,           pq,                       &
!     tendencies/fluxes   
     &             pdtdtin,      pdqdtin,       zdcwdin,   &
     &             zdicedin,                               &
     &             zdiftql,      zdiftqn,       zdiftq,    &
     &             zdifts,                                 &
!     other parameters  
     &             zdpf                                    )

!    ==============================================================
!     3.   Call the main routine 'hlconds' of the STRACO
!          cloud-and condensation scheme. 

!          Concerning ALADIN implementation: 

!          NB: The output variables pdifcq, pdifcql,pdifcqn, 
!          'pfccql','pfccqn','pfcsql' and 'pfcsqn'
!          are all tendencies first  
!          (kg water/(kg*s))  while 'pdifcs'
!          is temperature output tendency (layer values due
!          to convection processes). 

!          For ALADIN the relevant conversions to fluxes are done 
!          by calling 'tend2flx' separately for each output flux.
!          For the precipitation fluxes
!          ( pfplcl, pfplcn, pfplsl, pfplsn) no conversion is 
!          needed since fluxes are already available. 
!     -------------------------------------------------------------

      call hlconds (                                       &    
!     integers and logicals
     &             kcall,        khor,          klev,      &
     &             kstart,       kstop,         kstep,     &
     &             pdtime,       pconacc,       pdtheta,   &
     &             ldstat,       llpupd,                   &
!     prognostic variables 
     &             pu,           pv,            pt,        &
     &             pq,           zcw,           zcice,     &
     &             pqampli,      ztke,                     &
!     diagnostic variables 
     &             pomf,         ptotcov,       pfrland,   &  
     &             pdtdtin,      zdtdt,         pdqdtin,   &
     &             zdqdt,        zdcwdin,       zdcwdt,    &
     &             zdicedin,     zdicedt,                  &
     &             zpf,          zdpf,          zps,       &
     &             zdpsdin,      zahyb,         zbhyb,     &

     &             pcov2d,       pcwpath,       pdraindt,  &
     &             pdsnowdt,     preta,         prcpst,    &
     &             prcpcu,       pstsnow,       pcusnow,   &
     &             pstcon,                                 &
     &             pstdcon,      pstcov,        pstdcov,   &
     &             pstccov,      pstscov,       pstcw,     &
     &             pstcpnt,      psttpnt,       pstscal,   & 
!     fluxes/tendencies 
     &             zdifcq,       zdifcqn,       zdifcql,   &
     &             zdifcs,       zfccql,        zfccqn,    &
     &             zfplcl,       zfplcn,        zfplsl,    &
     &             zfplsn,       zfcsql,        zfcsqn     )
   
!     ===========================================================
!     4. Save effect of tendencies due to condensation/convection 
!        processes by adding them to the accumulated physics 
!        tendencies.
!     ===========================================================
     
      if( lhlorig ) then
!     ************************************************************* 
        if( ksvar <= 1 ) then
          do jk=1,klev
            do jl=kstart,kstop
               ptendt(jl,jk)  =  zdtdt(jl,jk)
               ptendq(jl,jk)  =  zdqdt(jl,jk)
               ptendql(jl,jk) =  zdcwdt(jl,jk) +zdicedt(jl,jk)
            enddo
          enddo
        endif

        if( ksvar == 2 ) then
          do jk=1,klev
            do jl=kstart,kstop
              ptendt(jl,jk)     =  zdtdt(jl,jk)
              ptendq(jl,jk)     =  zdqdt(jl,jk)
              ptendql(jl,jk)    =  zdcwdt(jl,jk)
              ptendsv(jl,jk,2)  =  zdicedt(jl,jk)
            enddo
          enddo
        endif 
    else  
!   ************************************************************* 

!     =========================================================
!     5. Convert 'zdifcs' (temporary temperature tendency) and 
!        'zdifcq' (temporary humidity tendency) to enthalpy 
!        tendency ("enthalpy") = T*cpair*ccpq* dq/dt + cp*dT/dt
!     =========================================================

      do jk=1,klev
        do jl=kstart,kstop
          zdifcs(jl,jk)=                              &
     &    zdifcs(jl,jk)*cpair*(1. +ccpq*pq(jl,jk)) +  &
     &    pt(jl,jk)*cpair*ccpq*zdifcq(jl,jk)
        enddo 
      enddo 

!     =========================================================
!     6. Compute the conversion from tendencies to fluxes for
!        the different output parameters , by calls to the
!        conversion routine 'hltend2flx':
!     ---------------------------------------------------------

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    zdifcs     )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    zdifcq     )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    zdifcql    )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    zdifcqn    )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    zfccql     )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    zfccqn     )
   
      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    zfcsql     )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    zfcsqn     )

    endif 
!  ***********************************************************   

    if (lhook) call dr_hook('hlconds',1,zhook_handle)

end subroutine hlaconds
