! Library:phys $Id: hlavcbr.F90 6272 2008-10-21 13:51:02Z towil $

!options xopt(noeval)
subroutine hlavcbr(  koption ,                       &
     &               ksvar   ,  kstep   ,  ldstat  , &
     &               khor    ,  klev   ,   kstart  , &
     &               kstop   ,  pdtime ,   pt      , &
     &               pq      ,  pu     ,   pv      , &
     &               pcw     ,  psv    ,             &
     &               pgpot   ,  ppf    ,   pph     , &
     &               pps     ,  pts    ,   ppblh   , &
     &               pcld2   ,  psenf  ,   platf   , &
     &               ptaux   ,  ptauy  ,   pustaro , &
     &               ptendu  ,  ptendv ,   ptendt  , &
     &               ptendq  ,  ptendql,   ptendsv , &
     &               pstddif ,  pstdif               )

!     -------------------------------------------------------------------
!     PURPOSE: (*)  Interface routine between the physics calling
!     --------      routine PHYS and the HIRLAM turbulence scheme 
!                   HLVCBR. 

!     -------------------------------------------------------------------
!     INTERFACE  :  subroutine 'HLAVCBR' is called 
!     ------------  from subroutine 'PHYS' 

!     INPUT/OUTPUT  arguments  (arguments d'entree/sortie)
!     -----------------------------------------------------  
!     koption: differnt options for physic 
!     ksvar  : number of extra prognostic fields  
!     kstep  : time step number of forecast
!     khor   : number of horizontal grid points in loops
!     klev   : number of vertical levels 
!     kstart : start index of horizontal loops 
!     kstop  : end index of horiczontal loops 
!     pdtime : time step (s)
!     pt     : current time level model level temperature (K)  
!     pq     : current time level model level specific humidity 
!     pu     : current time level model level wind towards east 
!     pv     : current time level model level wind towards north
!     pcw    : current time level model level cloud water 
!     psv    : current time level array for extra prognostic variables
!     pgpoth : half level geopotential relative to MSL ( m**2/s**2 )
!              (NB: pgpoth is not used in pure HIRLAM version) 
!     pgpot  : full level geopotential, In ALADIN relative to MSL
!              (m**2/s**2 ), BUT in HIRLAM  "pgpot" is relative to
!              local orography
!     ppf    : full level model pressure (Pa) 
!     pph    : half level model pressure (Pa), NB: defined from 0:NLEV 
!     pts    : surface temperature (K)
!     psenf  : surface sensible heat flux (W/m2) ,positive downwards.
!     platf  : surface latent   heat flux (W/m2) ,positive downwards.
!     ptaux  : surface turb. flux of momentum (Kg/(m s2)) (east component)
!     ptauy  : surface turb. flux of momentum (Kg/(m s2)) (west component)
!     pblh   : boundary layer height (m) 
!     pcld2  : model level total cloud cover  
!     --------------------------------------------------------------------

!     Parameters needed to transform HIRLAM input SVAR() to 'turbulent
!     kinetic energy', and 'prognostic cloud water' and 'prognostic ice'.
!     --------------------------------------------------------------------
!     zclq   : current time level model level cloud liquid water
!     zcice  : current time level model level cloud ice
!     ztke   : current time level model level turb.kin. energy.(m**2/s**2)

!     Parameters coming from ALADIN implementation, not needed in HIRLAM

!     zdifts   : flux profile of enthalpy due to turbulence  
!     zdiftq   : flux profile of specific humidity due to turbulence
!     zdiftql  : flux profile of cloud water due to turbulence
!     zdiftqn  : flux profile of cloud ice due to turbulence 
!     zdiftke  : flux profile of TKE due to turbulence 
!     zstrtu   : flux profile of momentum (Kg/(m s2)) due to turbulence
!     zstrtv   : flux profile of momentum (Kg/(m s2)) due to turbulence
!     ------------------------------------------------------------------
!     Tendencies which are in ALADIN accumulated process tendencies 
!     but in HIRLAM true turbulence tendencies (se below in present 
!     subroutine).

!     ptendu   : u-momentum physics tendency     
!     ptendv   : v-momentum physics tendency
!     ptendt   : temperature physics tendency 
!     ptendq   : specific humidity physics tendency
!     ptendql  : specific cloud water or condensate physics tendency
!     ptendsv  : tendencies of extra scalar variables (TKE, cloud ice)
!     ------------------------------------------------------------------

!     EXTERNAL SUBROUTINES:   VCBR
!     ----------------------     

!     AUTHOR:         Bent H. Sass,   20 July 2006 
!     -------      

!     ==================================================================
!     1. Declarations. 
!     ==================================================================

!     1.1 MODULES USED  

use parkind1  ,only : jpim, jprb
use yhlconst  ,only : cpair, ccpq
use yhloption ,only : lhlorig
use yomhook   ,only : lhook, dr_hook

!     1.2 ARGUMENT DECLARATIONS

implicit none


integer(kind=jpim),  intent(in)           ::   koption(20)
integer(kind=jpim),  intent(in)           ::   ksvar
integer(kind=jpim),  intent(in)           ::   kstep
integer(kind=jpim),  intent(in)           ::   khor
integer(kind=jpim),  intent(in)           ::   klev
integer(kind=jpim),  intent(in)           ::   kstart
integer(kind=jpim),  intent(in)           ::   kstop
real(kind=jprb)   ,  intent(in)           ::   pdtime

logical           ,  intent(in)           ::   ldstat

real(kind=jprb)   ,  intent(in)           ::   pps(khor)
real(kind=jprb)   ,  intent(inout)        ::   pstdif(klev)
real(kind=jprb)   ,  intent(inout)        ::   pstddif(klev)

real(kind=jprb)   ,  intent(in)           ::   pts(khor)
real(kind=jprb)   ,  intent(in)           ::   psenf(khor)
real(kind=jprb)   ,  intent(in)           ::   platf(khor)
real(kind=jprb)   ,  intent(in)           ::   ptaux(khor)
real(kind=jprb)   ,  intent(in)           ::   ptauy(khor)
real(kind=jprb)   ,  intent(inout)        ::   pustaro(khor)
real(kind=jprb)   ,  intent(in)           ::   ppblh(khor)

real(kind=jprb)   ,  intent(in)           ::   pt(khor,klev) 
real(kind=jprb)   ,  intent(in)           ::   pq(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   pu(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   pv(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   pcw(khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   psv(khor,klev,*)

real(kind=jprb)   ,  intent(in)           ::   pgpot(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   ppf(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   pph(khor,0:klev)
real(kind=jprb)   ,  intent(in)           ::   pcld2(khor,klev)

real(kind=jprb)   ,  intent(inout)        ::   ptendu  (khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   ptendv  (khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   ptendt  (khor,klev)

real(kind=jprb)   ,  intent(inout)        ::   ptendq  (khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   ptendql (khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   ptendsv (khor,klev,*)

!  1.3  OTHER DECLARATIONS

! - Parameters needed for internal conversions

real(kind=jprb)   ::   zclq(khor,klev)
real(kind=jprb)   ::   zcice(khor,klev)
real(kind=jprb)   ::   ztke(khor,klev)

! - Parameters used in ALADIN implementation

real(kind=jprb)     ::   zdifts(khor, 0:klev)
real(kind=jprb)     ::   zdiftq(khor, 0:klev)
real(kind=jprb)     ::   zdiftql(khor,0:klev)
real(kind=jprb)     ::   zdiftqn(khor,0:klev)
real(kind=jprb)     ::   zdiftke(khor,0:klev)
real(kind=jprb)     ::   zstrtu(khor, 0:klev)
real(kind=jprb)     ::   zstrtv(khor, 0:klev)

! - Other local declarations

real(kind=jprb)     ::   zdtdt(khor,klev)
real(kind=jprb)     ::   zdqdt(khor,klev)
real(kind=jprb)     ::   zdudt(khor,klev)
real(kind=jprb)     ::   zdvdt(khor,klev)
real(kind=jprb)     ::   zdcwdt(khor,klev)
real(kind=jprb)     ::   zdicedt(khor,klev)
real(kind=jprb)     ::   zdtkedt(khor,klev)
real(kind=jprb)     ::   zdpf(khor,klev)
real(kind=jprb)     ::   zhook_handle

integer(kind=jpim)  ::   jk, jl

!     1.5  Special include files (interface files):
!     ---------------------------------------------
#include "hlvcbr.intfb.h"
#include "hltend2flx.intfb.h"

      if (lhook) call dr_hook('hlavcbr',0,zhook_handle)

!     ----------------------------------------------- 
!     2.   Call the turbulence scheme 
!     -----------------------------------------------

!     ======================================================
!     2.0: Preparation of the correct input arguments 

      do jk=1,klev
         do jl=kstart,kstop
            zclq(jl,jk) = max(pcw(jl,jk),0._JPRB)
            ztke(jl,jk) = max(psv(jl,jk,1),0._JPRB)
            zcice(jl,jk)= max(psv(jl,jk,2),0._JPRB)
         enddo
      enddo
      if ( ksvar == 1 ) then 
      do jk=1,klev
         do jl=kstart,kstop
            zclq(jl,jk)=max(pcw(jl,jk)-zcice(jl,jk),0._JPRB)
         enddo
      enddo
      endif

      do jk=1,klev
        pstdif(jk)=0._JPRB
        pstddif(jk)=0._JPRB
      enddo 

      do jl=kstart,kstop 
        pustaro(jl)= max( pustaro(jl) , 0.005_jprb )
      enddo 

      do jk=1,klev
        do jl=kstart,kstop
          zdtdt  (jl,jk)=0.0_jprb
          zdqdt  (jl,jk)=0.0_jprb
          zdcwdt (jl,jk)=0.0_jprb
          zdicedt(jl,jk)=0.0_jprb
          zdtkedt(jl,jk)=0.0_jprb
          zdudt  (jl,jk)=0.0_jprb
          zdvdt  (jl,jk)=0.0_jprb
        enddo 
      enddo 

!     ---------------------------------------------------------
!     Call the CBR scheme 
!     ---------------------------------------------------------

      call hlvcbr( khor    ,  klev   ,    kstart ,    &
     &             kstop   ,  pdtime ,    ldstat ,    &
     &             pt      ,  pq     ,    pu     ,    &
     &             pv      ,  zclq   ,    zcice  ,    &
     &             2       ,                          &
     &             ztke    ,  pgpot  ,    ppf    ,    &
     &             pph     ,  pcld2  ,                &
     &             pts     ,  pps    ,    ptaux  ,    &
     &             ptauy   ,  psenf  ,    platf  ,    &
     &             pustaro ,  ppblh  ,    zdtdt  ,    &
     &             zdqdt   ,  zdudt  ,    zdvdt  ,    &
     &             zdcwdt  ,  zdicedt,    zdtkedt,    &
     &             pstdif  ,  pstddif,    'hlvcbr'    )

!     =========================================================
!     3. Make appropriate conversions to produce 
!        suitable output
!     ---------------------------------------------------------

    if( lhlorig) then 
!   ***********************************************************
      if( ksvar == 1 ) then
        do jk=1,klev
          do jl=kstart,kstop
            ptendu(jl,jk)   = zdudt(jl,jk)
            ptendv(jl,jk)   = zdvdt(jl,jk)
            ptendt(jl,jk)   = zdtdt(jl,jk)
            ptendq(jl,jk)   = zdqdt(jl,jk)
            ptendql(jl,jk)  = zdcwdt(jl,jk) +zdicedt(jl,jk)
            ptendsv(jl,jk,1)= zdtkedt(jl,jk)
          enddo
        enddo
      endif

      if( ksvar == 2 ) then
        do jk=1,klev
          do jl=kstart,kstop
            ptendu(jl,jk)   = zdudt(jl,jk)
            ptendv(jl,jk)   = zdvdt(jl,jk)
            ptendt(jl,jk)   = zdtdt(jl,jk)
            ptendq(jl,jk)   = zdqdt(jl,jk)
            ptendql(jl,jk)  = zdcwdt(jl,jk)
            ptendsv(jl,jk,1)= zdtkedt(jl,jk)
            ptendsv(jl,jk,2)= zdicedt(jl,jk)
          enddo
        enddo 
      endif

    else  
!   ***********************************************************
!     3.1 Save tendencies in flux arrays for ALADIN  

      do jk=1,klev
        do jl=kstart,kstop
          zdifts (jl,jk) = zdtdt(jl,jk)
          zdiftq (jl,jk) = zdqdt(jl,jk) 
          zdiftql(jl,jk) = zdcwdt(jl,jk)
          zdiftqn(jl,jk) = zdicedt(jl,jk)
          zdiftke(jl,jk) = zdtkedt(jl,jk)
          zstrtu (jl,jk) = zdudt(jl,jk)
          zstrtv (jl,jk) = zdvdt(jl,jk) 
        enddo 
      enddo 

        do jl=kstart,kstop
          zdifts  (jl,0) = 0.0_jprb
          zdiftq  (jl,0) = 0.0_jprb
          zdiftql (jl,0) = 0.0_jprb
          zdiftqn (jl,0) = 0.0_jprb
          zdiftke (jl,0) = 0.0_jprb 
          zstrtu  (jl,0) = 0.0_jprb
          zstrtv  (jl,0) = 0.0_jprb
        enddo 

!     ----------------------------------------------------------
!     3.2 Convert 'pdifts' (temporary temperature tendency) and 
!         'pdiftq' (temporary humidity tendency) to enthalpy 
!         tendency ("enthalpy") = T*cpair*ccpq* dq/dt + cp*dT/dt
!         Compute pressure thickness 'zdpf' 
!     ----------------------------------------------------------      

      do jk=1,klev
        do jl=kstart,kstop
          zdifts(jl,jk)=                              &
     &    zdifts(jl,jk)*cpair*(1. +ccpq*pq(jl,jk)) +  &
     &    pt(jl,jk)*cpair*ccpq*zdiftq(jl,jk)
          zdpf(jl,jk)=pph(jl,jk) -pph(jl,jk-1)
        enddo 
      enddo 

!     ---------------------------------------------------------
!     3.3 Compute the conversion from tendencies to fluxes for
!         the different output parameters , by calls to the
!         conversion routine 'tend2flx':
!     ---------------------------------------------------------

      call hltend2flx ( khor  ,    klev ,    kstart,  &
     &                  kstop ,    zdpf ,    zdifts   )

      call hltend2flx ( khor  ,    klev ,    kstart,  &
     &                  kstop ,    zdpf ,    zdiftq   )

      call hltend2flx ( khor  ,    klev ,    kstart,  &
     &                  kstop ,    zdpf ,    zdiftql  )

      call hltend2flx ( khor  ,    klev ,    kstart,  &
     &                  kstop ,    zdpf ,    zdiftqn  )

      call hltend2flx ( khor  ,    klev ,    kstart,  &
     &                  kstop ,    zdpf ,    zdiftke  )

      call hltend2flx ( khor  ,    klev ,    kstart,  &
     &                  kstop ,    zdpf ,    zstrtu   )

      call hltend2flx ( khor  ,    klev ,    kstart,  &
     &                  kstop ,    zdpf ,    zstrtv   )

  endif 
! ************************************************************
  if (lhook) call dr_hook('hlavcbr',1,zhook_handle)
end subroutine hlavcbr
