! Library:phys $Id: hlconds.F90 5391 2007-09-25 09:22:24Z xiaohua $

!options xopt(noeval)
  subroutine hlconds(                                          &   
!    integers and logicals
     &                 kcall,        khor,          klev,      &
     &                 kstart,       kstop,         kstep,     &
     &                 pdtime,       pconacc,       pdtheta,   & 
     &                 ldstat,       ldupd,                    &
!    prognostic variables 
     &                 pu,           pv,            pt,        &
     &                 pq,           pcw,           pcice,     &
     &                 pqampli,      ptke,                     &
!    diagnostic variables 
     &                 pomf,         ptotcov,       pfrland,   &  
     &                 pdtdtin,      pdtdt,         pdqdtin,   &
     &                 pdqdt,        pdcwdin,       pdcwdt,    &
     &                 pdicedin,     pdicedt,                  &
     &                 ppf,          pdpf,          pps,       &
     &                 pdpsdin,      pahyb,         pbhyb,     &
     &                 pcov2d,       pcwpath,       pdraindt,  &
     &                 pdsnowdt,     preta,         prcpst,    &
     &                 prcpcu,       pstsnow,       pcusnow,   &
     &                 pstcon,                                 &
     &                 pstdcon,      pstcov,        pstdcov,   &
     &                 pstccov,      pstscov,       pstcw,     &
     &                 pstcpnt,      psttpnt,       pstscal,   &
!    fluxes/tendencies 
     &                 pdifcq,       pdifcqn,       pdifcql,   &
     &                 pdifcs,       pfccql,        pfccqn,    &
     &                 pfplcl,       pfplcn,        pfplsl,    &
     &                 pfplsn,       pfcsql,        pfcsqn     )

!     ===============================================================
!     PURPOSE:
!     --------   
!     Parameterize the effect of C C P E =
!     Cloud Condensation Precipitation Evaporation processes.
!     A detailed documentation of the scheme (April 2003)
!     is provided in DMI Technical report no. 02-10.
!     References to related work is included in this report.
!     The scheme is called 'STRACO' (Soft TRAnsition COndensation)

!     INTERFACE:  subroutine 'conds' called from subroutine 'straco'
!     ----------

!     EXTERNAL SUBROUTINES:
!     ---------------------
!     'hlqcampli'    ( computes relative amplitude of total
!                      humidity variations in grid box )
!     'hlnocondcv'   ( prepares for condensation without
!                      convective parameterization in 'hlcondcv' )
!     'hlcondcv'     ( computes heating and moistening due to
!                      convection )
!     'hlcloudcv'    ( computes cloud cover in all model layers )
!     'hlcondst'     ( computes heating and moistening due to
!                      stratiform condensation )
!     'hlprevap'     ( computes precipitation release and evaporation
!                      of precipitation )


!     LITTERATURE:   DMI Technical Report no. 02-10.
!     ------------   ( Available from the
!                    Danish Meteorological Institute )
!     AUTHOR:
!     -------
!     Bent H. Sass, (DMI), based on the research during the
!                    the HIRLAM I-V projects.

!     MODIFICATIONS:
!     ------------- 
!     Original version for ALADIN/ARPEGE: 2005-12-27

!     =========================================================
!     1. Declarations     
!     ---------------

!     1.1: MODULES USED:

use parkind1  ,only : jpim     ,jprb
use yomhook   ,only : lhook    ,dr_hook
use yhloption ,only : hloption  
use yhlconst  ,only : gravit

! ----------------------------------------------------------------------- 
!     1.2: Declaration of arguments
! ----------------------------------------------------------------------- 

implicit none

! Scalars and logicals 

integer(kind=jpim),intent(in)    ::  kcall    ! mode of call 
integer(kind=jpim),intent(in)    ::  khor     ! hor. loop dimension
integer(kind=jpim),intent(in)    ::  klev     ! number of vert. lev.
integer(kind=jpim),intent(in)    ::  kstart   ! start index of loops
integer(kind=jpim),intent(in)    ::  kstop    ! end index of loops 
integer(kind=jpim),intent(in)    ::  kstep    ! step no. in forecast
real(kind=jprb),   intent(in)    ::  pdtime   ! time step (s)
real(kind=jprb),   intent(in)    ::  pconacc  ! factor for accum. statistics
real(kind=jprb),   intent(in)    ::  pdtheta  ! model resolution (deg.)

logical,           intent(in)    ::  ldstat   ! statistics switch
logical,           intent(in)    ::  ldupd    ! stratiform cond. on preliminary
                                              ! forecast values or on old values
! Prognostic variables

real(kind=jprb),   intent(in)    ::  pu(khor,klev)    
!                                    first horiz. wind component  (m/s) 
real(kind=jprb),   intent(in)    ::  pv(khor,klev)
!                                    second horiz. wind component (m/s)  
real(kind=jprb),   intent(in)    ::  pt(khor,klev)
!                                    temperature (K)
real(kind=jprb),   intent(in)    ::  pq(khor,klev)
!                                    specific humidity    (kg/kg)
real(kind=jprb),   intent(inout) ::  pcw(khor,klev)
!                                    specific cloud water (kg/kg) 
real(kind=jprb),   intent(inout) ::  pcice(khor,klev)
!                                    specific cloud ice   (kg/kg)
real(kind=jprb),   intent(inout) ::  pqampli(khor,klev)
!                                    amplitude of hum. fluctuations
real(kind=jprb),   intent(inout) ::  ptke(khor,klev)

! Diagnostic variables 

real(kind=jprb),   intent(in)    ::  pomf(khor,klev)
!                                    vertical velocity (Pa/s)
real(kind=jprb),   intent(inout) ::  ptotcov(khor,klev)
!                                    fractional cloud cover
real(kind=jprb),   intent(in)    ::  pfrland(khor)
!                                    fraction of land
real(kind=jprb),   intent(inout) ::  pdqdtin(khor,klev)
!                                    input specific humidity tendency (1/s) 
real(kind=jprb),   intent(inout) ::  pdqdt(khor,klev)
!                                    specific humidity tendency (1/s)
real(kind=jprb),   intent(inout) ::  pdcwdin(khor,klev)
!                                    input specific cloud water tendency (1/s)  
real(kind=jprb),   intent(inout) ::  pdcwdt(khor,klev)
!                                    specific cloud water tendency (1/s)
real(kind=jprb),   intent(inout) ::  pdicedin(khor,klev)
!                                    input specific cloud ice tendency (1/s) 
real(kind=jprb),   intent(inout) ::  pdicedt(khor,klev)
!                                    specific cloud ice tendency (1/s)
real(kind=jprb),   intent(inout) ::  pdtdtin(khor,klev)
!                                    input temperature tendency (K/s) 
real(kind=jprb),   intent(inout) ::  pdtdt(khor,klev)
!                                    temperature tendency (K/s)
real(kind=jprb),   intent(in)    ::  ppf(khor,klev)
!                                    full level pressure (Pa) 
real(kind=jprb),   intent(in)    ::  pdpf(khor,klev)
!                                    pressure thickness (half level diff. in Pa)
real(kind=jprb),   intent(in)    ::  pps(khor)
!                                    surface pressure (Pa) 
real(kind=jprb),   intent(in)    ::  pdpsdin(khor)
!                                    input surface pressure tendency (Pa/s)
real(kind=jprb),   intent(in)    ::  pahyb(khor,klev+1)
!                                    hybrid (A) type of parameter (Pa)
real(kind=jprb),   intent(in)    ::  pbhyb(khor,klev+1)
!                                    hybrid (B) type of parameter 
 
real(kind=jprb),   intent(inout) ::  pcov2d(khor)
real(kind=jprb),   intent(inout) ::  pcwpath(khor)
real(kind=jprb),   intent(inout) ::  pdraindt(khor)
real(kind=jprb),   intent(inout) ::  pdsnowdt(khor)
real(kind=jprb),   intent(inout) ::  preta(khor,klev)
real(kind=jprb),   intent(inout) ::  prcpst(khor)
real(kind=jprb),   intent(inout) ::  prcpcu(khor)
real(kind=jprb),   intent(inout) ::  pstsnow(khor)
real(kind=jprb),   intent(inout) ::  pcusnow(khor)
real(kind=jprb),   intent(inout) ::  pstcon(klev)
real(kind=jprb),   intent(inout) ::  pstdcon(klev)
real(kind=jprb),   intent(inout) ::  pstcov(klev)
real(kind=jprb),   intent(inout) ::  pstdcov(klev)
real(kind=jprb),   intent(inout) ::  pstccov(klev)
real(kind=jprb),   intent(inout) ::  pstscov(klev)
real(kind=jprb),   intent(inout) ::  pstcw(klev)
real(kind=jprb),   intent(inout) ::  pstcpnt(klev)
real(kind=jprb),   intent(inout) ::  psttpnt(klev)
real(kind=jprb),   intent(inout) ::  pstscal(klev)

! Fluxes 

real(kind=jprb),   intent(inout) ::  pdifcq(khor,0:klev)
!                                    convective flux of spec. hum.  
real(kind=jprb),   intent(inout) ::  pdifcql(khor,0:klev)
!                                    convective flux of liquid water
real(kind=jprb),   intent(inout) ::  pdifcqn(khor,0:klev) 
!                                    convective flux of ice
real(kind=jprb),   intent(inout) ::  pdifcs(khor,0:klev)
!                                    convective flux of enthalpy
real(kind=jprb),   intent(inout) ::  pfccql(khor,0:klev)
!                                    convective liquid condensation flux
real(kind=jprb),   intent(inout) ::  pfccqn(khor,0:klev)
!                                    convective ice condensation flux
real(kind=jprb),   intent(inout) ::  pfcsql(khor,0:klev)
!                                    stratiform liquid condensation flux
real(kind=jprb),   intent(inout) ::  pfcsqn(khor,0:klev)
!                                    stratiform ice condensation flux
real(kind=jprb),   intent(inout) ::  pfplcl(khor,0:klev)
!                                    convective liquid precipitation flux
real(kind=jprb),   intent(inout) ::  pfplcn(khor,0:klev)
!                                    convective ice condensation flux
real(kind=jprb),   intent(inout) ::  pfplsl(khor,0:klev)
!                                    stratiform liquid precipitation flux
real(kind=jprb),   intent(inout) ::  pfplsn(khor,0:klev)  
!                                    stratiform ice precipitation flux 

! -----------------------------------------------------------------------
! 1.3 Work arrays used for computations in various subroutines  
! ----------------------------------------------------------------------- 

! First indicator arrays which are integer arrays.

integer(kind=jpim)      ::  jlabcv(khor,klev)
integer(kind=jpim)      ::  jlabc(khor,klev)
integer(kind=jpim)      ::  jlab(khor,klev)
integer(kind=jpim)      ::  jindic(khor)

! Other work arrays 

real(kind=jprb)         ::  zrhuc(khor,klev)
real(kind=jprb)         ::  zdcusn(khor)
real(kind=jprb)         ::  zdstsn(khor)
real(kind=jprb)         ::  zmr(khor)
real(kind=jprb)         ::  zpcoef(khor)
real(kind=jprb)         ::  zpsp(khor)
real(kind=jprb)         ::  zrk1(khor)
real(kind=jprb)         ::  zrk2(khor)
real(kind=jprb)         ::  zldcp(khor,klev)
real(kind=jprb)         ::  zdqsdt(khor)
real(kind=jprb)         ::  zdqsdp(khor)
real(kind=jprb)         ::  zqmax(khor)
real(kind=jprb)         ::  zqsatc(khor)
real(kind=jprb)         ::  zqsat(khor,klev)
real(kind=jprb)         ::  zqp(khor,klev)
real(kind=jprb)         ::  ztp(khor,klev)
real(kind=jprb)         ::  zcwp(khor,klev)
real(kind=jprb)         ::  zcicep(khor,klev)
real(kind=jprb)         ::  ztc(khor,klev)
real(kind=jprb)         ::  zqc(khor,klev)
real(kind=jprb)         ::  zrhu(khor,klev)
real(kind=jprb)         ::  zqliq(khor,klev)
real(kind=jprb)         ::  ztcdif(khor,klev)
real(kind=jprb)         ::  zqcdif(khor,klev)
real(kind=jprb)         ::  zqac(khor,klev)
real(kind=jprb)         ::  zdptot(khor,klev)
real(kind=jprb)         ::  zdpcld(khor,klev)
real(kind=jprb)         ::  zbfrac(khor,klev)
real(kind=jprb)         ::  zdift(khor,klev)
real(kind=jprb)         ::  zdifq(khor,klev) 
real(kind=jprb)         ::  zbu(khor,klev)
real(kind=jprb)         ::  zbuca(khor,klev)
real(kind=jprb)         ::  zgpot(khor,klev)
real(kind=jprb)         ::  zscdif(khor,klev)
real(kind=jprb)         ::  zdifs(khor,klev)
real(kind=jprb)         ::  zdptop(khor,klev)
real(kind=jprb)         ::  zcptop(khor,klev)
real(kind=jprb)         ::  zcpcld(khor,klev)
real(kind=jprb)         ::  zten(khor,klev)
real(kind=jprb)         ::  ztentop(khor,klev)
real(kind=jprb)         ::  ztencld(khor,klev)
real(kind=jprb)         ::  zqentop(khor,klev)
real(kind=jprb)         ::  zqencld(khor,klev)
real(kind=jprb)         ::  zqen(khor,klev)
real(kind=jprb)         ::  zqcen(khor,klev)
real(kind=jprb)         ::  zqcetop(khor,klev)
real(kind=jprb)         ::  zqcecld(khor,klev)
real(kind=jprb)         ::  zimfac(khor,klev)
real(kind=jprb)         ::  ztet(khor,klev)
real(kind=jprb)         ::  zcoefh(khor,klev)
real(kind=jprb)         ::  zedep(khor)
real(kind=jprb)         ::  zwq(khor,klev)
real(kind=jprb)         ::  zwql(khor,klev)
real(kind=jprb)         ::  zwqi(khor,klev)
real(kind=jprb)         ::  zwt(khor,klev)
!    treatment of additional near surface loss of cloud condensate.

real(kind=jprb)         ::  zpresur(klev)
 
!    -----------------------------------------------------------
!    1.5 Other local variables
!    -----------------------------------------------------------
real(kind=jprb)      ::   zsigint,zaddpr
real(kind=jprb)      ::   zwrk1, zwrk2, zwrk3, zwrk4, &
 &                        zwrk5, zwrk6, zwrk7, zwrk8
real(kind=jprb)      ::   zhook_handle
integer(kind=jpim)   ::   jl, jk

!    -----------------------------------------------------------------------      
!    1.6 Nomenclature for labels describing convective structure  

!       jlabcv       label to identify convective cloud layers
!       jlabc        label to identify convective cloud layers (1 or 0)
!       jlab         label to identify bottom vertical level of new 
!                    convective 'entity', It contains the number 
!                    of the convective entity. 
!    -----------------------------------------------------------------------
!    1.7 Detailed parameter definition of fluxes for updating ALADIN:

!       pdifcs       convective temperature tendency associated with
!                    transport processes
!                    (disregarding precipitation and condensation processes)
!       pdifcq       convective specific humidity tendency associated with
!                    transport processes
!                    (disregarding precipitation and condensation processes)
!       pdifcql      convective tendency of liquid water associated with
!                    transport processes
!                    (disregarding precipitation and condensation processes)
!       pdifcqn      convective tendency of ice associated with
!                    transport processes
!       pfccql       convective tendency for specific humidity 
!                    (normally negative)
!                    associated with the convective liquid water 
!                    condensation (disregarding precipitation)
!       pfccqn       convective tendency for specific humidity
!                    (normally negative)
!                    associated with the convective ice condensation
!                    (disregarding precipitation)
!       pfplcl      flux profile of convective rain  
!       pfplcn      flux profile of convective snow  
!       pfplsl      flux profile of stratiform rain  
!       pfplsn      flux profile of stratiform snow
!       pfcsql      tendency of specific humidity associated with
!                   condensation of cloud water
!                   involving latent heat release but not
!                   precipitation.
!       pfcsqn      tendency of specific humidity associated with
!                   condensation of cloud ice
!                   involving latent heat release but not
!                   precipitation.
!     ==============================================================
!     1.8  Special include files (interface files):

#include "hlqcampli.intfb.h"
#include "hlnocondcv.intfb.h"
#include "hlcondst.intfb.h"
#include "hlcondcv.intfb.h"
#include "hlcloudcv.intfb.h"
#include "hlprevap.intfb.h"

      if(lhook) call dr_hook('hlconds',0,zhook_handle)

!     ==============================================================
!     2. Provisional updates due to 
!        dynamics, vertical diffusion and radiation 
!     ==============================================================

      do jk=1,klev
        do jl=kstart,kstop
          ztp(jl,jk)   =pt(jl,jk)      + pdtime*pdtdtin(jl,jk)
          zqp(jl,jk)   =pq(jl,jk)      + pdtime*pdqdtin(jl,jk)
          zcwp(jl,jk)  =pcw(jl,jk)     + pdtime*pdcwdin(jl,jk)
          zcicep(jl,jk)=pcice(jl,jk)   + pdtime*pdicedin(jl,jk)

!         Security measures that do not involve moisture 
!         budget assessments. 

          zqp(jl,jk)    = max( zqp(jl,jk),    0._jprb )
          zcwp(jl,jk)   = max( zcwp(jl,jk),   0._jprb )
          zcicep(jl,jk) = max( zcicep(jl,jk), 0._jprb )

          pdtdt(jl,jk)  =( ztp(jl,jk)    - pt(jl,jk)    )/pdtime 
          pdqdt(jl,jk)  =( zqp(jl,jk)    - pq(jl,jk)    )/pdtime 
          pdcwdt(jl,jk) =( zcwp(jl,jk)   - pcw(jl,jk)   )/pdtime
          pdicedt(jl,jk)=( zcicep(jl,jk) - pcice(jl,jk) )/pdtime

!         The tendencies right above are no more used in 
!         the IFS/ALADIN model.

        enddo
      enddo

      do jl=kstart,kstop
        zpsp(jl)=pps(jl) +pdtime*pdpsdin(jl) 
      enddo 

!     =========================================================
!     3.  First estimate of the relative variation of  
!         total specific humidity in the absence of moist 
!         convection penetrating the grid cell 
!         (subroutine 'qcampli')     
!     =========================================================

      call hlqcampli(                                     &
!    scalars and logicals 
     &                     khor,      klev,     kstart,   &
     &                     kstop,     1,        pdtime,   &
     &                     pdtheta,                       &
!    prognostic variables  
     &                     pq,        pcw,      pcice,    & 
     &                     pqampli,                       &
!    diagnostic fields 
     &                     pps,       ppf,                &
!    work arrays 
     &                     zqp,       zcwp,     zcicep,   &
     &                     zrhuc                          )

      if( kcall /= 0 ) then

!     =========================================================
!     4. Determine stratiform condensation and moistening,
!        and compute tendencies due to condensation
!        processes by calling subroutine 'condst'
!        Fist call routine 'hlnocondcv' to initialize labels. 
!     =========================================================

        call hlnocondcv(                                  &
!    integers and logicals  
     &                     kcall,     khor,     klev,     &
     &                     kstart,    kstop,              &
!    work arrays   
     &                     jlabcv,    jlabc,    jlab      )

!     =========================================================
        if( ldupd ) then  
!     --------------------------------------------------------- 
          call hlcondst(                               & 
!    scalars and logicals 
     &                  1,         khor,     klev,     &
     &                  kstep,     kstart,   kstop,    &
     &                  pdtime,                        &
!    prognostic variables 
     &                  pt,        pq,       pcw,      &
     &                  pcice,     pqampli,            &
!    diagnostic fields 
     &                  ppf,       pdpf,               & 
!    work arrays 
     &                  jlabcv,    jlab,               &
     &                  zpsp,      zldcp,    zqsat,    &
     &                  zdqsdt,    zdqsdp,   zqp,      &
     &                  ztp,       zcwp,     zcicep,   &
!    fluxes/tendencies  
     &                  pfcsql,    pfcsqn              )

!    ----------------------------------------------------------
        else
!    ---------------------------------------------------------

           do jk=1,klev
              do jl=kstart,kstop
                 zwq(jl,jk)  =pq(jl,jk)
                 zwql(jl,jk) =pcw(jl,jk)
                 zwqi(jl,jk) =pcice(jl,jk)
                 zwt(jl,jk)  =pt(jl,jk) 
              enddo
           enddo
       
           call hlcondst(                              & 
!    scalars and logicals 
     &                  1,         khor,     klev,     &
     &                  kstep,     kstart,   kstop,    &
     &                  pdtime,                        &
!    prognostic variables 
     &                  pt,        pq,       pcw,      &
     &                  pcice,     pqampli,            &
!    diagnostic fields 
     &                  ppf,       pdpf,               & 
!    work arrays 
     &                  jlabcv,    jlab,               &
     &                  zpsp,      zldcp,    zqsat,    &
     &                  zdqsdt,    zdqsdp,   zwq,      &
     &                  zwt,       zwql,     zwqi,     &
!    fluxes/tendencies  
     &                  pfcsql,    pfcsqn              )

!     -------------------------------------------------------
        endif
!     -------------------------------------------------------

!     =========================================================
!     5. Convection scheme is to be called unless
!        option hloption(2)=5 when only stratiform
!        scheme 'hlcondst' should be called
!     =========================================================

        if( hloption(2) /= 5 ) then
          call hlcondcv(                                  &
!    scalars and logicals 
     &                     kcall,     khor,     klev,     &
     &                     kstart,    kstop,    kstep,    &
     &                     pdtime,    pconacc,  pdtheta,  &
!    prognostic variables 
     &                     pu,        pv,       pt,       &
     &                     pq,        pcw,      pcice,    &  
     &                     pqampli,   ptke,               &
!    diagnostic fields
     &                     ptotcov,   pahyb,    pbhyb,    & 
     &                     ppf,       pdpf,     pdqdtin,  &
!    work arrays  
     &                     jlabcv,    jlabc,    jlab,     & 
     &                     jindic,                        &
     &                     zpsp,      zldcp,    ztc,      &
     &                     ztcdif,    zqc,      zqcdif,   &
     &                     zqp,       ztp,      zcwp,     &
     &                     zcicep,    zwt,      zedep,    &
     &                     zqsat,     zqsatc,   zdqsdt,   &
     &                     zrhu,      zqmax,    zqliq,    &
     &                     zqac,      zbfrac,   zdift,    &
     &                     zdifq,     zscdif,   zdifs,    &      
     &                     zbu,       zbuca,    zdpcld,   &
     &                     zdptot,    zgpot,    zdptop,   &
     &                     zcptop,    zcpcld,   zten,     &
     &                     ztentop,   ztencld,  zqen,     &
     &                     zqentop,   zqencld,  zqcen,    &
     &                     zqcetop,   zqcecld,  zimfac,   &
     &                     ztet,      zcoefh,             &
!    fluxes/tendencies 
     &                     pdifcq,    pdifcql,  pdifcqn,  &
     &                     pdifcs,    pfccql,   pfccqn    )
        endif 

!     =========================================================
!     6.  Determine cloud cover. This is needed not only 
!         in radiation scheme but also in the precipitation 
!         release routine 'hlprevap'. 
!     =========================================================

        call hlcloudcv(                                   &
!    scalars and logicals 
     &                     khor,      klev,     kstart,   &
     &                     kstop,     kstep,    pdtime,   &
     &                     pconacc,                       &
!    prognostic variables 
     &                     pt,        pq,       pcw,      &
     &                     pcice,     pqampli,            & 
!    diagnostic fields 
     &                     ptotcov,   ppf,                &
!    work arrays  
     &                     jlab,      jlabcv,             &
     &                     zrhuc,     zgpot,    ztc,      &
     &                     zqp,       zcwp,     zcicep,   & 
     &                     zqsat,     zqsatc,   zqac      )

!     =========================================================
!     7. Determine precipitation and associated evaporation
!        by calling subroutine 'hlprevap'.
!        The computations are based on forecast variables
!        updated due to condensation processes during the
!        current time step. 
!     =========================================================
!     Compute array factor 'zpresur' modifying precipitation release

      zsigint=0.02_jprb
      zaddpr =60.0_jprb

      do jk=1,klev
        zpresur(jk)=1.0_jprb
        if( pbhyb(kstart,jk) > 1.0_jprb -zsigint ) &
        & zpresur(jk)=1.0_jprb +  &
        & zaddpr*(zsigint -1.0_jprb +pbhyb(kstart,jk))/zsigint
        zpresur(jk)=max( zpresur(jk), 1.0_jprb )
      enddo

        call hlprevap (                                 & 
!    scalars and logicals 
     &                 khor,      klev,     kstep,      &
     &                 kstart,    kstop,    pdtime,     &
!    diagnostic variables 
     &                 ptotcov,   pomf,     pdpf,       &
     &                 pfrland,   pahyb,    pbhyb,      &
     &                 pqampli,                         & 
!    work arrays  
     &                 jlabcv,    jlab,                 &
     &                 zbu,       preta,    pdraindt,   &
     &                 pdsnowdt,  prcpcu,   prcpst,     & 
     &                 pcusnow,   pstsnow,  zpcoef,     &
     &                 zmr,       zdcusn,   zdstsn,     &
     &                 zpsp,      zldcp,    zbuca,      &
     &                 zqp,       ztp,      zcwp,       &
     &                 zcicep,    zqsat,    zdqsdt,     &
     &                 zpresur,                         &
!    fluxes/tendencies 
     &                 pfplcl,    pfplcn,               &
     &                 pfplsl,    pfplsn,   pfccql,     &
     &                 pfccqn,    pfcsql,   pfcsqn      )

!     ============================================================
!     8. For the case of HIRLAM compute final tendencies 
!        due to condensation, of relevant variables. 
!     ===========================================================

        do jk=1,klev
        do jl=kstart,kstop
          pdtdt(jl,jk)=(ztp(jl,jk) -pt(jl,jk))/pdtime        -  &
     &    pdtdtin(jl,jk) 
          pdqdt(jl,jk)=(zqp(jl,jk) -pq(jl,jk))/pdtime        -  &
     &    pdqdtin(jl,jk) 
          pdcwdt(jl,jk)=(zcwp(jl,jk) -pcw(jl,jk))/pdtime     -  &
     &    pdcwdin(jl,jk) 
          pdicedt(jl,jk)=(zcicep(jl,jk) -pcice(jl,jk))/pdtime - &
     &    pdicedin(jl,jk)            
        enddo 
        enddo 

!     ===========================================================
!     9. Second call to subroutine 'qcampli' to define
!        the time variation of total specific humidity  
!        within the grid box in the absence of penetrating
!        convection.
!     =======================================================

        call hlqcampli(                                   &
!    scalars and logicals 
     &                     khor,      klev,     kstart,   &
     &                     kstop,     2,        pdtime,   &
     &                     pdtheta,                       &
!    prognostic variables  
     &                     pq,        pcw,      pcice,    & 
     &                     pqampli,                       &
!    diagnostic fields 
     &                     pps,       ppf,                &
!    work arrays 
     &                     zqp,       zcwp,     zcicep,   &
     &                     zrhuc                          )

!     =======================================================
!     10. Diagnostic calculations  
!     =======================================================

!     10.1  Initialize arrays and local constants.
!     -------------------------------------------------------
!     'zrk1'  : work array
!     'zrk2'  : work array
!     'pcov2d' : horizontal total cloud cover field 
!               (dimensionless)
!     'pcwpath': verticallly integrated cloud water
!               (kg water/m2)
!     ---------------------------------------------------------
        do jl = kstart,kstop
          zrk1(jl) = 0._JPRB
          zrk2(jl) = 0._JPRB
          pcov2d(jl)= 1._JPRB
          pcwpath(jl)=0._JPRB
        enddo
!     ---------------------------------------------------------
!     10.2 compute total horizontal cloud cover in two steps
!         compute  vertically integrated cloud water.
!     ---------------------------------------------------------
        do jk=1,klev
          do jl=kstart,kstop
            if(jk > 1) then
               pcov2d(jl) = pcov2d(jl)*                   &
     &          (1._JPRB - max(ptotcov(jl,jk-1),          &
     &          ptotcov(jl,jk))) /                        & 
     &          (1._JPRB - min(ptotcov(jl,jk-1),0.99_JPRB ))
            endif

            pcwpath(jl)=pcwpath(jl) +pcw(jl,jk)*pdpf(jl,jk)/gravit 
          enddo 
        enddo 

!       Final horizontal field 'pcov2d' for total cloud cover

        do jl=kstart,kstop
          pcov2d(jl) = 1._JPRB - pcov2d(jl)
        enddo

!     -------------------------------------------------------
!     10.3 Compute remaining diagnostic variables. 
!     -------------------------------------------------------

        if (ldstat) then
           zwrk1=0._JPRB
           do jk = 1,klev
              zwrk2=0._JPRB
              zwrk3=0._JPRB
              zwrk4=0._JPRB
              zwrk5=0._JPRB
              zwrk6=0._JPRB
              zwrk7=0._JPRB           
              zwrk8=0._JPRB

              do jl = kstart,kstop
                 zwrk2 = zwrk2 + pdtdt(jl,jk)
                 zwrk3 = zwrk3 + pdtdt(jl,jk)*pdtdt(jl,jk)
                 zwrk4 = zwrk4 + pqampli(jl,jk)
                 zwrk5 = zwrk5 + ptotcov(jl,jk)
                 zwrk6 = zwrk6  +  zcwp(jl,jk)*pdpf(jl,jk)/gravit
!                 zwrk7 = zwrk7 + nint( pqampli(jl,jk)  + 0.499_JPRB)
                 zwrk8 = zwrk8 + nint( ptotcov(jl,jk) + 0.499_JPRB)
                 zrk2(jl) = max(zrk2(jl), pqampli(jl,jk))

                 if( (pqampli(jl,jk) > 0._JPRB) .and. &
      &              (zrk1(jl) == 0._JPRB) ) then
                    zrk1(jl) = 1._JPRB
                    zwrk1=zwrk1 +1._JPRB
                 endif
              enddo

              pstcon (jk) = pstcon (jk) + zwrk2
              pstdcon(jk) = pstdcon(jk) + zwrk3
              pstcov (jk) = pstcov (jk) + 0._JPRB
              pstdcov(jk) = pstdcov(jk) + 0._JPRB
              pstccov(jk) = pstccov(jk) + zwrk4
              pstscov(jk) = pstscov(jk) + zwrk5
              pstcw  (jk) = pstcw  (jk) + zwrk6
              pstcpnt(jk) = pstcpnt(jk) + zwrk7
              psttpnt(jk) = psttpnt(jk) + zwrk8
              pstscal(5)  = pstscal(5)  + zwrk8
           enddo

           do jl = kstart, kstop
              pstscal(1) = pstscal(1) + zrk2(jl)
              pstscal(2) = pstscal(2) + pcov2d(jl)
              pstscal(3) = pstscal(3) + pcwpath(jl)
              pstscal(6) = pstscal(6) + prcpcu(jl)
              pstscal(7) = pstscal(7) + prcpst(jl)
           enddo

           pstscal(4) = pstscal(4) + zwrk1
           pstscal(8) = pstscal(8) 
           pstscal(9) = pstscal(9) 
           pstscal(10)= pstscal(10)
           pstscal(11)= pstscal(11)
           pstscal(12)= pstscal(12)
        endif

!   --------------------------------------------------
    endif 
!   =============

    if (lhook) call dr_hook('hlconds',1,zhook_handle)

end subroutine hlconds
