! Library:phys $Id: hlprevap.F90 5696 2008-02-01 08:01:46Z ksa $

!options xopt(noeval)
 subroutine hlprevap (                                 &  
! scalars and logicals 
     &                 khor,      klev,     kstep,     &
     &                 kstart,    kstop,    pdtime,    &
! diagnostic variables 
     &                 ptotcov,   pomf,     pdpf,      &
     &                 pfrland,   pahyb,    pbhyb,     &
     &                 pqampli,                        &
! work arrays  
     &                 klabcv,    klab,                &
     &                 pzbu,      pzpreta,  pzdraindt, &
     &                 pzdsnowdt, pzprcu,   pzprst,    & 
     &                 pzcusno,   pzstsno,  pzpcoef,   &
     &                 pzmr,      pzdcusn,  pzdstsn,   &
     &                 pzpsp,     pzldcp,   pzbuca,    &
     &                 pzqp,      pztp,     pzcwp,     &
     &                 pzcicep,   pzqsat,   pzdqsdt,   &
     &                 ppresur,                        &
! fluxes/tendencies 
     &                 pfplcl,    pfplcn,              &
     &                 pfplsl,    pfplsn,   pfccql,    &
     &                 pfccqn,    pfcsql,   pfcsqn     )

!     ---------------------------------------------------------------
!     --------   
!     PURPOSE:   Compute precipitation release and the evaporation of
!     --------   precipitation. Also compute the associated
!                modifications of cloud water, specific humidity
!                and temperature.
!     ---------------------------------------------------------------
!     INTERFACE :   subroutine 'hlprevap' is called from
!     ------------  subroutine 'hlconds'
!     INPUT:
!     * Argument type = Unmodified input *
!     -----------------------------------------------------------------
!       khor          number of points in horizontal loops 
!       klev          number of vertical levels         
!       kstep         current time step number 
!       kstart        start index of horizontal loop       
!       kstop         end index of horizontal loop 
!       pdtime        time step for the process 
!       pahyb         first hybrid coordinate array 
!       pbhyb         second hybrid coordinate array
!       pdpf          delta-p around 'full' level
!       klabcv        is a convective label. Convective layers are
!                     first assigned a value of klabcv=2, and a final
!                     value of 3. Convective sub-cloud layers are
!                     assigned a value of 1. New lifting levels
!                     however, are assigned a value of 4.
!       klab          number of the convective entity in the vertical
!       ptotcov       fractional cloud cover field
!       pqampli       amplitude of humidity fluctuations
!       pomf          vertical velocity in the pressure system
!       pzbu          averarage buoyancy '(tc -t)/t)' in the
!                     convective 'entity'
!       pfrland       fraction of land in model grid.
!                     currently only 0 or 1
!       pzpsp         provisional value of surface pressure
!       ppresur       coefficient array to augment precip from fog
!     ---------------------------------------------------------------
!     OUTPUT:
!     * Argument type = new output or modified input *
!     ---------------------------------------------------------------
!       pzdraindt     rain intensity (kg/(m2 s))
!       pzdsnowdt     snow intensity (kg/(m2 s)) 
!       pzpreta       precipitation intensity at model levels  
!       pzprcu        convective rain intensity
!       pzprst        stratiform rain intensity
!       pzcusno       convective snow intensity
!       pzstsno       stratiform snow intensity
!       pzpcoef       precipitation coefficient 
!       pzmr          cloud condensate critical level      
!       pzdcusn       convective snow release  
!       pzdstsn       stratiform snow release
!       pzldcp        (1/cpair)*latent heat as a function of
!                     temperature by weighting in latent heat of
!                     freezing times probability of ice crystals
!       pzqsat        saturation mixing ratio with respect to
!                     gridpoint temperature
!       pzdqsdt        derivative of saturation water vapour 
!                     pressure with respect to temperature
!       pzbuca        is here used as work space for computing 
!                     maximum cloud cover above a given level, 
!                     used in the coalescence parameterization.
!       pzqp          provisional value of specific humidity
!       pztp          provisional value of temperature
!       pzcwp         provisional value of specific cloud water
!       pzcicep       provisional value of specific cloud ice 
!       pfccql       convective tendency for specific humidity 
!                    (normally negative)
!                    associated with the convective liquid water 
!                    condensation (disregarding precipitation)
!       pfccqn       convective tendency for specific humidity 
!                    (normally negative)
!                    associated with the convective ice condensation
!                    (disregarding precipitation)
!       pfcsql       Stratiform tendency for decreasing specific 
!                    humidity associated with cloud water formation  
!                    involving latent heat release but not
!                    precipitation.
!       pfcsqn       Stratiform tendency for decreasing specific
!                    humidity associated with cloud ice formation 
!                    involving latent heat release but not
!                    precipitation.
!       pfplcl       convective precipitation flux as rain
!       pfplcn       convective precipitation flux as snow
!       pfplsl       stratiform precipitation flux as rain 
!       pfplsn       stratiform precipitation flux as snow
!     ------------------------------------------------------------
!     EXTERNAL SUBROUTINES:    none
!     ------------------------------------------------------------

!     AUTHOR:         Bent H. Sass, (DMI), based on the research
!     -------         during HIRLAM III-V projects.

!     MODIFICATIONS: 
!     --------------
!     Original version for ALADIN/ARPEGE: 2005-12-29

!     ============================================================
!     1. Declarations. 
!     ============================================================

!     1.1 MODULES USED:

use parkind1  ,only : jpim     ,jprb
use yomhook   ,only : lhook,   dr_hook

use yhlconst  ,only : latvap,   cpair,   epsilo,   &
     &                gravit,   tmelt,   latice                 
use yhlcond   ,only : coales,   hccu,    hmrcu,    &
     &                hmrst,    cbfeff,  hcst,     &
     &                prbice,                      &
     &                hdewi,                       &
     &                hmroft,                      &
     &                dttabl

!     1.2 OTHER DECLARATIONS.

      implicit none                                                           
integer(kind=jpim), intent(in)     ::   khor
integer(kind=jpim), intent(in)     ::   klev
integer(kind=jpim), intent(in)     ::   kstep
integer(kind=jpim), intent(in)     ::   kstart
integer(kind=jpim), intent(in)     ::   kstop
real(kind=jprb)   , intent(in)     ::   pdtime 
real(kind=jprb)   , intent(in)     ::   pahyb(khor,klev+1)
real(kind=jprb)   , intent(in)     ::   pbhyb(khor,klev+1)
real(kind=jprb)   , intent(in)     ::   pdpf(khor,klev)
 
integer(kind=jpim), intent(in)     ::   klabcv(khor,klev) 
integer(kind=jpim), intent(in)     ::   klab(khor,klev)
 
real(kind=jprb)   , intent(in)     ::   ptotcov(khor,klev)
real(kind=jprb)   , intent(in)     ::   pqampli(khor,klev)
real(kind=jprb)   , intent(in)     ::   pfrland(khor)
real(kind=jprb)   , intent(in)     ::   pomf(khor,klev) 
 
real(kind=jprb)   , intent(in)     ::   pzbu(khor,klev) 
real(kind=jprb)   , intent(inout)  ::   pzpreta(khor,klev)
real(kind=jprb)   , intent(inout)  ::   pzdraindt(khor)   
real(kind=jprb)   , intent(inout)  ::   pzdsnowdt(khor)
real(kind=jprb)   , intent(inout)  ::   pzprcu(khor)
real(kind=jprb)   , intent(inout)  ::   pzprst(khor)
real(kind=jprb)   , intent(inout)  ::   pzcusno(khor)
real(kind=jprb)   , intent(inout)  ::   pzstsno(khor)
real(kind=jprb)   , intent(inout)  ::   pzpcoef(khor) 
real(kind=jprb)   , intent(inout)  ::   pzmr(khor)
real(kind=jprb)   , intent(inout)  ::   pzdcusn(khor)  
real(kind=jprb)   , intent(inout)  ::   pzdstsn(khor)
real(kind=jprb)   , intent(in)     ::   pzpsp(khor)
real(kind=jprb)   , intent(inout)  ::   pzldcp(khor,klev)
real(kind=jprb)   , intent(inout)  ::   pzbuca(khor,klev)
real(kind=jprb)   , intent(inout)  ::   pzqp(khor,klev)
real(kind=jprb)   , intent(inout)  ::   pztp(khor,klev) 
real(kind=jprb)   , intent(inout)  ::   pzcwp(khor,klev)
real(kind=jprb)   , intent(inout)  ::   pzcicep(khor,klev)
real(kind=jprb)   , intent(inout)  ::   pzqsat(khor,klev)
real(kind=jprb)   , intent(inout)  ::   pzdqsdt(khor)
real(kind=jprb)   , intent(inout)  ::   ppresur(klev)

real(kind=jprb)   , intent(inout)  ::   pfplcl(khor,0:klev) 
real(kind=jprb)   , intent(inout)  ::   pfplcn(khor,0:klev)
real(kind=jprb)   , intent(inout)  ::   pfplsl(khor,0:klev)
real(kind=jprb)   , intent(inout)  ::   pfplsn(khor,0:klev)
real(kind=jprb)   , intent(inout)  ::   pfccql(khor,0:klev)
real(kind=jprb)   , intent(inout)  ::   pfccqn(khor,0:klev)
real(kind=jprb)   , intent(inout)  ::   pfcsql(khor,0:klev)
real(kind=jprb)   , intent(inout)  ::   pfcsqn(khor,0:klev)

integer(kind=jpim)            ::  jk,  jl, jkp1, &
 &                                jintt, iklab,  &       
 &                                jkm1,  jkm2
 
real(kind=jprb)               :: zfa,     zfb,     zcwsca,  &
     &                           zese,    zpf,              &
     &                           zdpf,    zsec,    zsec1,   &
     &                           zecoef,  zcpi,    zdpdg,   &
     &                           zdqlim,  zpodif,  zsqrt,   &  
     &                           zevapc,  zevaps,  zfbf,    & 
     &                           zprbi,   zfcocv,  zfcost,  &
     &                           zftco,   z1,      zp1,     &
     &                           zresat,  zcevap1, zcevap2, &
     &                           zhccof,  zcwi,             &
     &                           zphase,  ztmp,    ztmp1,   &
     &                           ztmp2,   ztmp3,   ztmp4,   &
     &                           zcofom1, zcofbu,  zhc00,   &
     &                           zc1,     zc2
real(kind=jprb)  ::              zdcu,    zdst,    zprtcw,  &    
     &                           zprtci,  zhmcof,  zprtsw,  &
     &                           zprtsi,  zswi
real(kind=jprb)               :: zhmrcu,  zmr,     ztunco
real(kind=jprb)               :: zfrac,      zfrac1,   zidtime,  & 
     &                           zstreltot,  zcvreltot
real(kind=jprb)         :: zhook_handle
!     -------------------------------------------------------------
!     Include saturation table function 
 
#include "hlesat.h"
!     =============================================================
!     2. Initialize various constants and arrays.
!     =============================================================
!     2.1 local constants   
!     -------------------
!     'zcpi'    inverse of specific heat capacity
!     'zsec'    is a security constant.
!     'zhc00'   minimum precipitation basic release coefficient
!               realized during strong updraft conditions. 
!     'zecoef'  is an evaporation coefficient applied 
!               in computing evaporation of precipitation.
!     'zcevap1' evaporation coefficient.
!     'zcevap2' evaporation coefficient.
!     'zphase'  melting coefficient to describe melting 
!               of snow. 
!     'zcofom1' coefficient used to describe influence of 
!               resolved scale vertical velocity on
!               precipitation release.
!     'zcofbu'  coefficient used to describe influence
!               of air parcel buoyancy on precipitation  
!               release.
!     'ztunco'  tuning constant for convective precipitation 
!               release at low temperatures.   
!     'zidtime' inverse of current time step
!     -------------------------------------------------------
     if(lhook) call dr_hook('hlprevap',0,zhook_handle)

      zsec=1.e-10_jprb

!     NB: security constant tentatively 0.5 percent
      zsec1=0.005_jprb
      zecoef=1.30e-3_jprb
      zcevap1=700._jprb
      zcevap2=6.0E9_jprb
      zcpi=1./cpair 
      zphase=4.0e-4_jprb
      zcofbu=250._jprb
      ztunco=1.0_jprb
      zcofom1=0.010_jprb
      zhc00=1.0e-5_jprb
      zcwsca=1.0e5_jprb
      zmr=0.00175_jprb
      zfrac1=1.0_jprb/(tmelt*tmelt)
      zidtime=1._jprb/pdtime

!     -------------------------------------------------------------
!     2.2 arrays:
!     -------------------------------------------------------------
      do jl=kstart,kstop

!     Initialize precipitation intensity arrays and
!     max cloud array (pzbuca) to zero.
!     -------------------------------------------------------------
 
        pzprcu(jl)   = 0._jprb
        pzprst(jl)   = 0._jprb
        pzcusno(jl)  = 0._jprb
        pzstsno(jl)  = 0._jprb
 
        pzbuca(jl,1) = zsec1
        pzbuca(jl,2) = zsec1
         
!       flux of precipitation  initialized to zero at the top.       
 
        pfplcl(jl,0)= 0._JPRB
        pfplcn(jl,0)= 0._JPRB
        pfplsl(jl,0)= 0._JPRB
        pfplsn(jl,0)= 0._JPRB
 
      enddo 
!     -------------------------------------------------------------
!     Preparative computations  
 
        do jk=3,klev
          jkm1=jk-1
          jkm2=jk-2
          do jl=kstart,kstop 
            pzbuca(jl,jk)=pzbuca(jl,jkm1)
            if( ptotcov(jl,jkm1) > ptotcov(jl,jkm2) ) then
              pzbuca(jl,jk)=ptotcov(jl,jkm1)
            endif 
          enddo 
        enddo 
 
        do jk=1,klev
          do jl=kstart,kstop
            pzpreta(jl,jk)=0.0_jprb
          enddo 
        enddo 
 
!     ===================================================
!       Start main loop to compute precipitation release
!       and evaporation of precipitation.
!     ---------------------------------------------------
 
      do jk=1,klev
 
!     --------------------
        jkp1=jk+1
        do jl=kstart,kstop
!       ------------------
  
             pzprst(jl)=max( pzprst(jl), 0._jprb )
             pzprcu(jl)=max( pzprcu(jl), 0._jprb )
   
!         3. Evaporation of precipitation
!         ===================================================
!         3.1 compute help parameters such as
!             pressure in middle of a layer (zpf) 
!             pressure thickness (zdpf), 
!             thickness in terms of mass (zdpdg)
!             latent heat divided with heat capacity (wldcp)
!             saturation specific humidity (wqsat)
!             the derivative of 'wqsat' with respect 
!             to temperature (wdqsdt)  
!         ----------------------------------------------------------

          zpf=0.5_jprb*( pahyb(jl,jk) +pahyb(jl,jkp1) +       &
     &           (pbhyb(jl,jk) +pbhyb(jl,jkp1))*pzpsp(jl) )
          zdpf=pahyb(jl,jkp1) -pahyb(jl,jk) + & 
     &         pzpsp(jl)*(pbhyb(jl,jkp1) -pbhyb(jl,jk))
 
          zdpdg=zdpf/gravit    
 
          jintt = max(1, 1 + int((tmelt - pztp(jl,jk))/dttabl))
          jintt = min(jintt, 1750)
  
          zfa=(pzcwp(jl,jk) +pzcicep(jl,jk))*zcwsca
          zfb=max( 0._jprb , min( zfa,1._jprb) )
 
          zese=zfb*( esatw(pztp(jl,jk))*(1._jprb -prbice(jintt)) + &
     &               esati(pztp(jl,jk))*prbice(jintt) ) +          &
     &               (1._jprb -zfb)*esatw(pztp(jl,jk))
  
          pzldcp(jl,jk) =(latvap + latice*  &
     &                  prbice(jintt))/cpair
 
          zresat=1./pecor(zpf,pztp(jl,jk))
          pzqsat(jl,jk)=epsilo*zese*zresat
          pzdqsdt(jl)=epsilo*zpf*desdt(pztp(jl,jk))*    &
     &               zresat*zresat

!         =========================================================
!         3.2 Convective evaporation and stratiform evaporation are
!             treated separately. Two constraints should apply:

!         i)  First,  the evaporation in a layer must not exceed
!             the precipitation passing through the layer in a
!             given time step.
!         ii) Second, the evaporation must not exceed the satu-
!             ration deficit ( wqsat -wqp )
!         ---------------------------------------------------------
!             Update specific humidity and temperature
!             as a result of evaporation of convective
!             precipitation.
!         ---------------------------------------------------------

          zpodif=max( pzqsat(jl,jk) -pzqp(jl,jk), zsec )/   &
     &           (1._jprb +pzldcp(jl,jk)*pzdqsdt(jl) )
 
          zsqrt= zcevap1*( pzprcu(jl) +pzcusno(jl) )/ &
     &    (1._jprb +zcevap2*( pzprcu(jl)*pzprcu(jl) +  &
     &    pzcusno(jl)*pzcusno(jl) +2.0_jprb*pzprcu(jl)*pzcusno(jl) )) + &
     &    sqrt( pzprcu(jl) +pzcusno(jl) +zsec )

          zfrac=pztp(jl,jk)*pztp(jl,jk)*zfrac1
          zevapc=pdtime*zecoef*min( zfrac, 1.0_jprb )*zsqrt*zpodif
          zdqlim=(pzprcu(jl) +pzcusno(jl))*pdtime*gravit/pdpf(jl,jk)
          zevapc=min( zevapc,zdqlim )
          zevapc=min( zevapc,zpodif )

!         updates 

          pfccql(jl,jk)=pfccql(jl,jk) + zevapc*zidtime*  &
     &    pzprcu(jl)/(pzprcu(jl) +pzcusno(jl) +zsec)
          pfccqn(jl,jk)=pfccqn(jl,jk) + zevapc*zidtime*  &
     &    pzcusno(jl)/(pzprcu(jl) +pzcusno(jl) +zsec)
  
          pzqp(jl,jk)=pzqp(jl,jk) +zevapc
          pztp(jl,jk)=pztp(jl,jk) -zevapc*  &
     &    zcpi*(latvap +latice*pzcusno(jl)/ &
     &    (pzprcu(jl) +pzcusno(jl) +zsec) )

!         --------------------------------------------------------
!             Update specific humidity and temperature
!             as a result of evaporation of stratiform
!             precipitation.
!         --------------------------------------------------------

          zpodif=max( pzqsat(jl,jk) -pzqp(jl,jk), zsec )/   &
     &            (1._jprb +pzldcp(jl,jk)*pzdqsdt(jl) )
          zsqrt=zcevap1*(pzprst(jl) + pzstsno(jl))/   &
     &    (1._jprb +zcevap2*(pzprst(jl)*pzprst(jl) +  &
     &    pzstsno(jl)*pzstsno(jl) +2.0_jprb*pzprst(jl)*pzstsno(jl)) ) + &
     &    sqrt(pzprst(jl) +pzstsno(jl) +zsec)

          zfrac=pztp(jl,jk)*pztp(jl,jk)*zfrac1
          zevaps=pdtime*zecoef*min( zfrac, 1.0_jprb )*zsqrt*zpodif
          zdqlim=(pzprst(jl) +pzstsno(jl))*pdtime*gravit/pdpf(jl,jk)
          zevaps=min( zevaps,zdqlim )
          zevaps=min( zevaps,zpodif )

!         tendency updates for further processing later

          pfcsql(jl,jk)=pfcsql(jl,jk) +1.0_jprb*zevaps*zidtime*  &
     &    pzprst(jl)/(pzprst(jl) +pzstsno(jl) +zsec)
          pfcsqn(jl,jk)=pfcsqn(jl,jk) +1.0_jprb*zevaps*zidtime*  &
     &    pzstsno(jl)/(pzprst(jl) +pzstsno(jl) +zsec)

          pzqp(jl,jk)=pzqp(jl,jk) +zevaps
          pztp(jl,jk)=pztp(jl,jk) -zevaps*              &
     &               zcpi*(latvap +latice*pzstsno(jl)/ &
     &               (pzprst(jl) +pzstsno(jl) +zsec))

!         -------------------------------------------------------
!         3.3 Update convective precipitation due to evaporation
!         -------------------------------------------------------
          zc1=zdpdg*zevapc/pdtime
          pzprcu(jl)=pzprcu(jl) -zc1*  &
     &    pzprcu(jl)/(pzprcu(jl) +pzcusno(jl) +zsec)   
          pzprcu(jl)=max( pzprcu(jl),0.00_jprb )

          pzcusno(jl)=pzcusno(jl) -zc1*  &
     &    pzcusno(jl)/(pzprcu(jl) +pzcusno(jl) +zsec)
          pzcusno(jl)=max( pzcusno(jl), 0._JPRB )

!         -------------------------------------------------------
!         3.4 Update stratiform precipitation due to evaporation
!         -------------------------------------------------------
          zc2=zdpdg*zevaps/pdtime
          pzprst(jl)=pzprst(jl) -1.0_jprb*zc2*  &
     &    pzprst(jl)/(pzprst(jl) +pzstsno(jl) +zsec) 
          pzprst(jl)=max( pzprst(jl),0.00_jprb )    

          pzstsno(jl)=pzstsno(jl) -1.0_jprb*zc2*   &
     &    pzstsno(jl)/(pzprst(jl) +pzstsno(jl) +zsec) 
          pzstsno(jl)=max( pzstsno(jl), 0.00_jprb ) 

!         =======================================================
!         4. Precipitation release 
!         ---------------------------------------------------------
!         4.1 modified ice probability 'zprbi'
!             (eq.2.2.3.20) in HIRLAM-2.5 documentation )
!         --------------------------------------------------------- 

          zprbi=prbice(jintt) +(1._jprb -prbice(jintt))*  &
     &    (pzcusno(jl) +pzstsno(jl))/ &
     &    (pzprcu(jl) +pzcusno(jl) +pzprst(jl) +pzstsno(jl) +zsec)
          zprbi=max( 0.0_jprb, min( zprbi,1.0_jprb ) )
          zfbf=zprbi*(1._jprb -prbice(jintt))*hdewi(jintt)

!         ---------------------------------------------------------
!         4.2 extra temperature dependency term 'zftco'
!             (eq. 2.2.3.25)
!         ---------------------------------------------------------
          zftco=0.0_jprb
          if( pztp(jl,jk) <= 273.15_jprb )  &
     &    zftco=0.231_jprb*(273.15_jprb -pztp(jl,jk))
          if( pztp(jl,jk) <= 260.15_jprb )  &
     &    zftco=3.0_jprb + 0.666_jprb*(260.15_jprb -pztp(jl,jk))
          if( pztp(jl,jk) <= 230.15_jprb ) zftco=23.0_jprb

!         ---------------------------------------------------------
!         Convective case
!         ----------------
!         'active' convection with positive moisture accesion 
!                   (klabcv=3)
!         'passive' convection with non-positive moisture 
!         accession (klabcv=2) 

          zhmrcu=pfrland(jl)*(hmrcu +zmr)  + &
    &     (1. -pfrland(jl)) *(hmrcu -zmr)


!         =========================================================
          if( (klabcv(jl,jk) == 3).or.(klabcv(jl,jk) == 2) ) then
!         =========================================================

!         4.3.0 time variable precipitation coefficients 

          iklab=klab(jl,jk)
          ztmp2=zcofbu*max( pzbu(jl,iklab), zsec )   
          ztmp1=max( -zcofom1*pomf(jl,jk), 0._jprb ) +ztmp2
          ztmp=min( ztmp1, 1._JPRB )
          ztmp4=ztmp1 -ztmp2
          ztmp3=min( ztmp4, 1._jprb )
          zhmcof=hmrst +ztmp*( zhmrcu -hmrst )
          zhccof=hccu +ztmp3*( zhc00 -hccu )

!         4.3 take in to account collection effect and
!             Bergeron-Findeisen effect in parameter 'zfcocv'
!             (eq. 2.2.3.15) in HIRLAM-2.5 documentation )
!         ---------------------------------------------------------

            zfcocv=1._jprb + &
  &                coales*sqrt( (pzprcu(jl) +pzcusno(jl) +zsec)/ &
  &                (pzbuca(jl,jk) +zsec1) )  +cbfeff*zfbf

!         4.4 compute cloud water humidity threshold 'pzmr' 
!             for convective precipitation release (eq. 2.2.3.22)
!         ---------------------------------------------------------
            pzmr(jl)=zhmcof*( hmroft(jintt)/(zfcocv +zsec) )

!         ---------------------------------------------------------
!         4.5 precipitation coefficient 'pzpcoef' (eq. 2.2.3.14)     
!         ---------------------------------------------------------

            pzpcoef(jl)=ppresur(jk)*zhccof*zfcocv* &
          &           (1.0_jprb +zftco*ztunco)
!         --------------------------------------------------------
!         4.6 release of convective precipitation in the layer.
!         -------------------------------------------------------

            zcwi=pzcwp(jl,jk) +pzcicep(jl,jk)
            z1=zcwi/((zsec1 +ptotcov(jl,jk))*pzmr(jl))
            zp1=pdtime*pzpcoef(jl)*(1._jprb -exp( -z1*z1 ))
            zp1=max( zp1, 0.0_jprb )

            pzcwp(jl,jk)=(1.0_jprb -prbice(jintt))*zcwi/(1._jprb +zp1)
            pzcicep(jl,jk)=prbice(jintt)*zcwi/(1._jprb +zp1)
 
            zcvreltot=zcwi*(1.0_jprb - 1.0_jprb/(1._jprb +zp1))
            zprtcw=(1.0_jprb -zprbi)*zcvreltot
            zprtci=zprbi*zcvreltot

            pzprcu(jl)=pzprcu(jl)   + zprtcw*zdpdg/pdtime
            pzcusno(jl)=pzcusno(jl) + zprtci*zdpdg/pdtime

!         -------------------------------------------------------
!         4.8 Release heat due to precipitation induced freezing
!             of convective precipitation
!         ------------------------------------------------------
           
          zdcu=zcvreltot*(zprbi -prbice(jintt))

          pfccql(jl,jk)=pfccql(jl,jk) +zidtime*zdcu 
          pfccqn(jl,jk)=pfccqn(jl,jk) -zidtime*zdcu 

          pztp(jl,jk)=pztp(jl,jk) + latice*zcpi*zdcu 
!         --------------------------------------------------------
          else
!         ========================================================
!         4.9.0 time variable precipitation coefficients 
!               ( the precipitation release below simulates 
!                 suspension of precipitation which is treated
!                 from the vertical velocity term and the resolved
!                 scale precipitation resembles convection for 
!                 resolved vertical motion of a few metres per 
!                 second ) 

            ztmp1=-zcofom1*pomf(jl,jk)
            ztmp=max( 0._jprb, min( ztmp1,1._JPRB) )
  
            zhmcof=hmrst +ztmp*( zhmrcu -hmrst )
            zhccof=hcst + ztmp*( zhc00 -hcst )

!         4.9.1 take in to account collection effect and
!              Bergeron-Findeisen effect in parameter 'zfcost'
!              (eq. 2.2.3.15) in HIRLAM-2.5 documentation )
!         ---------------------------------------------------------

            zfcost=1._jprb +coales* &
     &             sqrt( (pzprst(jl) +pzstsno(jl) +zsec)/  &
     &             (pzbuca(jl,jk) +zsec1) )  +cbfeff*zfbf          

!         4.9.2 Compute cloud water humidity threshold 'wmr'
!             for stratiform precipitation release (eq. 2.2.3.22)
!         ---------------------------------------------------------
            pzmr(jl)=zhmcof*( hmroft(jintt)/(zfcost +zsec) )
!         ---------------------------------------------------------
!         4.10 Precipitation coefficient 'pzpcoef' (eq. 2.2.3.14)
!         ---------------------------------------------------------
            pzpcoef(jl)=ppresur(jk)*zhccof*zfcost*(1.0_jprb +zftco)
!         ---------------------------------------------------------
!         4.11 Release of stratiform precipitation in the layer.
!         ---------------------------------------------------------

            zswi=pzcwp(jl,jk) +pzcicep(jl,jk)
            z1=zswi/((zsec1 +ptotcov(jl,jk))*pzmr(jl))  
            zp1=pdtime*pzpcoef(jl)*(1._jprb -exp( -z1*z1 ))
            zp1=max( zp1, 0.0_jprb )

            pzcwp(jl,jk)=(1.0_jprb -prbice(jintt))*zswi/(1._jprb +zp1)
            pzcicep(jl,jk)=prbice(jintt)*zswi/(1._jprb +zp1)

            zstreltot=zswi*(1.0_jprb - 1.0_jprb/(1._jprb +zp1))
            zprtsw=(1.0_jprb -zprbi)*zstreltot
            zprtsi=zprbi*zstreltot

            pzprst(jl)=pzprst(jl)   + zprtsw*zdpdg/pdtime
            pzstsno(jl)=pzstsno(jl) + zprtsi*zdpdg/pdtime

!         --------------------------------------------------------
!         4.13 Release heat due to precipitation induced freezing
!              of stratiform precipitation
!         --------------------------------------------------------

            zdst=zstreltot*(zprbi -prbice(jintt))
            pfcsql(jl,jk)=pfcsql(jl,jk) + 1.0_jprb*zidtime*zdst 
            pfcsqn(jl,jk)=pfcsqn(jl,jk) - 1.0_jprb*zidtime*zdst
  
            pztp(jl,jk)=pztp(jl,jk) + latice*zcpi*zdst 
 
!         =====
          endif  
!         =====

!         -------------------------------------------------------
!         5.  Adjust snow flux due to melting of precipitation 
!             and release associated latent heat 
!             Note that the case of re-freezing rain 
!             is not considered at present. 
!         -------------------------------------------------------

          pzdcusn(jl)=zphase*cpair*zdpdg*  &
     &                max( pztp(jl,jk) -tmelt, 0.0_jprb )/latice
          pzdcusn(jl)=-min( pzdcusn(jl), pzcusno(jl) ) 
          pzcusno(jl)=pzcusno(jl) +pzdcusn(jl) 
          pzprcu(jl) =pzprcu(jl)  -pzdcusn(jl)

!   changed sign 050523 
          pfccql(jl,jk)=pfccql(jl,jk) +pzdcusn(jl)/zdpdg
          pfccqn(jl,jk)=pfccqn(jl,jk) -pzdcusn(jl)/zdpdg
  
          pztp(jl,jk)=pztp(jl,jk) +pdtime*latice*zcpi*  &
     &               pzdcusn(jl)/zdpdg 
 
          pzdstsn(jl)=zphase*cpair*zdpdg*  &
     &                max( pztp(jl,jk) -tmelt, 0.0_jprb )/latice
          pzdstsn(jl)=-min( pzdstsn(jl), pzstsno(jl) )
          pzstsno(jl)=pzstsno(jl) +1.0_jprb*pzdstsn(jl)
          pzprst(jl) =pzprst(jl)  -1.0_jprb*pzdstsn(jl)

! Check the signs of the next two lines !!!! 
! The sign has been changed 050523 (note that wdstsn is negative !)

          pfcsql(jl,jk)=pfcsql(jl,jk) +1.0_jprb*pzdstsn(jl)/zdpdg
          pfcsqn(jl,jk)=pfcsqn(jl,jk) -1.0_jprb*pzdstsn(jl)/zdpdg
  
          pztp(jl,jk)=pztp(jl,jk) +pdtime*latice*zcpi*  &
     &               pzdstsn(jl)/zdpdg

!     ------------------------------------------------------------
!       Precipitation flux at a model level, for output

        pzpreta(jl,jk)=pzprst(jl) + pzprcu(jl) +pzstsno(jl) +pzcusno(jl)

!       Final computation of precipitation fluxes at half levels

        pfplcl(jl,jk)=pzprcu(jl) 
        pfplcn(jl,jk)=pzcusno(jl)
        pfplsl(jl,jk)=pzprst(jl) 
        pfplsn(jl,jk)=pzstsno(jl)

!       -----
        enddo
!       -----
      enddo
!     -----

!     =============================================================
!     6. Update rain intensity 'pzdraindt' 
!        and snow intensity 'pzdsnowdt',
!     =============================================================

      do jl=kstart,kstop
!     ------------------

        pzdraindt(jl)=pzprcu(jl)  +pzprst(jl)
        pzdsnowdt(jl)=pzstsno(jl) +pzcusno(jl)

      enddo 
!     -----

     if (lhook) call dr_hook('hlprevap',1,zhook_handle)
end subroutine hlprevap
