!OPTIONS XOPT(NOEVAL)
SUBROUTINE HLRAD(                                                      &
     &   KSTEP,    KLON,    KLEV,   KIDIA,  KFDIA, PVETAF,  PVETAH,    &
     &  PGELAM,   PGEMU,    PALB,   PEMIS,    PTS,     PT,      PQ,    &
     &     PQL,     PQI,  PAPRSF,   PAPRS,   PLSM,  PDELP,      PA,    &
     &    PMU0, PFRTHDS, PFRSOFS, PFRSODS,  PFRTH,  PFRSO,  PTENDT     &
     &               )

!     Purpose:
!     --------
!     *HLRAD* : An interface to plug the HIRLAM's radiation scheme HLRADIA
!               in ARPEGE/IFS/ALADIN environment.

!     Interface:
!     ----------

!     - Input arguments:
!     ------------------
!     KSTEP   : TIME STEP NUMBER.
!     KLON    : HORIZONTAL DIMENSION.
!     KLEV    : END OF VERTICAL LOOP AND VERTICAL DIMENSION.
!     KIDIA,
!     KFDIA   : START/END OF HORIZONTAL LOOP.
!     PVETAF  : HYBRID LEVELS (full)
!     PVETAH  : HYBRID LEVELS (half)
!     PGELAM  : GEOGRAPHICAL LONGITUDE
!     PGEMU   : SINE OF LATITUDE
!     PALB    : MODEL SURFACE SHORTWAVE ALBEDO.
!     PEMIS   : MODEL SURFACE LONGWAVE EMISSIVITY.
!     PTS     : SURFACE LAYER TEMPERATURE.
!     PT      : TEMPERATURE 
!     PQ      : SPECIFIC HUMIDITY OF WATER VAPOUR.
!     PQL     : LIQUID WATER.
!     PQI     : ICE
!     PAPRSF  : PRESSURE ON FULL LEVELS.
!     PAPRS   : PRESSURE ON HALF-LEVELS.
!     PLSM    : LAND/SEA MASK.
!     PDELP   : LAYER THICKNESS IN PRESSURE UNITS.
!     PA      : 3-D CLOUD COVER
!     PMU0    : LOCAL COSINE OF INSTANTANEOUS SOLAR ZENITH ANGLE.

!     - Input/Output arguments:
!     -------------------------
!     PFRTHDS : SURFACE DOWNWARDS IR FLUX

!     - Output arguments:
!     -------------------
!     PFRSOFS : SURFACE DOWNWARDS DIFFUSE SOLAR FLUX
!     PFRSODS : SURFACE DOWNWARDS SOLAR FLUX 
!     PFRTH   : LONGWAVE RADIATIVE NET FLUX. 
!     PFRSO   : SHORTWAVE RADIATIVE NET FLUX.
!     PTENDT  : TOTAL TEMPERATURE TENDENCY UPDATED WITH RADIATIVE PROCESSES

!     Externals:
!     ----------
!     *HLCLDIAG*
!     *HLRADIA*

!     Method: 
!     -------
!     i)   Initialize the remaing part of the module YHLRAD      
!     ii)  Calculate parameters, which are needed in HLRADIA
!     iii) Call HLRADIA

!     Reference:
!     ----------

!     Author:
!     -------
!       07-Jun-2005  S.Niemela     *FMI*

!     Modifications: 
!     --------------
!       28-Sep-2005  S.Niemela     Renaming of HIRLAM subroutines and modules
!       04-Oct-2005  S.Niemela     HLCLOUD replaced by HLCLDIAG.
!       11-May-2007  S.Niemela     Interface updated to HIRLAM 7.1 physics

!     End modifications

!     ------------------------------------------------------
!     ALADIN/ARPEGE       - HIRLAM CORRESPONDENCE

!     KSTEP               - kstep
!     KLON                - nhor
!     KLEV                - nlev
!     KIDIA               - kstart
!     KFDIA               - kstop
!     PVETAF              - hybf
!     PVETAH(0:KLEV)      - hybh(nlev+1)
!     PGELAM              - along
!     SQRT(1-PGEMU*PGEMU) - coslat
!     PGEMU               - sinlat
!     PALB                - albedo
!     PEMIS               - emsurf
!     PTS                 - tsk 
!     PT                  - t
!     PQ                  - q
!     PQL + PQI           - cw (combined water+ice) ???
!     PAPRSF              - pf
!     PAPRS(KLON,0:KLON)  - ph(nhor,nlev+1)
!     PLSM                - frland
!     PDELP               - dpf
!     PA (from straco)    - totcov
!     PMU0                - scos
!     PFRTHDS (inout)     - slwdn (inout)
!     PFRSOFS             - sswdf
!     PFRSODS             - sswdn
!     PFRTH(KLON,0:KLEV)  - lwnet(nhor,nlev+1)
!     PFRSO(KLON,0:KLEV)  - swnet(nhor,nlev+1)

!     LCONDWT             - locw


!     Detailed comments are given in APLPAR(H) and HLRADIA.
!     --------------------------------------------------------

! MODULES

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY    ,ONLY : LCONDWT
! Solar radiative flux on top of the atmosphere
USE YOMPHY3   ,ONLY : RII0

USE YHLOPTION ,ONLY : LHLTEND
USE YHLRAD    ,ONLY : COSMIN


IMPLICIT NONE

! INPUT/OUTPUT PARAMETERS

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 

REAL(KIND=JPRB),INTENT(IN)       :: PVETAH(0:KLEV)
REAL(KIND=JPRB),INTENT(IN)       :: PVETAF(KLEV)
REAL(KIND=JPRB),INTENT(IN)       :: PGELAM(KLON) 
REAL(KIND=JPRB),INTENT(IN)       :: PGEMU(KLON) 
REAL(KIND=JPRB),INTENT(IN)       :: PALB(KLON) 
REAL(KIND=JPRB),INTENT(IN)       :: PEMIS(KLON) 
REAL(KIND=JPRB),INTENT(IN)       :: PTS(KLON) 
REAL(KIND=JPRB),INTENT(IN)       :: PT(KLON,KLEV) 
REAL(KIND=JPRB),INTENT(IN)       :: PQ(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)       :: PQL(KLON,KLEV) 
REAL(KIND=JPRB),INTENT(IN)       :: PQI(KLON,KLEV) 
REAL(KIND=JPRB),INTENT(IN)       :: PAPRS(KLON,0:KLEV) 
REAL(KIND=JPRB),INTENT(IN)       :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB),INTENT(IN)       :: PLSM(KLON) 
REAL(KIND=JPRB),INTENT(IN)       :: PDELP(KLON,KLEV) 
REAL(KIND=JPRB),INTENT(IN)       :: PA(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)       :: PMU0(KLON) 

REAL(KIND=JPRB),INTENT(INOUT)    :: PFRTHDS(KLON)
REAL(KIND=JPRB),INTENT(OUT)      :: PFRSOFS(KLON) 
REAL(KIND=JPRB),INTENT(OUT)      :: PFRSODS(KLON) 
REAL(KIND=JPRB),INTENT(OUT)      :: PFRTH(KLON,0:KLEV) 
REAL(KIND=JPRB),INTENT(OUT)      :: PFRSO(KLON,0:KLEV) 

REAL(KIND=JPRB),INTENT(INOUT)    :: PTENDT(KLON,KLEV)

! LOCAL VARIABLES

REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB) :: zclq(KLON,KLEV) 
REAL(KIND=JPRB) :: zcice(KLON,KLEV) 
REAL(KIND=JPRB) :: zsalbcor(KLON)
REAL(KIND=JPRB) :: zdtdt(KLON,KLEV)
REAL(KIND=JPRB) :: zstrad(KLEV)
REAL(KIND=JPRB) :: zstdrad(KLEV)
REAL(KIND=JPRB) :: zstclo(KLEV)
REAL(KIND=JPRB) :: zstdclo(KLEV)
REAL(KIND=JPRB) :: zslwnet(KLON)
REAL(KIND=JPRB) :: zsswnet(KLON)
REAL(KIND=JPRB) :: ztlwnet(KLON)
REAL(KIND=JPRB) :: ztswnet(KLON)
REAL(KIND=JPRB) :: ztswdn(KLON)
REAL(KIND=JPRB) :: ztotcov(KLON,KLEV)

! Dummy argument for HLCLDIAG
REAL(KIND=JPRB) :: zcw(KLON,KLEV) 

LOGICAL            :: llstat

INTEGER(KIND=JPIM) :: jl,jk

#include "hlcldiag.intfb.h"
#include "hlradia.intfb.h"

IF (LHOOK) CALL DR_HOOK('HLRAD',0,ZHOOK_HANDLE)

! Prepare cloud liquid, cloud ice and total cloud
! cover information for the radiation scheme.
! -----------------------------------------------
IF( KSTEP <= 2 ) THEN

  CALL HLCLDIAG( KLON   , KLEV   , KIDIA  , KFDIA, PT,     &
       &         PQ     , zcw    , PAPRSF , PDELP, PVETAF, &
       &         .FALSE., .FALSE., .FALSE.,                &
!   -- IN-/OUTPUT --
       &         ztotcov, zcice  ,                         &
!      -- OUTPUT -- 
       &         zclq)

ELSE

  IF (.NOT.LCONDWT) THEN
    CALL HLCLDIAG( KLON   , KLEV   , KIDIA  , KFDIA, PT,     &
     &             PQ     , zcw    , PAPRSF , PDELP, PVETAF, &
     &             LCONDWT, LCONDWT, LCONDWT,                &
!   -- IN-/OUTPUT --
     &             ztotcov, zcice  ,                         &
!    -- OUTPUT -- 
     &             zclq)
  ELSE
    DO jk=1,KLEV
      DO jl=KIDIA,KFDIA
        ztotcov(jl,jk) = PA(jl,jk)
      ENDDO
    ENDDO
  ENDIF

ENDIF 

! Calculate/define rest of the input parameters.
! ----------------------------------------------

! No statistics
llstat=.FALSE.

! Security check
IF (.NOT.LCONDWT .OR. (KSTEP <= 2) ) THEN
  DO jk=1,KLEV
   DO jl=KIDIA,KFDIA
      zclq(jl,jk) = MAX(zclq(jl,jk), 0.0_JPRB)
      zcice(jl,jk) = MAX(zcice(jl,jk), 0.0_JPRB)
   ENDDO
  ENDDO
ELSE
  DO jk=1,KLEV
   DO jl=KIDIA,KFDIA
      zclq(jl,jk) = MAX(PQL(jl,jk), 0.0_JPRB)
      zcice(jl,jk) = MAX(PQI(jl,jk), 0.0_JPRB)
   ENDDO
  ENDDO
ENDIF

!     Solar angle correction to surface alb 
DO jl = KIDIA,KFDIA
  zsalbcor(jl)=0.2/(1.+MAX(PMU0(jl),COSMIN)) - 0.12
ENDDO

! Call subroutine HLRADIA

CALL HLRADIA(  KLON    , KLEV    , KIDIA   , KFDIA  , PVETAF  ,  &
     &         PVETAH  , RII0    , PMU0    , PALB   , PEMIS   ,  &
     &         PLSM    , PTS     , PT      , PQ     , zclq    ,  &
     &         zcice   , PAPRSF  , PAPRS   , PDELP  , ztotcov ,  &
     &         llstat  , zsalbcor,                               &
!    -- INPUT/OUTPUT --
     &         zstrad  , zstdrad , zstclo  , zstdclo, PFRTHDS ,  &
!    -- OUTPUT --
     &         PFRSOFS ,                                         &
     &         PFRSODS , zslwnet , zsswnet , ztswdn , ztlwnet ,  &
     &         ztswnet , PFRTH   , PFRSO   , zdtdt )

  IF( LHLTEND ) THEN 
    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        PTENDT(JL,JK)=PTENDT(JL,JK) + zdtdt(jl,jk)
      ENDDO
    ENDDO
  ENDIF  

IF (LHOOK) CALL DR_HOOK('HLRAD',1,ZHOOK_HANDLE)
END SUBROUTINE HLRAD
