!options xopt(noeval)
subroutine hlstraco(                             &
!     scalars and logicals 
     &              kcall,    khor,    klev,     &
     &              kstart,   kstop,   kstep,    &
     &              pdtime,   ldstat,            &
!     prognostic arrays
     &              pu,       pv,      pt,       & 
     &              pq,       pcw,     pcice,    &
     &              pqampli,  ptke,              &
!     diagnostic arrays   
     &              ptotcov,  pomf,    pfrland,  &
     &              pdqdtin,  pdcwdin, pdicedin, & 
     &              pdtdtin,  pf,      phalf,    &
     &              ptendt,   ptendq,   ptendql, &
     &              ptendqi,                     &
!     fluxes 
     &              pdiftq,   pdiftql, pdiftqn,  & 
     &              pdifts,   pdifcq,  pdifcql,  & 
     &              pdifcqn,  pdifcs,  pfccql,   &    
     &              pfccqn,   pfcsql,  pfcsqn,   &
     &              pfplcl,   pfplcn,  pfplsl,   &
     &              pfplsn,                      &
!     closing 
     &              cdlock                       )
!     ======================================================
!     PURPOSE:

!     Subroutine to compute fluxes or tendencies 
!     due to condensation and evaporation/sublimation 
!     processes in the atmosphere. 
!     STRACO (Soft TRAnsition COndenstion)
!     implies that soft transitions between convective
!     and stratiform type of conditions are treated 
!     by the scheme.

!     METHOD: 
!     ------------------------------------------------------
!     The implementation of STRACO in ARPEGE/ALADIN  
!     environment imposes changes compared to the 
!     original implementation in HIRLAM. This is due to 

!     i)   application of fluxes instead of tendencies
!     ii)  use of enthalpy instead of temperature
!     iii) the order of physical parameterization calls
!          is different between HIRLAM and ARPEGE. 
!          As a consequence the heating effects 
!          (enthalpy fluxes) are involved in the  
!          convection scheme of ARPEGE. To be consistent,
!          the output enthalpy flux PDIFCS from ACONDS
!          (the STRACO scheme) will include enthalpy 
!          flux.

!     Remark:

!     A new pseudo-prognostic array `qampli( , )' is 
!     stored at the field location `pqva' ("QVA"-pointer)

!     ------------------------------------------------------
!     AUTHOR: Bent H. Sass, DMI.

!     MODIFICATIONS: 
!     Original version 2004-09-02.
!     13-May-2007  S.Niemela     Renaming from HLACONDS to HLSTRACO in order not
!                                to mix this interface to HIRLAM one.

!     ======================================================
!     1. DECLARATIONS.

!     1.1 MODULES USED:

use parkind1  ,only : jpim, jprb
use yomhook   ,only : lhook, dr_hook
use yomct0    ,only : ltwotl
use yemgeo    ,only : edelx,  edely
use yhlconst  ,only : pi, rearth, cpair, ccpq
use yhloption ,only : lhltend

!     1.2 OTHER DECLARATIONS:

      implicit none                                                           

! Scalars 

integer(kind=jpim),intent(in)    ::  kcall    ! mode of call 
integer(kind=jpim),intent(in)    ::  khor     ! hor. loop dimension
integer(kind=jpim),intent(in)    ::  klev     ! number of vert. lev.
integer(kind=jpim),intent(in)    ::  kstart   ! start index of loops
integer(kind=jpim),intent(in)    ::  kstop    ! end index of loops 
integer(kind=jpim),intent(in)    ::  kstep    ! step no. in forecast
real(kind=jprb),   intent(in)    ::  pdtime   ! time step (s)
logical,           intent(in)    ::  ldstat   ! statistics switch

! Prognostic variables

real(kind=jprb),   intent(in)    ::  pu(khor,klev)    
!                                    first horiz. wind component  (m/s) 
real(kind=jprb),   intent(in)    ::  pv(khor,klev)
!                                    second horiz. wind component (m/s)  
real(kind=jprb),   intent(in)    ::  pt(khor,klev)
!                                    temperature (K)
real(kind=jprb),   intent(in)    ::  pq(khor,klev)
!                                    specific humidity    (kg/kg)
real(kind=jprb),   intent(inout) ::  pcw(khor,klev)
!                                    specific cloud water (kg/kg) 
real(kind=jprb),   intent(inout) ::  pcice(khor,klev)
!                                    specific cloud ice   (kg/kg)
real(kind=jprb),   intent(inout) ::  pqampli(khor,klev)
!                                    amplitude of hum. fluctuations
real(kind=jprb),   intent(inout) ::  ptke(khor,klev)
!                                    TKE

! Diagnostic variables 

real(kind=jprb),   intent(inout) ::  ptotcov(khor,klev)
!                                    fractional cloud cover
real(kind=jprb),   intent(in)    ::  pomf(khor,klev)
!                                    vertical velocity (Pa/s)
real(kind=jprb),   intent(in)    ::  pfrland(khor)
!                                    fraction of land
real(kind=jprb),   intent(inout) ::  pdqdtin(khor,klev)
!                                    specific humidity tendency (1/s) 
real(kind=jprb),   intent(inout) ::  pdcwdin(khor,klev)
!                                    specific cloud water tendency (1/s)  
real(kind=jprb),   intent(inout) ::  pdicedin(khor,klev)
!                                    specific cloud ice tendency (1/s) 
real(kind=jprb),   intent(inout) ::  pdtdtin(khor,klev)
!                                    temperature tendency (K/s) 
real(kind=jprb),   intent(in)    ::  pf(khor,klev)
!                                    full level pressure (Pa) 
real(kind=jprb),   intent(in)    ::  phalf(khor,0:klev)
!                                    half level pressure (Pa)
real(kind=jprb),   intent(inout) ::  ptendt(khor,klev)
!                                    accumulated temperature ph-tendency
real(kind=jprb),   intent(inout) ::  ptendq(khor,klev)
!                                    accumulated spec. hum. ph-tendency 
real(kind=jprb),   intent(inout) ::  ptendql(khor,klev)
!                                    accumulated spec. cloud water ph-tendency
real(kind=jprb),   intent(inout) ::  ptendqi(khor,klev)
!                                    accumulated spec. cloud ice ph-tendency  

! Fluxes 

real(kind=jprb),   intent(inout) ::  pdiftq(khor,0:klev)
!                                    turbulence flux of spec. hum.
real(kind=jprb),   intent(inout) ::  pdiftql(khor,0:klev)
!                                    turbulence liquid water flux 
real(kind=jprb),   intent(inout) ::  pdiftqn(khor,0:klev)
!                                    turbulence snow/ice flux     
real(kind=jprb),   intent(inout) ::  pdifts(khor,0:klev)
!                                    turbulence flux of enthalpy
real(kind=jprb),   intent(inout) ::  pdifcq(khor,0:klev)
!                                    convective flux of spec. hum.  
real(kind=jprb),   intent(inout) ::  pdifcql(khor,0:klev)
!                                    convective flux of liquid water
real(kind=jprb),   intent(inout) ::  pdifcqn(khor,0:klev) 
!                                    convective flux of ice
real(kind=jprb),   intent(inout) ::  pdifcs(khor,0:klev)
!                                    convective flux of enthalpy
real(kind=jprb),   intent(inout) ::  pfccql(khor,0:klev)
!                                    convective liquid condensation flux
real(kind=jprb),   intent(inout) ::  pfccqn(khor,0:klev)
!                                    convective ice condensation flux
real(kind=jprb),   intent(inout) ::  pfcsql(khor,0:klev)
!                                    stratiform liquid condensation flux
real(kind=jprb),   intent(inout) ::  pfcsqn(khor,0:klev)
!                                    stratiform ice condensation flux
real(kind=jprb),   intent(inout) ::  pfplcl(khor,0:klev)
!                                    convective liquid precipitation flux
real(kind=jprb),   intent(inout) ::  pfplcn(khor,0:klev)
!                                    convective ice condensation flux
real(kind=jprb),   intent(inout) ::  pfplsl(khor,0:klev)
!                                    stratiform liquid precipitation flux
real(kind=jprb),   intent(inout) ::  pfplsn(khor,0:klev)  
!                                    stratiform ice precipitation flux 

character(len=8)  ,intent(in)    ::  cdlock

!========================================================================
!     Other declarations  

logical              ::   llpupd

real(kind=jprb)      ::   zahyb(khor,klev+1),zbhyb(khor,klev+1)
real(kind=jprb)      ::   zdtdt(khor,klev),zdcwdt(khor,klev)
real(kind=jprb)      ::   zdicedt(khor,klev)
real(kind=jprb)      ::   zdqdt(khor,klev)
real(kind=jprb)      ::   zps(khor), zdpsdin(khor)
real(kind=jprb)      ::   zdpf(khor,klev),zpf(khor,klev)
real(kind=jprb)      ::   zconacc
real(kind=jprb)      ::   zdtheta
real(kind=jprb)      ::   zhook_handle

!  Old HIRLAM diagnostic parameters to be computed   

real(kind=jprb)  ::   zcov2d  (khor)
real(kind=jprb)  ::   zcwpath (khor)
real(kind=jprb)  ::   zdraindt(khor)
real(kind=jprb)  ::   zdsnowdt(khor)
real(kind=jprb)  ::   zpreta   (khor,klev)
real(kind=jprb)  ::   zprcpst  (khor) 
real(kind=jprb)  ::   zprcpcu  (khor)
real(kind=jprb)  ::   zstsnow (khor)
real(kind=jprb)  ::   zcusnow (khor)
real(kind=jprb)  ::   zstcon  (klev)
real(kind=jprb)  ::   zstdcon (klev)
real(kind=jprb)  ::   zstcov  (klev)
real(kind=jprb)  ::   zstdcov (klev)
real(kind=jprb)  ::   zstccov (klev)
real(kind=jprb)  ::   zstscov (klev) 
real(kind=jprb)  ::   zstcw   (klev)
real(kind=jprb)  ::   zstcpnt (klev)
real(kind=jprb)  ::   zsttpnt (klev)
real(kind=jprb)  ::   zstscal (klev)

integer(kind=jpim)   ::   jl,  jk

#include "hlcondfc.intfb.h"
#include "hlconds.intfb.h"
#include "hltend2flx.intfb.h"

if (lhook) call dr_hook('hlstraco',0,zhook_handle)
!  ======================================================================
!     1) Make precautions for input fields to be within realistic bounds

      do jk=1,klev
        do jl=kstart,kstop  
          if( kstep <= 1 ) pqampli(jl,jk)=0.05_JPRB 
          pqampli(jl,jk)=max(min( pqampli(jl,jk),0.5_JPRB ), 0.005_JPRB )
          ptotcov(jl,jk)=max(min( ptotcov(jl,jk),1.0_JPRB ), 0.0_JPRB )
          pcw(jl,jk)=max( pcw(jl,jk), 0.0_JPRB )
          pcice(jl,jk)=max( pcice(jl,jk), 0.0_JPRB )
        enddo 
      enddo 

!     ===================================================================   
!     2.  Transformation of input parameters to further treatment
!         by subroutines CONDS.

!     2.1  Basic parameters. 

!     logical switch "llpupd" defining if stratiform condensation 
!     is to be done on old (current variables' state (llpupd is false) 
!     or on preliminary updated state (llpupd is true)  

      llpupd=.true. 

!     Accumulation factor `zconacc' for time integr. of
!     precipitation

      if( ltwotl ) then
        zconacc=1.0_JPRB    
      else 
        zconacc=0.5_JPRB 
      endif 

!     Equivalent model resolution in deg. `zdtheta' 
!     used for parameterizing scale dependent processes
!     in the STRACO scheme.

      zdtheta=min( edelx,edely )*180._JPRB/(pi*rearth)

!     surface pressure and tendency
!     ( zero tendency is an approximation )

      do jl=kstart,kstop
        zps(jl)=phalf(jl,klev) 
        zdpsdin(jl)=0._JPRB
      enddo      

!     Pseudo hybrid values `ahyb',`bhyb':
!     ( note that ahyb,bhyb are defined from 1:nlev,
!     but phalf is defined from 0:nlev.

      do jk=1,klev+1
        do jl=kstart,kstop
          zahyb(jl,jk)=0._JPRB
          zbhyb(jl,jk)=phalf(jl,jk-1)/phalf(jl,klev)
        enddo 
      enddo 

!     Initialize local arrays. 

      do jk=1,klev
        do jl=kstart,kstop
          zdtdt(jl,jk)  =0.0_jprb
          zdqdt(jl,jk)  =0.0_jprb
          zdcwdt(jl,jk) =0.0_jprb
          zdicedt(jl,jk)=0.0_jprb
          zpf(jl,jk)=pf(jl,jk)
          zdpf(jl,jk)=phalf(jl,jk) -phalf(jl,jk-1) 

          zpreta(jl,jk)=0.0_jprb
        enddo
      enddo

      do jl=kstart,kstop
         zcov2d (jl)=0.0_jprb
         zcwpath (jl)=0.0_jprb
         zdraindt(jl)=0.0_jprb
         zdsnowdt(jl)=0.0_jprb
         zprcpst(jl)=0.0_jprb
         zprcpcu(jl)=0.0_jprb
         zstsnow(jl)=0.0_jprb
         zcusnow(jl)=0.0_jprb
      enddo

      do jk=1,klev
         zstcon(jk)=0.0_jprb
         zstdcon(jk)=0.0_jprb
         zstcov(jk)=0.0_jprb
         zstdcov(jk)=0.0_jprb
         zstccov(jk)=0.0_jprb
         zstscov(jk)=0.0_jprb
         zstcw(jk)=0.0_jprb
         zstcpnt(jk)=0.0_jprb
         zsttpnt(jk)=0.0_jprb
         zstscal(jk)=0.0_jprb
      enddo


!     -------------------------------------------------------
!     2.2    Compute forcing from dynamics and turbulence
!     (fluxes of humidity variables and enthalpy are used
!     to produce temperature tendency and humidity tendency
!     forcings). 
!     -------------------------------------------------------
      call hlcondfc (                                      &
!     integers    
     &             khor,         klev,          kstart,    &
     &             kstop,        kstep,                    &
!     Prognostic variables   
     &             pt,           pq,                       &
!     Tendencies/fluxes   
     &             pdtdtin,      pdqdtin,       pdcwdin,   &
     &             pdicedin,                               &
     &             pdiftql,      pdiftqn,       pdiftq,    &
     &             pdifts,                                 &
!     other parameters  
     &             zdpf                                    )

!    ==============================================================
!     3.   Call the main routine 'hlconds' of the STRACO
!          cloud-and condensation scheme. 

!          NB: The output variables pdifcq, pdifcql,pdifcqn, 
!          'pfccql','pfccqn','pfcsql' and 'pfcsqn'
!          are all tendencies first  
!          (kg water/(kg*s))  while 'pdifcs'
!          is temperature output tendency (layer values due
!          to convection processes). 

!          For ALADIN the relevant conversions to fluxes are done 
!          by calling 'tend2flx' separately for each output flux.
!          For the precipitation fluxes
!          ( pfplcl, pfplcn, pfplsl, pfplsn) no conversion is 
!          needed since fluxes are already available. 
!     -------------------------------------------------------------
      call hlconds (                                       & 
!     integers and logicals
     &             kcall,        khor,          klev,      &
     &             kstart,       kstop,         kstep,     &
     &             pdtime,       zconacc,       zdtheta,   &
     &             ldstat,       llpupd,                   &
!     prognostic variables 
     &             pu,           pv,            pt,        &
     &             pq,           pcw,           pcice,     &
     &             pqampli,      ptke,                     &
!     diagnostic variables 
     &             pomf,         ptotcov,       pfrland,   &  
     &             pdtdtin,      zdtdt,         pdqdtin,   &
     &             zdqdt,        pdcwdin,       zdcwdt,    &
     &             pdicedin,     zdicedt,                  &
     &             zpf,          zdpf,          zps,       &
     &             zdpsdin,      zahyb,         zbhyb,     &
     &             zcov2d,       zcwpath,       zdraindt,  &
     &             zdsnowdt,     zpreta,        zprcpst,   &
     &             zprcpcu,      zstsnow,       zcusnow,   &
     &             zstcon,                                 &
     &             zstdcon,      zstcov,        zstdcov,   &
     &             zstccov,      zstscov,       zstcw,     &
     &             zstcpnt,      zsttpnt,       zstscal,   & 
!     fluxes/tendencies 
     &             pdifcq,       pdifcqn,       pdifcql,   &
     &             pdifcs,       pfccql,        pfccqn,    &
     &             pfplcl,       pfplcn,        pfplsl,    &
     &             pfplsn,       pfcsql,        pfcsqn     )

!     ===========================================================
!     4. Save effect of tendencies due to condensation/convection 
!        processes by adding them to the accumulated physics 
!        tendencies.
!     ===========================================================

      if( lhltend ) then     
        do jk=1,klev
          do jl=kstart,kstop
            ptendt(jl,jk)  = ptendt(jl,jk)  + zdtdt(jl,jk)
            ptendq(jl,jk)  = ptendq(jl,jk)  + zdqdt(jl,jk)
            ptendql(jl,jk) = ptendql(jl,jk) + zdcwdt(jl,jk)
            ptendqi(jl,jk) = ptendqi(jl,jk) + zdicedt(jl,jk)
          enddo
        enddo 
      endif 

!     =========================================================
!     5. Convert 'pdifcs' (temporary temperature tendency) and 
!        'pdifcq' (temporary humidity tendency) to enthalpy 
!        tendency ("enthalpy") = T*cpair*ccpq* dq/dt + cp*dT/dt
!     =========================================================

      do jk=1,klev
        do jl=kstart,kstop
          pdifcs(jl,jk)=                              &
     &    pdifcs(jl,jk)*cpair*(1. +ccpq*pq(jl,jk)) +  &
     &    pt(jl,jk)*cpair*ccpq*pdifcq(jl,jk)
        enddo 
      enddo 

!     =========================================================
!     6. Compute the conversion from tendencies to fluxes for
!        the different output parameters , by calls to the
!        conversion routine 'hltend2flx':
!     ---------------------------------------------------------

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    pdifcs     )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    pdifcq     )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    pdifcql    )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    pdifcqn    )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    pfccql     )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    pfccqn     )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    pfcsql     )

      call hltend2flx ( khor,     klev,    kstart,    &
     &                  kstop,    zdpf,    pfcsqn     )

!    =========================================================
if(lhook)  call dr_hook('hlstraco',1,zhook_handle)
end subroutine hlstraco
