! Library:phys $Id: hlvcbr.F90 5741 2008-02-27 12:39:34Z xiaohua $
 
!options xopt(noeval)
      subroutine hlvcbr(                           &
     &           khor   ,   klev   ,    kstart ,   & 
     &           kstop  ,   pdtime ,    ldstat ,   &
     &           pt     ,   pq     ,    pu     ,   &
     &           pv     ,   pcw    ,    pcice  ,   &
     &           kcdnst ,                          &
     &           ptke   ,   pgpot  ,    ppf    ,   &
     &           pph    ,   pcld2  ,               &
     &           pts    ,   pps    ,    ptaux  ,   &
     &           ptauy  ,   psenf  ,    platf  ,   &
     &           pustar ,   ppblh  ,    pdtdt  ,   &
     &           pdqdt  ,   pdudt  ,    pdvdt  ,   &
     &           pdcwdt ,   pdicedt,    pdtkedt,   &
     &           pstdif ,   pstddif,    cdoneup    )
!     -------------------------------------------------------------
!     PURPOSE: (*) 
!     --------     
!     Routine that performs the computation of the turbulent fluxes
!     and gives the corresponding tendencies to the prognostic variables.
!     Routine is based on VCBR (v1.2) by J. Cuxart and J. Calvo (see
!     references) but modified with respect to the following points.
!     CHANGE 1) New formulation length scale (see section 3)
!     CHANGE 2) TKE defined on half levels ! This increases
!     numerical precision considerably.
!     To make the change in code relatively simple, we used
!     tke(nlev) for the surface. Therefore, the grid index of tke
!     (and tke related variables) is one lower than the usual half
!     level numbering. Note this carefully !!!
!     This makes the scheme a bit incompatible with the computations
!     outside the tke scheme, although probably the errors introduced
!     are (very) small.
!     CHANGE 3) TKE equation more implicit. Buoyancy, shear and
!     dissipation are now all semi-implicit.
!     CHANGE 4) boundary condition momentum flux (explicitly prescribed
!     momentum flux) is replaced by implicitly computed one by
!     prescribing u = 0 (surface) and Km(surf)
!     Km(surf) is reconstructed from ustar. This change solves
!     instability problems as encountered (over mountains)
!     in various NWP runs when the surface momentum flux is large.
!     CHANGE 5) Solving the diffusion equation has been changed.
!     The original version did not incorporate density changes,
!     and leaks about 5-10 % of surface fluxes
!  ---------------------------------------------------------------
!  AUTHORS:
!  --------
!  2006/06/01    Bent H. Sass 
!  Release of recoded F90 scheme

!  2005/01/30    Bent H. Sass 
!  Modest tuning of turbulence scheme for a revised STRACO scheme 

!  2004/11/08    SanderTijm
!  Less stable mixing, as per Newsletter 46 "Tuning CBR"

!  2004/09/20    LauraRontu
!  zmix_help capped to 5 instead of 10

!  2004/06/28    GerardCats
!  Correct error in calculation of Richardson number

!  2004/03/29    ColinJones
!  Revised stable mixing

!  2003/06/23    ColinJones
!  Revert to version 6.1.0

!  2003/06/02    ColinJones
!  increased mixing in stable conditions

!  2002/03/11    GLenderink
!  Reformulation in conserved quantitities

!  2001/09/24    WimDeRooy
!  Geert Lenderink's modifications

!  2000/11/13    KalleEerola
!  Further removal of unnecessary printing

!  2000/05/22    BentHSass
!  Introduce a semi-implicit treatment of TKE dissipation.

!  1999/11/15    LauraRontu
!  Remove some printout 

!  1999/09/20    JoanCuxart
!  Corrections to surface moisture and momentum fluxes

!  1999:         JoanCuxart 
!  Original scheme  

!  ---------------------------------------------------------------------
!  DESCRIPTION:

!  Main routine of the scheme. It performs the computation of
!  the turbulent fluxes and gives the corresponding tendencies to
!  the prognostic variables.

!  The present version of the scheme computes dry turbulence, though
!  the scheme is prepared for an easy jump to a moist turbulence scheme.

!  REFERENCES:

!  J.Cuxart, P.Bougeault and Jean-Luc Redelsperger, Jan 2000
!  A turbulence scheme allowing for mesoscale and large-eddy simulations
!  Quarterly Journal of the Royal Meteorological Society, pp 1

!  J.Calvo, J.Cuxart and P.Kallberg:
!  Implementation of an E-L turbulence scheme in the HIRLAM model.
!  Submitted to Monthly Weather Review.

!  De Rooy, 2000: Experiences with the HIRLAM including the CBR scheme.
!  Hirlam Newsletter No. 35

!  Lenderink, De Rooy, 2000: A robust mixing length formulation for a
!  TKE-l turbulence scheme. Hirlam newsletter No. 36
!     ----------------------------------------------------------------
!     INTERFACE  :  subroutine 'VCBR' is called 
!     ------------  from subroutine 'PHYS'

!     ----------------------
!     EXTERNAL SUBROUTINES:  TRIDIAG
!     ----------------------     

!     ==================================================================
!     1. Declarations. 
!     ==================================================================
!     1.1 MODULES USED  
!     ----------------
use parkind1  ,only  : jpim     ,jprb
use yomhook   ,only  : lhook,   dr_hook

use yhloption ,only  : hloption
use yhlconst  ,only  : latvap, latice,  cpair, epsilo, &
 &                     tmelt,  gravit,  rair,  ccpq,  cpv
use yhlturb   ,only  : ct,     cq,      cte,   cu,  cdis

implicit none

!     1.2 ARGUMENT DECLARATIONS
!     -------------------------

integer(kind=jpim),  intent(in)           ::   khor
integer(kind=jpim),  intent(in)           ::   klev
integer(kind=jpim),  intent(in)           ::   kstart
integer(kind=jpim),  intent(in)           ::   kstop

real(kind=jprb)   ,  intent(in)           ::   pdtime

real(kind=jprb)   ,  intent(in)           ::   pts(khor)
real(kind=jprb)   ,  intent(in)           ::   pps(khor)
real(kind=jprb)   ,  intent(in)           ::   ptaux(khor)
real(kind=jprb)   ,  intent(in)           ::   ptauy(khor)
real(kind=jprb)   ,  intent(in)           ::   psenf(khor)
real(kind=jprb)   ,  intent(in)           ::   platf(khor)
real(kind=jprb)   ,  intent(in)           ::   pustar(khor)
real(kind=jprb)   ,  intent(in)           ::   ppblh(khor)

real(kind=jprb)   ,  intent(in)           ::   pt(khor,klev) 
real(kind=jprb)   ,  intent(in)           ::   pq(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   pu(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   pv(khor,klev)

integer(kind=jpim),  intent(in)           ::   kcdnst

real(kind=jprb)   ,  intent(in)           ::   pcw(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   pcice(khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   ptke(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   pgpot(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   pcld2(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   ppf(khor,klev)
real(kind=jprb)   ,  intent(in)           ::   pph(khor,0:klev)
real(kind=jprb)   ,  intent(inout)        ::   pdtdt(khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   pdqdt(khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   pdudt(khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   pdvdt(khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   pdcwdt(khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   pdicedt(khor,klev)
real(kind=jprb)   ,  intent(inout)        ::   pdtkedt(khor,klev)

logical           ,  intent(in)           ::   ldstat

real(kind=jprb)   ,  intent(inout)        ::   pstdif(klev)
real(kind=jprb)   ,  intent(inout)        ::   pstddif(klev) 
character(len=6)  ,  intent(in)           ::   cdoneup
   
!     1.2.1 LOCAL, FORMER ARGUMENT DECLARATIONS
!     -----------------------------------------

real(kind=jprb)    ::   zwsenf  (khor)
real(kind=jprb)    ::   zwcflx  (khor)
real(kind=jprb)    ::   zwths   (khor)
real(kind=jprb)    ::   zwthm   (khor,klev)
real(kind=jprb)    ::   zblength(khor,klev)
real(kind=jprb)    ::   zkeff   (khor,klev+1)
real(kind=jprb)    ::   zthl    (khor,klev+1)
real(kind=jprb)    ::   ztotw   (khor,klev+1)
real(kind=jprb)    ::   zwzhf   (khor,klev+1)
real(kind=jprb)    ::   zwzfl   (khor,klev+1)
real(kind=jprb)    ::   zahtf   (khor,klev)
real(kind=jprb)    ::   zbhtf   (khor,klev)
real(kind=jprb)    ::   zafth   (khor,klev+1)
real(kind=jprb)    ::   zbfth   (khor,klev+1)
real(kind=jprb)    ::   zwork1  (khor,klev)
real(kind=jprb)    ::   zwork2  (khor,klev+1)
real(kind=jprb)    ::   zqsat   (khor,klev)
real(kind=jprb)    ::   zjcoef  (khor,klev)
real(kind=jprb)    ::   ztpvir  (khor,klev+1)
real(kind=jprb)    ::   zphi3   (khor,klev+1)
real(kind=jprb)    ::   zasc    (khor,klev)
real(kind=jprb)    ::   zcsc    (khor,klev)
real(kind=jprb)    ::   zamom   (khor,klev)
real(kind=jprb)    ::   zcmom   (khor,klev)
real(kind=jprb)    ::   ztlfx   (khor,klev+1)
real(kind=jprb)    ::   zsource (khor,klev)
real(kind=jprb)    ::   ztkefx  (khor,klev)
real(kind=jprb)    ::   ztwfx   (khor,klev+1)
real(kind=jprb)    ::   zthlp   (khor,klev)
real(kind=jprb)    ::   ztotwp  (khor,klev)
real(kind=jprb)    ::   zccdnst (khor,klev,kcdnst)
real(kind=jprb)    ::   zccdnstp(khor,klev,kcdnst)
real(kind=jprb)    ::   zup     (khor,klev)
real(kind=jprb)    ::   zvp     (khor,klev)
real(kind=jprb)    ::   ztkep   (khor,klev)
real(kind=jprb)    ::   ztrtke  (khor,klev)
real(kind=jprb)    ::   zbuofx  (khor,klev+1)
real(kind=jprb)    ::   zwdzf   (khor,klev)
real(kind=jprb)    ::   zwdzh   (khor,klev+1)
real(kind=jprb)    ::   zdqsatdt(khor,klev)
real(kind=jprb)    ::   zidqs   (khor,klev)
real(kind=jprb)    ::   zcpair  (khor,klev)
real(kind=jprb)    ::   zwork   (khor)
real(kind=jprb)    ::   zbeta   (khor)
real(kind=jprb)    ::   zrho    (khor)
real(kind=jprb)    ::   zbuofxs (khor)
real(kind=jprb)    ::   zi      (khor)
real(kind=jprb)    ::   zsavetke(khor)
  
!     1.3  OTHER DECLARATIONS
!     -------------------------

real(kind=jprb)    ::  zatke    (khor,klev)
real(kind=jprb)    ::  zctke    (khor,klev)
real(kind=jprb)    ::  zgbuoy   (khor,klev+1) 
real(kind=jprb)    ::  zgri     (khor,klev+1)
real(kind=jprb)    ::  zmixdwh  (khor,klev+1)
real(kind=jprb)    ::  zmixuph  (khor,klev+1)
real(kind=jprb)    ::  zmixdwm  (khor,klev+1)
real(kind=jprb)    ::  zmixupm  (khor,klev+1)
real(kind=jprb)    ::  zmixquadh(khor,klev+1)
real(kind=jprb)    ::  zmixquadm(khor,klev+1)
real(kind=jprb)    ::  zmixquadd(khor,klev+1)
real(kind=jprb)    ::  zwdzfp   (khor,klev)
real(kind=jprb)    ::  zwdzhp   (khor,klev+1)
real(kind=jprb)    ::  zkeffp   (khor,klev+1)

real(kind=jprb)    ::  zwei(khor,klev) 

real(kind=jprb)    ::  zbuoythress , zprandt   ,                 &
&                      zpi2        , zkap      ,  zbuoy_ful   ,  &
&                      zdzm        , zdzh      ,  zdz         ,  &
&                      zx          , zentr     ,  zentrh      ,  &
&                      zentrm      , zmixch    ,  zmixcm      ,  &
&                      zlam        , ztkesq    ,  zmix2       ,  &
&                      zmixh       , zmixm     ,  zlengthrh   ,  &
&                      zlengthrm   ,                             &
&                      zzb         , zdisl     ,                 &
&                      zktest      , zimplmom  ,  zzrho       ,  &
&                      zxarm       , zxarh     ,  zxm         ,  &
&                      zxh         , zmix_help ,  zneutr      ,  & 
&                      zfineutr    , zfias     ,  zri_ful     ,  &
&                      zbri        , zave      ,  zmixh_stable,  &
&                      zmixm_stable

real(kind=jprb)    ::  zpfunc      , zwstar   ,  zlobukhov   ,  & 
&                      zginv       , zkappa   ,  zp0         ,  &
&                      zlati       , zlatw    ,  zlatin      ,  &
&                      zpf         , zresat   ,  zdeca       ,  & 
&                      zdeca1      , zca1     ,  zca2        ,  &
&                      zca3        , zlatsub  ,  zzfr

real(kind=jprb)    ::  zadry       , zbdry    ,  zamoist     ,  &
&                      zbmoist     , zldrt    ,  zldct       ,  &
&                      zathl       , zbqt

real(kind=jprb)    ::  zhook_handle
integer(kind=jpim) :: ik, i, j

!     1.4  Special include files (interface files):
!     ---------------------------------------------

#include "hltridiag.intfb.h"

#include "hlesat.h" 

if(lhook) call dr_hook('hlvcbr',0,zhook_handle)

!     1.5  Check if call to routine is correct
!     ----------------------------------------

     do ik=1,klev
       do i=kstart,kstop
         zccdnst(i,ik,1)=pcw(i,ik)
         if (kcdnst > 1) then
           zccdnst(i,ik,2)=pcice(i,ik)
         endif
       enddo
     enddo 

!    ---------------------------------------------- 
!     2   Preliminary settings
!    ----------------------------------------------

!     2.1 Define local constants
!    ----------------------------------------------
      zginv=1._jprb/gravit
      zkappa=rair/cpair
      zp0=1.0e5_jprb
      zlatsub=latvap + latice
      zca1=0.20_jprb
      zca2=200._jprb
      zca3=zca2*zca2
      zlam=75._JPRB

!     Some length scale constant and tuning parameters !

      zkap = 0.4_jprb
      zpi2 = 2._jprb * atan(1._jprb)
      zneutr = 1./sqrt( 3.75_jprb )

!     Tuning constants.
!     zprandt determines lm/lh convective limit
!     zfineutr scaling length scale near surface
!     zfineutr = zneutr *zkap standard value
!     (derived in Lenderink 2002)
!     zbri length scale dependency on Ri determined
!     from linearization flux prof. relations
!     zbri ~ 4-8.
!     see details see Lenderink 2002)
!     zfias convective limit Kh
!     zfias =  zneutr *zkap * 3. / zprandt  standard

      zbuoythress = 0.1e-4_jprb                   ! default
      zprandt     = 0.6_jprb                      ! default
      zfineutr    = zneutr*zkap*1.3_jprb          ! test:to compensate

      zfias       = zneutr*zkap*3.0_jprb/zprandt  ! default
      zbri        = 8.0_jprb

!     Reduce entrainment to produce lower surface temperatures
!     ( in winter ), important parameter for stable length scale
!     and stable mixing.

      zentr=0.13_jprb

!     2.2  Level closest to the ground
!     (sign of fluxes changed to have positive upwards)
!     -------------------------------------------------
      do i=kstart,kstop

!       Currently the temperature variation of LATVAP is not taken 
!       into account for surface latent energy computations. 
!       Therefore it should not either be accounted for 
!       at this place to convert back to moisture flux. 

        zlati=1./(latvap + latice)
        zlatw=1./latvap

        if(pts(i) < tmelt) then
          zlatin=zlati
        else
          zlatin=zlatw
        endif

!       A "dry" CP is used at the ground to be consistent with 
!       surface flux computations elsewhere.

        zrho(i)=ppf(i,klev)/rair/pt(i,klev)
        zwsenf(i)=-psenf(i)/(zrho(i)*cpair)
        zwcflx(i)=-platf(i)*zlatin/zrho(i)
        zwths(i)=pts(i)*(zp0/pps(i))**zkappa

      enddo

      do ik=1,klev
        do i=kstart,kstop
          zcpair(i,ik)=cpair*(1._jprb +ccpq*pq(i,ik))
          zpf=ppf(i,ik)
          zresat=1./pecor(zpf, pt(i,ik)) 
          zqsat(i,ik)=epsilo*esat( pt(i,ik) )*zresat
          zdqsatdt(i,ik)=epsilo*zpf*desdt(pt(i,ik))*zresat*zresat
          zzfr=pcw(i,ik)/max( 1.0E-12_JPRB, pcw(i,ik) +pcice(i,ik) )
          zzfr=max( 0.0_jprb, min( zzfr, 1.0_jprb ) )
          zidqs(i,ik)=1./(1.0_jprb +( latvap +flatvt(pt(i,ik))   &
  &       +(1._jprb -zzfr)*latice)*zdqsatdt(i,ik)/     &
  &       zcpair(i,ik)) 
        enddo
      enddo
 
!     2.3 geometric height(zwzfl), theta and Tv(zwork1) at full levels
!     ----------------------------------------------------------------
      do ik=1,klev
        do i=kstart,kstop
          zwzfl(i,ik)=pgpot(i,ik)*zginv
          zwork1(i,ik)=pt(i,ik)*(1.0_jprb + 0.61_jprb*pq(i,ik))
          zwthm(i,ik)=pt(i,ik)*(zp0/ppf(i,ik))**zkappa
        enddo
      enddo

      if( (hloption(2) == 1).or.(hloption(2) == 5) ) then
        do ik=1,klev
          do i=kstart,kstop
            zwei(i,ik)=pcld2(i,ik)*pcld2(i,ik)* &
  &                    pcld2(i,ik)*pcld2(i,ik)
          enddo
        enddo
      endif
      if( (hloption(2) == 4) ) then
        do ik=1,klev
          do i=kstart,kstop
            zwei(i,ik)=pcld2(i,ik)
          enddo
        enddo
      endif

      do i=kstart,kstop
        zbeta(i)=gravit/zwthm(i,klev)
        zwzfl(i,klev+1)=0.0_jprb
      enddo

!     2.4 geometric height (wzhf) of the half levels
!         zwork2 contains the geopotential of the intermediate levels
!     ---------------------------------------------------------------
      do i=kstart,kstop
        zwork2(i,klev+1)=0.0_jprb
        do ik=klev,2,-1
          zwork2(i,ik)=zwork2(i,ik+1)+rair*zwork1(i,ik)   &
 &                    *log(pph(i,ik)/pph(i,ik-1))
        enddo
        zwork2(i,1)=zwork2(i,2)+(pgpot(i,1)-pgpot(i,2))/2._jprb
      enddo

      do ik=1,klev+1
        do i=kstart,kstop
          zwzhf(i,ik)=zwork2(i,ik)*zginv
        enddo
      enddo

!     2.5 thickness between full and half levels 
!     (at half and full resp.)
!     ---------------------------------------------------------------
      do ik=1,klev-1
        do i=kstart,kstop
          zwdzf(i,ik)   = zwzhf(i,ik) -zwzhf(i,ik+1)
          zwdzh(i,ik+1) = zwzfl(i,ik) -zwzfl(i,ik+1)
          zwdzfp(i,ik)  = ( pph(i,ik) - pph(i,ik-1) ) *zginv
          zwdzhp(i,ik+1)= ( ppf(i,ik+1) - ppf (i,ik) )*zginv
        enddo
      enddo

      do i=kstart,kstop
        zwdzh(i,1) = 0.0_jprb
        zwdzf(i,klev)   = zwzhf(i,klev) -zwzhf(i,klev+1)
        zwdzh(i,klev+1) = zwzfl(i,klev)
        zwdzfp(i,klev)  = (pph(i,klev)-pph(i,klev-1))*zginv
        zwdzhp(i,klev+1)= (pph(i,klev)-ppf(i,klev))*zginv
        zwdzhp(i,1)     = 0.0_jprb
      enddo

      do ik=2,klev
        do i=kstart,kstop
          zbhtf(i,ik)= (zwzhf(i,ik) -zwzfl(i,ik)) / zwdzf(i,ik)
          zahtf(i,ik)= 1._jprb -  zbhtf(i,ik)
          zafth(i,ik)= (zwzhf(i,ik) -zwzfl(i,ik)) / zwdzh(i,ik)
          zbfth(i,ik)= 1._jprb -  zafth(i,ik)
        enddo
      enddo

      do i=kstart,kstop
        zafth(i,klev+1)= 0.0_jprb
        zbfth(i,klev+1)= 1.0_jprb
        zbhtf(i,1)     = (zwzhf(i,1) -zwzfl(i,1)) / zwdzf(i,1)
        zahtf(i,1)     = 1.0_jprb - zbhtf(i,1)
      enddo
 
!     2.6   Use the pbl that is already available in input
!     ----------------------------------------------------
      do i=kstart,kstop
        zi(i)=ppblh(i)
        zi(i)=max( zi(i), 50._jprb )
      enddo

      do i=kstart,kstop
        zbuofxs(i)=zwsenf(i)*(1._jprb + epsilo*pq(i,klev)) + &
  &               epsilo*zwthm(i,klev)*zwcflx(i)
      enddo

!     2.6.1  Save tke at lowest level to compute later a 
!            correct tendency of TKE at this layer.

      do i=kstart,kstop

        zsavetke(i)=ptke(i,klev)  

        if ( zbuofxs(i) > 0.0_jprb ) then
          zwstar=(  zbeta(i)*zi(i)*zbuofxs(i) ) **(0.3333_jprb)
          zlobukhov=-zwthm(i,klev)*(pustar(i)**3)/   &
  &       (0.4_jprb*gravit*zbuofxs(i))
          ptke(i,klev)=(3.75_jprb +  &
  &       ((-1.0_jprb)*zwzfl(i,klev)/zlobukhov)**(0.6666_jprb))  &
  &       *(pustar(i)**2) + 0.2_jprb*(zwstar**2)
        else
          ptke(i,klev)=3.75_jprb*(pustar(i)**2)
        endif

      enddo

!     2.7  TKE positivity control
!     ----------------------------------------------------------------
!     To prevent any occurrence of negative values produce by previous
!     manipulation of the field (for instance, through advection or
!     horizontal diffusion)

      do ik=1,klev
        do i=kstart,kstop
          ptke(i,ik)=max( 1.0e-4_jprb, ptke(i,ik) )
        enddo
      enddo

!     2.8 Construction of the conservative variables (Theta_l and qtot)
!     ----------------------------------------------------------------
!     Here the conservative variables for the moist non-precipitating
!     processes are defined.

      do ik=1,klev
        do i=kstart,kstop

          if (hloption(3) == 3) then                               ! Moist CBR
!         --------------------------------------------------------------------
            ztotw(i,ik)=pq(i,ik)
            do j=1,kcdnst
              ztotw(i,ik)=ztotw(i,ik) +zccdnst(i,ik,j)
            enddo

              zthl(i,ik)=zwthm(i,ik)*( 1._jprb                       &  
   &          - zccdnst(i,ik,1)*(latvap +flatvt(pt(i,ik)))/          & ! water 
   &            (pt(i,ik)*zcpair(i,ik))                              &     
   &          - zccdnst(i,ik,2)*(latvap +flatvt(pt(i,ik)) +latice)/  & ! ice 
   &            (pt(i,ik)*zcpair(i,ik)) )
            ztpvir(i,ik)=zwthm(i,ik)*(1. +1.61_jprb*pq(i,ik))/ &
   &        (1._jprb +ztotw(i,ik))

          endif

          if (hloption(3) == 2) then                             ! Dry CBR
            ztotw(i,ik)=pq(i,ik)
            zthl(i,ik)=zwthm(i,ik)
            ztpvir(i,ik)=zwthm(i,ik)*(1. +0.61_jprb*pq(i,ik))
          endif

        enddo
      enddo

!     2.9 Extrapolate some fields to surface 

      do i=kstart,kstop
        ztpvir(i,klev+1)=ztpvir(i,klev)*zwzfl(i,klev-1)/zwdzh(i,klev)   &
     &                  - ztpvir(i,klev-1)*zwzfl(i,klev)/zwdzh(i,klev)
       ztotw(i,klev+1)= ztotw(i,klev)*zwzfl(i,klev-1)/zwdzh(i,klev) &  
     &                - ztotw(i,klev-1)*zwzfl(i,klev)/zwdzh(i,klev) 
       zthl(i,klev+1)= zthl(i,klev)*zwzfl(i,klev-1)/zwdzh(i,klev)  &
     &               - zthl(i,klev-1)*zwzfl(i,klev)/zwdzh(i,klev)
      enddo

!     2.10  compute brunt vaisala frequency (stability) zgbuoy from
!     virtual potential temperature gradient
!     ----------------------------------------------------------------
      do ik=1,klev
        do i=kstart,kstop

!         Buoyancy valid for dry formulation 

          if (hloption(3) == 2_jpim) then
!         -------------------------------
            zgbuoy(i,ik+1) = gravit/ztpvir(i,ik)*   &
     &      (ztpvir(i,ik) -ztpvir(i,ik+1))/zwdzh(i,ik+1)
          endif
!         -----

!         Buoyancy valid for moist formulation 

          if (hloption(3) == 3_jpim) then
!         -------------------------------
            zldrt = (latvap + flatvt(pt(i,ik)))/(rair * pt(i,ik))  
            zldct = (latvap + flatvt(pt(i,ik)))/(zcpair(i,ik)*pt(i,ik))

            zadry = 1. + 0.61*ztotw(i,ik) 
            zamoist = (1._jprb -ztotw(i,ik) +1.61_jprb*zqsat(i,ik) &
     &        *(1._jprb +0.622_jprb*zldrt)) &
     &        /(1._jprb +0.622_jprb*zldrt*zldct*zqsat(i,ik)) 
            zbdry =   0.61 * zwthm(i,ik)
            zbmoist = (zldct * zamoist - 1._jprb) * zwthm(i,ik)
            zathl = (1.-zwei(i,ik))*zadry + zwei(i,ik)*zamoist
            zbqt  = (1.-zwei(i,ik))*zbdry + zwei(i,ik)*zbmoist

            zgbuoy(i,ik+1) = gravit/ztpvir(i,ik)* &
     &        (zathl*(zthl(i,ik) -zthl(i,ik+1)) &
     &        +zbqt*(ztotw(i,ik) -ztotw(i,ik+1)))/zwdzh(i,ik+1)
          endif
!         --------------------------------
        enddo
      enddo

      do i=kstart,kstop
        zgbuoy(i,1) = zgbuoy(i,2)
      enddo

!     Add computation Richardson number

      do ik=1,klev-1
        do i=kstart,kstop 
          zgri(i,ik+1) = zgbuoy(i,ik+1) / max(0.1_jprb ,            &
     &   ((pu(i,ik) -pu(i,ik+1))**2 + (pv(i,ik) -pv(i,ik+1))**2))   &
     &   *zwdzh(i,ik+1)**2
        enddo
      enddo

      do i=kstart,kstop
        zgri(i,klev+1) = zgbuoy(i,klev+1)*(zwzfl(i,klev)**2)       &
     &         / max( pu(i,klev)**2 + pv(i,klev)**2 ,0.1_jprb )

!       max as a security for noshear limit; no phys. relevance

        zgri(i,1) = zgri(i,2)
      enddo

!     3. Computation of the mixing length
!     -----------------------------------------------------------------
!     Compute a simple quadratic length scale
!     this is a preliminary code, which gives a "quadratic" lengthscale
!     according to 1/zmixquad = 1/zmixup + 1/zmixdw
!     zmixup: lengthscale from bottom (say ground)
!     lower boundary: ground or lowest position unstable layer
!     zmixdw: lengthscale from top (say inversion)
!     upper boundary: "inversion"
!     both zmixup and zmixdw increase for unstable and neutral
!     conditions, and decrease for stable conditions.
!     this is governed by zbuoythress; small zbuoythress ==>
!     strong damping stable conditions (inversion), and
!     increase for unstable conditions.
!     in neutral conditions the lengthscale zmixup ~ zmixquad ~ zkap*z
!     near the surface
!     zprandt: governs difference between lengthscale for momentum
!     and heat.

      zxarm = zbri * zpi2 * zneutr * zkap/(zfias*zprandt-zfineutr)
      zxarh = zbri * zpi2 * zneutr * zkap /(zfias - zfineutr)

!     Compute bottom up length scale

      do i=kstart,kstop               ! initialization at surface !
        zmixuph(i, klev+1) = 0.0_jprb
        zmixupm(i, klev+1) = 0.0_jprb
      enddo

!     Loop over model levels          ! bottom-up loop !

      do ik=klev,2,-1
        do i=kstart,kstop
          zbuoy_ful = 0.5_jprb*(zgbuoy(i,ik+1) + zgbuoy(i,ik))
          zri_ful   = 0.5_jprb*(zgri(i,ik+1) + zgri(i,ik))

          zdeca1 = zwzhf(i,ik)*zwzhf(i,ik)/zca3
          zdeca = ( zca1*exp ( - zdeca1 ) +1._jprb -zca1 )

          zxm = zxarm * zri_ful
          zxh = zxarh * zri_ful

          zdz = zwdzf(i,ik)
          zx = zri_ful

          if (zx > 0.0_jprb) then
            zdzm = zdeca*( zfineutr*zdz -                &              
     &      (zfias*zprandt - zfineutr) / zpi2*zdz*zxm )
            zdzh =zdeca*( zfineutr*zdz -                 & 
     &      (zfias -zfineutr) / zpi2*zdz*zxh )
          else
            zdzm = zdeca*( zfineutr*zdz -                & 
     &      (zfias*zprandt - zfineutr) /                 &
     &      zpi2*zdz*atan(zxm) )
            zdzh = zdeca*( zfineutr*zdz -                &
     &      (zfias - zfineutr)/ zpi2*zdz*atan(zxh) )
          endif

!         Bottom up lengthscale

          zmixuph(i,ik) = zmixuph (i,ik + 1) + zdzh
          zmixuph(i,ik) = max(zmixuph(i,ik) , min(zkap*zdz,60._jprb)) ! default
          zmixupm(i,ik) = zmixupm (i,ik + 1) + zdzm
          zmixupm(i,ik) = max(zmixupm(i,ik) , min(zkap*zdz,60._jprb)) ! default
        enddo
      enddo

!     Compute top down length scale

      do i=kstart,kstop           ! initialize at the top of domain
        zmixdwh(i, 1) = 0.
        zmixdwm(i, 1) = 0.
      enddo

      do ik = 2, klev              ! vertical loop top, down
        do i=kstart,kstop
          zbuoy_ful = 0.5_jprb*(zgbuoy(i,ik) + zgbuoy(i,ik+1))
          zri_ful   = 0.5_jprb*(zgri(i,ik)   + zgri(i,ik+1))

          zdeca1 = zwzhf(i,ik)*zwzhf(i,ik)/zca3
          zdeca = ( zca1*exp ( - zdeca1 ) +1._jprb -zca1 )

          zxm = zxarm * zri_ful
          zxh = zxarh * zri_ful
          zdz = zwdzf(i,ik-1)
          zx = zri_ful

          if (zx > 0.0_jprb) then
            zdzm = zdeca*( zfineutr*zdz -              &
     &      (zfias*zprandt - zfineutr) / zpi2*zdz*zxm )
            zdzh = zdeca*( zfineutr*zdz -              &
     &      (zfias -zfineutr) / zpi2*zdz*zxh )
          else
            zdzm = zdeca*( zfineutr*zdz -              &   
     &      (zfias*zprandt - zfineutr) /               &
     &      zpi2*zdz*atan(zxm) )
            zdzh = zdeca*( zfineutr*zdz -              &
     &      (zfias - zfineutr) / zpi2*zdz*atan(zxh) )
          endif

!         Limit downward length scale close to the surface
!         to reduce influence of it on zmixquad (final length
!         scale). This means that the zmixup will determine
!         scaling close to the surface.

          zmix_help = 75._jprb * exp ( - zwzhf(i,ik) / 500._jprb)

          zmixdwh(i,ik) = zmixdwh (i,ik-1)  +  zdzh
          zmixdwh(i,ik) = max (zmixdwh(i,ik),zmix_help)
          zmixdwm(i,ik) = zmixdwm (i,ik-1)  +  zdzm
          zmixdwm(i,ik) = max (zmixdwm(i,ik),zmix_help)

!         Composed length scale of top-down ls and bottom-up ls

          zave = 1._jprb
          zlengthrh = 1._jprb / zmixdwh(i,ik)**zave        &
     &              + 1._jprb / zmixuph(i,ik)**zave
          zmixquadh(i,ik) = 1._jprb / zlengthrh**(1._jprb/zave)
          zlengthrm = 1._jprb / zmixdwm(i,ik)**zave        &
     &              + 1._jprb / zmixupm(i,ik)**zave
          zmixquadm(i,ik) = 1._jprb / zlengthrm**(1._jprb/zave)
         enddo
      enddo                                  ! end vertical loop

!    4. Computation lengthscale for stable conditions.
!    At this moment basic convective lengthscale is in zmixquadm/h
!    final lengthscale is put in zmixquadm/h
!    ------------------------------------------------------------------

      do ik = 2, klev            ! vertical loop top, down
        do i=kstart,kstop

!       Make a function that goes from 1 at p(nlev) to 0 at (p< 100hpa)

          zpfunc=(ppf(i,ik) -10000._jprb)/(ppf(i,klev) -10000._jprb)
          if(zpfunc < 0._jprb) zpfunc=0._jprb

!         included to represent momentum mixing by gravity waves !
!         NB: The formulae in the next few lines should be
!         considered tentative ( have been made to produce
!         realistic model windprofiles in stable conditions ! )

          if (zgri(i,ik) < 5._jprb .and. zgri(i,ik) > 0._jprb) then
            zmix_help = 3.0_jprb * zgri(i,ik) * exp(-zgri(i,ik)**2)
            zmix_help = 0.0_jprb * zgri(i,ik) * exp(-zgri(i,ik)**2)
          else
            zmix_help = 0._jprb ! no need to evaluate exp: it is << 1
          endif
          zmix_help = max( 1._jprb, zpfunc*(1._jprb + zmix_help) )
          zentrh  = zentr
          zentrm  = zentr * zmix_help

          zmix2   = zlam /(1._jprb + zlam/ (zneutr*zkap * zwzhf(i,ik)))

          zave    = 2._jprb        !  interpolation
          zmixh   =  (zmix2**zave + zmixquadh(i,ik)**zave)**(1._jprb/zave)
          zmixm   =  (zmix2**zave + zmixquadm(i,ik)**zave)**(1._jprb/zave)

          ztkesq  = ptke(i,ik-1)
          ztkesq  = sqrt(max( ztkesq ,1.e-4_jprb))

          zave    = -2._jprb      ! interpolation

          if (zgbuoy(i,ik) > 0._jprb ) then               ! stable
            zmixh_stable = zentrh * ztkesq / sqrt(zgbuoy(i,ik))
            zmixch = (zmixh_stable**(zave) + &
     &      zmixh**(zave))**(1._jprb/zave)
            zmixm_stable = zentrm * ztkesq / sqrt(zgbuoy(i,ik))
            zmixcm = (zmixm_stable**(zave) + &
     &      zmixm**(zave))**(1._jprb/zave)
          else
             zmixch = zmixh
             zmixcm = zmixm
          endif

          zmixquadh(i,ik) = zmixch
          zmixquadm(i,ik) = zmixcm
          zmixquadd(i,ik) = zmixcm

!       Make it fit in the CBR scheme
!       phi3 is defined on half levels consistent with the CBR scheme

          zphi3(i,ik) = zmixquadh(i,ik) / zmixquadm(i,ik)

        enddo
      enddo

      do i=kstart,kstop
        zphi3(i,klev+1)      = zphi3(i,klev)
        zphi3(i,1)           = zphi3(i,2)
        zmixquadm(i,klev+1)  = 0.01_jprb
        zmixquadm(i,1)       = 0.01_jprb
        zmixquadd(i,klev+1)  = 0.01_jprb
        zmixquadd(i,1)       = 0.01_jprb
        zmixquadh(i,klev+1)  = 0.01_jprb
        zmixquadh(i,1)       = 0.01_jprb
        zblength(i,1)        = zblength(i,2)
      enddo

!        ------------------------------------------------
!     5. Turbulent fluxes and values of variables at t+1
!        ------------------------------------------------
!    The diffusion equation is solved implicitly in time.
!    The solver is not yet optimal; small errors may result
!    from the balance between explicit processes and the
!    implicit diffusion solver
!    (see e.g., Lenderink and Holtslag, MWR, 2000, 244-258)
!    In fact, stability should be computed from full time
!    level fields, whereas the diffusion solver should use
!    the updated (intermediate time level) fields.
!    In the present code this cannot be done as one needs
!    at least the tendencies due to the explicit processes
!    or the full and intermediate time level fields.
!    Errors are probably small for dry turbulence, but may
!    significant for cloudy BL and high vertical resolution.
!    The classical method of inverting a tridiagonal matrix
!    is used. The same routine is called and what differs 
!    between variables is the coefficients of the matrix, 
!    passed as arguments to TRIDIAG. The sources (basically
!    the surface fluxes) are introduced in the independent 
!    term of the System (the Y array). The output of HLTRIDIAG
!    is the value of the variable as if it has been only 
!    forced by the turbulence (variables thlp, qtotp, cwp,
!    cicep, up, vp). With these values the tendency is
!    constructed in the paragraph 8.

      zimplmom = 1._jprb      ! 1. surface momentum flux implicit
                              ! 0. explicit

!    Implicit is used to solve instability problem resulting from
!    high surface roughness areas (see Hirlam Newsletter 38, 70-73)

      do ik=2,klev
        do i=kstart,kstop
          zkeff(i,ik) = zmixquadm(i,ik)*sqrt(ptke(i,ik-1))
          zkeff(i,ik) = max(zkeff(i,ik),1.0e-2_jprb)
        enddo
      enddo

      do i=kstart,kstop
        zkeff(i,1)=zkeff(i,2)
        zkeff(i,klev+1) = zimplmom*      &
     &    pustar(i)**2*zwzfl(i,klev)/    &
     &    (cu*max(0.0001_jprb, sqrt( pu(i,klev)**2 + pv(i,klev)**2) ))
      enddo

!     Make from keffp = rho **2 * keff

      do ik=2,klev
        do i=kstart,kstop

          zzrho = 0.5_jprb*( ppf(i,ik)/rair/pt(i,ik)     &
     &          + ppf(i,ik-1)/rair/pt(i,ik-1) )

!     NB : Maybe the exchange coefficient should be limited below 
!          some value ( approximately 10 , for zwzhf(i,ik) > zi ) 

         if( zwzhf(i,ik) > zi(i) )  then
           zkeff(i,ik)=min( zkeff(i,ik),10.0_jprb )
         endif

          zkeffp(i,ik) = zkeff(i,ik) * zzrho ** 2

        enddo
      enddo

      do i=kstart,kstop
          zzrho = zrho(i)
          zkeffp(i,klev+1) = zkeff(i,klev+1) * zzrho ** 2
          zkeffp(i,1) = zkeffp(i,2)
      enddo

!     Coefficients for the tridiagonals
!     ---------------------------------
      do ik=klev,2,-1             
        do i=kstart,kstop
          zamom(i,ik)= -pdtime*zkeffp(i,ik)/       &  
     &            (zwdzhp(i,ik)*zwdzfp(i,ik))
          zasc(i,ik)= -pdtime*zkeffp(i,ik)*zphi3(i,ik)/   &
     &            (zwdzhp(i,ik)*zwdzfp(i,ik))

!         zatke is defined on TKE-levels

          zatke(i,ik) = -pdtime*                       & 
     &    0.5_jprb*( zkeffp(i,ik) + zkeffp(i,ik+1) ) /  & 
     &    (zwdzhp(i, min(klev,ik+1))*zwdzfp(i,ik))
        enddo
      enddo

      do ik=klev-1,1,-1
        do i=kstart,kstop
          zcmom(i,ik)= -pdtime*zkeffp(i,ik+1)/   & 
     &            (zwdzhp(i,ik+1)*zwdzfp(i,ik))
          zcsc(i,ik) = -pdtime*zkeffp(i,ik+1)*zphi3(i,ik+1)/  &
     &            (zwdzhp(i,ik+1)*zwdzfp(i,ik))

!         zctke defined on tke-levels 

          zctke(i,ik) = -pdtime*                          &
     &    0.5_jprb*( zkeffp(i,ik+1)+zkeffp(i,ik+2) ) /    &
     &             ( zwdzhp(i,ik+1)*zwdzfp(i,ik+1) )

          zsource(i,ik)=0._jprb
        enddo
      enddo

      do i=kstart,kstop
        zamom(i,1)    = 0.0_jprb
        zasc(i,1)     = 0.0_jprb
        zcmom(i,klev) = 0.0_jprb
        zcsc(i,klev)  = 0.0_jprb
        zatke(i,1)    = 0.0_jprb
        zctke(i,klev) = 0.0_jprb
      enddo

!     Source due to surface fluxes and splitted fields at t+1

!     Theta_l
!     -------
      do i=kstart,kstop
        zsource(i,klev)= zwsenf(i)/zwdzf(i,klev)
      enddo

      call hltridiag( khor    ,  klev   ,  kstart ,   &
     &                kstop   ,  zthl   ,  zasc   ,   &
     &                zcsc    ,  pdtime ,  ct     ,   &
     &                zsource ,  zthlp  ,  ztkefx ,   &
     &                ztrtke  ,  zwork1 ,  zwork      )

!     qtot
!     ----

      do i=kstart,kstop
        zsource(i,klev)= zwcflx(i)/zwdzf(i,klev)
      enddo

      call hltridiag( khor    , klev    ,  kstart ,   &
     &                kstop   , ztotw   ,  zasc   ,   & 
     &                zcsc    , pdtime  ,  cq     ,   &
     &                zsource , ztotwp  ,  ztkefx ,   & 
     &                ztrtke  , zwork1  ,  zwork      )

      if (hloption(3) == 2_jpim) then
!     -------------------------------
!     Traditionally cloud water is not diffused in the case 
!     of a moist CBR scheme. 

!     Cloud water/ice parameters
!     ----------------
      do j=1,kcdnst
        do i=kstart,kstop
          zsource(i,klev)=0.0_jprb
        enddo

        call hltridiag( khor    ,  klev           ,  kstart ,   &
       &                kstop   ,  zccdnst(1,1,j) ,  zasc   ,   & 
       &                zcsc    ,  pdtime         ,  cq     ,   &
       &                zsource ,  zccdnstp(1,1,j),  ztkefx ,   & 
       &                ztrtke  ,  zwork1         ,  zwork      )
      enddo

      endif ! excluded when moist CBR

!     u-component
!     -----------
      do i=kstart,kstop
        zsource(i,klev)= - (1._jprb - zimplmom)*    &    
     &    ptaux(i) / (zrho(i) * zwdzf(i,klev))
        zkeffp(i,klev+1) = zimplmom*                &
     &    zrho(i)*abs(ptaux(i))*zwzfl(i,klev)/      & 
     &    max(0.0001_jprb , abs(pu(i,klev)))
        zcmom(i,klev)= -pdtime*zkeffp(i,klev+1)/   &
     &    (zwdzhp(i,klev+1)*zwdzfp(i,klev))
      enddo

      call hltridiag( khor    , klev   ,  kstart ,   &
     &                kstop   , pu     ,  zamom  ,   &
     &                zcmom   , pdtime ,  cu     ,   &
     &                zsource , zup    ,  ztkefx ,   &
     &                ztrtke  , zwork1,   zwork      )

!     v-component
!     -----------
      do i=kstart,kstop
        zsource(i,klev)= - (1._jprb -zimplmom)*    &
     &    ptauy(i) / (zrho(i) * zwdzf(i,klev))
        zkeffp(i,klev+1) = zimplmom*               &
     &    zrho(i)*abs(ptauy(i))*zwzfl(i,klev)/     &
     &    max(0.0001_jprb ,abs(pv(i,klev)))  
        zcmom(i,klev)= -pdtime*zkeffp(i,klev+1)/  &
     &    (zwdzhp(i,klev+1)*zwdzfp(i,klev))
      enddo

      call hltridiag( khor    ,  klev   ,  kstart , &
     &                kstop   ,  pv     ,  zamom  , &
     &                zcmom   ,  pdtime ,  cu     , &
     &                zsource ,  zvp    ,  ztkefx , &
     &                ztrtke  ,  zwork1 ,  zwork    )
 
!     6. Turbulent fluxes  (half levels)
!     ----------------------------------
! Here the turbulent fluxes of heat and water are reconstructed at
! present for their use within the TKE equation (buoyancy term).
! They are consistently computed according to the values at t+1.
! When the moist scheme will be installed in the reference system,
! in this section, the fluxes, variances and correlations of the
! conservatives variables will be computed for the option of the
! subgrid-condensation scheme.

      do ik=klev,2,-1
        do i=kstart,kstop

!         Conservative potential temperature flux
!         ---------------------------------------
          ztlfx(i,ik)=-ct*zphi3(i,ik)*zkeff(i,ik)*         &
     &            (zthlp(i,ik-1) -zthlp(i,ik))/zwdzh(i,ik)

!         total water flux
!         ----------------
          ztwfx(i,ik)=-cq*zphi3(i,ik)*zkeff(i,ik)*         &
     &            (ztotwp(i,ik-1) -ztotwp(i,ik))/zwdzh(i,ik)
        enddo
      enddo

      do i=kstart,kstop
        ztlfx(i,1)      = 0.0_jprb
        ztwfx(i,1)      = 0.0_jprb
        ztlfx(i,klev+1)= zwsenf(i)
        ztwfx(i,klev+1)= zwcflx(i)
      enddo

!     7. Evolution of TKE
!     -------------------
!     Computation shear production

      do ik=klev,2,-1
        do i=kstart,kstop
          zwork1(i,ik-1)=cu*zkeff(i,ik)*(              &
     &    (zup(i,ik-1) -zup(i,ik))*(pu(i,ik-1)-pu(i,ik)) &
     &   +(zvp(i,ik-1) -zvp(i,ik))*(pv(i,ik-1)-pv(i,ik)) &
     &                           ) /                 &
     &    (zwdzh(i,ik)**2)
        enddo
      enddo

      do i=kstart,kstop
        zwork1(i,1)    = zwork1(i,2)
        zwork1(i,klev) = 0.0_jprb   ! geert : set to zero!
      enddo

!    NB: the above modifications are not
!        tested with a prognostic surface tke

!    Buoyancy production

      do ik=klev-1,1,-1
        do i=kstart,kstop
          zbuofx(i,ik)= -zgbuoy(i,ik+1)*ct*zkeff(i,ik+1)*zphi3(i,ik+1)
        enddo
      enddo

!    Dissipation

      do ik=1,klev-1
        do i=kstart,kstop
          zwork2(i,ik)=- cdis*(ptke(i,ik)**(1.5_jprb))    &
     &                  /zmixquadd(i,ik+1)
        enddo
      enddo

      do i=kstart,kstop
        zwork2(i,klev)=0.0_jprb
      enddo

!     Implicit dissipation shear, and buoyancy.

      do ik=1,klev-1
        do i=kstart,kstop

!         Method as in ECHAM4 (Brinkop and Roeckner, .
!         basically an implicit (in time) equation for the
!         sqrt(tke) is solved. This can be done exactly
!         for buoy, shear, and dissipation (given the length 
!         scales) the provisional value tke is put into 'tkep'.

          ztkesq = ptke(i,ik)
          ztkesq  = sqrt( max( ztkesq ,1.e-3_jprb) )
          zzb =  (zwork1(i,ik) + zbuofx(i,ik))/ztkesq
          zdisl=  zmixquadd(i,ik+1)/cdis/pdtime
          zktest= 1.+(zzb*pdtime + sqrt(ptke(i,ik))*2._jprb)/zdisl

          if (zktest <= 1._jprb) then
            ztkep(i,ik) = 1e-4
          else
            ztkep(i,ik) = max(1.e-4_jprb,  &
     &      (zdisl*(sqrt(zktest) -1._jprb))**2)
          endif

        enddo
      enddo

!     Source term (buoy,shear and dissipation)

      do ik=1,klev-1
        do i=kstart,kstop
          zsource(i,ik) = (ztkep(i,ik)-ptke(i,ik))/pdtime   ! implicit
        enddo
      enddo

!     Activate if e(u*,w*) is desired

      do i=kstart,kstop
        zsource(i,klev)=0.0_jprb
      enddo

!     coefficients now zatke,zctke

      call hltridiag( khor   ,  klev   ,  kstart ,   &
     &                kstop  ,  ptke   ,  zatke  ,   &
     &                zctke  ,  pdtime ,  cte    ,   &
     &                zsource,  ztkep  ,  ztkefx ,   &
     &                ztrtke ,  zjcoef ,  zwork      )

!     Positivity control

      do ik=1,klev
        do i=kstart,kstop
          ztkep(i,ik)=max(1.e-4_jprb , ztkep(i,ik))
        enddo
      enddo

!     8. Turbulence tendencies
!        ---------------------
!    The turbulence tendencies for the prognostic
!    variables are computed from the output values of
!    the tridiagonal solution.
!    Reesatblish original TKE at lowest level 
!    for correct output of original TKE state and 
!    the computed TKE tendencies. 
           
      do ik= klev,1,-1
        do i=kstart,kstop
          pdcwdt(i,ik) =0. 
          pdicedt(i,ik)=0.
          ptke(i,klev)=zsavetke(i)
        enddo
      enddo
      if (hloption(3) == 2_jpim) then
!     --------------------------------------------------------------
      do ik=1,klev 
        do i=kstart,kstop
            pdcwdt(i,ik) =(zccdnstp(i,ik,1) -zccdnst(i,ik,1))/pdtime 
            pdicedt(i,ik)=(zccdnstp(i,ik,2) -zccdnst(i,ik,2))/pdtime
            pdtdt(i,ik)  =(zthlp(i,ik)*((ppf(i,ik)/zp0)**zkappa)  &
     &               -pt(i,ik))/pdtime
            pdqdt(i,ik)  =(ztotwp(i,ik) -ztotw(i,ik))/pdtime
        enddo
      enddo 
      endif

      if (hloption(3) == 3_jpim) then
!     ---------------------------------------------------------------
      do ik=1,klev
        do i=kstart,kstop
            pdtdt(i,ik) =(zthlp(i,ik) - zthl(i,ik))/pdtime

!           Tendency of total specific humidity

            pdqdt(i,ik) =(ztotwp(i,ik) - ztotw(i,ik))/pdtime

!           Derivation of cloud condensate flux from a
!           linear relation between total hum. flux and liquid water
!           temperature flux, assuming saturated conditions inside
!           clouds.

            zzfr=pcw(i,ik)/max( 1.0E-12_JPRB, pcw(i,ik) +pcice(i,ik) )
            zzfr=max( 0.0_jprb, min( zzfr, 1.0_jprb ) )

            pdcwdt(i,ik)=zzfr*zidqs(i,ik)*pcld2(i,ik)*   &
     &      ( pdqdt(i,ik) -zdqsatdt(i,ik)*pdtdt(i,ik) )
            pdicedt(i,ik)=(1._jprb -zzfr)*zidqs(i,ik)*pcld2(i,ik)* &
     &      ( pdqdt(i,ik) -zdqsatdt(i,ik)*pdtdt(i,ik) )

!           Specific humidity tendency as a difference between total
!           humidity tendency and cloud condensate tendencies

            pdqdt(i,ik)=pdqdt(i,ik) -pdcwdt(i,ik) -pdicedt(i,ik)

!           Conversion to normal temperature tendency

            pdtdt(i,ik)=(ppf(i,ik)/zp0)**zkappa*( pdtdt(i,ik) + &
  &         (pdcwdt(i,ik) +pdicedt(i,ik))*(latvap +flatvt(pt(i,ik)) + &
  &         latice*(1._jprb -zzfr))/zcpair(i,ik) )
        enddo
      enddo

      endif

!iv   correct neg. cloudwater/ice:
      do ik= 1,klev
        do i=kstart,kstop
          if((pcw(i,ik)/pdtime + pdcwdt(i,ik)) < 0)then
             pdcwdt(i,ik)=-pcw(i,ik)/pdtime
             pdqdt(i,ik)= pdqdt(i,ik) + pcw(i,ik)/pdtime + pdcwdt(i,ik)
             if((pdqdt(i,ik)+pq(i,ik)/pdtime) < 0)then ! avoid too low q
                pdqdt(i,ik)= (- pq(i,ik)+1.0e-8)/pdtime
             endif
             pdtdt(i,ik) = pdtdt(i,ik) - &
     &            latvap/(cpair +pq(i,ik)*cpv)* &
     &            (pcw(i,ik)/pdtime + pdcwdt(i,ik))
          endif
!         ice :
          if((pcice(i,ik)/pdtime + pdicedt(i,ik)) < 0)then
             pdicedt(i,ik)=-pcice(i,ik)/pdtime
             pdqdt(i,ik)= pdqdt(i,ik) + pcice(i,ik)/pdtime  + pdicedt(i,ik)
             if((pdqdt(i,ik) + pq(i,ik)/pdtime) < 0)then
                pdqdt(i,ik)= (- pq(i,ik)+1.0e-8)/pdtime  ! avoid too low q
             endif
             pdtdt(i,ik) = pdtdt(i,ik) - &
     &            zlatsub/(cpair +pq(i,ik)*cpv)* &
     &            (pcice(i,ik)/pdtime + pdicedt(i,ik))
          endif

        enddo
      enddo

!     Tendencies of other variables  
!     ---------------------------------------------------
      do ik=1,klev
        do i=kstart,kstop
          pdudt(i,ik)  = ( zup(i,ik) - pu(i,ik) ) /pdtime
          pdvdt(i,ik)  = ( zvp(i,ik) - pv(i,ik) ) /pdtime
          pdtkedt(i,ik)= (ztkep(i,ik) -ptke(i,ik))/pdtime
        enddo 
      enddo 


!     10. Update the sum of temperature tendencies and
!        squares of tendencies.
!     ---------------------------------------------------

      if (ldstat) then
        do ik=1,klev
          do i=kstart,kstop
            pstdif(ik) =pstdif(ik)  + pdtdt(i,ik)
            pstddif(ik)=pstddif(ik) + pdtdt(i,ik)*pdtdt(i,ik)
          enddo
        enddo
      endif

if(lhook) call dr_hook('hlvcbr',1,zhook_handle)
end subroutine hlvcbr
