!OPTIONS XOPT(NOEVAL)
SUBROUTINE LW15 ( KIDIA,KFDIA,KLON,KTDIA,KLEV,&
 & KAER,KMODE,KFLUX,KRAD,&
 & PCCO2,PCLDLD,PCLDLU,PDP,PDT0,PEMIS,PPMB,PPSOL,PQOF,PTL,&
 & PAER,PTAVE,PVIEW,PWV,&
 & PCOLR,PCOLC,PFLUX,PFLUC             )  

!**** *LW15*   - ORGANIZES THE LONGWAVE CALCULATIONS
!****            FROZEN VERSION (CYCLE 15) OF LW

!     PURPOSE.
!     --------
!           DEPENDING ON KMODE, COMPUTES LONGWAVE FLUXES AND/OR
!           RADIANCES

!**   INTERFACE.
!     ----------

!        *LW15* IS CALLED FROM *RADLSW15*

!        EXPLICIT ARGUMENTS :
!        --------------------
! PAER   : (KLON,KLEV,6)     ; OPTICAL THICKNESS OF THE AEROSOLS
! PCCO2  :                     ; CONCENTRATION IN CO2 (PA/PA)
! PCLDLD : (KLON,KLEV)       ; DOWNWARD EFFECTIVE FRACTIONAL COVER
! PCLDLU : (KLON,KLEV)       ; UPWARD EFFECTIVE FRACTIONAL COVER
! PDP    : (KLON,KLEV)       ; LAYER PRESSURE THICKNESS
! PDT0   : (KLON)             ; SURFACE TEMPERATURE DISCONTINUITY  
! PEMIS  : (KLON)             ; SURFACE EMISSIVITY
! PPMB   : (KLON,KLEV+1)     ; HALF LEVEL PRESSURE
! PPSOL  : (KLON)             ; SURFACE PRESSURE (Pa) 
! PQOF   : (KLON,KLEV)       ; CONCENTRATION IN OZONE (PA/PA)
! PTAVE  : (KLON,KLEV)       ; TEMPERATURE
! PTL    : (KLON,KLEV+1)     ; HALF LEVEL TEMPERATURE
! PVIEW  : (KLON)             ; COSECANT OF VIEWING ANGLE
! PWV    : (KLON,KLEV)       ; SPECIFIC HUMIDITY  (PA/PA)
!     ==== OUTPUTS ===
!  IF KMODE = 0, 1, 2
! PFLUX(KLON,2,KLEV)         ; RADIATIVE FLUXES :
!                     1  ==>  UPWARD   FLUX TOTAL
!                     2  ==>  DOWNWARD FLUX TOTAL
! PFLUC(KLON,2,KLEV)         ; RADIATIVE FLUXES CLEAR SKY:
!                     1  ==>  UPWARD   FLUX TOTAL
!                     2  ==>  DOWNWARD FLUX TOTAL
! PCOLR(KLON,KLEV)           ; LONG-WAVE TENDENCY
! PCOLC(KLON,KLEV)           ; LONG-WAVE TENDENCY CLEAR SKY

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. COMPUTES THE PRESSURE AND TEMPERATURE WEIGHTED AMOUNTS OF
!     ABSORBERS.
!          2. COMPUTES THE PLANCK FUNCTIONS ON THE INTERFACES AND THE
!     GRADIENT OF PLANCK FUNCTIONS IN THE LAYERS.
!          3. PERFORMS THE VERTICAL INTEGRATION DISTINGUISHING THE CON-
!     TRIBUTIONS OF THE ADJACENT AND DISTANT LAYERS AND THOSE FROM THE
!     BOUNDARIES.
!          4. COMPUTES THE CLEAR-SKY DOWNWARD AND UPWARD EMISSIVITIES.
!          5. INTRODUCES THE EFFECTS OF THE CLOUDS ON THE FLUXES.

!     EXTERNALS.
!     ----------

!          *LWU15*, *LWBV15*, *LWC15*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        96-11: Ph. Dandin. Meteo-France
!        ORIGINAL : 89-07-14 BY JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Alias       16-Jan-2006 Change PAER to 6 aerosols

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLW15  , ONLY : NUA15
USE YOMRDU15 , ONLY : RCDAY15

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM)               :: KTDIA ! Argument NOT used
INTEGER(KIND=JPIM)               :: KAER ! Argument NOT used
INTEGER(KIND=JPIM)               :: KMODE ! Argument NOT used
INTEGER(KIND=JPIM)               :: KFLUX ! Argument NOT used
INTEGER(KIND=JPIM)               :: KRAD ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCO2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPMB(KLON,KLEV+1) 
REAL(KIND=JPRB)                  :: PPSOL(KLON) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQOF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER(KLON,KLEV,6) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAVE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVIEW(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOLR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOLC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLUX(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLUC(KLON,2,KLEV+1) 
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------
REAL(KIND=JPRB) :: ZABCU(KLON,NUA15,3*KLEV+1)&
 & ,  ZBINT(KLON,KLEV+1)&
 & ,  ZBSUI(KLON)&
 & ,  ZCTS(KLON,KLEV), ZCNTRB(KLON,KLEV+1,KLEV+1)&
 & ,  ZFDN(KLON,KLEV+1)&
 & ,  ZFUP(KLON,KLEV+1)  

INTEGER(KIND=JPIM) :: ILIM, JK, JKL, JL

REAL(KIND=JPRB) :: ZDFNET
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "lwbv15.intfb.h"
#include "lwc15.intfb.h"
#include "lwu15.intfb.h"

!     ------------------------------------------------------------------

!*         1.    INITIALIZATION
!                --------------

!     ------------------------------------------------------------------

!*         1.1   COMPUTES ABSORBER AMOUNTS
!                -------------------------

IF (LHOOK) CALL DR_HOOK('LW15',0,ZHOOK_HANDLE)
CALL LWU15 ( KIDIA,KFDIA,KLON,KLEV,KAER,KMODE,KFLUX,KRAD,&
 & PAER,PCCO2,PDP,PPMB,PPSOL,PQOF,PTAVE,PVIEW,PWV,&
 & ZABCU   )  

!     ------------------------------------------------------------------

!*         2.    COMPUTES PLANCK FUNCTIONS
!                -------------------------
!                PERFORMS THE VERTICAL INTEGRATION
!                ---------------------------------

CALL LWBV15( KIDIA,KFDIA,KLON,KLEV,&
 & KMODE,KFLUX,KRAD,ILIM,&
 & PDP,PDT0,PEMIS,PPMB,PTL,&
 & PTAVE,&
 & PCOLC,PFLUC,&
 & ZABCU,ZBINT,ZBSUI,ZCTS,ZCNTRB,ZFDN,ZFUP)  

!     ------------------------------------------------------------------

!*         4.    INTRODUCES THE EFFECTS OF CLOUDS
!                --------------------------------

CALL LWC15 ( KIDIA,KFDIA,KLON,KLEV,&
 & ILIM,&
 & ZBINT,ZBSUI,PCLDLD,PCLDLU,ZCNTRB,ZCTS,PEMIS,ZFDN,ZFUP,&
 & PFLUX                                                )  

DO JKL = 1 , KLEV
  JK = KLEV+1 - JKL
  DO JL = KIDIA,KFDIA
    ZDFNET = PFLUX(JL,1,JK+1) + PFLUX(JL,2,JK+1)&
     & -PFLUX(JL,1,JK  ) - PFLUX(JL,2,JK  )  
    PCOLR(JL,JK) = RCDAY15 * ZDFNET / PDP(JL,JKL)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LW15',1,ZHOOK_HANDLE)
END SUBROUTINE LW15
