!OPTIONS XOPT(NOEVAL)
SUBROUTINE LWB15 ( KIDIA,KFDIA,KLON,KLEV,KMODE,KFLUX,KRAD,&
 & PDT0,PTAVE,PTL,&
 & PB,PBINT,PBSUIN,PBSUR,PBTOP,PDBSL,&
 & PGA,PGB,PGASUR,PGBSUR,PGATOP,PGBTOP     )  

!**** *LWB15*   - COMPUTES BLACK-BODY FUNCTIONS FOR LW CALCULATIONS
!****             FROZEN VERSION (CYCLE 15) OF LWB

!     PURPOSE.
!     --------
!           COMPUTES PLANCK FUNCTIONS

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
! PDT0   : (KLON)             ; SURFACE TEMPERATURE DISCONTINUITY
! PTAVE  : (KLON,KLEV)       ; TEMPERATURE
! PTL    : (KLON,0:KLEV)     ; HALF LEVEL TEMPERATURE
!     ==== OUTPUTS ===
! PB     : (KLON,NINT15,KLEV+1); SPECTRAL HALF LEVEL PLANCK FUNCTION
! PBINT  : (KLON,KLEV+1)     ; HALF LEVEL PLANCK FUNCTION
! PBSUIN : (KLON)             ; SURFACE PLANCK FUNCTION
! PBSUR  : (KLON,NINT15)        ; SURFACE SPECTRAL PLANCK FUNCTION
! PBTOP  : (KLON,NINT15)        ; TOP SPECTRAL PLANCK FUNCTION
! PDBSL  : (KLON,NINT15,KLEV*2); SUB-LAYER PLANCK FUNCTION GRADIENT
! PGA    : (KLON,8,2,KLEV); dB/dT-weighted LAYER PADE APPROXIMANTS
! PGB    : (KLON,8,2,KLEV); dB/dT-weighted LAYER PADE APPROXIMANTS
! PGASUR, PGBSUR (KLON,8,2)   ; SURFACE PADE APPROXIMANTS
! PGATOP, PGBTOP (KLON,8,2)   ; T.O.A. PADE APPROXIMANTS

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. COMPUTES THE PLANCK FUNCTION ON ALL LEVELS AND HALF LEVELS
!     FROM A POLYNOMIAL DEVELOPMENT OF PLANCK FUNCTION

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS           "

!     AUTHOR.
!     -------
!        96-11: Ph. Dandin. Meteo-France
!        ORIGINAL : 89-07-14 BY JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLW15  , ONLY : MXIXT15  ,NINT15   ,GA15     ,GB15     ,&
 & TINTP15  ,TSTAND15 ,TSTP15   ,XP15  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM)               :: KMODE ! Argument NOT used
INTEGER(KIND=JPIM)               :: KFLUX ! Argument NOT used
INTEGER(KIND=JPIM)               :: KRAD ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAVE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PB(KLON,NINT15,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBINT(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBSUIN(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBSUR(KLON,NINT15) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBTOP(KLON,NINT15) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDBSL(KLON,NINT15,KLEV*2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGA(KLON,8,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGB(KLON,8,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGASUR(KLON,8,2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGBSUR(KLON,8,2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGATOP(KLON,8,2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGBTOP(KLON,8,2) 
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------
INTEGER(KIND=JPIM) :: INDB(KLON),INDS(KLON)
REAL(KIND=JPRB) :: ZBLAY(KLON,KLEV),ZBLEV(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZRES(KLON),ZRES2(KLON),ZTI(KLON),ZTI2(KLON)

INTEGER(KIND=JPIM) :: INDSU, INDT, INDTO, INDTP, IXTOX, IXTX, JF,&
 & JG, JK, JK1, JK2, JL, JNU  

REAL(KIND=JPRB) :: ZDST1, ZDSTO1, ZDSTOX, ZDSTX
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*         1.0     PLANCK FUNCTIONS AND GRADIENTS
!                  ------------------------------

IF (LHOOK) CALL DR_HOOK('LWB15',0,ZHOOK_HANDLE)
DO JK = 1 , KLEV+1
  DO JL = KIDIA,KFDIA
    PBINT(JL,JK) = 0.0_JPRB
  ENDDO
ENDDO
DO JL = KIDIA,KFDIA
  PBSUIN(JL) = 0.0_JPRB
ENDDO

DO JNU=1,NINT15

!*         1.1   LEVELS FROM SURFACE TO KLEV
!                ----------------------------

  DO JK = 1 , KLEV
    DO JL = KIDIA,KFDIA
      ZTI(JL)=(PTL(JL,JK)-TSTAND15)/TSTAND15
      ZRES(JL) = XP15(1,JNU)+ZTI(JL)*(XP15(2,JNU)+ZTI(JL)*(XP15(3,&
       & JNU)&
       & +ZTI(JL)*(XP15(4,JNU)+ZTI(JL)*(XP15(5,JNU)+ZTI(JL)*(XP15(6,&
       & JNU)&
       & )))))  
      PBINT(JL,JK)=PBINT(JL,JK)+ZRES(JL)
      PB(JL,JNU,JK)= ZRES(JL)
      ZBLEV(JL,JK) = ZRES(JL)
      ZTI2(JL)=(PTAVE(JL,JK)-TSTAND15)/TSTAND15
      ZRES2(JL)=XP15(1,JNU)+ZTI2(JL)*(XP15(2,JNU)+&
       & ZTI2(JL)*(XP15(3,JNU)&
       & +ZTI2(JL)*(XP15(4,JNU)+ZTI2(JL)*(XP15(5,JNU)+&
       & ZTI2(JL)*(XP15(6,JNU)&
       & )))))  
      ZBLAY(JL,JK) = ZRES2(JL)
    ENDDO
  ENDDO

!*         1.2   TOP OF THE ATMOSPHERE AND SURFACE
!                ---------------------------------

  DO JL = KIDIA,KFDIA
    ZTI(JL)=(PTL(JL,KLEV+1)-TSTAND15)/TSTAND15
    ZTI2(JL) = (PTL(JL,1) + PDT0(JL) - TSTAND15) / TSTAND15
    ZRES(JL) = XP15(1,JNU)+ZTI(JL)*(XP15(2,JNU)+ZTI(JL)*(XP15(3,&
     & JNU)&
     & +ZTI(JL)*(XP15(4,JNU)+ZTI(JL)*(XP15(5,JNU)+ZTI(JL)*(XP15(6,&
     & JNU)&
     & )))))  
    ZRES2(JL)=XP15(1,JNU)+ZTI2(JL)*(XP15(2,JNU)+ZTI2(JL)*(XP15(3,&
     & JNU)&
     & +ZTI2(JL)*(XP15(4,JNU)+ZTI2(JL)*(XP15(5,JNU)+&
     & ZTI2(JL)*(XP15(6,JNU)&
     & )))))  
    PBINT(JL,KLEV+1) = PBINT(JL,KLEV+1)+ZRES(JL)
    PB(JL,JNU,KLEV+1)= ZRES(JL)
    ZBLEV(JL,KLEV+1) = ZRES(JL)
    PBTOP(JL,JNU) = ZRES(JL)
    PBSUR(JL,JNU) = ZRES2(JL)
    PBSUIN(JL) = PBSUIN(JL) + ZRES2(JL)
  ENDDO

!*         1.3   GRADIENTS IN SUB-LAYERS
!                -----------------------

  DO JK = 1 , KLEV
    JK2 = 2 * JK
    JK1 = JK2 - 1
    DO JL = KIDIA,KFDIA
      PDBSL(JL,JNU,JK1) = ZBLAY(JL,JK  ) - ZBLEV(JL,JK)
      PDBSL(JL,JNU,JK2) = ZBLEV(JL,JK+1) - ZBLAY(JL,JK)
    ENDDO
  ENDDO

ENDDO

!*         2.0   CHOOSE THE RELEVANT SETS OF PADE APPROXIMANTS
!                ---------------------------------------------

DO JL=KIDIA,KFDIA
  ZDSTO1 = (PTL(JL,KLEV+1)-TINTP15(1)) / TSTP15
  IXTOX = MAX( 1, MIN( MXIXT15, INT( ZDSTO1 + 1.0_JPRB ) ) )
  ZDSTOX = (PTL(JL,KLEV+1)-TINTP15(IXTOX))/TSTP15
  IF (ZDSTOX < 0.5_JPRB) THEN
    INDTO=IXTOX
  ELSE
    INDTO=IXTOX+1
  ENDIF
  INDB(JL)=INDTO
  ZDST1 = (PTL(JL,1)-TINTP15(1)) / TSTP15
  IXTX = MAX( 1, MIN( MXIXT15, INT( ZDST1 + 1.0_JPRB ) ) )
  ZDSTX = (PTL(JL,1)-TINTP15(IXTX))/TSTP15
  IF (ZDSTX < 0.5_JPRB) THEN
    INDT=IXTX
  ELSE
    INDT=IXTX+1
  ENDIF
  INDS(JL)=INDT
ENDDO

DO JF=1,2
  DO JG=1, 8
    DO JL=KIDIA,KFDIA
      INDSU=INDS(JL)
      PGASUR(JL,JG,JF)=GA15(INDSU,2*JG-1,JF)
      PGBSUR(JL,JG,JF)=GB15(INDSU,2*JG-1,JF)
      INDTP=INDB(JL)
      PGATOP(JL,JG,JF)=GA15(INDTP,2*JG-1,JF)
      PGBTOP(JL,JG,JF)=GB15(INDTP,2*JG-1,JF)
    ENDDO
  ENDDO
ENDDO

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZDST1 = (PTAVE(JL,JK)-TINTP15(1)) / TSTP15
    IXTX = MAX( 1, MIN( MXIXT15, INT( ZDST1 + 1.0_JPRB ) ) )
    ZDSTX = (PTAVE(JL,JK)-TINTP15(IXTX))/TSTP15
    IF (ZDSTX < 0.5_JPRB) THEN
      INDT=IXTX
    ELSE
      INDT=IXTX+1
    ENDIF
    INDB(JL)=INDT
  ENDDO

  DO JF=1,2
    DO JG=1, 8
      DO JL=KIDIA,KFDIA
        INDT=INDB(JL)
        PGA(JL,JG,JF,JK)=GA15(INDT,2*JG,JF)
        PGB(JL,JG,JF,JK)=GB15(INDT,2*JG,JF)
      ENDDO
    ENDDO
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWB15',1,ZHOOK_HANDLE)
END SUBROUTINE LWB15
