!OPTIONS XOPT(NOEVAL)
SUBROUTINE LWV15 ( KIDIA,KFDIA,KLON,KLEV,&
 & KMODE,KFLUX,KRAD,KUAER,KTRAER,&
 & KLIM,&
 & PABCU,PB,PBINT,PBSUIN,PBSUR,PBTOP,PDBSL,PEMIS,PPMB,PTAVE,&
 & PGA,PGB,PGASUR,PGBSUR,PGATOP,PGBTOP,&
 & PCNTRB,PCTS,PCOLC,PFLUC                 )  

!**** *LWV15*   - LONGWAVE RADIATION, VERTICAL INTEGRATION
!****             FROZEN VERSION (CYCLE 15) OF LWV

!     PURPOSE.
!     --------
!           CARRIES OUT THE VERTICAL INTEGRATION TO GIVE LONGWAVE
!           FLUXES OR RADIANCES

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
! PABCU : (KLON,NUA15,3*KLEV+1); ABSORBER AMOUNTS
! PB     : (KLON,NINT15,KLEV+1); SPECTRAL HALF-LEVEL PLANCK FUNCTIONS
! PBINT  : (KLON,KLEV+1)     ; HALF-LEVEL PLANCK FUNCTIONS
! PBSUIN : (KLON)             ; SURFACE PLANCK FUNCTION
! PBSUR  : (KLON,NINT15)        ; SURFACE SPECTRAL PLANCK FUNCTION
! PBTOP  : (KLON,NINT15)        ; T.O.A. SPECTRAL PLANCK FUNCTION
! PDBSL  : (KLON,KLEV*2)     ; SUB-LAYER PLANCK FUNCTION GRADIENT
! PEMIS  : (KLON)             ; SURFACE EMISSIVITY
! PPMB   : )KLON,KLEV+1)     ; HALF-LEVEL PRESSURE (MB)
! PTAVE  : (KLON,KLEV)       ; TEMPERATURE
! PGA, PGB                     ; PADE APPROXIMANTS
! PGASUR, PGBSUR               ; SURFACE PADE APPROXIMANTS
! PGATOP, PGBTOP               ; T.O.A. PADE APPROXIMANTS
!     ==== OUTPUTS ===
! PCNTRB : (KLON,KLEV+1,KLEV+1); CLEAR-SKY ENERGY EXCHANGE MATRIX
! PCTS   : (KLON,KLEV)       ; COOLING-TO-SPACE TERM
! PFLUC(KLON,2,KLEV)         ; RADIATIVE FLUXES CLEAR-SKY:
!                     1  ==>  UPWARD   FLUX TOTAL
! PCOLC(KLON,KLEV)           ; LONG-WAVE TENDENCY CLEAR SKY

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. PERFORMS THE VERTICAL INTEGRATION DISTINGUISHING BETWEEN
!     CONTRIBUTIONS BY -  THE NEARBY LAYERS
!                      -  THE DISTANT LAYERS
!                      -  THE BOUNDARY TERMS
!          2. COMPUTES THE CLEAR-SKY DOWNWARD AND UPWARD EMISSIVITIES.

!     EXTERNALS.
!     ----------

!          *LWVN15*, *LWVD15*, *LWVB15*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        96-11: Ph. Dandin. Meteo-France
!        ORIGINAL : 89-07-14 BY JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLW15  , ONLY : NINT15   ,NUA15

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM)               :: KMODE ! Argument NOT used
INTEGER(KIND=JPIM)               :: KFLUX ! Argument NOT used
INTEGER(KIND=JPIM)               :: KRAD ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(IN)    :: KUAER 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAER 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLIM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU(KLON,NUA15,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PB(KLON,NINT15,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBINT(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBSUIN(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBSUR(KLON,NINT15) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBTOP(KLON,NINT15) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDBSL(KLON,NINT15,KLEV*2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS(KLON) 
REAL(KIND=JPRB)                  :: PPMB(KLON,KLEV+1) ! Argument NOT used
REAL(KIND=JPRB)                  :: PTAVE(KLON,KLEV) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGA(KLON,8,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGB(KLON,8,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGASUR(KLON,8,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGBSUR(KLON,8,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGATOP(KLON,8,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGBTOP(KLON,8,2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCNTRB(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCTS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOLC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLUC(KLON,2,KLEV+1) 
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZADJD(KLON,KLEV+1), ZADJU(KLON,KLEV+1)&
 & ,  ZDBDT(KLON,NINT15,KLEV)&
 & ,  ZDISD(KLON,KLEV+1), ZDISU(KLON,KLEV+1)  

INTEGER(KIND=JPIM) :: JK, JKL, JL

REAL(KIND=JPRB) :: ZDFNET
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "lwvb15.intfb.h"
#include "lwvd15.intfb.h"
#include "lwvn15.intfb.h"

!-----------------------------------------------------------------------

!*         1.    INITIALIZATION
!                --------------

!*         1.1     INITIALIZE LAYER CONTRIBUTIONS
!                  ------------------------------

IF (LHOOK) CALL DR_HOOK('LWV15',0,ZHOOK_HANDLE)
DO JK=1,KLEV+1
  DO JL=KIDIA,KFDIA
    ZADJD(JL,JK)=0.0_JPRB
    ZADJU(JL,JK)=0.0_JPRB
    ZDISD(JL,JK)=0.0_JPRB
    ZDISU(JL,JK)=0.0_JPRB
  ENDDO
ENDDO

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    PCTS(JL,JK)=0.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         2.      VERTICAL INTEGRATION
!                  --------------------

!     ------------------------------------------------------------------

!*         2.1     CONTRIBUTION FROM ADJACENT LAYERS
!                  ---------------------------------

CALL LWVN15 ( KIDIA,KFDIA,KLON,KLEV,KUAER,KTRAER,&
 & PABCU,PDBSL,PGA,PGB,&
 & ZADJD,ZADJU,PCNTRB,ZDBDT                              )  

!     ------------------------------------------------------------------

!*         2.2     CONTRIBUTION FROM DISTANT LAYERS
!                  ---------------------------------

CALL LWVD15 ( KIDIA,KFDIA,KLON,KLEV,KUAER,KTRAER,&
 & PABCU,ZDBDT,PGA,PGB,&
 & PCNTRB,ZDISD,ZDISU                             )  

!     ------------------------------------------------------------------

!*         2.3     EXCHANGE WITH THE BOUNDARIES
!                  ----------------------------

CALL LWVB15 ( KIDIA,KFDIA,KLON,KLEV,KUAER,KTRAER,&
 & KLIM,&
 & PABCU,ZADJD,ZADJU,PB,PBINT,PBSUIN,PBSUR,PBTOP,&
 & ZDISD,ZDISU,PEMIS,PPMB,&
 & PGA,PGB,PGASUR,PGBSUR,PGATOP,PGBTOP,&
 & PCTS,PFLUC                     )  

!*         2.4     CLEAR-SKY FLUXES
!                  ----------------

!     ------------------------------------------------------------------

!*         3.      EFFECTIVE DOWNWARD AND UPWARD CLEAR-SKY EMISSIVITIES
!                  ----------------------------------------------------

DO JKL = 1 , KLEV
  JK = KLEV+1 - JKL
  DO JL = KIDIA,KFDIA
    ZDFNET = PFLUC(JL,1,JK+1) + PFLUC(JL,2,JK+1)&
     & -PFLUC(JL,1,JK  ) - PFLUC(JL,2,JK  )  
    PCOLC(JL,JK) = ZDFNET
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('LWV15',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------
!                       FORMATS
!                       -------

884 FORMAT (1X,F7.2,18F6.1,F7.2)
886 FORMAT (1X,20I5)
888 FORMAT (1X,13E9.2)
890 FORMAT (1X,2I4,13F9.2)

END SUBROUTINE LWV15
