!OPTIONS XOPT(NOEVAL)
SUBROUTINE LWVB15 ( KIDIA,KFDIA,KLON,KLEV,KUAER,KTRAER,&
 & KLIM,&
 & PABCU,PADJD,PADJU,PB,PBINT,PBSUI,PBSUR,PBTOP,&
 & PDISD,PDISU,PEMIS,PPMB,&
 & PGA,PGB,PGASUR,PGBSUR,PGATOP,PGBTOP,&
 & PCTS,PFLUC                     )  

!**** *LWVB15*   - L.W., VERTICAL INTEGRATION, EXCHANGE WITH BOUNDARIES
!****              FROZEN VERSION (CYCLE 15) OF LWVB

!     PURPOSE.
!     --------
!           INTRODUCES THE EFFECTS OF THE BOUNDARIES IN THE VERTICAL
!           INTEGRATION

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
! PABCU : (KLON,NUA15,3*KLEV+1); ABSORBER AMOUNTS
! PADJ.. : (KLON,KLEV+1)     ; CONTRIBUTION BY ADJACENT LAYERS
! PB     : (KLON,NINT15,KLEV+1); SPECTRAL HALF-LEVEL PLANCK FUNCTIONS
! PBINT  : (KLON,KLEV+1)     ; HALF-LEVEL PLANCK FUNCTIONS
! PBSUI  : (KLON)             ; SURFACE PLANCK FUNCTION
! PBSUR  : (KLON,NINT15)        ; SPECTRAL SURFACE PLANCK FUNCTION
! PBTOP  : (KLON,NINT15)        ; SPECTRAL T.O.A. PLANCK FUNCTION
! PDIS.. : (KLON,KLEV+1)     ; CONTRIBUTION BY DISTANT LAYERS
! PEMIS  : (KLON)             ; SURFACE EMISSIVITY
! PPMB   : (KLON,KLEV+1)     ; PRESSURE MB
! PGA, PGB                     ; PADE APPROXIMANTS
! PGASUR, PGBSUR               ; SURFACE PADE APPROXIMANTS
! PGATOP, PGBTOP               ; T.O.A. PADE APPROXIMANTS 
!     ==== OUTPUTS ===
! PCTS   : (KLON,KLEV)       ; COOLING-TO-SPACE TERM
! PFLUC(KLON,2,KLEV)         ; RADIATIVE FLUXES CLEAR-SKY:
!                     1  ==>  UPWARD   FLUX TOTAL

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. COMPUTES THE ENERGY EXCHANGE WITH TOP AND SURFACE OF THE
!     ATMOSPHERE
!          2. COMPUTES THE COOLING-TO-SPACE AND HEATING-FROM-GROUND
!     TERMS FOR THE APPROXIMATE COOLING RATE ABOVE 10 HPA
!          3. ADDS UP ALL CONTRIBUTIONS TO GET THE CLEAR-SKY FLUXES

!     EXTERNALS.
!     ----------

!          *LWTT15*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        96-11: Ph. Dandin. Meteo-France
!        ORIGINAL : 89-07-14 BY JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLW15  , ONLY : NINT15   ,NTRA15   ,NUA15    ,NG1P115

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KUAER 
INTEGER(KIND=JPIM)               :: KTRAER ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(OUT)   :: KLIM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU(KLON,NUA15,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PADJD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PADJU(KLON,KLEV+1) 
REAL(KIND=JPRB)                  :: PB(KLON,NINT15,KLEV+1) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBINT(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBSUI(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBSUR(KLON,NINT15) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBTOP(KLON,NINT15) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDISD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDISU(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS(KLON) 
REAL(KIND=JPRB)                  :: PPMB(KLON,KLEV+1) ! Argument NOT used
REAL(KIND=JPRB)                  :: PGA(KLON,8,2,KLEV) ! Argument NOT used
REAL(KIND=JPRB)                  :: PGB(KLON,8,2,KLEV) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGASUR(KLON,8,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGBSUR(KLON,8,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGATOP(KLON,8,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGBTOP(KLON,8,2) 
REAL(KIND=JPRB)                  :: PCTS(KLON,KLEV) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLUC(KLON,2,KLEV+1) 
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZBGND(KLON), ZFD(KLON), ZFDN(KLON,KLEV+1)&
 & , ZFU(KLON), ZFUP(KLON,KLEV+1)&
 & ,  ZTT(KLON,NTRA15), ZTT1(KLON,NTRA15), ZTT2(KLON,NTRA15)&
 & ,  ZUU(KLON,NUA15) , ZCNSOL(KLON), ZCNTOP(KLON)  

INTEGER(KIND=JPIM) :: IN, IND1, IND2, IND3, IND4, JA, JK, JL, JLIM
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "lwtt15.intfb.h"

!-----------------------------------------------------------------------

!*         1.    INITIALIZATION
!                --------------

!*         1.2     INITIALIZE TRANSMISSION FUNCTIONS
!                  ---------------------------------

IF (LHOOK) CALL DR_HOOK('LWVB15',0,ZHOOK_HANDLE)
DO JA=1,NTRA15
  DO JL=KIDIA,KFDIA
    ZTT (JL,JA)=1.0_JPRB
    ZTT1(JL,JA)=1.0_JPRB
    ZTT2(JL,JA)=1.0_JPRB
  ENDDO
ENDDO

DO JA=1,NUA15
  DO JL=KIDIA,KFDIA
    ZUU(JL,JA)=1.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         2.      VERTICAL INTEGRATION
!                  --------------------

IND1=0
IND3=0
IND4=1
IND2=1

!*         2.3     EXCHANGE WITH TOP OF THE ATMOSPHERE
!                  -----------------------------------

DO JK = 1 , KLEV
  IN=(JK-1)*NG1P115+1

  DO JA=1,KUAER
    DO JL=KIDIA,KFDIA
      ZUU(JL,JA)=PABCU(JL,JA,IN)
    ENDDO
  ENDDO

  CALL LWTT15 ( KIDIA,KFDIA,KLON, PGATOP(1,1,1), PGBTOP(1,1,1), ZUU, ZTT )

  DO JL = KIDIA,KFDIA
    ZCNTOP(JL)=PBTOP(JL,1)*ZTT(JL,1)          *ZTT(JL,10)&
     & +PBTOP(JL,2)*ZTT(JL,2)*ZTT(JL,7)*ZTT(JL,11)&
     & +PBTOP(JL,3)*ZTT(JL,4)*ZTT(JL,8)*ZTT(JL,12)&
     & +PBTOP(JL,4)*ZTT(JL,5)*ZTT(JL,9)*ZTT(JL,13)&
     & +PBTOP(JL,5)*ZTT(JL,3)          *ZTT(JL,14)&
     & +PBTOP(JL,6)*ZTT(JL,6)          *ZTT(JL,15)  
    ZFD(JL)=ZCNTOP(JL)-PBINT(JL,JK)-PDISD(JL,JK)-PADJD(JL,JK)
    ZFDN(JL,JK)=ZFD(JL)
    PFLUC(JL,2,JK)=ZFD(JL)
  ENDDO

ENDDO

JK = KLEV+1
IN=(JK-1)*NG1P115+1

DO JL = KIDIA,KFDIA
  ZCNTOP(JL)= PBTOP(JL,1)&
   & + PBTOP(JL,2)&
   & + PBTOP(JL,3)&
   & + PBTOP(JL,4)&
   & + PBTOP(JL,5)&
   & + PBTOP(JL,6)  
  ZFD(JL)=ZCNTOP(JL)-PBINT(JL,JK)-PDISD(JL,JK)-PADJD(JL,JK)
  ZFDN(JL,JK)=ZFD(JL)
  PFLUC(JL,2,JK)=ZFD(JL)
ENDDO

!*         2.4     COOLING-TO-SPACE OF LAYERS ABOVE 10 HPA
!                  ---------------------------------------

!*         2.4.1   INITIALIZATION
!                  --------------

JLIM = KLEV
KLIM=JLIM

!*         2.5     EXCHANGE WITH LOWER LIMIT
!                  -------------------------

DO JL = KIDIA,KFDIA
  ZBGND(JL)=PBSUI(JL)*PEMIS(JL)-(1.0_JPRB-PEMIS(JL))*PFLUC(JL,2,1)-PBINT(JL,1)
ENDDO

JK = 1
IN=(JK-1)*NG1P115+1

DO JL = KIDIA,KFDIA
  ZCNSOL(JL)=PBSUR(JL,1)&
   & +PBSUR(JL,2)&
   & +PBSUR(JL,3)&
   & +PBSUR(JL,4)&
   & +PBSUR(JL,5)&
   & +PBSUR(JL,6)  
  ZCNSOL(JL)=ZCNSOL(JL)*ZBGND(JL)/PBSUI(JL)
  ZFU(JL)=ZCNSOL(JL)+PBINT(JL,JK)-PDISU(JL,JK)-PADJU(JL,JK)
  ZFUP(JL,JK)=ZFU(JL)
  PFLUC(JL,1,JK)=ZFU(JL)
ENDDO

DO JK = 2 , KLEV+1
  IN=(JK-1)*NG1P115+1

  DO JA=1,KUAER
    DO JL=KIDIA,KFDIA
      ZUU(JL,JA)=PABCU(JL,JA,1)-PABCU(JL,JA,IN)
    ENDDO
  ENDDO

  CALL LWTT15 ( KIDIA,KFDIA,KLON, PGASUR(1,1,1), PGBSUR(1,1,1), ZUU, ZTT )

  DO JL = KIDIA,KFDIA
    ZCNSOL(JL)=PBSUR(JL,1)*ZTT(JL,1)          *ZTT(JL,10)&
     & +PBSUR(JL,2)*ZTT(JL,2)*ZTT(JL,7)*ZTT(JL,11)&
     & +PBSUR(JL,3)*ZTT(JL,4)*ZTT(JL,8)*ZTT(JL,12)&
     & +PBSUR(JL,4)*ZTT(JL,5)*ZTT(JL,9)*ZTT(JL,13)&
     & +PBSUR(JL,5)*ZTT(JL,3)          *ZTT(JL,14)&
     & +PBSUR(JL,6)*ZTT(JL,6)          *ZTT(JL,15)  
    ZCNSOL(JL)=ZCNSOL(JL)*ZBGND(JL)/PBSUI(JL)
    ZFU(JL)=ZCNSOL(JL)+PBINT(JL,JK)-PDISU(JL,JK)-PADJU(JL,JK)
    ZFUP(JL,JK)=ZFU(JL)
    PFLUC(JL,1,JK)=ZFU(JL)
  ENDDO

ENDDO

!*         2.7     CLEAR-SKY FLUXES
!                  ----------------

DO JK = 1 , KLEV+1
  DO JL = KIDIA,KFDIA
    PFLUC(JL,1,JK) = ZFUP(JL,JK)
    PFLUC(JL,2,JK) = ZFDN(JL,JK)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWVB15',1,ZHOOK_HANDLE)
END SUBROUTINE LWVB15
