!OPTIONS XOPT(NOEVAL)
SUBROUTINE LWVD15 ( KIDIA,KFDIA,KLON,KLEV,KUAER,KTRAER,&
 & PABCU,PDBDT,&
 & PGA,PGB,&
 & PCNTRB,PDISD,PDISU                              )  

!**** *LWVD15*   - L.W., VERTICAL INTEGRATION, DISTANT LAYERS
!****              FROZEN VERSION (CYCLE 15) OF LWVD

!     PURPOSE.
!     --------
!           CARRIES OUT THE VERTICAL INTEGRATION ON THE DISTANT LAYERS

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
! PABCU : (KLON,NUA15,3*KLEV+1); ABSORBER AMOUNTS
! PDBDT  : (KLON,KLEV)       ; LAYER PLANCK FUNCTION GRADIENT
! PGA, PGB                     ; PADE APPROXIMANTS
!     ==== OUTPUTS ===
! PDIS.. : (KLON,KLEV+1)     ; CONTRIBUTION BY DISTANT LAYERS
! PCNTRB : (KLON,KLEV+1,KLEV+1); ENERGY EXCHANGE MATRIX

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. PERFORMS THE VERTICAL INTEGRATION CORRESPONDING TO THE
!     CONTRIBUTIONS OF THE DISTANT LAYERS USING TRAPEZOIDAL RULE

!     EXTERNALS.
!     ----------

!          *LWTTM15*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        96-11: Ph. Dandin. Meteo-France
!        ORIGINAL : 89-07-14 BY JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLW15  , ONLY : NINT15   ,NTRA15   ,NUA15    ,NG1P115

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM)               :: KUAER ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAER 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU(KLON,NUA15,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDBDT(KLON,NINT15,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGA(KLON,8,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGB(KLON,8,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCNTRB(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDISD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDISU(KLON,KLEV+1) 
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZGLAYD(KLON),ZGLAYU(KLON)&
 & ,  ZTT(KLON,NTRA15), ZTT1(KLON,NTRA15), ZTT2(KLON,NTRA15)  

INTEGER(KIND=JPIM) :: IJKL, IKD1, IKD2, IKJ, IKJP1, IKM1, IKN,&
 & IKP1, IKU1, IKU2, IND1, IND2, IND3, IND4, &
 & ITT, JA, JK, JKJ, JL, JLK  

REAL(KIND=JPRB) :: ZDZXDG, ZDZXMG, ZWW
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "lwttm15.intfb.h"

!-----------------------------------------------------------------------

!*         1.    INITIALIZATION
!                --------------

!*         1.1     INITIALIZE LAYER CONTRIBUTIONS
!                  ------------------------------

IF (LHOOK) CALL DR_HOOK('LWVD15',0,ZHOOK_HANDLE)
DO JK = 1, KLEV+1
  DO JL = KIDIA,KFDIA
    PDISD(JL,JK) = 0.0_JPRB
    PDISU(JL,JK) = 0.0_JPRB
  ENDDO
ENDDO

!*         1.2     INITIALIZE TRANSMISSION FUNCTIONS
!                  ---------------------------------

DO JA = 1, NTRA15
  DO JL = KIDIA,KFDIA
    ZTT (JL,JA) = 1.0_JPRB
    ZTT1(JL,JA) = 1.0_JPRB
    ZTT2(JL,JA) = 1.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         2.      VERTICAL INTEGRATION
!                  --------------------

IND1=0
IND3=0
IND4=1
IND2=1

!*         2.2     CONTRIBUTION FROM DISTANT LAYERS
!                  ---------------------------------

!*         2.2.1   DISTANT AND ABOVE LAYERS
!                  ------------------------

!*         2.2.2   FIRST UPPER LEVEL
!                  -----------------

DO JK = 1 , KLEV-1
  IKP1=JK+1
  IKN=(JK-1)*NG1P115+1
  IKD1= JK  *NG1P115+1

  CALL LWTTM15( KIDIA,KFDIA,KLON,&
   & PGA(1,1,1,JK), PGB(1,1,1,JK),&
   & PABCU(1,1,IKN),PABCU(1,1,IKD1),ZTT1)  

!*         2.2.3   HIGHER UP
!                  ---------

  ITT=1
  DO JKJ=IKP1,KLEV
    IF(ITT == 1) THEN
      ITT=2
    ELSE
      ITT=1
    ENDIF
    IKJP1=JKJ+1
    IKD2= JKJ  *NG1P115+1

    IF(ITT == 1) THEN
      CALL LWTTM15( KIDIA,KFDIA,KLON,&
       & PGA(1,1,1,JKJ),PGB(1,1,1,JKJ),&
       & PABCU(1,1,IKN),PABCU(1,1,IKD2),ZTT1)  
    ELSE
      CALL LWTTM15( KIDIA,KFDIA,KLON,&
       & PGA(1,1,1,JKJ),PGB(1,1,1,JKJ),&
       & PABCU(1,1,IKN),PABCU(1,1,IKD2),ZTT2)  
    ENDIF

    DO JA = 1, KTRAER
      DO JL = KIDIA,KFDIA
        ZTT(JL,JA) = (ZTT1(JL,JA)+ZTT2(JL,JA))*0.5_JPRB
      ENDDO
    ENDDO

    DO JL = KIDIA,KFDIA
      ZWW=PDBDT(JL,1,JKJ)*ZTT(JL,1)          *ZTT(JL,10)&
       & +PDBDT(JL,2,JKJ)*ZTT(JL,2)*ZTT(JL,7)*ZTT(JL,11)&
       & +PDBDT(JL,3,JKJ)*ZTT(JL,4)*ZTT(JL,8)*ZTT(JL,12)&
       & +PDBDT(JL,4,JKJ)*ZTT(JL,5)*ZTT(JL,9)*ZTT(JL,13)&
       & +PDBDT(JL,5,JKJ)*ZTT(JL,3)          *ZTT(JL,14)&
       & +PDBDT(JL,6,JKJ)*ZTT(JL,6)          *ZTT(JL,15)  
      ZGLAYD(JL)=ZWW
      ZDZXDG=ZGLAYD(JL)
      PDISD(JL,JK)=PDISD(JL,JK)+ZDZXDG
      PCNTRB(JL,JK,IKJP1)=ZDZXDG
    ENDDO

  ENDDO
ENDDO

!*         2.2.4   DISTANT AND BELOW LAYERS
!                  ------------------------

!*         2.2.5   FIRST LOWER LEVEL
!                  -----------------

DO JK=3,KLEV+1
  IKN=(JK-1)*NG1P115+1
  IKM1=JK-1
  IKJ=JK-2
  IKU1= IKJ  *NG1P115+1

  CALL LWTTM15( KIDIA,KFDIA,KLON,&
   & PGA(1,1,1,IKJ),PGB(1,1,1,IKJ),&
   & PABCU(1,1,IKU1),PABCU(1,1,IKN),ZTT1)  

!*         2.2.6   DOWN BELOW
!                  ----------

  ITT=1
  DO JLK=1,IKJ
    IF(ITT == 1) THEN
      ITT=2
    ELSE
      ITT=1
    ENDIF
    IJKL=IKM1-JLK
    IKU2=(IJKL-1)*NG1P115+1

    IF(ITT == 1) THEN
      CALL LWTTM15( KIDIA,KFDIA,KLON,&
       & PGA(1,1,1,IJKL),PGB(1,1,1,IJKL),&
       & PABCU(1,1,IKU2),PABCU(1,1,IKN),ZTT1)  
    ELSE
      CALL LWTTM15( KIDIA,KFDIA,KLON,&
       & PGA(1,1,1,IJKL),PGB(1,1,1,IJKL),&
       & PABCU(1,1,IKU2),PABCU(1,1,IKN),ZTT2)  
    ENDIF

    DO JA = 1, KTRAER
      DO JL = KIDIA,KFDIA
        ZTT(JL,JA) = (ZTT1(JL,JA)+ZTT2(JL,JA))*0.5_JPRB
      ENDDO
    ENDDO

    DO JL = KIDIA,KFDIA
      ZWW=PDBDT(JL,1,IJKL)*ZTT(JL,1)          *ZTT(JL,10)&
       & +PDBDT(JL,2,IJKL)*ZTT(JL,2)*ZTT(JL,7)*ZTT(JL,11)&
       & +PDBDT(JL,3,IJKL)*ZTT(JL,4)*ZTT(JL,8)*ZTT(JL,12)&
       & +PDBDT(JL,4,IJKL)*ZTT(JL,5)*ZTT(JL,9)*ZTT(JL,13)&
       & +PDBDT(JL,5,IJKL)*ZTT(JL,3)          *ZTT(JL,14)&
       & +PDBDT(JL,6,IJKL)*ZTT(JL,6)          *ZTT(JL,15)  
      ZGLAYU(JL)=ZWW
      ZDZXMG=ZGLAYU(JL)
      PDISU(JL,JK)=PDISU(JL,JK)+ZDZXMG
      PCNTRB(JL,JK,IJKL)=ZDZXMG
    ENDDO

  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('LWVD15',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------
!                       FORMATS
!                       -------

884 FORMAT (1X,F7.2,18F6.1,F7.2)
886 FORMAT (1X,20I5)
888 FORMAT (1X,13E9.2)
890 FORMAT (1X,2I4,13F9.2)

END SUBROUTINE LWVD15
