!OPTIONS XOPT(NOEVAL)
SUBROUTINE RADLSW15( KIDIA, KFDIA,KLON, KTDIA,KLEV,&
 & KMODE, KFLUX , KRAD , KAER,&
 & PRII0,&
 & PAER , PALBD , PALBP, PAPH , PAP,&
 & PCCO2, PCLFR , PDP  , PEMIS, PLSM, PMU0, POZON,&
 & PQ   , PQIWP , PQLWP, PQS,&
 & PTH  , PT    , PTS,&
 & PFCT , PFLT  , PFCS , PFLS , PFRSOD           )  

!**** *RADLSW* - INTERFACE TO ECMWF LW AND SW RADIATION SCHEMES
!****            FROZEN VERSION (CYCLE 15) OF RADLSW

!     PURPOSE.
!     --------
!           CONTROLS RADIATION COMPUTATIONS

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
! PAER   : (KLON,KLEV,6)     ; OPTICAL THICKNESS OF THE AEROSOLS
! PALBD  : (KLON,2)          ; SURF. SW ALBEDO FOR DIFFUSE RADIATION
! PALBP  : (KLON,2)          ; SURF. SW ALBEDO FOR PARALLEL RADIATION
! PAPH   : (KLON,KLEV+1)     ; HALF LEVEL PRESSURE
! PAP    : (KLON,KLEV)       ; FULL LEVEL PRESSURE
! PCCO2  :                   ; CONCENTRATION IN CO2 (PA/PA)
! PCLFR  : (KLON,KLEV)       ; CLOUD FRACTIONAL COVER
! PDP    : (KLON,KLEV)       ; LAYER PRESSURE THICKNESS
! PEMIS  : (KLON)            ; SURFACE EMISSIVITY
! PLSM   : (KLON)            ; LAND-SEA MASK
! PMU0   : (KLON)            ; SOLAR ANGLE
! POZON  : (KLON,KLEV)       ; CONCENTRATION IN OZONE (PA/PA)
! PQ     : (KLON,KLEV)       ; SPECIFIC HUMIDITY PA/PA
! PQIWP  : (KLON,KLEV)       ; SOLID  WATER KG/KG
! PQLWP  : (KLON,KLEV)       ; LIQUID WATER KG/KG
! PQS    : (KLON,KLEV)       ; SATURATION WATER VAPOR  KG/KG
! PTH    : (KLON,KLEV+1)     ; HALF LEVEL TEMPERATURE
! PT     : (KLON,KLEV)       ; FULL LEVEL TEMPERATURE
! PTS    : (KLON)            ; SURFACE TEMPERATURE
!     ==== OUTPUTS ===
! PFCT   : (KLON,KLEV+1)     ; CLEAR-SKY LW NET FLUXES
! PFLT   : (KLON,KLEV+1)     ; TOTAL LW NET FLUXES
! PFCS   : (KLON,KLEV+1)     ; CLEAR-SKY SW NET FLUXES
! PFLS   : (KLON,KLEV+1)     ; TOTAL SW NET FLUXES
! PFRSOD : (KLON)            ; TOTAL-SKY SURFACE SW DOWNWARD FLUX

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHORS.
!     --------
!        96-11: Ph. Dandin. Meteo-France
!        ORIGINAL : 88-02-04 BY J.-J. MORCRETTE         *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Alias       19-Dec-2005 adding Sulfate aerosols(Hu Rong Ming)
!        P. Marquet    feb-2006    REFFWIA15 introduced
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,R        ,RD
USE YOMLW15  , ONLY : NTRA15   ,NUA15
USE YOMRDU15 , ONLY : NUAER15  ,NTRAER15 ,REELOG15 ,REPSC15  ,DIFF15
USE YOMRDI15 , ONLY : REPCLC15
USE YOMCLOP15, ONLY : RYFWCA15 ,RYFWCB15 ,RYFWCC15 ,RYFWCD15 ,&
 & RYFWCE15 ,RYFWCF15 ,REBCUA15 ,REBCUB15 ,REBCUC15 ,&
 & REBCUD15 ,REBCUE15 ,REBCUF15 ,REFFIA15 ,REFFIB15 ,REFFWIA15 

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KTDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLUX
INTEGER(KIND=JPIM),INTENT(IN)    :: KRAD
INTEGER(KIND=JPIM),INTENT(IN)    :: KAER
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRII0
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER(KLON,KLEV,6)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD(KLON,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP(KLON,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCO2 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLFR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDP(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS(KLON) 
REAL(KIND=JPRB)                  :: PLSM(KLON) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMU0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POZON(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQIWP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLWP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCT(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLT(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCS(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLS(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOD(KLON) 
!-----------------------------------------------------------------------

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------
!     ==== COMPUTED IN RADLSW ===
!     -----------------------------------------------------------------

!*       0.2   LOCAL ARRAYS.
!              -------------
!     -----------------------------------------------------------------

!-- ARRAYS FOR LOCAL VARIABLES -----------------------------------------

REAL(KIND=JPRB) :: ZALBD(KLON,2), ZALBP(KLON,2) &
 & , ZCG(KLON,2,KLEV)   , ZCLDLD(KLON,KLEV), ZCLDLU(KLON,KLEV)&
 & , ZCLDSW(KLON,KLEV)  &
 & , ZDT0(KLON)         , ZEMIS(KLON)      , ZFLUX (KLON,2,KLEV+1)                 , ZFLUC(KLON,2,KLEV+1)&
 & , ZFIWP(KLON)        , ZFLWP(KLON)&
 & , ZMU0(KLON)         , ZOMEGA(KLON,2,KLEV+1)&
 & , ZOZ(KLON,KLEV)&
 & , ZPMB(KLON,KLEV+1)  , ZPSOL(KLON)  
REAL(KIND=JPRB) :: ZTAU (KLON,2,KLEV) , ZTAVE (KLON,KLEV), ZTL(KLON,KLEV+1)&
 & , ZFCDWN(KLON,KLEV+1), ZFCUP(KLON,KLEV+1)&
 & , ZFSDWN(KLON,KLEV+1), ZFSUP(KLON,KLEV+1)&
 & , ZFSUPN(KLON)       , ZFSUPV(KLON)&
 & , ZFCUPN(KLON)       , ZFCUPV(KLON)&
 & , ZFSDNN(KLON)       , ZFSDNV(KLON)&
 & , ZFCDNN(KLON)       , ZFCDNV(KLON)&
 & , ZCOOLR(KLON,KLEV)  , ZCOOLC(KLON,KLEV)&
 & , ZHEATR(KLON,KLEV)  , ZHEATC(KLON,KLEV)&
 & , ZVIEW(KLON)  
REAL(KIND=JPRB) ::ZRADIP(KLON)       , ZRADLP(KLON)

REAL(KIND=JPRB) :: ZDZHT(KLON,KLEV)  , ZMAER(KLON,KLEV)  ,&
 & Z_CDNC(KLON,KLEV)   , ZCDNC(KLON,KLEV)  ,&
 & ZRAVRG(KLON,KLEV)  

INTEGER(KIND=JPIM) :: IKL, JK, JKL, JKLP1, JL

LOGICAL :: LLINTRP

REAL(KIND=JPRB) :: ZASYMX1, ZASYMX2, ZG1I, ZG1L, ZG2I, ZG2L,&
 & ZICEWP, ZIWC, ZIWGKG, ZLIQWP, ZLWC, ZLWGKG, &
 & ZMSAID, ZMSAIU, ZMSALD, ZMSALU, ZMULTI, ZMULTL, &
 & ZO1I, ZO1L, ZO2I, ZO2L, ZOMGMX1, ZOMGMX2, &
 & ZPMBM, ZRMUZ, ZTAUMX1, ZTAUMX2, ZTO1I, ZTO1L, &
 & ZTO2I, ZTO2L, ZALPHAER, ZCOEFRFH, ZTAUTAER 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "lw15.intfb.h"
#include "sw15.intfb.h"

!     -----------------------------------------------------------------

!*         1.     SET-UP INPUT QUANTITIES FOR RADIATION
!                 -------------------------------------

IF (LHOOK) CALL DR_HOOK('RADLSW15',0,ZHOOK_HANDLE)
DO JL = KIDIA,KFDIA
  ZFCUP(JL,KLEV+1) = 0.0_JPRB
  ZFCDWN(JL,KLEV+1) = REELOG15
  ZFSUP(JL,KLEV+1) = 0.0_JPRB
  ZFSDWN(JL,KLEV+1) = REELOG15
  ZFLUX(JL,1,KLEV+1) = 0.0_JPRB
  ZFLUX(JL,2,KLEV+1) = 0.0_JPRB
  ZFLUC(JL,1,KLEV+1) = 0.0_JPRB
  ZFLUC(JL,2,KLEV+1) = 0.0_JPRB
  ZFSDNN(JL) = 0.0_JPRB
  ZFSDNV(JL) = 0.0_JPRB
  ZFCDNN(JL) = 0.0_JPRB
  ZFCDNV(JL) = 0.0_JPRB
  ZFSUPN(JL) = 0.0_JPRB
  ZFSUPV(JL) = 0.0_JPRB
  ZFCUPN(JL) = 0.0_JPRB
  ZFCUPV(JL) = 0.0_JPRB
  ZPSOL(JL) = PAPH(JL,KLEV+1)
  ZPMB(JL,1) = ZPSOL(JL) / 100._JPRB
  ZDT0(JL) = PTS(JL) - PTH(JL,KLEV+1)
ENDDO

!*         1.1    INITIALIZE VARIOUS FIELDS
!                 -------------------------

DO JL = KIDIA,KFDIA
  ZALBD(JL,1)=PALBD(JL,1)
  ZALBD(JL,2)=PALBD(JL,2)
  ZALBP(JL,1)=PALBP(JL,1)
  ZALBP(JL,2)=PALBP(JL,2)
  ZEMIS(JL)  =PEMIS(JL)
  ZMU0(JL)   =PMU0(JL)
ENDDO

DO JK = 1 , KLEV
  JKL = KLEV+ 1 - JK
  JKLP1 = JKL + 1
  DO JL = KIDIA,KFDIA
    ZPMB(JL,JK+1)=PAPH(JL,JKL)/100._JPRB
    ZOZ(JL,JK)   = POZON(JL,JKL) * 46.6968_JPRB / RG
    ZFCUP(JL,JK) = 0.0_JPRB
    ZFCDWN(JL,JK) = 0.0_JPRB
    ZFSUP(JL,JK) = 0.0_JPRB
    ZFSDWN(JL,JK) = 0.0_JPRB
    ZFLUX(JL,1,JK) = 0.0_JPRB
    ZFLUX(JL,2,JK) = 0.0_JPRB
    ZFLUC(JL,1,JK) = 0.0_JPRB
    ZFLUC(JL,2,JK) = 0.0_JPRB
  ENDDO
ENDDO

!** INPUTS ARE FULL LEVEL TEMPERATURES + SURFACE TEMPERATURE
!        INTERPOLATION TO GET HALF-LEVEL TEMPERATURES FOLLOWS
!        WHAT IS DONE IN *RADINT* AND *RADHEAT*

!* LLINTRP=.T.  Half-level temperatures on the coarse grid are
!               vertically interpolated linearly with horizontal
!               sampled pressure from the full-level temperatures
!               of the sampled grid.

!* LLINTRP=.F.  Half-level temperatures are those horizontally
!               sampled on the coarse grid

LLINTRP=.FALSE.
IF (LLINTRP) THEN
  DO JK=2,KLEV
    DO JL=KIDIA,KFDIA
      PTH(JL,JK)=(PT  (JL,JK-1)*PAP  (JL,JK-1)&
       & *(PAP  (JL,JK)-PAPH  (JL,JK))&
       & +PT  (JL,JK)*PAP  (JL,JK)*(PAPH  (JL,JK)-PAP  (JL,JK-1)))&
       & *(1.0_JPRB/(PAPH  (JL,JK)*(PAP  (JL,JK)-PAP  (JL,JK-1))))  
    ENDDO
  ENDDO
  DO JL=KIDIA,KFDIA
    PTH(JL,1)= PT  (JL,1)-PAP  (JL,1)*(PT  (JL,1)-PTH(JL,2))&
     & /(PAP  (JL,1)-PAPH  (JL,2))  
    PTH(JL,KLEV+1)= PTS(JL)
  ENDDO
ENDIF
DO JK=1,KLEV
  JKL=KLEV+1-JK
  JKLP1=JKL+1
  DO JL=KIDIA,KFDIA
    ZTL(JL,JK)=PTH(JL,JKLP1)
    ZTAVE(JL,JK)=PT(JL,JKL)
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  ZTL(JL,KLEV+1)= PTH(JL,1)
  ZPMB(JL,KLEV+1) = PAPH(JL,1)/100._JPRB
ENDDO
!***

!     ------------------------------------------------------------------

!*         2.     CLOUD AND AEROSOL PARAMETERS
!                 ----------------------------

DO JK = 1 , KLEV
  IKL = KLEV + 1 - JK

! --- INITIALIZE OPTICAL PROPERTIES TO CLEAR SKY VALUES
  DO JL = KIDIA,KFDIA
    ZCLDSW(JL,JK)  = 0.0_JPRB
    ZTAU(JL,1,JK)  = 0.0_JPRB
    ZTAU(JL,2,JK)  = 0.0_JPRB
    ZOMEGA(JL,1,JK)= 1.0_JPRB
    ZOMEGA(JL,2,JK)= 1.0_JPRB
    ZCG(JL,1,JK)   = 0.0_JPRB
    ZCG(JL,2,JK)   = 0.0_JPRB
    ZCLDLD(JL,JK)  = 0.0_JPRB
    ZCLDLU(JL,JK)  = 0.0_JPRB
  ENDDO

  DO JL = KIDIA,KFDIA
    ZPMBM=0.5_JPRB*(ZPMB(JL,JK)+ZPMB(JL,JK+1))
    PCLFR(JL,IKL)=MAX(REPSC15,MIN(PCLFR(JL,IKL),1.0_JPRB-REPSC15))

! --- LIQUID WATER CONTENT (g.m-3) AND LIQUID WATER PATH (g.m-2)
    ZLWGKG=MAX(PQLWP(JL,IKL)*1000._JPRB,0.0_JPRB)
    ZIWGKG=MAX(PQIWP(JL,IKL)*1000._JPRB,0.0_JPRB)
    IF (PCLFR(JL,IKL) > (2.0_JPRB*REPCLC15)) THEN
      ZLWGKG=ZLWGKG/PCLFR(JL,IKL)
      ZIWGKG=ZIWGKG/PCLFR(JL,IKL)
    ELSE
      ZLWGKG=0.0_JPRB
      ZIWGKG=0.0_JPRB
    ENDIF
    ZFLWP(JL)= ZLWGKG*PDP(JL,IKL)/RG
    ZFIWP(JL)= ZIWGKG*PDP(JL,IKL)/RG
!   ZLWC=ZLWGKG*100._JPRB*ZPMBM/(RD*ZTAVE(JL,JK))
    ZIWC=ZIWGKG*100._JPRB*ZPMBM/(RD*ZTAVE(JL,JK))

! --- EFFECTIVE RADIUS FOR WATER AND ICE PARTICLES

    ZTAUTAER=0.0_JPRB  ! No Indirect effect for Sulfate aerosols

    IF (ZTAUTAER < 1.0E-20_JPRB) THEN

      ZRADLP(JL)=REFFWIA15 + (1000._JPRB-ZPMBM)*0.035_JPRB

    ELSE

!+    for indirect effect of aerosols, HRM, Nov.24, 1998.

      ZALPHAER=5.0_JPRB
      ZCOEFRFH=1.7_JPRB
      ZDZHT(JL,IKL)=RD*PT(JL,IKL)*PDP(JL,IKL)/(PAP(JL,IKL)*RG)
      ZMAER(JL,IKL)=1.0E6_JPRB*ZTAUTAER/(ZALPHAER*ZCOEFRFH*ZDZHT(JL,IKL))
      Z_CDNC(JL,IKL)=10.0_JPRB**(2.21_JPRB+0.41_JPRB*LOG10(ZMAER(JL,IKL)))
      ZCDNC(JL,IKL)=4.0_JPRB*3.14159_JPRB*Z_CDNC(JL,IKL)*1.0E6_JPRB
      ZRAVRG(JL,IKL)=3.0_JPRB*ZFLWP(JL)/ZDZHT(JL,IKL)/ZCDNC(JL,IKL)
      ZRADLP(JL)=5.0_JPRB+1.1_JPRB*ZRAVRG(JL,IKL)**(1.0_JPRB/3.0_JPRB)*1.0E4_JPRB
    ENDIF
! ===========================================================

! Modifications Martin et al.
!      IF (PLSM(JL).LT.0.5) THEN
!         ZASEA=150.
!         ZD=0.33
!         ZNTOT=-1.15E-03*ZASEA*ZASEA+0.963*ZASEA+5.30
!      ELSE
!         ZALND=900.
!         ZD=0.43
!         ZNTOT=-2.10E-04*ZALND*ZALND+0.568*ZALND-27.9
!      ENDIF
!      ZNUM=3.*ZLWC*(1.+3.*ZD*ZD)**2
!      ZDEN=4*RPI*ZNTOT*(1.+ZD*ZD)**3
!      ZRADLP(JL)=(ZNUM/ZDEN)**0.333

! ===========================================================
! ___________________________________________________________

! Modifications de la parametrisation des rayons effectifs :
! Dandin Ph., C. Pontikis, and E. Hicks, 1996: Sensitivity of a
! GCM to changes in the droplet effective radius parameterization.
! Subm. to Geoph. Res. Letters.
! d'apres :
! Pontikis C., 1995: Contribution to the droplet effective radius
! parameterization of warm convective clouds: Clear air entrainment
! effects. Geoph. Res. Letters, 22, 795-798.
!c il peut etre utile dans surdi.F de faire :
!                           s/REPCLC15=1.E-12/REPCLC15=1.E-3/
!     IF(PLSM(JL).LT.0.5) THEN
! On est sur Mer
!        ZERCST = 15.23
!     ELSE
! On est sur Terre
!        ZERCST = 8.93
!     ENDIF

!     IF(ZLWGKG.NE.0.) THEN
!        ZRADLP(JL) = ZERCST * (ZLWGKG ** (1./3.))
!     ELSE
!        ZRADLP(JL) = 0.
!     ENDIF
! ___________________________________________________________

    ZRADIP(JL)=REFFIA15 + REFFIB15 * ZIWC

  ENDDO

!   -------------------------
! --+ SW OPTICAL PARAMETERS +  Water clouds after Fouquart (1987)
!   -------------------------  Ice clouds (Ebert, Curry, 1992)

  DO JL = KIDIA,KFDIA
    IF (ZFLWP(JL)+ZFIWP(JL)  /= 0.0_JPRB) THEN
      ZICEWP = ZFIWP(JL)
      ZLIQWP = ZFLWP(JL)
      ZTO1L =ZLIQWP*(RYFWCA15(1)+RYFWCB15(1)/ZRADLP(JL))
      ZTO2L =ZLIQWP*(RYFWCA15(2)+RYFWCB15(2)/ZRADLP(JL))
      ZG1L = RYFWCF15(1)
      ZG2L = RYFWCF15(2)
      ZO1L = RYFWCC15(1)-RYFWCD15(1)*EXP(-RYFWCE15(1)*ZTO1L)
      ZO2L = RYFWCC15(2)-RYFWCD15(2)*EXP(-RYFWCE15(2)*ZTO1L)

      ZTO1I = ZICEWP*(REBCUA15(1)+REBCUB15(1)/ZRADIP(JL))
      ZTO2I = ZICEWP*(REBCUA15(2)+REBCUB15(2)/ZRADIP(JL))
      ZG1I  = REBCUE15(1)+REBCUF15(1)*ZRADIP(JL)
      ZG2I  = REBCUE15(2)+REBCUF15(2)*ZRADIP(JL)
      ZO1I  = REBCUC15(1)-REBCUD15(1)*ZRADIP(JL)
      ZO2I  = REBCUC15(2)-REBCUD15(2)*ZRADIP(JL)

!  - MIX of WATER and ICE CLOUDS
      ZTAUMX1= ZTO1L + ZTO1I
      ZTAUMX2= ZTO2L + ZTO2I
      ZOMGMX1= ZTO1L*ZO1L + ZTO1I*ZO1I
      ZOMGMX2= ZTO2L*ZO2L + ZTO2I*ZO2I
      ZASYMX1= ZTO1L*ZO1L*ZG1L + ZTO1I*ZO1I*ZG1I
      ZASYMX2= ZTO2L*ZO2L*ZG2L + ZTO2I*ZO2I*ZG2I

      ZASYMX1= ZASYMX1/ZOMGMX1
      ZASYMX2= ZASYMX2/ZOMGMX2
      ZOMGMX1= ZOMGMX1/ZTAUMX1
      ZOMGMX2= ZOMGMX2/ZTAUMX2

! --- SW FINAL CLOUD OPTICAL PARAMETERS

      ZCLDSW(JL,JK)  = PCLFR(JL,IKL)
      ZTAU(JL,1,JK)  = ZTAUMX1
      ZTAU(JL,2,JK)  = ZTAUMX2
      ZOMEGA(JL,1,JK)= ZOMGMX1
      ZOMEGA(JL,2,JK)= ZOMGMX2
      ZCG(JL,1,JK)   = ZASYMX1
      ZCG(JL,2,JK)   = ZASYMX2
    ENDIF
  ENDDO

!   -------------------------
! --+ LW OPTICAL PARAMETERS +  Water and Ice from Smith and Shi (1992)
!   -------------------------

  DO JL = KIDIA,KFDIA
    IF (ZFLWP(JL)+ZFIWP(JL) /= 0.0_JPRB) THEN
      ZMULTL=1.2_JPRB-0.006_JPRB*ZRADLP(JL)
      ZMULTI=1.2_JPRB-0.006_JPRB*ZRADIP(JL)
      ZMSALD= 0.158_JPRB*ZMULTL
      ZMSALU= 0.130_JPRB*ZMULTL
      ZMSAID= 0.113_JPRB*ZMULTI
      ZMSAIU= 0.093_JPRB*ZMULTI
      ZCLDLD(JL,JK) = PCLFR(JL,IKL)*(1.0_JPRB-EXP(-ZMSALD*ZFLWP(JL)&
       & -ZMSAID*ZFIWP(JL)))  
      ZCLDLU(JL,JK) = PCLFR(JL,IKL)*(1.0_JPRB-EXP(-ZMSALU*ZFLWP(JL)&
       & -ZMSAIU*ZFIWP(JL)))  
    ENDIF
  ENDDO

ENDDO

NUAER15 = NUA15
NTRAER15 = NTRA15

!     ------------------------------------------------------------------

!*         2.1    DIFFUSIVITY FACTOR OR SATELLITE VIEWING ANGLE
!                 ---------------------------------------------

DO JL = KIDIA,KFDIA
  ZVIEW(JL) = DIFF15
ENDDO

!     IF (IGEO.GT.1) THEN
!        CALL RADGEO ( KIDIA,KFDIA,KLON
!    S               , ZLAT, ZLON
!    S               , ZVIEW                     )
!     ENDIF

!     ------------------------------------------------------------------

!*         3.     CALL LONGWAVE RADIATION CODE
!                 ----------------------------

!*         3.1    FULL LONGWAVE RADIATION COMPUTATIONS
!                 ------------------------------------

CALL LW15 ( KIDIA, KFDIA , KLON , KTDIA, KLEV,&
 & KAER , KMODE, KFLUX , KRAD,&
 & PCCO2, ZCLDLD,ZCLDLU, PDP   , ZDT0 , ZEMIS ,ZPMB,ZPSOL,POZON,&
 & ZTL  , PAER , ZTAVE , ZVIEW , PQ,&
 & ZCOOLR,ZCOOLC,ZFLUX , ZFLUC )  

!     ------------------------------------------------------------------

!*         4.     CALL SHORTWAVE RADIATION CODE
!                 -----------------------------

ZRMUZ=0.0_JPRB
DO JL = KIDIA,KFDIA
  ZRMUZ = MAX (ZRMUZ, ZMU0(JL))
ENDDO

IF (ZRMUZ > 0.0_JPRB) THEN

  CALL SW15 ( KIDIA,KFDIA, KLON , KTDIA , KLEV, KAER,&
   & PRII0 , PCCO2 , ZPSOL , ZALBD , ZALBP , PQ   , PQS,&
   & ZMU0  , ZCG   , ZCLDSW, PDP   , ZOMEGA, ZOZ  , ZPMB,&
   & ZTAU  , ZTAVE , PAER,&
   & ZHEATR, ZFSDWN, ZFSUP,&
   & ZHEATC, ZFCDWN, ZFCUP,&
   & ZFSDNN, ZFSDNV, ZFSUPN, ZFSUPV,&
   & ZFCDNN, ZFCDNV, ZFCUPN, ZFCUPV     )  

ENDIF

!     ------------------------------------------------------------------

!*         5.     FILL UP THE MODEL NET LW AND SW RADIATIVE FLUXES
!                 ------------------------------------------------

DO JKL = 1 , KLEV+1
  JK = KLEV+1 + 1 - JKL
  DO JL = KIDIA,KFDIA
    PFLS(JL,JKL) = ZFSDWN(JL,JK) - ZFSUP(JL,JK)
    PFLT(JL,JKL) = - ZFLUX(JL,1,JK) - ZFLUX(JL,2,JK)
    PFCS(JL,JKL) = ZFCDWN(JL,JK) - ZFCUP(JL,JK)
    PFCT(JL,JKL) = - ZFLUC(JL,1,JK) - ZFLUC(JL,2,JK)
  ENDDO
ENDDO

DO JL = KIDIA,KFDIA
  PFRSOD(JL)=ZFSDWN(JL,1)
ENDDO

IF (KMODE == 1) THEN
  DO JKL = 1 , KLEV+1
    JK = KLEV+1 + 1 - JKL
    DO JL = KIDIA,KFDIA
    ENDDO
  ENDDO
ENDIF

!     --------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RADLSW15',1,ZHOOK_HANDLE)
END SUBROUTINE RADLSW15
