!OPTIONS XOPT(NOEVAL)
SUBROUTINE SW1S15 ( KIDIA, KFDIA, KLON, KLEV , KAER , KNU,&
 & PAER  , PALBD , PALBP, PCG  , PCLD , PCLEAR, PCLDSW,&
 & PDSIG , POMEGA, POZ  , PRMU , PSEC , PTAU  , PUD,&
 & PFD   , PFU   , PFDC , PFUC                            )  

!**** *SW1S15* - SHORTWAVE RADIATION, FIRST SPECTRAL INTERVAL
!****            FROZEN VERSION (CYCLE 15) OF S12S

!     PURPOSE.
!     --------

!          THIS ROUTINE COMPUTES THE SHORTWAVE RADIATION FLUXES IN TWO
!     SPECTRAL INTERVALS FOLLOWING FOUQUART AND BONNEL (1980).

!**   INTERFACE.
!     ----------

!          *SW1S15* IS CALLED FROM *SW15*.

!        IMPLICIT ARGUMENTS :
!        --------------------

!     ==== INPUTS ===
!     ==== OUTPUTS ===

!     METHOD.
!     -------

!          1. COMPUTES UPWARD AND DOWNWARD FLUXES CORRESPONDING TO
!     CONTINUUM SCATTERING
!          2. MULTIPLY BY OZONE TRANSMISSION FUNCTION

!     EXTERNALS.
!     ----------

!          *SWCLR15*, *SWR15*, *SWTT15*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE ECMWF RESEARCH DEPARTMENT
!        DOCUMENTATION, AND FOUQUART AND BONNEL (1980)

!     AUTHOR.
!     -------
!        96-11: Ph. Dandin. Meteo-France
!        ORIGINAL : 89-07-14 BY JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        A.Alias       19-09-2005 Sulfate aerosols (Hu Rong Ming)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMSW15  , ONLY : RRAY15   ,RSUN15

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KAER 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNU 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER(KLON,KLEV,6) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD(KLON,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP(KLON,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCG(KLON,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLEAR(KLON) 
REAL(KIND=JPRB)                  :: PCLDSW(KLON,KLEV) ! Argument NOT used
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDSIG(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POMEGA(KLON,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMU(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSEC(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAU(KLON,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUD(KLON,5,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFU(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFDC(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFUC(KLON,KLEV+1) 
!     ------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!     ------------------------------------------------------------------

!              ------------

INTEGER(KIND=JPIM) :: IIND(4)

REAL(KIND=JPRB) :: ZCGAZ(KLON,KLEV)&
 & ,  ZDIFF(KLON)        , ZDIRF(KLON)        &
 & ,  ZPIZAZ(KLON,KLEV)&
 & ,  ZRAYL(KLON), ZRAY1(KLON,KLEV+1), ZRAY2(KLON,KLEV+1)&
 & ,  ZREFZ(KLON,2,KLEV+1)&
 & ,  ZRJ(KLON,6,KLEV+1), ZRJ0(KLON,6,KLEV+1)&
 & ,  ZRK(KLON,6,KLEV+1), ZRK0(KLON,6,KLEV+1)&
 & ,  ZRMUE(KLON,KLEV+1), ZRMU0(KLON,KLEV+1)&
 & ,  ZR(KLON,4)&
 & ,  ZTAUAZ(KLON,KLEV)&
 & ,  ZTRA1(KLON,KLEV+1), ZTRA2(KLON,KLEV+1)&
 & ,  ZW(KLON,4)  

INTEGER(KIND=JPIM) :: IKL, IKM1, JAJ, JK, JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "swclr15.intfb.h"
#include "swr15.intfb.h"
#include "swtt115.intfb.h"

!     ------------------------------------------------------------------

!*         1.     FIRST SPECTRAL INTERVAL (0.25-0.68 MICRON)
!                 ----------------------- ------------------

!*         1.1    OPTICAL THICKNESS FOR RAYLEIGH SCATTERING
!                 -----------------------------------------

IF (LHOOK) CALL DR_HOOK('SW1S15',0,ZHOOK_HANDLE)
DO JL = KIDIA,KFDIA
  ZRAYL(JL) =  RRAY15(KNU,1) + PRMU(JL) * (RRAY15(KNU,2) +&
   & PRMU(JL)&
   & * (RRAY15(KNU,3) + PRMU(JL) * (RRAY15(KNU,4) + PRMU(JL)&
   & * (RRAY15(KNU,5) + PRMU(JL) *  RRAY15(KNU,6)       ))))  
ENDDO

!     ------------------------------------------------------------------

!*         2.    CONTINUUM SCATTERING CALCULATIONS
!                ---------------------------------

!*         2.1   CLEAR-SKY FRACTION OF THE COLUMN
!                --------------------------------

CALL SWCLR15 ( KIDIA , KFDIA , KLON, KLEV, KAER , KNU,&
 & PAER   , PALBP , PDSIG , ZRAYL, PSEC,&
 & ZCGAZ  , ZPIZAZ, ZRAY1 , ZRAY2, ZREFZ, ZRJ0,&
 & ZRK0   , ZRMU0 , ZTAUAZ, ZTRA1, ZTRA2             )  

!*         2.2   CLOUDY FRACTION OF THE COLUMN
!                -----------------------------

CALL SWR15 ( KIDIA ,KFDIA ,KLON ,KLEV ,KAER ,KNU,&
 & PALBD ,PCG   ,PCLD  ,PDSIG ,POMEGA,PSEC ,PTAU,&
 & ZCGAZ ,ZPIZAZ,ZRAY1 ,ZRAY2 ,ZREFZ ,ZRJ  ,ZRK,ZRMUE,&
 & ZTAUAZ,ZTRA1 ,ZTRA2                                  )  

!     ------------------------------------------------------------------

!*         3.    OZONE ABSORPTION
!                ----------------

IIND(1)=1
IIND(2)=3
IIND(3)=1
IIND(4)=3

!*         3.1   DOWNWARD FLUXES
!                ---------------

JAJ = 2

DO JL = KIDIA,KFDIA
  ZW(JL,1)=0.0_JPRB
  ZW(JL,2)=0.0_JPRB
  ZW(JL,3)=0.0_JPRB
  ZW(JL,4)=0.0_JPRB
  PFD(JL,KLEV+1)=((1.0_JPRB-PCLEAR(JL))*ZRJ(JL,JAJ,KLEV+1)&
   & + PCLEAR(JL) *ZRJ0(JL,JAJ,KLEV+1)) * RSUN15(KNU)  
  PFDC(JL,KLEV+1) = ZRJ0(JL,JAJ,KLEV+1) * RSUN15(KNU)
ENDDO
DO JK = 1 , KLEV
  IKL = KLEV+1-JK
  DO JL = KIDIA,KFDIA
    ZW(JL,1)=ZW(JL,1)+PUD(JL,1,IKL)/ZRMUE(JL,IKL)
    ZW(JL,2)=ZW(JL,2)+POZ(JL,  IKL)/ZRMUE(JL,IKL)
    ZW(JL,3)=ZW(JL,3)+PUD(JL,1,IKL)/ZRMU0(JL,IKL)
    ZW(JL,4)=ZW(JL,4)+POZ(JL,  IKL)/ZRMU0(JL,IKL)
  ENDDO

  CALL SWTT115 ( KIDIA, KFDIA, KLON, KNU, 4,&
   & IIND,&
   & ZW,&
   & ZR                          )  

  DO JL = KIDIA,KFDIA
    ZDIFF(JL) = ZR(JL,1)*ZR(JL,2)*ZRJ(JL,JAJ,IKL)
    ZDIRF(JL) = ZR(JL,3)*ZR(JL,4)*ZRJ0(JL,JAJ,IKL)
    PFD(JL,IKL) = ((1.0_JPRB-PCLEAR(JL)) * ZDIFF(JL)&
     & +PCLEAR(JL)  * ZDIRF(JL)) * RSUN15(KNU)  
    PFDC(JL,IKL) = ZDIRF(JL) * RSUN15(KNU)
  ENDDO
ENDDO

!*         3.2   UPWARD FLUXES
!                -------------

DO JL = KIDIA,KFDIA
  PFU(JL,1) = ((1.0_JPRB-PCLEAR(JL))*ZDIFF(JL)*PALBD(JL,KNU)&
   & + PCLEAR(JL) *ZDIRF(JL)*PALBP(JL,KNU))&
   & * RSUN15(KNU)  
  PFUC(JL,1) = ZDIRF(JL) * PALBP(JL,KNU) * RSUN15(KNU)
ENDDO

DO JK = 2 , KLEV+1
  IKM1=JK-1
  DO JL = KIDIA,KFDIA
    ZW(JL,1)=ZW(JL,1)+PUD(JL,1,IKM1)*1.66_JPRB
    ZW(JL,2)=ZW(JL,2)+POZ(JL,  IKM1)*1.66_JPRB
    ZW(JL,3)=ZW(JL,3)+PUD(JL,1,IKM1)*1.66_JPRB
    ZW(JL,4)=ZW(JL,4)+POZ(JL,  IKM1)*1.66_JPRB
  ENDDO

  CALL SWTT115 ( KIDIA, KFDIA, KLON, KNU, 4,&
   & IIND,&
   & ZW,&
   & ZR                          )  

  DO JL = KIDIA,KFDIA
    ZDIFF(JL) = ZR(JL,1)*ZR(JL,2)*ZRK(JL,JAJ,JK)
    ZDIRF(JL) = ZR(JL,3)*ZR(JL,4)*ZRK0(JL,JAJ,JK)
    PFU(JL,JK) = ((1.0_JPRB-PCLEAR(JL)) * ZDIFF(JL)&
     & +PCLEAR(JL)  * ZDIRF(JL)) * RSUN15(KNU)  
    PFUC(JL,JK) = ZDIRF(JL) * RSUN15(KNU)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SW1S15',1,ZHOOK_HANDLE)
END SUBROUTINE SW1S15
