!OPTIONS XOPT(HSFUN)
SUBROUTINE VDFHGHTNHL (KIDIA   , KFDIA   , KLON    , KLEV   , KDRAFT, PTMST, KSTEP, &
                   & PUM1    , PVM1    , PTM1    , PQM1    , PLM1    , PIM1   , PAM1,&
                   & PAPHM1  , PAPM1   , PGEOM1  , PGEOH  , &
                   & PKMFL   , PKHFL   , PKQFL   , PMFLX  , &
! DIAGNOSTIC OUTPUT
                   & PEXTR2  , KFLDX2  , PEXTRA  , KLEVX  , KFLDX , &
!                   
                   & PUUH    , PVUH    , PSLGUH  , PQTUH  , PTHTVUH, PFRACB, &
                   & PZPTOP  , KPTOP   , PZPLCL  , KPLCL  , KPLZB   , &
                   & PWUAVG  , PRICUI  , PMCU    , &
                   & PFPLVL  , PFPLVN  , &
                   & PBIR    , LDNODECP, LDRUNDRY, KPBLTYPE, PWQT, PWQT2,&
                   & KGFL_EZDIAG, PEZDIAG )
                   
                     
!     ------------------------------------------------------------------

!**   * VDFHGHTNHL* - DETERMINES THE PBL-HEIGHT AND STRONG UPDRAFT FIELDS
!                  USING A ENTRAINING PARCEL ASCENT METHOD.

!     A.P. SIEBESMA                30/06/1999  Original (dry)
!     M. Ko"hler                    3/12/2004  Moist Version
!     Roel Neggers                 12/04/2005  Multiple updraft extension
!     Wim de Rooy/Geert Lenderink  13/06/2008  Updates to combine TKE turbulence
!                                              with dual_M EDMF. Detrainment according
!                                              to de Rooy & Siebesma MWR 2008
!                                              Also, include statistical cloud scheme
!                                              (Lenderink & Siebesma 2000) just for the
!                                              Mb closure of Roel Neggers et al. (MWR 2004)




!     PURPOSE
!     -------

!     DETERMINE PBL HEIGHT AND UPDRAFT FIELDS

!     INTERFACE
!     ---------

!     * VDFHGHTNHL* IS CALLED BY *VDFHGHTHL*

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!     *KIDIA*        START POINT
!     *KFDIA*        END POINT
!     *KLEV*         NUMBER OF LEVELS
!     *KLON*         NUMBER OF GRID POINTS PER PACKET
!     *KDRAFT*       NUMBER OF EXPLICITLY MODELED DRAFTS - CURRENTLY 3:
!                    1: test parcel
!                    2: rising dry thermals which stop at cloud base or inversion
!                    3: rising dry thermals which become cloudy
!                    (4: downdrafts .. to be done?)

!     INPUT PARAMETERS (REAL):

!     *PTMST*        DOUBLE TIME STEP (SINGLE AT 1TH STEP)        S
!     *PUM1*         X-VELOCITY COMPONENT AT T-1                  M/S
!     *PVM1*         Y-VELOCITY COMPONENT AT T-1                  M/S
!     *PTM1*         TEMPERATURE AT T-1                           K
!     *PQM1*         SPECIFIC HUMUDITY AT T-1                     KG/KG
!     *PLM1*         SPECIFIC CLOUD LIQUID WATER AT T-1           KG/KG
!     *PIM1*         SPECIFIC CLOUD ICE AT T-1                    KG/KG
!     *PAM1*         CLOUD FRACTION AT T-1                        KG/KG
!     *PAPHM1*       PRESSURE AT HALF LEVEL AT T-1                PA
!     *PAPM1*        PRESSURE AT FULL LEVEL AT T-1                PA
!     *PGEOM1*       GEOPOTENTIAL AT T-1                          M2/S2
!     *PGEOH*        GEOPOTENTIAL AT HALF LEVEL                   M2/S2
!     *PKMFL*        SURFACE KINEMATIC MOMENTUM FLUX              M2/S2  
!     *PKHFL*        SURFACE KINEMATIC HEAT FLUX                  K*M/S
!     *PKQFL*        SURFACE KINEMATIC MOISTURE FLUX              M/S
!     *PBIR*         BUOYANCY-FLUX INTEGRAL RATIO (-N/P)
!                    USED FOR DECOUPLING CRITERIA

!     INPUT PARAMETERS (LOGICAL):

!     *LDNODECP*     TRUE:  NEVER DECOUPLE
!                    FALSE: MAYBE DECOUPLE
!     *LDRUNDRY*     TRUE:  RUN PARCEL WITHOUT CONDENSATION
!                    FALSE: RUN PARCEL WITH CONDENSATION

!     OUTPUT PARAMETERS (REAL):

!     *PFPLVL*       PBL PRECIPITATION FLUX AS RAIN                KG/(M**2*S)
!     *PFPLVN*       PBL PRECIPITATION FLUX AS SNOW                KG/(M**2*S)

!     *PUUH*         UPDRAFT X-MOMENTUM
!     *PVUH*         UPDRAFT Y-MOMENTUM
!     *PSLGUH*       UPDRAFT GENERALIZED LIQUID STATIC ENERGY (SLG)
!                    AT HALF LEVEL                                M2/S2
!     *PQTUH*        UPDRAFT SPECIFIC TOTAL WATER AT HALF LEVEL   KG/KG
!     *PTHTVUH*      UPDRAFT virt potential temp at HALF LEVEL (for TKE)  K
!     *PMFLX*        PBL MASS FLUX                                M/S
!     *PZPLCL*       HEIGHT OF LIFTING CONDENSATION LEVEL OF UPDRAFT          M
!     *PZPTOP*       HEIGHT OF LEVEL OF ZERO KINETIC ENERGY (W=0) OF UPDRAFT  M

!     OUTPUT PARAMETERS (INTEGER):

!     *KPLCL*         FIRST HALF LEVEL ABOVE REAL HEIGHT OF UPRAFT LCL
!     *KPTOP*         HIGHEST HALF LEVEL BELOW PZTOP, AND
!                       UPDRAFT TOP FULL LEVEL (PZTOP IS WITHIN THAT LAYER)
!     *KPLZB*         LEVEL OF UPRAFT ZERO BUOYANCY (LAST FULL LEVEL THAT IS POS. BUOYANT)
!     *KPBLTYPE*    -1: not defined yet
!                    0: stable PBL
!                    1: dry convective PBL (no cloud below parcel top)
!                    2: stratocumulus
!                    3: shallow cumulus
!                    4: deep cumulus

!     METHOD
!     ------

!     SEE DOCUMENTATION

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RD       ,RCPD     ,RETV     ,RLVTT    ,&
                    & RLSTT    ,RATM     ,RTT      ,RLMLT

USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
                    & R4IES    ,R5LES    ,R5IES    ,RVTMP2   ,R5ALVCP  ,&
                    & R5ALSCP  ,RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE    ,&
                    & RTICECU  ,RTWAT_RTICE_R      ,RTWAT_RTICECU_R


USE YOEVDF   , ONLY : RKAP     ,RVDIFTS
USE YOECUMF  , ONLY : RTAUMEL

USE YDUALM_TKE , ONLY :  RCCSTAT

IMPLICIT NONE


!*         0.1    GLOBAL VARIABLES

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDRAFT
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
INTEGER(KIND=JPIM),INTENT(INOUT) :: KPLCL(KLON,KDRAFT) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KPTOP(KLON,KDRAFT) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KPLZB(KLON,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTMST 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUM1(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVM1(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHM1(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKMFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKHFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKQFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFLX(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUUH(KLON,0:KLEV,KDRAFT)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVUH(KLON,0:KLEV,KDRAFT)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSLGUH(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQTUH(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTHTVUH(KLON,0:KLEV,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRACB(KLON,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZPLCL(KLON,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZPTOP(KLON,KDRAFT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWUAVG(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLVL(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFPLVN(KLON,0:KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBIR(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRICUI(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMCU(KLON) 
LOGICAL           ,INTENT(IN)    :: LDNODECP(KLON) 
!ldrundry not used now
LOGICAL           ,INTENT(IN)    :: LDRUNDRY(KLON) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KPBLTYPE(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWQT(KLON,0:KLEV),PWQT2(KLON,0:KLEV)

!diagnostic output
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX2, KLEVX, KFLDX
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEXTR2(KLON,KFLDX2), PEXTRA(KLON,KLEVX,KFLDX)

INTEGER(KIND=JPIM),INTENT(IN)    :: KGFL_EZDIAG
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEZDIAG(KLON,KLEV,KGFL_EZDIAG) 


!*         0.2    LOCAL VARIABLES

!--- mean & environmental properties ---
REAL(KIND=JPRB) ::    ZUSTAR (KLON)      , ZWSTAR(KLON)       , ZKHVFL(KLON)       , &
                    & ZWSIGMA(KLON)      , &
                    & ZSLGENH(KLON,0:KLEV),ZQLENH(KLON,0:KLEV), ZQIENH(KLON,0:KLEV), &
                    & ZQTENH(KLON,0:KLEV), ZUENH(KLON,0:KLEV) , ZVENH(KLON,0:KLEV) , &
                    & ZTVEN(KLON,KLEV),    ZQTM1 (KLON,KLEV)  , &
                    & ZSLGM1(KLON,KLEV)  , ZMGEOM(KLON,0:KLEV), &
                    & ZTENH(KLON,0:KLEV) , ZRHOH (KLON,0:KLEV), ZTHVEN(KLON,KLEV)
    
REAL(KIND=JPRB) ::    ZSTAR(KLON), ZCHICRIT(KLON,0:KLEV),ZMEANCHICRIT(KLON)   , &
                    & ZDETRSHALLOW(KLON), VARQ(KLON,0:KLEV), Q1(KLON,0:KLEV)  , &
                    & ZFRACMB(KLON) , ZMINIMUM(KLON) , ZTVUH(KLON,0:KLEV)     , &
                    & ZTVENH(KLON,0:KLEV), ZEXNERH(KLON,0:KLEV), ZQVENH(KLON,0:KLEV), &
                    & ZTHTVENH(KLON,0:KLEV), ZTHTLUH(KLON,0:KLEV), &
                    & ZTHTENH(KLON,0:KLEV), &
                    & ZTHTVUH(KLON,0:KLEV), ZTHTLEH(KLON,0:KLEV), ZESATU(KLON,0:KLEV),&
                    & ZDQSDTU(KLON,0:KLEV),ZGAMMA(KLON,0:KLEV),ZDUMFUNC(KLON,0:KLEV), &
                    & ZPRES_0, ZKAPPA,ZQSATU(KLON,0:KLEV), DQDZ(KLON,0:KLEV), RHOUP(KLON,0:KLEV)

REAL(KIND=JPRB) ::    WSTCLD , DZFULL

!--- updraft parameters ---
REAL(KIND=JPRB) ::    ZWU2H (KLON,0:KLEV,KDRAFT), ZWUH, &
                    & ZQCUH (KLON,0:KLEV,KDRAFT), ZQUH  (KLON,0:KLEV,KDRAFT), &
    & ZTUH  (KLON,0:KLEV,KDRAFT), ZEPS  (KLON,0:KLEV,KDRAFT), &
    & ZDETR (KLON,0:KLEV,KDRAFT), ZFRAC (KLON,0:KLEV,KDRAFT), &
    & ZBUOF (KLON,KLEV,KDRAFT)  , ZMCLD (KLON)              , &
                    & ZBUOH (KLON,0:KLEV,KDRAFT), &
                    & ZDELTAMINEPS(KLON,0:KLEV), ZQSATH(KLON,0:KLEV), &
                    & ZQSATH2(KLON,0:KLEV), ZESATH2(KLON,0:KLEV),&
                    & ZMSTAR, ZMSTARLCL, ZMSTARTOP(KLON), ZMSTARTOPMAX, &
                    & ZRICUCRIT, ZZPLZB(KLON,KDRAFT), ZCAPE1(KLON)

REAL(KIND=JPRB) ::    ZQSATM, ZSATDEF, &
                    & ZUPFLXL(KLON,0:KLEV,KDRAFT), ZUPFLXN(KLON,0:KLEV,KDRAFT), &
                    & ZUPGENL(KLON,KLEV,KDRAFT), ZUPGENN(KLON,KLEV,KDRAFT), &
                    & ZDZRHO, ZPFLXTOT, ZPEVAPUP, ZFAC, ZUPMELT

REAL(KIND=JPRB) ::    ZFRACB(KLON,KDRAFT), ZMFLXB(KLON,KDRAFT)
      
REAL(KIND=JPRB) ::    ZFRACMAX , ZFACMAXEXC , ZFRACTEST , ZFACTESTEXC , &
                    & ZFACEXC(KLON,KDRAFT), ZDUMFRAC, ZDUMR, ZMASSCAPDEPTH

LOGICAL ::            LLDONE(KLON,KDRAFT)


INTEGER(KIND=JPIM) :: IS, JK, JL, JD, IBASE, ITJE

INTEGER(KIND=JPIM) :: KSTAR(KLON), FIRSTTIME

REAL(KIND=JPRB) ::    ZQEXC   , ZTEXC   , ZDZ     , &
                    & ZCONS10 , ZCFNC1(KLON,0:KLEV)      , ZTVMEAN     , &
                    & ZRG     , ZMFMAX  , ZMFS(KLON,KDRAFT)

!          REMAINING MODEL PARAMETERS

REAL(KIND=JPRB) ::    ZTAUEPS(KLON) , ZCLDDEPTH  , &
                    & ZW2THRESH         , ZSTABTHRESH       , ZBIRTHRESH , &
                    & ZCLDDEPTHDP       , ZDZCLOUD(KLON), &
                    & ZREPUST, ZGHM1
    
REAL(KIND=JPRB) ::    ZZI(KLON)

!cstep 30082007: introduce two parcel time scales, one for the test parcel (ZTAUEPS_TEST) and one
!              : for the actual parcels (ZTAUEPS)
REAL(KIND=JPRB) :: ZTAUEPS_TEST

INTEGER(KIND=JPIM) :: IZI(KLON,KDRAFT)

REAL(KIND=JPRB) ::    ZGRAD

REAL(KIND=JPRB) ::    ZHOOK_HANDLE
LOGICAL :: LLDIAG
!CGL variables to compute mass flux profile
REAL(KIND=JPRB) ::    ZMASSFLUXB, ZMASSFLUXH, ZLCL, ZTOP, ZZSCALED, ZALPHA, ZBETA
REAL(KIND=JPRB) ::    ZZSCALEDH

INTEGER(KIND=JPIM) :: IOPT

INTERFACE
#include "surf_inq.h"
END INTERFACE

#include "vdfparcelhl.intfb.h"
#include "vdfstcucrithl.intfb.h"
#include "vdfpdftablehl.intfb.h"
#include "fcttre.h"



!     ------------------------------------------------------------------

!*         1.     INITIALIZATION
!                 --------------

IF (LHOOK) CALL DR_HOOK('VDFHGHTNHL',0,ZHOOK_HANDLE)

LLDIAG = .FALSE.

ZPRES_0 = 100000._JPRB     ! standard pressure consistent with vdfexcu
ZKAPPA = RD / RCPD         ! consistent with vdfexcu

ZFRACTEST   = 0.002_JPRB   ! top % of the PDF associated with the test parcel
CALL VDFPDFTABLEHL (ZFRACTEST, ZFACTESTEXC, ZDUMR, ZDUMR, 0) ! associated PDF scaling factor


ZFRACMAX    = 0.1_JPRB     ! total convective area fraction that is done with mass flux

CALL VDFPDFTABLEHL (ZFRACMAX, ZFACMAXEXC, ZDUMR, ZDUMR, 0) ! associated PDF scaling factor

! eddy turnover time scale used in parcel entrainment [s]  (Neggers, Siebesma & Jonker, JAS 2002)
ZTAUEPS_TEST = 400._JPRB   ! Roel's original ZTAUEPS value


!ZW2THRESH  = -1._JPRB     ! threshold parcel vertical velocity squared [m2/s2]
!CGL
ZW2THRESH   = 0.0_JPRB      

ZCLDDEPTH   = 2000._JPRB   ! threshold cloud thickness for stcu/cu transition [m]

ZCLDDEPTHDP = 100000._JPRB ! threshold cloud thickness used in shallow/deep decision [m]  

ZSTABTHRESH = 20._JPRB     ! threshold stability (Klein & Hartmann criteria) [K]
ZBIRTHRESH  = 0.1_JPRB     ! threshold BIR (TKE decoupling criteria) [1]

CALL SURF_INQ(PREPUST=ZREPUST)
                 
! switch for moist mass flux depth limiter - *experimental*            
ZMASSCAPDEPTH = 4000._JPRB 

! switch for flexible vertical structure of cloudy mass flux - *experimental*
ZRICUCRIT    = 5.0_JPRB  !critical Richardson number of Cu inversion 

! optimization
ZRG         = 1.0_JPRB/RG

!  ccstat from t-1 is needed for (mini)statistical cloud scheme
!  This cc is used for the Mb closure (Roel Neggers et al. MWR 2004)

IF (.NOT. ALLOCATED(RCCSTAT))  ALLOCATE(RCCSTAT(KLON,0:KLEV))
! set some stuff to zero
IF (KSTEP .EQ. 0) THEN
  ! to start shallow convection (exact value is unimportant)
  RCCSTAT(:,:)=0.01_JPRB
ENDIF

DO JL=KIDIA,KFDIA
  PWUAVG(JL)       = 0.0_JPRB
  KPBLTYPE(JL)     = -1           ! -1 means: yet unknown
  
  ZZI(JL)          = 0._JPRB      ! mixed layer scalings
  ZWSTAR(JL)       = 0._JPRB        
  
  PRICUI(JL)       = 1._JPRB      ! 1 / cumulus inversion Richardson number 
  ZCAPE1(JL)       = 0._JPRB
  ZMCLD(JL)        = 0._JPRB
  PMCU(JL)         = 0._JPRB      ! cloud-depth average moist updraft mass flux
  
  PEXTR2(JL,24:49) = 0._JPRB
ENDDO
  
DO JD=1,KDRAFT
  DO JL=KIDIA,KFDIA
    PZPLCL(JL,JD)  = -100._JPRB  ! default value: -100 (no LCL)
    PZPTOP(JL,JD)  = 0._JPRB     
    KPLCL(JL,JD)   = 0           ! default value: 0 (no PBL cloud)
    KPTOP(JL,JD)   = 0          
    KPLZB(JL,JD)   = 0          
    LLDONE(JL,JD)  = .TRUE.      ! default: TRUE (don't launch the parcel)
    ZFRACB(JL,JD)  = 0._JPRB 
    PFRACB(JL,JD)  = 0._JPRB 
    ZFACEXC(JL,JD) = 0._JPRB 
    ZFACEXC(JL,JD) = 0._JPRB 
    ZMFLXB(JL,JD)  = 0._JPRB 
    IZI(JL,JD)     = 0
  ENDDO
ENDDO

DO JK=0,KLEV
  DO JL=KIDIA,KFDIA
    PWQT(JL,JK)  = 0._JPRB  
    PWQT2(JL,JK) = 0._JPRB  
    ZCHICRIT(JL,JK) = 0._JPRB  
  ENDDO
ENDDO

!--- parcel half level parameters ---
DO JD=1,KDRAFT
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      PUUH(JL,JK,JD)    = 0.0_JPRB
      PVUH(JL,JK,JD)    = 0.0_JPRB
      PSLGUH(JL,JK,JD)  = 0.0_JPRB
      PQTUH(JL,JK,JD)   = 0.0_JPRB
      PTHTVUH(JL,JK,JD) = 0.0_JPRB
      PMFLX(JL,JK,JD)   = 0.0_JPRB 
      ZTUH(JL,JK,JD)    = 0.0_JPRB
      ZQUH(JL,JK,JD)    = 0.0_JPRB
      ZQCUH(JL,JK,JD)   = 0.0_JPRB
      ZEPS(JL,JK,JD)    = 0.0_JPRB
      ZDETR(JL,JK,JD)   = 0.0_JPRB
      ZWU2H(JL,JK,JD)   = 0.0_JPRB
      ZFRAC(JL,JK,JD)   = 0.0_JPRB
      ZUPFLXL(JL,JK,JD) = 0.0_JPRB
      ZUPFLXN(JL,JK,JD) = 0.0_JPRB
      VARQ(JL,JK)       = 0.0_JPRB
    ENDDO
  ENDDO
ENDDO

!--- parcel full level parameters --- 
DO JD=1,KDRAFT
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZBUOF(JL,JK,JD)    = 0.0_JPRB
      ZUPGENL(JL,JK,JD)  = 0.0_JPRB
      ZUPGENN(JL,JK,JD)  = 0.0_JPRB
    ENDDO
  ENDDO
ENDDO

!--- reset output stuff ---
IF (LLDIAG)  THEN
  DO JK=1,KLEVX
    DO JL=KIDIA,KFDIA
      PEXTRA(JL,JK,49) = 0._JPRB
      PEXTRA(JL,JK,50) = 0._JPRB
    ENDDO
  ENDDO
ENDIF


!     -----------------------------------------------------------------

!*         2.     PREPARE FIELDS ON HALF LEVELS BY LINEAR INTERPOLATION
!*                OF CONSERVED VARIABLES
!                 -----------------------------------------------------

!*         2.1  full level cpm, slg, qt and Tv
!*

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZSLGM1(JL,JK) = RCPD * PTM1(JL,JK) + PGEOM1(JL,JK) &
                  & - RLVTT * PLM1(JL,JK) - RLSTT * PIM1(JL,JK)  
      ZQTM1 (JL,JK) = PQM1(JL,JK) + PLM1(JL,JK) + PIM1(JL,JK)

!          parcel goes through cloud portion of environment
!          (added ql loading; ql,cld=ql,mean/fc; qv = qsat) 
!          safety: fc>0.1; linear interpolation between overcast 
!                  and cloudy portion for 0<fc<0.1
!                  guaranteed to be < tv from mean conditions

!          grid box mean virtual effect
      ZTVMEAN       = PTM1(JL,JK) * ( 1.0_JPRB + RETV * PQM1(JL,JK) &
                  & - PLM1(JL,JK) - PIM1(JL,JK) )       !qli loading  
      ZTVEN(JL,JK) = ZTVMEAN
      ZTHVEN(JL,JK) = ( PAPM1(JL,JK)/RATM )**(-RD/RCPD) * ZTVEN(JL,JK)
      
    ENDDO
  ENDDO


!*         2.2  half-level environment interpolation (qt, ql, qi, slg)
!*              attention:  not good to interpolate everything independently
!*              better:     interpolate conserved variables and derive rest!!!
!*

  DO JK=1,KLEV-1
  DO JL=KIDIA,KFDIA

    IF (JK==1) THEN
      ZGHM1 = PGEOH(JL,JK) + 50000._JPRB*RG   !avoid using top half level (=inf)
    ELSE
      ZGHM1 = PGEOH(JL,JK-1)
    ENDIF  
    
    ZQTENH(JL,JK) = ( ZQTM1(JL,JK+1) *(ZGHM1-PGEOH(JL,JK  )) &
                & +   ZQTM1(JL,JK)   *(PGEOH(JL,JK  )-PGEOH(JL,JK+1)) &
                &   )                /(ZGHM1-PGEOH(JL,JK+1))
    ZQLENH(JL,JK) = ( PLM1(JL,JK+1)  *(ZGHM1-PGEOH(JL,JK  )) &
                & +   PLM1(JL,JK)    *(PGEOH(JL,JK  )-PGEOH(JL,JK+1)) &
                &   )                /(ZGHM1-PGEOH(JL,JK+1))
    ZQIENH(JL,JK) = ( PIM1(JL,JK+1)  *(ZGHM1-PGEOH(JL,JK  )) &
                & +   PIM1(JL,JK)    *(PGEOH(JL,JK  )-PGEOH(JL,JK+1)) &
                &   )                /(ZGHM1-PGEOH(JL,JK+1))

    ZQVENH(JL,JK) = ZQTENH(JL,JK) - ZQLENH(JL,JK) - ZQIENH(JL,JK)

    ZSLGENH(JL,JK)= ( ZSLGM1(JL,JK+1)*(ZGHM1-PGEOH(JL,JK  )) &
                & +   ZSLGM1(JL,JK)  *(PGEOH(JL,JK  )-PGEOH(JL,JK+1)) &
                &   )                /(ZGHM1-PGEOH(JL,JK+1))
    ZUENH(JL,JK)  = ( PUM1(JL,JK+1)  *(ZGHM1-PGEOH(JL,JK  )) &
                & +   PUM1(JL,JK)    *(PGEOH(JL,JK  )-PGEOH(JL,JK+1)) &
                &   )                /(ZGHM1-PGEOH(JL,JK+1))
    ZVENH(JL,JK)  = ( PVM1(JL,JK+1)  *(ZGHM1-PGEOH(JL,JK  )) &
                & +   PVM1(JL,JK)    *(PGEOH(JL,JK  )-PGEOH(JL,JK+1)) &
                &   )                /(ZGHM1-PGEOH(JL,JK+1))


    !  Calculate T at half levels from sl, for later use in density calculations
    ZTENH(JL,JK)  =  ( PTM1(JL,JK+1)  *(ZGHM1-PGEOH(JL,JK  )) &
                 & +   PTM1(JL,JK)    *(PGEOH(JL,JK  )-PGEOH(JL,JK+1)) &
                 &   )                /(ZGHM1-PGEOH(JL,JK+1))
    !  Determine thetav environment
    ZEXNERH(JL,JK) = (PAPHM1(JL,JK)/ZPRES_0)**ZKAPPA
    ZTHTENH(JL,JK) = (ZTENH(JL,JK)/ZEXNERH(JL,JK)) 
    ZTHTVENH(JL,JK) = (ZTENH(JL,JK)/ZEXNERH(JL,JK)) &
              * (1._JPRB + RETV * ZQVENH(JL,JK) & 
    ! add qice correction
               - ZQLENH(JL,JK) ) 

    ! initialize updraft thetav for dry and moist updraft with environment values
    PTHTVUH(JL,JK,2)=ZTHTVENH(JL,JK)
    PTHTVUH(JL,JK,3)=ZTHTVENH(JL,JK)
    PQTUH(JL,JK,2)=ZQTENH(JL,JK)
    PQTUH(JL,JK,3)=ZQTENH(JL,JK)
               
    ZRHOH(JL,JK) = PAPHM1(JL,JK)/(RD*ZTENH(JL,JK))
    
  ENDDO
  ENDDO


!     -----------------------------------------------------------------
!*         3.     RELEASE THE FIRST (TEST) UPDRAFT TO GET PBL HEIGHTS
  
  
  !* set updraft index to 1
  JD = 1   

  DO JL=KIDIA,KFDIA
     
    ZFRACB(JL,JD) = ZFRACMAX  !CGL replaced ZFRACB(JL,JD) = ZFRACTEST
    
    !* 3.1    Determine stability of BL using the surface buoyancy flux
 
    ZKHVFL(JL)  = ( 1.0_JPRB + RETV *  ZQTM1(JL,KLEV) ) * PKHFL(JL) + &
                & ( RETV * ZSLGM1(JL,KLEV) / RCPD )     * PKQFL(JL) 

    IF ( ZKHVFL(JL) >= 0.0_JPRB ) THEN
      
    ! stable BL (no updrafts expected/needed)
      KPBLTYPE(JL)  = 0

    ELSE

      LLDONE(JL,JD) = .FALSE.  !confirm launch
     
      !* 3.2    Sigma-w-L60 (ignore 1-z/zi term)
 
      ZUSTAR  (JL)  = MAX( SQRT(PKMFL(JL)), ZREPUST )      ! u* (repust=10e-4

      ZWSIGMA(JL)      = 1.2_JPRB &
       & * ( ZUSTAR(JL)**3 &
       & - 1.5_JPRB * RKAP * ZKHVFL(JL) * PGEOH(JL,KLEV-1) / PTM1(JL,KLEV-1) &
       & ) ** ( 1.0_JPRB/3._JPRB )                         ! Kolmogorov 1/3-power



      !* 3.3    Initialize updraft
       
      !get the constant associated with the top ZFRACTEST % of the PDF
      ZFACEXC(JL,1) = ZFACTESTEXC                              
      !calculate the initial excess values
      ZWU2H(JL,KLEV-1,JD) = ( ZFACEXC(JL,1) * ZWSIGMA(JL) )**2               
      ZTEXC               = - ZFACEXC(JL,1) * PKHFL(JL) / ZWSIGMA(JL) 
      ZQEXC               = - ZFACEXC(JL,1) * PKQFL(JL) / ZWSIGMA(JL)       
      ZTEXC            = MAX(ZTEXC, 0.0_JPRB)
      ZQEXC            = MAX(ZQEXC, 0.0_JPRB)
      PQTUH(JL,KLEV-1,JD) = ZQTENH(JL,KLEV-1) + ZQEXC
      ZQCUH(JL,KLEV-1,JD) = ZQLENH(JL,KLEV-1) + ZQIENH(JL,KLEV-1)
      ZQUH (JL,KLEV-1,JD) = PQTUH(JL,KLEV-1,JD)  - ZQCUH(JL,KLEV-1,JD)
      PSLGUH(JL,KLEV-1,JD)= ZSLGENH(JL,KLEV-1) + RCPD * ZTEXC
      ZTUH (JL,KLEV-1,JD) = ( PSLGUH (JL,KLEV-1,JD) - PGEOH(JL,KLEV-1) &
                       & + RLVTT*ZQLENH(JL,KLEV-1) + RLSTT*ZQIENH(JL,KLEV-1) &
                       & ) / RCPD
      PUUH(JL,KLEV-1,JD)= ZUENH(JL,KLEV-1)
      PVUH(JL,KLEV-1,JD)= ZVENH(JL,KLEV-1)
      
    ENDIF
  ENDDO !JL


  !* 3.4   Release the test updraft #1
  !*          - Mainly used to get a first guess of the heights of cloud base & inversion,
  !*            and to determine PBL type accordingly.
  ! CGL for the moment a constant time scale testing testing
  ZTAUEPS(:) = 400.

  ! iteration is done to increase the accuracy of the 
  ! estimate of the inversion height (or cloud base height) zi
  ! used in the parameterization of epsilon (entrainment)
  !
  DO FIRSTTIME=0,1
    CALL VDFPARCELHL (KIDIA  , KFDIA   , KLON    , KLEV   , KDRAFT, &
                & PGEOH   , PGEOM1  , PAPHM1  , &
                & PUM1    , PVM1    , ZQTM1   , ZSLGM1  , ZTVEN   , &
& PUUH    , PVUH    , PSLGUH  , PQTUH   , ZWU2H   , ZQCUH  , ZBUOF , &
& ZQUH    , ZTUH    , ZEPS    , &
& PZPLCL  , KPLCL   , PZPTOP  , KPTOP  , KPLZB , &
& JD      , ZUPGENL , ZUPGENN , &
& ZTAUEPS , ZW2THRESH, LLDONE, KPBLTYPE, KSTEP, firsttime,ZFRACB)  
  ENDDO



!     -----------------------------------------------------------------
!*         4.     CLASSIFICATION OF THE CONVECTIVE PBL
!                 ------------------------------------


  !* 4.1    Classify the convective PBL
  !*
  
      
   DO JL=KIDIA,KFDIA
     IF ( KPBLTYPE(JL)/=0 ) THEN
    
       !CGL loose criterium by 100 m
       IF ( PZPLCL(JL,1) > (PZPTOP(JL,1)+100._JPRB) .OR. KPLCL(JL,1) == 0 ) THEN
      
         !dry convective PBL
         KPBLTYPE(JL)  = 1                   !dry convective PBL
         ZDZCLOUD(JL)  = 0.0_JPRB            !cloud thickness

       ELSE

         !moist convective PBL
         ZDZCLOUD(JL)  = PZPTOP(JL,1) - PZPLCL(JL,1) !cloud thickness
         IF (ZDZCLOUD(JL)>ZCLDDEPTHDP) THEN
        
   !deep convection
   KPBLTYPE(JL) = 4
          
 ELSE

           KPBLTYPE(JL) = 2   !set the type to stratocumulus for the moment
          
         ENDIF

       ENDIF

    ENDIF !KPBLTYPE /=0
  ENDDO !JL
  
  
  
  !* 4.2    Check the stratocumulus/shallow cumulus criterion (trigger function)
  !*        If shallow cumulus is diagnosed, KPBLTYPE will be set to 3
  !*
  CALL VDFSTCUCRITHL ( KIDIA , KFDIA  , KLON  , KLEV , KDRAFT , &
&    PTM1  , ZSLGM1 , ZQTM1 , PAPM1 , &
&    ZSTABTHRESH, ZCLDDEPTH, ZBIRTHRESH, ZDZCLOUD, &
&    KPTOP , KPBLTYPE, LDNODECP)


!     -----------------------------------------------------------------

!*         5.     CLOSURE FOR ORGANIZED UPDRAFTS (JD=2,3)
!                 ---------------------------------------


  !* 5.1    Determine some mixed layer scalings
  !*

  DO JL=KIDIA,KFDIA
    
    IF ( KPBLTYPE(JL)/=0 ) THEN    !don't do this for stable PBL
      
       SELECT CASE (KPBLTYPE(JL))
    
        CASE(1)
          !Dry convective PBL - Inversion height
          ZZI(JL)   = PZPTOP(JL,1)

        CASE(2)
          !Stratocumulus - Inversion height
          !CAUTION: During decoupling in the intermediate regime (e.g. ASTEX/ATEX) the
  !   relevant ML scaling height changes from PBL inversion to level of minimum
          !   buoyancy flux. In the current setup this is not modelled yet!
          ZZI(JL)   = PZPTOP(JL,1)
  
        CASE(3)
          !Shallow cumulus - Level of minimum buoyancy flux
  !Assume that the moist updraft LCL is very close to this level
          ZZI(JL)   = PZPLCL(JL,1)
  
        CASE(4)
          !Deep cumulus - Only do a dry parcel up to cloud base
          ZZI(JL)   = PZPLCL(JL,1)

      END SELECT 
      
     !--- Mixed layer convective velocity scale ---
      ZWSTAR(JL) = ( -ZKHVFL(JL) * RG * ZZI(JL) / ZTHVEN(JL,KLEV)  ) ** (1._JPRB/3._JPRB)
      PEXTR2(JL,46) = ZWSTAR(JL)
     ! CGL for the moment revert back to old constant time scale 
      ZTAUEPS(JL) = 400._JPRB

      
    ENDIF
    
  ENDDO  

    
  !* 5.3    Closure of updraft area fractions (JD=2,3)
  !*
    
  DO JL=KIDIA,KFDIA
    
    IF ( KPBLTYPE(JL)/=0 ) THEN    !don't do this for stable PBL

      SELECT CASE (KPBLTYPE(JL))
    
        CASE(1)
          !Dry convective PBL
  ZFRACB(JL,3) = 0._JPRB
           ZFRACB(JL,2) = ZFRACMAX - ZFRACB(JL,3) 
        CASE(2)
          !Stratocumulus
  ZFRACB(JL,3) = 0.1_JPRB 
          ZFRACB(JL,2) = ZFRACMAX - ZFRACB(JL,3) 
        CASE(3)
          !Shallow cumulus            
       ZFRACB(JL,3)  = 0.03_JPRB
         ZFRACB(JL,2) = ZFRACMAX - ZFRACB(JL,3)

        CASE(4)
          !Deep cumulus
  ZFRACB(JL,3) = 0._JPRB
          ZFRACB(JL,2) = ZFRACMAX - ZFRACB(JL,3) 

      END SELECT !KPBLTYPE
      
      PEXTR2(JL,45) = 100._JPRB*ZFRACB(JL,3)

    ENDIF !KPBLTYPE /=0
    
  ENDDO !JL


      
!     -----------------------------------------------------------------

!*         6.     CALCULATE VERTICAL PROFILES OF ALL UPDRAFTS (JD=2,3)
!                 ----------------------------------------------------


  !*       6.1    Calculate the scaling factors of the updraft excess with the surface joint PDFs
  !*
  DO JD = 2,KDRAFT
    DO JL=KIDIA,KFDIA
      
      IF ( KPBLTYPE(JL)/=0 .AND. ZFRACB(JL,JD)>0._JPRB ) THEN
        
        !-- Get the PDF scaling factor --
SELECT CASE (JD)
    
  CASE(2)
    !lower part of top ZFRACMAX %
    ZDUMFRAC = ZFRACMAX - ZFRACB(JL,2)
            CALL VDFPDFTABLEHL(ZDUMFRAC , ZFACEXC(JL,2), ZDUMR, ZDUMR, 0)
    ZFACEXC(JL,2) = ( ZFRACMAX * ZFACMAXEXC - ZDUMFRAC * ZFACEXC(JL,2) ) / ZFRACB(JL,2)
  CASE(3)
    !upper part of top ZFRACMAX %
    ZDUMFRAC = ZFRACB(JL,JD)
            CALL VDFPDFTABLEHL(ZDUMFRAC , ZFACEXC(JL,3), ZDUMR, ZDUMR, 0)
    
END SELECT

      ENDIF !KPBLTYPE & ZFRACB

    ENDDO !JL
  ENDDO !JD
    
    
  !*       6.2    Vertical integration of dry & moist updraft budgets (JD=2,3)
  !*
  DO JD = 2,KDRAFT
  
    !-- Initialize updraft --
    DO JL=KIDIA,KFDIA
        
      IF ( KPBLTYPE(JL)/=0 .AND. ZFRACB(JL,JD)>0._JPRB ) THEN
        
        LLDONE(JL,JD) = .FALSE. !confirm launch
          
        ZWU2H(JL,KLEV-1,JD) = ( ZFACEXC(JL,JD) * ZWSIGMA(JL) )**2         
        ZTEXC            = - ZFACEXC(JL,JD) * PKHFL(JL) / ZWSIGMA(JL) 
        ZQEXC            = - ZFACEXC(JL,JD) * PKQFL(JL) / ZWSIGMA(JL) 
      

! CGL limit the updraft velocity. Gives instability near the surface.
! only done for dry updraft, moist updraft Wu is not used for massflux
! thus no problem

 IF (JD.EQ.2) THEN
            ZWU2H(JL,KLEV-1,JD) = (        0.25_JPRB*ZWSIGMA(JL) )**2  
 ELSE   
            ZWU2H(JL,KLEV-1,JD) = (              ZWSIGMA(JL) )**2  
 ENDIF   

        ZTEXC            = MAX(ZTEXC, 0.0_JPRB)
        ZQEXC            = MAX(ZQEXC, 0.0_JPRB)
        PQTUH(JL,KLEV-1,JD) = ZQTENH(JL,KLEV-1) + ZQEXC
        ZQCUH(JL,KLEV-1,JD) = ZQLENH(JL,KLEV-1) + ZQIENH(JL,KLEV-1)
        ZQUH (JL,KLEV-1,JD) = PQTUH(JL,KLEV-1,JD)  - ZQCUH(JL,KLEV-1,JD)
        PSLGUH(JL,KLEV-1,JD)= ZSLGENH(JL,KLEV-1) + RCPD * ZTEXC
        ZTUH (JL,KLEV-1,JD) = ( PSLGUH (JL,KLEV-1,JD) - PGEOH(JL,KLEV-1) &
                       & + RLVTT*ZQLENH(JL,KLEV-1) + RLSTT*ZQIENH(JL,KLEV-1) &
                       & ) / RCPD
        PUUH(JL,KLEV-1,JD)= ZUENH(JL,KLEV-1)
        PVUH(JL,KLEV-1,JD)= ZVENH(JL,KLEV-1)
 
      ENDIF !KPBLTYPE & ZFRACB
      
    ENDDO !JL
    
    
    !-- Release the updraft --
    ! iteration is done to increase the accuracy of the 
    ! estimate of the inversion height (or cloud base height) zi
    ! used in the parameterization of epsilon (entrainment)

    DO FIRSTTIME=0,1
      CALL VDFPARCELHL (KIDIA   , KFDIA   , KLON    , KLEV   , KDRAFT, &
                & PGEOH   , PGEOM1  , PAPHM1  , &
                & PUM1    , PVM1    , ZQTM1   , ZSLGM1  , ZTVEN   , &
                & PUUH    , PVUH    , PSLGUH  , PQTUH   , ZWU2H   , ZQCUH  , ZBUOF , &
                & ZQUH    , ZTUH    , ZEPS    , &
                & PZPLCL  , KPLCL   , PZPTOP  , KPTOP  , KPLZB , &
                & JD      , ZUPGENL , ZUPGENN , &
                & ZTAUEPS , ZW2THRESH, LLDONE, KPBLTYPE, KSTEP, firsttime, ZFRACB)
    ENDDO

    !  do some consistency check; to be done    

  ENDDO !JD 


  !*        6.3. In case no lcl is found in final updraft calculation, do some resque
  !*
  !*
  ! CGL      made an adjustment in vdfparcel to initialize pzplcl = -100 
  
  DO JL=KIDIA,KFDIA

    IF ( KPBLTYPE(JL)==2 .OR. KPBLTYPE(JL)==3) THEN

      IF (  PZPLCL(JL,3) < 0._JPRB .OR. KPLCL(JL,3).LT.KPTOP(JL,3) .or. KPLCL(JL,3).EQ.0  .or. KPTOP(JL,3).EQ.0 ) THEN
        KPBLTYPE(JL) = 1.
        ZFRACB(JL,2) = ZFRACMAX 
ZFRACB(JL,3) = 0._JPRB
KPLCL(JL,3) = 0
      ENDIF

    ENDIF

  ENDDO !JL 

  !*        6.5  Updraft precipitation fluxes (rain and snow)
  !*
  DO JD = 3,KDRAFT  !moist updrafts only

    DO JK=2,KLEV
      DO JL=KIDIA,KFDIA

        ZDZRHO = ZRG * ( PAPHM1(JL,JK)-PAPHM1(JL,JK-1) )

        !-- Add precip generation to flux [kg /m2 /s: tendency * layer depth * air density] --
        ZUPFLXL(JL,JK,JD) = ZUPFLXL(JL,JK-1,JD) + ZUPGENL(JL,JK,JD) * ZDZRHO
        ZUPFLXN(JL,JK,JD) = ZUPFLXN(JL,JK-1,JD) + ZUPGENN(JL,JK,JD) * ZDZRHO

        !-- Do some melting at freezing level (snow->rain) --
        IF (ZUPFLXN(JL,JK,JD)>0._JPRB .AND. PTM1(JL,JK) > RTT) THEN
          ZUPMELT = (1.0_JPRB+0.5_JPRB*(PTM1(JL,JK)-RTT)) * &
                  & (PTM1(JL,JK)-RTT) * RCPD/(RLMLT*RTAUMEL) * ZDZRHO
          ZUPMELT = MIN(ZUPFLXN(JL,JK,JD),ZUPMELT)
          ZUPFLXL(JL,JK,JD) = ZUPFLXL(JL,JK,JD) + ZUPMELT
          ZUPFLXN(JL,JK,JD) = ZUPFLXN(JL,JK,JD) - ZUPMELT
        ENDIF

        !-- Saturation deficit of mean state T --
        ZQSATM = FOEEWM(PTM1(JL,JK))/PAPM1(JL,JK)
        ZQSATM = MIN(0.5_JPRB,ZQSATM)
        ZQSATM = ZQSATM/(1.0_JPRB-RETV*ZQSATM)
        ZSATDEF = MAX( 0._JPRB, ZQSATM-PQM1(JL,JK) )

        ZPFLXTOT = ZUPFLXL(JL,JK,JD) + ZUPFLXN(JL,JK,JD)
        IF (ZPFLXTOT>0._JPRB) THEN

          !-- Precip evaporation tendency [kg/kg /s] (Kessler 1969, Tiedtke 1993) --
          ZPEVAPUP = 0.001_JPRB * ZSATDEF * ( &             !cy32r1
              & ( ZPFLXTOT / 0.00509_JPRB ) * &
              & ( PAPM1(JL,JK)/PAPHM1(JL,KLEV) )**0.5_JPRB &
              & )**0.5777_JPRB

          !-- Back-partition evaporation and substract from fluxes --
          ZFAC = ZUPFLXL(JL,JK,JD) / ZPFLXTOT
          ZUPFLXL(JL,JK,JD) = ZUPFLXL(JL,JK,JD) - ZPEVAPUP * ZDZRHO * ZFAC
          ZUPFLXN(JL,JK,JD) = ZUPFLXN(JL,JK,JD) - ZPEVAPUP * ZDZRHO * (1._JPRB - ZFAC)
          ZUPFLXL(JL,JK,JD) = MAX(0._JPRB,ZUPFLXL(JL,JK,JD))
          ZUPFLXN(JL,JK,JD) = MAX(0._JPRB,ZUPFLXN(JL,JK,JD))
        ENDIF

      ENDDO
    ENDDO

    !Add contribution to total flux - weight by updraft area fraction
    DO JK=0,KLEV
      DO JL=KIDIA,KFDIA
        PFPLVL(JL,JK) = PFPLVL(JL,JK) + ZFRACB(JL,JD) * ZUPFLXL(JL,JK,JD)
        PFPLVN(JL,JK) = PFPLVN(JL,JK) + ZFRACB(JL,JD) * ZUPFLXN(JL,JK,JD)
      ENDDO
    ENDDO

  ENDDO !JD

 

!     -----------------------------------------------------------------

!*         7.     CONSTRUCT MASS FLUX PROFILES (JD=2,3)
!                 -------------------------------------


  !*         7.1  Determine the mixed layer scaling height for JD=2,3
  !*


  !*         7.2  Construct subcloud / mixed layer mass fluxes
  !*               - use constant area fraction, and multiply by parcel w
  !*
  DO JD = 2,KDRAFT

    DO JK=KLEV-1,1,-1

      DO JL=KIDIA,KFDIA

         IF ( KPBLTYPE(JL)/=0 .AND. ZFRACB(JL,JD)>0._JPRB) THEN
  
          ZWUH = ZWU2H(JL,JK,JD)**0.5_JPRB
       
          PMFLX(JL,JK,JD)  = ZFRACB(JL,JD) * ZWUH * ZRHOH(JL,JK)
  
  IF (JD .EQ. 3 .AND. JK .GE. KPLCL(JL,3) ) THEN
    ! do only below LCL !!
            ! Now include a robust mini statistical cloud scheme (Lenderink & Siebesma 2000)
            ! Use RCCSTAT that comes from t-1 timestep

            PMFLX(JL,JK,JD)  = 0.3_JPRB * RCCSTAT(JL,KPLCL(JL,3))* ZWSTAR(JL) *  &
     &                         ZRHOH(JL,JK)*PGEOH(JL,JK)/PGEOH(JL,KPLCL(JL,3))
 
    !CGL cloud depth correction, if cloud thin then limit 
    ZDZCLOUD(JL) = MAX(PGEOH(JL,KPTOP(JL,3))*ZRG-PGEOH(JL,KPLCL(JL,3))*ZRG,0._JPRB)
     IF( ZDZCLOUD(JL).LT.400._JPRB ) THEN
       PMFLX(JL,JK,JD) = PMFLX(JL,JK,JD) * ZDZCLOUD(JL) / 400._JPRB
     ENDIF  
        
  ENDIF 
  
  IF (ZWU2H(JL,JK,JD)>0._JPRB) THEN
    ZFRAC(JL,JK,JD) = ZFRACB(JL,JD)   
  ELSE
    ZFRAC(JL,JK,JD) = 0._JPRB
  ENDIF
          
 ENDIF

      ENDDO !JL
    
    ENDDO !JK
    
  ENDDO !JD
  
  
  ! Do computation for Chi critical dpendency 
  !First, determine chicritmean in lower half of the
  !                 cloud layer. chicritmean is used to determine
  !                 the fractional detrainment coefficient for
  !                 the lower half of the cloud layer according
  !                 to De Rooy & Siebesma MWR 2008. In the upper
  !                 half of the cloud layer a linear decreasing
  !                 mass flux to 0 at cloud layer top is prescribed.

  DO JL=KIDIA,KFDIA

    ! In between; determine thetavup of dry updraft (2) 
    ! This is used for the thetav flux contribution to TKE (only in AROME)

    DO JK=KLEV-1,1,-1
      IF (KPTOP(JL,2) <= JK .AND. KPBLTYPE(JL) > 0) THEN
        ZEXNERH(JL,JK) = (PAPHM1(JL,JK)/ZPRES_0)**ZKAPPA
        PTHTVUH(JL,JK,2) = (ZTUH(JL,JK,2)/ ZEXNERH(JL,JK)) &
                        &  * (1._JPRB + RETV * ZQUH(JL,JK,2) )
      ENDIF
    ENDDO

    IF (KPBLTYPE(JL) .EQ. 3   .OR. KPBLTYPE(JL) .EQ. 2 ) THEN
     
      ZSTAR(JL) = (PGEOH(JL,KPTOP(JL,3))*ZRG+PGEOH(JL,KPLCL(JL,3))*ZRG)/2._JPRB 
      ZMINIMUM(JL) = 10000._JPRB
      ZMEANCHICRIT(JL) = 0._JPRB
  
      ! Find level half way the cloud layer
 
      !CGL added security 
      KSTAR(JL) = KPTOP(JL,3)

      DO JK=KPLCL(JL,3),KPTOP(JL,3),-1

 
        IF (ABS(PGEOH(JL,JK)*ZRG - ZSTAR(JL)) .LT. ZMINIMUM(JL)) THEN
          ZMINIMUM(JL) = ABS(PGEOH(JL,JK)*ZRG - ZSTAR(JL))
          KSTAR(JL) = JK
        ENDIF
  
      
        ! determine chicrit in cloud layer

        ZEXNERH(JL,JK) = (PAPHM1(JL,JK)/ZPRES_0)**ZKAPPA
        ZTHTVENH(JL,JK) = (ZTENH(JL,JK)/ZEXNERH(JL,JK)) &
              * (1._JPRB + RETV * ZQVENH(JL,JK) & 
        ! add qice correction
                        &    - ZQLENH(JL,JK) -ZQIENH(JL,JK) ) 
        ZTHTLUH(JL,JK) = (ZTUH(JL,JK,3) / ZEXNERH(JL,JK)) - &
                        & ((RLVTT/(RCPD*ZEXNERH(JL,JK)))*ZQCUH(JL,JK,3))
        !  N.B. ZQCUH is initialized with ql and qi from the environment!?!

        ZTHTVUH(JL,JK) = (ZTUH(JL,JK,3)/ ZEXNERH(JL,JK)) &
                       &  * (1._JPRB + RETV * ZQUH(JL,JK,3) &
                       &    - ZQCUH(JL,JK,3)) 
        PTHTVUH(JL,JK,3)=ZTHTVUH(JL,JK)
        ZTHTLEH(JL,JK) = (ZTENH(JL,JK) / ZEXNERH(JL,JK)) - &
                      & (RLVTT/(RCPD*ZEXNERH(JL,JK)))* &
                      & ZQLENH(JL,JK)
        ZESATU(JL,JK) = 0.61078_JPRB * EXP(17.2694_JPRB * &
                      &  (ZTUH(JL,JK,3)-RTT)/(ZTUH(JL,JK,3)-35.86_JPRB))

        ZQSATU(JL,JK) = FOEEWM(ZTUH(JL,JK,3))/PAPHM1(JL,JK)
        ZQSATU(JL,JK) = MIN(0.5_JPRB,ZQSATU(JL,JK))
        ZQSATU(JL,JK) = ZQSATU(JL,JK)/(1.0_JPRB-RETV*ZQSATU(JL,JK))
        ZDQSDTU(JL,JK) = (0.622_JPRB * RLVTT * ZQSATU(JL,JK))/ &
                      & (RD * ZTUH(JL,JK,3) * ZTUH(JL,JK,3)) 
        ZGAMMA(JL,JK) = (RLVTT/RCPD)*ZDQSDTU(JL,JK)
        ZDUMFUNC(JL,JK) = (1._JPRB/(1._JPRB+ZGAMMA(JL,JK)))* &
                      & ((ZQTENH(JL,JK)-PQTUH(JL,JK,3)) - &
                      &  ZEXNERH(JL,JK) * ZDQSDTU(JL,JK) * &
                      &  (ZTHTLEH(JL,JK) - ZTHTLUH(JL,JK)))
        ZCHICRIT(JL,JK) = (ZTHTVENH(JL,JK)-ZTHTVUH(JL,JK))/ &
                      & (ZTHTLUH(JL,JK)*(RETV*(ZQTENH(JL,JK) - &
                      & PQTUH(JL,JK,3)) -  (1._JPRB +RETV)* &
                      & ZDUMFUNC(JL,JK)) + (ZTHTLEH(JL,JK) - &
                      & ZTHTLUH(JL,JK)) * (1._JPRB + RETV* &
                      & PQTUH(JL,JK,3) - (1._JPRB + RETV)* &
            & ZQCUH(JL,JK,3)) + (RLVTT/(ZEXNERH(JL,JK)*RCPD))* &
            &  ZDUMFUNC(JL,JK))
    
  !CGL ; do some safety
    
ZCHICRIT(JL,JK) =  MIN(ZCHICRIT(JL,JK),1._JPRB)  
         
      ENDDO !JK

      !   Calculate weighted mean chicrit over lower half cloud layer
      !   Exclude cloud base level (see de Rooy & Siebesma MWR 2008)

      IF (KPLCL(JL,3)-KSTAR(JL) .GT. 1) THEN
        DO JK= KPLCL(JL,3)-1,KSTAR(JL),-1
          ZMEANCHICRIT(JL)=ZMEANCHICRIT(JL)+ZCHICRIT(JL,JK)* &
            & (PGEOM1(JL,JK)*ZRG-PGEOM1(JL,JK+1)*ZRG)
        ENDDO
        ZMEANCHICRIT(JL)=ZMEANCHICRIT(JL)/(PGEOM1(JL,KSTAR(JL))*ZRG- &
             &                             PGEOM1(JL,KPLCL(JL,3))*ZRG)

        !   Fraction of mass flux that is left half way the cloud layer (ZFRACMB)
        !   is determined using LES based relation between mean chicrit and ZFRACMB
        !   Small adjustment to somewhat more active Mass flux profiles
        !   Note that the parameterization counteracts changes in the relation, therefore
        !   the ultimate differences are small
        
        ZFRACMB(JL)=MAX(0.05_JPRB,5.24_JPRB*ZMEANCHICRIT(JL)-0.39_JPRB)
        ZFRACMB(JL)=MIN(1.0_JPRB,ZFRACMB(JL))     !safety CGL
       

        ! Determine (constant) detrainment coefficient 
        ! because of new eps=1/(z-zb+500) detr should be adapted
        ! However we can also use original detr formulation as long as we also
        ! use eps=1/z in the mass flux calculation (the new eps is then only used
        ! for the updraft dilution)
        ZDETRSHALLOW(JL)=LOG((PGEOH(JL,KSTAR(JL))*ZRG)/(PGEOH(JL,KPLCL(JL,3))*ZRG &
            &   *ZFRACMB(JL)))/(PGEOH(JL,KSTAR(JL))*ZRG-PGEOH(JL,KPLCL(JL,3))*ZRG)
        ! see below the new consistent formulation (if used also change at determination mass flux)
      ELSE
        !  only for very shallow (less that 2 layers) cloud layers
        ZDETRSHALLOW(JL)=0.00275_JPRB 
        ! the lower assignement only has a meaning if used in combination with 
        ! extremely thin clouds
        ZFRACMB(JL)=0.3
      ENDIF

  ENDIF ! KPBLTYPE=3

 ENDDO !JL  
 
 
! end computation by for chicritical
       
 
!*         7.3  Construct cloudy mass flux profile (JD=3 only)
!*


  DO JK=KLEV-2,1,-1
  
    DO JL=KIDIA,KFDIA
   
      IF ( KPBLTYPE(JL)/=0 .AND. KPBLTYPE(JL)/=4 .AND. ZFRACB(JL,3)> 0._JPRB ) THEN        
        
        IF (JK>=KPTOP(JL,3) .AND. JK<KPLCL(JL,3)) THEN
 
           ZWUH = ZWU2H(JL,JK,3)**0.5_JPRB
        
           !  Cloud layer vertical structure
          
           ! ZDELTAMINEPS should be named ZEPSMINDELTA 
      
           IF (JK+1 .GE. KSTAR(JL)) THEN !lower half cloud layer

              ! With changed eps formulation change zdeltamineps also

              ZDELTAMINEPS(JL,JK+1) = (RG/PGEOH(JL,JK+1)-ZDETRSHALLOW(JL))

           ELSE !upper half cloud layer
              ZDELTAMINEPS(JL,JK+1) = -1._JPRB/(PGEOH(JL,KPTOP(JL,3))*ZRG - &
                  &  PGEOH(JL,JK+1)*ZRG)
           ENDIF

           ZDZ          = (PGEOH(JL,JK) - PGEOH(JL,JK+1))*ZRG
 
           PMFLX(JL,JK,3) = PMFLX(JL,JK+1,3) * EXP( ZDZ * ( ZDELTAMINEPS(JL,JK+1) ) ) !exact

   ! make a guess at area fraction; 
  
           ZWUH = ZWU2H(JL,JK,3)**0.5_JPRB
           ZFRAC(JL,JK,3) = PMFLX(JL,JK,3) / ( ZWUH * ZRHOH(JL,JK) )

ENDIF 
        
      ENDIF
     
    ENDDO !JL
     
  ENDDO !JK
  
  
!           7.3b Mini statistical cloud scheme (according to Lenderink/Siebesma 2000)

  DO JL=KIDIA,KFDIA
    IF ( KPBLTYPE(JL)==3 .AND. ZFRACB(JL,3)> 0._JPRB .and.  &
     & (KPLCL(JL,3)-KPTOP(JL,3)) .gt. 0) THEN
      WSTCLD = 0._JPRB
      DO JK=KPLCL(JL,3),KPTOP(JL,3),-1
        WSTCLD=WSTCLD + (RG/(ZTHTENH(JL,JK))*PMFLX(JL,JK,3)*(ZTHTVUH(JL,JK)- &
         &    ZTHTVENH(JL,JK))*(PGEOM1(JL,JK)*ZRG-PGEOM1(JL,JK+1)*ZRG))
        DZFULL = (PGEOM1(JL,JK) - PGEOM1(JL,JK+1))*ZRG
        DQDZ(JL,JK) = (ZQTM1(JL,JK)-ZQTM1(JL,JK+1))/DZFULL
        RHOUP(JL,JK) = PAPHM1(JL,JK)/(RD*ZTHTVUH(JL,JK))
        !-- Saturation of mean state T at half level--
        ZQSATH(JL,JK) = FOEEWM(ZTENH(JL,JK))/PAPHM1(JL,JK)
        ZQSATH(JL,JK) = MIN(0.5_JPRB,ZQSATH(JL,JK))
        ZQSATH(JL,JK) = ZQSATH(JL,JK)/(1.0_JPRB-RETV*ZQSATH(JL,JK))
        ZESATH2(JL,JK) = 0.61078_JPRB * EXP(17.2694_JPRB * &
                      &  (ZTENH(JL,JK)-RTT)/(ZTENH(JL,JK)-35.86_JPRB))
        ZQSATH2(JL,JK) = 0.622_JPRB * ZESATH2(JL,JK)/ &
                      & ((PAPHM1(JL,JK)/1000._JPRB)-ZESATH2(JL,JK))
      ENDDO
      IF (WSTCLD .GT. 0.) THEN
        WSTCLD=WSTCLD**(1._JPRB/3._JPRB)
      ELSE
        WSTCLD=0.3_JPRB
      ENDIF
      DO JK=KPLCL(JL,3),KPTOP(JL,3),-1
        VARQ(JL,JK)=PMFLX(JL,JK,3)*(PQTUH(JL,JK,3)-ZQTENH(JL,JK))*ZDZCLOUD(JL)*  &
         &          ABS(DQDZ(JL,JK))/(WSTCLD*RHOUP(JL,JK))
        IF (VARQ(JL,JK) .LE. 0._JPRB) THEN
          IF (ZQTENH(JL,JK)-ZQSATH(JL,JK) .LT. 0._JPRB) THEN
            Q1(JL,JK) = -9._JPRB
          ELSE
            Q1(JL,JK) = 9._JPRB
          ENDIF
        ELSE
          Q1(JL,JK) = (ZQTENH(JL,JK)-ZQSATH(JL,JK))/ &
           &         ( SQRT(VARQ(JL,JK))+0.02_JPRB*ZQSATH(JL,JK))
        ENDIF

        RCCSTAT(JL,JK)=0.5_JPRB+0.36_JPRB*ATAN(1.55_JPRB*Q1(JL,JK))

        IF (RCCSTAT(JL,JK) .LE. 0._JPRB) THEN
          !  Watch out!
          !  because the convective parameterization must be able to start up
          !  again after a timestep with ccstat=0 we here prescribe a small minimum
          !  value. Of course the "real" ccstat (used for radiation etc.)
          !  should be put in a seperate array, however here ccstat is only used
          !  in the Mb closure.

          RCCSTAT(JL,JK) = 0.01_JPRB
        ENDIF
      ENDDO
    ENDIF
  ENDDO
  !*        7.4  Compute some fluxes used in the qt-variance budget in VDFMAIN.
  !*
  DO JK=KLEV-1,1,-1
    DO JL=KIDIA,KFDIA
      
      !-- calculate w'qt' at half levels --
      PWQT(JL,JK) = PMFLX(JL,JK,3) * (PQTUH(JL,JK,3) - ZQTENH(JL,JK)) + &
               & PMFLX(JL,JK,2) * (PQTUH(JL,JK,2) - ZQTENH(JL,JK))
                 
      !-- calculate w'qt'qt' at half levels --
      PWQT2(JL,JK) =  PMFLX(JL,JK,3) * (PQTUH(JL,JK,3) - ZQTENH(JL,JK))**2._JPRB + &
                  &  PMFLX(JL,JK,2) * (PQTUH(JL,JK,2) - ZQTENH(JL,JK))**2._JPRB

    ENDDO !JL
  ENDDO !JK



  !*        7.5  Mass flux limit according to CFL criterion
  !*              (reduce M profiles uniformly by maximum excess)
  !* 
  ZCONS10 = 1.0_JPRB/(RG*PTMST)
  DO JD = 2,KDRAFT
    DO JL=KIDIA,KFDIA
      ZMFS(JL,JD) = 1.0_JPRB  ! mass flux scaling value (reduction)
    ENDDO
  ENDDO
  
  DO JD = 2,KDRAFT
    DO JK=1,KLEV-1
      DO JL=KIDIA,KFDIA
        IF ( JK >= KPTOP(JL,JD) .AND. KPTOP(JL,JD)>0) THEN
          ZMFMAX = (PAPM1(JL,JK+1)-PAPM1(JL,JK)) * ZCONS10
          IF ( PMFLX(JL,JK,JD) > ZMFMAX ) THEN
            ZMFS(JL,JD) = MIN(ZMFS(JL,JD),ZMFMAX/PMFLX(JL,JK,JD))
          ENDIF
        ENDIF
      ENDDO
    ENDDO
  ENDDO
  
  DO JD = 2,KDRAFT
    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        PMFLX(JL,JK,JD) = PMFLX(JL,JK,JD)*ZMFS(JL,JD)
      ENDDO
    ENDDO
  ENDDO



  !*        7.6  Cloud-layer average moist updraft mass flux.
  !*               (used in VDFEXCU for estimating entrainment-K at cumulus PBL top)  
  DO JK=KLEV-1,1,-1
    DO JL=KIDIA,KFDIA
      !-- cloudy mass flux: use moist updraft M (depth average) --
      IF ( ZQCUH(JL,JK,3)>0._JPRB .AND. PMFLX(JL,JK,3)>0._JPRB ) THEN  
        ZMCLD(JL) = ZMCLD(JL) + PMFLX(JL,JK,3) * &
                &     ZRG*( PGEOH(JL,JK-1) - PGEOH(JL,JK) )
      ENDIF
    ENDDO
  ENDDO
  
  DO JL=KIDIA,KFDIA
    IF (PZPTOP(JL,3)-PZPLCL(JL,3)>0._JPRB ) THEN  
      PMCU(JL) = ZMFLXB(JL,3)
    ENDIF
    PEXTR2(JL,18) = PMCU(JL)
  ENDDO


    
  !*        7.7  Scaling (see VDFEXCU)
  !* 
  ! In AROME NO SCALING!



!     -----------------------------------------------------------------

!*         8.     W-SCALE USED IN CLOUD VARIANCE DISSIPATION (VDFMAIN)
!                 ----------------------------------------------------

  DO JL=KIDIA,KFDIA
    PWUAVG(JL) = ZWSTAR(JL)
    
    PEXTR2(JL,37) = PWUAVG(JL)
    PEXTR2(JL,38) = SQRT ( ZWU2H(JL,KLEV-1,1) )
  ENDDO

  !CGL fill this array at the end
  PFRACB = ZFRACB


  !---------------- some output ------------------------

  DO JL=KIDIA,KFDIA

    ! linear interpolation of buoyancy to compute half level mass flux buoyancy flux
    DO JK=2,KLEV-1
    DO JD = 2,KDRAFT
       ZGRAD = (ZBUOF(JL,JK-1,JD) - ZBUOF(JL,JK,JD))/(PAPM1(JL,JK-1) - PAPM1(JL,JK))
       ZBUOH(JL,JK-1,JD) = ZBUOF (JL,JK,JD) + ZGRAD * (PAPHM1(JL,JK-1) - PAPM1(JL,JK))
    ENDDO
    ENDDO

    JK = KLEV-1
    DO JD = 2,KDRAFT
      ZBUOH(JL,JK,JD) = RG/PTM1(JL,JK) * &  !cstep for diagnostics
         &            (PSLGUH (JL,JK,JD)/RCPD  * (1+RETV * PQTUH  (JL,JK,JD))-&
         &             ZSLGENH (JL,JK) /RCPD  *  (1+RETV * ZQTENH (JL,JK)))  !assume no ql
    ENDDO

  ENDDO !JL

IF (LHOOK) CALL DR_HOOK('VDFHGHTNHL',1,ZHOOK_HANDLE)

END SUBROUTINE VDFHGHTNHL
