SUBROUTINE AER_DMSO &
  &( KIDIA , KFDIA , KLON  , &
  &  PCDMS , PFOCEN, PFSICE, PTOCEN, PWIND, &
  &  PFLXDMS &
  & ) 

!*** * AER_DMSO* - SOURCE TERM FOR OCEANIC DMS

!**   INTERFACE.
!     ----------
!          *AER_DMSO* IS CALLED FROM *AER_SRC*.

!* INPUTS:
!  -------
! PCDMS  (KLON) : DMS concentration                   (nmol l-1)
! PFOCEN (KLON) : fraction of grid covered by ocean
! PFSICE (KLON) : fraction of grid covered by sea-ice
! PTOCEN (KLON) : ocean skin temperature              (K)
! PWIND  (KLON) : wind strength at 10 m               (m s-1)

!* OUTPUTS:
!  --------
! PFLXDMS(KLON)        : Flux of D.M.S.               (molec cm-2 s-1)

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*
!        FROM E.COSME's liss (2002-11-01) 

!     SOURCE.
!     -------
!     LISS & MERLIVAT (The Role of Air-Sea Exchange in Geochemical Cycling
!     P. Buat-Menart, Ed., 113-127     

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2004-05-12

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST    ,ONLY : RTT  , RNAVO

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)  :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)  :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)  :: KFDIA 

REAL(KIND=JPRB)   ,INTENT(IN)  :: PCDMS(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PFOCEN(KLON), PFSICE(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)  :: PTOCEN(KLON), PWIND(KLON)

REAL(KIND=JPRB)   ,INTENT(OUT) :: PFLXDMS(KLON)    


!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JL
REAL(KIND=JPRB) :: ZCDMS(KLON)
REAL(KIND=JPRB) :: ZLISS, ZSCAL, ZSCHMIDT, ZSIVSOC, ZTOCE
  
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_DMSO',0,ZHOOK_HANDLE)

DO JL=KIDIA,KFDIA
  PFLXDMS(JL)=0._JPRB

  IF (PFOCEN(JL) < 1.E-10_JPRB) THEN
    ZTOCE=PTOCEN(JL)-RTT
    ZTOCE=MAX(ZTOCE,35._JPRB)
    ZSCHMIDT=2674._JPRB - ZTOCE* (147.12_JPRB - ZTOCE* (3.726_JPRB &
      & -ZTOCE* 0.038_JPRB ))

    IF (PWIND(JL) <= 3.6_JPRB) THEN
      ZLISS = 0.17_JPRB * PWIND(JL)
      ZLISS = ZLISS * (600._JPRB / ZSCHMIDT)**0.6666_JPRB
    ELSEIF (PWIND(JL) <= 13._JPRB) THEN
      ZLISS = 2.85_JPRB * PWIND(JL) - 9.65_JPRB
      ZLISS = ZLISS * (600._JPRB / ZSCHMIDT)**0.5_JPRB
    ELSE
      ZLISS = 5.9_JPRB * PWIND(JL) - 49.3_JPRB
      ZLISS = ZLISS * (600._JPRB / ZSCHMIDT)**0.5_JPRB
    ENDIF

! - scaling down if sea-ice covers from 60% to 100%
    ZSIVSOC=PFSICE(JL)/PFOCEN(JL)
    IF (ZSIVSOC > 0.6_JPRB) THEN
      ZSCAL=ZSIVSOC-0.6_JPRB / 0.4_JPRB 
      ZLISS = ZLISS * ZSCAL
    ENDIF

! - DMS flux
    ZCDMS(JL)=MAX(0.2_JPRB,PCDMS(JL))         ! correction on input source file   
    PFLXDMS(JL)=ZLISS*ZCDMS(JL)               ! in nmol*cm dm-3 h-1
    PFLXDMS(JL)=PFLXDMS(JL)/3600._JPRB        ! in nmol*cm dm-3 s-1
    PFLXDMS(JL)=PFLXDMS(JL)*RNAVO*1.E-09_JPRB ! in molec*cm dm-3 s-1
    PFLXDMS(JL)=PFLXDMS(JL)*0.001_JPRB        ! in molec cm-2 s-1
      
  ENDIF

ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_DMSO',1,ZHOOK_HANDLE)
END SUBROUTINE AER_DMSO

