SUBROUTINE AER_PHY2 &
 &( KIDIA, KFDIA, KLON , KTDIA  , KLEV , KSTART , KSTEP, KTILES, &
 &  KTRAC, KAERO, KSW  , &
 &  PRS1 , PRSF1, PTP  , PVERVEL, PCEN , &
 &  PALBF, PALBP, PALUVP, &
 &  PAERDEP,PAERLTS,PAERSCC     , PAERWS,PAERGUST, PAERUST,&
 &  PBCBF, PBCFF, PBCGF, POMBF  , POMFF , POMGF  , PSO2L  , PSO2H, PSOGF, PVOLC , PVOLE, PSOA, &
 &  PCI  , PFRTI, PLSM , PPHIS, PQP  , PSNS , &
 &  PTL  , PGELAM, PGEMU, PHSDFOR, &
 &  PUP  , PVP  , PWSA1, PTSPHY , &
!-- OUTPUTS
 &  PCFLX, PTENC, &
 &  PAERDDP, PAERSDM, PAERSRC, PAERMAP, PAERFLX, PAERLIF &
 &)

!**** *AER_PHY2* - ROUTINE DEALING WITH AEROSOL SOURCES, DRY DEPOSITION 
!                  AND SEDIMENTATION

!      JJ.MORCRETTE , ECMWF


!**   INTERFACE.
!     ----------
!          *AER_PHY2* IS CALLED FROM *CALLPAR*.

! INPUTS:
! -------
! PRS1 (KLON,0:KLEV)  : HALF-LEVEL PRESSURE           (Pa)
! PRSF1(KLON,KLEV)    : FULL-LEVEL PRESSURE           (Pa)
! PTP(KLON,KLEV)      : FULL-LEVEL TEMPERATURE (W. DYN.TEND.) (K)
! PVERVEL(KLON,KLEV)  : FULL-LEVEL VERTICAL VELOCITY  (Pa s-1)
! PCEN(KLON,KLEV,KTRAC): TRACER CONCENTRATION
! PALBF(KLON)         : SURFACE FORECAST ALBEDO
! PALUVP(KLON)        : MODIS ALBEDO UV-VIS. PARALLEL
! PCI(KLON)           : FRACTION OF SEA-ICE
! PFRTI(KLON,KTILES)  : FRACTION OF VARIOUS SURFACES (TILES)
! PLSM(KLON)          : LAND-SEA MASK
! PPHIS(KLON)         : SURFACE GEOPOTENTIAL
! PQP(KLON,KLEV)      : FULL-LEVEL HUMIDITY (W. DYN.TEND.) (kg kg-1)
! PSNS(KLON)          : SNOW DEPTH
! PTL(KLON)           : SURFACE TEMPERATURE
! PGELAM(KLON)        : LONGITUDE
! PGEMU(KLON)         : SINE OF LATITUDE
! PUP(KLON)           : LOWEST MODEL U-COMPONENT OF WIND (W. DYN.TEND.) 
! PVP(KLON)           : LOWEST MODEL V-COMPONENT OF WIND (W. DYN.TEND.) 
! PWSA1(KLON)         : MOISTURE IN TOP LAYER OF SURFACE
! PTSPHY              : PHYSICS TIME-STEP
!-- aerosol climatological fields
! PAERDEP, PAERLTS, PAERSCC, PBCBF, PBCFF, PBCGF, POMBF, POMFF, POMGF, PSO2L, PSO2H, PSOGF, PVOLC, PVOLE, PSOA
!-- dust aerosol predictors
! PAERWS              : Wind speed (average of horizontal wind speed)   (m/s)
! PAERGUST            : Wind gust (maximum 3 second gust in the hour)   (m/s)
! PAERUST             : Friction velocity 
! PHSDFOR             : Standard deviation of filtered sub-grid orography 

! INPUTS/OUTPUTS:
! ---------------
! PTENC(KLON,KLEV,KTRAC)    : TENDENCY OF TRACER CONCENTRATION
 

! OUTPUTS:
! --------
! PCFLX(KLON,KTRAC)      : SURFACE FLUX OF TRACERS              (xx m-2)
! PAERDDP(KLON,NACTAERO) : DRY DEPOSITION FLUX                  (xx m-2)
! PAERSDM(KLON,NACTAERO) : SEDIMENTATION FLUX                   (xx m-2)
! PAERSRC(KLON,NACTAERO) : SOURCE FLUX                          (xx m-2) 
! PAERMAP(KLON,2)
! PAERFLX(KLON,12,9)     : DIAGNOSTIC DUST SOURCE FLUXES
! PAERLIF(KLON,9)        : DIAGNOSTIC LIFTING THRESHOLD SPEED

!     EXTERNALS.
!     ----------
!          *AER_SRC*, *AER_DRYDEP*, *AER_SEDIMNT*

!     MODIFICATIONS.
!     -------------
!          Original JJ.Morcrette, 20060220

!     SWITCHES.
!     --------

!     MODEL PARAMETERS
!     ----------------

!-----------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK,   DR_HOOK

USE YOEAERSRC, ONLY : NTYPAER
USE YOEAERATM, ONLY : LAERSEDIM,LAERSURF ,LAERDRYDP
USE YOM_YGFL , ONLY : NAERO, NACTAERO
USE YOMCST   , ONLY : RD, RG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA, KFDIA, KLON
INTEGER(KIND=JPIM),INTENT(IN) :: KTDIA, KLEV
INTEGER(KIND=JPIM),INTENT(IN) :: KSTART, KSTEP
INTEGER(KIND=JPIM),INTENT(IN) :: KSW
INTEGER(KIND=JPIM),INTENT(IN) :: KTILES 
INTEGER(KIND=JPIM),INTENT(IN) :: KTRAC
INTEGER(KIND=JPIM),INTENT(IN) :: KAERO(NAERO)

REAL(KIND=JPRB),INTENT(IN)    :: PRS1(KLON,0:KLEV), PRSF1(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PTP(KLON,KLEV)   , PVERVEL(KLON,KLEV)
!!!!!!!! PCEN is IN or INOUT
REAL(KIND=JPRB),INTENT(INOUT) :: PCEN(KLON,KLEV,KTRAC)

REAL(KIND=JPRB),INTENT(IN)    :: PALBF(KLON) , PALBP(KLON,KSW)
REAL(KIND=JPRB),INTENT(IN)    :: PALUVP(KLON)
REAL(KIND=JPRB),INTENT(IN)    :: PAERDEP(KLON), PAERLTS(KLON), PAERSCC(KLON)
REAL(KIND=JPRB),INTENT(IN)    :: PAERWS(KLON) , PAERGUST(KLON), PAERUST(KLON)
REAL(KIND=JPRB),INTENT(IN)    :: PBCBF(KLON), PBCFF(KLON), PBCGF(KLON), POMBF(KLON), POMFF(KLON), POMGF(KLON)
REAL(KIND=JPRB),INTENT(IN)    :: PSO2L(KLON), PSO2H(KLON), PSOGF(KLON), PVOLC(KLON), PVOLE(KLON), PSOA(KLON) 
REAL(KIND=JPRB),INTENT(IN)    :: PCI(KLON)  , PFRTI(KLON,KTILES), PLSM(KLON), PPHIS(KLON), PSNS(KLON)
REAL(KIND=JPRB),INTENT(IN)    :: PGELAM(KLON), PGEMU(KLON), PHSDFOR(KLON) 
REAL(KIND=JPRB),INTENT(IN)    :: PTL(KLON)  , PQP(KLON,KLEV), PUP(KLON)  , PVP(KLON)  , PWSA1(KLON)
REAL(KIND=JPRB),INTENT(IN)    :: PTSPHY

REAL(KIND=JPRB),INTENT(INOUT) :: PTENC(KLON,KLEV,KTRAC)
REAL(KIND=JPRB),INTENT(OUT)   :: PCFLX(KLON,KTRAC)
REAL(KIND=JPRB),INTENT(OUT)   :: PAERDDP(KLON,NACTAERO), PAERSDM(KLON,NACTAERO), PAERSRC(KLON,NACTAERO)
REAL(KIND=JPRB),INTENT(OUT)   :: PAERMAP(KLON,2)
REAL(KIND=JPRB),INTENT(OUT)   :: PAERFLX(KLON,12,9), PAERLIF(KLON,9)

!*   0.5    LOCAL VARIABLES
!           ---------------

INTEGER(KIND=JPIM) :: JAER,JK,JL
INTEGER(KIND=JPIM) :: IBINDD, IBINSS, IDRYDP, ISEDIM, ITRAER2
LOGICAL :: LLPRINT
REAL(KIND=JPRB) :: ZAER(KLON,KLEV)  , ZAEROP(KLON,KLEV,NACTAERO), ZTAERI(KLON,KLEV,NACTAERO)
REAL(KIND=JPRB) :: ZTAER(KLON,KLEV) , ZTAERO(KLON,KLEV)
REAL(KIND=JPRB) :: ZALT(KLON,0:KLEV), ZDP(KLON,KLEV), ZDZ(KLON,KLEV)
REAL(KIND=JPRB) :: ZFAER(KLON), ZFAERO(KLON,NACTAERO)
!!! REAL(KIND=JPRB), ALLOCATABLE :: ZAEDRYDP(:,:,:), ZFAEDRYDP(:,:)
REAL(KIND=JPRB) :: ZRHO(KLON,KLEV)
REAL(KIND=JPRB) :: ZWND(KLON)

#include "aer_src.intfb.h"
#include "aer_drydep.intfb.h"
#include "aer_sedimnt.intfb.h"

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_PHY2',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*         1.     PROGNOSTIC AEROSOLS - INITIAL COMPUTATIONS
!                 ------------------------------------------


IBINSS=NTYPAER(1)
IBINDD=NTYPAER(2)

ZAEROP(:,:,:) = 0.0_JPRB

LLPRINT=.FALSE.

ZAEROP(KIDIA:KFDIA,1:KLEV,1:NACTAERO) = PCEN(KIDIA:KFDIA,1:KLEV,KAERO(1):KAERO(NACTAERO))
ZTAERI(KIDIA:KFDIA,1:KLEV,1:NACTAERO) = PTENC(KIDIA:KFDIA,1:KLEV,KAERO(1):KAERO(NACTAERO))

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZRHO(JL,JK)=PRSF1(JL,JK)/(RD*PTP(JL,JK))
    ZDP(JL,JK)= PRS1(JL,JK) - PRS1(JL,JK-1)
    ZDZ(JL,JK)= ZDP(JL,JK) / (ZRHO(JL,JK)*RG)  
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  ZALT(JL,KLEV)=PPHIS(JL)/RG
ENDDO
DO JK=KLEV-1,0,-1
  DO JL=KIDIA,KFDIA
    ZALT(JL,JK)=ZALT(JL,JK+1)+ZDZ(JL,JK+1)
  ENDDO
ENDDO



! NB: The physical calculations for the prognostic aerosols are done in a 
!     sequential mode, with any given process adding its tendency to the 
!     initial tendency (giving ZTENC), the field itself (ZCEN) remaining 
!     the same.
 
! NB: for multi-bins, 1-3 is sea-salt, 4-6 is desert dust

! define a composite index for each bin of each different aerosol type to be used
! in source, sedimentation and deposition routines

! NB: now done in SU_AERW 


ITRAER2=IBINSS+IBINDD

IF (LAERSURF) THEN

!*         2.     SURFACE FLUXES OF AEROSOLS
!                 --------------------------

! N.B: COMPUTATIONS USE SURFACE PRECIPITATION AT T-DT

  DO JL=KIDIA,KFDIA
    ZWND(JL)=SQRT(PUP(JL)*PUP(JL)+PVP(JL)*PVP(JL))
  ENDDO  

!- surface (mainly) fluxes for different tropospheric aerosols

  CALL AER_SRC &
    &( KIDIA  , KFDIA , KLON , KTDIA, KLEV , KTILES, KSTART, KSTEP , KSW ,&
    &  KTRAC  , KAERO , IBINSS, IBINDD, & 
    &  PALBF  , PALBP  , PALUVP, &
    &  PAERDEP,PAERLTS, PAERSCC, PAERGUST, &
    &  PBCBF  , PBCFF , PBCGF, POMBF, POMFF, POMGF, PSO2L, PSO2H , PSOGF, PSOA ,&
    &  PRS1   , PRSF1  , PCI  , PGELAM, PGEMU, PFRTI, PHSDFOR, &
    &  PLSM   , PQP   , PSNS , PTP  , PTL  , &
    &  ZWND   , PWSA1 , &
    &  PAERFLX, PAERLIF, PAERMAP,PCFLX, PCEN , PTENC )

!  WRITE(NULERR,FMT='(''aer_phy2 coming_out_of_aer_src'')')
 
!-- VERY IMPORTANT
! sea salt and desert dust fluxes in kg m-2 s-1, thus:
! dynamics, vert.diff, convection see mixing ratio in kg kg-1 (PCEN, ZAEROP) 
! and corresponding tendencies ZTENC are in kg kg-1 s-1 

  DO JAER=1,NACTAERO
    DO JL=KIDIA,KFDIA
      PAERSRC(JL,JAER)=PCFLX(JL,KAERO(JAER))
    ENDDO
  ENDDO

!*         3.     DRY DEPOSITION INCLUDED AS MODIFICATION TO SURFACE FLUXES
!                 ---------------------------------------------------------

!  WRITE(NULERR,FMT='(''aer_phy2 before drydep'')')

  IF (LAERDRYDP) THEN

    IDRYDP = NTYPAER(1) + NTYPAER(2) + NTYPAER(3) + NTYPAER(4) + NTYPAER(5) + NTYPAER(6)

    PAERDDP(:,:) = 0.0_JPRB

    CALL AER_DRYDEP ( &
      & KIDIA , KFDIA, KLON , KLEV , IDRYDP, KSTEP , &
      & ZAEROP, PCI  , PCFLX(:,KAERO(1):KAERO(IDRYDP)), PLSM , ZRHO , &   
      & ZFAERO )

! sea salt and desert dust fluxes in kg m-2 s-1
    DO JAER=1,IDRYDP
      DO JL=KIDIA,KFDIA
        PAERDDP(JL,JAER)=ZFAERO(JL,JAER)-PCFLX(JL,KAERO(JAER))
        PCFLX(JL,KAERO(JAER))=ZFAERO(JL,JAER)
      ENDDO
    ENDDO
  ENDIF
          
ENDIF

!*         4.     SEDIMENTATION OF AEROSOLS (ALL, SMALL AND LARGE, PARTICLES)
!                 -----------------------------------------------------------

IF (LAERSEDIM) THEN

  PAERSDM(:,:) = 0.0_JPRB

!-- test development: only sea-salt and desert dust particles are assumed to sediment
!  ISEDIM = NTYPAER(1) + NTYPAER(2)
!-- production: all particles are assumed to sediment
  ISEDIM = NTYPAER(1) + NTYPAER(2) + NTYPAER(3) + NTYPAER(4) + NTYPAER(5)

  DO JAER=1,ISEDIM

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZAER(JL,JK)  =PCEN(JL,JK,KAERO(JAER))
        ZTAERO(JL,JK)=PTENC(JL,JK,KAERO(JAER))
      ENDDO
    ENDDO

!-- sedimentation of aerosols follows closely sedimentation of ice particles by A. Tompkins

    CALL AER_SEDIMNT &
      &( KIDIA , KFDIA , KLON , KLEV, JAER   , &
      &  PTSPHY, PTP   , PRSF1, PRS1, PVERVEL, & 
      &  ZAER  , ZTAERO, &
      &  ZTAER , ZFAER ) 

    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        PTENC(JL,JK,KAERO(JAER))=ZTAER(JL,JK)
      ENDDO
    ENDDO
    DO JL=KIDIA,KFDIA
      PAERSDM(JL,JAER)=ZFAER(JL)
    ENDDO

  ENDDO

ENDIF

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_PHY2',1,ZHOOK_HANDLE)
END SUBROUTINE AER_PHY2
