SUBROUTINE AER_SCAVIN &
 & ( KIDIA , KFDIA  , KLON , KLEV , KAER , KSTART, KSTEP , &
 &   PDP   , PDZ    , PRHO , PFLXR, PFLXS, PCLCOV, PCLWAT, PAERO, PTAERI, &
 &   PSCAV , PTSPHY, &
 &   PTAERO, PFAERO &
 & )

!*** * AER_SCAVIN* - IN-CLOUD SCAVENGING OF TRACERS


! N.B.: only SEA-SALT and DUST are presently catered for.
! =======================================================


!**   INTERFACE.
!     ----------
!          *AER_SCAVIN* IS CALLED FROM *CALLPAR*.

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*
!        FROM O. BOUCHER (LOA, 1998-03) in_scav

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2004-05-07
!        P.BECHTOLD 2009-04-22 Clean/optimise and rewrite to externalise 
!                              scavenging coefficient

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST    ,ONLY : RG, RNAVO

USE YOEAERATM ,ONLY : RMASSE

USE YOEAERSNK ,ONLY : RFRAER 
USE YOEAERSRC ,ONLY : NINDAER

USE YOMLUN   , ONLY : NULOUT
USE YOEDBUG   ,ONLY : KSTPDBG, NSTPDBG

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1  ARGUMENTS
!             ---------

INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA, KFDIA, KLON, KLEV, KAER, KSTART, KSTEP

REAL(KIND=JPRB),INTENT(IN)    :: PDP(KLON,KLEV)    , PDZ(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PCLCOV(KLON,KLEV) , PCLWAT(KLON,KLEV)  , PRHO(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PFLXR(KLON,KLEV+1), PFLXS(KLON,KLEV+1)
REAL(KIND=JPRB),INTENT(IN)    :: PAERO(KLON,KLEV)  , PTAERI(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN)    :: PSCAV
REAL(KIND=JPRB),INTENT(IN)    :: PTSPHY

REAL(KIND=JPRB),INTENT(OUT)   :: PFAERO(KLON), PTAERO(KLON,KLEV)

!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JK, JL, IBIN, ITYP

REAL(KIND=JPRB) :: ZAERI(KLON,KLEV)
REAL(KIND=JPRB) :: ZBETA, ZBETAI, ZBETAK, ZBETAR
REAL(KIND=JPRB) :: ZCST1, ZDP, ZDX, ZDZ, ZFRAC, ZFUNC, ZMASSE, ZRHO
REAL(KIND=JPRB) :: ZEPSQLIQ, ZEPSFLX, ZTSPHY, ZRG
LOGICAL :: LLPRINT

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_SCAVIN',0,ZHOOK_HANDLE)

!- PAERO  in kg cm-3
!  PDZ    in m
!  PFLXx  in kg m-2 s-1
!  PCLWAT  in kg kg-1
!  PRHO   in kg m-3
!  PSCAV  a fraction N.D.
!  PTAERI in kg m-3 s-1
!  PTAERO in KG m-3 s-1


! NB: PCLWAT is the in-cloud water mixing ratio

ZRG=1.0_JPRB/RG
ZTSPHY=1.0_JPRB/PTSPHY

LLPRINT=.FALSE.
!DO JL=1,NSTPDBG
!  IF (KSTEP == KSTPDBG(JL)) THEN
!    LLPRINT=.TRUE.
!  ENDIF
!ENDDO

ZEPSFLX =1.E-18_JPRB
ZEPSQLIQ=1.E-18_JPRB
ZMASSE=RMASSE(KAER)
ZCST1=ZMASSE*RNAVO/PTSPHY
ZFRAC=RFRAER

DO JL=KIDIA,KFDIA
  PFAERO(JL)=0._JPRB
ENDDO

ITYP=NINDAER(KAER)/10
IBIN=NINDAER(KAER)-ITYP*10

IF (LLPRINT) THEN
  WRITE(UNIT=NULOUT,FMT='(1x,''PSCAV0'',I3,3I8,F6.0)') KSTEP,NINDAER(KAER),ITYP,IBIN,PTSPHY
ENDIF

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA

!-- initialisation

    ZAERI(JL,JK) = PAERO(JL,JK)

    ZRHO=PRHO(JL,JK)                          ! in kg m-3
    ZDZ =PDZ(JL,JK)                           ! in m
    ZDP =PDP(JL,JK)                           ! in Pa (kg m-1 s-2) 

!-- scavenging
    ZBETAI=PFLXR(JL,JK+1)-PFLXR(JL,JK) + PFLXS(JL,JK+1)-PFLXS(JL,JK)   ! in kg m-2 s-1 (w/o cloud fraction effect)

    ZBETAK=( PFLXR(JL,JK+1)-PFLXR(JL,JK) + PFLXS(JL,JK+1)-PFLXS(JL,JK) ) / MAX (0.01_JPRB, PCLCOV(JL,JK) )  
                                                                       ! in kg m-2 s-1
    ZBETA=ZBETAK

!-- security added
    IF (PCLWAT(JL,JK) > ZEPSQLIQ ) THEN
      ZBETA=ZBETA/(ZDP*PCLWAT(JL,JK)*ZRG) ! in s-1
    ENDIF

    IF (LLPRINT .AND. ZBETA /= 0._JPRB .AND. PCLWAT(JL,JK) > ZEPSQLIQ ) THEN
      WRITE(UNIT=NULOUT,FMT='(1x,''PSCAV1'',5I3,(8E10.3))') KSTEP,JL,JK,ITYP,IBIN,&
        & ZBETAK,ZBETA,PSCAV,ZDZ,ZRHO,PCLCOV(JL,JK),PCLWAT(JL,JK)
    ENDIF

    ZBETA=MIN( 200._JPRB, MAX( ZBETA, 0._JPRB ))
    ZFUNC=EXP(-PSCAV*ZBETA*PTSPHY)                           ! N.D.       (N.D.)

!    ZDX = ZAERI(JL,JK)*(ZFUNC - 1._JPRB)                    ! in kg kg-1

    ZDX = ZAERI(JL,JK)*(ZFUNC - 1._JPRB) * PCLCOV(JL,JK)      ! in kg kg-1

    PTAERO(JL,JK) = PTAERI(JL,JK) + ZDX*ZTSPHY                ! in kg kg-1 s-1

!-- factor ZMASSE/RNAVO =1.

    PFAERO(JL) = PFAERO(JL) - ZDX*ZTSPHY * ZDP*ZRG             ! in kg m-2 s-1
    
    IF (LLPRINT .AND. ZBETA > 0._JPRB) THEN
      WRITE(UNIT=NULOUT,FMT='(1x,''PSCAV2'',5I3,(9E10.3))') &
      &   KSTEP,JL,JK,ITYP,IBIN,PAERO(JL,JK),PTAERI(JL,JK),PTAERO(JL,JK),ZBETA,ZFUNC,ZDX,PFAERO(JL)
    ENDIF

!-- reevaporation  (NB: this uses ZBETAI, i.e., independent of the cloud fraction)
!-- security on fluxes added
    IF (ZBETAI < 0._JPRB .AND. PFLXR(JL,JK)+PFLXS(JL,JK) > ZEPSFLX) THEN
      ZBETAR=ZBETAI/(PFLXR(JL,JK)+PFLXS(JL,JK))
    ELSE
      ZBETAR=ZBETAI
    ENDIF

    IF ( PFLXR(JL,JK+1)+PFLXS(JL,JK+1) <= ZEPSFLX) THEN
!-- total reevaporation
      ZBETAR=MIN(MAX(0._JPRB, -ZBETAR), 1._JPRB)              ! N.D.
    ELSE
!-- non total reevaporation for the aerosols
      ZBETAR=MIN(MAX(0._JPRB, -ZBETAR)*ZFRAC, 1._JPRB)        ! N.D.
    ENDIF
!-- factor ZMASSE/RNAVO =1.

    ZDX = ZBETAR*PFAERO(JL) * RG/ZDP 
    PTAERO(JL,JK) = PTAERO(JL,JK) + ZDX
    PFAERO(JL) = (1._JPRB-ZBETAR) * PFAERO(JL)

    IF (LLPRINT .AND. ZBETAR > 0._JPRB) THEN
      WRITE(UNIT=NULOUT,FMT='(1x,''PSCAV3'',5I3,(11E10.3))') KSTEP,JL,JK,ITYP,IBIN,&
       & PAERO(JL,JK),ZAERI(JL,JK),PTAERI(JL,JK),PTAERO(JL,JK),PFAERO(JL),ZBETAR,ZDX
    ENDIF
  ENDDO
ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_SCAVIN',1,ZHOOK_HANDLE)
END SUBROUTINE AER_SCAVIN
