SUBROUTINE AER_SDUST &
  &( KIDIA , KFDIA , KLON , KLEV  , KBINDD, &
  &  PSCC  , PDEP  , PLTS , &
  &  PLSM  , PPRECS, PT   , PTOTPR, PTSTEP, PWIND, &
  &  PFLXDD &
  & ) 

!*** * AER_SDUST* - SOURCE TERMS FOR SAND DUST AEROSOLS

!**   INTERFACE.
!     ----------
!          *AER_SDUST* IS CALLED FROM *AER_SRC*.

!* INPUTS:
!  -------
! PSCC   (KLON)        : Soil clay content                   (%)
! PDEP   (KLON)        : Dust emission potential             (kg s2 m-5)
! PLTS   (KLON)        : Lifting threshold speed             (m s-1)
! PLSM   (KLON)        : Land-sea mask
! PPRECS (KLON)        : Precip in soil
! PT     (KLON,KLEV)   : Temperature
! PTOTPR (KLON)        : Surface flux of total precipitation (kg m-2 s-1)
! PTSTEP               : Time-step
! PWIND  (KLON)        : Wind strength at 10 m (m s-1)
!-- old stuff
! PFRLS  (KLON)        : Rain flux large-scale at surface
! PFSLS  (KLON)        : Snow flux large-scale at surface
! PFRMF  (KLON)        : Rain flux mass-flux at surface
! PFSMF  (KLON)        : Snow flux mass-flux at surface

!* OUTPUTS:
!  --------
! PFLXDD (KLON)        : Flux of mineral dust (kg m-2 s-1)

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*
!        FROM M.SCHULZ's dustsource (2001-02-01) 

!     SOURCE.
!     -------

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2004-05-12
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST    ,ONLY : RTT

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KBINDD 


REAL(KIND=JPRB)   ,INTENT(IN)    :: PSCC(KLON), PDEP(KLON), PLTS(KLON) 
!REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRLS(KLON), PFSLS(KLON)
!REAL(KIND=JPRB)   ,INTENT(IN)    :: PFRMF(KLON), PFSMF(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTOTPR(KLON), PWIND(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTEP

REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPRECS(KLON)

REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLXDD(KLON,KBINDD)


!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JBIN, JL
REAL(KIND=JPRB) :: ZAVDRYRATE, ZCLAYFAC, ZDRYING, ZFCTR, ZEPSIDD

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_SDUST',0,ZHOOK_HANDLE)

ZAVDRYRATE=300._JPRB/365._JPRB*PTSTEP/86400._JPRB
ZFCTR=1._JPRB
ZEPSIDD=1.E-06_JPRB

DO JBIN=1,KBINDD
  DO JL=KIDIA,KFDIA
    PFLXDD(JL,JBIN) = 0._JPRB

! All following quantities in mm except PFLXDD in kg m-2 s-1

    IF (PSCC(JL) < 9990._JPRB .AND. PLTS(JL) < 9990._JPRB) THEN
      PPRECS(JL) = PPRECS(JL) + PTSTEP * PTOTPR(JL) * ZFCTR

! ZCLAYFAC: maximum amount of water hold in top soil layer (mm)
      ZCLAYFAC=MIN(16._JPRB, PSCC(JL)*0.4_JPRB + 8._JPRB)  
! ZDRYING: drying factor (mm)         
      ZDRYING=ZAVDRYRATE*EXP(0.03905491_JPRB*EXP(0.17446_JPRB*(PT(JL,KLEV)-RTT)))
! PPRECS: precipitation in soil (mm)
      PPRECS(JL)=MIN(MAX(0._JPRB,PPRECS(JL)-ZDRYING),ZCLAYFAC)
  
      IF (PPRECS(JL) < 1.E-08_JPRB) THEN
        PFLXDD(JL,JBIN) = MAX( PDEP(JL)*(PWIND(JL)-PLTS(JL)) *PWIND(JL)*PWIND(JL), 0._JPRB)
        PFLXDD(JL,JBIN) = PFLXDD(JL,JBIN)*PLSM(JL)
      ENDIF

    ENDIF
  ENDDO
ENDDO
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_SDUST',1,ZHOOK_HANDLE)
END SUBROUTINE AER_SDUST

