SUBROUTINE AER_SEDIMNT &
 !---input
 & ( KIDIA , KFDIA   , KLON, KLEV, KAER   , &
 &   PTSPHY, PT      , PAP , PAPH, PVERVEL, &
 !---prognostic fields
 &   PAERO , PTAERI , &
 !---output
 &   PTAERO, PFLUXAER )  

!**** *AER_SEDIMNT* -  ROUTINE FOR PARAMETRIZATION OF AEROSOL SEDIMENTATION

!      Olivier Boucher & Jean-Jacques Morcrette 
!      following the ice sedimentation scheme of Adrian Tompkins

!**   INTERFACE.
!     ----------
!          *AER_SEDIMNT* IS CALLED FROM *CALLPAR*.

! INPUTS:
! -------
! PTSPHY                : TIMESTEP                  (s)
! PAP     (KLON,KLEV)   : LEVEL PRESSURE            (Pa)
! PAPH    (KLON,KLEV+1) : HALF-LEVEL PRESSURE       (Pa)
! PT      (KLON,KLEV)   : LEVEL TEMPERATURE         (K)#
! PTAERI   (KLON,KLEV)  : INPUT TENDENCY            (xx kg-1 s-1)
! PVERVEL (KLON,KLEV)   : VERTICAL VELOCITY         (Pa s-1)
! PAERO    (KLON,KLEV)  : CONCENTRATION OF TRACERS  (xx kg-1)

! OUTPUTS:
! --------
! PTAERO (KLON,KLEV)   : TENDENCY                  (xx kg-1 s-1)
! PFLUXAER(KLON)        : SURFACE FLUX              (xx m-2 s-1)

!     EXTERNALS.
!     ----------
!          NONE

!     MODIFICATIONS.
!     -------------

!     SWITCHES.
!     --------

!     MODEL PARAMETERS
!     ----------------

!-----------------------------------------------------------------------
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEAERSRC ,ONLY : NINDAER
!USE YOEAERSNK ,ONLY : RVSEDOCE, RVSEDSIC, RVSEDLND, RVSEDLIC
USE YOEAERSNK ,ONLY : RVSEDOCE
USE YOMCST    ,ONLY : RG, RD

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1  ARGUMENTS
!             ---------

!---input fields
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KAER
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAERO(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAERI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVERVEL(KLON,KLEV)

!---output fields
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLUXAER(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTAERO(KLON,KLEV) 

!---local fields
REAL(KIND=JPRB) :: ZSEDFLX(KLON), ZAERONWM1(KLON), ZAERI(KLON,KLEV), &
       &           ZSOLAERS, ZSOLAERB, ZGDP, ZDTGDP, ZKK, ZVAER, & 
       &           ZRHO, ZAERONW, ZTAERO(KLON,KLEV)

INTEGER(KIND=JPIM) :: IBIN, ITYP
INTEGER(KIND=JPIM) :: JK, JL

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_SEDIMNT',0,ZHOOK_HANDLE)

!- PAERO     in unit of xx kg-1 (mixing ratio)
!- ZAERONW   in unit of xx kg-1
!- ZSOLAERS, ZSOLAERB in xx kg-1  as m s-2/DeltaP xx m-2 = m s-2/(kg m-1 s-2) xx m-2
!- ZSEDFLX   in unit of xx m-2
!- ZWAIR     in unit of m s-1  = -RD *PVERVEL*PT / (RG*PAP)       vertical speed
!- PVERVEL   in unit of Pa s-1                                    vertical velocity
!- ZRHO      in unit of kg m-3
!- ZVAER     in unit of m s-1     Stokes speed (balance between gravitation and friction)


!--constants
ZKK= -RD/RG    ! in J kg-1 K-1 m-1 s2
!ZVAER         ! in m s-1

!-get the relevant sedimentation velocity
ITYP=NINDAER(KAER)/10
IBIN=NINDAER(KAER)-ITYP*10

! should in principle be a value, weighted averaged by fractions of sea, sea-ice, land
! ... but, at present, they are all the same.

IF (ITYP == 1) THEN
  ZVAER=RVSEDOCE(IBIN)
ELSEIF (ITYP == 2) THEN
  ZVAER=RVSEDOCE(3+IBIN)
ELSEIF (ITYP == 3) THEN
  ZVAER=RVSEDOCE(6+IBIN)
ELSEIF (ITYP == 4) THEN
  ZVAER=RVSEDOCE(8+IBIN)
ELSEIF (ITYP == 5) THEN
  ZVAER=RVSEDOCE(10+IBIN)
ELSEIF (ITYP == 6) THEN
  ZVAER=RVSEDOCE(12+IBIN)
ELSE 
  ZVAER=0._JPRB
ENDIF

!--initialisations of variables carried out from one layer to the next layer
!--actually not needed if (JK>1) test is on
DO JL=KIDIA,KFDIA 
  ZSEDFLX(JL)=0.0_JPRB
  ZAERONWM1(JL)=0.0_JPRB
ENDDO

DO JK=1, KLEV

  DO JL=KIDIA, KFDIA
!--initialisations
    ZSOLAERS=0.0
    ZSOLAERB=0.0
    ZGDP=RG/(PAPH(JL,JK+1)-PAPH(JL,JK))
    ZDTGDP=PTSPHY*ZGDP

    ZAERI(JL,JK) = PAERO(JL,JK)

! source from above
    IF (JK>1) THEN 
      ZSEDFLX(JL)=ZSEDFLX(JL)*ZAERONWM1(JL)  
      ZSOLAERS=ZSOLAERS+ZSEDFLX(JL)*ZDTGDP
    ENDIF

! sink to next layer
    ZRHO=PAP(JL,JK)/(RD*PT(JL,JK))
! ZWAIR is W =-RD/RG * PT * PVERVEL / PAP in m s-1 from PVERVEL in Pa s-1
!    ZWAIR=ZKK*PT(JL,JK)*PVERVEL(JL,JK)/PAP(JL,JK)
!    ZSEDFLX(JL)=ZWAIR*ZRHO*PAERO(JL,JK)

    ZSEDFLX(JL)=ZVAER*ZRHO
    ZSOLAERB=ZSOLAERB+ZDTGDP*ZSEDFLX(JL)

!---implicit solver
    ZAERONW=(ZAERI(JL,JK)+ZSOLAERS)/(1.0_JPRB+ZSOLAERB)

!---new time-step AER variable needed for next layer
    ZAERONWM1(JL)=ZAERONW

!---tendency in unit of xx kg-1 s-1
    ZTAERO(JL,JK)=(ZAERONW-ZAERI(JL,JK))/PTSPHY

!-NB: tendencies from sedimentation is added to original tendencies
    PTAERO(JL,JK)=ZTAERO(JL,JK)+PTAERI(JL,JK)
!    PTAERO(JL,JK)=ZTAERO(JL,JK)
  ENDDO
ENDDO

!---sedimentation flux to the surface
!---ZAERONWM1 now contains the surface concentration at the new timestep
!---PFLUXAER in unit of xx m-2 s-1 
DO JL=KIDIA,KFDIA 
  ZRHO=PAP(JL,KLEV)/(RD*PT(JL,KLEV))

!  ZWAIR=ZKK*PT(JL,KLEV)*PVERVEL(JL,KLEV)/PAP(JL,KLEV)
!  PFLUXAER(JL)=ZRHO*ZAERONWM1(JL)*ZWAIR

  PFLUXAER(JL)=ZRHO*ZAERONWM1(JL)*ZVAER
ENDDO

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_SEDIMNT',1,ZHOOK_HANDLE)
END SUBROUTINE AER_SEDIMNT

