SUBROUTINE AER_STRAERO &
  &( KIDIA  , KFDIA  , KLON  , KLAT, KLEV, &
  &  PH2SO4 , PQ     , PT    , PAP , &
  &  PRSTDEV, PRMODRA, PRNLOG, PR2SO4, PDENSO4, &
  &  PVSHS4A, PCONHS4 ) 
  
!**** *AER_STRAERO*   - MAIN ROUTINE FOR PROGNOSTIC STRATOSPHERIC AEROSOLS

!    PURPOSE.
!    --------

!**  INTERFACE.
!    ----------
!         *AER_STRAERO* IS CALLED FROM *AER_PHY3*

!**  AUTHOR.
!    -------
!         from S.BEKKI's *Strataero* 

!     MODIFICATIONS.
!     -------------
!         Original: JJ.Morcrette, ECMWF, 20080423  rewritten for IFS

!**  ARGUMENTS:     
!    ----------
!-- Inputs
!   ------
! PH2SO4   (KLON,KLEV) : Total in volume mixing ratio (VMR)
! PQ       (KLON,KLEV) : Water vapour in mass mixing ratio (MMR)

!-- Outputs
!   -------
! PRNLOG   (KLON,KLEV) : Total number of particles (VMR)
! PR2SO4   (KLON,KLEV) : Aerosol composition (in H2SO4 weight fraction in percent)
! PVSHS4A  (KLON,KLEV) : Mass-weighted mean sedimentation velocity (cm.sec-1)
! PCONHS4  (KLON,KLEV) : Fraction of condensed H2SO4 (H2SO4c / H2SO4total)


!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA, KFDIA, KLON, KLAT, KLEV
  
!-- INPUT variables

REAL(KIND=JPRB),INTENT(IN) :: PH2SO4(KLON,KLEV)
REAL(KIND=JPRB),INTENT(IN) :: PAP(KLON,KLEV), PT(KLON,KLEV), PQ(KLON,KLEV)

! climatological width and mode radius (in cm)
REAL(KIND=JPRB),INTENT(IN) :: PRSTDEV(KLAT,KLEV), PRMODRA(KLAT,KLEV)

!-- OUTPUT variables

REAL(KIND=JPRB),INTENT(OUT) :: PRNLOG(KLON,KLEV)
REAL(KIND=JPRB),INTENT(OUT) :: PR2SO4(KLON,KLEV)
REAL(KIND=JPRB),INTENT(OUT) :: PDENSO4(KLON,KLEV)
REAL(KIND=JPRB),INTENT(OUT) :: PVSHS4A(KLON,KLEV)
REAL(KIND=JPRB),INTENT(OUT) :: PCONHS4(KLON,KLEV)
 
!     local variables
REAL(KIND=JPRB) :: ZACTSO4(KLON,KLEV)
REAL(KIND=JPRB) :: ZCONCM(KLON,KLEV)
REAL(KIND=JPRB) :: ZFPATH(KLON,KLEV)
REAL(KIND=JPRB) :: ZPMB(KLON,KLEV), ZH2O(KLON,KLEV)


!INTEGER(KIND=JPIM) :: JI,JK,JL
INTEGER(KIND=JPIM) :: JK,JL

INTEGER(KIND=JPIM) :: ILAT, ILEV

#include "aer_strcomp.intfb.h"
#include "aer_straact.intfb.h"
#include "aer_strdens.intfb.h"
#include "aer_strclog.intfb.h"
#include "aer_strrlog.intfb.h"
#include "aer_strfree.intfb.h"
#include "aer_strvelo.intfb.h"

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_STRAERO',0,ZHOOK_HANDLE)

! H2SO4 mass-weighted mean sedimentation velocity (cm.sec-1)
!     Treat it as a vertical velocity on H2SO4total and use the 
!     vertical advection scheme. 

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
!  pressure in hPa 
    ZPMB(JL,JK)= PAP(JL,JK) / 100._JPRB
! H2O volume mixing ratio      
    ZH2O(JL,JK)= PQ(JL,JK)
!  total number density (molec.cm-3)
    ZCONCM(JL,JK)= (ZPMB(JL,JK)/1000._JPRB) / ( 1.38E-22_JPRB * PT(JL,JK) )
  ENDDO
ENDDO

!-- watch out: everything below is 2D lat-height

ILAT=1
ILEV=1

!     H2O, P, T -> aerosol composition (R2SO4) 
CALL AER_STRCOMP &
  &( ILAT  , ILEV, &
  &  ZH2O , PT   , ZPMB, &
  &  PR2SO4 )

!     R2SO4 -> activity coeff (ACTSO4)
CALL AER_STRAACT &
  &( ILAT, ILEV, &
  &  PR2SO4, &
  &  ZACTSO4 )

!     R2SO4, T -> aerosol density (R2SO4)
CALL AER_STRDENS &
  &( ILAT, ILEV, &
  &  PR2SO4 , PT   , &
  &  PDENSO4 )

!     H2SO4, Rm, Sigma -> No (total number of aerosol particles, RNLOG)
CALL AER_STRCLOG &
  &( ILAT   , ILEV, &
  &  PH2SO4, PR2SO4, PDENSO4, PRSTDEV, PRMODRA, PT, PRNLOG )

!     H2SO4, ACTSO4, RNLOG, T -> fraction of condensed H2SO4,new RNLOG
CALL AER_STRRLOG &
  &( ILAT  , ILEV, &
  &  ZACTSO4, PH2SO4 , PT    , ZCONCM, &
  &  PCONHS4, PRNLOG )
    
CALL AER_STRFREE &
  &( ILAT, ILEV, &
  &  ZPMB   , PT    , PRMODRA, &
  &  ZFPATH )

!-------------------------------------------------------------
!     new H2SO4 sedimentation velocity from analytical expression
!     T,FPATH,R2SO4,DENSO4,RNLOG,RSTDEV,RMODRA -> 
!          VSHS4A (sedimentation velocity on H2SO4total) cm/sec

CALL AER_STRVELO &
  &( ILAT  , ILEV   , &
  &  PT    , ZFPATH , &
  &  PR2SO4, PDENSO4, PCONHS4, PRNLOG, PRSTDEV, PRMODRA, &
  &  PVSHS4A )

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_STRAERO',1,ZHOOK_HANDLE)
END SUBROUTINE AER_STRAERO
