SUBROUTINE AER_STRFREE &
  &( KLAT, KLEV, &
  &  PPMB , PT   , PRMODRA, &
  &  PFPATH )

!*** *AER_STRFREE* - DERIVE FREE PATH AND KNUDSEN NUMBER

!     Molecular mean free path of air molecules.
!     Source: Prupacher & Klett:Microphysics of clouds and precipitation,
!                               (1980), 10-106, p.323.
!     Formula used:             FPLAIR=2.281238E-5*TAIR/PAIR
!     Input:  TAIR:     Air temperature (K)
!             PAIR:     Air pressure (Pa)
!     Output:           Molecular mean free path (m)
!      FPLAIR=(2.281238E-5)*TAIR/PAIR
!     1.E2 (m to cm), 1.E5 for Pa (4 because of mean of T and )

!     AUTHOR.
!     -------
!        FROM S.BEKKI's *freepath*
!        JJMorcrette  ECMWF 20080423 rewritten for IFS

! INPUTS
! ------
!   PT     : temperature (K)
!   PPMB   : pressure (mb)
!   PRMODRA: mode radius (cm)
!   KLAT   : number of latitude bands in the model domain
!   KLEV   : number of altitude bands in the model domain
!   NBINS  : number of size bins
!   the first (IK=1) and last bin (IK=NBINS) are not used
!   for IFS: perhaps add another dimension for longitude

! OUTPUTS
! ------- 
!   PFPATH: air mean free path: pressure (cm)
!   PKNUD: Knudsen number(adim) 

!------------------------------------------------------------------------------
 
USE PARKIND1  ,ONLY : JPRB , JPIM
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE
   
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

INTEGER(KIND=JPIM),INTENT(IN) :: KLAT,KLEV

REAL(KIND=JPRB),INTENT(IN) :: PT(KLAT,KLEV), PPMB(KLAT,KLEV)
REAL(KIND=JPRB),INTENT(IN) :: PRMODRA (KLAT,KLEV)

REAL(KIND=JPRB),INTENT(OUT) :: PFPATH (KLAT,KLEV) 

!*       0.5   LOCAL VARIABLES
!              ---------------

INTEGER(KIND=JPIM) :: JI,JK
 
REAL(KIND=JPRB) :: ZCST, ZCST1, ZRKNUD

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_STRFREE',0,ZHOOK_HANDLE)

ZCST=1.E2_JPRB * 2.281238E-5_JPRB * 1.E3_JPRB / (1.E5_JPRB * 4._JPRB)

!     Reduction factor of mean free path for sulfuric acid vapor 
!     with respect to air (0.696E0)
ZCST1=0.696_JPRB * 1.E2_JPRB * 2.281238E-5_JPRB * 1.E3_JPRB / (1.E5_JPRB * 4._JPRB)

DO JI=1,KLAT
  DO JK=2,KLEV

!    AIR MEAN FREE PATH(ATKINS AND WAYNE) (CM)
!    SENFELD
!    FPATH=2.0/( PI*SQRT(2.0)*SIGAIR*(M(I,J)+M(I,J-1)) )
!    ATKINS
!    FPATH=2.0/( SQRT(2.0)*SIGAIR*(M(I,J)+M(I,J-1)) )

! Prupacher & Klett (p323)
    PFPATH(JI,JK)=ZCST*(PT(JI,JK)+PT(JI,JK-1))/(PPMB(JI,JK)+PPMB(JI,JK-1))
!   PFPATH(JI,JK)=1.E10

!    PKNUD(JI,JK)=PFPATH(JI,JK)/PRMODRA(JI,JK)

!     ZRKNUD=0.696*KNUD(I,J)
     ZRKNUD=0.696_JPRB*PFPATH(JI,JK)/PRMODRA(JI,JK)

!    CONDENSATION
!     SENFELD
!        FL=FL1*RRSI(IK)*RRSI(IK)*( 1.0 +RKNUD )
!     *     /( 1.0 +ALPHA/(2.0*RKNUD) +RKNUD )
!        explain: Jc=Jk*(4*D)/(alpha*vtk*R) p601, Seinfeld
!        since FPATH(h2SO4)=2*D/vtk for dahneke p604, S
!        INTERESTING: Jc=Jk*2*FPATH(h2SO4)/(ALPHA*R)
!                     Jc=Jk*2*Kn(H2SO4)/alpha 
!        since FPATH(h2SO4)=2*D/vtk for dahneke p604, S
!        D(h2SO4)=FPATH(h2SO4)*vtk(h2SO4)/2
!        so Jc=Jk*(4*FPATH*vtk/2)/(alpha*vtk*R)
!        Jc=Jk*2*FPATH/(alpha*R)=Jk*2*Kn(h2so4)/alpha
!        J=Jc*(1+Kn)/( 1+2Kn*((1+Kn)/alpha) )
!        J=Jk*(1+Kn)/( (alpha/2*Kn)* (1+2Kn*((1+Kn)/alpha)) )
!        J=Jk*(1+Kn)/( (alpha/2*Kn) + 1+Kn )
!        as Jk=Jc / (2*Kn/alpha)
!        j=Jc*(1+Kn)/( (2.*Kn/alpha)*((alpha/2*Kn) + 1+Kn) )
 
!        conkin=1 (Kn>>1), flux prop to R*R
!        CONKIN(I,J)=( 1.0 +ZRKNUD )
!     *     /( 1.0 +1./(2.0*ZRKNUD) +ZRKNUD )
!       concon=1 (Kn<<1), flux prop to R
!        CONCON(I,J)=( 1.0 +ZRKNUD )
!     *     /( 2.*ZRKNUD* (1.0 +1./(2.0*ZRKNUD) +ZRKNUD) )
   ENDDO
ENDDO
 
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('AER_STRFREE',1,ZHOOK_HANDLE)
END SUBROUTINE AER_STRFREE
