SUBROUTINE CLDPPAD ( KIDIA , KFDIA  , KLON   , KLEV,&
 & PA5    , PL5    , PI5,&
 & PCLC5  , PQIWP5 , PQLWP5,&
 & PA     , PL     , PI,&
 & PCLC   , PQIWP  , PQLWP  )  

!**** *CLDPPAD*  - COMPUTES CLOUD COVER AND LIQUID WATER PATH
!                  (adjoint)

!     PURPOSE.
!     --------
!         THIS ROUTINE COMPUTES LIQUID WATER PATH, CLOUD WATER AND 
!         CLOUD ICE CONTENTS REQUIRED BY THE RADIATION SCHEME FOR 
!         THE COMPUTATION OF THE FLUXES. INPUT ARE THE LIQUID WATER 
!         CONTENT AND CLOUD  FRACTION CALCULATED IN CLOUDSTAD IN 
!         THE PREVIOUS TIME STEP

!**   INTERFACE.
!     ----------
!              *CLDPPTL* IS CALLED FROM *CALLPARAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS

!     INPUT PARAMETERS (REAL)

!    *PAPRSM15*  PRESSURE ON HALF-LEVELS (T-1)               (Trajectory)
!    *PA5*       FRACTIONAL CLOUD COVER (T-1)                (Trajectory)
!    *PL5*       LIQUID WATER CONTENT (T-1) (IN GRID VOLUME) (Trajectory)
!    *PI5*       ICE WATER CONTENT (T-1) (IN GRID VOLUME)    (Trajectory)
!    *PAPRSM1*   PRESSURE ON HALF-LEVELS (T-1)                  PA
!    *PA*        FRACTIONAL CLOUD COVER (T-1)
!    *PL*        LIQUID WATER CONTENT (T-1) (IN GRID VOLUME)   KG/KG
!    *PI*        ICE WATER CONTENT (T-1) (IN GRID VOLUME)      KG/KG

!     OUTPUT PARAMETERS (REAL)

!     FOR RADIATION:

!    *PCLC5*     FRACTIONAL CLOUD COVER                  (Trajectory)  
!    *PQIWP5*    CLOUD ICE CONTENT                       (Trajectory)       
!    *PQLWP5*    CLOUD WATER CONTENT                     (Trajectory)      
!    *PCLC*      FRACTIONAL CLOUD COVER
!    *PQIWP*     CLOUD ICE CONTENT                             KG/KG
!    *PQLWP*     CLOUD WATER CONTENT                           KG/KG

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!     AUTHOR.
!     -------
!     M. Janiskova         *ECMWF*       2002-11-01

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOERDI   , ONLY : REPCLC

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PA5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PL5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLC5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQIWP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQLWP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PA(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQIWP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQLWP(KLON,KLEV) 
!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB) :: ZPA5(KLON,KLEV)

INTEGER(KIND=JPIM) :: JK, JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CLDPPAD',0,ZHOOK_HANDLE)
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF (PA5(JL,JK) <= REPCLC) THEN
      ZPA5(JL,JK) = REPCLC
    ELSE
      ZPA5(JL,JK) = PA5(JL,JK)
    ENDIF
    IF (ZPA5(JL,JK) > (1.0_JPRB-REPCLC)) THEN
      PCLC5(JL,JK) = 1.0_JPRB-REPCLC
    ELSE
      PCLC5(JL,JK) = ZPA5(JL,JK)
    ENDIF
  ENDDO
ENDDO

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF(PCLC5(JL,JK) > REPCLC) THEN
      PQLWP5(JL,JK) = PL5(JL,JK)
      PQIWP5(JL,JK) = PI5(JL,JK)
    ELSE
      PQLWP5(JL,JK) = 0.0_JPRB
      PQIWP5(JL,JK) = 0.0_JPRB
    ENDIF
  ENDDO
ENDDO

! ------------------------------------------------------------------

!                 ----- ADJOINT COMPUTATION ------

!     ------------------------------------------------------------------

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF(PCLC5(JL,JK) > REPCLC) THEN
      PI (JL,JK) = PI (JL,JK) + PQIWP (JL,JK)
      PL (JL,JK) = PL (JL,JK) + PQLWP (JL,JK)
    ENDIF
    PQIWP (JL,JK) = 0.0_JPRB
    PQLWP (JL,JK) = 0.0_JPRB
  ENDDO
ENDDO

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF (ZPA5(JL,JK) <= (1.0_JPRB-REPCLC)) THEN
      PA (JL,JK) = PA (JL,JK) + PCLC (JL,JK)
    ENDIF
    PCLC (JL,JK) = 0.0_JPRB

    IF (PA5(JL,JK) <= REPCLC) THEN
      PA (JL,JK) = 0.0_JPRB
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CLDPPAD',1,ZHOOK_HANDLE)
END SUBROUTINE CLDPPAD
