!OPTIONS XOPT(HSFUN)
#ifdef RS6K
@PROCESS HOT(NOVECTOR) NOSTRICT
#endif
SUBROUTINE CLOUDSC &
 !---input
 & (KIDIA,    KFDIA,    KLON,    KLEV,&
 & PTSPHY,&
 & PGEOH, PGELAM, PGELAT,&
 & PTORIG, PQORIG, PT, PQ, PTENT, PTENQ,&
 & PVFA, PVFL, PVFI, PDYNA, PDYNL, PDYNI, &
 & PHRSW,    PHRLW,&
 & PVERVEL,  PAP,      PAPH,&
 & PLSM,     LDCUM,    KTYPE, &
 & PLU,      PLUDE,    PMFU,     PMFD,&
 & LDSLPHY,  LDMAINCALL, &
! & PVFL,     PVFI, &
 !---prognostic fields
 & PAORIG,&
 & PL,       PI,       PA,&
 & PCLV,     PTENCLV, &
 & PTENL,    PTENI,    PTENA,&
 & PSUPSAT,&
!-- arrays for aerosol-cloud interactions
!!! & PQAER,    KAER, &
 !>>dod removed PLCRIT_AER, PICRIT_AER, renamed PCCN to PCDNC, renamed PNICE TO PICNC
 !! & PLCRIT_AER,PICRIT_AER,&
 & PRE_ICE,&
 & PCDNC,     PICNC,&
 !<<dod
 !---diagnostic output
 & PCOVPTOT,&
 !---resulting fluxes
 & PFSQLF,   PFSQIF ,  PFCQNNG,  PFCQLNG,&
 & PFPLSL,   PFPLSN,   PFHPSL,   PFHPSN,&
 & PEXTRA,   KFLDX)  

!===============================================================================
!**** *CLOUDSC* -  ROUTINE FOR PARAMATERIZATION OF CLOUD PROCESSES
!                  FOR PROGNOSTIC CLOUD SCHEME
! 
!     M.TIEDTKE      E.C.M.W.F.     8/1988, 2/1990
! 
!     PURPOSE
!     -------
! 
!          THIS ROUTINE UPDATES THE CONV/STRAT CLOUD FIELDS.
!          THE FOLLOWING PROCESSES ARE CONSIDERED:
! 
!      (1) Detrainment of cloud water from convective updrafts
!      (2) Evaporation/condensation of cloud water in connection
!           with heating/cooling such as by subsidence/ascent
!      (3) Erosion of clouds by turbulent mixing of cloud air
!           with unsaturated environmental air
!      (4) Turbulent transports of s,q,u,v at cloud tops due to
!           buoyancy fluxes and lw radiative cooling
!      (5) Conversion of cloud water into precipitation
!      (6) Evaporation of rain
!      (7) Melting of snow
!
!      Note: Formation of clouds at top of convective boundary layers
!            is now treated in the VDF scheme
!
!**   INTERFACE.
!     ----------
! 
!          *CLOUDSC* IS CALLED FROM *CALLPAR*
!     THE ROUTINE TAKES ITS INPUT FROM THE LONG-TERM STORAGE:
!     T,Q,L,PHI AND DETRAINMENT OF CLOUD WATER FROM THE
!     CONVECTIVE CLOUDS (MASSFLUX CONVECTION SCHEME), BOUNDARY
!     LAYER TURBULENT FLUXES OF HEAT AND MOISTURE, RADIATIVE FLUXES,
!     OMEGA.
!     IT RETURNS ITS OUTPUT TO:
!      1.MODIFIED TENDENCIES OF MODEL VARIABLES T AND Q
!        AS WELL AS CLOUD VARIABLES L AND C
!      2.GENERATES PRECIPITATION FLUXES FROM STRATIFORM CLOUDS
! 
!     EXTERNALS.
!     ----------
!          NONE
! 
!     MODIFICATIONS.
!     -------------
!     CH. JAKOB      E.C.M.W.F.     2/1994 IMPLEMENTATION INTO IFS
! 
!     A.TOMPKINS     E.C.M.W.F.     2002   NEW NUMERICS
!        01-05-22 : D.Salmond   Safety modifications
!        02-05-29 : D.Salmond   Optimisation
!        03-01-13 : J.Hague     MASS Vector Functions  J.Hague
!        03-10-01 : M.Hamrud    Cleaning
!        04-12-14 : A.Tompkins  New implicit solver and physics changes
!        04-12-03 : A.Tompkins & M.Ko"hler  moist PBL
!     G.Mozdzynski  09-Jan-2006  EXP security fix
!        19-01-09 : P.Bechtold  Changed increased RCLDIFF value for KTYPE=2
!        07-07-01 : A.Tompkins/R.Forbes  4-Phase flexible microphysics
!
!     SWITCHES.
!     --------
! 
!          LCLOUD=.TRUE.   PREDICTION OF CONV/STRAT CLOUD-FIELDS
! 
!     MODEL PARAMETERS
!     ----------------
! 
!     RCLDIFF:    PARAMETER FOR EROSION OF CLOUDS
!     RCLCRIT:    THRESHOLD VALUE FOR RAIN AUTOCONVERSION
!     RLCRITSNOW: THRESHOLD VALUE FOR SNOW AUTOCONVERSION
!     RKCONV:     PARAMETER FOR AUTOCONVERSION OF CLOUDS (KESSLER)
!     RCLDMAX:    MAXIMUM POSSIBLE CLW CONTENT (MASON,1971)
! 
!     REFERENCES.
!     ----------
! 
!     TIEDTKE MWR 1993
!     JAKOB PhD 2000
!     GREGORY ET AL. QJRMS 2000
!     TOMPKINS ET AL. QJRMS 2007
! 
! variables:
! PFSQLF : Flux of liquid for DDH enthalpy budget
! PFSQIF : Flux of ice for DDH enthalpy budget
!===============================================================================

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LSCMEC
USE YOMCT3   , ONLY : NSTEP
USE YOMCST   , ONLY :  RG       ,RD       ,RCPD     ,RETV     ,&
 & RLVTT    ,RLSTT    ,RLMLT    ,RTT      ,RV       ,RPI, RDAY
USE YOETHF   , ONLY :  R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RALFDCP  ,RTWAT    ,&
!& RTBER    ,&
 & RTICE    ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R,&
 & RKOOP1   ,RKOOP2
USE YOECLDP  , ONLY : RAMID     ,RCLDIFF  ,RKCONV   , RCLDIFFC,&
 & RPRC1    ,&
! & RPRC2   ,&
! & RCLDMAX,&  
 & RPECONS  ,RTAUMEL  ,&
 & RAMIN    ,RLMIN    ,RCOVPMIN ,&
 & RLCRITSNOW         ,RCLCRIT  ,RSNOWLIN1,RSNOWLIN2,&
 & RKOOPTAU ,NSSOPT   ,NCLDTOP ,RTHOMO , &
 & RVICE    ,RVRAIN   ,RVSNOW, &
 & RICEHI1  ,RICEHI2  ,RICEINIT ,RPRECRHMAX, &
 & NCLDQV   ,NCLDQL   ,NCLDQR   ,NCLDQI   ,NCLDQS   , NCLV  ,&
 & RNICE    ,RCCN     ,&
 & LAERLIQAUTOLSP, LAERLIQCOLL, LAERICESED, LAERICEAUTO, &    
 & LCLDBUDGET, &
 !>>dod
 & PPRHO_WAT, RLCRIT_UPHYS, PPRHO_ICE, PPRICE_CRIT, NCLOUDACT
 !<<dod
USE YOMJFH   , ONLY : N_VMASS

USE YOERAD   , ONLY : LMAC2SPACI
USE YOE_AER_ACTIV , ONLY : GET_CDNC_FACTOR

IMPLICIT NONE

!-------------------------------------------------------------------------------
!
!                 Declare input/output arguments
!
!-------------------------------------------------------------------------------
!
! PLCRIT_AER : critical liquid mmr for rain autoconversion process
! PICRIT_AER : critical liquid mmr for snow autoconversion process
! PRE_LIQ : liq Re
! PRE_ICE : ice Re
! PCCN    : liquid cloud condensation nuclei
! PICNC   : ice number concentration (cf. CCN)
!>>dod
!  deleted PLCRIT_AER, PICRIT_AER, renamed PCCN and PNICE
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRE_ICE(KLON,KLEV)  ! ice crystal effective radius
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCDNC(KLON,KLEV)    ! cloud droplet number concentration
REAL(KIND=JPRB)   ,INTENT(IN)    :: PICNC(KLON,KLEV)    ! ice crystal number concentration 
!<<dod

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON             ! Number of grid points
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV             ! Number of levels
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY            ! Physics timestep
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH(KLON,KLEV+1)! Geopotential on half levels
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(KLON)      ! LONGITUDE (RADIANS)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAT(KLON)      ! LATITUDE (RADIANS)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTORIG(KLON,KLEV) ! T at start of callpar
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQORIG(KLON,KLEV) ! Q at start of callpar
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV)    ! Temperature (K) f.guess
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV)    ! Spec. humid. (kg/kg) f.g.
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFA(KLON,KLEV)  ! CC from VDF scheme
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFL(KLON,KLEV)  ! Liq from VDF scheme
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFI(KLON,KLEV)  ! Ice from VDF scheme
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNA(KLON,KLEV) ! CC from Dynamics
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNL(KLON,KLEV) ! Liq from Dynamics
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNI(KLON,KLEV) ! Liq from Dynamics
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHRSW(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHRLW(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVERVEL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP(KLON,KLEV)   ! Pressure on full levels
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KLON,KLEV+1)! Pressure on half levels
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(KLON)       ! Land fraction (0-1) 
LOGICAL           ,INTENT(IN)    :: LDCUM(KLON)      ! Convection active
INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPE(KLON)      ! Convection type 0,1,2
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLU(KLON,KLEV)   ! Conv. condensate
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLUDE(KLON,KLEV) ! Conv. detrained water 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFU(KLON,KLEV)  ! Conv. mass flux up
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFD(KLON,KLEV)  ! Conv. mass flux down
LOGICAL           ,INTENT(IN)    :: LDSLPHY 
LOGICAL           ,INTENT(IN)    :: LDMAINCALL 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAORIG(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PL(KLON,KLEV)  ! Liquid water (kg/kg)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI(KLON,KLEV)  ! Ice water (kg/kg)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PA(KLON,KLEV)  ! Cloud fraction 

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDX 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEXTRA(KLON,KLEV,KFLDX) ! extra fields

REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLV(KLON,KLEV,NCLV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENCLV(KLON,KLEV,NCLV) 

REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENA(KLON,KLEV) 

 ! Supersat clipped at previous time level in SLTEND
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSUPSAT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOVPTOT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFSQLF(KLON,KLEV+1)  ! Flux diagnostics
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFSQIF(KLON,KLEV+1)  !    for DDH
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQNNG(KLON,KLEV+1) ! Flux diagnostics
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQLNG(KLON,KLEV+1) !    for DDH
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLSL(KLON,KLEV+1) ! liq+rain sedim flux
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLSN(KLON,KLEV+1) ! ice+snow sedim flux
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSL(KLON,KLEV+1) ! Enthalpy flux for liq
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSN(KLON,KLEV+1) ! Enthalp flux for ice

!-------------------------------------------------------------------------------
!
!                       Declare local variables
!
!-------------------------------------------------------------------------------

REAL(KIND=JPRB) :: &
!  condensation and evaporation terms
 & ZLCOND1(KLON), ZLCOND2(KLON),&
 & ZLEVAP,        ZLEROS,&
 & ZLEVAPL(KLON), ZLEVAPI(KLON),&
! autoconversion terms
 & ZRAINAUT(KLON), ZSNOWAUT(KLON), &
 & ZLIQCLD(KLON),  ZICECLD(KLON)
REAL(KIND=JPRB) :: ZFOKOOP(KLON), ZFOEALFA(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZICENUCLEI(KLON) ! number concentration of ice nuclei

REAL(KIND=JPRB) :: ZLICLD(KLON)
REAL(KIND=JPRB) :: ZACOND
REAL(KIND=JPRB) :: ZAEROS
REAL(KIND=JPRB) :: ZLFINALSUM(KLON)
REAL(KIND=JPRB) :: ZDQS(KLON)
REAL(KIND=JPRB) :: ZTOLD(KLON)
REAL(KIND=JPRB) :: ZQOLD(KLON)  
REAL(KIND=JPRB) :: ZDTGDP(KLON) 
REAL(KIND=JPRB) :: ZRDTGDP(KLON)  
REAL(KIND=JPRB) :: ZTRPAUS(KLON)
REAL(KIND=JPRB) :: ZCOVPCLR(KLON)   
REAL(KIND=JPRB) :: ZCOVPCLD(KLON)
REAL(KIND=JPRB) :: ZPRECLR
REAL(KIND=JPRB) :: ZCOVPTOT(KLON)    
REAL(KIND=JPRB) :: ZQPRETOT(KLON)
REAL(KIND=JPRB) :: ZDPEVAP
REAL(KIND=JPRB) :: ZDTFORC
REAL(KIND=JPRB) :: ZDTDIAB
REAL(KIND=JPRB) :: ZTP1(KLON,KLEV)   
REAL(KIND=JPRB) :: ZLDEFR(KLON)
REAL(KIND=JPRB) :: ZLDIFDT(KLON)
REAL(KIND=JPRB) :: ZDTGDPF(KLON)
REAL(KIND=JPRB) :: ZLCUST(KLON,NCLV)
REAL(KIND=JPRB) :: ZACUST(KLON)
REAL(KIND=JPRB) :: ZMF(KLON) 

REAL(KIND=JPRB) :: ZRHO(KLON)
REAL(KIND=JPRB) :: ZTMP1(KLON),ZTMP2(KLON),ZTMP3(KLON)
REAL(KIND=JPRB) :: ZTMP4(KLON),ZTMP5(KLON),ZTMP6(KLON),ZTMP7(KLON)
REAL(KIND=JPRB) :: ZALFAWM(KLON)

! Accumulators of A,B,and C factors for cloud equations
REAL(KIND=JPRB) :: ZSOLAB(KLON) ! -ve implicit CC
REAL(KIND=JPRB) :: ZSOLAC(KLON) ! linear CC
REAL(KIND=JPRB) :: ZANEW
REAL(KIND=JPRB) :: ZANEWM1(KLON) 

REAL(KIND=JPRB) :: ZGDP(KLON)

!---for flux calculation
REAL(KIND=JPRB) :: ZDA(KLON)
REAL(KIND=JPRB) :: ZLI(KLON,KLEV),           ZA(KLON,KLEV)
REAL(KIND=JPRB) :: ZAORIG(KLON,KLEV) ! start of scheme value for CC

LOGICAL ::  LLFLAG(KLON)
LOGICAL :: LLO1

INTEGER(KIND=JPIM) :: ICALL, IK, JK, JL, JM, JN, JO, JLEN, IS

REAL(KIND=JPRB) :: ZDP(KLON), ZPAPHD(KLON)

REAL(KIND=JPRB) :: ZALFA
! & ZALFACU, ZALFALS
REAL(KIND=JPRB) :: ZALFAW
REAL(KIND=JPRB) :: ZBETA,ZBETA1
REAL(KIND=JPRB) :: ZBOTT
REAL(KIND=JPRB) :: ZCFPR
REAL(KIND=JPRB) :: ZCOR
REAL(KIND=JPRB) :: ZCDMAX
REAL(KIND=JPRB) :: ZMIN(KLON)
REAL(KIND=JPRB) :: ZLCONDLIM
REAL(KIND=JPRB) :: ZDENOM
REAL(KIND=JPRB) :: ZDPMXDT
REAL(KIND=JPRB) :: ZDPR
REAL(KIND=JPRB) :: ZDTDP
REAL(KIND=JPRB) :: ZE
REAL(KIND=JPRB) :: ZEPSEC
REAL(KIND=JPRB) :: ZFAC, ZFACI, ZFACW
REAL(KIND=JPRB) :: ZGDCP
REAL(KIND=JPRB) :: ZINEW
REAL(KIND=JPRB) :: ZLCRIT
!>>dod
REAL(KIND=JPRB) :: ZICRIT
!<<dod
REAL(KIND=JPRB) :: ZMFDN
REAL(KIND=JPRB) :: ZPRECIP
REAL(KIND=JPRB) :: ZQE
REAL(KIND=JPRB) :: ZQSAT, ZQTMST, ZRDCP
REAL(KIND=JPRB) :: ZRHC, ZSIG, ZSIGK
!J---Start---
!J REAL(KIND=JPRB) :: ZCOVPTOTM1
!J---Start---
REAL(KIND=JPRB) :: ZWTOT
REAL(KIND=JPRB) :: ZZCO, ZZDL, ZZRH, ZZZDT, ZQADJ
REAL(KIND=JPRB) :: ZQNEW, ZTNEW
REAL(KIND=JPRB) :: ZRG_R,ZGDPH_R,ZCONS1,ZCOND,ZCONS1A,ZRPI
REAL(KIND=JPRB) :: ZLFINAL
REAL(KIND=JPRB) :: ZMELT
REAL(KIND=JPRB) :: ZEVAP
REAL(KIND=JPRB) :: ZFRZ
REAL(KIND=JPRB) :: ZVPLIQ, ZVPICE
REAL(KIND=JPRB) :: ZADD, ZBDD, ZCVDS, ZICE0, ZDEPOS
REAL(KIND=JPRB) :: ZSUPSAT(KLON)
REAL(KIND=JPRB) :: ZFALL
REAL(KIND=JPRB) :: ZRE_ICE
REAL(KIND=JPRB) :: ZRLDCP

!----------------------------
! Arrays for new microphysics
!----------------------------
!
!
INTEGER(KIND=JPIM) :: IPHASE(NCLV) ! marker for water phase of each species
                                   ! 0=vapour, 1=liquid, 2=ice

INTEGER(KIND=JPIM) :: IMELT(NCLV)  ! marks melting linkage for ice categories
                                   ! ice->liquid, snow->rain

LOGICAL :: LLFALL(NCLV)      ! marks falling species
                             ! LLFALL=0, cloud cover must > 0 for zqx > 0
                             ! LLFALL=1, no cloud needed, zqx can evaporate

LOGICAL            :: LLINDEX1(KLON,NCLV)      ! index variable
INTEGER(KIND=JPIM) :: JINDEX2(KLON,NCLV)       ! index variable
LOGICAL            :: LLINDEX3(KLON,NCLV,NCLV) ! index variable
REAL(KIND=JPRB)    :: ZMAX
REAL(KIND=JPRB)    :: ZRAT 
INTEGER(KIND=JPIM) :: IORDER(KLON,NCLV) ! array for sorting explicit terms

REAL(KIND=JPRB) :: ZLIQFRAC(KLON,KLEV)  ! cloud liquid water fraction: ql/(ql+qi)
REAL(KIND=JPRB) :: ZICEFRAC(KLON,KLEV)  ! cloud ice water fraction: qi/(ql+qi)
REAL(KIND=JPRB) :: ZQX(KLON,KLEV,NCLV)  ! water variables
REAL(KIND=JPRB) :: ZQX0(KLON,KLEV,NCLV) ! water variables at start of scheme
REAL(KIND=JPRB) :: ZQXN(KLON,NCLV)      ! new values for zqx at time+1
REAL(KIND=JPRB) :: ZQXFG(KLON,NCLV)     ! first guess values including precip
REAL(KIND=JPRB) :: ZQXNM1(KLON,NCLV)    ! new values for zqx at time+1 at level above
REAL(KIND=JPRB) :: ZFLUXQ(KLON,NCLV)    ! fluxes convergence of species (needed?)
! Keep the following for possible future total water variance scheme?
!REAL(KIND=JPRB) :: ZTL(KLON,KLEV)       ! liquid water temperature
!REAL(KIND=JPRB) :: ZABETA(KLON,KLEV)    ! cloud fraction
!REAL(KIND=JPRB) :: ZVAR(KLON,KLEV)      ! temporary variance
!REAL(KIND=JPRB) :: ZQTMIN(KLON,KLEV)
!REAL(KIND=JPRB) :: ZQTMAX(KLON,KLEV)

REAL(KIND=JPRB) :: ZPFPLSX(KLON,KLEV+1,NCLV) ! generalized precipitation flux
REAL(KIND=JPRB) :: ZLNEG(KLON,KLEV,NCLV)     ! for negative correction diagnostics
REAL(KIND=JPRB) :: ZMELTMAX(KLON)
REAL(KIND=JPRB) :: ZFRZMAX(KLON)
REAL(KIND=JPRB) :: ZICETOT(KLON)

! temporary dummy variables for development
REAL(KIND=JPRB) :: ZQXN2D(KLON,KLEV,NCLV)   ! water variables store

REAL(KIND=JPRB) :: ZQSMIX(KLON,KLEV) ! diagnostic mixed phase saturation 
!REAL(KIND=JPRB) :: ZQSBIN(KLON,KLEV) ! binary switched ice/liq saturation
REAL(KIND=JPRB) :: ZQSLIQ(KLON,KLEV) ! liquid water saturation
REAL(KIND=JPRB) :: ZQSICE(KLON,KLEV) ! ice water saturation

!REAL(KIND=JPRB) :: ZRHM(KLON,KLEV) ! diagnostic mixed phase RH
!REAL(KIND=JPRB) :: ZRHL(KLON,KLEV) ! RH wrt liq
!REAL(KIND=JPRB) :: ZRHI(KLON,KLEV) ! RH wrt ice

REAL(KIND=JPRB) :: ZFOEEWMT(KLON,KLEV)
REAL(KIND=JPRB) :: ZFOEEW(KLON,KLEV)
REAL(KIND=JPRB) :: ZFOEELIQT(KLON,KLEV)
!REAL(KIND=JPRB) :: ZFOEEICET(KLON,KLEV)

REAL(KIND=JPRB) :: ZDQSLIQDT(KLON), ZDQSICEDT(KLON), ZDQSMIXDT(KLON)
REAL(KIND=JPRB) :: ZCORQSLIQ(KLON)
REAL(KIND=JPRB) :: ZCORQSICE(KLON) 
!REAL(KIND=JPRB) :: ZCORQSBIN(KLON)
REAL(KIND=JPRB) :: ZCORQSMIX(KLON)
REAL(KIND=JPRB) :: ZEVAPLIMLIQ(KLON), ZEVAPLIMICE(KLON), ZEVAPLIMMIX(KLON)

!-------------------------------------------------------
! SOURCE/SINK array for implicit and explicit terms
!-------------------------------------------------------
!
! a POSITIVE value entered into the arrays is a...
!
!            Source of this variable
!            |
!            |   Sink of this variable
!            |   |
!            V   V
! ZSOLQA(JL,IQa,IQb)  = explicit terms
! ZSOLQB(JL,IQa,IQb)  = implicit terms
!
! Thus if ZSOLAB(JL,NCLDQL,IQV)=K where K>0 then this is 
! a source of NCLDQL and a sink of IQV
!
! put 'magic' source terms such as PLUDE from 
! detrainment into explicit source/sink array diagnognal
! ZSOLQA(NCLDQL,NCLDQL)= -PLUDE
! i.e. A positive value is a sink!????? weird... 
!-------------------------------------------------------

REAL(KIND=JPRB) :: ZSOLQA(KLON,NCLV,NCLV) ! explicit sources and sinks
REAL(KIND=JPRB) :: ZSOLQB(KLON,NCLV,NCLV) ! implicit sources and sinks
                        ! e.g. microphysical pathways between ice variables.
REAL(KIND=JPRB) :: ZQLHS(KLON,NCLV,NCLV)  ! n x n matrix storing the LHS of implicit solver
REAL(KIND=JPRB) :: ZVQX(NCLV)        ! fall speeds of three categories
REAL(KIND=JPRB) :: ZEXPLICIT, ZRATIO(KLON,NCLV), ZSINKSUM(KLON,NCLV)

! for sedimentation source/sink terms
REAL(KIND=JPRB) :: ZFALLSINK(KLON,NCLV)
REAL(KIND=JPRB) :: ZFALLSRCE(KLON,NCLV)

! for convection detrainment source and subsidence source/sink terms
REAL(KIND=JPRB) :: ZCONVSRCE(KLON,NCLV)
REAL(KIND=JPRB) :: ZCONVSINK(KLON,NCLV)

! for supersaturation source term from previous timestep
REAL(KIND=JPRB) :: ZPSUPSATSRCE(KLON,NCLV)

! Numerical fit to wet bulb temperature
REAL(KIND=JPRB),PARAMETER :: ZTW1 = 1329.31_JPRB
REAL(KIND=JPRB),PARAMETER :: ZTW2 = 0.0074615_JPRB
REAL(KIND=JPRB),PARAMETER :: ZTW3 = 0.85E5_JPRB
REAL(KIND=JPRB),PARAMETER :: ZTW4 = 40.637_JPRB
REAL(KIND=JPRB),PARAMETER :: ZTW5 = 275.0_JPRB

REAL(KIND=JPRB) :: ZSUBSAT  ! Subsaturation for snow melting term         
REAL(KIND=JPRB) :: ZTDMTW0  ! Diff between dry-bulb temperature and 
                            ! temperature when wet-bulb = 0degC 
!----------------------------
! End: new microphysics
!----------------------------

!----------------------
! SCM budget statistics 
!----------------------
REAL(KIND=JPRB), ALLOCATABLE :: ZSUMQ0(:,:),  ZSUMQ1(:,:) , ZERRORQ(:,:), &
                               &ZSUMH0(:,:),  ZSUMH1(:,:) , ZERRORH(:,:), &
                               &ZTENKEEP(:,:,:)
REAL(KIND=JPRB) :: ZRAIN

REAL(KIND=JPRB) :: Z_TMP1(KFDIA-KIDIA+1)
REAL(KIND=JPRB) :: Z_TMP2(KFDIA-KIDIA+1)
REAL(KIND=JPRB) :: Z_TMP3(KFDIA-KIDIA+1)
REAL(KIND=JPRB) :: Z_TMP4(KFDIA-KIDIA+1)
!REAL(KIND=JPRB) :: Z_TMP5(KFDIA-KIDIA+1)
REAL(KIND=JPRB) :: Z_TMP6(KFDIA-KIDIA+1)
REAL(KIND=JPRB) :: Z_TMP7(KFDIA-KIDIA+1)
REAL(KIND=JPRB) :: Z_TMPK(KFDIA-KIDIA+1,KLEV)
!REAL(KIND=JPRB) :: ZCON1,ZCON2
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZTMPL,ZTMPI,ZTMPA

!J---Start---
REAL(KIND=JPRB) :: ZMM,ZRR
REAL(KIND=JPRB) :: ZRG(KLON)
!J---End---

LOGICAL :: LLCLDBUDCC    ! Cloud fraction budget 
LOGICAL :: LLCLDBUDL     ! Cloud liquid condensate budget
LOGICAL :: LLCLDBUDI     ! Cloud ice condensate budget

REAL(KIND=JPRB) :: ZBUDCC(KLON,KFLDX) ! extra fields
REAL(KIND=JPRB) :: ZBUDL(KLON,KFLDX) ! extra fields
REAL(KIND=JPRB) :: ZBUDI(KLON,KFLDX) ! extra fields

!-- variables for MAC2SP anthropogenic aerosol
REAL(KIND=JPRB)   :: ZMAC2SP_CDNC_FACTOR(KLON)

!-----------------------------------------------------------------------


!#include "cloudvar.intfb.h"
!#include "cover.intfb.h"
#include "abor1.intfb.h"
#include "cuadjtq.intfb.h"
#include "lubksb.intfb.h"
#include "ludcmp.intfb.h"

!DIR$ VFUNCTION EXPHF
#include "fcttre.h"
#include "fccld.h"

!===============================================================================
!  0.0     Beginning of timestep book-keeping
!----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('CLOUDSC',0,ZHOOK_HANDLE)


!######################################################################
!
!
!             0.  *** SET UP CONSTANTS ***
!
!
!######################################################################


! --------------------------------------------------------------
! Set LCLDBUD logicals to store enthalpy and total water budgets
! Initialise PEXTRA if true.
! --------------------------------------------------------------
LLCLDBUDCC = .FALSE. 
LLCLDBUDL  = .FALSE.
LLCLDBUDI  = .FALSE.
IF (LLCLDBUDCC .OR. LLCLDBUDL .OR. LLCLDBUDI) PEXTRA(:,:,:)=0.0_JPRB

IF (LLCLDBUDCC .AND. KFLDX <11) THEN
  CALL ABOR1('ERROR: Not enough PEXTRA variables for cloud frac budget.')
ENDIF

! ---------------------
! Some simple constants
! ---------------------
ZQTMST  = 1.0_JPRB/PTSPHY
ZGDCP   = RG/RCPD
ZRDCP   = RD/RCPD
ZCONS1A = RCPD/(RLMLT*RG*RTAUMEL)
ZEPSEC  = 1.E-14_JPRB
ZRG_R   = 1.0_JPRB/RG
ZRLDCP  = 1.0_JPRB/(RALSDCP-RALVDCP)
ZRPI=1.0_JPRB/RPI

! Note: Defined in module/yoecldp.F90
! NCLDQL=1    ! liquid cloud water
! NCLDQI=2    ! ice cloud water
! NCLDQR=3    ! rain water
! NCLDQS=4    ! snow
! NCLDQV=5    ! vapour

! -----------------------------------------------
! Define species phase, 0=vapour, 1=liquid, 2=ice
! -----------------------------------------------
IPHASE(NCLDQV)=0
IPHASE(NCLDQL)=1
IPHASE(NCLDQR)=1
IPHASE(NCLDQI)=2
IPHASE(NCLDQS)=2

! ---------------------------------------------------
! Set up melting/freezing index, 
! if an ice category melts/freezes, where does it go?
! ---------------------------------------------------
IMELT(NCLDQV)=-99
IMELT(NCLDQL)=NCLDQI
IMELT(NCLDQR)=NCLDQS
IMELT(NCLDQI)=NCLDQL
IMELT(NCLDQS)=NCLDQR

! -------------------------
! set up fall speeds in m/s
! -------------------------
ZVQX(NCLDQV)=0.0_JPRB 
ZVQX(NCLDQL)=0.0_JPRB 
ZVQX(NCLDQI)=RVICE 
ZVQX(NCLDQR)=RVRAIN
ZVQX(NCLDQS)=RVSNOW
LLFALL(:)=.FALSE.
DO JM=1,NCLV
  IF (ZVQX(JM)>0.0_JPRB) LLFALL(JM)=.TRUE. ! falling species
ENDDO
! Set LLFALL to false for ice (but ice still sediments!)
! Need to rationalise this at some point
LLFALL(NCLDQI)=.FALSE.

!######################################################################
!
!
!         1.a  *** CALL to MAC-SP anthropogenic simple plume model ***
!
!
!######################################################################

! multiply CDNC with MAC2SP factor only if it hasn't previoulsy been done in aer_activ (i.e. NCLOUDACT==0)
IF (LDMAINCALL .AND. LMAC2SPACI .AND. NCLOUDACT==0 .AND. (LAERLIQAUTOLSP .OR. LAERLIQCOLL) ) THEN

  CALL GET_CDNC_FACTOR(KIDIA,KFDIA,KLON,SIN(PGELAT),PGELAM,ZMAC2SP_CDNC_FACTOR)

  ! Include effect on CDNC
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      PCDNC(JL,JK) = PCDNC(JL,JK)*ZMAC2SP_CDNC_FACTOR(JL)
    END DO
  END DO
ENDIF

!######################################################################
!
!
!             1.b  *** INITIAL VALUES FOR VARIABLES ***
!
!
!######################################################################

! ----------------------
! non-CLV initialization
! ----------------------
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF ( LDSLPHY ) THEN
      ZA(JL,JK)          = PA(JL,JK)
      ZAORIG(JL,JK)      = PA(JL,JK)
      ZTP1(JL,JK)        = PT(JL,JK)
      ZQX(JL,JK,NCLDQV)  = PQ(JL,JK)
      ZQX0(JL,JK,NCLDQV) = PQ(JL,JK)
    ELSE
      ZLI(JL,JK)    = PL(JL,JK)+PI(JL,JK)+PTSPHY*(PTENL(JL,JK)+PTENI(JL,JK))
      ZA(JL,JK)     = PA(JL,JK)+PTSPHY*PTENA(JL,JK)
      ZAORIG(JL,JK) = PA(JL,JK)+PTSPHY*PTENA(JL,JK)
      ZTP1(JL,JK)   = PT(JL,JK)+PTSPHY*PTENT(JL,JK)
      ZQX(JL,JK,NCLDQV)  = PQ(JL,JK)+PTSPHY*PTENQ(JL,JK) 
      ZQX0(JL,JK,NCLDQV) = PQ(JL,JK)+PTSPHY*PTENQ(JL,JK)
    ENDIF
  ENDDO
ENDDO

! -------------------------------------
! initialization for CLV family
! -------------------------------------
DO JM=1,NCLV-1
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      IF ( LDSLPHY ) THEN
        ZQX(JL,JK,JM)  = PCLV(JL,JK,JM)
        ZQX0(JL,JK,JM) = PCLV(JL,JK,JM)
      ELSE
        ZQX(JL,JK,JM)  = PCLV(JL,JK,JM)+PTSPHY*PTENCLV(JL,JK,JM)
        ZQX0(JL,JK,JM) = PCLV(JL,JK,JM)+PTSPHY*PTENCLV(JL,JK,JM)
      ENDIF
    ENDDO
  ENDDO
ENDDO

!-------------------------------------------------------------------
! Ensure convective variables are 0 where convection is not present
! This is here for security but may no longer be needed. 
!-------------------------------------------------------------------
DO JL=KIDIA,KFDIA
  IF(.NOT.LDCUM(JL)) THEN
    DO JK=1,KLEV
      PLUDE(JL,JK) = 0.0_JPRB
      PMFU(JL,JK)  = 0.0_JPRB
      PMFD(JL,JK)  = 0.0_JPRB
      PLU(JL,JK)   = 0.0_JPRB
    ENDDO
  ENDIF
ENDDO


IF (LDMAINCALL) THEN  ! *** Only executed in main cloud scheme call***

!-------------
! zero arrays
!-------------
ZPFPLSX(:,:,:) = 0.0_JPRB ! precip fluxes
ZQXN2D(:,:,:)  = 0.0_JPRB ! end of timestep values in 2D
ZLNEG(:,:,:)   = 0.0_JPRB ! negative input check

! -------------------------------------------
! Total water and enthalpy budget diagnostics
! -------------------------------------------
IF (LSCMEC.OR.LCLDBUDGET) THEN

  IF (.NOT. ALLOCATED(ZSUMQ0))   ALLOCATE(ZSUMQ0(KLON,KLEV))
  IF (.NOT. ALLOCATED(ZSUMQ1))   ALLOCATE(ZSUMQ1(KLON,KLEV))
  IF (.NOT. ALLOCATED(ZSUMH0))   ALLOCATE(ZSUMH0(KLON,KLEV))
  IF (.NOT. ALLOCATED(ZSUMH1))   ALLOCATE(ZSUMH1(KLON,KLEV))
  IF (.NOT. ALLOCATED(ZERRORQ))  ALLOCATE(ZERRORQ(KLON,KLEV))
  IF (.NOT. ALLOCATED(ZERRORH))  ALLOCATE(ZERRORH(KLON,KLEV))
  IF (.NOT. ALLOCATED(ZTENKEEP)) ALLOCATE(ZTENKEEP(KLON,KLEV,NCLV+1))

  ! record the tendencies
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENKEEP(JL,JK,NCLDQL)=PTENCLV(JL,JK,NCLDQL)
      ZTENKEEP(JL,JK,NCLDQI)=PTENCLV(JL,JK,NCLDQI)
      ZTENKEEP(JL,JK,NCLDQR)=PTENCLV(JL,JK,NCLDQR)
      ZTENKEEP(JL,JK,NCLDQS)=PTENCLV(JL,JK,NCLDQS)
      ZTENKEEP(JL,JK,NCLDQV)=PTENQ(JL,JK)
      ZTENKEEP(JL,JK,NCLV+1)=PTENT(JL,JK)
    ENDDO
  ENDDO

  ! initialize the flux arrays
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTNEW=PT(JL,JK)+PTSPHY*(PTENT(JL,JK)-ZTENKEEP(JL,JK,NCLV+1))
      IF (JK==1) THEN
        ZSUMQ0(JL,JK)=0.0_JPRB ! total water
        ZSUMH0(JL,JK)=0.0_JPRB ! liquid water temperature
      ELSE
        ZSUMQ0(JL,JK)=ZSUMQ0(JL,JK-1)
        ZSUMH0(JL,JK)=ZSUMH0(JL,JK-1)
      ENDIF

      ! Total for liquid
      ZTMPL = (PCLV(JL,JK,NCLDQL)+PCLV(JL,JK,NCLDQR) &
            &  +(PTENCLV(JL,JK,NCLDQL)-ZTENKEEP(JL,JK,NCLDQL))*PTSPHY &
            &  +(PTENCLV(JL,JK,NCLDQR)-ZTENKEEP(JL,JK,NCLDQR))*PTSPHY)
      ! Total for frozen
      ZTMPI = (PCLV(JL,JK,NCLDQI)+PCLV(JL,JK,NCLDQS) &
            &  +(PTENCLV(JL,JK,NCLDQI)-ZTENKEEP(JL,JK,NCLDQI))*PTSPHY &
            &  +(PTENCLV(JL,JK,NCLDQS)-ZTENKEEP(JL,JK,NCLDQS))*PTSPHY)
      ZTNEW = ZTNEW - RALVDCP*ZTMPL - RALSDCP*ZTMPI
      ZSUMQ0(JL,JK)=ZSUMQ0(JL,JK)*(ZTMPL+ZTMPI)*(PAPH(JL,JK+1)-PAPH(JL,JK))*ZRG_R

      ! detrained water treated here
      ZQE=PLUDE(JL,JK)*PTSPHY*RG/(PAPH(JL,JK+1)-PAPH(JL,JK))
      IF (ZQE>RLMIN) THEN
        ZSUMQ0(JL,JK)=ZSUMQ0(JL,JK)+PLUDE(JL,JK)*PTSPHY
        ZALFAW=FOEALFA(ZTP1(JL,JK))
        ZTNEW=ZTNEW-(RALVDCP*ZALFAW+RALSDCP*(1.0_JPRB-ZALFAW))*ZQE
      ENDIF

      ZSUMH0(JL,JK)=ZSUMH0(JL,JK)+(PAPH(JL,JK+1)-PAPH(JL,JK))*ZTNEW 
      ZSUMQ0(JL,JK)=ZSUMQ0(JL,JK)+(PQ(JL,JK)+(PTENQ(JL,JK)-ZTENKEEP(JL,JK,NCLDQV))* &
                    & PTSPHY)*(PAPH(JL,JK+1)-PAPH(JL,JK))*ZRG_R
    ENDDO
  ENDDO
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZSUMH0(JL,JK)=ZSUMH0(JL,JK)/PAPH(JL,JK+1)
    ENDDO
  ENDDO
ENDIF

! ----------------------------------------------------
! Tidy up very small cloud cover or total cloud water
! ----------------------------------------------------
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF (ZQX(JL,JK,NCLDQL)+ZQX(JL,JK,NCLDQI)<RLMIN.OR.ZA(JL,JK)<RAMIN) THEN

      ! Evaporate small cloud liquid water amounts
      ZLNEG(JL,JK,NCLDQL) = ZLNEG(JL,JK,NCLDQL)+ZQX(JL,JK,NCLDQL)
      ZQADJ               = ZQX(JL,JK,NCLDQL)*ZQTMST
      PTENQ(JL,JK)        = PTENQ(JL,JK)+ZQADJ
      PTENT(JL,JK)        = PTENT(JL,JK)-RALVDCP*ZQADJ
      ZQX(JL,JK,NCLDQV)   = ZQX(JL,JK,NCLDQV)+ZQX(JL,JK,NCLDQL)
      ZQX(JL,JK,NCLDQL)   = 0.0_JPRB

      ! Evaporate small cloud ice water amounts
      ZLNEG(JL,JK,NCLDQI) = ZLNEG(JL,JK,NCLDQI)+ZQX(JL,JK,NCLDQI)
      ZQADJ               = ZQX(JL,JK,NCLDQI)*ZQTMST
      PTENQ(JL,JK)        = PTENQ(JL,JK)+ZQADJ
      PTENT(JL,JK)        = PTENT(JL,JK)-RALSDCP*ZQADJ
      ZQX(JL,JK,NCLDQV)   = ZQX(JL,JK,NCLDQV)+ZQX(JL,JK,NCLDQI)
      ZQX(JL,JK,NCLDQI)   = 0.0_JPRB

      ! Set cloud cover to zero
      ZA(JL,JK)           = 0.0_JPRB

    ENDIF
  ENDDO
ENDDO

! ---------------------------------
! Tidy up small CLV variables
! ---------------------------------
DO JM=1,NCLV-1
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      IF (ZQX(JL,JK,JM)<RLMIN) THEN
        ZLNEG(JL,JK,JM) = ZLNEG(JL,JK,JM)+ZQX(JL,JK,JM)
        ZQADJ               = ZQX(JL,JK,JM)*ZQTMST
        PTENQ(JL,JK)        = PTENQ(JL,JK)+ZQADJ
        IF (IPHASE(JM)==1) PTENT(JL,JK) = PTENT(JL,JK)-RALVDCP*ZQADJ
        IF (IPHASE(JM)==2) PTENT(JL,JK) = PTENT(JL,JK)-RALSDCP*ZQADJ
        ZQX(JL,JK,NCLDQV)   = ZQX(JL,JK,NCLDQV)+ZQX(JL,JK,JM)
        ZQX(JL,JK,JM)       = 0.0_JPRB
      ENDIF
    ENDDO
  ENDDO
ENDDO

ENDIF ! on LDMAINCALL ***Above only executed in main cloud scheme call***


! ------------------------------
! Define saturation values
! ------------------------------
IF(N_VMASS > 1) THEN
  JLEN=KFDIA-KIDIA+1
ENDIF

DO JK=1,KLEV
  IF(N_VMASS > 0) THEN ! Using Vector VMASS

    !------------------------------------------------------------------------
    ! Calculate Koop supersaturation limit function using Vector VMass
    !  FOEELIQ(PTARE) = R2ES*EXP(R3LES*(PTARE-RTT)/(PTARE-R4LES))
    !  FOEEICE(PTARE) = R2ES*EXP(R3IES*(PTARE-RTT)/(PTARE-R4IES))
    !  Z_TMPK = FOEELIQ(PTARE)/FOEEICE(PTARE)
    !  Used in function: 
    !  FOKOOP(PTARE) = MIN(RKOOP1-RKOOP2*PTARE,FOEELIQ(PTARE)/FOEEICE(PTARE))
    !------------------------------------------------------------------------
    DO JL=KIDIA,KFDIA
      Z_TMP1(JL-KIDIA+1)=R3LES*(ZTP1(JL,JK)-RTT)
      Z_TMP2(JL-KIDIA+1)=R3IES*(ZTP1(JL,JK)-RTT)
      Z_TMP3(JL-KIDIA+1)=ZTP1(JL,JK)-R4LES
      Z_TMP4(JL-KIDIA+1)=ZTP1(JL,JK)-R4IES
    ENDDO
    CALL VDIV(Z_TMP6,Z_TMP1,Z_TMP3,JLEN)
    CALL VDIV(Z_TMP7,Z_TMP2,Z_TMP4,JLEN)
    CALL VEXP(Z_TMP1,Z_TMP6,JLEN)
    CALL VEXP(Z_TMP2,Z_TMP7,JLEN)
    CALL VREC(Z_TMP3,PAP(KIDIA,JK),JLEN)
    CALL VDIV(Z_TMPK(1,JK),Z_TMP1,Z_TMP2,JLEN)

    DO JL=KIDIA,KFDIA
      !----------------------------------------
      ! old *diagnostic* mixed phase saturation
      !---------------------------------------- 
      ZFOEALFA(JL,JK)=FOEALFA(ZTP1(JL,JK))
      ZFOEEWMT(JL,JK)=R2ES*(ZFOEALFA(JL,JK)*Z_TMP1(JL-KIDIA+1)+&
       & (1.0_JPRB-ZFOEALFA(JL,JK))*Z_TMP2(JL-KIDIA+1))*Z_TMP3(JL-KIDIA+1)  
      ZFOEEWMT(JL,JK)=MIN(0.5_JPRB,ZFOEEWMT(JL,JK))
      ZQSMIX(JL,JK)=ZFOEEWMT(JL,JK)/(1.0_JPRB-RETV*ZFOEEWMT(JL,JK))
      !---------------------------------------------
      ! binary qsat - liq for T>273K, ice for T<273K 
      !---------------------------------------------
      ZALFA = FOEDELTA(ZTP1(JL,JK))
      ZFOEEW(JL,JK)=R2ES*(ZALFA*Z_TMP1(JL-KIDIA+1)+&
       & (1.0_JPRB-ZALFA)*Z_TMP2(JL-KIDIA+1))*Z_TMP3(JL-KIDIA+1)  
      ZFOEEW(JL,JK)=MIN(0.5_JPRB,ZFOEEW(JL,JK))
      ZQSICE(JL,JK)=ZFOEEW(JL,JK)/(1.0_JPRB-RETV*ZFOEEW(JL,JK))
      !----------------------------------
      ! liquid water saturation
      !---------------------------------- 
      ZFOEELIQT(JL,JK)=MIN(R2ES*Z_TMP1(JL-KIDIA+1)*Z_TMP3(JL-KIDIA+1),0.5_JPRB)
      ZQSLIQ(JL,JK)=ZFOEELIQT(JL,JK)/(1.0_JPRB-RETV*ZFOEELIQT(JL,JK))

!      !----------------------------------
!      ! ice water saturation
!      !---------------------------------- 
!      ZFOEEICET(JL,JK)=MIN(R2ES*Z_TMP2(JL-KIDIA+1)*Z_TMP3(JL-KIDIA+1),0.5_JPRB)
!      ZQSICE(JL,JK)=ZFOEEICET(JL,JK)/(1.0_JPRB-RETV*ZFOEEICET(JL,JK))
    ENDDO
  ELSE
    DO JL=KIDIA,KFDIA
      !----------------------------------------
      ! old *diagnostic* mixed phase saturation
      !---------------------------------------- 
      ZFOEALFA(JL,JK)=FOEALFA(ZTP1(JL,JK))
      ZFOEEWMT(JL,JK)=MIN(FOEEWM(ZTP1(JL,JK))/PAP(JL,JK),0.5_JPRB)
      ZQSMIX(JL,JK)=ZFOEEWMT(JL,JK)
      ZQSMIX(JL,JK)=ZQSMIX(JL,JK)/(1.0_JPRB-RETV*ZQSMIX(JL,JK))

      !---------------------------------------------
      ! ice saturation T<273K
      ! liquid water saturation for T>273K 
      !---------------------------------------------
      ZALFA=FOEDELTA(ZTP1(JL,JK))
      ZFOEEW(JL,JK)=MIN((ZALFA*FOEELIQ(ZTP1(JL,JK))+ &
        &  (1.0_JPRB-ZALFA)*FOEEICE(ZTP1(JL,JK)))/PAP(JL,JK),0.5_JPRB)
      ZFOEEW(JL,JK)=MIN(0.5_JPRB,ZFOEEW(JL,JK))
      ZQSICE(JL,JK)=ZFOEEW(JL,JK)/(1.0_JPRB-RETV*ZFOEEW(JL,JK))

      !----------------------------------
      ! liquid water saturation
      !---------------------------------- 
      ZFOEELIQT(JL,JK)=MIN(FOEELIQ(ZTP1(JL,JK))/PAP(JL,JK),0.5_JPRB)
      ZQSLIQ(JL,JK)=ZFOEELIQT(JL,JK)
      ZQSLIQ(JL,JK)=ZQSLIQ(JL,JK)/(1.0_JPRB-RETV*ZQSLIQ(JL,JK))

!      !----------------------------------
!      ! ice water saturation
!      !---------------------------------- 
!      ZFOEEICET(JL,JK)=MIN(FOEEICE(ZTP1(JL,JK))/PAP(JL,JK),0.5_JPRB)
!      ZQSICE(JL,JK)=ZFOEEICET(JL,JK)
!      ZQSICE(JL,JK)=ZQSICE(JL,JK)/(1.0_JPRB-RETV*ZQSICE(JL,JK))
    ENDDO
  ENDIF

! Calculation of relative humidity for diagnostics
! Not used at present
!  DO JL=KIDIA,KFDIA
!    ZRHM(JL,JK)=ZQX(JL,JK,NCLDQV)/ZQSMIX(JL,JK)
!    ZRHL(JL,JK)=ZQX(JL,JK,NCLDQV)/ZQSLIQ(JL,JK)
!    ZRHI(JL,JK)=ZQX(JL,JK,NCLDQV)/ZQSICE(JL,JK)
!  ENDDO

ENDDO

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA


    !------------------------------------------
    ! Ensure cloud fraction is between 0 and 1
    !------------------------------------------
    ZA(JL,JK)=MAX(0.0_JPRB,MIN(1.0_JPRB,ZA(JL,JK)))

    !-------------------------------------------------------------------
    ! Calculate liq/ice fractions (no longer a diagnostic relationship)
    !-------------------------------------------------------------------
    ZLI(JL,JK)=ZQX(JL,JK,NCLDQL)+ZQX(JL,JK,NCLDQI)
    IF (ZLI(JL,JK)>RLMIN) THEN
      ZLIQFRAC(JL,JK)=ZQX(JL,JK,NCLDQL)/ZLI(JL,JK)
      ZICEFRAC(JL,JK)=1.0_JPRB-ZLIQFRAC(JL,JK)
    ELSE
      ZLIQFRAC(JL,JK)=0.0_JPRB
      ZICEFRAC(JL,JK)=0.0_JPRB
    ENDIF

  ENDDO
ENDDO

!----------------------------------------------------------------------
! This is test code. Instead of resetting cloud water or cloud cover
! to zero if one of the two is zero, here we slave the cloud cover
! to the cloud water variable. I.e. if cloud cover is zero it is 
! set to an appropriate non-zero value.
! It uses a Beta curve to get the variance and then derive cloud cover.
! It is quite slow since it involves iteration, and should be left 
! until a fully prognostic variance equation for total water is 
! implemented
!-----------------------------------------------------------------------
!IF (.FALSE.) THEN
!  CALL CLOUDVAR &
!---input
! & ( KIDIA, KFDIA, KLON  , KLEV  , 1    , KLEV, &
! &   ZTP1, ZQP1, ZQSMIX, ZLI, PAP, &
!---output
! &   ZVAR, ZQTMIN, ZQTMAX ) !last two are dummy args  

!  CALL COVER &
!---input
! & ( KIDIA, KFDIA , KLON, KLEV, 1, KLEV, &
! &   ZA, ZTP1,  ZQP1, ZQSMIX, ZLI, PAP, ZVAR, &
!---output
! &   ZQTMAX, ZABETA )  

!  DO JK=1,KLEV
!    DO JL=KIDIA,KFDIA
!      IF (ZLI(JL,JK)/MAX(ZA(JL,JK),ZEPSEC)>RCLDMAX) THEN
!        ZA(JL,JK)=ZABETA(JL,JK) ! not part of tendency       
!      ENDIF
!    ENDDO
!  ENDDO
!ENDIF


!--------------------------------------
! NPM
! Initialize liq water temperature T_L 
! Not used at present
!--------------------------------------
!ZTL(:,:)=ZTP1(:,:)
!DO JM=1,NCLV
!  DO JK=1,KLEV
!    DO JL=KIDIA,KFDIA
!      IF (IPHASE(JM)==1) ZTL(JL,JK)=ZTL(JL,JK)-RALVDCP*ZQX(JL,JK,JM)
!      IF (IPHASE(JM)==2) ZTL(JL,JK)=ZTL(JL,JK)-RALSDCP*ZQX(JL,JK,JM)
!    ENDDO
!  ENDDO
!ENDDO


!######################################################################
!
!
!        2.       *** CONSTANTS AND PARAMETERS ***
!
!
!######################################################################
!  Calculate L in updrafts of bl-clouds
!  Specify QS, P/PS for tropopause (for c2)
!  And initialize variables
!------------------------------------------

!---------------------------------
! Find tropopause level (ZTRPAUS)
!---------------------------------
DO JL=KIDIA,KFDIA
  ZTRPAUS(JL)=0.1_JPRB
  ZPAPHD(JL)=1.0_JPRB/PAPH(JL,KLEV+1)
ENDDO
DO JK=1,KLEV-1
  DO JL=KIDIA,KFDIA
    ZSIG=PAP(JL,JK)*ZPAPHD(JL)
    IF (ZSIG>0.1_JPRB.AND.ZSIG<0.4_JPRB.AND.ZTP1(JL,JK)>ZTP1(JL,JK+1)) THEN
      ZTRPAUS(JL)=ZSIG
    ENDIF
  ENDDO
ENDDO

!-----------------------------
! Reset single level variables
!-----------------------------

ZANEWM1(:)  = 0.0_JPRB
ZDA(:)      = 0.0_JPRB
ZCOVPCLR(:) = 0.0_JPRB
ZCOVPCLD(:) = 0.0_JPRB  
ZCOVPTOT(:) = 0.0_JPRB

IF( N_VMASS > 0 )THEN
  DO JL=KIDIA,KFDIA
    ZTMP1(JL-KIDIA+1)=(-0.178)
    ZTMP2(JL-KIDIA+1)=(-0.394)
  ENDDO
ENDIF

!######################################################################
!
!
!           3.       *** PHYSICS ***
!
!
!######################################################################


!----------------------------------------------------------------------
!
!                       START OF VERTICAL LOOP
!
!----------------------------------------------------------------------

!J---Start---
IF(N_VMASS >0 ) THEN ! Using Vector MASS
  ZRG(:)=RG
ENDIF
!J---End---

DO JK=NCLDTOP,KLEV

!----------------------------------------------------------------------
! 3.0 INITIALIZE VARIABLES
!----------------------------------------------------------------------

  !---------------------------------
  ! First guess microphysics
  !---------------------------------
  DO JM=1,NCLV
    DO JL=KIDIA,KFDIA
      ZQXFG(JL,JM)=ZQX(JL,JK,JM)
    ENDDO
  ENDDO

  !---------------------------------
  ! Set KLON arrays to zero
  !---------------------------------

  ZLICLD(:)   = 0.0_JPRB                                
  ZRAINAUT(:) = 0.0_JPRB  ! currently needed for diags  
  ZSNOWAUT(:) = 0.0_JPRB  ! needed                      
  ZLDEFR(:)   = 0.0_JPRB                                
  ZACUST(:)   = 0.0_JPRB  ! set later when needed       
  ZQPRETOT(:) = 0.0_JPRB                                
  ZLFINALSUM(:)= 0.0_JPRB                               

  ! Required for first guess call
  ZLCOND1(:) = 0.0_JPRB
  ZLCOND2(:) = 0.0_JPRB
  ZSUPSAT(:) = 0.0_JPRB
  ZLEVAPL(:) = 0.0_JPRB
  ZLEVAPI(:) = 0.0_JPRB

  !-------------------------------------                
  ! solvers for cloud fraction                          
  !-------------------------------------                
  ZSOLAB(:) = 0.0_JPRB
  ZSOLAC(:) = 0.0_JPRB

  !------------------------------------------           
  ! reset matrix so missing pathways are set            
  !------------------------------------------           
  ZSOLQB(:,:,:) = 0.0_JPRB
  ZSOLQA(:,:,:) = 0.0_JPRB

  !----------------------------------                   
  ! reset new microphysics variables                    
  !----------------------------------                   
  ZFALLSRCE(:,:) = 0.0_JPRB
  ZFALLSINK(:,:) = 0.0_JPRB
  ZCONVSRCE(:,:) = 0.0_JPRB
  ZCONVSINK(:,:) = 0.0_JPRB
  ZPSUPSATSRCE(:,:) = 0.0_JPRB
  ZRATIO(:,:)    = 0.0_JPRB
  ZICETOT(:)     = 0.0_JPRB                            

  ! Cloud budget arrays                                 
  IF (LLCLDBUDCC) ZBUDCC(:,:) = 0.0_JPRB                
  IF (LLCLDBUDL)  ZBUDL(:,:) = 0.0_JPRB                 
  IF (LLCLDBUDI)  ZBUDI(:,:) = 0.0_JPRB                 
  
!J---Start---
IF(N_VMASS > 1) THEN
  DO JL=KIDIA,KFDIA
    ZDP(JL)  = PAPH(JL,JK+1)-PAPH(JL,JK)     ! dp
    ZRHO(JL) = RD*ZTP1(JL,JK)
  ENDDO
  CALL VDIV(ZGDP(KIDIA),ZRG(KIDIA),ZDP(KIDIA),KFDIA-KIDIA+1)
  CALL VDIV(ZRHO(KIDIA),PAP(KIDIA,JK),ZRHO(KIDIA),KFDIA-KIDIA+1)
ENDIF
!J---End---

  DO JL=KIDIA,KFDIA

    !-------------------------
    ! derived variables needed
    !-------------------------
!J---Start---
    IF(N_VMASS <= 1) THEN
      ZDP(JL)     = PAPH(JL,JK+1)-PAPH(JL,JK)     ! dp
      ZGDP(JL)    = RG/ZDP(JL)                    ! g/dp
      ZRHO(JL)    = PAP(JL,JK)/(RD*ZTP1(JL,JK))   ! p/RT air density
    ENDIF
!J---End---
    ZDTGDP(JL)  = PTSPHY*ZGDP(JL)               ! dt g/dp
!J---Start----
!J  ZRDTGDP(JL) = 1.0_JPRB/ZDTGDP(JL)           ! 1/(dt g/dp)
    ZRDTGDP(JL) = ZDP(JL)*(1.0_JPRB/(PTSPHY*RG))  ! 1/(dt g/dp)
!J---End----
    IF (JK>1) ZDTGDPF(JL) = PTSPHY*RG/(PAP(JL,JK)-PAP(JL,JK-1))

    !------------------------------------
    ! Calculate dqs/dT correction factor
    !------------------------------------
    ! Reminder: RETV=RV/RD-1
    
    ! liquid
    ZFACW         = R5LES/((ZTP1(JL,JK)-R4LES)**2)
    ZCOR          = 1.0_JPRB/(1.0_JPRB-RETV*ZFOEELIQT(JL,JK))
    ZDQSLIQDT(JL) = ZFACW*ZCOR*ZQSLIQ(JL,JK)
    ZCORQSLIQ(JL) = 1.0_JPRB+RALVDCP*ZDQSLIQDT(JL)

    ! ice
    ZFACI         = R5IES/((ZTP1(JL,JK)-R4IES)**2)
    ZCOR          = 1.0_JPRB/(1.0_JPRB-RETV*ZFOEEW(JL,JK))
    ZDQSICEDT(JL) = ZFACI*ZCOR*ZQSICE(JL,JK)
    ZCORQSICE(JL) = 1.0_JPRB+RALSDCP*ZDQSICEDT(JL)

    ! diagnostic mixed
    ZALFAW        = ZFOEALFA(JL,JK)
    ZALFAWM(JL)   = ZALFAW
    ZFAC          = ZALFAW*ZFACW+(1.0_JPRB-ZALFAW)*ZFACI
    ZCOR          = 1.0_JPRB/(1.0_JPRB-RETV*ZFOEEWMT(JL,JK))
    ZDQSMIXDT(JL) = ZFAC*ZCOR*ZQSMIX(JL,JK)
!J---Start---
!J  ZCORQSMIX(JL) = 1.0_JPRB+FOELDCPM(ZTP1(JL,JK))*ZDQSMIXDT(JL)
    ZCORQSMIX(JL) = 1.0_JPRB/(1.0_JPRB+FOELDCPM(ZTP1(JL,JK))*ZDQSMIXDT(JL))
!J---End---

    ! evaporation/sublimation limits
!J---Start---
!J  ZEVAPLIMMIX(JL) = MAX((ZQSMIX(JL,JK)-ZQX(JL,JK,NCLDQV))/ZCORQSMIX(JL),0.0_JPRB)
    ZEVAPLIMMIX(JL) = MAX((ZQSMIX(JL,JK)-ZQX(JL,JK,NCLDQV))*ZCORQSMIX(JL),0.0_JPRB)
!J---End---
    ZEVAPLIMLIQ(JL) = MAX((ZQSLIQ(JL,JK)-ZQX(JL,JK,NCLDQV))/ZCORQSLIQ(JL),0.0_JPRB)
    ZEVAPLIMICE(JL) = MAX((ZQSICE(JL,JK)-ZQX(JL,JK,NCLDQV))/ZCORQSICE(JL),0.0_JPRB)

    !--------------------------------
    ! in-cloud consensate amount
    !--------------------------------
    ZTMPA = 1.0_JPRB/MAX(ZA(JL,JK),ZEPSEC)
    ZLIQCLD(JL) = ZQX(JL,JK,NCLDQL)*ZTMPA
    ZICECLD(JL) = ZQX(JL,JK,NCLDQI)*ZTMPA
    ZLICLD(JL)  = ZLIQCLD(JL)+ZICECLD(JL)

  ENDDO
  
  !------------------------------------------------
  ! Evaporate very small amounts of liquid and ice
  !------------------------------------------------
  DO JL=KIDIA,KFDIA

    IF (ZQX(JL,JK,NCLDQL) < RLMIN) THEN
      ZSOLQA(JL,NCLDQV,NCLDQL) = ZQX(JL,JK,NCLDQL)
      ZSOLQA(JL,NCLDQL,NCLDQV) = -ZQX(JL,JK,NCLDQL)
    ENDIF

    IF (ZQX(JL,JK,NCLDQI) < RLMIN) THEN
      ZSOLQA(JL,NCLDQV,NCLDQI) = ZQX(JL,JK,NCLDQI)
      ZSOLQA(JL,NCLDQI,NCLDQV) = -ZQX(JL,JK,NCLDQI)
    ENDIF

  ENDDO
  
  !---------------------------------------------------------------------
  !  3.1  ICE SUPERSATURATION ADJUSTMENT
  !---------------------------------------------------------------------
  ! Note that the supersaturation adjustment is made with respect to 
  ! liquid saturation:  when T>0C 
  ! ice saturation:     when T<0C
  !                     with an adjustment made to allow for ice 
  !                     supersaturation in the clear sky
  !
  ! Note also that the KOOP factor automatically clips the supersaturation
  ! to a maximum set by the liquid water saturation mixing ratio
  ! important for temperatures near to but below 0C
  !----------------------------------------------------------------------- 

  DO JL=KIDIA,KFDIA

    !-----------------------------------
    ! 3.1.1 Supersaturation limit (from Koop)
    !-----------------------------------
    ! Needs to be set for all temperatures
    IF(N_VMASS >0 ) THEN ! Using Vector MASS
      ZFOKOOP(JL) = MIN(RKOOP1-RKOOP2*ZTP1(JL,JK), Z_TMPK(JL-KIDIA+1,JK))
    ELSE
      ZFOKOOP(JL)=FOKOOP(ZTP1(JL,JK))
    ENDIF

!    IF (ZTP1(JL,JK)>=RTT .OR. NSSOPT==0) THEN
    IF (ZTP1(JL,JK)>=RTICE .OR. NSSOPT==0) THEN
      ZFAC  = 1.0_JPRB
      ZFACI = 1.0_JPRB
    ELSE
      ZFAC  = ZA(JL,JK)+ZFOKOOP(JL)*(1.0_JPRB-ZA(JL,JK))
      ZFACI = PTSPHY/RKOOPTAU
    ENDIF

    !-------------------------------------------------------------------
    ! 3.1.2 Calculate supersaturation wrt Koop including dqs/dT 
    !       correction factor
    ! [#Note: Need to move away from using QSMIX]
    !-------------------------------------------------------------------

!J---Start---
!J  ZSUPSAT(JL) = MAX((ZQX(JL,JK,NCLDQV)-ZFAC*ZQSMIX(JL,JK))/ZCORQSMIX(JL),0.0_JPRB)
    ZSUPSAT(JL) = MAX((ZQX(JL,JK,NCLDQV)-ZFAC*ZQSMIX(JL,JK))*ZCORQSMIX(JL),0.0_JPRB)
!J---End---
    
    !-------------------------------------------------------------------
    ! Here the supersaturation is turned into liquid water
    ! However, if the temperature is below the threshold for homogeneous
    ! freezing then the supersaturation is turned instantly to ice.
    !--------------------------------------------------------------------

    IF (ZSUPSAT(JL) > ZEPSEC) THEN

      IF (ZTP1(JL,JK) >= RTHOMO) THEN
        ! Turn supersaturation into liquid water        
        ZSOLQA(JL,NCLDQL,NCLDQV) = ZSOLQA(JL,NCLDQL,NCLDQV)+ZSUPSAT(JL)
        ZSOLQA(JL,NCLDQV,NCLDQL) = ZSOLQA(JL,NCLDQV,NCLDQL)-ZSUPSAT(JL)
        ! Include liquid in first guess
        ZQXFG(JL,NCLDQL)=ZQXFG(JL,NCLDQL)+ZSUPSAT(JL)
      ELSE
        ! Turn supersaturation into ice water        
        ZSOLQA(JL,NCLDQI,NCLDQV) = ZSOLQA(JL,NCLDQI,NCLDQV)+ZSUPSAT(JL)
        ZSOLQA(JL,NCLDQV,NCLDQI) = ZSOLQA(JL,NCLDQV,NCLDQI)-ZSUPSAT(JL)
      ENDIF

      ! Increase cloud amount using RKOOPTAU timescale
      ZSOLAC(JL) = (1.0_JPRB-ZA(JL,JK))*ZFACI
 
      ! Store cloud budget diagnostics if required
      IF (LLCLDBUDL.AND. ZTP1(JL,JK)>=RTHOMO) ZBUDL(JL,1)=ZSUPSAT(JL)*ZQTMST
      IF (LLCLDBUDI.AND. ZTP1(JL,JK)<RTHOMO)  ZBUDI(JL,1)=ZSUPSAT(JL)*ZQTMST
      IF (LLCLDBUDCC) ZBUDCC(JL,1)=ZSOLAC(JL)*ZQTMST

    ENDIF

    !-------------------------------------------------------
    ! 3.1.3 Include supersaturation from previous timestep
    ! (Calculated in sltend if semi-lagrangian LDSLPHY=T)
    ! [#Note: Use RTT or RTICE or RTHOMO ?]
    !-------------------------------------------------------
   IF (LDMAINCALL) THEN  ! ***Only included if main call to cloudsc***
    
    IF( LDSLPHY ) THEN

      IF (PSUPSAT(JL,JK)>ZEPSEC) THEN
        IF (ZTP1(JL,JK) >= RTT) THEN
          ! Turn supersaturation into liquid water
          ZSOLQA(JL,NCLDQL,NCLDQL) = ZSOLQA(JL,NCLDQL,NCLDQL)+PSUPSAT(JL,JK)
          ZPSUPSATSRCE(JL,NCLDQL) = PSUPSAT(JL,JK)
          ! Store cloud budget diagnostics if required
          IF (LLCLDBUDL) ZBUDL(JL,2) = PSUPSAT(JL,JK)*ZQTMST
        ELSE
          ! Turn supersaturation into ice water
          ZSOLQA(JL,NCLDQI,NCLDQI) = ZSOLQA(JL,NCLDQI,NCLDQI)+PSUPSAT(JL,JK)
          ZPSUPSATSRCE(JL,NCLDQI) = PSUPSAT(JL,JK)
          ! Store cloud budget diagnostics if required
          IF (LLCLDBUDI) ZBUDI(JL,2) = PSUPSAT(JL,JK)*ZQTMST
        ENDIF

        ! Increase cloud amount using RKOOPTAU timescale
        ZSOLAC(JL)=(1.0_JPRB-ZA(JL,JK))*ZFACI
        ! Store cloud budget diagnostics if required
        IF (LLCLDBUDCC) ZBUDCC(JL,2)=ZSOLAC(JL)*ZQTMST

      ENDIF
    ENDIF

   ENDIF ! on LDMAINCALL ******************************

  ENDDO ! on JL

  !---------------------------------------------------------------------
  !  3.2  DETRAINMENT FROM CONVECTION
  !---------------------------------------------------------------------
  !
  ! * Diagnostic T-ice/liq split retained for convection
  !    Note: This link is now flexible and a future convection 
  !    scheme can detrain explicit seperate budgets of:
  !    cloud water, ice, rain and snow
  ! 
  ! * There is no (1-ZA) multiplier term on the cloud detrainment 
  !    term, since is now written in mass-flux terms  
  !
  ! [#Note: Should use ZFOEALFACU used in convection rather than ZFOEALFA]
  !---------------------------------------------------------------------
 IF (LDMAINCALL) THEN  ! ***Only included if main call to cloudsc***

  IF (JK < KLEV .AND. JK>=NCLDTOP) THEN

    DO JL=KIDIA,KFDIA
    
      PLUDE(JL,JK)=PLUDE(JL,JK)*ZDTGDP(JL)

      IF(LDCUM(JL).AND.PLUDE(JL,JK) > RLMIN.AND.PLU(JL,JK+1)> ZEPSEC) THEN
    
        ZSOLAC(JL)=ZSOLAC(JL)+PLUDE(JL,JK)/PLU(JL,JK+1)
        ! *diagnostic temperature split*
        ZALFAW               = ZFOEALFA(JL,JK)
        ZCONVSRCE(JL,NCLDQL) = ZALFAW*PLUDE(JL,JK)
        ZCONVSRCE(JL,NCLDQI) = (1.0_JPRB-ZALFAW)*PLUDE(JL,JK)
        ZSOLQA(JL,NCLDQL,NCLDQL) = ZSOLQA(JL,NCLDQL,NCLDQL)+ZCONVSRCE(JL,NCLDQL)
        ZSOLQA(JL,NCLDQI,NCLDQI) = ZSOLQA(JL,NCLDQI,NCLDQI)+ZCONVSRCE(JL,NCLDQI)
        
        ! Store cloud budget diagnostics if required
        IF (LLCLDBUDL) ZBUDL(JL,3)=ZCONVSRCE(JL,NCLDQL)*ZQTMST
        IF (LLCLDBUDI) ZBUDI(JL,3)=ZCONVSRCE(JL,NCLDQI)*ZQTMST
        IF (LLCLDBUDCC) ZBUDCC(JL,3)=ZQTMST*PLUDE(JL,JK)/PLU(JL,JK+1)

      ELSE

        PLUDE(JL,JK)=0.0_JPRB
    
      ENDIF
    
    ENDDO

  ENDIF ! JK<KLEV

 ENDIF ! on LDMAINCALL ******************************

  !---------------------------------------------------------------------
  !  3.3  SUBSIDENCE COMPENSATING CONVECTIVE UPDRAUGHTS
  !---------------------------------------------------------------------
  !
  ! Three terms:
  ! * Convective subsidence source of cloud from layer above
  ! * Evaporation of cloud within the layer
  ! * Subsidence sink of cloud to the layer below (Implicit solution)
  !
  !---------------------------------------------------------------------

 IF (LDMAINCALL) THEN  ! ***Only included if main call to cloudsc***

  !-----------------------------------------------
  ! Subsidence source from layer above
  !               and 
  ! Evaporation of cloud within the layer
  !-----------------------------------------------
  IF (JK > NCLDTOP) THEN

    DO JL=KIDIA,KFDIA
      ZMF(JL)=MAX(0.0_JPRB,(PMFU(JL,JK)+PMFD(JL,JK))*ZDTGDP(JL))
      ZACUST(JL)=ZMF(JL)*ZANEWM1(JL)
    ENDDO

    DO JM=1,NCLV
      IF (.NOT.LLFALL(JM).AND.IPHASE(JM)>0) THEN 
        DO JL=KIDIA,KFDIA
          ZLCUST(JL,JM)=ZMF(JL)*ZQXNM1(JL,JM)
          ! record total flux for enthalpy budget:
          ZCONVSRCE(JL,JM)=ZCONVSRCE(JL,JM)+ZLCUST(JL,JM)
        ENDDO
      ENDIF
    ENDDO

    ! Now have to work out how much liquid evaporates at arrival point 
    ! since there is no prognostic memory for in-cloud humidity, i.e. 
    ! we always assume cloud is saturated. 

    DO JL=KIDIA,KFDIA
      ZDTDP=ZRDCP*0.5_JPRB*(ZTP1(JL,JK-1)+ZTP1(JL,JK))/PAPH(JL,JK)
      ZDTFORC = ZDTDP*(PAP(JL,JK)-PAP(JL,JK-1))
      ![#Note: Diagnostic mixed phase should be replaced below]
      ZDQS(JL)=ZANEWM1(JL)*ZDTFORC*ZDQSMIXDT(JL)
    ENDDO

    DO JM=1,NCLV
      IF (.NOT.LLFALL(JM).AND.IPHASE(JM)>0) THEN 
        DO JL=KIDIA,KFDIA
          ZLFINAL=MAX(0.0_JPRB,ZLCUST(JL,JM)-ZDQS(JL)) !lim to zero
          ! no supersaturation allowed incloud ---V
          ZEVAP=MIN((ZLCUST(JL,JM)-ZLFINAL),ZEVAPLIMMIX(JL)) 
!          ZEVAP=0.0_JPRB
          ZLFINAL=ZLCUST(JL,JM)-ZEVAP 
          ZLFINALSUM(JL)=ZLFINALSUM(JL)+ZLFINAL ! sum 

          ZSOLQA(JL,JM,JM)     = ZSOLQA(JL,JM,JM)+ZLCUST(JL,JM) ! whole sum 
          ZSOLQA(JL,NCLDQV,JM) = ZSOLQA(JL,NCLDQV,JM)+ZEVAP
          ZSOLQA(JL,JM,NCLDQV) = ZSOLQA(JL,JM,NCLDQV)-ZEVAP
          ! Store cloud liquid diagnostic if required
          IF (LLCLDBUDL.AND.JM == NCLDQL) ZBUDL(JL,4)=ZLCUST(JL,JM)*ZQTMST
          IF (LLCLDBUDI.AND.JM == NCLDQI) ZBUDI(JL,4)=ZLCUST(JL,JM)*ZQTMST
          IF (LLCLDBUDL.AND.JM == NCLDQL) ZBUDL(JL,5)=-ZEVAP*ZQTMST
          IF (LLCLDBUDI.AND.JM == NCLDQI) ZBUDI(JL,5)=-ZEVAP*ZQTMST
        ENDDO
      ENDIF
    ENDDO

    !  Reset the cloud contribution if no cloud water survives to this level:
    DO JL=KIDIA,KFDIA
      IF (ZLFINALSUM(JL)<ZEPSEC) ZACUST(JL)=0.0_JPRB
      ZSOLAC(JL)=ZSOLAC(JL)+ZACUST(JL)
      ! Store cloud fraction diagnostic if required
      IF (LLCLDBUDCC) ZBUDCC(JL,4)=ZACUST(JL)*ZQTMST
      !ZBUDCC(JL,5) isn't included as only reduced if cloud->zero
    ENDDO

  ENDIF ! on  JK>NCLDTOP

  !---------------------------------------------------------------------
  ! Subsidence sink of cloud to the layer below 
  ! (Implicit - re. CFL limit on convective mass flux)
  !---------------------------------------------------------------------

  DO JL=KIDIA,KFDIA

    IF(JK<KLEV) THEN

      ZMFDN=MAX(0.0_JPRB,(PMFU(JL,JK+1)+PMFD(JL,JK+1))*ZDTGDP(JL))
      
      ZSOLAB(JL)=ZSOLAB(JL)+ZMFDN
      ZSOLQB(JL,NCLDQL,NCLDQL)=ZSOLQB(JL,NCLDQL,NCLDQL)+ZMFDN
      ZSOLQB(JL,NCLDQI,NCLDQI)=ZSOLQB(JL,NCLDQI,NCLDQI)+ZMFDN

      ! Record sink for cloud budget and enthalpy budget diagnostics
      ZCONVSINK(JL,NCLDQL) = ZMFDN
      ZCONVSINK(JL,NCLDQI) = ZMFDN

    ENDIF

  ENDDO

 ENDIF ! on LDMAINCALL ******************************

  !----------------------------------------------------------------------
  ! 3.4  EROSION OF CLOUDS BY TURBULENT MIXING
  !----------------------------------------------------------------------
  !
  ! NOTE: In default tiedtke scheme this process decreases the cloud 
  !       area but leaves the specific cloud water content
  !       within clouds unchanged
  !
  !----------------------------------------------------------------------

 IF (LDMAINCALL) THEN  ! ***Only included if main call to cloudsc***

  ! ------------------------------
  ! Define turbulent erosion rate
  ! ------------------------------
  DO JL=KIDIA,KFDIA
    ZLDIFDT(JL)=RCLDIFF*PTSPHY !original version
    !Increase by factor of 5 for convective points
    IF(KTYPE(JL) > 0 .AND. PLUDE(JL,JK) > ZEPSEC)&
! Increased cloud erosion in RCLDIFFC for tuning (default 5._JPRB)
       & ZLDIFDT(JL)=RCLDIFFC*ZLDIFDT(JL)  
  ENDDO

  ! At the moment, works on mixed RH profile and partitioned ice/liq fraction
  ! so that it is similar to previous scheme
  ! Should apply RHw for liquid cloud and RHi for ice cloud separately 
  DO JL=KIDIA,KFDIA
    IF(ZLI(JL,JK) > ZEPSEC) THEN
      ! Calculate environmental humidity
!      ZQE=(ZQX(JL,JK,NCLDQV)-ZA(JL,JK)*ZQSMIX(JL,JK))/&
!    &      MAX(ZEPSEC,1.0_JPRB-ZA(JL,JK))  
!      ZE=ZLDIFDT(JL)*MAX(ZQSMIX(JL,JK)-ZQE,0.0_JPRB)
      ZE=ZLDIFDT(JL)*MAX(ZQSMIX(JL,JK)-ZQX(JL,JK,NCLDQV),0.0_JPRB)
      ZLEROS=ZA(JL,JK)*ZE
      ZLEROS=MIN(ZLEROS,ZEVAPLIMMIX(JL))
      ZLEROS=MIN(ZLEROS,ZLI(JL,JK))
      ZAEROS=ZLEROS/ZLICLD(JL)  !if linear term

      ! Erosion is -ve LINEAR in L,A
      ZSOLAC(JL)=ZSOLAC(JL)-ZAEROS !linear

      ZSOLQA(JL,NCLDQV,NCLDQL) = ZSOLQA(JL,NCLDQV,NCLDQL)+ZLIQFRAC(JL,JK)*ZLEROS
      ZSOLQA(JL,NCLDQL,NCLDQV) = ZSOLQA(JL,NCLDQL,NCLDQV)-ZLIQFRAC(JL,JK)*ZLEROS
      ZSOLQA(JL,NCLDQV,NCLDQI) = ZSOLQA(JL,NCLDQV,NCLDQI)+ZICEFRAC(JL,JK)*ZLEROS
      ZSOLQA(JL,NCLDQI,NCLDQV) = ZSOLQA(JL,NCLDQI,NCLDQV)-ZICEFRAC(JL,JK)*ZLEROS

      ! Store cloud budget diagnostics if required
      IF (LLCLDBUDL) ZBUDL(JL,7)=-ZLIQFRAC(JL,JK)*ZLEROS*ZQTMST
      IF (LLCLDBUDI) ZBUDI(JL,7)=-ZICEFRAC(JL,JK)*ZLEROS*ZQTMST
      IF (LLCLDBUDCC) ZBUDCC(JL,7)=-ZAEROS*ZQTMST

    ENDIF
  ENDDO

 ENDIF ! on LDMAINCALL ******************************

  !----------------------------------------------------------------------
  ! 3.4  CONDENSATION/EVAPORATION DUE TO DQSAT/DT
  !----------------------------------------------------------------------
  !
  !  calculate dqs/dt
  !
  !  Note: For the separate prognostic Qi and Ql, one would ideally use
  !  Qsat/DT wrt liquid/Koop here, since the physics is that new clouds
  !  forms by liquid droplets [liq] or when aqueous aerosols [Koop] form.
  !  These would then instantaneous freeze if T<-38C or lead to ice growth 
  !  by deposition in warmer mixed phase clouds.  However, since we do 
  !  not have a separate prognostic equation for in-cloud humidity or a 
  !  statistical scheme approach in place, the depositional growth of ice 
  !  in the mixed phase can not be modelled and we resort to supersaturation  
  !  wrt ice instanteously converting to ice over one timestep 
  !  (see Tompkins et al. QJRMS 2007 for details)
  !  Thus for the initial implementation the diagnostic mixed phase is 
  !  retained for the moment, and the level of approximation noted.  
  !  
  !----------------------------------------------------------------------

  DO JL=KIDIA,KFDIA
    ZDTDP   = ZRDCP*ZTP1(JL,JK)/PAP(JL,JK)
    ZDPMXDT = ZDP(JL)*ZQTMST
    ZMFDN   = 0.0_JPRB
    IF(JK < KLEV) ZMFDN=PMFU(JL,JK+1)+PMFD(JL,JK+1)
    ZWTOT   = PVERVEL(JL,JK)+0.5_JPRB*RG*(PMFU(JL,JK)+PMFD(JL,JK)+ZMFDN)
    ZWTOT   = MIN(ZDPMXDT,MAX(-ZDPMXDT,ZWTOT))
    ZZZDT   = PHRSW(JL,JK)+PHRLW(JL,JK)
    ZDTDIAB = MIN(ZDPMXDT*ZDTDP,MAX(-ZDPMXDT*ZDTDP,ZZZDT))&
                    & *PTSPHY+RALFDCP*ZLDEFR(JL)  
! Note: ZLDEFR should be set to the difference between the mixed phase functions
! in the convection and cloud scheme, but this is not calculated, so is zero and
! the functions must be the same!
    ZDTFORC = ZDTDP*ZWTOT*PTSPHY+ZDTDIAB
    ZQOLD(JL)   = ZQSMIX(JL,JK)
    ZTOLD(JL)   = ZTP1(JL,JK)
    ZTP1(JL,JK) = ZTP1(JL,JK)+ZDTFORC
    ZTP1(JL,JK) = MAX(ZTP1(JL,JK),160.0_JPRB)
    LLFLAG(JL)  = .TRUE.
  ENDDO

  IK=JK
  ICALL=5
  CALL CUADJTQ &
   & (KIDIA,    KFDIA,   KLON,     KLEV,    IK,&
   & PAP(KIDIA,JK), ZTP1,    ZQSMIX,   LLFLAG,  ICALL)  

  DO JL=KIDIA,KFDIA
    ZDQS(JL)      = ZQSMIX(JL,JK)-ZQOLD(JL)
    ZQSMIX(JL,JK) = ZQOLD(JL)
    ZTP1(JL,JK)   = ZTOLD(JL)
  ENDDO

  !----------------------------------------------------------------------
  ! 3.4a  ZDQS(JL) > 0:  EVAPORATION OF CLOUDS
  ! ----------------------------------------------------------------------
  ! Erosion term is LINEAR in L
  ! Changed to be uniform distribution in cloud region

  DO JL=KIDIA,KFDIA

    ! Previous function based on DELTA DISTRIBUTION in cloud:
   IF (ZDQS(JL) > 0.0_JPRB) THEN
!    If subsidence evaporation term is turned off, then need to use updated
!    liquid and cloud here?
!    ZLEVAP = MAX(ZA(JL,JK)+ZACUST(JL),1.0_JPRB)*MIN(ZDQS(JL),ZLICLD(JL)+ZLFINALSUM(JL))
    ZLEVAP = ZA(JL,JK)*MIN(ZDQS(JL),ZLICLD(JL))
    ZLEVAP = MIN(ZLEVAP,ZEVAPLIMMIX(JL))
    ZLEVAP = MIN(ZLEVAP,MAX(ZQSMIX(JL,JK)-ZQX(JL,JK,NCLDQV),0.0_JPRB))

    ! For first guess call
    ZLEVAPL(JL) = ZLIQFRAC(JL,JK)*ZLEVAP
    ZLEVAPI(JL) = ZICEFRAC(JL,JK)*ZLEVAP

    ZSOLQA(JL,NCLDQV,NCLDQL) = ZSOLQA(JL,NCLDQV,NCLDQL)+ZLIQFRAC(JL,JK)*ZLEVAP
    ZSOLQA(JL,NCLDQL,NCLDQV) = ZSOLQA(JL,NCLDQL,NCLDQV)-ZLIQFRAC(JL,JK)*ZLEVAP

    ZSOLQA(JL,NCLDQV,NCLDQI) = ZSOLQA(JL,NCLDQV,NCLDQI)+ZICEFRAC(JL,JK)*ZLEVAP
    ZSOLQA(JL,NCLDQI,NCLDQV) = ZSOLQA(JL,NCLDQI,NCLDQV)-ZICEFRAC(JL,JK)*ZLEVAP

    ! Store cloud budget diagnostics if required
    IF (LLCLDBUDL) ZBUDL(JL,8)=-ZLIQFRAC(JL,JK)*ZLEVAP*ZQTMST
    IF (LLCLDBUDI) ZBUDI(JL,8)=-ZICEFRAC(JL,JK)*ZLEVAP*ZQTMST

   ENDIF

  ENDDO

  !----------------------------------------------------------------------
  ! 3.4b ZDQS(JL) < 0: FORMATION OF CLOUDS
  !----------------------------------------------------------------------
  !
  ! (1) Increase of cloud water in existing clouds
  !
  DO JL=KIDIA,KFDIA
    IF(ZA(JL,JK) > ZEPSEC.AND.ZDQS(JL) <= -RLMIN) THEN
      ZLCOND1(JL)=MAX(-ZDQS(JL),0.0_JPRB) !new limiter

!old limiter (significantly improves upper tropospheric humidity rms)
      IF(ZA(JL,JK) > 0.99_JPRB) THEN
        ZCOR=1.0_JPRB/(1.0_JPRB-RETV*ZQSMIX(JL,JK))
        ZCDMAX=(ZQX(JL,JK,NCLDQV)-ZQSMIX(JL,JK))/&
         & (1.0_JPRB+ZCOR*ZQSMIX(JL,JK)*FOEDEM(ZTP1(JL,JK)))  
      ELSE
        ZCDMAX=(ZQX(JL,JK,NCLDQV)-ZA(JL,JK)*ZQSMIX(JL,JK))/ZA(JL,JK)
      ENDIF
      ZLCOND1(JL)=MAX(MIN(ZLCOND1(JL),ZCDMAX),0.0_JPRB)
! end old limiter

      ZLCOND1(JL)=ZA(JL,JK)*ZLCOND1(JL)
      IF(ZLCOND1(JL) < RLMIN) ZLCOND1(JL)=0.0_JPRB

      !-------------------------------------------------------------------------
      ! All increase goes into liquid unless so cold cloud homogeneously freezes
      ! Include new liquid formation in first guess value, otherwise liquid 
      ! remains at cold temperatures until next timestep.
      !-------------------------------------------------------------------------
      IF (ZTP1(JL,JK)>RTHOMO) THEN
        ZSOLQA(JL,NCLDQL,NCLDQV)=ZSOLQA(JL,NCLDQL,NCLDQV)+ZLCOND1(JL)
        ZSOLQA(JL,NCLDQV,NCLDQL)=ZSOLQA(JL,NCLDQV,NCLDQL)-ZLCOND1(JL)
        ZQXFG(JL,NCLDQL)=ZQXFG(JL,NCLDQL)+ZLCOND1(JL)
        ! Store cloud liquid diagnostic if required
        IF (LLCLDBUDL) ZBUDL(JL,9)=ZLCOND1(JL)*ZQTMST
      ELSE
        ZSOLQA(JL,NCLDQI,NCLDQV)=ZSOLQA(JL,NCLDQI,NCLDQV)+ZLCOND1(JL)
        ZSOLQA(JL,NCLDQV,NCLDQI)=ZSOLQA(JL,NCLDQV,NCLDQI)-ZLCOND1(JL)
        ZQXFG(JL,NCLDQI)=ZQXFG(JL,NCLDQI)+ZLCOND1(JL)
        ! Store cloud ice diagnostic if required
        IF (LLCLDBUDI) ZBUDI(JL,9)=ZLCOND1(JL)*ZQTMST
      ENDIF
    ENDIF
  ENDDO

  !
  ! (2) Generation of new clouds (da/dt>0)
  !
  
  DO JL=KIDIA,KFDIA

    IF(ZDQS(JL) <= -RLMIN .AND. ZA(JL,JK)<1.0_JPRB-ZEPSEC) THEN

      !---------------------------
      ! Critical relative humidity
      !---------------------------
      ZRHC=RAMID
      ZSIGK=PAP(JL,JK)/PAPH(JL,KLEV+1)
      ! Increase RHcrit to 1.0 towards the surface (eta>0.8)
      IF(ZSIGK > 0.8_JPRB) THEN
        ZRHC=RAMID+(1.0_JPRB-RAMID)*((ZSIGK-0.8_JPRB)/0.2_JPRB)**2
      ENDIF

      ! Increase RHcrit to 1.0 towards the tropopause (trop-0.2) and above
      ZBOTT=ZTRPAUS(JL)+0.2_JPRB
      IF(ZSIGK < ZBOTT) THEN
        ZRHC=RAMID+(1.0_JPRB-RAMID)*MIN(((ZBOTT-ZSIGK)/0.2_JPRB)**2,1.0_JPRB)
      ENDIF

      !---------------------------
      ! Supersaturation options
      !---------------------------      
      IF (NSSOPT==0) THEN 
        ! No scheme
        ZQE=(ZQX(JL,JK,NCLDQV)-ZA(JL,JK)*ZQSICE(JL,JK))/&
            & MAX(ZEPSEC,1.0_JPRB-ZA(JL,JK))  
        ZQE=MAX(0.0_JPRB,ZQE)
      ELSEIF (NSSOPT==1) THEN 
        ! Tompkins 
        ZQE=(ZQX(JL,JK,NCLDQV)-ZA(JL,JK)*ZQSICE(JL,JK))/&
            & MAX(ZEPSEC,1.0_JPRB-ZA(JL,JK))  
        ZQE=MAX(0.0_JPRB,ZQE)
      ELSEIF (NSSOPT==2) THEN 
        ! Lohmann and Karcher
        ZQE=ZQX(JL,JK,NCLDQV)  
      ELSEIF (NSSOPT==3) THEN 
        ! Gierens
        ZQE=ZQX(JL,JK,NCLDQV)+ZLI(JL,JK)
      ENDIF

      IF (ZTP1(JL,JK)>=RTT .OR. NSSOPT==0) THEN 
        ! No ice supersaturation allowed
        ZFAC=1.0_JPRB        
      ELSE
        ! Ice supersaturation
        ZFAC=ZFOKOOP(JL)
      ENDIF

      IF(ZQE >= ZRHC*ZQSICE(JL,JK)*ZFAC.AND.ZQE<ZQSICE(JL,JK)*ZFAC) THEN
        ! note: not **2 on 1-a term if ZQE is used. 
        ! Added correction term ZFAC to numerator 15/03/2010
        ZACOND=-(1.0_JPRB-ZA(JL,JK))*ZFAC*ZDQS(JL)/&
         &MAX(2.0_JPRB*(ZFAC*ZQSICE(JL,JK)-ZQE),ZEPSEC)

        ZACOND=MIN(ZACOND,1.0_JPRB-ZA(JL,JK))  !PUT THE LIMITER BACK

        ! Linear term:
        ! Added correction term ZFAC 15/03/2010
        ZLCOND2(JL)=-ZFAC*ZDQS(JL)*0.5_JPRB*ZACOND !mine linear

        ! new limiter formulation
        ZZDL=2.0_JPRB*(ZFAC*ZQSICE(JL,JK)-ZQE)/MAX(ZEPSEC,1.0_JPRB-ZA(JL,JK))
        ! Added correction term ZFAC 15/03/2010
        IF (ZFAC*ZDQS(JL)<-ZZDL) THEN
          ! ZLCONDLIM=(ZA(JL,JK)-1.0_JPRB)*ZDQS(JL)-ZQSICE(JL,JK)+ZQX(JL,JK,NCLDQV)
          ZLCONDLIM=(ZA(JL,JK)-1.0_JPRB)*ZFAC*ZDQS(JL)- &
     &               ZFAC*ZQSICE(JL,JK)+ZQX(JL,JK,NCLDQV)
          ZLCOND2(JL)=MIN(ZLCOND2(JL),ZLCONDLIM)
        ENDIF
        ZLCOND2(JL)=MAX(ZLCOND2(JL),0.0_JPRB)

        IF(ZLCOND2(JL) < RLMIN .OR. (1.0_JPRB-ZA(JL,JK))<ZEPSEC ) THEN
          ZLCOND2(JL) = 0.0_JPRB
          ZACOND      = 0.0_JPRB
        ENDIF
        IF(ZLCOND2(JL) == 0.0_JPRB) ZACOND=0.0_JPRB

        ! Large-scale generation is LINEAR in A and LINEAR in L
        ZSOLAC(JL) = ZSOLAC(JL)+ZACOND !linear
        
        ! Store cloud fraction diagnostic if required
        IF (LLCLDBUDCC) ZBUDCC(JL,10)=ZACOND*ZQTMST

        !------------------------------------------------------------------------
        ! All increase goes into liquid unless so cold cloud homogeneously freezes
        ! Include new liquid formation in first guess value, otherwise liquid 
        ! remains at cold temperatures until next timestep.
        !------------------------------------------------------------------------
        IF (ZTP1(JL,JK)>RTHOMO) THEN
          ZSOLQA(JL,NCLDQL,NCLDQV)=ZSOLQA(JL,NCLDQL,NCLDQV)+ZLCOND2(JL)
          ZSOLQA(JL,NCLDQV,NCLDQL)=ZSOLQA(JL,NCLDQV,NCLDQL)-ZLCOND2(JL)
          ZQXFG(JL,NCLDQL)=ZQXFG(JL,NCLDQL)+ZLCOND2(JL)
          ! Store cloud liquid diagnostic if required
          IF (LLCLDBUDL) ZBUDL(JL,10)=ZLCOND2(JL)*ZQTMST
        ELSE ! homogeneous freezing
          ZSOLQA(JL,NCLDQI,NCLDQV)=ZSOLQA(JL,NCLDQI,NCLDQV)+ZLCOND2(JL)
          ZSOLQA(JL,NCLDQV,NCLDQI)=ZSOLQA(JL,NCLDQV,NCLDQI)-ZLCOND2(JL)
          ZQXFG(JL,NCLDQI)=ZQXFG(JL,NCLDQI)+ZLCOND2(JL)
          ! Store cloud ice diagnostic if required
          IF (LLCLDBUDI) ZBUDI(JL,10)=ZLCOND2(JL)*ZQTMST
        ENDIF

      ENDIF
    ENDIF
  ENDDO

  !----------------------------------------------------------------------
  !
  ! 3.7 Growth of ice by vapour deposition 
  !
  !----------------------------------------------------------------------
  ! Following Rotstayn et al. 2001:
  ! does not use the ice nuclei number from cloudaer.F90
  ! but rather a simple Meyers et al. 1992 form based on the 
  ! supersaturation and assuming clouds are saturated with 
  ! respect to liquid water (well mixed), (or Koop adjustment)
  ! Growth considered as sink of liquid water if present so 
  ! Bergeron-Findeisen adjustment in autoconversion term no longer needed
  !----------------------------------------------------------------------

 IF (LDMAINCALL) THEN  ! ***Only included if main call to cloudsc***

  DO JL=KIDIA,KFDIA
    !--------------------------------------------------------------
    ! only treat depositional growth if liquid present. due to fact 
    ! that can not model ice growth from vapour without additional 
    ! in-cloud water vapour variable
    !--------------------------------------------------------------
    IF (ZTP1(JL,JK)<RTT .AND. ZQXFG(JL,NCLDQL)>RLMIN) THEN  ! T<273K
      ZVPICE=FOEEICE(ZTP1(JL,JK))
      ZVPLIQ=ZVPICE*ZFOKOOP(JL) 
      ZICENUCLEI(JL)=1000.0_JPRB*EXP(12.96_JPRB*(ZVPLIQ-ZVPICE)/ZVPLIQ-0.639_JPRB)

      !------------------------------------------------
      !   2.4e-2 is conductivity of air
      !   8.8 = 700**1/3 = density of ice to the third
      !------------------------------------------------
      ZADD=RLSTT*(RLSTT/(RV*ZTP1(JL,JK))-1.0_JPRB)/(2.4E-2_JPRB*ZTP1(JL,JK))
      ZBDD=RV*ZTP1(JL,JK)*PAP(JL,JK)/(2.21_JPRB*ZVPICE)
      ZCVDS=7.8_JPRB*(ZICENUCLEI(JL)/ZRHO(JL))**0.666_JPRB*(ZVPLIQ-ZVPICE) / &
         & (8.87_JPRB*(ZADD+ZBDD)*ZVPICE)

      !-----------------------------------------------------
      ! RICEINIT=1.E-12_JPRB is initial mass of ice particle
      !-----------------------------------------------------
      ZICE0=MAX(ZICECLD(JL), ZICENUCLEI(JL)*RICEINIT/ZRHO(JL))

      !------------------
      ! new value of ice:
      !------------------
      ZINEW=(0.666_JPRB*ZCVDS*PTSPHY+ZICE0**0.666_JPRB)**1.5_JPRB

      !---------------------------
      ! grid-mean deposition rate:
      !--------------------------- 
      ZDEPOS=MAX(ZA(JL,JK)*(ZINEW-ZICE0),0.0_JPRB)*2.0_JPRB
      ! Above increased by factor of 2 to retain similar mixed phase liq 
      ! as in diagnostic scheme

      !--------------------------------------------------------------------
      ! Limit deposition to liquid water amount
      ! If liquid is all frozen, ice would use up reservoir of water 
      ! vapour in excess of ice saturation mixing ratio - However this 
      ! can not be represented without a in-cloud humidity variable. Using 
      ! the grid-mean humidity would imply a large artificial horizontal 
      ! flux from the clear sky to the cloudy area. We thus rely on the 
      ! supersaturation check to clean up any remaining supersaturation
      !--------------------------------------------------------------------
      ZDEPOS=MIN(ZDEPOS,ZQXFG(JL,NCLDQL)) ! limit to liquid water amount

      !--------------
      ! add to matrix 
      !--------------
      ZSOLQA(JL,NCLDQI,NCLDQL)=ZSOLQA(JL,NCLDQI,NCLDQL)+ZDEPOS
      ZSOLQA(JL,NCLDQL,NCLDQI)=ZSOLQA(JL,NCLDQL,NCLDQI)-ZDEPOS
      ZQXFG(JL,NCLDQI)=ZQXFG(JL,NCLDQI)+ZDEPOS
      ZQXFG(JL,NCLDQL)=ZQXFG(JL,NCLDQL)-ZDEPOS
      ! Store cloud budget diagnostics if required
      IF (LLCLDBUDL) ZBUDL(JL,11)=-ZDEPOS*ZQTMST
      IF (LLCLDBUDI) ZBUDI(JL,11)=ZDEPOS*ZQTMST

    ENDIF
  ENDDO
 
 ENDIF ! on LDMAINCALL ******************************
 
  !######################################################################
  !
  !              4  *** PRECIPITATION PROCESSES ***
  !
  !######################################################################

  !----------------------------------
  ! revise in-cloud consensate amount
  !----------------------------------
  DO JL=KIDIA,KFDIA
    ZTMPA = 1.0_JPRB/MAX(ZA(JL,JK),ZEPSEC)
    ZLIQCLD(JL) = ZQXFG(JL,NCLDQL)*ZTMPA
    ZICECLD(JL) = ZQXFG(JL,NCLDQI)*ZTMPA
    ZLICLD(JL)  = ZLIQCLD(JL)+ZICECLD(JL)
  ENDDO

  IF( N_VMASS > 0 )THEN
    DO JL = KIDIA,KFDIA
      ZTMP3(JL-KIDIA+1) = PAP(JL,JK)*RICEHI1
      ZTMP4(JL-KIDIA+1) = ZTP1(JL,JK)*RICEHI2
    ENDDO
    CALL VPOW(ZTMP5,ZTMP3,ZTMP1,KFDIA-KIDIA+1)
    CALL VPOW(ZTMP6,ZTMP4,ZTMP2,KFDIA-KIDIA+1)
    DO JL = KIDIA,KFDIA
      ZTMP7(JL-KIDIA+1) = ZTMP5(JL-KIDIA+1)*ZTMP6(JL-KIDIA+1)
    ENDDO
  ENDIF



  !----------------------------------------------------------------------
  ! 4.2 SEDIMENTATION/FALLING OF *ALL* MICROPHYSICAL SPECIES
  !     now that rain, snow, graupel species are prognostic
  !     the precipitation flux can be defined directly level by level
  !     There is no vertical memory required from the flux variable
  !----------------------------------------------------------------------

 IF (LDMAINCALL) THEN  ! ***Only included if main call to cloudsc***

  DO JM = 1,NCLV
    IF (LLFALL(JM) .OR. JM == NCLDQI) THEN
      DO JL=KIDIA,KFDIA
        !------------------------
        ! source from layer above 
        !------------------------
        IF (JK > NCLDTOP) THEN
          ZFALLSRCE(JL,JM) = ZPFPLSX(JL,JK,JM)*ZDTGDP(JL) 
          ZSOLQA(JL,JM,JM) = ZSOLQA(JL,JM,JM)+ZFALLSRCE(JL,JM)
          ZQXFG(JL,JM)     = ZQXFG(JL,JM)+ZFALLSRCE(JL,JM)
          ! use first guess precip----------V
          ZQPRETOT(JL)     = ZQPRETOT(JL)+ZQXFG(JL,JM) 
          IF (LLCLDBUDI .AND. JM == NCLDQI) THEN
            ZBUDI(JL,12)=ZFALLSRCE(JL,JM)*ZQTMST
          ENDIF
        ENDIF
        !-------------------------------------------------
        ! sink to next layer, constant fall speed
        !-------------------------------------------------
        ! if aerosol effect then override 
        !  note that for T>233K this is the same as above.
        IF (LAERICESED .AND. JM == NCLDQI) THEN
          ZRE_ICE=PRE_ICE(JL,JK) 
          ! The exponent value is from 
          ! Morrison et al. JAS 2005 Appendix
          ZVQX(NCLDQI) = 0.002_JPRB*ZRE_ICE**1.0_JPRB
        ENDIF
        ZFALL=ZVQX(JM)*ZRHO(JL)
        !-------------------------------------------------
        ! modified by Heymsfield and Iaquinta JAS 2000
        !-------------------------------------------------
        IF( N_VMASS > 0 )THEN
          ZFALL = ZFALL*ZTMP7(JL-KIDIA+1)
        ELSE
          ZFALL = ZFALL*((PAP(JL,JK)*RICEHI1)**(-0.178_JPRB)) &
                     &*((ZTP1(JL,JK)*RICEHI2)**(-0.394_JPRB))
        ENDIF

        ZFALLSINK(JL,JM)=ZDTGDP(JL)*ZFALL
        ! Cloud budget diagnostic stored at end as implicit
      ENDDO ! jl  
    ENDIF ! LLFALL
  ENDDO ! jm

 ENDIF ! on LDMAINCALL ******************************

  !---------------------------------------------------------------
  ! Precip cover overlap using MAX-RAN Overlap
  ! Since precipitation is now prognostic we must 
  !   1) apply an arbitrary minimum coverage (0.3) if precip>0
  !   2) abandon the 2-flux clr/cld treatment
  !   3) Thus, since we have no memory of the clear sky precip
  !      fraction, we mimic the previous method by reducing 
  !      ZCOVPTOT(JL), which has the memory, proportionally with 
  !      the precip evaporation rate, taking cloud fraction 
  !      into account
  !
  !   #3 above leads to much smoother vertical profiles of 
  !   precipitation fraction than the Klein-Jakob scheme which 
  !   monotonically increases precip fraction and then resets 
  !   it to zero in a step function once clear-sky precip reaches
  !   zero.
  !---------------------------------------------------------------
  DO JL=KIDIA,KFDIA
!J---Start---
!J  ZCOVPTOTM1=ZCOVPTOT(JL)
!J  ZCOVPTOT(JL) = 1.0_JPRB - ((1.0_JPRB-ZCOVPTOT(JL))*&
!J   & (1.0_JPRB - MAX(ZA(JL,JK),ZA(JL,JK-1)))/&
!J   & (1.0_JPRB - MIN(ZA(JL,JK-1),1.0_JPRB-1.E-06_JPRB)) )  
!J  IF (ZQPRETOT(JL)>ZEPSEC) THEN
!J    ZCOVPTOT(JL)=MAX(ZCOVPTOT(JL),RCOVPMIN)
!J  ELSE
!J    ZCOVPTOT(JL)=0.0_JPRB ! no flux - reset cover
!J  ENDIF
!J  ZCOVPCLR(JL)=MAX(0.0_JPRB,ZCOVPTOT(JL)-ZA(JL,JK)) ! clear sky proportion
!J  ! Changing above to ZCOVPCLR(JL)=ZCOVPTOT(JL) didn't have much impact
    IF (ZQPRETOT(JL)>ZEPSEC) THEN
      ZCOVPTOT(JL) = 1.0_JPRB - ((1.0_JPRB-ZCOVPTOT(JL))*&
       & (1.0_JPRB - MAX(ZA(JL,JK),ZA(JL,JK-1)))/&
       & (1.0_JPRB - MIN(ZA(JL,JK-1),1.0_JPRB-1.E-06_JPRB)) )  
      ZCOVPTOT(JL)=MAX(ZCOVPTOT(JL),RCOVPMIN)
      ZCOVPCLR(JL)=MAX(0.0_JPRB,ZCOVPTOT(JL)-ZA(JL,JK)) ! clear sky proportion
    ELSE
      ZCOVPTOT(JL)=0.0_JPRB ! no flux - reset cover
!!    ZCOVPCLR(JL)=MAX(0.0_JPRB,-ZA(JL,JK)) ! clear sky proportion
      ZCOVPCLR(JL)=0.0_JPRB   ! reset clear sky proportion 
    ENDIF
    ! Changing above to ZCOVPCLR(JL)=ZCOVPTOT(JL) didn't have much impact
!J---End---
  ENDDO


 IF (LDMAINCALL) THEN  ! ***Only included if main call to cloudsc***
  
  !----------------------------------------------------------------------
  ! 4.3a AUTOCONVERSION TO SNOW
  !----------------------------------------------------------------------
  DO JL=KIDIA,KFDIA
    IF(ZTP1(JL,JK) <= RTT) THEN
  !-----------------------------------------------------
  !     Snow Autoconversion rate follow Lin et al. 1983
  !-----------------------------------------------------
      IF (ZICECLD(JL)>ZEPSEC) THEN
        ZZCO=PTSPHY*RSNOWLIN1*EXP(RSNOWLIN2*(ZTP1(JL,JK)-RTT))

        IF (LAERICEAUTO) THEN

          !>>dod moved from aer_cld and aer_clcld
          !---------------------------------------------------------------------
          ! Turn IN Number concentration for ice into critical 
          ! mixing ratio for autoconversion process
          ! Obvious, but also from Rotstayn and penner JAS 2001
          !---------------------------------------------------------------------
             
          ! Number is 1.e6. 4/3 .pi.rho_l. r_crit**3
          ! 1e6 since IN in units of  N cm**-3
          ! r_crit=9.3 microns
          ZICRIT=1.333E6_JPRB*RPI*PPRHO_ICE*PICNC(JL,JK)*PPRICE_CRIT**3.0_JPRB/ZRHO(JL) 

          ! limit the effect to ratio of the "background" value
          ZICRIT=MAX(ZICRIT,0.1_JPRB*RLCRITSNOW)
          ZICRIT=MIN(ZICRIT,10.0_JPRB*RLCRITSNOW)
          !<<dod

          ! 0.3 = N**0.333 with N=0.027 
          ZZCO=ZZCO*(RNICE/PICNC(JL,JK))**0.333_JPRB
        ELSE
          ZICRIT=RLCRITSNOW
        ENDIF

         !-------------------------------------------------------
         ! parameters for cloud collection by rain and snow drops
         !-------------------------------------------------------
!        IF (ZTP1(JL,JK)>RTICE) THEN ! fudge
!          ZPRECIP=(ZPFPLSX(JL,JK,NCLDQS)+ZPFPLSX(JL,JK,NCLDQR))&
!           & /MAX(ZEPSEC,ZCOVPTOT(JL))
!          ZCFPR=1.0_JPRB+RPRC1*SQRT(MAX(ZPRECIP,0.0_JPRB))*&
!           & ZCOVPTOT(JL)/(MAX(ZA(JL,JK),ZEPSEC))
!          ZZCO=ZZCO*ZCFPR
!          ZLCRIT=ZLCRIT/MAX(ZCFPR,ZEPSEC)
!        ENDIF

        ZSNOWAUT(JL)=ZZCO*(1.0_JPRB-EXP(-(ZICECLD(JL)/ZICRIT)**2))
        ZSOLQB(JL,NCLDQS,NCLDQI)=ZSOLQB(JL,NCLDQS,NCLDQI)+ZSNOWAUT(JL)

        
   ENDIF
  ENDIF !
  
  !----------------------------------------------------------------------
  ! 4.3b AUTOCONVERSION WARM CLOUDS
  !   Collection and accretion will require separate treatment
  !   but for now we keep this simple treatment
  !----------------------------------------------------------------------

  !------------------------------------------------
  ! Rain Autoconversion rate follow Sundqvist 1989
  !------------------------------------------------
    IF (ZLIQCLD(JL)>ZEPSEC) THEN
      ZZCO=RKCONV*PTSPHY

      IF (LAERLIQAUTOLSP) THEN
        !---------------------------------------------------------------------
        ! Turn CCN Number concentration for warm rain into critical 
        ! mixing ratio for autoconversion process
        ! Obvious, but also from Rotstayn and penner JAS 2001
        !---------------------------------------------------------------------
         
        ! Number is 1.e6. 4/3 .pi.rho_l. r_crit**3
        ! 1e6 since CCN in units of  N cm**-3
        ! r_crit=9.3 microns

        ZLCRIT=1.333E6_JPRB*RPI*PPRHO_WAT*PCDNC(JL,JK)*RLCRIT_UPHYS**3/ZRHO(JL)
        !      PLCRIT_AER(JL,JK)=3.36e-6_JPRB*PCCN(JL,JK)/ZRHO(JL,JK)
        !      IF (KFLDX>0 .AND. LDUMPEXTRA) PEXTRA(JL,JK,10)=PLCRIT_AER(JL,JK)
        
        ! limit the effect to ratio of the "background" value
        ZLCRIT=MAX(ZLCRIT,0.1_JPRB*RCLCRIT)
        ZLCRIT=MIN(ZLCRIT,10.0_JPRB*RCLCRIT)
        ! 0.3 = N**0.333 with N=125 cm-3 
        ZZCO=ZZCO*(RCCN/PCDNC(JL,JK))**0.333_JPRB
      ELSE
        ZLCRIT=RCLCRIT

        ! Modify RCLCRIT dependent on land or ocean
        ! Need to put these in to sucldp 
        IF (PLSM(JL) > 0.5_JPRB) THEN
          ZLCRIT=5.E-4_JPRB ! land
        ELSE
          ZLCRIT=3.E-4_JPRB ! ocean
        ENDIF
      ENDIF
        
      !------------------------------------------------------------------
      ! Parameters for cloud collection by rain and snow.
      ! Note that with new prognostic variable it is now possible 
      ! to REPLACE this with an explicit collection parametrization
      !------------------------------------------------------------------   
      ZPRECIP=(ZPFPLSX(JL,JK,NCLDQS)+ZPFPLSX(JL,JK,NCLDQR))/MAX(ZEPSEC,ZCOVPTOT(JL))
      ZCFPR=1.0_JPRB+RPRC1*SQRT(MAX(ZPRECIP,0.0_JPRB))
!      ZCFPR=1.0_JPRB*RPRC1*SQRT(MAX(ZPRECIP,0.0_JPRB))*&
!       &ZCOVPTOT(JL)/(MAX(ZA(JL,JK),ZEPSEC))

      IF (LAERLIQCOLL .AND. (.NOT. LAERLIQAUTOLSP)) THEN 
        ! 5.0 = N**0.333 with N=125 cm-3 
        ZCFPR=ZCFPR*(RCCN/PCDNC(JL,JK))**0.333_JPRB
      ENDIF

      ZZCO=ZZCO*ZCFPR
      ZLCRIT=ZLCRIT/MAX(ZCFPR,ZEPSEC)
  
      IF(ZLIQCLD(JL)/ZLCRIT < 20.0_JPRB )THEN ! Security for exp for some compilers
        ZRAINAUT(JL)=ZZCO*(1.0_JPRB-EXP(-(ZLIQCLD(JL)/ZLCRIT)**2))
      ELSE
        ZRAINAUT(JL)=ZZCO
      ENDIF

      !-----------------------
      ! rain freezes instantly
      !-----------------------
      IF(ZTP1(JL,JK) <= RTT) THEN
        ZSOLQB(JL,NCLDQS,NCLDQL)=ZSOLQB(JL,NCLDQS,NCLDQL)+ZRAINAUT(JL)
      ELSE
        ZSOLQB(JL,NCLDQR,NCLDQL)=ZSOLQB(JL,NCLDQR,NCLDQL)+ZRAINAUT(JL)
      ENDIF

   ENDIF
 ENDDO

  !----------------------------------------------------------------------
  ! 4.4a  MELTING OF SNOW and ICE
  !       with new implicit solver this also has to treat snow or ice
  !       precipitating from the level above... i.e. local ice AND flux.
  !       in situ ice and snow: could arise from LS advection or warming
  !       falling ice and snow: arrives by precipitation process
  !----------------------------------------------------------------------
  DO JL=KIDIA,KFDIA
    
    ZICETOT(JL)=ZQXFG(JL,NCLDQI)+ZQXFG(JL,NCLDQS)
    ZMELTMAX(JL) = 0.0_JPRB

    ! If there are frozen hydrometeors present and dry-bulb temperature > 0degC
    IF(ZICETOT(JL) > ZEPSEC .AND. ZTP1(JL,JK) > RTT) THEN

      ! Calculate subsaturation
      ZSUBSAT = MAX(ZQSICE(JL,JK)-ZQX(JL,JK,NCLDQV),0.0_JPRB)
      
      ! Calculate difference between dry-bulb (ZTP1) and the temperature 
      ! at which the wet-bulb=0degC (RTT-ZSUBSAT*....) using an approx.
      ! Melting only occurs if the wet-bulb temperature >0
      ! i.e. warming of ice particle due to melting > cooling 
      ! due to evaporation.
      ZTDMTW0 = ZTP1(JL,JK)-RTT-ZSUBSAT* &
                & (ZTW1+ZTW2*(PAP(JL,JK)-ZTW3)-ZTW4*(ZTP1(JL,JK)-ZTW5))
      ! Not implicit yet... 
      ZCONS1   = ABS(PTSPHY*(1.0_JPRB+0.5_JPRB*ZTDMTW0)/RTAUMEL)
      ZMELTMAX(JL) = MAX(ZTDMTW0*ZCONS1*ZRLDCP,0.0_JPRB)
    ENDIF
  ENDDO

  ! Loop over frozen hydrometeors (ice, snow)
  DO JM=1,NCLV
   IF (IPHASE(JM) == 2) THEN
    JN = IMELT(JM)
    DO JL=KIDIA,KFDIA
      IF(ZMELTMAX(JL)>ZEPSEC .AND. ZICETOT(JL)>ZEPSEC) THEN
        ! Apply melting in same proportion as frozen hydrometeor fractions 
        ZALFA = ZQXFG(JL,JM)/ZICETOT(JL)
        ZMELT = MIN(ZQXFG(JL,JM),ZALFA*ZMELTMAX(JL))
        ! needed in first guess
        ! This implies that zqpretot has to be recalculated below
        ! since is not conserved here if ice falls and liquid doesn't
        ZQXFG(JL,JM)     = ZQXFG(JL,JM)-ZMELT
        ZQXFG(JL,JN)     = ZQXFG(JL,JN)+ZMELT
        ZSOLQA(JL,JN,JM) = ZSOLQA(JL,JN,JM)+ZMELT
        ZSOLQA(JL,JM,JN) = ZSOLQA(JL,JM,JN)-ZMELT
        IF (LLCLDBUDI.AND.JM==NCLDQI) ZBUDI(JL,15)=-ZMELT*ZQTMST
        IF (LLCLDBUDI.AND.JM==NCLDQS) ZBUDI(JL,16)=-ZMELT*ZQTMST
        IF (LLCLDBUDL.AND.JM==NCLDQI) ZBUDL(JL,17)=ZMELT*ZQTMST
      ENDIF
    ENDDO
   ENDIF
  ENDDO
  
  !----------------------------------------------------------------------
  ! 4.4b  FREEZING of RAIN
  !----------------------------------------------------------------------
  DO JL=KIDIA,KFDIA ! 
    ! not implicit yet... 
    ZFRZMAX(JL)=MAX((RTT-ZTP1(JL,JK))*ZRLDCP,0.0_JPRB)
  ENDDO

  JM = NCLDQR
  JN = IMELT(JM)
  DO JL=KIDIA,KFDIA
    IF(ZFRZMAX(JL)>ZEPSEC .AND. ZQXFG(JL,JM)>ZEPSEC) THEN
      ZFRZ = MIN(ZQXFG(JL,JM),ZFRZMAX(JL))
      ZSOLQA(JL,JN,JM) = ZSOLQA(JL,JN,JM)+ZFRZ
      ZSOLQA(JL,JM,JN) = ZSOLQA(JL,JM,JN)-ZFRZ
      IF (LLCLDBUDL) ZBUDL(JL,18)=ZFRZ*ZQTMST
    ENDIF
  ENDDO


  !----------------------------------------------------------------------
  ! 4.4c  FREEZING of LIQUID 
  !----------------------------------------------------------------------
  DO JL=KIDIA,KFDIA ! 
    ! not implicit yet... 
    ZFRZMAX(JL)=MAX((RTHOMO-ZTP1(JL,JK))*ZRLDCP,0.0_JPRB)
  ENDDO

  JM = NCLDQL
  JN = IMELT(JM)
  DO JL=KIDIA,KFDIA
    IF(ZFRZMAX(JL)>ZEPSEC .AND. ZQXFG(JL,JM)>ZEPSEC) THEN
      ZFRZ = MIN(ZQXFG(JL,JM),ZFRZMAX(JL))
      ZSOLQA(JL,JN,JM) = ZSOLQA(JL,JN,JM)+ZFRZ
      ZSOLQA(JL,JM,JN) = ZSOLQA(JL,JM,JN)-ZFRZ
      IF (LLCLDBUDL) ZBUDL(JL,18)=ZFRZ*ZQTMST
    ENDIF
  ENDDO

  !----------------------------------------------------------------------
  ! 4.5   EVAPORATION OF RAIN/SNOW/ALL FALLING SPECIES
  !       at the moment all species evap/sublime in same way for 
  !       compatibility with old scheme, but this will be updated to 
  !       treat each variable separately. 
  !----------------------------------------------------------------------

  !------------------------------------------------------------
  ! recalculate zqpretot since melting term may have changed it
  !------------------------------------------------------------ 
  DO JL=KIDIA,KFDIA
    ZQPRETOT(JL) = ZQXFG(JL,NCLDQS)+ZQXFG(JL,NCLDQR)
  ENDDO

  
  ! Rain
  
  DO JL=KIDIA,KFDIA
    ZZRH=RPRECRHMAX+(1.0_JPRB-RPRECRHMAX)*ZCOVPCLR(JL)/MAX(ZEPSEC,1.0_JPRB-ZA(JL,JK))
    ZZRH=MIN(MAX(ZZRH,RPRECRHMAX),1.0_JPRB)

    ZQE=(ZQX(JL,JK,NCLDQV)-ZA(JL,JK)*ZQSLIQ(JL,JK))/&
    & MAX(ZEPSEC,1.0_JPRB-ZA(JL,JK))  
    !---------------------------------------------
    ! humidity in moistest ZCOVPCLR part of domain
    !---------------------------------------------
    ZQE=MAX(0.0_JPRB,MIN(ZQE,ZQSLIQ(JL,JK)))
    LLO1=ZCOVPCLR(JL)>ZEPSEC .AND. &
!      & ZQPRETOT(JL)>ZEPSEC .AND. & 
       & ZQXFG(JL,NCLDQR)>RLMIN .AND. &
       & ZQE<ZZRH*ZQSLIQ(JL,JK)
    IF(LLO1) THEN
      ! note: units of zpreclr and zqpretot differ
      !       zqpretot is a mixing ratio (hence "q" in name)
      !       zpreclr is a rain flux
      ZPRECLR=ZQPRETOT(JL)*ZCOVPCLR(JL)/(ZCOVPTOT(JL)*ZDTGDP(JL))

      !--------------------------------------
      ! actual microphysics formula in zbeta
      !--------------------------------------
      ! Sensitivity test showed multiply rain evap rate by 0.5
      ! Need to investigate formulation
!      ZBETA=RG*RPECONS*(SQRT(PAP(JL,JK)/&

      ZBETA1=SQRT(PAP(JL,JK)/&
       & PAPH(JL,KLEV+1))/5.09E-3_JPRB*ZPRECLR/&
       & MAX(ZCOVPCLR(JL),ZEPSEC)

      IF(ZBETA1 >=0.0_JPRB)THEN
        ZBETA=RG*RPECONS*0.5_JPRB*(ZBETA1)**0.5777_JPRB  

        ZDENOM=1.0_JPRB+ZBETA*PTSPHY*ZCORQSLIQ(JL)
        ZDPR = ZCOVPCLR(JL)*ZBETA*(ZQSLIQ(JL,JK)-ZQE)/ZDENOM*ZDP(JL)*ZRG_R
        ZDPEVAP=ZDPR*ZDTGDP(JL)

      !---------------------------------------------------------
      ! add evaporation term to explicit sink.
      ! this has to be explicit since if treated in the implicit
      ! term evaporation can not reduce rain to zero and model
      ! produces small amounts of rainfall everywhere. 
      !---------------------------------------------------------
      
      ! Evaporate rain
        ZEVAP = MIN(ZDPEVAP,ZQXFG(JL,NCLDQR))
      ELSE
        ZEVAP = ZQXFG(JL,NCLDQR)
      ENDIF

      ZSOLQA(JL,NCLDQV,NCLDQR) = ZSOLQA(JL,NCLDQV,NCLDQR)+ZEVAP
      ZSOLQA(JL,NCLDQR,NCLDQV) = ZSOLQA(JL,NCLDQR,NCLDQV)-ZEVAP
      ZQXFG(JL,NCLDQR)         = ZQXFG(JL,NCLDQR)-ZEVAP
      IF (LLCLDBUDL) ZBUDL(JL,19)=-ZEVAP*ZQTMST

      !-------------------------------------------------------------
      ! Reduce the total precip coverage proportional to evaporation
      ! to mimic the previous scheme which had a diagnostic
      ! 2-flux treatment, abandoned due to the new prognostic precip
      !-------------------------------------------------------------
      IF(ZBETA1 >=0.0_JPRB)THEN
        ZCOVPTOT(JL)=ZCOVPTOT(JL)-MAX(0.0_JPRB, &
        &           (ZCOVPTOT(JL)-ZA(JL,JK))*ZDPEVAP/ZQPRETOT(JL))
      ENDIF
    ENDIF
  ENDDO

  ! Snow
  
  DO JL=KIDIA,KFDIA
    ZZRH=RPRECRHMAX+(1.0_JPRB-RPRECRHMAX)*ZCOVPCLR(JL)/MAX(ZEPSEC,1.0_JPRB-ZA(JL,JK))
    ZZRH=MIN(MAX(ZZRH,RPRECRHMAX),1.0_JPRB)
    ! ZQE=(ZQX(JL,JK,NCLDQV)-ZA(JL,JK)*ZQSMIX(JL,JK))/&
    ZQE=(ZQX(JL,JK,NCLDQV)-ZA(JL,JK)*ZQSICE(JL,JK))/&
    & MAX(ZEPSEC,1.0_JPRB-ZA(JL,JK))  
    !---------------------------------------------
    ! humidity in moistest ZCOVPCLR part of domain
    !---------------------------------------------
    ZQE=MAX(0.0_JPRB,MIN(ZQE,ZQSICE(JL,JK)))
    LLO1=ZCOVPCLR(JL)>ZEPSEC .AND. &
!      & ZQPRETOT(JL)>ZEPSEC .AND. & 
       & ZQXFG(JL,NCLDQS)>RLMIN .AND. &
       & ZQE<ZZRH*ZQSICE(JL,JK)
    IF(LLO1) THEN
      ! note: units of zpreclr and zqpretot differ
      !       zqpretot is a mixing ratio (hence "q" in name)
      !       zpreclr is a rain flux
      ZPRECLR=ZQPRETOT(JL)*ZCOVPCLR(JL)/(ZCOVPTOT(JL)*ZDTGDP(JL))

      !--------------------------------------
      ! actual microphysics formula in zbeta
      !--------------------------------------

      ZBETA1=SQRT(PAP(JL,JK)/&
       & PAPH(JL,KLEV+1))/5.09E-3_JPRB*ZPRECLR/&
       & MAX(ZCOVPCLR(JL),ZEPSEC)

      IF(ZBETA1 >= 0.0_JPRB)THEN
        ZBETA=RG*RPECONS*(ZBETA1)**0.5777_JPRB  

        ZDENOM=1.0_JPRB+ZBETA*PTSPHY*ZCORQSICE(JL)
        ZDPR = ZCOVPCLR(JL)*ZBETA*(ZQSICE(JL,JK)-ZQE)/ZDENOM*ZDP(JL)*ZRG_R
        ZDPEVAP=ZDPR*ZDTGDP(JL)

      !---------------------------------------------------------
      ! add evaporation term to explicit sink.
      ! this has to be explicit since if treated in the implicit
      ! term evaporation can not reduce snow to zero and model
      ! produces small amounts of snowfall everywhere. 
      !---------------------------------------------------------
      
      ! Evaporate snow
        ZEVAP = MIN(ZDPEVAP,ZQXFG(JL,NCLDQS))
      ELSE
        ZEVAP = ZQXFG(JL,NCLDQS)
      ENDIF
      ZSOLQA(JL,NCLDQV,NCLDQS) = ZSOLQA(JL,NCLDQV,NCLDQS)+ZEVAP
      ZSOLQA(JL,NCLDQS,NCLDQV) = ZSOLQA(JL,NCLDQS,NCLDQV)-ZEVAP
      ZQXFG(JL,NCLDQS)         = ZQXFG(JL,NCLDQS)-ZEVAP
      IF (LLCLDBUDL) ZBUDi(JL,17)=-ZEVAP*ZQTMST
      
      !-------------------------------------------------------------
      ! Reduce the total precip coverage proportional to evaporation
      ! to mimic the previous scheme which had a diagnostic
      ! 2-flux treatment, abandoned due to the new prognostic precip
      !-------------------------------------------------------------
      IF(ZBETA1 >= 0.0_JPRB)THEN
        ZCOVPTOT(JL)=ZCOVPTOT(JL)-MAX(0.0_JPRB, &
        &           (ZCOVPTOT(JL)-ZA(JL,JK))*ZDPEVAP/ZQPRETOT(JL))
      ENDIF
    ENDIF
  ENDDO

  !--------------------------------------
  ! Evaporate small precipitation amounts
  !--------------------------------------
  DO JM=1,NCLV
   IF (LLFALL(JM)) THEN 
    DO JL=KIDIA,KFDIA
      IF (ZQXFG(JL,JM)<RLMIN) THEN
        ZSOLQA(JL,NCLDQV,JM) = ZSOLQA(JL,NCLDQV,JM)+ZQXFG(JL,JM)
        ZSOLQA(JL,JM,NCLDQV) = ZSOLQA(JL,JM,NCLDQV)-ZQXFG(JL,JM)
      ENDIF
    ENDDO
   ENDIF
  ENDDO
  
  !------------------------------------
  ! recalculate zqpretot to see if zero
  !------------------------------------
  DO JL=KIDIA,KFDIA
    ZQPRETOT(JL) =ZQXFG(JL,NCLDQS)+ZQXFG(JL,NCLDQR)
  ENDDO
  DO JL=KIDIA,KFDIA
    IF (ZQPRETOT(JL)<ZEPSEC) ZCOVPTOT(JL)=0.0_JPRB
  ENDDO
  
  !######################################################################
  !            5.0  *** SOLVERS FOR A AND L ***
  !
  ! now use an implicit solution rather than exact solution
  ! solver is forward in time, upstream difference for advection
  !######################################################################

  !---------------------------
  ! 5.1 solver for cloud cover
  !---------------------------
  DO JL=KIDIA,KFDIA
    ZANEW=(ZA(JL,JK)+ZSOLAC(JL))/(1.0_JPRB+ZSOLAB(JL))
    ZANEW=MIN(ZANEW,1.0_JPRB)
    IF (ZANEW<RAMIN) ZANEW=0.0_JPRB
    ZDA(JL)=ZANEW-ZAORIG(JL,JK)
    !---------------------------------
    ! variables needed for next level
    !---------------------------------
    ZANEWM1(JL)=ZANEW
  ENDDO

  !--------------------------------
  ! 5.2 solver for the microphysics
  !--------------------------------

  !--------------------------------------------------------------
  ! Truncate explicit sinks to avoid negatives 
  ! Note: Species are treated in the order in which they run out
  ! since the clipping will alter the balance for the other vars
  !--------------------------------------------------------------

  DO JM=1,NCLV
    DO JN=1,NCLV
      DO JL=KIDIA,KFDIA
        LLINDEX3(JL,JN,JM)=.FALSE.
      ENDDO
    ENDDO
    DO JL=KIDIA,KFDIA
      ZSINKSUM(JL,JM)=0.0_JPRB
    ENDDO
  ENDDO

  !----------------------------
  ! collect sink terms and mark
  !----------------------------
  DO JM=1,NCLV
    DO JN=1,NCLV
      DO JL=KIDIA,KFDIA
        ZSINKSUM(JL,JM)=ZSINKSUM(JL,JM)-ZSOLQA(JL,JM,JN) ! +ve total is bad
      ENDDO
    ENDDO
  ENDDO

  !---------------------------------------
  ! calculate overshoot and scaling factor
  !---------------------------------------
  DO JM=1,NCLV
    DO JL=KIDIA,KFDIA
      ZMAX=MAX(ZQX(JL,JK,JM),ZEPSEC)
      ZRAT=MAX(ZSINKSUM(JL,JM),ZMAX)
      ZRATIO(JL,JM)=ZMAX/ZRAT
    ENDDO
  ENDDO
  !--------------------------------------------------------
  ! now sort zratio to find out which species run out first
  !--------------------------------------------------------
  DO JM=1,NCLV
    DO JL=KIDIA,KFDIA
      IORDER(JL,JM)=-999
    ENDDO
  ENDDO
  DO JN=1,NCLV
    DO JL=KIDIA,KFDIA
      LLINDEX1(JL,JN)=.TRUE.
    ENDDO
  ENDDO
  DO JM=1,NCLV
    DO JL=KIDIA,KFDIA
      ZMIN(JL)=1.E32_JPRB
    ENDDO
    DO JN=1,NCLV
      DO JL=KIDIA,KFDIA
        IF (LLINDEX1(JL,JN) .AND. ZRATIO(JL,JN)<ZMIN(JL)) THEN
          IORDER(JL,JM)=JN
          ZMIN(JL)=ZRATIO(JL,JN)
        ENDIF
      ENDDO
    ENDDO
    DO JL=KIDIA,KFDIA
      LLINDEX1(JL,IORDER(JL,JM))=.FALSE. ! marked as searched
    ENDDO
  ENDDO

  !--------------------------------------------
  ! scale the sink terms, in the correct order, 
  ! recalculating the scale factor each time
  !--------------------------------------------
  DO JM=1,NCLV
    DO JL=KIDIA,KFDIA
      ZSINKSUM(JL,JM)=0.0_JPRB
    ENDDO
  ENDDO

  !----------------
  ! recalculate sum
  !----------------
  DO JM=1,NCLV
    DO JN=1,NCLV
      DO JL=KIDIA,KFDIA
        JO=IORDER(JL,JM)
!J---Start---
!J      IF (ZSOLQA(JL,JO,JN)<0.0_JPRB) THEN
!J         LLINDEX3(JL,JO,JN)=.TRUE. 
!J      ENDIF
        LLINDEX3(JL,JO,JN)=ZSOLQA(JL,JO,JN)<0.0_JPRB
!J---End---
        ZSINKSUM(JL,JO)=ZSINKSUM(JL,JO)-ZSOLQA(JL,JO,JN)
      ENDDO
    ENDDO
    !---------------------------
    ! recalculate scaling factor
    !---------------------------
    DO JL=KIDIA,KFDIA
      JO=IORDER(JL,JM)
!J---Start---
!J    ZRATIO(JL,JO)=MAX(ZSINKSUM(JL,JO)/MAX(ZQX(JL,JK,JO),ZEPSEC),1.0_JPRB)
!J    ZRATIO(JL,JO)=1.0_JPRB/ZRATIO(JL,JO)
      ZMM=MAX(ZQX(JL,JK,JO),ZEPSEC)
      ZRR=MAX(ZSINKSUM(JL,JO),ZMM)
      ZRATIO(JL,JO)=ZMM/ZRR
!J---End---
    ENDDO
    !------
    ! scale
    !------
    DO JN=1,NCLV
      DO JL=KIDIA,KFDIA
        JO=IORDER(JL,JM)
        IF (LLINDEX3(JL,JO,JN)) THEN
!       IF (ZSOLQA(JL,JO,JN)<0.0_JPRB) THEN
          ZSOLQA(JL,JO,JN)=ZSOLQA(JL,JO,JN)*ZRATIO(JL,JO)
          ZSOLQA(JL,JN,JO)=ZSOLQA(JL,JN,JO)*ZRATIO(JL,JO)
        ENDIF
      ENDDO
    ENDDO
  ENDDO

  !--------------------------------------------------------------
  !
  ! 5.2.2 Solver
  !
  !------------------------

  !------------------------
  ! set the LHS of equation  
  !------------------------
  DO JM=1,NCLV
    DO JN=1,NCLV
      DO JL=KIDIA,KFDIA
        !----------------------------------------------
        ! diagonals: microphysical sink terms+transport
        !----------------------------------------------
        IF (JN==JM) THEN
          ZQLHS(JL,JN,JM)=1.0_JPRB + ZFALLSINK(JL,JM)
          DO JO=1,NCLV
            ZQLHS(JL,JN,JM)=ZQLHS(JL,JN,JM) + ZSOLQB(JL,JO,JN)
          ENDDO
        !------------------------------------------
        ! non-diagonals: microphysical source terms
        !------------------------------------------
        ELSE
          ZQLHS(JL,JN,JM)= -ZSOLQB(JL,JN,JM) ! here is the delta T - missing from doc.
       ENDIF
      ENDDO
    ENDDO
  ENDDO
  !------------------------
  ! set the RHS of equation  
  !------------------------
  DO JM=1,NCLV
    DO JL=KIDIA,KFDIA
      !---------------------------------
      ! sum the explicit source and sink
      !---------------------------------
      ZEXPLICIT=0.0_JPRB
      DO JN=1,NCLV
        ZEXPLICIT=ZEXPLICIT+ZSOLQA(JL,JM,JN) ! sum over middle index
      ENDDO
      ZQXN(JL,JM)=ZQX(JL,JK,JM)+ZEXPLICIT
    ENDDO
  ENDDO

  !-----------------------------------
  ! *** solve by LU decomposition: ***
  !-----------------------------------
  CALL LUDCMP(ZQLHS,KLON,KIDIA,KFDIA,NCLV,NCLV,JINDEX2)
  CALL LUBKSB(ZQLHS,KLON,KIDIA,KFDIA,NCLV,NCLV,JINDEX2,ZQXN)

  !--------------------------------
  ! variables needed for next level
  !--------------------------------
  DO JM=1,NCLV
    DO JL=KIDIA,KFDIA
      ZQXNM1(JL,JM)    = ZQXN(JL,JM)
      ZQXN2D(JL,JK,JM) = ZQXN(JL,JM)
    ENDDO
  ENDDO

  !------------------------------------------------------------------------
  ! 5.3 Precipitation/sedimentation fluxes to next level
  !     diagnostic precipitation fluxes
  !     It is this scaled flux that must be used for source to next layer
  !------------------------------------------------------------------------

  DO JM=1,NCLV
    DO JL=KIDIA,KFDIA
      ZPFPLSX(JL,JK+1,JM) = ZFALLSINK(JL,JM)*ZQXN(JL,JM)*ZRDTGDP(JL)
    ENDDO
  ENDDO

  DO JL=KIDIA,KFDIA
    IF(ZTP1(JL,JK) <= RTT) THEN
      ! Store liq->snow autoconversion diagnostic if required
      IF (LLCLDBUDL) ZBUDL(JL,15)=ZRAINAUT(JL)*ZQXN(JL,NCLDQL)*ZQTMST
    ELSE
      ! Store liq->rain autoconversion diagnostic if required
      IF (LLCLDBUDL) ZBUDL(JL,16)=ZRAINAUT(JL)*ZQXN(JL,NCLDQL)*ZQTMST
    ENDIF
  ENDDO

  DO JL=KIDIA,KFDIA
    IF (LLCLDBUDL) ZBUDL(JL,6)=-ZCONVSINK(JL,NCLDQL)*ZQXN(JL,NCLDQL)*ZQTMST
    IF (LLCLDBUDI) ZBUDI(JL,6)=-ZCONVSINK(JL,NCLDQI)*ZQXN(JL,NCLDQI)*ZQTMST
    IF (LLCLDBUDI) ZBUDI(JL,13)=ZFALLSINK(JL,NCLDQI)*ZQXN(JL,NCLDQI)*ZQTMST
    IF (LLCLDBUDI) ZBUDI(JL,14)=ZSNOWAUT(JL)*ZQXN(JL,NCLDQI)*ZQTMST
  ENDDO

  !######################################################################
  !              6  *** UPDATE TENDANCIES ***
  !######################################################################

  !--------------------------------
  ! 6.1 Temperature and CLV budgets 
  !--------------------------------

  DO JM=1,NCLV-1
    DO JL=KIDIA,KFDIA

      ! calculate fluxes in and out of box for conservation of TL
      ZFLUXQ(JL,JM)=ZPSUPSATSRCE(JL,JM)+ZCONVSRCE(JL,JM)+ZFALLSRCE(JL,JM)-&
                    & (ZFALLSINK(JL,JM)+ZCONVSINK(JL,JM))*ZQXN(JL,JM)

      IF (IPHASE(JM)==1) THEN
        PTENT(JL,JK)=PTENT(JL,JK)+ &
          & RALVDCP*(ZQXN(JL,JM)-ZQX(JL,JK,JM)-ZFLUXQ(JL,JM))*ZQTMST
      ENDIF

      IF (IPHASE(JM)==2) THEN
        PTENT(JL,JK)=PTENT(JL,JK)+ &
          & RALSDCP*(ZQXN(JL,JM)-ZQX(JL,JK,JM)-ZFLUXQ(JL,JM))*ZQTMST
      ENDIF

      !----------------------------------------------------------------------
      ! New prognostic tendencies - ice,liquid rain,snow 
      ! Note: CLV arrays use PCLV in calculation of tendency while humidity
      !       uses ZQX. This is due to clipping at start of cloudsc which
      !       include the tendency already in PTENT and PTENQ. ZQX was reset
      !----------------------------------------------------------------------
      PTENCLV(JL,JK,JM)=PTENCLV(JL,JK,JM)+(ZQXN(JL,JM)-ZQX0(JL,JK,JM))*ZQTMST

    ENDDO
  ENDDO

  DO JL=KIDIA,KFDIA
    !----------------------
    ! 6.2 Humidity budget
    !----------------------
    PTENQ(JL,JK)=PTENQ(JL,JK)+(ZQXN(JL,NCLDQV)-ZQX(JL,JK,NCLDQV))*ZQTMST

    !-------------------
    ! 6.3 cloud cover 
    !-----------------------
    PTENA(JL,JK)=PTENA(JL,JK)+ZDA(JL)*ZQTMST

    !------------------------------------------------------------------
    ! check for supersaturation when Semi-Langrangian advection is off
    !------------------------------------------------------------------
    IF( .NOT. LDSLPHY ) THEN
      ZQNEW=PQORIG(JL,JK)+PTSPHY*PTENQ(JL,JK)
      ZTNEW=PTORIG(JL,JK)+PTSPHY*PTENT(JL,JK)
      ZANEW=PA(JL,JK)+PTSPHY*PTENA(JL,JK)
      ZALFAW=FOEALFA(ZTNEW)
      IF (ZTNEW>=RTT .OR. NSSOPT==0) THEN
        ZFAC=1.0_JPRB
        ZFACI=ZQTMST
      ELSE
        ZFAC=ZANEW+FOKOOP(ZTNEW)*(1.0_JPRB-ZANEW)
        ZFACI=1.0_JPRB/RKOOPTAU
      ENDIF
      ZQSAT=FOEEWM(ZTNEW)/PAP(JL,JK)
      ZQSAT=MIN(0.5_JPRB,ZQSAT)
      ZQSAT=ZQSAT/(1.0_JPRB-RETV*ZQSAT)
      IF (ZQNEW>ZFAC*ZQSAT) THEN 
!J---Start---
!J      ZCOND=(ZQNEW-ZFAC*ZQSAT)/ZCORQSMIX(JL)
        ZCOND=(ZQNEW-ZFAC*ZQSAT)*ZCORQSMIX(JL)
!J---End---
        PTENQ(JL,JK)=PTENQ(JL,JK)-ZCOND*ZQTMST
        PTENT(JL,JK)=PTENT(JL,JK)+FOELDCPM(ZTP1(JL,JK))*ZCOND*ZQTMST
        PTENA(JL,JK)=PTENA(JL,JK)+(1.0_JPRB-ZANEW)*ZFACI
!       ZQXN(JL,NCLDQL)=ZQXN(JL,NCLDQL)+ZCOND*ZALFAW
!       ZQXN(JL,NCLDQI)=ZQXN(JL,NCLDQI)+ZCOND*(1.0_JPRB-ZALFAW)
        PTENCLV(JL,JK,NCLDQL)=PTENCLV(JL,JK,NCLDQL)+ZCOND*ZALFAW*ZQTMST
        PTENCLV(JL,JK,NCLDQI)=PTENCLV(JL,JK,NCLDQI)+ZCOND*(1.0_JPRB-ZALFAW)*ZQTMST
      ENDIF
    ENDIF
  ENDDO


 ENDIF ! on LDMAINCALL ******************************


!------------------------------------------------------------------
! Add T and Q tendencies for first guess call of cloud scheme only
!------------------------------------------------------------------
 IF (.NOT. LDMAINCALL) THEN

  DO JL=KIDIA,KFDIA
    IF (ZTP1(JL,JK) > RTHOMO) THEN
      PTENT(JL,JK)=PTENT(JL,JK)+ &
          & RALVDCP*(ZLCOND1(JL)+ZLCOND2(JL)+ZSUPSAT(JL))*ZQTMST
    ELSE
      PTENT(JL,JK)=PTENT(JL,JK)+ &
          & RALSDCP*(ZLCOND1(JL)+ZLCOND2(JL)+ZSUPSAT(JL))*ZQTMST
    ENDIF
    ! Add evaporation
    PTENT(JL,JK)=PTENT(JL,JK)- &
          & (RALVDCP*ZLEVAPL(JL)+RALSDCP*ZLEVAPI(JL))*ZQTMST
    PTENQ(JL,JK)=PTENQ(JL,JK)-(ZLCOND1(JL)+ZLCOND2(JL)+ZSUPSAT(JL) &
                           &  -ZLEVAPL(JL)-ZLEVAPI(JL))*ZQTMST
  ENDDO
  ! And what about if LDSLPHY=.FALSE. (i.e. not semi-lagrangian -> SCM)

 ENDIF

!-----------------------------------------------------------------
! Cloud fraction budget 
!-----------------------------------------------------------------
  IF (LLCLDBUDCC) THEN
  
    DO JL=KIDIA,KFDIA
      IS = 0
      PEXTRA(JL,JK,IS+1)  = PAORIG(JL,JK) ! Initial total cloud fraction
      PEXTRA(JL,JK,IS+2)  = ZBUDCC(JL,1)  ! Supersat clipping
      PEXTRA(JL,JK,IS+3)  = ZBUDCC(JL,2)  ! Supersat clipping from t-1 sltend
      PEXTRA(JL,JK,IS+4)  = ZBUDCC(JL,3)  ! Convective detrainment
      PEXTRA(JL,JK,IS+5)  = ZBUDCC(JL,4)  ! Source from convective mass flux
      PEXTRA(JL,JK,IS+6)  = ZBUDCC(JL,6)  ! Sink from convective mass flux (IMPL)
      PEXTRA(JL,JK,IS+7)  = ZBUDCC(JL,7)  ! Turbulent erosion
      PEXTRA(JL,JK,IS+8)  = ZBUDCC(JL,10) ! Condensation of new cloud
      PEXTRA(JL,JK,IS+9)  = ZBUDCC(JL,11) ! Small value tidy up
      PEXTRA(JL,JK,IS+10) = PVFA(JL,JK)   ! Vertical diffusion
      PEXTRA(JL,JK,IS+12) = PDYNA(JL,JK)  ! Advection from dynamics
      PEXTRA(JL,JK,IS+13) = PAORIG(JL,JK)+ZDA(JL) ! Final cloud frac
    ENDDO

  ENDIF


  IF (LLCLDBUDL) THEN
  
    DO JL=KIDIA,KFDIA
      IS = 0
      PEXTRA(JL,JK,IS+1)  = ZQX0(JL,JK,NCLDQL) ! Initial condensate
      PEXTRA(JL,JK,IS+2)  = ZBUDL(JL,1)  ! + Supersat clipping so far this timestep                          
      PEXTRA(JL,JK,IS+3)  = ZBUDL(JL,2)  ! + Supersat clipping from t-1 sltend (PSUPSAT)                     
      PEXTRA(JL,JK,IS+4)  = ZBUDL(JL,3)  ! + Convective detrainment                                          
      PEXTRA(JL,JK,IS+5)  = ZBUDL(JL,4)  ! + Convective subsidence source from layer above                   
      PEXTRA(JL,JK,IS+6)  = ZBUDL(JL,5)  ! - Convective subsidence source evaporation in layer               
      PEXTRA(JL,JK,IS+7)  = ZBUDL(JL,6)  ! - Convective subsidence sink to layer below (IMPLICIT) (ZCONVSINK)
      PEXTRA(JL,JK,IS+8)  = ZBUDL(JL,7)  ! - Turbulent erosion                                               
      PEXTRA(JL,JK,IS+9)  = ZBUDL(JL,8)  ! - Evaporation of existing cloud (dqs increasing = subsat)         
      PEXTRA(JL,JK,IS+10) = ZBUDL(JL,9)  ! + Condensation of existing cloud (dqs decreasing = supersat)      
      PEXTRA(JL,JK,IS+11) = ZBUDL(JL,10) ! + Condensation of new cloud (dqs decreasing = supersat)           
      PEXTRA(JL,JK,IS+12) = ZBUDL(JL,11) ! - Deposition of liquid to ice                                     
      PEXTRA(JL,JK,IS+13) = ZBUDL(JL,15) ! - Autoconversion to rain (IMPLICIT)(ZRAINAUT)                     
      PEXTRA(JL,JK,IS+14) = ZBUDL(JL,16) ! - Autoconversion to rain+freezing->snow (IMPLICIT)(ZRAINAUT)      
      PEXTRA(JL,JK,IS+15) = ZBUDL(JL,17) ! + Melting of ice to liquid                                        
      PEXTRA(JL,JK,IS+16) = ZBUDL(JL,18) ! - Freezing of rain to snow                                        
      PEXTRA(JL,JK,IS+17) = ZBUDL(JL,19) ! - Freezing of liquid to ice                                       
!      PEXTRA(JL,JK,IS+18) = ZBUDL(JL,20) ! - Evaporation of rain
      PEXTRA(JL,JK,IS+18) = PVFL(JL,JK)+PVFI(JL,JK)  ! Vertical diffusion
      PEXTRA(JL,JK,IS+19) = PDYNL(JL,JK)+PDYNI(JL,JK) ! Advection from dynamics
      PEXTRA(JL,JK,IS+20) = ZQXN(JL,NCLDQL)  ! Final condensate
    ENDDO

  ENDIF
  IF (LLCLDBUDI) THEN
    IF (LLCLDBUDL) THEN
      IS = 20
    ELSE
      IS = 0
    ENDIF
    DO JL=KIDIA,KFDIA
      PEXTRA(JL,JK,IS+1)  = ZQX0(JL,JK,NCLDQI) ! Initial condensate
      PEXTRA(JL,JK,IS+2)  = ZBUDI(JL,1)  ! + Supersat clipping so far this timestep 
      PEXTRA(JL,JK,IS+3)  = ZBUDI(JL,2)  ! + Supersat clipping from t-1 sltend (PSUPSAT)
      PEXTRA(JL,JK,IS+4)  = ZBUDI(JL,3)  ! + Convective detrainment
      PEXTRA(JL,JK,IS+5)  = ZBUDI(JL,4)  ! + Convective subsidence source from layer above
      PEXTRA(JL,JK,IS+6)  = ZBUDI(JL,5)  ! - Convective subsidence source evaporation in layer
      PEXTRA(JL,JK,IS+7)  = ZBUDI(JL,6)  ! - Convective subsidence sink to layer below (IMPLICIT) (ZCONVSINK)
      PEXTRA(JL,JK,IS+8)  = ZBUDI(JL,7)  ! - Turbulent erosion
      PEXTRA(JL,JK,IS+9)  = ZBUDI(JL,8)  ! - Evaporation of existing cloud (dqs increasing = subsat)
      PEXTRA(JL,JK,IS+10) = ZBUDI(JL,9)  ! + Condensation of existing cloud (dqs decreasing = supersat)
      PEXTRA(JL,JK,IS+11) = ZBUDI(JL,10) ! + Condensation of new cloud (dqs decreasing = supersat)
!      PEXTRA(JL,JK,IS+12) = ZBUDI(JL,11) ! + Deposition of liquid to ice
      PEXTRA(JL,JK,IS+12) = ZBUDI(JL,12) ! + Ice sedimentation source from above
      PEXTRA(JL,JK,IS+13) = ZBUDI(JL,13) ! - Ice sedimentation sink to below (IMPLICIT)(ZFALLSINK)
      PEXTRA(JL,JK,IS+14) = ZBUDI(JL,14) ! - Autoconversion to snow (IMPLICIT) (ZSNOWAUT)
!      PEXTRA(JL,JK,IS+16) = ZBUDI(JL,15) ! - Melting of ice to liquid
      PEXTRA(JL,JK,IS+15) = ZBUDI(JL,16) ! - Melting of snow to rain
!      PEXTRA(JL,JK,IS+16) = ZBUDI(JL,17) ! - Evaporation of snow
!      PEXTRA(JL,JK,IS+17) = PVFI(JL,JK)  ! Vertical diffusion
!      PEXTRA(JL,JK,IS+18) = PDYNI(JL,JK) ! Advection from dynamics
      PEXTRA(JL,JK,IS+16) = ZQXN(JL,NCLDQI)  ! Final condensate
    ENDDO

  ENDIF
 
!--------------------------------------------------
! Copy precipitation fraction into output variable
!-------------------------------------------------
  DO JL=KIDIA,KFDIA
    PCOVPTOT(JL,JK) = ZCOVPTOT(JL)
  ENDDO
 
ENDDO ! on vertical level JK
!----------------------------------------------------------------------
!                       END OF VERTICAL LOOP
!----------------------------------------------------------------------


IF (LDMAINCALL) THEN  ! ***Only included if main call to cloudsc***


!######################################################################
!              8  *** FLUX/DIAGNOSTICS COMPUTATIONS ***
!######################################################################

!-------------------------------------
! Enthalpy and total water diagnostics (LSCMEC and LCLDBUDGET normally false)
!-------------------------------------
IF (LSCMEC.OR.LCLDBUDGET) THEN

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTNEW=PT(JL,JK)+PTSPHY*(PTENT(JL,JK)-ZTENKEEP(JL,JK,NCLV+1))
      IF (JK==1) THEN
        ZSUMQ1(JL,JK)=0.0_JPRB
        ZSUMH1(JL,JK)=0.0_JPRB
      ELSE
        ZSUMQ1(JL,JK)=ZSUMQ1(JL,JK-1)
        ZSUMH1(JL,JK)=ZSUMH1(JL,JK-1)
      ENDIF

      ! cld vars
      DO JM=1,NCLV-1
        IF (IPHASE(JM)==1) ZTNEW=ZTNEW-RALVDCP*(PCLV(JL,JK,JM)+ &
          & (PTENCLV(JL,JK,JM)-ZTENKEEP(JL,JK,JM))*PTSPHY)
        IF (IPHASE(JM)==2) ZTNEW=ZTNEW-RALSDCP*(PCLV(JL,JK,JM)+ &
          & (PTENCLV(JL,JK,JM)-ZTENKEEP(JL,JK,JM))*PTSPHY)
        ZSUMQ1(JL,JK)=ZSUMQ1(JL,JK)+ &
        & (PCLV(JL,JK,JM)+(PTENCLV(JL,JK,JM)-ZTENKEEP(JL,JK,JM))*PTSPHY)* &
        & (PAPH(JL,JK+1)-PAPH(JL,JK))*ZRG_R
      ENDDO
      ZSUMH1(JL,JK)=ZSUMH1(JL,JK)+(PAPH(JL,JK+1)-PAPH(JL,JK))*ZTNEW 

      ! humidity
      ZSUMQ1(JL,JK)=ZSUMQ1(JL,JK)+ &
        &(PQ(JL,JK)+(PTENQ(JL,JK)-ZTENKEEP(JL,JK,NCLDQV))*PTSPHY)*(PAPH(JL,JK+1)-PAPH(JL,JK))*ZRG_R

      ZRAIN=0.0_JPRB
      DO JM=1,NCLV
        ZRAIN=ZRAIN+PTSPHY*ZPFPLSX(JL,JK+1,JM)
      ENDDO
      ZERRORQ(JL,JK)=ZSUMQ1(JL,JK)+ZRAIN-ZSUMQ0(JL,JK)
    ENDDO
  ENDDO

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZDTGDP(JL)=PTSPHY*RG/(PAPH(JL,JK+1)-PAPH(JL,JK))
      ZRAIN=0.0_JPRB
      DO JM=1,NCLV
        IF (IPHASE(JM)==1) ZRAIN=ZRAIN+RALVDCP*ZDTGDP(JL)*ZPFPLSX(JL,JK+1,JM)* &
                           & (PAPH(JL,JK+1)-PAPH(JL,JK))
        IF (IPHASE(JM)==2) ZRAIN=ZRAIN+RALSDCP*ZDTGDP(JL)*ZPFPLSX(JL,JK+1,JM)* &
                           & (PAPH(JL,JK+1)-PAPH(JL,JK))
      ENDDO
      ZSUMH1(JL,JK)=(ZSUMH1(JL,JK)-ZRAIN)/PAPH(JL,JK+1)
      ZERRORH(JL,JK)=ZSUMH1(JL,JK)-ZSUMH0(JL,JK)
    ENDDO
  ENDDO

  DO JL=KIDIA,KFDIA
    IF (ABS(ZERRORQ(JL,KLEV))>1.e-13_JPRB.OR.ABS(ZERRORH(JL,KLEV))>1.e-13_JPRB) THEN
      ZQADJ=0.0_JPRB ! dummy statement
                     ! place totalview break here to catch non-conservation
    ENDIF
  ENDDO

  IF (ALLOCATED(ZSUMQ0))  DEALLOCATE(ZSUMQ0)
  IF (ALLOCATED(ZSUMQ1))  DEALLOCATE(ZSUMQ1)
  IF (ALLOCATED(ZSUMH0))  DEALLOCATE(ZSUMH0)
  IF (ALLOCATED(ZSUMH1))  DEALLOCATE(ZSUMH1)
  IF (ALLOCATED(ZERRORQ)) DEALLOCATE(ZERRORQ)
  IF (ALLOCATED(ZERRORH)) DEALLOCATE(ZERRORH)
  IF (ALLOCATED(ZTENKEEP))DEALLOCATE(ZTENKEEP)

ENDIF


!--------------------------------------------------
! Copy precipitation fraction into output variable
!-------------------------------------------------
!DO JL=KIDIA,KFDIA
!  PCOVPTOT(JL) = ZCOVPTOT(JL)
!ENDDO

!--------------------------------------------------------------------
! Copy general precip arrays back into PFP arrays for GRIB archiving
! Add rain and liquid fluxes, ice and snow fluxes
!--------------------------------------------------------------------
DO JK=1,KLEV+1
  DO JL=KIDIA,KFDIA
    PFPLSL(JL,JK) = ZPFPLSX(JL,JK,NCLDQR)+ZPFPLSX(JL,JK,NCLDQL)
    PFPLSN(JL,JK) = ZPFPLSX(JL,JK,NCLDQS)+ZPFPLSX(JL,JK,NCLDQI)
  ENDDO
ENDDO

!--------
! Fluxes:
!--------
DO JL=KIDIA,KFDIA
  PFSQLF(JL,1)  = 0.0_JPRB
  PFSQIF(JL,1)  = 0.0_JPRB
  PFCQLNG(JL,1) = 0.0_JPRB
  PFCQNNG(JL,1) = 0.0_JPRB
ENDDO
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZGDPH_R = -ZRG_R*(PAPH(JL,JK+1)-PAPH(JL,JK))*ZQTMST
    PFSQLF(JL,JK+1)  = PFSQLF(JL,JK)
    PFSQIF(JL,JK+1)  = PFSQIF(JL,JK)
    PFCQLNG(JL,JK+1) = PFCQLNG(JL,JK)
    PFCQNNG(JL,JK+1) = PFCQNNG(JL,JK)

    ZALFAW=ZFOEALFA(JL,JK)

    ! Liquid 
    PFSQLF(JL,JK+1)=PFSQLF(JL,JK+1)+ &
     &(ZQXN2D(JL,JK,NCLDQL)-ZQX0(JL,JK,NCLDQL)+PVFL(JL,JK)*PTSPHY-ZALFAW*PLUDE(JL,JK))*ZGDPH_R
    PFCQLNG(JL,JK+1)=PFCQLNG(JL,JK+1)+ZLNEG(JL,JK,NCLDQL)*ZGDPH_R
    
    ! Rain 
    PFSQLF(JL,JK+1)=PFSQLF(JL,JK+1)+ &
     &(ZQXN2D(JL,JK,NCLDQR)-ZQX0(JL,JK,NCLDQR))*ZGDPH_R
    PFCQLNG(JL,JK+1)=PFCQLNG(JL,JK+1)+ZLNEG(JL,JK,NCLDQR)*ZGDPH_R

    ! Ice 
    PFSQIF(JL,JK+1)=PFSQIF(JL,JK+1)+ &
     & (ZQXN2D(JL,JK,NCLDQI)-ZQX0(JL,JK,NCLDQI)+PVFI(JL,JK)*PTSPHY-(1.0_JPRB-ZALFAW)*PLUDE(JL,JK))*ZGDPH_R
    PFCQNNG(JL,JK+1)=PFCQNNG(JL,JK+1)+ZLNEG(JL,JK,NCLDQI)*ZGDPH_R

    ! Snow
    PFSQIF(JL,JK+1)=PFSQIF(JL,JK+1)+ &
     & (ZQXN2D(JL,JK,NCLDQS)-ZQX0(JL,JK,NCLDQS))*ZGDPH_R
    PFCQNNG(JL,JK+1)=PFCQNNG(JL,JK+1)+ZLNEG(JL,JK,NCLDQS)*ZGDPH_R

  ENDDO
ENDDO

!-----------------------------------
! enthalpy flux due to precipitation
!-----------------------------------
DO JK=1,KLEV+1
  DO JL=KIDIA,KFDIA
    PFHPSL(JL,JK) = -RLVTT*PFPLSL(JL,JK)
    PFHPSN(JL,JK) = -RLSTT*PFPLSN(JL,JK)
  ENDDO
ENDDO

ENDIF ! on LDMAINCALL ******************************


IF (LHOOK) CALL DR_HOOK('CLOUDSC',1,ZHOOK_HANDLE)
END SUBROUTINE CLOUDSC
