SUBROUTINE CONDTL &
 & ( KIDIA , KFDIA , KLON , KLEV,&
 & PTSPHY,&
 & PTM15  , PQM15,   PAPHP15, PAPP15,&
 & PTENT5,  PTENQ5,&
 & PFHPSL5, PFHPSN5, PFPLSL5, PFPLSN5,&
 & PTM1  , PQM1,   PAPHP1, PAPP1,&
 & PTENT,  PTENQ,&
 & PFHPSL, PFHPSN, PFPLSL, PFPLSN                      )  

!**** *CONDTL* - COMPUTES LARGE-SCALE WATER PHASE CHANGES.
!                (Tangent linear)   

!     PURPOSE.
!     --------

!          THIS ROUTINE COMPUTES THE PHYSICAL TENDENCIES OF THE TWO
!     PROGNOSTIC VARIABLES T AND Q DUE TO THE WATER PHASE CHANGES:
!     CONDENSATION (ASSUMED TO BE TOTAL FOR ANY SUPERSATURATION) AS SNOW
!     OR LIQUID WATER DEPENDING ON TEMPERATURE, EVAPORATION OF FALLING
!     PRECIPITATIONS IN UNSATURATED LAYERS AND MELTING OF
!     THE FALLING SNOW. THE WHOLE COMPUTATION IS AN INSTANTANEOUS
!     ADAPTATION PROCESS ON THE T+1 VALUES. RAIN AND SNOWFALL SURFACE
!     FLUXES, LATER TO BE USED FOR SOIL PROCESS ARE STORED.

!**   INTERFACE.
!     ----------

!          *CONDTL* IS CALLED FROM *CALLPARTL*.
!          THE ROUTINE TAKES ITS INPUT FROM THE LONG-TERM STORAGE:
!     T AND Q AT T+1 AND P ON LEVEL BOUNDARIES AT T+1 FROM WHICH THE P
!     FOR THE PROGNOSTIC LEVELS (NECESSARY FOR THE SATURATION MIXING
!     RATIO COMPUTATIONS) HAVE BEEN COMPUTED ASSUMING A 0.5/0.5 LINEAR
!     INTERPOLATION. IT RETURNS ITS OUTPUT TO THE SAME SPACE: TENDENCIES
!     OF T AND Q AND PRECIPITATION FLUXES FOR RAIN AND SNOW.

!     METHOD.
!     -------

!          THE ATMOSPHERIC COLUMN IS SCANNED FROM TOP TO BOTTOM
!     OBVIOUSLY. THE POSSIBLE MELTING OF INCOMING SNOW IN THE LAYER
!     IS ENVISAGED FIRST (NO ACTION IN THE UPMOST LAYER) AND OCCURS
!     AS SOON AS THE TEMPERATURE IS LARGER THAN 2 CELSIUS. THE
!     MELTING IS LIMITED BY THE SNOW AMOUNT EVIDENTLY AND THE
!     INDUCED COOLING WHICH SHOULD MAINTAIN THE TEMPERATURE ABOVE
!     THE 2 CELSIUS THRESHOLD BUT IRRESPECTIVELY OF ANY SATURATION
!     CRITERIA AT THIS STAGE.
!         THEN WITH REFERENCE TO THE NEW TEMPERATURE
!     AFTER MELTING A TEST OF THE
!     RELATIVE HUMIDITY IS FIRST PERFORMED. IF THERE IS SUPERSATURATION
!     TEMPERATURE AND RELATIVE HUMIDITY ARE MODIFIED TO BRING THE LAYER
!     BACK TO A STATE OF QUASI EXACT SATURATION (ESTIMATED FROM A FIRST
!     ORDER *TAYLOR DEVELOPMENT OF QS(T) FROM THE INITIAL TEMPERATURE).
!     THE AMOUNT OF PRECIPITATIONS OBTAINED IS ASSIGNED TO THE SNOW OR
!     RAIN FLUX DEPENDING ON THE LOCAL TEMPERATURE RELATIVE TO THE
!     FREEZING POINT OF WATER. IF THERE IS UNDERSATURATION THE
!     REMAINING PRECIPITATION (SNOW + RAIN) EVAPORATES
!     ACROSS THE LAYER:THE RATE IS SUCH AS TO EFFECTIVELY MOISTEN THE
!     SUB CLOUD-LAYERS.
!          AT THE END THE SURFACE VALUES FOR RAIN AND SNOW FALL ARE
!     PREPARED.

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS

!     INPUT PARAMETERS (LOGICAL)

!     INPUT PARAMETERS (REAL)

!    *PTSPHY*       TIME STEP FOR THE PHYSICS                         S
!    *PTM15*        TEMPERATURE (T-1)                           (Trajectory)            
!    *PQM15*        SPECIFIC HUMIDITY (T-1)                     (Trajectory)           
!    *PAPHP15*      PROVISIONAL PRESSURE ON HALF LEVELS         (Trajectory)
!    *PAPP15*       PROVISIONAL PRESSURE ON FULL LEVELS         (Trajectory)
!    *PTM1*         TEMPERATURE (T-1)                                 K
!    *PQM1*         SPECIFIC HUMIDITY (T-1)                           KG/KG
!    *PAPHP1*       PROVISIONAL PRESSURE ON HALF LEVELS               PA
!    *PAPP1*        PROVISIONAL PRESSURE ON FULL LEVELS               PA

!    UPDATED PARAMETERS (REAL):

!    *PTENT5*       TEMPERATURE TENDENCY                        (Trajectory)       
!    *PTENQ5*       MOISTURE TENDENCY                           (Trajectory)       
!    *PTENT*        TEMPERATURE TENDENCY                          K/S
!    *PTENQ*        MOISTURE TENDENCY                            KG/(KG S)

!    OUTPUT PARAMETERS (REAL):

!    *PFHPSL5*      ENTHALPY FLUX DUE TO LARGE SCALE RAIN       (Trajectory)
!    *PFHPSN5*      ENTHALPY FLUX DUE TO LARGE SCALE SNOW       (Trajectory)
!    *PFPLSL5*      LARGE SCALE RAIN FLUX                       (Trajectory)
!    *PFPLSN5*      LARGE SCALE SNOW FLUX                       (Trajectory)
!    *PFHPSL*       ENTHALPY FLUX DUE TO LARGE SCALE RAIN         J/(M2*S)
!    *PFHPSN*       ENTHALPY FLUX DUE TO LARGE SCALE SNOW         J/(M2*S)
!    *PFPLSL*       LARGE SCALE RAIN FLUX                        KG/(M2*S)
!    *PFPLSN*       LARGE SCALE SNOW FLUX                        KG/(M2*S)

!     EXTERNALS.
!     ----------

!          *CUADJTQTL*   ADJUST T AND Q DUE TO CONDENSATION (Tangent linear)

!     REFERENCE.
!     ----------

!          SEE LARGE SCALE PHASE CHANGES' PART OF THE MODEL'S
!     DOCUMENTATION FOR DETAILS ABOUT THE MATHEMATICS OF THIS ROUTINE.

!     MODIFICATIONS
!     -------------
!        92-09-17 : Update to Cy44      J.-J. MORCRETTE
!        95-11-09 : Adaptation for TL/AD versions J.-F. MAHFOUF
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Ko"hler     03-Dec-2004 cpmoist using RVTMP2
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,R        ,RCPD     ,&
 & RLVTT    ,RLSTT    ,RLMLT    ,RTT  
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE   ,RTICECU   ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R   ,RVTMP2  
USE YOECND   , ONLY : REPFLM
USE YOEPHLI  , ONLY : LPHYLIN  ,RLPTRC   ,RLPEVAP
USE YOMLCZ   , ONLY : LRAIN  ,  RLRAIN   ,NRAINSTART
USE YOMCT3   , ONLY : NSTEP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHP15(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPP15(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSL5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSN5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLSL5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLSN5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHP1(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPP1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSL(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFHPSN(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLSL(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFPLSN(KLON,KLEV+1) 

LOGICAL ::  LLFLAG(KLON)

REAL(KIND=JPRB) :: ZRFL5(KLON),                ZSFL5(KLON),&
 & ZTP15(KLON),                ZQP15(KLON),&
 & ZLVDCP5(KLON),              ZLSDCP5(KLON),&
 & ZDP5(KLON),                 ZDR5(KLON),                 ZLFDCP5(KLON),&
 & ZRFLN5(KLON),               ZSFLN5(KLON),&
 & ZFLN5(KLON),                 &
 & ZDTDT5(KLON),               ZDQDT5(KLON)  
REAL(KIND=JPRB) :: ZPP5(KLON)     , ZQOLD5(KLON)
REAL(KIND=JPRB) :: ZTT5(KLON,KLEV), ZQQ5(KLON,KLEV)

REAL(KIND=JPRB) :: ZRFL(KLON),                 ZSFL(KLON),&
 & ZTP1(KLON),                 ZQP1(KLON),&
 & ZLVDCP(KLON),               ZLSDCP(KLON),&
 & ZDP(KLON),                  ZDR(KLON),                  ZLFDCP(KLON),&
 & ZRFLN(KLON),                ZSFLN(KLON),&
 & ZFLN(KLON),                 &
 & ZDTDT(KLON),                ZDQDT(KLON)  
REAL(KIND=JPRB) :: ZPP(KLON)     , ZQOLD(KLON)
REAL(KIND=JPRB) :: ZTT(KLON,KLEV), ZQQ(KLON,KLEV)

INTEGER(KIND=JPIM) :: ICALL, IK, JK, JL

REAL(KIND=JPRB) :: Z2S, Z2S5, Z3S, Z3S5, Z4S, Z4S5,&
 & ZALFAW, ZALFAW5, ZCEVAP, ZCONS, ZCONS2, &
 & ZCONS5, ZMELTP2, ZRN, ZRN5, ZSN, ZSN5, ZSNMLT, &
 & ZSNMLT5, ZTMST, ZZ2S, ZZ2S5, ZZZ, ZZZ5  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cuadjtqstl.intfb.h"

#include "fcttre.h"
!     ------------------------------------------------------------------

!*    Computational constants
!     -----------------------

IF (LHOOK) CALL DR_HOOK('CONDTL',0,ZHOOK_HANDLE)
ZTMST=PTSPHY
ZCONS2 =1.0_JPRB/(ZTMST*RG)
ZMELTP2=RTT+2.0_JPRB

!     Fraction of rainfall evaporation in subsaturated layers
!     -------------------------------------------------------

IF (LPHYLIN) THEN
  ZCEVAP=RLPEVAP
ELSE
  ZCEVAP=0.95_JPRB
ENDIF

!     ------------------------------------------------------------------

!*         2.     TOP BOUNDARY CONDITION AND START OF VERTICAL LOOP.
!                 --- -------- --------- --- ----- -- -------- -----

!*         2.1     SET TO ZERO PRECIPITATION FLUXES AT THE TOP.

DO JL=KIDIA,KFDIA
  ZRFL   (JL)  =0.0_JPRB
  ZSFL   (JL)  =0.0_JPRB
  PFPLSL (JL,1)=0.0_JPRB
  PFPLSN (JL,1)=0.0_JPRB
  ZRFL5  (JL)  =0.0_JPRB
  ZSFL5  (JL)  =0.0_JPRB
  PFPLSL5(JL,1)=0.0_JPRB
  PFPLSN5(JL,1)=0.0_JPRB
ENDDO

!*         2.2     VERTICAL LOOP.

DO JK=1,KLEV

!     ------------------------------------------------------------------

!*         3.     COMPUTATION OF THE CONDENSATION RATES.
!                 ----------- -- --- ------------ ------

!*         3.1     T AND Q PROVISIONAL VALUES AT T+1 AND EFFECTIVES L/CP
!*                 FOR VAPORIZATION AND SUBLIMATION.

  DO JL=KIDIA,KFDIA
    ZDP (JL)=PAPHP1 (JL,JK+1)-PAPHP1 (JL,JK)
    ZDP5(JL)=PAPHP15(JL,JK+1)-PAPHP15(JL,JK)
    ZTP1 (JL)=PTM1 (JL,JK)+ZTMST*PTENT (JL,JK)
    ZTP15(JL)=PTM15(JL,JK)+ZTMST*PTENT5(JL,JK)
    ZQP1 (JL)=PQM1 (JL,JK)+ZTMST*PTENQ (JL,JK)
    ZQP15(JL)=PQM15(JL,JK)+ZTMST*PTENQ5(JL,JK)
    ZZZ = -RCPD*RVTMP2*ZQP1(JL)/(RCPD+RCPD*RVTMP2*ZQP15(JL))**2
    ZZZ5=1.0_JPRB/(RCPD+RCPD*RVTMP2*ZQP15(JL))
    ZLFDCP (JL)=RLMLT*ZZZ
    ZLFDCP5(JL)=RLMLT*ZZZ5
    ZLVDCP (JL)=RLVTT*ZZZ
    ZLVDCP5(JL)=RLVTT*ZZZ5
    ZLSDCP (JL)=RLSTT*ZZZ
    ZLSDCP5(JL)=RLSTT*ZZZ5
    ZPP (JL)=PAPP1 (JL,JK)
    ZPP5(JL)=PAPP15(JL,JK)
    ZTT (JL,JK)=ZTP1 (JL)
    ZTT5(JL,JK)=ZTP15(JL)
    ZQQ (JL,JK)=ZQP1 (JL)
    ZQQ5(JL,JK)=ZQP15(JL)
    ZQOLD (JL)=ZQP1 (JL)
    ZQOLD5(JL)=ZQP15(JL)
    LLFLAG(JL)=.TRUE.
  ENDDO

!*         3.2    MELTING OF INCOMING SNOW

  DO JL=KIDIA,KFDIA

    IF (ZSFL5(JL) /= 0.0_JPRB) THEN
      ZCONS =ZCONS2*(ZDP(JL)*ZLFDCP5(JL)-&
       & ZDP5(JL)*ZLFDCP(JL))/ZLFDCP5(JL)**2    
      ZCONS5=ZCONS2*ZDP5(JL)/ZLFDCP5(JL)
      IF ((ZTP15(JL)-ZMELTP2) > 0.0_JPRB) THEN
        ZZ2S =ZCONS5*ZTP1 (JL)+ZCONS*(ZTP15(JL)-ZMELTP2)
        ZZ2S5=ZCONS5*(ZTP15(JL)-ZMELTP2)
      ELSE
        ZZ2S =0.0_JPRB
        ZZ2S5=0.0_JPRB
      ENDIF
      IF (ZSFL5(JL) <= ZZ2S5) THEN
        ZSNMLT =ZSFL (JL)
        ZSNMLT5=ZSFL5(JL)
      ELSE
        ZSNMLT =ZZ2S
        ZSNMLT5=ZZ2S5
      ENDIF
      ZRFLN (JL)=ZRFL (JL)+ZSNMLT
      ZRFLN5(JL)=ZRFL5(JL)+ZSNMLT5
      ZSFLN (JL)=ZSFL (JL)-ZSNMLT
      ZSFLN5(JL)=ZSFL5(JL)-ZSNMLT5
      ZTP1 (JL)=ZTP1 (JL)-(ZSNMLT*ZCONS5-ZCONS*ZSNMLT5)/ZCONS5**2
      ZTP15(JL)=ZTP15(JL)- ZSNMLT5/ZCONS5
    ELSE
      ZRFLN (JL)=ZRFL (JL)
      ZRFLN5(JL)=ZRFL5(JL)
      ZSFLN (JL)=ZSFL (JL)
      ZSFLN5(JL)=ZSFL5(JL)
    ENDIF

  ENDDO

!*              3.3    NEW PRECIPITATION FLUXES AFTER CONDENSATION
!*                     AND TOTAL FLUXES

  IK=JK
  ICALL=0
  CALL CUADJTQSTL ( KIDIA, KFDIA, KLON, KLEV, IK,&
   & ZPP5 , ZTT5 , ZQQ5,&
   & ZPP  , ZTT  , ZQQ , LLFLAG, ICALL  )  

  DO JL=KIDIA,KFDIA
    ZDR (JL)=ZCONS2*(ZDP(JL)*(ZQOLD5(JL)-ZQQ5(JL,JK))+&
     & ZDP5(JL)*(ZQOLD(JL)-ZQQ(JL,JK)))  
    ZDR5(JL)=ZCONS2*ZDP5(JL)*(ZQOLD5(JL)-ZQQ5(JL,JK))
    ZALFAW =0.545_JPRB*0.17_JPRB*ZTP1(JL)/COSH(0.17_JPRB*(ZTP15(JL)-RLPTRC))**2
    ZALFAW5=0.545_JPRB*(1.0_JPRB+TANH(0.17_JPRB*(ZTP15(JL)-RLPTRC)))
    IF (ZDR5(JL) > 0.0_JPRB) THEN
      Z2S =ZDR (JL)
      Z2S5=ZDR5(JL)
    ELSE
      Z2S =0.0_JPRB
      Z2S5=0.0_JPRB
    ENDIF
    ZRN =    ZALFAW   * Z2S5 +     ZALFAW5  * Z2S
    ZRN5=    ZALFAW5  * Z2S5
    ZSN =  - ZALFAW   * Z2S5 + (1.0_JPRB-ZALFAW5) * Z2S
    ZSN5=(1.0_JPRB-ZALFAW5) * Z2S5

!          NEW PRECIPITATION FLUXES

    ZRFLN (JL)=ZRFLN (JL)+ZRN
    ZRFLN5(JL)=ZRFLN5(JL)+ZRN5
    ZSFLN (JL)=ZSFLN (JL)+ZSN
    ZSFLN5(JL)=ZSFLN5(JL)+ZSN5

!          TOTAL FLUXES

    ZFLN (JL)=ZRFLN (JL)+ZSFLN (JL)
    ZFLN5(JL)=ZRFLN5(JL)+ZSFLN5(JL)

  ENDDO

!*         3.4     AT TOP LAYER OR IF THERE ARE NO PRECIPITATION AVOID
!*                 EVAPORATION COMPUTATION

!     ------------------------------------------------------------------

!*         4.     COMPUTATION OF EVAPORATION
!*                ----------- -- -----------

  DO JL=KIDIA,KFDIA
    IF (ZFLN5(JL) /= 0.0_JPRB.AND.JK > 1) THEN
      IF ((ZDR5(JL)*ZCEVAP) <= 0.0_JPRB) THEN
        Z3S =ZDR (JL)*ZCEVAP
        Z3S5=ZDR5(JL)*ZCEVAP
      ELSE
        Z3S =0.0_JPRB
        Z3S5=0.0_JPRB
      ENDIF
      IF ((ZFLN5(JL)+Z3S5) > 0.0_JPRB) THEN
        ZFLN (JL)=ZFLN (JL)+Z3S
        ZFLN5(JL)=ZFLN5(JL)+Z3S5
      ELSE
        ZFLN (JL)=0.0_JPRB
        ZFLN5(JL)=0.0_JPRB
      ENDIF
      IF ((ZRFLN5(JL)+ZSFLN5(JL)) > REPFLM) THEN
        Z4S =ZRFLN (JL)+ZSFLN (JL)
        Z4S5=ZRFLN5(JL)+ZSFLN5(JL)
      ELSE
        Z4S =0.0_JPRB
        Z4S5=REPFLM
      ENDIF
      ZRFLN (JL)=((ZRFLN5(JL)*ZFLN(JL)+ZRFLN(JL)*ZFLN5(JL))*Z4S5-&
       & Z4S*ZRFLN5(JL)*ZFLN5(JL))/Z4S5**2  
      ZRFLN5(JL)=ZRFLN5(JL)*ZFLN5(JL)/Z4S5
      ZSFLN (JL)=((ZSFLN5(JL)*ZFLN(JL)+ZSFLN(JL)*ZFLN5(JL))*Z4S5-&
       & Z4S*ZSFLN5(JL)*ZFLN5(JL))/Z4S5**2  
      ZSFLN5(JL)=ZSFLN5(JL)*ZFLN5(JL)/Z4S5
    ENDIF
  ENDDO

!     ------------------------------------------------------------------

!*         6.     INCREMENTATION OF T AND Q TENDENCIES AND FLUXES' SWAP.
!                 -------------- -- - --- - ---------- --- ------- -----

!*         6.1     RESUME COMPUTATIONS FOR THE TOP LAYER.

!*         6.2     MODIFICATION OF THE T AND Q TENDENCIES.

  DO JL=KIDIA,KFDIA
    ZDQDT (JL)=-RG*(((ZRFLN(JL)-ZRFL(JL))+(ZSFLN(JL)-ZSFL(JL)))&
     & *ZDP5(JL)-ZDP(JL)*((ZRFLN5(JL)-ZRFL5(JL))+&
     & (ZSFLN5(JL)-ZSFL5(JL))))/ZDP5(JL)**2  
    ZDQDT5(JL)=-((ZRFLN5(JL)-ZRFL5(JL))+(ZSFLN5(JL)-ZSFL5(JL)))&
     & *(RG/ZDP5(JL))  
    PTENQ (JL,JK)=PTENQ (JL,JK)+ZDQDT (JL)
    PTENQ5(JL,JK)=PTENQ5(JL,JK)+ZDQDT5(JL)
    ZDTDT (JL)=((ZLVDCP5(JL)*(ZRFLN(JL)-ZRFL(JL))+ZLSDCP5(JL)&
     & *(ZSFLN(JL)-ZSFL(JL))+&
     & ZLVDCP(JL)*(ZRFLN5(JL)-ZRFL5(JL))+ZLSDCP(JL)&
     & *(ZSFLN5(JL)-ZSFL5(JL)))*ZDP5(JL)-ZDP(JL)*&
     & (ZLVDCP5(JL)*(ZRFLN5(JL)-ZRFL5(JL))+ZLSDCP5(JL)&
     & *(ZSFLN5(JL)-ZSFL5(JL))))*RG/ZDP5(JL)**2         
    ZDTDT5(JL)=(ZLVDCP5(JL)*(ZRFLN5(JL)-ZRFL5(JL))+ZLSDCP5(JL)&
     & *(ZSFLN5(JL)-ZSFL5(JL)))*(RG/ZDP5(JL))  
    PTENT (JL,JK)=PTENT (JL,JK)+ZDTDT (JL)
    PTENT5(JL,JK)=PTENT5(JL,JK)+ZDTDT5(JL)
    PFPLSL (JL,JK+1)=ZRFLN (JL)
    PFPLSL5(JL,JK+1)=ZRFLN5(JL)
    PFPLSN (JL,JK+1)=ZSFLN (JL)
    PFPLSN5(JL,JK+1)=ZSFLN5(JL)
  ENDDO

!*         6.4     SWAP OF THE FLUXES AND END OF THE VERTICAL LOOP.

  DO JL=KIDIA,KFDIA
    ZRFL (JL)=ZRFLN (JL)
    ZRFL5(JL)=ZRFLN5(JL)
    ZSFL (JL)=ZSFLN (JL)
    ZSFL5(JL)=ZSFLN5(JL)
  ENDDO

ENDDO

!*         6.5     CUMULATIVE RAIN

IF (LRAIN.AND.(NSTEP >= NRAINSTART)) THEN
  IF (NSTEP == NRAINSTART) RLRAIN(:)=0.0_JPRB
  DO JL=KIDIA,KFDIA
    RLRAIN(JL)=RLRAIN(JL)+PFPLSL(JL,KLEV+1)
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*         7.     ENTHALPY FLUXES DUE TO PRECIPITATION
!                 ------------------------------------

DO JK=1,KLEV+1
  DO JL=KIDIA,KFDIA
    PFHPSL (JL,JK)=-PFPLSL (JL,JK)*RLVTT
    PFHPSL5(JL,JK)=-PFPLSL5(JL,JK)*RLVTT
    PFHPSN (JL,JK)=-PFPLSN (JL,JK)*RLSTT
    PFHPSN5(JL,JK)=-PFPLSN5(JL,JK)*RLSTT
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CONDTL',1,ZHOOK_HANDLE)
END SUBROUTINE CONDTL
