SUBROUTINE CUDDRAFN2TL &
 & (KIDIA,    KFDIA,    KLON,    KLEV,&
 & KCBOT,    KCTOP,    LDCUM,&
 & PTENH5,   PQENH5,&
 & PTEN5,    PQSEN5,   PGEO5,&
 & PGEOH5,   PAPH5,    PTU5,     PQU5,&
 & PRFL5,    PTD5,     PQD5,     PMFU5,&
 & PMFD5,    PMFDS5,   PMFDQ5,   PDMFDP5,&
 & KDTOP,    LDDRAF,&
 & PMFDDE_RATE5, PERATED5,&
 & PTENH,    PQENH,&
 & PGEOH,    PAPH,     PTU,      PQU,&
 & PRFL,     PTD,      PQD,&
 & PMFD,     PMFDS,    PMFDQ,    PDMFDP,&
 & PMFDDE_RATE, PERATED )  

!          THIS ROUTINE CALCULATES LEVEL OF FREE SINKING FOR
!          CUMULUS DOWNDRAFTS, PROFILES OF T,Q,U AND V VALUES,
!          AND PROFILE OF Md(Z)/Md(LFS).
!          (TANGENT LINEAR VERSION)

!          P.LOPEZ      E.C.M.W.F.   March 2002 
!          Inspired from Tietdke's original scheme (1989/1993).

!          PURPOSE.
!          --------
!          TO PRODUCE CUMULUS DOWNDRAFTS FOR THE
!          SIMPLIFIED MASSFLUX CUMULUS PARAMETERIZATION

!          INTERFACE
!          ---------
!          THIS ROUTINE IS CALLED FROM *CUMASTRNTL*.
!          INPUT ARE ENVIRONMENTAL VALUES OF T,Q,U,V,P,PHI
!          AND UPDRAFT VALUES T,Q,U AND V AND ALSO
!          CLOUD BASE MASSFLUX AND CU-PRECIPITATION RATE.
!          IT RETURNS T,Q,U AND V VALUES AND MASSFLUX AT LFS.

!          METHOD.

!          CHECK FOR NEGATIVE BUOYANCY OF AIR OF EQUAL PARTS OF
!          MOIST ENVIRONMENTAL AIR AND CLOUD AIR.

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS
!    *KCBOT*        CLOUD BASE LEVEL
!    *KCTOP*        CLOUD TOP LEVEL

!    INPUT PARAMETERS (LOGICAL):

!    *LDCUM*        FLAG: .TRUE. FOR CONVECTIVE POINTS

!    INPUT PARAMETERS (REAL):

!    *PTENH5*       ENV. TEMPERATURE (T+1) ON HALF LEVELS        (Trajectory)
!    *PQENH5*       ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS     (Trajectory)
!    *PTEN5*        PROVISIONAL ENVIRONMENT TEMPERATURE (T+1)    (Trajectory)
!    *PQSEN5*       ENVIRONMENT SPEC. SATURATION HUMIDITY (T+1)  (Trajectory)
!    *PGEO5*        GEOPOTENTIAL                                 (Trajectory)
!    *PGEOH5*       GEOPOTENTIAL ON HALF LEVELS                  (Trajectory)
!    *PAPH5*        PROVISIONAL PRESSURE ON HALF LEVELS          (Trajectory)
!    *PTU5*         TEMPERATURE IN UPDRAFTS                      (Trajectory)
!    *PQU5*         SPEC. HUMIDITY IN UPDRAFTS                   (Trajectory)
!    *PMFU5*        MASSFLUX IN UPDRAFTS                         (Trajectory)

!    *PTENH*        ENV. TEMPERATURE (T+1) ON HALF LEVELS          K
!    *PQENH*        ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS     KG/KG
!    *PGEOH*        GEOPOTENTIAL ON HALF LEVELS                  M2/S2
!    *PAPH*         PROVISIONAL PRESSURE ON HALF LEVELS           PA
!    *PTU*          TEMPERATURE IN UPDRAFTS                        K
!    *PQU*          SPEC. HUMIDITY IN UPDRAFTS                   KG/KG

!    UPDATED PARAMETERS (REAL):

!    *PRFL5*        PRECIPITATION RATE                           (Trajectory)

!    *PRFL*         PRECIPITATION RATE                           KG/(M2*S)

!    OUTPUT PARAMETERS (REAL):

!    *PTD5*         TEMPERATURE IN DOWNDRAFTS                    (Trajectory)
!    *PQD5*         SPEC. HUMIDITY IN DOWNDRAFTS                 (Trajectory)
!    *PMFD5*        MASSFLUX IN DOWNDRAFTS                       (Trajectory)
!    *PMFDS5*       FLUX OF DRY STATIC ENERGY IN DOWNDRAFTS      (Trajectory)
!    *PMFDQ5*       FLUX OF SPEC. HUMIDITY IN DOWNDRAFTS         (Trajectory)
!    *PDMFDP5*      FLUX DIFFERENCE OF PRECIP. IN DOWNDRAFTS     (Trajectory)
!    *PMFDDE_RATE5* DOWNDRAFT DETRAINMENT RATE                   (Trajectory)
!    *PERATED5*     DOWNDRAFT ENTRAINMENT                        (Trajectory)

!    *PTD*          TEMPERATURE IN DOWNDRAFTS                      K
!    *PQD*          SPEC. HUMIDITY IN DOWNDRAFTS                 KG/KG
!    *PMFD*         MASSFLUX IN DOWNDRAFTS                       KG/(M2*S)
!    *PMFDS*        FLUX OF DRY STATIC ENERGY IN DOWNDRAFTS       J/(M2*S)
!    *PMFDQ*        FLUX OF SPEC. HUMIDITY IN DOWNDRAFTS         KG/(M2*S)
!    *PDMFDP*       FLUX DIFFERENCE OF PRECIP. IN DOWNDRAFTS     KG/(M2*S)
!    *PMFDDE_RATE*  DOWNDRAFT DETRAINMENT RATE                   KG/(M2*S)
!    *PERATED*      DOWNDRAFT ENTRAINMENT                  

!    OUTPUT PARAMETERS (INTEGER):

!    *KDTOP*        TOP LEVEL OF DOWNDRAFTS

!    OUTPUT PARAMETERS (LOGICAL):

!    *LDDRAF*       .TRUE. IF DOWNDRAFTS EXIST

!    EXTERNALS
!    ---------
!    *CUADJTQSTL* FOR CALCULATING WET BULB T AND Q AT LFS

!    MODIFICATIONS
!    -------------
!      M.Hamrud      01-Oct-2003 CY28 Cleaning
!      P. Lopez      11-01-2007 Removed LPHYLIN switch
!      P. Lopez      20-06-2007 Bug correction in latent heat ZOELHM5
!      P. Lopez      04-09-2007 Revision to improve match to Tiedtke scheme
!      (Note: !LCO = useless lines, but should not be removed for clarity!)

!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RCPD     ,RETV     ,RLVTT    ,RLSTT    ,RG  
USE YOECUMF2 , ONLY : RMFDEPS2 ,NJKT32   ,ENTRDD2
USE YOEPHLI  , ONLY : RLPTRC

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCBOT(KLON)
INTEGER(KIND=JPIM),INTENT(IN)    :: KCTOP(KLON)
LOGICAL           ,INTENT(IN)    :: LDCUM(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQSEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEO5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRFL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMFU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDS5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDMFDP5(KLON,KLEV) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KDTOP(KLON) 
LOGICAL           ,INTENT(INOUT) :: LDDRAF(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDDE_RATE5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PERATED5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQENH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDMFDP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDDE_RATE(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PERATED(KLON,KLEV)

! local arrays 
INTEGER(KIND=JPIM) :: IKHSMIN(KLON)

REAL(KIND=JPRB) :: ZTENWB5(KLON,KLEV),  ZQENWB5(KLON,KLEV),&
                 & ZCOND25(KLON),       ZPH5(KLON),&
                 & ZHSMIN5(KLON),       ZSDH5(KLON,KLEV),&
                 & ZSENH5(KLON,KLEV),   ZDTOPDE5(KLON)

REAL(KIND=JPRB) :: ZTENWB(KLON,KLEV),   ZQENWB(KLON,KLEV),&
                 & ZCOND2(KLON),        ZPH(KLON),&
                 & ZSDH(KLON,KLEV),     ZSENH(KLON,KLEV),&
                 & ZDTOPDE(KLON)
!LCO REAL(KIND=JPRB) :: ZHSMIN(KLON)

LOGICAL ::  LLO2(KLON)

INTEGER(KIND=JPIM) :: ICALL, IK, IKE, IS, JK, JL, ITOPDE

REAL(KIND=JPRB) :: ZBUO5, ZHSK5, ZOEALFA5, ZDZ5,&
                 & ZOELHM5, ZQTEST5, ZTARG5, ZTTEST5, ZCOND15,&
                 & ZENT5, ZEPS5, ZDEL5, ZRAIN5, ZDMFEN5, ZDMFDE5
                
REAL(KIND=JPRB) :: ZMFTOP, ZDZ, ZQTEST, ZTTEST, ZCOND1,&
                 & ZEPS, ZDEL, ZENT, ZCD, ZDMFEN, ZDMFDE,&
                 & ZFACT1, ZRG, ZRCPD, ZRFLMIN
!LCO REAL(KIND=JPRB) :: ZHSK, ZOELHM, ZTARG, ZOEALFA, ZBUO, ZRAIN

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cuadjtqstl.intfb.h"

!----------------------------------------------------------------------

!     0.           INITIALIZATION OF CONSTANTS AND ARRAYS
!                  --------------------------------------

IF (LHOOK) CALL DR_HOOK('CUDDRAFN2TL',0,ZHOOK_HANDLE)
ZCD=0.3_JPRB
ZRG=1.0_JPRB/RG
ZRCPD=1.0_JPRB/RCPD

ZSDH5(:,:)=0.0_JPRB
ZSDH(:,:)=0.0_JPRB

ITOPDE=NJKT32

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZSENH5(JL,JK) = RCPD*PTENH5(JL,JK)+PGEOH5(JL,JK)
    ZSENH (JL,JK) = RCPD*PTENH (JL,JK)+PGEOH (JL,JK)
  ENDDO
ENDDO

!     1.           SET DEFAULT VALUES FOR DOWNDRAFTS
!                  ---------------------------------

DO JL=KIDIA,KFDIA
  LDDRAF(JL)=.FALSE.
  KDTOP(JL)=KLEV+1
  IKHSMIN(JL)=KLEV+1
  ZHSMIN5(JL)=1.E8_JPRB
!LCO  ZHSMIN (JL)=0.0_JPRB
  ZFACT1=1.0_JPRB/(PGEOH5(JL,ITOPDE)-PGEOH5(JL,KLEV+1))
  ZDTOPDE5(JL)= RG*ZFACT1
  ZDTOPDE (JL)=-RG*(PGEOH (JL,ITOPDE)-PGEOH (JL,KLEV+1))*ZFACT1**2
ENDDO

!----------------------------------------------------------------------

!     2.           DETERMINE LEVEL OF FREE SINKING:
!                  DOWNDRAFTS SHALL START AT MODEL LEVEL OF MINIMUM
!                  OF SATURATION MOIST STATIC ENERGY OR BELOW
!                  RESPECTIVELY

!                  FOR EVERY POINT AND PROCEED AS FOLLOWS:

!                    (1) DETERMINE LEVEL OF MINIMUM OF HS
!                    (2) DETERMINE WET BULB ENVIRONMENTAL T AND Q
!                    (3) DO MIXING WITH CUMULUS CLOUD AIR
!                    (4) CHECK FOR NEGATIVE BUOYANCY
!                    (5) IF BUOYANCY>0 REPEAT (2) TO (4) FOR NEXT
!                        LEVEL BELOW

!                  THE ASSUMPTION IS THAT AIR OF DOWNDRAFTS IS MIXTURE
!                  OF 50% CLOUD AIR + 50% ENVIRONMENTAL AIR AT WET BULB
!                  TEMPERATURE (I.E. WHICH BECAME SATURATED DUE TO
!                  EVAPORATION OF RAIN AND CLOUD WATER)
!                  ----------------------------------------------------

DO JK=3,KLEV-2
  DO JL=KIDIA,KFDIA
    ZTARG5=PTEN5(JL,JK)
!LCO    ZTARG=PTEN(JL,JK)
    ZOEALFA5=0.545_JPRB*(TANH(0.17_JPRB*(ZTARG5-RLPTRC))+1.0_JPRB)
!LCO    ZOEALFA=0.545_JPRB*(1.0_JPRB-TANH(0.17_JPRB*(ZTARG5-RLPTRC))**2) &
!LCO     & *0.17_JPRB*ZTARG  
    ZOELHM5=ZOEALFA5*RLVTT+(1.0_JPRB-ZOEALFA5)*RLSTT
!LCO    ZOELHM =ZOEALFA*RLVTT-ZOEALFA*RLSTT  
    ZHSK5=RCPD*PTEN5(JL,JK)+PGEO5(JL,JK)+ZOELHM5*PQSEN5(JL,JK)
!LCO    ZHSK=RCPD*PTEN(JL,JK)+PGEO(JL,JK)+ZOELHM5*PQSEN(JL,JK) &
!LCO     & +ZOELHM*PQSEN5(JL,JK)  
    IF(ZHSK5 < ZHSMIN5(JL)) THEN
      ZHSMIN5(JL)=ZHSK5
!LCO      ZHSMIN(JL)=ZHSK
      IKHSMIN(JL)=JK
    ENDIF
  ENDDO
ENDDO
IKE=KLEV-3
DO JK=3,IKE

!     2.1          CALCULATE WET-BULB TEMPERATURE AND MOISTURE
!                  FOR ENVIRONMENTAL AIR IN *CUADJTQ*
!                  -------------------------------------------

  IS=0
  DO JL=KIDIA,KFDIA
    ZTENWB5(JL,JK)=PTENH5(JL,JK)
    ZTENWB(JL,JK)=PTENH(JL,JK)
    ZQENWB5(JL,JK)=PQENH5(JL,JK)
    ZQENWB(JL,JK)=PQENH(JL,JK)
    ZPH5(JL)=PAPH5(JL,JK)
    ZPH(JL)=PAPH(JL,JK)
    LLO2(JL)=LDCUM(JL).AND.PRFL5(JL) > 0.0_JPRB.AND..NOT.LDDRAF(JL).AND.&
     & (JK < KCBOT(JL).AND.JK > KCTOP(JL)).AND.&
     & JK >= IKHSMIN(JL)  
    IF(LLO2(JL))THEN
      IS=IS+1
    ENDIF
  ENDDO
  IF(IS == 0) CYCLE

  IK=JK
  ICALL=2
  CALL CUADJTQSTL &
   & ( KIDIA,    KFDIA,    KLON,    KLEV,&
   & IK,&
   & ZPH5,     ZTENWB5,  ZQENWB5,&
   & ZPH,      ZTENWB,   ZQENWB,   LLO2,     ICALL)  

!     2.2          DO MIXING OF CUMULUS AND ENVIRONMENTAL AIR
!                  AND CHECK FOR NEGATIVE BUOYANCY.
!                  THEN SET VALUES FOR DOWNDRAFT AT LFS.
!                  ----------------------------------------

!DIR$ IVDEP
!OCL NOVREC
  DO JL=KIDIA,KFDIA
    IF(LLO2(JL)) THEN
      ZTTEST5=0.5_JPRB*(PTU5(JL,JK)+ZTENWB5(JL,JK))
      ZTTEST =0.5_JPRB*(PTU (JL,JK)+ZTENWB (JL,JK))
      ZQTEST5=0.5_JPRB*(PQU5(JL,JK)+ZQENWB5(JL,JK))
      ZQTEST =0.5_JPRB*(PQU (JL,JK)+ZQENWB (JL,JK))
      ZBUO5= ZTTEST5*(1.0_JPRB+RETV*ZQTEST5) &
         & - PTENH5(JL,JK)*(1.0_JPRB+RETV*PQENH5(JL,JK))  
!LCO      ZBUO= ZTTEST*(1.0_JPRB+RETV*ZQTEST5) &
!LCO       & + ZTTEST5*RETV*ZQTEST &
!LCO       & - PTENH(JL,JK)*(1.0_JPRB+RETV*PQENH5(JL,JK)) &
!LCO       & - PTENH5(JL,JK)*RETV*PQENH(JL,JK)  
!LCO!  Reduce buoyancy perturbation if required
!LCO      IF (LREGCV) THEN
!LCO        ZBUO=ZBUO*0.35_JPRB
!LCO      ENDIF
      ZCOND15=PQENH5(JL,JK)-ZQENWB5(JL,JK)
      ZCOND1 =PQENH (JL,JK)-ZQENWB (JL,JK)
      ZMFTOP=-RMFDEPS2
      ZRFLMIN=10._JPRB*ZMFTOP*ZCOND15
      IF(ZBUO5 < 0.0_JPRB .AND. PRFL5(JL) > ZRFLMIN) THEN
        KDTOP(JL)=JK
        LDDRAF(JL)=.TRUE.
        PTD5(JL,JK)=ZTTEST5
        PTD (JL,JK)=ZTTEST
        PQD5(JL,JK)=ZQTEST5
        PQD (JL,JK)=ZQTEST
        ZSDH5(JL,JK)=RCPD*PTD5(JL,JK)+PGEOH5(JL,JK)
        ZSDH (JL,JK)=RCPD*PTD (JL,JK)+PGEOH (JL,JK)
        PMFD5(JL,JK)=ZMFTOP
        PMFD (JL,JK)=0.0_JPRB
        PMFDS5(JL,JK)=PTD5(JL,JK)
        PMFDS (JL,JK)=PTD (JL,JK)
        PMFDQ5(JL,JK)=PQD5(JL,JK)
        PMFDQ (JL,JK)=PQD (JL,JK)
        PDMFDP5(JL,JK-1)=-0.5_JPRB*PMFD5(JL,JK)*ZCOND15
        PDMFDP (JL,JK-1)=-0.5_JPRB*(PMFD5(JL,JK)*ZCOND1 &
                      & + PMFD (JL,JK)*ZCOND15)  
        PRFL5(JL)=PRFL5(JL)+PDMFDP5(JL,JK-1)
        PRFL (JL)=PRFL (JL)+PDMFDP (JL,JK-1)
      ENDIF
    ENDIF
  ENDDO

ENDDO

!   ** COMPUTE VERTICAL PROFILES OF DOWNDRAFT CHARACTERISTICS
!   ** WITH INCLUSION OF ENTRAINMENT 
!   ** COMPUTE VERTICAL PROFILE OF Md(z)/Md(LSF) 
!   ** WITH INCLUSION OF ENTRAINMENT AND DETRAINMENT

DO JK=3,KLEV

  IS=0 
  DO JL=KIDIA,KFDIA
    LLO2(JL)=LDDRAF(JL).AND.PMFD5(JL,JK-1) < 0.0_JPRB
    IF(LLO2(JL)) THEN
      IS=IS+1
    ENDIF
  ENDDO
  IF(IS == 0) CYCLE

  DO JL=KIDIA,KFDIA
    IF (LLO2(JL)) THEN
      ZDZ5 = (PGEOH5(JL,JK-1)-PGEOH5(JL,JK))*ZRG
      ZDZ  = (PGEOH (JL,JK-1)-PGEOH (JL,JK))*ZRG

! Entrainment/detrainment rates
      IF (JK > ITOPDE) THEN
        ZEPS5 = 0.0_JPRB
        ZEPS  = 0.0_JPRB
        ZDEL5 = PMFD5(JL,ITOPDE) * ZDTOPDE5(JL)
        ZDEL  = PMFD (JL,ITOPDE) * ZDTOPDE5(JL) &
            & + PMFD5(JL,ITOPDE) * ZDTOPDE (JL)
      ELSE
        ZEPS5 = ENTRDD2 
        ZEPS  = 0.0_JPRB
        ZDEL5 = ENTRDD2 * PMFD5(JL,JK-1)
        ZDEL  = ENTRDD2 * PMFD (JL,JK-1)
      ENDIF

      ZENT5   = ZEPS5 * ZDZ5
      ZENT    = ZEPS  * ZDZ5 + ZEPS5 * ZDZ 
      ZDMFDE5 = ZDEL5 * ZDZ5 
      ZDMFDE  = ZDEL  * ZDZ5 + ZDEL5 * ZDZ  
      ZDMFEN5 = ZENT5 * PMFD5(JL,JK-1)
      ZDMFEN  = ZENT  * PMFD5(JL,JK-1) + ZENT5 * PMFD (JL,JK-1)

! Downdraft temperature and specific humidity
      PQD5(JL,JK) = PQD5(JL,JK-1)*(1.0_JPRB-ZENT5) + PQENH5(JL,JK-1)*ZENT5
      PQD (JL,JK) = PQD (JL,JK-1)*(1.0_JPRB-ZENT5) + PQENH (JL,JK-1)*ZENT5 &
               &  + (PQENH5(JL,JK-1)-PQD5(JL,JK-1))*ZENT 

      ZSDH5(JL,JK) = ZSDH5(JL,JK-1)*(1.0_JPRB-ZENT5) + ZSENH5(JL,JK-1)*ZENT5 
      ZSDH (JL,JK) = ZSDH (JL,JK-1)*(1.0_JPRB-ZENT5) + ZSENH (JL,JK-1)*ZENT5 &
               &   + (ZSENH5(JL,JK-1)-ZSDH5(JL,JK-1))*ZENT 

      ZCOND25(JL) = PQD5(JL,JK)
      ZCOND2 (JL) = PQD (JL,JK)
      PTD5(JL,JK) = (ZSDH5(JL,JK)-PGEOH5(JL,JK))*ZRCPD
      PTD (JL,JK) = (ZSDH (JL,JK)-PGEOH (JL,JK))*ZRCPD
      ZPH5(JL) = PAPH5(JL,JK)
      ZPH (JL) = PAPH (JL,JK)

! Store entrainment rates for momentum computations
      PERATED5 (JL,JK) = ZENT5
      PERATED  (JL,JK) = ZENT

! Downdraft mass-flux
      PMFD5(JL,JK) = PMFD5(JL,JK-1) + ZDMFEN5 - ZDMFDE5
      PMFD (JL,JK) = PMFD (JL,JK-1) + ZDMFEN  - ZDMFDE
    
! Detrainment rate
      PMFDDE_RATE5(JL,JK) = -ZDMFDE5
      PMFDDE_RATE (JL,JK) = -ZDMFDE
    ENDIF
  ENDDO

  IK=JK
  ICALL=2
 
  CALL CUADJTQSTL &
   & ( KIDIA,    KFDIA,    KLON,    KLEV,&
   & IK,&
   & ZPH5,     PTD5,     PQD5,&
   & ZPH,      PTD,      PQD,&
   & LLO2,   ICALL)  

  DO JL=KIDIA,KFDIA
    IF(LLO2(JL)) THEN
      ZSDH5(JL,JK)=RCPD*PTD5(JL,JK)+PGEOH5(JL,JK)
      ZSDH (JL,JK)=RCPD*PTD (JL,JK)+PGEOH (JL,JK)
      ZCOND25(JL)=ZCOND25(JL)-PQD5(JL,JK)
      ZCOND2 (JL)=ZCOND2 (JL)-PQD (JL,JK)
      ZBUO5=PTD5(JL,JK)*(1.0_JPRB+RETV  *PQD5(JL,JK)) &
         & -PTENH5(JL,JK)*(1.0_JPRB+RETV  *PQENH5(JL,JK))  
!LCO      ZBUO =PTD(JL,JK) *(1.0_JPRB+RETV  *PQD5(JL,JK)) &
!LCO         & +PTD5(JL,JK)*RETV*PQD(JL,JK) &
!LCO         & -PTENH(JL,JK)*(1.0_JPRB+RETV  *PQENH5(JL,JK)) &
!LCO         & -PTENH5(JL,JK)*RETV*PQENH(JL,JK)  
!LCO!  Reduce buoyancy perturbation if required
!LCO      IF (LREGCV) THEN
!LCO        ZBUO= ZBUO*0.35_JPRB
!LCO      ENDIF
      IF(PRFL5(JL) > 0.0_JPRB .AND. PMFU5(JL,JK) > 0.0_JPRB) THEN
        ZRAIN5=PRFL5(JL)/PMFU5(JL,JK)
!LCO        ZRAIN =(PRFL (JL)-PRFL5(JL)*PMFU (JL,JK)/PMFU5(JL,JK))/PMFU5(JL,JK)  
        ZBUO5=ZBUO5-PTD5(JL,JK)*ZRAIN5
!LCO        ZBUO =ZBUO -PTD5(JL,JK)*ZRAIN -PTD (JL,JK)*ZRAIN5
      ENDIF
      ZRFLMIN=PMFD5(JL,JK)*ZCOND25(JL)
      IF(ZBUO5 >= 0.0_JPRB .OR. PRFL5(JL) <= ZRFLMIN .OR. &
         & PMFD5(JL,JK) > 0.0_JPRB) THEN  
        PMFD5(JL,JK)=0.0_JPRB
        PMFD (JL,JK)=0.0_JPRB
      ENDIF
      PMFDS5(JL,JK)=PTD5(JL,JK)
      PMFDS (JL,JK)=PTD (JL,JK)  
      PMFDQ5(JL,JK)=PQD5(JL,JK)
      PMFDQ (JL,JK)=PQD (JL,JK)
      PDMFDP5(JL,JK-1)=-ZCOND25(JL)*PMFD5(JL,JK)
      PDMFDP (JL,JK-1)=-ZCOND25(JL)*PMFD (JL,JK)-ZCOND2 (JL)*PMFD5(JL,JK)
      PRFL5(JL)=PRFL5(JL)+PDMFDP5(JL,JK-1)
      PRFL (JL)=PRFL (JL)+PDMFDP (JL,JK-1)
    ENDIF
  ENDDO

ENDDO

IF (LHOOK) CALL DR_HOOK('CUDDRAFN2TL',1,ZHOOK_HANDLE)
END SUBROUTINE CUDDRAFN2TL
