SUBROUTINE CUDDRAFNTL &
 & ( KIDIA,    KFDIA,    KLON,     KLEV,&
 & LDDRAF,&
 & PTENH5,   PQENH5,&
 & PGEO5,    PGEOH5,   PAPH5,    PRFL5,&
 & PTD5,     PQD5,     PMFU5,&
 & PMFD5,    PMFDS5,   PMFDQ5,   PDMFDP5,&
 & PDMFDE5,  PMFDDE_RATE5,       PKINED5 )

!          THIS ROUTINE CALCULATES CUMULUS DOWNDRAFT DESCENT
!          (Tangent linear)

!          J.F. MAHFOUF      E.C.M.W.F.    05/12/96

!          Adapted from

!          M.TIEDTKE         E.C.M.W.F.    12/86 MODIF. 12/89

!          PURPOSE.
!          --------
!          TO PRODUCE THE VERTICAL PROFILES FOR CUMULUS DOWNDRAFTS
!          (I.E. T,Q,U AND V AND FLUXES)

!          INTERFACE
!          ---------

!          THIS ROUTINE IS CALLED FROM *CUMASTRNTL*.
!          INPUT IS T,Q,P,PHI,U,V AT HALF LEVELS.
!          IT RETURNS FLUXES OF S,Q AND EVAPORATION RATE
!          AND U,V AT LEVELS WHERE DOWNDRAFT OCCURS

!          METHOD.
!          --------
!          CALCULATE MOIST DESCENT FOR ENTRAINING/DETRAINING PLUME BY
!          A) MOVING AIR DRY-ADIABATICALLY TO NEXT LEVEL BELOW AND
!          B) CORRECTING FOR EVAPORATION TO OBTAIN SATURATED STATE.

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS

!    INPUT PARAMETERS (LOGICAL):

!    *LDDRAF*       .TRUE. IF DOWNDRAFTS EXIST

!    INPUT PARAMETERS (REAL):

!    *PTENH5*       ENV. TEMPERATURE (T+1) ON HALF LEVELS      (Trajectory)
!    *PQENH5*       ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS   (Trajectory)
!    *PGEO5*        GEOPOTENTIAL                               (Trajectory)
!    *PGEOH5*       GEOPOTENTIAL ON HALF LEVELS                (Trajectory)
!    *PAPH5*        PROVISIONAL PRESSURE ON HALF LEVELS        (Trajectory)
!    *PMFU5*        MASSFLUX UPDRAFTS                          (Trajectory)

!    UPDATED PARAMETERS (REAL):

!    *PRFL5*        PRECIPITATION RATE                         (Trajectory) 

!    OUTPUT PARAMETERS (REAL):

!    *PTD5*         TEMPERATURE IN DOWNDRAFTS                  (Trajectory)
!    *PQD5*         SPEC. HUMIDITY IN DOWNDRAFTS               (Trajectory)
!    *PMFD5*        MASSFLUX IN DOWNDRAFTS                     (Trajectory)
!    *PMFDS5*       FLUX OF DRY STATIC ENERGY IN DOWNDRAFTS    (Trajectory)
!    *PMFDQ5*       FLUX OF SPEC. HUMIDITY IN DOWNDRAFTS       (Trajectory)
!    *PDMFDP5*      FLUX DIFFERENCE OF PRECIP. IN DOWNDRAFTS   (Trajectory)
!    *PMFDDE_RATE5* DOWNDRAFT DETRAINMENT RATE                 (Trajectory)
!    *PKINED5*      DOWNDRAFT KINETIC ENERGY                   (Trajectory)

!          EXTERNALS
!          ---------
!          *CUADJTQTL* FOR ADJUSTING T AND Q DUE TO EVAPORATION IN
!          SATURATED DESCENT

!          REFERENCE
!          ---------
!          (TIEDTKE,1989)

!          MODIFICATIONS
!          -------------
!             92-09-21 : Update to Cy44      J.-J. MORCRETTE
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RCPD     ,RETV
USE YOECUMF  , ONLY : ENTRDD   ,RMFCMIN  ,NJKT3

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
LOGICAL           ,INTENT(IN)    :: LDDRAF(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEO5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRFL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMFU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDS5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDMFDP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDMFDE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFDDE_RATE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PKINED5(KLON,KLEV) 
REAL(KIND=JPRB) ::     ZDMFEN5(KLON),          ZDMFDE5(KLON),&
 & ZCOND5(KLON),           ZOENTR5(KLON),&
 & ZBUOY5(KLON)  
REAL(KIND=JPRB) ::     ZPH5(KLON)

LOGICAL ::  LLO2(KLON)
INTEGER(KIND=JPIM) :: ICALL, IK, IS, ITOPDE, JK, JL

REAL(KIND=JPRB) :: Z1S5, Z2S5, ZBUO5, ZBUOYZ5, ZBUOYV5, ZDMFDP5,&
 & ZDZ5, ZENTR5, ZMFDQK5, ZMFDSK5, &
 & ZQDDE5, ZQEEN5, ZRAIN5, ZSDDE5, &
 & ZSEEN5, ZZENTR5, ZRG5, ZFACBUO5, Z_CWDRAG5, ZDKBUO5, ZDKEN5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cuadjtq.intfb.h"

IF (LHOOK) CALL DR_HOOK('CUDDRAFNTL',0,ZHOOK_HANDLE)
ITOPDE=NJKT3
ZRG5=1.0_JPRB/RG
ZFACBUO5=0.5_JPRB/(1.0_JPRB+0.5_JPRB)
Z_CWDRAG5=(3._JPRB/8._JPRB)*0.506_JPRB/0.2_JPRB/RG
!----------------------------------------------------------------------

!     1.           CALCULATE MOIST DESCENT FOR CUMULUS DOWNDRAFT BY
!                     (A) CALCULATING ENTRAINMENT/DETRAINMENT RATES, 
!                         INCLUDING ORGANIZED ENTRAINMENT DEPENDENT ON
!                         NEGATIVE BUOYANCY AND ASSUMING
!                         LINEAR DECREASE OF MASSFLUX IN PBL
!                     (B) DOING MOIST DESCENT - EVAPORATIVE COOLING
!                         AND MOISTENING IS CALCULATED IN *CUADJTQ*
!                     (C) CHECKING FOR NEGATIVE BUOYANCY AND
!                         SPECIFYING FINAL T,Q,U,V AND DOWNWARD FLUXES
!                    -------------------------------------------------

DO JL=KIDIA,KFDIA
  ZOENTR5(JL)=0.0_JPRB
  ZBUOY5(JL)=0.0_JPRB
  ZDMFEN5(JL)=0.0_JPRB
  ZDMFDE5(JL)=0.0_JPRB
  PDMFDE5(JL,:)=0.0_JPRB
  PMFDDE_RATE5(JL,:)=0.0_JPRB
  PKINED5(JL,:)=0.0_JPRB
ENDDO

DO JK=3,KLEV
  IS=0
  DO JL=KIDIA,KFDIA
    ZPH5(JL)=PAPH5(JL,JK)
    LLO2(JL)=LDDRAF(JL).AND.PMFD5(JL,JK-1) < 0.0_JPRB
    IF(LLO2(JL)) THEN
      IS=IS+1
    ENDIF
  ENDDO
  IF(IS == 0) CYCLE

  DO JL=KIDIA,KFDIA
    IF(LLO2(JL)) THEN
      ZENTR5=ENTRDD*PMFD5(JL,JK-1)*(PGEOH5(JL,JK-1)-PGEOH5(JL,JK))*ZRG5
      ZDMFEN5(JL)=ZENTR5
      ZDMFDE5(JL)=ZENTR5
    ENDIF
  ENDDO

!  IF(KLEV == 60.OR.KLEV == 40) THEN
!    ITOPDE=KLEV-6
!  ELSE
!    ITOPDE=KLEV-2
!  ENDIF

  IF(JK > ITOPDE) THEN
    DO JL=KIDIA,KFDIA
      IF(LLO2(JL)) THEN
        ZDMFEN5(JL)=0.0_JPRB
        ZDMFDE5(JL)=PMFD5(JL,ITOPDE)*&
         & (PAPH5(JL,JK)-PAPH5(JL,JK-1))/&
         & (PAPH5(JL,KLEV+1)-PAPH5(JL,ITOPDE))  
      ENDIF
    ENDDO
  ENDIF

  IF(JK <= ITOPDE) THEN
    DO JL=KIDIA,KFDIA
      IF(LLO2(JL)) THEN
        ZDZ5=-(PGEOH5(JL,JK-1)-PGEOH5(JL,JK))*ZRG5
        ZZENTR5=ZOENTR5(JL)*ZDZ5*PMFD5(JL,JK-1)
        ZDMFEN5(JL)=ZDMFEN5(JL)+ZZENTR5
!jfm            ZDMFEN5(JL)=MAX(ZDMFEN5(JL),0.3*PMFD5(JL,JK-1))
!jfm            ZDMFEN5(JL)=MAX(ZDMFEN5(JL),-0.75*PMFU5(JL,JK)-
!jfm     *                    (PMFD5(JL,JK-1)-ZDMFDE5(JL)))
!jfm            ZDMFEN5(JL)=MIN(ZDMFEN5(JL),0.)
        IF (ZDMFEN5(JL) < (0.3_JPRB*PMFD5(JL,JK-1))) THEN
          ZDMFEN5(JL) =0.3_JPRB*PMFD5(JL,JK-1)
        ENDIF
        Z1S5 = -0.75_JPRB*PMFU5(JL,JK)-(PMFD5(JL,JK-1)-ZDMFDE5(JL))
        IF (ZDMFEN5(JL) < Z1S5) THEN
          ZDMFEN5(JL) = Z1S5
        ENDIF
        IF (ZDMFEN5(JL) > 0.0_JPRB) THEN
          ZDMFEN5(JL) = 0.0_JPRB
        ENDIF
      ENDIF

      PDMFDE5(JL,JK)=ZDMFEN5(JL)-ZDMFDE5(JL)

    ENDDO
  ENDIF
  DO JL=KIDIA,KFDIA
    IF(LLO2(JL)) THEN
      PMFD5(JL,JK)=PMFD5(JL,JK-1)+ZDMFEN5(JL)-ZDMFDE5(JL)
      ZSEEN5=(RCPD*PTENH5(JL,JK-1)+PGEOH5(JL,JK-1))*ZDMFEN5(JL)
      ZQEEN5=PQENH5(JL,JK-1)*ZDMFEN5(JL)
      ZSDDE5=(RCPD*PTD5(JL,JK-1)+PGEOH5(JL,JK-1))*ZDMFDE5(JL)
      ZQDDE5=PQD5(JL,JK-1)*ZDMFDE5(JL)
      ZMFDSK5=PMFDS5(JL,JK-1)+ZSEEN5-ZSDDE5
      ZMFDQK5=PMFDQ5(JL,JK-1)+ZQEEN5-ZQDDE5
!jfm            PQD5(JL,JK)=ZMFDQK5*(1./MIN(-RMFCMIN,PMFD5(JL,JK)))
!jfm            PTD5(JL,JK)=(ZMFDSK5*(1./MIN(-RMFCMIN,PMFD5(JL,JK)))-
!jfm     1                  PGEOH5(JL,JK))/RCPD
!jfm            PTD5(JL,JK)=MIN(400.,PTD5(JL,JK))
!jfm            PTD5(JL,JK)=MAX(100.,PTD5(JL,JK))
      IF (PMFD5(JL,JK) < -RMFCMIN) THEN
        Z2S5 =  1.0_JPRB/PMFD5(JL,JK)
      ELSE
        Z2S5 = -1.0_JPRB/RMFCMIN
      ENDIF
      PQD5(JL,JK)=ZMFDQK5*Z2S5
      PTD5(JL,JK)=(ZMFDSK5*Z2S5-PGEOH5(JL,JK))/RCPD
      IF (PTD5(JL,JK) > 400._JPRB) THEN
        PTD5(JL,JK) = 400._JPRB
      ELSEIF (PTD5(JL,JK) < 100._JPRB) THEN
        PTD5(JL,JK) = 100._JPRB
      ENDIF
      ZCOND5(JL)=PQD5(JL,JK)
    ENDIF
  ENDDO

  IK=JK
  ICALL=2
!      CALL CUADJTQTL
!     *   ( KIDIA,    KFDIA,    KLON,     KTDIA,    KLEV
!     *   , IK
!     *   , ZPH5,     PTD5,     PQD5   
!     *   , ZPH,      PTD,      PQD,      LLO2,     ICALL )

  CALL CUADJTQ &
   & ( KIDIA,    KFDIA,    KLON,     KLEV,     IK, &
   &   ZPH5,     PTD5,     PQD5,     LLO2,     ICALL )  

  DO JL=KIDIA,KFDIA
    IF(LLO2(JL)) THEN
      ZCOND5(JL)=ZCOND5(JL)-PQD5(JL,JK)
      ZBUO5=PTD5(JL,JK)*(1.0_JPRB+RETV  *PQD5(JL,JK))-&
       & PTENH5(JL,JK)*(1.0_JPRB+RETV  *PQENH5(JL,JK))  
      IF(PRFL5(JL) > 0.0_JPRB.AND.PMFU5(JL,JK) > 0.0_JPRB) THEN
        ZRAIN5=PRFL5(JL)/PMFU5(JL,JK)
        ZBUO5=ZBUO5-PTD5(JL,JK)*ZRAIN5
      ENDIF
      IF(ZBUO5 >= 0.0_JPRB.OR.PRFL5(JL) <= (PMFD5(JL,JK)*ZCOND5(JL))) THEN
        PMFD5(JL,JK)=0.0_JPRB
        ZBUO5=0.0_JPRB
      ENDIF
      PMFDS5(JL,JK)=(RCPD*PTD5(JL,JK)+PGEOH5(JL,JK))*PMFD5(JL,JK)
      PMFDQ5(JL,JK)=PQD5(JL,JK)*PMFD5(JL,JK)
      ZDMFDP5=-PMFD5(JL,JK)*ZCOND5(JL)
      PDMFDP5(JL,JK-1)=ZDMFDP5
      PRFL5(JL)=PRFL5(JL)+ZDMFDP5

! COMPUTE ORGANIZED ENTRAINMENT FOR USE AT NEXT LEVEL

      ZBUOYZ5=ZBUO5/PTENH5(JL,JK)
      ZBUOYV5=ZBUOYZ5
!jfm            ZBUOYZ5=MIN(ZBUOYZ5,0.0)
      IF (ZBUOYZ5 > 0.0_JPRB) THEN
        ZBUOYZ5=0.0_JPRB
      ENDIF
      ZDZ5=-(PGEO5(JL,JK-1)-PGEO5(JL,JK))
      ZBUOY5(JL)=ZBUOY5(JL)+ZBUOYZ5*ZDZ5
      ZOENTR5(JL)=RG*ZBUOYZ5*0.5_JPRB/(1.0_JPRB+ZBUOY5(JL))

! STORE DOWNDRAUGHT DETRAINMENT RATES

      PMFDDE_RATE5(JL,JK)=-ZDMFDE5(JL)

! COMPUTE KINETIC ENERGY

      ZDKBUO5=ZDZ5*ZBUOYV5*ZFACBUO5
      IF(ZDMFEN5(JL) < 0.0_JPRB)THEN
        ZDKEN5=MIN(1.0_JPRB,(1 + RG*Z_CWDRAG5)*&
         & ZDMFEN5(JL)/MIN(-RMFCMIN,PMFD5(JL,JK-1)))  
      ELSE
        ZDKEN5=MIN(1.0_JPRB,(1 + RG*Z_CWDRAG5)*&
         & ZDMFDE5(JL)/MIN(-RMFCMIN,PMFD5(JL,JK-1)))  
      ENDIF
      PKINED5(JL,JK)=MAX(0.0_JPRB,(PKINED5(JL,JK-1)*(1-ZDKEN5)+ZDKBUO5)/(1+ZDKEN5))

    ENDIF
  ENDDO

ENDDO

IF (LHOOK) CALL DR_HOOK('CUDDRAFNTL',1,ZHOOK_HANDLE)
END SUBROUTINE CUDDRAFNTL
