SUBROUTINE CUDLFSNAD &
 & (KIDIA,    KFDIA,    KLON,    KLEV,&
 & KCBOT,    KCTOP,   LDCUM,&
 & PTENH5,   PQENH5,   PUEN5,    PVEN5,&
 & PTEN5,    PQSEN5,   PGEO5,&
 & PGEOH5,   PAPH5,    PTU5,     PQU5,&
 & PUU5,     PVU5,     PMFUB5,   PRFL5,&
 & PTD5,     PQD5,&
 & PMFD5,    PMFDS5,   PMFDQ5,   PDMFDP5,&
 & KDTOP,    LDDRAF )  

!          THIS ROUTINE CALCULATES LEVEL OF FREE SINKING FOR
!          CUMULUS DOWNDRAFTS AND SPECIFIES T,Q,U AND V VALUES
!          (Adjoint)

!          J.F. MAHFOUF      E.C.M.W.F.    16/01/97

!          Adapted from

!          M.TIEDTKE         E.C.M.W.F.    12/86 MODIF. 12/89

!          PURPOSE.
!          --------
!          TO PRODUCE LFS-VALUES FOR CUMULUS DOWNDRAFTS
!          FOR MASSFLUX CUMULUS PARAMETERIZATION

!          INTERFACE
!          ---------
!          THIS ROUTINE IS CALLED FROM *CUMASTRNAD*.
!          INPUT ARE ENVIRONMENTAL VALUES OF T,Q,U,V,P,PHI
!          AND UPDRAFT VALUES T,Q,U AND V AND ALSO
!          CLOUD BASE MASSFLUX AND CU-PRECIPITATION RATE.
!          IT RETURNS T,Q,U AND V VALUES AND MASSFLUX AT LFS.

!          METHOD.

!          CHECK FOR NEGATIVE BUOYANCY OF AIR OF EQUAL PARTS OF
!          MOIST ENVIRONMENTAL AIR AND CLOUD AIR.

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS
!    *KCBOT*        CLOUD BASE LEVEL
!    *KCTOP*        CLOUD TOP LEVEL

!    INPUT PARAMETERS (LOGICAL):

!    *LDCUM*        FLAG: .TRUE. FOR CONVECTIVE POINTS

!    INPUT PARAMETERS (REAL):

!    *PTENH5*       ENV. TEMPERATURE (T+1) ON HALF LEVELS       (Trajectory)
!    *PQENH5*       ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS    (Trajectory)
!    *PUEN5*        PROVISIONAL ENVIRONMENT U-VELOCITY (T+1)    (Trajectory)
!    *PVEN5*        PROVISIONAL ENVIRONMENT V-VELOCITY (T+1)    (Trajectory)
!    *PTEN5*        PROVISIONAL ENVIRONMENT TEMPERATURE (T+1)   (Trajectory)
!    *PQSEN5*       ENVIRONMENT SPEC. SATURATION HUMIDITY (T+1) (Trajectory)
!    *PGEO5*        GEOPOTENTIAL                                (Trajectory)
!    *PGEOH5*       GEOPOTENTIAL ON HALF LEVELS                 (Trajectory)
!    *PAPH5*        PROVISIONAL PRESSURE ON HALF LEVELS         (Trajectory)
!    *PTU5*         TEMPERATURE IN UPDRAFTS                     (Trajectory)
!    *PQU5*         SPEC. HUMIDITY IN UPDRAFTS                  (Trajectory)
!    *PUU5*         U-VELOCITY IN UPDRAFTS                      (Trajectory)
!    *PVU5*         V-VELOCITY IN UPDRAFTS                      (Trajectory)
!    *PMFUB5*       MASSFLUX IN UPDRAFTS AT CLOUD BASE          (Trajectory)

!    UPDATED PARAMETERS (REAL):

!    *PRFL5*        PRECIPITATION RATE                          (Trajectory)
!    *PRFL*         PRECIPITATION RATE                           KG/(M2*S)

!    OUTPUT PARAMETERS (REAL):

!    *PTD5*         TEMPERATURE IN DOWNDRAFTS                  (Trajectory) 
!    *PQD5*         SPEC. HUMIDITY IN DOWNDRAFTS               (Trajectory) 
!    *PMFD5*        MASSFLUX IN DOWNDRAFTS                     (Trajectory) 
!    *PMFDS5*       FLUX OF DRY STATIC ENERGY IN DOWNDRAFTS    (Trajectory) 
!    *PMFDQ5*       FLUX OF SPEC. HUMIDITY IN DOWNDRAFTS       (Trajectory) 
!    *PDMFDP5*      FLUX DIFFERENCE OF PRECIP. IN DOWNDRAFTS   (Trajectory) 

!    *PTD*          TEMPERATURE IN DOWNDRAFTS                      K
!    *PQD*          SPEC. HUMIDITY IN DOWNDRAFTS                 KG/KG
!    *PMFD*         MASSFLUX IN DOWNDRAFTS                       KG/(M2*S)
!    *PMFDS*        FLUX OF DRY STATIC ENERGY IN DOWNDRAFTS       J/(M2*S)
!    *PMFDQ*        FLUX OF SPEC. HUMIDITY IN DOWNDRAFTS         KG/(M2*S)
!    *PDMFDP*       FLUX DIFFERENCE OF PRECIP. IN DOWNDRAFTS     KG/(M2*S)

!    OUTPUT PARAMETERS (INTEGER):

!    *KDTOP*        TOP LEVEL OF DOWNDRAFTS

!    OUTPUT PARAMETERS (LOGICAL):

!    *LDDRAF*       .TRUE. IF DOWNDRAFTS EXIST

!          EXTERNALS
!          ---------
!          *CUADJTQAD* FOR CALCULATING WET BULB T AND Q AT LFS

!          MODIFICATIONS
!          -------------
!             92-09-21 : Update to Cy44      J.-J. MORCRETTE
!             99-06-14 : Optimisation        D. Salmond
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        P. Lopez      20-Jun-2007 CY32R2 Bug correction in latent heat
!                                         when LPHYLIN=T.

!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RCPD     ,RETV     ,RLVTT    ,RLSTT    ,RTT
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE   ,RTICECU   ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOECUMF  , ONLY : RMFDEPS  ,LMFDD
USE YOEPHLI  , ONLY : LPHYLIN  ,RLPTRC

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCBOT(KLON)
INTEGER(KIND=JPIM),INTENT(IN)    :: KCTOP(KLON)
LOGICAL           ,INTENT(IN)    :: LDCUM(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQSEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEO5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMFUB5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRFL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFDS5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFDQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDMFDP5(KLON,KLEV) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KDTOP(KLON) 
LOGICAL           ,INTENT(OUT)   :: LDDRAF(KLON) 
INTEGER(KIND=JPIM) ::  IKHSMIN(KLON)
REAL(KIND=JPRB) ::     ZTENWB5(KLON,KLEV),     ZQENWB5(KLON,KLEV),&
 & ZCOND5(KLON),           ZPH5(KLON),&
 & ZHSMIN5(KLON)  

LOGICAL ::  LLO2(KLON)

INTEGER(KIND=JPIM) :: ICALL, IK, IKE, IS, JK, JL

REAL(KIND=JPRB) ::  ZBUO5, ZHSK5,&
 & ZMFTOP5, ZOEALFA5, ZOELHM5, ZQTEST5, ZTARG5, &
 & ZTTEST5  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cuadjtq.intfb.h"

#include "fcttre.h"
!----------------------------------------------------------------------

!     1.           SET DEFAULT VALUES FOR DOWNDRAFTS
!                  ---------------------------------

IF (LHOOK) CALL DR_HOOK('CUDLFSNAD',0,ZHOOK_HANDLE)
DO JL=KIDIA,KFDIA
  LDDRAF(JL)=.FALSE.
  KDTOP(JL)=KLEV+1
  IKHSMIN(JL)=KLEV+1
  ZHSMIN5(JL)=1.E8_JPRB
ENDDO

IF(LMFDD) THEN

!----------------------------------------------------------------------

!     2.           DETERMINE LEVEL OF FREE SINKING:
!                  DOWNDRAFTS SHALL START AT MODEL LEVEL OF MINIMUM
!                  OF SATURATION MOIST STATIC ENERGY OR BELOW
!                  RESPECTIVELY

!                  FOR EVERY POINT AND PROCEED AS FOLLOWS:

!                    (1) DETERMINE LEVEL OF MINIMUM OF HS
!                    (2) DETERMINE WET BULB ENVIRONMENTAL T AND Q
!                    (3) DO MIXING WITH CUMULUS CLOUD AIR
!                    (4) CHECK FOR NEGATIVE BUOYANCY
!                    (5) IF BUOYANCY>0 REPEAT (2) TO (4) FOR NEXT
!                        LEVEL BELOW

!                  THE ASSUMPTION IS THAT AIR OF DOWNDRAFTS IS MIXTURE
!                  OF 50% CLOUD AIR + 50% ENVIRONMENTAL AIR AT WET BULB
!                  TEMPERATURE (I.E. WHICH BECAME SATURATED DUE TO
!                  EVAPORATION OF RAIN AND CLOUD WATER)
!                  ----------------------------------------------------

DO JK=3,KLEV-2

  IF (LPHYLIN) THEN

    DO JL=KIDIA,KFDIA
      ZTARG5=PTEN5(JL,JK)
      ZOEALFA5=0.545_JPRB*(TANH(0.17_JPRB*(ZTARG5-RLPTRC))+1.0_JPRB)
      ZOELHM5 =ZOEALFA5*RLVTT+(1.0_JPRB-ZOEALFA5)*RLSTT
      ZHSK5=RCPD*PTEN5(JL,JK)+PGEO5(JL,JK)+ZOELHM5*PQSEN5(JL,JK)
      IF(ZHSK5 < ZHSMIN5(JL)) THEN
        ZHSMIN5(JL)=ZHSK5
        IKHSMIN(JL)=JK
      ENDIF
    ENDDO

  ELSE

    DO JL=KIDIA,KFDIA
      ZHSK5=RCPD*PTEN5(JL,JK)+PGEO5(JL,JK)+FOELHM(PTEN5(JL,JK))*PQSEN5(JL,JK)
      IF(ZHSK5 < ZHSMIN5(JL)) THEN
        ZHSMIN5(JL)=ZHSK5
        IKHSMIN(JL)=JK
      ENDIF
    ENDDO

  ENDIF

ENDDO
IKE=KLEV-3
DO JK=3,IKE

!     2.1          CALCULATE WET-BULB TEMPERATURE AND MOISTURE
!                  FOR ENVIRONMENTAL AIR IN *CUADJTQ*
!                  -------------------------------------------

  IS=0
  DO JL=KIDIA,KFDIA
    ZTENWB5(JL,JK)=PTENH5(JL,JK)
    ZQENWB5(JL,JK)=PQENH5(JL,JK)
    ZPH5(JL)=PAPH5(JL,JK)
    LLO2(JL)=&
     & LDCUM(JL).AND.PRFL5(JL) > 0.0_JPRB.AND..NOT.LDDRAF(JL).AND.&
     & (JK < KCBOT(JL).AND.JK > KCTOP(JL)).AND.&
     & JK >= IKHSMIN(JL)  
    IF(LLO2(JL))THEN
      IS=IS+1
    ENDIF
  ENDDO
  IF(IS == 0) CYCLE

  IK=JK
  ICALL=2
  CALL CUADJTQ &
   & ( KIDIA,    KFDIA,    KLON,     KLEV,     IK,&
   &   ZPH5,     ZTENWB5,  ZQENWB5,  LLO2,     ICALL)  

!     2.2          DO MIXING OF CUMULUS AND ENVIRONMENTAL AIR
!                  AND CHECK FOR NEGATIVE BUOYANCY.
!                  THEN SET VALUES FOR DOWNDRAFT AT LFS.
!                  ----------------------------------------

!DIR$ IVDEP
!OCL NOVREC
  DO JL=KIDIA,KFDIA
    IF(LLO2(JL)) THEN
      ZTTEST5=0.5_JPRB*(PTU5(JL,JK)+ZTENWB5(JL,JK))
      ZQTEST5=0.5_JPRB*(PQU5(JL,JK)+ZQENWB5(JL,JK))
      ZBUO5=ZTTEST5*(1.0_JPRB+RETV  *ZQTEST5)-&
       & PTENH5(JL,JK)*(1.0_JPRB+RETV  *PQENH5(JL,JK))  
      ZCOND5(JL)=PQENH5(JL,JK)-ZQENWB5(JL,JK)
      ZMFTOP5=-RMFDEPS*PMFUB5(JL)
      IF(ZBUO5 < 0.0_JPRB.AND.PRFL5(JL) > 10._JPRB*ZMFTOP5*ZCOND5(JL)) THEN
        KDTOP(JL)=JK
        LDDRAF(JL)=.TRUE.
        PTD5(JL,JK)=ZTTEST5
        PQD5(JL,JK)=ZQTEST5
        PMFD5(JL,JK)=ZMFTOP5
        PMFDS5(JL,JK)=PMFD5(JL,JK)*(RCPD*PTD5(JL,JK)+PGEOH5(JL,JK))
        PMFDQ5(JL,JK)=PMFD5(JL,JK)*PQD5(JL,JK)
        PDMFDP5(JL,JK-1)=-0.5_JPRB*PMFD5(JL,JK)*ZCOND5(JL)
        PRFL5(JL)=PRFL5(JL)+PDMFDP5(JL,JK-1)
      ENDIF
    ENDIF
  ENDDO

ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('CUDLFSNAD',1,ZHOOK_HANDLE)
END SUBROUTINE CUDLFSNAD
