!OPTIONS XOPT(HSFUN)
SUBROUTINE CUFLXNTL &
 & (  KIDIA,    KFDIA,    KLON,   KLEV,&
 & PTSPHY,&
 & PTEN5,    PQEN5,    PQSEN5,   PTENH5,   PQENH5,&
 & PAPH5,    PAP5,     PGEOH5,   LDLAND,   LDCUM,&
 & KCBOT,    KCTOP,    KDTOP,    KTOPM2,&
 & KTYPE,    LDDRAF,&
 & PMFU5,    PMFD5,    PMFUS5,   PMFDS5,&
 & PMFUQ5,   PMFDQ5,   PMFUL5,   PLUDE5,&
 & PDMFUP5,  PDMFDP5,  PDPMEL5,  PLGLAC5,&
 & PMFLXR5,  PMFLXS5,  PRAIN5,   PMFDDE_RATE5 )  

!          J.F. MAHFOUF      E.C.M.W.F.     06/12/96

!          Adapted from

!          M.TIEDTKE         E.C.M.W.F.     7/86 MODIF. 12/89

!          PURPOSE
!          -------

!          THIS ROUTINE DOES THE FINAL CALCULATION OF CONVECTIVE
!          FLUXES IN THE CLOUD LAYER AND IN THE SUBCLOUD LAYER
!          (Tangent linear)

!          INTERFACE
!          ---------
!          THIS ROUTINE IS CALLED FROM *CUMASTRNTL*.

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS
!    *KCBOT*        CLOUD BASE LEVEL
!    *KCTOP*        CLOUD TOP LEVEL
!    *KDTOP*        TOP LEVEL OF DOWNDRAFTS

!    INPUT PARAMETERS (LOGICAL):

!    *LDLAND*       LAND SEA MASK (.TRUE. FOR LAND)
!    *LDCUM*        FLAG: .TRUE. FOR CONVECTIVE POINTS

!    INPUT PARAMETERS (REAL):

!    *PTSPHY*       TIME STEP FOR THE PHYSICS                       S
!    *PTEN5*        PROVISIONAL ENVIRONMENT TEMPERATURE (T+1)    (Trajectory)
!    *PQEN5*        PROVISIONAL ENVIRONMENT SPEC. HUMIDITY (T+1) (Trajectory)
!    *PQSEN5*       ENVIRONMENT SPEC. SATURATION HUMIDITY (T+1)  (Trajectory)
!    *PTENH5*       ENV. TEMPERATURE (T+1) ON HALF LEVELS        (Trajectory)
!    *PQENH5*       ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS     (Trajectory)
!    *PAPH5*        PROVISIONAL PRESSURE ON HALF LEVELS          (Trajectory)
!    *PAP5*         PROVISIONAL PRESSURE ON FULL LEVELS          (Trajectory)
!    *PGEOH5*       GEOPOTENTIAL ON HALF LEVELS                  (Trajectory)

!    UPDATED PARAMETERS (INTEGER):

!    *KTYPE*        SET TO ZERO IF LDCUM=.FALSE.

!    UPDATED PARAMETERS (LOGICAL):

!    *LDDRAF*       SET TO .FALSE. IF LDCUM=.FALSE. OR KDTOP<KCTOP   

!    UPDATED PARAMETERS (REAL):

!    *PMFU5*        MASSFLUX IN UPDRAFTS                        (Trajectory)  
!    *PMFD5*        MASSFLUX IN DOWNDRAFTS                      (Trajectory)  
!    *PMFUS5*       FLUX OF DRY STATIC ENERGY IN UPDRAFTS       (Trajectory)  
!    *PMFDS5*       FLUX OF DRY STATIC ENERGY IN DOWNDRAFTS     (Trajectory)
!    *PMFUQ5*       FLUX OF SPEC. HUMIDITY IN UPDRAFTS          (Trajectory)  
!    *PMFDQ5*       FLUX OF SPEC. HUMIDITY IN DOWNDRAFTS        (Trajectory)  
!    *PMFUL5*       FLUX OF LIQUID WATER IN UPDRAFTS            (Trajectory)  
!    *PLUDE5*       DETRAINED LIQUID WATER                      (Trajectory)  
!    *PDMFUP5*      FLUX DIFFERENCE OF PRECIP. IN UPDRAFTS      (Trajectory)  
!    *PDMFDP5*      FLUX DIFFERENCE OF PRECIP. IN DOWNDRAFTS    (Trajectory)  
!    *PMFDDE_RATE5* DOWNDRAFT DETRAINMENT RATE                  (Trajectory)

!    OUTPUT PARAMETERS (REAL):

!    *PDPMEL5*      CHANGE IN PRECIP.-FLUXES DUE TO MELTING     (Trajectory) 
!    *PLGLAC5*      FLUX OF FROZEN CLOUD WATER IN UPDRAFTS      (Trajectory) 
!    *PMFLXR5*      CONVECTIVE RAIN FLUX                        (Trajectory) 
!    *PMFLXS5*      CONVECTIVE SNOW FLUX                        (Trajectory) 
!    *PRAIN5*       TOTAL PRECIP. PRODUCED IN CONV. UPDRAFTS    (Trajectory) 
!                   (NO EVAPORATION IN DOWNDRAFTS)       

!          EXTERNALS
!          ---------
!          NONE

!          D.SALMOND   ECMWF       99-06-04   Optimisation
!          P. BECHTOLD ECMWF       03-08-28 : Clean up LINPHYS    
!                                             Bugs in Evapor.

!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RCPD     ,RLVTT    ,RLSTT    ,RLMLT    ,RTT
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE   ,RTICECU   ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOEPHLI  , ONLY : LPHYLIN  ,RLPTRC
USE YOECUMF  , ONLY : RCUCOV   ,RCPECONS ,RTAUMEL  , RMFCFL

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQSEN5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH5(KLON,KLEV+1) 
LOGICAL           ,INTENT(IN)    :: LDLAND(KLON) 
LOGICAL           ,INTENT(IN)    :: LDCUM(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCBOT(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCTOP(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDTOP(KLON) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTOPM2 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KTYPE(KLON) 
LOGICAL           ,INTENT(INOUT) :: LDDRAF(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUS5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDS5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFUL5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLUDE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDMFUP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDMFDP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDPMEL5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLGLAC5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFLXR5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFLXS5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAIN5(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFDDE_RATE5(KLON,KLEV) 

REAL(KIND=JPRB)    :: ZRHEBC5(KLON)
INTEGER(KIND=JPIM) :: IK, IKB, JK, JL
INTEGER(KIND=JPIM) :: IDBAS(KLON)
LOGICAL :: LLDDRAF

REAL(KIND=JPRB) :: Z1S5, Z2S5, ZALFAW5, ZCONS15, ZCONS1A,&
 & ZCONS2, ZDENOM5, ZDRFL15, &
 & ZDRFL5, ZFAC5, ZFOEEWI5, ZFOEEWL5, ZOEALFA5, &
 & ZOEEWM5, ZOELHM5, ZPDR5, ZPDS5, ZRFL5, ZRFLN5, &
 & ZRMIN5, ZRNEW5, ZSNMLT5, ZTARG5, &
 & ZTMST, ZZP5  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!DIR$ VFUNCTION EXPHF
#include "fcttre.h"

!--------------------------------------------------------------------
!*             SPECIFY CONSTANTS

IF (LHOOK) CALL DR_HOOK('CUFLXNTL',0,ZHOOK_HANDLE)
ZTMST=PTSPHY
ZCONS1A=RCPD/(RLMLT*RG*RTAUMEL)
!ZCONS2=1.0_JPRB/(RG*ZTMST)
ZCONS2=RMFCFL/(RG*ZTMST)

!*    1.0          DETERMINE FINAL CONVECTIVE FLUXES
!                  ---------------------------------

DO JL=KIDIA,KFDIA
  PRAIN5(JL)=0.0_JPRB
  IF(.NOT.LDCUM(JL).OR.KDTOP(JL) < KCTOP(JL)) LDDRAF(JL)=.FALSE.
  IF(.NOT.LDCUM(JL)) KTYPE(JL)=0
  IDBAS(JL)=KLEV
  if(ldland(jl)) then
  ! zrhebc5(jl)=rhebc
    zrhebc5(jl)=0.7_jprb
  else
  ! zrhebc5(jl)=rhebc
    zrhebc5(jl)=0.9_jprb
  endif
ENDDO
! TO GET IDENTICAL RESULTS FOR DIFFERENT NPROMA FORCE KTOPM2 TO 2
KTOPM2=2
DO JK=KTOPM2,KLEV
!DIR$ IVDEP
!OCL NOVREC
  IKB=MIN(JK+1,KLEV)
  DO JL=KIDIA,KFDIA
    PMFLXR5(JL,JK)=0.0_JPRB
    PMFLXS5(JL,JK)=0.0_JPRB
    PDPMEL5(JL,JK)=0.0_JPRB
    IF(LDCUM(JL).AND.JK >= KCTOP(JL)) THEN
      PMFUS5(JL,JK)=PMFUS5(JL,JK)-PMFU5(JL,JK)*&
       & (RCPD*PTENH5(JL,JK)+PGEOH5(JL,JK))  
      PMFUQ5(JL,JK)=PMFUQ5(JL,JK)-PMFU5(JL,JK)*PQENH5(JL,JK)
      PLGLAC5(JL,JK)=PMFU5(JL,JK)*PLGLAC5(JL,JK)
      LLDDRAF=LDDRAF(JL).AND.JK >= KDTOP(JL)  
      IF(LLDDRAF) THEN
        PMFDS5(JL,JK)=PMFDS5(JL,JK)-PMFD5(JL,JK)*&
         & (RCPD*PTENH5(JL,JK)+PGEOH5(JL,JK))  
        PMFDQ5(JL,JK)=PMFDQ5(JL,JK)-PMFD5(JL,JK)*PQENH5(JL,JK)
      ELSE
        PMFD5(JL,JK)=0.0_JPRB
        PMFDS5(JL,JK)=0.0_JPRB
        PMFDQ5(JL,JK)=0.0_JPRB
        PDMFDP5(JL,JK-1)=0.0_JPRB
      ENDIF
      IF(LLDDRAF.AND.PMFD5(JL,JK)<0._JPRB .AND. PMFD5(JL,IKB)==0._JPRB) THEN
        IDBAS(JL)=JK
      ENDIF     
    ELSE
      PMFU5(JL,JK)=0.0_JPRB
      PMFD5(JL,JK)=0.0_JPRB
      PMFUS5(JL,JK)=0.0_JPRB
      PMFDS5(JL,JK)=0.0_JPRB
      PMFUQ5(JL,JK)=0.0_JPRB
      PMFDQ5(JL,JK)=0.0_JPRB
      PMFUL5(JL,JK)=0.0_JPRB
      PLGLAC5(JL,JK)=0.0_JPRB
      PDMFUP5(JL,JK-1)=0.0_JPRB
      PDMFDP5(JL,JK-1)=0.0_JPRB
      PLUDE5(JL,JK-1)=0.0_JPRB
    ENDIF
  ENDDO
ENDDO
PMFLXR5(:,KLEV+1)=0.0_JPRB
PMFLXS5(:,KLEV+1)=0.0_JPRB

!DIR$ IVDEP
!OCL NOVREC
DO JL=KIDIA,KFDIA
  IF(LDCUM(JL)) THEN
    IKB=KCBOT(JL)
    IK=IKB+1
    ZZP5=((PAPH5(JL,KLEV+1)-PAPH5(JL,IK))/(PAPH5(JL,KLEV+1)-PAPH5(JL,IKB)))
    IF(KTYPE(JL) == 3) ZZP5=ZZP5*ZZP5
    PMFU5(JL,IK)=PMFU5(JL,IKB)*ZZP5
    IF (LPHYLIN) THEN
      ZTARG5=PTENH5(JL,IKB)
      ZOEALFA5=0.545_JPRB*(TANH(0.17_JPRB*(ZTARG5-RLPTRC))+1.0_JPRB)
      ZOELHM5=ZOEALFA5*RLVTT+(1.0_JPRB-ZOEALFA5)*RLSTT
      PMFUS5(JL,IK)=(PMFUS5(JL,IKB)-&
       & ZOELHM5               *PMFUL5(JL,IKB))*ZZP5  
    ELSE
      PMFUS5(JL,IK)=(PMFUS5(JL,IKB)-&
       & FOELHM(PTENH5(JL,IKB))*PMFUL5(JL,IKB))*ZZP5  
    ENDIF
    PMFUQ5(JL,IK)=(PMFUQ5(JL,IKB)+PMFUL5(JL,IKB))*ZZP5
    PMFUL5(JL,IK)=0.0_JPRB
  ENDIF
ENDDO
DO JK=KTOPM2,KLEV
!DIR$ IVDEP
!OCL NOVREC
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK > KCBOT(JL)+1) THEN
      IKB=KCBOT(JL)+1
      ZZP5=((PAPH5(JL,KLEV+1)-PAPH5(JL,JK))/(PAPH5(JL,KLEV+1)-PAPH5(JL,IKB)))
      IF(KTYPE(JL) == 3) ZZP5=ZZP5*ZZP5
      PMFU5(JL,JK)=PMFU5(JL,IKB)*ZZP5
      PMFUS5(JL,JK)=PMFUS5(JL,IKB)*ZZP5
      PMFUQ5(JL,JK)=PMFUQ5(JL,IKB)*ZZP5
      PMFUL5(JL,JK)=0.0_JPRB
    ENDIF
    IK=IDBAS(JL)
    LLDDRAF=LDDRAF(JL).AND.JK>IK.AND.IK<KLEV 
    IF(LLDDRAF.AND.IK==KCBOT(JL)+1) THEN
      ZZP5=((PAPH5(JL,KLEV+1)-PAPH5(JL,JK))/(PAPH5(JL,KLEV+1)-PAPH5(JL,IK)))
      IF(KTYPE(JL) == 3)  ZZP5=ZZP5*ZZP5
      PMFD5(JL,JK)=PMFD5(JL,IK)*ZZP5
      PMFDS5(JL,JK)=PMFDS5(JL,IK)*ZZP5
      PMFDQ5(JL,JK)=PMFDQ5(JL,IK)*ZZP5
      PMFDDE_RATE5(JL,JK)=-(PMFD5(JL,JK-1)-PMFD5(JL,JK))
    ELSEIF(LLDDRAF.AND.IK/=KCBOT(JL)+1.AND.JK==IK+1) THEN
      PMFDDE_RATE5(JL,JK)=-(PMFD5(JL,JK-1)-PMFD5(JL,JK))
    ENDIF
  ENDDO
ENDDO

!*    2.            CALCULATE RAIN/SNOW FALL RATES
!*                  CALCULATE MELTING OF SNOW
!*                  CALCULATE EVAPORATION OF PRECIP
!                   -------------------------------

DO JK=KTOPM2,KLEV
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK >= KCTOP(JL)-1) THEN
      PRAIN5(JL)=PRAIN5(JL)+PDMFUP5(JL,JK)
      IF(PMFLXS5(JL,JK) > 0.0_JPRB.AND.PTEN5(JL,JK) > RTT) THEN
        ZCONS15=ZCONS1A*(1.0_JPRB+0.5_JPRB*(PTEN5(JL,JK)-RTT))
        ZFAC5=ZCONS15*(PAPH5(JL,JK+1)-PAPH5(JL,JK))
!jfm            ZSNMLT5=MIN(PMFLXS5(JL,JK),ZFAC5*(PTEN5(JL,JK)-RTT))
        IF (PMFLXS5(JL,JK) < ZFAC5*(PTEN5(JL,JK)-RTT)) THEN
          ZSNMLT5 = PMFLXS5(JL,JK)
        ELSE
          ZSNMLT5 = ZFAC5*(PTEN5(JL,JK)-RTT)
        ENDIF
        PDPMEL5(JL,JK)=ZSNMLT5
        IF (LPHYLIN) THEN
          ZTARG5=PTEN5(JL,JK)-ZSNMLT5/ZFAC5
          ZOEALFA5=0.545_JPRB*(TANH(0.17_JPRB*(ZTARG5-RLPTRC))+1.0_JPRB)
          ZFOEEWL5=R2ES*EXP(R3LES*(ZTARG5-RTT)/(ZTARG5-R4LES))
          ZFOEEWI5=R2ES*EXP(R3IES*(ZTARG5-RTT)/(ZTARG5-R4IES))
          ZOEEWM5=ZOEALFA5*ZFOEEWL5 + (1.0_JPRB-ZOEALFA5)*ZFOEEWI5
          PQSEN5(JL,JK)=ZOEEWM5/PAP5(JL,JK)
        ELSE
          PQSEN5(JL,JK)=FOEEWM(PTEN5(JL,JK)-ZSNMLT5/ZFAC5)/PAP5(JL,JK)
        ENDIF
      ENDIF
      IF (LPHYLIN) THEN
        ZALFAW5=0.545_JPRB*(TANH(0.17_JPRB*(PTEN5(JL,JK)-RLPTRC))+1.0_JPRB)
      ELSE
        ZALFAW5=FOEALFA(PTEN5(JL,JK))
      ENDIF
!no liquid precipitation above melting level
      IF(PTEN5(JL,JK) < RTT .AND. ZALFAW5 > 0.0_JPRB) THEN
        PLGLAC5(JL,JK)=PLGLAC5(JL,JK)+ZALFAW5*(PDMFUP5(JL,JK)+PDMFDP5(JL,JK))
        ZALFAW5=0.0_JPRB
      ENDIF

      PMFLXR5(JL,JK+1)=PMFLXR5(JL,JK)+ZALFAW5*&
       & (PDMFUP5(JL,JK)+PDMFDP5(JL,JK))+PDPMEL5(JL,JK)  
      PMFLXS5(JL,JK+1)=PMFLXS5(JL,JK)+(1.0_JPRB-ZALFAW5)*&
       & (PDMFUP5(JL,JK)+PDMFDP5(JL,JK))-PDPMEL5(JL,JK)  
      IF(PMFLXR5(JL,JK+1)+PMFLXS5(JL,JK+1) < 0.0_JPRB) THEN
        PDMFDP5(JL,JK)=-(PMFLXR5(JL,JK)+PMFLXS5(JL,JK)+PDMFUP5(JL,JK))
        PMFLXR5(JL,JK+1)=0.0_JPRB
        PMFLXS5(JL,JK+1)=0.0_JPRB
        PDPMEL5(JL,JK)  =0.0_JPRB
      ELSEIF(PMFLXR5(JL,JK+1) < 0.0_JPRB ) THEN
        PMFLXS5(JL,JK+1)=PMFLXS5(JL,JK+1)+PMFLXR5(JL,JK+1)
        PMFLXR5(JL,JK+1)=0.0_JPRB
      ELSEIF(PMFLXS5(JL,JK+1) < 0.0_JPRB ) THEN
        PMFLXR5(JL,JK+1)=PMFLXR5(JL,JK+1)+PMFLXS5(JL,JK+1)
        PMFLXS5(JL,JK+1)=0.0_JPRB
      ENDIF
    ENDIF
  ENDDO
ENDDO
DO JK=KTOPM2,KLEV
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL).AND.JK >= KCBOT(JL)) THEN
      ZRFL5=PMFLXR5(JL,JK)+PMFLXS5(JL,JK)
      IF(ZRFL5 > 1.E-20_JPRB) THEN
!jfm            ZDRFL15=RCPECONS*MAX(0.,PQSEN5(JL,JK)-PQEN5(JL,JK))*RCUCOV*
!jfm     1             (SQRT(PAPH5(JL,JK)/PAPH5(JL,KLEV+1))/5.09E-3*
!jfm     2             ZRFL5/RCUCOV)**0.5777*
!jfm     3             (PAPH5(JL,JK+1)-PAPH5(JL,JK))
        IF ((PQSEN5(JL,JK)-PQEN5(JL,JK)) > 0.0_JPRB) THEN
          Z1S5 = PQSEN5(JL,JK)-PQEN5(JL,JK)
        ELSE
          Z1S5 = 0.0_JPRB
        ENDIF
        ZDRFL15=RCPECONS*Z1S5*RCUCOV*&
         & (SQRT(PAPH5(JL,JK)/PAPH5(JL,KLEV+1))/5.09E-3_JPRB*&
         & ZRFL5/RCUCOV)**0.5777_JPRB*&
         & (PAPH5(JL,JK+1)-PAPH5(JL,JK))  
        ZRNEW5=ZRFL5-ZDRFL15
!jfm            ZRMIN5=ZRFL5-RCUCOV*MAX(0.,RHEBC*PQSEN5(JL,JK)-PQEN5(JL,JK))
!jfm     1            *ZCONS2*(PAPH5(JL,JK+1)-PAPH5(JL,JK))
!jfm            ZRNEW5=MAX(ZRNEW5,ZRMIN5)
!jfm            ZRFLN5=MAX(ZRNEW5,0.)
!jfm            ZDRFL5=MIN(0.,ZRFLN5-ZRFL5)
!jfm            ZDENOM5=1./MAX(1.E-20,PMFLXR5(JL,JK)+PMFLXS5(JL,JK))
        IF ((ZRHEBC5(JL)*PQSEN5(JL,JK)-PQEN5(JL,JK)) > 0.0_JPRB) THEN
          Z2S5 = ZRHEBC5(JL)*PQSEN5(JL,JK)-PQEN5(JL,JK)
        ELSE
          Z2S5 = 0.0_JPRB
        ENDIF
        ZRMIN5=ZRFL5-RCUCOV*Z2S5*ZCONS2*(PAPH5(JL,JK+1)-PAPH5(JL,JK))
        IF (ZRNEW5 < ZRMIN5) THEN
          ZRNEW5 = ZRMIN5
        ENDIF
        IF (ZRNEW5 > 0.0_JPRB) THEN
          ZRFLN5 = ZRNEW5
        ELSE
          ZRFLN5 = 0.0_JPRB
        ENDIF
        IF ((ZRFLN5-ZRFL5) < 0.0_JPRB) THEN
          ZDRFL5 = ZRFLN5-ZRFL5
        ELSE
          ZDRFL5 = 0.0_JPRB
        ENDIF
        IF ((PMFLXR5(JL,JK)+PMFLXS5(JL,JK)) > 1.E-20_JPRB) THEN
          ZDENOM5 = 1.0_JPRB/(PMFLXR5(JL,JK)+PMFLXS5(JL,JK))
        ELSE
          ZDENOM5 = 1.E+20_JPRB
        ENDIF
        IF (LPHYLIN) THEN
          ZALFAW5=0.545_JPRB*(TANH(0.17_JPRB*(PTEN5(JL,JK)-RLPTRC))+1.0_JPRB)
        ELSE
          ZALFAW5=FOEALFA(PTEN5(JL,JK))
        ENDIF
        IF(PTEN5(JL,JK) < RTT) ZALFAW5=0.0_JPRB
        ZPDR5=ZALFAW5*PDMFDP5(JL,JK)
        ZPDS5=(1.0_JPRB-ZALFAW5)*PDMFDP5(JL,JK)
        PMFLXR5(JL,JK+1)=PMFLXR5(JL,JK)+ZPDR5 &
         & +PDPMEL5(JL,JK)+ZDRFL5*PMFLXR5(JL,JK)*ZDENOM5  
        PMFLXS5(JL,JK+1)=PMFLXS5(JL,JK)+ZPDS5 &
         & -PDPMEL5(JL,JK)+ZDRFL5*PMFLXS5(JL,JK)*ZDENOM5  
        PDMFUP5(JL,JK)=PDMFUP5(JL,JK)+ZDRFL5
        IF(PMFLXR5(JL,JK+1)+PMFLXS5(JL,JK+1) < 0.0_JPRB) THEN
          PDMFUP5(JL,JK)=PDMFUP5(JL,JK)-(PMFLXR5(JL,JK+1)+PMFLXS5(JL,JK+1))
          PMFLXR5(JL,JK+1)=0.0_JPRB
          PMFLXS5(JL,JK+1)=0.0_JPRB
          PDPMEL5(JL,JK)  =0.0_JPRB
        ELSEIF(PMFLXR5(JL,JK+1) < 0.0_JPRB ) THEN
          PMFLXS5(JL,JK+1)=PMFLXS5(JL,JK+1)+PMFLXR5(JL,JK+1)
          PMFLXR5(JL,JK+1)=0.0_JPRB
        ELSEIF(PMFLXS5(JL,JK+1) < 0.0_JPRB ) THEN
          PMFLXR5(JL,JK+1)=PMFLXR5(JL,JK+1)+PMFLXS5(JL,JK+1)
          PMFLXS5(JL,JK+1)=0.0_JPRB
        ENDIF
      ELSE
        PMFLXR5(JL,JK+1)=0.0_JPRB
        PMFLXS5(JL,JK+1)=0.0_JPRB
        PDMFDP5(JL,JK)=0.0_JPRB
        PDPMEL5(JL,JK)=0.0_JPRB
      ENDIF
    ENDIF
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('CUFLXNTL',1,ZHOOK_HANDLE)
END SUBROUTINE CUFLXNTL
