SUBROUTINE CUMASTRN2TL &
 & (  KIDIA,    KFDIA,    KLON,    KLEV,&
 & LDLAND,   LDRAIN1D,   PTSPHY,&
 & PTEN5,    PQEN5,    PUEN5,    PVEN5,&
 & PVERVEL5, PQSEN5,   PQHFL5,   PAHFS5,&
 & PAP5,     PAPH5,    PGEO5,    PGEOH5,&
 & PTENT5,   PTENQ5,   PTENU5,   PTENV5,&
 & LDCUM,    KTYPE,    KCBOT,    KCTOP,&
 & KBOTSC,   LDSC,&
 & PTU5,     PQU5,     PLU5,     PLUDE5,&
 & PENTH5,   PMFLXR5,  PMFLXS5,  PRAIN5,&
 & PMFU5,    PMFD5,&
 & KTRAC,    PCEN5,    PTENC5,   PSCAV5,&
 & PTEN,     PQEN,     PUEN,     PVEN,&
 & PVERVEL,  PQSEN,    PQHFL,    PAHFS,&
 & PAP,      PAPH,     PGEO,     PGEOH,&
 & PTENT,    PTENQ,    PTENU,    PTENV,&
 & PTU,      PQU,      PLU,      PLUDE,&
 & PENTH,    PMFLXR,   PMFLXS,   PRAIN,&
 & PMFU,     PMFD,&
 & PCEN,     PTENC )       

!**** *CUMASTRN2TL*  MASTER ROUTINE FOR SIMPLIFIED CUMULUS MASSFLUX-SCHEME
!                   (Tangent linear)

!     P. LOPEZ   E.C.M.W.F      15/12/2003

!     Adapted from

!     M.TIEDTKE      E.C.M.W.F.     1986/1987/1989

!     PURPOSE
!     -------

!          THIS ROUTINE COMPUTES THE PHYSICAL TENDENCIES OF THE
!     PROGNOSTIC VARIABLES T,Q,U AND V DUE TO CONVECTIVE PROCESSES.
!     PROCESSES CONSIDERED ARE: CONVECTIVE FLUXES, FORMATION OF
!     PRECIPITATION, EVAPORATION OF FALLING RAIN BELOW CLOUD BASE,
!     SATURATED CUMULUS DOWNDRAFTS.

!**   INTERFACE.
!     ----------

!          *CUMASTRN2TL* IS CALLED FROM *CUCALLN2TL*
!     THE ROUTINE TAKES ITS INPUT FROM THE LONG-TERM STORAGE
!     T,Q,U,V,PHI AND P AND MOISTURE TENDENCIES.
!     IT RETURNS ITS OUTPUT TO THE SAME SPACE
!      1.MODIFIED TENDENCIES OF MODEL VARIABLES
!      2.RATES OF CONVECTIVE PRECIPITATION
!        (USED IN SUBROUTINE SURF)
!      3.CLOUD BASE, CLOUD TOP AND PRECIP FOR RADIATION
!        (USED IN SUBROUTINE CLOUD)

!     METHOD
!     -------

!     PARAMETERIZATION IS DONE USING A MASSFLUX-SCHEME.
!        (1) DEFINE CONSTANTS AND PARAMETERS
!        (2) SPECIFY VALUES (T,Q,QS...) AT HALF LEVELS AND
!            INITIALIZE UPDRAFT- AND DOWNDRAFT-VALUES IN 'CUININ2TL'
!        (3) CALCULATE CLOUD BASE IN 'CUBASEN2TL'
!            AND SPECIFY CLOUD BASE MASSFLUX FROM PBL MOISTURE BUDGET
!        (4) DO CLOUD ASCENT IN 'CUASCN2TL' IN ABSENCE OF DOWNDRAFTS
!        (5) DO DOWNDRAFT CALCULATIONS IN CUDDRAFN2TL
!        (6) SCALING OF MASSFLUX CONSIDERING THE CAPE CLOSURE
!        (7) DO FINAL ADJUSMENTS TO CONVECTIVE FLUXES IN 'CUFLXN2TL',
!            DO EVAPORATION IN SUBCLOUD LAYER
!        (8) CALCULATE INCREMENTS OF T AND Q IN 'CUDTDQN2TL'
!        (9) CALCULATE INCREMENTS OF U AND V IN 'CUDUDV2TL'

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS
!    *KTRAC*        NUMBER OF TRACERS

!     INPUT PARAMETERS (LOGICAL)

!    *LDLAND*       LAND SEA MASK (.TRUE. FOR LAND)

!     INPUT PARAMETERS (REAL)

!    *PTSPHY*       TIME STEP FOR THE PHYSICS                       S
!    *PTEN5*        PROVISIONAL ENVIRONMENT TEMPERATURE (T+1)    (Trajectory)
!    *PQEN5*        PROVISIONAL ENVIRONMENT SPEC. HUMIDITY (T+1) (Trajectory)
!    *PUEN5*        PROVISIONAL ENVIRONMENT U-VELOCITY (T+1)     (Trajectory)
!    *PVEN5*        PROVISIONAL ENVIRONMENT V-VELOCITY (T+1)     (Trajectory)
!    *PCEN5*        CHEMICAL TRACERS                             (Trajectory) 
!    *PVERVEL5*     VERTICAL VELOCITY                            (Trajectory)
!    *PQSEN5*       ENVIRONMENT SPEC. SATURATION HUMIDITY (T+1)  (Trajectory)
!    *PQHFL5*       MOISTURE FLUX (EXCEPT FROM SNOW EVAP.)       (Trajectory) 
!    *PAHFS5*       SENSIBLE HEAT FLUX                           (Trajectory)
!    *PAP5*         PROVISIONAL PRESSURE ON FULL LEVELS          (Trajectory)
!    *PAPH5*        PROVISIONAL PRESSURE ON HALF LEVELS          (Trajectory)
!    *PGEO5*        GEOPOTENTIAL                                 (Trajectory) 
!    *PGEOH5*       GEOPOTENTIAL ON HALF LEVELS                  (Trajectory) 
!    *PSCAV5*       SCAVENGING COEFFICIENT                       (Trajectory)

!    *PTEN*         PROVISIONAL ENVIRONMENT TEMPERATURE (T+1)       K
!    *PQEN*         PROVISIONAL ENVIRONMENT SPEC. HUMIDITY (T+1)  KG/KG
!    *PUEN*         PROVISIONAL ENVIRONMENT U-VELOCITY (T+1)       M/S
!    *PVEN*         PROVISIONAL ENVIRONMENT V-VELOCITY (T+1)       M/S
!    *PCEN*         CHEMICAL TRACERS                             
!    *PVERVEL*      VERTICAL VELOCITY                             PA/S
!    *PQSEN*        ENVIRONMENT SPEC. SATURATION HUMIDITY (T+1)   KG/KG
!    *PQHFL*        MOISTURE FLUX (EXCEPT FROM SNOW EVAP.)        KG/(SM2)
!    *PAHFS*        SENSIBLE HEAT FLUX                            W/M2
!    *PAP*          PROVISIONAL PRESSURE ON FULL LEVELS             PA
!    *PAPH*         PROVISIONAL PRESSURE ON HALF LEVELS             PA
!    *PGEO*         GEOPOTENTIAL                                  M2/S2
!    *PGEOH*        GEOPOTENTIAL ON HALF LEVELS                   M2/S2

!    UPDATED PARAMETERS (REAL):

!    *PTENT5*       TEMPERATURE TENDENCY                         (Trajectory)     
!    *PTENQ5*       MOISTURE TENDENCY                            (Trajectory)  
!    *PTENU5*       TENDENCY OF U-COMP. OF WIND                  (Trajectory)  
!    *PTENV5*       TENDENCY OF V-COMP. OF WIND                  (Trajectory)  
!    *PTENC5*       TRACER TENDENCY                              (Trajectory)

!    *PTENT*        TEMPERATURE TENDENCY                           K/S
!    *PTENQ*        MOISTURE TENDENCY                             KG/(KG S)
!    *PTENU*        TENDENCY OF U-COMP. OF WIND                    M/S2
!    *PTENV*        TENDENCY OF V-COMP. OF WIND                    M/S2
!    *PTENC*        TRACER TENDENCY                                1/S

!    OUTPUT PARAMETERS (LOGICAL):

!    *LDCUM*        FLAG: .TRUE. FOR CONVECTIVE POINTS 
!    *LDSC*         FLAG: .TRUE. FOR SC-POINTS

!    OUTPUT PARAMETERS (INTEGER):

!    *KTYPE*        TYPE OF CONVECTION
!                       1 = PENETRATIVE CONVECTION
!                       2 = SHALLOW CONVECTION
!                       3 = MIDLEVEL CONVECTION
!    *KCBOT*        CLOUD BASE LEVEL
!    *KCTOP*        CLOUD TOP LEVEL
!    *KBOTSC*       CLOUD BASE LEVEL FOR SC-CLOUDS

!    OUTPUT PARAMETERS (REAL):

!    *PTU5*         TEMPERATURE IN UPDRAFTS                      (Trajectory)    
!    *PQU5*         SPEC. HUMIDITY IN UPDRAFTS                   (Trajectory)   
!    *PLU5*         LIQUID WATER CONTENT IN UPDRAFTS             (Trajectory)   
!    *PLUDE5*       DETRAINED LIQUID WATER                       (Trajectory)    
!    *PENTH5*       INCREMENT OF DRY STATIC ENERGY               (Trajectory)    
!    *PMFLXR5*      CONVECTIVE RAIN FLUX                         (Trajectory)    
!    *PMFLXS5*      CONVECTIVE SNOW FLUX                         (Trajectory)    
!    *PRAIN5*       TOTAL PRECIP. PRODUCED IN CONV. UPDRAFTS     (Trajectory)    
!                   (NO EVAPORATION IN DOWNDRAFTS)
!    *PMFU*         MASSFLUX UPDRAFTS                            (Trajectory)
!    *PMFD*         MASSFLUX DOWNDRAFTS                          (Trajectory)

!    *PTU*          TEMPERATURE IN UPDRAFTS                         K
!    *PQU*          SPEC. HUMIDITY IN UPDRAFTS                    KG/KG
!    *PLU*          LIQUID WATER CONTENT IN UPDRAFTS              KG/KG
!    *PLUDE*        DETRAINED LIQUID WATER                        KG/(M3*S)
!    *PENTH*        INCREMENT OF DRY STATIC ENERGY                 J/(KG*S)
!    *PMFLXR*       CONVECTIVE RAIN FLUX                          KG/(M2*S)
!    *PMFLXS*       CONVECTIVE SNOW FLUX                          KG/(M2*S)
!    *PRAIN*        TOTAL PRECIP. PRODUCED IN CONV. UPDRAFTS      KG/(M2*S)
!                   (NO EVAPORATION IN DOWNDRAFTS)
!    *PMFU*         MASSFLUX UPDRAFTS                             KG/(M2*S)
!    *PMFD*         MASSFLUX DOWNDRAFTS                           KG/(M2*S)

!     EXTERNALS.
!     ----------

!       CUININTL:  INITIALIZES VALUES AT VERTICAL GRID USED IN CU-PARAMETR.
!       CUBASENTL: CLOUD BASE CALCULATION FOR PENETR.AND SHALLOW CONVECTION
!       CUASCNTL:  CLOUD ASCENT FOR ENTRAINING PLUME
!       CUDLFSNTL: DETERMINES VALUES AT LFS FOR DOWNDRAFTS
!       CUDDRAFNTL:DOES MOIST DESCENT FOR CUMULUS DOWNDRAFTS
!       CUFLXNTL:  FINAL ADJUSTMENTS TO CONVECTIVE FLUXES (ALSO IN PBL)
!       CUDQDTNTL: UPDATES TENDENCIES FOR T AND Q
!       CUDUDVTL: UPDATES TENDENCIES FOR U AND V

!     SWITCHES.
!     --------

!     LMFDD2=.TRUE.    CUMULUS DOWNDRAFTS SWITCHED ON
!     LMFDUDV2=.TRUE.  CUMULUS FRICTION SWITCHED ON

!     MODEL PARAMETERS (DEFINED IN SUBROUTINE SUCUMF)
!     ------------------------------------------------
!     RMFDEPS    FRACTIONAL MASSFLUX FOR DOWNDRAFTS AT LFS

!     REFERENCE.
!     ----------

!          PAPER ON MASSFLUX SCHEME (TIEDTKE,1989)
!          PAPER ON LINEARIZED CONVECTION SCHEME (LOPEZ and MOREAU, 2005)

!     MODIFICATIONS
!     -------------
!     M.Hamrud     01-Oct-2003 CY28 Cleaning
!     P.Lopez      01-Oct-2005 New version of linearized convection
!     P.Lopez      14-02-2006  Added transport of tracers        
!     P.Lopez      19-Oct-2007 Revised version to improve match to Tiedtke scheme
!     A.Geer       01-Oct-2008 LDRAIN1D name change to reflect usage
!     P.Bechtold   07-Oct-2009 Add KE dissipation and Tracer scavenging

!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RD       ,RCPD     ,&
 & RLVTT    ,RLSTT    ,RTT     ,RETV       
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE    ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOECUMF2 , ONLY : LMFDD2    ,LMFDUDV2  ,LMFUVDIS2  ,RTAU2      ,RDEPTHS2,&
                    & RUVPER2   ,RMFSOLCT2 ,RMFSOLTQ2  ,RMFSOLUV2  ,&
                    & RMFCMIN2  ,RMFCFL2   ,RMFLIC2    ,RMFLIA2
USE YOMCUMFS , ONLY : LREGCV
USE YOEPHY   , ONLY : LMFTRAC   ,LMFSCAV

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAC
LOGICAL           ,INTENT(IN)    :: LDLAND(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTEN5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQEN5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUEN5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEN5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCEN5(KLON,KLEV,KTRAC) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSCAV5(KTRAC) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVERVEL5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQSEN5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQHFL5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAHFS5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEO5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENU5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENV5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENC5(KLON,KLEV,KTRAC)
LOGICAL           ,INTENT(IN)    :: LDRAIN1D
LOGICAL           ,INTENT(OUT)   :: LDCUM(KLON)
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTYPE(KLON)
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCBOT(KLON)
INTEGER(KIND=JPIM),INTENT(OUT)   :: KCTOP(KLON)
INTEGER(KIND=JPIM),INTENT(OUT)   :: KBOTSC(KLON)
LOGICAL           ,INTENT(OUT)   :: LDSC(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTU5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQU5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLU5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLUDE5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PENTH5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFLXR5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFLXS5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAIN5(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFU5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFD5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTEN(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQEN(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUEN(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVEN(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCEN(KLON,KLEV,KTRAC) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVERVEL(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQSEN(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQHFL(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAHFS(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEO(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENV(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENC(KLON,KLEV,KTRAC) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLUDE(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PENTH(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFLXR(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PMFLXS(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRAIN(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFU(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PMFD(KLON,KLEV)
REAL(KIND=JPRB) ::     ZTENH5(KLON,KLEV),      ZQENH5(KLON,KLEV),&
                     & ZQSENH5(KLON,KLEV),&
                     & ZTD5(KLON,KLEV),        ZQD5(KLON,KLEV),&
                     & ZMFUS5(KLON,KLEV),      ZMFDS5(KLON,KLEV),&
                     & ZMFUQ5(KLON,KLEV),      ZMFDQ5(KLON,KLEV),&
                     & ZDMFUP5(KLON,KLEV),     ZDMFDP5(KLON,KLEV),&
                     & ZDMFUPC5(KLON,KLEV),    ZDMFDPC5(KLON,KLEV),&
                     & ZMFUL5(KLON,KLEV),      ZRFL5(KLON),&
                     & ZUU5(KLON,KLEV),        ZVU5(KLON,KLEV),&
                     & ZUD5(KLON,KLEV),        ZVD5(KLON,KLEV)  
REAL(KIND=JPRB) ::     ZMFUB5(KLON),           ZDPMEL5(KLON,KLEV),&
                     & ZLGLAC5(KLON,KLEV),     ZQPRCV5(KLON,KLEV)  
REAL(KIND=JPRB) ::     ZWUBASE5(KLON),         ZBUOH5(KLON,KLEV)

REAL(KIND=JPRB) ::     ZTENH(KLON,KLEV),       ZQENH(KLON,KLEV),&
                     & ZQSENH(KLON,KLEV),&
                     & ZTD(KLON,KLEV),         ZQD(KLON,KLEV),&
                     & ZMFUS(KLON,KLEV),       ZMFDS(KLON,KLEV),&
                     & ZMFUQ(KLON,KLEV),       ZMFDQ(KLON,KLEV),&
                     & ZDMFUP(KLON,KLEV),      ZDMFDP(KLON,KLEV),&
                     & ZDMFUPC(KLON,KLEV),     ZDMFDPC(KLON,KLEV),&
                     & ZMFUL(KLON,KLEV),       ZRFL(KLON),&
                     & ZUU(KLON,KLEV),         ZVU(KLON,KLEV),&
                     & ZUD(KLON,KLEV),         ZVD(KLON,KLEV)  
REAL(KIND=JPRB) ::     ZMFUB(KLON),            ZDPMEL(KLON,KLEV),&
                     & ZLGLAC(KLON,KLEV),      ZQPRCV(KLON,KLEV)  
REAL(KIND=JPRB) ::     ZWUBASE(KLON),          ZBUOH(KLON,KLEV)
REAL(KIND=JPRB) ::     ZRESCAL5(KLON)
REAL(KIND=JPRB) ::     ZDHPBL5(KLON), ZDHPBL(KLON)

INTEGER(KIND=JPIM) ::  ILAB(KLON,KLEV), IDTOP(KLON), ICTOP0(KLON), ILWMIN(KLON)
INTEGER(KIND=JPIM) ::  ISTUP(KLON)
REAL(KIND=JPRB) ::     ZCAPE5(KLON), ZHEAT5(KLON)
REAL(KIND=JPRB) ::     ZCAPE (KLON), ZHEAT (KLON)
REAL(KIND=JPRB) ::     ZMFUDE_RATE5 (KLON,KLEV), ZMFDDE_RATE5 (KLON,KLEV)
REAL(KIND=JPRB) ::     ZMFUDE_RATE  (KLON,KLEV), ZMFDDE_RATE  (KLON,KLEV)
REAL(KIND=JPRB) ::     ZERATEU5  (KLON,KLEV), ZERATED5  (KLON,KLEV)
REAL(KIND=JPRB) ::     ZERATEU   (KLON,KLEV), ZERATED   (KLON,KLEV)

LOGICAL ::  LLDDRAF(KLON), LLDDRAF3(KLON), LLDCUM(KLON)
LOGICAL ::  LLO1, LLO2

INTEGER(KIND=JPIM) :: IKB, ITOPM2, JK, JL, IK

REAL(KIND=JPRB) :: ZCONS2, ZDZ5, ZMFMAX5, ZPBMPT5, ZRO5, ZZBUO5, ZDS5 
REAL(KIND=JPRB) :: ZCONS,  ZCONS1, ZCONS3, ZCONS4, ZEPS
REAL(KIND=JPRB) :: ZQUMQE5, ZDQMIN5, ZDH5, ZTENDH5
REAL(KIND=JPRB) :: ZQUMQE,  ZDQMIN,  ZDH,  ZTENDH
REAL(KIND=JPRB) :: ZWM5,  ZTAU5, ZTV5
REAL(KIND=JPRB) :: ZWM ,  ZTAU , ZTV
REAL(KIND=JPRB) :: ZRCPD, ZMFA5, ZMFA

REAL(KIND=JPRB) :: ZRO, ZDZ, ZZBUO, ZDS
REAL(KIND=JPRB) :: ZFACT1, ZFACT3, ZFACT4, ZFACT5
REAL(KIND=JPRB) :: ZFACT7, ZFACT8, ZFACT9, ZFACT10
REAL(KIND=JPRB) :: ZFACT11, ZFACT12, ZFACT13
REAL(KIND=JPRB) :: ZMFMAX, ZRMFCMIN

! scaling factor and momentum massflux
REAL(KIND=JPRB) :: ZMFS5(KLON), ZMFUUS5(KLON,KLEV), ZMFDUS5(KLON,KLEV) 
REAL(KIND=JPRB) :: ZMFS (KLON), ZMFUUS (KLON,KLEV), ZMFDUS (KLON,KLEV)
REAL(KIND=JPRB) :: ZTENU5(KLON,KLEV), ZTENV5(KLON,KLEV)
REAL(KIND=JPRB) :: ZTENU (KLON,KLEV), ZTENV (KLON,KLEV)
REAL(KIND=JPRB) :: ZDMFCOR5(KLON)
REAL(KIND=JPRB) :: ZDMFCOR (KLON)
REAL(KIND=JPRB) :: ZWMEAN5(KLON)
REAL(KIND=JPRB) :: ZWMEAN (KLON)
REAL(KIND=JPRB) :: ZMFUUB5(KLON), ZMFUVB5(KLON)
REAL(KIND=JPRB) :: ZMFUUB (KLON), ZMFUVB (KLON)

! Allocatable arrays for Tracers
REAL(KIND=JPRB) :: ZSCAV5(KTRAC)
REAL(KIND=JPRB), DIMENSION(:,:) ,ALLOCATABLE  :: ZMFUUC5, ZMFDUC5, ZMFUDR5, ZMFDDR5,&
                                               & ZMFUUC,  ZMFDUC,  ZMFUDR,  ZMFDDR

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cuascn2tl.intfb.h"
#include "cubasen2tl.intfb.h"
#include "cuddrafn2tl.intfb.h"
#include "cudtdqn2tl.intfb.h"
#include "cududv2tl.intfb.h"
#include "cuflx2tl.intfb.h"
#include "cuinin2tl.intfb.h"
#include "cuctracertl.intfb.h"

#include "fcttre.h"

!---------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CUMASTRN2TL',0,ZHOOK_HANDLE)

!     1.           SPECIFY CONSTANTS AND PARAMETERS
!                  --------------------------------
ZEPS=1.0E-10_JPRB
ZRCPD=1.0_JPRB/RCPD
ZCONS1=1.0E-06_JPRB
ZCONS =1.0_JPRB/(RG*PTSPHY)
ZCONS2=RMFCFL2/(RG*PTSPHY)
ZCONS3=RCPD*ZCONS1
ZCONS4=RLVTT*ZCONS1
ZRMFCMIN=1.0_JPRB/RMFCMIN2

!----------------------------------------------------------------------

!*    2.           INITIALIZE VALUES AT VERTICAL GRID POINTS IN 'CUINITL'
!                  -----------------------------------------------------

CALL CUININ2TL &
 & ( KIDIA,    KFDIA,    KLON,    KLEV,&
 & PTEN5,    PQEN5,    PQSEN5,   PUEN5,    PVEN5,&
 & PVERVEL5, PGEO5,    PAPH5,&
 & ILWMIN,   ILAB,&
 & ZTENH5,   ZQENH5,   ZQSENH5,  PGEOH5,&
 & PTU5,     PQU5,     ZTD5,     ZQD5,&
 & ZUU5,     ZVU5,     ZUD5,     ZVD5,&
 & PLU5,&
 & PTEN,     PQEN,     PQSEN,    PUEN,     PVEN,&
 & PVERVEL,  PGEO,     PAPH,&
 & ZTENH,    ZQENH,    ZQSENH,   PGEOH,&
 & PTU,      PQU,      ZTD,      ZQD,&
 & ZUU,      ZVU,      ZUD,      ZVD,&
 & PLU     )  

!---------------------------------------------------------------------

!*    3.           CLOUD BASE CALCULATIONS
!                  -----------------------

!*                 DETERMINE CLOUD BASE VALUES IN 'CUBASEN2TL'
!                  -------------------------------------------

CALL CUBASEN2TL &
 & ( KIDIA,    KFDIA,    KLON,     KLEV,&
 & ZTENH5,   ZQENH5,   PGEOH5,   PAP5,     PAPH5,&
 & PQHFL5,   PAHFS5,&
 & PTEN5,    PQEN5,    PGEO5,&
 & PTU5,     PQU5,     PLU5,&
 & ZBUOH5,   ZWUBASE5, ZLGLAC5,  ZQPRCV5,&
 & KTYPE,    ILAB,     LDCUM,    LDSC,&
 & KCBOT,    KBOTSC,   ICTOP0,   ISTUP,&
 & ZTENH,    ZQENH,    PGEOH,    PAP,      PAPH,&
 & PQHFL,    PAHFS,&
 & PTEN,     PQEN,     PGEO,&
 & PTU,      PQU,      PLU,&
 & ZBUOH,    ZWUBASE,  ZLGLAC,   ZQPRCV )  

! CALCULATE COLUMN AND SUB CLOUD LAYER MOISTURE CONVERGENCE
! AND SUB CLOUD LAYER MOIST STATIC ENERGY CONVERGENCE
ZDHPBL5(:)=0.0_JPRB
ZDHPBL (:)=0.0_JPRB
DO JK=2,KLEV
  DO JL=KIDIA,KFDIA
    IF(LDCUM(JL) .AND. JK >= KCBOT(JL)) THEN
      ZTENDH5=RLVTT*PTENQ5(JL,JK)+RCPD*PTENT5(JL,JK)
      ZTENDH =RLVTT*PTENQ (JL,JK)+RCPD*PTENT (JL,JK)

      ZDHPBL5(JL)=ZDHPBL5(JL)+ZTENDH5 *(PAPH5(JL,JK+1)-PAPH5(JL,JK))*ZCONS1
      ZDHPBL (JL)=ZDHPBL (JL)+(ZTENDH  *(PAPH5(JL,JK+1)-PAPH5(JL,JK)) &
              &              + ZTENDH5 *(PAPH (JL,JK+1)-PAPH (JL,JK)))*ZCONS1
    ENDIF
  ENDDO
ENDDO

!*                 SPECIFY INITIAL CLOUD TYPE
!*

DO JL=KIDIA,KFDIA
  IF (LDCUM(JL)) THEN
    IF (KTYPE(JL) /= 3) THEN
      ZPBMPT5=PAPH5(JL,KCBOT(JL))-PAPH5(JL,ICTOP0(JL))
      IF (ZPBMPT5 >= RDEPTHS2) THEN
        KTYPE(JL)=1
      ELSE
        KTYPE(JL)=2
      ENDIF
    ENDIF
  ENDIF
ENDDO

!*    4.        SIMPLIFIED CONVECTION SCHEME
!*              COMPUTE VERTICAL PROFILE OF RATIO M(z)/Mbase
!               --------------------------------------------

!DIR$ IVDEP
!OCL NOVREC
DO JL=KIDIA,KFDIA
  IF (LDCUM(JL)) THEN
    ZMFUB5(JL)=1.0_JPRB
    KCTOP(JL)=ICTOP0(JL)
  ELSE
    ZMFUB5(JL)=0.0_JPRB
  ENDIF
  ZMFUB(JL)=0.0_JPRB
ENDDO

CALL CUASCN2TL &
 & ( KIDIA,    KFDIA,    KLON,   KLEV,&
 & PTEN5,    ZTENH5,    ZQENH5,&
 & PGEOH5,   PQEN5,    PQSEN5,&
 & PAPH5,    ZLGLAC5,  ZQPRCV5,&
 & ZBUOH5,   ZWUBASE5, LDCUM,    KTYPE,&
 & PTU5,     PQU5,     PLU5,&
 & PMFU5,    ZMFUB5,&
 & ZMFUS5,   ZMFUQ5,   ZMFUL5,   PLUDE5,   ZDMFUP5,&
 & KCBOT,    KCTOP,    ISTUP,&
 & ZMFUDE_RATE5, ZWMEAN5, ZERATEU5,&
 & PTEN,     ZTENH,    ZQENH,&
 & PGEOH,    PQEN,     PQSEN,&
 & PAPH,     ZLGLAC,   ZQPRCV,&
 & ZBUOH,    ZWUBASE,&
 & PTU,      PQU,      PLU,&
 & PMFU,     ZMFUB,&
 & ZMFUS,    ZMFUQ,    ZMFUL,    PLUDE,    ZDMFUP,&
 & ZMFUDE_RATE, ZWMEAN, ZERATEU )  


!*                 SPECIFY CLOUD TYPE
!*

DO JL=KIDIA,KFDIA
  IF (LDCUM(JL)) THEN
    IF (KTYPE(JL) /= 3) THEN
      ZPBMPT5=PAPH5(JL,KCBOT(JL))-PAPH5(JL,KCTOP(JL))
      IF (ZPBMPT5 >= RDEPTHS2) THEN
        KTYPE(JL)=1
      ELSE
        KTYPE(JL)=2
      ENDIF
    ENDIF
  ENDIF
ENDDO

DO JL=KIDIA,KFDIA
  ZRFL5(JL)=ZDMFUP5(JL,1)
  ZRFL (JL)=ZDMFUP (JL,1)
ENDDO
DO JK=2,KLEV
  DO JL=KIDIA,KFDIA
    ZRFL5(JL)=ZRFL5(JL)+ZDMFUP5(JL,JK)
    ZRFL (JL)=ZRFL (JL)+ZDMFUP (JL,JK)
  ENDDO
ENDDO

! Initializations 

ITOPM2=0
LLDDRAF(:)=.FALSE.
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    PMFD5(JL,JK)=0.0_JPRB
    PMFD (JL,JK)=0.0_JPRB
    ZMFDS5(JL,JK)=0.0_JPRB
    ZMFDS (JL,JK)=0.0_JPRB
    ZMFDQ5(JL,JK)=0.0_JPRB
    ZMFDQ (JL,JK)=0.0_JPRB
    ZDMFDP5(JL,JK)=0.0_JPRB
    ZDMFDP (JL,JK)=0.0_JPRB
  ENDDO
ENDDO
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZDPMEL5(JL,JK)=0.0_JPRB
    ZDPMEL (JL,JK)=0.0_JPRB
    ZMFDDE_RATE5(JL,JK)=0.0_JPRB
    ZMFDDE_RATE (JL,JK)=0.0_JPRB
    ZERATED5(JL,JK)=0.0_JPRB
    ZERATED (JL,JK)=0.0_JPRB
  ENDDO
ENDDO
IF(LMFUVDIS2) THEN
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZTENU5(JL,JK)=PTENU5(JL,JK)
      ZTENV5(JL,JK)=PTENV5(JL,JK)
      ZTENU (JL,JK)=PTENU (JL,JK)
      ZTENV (JL,JK)=PTENV (JL,JK)
    ENDDO
  ENDDO
ENDIF

!-----------------------------------------------------------------------

!*    5.           CUMULUS DOWNDRAFT CALCULATIONS
!                  ------------------------------

IF(LMFDD2) THEN

!*            DETERMINE LFS AND DOWNDRAFTS IN 'CUDDRAFN2TL'
!             SIMPLIFIED SCHEME
!             ---------------------------------------------

  CALL CUDDRAFN2TL &
   & ( KIDIA,    KFDIA,    KLON,    KLEV,&
   & KCBOT,    KCTOP,    LDCUM,&
   & ZTENH5,   ZQENH5,&
   & PTEN5,    PQSEN5,   PGEO5,&
   & PGEOH5,   PAPH5,    PTU5,     PQU5,&
   & ZRFL5,    ZTD5,     ZQD5,     PMFU5,&
   & PMFD5,    ZMFDS5,   ZMFDQ5,   ZDMFDP5,&
   & IDTOP,    LLDDRAF,&
   & ZMFDDE_RATE5, ZERATED5,&
   & ZTENH,    ZQENH,&
   & PGEOH,    PAPH,     PTU,      PQU,&
   & ZRFL,     ZTD,      ZQD,&
   & PMFD,     ZMFDS,    ZMFDQ,    ZDMFDP,&
   & ZMFDDE_RATE, ZERATED )  

ENDIF

!*     6.       SIMPLIFIED CONVECTION SCHEME:
!*              CALCULATE CLOUD BASE MASSFLUX FROM A
!*              CAPE CLOSURE FOR ALL TYPES OF CONVECTION,
!*              AND FROM THE PROFILE OF RATIO M(z)/Mbase
!*              WHICH WAS CALCULATED IN CUASCN2.
!*              RESCALE ALL MASS-FLUXES ACCORDINGLY.
!           ---------------------------------------------

DO JL=KIDIA,KFDIA
  ZHEAT5(JL)=0.0_JPRB
  ZHEAT (JL)=0.0_JPRB
  ZCAPE5(JL)=0.0_JPRB
  ZCAPE (JL)=0.0_JPRB
ENDDO

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF (LDCUM(JL) .AND. KTYPE(JL) /= 2  .AND. &
      & JK <= KCBOT(JL) .AND. JK > KCTOP(JL)) THEN
      ZTV5=ZTENH5(JL,JK)*(1.0_JPRB+RETV*ZQENH5(JL,JK))
      ZTV =ZTENH (JL,JK)*(1.0_JPRB+RETV*ZQENH5(JL,JK)) &
        & +ZTENH5(JL,JK)*RETV*ZQENH (JL,JK)

      ZRO5=PAPH5(JL,JK)/(RD*ZTV5)
      ZRO =ZRO5*(PAPH (JL,JK)/PAPH5(JL,JK)-ZTV /ZTV5)  

      ZDZ5=PGEOH5(JL,JK-1)-PGEOH5(JL,JK)
      ZDZ =PGEOH (JL,JK-1)-PGEOH (JL,JK)

      ZFACT5=1.0_JPRB/ZTENH5(JL,JK)
      ZDS5=(PTEN5(JL,JK-1)-PTEN5(JL,JK)+ZDZ5/RCPD)*ZFACT5  
      ZDS =( (PTEN (JL,JK-1)-PTEN (JL,JK)+ZDZ /RCPD) &
        & -ZDS5*ZTENH (JL,JK) )*ZFACT5  

      ZHEAT5(JL)=ZHEAT5(JL) &
       & +(ZDS5 + RETV*(PQEN5(JL,JK-1)-PQEN5(JL,JK))) &
       & *(RG*(PMFU5(JL,JK)+PMFD5(JL,JK)))/ZRO5  

      ZFACT1=RG/ZRO5
      ZFACT3=ZFACT1*(PMFU5(JL,JK)+PMFD5(JL,JK))
      ZFACT4=ZFACT1*(ZDS5+RETV*(PQEN5(JL,JK-1)-PQEN5(JL,JK)))
      ZHEAT (JL)=ZHEAT (JL)+ &
       & (ZDS + RETV*(PQEN (JL,JK-1)-PQEN (JL,JK)))*ZFACT3 &
       & + ZFACT4 * ( PMFU (JL,JK)+PMFD (JL,JK) &
       & - (PMFU5(JL,JK)+PMFD5(JL,JK))*ZRO /ZRO5 )  

      ZZBUO5=PTU5(JL,JK)*ZFACT5-1.0_JPRB
      ZZBUO=(PTU (JL,JK)-PTU5(JL,JK)*ZTENH (JL,JK)*ZFACT5)*ZFACT5 

      ZCAPE5(JL)=ZCAPE5(JL) + &
       & ( ZZBUO5 + RETV*(PQU5(JL,JK)-ZQENH5(JL,JK))&
       & -  PLU5(JL,JK) ) * ZDZ5  
      ZCAPE(JL)=ZCAPE(JL) + &
       & ( (ZZBUO  + RETV*(PQU (JL,JK)-ZQENH (JL,JK))&
       & -  PLU (JL,JK)) * ZDZ5 &
       & + (ZZBUO5 + RETV*(PQU5(JL,JK)-ZQENH5(JL,JK))&
       & -  PLU5(JL,JK)) * ZDZ )  
    ENDIF
  ENDDO
ENDDO

! CFL rescaling for cloud base mass flux
ZRESCAL5(:)=0.0_JPRB
DO JK=2,KLEV
  DO JL=KIDIA,KFDIA
    IF (LDCUM(JL) .AND. (JK >= KCTOP(JL) .AND. JK <= KCBOT(JL))) THEN
      ZMFMAX5=(PAPH5(JL,JK)-PAPH5(JL,JK-1))*ZCONS2*RMFLIC2+RMFLIA2
      ZRESCAL5(JL)=MAX(ZRESCAL5(JL),PMFU5(JL,JK)/ZMFMAX5)
    ENDIF
  ENDDO
ENDDO

DO JL=KIDIA,KFDIA
  IF(LDCUM(JL)) THEN
    IKB=KCBOT(JL)
    ZMFMAX5=1.0_JPRB/ZRESCAL5(JL)
! Note: Perturbation of mass-flux limiter is neglected.
    IF (KTYPE(JL) /= 2) THEN
      IK=KCTOP(JL)
      IF (ZWMEAN5(JL) < 15.0_JPRB) THEN
        ZWM5 = 2.0_JPRB + ZWMEAN5(JL)
        ZWM  = ZWMEAN (JL)
      ELSE
        ZWM5 = 17.0_JPRB
        ZWM  = 0.0_JPRB
      ENDIF
      ZFACT7=RTAU2/(ZWM5*RG)
      ZTAU5=(PGEOH5(JL,IK)-PGEOH5(JL,IKB))*ZFACT7
      ZTAU =((PGEOH (JL,IK)-PGEOH (JL,IKB)) &
         & - (PGEOH5(JL,IK)-PGEOH5(JL,IKB))*ZWM /ZWM5)*ZFACT7
      ZFACT8=1.0_JPRB/(ZHEAT5(JL)*ZTAU5)
      ZMFUB5(JL)= ZCAPE5(JL)*ZFACT8
      ZMFUB (JL)=(ZCAPE (JL)-ZCAPE5(JL) &
              & *(ZHEAT (JL)/ZHEAT5(JL) + ZTAU /ZTAU5))*ZFACT8
    ELSE
      IF (ZDHPBL5(JL) > 0.0_JPRB) THEN
        ZQUMQE5=PQU5(JL,IKB)+PLU5(JL,IKB)-ZQENH5(JL,IKB)
        ZQUMQE =PQU (JL,IKB)+PLU (JL,IKB)-ZQENH (JL,IKB)

        ZDQMIN5=0.01_JPRB*ZQENH5(JL,IKB)
        ZDQMIN =0.01_JPRB*ZQENH (JL,IKB)

        LLO2=(ZDQMIN5 < ZEPS)
        IF (LLO2) THEN
          ZDQMIN5=ZEPS
          ZDQMIN =0.0_JPRB
        ENDIF

        ZDH5=ZCONS3*(PTU5(JL,IKB)-ZTENH5(JL,IKB))+ZCONS4*ZQUMQE5
        ZDH =ZCONS3*(PTU (JL,IKB)-ZTENH (JL,IKB))+ZCONS4*ZQUMQE

        IF (LREGCV) ZDH = ZDH * 0.1_JPRB

        ZDQMIN5=ZCONS1*1.0E5_JPRB*ZDQMIN5
        ZDQMIN =ZCONS1*1.0E5_JPRB*ZDQMIN 

        LLO1=(ZDH5 < ZDQMIN5)
        IF (LLO1) THEN
          ZDH5=ZDQMIN5
          ZDH =ZDQMIN 
        ENDIF
        ZDH =RG*ZDH
        ZDH5=RG*ZDH5

        ZFACT9=1.0_JPRB/ZDH5
        ZMFUB5(JL)=ZDHPBL5(JL)*ZFACT9
        ZMFUB (JL)=(ZDHPBL (JL)-ZDH*ZDHPBL5(JL)*ZFACT9)*ZFACT9

        IF (LREGCV) THEN
          IF (ZDHPBL5(JL)*ZFACT9**2 > 1.E+03_JPRB) ZMFUB(JL)=ZMFUB(JL)*0.2_JPRB
        ENDIF

!EPS: temporary solution
        IF(PTSPHY > 1800._JPRB) THEN
          ZMFMAX5=3.0_JPRB*ZMFMAX5
        ENDIF

      ELSE
        ZMFUB5(JL)=ZMFMAX5*0.1_JPRB
        ZMFUB (JL)=0.0_JPRB
      ENDIF
!LOP Reduce cloud base mass flux for shallow convection
!LLL      ZMFUB5(JL)=ZMFUB5(JL)*0.3_JPRB  
!LLL      ZMFUB (JL)=ZMFUB (JL)*0.3_JPRB      
    ENDIF
    
    IF (ZMFUB5(JL) < 0.001_JPRB) ZMFUB(JL)=0.0_JPRB
    ZMFUB5(JL)=MAX(ZMFUB5(JL),0.001_JPRB)
    IF (ZMFUB5(JL) > ZMFMAX5) ZMFUB(JL)=0.0_JPRB
    ZMFUB5(JL)=MIN(ZMFUB5(JL),ZMFMAX5)
    
! Reduce cloud base mass flux perturbation if required
    IF (LREGCV) THEN
      ZMFUB(JL)=ZMFUB(JL)*0.25_JPRB
    ENDIF
  ELSE
    ZMFUB5(JL)=0.0_JPRB
    ZMFUB (JL)=0.0_JPRB
  ENDIF
ENDDO

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    PMFU (JL,JK)  = ZMFUB5(JL)*PMFU (JL,JK)+ZMFUB(JL)*PMFU5(JL,JK)
    PMFU5(JL,JK)  = ZMFUB5(JL)*PMFU5(JL,JK)
    ZDMFUP (JL,JK)= ZMFUB5(JL)*ZDMFUP (JL,JK)+ZMFUB(JL)*ZDMFUP5(JL,JK)
    ZDMFUP5(JL,JK)= ZMFUB5(JL)*ZDMFUP5(JL,JK)
    PMFD (JL,JK)  = ZMFUB5(JL)*PMFD (JL,JK)+ZMFUB(JL)*PMFD5(JL,JK)
    PMFD5(JL,JK)  = ZMFUB5(JL)*PMFD5(JL,JK)
    ZDMFDP (JL,JK)= ZMFUB5(JL)*ZDMFDP (JL,JK)+ZMFUB(JL)*ZDMFDP5(JL,JK)
    ZDMFDP5(JL,JK)= ZMFUB5(JL)*ZDMFDP5(JL,JK)
    ZMFUDE_RATE (JL,JK)= ZMFUB5(JL)*ZMFUDE_RATE (JL,JK) &
                     & + ZMFUB (JL)*ZMFUDE_RATE5(JL,JK)
    ZMFUDE_RATE5(JL,JK)= ZMFUB5(JL)*ZMFUDE_RATE5(JL,JK)
    ZMFDDE_RATE (JL,JK)= ZMFUB5(JL)*ZMFDDE_RATE (JL,JK) &
                     & + ZMFUB (JL)*ZMFDDE_RATE5(JL,JK)
    ZMFDDE_RATE5(JL,JK)= ZMFUB5(JL)*ZMFDDE_RATE5(JL,JK)
  ENDDO
ENDDO

!-----------------------------------------------------------------------

!*    7.     DETERMINE FINAL CONVECTIVE FLUXES IN 'CUFLX2TL'
!            -----------------------------------------------

CALL CUFLX2TL &
 & ( KIDIA,    KFDIA,    KLON,    KLEV,&
 & PTSPHY,&
 & PTEN5,    PQEN5,    PQSEN5,   ZTENH5,   ZQENH5,&
 & PAPH5,    PAP5,     LDLAND,   LDCUM,&
 & KCBOT,    KCTOP,    IDTOP,    ITOPM2,&
 & KTYPE,    LLDDRAF,&
 & PMFU5,    PMFD5,    ZMFUS5,   ZMFDS5,&
 & ZMFUQ5,   ZMFDQ5,   ZMFUL5,   PLUDE5,&
 & ZDMFUP5,  ZDMFDP5,  ZDPMEL5,  ZLGLAC5,&
 & PMFLXR5,  PMFLXS5,  PRAIN5,&
 & PTEN,     PQEN,     PQSEN,    ZTENH,    ZQENH,&
 & PAPH,     PAP,&
 & PMFU,     PMFD,     ZMFUS,    ZMFDS,&
 & ZMFUQ,    ZMFDQ,    ZMFUL,    PLUDE,&
 & ZDMFUP,   ZDMFDP,   ZDPMEL,   ZLGLAC,&
 & PMFLXR,   PMFLXS,   PRAIN )  


!- rescale DD fluxes if total mass flux becomes negative
!- correct DD detrainment rates if entrainment becomes negative
!- correct UD detrainment rates if entrainment becomes negative
!- conservation correction for precip

ZMFS5(:)=1.0_JPRB
ZMFS (:)=0.0_JPRB
DO JK=2,KLEV
  DO JL=KIDIA,KFDIA
    IF (LLDDRAF(JL) .AND. JK >= IDTOP(JL)-1) THEN
       ZMFMAX5=PMFU5(JL,JK)*0.98_JPRB
       ZMFMAX =PMFU (JL,JK)*0.98_JPRB
       IF(PMFD5(JL,JK)+ZMFMAX5+1.E-15_JPRB < 0.0_JPRB) THEN
         ZFACT10=1.0_JPRB/PMFD5(JL,JK)
         IF (ZMFS5(JL) > -ZMFMAX5*ZFACT10) THEN
           ZMFS5(JL)=-ZMFMAX5*ZFACT10
           ZMFS (JL)=(-ZMFMAX +PMFD (JL,JK)*ZMFMAX5*ZFACT10)*ZFACT10
         ENDIF
       ENDIF
    ENDIF
  ENDDO
ENDDO
ZDMFCOR5(:)=0.0_JPRB
ZDMFCOR (:)=0.0_JPRB
DO JK=2,KLEV
  DO JL=KIDIA,KFDIA
    IF (ZMFS5(JL) < 1.0_JPRB .AND. JK >= IDTOP(JL)-1) THEN
      PMFD (JL,JK)=PMFD (JL,JK)*ZMFS5(JL)+PMFD5(JL,JK)*ZMFS (JL)
      PMFD5(JL,JK)=PMFD5(JL,JK)*ZMFS5(JL)
      ZMFDS (JL,JK)=ZMFDS (JL,JK)*ZMFS5(JL)+ZMFDS5(JL,JK)*ZMFS (JL)
      ZMFDS5(JL,JK)=ZMFDS5(JL,JK)*ZMFS5(JL)
      ZMFDQ (JL,JK)=ZMFDQ (JL,JK)*ZMFS5(JL)+ZMFDQ5(JL,JK)*ZMFS (JL)
      ZMFDQ5(JL,JK)=ZMFDQ5(JL,JK)*ZMFS5(JL)
      ZMFDDE_RATE (JL,JK)=ZMFDDE_RATE (JL,JK)*ZMFS5(JL)&
                       & +ZMFDDE_RATE5(JL,JK)*ZMFS (JL)
      ZMFDDE_RATE5(JL,JK)=ZMFDDE_RATE5(JL,JK)*ZMFS5(JL)
      ZDMFCOR (JL)=ZDMFCOR (JL)-(1.0_JPRB-ZMFS5(JL))*ZDMFDP (JL,JK)&
                & +ZMFS (JL)*ZDMFDP5(JL,JK)
      ZDMFCOR5(JL)=ZDMFCOR5(JL)-(1.0_JPRB-ZMFS5(JL))*ZDMFDP5(JL,JK)
      PMFLXR (JL,JK+1)=PMFLXR (JL,JK+1)+ZDMFCOR (JL)
      PMFLXR5(JL,JK+1)=PMFLXR5(JL,JK+1)+ZDMFCOR5(JL)
      ZDMFDP (JL,JK)=ZDMFDP (JL,JK)*ZMFS5(JL)+ZDMFDP5(JL,JK)*ZMFS (JL)
      ZDMFDP5(JL,JK)=ZDMFDP5(JL,JK)*ZMFS5(JL)
    ENDIF
   !ZDMFUPC5(JL,JK)=ZDMFUP5(JL,JK)
   !ZDMFUPC(JL,JK)=ZDMFUP(JL,JK)
   !ZDMFDPC5(JL,JK)=ZDMFDP5(JL,JK)
   !ZDMFDPC(JL,JK)=ZDMFDP(JL,JK)

  ENDDO
ENDDO

!----------------------------------------------------------------------

!*    8.       UPDATE TENDENCIES FOR T AND Q IN SUBROUTINE CUDTDQN2TL
!              ------------------------------------------------------

IF ( RMFSOLTQ2 > 0.0_JPRB) THEN
! derive draught properties for implicit
  DO JK=KLEV,2,-1
    DO JL=KIDIA,KFDIA
      IF (LDCUM(JL)) THEN
        IF (JK > KCBOT(JL)) THEN
          IF (PMFU5(JL,JK) > 1.E-15_JPRB) THEN
            ZFACT11=1.0_JPRB/PMFU5(JL,JK)
            ZMFA5=ZFACT11
            ZMFA =-PMFU (JL,JK)*ZFACT11*ZFACT11
          ELSE
            ZMFA5=1.E+15_JPRB
            ZMFA =0.0_JPRB
          ENDIF
          PQU5(JL,JK)=ZQENH5(JL,JK)+ZMFUQ5(JL,JK)*ZMFA5
          PQU (JL,JK)=ZQENH (JL,JK)+ZMFUQ5(JL,JK)*ZMFA +ZMFUQ (JL,JK)*ZMFA5
          PTU5(JL,JK)=ZTENH5(JL,JK)+ZMFUS5(JL,JK)*ZMFA5*ZRCPD
          PTU (JL,JK)=ZTENH (JL,JK)&
                   & +(ZMFUS5(JL,JK)*ZMFA +ZMFUS (JL,JK)*ZMFA5)*ZRCPD
          ZMFUS5(JL,JK)=PMFU5(JL,JK)*(RCPD*PTU5(JL,JK)+PGEOH5(JL,JK))
          ZMFUS (JL,JK)=PMFU (JL,JK)*(RCPD*PTU5(JL,JK)+PGEOH5(JL,JK))&
                     & +PMFU5(JL,JK)*(RCPD*PTU (JL,JK)+PGEOH (JL,JK))
          ZMFUQ5(JL,JK)=PMFU5(JL,JK)*PQU5(JL,JK)
          ZMFUQ (JL,JK)=PMFU5(JL,JK)*PQU (JL,JK)+PMFU (JL,JK)*PQU5(JL,JK)
          IF (LLDDRAF(JL)) THEN
            IF (PMFD5(JL,JK) < -1.E-15_JPRB) THEN
              ZFACT11=1.0_JPRB/PMFD5(JL,JK)
              ZMFA5=ZFACT11
              ZMFA =-PMFD (JL,JK)*ZFACT11*ZFACT11
            ELSE
              ZMFA5=-1.E+15_JPRB
              ZMFA =0.0_JPRB
            ENDIF
            ZQD5(JL,JK)=ZQENH5(JL,JK)+ZMFDQ5(JL,JK)*ZMFA5
            ZQD (JL,JK)=ZQENH (JL,JK)+ZMFDQ5(JL,JK)*ZMFA +ZMFDQ (JL,JK)*ZMFA5
            ZTD5(JL,JK)=ZTENH5(JL,JK)+ZMFDS5(JL,JK)*ZMFA5*ZRCPD
            ZTD (JL,JK)=ZTENH (JL,JK)&
                     & +(ZMFDS5(JL,JK)*ZMFA +ZMFDS (JL,JK)*ZMFA5)*ZRCPD
            ZMFDQ5(JL,JK)=PMFD5(JL,JK)*ZQD5(JL,JK)
            ZMFDQ (JL,JK)=PMFD (JL,JK)*ZQD5(JL,JK)+PMFD5(JL,JK)*ZQD (JL,JK)
            ZMFDS5(JL,JK)=PMFD5(JL,JK)*(RCPD*ZTD5(JL,JK)+PGEOH5(JL,JK))
            ZMFDS (JL,JK)=PMFD (JL,JK)*(RCPD*ZTD5(JL,JK)+PGEOH5(JL,JK))&
                       & +PMFD5(JL,JK)*(RCPD*ZTD (JL,JK)+PGEOH (JL,JK))
          ENDIF
        ELSEIF (JK <= KCBOT(JL) .AND. JK >= KCTOP(JL)) THEN
          ZMFUS5(JL,JK)=PMFU5(JL,JK)*(RCPD*PTU5(JL,JK)+PGEOH5(JL,JK))
          ZMFUS (JL,JK)=PMFU (JL,JK)*(RCPD*PTU5(JL,JK)+PGEOH5(JL,JK))&
                     & +PMFU5(JL,JK)*(RCPD*PTU (JL,JK)+PGEOH (JL,JK))
          ZMFUQ5(JL,JK)=PMFU5(JL,JK)*PQU5(JL,JK)
          ZMFUQ (JL,JK)=PMFU5(JL,JK)*PQU (JL,JK)+PMFU (JL,JK)*PQU5(JL,JK)
          ZMFDS5(JL,JK)=PMFD5(JL,JK)*(RCPD*ZTD5(JL,JK)+PGEOH5(JL,JK))
          ZMFDS (JL,JK)=PMFD (JL,JK)*(RCPD*ZTD5(JL,JK)+PGEOH5(JL,JK))&
                     & +PMFD5(JL,JK)*(RCPD*ZTD (JL,JK)+PGEOH (JL,JK))
          ZMFDQ5(JL,JK)=PMFD5(JL,JK)*ZQD5(JL,JK)
          ZMFDQ (JL,JK)=PMFD5(JL,JK)*ZQD (JL,JK)+PMFD (JL,JK)*ZQD5(JL,JK)
        ENDIF
      ENDIF
    ENDDO
  ENDDO
ENDIF

CALL CUDTDQN2TL &
 & ( KIDIA,    KFDIA,    KLON,    KLEV,&
 & ITOPM2,   KCTOP,    IDTOP,&
 & LDRAIN1D, LDCUM,    LLDDRAF,  PTSPHY,&
 & PAPH5,    PGEOH5,   PGEO5,&
 & PTEN5,    ZTENH5,   PQEN5,    ZQENH5,   PQSEN5,&
 & ZLGLAC5,  PLUDE5,   PMFU5,    PMFD5,&
 & ZMFUS5,   ZMFDS5,   ZMFUQ5,   ZMFDQ5,&
 & ZMFUL5,   ZDMFUP5,  ZDMFDP5,  ZDPMEL5,&
 & PTENT5,   PTENQ5,   PENTH5,&
 & PAPH,     PGEOH,    PGEO,&
 & PTEN,     ZTENH,    PQEN,     ZQENH,    PQSEN,&
 & ZLGLAC,   PLUDE,    PMFU,     PMFD,&
 & ZMFUS,    ZMFDS,    ZMFUQ,    ZMFDQ,&
 & ZMFUL,    ZDMFUP,   ZDMFDP,   ZDPMEL,&
 & PTENT,    PTENQ,    PENTH  )  

!----------------------------------------------------------------------

!*    9.0        COMPUTE MOMENTUM IN UPDRAUGHT AND DOWNDRAUGHT
!                ---------------------------------------------

IF(LMFDUDV2) THEN

! Updraught and downdraught wind computations
!--------------------------------------------

  DO JK=KLEV-1,2,-1
    IK=JK+1
    DO JL=KIDIA,KFDIA
      IF(LDCUM(JL)) THEN
        IF (JK == KCBOT(JL) .AND. KTYPE(JL) < 3) THEN
           IKB=ISTUP(JL)
           ZUU5(JL,JK)=PUEN5(JL,IKB-1)
           ZUU (JL,JK)=PUEN (JL,IKB-1)
           ZVU5(JL,JK)=PVEN5(JL,IKB-1)
           ZVU (JL,JK)=PVEN (JL,IKB-1)
        ELSEIF (JK == KCBOT(JL) .AND. KTYPE(JL) == 3) THEN
           ZUU5(JL,JK)=PUEN5(JL,JK-1)
           ZUU (JL,JK)=PUEN (JL,JK-1)
           ZVU5(JL,JK)=PVEN5(JL,JK-1)
           ZVU (JL,JK)=PVEN (JL,JK-1)
        ENDIF
        IF (JK < KCBOT(JL) .AND. JK >= KCTOP(JL)) THEN
          ZUU5(JL,JK)=ZUU5(JL,IK)*(1.0_JPRB-ZERATEU5(JL,JK))&
                   & +PUEN5(JL,JK)*ZERATEU5(JL,JK)
          ZUU (JL,JK)=ZUU (JL,IK)*(1.0_JPRB-ZERATEU5(JL,JK))&
                   & +PUEN (JL,JK)*ZERATEU5(JL,JK) &
                   & +(PUEN5(JL,JK)-ZUU5(JL,IK))*ZERATEU (JL,JK)
          ZVU5(JL,JK)=ZVU5(JL,IK)*(1.0_JPRB-ZERATEU5(JL,JK))&
                   & +PVEN5(JL,JK)*ZERATEU5(JL,JK)
          ZVU (JL,JK)=ZVU (JL,IK)*(1.0_JPRB-ZERATEU5(JL,JK))&
                   & +PVEN (JL,JK)*ZERATEU5(JL,JK) &
                   & +(PVEN5(JL,JK)-ZVU5(JL,IK))*ZERATEU (JL,JK)
        ENDIF
      ENDIF
    ENDDO
  ENDDO
  DO JK=3,KLEV
    IK=JK-1
    DO JL=KIDIA,KFDIA
      IF(LDCUM(JL) .AND. LLDDRAF(JL)) THEN
        IF (JK == IDTOP(JL)) THEN
          ZUD5(JL,JK)=0.5_JPRB*(ZUU5(JL,JK)+PUEN5(JL,IK))
          ZUD (JL,JK)=0.5_JPRB*(ZUU (JL,JK)+PUEN (JL,IK))
          ZVD5(JL,JK)=0.5_JPRB*(ZVU5(JL,JK)+PVEN5(JL,IK))
          ZVD (JL,JK)=0.5_JPRB*(ZVU (JL,JK)+PVEN (JL,IK))
        ELSEIF (JK > IDTOP(JL)) THEN
          ZUD5(JL,JK)=ZUD5(JL,IK)*(1.0_JPRB-ZERATED5(JL,JK))&
                   & +PUEN5(JL,IK)*ZERATED5(JL,JK)
          ZUD (JL,JK)=ZUD (JL,IK)*(1.0_JPRB-ZERATED5(JL,JK))&
                   & +PUEN (JL,IK)*ZERATED5(JL,JK) &
                   & +(PUEN5(JL,IK)-ZUD5(JL,IK))*ZERATED (JL,JK)
          ZVD5(JL,JK)=ZVD5(JL,IK)*(1.0_JPRB-ZERATED5(JL,JK))&
                   & +PVEN5(JL,IK)*ZERATED5(JL,JK)
          ZVD (JL,JK)=ZVD (JL,IK)*(1.0_JPRB-ZERATED5(JL,JK))&
                   & +PVEN (JL,IK)*ZERATED5(JL,JK) &
                   & +(PVEN5(JL,IK)-ZVD5(JL,IK))*ZERATED (JL,JK)
        ENDIF
      ENDIF
    ENDDO
  ENDDO

!*    9.1        UPDATE TENDENCIES FOR U AND V IN SUBROUTINE CUDUDV2TL
!                -----------------------------------------------------

! Rescale mass fluxes for stability in momentum
!----------------------------------------------

  DO JL=KIDIA,KFDIA
    ZMFS5(JL)=1.0_JPRB
    ZMFS (JL)=0.0_JPRB
  ENDDO

  IF (RMFSOLUV2 <= 1.0_JPRB) THEN
    DO JK=2,KLEV
      DO JL=KIDIA,KFDIA
        IF(LDCUM(JL) .AND. JK >= KCTOP(JL)-1) THEN
          ZMFMAX5=(PAPH5(JL,JK)-PAPH5(JL,JK-1))*ZCONS
          ZMFMAX =(PAPH (JL,JK)-PAPH (JL,JK-1))*ZCONS
          IF (PMFU5(JL,JK) > ZMFMAX5 .AND. JK >= KCTOP(JL)) THEN 
            ZFACT10=1.0_JPRB/PMFU5(JL,JK)
            IF (ZMFMAX5*ZFACT10 < ZMFS5(JL)) THEN
              ZMFS(JL)=(ZMFMAX -PMFU (JL,JK)*ZMFMAX5*ZFACT10)*ZFACT10
            ENDIF
            ZMFS5(JL)=MIN(ZMFS5(JL),ZMFMAX5*ZFACT10)
          ENDIF
        ENDIF
      ENDDO
    ENDDO
  ENDIF

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      ZMFUUS5(JL,JK)=PMFU5(JL,JK)
      ZMFUUS (JL,JK)=PMFU (JL,JK)
      ZMFDUS5(JL,JK)=PMFD5(JL,JK)
      ZMFDUS (JL,JK)=PMFD (JL,JK)
      IF (LDCUM(JL) .AND. JK >= KCTOP(JL)-1) THEN
        ZMFUUS5(JL,JK)=PMFU5(JL,JK)*ZMFS5(JL)
        ZMFUUS (JL,JK)=PMFU (JL,JK)*ZMFS5(JL)+PMFU5(JL,JK)*ZMFS (JL)
        ZMFDUS5(JL,JK)=PMFD5(JL,JK)*ZMFS5(JL)
        ZMFDUS (JL,JK)=PMFD (JL,JK)*ZMFS5(JL)+PMFD5(JL,JK)*ZMFS (JL)
      ENDIF
    ENDDO
  ENDDO

! Recompute draught properties below for implicit
! based on linear flux profiles

  IF (RMFSOLUV2 > 0.0_JPRB) THEN

    DO JL=KIDIA,KFDIA
      IF (LDCUM(JL)) THEN
        JK=KCBOT(JL)
        IK=JK-1
        ZMFUUB5(JL)=ZMFUUS5(JL,JK)*(ZUU5(JL,JK)-PUEN5(JL,IK))
        ZMFUUB (JL)=ZMFUUS5(JL,JK)*(ZUU (JL,JK)-PUEN (JL,IK))&
                 & +ZMFUUS (JL,JK)*(ZUU5(JL,JK)-PUEN5(JL,IK))
        ZMFUVB5(JL)=ZMFUUS5(JL,JK)*(ZVU5(JL,JK)-PVEN5(JL,IK))
        ZMFUVB (JL)=ZMFUUS5(JL,JK)*(ZVU (JL,JK)-PVEN (JL,IK))&
                 & +ZMFUUS (JL,JK)*(ZVU5(JL,JK)-PVEN5(JL,IK))
      ENDIF
    ENDDO

    DO JK=2,KLEV
      IK=JK-1
      DO JL=KIDIA,KFDIA
        IF (LDCUM(JL) .AND. JK > KCBOT(JL)) THEN
          IKB=KCBOT(JL)
          ZFACT12=1.0_JPRB/(PAPH5(JL,KLEV+1)-PAPH5(JL,IKB))
          ZDZ5=(PAPH5(JL,KLEV+1)-PAPH5(JL,JK))*ZFACT12
          ZDZ =( (PAPH (JL,KLEV+1)-PAPH (JL,JK)) &
            & -  (PAPH (JL,KLEV+1)-PAPH (JL,IKB))*ZDZ5 )*ZFACT12
          IF (KTYPE(JL) == 3) THEN
            ZDZ =2.0_JPRB*ZDZ5*ZDZ
            ZDZ5=ZDZ5*ZDZ5
          ENDIF
          IF (ZMFUUS5(JL,JK) > RMFCMIN2) THEN
            ZFACT13=1.0_JPRB/ZMFUUS5(JL,JK)
            ZMFA5=ZFACT13
            ZMFA =-ZMFUUS (JL,JK)*ZFACT13*ZFACT13
          ELSE
            ZMFA5=ZRMFCMIN
            ZMFA =0.0_JPRB
          ENDIF

          ZUU5(JL,JK)=PUEN5(JL,IK)+ZMFUUB5(JL)*ZDZ5*ZMFA5
          ZUU (JL,JK)=PUEN (JL,IK)+ZMFUUB (JL)*ZDZ5*ZMFA5 &
                   &              +ZMFUUB5(JL)*ZDZ *ZMFA5 &
                   &              +ZMFUUB5(JL)*ZDZ5*ZMFA

          ZVU5(JL,JK)=PVEN5(JL,IK)+ZMFUVB5(JL)*ZDZ5*ZMFA5
          ZVU (JL,JK)=PVEN (JL,IK)+ZMFUVB (JL)*ZDZ5*ZMFA5 &
                   &              +ZMFUVB5(JL)*ZDZ *ZMFA5 &
                   &              +ZMFUVB5(JL)*ZDZ5*ZMFA

          ZMFDUS5(JL,JK)=ZMFDUS5(JL,IKB)*ZDZ5
          ZMFDUS (JL,JK)=ZMFDUS5(JL,IKB)*ZDZ +ZMFDUS (JL,IKB)*ZDZ5

          ZUD5(JL,JK)=PUEN5(JL,IK)+ZUD5(JL,IKB)-PUEN5(JL,IKB-1)
          ZUD (JL,JK)=PUEN (JL,IK)+ZUD (JL,IKB)-PUEN (JL,IKB-1)

          ZVD5(JL,JK)=PVEN5(JL,IK)+ZVD5(JL,IKB)-PVEN5(JL,IKB-1)
          ZVD (JL,JK)=PVEN (JL,IK)+ZVD (JL,IKB)-PVEN (JL,IKB-1)
        ENDIF
! Add UV perturb to correct wind bias
        IF (LDCUM(JL) .AND. JK >= KCTOP(JL)) THEN
          ZUU5(JL,JK)=ZUU5(JL,JK)-RUVPER2*SIGN(1.0_JPRB,ZUU5(JL,JK))
          ZVU5(JL,JK)=ZVU5(JL,JK)-RUVPER2*SIGN(1.0_JPRB,ZVU5(JL,JK))
        ENDIF
      ENDDO
    ENDDO

  ENDIF

!-------------------------------------------------------------------

  CALL CUDUDV2TL &
   & ( KIDIA,    KFDIA,    KLON,    KLEV,&
   &   ITOPM2,   KTYPE,    KCBOT,    KCTOP,    LDCUM,    PTSPHY,&
   &   PAPH5,    PUEN5,    PVEN5,    ZMFUUS5,  ZMFDUS5,&
   &   ZUU5,     ZUD5,     ZVU5,     ZVD5,&
   &   PTENU5,   PTENV5,&
   &   PAPH,     PUEN,     PVEN,     ZMFUUS,   ZMFDUS,&
   &   ZUU,      ZUD,      ZVU,      ZVD,&
   &   PTENU,    PTENV     )  

  IF(LMFUVDIS2) THEN
! add KE dissipation
  ENDIF

ENDIF

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF (.NOT.LDCUM(JL)) THEN
      PLUDE5(JL,JK)=0.0_JPRB
      PLUDE (JL,JK)=0.0_JPRB
      PLU5(JL,JK)=0.0_JPRB
      PLU (JL,JK)=0.0_JPRB
    ENDIF
  ENDDO
ENDDO

!----------------------------------------------------------------------

!*   11.0          CHEMICAL TRACER TRANSPORT
!                  -------------------------

IF ( LMFTRAC .AND. KTRAC > 0 ) THEN

  ALLOCATE(ZMFUUC5(KLON,KLEV))
  ALLOCATE(ZMFDUC5(KLON,KLEV))
  ALLOCATE(ZMFUDR5(KLON,KLEV))
  ALLOCATE(ZMFDDR5(KLON,KLEV))

  ALLOCATE(ZMFUUC(KLON,KLEV))
  ALLOCATE(ZMFDUC(KLON,KLEV))
  ALLOCATE(ZMFUDR(KLON,KLEV))
  ALLOCATE(ZMFDDR(KLON,KLEV))

! transport switched off for mid-level convection
  DO JL=KIDIA,KFDIA
    IF (LDCUM(JL) .AND. KTYPE(JL)/=3 .AND. KCBOT(JL)-KCTOP(JL)>= 1) THEN
      LLDCUM(JL)=.TRUE.
      LLDDRAF3(JL)=LLDDRAF(JL)
    ELSE
      LLDCUM(JL)=.FALSE.
      LLDDRAF3(JL)=.FALSE.
    ENDIF
  ENDDO
  
! check and correct mass fluxes for CFL criterion

  ZMFS5(:)=1.0_JPRB
  ZMFS (:)=0.0_JPRB

  IF (RMFSOLCT2 <= 3.0_JPRB) THEN
    DO JK=2,KLEV
      DO JL=KIDIA,KFDIA
        IF (LLDCUM(JL) .AND. JK >= KCTOP(JL)) THEN
          ZMFMAX5=(PAPH5(JL,JK)-PAPH5(JL,JK-1))*0.8_JPRB*ZCONS
!LOP No perturbation of mass flux limiter
          IF(PMFU5(JL,JK) > ZMFMAX5) THEN
            IF (ZMFS5(JL) > ZMFMAX5/PMFU5(JL,JK)) THEN
              ZMFS5(JL)=ZMFMAX5/PMFU5(JL,JK)
              ZMFS (JL)=-PMFU (JL,JK)*ZMFMAX5/(PMFU5(JL,JK)*PMFU5(JL,JK))
            ENDIF
          ENDIF
        ENDIF
      ENDDO
    ENDDO
  ENDIF

  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      IF (LLDCUM(JL) .AND. JK >= KCTOP(JL)-1) THEN
        ZMFUUC5(JL,JK)=PMFU5(JL,JK)*ZMFS5(JL)
        ZMFUUC (JL,JK)=PMFU5(JL,JK)*ZMFS (JL)+PMFU (JL,JK)*ZMFS5(JL)

        ZMFUDR5(JL,JK)=ZMFUDE_RATE5(JL,JK)*ZMFS5(JL)
        ZMFUDR (JL,JK)=ZMFUDE_RATE5(JL,JK)*ZMFS (JL) &
                    & +ZMFUDE_RATE (JL,JK)*ZMFS5(JL)

        ZDMFUPC5(JL,JK)=(PMFLXR5(JL,JK)+PMFLXS5(JL,JK))*ZMFS5(JL)
        ZDMFUPC (JL,JK)=(PMFLXR5(JL,JK)+PMFLXS5(JL,JK))*ZMFS (JL)+&
                       &(PMFLXR (JL,JK)+PMFLXS (JL,JK))*ZMFS5(JL)
        ikb=kcbot(jl)
        if (jk>ikb) then
          zdmfupc5(jl,jk)=zdmfupc5(jl,ikb)
          zdmfupc (jl,jk)=zdmfupc (jl,ikb)
        endif

      ELSE
        ZMFUUC5(JL,JK)=0.0_JPRB
        ZMFUUC (JL,JK)=0.0_JPRB

        ZMFUDR5(JL,JK)=0.0_JPRB
        ZMFUDR (JL,JK)=0.0_JPRB
      ENDIF
      IF (LLDDRAF3(JL) .AND. JK >= IDTOP(JL)-1) THEN
        ZMFDUC5(JL,JK)=PMFD5(JL,JK)*ZMFS5(JL)
        ZMFDUC (JL,JK)=PMFD5(JL,JK)*ZMFS (JL)+PMFD (JL,JK)*ZMFS5(JL)

        ZMFDDR5(JL,JK)=ZMFDDE_RATE5(JL,JK)*ZMFS5(JL)
        ZMFDDR (JL,JK)=ZMFDDE_RATE5(JL,JK)*ZMFS (JL) &
                    & +ZMFDDE_RATE (JL,JK)*ZMFS5(JL)

        ZDMFDPC5(JL,JK)=0.0_JPRB
        ZDMFDPC (JL,JK)=0.0_JPRB

      ELSE
        ZMFDUC5(JL,JK)=0.0_JPRB
        ZMFDUC (JL,JK)=0.0_JPRB

        ZMFDDR5(JL,JK)=0.0_JPRB
        ZMFDDR (JL,JK)=0.0_JPRB
      ENDIF
    ENDDO
  ENDDO

  IF(LMFSCAV) THEN
    ZSCAV5(:)=PSCAV5(:)
  ELSE
    ZSCAV5(:)=0.0_JPRB
    ZDMFUPC5(:,:)=0.0_JPRB
    ZDMFUPC (:,:)=0.0_JPRB
  ENDIF

  CALL CUCTRACERTL &
   & ( KIDIA,    KFDIA,    KLON,     KLEV,     KTRAC,&
   &   KCTOP,    IDTOP,&
   &   LLDCUM,   LLDDRAF3, PTSPHY,&
   &   PAPH5,    ZMFUUC5,  ZMFDUC5,  ZMFUDR5,  ZMFDDR5,&
   &   ZDMFUPC5, ZDMFDPC5,&
   &   PCEN5,    PTENC5,   ZSCAV5,&
   &   PAPH,     ZMFUUC,   ZMFDUC,   ZMFUDR,   ZMFDDR,&
   &   ZDMFUPC,  ZDMFDPC,&
   &   PCEN,     PTENC )

  DEALLOCATE(ZMFDDR)
  DEALLOCATE(ZMFUDR)
  DEALLOCATE(ZMFDUC)
  DEALLOCATE(ZMFUUC)

  DEALLOCATE(ZMFDDR5)
  DEALLOCATE(ZMFUDR5)
  DEALLOCATE(ZMFDUC5)
  DEALLOCATE(ZMFUUC5)

ENDIF

IF (LHOOK) CALL DR_HOOK('CUMASTRN2TL',1,ZHOOK_HANDLE)
END SUBROUTINE CUMASTRN2TL
