SUBROUTINE CUPDRATL &
 & ( KIDIA,    KFDIA,    KLON,     KLEV,&
 & K_JKSTART,  LD_LLGO_ON,&
 & PQENH5,   PGEOH5,   PAPH5,    PGEO5,&
 & PTU5,     PQU5,     PLU5,&
 & PSENH5,   PWU2H5,   PSUH5,&
 & PBUOH5,   PLGLAC5,  PQPRCV5,&
 & KLAB,     LDCUM,    LDSC,     KCBOT,&
 & KBOTSC,   KCTOP,    KSTUP,&
 & PQENH,    PGEOH,    PAPH,     PGEO,&
 & PTU,      PQU,      PLU,&
 & PSENH,    PWU2H,    PSUH,&
 & PBUOH,    PLGLAC,   PQPRCV )  

!          THIS ROUTINE CALCULATES CHARACTERISTICS OF 
!          THE CUMULUS UPDRAFTS, CLOUD BASE HEIGHT AND 
!          CLOUD TOP HEIGHT (TANGENT LINEAR VERSION).

!          Ph Lopez (ECMWF) (05/2002) 
!          inspired from A. Pier Siebesma (KNMI)
!          and C. Jakob (ECMWF) (01/2001)

!          PURPOSE.
!          --------
!          TO PRODUCE UPDRAFT CALCULATION, CLOUD BASE AND CLOUD 
!          TOP VALUES FOR SIMPLIFIED CONVECTIVE PARAMETRIZATION

!          INTERFACE
!          ---------
!          THIS ROUTINE IS CALLED FROM *CUBASEN2TL*.
!          INPUT ARE ENVIRONM. VALUES OF T,Q,P,PHI AT HALF LEVELS.
!          IT RETURNS CLOUD FIELDS VALUES AND FLAGS AS FOLLOWS;
!                 KLAB=0 FOR STABLE LAYERS
!                 KLAB=1 FOR SUBCLOUD LEVELS
!                 KLAB=2 FOR CLOUD LEVELS LEVEL

!          METHOD.
!          -------
!          LIFT SURFACE AIR DRY-ADIABATICALLY TO CLOUD TOP.
!          ENTRAINING PLUME WITH ENTRAINMENT PROPORTIONAL TO 1/Z 
!          FOR SHALLOW CONVECTION (STARTING FROM LOWEST MODEL LEVEL)
!          AND FIXED OTHERWISE. 

!     PARAMETER     DESCRIPTION                                   UNITS
!     ---------     -----------                                   -----
!     INPUT PARAMETERS (INTEGER):

!    *KIDIA*        START POINT
!    *KFDIA*        END POINT
!    *KLON*         NUMBER OF GRID POINTS PER PACKET
!    *KLEV*         NUMBER OF LEVELS

!    INPUT PARAMETERS (REAL):

!    *PQENH5*       ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS      (Trajectory)
!    *PGEOH5*       GEOPOTENTIAL ON HALF LEVELS                   (Trajectory)
!    *PAPH5*        PROVISIONAL PRESSURE ON HALF LEVELS           (Trajectory)
!    *PGEO5*        GEOPOTENTIAL                                  (Trajectory)

!    *PQENH*        ENV. SPEC. HUMIDITY (T+1) ON HALF LEVELS      KG/KG
!    *PGEOH*        GEOPOTENTIAL ON HALF LEVELS                   M2/S2
!    *PAPH*         PROVISIONAL PRESSURE ON HALF LEVELS           PA
!    *PGEO*         GEOPOTENTIAL                                  M2/S2

!    UPDATED PARAMETERS (REAL):

!    *PTU5*         TEMPERATURE IN UPDRAFTS                       (Trajectory)
!    *PQU5*         SPEC. HUMIDITY IN UPDRAFTS                    (Trajectory)
!    *PLU5*         LIQUID WATER CONTENT IN UPDRAFTS              (Trajectory)
!    *PBUOH5*       BUOYANCY ON HALF LEVELS                       (Trajectory)

!    *PTU*          TEMPERATURE IN UPDRAFTS                        K
!    *PQU*          SPEC. HUMIDITY IN UPDRAFTS                     KG/KG
!    *PLU*          LIQUID WATER CONTENT IN UPDRAFTS               KG/KG
!    *PBUOH*        BUOYANCY ON HALF LEVELS                        M/S2

!    UPDATED PARAMETERS (INTEGER):

!    *KLAB*         FLAG KLAB=1 FOR SUBCLOUD LEVELS
!                        KLAB=2 FOR CLOUD LEVELS

!    OUTPUT PARAMETERS (LOGICAL):

!    *LDCUM*        FLAG: .TRUE. FOR CONVECTIVE POINTS 
!    *LDSC*         FLAG: .TRUE. IF BL-CLOUDS EXIST

!    OUTPUT PARAMETERS (INTEGER):

!    *KSTUP*       LEVEL AT WHICH THE UPDRAFT STARTS 
!    *KCBOT*       CLOUD BASE LEVEL     
!    *KCTOP*       CLOUD TOP LEVEL = HEIGHEST HALF LEVEL 
!                  WITH A NON-ZERO CLOUD UPDRAFT.
!    *KBOTSC*      CLOUD BASE LEVEL OF BL-CLOUDS

!    OUTPUT PARAMETERS (REAL):

!    *PLGLAC5*     FROZEN CLOUD WATER CONTENT                   (Trajectory)
!    *PQPRCV5*     CONVECTIVE PRECIPITATION CONTENT             (Trajectory)

!    *PLGLAC*      FROZEN CLOUD WATER CONTENT                   KG/KG 
!    *PQPRCV*      CONVECTIVE PRECIPITATION CONTENT             KG/KG 

!    EXTERNALS
!    ---------
!    *CUADJTQSTL* FOR ADJUSTING T AND Q DUE TO CONDENSATION IN ASCENT

!    MODIFICATIONS
!    -------------
!      P.Lopez    01-Oct-2005  New version of linearized convection
!      P.Lopez    21-Mar-2007  Modified regularization of ZWU for SVs
!      P.Lopez    04-Sep-2007  Revised version to improve match to Tiedtke scheme

!----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM,    JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY :  RETV,   RD,    RG,  RCPD,&
                     & RLVTT,  RLSTT, RTT  
USE YOECUMF2 , ONLY : ENTRPEN2, NJKT22
USE YOECLDP  , ONLY : RLMIN
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RALFDCP  ,RTWAT    ,RTICE    ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R
USE YOMCUMFS , ONLY :  LREGCV 

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: K_JKSTART 
LOGICAL           ,INTENT(INOUT) :: LD_LLGO_ON(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQENH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEO5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSENH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWU2H5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSUH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBUOH5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLGLAC5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQPRCV5(KLON,KLEV) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLAB(KLON,KLEV) 
LOGICAL           ,INTENT(INOUT) :: LDCUM(KLON) 
LOGICAL           ,INTENT(INOUT) :: LDSC(KLON) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KCBOT(KLON) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KBOTSC(KLON) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KCTOP(KLON) 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KSTUP(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQENH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEO(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSENH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWU2H(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSUH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBUOH(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PLGLAC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQPRCV(KLON,KLEV) 

!             LOCAL STORAGE
!             ----- -------

INTEGER(KIND=JPIM) :: ICALL, IK, JOK, JK, JL, JKT2

REAL(KIND=JPRB) :: ZQOLD5(KLON),ZPH5(KLON)
REAL(KIND=JPRB) :: ZMIX5(KLON),ZDMIX5(KLON)
REAL(KIND=JPRB) :: ZDZ5(KLON),ZCBASE5(KLON)

REAL(KIND=JPRB) :: ZQOLD(KLON),ZPH(KLON)
REAL(KIND=JPRB) :: ZMIX(KLON)       
REAL(KIND=JPRB) :: ZDZ(KLON),ZCBASE(KLON)

REAL(KIND=JPRB) :: ZBUOF, ZLUOLD, ZCOND, ZLNEW, ZLMAX
REAL(KIND=JPRB) :: ZEPS      ! FRACTIONAL ENTRAINMENT RATE   [M^-1]
REAL(KIND=JPRB) :: ZTVENH    ! ENVIRONMENT VIRTUAL TEMPERATURE AT HALF LEVELS (K)  
REAL(KIND=JPRB) :: ZTVUH     ! UPDRAFT VIRTUAL TEMPERATURE AT HALF LEVELS     (K)
REAL(KIND=JPRB) :: ZQSU, ZCOR, ZDQ, ZFACI, ZFACW, ZFAC, &
                 & ZESDP,ZDQSDT,ZDTDP,ZDP,ZPDIFFTOP,ZPDIFFBOT,&
                 & ZSF,ZQF,ZAW,ZBW,ZALFAW  

REAL(KIND=JPRB) :: ZBUOF5, ZLUOLD5, ZFACI5, ZFACW5, ZFAC5
REAL(KIND=JPRB) :: ZCOND5, ZLNEW5, ZLMAX5
REAL(KIND=JPRB) :: ZEPS5, ZTVENH5, ZTVUH5, ZQSU5, ZCOR5, ZDQ5,&
                 & ZALFAW5,ZESDP5,ZDQSDT5,ZDTDP5,ZDP5,ZPDIFFTOP5,ZPDIFFBOT5,&
                 & ZSF5,ZQF5,ZDMIX25,Z1DPH5,ZFACT1,ZFACT3
REAL(KIND=JPRB) :: ZRG, ZRCPD

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "cuadjtqstl.intfb.h"

!DIR$ VFUNCTION EXPHF
#include "fcttre.h"
#include "fcttretl.h"

!----------------------------------------------------------------------
!     0.           INITIALIZE CONSTANTS AND FIELDS
!----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('CUPDRATL',0,ZHOOK_HANDLE)

ZAW=1.0_JPRB
ZBW=1.0_JPRB
ZRG=1.0_JPRB/RG
ZRCPD=1.0_JPRB/RCPD

JKT2=NJKT22

!----------------------------------------------------------------------

!     1.0          DO ASCENT IN SUBCLOUD AND LAYER,
!                  CHECK FOR EXISTENCE OF CONDENSATION LEVEL,
!                  ADJUST T,Q AND L ACCORDINGLY IN *CUADJTQSTL*,
!                  CHECK FOR BUOYANCY AND SET FLAGS
!                  --------------------------------------------
!       ------------------------------------------------------------
!        1.1  DO THE VERTICAL ASCENT UNTIL VELOCITY BECOMES NEGATIVE
!       ------------------------------------------------------------

DO JK=K_JKSTART,JKT2,-1
  JOK=0
  DO JL=KIDIA,KFDIA
    IF (LD_LLGO_ON(JL)) THEN
      JOK=JOK+1
      ZDZ5(JL)= (PGEOH5(JL,JK) - PGEOH5(JL,JK+1))*ZRG
      ZDZ(JL) = (PGEOH (JL,JK) - PGEOH (JL,JK+1))*ZRG

      IF (K_JKSTART == KLEV-1) THEN
        ZFACT1 = 0.55_JPRB/(PGEO5(JL,JK)*ZRG + ZDZ5(JL))**2
        ZEPS5 =  0.55_JPRB/(PGEO5(JL,JK)*ZRG + ZDZ5(JL)) + 1.E-04_JPRB
        ZEPS  = -ZFACT1*(PGEO (JL,JK)*ZRG + ZDZ (JL)) 

        ZMIX5(JL)= 0.5_JPRB*ZDZ5(JL)*ZEPS5
        ZMIX(JL)= 0.5_JPRB*(ZDZ5(JL)*ZEPS+ZDZ(JL)*ZEPS5)
        ZDMIX5(JL)=1.0_JPRB/(1.0_JPRB+ZMIX5(JL))
  
        ZQF5= (PQENH5(JL,JK+1) + PQENH5(JL,JK))*0.5_JPRB
        ZQF = (PQENH (JL,JK+1) + PQENH (JL,JK))*0.5_JPRB

        ZSF5= (PSENH5(JL,JK+1) + PSENH5(JL,JK))*0.5_JPRB
        ZSF = (PSENH (JL,JK+1) + PSENH (JL,JK))*0.5_JPRB
  
        PQU5(JL,JK)= (PQU5(JL,JK+1)*(1.0_JPRB-ZMIX5(JL))&
         & + 2.0_JPRB*ZQF5*ZMIX5(JL))*ZDMIX5(JL)  
        PQU(JL,JK)=( PQU(JL,JK+1)*(1.0_JPRB-ZMIX5(JL)) &
         & -  PQU5(JL,JK+1)*ZMIX(JL) &
         & +  2.0_JPRB*(ZQF*ZMIX5(JL)+ZQF5*ZMIX(JL)) &
         & - PQU5(JL,JK)*ZMIX(JL) ) * ZDMIX5(JL)  
 
        PSUH5(JL,JK)= (PSUH5(JL,JK+1)*(1.0_JPRB-ZMIX5(JL))&
         & + 2.0_JPRB*ZSF5*ZMIX5(JL))*ZDMIX5(JL)  
        PSUH(JL,JK)=( PSUH(JL,JK+1)*(1.0_JPRB-ZMIX5(JL)) &
         & -  PSUH5(JL,JK+1)*ZMIX(JL) &
         & +  2.0_JPRB*(ZSF*ZMIX5(JL)+ZSF5*ZMIX(JL)) &
         & - PSUH5(JL,JK)*ZMIX(JL) ) * ZDMIX5(JL)   
      ELSE
        ZEPS5 = 2.0_JPRB*ENTRPEN2*(PAPH5(JL,JK)/PAPH5(JL,KLEV+1))**3
        ZEPS  = 3.0_JPRB*ZEPS5*(PAPH (JL,JK)/PAPH5(JL,JK)-PAPH (JL,KLEV+1)/PAPH5(JL,KLEV+1))
        
        ZMIX5(JL)= ZDZ5(JL)*ZEPS5
        ZMIX (JL)= ZDZ5(JL)*ZEPS + ZDZ (JL)*ZEPS5

        ZQF5 = (PQENH5(JL,JK+1) + PQENH5(JL,JK))*0.5_JPRB
        ZQF  = (PQENH (JL,JK+1) + PQENH (JL,JK))*0.5_JPRB

        ZSF5 = (PSENH5(JL,JK+1) + PSENH5(JL,JK))*0.5_JPRB
        ZSF  = (PSENH (JL,JK+1) + PSENH (JL,JK))*0.5_JPRB

        PQU5(JL,JK) = PQU5(JL,JK+1) *(1.0_JPRB-ZMIX5(JL)) + ZQF5 * ZMIX5(JL)
        PQU (JL,JK) = PQU (JL,JK+1) *(1.0_JPRB-ZMIX5(JL)) + ZQF  * ZMIX5(JL) &
                  & + ZMIX (JL) * (ZQF5 - PQU5(JL,JK+1))

        PSUH5(JL,JK) = PSUH5(JL,JK+1) *(1.0_JPRB-ZMIX5(JL)) + ZSF5 * ZMIX5(JL) 
        PSUH (JL,JK) = PSUH (JL,JK+1) *(1.0_JPRB-ZMIX5(JL)) + ZSF  * ZMIX5(JL) &
                   & + ZMIX (JL) * (ZSF5 - PSUH5(JL,JK+1))
      ENDIF
      
      ZQOLD5(JL)  = PQU5(JL,JK)
      ZQOLD(JL)   = PQU(JL,JK)
      PTU5(JL,JK) = (PSUH5(JL,JK)-PGEOH5(JL,JK))*ZRCPD
      PTU (JL,JK) = (PSUH(JL,JK)-PGEOH(JL,JK))*ZRCPD
      ZPH5 (JL)   = PAPH5(JL,JK)
      ZPH  (JL)   = PAPH(JL,JK)
    ENDIF
  ENDDO
  
  IK=JK
  ICALL=1
  
  CALL CUADJTQSTL &
   & ( KIDIA,    KFDIA,    KLON,    KLEV,&
   & IK,&
   & ZPH5,     PTU5,     PQU5,&
   & ZPH,      PTU,      PQU,      LD_LLGO_ON,   ICALL)  

!DIR$ IVDEP
!OCL NOVREC

  DO JL=KIDIA,KFDIA
    IF(LD_LLGO_ON(JL)) THEN

! condensation 

      ZCOND5=MAX(ZQOLD5(JL)-PQU5(JL,JK),0.0_JPRB)
      IF (ZQOLD5(JL)-PQU5(JL,JK) > 0.0_JPRB) THEN
        ZCOND=ZQOLD(JL)-PQU(JL,JK)
      ELSE
        ZCOND=0.0_JPRB
      ENDIF

! microphysics

      ZLUOLD5=PLU5(JL,JK+1)
      ZLUOLD=PLU(JL,JK+1)
      ZLMAX5=MAX(ZLUOLD5+ZCOND5,0.0_JPRB)
      IF (ZLUOLD5+ZCOND5 > 0.0_JPRB) THEN
        ZLMAX=ZLUOLD+ZCOND
      ELSE
        ZLMAX=0.0_JPRB
      ENDIF

! Simple microphysics at this stage (to mimic Tiedtke scheme)

      IF (K_JKSTART == KLEV-1) THEN  ! No precip for shallow convection
        IF (ZLMAX5 < 5.E-3_JPRB) THEN
          ZLNEW5=ZLMAX5
          ZLNEW =ZLMAX
        ELSE
          ZLNEW5=5.E-3_JPRB
          ZLNEW =0.0_JPRB
        ENDIF
      ELSE 
        ZLNEW5=0.5_JPRB*ZLMAX5
        ZLNEW =0.5_JPRB*ZLMAX
      ENDIF

      PQPRCV5(JL,JK)=MAX(0.0_JPRB,ZLMAX5-ZLNEW5)
      IF (ZLMAX5-ZLNEW5 > 0.0_JPRB) THEN
        PQPRCV (JL,JK)=ZLMAX -ZLNEW
      ELSE
        PQPRCV (JL,JK)=0.0_JPRB
      ENDIF
      PLU5(JL,JK)=ZLNEW5
      PLU (JL,JK)=ZLNEW

! frozen water

      PLGLAC5(JL,JK)=ZCOND5*((1.0_JPRB-FOEALFCU(PTU5(JL,JK))) &
       & -(1.0_JPRB-FOEALFCU(PTU5(JL,JK+1))))  
      PLGLAC(JL,JK)=ZCOND5*(-FOEALFCUTL(PTU(JL,JK),PTU5(JL,JK)) &
       & +FOEALFCUTL(PTU(JL,JK+1),PTU5(JL,JK+1))) &
       & +ZCOND*((1.0_JPRB-FOEALFCU(PTU5(JL,JK))) &
       & -(1.0_JPRB-FOEALFCU(PTU5(JL,JK+1)))) 
                  
! update dry static energy after condensation + freezing

      PSUH5(JL,JK) = RCPD*(PTU5(JL,JK)+RALFDCP*PLGLAC5(JL,JK))+PGEOH5(JL,JK)
      PSUH(JL,JK)  = RCPD*(PTU (JL,JK)+RALFDCP*PLGLAC (JL,JK))+PGEOH (JL,JK)
      
! Buoyancy on half and full levels
         
      ZTVUH5 = (1.0_JPRB+RETV*PQU5(JL,JK)-PLU5(JL,JK))*PTU5(JL,JK)&
       & + RALFDCP*PLGLAC5(JL,JK)  
      ZTVUH  = (RETV*PQU(JL,JK)-PLU(JL,JK))*PTU5(JL,JK)&
       & + (1.0_JPRB+RETV*PQU5(JL,JK)-PLU5(JL,JK))*PTU(JL,JK)&
       & + RALFDCP*PLGLAC(JL,JK)  
      ZTVENH5 = (1.0_JPRB+RETV*PQENH5(JL,JK)) &
       & * (PSENH5(JL,JK)-PGEOH5(JL,JK))*ZRCPD  
      ZTVENH  = ((1.0_JPRB+RETV*PQENH5(JL,JK))*(PSENH(JL,JK)-PGEOH(JL,JK)) &
       & + RETV*PQENH(JL,JK)*(PSENH5(JL,JK)-PGEOH5(JL,JK)))*ZRCPD   
      PBUOH5(JL,JK) = RG*(ZTVUH5-ZTVENH5)/ZTVENH5
      PBUOH(JL,JK)  = RG*(ZTVUH*ZTVENH5-ZTVUH5*ZTVENH)/(ZTVENH5**2)
      ZBUOF5 = (PBUOH5(JL,JK) + PBUOH5(JL,JK+1))*0.5_JPRB
      ZBUOF  = (PBUOH(JL,JK)  + PBUOH(JL,JK+1)) *0.5_JPRB

! Reduce buoyancy perturbation if required
      IF (LREGCV) THEN
        ZBUOF=ZBUOF*0.33_JPRB
      ENDIF

! solve kinetic energy equation

      ZDMIX25=1.0_JPRB/(1.0_JPRB+2.0_JPRB*ZBW*ZMIX5(JL))
      PWU2H5(JL,JK) = (PWU2H5(JL,JK+1)*(1.0_JPRB-2.0_JPRB*ZBW*ZMIX5(JL)) &
       & + 2.0_JPRB*ZAW*ZBUOF5*ZDZ5(JL))*ZDMIX25  
      PWU2H(JL,JK) =( PWU2H(JL,JK+1)*(1.0_JPRB-2.0_JPRB*ZBW*ZMIX5(JL)) &
       & - 2.0_JPRB*ZBW*ZMIX(JL)*(PWU2H5(JL,JK+1)+PWU2H5(JL,JK)) &
       & + 2.0_JPRB*ZAW*(ZBUOF5*ZDZ(JL)+ZBUOF*ZDZ5(JL)) )*ZDMIX25  

! first layer with liquid water - find exact cloud base

      IF (PLU5(JL,JK) >0.0_JPRB.AND.KLAB(JL,JK+1)==1) THEN
        
        IK=JK+1

        ZALFAW5=FOEALFA(PTU5(JL,IK))
        ZALFAW=FOEALFATL(PTU(JL,IK),PTU5(JL,IK))
        Z1DPH5=1.0_JPRB/PAPH5(JL,IK)
        ZQSU5=FOEEWM(PTU5(JL,IK))*Z1DPH5
        ZQSU=(FOEEWMTL(PTU(JL,IK),PTU5(JL,IK),ZALFAW)*PAPH5(JL,IK) &
         & -FOEEWM(PTU5(JL,IK))*PAPH(JL,IK))*(Z1DPH5**2)  
        IF (ZQSU5 > 0.5_JPRB) ZQSU=0.0_JPRB
        ZQSU5=MIN(0.5_JPRB,ZQSU5)
        ZCOR5=1.0_JPRB/(1.0_JPRB-RETV*ZQSU5)
        ZCOR=RETV*ZQSU*(ZCOR5**2)
        ZQSU=ZQSU5*ZCOR+ZQSU*ZCOR5
        ZQSU5=ZQSU5*ZCOR5
        ZDQ5=PQU5(JL,IK)-ZQSU5
        ZDQ=PQU(JL,IK)-ZQSU

        ZFACW5=R5LES/((PTU5(JL,IK)-R4LES)**2)
        ZFACW=-2.0_JPRB*PTU(JL,IK)*ZFACW5/(PTU5(JL,IK)-R4LES)
        ZFACI5=R5IES/((PTU5(JL,IK)-R4IES)**2)
        ZFACI=-2.0_JPRB*PTU(JL,IK)*ZFACI5/(PTU5(JL,IK)-R4IES)
        ZFAC5=ZALFAW5*ZFACW5+(1.-ZALFAW5)*ZFACI5
        ZFAC=ZALFAW5*ZFACW+ZALFAW*ZFACW5+(1.-ZALFAW5)*ZFACI-ZALFAW*ZFACI5

        ZESDP5=FOEEWM(PTU5(JL,IK))*Z1DPH5
        ZESDP=(FOEEWMTL(PTU(JL,IK),PTU5(JL,IK),ZALFAW)*PAPH5(JL,IK) &
         & - FOEEWM(PTU5(JL,IK))*PAPH(JL,IK))*(Z1DPH5**2)  
        ZCOR5=1.0_JPRB/(1.0_JPRB-RETV*ZESDP5)
        ZCOR=RETV*ZESDP*(ZCOR5**2)
        ZDQSDT5=ZFAC5*ZCOR5*ZQSU5
        ZDQSDT=ZFAC5*ZCOR5*ZQSU+ZFAC5*ZCOR*ZQSU5+ZFAC*ZCOR5*ZQSU5
        ZDTDP5=RD*PTU5(JL,IK)*ZRCPD*Z1DPH5
        ZDTDP=ZDTDP5*(PTU(JL,IK)/PTU5(JL,IK)-PAPH(JL,IK)*Z1DPH5)
        ZFACT3=1.0_JPRB/(ZDQSDT5*ZDTDP5)
        ZDP5=ZDQ5*ZFACT3
        ZDP =(ZDQ-ZDQ5*(ZDQSDT*ZDTDP5+ZDQSDT5*ZDTDP)*ZFACT3)*ZFACT3
        ZCBASE5(JL)=PAPH5(JL,IK)+ZDP5
        ZCBASE(JL)=PAPH(JL,IK)+ZDP
        
! chose nearest half level as cloud base

        ZPDIFFTOP5=ZCBASE5(JL)-PAPH5(JL,JK)
        ZPDIFFTOP=ZCBASE(JL)-PAPH(JL,JK)
        ZPDIFFBOT5=PAPH5(JL,JK+1)-ZCBASE5(JL)
        ZPDIFFBOT=PAPH(JL,JK+1)-ZCBASE(JL)
        
        IF(ZPDIFFTOP5 > ZPDIFFBOT5 .AND. PWU2H5(JL,JK+1) > 0.0_JPRB &
           & .AND. JK < KLEV-1) THEN  
          KLAB(JL,JK)=2
          KLAB(JL,JK+1)=2
          LDSC(JL)   =.TRUE.
          KBOTSC(JL) =JK+1
          KCBOT(JL)  =JK+1
          PLU5(JL,JK+1)= RLMIN
          PLU(JL,JK+1) = 0.0_JPRB
        ELSEIF((ZPDIFFTOP5 <= ZPDIFFBOT5 .OR. &
           & (ZPDIFFTOP5 > ZPDIFFBOT5 .AND. JK == KLEV-1)) .AND. &
           & PWU2H5(JL,JK) > 0.0_JPRB) THEN  
          KLAB(JL,JK)=2
          LDSC(JL)   =.TRUE.
          KBOTSC(JL) =JK
          KCBOT(JL)  =JK
        ENDIF
      ENDIF

! decide on presence of convection, cloud base and cloud top based on
! kinetic energy

      IF (PWU2H5(JL,JK) < 0.0_JPRB) THEN
        LD_LLGO_ON(JL) = .FALSE.
        IF (PLU5(JL,JK+1) > 0.0_JPRB .AND. JK < K_JKSTART) THEN
          KCTOP(JL)   = JK
          KSTUP(JL)   = K_JKSTART+1
          LDCUM(JL)   = .TRUE.
        ELSE
          LDCUM(JL)   = .FALSE.
          KCBOT(JL)   = -1
          KCTOP(JL)   = -1
        ENDIF
      ELSE
        IF (PLU5(JL,JK)>0.0_JPRB) THEN
          KLAB(JL,JK) = 2
        ELSE
          KLAB(JL,JK) = 1
        ENDIF
      ENDIF
    ENDIF
  ENDDO

  IF (JOK == 0) EXIT

ENDDO

IF (LHOOK) CALL DR_HOOK('CUPDRATL',1,ZHOOK_HANDLE)
END SUBROUTINE CUPDRATL
