!OPTIONS XOPT(HSFUN)
SUBROUTINE GWPROFIL &
 & ( KIDIA ,KFDIA   ,KLON   , KLEV,&
 & LDGWD,&
 & KKCRITH, KCRIT , KKENVH, KKNU,KKNU2,&
 & PAPHM1, PRHO   , PSTAB , PVPH , PRI , PTAU,&
 & PDMOD , PSIG   , PHSTD      )  

!**** *GWPROFIL*

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!          FROM *GWDRAG*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===

!        *KIDIA*       START POINT
!        *KFDIA*       END POINT
!        *KLON*        NUMBER OF GRID POINTS PER PACKET
!        *KLEV*        NUMBER OF VERTICAL LEVELS
!        *LDGWD*       TRUE IF SSO ACTIVE
!        *KKCRITH*     MAXIMUM HALF-LEVEL FOR WAVE BREAKING
!        *KCRIT*       CRITICAL LEVEL
!        *KKENVH*      HALF-LEVEL FOR BLOCKING (TOP OF ENVELOPE LAYER)
!        *KKNU*        MODEL LEVEL AT 4*PHSTD HEIGHT 
!        *KKNU2*       MODEL LEVEL AT 3*PHSTD HEIGHT
!        *PAPHM1*      PRESSURE AT HALF LEVELS AT T-1
!        *PRHO*        AIR DENSITY
!        *PSTAB*       SQUARE OF THE BRUNT-VAISALA FREQUENCY
!        *PVPH*        WIND PROJECTION ALONG SURFACE STRESS AT HALF LEVELS
!        *PRI*         MEAN FLOW RICHARDSON NUMBER
!        *PTAU*        STRESS PRODUCED BY GRAVITY WAVES
!        *PDMOD*       MODULUS OF THE SURFACE STRESS (PART)
!        *PSIG*        MEAN SLOPE OF THE SUBGRID SCALE OROGRAPHY
!        *PHSTD*       STANDARD DEVIATION OF THE SUBGRID SCALE OROGRAPHY

!     ==== OUTPUTS ===

!        *PTAU*        STRESS PRODUCED BY GRAVITY WAVES

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD:
!     -------
!     THE STRESS PROFILE FOR GRAVITY WAVES IS COMPUTED AS FOLLOWS:
!     IT IS CONSTANT (NO GWD) AT THE LEVELS BETWEEN THE GROUND
!     AND THE TOP OF THE BLOCKED LAYER (KKENVH).
!     IT DECREASES LINEARLY WITH HEIGHTS FROM THE TOP OF THE 
!     BLOCKED LAYER TO 3*VAROR (KKNU), TO SIMULATES LEE WAVES OR 
!     NONLINEAR GRAVITY WAVE BREAKING.
!     ABOVE IT IS CONSTANT, EXCEPT WHEN THE WAVE ENCOUNTERS A CRITICAL
!     LEVEL (KCRIT) OR WHEN IT BREAKS.

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------

!        SEE ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S."

!     AUTHOR.
!     -------

!     MODIFICATIONS.
!     --------------
!     MIGRATION OF THE NEW GWDRAG TO I.F.S. (F. LOTT, 22/11/93)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEGWD   , ONLY : GRCRIT   ,GKDRAG   ,GSSEC    ,GTSEC
USE YOEPHLI  , ONLY : LPHYLIN  ,RLPDRAG

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
LOGICAL           ,INTENT(IN)    :: LDGWD(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KKCRITH(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCRIT(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KKENVH(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KKNU(KLON) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KKNU2(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHM1(KLON,0:KLEV) !note callpar indexing convention
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRHO(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTAB(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVPH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRI(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTAU(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDMOD(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSIG(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHSTD(KLON) 
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZDZ2 (KLON,KLEV) , ZNORM(KLON) , ZORO(KLON)
REAL(KIND=JPRB) :: ZTAU (KLON,KLEV+1)

INTEGER(KIND=JPIM) :: JK, JL

REAL(KIND=JPRB) :: ZALFA, ZALPHA, ZB, ZDEL, ZDELP, ZDELPT, ZDZ2N, ZKDRAG, ZRIW, ZSQR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

!*         1.    INITIALIZATION
!                --------------

!*    COMPUTATIONAL CONSTANTS.
!     ------------- ----------

!     Constant for surface stress

IF (LHOOK) CALL DR_HOOK('GWPROFIL',0,ZHOOK_HANDLE)
IF (LPHYLIN) THEN
  ZKDRAG=RLPDRAG
ELSE
  ZKDRAG=GKDRAG
ENDIF

DO JL=KIDIA,KFDIA
  IF(LDGWD(JL)) THEN
    ZORO(JL)=PSIG(JL)*PDMOD(JL)/4._JPRB/MAX(PHSTD(JL),1.0_JPRB)
    ZTAU(JL,KKNU(JL)+1)=PTAU(JL,KKNU(JL)+1)
    ZTAU(JL,KLEV+1)    =PTAU(JL,KLEV+1)
  ENDIF
ENDDO

DO JK=KLEV,2,-1

!*         1.1    CONSTANT WAVE STRESS UNTIL TOP OF THE
!                 BLOCKING LAYER.

  DO JL=KIDIA,KFDIA
    IF(LDGWD(JL)) THEN
      IF(JK >= KKNU2(JL)) THEN
        PTAU(JL,JK)=ZTAU(JL,KLEV+1)
      ENDIF
    ENDIF
  ENDDO

!*         1.2    WAVE DISPLACEMENT AT NEXT LEVEL.

!OCL  VCT(CEX)
  DO JL=KIDIA,KFDIA
    IF(LDGWD(JL)) THEN
      IF(JK < KKNU2(JL)) THEN
        ZNORM(JL)=ZKDRAG*PRHO(JL,JK)*SQRT(PSTAB(JL,JK))*PVPH(JL,JK)*ZORO(JL)
        ZDZ2(JL,JK)=PTAU(JL,JK+1)/MAX(ZNORM(JL),GSSEC)
      ENDIF
    ENDIF
  ENDDO

!*         1.3    WAVE RICHARDSON NUMBER, NEW WAVE DISPLACEMENT
!*                AND STRESS:  BREAKING EVALUATION AND CRITICAL 
!                 LEVEL

  DO JL=KIDIA,KFDIA
    IF(LDGWD(JL)) THEN
      IF(JK < KKNU2(JL)) THEN
        IF((PTAU(JL,JK+1) < GTSEC).OR.(JK <= KCRIT(JL))) THEN
          PTAU(JL,JK)=0.0_JPRB
        ELSE
          ZSQR=SQRT(PRI(JL,JK))
          ZALFA=SQRT(PSTAB(JL,JK)*ZDZ2(JL,JK))/PVPH(JL,JK)
          ZRIW=PRI(JL,JK)*(1.0_JPRB-ZALFA)/(1+ZALFA*ZSQR)**2
          IF(ZRIW < GRCRIT) THEN
            ZDEL=4._JPRB/ZSQR/GRCRIT+1.0_JPRB/GRCRIT**2+4._JPRB/GRCRIT
            ZB=1.0_JPRB/GRCRIT+2.0_JPRB/ZSQR
            ZALPHA=0.5_JPRB*(-ZB+SQRT(ZDEL))
            ZDZ2N=(PVPH(JL,JK)*ZALPHA)**2/PSTAB(JL,JK)
            PTAU(JL,JK)=ZNORM(JL)*ZDZ2N
          ELSE
            PTAU(JL,JK)=ZNORM(JL)*ZDZ2(JL,JK)
          ENDIF
          PTAU(JL,JK)=MIN(PTAU(JL,JK),PTAU(JL,JK+1))
        ENDIF
      ENDIF
    ENDIF
  ENDDO

ENDDO

!  REORGANISATION OF THE STRESS PROFILE
!  IF BREAKING OCCURS AT LOW LEVEL:
DO JL=KIDIA,KFDIA
  IF(LDGWD(JL)) THEN
    ZTAU(JL,KKENVH(JL)) =PTAU(JL,KKENVH(JL))
    ZTAU(JL,KKCRITH(JL))=PTAU(JL,KKCRITH(JL))
  ENDIF
ENDDO

DO JK=1,KLEV

  DO JL=KIDIA,KFDIA
    IF(LDGWD(JL)) THEN
      IF(JK > KKCRITH(JL).AND.JK < KKENVH(JL))THEN

        ZDELP=PAPHM1(JL,JK-1)-PAPHM1(JL,KKENVH(JL)-1)
        ZDELPT=PAPHM1(JL,KKCRITH(JL)-1)-PAPHM1(JL,KKENVH(JL)-1)
        PTAU(JL,JK)=ZTAU(JL,KKENVH(JL)) +&
         & (ZTAU(JL,KKCRITH(JL))-ZTAU(JL,KKENVH(JL)) )*&
         & ZDELP/ZDELPT  

      ENDIF
    ENDIF
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('GWPROFIL',1,ZHOOK_HANDLE)
END SUBROUTINE GWPROFIL
