SUBROUTINE m7 ( kidia, kfdia,   klon,   klev  , & ! ECMWF indices
  &             papp1, prelhum, ptp1,   pdz   , & !   "   thermodynamics
  &             pso4g, pso2g,   paerml, paernl, & !  M7   tracers
  &             pm6rp, pm6dry,  prhop,  pww     ) !   "   aerosol properties
  !
  !   ****m7* Aerosol model for the system so4,bc,oc,ss,dust in 7 modes.
  !
  !   Authors:
  !   ---------
  !   E. Vignati, JRC/EI     (original source)                   2000
  !   P. Stier, MPI          (f90-version, changes, comments)    2001 
  !
  !   Purpose
  !   ---------
  !   Aerosol model for the system so4,bc,oc,ss,dust in 7 modes.
  !
  !   Externals
  !   ---------
  !
  !   *m7_averageproperties* 
  !       calculates the average mass for all modes and the particle 
  !       dry radius and density for the insoluble modes.
  !    
  !   *m7_equiz*   
  !       calculates the ambient radius of sulphate particles
  !
  !   *m7_equimix* 
  !       calculates the ambient radius of so4,bc,oc (dust) particles
  !
  !   *m7_equil* 
  !       calculates the ambient radius of so4,ss particles 
  !
  !   *m7_dgas*    
  !       calculates the sulfate condensation on existing particles
  !
  !   *m7_dnum*    
  !       calculates new gas phase sulfate and aerosol numbers and masses
  !       after condensation, nucleation and coagulation over one timestep
  !
  !   *m7_dconc*   
  !       repartitions aerosol number and mass between the
  !       the modes to account for condensational growth and the formation
  !       of an accumulation mode from the upper tail of the aitken mode and 
  !       of a coarse mode from the upper tail of the accumulation mode
  !
  !  Modifications:
  !  --------------
  !  20091112: JJMorcrette, Adaptation to ECMWF IFS 
  !


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YO_AERO_M7 ,ONLY : lsnucl, lscoag, lscond,         &
  &                    nmod,   nss,    nsol,   naermod

  IMPLICIT NONE 

  !--- Parameter list:
  !
  !  papp1      = atmospheric pressure at time t+1 [Pa]
  !  prelhum    = atmospheric relative humidity [% (0-1)]
  !  ptp1       = atmospheric temperature at time t+1 [K]
  !  pso4g      = mass of gas phase sulfate [molec. cm-3]
  !  pso2g      = mass of gas phase SO2 [molec. cm-3]
  !  paerml     = total aerosol mass for each compound 
  !               [molec. cm-3 for sulphate and ug m-3 for bc, oc, ss, and dust]
  !  paernl     = aerosol number for each mode [cm-3]
  !  prhop      = mean mode particle density [g cm-3]
  !  pdz        = layer thickness dz for diagnostics [m]
  !  pm6rp      = mean mode actual radius (wet radius for soluble modes 
  !               and dry radius for insoluble modes) [cm]
  !  pm6dry     = dry radius for soluble modes [cm]
  !  pww        = aerosol water content for each mode [kg(water) m-3(air)]
  !
  !--- Local variables:
  !
  !  zttn       = average mass for single compound in each mode 
  !               [in molec. for sulphate and in ug for bc, oc, ss, and dust]
  !  zhplus     = number of h+ in mole [???] (kg water)-1
  !  zso4_x     = mass of sulphate condensed on insoluble mode x [molec. cm-3]
  !               (calculated in dgas used in concoag)


  ! Parameters:

INTEGER(KIND=JPIM),INTENT(IN) :: kidia, kfdia, klon, klev

REAL(KIND=JPRB),INTENT(IN) :: &
  & prelhum(klon ,klev),        papp1(klon ,klev),           &
  & ptp1(klon ,klev),           pdz(klon ,klev),             &
  & pso2g(klon ,klev)
 
REAL(KIND=JPRB),INTENT(OUT):: &
  & paerml(klon ,klev,naermod), paernl(klon ,klev,nmod),     &
  & pm6rp(klon ,klev,nmod),     pm6dry(klon ,klev,nsol),     &
  & prhop(klon ,klev,nmod),     pww(klon ,klev,nmod)

REAL(KIND=JPRB),INTENT(INOUT) :: pso4g(klon ,klev)

  ! Local variables:

REAL(KIND=JPRB) :: &
  & zso4_5(klon ,klev),         zso4_6(klon ,klev),          &
  & zso4_7(klon ,klev)

REAL(KIND=JPRB) :: zhplus(klon ,klev,nss)

REAL(KIND=JPRB) :: zttn(klon ,klev,naermod)

#include "m7_averageproperties.intfb.h"
#include "m7_dnum.intfb.h"
#include "m7_dgas.intfb.h"
#include "m7_equiz.intfb.h"
#include "m7_equil.intfb.h"
#include "m7_equimix.intfb.h"
#include "m7_dconc.intfb.h"

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!--------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7',0,ZHOOK_HANDLE)

  !
  !--- 0) Initialisations: -------------------------------------------------
  !
  zhplus(:,:,:) = 0._JPRB
  pm6dry(:,:,:) = 0._JPRB 
  pm6rp(:,:,:)  = 0._JPRB
  zttn(:,:,:)   = 0._JPRB
  prhop(:,:,:)  = 0._JPRB
  pww(:,:,:)    = 0._JPRB 
  zso4_5(:,:)   = 0._JPRB
  zso4_6(:,:)   = 0._JPRB
  zso4_7(:,:)   = 0._JPRB
  !
  !
  !--- 1) Calculation of particle properties under ambient conditions: -----
  !
  !--- 1.1) Calculate mean particle mass for all modes 
  !         and dry radius and density for the insoluble modes.
  !
  CALL m7_averageproperties(kidia, kfdia, klon , klev, paernl, paerml, zttn, pm6rp, prhop)
  !
  !--- 1.2) Calculate ambient count median radii and density 
  !         for lognormal distribution of particles.
  !
  !         Sulfate particles:
  !
  CALL m7_equiz(kidia  , kfdia, klon  , klev, &
    &           papp1  , zttn , ptp1  , &
    &           prelhum, pm6rp, pm6dry, &
    &           prhop  , pww  , paernl  )
  !         
  !         Mixed particles with sulfate, b/o carbon and dust: 
  !
  CALL m7_equimix(kidia  , kfdia, klon  , klev, &
    &             papp1  , zttn , ptp1  , &
    &             prelhum, pm6rp, pm6dry, &
    &             prhop  , pww  , paernl  )
  !
  !         Accumulation and coarse mode particles in presence of
  !         sea salt particles:
  !
  CALL m7_equil(kidia  , kfdia , klon  , klev, &
    &           prelhum, paerml, paernl, &
    &           pm6rp  , pm6dry, zhplus, pww , prhop  , ptp1 )




!---------------------------------------------------------------------------------
  !
  !--- 2) Calculate changes in aerosol mass and gas phase sulfate ----------
  !       due to sulfate condensation:
  !       No change in particle mass/number relationships.
  !
!!  IF (lscond) CALL m7_dgas(kidia , kfdia , klon  , klev, krow, &
  IF (lscond) CALL m7_dgas(kidia , kfdia , klon  , klev, &
                &          pso4g , paerml, paernl, & 
                &          ptp1  , papp1 , pm6rp , &
                &          zso4_5, zso4_6, zso4_7, &
                &          pdz                     )



!---------------------------------------------------------------------------------
  !
  !
  !--- 3) Calculate change in particle number concentrations ---------------
  !       due to nucleation and coagulation:
  !       Change particle mass/number relationships.
  !
!!  IF (lsnucl.OR.lscoag) CALL m7_dnum(kidia , kfdia  , klon  , klev , krow, &
  IF (lsnucl.OR.lscoag) CALL m7_dnum(kidia , kfdia  , klon  , klev , &
                          &          pso4g , paerml , paernl, ptp1 , &
                          &          papp1 , prelhum, pm6rp , prhop, &
                          &          zso4_5, zso4_6 , zso4_7, pdz    )



!---------------------------------------------------------------------------------
  !
  !
  !--- 4) Recalculation of particle properties under ambient conditions: ---
  !
  !--- 4.1) Recalculate mean masses for all modes 
  !         and dry radius and density for the insoluble modes.
  !
  CALL m7_averageproperties(kidia, kfdia, klon , klev, paernl, paerml, zttn, pm6rp, prhop)
  !
  !--- 4.2) Calculate ambient count median radii and density 
  !         for lognormal distribution of particles.
  !
  !         Sulfate particles:
  !
  CALL m7_equiz(kidia  , kfdia, klon  , klev, &
    &           papp1  , zttn , ptp1  , &
    &           prelhum, pm6rp, pm6dry, &
    &           prhop  , pww  , paernl  )
  !
  !         Mixed particles with sulfate, b/o carbon and dust:
  !
  CALL m7_equimix(kidia  , kfdia, klon  , klev, &
    &             papp1  , zttn , ptp1  , &
    &             prelhum, pm6rp, pm6dry, &
    &             prhop  , pww  , paernl  )
  !
  !         Accumulation and coarse mode particles in presence of
  !         sea salt particles:  
  ! 
  CALL m7_equil(kidia  , kfdia , klon  , klev, &
    &           prelhum, paerml, paernl, &
    &           pm6rp  , pm6dry, zhplus, pww , prhop  , ptp1 )



!---------------------------------------------------------------------------------
  !
  !--- 5) Repartitition particles among the modes: -------------------------
  !
  IF (lscond.OR.lscoag) THEN
     
     CALL m7_dconc(kidia, kfdia, klon , klev, paerml, paernl, pm6dry)
     
  ENDIF



!---------------------------------------------------------------------------------
  !
  !--- 6) Recalculation of particle properties under ambient conditions: ---
  !
  !--- 6.1) Calculate mean particle mass for all modes 
  !         and dry radius and density for the insoluble modes:
  !
  CALL m7_averageproperties(kidia, kfdia, klon , klev, paernl, paerml, zttn, pm6rp, prhop)
  !
  !--- 6.2) Calculate ambient count median radii and density 
  !         for lognormal distribution of particles.
  !
  !         Sulfate particles:
  !
  CALL m7_equiz(kidia  , kfdia, klon  , klev, &
    &           papp1  , zttn , ptp1  , &
    &           prelhum, pm6rp, pm6dry, &
    &           prhop  , pww  , paernl  )
  !
  !         Mixed particles with sulfate, b/o carbon and dust: 
  !
  CALL m7_equimix(kidia  , kfdia, klon  , klev, &
    &             papp1  , zttn , ptp1  , &
    &             prelhum, pm6rp, pm6dry, &
    &             prhop  , pww  , paernl  )
  !
  !         Accumulation and coarse mode particles in presence of
  !         sea salt particles:
  !
  CALL m7_equil(kidia  , kfdia , klon  , klev, &
    &           prelhum, paerml, paernl, &
    &           pm6rp  , pm6dry, zhplus, pww , prhop  , ptp1 )
  !

!----------------------------------------------------------------------
 IF (LHOOK) CALL DR_HOOK('M7',1,ZHOOK_HANDLE)
END SUBROUTINE m7
