SUBROUTINE m7_coaset &
  & (kidia , kfdia, klon , klev, &
  &  paernl, ptp1 , papp1, pm6rp, prhop, &
  &  pcom )
  !
  ! *m7_coaset*  calculates the coagulation kernels between the modes
  !
  ! Authors:
  ! ---------
  ! J. Wilson  and E. Vignati, JRC/EI (original source)                09/2000
  ! P. Stier, MPI                     (f90-version, changes, comments)    2001 
  !
  ! Modifications:
  ! --------------
  ! Philip Stier, MPI                             2001
  !
  ! Purpose
  ! ---------
  ! This routine calculates the coaglation kernels between particles
  ! with the count median radii of the three modes.
  ! Coagulation allowed between the following modes:
  ! soluble modes:   1+1=1, 2+2=2, 1+2=2, 1+3=3, 1+4=4, 2+3=3, 2+4=4
  ! insoluble modes: 2i+2i=2i
  ! mixed modes:     1+2i=2, 1+4i=4, 2+2i=2, 2+4i=4, 3+2i=3, 4+2i=4
  !
  ! Interface:
  ! -----------
  ! *m7_coaset* is called from *m7_dnum*
  !
  ! Externals:
  ! -----------
  ! none
  !
  ! Reference:
  ! -----------
  ! The calculations are based on:
  ! Fuchs, N.A. (1964). The Mechanics of Aerosols. Pergamon Press. Oxford. 
  ! (Chapter VII, Section 49)
  !
  !  Warning:
  !  --------
  !  For optimization purposes currently only "physically reasonable" elements of the
  !  coagulation kernel pcom are calculated in m7_concoag. These elements are specified
  !  in the matrix locoagmask in mo_aero_m7. Check carefully and adapt locoagmask 
  !  accordingly  before changes in the code below.
  !
  !  Modifications:
  !  --------------
  !  20091112: JJMorcrette, Adaptation to ECMWF IFS 
  !


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YO_AERO_M7,ONLY : bk, nmod, locoagmask
USE YOMCST    ,ONLY : RPI
  !
  IMPLICIT NONE
  !
!--- Parameter list:
!
!- Inputs
!  ------
!  paernl            = aerosol number for each mode [cm-3]
!  ptp1              = atmospheric temperature at time t+1 [K]
!  papp1             = atmospheric pressure at time t+1 [Pa]
!  pm6rp             = mean mode actual radius (wet radius for soluble modes 
!                      and dry radius for insoluble modes) [cm]
!  prhop             = mean mode particle density [g cm-3]

!- Outputs
!  -------
!  pcom(:,:,jm1,jm2) = Coagulation coefficient for modes jm1 and jm2 []

  !
  !--- List of local variables:
  !
  ! zwlc              = mean free pathlength []
  ! zairvisc          = air viscosity []
  ! zrpav             = average radius of the two interacting modes
  ! zpvx              = volume of the xth interacting mode
  ! zpmx              = mass of the xth interacting mode
  !
  ! zrknudx           = knudsen number of the xth interacting mode
  ! zpd2x             = particle diffusion of the xth interacting mode
  !
  !--- Parameters:
  !
INTEGER(KIND=JPIM),INTENT(IN) :: kidia, kfdia, klon, klev

REAL(KIND=JPRB),INTENT(IN) :: ptp1(klon ,klev),          papp1(klon ,klev)

REAL(KIND=JPRB),INTENT(OUT):: pm6rp(klon ,klev,nmod),    prhop(klon ,klev,nmod),                 &
  &                           paernl(klon ,klev,nmod)

REAL(KIND=JPRB),INTENT(OUT):: pcom(klon ,klev,nmod,nmod)

  !--- Local variables:
  
INTEGER(KIND=JPIM) :: jm2, jm1, jl, jk
  !
REAL(KIND=JPRB) :: zpbyone,     zwlc,        zairvisc,    zbtketc,   &
  &                zrpvm1,      zrpvm2,      zrpav,       zpv1,      &
  &                zpm1,        zcv21,       zpv2,        zpm2,      &
  &                zcv22,       zcv2av,      zrknud1,     zrknud2,   &
  &                ze1,         ze2,         zpd21,       zpd22,     &
  &                zpdav,       zxd1,        zh21,        zxd2,      &
  &                zh22,        zh2av,       zcoc,        zhu1,      &
  &                zhu2,        zhu,         zsqrt2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!--------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_COASET',0,ZHOOK_HANDLE)

ZSQRT2=SQRT(2._JPRB)

  !--- 1) Calculation of the coagulation coefficient: ---------------------------
  !
  DO jm2=1,nmod
     DO jm1=jm2,nmod

        IF (locoagmask(jm1,jm2)) THEN

        DO jk=1,klev
           DO jl=kidia,kfdia
             IF (paernl(jl,jk,jm1) > 1.E-10_JPRB .AND. paernl(jl,jk,jm2) > 1.E-10_JPRB .AND. &
               &  pm6rp(jl,jk,jm1)  > 1.E-10_JPRB .AND. pm6rp(jl,jk,jm2)  > 1.E-10_JPRB .AND. &
               &  prhop(jl,jk,jm1)  > 1.E-10_JPRB .AND. prhop(jl,jk,jm2)  > 1.E-10_JPRB        ) THEN        
           
                 !--- 1.1) Calculate ambient properties:

                 !--- Mean free pathlength ? (from Knudsen Number below):
                 !    Parametrisation?
                 zpbyone=1000.0_JPRB / (papp1(jl,jk)/100.0_JPRB)
                 zwlc=6.6e-6_JPRB * ptp1(jl,jk) / 293.15_JPRB * zpbyone

                 !--- Viscosity:
                 zairvisc=1.827e-4_JPRB * (ptp1(jl,jk) / 291.15_JPRB)**0.74_JPRB

                 !--- 
                 zbtketc=bk * ptp1(jl,jk) / 6.0_JPRB / rpi / zairvisc

                 !--- Count median radii of the respective modes:
                 zrpvm1=pm6rp(jl,jk,jm1)
                 zrpvm2=pm6rp(jl,jk,jm2)

                 !--- Average radius of the modes:
                 zrpav=(zrpvm1 + zrpvm2) / 2.0_JPRB

                 !--- Volume and mass of mode jm1:
                 zpv1=4.0_JPRB / 3.0_JPRB * rpi * zrpvm1**3.0_JPRB
                 zpm1=zpv1 * prhop(jl,jk,jm1)

                 !--- Squared mean particle velocity of mode jm1:
                 zcv21=8.0_JPRB * bk * ptp1(jl,jk) / (rpi * zpm1)

                 !--- Volume and mass of particles in mode jm2:
                 zpv2=4.0_JPRB / 3.0_JPRB * rpi * zrpvm2**3.0_JPRB
                 zpm2=zpv2 * prhop(jl,jk,jm2)

                 !--- Squared mean particle velocity of mode jm2:
                 zcv22=8.0_JPRB * bk * ptp1(jl,jk) / (rpi * zpm2)

                 !--- Fuchs: G_r (below Eq. 49.27):
                 zcv2av=SQRT(zcv21 + zcv22)

                 !--- Knudsen numbers of the modes:
                 !@@@ Check: is zwlc mean free path then zrknud=zwlc/zrpvm!
                 zrknud1=zwlc/zrpvm1/2.0_JPRB
                 zrknud2=zwlc/zrpvm2/2.0_JPRB

                 !--- Diffusivities of the respective modes:
                 !    !@@@ Parameterisation?
                 ze1=EXP(-0.43_JPRB/zrknud1)
                 ze2=EXP(-0.43_JPRB/zrknud2)
                 zpd21=zbtketc * (1.0_JPRB + zrknud1*2.492_JPRB + zrknud1*0.84_JPRB*ze1) / zrpvm1
                 zpd22=zbtketc * (1.0_JPRB + zrknud2*2.492_JPRB + zrknud1*0.84_JPRB*ze2) / zrpvm2

                 !--- Average diffusivity of the modes:
                 zpdav=(zpd21 + zpd22) / 2.0_JPRB

                 !--- Average mean free path of particles in jm1:
                 zxd1=8.0_JPRB * zpd21 / (rpi*SQRT(zcv21))

                 !--- Mean distance from surface after mean free path (Eq. 49.13):
                 zh21=(((2.0_JPRB*zrpvm1 + zxd1)**3.0_JPRB -                        &
                   &   (4.0_JPRB*zrpvm1*zrpvm1 + zxd1*zxd1)**1.5_JPRB) /            &
                   &   (6.0_JPRB*zrpvm1*zxd1) - 2._JPRB*zrpvm1           ) * zsqrt2

                 !--- Average mean free path of particles in jm2:
                 zxd2=8.0_JPRB * zpd22 / (rpi*SQRT(zcv22))

                 !--- Mean distance from surface after mean free path (Eq. 49.13):

                 zh22=(((2.0_JPRB*zrpvm2 + zxd2)**3.0_JPRB -                        &
                   &   (4.0_JPRB*zrpvm2*zrpvm2 + zxd2*zxd2)**1.5_JPRB) /            &
                   &   (6.0_JPRB*zrpvm2*zxd2) - 2._JPRB*zrpvm2           ) * zsqrt2

                 !--- Fuchs: delta_r !@@@ (why division by zsqrt2?)
                 zh2av=SQRT(zh21*zh21 + zh22*zh22) / zsqrt2
                 
                 !--- 1.2) Calculation of the coagulation coefficient pcom (Eq. 49.26):
                 !         Factor 16 instead factor 8 as in Fuchs as his formulation
                 !         applies for the inter-modal coagulation. This is taken into
                 !         account in the assignment of the inter-modal coagulation
                 !         coefficient.

                 zcoc=16.0_JPRB * rpi * zpdav * zrpav

                 !--- Calculation of beta=1/zhu (Eq. 49.27):
                 zhu1=4.0_JPRB * zpdav / (zcv2av * zrpav)
                 zhu2=zrpav / (zrpav + zh2av /2.0_JPRB)
                 zhu=zhu1 +  zhu2

                 !--- Coagulation coefficient following (Eq.49.26):
                 pcom(jl,jk,jm1,jm2)=zcoc / zhu

              ELSE
                 pcom(jl,jk,jm1,jm2)=0._JPRB
              ENDIF

              !--- 2) Mirror the coagulation matrix (symmetric): -----------------

              pcom(jl,jk,jm2,jm1)=pcom(jl,jk,jm1,jm2)

           ENDDO
        ENDDO

        ELSE

           pcom(kidia:kfdia,:,jm1,jm2)=0._JPRB

        ENDIF ! locoagmask

     ENDDO
  ENDDO

!--------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_COASET',1,ZHOOK_HANDLE)
END SUBROUTINE m7_coaset
