SUBROUTINE m7_cumnor ( parg, presult, pccum )
  !
  !*******************************************************************************
  !
  !! CUMNOR computes the cumulative normal distribution.
  !
  !
  !     the integral from -infinity to x of
  !          (1/sqrt(2*pi)) exp(-u*u/2) du
  !
  !  Author:
  !  -------
  !  Original source:
  !
  !    W. J. Cody    Mathematics and Computer Science Division
  !                  Argonne National Laboratory
  !                  Argonne, IL 60439
  !
  !    DCDFLIB is attributed to Barry Brown, James Lovato, and Kathy Russell
  !            bwb@odin.mda.uth.tmc.edu.
  !
  !    Adopted to ECHAM/M7:
  !
  !    Philip Stier  (MPI-MET)                    2001
  !
  !
  !  Reference:
  !  ----------
  !
  !    W D Cody, 
  !    "ALGORITHM 715: SPECFUN - A Portable FORTRAN Package of Special 
  !    Function Routines and Test Drivers"
  !    ACM Transactions on Mathematical Software,
  !    Volume 19, 1993, pages 22-32.
  !  !  Parameters:
  !  Parameters:
  !
  !     ARG --> Upper limit of integration.
  !                                        X is double precision
  !
  !     RESULT <-- Cumulative normal distribution.
  !                                        RESULT is double precision
  !
  !     CCUM <-- Complement of Cumulative normal distribution.
  !                                        CCUM is double precision
  !
  !
  ! Original Comments:
  !
  !
  ! This function evaluates the normal distribution function:
  !
  !                              / x
  !                     1       |       -t*t/2
  !          P(x) = ----------- |      e       dt
  !                 sqrt(2 pi)  |
  !                             /-oo
  !
  !   The main computation evaluates near-minimax approximations
  !   derived from those in "Rational Chebyshev approximations for
  !   the error function" by W. J. Cody, Math. Comp., 1969, 631-637.
  !   This transportable program uses rational functions that
  !   theoretically approximate the normal distribution function to
  !   at least 18 significant decimal digits.  The accuracy achieved
  !   depends on the arithmetic system, the compiler, the intrinsic
  !   functions, and proper selection of the machine-dependent
  !   constants.
  !
  !  Explanation of machine-dependent constants.
  !
  !   MIN   = smallest machine representable number.
  !
  !   EPS   = argument below which anorm(x) may be represented by
  !           0.5  and above which  x*x  will not underflow.
  !           A conservative value is the largest machine number X
  !           such that   1.0 + X = 1.0   to machine precision.
  !
  !  Error returns
  !
  !  The program returns  ANORM = 0     for  ARG .LE. XLOW.
  !
  !  Author: 
  !
  !    W. J. Cody
  !    Mathematics and Computer Science Division
  !    Argonne National Laboratory
  !    Argonne, IL 60439
  !
  !  Latest modification: March 15, 1992
  !
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK
  !
IMPLICIT NONE

REAL(KIND=JPRB),INTENT(IN)  :: PARG

REAL(KIND=JPRB),INTENT(OUT) :: PCCUM, PRESULT
  !
REAL(KIND=JPRB), PARAMETER, DIMENSION ( 5 ) :: za = (/ &
  &     2.2352520354606839287e00_JPRB, &
  &     1.6102823106855587881e02_JPRB, &
  &     1.0676894854603709582e03_JPRB, &
  &     1.8154981253343561249e04_JPRB, &
  &     6.5682337918207449113e-2_JPRB /)

REAL(KIND=JPRB), PARAMETER, DIMENSION ( 4 ) :: zb = (/ &
  &     4.7202581904688241870e01_JPRB, &
  &     9.7609855173777669322e02_JPRB, &
  &     1.0260932208618978205e04_JPRB, &
  &     4.5507789335026729956e04_JPRB /)
REAL(KIND=JPRB), PARAMETER, DIMENSION ( 9 ) :: zc = (/ &
  &     3.9894151208813466764e-1_JPRB, &
  &     8.8831497943883759412e00_JPRB, &
  &     9.3506656132177855979e01_JPRB, &
  &     5.9727027639480026226e02_JPRB, &
  &     2.4945375852903726711e03_JPRB, &
  &     6.8481904505362823326e03_JPRB, &
  &     1.1602651437647350124e04_JPRB, &
  &     9.8427148383839780218e03_JPRB, &
  &     1.0765576773720192317e-8_JPRB /)

REAL(KIND=JPRB), PARAMETER, DIMENSION ( 8 ) :: zd = (/ &
  &     2.2266688044328115691e01_JPRB, &
  &     2.3538790178262499861e02_JPRB, &
  &     1.5193775994075548050e03_JPRB, &
  &     6.4855582982667607550e03_JPRB, &
  &     1.8615571640885098091e04_JPRB, &
  &     3.4900952721145977266e04_JPRB, &
  &     3.8912003286093271411e04_JPRB, &
  &     1.9685429676859990727e04_JPRB /)
REAL(KIND=JPRB) :: ZDEL
!@@@ REAL(KIND=JPRB) :: dpmpar
REAL(KIND=JPRB) :: ZEPS
INTEGER(KIND=JPIM) :: i
REAL(KIND=JPRB) :: zmin
REAL(KIND=JPRB), PARAMETER, DIMENSION ( 6 ) :: zp = (/ &
  &     2.1589853405795699e-1_JPRB, &
  &     1.274011611602473639e-1_JPRB, &
  &     2.2235277870649807e-2_JPRB, &
  &     1.421619193227893466e-3_JPRB, &
  &     2.9112874951168792e-5_JPRB, &
  &     2.307344176494017303e-2_JPRB /)
REAL(KIND=JPRB), PARAMETER, DIMENSION ( 5 ) :: zq = (/ &
  &     1.28426009614491121e00_JPRB, &
  &     4.68238212480865118e-1_JPRB, &
  &     6.59881378689285515e-2_JPRB, &
  &     3.78239633202758244e-3_JPRB, &
  &     7.29751555083966205e-5_JPRB /)

REAL(KIND=JPRB), PARAMETER :: root32 = 5.656854248_JPRB
REAL(KIND=JPRB), PARAMETER :: sixten = 16.0_JPRB
REAL(KIND=JPRB) :: ZTEMP
REAL(KIND=JPRB), PARAMETER :: sqrpi = 3.9894228040143267794e-1_JPRB
REAL(KIND=JPRB), PARAMETER :: thrsh = 0.66291_JPRB
REAL(KIND=JPRB) :: ZX
REAL(KIND=JPRB) :: ZDEN
REAL(KIND=JPRB) :: ZNUM
REAL(KIND=JPRB) :: ZY
REAL(KIND=JPRB) :: ZXSQ

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!--------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_CUMNOR',0,ZHOOK_HANDLE)
  !
  !  Machine dependent constants
  !
!!  ZEPS = EPSILON ( 1.0_JPRB ) * 0.5_JPRB
!  zeps = TINY(1.0_JPRB)
  zeps=1.E-32_JPRB 
  !
  !@@@ Simplified calculation of the smallest machine representable number
  !    (Higher accuracy than needed!)
  !
  !@@@ min = dpmpar(2)

!  zmin = EPSILON ( 1.0_JPRB )
  zmin = TINY(1.0_JPRB)

  zx = PARG
  zy = ABS ( zx )

  IF ( zy <= thrsh ) THEN
     !
     !  Evaluate  anorm  for  |X| <= 0.66291
     !
     IF ( zy > ZEPS ) THEN
        ZXSQ = zx * zx
     ELSE
        ZXSQ = 0.0_JPRB
     ENDIF

     ZNUM = ZA(5) * ZXSQ
     ZDEN = ZXSQ
     DO i = 1, 3
        ZNUM = ( ZNUM + ZA(i) ) * ZXSQ
        ZDEN = ( ZDEN + ZB(i) ) * ZXSQ
     ENDDO
     presult = zx * ( ZNUM + ZA(4) ) / ( ZDEN + ZB(4) )
     ZTEMP = presult
     presult = 0.5_JPRB + ZTEMP
     PCCUM   = 0.5_JPRB - ZTEMP
     !
     !  Evaluate ANORM for 0.66291 <= |X| <= sqrt(32)
     !
  ELSEIF ( zy <= root32 ) THEN

     ZNUM = ZC(9) * zy
     ZDEN = zy
!CDIR UNROLL=7
     DO i = 1, 7
        ZNUM = ( ZNUM + ZC(i) ) * zy
        ZDEN = ( ZDEN + ZD(i) ) * zy
     ENDDO
     presult = ( ZNUM + ZC(8) ) / ( ZDEN + ZD(8) )
     ZXSQ = AINT ( zy * sixten ) / sixten
     ZDEL = ( zy - ZXSQ ) * ( zy + ZXSQ )
     presult = EXP(-ZXSQ*ZXSQ*0.5_JPRB) * EXP(-ZDEL*0.5_JPRB) * presult
     PCCUM = 1.0_JPRB - presult

     IF ( zx > 0.0_JPRB ) THEN
        ZTEMP = presult
        presult = PCCUM
        PCCUM = ZTEMP
     ENDIF
     !
     !  Evaluate  anorm  for |X| > sqrt(32).
     !
  ELSE

     presult = 0.0_JPRB
     ZXSQ = 1.0_JPRB / ( zx * zx )
     ZNUM = ZP(6) * ZXSQ
     ZDEN = ZXSQ
     DO i = 1, 4
        ZNUM = ( ZNUM + ZP(i) ) * ZXSQ
        ZDEN = ( ZDEN + ZQ(i) ) * ZXSQ
     ENDDO

     presult = ZXSQ * ( ZNUM + ZP(5) ) / ( ZDEN + ZQ(5) )
     presult = ( sqrpi - presult ) / zy
     ZXSQ = AINT ( zx * sixten ) / sixten
     ZDEL = ( zx - ZXSQ ) * ( zx + ZXSQ )
     presult = EXP ( - ZXSQ * ZXSQ * 0.5_JPRB ) * EXP ( - ZDEL * 0.5_JPRB ) * presult
     PCCUM = 1.0_JPRB - presult  

     IF ( zx > 0.0_JPRB ) THEN
        ZTEMP = presult
        presult = PCCUM
        PCCUM = ZTEMP
     ENDIF

  ENDIF

  IF ( presult < zmin ) THEN
     presult = 0.0_JPRB
  ENDIF

  IF ( PCCUM < zmin ) THEN
     PCCUM = 0.0_JPRB
  ENDIF

!----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_CUMNOR',1,ZHOOK_HANDLE)
END SUBROUTINE m7_cumnor
