SUBROUTINE m7_dgas &
  &( kidia , kfdia , klon  , klev, &
  &  pso4g , paerml, paernl, &
  &  ptp1  , papp1 , pm6rp , &
  &  pso4_5, pso4_6, pso4_7, &
  &  pdz                     )
  !
  !**** *m7_dgas*  calculates the transfer of mass due to 
  !                sulfate condensation on soluble and 
  !                insoluble modes.
  !
  !    Authors:
  !    -----------
  !    J. Wilson, E. Vignati, JRC/EI (original source)                05/2000
  !    P. Stier, MPI                 (f90-version, changes, comments)    2001   
  !
  !    Purpose:
  !    -----------
  !    This routine calculates the changes in aerosol mass and
  !    gas phase sulfate due to sulfate condensation.
  !
  !**  Interface:
  !    -----------
  !    *m7_dgas* is called from *m7*
  !
  !    Method:
  !    -----------------
  !    The transfer of sulfate to the particles is based on
  !    Fuchs (1959). Soluble and insoluble particles are distinguished
  !    by a different accommodation coefficient "caccso4" defined
  !    in mo_aero_m7.
  !    (Currently 1.0 for soluble and 0.3 for insoluble modes). 
  !
  !    Externals
  !    -----------
  !    none
  !
  !    References:
  !    -----------
  !    Fuchs, N.A. (1959). Evaporation and droplet growth in gaseous media;
  !       Pergamon, New York, pp72.
  ! 
  !  Modifications:
  !  --------------
  !  20091112: JJMorcrette, Adaptation to ECMWF IFS 
  !

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

!USE mo_time_control,  ONLY: time_step_len, delta_time
USE YOMPHY2   ,ONLY : TSPHY
!USE YO_AERO_M7,ONLY : wh2so4,  rerg,    avo,                     &
USE YO_AERO_M7,ONLY : wh2so4,  rerg,                     &
  &                   fmax,    caccso4,                  &
  &                   nmod,    naermod,                  &
  &                   iso4ns,  iso4ks,  iso4as,  iso4cs

!no_diag USE YO_DIA_M7  ,ONLY : d_cond_so4	! ??????
USE YOMCST    ,ONLY : RPI

  IMPLICIT NONE
  !
  !--- Parameter list:
  !
!- Inputs
!  ------
! papp1
! ptp1
! pm6rp     = mean mode actual radius (wet radius for soluble modes 
!             and dry radius for insoluble modes) [cm]
! pdz       = layer thickness dz for diagnostics [m]

!- Outputs
!  -------
! pso4_x    = mass of sulphate condensed on insoluble mode x [molec. cm-3]


!- In/Out
!  ------
! pso4g     = mass of gas phase sulfate [molec. cm-3]


  ! 
  !--- Local Variables:
  !
  ! zde       = molecular diffusion []
  ! zvelb     = velocity []
  ! zcondo    = condensation coefficient []
  ! zc2(nmod) = flux of sulfate condensing on the respective mode 
  !             per sulfate gas phase concentration []
  ! zcondo    = total flux of condensing sulfate 
  !             per sulfate gas phase concentration []
  ! zfcond    = total mass of condensing sulfate for one timestep []

!no_diag INTEGER(KIND=JPIM),INTENT(IN) :: kidia, kfdia, klon, klev, krow
INTEGER(KIND=JPIM),INTENT(IN) :: kidia, kfdia, klon, klev

REAL(KIND=JPRB),INTENT(IN) :: ptp1(klon ,klev), papp1(klon ,klev)     , &
  &                           pdz(klon ,klev) , pm6rp(klon ,klev,nmod)
  !
REAL(KIND=JPRB),INTENT(OUT):: paernl(klon ,klev,nmod), paerml(klon ,klev,naermod), &
  &                           pso4_5(klon ,klev)     , &
  &                           pso4_6(klon ,klev)     , pso4_7(klon ,klev)

REAL(KIND=JPRB),INTENT(INOUT)::pso4g(klon ,klev)
  
  !
  ! Local variables:

INTEGER(KIND=JPIM) :: jl,         jk,         jmod
  
REAL(KIND=JPRB) :: zfcond,     zftot,      zpbyone,    zde2,                   & 
  &                zvelb,      zxibc,      zm6rp,      zf1,                    &
  &                zqtmst,     ztsphy

REAL(KIND=JPRB) :: zcondo(klon ,klev)

REAL(KIND=JPRB) :: zc2(klon ,klev,nmod)

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!--------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_DGAS',0,ZHOOK_HANDLE)


  !--- 0) Initialisations: -------------------------------------------------

ztsphy=TSPHY

  zcondo(:,:)=0.0_JPRB
      
  zc2(:,:,:) = 0.0_JPRB  

  zqtmst=1._JPRB/ztsphy
  
  !--- 1) Calculate condensation rate for cm diameter sulphate aerosols: ---
  !
  DO jmod=1,nmod
     DO jk=1,klev
        DO jl=kidia,kfdia
           IF (pm6rp(jl,jk,jmod) > 0._JPRB) THEN

              !--- Diffusion coefficient (Reference???):

              zpbyone=1000.0_JPRB / (papp1(jl,jk)/100.0_JPRB)

              zde2=0.073_JPRB * zpbyone * (ptp1(jl,jk) / 298.15_JPRB)**1.5_JPRB  

              !--- Mean molecule velocity (Moore, 1962 (S+P equ. 8.2)):

              zvelb=SQRT(8.0_JPRB * rerg * ptp1(jl,jk) / rpi / wh2so4)  

              !--- ???Fuchs???

              zxibc=8.0_JPRB * zde2 / rpi / zvelb
              !
              ! Use count median radius:

              zm6rp=pm6rp(jl,jk,jmod)

              !--- Distance from particle up to which the kinetic regime applies:

              zf1=( (zm6rp + zxibc)**3 - (zm6rp**2 + zxibc**2)**1.5_JPRB ) / &
                &   (3.0_JPRB * zm6rp * zxibc) - zm6rp

              !--- Diffusive flux to single particle surface:
              !    (Elisabetta's thesis: fraction in equ. 2.26)

              zc2(jl,jk,jmod)=(4.0_JPRB * rpi * zde2 * zm6rp ) /                      &
                &            ((4.0_JPRB * zde2) / (zvelb * zm6rp * caccso4(jmod)) +   &
                &             (zm6rp/(zm6rp+zf1))                              )

              !--- Total diffusive flux to all particles in the respective mode:
              !    (per concentration of gas phase sulfate)

              zc2(jl,jk,jmod)=zc2(jl,jk,jmod) * paernl(jl,jk,jmod)

              !--- Total diffusive flux to all particles in all modes:
              !    (per concentration of gas phase sulfate)

              zcondo(jl,jk)=zcondo(jl,jk)+ zc2(jl,jk,jmod)  

           ENDIF
        ENDDO
     ENDDO
  ENDDO
  !
  !--- 2) Calculation of the new sulfate aerosol masses and of the ---------
  !       mass of sulfate condensing on the respective modes:
  !
  DO jk=1,klev
     DO jl=kidia,kfdia
        IF(zcondo(jl,jk) > 0._JPRB.AND.pso4g(jl,jk) > 1.e-10_JPRB) THEN

           !--- Total diffusive flux to all particles in all modes:

           zfcond=zcondo(jl,jk)*pso4g(jl,jk)

           !--- Total mass of sulfate condensing during 1 timestep:

           zftot=zfcond*ztsphy
           
           !--- Limit condensing sulfate to 
           !    fmax[%] x (available gas-phase sulfate) :

           zfcond=MIN(zftot,(pso4g(jl,jk)*fmax)) 

           !--- Remaining gas phase sulfate:
           
           pso4g(jl,jk)=pso4g(jl,jk)-zfcond
           
           !--- Add mass to sulfate compounds:
           !    zc2(:,:,jmod)/zcondo = fraction of total sulfate flux
           !                           that condenses on mode jmod
           !    => (   "    )*zfcond = mass of sulfate condensing on
           !                           the respective mode

           paerml(jl,jk,iso4ns)=paerml(jl,jk,iso4ns)+ &
             &                  zc2(jl,jk,iso4ns)/zcondo(jl,jk)*zfcond
           paerml(jl,jk,iso4ks)=paerml(jl,jk,iso4ks)+ &
             &                  zc2(jl,jk,iso4ks)/zcondo(jl,jk)*zfcond
           paerml(jl,jk,iso4as)=paerml(jl,jk,iso4as)+ &
             &                  zc2(jl,jk,iso4as)/zcondo(jl,jk)*zfcond
           paerml(jl,jk,iso4cs)=paerml(jl,jk,iso4cs)+ &
             &                  zc2(jl,jk,iso4cs)/zcondo(jl,jk)*zfcond

           !--- Mass of sulphate condensing on the insoluble modes:
           !    (Transfer from insoluble to soluble modes 
           !     calculated in m7_concoag.)
           
           pso4_5(jl,jk)=zc2(jl,jk,5)/zcondo(jl,jk)*zfcond
           pso4_6(jl,jk)=zc2(jl,jk,6)/zcondo(jl,jk)*zfcond
           pso4_7(jl,jk)=zc2(jl,jk,7)/zcondo(jl,jk)*zfcond

           !--- Vertically integrate mass of condensated sulfate for diagnostics
           !    Convert [molec. cm-3] to [kg(S) m-2]:
           
!no_diag           d_cond_so4(jl,krow)=d_cond_so4(jl,krow) + ((zfcond*32._JPRB*1.E3_JPRB)/avo)*pdz(jl,jk)&
!no_diag                               *zqtmst*ztsphy
!!!                               *zqtmst*delta_time   (funny bit in the original code)  !!!

        ELSE 

           pso4_5(jl,jk)=0._JPRB
           pso4_6(jl,jk)=0._JPRB
           pso4_7(jl,jk)=0._JPRB

        ENDIF
     ENDDO
  ENDDO

!--------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_DGAS',1,ZHOOK_HANDLE)
END SUBROUTINE m7_dgas
