SUBROUTINE m7_nucl_ve &
  &( KIDIA   , KFDIA, KLON       , KLEV , &  ! ECMWF dimensions
  &  ptp1    , prhd , pmolecH2SO4,        &  ! ECMWF temperature, relative humidity
  &  pxtrnucr, pntot                      )  ! nucleation rate, number of molecules in the
                                             ! critical cluster
    !
    !   Authors:
    !   ---------
    !   C. TIMMRECK, MPI HAMBURG                                             2002
 
    !   JJMorcrette, 20100115  Adaptation to ECMWF-IFS
    !
    !   Purpose
    !   ---------
    !   Calculation of classical nucleation rate
    !               
    !   calculation of the nucleation rate after Vehkamaeki et al. (2002)
    !   The calculation of the nucrate ZKNH2SO4 is in cm^-3 s^-1
    !   and a coarse approxmation for the first class
    !
    !   Modifications:
    !   --------------
    !   R. Hommel; rewrite in f90, adopted to ECHAM5;        MPI HAMBURG;      Dec. 2002
    !   P. Stier; bugfixes, modularisation and optimization; MPI HAMBURG;      2003-2004
    !
    !   H2SO4 still fixed to xxx molc/cm3, no sulfur cycle coupling yet
    !
    !   References:
    !   -----------
    !   Vehkamaeki et al. (2002), An improved parameterization for sulfuric
    !      acid/water nucleation rates for tropospheric and stratospheric
    !      conditions, J. Geophys. Res, 107, D22, 4622
    !
    !   Parameters
    !   ----------
    !   prho = prhop_neu in *sam*
    !
    !   prhd = relative humidity in sam_aeroprop & sam_nucl
    !
    !   pxtrnucr = nucleation rate in [1/m3s]
    !   xrhoc    = density of the critical nucleus in kg/m^3
    !   zrxc = ?

  !----------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  IMPLICIT NONE

  !----------------------------------------------------
  
INTEGER(KIND=JPIM),INTENT(IN) :: KIDIA, KFDIA, KLON, KLEV


REAL(KIND=JPRB),INTENT(IN) :: ptp1(klon,klev)       , prhd(klon ,klev), &
  &                           pmolecH2SO4(klon,klev)                    ! revisited, ok

REAL(KIND=JPRB),INTENT(OUT) :: pntot(klon ,klev), pxtrnucr(klon ,klev)
  
INTEGER(KIND=JPIM) :: jk, jl
  
  !----------------------------------------------------  
  ! Local Arrays
  
REAL(KIND=JPRB) :: zrxc(klon ) 

REAL(KIND=JPRB) :: zrhoa, zrh, zt, zx, zjnuc, zrc, zxmole, zntot


REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_NUCL_VE',0,ZHOOK_HANDLE)

  !--- 0) Initializations:

  DO jk=1, klev
     DO jl=KIDIA,KFDIA

  !----1.) Parameterization of  nucleation rate after Vehkamaeki et al. (2002)

        ! t: temperature in K (190.15-300.15K)                                  
        ! zrh: saturatio ratio of water (0.0001-1)                               
        ! zrhoa: sulfuric acid concentration in 1/cm3 (10^4-10^11 1/cm3)         
        ! jnuc: nucleation rate in 1/cm3s (10^-7-10^10 1/cm3s)                  
        ! ntot: total number of molecules in the critical cluster (ntot>4)      
        ! zx: molefraction of H2SO4 in the critical cluster                      
        ! rc: radius of the critical cluster in nm                              

        ! Calculate nucleation only for valid thermodynamic conditions:

        IF( (pmolecH2SO4(jl,jk) >= 1.E+4_JPRB) .AND. &
          &        (prhd(jl,jk) >=1.E-4_JPRB)  .AND. &
          &        (ptp1(jl,jk)>=190.15_JPRB   .AND. ptp1(jl,jk)<=300.15_JPRB) ) THEN

        zrhoa=MIN(pmolecH2SO4(jl,jk),1.e11_JPRB)
        zrh=MIN(prhd(jl,jk),1.0_JPRB)
        zt=ptp1(jl,jk)

        ! Equation (11) - molefraction of H2SO4 in the critical cluster

        zx=0.7409967177282139_JPRB - 0.002663785665140117_JPRB*zt   &
          & + 0.002010478847383187_JPRB*LOG(zrh)    &
          & - 0.0001832894131464668_JPRB*zt*LOG(zrh)    &
          & + 0.001574072538464286_JPRB*LOG(zrh)**2        &  
          & - 0.00001790589121766952_JPRB*zt*LOG(zrh)**2    &
          & + 0.0001844027436573778_JPRB*LOG(zrh)**3     &
          & -  1.503452308794887e-6_JPRB*zt*LOG(zrh)**3    &
          & - 0.003499978417957668_JPRB*LOG(zrhoa)   &
          & + 0.0000504021689382576_JPRB*zt*LOG(zrhoa)   

        zxmole=zx

        ! Equation (12) - nucleation rate in 1/cm3s

        zjnuc=0.1430901615568665_JPRB + 2.219563673425199_JPRB*zt -   &
          &   0.02739106114964264_JPRB*zt**2 +     &
          &   0.00007228107239317088_JPRB*zt**3 + 5.91822263375044_JPRB/zx +     &
          &   0.1174886643003278_JPRB*LOG(zrh) + 0.4625315047693772_JPRB*zt*LOG(zrh) -     &
          &   0.01180591129059253_JPRB*zt**2*LOG(zrh) +     &
          &   0.0000404196487152575_JPRB*zt**3*LOG(zrh) +    &
          &   (15.79628615047088_JPRB*LOG(zrh))/zx -     &
          &   0.215553951893509_JPRB*LOG(zrh)**2 -    &
          &   0.0810269192332194_JPRB*zt*LOG(zrh)**2 +     &
          &   0.001435808434184642_JPRB*zt**2*LOG(zrh)**2 -    & 
          &   4.775796947178588e-6_JPRB*zt**3*LOG(zrh)**2 -     &
          &   (2.912974063702185_JPRB*LOG(zrh)**2)/zx -   &
          &   3.588557942822751_JPRB*LOG(zrh)**3 +     &
          &   0.04950795302831703_JPRB*zt*LOG(zrh)**3 -     &
          &   0.0002138195118737068_JPRB*zt**2*LOG(zrh)**3 +    & 
          &   3.108005107949533e-7_JPRB*zt**3*LOG(zrh)**3 -     &
          &   (0.02933332747098296_JPRB*LOG(zrh)**3)/zx +     &
          &   1.145983818561277_JPRB*LOG(zrhoa) -    &
          &   0.6007956227856778_JPRB*zt*LOG(zrhoa) +    &
          &   0.00864244733283759_JPRB*zt**2*LOG(zrhoa) -    &
          &   0.00002289467254710888_JPRB*zt**3*LOG(zrhoa) -    &
          &   (8.44984513869014_JPRB*LOG(zrhoa))/zx +    &
          &   2.158548369286559_JPRB*LOG(zrh)*LOG(zrhoa) +   & 
          &   0.0808121412840917_JPRB*zt*LOG(zrh)*LOG(zrhoa) -    &
          &   0.0004073815255395214_JPRB*zt**2*LOG(zrh)*LOG(zrhoa) -   & 
          &   4.019572560156515e-7_JPRB*zt**3*LOG(zrh)*LOG(zrhoa) +    &
          &   (0.7213255852557236_JPRB*LOG(zrh)*LOG(zrhoa))/zx +    &
          &   1.62409850488771_JPRB*LOG(zrh)**2*LOG(zrhoa) -    &
          &   0.01601062035325362_JPRB*zt*LOG(zrh)**2*LOG(zrhoa) +   & 
          &   0.00003771238979714162_JPRB*zt**2*LOG(zrh)**2*LOG(zrhoa) +    &
          &   3.217942606371182e-8_JPRB*zt**3*LOG(zrh)**2*LOG(zrhoa) -    &
          &   (0.01132550810022116_JPRB*LOG(zrh)**2*LOG(zrhoa))/zx +    &
          &   9.71681713056504_JPRB*LOG(zrhoa)**2 -    &
          &   0.1150478558347306_JPRB*zt*LOG(zrhoa)**2 +    &
          &   0.0001570982486038294_JPRB*zt**2*LOG(zrhoa)**2 +    &
          &   4.009144680125015e-7_JPRB*zt**3*LOG(zrhoa)**2 +    &
          &   (0.7118597859976135_JPRB*LOG(zrhoa)**2)/zx -    &
          &   1.056105824379897_JPRB*LOG(zrh)*LOG(zrhoa)**2 +    &
          &   0.00903377584628419_JPRB*zt*LOG(zrh)*LOG(zrhoa)**2 -    &
          &   0.00001984167387090606_JPRB*zt**2*LOG(zrh)*LOG(zrhoa)**2 +    &
          &   2.460478196482179e-8_JPRB*zt**3*LOG(zrh)*LOG(zrhoa)**2 -    &
          &   (0.05790872906645181_JPRB*LOG(zrh)*LOG(zrhoa)**2)/zx -    &
          &   0.1487119673397459_JPRB*LOG(zrhoa)**3 +    &
          &   0.002835082097822667_JPRB*zt*LOG(zrhoa)**3 -    &
          &   9.24618825471694e-6_JPRB*zt**2*LOG(zrhoa)**3 +    &
          &   5.004267665960894e-9_JPRB*zt**3*LOG(zrhoa)**3 -    &
          &   (0.01270805101481648_JPRB*LOG(zrhoa)**3)/zx
        
        zjnuc=EXP(zjnuc)      !   add. Eq. (12) [1/(cm^3s)]      


        ! Equation (13) - total number of molecules in the critical cluster

        zntot=-0.002954125078716302_JPRB - 0.0976834264241286_JPRB*zt +   &
          &    0.001024847927067835_JPRB*zt**2 - 2.186459697726116e-6_JPRB*zt**3 -    &
          &    0.1017165718716887_JPRB/zx - 0.002050640345231486_JPRB*LOG(zrh) -   &
          &    0.007585041382707174_JPRB*zt*LOG(zrh) +    &
          &    0.0001926539658089536_JPRB*zt**2*LOG(zrh) -   &
          &    6.70429719683894e-7_JPRB*zt**3*LOG(zrh) -    &
          &    (0.2557744774673163_JPRB*LOG(zrh))/zx +   &
          &    0.003223076552477191_JPRB*LOG(zrh)**2 +   &
          &    0.000852636632240633_JPRB*zt*LOG(zrh)**2 -    &
          &    0.00001547571354871789_JPRB*zt**2*LOG(zrh)**2 +   &
          &    5.666608424980593e-8_JPRB*zt**3*LOG(zrh)**2 +    &
          &    (0.03384437400744206_JPRB*LOG(zrh)**2)/zx +   &
          &    0.04743226764572505_JPRB*LOG(zrh)**3 -    &
          &    0.0006251042204583412_JPRB*zt*LOG(zrh)**3 +   &
          &    2.650663328519478e-6_JPRB*zt**2*LOG(zrh)**3 -    &
          &    3.674710848763778e-9_JPRB*zt**3*LOG(zrh)**3 -   &
          &    (0.0002672510825259393_JPRB*LOG(zrh)**3)/zx -    &
          &    0.01252108546759328_JPRB*LOG(zrhoa) +   &
          &    0.005806550506277202_JPRB*zt*LOG(zrhoa) -    &
          &    0.0001016735312443444_JPRB*zt**2*LOG(zrhoa) +   &
          &    2.881946187214505e-7_JPRB*zt**3*LOG(zrhoa) +    &
          &    (0.0942243379396279_JPRB*LOG(zrhoa))/zx -   &
          &    0.0385459592773097_JPRB*LOG(zrh)*LOG(zrhoa) -   & 
          &    0.0006723156277391984_JPRB*zt*LOG(zrh)*LOG(zrhoa) +   &
          &    2.602884877659698e-6_JPRB*zt**2*LOG(zrh)*LOG(zrhoa) +    &
          &    1.194163699688297e-8_JPRB*zt**3*LOG(zrh)*LOG(zrhoa) -   &
          &    (0.00851515345806281_JPRB*LOG(zrh)*LOG(zrhoa))/zx -    &
          &    0.01837488495738111_JPRB*LOG(zrh)**2*LOG(zrhoa) +   &
          &    0.0001720723574407498_JPRB*zt*LOG(zrh)**2*LOG(zrhoa) -   & 
          &    3.717657974086814e-7_JPRB*zt**2*LOG(zrh)**2*LOG(zrhoa) -    &
          &    5.148746022615196e-10_JPRB*zt**3*LOG(zrh)**2*LOG(zrhoa) +    &
          &    (0.0002686602132926594_JPRB*LOG(zrh)**2*LOG(zrhoa))/zx -   &
          &    0.06199739728812199_JPRB*LOG(zrhoa)**2 +    &
          &    0.000906958053583576_JPRB*zt*LOG(zrhoa)**2 -   &
          &    9.11727926129757e-7_JPRB*zt**2*LOG(zrhoa)**2 -    &
          &    5.367963396508457e-9_JPRB*zt**3*LOG(zrhoa)**2 -   &
          &    (0.007742343393937707_JPRB*LOG(zrhoa)**2)/zx +    &
          &    0.0121827103101659_JPRB*LOG(zrh)*LOG(zrhoa)**2 -   &
          &    0.0001066499571188091_JPRB*zt*LOG(zrh)*LOG(zrhoa)**2 +    &
          &    2.534598655067518e-7_JPRB*zt**2*LOG(zrh)*LOG(zrhoa)**2 -    &
          &    3.635186504599571e-10_JPRB*zt**3*LOG(zrh)*LOG(zrhoa)**2 +    &
          &    (0.0006100650851863252_JPRB*LOG(zrh)*LOG(zrhoa)**2)/zx +   &
          &    0.0003201836700403512_JPRB*LOG(zrhoa)**3 -    &
          &    0.0000174761713262546_JPRB*zt*LOG(zrhoa)**3 +   &
          &    6.065037668052182e-8_JPRB*zt**2*LOG(zrhoa)**3 -    &
          &    1.421771723004557e-11_JPRB*zt**3*LOG(zrhoa)**3 +   &
          &    (0.0001357509859501723_JPRB*LOG(zrhoa)**3)/zx

        zntot=EXP(zntot)  !  add. Eq. (13)

          
        ! Equation (14) - radius of the critical cluster in nm

        zrc=EXP(-1.6524245_JPRB+0.42316402_JPRB*zx+0.33466487_JPRB*LOG(zntot))    ! [nm]

        ! Conversion [nm -> m]

        zrxc(jl)=zrc*1e-9_JPRB

        !----1.2) Limiter

        IF(zjnuc<1.e-7_JPRB .OR. zntot<4.0_JPRB) zjnuc=0.0_JPRB

        ! limitation to 1E+10 [1/cm3s]
      
        zjnuc=MIN(zjnuc,1.e10_JPRB)

        pxtrnucr(jl,jk) = zjnuc

        ! convert total number of molecules in the critical cluster
        ! to number of sulfate molecules:

        pntot(jl,jk)=zntot*zxmole

        ELSE ! pmolecH2SO4, ptp1 , prhd out of range

        pntot(jl,jk)   =0.0_JPRB
        pxtrnucr(jl,jk)=0.0_JPRB

        ENDIF

     ENDDO ! kproma
  ENDDO ! klev

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('M7_NUCL_VE',1,ZHOOK_HANDLE)
END SUBROUTINE m7_nucl_ve
