SUBROUTINE QNEGAT &
 & ( KIDIA, KFDIA, KLON, KLEV,&
 & PTSPHY,&
 & PQM1,  PQE,    PAPHP1,&
 & PFCQNG               )  

!**** *QNEGAT* - AVOIDS NEGATIVE VALUES FOR SPECIFIC HUMIDITY.

!     B.RITTER     E.C.M.W.F.     29/04/83.

!     PURPOSE.
!     --------

!          THIS ROUTINE RESETS TO A VERY SMALL MINIMUM VALUE ALL THE
!     SPECIFIC HUMIDITIES SMALLER THAN THIS (IN PARTICULAR ALL THE
!     NEGATIVE VALUES THAT MIGHT HAVE APPEARED IN THE COURSE OF THE
!     TIME STEP).

!**   INTERFACE.
!     ----------

!          *QNEGAT* IS CALLED FROM *CALLPAR*.
!          THE ROUTINE TAKES ITS INPUT FROM THE LONG-TERM STORAGE: Q
!     AT T+1 AND P ON LEVEL BOUNDARIES AT T+1. IT RETURNS ITS OUTPUT
!     TO THE SAME SPACE: CORRECTED Q TENDENCIES.
!     THE TEMPERATURE WILL NOT BE ADJUSTED AND SURFACE
!     FLUXES (EVAPORATION) WILL NOT BE MODIFIED.

!     METHOD.
!     -------

!          FOR EACH LAYER (FROM TOP TO BOTTOM) THE SPECIFIC HUMIDITY IS
!     COMPARED WITH ITS CRITICAL VALUE. IF NECESSARY THE ADJUSTMENT IS
!     PERFORMED AND THE NECESSARY WATER IS TAKEN FROM THE LAYER
!     UNDERNEATH.

!     EXTERNALS.
!     ----------

!          NONE.

!     REFERENCE.
!     ----------

!          SEE RELEVANT PART OF THE DOCUMENTATION.

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG
USE YOECND   , ONLY : REPQMI

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHP1(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFCQNG(KLON,KLEV+1) 

!*         flux outputs

! PFCQNG  : PSEUDO-FLUX OF WATER TO CORRECT FOR NEGATIVE HUMIDITY


REAL(KIND=JPRB) :: ZDP(KLON),                  ZDPO(KLON),                ZQOLD(KLON,KLEV)

INTEGER(KIND=JPIM) :: JK, JL

REAL(KIND=JPRB) :: ZCONS1, ZDQ, ZGDPH, ZTMST
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!*    COMPUTATIONAL CONSTANTS.
!     ------------- ----------

!     ZTMST=TWODT
!     IF (NSTEP.EQ.NSTART) ZTMST=0.5*TWODT
IF (LHOOK) CALL DR_HOOK('QNEGAT',0,ZHOOK_HANDLE)
ZTMST=PTSPHY

ZCONS1=1.0_JPRB/ZTMST
!***

!     ------------------------------------------------------------------

!*         2.     ADJUSTMENT FOR ALL LAYERS.
!                 ---------- --- --- -------

!*         2.1      SAVE OLD PQE VALUES.

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZQOLD(JL,JK)=PQE(JL,JK)
  ENDDO
ENDDO

!*         2.2      PRELIMINARY COMPUTATIONS FOR THE TOP LAYER.

DO JL=KIDIA,KFDIA
  ZDP(JL)=PAPHP1(JL,2)-PAPHP1(JL,1)
ENDDO

!*         2.3     VERTICAL LOOP EXCLUDING THE BOTTOM LAYER.

DO JK=1,KLEV-1
  DO JL=KIDIA,KFDIA
    ZDPO(JL)=ZDP(JL)
  ENDDO
  DO JL=KIDIA,KFDIA
    ZDP(JL)=PAPHP1(JL,JK+2)-PAPHP1(JL,JK+1)
    ZDQ=ZCONS1*MAX(0.0_JPRB,REPQMI-(PQM1(JL,JK)+ZTMST*PQE(JL,JK)))
    PQE(JL,JK)=PQE(JL,JK)+ZDQ
    ZDQ=ZDQ*ZDPO(JL)/ZDP(JL)
    PQE(JL,JK+1)=PQE(JL,JK+1)-ZDQ
  ENDDO
ENDDO

!*         2.4      BOTTOM LAYER.

DO JL=KIDIA,KFDIA
  ZDQ=ZCONS1*MAX(0.0_JPRB,REPQMI-(PQM1(JL,KLEV)+ZTMST*PQE(JL,KLEV)))
  PQE(JL,KLEV)=PQE(JL,KLEV)+ZDQ
ENDDO

!*         3.       FLUX COMPUTATIONS

DO JL=KIDIA,KFDIA
  PFCQNG(JL,1)=0.0_JPRB
ENDDO
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZGDPH = -RG  /(PAPHP1(JL,JK+1)-PAPHP1(JL,JK))
    PFCQNG(JL,JK+1)=(PQE(JL,JK)-ZQOLD(JL,JK))/ZGDPH + PFCQNG(JL,JK)
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('QNEGAT',1,ZHOOK_HANDLE)
END SUBROUTINE QNEGAT
