!OPTIONS XOPT(HSFUN)
SUBROUTINE RADDIAG ( KGL1, KGL2, KPROMA, KLEV,&
 & LDLAND,&
 & PALBD , PALBP , PAPRS , PCLFR,&
 & PCC   , PCH   , PCL   , PCM   , PCT,&
 & PEMIS , PEMIW , PEMIT,&
 & PGEMU , PMU0  , PSLM,&
 & PT    , PTS,&
 & PEMTD , PTRSO,&
 & PLWFC , PLWFT , PSWFC , PSWFT , PTH    , PTRSOC , PEMTEC )  

!**** *RADDIAG* - RADIATION DIAGNOSTICS ROUTINE

!     PURPOSE.
!     --------
!           TO PRODUCE AND WRITE OUT RADIATION DIAGNOSTICS

!**   INTERFACE.
!     ----------
!           CALLED FROM AMRADIN (METEO-FRANCE) AND RADDRV (ECMWF)

!        EXPLICIT ARGUMENTS :
!        --------------------
! KPROMA : VECTOR LENGTH
! KTDIA  : INDEX OF TOP LEVEL FROM WHICH COMPUTATIONS ARE ACTIVE
! KLEV   : NUMBER OF LEVELS

! LDLAND : (KPROMA)             ; .T. OVER LAND

! PALBD  : (KPROMA,NTSW)        ; DIFFUSE ALBEDO IN THE NSW SW INTERVALS
! PALBP  : (KPROMA,NTSW)        ; PARALLEL ALBEDO IN THE NSW SW INTERVALS
! PAPRS  : (KPROMA,KLEV+1)      ; HALF LEVEL PRESSURE
! PCLFR  : (KPROMA,KLEV )       ; CLOUD FRACTIONAL COVER
! PCC    : (KPROMA)             ; CONVECTIVE CLOUDINESS
! PCH    : (KPROMA)             ; HIGH  -LEVEL CLOUDINESS
! PCL    : (KPROMA)             ; LOW   -LEVEL CLOUDINESS
! PCM    : (KPROMA)             ; MEDIUM-LEVEL CLOUDINESS
! PCT    : (KPROMA)             ; TOTAL        CLOUDINESS
! PEMIS  : (KPROMA)             ; SURFACE EMISSIVITY OUTSIDE WINDOW
! PEMIW  : (KPROMA)             ; SURFACE WINDOW EMISSIVITY
! PEMIT  : (KPROMA)             ; TOTAL SURFACE EMISSIVITY
! PGEMU  : (KPROMA)             ; SINE OF LATITUDE ON THE REAL EARTH
! PMU0   : (KPROMA)             ; SOLAR ANGLE
! PSLM   : (KPROMA)             ; LAND-SEA MASK
! PT     : (KPROMA,KLEV)        ; FULL LEVEL TEMPERATURE
! PTS    : (KPROMA)             ; SURFACE TEMPERATURE
! PEMTD  : (KPROMA,KLEV+1)      ; CLEAR-SKY LONGWAVE FLUXES
! PTRSO  : (KPROMA,KLEV+1)      ; CLEAR-SKY LONGWAVE FLUXES
! PLWFC  : (KPROMA,2     )      ; CLEAR-SKY LONGWAVE FLUXES
! PLWFT  : (KPROMA,KLEV+1)      ; TOTAL-SKY LONGWAVE FLUXES
! PSWFC  : (KPROMA,2     )      ; CLEAR-SKY SHORTWAVE FLUXES
! PSWFT  : (KPROMA,KLEV+1)      ; TOTAL-SKY SHORTWAVE FLUXES
! PTH    : (KPROMA,KLEV+1)      ; HALF LEVEL TEMPERATURE
! PTRSOC : (KPROMA,KLEV+1)      ; CLEAR-SKY SHORTWAVE TRANSMISSIVITY
! PEMTEC : (KPROMA,KLEV+1)      ; CLEAR-SKY NET LONGWAVE EMISSIVITY

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHORS.
!     --------
!        ORIGINAL BY  GEORGE MOZDZYNSKI   *ECMWF*        94-12-21
!        (ESSENTIALLY DIAGNOSTICS CODE EXTRACTED FROM RADINT)

!     MODIFICATIONS.
!     --------------
!        96-06    : Th. LAGARDE      KST AND KEND FOR 2 CALLS TO PHYSICS
!        96-08: noticed by PhD: PEMTD and PEMTEC are now the LW fluxes    
!        97-06: fix a bug in the tests on PMU0 leading to 10 W/m**2 in 
!                the polar night (M. Deque after P. Raisanen)
!        99-07    : JJMorcrette clear-sky fluxes
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NDGLG    ,NDGNH
USE YOMCT0   , ONLY : N_REGIONS_NS    ,N_REGIONS_EW
USE YOMCT3   , ONLY : NSTEP
USE YOMGEM   , ONLY : NSTAGP
USE YOMPHY3  , ONLY : RII0
USE YOMCST   , ONLY : RPI      ,RDAY     ,RG       ,RSIGMA   ,RCPD
USE YOMLUN   , ONLY :    NULRAD
USE YOERAD   , ONLY : NRADPFR  ,NRADPLA  ,NSW      ,NTSW
USE YOECLD   , ONLY : REPSEC
USE YOESW    , ONLY : RSUN     ,D
USE YOERDI   , ONLY : RRAE
USE YOMMP    , ONLY : LSPLIT   ,MY_REGION_NS   ,MY_REGION_EW   ,NONL     ,&
 & NFRSTLAT ,NPTRFLOFF  

IMPLICIT NONE

!     EXTERNAL REAL FUNCTIONS
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGL2 
LOGICAL           ,INTENT(IN)    :: LDLAND(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD(KPROMA,NTSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP(KPROMA,NTSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KPROMA,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCC(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIW(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMU0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMTD(KPROMA,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTRSO(KPROMA,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLWFC(KPROMA,2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLWFT(KPROMA,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSWFC(KPROMA,2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSWFT(KPROMA,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTH(KPROMA,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTRSOC(KPROMA,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMTEC(KPROMA,KLEV+1) 
REAL(KIND=JPRB), EXTERNAL :: RDOT
REAL(KIND=JPRB), EXTERNAL :: RSUM

!-----------------------------------------------------------------------

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

!     ==== COMPUTED IN RADITE ===
!     ------------------------------------------------------------------

!*       0.2   LOCAL ARRAYS.
!              -------------

!- DIAGNOSTICS

REAL(KIND=JPRB) :: ZDIA(KPROMA)    , ZDIAC(3,16)  , ZDIAF(KPROMA,13)&
 & , ZDIAG(KLEV+1,12), ZDIAT(KLEV+1), ZDIA1(39)  

REAL(KIND=JPRB) :: ZALBTO(KPROMA), ZAMU0(KPROMA)

INTEGER(KIND=JPIM) :: I16, I3, IBEG, IDIAL, IDIAS, IEND, ILONS,&
 & INULRAD, IRADPFR, IRADPLA, IROW, ISTEP, IWH, &
 & J9, JGL, JGLGLO, JK, JKK, JL, JN, JSW  

LOGICAL :: LLDIA, LLDIAP

REAL(KIND=JPRB) :: ZBUD, ZCRAE, ZDEGDAY, ZDIAL, ZDIALS, ZDIAS,&
 & ZDIFT1, ZDIFT11, ZDIFT12, ZDIFT13, ZDIFT14, &
 & ZDIFT15, ZDIFT2, ZDIFT21, ZDIFT22, ZDIFT23, &
 & ZDIFT24, ZDIFT25, ZDIFT26, ZDIFT3, ZDIFT31, &
 & ZDIFT32, ZDIFT33, ZDIFT34, ZDIFT35, ZDIFT4, &
 & ZDIFT5, ZLATD, ZRII0, ZZDI, ZZDIL  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    PREPARATORY WORK
!              ----------------

!      print *,' Enter RADDIAG '

IF (LHOOK) CALL DR_HOOK('RADDIAG',0,ZHOOK_HANDLE)
ZRII0=RII0

!*       1.01    PARAMETERS EXTRACTED FROM COMDECKS (NON-TRANSPORTABLE)
!                ------------------------------------------------------

IRADPFR=NRADPFR
IRADPLA=NRADPLA
ISTEP  =NSTEP

!*       1.02    PARAMETERS FOR LOCAL DIAGNOSTICS
!                --------------------------------

ZBUD=1.0_JPRB

!*       1.03    LOCAL CONSTANTS
!                ---------------

ZCRAE=RRAE*(RRAE+2.0_JPRB)

!*       1.1     UNMERGE THE LATITUDE LINES IN KPROMA-LONG VECTOR
!                ------------------------------------------------

!     ------------------------------------------------------------------

DO JGL=KGL1,KGL2

  IF( NSTAGP(JGL) == 0 ) CYCLE
  IBEG=NSTAGP(JGL)
  IEND=NSTAGP(JGL)+NONL(NPTRFLOFF+JGL,MY_REGION_EW)-1
  ILONS=IEND-IBEG+1

!*         7.3   RECONSTRUCT FLUXES FOR DIAGNOSTICS

!3730  CONTINUE
  DO JL=IBEG,IEND
    IF (PMU0(JL) >= 1.E-10_JPRB) THEN
      ZAMU0(JL)=RRAE/(SQRT(PMU0(JL)**2+ZCRAE)-PMU0(JL))
    ELSE
      ZAMU0(JL)=0.0_JPRB
    ENDIF
  ENDDO
  DO JK=1,KLEV+1
    DO JL=IBEG,IEND
      PLWFT(JL,JK)=PEMTD(JL,JK)
      PSWFT(JL,JK)=ZAMU0(JL)*ZRII0*PTRSO(JL,JK)
    ENDDO
  ENDDO
  DO JL=IBEG,IEND
    PSWFC(JL, 1)=ZAMU0(JL)*ZRII0*PTRSOC(JL,1)
    PSWFC(JL, 2)=ZAMU0(JL)*ZRII0*PTRSOC(JL,KLEV+1)
    PLWFC(JL, 1)=PEMTEC(JL,1)
    PLWFC(JL, 2)=PEMTEC(JL,KLEV+1)
  ENDDO

! DETERMINE WHETHER FULL LATITUDES ARE AVAILABLE FOR FOLLOWING DIAGNOSTICS

  IF( LSPLIT.AND.N_REGIONS_NS > 1.OR. N_REGIONS_EW > 1 )THEN
    CYCLE
  ENDIF

!*       1.3     PREPARATION OF THE RADIATION DIAGNOSTICS
!                ----------------------------------------

  INULRAD=NULRAD
  LLDIAP=.FALSE.
  LLDIA =.FALSE.
  IF(IRADPFR /= 0) THEN
    LLDIA=MOD(ISTEP,IRADPFR) == 0
  ENDIF
  IF(IRADPLA /= 0.AND.LLDIA) THEN
    JGLGLO=NFRSTLAT(MY_REGION_NS)+JGL-1
    IF(JGLGLO <= NDGNH) THEN
      IROW=2*JGLGLO-1
    ELSE
      IROW=2*(NDGLG+1-JGLGLO)
    ENDIF
    LLDIAP=MOD(IABS(NDGLG+1-2*((IROW+1)/2)),IRADPLA) == 0
  ENDIF
  IF(LLDIA) THEN
    DO JN=1,12
      DO JK=1,KLEV+1
        ZDIAG(JK,JN)=0.0_JPRB
      ENDDO
    ENDDO
    DO JN=1,39
      ZDIA1(JN)=0.0_JPRB
    ENDDO
    IDIAL=0
    DO JL=IBEG,IEND
      IF(LDLAND(JL)) IDIAL=IDIAL+1
    ENDDO
    DO I3=1,3
      DO I16=1,16
        ZDIAC(I3,I16)=0.0_JPRB
      ENDDO
    ENDDO
    IDIAS=ILONS-IDIAL
    ZDIAL=0.0_JPRB
    ZDIAS=0.0_JPRB
    IF(IDIAL /= 0) ZDIAL=1.0_JPRB/IDIAL
    IF(IDIAS /= 0) ZDIAS=1.0_JPRB/IDIAS
    ZDIALS=1.0_JPRB/ILONS
  ENDIF

!*         1.4     INPUT DIAGNOSTICS FOR TEMPERATURE, CLOUDS, ALBEDO
!                  -------------------------------------------------

!*****
  IF(LLDIA) THEN
!*****
    ZZDI=RSUM(ILONS,PTS(IBEG),1)
    ZZDIL=RDOT(ILONS,PTS(IBEG),1,PSLM(IBEG),1)
    ZDIAG(1,9)=ZBUD*ZZDI
    ZDIAG(1,1)=ZDIAL*ZZDIL
    ZDIAG(1,2)=ZDIAS*(ZZDI-ZZDIL)
    DO JK=1,KLEV
      JKK=KLEV+2-JK
      ZZDI=RSUM(ILONS,PT(IBEG,JK),1)
      ZZDIL=RDOT(ILONS,PT(IBEG,JK),1,PSLM(IBEG),1)
      ZDIAG(JKK,9)=ZBUD*ZZDI
      ZDIAG(JKK,1)=ZDIAL*ZZDIL
      ZDIAG(JKK,2)=ZDIAS*(ZZDI-ZZDIL)
    ENDDO
    DO JL=IBEG,IEND
      ZDIA(JL)=1.0_JPRB-PCLFR(JL,1)
    ENDDO
    DO JK=2,KLEV
      DO JL=IBEG,IEND
        ZDIA(JL)=ZDIA(JL)*(1.0_JPRB-MAX(PCLFR(JL,JK),PCLFR(JL,JK-&
         & 1)))/(1.0_JPRB &
         & -MIN(PCLFR(JL,JK-1),1.0_JPRB-REPSEC))  
      ENDDO
    ENDDO
    DO JL=IBEG,IEND
      ZDIA(JL)=1.0_JPRB-ZDIA(JL)
    ENDDO
    ZZDI=RSUM(ILONS,ZDIA(IBEG),1)
    ZZDIL=RDOT(ILONS,ZDIA(IBEG),1,PSLM(IBEG),1)
    ZDIAG(1,10)=ZBUD*ZZDI
    ZDIAG(1,3)=ZDIAG(1,3)+ZDIAL*ZZDIL
    ZDIAG(1,4)=ZDIAG(1,4)+ZDIAS*(ZZDI-ZZDIL)
    DO JK=1,KLEV
      JKK=KLEV+2-JK
      ZZDI=RSUM(ILONS,PCLFR(IBEG,JK),1)
      ZZDIL=RDOT(ILONS,PCLFR(IBEG,JK),1,PSLM(IBEG),1)
      ZDIAG(JKK,10)=ZBUD*ZZDI
      ZDIAG(JKK,3)=ZDIAG(JKK,3)+ZDIAL*ZZDIL
      ZDIAG(JKK,4)=ZDIAG(JKK,4)+ZDIAS*(ZZDI-ZZDIL)
    ENDDO
    ZZDI=RSUM(ILONS,PCT(IBEG),1)
    ZZDIL=RDOT(ILONS,PCT(IBEG),1,PSLM(IBEG),1)
    ZDIAC(1,1)=ZBUD*ZZDI
    ZDIAC(2,1)=ZDIAC(2,1)+ZDIAL*ZZDIL
    ZDIAC(3,1)=ZDIAC(3,1)+ZDIAS*(ZZDI-ZZDIL)
    ZZDI=RSUM(ILONS,PCC(IBEG),1)
    ZZDIL=RDOT(ILONS,PCC(IBEG),1,PSLM(IBEG),1)
    ZDIAC(1,2)=ZBUD*ZZDI
    ZDIAC(2,2)=ZDIAC(2,2)+ZDIAL*ZZDIL
    ZDIAC(3,2)=ZDIAC(3,2)+ZDIAS*(ZZDI-ZZDIL)
    ZZDI=RSUM(ILONS,PCL(IBEG),1)
    ZZDIL=RDOT(ILONS,PCL(IBEG),1,PSLM(IBEG),1)
    ZDIAC(1,3)=ZBUD*ZZDI
    ZDIAC(2,3)=ZDIAC(2,3)+ZDIAL*ZZDIL
    ZDIAC(3,3)=ZDIAC(3,3)+ZDIAS*(ZZDI-ZZDIL)
    ZZDI=RSUM(ILONS,PCM(IBEG),1)
    ZZDIL=RDOT(ILONS,PCM(IBEG),1,PSLM(IBEG),1)
    ZDIAC(1,4)=ZBUD*ZZDI
    ZDIAC(2,4)=ZDIAC(2,4)+ZDIAL*ZZDIL
    ZDIAC(3,4)=ZDIAC(3,4)+ZDIAS*(ZZDI-ZZDIL)
    ZZDI=RSUM(ILONS,PCH(IBEG),1)
    ZZDIL=RDOT(ILONS,PCH(IBEG),1,PSLM(IBEG),1)
    ZDIAC(1,5)=ZBUD*ZZDI
    ZDIAC(2,5)=ZDIAC(2,5)+ZDIAL*ZZDIL
    ZDIAC(3,5)=ZDIAC(3,5)+ZDIAS*(ZZDI-ZZDIL)

    DO JSW=1,NSW
      ZZDI =RSUM(ILONS,PALBP(IBEG,JSW),1)
      ZZDIL=RDOT(ILONS,PALBP(IBEG,JSW),1,PSLM(IBEG),1)
      ZDIAC(2,5+JSW)=ZDIAC(2,5+JSW)+ZDIAL*ZZDIL
      ZDIAC(3,5+JSW)=ZDIAC(3,5+JSW)+ZDIAS*(ZZDI-ZZDIL)

      ZZDI =RSUM(ILONS,PALBD(IBEG,JSW),1)
      ZZDIL=RDOT(ILONS,PALBD(IBEG,JSW),1,PSLM(IBEG),1)
      ZDIAC(2,5+NSW+JSW)=ZDIAC(2,5+NSW+JSW)+ZDIAL*ZZDIL
      ZDIAC(3,5+NSW+JSW)=ZDIAC(3,5+NSW+JSW)+ZDIAS*(ZZDI-ZZDIL)
    ENDDO

    IWH=5+2*NSW
    ZZDI=RSUM(ILONS,PEMIS(IBEG),1)
    ZZDIL=RDOT(ILONS,PEMIS(IBEG),1,PSLM(IBEG),1)
    ZDIAC(1,IWH+1)=ZBUD*ZZDI
    ZDIAC(2,IWH+1)=ZDIAC(2,IWH+1)+ZDIAL*ZZDIL
    ZDIAC(3,IWH+1)=ZDIAC(3,IWH+1)+ZDIAS*(ZZDI-ZZDIL)

    ZZDI=RSUM(ILONS,PEMIW(IBEG),1)
    ZZDIL=RDOT(ILONS,PEMIW(IBEG),1,PSLM(IBEG),1)
    ZDIAC(1,IWH+2)=ZBUD*ZZDI
    ZDIAC(2,IWH+2)=ZDIAC(2,IWH+2)+ZDIAL*ZZDIL
    ZDIAC(3,IWH+2)=ZDIAC(3,IWH+2)+ZDIAS*(ZZDI-ZZDIL)

    ZZDI=RSUM(ILONS,PEMIT(IBEG),1)
    ZZDIL=RDOT(ILONS,PEMIT(IBEG),1,PSLM(IBEG),1)
    ZDIAC(1,IWH+3)=ZBUD*ZZDI
    ZDIAC(2,IWH+3)=ZDIAC(2,IWH+3)+ZDIAL*ZZDIL
    ZDIAC(3,IWH+3)=ZDIAC(3,IWH+3)+ZDIAS*(ZZDI-ZZDIL)
!*****
  ENDIF
!*****

!     ------------------------------------------------------------------

!*         6.     OUTPUT DIAGNOSTICS FOR HEATING RATES AND FLUXES.

!***
  IF(LLDIA) THEN
!***
    ZDEGDAY=RG*RDAY/RCPD
    DO JL=IBEG,IEND
      ZALBTO(JL)=0.0_JPRB
    ENDDO
    DO JSW=1,NSW
      DO JL=IBEG,IEND
        ZALBTO(JL)=ZALBTO(JL)+RSUN(JSW)*PALBD(JL,JSW)
      ENDDO
    ENDDO

    DO JL=IBEG,IEND
!         ZALBTO=RSUN(1)*PALBD(JL,1)+RSUN(2)*PALBD(JL,2)
      IF (PMU0(JL) >= 1.E-10_JPRB) THEN
        ZAMU0(JL)=RRAE/(SQRT(PMU0(JL)**2+ZCRAE)-PMU0(JL))
      ELSE
        ZAMU0(JL)=0.0_JPRB
      ENDIF
      ZDIAF(JL,1)=ZRII0*ZAMU0(JL)
      ZDIAF(JL,2)=ZDIAF(JL,1)*(1.0_JPRB-PTRSO(JL,1))
      ZDIAF(JL,3)=-PEMTD(JL,1)
      ZDIAF(JL,4)=ZAMU0(JL)*ZRII0*PTRSO(JL,KLEV+1)/(1.0_JPRB-ZALBTO(JL))
      ZDIAF(JL,5)=ZDIAF(JL,4)-ZAMU0(JL)*ZRII0*PTRSO(JL,KLEV+1)
      ZDIAF(JL,6)=(RSIGMA*PTH(JL,KLEV+1)**4+PEMTD(JL,KLEV+1)&
       & *(1.0_JPRB-PEMIS(JL))/PEMIS(JL))  
      ZDIAF(JL,7)=ZDIAF(JL,6)+PEMTD(JL,KLEV+1)

      ZDIAF(JL,8)=ZDIAF(JL,1)*(1.0_JPRB-PTRSOC(JL,1))
      ZDIAF(JL,9)=-PEMTEC(JL,1)
      ZDIAF(JL,10)=ZAMU0(JL)*ZRII0*PTRSOC(JL,KLEV+1)/(1.0_JPRB-ZALBTO(JL))
      ZDIAF(JL,11)=ZDIAF(JL,10)-ZAMU0(JL)*ZRII0*PTRSOC(JL,KLEV+1)
      ZDIAF(JL,12)=(RSIGMA*PTH(JL,KLEV+1)**4+PEMTEC(JL,KLEV+1)&
       & *(1.0_JPRB-PEMIT(JL))/PEMIT(JL))  
      ZDIAF(JL,13)=ZDIAF(JL,12)+PEMTEC(JL,KLEV+1)
    ENDDO
    DO JN=1,13
      ZZDI=RSUM(ILONS,ZDIAF(IBEG,JN),1)
      ZZDIL=RDOT(ILONS,ZDIAF(IBEG,JN),1,PSLM(IBEG),1)
      ZDIA1(26+JN)=ZBUD*ZZDI
      ZDIA1(2*JN-1)=ZDIA1(2*JN-1)+ZDIAL*ZZDIL
      ZDIA1(2*JN)=ZDIA1(2*JN)+ZDIAS*(ZZDI-ZZDIL)
    ENDDO
    DO JL=IBEG,IEND
      ZDIA(JL)=ZDEGDAY*ZAMU0(JL)*ZRII0*&
       & (PTRSO(JL,1)-PTRSO(JL,KLEV+1))&
       & /(PAPRS(JL,KLEV+1)-PAPRS(JL,1))  
    ENDDO
    ZZDI=RSUM(ILONS,ZDIA(IBEG),1)
    ZZDIL=RDOT(ILONS,ZDIA(IBEG),1,PSLM(IBEG),1)
    ZDIAG(1,11)=ZBUD*ZZDI
    ZDIAG(1,5)=ZDIAG(1,5)+ZDIAL*ZZDIL
    ZDIAG(1,6)=ZDIAG(1,6)+ZDIAS*(ZZDI-ZZDIL)
    DO JL=IBEG,IEND
      ZDIA(JL)=ZDEGDAY*(PEMTD(JL,1)-&
       & PEMTD(JL,KLEV+1))&
       & /(PAPRS(JL,KLEV+1)-PAPRS(JL,1))  
!         ZDIA(JL)=ZDEGDAY*RSIGMA*(PEMTD(JL,1)*PTH(JL,1)**4-
!     S                       PEMTD(JL,KLEV+1)*PTH(JL,KLEV+1)**4)
!     S         /(PAPRS(JL,KLEV+1)-PAPRS(JL,1))
    ENDDO
    ZZDI=RSUM(ILONS,ZDIA(IBEG),1)
    ZZDIL=RDOT(ILONS,ZDIA(IBEG),1,PSLM(IBEG),1)
    ZDIAG(1,12)=ZBUD*ZZDI
    ZDIAG(1,7)=ZDIAG(1,7)+ZDIAL*ZZDIL
    ZDIAG(1,8)=ZDIAG(1,8)+ZDIAS*(ZZDI-ZZDIL)
    DO JK=1,KLEV
      JKK=KLEV+2-JK
      DO JL=IBEG,IEND
        ZDIA(JL)=ZDEGDAY*ZAMU0(JL)*ZRII0*&
         & (PTRSO(JL,JK)-PTRSO(JL,JK+1))&
         & /(PAPRS(JL,JK+1)-PAPRS(JL,JK))  
      ENDDO
      ZZDI=RSUM(ILONS,ZDIA(IBEG),1)
      ZZDIL=RDOT(ILONS,ZDIA(IBEG),1,PSLM(IBEG),1)
      ZDIAG(JKK,11)=ZBUD*ZZDI
      ZDIAG(JKK,5)=ZDIAG(JKK,5)+ZDIAL*ZZDIL
      ZDIAG(JKK,6)=ZDIAG(JKK,6)+ZDIAS*(ZZDI-ZZDIL)
      DO JL=IBEG,IEND
        ZDIA(JL)=ZDEGDAY*(PEMTD(JL,JK)-&
         & PEMTD(JL,JK+1))&
         & /(PAPRS(JL,JK+1)-PAPRS(JL,JK))  
!         ZDIA(JL)=ZDEGDAY*RSIGMA*(PEMTD(JL,JK)*PTH(JL,JK)**4-
!     S                        PEMTD(JL,JK+1)*PTH(JL,JK+1)**4)
!     S         /(PAPRS(JL,JK+1)-PAPRS(JL,JK))
      ENDDO
      ZZDI=RSUM(ILONS,ZDIA(IBEG),1)
      ZZDIL=RDOT(ILONS,ZDIA(IBEG),1,PSLM(IBEG),1)
      ZDIAG(JKK,12)=ZBUD*ZZDI
      ZDIAG(JKK,7)=ZDIAG(JKK,7)+ZDIAL*ZZDIL
      ZDIAG(JKK,8)=ZDIAG(JKK,8)+ZDIAS*(ZZDI-ZZDIL)
    ENDDO

!***
  ENDIF
!***

!     ------------------------------------------------------------------

!*         7.     PRINT DIAGNOSTICS IF NECESSARY.
!                 ------ ---------- -- ----------

  IF(LLDIAP) THEN
!*LAND POINTS
    ZLATD=180._JPRB*ASIN(PGEMU(IBEG))/RPI
    DO JK=1,KLEV+1
      ZDIAT(JK)=ZDIAG(JK,5)+ZDIAG(JK,7)
    ENDDO
    ZDIFT1=ZDIA1(1)-ZDIA1(3)
    ZDIFT2=ZDIA1(7)-ZDIA1(9)
    ZDIFT3=ZDIA1(11)-ZDIA1(13)
    ZDIFT4=ZDIFT1-ZDIA1(5)
    ZDIFT5=ZDIFT2-ZDIFT3
    WRITE(INULRAD,911)
    WRITE(INULRAD,901)ZLATD
    WRITE(INULRAD,902)
    WRITE(INULRAD,903)(ZDIAG(JK,1),JK=1,KLEV+1)
    WRITE(INULRAD,914)(ZDIAC(2,J9),J9=1,9)
    WRITE(INULRAD,904)(ZDIAG(JK,3),JK=1,KLEV+1)
    WRITE(INULRAD,905)(ZDIAG(JK,5),JK=1,KLEV+1)
    WRITE(INULRAD,906)(ZDIAG(JK,7),JK=1,KLEV+1)
    WRITE(INULRAD,907)(ZDIAT(JK),JK=1,KLEV+1)
    WRITE(INULRAD,908)(ZDIA1(JN),JN=1,13,2),ZDIFT1,ZDIFT2,ZDIFT3 &
     & ,ZDIFT4,ZDIFT5  
!*cs
    ZDIFT11=ZDIA1(1)-ZDIA1(15)
    ZDIFT12=ZDIA1(19)-ZDIA1(21)
    ZDIFT13=ZDIA1(23)-ZDIA1(25)
    ZDIFT14=ZDIFT11-ZDIA1(17)
    ZDIFT15=ZDIFT12-ZDIFT13
    WRITE(INULRAD,915)(ZDIA1(JN),JN=15,25,2),ZDIFT11,ZDIFT12,&
     & ZDIFT13,ZDIFT14,ZDIFT15  
!*cdfrg
    ZDIFT21=ZDIA1(15)-ZDIA1(3)
    ZDIFT22=ZDIA1(17)-ZDIA1(5)
    ZDIFT23=ZDIA1(19)-ZDIA1(7)
    ZDIFT24=ZDIA1(21)-ZDIA1(9)
    ZDIFT25=ZDIA1(23)-ZDIA1(11)
    ZDIFT26=ZDIA1(25)-ZDIA1(13)
    ZDIFT31=ZDIFT11-ZDIFT1
    ZDIFT32=ZDIFT12-ZDIFT2
    ZDIFT33=ZDIFT13-ZDIFT3
    ZDIFT34=ZDIFT14-ZDIFT4
    ZDIFT35=ZDIFT15-ZDIFT5
    WRITE(INULRAD,916)ZDIFT21,ZDIFT22,ZDIFT23,ZDIFT24,ZDIFT25
    WRITE(INULRAD,914)(ZDIAC(2,J9),J9=1,5)
    WRITE(INULRAD,921)(ZDIAC(2,J9),J9=6,5+NSW)
    WRITE(INULRAD,922)(ZDIAC(2,J9),J9=5+NSW+1,5+2*NSW)
    WRITE(INULRAD,923)(ZDIAC(2,J9),J9=IWH+1,IWH+3)&
     & ,ZDIFT26,ZDIFT31,ZDIFT32,ZDIFT33,ZDIFT34,ZDIFT35  
!*OCEAN POINTS
    DO JK=1,KLEV+1
      ZDIAT(JK)=ZDIAG(JK,6)+ZDIAG(JK,8)
    ENDDO
    ZDIFT1=ZDIA1(2)-ZDIA1(4)
    ZDIFT2=ZDIA1(8)-ZDIA1(10)
    ZDIFT3=ZDIA1(12)-ZDIA1(14)
    ZDIFT4=ZDIFT1-ZDIA1(6)
    ZDIFT5=ZDIFT2-ZDIFT3
    WRITE(INULRAD,911)
    WRITE(INULRAD,909)
    WRITE(INULRAD,903)(ZDIAG(JK,2),JK=1,KLEV+1)
    WRITE(INULRAD,914)(ZDIAC(3,J9),J9=1,9)
    WRITE(INULRAD,904)(ZDIAG(JK,4),JK=1,KLEV+1)
    WRITE(INULRAD,905)(ZDIAG(JK,6),JK=1,KLEV+1)
    WRITE(INULRAD,906)(ZDIAG(JK,8),JK=1,KLEV+1)
    WRITE(INULRAD,907)(ZDIAT(JK),JK=1,KLEV+1)
    WRITE(INULRAD,908)(ZDIA1(JN),JN=2,14,2),ZDIFT1,ZDIFT2,ZDIFT3
    WRITE(INULRAD,914)(ZDIAC(3,J9),J9=1,5)
    WRITE(INULRAD,921)(ZDIAC(3,J9),J9=6,5+NSW)
    WRITE(INULRAD,922)(ZDIAC(3,J9),J9=5+NSW+1,5+2*NSW)
    WRITE(INULRAD,923)(ZDIAC(3,J9),J9=IWH+1,IWH+3),ZDIFT4,ZDIFT5
!*cs
    ZDIFT11=ZDIA1(2)-ZDIA1(16)
    ZDIFT12=ZDIA1(20)-ZDIA1(22)
    ZDIFT13=ZDIA1(24)-ZDIA1(26)
    ZDIFT14=ZDIFT11-ZDIA1(18)
    ZDIFT15=ZDIFT12-ZDIFT13
    WRITE(INULRAD,915)(ZDIA1(JN),JN=16,26,2),ZDIFT11,ZDIFT12,&
     & ZDIFT13,ZDIFT14,ZDIFT15  
!*cdfrg
    ZDIFT21=ZDIA1(16)-ZDIA1(4)
    ZDIFT22=ZDIA1(18)-ZDIA1(6)
    ZDIFT23=ZDIA1(20)-ZDIA1(8)
    ZDIFT24=ZDIA1(22)-ZDIA1(10)
    ZDIFT25=ZDIA1(24)-ZDIA1(12)
    ZDIFT26=ZDIA1(26)-ZDIA1(14)
    ZDIFT31=ZDIFT11-ZDIFT1
    ZDIFT32=ZDIFT12-ZDIFT2
    ZDIFT33=ZDIFT13-ZDIFT3
    ZDIFT34=ZDIFT14-ZDIFT4
    ZDIFT35=ZDIFT15-ZDIFT5
    WRITE(INULRAD,916)ZDIFT21,ZDIFT22,ZDIFT23,ZDIFT24,ZDIFT25 &
     & ,ZDIFT26,ZDIFT31,ZDIFT32,ZDIFT33,ZDIFT34,ZDIFT35  
!*ZONAL AVERAGES
    DO JN=1,7,2
      DO JK=1,KLEV+1
        ZDIAG(JK,JN)=(IDIAL*ZDIAG(JK,JN)+IDIAS*ZDIAG(JK,JN+1))*ZDIALS
      ENDDO
    ENDDO
    DO JN=1,25,2
      ZDIA1(JN)=(IDIAL*ZDIA1(JN)+IDIAS*ZDIA1(JN+1))*ZDIALS
    ENDDO
    DO JK=1,KLEV+1
      ZDIAT(JK)=ZDIAG(JK,5)+ZDIAG(JK,7)
    ENDDO
    ZDIFT1=ZDIA1(1)-ZDIA1(3)
    ZDIFT2=ZDIA1(7)-ZDIA1(9)
    ZDIFT3=ZDIA1(11)-ZDIA1(13)
    ZDIFT4=ZDIFT1-ZDIA1(5)
    ZDIFT5=ZDIFT2-ZDIFT3
    DO J9=1,5+2*NSW+3
      ZDIAC(1,J9)=(IDIAL*ZDIAC(2,J9)+IDIAS*ZDIAC(3,J9))*ZDIALS
    ENDDO
    WRITE(INULRAD,911)
    WRITE(INULRAD,910)
    WRITE(INULRAD,903)(ZDIAG(JK,1),JK=1,KLEV+1)
    WRITE(INULRAD,914)(ZDIAC(1,J9),J9=1,9)
    WRITE(INULRAD,904)(ZDIAG(JK,3),JK=1,KLEV+1)
    WRITE(INULRAD,905)(ZDIAG(JK,5),JK=1,KLEV+1)
    WRITE(INULRAD,906)(ZDIAG(JK,7),JK=1,KLEV+1)
    WRITE(INULRAD,907)(ZDIAT(JK),JK=1,KLEV+1)
    WRITE(INULRAD,908)(ZDIA1(JN),JN=1,13,2),ZDIFT1,ZDIFT2,ZDIFT3 &
     & ,ZDIFT4,ZDIFT5  
!*cs
    ZDIFT11=ZDIA1(1)-ZDIA1(15)
    ZDIFT12=ZDIA1(19)-ZDIA1(21)
    ZDIFT13=ZDIA1(23)-ZDIA1(25)
    ZDIFT14=ZDIFT11-ZDIA1(17)
    ZDIFT15=ZDIFT12-ZDIFT13
    WRITE(INULRAD,915)(ZDIA1(JN),JN=15,25,2),ZDIFT11,ZDIFT12,&
     & ZDIFT13,ZDIFT14,ZDIFT15  
!*cdfrg
    ZDIFT21=ZDIA1(15)-ZDIA1(3)
    ZDIFT22=ZDIA1(17)-ZDIA1(5)
    ZDIFT23=ZDIA1(19)-ZDIA1(7)
    ZDIFT24=ZDIA1(21)-ZDIA1(9)
    ZDIFT25=ZDIA1(23)-ZDIA1(11)
    ZDIFT26=ZDIA1(25)-ZDIA1(13)
    ZDIFT31=ZDIFT11-ZDIFT1
    ZDIFT32=ZDIFT12-ZDIFT2
    ZDIFT33=ZDIFT13-ZDIFT3
    ZDIFT34=ZDIFT14-ZDIFT4
    ZDIFT35=ZDIFT15-ZDIFT5
    WRITE(INULRAD,916)ZDIFT21,ZDIFT22,ZDIFT23,ZDIFT24,ZDIFT25
    WRITE(INULRAD,914)(ZDIAC(1,J9),J9=1,5)
    WRITE(INULRAD,921)(ZDIAC(1,J9),J9=6,5+NSW)
    WRITE(INULRAD,922)(ZDIAC(1,J9),J9=5+NSW+1,5+2*NSW)
    WRITE(INULRAD,923)(ZDIAC(1,J9),J9=IWH+1,IWH+3)&
     & ,ZDIFT26,ZDIFT31,ZDIFT32,ZDIFT33,ZDIFT34,ZDIFT35  
  ENDIF

!     ------------------------------------------------------------------

  1000 CONTINUE

  1001 continue
ENDDO

!      print *,' Just before coming out of RADDIAG '

!     -----------------------------------------------------------------

!*         9.     FORMAT STATEMENTS.
!                 ------ -----------

901 FORMAT (/,1X,5HLAT.=,F5.1,1X,27HRADIATION ZONALLY AVERAGED ,&
 & 54HDIAGNOSTICS:(TS,T(KLEV,1,-1)),(CLCTOT,CLC(KLEV,1,-1)),,&
 & 24H(SOLTOT,SOL(KLEV,1,-1)),/,12X,28H(THERMTOT,THERM(KLEV,1,-1)),,&
 & 62H(NETTOT,NET(KLEV,1,-1)) AND FLUXES(T=TOP,B=BOTTOM,D=DOWN,U=UP,,&
 & 30HS=SOLAR(D),L=THERMAL(U),N=NET))  
902 FORMAT (2X,21HMEAN OVER LAND POINTS)
903 FORMAT (3X,21HTEMPERATURE (DEG.K)  ,F8.1,2X,16F6.1,(/,34X,16F6.1))
904 FORMAT (3X,21HCLOUD COVER (NO UNIT),F8.3,2X,16F6.3,(/,34X,16F6.3))
905 FORMAT (3X,21HS.W.HEAT. (DEG.K/DAY),F8.2,2X,16F6.2,(/,34X,16F6.2))
906 FORMAT (3X,21HL.W.HEAT. (DEG.K/DAY),F8.2,2X,16F6.2,(/,34X,16F6.2))
907 FORMAT (3X,21HNET HEAT. (DEG.K/DAY),F8.2,2X,16F6.2,(/,34X,16F6.2))
908 FORMAT (3X,10HFLX(W/M/M),5H TSD=,F5.0,5H TSU=,F5.0,5H TLU=,F5.0,&
 & 5H BSD=,F5.0,5H BSU=,F5.0,5H BLU=,F5.0,5H BLD=,F5.0,5H TS =,F5.0,&
 & 5H BS =,F5.0,5H BL =,F5.0,5H TND=,F5.0,5H BND=,F5.0)  
909 FORMAT (2X,20HMEAN OVER SEA POINTS)
910 FORMAT (2X,20HMEAN OVER ALL POINTS)
911 FORMAT (/)
912 FORMAT (//)
914 FORMAT (3X,39HTOTAL, CONV., LOW, MEDIUM, HIGH CLOUD: ,5F8.4)

915 FORMAT (3X,10HFLX(W/M/M),10H CLEAR-SKY,5H TSU=,F5.0,5H TLU=,F5.0,&
 & 5H BSD=,F5.0,5H BSU=,F5.0,5H BLU=,F5.0,5H BLD=,F5.0,5H TS =,F5.0,&
 & 5H BS =,F5.0,5H BL =,F5.0,5H TND=,F5.0,5H BND=,F5.0)  
916 FORMAT (3X,10HFLX(W/M/M),10H CLOUD-FRG,5H TSU=,F5.0,5H TLU=,F5.0,&
 & 5H BSD=,F5.0,5H BSU=,F5.0,5H BLU=,F5.0,5H BLD=,F5.0,5H TS =,F5.0,&
 & 5H BS =,F5.0,5H BL =,F5.0,5H TND=,F5.0,5H BND=,F5.0)  

921 FORMAT (3X,28H SURFACE ALBEDO (PARALLEL): ,10F8.3)
922 FORMAT (3X,28H SURFACE ALBEDO (DIFFUSE) : ,10F8.3)
923 FORMAT (3X,28H SURFACE LW EMISSIVITY OWT: ,10F8.3)

!     -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RADDIAG',1,ZHOOK_HANDLE)
END SUBROUTINE RADDIAG
