SUBROUTINE RADINATL &
 & ( KIDIA , KFDIA , KLON , KLEV , KMODE,&
 & PALBD5, PALBP5, PAPRS5 , PAPRSF5,&
 & PCCNL5, PCCNO5, PCCO25 , PCLFR5,&
 & PGELAM, PCLON,  PSLON,&
 & PDP5,   PEMIR5, PEMIW5,&
 & PGEMU,  PMU0 ,  PQ5    , PQS5,&
 & PQIWP5, PQLWP5,&
 & PSLM5 , PT5   , PTS5,&
 & PEMTD5, PTRSO5,&
 & PTH5  , PTRSOD5,&
 & PALBD , PALBP , PAPRS  , PAPRSF,&
 & PCCNL , PCCNO , PCCO2  , PCLFR,&
 & PDP   , PEMIR , PEMIW,&
 & PQ    , PQS,&
 & PQIWP , PQLWP,&
 & PT    , PTS,&
 & PEMTD , PTRSO,&
 & PTH   , PTRSOD )  

!**** *RADINATL* - RADIATION INTERFACE TO RADIATION SCHEME USED
!                  DURING ASSIMILATION
!                  --> TANGENT LINEAR <--

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
! KLEV   : NUMBER OF LEVELS

! PAER5  : (KLON,6,KLEV)        ; OPTICAL THICKNESS OF THE AEROSOLS
! PALBD  : (KLON,NSW)           ; DIFFUSE ALBEDO IN THE NSW SW INTERVALS
! PALBP  : (KLON,NSW)           ; PARALLEL ALBEDO IN THE NSW SW INTERVALS
! PAPRS  : (KLON,KLEV+1)        ; HALF LEVEL PRESSURE
! PAPRSF : (KLON,KLEV )         ; FULL LEVEL PRESSURE
! PCCNL  : (KLON)               ; CCN CONCENTRATION OVER LAND
! PCCNO  : (KLON)               ; CCN CONCENTRATION OVER THE OCEAN
! PCCO2  :                      ; CONCENTRATION IN CO2 (PA/PA)
! PCLFR  : (KLON,KLEV )         ; CLOUD FRACTIONAL COVER
! PQO35  : (KLON,KLEV )         ; OZONE MIXING RATIO (MASS)
! PDP    : (KLON,KLEV)          ; LAYER PRESSURE THICKNESS
! PEMIR  : (KLON)               ; SURFACE EMISSIVITY OUTSIDE LW WINDOW
! PEMIW  : (KLON)               ; SURFACE LW WINDOW EMISSIVITY
! PGEMU  : (KLON)               ; SINE OF LATITUDE ON THE REAL EARTH
! PMU0   : (KLON)               ; SOLAR ANGLE
! PQ     : (KLON,KLEV )         ; SPECIFIC HUMIDITY PA/PA
! PQS    : (KLON,KLEV )         ; SATURATION SPECIFIC HUMIDITY PA/PA
! PQIWP  : (KLON,KLEV )         ; ICE    WATER KG/KG
! PQLWP  : (KLON,KLEV )         ; LIQUID WATER KG/KG
! PSLM5  : (KLON)               ; LAND-SEA MASK
! PT     : (KLON,KLEV)          ; FULL LEVEL TEMPERATURE
! PTS    : (KLON)               ; SURFACE TEMPERATURE
!     ==== OUTPUTS ===
! PEMTD (KLON,KLEV+1)           ; TOTAL DOWNWARD LONGWAVE EMISSIVITY
! PTRSO (KLON,KLEV+1)           ; TOTAL SHORTWAVE TRANSMISSIVITY
! PTH   (KLON,KLEV+1)           ; HALF LEVEL TEMPERATURE
! PTRSOD(KLON)                  ; TOTAL-SKY SURFACE SW TRANSMISSITY

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     MODIFICATIONS.
!     --------------
!        Original : 96-10-11 JJ MORCRETTE
!        Perturbations of constant arguments removed
!        (96-10-16 J.F. MAHFOUF)
!        Modified : 2000-02-25 M.Janiskova - adopted for revised
!                                            aerosols as in radina
!                   2000-11-21 M.Janiskova - safety checks modified
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!                   2008-03-18 M.Janiskova - commenting the computation
!                                            of diagnostic quantities
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY3  , ONLY : RII0
USE YOERAD   , ONLY : NAER     , NSW
USE YOEAERD  , ONLY : RCAEROS
USE YOERDU   , ONLY : NIMP     ,NOUT
USE YOERDI   , ONLY : RRAE     ,REPH2O
USE YOMPHY   , ONLY : LRAYFM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCNL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCNO5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCO25 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(KLON), PCLON(KLON), PSLON(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIR5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIW5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMU0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQIWP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLWP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSLM5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEMTD5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRSO5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRSOD5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRS(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPRSF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCNL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCNO(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCO2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIR(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIW(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQIWP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLWP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEMTD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRSO(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRSOD(KLON) 
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

!- trajectory values

!     ==== COMPUTED IN RADLSW ===

!- perturbations

!     ==== COMPUTED IN RADLSW ===

!     ------------------------------------------------------------------

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZQAER5(KLON,6,KLEV) , ZQOZ5(KLON,KLEV)

!REAL(KIND=JPRB) :: ZFRTHC5(KLON,KLEV+1), ZFRSOC5(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZFRTH5 (KLON,KLEV+1), ZFRSO5 (KLON,KLEV+1), ZFRSOD5(KLON)
REAL(KIND=JPRB) :: ZAMU05(KLON)
REAL(KIND=JPRB) :: ZCLFR5(KLON,KLEV)
REAL(KIND=JPRB) :: ZQ5(KLON,KLEV)    , ZQS5(KLON,KLEV)
REAL(KIND=JPRB) :: ZQIWP5(KLON,KLEV) , ZQLWP5(KLON,KLEV)
REAL(KIND=JPRB) :: ZQAER(KLON,6,KLEV), ZQOZ(KLON,KLEV)

!REAL(KIND=JPRB) :: ZFRTHC(KLON,KLEV+1), ZFRSOC(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZFRTH (KLON,KLEV+1), ZFRSO (KLON,KLEV+1), ZFRSOD(KLON)
REAL(KIND=JPRB) :: ZCLFR(KLON,KLEV)
REAL(KIND=JPRB) :: ZQ(KLON,KLEV)    , ZQS(KLON,KLEV)
REAL(KIND=JPRB) :: ZQIWP(KLON,KLEV) , ZQLWP(KLON,KLEV)

INTEGER(KIND=JPIM) :: JAER, JK, JL

REAL(KIND=JPRB) :: Z1S, Z1S5, Z2S, Z2S5, Z3S, Z3S5, ZCRAE, ZRII0, ZRII05
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "radaca.intfb.h"
#include "radlswtl.intfb.h"

!     ------------------------------------------------------------------

!*       1.    PREPARATORY WORK
!              ----------------

IF (LHOOK) CALL DR_HOOK('RADINATL',0,ZHOOK_HANDLE)
NIMP=0
NOUT=0
ZRII0  =0.0_JPRB
ZRII05 =RII0

!- various constants      

ZCRAE=RRAE*(RRAE+2.0_JPRB)

!     ------------------------------------------------------------------

!*         2.     HALF-LEVEL QUANTITIES

! 200 CONTINUE

DO JK=2,KLEV
  DO JL=KIDIA,KFDIA
    Z1S  = PAPRSF (JL,JK-1)*(PAPRSF5(JL,JK)-PAPRS5(JL,JK)) +&
     & PAPRSF5(JL,JK-1)*(PAPRSF (JL,JK)-PAPRS (JL,JK))  
    Z1S5 = PAPRSF5(JL,JK-1)*(PAPRSF5(JL,JK)-PAPRS5(JL,JK))
    Z2S  = PAPRSF (JL,JK)*(PAPRS5(JL,JK)-PAPRSF5(JL,JK-1)) +&
     & PAPRSF5(JL,JK)*(PAPRS (JL,JK)-PAPRSF (JL,JK-1))  
    Z2S5 = PAPRSF5(JL,JK)*(PAPRS5(JL,JK)-PAPRSF5(JL,JK-1))
    Z3S  = PAPRS (JL,JK)*(PAPRSF5(JL,JK)-PAPRSF5(JL,JK-1)) +&
     & PAPRS5(JL,JK)*(PAPRSF (JL,JK)-PAPRSF (JL,JK-1))  
    Z3S5 = PAPRS5(JL,JK)*(PAPRSF5(JL,JK)-PAPRSF5(JL,JK-1))
    PTH (JL,JK)=(PT (JL,JK-1)*Z1S5+PT (JL,JK)*Z2S5 +&
     & PT5(JL,JK-1)*Z1S +PT5(JL,JK)*Z2S )/Z3S5 -&
     & (PT5(JL,JK-1)*Z1S5+PT5(JL,JK)*Z2S5)*Z3S /&
     & Z3S5**2  
    PTH5(JL,JK)=(PT5(JL,JK-1)*Z1S5+PT5(JL,JK)*Z2S5)/Z3S5
  ENDDO
ENDDO

DO JL=KIDIA,KFDIA
  PTH (JL,1)= PT (JL,1)-PAPRSF (JL,1)*(PT5(JL,1)-PTH5(JL,&
   & 2))         &
   & /(PAPRSF5(JL,1)-PAPRS5(JL,2))  &
   & -PAPRSF5(JL,1)*(PT (JL,1)-PTH (JL,2))         &
   & /(PAPRSF5(JL,1)-PAPRS5(JL,2))  &
   & -PAPRSF5(JL,1)*(PT5(JL,1)-PTH5(JL,2))         &
   & *(-1)*(PAPRSF (JL,1)-PAPRS (JL,2))  &
   & /((PAPRSF5(JL,1)-PAPRS5(JL,2))**2)  

  PTH5(JL,1) =&
   & PT5(JL,1)-PAPRSF5(JL,1)&
   & *(PT5(JL,1)-PTH5(JL,2))         &
   & /(PAPRSF5(JL,1)-PAPRS5(JL,2))    

  PTH (JL,KLEV+1)= PTS (JL)
  PTH5(JL,KLEV+1)= PTS5(JL)
ENDDO

!     ------------------------------------------------------------------

!*         3.     SOLAR ZENITH ANGLE IS SAMPLED AND EARTH'S CURVATURE
!                                     CORRECTED

! 300 CONTINUE

DO JL=KIDIA,KFDIA
  IF (PMU0(JL) > 1.E-10_JPRB) THEN
    ZAMU05(JL)=RRAE/(SQRT(PMU0(JL)**2+ZCRAE)-PMU0(JL))
  ELSE
    ZAMU05(JL)= RRAE/SQRT(ZCRAE)
  ENDIF    
ENDDO

!*           3.3     SECURITY COMPUTATIONS FOR CRITICAL VARIABLES.

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    IF (PCLFR5(JL,JK) <= 0.0_JPRB) THEN
      ZCLFR (JL,JK) = 0.0_JPRB
      ZCLFR5(JL,JK) = 0.0_JPRB
    ELSEIF (PCLFR5(JL,JK) >= 1.0_JPRB) THEN
      ZCLFR (JL,JK) = 0.0_JPRB
      ZCLFR5(JL,JK) = 1.0_JPRB
    ELSE
      ZCLFR (JL,JK)=PCLFR (JL,JK)
      ZCLFR5(JL,JK)=PCLFR5(JL,JK)
    ENDIF

    IF (PQS5(JL,JK) <= 2.0_JPRB*REPH2O) THEN
      ZQS (JL,JK) = 0.0_JPRB
      ZQS5(JL,JK) = 2.0_JPRB*REPH2O
    ELSE
      ZQS (JL,JK)=PQS (JL,JK)
      ZQS5(JL,JK)=PQS5(JL,JK)
    ENDIF

    IF (PQ5(JL,JK) <= 2.0_JPRB*REPH2O) THEN
      ZQ (JL,JK) = 0.0_JPRB
      ZQ5(JL,JK) = 2.0_JPRB*REPH2O
    ELSEIF (PQ5(JL,JK) >= ZQS5(JL,JK)*(1.0_JPRB-REPH2O)) THEN
      ZQ (JL,JK) = ZQS (JL,JK)*(1.0_JPRB-REPH2O)
      ZQ5(JL,JK) = ZQS5(JL,JK)*(1.0_JPRB-REPH2O)
    ELSE
      ZQ (JL,JK)=PQ (JL,JK)
      ZQ5(JL,JK)=PQ5(JL,JK)
    ENDIF

    IF (PQLWP5(JL,JK) <= REPH2O) THEN
      ZQLWP (JL,JK) = 0.0_JPRB
      ZQLWP5(JL,JK) = REPH2O
    ELSE
      ZQLWP (JL,JK)=PQLWP (JL,JK)
      ZQLWP5(JL,JK)=PQLWP5(JL,JK)
    ENDIF
    IF (PQIWP5(JL,JK) <= REPH2O) THEN
      ZQIWP (JL,JK) = 0.0_JPRB
      ZQIWP5(JL,JK) = REPH2O
    ELSE
      ZQIWP (JL,JK)=PQIWP (JL,JK)
      ZQIWP5(JL,JK)=PQIWP5(JL,JK)
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         4.     OZONE AND AEROSOLS

IF(.NOT.LRAYFM) THEN
  CALL  RADACA ( KIDIA , KFDIA , KLON  , KLEV,&
   & PAPRS5, PGELAM, PGEMU, PCLON, PSLON, PTH5,&
   & ZQAER5, ZQOZ5 )  

  DO JAER = 1 , 6
    DO JK = 1 , KLEV
      DO JL = KIDIA,KFDIA
        ZQAER (JL,JAER,JK) = 0.0_JPRB
      ENDDO
    ENDDO
  ENDDO

!  IF (NOZOCL == 1.AND..NOT.LEPO3RA) THEN
!    CALL RADOZC ( KIDIA , KFDIA , KLON, KTDIA, KLEV &
!     &, 1     , KLON , 0  &
!     &, PAPRS5, PGEMU &
!     &, ZQOZ5 )
!  ENDIF

  DO JK = 1 , KLEV
    DO JL = KIDIA,KFDIA
      ZQOZ (JL,JK) = 0.0_JPRB
    ENDDO
  ENDDO

ENDIF

IF (NAER == 0) THEN
  DO JAER = 1 , 6
    DO JK = 1 , KLEV
      DO JL = KIDIA,KFDIA
        ZQAER (JL,JAER,JK) = 0.0_JPRB
        ZQAER5(JL,JAER,JK) = RCAEROS
      ENDDO
    ENDDO
  ENDDO
ELSE
  DO JK = 1 , KLEV
    DO JL = KIDIA,KFDIA
      ZQAER (JL,5,JK) = 0.0_JPRB
      ZQAER5(JL,5,JK) = RCAEROS
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

!*         5.      CALL TO ACTUAL RADIATION SCHEME (Tangent linear)

CALL RADLSWTL &
 & ( KIDIA , KFDIA , KLON , KLEV ,  KMODE , NAER,&
 & ZRII05,&
 & ZQAER5, PALBD5, PALBP5, PAPRS5, PAPRSF5,&
 & PCCNL5, PCCNO5,&
 & PCCO25, ZCLFR5, PDP5  , PEMIR5, PEMIW5,  ZQOZ5,&
 & PSLM5 , ZAMU05,&
 & ZQ5   , ZQIWP5, ZQLWP5, ZQS5  ,&
 & PTH5  , PT5   , PTS5,&
 & ZFRTH5, ZFRSO5, ZFRSOD5,&
 & ZQAER , PALBD , PALBP , PAPRS  , PAPRSF,&
 & PCCO2 , ZCLFR , PDP   , PEMIR  , PEMIW  , ZQOZ,&
 & ZQ    , ZQIWP , ZQLWP , ZQS    ,&
 & PTH   , PT    , PTS,&
 & ZFRTH , ZFRSO , ZFRSOD &
 & )   

!     ------------------------------------------------------------------

!*         6.     TRANSFORM FLUXES TO MODEL HISTORICAL VARIABLES

! 600 CONTINUE

DO JK=1,KLEV+1
  DO JL=KIDIA,KFDIA
    PTRSO (JL,JK)=ZFRSO (JL,JK)/(ZRII05*ZAMU05(JL))
    PTRSO5(JL,JK)=ZFRSO5(JL,JK)/(ZRII05*ZAMU05(JL))
    PEMTD (JL,JK)=ZFRTH (JL,JK)
    PEMTD5(JL,JK)=ZFRTH5(JL,JK)
  ENDDO
ENDDO

DO JL=KIDIA,KFDIA
  PTRSOD (JL)=ZFRSOD (JL)/(ZRII05*ZAMU05(JL))
  PTRSOD5(JL)=ZFRSOD5(JL)/(ZRII05*ZAMU05(JL))
ENDDO

! diagnostic clear sky quantities
!DO JL=KIDIA,KFDIA
!  PCTRSO (JL,1) =ZFRSOC (JL,1)/(ZRII05*ZAMU05(JL))
!  PCTRSO5(JL,1) =ZFRSOC5(JL,1)/(ZRII05*ZAMU05(JL))
!  PCTRSO (JL,2) =ZFRSOC (JL,KLEV+1)/(ZRII05*ZAMU05(JL))
!  PCTRSO5(JL,2) =ZFRSOC5(JL,KLEV+1)/(ZRII05*ZAMU05(JL))

!  PCEMTR (JL,1)=ZFRTHC (JL,     1)
!  PCEMTR5(JL,1)=ZFRTHC5(JL,     1)
!  PCEMTR (JL,2)=ZFRTHC (JL,KLEV+1)
!  PCEMTR5(JL,2)=ZFRTHC5(JL,KLEV+1)
!ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RADINATL',1,ZHOOK_HANDLE)
END SUBROUTINE RADINATL
