SUBROUTINE RADLSWTL &
 & (  KIDIA , KFDIA  , KLON  , KLEV   , KMODE, KAER,&
 & PRII05,&
 & PAER5 , PALBD5 , PALBP5, PAPH5 , PAP5,&
 & PCCNL5, PCCNO5,&
 & PCCO25, PCLFR5 , PDP5  , PEMIR5, PEMIW5 , POZON5,&
 & PLSM5 , PMU05,&
 & PQ5   , PQIWP5 , PQLWP5, PQS5  ,&
 & PTH5  , PT5    , PTS5,&
 & PFLT5 , PFLS5  , PFRSOD5,&
 & PAER  , PALBD  , PALBP , PAPH  , PAP,&
 & PCCO2 , PCLFR  , PDP   , PEMIR , PEMIW  , POZON,&
 & PQ    , PQIWP  , PQLWP , PQS   ,&
 & PTH   , PT     , PTS,&
 & PFLT  , PFLS   , PFRSOD    &
 & )  

!**** *RADLSWTL* - INTERFACE TO ECMWF LW AND SW RADIATION SCHEMES
!                  (Tangent-linear)

!     PURPOSE.
!     --------
!           CONTROLS RADIATION COMPUTATIONS

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------

! PAER   : (KLON,6,KLEV)     ; OPTICAL THICKNESS OF THE AEROSOLS
! PALBD  : (KLON,NSW)        ; SURF. SW ALBEDO FOR DIFFUSE RADIATION
! PALBP  : (KLON,NSW)        ; SURF. SW ALBEDO FOR PARALLEL RADIATION
! PAPH   : (KLON,KLEV+1)     ; HALF LEVEL PRESSURE
! PAP    : (KLON,KLEV)       ; FULL LEVEL PRESSURE
! PCCO2  :                   ; CONCENTRATION IN CO2 (KG/KG)
! PCLFR  : (KLON,KLEV)       ; CLOUD FRACTIONAL COVER
! PDP    : (KLON,KLEV)       ; LAYER PRESSURE THICKNESS
! PEMIR  : (KLON)            ; SURFACE EMISSIVITY OUTSIDE LW WINDOW
! PEMIW  : (KLON)            ; SURFACE LW WINDOW EMISSIVITY
! PLSM5  : (KLON)            ; LAND-SEA MASK
! PMU05   : (KLON)           ; SOLAR ANGLE
! POZON  : (KLON,KLEV)       ; OZONE AMOUNT IN LAYER (KG/KG*PA)
! PQ     : (KLON,KLEV)       ; SPECIFIC HUMIDITY KG/KG
! PQIWP  : (KLON,KLEV)       ; SOLID  WATER KG/KG
! PQLWP  : (KLON,KLEV)       ; LIQUID WATER KG/KG
! PQS    : (KLON,KLEV)       ; SATURATION WATER VAPOR  KG/KG
! PTH    : (KLON,KLEV+1)     ; HALF LEVEL TEMPERATURE
! PT     : (KLON,KLEV)       ; FULL LEVEL TEMPERATURE
! PTS    : (KLON)            ; SURFACE TEMPERATURE

!     ==== OUTPUTS ===

! PFLT   : (KLON,KLEV+1)     ; TOTAL LW NET FLUXES
! PFLS   : (KLON,KLEV+1)     ; TOTAL SW NET FLUXES
! PFRSOD : (KLON)            ; TOTAL-SKY SURFACE SW DOWNWARD FLUX

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHORS.
!     --------
!        J.-F. MAHFOUF          *ECMWF*

!        adapted from 

!        J.-J. MORCRETTE        *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!        ORIGINAL       : 97-07-01
!        CY18r3 version : 97-12-16         

!        Modified: 2000-02-25 M.Janiskova - adoption for modifications
!                                    for effect.radius, inhomogenity 
!                                    factors and RRTM-140gp
!                  JJMorcrette 011029 CCNs

!                  2002-05-08 M. Janiskova - nearly completely re-written
!                                    due to all modifications of JJM      
!                                          - call for linearized LW   
!                  JJMorcrette 041202 new ice optical properties 
!                  2006-02-03 M. Janiskova - call to LWTL instead of LWPTL
!                  2008-04-18 M. Janiskova - cleaning & modified computation
!                                            of net fluxes
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RD       ,RTT      ,RPI
USE YOERAD   , ONLY : NSW      ,LRRTM    ,LCCNL    , LCCNO  ,LDIFFC  ,&
 & NRADIP , NRADLP , NICEOPT, NLIQOPT, NINHOM, RCCNLND, RCCNSEA, &
 & RLWINHF, RSWINHF, RRe2De  
USE YOELW    , ONLY : NSIL     ,NTRA     ,NUA      ,TSTAND   ,XP
USE YOESW    , ONLY : RYFWCA   ,RYFWCB   ,RYFWCC   ,RYFWCD   ,&
 & RYFWCE   ,RYFWCF   ,REBCUA   ,REBCUB   ,REBCUC   ,&
 & REBCUD   ,REBCUE   ,REBCUF   ,REBCUI   ,REBCUJ   ,&
 & REBCUG   ,REBCUH   ,RHSAVI   ,RFULIO   ,RFLAA0   ,&
 & RFLAA1   ,RFLBB0   ,RFLBB1   ,RFLBB2   ,RFLBB3   ,&
 & RFLCC0   ,RFLCC1   ,RFLCC2   ,RFLCC3   ,RFLDD0   ,&
 & RFLDD1   ,RFLDD2   ,RFLDD3   ,RFUETA   ,RFUETB   ,RFUETC  ,RASWCA   ,&
 & RASWCB   ,RASWCC   ,RASWCD   ,RASWCE   ,RASWCF   ,&
 & RFUAA0   ,RFUAA1   ,RFUBB0   ,RFUBB1   ,RFUBB2   ,&
 & RFUBB3   ,RFUCC0   ,RFUCC1   ,RFUCC2   ,RFUCC3   ,&
 & RLILIA   ,RLILIB
USE YOERDU   , ONLY : NUAER    ,NTRAER   ,REPLOG   ,REPSC    ,REPSCW   ,DIFF
USE YOETHF   , ONLY : RTICE
USE YOPHNC   , ONLY : LERADSW2 ,LERADN2  ,LERADLW2
USE YOEPHLI  , ONLY : LPHYLIN
USE YOERRTWN , ONLY : DELWAVE, TOTPLNK
USE YOMCT3   , ONLY : NSTEP
USE YOMCT0   , ONLY : NSTOP
USE YOELWRAD , ONLY : NLEVLWC, NLWFR,  LWLCLHR 

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE
INTEGER(KIND=JPIM),INTENT(IN)    :: KAER 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRII05 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER5(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP5(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCNL5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCNO5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCO25 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIR5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIW5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POZON5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMU05(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQIWP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLWP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTH5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLT5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLS5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOD5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBD(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALBP(KLON,NSW) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCO2 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIR(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIW(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POZON(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQIWP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQLWP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQS(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTH(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLT(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLS(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRSOD(KLON) 
!-----------------------------------------------------------------------

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------
!     ==== COMPUTED IN RADLSW ===
!     -----------------------------------------------------------------

!*       0.2   LOCAL ARRAYS.
!              -------------
!     -----------------------------------------------------------------

!-- ARRAYS FOR LOCAL VARIABLES -----------------------------------------

!   1.0  Trajectory arrays 

REAL(KIND=JPRB) ::&
 & ZALBD5(KLON,NSW)    , ZALBP5(KLON,NSW)&
 & , ZCG5(KLON,NSW,KLEV) , ZOMEGA5(KLON,NSW,KLEV)&
 & , ZTAU5 (KLON,NSW,KLEV) &
 & , ZTAUCLD5(KLON,KLEV,16)  
REAL(KIND=JPRB) ::&
 & ZCLDLD5(KLON,KLEV)  , ZCLDLU5(KLON,KLEV)&
 & , ZCLDSW5(KLON,KLEV)  , ZCLD05(KLON,KLEV)&
 & , ZDT05(KLON)        &
 & , ZEMIR5(KLON)        , ZEMIW5(KLON)&
 & , ZFLUX5 (KLON,2,KLEV+1)                 , ZFLUC5(KLON,2,KLEV+1)&
 & , ZFIWP5(KLON)        , ZFLWP5(KLON)      , ZFRWP5(KLON)&
 & , ZIWC5(KLON)         , ZLWC5(KLON)&
 & , ZMU05(KLON)         , ZOZ5(KLON,KLEV)  , ZPMB5(KLON,KLEV+1) &
 & , ZPSOL5(KLON)&
 & , ZTAVE5 (KLON,KLEV)  , ZTL5(KLON,KLEV+1)&
 & , ZVIEW5(KLON)  
REAL(KIND=JPRB) ::&
 & ZFCDWN5(KLON,KLEV+1), ZFCUP5(KLON,KLEV+1)&
 & , ZFSDWN5(KLON,KLEV+1), ZFSUP5(KLON,KLEV+1)
REAL(KIND=JPRB) ::&
 & ZALFICE5(KLON)      , ZGAMICE5(KLON)     , ZBICE5(KLON)&
 & , ZDESR5(KLON)        , ZRADIP5(KLON)      , ZRADLP5(KLON) &
 !cc           , ZRADRD5(KLON)
 & , ZRAINT5(KLON)       , ZRES5(KLON)&
 & , ZTICE5(KLON)        , ZEMIT5(KLON)  
REAL(KIND=JPRB) :: ZBICFU5(KLON), ZKICFU5(KLON)
REAL(KIND=JPRB) :: ZFACT5(KLON)
REAL(KIND=JPRB) :: ZCOND5(KLON,KLEV)

!   2.0 Perturbation arrays

REAL(KIND=JPRB) ::&
 & ZALBD(KLON,NSW)    , ZALBP(KLON,NSW)&
 & , ZCG(KLON,NSW,KLEV) , ZOMEGA(KLON,NSW,KLEV)&
 & , ZTAU (KLON,NSW,KLEV) &
 & , ZTAUCLD(KLON,KLEV,16)  
REAL(KIND=JPRB) ::&
 & ZCLDLD(KLON,KLEV)  , ZCLDLU(KLON,KLEV)&
 & , ZCLDSW(KLON,KLEV)  , ZCLD0(KLON,KLEV)&
 & , ZDT0(KLON)        &
 & , ZEMIR(KLON)        , ZEMIW(KLON)&
 & , ZFLUX (KLON,2,KLEV+1)                 , ZFLUC(KLON,2,KLEV+1)&
 & , ZFIWP(KLON)        , ZFLWP(KLON)      , ZFRWP(KLON)&
 & , ZIWC(KLON)         , ZLWC(KLON)&
 & , ZMU0(KLON)         , ZOZ(KLON,KLEV)   , ZPMB(KLON,KLEV+1)  , ZPSOL(KLON)&
 & , ZTAVE (KLON,KLEV)  , ZTL(KLON,KLEV+1)&
 & , ZVIEW(KLON)  
REAL(KIND=JPRB) ::&
 & ZFCDWN(KLON,KLEV+1), ZFCUP(KLON,KLEV+1)&
 & , ZFSDWN(KLON,KLEV+1), ZFSUP(KLON,KLEV+1)
REAL(KIND=JPRB) ::&
 & ZALFICE(KLON)      , ZGAMICE(KLON)     , ZBICE(KLON)&
 & , ZDESR(KLON)        , ZRADIP(KLON)      , ZRADLP(KLON)&
 !cc           , ZRADRD(KLON)
 & , ZRAINT(KLON)       , ZRES(KLON)&
 & , ZTICE(KLON)        , ZEMIT(KLON)  
REAL(KIND=JPRB) :: ZBICFU (KLON), ZKICFU (KLON)

REAL(KIND=JPRB) :: ZCLFR(KLON,KLEV)
INTEGER(KIND=JPIM) :: IKL, JK, JKL, JKLP1, JKP1, JL, JNU, JSW,JRTM

REAL(KIND=JPRB) ::  ZRII0,  &
 & ZASYMX, ZASYMX5, ZDIFFD, ZDIFFD5, ZGI, &
 & ZGI5, ZGL, ZGL5, ZGR, ZGR5, ZIWGKG, ZIWGKG5, &
 & ZLWGKG, ZLWGKG5, ZMSAID, ZMSAID5, ZMSAIU, &
 & ZMSAIU5, ZMSALD, ZMSALD5, ZMSALU, ZMSALU5, ZRSAID, ZRSAID5, ZRSALD, &
 & ZRSALD5, ZMULTL, ZMULTL5, &
 & ZOI, ZOI5, ZOL, ZOL5, ZOMGMX, ZOMGMX5, ZOR, ZOR5, &
 & ZRMUZ, ZRWGKG, ZRWGKG5, ZTAUD, ZTAUD5, ZTAUMX, &
 & ZTAUMX5, ZTEMPC, ZTEMPC5, ZTOI, &
 & ZTOI5, ZTOL, ZTOL5, ZTOR, ZTOR5, ZZFIWP, &
 & ZZFIWP5, ZZFLWP, ZZFLWP5, ZDPOG, ZDPOG5, &
 & ZPODT, ZPODT5  
REAL(KIND=JPRB) :: ZALND, ZASEA, ZD, ZDEN, ZNTOT, ZNUM, ZRATIO,&
 & ZBETAI, ZOMGI, ZOMGP, ZFDEL  
REAL(KIND=JPRB) :: ZNUM5, ZRATIO5, &
 & ZBETAI5, ZOMGI5, ZOMGP5, ZFDEL5  
REAL(KIND=JPRB) :: ZAIWC , ZBIWC , ZFSR , ZTCELS, Z1RADI, Z1RADL, ZMULTI
REAL(KIND=JPRB) :: ZAIWC5, ZBIWC5, ZFSR5, ZTCELS5, Z1RADI5, Z1RADL5, ZMULTI5
REAL(KIND=JPRB) :: ZADDPLK

REAL(KIND=JPRB) :: ZEXPRS , ZTBLAY(KLON) , ZPLANCK , ZMABSD
REAL(KIND=JPRB) :: ZEXPRS5, ZTBLAY5(KLON), ZPLANCK5, ZMABSD5
REAL(KIND=JPRB) :: ZEXTCF , Z1MOMG  , ZDefRe, ZRefDe
REAL(KIND=JPRB) :: ZEXTCF5, Z1MOMG5
REAL(KIND=JPRB) :: ZRSAIE , ZRSAIA , ZRSAIG , ZRSAIF
REAL(KIND=JPRB) :: ZRSAIE5, ZRSAIA5, ZRSAIG5, ZRSAIF5

INTEGER(KIND=JPIM) :: INDLAY(KLON)
INTEGER(KIND=JPIM) :: ILEV(KLON),ILEVR,ITOPC
INTEGER(KIND=JPIM) :: ISTEP

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "lwptl.intfb.h"
#include "lwtl.intfb.h"
#include "swtl.intfb.h"

!     -----------------------------------------------------------------

!*         1.     SET-UP INPUT QUANTITIES FOR RADIATION
!                 -------------------------------------

IF (LHOOK) CALL DR_HOOK('RADLSWTL',0,ZHOOK_HANDLE)
ZRefDe = RRe2De
ZDefRe = 1.0_JPRB / ZRefDe

ZRII0 = 0.0_JPRB

DO JL = KIDIA,KFDIA
  ZFCUP (JL,KLEV+1) = 0.0_JPRB
  ZFCUP5(JL,KLEV+1) = 0.0_JPRB
  ZFCDWN (JL,KLEV+1) = 0.0_JPRB
  ZFCDWN5(JL,KLEV+1) = REPLOG
  ZFSUP (JL,KLEV+1) = 0.0_JPRB
  ZFSUP5(JL,KLEV+1) = 0.0_JPRB
  ZFSDWN (JL,KLEV+1) = 0.0_JPRB
  ZFSDWN5(JL,KLEV+1) = REPLOG
  ZFLUX (JL,1,KLEV+1) = 0.0_JPRB
  ZFLUX5(JL,1,KLEV+1) = 0.0_JPRB
  ZFLUX (JL,2,KLEV+1) = 0.0_JPRB
  ZFLUX5(JL,2,KLEV+1) = 0.0_JPRB
  ZFLUC (JL,1,KLEV+1) = 0.0_JPRB
  ZFLUC5(JL,1,KLEV+1) = 0.0_JPRB
  ZFLUC (JL,2,KLEV+1) = 0.0_JPRB
  ZFLUC5(JL,2,KLEV+1) = 0.0_JPRB
  ZPSOL (JL) = PAPH (JL,KLEV+1)
  ZPSOL5(JL) = PAPH5(JL,KLEV+1)
  ZPMB (JL,1) = ZPSOL (JL) / 100.0_JPRB
  ZPMB5(JL,1) = ZPSOL5(JL) / 100.0_JPRB
  ZDT0 (JL) = PTS (JL) - PTH (JL,KLEV+1)
  ZDT05(JL) = PTS5(JL) - PTH5(JL,KLEV+1)
ENDDO

!*         1.1    INITIALIZE VARIOUS FIELDS
!                 -------------------------

DO JSW=1,NSW
  DO JL = KIDIA,KFDIA
    ZALBD (JL,JSW)=PALBD (JL,JSW)
    ZALBD5(JL,JSW)=PALBD5(JL,JSW)
    ZALBP (JL,JSW)=PALBP (JL,JSW)
    ZALBP5(JL,JSW)=PALBP5(JL,JSW)
  ENDDO
ENDDO
DO JL = KIDIA,KFDIA
  ZEMIR (JL)  =PEMIR (JL)
  ZEMIR5(JL)  =PEMIR5(JL)
  ZEMIW (JL)  =PEMIW (JL)
  ZEMIW5(JL)  =PEMIW5(JL)
  ZMU0 (JL)   =0.0_JPRB
  ZMU05(JL)   =PMU05(JL)
ENDDO

DO JK = 1 , KLEV
  JKP1 = JK + 1
  JKL = KLEV+ 1 - JK
  JKLP1 = JKL + 1
  DO JL = KIDIA,KFDIA
    ZPMB (JL,JK+1)=PAPH (JL,JKL)/100.0_JPRB
    ZPMB5(JL,JK+1)=PAPH5(JL,JKL)/100.0_JPRB
!-- ZOZ in cm.atm for SW scheme
    ZOZ (JL,JK)   = POZON (JL,JKL) * 46.6968_JPRB / RG
    ZOZ5(JL,JK)   = POZON5(JL,JKL) * 46.6968_JPRB / RG

    ZCLD0 (JL,JK) = 0.0_JPRB
    ZCLD05(JL,JK) = 0.0_JPRB
    ZFCUP (JL,JK) = 0.0_JPRB
    ZFCUP5(JL,JK) = 0.0_JPRB
    ZFCDWN (JL,JK) = 0.0_JPRB
    ZFCDWN5(JL,JK) = 0.0_JPRB
    ZFSUP (JL,JK) = 0.0_JPRB
    ZFSUP5(JL,JK) = 0.0_JPRB
    ZFSDWN (JL,JK) = 0.0_JPRB
    ZFSDWN5(JL,JK) = 0.0_JPRB
    ZFLUX (JL,1,JK) = 0.0_JPRB
    ZFLUX5(JL,1,JK) = 0.0_JPRB
    ZFLUX (JL,2,JK) = 0.0_JPRB
    ZFLUX5(JL,2,JK) = 0.0_JPRB
    ZFLUC (JL,1,JK) = 0.0_JPRB
    ZFLUC5(JL,1,JK) = 0.0_JPRB
    ZFLUC (JL,2,JK) = 0.0_JPRB
    ZFLUC5(JL,2,JK) = 0.0_JPRB
  ENDDO
ENDDO


DO JK=1,KLEV
  JKL=KLEV+1-JK
  JKLP1=JKL+1
  DO JL=KIDIA,KFDIA
    ZTL (JL,JK)=PTH (JL,JKLP1)
    ZTL5(JL,JK)=PTH5(JL,JKLP1)
    ZTAVE (JL,JK)=PT (JL,JKL)
    ZTAVE5(JL,JK)=PT5(JL,JKL)
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  ZTL (JL,KLEV+1)= PTH (JL,1)
  ZTL5(JL,KLEV+1)= PTH5(JL,1)
  ZPMB (JL,KLEV+1) = PAPH (JL,1)/100.0_JPRB
  ZPMB5(JL,KLEV+1) = PAPH5(JL,1)/100.0_JPRB
ENDDO
!***

!     ------------------------------------------------------------------

!*         2.     CLOUD AND AEROSOL PARAMETERS
!                 ----------------------------

DO JK = 1 , KLEV
  IKL = KLEV + 1 - JK

!          2.1    INITIALIZE OPTICAL PROPERTIES TO CLEAR SKY VALUES
!                 -------------------------------------------------

  DO JSW = 1,NSW
    DO JL = KIDIA,KFDIA
      ZTAU (JL,JSW,JK)  = 0.0_JPRB
      ZTAU5(JL,JSW,JK)  = 0.0_JPRB
      ZOMEGA (JL,JSW,JK)= 0.0_JPRB
      ZOMEGA5(JL,JSW,JK)= 1.0_JPRB
      ZCG (JL,JSW,JK)   = 0.0_JPRB
      ZCG5(JL,JSW,JK)   = 0.0_JPRB
    ENDDO
  ENDDO
  DO JL = KIDIA,KFDIA
    ZCLDSW (JL,JK)  = 0.0_JPRB
    ZCLDSW5(JL,JK)  = 0.0_JPRB
    ZCLDLD (JL,JK)  = 0.0_JPRB
    ZCLDLD5(JL,JK)  = 0.0_JPRB
    ZCLDLU (JL,JK)  = 0.0_JPRB
    ZCLDLU5(JL,JK)  = 0.0_JPRB

    IF (LERADLW2) THEN
      ZCLFR(JL,IKL) = PCLFR(JL,IKL)/3.0_JPRB
    ELSE
      ZCLFR(JL,IKL) = PCLFR(JL,IKL)/10.0_JPRB
    ENDIF
  ENDDO

!          2.2    CLOUD ICE AND LIQUID CONTENT AND PATH
!                 -------------------------------------

  DO JL = KIDIA,KFDIA

! --- LIQUID WATER CONTENT (g.m-3) AND LIQUID WATER PATH (g.m-2)
    IF (PCLFR5(JL,IKL) > REPSC ) THEN

      IF (PQLWP5(JL,IKL)*1000.0_JPRB  >= 0.0_JPRB) THEN
        ZLWGKG = PQLWP (JL,IKL)*1000.0_JPRB
        ZLWGKG5= PQLWP5(JL,IKL)*1000.0_JPRB
      ELSE
        ZLWGKG = 0.0_JPRB
        ZLWGKG5= 0.0_JPRB
      ENDIF
      IF (PQIWP5(JL,IKL)*1000.0_JPRB  >= 0.0_JPRB) THEN
        ZIWGKG = PQIWP (JL,IKL)*1000.0_JPRB
        ZIWGKG5= PQIWP5(JL,IKL)*1000.0_JPRB
      ELSE
        ZIWGKG = 0.0_JPRB
        ZIWGKG5= 0.0_JPRB
      ENDIF
      
      ZLWGKG =ZLWGKG /PCLFR5(JL,IKL) -&
       & ZLWGKG5*PCLFR (JL,IKL)/PCLFR5(JL,IKL)**2  
      ZLWGKG5=ZLWGKG5/PCLFR5(JL,IKL)
      ZIWGKG =ZIWGKG /PCLFR5(JL,IKL) -&
       & ZIWGKG5*PCLFR (JL,IKL)/PCLFR5(JL,IKL)**2  
      ZIWGKG5=ZIWGKG5/PCLFR5(JL,IKL)
    ELSE
      ZLWGKG =0.0_JPRB
      ZLWGKG5=0.0_JPRB
      ZIWGKG =0.0_JPRB
      ZIWGKG5=0.0_JPRB
    ENDIF
    ZRWGKG = 0.0_JPRB
    ZRWGKG5= 0.0_JPRB
    ZRAINT(JL) = 0.0_JPRB
    ZRAINT5(JL)= 0.0_JPRB

! --- RAIN LIQUID WATER CONTENT (g.m-3) AND LIQUID WATER PATH (g.m-2)
!      IF (PRAINT5(JL,IKL) >= REPSCW ) THEN
!         ZRWGKG5=MAX(PQRAIN5(JL,IKL)*1000., 0.0)
!         ZRAINT5(JL)=PRAINT5(JL,IKL)*3600.*1000.
!- no radiative effect of rain (for the moment)
!         ZRWGKG5=0.
!         ZRAINT5(JL)=0.
! ===========================================================

! Modifications Martin et al.
!      ELSE
!      ENDIF

    ZDPOG =PDP(JL,IKL)/RG
    ZDPOG5=PDP5(JL,IKL)/RG
    ZFLWP(JL) = ZLWGKG5*ZDPOG + ZDPOG5*ZLWGKG
    ZFLWP5(JL)= ZLWGKG5*ZDPOG5
    ZFIWP(JL) = ZIWGKG5*ZDPOG + ZDPOG5*ZIWGKG 
    ZFIWP5(JL)= ZIWGKG5*ZDPOG5
    ZFRWP(JL) = ZRWGKG5*ZDPOG + ZDPOG5*ZRWGKG
    ZFRWP5(JL)= ZRWGKG5*ZDPOG5
    ZPODT = PAP(JL,IKL)/(RD*PT5(JL,IKL))&
     & - PAP5(JL,IKL)*PT(JL,IKL)/(RD*PT5(JL,IKL)**2)   
    ZPODT5= PAP5(JL,IKL)/(RD*PT5(JL,IKL))
    ZLWC(JL) = ZLWGKG5*ZPODT + ZPODT5*ZLWGKG 
    ZLWC5(JL)= ZLWGKG5*ZPODT5
    ZIWC(JL) = ZIWGKG5*ZPODT + ZPODT5*ZIWGKG 
    ZIWC5(JL)= ZIWGKG5*ZPODT5
  ENDDO
  
  DO JL = KIDIA,KFDIA

! --- EFFECTIVE RADIUS FOR WATER AND ICE PARTICLES

! very old parametrization as f(pressure)

    IF (NRADLP == 0) THEN 
      ZRADLP(JL) = -PAP(JL,IKL)*3.5_JPRB
      ZRADLP5(JL)= 10.0_JPRB + (100000.0_JPRB-PAP5(JL,IKL))*3.5_JPRB

    ELSEIF (NRADLP == 1) THEN
! old simple distinction between land (10) and ocean (13)
      IF (PLSM5(JL) < 0.5_JPRB) THEN 
        ZRADLP (JL)=0.0_JPRB
        ZRADLP5(JL)=13.0_JPRB
      ELSE
        ZRADLP (JL)=0.0_JPRB
        ZRADLP5(JL)=10.0_JPRB
      ENDIF
      
    ELSEIF (NRADLP == 2) THEN
!--  based on Martin et al., 1994, JAS
      IF (PLSM5(JL) < 0.5_JPRB) THEN
        IF (LCCNO) THEN
          ZASEA=PCCNO5(JL)
        ELSE  
          ZASEA=RCCNSEA
        ENDIF  
        ZD=0.33_JPRB
        ZNTOT=-1.15E-03_JPRB*ZASEA*ZASEA+0.963_JPRB*ZASEA+5.30_JPRB
      ELSE
        IF (LCCNL) THEN 
          ZALND=PCCNL5(JL)
        ELSE  
          ZALND=RCCNLND
        ENDIF  
        ZD=0.43_JPRB
        ZNTOT=-2.10E-04_JPRB*ZALND*ZALND+0.568_JPRB*ZALND-27.9_JPRB
      ENDIF
      ZNUM = 3.0_JPRB*ZLWC(JL)*(1.0_JPRB+3.0_JPRB*ZD*ZD)**2
      ZNUM5= 3.0_JPRB*ZLWC5(JL)*(1.0_JPRB+3.0_JPRB*ZD*ZD)**2
      ZDEN=4.0_JPRB*RPI*ZNTOT*(1.0_JPRB+ZD*ZD)**3
      IF ((ZNUM5/ZDEN) > REPLOG) THEN
        ZRADLP5(JL)=100.0_JPRB*EXP(0.333_JPRB*LOG(ZNUM5/ZDEN))
        ZRADLP(JL) = (0.333_JPRB*ZRADLP5(JL)/ZNUM5) * ZNUM
        IF (ZRADLP5(JL) < 4.0_JPRB) THEN
          ZRADLP(JL) = 0.0_JPRB
          ZRADLP5(JL)= 4.0_JPRB
        ENDIF  
        IF (ZRADLP5(JL) > 16.0_JPRB) THEN
          ZRADLP(JL) = 0.0_JPRB
          ZRADLP5(JL)= 16.0_JPRB
        ENDIF
      ELSE
        ZRADLP(JL) = 0.0_JPRB
        ZRADLP5(JL)= 4.0_JPRB
      ENDIF
    ENDIF

! ===========================================================
! ___________________________________________________________

! rain drop from          : unused as ZRAINT is 0.
!      ZRADRD5(JL)=500.0_JPRB*ZRAINT5(JL)**0.22_JPRB
!      IF (ZFLWP5(JL).GT.0.) THEN
!         ZRADRD5(JL)=ZRADLP5(JL)+ZRADRD5(JL)
!      ENDIF   

  ENDDO
  
  DO JL = KIDIA,KFDIA
  
! diagnosing the ice particle effective radius/diameter

!- ice particle effective radius =f(T) from Liou and Ou (1994)
 
    IF (PT5(JL,IKL) < RTICE) THEN
      ZTEMPC =PT (JL,IKL)
      ZTEMPC5=PT5(JL,IKL)-RTT
    ELSE
      ZTEMPC =0.0_JPRB
      ZTEMPC5=RTICE-RTT
    ENDIF

    ZRADIP (JL)=      ZTEMPC *(12.42_JPRB + ZTEMPC5*(2.0_JPRB*0.197_JPRB +&
     & 3.0_JPRB*ZTEMPC5*0.0012_JPRB))  
    ZRADIP5(JL)=326.3_JPRB+ZTEMPC5*(12.42_JPRB + ZTEMPC5*(   0.197_JPRB +&
     & ZTEMPC5*0.0012_JPRB))  
      
    IF (NRADIP == 0) THEN
!-- fixed 40 micron effective radius
      ZRADIP(JL)= 0.0_JPRB
      ZDESR(JL) = ZDefRe * ZRADIP(JL)
      ZRADIP5(JL)= 40.0_JPRB
      ZDESR5(JL)= ZDefRe * ZRADIP5(JL)
      
    ELSEIF (NRADIP == 1) THEN 
!-- old formulation based on Liou & Ou (1994) temperature (40-130microns)
      IF (ZRADIP5(JL) < 40.0_JPRB) THEN
        ZRADIP (JL)=0.0_JPRB
        ZRADIP5(JL)=40.0_JPRB
      ENDIF
      ZDESR(JL) = ZDefRe * ZRADIP(JL)
      ZDESR5(JL)= ZDefRe * ZRADIP5(JL)
      
    ELSEIF (NRADIP == 2) THEN  
!-- formulation following Jakob, Klein modifications to ice content    
      IF (ZRADIP5(JL) < 30.0_JPRB) THEN
        ZRADIP (JL)=0.0_JPRB
        ZRADIP5(JL)=30.0_JPRB
      ENDIF
      IF (ZRADIP5(JL) > 60.0_JPRB) THEN
        ZRADIP (JL)=0.0_JPRB
        ZRADIP5(JL)=60.0_JPRB
      ENDIF
      ZDESR(JL) = ZDefRe * ZRADIP(JL)
      ZDESR5(JL)= ZDefRe * ZRADIP5(JL)
      
    ELSEIF (NRADIP == 3 ) THEN
      IF (ZIWC5(JL) > 0.0_JPRB ) THEN
!- ice particle effective radius =f(T,IWC) from Sun and Rikus (1999)
! revised by Sun (2001)
        ZTEMPC  = PT (JL,IKL)
        ZTEMPC5 = PT5(JL,IKL)-83.15_JPRB
        ZTCELS  = PT (JL,IKL)
        ZTCELS5 = PT5(JL,IKL)-RTT
        ZFSR  = 0.0105_JPRB * ZTCELS
        ZFSR5 = 1.2351_JPRB +0.0105_JPRB * ZTCELS5
! Sun, 2001
        ZAIWC5 = 45.8966_JPRB * ZIWC5(JL)**0.2214_JPRB
        ZAIWC  = 0.2214_JPRB*ZAIWC5*ZIWC(JL)/ZIWC5(JL)
        ZBIWC5 = 0.7957_JPRB * ZIWC5(JL)**0.2535_JPRB
        ZBIWC  = 0.2535_JPRB*ZBIWC5*ZIWC(JL)/ZIWC5(JL) 
        ZDESR(JL) = ZFSR *  (ZAIWC5 + ZBIWC5*ZTEMPC5) &
         & + ZFSR5 * (ZAIWC + ZBIWC*ZTEMPC5+ZBIWC5*ZTEMPC)  
        ZDESR5(JL) = ZFSR5 * (ZAIWC5 + ZBIWC5*ZTEMPC5)
        IF (ZDESR5(JL) < 30.0_JPRB) THEN
          ZDESR(JL) = 0.0_JPRB
          ZDESR5(JL)= 30.0_JPRB
        ENDIF
        IF (ZDESR5(JL) > 155.0_JPRB) THEN
          ZDESR(JL) = 0.0_JPRB
          ZDESR5(JL)= 155.0_JPRB
        ENDIF
        ZRADIP(JL) = ZRefDe * ZDESR(JL)
        ZRADIP5(JL)= ZRefDe * ZDESR5(JL)
      ELSE
        ZDESR(JL) = 0.0_JPRB
        ZRADIP(JL)= ZRefDe * ZDESR(JL)
        ZDESR5(JL) = 80.0_JPRB
        ZRADIP5(JL)= ZRefDe * ZDESR5(JL)
      ENDIF
    ENDIF
    
  ENDDO

!          2.3    CLOUD SHORTWAVE OPTICAL PROPERTIES
!                 ----------------------------------

!   -------------------------
! --+ SW OPTICAL PARAMETERS +  Water clouds after Fouquart (1987)
!   -------------------------  Ice clouds (Ebert, Curry, 1992)

  IF (LERADSW2) THEN
    DO JSW=1,NSW
      DO JL = KIDIA,KFDIA
        ZTOL =0.0_JPRB
        ZTOL5=0.0_JPRB
        ZGL  =0.0_JPRB
        ZGL5 =0.0_JPRB
        ZOL  =0.0_JPRB
        ZOL5 =0.0_JPRB
        ZTOI =0.0_JPRB
        ZTOI5=0.0_JPRB
        ZGI  =0.0_JPRB
        ZGI5 =0.0_JPRB
        ZOI  =0.0_JPRB
        ZOI5 =0.0_JPRB
        ZTOR =0.0_JPRB
        ZTOR5=0.0_JPRB
        ZGR  =0.0_JPRB
        ZGR5 =0.0_JPRB
        ZOR  =0.0_JPRB
        ZOR5 =0.0_JPRB
        IF (ZFLWP5(JL)+ZFIWP5(JL)+ZFRWP5(JL) >= REPSCW ) THEN
          IF (ZFLWP5(JL)  >= REPSCW ) THEN
            IF (NLIQOPT /= 0 ) THEN
!-- SW: Slingo, 1989
              ZTOL  = ZFLWP(JL)&
               & * (RASWCA(JSW)+RASWCB(JSW)/ZRADLP5(JL))&
               & -  RASWCB(JSW)*ZFLWP5(JL)*ZRADLP(JL)&
               & / ZRADLP5(JL)**2   
              ZTOL5 = ZFLWP5(JL)&
               & *(RASWCA(JSW)+RASWCB(JSW)/ZRADLP5(JL))  
              ZGL   = RASWCF(JSW)*ZRADLP(JL) 
              ZGL5  = RASWCE(JSW)+RASWCF(JSW)*ZRADLP5(JL)
              ZOL   = - RASWCD(JSW)*ZRADLP(JL)
              ZOL5  = 1.0_JPRB - RASWCC(JSW)-RASWCD(JSW)*ZRADLP5(JL)
            ELSE
!-- SW: Fouquart, 1991
              ZTOL  = ZFLWP(JL)&
               & * (RYFWCA(JSW)+RYFWCB(JSW)/ZRADLP5(JL))&
               & - RYFWCB(JSW)*ZFLWP5(JL)*ZRADLP(JL)&
               & / ZRADLP5(JL)**2  
              ZTOL5 = ZFLWP5(JL)&
               & * (RYFWCA(JSW)+RYFWCB(JSW)/ZRADLP5(JL))  
              ZGL   = 0.0_JPRB
              ZGL5  = RYFWCF(JSW)
              ZOL   = ZTOL*RYFWCE(JSW)*RYFWCD(JSW)*EXP(-RYFWCE(JSW)*ZTOL5)
              ZOL5  = RYFWCC(JSW)-RYFWCD(JSW)*EXP(-RYFWCE(JSW)*ZTOL5)
            ENDIF
          ENDIF

          IF (ZFIWP5(JL)  >=  REPSCW ) THEN
            IF (NICEOPT <= 1) THEN
!-- SW: Ebert-Curry
              ZTOI  = ZFIWP (JL)&
               & *(REBCUA(JSW)+REBCUB(JSW)/ZRADIP5(JL))&
               & - ZFIWP5(JL)*ZRADIP(JL)*REBCUB(JSW)&
               & /ZRADIP5(JL)**2  
              ZTOI5 = ZFIWP5(JL)&
               & *(REBCUA(JSW)+REBCUB(JSW)/ZRADIP5(JL))  
              ZGI   =             REBCUF(JSW)*ZRADIP (JL)
              ZGI5  = REBCUE(JSW)+REBCUF(JSW)*ZRADIP5(JL)
              ZOI  =                 -REBCUD(JSW)*ZRADIP (JL)
! modification to avoid noise in TL
              IF (JSW == 4) THEN
                ZOI5 = 1.0_JPRB - REBCUC(JSW)/10.0_JPRB-REBCUD(JSW)*ZRADIP5(JL)
              ELSE
                ZOI5 = 1.0_JPRB - REBCUC(JSW)-REBCUD(JSW)*ZRADIP5(JL)
              ENDIF
! end modif

            ELSEIF (NICEOPT == 2) THEN  
!-- SW: Fu-Liou
              Z1RADI  = -ZDESR(JL)/ZDESR5(JL)**2
              Z1RADI5 = 1.0_JPRB / ZDESR5(JL)
              ZBETAI  = Z1RADI * RFLAA1(JSW)
              ZBETAI5 = RFLAA0(JSW)+Z1RADI5* RFLAA1(JSW)
              ZTOI  = ZFIWP5(JL) * ZBETAI &
               &     + ZBETAI5 * ZFIWP(JL)   
              ZTOI5 = ZFIWP5(JL) * ZBETAI5
              ZOMGI = (RFLBB1(JSW)+2.0_JPRB*RFLBB2(JSW)*ZRADIP5(JL)&
               & + 3.0_JPRB*RFLBB3(JSW)*ZRADIP5(JL)**2)*ZRADIP(JL)   
              ZOMGI5= RFLBB0(JSW)+ZRADIP5(JL)*(RFLBB1(JSW)+ZRADIP5(JL)&
               & *(RFLBB2(JSW)+ZRADIP5(JL)* RFLBB3(JSW) ))  
              ZOI   = - ZOMGI
              ZOI5  = 1.0_JPRB - ZOMGI5
              ZOMGP = (RFLCC1(JSW)+2.0_JPRB*RFLCC2(JSW)*ZRADIP5(JL)&
               & + 3.0_JPRB*RFLCC3(JSW)*ZRADIP5(JL)**2)*ZRADIP(JL)  
              ZOMGP5= RFLCC0(JSW)+ZRADIP5(JL)*(RFLCC1(JSW)+ZRADIP5(JL)&
               & *(RFLCC2(JSW)+ZRADIP5(JL)* RFLCC3(JSW) ))  
              ZFDEL = (RFLDD1(JSW)+2.0_JPRB*RFLDD2(JSW)*ZRADIP5(JL)&
               & + 3.0_JPRB*RFLDD3(JSW)*ZRADIP5(JL)**2)*ZRADIP(JL)  
              ZFDEL5= RFLDD0(JSW)+ZRADIP5(JL)*(RFLDD1(JSW)+ZRADIP5(JL)&
               & *(RFLDD2(JSW)+ZRADIP5(JL)* RFLDD3(JSW) ))  
              ZGI  = (3.0_JPRB-ZOMGP5)*ZFDEL/3.0_JPRB &
               & + (1.0_JPRB-ZFDEL5)*ZOMGP/3.0_JPRB  
              ZGI5  = ((1.0_JPRB-ZFDEL5)*ZOMGP5 + ZFDEL5*3.0_JPRB)/3.0_JPRB
            
            ELSEIF (NICEOPT == 3) THEN
!-- SW: Fu 1996
              Z1RADI  = -ZDESR(JL)/ZDESR5(JL)**2
              Z1RADI5 = 1.0_JPRB / ZDESR5(JL)
              ZBETAI  =             Z1RADI * RFUAA1(JSW)
              ZBETAI5 = RFUAA0(JSW)+Z1RADI5* RFUAA1(JSW)
              ZTOI  = ZFIWP (JL) * ZBETAI5 &
                &   + ZFIWP5(JL) * ZBETAI
              ZTOI5 = ZFIWP5(JL) * ZBETAI5
              ZOMGI = ZDESR(JL)*(                      RFUBB1(JSW) &
                &                + 2.0_JPRB*ZDESR5(JL)*RFUBB2(JSW) &
                &                + 3.0_JPRB*ZDESR5(JL)*RFUBB3(JSW))   
              ZOMGI5= RFUBB0(JSW)+ZDESR5(JL)*(RFUBB1(JSW) + ZDESR5(JL) &
               &    *(RFUBB2(JSW)+ZDESR5(JL)* RFUBB3(JSW) ))            
              ZOI   =       - ZOMGI
              ZOI5  = 1.0_JPRB - ZOMGI5
              ZGI   = ZDESR(JL)*(                        RFUCC1(JSW) &
                &                + 2.0_JPRB * ZDESR5(JL)*RFUCC2(JSW) &
                &                + 3.0_JPRB * ZDESR5(JL)*RFUCC3(JSW)) 
              ZGI5  = RFUCC0(JSW)+ZDESR5(JL)*(RFUCC1(JSW) + ZDESR5(JL) &
               &    *(RFUCC2(JSW)+ZDESR5(JL)* RFUCC3(JSW) )) 
              IF (ZGI5 > 1.0_JPRB) THEN
                ZGI  = 0.0_JPRB
                ZGI5 = 1.0_JPRB
              ENDIF
            ENDIF
          ENDIF

!  - MIX of WATER and ICE CLOUDS
          ZTAUMX = ZTOL  + ZTOI  + ZTOR
          ZTAUMX5= ZTOL5 + ZTOI5 + ZTOR5
          ZOMGMX = ZTOL5*ZOL  + ZTOI5*ZOI  + ZTOR5*ZOR +&
           & ZTOL *ZOL5 + ZTOI *ZOI5 + ZTOR *ZOR5  
          ZOMGMX5= ZTOL5*ZOL5 + ZTOI5*ZOI5 + ZTOR5*ZOR5
          ZASYMX = ZTOL5*ZOL5*ZGL   + ZTOI5*ZOI5*ZGI   +&
           & ZTOR5*ZOR5*ZGR   +&
           & ZTOL *ZOL5*ZGL5  + ZTOI *ZOI5*ZGI5  + &
           & ZTOR *ZOR5*ZGR5  +&
           & ZTOL5*ZOL *ZGL5  + ZTOI5*ZOI *ZGI5  + &
           & ZTOR5*ZOR *ZGR5     
          ZASYMX5= ZTOL5*ZOL5*ZGL5 + ZTOI5*ZOI5*ZGI5 + ZTOR5*ZOR5*ZGR5

          ZASYMX = ZASYMX /ZOMGMX5 - ZASYMX5*ZOMGMX/ZOMGMX5**2
          ZASYMX5= ZASYMX5/ZOMGMX5
          ZOMGMX = ZOMGMX /ZTAUMX5 - ZOMGMX5*ZTAUMX/ZTAUMX5**2
          ZOMGMX5= ZOMGMX5/ZTAUMX5

! --- SW FINAL CLOUD OPTICAL PARAMETERS

          ZCLDSW (JL,JK)  = PCLFR (JL,IKL)
          ZCLDSW5(JL,JK)  = PCLFR5(JL,IKL)
          ZTAU (JL,JSW,JK)  = ZTAUMX
          ZTAU5(JL,JSW,JK)  = ZTAUMX5
          ZOMEGA (JL,JSW,JK)= ZOMGMX
          ZOMEGA5(JL,JSW,JK)= ZOMGMX5
          ZCG (JL,JSW,JK)   = ZASYMX
          ZCG5(JL,JSW,JK)   = ZASYMX5
        ENDIF
      ENDDO
    ENDDO
  ENDIF

!          2.4    CLOUD LONGWAVE OPTICAL PROPERTIES FOR EC-OPE
!                 --------------------------------------------

!   -------------------------
! --+ LW OPTICAL PARAMETERS +  Water and Ice from Smith and Shi (1992)
!   -------------------------

  IF (LERADN2) THEN
    IF (.NOT.LRRTM .OR. LPHYLIN) THEN
      DO JL = KIDIA,KFDIA
        ZALFICE (JL)=0.0_JPRB
        ZALFICE5(JL)=0.0_JPRB
        ZGAMICE (JL)=0.0_JPRB
        ZGAMICE5(JL)=0.0_JPRB
        ZBICE (JL)=0.0_JPRB
        ZBICE5(JL)=0.0_JPRB
        ZTICE (JL)= ZTAVE (JL,JK)/TSTAND
        ZTICE5(JL)=(ZTAVE5(JL,JK)-TSTAND)/TSTAND
        ZBICFU (JL)=0.0_JPRB
        IF (NICEOPT == 1) THEN
          ZBICFU5(JL)=1.0_JPRB
        ELSE
          ZBICFU5(JL)=0.0_JPRB
        ENDIF
        ZKICFU (JL)=0.0_JPRB
        ZKICFU5(JL)=0.0_JPRB
      ENDDO
      
      DO JNU= 1,NSIL
        DO JL = KIDIA,KFDIA
          ZRES (JL)  = ZTICE (JL)*&
           & (   XP(2,JNU)+ZTICE5(JL)*&
           & (2.0_JPRB*XP(3,JNU)+ZTICE5(JL)*&
           & (3.0_JPRB*XP(4,JNU)+ZTICE5(JL)*&
           & (4.0_JPRB*XP(5,JNU)+ZTICE5(JL)*(5.0_JPRB*XP(6,JNU))))))  
          ZRES5(JL)  = XP(1,JNU)+ZTICE5(JL)*(XP(2,JNU)+ZTICE5(JL)*&
           & (XP(3,JNU)+ZTICE5(JL)*(XP(4,JNU)+ZTICE5(JL)*&
           & (XP(5,JNU)+ZTICE5(JL)*(XP(6,JNU))))))  
          ZBICE (JL) = ZBICE (JL) + ZRES (JL)
          ZBICE5(JL) = ZBICE5(JL) + ZRES5(JL)
          ZGAMICE (JL) = ZGAMICE (JL) + REBCUI(JNU)*ZRES (JL)
          ZGAMICE5(JL) = ZGAMICE5(JL) + REBCUI(JNU)*ZRES5(JL)
          ZALFICE (JL) = ZALFICE (JL) + REBCUJ(JNU)*ZRES (JL)
          ZALFICE5(JL) = ZALFICE5(JL) + REBCUJ(JNU)*ZRES5(JL)
        ENDDO
      ENDDO

      IF (NICEOPT == 2 .OR. NICEOPT == 3) THEN 
!-- Fu-type parametrisations with M'91 (and Neuro-Net) LW scheme
          DO JL=KIDIA,KFDIA
            IF (PT5(JL,IKL) < 160.0_JPRB) THEN
              INDLAY(JL)=1
              ZTBLAY(JL) =PT(JL,IKL)
              ZTBLAY5(JL)=PT5(JL,IKL)-160.0_JPRB
            ELSEIF (PT5(JL,IKL) < 339.0_JPRB) THEN
              INDLAY(JL)=PT5(JL,IKL)-159.0_JPRB
              INDLAY(JL) = MAX(INDLAY(JL),1)
              ZTBLAY(JL) =PT(JL,IKL)
              ZTBLAY5(JL)=PT5(JL,IKL)-INT(PT5(JL,IKL))
            ELSE
              INDLAY(JL)=180
              ZTBLAY(JL) =PT(JL,IKL)
              ZTBLAY5(JL)=PT5(JL,IKL)-339.0_JPRB
            ENDIF
          ENDDO
        DO JRTM=1,16
          DO JL=KIDIA,KFDIA
            ZADDPLK = TOTPLNK(INDLAY(JL)+1,JRTM)-TOTPLNK(INDLAY(JL),JRTM)
            ZPLANCK = DELWAVE(JRTM) * (                       ZTBLAY(JL) *ZADDPLK)
            ZPLANCK5= DELWAVE(JRTM) * (TOTPLNK(INDLAY(JL),JRTM) + ZTBLAY5(JL)*ZADDPLK)
            ZBICFU(JL) = ZBICFU(JL) + ZPLANCK
            ZBICFU5(JL)=ZBICFU5(JL) + ZPLANCK5

            IF (ZIWC5(JL) > 0.0_JPRB ) THEN
              ZRATIO  = -ZDESR(JL)/(ZDESR5(JL)*ZDESR5(JL))
              ZRATIO5 = 1.0_JPRB /  ZDESR5(JL)
              IF (NICEOPT == 2) THEN
! ice cloud spectral emissivity a la Fu & Liou (1993)
                ZMABSD  = ZRATIO &
                 & * (RFULIO(JRTM,2) + 2.0_JPRB*ZRATIO5*RFULIO(JRTM,3))
                ZMABSD5 = RFULIO(JRTM,1) + ZRATIO5 &
                 & * (RFULIO(JRTM,2) + ZRATIO5*RFULIO(JRTM,3))

! ice cloud spectral emissivity a la Fu et al. (1998)
              ELSEIF (NICEOPT == 3) THEN
                ZMABSD  = ZRATIO &
                 & * (RFUETA(JRTM,2) + 2.0_JPRB*ZRATIO5*RFUETA(JRTM,3))
                ZMABSD5 = RFUETA(JRTM,1) + ZRATIO5 &
                 & * (RFUETA(JRTM,2) + ZRATIO5*RFUETA(JRTM,3))
              ENDIF
              ZKICFU(JL)  = ZKICFU (JL) + ZMABSD  * ZPLANCK5 &
               &           + ZMABSD5 * ZPLANCK
              ZKICFU5(JL) = ZKICFU5(JL) + ZMABSD5 * ZPLANCK5
            ENDIF
          ENDDO
        ENDDO
      ENDIF
      
      DO JL = KIDIA,KFDIA
        ZGAMICE (JL) = ZGAMICE (JL) / ZBICE5(JL) -&
         & ZGAMICE5(JL) * ZBICE (JL) / ZBICE5(JL)**2  
        ZGAMICE5(JL) = ZGAMICE5(JL) / ZBICE5(JL)
        ZALFICE (JL) = ZALFICE (JL) / ZBICE5(JL) -&
         & ZALFICE5(JL) * ZBICE (JL) / ZBICE5(JL)**2  
        ZALFICE5(JL) = ZALFICE5(JL) / ZBICE5(JL)
        ZKICFU(JL) = ZKICFU(JL) / ZBICFU5(JL) -&
         & ZKICFU5(JL) * ZBICFU(JL) / ZBICFU5(JL)**2
        ZKICFU5(JL) = ZKICFU5(JL) / ZBICFU5(JL)
                
        IF (ZFLWP5(JL)+ZFIWP5(JL) >=  REPSCW ) THEN
          
          IF (NLIQOPT == 0) THEN
! water cloud emissivity a la Smith-Shi
            ZMULTL =   -0.006_JPRB*ZRADLP (JL)
            ZMULTL5=1.2_JPRB-0.006_JPRB*ZRADLP5(JL)
            ZMSALD = 0.158_JPRB*ZMULTL
            ZMSALD5= 0.158_JPRB*ZMULTL5
            ZMSALU = 0.130_JPRB*ZMULTL
            ZMSALU5= 0.130_JPRB*ZMULTL5
          ELSE
! water cloud emissivity a la Savijarvi (1997)
            ZMSALU = -0.0105_JPRB*ZRADLP(JL)
            ZMSALU5= 0.2441_JPRB-0.0105_JPRB*ZRADLP5(JL)
            ZMSALD = 1.2154_JPRB*ZMSALU
            ZMSALD5= 1.2154_JPRB*ZMSALU5
          ENDIF

          IF (NICEOPT == 0) THEN
! ice cloud emissivity a la Smith & Shi (1992)
            ZMULTI = -0.006_JPRB*ZRADIP(JL)
            ZMULTI5=1.2_JPRB-0.006_JPRB*ZRADIP5(JL)
            ZMSAID = 0.113_JPRB*ZMULTI
            ZMSAID5= 0.113_JPRB*ZMULTI5
            ZMSAIU = 0.093_JPRB*ZMULTI
            ZMSAIU5= 0.093_JPRB*ZMULTI5
          ELSEIF (NICEOPT == 1) THEN
! ice cloud emissivity a la Ebert & Curry (1992)
            ZMSAID = 1.66_JPRB*(ZALFICE (JL)+ZGAMICE (JL)/ZRADIP5(JL))-&
             & 1.66_JPRB* ZGAMICE5(JL)*ZRADIP(JL)/ZRADIP5(JL)**2     
            ZMSAID5= 1.66_JPRB*(ZALFICE5(JL)+ZGAMICE5(JL)/ZRADIP5(JL))
            ZMSAIU = ZMSAID
            ZMSAIU5= ZMSAID5
! ice cloud emissivity a la Fu & Fiou (1993) or Fu et al. (1998)
          ELSEIF (NICEOPT == 2 .OR. NICEOPT == 3) THEN
            ZMSAID = 1.66_JPRB * ZKICFU(JL)
            ZMSAID5= 1.66_JPRB * ZKICFU5(JL)
            ZMSAIU = ZMSAID
            ZMSAIU5= ZMSAID5
          ENDIF
          
          IF (NINHOM == 1) THEN
            ZZFLWP = ZFLWP (JL) * RLWINHF
            ZZFLWP5= ZFLWP5(JL) * RLWINHF
            ZZFIWP = ZFIWP (JL) * RLWINHF
            ZZFIWP5= ZFIWP5(JL) * RLWINHF
          ELSEIF (NINHOM == 0) THEN
            ZZFLWP = ZFLWP (JL)
            ZZFLWP5= ZFLWP5(JL)
            ZZFIWP = ZFIWP (JL)
            ZZFIWP5= ZFIWP5(JL)
          ELSE
            CALL ABOR1('TL model cannot be used with NINHOM > 1')
          ENDIF

! effective cloudiness accounting for condensed water
!          ZCLDLD (JL,JK) = PCLFR (JL,IKL)*(1.0_JPRB-EXP(-ZMSALD5*ZZFLWP5 &
          ZCLDLD (JL,JK) = ZCLFR (JL,IKL)*(1.0_JPRB-EXP(-ZMSALD5*ZZFLWP5 &
           & -ZMSAID5*ZZFIWP5)) +&
           & PCLFR5(JL,IKL)*(ZMSALD *ZZFLWP5 +&
           & ZMSALD5*ZZFLWP  +&
           & ZMSAID *ZZFIWP5 +&
           & ZMSAID5*ZZFIWP    )*&
           & EXP(-ZMSALD5*ZZFLWP5-ZMSAID5*ZZFIWP5)  
          ZCLDLD5(JL,JK) = PCLFR5(JL,IKL)*(1.0_JPRB-EXP(-ZMSALD5*ZZFLWP5 &
           & -ZMSAID5*ZZFIWP5))  
!          ZCLDLU (JL,JK) = PCLFR (JL,IKL)*(1.0_JPRB-EXP(-ZMSALU5*ZZFLWP5 &
          ZCLDLU (JL,JK) = ZCLFR (JL,IKL)*(1.0_JPRB-EXP(-ZMSALU5*ZZFLWP5 &
           & -ZMSAIU5*ZZFIWP5))+&
           & PCLFR5(JL,IKL)*(ZMSALU *ZZFLWP5 +&
           & ZMSALU5*ZZFLWP  +&
           & ZMSAIU *ZZFIWP5 +&
           & ZMSAIU5*ZZFIWP    )*&
           & EXP(-ZMSALU5*ZZFLWP5-ZMSAIU5*ZZFIWP5)  
          ZCLDLU5(JL,JK) = PCLFR5(JL,IKL)*(1.0_JPRB-EXP(-ZMSALU5*ZZFLWP5 &
           & -ZMSAIU5*ZZFIWP5))  
        ENDIF
      ENDDO

    ELSE

!          2.5    CLOUD LONGWAVE OPTICAL PROPERTIES FOR RRTM
!                 ------------------------------------------

!   -------------------------
! --+ LW OPTICAL PARAMETERS +  Water (and Ice) from Savijarvi (1998)
!   -------------------------  Ice clouds (Ebert, Curry, 1992)

! No need for a fixed diffusivity factor, accounted for spectrally below
! The detailed spectral structure does not require defining upward and
! downward effective optical properties

! A bit irrelevant, as RRTM_TL does not exist

      DO JRTM=1,16
        DO JL = KIDIA,KFDIA
          ZTAUCLD(JL,JK,JRTM)  = 0.0_JPRB
          ZTAUCLD5(JL,JK,JRTM) = 0.0_JPRB
          ZMSALD  = 0.0_JPRB
          ZMSALD5 = 0.0_JPRB
          ZMSAID  = 0.0_JPRB
          ZMSAID5 = 0.0_JPRB

          IF (ZFLWP5(JL)+ZFIWP5(JL) >= REPSCW ) THEN

            IF (NLIQOPT == 0 .OR. NLIQOPT >= 3) THEN
! water cloud total emissivity a la Smith and Shi (1992)
              ZMULTL  = -0.006_JPRB*ZRADLP(JL)
              ZMULTL5 = 1.2_JPRB-0.006_JPRB*ZRADLP5(JL)
              ZRSALD  = 0.144_JPRB*ZMULTL / 1.66_JPRB
              ZRSALD5 = 0.144_JPRB*ZMULTL5 / 1.66_JPRB
            ELSEIF (NLIQOPT == 1) THEN            
! water cloud spectral emissivity a la Savijarvi (1997)
              ZRSALD = (RHSAVI(JRTM,2)+2.0_JPRB*RHSAVI(JRTM,3)&
               & *ZRADLP5(JL))* ZRADLP(JL)  
              ZRSALD5= RHSAVI(JRTM,1) + ZRADLP5(JL)&
               & *(RHSAVI(JRTM,2) + ZRADLP5(JL)*RHSAVI(JRTM,3))  
            ELSEIF (NLIQOPT == 2) THEN
! water cloud spectral emissivity a la Lindner and Li (2000)
              Z1RADL = - ZRADLP(JL)/ZRADLP5(JL)**2
              Z1RADL5 = 1.0_JPRB / ZRADLP5(JL)

              ZEXTCF  = ZRADLP(JL) * RLILIA(JRTM,2) &
               & + Z1RADL  * (RLILIA(JRTM,3) &
               & + 2.0_JPRB*Z1RADL5 * RLILIA(JRTM,4) &
               & + 3.0_JPRB*Z1RADL5**2*RLILIA(JRTM,5) )  
              ZEXTCF5 = RLILIA(JRTM,1) + ZRADLP5(JL) * RLILIA(JRTM,2) &
               & + Z1RADL5 * (RLILIA(JRTM,3) &
               & + Z1RADL5 * (RLILIA(JRTM,4) &
               & + Z1RADL5 *  RLILIA(JRTM,5) ))
              Z1MOMG = Z1RADL * RLILIB(JRTM,2) &
               & + ZRADLP(JL) * (RLILIB(JRTM,3) + 2.0_JPRB*ZRADLP5(JL)&
               & * RLILIB(JRTM,4) )
              Z1MOMG5 = RLILIB(JRTM,1) + Z1RADL5 * RLILIB(JRTM,2) &
               & + ZRADLP5(JL) * (RLILIB(JRTM,3) &
               & + ZRADLP5(JL)*RLILIB(JRTM,4) )

              ZRSALD  = Z1MOMG  * ZEXTCF5 + Z1MOMG5 * ZEXTCF
              ZRSALD5 = Z1MOMG5 * ZEXTCF5
            ENDIF

            IF (NICEOPT == 0) THEN
! ice cloud spectral emissivity a la Smith & Shi (1992)
              ZMULTI = -0.006_JPRB*ZRADIP(JL)
              ZMULTI5= 1.2_JPRB-0.006_JPRB*ZRADIP5(JL)
              ZRSAID = 0.103_JPRB*ZMULTI / 1.66_JPRB
              ZRSAID5= 0.103_JPRB*ZMULTI5 / 1.66_JPRB  
            ELSEIF (NICEOPT == 1) THEN
! ice cloud spectral emissivity a la Ebert-Curry (1992)
              ZRSAID  = -REBCUG(JRTM)*ZRADIP(JL)/ZRADIP5(JL)**2
              ZRSAID5 = REBCUH(JRTM)+REBCUG(JRTM)/ZRADIP5(JL)

            ELSEIF (NICEOPT == 2) THEN                         
! ice cloud spectral emissivity a la Fu & Liou (1993)
              Z1RADI  = -ZDESR(JL)/ZDESR5(JL)**2 
              Z1RADI5 = 1.0_JPRB / ZDESR5(JL)
              ZRSAID  =                  Z1RADI  &
               & * (RFULIO(JRTM,2) + 2.0_JPRB*Z1RADI5 * RFULIO(JRTM,3))
              ZRSAID5 = RFULIO(JRTM,1) + Z1RADI5 &
               & * (RFULIO(JRTM,2) +          Z1RADI5 * RFULIO(JRTM,3))  
            ELSEIF (NICEOPT == 3) THEN
! ice cloud spectral emissivity a la Fu et al (1998)
              Z1RADI  = - ZDESR(JL)/ZDESR5(JL)**2
              Z1RADI5 = 1.0_JPRB / ZDESR5(JL)
              ZRSAIE = (RFUETA(JRTM,2)+2*RFUETA(JRTM,3)*Z1RADI5) &
               & *Z1RADI
              ZRSAIE5 = RFUETA(JRTM,1) + Z1RADI5 &
               & *(RFUETA(JRTM,2) + Z1RADI5 * RFUETA(JRTM,3)) 
              ZRSAIA = (RFUETB(JRTM,1)+ZDESR5(JL)*( RFUETB(JRTM,2) &
               & +ZDESR5(JL)*( RFUETB(JRTM,3) +ZDESR5(JL)*RFUETB(JRTM,4)))) &
               & *Z1RADI &
               & +Z1RADI5*(RFUETB(JRTM,2) &
               & +ZDESR5(JL)*(2*RFUETB(JRTM,3)+3*RFUETB(JRTM,4)*ZDESR5(JL))) &
               & *ZDESR(JL)
              ZRSAIA5 = Z1RADI5*(RFUETB(JRTM,1) &
               & +ZDESR5(JL)*( RFUETB(JRTM,2) &
               & +ZDESR5(JL)*( RFUETB(JRTM,3) +ZDESR5(JL)*RFUETB(JRTM,4))))
              ZRSAIG = (RFUETC(JRTM,2)+ZDESR5(JL)*(2*RFUETC(JRTM,3) &
               & +3*RFUETC(JRTM,4)*ZDESR5(JL)))*ZDESR(JL)
              ZRSAIG5 = RFUETC(JRTM,1) +ZDESR5(JL)*( RFUETC(JRTM,2) &
               & +ZDESR5(JL)*( RFUETC(JRTM,3) +ZDESR5(JL)* RFUETC(JRTM,4)))
              ZRSAIF = (0.3738_JPRB+ZRSAIG5*(2*0.0076_JPRB &
               & +3*0.1186_JPRB*ZRSAIG5))*ZRSAIG
              ZRSAIF5 = 0.5_JPRB + ZRSAIG5*( 0.3738_JPRB &
               & + ZRSAIG5*( 0.0076_JPRB + ZRSAIG5*0.1186_JPRB ) )
              ZRSAID = ZRSAIE - ZRSAIA5*ZRSAIF - ZRSAIF5*ZRSAIA
              ZRSAID5 = ZRSAIE5 - ZRSAIA5*ZRSAIF5
!              ZRSAID5 = (1.0_JPRB - ZRSAIA5/ZRSAIE5 * ZRSAIF5) * ZRSAIE5
            ENDIF

            ZTAUD  = ZRSALD5*ZFLWP(JL) + ZRSALD *ZFLWP5(JL) &
             &     + ZRSAID5*ZFIWP(JL) + ZRSAID *ZFIWP5(JL)  
            ZTAUD5 = ZRSALD5*ZFLWP5(JL)+ ZRSAID5*ZFIWP5(JL)

! Diffusivity correction within clouds a la Savijarvi
! (no need for increment on ZDIFFD as RRTM is not the LW radiation scheme 
! used in TL computations)
            IF (LDIFFC) THEN 
              ZEXPRS =           -0.156_JPRB*ZTAUD/ZTAUD5
              ZEXPRS5= 1.517_JPRB-0.156_JPRB*LOG(ZTAUD5)   
              IF (ZEXPRS5 > 1.0_JPRB) THEN
                ZDIFFD  = ZEXPRS
                ZDIFFD5 = ZEXPRS5
              ELSE
                ZDIFFD  = 0.0_JPRB
                ZDIFFD5 = 1.0_JPRB
              ENDIF
              IF (ZDIFFD5 >= 2.0_JPRB) THEN
                ZDIFFD  = 0.0_JPRB
                ZDIFFD5 = 2.0_JPRB
              ENDIF
            ELSE 
              ZDIFFD  = 0.0_JPRB
              ZDIFFD5 = 1.66_JPRB
            ENDIF
            ZTAUCLD(JL,JK,JRTM)  = ZDIFFD5 * ZTAUD + ZDIFFD * ZTAUD5
            ZTAUCLD5(JL,JK,JRTM) = ZTAUD5 * ZDIFFD5
          ENDIF

        ENDDO
      ENDDO

    ENDIF
  ENDIF
 
ENDDO

NUAER = NUA
NTRAER = NTRA

!     ------------------------------------------------------------------
!          2.6    SCALING OF OPTICAL THICKNESS
!                 SPECTRALLY, ACCOUNTING FOR VERTICAL VARIABILITY

IF (NINHOM == 1) THEN
!-- simple scaling a la Tiedtke (1996) with RSWINHF in SW and RLWINHF in LW
  DO JSW=1,NSW
    DO JK=1,KLEV
      DO JL=KIDIA,KFDIA
        ZTAU (JL,JSW,JK)=ZTAU (JL,JSW,JK) * RSWINHF
        ZTAU5(JL,JSW,JK)=ZTAU5(JL,JSW,JK) * RSWINHF
      ENDDO
    ENDDO
  ENDDO

!-- scaling of NeuroNet cloud optical thickness (already done)

!-- scaling of RRTM cloud optical thickness (no TL code available)

!  DO JRTM=1,16
!    DO JK=1,KLEV
!      DO JL=KIDIA,KFDIA
!        ZTAUCLD (JL,JK,JRTM) = ZTAUCLD (JL,JK,JRTM) * RLWINHF  
!        ZTAUCLD5(JL,JK,JRTM) = ZTAUCLD5(JL,JK,JRTM) * RLWINHF
!      ENDDO
!    ENDDO
!  ENDDO 
ELSEIF (NINHOM >= 2) THEN
  CALL ABOR1('TL model cannot be used with NINHOM > 1')  
ENDIF

!     ------------------------------------------------------------------

!*         2.7    DIFFUSIVITY FACTOR OR SATELLITE VIEWING ANGLE
!                 ---------------------------------------------

DO JL = KIDIA,KFDIA
  ZVIEW (JL) = 0.0_JPRB
  ZVIEW5(JL) = DIFF
ENDDO

!     ------------------------------------------------------------------

!*         3.     CALL LONGWAVE RADIATION CODE
!                 ----------------------------

!*         3.1    FULL LONGWAVE RADIATION COMPUTATIONS
!                 ------------------------------------

! Initialization of output variables from longwave radiation
! computation

ZEMIT5(:)=0.0_JPRB
ZEMIT (:)=0.0_JPRB
!ZFLUX5(:,:,:)=0.0_JPRB
!ZFLUX (:,:,:)=0.0_JPRB
!ZFLUC5(:,:,:)=0.0_JPRB
!ZFLUC (:,:,:)=0.0_JPRB

IF (LERADN2) THEN

  IF (LERADLW2) THEN 
    ISTEP = MOD(NSTEP,NLWFR)

    IF (ISTEP == 0 .OR. NSTEP == (NSTOP-1)) THEN

!* Computation of the cloud top level up to which cloud contribution
!  is computed in lwc.F90
      IF (LWLCLHR) THEN
        DO JL = KIDIA,KFDIA
          ILEV(JL)=KLEV
        ENDDO
        DO JK = 1 , KLEV
          DO JL = KIDIA,KFDIA
            ZCOND5(JL,JK) = PQLWP5(JL,JK)+PQIWP5(JL,JK)
            IF (PCLFR5(JL,JK)>=1.E-04_JPRB &
             & .AND. ZCOND5(JL,JK)>=0.5E-08_JPRB) THEN
              ILEV(JL) = MIN(ILEV(JL),JK)
            ENDIF
          ENDDO
        ENDDO

        ILEVR=KLEV
        DO JL = KIDIA,KFDIA
          ILEVR = MAX(NLEVLWC,MIN(ILEVR,ILEV(JL)))
        ENDDO
!orig    ITOPC = KLEV-ILEVR
        ITOPC = KLEV-ILEVR+5
      ELSE
        ITOPC = KLEV
      ENDIF
!* End of the cloud top computation

      CALL LWTL &
       & ( KIDIA , KFDIA  , KLON   , KLEV   , KMODE, ITOPC,&
       & PCCO25  , ZVIEW5 , PCLFR5 , ZCLDLD5, ZCLDLU5,&
       & PDP5    , ZDT05  , ZEMIR5 , ZEMIW5,&
       & ZPMB5   , POZON5 , ZTL5   ,&
       & PAER5   , ZTAVE5 , PQ5    ,&
       & ZEMIT5  , ZFLUX5 , ZFLUC5 ,&
!       & PCLFR   , ZCLDLD , ZCLDLU ,&
       & ZCLFR   , ZCLDLD , ZCLDLU ,&
       & PDP     , ZDT0   , ZEMIR  , ZEMIW ,&
       & ZPMB    , POZON  , ZTL    ,&
       & PAER    , ZTAVE  , PQ     ,&
       & ZEMIT   , ZFLUX  , ZFLUC  &
       & )
    ENDIF

  ELSE

    CALL LWPTL &
     & ( KIDIA , KFDIA  , KLON , KLEV ,&
     & PCCO25, ZCLDLD5, ZCLDLU5, PCLFR5,&
     & PDP5  , ZDT05,&
     & ZPMB5 , POZON5 , ZTL5,&
     & ZTAVE5 , PQ5,&
     & ZEMIT5 , ZFLUX5, ZFLUC5,&
   !  &, PCCO2 , ZCLDLD , ZCLDLU , PCLFR &
     & ZCLDLD , ZCLDLU , ZCLFR,&
     & ZDT0   , ZPMB  , ZTL,&
     & ZTAVE  , PQ,&
     & ZEMIT  , ZFLUX , ZFLUC &
     & )
  ENDIF
ENDIF

! ******

!     ------------------------------------------------------------------

!*         4.     CALL SHORTWAVE RADIATION CODE
!                 -----------------------------

ZRMUZ=0.0_JPRB
DO JL = KIDIA,KFDIA
  ZRMUZ = MAX (ZRMUZ, ZMU05(JL))
ENDDO

IF (ZRMUZ > 0.0_JPRB .AND. LERADSW2) THEN

  CALL SWTL &
   & ( KIDIA  , KFDIA  , KLON , KLEV   , KAER,&
   & PRII05 , PCCO25 , ZPSOL5 , ZALBD5 , ZALBP5 , PQ5   , PQS5,&
   & ZMU05  , ZCG5   , ZCLDSW5, ZOMEGA5, ZOZ5   , ZPMB5,&
   & ZTAU5  , ZTAVE5 , PAER5,&
   & ZFACT5 , ZFSDWN5, ZFSUP5,&
!   & ZFCDWN5, ZFCUP5,&
   & ZRII0  , PCCO2  , ZPSOL  , ZALBD  , ZALBP  , PQ    , PQS,&
   & ZMU0   , ZCG    , ZCLDSW , ZOMEGA , ZOZ    , ZPMB,&
   & ZTAU   , ZTAVE  , PAER,&
   & ZFSDWN , ZFSUP&
!   & ZFCDWN , ZFCUP &
   & )  

ENDIF

!     ------------------------------------------------------------------

!*         5.     FILL UP THE MODEL NET LW AND SW RADIATIVE FLUXES
!                 ------------------------------------------------

DO JK = 1 , KLEV+1
  DO JL = KIDIA,KFDIA
    ZFSUP  (JL,JK) = ZFSUP(JL,JK)*ZFACT5(JL)
    ZFSDWN (JL,JK) = ZFSDWN(JL,JK)*ZFACT5(JL)
    ZFSUP5 (JL,JK) = ZFSUP5(JL,JK) * ZFACT5(JL)
    ZFSDWN5(JL,JK) = ZFSDWN5(JL,JK) * ZFACT5(JL)

! diagnostic clear sky quantities
!    ZFCUP  (JL,JK) = ZFCUP(JL,JK)*ZFACT5(JL)
!    ZFCDWN (JL,JK) = ZFCDWN(JL,JK)*ZFACT5(JL)
!    ZFCUP5 (JL,JK) = ZFCUP5(JL,JK) * ZFACT5(JL)
!    ZFCDWN5(JL,JK) = ZFCDWN5(JL,JK) * ZFACT5(JL)
  ENDDO
ENDDO

DO JKL = 1 , KLEV+1
  JK = KLEV+1 + 1 - JKL
  DO JL = KIDIA,KFDIA
    PFLS (JL,JKL) = ZFSDWN (JL,JK) - ZFSUP (JL,JK)
    PFLS5(JL,JKL) = ZFSDWN5(JL,JK) - ZFSUP5(JL,JK)
    PFLT (JL,JKL) = - ZFLUX (JL,1,JK) - ZFLUX (JL,2,JK)
    PFLT5(JL,JKL) = - ZFLUX5(JL,1,JK) - ZFLUX5(JL,2,JK)

! diagnostic clear sky quantities
!    PFCS (JL,JKL) = ZFCDWN (JL,JK) - ZFCUP (JL,JK)
!    PFCS5(JL,JKL) = ZFCDWN5(JL,JK) - ZFCUP5(JL,JK)
!    PFCT (JL,JKL) = - ZFLUC (JL,1,JK) - ZFLUC (JL,2,JK)
!    PFCT5(JL,JKL) = - ZFLUC5(JL,1,JK) - ZFLUC5(JL,2,JK)
  ENDDO
ENDDO

DO JL = KIDIA,KFDIA
  PFRSOD (JL)=ZFSDWN (JL,1)
  PFRSOD5(JL)=ZFSDWN5(JL,1)
ENDDO

!     --------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RADLSWTL',1,ZHOOK_HANDLE)
END SUBROUTINE RADLSWTL
