#ifdef RS6K
@PROCESS HOT(NOVECTOR) NOSTRICT
#endif
SUBROUTINE SLTEND( KIDIA, KFDIA, KLON, KLEV, KTEND,&
 & KSTEP, KSTART,&
 & PT,  PQ, PL, PI, PA, PGFL, PRSF1, PRS1, &
 & PDYNU, PDYNV, PDYNT, PDYNQ, PDYNO3, PDYNA, PDYNCLV, &
 & PTENU, PTENV, PTENT, PTENQ, PTENO3, PTENL, PTENI, PTENA, PTENGFL,&
 & PPHTENU, PPHTENV, PPHTENT, PPHTENQ, PPHTENO3, PPHTENA,&
 & PVFU, PVFV, PVFT, PVFQ, PVFA, PVFCLV, &
 & PSAVTEND, PGFLSLP,&
 & PFSQLF,   PFSQIF )  


!**** *SLTEND * - compute sl tendencies for next time step and the 
!                 updates of the current profile. 
!               - remove supersaturation on final profiles and 
!                 save it for the next time step
!     PURPOSE.
!     --------
!               second order physics, computing the total dynamical and physical 
!               tendencies and the contribution of te physical tendencies
!               to be used in the next time step as part of the first guess
!               and as part of the final profile.

!**   Interface.
!     ----------
!        *CALL* *SLTEND*

! INPUT:

! KIDIA      : START OF HORIZONTAL LOOP
! KFDIA      : END   OF HORIZONTAL LOOP
! KLON       : HORIZONTAL DIMENSION
! KLEV       : END OF VERTICAL LOOP AND VERTICAL DIMENSION
! KTEND      : NUMBER OF PARAMETERS IN PSAVTEND
! KTRAC      : NUMBER OF ACTIVE TRACERS IN PVFC: NGHG+NTRAC+NGRG+NAER+NGFL_EXT
! KSTART     : FIRST STEP OF MODEL
! KSTEP      : CURRENT TIME STEP

! PT         : TEMPERATURE AT TIME t
! PQ         : HUMIDITY AT TIME t
! PL         : CLOUD WATER AT TIME t
! PI         : CLOUD ICE WATER AT TIME t
! PA         : CLOUD FRACTION AT TIME t
! PGFL       : GFL FIELDS 

! PRSF1      : PROVISIONAL T+DT PRESSURE ON FULL LEVELS
! PRS1       : PROVISIONAL T+DT PRESSURE ON HALF LEVELS

! time level (t+1)-(t)
! PDYNU      : EUL. DYNAMICAL TENDENCY OF U-COMP. OF WIND.
! PDYNV      : EUL. DYNAMICAL TENDENCY OF V-COMP. OF WIND.
! PDYNT      : EUL. DYNAMICAL TENDENCY OF TEMPERATURE.
! PDYNQ      : EUL. DYNAMICAL TENDENCY OF HUMIDITY
! PDYNO3     : EUL. DYNAMICAL TENDENCY OF OZONE MIXING RATIO (ECMWF PROG. OZONE)
! PDYNA      : EUL. DYNAMICAL TENDENCY OF CLOUD FRACTION
! PDYNCLV    : EUL. DYNAMICAL TENDENCY OF CLOUD VARIABLS (Not used)

! (note: the updated total cloud liquid water and ice are computed in CLOUDSC !)

! time level (t) at departure point
! PPHTENU    : 0.5*PHYSICAL TENDENCY OF U-COMP. OF WIND
! PPHTENV    : 0.5*PHYSICAL TENDENCY OF V-COMP. OF WIND
! PPHTENT    : 0.5*PHYSICAL TENDENCY OF TEMPERATURE
! PPHTENQ    : 0.5*PHYSICAL TENDENCY OF HUMIDITY
! PPHTENO3   : 0.5*PHYSICAL TENDENCY OF OZONE MIXING RATIO (ECMWF PROG. OZONE)
! PPHTENA    : 0.5*PHYSICAL TENDENCY OF CLOUD FRACTION
! PPHTENGFL  : 0.5*PHYSICAL TENDENCY OF ALL GFL FIELDS

! arriv. point only at time level (t+1)
! PVFU       : ARRIV. POINT TENDENCY FOR VDIF + GWDRAG OF U-COMP. OF WIND
! PVFV       : ARRIV. POINT TENDENCY FOR VDIF + GWDRAG OF V-COMP. OF WIND
! PVFT       : ARRIV. POINT TENDENCY FOR VDIF OF TEMPERATURE
! PVFQ       : ARRIV. POINT TENDENCY FOR VDIF OF HUMIDITY
! PVFA       : ARRIV. POINT TENDENCY FOR VDIF OF CLOUD FRACTION
! PVFCLV     : ARRIV. POINT TENDENCY FOR VDIF OF CLOUD VARIABLES

! UPDATED:

! input : arriv. point only at time level (t+1),
! output: complete arriv. + departure point tendencies including the dynamical tendencies
! PTENU      : TENDENCY OF U-COMP. OF WIND.
! PTENV      : TENDENCY OF V-COMP. OF WIND.
! PTENT      : TENDENCY OF TEMPERATURE.
! PTENQ      : TENDENCY OF HUMIDITY
! PTENO3     : TENDENCY OF OZONE MIXING RATIO (ECMWF PROG. OZONE)
! PTENL      : TENDENCY OF CLOUD WATER
! PTENI      : TENDENCY OF CLOUD ICE WATER
! PTENA      : TENDENCY OF CLOUD FRACTION
! PTENGFL    : TENDENCY OF ALL GFL FIELDS

! OUTPUT:
! PFSQLF : Flux of liquid for DDH enthalpy budget
! PFSQIF : Flux of ice for DDH enthalpy budget

! PSAVTEND   : ARRAY OF TENDENCIES + AUX. SUPERSATURATION TO BE SAVED FOR NEXT TIME STEP

!-----------------------------------------------------------------------
!     Method. See documentation.
!     -------
!     Modifications.
!     --------------
!     ORIGINAL 2001-06-25, Nils Wedi
!     M.Hamrud    : 03-08-01 GFL introduction
!     M.Hamrud    : 01-Oct-2003 CY28 Cleaning
!     A.Untch     : March-2004 Introduced EXTRA GFL fields (GFL_EXT)
!     M.Ko"hler   : 03-12-2004 VDF cloud tendency added for moist
!                              advection-diffusion PBL
!     A.Untch     : 12-03-2005 Aerosols as named GFL fields
!     J.Flemming  : 11-04-2005 Aerosols replaced with GEMS GRG 
!     A.Tompkins  : CY31R2     Changes for supersaturation code
!     P.Bechtold  : 11-12-2005 Reorganize GFL/Tracer part
!     M.Janiskova : 21-12-2005 Modified condition for using cloud tendency
!     D.Salmond     22-Nov-2005 Mods for coarser/finer physics
!     S. Serrar     07-Sep-2006 tracers added for diagnostics (GEMS)
!     P.Bechtold    04-10-2006 Remove GFL tracers as SLTEND does not give positive definit results
!     A.Tompkins/R.Forbes : 15-03-2010  New CLV cloud variables added
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY2  , ONLY : TSPHY
USE YOEPHY   , ONLY : LEO3CH, LEPCLD
USE YOECLDP  , ONLY : RAMIN, RLMIN, NCLDTOP, NSSOPT ,&
 & NCLV, RKOOPTAU 
USE YOMSLPHY , ONLY : RSLWX, MSAT_SAVTEND
USE YOMCST   , ONLY : RG       ,RLVTT    ,RLSTT    ,RTT
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE    ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R,&
 & RKOOP1   ,RKOOP2  

USE YOM_YGFL , ONLY : YGFL, YQ, YO3, YL, YI, YR, YS, YA

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PA(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGFL(KLON,KLEV,YGFL%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSF1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRS1(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNO3(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNA(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENGFL(KLON,KLEV,YGFL%NDIM1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNCLV(KLON,KLEV,NCLV)
!REAL(KIND=JPRB)   ,INTENT(IN)    :: PLITOT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENO3(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENA(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENO3(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENA(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFCLV(KLON,KLEV,NCLV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFA(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSAVTEND(KLON,KLEV,KTEND) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGFLSLP(KLON,KLEV,YGFL%NDIMSLP) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFSQLF(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFSQIF(KLON,KLEV+1) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JK, JL, IK, ICALL, IU, IV, IT

LOGICAL :: LLFLAG(KLON)

! required to record supersaturation adjustments
REAL(KIND=JPRB) :: ZSUBQ(KLON)
REAL(KIND=JPRB) :: ZSUBT(KLON)
REAL(KIND=JPRB) :: ZSUBA(KLON)
REAL(KIND=JPRB) :: ZADJQ(KLON,KLEV)
REAL(KIND=JPRB) :: ZADJT(KLON,KLEV)
REAL(KIND=JPRB) :: ZADJA(KLON,KLEV)
REAL(KIND=JPRB) :: ZADJL(KLON,KLEV)
REAL(KIND=JPRB) :: ZADJI(KLON,KLEV)

! adjustment variables
REAL(KIND=JPRB) :: ZQOLD(KLON),ZPP(KLON),ZQAD(KLON)
REAL(KIND=JPRB) :: ZCONS,ZCONS2
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZLITOT, ZFAC, ZFACCC, ZEPSEC, ZCCICETAU, &
                 & ZRG_R, ZGDPH_R

#include "cuadjtq.intfb.h"
#include "fcttre.h"
#include "fccld.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SLTEND',0,ZHOOK_HANDLE)
ZEPSEC=1.E-14_JPRB
ZCONS=1.0_JPRB/TSPHY
ZCCICETAU=(1.0_JPRB-EXP(-TSPHY/RKOOPTAU)) ! factor for CC supersat adjustment
ZRG_R=1.0_JPRB/RG

ZCONS2=1.0_JPRB
IF (KSTEP==KSTART) THEN
  ZCONS2=0.0_JPRB
ENDIF

IU=1
IV=2
IT=3

LLFLAG(:)=.TRUE.


DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    PSAVTEND(JL,JK,MSAT_SAVTEND)=0.0_JPRB
    ZADJT(JL,JK)=0.0_JPRB
    ZADJQ(JL,JK)=0.0_JPRB
    ZADJA(JL,JK)=0.0_JPRB
    ZADJL(JL,JK)=0.0_JPRB
    ZADJI(JL,JK)=0.0_JPRB
  ENDDO
ENDDO

!-----------------------------------------------------------------------
! note that the supersaturation adjustment is made with respect to 
! liquid saturation:  when T>0C 
! ice saturation:     when T<0C
!                     with an adjustment made to allow for ice 
!                     supersaturation in the clear sky
!
! Note also that the KOOP factor automatically clips the supersaturation
! to a maximum set by the liquid water saturation mixing ratio
! important for temperatures near to but below 0C
!
! This treatment mimics that at the start of CLOUDSC
!----------------------------------------------------------------------- 
DO JK=NCLDTOP,KLEV
  DO JL=KIDIA,KFDIA
    IF (KSTEP==KSTART) THEN
      ZSUBT(JL)=PT(JL,JK)+TSPHY*PTENT(JL,JK)
      ZSUBQ(JL)=PQ(JL,JK)+TSPHY*PTENQ(JL,JK)
      ZSUBA(JL)=PA(JL,JK)+TSPHY*PTENA(JL,JK)
    ELSE
      ZSUBT(JL)=PT(JL,JK)+RSLWX*TSPHY*PTENT(JL,JK)+&
       & TSPHY*(1.0_JPRB-RSLWX)*(PVFT(JL,JK)+PDYNT(JL,JK))+PPHTENT(JL,JK)  
      ZSUBQ(JL)=PQ(JL,JK)+RSLWX*TSPHY*PTENQ(JL,JK)+&
       & TSPHY*(1.0_JPRB-RSLWX)*(PVFQ(JL,JK)+PDYNQ(JL,JK))+PPHTENQ(JL,JK)  
      ZSUBA(JL)=PA(JL,JK)+RSLWX*TSPHY*PTENA(JL,JK)+&
       & TSPHY*(1.0_JPRB-RSLWX)*(PVFA(JL,JK)+PDYNA(JL,JK))+PPHTENA(JL,JK)  
    ENDIF

    ZQOLD(JL)=ZSUBQ(JL)
    ZSUBT(JL)=MAX(ZSUBT(JL),160.0_JPRB) !safety
    ZPP(JL)=PRSF1(JL,JK)
  ENDDO

  IK=1
  ICALL=5
  CALL CUADJTQ &
   & (KIDIA,    KFDIA,    KLON,     1,        IK,&
   &  ZPP,      ZSUBT,    ZSUBQ,    LLFLAG,   ICALL)  
  
  DO JL=KIDIA,KFDIA
    ZGDPH_R=-ZRG_R*(PRS1(JL,JK+1)-PRS1(JL,JK))
    !----------------------------
    ! supersaturation adjustments
    !----------------------------
    IF (PT(JL,JK)>=RTT .OR. NSSOPT==0) THEN
      ZFAC=1.0_JPRB
      ZFACCC=1.0_JPRB
    ELSE
      ZFAC=ZSUBA(JL)+FOKOOP(PT(JL,JK))*(1.0_JPRB-ZSUBA(JL))
!      ZFAC=PA(JL,JK)+FOKOOP(PT(JL,JK))*(1.0_JPRB-PA(JL,JK))
      ZFACCC=ZCCICETAU
    ENDIF

    ZSUBQ(JL)=ZSUBQ(JL)*ZFAC
    ZQAD(JL)=ZCONS*MAX(ZQOLD(JL)-ZSUBQ(JL),0.0_JPRB)

    !-------------------------------------------------------------------
    ! Here the supersaturation is turned into liquid water
    ! However, if the temperature is below the threshold for homogeneous
    ! freezing then the supersaturation is turned instantly to ice.
    !--------------------------------------------------------------------
!    IF (ZQAD(JL)>ZEPSEC) THEN 
      IF (PT(JL,JK)>=RTT) THEN
        ZADJT(JL,JK)=RALVDCP*ZQAD(JL)
      ELSE
        ZADJT(JL,JK)=RALSDCP*ZQAD(JL)
      ENDIF
!    ENDIF
    ZADJQ(JL,JK)=ZQAD(JL)
    ! any surplus is added to the cloud at the next time-step
    ! to ensure consistent treatment from cloud scheme
    PSAVTEND(JL,JK,MSAT_SAVTEND)=ZQAD(JL)*TSPHY
  ENDDO

  DO JL=KIDIA,KFDIA
    ZSUBT(JL)=MAX(ZSUBT(JL),160.0_JPRB) !safety
    ZQOLD(JL)=ZSUBQ(JL)
  ENDDO

ENDDO

!---save physics tendencies for next time step(t+1-->t)
! If all true then speed up code, otherwise check on indiviual switches
IF( LEO3CH .AND. YL%LACTIVE .AND. YI%LACTIVE .AND. YA%LT1 &
  &  .AND. YR%LACTIVE .AND. YS%LACTIVE ) THEN
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      PSAVTEND(JL,JK,IU)=(1.0_JPRB-RSLWX)*(PTENU(JL,JK)-PVFU(JL,JK)-PDYNU(JL,JK))
      PSAVTEND(JL,JK,IV)=(1.0_JPRB-RSLWX)*(PTENV(JL,JK)-PVFV(JL,JK)-PDYNV(JL,JK))
      PSAVTEND(JL,JK,IT)=(1.0_JPRB-RSLWX)*(PTENT(JL,JK)-PVFT(JL,JK)-PDYNT(JL,JK))
      PGFLSLP(JL,JK,YQ%MPSLP)=(1.0_JPRB-RSLWX)*(PTENQ(JL,JK)-PVFQ(JL,JK)-PDYNQ(JL,JK))
      PGFLSLP(JL,JK,YO3%MPSLP)=(1.0_JPRB-RSLWX)*(PTENO3(JL,JK)-PDYNO3(JL,JK))
      PGFLSLP(JL,JK,YA%MPSLP)=(1.0_JPRB-RSLWX)*(PTENA(JL,JK)-PVFA(JL,JK)-PDYNA(JL,JK))
      !------------------------------------
      ! all cloud variables are first order
      !------------------------------------ 
      PGFLSLP(JL,JK,YL%MPSLP)= PGFL(JL,JK,YL%MP9)+TSPHY*PTENGFL(JL,JK,YL%MP1)
      PGFLSLP(JL,JK,YI%MPSLP)= PGFL(JL,JK,YI%MP9)+TSPHY*PTENGFL(JL,JK,YI%MP1)
      PGFLSLP(JL,JK,YR%MPSLP)= PGFL(JL,JK,YR%MP9)+TSPHY*PTENGFL(JL,JK,YR%MP1)
      PGFLSLP(JL,JK,YS%MPSLP)= PGFL(JL,JK,YS%MP9)+TSPHY*PTENGFL(JL,JK,YS%MP1)
    ENDDO
  ENDDO
ELSE
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      PSAVTEND(JL,JK,IU)=(1.0_JPRB-RSLWX)*(PTENU(JL,JK)-PVFU(JL,JK)-PDYNU(JL,JK))
      PSAVTEND(JL,JK,IV)=(1.0_JPRB-RSLWX)*(PTENV(JL,JK)-PVFV(JL,JK)-PDYNV(JL,JK))
      PSAVTEND(JL,JK,IT)=(1.0_JPRB-RSLWX)*(PTENT(JL,JK)-PVFT(JL,JK)-PDYNT(JL,JK))

      PGFLSLP(JL,JK,YQ%MPSLP)=(1.0_JPRB-RSLWX)*(PTENQ(JL,JK)-PVFQ(JL,JK)-PDYNQ(JL,JK))
      IF (LEO3CH) PGFLSLP(JL,JK,YO3%MPSLP)= &
                 & (1.0_JPRB-RSLWX)*(PTENO3(JL,JK)-PDYNO3(JL,JK))
      IF (YA%LT1) PGFLSLP(JL,JK,YA%MPSLP)= &
                 & (1.0_JPRB-RSLWX)*(PTENA(JL,JK)-PVFA(JL,JK)-PDYNA(JL,JK))
  
      !------------------------------------
      ! all cloud variables are first order
      !------------------------------------ 
      IF (YL%LACTIVE) PGFLSLP(JL,JK,YL%MPSLP)= &
                      & PGFL(JL,JK,YL%MP9)+TSPHY*PTENGFL(JL,JK,YL%MP1)
      IF (YI%LACTIVE) PGFLSLP(JL,JK,YI%MPSLP)= &
                      & PGFL(JL,JK,YI%MP9)+TSPHY*PTENGFL(JL,JK,YI%MP1)
      IF (YR%LACTIVE) PGFLSLP(JL,JK,YR%MPSLP)= &
                      & PGFL(JL,JK,YR%MP9)+TSPHY*PTENGFL(JL,JK,YR%MP1)
      IF (YS%LACTIVE) PGFLSLP(JL,JK,YS%MPSLP)= &
                      & PGFL(JL,JK,YS%MP9)+TSPHY*PTENGFL(JL,JK,YS%MP1)

    ENDDO
  ENDDO
ENDIF

!---create updates for current time-step(t+1)
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    PTENU(JL,JK)=PTENU(JL,JK)-ZCONS2*PSAVTEND(JL,JK,IU)+ZCONS*PPHTENU(JL,JK)
    PTENV(JL,JK)=PTENV(JL,JK)-ZCONS2*PSAVTEND(JL,JK,IV)+ZCONS*PPHTENV(JL,JK)

    ! add supersaturation adjustement to t+1 terms only
    PTENT(JL,JK)=PTENT(JL,JK)-ZCONS2*PSAVTEND(JL,JK,IT)+ZADJT(JL,JK)&
     & +ZCONS*PPHTENT(JL,JK)  
    PTENQ(JL,JK)=PTENQ(JL,JK)-ZCONS2*PGFLSLP(JL,JK,YQ%MPSLP)-ZADJQ(JL,JK)&
     & +ZCONS*PPHTENQ(JL,JK)  
    IF(YA%LT1) THEN
      PTENA(JL,JK)=PTENA(JL,JK)-ZCONS2*PGFLSLP(JL,JK,YA%MPSLP)+ZCONS*PPHTENA(JL,JK)
    ENDIF

!    ZALFAW = FOEALFA(PT(JL,JK)+TSPHY*PTENT(JL,JK))
! 1st order, as before
!    PTENL(JL,JK)=ZCONS*(ZALFAW*PLITOT(JL,JK)-PL(JL,JK))
!    PTENI(JL,JK)=ZCONS*((1.0_JPRB-ZALFAW)*PLITOT(JL,JK)-PI(JL,JK))
    PTENL(JL,JK)=PTENGFL(JL,JK,YL%MP1)
    PTENI(JL,JK)=PTENGFL(JL,JK,YI%MP1)

    IF(LEO3CH) THEN
      PTENO3(JL,JK)=PTENO3(JL,JK)-ZCONS2*PGFLSLP(JL,JK,YO3%MPSLP)+ZCONS*PPHTENO3(JL,JK)
    ENDIF
  ENDDO
ENDDO

! Tidy up small cloud/precip values
IF( LEPCLD ) THEN
  DO JK=1,KLEV
    DO JL=KIDIA,KFDIA
      IF (YL%LT1) THEN
        IF (PGFLSLP(JL,JK,YL%MPSLP)<RLMIN) THEN
          PTENGFL(JL,JK,YL%MP1)=-ZCONS*PGFL(JL,JK,YL%MP9)
          PGFLSLP(JL,JK,YL%MPSLP)=0.0_JPRB
        ENDIF
      ENDIF
      IF (YI%LACTIVE) THEN
        IF (PGFLSLP(JL,JK,YI%MPSLP)<RLMIN) THEN
          PTENGFL(JL,JK,YI%MP1)=-ZCONS*PGFL(JL,JK,YI%MP9)
          PGFLSLP(JL,JK,YI%MPSLP)=0.0_JPRB
        ENDIF
      ENDIF
      IF (YR%LACTIVE) THEN
        IF (PGFLSLP(JL,JK,YR%MPSLP)<RLMIN) THEN
          PTENGFL(JL,JK,YR%MP1)=-ZCONS*PGFL(JL,JK,YR%MP9)
          PGFLSLP(JL,JK,YR%MPSLP)=0.0_JPRB
        ENDIF
      ENDIF
      IF (YS%LACTIVE) THEN
        IF (PGFLSLP(JL,JK,YS%MPSLP)<RLMIN) THEN
          PTENGFL(JL,JK,YS%MP1)=-ZCONS*PGFL(JL,JK,YS%MP9)
          PGFLSLP(JL,JK,YS%MPSLP)=0.0_JPRB
        ENDIF
      ENDIF
      IF (YL%LACTIVE.AND.YI%LACTIVE.AND.YA%LT1) THEN
        ZLITOT=PGFLSLP(JL,JK,YL%MPSLP)+PGFLSLP(JL,JK,YI%MPSLP)
       IF (PA(JL,JK)+TSPHY*PTENA(JL,JK)<RAMIN.OR.ZLITOT<RLMIN) THEN 
!        IF (PA(JL,JK)+TSPHY*PTENA(JL,JK)<RAMIN) THEN 
          PTENA(JL,JK)=-ZCONS*PA(JL,JK)
          PGFLSLP(JL,JK,YA%MPSLP)=0.0_JPRB
          PTENGFL(JL,JK,YL%MP1)=-ZCONS*PGFL(JL,JK,YL%MP9)
          PGFLSLP(JL,JK,YL%MPSLP)=0.0_JPRB
          PTENGFL(JL,JK,YI%MP1)=-ZCONS*PGFL(JL,JK,YI%MP9)
          PGFLSLP(JL,JK,YI%MPSLP)=0.0_JPRB
        ENDIF
      ENDIF
      IF (YA%LT1) THEN
        IF( (PA(JL,JK)+TSPHY*PTENA(JL,JK)) > 1.0_JPRB ) THEN
          PTENA(JL,JK)=ZCONS*(1.0_JPRB-PA(JL,JK))
        ENDIF
      ENDIF
    ENDDO
  ENDDO
ENDIF


IF (LHOOK) CALL DR_HOOK('SLTEND',1,ZHOOK_HANDLE)
END SUBROUTINE SLTEND
