SUBROUTINE SLTEND1( KIDIA, KFDIA, KLON, KLEV, KTEND,&
 & PDYNU, PDYNV, PDYNT, PDYNQ, PDYNO3, PDYNA, PLITOT,&
 & PTENU, PTENV, PTENT, PTENQ, PTENO3, PTENA,&
 & PVFU, PVFV, PVFT, PVFQ, PVFA,&
 & PSAVTEND,PGFLSLP)  


!**** *SLTEND1 * - compute sl tendencies for next time step and the 
!                 updates of the current profile. 
!               - remove supersaturation on final profiles and 
!                 save it for the next time step
!     PURPOSE.
!     --------
!               second order physics, computing the total dynamical and physical 
!               tendencies and the contribution of te physical tendencies
!               to be used in the next time step as part of the first guess
!               and as part of the final profile.

!**   Interface.
!     ----------
!        *CALL* *SLTEND1*

! INPUT:

! KIDIA      : START OF HORIZONTAL LOOP
! KFDIA      : END   OF HORIZONTAL LOOP
! KLON       : HORIZONTAL DIMENSION
! KLEV       : END OF VERTICAL LOOP AND VERTICAL DIMENSION
! KTEND      : NUMBER OF PARAMETERS IN PSAVTEND

! time level (t+1)-(t)
! PDYNU      : EUL. DYNAMICAL TENDENCY OF U-COMP. OF WIND.
! PDYNV      : EUL. DYNAMICAL TENDENCY OF V-COMP. OF WIND.
! PDYNT      : EUL. DYNAMICAL TENDENCY OF TEMPERATURE.
! PDYNQ      : EUL. DYNAMICAL TENDENCY OF HUMIDITY
! PDYNO3     : EUL. DYNAMICAL TENDENCY OF OZONE MIXING RATIO (ECMWF PROG. OZONE)
! PDYNA      : EUL. DYNAMICAL TENDENCY OF CLOUD FRACTION
! (note: the updated total cloud liquid water and ice are computed in CLOUDSC !)
! PLITOT     : TOTAL SUM OF CLOUD LIQUID WATER AND ICE

! arriv. point only at time level (t+1)
! PVFU       : ARRIV. POINT TENDENCY FOR VDIF + GWDRAG OF U-COMP. OF WIND
! PVFV       : ARRIV. POINT TENDENCY FOR VDIF + GWDRAG OF V-COMP. OF WIND
! PVFT       : ARRIV. POINT TENDENCY FOR VDIF OF TEMPERATURE
! PVFQ       : ARRIV. POINT TENDENCY FOR VDIF OF HUMIDITY
! PVFA       : ARRIV. POINT TENDENCY FOR VDIF OF CLOUD FRACTION

! UPDATED:

! input : arriv. point only at time level (t+1),
! output: complete arriv. + departure point tendencies including the dynamical tendencies
! PTENU      : TENDENCY OF U-COMP. OF WIND.
! PTENV      : TENDENCY OF V-COMP. OF WIND.
! PTENT      : TENDENCY OF TEMPERATURE.
! PTENQ      : TENDENCY OF HUMIDITY
! PTENO3     : TENDENCY OF OZONE MIXING RATIO (ECMWF PROG. OZONE)
! PTENL      : TENDENCY OF CLOUD WATER
! PTENI      : TENDENCY OF CLOUD ICE WATER
! PTENA      : TENDENCY OF CLOUD FRACTION
! PTENGFL    : TENDENCY OF ALL GFL FIELDS

! OUTPUT:

! PSAVTEND   : ARRAY OF TENDENCIES + AUX. SUPERSATURATION TO BE SAVED FOR NEXT TIME STEP

!-----------------------------------------------------------------------
!     Method. See documentation.
!     -------
!     Modifications.
!     --------------
!     ORIGINAL 2001-06-25, Nils Wedi
!     M.Hamrud    : 03-08-01 GFL introduction
!     M.Hamrud    : 01-Oct-2003 CY28 Cleaning
!     A.Untch     : March-2004 Introduced EXTRA GFL fields (GFL_EXT)
!     M.Ko"hler   : 03-12-2004 VDF cloud tendency added for moist
!                              advection-diffusion PBL
!     A.Untch     : 12-03-2005 Aerosols as named GFL fields
!     P.Bechtold  : 11-12-2005 Reorganize GFL/Tracer part
!     M.Janiskova : 21-12-2005 Modified condition for using cloud tendency
!     D.Salmond     22-Nov-2005 Mods for coarser/finer physics
!     S. Serrar     07-Sep-2006 Tracers added foe diagnostics (GEMS)
!     P.Bechtold    04-10-2006 Remove GFL tracers as SLTEND does not give positive definit results
!-------------------------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEPHY   , ONLY : LEO3CH
USE YOMSLPHY , ONLY : RSLWX, MU_SAVTEND_S,MV_SAVTEND_S,MT_SAVTEND_S,MSAVTEND_S
USE YOM_YGFL , ONLY : YGFL,YQ, YO3, YL, YI, YA

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNO3(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDYNA(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLITOT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENO3(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENA(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVFA(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSAVTEND(KLON,KLEV,KTEND) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGFLSLP(KLON,KLEV,YGFL%NDIMSLP) 
!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JK, JL

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SLTEND1',0,ZHOOK_HANDLE)
!---sav phy. tend for next time step(t+1-->t)
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    PSAVTEND(JL,JK,MU_SAVTEND_S)=(1.0_JPRB-RSLWX)*(PTENU(JL,JK)-PVFU(JL,JK)-PDYNU(JL,JK))
    PSAVTEND(JL,JK,MV_SAVTEND_S)=(1.0_JPRB-RSLWX)*(PTENV(JL,JK)-PVFV(JL,JK)-PDYNV(JL,JK))
    PSAVTEND(JL,JK,MT_SAVTEND_S)=(1.0_JPRB-RSLWX)*(PTENT(JL,JK)-PVFT(JL,JK)-PDYNT(JL,JK))

    PGFLSLP(JL,JK,YQ%MPSLP+MSAVTEND_S)=(1.0_JPRB-RSLWX)*(PTENQ(JL,JK)-PVFQ(JL,JK)-PDYNQ(JL,JK))
    IF(LEO3CH) THEN
      PGFLSLP(JL,JK,YO3%MPSLP+MSAVTEND_S)=(1.0_JPRB-RSLWX)*(PTENO3(JL,JK)-PDYNO3(JL,JK))
    ENDIF
    IF(YL%LACTIVE) THEN
      PGFLSLP(JL,JK,YL%MPSLP+MSAVTEND_S)=PLITOT(JL,JK)
    ENDIF
    IF(YI%LACTIVE) THEN
      PGFLSLP(JL,JK,YI%MPSLP+MSAVTEND_S)=PLITOT(JL,JK)
    ENDIF
    IF(YA%LT1) THEN
      PGFLSLP(JL,JK,YA%MPSLP+MSAVTEND_S)=(1.0_JPRB-RSLWX)*(PTENA(JL,JK)-PVFA(JL,JK)-PDYNA(JL,JK))
    ENDIF
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('SLTEND1',1,ZHOOK_HANDLE)
END SUBROUTINE SLTEND1
