SUBROUTINE SLTEND2( KIDIA, KFDIA, KLON, KLEV, KTEND,&
 & KSTEP, KSTART,&
 & PT,  PQ, PL, PI, PA, &
 & PTENU, PTENV, PTENT, PTENQ, PTENO3, PTENL, PTENI, PTENA,&
 & PPHTENU, PPHTENV, PPHTENT, PPHTENQ, PPHTENO3, PPHTENA,&
 & PSAVTEND,PGFLSLP,PGMVT1S)  


!**** *SLTEND2 * - compute sl tendencies for next time step and the 
!                 updates of the current profile. 
!               - remove supersaturation on final profiles and 
!                 save it for the next time step
!     PURPOSE.
!     --------
!               second order physics, computing the total dynamical and physical 
!               tendencies and the contribution of te physical tendencies
!               to be used in the next time step as part of the first guess
!               and as part of the final profile.

!**   Interface.
!     ----------
!        *CALL* *SLTEND2*

! INPUT:

! KIDIA      : START OF HORIZONTAL LOOP
! KFDIA      : END   OF HORIZONTAL LOOP
! KLON       : HORIZONTAL DIMENSION
! KLEV       : END OF VERTICAL LOOP AND VERTICAL DIMENSION
! KTEND      : NUMBER OF PARAMETERS IN PSAVTEND
! KSTART     : FIRST STEP OF MODEL
! KSTEP      : CURRENT TIME STEP

! PT         : TEMPERATURE AT TIME t
! PQ         : HUMIDITY AT TIME t
! PL         : CLOUD WATER AT TIME t
! PI         : CLOUD ICE WATER AT TIME t
! PA         : CLOUD FRACTION AT TIME t

! (note: the updated total cloud liquid water and ice are computed in CLOUDSC !)

! time level (t) at departure point
! PPHTENU    : 0.5*PHYSICAL TENDENCY OF U-COMP. OF WIND
! PPHTENV    : 0.5*PHYSICAL TENDENCY OF V-COMP. OF WIND
! PPHTENT    : 0.5*PHYSICAL TENDENCY OF TEMPERATURE
! PPHTENQ    : 0.5*PHYSICAL TENDENCY OF HUMIDITY
! PPHTENO3   : 0.5*PHYSICAL TENDENCY OF OZONE MIXING RATIO (ECMWF PROG. OZONE)
! PPHTENA    : 0.5*PHYSICAL TENDENCY OF CLOUD FRACTION

! UPDATED:

! input : arriv. point only at time level (t+1),
! output: complete arriv. + departure point tendencies including the dynamical tendencies
! PTENU      : TENDENCY OF U-COMP. OF WIND.
! PTENV      : TENDENCY OF V-COMP. OF WIND.
! PTENT      : TENDENCY OF TEMPERATURE.
! PTENQ      : TENDENCY OF HUMIDITY
! PTENO3     : TENDENCY OF OZONE MIXING RATIO (ECMWF PROG. OZONE)
! PTENL      : TENDENCY OF CLOUD WATER
! PTENI      : TENDENCY OF CLOUD ICE WATER
! PTENA      : TENDENCY OF CLOUD FRACTION

! OUTPUT:

! PSAVTEND   : ARRAY OF TENDENCIES + AUX. SUPERSATURATION TO BE SAVED FOR NEXT TIME STEP

!-----------------------------------------------------------------------
!     Method. See documentation.
!     -------
!     Modifications.
!     --------------
!     ORIGINAL 2001-06-25, Nils Wedi
!     M.Hamrud    : 03-08-01 GFL introduction
!     M.Hamrud    : 01-Oct-2003 CY28 Cleaning
!     A.Untch     : March-2004 Introduced EXTRA GFL fields (GFL_EXT)
!     M.Ko"hler   : 03-12-2004 VDF cloud tendency added for moist
!                              advection-diffusion PBL
!     A.Untch     : 12-03-2005 Aerosols as named GFL fields
!     P.Bechtold  : 11-12-2005 Reorganize GFL/Tracer part
!     M.Janiskova : 21-12-2005 Modified condition for using cloud tendency
!     D.Salmond     22-Nov-2005 Mods for coarser/finer physics
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMPHY2  , ONLY : TSPHY
USE YOEPHY   , ONLY : LEO3CH, LEPCLD, LEQNGT
USE YOECLDP  , ONLY : RAMIN, RLMIN, NCLDTOP, NSSOPT
USE YOMCST   , ONLY :        RLVTT    ,RLSTT    ,RTT
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE    ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R,&
 & RKOOP1   ,RKOOP2  

USE YOM_YGFL , ONLY : YGFL,YQ, YO3, YL, YI, YA
USE YOMSLPHY , ONLY :  MSAT_SAVTEND_S,MU_SAVTEND_S,MV_SAVTEND_S,MT_SAVTEND_S,&
 & MSAVTEND_S
USE YOMGEM   , ONLY : VBH      ,VAH
USE YOMDYN   , ONLY : NSITER   ,NCURRENT_ITER
USE YOMCT0   , ONLY : LPC_FULL, LPC_OLD


IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PA(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENO3(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENI(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENA(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENQ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENO3(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPHTENA(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSAVTEND(KLON,KLEV,KTEND) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGFLSLP(KLON,KLEV,YGFL%NDIMSLP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVT1S(KLON) 
!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JK, JL, IK, ICALL

LOGICAL :: LLFLAG(KLON)

REAL(KIND=JPRB) :: ZSUBQ(KLON)
REAL(KIND=JPRB) :: ZSUBT(KLON)
REAL(KIND=JPRB) :: ZADJQ(KLON,KLEV)
REAL(KIND=JPRB) :: ZADJT(KLON,KLEV)
REAL(KIND=JPRB) :: ZPRSF1(KLON,KLEV), ZPRS1(KLON,0:KLEV)
REAL(KIND=JPRB) :: ZFCQNG(KLON,0:KLEV)

! adjustment variables
REAL(KIND=JPRB) :: ZQOLD(KLON),ZPP(KLON),ZQAD(KLON)
REAL(KIND=JPRB) :: ZCONS,ZCONS2,ZLITOT,ZALFAW
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "qnegat.intfb.h"
#include "cuadjtq.intfb.h"
#include "fcttre.h"
#include "fccld.h"
#include "gppre.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SLTEND2',0,ZHOOK_HANDLE)
ZCONS=1.0_JPRB/TSPHY

ZCONS2=1.0_JPRB
IF (KSTEP==KSTART) THEN
  ZCONS2=0.0_JPRB
ENDIF

LLFLAG(:)=.TRUE.

DO JL=KIDIA,KFDIA
  ZPRS1(JL,KLEV) = EXP(PGMVT1S(JL))
ENDDO
CALL GPPRE(KLON,KIDIA,KFDIA,KLEV,VAH,VBH,ZPRS1,ZPRSF1)

DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    PSAVTEND(JL,JK,MSAT_SAVTEND_S)=0.0_JPRB
    ZADJT(JL,JK)=0.0_JPRB
    ZADJQ(JL,JK)=0.0_JPRB
  ENDDO
ENDDO

IF( (LPC_FULL .AND. NCURRENT_ITER==NSITER).OR. &
 &(.NOT.LPC_FULL.AND..NOT.LPC_OLD) ) THEN

  DO JK=NCLDTOP,KLEV
    DO JL=KIDIA,KFDIA
  
      IF (KSTEP==KSTART) THEN
        ZSUBT(JL)=PT(JL,JK)+TSPHY*PTENT(JL,JK)
        ZSUBQ(JL)=PQ(JL,JK)+TSPHY*PTENQ(JL,JK)
      ELSE
        ZSUBT(JL)=PT(JL,JK)+TSPHY*(PTENT(JL,JK)- &
                  & ZCONS2*PSAVTEND(JL,JK,MT_SAVTEND_S)&
       & +ZCONS*PPHTENT(JL,JK))
        ZSUBQ(JL)=PQ(JL,JK)+TSPHY*(PTENQ(JL,JK)- &
                  & ZCONS2*PGFLSLP(JL,JK,YQ%MPSLP+MSAVTEND_S)&
       & +ZCONS*PPHTENQ(JL,JK))
      ENDIF
  
      ZQOLD(JL)=ZSUBQ(JL)
      ZSUBT(JL)=MAX(ZSUBT(JL),160.0_JPRB) !safety
      ZPP(JL)=ZPRSF1(JL,JK)
    ENDDO

    IK=1
    ICALL=5
    CALL CUADJTQ &
     & (KIDIA,    KFDIA,    KLON,     1,        IK,&
     &  ZPP,      ZSUBT,    ZSUBQ,    LLFLAG,   ICALL)  
  
    DO JL=KIDIA,KFDIA
      IF (PT(JL,JK)<RTICE .AND. NSSOPT /= 0) &
        & ZSUBQ(JL)=ZSUBQ(JL)*(PA(JL,JK)+FOKOOP(PT(JL,JK))*(1.0_JPRB-PA(JL,JK))) 

      ZQAD(JL)=MAX(ZQOLD(JL)-ZSUBQ(JL),0.0_JPRB)
      ZADJQ(JL,JK)=ZCONS*ZQAD(JL)
      ZADJT(JL,JK)=ZCONS*FOELDCPM(ZSUBT(JL))*ZQAD(JL)
! any surplus is added to the rain at the next time-step
! to ensure consistent treatment from cloud scheme
      PSAVTEND(JL,JK,MSAT_SAVTEND_S)=ZQAD(JL)
    ENDDO

    DO JL=KIDIA,KFDIA
      ZSUBT(JL)=MAX(ZSUBT(JL),160.0_JPRB) !safety
      ZQOLD(JL)=ZSUBQ(JL)
    ENDDO

    IF (NSSOPT == 0) THEN
      IK=1
      ICALL=5
      CALL CUADJTQ &
     & (KIDIA,    KFDIA,    KLON,     1,        IK,&
     &  ZPP,      ZSUBT,    ZSUBQ,    LLFLAG,   ICALL)  
 
      DO JL=KIDIA,KFDIA
        ZQAD(JL)=MAX(ZQOLD(JL)-ZSUBQ(JL),0.0_JPRB)
        ZADJQ(JL,JK)=ZADJQ(JL,JK)+ZCONS*ZQAD(JL)
        ZADJT(JL,JK)=ZADJT(JL,JK)+ZCONS*FOELDCPM(ZSUBT(JL))*ZQAD(JL)
! save surplus for next time-step
        PSAVTEND(JL,JK,MSAT_SAVTEND_S)=PSAVTEND(JL,JK,MSAT_SAVTEND_S)+ZQAD(JL)
      ENDDO
    ENDIF
  ENDDO

ENDIF

!---create updates for current time-step(t+1)
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    PTENU(JL,JK)=PTENU(JL,JK)-ZCONS2*PSAVTEND(JL,JK,MU_SAVTEND_S)+ZCONS*PPHTENU(JL,JK)
    PTENV(JL,JK)=PTENV(JL,JK)-ZCONS2*PSAVTEND(JL,JK,MV_SAVTEND_S)+ZCONS*PPHTENV(JL,JK)
    ! add supersaturation adjustement to t+1 terms only
    PTENT(JL,JK)=PTENT(JL,JK)-ZCONS2*PSAVTEND(JL,JK,MT_SAVTEND_S)+ZADJT(JL,JK)&
     & +ZCONS*PPHTENT(JL,JK)  
    PTENQ(JL,JK)=PTENQ(JL,JK)-ZCONS2*PGFLSLP(JL,JK,YQ%MPSLP+MSAVTEND_S)-ZADJQ(JL,JK)&
     & +ZCONS*PPHTENQ(JL,JK)  

! The future total amount of liquid water and ice is in PGFLSLP(JL,JK,YL%MPSLP+MSAVTEND_S)
! The total amount of liquid water and ice interpolated from dep. point
! is in PGFLSLP(JL,JK,YL%MPSLP)
! The Eulerian like dynamic tendencies of liquid water and ice 
! and vdf contributions from physics are in PTENL and PTENI resp.
! The PTEN[X] are reset on every time/iteration step in GPMKTEND and stored physics added
    ZALFAW = FOEALFA(PT(JL,JK)+TSPHY*PTENT(JL,JK))
! 2nd order 
    ZLITOT = 0.5*(PGFLSLP(JL,JK,YL%MPSLP+MSAVTEND_S)+PGFLSLP(JL,JK,YL%MPSLP))
! 1st order 
!    ZLITOT = PGFLSLP(JL,JK,YL%MPSLP+MSAVTEND_S)
    PTENL(JL,JK)=ZCONS*(ZALFAW*ZLITOT-PL(JL,JK))
    PTENI(JL,JK)=ZCONS*((1.0_JPRB-ZALFAW)*ZLITOT-PI(JL,JK))

    IF(LEO3CH) THEN
      PTENO3(JL,JK)=PTENO3(JL,JK)-ZCONS2*PGFLSLP(JL,JK,YO3%MPSLP+MSAVTEND_S)+ZCONS*PPHTENO3(JL,JK)
    ENDIF
    IF(YA%LT1) THEN
      PTENA(JL,JK)=PTENA(JL,JK)-ZCONS2*PGFLSLP(JL,JK,YA%MPSLP+MSAVTEND_S)+ZCONS*PPHTENA(JL,JK)
    ENDIF

! fix cloud cover and liquid water overshoots
    IF( (LPC_FULL .AND. NCURRENT_ITER==NSITER).OR. &
     &(.NOT.LPC_FULL.AND..NOT.LPC_OLD) ) THEN
      IF( LEPCLD ) THEN
        ZLITOT=PL(JL,JK)+PI(JL,JK)+TSPHY*(PTENL(JL,JK)+PTENI(JL,JK))
        IF((PA(JL,JK)+TSPHY*PTENA(JL,JK)) < RAMIN.OR. ZLITOT < RLMIN) THEN
          PTENL(JL,JK)=-ZCONS*PL(JL,JK)
          PGFLSLP(JL,JK,YL%MPSLP+MSAVTEND_S)=0.0_JPRB
          PTENI(JL,JK)=-ZCONS*PI(JL,JK)
          PGFLSLP(JL,JK,YI%MPSLP+MSAVTEND_S)=0.0_JPRB
          PTENA(JL,JK)=-ZCONS*PA(JL,JK)
          PGFLSLP(JL,JK,YA%MPSLP+MSAVTEND_S)=0.0_JPRB
        ENDIF
        IF( (PA(JL,JK)+TSPHY*PTENA(JL,JK)) > 1.0_JPRB ) THEN
          PTENA(JL,JK)=ZCONS*(1.0_JPRB-PA(JL,JK))
        ENDIF
      ENDIF
    ENDIF

  ENDDO
ENDDO

IF( .NOT.LEQNGT ) THEN
!*        14.3     NECESSARY COMPUTATIONS IF SUBROUTINE IS BY-PASSED.
  DO JK=0,KLEV
    DO JL=KIDIA,KFDIA
      ZFCQNG(JL,JK)=0.0_JPRB
    ENDDO
  ENDDO
ELSE
!       Full-Budget-Run
!       WRITE QNEGATICE
!  IF(LBUD23) THEN
!    ZQFIELD(:,:)=PTENQ(:,:)
!  ENDIF
!*         14.1   CALL QNEGAT
  CALL QNEGAT &
   & ( KIDIA , KFDIA , KLON , KLEV,&
   & TSPHY,&
   & PQ , PTENQ, ZPRS1,&
   ! FLUX OUTPUTS
   & ZFCQNG )
ENDIF

IF (LHOOK) CALL DR_HOOK('SLTEND2',1,ZHOOK_HANDLE)
END SUBROUTINE SLTEND2
