SUBROUTINE STOCHADIATEN (KIDIA,KFDIA,KLON,KLEV,&
 & PTSPHY,PTSL,PQSL,PA,PAP,&
 & PTENUIN,PTENVIN,PTENTIN,PTENQIN,&
 & PSTOPHU,PSTOPHV,PSTOPHT,PSTOPHQ,&
 & PTENU  ,PTENV  ,PTENT  ,PTENQ  )  

!**** *STOCHADIATEN * - Add stochastic diabatic perturbation

!     PURPOSE.
!     --------
!       Add stochastic diabatic perturbation

!**   INTERFACE.
!     ----------
!        *CALL* *STOCHADIATEN(...)*

!     INPUT ARGUMENTS.

!     KIDIA   : start of horizontal loop
!     KFDIA   : start of horizontal loop
!     KLON    : horizontal dimension
!     KLEV    : end of vertical loop and vertical dimension
!     PSTOPHx : random number for defining stochastic perturbation
!                  for U, V, T, and Q diabatic tendency.
!     PTENxIN : initial tendency OF U, V, T and Q 

!     OUTPUT ARGUMENTS.

!     PTENx   : tendency of U, V, T, and Q

!     EXTERNALS.  NONE
!     ---------  

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!     98-01-01 R Buizza-L Isaksen-M Miller

!     MODIFICATIONS.
!     --------------
!     98-06-01 R Buizza-C Jakob: supersaturation check
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Leutbecher  11-Feb-2009: spectral stochastic diabatic tendency scheme
!        M.Leutbecher  18-Dec-2009: revised stochastic tendency scheme moved to sppten
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE STOPH_MIX ,ONLY : AMAGSTOPH
USE YOMCST   , ONLY : RETV     ,RTT      ,RLSTT    ,RLVTT
USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,&
 & RTICE    ,RTICECU  ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  ,&
 & RKOOP1   ,RKOOP2
USE YOECLDP  , ONLY : NSSOPT 

USE YOMSLPHY , ONLY : RSLWX

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSPHY 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQSL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PA(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENUIN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENVIN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENTIN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTENQIN(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTOPHU(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTOPHV(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTOPHT(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTOPHQ(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENT(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTENQ(KLON,KLEV) 
INTEGER(KIND=JPIM) :: JK, JL

REAL(KIND=JPRB) :: ZDZPQ, ZDZPT, ZDZPU, ZDZPV, ZQP1, ZQS, ZTP1
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcttre.h"
#include "fccld.h"

!-----------------------------------------------------------------------

!     1.  ADD STOCHATIC PERTURBATION TO PHYSICS TENDENCY IF REQUIRED
!         (FOR EPS, if LSTOPH=.T., with LSTOPH defined in surand1.f)
!         ----------------------------------------------------------

!     The perturbation is defined as:  dp/p=1+a*(1-2*r)
!     where:
!     p  : tendency
!     dp : perturbed tendency
!     a  : perturbation amplitude (=AMAGSTOPH, set in surand1.F)
!     r  : random number, 0<r<1 (PSTOPHx, defined in surand2.F)

!     Thus, depending on a(=AMAGSTOPH), for random numbers selected
!     between 0 and 1, the relative size of the diabatic tendency 
!     perturbation can be:   (1-a) < dp/p < (1+a)

IF (LHOOK) CALL DR_HOOK('STOCHADIATEN',0,ZHOOK_HANDLE)
DO JK=1,KLEV
  DO JL=KIDIA,KFDIA
    ZDZPU=PTENU(JL,JK)-PTENUIN(JL,JK)
    ZDZPV=PTENV(JL,JK)-PTENVIN(JL,JK)
    ZDZPT=PTENT(JL,JK)-PTENTIN(JL,JK)
    ZDZPQ=PTENQ(JL,JK)-PTENQIN(JL,JK)

    PTENU(JL,JK)=PTENUIN(JL,JK)+&
         & (1.0_JPRB+AMAGSTOPH-2.0_JPRB*AMAGSTOPH*PSTOPHU(JL))*ZDZPU  
    PTENV(JL,JK)=PTENVIN(JL,JK)+&
         & (1.0_JPRB+AMAGSTOPH-2.0_JPRB*AMAGSTOPH*PSTOPHV(JL))*ZDZPV  
    PTENT(JL,JK)=PTENTIN(JL,JK)+&
         & (1.0_JPRB+AMAGSTOPH-2.0_JPRB*AMAGSTOPH*PSTOPHT(JL))*ZDZPT  
    PTENQ(JL,JK)=PTENQIN(JL,JK)+&
         & (1.0_JPRB+AMAGSTOPH-2.0_JPRB*AMAGSTOPH*PSTOPHQ(JL))*ZDZPQ  

!     Supersaturation check

    ZTP1=PTSL(JL,JK)+RSLWX*PTENT(JL,JK)*PTSPHY
    ZQP1=PQSL(JL,JK)+RSLWX*PTENQ(JL,JK)*PTSPHY

    ZQS=FOEEWM(ZTP1)/PAP(JL,JK)
    ZQS=MIN(0.5_JPRB,ZQS)
    ZQS=ZQS/(1.0_JPRB-RETV*ZQS)

    !----------------------------
    ! supersaturation adjustments
    !----------------------------
    IF (ZTP1<RTICE.AND.NSSOPT>0) &
    & ZQS=ZQS*(PA(JL,JK)+FOKOOP(ZTP1)*(1.0_JPRB-PA(JL,JK)))
    
    IF (ZQP1 > ZQS) THEN
      PTENQ(JL,JK)=PTENQIN(JL,JK)+ZDZPQ
      PTENT(JL,JK)=PTENTIN(JL,JK)+ZDZPT
    ENDIF

  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('STOCHADIATEN',1,ZHOOK_HANDLE)
END SUBROUTINE STOCHADIATEN
