SUBROUTINE SUCLDP

!**** *SUCLDP*   - INITIALIZE COMMON YOECLD CONTROLLING *CLOUDSC*

!     PURPOSE.
!     --------
!           INITIALIZE YOECLDP

!**   INTERFACE.
!     ----------
!        CALL *SUCLDP* FROM *SUPHEC*
!              ------        ------

!        EXPLICIT ARGUMENTS :
!        --------------------
!        NONE

!        IMPLICIT ARGUMENTS :
!        --------------------
!        COMMON YOECLDP

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE
!     "INTEGRATED FORECASTING SYSTEM"

!     AUTHOR.
!     -------
!        C.JAKOB   *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-02-07
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Ko"hler &   03-Dec-2004 total water variance setup for
!        A.Tompkins                moist advection-diffusion PBL and
!                                  7200s autoconversion timescale (RKCONV)
!                                  instead of 10000s
!        R.Forbes      28-May-2008 Changed factor in RTAUMEL from 1.5 to 0.66 
!        A.Tompkins/JJM 20080729   cloud-aerosol interactions
!        P.Bechtold    19-Jan-2009 Changed RCLDIFF from 3.E-6 to 5.E-6
!        A.Tompkins/RF  Jul 2009   Many new constants for multi-phase microphysics
!                                  Addition of namcldp namelist
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY :  JPIM     ,JPRB
USE YOMHOOK   ,ONLY :  LHOOK,   DR_HOOK
USE YOMCST   , ONLY :  RG       ,RTT        
USE YOECLDP  , ONLY :  RAMID    ,RCLDIFF  ,RCLCRIT  ,RKCONV, RCLDIFFC,&
 & RPRC1    ,RPRC2    ,RCLDMAX  ,RPECONS  ,RTAUMEL  ,&
 & RAMIN    ,RLMIN    ,&
 & RKOOPTAU ,NSSOPT   ,&
 & NCLDDIAG ,LCLDBUDGET ,LCLDEXTRA,&
 & RLCRITSNOW         ,RSNOWLIN1,RSNOWLIN2,&
 & RVICE    ,RVRAIN   ,RVSNOW,&
 & RICEHI1  ,RICEHI2  ,RICEINIT, RPRECRHMAX,&
 & RBETAP1  ,NSHAPEP  ,NSHAPEQ  ,NBETA    ,RBETA    ,&
 & RTHOMO   ,RCOVPMIN, &   
 & RNICE    ,RCCN, &
 & NAERCLD  ,&
 & LAERLIQAUTOLSP, LAERLIQAUTOCP, LAERLIQAUTOCPB, &
 & LAERLIQCOLL, LAERICESED, LAERICEAUTO, &
 & RCDNCOM, RCDNCSS, RCDNCSU, NCLOUDACT, LACI_DIAG, NACTPDF, &
 & RCLDTOPP ,NCLDTOP, RLCRIT_UPHYS
USE YOMLUN   , ONLY : NULNAM    ,NULOUT  
USE YOMDIM   , ONLY : NFLEVG 
USE YOMSTA   , ONLY : STPRE
USE YOE_AER_ACTIV, ONLY: SETUP_ACI_DIAG

IMPLICIT NONE

REAL(KIND=JPRB) :: ZX
INTEGER(KIND=JPIM) :: IX, JLEV
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "posnam.intfb.h"
#include "namcldp.h"
#include "abor1.intfb.h"

!------------------------
!*       1.    SET VALUES
!------------------------

IF (LHOOK) CALL DR_HOOK('SUCLDP',0,ZHOOK_HANDLE)

!----------------
! Misc constants 
!----------------
RAMID=0.8_JPRB
RPRC1=100._JPRB
RPRC2=0.5_JPRB

!------------------
! safety thresholds
!------------------
RCLDMAX=5.E-3_JPRB
RAMIN=1.E-8_JPRB
RLMIN=1.E-8_JPRB

!---------------------------------------------------------
! Mixing coefficient for turbulence
! Note: this is modified in cloudsc for convective points
!---------------------------------------------------------
!RCLDIFF=2.E-6_JPRB  !1e-6 pre 25r1 default 
RCLDIFF=3.E-6_JPRB
RCLDIFFC=5._JPRB     ! erosion for convectove points to be tuned   

!--------------------------------
! Rain auto conversion constants
! Note: this is modified in cloudsc and cuascn for land/sea
!--------------------------------
!RCLCRIT=0.3E-3_JPRB         ! was 0.3E-3 , critical autoconversion threshold
!RKCONV=1._JPRB/7200._JPRB ! 1/autoconversion time scale
RCLCRIT=0.4E-3_JPRB         ! critical autoconversion threshold
RKCONV=1._JPRB/6000._JPRB   ! 1/autoconversion time scale (s)

!---value for use when using cloud microphysics (NAERCLD/bit0 = 1 or NAERCLD/bit3=1)
RLCRIT_UPHYS = 9.3E-6_JPRB   ! value is from Rotstayn and Penner (J. Clim 2001)

!--------------------------------
! Snow auto conversion constants
!--------------------------------
! Changing RSNOWLIN1, RSNOWLIN2 makes a bigger difference at high levels 
! where IWC values are low,
RSNOWLIN1=1._JPRB/1000._JPRB ! 1000s = Lin et al. 83
RSNOWLIN2=0.025_JPRB  ! 0.025 = Lin et al. 83
RLCRITSNOW=5.0E-5_JPRB ! critical autoconversion threshold

!-----------------------------------
! Base hydrometeor fall speeds m/s
!-----------------------------------
RVICE=0.15_JPRB
RVRAIN=4.0_JPRB
RVSNOW=1.0_JPRB

!---------------------------------------------------
! Used in ice fall-speed modification by air density
! Heymsfield and Iaquinta JAS 2000
!--------------------------------------------------- 
RICEHI1=1.0_JPRB/30000._JPRB
RICEHI2=1.0_JPRB/233._JPRB

!---------------------------------------------------
! Default ice number and liq ccn 
!---------------------------------------------------
RNICE=0.027_JPRB 
RCCN=125.0_JPRB

!------------------------------
! initial mass of ice particle
!------------------------------
RICEINIT=1.E-12_JPRB

!---------------------------
! ice microphysics constants
!---------------------------
RTHOMO=RTT-38.0_JPRB ! threshold for homogeneous freezing

!--------------------------
! precipitation evaporation
!--------------------------
RPRECRHMAX=0.7_JPRB  ! Max threshold RH for evaporation for
                     ! a precip coverage of zero 
                     ! RMFsens20081111  
RPECONS=5.44E-4_JPRB/RG !evaporation rate coefficient
RCOVPMIN=0.1_JPRB  

!----------------------
! timescale for melting
!----------------------
! RTAUMEL=5._JPRB*3600._JPRB in CY31R1
! RTAUMEL=0.005*3.6E3_JPRB in EC-Earth 3.1, equivalent to 13.5h
RTAUMEL=2.0_JPRB*3.6E3_JPRB

!----------------------------------------------------
! Timescale for supersaturation to cause overcast sky
!----------------------------------------------------
RKOOPTAU=10800._JPRB

!--------------------------------
! Tompkins supersaturation scheme
!--------------------------------
NSSOPT=1 

!----------------------------------------------------------
! Control Cloudsc diagnostics
! Add up these options to get the integer for NCLDDIAG
! 0=no diagnostics
! 1=place microphysics pathways into PEXTRA extra fields
! 2=Perform diagnostics for total water and enthalpy
!----------------------------------------------------------
NCLDDIAG=0

!-------------------------------
! Aerosol-cloud indirect effects
!-------------------------------
NAERCLD=0 ! 0 = no aerosol interactions
LAERLIQAUTOLSP = .FALSE.
LAERLIQAUTOCP  = .FALSE.
LAERLIQAUTOCPB = .FALSE.
LAERLIQCOLL    = .FALSE.
LAERICESED     = .FALSE.
LAERICEAUTO    = .FALSE.
!>>dod
NCLOUDACT = 0
LACI_DIAG = .FALSE.
NACTPDF = 1
!<<dod
!-- relationship aerosol mass - CDNC numbers (Menon et al., 2002, JAS 59, 695)
!>>dod renamed
RCDNCOM = 0.13_JPRB
RCDNCSS = 0.05_JPRB
RCDNCSU = 0.50_JPRB 
!<<dod

!---------------------
! variance definitions
!---------------------
NSHAPEP=1.0_JPRB+SQRT(2.0_JPRB)
NSHAPEQ=1.0_JPRB+SQRT(2.0_JPRB)
NBETA=100 ! must equal array size of RBETA
DO IX=0,NBETA
  ZX=REAL(IX)/NBETA
  ZX=MAX(MIN(ZX,1.0_JPRB),0.0_JPRB)
  RBETA(IX)  =BETAI(NSHAPEP,NSHAPEQ,ZX)
  RBETAP1(IX)=BETAI(NSHAPEP+1,NSHAPEQ,ZX)
ENDDO

! Calculate model level above which cloud scheme is not called
! using a "standard atmosphere" profile
NCLDTOP=2
DO JLEV=NFLEVG,2,-1
  IF (STPRE(JLEV) > RCLDTOPP) NCLDTOP=JLEV
ENDDO

!------------------------------------------------------------
! override the options for the cloud scheme from the namelist
!------------------------------------------------------------
CALL POSNAM(NULNAM,'NAMCLDP')
READ (NULNAM,NAMCLDP)

!--------------------------------------------
! diagnostic switches based on NCLDDIAG
! 1=Extra fields defined 
! 2=Perforce Enthalpy and total water budgets
!--------------------------------------------
LCLDEXTRA  =BTEST(NCLDDIAG,0)
LCLDBUDGET =BTEST(NCLDDIAG,1)

!------------------------
! output the final values
!------------------------
WRITE(NULOUT,'("SUCLDP: RCLDIFF =",E12.4)')    RCLDIFF
WRITE(NULOUT,'("SUCLDP: RCLDIFFC =",E12.4)')    RCLDIFFC
WRITE(NULOUT,'("SUCLDP: RCLCRIT =",E12.4)')    RCLCRIT
WRITE(NULOUT,'("SUCLDP: RKCONV =",E12.4)')     RKCONV
WRITE(NULOUT,'("SUCLDP: RLCRITSNOW =",E12.4)') RLCRITSNOW
WRITE(NULOUT,'("SUCLDP: RSNOWLIN1 =",E12.4)')  RSNOWLIN1
WRITE(NULOUT,'("SUCLDP: RSNOWLIN2 =",E12.4)')  RSNOWLIN2
WRITE(NULOUT,'("SUCLDP: RVICE =",E12.4)')      RVICE
WRITE(NULOUT,'("SUCLDP: RVRAIN =",E12.4)')     RVRAIN
WRITE(NULOUT,'("SUCLDP: RVSNOW =",E12.4)')     RVSNOW
WRITE(NULOUT,'("SUCLDP: RICEHI1 =",E12.4)')    RICEHI1
WRITE(NULOUT,'("SUCLDP: RICEHI2 =",E12.4)')    RICEHI2
WRITE(NULOUT,'("SUCLDP: RICEINIT =",E12.4)')   RICEINIT
WRITE(NULOUT,'("SUCLDP: RTHOMO =",E12.4)')     RTHOMO
WRITE(NULOUT,'("SUCLDP: RCOVPMIN =",E12.4)')   RCOVPMIN
WRITE(NULOUT,'("SUCLDP: RPECONS =",E12.4)')    RPECONS
WRITE(NULOUT,'("SUCLDP: RTAUMEL =",E12.4)')    RTAUMEL
WRITE(NULOUT,'("SUCLDP: RKOOPTAU =",E12.4)')   RKOOPTAU
WRITE(NULOUT,'("SUCLDP: RPRECRHMAX =",E12.4)') RPRECRHMAX
WRITE(NULOUT,'("SUCLDP: NSSOPT =",I6)')   NSSOPT
WRITE(NULOUT,'("SUCLDP: NCLDDIAG =",I6)') NCLDDIAG
IF (LCLDEXTRA) WRITE(NULOUT,'("SUCLDP: LCLDEXTRA IS TRUE")') 
IF (LCLDBUDGET) WRITE(NULOUT,'("SUCLDP: LCLDBUDGET IS TRUE")') 

LAERLIQAUTOLSP = BTEST(NAERCLD,0)
LAERLIQAUTOCP  = BTEST(NAERCLD,1)
LAERLIQAUTOCPB = BTEST(NAERCLD,2)
LAERLIQCOLL    = BTEST(NAERCLD,3)
LAERICESED     = BTEST(NAERCLD,4)
LAERICEAUTO    = BTEST(NAERCLD,5)

! check: Can't not have both LAERLIQAUTOCP and LAERLIQAUTOCPB
IF (LAERLIQAUTOCPB .AND. LAERLIQAUTOCP) THEN
  WRITE(NULOUT,'("Resetting LAERLIQAUTOCP, choose one convective option only!")')
  LAERLIQAUTOCP=.FALSE.
ENDIF

IF (NCLOUDACT < 0 .OR. NCLOUDACT > 3) THEN
  CALL ABOR1('SUCLDP: Unsupported NCLOUDACT value.')
END IF
IF (NACTPDF < 1) THEN
  NACTPDF = 1
  WRITE(NULOUT,'("SUCLDP: Invalid NACTPDF value in NAMCLDP, reset to 1")')
END IF

IF (LACI_DIAG) THEN
   CALL SETUP_ACI_DIAG
END IF

WRITE(NULOUT,'("SUCLDP: NAERCLD=",I2)')NAERCLD
WRITE(NULOUT,'("SUCLDP: NCLOUDACT=",I0)') NCLOUDACT
WRITE(NULOUT,'("SUCLDP: LACI_DIAG=",L1)') LACI_DIAG
WRITE(NULOUT,'("SUCLDP: NACTPDF=",I0)') NACTPDF
WRITE(NULOUT,'("SUCLDP: RLCRIT_UPHYS=",E12.4)') RLCRIT_UPHYS

IF (LAERLIQAUTOLSP) WRITE(NULOUT,'("SUCLDP: LAERLIQAUTOLSP ON")')
IF (LAERLIQAUTOCP)  WRITE(NULOUT,'("SUCLDP: LAERLIQAUTOCP  ON")') 
IF (LAERLIQAUTOCPB) WRITE(NULOUT,'("SUCLDP: LAERLIQAUTOCPB  ON")') 
IF (LAERLIQCOLL)    WRITE(NULOUT,'("SUCLDP: LAERLIQCOLL    ON")') 
IF (LAERICESED)     WRITE(NULOUT,'("SUCLDP: LAERICESED     ON")') 
IF (LAERICEAUTO)    WRITE(NULOUT,'("SUCLDP: LAERICEAUTO    ON")') 
WRITE(NULOUT,'("SUCLDP: RCDNCOM=",F8.3,2X,"RCDNCSS=",F8.3,2X,"RCDNCSU=",F8.3)') RCDNCOM,RCDNCSS,RCDNCSU

!     -----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUCLDP',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------
CONTAINS
FUNCTION BETAI(PP,PQ,PX)

  ! Description:
  !
  ! USES betacf,gammln Returns the incomplete beta function I x (a; b).  
  !
  ! Method:
  !   See Numerical Recipes (Fortran)
  !
  ! Author:
  !   A. Tompkins 2000
  !
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
IMPLICIT NONE

  ! input parameters:
REAL(KIND=JPRB)   ,INTENT(IN)    :: PP ! beta shape parameters
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ ! beta shape parameters
REAL(KIND=JPRB)   ,INTENT(IN)    :: PX ! integration limit  

  !  local scalars: 
REAL(KIND=JPRB) :: Z_BT, BETAI 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUCLDP:BETAI',0,ZHOOK_HANDLE)
IF(PX>0.0_JPRB .AND. PX<1.0_JPRB )THEN !FACTORS IN FRONT OF THE CONTINUED FRACTION. 
  Z_BT=EXP(GAMMLN(PP+PQ)-GAMMLN(PP)-GAMMLN(PQ)+PP*LOG(PX)+PQ*LOG(1.0_JPRB-PX)) 
ELSE  
  Z_BT=0.0_JPRB 
ENDIF 
IF(PX<(PP+1.0_JPRB)/(PP+PQ+2.0_JPRB))THEN ! USE CONTINUED FRACTION DIRECTLY.
  BETAI=Z_BT*BETACF(PP,PQ,PX)/PP 
  IF (LHOOK) CALL DR_HOOK('SUCLDP:BETAI',1,ZHOOK_HANDLE)
  RETURN 
ELSE 
  BETAI=1.0_JPRB-Z_BT*BETACF(PQ,PP,1.0_JPRB-PX)/PQ !
!   use continued fraction after making the symmetry transformation. 
  IF (LHOOK) CALL DR_HOOK('SUCLDP:BETAI',1,ZHOOK_HANDLE)
  RETURN 
ENDIF 
IF (LHOOK) CALL DR_HOOK('SUCLDP:BETAI',1,ZHOOK_HANDLE)
END FUNCTION BETAI

!-----------------------------------------------------------------------

FUNCTION BETACF(PP,PQ,PX) 
  ! Description:
  !
  !  used by betai: evaluates continued fraction for incomplete 
  !  beta function by modified lentz's method ( x 5.2). 
  !  first step of lentz's method. 
  !
  ! Method:
  !   See Numerical Recipes (Fortran)
  !
  ! Author:
  !   A. Tompkins 2000
  !
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
IMPLICIT NONE

  ! input parameters:
REAL(KIND=JPRB)   ,INTENT(IN)    :: PP  ! beta shape parameters
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ  ! beta shape parameters
REAL(KIND=JPRB)   ,INTENT(IN)    :: PX  ! integration limit  

INTEGER(KIND=JPIM) :: I_MAXIT, I_M,I_M2 
REAL(KIND=JPRB) :: ZEPS,Z_FPMIN,Z_AA,Z_C,Z_D,Z_DEL,Z_H,Z_QAB,Z_QAM,Z_QAP, BETACF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUCLDP:BETACF',0,ZHOOK_HANDLE)
I_MAXIT=100
ZEPS = 3.E-7_JPRB
Z_FPMIN=1.E-30_JPRB

Z_QAB=PP+PQ 

!  these q's will be used in factors that occur in the coe cients (6.4.6). 

Z_QAP=PP+1.0_JPRB
Z_QAM=PP-1.0_JPRB 
Z_C=1.0_JPRB 
Z_D=1.0_JPRB-Z_QAB*PX/Z_QAP 
IF(ABS(Z_D)<Z_FPMIN)Z_D=Z_FPMIN 
Z_D=1.0_JPRB/Z_D 
Z_H=Z_D 
I_M=1.0_JPRB
Z_DEL = 2.0_JPRB 
DO WHILE (ABS(Z_DEL-1.0_JPRB)>ZEPS)
  I_M2=2*I_M 
  Z_AA=I_M*(PQ-I_M)*PX/((Z_QAM+I_M2)*(PP+I_M2))
  Z_D=1.0_JPRB+Z_AA*Z_D  ! one step (the even one) of the recurrence. 
  IF (ABS(Z_D)<Z_FPMIN)Z_D=Z_FPMIN 
  Z_C=1.0_JPRB+Z_AA/Z_C 
  IF (ABS(Z_C)<Z_FPMIN)Z_C=Z_FPMIN 
  Z_D=1.0_JPRB/Z_D 
  Z_H=Z_H*Z_D*Z_C 
  Z_AA=-(PP+I_M)*(Z_QAB+I_M)*PX/((PP+I_M2)*(Z_QAP+I_M2)) 
  Z_D=1.0_JPRB+Z_AA*Z_D ! next step of the recurrence (the odd one). 
  IF (ABS(Z_D)<Z_FPMIN)Z_D=Z_FPMIN 
  Z_C=1.0_JPRB+Z_AA/Z_C 
  IF(ABS(Z_C)<Z_FPMIN)Z_C=Z_FPMIN 
  Z_D=1.0_JPRB/Z_D 
  Z_DEL=Z_D*Z_C 
  Z_H=Z_H*Z_DEL 
  I_M=I_M+1.0_JPRB !*AMT*
  IF(I_M>I_MAXIT) THEN 
    Z_DEL=1.0_JPRB
  ENDIF  
ENDDO 
BETACF=Z_H 
IF (LHOOK) CALL DR_HOOK('SUCLDP:BETACF',1,ZHOOK_HANDLE)
END FUNCTION BETACF

!-----------------------------------------------------------------------

FUNCTION GAMMLN(P_XX) 

  ! Description:
  !
  ! Gamma function calculation
  ! returns the value ln[g(xx)] for xx > _ZERO_
  !
  ! Method:
  !   See Numerical Recipes
  !
  ! Author:
  !   A. Tompkins 2000
  !
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
IMPLICIT NONE

REAL(KIND=JPRB)   ,INTENT(IN)    :: P_XX

REAL(KIND=JPRB) :: GAMMLN
INTEGER(KIND=JPIM) :: J 
REAL(KIND=JPRB) :: Z_SER,Z_STP,Z_TMP,Z_X,Z_Y,Z_COF(6) 
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('SUCLDP:GAMMLN',0,ZHOOK_HANDLE)
Z_COF(:)= (/&
 & 76.18009172947146_JPRB, &
 & -86.50532032941677_JPRB, &
 & 24.01409824083091_JPRB, &
 & -1.231739572450155_JPRB, &
 & 0.1208650973866179E-2_JPRB, &
 & -.5395239384953E-5_JPRB/)  

Z_STP=2.5066282746310005_JPRB

Z_X=P_XX 
Z_Y=Z_X 
Z_TMP=Z_X+5.5_JPRB
Z_TMP=(Z_X+0.5_JPRB)*LOG(Z_TMP)-Z_TMP 
Z_SER=1.000000000190015_JPRB
DO J=1,6
  Z_Y=Z_Y+1.0_JPRB 
  Z_SER=Z_SER+Z_COF(J)/Z_Y 
ENDDO 
GAMMLN=Z_TMP+LOG(Z_STP*Z_SER/Z_X) 
IF (LHOOK) CALL DR_HOOK('SUCLDP:GAMMLN',1,ZHOOK_HANDLE)
END FUNCTION GAMMLN
END SUBROUTINE SUCLDP
