SUBROUTINE VDFDIFM(KIDIA,KFDIA,KLON,KLEV,KTOP,&
 & PTMST,PUM1,PVM1,PAPHM1,PCFM,PTODC,&
 & PSOTEU,PSOTEV,PSOC,&
 & PVOM,PVOL,PUCURR,PVCURR,PUDIF,PVDIF)  
!     ------------------------------------------------------------------

!**   *VDFDIFM* - DOES THE IMLPLICIT CALCULATION FOR MOMENTUM DIFFUSION

!     DERIVED FROM VDIFF (CY34) BY
!     A.C.M. BELJAARS       E.C.M.W.F.    10-11-89
!     MODIFIED A BELJAARS   E.C.M.W.F.    17-11-02 Turb. Orogr. Drag  
!     MODIFIED A BELJAARS   E.C.M.W.F.    30-09-05 Include Subgr. Oro. in solver  
!     modified A.Beljaars 12/11/02   Ocean current b.c.
!     modified P.de Rosnay/G.Balsamo      07-03-09 Offline Jacobians EKF 
!     modified P.de Rosnay/G.Balsamo      October 09 Offline Jacobians EKF 

!     PURPOSE
!     -------

!     SOLVE TRIDIAGONAL MATRICES FOR MOMENTUM DIFFUSION

!     INTERFACE
!     ---------

!     *VDFDIFM* IS CALLED BY *VDFMAIN*

!     INPUT PARAMETERS (INTEGER):

!     *KIDIA*        START POINT
!     *KFDIA*        END POINT
!     *KLEV*         NUMBER OF LEVELS
!     *KLON*         NUMBER OF GRID POINTS PER PACKET
!     *KTOP*         INDEX FOR BOUNDARY LAYER TOP

!     INPUT PARAMETERS (REAL):

!     *PTMST*        DOUBLE TIME STEP (SINGLE AT 1TH STEP)
!     *PUM1*         X-VELOCITY COMPONENT AT T-1
!     *PVM1*         Y-VELOCITY COMPONENT AT T-1
!     *PAPHM1*       PRESSURE AT T-1
!     *PCFM*         PROP.TO EXCH.COEFF. FOR MOMENTUM (C,K-STAR IN DOC.)
!     *PTODC*        TURB. OROGR. DRAG (IMPLICIT) COEFFICIENTS 
!     *PSOTEU*       Explicit part of U-tendency from subgrid orography scheme    
!     *PSOTEV*       Explicit part of V-tendency from subgrid orography scheme     
!     *PSOC*         Implicit part of subgrid orography (df/dt=PSOTE-PSOC*f/alpha)
!     *PVOM*         U-TENDENCY
!     *PVOL*         V-TENDENCY
!     *PUCURR*       OCEAN CURRENT X-COMPONENT
!     *PVCURR*       OCEAN CURRENT Y-COMPONENT

!     OUTPUT PARAMETERS (REAL):

!     *PUDIF*        U-DOUBLE-TILDE DEVIDED BY ALFA
!     *PVDIF*        V-DOUBLE-TILDE DEVIDED BY ALFA

!     METHOD
!     ------

!     *LU*-DECOMPOSITION AND BACK SUBSTITUTION IN ONE DOWNWARD SCAN
!     AND ONE UPWARD SCAN.

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEVDF   , ONLY : RVDIFTS
USE YOMSEKF  , ONLY : N_SEKF_PT, LUSEKF_REF, LUSE_JATM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTOP 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTMST 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHM1(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCFM(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTODC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOTEU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOTEV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOC(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOM(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOL(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUCURR(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVCURR(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUDIF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVDIF(KLON,KLEV) 
!*    LOCAL STORAGE
!     ----- -------

REAL(KIND=JPRB) ::  ZTCOE(KLON), ZEBSM(KLON,KLEV)

INTEGER(KIND=JPIM) :: ILEVM1, ITOPP1, JK, JL

REAL(KIND=JPRB) :: ZDISC, ZDISCI, ZFAC, ZQDP, ZTPFAC2 ,ZHUCURR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('VDFDIFM',0,ZHOOK_HANDLE)
ZTPFAC2=1.0_JPRB/RVDIFTS
ILEVM1=KLEV-1
ITOPP1=KTOP+1

!*         1.1     SETTING OF RIGHT HAND SIDES.

DO JK=KTOP,KLEV
  DO JL=KIDIA,KFDIA
    PUDIF(JL,JK)=ZTPFAC2*PUM1(JL,JK)+PTMST*(PVOM(JL,JK)+PSOTEU(JL,JK))
    PVDIF(JL,JK)=ZTPFAC2*PVM1(JL,JK)+PTMST*(PVOL(JL,JK)+PSOTEV(JL,JK))
  ENDDO
ENDDO

!IF (.NOT.LUSEKF_REF.OR.LUSE_JATM.OR.N_SEKF_PT==0) THEN
! Coupled surface 
!*         1.2     TOP LAYER ELIMINATION.

DO JL=KIDIA,KFDIA
  ZTCOE(JL)=PCFM(JL,KTOP)
  ZQDP=1.0_JPRB/(PAPHM1(JL,ITOPP1)-PAPHM1(JL,KTOP))
  ZDISCI=1.0_JPRB+PCFM(JL,KTOP)*ZQDP+PTODC(JL,KTOP)+PSOC(JL,KTOP)
  ZDISC=1.0_JPRB/ZDISCI
  ZEBSM(JL,KTOP)=ZDISC*(PCFM(JL,KTOP)*ZQDP)
  PUDIF(JL,KTOP)=ZDISC*PUDIF(JL,KTOP)
  PVDIF(JL,KTOP)=ZDISC*PVDIF(JL,KTOP)
ENDDO

!*         1.3     ELIMINATION FOR MIDDLE LAYERS.

DO JK=ITOPP1,ILEVM1
  DO JL=KIDIA,KFDIA
    ZQDP=1.0_JPRB/(PAPHM1(JL,JK+1)-PAPHM1(JL,JK))
    ZFAC=ZTCOE(JL)*ZQDP
    ZTCOE(JL)=PCFM(JL,JK)
    ZDISCI=1.0_JPRB+ZFAC*(1.0_JPRB-ZEBSM(JL,JK-1)) &
     &    +PCFM(JL,JK)*ZQDP+PTODC(JL,JK)+PSOC(JL,JK)
    ZDISC=1.0_JPRB/ZDISCI
    ZEBSM(JL,JK)=ZDISC*(PCFM(JL,JK)*ZQDP)
    PUDIF(JL,JK)=ZDISC*(PUDIF(JL,JK)+ZFAC*PUDIF(JL,JK-1))
    PVDIF(JL,JK)=ZDISC*(PVDIF(JL,JK)+ZFAC*PVDIF(JL,JK-1))
  ENDDO
ENDDO

!*         1.4     BOTTOM LAYER ELIMINATION.

DO JL=KIDIA,KFDIA
  ZQDP=1.0_JPRB/(PAPHM1(JL,KLEV+1)-PAPHM1(JL,KLEV))
  ZFAC=ZTCOE(JL)*ZQDP
  ZDISCI=1.0_JPRB+ZFAC*(1.0_JPRB-ZEBSM(JL,ILEVM1)) &
   &    +PCFM(JL,KLEV)*ZQDP+PTODC(JL,KLEV)+PSOC(JL,KLEV)
  ZDISC=1.0_JPRB/ZDISCI
  ZHUCURR=PCFM(JL,KLEV)*ZQDP*ZTPFAC2
  PUDIF(JL,KLEV)=ZDISC*(PUDIF(JL,KLEV)+ZHUCURR*PUCURR(JL)+ZFAC*PUDIF(JL,ILEVM1))
  PVDIF(JL,KLEV)=ZDISC*(PVDIF(JL,KLEV)+ZHUCURR*PVCURR(JL)+ZFAC*PVDIF(JL,ILEVM1))
ENDDO

!*         1.5     BACK-SUBSTITUTION.

DO JK=ILEVM1,KTOP,-1
  DO JL=KIDIA,KFDIA
    PUDIF(JL,JK)=PUDIF(JL,JK)+ZEBSM(JL,JK)*PUDIF(JL,JK+1)
    PVDIF(JL,JK)=PVDIF(JL,JK)+ZEBSM(JL,JK)*PVDIF(JL,JK+1)
  ENDDO
ENDDO
!ENDIF !Offline Jacobians

IF (LHOOK) CALL DR_HOOK('VDFDIFM',1,ZHOOK_HANDLE)
END SUBROUTINE VDFDIFM
