!OPTIONS XOPT(HSFUN)
SUBROUTINE VDFEXCUS (KIDIA , KFDIA, KLON, KLEV, PTMST,&
 & PZ0MM  , PUM1   , PVM1   , PTM1  , PQM1 , &
 & PAPHM1 , PGEOM1 , PGEOH , PCPTGZ, &
 & PKMFL  , PKHFL  , PKQFL , PZINV , PCFM , PCFH)
!     ------------------------------------------------------------------

!**   *VDFEXCUS* - DETERMINES THE EXCHANGE COEFFICIENTS BETWEEN THE
!                   UPPER MODEL LEVELS WITH STABILITY AS A FUNCTION OF
!                   RICHARDSON NUMBER
!                   (Nonlinear version for trajectory in adjoint)

!     J.F. MAHFOUF    02/10/95     E.C.M.W.F. 

!     Modified by
!     -----------

!     P. LOPEZ        25/02/05     nonlinear version for
!                                  trajectory in adjoint
!     M. Ko"hler      25/09/07     add mixed layer
!                                  reduce K towards MO higher up 
!     M. Janiskova    19/12/07     modified wind shear for Ri computation

!     PURPOSE
!     -------

!     DETERMINE EXCHANGE COEFFICIENTS BETWEEN THE UPPER MODEL LEVELS

!     INTERFACE
!     ---------

!     *VDFEXCUS* IS CALLED BY *VDFMAINS*

!     INPUT PARAMETERS (INTEGER):

!     *KIDIA*        START POINT
!     *KFDIA*        END POINT
!     *KLEV*         NUMBER OF LEVELS
!     *KLON*         NUMBER OF GRID POINTS PER PACKET

!     INPUT PARAMETERS (REAL):

!     *PTMST*        DOUBLE TIME STEP (SINGLE AT 1TH STEP)
!     *PUM1*         X-VELOCITY COMPONENT AT T-1  (Trajectory)
!     *PVM1*         Y-VELOCITY COMPONENT AT T-1  (Trajectory)
!     *PTM1*         TEMPERATURE AT T-1           (Trajectory)
!     *PQM1*         SPECIFIC HUMUDITY AT T-1     (Trajectory)
!     *PAPHM1*       PRESSURE AT T-1              (Trajectory)
!     *PGEOM1*       GEOPOTENTIAL AT T-1          (Trajectory)
!     *PGEOH*        GEOPOTENTIAL AT HALF LEVELS  (Trajectory)
!     *PCPTGZ*       DRY STATIC ENERGY            (Trajectory)
!     *PKMFL*        KINEMATIC MOMENTUM FLUX   
!     *PKHFL*        KINEMATIC HEAT FLUX       
!     *PKQFL*        KINEMATIC MOISTURE FLUX  
!     *PZINV*        INVERSION HEIGHT         

!     OUTPUT PARAMETERS (REAL):

!     *PCFM*         PROP. TO EXCH. COEFF. FOR MOMENTUM     (Trajectory)
!     *PCFH*         PROP. TO EXCH. COEFF. FOR HEAT         (Trajectory)
!                    (ONLY PCFM(*,1:KLEV-1) AND
!                          PCFH(*,1:KLEV-1) ARE COMPUTED)

!     METHOD
!     ------

!     SEE DOCUMENTATION

!     THE EXCHANGE COEFFICIENTS ARE EXPRESSED ACCORDING TO 
!     LOUIS et al. (1982) PROPOSAL NEAR THE SURFACE.  HIGHER
!     UP MONIN-OBUKOV VALUES ARE USED WITH A LENGTH SCALE OF 
!     300 M (150M).
!     POSSIBILITY TO USE LOUIS et al. (1982) PROPOSAL IN STABLE
!     SITUATIONS STILL KEPT.

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG       ,RD       ,RCPD     ,RETV
USE YOEVDF   , ONLY : RLAM     ,RKAP     ,RVDIFTS  ,REPDU2
USE YOEVDFS  , ONLY : JPRITBL  ,RITBL    ,ARITBL   ,RCHBA    ,&
                    & RCHBB    ,RCHBD    ,RCHB23A  ,RCHBBCD  ,RCHBCD   ,&
                    & RCHETA   ,RCHETB   ,RCDHALF  ,RCDHPI2  ,RIMAX    ,&
                    & DRITBL   ,DRI26  
USE YOEPHLI  , ONLY : RLPCC    ,RLPMIXL  ,RLPBETA

IMPLICIT NONE


!*         0.1    GLOBAL VARIABLES

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTMST 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZ0MM(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHM1(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOM1(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEOH(KLON,0:KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPTGZ(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKMFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKHFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PKQFL(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZINV(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCFM(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCFH(KLON,KLEV) 

!*         0.2    LOCAL VARIABLES

INTEGER(KIND=JPIM) :: JK, JL, IRIB

REAL(KIND=JPRB) :: ZRI(KLON),ZDU1(KLON),ZDU2(KLON),ZMGEOM(KLON),ZUST(KLON), &
                    & ZDTV(KLON),ZKHVFL(KLON),ZL(KLON), ZLI(KLON),&
                    & ZPHIM(KLON), ZPHIH(KLON)
REAL(KIND=JPRB) :: ZDTVR(KLON)

REAL(KIND=JPRB) ::    ZENTRSFC, ZWTVENTR, ZCFHNEW, Z2GEOMF,&
                    & ZA, ZB, ZCB, ZCD, &
                    & ZCFNC1, ZDRORO, ZSCF, ZEPS, &
                    & ZZ, ZDH, ZDL, ZETA, ZPHIKH, ZPHIKM, ZPHIKHI, ZPHIKMI,&
                    & ZDHZZ, ZKH, ZREPUST, ZCONS13, ZRG, &
                    & ZX2, ZBM, ZBH, ZCM, ZCH, ZIFMOM, ZIFMOH, &
                    & ZDUDZ, ZDUDZ1, ZDUDZ2, &
                    & ZZH, ZIFLTGM, ZIFLTGH, ZLENMO

REAL(KIND=JPRB) ::    Z1S, Z2S, Z3S, Z4S, Z5S, Z6S, ZZI, ZZI2, ZCC, &
                    & ZCONS2, ZCONS3, ZCONS8, ZCONS9, ZCONS10, ZCONS11, &
                    & ZCONS23, ZALH2, ZALM2, ZHLM, ZHLM2, ZLIM, ZLIM2, &
                    & ZUMBETA, ZCFNC, ZCONSHG
REAL(KIND=JPRB) :: ZSQRT1, ZSQRT2, ZSQRT3, ZSQRT4, ZSQRT5, ZSQRT6

LOGICAL :: LLMONOB

REAL(KIND=JPRB) ::    ZHOOK_HANDLE

INTERFACE
#include "surf_inq.h"
END INTERFACE

#include "fcvdfs.h"


!     ------------------------------------------------------------------

!*         1.     INITIALIZE CONSTANTS
!                 --------------------

IF (LHOOK) CALL DR_HOOK('VDFEXCUS',0,ZHOOK_HANDLE)

! logical switch for using either Monin-Obuchov (true) or Louis (false)
! for stable conditions

LLMONOB = .TRUE.
!LLMONOB = .FALSE.

! different constants

ZENTRSFC  = 0.2_JPRB       ! factor for surface based top entrainment 
ZCD       = 1.0_JPRB
ZCB       = 5.0_JPRB
ZEPS      = 1.E-10_JPRB
!or ZLENMO    = 150.0_JPRB
ZLENMO    = 300.0_JPRB

ZCONS2    = (0.5_JPRB*RKAP)/RG
ZCONS3    = 1.0_JPRB/RLAM
ZCONS23   = ZCONS2*ZCONS3

ZCC       = RLPCC
ZCONS8    = 2.0_JPRB*ZCB
ZCONS9    = 3._JPRB*ZCB
ZCONS10   = (RVDIFTS*PTMST*RG**2)/(0.5_JPRB*RD)
ZCONS11   = (3._JPRB*ZCB*ZCC)/SQRT(27._JPRB)
ZUMBETA   = 1.0_JPRB - RLPBETA
ZCONSHG   = 2.0_JPRB*RG
ZHLM      = ZCONSHG*RLPMIXL
ZHLM2     = ZHLM*ZHLM

CALL SURF_INQ(PREPUST=ZREPUST)

! optimization
ZRG       = 1.0_JPRB/RG
ZCONS13   = 1.0_JPRB/3._JPRB


!     ------------------------------------------------------------------

!*         2.     PREPARE SCALING COEFFICIENTS FOR MIXED LAYER
!                 --------------------------------------------

DO JL=KIDIA,KFDIA
  ZUST  (JL)=MAX(SQRT(PKMFL(JL)),ZREPUST)
  ZKHVFL(JL)=PKHFL(JL)+RETV*PTM1(JL,KLEV)*PKQFL(JL)
  IF (ZKHVFL(JL)  <  0.0_JPRB) THEN
    ZL (JL)  = ZUST (JL)**3*PTM1(JL,KLEV)/(RKAP*RG*(ZKHVFL(JL)-ZEPS))
    ZLI(JL) = 1.0_JPRB/ZL(JL)
  ENDIF
ENDDO


!     ------------------------------------------------------------------

!*         3.     VERTICAL LOOP
!                 -------------

!***
DO JK = KLEV-1, 1, -1
!***

  DO JL = KIDIA, KFDIA
    PCFM(JL,JK) = 0.0_JPRB
    PCFH(JL,JK) = 0.0_JPRB
  ENDDO

!           COMPUTE RI-NUMBER

  DO JL = KIDIA, KFDIA
    ZMGEOM(JL) = PGEOM1(JL,JK)-PGEOM1(JL,JK+1)
    Z1S = (PUM1(JL,JK)-PUM1(JL,JK+1))**2+(PVM1(JL,JK)-PVM1(JL,JK+1))**2
    ZDU1(JL) = MAX(REPDU2,Z1S)
    ZSQRT1 = SQRT(ZDU1(JL))
    ZDUDZ1 = ZSQRT1/ZMGEOM(JL)*RG              !shear

!     shear factor: f=27.2 * (1-p/ps) * exp(-10*(1-p/ps))
!                   f=0 at p/ps=0 and 1
!                   max(f)=1.0 at p/ps=0.9

    ZDUDZ2 = ZDUDZ1 + 0.005_JPRB  &
     & * 27.2_JPRB*(1.0_JPRB - PAPHM1(JL,JK)/PAPHM1(JL,KLEV)) &
     & * EXP(-10.0_JPRB*(1.0_JPRB - PAPHM1(JL,JK)/PAPHM1(JL,KLEV)))
    ZDU2(JL) = (ZDUDZ2*ZMGEOM(JL)*ZRG) **2

    ZDRORO = 2.0_JPRB*(PCPTGZ(JL,JK)-PCPTGZ(JL,JK+1))/&
     & (PCPTGZ(JL,JK)+PCPTGZ(JL,JK+1)-PGEOM1(JL,JK)-&
     & PGEOM1(JL,JK+1))+&
     & RETV*(PQM1(JL,JK)-PQM1(JL,JK+1))
    ZDTV(JL)= PCPTGZ(JL,JK)-PCPTGZ(JL,JK+1) &
     & + RETV * 0.5_JPRB * (PQM1(JL,JK)-PQM1(JL,JK+1))&
     & * (PCPTGZ(JL,JK)+PCPTGZ(JL,JK+1))
    ZDTVR(JL) = RCPD/ZDTV(JL)
    ZRI(JL) = ZMGEOM(JL)*ZDRORO/ZDU2(JL)
  ENDDO

  DO JL=KIDIA,KFDIA

!           COMPUTE STABILITY FUNCTIONS

    IF (ZRI(JL) > 0.0_JPRB .AND. LLMONOB) THEN

!           INTERPLOLATE ETA WITH SPLINES FOR POSITIVE RICHARDSON
!           NUMBERS

      IRIB=INT(ZRI(JL)*(1.0_JPRB/DRITBL))+1
      IF (IRIB  >=  JPRITBL) THEN
!           LINEAR EXTENSION OF LOOK-UP TABLE
        ZETA = RITBL(JPRITBL)*(ZRI(JL)*(1.0_JPRB/RIMAX))
      ELSE
        ZX2  = IRIB*DRITBL
        ZA   = (ZX2-ZRI(JL))*(1.0_JPRB/DRITBL)
        ZB   = 1.0_JPRB-ZA
        ZETA = ZA*RITBL(IRIB) + ZB*RITBL(IRIB+1)&
         & +( (ZA**3-ZA)*ARITBL(IRIB)&
         & +(  ZB**3-ZB)*ARITBL(IRIB+1) )*DRI26  
      ENDIF

!           STABLE PHI-FUNCTIONS

      ZPHIM(JL) = PHIMS(ZETA)
      ZPHIH(JL) = PHIHS(ZETA)
 
    ENDIF

  ENDDO

  DO JL = KIDIA, KFDIA

    IF (ZRI(JL) > 0.0_JPRB) THEN  ! statically stable

      IF (LLMONOB) THEN  ! Monin-Obuchov 

!           COMMON FACTORS FOR STABLE AND UNSTABLE
!           new K: K,LTG scales with l=kappa*z 
!                  K,MO  scales with l=150m above surface layer

        ZIFMOM  = 1.0_JPRB / (ZPHIM(JL)**2)              !F(MO),M
        ZIFMOH  = 1.0_JPRB / (ZPHIM(JL)*ZPHIH(JL))       !F(MO),H
        ZSQRT2 = SQRT(ZDU2(JL))
        ZDUDZ = ZSQRT2/ZMGEOM(JL)*RG                     !shear

!           DIMENSIONLESS COEFFICIENTS MULTIPLIED BY PRESSURE
!           THICKNESSES FOR MOMENTUM AND HEAT EXCHANGE.

        ZZH     = 0.5_JPRB * ZRG * (PGEOM1(JL,JK)+PGEOM1(JL,JK+1)) + PZ0MM(JL)
        ZSCF    = SQRT(1.0_JPRB+ZCD*ZRI(JL))
!or      ZIFLTGM = 1.0_JPRB/(1.0_JPRB + 2.0_JPRB *ZCB * ZRI(JL)/ZSCF) !F(LTG),M
        ZIFLTGM = ZSCF/(ZSCF+2.0_JPRB*ZCB*ZRI(JL))
        ZIFLTGH = 1.0_JPRB/(1.0_JPRB + 2.0_JPRB *ZCB * ZRI(JL)*ZSCF) !F(LTG),H
        ZSQRT3 = SQRT(ZIFLTGM)
        ZBM = RKAP*ZZH*ZSQRT3
        ZSQRT4 = SQRT(ZIFLTGH)
        ZBH = RKAP*ZZH*ZSQRT4
        ZSQRT5 = SQRT(ZIFMOM)
        ZCM = ZLENMO*ZSQRT5
        ZSQRT6 = SQRT(ZIFMOH)
        ZCH = ZLENMO*ZSQRT6

        PCFM(JL,JK) = ZDUDZ * (ZBM*ZCM/(ZBM+ZCM))**2
        PCFH(JL,JK) = ZDUDZ * (ZBH*ZCH/(ZBH+ZCH))**2

      ELSE  ! Louis
        Z2GEOMF =PGEOM1(JL,JK)+PGEOM1(JL,JK+1)+ZCONSHG*PZ0MM(JL)
        ZLIM = RLPBETA+ZUMBETA/(1.0_JPRB+(Z2GEOMF*Z2GEOMF)/ZHLM2)
        ZLIM2 = ZLIM*ZLIM
        ZALM2 = ZLIM2*((ZCONS2*Z2GEOMF)/(1.0_JPRB+ZCONS23*Z2GEOMF))**2
        ZALH2 = ZALM2
        ZSQRT2 = SQRT(ZDU2(JL))
        ZCFNC = RG*ZSQRT2/ZMGEOM(JL)

!           DIMENSIONLESS COEFFICIENTS MULTIPLIED BY PRESSURE
!           THICKNESSES FOR MOMENTUM AND HEAT EXCHANGE.

        ZSCF = SQRT(1.0_JPRB+ZCD*ZRI(JL))
        PCFM(JL,JK) = (ZCFNC*ZALM2)/(1.0_JPRB+ZCONS8*ZRI(JL)/ZSCF)
        PCFH(JL,JK) = (ZCFNC*ZALH2)/(1.0_JPRB+ZCONS9*ZRI(JL)*ZSCF)
      ENDIF

    ELSE  ! statically unstable

!           COMMON FACTORS FOR STABLE AND UNSTABLE

      Z2GEOMF =PGEOM1(JL,JK)+PGEOM1(JL,JK+1)+ZCONSHG*PZ0MM(JL)
      ZLIM = RLPBETA+ZUMBETA/(1.0_JPRB+(Z2GEOMF*Z2GEOMF)/ZHLM2)
      ZLIM2 = ZLIM*ZLIM
      ZALM2 = ZLIM2*((ZCONS2*Z2GEOMF)/(1.0_JPRB+ZCONS23*Z2GEOMF))**2
      ZALH2 = ZALM2
      ZSQRT2 = SQRT(ZDU2(JL))
      ZCFNC = RG*ZSQRT2/ZMGEOM(JL)
       
!           DIMENSIONLESS COEFFICIENTS MULTIPLIED BY PRESSURE
!           THICKNESSES FOR MOMENTUM AND HEAT EXCHANGE.

      ZZI =  ZCONSHG/Z2GEOMF
      ZZI2 =    ZZI*ZZI
      Z2S = SQRT(-ZRI(JL))
      Z3S = ZCONS11*ZALM2*ZZI2
      Z4S = ZCONS11*ZALH2*ZZI2
      Z5S = ZCFNC*ZALM2
      PCFM(JL,JK) = Z5S*(1.0_JPRB-(ZCONS8*ZRI(JL))/(1.0_JPRB+Z3S*Z2S))
      Z6S = ZCFNC*ZALH2
      PCFH(JL,JK) = Z6S*(1.0_JPRB-(ZCONS9*ZRI(JL))/(1.0_JPRB+Z4S*Z2S))

    ENDIF

!           ADD MIXED LAYER PARAMETRIZATION (SURF. AND OUTER LAYER)

    IF (PGEOH(JL,JK-1)*ZRG <= PZINV(JL) ) THEN  ! up to level below entr. level

      ZZ     = PGEOH(JL,JK)*ZRG
      ZDH    = ZZ/PZINV(JL)
      ZDL    = ZZ*ZLI(JL)
      ZPHIKH = (1.0_JPRB-39._JPRB*ZDL)**(-ZCONS13)
      ZPHIKM = (1.0_JPRB-15._JPRB*ZDL)**(-ZCONS13)

!           K,surface

      ZDHZZ = ZUST(JL) * ZZ * (1.0_JPRB-ZDH)**2
      ZPHIKHI = RKAP / ZPHIKH
      PCFH(JL,JK) = ZPHIKHI * ZDHZZ
      ZPHIKMI = RKAP / ZPHIKM
      PCFM(JL,JK) = ZPHIKMI * ZDHZZ

    ENDIF

!           ADD ENTRAINMENT
!           entrainment velocity * T,v jump due to sfc flux

    IF (PGEOH(JL,JK)*ZRG<=PZINV(JL) .AND. PZINV(JL)<PGEOH(JL,JK-1)*ZRG) THEN

      ZWTVENTR = - ZENTRSFC * ZKHVFL(JL)              !sfc flux
      ZWTVENTR = MAX(0.0_JPRB,ZWTVENTR)
      ZKH     = ZWTVENTR * ZRG * ZMGEOM(JL) * ZDTVR(JL) 

      IF (ZKH < 0.0_JPRB) THEN
        ZCFHNEW  = 0.0_JPRB
      ELSE
        ZCFHNEW  = ZKH
      ENDIF
      IF (ZCFHNEW > PCFH(JL,JK)) THEN !protection against K=0
        PCFH (JL,JK) = ZCFHNEW
      ENDIF
      IF (0.75_JPRB*ZCFHNEW > PCFM(JL,JK)) THEN
        PCFM (JL,JK) = 0.75_JPRB*ZCFHNEW
      ENDIF

    ENDIF

!           SCALE DIFFUSION COEFFICIENTS FOR USE IN VDFDIFH/M

    ZCFNC1 = (ZCONS10*PAPHM1(JL,JK))/(ZMGEOM(JL) &
     & *(PTM1(JL,JK)*(1.0_JPRB+RETV*PQM1(JL,JK)) &
     & + PTM1(JL,JK+1)*(1.0_JPRB+RETV*PQM1(JL,JK+1))))
    PCFH(JL,JK) = PCFH(JL,JK) * ZCFNC1
    PCFM(JL,JK) = PCFM(JL,JK) * ZCFNC1

  ENDDO

!***
ENDDO
!***

IF (LHOOK) CALL DR_HOOK('VDFEXCUS',1,ZHOOK_HANDLE)
END SUBROUTINE VDFEXCUS
