SUBROUTINE WVXF2GB(K_NFIELDS, K_NC, K_NR, P_FCRANGE,&
 & K_ISEC1, K_ISEC2, LDWCOUNORMS, P_FIELDS, KGPTOT, KGPTOTG, KPROMA, KDGLG)

!**** *WVXF2GB*  - Initialize the wave gridpoint fields from GRIB

!     Purpose.
!     --------
!           Initialize the gridpoint fields of the model from GRIB file.

!**   Interface.
!     ----------
!        *CALL* *WVXF2GB(NFIELDS, NWVFIELDS, NC, NR, P_FCRANGE,
!                        ISEC1, ISEC2, FIELDS)*
!        Explicit arguments : 
!        ------------------
!            NFIELDS - Number of ATM fields to pass to WAVE
!            NWVFIELDS - Number of WAVE fields to pass to ATM
!            NC - Number of longitude lines
!            NR - Number of latitude lines
!            P_FCRANGE - FORECAST RANGE IN SECONDS
!            ISEC1 - GRIB ISEC1 ... Only date elements are used.
!            ISEC2 - GRIB ISEC2 ... ATM grid characteristics.
!            LDWCOUNORMS - global norms flag
!            FIELDS - Array holding the ATM fields.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        LARS ISAKSEN *ECMWF*
!        JIM DOYLE

!     Modifications:
!     --------------
!        J. Bidlot   : 97-06-13: make use of the wave part of the work array.
!        R. El Khatib: 99-12-28: remove poles rows.
!        D.Dent      : 00-09-01: convert from MPE to MPL.
!        S. Abdalla  : 01-11-27: Generalise the ATM-WAVE interface.
!        M.Hamrud    : 02-01-10 Replaced call to MODNSEC with call to
!                                new routine PREGRBENC
!        D.Salmond   : 02-08-14  MPL_BROADCAST inserted instead of 
!                                MPL_SEND / MPL_RECV
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        D.Salmond     22-Nov-2005 Mods for coarser/finer physics
!        M.Hamrud      01-Jul-2006  Revised surface fields
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE SURFACE_FIELDS_MIX   ,ONLY : SD_WW, YSD_WW
USE YOMDIM   , ONLY : NDLON
USE YOMCT0   , ONLY : NSTEPINI, CTYPE, LOBSC1, NPROC
USE YOMGEM   , ONLY : NGPTOTG
USE YOMGRB   , ONLY : NSEC0    ,NSEC1    ,NSEC3    ,NSEC4    ,NSTREAM  ,NLEG
USE YOMTAG   , ONLY :  !Associated with a commented statement!
USE YOMCT3   , ONLY : NSTEP
USE YOMDYN   , ONLY : TSTEP
USE YOEWCOU  , ONLY : MASK_WAVE_IN ,&
 & LWVIN_MASK_NOT_SET ,LWVIN_UNINITIALISED ,&
 & MWVIN_SENDCNT ,MWVIN_RECVCNT ,MWVIN_SENDOFF ,MWVIN_RECVOFF ,&
 & MWVIN_SENDBUF ,MWVIN_RECVBUF ,MWVIN_SENDIND ,&
 & MWVIN_SENDTOT ,MWVIN_RECVTOT
USE YOMMP    , ONLY : MYPROC   ,NPRCIDS ,NGLOBALPROC ,NLOCALINDEX
USE YOMVAREPS, ONLY : NFCHO_TRUNC_INI, LVAREPS
USE MPL_MODULE , ONLY : MPL_SEND, MPL_RECV, MPL_WAIT,&
 & JP_BLOCKING_STANDARD, JP_NON_BLOCKING_STANDARD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: K_NFIELDS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDGLG
INTEGER(KIND=JPIM),INTENT(OUT)   :: K_NC 
INTEGER(KIND=JPIM),INTENT(OUT)   :: K_NR 
INTEGER(KIND=JPIM),INTENT(OUT)   :: K_ISEC1(SIZE(NSEC1))
INTEGER(KIND=JPIM),INTENT(OUT)   :: K_ISEC2(22+KDGLG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_FCRANGE
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_FIELDS(KGPTOTG,K_NFIELDS) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGPTOTG
INTEGER(KIND=JPIM),INTENT(IN)    :: KGPTOT
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA

REAL(KIND=JPRB) :: ZFLOC(KGPTOT*K_NFIELDS)
LOGICAL,INTENT(INOUT) :: LDWCOUNORMS
INTEGER(KIND=JPIM) :: ISEC0(SIZE(NSEC0))
INTEGER(KIND=JPIM) :: ISEC3(SIZE(NSEC3)),ISEC4(SIZE(NSEC4))

INTEGER(KIND=JPIM) :: I, J, JF, IEND, &
 & IBL, IST, IGRIBCD, ISENDLEN, IRECVLEN, &
 & JGL, JSTGLO, JROC, IPROC, ITAG, IREQ,  &
 & ISEC, IHOUR, IDAY

LOGICAL :: LWVDEBUG=.false.
 

REAL(KIND=JPRB) :: ZDUMMY(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB),ALLOCATABLE :: ZSENDBUF(:)
REAL(KIND=JPRB),ALLOCATABLE :: ZRECVBUF(:)
INTEGER(KIND=JPIM),ALLOCATABLE :: IMASK_WAVE_IN(:)
INTEGER(KIND=JPIM),ALLOCATABLE :: ILOCALINDEX(:)
INTEGER(KIND=JPIM),ALLOCATABLE :: IGLOBALPROC(:)
INTEGER(KIND=JPIM) :: ISENDREQ(NPROC)

#include "gathergpf.intfb.h"
#include "pregrbenc.intfb.h"
#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('WVXF2GB',0,ZHOOK_HANDLE)

CALL GSTATS_BARRIER(733)

! 1* Copy the grid definition from NSEC2GG, NDLON, and NDGLG and then
!    Prepare the time of the fields using NSEC1: 

K_NR = KDGLG

K_NC = NDLON

!*       WRITE SURFACE FIELDS
!  GIVE A DUMMY GRIB CODE (165 is used: U COMPONENT OF 10m WIND)

IGRIBCD=165
CALL PREGRBENC(IGRIBCD,0,'GG','SFC',&
   & ISEC0,K_ISEC1,K_ISEC2,ISEC3,ISEC4,&
   & ZDUMMY)
ALLOCATE(ILOCALINDEX(NGPTOTG))
ALLOCATE(IGLOBALPROC(NGPTOTG))
IGLOBALPROC=NGLOBALPROC
ILOCALINDEX=NLOCALINDEX

K_ISEC1(6)=165

! WAM REQUIRES THE STREAM TO BE DEFINED
IF (K_ISEC1(40) == 0 ) THEN
  IF(NSTREAM == 0) THEN
    K_ISEC1(40) = 1025
  ELSE
    K_ISEC1(40) = NSTREAM
  ENDIF
ENDIF  
! Set forecast range in minutes. Add the initial range nstepini (hours)
! of 4DVar trajectory run in case it starts from type=fc.
! Note that K_ISEC1(16) is no longer used by WAM but rather P_FCRANGE
! which is set in seconds except for output to grib if WAM uses the
! option of using the IFS grib header 1.

IF ((CTYPE == 'fc').AND.LOBSC1) THEN
  P_FCRANGE=REAL(NSTEP,JPRB)*TSTEP+NSTEPINI*3600
ELSE
  P_FCRANGE=REAL(NSTEP,JPRB)*TSTEP
ENDIF
IF(LVAREPS.AND.(NLEG >= 2)) THEN
  P_FCRANGE=REAL(NFCHO_TRUNC_INI,JPRB)*3600+P_FCRANGE
ENDIF

ISEC=NINT(P_FCRANGE)
IHOUR = ISEC/3600
IDAY  = IHOUR/24
IF (IHOUR <= 255) THEN
  K_ISEC1(15) = 1
  K_ISEC1(16) = IHOUR
  K_ISEC1(18) = 0
ELSEIF (IHOUR <= 65535) THEN
  K_ISEC1(15) = 1
  K_ISEC1(16) = IHOUR
  K_ISEC1(18) = 10
ELSEIF (IHOUR <= 65535*3) THEN
  K_ISEC1(15) = 10
  K_ISEC1(16) = IHOUR/3
  K_ISEC1(18) = 10
ELSEIF (IHOUR <= 65535*6) THEN
  K_ISEC1(15) = 11
  K_ISEC1(16) = IHOUR/6
  K_ISEC1(18) = 10
ELSEIF (IHOUR <= 65535*12) THEN
  K_ISEC1(15) = 12
  K_ISEC1(16) = IHOUR/12
  K_ISEC1(18) = 10
ELSE
  K_ISEC1(15) = 2
  K_ISEC1(16) = IDAY
  K_ISEC1(18) = 10
ENDIF
K_ISEC1(17) = 0



! 2* Read the work file from the grid-point-buffer:
! 2.1* READ WORK FILE

CALL GSTATS(1425,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JSTGLO,IST,IEND,IBL,I,JGL)
DO JSTGLO=1,KGPTOT,KPROMA
  IST    = 1
  IEND   = MIN(KPROMA,KGPTOT-JSTGLO+1)
  IBL=(JSTGLO-1)/KPROMA+1

! 2.2* COPY DATA INTO WORKFILE
  DO I=1, K_NFIELDS
    DO JGL =IST,IEND
      ZFLOC(JSTGLO+JGL-1+KGPTOT*(I-1)) = SD_WW(JGL,YSD_WW%YWW(I)%MP,IBL)
    ENDDO
  ENDDO
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1425,1)

! 3* Collect the fields from all PE's to all PEs:
IF( LWVIN_MASK_NOT_SET )THEN
  IF( SUM(MASK_WAVE_IN) > 0 )THEN
    LWVIN_MASK_NOT_SET=.false.
  ENDIF
ENDIF

IF( LWVIN_MASK_NOT_SET .OR. LDWCOUNORMS )THEN

! Get the whole field, as MASK_WAVE_IN is only initialised on
! the first call to the wave model 
! We will also get the whole field if the wave model requires 
! to produce global norms of input fields
  CALL GATHERGPF(ZFLOC,P_FIELDS,K_NFIELDS,-1)

ELSE

! Get only the subset of the fields that are needed on each PE
! using MASK_WAVE_IN which is now initialised

! Debug option allows us to check the correctness of the
! optimised communications
  IF( LWVDEBUG ) CALL GATHERGPF(ZFLOC,P_FIELDS,K_NFIELDS,-1)

  CALL GSTATS(678,0)

! Initialise data structures for optimised communications
  IF( LWVIN_UNINITIALISED )THEN
    LWVIN_UNINITIALISED=.false.
    ALLOCATE(MWVIN_SENDCNT(NPROC))
    ALLOCATE(MWVIN_SENDOFF(NPROC))
    ALLOCATE(MWVIN_RECVCNT(NPROC))
    ALLOCATE(MWVIN_RECVOFF(NPROC))
    MWVIN_SENDCNT(:)=0
    DO J=1,KGPTOTG
      IF( MASK_WAVE_IN(J) == 1 )THEN
        IPROC=IGLOBALPROC(J)
        MWVIN_SENDCNT(IPROC)=MWVIN_SENDCNT(IPROC)+1
      ENDIF
    ENDDO
    MWVIN_SENDOFF(:)=0
    MWVIN_SENDTOT=0
    DO JROC=1,NPROC
      IF( MWVIN_SENDCNT(JROC) > 0 )THEN
        MWVIN_SENDOFF(JROC)=MWVIN_SENDTOT
        MWVIN_SENDTOT=MWVIN_SENDTOT+MWVIN_SENDCNT(JROC)
      ENDIF
    ENDDO
    ALLOCATE(MWVIN_SENDBUF(MWVIN_SENDTOT))
    ALLOCATE(MWVIN_SENDIND(MWVIN_SENDTOT))
    MWVIN_SENDCNT(:)=0
    DO J=1,KGPTOTG
      IF( MASK_WAVE_IN(J) == 1 )THEN
        IPROC=IGLOBALPROC(J)
        MWVIN_SENDCNT(IPROC)=MWVIN_SENDCNT(IPROC)+1
        MWVIN_SENDBUF(MWVIN_SENDOFF(IPROC)+MWVIN_SENDCNT(IPROC))=ILOCALINDEX(J)
        MWVIN_SENDIND(MWVIN_SENDOFF(IPROC)+MWVIN_SENDCNT(IPROC))=J
      ENDIF
    ENDDO

    MWVIN_RECVCNT(:)=0
    ITAG=1
    DO JROC=1,NPROC-1
      IPROC=MOD(MYPROC+JROC-1,NPROC)+1
      CALL MPL_SEND(MWVIN_SENDCNT(IPROC),KDEST=NPRCIDS(IPROC),KTAG=ITAG,&
      & KMP_TYPE=JP_NON_BLOCKING_STANDARD,KREQUEST=ISENDREQ(JROC), &
      & CDSTRING='WVXF2GB: SEND COUNT ' )
    ENDDO
    DO JROC=1,NPROC-1
      IPROC=MOD(-JROC-1+MYPROC+NPROC,NPROC)+1
      CALL MPL_RECV(MWVIN_RECVCNT(IPROC),KSOURCE=NPRCIDS(IPROC),KTAG=ITAG,&
      & KMP_TYPE=JP_BLOCKING_STANDARD, &
      & CDSTRING='WVXF2GB: RECV COUNT ' )
    ENDDO
    CALL MPL_WAIT(MWVIN_SENDCNT(:),KREQUEST=ISENDREQ(1:NPROC-1), &
     & CDSTRING='WVXF2GB: WAIT SEND COUNT')

    MWVIN_RECVOFF(:)=0
    MWVIN_RECVTOT=0
    DO JROC=1,NPROC
      IF( MWVIN_RECVCNT(JROC) > 0 )THEN
        MWVIN_RECVOFF(JROC)=MWVIN_RECVTOT
        MWVIN_RECVTOT=MWVIN_RECVTOT+MWVIN_RECVCNT(JROC)
      ENDIF
    ENDDO

    ALLOCATE(MWVIN_RECVBUF(MWVIN_RECVTOT))

    ITAG=2
    IREQ=0
    DO JROC=1,NPROC-1
      IPROC=MOD(MYPROC+JROC-1,NPROC)+1
      IF( MWVIN_SENDCNT(IPROC) > 0 )THEN
        IREQ=IREQ+1
        CALL MPL_SEND(MWVIN_SENDBUF(MWVIN_SENDOFF(IPROC)+1:&
                                   &MWVIN_SENDOFF(IPROC)+MWVIN_SENDCNT(IPROC)),&
        & KDEST=NPRCIDS(IPROC),KTAG=ITAG,&
        & KMP_TYPE=JP_NON_BLOCKING_STANDARD,KREQUEST=ISENDREQ(IREQ), &
        & CDSTRING='WVXF2GB: SEND LIST ' )
      ENDIF
    ENDDO
    DO JROC=1,NPROC-1
      IPROC=MOD(-JROC-1+MYPROC+NPROC,NPROC)+1
      IF( MWVIN_RECVCNT(IPROC) > 0 )THEN
        CALL MPL_RECV(MWVIN_RECVBUF(MWVIN_RECVOFF(IPROC)+1:&
                                   &MWVIN_RECVOFF(IPROC)+MWVIN_RECVCNT(IPROC)),&
        & KSOURCE=NPRCIDS(IPROC),KTAG=ITAG,&
        & KMP_TYPE=JP_BLOCKING_STANDARD, &
        & CDSTRING='WVXF2GB: RECV LIST ' )
      ENDIF
    ENDDO
    CALL MPL_WAIT(MWVIN_SENDBUF(:),KREQUEST=ISENDREQ(1:IREQ), &
     & CDSTRING='WVXF2GB: WAIT SEND LIST')

  ENDIF
! End of initialisation of data structures


! MWVIN_SENDBUF contains local indexes of data on remote tasks that 
! this task needs 
! MWVIN_RECVBUF contains local indexes of data on this task that 
! remote tasks need

  ISENDLEN=MWVIN_RECVTOT*K_NFIELDS
  ALLOCATE(ZSENDBUF(ISENDLEN))

  ITAG=3
  IREQ=0
  I=0
  DO JROC=1,NPROC-1
    IPROC=MOD(MYPROC+JROC-1,NPROC)+1
    IF( MWVIN_RECVCNT(IPROC) > 0 )THEN
      IREQ=IREQ+1
      IST=I+1
      DO JF=1,K_NFIELDS
        DO J=1,MWVIN_RECVCNT(IPROC)
          I=I+1
          ZSENDBUF(I)=ZFLOC(MWVIN_RECVBUF(MWVIN_RECVOFF(IPROC)+J)+KGPTOT*(JF-1))
        ENDDO
      ENDDO
      IEND=I
      CALL MPL_SEND(ZSENDBUF(IST:IEND),KDEST=NPRCIDS(IPROC),KTAG=ITAG,&
      & KMP_TYPE=JP_NON_BLOCKING_STANDARD,KREQUEST=ISENDREQ(IREQ), &
      & CDSTRING='WVXF2GB: SEND DATA ' )
    ENDIF
  ENDDO
  IF( LWVDEBUG)THEN
    ALLOCATE(IMASK_WAVE_IN(KGPTOTG))
    IMASK_WAVE_IN=0
  ENDIF
  DO JROC=1,NPROC-1
    IPROC=MOD(-JROC-1+MYPROC+NPROC,NPROC)+1
    IF( MWVIN_SENDCNT(IPROC) > 0 )THEN
      IRECVLEN=MWVIN_SENDCNT(IPROC)*K_NFIELDS
      ALLOCATE(ZRECVBUF(1:IRECVLEN))
      CALL MPL_RECV(ZRECVBUF(1:IRECVLEN),KSOURCE=NPRCIDS(IPROC),KTAG=ITAG,&
      & KMP_TYPE=JP_BLOCKING_STANDARD, &
      & CDSTRING='WVXF2GB: RECV DATA ' )
      IF( LWVDEBUG )THEN
        DO JF=1,K_NFIELDS
          DO J=1,MWVIN_SENDCNT(IPROC)
            IF( P_FIELDS(MWVIN_SENDIND(MWVIN_SENDOFF(IPROC)+J),JF) /= &
              & ZRECVBUF(J+MWVIN_SENDCNT(IPROC)*(JF-1)) )THEN
              CALL ABOR1("WVXF2GB: PROBLEM OOPS1")
            ENDIF
            IMASK_WAVE_IN(MWVIN_SENDIND(MWVIN_SENDOFF(IPROC)+J))=1
          ENDDO
        ENDDO
      ELSE
        DO JF=1,K_NFIELDS
          DO J=1,MWVIN_SENDCNT(IPROC)
            P_FIELDS(MWVIN_SENDIND(MWVIN_SENDOFF(IPROC)+J),JF)=&
             &ZRECVBUF(J+MWVIN_SENDCNT(IPROC)*(JF-1))
          ENDDO
        ENDDO
      ENDIF
      DEALLOCATE(ZRECVBUF)
    ENDIF
  ENDDO
  CALL MPL_WAIT(ZSENDBUF(:),KREQUEST=ISENDREQ(1:IREQ), &
   & CDSTRING='WVXF2GB: WAIT SEND DATA')

! Now copy data that was located on this processor
  IF( LWVDEBUG )THEN
    DO JF=1,K_NFIELDS
      DO J=1,MWVIN_SENDCNT(MYPROC)
        IF( P_FIELDS(MWVIN_SENDIND(MWVIN_SENDOFF(MYPROC)+J),JF) /= &
          & ZFLOC(MWVIN_SENDBUF(MWVIN_SENDOFF(MYPROC)+J)+KGPTOT*(JF-1)) )THEN
          CALL ABOR1("WVXF2GB: PROBLEM OOPS2")
        ENDIF
        IMASK_WAVE_IN(MWVIN_SENDIND(MWVIN_SENDOFF(MYPROC)+J))=1
      ENDDO
    ENDDO
  ELSE
    DO JF=1,K_NFIELDS
      DO J=1,MWVIN_SENDCNT(MYPROC)
        P_FIELDS(MWVIN_SENDIND(MWVIN_SENDOFF(MYPROC)+J),JF)=&
         &ZFLOC(MWVIN_SENDBUF(MWVIN_SENDOFF(MYPROC)+J)+KGPTOT*(JF-1))
      ENDDO
    ENDDO
  ENDIF


  IF( LWVDEBUG )THEN
    DO J=1,KGPTOTG
      IF( MASK_WAVE_IN(J) == 1 .AND. IMASK_WAVE_IN(J) == 0 )THEN
        CALL ABOR1("WVXF2GB: PROBLEM OOPS3")
      ENDIF
      IF( MASK_WAVE_IN(J) == 0 .AND. IMASK_WAVE_IN(J) == 1 )THEN
        CALL ABOR1("WVXF2GB: PROBLEM OOPS4")
      ENDIF
    ENDDO
    DEALLOCATE(IMASK_WAVE_IN)
  ENDIF

  DEALLOCATE(ZSENDBUF)

  CALL GSTATS(678,1)

ENDIF
DEALLOCATE(ILOCALINDEX)
DEALLOCATE(IGLOBALPROC)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WVXF2GB',1,ZHOOK_HANDLE)
END SUBROUTINE WVXF2GB
