SUBROUTINE LWAD &
 & ( KIDIA, KFDIA , KLON   , KLEV  , KMODE, KTOPC, KTRAER,&
 & PCCO2  , PVIEW , &
 & PCLFR5, PCLDLD5, PCLDLU5,&
 & PDP5  , PDT05  , PEMIS5 , PEMIW5,&
 & PPMB5 , PQOF5  , PTL5   ,&
 & PAER5 , PTAVE5 , PWV5  ,&
 & PEMIT5, PFLUX5 , PFLUC5 ,&
 & PCLFR , PCLDLD , PCLDLU ,&
 & PDP   , PDT0   , PEMIS  , PEMIW ,&
 & PPMB  , PQOF   , PTL    ,&
 & PAER  , PTAVE  , PWV   ,&
 & PEMIT , PFLUX  , PFLUC  &
 & )

!**** *LWAD*   - ORGANIZES THE LONGWAVE CALCULATIONS
!                      (adjoint)

!     PURPOSE.
!     --------
!           COMPUTES LONGWAVE FLUXES 

!**   INTERFACE.
!     ----------

!        *LWTL* IS CALLED FROM *RADLSWTL*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!  Trajectory  Perturbation  Description                         Dimension
!  PCCO2       ---           CONCENTRATION IN CO2 (PA/PA)
!  PVIEW       ---           COSECANT OF VIEWING ANGLE           (KLON)
!  PAER5       PAER          OPTICAL THICKNESS OF THE AEROSOLS   (KLON,6,KLEV)
!  PCLFR5      PCLFR         CLOUD FRACTIONAL COVER              (KLON,KLEV)
!  PCLDLD5     PCLDLD        DOWNWARD EFFECTIVE CLOUD FRACTION   (KLON,KLEV)
!  PCLDLU5     PCLDLU        UPWARD EFFECTIVE CLOUD FRACTION     (KLON,KLEV)
!  PDP5        PDP           LAYER PRESSURE THICKNESS            (KLON,KLEV)
!  PDT05       PDT0          SURFACE TEMPERATURE DISCONTINUITY   (KLON)
!  PEMIS5      PEMIS         SURFACE LW EMISSIVITY               (KLON)
!  PEMIW5      PEMIW         SURFACE LW WINDOW EMISSIVITY        (KLON)
!  PPMB5       PPMB          HALF LEVEL PRESSURE                 (KLON,KLEV+1)
!  PQOF5       PQOF          CONCENTRATION IN OZONE (PA/PA)      (KLON,KLEV)
!  PTAVE5      PTAVE         TEMPERATURE                         (KLON,KLEV)
!  PTL5        PTL           HALF LEVEL TEMPERATURE              (KLON,KLEV+1)
!  PWV5        PWV           SPECIFIC HUMIDITY  (PA/PA)          (KLON,KLEV)

!     ==== OUTPUTS ===
!  Trajectory  Perturbation  Description                         Dimension
!  PEMIT5      PEMIT         SURFACE TOTAL LW EMISSIVITY         (KLON)
!  PFLUX5      PFLUX         RADIATIVE FLUXES :                  (KLON,2,KLEV)
!                     1  ==>  UPWARD   FLUX TOTAL
!                     2  ==>  DOWNWARD FLUX TOTAL
!  PFLUC5      PFLUC         RADIATIVE FLUXES CLEAR SKY:         (KLON,2,KLEV)
!                     1  ==>  UPWARD   FLUX TOTAL
!                     2  ==>  DOWNWARD FLUX TOTAL

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. COMPUTES THE PRESSURE AND TEMPERATURE WEIGHTED AMOUNTS OF
!     ABSORBERS.
!          2. COMPUTES THE PLANCK FUNCTIONS ON THE INTERFACES AND THE
!     GRADIENT OF PLANCK FUNCTIONS IN THE LAYERS.
!          3. PERFORMS THE VERTICAL INTEGRATION DISTINGUISHING THE CON-
!     TRIBUTIONS OF THE ADJACENT AND DISTANT LAYERS AND THOSE FROM THE
!     BOUNDARIES.
!          4. COMPUTES THE CLEAR-SKY DOWNWARD AND UPWARD EMISSIVITIES.
!          5. INTRODUCES THE EFFECTS OF THE CLOUDS ON THE FLUXES.

!     EXTERNALS.
!     ----------

!          *LWUAD*, *LWBVAD*, *LWCAD*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        M. Janiskova  *ECMWF*       04-Jan-2007

!     MODIFICATIONS.
!     --------------
!        M. Janiskova   4-Apr-2007 additional arguments passed out/in 
!                                  the routine for AD computation
!        M. Janiskova  20-May-2008 using shorter NPROMALW instead of 
!                                  KLON for some routines
!        D. Salmond    28-May-2008 correction of loop over NPROMALW
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOELW     ,ONLY : NUA     , NSIL     ,NIPD
USE YOEPHLI   ,ONLY : LPHYLIN
USE YOPHNC    ,ONLY : LH2OCO2 , LWLOPT
USE YOELWRAD  ,ONLY : NPROMALW, NLOOPLWO

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTOPC
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAER
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCO2
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVIEW(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLD5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLU5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDP5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT05(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS5(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIW5(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPMB5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQOF5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL5(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER5(KLON,6,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAVE5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWV5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIT5(KLON)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLUX5(KLON,2,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLUC5(KLON,2,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLFR(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLDLD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLDLU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDT0(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIS(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIW(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PPMB(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQOF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTL(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAER(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTAVE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PWV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PEMIT(KLON) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLUX(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLUC(KLON,2,KLEV+1)

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-------------------------------------------------------------------------

REAL(KIND=JPRB) :: ZABCU(KLON,NUA,3*KLEV+1)
REAL(KIND=JPRB) :: ZBINT(KLON,KLEV+1), ZBSUI(KLON)
REAL(KIND=JPRB) :: ZCNTRB(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB) :: ZBINT5(KLON,KLEV+1), ZBSUI5(KLON)

REAL(KIND=JPRB) :: ZGA(KLON,8,2,KLEV)  , ZGB(KLON,8,2,KLEV)
REAL(KIND=JPRB) :: ZGASUR(KLON,8,2)    , ZGBSUR(KLON,8,2)
REAL(KIND=JPRB) :: ZGATOP(KLON,8,2)    , ZGBTOP(KLON,8,2)
REAL(KIND=JPRB) :: ZB5(KLON,6,KLEV+1), ZBSUR5(KLON,6), ZBTOP5(KLON,6)
REAL(KIND=JPRB) :: ZDBSL5(KLON,6,KLEV*2)

REAL(KIND=JPRB) :: ZADJD5(KLON,KLEV+1), ZADJU5(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZDBDT5(KLON,NSIL,KLEV), ZDWFSU5(KLON,NSIL)
REAL(KIND=JPRB) :: ZDISD5(KLON,KLEV+1), ZDISU5(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTTPA15(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTTPB15(KLON,KLEV+1)

REAL(KIND=JPRB),ALLOCATABLE :: ZCOND5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZABCU5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZCNTRB5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZTTPA5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZTTPB5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZTTA5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZTTB5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZZZA5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZZZB5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZXNA5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZXNB5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZXDIVA5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZXDIVB5(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZZZA15(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZZZB15(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZXNA15(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZXNB15(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZXDIVA15(:,:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZXDIVB15(:,:,:)

INTEGER(KIND=JPIM) :: JLOOP
INTEGER(KIND=JPIM) :: IIDIA, IFDIA
INTEGER(KIND=JPIM) :: ILOOPLWO

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "lwu.intfb.h"
#include "lwbv.intfb.h"
#include "lwbvad.intfb.h"
#include "lwcad.intfb.h"
#include "lwuad.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWAD',0,ZHOOK_HANDLE)

!*         1.    INITIALIZATION
!                --------------

!100  CONTINUE

IF (LPHYLIN .AND. LH2OCO2) THEN
  ALLOCATE(ZCOND5(NPROMALW,KLEV+1,KLEV+1))
  ALLOCATE(ZTTPA5(NPROMALW,KLEV+1,KLEV+1))
  ALLOCATE(ZTTPB5(NPROMALW,KLEV+1,KLEV+1))
  ALLOCATE(ZTTA5(NPROMALW,KTRAER*KLEV,KLEV+1))
  ALLOCATE(ZTTB5(NPROMALW,KTRAER*KLEV,KLEV+1))
  ALLOCATE(ZZZA5(NPROMALW,KTRAER*KLEV,KLEV+1))
  ALLOCATE(ZZZB5(NPROMALW,KTRAER*KLEV,KLEV+1))
  ALLOCATE(ZXNA5(NPROMALW,KTRAER*KLEV,KLEV+1))
  ALLOCATE(ZXNB5(NPROMALW,KTRAER*KLEV,KLEV+1))
  ALLOCATE(ZXDIVA5(NPROMALW,KTRAER*KLEV,KLEV+1))
  ALLOCATE(ZXDIVB5(NPROMALW,KTRAER*KLEV,KLEV+1))
  ALLOCATE(ZZZA15(NPROMALW,KTRAER,KLEV+1))
  ALLOCATE(ZZZB15(NPROMALW,KTRAER,KLEV+1))
  ALLOCATE(ZXNA15(NPROMALW,KTRAER,KLEV+1))
  ALLOCATE(ZXNB15(NPROMALW,KTRAER,KLEV+1))
  ALLOCATE(ZXDIVA15(NPROMALW,KTRAER,KLEV+1))
  ALLOCATE(ZXDIVB15(NPROMALW,KTRAER,KLEV+1))

  ALLOCATE(ZABCU5(KLON,NUA,3*KLEV+1))
  ALLOCATE(ZCNTRB5(KLON,KLEV+1,KLEV+1))
ELSEIF (LPHYLIN .AND. LWLOPT) THEN
  ALLOCATE(ZTTA5(KLON,KTRAER*KLEV,KLEV+1))
  ALLOCATE(ZTTB5(KLON,KTRAER*KLEV,KLEV+1))
ENDIF

!     ------------------------------------------------------------------

!*         1.1   COMPUTES ABSORBER AMOUNTS
!                -------------------------

CALL LWU &
 & ( KIDIA, KFDIA, KLON, KLEV,&
 & PAER5 , PCCO2, PDP5 , PPMB5, PQOF5 , PTAVE5, PVIEW, PWV5,&
 & ZABCU5 &
 & )

!     ------------------------------------------------------------------
! moved here not to set variables to zero inside a special loop for NPROMALW

!*         0.    ADJOINT CALCULATIONS
!                --------------------

!* Set local variables to zero

ZABCU (:,:,:) = 0.0_JPRB
ZCNTRB(:,:,:) = 0.0_JPRB
ZBINT(:,:) = 0.0_JPRB
ZBSUI(:) = 0.0_JPRB

!     ------------------------------------------------------------------

! Loops for shorter NPROMALW to achieve KLON dimension

IF((KFDIA-KIDIA+1) <= NPROMALW)THEN
  ILOOPLWO=1
ELSE
  ILOOPLWO=NLOOPLWO
ENDIF

DO JLOOP = 1, ILOOPLWO

   IIDIA=((KFDIA-KIDIA+1)*(JLOOP-1))/ILOOPLWO+1
   IFDIA=((KFDIA-KIDIA+1)*(JLOOP))/ILOOPLWO

!     ------------------------------------------------------------------

!*         2.    COMPUTES PLANCK FUNCTIONS
!                -------------------------
!                PERFORMS THE VERTICAL INTEGRATION
!                ---------------------------------

  CALL LWBV &
   & ( IIDIA, IFDIA , KLON   , KLEV  , KMODE , KTRAER, &
   & PDT05 , PEMIS5, PEMIW5, PTL5   , PTAVE5,&
   & PEMIT5, PFLUC5,&
   & ZABCU5, ZBINT5, ZBSUI5, ZCNTRB5, &
! for adjoint computation
   & ZB5, ZBSUR5, ZBTOP5, ZDBSL5, &
   & ZGA, ZGB, ZGASUR, ZGBSUR, ZGATOP, ZGBTOP, &
   & ZADJD5, ZADJU5, ZDBDT5, ZDISD5, ZDISU5, ZDWFSU5, &
   & ZCOND5  , ZTTPA5  , ZTTPB5 , ZTTA5 , ZTTB5  ,&
   & ZZZA5   , ZZZB5   , ZXNA5  , ZXNB5 , ZXDIVA5,&
   & ZXDIVB5 , ZZZA15  , ZZZB15 , ZXNA15, ZXNB15 ,&
   & ZXDIVA15, ZXDIVB15, ZTTPA15, ZTTPB15 &
   & )

!     ------------------------------------------------------------------
! moved before special loop
!*
!!*         0.    ADJOINT CALCULATIONS
!!                --------------------
!!* Set local variables to zero
!ZABCU (:,:,:) = 0.0_JPRB
!ZCNTRB(:,:,:) = 0.0_JPRB
!ZBINT(:,:) = 0.0_JPRB
!ZBSUI(:) = 0.0_JPRB

ZABCU (IIDIA:IFDIA,:,:) = 0.0_JPRB
ZCNTRB(IIDIA:IFDIA,:,:) = 0.0_JPRB
ZBINT(IIDIA:IFDIA,:) = 0.0_JPRB
ZBSUI(IIDIA:IFDIA) = 0.0_JPRB

!     ------------------------------------------------------------------

!*         1.4.  INTRODUCES THE EFFECTS OF CLOUDS
!                --------------------------------

!  IF (KTOPC > 1) THEN

    CALL LWCAD &
     & ( IIDIA , IFDIA , KLON   , KLEV, KTOPC,&
     & ZBINT5  , ZBSUI5, PCLFR5 , PCLDLD5, PCLDLU5,&
     & ZCNTRB5 , PEMIT5, PFLUC5 ,&
     & PFLUX5  , &
     & ZBINT   , ZBSUI , PCLFR  , PCLDLD , PCLDLU ,&
     & ZCNTRB  , PEMIT , PFLUC  ,&
     & PFLUX   &
     & )

!  ENDIF

!     ------------------------------------------------------------------

!*         1.2.  COMPUTES PLANCK FUNCTIONS
!                -------------------------
!                PERFORMS THE VERTICAL INTEGRATION
!                ---------------------------------

  CALL LWBVAD &
   & ( IIDIA, IFDIA , KLON  , KLEV  , KMODE , KTRAER,&
   & PDT05  , PEMIS5, PEMIW5, PTL5  , PTAVE5,&
   & PEMIT5 , PFLUC5, &
   & ZABCU5 , ZBINT5, ZBSUI5, ZCNTRB5 ,&
! saved in NL coputation - lwbv
   & ZB5, ZBSUR5, ZBTOP5, ZDBSL5, &
   & ZGA, ZGB, ZGASUR, ZGBSUR, ZGATOP, ZGBTOP, &
   & ZADJD5, ZADJU5, ZDBDT5, ZDISD5, ZDISU5, ZDWFSU5, &
! saved in NL coputation - lwvdr
   & ZCOND5  , ZTTPA5  , ZTTPB5 , ZTTA5 , ZTTB5  ,&
   & ZZZA5   , ZZZB5   , ZXNA5  , ZXNB5 , ZXDIVA5,&
   & ZXDIVB5 , ZZZA15  , ZZZB15 , ZXNA15, ZXNB15 ,&
   & ZXDIVA15, ZXDIVB15, ZTTPA15, ZTTPB15, &
! perturbation
   & PDT0   , PEMIS , PEMIW , PTL   , PTAVE,&
   & PEMIT  , PFLUC ,&
   & ZABCU  , ZBINT , ZBSUI , ZCNTRB &
   & )

ENDDO ! end of loop for shorter NPROMALW


!*         1.1   COMPUTES ABSORBER AMOUNTS
!                -------------------------

CALL LWUAD &
 & (  KIDIA, KFDIA, KLON, KLEV,&
 & PVIEW , PCCO2, PAER5 , PDP5 , PPMB5, PQOF5 , PTAVE5, PWV5,&
 & ZABCU5, &
 & PAER  , PDP  , PPMB  , PQOF , PTAVE, PWV   ,&
 & ZABCU &
 & )

!     ------------------------------------------------------------------

IF (LPHYLIN .AND. LH2OCO2) THEN
  DEALLOCATE(ZCOND5)
  DEALLOCATE(ZTTPA5)
  DEALLOCATE(ZTTPB5)
  DEALLOCATE(ZTTA5)
  DEALLOCATE(ZTTB5)
  DEALLOCATE(ZZZA5)
  DEALLOCATE(ZZZB5)
  DEALLOCATE(ZXNA5)
  DEALLOCATE(ZXNB5)
  DEALLOCATE(ZXDIVA5)
  DEALLOCATE(ZXDIVB5)
  DEALLOCATE(ZZZA15)
  DEALLOCATE(ZZZB15)
  DEALLOCATE(ZXNA15)
  DEALLOCATE(ZXNB15)
  DEALLOCATE(ZXDIVA15)
  DEALLOCATE(ZXDIVB15)
  DEALLOCATE(ZABCU5)
  DEALLOCATE(ZCNTRB5)
ELSEIF (LPHYLIN .AND. LWLOPT) THEN
  DEALLOCATE(ZTTA5)
  DEALLOCATE(ZTTB5)
ENDIF

IF (LHOOK) CALL DR_HOOK('LWAD',1,ZHOOK_HANDLE)
END SUBROUTINE LWAD
