SUBROUTINE LWAI( KIDIA , KFDIA, KLON  , KLEV, PCLDLD, PCLDLU, PCLFR, P_AI, P_AIC)

!**** *LWAI*   - LONGWAVE RADIATION, CLOUD EFFECTS

!     PURPOSE.
!     --------
!           COMPUTES THE WEIGHTS OF THE BLACK BODY-DERIVED FLUXES
!           RADIANCES

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
! PCLDLD : (KLON,KLEV)         ; DOWNWARD EFFECTIVE CLOUD FRACTION
! PCLDLU : (KLON,KLEV)         ; UPWARD EFFECTIVE CLOUD FRACTION
! PCLFR  : (KLON,KLEV)         ; CLOUD FRACTIONAL COVER
!     ==== OUTPUTS ===
! AI
!                     1  ==>  UPWARD   FLUX TOTAL
!                     2  ==>  DOWNWARD FLUX TOTAL

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. INITIALIZES ALL FLUXES TO CLEAR-SKY VALUES
!          2. EFFECT OF ONE OVERCAST UNITY EMISSIVITY CLOUD LAYER
!          3. EFFECT OF SEMI-TRANSPARENT, PARTIAL OR MULTI-LAYERED
!     CLOUDS

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 89-07-14
!        JJ Morcrette         97-04-18   Cleaning
!        F. Chevallier        99-05 ai extraction
!        F. Chevallier        01-04 P. Raisanen overlap scheme
!        M. Janiskova         01-11 included safety checks for pclfr
!        D.Salmond & J.Hague  03-04 Optimisation
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOERAD   , ONLY : NOVLP
USE YOERDI   , ONLY : REPCLC
USE YOEPHLI  , ONLY : LRAISANEN

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_AI(KLON,KLEV+1,KLEV+1,1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_AIC(KLON,KLEV+1) 
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZCLFR(KLON,KLEV)
REAL(KIND=JPRB) :: ZCLEAR(KLON) , ZCLOUD(KLON),  ZCLM(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) :: ZEMIS(KLON,KLEV)
REAL(KIND=JPRB) :: ZSUM(KLON)

REAL(KIND=JPRB) :: ZTMP1(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTMP3(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTMP4(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTMP5(KLON,KLEV+1)

INTEGER(KIND=JPIM) :: IMAXC, IMXM1, IMXP1, JK, JK1, JL, IKL

REAL(KIND=JPRB) :: ZCFRAC
REAL(KIND=JPRB) :: ZTR1, ZTR2, ZCADJ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*         1.     INITIALIZATION
!                 --------------

!100  CONTINUE

IF (LHOOK) CALL DR_HOOK('LWAI',0,ZHOOK_HANDLE)
DO JL = KIDIA,KFDIA
  ZCLOUD(JL) = 0.0_JPRB
ENDDO

!GM*******
IMAXC=KLEV
!GM*******

!     ------------------------------------------------------------------

!*         2.      EFFECT OF CLOUDINESS ON LONGWAVE FLUXES
!                  ---------------------------------------

IMXP1 = IMAXC + 1
IMXM1 = IMAXC - 1

!*         2.2     CLOUD COVER MATRIX
!                  ------------------

!*    ZCLM(JK1,JK2) IS THE OBSCURATION FACTOR BY CLOUD LAYERS BETWEEN
!     HALF-LEVELS JK1 AND JK2 AS SEEN FROM JK1

!220  CONTINUE

! warning: ZCLM(JL,KLEV+1,JK) is not initialized, because it is not used
! matrix notation for ZCLM is not standard, but:
!    z11  z21  z31  ...
!    z12  z22  z32  ...
!    z13  z23  z33  ...
!     :    :    :    :

!*         2.4     CLOUD COVER BELOW THE LEVEL OF CALCULATION
!                  ------------------------------------------

!240  CONTINUE

IF (LRAISANEN) THEN
!* maximum-random (Raisanen 1998)

  ZEMIS(:,:) = 0.
  DO JK = 1,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZCLFR(JL,IKL) = PCLFR(JL,IKL)      
      IF (ZCLFR(JL,IKL) <= REPCLC) THEN
        ZCLFR(JL,IKL) = REPCLC
      ENDIF
      IF (ZCLFR(JL,IKL) >= 1.0_JPRB-REPCLC) THEN
        ZCLFR(JL,IKL) = 1.0_JPRB-REPCLC
      ENDIF
      ZEMIS(JL,JK) = PCLDLU(JL,JK) / ZCLFR(JL,IKL)
    ENDDO
  ENDDO

  DO JK = 1,KLEV-1
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZTMP1(JL,JK)=1.0_JPRB/MAX(REPCLC,PCLDLU(JL,JK+1))
      ZCADJ = MIN(ZCLFR(JL,IKL),ZCLFR(JL,IKL-1))
      ZTMP3(JL,JK)=1.0_JPRB/(1.-ZCLFR(JL,IKL-1))
      ZTMP4(JL,JK)=ZCADJ*(1.-ZEMIS(JL,JK+1))
      ZTMP5(JL,JK)=ZCLFR(JL,IKL)-ZCADJ
    ENDDO
  ENDDO

  DO JK1 = 2 , KLEV+1
    DO JL = KIDIA,KFDIA
      ZCLOUD(JL) = PCLDLU(JL,JK1-1)
      ZSUM(JL) = ZCLOUD(JL)
      ZCLM(JL,JK1-1,JK1) = ZSUM(JL)
    ENDDO
    DO JK = JK1 - 2 , 1 , -1
      IKL = KLEV+1 - JK
      DO JL = KIDIA,KFDIA
        ZTR1 = ZCLOUD(JL) * ZTMP1(JL,JK)
        ZTR2 = (1.-(ZSUM(JL)-ZCLOUD(JL))-ZTR1*ZCLFR(JL,IKL-1))*ZTMP3(JL,JK)
        ZCLOUD(JL) = ZEMIS(JL,JK)*(ZTMP4(JL,JK)*ZTR1 + ZTMP5(JL,JK)*ZTR2)
        ZSUM(JL) = ZSUM(JL) + ZCLOUD(JL)
        ZCLM(JL,JK,JK1 ) = ZSUM(JL)
      ENDDO
    ENDDO
  ENDDO

ELSE

  DO JK1 = 2 , KLEV+1
    DO JL = KIDIA,KFDIA
      ZCLEAR(JL)=1.0_JPRB
      ZCLOUD(JL)=0.0_JPRB
    ENDDO

    DO JK = JK1 - 1 , 1 , -1
      DO JL = KIDIA,KFDIA
        IF (NOVLP == 1) THEN
!* maximum-random      Geleyn and Hollingsworth 1979
          ZCLEAR(JL)=ZCLEAR(JL)*(1.0_JPRB-MAX(PCLDLU(JL,JK),ZCLOUD(JL)))&
           & /(1.0_JPRB-MIN(ZCLOUD(JL),1.0_JPRB-REPCLC))  
          ZCLM(JL,JK,JK1) = 1.0_JPRB - ZCLEAR(JL)
          ZCLOUD(JL) = PCLDLU(JL,JK)
        ELSEIF (NOVLP == 2) THEN
!* maximum
          ZCLOUD(JL) = MAX(ZCLOUD(JL) , PCLDLU(JL,JK))
          ZCLM(JL,JK,JK1) = ZCLOUD(JL)
        ELSEIF (NOVLP == 3) THEN
!* random
          ZCLEAR(JL) = ZCLEAR(JL)*(1.0_JPRB - PCLDLU(JL,JK))
          ZCLOUD(JL) = 1.0_JPRB - ZCLEAR(JL)
          ZCLM(JL,JK,JK1) = ZCLOUD(JL)
        ENDIF
      ENDDO
    ENDDO

  ENDDO

ENDIF

!*         2.5     CLOUD COVER ABOVE THE LEVEL OF CALCULATION
!                  ------------------------------------------

!250  CONTINUE

IF (LRAISANEN) THEN
!* maximum-random (Raisanen 1998)

  ZEMIS(:,:) = 0.
  DO JK = 1,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZEMIS(JL,JK) = PCLDLD(JL,JK) / ZCLFR(JL,IKL)
    ENDDO
  ENDDO

  DO JK = 2,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZTMP1(JL,JK)=1.0_JPRB/MAX(REPCLC,PCLDLD(JL,JK-1))
      ZCADJ = MIN(ZCLFR(JL,IKL),ZCLFR(JL,IKL+1))
      ZTMP3(JL,JK)=1.0_JPRB/(1.-ZCLFR(JL,IKL+1))
      ZTMP4(JL,JK)=ZCADJ*(1.-ZEMIS(JL,JK-1))
      ZTMP5(JL,JK)=ZCLFR(JL,IKL)-ZCADJ
    ENDDO
  ENDDO

  DO JK1 = 1 , KLEV 
    DO JL = KIDIA,KFDIA
      ZCLOUD(JL) = PCLDLD(JL,JK1)
      ZSUM(JL) = ZCLOUD(JL)
      ZCLM(JL,JK1,JK1  ) = ZSUM(JL)
    ENDDO
    DO JK = JK1+1 , KLEV
      IKL = KLEV + 1 - JK
      DO JL = KIDIA,KFDIA
        ZTR1 = ZCLOUD(JL)*ZTMP1(JL,JK)
        ZTR2 = (1.-(ZSUM(JL)-ZCLOUD(JL))-ZTR1*ZCLFR(JL,IKL+1))*ZTMP3(JL,JK)
        ZCLOUD(JL) = ZEMIS(JL,JK)*(ZTMP4(JL,JK)*ZTR1 + ZTMP5(JL,JK)*ZTR2)
        ZSUM(JL) = ZSUM(JL) + ZCLOUD(JL)
        ZCLM(JL,JK,JK1 ) = ZSUM(JL)
      ENDDO
    ENDDO
  ENDDO

ELSE

  DO JK1 = 1 , KLEV
    DO JL = KIDIA,KFDIA
      ZCLEAR(JL)=1.0_JPRB
      ZCLOUD(JL)=0.0_JPRB
    ENDDO

    DO JK = JK1 , KLEV
      DO JL = KIDIA,KFDIA
        IF (NOVLP == 1) THEN
!* maximum-random
          ZCLEAR(JL)=ZCLEAR(JL)*(1.0_JPRB-MAX(PCLDLD(JL,JK),ZCLOUD(JL)))&
           & /(1.0_JPRB-MIN(ZCLOUD(JL),1.0_JPRB-REPCLC))  
          ZCLM(JL,JK,JK1) = 1.0_JPRB - ZCLEAR(JL)
          ZCLOUD(JL) = PCLDLD(JL,JK)
        ELSEIF (NOVLP == 2) THEN
!* maximum
          ZCLOUD(JL) = MAX(ZCLOUD(JL) , PCLDLD(JL,JK))
          ZCLM(JL,JK,JK1) = ZCLOUD(JL)
        ELSEIF (NOVLP == 3) THEN
!* random
          ZCLEAR(JL) = ZCLEAR(JL)*(1.0_JPRB - PCLDLD(JL,JK))
          ZCLOUD(JL) = 1.0_JPRB - ZCLEAR(JL)
          ZCLM(JL,JK,JK1) = ZCLOUD(JL)
        ENDIF
      ENDDO
    ENDDO
  ENDDO

ENDIF

!*         3.      FLUXES FOR PARTIAL/MULTIPLE LAYERED CLOUDINESS
!                  ----------------------------------------------

!300  CONTINUE

DO JL = KIDIA,KFDIA
! AI(JL,1,KLEV+1,2) = _ZERO_
  P_AIC(JL,KLEV+1) = 0.0_JPRB
ENDDO

!*         3.1     DOWNWARD FLUXES
!                  ---------------

!310  CONTINUE

DO JK1 = 1, KLEV 

!*                 CONTRIBUTION FROM CLEAR-SKY FRACTION

  DO JL = KIDIA,KFDIA
!   AI(JL,1,JK1,2) = (_ONE_ - ZCLM(JL,KLEV,JK1))
    P_AIC(JL,JK1) = (1.0_JPRB - ZCLM(JL,KLEV,JK1))

!*                 CONTRIBUTION FROM ADJACENT CLOUD

    P_AI(JL,JK1+1,JK1,1) = ZCLM(JL,JK1,JK1)
  ENDDO

!*                 CONTRIBUTION FROM OTHER CLOUDY FRACTIONS

  DO JK = JK1, KLEV-1
    DO JL = KIDIA,KFDIA
      ZCFRAC = ZCLM(JL,JK+1,JK1) - ZCLM(JL,JK,JK1)
      P_AI(JL,JK+2,JK1,1) =  ZCFRAC
    ENDDO
  ENDDO

ENDDO

!*         3.2     UPWARD FLUX AT THE SURFACE
!                  --------------------------

DO JL = KIDIA,KFDIA
  P_AI(JL,1,1,1) = 1.0_JPRB
ENDDO

!320  CONTINUE

!*         3.3     UPWARD FLUXES
!                  -------------

!330  CONTINUE

DO JK1 = 2 , KLEV+1

!*                 CONTRIBUTION FROM CLEAR-SKY FRACTION

  DO JL = KIDIA,KFDIA
    P_AI(JL,1,JK1,1) = (1.0_JPRB - ZCLM(JL,1,JK1))
  ENDDO

!*                 CONTRIBUTION FROM OTHER CLOUDY FRACTIONS

  DO JK = 2 , JK1-1
    DO JL = KIDIA,KFDIA
      ZCFRAC = ZCLM(JL,JK-1,JK1) - ZCLM(JL,JK,JK1)
      P_AI(JL,JK,JK1,1) = ZCFRAC
    ENDDO
  ENDDO

!*                 CONTRIBUTION FROM ADJACENT CLOUD

  DO JL = KIDIA,KFDIA
    P_AI(JL,JK1,JK1,1) =  ZCLM(JL,JK1-1,JK1)
  ENDDO

ENDDO

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWAI',1,ZHOOK_HANDLE)
END SUBROUTINE LWAI
