SUBROUTINE LWAIAD ( KIDIA , KFDIA, KLON  , KLEV,&
 & PCLDLD5, PCLDLU5, PCLFR5, P_AI5, P_AIC5,&
 & PCLDLD , PCLDLU , PCLFR , P_AI , P_AIC )  

!**** *LWAIAD*   -  LONGWAVE RADIATION, CLOUD EFFECTS
!                      (adjoint)

!     PURPOSE.
!     --------
!           COMPUTES THE WEIGHTS OF THE BLACK BODY-DERIVED FLUXES
!           RADIANCES

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
! PCLDLD5 : (KLON,KLEV)   DOWNWARD EFFECTIVE CLOUD FRACTION (Trajectory)
! PCLDLU5 : (KLON,KLEV)   UPWARD EFFECTIVE CLOUD FRACTION   (Trajectory)
! PCLFR5  : (KLON,KLEV)   CLOUD FRACTIONAL COVER (Trajectory)
! PCLDLD : (KLON,KLEV)    DOWNWARD EFFECTIVE CLOUD FRACTION
! PCLDLU : (KLON,KLEV)    UPWARD EFFECTIVE CLOUD FRACTION
! PCLFR  : (KLON,KLEV)    CLOUD FRACTIONAL COVER

!     ==== OUTPUTS ===
! AI5                     (Trajectory)
! AI
!                     1  ==>  UPWARD   FLUX TOTAL
!                     2  ==>  DOWNWARD FLUX TOTAL

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. INITIALIZES ALL FLUXES TO CLEAR-SKY VALUES
!          2. EFFECT OF ONE OVERCAST UNITY EMISSIVITY CLOUD LAYER
!          3. EFFECT OF SEMI-TRANSPARENT, PARTIAL OR MULTI-LAYERED
!             CLOUDS

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!       M. Janiskova  2000-01     adjoint of lwai 

!     MODIFICATIONS.
!     --------------
!       M. Janiskova  2001-05  AD of Raisanen overlap scheme
!       M. Janiskova  2001-11  included safety checks for pclfr
!       M. Janiskova  2003-05  AD version for optimisation of DS&JH
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOERAD   , ONLY : NOVLP
USE YOERDI   , ONLY : REPCLC
USE YOEPHLI  , ONLY : LRAISANEN

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_AI5(KLON,KLEV+1,KLEV+1,1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: P_AIC5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLDLD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLDLU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLFR(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P_AI(KLON,KLEV+1,KLEV+1,1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: P_AIC(KLON,KLEV+1) 
!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZCLFR5(KLON,KLEV)
REAL(KIND=JPRB) :: ZCLEAR5(KLON,KLEV+1,KLEV+1) , ZCLOUD5(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) :: ZCLM5(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) :: ZNUM5(KLON,KLEV+1,KLEV+1) , ZDEN5(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) :: ZCLEARA5(KLON,KLEV+1,KLEV+1),ZCLOUDA5(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) :: ZSUMA5(KLON,KLEV+1,KLEV+1), ZSUMB5(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) :: ZTMPA15(KLON,KLEV+1), ZTMPA35(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTMPA45(KLON,KLEV+1), ZTMPA55(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTMPB15(KLON,KLEV+1), ZTMPB35(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTMPB45(KLON,KLEV+1), ZTMPB55(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTR15(KLON,KLEV+1,KLEV+1) , ZTR25(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) :: ZEMISA5(KLON,KLEV), ZCADJA5(KLON,KLEV)
REAL(KIND=JPRB) :: ZEMISB5(KLON,KLEV), ZCADJB5(KLON,KLEV)

REAL(KIND=JPRB) :: ZCLFR(KLON,KLEV)
REAL(KIND=JPRB) :: ZCLEAR(KLON) , ZCLOUD(KLON),  ZCLM(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) :: ZSUM(KLON), ZEMIS(KLON,KLEV)
REAL(KIND=JPRB) :: ZTMP1(KLON,KLEV+1), ZTMP3(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTMP4(KLON,KLEV+1), ZTMP5(KLON,KLEV+1)

INTEGER(KIND=JPIM) :: IMAXC, IMXM1, IMXP1, JK, JK1, JL, IKL

REAL(KIND=JPRB) :: ZCFRAC, ZNUM, ZDEN, ZCFRAC5 
REAL(KIND=JPRB) :: ZTR1, ZTR2, ZCADJ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*         1.     INITIALIZATION
!                 --------------

!100  CONTINUE

IF (LHOOK) CALL DR_HOOK('LWAIAD',0,ZHOOK_HANDLE)
ZCLOUD5(:,:,:) = 0.0_JPRB 

!GM*******
IMAXC=KLEV
!GM*******

!     ------------------------------------------------------------------

!*         2.      EFFECT OF CLOUDINESS ON LONGWAVE FLUXES
!                  ---------------------------------------

IMXP1 = IMAXC + 1
IMXM1 = IMAXC - 1

!*         2.2     CLOUD COVER MATRIX
!                  ------------------

!*    ZCLM(JK1,JK2) IS THE OBSCURATION FACTOR BY CLOUD LAYERS BETWEEN
!     HALF-LEVELS JK1 AND JK2 AS SEEN FROM JK1

!220  CONTINUE

! warning: ZCLM(JL,KLEV+1,JK) is not initialized, because it is not used
! matrix notation for ZCLM is not standard, but:
!    z11  z21  z31  ...
!    z12  z22  z32  ...
!    z13  z23  z33  ...
!     :    :    :    :

!*         2.4     CLOUD COVER BELOW THE LEVEL OF CALCULATION
!                  ------------------------------------------

!240  CONTINUE

IF (LRAISANEN) THEN

!* maximum-random (Raisanen 1998)

  ZEMISB5(:,:) = 0.0_JPRB
  DO JK = 1,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZCLFR5(JL,IKL) = PCLFR5(JL,IKL)
      IF (ZCLFR5(JL,IKL) <= REPCLC) THEN
        ZCLFR5(JL,IKL) = REPCLC
      ENDIF
      IF (ZCLFR5(JL,IKL) >= 1.0_JPRB-REPCLC) THEN
        ZCLFR5(JL,IKL) = 1.0_JPRB-REPCLC
      ENDIF
      ZEMISB5(JL,JK) = PCLDLU5(JL,JK) / ZCLFR5(JL,IKL)
    ENDDO
  ENDDO

  DO JK = 1,KLEV-1
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      IF (PCLDLU5(JL,JK+1) <= REPCLC) THEN
        ZTMPB15(JL,JK) = 1.0_JPRB/REPCLC
      ELSE
        ZTMPB15(JL,JK) = 1.0_JPRB/PCLDLU5(JL,JK+1)
      ENDIF
      IF (ZCLFR5(JL,IKL) <= ZCLFR5(JL,IKL-1)) THEN
        ZCADJB5(JL,JK) = ZCLFR5(JL,IKL)
      ELSE
        ZCADJB5(JL,JK) = ZCLFR5(JL,IKL-1)
      ENDIF      
      ZTMPB35(JL,JK) = 1.0_JPRB/(1.0_JPRB-ZCLFR5(JL,IKL-1))
      ZTMPB45(JL,JK) = ZCADJB5(JL,JK)*(1.0_JPRB-ZEMISB5(JL,JK+1))
      ZTMPB55(JL,JK) = ZCLFR5(JL,IKL)-ZCADJB5(JL,JK)
    ENDDO
  ENDDO

  DO JK1 = 2 , KLEV+1
    DO JL = KIDIA,KFDIA
      ZCLOUD5(JL,JK1-1,JK1) = PCLDLU5(JL,JK1-1)
      ZSUMB5(JL,JK1-1,JK1)  = ZCLOUD5(JL,JK1-1,JK1)
      ZCLM5(JL,JK1-1,JK1) = ZSUMB5(JL,JK1,JK1-1)
    ENDDO
    DO JK = JK1 - 2 , 1 , -1
      IKL = KLEV+1 - JK
      DO JL = KIDIA,KFDIA
        ZTR15(JL,JK,JK1) = ZCLOUD5(JL,JK+1,JK1)*ZTMPB15(JL,JK)
        ZTR25(JL,JK,JK1) = (1.0_JPRB-(ZSUMB5(JL,JK+1,JK1) &
         & -ZCLOUD5(JL,JK+1,JK1))-ZTR15(JL,JK,JK1) &
         & *ZCLFR5(JL,IKL-1))*ZTMPB35(JL,JK)  
        ZCLOUD5(JL,JK,JK1) = ZEMISB5(JL,JK) &
         & * (ZTMPB45(JL,JK)*ZTR15(JL,JK,JK1) &
         & + ZTMPB55(JL,JK)*ZTR25(JL,JK,JK1))  

        ZSUMB5(JL,JK,JK1) = ZSUMB5(JL,JK+1,JK1) + ZCLOUD5(JL,JK,JK1)
        ZCLM5(JL,JK,JK1) = ZSUMB5(JL,JK,JK1)
      ENDDO
    ENDDO
  ENDDO

ELSE

  DO JK1 = 2 , KLEV+1
    DO JL = KIDIA,KFDIA
      ZCLEAR5(JL,JK1,JK1)=1.0_JPRB
      ZCLOUD5(JL,JK1,JK1)=0.0_JPRB
    ENDDO

    DO JK = JK1 - 1 , 1 , -1
      DO JL = KIDIA,KFDIA
        IF (NOVLP == 1) THEN
!* maximum-random
          IF (PCLDLU5(JL,JK) >= ZCLOUD5(JL,JK+1,JK1)) THEN
            ZNUM5(JL,JK,JK1) = PCLDLU5(JL,JK)
          ELSE
            ZNUM5(JL,JK,JK1) = ZCLOUD5(JL,JK+1,JK1)
          ENDIF
          IF (ZCLOUD5(JL,JK+1,JK1) <= 1.0_JPRB-REPCLC) THEN
            ZDEN5(JL,JK,JK1) = ZCLOUD5(JL,JK+1,JK1)
          ELSE
            ZDEN5(JL,JK,JK1)= 1.0_JPRB-REPCLC
          ENDIF
          ZCLEAR5(JL,JK,JK1)=ZCLEAR5(JL,JK+1,JK1) &
           & *(1.0_JPRB-ZNUM5(JL,JK,JK1))/(1.0_JPRB-ZDEN5(JL,JK,JK1))  
          ZCLM5(JL,JK,JK1) = 1.0_JPRB - ZCLEAR5(JL,JK,JK1)
          ZCLOUD5(JL,JK,JK1) = PCLDLU5(JL,JK)
        ELSEIF (NOVLP == 2) THEN
!* maximum
          IF (PCLDLU5(JL,JK) >= ZCLOUD5(JL,JK+1,JK1)) THEN
            ZCLOUD5(JL,JK,JK1) = PCLDLU5(JL,JK)
          ELSE
            ZCLOUD5(JL,JK,JK1) = ZCLOUD5(JL,JK+1,JK1)  
          ENDIF
          ZCLM5(JL,JK,JK1) = ZCLOUD5(JL,JK,JK1)
        ELSEIF (NOVLP == 3) THEN
!* random
          ZCLEAR5(JL,JK,JK1) = ZCLEAR5(JL,JK+1,JK1) &
           & *(1.0_JPRB - PCLDLU5(JL,JK))  
          ZCLOUD5(JL,JK,JK1) = 1.0_JPRB - ZCLEAR5(JL,JK,JK1)
          ZCLM5(JL,JK,JK1) = ZCLOUD5(JL,JK,JK1)
        ENDIF
      ENDDO
    ENDDO
  ENDDO

ENDIF

!*         2.5     CLOUD COVER ABOVE THE LEVEL OF CALCULATION
!                  ------------------------------------------

!250  CONTINUE

IF (LRAISANEN) THEN
!* maximum-random (Raisanen 1998)

  ZEMISA5(:,:) = 0.0_JPRB
  DO JK = 1,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZEMISA5(JL,JK) = PCLDLD5(JL,JK) / ZCLFR5(JL,IKL)
    ENDDO
  ENDDO

  DO JK = 2,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      IF (PCLDLD5(JL,JK-1) <= REPCLC) THEN
        ZTMPA15(JL,JK) = 1.0_JPRB/REPCLC
      ELSE
        ZTMPA15(JL,JK) = 1.0_JPRB/PCLDLD5(JL,JK-1)
      ENDIF
      IF (ZCLFR5(JL,IKL) <= ZCLFR5(JL,IKL+1)) THEN
        ZCADJA5(JL,JK) = ZCLFR5(JL,IKL)
      ELSE
        ZCADJA5(JL,JK) = ZCLFR5(JL,IKL+1)
      ENDIF     
      ZTMPA35(JL,JK) = 1.0_JPRB/(1.0_JPRB-ZCLFR5(JL,IKL+1))
      ZTMPA45(JL,JK) = ZCADJA5(JL,JK)*(1.0_JPRB-ZEMISA5(JL,JK-1))
      ZTMPA55(JL,JK) = ZCLFR5(JL,IKL)-ZCADJA5(JL,JK)
    ENDDO
  ENDDO

  DO JK1 = 1 , KLEV
    DO JL = KIDIA,KFDIA
      ZCLOUDA5(JL,JK1,JK1) = PCLDLD5(JL,JK1)
      ZSUMA5(JL,JK1,JK1)   = ZCLOUDA5(JL,JK1,JK1)
      ZCLM5(JL,JK1,JK1)    = ZSUMA5(JL,JK1,JK1)
    ENDDO

    DO JK = JK1+1 , KLEV
      IKL = KLEV + 1 - JK
      DO JL = KIDIA,KFDIA
        ZTR15(JL,JK,JK1) = ZCLOUDA5(JL,JK-1,JK1)*ZTMPA15(JL,JK)
        ZTR25(JL,JK,JK1) = (1.0_JPRB-(ZSUMA5(JL,JK-1,JK1) &
         & -ZCLOUDA5(JL,JK-1,JK1))-ZTR15(JL,JK,JK1) &
         & *ZCLFR5(JL,IKL+1))*ZTMPA35(JL,JK)  
        ZCLOUDA5(JL,JK,JK1) = ZEMISA5(JL,JK) &
         & * (ZTMPA45(JL,JK)*ZTR15(JL,JK,JK1) &
         & + ZTMPA55(JL,JK)*ZTR25(JL,JK,JK1))  

        ZSUMA5(JL,JK,JK1) = ZSUMA5(JL,JK-1,JK1) + ZCLOUDA5(JL,JK,JK1)
        ZCLM5(JL,JK,JK1 ) = ZSUMA5(JL,JK,JK1)
      ENDDO
    ENDDO
  ENDDO

ELSE

  DO JK1 = 1 , KLEV
    DO JL = KIDIA,KFDIA
      ZCLEARA5(JL,JK1,JK1)=1.0_JPRB
      ZCLOUDA5(JL,JK1,JK1)=0.0_JPRB
    ENDDO

    DO JK = JK1 , KLEV
      DO JL = KIDIA,KFDIA
        IF (JK /= JK1) THEN
          ZCLOUDA5(JL,JK,JK1)=ZCLOUD5(JL,JK-1,JK1)
          ZCLEARA5(JL,JK,JK1)=ZCLEAR5(JL,JK-1,JK1)
        ENDIF
        IF (NOVLP == 1) THEN
!* maximum-random
          IF (PCLDLD5(JL,JK) >= ZCLOUDA5(JL,JK,JK1)) THEN
            ZNUM5(JL,JK,JK1) = PCLDLD5(JL,JK)
          ELSE
            ZNUM5(JL,JK,JK1) = ZCLOUDA5(JL,JK,JK1)
          ENDIF
          IF (ZCLOUDA5(JL,JK,JK1) <= 1.0_JPRB-REPCLC) THEN
            ZDEN5(JL,JK,JK1) = ZCLOUDA5(JL,JK,JK1)
          ELSE
            ZDEN5(JL,JK,JK1) = 1.0_JPRB-REPCLC
          ENDIF
          ZCLEAR5(JL,JK,JK1)=ZCLEARA5(JL,JK,JK1) &
           & *(1.0_JPRB-ZNUM5(JL,JK,JK1))/(1.0_JPRB-ZDEN5(JL,JK,JK1))  
          ZCLM5(JL,JK,JK1) = 1.0_JPRB - ZCLEAR5(JL,JK,JK1)
          ZCLOUD5(JL,JK,JK1) = PCLDLD5(JL,JK)
        ELSEIF (NOVLP == 2) THEN
!* maximum
          IF (PCLDLD5(JL,JK) >= ZCLOUDA5(JL,JK,JK1)) THEN
            ZCLOUD5(JL,JK,JK1) = PCLDLD5(JL,JK)
          ELSE
            ZCLOUD5(JL,JK,JK1) = ZCLOUDA5(JL,JK,JK1)
          ENDIF
          ZCLM5(JL,JK,JK1) = ZCLOUD5(JL,JK,JK1)
        ELSEIF (NOVLP == 3) THEN
!* random
          ZCLEAR5(JL,JK,JK1) = ZCLEARA5(JL,JK,JK1) &
           & *(1.0_JPRB - PCLDLD5(JL,JK))  
          ZCLOUD5(JL,JK,JK1) = 1.0_JPRB - ZCLEAR5(JL,JK,JK1)
          ZCLM5(JL,JK,JK1) = ZCLOUD5(JL,JK,JK1)
        ENDIF
      ENDDO
    ENDDO
  ENDDO

ENDIF

!*         3.      FLUXES FOR PARTIAL/MULTIPLE LAYERED CLOUDINESS
!                  ----------------------------------------------

!300  CONTINUE

!AI5 = _ZERO_
DO JL = KIDIA,KFDIA
  P_AIC5(JL,KLEV+1) = 0.0_JPRB
ENDDO

!*         3.1     DOWNWARD FLUXES
!                  ---------------

!310  CONTINUE

DO JK1 = 1, KLEV 

!*                 CONTRIBUTION FROM CLEAR-SKY FRACTION

  DO JL = KIDIA,KFDIA
!    AI5(JL,2,1,JK1) = (_ONE_ - ZCLM5(JL,JK1,KLEV))
    P_AIC5(JL,JK1) = (1.0_JPRB - ZCLM5(JL,KLEV,JK1))

!*                 CONTRIBUTION FROM ADJACENT CLOUD

    P_AI5(JL,JK1+1,JK1,1) = ZCLM5(JL,JK1,JK1)
  ENDDO

!*                 CONTRIBUTION FROM OTHER CLOUDY FRACTIONS

  DO JK = JK1, KLEV-1 
    DO JL = KIDIA,KFDIA
      ZCFRAC5 = ZCLM5(JL,JK+1,JK1) - ZCLM5(JL,JK,JK1)
      P_AI5(JL,JK+2,JK1,1) =  ZCFRAC5
    ENDDO
  ENDDO

ENDDO

!*         3.2     UPWARD FLUX AT THE SURFACE
!                  --------------------------

DO JL = KIDIA,KFDIA
  P_AI5(JL,1,1,1) = 1.0_JPRB
ENDDO

!320  CONTINUE

!*         3.3     UPWARD FLUXES
!                  -------------

!330  CONTINUE

DO JK1 = 2 , KLEV+1

!*                 CONTRIBUTION FROM CLEAR-SKY FRACTION

  DO JL = KIDIA,KFDIA
    P_AI5(JL,1,JK1,1) = (1.0_JPRB - ZCLM5(JL,1,JK1))
  ENDDO

!*                 CONTRIBUTION FROM OTHER CLOUDY FRACTIONS

  DO JK = 2 , JK1-1
    DO JL = KIDIA,KFDIA
      ZCFRAC5 = ZCLM5(JL,JK-1,JK1) - ZCLM5(JL,JK,JK1)
      P_AI5(JL,JK,JK1,1) = ZCFRAC5
    ENDDO
  ENDDO

!*                 CONTRIBUTION FROM ADJACENT CLOUD

  DO JL = KIDIA,KFDIA
    P_AI5(JL,JK1,JK1,1) =  ZCLM5(JL,JK1-1,JK1)
  ENDDO

ENDDO

!     ------------------------------------------------------------------

!                 ----- ADJOINT COMPUTATION ------

!     ------------------------------------------------------------------

!*                        INITIALIZATIONS
!                         ---------------

ZCLOUD(:) = 0.0_JPRB
ZCLEAR(:) = 0.0_JPRB
ZSUM(:)   = 0.0_JPRB
ZEMIS(:,:)  = 0.0_JPRB
ZCLM(:,:,:) = 0.0_JPRB
ZCLFR(:,:) = 0.0_JPRB
ZTMP1(:,:) = 0.0_JPRB
ZTMP3(:,:) = 0.0_JPRB
ZTMP4(:,:) = 0.0_JPRB
ZTMP5(:,:) = 0.0_JPRB

!*         3.3     UPWARD FLUXES
!                  -------------

!1330  CONTINUE

DO JK1 = KLEV+1, 2, -1

!*                 CONTRIBUTION FROM ADJACENT CLOUD

  DO JL = KIDIA,KFDIA
    ZCLM(JL,JK1-1,JK1) = ZCLM(JL,JK1-1,JK1) + P_AI (JL,JK1,JK1,1)
    P_AI (JL,JK1,JK1,1) = 0.0_JPRB
  ENDDO

!*                 CONTRIBUTION FROM OTHER CLOUDY FRACTIONS

  DO JK = JK1-1, 2, -1
    DO JL = KIDIA,KFDIA
      ZCFRAC = 0.0_JPRB
      
      ZCFRAC = ZCFRAC + P_AI (JL,JK,JK1,1)
      P_AI (JL,JK,JK1,1) = 0.0_JPRB
      
      ZCLM(JL,JK-1,JK1) = ZCLM(JL,JK-1,JK1) + ZCFRAC
      ZCLM(JL,JK,JK1)   = ZCLM(JL,JK,JK1) - ZCFRAC
    ENDDO
  ENDDO
    
!*                 CONTRIBUTION FROM CLEAR-SKY FRACTION

  DO JL = KIDIA,KFDIA
    ZCLM(JL,1,JK1) = ZCLM(JL,1,JK1) - P_AI (JL,1,JK1,1)
    P_AI (JL,1,JK1,1) = 0.0_JPRB
  ENDDO

ENDDO

!*         3.      FLUXES FOR PARTIAL/MULTIPLE LAYERED CLOUDINESS
!                  ----------------------------------------------

!*         3.2     UPWARD FLUX AT THE SURFACE
!                  --------------------------

!320  CONTINUE

DO JL = KIDIA,KFDIA
  P_AI (JL,1,1,1) = 0.0_JPRB
ENDDO

!*         3.1     DOWNWARD FLUXES
!                  ---------------

!310  CONTINUE

DO JK1 = KLEV, 1, -1

!*                 CONTRIBUTION FROM OTHER CLOUDY FRACTIONS

  DO JK = KLEV-1, JK1, -1
    DO JL = KIDIA,KFDIA
      ZCFRAC = 0.0_JPRB
      
      ZCFRAC = ZCFRAC + P_AI (JL,JK+2,JK1,1)
      P_AI (JL,JK+2,JK1,1) = 0.0_JPRB
      
      ZCLM(JL,JK+1,JK1) = ZCLM(JL,JK+1,JK1) + ZCFRAC
      ZCLM(JL,JK,JK1) = ZCLM(JL,JK,JK1) - ZCFRAC
    ENDDO
  ENDDO
  
!*                 CONTRIBUTION FROM ADJACENT CLOUD

  DO JL = KIDIA,KFDIA
    ZCLM(JL,JK1,JK1) = ZCLM(JL,JK1,JK1) + P_AI (JL,JK1+1,JK1,1)
    P_AI (JL,JK1+1,JK1,1) = 0.0_JPRB

!*                 CONTRIBUTION FROM CLEAR-SKY FRACTION

    ZCLM(JL,KLEV,JK1) = ZCLM(JL,KLEV,JK1) - P_AIC (JL,JK1)
    P_AIC (JL,JK1) = 0.0_JPRB
  ENDDO    
  
ENDDO
      
!300  CONTINUE

DO JL = KIDIA,KFDIA
  P_AIC(JL,KLEV+1)  = 0.0_JPRB
ENDDO

!*         2.5     CLOUD COVER ABOVE THE LEVEL OF CALCULATION
!                  ------------------------------------------

!250  CONTINUE

IF (LRAISANEN) THEN
!* maximum-random (Raisanen 1998)

  DO JK1 = KLEV, 1, -1
    DO JK = KLEV, JK1+1, -1
      IKL = KLEV + 1 - JK
      DO JL = KIDIA,KFDIA

        ZTR1 = 0.0_JPRB
        ZTR2 = 0.0_JPRB

        ZSUM(JL) = ZSUM(JL)+ZCLM(JL,JK,JK1)
        ZCLM(JL,JK,JK1) = 0.0_JPRB
        ZCLOUD(JL) = ZCLOUD(JL)+ZSUM(JL)

        ZEMIS(JL,JK) = ZEMIS(JL,JK) &
         & + (ZTR15(JL,JK,JK1)*ZTMPA45(JL,JK) &
         & + ZTR25(JL,JK,JK1)*ZTMPA55(JL,JK)) &
         & *ZCLOUD(JL)  
        ZTR1 = ZTR1 + ZEMISA5(JL,JK)*ZTMPA45(JL,JK)*ZCLOUD(JL)
        ZTMP4(JL,JK) = ZTMP4(JL,JK)+ZEMISA5(JL,JK)*ZTR15(JL,JK,JK1) &
         & * ZCLOUD(JL)  
        ZTR2 = ZTR2 + ZEMISA5(JL,JK)*ZTMPA55(JL,JK)*ZCLOUD(JL)
        ZTMP5(JL,JK) = ZTMP5(JL,JK)+ZEMISA5(JL,JK)*ZTR25(JL,JK,JK1) &
         & * ZCLOUD(JL)  
        ZCLOUD(JL)=0.0_JPRB

        ZSUM(JL) = ZSUM(JL) - ZTMPA35(JL,JK)*ZTR2
        ZCLOUD(JL) = ZCLOUD(JL) + ZTMPA35(JL,JK)*ZTR2
        ZTR1 = ZTR1 - ZCLFR5(JL,IKL+1)*ZTMPA35(JL,JK)*ZTR2 
        ZCLFR(JL,IKL+1) = ZCLFR(JL,IKL+1) &
         & - ZTR15(JL,JK,JK1)*ZTMPA35(JL,JK)*ZTR2  
        ZTMP3(JL,JK) = ZTMP3(JL,JK) &
         & + (1.0_JPRB-ZSUMA5(JL,JK-1,JK1)+ZCLOUDA5(JL,JK-1,JK1) &
         & - ZTR15(JL,JK,JK1)*ZCLFR5(JL,IKL+1))*ZTR2  
        
        ZCLOUD(JL) = ZCLOUD(JL) + ZTMPA15(JL,JK)*ZTR1
        ZTMP1(JL,JK) = ZTMP1(JL,JK)+ZCLOUDA5(JL,JK-1,JK1)*ZTR1
      ENDDO
    ENDDO

    DO JL = KIDIA,KFDIA
      ZSUM(JL) = ZSUM(JL) + ZCLM (JL,JK1,JK1)
      ZCLM (JL,JK1,JK1) = 0.0_JPRB
      ZCLOUD(JL) = ZCLOUD(JL) + ZSUM (JL)
      ZSUM (JL) = 0.0_JPRB
      PCLDLD(JL,JK1) = PCLDLD(JL,JK1) + ZCLOUD (JL)
      ZCLOUD (JL) = 0.0_JPRB
    ENDDO
  ENDDO

  DO JK = KLEV,2,-1
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZCADJ = 0.0_JPRB

      ZCLFR(JL,IKL) = ZCLFR(JL,IKL)+ZTMP5(JL,JK)
      ZCADJ = ZCADJ-ZTMP5(JL,JK)
      ZTMP5(JL,JK) = 0.0_JPRB

      ZCADJ = ZCADJ+(1.0_JPRB-ZEMISA5(JL,JK-1))*ZTMP4(JL,JK)
      ZEMIS(JL,JK-1) = ZEMIS(JL,JK-1)-ZCADJA5(JL,JK)*ZTMP4(JL,JK)
      ZTMP4(JL,JK) = 0.0_JPRB

      ZCLFR(JL,IKL+1) = ZCLFR(JL,IKL+1) &
       & + ZTMP3(JL,JK)/(1.0_JPRB-ZCLFR5(JL,IKL+1))**2  
      ZTMP3(JL,JK) = 0.0_JPRB

      IF (ZCLFR5(JL,IKL) <= ZCLFR5(JL,IKL+1)) THEN
        ZCLFR(JL,IKL) = ZCLFR(JL,IKL)+ZCADJ
      ELSE
        ZCLFR(JL,IKL+1) = ZCLFR(JL,IKL+1)+ZCADJ
      ENDIF
      IF (PCLDLD5(JL,JK-1) > REPCLC) THEN
        PCLDLD(JL,JK-1) = PCLDLD(JL,JK-1) &
         & - ZTMP1(JL,JK)/PCLDLD5(JL,JK-1)**2  
      ENDIF
      ZTMP1(JL,JK) = 0.0_JPRB
    ENDDO
  ENDDO

  DO JK = KLEV, 1, -1
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      PCLDLD(JL,JK) = PCLDLD(JL,JK) + ZEMIS(JL,JK)/ZCLFR5(JL,IKL)
      ZCLFR(JL,IKL) = ZCLFR(JL,IKL) &
       & - PCLDLD5(JL,JK)*ZEMIS(JL,JK)/(ZCLFR5(JL,IKL)**2)  
      ZEMIS(JL,JK) = 0.0_JPRB
    ENDDO
  ENDDO

  ZEMIS(:,:)  = 0.0_JPRB

ELSE

  DO JK1 = KLEV, 1, -1
  
    DO JK = KLEV, JK1, -1
      DO JL = KIDIA,KFDIA
        ZDEN = 0.0_JPRB
        ZNUM = 0.0_JPRB
      
        IF (NOVLP == 1) THEN
!* maximum-random
          PCLDLD(JL,JK) = PCLDLD(JL,JK) + ZCLOUD (JL)
          ZCLOUD (JL) = 0.0_JPRB
          ZCLEAR(JL) = ZCLEAR(JL) - ZCLM (JL,JK,JK1)
          ZCLM (JL,JK,JK1) = 0.0_JPRB
          ZNUM = ZNUM - ZCLEAR(JL)*ZCLEARA5(JL,JK,JK1) &
           & /(1.0_JPRB-ZDEN5(JL,JK,JK1))  
          ZDEN = ZDEN + ZCLEAR(JL)*ZCLEARA5(JL,JK,JK1) &
           & *(1.0_JPRB-ZNUM5(JL,JK,JK1))/((1.0_JPRB-ZDEN5(JL,JK,JK1))**2)  
          ZCLEAR(JL)=ZCLEAR(JL)*(1.0_JPRB- ZNUM5(JL,JK,JK1)) &
           & /(1.0_JPRB-ZDEN5(JL,JK,JK1))  
        
          IF (ZCLOUDA5(JL,JK,JK1) <= 1.0_JPRB-REPCLC) THEN  
            ZCLOUD(JL) = ZCLOUD(JL) + ZDEN
          ENDIF
          IF (PCLDLD5(JL,JK) >= ZCLOUDA5(JL,JK,JK1)) THEN
            PCLDLD (JL,JK) = PCLDLD (JL,JK) + ZNUM
          ELSE
            ZCLOUD(JL) = ZCLOUD(JL) + ZNUM
          ENDIF

        ELSEIF (NOVLP == 2) THEN
!* maximum
          ZCLOUD(JL) = ZCLOUD(JL) + ZCLM (JL,JK,JK1)
          ZCLM (JL,JK,JK1) = 0.0_JPRB
          IF (PCLDLD5(JL,JK) >= ZCLOUDA5(JL,JK,JK1)) THEN
            PCLDLD(JL,JK) = PCLDLD(JL,JK) + ZCLOUD (JL)
            ZCLOUD (JL) = 0.0_JPRB
          ENDIF

        ELSEIF (NOVLP == 3) THEN
!* random
          ZCLOUD(JL) = ZCLOUD(JL) + ZCLM (JL,JK,JK1)
          ZCLM (JL,JK,JK1) = 0.0_JPRB
          ZCLEAR(JL) = ZCLEAR(JL) - ZCLOUD (JL)
          ZCLOUD (JL) = 0.0_JPRB
          PCLDLD(JL,JK) = PCLDLD(JL,JK) - ZCLEAR(JL) &
           & * ZCLEARA5(JL,JK,JK1)  
          ZCLEAR(JL)  = ZCLEAR(JL)*(1.0_JPRB - PCLDLD5(JL,JK))
        ENDIF
      ENDDO
    ENDDO
  
    DO JL = KIDIA,KFDIA
      ZCLOUD(JL) =0.0_JPRB
      ZCLEAR(JL) =0.0_JPRB
    ENDDO
  ENDDO

ENDIF

!*         2.4     CLOUD COVER BELOW THE LEVEL OF CALCULATION
!                  ------------------------------------------

!240  CONTINUE

IF (LRAISANEN) THEN

!* maximum-random (Raisanen 1998)

  DO JK1 = KLEV+1, 2, -1
    DO JK = 1, JK1 - 2
      IKL = KLEV+1-JK
      DO JL = KIDIA,KFDIA

        ZTR1 = 0.0_JPRB
        ZTR2 = 0.0_JPRB

        ZSUM(JL) = ZSUM(JL) + ZCLM(JL,JK,JK1)
        ZCLM(JL,JK,JK1) = 0.0_JPRB
        ZCLOUD(JL) = ZCLOUD(JL) + ZSUM(JL)

        ZEMIS(JL,JK) = ZEMIS(JL,JK) &
         & + (ZTR15(JL,JK,JK1)*ZTMPB45(JL,JK) &
         & + ZTR25(JL,JK,JK1)*ZTMPB55(JL,JK))*ZCLOUD(JL)  
        ZTR1 = ZTR1 + ZEMISB5(JL,JK)*ZTMPB45(JL,JK)*ZCLOUD(JL)
        ZTMP4(JL,JK) = ZTMP4(JL,JK) &
         & + ZEMISB5(JL,JK)*ZTR15(JL,JK,JK1)*ZCLOUD(JL)  
        ZTR2 = ZTR2 + ZEMISB5(JL,JK)*ZTMPB55(JL,JK)*ZCLOUD(JL)
        ZTMP5(JL,JK) = ZTMP5(JL,JK) &
         & + ZEMISB5(JL,JK)*ZTR25(JL,JK,JK1)*ZCLOUD(JL)  
        ZCLOUD(JL) = 0.0_JPRB

        ZSUM(JL) = ZSUM(JL) - ZTMPB35(JL,JK)*ZTR2
        ZCLOUD(JL) = ZCLOUD(JL) + ZTMPB35(JL,JK)*ZTR2
        ZTR1 = ZTR1 - ZTMPB35(JL,JK)*ZCLFR5(JL,IKL-1)*ZTR2
        ZCLFR(JL,IKL-1) = ZCLFR(JL,IKL-1) &
         & - ZTR15(JL,JK,JK1)*ZTMPB35(JL,JK)*ZTR2  
        ZTMP3(JL,JK) = ZTMP3(JL,JK)+ (1.0_JPRB -ZSUMB5(JL,JK+1,JK1) &
         & + ZCLOUD5(JL,JK+1,JK1) &
         & - ZTR15(JL,JK,JK1)*ZCLFR5(JL,IKL-1))*ZTR2  

        ZCLOUD(JL) = ZCLOUD(JL) + ZTMPB15(JL,JK)*ZTR1
        ZTMP1(JL,JK) = ZTMP1(JL,JK)+ZCLOUD5(JL,JK+1,JK1)*ZTR1
      ENDDO
    ENDDO

    DO JL = KIDIA,KFDIA
      ZSUM(JL) = ZSUM(JL) + ZCLM (JL,JK1-1,JK1)
      ZCLM (JL,JK1-1,JK1) = 0.0_JPRB
      ZCLOUD(JL) = ZCLOUD(JL) + ZSUM (JL)
      ZSUM (JL) = 0.0_JPRB
      PCLDLU(JL,JK1-1) = PCLDLU(JL,JK1-1) + ZCLOUD (JL)
      ZCLOUD (JL) = 0.0_JPRB
    ENDDO
  ENDDO

  DO JK = KLEV-1,1,-1
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZCADJ = 0.0_JPRB
      
      ZCLFR(JL,IKL) = ZCLFR(JL,IKL)+ZTMP5(JL,JK)
      ZCADJ = ZCADJ-ZTMP5(JL,JK)
      ZTMP5(JL,JK) = 0.0_JPRB

      ZCADJ = ZCADJ+(1.0_JPRB-ZEMISB5(JL,JK+1))*ZTMP4 (JL,JK)
      ZEMIS(JL,JK+1) = ZEMIS(JL,JK+1)-ZCADJB5(JL,JK)*ZTMP4 (JL,JK)
      ZTMP4 (JL,JK) = 0.0_JPRB

      ZCLFR(JL,IKL-1) = ZCLFR(JL,IKL-1) &
       & + ZTMP3(JL,JK)/(1.0_JPRB-ZCLFR5(JL,IKL-1))**2  
      ZTMP3(JL,JK) = 0.0_JPRB

      IF (ZCLFR5(JL,IKL) <= ZCLFR5(JL,IKL-1)) THEN
        ZCLFR(JL,IKL) = ZCLFR(JL,IKL)+ZCADJ
      ELSE
        ZCLFR(JL,IKL-1) = ZCLFR(JL,IKL-1)+ZCADJ
      ENDIF

      IF (PCLDLU5(JL,JK+1) > REPCLC) THEN
        PCLDLU(JL,JK+1) = PCLDLU(JL,JK+1) &
         & - ZTMP1 (JL,JK)/PCLDLU5(JL,JK+1)**2  
      ENDIF
      ZTMP1(JL,JK) = 0.0_JPRB
    ENDDO
  ENDDO

  DO JK = KLEV, 1, -1
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      PCLDLU(JL,JK) = PCLDLU(JL,JK) + ZEMIS(JL,JK)/ZCLFR5(JL,IKL)
      ZCLFR(JL,IKL) = ZCLFR(JL,IKL) &
       & - PCLDLU5(JL,JK)*ZEMIS(JL,JK)/(ZCLFR5(JL,IKL)**2)  
      ZEMIS(JL,JK) = 0.0_JPRB
      
      IF (ZCLFR5(JL,IKL) >= 1.0_JPRB-REPCLC) THEN
        ZCLFR (JL,IKL) = 0.0_JPRB
      ENDIF
      IF (ZCLFR5(JL,IKL) <= REPCLC) THEN
        ZCLFR (JL,IKL) = 0.0_JPRB
      ENDIF
      PCLFR (JL,IKL) = PCLFR (JL,IKL) + ZCLFR (JL,IKL)
      ZCLFR (JL,IKL) = 0.0_JPRB      
    ENDDO
  ENDDO

  ZEMIS(:,:)  = 0.0_JPRB

ELSE

  DO JK1 = KLEV+1, 2, -1

    DO JL = KIDIA,KFDIA
      ZCLEAR5(JL,JK1,JK1)= 1.
      ZCLOUD5(JL,JK1,JK1)= 0.
    ENDDO

    DO JK = 1, JK1 - 1
      DO JL = KIDIA,KFDIA
        ZDEN = 0.0_JPRB
        ZNUM = 0.0_JPRB
      
        IF (NOVLP == 1) THEN
!* maximum-random
          PCLDLU(JL,JK) = PCLDLU(JL,JK) + ZCLOUD (JL)
          ZCLOUD (JL) = 0.0_JPRB
          ZCLEAR(JL) = ZCLEAR(JL) - ZCLM (JL,JK,JK1)
          ZCLM (JL,JK,JK1) = 0.0_JPRB
          ZNUM = ZNUM - ZCLEAR(JL)*ZCLEAR5(JL,JK+1,JK1) &
           & /(1.0_JPRB-ZDEN5(JL,JK,JK1))  
          ZDEN = ZDEN + ZCLEAR(JL)*ZCLEAR5(JL,JK+1,JK1) &
           & *(1.0_JPRB-ZNUM5(JL,JK,JK1))/((1.0_JPRB-ZDEN5(JL,JK,JK1))**2)  
          ZCLEAR(JL)=ZCLEAR(JL)*(1.0_JPRB-ZNUM5(JL,JK,JK1)) &
           & /(1.0_JPRB-ZDEN5(JL,JK,JK1))  
       
          IF (ZCLOUD5(JL,JK+1,JK1) <= 1.0_JPRB-REPCLC) THEN
            ZCLOUD(JL) = ZCLOUD(JL) + ZDEN
          ENDIF
          IF (PCLDLU5(JL,JK) >= ZCLOUD5(JL,JK+1,JK1)) THEN
            PCLDLU (JL,JK) = PCLDLU (JL,JK) + ZNUM
          ELSE
            ZCLOUD(JL) = ZCLOUD(JL) + ZNUM
          ENDIF
        
        ELSEIF (NOVLP == 2) THEN
!* maximum
          ZCLOUD(JL) = ZCLOUD(JL) + ZCLM (JL,JK,JK1)
          ZCLM (JL,JK,JK1) = 0.0_JPRB
          IF (PCLDLU5(JL,JK) >= ZCLOUD5(JL,JK+1,JK1)) THEN
            PCLDLU(JL,JK) =  PCLDLU(JL,JK) + ZCLOUD (JL)
            ZCLOUD (JL) = 0.0_JPRB
          ENDIF
        
        ELSEIF (NOVLP == 3) THEN
!* random
          ZCLOUD(JL) = ZCLOUD(JL) + ZCLM (JL,JK,JK1)
          ZCLM (JL,JK,JK1) = 0.0_JPRB
          ZCLEAR(JL) = ZCLEAR(JL) - ZCLOUD (JL)
          ZCLOUD (JL) = 0.0_JPRB
          PCLDLU(JL,JK) = PCLDLU(JL,JK) - ZCLEAR(JL) &
           & *ZCLEAR5(JL,JK+1,JK1)  
          ZCLEAR(JL)  = ZCLEAR(JL)*(1.0_JPRB - PCLDLU5(JL,JK))
        ENDIF
      ENDDO
    ENDDO
  
    DO JL = KIDIA,KFDIA
      ZCLOUD(JL) =0.0_JPRB
      ZCLEAR(JL) =0.0_JPRB
    ENDDO 
  ENDDO

ENDIF

!*         2.      EFFECT OF CLOUDINESS ON LONGWAVE FLUXES
!                  ---------------------------------------

!*         2.2     CLOUD COVER MATRIX
!                  ------------------

!220  CONTINUE

!ZCLM (:,:,:) = _ZERO_

!     ------------------------------------------------------------------

!*         1.     INITIALIZATION
!                 --------------

!100  CONTINUE

ZCLOUD(:)  = 0.0_JPRB
        
!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWAIAD',1,ZHOOK_HANDLE)
END SUBROUTINE LWAIAD

