SUBROUTINE LWBV &
 & ( KIDIA, KFDIA, KLON , KLEV  , KMODE, KTRAER, &
 & PDT0 , PEMIS, PEMIW,&
 & PTL  , PTAVE,&
 & PEMIT, PFLUC,&
 & PABCU, PBINT, PBSUI, PCNTRB, &
! for adjoint computation
 & PB, PBSUR, PBTOP, PDBSL, PGA, PGB, &
 & PGASUR, PGBSUR, PGATOP, PGBTOP, &
 & PADJD, PADJU, PDBDT, PDISD, PDISU, PDWFSU, &
 & PCOND  , PTTPA  , PTTPB , PTTA , PTTB  ,&
 & PZZA   , PZZB   , PXNA  , PXNB , PXDIVA,&
 & PXDIVB , PZZA1  , PZZB1 , PXNA1, PXNB1 ,&
 & PXDIVA1, PXDIVB1, PTTPA1, PTTPB1 &
 & )  

!**** *LWBV*   - COMPUTE PLANCK FUNC., PERF. VERT. INTEGRATION

!     PURPOSE.
!     --------
!           TO COMPUTE THE PLANCK FUNCTION AND PERFORM THE
!           VERTICAL INTEGRATION. SPLIT OUT FROM LW FOR MEMORY
!           SAVING

!**   INTERFACE.
!     ----------

!        *LWVB* IS CALLED FROM *LW*

!        EXPLICIT ARGUMENTS :
!        --------------------
! PDT0   : (KLON)            ; SURFACE TEMPERATURE DISCONTINUITY
! PEMIS  : (KLON)            ; SURFACE LW EMISSIVITY
! PEMIW  : (KLON)            ; SURFACE LW WINDOW EMISSIVITY
! PTAVE  : (KLON,KLEV)       ; TEMPERATURE
! PTL    : (KLON,KLEV+1)     ; HALF LEVEL TEMPERATURE
!     ==== OUTPUTS ===
! PABCU  :
! PBINT  :
! PBSUI  :
! PCNTRB : 
! PCOLC  :
! PEMIT  :
! PFLUC  :
!     ==== OUTPUTS FOR ADJOINT COMPUTATION ===
! PB     : (KLON,NSIL,KLEV+1); SPECTRAL HALF-LEVEL PLANCK FUNCTIONS
! PBSUR  : (KLON,NSIL)       ; SPECTRAL SURFACE PLANCK FUNCTION
! PBTOP  : (KLON,NSIL)       ; SPECTRAL T.O.A. PLANCK FUNCTION
! PDBSL  : (KLON,KLEV*2)     ; SUB-LAYER PLANCK FUNCTION GRADIENT
! PGA, PGB                   ; PADE APPROXIMANTS
! PGASUR, PGBSUR             ; SURFACE PADE APPROXIMANTS
! PGATOP, PGBTOP             ; T.O.A. PADE APPROXIMANTS
! PADJ.. : (KLON,KLEV+1)     ; CONTRIBUTION BY ADJACENT LAYERS
! PDBDT  : (KLON,NUA,KLEV)   ; LAYER PLANCK FUNCTION GRADIENT
! PDIS.. : (KLON,KLEV+1)     ; CONTRIBUTION BY DISTANT LAYERS


!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. COMPUTES THE PLANCK FUNCTIONS ON THE INTERFACES AND THE
!     GRADIENT OF PLANCK FUNCTIONS IN THE LAYERS.
!          2. PERFORMS THE VERTICAL INTEGRATION DISTINGUISHING THE CON-
!     TRIBUTIONS OF THE ADJACENT AND DISTANT LAYERS AND THOSE FROM THE
!     BOUNDARIES.
!          3. COMPUTES THE CLEAR-SKY COOLING RATES.

!     EXTERNALS.
!     ----------

!          *LWB*, *LWV*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 89-07-14
!        MODIFICATION : 93-10-15 M.HAMRUD (SPLIT OUT FROM LW TO SAVE
!                                          MEMORY)
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Janiskova   22-Nov-2006 changed number of absorbers and
!                                  transmission functions for lin.phys.
!        M. Janiskova   4-Apr-2007 additional arguments passed from
!                                  the routine for AD computation
!        M. Janiskova  19-May-2008 using shorter NPROMALW instead of KLON
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOELW    , ONLY : NSIL     ,NIPD     ,NUA
USE YOERDU   , ONLY : NUAER    ,NTRAER
USE YOEPHLI  , ONLY : LPHYLIN
USE YOPHNC   , ONLY : LH2OCO2
USE YOELWRAD , ONLY : NPROMALW

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMODE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAER
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDT0(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIS(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIW(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAVE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PEMIT(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLUC(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBINT(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBSUI(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCNTRB(KLON,KLEV+1,KLEV+1) 
! for adjoint computation
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PB(KLON,NSIL,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBSUR(KLON,NSIL)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PBTOP(KLON,NSIL)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDBSL(KLON,NSIL,KLEV*2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGA(KLON,NIPD,2,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGB(KLON,NIPD,2,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGASUR(KLON,NIPD,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGBSUR(KLON,NIPD,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGATOP(KLON,NIPD,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGBTOP(KLON,NIPD,2)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PADJD(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PADJU(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDBDT(KLON,NSIL,KLEV)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDISD(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDISU(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDWFSU(KLON,NSIL)
! for adjoint computation - from lwvdr
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCOND(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTTPA(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTTPB(NPROMALW,KLEV+1,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTTA(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTTB(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZZA(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZZB(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXNA(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXNB(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXDIVA(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXDIVB(NPROMALW,KTRAER*KLEV,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZZA1(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PZZB1(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXNA1(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXNB1(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXDIVA1(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXDIVB1(NPROMALW,KTRAER,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTTPA1(KLON,KLEV+1)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTTPB1(KLON,KLEV+1)

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-------------------------------------------------------------------------

!              ------------
INTEGER(KIND=JPIM) :: JL, JLW
INTEGER(KIND=JPIM) :: IUAER, ITRAER
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "lwb.intfb.h"
#include "lwv.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWBV',0,ZHOOK_HANDLE)

!*         1.    INITIALIZATION
!                --------------

IF (LPHYLIN .AND. LH2OCO2) THEN
  IUAER  = 9
  ITRAER = 9
ELSE
  IUAER  = NUAER
  ITRAER = NTRAER
ENDIF

!*         2.    COMPUTES PLANCK FUNCTIONS
!                -------------------------

CALL LWB &
 & ( KIDIA, KFDIA, KLON  , KLEV  , KMODE,&
 & PDT0 , PTAVE, PTL,&
 & PB   , PBINT, PBSUR , PBTOP , PDBSL,&
 & PGA  , PGB  , PGASUR, PGBSUR, PGATOP, PGBTOP    &
 & )  

!     ------------------------------------------------------------------

!*         3.    PERFORMS THE VERTICAL INTEGRATION
!                ---------------------------------

CALL LWV &
 & ( KIDIA , KFDIA, KLON  , KLEV,&
! & NUAER , NTRAER,&
 & IUAER , ITRAER,&
 & PABCU , PB   , PBINT , PBSUR , PBTOP , PDBSL,&
 & PEMIS , PEMIW,&
 & PGA   , PGB  , PGASUR, PGBSUR, PGATOP, PGBTOP,&
 & PCNTRB, PFLUC, &
! for adjoint computation
 & PADJD, PADJU, PDBDT, PDISD, PDISU, PDWFSU, &
 & PCOND  , PTTPA  , PTTPB , PTTA , PTTB  ,&
 & PZZA   , PZZB   , PXNA  , PXNB , PXDIVA,&
 & PXDIVB , PZZA1  , PZZB1 , PXNA1, PXNB1 ,&
 & PXDIVA1, PXDIVB1, PTTPA1, PTTPB1 &
 & )  

DO JL=KIDIA,KFDIA
  PEMIT(JL)=0.0_JPRB
  PBSUI(JL)=0.0_JPRB
ENDDO
DO JLW=1,NSIL
  DO JL=KIDIA,KFDIA
    PBSUI(JL)=PBSUI(JL)+PBSUR(JL,JLW)
    IF (JLW >= 3.AND. JLW <= 4) THEN
      PEMIT(JL)=PEMIT(JL)+PBSUR(JL,JLW)*PEMIW(JL)
    ELSE
      PEMIT(JL)=PEMIT(JL)+PBSUR(JL,JLW)*PEMIS(JL)
    ENDIF
  ENDDO
ENDDO
DO JL=KIDIA,KFDIA
  PEMIT(JL)=PEMIT(JL)/PBSUI(JL)
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWBV',1,ZHOOK_HANDLE)
END SUBROUTINE LWBV
