SUBROUTINE LWCTL &
 & ( KIDIA , KFDIA, KLON  , KLEV, KTOPC,&
 & PBINT5 , PBSUI5, PCLFR5, PCLDLD5, PCLDLU5,&
 & PCNTRB5, PEMIT5, PFLUC5,&
 & PFLUX5 ,&
 & PBINT  , PBSUI , PCLFR , PCLDLD , PCLDLU ,&
 & PCNTRB , PEMIT , PFLUC ,&
 & PFLUX  &
 & )

!**** *LWCTL*   - LONGWAVE RADIATION, CLOUD EFFECTS
!                      (tangent linear)

!     PURPOSE.
!     --------
!           INTRODUCES CLOUD EFFECTS ON LONGWAVE FLUXES OR
!           RADIANCES

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!  Trajectory  Perturbation  Description                         Dimension
!  PBINT5      PBINT         HALF LEVEL PLANCK FUNCTION          (KLON,KLEV+1)
!  PBSUI5      PBSUI         SURFACE PLANCK FUNCTION             (KLON)
!  PCLFR5      PCLFR         CLOUD FRACTIONAL COVER              (KLON,KLEV)
!  PCLDLD5     PCLDLD        DOWNWARD EFFECTIVE CLOUD FRACTION   (KLON,KLEV)
!  PCLDLU5     PCLDLU        UPWARD EFFECTIVE CLOUD FRACTION     (KLON,KLEV)
!  PCNTRB5     PCNTRB        CLEAR-SKY ENERGY EXCHANGE 
!                            MATRIX                        (KLON,KLEV+1,KLEV+1)
!  PEMIT5      PEMIT         SURFACE TOTAL LW EMISSIVITY         (KLON)
!  PFLUC5      PFLUC         CLEAR-SKY LW RADIATIVE FLUXES      (KLON,2,KLEV+1)

!     ==== OUTPUTS ===
!  Trajectory  Perturbation  Description                         Dimension
!  PFLUX5      PFLUX         TOTAL SKY LW RADIATIVE FLUXES      (KLON,2,KLEV+1)
!                     1  ==>  UPWARD   FLUX TOTAL
!                     2  ==>  DOWNWARD FLUX TOTAL

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. INITIALIZES ALL FLUXES TO CLEAR-SKY VALUES
!          2. EFFECT OF ONE OVERCAST UNITY EMISSIVITY CLOUD LAYER
!          3. EFFECT OF SEMI-TRANSPARENT, PARTIAL OR MULTI-LAYERED
!     CLOUDS

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        M. Janiskova  *ECMWF*       31-Jan-2006

!     MODIFICATIONS.
!     --------------

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOERAD   , ONLY : NOVLP
USE YOERDI   , ONLY : REPCLC
USE YOEOVLP  , ONLY : RA1OVLP
USE YOEPHLI  , ONLY : LRAISANEN

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTOPC
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBINT5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBSUI5(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR5(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLD5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLU5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCNTRB5(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIT5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLUC5(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLUX5(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBINT(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBSUI(KLON)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLFR(KLON,KLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLD(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLDLU(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCNTRB(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEMIT(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLUC(KLON,2,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLUX(KLON,2,KLEV+1)

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZCLEAR(KLON), ZCLOUD(KLON)
REAL(KIND=JPRB) :: ZCLM(KLON,KLEV+1,KLEV+1), ZDNF(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) :: ZFD(KLON), ZFU(KLON), ZUPF(KLON,KLEV+1,KLEV+1)  
REAL(KIND=JPRB) :: ZCLFR(KLON,KLEV), ZEMIS(KLON,KLEV), ZSUM(KLON)
REAL(KIND=JPRB) :: ZACLFR(KLON,KLEV),ZCLDLD(KLON,KLEV),ZCLDLU(KLON,KLEV)
REAL(KIND=JPRB) :: ZTMP1(KLON,KLEV+1), ZTMP3(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTMP4(KLON,KLEV+1), ZTMP5(KLON,KLEV+1)

REAL(KIND=JPRB) :: ZCLEAR5(KLON), ZCLOUD5(KLON)
REAL(KIND=JPRB) :: ZCLM5(KLON,KLEV+1,KLEV+1), ZDNF5(KLON,KLEV+1,KLEV+1)
REAL(KIND=JPRB) :: ZFD5(KLON), ZFU5(KLON), ZUPF5(KLON,KLEV+1,KLEV+1)  
REAL(KIND=JPRB) :: ZCLFR5(KLON,KLEV), ZEMIS5(KLON,KLEV), ZSUM5(KLON)
REAL(KIND=JPRB) :: ZACLFR5(KLON,KLEV),ZCLDLD5(KLON,KLEV),ZCLDLU5(KLON,KLEV)
REAL(KIND=JPRB) :: ZTMP15(KLON,KLEV+1), ZTMP35(KLON,KLEV+1)
REAL(KIND=JPRB) :: ZTMP45(KLON,KLEV+1), ZTMP55(KLON,KLEV+1)

INTEGER(KIND=JPIM) :: IKCP1, IKM1, IKP1, IMAXC, JCLOUD,&
 & JK, JK1, JK2, JKJ, JL, IKL, IJK 

REAL(KIND=JPRB) :: ZALPHA1, ZCFRAC, ZNUM, ZDEN
REAL(KIND=JPRB) :: ZTR1, ZTR2, ZCADJ
REAL(KIND=JPRB) :: ZCFRAC5, ZNUM5, ZDEN5
REAL(KIND=JPRB) :: ZTR15, ZTR25, ZCADJ5
REAL(KIND=JPRB) :: ZDIV15, ZDIV25, ZDIV35, ZDIV45, ZDIV55, ZDIV65

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*         1.     INITIALIZATION
!                 --------------

!100  CONTINUE

!      print *,' Enter LWC '
IF (LHOOK) CALL DR_HOOK('LWCTL',0,ZHOOK_HANDLE)

DO JK = 1 , KLEV+1
  DO JL = KIDIA,KFDIA
    PFLUX (JL,1,JK) = PFLUC (JL,1,JK)
    PFLUX5(JL,1,JK) = PFLUC5(JL,1,JK)
    PFLUX (JL,2,JK) = PFLUC (JL,2,JK)
    PFLUX5(JL,2,JK) = PFLUC5(JL,2,JK)
  ENDDO
ENDDO

!GM*******
IMAXC=KLEV
!GM*******

!Experimentation with reduced ITOPC 
!ITOPC=KLEV-10
!ITOPC=KLEV-20
!ITOPC=KLEV

!     ------------------------------------------------------------------

!*         2.      EFFECT OF CLOUDINESS ON LONGWAVE FLUXES
!                  ---------------------------------------

!*         2.0     INITIALIZE TO CLEAR-SKY FLUXES
!                  ------------------------------

!200  CONTINUE

DO JK = 1,KTOPC+1
  DO JL = KIDIA,KFDIA
    ZDNF (JL,1,JK) = PFLUC (JL,2,JK)
    ZDNF5(JL,1,JK) = PFLUC5(JL,2,JK)
  ENDDO
ENDDO

DO JK = 1,KLEV+1
  DO JL = KIDIA,KFDIA
    ZUPF (JL,1,JK) = PFLUC (JL,1,JK)
    ZUPF5(JL,1,JK) = PFLUC5(JL,1,JK)
  ENDDO
ENDDO

!      print *,' LWC after Initialisation to clear-sky fluxes'

!*         2.1     FLUXES FOR ONE OVERCAST UNITY EMISSIVITY CLOUD
!                  ----------------------------------------------

!210  CONTINUE

DO JCLOUD = 1 , KTOPC
  IKCP1=JCLOUD+1

!*         2.1.1   ABOVE THE CLOUD
!                  ---------------

!2110 CONTINUE

  DO JK=IKCP1,KLEV+1
    IKM1=JK-1
    DO JL = KIDIA,KFDIA
      ZFU (JL)=0.0_JPRB
      ZFU5(JL)=0.0_JPRB
    ENDDO

    IF (JK  >  IKCP1) THEN
      DO JKJ=IKCP1,IKM1
        DO JL = KIDIA,KFDIA
          ZFU (JL) = ZFU (JL) + PCNTRB (JL,JK,JKJ)
          ZFU5(JL) = ZFU5(JL) + PCNTRB5(JL,JK,JKJ)
        ENDDO
      ENDDO
    ENDIF

    DO JL = KIDIA,KFDIA
      ZUPF (JL,IKCP1,JK) = PBINT (JL,JK)-ZFU (JL)
      ZUPF5(JL,IKCP1,JK) = PBINT5(JL,JK)-ZFU5(JL)
    ENDDO
  ENDDO

!*         2.1.2   BELOW THE CLOUD
!                  ---------------

!2120 CONTINUE

  DO JK=1,JCLOUD
    IKP1=JK+1
    DO JL = KIDIA,KFDIA
      ZFD (JL) = 0.0_JPRB
      ZFD5(JL) = 0.0_JPRB
    ENDDO

    IF (JK  <  JCLOUD) THEN
      DO JKJ=IKP1,JCLOUD
        DO JL = KIDIA,KFDIA
          ZFD (JL) = ZFD (JL) + PCNTRB (JL,JK,JKJ)
          ZFD5(JL) = ZFD5(JL) + PCNTRB5(JL,JK,JKJ)
        ENDDO
      ENDDO
    ENDIF

    DO JL = KIDIA,KFDIA
      ZDNF (JL,IKCP1,JK) = -PBINT (JL,JK)-ZFD (JL)
      ZDNF5(JL,IKCP1,JK) = -PBINT5(JL,JK)-ZFD5(JL)
    ENDDO
  ENDDO

ENDDO
!      print *,' LWC after 213: Fluxes for unity emissivity'

!*         2.2     CLOUD COVER MATRIX
!                  ------------------

!*    ZCLM(JK1,JK2) IS THE OBSCURATION FACTOR BY CLOUD LAYERS BETWEEN
!     HALF-LEVELS JK1 AND JK2 AS SEEN FROM JK1

!220  CONTINUE

! matrix notation for ZCLM is not standard, but:
!    z11  z21  z31  ...
!    z12  z22  z32  ...
!    z13  z23  z33  ...
!     :    :    :    :

!DO JK = KTOPC,KLEV+1
!  DO JL = KIDIA,KFDIA
!    ZCLM (JL,KTOPC,JK) = 0.0_JPRB
!    ZCLM5(JL,KTOPC,JK) = 0.0_JPRB
!    ZCLM (JL,KTOPC+1,JK) = 0.0_JPRB
!    ZCLM5(JL,KTOPC+1,JK) = 0.0_JPRB
!  ENDDO
!ENDDO

DO JK2 = KTOPC , KLEV+1
  DO JK1 = KTOPC , KLEV+1
    DO JL = KIDIA,KFDIA
      ZCLM (JL,JK1,JK2) = 0.0_JPRB
      ZCLM5(JL,JK1,JK2) = 0.0_JPRB
    ENDDO
  ENDDO
ENDDO

IF (.NOT.LRAISANEN) THEN
  DO JK = KTOPC+2,KLEV
    DO JL = KIDIA,KFDIA
      ZCLM (JL,JK,JK+1) = 0.0_JPRB
      ZCLM5(JL,JK,JK+1) = 0.0_JPRB
    ENDDO
  ENDDO
ENDIF

! Securities

!DO JK = 1,KLEV
!  IKL = KLEV + 1 - JK
!  DO JL = KIDIA,KFDIA
!    IF (PCLFR5(JL,IKL) <= REPCLC) THEN
!      ZCLDLD (JL,JK) = 0.0_JPRB
!      ZCLDLD5(JL,JK) = REPCLC
!      ZCLDLU (JL,JK) = 0.0_JPRB
!      ZCLDLU5(JL,JK) = REPCLC
!    ELSE
!      ZCLDLD (JL,JK) = PCLDLD (JL,JK)
!      ZCLDLD5(JL,JK) = PCLDLD5(JL,JK)
!      ZCLDLU (JL,JK) = PCLDLU (JL,JK)
!      ZCLDLU5(JL,JK) = PCLDLU5(JL,JK)
!    ENDIF
!  ENDDO
!ENDDO

IF (KTOPC /= KLEV) THEN
  DO JK = 1,KTOPC
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZACLFR (JL,IKL) = PCLFR (JL,IKL)
      ZACLFR5(JL,IKL) = PCLFR5(JL,IKL)

      ZCLDLD (JL,JK) = PCLDLD (JL,JK)
      ZCLDLD5(JL,JK) = PCLDLD5(JL,JK)
      ZCLDLU (JL,JK) = PCLDLU (JL,JK)
      ZCLDLU5(JL,JK) = PCLDLU5(JL,JK)
    ENDDO
  ENDDO
  DO JK = KTOPC+1,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZACLFR (JL,IKL) = 0.0_JPRB
      ZACLFR5(JL,IKL) = REPCLC

      ZCLDLD (JL,JK) = 0.0_JPRB
      ZCLDLD5(JL,JK) = 0.0_JPRB
      ZCLDLU (JL,JK) = 0.0_JPRB
      ZCLDLU5(JL,JK) = 0.0_JPRB
    ENDDO
  ENDDO
ELSE
  DO JK = 1,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZACLFR (JL,IKL) = PCLFR (JL,IKL)
      ZACLFR5(JL,IKL) = PCLFR5(JL,IKL)

      ZCLDLD (JL,JK) = PCLDLD (JL,JK)
      ZCLDLD5(JL,JK) = PCLDLD5(JL,JK)
      ZCLDLU (JL,JK) = PCLDLU (JL,JK)
      ZCLDLU5(JL,JK) = PCLDLU5(JL,JK)
    ENDDO
  ENDDO
ENDIF

IF (LRAISANEN) THEN
  DO JK = 1,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      IF (PCLDLU5(JL,JK) <= REPCLC .OR. PCLDLD5(JL,JK) <= REPCLC) THEN
        ZACLFR (JL,IKL) = 0.0_JPRB
        ZACLFR5(JL,IKL) = REPCLC
      ELSE
        ZACLFR (JL,IKL) = PCLFR (JL,IKL)
        ZACLFR5(JL,IKL) = ZACLFR5(JL,IKL)
      ENDIF
    ENDDO
  ENDDO  
ENDIF

!      print *,' LWC after Initialisation CC matrix'

!*         2.4     CLOUD COVER BELOW THE LEVEL OF CALCULATION
!                  ------------------------------------------

!240  CONTINUE

IF (LRAISANEN) THEN
!* maximum-random (Raisanen 1998)

  ZEMIS (:,:) = 0.0_JPRB
  ZEMIS5(:,:) = 0.0_JPRB
  DO JK = 1,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZCLFR (JL,IKL) = ZACLFR (JL,IKL)
      ZCLFR5(JL,IKL) = ZACLFR5(JL,IKL)
      IF (ZCLFR5(JL,IKL) <= REPCLC) THEN
        ZCLFR (JL,IKL) = 0.0_JPRB
        ZCLFR5(JL,IKL) = REPCLC
      ENDIF
      IF (ZCLFR5(JL,IKL) >= 1.0_JPRB-REPCLC) THEN
        ZCLFR (JL,IKL) = 0.0_JPRB
        ZCLFR5(JL,IKL) = 1.0_JPRB-REPCLC
      ENDIF
      ZDIV15 = 1.0_JPRB/ZCLFR5(JL,IKL)
      ZEMIS (JL,JK) = ZCLDLU(JL,JK)*ZDIV15 &
       & -ZCLDLU5(JL,JK)*ZDIV15*ZDIV15*ZCLFR(JL,IKL)
      ZEMIS5(JL,JK) = ZCLDLU5(JL,JK)*ZDIV15
    ENDDO
  ENDDO

  DO JK = 1,KTOPC-1
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      IF (ZCLDLU5(JL,JK+1) <= REPCLC) THEN
        ZTMP1 (JL,JK) = 0.0_JPRB
        ZTMP15(JL,JK) = 1.0_JPRB/REPCLC
      ELSE
        ZTMP15(JL,JK) = 1.0_JPRB/ZCLDLU5(JL,JK+1)
        ZTMP1 (JL,JK) = -ZCLDLU(JL,JK+1)*ZTMP15(JL,JK)*ZTMP15(JL,JK)
      ENDIF
      IF (ZCLFR5(JL,IKL) <= ZCLFR5(JL,IKL-1)) THEN
        ZCADJ  = ZCLFR(JL,IKL)
        ZCADJ5 = ZCLFR5(JL,IKL)
      ELSE
        ZCADJ  = ZCLFR(JL,IKL-1)
        ZCADJ5 = ZCLFR5(JL,IKL-1)
      ENDIF      
!* Modified for regularization 
      IF (ZCLFR5(JL,IKL-1) > 0.999_JPRB) THEN
        ZCLFR(JL,IKL-1) = 0._JPRB
        ZCLFR5(JL,IKL-1) = 0.999_JPRB
      ENDIF
!* End of modification
      ZTMP35(JL,JK) = 1.0_JPRB/(1.0_JPRB-ZCLFR5(JL,IKL-1))
      ZTMP3 (JL,JK) = ZCLFR(JL,IKL-1)*ZTMP35(JL,JK)*ZTMP35(JL,JK)
      ZTMP4 (JL,JK) = (1.0_JPRB-ZEMIS5(JL,JK+1))*ZCADJ &
       & - ZCADJ5*ZEMIS(JL,JK+1)      
      ZTMP45(JL,JK) = ZCADJ5*(1.0_JPRB-ZEMIS5(JL,JK+1))
      ZTMP5 (JL,JK) = ZCLFR(JL,IKL)-ZCADJ
      ZTMP55(JL,JK) = ZCLFR5(JL,IKL)-ZCADJ5
    ENDDO
  ENDDO

  DO JK1 = 2 , KLEV+1
    DO JL = KIDIA,KFDIA
      ZCLOUD (JL) = ZCLDLU(JL,JK1-1)
      ZCLOUD5(JL) = ZCLDLU5(JL,JK1-1)
      ZSUM (JL)  = ZCLOUD(JL)
      ZSUM5(JL)  = ZCLOUD5(JL)
      ZCLM (JL,JK1-1,JK1) = ZSUM(JL)
      ZCLM5(JL,JK1-1,JK1) = ZSUM5(JL)
    ENDDO
    IJK=MIN(KTOPC+1,JK1)

    DO JK = IJK - 2 , 1 , -1
      IKL = KLEV+1 - JK
      DO JL = KIDIA,KFDIA
        ZTR1  = ZCLOUD5(JL)*ZTMP1(JL,JK) &
         & + ZTMP15(JL,JK)*ZCLOUD(JL)  
        ZTR15 = ZCLOUD5(JL)*ZTMP15(JL,JK)
        ZTR2  = (-ZSUM(JL)+ZCLOUD(JL)-ZCLFR5(JL,IKL-1)*ZTR1 &
         & - ZTR15*ZCLFR(JL,IKL-1))*ZTMP35(JL,JK) &
         & + (1.0_JPRB -ZSUM5(JL)+ZCLOUD5(JL)-ZTR15*ZCLFR5(JL,IKL-1)) &
         & * ZTMP3(JL,JK)  
        ZTR25 = (1.0_JPRB-(ZSUM5(JL)-ZCLOUD5(JL))-ZTR15*ZCLFR5(JL,IKL-1)) &
         & * ZTMP35(JL,JK)  

        ZCLOUD (JL) = (ZTMP45(JL,JK)*ZTR15 + ZTMP55(JL,JK)*ZTR25) &
         & * ZEMIS(JL,JK) &
         & + ZEMIS5(JL,JK)*(ZTMP45(JL,JK)*ZTR1 &
         & + ZTR15*ZTMP4(JL,JK)+ZTMP55(JL,JK)*ZTR2 &
         & + ZTR25*ZTMP5(JL,JK))  
        ZCLOUD5(JL) = ZEMIS5(JL,JK) &
         & * (ZTMP45(JL,JK)*ZTR15 + ZTMP55(JL,JK)*ZTR25)  

        ZSUM (JL) = ZSUM(JL) + ZCLOUD(JL)
        ZSUM5(JL) = ZSUM5(JL) + ZCLOUD5(JL)
        ZCLM (JL,JK,JK1) = ZSUM(JL)
        ZCLM5(JL,JK,JK1) = ZSUM5(JL)
      ENDDO
    ENDDO
  ENDDO

ELSE

  DO JK1 = 2 , KLEV+1
    DO JL = KIDIA,KFDIA
      ZCLEAR(JL) =0.0_JPRB
      ZCLEAR5(JL)=1.0_JPRB
      ZCLOUD(JL) =0.0_JPRB
      ZCLOUD5(JL)=0.0_JPRB
    ENDDO

    IJK=MIN(KTOPC,JK1)
    DO JK = IJK - 1 , 1, -1
      ZALPHA1=RA1OVLP(KLEV+1-JK)

      DO JL = KIDIA,KFDIA
        IF (NOVLP == 1) THEN
!* maximum-random    Geleyn and Hollingsworth 1979
          IF (ZCLDLU5(JL,JK) >= ZCLOUD5(JL)) THEN
            ZNUM = ZCLDLU (JL,JK)
            ZNUM5= ZCLDLU5(JL,JK)
          ELSE
            ZNUM = ZCLOUD(JL)
            ZNUM5= ZCLOUD5(JL)
          ENDIF
          IF (ZCLOUD5(JL) <= 1.0_JPRB-REPCLC) THEN
            ZDEN = ZCLOUD(JL)
            ZDEN5= ZCLOUD5(JL)
          ELSE
            ZDEN = 0.0_JPRB
            ZDEN5= 1.0_JPRB-REPCLC
          ENDIF
          ZDIV25 = 1.0_JPRB/(1.0_JPRB-ZDEN5)
          ZCLEAR (JL) = ZCLEAR(JL)*(1.0_JPRB-ZNUM5)*ZDIV25 &
           & +ZCLEAR5(JL)*( -ZNUM )*ZDIV25 &
           & -ZCLEAR5(JL)*(1.0_JPRB-ZNUM5)*( -ZDEN )*ZDIV25*ZDIV25
          ZCLEAR5(JL) = ZCLEAR5(JL)*(1.0_JPRB-ZNUM5)*ZDIV25
          ZCLM (JL,JK,JK1) = - ZCLEAR(JL)
          ZCLM5(JL,JK,JK1) = 1.0_JPRB - ZCLEAR5(JL)
          ZCLOUD (JL) = ZCLDLU(JL,JK)
          ZCLOUD5(JL) = ZCLDLU5(JL,JK)
        ELSEIF (NOVLP == 2) THEN
!* maximum
          IF (ZCLDLU5(JL,JK) >= ZCLOUD5(JL)) THEN
            ZCLOUD (JL) = ZCLDLU (JL,JK)
            ZCLOUD5(JL) = ZCLDLU5(JL,JK)
          ENDIF
          ZCLM (JL,JK,JK1) = ZCLOUD (JL)
          ZCLM5(JL,JK,JK1) = ZCLOUD5(JL)
        ELSEIF (NOVLP == 3) THEN
!* random
          ZCLEAR(JL)  = ZCLEAR(JL)*(1.0_JPRB - ZCLDLU5(JL,JK))&
           & + ZCLEAR5(JL)*(-ZCLDLU(JL,JK))  
          ZCLEAR5(JL) = ZCLEAR5(JL)*(1.0_JPRB - ZCLDLU5(JL,JK))
          ZCLOUD (JL) = - ZCLEAR(JL)
          ZCLOUD5(JL) = 1.0_JPRB - ZCLEAR5(JL)
          ZCLM (JL,JK,JK1) = ZCLOUD(JL)
          ZCLM5(JL,JK,JK1) = ZCLOUD5(JL)
        ELSEIF (NOVLP == 4) THEN
!** Hogan & Illingworth (2001)
          IF (ZCLDLU5(JL,JK) >= ZCLOUD5(JL)) THEN
            ZNUM = ZCLDLU (JL,JK)
            ZNUM5= ZCLDLU5(JL,JK)
          ELSE
            ZNUM = ZCLOUD(JL)
            ZNUM5= ZCLOUD5(JL)
          ENDIF
          IF (ZCLOUD5(JL) <= 1.0_JPRB-REPCLC) THEN
            ZDEN = ZCLOUD(JL)
            ZDEN5= ZCLOUD5(JL)
          ELSE
            ZDEN = 0.0_JPRB
            ZDEN5= 1.0_JPRB-REPCLC
          ENDIF         
          ZDIV35 = 1.0_JPRB/(1.0_JPRB-ZDEN5)
          ZCLEAR (JL) = -(ZALPHA1*ZCLEAR5(JL)*ZDIV35)*ZNUM &
           & + (ZALPHA1*ZCLEAR5(JL)*(1.0_JPRB-ZNUM5)*ZDIV35*ZDIV35)  &
           & * ZDEN &
           & - (1.0_JPRB-ZALPHA1)*ZCLEAR5(JL)*ZCLDLU(JL,JK) &
           & + (ZALPHA1*(1.0_JPRB-ZNUM5)*ZDIV35 &
           & + (1.0_JPRB-ZALPHA1)*(1.0_JPRB-ZCLDLU5(JL,JK)))*ZCLEAR(JL)
          ZCLEAR5(JL) = ZCLEAR5(JL) &
           & * (ZALPHA1*(1.0_JPRB-ZNUM5)*ZDIV35 &
           & + (1.0_JPRB-ZALPHA1)*(1.0_JPRB-ZCLDLU5(JL,JK)) ) 
          ZCLM (JL,JK,JK1) = -ZCLEAR(JL)
          ZCLM5(JL,JK,JK1) = 1.0_JPRB - ZCLEAR5(JL)
          ZCLOUD (JL) = ZCLDLU (JL,JK)
          ZCLOUD5(JL) = ZCLDLU5(JL,JK)
        ENDIF
      ENDDO
    ENDDO

  ENDDO

ENDIF

!      print *,' LWC after 244: CC below level of calculation'

!*         2.5     CLOUD COVER ABOVE THE LEVEL OF CALCULATION
!                  ------------------------------------------

!250  CONTINUE

IF (LRAISANEN) THEN
!* maximum-random (Raisanen 1998)

  ZEMIS (:,:)  = 0.0_JPRB
  ZEMIS5(:,:) = 0.0_JPRB

  DO JK = 1,KLEV
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      ZDIV45 = 1.0_JPRB/ZCLFR5(JL,IKL)
      ZEMIS (JL,JK)  = ZCLDLD(JL,JK)*ZDIV45 &
       & - ZCLDLD5(JL,JK)*ZCLFR(JL,IKL)*ZDIV45*ZDIV45
      ZEMIS5(JL,JK) = ZCLDLD5(JL,JK)*ZDIV45
    ENDDO
  ENDDO

  DO JK = 2,KTOPC
    IKL = KLEV + 1 - JK
    DO JL = KIDIA,KFDIA
      IF (ZCLDLD5(JL,JK-1) <= REPCLC) THEN
        ZTMP1 (JL,JK) = 0.0_JPRB
        ZTMP15(JL,JK) = 1.0_JPRB/REPCLC
      ELSE
        ZTMP15(JL,JK) = 1.0_JPRB/ZCLDLD5(JL,JK-1)
        ZTMP1 (JL,JK) = -ZCLDLD(JL,JK-1)*ZTMP15(JL,JK)*ZTMP15(JL,JK)
      ENDIF
      IF (ZCLFR5(JL,IKL) <= ZCLFR5(JL,IKL+1)) THEN
        ZCADJ  = ZCLFR(JL,IKL)
        ZCADJ5 = ZCLFR5(JL,IKL)
      ELSE
        ZCADJ  = ZCLFR(JL,IKL+1)
        ZCADJ5 = ZCLFR5(JL,IKL+1)
      ENDIF
!* Modified for regularization 
      IF (ZCLFR5(JL,IKL+1) > 0.999_JPRB) THEN
        ZCLFR(JL,IKL+1) = 0._JPRB
        ZCLFR5(JL,IKL+1) = 0.999_JPRB
      ENDIF
!* End of modification
      ZTMP35(JL,JK) = 1.0_JPRB/(1.0_JPRB-ZCLFR5(JL,IKL+1))
      ZTMP3 (JL,JK) = ZCLFR(JL,IKL+1)*ZTMP35(JL,JK)*ZTMP35(JL,JK)
      ZTMP4 (JL,JK) = (1.0_JPRB-ZEMIS5(JL,JK-1))*ZCADJ &
       & - ZCADJ5*ZEMIS(JL,JK-1)  
      ZTMP45(JL,JK) = ZCADJ5*(1.0_JPRB-ZEMIS5(JL,JK-1))
      ZTMP5 (JL,JK) = ZCLFR(JL,IKL)-ZCADJ
      ZTMP55(JL,JK) = ZCLFR5(JL,IKL)-ZCADJ5
    ENDDO
  ENDDO

  DO JK1 = 1 ,KTOPC 
    DO JL = KIDIA,KFDIA
      ZCLOUD (JL) = ZCLDLD(JL,JK1)
      ZCLOUD5(JL) = ZCLDLD5(JL,JK1)
      ZSUM (JL)  = ZCLOUD(JL)
      ZSUM5(JL)  = ZCLOUD5(JL)
      ZCLM (JL,JK1,JK1) = ZSUM(JL)
      ZCLM5(JL,JK1,JK1) = ZSUM5(JL)
    ENDDO

    DO JK = JK1+1 , KTOPC
      IKL = KLEV + 1 - JK
      DO JL = KIDIA,KFDIA
        ZTR1  = ZCLOUD5(JL)*ZTMP1(JL,JK) &
         & + ZTMP15(JL,JK)*ZCLOUD(JL)  
        ZTR15 = ZCLOUD5(JL)*ZTMP15(JL,JK)

        ZTR2  = (-ZSUM(JL)+ZCLOUD(JL)-ZCLFR5(JL,IKL+1)*ZTR1 &
         & - ZTR15*ZCLFR(JL,IKL+1))*ZTMP35(JL,JK) &
         & + (1.0_JPRB -ZSUM5(JL)+ZCLOUD5(JL)-ZTR15*ZCLFR5(JL,IKL+1)) &
         & * ZTMP3(JL,JK)  
        ZTR25 = (1.0_JPRB-(ZSUM5(JL)-ZCLOUD5(JL))-ZTR15*ZCLFR5(JL,IKL+1)) &
         & * ZTMP35(JL,JK)  
        ZCLOUD (JL) = (ZTMP45(JL,JK)*ZTR15 + ZTMP55(JL,JK)*ZTR25) &
         & * ZEMIS(JL,JK) &
         & + ZEMIS5(JL,JK)*(ZTMP45(JL,JK)*ZTR1 &
         & + ZTR15*ZTMP4(JL,JK)+ZTMP55(JL,JK)*ZTR2 &
         & + ZTR25*ZTMP5(JL,JK))  
        ZCLOUD5(JL) = ZEMIS5(JL,JK) &
         & * (ZTMP45(JL,JK)*ZTR15 + ZTMP55(JL,JK)*ZTR25)  

        ZSUM(JL)  = ZSUM(JL) + ZCLOUD(JL)
        ZSUM5(JL) = ZSUM5(JL) + ZCLOUD5(JL)
        ZCLM (JL,JK,JK1)  = ZSUM(JL)
        ZCLM5(JL,JK,JK1) = ZSUM5(JL)
      ENDDO
    ENDDO
  ENDDO

ELSE

  DO JK1 = 1 , KTOPC
    DO JL = KIDIA,KFDIA
      ZCLEAR (JL)=0.0_JPRB
      ZCLEAR5(JL)=1.0_JPRB
      ZCLOUD (JL)=0.0_JPRB
      ZCLOUD5(JL)=0.0_JPRB
    ENDDO

    DO JK = JK1 , KTOPC
      ZALPHA1=RA1OVLP(KLEV+1-JK)

      DO JL = KIDIA,KFDIA
        IF (NOVLP == 1) THEN
!* maximum-random
          IF (ZCLDLD5(JL,JK) >= ZCLOUD5(JL)) THEN
            ZNUM = ZCLDLD (JL,JK)
            ZNUM5= ZCLDLD5(JL,JK)
          ELSE
            ZNUM = ZCLOUD(JL)
            ZNUM5= ZCLOUD5(JL)
          ENDIF
          IF (ZCLOUD5(JL) <= 1.0_JPRB-REPCLC) THEN
            ZDEN = ZCLOUD(JL)
            ZDEN5= ZCLOUD5(JL)
          ELSE
            ZDEN = 0.0_JPRB
            ZDEN5= 1.0_JPRB-REPCLC
          ENDIF
          ZDIV55 = 1.0_JPRB/(1.0_JPRB-ZDEN5)
          ZCLEAR(JL)=ZCLEAR(JL)*(1.0_JPRB-ZNUM5)*ZDIV55&
           & +ZCLEAR5(JL)*( -ZNUM )*ZDIV55&
           & -ZCLEAR5(JL)*(1.0_JPRB-ZNUM5)*( -ZDEN )*ZDIV55*ZDIV55  
          ZCLEAR5(JL)=ZCLEAR5(JL)*(1.0_JPRB-ZNUM5)*ZDIV55
          ZCLM (JL,JK,JK1) = - ZCLEAR(JL)
          ZCLM5(JL,JK,JK1) = 1.0_JPRB - ZCLEAR5(JL)
          ZCLOUD (JL) = ZCLDLD(JL,JK)
          ZCLOUD5(JL) = ZCLDLD5(JL,JK)
        ELSEIF (NOVLP == 2) THEN
!* maximum
          IF (ZCLDLD5(JL,JK) >= ZCLOUD5(JL)) THEN
            ZCLOUD (JL) = ZCLDLD(JL,JK)
            ZCLOUD5(JL) = ZCLDLD5(JL,JK)
          ENDIF
          ZCLM (JL,JK,JK1) = ZCLOUD(JL)
          ZCLM5(JL,JK,JK1) = ZCLOUD5(JL)
        ELSEIF (NOVLP == 3) THEN
!* random
          ZCLEAR(JL)  = ZCLEAR(JL)*(1.0_JPRB - ZCLDLD5(JL,JK))&
           & + ZCLEAR5(JL)*(-ZCLDLD(JL,JK))  
          ZCLEAR5(JL) = ZCLEAR5(JL)*(1.0_JPRB - ZCLDLD5(JL,JK))
          ZCLOUD (JL) = - ZCLEAR(JL)
          ZCLOUD5(JL) = 1.0_JPRB - ZCLEAR5(JL)
          ZCLM (JL,JK,JK1) = ZCLOUD(JL)
          ZCLM5(JL,JK,JK1) = ZCLOUD5(JL)
        ELSEIF (NOVLP == 4) THEN
!** Hogan & Illingworth (2001)
          IF (ZCLDLD5(JL,JK) >= ZCLOUD5(JL)) THEN
            ZNUM  = ZCLDLD (JL,JK)
            ZNUM5 = ZCLDLD5(JL,JK)
          ELSE
            ZNUM  = ZCLOUD (JL)
            ZNUM5 = ZCLOUD5(JL)
          ENDIF
          IF (ZCLOUD5(JL) <= 1.0_JPRB-REPCLC) THEN
            ZDEN = ZCLOUD (JL)
            ZDEN5= ZCLOUD5(JL)
          ELSE
            ZDEN = 0.0_JPRB
            ZDEN5= 1.0_JPRB-REPCLC
          ENDIF
          ZDIV65 = 1.0_JPRB/(1.0_JPRB-ZDEN5)
          ZCLEAR (JL) = -(ZALPHA1*ZCLEAR5(JL)*ZDIV65)*ZNUM &
           & + (ZALPHA1*ZCLEAR5(JL)*(1.0_JPRB-ZNUM5)*ZDIV65*ZDIV65) &
           & * ZDEN &
           & - (1.0_JPRB-ZALPHA1)*ZCLEAR5(JL)*ZCLDLD(JL,JK) &
           & + (ZALPHA1*(1.0_JPRB-ZNUM5)*ZDIV65 &
           & + (1.0_JPRB-ZALPHA1)*(1.0_JPRB-ZCLDLD5(JL,JK)))*ZCLEAR(JL)  
          ZCLEAR5(JL)=ZCLEAR5(JL) &
           & * (ZALPHA1*(1.0_JPRB-ZNUM5)*ZDIV65 &
           & + (1.0_JPRB-ZALPHA1)*(1.0_JPRB-ZCLDLD5(JL,JK)) ) 
          ZCLM (JL,JK,JK1) = - ZCLEAR(JL)
          ZCLM5(JL,JK,JK1) = 1.0_JPRB - ZCLEAR5(JL)
          ZCLOUD (JL) = ZCLDLD (JL,JK)
          ZCLOUD5(JL) = ZCLDLD5(JL,JK)
        ENDIF
      ENDDO
    ENDDO
  ENDDO

ENDIF

!      print *,' LWC after 254: CC above level of calculation'

!*         3.      FLUXES FOR PARTIAL/MULTIPLE LAYERED CLOUDINESS
!                  ----------------------------------------------

!300  CONTINUE

!*         3.1     DOWNWARD FLUXES
!                  ---------------

!310  CONTINUE

DO JL = KIDIA,KFDIA
  PFLUX (JL,2,KLEV+1) = 0.0_JPRB
  PFLUX5(JL,2,KLEV+1) = 0.0_JPRB
ENDDO

DO JK1 = 1, KTOPC

!*                 CONTRIBUTION FROM CLEAR-SKY FRACTION

  DO JL = KIDIA,KFDIA
    ZFD (JL) = (1.0_JPRB-ZCLM5(JL,KTOPC,JK1))*ZDNF(JL,1,JK1) &
     & - ZDNF5(JL,1,JK1)*ZCLM(JL,KTOPC,JK1)
    ZFD5(JL) = (1.0_JPRB - ZCLM5(JL,KTOPC,JK1))*ZDNF5(JL,1,JK1)

!*                 CONTRIBUTION FROM ADJACENT CLOUD

    ZFD (JL) = ZFD(JL)+ZCLM5(JL,JK1,JK1)*ZDNF(JL,JK1+1,JK1) &
     & + ZDNF5(JL,JK1+1,JK1)*ZCLM(JL,JK1,JK1)
    ZFD5(JL) = ZFD5(JL) + ZCLM5(JL,JK1,JK1) * ZDNF5(JL,JK1+1,JK1)
  ENDDO

!*                 CONTRIBUTION FROM OTHER CLOUDY FRACTIONS

  DO JK = JK1, KTOPC-1 
    DO JL = KIDIA,KFDIA
      ZCFRAC  = ZCLM (JL,JK+1,JK1) - ZCLM (JL,JK,JK1)
      ZCFRAC5 = ZCLM5(JL,JK+1,JK1) - ZCLM5(JL,JK,JK1)
      ZFD (JL) = ZFD(JL)+ZCFRAC5*ZDNF(JL,JK+2,JK1) &
       & + ZDNF5(JL,JK+2,JK1)*ZCFRAC
      ZFD5(JL) = ZFD5(JL)+ZCFRAC5*ZDNF5(JL,JK+2,JK1)
    ENDDO
  ENDDO

  DO JL = KIDIA,KFDIA
    PFLUX (JL,2,JK1) = ZFD (JL)
    PFLUX5(JL,2,JK1) = ZFD5(JL)
  ENDDO

ENDDO
!      print *,' LWC after 317: Downward fluxes'

!*         3.2     UPWARD FLUX AT THE SURFACE
!                  --------------------------

!320  CONTINUE

DO JL = KIDIA,KFDIA
  PFLUX (JL,1,1) = PEMIT5(JL)*PBSUI(JL)+PBSUI5(JL)*PEMIT(JL) &
   & - (1.0_JPRB-PEMIT5(JL))*PFLUX(JL,2,1)+PFLUX5(JL,2,1)*PEMIT(JL)
  PFLUX5(JL,1,1) = PEMIT5(JL)*PBSUI5(JL) &
   & -(1.0_JPRB-PEMIT5(JL))*PFLUX5(JL,2,1)
ENDDO

!*         3.3     UPWARD FLUXES
!                  -------------

!330  CONTINUE

DO JK1 = 2 , KLEV+1

!*                 CONTRIBUTION FROM CLEAR-SKY FRACTION

  DO JL = KIDIA,KFDIA
    ZFU (JL) = (1.0_JPRB-ZCLM5(JL,1,JK1))*ZUPF(JL,1,JK1) &
     & - ZUPF5(JL,1,JK1)*ZCLM(JL,1,JK1)
    ZFU5(JL) = (1.0_JPRB - ZCLM5(JL,1,JK1)) * ZUPF5(JL,1,JK1)
  ENDDO

!*                 CONTRIBUTION FROM OTHER CLOUDY FRACTIONS

  IJK=MIN(KTOPC+1,JK1)
  DO JK = 2 , IJK-1
    DO JL = KIDIA,KFDIA
      ZCFRAC  = ZCLM (JL,JK-1,JK1) - ZCLM (JL,JK,JK1)
      ZCFRAC5 = ZCLM5(JL,JK-1,JK1) - ZCLM5(JL,JK,JK1)
      ZFU (JL) =  ZFU(JL)+ZCFRAC5*ZUPF(JL,JK,JK1) &
       & + ZUPF5(JL,JK,JK1)*ZCFRAC
      ZFU5(JL) = ZFU5(JL) + ZCFRAC5 * ZUPF5(JL,JK  ,JK1)
    ENDDO
  ENDDO

!*                 CONTRIBUTION FROM ADJACENT CLOUD

  DO JL = KIDIA,KFDIA
    ZFU (JL) = ZFU(JL)+ZCLM5(JL,JK1-1,IJK)*ZUPF(JL,IJK,IJK) &
     & + ZUPF5(JL,IJK,IJK)*ZCLM(JL,JK1-1,IJK)
    ZFU5(JL) = ZFU5(JL) + ZCLM5(JL,JK1-1,IJK) * ZUPF5(JL,IJK,IJK)
  ENDDO

  DO JL = KIDIA,KFDIA
    PFLUX (JL,1,JK1) = ZFU (JL)
    PFLUX5(JL,1,JK1) = ZFU5(JL)
  ENDDO
ENDDO

!      print *,' LWC after 337: Upward fluxes'

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWCTL',1,ZHOOK_HANDLE)
END SUBROUTINE LWCTL
