!OPTIONS XOPT(HSFUN)
SUBROUTINE LWTTTL ( KIDIA, KFDIA, KLON, PGA, PGB, PUU5, PTT5, PUU, PTT )

!**** *LWTTTL* - LONGWAVE TRANSMISSION FUNCTIONS
!                      (tangent linear)

!     PURPOSE.
!     --------
!           THIS ROUTINE COMPUTES THE TRANSMISSION FUNCTIONS FOR ALL THE
!     ABSORBERS (H2O, UNIFORMLY MIXED GASES, AND O3) IN ALL SIX SPECTRAL
!     INTERVALS.

!**   INTERFACE.
!     ----------
!          *LWTTTL* IS CALLED FROM *LWVNTL*, *LWVBTL*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
! KND    :                    ; WEIGHTING INDEX
!  Trajectory  Perturbation  Description                         Dimension
!  PGA         ---           PADE APPROXIMANTS                   (KLON,NIPD,2)
!  PGB         ---           PADE APPROXIMANTS                   (KLON,NIPD,2)
!  PUU5        PUU           ABSORBER AMOUNTS                    (KLON,NUA)

!     ==== OUTPUTS ===
!  Trajectory  Perturbation  Description                         Dimension
!  PTT5        PTT           TRANSMISSION FUNCTIONS              (KLON,NTRA)

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. TRANSMISSION FUNCTION BY H2O AND UNIFORMLY MIXED GASES ARE
!     COMPUTED USING PADE APPROXIMANTS AND HORNER'S ALGORITHM.
!          2. TRANSMISSION BY O3 IS EVALUATED WITH MALKMUS'S BAND MODEL.
!          3. TRANSMISSION BY H2O CONTINUUM AND AEROSOLS FOLLOW AN
!     A SIMPLE EXPONENTIAL DECREASE WITH ABSORBER AMOUNT.

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        M. Janiskova  *ECMWF*       18-Jan-2006

!     MODIFICATIONS.
!     --------------
    
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOELW    , ONLY : NIPD , NTRA  ,NUA  ,RPTYPE   ,RETYPE   ,&
 & RO1H     ,RO2H     ,RPIALF0  
USE YOELWCONST,ONLY : RCH4A, RCH4B, RCN2OA, RCN2OB

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGA(KLON,NIPD,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGB(KLON,NIPD,2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUU5(KLON,NUA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTT5(KLON,NTRA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUU(KLON,NUA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTT(KLON,NTRA) 
!     ------------------------------------------------------------------

!*        0.1   ARGUMENTS
!               ---------

INTEGER(KIND=JPIM) :: JA, JL

REAL(KIND=JPRB) :: ZA11, ZA12, ZAERCN, ZEU10, ZEU11, ZEU12,&
 & ZEU13, ZODH41, ZODH42, ZODN21, ZODN22, ZPU10, &
 & ZPU11, ZPU12, ZPU13, ZSQ1, ZSQ2, ZSQH41, &
 & ZSQH42, ZSQN21, ZSQN22, ZTO1, ZTO2, ZTTF11, &
 & ZTTF12, ZUU11, ZUU12, ZUXY, ZVXY, &
 & ZXD, ZXN, ZZ, ZUXYCH4, ZVXYCH4, &
 & ZUXYN2O, ZVXYN2O
REAL(KIND=JPRB) :: ZZ5, ZXD5, ZXN5, ZPU105, ZPU115, ZPU125, ZPU135 
REAL(KIND=JPRB) :: ZEU105, ZEU115, ZEU125, ZEU135
REAL(KIND=JPRB) :: ZUXY5, ZVXY5, ZSQ15, ZSQ25, ZAERCN5, ZTO15, ZTO25
REAL(KIND=JPRB) :: ZUXYCH45, ZVXYCH45, ZSQH415,ZODH415, ZUXYN2O5, ZVXYN2O5
REAL(KIND=JPRB) :: ZSQN215, ZODN215, ZODN225, ZSQN225 
REAL(KIND=JPRB) :: ZSQH425, ZODH425, ZA115, ZA125 
REAL(KIND=JPRB) :: ZTTF115,ZTTF125, ZUU115, ZUU125
REAL(KIND=JPRB) :: ZXDIV5, ZDIV15, ZDIV25, ZDIV35, ZDIV45, ZDIV55, ZDIV65
REAL(KIND=JPRB) :: ZZ15, ZSQRT15, ZZ25, ZSQRT25, ZZ35, ZSQRT35, ZZ45, ZSQRT45
REAL(KIND=JPRB) :: ZZ55, ZSQRT55, ZZ65, ZSQRT65, ZEXP15, ZEXP25
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!               ------------

!     ------------------------------------------------------------------
!DIR$ VFUNCTION SQRTHF

!*         1.     HORNER'S ALGORITHM FOR H2O AND CO2 TRANSMISSION
!                 -----------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWTTTL',0,ZHOOK_HANDLE)
DO JA = 1 , 8
  DO JL = KIDIA,KFDIA
    ZZ5 = SQRT(PUU5(JL,JA))
    IF (ZZ5 > 0.0_JPRB) THEN
      ZZ  = 0.5_JPRB*PUU(JL,JA)/ZZ5
    ELSE
      ZZ  = 0.0_JPRB
    ENDIF
    ZXD  = (PGB( JL,JA,2)+2.0_JPRB*ZZ5)*ZZ
    ZXD5 = PGB( JL,JA,1) + ZZ5* (PGB( JL,JA,2) + ZZ5 )
    ZXN  = ZZ*PGA( JL,JA,2)
    ZXN5 = PGA( JL,JA,1) + ZZ5*PGA( JL,JA,2)
    ZXDIV5 = 1.0_JPRB/ZXD5
    PTT (JL,JA) = ZXN*ZXDIV5 - ZXN5*ZXDIV5*ZXDIV5*ZXD
    PTT5(JL,JA) = ZXN5*ZXDIV5
  ENDDO
ENDDO

DO JL = KIDIA,KFDIA
  IF (PTT5(JL,3) < 0.0_JPRB) THEN
    PTT (JL,3) = 0.0_JPRB
    PTT5(JL,3) = 0.0_JPRB
  ENDIF
ENDDO

!     ------------------------------------------------------------------

!*         2.     CONTINUUM, OZONE AND AEROSOL TRANSMISSION FUNCTIONS
!                 ---------------------------------------------------

DO JL = KIDIA,KFDIA
  PTT (JL, 9) = PTT (JL, 8)
  PTT5(JL, 9) = PTT5(JL, 8)

!-  CONTINUUM ABSORPTION: E- AND P-TYPE (from Giorgetta and Wild, 1997)

  ZPU10  = RPTYPE(1) * PUU (JL,10)
  ZPU105 = RPTYPE(1) * PUU5(JL,10)
  ZPU11  = RPTYPE(2) * PUU (JL,10)
  ZPU115 = RPTYPE(2) * PUU5(JL,10)
  ZPU12  = RPTYPE(3) * PUU (JL,10)
  ZPU125 = RPTYPE(3) * PUU5(JL,10)
  ZPU13  = RPTYPE(4) * PUU (JL,10)
  ZPU135 = RPTYPE(4) * PUU5(JL,10)
  ZEU10  = RETYPE(1) * PUU (JL,11)
  ZEU105 = RETYPE(1) * PUU5(JL,11)
  ZEU11  = RETYPE(2) * PUU (JL,11)
  ZEU115 = RETYPE(2) * PUU5(JL,11)
  ZEU12  = RETYPE(3) * PUU (JL,11)
  ZEU125 = RETYPE(3) * PUU5(JL,11)
  ZEU13  = RETYPE(4) * PUU (JL,11)
  ZEU135 = RETYPE(4) * PUU5(JL,11)

!-  OZONE ABSORPTION

  ZDIV15 = 1.0_JPRB/(RPIALF0 * PUU5(JL,13))
  ZUXY  = 8.0_JPRB*PUU5(JL,12)*ZDIV15*PUU(JL,12) &
   & - 4._JPRB*RPIALF0*PUU5(JL,12)*PUU5(JL,12)*ZDIV15*ZDIV15*PUU(JL,13)
  ZUXY5 = 4.0_JPRB * PUU5(JL,12) * PUU5(JL,12) * ZDIV15
  ZZ15 = 1.0_JPRB + RO1H * ZUXY5
  ZSQRT15 = SQRT(ZZ15)
  IF (ZZ15 > 0.0_JPRB) THEN
    ZSQ1  = 0.5_JPRB*RO1H*ZUXY/ZSQRT15
    ZSQ15 = ZSQRT15 - 1.0_JPRB
  ELSE
    ZSQ1  = 0.0_JPRB
    ZSQ15 = - 1.0_JPRB
  ENDIF
  ZZ25 = 1.0_JPRB + RO2H * ZUXY5
  ZSQRT25 = SQRT(ZZ25)
  IF (ZZ25 > 0.0_JPRB) THEN
    ZSQ2  = 0.5_JPRB*RO2H*ZUXY/ZSQRT25
    ZSQ25 = ZSQRT25 - 1.0_JPRB
  ELSE
    ZSQ2  = 0.0_JPRB
    ZSQ25 = - 1.0_JPRB
  ENDIF
  ZDIV25 = 1.0_JPRB/PUU5(JL,12)
  ZVXY  = 0.5_JPRB*RPIALF0*(PUU(JL,13)*ZDIV25 &
   & -PUU5(JL,13)*ZDIV25*ZDIV25*PUU(JL,12))
  ZVXY5 = 0.5_JPRB * RPIALF0 * PUU5(JL,13) * ZDIV25
  ZAERCN  = PUU(JL,17) + ZEU12 + ZPU12  
  ZAERCN5 = PUU5(JL,17) + ZEU125 + ZPU125
  ZTO15 = EXP( - ZVXY5 * ZSQ15 - ZAERCN5 )
  ZTO1  = ZTO15*(-ZVXY5*ZSQ1-ZSQ15*ZVXY-ZAERCN)
  ZTO25 = EXP( - ZVXY5 * ZSQ25 - ZAERCN5 )
  ZTO2  = ZTO25*(-ZVXY5*ZSQ2-ZSQ25*ZVXY-ZAERCN)

!-- TRACE GASES (CH4, N2O, CFC-11, CFC-12)

!* CH4 IN INTERVAL 800-970 + 1110-1250 CM-1

  ZDIV35 = 1.0_JPRB/PUU5(JL,20)
  ZUXYCH4  = 2.0_JPRB*PUU5(JL,19)*ZDIV35*PUU(JL,19) &
   & -PUU5(JL,19)*PUU5(JL,19)*ZDIV35*ZDIV35*PUU(JL,20)
  ZUXYCH45 = PUU5(JL,19)*PUU5(JL,19)*ZDIV35
  ZDIV45 = 1.0_JPRB/PUU5(JL,19)
  ZVXYCH4  = PUU(JL,20)*ZDIV45-PUU5(JL,20)*ZDIV45*ZDIV45*PUU(JL,19)
  ZVXYCH45 = PUU5(JL,20)*ZDIV45
  ZUXY  = RCH4A * ZUXYCH4
  ZUXY5 = RCH4A * ZUXYCH45
  ZZ35 = 1.0_JPRB + 33.7_JPRB * ZUXY5
  ZSQRT35 = SQRT(ZZ35)
  IF (ZZ35 > 0.0_JPRB) THEN
    ZSQH41  = 16.85_JPRB*ZUXY/ZSQRT35
    ZSQH415 = ZSQRT35 - 1.0_JPRB
  ELSE
    ZSQH41  = 0.0_JPRB
    ZSQH415 = - 1.0_JPRB
  ENDIF
  ZVXY  = 0.0515_JPRB * ZVXYCH4
  ZVXY5 = 0.0515_JPRB * ZVXYCH45
  ZODH41  = ZVXY5*ZSQH41+ZSQH415*ZVXY
  ZODH415 = ZVXY5 * ZSQH415

!* N2O IN INTERVAL 800-970 + 1110-1250 CM-1

  ZDIV55 = 1.0_JPRB/PUU5(JL,22)
  ZUXYN2O  = 2.0_JPRB*PUU5(JL,21)*ZDIV55*PUU(JL,21) &
   & -PUU5(JL,21)*PUU5(JL,21)*ZDIV55*ZDIV55*PUU(JL,22)
  ZUXYN2O5 = PUU5(JL,21)*PUU5(JL,21)*ZDIV55
  ZDIV65 = 1.0_JPRB/PUU5(JL,21)
  ZVXYN2O  = PUU(JL,22)*ZDIV65-PUU5(JL,22)*ZDIV65*ZDIV65*PUU(JL,21)
  ZVXYN2O5 = PUU5(JL,22)*ZDIV65
  ZUXY  = RCN2OA * ZUXYN2O
  ZUXY5 = RCN2OA * ZUXYN2O5
  ZZ45 = 1.0_JPRB + 21.3_JPRB * ZUXY5
  ZSQRT45 = SQRT(ZZ45)
  IF (ZZ45 > 0.0_JPRB) THEN
    ZSQN21  = 10.65_JPRB*ZUXY/ZSQRT45
    ZSQN215 = ZSQRT45 - 1.0_JPRB
  ELSE
    ZSQN21  = 0.0_JPRB
    ZSQN215 = - 1.0_JPRB
  ENDIF
  ZVXY  = 0.208_JPRB * ZVXYN2O
  ZVXY5 = 0.208_JPRB * ZVXYN2O5
  ZODN21  = ZVXY5*ZSQN21+ZSQN215*ZVXY
  ZODN215 = ZVXY5 * ZSQN215

!* CH4 IN INTERVAL 1250-1450 + 1880-2820 CM-1

  ZUXY  = RCH4B * ZUXYCH4
  ZUXY5 = RCH4B * ZUXYCH45
  ZZ55 = 1.0_JPRB + 400._JPRB * ZUXY5
  ZSQRT55 = SQRT(ZZ55)
  IF (ZZ55 > 0.0_JPRB) THEN
    ZSQH42  = 200._JPRB*ZUXY/ZSQRT55
    ZSQH425 = ZSQRT55 - 1.0_JPRB
  ELSE
    ZSQH42  = 0.0_JPRB
    ZSQH425 = - 1.0_JPRB
  ENDIF
  ZVXY  = 0.0565_JPRB * ZVXYCH4
  ZVXY5 = 0.0565_JPRB * ZVXYCH45
  ZODH42  = ZVXY5*ZSQH42+ZSQH425*ZVXY
  ZODH425 = ZVXY5 * ZSQH425

!* N2O IN INTERVAL 1250-1450 + 1880-2820 CM-1

  ZUXY  = RCN2OB * ZUXYN2O
  ZUXY5 = RCN2OB * ZUXYN2O5
  ZZ65 = 1.0_JPRB + 2000._JPRB * ZUXY5
  ZSQRT65 = SQRT(ZZ65)
  IF (ZZ65 > 0.0_JPRB) THEN
    ZSQN22  = 1000._JPRB*ZUXY/ZSQRT65
    ZSQN225 = ZSQRT65 - 1.0_JPRB
  ELSE
    ZSQN22  = 0.0_JPRB
    ZSQN225 = - 1.0_JPRB
  ENDIF
  ZVXY  = 0.0985_JPRB * ZVXYN2O
  ZVXY5 = 0.0985_JPRB * ZVXYN2O5
  ZODN22  = ZVXY5*ZSQN22+ZSQN225*ZVXY
  ZODN225 = ZVXY5 * ZSQN225

!* CFC-11 IN INTERVAL 800-970 + 1110-1250 CM-1

  ZA11  = 2.0_JPRB * PUU (JL,23) * 4.404E+05_JPRB
  ZA115 = 2.0_JPRB * PUU5(JL,23) * 4.404E+05_JPRB
  ZTTF11  = - ZA11 * 0.003225_JPRB
  ZTTF115 = 1.0_JPRB - ZA115 * 0.003225_JPRB

!* CFC-12 IN INTERVAL 800-970 + 1110-1250 CM-1

  ZA12  = 2.0_JPRB * PUU (JL,24) * 6.7435E+05_JPRB
  ZA125 = 2.0_JPRB * PUU5(JL,24) * 6.7435E+05_JPRB
  ZTTF12  = - ZA12 * 0.003225_JPRB
  ZTTF125 = 1.0_JPRB - ZA125 * 0.003225_JPRB
  ZUU11  = - PUU (JL,15) - ZEU10 - ZPU10
  ZUU115 = - PUU5(JL,15) - ZEU105 - ZPU105
  ZUU12  = - PUU (JL,16) - ZEU11 - ZPU11 - ZODH41 - ZODN21
  ZUU125 = - PUU5(JL,16) - ZEU115 - ZPU115 - ZODH415 - ZODN215
  PTT5(JL,10) = EXP( - PUU5(JL,14) )
  PTT (JL,10) = -PUU(JL,14)*PTT5(JL,10)
  PTT5(JL,11) = EXP( ZUU115 )
  PTT (JL,11) = ZUU11*PTT5(JL,11)
  ZEXP15 = EXP( ZUU125)
  PTT (JL,12) = ZEXP15*ZTTF115*ZTTF12+ZEXP15*ZTTF125*ZTTF11 &
   & +ZEXP15*ZTTF115*ZTTF125*ZUU12
  PTT5(JL,12) = ZEXP15 * ZTTF115 * ZTTF125
  PTT (JL,13) = 0.7554_JPRB * ZTO1 + 0.2446_JPRB * ZTO2
  PTT5(JL,13) = 0.7554_JPRB * ZTO15 + 0.2446_JPRB * ZTO25
  ZEXP25 = EXP( - ZEU135 - ZPU135 )
  PTT (JL,14) = ZEXP25*PTT(JL,10)+PTT5(JL,10)*ZEXP25*(-ZEU13-ZPU13)
  PTT5(JL,14) = PTT5(JL,10) * ZEXP25
  PTT5(JL,15) = EXP ( - PUU5(JL,14) - ZODH425 - ZODN225 )
  PTT (JL,15) = PTT5(JL,15)*(-PUU(JL,14)-ZODH42-ZODN22)
ENDDO

IF (LHOOK) CALL DR_HOOK('LWTTTL',1,ZHOOK_HANDLE)
END SUBROUTINE LWTTTL
