SUBROUTINE LWUTL &
 & ( KIDIA, KFDIA, KLON, KLEV,&
 & PVIEW  , PCCO2 , PAER5 , PDP5 , PPMB5, PQOF5 , PTAVE5, PWV5,&
 & PABCU5 ,&
 & PAER   , PDP   , PPMB  , PQOF , PTAVE , PWV   ,&
 & PABCU  &
 & )

!**** *LWUTL* - LONGWAVE EFFECTIVE ABSORBER AMOUNTS
!                      (tangent linear)

!     PURPOSE.
!     --------
!           COMPUTES ABSORBER AMOUNTS INCLUDING PRESSURE AND
!           TEMPERATURE EFFECTS

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!  Trajectory  Perturbation  Description                         Dimension
!  PAER5       PAER          OPTICAL THICKNESS OF THE AEROSOLS   (KLON,6,KLEV)
!  PCCO2       ---           CONCENTRATION IN CO2 (PA/PA)        
!  PDP5        PDP           LAYER PRESSURE THICKNESS (PA)       (KLON,KLEV)
!  PPMB5       PPMB          HALF LEVEL PRESSURE                 (KLON,KLEV+1)
!  PQOF5       PQOF          CONCENTRATION IN OZONE (PA/PA)      (KLON,KLEV)
!  PTAVE5      PTAVE         TEMPERATURE                         (KLON,KLEV)
!  PWV5        PWV           SPECIFIC HUMIDITY PA/PA             (KLON,KLEV)
!  PVIEW      ---            COSECANT OF VIEWING ANGLE           (KLON)

!     ==== OUTPUTS ===
!  Trajectory  Perturbation  Description                         Dimension
!  PABCU5      PABCU         EFFECTIVE ABSORBER AMOUNTS     (KLON,NUA,3*KLEV+1)

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. COMPUTES THE PRESSURE AND TEMPERATURE WEIGHTED AMOUNTS OF
!     ABSORBERS.

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        M. Janiskova  *ECMWF*       30-Jan-2006

!     MODIFICATIONS.
!     --------------

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RG
USE YOESW    , ONLY : RAER
USE YOELW    , ONLY : NSIL     ,NUA      ,NG1      ,NG1P1    ,&
 & ALWT     ,BLWT     ,RO3T     ,RT1      ,TREF     ,&
 & RVGCO2   ,RVGH2O   ,RVGO3  
USE YOERDI   , ONLY : RCH4     ,RN2O     ,RCFC11   ,RCFC12
USE YOERDU   , ONLY : R10E     ,REPSCO   ,REPSCQ
USE YOPHNC   , ONLY : LH2OCO2

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVIEW(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCCO2
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER5(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDP5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPMB5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQOF5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAVE5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWV5(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PABCU5(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAER(KLON,6,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDP(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PPMB(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQOF(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTAVE(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWV(KLON,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PABCU(KLON,NUA,3*KLEV+1)

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------
REAL(KIND=JPRB) :: ZABLY(KLON,7,3*KLEV+1)  , ZDPM(KLON,3*KLEV)
REAL(KIND=JPRB) :: ZUPM(KLON,3*KLEV)  
REAL(KIND=JPRB) :: ZPHIO(KLON),ZPSC2(KLON) , ZPSC3(KLON), ZPSH1(KLON)
REAL(KIND=JPRB) :: ZPSH2(KLON),ZPSH3(KLON) , ZPSH4(KLON), ZPSH5(KLON)
REAL(KIND=JPRB) :: ZPSH6(KLON),ZPSIO(KLON) , ZTCON(KLON)
REAL(KIND=JPRB) :: ZPHM6(KLON),ZPSM6(KLON) , ZPHN6(KLON), ZPSN6(KLON)  
REAL(KIND=JPRB) :: ZSSIG(KLON,3*KLEV+1)
REAL(KIND=JPRB) :: ZUAER(KLON,NSIL) , ZUAERS(KLON,NSIL)
REAL(KIND=JPRB) :: ZXOZ(KLON) , ZXWV(KLON) , ZTX(KLON), ZTX2(KLON)
REAL(KIND=JPRB) :: ZFPPW(KLON), ZSUM(KLON) , ZDIV(KLON)

REAL(KIND=JPRB) :: ZABLY5(KLON,7,3*KLEV+1)  , ZDPM5(KLON,3*KLEV)
REAL(KIND=JPRB) :: ZUPM5(KLON,3*KLEV)  
REAL(KIND=JPRB) :: ZPHIO5(KLON),ZPSC25(KLON) , ZPSC35(KLON), ZPSH15(KLON)
REAL(KIND=JPRB) :: ZPSH25(KLON),ZPSH35(KLON) , ZPSH45(KLON), ZPSH55(KLON)
REAL(KIND=JPRB) :: ZPSH65(KLON),ZPSIO5(KLON) , ZTCON5(KLON)
REAL(KIND=JPRB) :: ZPHM65(KLON),ZPSM65(KLON) , ZPHN65(KLON), ZPSN65(KLON)  
REAL(KIND=JPRB) :: ZSSIG5(KLON,3*KLEV+1)
REAL(KIND=JPRB) :: ZUAER5(KLON,NSIL), ZUAERS5(KLON,NSIL)
REAL(KIND=JPRB) :: ZXOZ5(KLON) , ZXWV5(KLON) , ZTX5(KLON), ZTX25(KLON)
REAL(KIND=JPRB) :: ZFPPW5(KLON), ZSUM5(KLON) , ZDIV5(KLON)



INTEGER(KIND=JPIM) :: IAE1, IAE2, IAE3, IC, ICP1, IG1, IJ, IJPN,&
 & IKIP1, IKJ, IKJP, IKJPN, IKJR, IKL, JA, JAE, &
 & JK, JKI, JKK, JL  

REAL(KIND=JPRB) :: ZALUP, ZCAC8, ZCAH1, ZCAH2, ZCAH3, ZCAH4,&
 & ZCAH5, ZCAH6, ZCBC8, ZCBH1, ZCBH2, ZCBH3, &
 & ZCBH4, ZCBH5, ZCBH6, ZDPMP0,  &
 & ZU6, ZUPMG, ZUP, ZUPMCO2,  &
 & ZUPMH2O, ZUPMO3, ZZABLY  


REAL(KIND=JPRB) :: ZDPMP05, ZUPMG5, ZUPMCO25, ZUPMH2O5, ZUPMO35
REAL(KIND=JPRB) :: ZZABLY5, ZCAH15, ZCAH25, ZCAH35, ZCAH45, ZCAH55, ZCAH65
REAL(KIND=JPRB) :: ZCBH15, ZCBH25, ZCBH35, ZCBH45, ZCBH55, ZCBH65
REAL(KIND=JPRB) :: ZU65, ZUP5, ZALUP5, ZCAC85, ZCBC85, ZDUCM5
REAL(KIND=JPRB) :: ZDIV15, ZDIV25, ZDIV35, ZLOG15
REAL(KIND=JPRB) :: ZUP1, ZUP15, ZUP25, ZEQ15, ZXOZZ, ZXOZZ5

REAL(KIND=JPRB) :: ZRGI, ZINORMP, ZDUCM, ZIPCCO2, ZDIFFCO2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWUTL',0,ZHOOK_HANDLE)

!*         1.    INITIALIZATION
!                --------------

ZRGI = 0.1_JPRB/RG
ZIPCCO2 = 1.0_JPRB/PCCO2
ZINORMP = 1.0_JPRB/101325._JPRB

!-----------------------------------------------------------------------

!*         2.    PRESSURE OVER GAUSS SUB-LEVELS
!                ------------------------------

DO JL = KIDIA,KFDIA
  ZSSIG (JL, 1 ) = PPMB (JL,1) * 100._JPRB
  ZSSIG5(JL, 1 ) = PPMB5(JL,1) * 100._JPRB
ENDDO

DO JK = 1 , KLEV
  IKJ=(JK-1)*NG1P1+1
  IKJR = IKJ
  IKJP = IKJ + NG1P1
  DO JL = KIDIA,KFDIA
    ZSSIG (JL,IKJP) = PPMB( JL,JK+1)* 100._JPRB
    ZSSIG5(JL,IKJP) = PPMB5(JL,JK+1)* 100._JPRB
  ENDDO
  DO IG1=1,NG1
    IKJ=IKJ+1
    DO JL = KIDIA,KFDIA
      ZSSIG (JL,IKJ) = (ZSSIG (JL,IKJR) + ZSSIG (JL,IKJP)) * 0.5_JPRB &
       &  + RT1(IG1) * (ZSSIG (JL,IKJP) - ZSSIG (JL,IKJR)) * 0.5_JPRB
      ZSSIG5(JL,IKJ) = (ZSSIG5(JL,IKJR) + ZSSIG5(JL,IKJP)) * 0.5_JPRB &
       &  + RT1(IG1) * (ZSSIG5(JL,IKJP) - ZSSIG5(JL,IKJR)) * 0.5_JPRB  
    ENDDO
  ENDDO
ENDDO

!-----------------------------------------------------------------------

!*         4.    PRESSURE THICKNESS AND MEAN PRESSURE OF SUB-LAYERS
!                --------------------------------------------------

DO JKI=1,3*KLEV
  IKIP1=JKI+1
  DO JL = KIDIA,KFDIA
    ZUPM (JL,JKI) = (ZSSIG (JL,JKI)+ZSSIG (JL,IKIP1))*0.5_JPRB
    ZUPM5(JL,JKI) = (ZSSIG5(JL,JKI)+ZSSIG5(JL,IKIP1))*0.5_JPRB
    ZDPM (JL,JKI) = (ZSSIG (JL,JKI)-ZSSIG (JL,IKIP1))*ZRGI
    ZDPM5(JL,JKI) = (ZSSIG5(JL,JKI)-ZSSIG5(JL,IKIP1))*ZRGI
  ENDDO
ENDDO

IF (LH2OCO2) THEN
  DO JK = 1 , KLEV
    IKL = KLEV+1 - JK
    DO JL = KIDIA,KFDIA
      IF (PWV5(JL,IKL) > REPSCQ) THEN
        ZXWV (JL) = PWV (JL,IKL)
        ZXWV5(JL) = PWV5(JL,IKL)
      ELSE
        ZXWV (JL) = 0.0_JPRB
        ZXWV5(JL) = REPSCQ
      ENDIF
    ENDDO
    IKJ=(JK-1)*NG1P1+1
    IKJPN=IKJ+NG1
    DO JKK=IKJ,IKJPN
      DO JL = KIDIA,KFDIA
        ZDPMP0  = ZDPM (JL,JKK) * ZINORMP
        ZDPMP05 = ZDPM5(JL,JKK) * ZINORMP
        ZUPMCO2  = (ZUPM5(JL,JKK)+RVGCO2)*ZDPMP0+ZDPMP05*ZUPM(JL,JKK)
        ZUPMCO25 = (ZUPM5(JL,JKK)+RVGCO2)*ZDPMP05
        ZUPMH2O  = (ZUPM5(JL,JKK)+RVGH2O)*ZDPMP0+ZDPMP05*ZUPM(JL,JKK)
        ZUPMH2O5 = (ZUPM5(JL,JKK)+RVGH2O)*ZDPMP05

!      ZFPPW = 1.6078_JPRB * ZXWV(JL) / (1.0_JPRB+0.608_JPRB*ZXWV(JL))
!- H2O
        ZABLY (JL,1,JKK) = ZXWV5(JL)*ZUPMH2O+ZUPMH2O5*ZXWV(JL)
        ZABLY5(JL,1,JKK) = ZXWV5(JL)*ZUPMH2O5
!- CO2 (CH4,N2O)
        ZABLY (JL,3,JKK) = PCCO2*ZUPMCO2
        ZABLY5(JL,3,JKK) = PCCO2*ZUPMCO25
      ENDDO
    ENDDO
  ENDDO

ELSE

  DO JK = 1 , KLEV
    IKL = KLEV+1 - JK
    DO JL = KIDIA,KFDIA
      IF (PWV5(JL,IKL) > REPSCQ) THEN
        ZXWV (JL) = PWV (JL,IKL)
        ZXWV5(JL) = PWV5(JL,IKL)
      ELSE
        ZXWV (JL) = 0.0_JPRB
        ZXWV5(JL) = REPSCQ
      ENDIF

!- H2O continuum
      ZDIV15 = 1.0_JPRB/(1.0_JPRB+0.608_JPRB*ZXWV5(JL))
      ZFPPW (JL) = 1.6078_JPRB*ZXWV(JL)*ZDIV15*ZDIV15
      ZFPPW5(JL) = 1.6078_JPRB*ZXWV5(JL)*ZDIV15
!- O3
      ZDIV25 = 1.0_JPRB/PDP5(JL,IKL)
      ZXOZZ  = PQOF(JL,IKL)*ZDIV25-PQOF5(JL,IKL)*ZDIV25*ZDIV25*PDP(JL,IKL)
      ZXOZZ5 = PQOF5(JL,IKL)*ZDIV25
      IF (ZXOZZ5 > REPSCO) THEN
        ZXOZ (JL) = ZXOZZ
        ZXOZ5(JL) = ZXOZZ5
      ELSE
        ZXOZ (JL) = 0.0_JPRB
        ZXOZ5(JL) = REPSCO
      ENDIF
    ENDDO
    IKJ=(JK-1)*NG1P1+1
    IKJPN=IKJ+NG1
    DO JKK=IKJ,IKJPN
      DO JL = KIDIA,KFDIA
        ZDPMP0  = ZDPM (JL,JKK) * ZINORMP
        ZDPMP05 = ZDPM5(JL,JKK) * ZINORMP
        ZUPMCO2  = (ZUPM5(JL,JKK)+RVGCO2)*ZDPMP0+ZDPMP05*ZUPM(JL,JKK)
        ZUPMCO25 = (ZUPM5(JL,JKK)+RVGCO2)*ZDPMP05
        ZUPMH2O  = (ZUPM5(JL,JKK)+RVGH2O)*ZDPMP0+ZDPMP05*ZUPM(JL,JKK)
        ZUPMH2O5 = (ZUPM5(JL,JKK)+RVGH2O)*ZDPMP05

!      ZFPPW = 1.6078_JPRB * ZXWV(JL) / (1.0_JPRB+0.608_JPRB*ZXWV(JL))
!- H2O
        ZABLY (JL,1,JKK) = ZXWV5(JL)*ZUPMH2O+ZUPMH2O5*ZXWV(JL)
        ZABLY5(JL,1,JKK) = ZXWV5(JL)*ZUPMH2O5
!- CO2 (CH4,N2O)
        ZABLY (JL,3,JKK) = PCCO2*ZUPMCO2
        ZABLY5(JL,3,JKK) = PCCO2*ZUPMCO25
!- O3
        ZUPMO3  = (ZUPM5(JL,JKK)+RVGO3)*ZDPMP0+ZDPMP05*ZUPM(JL,JKK)
        ZUPMO35 = (ZUPM5(JL,JKK)+RVGO3)*ZDPMP05
        ZABLY (JL,6,JKK) = ZXOZ5(JL)*ZDPM(JL,JKK)+ZDPM5(JL,JKK)*ZXOZ(JL)
        ZABLY5(JL,6,JKK) = ZXOZ5(JL)*ZDPM5(JL,JKK)
        ZABLY (JL,7,JKK) = ZXOZ5(JL)*ZUPMO3+ZUPMO35*ZXOZ(JL)
        ZABLY5(JL,7,JKK) = ZXOZ5(JL)*ZUPMO35
!- H2O continuum
        ZUPMG  = ZUPM5(JL,JKK)*ZDPMP0+ZDPMP05*ZUPM(JL,JKK)
        ZUPMG5 = ZUPM5(JL,JKK) * ZDPMP05
        ZU6  = ZXWV5(JL)*ZUPMG+ZUPMG5*ZXWV(JL)
        ZU65 = ZXWV5(JL)*ZUPMG5
        ZABLY (JL,5,JKK) = ZU65*ZFPPW(JL)+ZFPPW5(JL)*ZU6
        ZABLY5(JL,5,JKK) = ZU65*ZFPPW5(JL)
        ZABLY (JL,4,JKK) = (1.0_JPRB-ZFPPW5(JL))*ZU6-ZU65*ZFPPW(JL)
        ZABLY5(JL,4,JKK) = ZU65*(1.0_JPRB-ZFPPW5(JL))

        ZABLY (JL,2,JKK) = PCCO2*ZDPM(JL,JKK)
        ZABLY5(JL,2,JKK) = PCCO2*ZDPM5(JL,JKK)
      ENDDO
    ENDDO
  ENDDO

ENDIF

!-----------------------------------------------------------------------

!*         5.    CUMULATIVE ABSORBER AMOUNTS FROM TOP OF ATMOSPHERE
!                --------------------------------------------------

DO JA = 1, NUA
  DO JL = KIDIA,KFDIA
    PABCU (JL,JA,3*KLEV+1) = 0.0_JPRB
    PABCU5(JL,JA,3*KLEV+1) = 0.0_JPRB
  ENDDO
ENDDO

DO JK = 1 , KLEV
  IJ=(JK-1)*NG1P1+1
  IJPN=IJ+NG1
  IKL=KLEV+1-JK

!*         5.1  CUMULATIVE AEROSOL AMOUNTS FROM TOP OF ATMOSPHERE
!               --------------------------------------------------
! --            NB: 'PAER' AEROSOLS ARE ENTERED FROM TOP TO BOTTOM

  IAE1=3*KLEV+1-IJ
  IAE2=3*KLEV+1-(IJ+1)
  IAE3=3*KLEV+1-IJPN
  DO JL = KIDIA,KFDIA
    ZSUM (JL) = ZDPM (JL,IAE1)+ZDPM (JL,IAE2)+ZDPM (JL,IAE3)
    ZSUM5(JL) = ZDPM5(JL,IAE1)+ZDPM5(JL,IAE2)+ZDPM5(JL,IAE3)
    ZDIV5(JL) = 1.0_JPRB/ZSUM5(JL)
    ZDIV (JL) = -ZSUM(JL)*ZDIV5(JL)*ZDIV5(JL)
  ENDDO

  DO JAE=1,6
    DO JL = KIDIA,KFDIA
      ZUAERS (JL,JAE) =&
       &  RAER(JAE,1)*PAER (JL,1,JK)+RAER(JAE,2)*PAER (JL,2,JK)&
       & +RAER(JAE,3)*PAER (JL,3,JK)+RAER(JAE,4)*PAER (JL,4,JK)&
       & +RAER(JAE,5)*PAER (JL,5,JK)+RAER(JAE,6)*PAER (JL,6,JK)
      ZUAERS5(JL,JAE) =&
       &  RAER(JAE,1)*PAER5(JL,1,JK)+RAER(JAE,2)*PAER5(JL,2,JK)&
       & +RAER(JAE,3)*PAER5(JL,3,JK)+RAER(JAE,4)*PAER5(JL,4,JK)&
       & +RAER(JAE,5)*PAER5(JL,5,JK)+RAER(JAE,6)*PAER5(JL,6,JK)
      ZUAER (JL,JAE) = ZUAERS5(JL,JAE)*ZDIV(JL)+ZDIV5(JL)*ZUAERS(JL,JAE)
      ZUAER5(JL,JAE) = ZUAERS5(JL,JAE)*ZDIV5(JL)
    ENDDO
  ENDDO

!*         5.2  INTRODUCES TEMPERATURE EFFECTS ON ABSORBER AMOUNTS
!               --------------------------------------------------

  DO JL = KIDIA,KFDIA
!    ZFACT(JL)=1.0_JPRB-ZTAVI(JL)/296._JPRB
!    ZTCON(JL)=EXP(6.08*ZFACT(JL))
    ZTX (JL) = PTAVE (JL,IKL)
    ZTX5(JL) = PTAVE5(JL,IKL)-TREF
    ZTX2 (JL) = 2.0_JPRB*ZTX5(JL)*ZTX(JL)
    ZTX25(JL) = ZTX5(JL)*ZTX5(JL)
    ZZABLY  = ZABLY (JL,1,IAE1)+ZABLY (JL,1,IAE2)+ZABLY (JL,1,IAE3)
    ZZABLY5 = ZABLY5(JL,1,IAE1)+ZABLY5(JL,1,IAE2)+ZABLY5(JL,1,IAE3)
    ZLOG15 = 0.5_JPRB*R10E*LOG(ZZABLY5)+5._JPRB
    IF (ZLOG15 > 0.0_JPRB) THEN
      ZUP1  = 0.5_JPRB*R10E*ZZABLY/ZZABLY5
      ZUP15 = ZLOG15
    ELSE
      ZUP1  = 0.0_JPRB
      ZUP15 = 0.0_JPRB
    ENDIF
    IF (ZUP15 < 6.0_JPRB) THEN
      ZUP  = ZUP1
      ZUP5 = ZUP15
    ELSE
      ZUP  = 0.0_JPRB
      ZUP5 = 6.0_JPRB
    ENDIF

!- H2O
    ZCAH1  = ZUP*(ALWT(1,2)+2*ALWT(1,3)*ZUP5)
    ZCAH15 = ALWT(1,1)+ZUP5*(ALWT(1,2)+ZUP5*ALWT(1,3))
    ZCBH1  = ZUP*(BLWT(1,2)+2*BLWT(1,3)*ZUP5)
    ZCBH15 = BLWT(1,1)+ZUP5*(BLWT(1,2)+ZUP5*BLWT(1,3))
    ZPSH15(JL) = EXP(ZCAH15*ZTX5(JL) + ZCBH15*ZTX25(JL))
    ZPSH1 (JL) = ZPSH15(JL)*(ZCAH15*ZTX(JL)+ZTX5(JL)*ZCAH1 &
     & + ZCBH15*ZTX2(JL)+ZTX25(JL)*ZCBH1)
    ZCAH2  = ZUP*(ALWT(2,2)+2*ALWT(2,3)*ZUP5)
    ZCAH25 = ALWT(2,1)+ZUP5*(ALWT(2,2)+ZUP5*ALWT(2,3))
    ZCBH2  = ZUP*(BLWT(2,2)+2*BLWT(2,3)*ZUP5)
    ZCBH25 = BLWT(2,1)+ZUP5*(BLWT(2,2)+ZUP5*BLWT(2,3))
    ZPSH25(JL) = EXP(ZCAH25*ZTX5(JL) + ZCBH25*ZTX25(JL))
    ZPSH2 (JL) = ZPSH25(JL)*(ZCAH25*ZTX(JL)+ZTX5(JL)*ZCAH2 &
     & + ZCBH25*ZTX2(JL)+ZTX25(JL)*ZCBH2)
    ZCAH3  = ZUP*(ALWT(3,2)+2*ALWT(3,3)*ZUP5)
    ZCAH35 = ALWT(3,1)+ZUP5*(ALWT(3,2)+ZUP5*ALWT(3,3))
    ZCBH3  = ZUP*(BLWT(3,2)+2*BLWT(3,3)*ZUP5)
    ZCBH35 = BLWT(3,1)+ZUP5*(BLWT(3,2)+ZUP5*BLWT(3,3))
    ZPSH35(JL) = EXP(ZCAH35*ZTX5(JL)+ZCBH35*ZTX25(JL))
    ZPSH3 (JL) = ZPSH35(JL)*(ZCAH35*ZTX(JL)+ZTX5(JL)*ZCAH3 &
     & + ZCBH35*ZTX2(JL)+ZTX25(JL)*ZCBH3)
    ZCAH4  = ZUP*(ALWT(4,2)+2*ALWT(4,3)*ZUP5)
    ZCAH45 = ALWT(4,1)+ZUP5*(ALWT(4,2)+ZUP5*ALWT(4,3))
    ZCBH4  = ZUP*(BLWT(4,2)+2*BLWT(4,3)*ZUP5)
    ZCBH45 = BLWT(4,1)+ZUP5*(BLWT(4,2)+ZUP5*BLWT(4,3))
    ZPSH45(JL) = EXP(ZCAH45*ZTX5(JL) + ZCBH45*ZTX25(JL))
    ZPSH4 (JL) = ZPSH45(JL)*(ZCAH45*ZTX(JL)+ZTX5(JL)*ZCAH4 &
     & + ZCBH45*ZTX2(JL)+ZTX25(JL)*ZCBH4)
    ZCAH5  = ZUP*(ALWT(5,2)+2*ALWT(5,3)*ZUP5)
    ZCAH55 = ALWT(5,1)+ZUP5*(ALWT(5,2)+ZUP5*ALWT(5,3))
    ZCBH5  = ZUP*(BLWT(5,2)+2*BLWT(5,3)*ZUP5)
    ZCBH55 = BLWT(5,1)+ZUP5*(BLWT(5,2)+ZUP5*BLWT(5,3))
    ZPSH55(JL) = EXP(ZCAH55*ZTX5(JL) + ZCBH55*ZTX25(JL))
    ZPSH5 (JL) = ZPSH55(JL)*(ZCAH55*ZTX(JL)+ZTX5(JL)*ZCAH5 &
     & + ZCBH55*ZTX2(JL)+ZTX25(JL)*ZCBH5)
    ZCAH6  = ZUP*(ALWT(6,2)+2*ALWT(6,3)*ZUP5)
    ZCAH65 = ALWT(6,1)+ZUP5*(ALWT(6,2)+ZUP5*ALWT(6,3))
    ZCBH6  = ZUP*(BLWT(6,2)+2*BLWT(6,3)*ZUP5)
    ZCBH65 = BLWT(6,1)+ZUP5*(BLWT(6,2)+ZUP5*BLWT(6,3))
    ZPSH65(JL) = EXP(ZCAH65*ZTX5(JL) + ZCBH65*ZTX25(JL))
    ZPSH6 (JL) = ZPSH65(JL)*(ZCAH65*ZTX(JL)+ZTX5(JL)*ZCAH6 &
     & + ZCBH65*ZTX2(JL)+ZTX25(JL)*ZCBH6)

    IF (.NOT.LH2OCO2) THEN
!- CH4
      ZPHM65(JL) = EXP(-5.81E-4_JPRB*ZTX5(JL) - 1.13E-6_JPRB*ZTX25(JL))
      ZPHM6 (JL) = ZPHM65(JL)*(-5.81E-4_JPRB*ZTX(JL)-1.13E-6_JPRB*ZTX2(JL))
      ZPSM65(JL) = EXP(-5.57E-4_JPRB*ZTX5(JL) - 3.30E-6_JPRB*ZTX25(JL))
      ZPSM6 (JL) = ZPSM65(JL)*(-5.57E-4_JPRB*ZTX(JL)-3.30E-6_JPRB*ZTX2(JL))
!- N2O
      ZPHN65(JL) = EXP(-3.46E-5_JPRB*ZTX5(JL) + 2.05E-7_JPRB*ZTX25(JL))
      ZPHN6 (JL) = ZPHN65(JL)*(-3.46E-5_JPRB*ZTX(JL)+2.05E-7_JPRB*ZTX2(JL))
      ZPSN65(JL) = EXP(3.70E-3_JPRB*ZTX5(JL)-2.30E-6_JPRB*ZTX25(JL))
      ZPSN6 (JL) = ZPSN65(JL)*(3.70E-3_JPRB*ZTX(JL)-2.30E-6_JPRB*ZTX2(JL))
    ENDIF
  ENDDO

  DO JL = KIDIA,KFDIA
    ZZABLY  = ZABLY (JL,3,IAE1)+ZABLY (JL,3,IAE2)+ZABLY (JL,3,IAE3)
    ZZABLY5 = ZABLY5(JL,3,IAE1)+ZABLY5(JL,3,IAE2)+ZABLY5(JL,3,IAE3)
    ZALUP  = R10E*ZZABLY/ZZABLY5
    ZALUP5 = R10E * LOG(ZZABLY5)
    ZUP25 = 5.0_JPRB+0.5_JPRB*ZALUP5
    IF (ZUP25 > 0.0_JPRB) THEN
      ZUP  = 0.5_JPRB*ZALUP
      ZUP5 = ZUP25
    ELSE
      ZUP  = 0.0_JPRB
      ZUP5 = 0.0_JPRB
    ENDIF
!- CO2
    ZEQ15 = PTAVE5(JL,IKL)/TREF
    ZPSC25 (JL) = ZEQ15 ** ZUP5
    ZPSC2 (JL) = ZPSC25(JL) &
     & *(LOG(ZEQ15)*ZUP+ZUP5*PTAVE(JL,IKL)/PTAVE5(JL,IKL))
    ZCAC8  = ZUP*(ALWT(8,2)+2*ALWT(8,3)*ZUP5)
    ZCAC85 = ALWT(8,1)+ZUP5*(ALWT(8,2)+ZUP5*ALWT(8,3))
    ZCBC8  = ZUP*(BLWT(8,2)+2*BLWT(8,3)*ZUP5)
    ZCBC85 = BLWT(8,1)+ZUP5*(BLWT(8,2)+ZUP5*BLWT(8,3))
    ZPSC35(JL) = EXP(ZCAC85*ZTX5(JL)+ZCBC85*ZTX25(JL))
    ZPSC3 (JL) = ZPSC35(JL)*(ZCAC85*ZTX(JL)+ZTX5(JL)*ZCAC8 &
     & + ZCBC85*ZTX2(JL)+ZTX25(JL)*ZCBC8)

    IF (.NOT.LH2OCO2) THEN
!- O3
      ZPHIO5(JL) = EXP( RO3T(1)*ZTX5(JL) + RO3T(2)*ZTX25(JL))
      ZPHIO (JL) = ZPHIO5(JL)*(RO3T(1)*ZTX(JL)+RO3T(2)*ZTX2(JL))
      ZPSIO5(JL) = EXP(2.0_JPRB*(RO3T(3)*ZTX5(JL)+RO3T(4)*ZTX25(JL)))
      ZPSIO (JL) = ZPSIO5(JL)*(2.0_JPRB*(RO3T(3)*ZTX(JL)+RO3T(4)*ZTX2(JL)))
    ENDIF
  ENDDO

  DO JKK=IJ,IJPN
    IC=3*KLEV+1-JKK
    ICP1=IC+1
    DO JL = KIDIA,KFDIA
      ZDIFFCO2 = ZIPCCO2*PVIEW(JL)
      ZDUCM  = ZDPM (JL,IC)*PVIEW(JL)
      ZDUCM5 = ZDPM5(JL,IC)*PVIEW(JL)
!- CO2
      PABCU (JL,7,IC) = PABCU(JL,7,ICP1)+PVIEW(JL) &
       & *(ZABLY5(JL,3,IC)*ZPSC2(JL)+ZPSC25(JL)*ZABLY(JL,3,IC))
      PABCU5(JL,7,IC) = PABCU5(JL,7,ICP1) &
       & + ZABLY5(JL,3,IC)*ZPSC25(JL)*PVIEW(JL)
      PABCU (JL,8,IC) = PABCU (JL,8,ICP1)+PVIEW(JL) &
       & *(ZABLY5(JL,3,IC)*ZPSC3(JL)+ZPSC35(JL)*ZABLY(JL,3,IC))
      PABCU5(JL,8,IC) = PABCU5(JL,8,ICP1) &
       & + ZABLY5(JL,3,IC)*ZPSC35(JL)*PVIEW(JL)
      PABCU (JL,9,IC) = PABCU(JL,9,ICP1)+PVIEW(JL) &
       & *(ZABLY5(JL,3,IC)*ZPSC3(JL)+ZPSC35(JL)*ZABLY(JL,3,IC))
      PABCU5(JL,9,IC) = PABCU5(JL,9,ICP1) &
       & + ZABLY5(JL,3,IC)*ZPSC35(JL)*PVIEW(JL)
!- H2O
      PABCU (JL,1,IC) = PABCU (JL,1,ICP1)+ZABLY5(JL,1,IC)*ZPSH1(JL) &
       & + ZPSH15(JL)*ZABLY(JL,1,IC)
      PABCU5(JL,1,IC) = PABCU5(JL,1,ICP1)+ZABLY5(JL,1,IC)*ZPSH15(JL)
      PABCU (JL,2,IC) = PABCU (JL,2,ICP1)+ZABLY5(JL,1,IC)*ZPSH2(JL) &
       & + ZPSH25(JL)*ZABLY(JL,1,IC)
      PABCU5(JL,2,IC) = PABCU5(JL,2,ICP1)+ZABLY5(JL,1,IC)*ZPSH25(JL)
      PABCU (JL,3,IC) = PABCU (JL,3,ICP1)+PVIEW(JL) &
       & *(ZABLY5(JL,1,IC)*ZPSH5(JL)+ZPSH55(JL)*ZABLY(JL,1,IC))
      PABCU5(JL,3,IC) = PABCU5(JL,3,ICP1) &
       & + ZABLY5(JL,1,IC)*ZPSH55(JL)*PVIEW(JL)
      PABCU (JL,4,IC) = PABCU (JL,4,ICP1)+ZABLY5(JL,1,IC)*ZPSH3(JL) &
       & + ZPSH35(JL)*ZABLY(JL,1,IC)
      PABCU5(JL,4,IC) = PABCU5(JL,4,ICP1)+ZABLY5(JL,1,IC)*ZPSH35(JL)
      PABCU (JL,5,IC) = PABCU (JL,5,ICP1)+ZABLY5(JL,1,IC)*ZPSH4(JL) &
       & +ZPSH45(JL)*ZABLY(JL,1,IC)
      PABCU5(JL,5,IC) = PABCU5(JL,5,ICP1)+ZABLY5(JL,1,IC)*ZPSH45(JL)
      PABCU (JL,6,IC) = PABCU (JL,6,ICP1)+PVIEW(JL) &
       & *(ZABLY5(JL,1,IC)*ZPSH6(JL)+ZPSH65(JL)*ZABLY(JL,1,IC))
      PABCU5(JL,6,IC) = PABCU5(JL,6,ICP1) &
       & + ZABLY5(JL,1,IC)*ZPSH65(JL)*PVIEW(JL)

      IF (.NOT.LH2OCO2) THEN
!- H2O continuum
        ZDIV35 = 1.0_JPRB/PTAVE5(JL,IKL)
        ZTCON5(JL) = EXP(6.08_JPRB*(296._JPRB*ZDIV35-1.0_JPRB))
        ZTCON (JL) = -ZTCON5(JL)*1799.68_JPRB*ZDIV35*ZDIV35*PTAVE(JL,IKL)
        PABCU (JL,10,IC) = PABCU (JL,10,ICP1)+ZABLY (JL,4,IC)*PVIEW(JL)
        PABCU5(JL,10,IC) = PABCU5(JL,10,ICP1)+ZABLY5(JL,4,IC)*PVIEW(JL)
        PABCU (JL,11,IC) = PABCU (JL,11,ICP1)+PVIEW(JL) &
         & *(ZABLY5(JL,5,IC)*ZTCON(JL)+ZTCON5(JL)*ZABLY(JL,5,IC))
        PABCU5(JL,11,IC) = PABCU5(JL,11,ICP1) &
         & + ZABLY5(JL,5,IC)*ZTCON5(JL)*PVIEW(JL)
!- O3
        PABCU (JL,12,IC) = PABCU (JL,12,ICP1)+PVIEW(JL) &
         & *(ZABLY5(JL,6,IC)*ZPHIO(JL)+ZPHIO5(JL)*ZABLY(JL,6,IC))
        PABCU5(JL,12,IC) = PABCU5(JL,12,ICP1) &
         & + ZABLY5(JL,6,IC)*ZPHIO5(JL)*PVIEW(JL)
        PABCU (JL,13,IC) = PABCU (JL,13,ICP1)+PVIEW(JL) &
         & *(ZABLY5(JL,7,IC)*ZPSIO(JL)+ZPSIO5(JL)*ZABLY(JL,7,IC))
        PABCU5(JL,13,IC) = PABCU5(JL,13,ICP1) &
         & + ZABLY5(JL,7,IC)*ZPSIO5(JL)*PVIEW(JL)
!- aerosols
        PABCU (JL,14,IC) = PABCU (JL,14,ICP1)+ZUAER5(JL,1)*ZDUCM &
         & + ZDUCM5*ZUAER(JL,1)
        PABCU5(JL,14,IC) = PABCU5(JL,14,ICP1)+ZUAER5(JL,1)*ZDUCM5
        PABCU (JL,15,IC) = PABCU (JL,15,ICP1)+ZUAER5(JL,2)*ZDUCM &
         & + ZDUCM5*ZUAER(JL,2)
        PABCU5(JL,15,IC) = PABCU5(JL,15,ICP1)+ZUAER5(JL,2)*ZDUCM5
        PABCU (JL,16,IC) = PABCU (JL,16,ICP1)+ZUAER5(JL,3)*ZDUCM &
         & + ZDUCM5*ZUAER(JL,3)
        PABCU5(JL,16,IC) = PABCU5(JL,16,ICP1)+ZUAER5(JL,3)*ZDUCM5
        PABCU (JL,17,IC) = PABCU (JL,17,ICP1)+ZUAER5(JL,4)*ZDUCM &
         & + ZDUCM5*ZUAER(JL,4)
        PABCU5(JL,17,IC) = PABCU5(JL,17,ICP1)+ZUAER5(JL,4)*ZDUCM5
        PABCU (JL,18,IC) = PABCU (JL,18,ICP1)+ZUAER5(JL,5)*ZDUCM &
         & + ZDUCM5*ZUAER(JL,5)
        PABCU5(JL,18,IC) = PABCU5(JL,18,ICP1)+ZUAER5(JL,5)*ZDUCM5
!- CH4
        PABCU (JL,19,IC) = PABCU (JL,19,ICP1)+RCH4*ZDIFFCO2 &
         & *(ZABLY5(JL,2,IC)*ZPHM6(JL)+ZPHM65(JL)*ZABLY(JL,2,IC))
        PABCU5(JL,19,IC) = PABCU5(JL,19,ICP1)&
         & + ZABLY5(JL,2,IC)*RCH4*ZPHM65(JL)*ZDIFFCO2
        PABCU (JL,20,IC) = PABCU (JL,20,ICP1)+RCH4*ZDIFFCO2 &
         & *(ZABLY5(JL,3,IC)*ZPSM6(JL)+ZPSM65(JL)*ZABLY(JL,3,IC))
        PABCU5(JL,20,IC) = PABCU5(JL,20,ICP1)&
         & + ZABLY5(JL,3,IC)*RCH4*ZPSM65(JL)*ZDIFFCO2
!- N2O
        PABCU (JL,21,IC) = PABCU (JL,21,ICP1)+RN2O*ZDIFFCO2 &
         & *(ZABLY5(JL,2,IC)*ZPHN6(JL)+ZPHN65(JL)*ZABLY(JL,2,IC))
        PABCU5(JL,21,IC) = PABCU5(JL,21,ICP1)&
         & + ZABLY5(JL,2,IC)*RN2O*ZPHN65(JL)*ZDIFFCO2 
        PABCU (JL,22,IC) = PABCU (JL,22,ICP1)+RN2O*ZDIFFCO2 &
         & *(ZABLY5(JL,3,IC)*ZPSN6(JL)+ZPSN65(JL)*ZABLY(JL,3,IC))
        PABCU5(JL,22,IC) = PABCU5(JL,22,ICP1)&
         & + ZABLY5(JL,3,IC)*RN2O*ZPSN65(JL)*ZDIFFCO2
!- CFC11
        PABCU (JL,23,IC) = PABCU (JL,23,ICP1)&
         & + ZABLY (JL,2,IC)*RCFC11*ZDIFFCO2
        PABCU5(JL,23,IC) = PABCU5(JL,23,ICP1)&
         & + ZABLY5(JL,2,IC)*RCFC11*ZDIFFCO2
!- CFC12
        PABCU (JL,24,IC) = PABCU (JL,24,ICP1)&
         & + ZABLY (JL,2,IC)*RCFC12*ZDIFFCO2
        PABCU5(JL,24,IC) = PABCU5(JL,24,ICP1)&
         & + ZABLY5(JL,2,IC)*RCFC12*ZDIFFCO2
      ENDIF

    ENDDO
  ENDDO

ENDDO
!      print *,'END OF LWU'

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWUTL',1,ZHOOK_HANDLE)
END SUBROUTINE LWUTL
