#ifdef RS6K
@PROCESS HOT  
#endif
SUBROUTINE LWVDRTL &
 & ( KIDIA, KFDIA, KLON , KLEV  , KTRAER,&
 & PGA    , PGB    ,&
 & PABCU5 , PDBDT5 ,&
 & PCNTRB5, PDISD5 , PDISU5, PDWFSU5, &
 & PABCU  , PDBDT  ,&
 & PCNTRB , PDISD  , PDISU , PDWFSU &
 & )

!**** *LWVDRTL*   - L.W., VERTICAL INTEGRATION, DISTANT LAYERS
!                      (tangent linear)

!     PURPOSE.
!     --------
!           CARRIES OUT THE VERTICAL INTEGRATION ON THE DISTANT LAYERS

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!  Trajectory  Perturbation  Description                    Dimension
!  PABCU5      PABCU         ABSORBER AMOUNTS               (KLON,NUA,3*KLEV+1)
!  PDBDT5      PDBDT         LAYER PLANCK FUNCTION GRADIENT (KLON,KLEV)
!  PGA         ---           PADE APPROXIMANTS              (KLON,NIPD,2)
!  PGB         ---           PADE APPROXIMANTS              (KLON,NIPD,2)

!     ==== OUTPUTS ===
!  Trajectory  Perturbation  Description                       Dimension
!  PCNTRB5     PCNTRB        ENERGY EXCHANGE MATRIX        (KLON,KLEV+1,KLEV+1)
!  PDISD5      PDISD         CONTRIBUTION BY DISTANT LAYERS    (KLON,KLEV+1)
!  PDISU5      PDISU         CONTRIBUTION BY DISTANT LAYERS    (KLON,KLEV+1)
!  PDWFSU5     PDWFSU        SPECTRAL DOWNWARD FLUX AT SURFACE (KLON,NSIL)

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. PERFORMS THE VERTICAL INTEGRATION CORRESPONDING TO THE
!     CONTRIBUTIONS OF THE DISTANT LAYERS USING TRAPEZOIDAL RULE

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        M. Janiskova  *ECMWF*       27-Jan-2006

!     MODIFICATIONS.
!     --------------
!        M.Janiskova   23-Nov-2006 reduced version for H2O and CO2
!                                  based on lwvdtl
!        M.Janiskova   23-Jan-2008 condition PCOND for reduced loops
!        J.Hague       08-Feb-2008 code optimization

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOELW    , ONLY : NSIL     ,NIPD     ,NUA      ,NG1P1

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTRAER 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGA(KLON,NIPD,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGB(KLON,NIPD,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU5(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDBDT5(KLON,NSIL,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCNTRB5(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDISD5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDISU5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDWFSU5(KLON,NSIL) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDBDT(KLON,NSIL,KLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCNTRB(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDISD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDISU(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDWFSU(KLON,NSIL)

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZTT(KLON,KTRAER), ZTTP(KLON,KTRAER,KLEV+1)
REAL(KIND=JPRB) :: ZTT5(KLON,KTRAER), ZTTP5(KLON,KTRAER,KLEV+1)
REAL(KIND=JPRB) :: ZZCOND5(KLON)

REAL(KIND=JPRB) :: ZWW1, ZWW2, ZWW3, ZWW4, ZWW5, ZWW6
REAL(KIND=JPRB) :: ZWW15, ZWW25, ZWW35, ZWW45, ZWW55, ZWW65
REAL(KIND=JPRB) :: ZWD1, ZWD2, ZWD3, ZWD4, ZWD5, ZWD6
REAL(KIND=JPRB) :: ZWD15, ZWD25, ZWD35, ZWD45, ZWD55, ZWD65
REAL(KIND=JPRB) :: ZXN, ZXD, ZZ
REAL(KIND=JPRB) :: ZXN5, ZXD5, ZZ5
REAL(KIND=JPRB) :: ZXDIV5
REAL(KIND=JPRB) :: ZZX5

LOGICAL :: LL_DO(KLON)
INTEGER(KIND=JPIM) :: IC
INTEGER(KIND=JPIM) :: JJ, JN, JX(KLON)

INTEGER(KIND=JPIM) :: IJKL, IKD1, IKD2, IKJ, IKJP1, IKM1, IKN,&
 & IKP1, IKU1, IKU2, JA, JK, JKJ, JL, JLK, IKJ1 

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!#include "lwttmtl.intfb.h"

!-----------------------------------------------------------------------

!*         1.    INITIALIZATION
!                --------------

!*         1.1     INITIALIZE LAYER CONTRIBUTIONS
!                  ------------------------------

IF (LHOOK) CALL DR_HOOK('LWVDRTL',0,ZHOOK_HANDLE)
DO JK = 1, KLEV+1
  DO JL = KIDIA,KFDIA
    PDISD (JL,JK) = 0.0_JPRB
    PDISD5(JL,JK) = 0.0_JPRB
    PDISU (JL,JK) = 0.0_JPRB
    PDISU5(JL,JK) = 0.0_JPRB
  ENDDO
ENDDO

!*         1.2     INITIALIZE TRANSMISSION FUNCTIONS
!                  ---------------------------------

!DO JA = 1, NTRA
DO JA = 1,KTRAER
  DO JL = KIDIA,KFDIA
    ZTT (JL,JA) = 0.0_JPRB
    ZTT5(JL,JA) = 1.0_JPRB
    ZTTP (JL,JA,:) = 0.0_JPRB
    ZTTP5(JL,JA,:) = 1.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         2.      VERTICAL INTEGRATION
!                  --------------------

!*         2.2     CONTRIBUTION FROM DISTANT LAYERS
!                  ---------------------------------

!*         2.2.1   DISTANT AND ABOVE LAYERS
!                  ------------------------

!*         2.2.2   FIRST UPPER LEVEL
!                  -----------------

DO JK = KLEV-1,1,-1
  IKP1=JK+1
  IKN=(JK-1)*NG1P1+1
  IKD1= JK  *NG1P1+1

  DO JA = 1 , 8
    DO JL = KIDIA,KFDIA
!      ZZ5 = SQRT(PABCU5(JL,JA,IKN)-PABCU5(JL,JA,IKD1))
!      IF (ZZ5 > 0.0_JPRB) THEN
!        ZZ  = 0.5_JPRB*(PABCU(JL,JA,IKN)-PABCU(JL,JA,IKD1))/ZZ5
!      ELSE
!        ZZ = 0.0_JPRB
!      ENDIF
      ZZ5 = SQRT(PABCU5(JL,JA,IKN)-PABCU5(JL,JA,IKD1)+1E-25_JPRB)
      ZZ  = 0.5_JPRB*(PABCU(JL,JA,IKN)-PABCU(JL,JA,IKD1))/ZZ5
      ZXD  = (PGB( JL,JA,2,JK)+2.0_JPRB*ZZ5)*ZZ
      ZXD5 = PGB( JL,JA,1,JK) + ZZ5* (PGB( JL,JA,2,JK) + ZZ5 )
      ZXN  = ZZ*PGA( JL,JA,2,JK)
      ZXN5 = PGA( JL,JA,1,JK) + ZZ5*PGA( JL,JA,2,JK)
      ZXDIV5 = 1.0_JPRB/ZXD5
      ZTTP (JL,JA,1) = ZXN*ZXDIV5 - ZXN5*ZXDIV5*ZXDIV5*ZXD
      ZTTP5(JL,JA,1) = ZXN5*ZXDIV5
    ENDDO
  ENDDO

  DO JL = KIDIA,KFDIA
    IF (ZTTP5(JL,3,1) < 0.0_JPRB) THEN
      ZTTP (JL,3,1) = 0.0_JPRB
      ZTTP5(JL,3,1) = 0.0_JPRB
    ENDIF
    ZTTP (JL,9,1) = ZTTP (JL,8,1)
    ZTTP5(JL,9,1) = ZTTP5(JL,8,1)
  ENDDO

!*         2.2.3   HIGHER UP
!                  ---------

  DO JKJ=IKP1,KLEV
    IKJP1=JKJ+1
    IKD2= JKJ  *NG1P1+1
    IKJ1=JKJ-JK

    DO JL = KIDIA,KFDIA
!      IF (JK > 1 .AND. JK < KLEV-2 .AND. (IKJP1-JK) > 3) THEN
      IF (JK > 1 .AND. (IKJP1-JK) > 3) THEN
        ZZCOND5(JL) = ABS(PCNTRB5(JL,JK+2,IKJP1)-PCNTRB5(JL,JK+1,IKJP1))
      ELSE
        ZZCOND5(JL) = 9999._JPRB
      ENDIF
    ENDDO

    IC=0
    LL_DO(:)=.TRUE.
    DO JL = KIDIA,KFDIA
!exp_A
!      IF (ZZCOND(JL) < 0.0001_JPRB) THEN
!exp_B
!      IF (ZZCOND(JL) < 0.00001_JPRB) THEN
!exp_C
      IF (ZZCOND5(JL) < 0.001_JPRB) THEN
        PCNTRB (JL,JK,IKJP1) = PCNTRB (JL,JK+1,IKJP1)
        PCNTRB5(JL,JK,IKJP1) = PCNTRB5(JL,JK+1,IKJP1)
        PDISD (JL,JK) = PDISD (JL,JK)+PCNTRB (JL,JK,IKJP1)
        PDISD5(JL,JK) = PDISD5(JL,JK)+PCNTRB5(JL,JK,IKJP1)
        IC=IC+1
        LL_DO(JL)=.FALSE.
      ENDIF
    ENDDO

    JJ=0
    DO JL = KIDIA,KFDIA
      IF (LL_DO(JL)) THEN
        JJ=JJ+1
        JX(JJ)=JL
      ENDIF
    ENDDO
    JN=JJ


    IF (IC /= KFDIA-KIDIA+1) THEN

      DO JA = 1 , 8, 2
        DO JJ=1,JN
          JL=JX(JJ)
!            ZZ5 = SQRT(PABCU5(JL,JA,IKN)-PABCU5(JL,JA,IKD2))
!            IF (ZZ5 > 0.0_JPRB) THEN
!              ZZ  = 0.5_JPRB*(PABCU(JL,JA,IKN)-PABCU(JL,JA,IKD2))/ZZ5
!            ELSE
!              ZZ  = 0.0_JPRB
!            ENDIF
            ZZ5 = SQRT(PABCU5(JL,JA,IKN)-PABCU5(JL,JA,IKD2)+1E-25_JPRB)
            ZZ  = 0.5_JPRB*(PABCU(JL,JA,IKN)-PABCU(JL,JA,IKD2))/ZZ5
            ZXD  = (PGB( JL,JA,2,JKJ)+2.0_JPRB*ZZ5)*ZZ
            ZXD5 = PGB( JL,JA,1,JKJ) + ZZ5* (PGB( JL,JA,2,JKJ) + ZZ5 )
            ZXN  = ZZ*PGA( JL,JA,2,JKJ)
            ZXN5 = PGA( JL,JA,1,JKJ) + ZZ5*PGA( JL,JA,2,JKJ)
            ZXDIV5 = 1.0_JPRB/ZXD5
!            ZTTP (JL,JA,IKJ1+1) = ZXN*ZXDIV5 - ZXN5*ZXDIV5*ZXDIV5*ZXD
!            ZTTP5(JL,JA,IKJ1+1) = ZXN5*ZXDIV5
            ZZX5 = ZXN5*ZXDIV5
            ZTTP (JL,JA,IKJ1+1) = ZXN*ZXDIV5 - ZZX5*ZXDIV5*ZXD
            ZTTP5(JL,JA,IKJ1+1) = ZZX5

            ZZ5 = SQRT(PABCU5(JL,JA+1,IKN)-PABCU5(JL,JA+1,IKD2)+1E-25_JPRB)
            ZZ  = 0.5_JPRB*(PABCU(JL,JA+1,IKN)-PABCU(JL,JA+1,IKD2))/ZZ5
            ZXD  = (PGB( JL,JA+1,2,JKJ)+2.0_JPRB*ZZ5)*ZZ
            ZXD5 = PGB( JL,JA+1,1,JKJ) + ZZ5* (PGB( JL,JA+1,2,JKJ) + ZZ5 )
            ZXN  = ZZ*PGA( JL,JA+1,2,JKJ)
            ZXN5 = PGA( JL,JA+1,1,JKJ) + ZZ5*PGA( JL,JA+1,2,JKJ)
            ZXDIV5 = 1.0_JPRB/ZXD5
!           ZTTP (JL,JA+1,IKJ1+1) = ZXN*ZXDIV5 - ZXN5*ZXDIV5*ZXDIV5*ZXD
!           ZTTP5(JL,JA+1,IKJ1+1) = ZXN5*ZXDIV5
            ZZX5 = ZXN5*ZXDIV5
            ZTTP (JL,JA+1,IKJ1+1) = ZXN*ZXDIV5 - ZZX5*ZXDIV5*ZXD
            ZTTP5(JL,JA+1,IKJ1+1) = ZZX5
        ENDDO
      ENDDO

      DO JJ=1,JN
        JL=JX(JJ)
          IF (ZTTP5(JL,3,IKJ1+1) < 0.0_JPRB) THEN
            ZTTP (JL,3,IKJ1+1) = 0.0_JPRB
            ZTTP5(JL,3,IKJ1+1) = 0.0_JPRB
          ENDIF
          ZTTP (JL,9,IKJ1+1) = ZTTP (JL,8,IKJ1+1)
          ZTTP5(JL,9,IKJ1+1) = ZTTP5(JL,8,IKJ1+1)
      ENDDO

!      DO JA = 1, KTRAER
!        DO JL = KIDIA,KFDIA
!          IF (LL_DO(JL)) THEN
!            ZTT (JL,JA) = (ZTTP (JL,JA,IKJ1)+ZTTP (JL,JA,IKJ1+1))*0.5_JPRB
!            ZTT5(JL,JA) = (ZTTP5(JL,JA,IKJ1)+ZTTP5(JL,JA,IKJ1+1))*0.5_JPRB
!          ENDIF
!        ENDDO
!      ENDDO

      DO JA = 1, KTRAER-1,2
        DO JJ=1,JN
          JL=JX(JJ)
            ZTT (JL,JA) = (ZTTP (JL,JA,IKJ1)+ZTTP (JL,JA,IKJ1+1))*0.5_JPRB
            ZTT5(JL,JA) = (ZTTP5(JL,JA,IKJ1)+ZTTP5(JL,JA,IKJ1+1))*0.5_JPRB
            ZTT (JL,JA+1) =(ZTTP (JL,JA+1,IKJ1)+ZTTP (JL,JA+1,IKJ1+1))*0.5_JPRB
            ZTT5(JL,JA+1) =(ZTTP5(JL,JA+1,IKJ1)+ZTTP5(JL,JA+1,IKJ1+1))*0.5_JPRB
        ENDDO
      ENDDO
      IF (MOD(KTRAER,2) /= 0) THEN
        DO JJ=1,JN
          JL=JX(JJ)
            ZTT (JL,KTRAER) = &
             & (ZTTP (JL,KTRAER,IKJ1)+ZTTP (JL,KTRAER,IKJ1+1))*0.5_JPRB
            ZTT5(JL,KTRAER) = &
             & (ZTTP5(JL,KTRAER,IKJ1)+ZTTP5(JL,KTRAER,IKJ1+1))*0.5_JPRB
        ENDDO
      ENDIF

      DO JJ=1,JN
        JL=JX(JJ)
          ZWW1  = PDBDT5(JL,1,JKJ)*ZTT(JL,1) + ZTT5(JL,1)*PDBDT(JL,1,JKJ)
          ZWW15 = PDBDT5(JL,1,JKJ)*ZTT5(JL,1)
          ZWW2  = PDBDT5(JL,2,JKJ)*ZTT5(JL,2)*ZTT(JL,7) &
           &    + PDBDT5(JL,2,JKJ)*ZTT5(JL,7)*ZTT(JL,2) &
           &    + ZTT5(JL,2)*ZTT5(JL,7)*PDBDT(JL,2,JKJ)
          ZWW25 = PDBDT5(JL,2,JKJ)*ZTT5(JL,2)*ZTT5(JL,7)
          ZWW3  = PDBDT5(JL,3,JKJ)*ZTT5(JL,4)*ZTT(JL,8) &
           &    + PDBDT5(JL,3,JKJ)*ZTT5(JL,8)*ZTT(JL,4) &
           &    + ZTT5(JL,4)*ZTT5(JL,8)*PDBDT(JL,3,JKJ)
          ZWW35 = PDBDT5(JL,3,JKJ)*ZTT5(JL,4)*ZTT5(JL,8)
          ZWW4  = PDBDT5(JL,4,JKJ)*ZTT5(JL,5)*ZTT(JL,9) &
           &    + PDBDT5(JL,4,JKJ)*ZTT5(JL,9)*ZTT(JL,5) &
           &    + ZTT5(JL,5)*ZTT5(JL,9)*PDBDT(JL,4,JKJ)
          ZWW45 = PDBDT5(JL,4,JKJ)*ZTT5(JL,5)*ZTT5(JL,9)
          ZWW5  = PDBDT5(JL,5,JKJ)*ZTT(JL,3) + ZTT5(JL,3)*PDBDT(JL,5,JKJ)
          ZWW55 = PDBDT5(JL,5,JKJ)*ZTT5(JL,3)
          ZWW6  = PDBDT5(JL,6,JKJ)*ZTT(JL,6) + ZTT5(JL,6)*PDBDT(JL,6,JKJ)
          ZWW65 = PDBDT5(JL,6,JKJ)*ZTT5(JL,6)
          PCNTRB (JL,JK,IKJP1) = ZWW1+ZWW2+ZWW3+ZWW4+ZWW5+ZWW6
          PCNTRB5(JL,JK,IKJP1) = ZWW15+ZWW25+ZWW35+ZWW45+ZWW55+ZWW65
          PDISD (JL,JK) = PDISD (JL,JK)+PCNTRB (JL,JK,IKJP1)
          PDISD5(JL,JK) = PDISD5(JL,JK)+PCNTRB5(JL,JK,IKJP1)
          IF (JK == 1) THEN
            PDWFSU (JL,1) = PDWFSU (JL,1)+ZWW1
            PDWFSU5(JL,1) = PDWFSU5(JL,1)+ZWW15
            PDWFSU (JL,2) = PDWFSU (JL,2)+ZWW2
            PDWFSU5(JL,2) = PDWFSU5(JL,2)+ZWW25
            PDWFSU (JL,3) = PDWFSU (JL,3)+ZWW3
            PDWFSU5(JL,3) = PDWFSU5(JL,3)+ZWW35
            PDWFSU (JL,4) = PDWFSU (JL,4)+ZWW4
            PDWFSU5(JL,4) = PDWFSU5(JL,4)+ZWW45
            PDWFSU (JL,5) = PDWFSU (JL,5)+ZWW5
            PDWFSU5(JL,5) = PDWFSU5(JL,5)+ZWW55
            PDWFSU (JL,6) = PDWFSU (JL,6)+ZWW6
            PDWFSU5(JL,6) = PDWFSU5(JL,6)+ZWW65
          ENDIF
      ENDDO

    ENDIF

  ENDDO
ENDDO

!*         2.2.4   DISTANT AND BELOW LAYERS
!                  ------------------------

!*         2.2.5   FIRST LOWER LEVEL
!                  -----------------

DO JK=3,KLEV+1
  IKN=(JK-1)*NG1P1+1
  IKM1=JK-1
  IKJ=JK-2
  IKU1= IKJ  *NG1P1+1

  DO JA = 1 , 8
    DO JL = KIDIA,KFDIA
!      ZZ5 = SQRT(PABCU5(JL,JA,IKU1)-PABCU5(JL,JA,IKN))
!      IF (ZZ5 > 0.0_JPRB) THEN
!        ZZ  = 0.5_JPRB*(PABCU(JL,JA,IKU1)-PABCU(JL,JA,IKN))/ZZ5
!      ELSE
!        ZZ  = 0.0_JPRB
!      ENDIF
      ZZ5 = SQRT(PABCU5(JL,JA,IKU1)-PABCU5(JL,JA,IKN)+1E-25_JPRB)
      ZZ  = 0.5_JPRB*(PABCU(JL,JA,IKU1)-PABCU(JL,JA,IKN))/ZZ5
      ZXD  = (PGB( JL,JA,2,IKJ)+2.0_JPRB*ZZ5)*ZZ
      ZXD5 = PGB( JL,JA,1,IKJ) + ZZ5* (PGB( JL,JA,2,IKJ) + ZZ5 )
      ZXN  = ZZ*PGA( JL,JA,2,IKJ)
      ZXN5 = PGA( JL,JA,1,IKJ) + ZZ5*PGA( JL,JA,2,IKJ)
      ZXDIV5 = 1.0_JPRB/ZXD5
      ZTTP (JL,JA,1) = ZXN*ZXDIV5 - ZXN5*ZXDIV5*ZXDIV5*ZXD
      ZTTP5(JL,JA,1) = ZXN5*ZXDIV5
    ENDDO
  ENDDO

  DO JL = KIDIA,KFDIA
    IF (ZTTP5(JL,3,1) < 0.0_JPRB) THEN
      ZTTP (JL,3,1) = 0.0_JPRB
      ZTTP5(JL,3,1) = 0.0_JPRB
    ENDIF
    ZTTP (JL,9,1) = ZTTP (JL,8,1)
    ZTTP5(JL,9,1) = ZTTP5(JL,8,1)
  ENDDO
  
!*         2.2.6   DOWN BELOW
!                  ----------

  DO JLK=1,IKJ
    IJKL=IKM1-JLK
    IKU2=(IJKL-1)*NG1P1+1

    DO JL = KIDIA,KFDIA
!      IF (JK > 4 .AND. (JK-IJKL) > 3) THEN
      IF ((JK-IJKL) > 3) THEN
        ZZCOND5(JL) = ABS(PCNTRB5(JL,JK-1,IJKL)-PCNTRB5(JL,JK-2,IJKL))
      ELSE
        ZZCOND5(JL) = 9999._JPRB
      ENDIF
    ENDDO

    IC=0
    LL_DO(:)=.TRUE.
    DO JL = KIDIA,KFDIA
!exp_A
!      IF (ZZCOND(JL) < 0.0001_JPRB) THEN
!exp_B
!      IF (ZZCOND(JL) < 0.00001_JPRB) THEN
!exp_C
      IF (ZZCOND5(JL) < 0.001_JPRB) THEN
        IC=IC+1
        LL_DO(JL)=.FALSE.
        PCNTRB (JL,JK,IJKL) = PCNTRB (JL,JK-1,IJKL)
        PCNTRB5(JL,JK,IJKL) = PCNTRB5(JL,JK-1,IJKL)
        PDISU (JL,JK) = PDISU (JL,JK)+PCNTRB (JL,JK,IJKL)
        PDISU5(JL,JK) = PDISU5(JL,JK)+PCNTRB5(JL,JK,IJKL)
      ENDIF
    ENDDO

    JJ=0
    DO JL = KIDIA,KFDIA
      IF (LL_DO(JL)) THEN
        JJ=JJ+1
        JX(JJ)=JL
      ENDIF
    ENDDO
    JN=JJ

    IF (IC /= KFDIA-KIDIA+1) THEN

      DO JA = 1 , 8, 2
        DO JJ=1,JN
          JL=JX(JJ)
!            ZZ5 = SQRT(PABCU5(JL,JA,IKU2)-PABCU5(JL,JA,IKN))
!            IF (ZZ5 > 0.0_JPRB) THEN
!              ZZ  = 0.5_JPRB*(PABCU(JL,JA,IKU2)-PABCU(JL,JA,IKN))/ZZ5
!            ELSE
!              ZZ  = 0.0_JPRB
!            ENDIF
            ZZ5 = SQRT(PABCU5(JL,JA,IKU2)-PABCU5(JL,JA,IKN)+1E-25_JPRB)
            ZZ  = 0.5_JPRB*(PABCU(JL,JA,IKU2)-PABCU(JL,JA,IKN))/ZZ5
            ZXD  = (PGB( JL,JA,2,IJKL)+2.0_JPRB*ZZ5)*ZZ
            ZXD5 = PGB( JL,JA,1,IJKL) + ZZ5*(PGB( JL,JA,2,IJKL) + ZZ5 )
            ZXN  = ZZ*PGA( JL,JA,2,IJKL)
            ZXN5 = PGA( JL,JA,1,IJKL) + ZZ5*PGA( JL,JA,2,IJKL)
            ZXDIV5 = 1.0_JPRB/ZXD5
!            ZTTP (JL,JA,JLK+1) = ZXN*ZXDIV5 - ZXN5*ZXDIV5*ZXDIV5*ZXD
!            ZTTP5(JL,JA,JLK+1) = ZXN5*ZXDIV5
            ZZX5 = ZXN5*ZXDIV5
            ZTTP (JL,JA,JLK+1) = ZXN*ZXDIV5 - ZZX5*ZXDIV5*ZXD
            ZTTP5(JL,JA,JLK+1) = ZZX5

            ZZ5 = SQRT(PABCU5(JL,JA+1,IKU2)-PABCU5(JL,JA+1,IKN)+1E-25_JPRB)
            ZZ  = 0.5_JPRB*(PABCU(JL,JA+1,IKU2)-PABCU(JL,JA+1,IKN))/ZZ5
            ZXD  = (PGB( JL,JA+1,2,IJKL)+2.0_JPRB*ZZ5)*ZZ
            ZXD5 = PGB( JL,JA+1,1,IJKL) + ZZ5*(PGB( JL,JA+1,2,IJKL) + ZZ5 )
            ZXN  = ZZ*PGA( JL,JA+1,2,IJKL)
            ZXN5 = PGA( JL,JA+1,1,IJKL) + ZZ5*PGA( JL,JA+1,2,IJKL)
            ZXDIV5 = 1.0_JPRB/ZXD5
!           ZTTP (JL,JA+1,JLK+1) = ZXN*ZXDIV5 - ZXN5*ZXDIV5*ZXDIV5*ZXD
!           ZTTP5(JL,JA+1,JLK+1) = ZXN5*ZXDIV5
            ZZX5 = ZXN5*ZXDIV5
            ZTTP (JL,JA+1,JLK+1) = ZXN*ZXDIV5 - ZZX5*ZXDIV5*ZXD
            ZTTP5(JL,JA+1,JLK+1) = ZZX5
        ENDDO
      ENDDO

        DO JJ=1,JN
          JL=JX(JJ)
          IF (ZTTP5(JL,3,JLK+1) < 0.0_JPRB) THEN
            ZTTP (JL,3,JLK+1) = 0.0_JPRB
            ZTTP5(JL,3,JLK+1) = 0.0_JPRB
          ENDIF
          ZTTP (JL,9,JLK+1) = ZTTP (JL,8,JLK+1)
          ZTTP5(JL,9,JLK+1) = ZTTP5(JL,8,JLK+1)
      ENDDO

!      DO JA = 1, KTRAER
!        DO JL = KIDIA,KFDIA
!          IF (LL_DO(JL)) THEN
!            ZTT (JL,JA) = (ZTTP (JL,JA,JLK)+ZTTP (JL,JA,JLK+1))*0.5_JPRB
!            ZTT5(JL,JA) = (ZTTP5(JL,JA,JLK)+ZTTP5(JL,JA,JLK+1))*0.5_JPRB
!          ENDIF
!        ENDDO
!      ENDDO

      DO JA = 1, KTRAER-1,2
        DO JJ=1,JN
          JL=JX(JJ)
            ZTT (JL,JA) = (ZTTP (JL,JA,JLK)+ZTTP (JL,JA,JLK+1))*0.5_JPRB
            ZTT5(JL,JA) = (ZTTP5(JL,JA,JLK)+ZTTP5(JL,JA,JLK+1))*0.5_JPRB
            ZTT (JL,JA+1) = (ZTTP (JL,JA+1,JLK)+ZTTP (JL,JA+1,JLK+1))*0.5_JPRB
            ZTT5(JL,JA+1) = (ZTTP5(JL,JA+1,JLK)+ZTTP5(JL,JA+1,JLK+1))*0.5_JPRB
       ENDDO
      ENDDO
      IF (MOD(KTRAER,2) /= 0) THEN
        DO JJ=1,JN
          JL=JX(JJ)
            ZTT (JL,KTRAER) = &
             & (ZTTP (JL,KTRAER,JLK)+ZTTP (JL,KTRAER,JLK+1))*0.5_JPRB
            ZTT5(JL,KTRAER) = &
             & (ZTTP5(JL,KTRAER,JLK)+ZTTP5(JL,KTRAER,JLK+1))*0.5_JPRB
        ENDDO
      ENDIF

      DO JJ=1,JN
        JL=JX(JJ)
          ZWD1  = PDBDT5(JL,1,IJKL)*ZTT(JL,1) + ZTT5(JL,1)*PDBDT(JL,1,IJKL)
          ZWD15 = PDBDT5(JL,1,IJKL)*ZTT5(JL,1)
          ZWD2  = PDBDT5(JL,2,IJKL)*ZTT5(JL,2)*ZTT(JL,7) &
           & + PDBDT5(JL,2,IJKL)*ZTT5(JL,7)*ZTT(JL,2) &
           & + ZTT5(JL,2)*ZTT5(JL,7)*PDBDT(JL,2,IJKL)
          ZWD25 = PDBDT5(JL,2,IJKL)*ZTT5(JL,2)*ZTT5(JL,7)
          ZWD3  = PDBDT5(JL,3,IJKL)*ZTT5(JL,4)*ZTT(JL,8) &
           & + PDBDT5(JL,3,IJKL)*ZTT5(JL,8)*ZTT(JL,4) &
           & + ZTT5(JL,4)*ZTT5(JL,8)*PDBDT(JL,3,IJKL)
          ZWD35 = PDBDT5(JL,3,IJKL)*ZTT5(JL,4)*ZTT5(JL,8)
          ZWD4  = PDBDT5(JL,4,IJKL)*ZTT5(JL,5)*ZTT(JL,9) &
           & + PDBDT5(JL,4,IJKL)*ZTT5(JL,9)*ZTT(JL,5) &
           & + ZTT5(JL,5)*ZTT5(JL,9)*PDBDT(JL,4,IJKL)
          ZWD45 = PDBDT5(JL,4,IJKL)*ZTT5(JL,5)*ZTT5(JL,9)
          ZWD5  = PDBDT5(JL,5,IJKL)*ZTT(JL,3) + ZTT5(JL,3)*PDBDT(JL,5,IJKL)
          ZWD55 = PDBDT5(JL,5,IJKL)*ZTT5(JL,3)
          ZWD6  = PDBDT5(JL,6,IJKL)*ZTT(JL,6) + ZTT5(JL,6)*PDBDT(JL,6,IJKL)
          ZWD65 = PDBDT5(JL,6,IJKL)*ZTT5(JL,6)

          PCNTRB (JL,JK,IJKL) = ZWD1 +ZWD2 +ZWD3 +ZWD4 +ZWD5 +ZWD6
          PCNTRB5(JL,JK,IJKL) = ZWD15+ZWD25+ZWD35+ZWD45+ZWD55+ZWD65
          PDISU (JL,JK) = PDISU (JL,JK)+PCNTRB (JL,JK,IJKL)
          PDISU5(JL,JK) = PDISU5(JL,JK)+PCNTRB5(JL,JK,IJKL)
      ENDDO

    ENDIF

  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWVDRTL',1,ZHOOK_HANDLE)
END SUBROUTINE LWVDRTL
