SUBROUTINE LWVNTL &
 & ( KIDIA, KFDIA, KLON   , KLEV , KUAER,&
 & PGA   , PGB   , PABCU5 , PDBSL5, &
 & PADJD5, PADJU5, PCNTRB5, PDBDT5, PDWFSU5,&
 & PABCU , PDBSL , &
 & PADJD , PADJU , PCNTRB , PDBDT , PDWFSU &
 & )  

!**** *LWVNTL*   - L.W., VERTICAL INTEGRATION, NEARBY LAYERS
!                      (tangent linear)

!     PURPOSE.
!     --------
!           CARRIES OUT THE VERTICAL INTEGRATION ON NEARBY LAYERS
!           TO GIVE LONGWAVE FLUXES OR RADIANCES

!**   INTERFACE.
!     ----------

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!  Trajectory  Perturbation  Description                    Dimension
!  PABCU5      PABCU         ABSORBER AMOUNTS               (KLON,NUA,3*KLEV+1)
!  PDBSL5      PDBSL         SUB-LAYER PLANCK FUNCTION
!                            GRADIENT                       (KLON,KLEV*2)
!  PGA         ---           PADE APPROXIMANTS              (KLON,NIPD,2)
!  PGB         ---           PADE APPROXIMANTS              (KLON,NIPD,2)

!     ==== OUTPUTS ===
!  Trajectory  Perturbation  Description                       Dimension
!  PADJD5      PADJD         CONTRIBUTION OF ADJACENT LAYERS   (KLON,KLEV+1)
!  PADJU5      PADJU         CONTRIBUTION OF ADJACENT LAYERS   (KLON,KLEV+1)
!  PCNTRB5     PCNTRB        CLEAR-SKY ENERGY EXCHANGE 
!                            MATRIX                        (KLON,KLEV+1,KLEV+1)
!  PDBDT5      PDBDT         LAYER PLANCK FUNCTION GRADIENT    (KLON,NUA,KLEV)
!  PDWFSU5     PDWFSU        SPECTRAL DOWNWARD FLUX AT SURFACE (KLON,NSIL)

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          1. PERFORMS THE VERTICAL INTEGRATION CORRESPONDING TO THE
!     CONTRIBUTIONS OF THE ADJACENT LAYERS USING A GAUSSIAN QUADRATURE

!     EXTERNALS.
!     ----------

!          *LWTTTL*

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        M. Janiskova  *ECMWF*       26-Jan-2006

!     MODIFICATIONS.
!     --------------

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOELW    , ONLY : NSIL     ,NIPD     ,NTRA     ,NUA      ,&
 & NG1      ,NG1P1    ,WG1  

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KUAER 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGA(KLON,NIPD,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGB(KLON,NIPD,2,KLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU5(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDBSL5(KLON,NSIL,KLEV*2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PADJD5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PADJU5(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCNTRB5(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDBDT5(KLON,NSIL,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDWFSU5(KLON,NSIL) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PABCU(KLON,NUA,3*KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDBSL(KLON,NSIL,KLEV*2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PADJD(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PADJU(KLON,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCNTRB(KLON,KLEV+1,KLEV+1) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PDBDT(KLON,NSIL,KLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDWFSU(KLON,NSIL) 

!-----------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

!-----------------------------------------------------------------------

!              ------------

REAL(KIND=JPRB) :: ZTT(KLON,NTRA), ZUU(KLON,NUA)
REAL(KIND=JPRB) :: ZTT5(KLON,NTRA), ZUU5(KLON,NUA)
REAL(KIND=JPRB) :: ZWTRD, ZWTRU, ZWTR1, ZWTR2, ZWTR3 
REAL(KIND=JPRB) :: ZWTR4, ZWTR5, ZWTR6
REAL(KIND=JPRB) :: ZWTRD5, ZWTRU5, ZWTR15, ZWTR25, ZWTR35 
REAL(KIND=JPRB) :: ZWTR45, ZWTR55, ZWTR65

INTEGER(KIND=JPIM) :: IBS, IDD, IM12, IMU, IND, INU, IXD, IXU,&
 & JA, JG, JK, JK1, JK2, JL, JNU  

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "lwtttl.intfb.h"

!-----------------------------------------------------------------------

!*         1.    INITIALIZATION
!                --------------

!*         1.1     INITIALIZE LAYER CONTRIBUTIONS
!                  ------------------------------

IF (LHOOK) CALL DR_HOOK('LWVNTL',0,ZHOOK_HANDLE)
DO JK = 1 , KLEV+1
  DO JL = KIDIA,KFDIA
    PADJD (JL,JK) = 0.0_JPRB
    PADJD5(JL,JK) = 0.0_JPRB
    PADJU (JL,JK) = 0.0_JPRB
    PADJU5(JL,JK) = 0.0_JPRB
  ENDDO
ENDDO

!*         1.2     INITIALIZE TRANSMISSION FUNCTIONS
!                  ---------------------------------

DO JA = 1 , NTRA
  DO JL = KIDIA,KFDIA
    ZTT (JL,JA) = 0.0_JPRB
    ZTT5(JL,JA) = 1.0_JPRB
  ENDDO
ENDDO

DO JA = 1 , NUA
  DO JL = KIDIA,KFDIA
    ZUU (JL,JA) = 0.0_JPRB
    ZUU5(JL,JA) = 0.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

!*         2.      VERTICAL INTEGRATION
!                  --------------------

!*         2.1     CONTRIBUTION FROM ADJACENT LAYERS
!                  ---------------------------------

DO JK = 1 , KLEV

!*         2.1.1   DOWNWARD LAYERS
!                  ---------------

  IM12 = 2 * (JK - 1)
  IND = (JK - 1) * NG1P1 + 1
  IXD = IND
  INU = JK * NG1P1 + 1
  IXU = IND

  DO JG = 1 , NG1
    IBS = IM12 + JG
    IDD = IXD + JG

    DO JA = 1 , KUAER
      DO JL = KIDIA,KFDIA
        ZUU (JL,JA) = PABCU (JL,JA,IND) - PABCU (JL,JA,IDD)
        ZUU5(JL,JA) = PABCU5(JL,JA,IND) - PABCU5(JL,JA,IDD)
      ENDDO
    ENDDO

    CALL LWTTTL &
     & ( KIDIA        , KFDIA        , KLON,&
     & PGA(1,1,1,JK), PGB(1,1,1,JK),&
     & ZUU5, ZTT5, ZUU, ZTT &
     & )  

    DO JL = KIDIA,KFDIA
      ZWTR1  = PDBSL5(JL,1,IBS)*ZTT5(JL,1)*ZTT(JL,10) &
       &     + PDBSL5(JL,1,IBS)*ZTT5(JL,10)*ZTT(JL,1) &
       &     + ZTT5(JL,1)*ZTT5(JL,10)*PDBSL(JL,1,IBS)
      ZWTR15 = PDBSL5(JL,1,IBS)*ZTT5(JL,1)*ZTT5(JL,10)
      ZWTR2  = PDBSL5(JL,2,IBS)*ZTT5(JL,2)*ZTT5(JL,7)*ZTT(JL,11) &
       &     + PDBSL5(JL,2,IBS)*ZTT5(JL,2)*ZTT5(JL,11)*ZTT(JL,7) &
       &     + PDBSL5(JL,2,IBS)*ZTT5(JL,7)*ZTT5(JL,11)*ZTT(JL,2) &
       &     + ZTT5(JL,2)*ZTT5(JL,7)*ZTT5(JL,11)*PDBSL(JL,2,IBS)
      ZWTR25 = PDBSL5(JL,2,IBS)*ZTT5(JL,2)*ZTT5(JL,7)*ZTT5(JL,11)
      ZWTR3  = PDBSL5(JL,3,IBS)*ZTT5(JL,4)*ZTT5(JL,8)*ZTT(JL,12) &
       &     + PDBSL5(JL,3,IBS)*ZTT5(JL,4)*ZTT5(JL,12)*ZTT(JL,8) &
       &     + PDBSL5(JL,3,IBS)*ZTT5(JL,8)*ZTT5(JL,12)*ZTT(JL,4) &
       &     + ZTT5(JL,4)*ZTT5(JL,8)*ZTT5(JL,12)*PDBSL(JL,3,IBS)
      ZWTR35 = PDBSL5(JL,3,IBS)*ZTT5(JL,4)*ZTT5(JL,8)*ZTT5(JL,12)
      ZWTR4  = PDBSL5(JL,4,IBS)*ZTT5(JL,5)*ZTT5(JL,9)*ZTT(JL,13) &
       &     + PDBSL5(JL,4,IBS)*ZTT5(JL,5)*ZTT5(JL,13)*ZTT(JL,9) &
       &     + PDBSL5(JL,4,IBS)*ZTT5(JL,9)*ZTT5(JL,13)*ZTT(JL,5) &
       &     + ZTT5(JL,5)*ZTT5(JL,9)*ZTT5(JL,13)*PDBSL(JL,4,IBS)
      ZWTR45 = PDBSL5(JL,4,IBS)*ZTT5(JL,5)*ZTT5(JL,9)*ZTT5(JL,13)
      ZWTR5  = PDBSL5(JL,5,IBS)*ZTT5(JL,3)*ZTT(JL,14) &
       &     + PDBSL5(JL,5,IBS)*ZTT5(JL,14)*ZTT(JL,3) &
       &     + ZTT5(JL,3)*ZTT5(JL,14)*PDBSL(JL,5,IBS)
      ZWTR55 = PDBSL5(JL,5,IBS)*ZTT5(JL,3)*ZTT5(JL,14)
      ZWTR6  = PDBSL5(JL,6,IBS)*ZTT5(JL,6)*ZTT(JL,15) &
       &     + PDBSL5(JL,6,IBS)*ZTT5(JL,15)*ZTT(JL,6) &
       &     + ZTT5(JL,6)*ZTT5(JL,15)*PDBSL(JL,6,IBS)
      ZWTR65 = PDBSL5(JL,6,IBS)*ZTT5(JL,6)*ZTT5(JL,15)
      ZWTRD  = ZWTR1+ZWTR2+ZWTR3+ZWTR4+ZWTR5+ZWTR6
      ZWTRD5 = ZWTR15+ZWTR25+ZWTR35+ZWTR45+ZWTR55+ZWTR65
      PADJD (JL,JK) = PADJD (JL,JK) + ZWTRD * WG1(JG)
      PADJD5(JL,JK) = PADJD5(JL,JK) + ZWTRD5 * WG1(JG)
      IF (JK == 1) THEN
        PDWFSU (JL,1) = PDWFSU (JL,1)+WG1(JG)*ZWTR1
        PDWFSU5(JL,1) = PDWFSU5(JL,1)+WG1(JG)*ZWTR15
        PDWFSU (JL,2) = PDWFSU (JL,2)+WG1(JG)*ZWTR2
        PDWFSU5(JL,2) = PDWFSU5(JL,2)+WG1(JG)*ZWTR25
        PDWFSU (JL,3) = PDWFSU (JL,3)+WG1(JG)*ZWTR3
        PDWFSU5(JL,3) = PDWFSU5(JL,3)+WG1(JG)*ZWTR35
        PDWFSU (JL,4) = PDWFSU (JL,4)+WG1(JG)*ZWTR4
        PDWFSU5(JL,4) = PDWFSU5(JL,4)+WG1(JG)*ZWTR45
        PDWFSU (JL,5) = PDWFSU (JL,5)+WG1(JG)*ZWTR5
        PDWFSU5(JL,5) = PDWFSU5(JL,5)+WG1(JG)*ZWTR55
        PDWFSU (JL,6) = PDWFSU (JL,6)+WG1(JG)*ZWTR6
        PDWFSU5(JL,6) = PDWFSU5(JL,6)+WG1(JG)*ZWTR65
      ENDIF
    ENDDO

!*         2.1.2   UPWARD LAYERS
!                  -------------

    IMU = IXU + JG
    DO JA = 1 , KUAER
      DO JL = KIDIA,KFDIA
        ZUU (JL,JA) = PABCU (JL,JA,IMU) - PABCU (JL,JA,INU)
        ZUU5(JL,JA) = PABCU5(JL,JA,IMU) - PABCU5(JL,JA,INU)
      ENDDO
    ENDDO

    CALL LWTTTL &
     & ( KIDIA        , KFDIA        , KLON,&
     & PGA(1,1,1,JK), PGB(1,1,1,JK),&
     & ZUU5, ZTT5, ZUU, ZTT &
     & )  

    DO JL = KIDIA,KFDIA
      ZWTRU  = PDBSL5(JL,1,IBS)*ZTT5(JL,1)*ZTT(JL,10) &
       &     + PDBSL5(JL,1,IBS)*ZTT5(JL,10)*ZTT(JL,1) &
       &     + ZTT5(JL,1)*ZTT5(JL,10)*PDBSL(JL,1,IBS) &
       &     + PDBSL5(JL,2,IBS)*ZTT5(JL,2)*ZTT5(JL,7)*ZTT(JL,11) &
       &     + PDBSL5(JL,2,IBS)*ZTT5(JL,2)*ZTT5(JL,11)*ZTT(JL,7) &
       &     + PDBSL5(JL,2,IBS)*ZTT5(JL,7)*ZTT5(JL,11)*ZTT(JL,2) &
       &     + ZTT5(JL,2)*ZTT5(JL,7)*ZTT5(JL,11)*PDBSL(JL,2,IBS) &
       &     + PDBSL5(JL,3,IBS)*ZTT5(JL,4)*ZTT5(JL,8)*ZTT(JL,12) &
       &     + PDBSL5(JL,3,IBS)*ZTT5(JL,4)*ZTT5(JL,12)*ZTT(JL,8) &
       &     + PDBSL5(JL,3,IBS)*ZTT5(JL,8)*ZTT5(JL,12)*ZTT(JL,4) &
       &     + ZTT5(JL,4)*ZTT5(JL,8)*ZTT5(JL,12)*PDBSL(JL,3,IBS) &
       &     + PDBSL5(JL,4,IBS)*ZTT5(JL,5)*ZTT5(JL,9)*ZTT(JL,13) &
       &     + PDBSL5(JL,4,IBS)*ZTT5(JL,5)*ZTT5(JL,13)*ZTT(JL,9) &
       &     + PDBSL5(JL,4,IBS)*ZTT5(JL,9)*ZTT5(JL,13)*ZTT(JL,5) &
       &     + ZTT5(JL,5)*ZTT5(JL,9)*ZTT5(JL,13)*PDBSL(JL,4,IBS) &
       &     + PDBSL5(JL,5,IBS)*ZTT5(JL,3)*ZTT(JL,14) &
       &     + PDBSL5(JL,5,IBS)*ZTT5(JL,14)*ZTT(JL,3) &
       &     + ZTT5(JL,3)*ZTT5(JL,14)*PDBSL(JL,5,IBS) &
       &     + PDBSL5(JL,6,IBS)*ZTT5(JL,6)*ZTT(JL,15) &
       &     + PDBSL5(JL,6,IBS)*ZTT5(JL,15)*ZTT(JL,6) &
       &     + ZTT5(JL,6)*ZTT5(JL,15)*PDBSL(JL,6,IBS)
      ZWTRU5 = PDBSL5(JL,1,IBS)*ZTT5(JL,1)*ZTT5(JL,10)&
       &     + PDBSL5(JL,2,IBS)*ZTT5(JL,2)*ZTT5(JL,7)*ZTT5(JL,11)&
       &     + PDBSL5(JL,3,IBS)*ZTT5(JL,4)*ZTT5(JL,8)*ZTT5(JL,12)&
       &     + PDBSL5(JL,4,IBS)*ZTT5(JL,5)*ZTT5(JL,9)*ZTT5(JL,13)&
       &     + PDBSL5(JL,5,IBS)*ZTT5(JL,3)*ZTT5(JL,14)&
       &     + PDBSL5(JL,6,IBS)*ZTT5(JL,6)*ZTT5(JL,15)  
      PADJU (JL,JK+1) = PADJU (JL,JK+1) + ZWTRU * WG1(JG)
      PADJU5(JL,JK+1) = PADJU5(JL,JK+1) + ZWTRU5 * WG1(JG)
    ENDDO

  ENDDO

  DO JL = KIDIA,KFDIA
    PCNTRB (JL,JK,JK+1) = PADJD (JL,JK)
    PCNTRB5(JL,JK,JK+1) = PADJD5(JL,JK)
    PCNTRB (JL,JK+1,JK) = PADJU (JL,JK+1)
    PCNTRB5(JL,JK+1,JK) = PADJU5(JL,JK+1)
    PCNTRB (JL,JK  ,JK) = 0.0_JPRB
    PCNTRB5(JL,JK  ,JK) = 0.0_JPRB
  ENDDO

ENDDO

DO JK = 1 , KLEV
  JK2 = 2 * JK
  JK1 = JK2 - 1

  DO JNU = 1 , NSIL
    DO JL = KIDIA,KFDIA
      PDBDT (JL,JNU,JK) = PDBSL (JL,JNU,JK1) + PDBSL (JL,JNU,JK2)
      PDBDT5(JL,JNU,JK) = PDBSL5(JL,JNU,JK1) + PDBSL5(JL,JNU,JK2)
    ENDDO
  ENDDO
ENDDO

!-----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LWVNTL',1,ZHOOK_HANDLE)
END SUBROUTINE LWVNTL
