!OPTIONS XOPT(HSFUN)
SUBROUTINE SWDETL &
 & ( KIDIA, KFDIA, KLON,&
 & PGG5 , PREF5, PRMUZ5, PTO15, PW5,&
 & PRE15, PRE25, PTR15 , PTR25,&
 & PGG  , PREF , PRMUZ , PTO1 , PW,&
 & PRE1 , PRE2 , PTR1  , PTR2 &
 & )  

!**** *SWDETL* - DELTA-EDDINGTON IN A CLOUDY LAYER
!                -> TANGENT LINEAR <-

!     PURPOSE.
!     --------
!           COMPUTES THE REFLECTIVITY AND TRANSMISSIVITY OF A CLOUDY
!     LAYER USING THE DELTA-EDDINGTON'S APPROXIMATION.

!**   INTERFACE.
!     ----------
!          *SWDETL* IS CALLED BY *SWRTL*, *SWNITL*

!        EXPLICIT ARGUMENTS :
!        --------------------
! PGG    : (KLON)             ; ASSYMETRY FACTOR
! PREF   : (KLON)             ; REFLECTIVITY OF THE UNDERLYING LAYER
! PRMUZ  : (KLON)             ; COSINE OF SOLAR ZENITH ANGLE
! PTO1   : (KLON)             ; OPTICAL THICKNESS
! PW     : (KLON)             ; SINGLE SCATTERING ALBEDO
!     ==== OUTPUTS ===
! PRE1   : (KLON)             ; LAYER REFLECTIVITY ASSUMING NO
!                             ; REFLECTION FROM UNDERLYING LAYER
! PTR1   : (KLON)             ; LAYER TRANSMISSIVITY ASSUMING NO
!                             ; REFLECTION FROM UNDERLYING LAYER
! PRE2   : (KLON)             ; LAYER REFLECTIVITY ASSUMING
!                             ; REFLECTION FROM UNDERLYING LAYER
! PTR2   : (KLON)             ; LAYER TRANSMISSIVITY ASSUMING
!                             ; REFLECTION FROM UNDERLYING LAYER

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!          STANDARD DELTA-EDDINGTON LAYER CALCULATIONS.

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 96-09-27
!        Modified : 99-05-12 M.Janiskova - security in EXP()
!        03-10-10 Deborah Salmond and Marta Janiskova Optimisation
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        03-12-13 John Hague - MASS Vector Fns
!        08-03-26 M.Janiskova - code optimization and cleaning

!     ------------------------------------------------------------------

!     ------------------------------------------------------------------

!*       0.1   ARGUMENTS
!              ---------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOERDU   , ONLY : REPLOG

USE YOMJFH   , ONLY : N_VMASS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGG5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREF5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMUZ5(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTO15(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PW5(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE15(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE25(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTR15(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTR25(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGG(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PREF(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMUZ(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTO1(KLON) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PW(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE1(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PRE2(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTR1(KLON) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTR2(KLON) 
INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZA21,&
 & ZA215, ZA22, ZA225, ZA23, ZA235, ZALPHA, &
 & ZALPHA5, ZAM2B, ZAM2B5, ZAP2B, ZAP2B5, ZARG, &
 & ZARG2, ZARG25, ZARG5, ZB21, ZB215, ZB22, &
 & ZB225, ZB23, ZB235, ZBETA, ZBETA5, ZC1A, &
 & ZC1A5, ZC1B, ZC1B5, ZC2A, ZC2A5, ZC2B, ZC2B5, &
 & ZDENA, ZDENA5, ZDENB, ZDENB5, ZDT5, &
 & ZEXKM, ZEXKM5, ZEXKP, ZEXKP5, ZEXMU0, ZEXMU05, &
 & ZFF, ZFF5, ZGP, ZGP5, ZRI0A, ZRI0A5, ZRI0B, &
 & ZRI0B5, ZRI0C, ZRI0C5, ZRI0D, ZRI0D5, ZRI1A, &
 & ZRI1A5, ZRI1B, ZRI1B5, ZRI1C, ZRI1C5, ZRI1D, &
 & ZRI1D5, ZRK, ZRK5, ZRKI5, ZRM2, ZRM25, ZRP, ZRP5, &
 & ZTOP, ZTOP5, ZWCP, ZWCP5, ZWM, ZWM5, ZX1, &
 & ZX15, ZX2, ZX25, ZXM2P, ZXM2P5, ZXP2P, ZXP2P5  
REAL(KIND=JPRB) :: ZPRMUZ5,ZPRMUZ,ZIDENA5,ZIDENA,ZIDENB5,ZIDENB,ZRR5,ZRR
REAL(KIND=JPRB) :: ZDT25, ZPWFF5, ZPWFF
REAL(KIND=JPRB) :: ZDIV15, ZDIV25, ZDIV35

REAL(KIND=JPRB) :: ZQREPLOG
REAL(KIND=JPRB) :: ZHOOK_HANDLE

REAL(KIND=JPRB) :: ZTMP(12,KFDIA-KIDIA+1)
REAL(KIND=JPRB) :: ZARG5A  (KFDIA-KIDIA+1)
REAL(KIND=JPRB) :: ZARG25A (KFDIA-KIDIA+1)
REAL(KIND=JPRB) :: ZEXMU05A(KFDIA-KIDIA+1)
REAL(KIND=JPRB) :: ZEXKP5A (KFDIA-KIDIA+1)
INTEGER(KIND=JPIM) :: JLL,JLEN

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SWDETL',0,ZHOOK_HANDLE)

!*         1.      DELTA-EDDINGTON CALCULATIONS

! 100 CONTINUE

ZQREPLOG = SQRT(REPLOG)
ZDT5  = 2.0_JPRB/3._JPRB
ZDT25 = 2.0_JPRB*ZDT5

DO JL   =   KIDIA,KFDIA

!*         1.1     SET UP THE DELTA-MODIFIED PARAMETERS

! 110 CONTINUE

  ZFF  = 2*PGG (JL)*PGG5(JL)
  ZFF5 = PGG5(JL)*PGG5(JL)
  ZDIV15 = 1.0_JPRB/(1.0_JPRB+PGG5(JL))
  ZGP  = PGG (JL)*ZDIV15 &
   & - PGG5(JL)*PGG (JL)*(ZDIV15**2)
  ZGP5 = PGG5(JL)*ZDIV15
  ZPWFF = - PW5(JL)*ZFF - ZFF5*PW(JL)
  ZPWFF5 = 1.0_JPRB- PW5(JL) * ZFF5
  ZTOP = ZPWFF5*PTO1(JL)+PTO15(JL)*ZPWFF
  ZTOP5 = ZPWFF5 * PTO15(JL)
  ZDIV25 = 1.0_JPRB/ZPWFF5
  ZWCP = ((1.0_JPRB-ZFF5)*PW(JL)-PW5(JL)*ZFF)*ZDIV25 &
   & - (1.0_JPRB-ZFF5)*PW5(JL)*ZPWFF*(ZDIV25**2)
  ZWCP5 = (1.0_JPRB-ZFF5)* PW5(JL) * ZDIV25

  ZX1  =   -ZWCP *ZGP5 -ZWCP5*ZGP
  ZX15 = 1.0_JPRB-ZWCP5*ZGP5
  ZWM  =   -ZWCP
  ZWM5 = 1.0_JPRB-ZWCP5
! ZRM2 =  2*PRMUZ (JL) * PRMUZ5(JL)
! ZRM25=    PRMUZ5(JL) * PRMUZ5(JL)
!! ZRK  = _HALF_* 3._JPRB*(ZWM *ZX15+ZWM5*ZX1 ) / SQRT(3._JPRB*ZWM5*ZX15)
!! ZRK5 = SQRT(3._JPRB*ZWM5*ZX15)
  ZRKI5 = 3._JPRB*ZWM5*ZX15
  IF (ZRKI5 >= REPLOG) THEN
    ZRK5 = SQRT(3._JPRB*ZWM5*ZX15)
    ZRK  = 0.5_JPRB* 3._JPRB*(ZWM *ZX15+ZWM5*ZX1 ) / ZRK5
  ELSE
    ZRK  = 0.0_JPRB
    ZRK5 = ZQREPLOG
  ENDIF

  ZPRMUZ5=1.0_JPRB/PRMUZ5(JL)
  ZPRMUZ=-PRMUZ(JL)*(ZPRMUZ5**2)

!---------------------------
  IF (ZTOP5*ZPRMUZ5 >= 200._JPRB ) THEN
    ZARG =0.0_JPRB
    ZARG5=200._JPRB
  ELSE
    ZARG =ZTOP *ZPRMUZ5 + ZTOP5*ZPRMUZ
    ZARG5=ZTOP5*ZPRMUZ5
  ENDIF
  IF (ZARG5 < -200._JPRB) THEN
    ZARG5 = -200._JPRB
    ZARG  = 0.0_JPRB
  ENDIF
! ZEXMU05=EXP(-ZARG5)
! ZEXMU0 =-ZARG *ZEXMU05
  IF (ZRK5*ZTOP5 >= 200._JPRB) THEN
    ZARG2 = 0.0_JPRB
    ZARG25= 200._JPRB
  ELSE
    ZARG2 = ZRK *ZTOP5 + ZRK5*ZTOP
    ZARG25= ZRK5*ZTOP5
  ENDIF
! ZEXKP5=EXP(ZARG25)
! ZEXKP =ZARG2*ZEXKP5

  JLL=JL-KIDIA+1
  ZARG5A (JLL) = -ZARG5
  ZARG25A(JLL) = ZARG25
  ZTMP( 1,JLL) = ZGP
  ZTMP( 2,JLL) = ZGP5
  ZTMP( 3,JLL) = ZWCP
  ZTMP( 4,JLL) = ZWCP5
  ZTMP( 5,JLL) = ZX1
  ZTMP( 6,JLL) = ZX15
  ZTMP( 7,JLL) = ZWM
  ZTMP( 8,JLL) = ZWM5
  ZTMP( 9,JLL) = ZRK
  ZTMP(10,JLL) = ZRK5
  ZTMP(11,JLL) = ZARG
  ZTMP(12,JLL) = ZARG2

ENDDO

IF(N_VMASS /= 0 ) THEN  ! USING VECTOR MASS
  JLEN=KFDIA-KIDIA+1
  CALL VEXP(ZEXMU05A,ZARG5A, JLEN)
  CALL VEXP(ZEXKP5A,ZARG25A,JLEN)
ELSE
  DO JL   =   KIDIA,KFDIA
    JLL=JL-KIDIA+1
    ZEXMU05A(JLL) = EXP(ZARG5A(JLL))
    ZEXKP5A(JLL)  = EXP(ZARG25A(JLL))
  ENDDO
ENDIF

DO JL   =   KIDIA,KFDIA

  JLL=JL-KIDIA+1
  ZEXMU05 = ZEXMU05A(JLL)
  ZEXKP5  = ZEXKP5A(JLL)
  ZGP   = ZTMP( 1,JLL)
  ZGP5  = ZTMP( 2,JLL)
  ZWCP  = ZTMP( 3,JLL)
  ZWCP5 = ZTMP( 4,JLL)
  ZX1   = ZTMP( 5,JLL)
  ZX15  = ZTMP( 6,JLL)
  ZWM   = ZTMP( 7,JLL)
  ZWM5  = ZTMP( 8,JLL)
  ZRK   = ZTMP( 9,JLL)
  ZRK5  = ZTMP(10,JLL)
  ZARG  = ZTMP(11,JLL)
  ZARG2 = ZTMP(12,JLL)
  ZRM2  = 2*PRMUZ (JL) * PRMUZ5(JL)
  ZRM25 =   PRMUZ5(JL) * PRMUZ5(JL)
  ZEXMU0= -ZARG *ZEXMU05
  ZEXKP = ZARG2*ZEXKP5
!----------------------------------------

  ZX2  = (  -2.0_JPRB*ZRK *ZRK5*ZRM25 -ZRK5*ZRK5*ZRM2 )*ZDT25
  ZX25 = (1.0_JPRB-  ZRK5*ZRK5*ZRM25)*ZDT25
  ZRR5=1.0_JPRB/ZX25
  ZRR=-ZX2*(ZRR5*ZRR5)

  ZDIV35 = 1.0_JPRB/ZX15
  ZRP =ZRK*ZDIV35  -ZRK5*ZX1 *(ZDIV35**2)
  ZRP5=ZRK5*ZDIV35
  ZALPHA  = ZWCP *ZRM25*(1.0_JPRB+ZGP5*ZWM5)*ZRR5 &
   & + ZWCP5*ZRM2 *(1.0_JPRB+ZGP5*ZWM5)*ZRR5 &
   & + ZWCP5*ZRM25*(   ZGP *ZWM5+ZGP5*ZWM )*ZRR5 &
   & + ZWCP5*ZRM25*(1.0_JPRB+ZGP5*ZWM5)*ZRR  
  ZALPHA5 = ZWCP5*ZRM25*(1.0_JPRB+ZGP5*ZWM5)*ZRR5

  ZBETA  = ZWCP *PRMUZ5(JL)*(1.0_JPRB+3._JPRB*ZGP5*ZRM25*ZWM5)*ZRR5 &
   & + ZWCP5*PRMUZ (JL)*(1.0_JPRB+3._JPRB*ZGP5*ZRM25*ZWM5)*ZRR5 &
   & + ZWCP5*PRMUZ5(JL)*(   3._JPRB*ZGP *ZRM25*ZWM5 &
   & +   3._JPRB*ZGP5*ZRM2 *ZWM5 &
   & +   3._JPRB*ZGP5*ZRM25*ZWM )*ZRR5  &
   & + ZWCP5*PRMUZ5(JL)*(1.0_JPRB+3._JPRB*ZGP5*ZRM25*ZWM5)*ZRR  
  ZBETA5 = ZWCP5* PRMUZ5(JL) *(1.0_JPRB+3._JPRB*ZGP5*ZRM25*ZWM5)*ZRR5

  ZPRMUZ5=1.0_JPRB/PRMUZ5(JL)
  ZPRMUZ=-PRMUZ(JL)*(ZPRMUZ5**2)

! IF (ZTOP5*ZPRMUZ5 >= 200._JPRB) THEN
!   ZARG =0.0_JPRB
!   ZARG5=200._JPRB
! ELSE
!   ZARG =ZTOP *ZPRMUZ5 + ZTOP5*ZPRMUZ
!   ZARG5=ZTOP5*ZPRMUZ5
! ENDIF
! IF (ZARG5 < -200._JPRB) THEN
!   ZARG5 = -200._JPRB
!   ZARG  = 0.0_JPRB
! ENDIF
! ZEXMU05=EXP(-ZARG5)
! ZEXMU0 =-ZARG *ZEXMU05
! IF (ZRK5*ZTOP5 >= 200._JPRB) THEN
!   ZARG2 = 0.0_JPRB
!   ZARG25= 200._JPRB
! ELSE
!   ZARG2 = ZRK *ZTOP5 + ZRK5*ZTOP
!   ZARG25= ZRK5*ZTOP5
! ENDIF
! ZEXKP5=EXP(ZARG25)
! ZEXKP =ZARG2*ZEXKP5

  ZEXKM5 = 1.0_JPRB/ZEXKP5
  ZEXKM  = -ZEXKP*(ZEXKM5**2)
  ZXP2P  = ZDT5*ZRP
  ZXP2P5 = 1.0_JPRB+ZDT5*ZRP5
  ZXM2P  = - ZDT5*ZRP
  ZXM2P5 = 1.0_JPRB-ZDT5*ZRP5
  ZAP2B  = ZALPHA + ZDT5*ZBETA
  ZAP2B5 = ZALPHA5+ZDT5*ZBETA5
  ZAM2B  = ZALPHA -ZDT5*ZBETA
  ZAM2B5 = ZALPHA5-ZDT5*ZBETA5

!*         1.2     WITHOUT REFLECTION FROM THE UNDERLYING LAYER

! 120 CONTINUE

  ZA22  = ZXP2P *ZEXKP5 + ZXP2P5*ZEXKP
  ZA225 = ZXP2P5*ZEXKP5
  ZA21  = ZXM2P *ZEXKM5 + ZXM2P5*ZEXKM
  ZA215 = ZXM2P5*ZEXKM5
  ZA23  = ZAM2B *ZEXMU05 + ZAM2B5*ZEXMU0
  ZA235 = ZAM2B5*ZEXMU05
  ZDENA  = ZXP2P  * ZA225 - ZA21  * ZXM2P5+ ZXP2P5 * ZA22  - ZA215 * ZXM2P
  ZDENA5 = ZXP2P5 * ZA225 - ZA215 * ZXM2P5
  ZIDENA5 = 1.0_JPRB/ZDENA5
  ZIDENA = (-ZDENA)*(ZIDENA5**2)
  ZC1A  = (ZA22 *ZAP2B5-ZXM2P *ZA235 &
   & +ZA225*ZAP2B -ZXM2P5*ZA23 )*ZIDENA5 &
   & +(ZA225*ZAP2B5-ZXM2P5*ZA235)*ZIDENA  
  ZC1A5 = (ZA225*ZAP2B5-ZXM2P5*ZA235)*ZIDENA5
  ZC2A  = (ZXP2P *ZA235-ZA21 *ZAP2B5 &
   & +ZXP2P5*ZA23 -ZA215*ZAP2B )*ZIDENA5 &
   & +(ZXP2P5*ZA235-ZA215*ZAP2B5)*ZIDENA  
  ZC2A5 = (ZXP2P5*ZA235-ZA215*ZAP2B5)*ZIDENA5
  ZRI0A  = ZC1A +ZC2A -ZALPHA
  ZRI0A5 = ZC1A5+ZC2A5-ZALPHA5
  ZRI1A  = ZRP *(ZC1A5-ZC2A5)+ ZRP5*(ZC1A -ZC2A )-ZBETA
  ZRI1A5 = ZRP5*(ZC1A5-ZC2A5)-ZBETA5
  PRE1 (JL) = (ZRI0A -ZDT5*ZRI1A )*ZPRMUZ5&
   & + (ZRI0A5-ZDT5*ZRI1A5)*ZPRMUZ  
  PRE15(JL) = (ZRI0A5-ZDT5*ZRI1A5)*ZPRMUZ5
  ZRI0B  = ZC1A *ZEXKM5+ZC2A *ZEXKP5-ZALPHA *ZEXMU05 &
   & + ZC1A5*ZEXKM +ZC2A5*ZEXKP -ZALPHA5*ZEXMU0  
  ZRI0B5 = ZC1A5*ZEXKM5+ZC2A5*ZEXKP5-ZALPHA5*ZEXMU05
  ZRI1B  = ZRP *(ZC1A5*ZEXKM5-ZC2A5*ZEXKP5)&
   & + ZRP5*(ZC1A *ZEXKM5-ZC2A *ZEXKP5 &
   & +ZC1A5*ZEXKM -ZC2A5*ZEXKP )&
   & -ZBETA *ZEXMU05-ZBETA5*ZEXMU0  
  ZRI1B5 = ZRP5*(ZC1A5*ZEXKM5-ZC2A5*ZEXKP5)-ZBETA5*ZEXMU05
  PTR1(JL)  = ZEXMU0 +(ZRI0B +ZDT5*ZRI1B )*ZPRMUZ5&
   & + (ZRI0B5+ZDT5*ZRI1B5)*ZPRMUZ  
  PTR15(JL) = ZEXMU05+(ZRI0B5+ZDT5*ZRI1B5)*ZPRMUZ5

!*         1.3     WITH REFLECTION FROM THE UNDERLYING LAYER

! 130 CONTINUE

  ZB21  = ZA21 - PREF (JL) *ZXP2P5*ZEXKM5 &
   & - PREF5(JL) *ZXP2P *ZEXKM5 &
   & - PREF5(JL) *ZXP2P5*ZEXKM  
  ZB215 = ZA215- PREF5(JL) *ZXP2P5*ZEXKM5
  ZB22  = ZA22 - PREF (JL) *ZXM2P5*ZEXKP5 &
   & - PREF5(JL) *ZXM2P *ZEXKP5 &
   & - PREF5(JL) *ZXM2P5*ZEXKP  
  ZB225 = ZA225- PREF5(JL) *ZXM2P5*ZEXKP5
  ZB23  = ZA23 - PREF (JL) *ZEXMU05*(ZAP2B5 - PRMUZ5(JL) )&
   & - PREF5(JL) *ZEXMU0 *(ZAP2B5 - PRMUZ5(JL) )&
   & - PREF5(JL) *ZEXMU05*(ZAP2B  - PRMUZ (JL) )  
  ZB235 = ZA235- PREF5(JL) *ZEXMU05*(ZAP2B5 - PRMUZ5(JL) )
  ZDENB  = ZXP2P * ZB225 - ZB21 * ZXM2P5+ ZXP2P5* ZB22  - ZB215* ZXM2P
  ZDENB5 = ZXP2P5* ZB225 - ZB215* ZXM2P5
  ZIDENB5=1.0_JPRB/ZDENB5
  ZIDENB=-ZDENB*(ZIDENB5**2)
  ZC1B  = (ZB22 *ZAP2B5-ZXM2P *ZB235 &
   & +  ZB225*ZAP2B -ZXM2P5*ZB23 )*ZIDENB5 &
   & + (ZB225*ZAP2B5-ZXM2P5*ZB235)*ZIDENB  
  ZC1B5 = (ZB225*ZAP2B5-ZXM2P5*ZB235)*ZIDENB5
  ZC2B  = (ZXP2P *ZB235-ZB21 *ZAP2B5 &
   & +  ZXP2P5*ZB23 -ZB215*ZAP2B )*ZIDENB5 &
   & + (ZXP2P5*ZB235-ZB215*ZAP2B5)*ZIDENB  
  ZC2B5 = (ZXP2P5*ZB235-ZB215*ZAP2B5)*ZIDENB5
  ZRI0C  = ZC1B +ZC2B -ZALPHA
  ZRI0C5 = ZC1B5+ZC2B5-ZALPHA5
  ZRI1C  = ZRP *(ZC1B5-ZC2B5)+ ZRP5*(ZC1B -ZC2B )-ZBETA
  ZRI1C5 = ZRP5*(ZC1B5-ZC2B5)-ZBETA5
  PRE2 (JL) = (ZRI0C-ZDT5*ZRI1C ) *ZPRMUZ5&
   & + (ZRI0C5-ZDT5*ZRI1C5)*ZPRMUZ  
  PRE25(JL) = (ZRI0C5-ZDT5*ZRI1C5) *ZPRMUZ5
  ZRI0D  = ZC1B *ZEXKM5 + ZC2B *ZEXKP5 - ZALPHA *ZEXMU05 &
   & + ZC1B5*ZEXKM  + ZC2B5*ZEXKP  - ZALPHA5*ZEXMU0  
  ZRI0D5 = ZC1B5*ZEXKM5 + ZC2B5*ZEXKP5 - ZALPHA5*ZEXMU05
  ZRI1D  = ZRP  * (ZC1B5*ZEXKM5 - ZC2B5*ZEXKP5)&
   & + ZRP5 * (ZC1B *ZEXKM5 - ZC2B *ZEXKP5 &
   & + ZC1B5*ZEXKM  - ZC2B5*ZEXKP )&
   & - ZBETA *ZEXMU05 - ZBETA5*ZEXMU0  
  ZRI1D5 = ZRP5 * (ZC1B5*ZEXKM5 - ZC2B5*ZEXKP5) - ZBETA5*ZEXMU05
  PTR2 (JL) = ZEXMU0  + (ZRI0D + ZDT5*ZRI1D ) *ZPRMUZ5&
   & + (ZRI0D5 + ZDT5*ZRI1D5)*ZPRMUZ  
  PTR25(JL) = ZEXMU05 + (ZRI0D5 + ZDT5*ZRI1D5) *ZPRMUZ5

ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SWDETL',1,ZHOOK_HANDLE)
END SUBROUTINE SWDETL
