SUBROUTINE EXPBESU(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,&
 & PZO,LDBELS,LDBLES,POROG,&
 & PRESH,PT,&
 & PPPP)  

!**** *EXPBESU* - EXTRAPOLATE P BELOW SURFACE

!     PURPOSE.
!     --------

!        COMPUTES SURFACE PRESSURE BELOW MODEL'S SURFACE

!**   INTERFACE.
!     ----------

!        *CALL* *EXPBESU(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,
!                        PZO,LDBELS,LDBLES,POROG,
!                        PRESH,PT,
!                        PPPP)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.
!        KSTART                    - START OF WORK.
!        KPROF                     - DEPTH OF WORK.
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION
!        PZO                       - POST-PROCESSING LEVEL HEIGHTS
!        LDBELS                    - .TRUE. IF PRESSURE IS UNDER
!                                     MODEL SURFACE
!        LDBLES                    - .TRUE. IF LDBELS(J) IS CONTAINING
!                                    AT LEAST ONE .TRUE.
!        POROG                     - MODEL OROGRAPHY.
!        PRESH                     - HALF LEVEL PRESSURE
!        PT                        - TEMPERATURE
!        PPPP                      - POST-PROCESSED PRESSURE

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST,YOMGEM,YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS
!     ---------

!        NONE

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        D. VASILJEVIC   *ECMWF*

!     MODIFICATIONS.
!     --------------
!        L. Isaksen : 96-02-20  NFLEV --> NFLEVG
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMCST   , ONLY : RG       ,RD
USE YOMSTA   , ONLY : RDTDZ1
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZO(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELS(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLES(KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESH(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PPPP(KPROMA,KLEVP) 
REAL(KIND=JPRB) ::ZT    (KPROMA             )&
 & ,ZALPHA(KPROMA             )  
INTEGER(KIND=JPIM) :: JL, JLEVP

REAL(KIND=JPRB) :: Z1, Z2, Z3, Z4, ZDTDZSG, ZOROG, ZT0, ZTX, ZTY
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    EXTRAPOLATE PRESSURE BELOW MODEL'S SURFACE
!              ------------------------------------------

!*       1.1   PRE-CALCULATION

IF (LHOOK) CALL DR_HOOK('EXPBESU',0,ZHOOK_HANDLE)
ZTX=290.5_JPRB
ZTY=255.0_JPRB
ZDTDZSG=-RDTDZ1/RG
DO JL=KSTART,KPROF
  IF(PT    (JL) < ZTY) THEN
    ZT    (JL)=0.5_JPRB*(ZTY+PT    (JL))
  ELSEIF(PT    (JL) < ZTX) THEN
    ZT    (JL)=PT    (JL)
  ELSE
    ZT    (JL)=0.5_JPRB*(ZTX+PT    (JL))
  ENDIF

  ZT0=ZT    (JL)+ZDTDZSG*POROG(JL)
  IF(ZTX > ZT    (JL) .AND. ZT0 > ZTX) THEN
    ZT0=ZTX
  ELSEIF(ZTX <= ZT    (JL) .AND. ZT0 > ZT    (JL)) THEN
    ZT0=ZT    (JL)+0.0001_JPRB
  ENDIF

  ZOROG=SIGN(MAX(1.0_JPRB,ABS(POROG(JL))),POROG(JL))
  ZALPHA(JL)=RD*(ZT0-ZT    (JL))/ZOROG
ENDDO

!*       1.2   EXTRAPOLATION

DO JLEVP=KLOLEV,KLEVP
  IF(LDBLES(JLEVP)) THEN
    DO JL=KSTART,KPROF
      IF(LDBELS(JL,JLEVP)) THEN
        Z1            = (POROG(JL)-PZO(JL,JLEVP))*ZALPHA(JL)
        Z2            = Z1/(RD*ZT    (JL))
        Z3            = Z2 + 1.0_JPRB
        Z4            = LOG(Z3)/ZALPHA(JL) +LOG(PRESH(JL,NFLEVG))
        PPPP(JL,JLEVP)= EXP(Z4)
      ENDIF
    ENDDO
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('EXPBESU',1,ZHOOK_HANDLE)
END SUBROUTINE EXPBESU

