SUBROUTINE PPINTZ(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,&
 & KLEVB,KSLCT,KSTL,LDBE,LDBL,PZO,&
 & PRXZ,PRXZD,PFLDI,&
 & PFLDO)  

!**** *PPINTZ* - LINEARLY INTERPOLATE FIELDS TO GIVEN LEVEL

!     PURPOSE.
!     --------
!        PERFORMS THE VERTICAL LINEAR INTERPOLATION OF A FIELD
!    TO A GIVEN LEVEL.

!**   INTERFACE.
!     ----------
!        *CALL* *PPINTZ(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,
!                       KLEVB,KSLCT,KSTL,LDBE,LDBL,PZO,
!                       PRXP,PRXPD,PFLDI,
!                       PFLDO)

!        EXPLICIT ARGUMENTS :
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.
!        KSTART                    - START OF WORK.
!        KPROF                     - DEPTH OF WORK.
!        KFLEV                     - NUMBER OF INPUT LEVELS
!        KLEVP                     - INPUT LEVEL BELOW PZO
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION
!        KLEVB                       INPUT LEVEL BELOW PZO
!        KSLCT                     - INTERPOLATION CHOICE:
!                                    1= FIELD GIVEN ON HALF LEVELS,
!                                       INTERPOLATION IN Z
!                                    2= FIELD GIVEN ON FULL LEVELS,
!                                       INTERPOLATION IN Z
!                                    3= FIELD GIVEN ON HALF LEVELS,
!                                       INTERPOLATION IN LN(Z)
!                                    4= FIELD GIVEN ON FULL LEVELS,
!                                       INTERPOLATION IN LN(Z)
!        KSTL                      - START LEVEL FOR INPUT FIELD
!        LDBE  (KPROMA,KLEVP)      - .TRUE. IF HEIGHT IS UNDER
!                                     LOWEST  MODEL LEVEL
!        LDBL  (KLEVP)             - .TRUE. IF LDBELO IS   CONTAINING
!                                     AT LEAST ONE .TRUE.
!        PZO    (KPROMA,KLEVP)     - POST-PROCESSING LEVEL
!                                    HEIGHTS
!        PRXZ(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF,FULL LEVEL
!                                    HIGHTS (SEE PPINITZ)
!        PRXZD(KPROMA,0:KFLEV,NPPM)- 1./D(Z) AND 1./D(LN(Z))
!        PFLDI(KPROMA,KSTL:KFLEV)  - MODEL LEVEL FIELD VALUES

!        PFLDO(KPROMA,KLEVP)       - INTERPOLATED FIELD VALUES

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  NONE.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        D. VASILJEVIC  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTL 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVB(KPROMA,KLEVP,NPPM) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSLCT 
LOGICAL           ,INTENT(IN)    :: LDBE(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBL(KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PZO(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXZ(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXZD(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLDI(KPROMA,KSTL:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFLDO(KPROMA,KLEVP) 
INTEGER(KIND=JPIM) :: IAB, IBL, JL, JLEVP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    LINEAR INTERPOLATION AND UPWARD EXTRAPOLATION.
!              ----------------------------------------------

!*       1.1   INTERPOLATE

IF (LHOOK) CALL DR_HOOK('PPINTZ',0,ZHOOK_HANDLE)
DO JLEVP=KLOLEV,KLEVP
  IF(.NOT.LDBL  (JLEVP)) THEN
    DO JL=KSTART,KPROF
      IBL=KLEVB(JL,JLEVP,KSLCT)
      IAB=IBL-1
      PFLDO(JL,JLEVP)=PFLDI(JL,IBL)+(PFLDI(JL,IAB)-PFLDI(JL,IBL))*&
       & (PZO    (JL,JLEVP)-PRXZ(JL,IBL,KSLCT))*PRXZD(JL,IAB,KSLCT)  
    ENDDO
  ELSE
    DO JL=KSTART,KPROF
      IF(.NOT.LDBE  (JL,JLEVP)) THEN
        IBL=KLEVB(JL,JLEVP,KSLCT)
        IAB=IBL-1
        PFLDO(JL,JLEVP)=PFLDI(JL,IBL)+(PFLDI(JL,IAB)-PFLDI(JL,&
         & IBL))*&
         & (PZO    (JL,JLEVP)-PRXZ(JL,IBL,KSLCT))*PRXZD(JL,IAB,&
         & KSLCT)  
      ENDIF
    ENDDO
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPINTZ',1,ZHOOK_HANDLE)
END SUBROUTINE PPINTZ

