SUBROUTINE PPITPQAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KSL,&
 & PRPRES,PRXP,PFLD,&
 & PRXP5,PFLD5,&
 & PZPP)  

!**** *PPITPQAD* - POST-PROCESS TOP LEVELS QUADRATICALLY

!     PURPOSE.
!     --------
!           FITS A QUADRATIC POLYNOMIAL TO THE TO THREE LEVELS OF THE
!       MODEL. POST-PROCESSED VALUES ABOVE THE SECOND LEVEL ARE
!       OWER WRITTEN WITH NEW QUADRATICALLY INTERPOLATED VALUES.

!**   INTERFACE.
!     ----------
!        *CALL* *PPITPQAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KSL
!                         PRPRES,PRXP,PFLD,
!                         PRXP5,PFLD5,
!                         PZPP)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION    (INPUT-C)
!        KSL                       - 1:  FIELD GIVEN ON HALF LEVELS.
!                                        INTERPOLATE IN P
!                                  - 2:  FIELD GIVEN ON FULL LEVELS.
!                                        INTERPOLATE IN P
!                                  - 3:  FIELD GIVEN ON HALF LEVELS.
!                                        INTERPOLATE IN LN(P)
!                                  - 4:  FIELD GIVEN ON FULL LEVELS.
!                                        INTERPOLATE IN LN(P)          (INPUT-C)
!        PRPRES(KPROMA,KLEVP)      -  LOG(POST-PROCESSING LEVEL PRES.) (INPUT-C)
!        PRXP(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF,FULL LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT)
!        PFLD(KPROMA,0:KFLEV)      - MODEL LEVEL FIELD VALUES          (INPUT)

!        PRXP5(KPROMA,0:KFLEV,NPPM)- HALF,FULL AND LN HALF,FULL LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT-R)
!        PFLD5(KPROMA,0:KFLEV)     - MODEL LEVEL FIELD VALUES          (INPUT-R)

!        PZPP(KPROMA,KLEVP)        - POST-PROCESSED GEOPOTENTIAL       (OUTPUT)

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  NONE
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        ERIK ANDERSSON  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 92-04-07
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSL 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPRES(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PFLD(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP5(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PFLD5(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PZPP(KPROMA,KLEVP) 
INTEGER(KIND=JPIM) :: I1, I2, I3, JK, JL

REAL(KIND=JPRB) :: ZDEN1, ZDEN15, ZDEN2, ZDEN25
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*       1.    POST-PROCESS TOP LEVELS
!              -----------------------

!*       1.0   SET POINTERS TO TOP LEVELS

IF (LHOOK) CALL DR_HOOK('PPITPQAD',0,ZHOOK_HANDLE)
IF(KSL == 1.OR. KSL == 3) THEN
  I1=0
  I2=1
  I3=2
ELSE
  I1=1
  I2=2
  I3=3
ENDIF

!*       1.2   MODIFY INTERPOLATION/EXTRAPOLATION ABOVE SECOND
!*          LEVEL (JLEV=I2). FIT QUADRATIC POLYNOMIAL IN LN(P) TO THE
!*          GEOPOTENTIAL AT THE TOP THREE LEVELS (JLEV=I1,I2,I3) AND
!*          APPLY TO PRESSURES ABOVE THE SECOND LEVEL FROM THE TOP
!*          (JLEV=I2).

DO JK=KLOLEV,KLEVP
  DO JL=KSTART,KPROF
    IF(PRPRES(JL,JK) < PRXP5(JL,I2,KSL)) THEN
      ZDEN15=(PRXP5(JL,I2,KSL)-PRXP5(JL,I1,KSL))&
       & *(PRXP5(JL,I1,KSL)-PRXP5(JL,I3,KSL))  
      ZDEN25=(PRXP5(JL,I2,KSL)-PRXP5(JL,I3,KSL))&
       & *(PRXP5(JL,I1,KSL)-PRXP5(JL,I3,KSL))  
      ZDEN1=0.0_JPRB
      ZDEN2=0.0_JPRB

      ZDEN1=ZDEN1 &
       & -(PFLD5(JL,I2)-PFLD5(JL,I1))*(PRPRES(JL,JK)-PRXP5(JL,I2,KSL))&
       & *(PRPRES(JL,JK)-PRXP5(JL,I3,KSL))&
       & /(ZDEN15*ZDEN15) * PZPP(JL,JK)  
      ZDEN2=ZDEN2 &
       & +(PFLD5(JL,I2)-PFLD5(JL,I3))*(PRPRES(JL,JK)-PRXP5(JL,I1,KSL))&
       & *(PRPRES(JL,JK)-PRXP5(JL,I2,KSL))&
       & /(ZDEN25*ZDEN25) * PZPP(JL,JK)  

      PFLD(JL,I1)=PFLD(JL,I1)&
       & -(PRPRES(JL,JK)-PRXP5(JL,I2,KSL))*(PRPRES(JL,JK)-PRXP5(JL,I3,KSL))&
       & /ZDEN15 * PZPP(JL,JK)  
      PFLD(JL,I2)=PFLD(JL,I2) + PZPP(JL,JK)&
       & +(PRPRES(JL,JK)-PRXP5(JL,I2,KSL))*(PRPRES(JL,JK)-PRXP5(JL,I3,KSL))&
       & /ZDEN15 * PZPP(JL,JK)&
       & -(PRPRES(JL,JK)-PRXP5(JL,I1,KSL))*(PRPRES(JL,JK)-PRXP5(JL,I2,KSL))&
       & /ZDEN25 * PZPP(JL,JK)  
      PFLD(JL,I3)=PFLD(JL,I3)&
       & +(PRPRES(JL,JK)-PRXP5(JL,I1,KSL))*(PRPRES(JL,JK)-PRXP5(JL,I2,KSL))&
       & /ZDEN25 * PZPP(JL,JK)  

      PRXP(JL,I1,KSL)=PRXP(JL,I1,KSL)&
       & +(PFLD5(JL,I2)-PFLD5(JL,I3))*(PRPRES(JL,JK)-PRXP5(JL,I2,KSL))&
       & /ZDEN25 * PZPP(JL,JK)  
      PRXP(JL,I2,KSL)=PRXP(JL,I2,KSL)&
       & -(PFLD5(JL,I2)-PFLD5(JL,I1))*(PRPRES(JL,JK)-PRXP5(JL,I3,KSL))&
       & /ZDEN15 * PZPP(JL,JK)&
       & +(PFLD5(JL,I2)-PFLD5(JL,I3))*(PRPRES(JL,JK)-PRXP5(JL,I1,KSL))&
       & /ZDEN25 * PZPP(JL,JK)  
      PRXP(JL,I3,KSL)=PRXP(JL,I3,KSL)&
       & -(PFLD5(JL,I2)-PFLD5(JL,I1))*(PRPRES(JL,JK)-PRXP5(JL,I2,KSL))&
       & /ZDEN15 * PZPP(JL,JK)  

      PRXP(JL,I1,KSL)=PRXP(JL,I1,KSL)&
       & -(PRXP5(JL,I1,KSL)-PRXP5(JL,I3,KSL))*ZDEN1 &
       & +(PRXP5(JL,I2,KSL)-PRXP5(JL,I1,KSL))*ZDEN1 &
       & +(PRXP5(JL,I2,KSL)-PRXP5(JL,I3,KSL))*ZDEN2  
      PRXP(JL,I2,KSL)=PRXP(JL,I2,KSL)&
       & +(PRXP5(JL,I1,KSL)-PRXP5(JL,I3,KSL))*ZDEN1 &
       & +(PRXP5(JL,I1,KSL)-PRXP5(JL,I3,KSL))*ZDEN2  
      PRXP(JL,I3,KSL)=PRXP(JL,I3,KSL)&
       & -(PRXP5(JL,I2,KSL)-PRXP5(JL,I1,KSL))*ZDEN1 &
       & -(PRXP5(JL,I1,KSL)-PRXP5(JL,I3,KSL))*ZDEN2 &
       & -(PRXP5(JL,I2,KSL)-PRXP5(JL,I3,KSL))*ZDEN2  

      PZPP(JL,JK)=0.0_JPRB
    ENDIF
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPITPQAD',1,ZHOOK_HANDLE)
END SUBROUTINE PPITPQAD
