SUBROUTINE PPOBSAP(KDLEN,KDBDY,KLEN,KDCOUNT,KPNLV,CDPAR,&
 & KCOUNT,PRESO,PTF5,PQF5,PCLWF5,PCIWF5,PUF5,PVF5,PTS5,PRESH5,PALTI,POROG,&
 & KDPP,PXPP)  
!****-------------------------------------------------------------------
!****  PPOBSAP : UPPERAIR POST-PROCESSING CALLING APACHE
!****            FIND OUT AN EQUIVALENT MODEL PARAMETER FOR A GIVEN OBS
!****            called from HOP
!****-------------------------------------------------------------------

!     WHERE KDLEN   : FIRST DIMENSION (OBSERVATIONS)
!           KDBDY   : Second dimension: Max number of bodies, this set
!           KLEN    : ACTUAL NUMBER OF OBSERVATIONS
!           KDCOUNT : SECOND DIMENSION (Data)
!           CDPAR   : PARAMETER FOR POST-PROCESSING
!           KCOUNT  : COUNTER OF DATA PER OBS
!           PALTI   : OBS. OROGRAPHY
!           POROG   : MODEL OROGRAPHY
!           PRESH5  : HALF LEVEL PRESS. AT OBS. POINTS
!           PUF5    : U VALUES (MODEL) AT OB. POINS
!           PVF5    : V VALUES (MODEL) AT OB. POINS
!           PTF5    : T VALUES (MODEL) AT OB. POINS
!           PQF5    : Q VALUES (MODEL) AT OB. POINS
!           PCLWF5  : QL VALUES (MODEL) AT OB. POINS
!           PCIWF5  : QI VALUES (MODEL) AT OB. POINS
!           PTS5    : SURFACE TEMPERATURE
!           KDPP    : Number of fields, dimension of PXPP.
!           PXPP    : POST-PROCESSED VALUES

!**   EXTERNALS : APACHE
!     ---------
!     MODIFIED :
!     --------
!        P.Moll      99-10-19 : implementation in cy22  
!        F.Taillefer 00-09-10 : use of ODB instead of CMAFOC
!        R. El Khatib : 01-08-07 Pruning options
!        Y. Bouteloup : 05-01-13  Introduction of cloud condensate liquid and ice
!        Y. Bouteloup : 05-12-25  Set ZXF5 to 0 for security.
!****-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY : LCANARI
USE YOMCST   , ONLY : RG       ,RATM
USE YOMSTA   , ONLY : RDTDZ1
USE YOMANCS  , ONLY : RMDI
USE YOMGEM   , ONLY : VBH      ,VAH
USE YOMDB
USE YOM_YGFL , ONLY : YI,YL

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDCOUNT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDBDY
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPNLV(KDLEN,KDBDY) 
CHARACTER(LEN=3)  ,INTENT(IN)    :: CDPAR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCOUNT(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESO(KDLEN,KDCOUNT) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCLWF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCIWF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVF5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTS5(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRESH5(KDLEN,0:NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PALTI(KDLEN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KDLEN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDPP
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PXPP(KDLEN,KDCOUNT,KDPP) 
REAL(KIND=JPRB) :: ZDPBLC,ZV00,ZDTDZSG
REAL(KIND=JPRB) :: ZTS2(KDLEN),ZEPS2(KDLEN)
REAL(KIND=JPRB), DIMENSION(0:KDCOUNT) :: ZVAH2, ZVBH2
REAL(KIND=JPRB), DIMENSION(KDLEN,0:NFLEVG) :: ZXF5
REAL(KIND=JPRB), DIMENSION(KDLEN,KDCOUNT) :: ZUPP,ZVPP,ZTPP,ZHPP,ZQPP,ZQLPP,ZQIPP,ZZPP,ZPRESO2
REAL(KIND=JPRB) :: ZXPP(KDLEN,KDCOUNT), ZXPP2(KDLEN,KDCOUNT,0:1)
INTEGER(KIND=JPIM) :: IBODY,JOBS,JWAG
LOGICAL :: LLWAT,LLICE,LLRAIN,LLCFR,LLO3,LLNIVP,LLOBS,LLSPCH
LOGICAL :: LLSN,LLRR,LLGR,LLHL,LLTKE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "apache.intfb.h"

#include "openmp_obs.h"

!****-------------------------------------------------------------------

!   1. various initialisations for APACHE

IF (LHOOK) CALL DR_HOOK('PPOBSAP',0,ZHOOK_HANDLE)
LLWAT=YL%LACTIVE 
LLICE=YI%LACTIVE 
LLRAIN=.FALSE.
LLCFR=.FALSE.
LLO3=.FALSE.
LLNIVP=.TRUE.
LLOBS=.TRUE.
LLSPCH=.FALSE.
LLSN=.FALSE.
LLRR=.FALSE.
LLGR=.FALSE.
LLHL=.FALSE.
LLTKE=.FALSE.
ZDPBLC=17500._JPRB
ZV00=RATM
ZDTDZSG=-RDTDZ1/RG
DO JOBS=1,KLEN
  IF(PALTI(JOBS) == RMDI .OR. PALTI(JOBS) > 90000._JPRB) THEN
    ZTS2(JOBS)=PTS5(JOBS)
    PALTI(JOBS)=POROG(JOBS)
  ELSE
    ZTS2(JOBS)=PTS5(JOBS)+RDTDZ1*(PALTI(JOBS)-POROG(JOBS))/RG
  ENDIF
ENDDO

ZXF5(:,:) = 0.0_JPRB
CALL APACHE(LLNIVP,KDLEN,1,KLEN,NFLEVG,KDCOUNT,&
 & PTF5,PQF5,PUF5,PVF5,PRESH5(1,NFLEVG),POROG,VAH,VBH,&
 & PCLWF5,PCIWF5,ZXF5,ZXF5,ZXF5,ZXF5,ZXF5,ZXF5,ZXF5,ZXF5,&
 & ZXF5,.FALSE.,.FALSE.,.FALSE.,&
 & LLWAT,LLICE,LLCFR,LLO3,&
 & LLSN,LLRR,LLGR,LLHL,LLTKE,&
 & ZTPP,ZQPP,ZUPP,ZVPP,ZEPS2,PALTI,ZVAH2,ZVBH2,&
 & ZQLPP,ZQIPP,ZXPP,ZXPP,ZXPP,ZXPP,ZXPP,ZXPP,ZXPP,ZXPP,ZXPP2,ZXPP,&
 & ZZPP,ZHPP,ZPRESO2,LLOBS,&
 & PRESO,ZXPP2,PTS5,ZTS2,ZDPBLC,ZV00,LLSPCH,&
 & ZDTDZSG)  

!   2. fill the output arry PXPP

IF (CDPAR == 'H') THEN
  PXPP(1:KLEN,1:KDCOUNT,1) = ZHPP(1:KLEN,1:KDCOUNT)
ELSEIF (CDPAR == 'U') THEN
  PXPP(1:KLEN,1:KDCOUNT,1) = ZUPP(1:KLEN,1:KDCOUNT)
  PXPP(1:KLEN,1:KDCOUNT,2) = ZVPP(1:KLEN,1:KDCOUNT)
ELSEIF (CDPAR == 'T') THEN
  PXPP(1:KLEN,1:KDCOUNT,1) = ZTPP(1:KLEN,1:KDCOUNT)
ELSEIF (CDPAR == 'Q') THEN
  PXPP(1:KLEN,1:KDCOUNT,1) = ZQPP(1:KLEN,1:KDCOUNT)
ELSEIF (CDPAR == 'CLW') THEN
  PXPP(1:KLEN,1:KDCOUNT,1) = ZQLPP(1:KLEN,1:KDCOUNT)
ELSEIF (CDPAR == 'CIW') THEN
  PXPP(1:KLEN,1:KDCOUNT,1) = ZQIPP(1:KLEN,1:KDCOUNT)
ELSEIF (CDPAR == 'Z') THEN
  PXPP(1:KLEN,1:KDCOUNT,1) = ZZPP(1:KLEN,1:KDCOUNT)
ELSE
  WRITE(NULOUT,'('' *** ERROR IN PPOBSAP'')')
  WRITE(NULOUT,'('' *** REQUESTED PARAMETER='',A)') CDPAR
ENDIF

!   3. modify the "observed" pressure to be used in CANARI

IF(LCANARI)THEN
  DO JOBS = 1 , KLEN
    DO JWAG = 1 , KCOUNT(JOBS)
      IBODY = MLNKH2B(JOBS)+KPNLV(JOBS,JWAG)-1
      ROBODY(IBODY,MDBRBPIO) = LOG(ZPRESO2(JOBS,JWAG))
    ENDDO
  ENDDO
ENDIF

!**---------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PPOBSAP',1,ZHOOK_HANDLE)
END SUBROUTINE PPOBSAP
