SUBROUTINE PPT2M(&
 & KPROMA,KIDIA,KFDIA,PINTH,&
 & PQ,PAPHI,&
 & PCPTGZ,PCPTS,&
 & PRI,PBN,PBH,&
 & PT2M)  

!**** *PPT2M* - POST-PROCESSED 2 METRES TEMPERATURE

!     PURPOSE
!     -------
!          COMPUTE FROM THE MODEL VALUES AT THE OBSERVATION
!          POINT THE TEMPERATURE AT 2 METRES

!**   INTERFACE
!     ---------
!       *CALL* *PPT2M(
!     C               KPROMA,KIDIA,KFDIA,NFLEVG,
!     I               ZQF5(1,NFLEVG),ZAPHI(1,NFLEVG),
!     I               PCPTGZ,PCPTS,
!     I               ZRI,ZBN,ZBH,
!     O               PT2M)

!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA            - HORIZONTAL DIMENSION.                   (INPUT-C)
!        KIDIA             - INDEX OF START POINT                    (INPUT-C)
!        KFDIA             - INDEX OF END   POINT                    (INPUT-C)
!        PQ(KPROMA)        - SPECIFIC HUMIDITY               Kg/Kg   (INPUT)
!        PAPHI(KPROMA)     - GEOPOTENTIAL MODEL LEVELS     M**2/S**2 (INPUT)
!        PCPTGZ(KPROMA)    - STATIC ENERGY LOWER LEVEL     M**2/S**2 (INPUT)
!        PCPTS(KPROMA)     - STATIC ENERGY AT THE SURFACE  M**2/S**2 (INPUT)
!        PRI(KPROMA)       - RICHARSON NUMBER                        (OUTPUT)
!        PBN(KPROMA)       - NEUTRAL COEFFICIENT                     (OUTPUT)
!        PBH(KPROMA)       - HEAT COEFFICIENT                        (OUTPUT)
!        PT2M(KPROMA)      - POST-PROCESSED TEMPERATUE               (OUTPUT)   TPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        CARLA CARDINALI *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL   : 92-05-22
!        L. Isaksen : 96-02-20  NFLEV --> NFLEVG
!        E. Andersson 97-04-10  Change back from 15m to 2m.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY :        RCPD
USE YOETHF   , ONLY : RVTMP2

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINTH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPTGZ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT2M(KPROMA) 
INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZCBN, ZCBS, ZCBU, ZH2M, ZRAT, ZRED
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     1.     DEFINITION OF THE CONSTANTS
!            ---------------------------
!     ZHTQ=2.*RG
!     ZHTQ=PINTH

!     2.    POST-PROCESSED TEMPERATURE

IF (LHOOK) CALL DR_HOOK('PPT2M',0,ZHOOK_HANDLE)
DO JL=KIDIA,KFDIA
!       ZRAT=ZHTQ/PAPHI(JL)
  ZRAT=PINTH(JL)/PAPHI(JL)
  ZCBN=LOG(1.0_JPRB+(EXP(PBN(JL))-1.0_JPRB)*ZRAT)
  ZCBS=-(PBN(JL)-PBH(JL))*ZRAT
  ZCBU=-LOG(1.0_JPRB+(EXP(PBN(JL)-PBH(JL))-1.0_JPRB)*ZRAT)
  IF(PRI(JL) >= 0.0_JPRB) THEN
    ZRED=(ZCBN+ZCBS)/PBH(JL)
  ELSE
    ZRED=(ZCBN+ZCBU)/PBH(JL)
  ENDIF
  ZH2M=PCPTS(JL)+ZRED*(PCPTGZ(JL)-PCPTS(JL))
!       PT2M(JL)=(ZH2M-ZHTQ)/(RCPD*(1.+RVTMP2*PQ(JL)))
  PT2M(JL)=(ZH2M-PINTH(JL))/(RCPD*(1.0_JPRB+RVTMP2*PQ(JL)))
ENDDO

!     ----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPT2M',1,ZHOOK_HANDLE)
END SUBROUTINE PPT2M
