SUBROUTINE PPT2MAD(&
 & KPROMA,KIDIA,KFDIA,PINTH,&
 & PQ5,PAPHI5,&
 & PQ,PAPHI,&
 & PCPTGZ5,PCPTS5,&
 & PCPTGZ,PCPTS,&
 & PRI5,PBN5,PBH5,&
 & PBN,PBH,&
 & PT2M)  

!**** *PPT2MAD* - AJOINT OF TANGENT LINEAR OF 2 METRES TEMPERATURE

!     PURPOSE
!     -------
!          COMPUTE THE ADJOINT OF TANGENT LINEAR OF THE INTERPOLATED
!          TEMPERATURE AT 2 METRES

!**   INTERFACE
!     ---------
!       *CALL* *PPT2MAD(
!     C               KPROMA,KIDIA,KFDIA,NFLEVG,
!     I               PQF5(1,NFLEVG),PAPHI5(1,NFLEVG),
!     I               PQF(1,NFLEVG),PAPHI(1,NFLEVG),
!     I               PCPTGZ5,PCPTS5,
!     I               PCPTGZ,PCPTS,
!     I               PRI5,PBN5,PBH5,
!     I               PBN,PBH,
!     O               PT2M)

!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA            - HORIZONTAL DIMENSION.                   (INPUT-C)
!        KIDIA             - INDEX OF START POINT                    (INPUT-C)
!        KFDIA             - INDEX OF END   POINT                    (INPUT-C)
!        PQ(KPROMA)        - SPECIFIC HUMIDITY               Kg/Kg   (INPUT)
!        PAPHI(KPROMA)     - GEOPOTENTIAL MODEL LEVELS     M**2/S**2 (INPUT)
!        PCPTGZ(KPROMA)    - STATIC ENERGY LOWER LEVEL     M**2/S**2 (INPUT)
!        PCPTS(KPROMA)     - STATIC ENERGY AT THE SURFACE  M**2/S**2 (INPUT)
!        PRI(KPROMA)       - RICHARSON NUMBER                        (OUTPUT)
!        PBN(KPROMA)       - NEUTRAL COEFFICIENT                     (OUTPUT)
!        PBH(KPROMA)       - HEAT COEFFICIENT                        (OUTPUT)
!        PT2M(KPROMA)      - POST-PROCESSED TEMPERATUE               (OUTPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        CARLA CARDINALI *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL   : 92-05-22
!        M.Hamrud   : 95-09-25 Some cleaning, vectorization problems
!        L. Isaksen : 96-02-20  NFLEV --> NFLEVG
!        E. Andersson 97-04-10  Change back from 15m to 2m.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY :        RCPD
USE YOETHF   , ONLY : RVTMP2

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINTH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPTGZ5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPTS5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCPTGZ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCPTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBN5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBH5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT2M(KPROMA) 
INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZCBN, ZCBN5, ZCBS, ZCBS5, ZCBU, ZCBU5, ZH2M,&
 & ZH2M5, ZRAT, ZRAT5, ZRED, ZRED5  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     1.     DEFINITION OF THE CONSTANTS
!            ---------------------------
!     ZHTQ=2.*RG

!     2.    ADJOINT COMPUTATION

IF (LHOOK) CALL DR_HOOK('PPT2MAD',0,ZHOOK_HANDLE)
DO JL=KIDIA,KFDIA
!       ZRAT5=ZHTQ/PAPHI5(JL)
  ZRAT5=PINTH(JL)/PAPHI5(JL)
  ZCBN5=LOG(1.0_JPRB+(EXP(PBN5(JL))-1.0_JPRB)*ZRAT5)
  ZCBS5=-(PBN5(JL)-PBH5(JL))*ZRAT5
  ZCBU5=-LOG(1.0_JPRB+(EXP(PBN5(JL)-PBH5(JL))-1.0_JPRB)*ZRAT5)
  IF(PRI5(JL) >= 0.0_JPRB) THEN
    ZRED5=(ZCBN5+ZCBS5)/PBH5(JL)
  ELSE
    ZRED5=(ZCBN5+ZCBU5)/PBH5(JL)
  ENDIF
  ZH2M5=PCPTS5(JL)+ZRED5*(PCPTGZ5(JL)-PCPTS5(JL))

  ZH2M=0.0_JPRB
  ZRED=0.0_JPRB
  ZCBN=0.0_JPRB
  ZCBS=0.0_JPRB
  ZCBU=0.0_JPRB
  ZRAT=0.0_JPRB
  ZH2M=ZH2M+1.0_JPRB/(RCPD*(1.0_JPRB+RVTMP2*PQ5(JL)))*PT2M(JL)
!       PQ(JL)=PQ(JL)-(ZH2M5-ZHTQ)/(RCPD*(1.+RVTMP2*PQ5(JL))**2)
!    &          *RVTMP2*PT2M(JL)
  PQ(JL)=PQ(JL)-(ZH2M5-PINTH(JL))/(RCPD*(1.0_JPRB+RVTMP2*PQ5(JL))**2)&
   & *RVTMP2*PT2M(JL)  
  PCPTS(JL)=PCPTS(JL)+(1.0_JPRB-ZRED5)*ZH2M
  PCPTGZ(JL)=PCPTGZ(JL)+ZRED5*ZH2M
  ZRED=ZRED+(PCPTGZ5(JL)-PCPTS5(JL))*ZH2M
  IF(PRI5(JL) >= 0.0_JPRB) THEN
    ZCBN=ZCBN+1.0_JPRB/PBH5(JL)*ZRED
    ZCBS=ZCBS+1.0_JPRB/PBH5(JL)*ZRED
    PBH(JL)=PBH(JL)-(ZCBN5+ZCBS5)/PBH5(JL)**2*ZRED
  ELSE
    ZCBN=ZCBN+1.0_JPRB/PBH5(JL)*ZRED
    ZCBU=ZCBU+1.0_JPRB/PBH5(JL)*ZRED
    PBH(JL)=PBH(JL)-(ZCBN5+ZCBU5)/PBH5(JL)**2*ZRED
  ENDIF
  PBN(JL)=PBN(JL)-1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL)-PBH5(JL))-1.0_JPRB)*ZRAT5)*&
   & EXP(PBN5(JL)-PBH5(JL))*ZRAT5*ZCBU  
  PBH(JL)=PBH(JL)+1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL)-PBH5(JL))-1.0_JPRB)*ZRAT5)*&
   & EXP(PBN5(JL)-PBH5(JL))*ZRAT5*ZCBU  
  ZRAT=ZRAT-1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL)-PBH5(JL))-1.0_JPRB)*ZRAT5)&
   & *(EXP(PBN5(JL)-PBH5(JL))-1.0_JPRB)*ZCBU  
  PBN(JL)=PBN(JL)-ZRAT5*ZCBS
  PBH(JL)=PBH(JL)+ZRAT5*ZCBS
  ZRAT=ZRAT-(PBN5(JL)-PBH5(JL))*ZCBS
  PBN(JL)=PBN(JL)+1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL))-1.0_JPRB)*ZRAT5)*&
   & EXP(PBN5(JL))*ZRAT5*ZCBN  
  ZRAT=ZRAT+1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL))-1.0_JPRB)*ZRAT5)*&
   & (EXP(PBN5(JL))-1.0_JPRB)*ZCBN  
!       PAPHI(JL)=PAPHI(JL)-ZHTQ/PAPHI5(JL)**2*ZRAT
  PAPHI(JL)=PAPHI(JL)-PINTH(JL)/PAPHI5(JL)**2*ZRAT
ENDDO

!     ----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPT2MAD',1,ZHOOK_HANDLE)
END SUBROUTINE PPT2MAD
