SUBROUTINE PPT2MTL(&
 & KPROMA,KIDIA,KFDIA,PINTH,&
 & PQ5,PAPHI5,&
 & PQ,PAPHI,&
 & PCPTGZ5,PCPTS5,&
 & PCPTGZ,PCPTS,&
 & PRI5,PBN5,PBH5,&
 & PBN,PBH,&
 & PT2M)  

!**** *PPT2MTL* - TANGENT LINEAR OF 2 METRES TEMPERATURE

!     PURPOSE
!     -------
!          COMPUTE THE TANGENT LINEAR OF THE INTERPOLATED
!          TEMPERATURE AT 2 METRES

!**   INTERFACE
!     ---------
!       *CALL* *PPT2MTL(
!     C               KPROMA,KIDIA,KFDIA,NFLEVG,
!     I               PQF5(1,NFLEVG),PAPHI5(1,NFLEVG),
!     I               PQF(1,NFLEVG),PAPHI(1,NFLEVG),
!     I               PCPTGZ5,PCPTS5,
!     I               PCPTGZ,PCPTS,
!     I               PRI5,PBN5,PBH5,
!     I               PBN,PBH,
!     O               PT2M)

!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA            - HORIZONTAL DIMENSION.                   (INPUT-C)
!        KIDIA             - INDEX OF START POINT                    (INPUT-C)
!        KFDIA             - INDEX OF END   POINT                    (INPUT-C)
!        PQ(KPROMA)        - SPECIFIC HUMIDITY               Kg/Kg   (INPUT)
!        PAPHI(KPROMA)     - GEOPOTENTIAL MODEL LEVELS     M**2/S**2 (INPUT)
!        PCPTGZ(KPROMA)    - STATIC ENERGY LOWER LEVEL     M**2/S**2 (INPUT)
!        PCPTS(KPROMA)     - STATIC ENERGY AT THE SURFACE  M**2/S**2 (INPUT)
!        PRI(KPROMA)       - RICHARSON NUMBER                        (OUTPUT)
!        PBN(KPROMA)       - NEUTRAL COEFFICIENT                     (OUTPUT)
!        PBH(KPROMA)       - HEAT COEFFICIENT                        (OUTPUT)
!        PT2M(KPROMA)      - POST-PROCESSED TEMPERATUE               (OUTPUT)   TPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        CARLA CARDINALI *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL   : 92-05-22
!        L. Isaksen : 96-02-20  NFLEV --> NFLEVG
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY :        RCPD
USE YOETHF   , ONLY : RVTMP2

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINTH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPTGZ5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPTS5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPTGZ(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCPTS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBN5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBH5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PT2M(KPROMA) 
INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZCBN, ZCBN5, ZCBS, ZCBS5, ZCBU, ZCBU5, ZH2M,&
 & ZH2M5, ZRAT, ZRAT5, ZRED, ZRED5  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     1.     DEFINITION OF THE CONSTANTS
!            ---------------------------
!     ZHTQ=2.*RG

!     2.    TANGENT LINEAR COMPUTATION

IF (LHOOK) CALL DR_HOOK('PPT2MTL',0,ZHOOK_HANDLE)
DO JL=KIDIA,KFDIA
!       ZRAT5=ZHTQ/PAPHI5(JL)
  ZRAT5=PINTH(JL)/PAPHI5(JL)
  ZCBN5=LOG(1.0_JPRB+(EXP(PBN5(JL))-1.0_JPRB)*ZRAT5)
  ZCBS5=-(PBN5(JL)-PBH5(JL))*ZRAT5
  ZCBU5=-LOG(1.0_JPRB+(EXP(PBN5(JL)-PBH5(JL))-1.0_JPRB)*ZRAT5)
  IF(PRI5(JL) >= 0.0_JPRB) THEN
    ZRED5=(ZCBN5+ZCBS5)/PBH5(JL)
  ELSE
    ZRED5=(ZCBN5+ZCBU5)/PBH5(JL)
  ENDIF
  ZH2M5=PCPTS5(JL)+ZRED5*(PCPTGZ5(JL)-PCPTS5(JL))
!       ZRAT=-ZHTQ/PAPHI5(JL)**2*PAPHI(JL)
  ZRAT=-PINTH(JL)/PAPHI5(JL)**2*PAPHI(JL)
  ZCBN=1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL))-1.0_JPRB)*ZRAT5)*&
   & ( EXP(PBN5(JL))*ZRAT5*PBN(JL)&
   & +(EXP(PBN5(JL))-1.0_JPRB)*ZRAT )  
  ZCBS=-(PBN(JL)-PBH(JL))*ZRAT5-(PBN5(JL)-PBH5(JL))*ZRAT
  ZCBU=-1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL)-PBH5(JL))-1.0_JPRB)*ZRAT5)*&
   & ( EXP(PBN5(JL)-PBH5(JL))*ZRAT5*(PBN(JL)-PBH(JL))&
   & +(EXP(PBN5(JL)-PBH5(JL))-1.0_JPRB)*ZRAT )  
  IF(PRI5(JL) >= 0.0_JPRB) THEN
    ZRED=1.0_JPRB/PBH5(JL)*(ZCBN+ZCBS)-(ZCBN5+ZCBS5)/PBH5(JL)**2*PBH(JL)
  ELSE
    ZRED=1.0_JPRB/PBH5(JL)*(ZCBN+ZCBU)-(ZCBN5+ZCBU5)/PBH5(JL)**2*PBH(JL)
  ENDIF
  ZH2M=(1.0_JPRB-ZRED5)*PCPTS(JL)+ZRED5*PCPTGZ(JL)+(PCPTGZ5(JL)-PCPTS5(JL))*ZRED
!       PT2M(JL)=1./(RCPD*(1.+RVTMP2*PQ5(JL)))*ZH2M
!    &          -(ZH2M5-ZHTQ)/(RCPD*(1.+RVTMP2*PQ5(JL))**2)
!    &          *RVTMP2*PQ(JL)
  PT2M(JL)=1.0_JPRB/(RCPD*(1.0_JPRB+RVTMP2*PQ5(JL)))*ZH2M &
   & -(ZH2M5-PINTH(JL))/(RCPD*(1.0_JPRB+RVTMP2*PQ5(JL))**2)&
   & *RVTMP2*PQ(JL)  
ENDDO

!     ----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPT2MTL',1,ZHOOK_HANDLE)
END SUBROUTINE PPT2MTL
