SUBROUTINE PPTAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,&
 & PRES,PLNPRES,LDBELO,LDBELS,LDBLOW,LDBLES,LDEXTR,&
 & POROG,PRXP,PRXPD,PTSTAR,PT0,PR0,PTF,PTPP,&
 & PRXP5,PRXPD5,PTSTAR5,PT05,PR05,PTF5)  

!**** *PPTAD* - POST-PROCESS TEMPERATURE - ADJOINT.

!     PURPOSE.
!     --------
!           PERFORMS THE VERTICAL INTERPOLATION OF TEMPERATURETO A GIVEN
!       MODEL CO-ORDINATE LEVEL. ALSO EXTRAPOLATES TEMPERATURE
!       BELOW SURFACE AND BOTTOM MODEL LEVEL IF SO REQUESTED.

!**   INTERFACE.
!     ----------
!        *CALL* *PPTAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,
!                    PRPRES,POROG,LDBELO,LDBELS,LDBLOW,LDBLES,LDEXTR,
!                    PRXP,PRXPD,PTSTAR,PT0,
!                    PTF,PTPP,PRXP5,PRXPD5,PTSTAR5,PT05,PTF5)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        KLEVB(KPROMA,KLEVP,NPPM)  - INPUT LEVEL BELOW PRPRES          (INPUT-C)
!                                    (SEE PPFLEV)
!        PRPRES(KPROMA,KFLEV)       - POST-PROCESS   LEVEL PRESSURES.   (INPUT-C)
!        PLNPRT(KPROMA,KLEVP)      - PLNPRT(.,JLEVP)=
!                                     LOG(PRPRES(.,JLEVP)/PSOL)        (INPUT-C)
!        POROG(KPROMA)             - MODEL OROGRAPHY.                  (INPUT-C)
!        LDBELO(KPROMA,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     LOWEST (FULL) MODEL LEVEL        (INPUT-C)
!        LDBELS(KPROMA,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     MODEL SURFACE                    (INPUT-C)
!        LDBLOW(KLEVP)             - FIRST LEVEL OF LDBELO CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT-C)
!        LDBLES(KLEVP)             - FIRST LEVEL OF LDBELS CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT-C)
!        LDEXTR                    - .TRUE. IF EXTRAPOLATION REQUESTED (INPUT-C)
!        PRXP(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF,FULL LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT)
!        PRXPD(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))           (INPUT)
!        PTSTAR(KPROMA)            - SURFACE TEMPERATURE               (INPUT)
!        PT0(KPROMA)               - STANDARD SURFACE TEMPERATURE      (INPUT)
!        PTF(KPROMA,0:KFLEV)       - TEMPERATURE ON FULL INPUT LEVELS. (INPUT)

!        PTPP(KPROMA,KLEVP)        - POST-PROCESSED TEMPERATURE        (OUTPUT)

!        PRXP5(KPROMA,0:KFLEV,NPPM)- HALF,FULL AND LN HALF,FULL LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT-T)
!        PRXPD5(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))          (INPUT-T)
!        PTSTAR5(KPROMA)           - SURFACE TEMPERATURE               (INPUT-T)
!        PT05(KPROMA)              - STANDARD SURFACE TEMPERATURE      (INPUT-T)
!        PTF5(KPROMA,0:KFLEV)      - TEMPERATURE ON FULL INPUT LEVELS. (INPUT-T)

!        IMPLICIT ARGUMENTS :  CONSTANTS FROM YOMCST AND YOMSTA.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  PPINTP - LINEAR INTERPOLATION
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 91-01-16
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
USE YOMCST   , ONLY : RG       ,RD
USE YOMSTA   , ONLY : RDTDZ1
USE YOMCT0   , ONLY : LOLDPP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVB(KPROMA,KLEVP,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRES(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPRES(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELO(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELS(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLOW(KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLES(KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDEXTR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PRXPD(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTSTAR(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PT0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PR0(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTPP(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP5(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXPD5(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSTAR5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT05(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PR05(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTF5(KPROMA,0:KFLEV) 

REAL(KIND=JPRB) ::ZTPP5(KPROMA,KLEVP)
REAL(KIND=JPRB) :: ZTF(KPROMA,0:KFLEV),       ZSTZF(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZALPHA(KPROMA)
REAL(KIND=JPRB) :: ZALPHA5(KPROMA),           ZTF5(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZSTZP(KPROMA,KLEVP)
REAL(KIND=JPRB) :: ZR0(KPROMA,0:KFLEV),       ZRP(KPROMA,KLEVP)
REAL(KIND=JPRB) :: ZR05(KPROMA,0:KFLEV),      ZRP5(KPROMA,KLEVP)

INTEGER(KIND=JPIM) :: IBL, ISLCT, JL, JLEV, JLEVP

REAL(KIND=JPRB) :: ZALP, ZALP5, ZALPC, ZFI2000, ZFI2500, ZGAM,&
 & ZGAM5, ZS, ZS5, ZTXX, ZUSDFI, ZXX, ZXX5, &
 & ZXXX  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ppintp.intfb.h"
#include "ppintpad.intfb.h"
#include "ppsta.intfb.h"
#include "ppstaad.intfb.h"
#include "pptad_old.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPTAD',0,ZHOOK_HANDLE)
IF(LOLDPP) THEN
  CALL PPTAD_OLD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,&
   & PRES,POROG,LDBELO,LDBELS,LDBLOW,LDBLES,LDEXTR,&
   & PRXP,PRXPD,PTSTAR,PT0,PTF,PTPP,&
   & PRXP5,PRXPD5,PTSTAR5,PT05,PTF5)  
  IF (LHOOK) CALL DR_HOOK('PPTAD',1,ZHOOK_HANDLE)
  RETURN
ENDIF

CALL PPSTA('PPREF',KPROMA,KSTART,KPROF,KFLEV+1,1,&
 & PRXP5(1,0,2),PRXP5(1,0,4),ZTF5,ZSTZF)  

CALL PPSTA('PPREF',KPROMA,KSTART,KPROF,KLEVP,KLOLEV,PRES,PLNPRES,ZTPP5,ZSTZP)

DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF
    ZTF5(JL,JLEV)=PTF5(JL,JLEV)-ZTF5(JL,JLEV)*RD/PR05(JL,JLEV)
    ZR05(JL,JLEV) = PR05(JL,JLEV)
  ENDDO
ENDDO

DO JL = KSTART,KPROF
  ZTF5(JL,0) = ZTF5(JL,1)
  ZR05(JL,0) = ZR05(JL,1)
ENDDO

ISLCT=2
CALL PPINTP(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,ISLCT,&
 & LDBELO,LDBLOW,PRES,PRXP5,PRXPD5,ZR05,ZRP5)  

DO JL=KSTART,KPROF
  ZALPHA(JL)=0.0_JPRB
  DO JLEV=1,KLEVP
    ZRP(JL,JLEV)=0.0_JPRB
  ENDDO
ENDDO

DO JLEV=0,KFLEV
  DO JL=KSTART,KPROF
    ZTF(JL,JLEV)=0.0_JPRB
    ZR0(JL,JLEV)=0.0_JPRB
    ZSTZF(JL,JLEV)=0.0_JPRB
  ENDDO
ENDDO

!*       1.4   EXTRAPOLATE TEMPERATURE BELOW SURFACE.

IF (LDEXTR) THEN
  ZTXX    = 298.0_JPRB
  ZFI2000 = 2000.0_JPRB * RG
  ZFI2500 = 2500.0_JPRB * RG
  ZUSDFI  = 1.0_JPRB / (ZFI2500 - ZFI2000)
  ZALPC   = -RDTDZ1 * RD / RG

  DO JL = KSTART, KPROF
    ZXXX = RD * MIN(1.0_JPRB, MAX(0.0_JPRB, (POROG(JL)-ZFI2000) * ZUSDFI))
    ZALPHA5(JL) = MAX(0.0_JPRB, ZALPC + ZXXX * MIN(0.0_JPRB, ZTXX - PT05(JL))&
     & / MAX(POROG(JL), ZFI2000) )  
  ENDDO

  DO JLEVP = KLOLEV, KLEVP
    IF (LDBLES(JLEVP)) THEN
      DO JL = KSTART, KPROF
        IF (LDBELS(JL,JLEVP)) THEN
          ZALP5 = ZALPHA5(JL) * (PLNPRES(JL,JLEVP)-PRXP5(JL,KFLEV,3))
          PTSTAR(JL)=PTSTAR(JL) + PTPP(JL,JLEVP) *&
           & (1.0_JPRB + (1.0_JPRB + (0.5_JPRB + ZALP5/6._JPRB) * ZALP5) * ZALP5)  
          ZALP=PTPP(JL,JLEVP) * PTSTAR5(JL) *&
           & (1.0_JPRB + ( 1.0_JPRB + 0.5_JPRB * ZALP5) * ZALP5)  
          ZALPHA(JL)=ZALPHA(JL) + ZALP *(PLNPRES(JL,JLEVP)-PRXP5(JL,KFLEV,3))
          PRXP(JL,KFLEV,3)=PRXP(JL,KFLEV,3) - ZALP * ZALPHA5(JL)
          ZALP=0.0_JPRB
          PTPP(JL,JLEVP) = 0.0_JPRB
        ENDIF
      ENDDO
    ENDIF
  ENDDO

  DO JL = KSTART, KPROF
    ZXXX = RD * MIN(1.0_JPRB, MAX(0.0_JPRB, (POROG(JL)-ZFI2000) * ZUSDFI))
    IF(ZTXX - PT05(JL) <= 0.0_JPRB)THEN
      IF( ZALPC + ZXXX*MIN(0.0_JPRB, ZTXX - PT05(JL)) /&
         & MAX(POROG(JL),ZFI2000)  >=  0.0_JPRB) THEN  
        PT0(JL)=PT0(JL)-ZXXX*ZALPHA(JL)/MAX(POROG(JL),ZFI2000)
        ZALPHA(JL)=0.0_JPRB
      ELSE
        ZALPHA(JL)=0.0_JPRB
      ENDIF
    ELSE
      ZALPHA(JL)=0.0_JPRB
    ENDIF
  ENDDO

ELSE

  DO JLEVP = KLOLEV, KLEVP
    IF (LDBLES(JLEVP)) THEN
      DO JL = KSTART, KPROF
        IF (LDBELS(JL,JLEVP)) THEN
          PTF(JL,KFLEV)=PTF(JL,KFLEV)+PTPP(JL,JLEVP)
          PTPP(JL,JLEVP) = 0.0_JPRB
        ENDIF
      ENDDO
    ENDIF
  ENDDO

ENDIF

!*    1.2   INTERPOLATE TEMPERATURE QUADRATICALY IN LOG(P) TO KEEP GEOPOTENTIAL
!*          IN UPPERAIR AND BETWEEN LOWEST LEVEL AND SURFACE.

DO JLEVP = KLOLEV, KLEVP
  IF (.NOT. LDBLOW(JLEVP)) THEN
    DO JL = KSTART,KPROF
      ZGAM=0.0_JPRB
      ZXX=0.0_JPRB
      IBL = KLEVB(JL,JLEVP,4)
      ZXX5 = (PRXP5(JL,IBL,4)-PLNPRES(JL,JLEVP))&
       & / (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
      ZGAM5 = (PRXP5(JL,IBL-1,4)+PRXP5(JL,IBL,4)-2.0_JPRB*&
       & PRXP5(JL,IBL-1,3))/(PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  

      ZRP(JL,JLEVP)=ZRP(JL,JLEVP)&
       & - ZTPP5(JL,JLEVP) * (RD/(ZRP5(JL,JLEVP)**2))*PTPP(JL,JLEVP)  
      ZTF(JL,IBL)=ZTF(JL,IBL) + PTPP(JL,JLEVP)
      ZTF(JL,IBL)=ZTF(JL,IBL) - PTPP(JL,JLEVP) *&
       & (ZXX5 + 3._JPRB * ZGAM5 * ZXX5 * (ZXX5 - 1.0_JPRB))  
      ZTF(JL,IBL-1)=ZTF(JL,IBL-1) + PTPP(JL,JLEVP) *&
       & (ZXX5 + 3._JPRB * ZGAM5 * ZXX5 * (ZXX5 - 1.0_JPRB))  
      ZXX = PTPP(JL,JLEVP) * (ZTF5(JL,IBL-1)-ZTF5(JL,IBL))&
       & * (1.0_JPRB + 3._JPRB * ZGAM5 * (2.0_JPRB*ZXX5-1.0_JPRB) )  
      ZGAM = PTPP(JL,JLEVP) *(ZTF5(JL,IBL-1)-ZTF5(JL,IBL))&
       & * 3._JPRB * ZXX5 * (ZXX5 - 1.0_JPRB)  
      PTPP(JL,JLEVP) = PTPP(JL,JLEVP) * RD/ZRP5(JL,JLEVP)

      PRXP(JL,IBL-1,4)=PRXP(JL,IBL-1,4)+ZGAM*&
       & (1.0_JPRB+ZGAM5)/(PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
      PRXP(JL,IBL,4)=PRXP(JL,IBL,4)+ZGAM*&
       & (1.0_JPRB-ZGAM5)/(PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
      PRXP(JL,IBL-1,3)=PRXP(JL,IBL-1,3)-2.0_JPRB*ZGAM/&
       & (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
      ZGAM=0.0_JPRB

      PRXP(JL,IBL-1,4)=PRXP(JL,IBL-1,4)+ZXX5*ZXX &
       & / (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
      PRXP(JL,IBL,4)=PRXP(JL,IBL,4)+ZXX*&
       & (1.0_JPRB-ZXX5)/(PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
      ZXX=0.0_JPRB
    ENDDO
  ELSE
    DO JL = KSTART,KPROF
      IF (.NOT.LDBELO(JL,JLEVP)) THEN
        IBL = KLEVB(JL,JLEVP,4)
        ZGAM=0.0_JPRB
        ZXX=0.0_JPRB
        ZXX5 = (PRXP5(JL,IBL,4)-PLNPRES(JL,JLEVP))&
         & / (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
        ZGAM5 = (PRXP5(JL,IBL-1,4)+PRXP5(JL,IBL,4)-2.0_JPRB*&
         & PRXP5(JL,IBL-1,3))/(PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  

        ZRP(JL,JLEVP)=ZRP(JL,JLEVP)&
         & - ZTPP5(JL,JLEVP) * (RD/(ZRP5(JL,JLEVP)**2))*PTPP(JL,&
         & JLEVP)  
        ZTF(JL,IBL)=ZTF(JL,IBL) + PTPP(JL,JLEVP)
        ZTF(JL,IBL)=ZTF(JL,IBL) - PTPP(JL,JLEVP) *&
         & (ZXX5 + 3._JPRB * ZGAM5 * ZXX5 * (ZXX5 - 1.0_JPRB))  
        ZTF(JL,IBL-1)=ZTF(JL,IBL-1) + PTPP(JL,JLEVP) *&
         & (ZXX5 + 3._JPRB * ZGAM5 * ZXX5 * (ZXX5 - 1.0_JPRB))  
        ZXX = PTPP(JL,JLEVP) * (ZTF5(JL,IBL-1)-ZTF5(JL,IBL))&
         & * (1.0_JPRB + 3._JPRB * ZGAM5 * (2.0_JPRB*ZXX5-1.0_JPRB) )  
        ZGAM = PTPP(JL,JLEVP) *(ZTF5(JL,IBL-1)-ZTF5(JL,IBL))&
         & * 3._JPRB * ZXX5 * (ZXX5 - 1.0_JPRB)  
        PTPP(JL,JLEVP) = PTPP(JL,JLEVP) * RD/ZRP5(JL,JLEVP)

        PRXP(JL,IBL-1,4)=PRXP(JL,IBL-1,4)+ZGAM*&
         & (1.0_JPRB+ZGAM5)/(PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
        PRXP(JL,IBL,4)=PRXP(JL,IBL,4)+ZGAM*&
         & (1.0_JPRB-ZGAM5)/(PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
        PRXP(JL,IBL-1,3)=PRXP(JL,IBL-1,3)-2.0_JPRB*ZGAM/&
         & (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
        ZGAM=0.0_JPRB

        PRXP(JL,IBL-1,4)=PRXP(JL,IBL-1,4)+ZXX5*ZXX &
         & / (PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
        PRXP(JL,IBL,4)=PRXP(JL,IBL,4)+ZXX*&
         & (1.0_JPRB-ZXX5)/(PRXP5(JL,IBL,4)-PRXP5(JL,IBL-1,4))  
        ZXX=0.0_JPRB
      ELSEIF (LDBELO(JL,JLEVP) .AND. .NOT.LDBELS(JL,JLEVP)) THEN
        ZXX5 = (PRXP5(JL,KFLEV,3) - PLNPRES(JL,JLEVP))&
         & / (PRXP5(JL,KFLEV,3) - PRXP5(JL,KFLEV,4))  
        PTSTAR(JL) = PTSTAR(JL) + PTPP(JL,JLEVP) * (1.0_JPRB-ZXX5)
        PTF(JL,KFLEV) = PTF(JL,KFLEV) + PTPP(JL,JLEVP) * ZXX5
        ZXX = PTPP(JL,JLEVP)*(PTF5(JL,KFLEV)-PTSTAR5(JL))
        PTPP(JL,JLEVP) = 0.0_JPRB

        PRXP(JL,KFLEV,3)=PRXP(JL,KFLEV,3)+ZXX*(1-ZXX5)&
         & / (PRXP5(JL,KFLEV,3) - PRXP5(JL,KFLEV,4))  
        PRXP(JL,KFLEV,4)=PRXP(JL,KFLEV,4)+ZXX*ZXX5 &
         & / (PRXP5(JL,KFLEV,3) - PRXP5(JL,KFLEV,4))  
        ZXX=0.0_JPRB
      ENDIF
    ENDDO
  ENDIF
ENDDO

!     ------------------------------------------------------------------
!*    1.    POST-PROCESS TEMPERATURE.
!           -------------------------

!*    1.1   PREPARE FOR EXTRAPOLATION ABOVE TOP

CALL PPINTPAD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,ISLCT,&
 & LDBELO,LDBLOW,PRES,PRXP,PRXPD,ZR0,ZRP,&
 & PRXP5,PRXPD5,ZR05)  

DO JL = KSTART,KPROF
  ZTF(JL,1) = ZTF(JL,1) +  ZTF(JL,0)
  ZR0(JL,1) = ZR0(JL,1) +  ZR0(JL,0)
ENDDO

DO JLEV=1,KFLEV
  DO JL=KSTART,KPROF
    ZS=ZTF(JL,JLEV)
!        RE-COMPUTE ORIGINAL ZTF5 TRAJECTORY VALUE
    ZS5=ZTF5(JL,JLEV)
    ZTF5(JL,JLEV)=(PTF5(JL,JLEV)-ZS5)*PR05(JL,JLEV)/RD
!        ADJOINT
    PR0(JL,JLEV)=PR0(JL,JLEV)+ZTF5(JL,JLEV)*RD/(PR05(JL,JLEV)**2)*ZS
    PTF(JL,JLEV)=PTF(JL,JLEV)+ZS
    ZTF(JL,JLEV)=-RD/PR05(JL,JLEV)*ZS
    PR0(JL,JLEV)=PR0(JL,JLEV) + ZR0(JL,JLEV)
  ENDDO
ENDDO

DO JLEVP = KLOLEV, KLEVP
  DO JL = KSTART,KPROF
    PTPP(JL,JLEVP) = 0.0_JPRB
  ENDDO
ENDDO

CALL PPSTAAD('PPREF',KPROMA,KSTART,KPROF,KFLEV+1,1,&
 & PRXP(1,0,2),PRXP(1,0,4),ZTF,ZSTZF,&
 & PRXP5(1,0,2))  

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPTAD',1,ZHOOK_HANDLE)
END SUBROUTINE PPTAD

