SUBROUTINE PPUV(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,&
 & LDBELO,LDBLOW,PRPRES,PRXP,PRXPD,&
 & PUF,PVF,PUPP,PVPP)  

!**** *PPUV* - POST-PROCESS WINDS

!     PURPOSE.
!     --------
!           PERFORMS THE VERTICAL INTERPOLATION OF U AND V TO GIVEN
!       PRESSURE LEVELS

!**   INTERFACE.
!     ----------
!        *CALL* *PPUV(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,
!                     LDBELO,LDBLOW,PRPRES,PRXP,PRXPD,
!                     PUF,PVF,PUPP,PVPP)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION    (INPUT-C)
!        KLEVB(KPROMA,KLEVP,NPPM)  - INPUT LEVEL BELOW PRPRES          (INPUT-C)
!                                    (SEE PPFLEV)
!        PRPRES(KPROMA,KLEVP)      - LOG(POST-PROCESSING LEVEL PRES.)  (INPUT-C)
!        LDBELO(KPROMA,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     LOWEST  MODEL LEVEL              (INPUT-C)
!        LDBLOW(KLEV)              - .TRUE IF LDBELO(J) IS CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT-C)
!        PRXP(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF,FULL LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT)
!        PRXPD(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))           (INPUT)
!        PUF(KPROMA,0:KFLEV)       - ZONAL WIND ON FULL INPUT LEVELS   (INPUT)
!        PVF(KPROMA,0:KFLEV)       - MERID. WIND ON FULL INPUT LEVELS  (INPUT)

!        PUPP(KPROMA,KLEVP)        - POST-PROCESSED ZONAL WIND         (OUTPUT)
!        PVPP(KPROMA,KLEVP)        - POST-PROCESSED MERIDIONAL WIND    (OUTPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  PPINTP - LINEAR INTERPOLATION
!     ----------  PPITPQ - QUADRATIC INTERPOLATION FOR TOP LEVELS

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 89-01-26
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
USE YOMCT0   , ONLY : LOLDPP

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVB(KPROMA,KLEVP,NPPM) 
LOGICAL           ,INTENT(IN)    :: LDBELO(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLOW(KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPRES(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXPD(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUPP(KPROMA,KLEVP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVPP(KPROMA,KLEVP) 
INTEGER(KIND=JPIM) :: ISLCT, JL, JLEVP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ppintp.intfb.h"
#include "ppuv_old.intfb.h"

!        0. TEMPORARY CALL OF "OLD" PPUV

IF (LHOOK) CALL DR_HOOK('PPUV',0,ZHOOK_HANDLE)
IF(LOLDPP) THEN
  CALL PPUV_OLD(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,&
   & LDBELO,LDBLOW,PRPRES,PRXP,PRXPD,&
   & PUF,PVF,PUPP,PVPP)  
  IF (LHOOK) CALL DR_HOOK('PPUV',1,ZHOOK_HANDLE)
  RETURN
ENDIF
!     ------------------------------------------------------------------

!*       1.    POST-PROCESS WINDS.
!              -------------------

ISLCT=4

!*       1.1   PREPARE FOR EXTRAPOLATION ABOVE TOP MODEL LEVEL.

DO JL=KSTART,KPROF
  PUF(JL,0)=PUF(JL,2)+PRXP(JL,2,ISLCT)*PRXPD(JL,1,ISLCT)*&
   & (PUF(JL,1)-PUF(JL,2))  
  PVF(JL,0)=PVF(JL,2)+PRXP(JL,2,ISLCT)*PRXPD(JL,1,ISLCT)*&
   & (PVF(JL,1)-PVF(JL,2))  
ENDDO

!*       1.2   INTERPOLATE

CALL PPINTP(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,&
 & KLEVB,ISLCT,&
 & LDBELO,LDBLOW,PRPRES,PRXP,PRXPD,&
 & PUF,PUPP)  
CALL PPINTP(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,&
 & KLEVB,ISLCT,&
 & LDBELO,LDBLOW,PRPRES,PRXP,PRXPD,&
 & PVF,PVPP)  

!*       1.3   CONSTANT EXTRAPOLATION BELOW LOWEST MODEL LEVEL.

DO JLEVP=KLOLEV,KLEVP
  IF (LDBLOW(JLEVP)) THEN
    DO JL=KSTART,KPROF
      IF (LDBELO(JL,JLEVP)) THEN
        PUPP(JL,JLEVP)=PUF(JL,KFLEV)
        PVPP(JL,JLEVP)=PVF(JL,KFLEV)
      ENDIF
    ENDDO
  ENDIF
ENDDO

!*       1.4   MODIFY INTERPOLATION/EXTRAPOLATION ABOVE SECOND
!*          FULL MODEL LEVEL. FIT QUADRATIC POLYNOMIAL IN LN(P).

!     CALL PPITPQ(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,ISLCT,
!    1            PRPRES,PRXP,PUF,
!    2            PUPP)
!     CALL PPITPQ(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,ISLCT,
!    1            PRPRES,PRXP,PVF,
!    2            PVPP)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPUV',1,ZHOOK_HANDLE)
END SUBROUTINE PPUV
