SUBROUTINE PPUV10MAD(&
 & KPROMA,KIDIA,KFDIA,LDNEUTRAL,PINTH,&
 & PUB5,PVB5,PAPHI5,&
 & PUB,PVB,PAPHI,&
 & PRI5,PBN5,PBM5,&
 & PBN,PBM,&
 & PU05,PV05,&
 & PU10M,PV10M)  

!**** *PPUV10MAD* - ADJOINT OF TANGENT LINEAR OF POST-PROCESSED
!                   10 METRES WIND

!     PURPOSE
!     -------
!          COMPUTE THE ADJOINT OF TANGENT LINEAR OF THE WIND
!          AT 10 METRES

!**   INTERFACE
!     ---------
!       *CALL* *PPUV10MAD(
!     C                 KPROMA,KIDIA,KFDIA,LDNEUTRAL,PINTH,
!     I                 PUB5,PVB5,PAPHI5,
!     O                 PUB,PVB,PAPHI,
!     I                 PRI5,PBN5,PBM5,
!     O                 PBN,ZBM,
!     I                 PU10M,PV10M)

!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KIDIA                     - INDEX OF START POINT              (INPUT-C)
!        KFDIA                     - INDEX OF END   POINT              (INPUT-C)
!        LDNEUTRAL                 - CONTROL TYPE OF OUTPUT WINDS      (INPUT-C)
!                                    FALSE: REAL WIND,                 (INPUT-C)
!        PINTH                     - INTERPOLATION HEIGHT              (INPUT)
!        PUB5                      - MODEL VALUE OF THE U-WIND
!                                    COMPONENT AT THE OBSERVATION
!                                    POINT                             (INPUT)
!        PVB5                      - MODEL VALUE OF THE V-WIND
!                                    COMPONENT AT THE OBSERVATION
!                                    POINT                             (INPUT)
!        PAPHI5                    - GEOPOTENTIAL LOWER MODE
!                                    LEVEL                             (INPUT)
!        PUB(KPROMA)               - MODEL VALUE OF THE U-WIND
!                                    COMPONENT AT THE OBSERVATION
!                                    POINT                             (INPUT)
!        PVB(KPROMA)               - MODEL VALUE OF THE V-WIND
!                                    COMPONENT AT THE OBSERVATION
!                                    POINT                             (INPUT)
!        PAPHI(KPROMA)             - GEOPOTENTIAL LOWER MODEL
!                                    LEVEL                             (INPUT)
!        PAPHI5                    - GEOPOTENTIAL LOWER MODE
!        PRI5(KPROMA)              - RICHARSON NUMBER                  (INPUT)
!        PBN5(KPROMA)              - NEUTRAL COEFFICIENT               (INPUT)
!        PBM5(KPROMA)              - MOMENTUM COEFFICIENT              (INPUT)
!        PU05(KPROMA)              - U-COMP OCEAN CURRENT              (INPUT)
!        PV05(KPROMA)              - V-COMP OCEAN CURRENT              (INPUT)
!        PU10M(KPROMA)             - POST-PROCESSED U-COMPONENT        (OUTPUT)
!        PV10M(KPROMA)             - POST-PROCESSED V-COMPONENT        (OUTPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        INTERPOLATION METHOD OF J.F. GELEYN, TELLUS (1988), 40A, 347-351

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        CARLA CARDINALI *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL     : 92-05-22
!        M.Hamrud     : 95-09-25 Some cleaning, vectorization problems
!        L. Isaksen   : 96-02-20 NFLEV --> NFLEVG
!        D. VASILJEVIC: 99-02-08 INTRODUCTION OF INTERPOLATION HEIGHT
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        H. Hersbach  : 08-04-16 Option for neutral winds
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOEPHY   , ONLY : LECURR

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
LOGICAL,           INTENT(IN)    :: LDNEUTRAL
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINTH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUB5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVB5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PAPHI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBN5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBM5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU05(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV05(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PBM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PU10M(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PV10M(KPROMA) 
INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZCBN, ZCBN5, ZCBS, ZCBS5, ZCBU, ZCBU5, ZRAT, ZRAT5, ZRED, ZRED5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     1.     DEFINITION OF THE CONSTANTS
!            ---------------------------

!     ZHUV=10.*RG

!*         2.      ADJOINT COMPUTATION
!                  -------------------
IF (LHOOK) CALL DR_HOOK('PPUV10MAD',0,ZHOOK_HANDLE)
DO JL=KIDIA,KFDIA
  IF (PINTH(JL) < PAPHI5(JL) .OR. LDNEUTRAL) THEN
    ZRAT5=PINTH(JL)/PAPHI5(JL)
    ZCBN5=LOG(1.0_JPRB+(EXP(PBN5(JL))-1.0_JPRB)*ZRAT5)
    ZCBS5=-(PBN5(JL)-PBM5(JL))*ZRAT5
    ZCBU5=-LOG(1.0_JPRB+(EXP(PBN5(JL)-PBM5(JL))-1.0_JPRB)*ZRAT5)
    IF(LDNEUTRAL) THEN
      ZRED5=ZCBN5/PBM5(JL)
    ELSEIF(PRI5(JL) >= 0.0_JPRB) THEN
      ZRED5=(ZCBN5+ZCBS5)/PBM5(JL)
    ELSE
      ZRED5=(ZCBN5+ZCBU5)/PBM5(JL)
    ENDIF

    ZRED=0.0_JPRB
    ZCBN=0.0_JPRB
    ZCBS=0.0_JPRB
    ZCBU=0.0_JPRB
    ZRAT=0.0_JPRB
    PVB(JL)=PVB(JL)+ZRED5*PV10M(JL)
    ZRED=ZRED+PVB5(JL)*PV10M(JL)
    IF(LECURR)ZRED=ZRED-PV05(JL)*PV10M(JL)
    PUB(JL)=PUB(JL)+ZRED5*PU10M(JL)
    ZRED=ZRED+PUB5(JL)*PU10M(JL)
    IF(LECURR)ZRED=ZRED-PU05(JL)*PU10M(JL)
    IF (LDNEUTRAL) THEN
      ZCBN=ZCBN+1.0_JPRB/PBM5(JL)*ZRED
      PBM(JL)=PBM(JL)-ZCBN5/PBM5(JL)**2*ZRED
    ELSEIF(PRI5(JL) >= 0.0_JPRB) THEN
      ZCBN=ZCBN+1.0_JPRB/PBM5(JL)*ZRED
      ZCBS=ZCBS+1.0_JPRB/PBM5(JL)*ZRED
      PBM(JL)=PBM(JL)-(ZCBN5+ZCBS5)/PBM5(JL)**2*ZRED
    ELSE
      ZCBN=ZCBN+1.0_JPRB/PBM5(JL)*ZRED
      ZCBU=ZCBU+1.0_JPRB/PBM5(JL)*ZRED
      PBM(JL)=PBM(JL)-(ZCBN5+ZCBU5)/PBM5(JL)**2*ZRED
    ENDIF
    PBN(JL)=PBN(JL)-1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL)-PBM5(JL))-1.0_JPRB)*ZRAT5)*&
     & EXP(PBN5(JL)-PBM5(JL))*ZRAT5*ZCBU  
    PBM(JL)=PBM(JL)+1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL)-PBM5(JL))-1.0_JPRB)*ZRAT5)*&
     & EXP(PBN5(JL)-PBM5(JL))*ZRAT5*ZCBU  
    ZRAT=ZRAT-1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL)-PBM5(JL))-1.0_JPRB)*ZRAT5)*&
     & (EXP(PBN5(JL)-PBM5(JL))-1.0_JPRB)*ZCBU  
    PBN(JL)=PBN(JL)-ZRAT5*ZCBS
    PBM(JL)=PBM(JL)+ZRAT5*ZCBS
    ZRAT=ZRAT-(PBN5(JL)-PBM5(JL))*ZCBS
    PBN(JL)=PBN(JL)+1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL))-1.0_JPRB)*ZRAT5)*&
     & EXP(PBN5(JL))*ZRAT5*ZCBN  
    ZRAT=ZRAT+1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL))-1.0_JPRB)*ZRAT5)*&
     & (EXP(PBN5(JL))-1.0_JPRB)*ZCBN  
    PAPHI(JL)=PAPHI(JL)-PINTH(JL)/PAPHI5(JL)**2*ZRAT
  ELSE
    PUB(JL)=PUB(JL)+PU10M(JL)
    PVB(JL)=PVB(JL)+PV10M(JL)
  ENDIF
  PU10M(JL)=0.0_JPRB
  PV10M(JL)=0.0_JPRB
ENDDO

!     -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPUV10MAD',1,ZHOOK_HANDLE)
END SUBROUTINE PPUV10MAD

