SUBROUTINE PPUV10MTL(&
 & KPROMA,KIDIA,KFDIA,LDNEUTRAL,PINTH,&
 & PUB5,PVB5,PAPHI5,&
 & PUB,PVB,PAPHI,&
 & PRI5,PBN5,PBM5,&
 & PBN,PBM,&
 & PU05,PV05,&
 & PU10M,PV10M)

!**** *PPUV10MTL* - TANGENT LINEAR OF POST-PROCESSED 10 METRES WIND

!     PURPOSE
!     -------
!          COMPUTE THE TANGENT LINEAR OF THE WIND
!          AT 10 METRES

!**   INTERFACE
!     ---------
!       *CALL* *PPUV10MTL(
!     C                 KPROMA,KIDIA,KFDIA,LDNEUTRAL,PINTH,
!     R                 PUB5,PVB5,PAPHI5,
!     I                 PUB,PVB,PAPHI,
!     R                 PRI5,PBN5,ZBM5,
!     I                 PBN,PBM,
!     O                 PU10M,PV10M)

!        EXPLICIT ARGUMENTS
!        ------------------

!        KPROMA                    - HORIZONTAL DIMENSION.           (INPUT-C)
!        KIDIA                     - INDEX OF START POINT            (INPUT-C)
!        KFDIA                     - INDEX OF END   POINT            (INPUT-C)
!        LDNEUTRAL                 - CONTROL TYPE OF OUTPUT WINDS    (INPUT-C)
!                                    FALSE: REAL WIND,               (INPUT-C)
!                                    TRUE : EQUIVALENT NEUTRAL WIND
!        PINTH(KPROMA)             - INTERPOLATION HEIGHT            (INPUT)
!        PUB5(KPROMA)              - MODEL VALUE OF THE U-WIND
!                                    COMPONENT AT THE OBSERVATION
!                                    POINT                           (INPUT)
!        PVB5(KPROMA)              - MODEL VALUE OF THE V-WIND
!                                    COMPONENT AT THE OBSERVATION
!                                    POINT                           (INPUT)
!        PAPHI5(KPROMA)            - GEOPOTENTIAL LOWER MODEL
!                                    LEVEL                           (INPUT)
!        PUB(KPROMA)               - MODEL VALUE OF THE U-WIND
!                                    COMPONENT AT THE OBSERVATION
!                                    POINT                           (INPUT)
!        PVB(KPROMA)               - MODEL VALUE OF THE V-WIND
!                                    COMPONENT AT THE OBSERVATION
!                                    POINT                           (INPUT)
!        PAPHI(KPROMA)             - GEOPOTENTIAL LOWER MODEL
!                                    LEVEL                           (INPUT)
!        PRI(KPROMA)               - RICHARSON NUMBER                (INPUT)
!        PBN(KPROMA)               - NEUTRAL COEFFICIENT             (INPUT)
!        PBM(KPROMA)               - MOMENTUM COEFFICIENT            (INPUT)
!        PU05(KPROMA)              - U-COMP OCEAN CURRENT            (INPUT)
!        PV05(KPROMA)              - V-COMP OCEAN CURRENT            (INPUT)
!        PU10M(KPROMA)             - POST-PROCESSED U-COMPONENT      (OUTPUT)
!        PV10M(KPROMA)             - POST-PROCESSED V-COMPONENT      (OUTPUT)

!        IMPLICIT ARGUMENTS :  NONE.
!        --------------------

!     METHOD.
!     -------
!        INTERPOLATION METHOD OF J.F. GELEYN, TELLUS (1988), 40A, 347-351

!     EXTERNALS.
!     ----------
!        NONE

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        CARLA CARDINALI *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL    : 92-05-22
!        L. Isaksen  : 96-02-20  NFLEV --> NFLEVG
!        D. VASILEVIC: 99-02-08  INTRODUCTION OF INTERPOLATION HEIGHT
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        H.Hersbach  : 08-16-04  Option for neutral winds, include ocean current
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOEPHY   , ONLY : LECURR

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
LOGICAL,           INTENT(IN)    :: LDNEUTRAL
REAL(KIND=JPRB)   ,INTENT(IN)    :: PINTH(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUB5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVB5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVB(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PAPHI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRI5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBN5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBM5(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBN(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU05(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV05(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PU10M(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PV10M(KPROMA) 
INTEGER(KIND=JPIM) :: JL

REAL(KIND=JPRB) :: ZCBN, ZCBN5, ZCBS, ZCBS5, ZCBU, ZCBU5, ZRAT, ZRAT5, ZRED, ZRED5
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     1.     DEFINITION OF THE CONSTANTS
!            ---------------------------

!     ZHUV=10.*RG

!*         2.      WIND INTERPOLATION.
!                  -------------------
IF (LHOOK) CALL DR_HOOK('PPUV10MTL',0,ZHOOK_HANDLE)
DO JL=KIDIA,KFDIA
  IF (PINTH(JL) < PAPHI5(JL) .OR. LDNEUTRAL) THEN
    ZRAT5=PINTH(JL)/PAPHI5(JL)
    ZCBN5=LOG(1.0_JPRB+(EXP(PBN5(JL))-1.0_JPRB)*ZRAT5)
    ZCBS5=-(PBN5(JL)-PBM5(JL))*ZRAT5
    ZCBU5=-LOG(1.0_JPRB+(EXP(PBN5(JL)-PBM5(JL))-1.0_JPRB)*ZRAT5)
    IF(LDNEUTRAL) THEN
      ZRED5=ZCBN5/PBM5(JL)
    ELSEIF(PRI5(JL) >= 0.0_JPRB) THEN
      ZRED5=(ZCBN5+ZCBS5)/PBM5(JL)
    ELSE
      ZRED5=(ZCBN5+ZCBU5)/PBM5(JL)
    ENDIF

    ZRAT=-PINTH(JL)/PAPHI5(JL)**2*PAPHI(JL)
    ZCBN=1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL))-1.0_JPRB)*ZRAT5)*&
     & ( EXP(PBN5(JL))*ZRAT5*PBN(JL)&
     & +(EXP(PBN5(JL))-1.0_JPRB)*ZRAT )  
    ZCBS=-(PBN(JL)-PBM(JL))*ZRAT5-(PBN5(JL)-PBM5(JL))*ZRAT
    ZCBU=-1.0_JPRB/(1.0_JPRB+(EXP(PBN5(JL)-PBM5(JL))-1.0_JPRB)*ZRAT5)*&
     & (  EXP(PBN5(JL)-PBM5(JL))*ZRAT5*(PBN(JL)-PBM(JL))&
     & +(EXP(PBN5(JL)-PBM5(JL))-1.0_JPRB)*ZRAT )  
    IF(LDNEUTRAL) THEN
      ZRED= ZCBN/PBM5(JL)-ZCBN5/PBM5(JL)**2*PBM(JL)
    ELSEIF(PRI5(JL) >= 0.0_JPRB) THEN
      ZRED=(ZCBN+ZCBS)/PBM5(JL)-(ZCBN5+ZCBS5)/PBM5(JL)**2*PBM(JL)
    ELSE
      ZRED=(ZCBN+ZCBU)/PBM5(JL)-(ZCBN5+ZCBU5)/PBM5(JL)**2*PBM(JL)
    ENDIF
    IF(LECURR) THEN
      PU10M(JL)=ZRED5*PUB(JL)+(PUB5(JL)-PU05(JL))*ZRED
      PV10M(JL)=ZRED5*PVB(JL)+(PVB5(JL)-PV05(JL))*ZRED
    ELSE
      PU10M(JL)=ZRED5*PUB(JL)+PUB5(JL)*ZRED
      PV10M(JL)=ZRED5*PVB(JL)+PVB5(JL)*ZRED
    ENDIF
  ELSE
    PU10M(JL)=PUB(JL)
    PV10M(JL)=PVB(JL)
  ENDIF
ENDDO

!     -----------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPUV10MTL',1,ZHOOK_HANDLE)
END SUBROUTINE PPUV10MTL
