SUBROUTINE PPVVEL(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,KLEVB,&
 & PRPRES,LDBELO,LDBLOW,&
 & PRXP,PRXPD,PDELP,PRDELP,PLNPR,PALPH,PRTGR,&
 & PUF,PVF,PDIVF,PSPL,PSPM,PEVT0,PVVPP,LDRUBC,PRPRESF,LDETADOT)  

!**** *PPVVEL* - POST-PROCESS VERTICAL VELOCITY OMEGA.

!     PURPOSE.
!     --------
!           COMPUTES VERTICAL VELOCITY OMEGA AT FULL LEVELS AND
!       PERFORMS THE VERTICAL INTERPOLATION TO A GIVEN
!       MODEL CO-ORDINATE LEVEL. SETS OMEGA TO 0 IF PRESSURE IS BIGGER
!       THAN THE PRESSURE OF THE LOWER FULL LEVEL .

!**   INTERFACE.
!     ----------
!       *CALL*  *PPVVEL(...)

!        EXPLICIT ARGUMENTS
!        --------------------

!        KPROMA                    - HORIZONTAL DIMENSION.             (INPUT-C)
!        KSTART                    - START OF WORK.                    (INPUT-C)
!        KPROF                     - DEPTH OF WORK.                    (INPUT-C)
!        KFLEV                     - NUMBER OF INPUT PRESSURE LEVELS   (INPUT-C)
!        KLEVP                     - NUMBER OF OUTPUT PRESSURE LEVELS  (INPUT-C)
!        KLOLEV                    - BEGINING FOR THE INTERPOLATION    (INPUT-C)
!        KLEVB(KPROMA,KLEVP,NPPM)  - INPUT LEVEL BELOW PRPRES          (INPUT-C)
!                                    (SEE PPFLEV)
!        PRPRES(KPROMA,KLEVP)      -  POST-PROCESSING LEVEL PRESSURES  (INPUT-C)
!        LDBELO(KPROMA,KLEVP)      - .TRUE. IF PRESSURE IS UNDER
!                                     LOWEST MODEL LEVEL               (INPUT-C)
!        LDBLOW(KLEVP)             - .TRUE. IF LOBELO(J) IS CONTAINING
!                                    AT LEAST ONE .TRUE.               (INPUT-C)
!        PRXP(KPROMA,0:KFLEV,NPPM) - HALF,FULL AND LN HALF,FULL LEVEL
!                                    PRESSURES (SEE PPINIT)            (INPUT)
!        PRXPD(KPROMA,0:KFLEV,NPPM)- 1./D(P) AND 1./D(LN(P))           (INPUT)
!                                    AT LEAST ONE .TRUE.               (INPUT)
!        PDELP (KPROMA,KFLEV)      - HALF LEVEL PRESSURE DIFFERENCE
!                                    ACROSS LAYERS (PREVIOUSLY
!                                    COMPUTED IN GPXYB)                (INPUT)
!        PRDELP(KPROMA,KFLEV)      - THEIR INVERSE (COMPUTED IN GPXYB) (INPUT)
!        PLNPR (KPROMA,KFLEV)      - LOGARITHM OF RATIO OF HALF LEVEL
!                                    PRESSURES (COMPUTED IN GPXYB)     (INPUT)
!        PALPH (KPROMA,KFLEV)      - COEFFICIENTS OF THE HYDROSTATICS
!                                    (COMPUTED IN GPXYB)               (INPUT)
!        PRTGR (KPROMA,KFLEV)      - FOR PRESSURE GRADIENT TERM AND
!                                    ENERGY CONVERSION (SEE GPXYB)     (INPUT)
!        PUF(KPROMA,0:KFLEV)       - U-WIND ON MODEL FULL LEVELS       (INPUT)
!        PVF(KPROMA,0:KFLEV)       - V-WIND ON MODEL FULL LEVELS       (INPUT)
!        PDIVF(KPROMA,KFLEV)       - DIVERGENCE ON MODEL FULL LEVELS   (INPUT)
!        PSPL(KPROMA)              - LONGITUDINAL DERIVATIVE OF PS     (INPUT)
!        PSPM(KPROMA)              - MERIDIONAL DERIVATIVE OF PS       (INPUT)
!        PEVT0                     - <eta dot dp/deta> on top of atmosphere(input)
!        PVVPP(KPROMA,KLEVP)       - POST-PROCESSED VERTICAL VELOCITY  (OUTPUT)
!        LDETADOT                  - COMPUTE ETADOT INSTEAD OF OMEGA   (INPUT)

!        IMPLICIT ARGUMENTS :
!        --------------------

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.  GPCTY  - COMPUTE OMEGA/P AT FULL LEVELS.
!    ----------   PPINTP - LINEAR INTERPOLATION

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 89-01-26
!        MODIFIED : 92-12-02 M.HAMRUD ( EXTRAP. UNDER SURFACE)
!        R. El Khatib : 97-12-23 PEVT0 ,LDRUBC in GPCTY
!        K. YESSAD    : 99-11-15 Add dummy "zpsdvbc" in "gpcty".
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        N. Wedi      : add etadot as possible output
!        K. Yessad (Dec 2008) : remove useless dummy arguments.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM
USE YOMGEM   , ONLY : VBH      ,VDELB, VETAH
USE YOMCVER  , ONLY : LVERTFE

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLOLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVB(KPROMA,KLEVP,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPRES(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBELO(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDBLOW(KLEVP) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXP(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRXPD(KPROMA,0:KFLEV,NPPM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDELP(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRDELP(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLNPR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PALPH(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRTGR(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVF(KPROMA,0:KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PDIVF(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVT0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVVPP(KPROMA,KLEVP) 
LOGICAL           ,INTENT(IN)    :: LDRUBC 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRPRESF(KPROMA,KFLEV) 
LOGICAL           ,INTENT(IN),OPTIONAL    :: LDETADOT

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZEVEL(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZVVEL(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZPSDIV(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZPSDVBC(KPROMA,0:KFLEV)
REAL(KIND=JPRB) :: ZDIVDP(KPROMA,KFLEV)

INTEGER(KIND=JPIM) :: IPROMVC, ISLCT, JL, JLEV, JLEVP

LOGICAL :: LLVERCOR
LOGICAL :: LL_ETAD

! * useless dummy arguments if "lvercor=.f.":
REAL(KIND=JPRB) :: ZVCRSSAF(1),ZVCRSSAH(1),ZVCASRSF(1),ZVCASRSH(1)
REAL(KIND=JPRB) :: ZVCRSSAFL(1),ZVCRSSAFM(1)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "gpcty.intfb.h"
#include "ppintp.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPVVEL',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    POST-PROCESS VERTICAL VELOCITY OMEGA
!              ------------------------------------

! --- Relaxation of thin layer hypothesis not yet coded ----------------------
LLVERCOR=.FALSE.
IPROMVC=1
! ----------------------------------------------------------------------------

!*       1.1   COMPUTE OMEGA/P ON FULL MODEL LEVELS.

CALL GPCTY (LLVERCOR,&
 & KPROMA,IPROMVC,KSTART,KPROF,KFLEV,VDELB,VBH,PUF(1,1),PVF(1,1),&
 & PDIVF,PEVT0,LDRUBC,PDELP,PRDELP,PLNPR,PALPH,PRTGR,&
 & PSPL,PSPM,&
 & ZVCRSSAF,ZVCRSSAH,ZVCASRSF,ZVCASRSH,ZVCRSSAFL,ZVCRSSAFM,&
 & ZEVEL,ZVVEL(1,1),ZPSDIV,ZPSDVBC,ZDIVDP,PRPRESF)

LL_ETAD = .FALSE.
IF(PRESENT(LDETADOT)) THEN
  LL_ETAD = LDETADOT
ENDIF

!*       1.2   COMPUTE OMEGA OR ETADOT ON FULL LEVELS

IF( LL_ETAD ) THEN
  IF(LVERTFE) THEN
    DO JLEV=1,KFLEV
      DO JL  =KSTART,KPROF
        ZVVEL(JL,JLEV)=ZEVEL(JL,JLEV)&
         & *(VETAH(JLEV)-VETAH(JLEV-1))*PRDELP(JL,JLEV)  
      ENDDO
    ENDDO
  ELSE
    DO JLEV=1,KFLEV
      DO JL  =KSTART,KPROF
        ZVVEL(JL,JLEV)=&
         & 0.5_JPRB*(ZEVEL(JL,JLEV)+ZEVEL(JL,JLEV-1))&
         & *(VETAH(JLEV)-VETAH(JLEV-1))*PRDELP(JL,JLEV)  
      ENDDO
    ENDDO
  ENDIF
ELSE
  DO JLEV=1,KFLEV
    DO JL  =KSTART,KPROF
      ZVVEL(JL,JLEV)=PRXP(JL,JLEV,2)*ZVVEL(JL,JLEV)
    ENDDO
  ENDDO
ENDIF

!*       1.3   PREPARE FOR EXTRAPOLATION ABOVE TOP.

DO JL=KSTART,KPROF
  ZVVEL(JL,0)=0.0_JPRB
ENDDO

!*       1.4   INTERPOLATE.

ISLCT=2
CALL PPINTP(KPROMA,KSTART,KPROF,KFLEV,KLEVP,KLOLEV,&
 & KLEVB,ISLCT,&
 & LDBELO,LDBLOW,PRPRES,PRXP,PRXPD,&
 & ZVVEL,PVVPP)  

!*       1.5   CONSTANT EXTRAPOLATION UNDER LOWEST MODEL SURFACE

DO JLEVP=KLOLEV,KLEVP
  IF (LDBLOW(JLEVP)) THEN
    DO JL=KSTART,KPROF
      IF (LDBELO(JL,JLEVP)) THEN
        PVVPP(JL,JLEVP) = ZVVEL(JL,KFLEV)
      ENDIF
    ENDDO
  ENDIF
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PPVVEL',1,ZHOOK_HANDLE)
END SUBROUTINE PPVVEL
