SUBROUTINE SEKF_GAIN(KL,KBLOCK,KOBS,KVAR,POERR,PBACK,POBS,PMOD,POCGR,PINC,PCANER,PKMAT,KINFO)

!**** *SEKF_GAIN*  - Compute Kalman Gain

!     Input parameters.
!     -----------------

!     KL         :  Grid point index within NPROMA chunk
!     KBLOCK     :  NPROMA chunk index
!     KOBS       :  number of observations
!     KVAR       :  number of control variables
!     POERR      :  observation errors
!     PBACK      :  background error covariance matrix
!     POBS       :  observations
!     PMOD       :  modeled first guess
!     POCGR      :  linearized observation operator H

!     Output parameters.
!     -----------------

!     PINC       :  soil mositure analysis increments
!     PCANER     :  updated analysis error
!     PKMAT      :  Kalman gain
!     KINFO      :  Information on matrix inversion

!     Purpose.
!     --------
!     (1) Compute the gain matrix K for the SEKF soil moisture analysis

!**   Interface.
!     ----------
!        *CALL* *SEKF_GAIN(...)

!     Explicit arguments :      None
!     --------------------

!     Implicit arguments :      None
!     --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        M. Drusch  *ECMWF*

!     Modifications.
!     --------------

!         2008-08-03 - K. Scipal - changed calc. of increments from 
!                              SCM setting to gain matrix setting

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM, JPRB
USE YOMHOOK   ,ONLY : LHOOK, DR_HOOK
USE YOMLUN    ,ONLY : NULOUT

USE YOMSEKF   ,ONLY : NREF_BLOCK, NREF_GP

IMPLICIT NONE

REAL(KIND=JPRB)     ::  ZHOOK_HANDLE

INTEGER(KIND=JPIM), INTENT(IN)  ::  KVAR
INTEGER(KIND=JPIM), INTENT(IN)  ::  KOBS
INTEGER(KIND=JPIM), INTENT(IN)  ::  KL
INTEGER(KIND=JPIM), INTENT(IN)  ::  KBLOCK

REAL(KIND=JPRB), INTENT(IN)     ::  POCGR(KOBS,KVAR)
REAL(KIND=JPRB), INTENT(IN)     ::  PBACK(KVAR,KVAR)
REAL(KIND=JPRB), INTENT(IN)     ::  POERR(KOBS)
REAL(KIND=JPRB), INTENT(IN)     ::  POBS(KOBS)
REAL(KIND=JPRB), INTENT(IN)     ::  PMOD(KOBS)

REAL(KIND=JPRB), INTENT(INOUT)  ::  PCANER(KVAR,KVAR)
REAL(KIND=JPRB), INTENT(OUT)    ::  PINC(KVAR)
REAL(KIND=JPRB), INTENT(OUT)    ::  PKMAT(KVAR,KOBS)
INTEGER(KIND=JPIM), INTENT(OUT) ::  KINFO

REAL(KIND=JPRB)     ::  ZOERR_I(KOBS,KOBS)
REAL(KIND=JPRB)     ::  ZBACK_I(KVAR,KVAR)
REAL(KIND=JPRB)     ::  ZMAT(KVAR,KOBS)
REAL(KIND=JPRB)     ::  ZMAT1(KOBS)
REAL(KIND=JPRB)     ::  ZMAT2(KVAR)
REAL(KIND=JPRB)     ::  ZMAT3(KVAR,KOBS)
REAL(KIND=JPRB)     ::  ZAERRM(KVAR,KVAR)
REAL(KIND=JPRB)     ::  ZMMO(KOBS)
REAL(KIND=JPRB)     ::  ZINC2(KVAR)

REAL(KIND=JPRB)     ::  ZEPS

INTEGER(KIND=JPIM)  ::  J1, J2, J3, INFO

!#include "sekf_matinv.intfb.h"
#include "matrixin.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SEKF_GAIN',0,ZHOOK_HANDLE)

!*    1.0 Initialize dummy matrices and PINC
!     --------------------------------------

KINFO = 0
ZEPS = 100.0_JPRB * tiny (1.0_JPRB)

ZMAT(:,:) = 0.0_JPRB
ZMAT3(:,:) = 0.0_JPRB
PKMAT(:,:) = 0.0_JPRB
PINC(:) = 0.0_JPRB
ZINC2(:) = 0.0_JPRB
ZMAT2(:) = 0.0_JPRB
ZMAT1(:) = 0.0_JPRB
ZAERRM(:,:) = 0.0_JPRB

ZOERR_I(:,:) = 0.0_JPRB
DO J1 = 1,KOBS
  ZOERR_I(J1,J1) = POERR(J1)
ENDDO

ZBACK_I(:,:) = 0.0_JPRB
DO J1 = 1,KVAR
  DO J2 = 1,KVAR
    ZBACK_I(J1,J2) = PBACK(J1,J2)
  ENDDO
ENDDO


!*    2.0 Invert error matrices R and B
!     ---------------------------------

IF (KL==NREF_GP .AND. KBLOCK==NREF_BLOCK) THEN
  WRITE(NULOUT,*) 'R ... (KOBS*KOBS):'
  DO J1 = 1,KOBS
    WRITE(NULOUT,*) (ZOERR_I(J1,J2),J2=1,KOBS)
  ENDDO
ENDIF

CALL MATRIXIN ("CHOLESKI", .false., ZOERR_I(:,:), KOBS, KOBS, ZEPS, INFO)

IF (KL==NREF_GP .AND. KBLOCK==NREF_BLOCK) THEN
  WRITE(NULOUT,*) 'R-1 ... (KOBS*KOBS):'
  WRITE(NULOUT,*) 'INFO from MATRIXIN:', INFO
  DO J1 = 1,KOBS
    WRITE(NULOUT,*) (ZOERR_I(J1,J2),J2=1,KOBS)
  ENDDO
ENDIF

IF (INFO /= 0) THEN
  KINFO = 1
  PINC(:) = 0._JPRB
  PKMAT(:,:) = 0._JPRB
  GOTO 666
ENDIF


IF (KL==NREF_GP .AND. KBLOCK==NREF_BLOCK) THEN
  WRITE(NULOUT,*) 'Background error B ... (NVAR*NVAR):'
  DO J1 = 1,KVAR
    WRITE(NULOUT,*) (ZBACK_I(J1,J2),J2=1,KVAR)
  ENDDO
ENDIF

CALL MATRIXIN ("CHOLESKI", .false., ZBACK_I(:,:), KVAR, KVAR, ZEPS, INFO)

IF (KL==NREF_GP .AND. KBLOCK==NREF_BLOCK) THEN
  WRITE(NULOUT,*) 'B-1 ... (NVAR*NVAR):'
  DO J1 = 1,KVAR
    WRITE(NULOUT,*) (ZBACK_I(J1,J2),J2=1,KVAR)
  ENDDO
ENDIF

IF (INFO /= 0) THEN
  KINFO = 2
  PINC(:) = 0._JPRB
  PKMAT(:,:) = 0._JPRB
  GOTO 666
ENDIF

!*    3.0 Compute [Ht*(R-1)*H + (B-1)]-1
!     ----------------------------------

DO J1 = 1,KVAR
  DO J2 = 1,KOBS
    DO J3 = 1,KOBS
      ZMAT(J1,J2) = ZMAT(J1,J2) + POCGR(J3,J1)*ZOERR_I(J3,J2)
    ENDDO
  ENDDO
ENDDO

DO J1 = 1,KVAR
  DO J2 = 1,KVAR
    DO J3 = 1,KOBS
      ZAERRM(J1,J2) = ZAERRM(J1,J2) + ZMAT(J1,J3)*POCGR(J3,J2)
    ENDDO
  ENDDO
ENDDO

DO J1 = 1,KVAR
  DO J2 = 1,KVAR
    ZAERRM(J1,J2) = ZAERRM(J1,J2) + ZBACK_I(J1,J2)
  ENDDO
ENDDO

CALL MATRIXIN ("CHOLESKI", .false., ZAERRM(:,:), KVAR, KVAR, ZEPS, INFO)

IF (INFO /= 0) THEN
  KINFO = 3
  PINC(:) = 0._JPRB
  PKMAT(:,:) = 0._JPRB
  GOTO 666
ENDIF

PCANER(:,:) = ZAERRM(:,:)

IF (KL==NREF_GP .AND. KBLOCK==NREF_BLOCK) THEN
  WRITE(NULOUT,*) '[Ht*(R-1)*H + (B-1)]-1 (NVAR*NVAR):'
  DO J1 = 1,KVAR
    WRITE(NULOUT,*) (ZAERRM(J1,J2),J2=1,KVAR)
  ENDDO
ENDIF

!*    4.0 Compute gain K
!     ---------------------------------------------

DO J1 = 1,KVAR
  DO J2 = 1,KOBS
    DO J3 = 1,KOBS
      ZMAT3(J1,J2) = ZMAT3(J1,J2) + ZOERR_I(J2,J3) * POCGR(J3,J1)
    ENDDO
  ENDDO
ENDDO

DO J1 = 1,KOBS
  DO J2 = 1,KVAR
    DO J3 = 1,KVAR
      PKMAT(J2,J1) = PKMAT(J2,J1) + ZAERRM(J2,J3) * ZMAT3(J3,J1)
    ENDDO
  ENDDO
ENDDO

IF (KL==NREF_GP .AND. KBLOCK==NREF_BLOCK) THEN
  WRITE(NULOUT,*) 'Kalman Gain G (NVAR*NOBS):'
  DO J2 = 1,KVAR
    WRITE(NULOUT,'(6(1x,f12.8))') (PKMAT(J2,J1),J1=1,KOBS)
  ENDDO
ENDIF

!*    5.0 Compute y - H*xb / observation - forecast
!     ---------------------------------------------

DO J1 = 1,KOBS
  ZMMO(J1) = POBS(J1) - PMOD(J1)
ENDDO

IF (KL==NREF_GP .AND. KBLOCK==NREF_BLOCK) THEN
  WRITE(NULOUT,*) 'y - H*xb / observation - forecast (KOBS):'
  WRITE(NULOUT,'(6(1x,f12.8))') (ZMMO(J1),J1=1,KOBS)
ENDIF

!*    6.0 Compute analysis increments (SCM set up)
!     --------------------------------------------

!DO J1 = 1,KOBS
!  DO J2 = 1,KOBS
!      ZMAT1(J1) = ZMAT1(J1) + ZMMO(J2) * ZOERR_I(J1,J2)
!  ENDDO
!ENDDO

!DO J1 = 1,KVAR
!  DO J2 = 1,KOBS
!      ZMAT2(J1) = ZMAT2(J1) + ZMAT1(J2) * POCGR(J2,J1)
!  ENDDO
!ENDDO

!DO J1 = 1,KVAR
!  DO J2 = 1,KVAR
!      PINC(J1) = PINC(J1) + ZAERRM(J1,J2) * ZMAT2(J2)
!  ENDDO
!ENDDO

!*    7.0 Compute analysis increments from gain matrix
!     ------------------------------------------------

DO J1 = 1,KVAR
  DO J2 = 1,KOBS
    PINC(J1) = PINC(J1) + PKMAT(J1,J2) * ZMMO(J2)
  ENDDO
ENDDO

666 CONTINUE

IF (KL==NREF_GP .AND. KBLOCK==NREF_BLOCK) THEN
  WRITE(NULOUT,*) 'Soil moisture increments (PINC):'
  WRITE(NULOUT,*) (PINC(J1),J1=1,KVAR)
  WRITE(NULOUT,*) 'Status of matrix inversion is:',KINFO
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SEKF_GAIN',1,ZHOOK_HANDLE)
END SUBROUTINE SEKF_GAIN
