SUBROUTINE SUGRIDSPA(&
 ! ----- INPUT -----------------------------------------
 & LDINOR,KFILE, &
 ! ----- OUTPUT ----------------------------------------
 & LDATA,PSPVOR,PSPDIV,PSPT,PSPGFL,PSPSP,PSPOR,PSPSPD, &
 & PSPSVD,PSPUB,PSPVB)  

!**** *SUGRIDSPA*  - Initialize the spectral fields from FA (Arpege or Aladin)
!                    when the spectral fields (including the surface orography)
!                    are stored as grid-point fields in files.

!     Purpose.
!     --------
!         Initialize the spectral fields of the model from ARPEGE ALADIN file
!         when the spectral fields (including the surface orography)
!         are stored as grid-point fields in files.
!         All the grid-point fields read on files are then transformed into
!         spectral ones.
!         In this case, for ARPEGE, the files contain (U,V) and not (Psi,Khi).

!         Note that the structure of this routine must remain close to SUSPECA.
!         The part reading grid-point fields (and converting then in spectral
!         space) has been written after SUGRIDUA (file reading is done under
!         RDPXFA like in SUGRIDUA).

!         Note that there is currently no distinction between the GFL and the
!         GMV fields: it is currently not possible to read the GMV fields as
!         grid-point fields, and the spectral GFL fields as spectral fields
!         (using for example an attribute of YGFL).

!         Example: hydrostatic model with spectral humidity,
!         grid-point ql,qi,rain.
!          We want to read an initial file containing:
!          - Grid point (U,V).
!          - Grid point T.
!          - Grid point q.
!          - Grid point ql,qi,rain (done in SUGRIDUA for ARPEGE files).
!          - Grid point log(surface pressure).
!          - Grid point orography.
!          - Grid point surface fields (done in SUGRIDF for ARPEGE files).
!          Such a file can be provided by a FULL-POS run with CFPFMT='GAUSS',
!           LSPSPEC=.FALSE.

!         A specific remark must be done for surface orography: we must
!          read the grid-point field 'SURFGEOPOTENTIEL' (name contained
!          in variable GFP_GFIS) and not the grid-point field 'SPECSURFGEOPOTEN'
!          (name contained in variable TFP_FIS) for the following reasons:
!          we will read files produced by a FULL-POS transformation with
!          CFPFMT='GAUSS', LFPSPEC=.F. and generally NFPCLI=3.
!          Such FULL-POS jobs will write two
!          articles containing surface orography:
!          - an article named 'SPECSURFGEOPOTEN'
!            (name contained in variable TFP_FIS):
!            it contains a GRID POINT orography which is the interpolation of
!            the departure orography, and it is not the surface orography
!            we want to use in our simulations (it is not spectrally fitted
!            according to the final resolution).
!          - an article named 'SURFGEOPOTENTIEL' (name contained in variable
!            GFP_GFIS):
!            it contains a GRID POINT orography which is the final resolution 
!            (spectrally fitted) orography (if the FULL-POS job is run with
!            NFPCLI=3, this is the orography of the output climatologic field).
!            This is the orography we want to use, and this orography must be
!            spectrally fitted (REESPE+SPEREE must keep this field unchanged).

!**   Interface.
!     ----------
!        *CALL* *SUGRIDSPA(.....)*

!        Explicit arguments :
!        --------------------
!        LDINOR - switch for initializing the orography              (I)
!        KFILE : an indicator for which spectral file is to be read  (I)
!        LDATA : .TRUE. if data is returned from file                (O)
!        PSPVOR etc. - spectral fields                               (O)
!          (all PSP[X] are output but some informatical constraints
!          obliges to declare some of them as INOUT)

!        Implicit arguments :
!        --------------------
!        See modules above.

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        see calls below.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP) after SUSPECA and SUGRIDUA.

!     Modifications.
!     --------------
!      Original     : Dec 2005.
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMAFN   , ONLY : GFP_GFIS
USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NFLEVG   ,NFLSUR   ,NSPEC2   ,NF3D
USE YOMDYNA  , ONLY : LNHX
USE YOMCT0   , ONLY : N3DINI   ,LNHDYN   ,LELAM
USE YOMFA    , ONLY : YFAT     ,YFAPD    ,YFAVD    ,YFASP    ,YFAUGEO  ,YFAVGEO
USE YOM_YGFL , ONLY : YGFL     ,YGFLC   ,YCVGQ
USE YOMGEM   , ONLY : NGPTOT

!LREACUF not yet coded.
!nyc USE YOMMCUF  ,ONLY  : NCUFNR   ,LREACUF   ,RMCUFFP
USE YOMMCUF  ,ONLY  : LREACUF

!     -------------------------------------------------------------------------

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDINOR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFILE 
LOGICAL           ,INTENT(OUT)   :: LDATA 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPVOR(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPDIV(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPT(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPGFL(:,:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPSP(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPOR(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPSPD(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPSVD(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPUB(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPVB(:) 

!     -------------------------------------------------------------------------

!     IFLDGPG: total number of fields to read in grid-point.
!     CLPREF : prefix of fields to read 
!     CLSUFF : suffix of fields to read

INTEGER(KIND=JPIM) :: IFLDGPG
INTEGER(KIND=JPIM) :: IPT, I3DINI
INTEGER(KIND=JPIM) :: JGFL, JLEV
INTEGER(KIND=JPIM) :: IERR, IFLD, IGMV, IOFF, IGFL, IGSP, IST, IEND
INTEGER(KIND=JPIM) :: ISPGFL

!nyc -- will be used in the call to SUGRIDSPG1 --
!nyc LOGICAL   :: LLSPOR
LOGICAL   :: LLINOR
LOGICAL   :: LLREAD

INTEGER(KIND=JPIM),   ALLOCATABLE :: ILEV(:)
CHARACTER (LEN = 4),  ALLOCATABLE :: CLPREF(:)
CHARACTER (LEN = 12), ALLOCATABLE :: CLSUFF(:)
REAL(KIND=JPRB),      ALLOCATABLE :: ZSPU(:,:),ZSPV(:,:)
REAL(KIND=JPRB),      ALLOCATABLE :: ZZSP(:),ZZSP2(:),ZZX(:,:),ZZU(:,:),ZZV(:,:)
REAL(KIND=JPRB),      ALLOCATABLE :: ZGPFIELDS(:,:)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -------------------------------------------------------------------------

#include "abor1.intfb.h"
!nyc #include "euvgeovd.intfb.h"
#include "rdpxfa.intfb.h"
#include "reespe.intfb.h"
#include "speree.intfb.h"
!nyc #include "suspecg1.intfb.h"
#include "uvspe.intfb.h"

!     -------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDSPA',0,ZHOOK_HANDLE)

!     -------------------------------------------------------------------------

!*       0. PRELIMINAR INITIALISATIONS
!           --------------------------

I3DINI=N3DINI
LLINOR=LDINOR

IERR=0
IF (I3DINI /= 0) THEN
  WRITE(NULOUT,*) ' SUGRIDSPA: N3DINI different from 0 not yet coded!'
  IERR=IERR+1
ENDIF
IF (LELAM) THEN
  WRITE(NULOUT,*) ' SUGRIDSPA: case LELAM not yet coded!'
  IERR=IERR+1
ENDIF
IF (LREACUF) THEN
  WRITE(NULOUT,*) ' SUGRIDSPA: reading of MCUF fields not yet coded!'
  IERR=IERR+1
ENDIF
IF (IERR > 0) THEN
  CALL ABOR1(' SUGRIDSPA: ABOR1 CALLED')
ENDIF

!     -------------------------------------------------------------------------

!*       1. FIND NUMBER OF FIELDS TO READ FROM FILE
!           ---------------------------------------

!        1.1 Open file

IF (KFILE == 4) THEN
  CALL ABOR1('SUGRIDSPA : BLENDING REMOVED')
ELSE
  LDATA = .TRUE.
  ! k.y.: call to OPENFA is done under RDPXFA.
ENDIF

!        1.2 Conditional allocations

IF (LELAM) THEN
  ALLOCATE(ZSPU(NFLSUR,NSPEC2))
  ALLOCATE(ZSPV(NFLSUR,NSPEC2))
ENDIF

!        1.3 Count fields read in g.p.

IFLDGPG=0
! Orography :
IF (LLINOR) THEN
  IFLDGPG=IFLDGPG+1
ENDIF
! Ps and 3D fields :
IF (I3DINI == 0)  THEN
  !ky: LREACUF=T not yet coded.
  !nyc  IF (LREACUF) THEN
  !nyc    IFLDGPG = IFLDGPG + 1 + NF3D*NFLEVG + NCUFNR
  !nyc  ELSE
    IFLDGPG = IFLDGPG + 1 + NF3D*NFLEVG 
  !nyc  ENDIF
ENDIF
! Remove "X" divergence term from reading when set
IF (LNHX) THEN
  IFLDGPG = IFLDGPG - NFLEVG
ENDIF
! Remove YCVGQ from reading when set spectral
IF (YCVGQ%LSP) THEN
  IFLDGPG = IFLDGPG - NFLEVG
ENDIF   

!     -------------------------------------------------------------------------

!*       2. INITIALIZE FROM FILE
!           --------------------

!*       2.1 FIELDS STORED AS GRID-POINT FIELDS IN FILES

! k.y.: case I3DINI /= 0 not yet coded.

IF (I3DINI == 0 .AND. IFLDGPG > 0) THEN

  !     2.1.1 Allocations, setup descriptors.

  ALLOCATE(CLPREF(IFLDGPG),ILEV(IFLDGPG),CLSUFF(IFLDGPG))

  IFLD=0 ! numbering of fields read on files.

  ! MCUF variables (not yet coded):
  !nyc IF (LREACUF) THEN
  !nyc   treatment of fields FSP1 to FSP5 not yet coded.
  !nyc ENDIF

  ! GMV variables:
  IGMV=0
  IF (NF3D > 0) THEN
    ! U:
    IGMV=IGMV+1
    IFLD=IFLD+1
    IOFF=(IFLD-1)*NFLEVG 
    DO JLEV=1,NFLEVG
      CLPREF(IOFF+JLEV)='S   '
      ILEV(IOFF+JLEV)=JLEV
      CLSUFF(IOFF+JLEV)=YFAUGEO%CLNAME(1:12)
    ENDDO
    ! V:
    IGMV=IGMV+1
    IFLD=IFLD+1
    IOFF=(IFLD-1)*NFLEVG 
    DO JLEV=1,NFLEVG
      CLPREF(IOFF+JLEV)='S   '
      ILEV(IOFF+JLEV)=JLEV
      CLSUFF(IOFF+JLEV)=YFAVGEO%CLNAME(1:12)
    ENDDO
    ! Temperature :
    IGMV=IGMV+1
    IFLD=IFLD+1
    IOFF=(IFLD-1)*NFLEVG 
    DO JLEV=1,NFLEVG
      CLPREF(IOFF+JLEV)='S   '
      ILEV(IOFF+JLEV)=JLEV
      CLSUFF(IOFF+JLEV)=YFAT%CLNAME(1:12)
    ENDDO
    ! Pressure departure variable:
    IF (LNHDYN) THEN
      IGMV=IGMV+1
      IFLD=IFLD+1
      IOFF=(IFLD-1)*NFLEVG 
      DO JLEV=1,NFLEVG
        CLPREF(IOFF+JLEV)='S   '
        ILEV(IOFF+JLEV)=JLEV
        CLSUFF(IOFF+JLEV)=YFAPD%CLNAME(1:12)
      ENDDO
    ENDIF
    ! Vertical divergence variable:
    IF (LNHDYN) THEN
      IGMV=IGMV+1
      IFLD=IFLD+1
      IOFF=(IFLD-1)*NFLEVG 
      DO JLEV=1,NFLEVG
        CLPREF(IOFF+JLEV)='S   '
        ILEV(IOFF+JLEV)=JLEV
        CLSUFF(IOFF+JLEV)=YFAVD%CLNAME(1:12)
      ENDDO
    ENDIF
  ENDIF

  ! GFL variables:
  IGFL=0 ! numbering of spectral GFL fields read on files.
  IGSP=0 ! numbering of spectral GFL (read or not).
  DO JGFL=1,YGFL%NUMFLDS
    SELECT CASE (YGFLC(JGFL)%LSP)
    CASE(.TRUE.)
      IGSP=IGSP+1
      SELECT CASE (YGFLC(JGFL)%NREQIN)
      CASE(1)
        IGFL=IGFL+1
        IFLD=IFLD+1
        IF (IGFL > YGFL%NUMSPFLDS) CALL ABOR1('SUGRIDUA : ERROR IGFL')
        IOFF=(IFLD-1)*NFLEVG 
        DO JLEV=1,NFLEVG
          CLPREF(IOFF+JLEV)='S   '
          ILEV(IOFF+JLEV)=JLEV
          CLSUFF(IOFF+JLEV)=YGFLC(JGFL)%CNAME(1:12)
        ENDDO
      END SELECT
    END SELECT
  ENDDO
  IF (IGSP /= YGFL%NUMSPFLDS) CALL ABOR1('SUGRIDSPA : ERROR IGSP')

  ! GMVS: Surface pressure:
  IPT=IFLD*NFLEVG+1
  CLPREF(IPT)=YFASP%CLNAME(1:4)
  ILEV(IPT)=0
  CLSUFF(IPT)=YFASP%CLNAME(5:)

  ! Surface orography:
  !  See remark above, this is the article named via GFP_GFIS and not
  !   the one named via TFP_FIS we want to read.
  IF (LLINOR) THEN
    IPT=IPT+1
    CLPREF(IPT)=GFP_GFIS%CLNAME(1:4)
    ILEV(IPT)=0
    CLSUFF(IPT)=GFP_GFIS%CLNAME(5:)
  ENDIF

  WRITE(NULOUT,'(1X,A,1X,I5,1X,I5)') &
   & 'SUGRIDSPA: Number of g.p. 2D fields read:',IPT,IFLDGPG
  call flush(nulout)

  IF (IPT /= IFLDGPG) THEN
    CALL ABOR1('SUGRIDSPA : FIELDS DESCRIPTORS ARE UNCOMPLETE !')
  ENDIF

  !     2.1.2 Read and distribute grid-point fields.

  ALLOCATE(ZGPFIELDS(NGPTOT,IFLDGPG))
  CALL RDPXFA(ZGPFIELDS,CLPREF,ILEV,CLSUFF,IFLDGPG,KFILE)

  !     2.1.3 Copy g-point fields to buffer then transform into spectral space.

  ALLOCATE(ZZU(NGPTOT,NFLEVG))
  ALLOCATE(ZZV(NGPTOT,NFLEVG))
  ALLOCATE(ZZX(NGPTOT,NFLEVG))
  ALLOCATE(ZZSP(NGPTOT))
  ALLOCATE(ZZSP2(NGPTOT))

  IST=1
  IEND=NGPTOT

  IFLD=0 ! numbering of fields read on files.

  ! MCUF variables (not yet coded):
  !nyc IF (LREACUF) THEN
  !nyc   treatment of fields FSP1 to FSP5 not yet coded.
  !nyc ENDIF

  ! GMV variables:
  IGMV=0
  IF (NF3D > 0) THEN

    ! U and V:
    IGMV=IGMV+1
    IFLD=IFLD+1
    IOFF=(IFLD-1)*NFLEVG 
    DO JLEV=1,NFLEVG
      ZZU(IST:IEND,JLEV)=ZGPFIELDS(IST:IEND,IOFF+JLEV)
    ENDDO
    IGMV=IGMV+1
    IFLD=IFLD+1
    IOFF=(IFLD-1)*NFLEVG 
    DO JLEV=1,NFLEVG
      ZZV(IST:IEND,JLEV)=ZGPFIELDS(IST:IEND,IOFF+JLEV)
    ENDDO
    ! Provides Vor and Div.
    IF (LELAM) THEN
      !nyc Not yet coded, probably one of the following possibilities:
      !nyc Either CALL EREESPE on (U,V) -> ZSPU,ZSPV
      !nyc  then CALL EUVGEOVD(ZSPU,ZSPV,PSPVOR,PSPDIV,PSPUB,PSPVB)
      !nyc  or CALL EUVSPE(PSPVOR,PSPDIV,ZZU,ZZV,NFLSUR,NFLEVG,1)
      !nyc  which directly provides (PSPVOR,PSPDIV).
    ELSE
      CALL UVSPE(PSPVOR,PSPDIV,ZZU,ZZV,NFLSUR,NFLEVG,1)
    ENDIF

    ! Temperature :
    IGMV=IGMV+1
    IFLD=IFLD+1
    IOFF=(IFLD-1)*NFLEVG 
    DO JLEV=1,NFLEVG
      ZZX(IST:IEND,JLEV)=ZGPFIELDS(IST:IEND,IOFF+JLEV)
    ENDDO
    CALL REESPE(NFLSUR,NFLEVG,PSPT,ZZX)

    ! Pressure departure variable:
    IF (LNHDYN) THEN
      IGMV=IGMV+1
      IFLD=IFLD+1
      IOFF=(IFLD-1)*NFLEVG 
      DO JLEV=1,NFLEVG
        ZZX(IST:IEND,JLEV)=ZGPFIELDS(IST:IEND,IOFF+JLEV)
      ENDDO
      CALL REESPE(NFLSUR,NFLEVG,PSPSPD,ZZX)
    ENDIF

    ! Vertical divergence variable:
    IF (LNHDYN) THEN
      IGMV=IGMV+1
      IFLD=IFLD+1
      IOFF=(IFLD-1)*NFLEVG 
      DO JLEV=1,NFLEVG
        ZZX(IST:IEND,JLEV)=ZGPFIELDS(IST:IEND,IOFF+JLEV)
      ENDDO
      CALL REESPE(NFLSUR,NFLEVG,PSPSVD,ZZX)
    ENDIF

  ENDIF

  ! GFL variables:
  IGFL=0
  ISPGFL=0
  DO JGFL=1,YGFL%NUMFLDS
    LLREAD=(YGFLC(JGFL)%NREQIN==1).AND.((YGFLC(JGFL)%NCOUPLING==1).OR.(KFILE/=9))
    SELECT CASE (YGFLC(JGFL)%LSP)
    CASE(.TRUE.)
      ISPGFL=ISPGFL+1
      SELECT CASE (YGFLC(JGFL)%NREQIN)
      CASE(1)
        SELECT CASE (LLREAD)
        CASE(.TRUE.)
          IGFL=IGFL+1
          IFLD=IFLD+1
          IOFF=(IFLD-1)*NFLEVG 
          DO JLEV=1,NFLEVG
            ZZX(IST:IEND,JLEV)=ZGPFIELDS(IST:IEND,IOFF+JLEV)
          ENDDO
          CALL REESPE(NFLSUR,NFLEVG,PSPGFL(1:NFLSUR,1:NSPEC2,ISPGFL:ISPGFL),ZZX)
        CASE DEFAULT
          ! fill directly the spectral GFL to zero!
          PSPGFL(:,:,ISPGFL)=0.0_JPRB
        END SELECT
      CASE(-1) 
        DO JLEV=1,NFLEVG
          ZZX(IST:IEND,JLEV)=YGFLC(ISPGFL)%REFVALI
        ENDDO
        CALL REESPE(NFLSUR,NFLEVG,PSPGFL(1:NFLSUR,1:NSPEC2,ISPGFL:ISPGFL),ZZX)
      CASE DEFAULT
        ! fill directly the spectral GFL to zero!
        PSPGFL(:,:,ISPGFL)=0.0_JPRB
      END SELECT
    END SELECT
  ENDDO

  ! GMVS: Surface pressure:
  IPT=IFLD*NFLEVG+1
  ZZSP(IST:IEND)=ZGPFIELDS(IST:IEND,IPT)
  IF (KFILE /= 3 .AND. I3DINI == 0) THEN
    ! If ZZSP contains pressure, transform it in log(pressure).
    ! The following test must remain consistent with the one done in SUSPSP
    ! (value 8E+4_JPRB must be put in a module constant in the future).
    ! This transformation must not be done if KFILE=3 because in this case
    ! ZZSP contains pressure increments.
    IF (MAXVAL(ZZSP) > 8E+4_JPRB) THEN
      ZZSP(IST:IEND)=LOG(ZZSP(IST:IEND))
    ENDIF
  ENDIF
  CALL REESPE(1,1,PSPSP,ZZSP)

  ! Surface orography:
  IF (LLINOR) THEN
    IPT=IPT+1
    ZZSP(IST:IEND)=ZGPFIELDS(IST:IEND,IPT)
    CALL REESPE(1,1,PSPOR,ZZSP)
    ! * Check that the g.p. initial orography is spectrally fitted,
    !   otherwise abort.
    CALL SPEREE(1,1,PSPOR,ZZSP2)
    ZZSP2(IST:IEND)=ABS(ZZSP2(IST:IEND)-ZZSP(IST:IEND))
    IF (MAXVAL(ZZSP2(IST:IEND)) > 0.000001_JPRB) THEN
      WRITE(NULOUT,*) ' SUGRIDSPA: gp orography SURFGEOPOTENTIEL'
      WRITE(NULOUT,*) '  read on ARPEGE files not well spectrally fitted!'
      WRITE(NULOUT,*) '  maxval(abs(speree*reespe*orog-orog))=',MAXVAL(ZZSP2(IST:IEND))
      CALL ABOR1(' SUGRIDSPA: ABOR1 CALLED')
    ENDIF
  ENDIF

  WRITE(NULOUT,'(1X,A,1X,I5,1X,I5)') &
   & 'SUGRIDSPA: Number of g.p. 2D fields transformed in spec:',IPT,IFLDGPG
  call flush(nulout)

  DEALLOCATE(ZGPFIELDS)
  DEALLOCATE(ZZU)
  DEALLOCATE(ZZV)
  DEALLOCATE(ZZX)
  DEALLOCATE(ZZSP)
  DEALLOCATE(ZZSP2)

  !     2.1.4 Release descriptors

  DEALLOCATE(CLPREF)
  DEALLOCATE(CLSUFF)
  DEALLOCATE(ILEV  )

ENDIF ! (IFLDGPG > 0)

!*       2.2 DEALLOCATE ARRAYS AND CLOSE FILE.

IF (LELAM) THEN
  DEALLOCATE(ZSPU)
  DEALLOCATE(ZSPV)
ENDIF

! k.y.: call to FAIRME has already been done in RDPXFA.

!     -------------------------------------------------------------------------

!*       3. INITIALIZE WITH ARTIFICIAL DATA
!           -------------------------------

! k.y.: not yet coded (call SUGRIDSPG1 instead of SUSPECG1, all the fields
!       other than orography are assumed to be stored as grid-point fields).
!nyc IF (I3DINI /= 0) THEN
!nyc   LLINOR=.FALSE.
!nyc   IF (LELAM .OR. LNHDYN) THEN
!nyc     CALL ABOR1('SUGRIDSPA : MISSING CODE IN SUSPECG1')
!nyc   ENDIF
!nyc   CALL SUGRIDSPG1(PSPVOR,PSPDIV,PSPT,PSPGFL,PSPSP,PSPOR,&
!nyc    & LLINOR,LLSPOR)  
!nyc ENDIF

!     -------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUGRIDSPA',1,ZHOOK_HANDLE)
END SUBROUTINE SUGRIDSPA
