SUBROUTINE SUHEG

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NSMAX    ,NSPEC    ,NUMP
USE YOMCT0   , ONLY : LNHDYN   ,LSLAG    ,LTWOTL
USE YOMDYNA  , ONLY : LNH_GEOGW
USE YOMDYN   , ONLY : SCGMAP   ,SIVP     ,SIHEG    ,SIHEG2   ,&
 & TDT      ,BETADT   ,SITIME   ,VESL     ,XIDT     ,LSIDG
USE YOMLAP   , ONLY : NSE0L    ,RLAPDI   ,RLAPIN   ,MYMS
USE YOMLUN   , ONLY : NULOUT

!**** *SUHEG*   - Initialize the solver of the Helmholz equation

!     Purpose.    Initialize the solver of the Helmholz equation
!     --------    in the semi-implicit, in case of stretching and
!                 use of geographical divergence (LSIDG=.T.).
!                 - Hydrostatic model
!                 - Phi-gw NH model

!**   Interface.
!     ----------
!        *CALL* *SUHEG

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. YESSAD (OCTOBER 1993)

!     Modifications.
!     --------------
!   M. Tolstykh 31-08-94  Variable decentering option for semi-Lag. scheme
!   K. YESSAD (MARCH 1995):
!          - Optimisation in case of variable decentering.
!          - Remove of IMSL routines.
!          - As precaution put 0 in Helmholtz operators before computations
!            in order to replace uninitialised parts of them by zeros.
!   L. Isaksen 05-05-96  NFLEV --> NFLEVG
!   Modified OCT 1996 by K. YESSAD: removal of variable uncentering.
!   C. Temperton 97-02-24: modified first timestep when XIDT>0
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   K. Yessad 15-May-2006: memory optimisations for stretched geometry
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) :: ZD(NFLEVG,0:NSMAX)
REAL(KIND=JPRB) :: ZE(NFLEVG,0:NSMAX)
REAL(KIND=JPRB) :: ZF(NFLEVG,0:NSMAX)
REAL(KIND=JPRB) :: ZES(NFLEVG,0:NSMAX)
REAL(KIND=JPRB) :: ZFS(NFLEVG,0:NSMAX)

INTEGER(KIND=JPIM) :: ILUX, IS0, ISE, JL, JN, JSE, IM, JMLOC
LOGICAL :: LLCOMPSIHEG

REAL(KIND=JPRB) :: ZBETADT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!      ----------------------------------------------------------------

#include "suher.h"
#include "suhes.h"

!      ----------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUHEG',0,ZHOOK_HANDLE)

!      ----------------------------------------------------------------

IF((.NOT.LNHDYN) .AND. (SITIME /= TDT))THEN
  SITIME=TDT
  LLCOMPSIHEG=.TRUE.
  WRITE(NULOUT,*) ' SUHEG: COMPUTES HELMHOLTZ EQUATION OPERATOR'&
   & ,' IN CASE OF SEMI-IMPLICIT SCHEME IN NOT'&
   & ,' REDUCED DIVERGENCE (LSIDG=.T.): HYDROSTATIC MODEL'  
ELSEIF (LNHDYN .AND. LNH_GEOGW) THEN
  ! in this case SITIME has been already updated in the caller.
  ! (cf. SUNHHEG).
  LLCOMPSIHEG=.TRUE.
  WRITE(NULOUT,*) ' SUHEG: COMPUTES HELMHOLTZ EQUATION OPERATOR'&
   & ,' IN CASE OF SEMI-IMPLICIT SCHEME IN NOT'&
   & ,' REDUCED DIVERGENCE (LSIDG=.T.): Phi-gw NH MODEL'  
ELSE
  LLCOMPSIHEG=.FALSE.
ENDIF

IF(LLCOMPSIHEG)THEN

!*       1.    INITIALIZE HELMHOLTZ EQUATION SOLVER.
!              -------------------------------------

  ILUX=NSPEC

  IF (LSLAG) THEN
    IF (LTWOTL) THEN
      ZBETADT=(1.0_JPRB+VESL)*(1.0_JPRB+XIDT)*BETADT
    ELSE
      ZBETADT=(1.0_JPRB+VESL)*BETADT
    ENDIF
  ELSE
    ZBETADT=BETADT
  ENDIF

  DO JL=1,NFLEVG
    DO JSE=1,ILUX
      SIHEG(JL,JSE,1)=0.0_JPRB
      SIHEG(JL,JSE,2)=0.0_JPRB
      SIHEG(JL,JSE,3)=0.0_JPRB
    ENDDO
  ENDDO

  DO JMLOC=1,NUMP

    IM=MYMS(JMLOC)

    IF (IM == 0) THEN

      DO JL=1,NFLEVG
        DO JSE=1,NSMAX+1
          SIHEG2(JL,JSE,2)=0.0_JPRB
          SIHEG2(JL,JSE,3)=0.0_JPRB
        ENDDO
      ENDDO

      IS0=NSE0L(JMLOC)
      DO JL=1,NFLEVG
        DO JN=0,NSMAX
          ISE=IS0+JN+1
          ZD(JL,JN)=1.0_JPRB &
           & -SIVP(JL)*RLAPDI(JN)*SCGMAP(ISE,1)*(ZBETADT*SITIME*0.5_JPRB)**2  
        ENDDO
        DO JN=0,NSMAX-1
          ISE=IS0+JN+1
          ZE(JL,JN)=-SIVP(JL)*RLAPDI(JN+1)*SCGMAP(ISE,2)&
           & *(ZBETADT*SITIME*0.5_JPRB)**2  
          ZES(JL,JN)=-SIVP(JL)*RLAPDI(JN)*SCGMAP(ISE,2)&
           & *(ZBETADT*SITIME*0.5_JPRB)**2  
        ENDDO
        DO JN=0,NSMAX-2
          ISE=IS0+JN+1
          ZF(JL,JN)=-SIVP(JL)*RLAPDI(JN+2)*SCGMAP(ISE,3)&
           & *(ZBETADT*SITIME*0.5_JPRB)**2  
          ZFS(JL,JN)=-SIVP(JL)*RLAPDI(JN)*SCGMAP(ISE,3)&
           & *(ZBETADT*SITIME*0.5_JPRB)**2  
        ENDDO
      ENDDO
      CALL SUHER(NSMAX+1,NFLEVG,NFLEVG,ZD(1,0),ZE(1,0),ZES(1,0),&
       & ZF(1,0),ZFS(1,0),SIHEG(1,IS0+1,1),SIHEG(1,IS0+1,2),&
       & SIHEG(1,IS0+1,3),SIHEG2(1,1,2),SIHEG2(1,1,3))  

    ELSE

      IS0=NSE0L(JMLOC)
      DO JL=1,NFLEVG
        DO JN=IM,NSMAX
          ISE=IS0+JN+1-IM
          ZD(JL,JN)=RLAPIN(JN)&
           & -SIVP(JL)*SCGMAP(ISE,1)*(ZBETADT*SITIME*0.5_JPRB)**2  
          ZE(JL,JN)=-SIVP(JL)*SCGMAP(ISE,2)*(ZBETADT*SITIME*0.5_JPRB)**2
          ZF(JL,JN)=-SIVP(JL)*SCGMAP(ISE,3)*(ZBETADT*SITIME*0.5_JPRB)**2
        ENDDO
      ENDDO
      CALL SUHES(NSMAX+1-IM,NFLEVG,NFLEVG,ZD(1,IM),ZE(1,IM),ZF(1,IM),&
       & SIHEG(1,IS0+1,1),SIHEG(1,IS0+1,2),SIHEG(1,IS0+1,3))  

    ENDIF

  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUHEG',1,ZHOOK_HANDLE)
END SUBROUTINE SUHEG
