SUBROUTINE SUMP_DILB(KNSMIN,KNSMAX,KULOUT, &
 & KNASM0,KNPROCM,KNUMPP,KNUMP,KNUMPX,KMYMS, &
 & KRDEBL,KRDEBXL,KPNML,KPNMXL)

!**** *SUMP_DILB* - Routine to initialize parallel environment for conf 911.
!                   Distribution of zonal wavenumbers.

!     Purpose.
!     --------
!      see above.

!**   Interface.
!     ----------
!        *CALL* *SUMP_DILB*

!        Explicit arguments :
!        --------------------

!         INPUT:
!          KNSMIN   : Lower truncation.
!          KNSMAX   : Upper truncation.
!          KULOUT   : Logical unit for prints.

!         OUTPUT:
!          KNASM0   : KNASM0(JM): index of the coefficient
!                     corresponding to the wave number (m=JM,n=JM)
!                     for the lower truncation.
!          KNPROCM  : KNPROCM(JM): the processor which treats the zonal
!                     wavenumber m=JM.
!          KNUMPP   : Number of spectral waves handled by this
!                     processor for KNSMIN, for all procs.
!          KNUMP    : KNUMPP of the current processor.
!          KNUMPX   : The max of KNUMPP over all processors.
!          KMYMS    : Actual wave numbers handled by this processor.
!          KRDEBL   : Index array for the PNM of the lower truncation.
!          KRDEBXL  : Index array for the PNM of the upper truncation.
!          KPNML    : DM-local dimension for the PNM of the lower truncation.
!          KPNMXL   : DM-local dimension for the PNM of the upper truncation.

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. Yessad (CNRM/GMAP), after SUMP

!     Modifications.
!     --------------
!        Original   : July 2005
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP    , ONLY : MYSETW
USE YOMCT0   , ONLY : NPRGPNS  ,NPRGPEW  ,NPRTRW   ,NPRTRV   ,NPROC    ,&
 & NPRINTLEV

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)  :: KNSMIN
INTEGER(KIND=JPIM),INTENT(IN)  :: KNSMAX
INTEGER(KIND=JPIM),INTENT(IN)  :: KULOUT
INTEGER(KIND=JPIM),INTENT(OUT) :: KNASM0(0:KNSMIN)
INTEGER(KIND=JPIM),INTENT(OUT) :: KNPROCM(0:KNSMIN)
INTEGER(KIND=JPIM),INTENT(OUT) :: KNUMPP(NPRTRW)
INTEGER(KIND=JPIM),INTENT(OUT) :: KNUMP
INTEGER(KIND=JPIM),INTENT(OUT) :: KNUMPX
INTEGER(KIND=JPIM),INTENT(OUT) :: KMYMS(KNSMIN+1)
INTEGER(KIND=JPIM),INTENT(OUT) :: KRDEBL(KNSMIN+2)
INTEGER(KIND=JPIM),INTENT(OUT) :: KRDEBXL(KNSMIN+2)
INTEGER(KIND=JPIM),INTENT(OUT) :: KPNML
INTEGER(KIND=JPIM),INTENT(OUT) :: KPNMXL

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IERR
INTEGER(KIND=JPIM) :: INSPEC,INSPEC2
INTEGER(KIND=JPIM) :: JMLOC,JMGLO,IMGLO,INCRMIN,INCRMAX,JROC

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!      -----------------------------------------------------------------

INTERFACE
#include "ini_spec_dist.h"
END INTERFACE

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUMP_DILB',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

! * Check that there is only A-level of parallelisation.
IERR=0
IF (NPRGPNS /= NPRTRW) IERR=IERR+1
IF (NPRGPNS /= NPROC) IERR=IERR+1
IF (NPRGPEW /= 1) IERR=IERR+1
IF (NPRTRV /= 1) IERR=IERR+1

IF (IERR > 0) THEN
  WRITE(KULOUT,*) ' SUMP_DILB: in configuration 911, the following'
  WRITE(KULOUT,*) '  conditions must be fulfilled:'
  WRITE(KULOUT,*) '  NPRGPNS=NPRTRW=NPROC'
  WRITE(KULOUT,*) '  NPRGPEW=1; NPRTRV=1'
  CALL ABOR1(' SUMP_DILB: ABOR1 CALLED')
ENDIF

!     ------------------------------------------------------------------

! * These calculations must match what is done under SUMP, but
!   we don't want to call TRANS_INQ nor any other 'TFL' routine here.

! * Calculation of KNASM0,KNPROCM,KNUMPP,KMYMS:

CALL INI_SPEC_DIST(KNSMIN,KNSMIN,&
 & NPRTRW,MYSETW,KASM0=KNASM0,KPROCM=KNPROCM,KUMPP=KNUMPP,&
 & KSPEC=INSPEC,KSPEC2=INSPEC2,KMYMS=KMYMS)
KNUMP=KNUMPP(MYSETW) 
KNUMPX=MAXVAL(KNUMPP)

!     ------------------------------------------------------------------

! * Calculation of KRDEBL,KRDEBXL,KPNML,KPNMXL:

KRDEBL(1)=1
KRDEBXL(1)=1

DO JMLOC=1,KNUMP
  IMGLO=KMYMS(JMLOC)
  INCRMIN=KNSMIN+1-IMGLO
  INCRMAX=KNSMAX+1-IMGLO
  KRDEBL(JMLOC+1)=KRDEBL(JMLOC)+INCRMIN
  KRDEBXL(JMLOC+1)=KRDEBXL(JMLOC)+INCRMAX
ENDDO

KPNML=KRDEBL(KNUMP+1)-1
KPNMXL=KRDEBXL(KNUMP+1)-1

!     ------------------------------------------------------------------

! * Printings:

IF (NPRINTLEV > 0) THEN
  WRITE(KULOUT,'(A)')
  WRITE(KULOUT,'(A)') ' --- SUMP_DILB:'
  WRITE(KULOUT,'(A)') ' KNPROCM:'
  WRITE(KULOUT,'(20(1X,I3))') (KNPROCM(JMGLO), JMGLO=0,KNSMIN)
  WRITE(KULOUT,'(A)') ' KNUMPP:'
  WRITE(KULOUT,'(20(1X,I3))') (KNUMPP(JROC), JROC=1,NPRTRW)
  WRITE(KULOUT,'(A)') ' KMYMS:'
  WRITE(KULOUT,'(20(1X,I3))') (KMYMS(JMLOC), JMLOC=1,KNUMP)
  WRITE(KULOUT,'(A)') ' KRDEBL:'
  WRITE(KULOUT,'(10(1X,I6))') (KRDEBL(JMLOC), JMLOC=1,KNUMP)
  WRITE(KULOUT,'(A)') ' KRDEBXL:'
  WRITE(KULOUT,'(10(1X,I6))') (KRDEBXL(JMLOC), JMLOC=1,KNUMP)
  WRITE(KULOUT,'(A,1X,I6)') ' KPNML:',KPNML
  WRITE(KULOUT,'(A,1X,I6)') ' KPNMXL:',KPNMXL
  CALL FLUSH(KULOUT)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUMP_DILB',1,ZHOOK_HANDLE)
END SUBROUTINE SUMP_DILB
