SUBROUTINE SUNCMAX(KNMAX,PSTRET,PSEUIL,KNCMAX)

!**** *SUNCMAX - Computation of an optimal value for upper truncation used
!                by dilatation matrices.

!     Purpose.
!     --------

!     Computation of an optimal value for upper truncation used
!     by dilatation matrices.

!     When stretching, computation follows the following algorithm:
!     - Ns is in variable KNMAX; c (stretching) is in variable PSTRET.
!     - One computes the ratio Nc/(c*Ns) which allows the norm
!       || Cont(m=0) * Dila(m=0) || to be <= PSEUIL.
!       Cont and Dila are the contraction/dilatation matrices.
!       Results show that:
!        * log(|| Cont(m=0) * Dila(m=0) ||) is close
!          to be proportional to [ Nc/(c*Ns) - 1 ]. 
!        * the quasi-constant
!          a2 = log(|| Cont(m=0) * Dila(m=0) ||)/[ Nc/(c*Ns) - 1 ]
!          can be adjusted by a curve: alpha(c) (Ns)**r + beta(c)
!          where:
!          exponent "r" is quasi-independant from c: r=2/3 is satisfactory.
!          "alpha" only depends on stretching "c": an adjustment gives
!           alpha = [ (-1.08) / (c-1)**0.86 ] - 6.54
!          "beta" only depends on stretching "c": an adjustment gives
!           beta = [ (0.65) / (c-1)**0.86 ] + 5.91
!     - Once the ratio ZRAP is computed, one finds the admissible
!       truncation KNCMAX immediately above the Nc found by multiplying
!       the ratio found by c*Ns.
!       An admissible truncation is a truncation, the number of longitudes
!       of its Gaussian grid writes as 2**(1+r1) * 3**r2 * 5**r3.

!     Previous formulae are not completely exact, so a correction is
!     applied to Nc to satisfy to the two following constraints: 
!     - if the threshold PSEUIL is 1.E-13 and the stretching coefficient
!       is 1.5, 2.0, 2.5, 3.0, 3.5 or 4.0, Nc is reset to an "exact" value
!       (correction actually acts for some isolated cases).
!     - Nc is a monotonic function of:
!       * the threshold PSEUIL.
!       * the stretching "c".
!       * tre lower truncation Ns.

!**   Interface.
!     ----------
!        *CALL* *SUNCMAX(...)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KNMAX     - lower truncation.
!          PSTRET    - stretching coefficient.
!          PSEUIL    - allowed threshold for Norm ( Cont(m=0)*Dila(m=0) )

!        OUTPUT:
!          KNCMAX    - upper truncation.

!        Implicit arguments : none.
!        --------------------

!     Method.
!     -------
!        See documentation about conf 911.

!     Externals.
!     ----------
!       SUADMI,SUNCET13.

!     Reference.
!     ----------

!     Author.
!     -------
!        K. YESSAD (CNRM/GMAP). 

!     Modifications.
!     --------------
!        Original : APRIL 1999
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

! * DUMMY:
INTEGER(KIND=JPIM),INTENT(IN)    :: KNMAX 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSTRET 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSEUIL 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KNCMAX 
! * PARAMETER:
INTEGER(KIND=JPIM),PARAMETER :: JPNMAXN=20
INTEGER(KIND=JPIM),PARAMETER :: JPNMAXX=332
INTEGER(KIND=JPIM),PARAMETER :: JPNMAXXX=16000

INTEGER(KIND=JPIM) :: INCMAX
INTEGER(KIND=JPIM) :: INCMAX13
INTEGER(KIND=JPIM) :: INCMAXI13
INTEGER(KIND=JPIM) :: INCMAXS13
INTEGER(KIND=JPIM) :: INCMAXET13(JPNMAXN:JPNMAXX,6)
INTEGER(KIND=JPIM) :: INMADMI(JPNMAXXX,2)
INTEGER(KIND=JPIM) :: INMAXN
INTEGER(KIND=JPIM) :: INMAXX
INTEGER(KIND=JPIM) :: INMAXXX
INTEGER(KIND=JPIM) :: IS1

LOGICAL :: LLADMIS(JPNMAXXX)
LOGICAL :: LLCH
LOGICAL :: LLCM1
LOGICAL :: LLCNH
LOGICAL :: LLCP6
LOGICAL :: LLC1
LOGICAL :: LLC12
LOGICAL :: LLC2
LOGICAL :: LLC23
LOGICAL :: LLC3
LOGICAL :: LLC34
LOGICAL :: LLC4
LOGICAL :: LLC45
LOGICAL :: LLC5
LOGICAL :: LLC56
LOGICAL :: LLC6

REAL(KIND=JPRB) :: ZA2
REAL(KIND=JPRB) :: ZA2I
REAL(KIND=JPRB) :: ZA2S
REAL(KIND=JPRB) :: ZALPHA
REAL(KIND=JPRB) :: ZALPHA0
REAL(KIND=JPRB) :: ZBETA
REAL(KIND=JPRB) :: ZBETA0
REAL(KIND=JPRB) :: ZKA
REAL(KIND=JPRB) :: ZKB
REAL(KIND=JPRB) :: ZNCMAX
REAL(KIND=JPRB) :: ZNCMAX13
REAL(KIND=JPRB) :: ZNCMAXI13
REAL(KIND=JPRB) :: ZNCMAXS13
REAL(KIND=JPRB) :: ZRA
REAL(KIND=JPRB) :: ZRAP
REAL(KIND=JPRB) :: ZRAP13
REAL(KIND=JPRB) :: ZRAPI13
REAL(KIND=JPRB) :: ZRAPS13
REAL(KIND=JPRB) :: ZRB
REAL(KIND=JPRB) :: ZRR
REAL(KIND=JPRB) :: ZSEUIL13
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "suadmi.intfb.h"
#include "suncet13.intfb.h"

! ------------------------------------------------------------------

!*  0. PRELIMINARY INITIALISATIONS.
!   -------------------------------

IF (LHOOK) CALL DR_HOOK('SUNCMAX',0,ZHOOK_HANDLE)
ZRA=0.86_JPRB
ZKA=-1.08_JPRB 
ZALPHA0=-6.54_JPRB

ZRB=0.86_JPRB
ZKB=0.65_JPRB
ZBETA0=5.91_JPRB

ZRR=2._JPRB/3._JPRB

INMAXN=JPNMAXN
INMAXX=JPNMAXX

LLC1=(PSTRET == 1.5_JPRB)
LLC2=(PSTRET == 2.0_JPRB)
LLC3=(PSTRET == 2.5_JPRB)
LLC4=(PSTRET == 3.0_JPRB)
LLC5=(PSTRET == 3.5_JPRB)
LLC6=(PSTRET == 4.0_JPRB)
LLC12=(PSTRET > 1.5_JPRB).AND.(PSTRET < 2.0_JPRB)
LLC23=(PSTRET > 2.0_JPRB).AND.(PSTRET < 2.5_JPRB)
LLC34=(PSTRET > 2.5_JPRB).AND.(PSTRET < 3.0_JPRB)
LLC45=(PSTRET > 3.0_JPRB).AND.(PSTRET < 3.5_JPRB)
LLC56=(PSTRET > 3.5_JPRB).AND.(PSTRET < 4.0_JPRB)

LLCH=LLC1.OR.LLC2.OR.LLC3.OR.LLC4.OR.LLC5.OR.LLC6
LLCNH=LLC12.OR.LLC23.OR.LLC34.OR.LLC45.OR.LLC56

LLCM1=(PSTRET < 1.5_JPRB)
LLCP6=(PSTRET > 4.0_JPRB)

! ------------------------------------------------------------------

!*  1. CASE NO STRETCHING.
!   ----------------------

IF (PSTRET == 1.0_JPRB) THEN
  KNCMAX=KNMAX
ENDIF

! ------------------------------------------------------------------

!*  2. CASE OF STRETCHING. FIRST GUESS FOR Nc.
!   ------------------------------------------

IF (PSTRET > 1.0_JPRB) THEN

  INMAXXX=JPNMAXXX

  CALL SUADMI(INMAXXX,LLADMIS,INMADMI)

! * computation of ratio ZRAP. 
  ZALPHA=ZKA/(PSTRET-1.0_JPRB)**ZRA + ZALPHA0
  ZBETA=ZKB/(PSTRET-1.0_JPRB)**ZRB + ZBETA0
  ZA2=ZALPHA*(REAL(KNMAX,JPRB)**ZRR)+ZBETA
  ZRAP=1.0_JPRB+LOG(PSEUIL)/ZA2

! * computation of admissible truncation KNCMAX.
  ZNCMAX=REAL(KNMAX,JPRB)*PSTRET*ZRAP
  INCMAX=INT(ZNCMAX)+1
  KNCMAX=INMADMI(INCMAX,2)

ENDIF

! ------------------------------------------------------------------

!*  3. CASE OF STRETCHING. CORRECTION.
!   ----------------------------------

IF (PSTRET > 1.0_JPRB) THEN

! * Computes Nc for a threshold of ZSEUIL13

  ZSEUIL13=1.E-13_JPRB
  IF (PSEUIL /= ZSEUIL13) THEN
    ZRAP13=1.0_JPRB+LOG(ZSEUIL13)/ZA2
    ZNCMAX13=REAL(KNMAX,JPRB)*PSTRET*ZRAP13
    INCMAX13=INT(ZNCMAX13)+1
    INCMAX13=INMADMI(INCMAX13,2)
  ELSE
    INCMAX13=KNCMAX 
  ENDIF

! * Corrects this value:

  CALL SUNCET13(INMAXN,INMAXX,INCMAXET13)

  IF (LLADMIS(KNMAX).AND.(KNMAX>=INMAXN).AND.(KNMAX<=INMAXX)) THEN
    IF (LLCH) THEN
      IS1=NINT(2.0_JPRB*PSTRET)-2
      INCMAX13=INCMAXET13(KNMAX,IS1)
    ELSEIF (LLCNH) THEN
      IS1=INT(2.0_JPRB*PSTRET)-2
      INCMAX13=MAX(MIN(INCMAX13,INCMAXET13(KNMAX,IS1+1)),INCMAXET13(KNMAX,IS1))
    ELSEIF (LLCM1) THEN
      IS1=0
      INCMAX13=MIN(INCMAX13,INCMAXET13(KNMAX,IS1+1))
    ELSEIF (LLCP6) THEN
      IS1=6
      INCMAX13=MAX(INCMAX13,INCMAXET13(KNMAX,IS1))
    ENDIF
  ELSEIF((.NOT.LLADMIS(KNMAX)).AND.(KNMAX>=INMAXN).AND.(KNMAX<=INMAXX)) THEN
    ZA2I=ZALPHA*(REAL(INMADMI(KNMAX,1),JPRB)**ZRR)+ZBETA
    ZRAPI13=1.0_JPRB+LOG(ZSEUIL13)/ZA2I
    ZNCMAXI13=REAL(INMADMI(KNMAX,1),JPRB)*PSTRET*ZRAPI13
    INCMAXI13=INT(ZNCMAXI13)+1
    INCMAXI13=INMADMI(INCMAXI13,2)
    ZA2S=ZALPHA*(REAL(INMADMI(KNMAX,2),JPRB)**ZRR)+ZBETA
    ZRAPS13=1.0_JPRB+LOG(ZSEUIL13)/ZA2S
    ZNCMAXS13=REAL(INMADMI(KNMAX,2),JPRB)*PSTRET*ZRAPS13
    INCMAXS13=INT(ZNCMAXS13)+1
    INCMAXS13=INMADMI(INCMAXS13,2)
    IF (LLCH) THEN
      IS1=NINT(2.0_JPRB*PSTRET)-2
      INCMAXI13=INCMAXET13(INMADMI(KNMAX,1),IS1)
      INCMAXS13=INCMAXET13(INMADMI(KNMAX,2),IS1)
    ELSEIF (LLCNH) THEN
      IS1=INT(2.0_JPRB*PSTRET)-2
      INCMAXI13=MAX(MIN(INCMAXI13,INCMAXET13(INMADMI(KNMAX,1),IS1+1)),&
       & INCMAXET13(INMADMI(KNMAX,1),IS1))  
      INCMAXS13=MAX(MIN(INCMAXS13,INCMAXET13(INMADMI(KNMAX,2),IS1+1)),&
       & INCMAXET13(INMADMI(KNMAX,2),IS1))  
    ELSEIF (LLCM1) THEN
      IS1=0
      INCMAXI13=MIN(INCMAXI13,INCMAXET13(INMADMI(KNMAX,1),IS1+1))
      INCMAXS13=MIN(INCMAXS13,INCMAXET13(INMADMI(KNMAX,2),IS1+1))
    ELSEIF (LLCP6) THEN
      IS1=6
      INCMAXI13=MAX(INCMAXI13,INCMAXET13(INMADMI(KNMAX,1),IS1))
      INCMAXS13=MAX(INCMAXS13,INCMAXET13(INMADMI(KNMAX,2),IS1))
    ENDIF
    INCMAX13=MAX(MIN(INCMAX13,INCMAXS13),INCMAXI13)
  ELSEIF(KNMAX<INMAXN) THEN
    ZA2S=ZALPHA*(REAL(INMAXN,JPRB)**ZRR)+ZBETA
    ZRAPS13=1.0_JPRB+LOG(ZSEUIL13)/ZA2S
    ZNCMAXS13=REAL(INMAXN,JPRB)*PSTRET*ZRAPS13
    INCMAXS13=INT(ZNCMAXS13)+1
    INCMAXS13=INMADMI(INCMAXS13,2)
    IF (LLCH) THEN
      IS1=NINT(2.0_JPRB*PSTRET)-2
      INCMAXS13=INCMAXET13(INMAXN,IS1)
    ELSEIF (LLCNH) THEN
      IS1=INT(2.0_JPRB*PSTRET)-2
      INCMAXS13=MAX(MIN(INCMAXS13,INCMAXET13(INMAXN,IS1+1)),&
       & INCMAXET13(INMAXN,IS1))  
    ELSEIF (LLCM1) THEN
      IS1=0
      INCMAXS13=MIN(INCMAXS13,INCMAXET13(INMAXN,IS1+1))
    ELSEIF (LLCP6) THEN
      IS1=6
      INCMAXS13=MAX(INCMAXS13,INCMAXET13(INMAXN,IS1))
    ENDIF
    INCMAX13=MIN(INCMAX13,INCMAXS13)
  ELSE
    ZA2I=ZALPHA*(REAL(INMAXX,JPRB)**ZRR)+ZBETA
    ZRAPI13=1.0_JPRB+LOG(ZSEUIL13)/ZA2I
    ZNCMAXI13=REAL(INMAXX,JPRB)*PSTRET*ZRAPI13
    INCMAXI13=INT(ZNCMAXI13)+1
    INCMAXI13=INMADMI(INCMAXI13,2)
    IF (LLCH) THEN
      IS1=NINT(2.0_JPRB*PSTRET)-2
      INCMAXI13=INCMAXET13(INMAXX,IS1)
    ELSEIF (LLCNH) THEN
      IS1=INT(2.0_JPRB*PSTRET)-2
      INCMAXI13=MAX(MIN(INCMAXI13,INCMAXET13(INMAXX,IS1+1)),&
       & INCMAXET13(INMAXX,IS1))  
    ELSEIF (LLCM1) THEN
      IS1=0
      INCMAXI13=MIN(INCMAXI13,INCMAXET13(INMAXX,IS1+1))
    ELSEIF (LLCP6) THEN
      IS1=6
      INCMAXI13=MAX(INCMAXI13,INCMAXET13(INMAXX,IS1))
    ENDIF
    INCMAX13=MAX(INCMAX13,INCMAXI13)
  ENDIF

! * Final value of KNCMAX:

  IF (PSEUIL <= ZSEUIL13) THEN
    KNCMAX=MIN(KNCMAX,INCMAX13)
  ENDIF
  IF (PSEUIL >= ZSEUIL13) THEN
    KNCMAX=MAX(KNCMAX,INCMAX13)
  ENDIF

ENDIF

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUNCMAX',1,ZHOOK_HANDLE)
END SUBROUTINE SUNCMAX
