SUBROUTINE SUNHBMAT

!**** *SUNHBMAT*  Initialize vertical structure matrix for NH model.

!     Purpose.
!     --------
!      Initialize vertical structure matrix BB used for semi-implicit
!      scheme in the NH model.
!      This matrix is stored in the array SIB.

!      BB = C**2
!       * [ I - beta**2 (Delta t)**2 C**2 (SITR/SITRA) (LLstar/H**2) ]**(-1)
!       * [ I + beta**2 (Delta t)**2 N**2 (SITR/SITRA) TTstar ]
!      where:
!       C = sqrt( Rd * SITR * ( cpdry/cvdry ) )
!       H = Rd * SITR / g
!       N = g / sqrt( cpdry * SITR )
!       LLstar is a tridiagonal "Laplacian" operator.
!       TTstar is a tridiagonal operator equal to (I + LLstar * QQstar)
!        (QQstar is the diagonal matrix "deltastar - 2*alphastar")

!      The intermediate factor:
!       [ I - beta**2 (Delta t)**2 C**2 (SITR/SITRA) (LLstar/H**2) ]
!       is stored in SIFAC.

!      The intermediate factor:
!       [ I - beta**2 (Delta t)**2 C**2 (SITR/SITRA) (LLstar/H**2) ]**(-1)
!       is stored in SIFACI. It is required in the RHS of the Helmholtz
!       equation.

!      The LHS of the Helmholtz equation writes:
!       (I - beta**2 (Delta t)**2 BB Mbabar2 vnablaprim**2) svd
!      where
!       Mbabar2 = c**2 if LSIDG=F
!       Mbabar2 = M**2 if LSIDG=T (in this case a pentadiagonal operator
!        must be computed in the routine SUNHHEG).
!       "svd" is the vertical divergence variable.

!      Replace "beta" by "(1+epsilon_uncentering) * beta"
!       in the previous formulae
!       when there is uncentering (in the SL scheme).

!     Remark: the upper boundary condition is well specified at least
!      for models with "prehyd_top = 0" but it is not sure that it
!      is well specied if "prehyd_top > 0" (more generally it is
!      not currently desirable to run the NH model with "prehyd_top > 0").
!      At least for prehyd_top = 0, QQstar(1,1) must be set to zero.

!**   Interface.
!     ----------
!        *CALL* *SUNHBMAT

!     Explicit arguments :  None
!     --------------------

!     Implicit arguments :
!     --------------------
!        Matrix SIB, SIFAC, SIFACI in YOMDYN

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. YESSAD (after routines SUBMAT and SUEHEL).

!     Modifications.
!     --------------
!   Original : 09-12-2004
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

USE YOMDIM   , ONLY : NFLEVG   
USE YOMDYN   , ONLY : SIALPH   ,SILNPR   ,SIB      ,SIFAC    ,SIFACI   ,&
 & BETADT   ,SITR    ,SITRA    ,SITIME   ,VESL     ,XIDT
USE YOMCT0   , ONLY : LSLAG    ,LTWOTL   
USE YOMCST   , ONLY : RG       ,RD       ,RCPD     ,RCVD
USE YOMGEM   , ONLY : VAH      ,VBH      ,VP00     ,TOPPRES

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JL1, JL2, IFIRST

REAL(KIND=JPRB) :: ZBETADT, ZBT, ZBT2
REAL(KIND=JPRB) :: ZCOEF, ZDET, ZEPS
REAL(KIND=JPRB) :: ZC2, ZH, ZH2, ZN2
REAL(KIND=JPRB) :: ZD(NFLEVG,NFLEVG)
REAL(KIND=JPRB) :: ZMO(NFLEVG,NFLEVG)
REAL(KIND=JPRB) :: ZSIB(NFLEVG,NFLEVG)
REAL(KIND=JPRB) :: ZSIBIN(NFLEVG,NFLEVG)
REAL(KIND=JPRB) :: ZSIBOUT(NFLEVG,NFLEVG)
REAL(KIND=JPRB) :: ZWORK(2*NFLEVG)
REAL(KIND=JPRB) :: ZDIAG_AVG

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "mxmaop.h"
END INTERFACE

#include "siseve.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUNHBMAT',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    INITIALIZE.
!              -----------

! * ZD contains the identity matrix "I":
DO JL1=1,NFLEVG
  DO JL2=1,NFLEVG
    ZD(JL2,JL1)=1.0_JPRB-MIN(ABS(REAL(JL2-JL1,JPRB)),1.0_JPRB)
  ENDDO
ENDDO

! * Compute the constants C**2, H**2 and N**2:
ZC2=RD*SITR*RCPD/RCVD
ZH=RD*SITR/RG
ZH2=ZH*ZH
ZN2=(RG*RG)/(RCPD*SITR)

! * Compute ZBT2 = (1+epsilon_uncentering)**2 * BETADT**2 * TSTEP**2:
IF (LSLAG) THEN
  IF (LTWOTL) THEN
    ZBETADT=(1.0_JPRB+VESL)*(1.0_JPRB+XIDT)*BETADT
  ELSE
    ZBETADT=(1.0_JPRB+VESL)*BETADT
  ENDIF
ELSE
  ZBETADT=BETADT
ENDIF
ZBT=ZBETADT*SITIME*0.5_JPRB
ZBT2=ZBT*ZBT

! * In the case where prehyd_top=0, a particular treatment
!   is applied to some quantities at the first full level;
!   for ex QQstar(1,1)=0
!   instead of
!   QQstar(1,1)=deltastar(1)-2*alphastar(1)
IF(VAH(0)+VBH(0)*VP00 <= TOPPRES)THEN
  IFIRST=2
ELSE
  IFIRST=1
ENDIF

!      ---------------------------------------------------------------

!*       2.    COMPUTES THE MATRIX SIFAC.
!              --------------------------

! * Apply the tridiagonal "Laplacian" operator LLstar to ZD.
CALL SISEVE(1,NFLEVG,ZD,SIFAC,NFLEVG)

! * Multiply by
! "-(1+epsilon_uncentering)**2 beta**2 (Delta t)**2 C**2 (SITR/SITRA) (1/H**2)"
! then add the identity matrix:
ZCOEF=ZBT2*ZC2*SITR/(SITRA*ZH2)
DO JL1=1,NFLEVG
  DO JL2=1,NFLEVG
    SIFAC(JL2,JL1)=ZD(JL2,JL1)-ZCOEF*SIFAC(JL2,JL1)
  ENDDO
ENDDO

!      ---------------------------------------------------------------

!*       3.    INVERTS THE MATRIX SIFAC.
!              --------------------------

DO JL1=1,NFLEVG
  DO JL2=1,NFLEVG
    ZMO(JL1,JL2)=SIFAC(JL1,JL2)
  ENDDO
ENDDO


!* for more than 83 levels it is not possible to pass through MINV 
!  and therefore matrix has to be rescaled before inversion
!  and than set back. We use average absolute value on diagonal
!  ZDIAG_AVG to rescale matrix
ZDIAG_AVG=0.0_JPRB
DO JL1=1,NFLEVG
  ZDIAG_AVG = ZDIAG_AVG + ABS(ZMO(JL1,JL1))
ENDDO
ZDIAG_AVG = ZDIAG_AVG/REAL(NFLEVG,JPRB)
! rescaling
IF( ZDIAG_AVG > 0.0_JPRB )THEN
  DO JL1=1,NFLEVG
    DO JL2=1,NFLEVG
      ZMO(JL1,JL2)=ZMO(JL1,JL2)/ZDIAG_AVG
    ENDDO
  ENDDO
ENDIF

ZEPS=100.0_JPRB*TINY(1.0_JPRB)
CALL MINV(ZMO,NFLEVG,NFLEVG,ZWORK,ZDET,ZEPS,0,1)

! rescaling back to original values
IF( ZDIAG_AVG > 0.0_JPRB )THEN
  DO JL1=1,NFLEVG
    DO JL2=1,NFLEVG
      ZMO(JL1,JL2)=ZMO(JL1,JL2)/ZDIAG_AVG
    ENDDO
  ENDDO
ENDIF


DO JL1=1,NFLEVG
  DO JL2=1,NFLEVG
    SIFACI(JL1,JL2)=ZMO(JL1,JL2)
  ENDDO
ENDDO

!      ---------------------------------------------------------------

!*       4.    COMPUTES THE MATRIX SIB.
!              ------------------------

! * Apply the diagonal operator QQstar:
ZSIBIN(1:NFLEVG,1:NFLEVG)=0.0_JPRB
DO JL1=IFIRST,NFLEVG
  ZSIBIN(JL1,JL1)=SILNPR(JL1)-2.0_JPRB*SIALPH(JL1)
ENDDO

! * Apply the tridiagonal operator LLstar:
CALL SISEVE(1,NFLEVG,ZSIBIN,ZSIBOUT,NFLEVG)

! * Add the identity matrix:
DO JL1=1,NFLEVG
  ZSIBOUT(JL1,JL1)=1.0_JPRB+ZSIBOUT(JL1,JL1)
ENDDO

! * Multiply by
! "(1+epsilon_uncentering)**2 beta**2 (Delta t)**2 N**2 (SITR/SITRA)",
! then adds the identity matrix,
! then multiply by C**2:
ZCOEF=ZBT2*ZN2*SITR/SITRA
DO JL1=1,NFLEVG
  DO JL2=1,NFLEVG
    ZSIBOUT(JL2,JL1)=ZC2*(ZD(JL2,JL1)+ZCOEF*ZSIBOUT(JL2,JL1))
  ENDDO
ENDDO

! * Multiply by SIFACI:
CALL MXMAOP(SIFACI,1,NFLEVG,ZSIBOUT,1,NFLEVG,ZSIB,1,NFLEVG,&
 & NFLEVG,NFLEVG,NFLEVG)

!      ----------------------------------------------------------------

!*       5.    STORAGE OF MATRIX.
!              ------------------

DO JL1=1,NFLEVG
  DO JL2=1,NFLEVG
    SIB(JL2,JL1)=ZSIB(JL2,JL1)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUNHBMAT',1,ZHOOK_HANDLE)
END SUBROUTINE SUNHBMAT
