SUBROUTINE SUNHHEG

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NSMAX    ,NSPEC    ,NUMP
USE YOMCT0   , ONLY : LSLAG    ,LTWOTL
USE YOMDYN   , ONLY : SCGMAP   ,SIVP     ,SIHEG    ,SIHEG2   ,&
 & SIHEGB   ,SIHEGB2  ,SITR     ,&
 & BETADT   ,SITIME   ,VESL     ,XIDT     ,LSIDG
USE YOMLAP   , ONLY : NSE0L    ,RLAPDI   ,RLAPIN   ,MYMS
USE YOMLUN   , ONLY : NULOUT
USE YOMCST   , ONLY : RD       ,RCPD     ,RCVD

!**** *SUNHHEG*   - Initialize the solver of the Helmholz equation
!                   for NH model.

!     Purpose.    Initialize the solver of the Helmholz equation
!     --------    in the semi-implicit, in case of stretching and
!                 use of geographical divergence (LSIDG=.T.).
!                 For NH model.
!                 This code is valid only for a triangular truncation.
!                 (the condition "n is in [m,NSMAX]" is explicitly used).

!**   Interface.
!     ----------
!        *CALL* *SUNHHEG

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!          Called by SUNHSI.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        K. YESSAD (DEC 2005)

!     Modifications.
!     --------------
!   K. Yessad 15-May-2006: memory optimisations for stretched geometry
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB) :: ZD(NFLEVG,0:NSMAX)
REAL(KIND=JPRB) :: ZE(NFLEVG,0:NSMAX)
REAL(KIND=JPRB) :: ZF(NFLEVG,0:NSMAX)
REAL(KIND=JPRB) :: ZES(NFLEVG,0:NSMAX)
REAL(KIND=JPRB) :: ZFS(NFLEVG,0:NSMAX)

REAL(KIND=JPRB) :: ZCDEU,ZMUL

INTEGER(KIND=JPIM) :: ILUX, IS0, ISE, JL, JN, JSE, IM, JMLOC

REAL(KIND=JPRB) :: ZBETADT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "suher.h"
#include "suhes.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUNHHEG',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

! Contrary to the hydrostatic case (SUHEG), the test on SITIME
! and the resetting of SITIME is already done in SUNHSI, so it has not
! to be redone in SUNHHEG.

WRITE(NULOUT,*) ' SUNHHEG: COMPUTES HELMHOLTZ EQUATION OPERATOR'&
 & ,' IN CASE OF SEMI-IMPLICIT SCHEME IN NOT'&
 & ,' REDUCED DIVERGENCE (LSIDG=.T.): NH model.'  

!     ------------------------------------------------------------------

!*       1.    INITIALIZE HELMHOLTZ EQUATION SOLVER.
!              -------------------------------------

!        1.1 Initial settings.

ILUX=NSPEC

IF (LSLAG) THEN
  IF (LTWOTL) THEN
    ZBETADT=(1.0_JPRB+VESL)*(1.0_JPRB+XIDT)*BETADT
  ELSE
    ZBETADT=(1.0_JPRB+VESL)*BETADT
  ENDIF
ELSE
  ZBETADT=BETADT
ENDIF

DO JL=1,NFLEVG
  DO JSE=1,ILUX
    SIHEG(JL,JSE,1)=0.0_JPRB
    SIHEG(JL,JSE,2)=0.0_JPRB
    SIHEG(JL,JSE,3)=0.0_JPRB
  ENDDO
ENDDO

DO JMLOC=1,NUMP

  IM=MYMS(JMLOC)

  IF (IM == 0) THEN

    !        1.2 Case m=0.

    ! Calculation of I - sivp M**2 grad'**2 zbetadt**2 Dt**2.
    ! Remark: the only difference with what is coded in SUHEG is the fact
    ! that the matricial product is now M**2 grad'**2 and not grad'**2 M**2.
    ! These two matricial operators do not commute but M**2 grad'**2
    ! is simply the transposition of grad'**2 M**2: compared to SUHEG,
    ! ZD remains unchanged, ZE and ZES are exchanged, ZF and ZFS are exchanged.

    DO JL=1,NFLEVG
      DO JSE=1,NSMAX+1
        SIHEG2(JL,JSE,2)=0.0_JPRB
        SIHEG2(JL,JSE,3)=0.0_JPRB
      ENDDO
    ENDDO

    IS0=NSE0L(JMLOC)
    DO JL=1,NFLEVG
      DO JN=0,NSMAX
        ISE=IS0+JN+1
        ZD(JL,JN)=1.0_JPRB &
         & -SIVP(JL)*RLAPDI(JN)*SCGMAP(ISE,1)*(ZBETADT*SITIME*0.5_JPRB)**2  
      ENDDO
      DO JN=0,NSMAX-1
        ISE=IS0+JN+1
        ZES(JL,JN)=-SIVP(JL)*RLAPDI(JN+1)*SCGMAP(ISE,2)&
         & *(ZBETADT*SITIME*0.5_JPRB)**2  
        ZE(JL,JN)=-SIVP(JL)*RLAPDI(JN)*SCGMAP(ISE,2)&
         & *(ZBETADT*SITIME*0.5_JPRB)**2  
      ENDDO
      DO JN=0,NSMAX-2
        ISE=IS0+JN+1
        ZFS(JL,JN)=-SIVP(JL)*RLAPDI(JN+2)*SCGMAP(ISE,3)&
         & *(ZBETADT*SITIME*0.5_JPRB)**2  
        ZF(JL,JN)=-SIVP(JL)*RLAPDI(JN)*SCGMAP(ISE,3)&
         & *(ZBETADT*SITIME*0.5_JPRB)**2  
      ENDDO
    ENDDO
    CALL SUHER(NSMAX+1,NFLEVG,NFLEVG,ZD(1,0),ZE(1,0),ZES(1,0),&
     & ZF(1,0),ZFS(1,0),SIHEG(1,IS0+1,1),SIHEG(1,IS0+1,2),&
     & SIHEG(1,IS0+1,3),SIHEG2(1,1,2),SIHEG2(1,1,3))  

  ELSE

    !        1.3 Case m>0 (code identical to the corresponding one in SUHEG).

    ! Calculation of grad'**(-2) - C**2 M**2 zbetadt**2 Dt**2.

    IS0=NSE0L(JMLOC)
    DO JL=1,NFLEVG
      DO JN=IM,NSMAX
        ISE=IS0+JN+1-IM
        ZD(JL,JN)=RLAPIN(JN)&
         & -SIVP(JL)*SCGMAP(ISE,1)*(ZBETADT*SITIME*0.5_JPRB)**2  
        ZE(JL,JN)=-SIVP(JL)*SCGMAP(ISE,2)*(ZBETADT*SITIME*0.5_JPRB)**2
        ZF(JL,JN)=-SIVP(JL)*SCGMAP(ISE,3)*(ZBETADT*SITIME*0.5_JPRB)**2
      ENDDO
    ENDDO
    CALL SUHES(NSMAX+1-IM,NFLEVG,NFLEVG,ZD(1,IM),ZE(1,IM),ZF(1,IM),&
     & SIHEG(1,IS0+1,1),SIHEG(1,IS0+1,2),SIHEG(1,IS0+1,3))  

  ENDIF

ENDDO

!     ------------------------------------------------------------------

!*       2.    TREATMENT OF "I - zbetadt**2 Dt**2 C**2 grad'**2 M**2".
!              -------------------------------------------------------

!        2.1 Initial settings.

! * constant C**2 in ZCDEU:
ZCDEU=RD*SITR*(RCPD/RCVD)
ZMUL=ZCDEU*(ZBETADT*SITIME*0.5_JPRB)**2

DO JL=1,NFLEVG
  DO JSE=1,ILUX
    SIHEGB(JL,JSE,1)=0.0_JPRB
    SIHEGB(JL,JSE,2)=0.0_JPRB
    SIHEGB(JL,JSE,3)=0.0_JPRB
  ENDDO
ENDDO

DO JMLOC=1,NUMP

  IM=MYMS(JMLOC)

  IF (IM == 0) THEN

    !        2.2 Case m=0.

    ! Calculation of I - C**2 grad'**2 M**2 zbetadt**2 Dt**2.

    DO JL=1,NFLEVG
      DO JSE=1,NSMAX+1
        SIHEGB2(JL,JSE,2)=0.0_JPRB
        SIHEGB2(JL,JSE,3)=0.0_JPRB
      ENDDO
    ENDDO

    IS0=NSE0L(JMLOC)
    DO JL=1,NFLEVG
      DO JN=0,NSMAX
        ISE=IS0+JN+1
        ZD(JL,JN)=1.0_JPRB-RLAPDI(JN)*SCGMAP(ISE,1)*ZMUL
      ENDDO
      DO JN=0,NSMAX-1
        ISE=IS0+JN+1
        ZE(JL,JN)=-RLAPDI(JN+1)*SCGMAP(ISE,2)*ZMUL
        ZES(JL,JN)=-RLAPDI(JN)*SCGMAP(ISE,2)*ZMUL
      ENDDO
      DO JN=0,NSMAX-2
        ISE=IS0+JN+1
        ZF(JL,JN)=-RLAPDI(JN+2)*SCGMAP(ISE,3)*ZMUL
        ZFS(JL,JN)=-RLAPDI(JN)*SCGMAP(ISE,3)*ZMUL
      ENDDO
    ENDDO
    CALL SUHER(NSMAX+1,NFLEVG,NFLEVG,ZD(1,0),ZE(1,0),ZES(1,0),&
     & ZF(1,0),ZFS(1,0),SIHEGB(1,IS0+1,1),SIHEGB(1,IS0+1,2),&
     & SIHEGB(1,IS0+1,3),SIHEGB2(1,1,2),SIHEGB2(1,1,3))  

  ELSE

    !        2.3 Case m>0 (code identical to the corresponding one in SUHEG).

    ! Calculation of grad'**(-2) - C**2 M**2 zbetadt**2 Dt**2.

    IS0=NSE0L(JMLOC)
    DO JL=1,NFLEVG
      DO JN=IM,NSMAX
        ISE=IS0+JN+1-IM
        ZD(JL,JN)=RLAPIN(JN)-SCGMAP(ISE,1)*ZMUL
        ZE(JL,JN)=-SCGMAP(ISE,2)*ZMUL
        ZF(JL,JN)=-SCGMAP(ISE,3)*ZMUL
      ENDDO
    ENDDO
    CALL SUHES(NSMAX+1-IM,NFLEVG,NFLEVG,ZD(1,IM),ZE(1,IM),ZF(1,IM),&
     & SIHEGB(1,IS0+1,1),SIHEGB(1,IS0+1,2),SIHEGB(1,IS0+1,3))  

  ENDIF

ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUNHHEG',1,ZHOOK_HANDLE)
END SUBROUTINE SUNHHEG
