SUBROUTINE SUOROG(PSPOR)

!**** *SUOROG*  - Routine to initialize the grid-point orography fields.

!     Purpose.
!     --------
!           Initialize the grid-point orography fields of the model.

!**   Interface.
!     ----------
!        *CALL* *SUOROG(.....)*

!        Explicit arguments :
!        --------------------
!          INPUT:
!            PSPOR   - spectral orography field

!        Implicit arguments :
!        --------------------
!        The grid-point fields of the model.
!        The boundary condition fields of the model.

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        SPEREESM/SPEREEDM
!        REESPESM/REESPEDM
!        SPEUVSM/SPEUVDM
!        EXTPOLB if shared memory.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        David Dent *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-05-19  following code in subroutine SUSPEC
!        Modified : 93-04-13  L.Labbe (Switch for trajectory or current
!                                      value of grid point orography)
!        Modified : 94-05-27  R. El Khatib (Initialize extensions)
!        Modified : 95-01-26  R. Bubnova   (Second order derivatives)
!        Modified : 95-06-06  L. Isaksen (Reordering of spectral arrays)
!        Modified : 98-02-11  R. El Khatib (Case LELAM)
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option + cleanings.
!         - removal of LRPOLE in YOMCT0.
!         - removal of LLPOLE and code under LRPOLE, LLPOLE.
!         - removal of LLPOLE in 'CALL SPEREESM/DM', 'CALL SPEUVSM/DM'.
!         - correct extra-polar calculations (previously incorrect and
!           not valid for any value of NDGSUR) if SM mode, at least to
!           avoid the use of false or uninitialised values in NGPTOT-do loops
!           memory transfers (in practical the extrapolar values of OROG..
!           arrays are currently not used elsewhere in the code, at least 
!           in cycle 19).
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NSPEC2
USE YOMGEM   , ONLY : NGPTOT
USE YOMCT0   , ONLY : LNHDYN   ,LELAM
USE YOMGC    , ONLY : OROG     ,OROGL    ,OROGM    ,OROGLL   ,&
 & OROGMM   ,OROGLM  

IMPLICIT NONE

! * INPUT:
REAL(KIND=JPRB)   ,INTENT(INOUT)    :: PSPOR(NSPEC2) 
REAL(KIND=JPRB), ALLOCATABLE :: ZREEL(:), ZU(:), ZV(:), ZDIV(:), ZSWORK(:)

INTEGER(KIND=JPIM) :: JROF, JSE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "reespe.intfb.h"
#include "speree.intfb.h"
#include "speuv.intfb.h"
#include "sueorog.intfb.h"

!     ------------------------------------------------------------------

!*       1.    INITIALIZE GRIDPOINT OROGRAPHY.
!              -------------------------------

!*       1.1   PLANE GEOMETRY.

IF (LHOOK) CALL DR_HOOK('SUOROG',0,ZHOOK_HANDLE)
IF (LELAM) THEN
  CALL SUEOROG(PSPOR)
ELSE

!*       1.2   SPHERICAL GEOMETRY.

!*       1.2.1   ALLOCATIONS.

  ALLOCATE(ZREEL(NGPTOT))
  ALLOCATE(ZU(NGPTOT))
  ALLOCATE(ZV(NGPTOT))
  ALLOCATE(ZDIV(NSPEC2))
  ALLOCATE(ZSWORK(NSPEC2))

!*       1.2.2   INITIALIZE GRIDPOINT OROGRAPHY

  CALL SPEREE(1,1,PSPOR,ZREEL)
  DO JROF=1,NGPTOT
    OROG(JROF) = ZREEL(JROF)
  ENDDO

!*       1.2.3   COMPUTE FIRST ORDER DERIVATIVES OF OROGRAPHY.

  DO JSE=1,NSPEC2
    ZDIV(JSE)=0.0_JPRB
  ENDDO

  CALL SPEUV(ZDIV,PSPOR,ZU,ZV,1,1,2)

  DO JROF =1,NGPTOT
    OROGL(JROF) = ZU(JROF)
    OROGM(JROF) = ZV(JROF)
  ENDDO

!*       1.2.4   COMPUTE SOME SECOND ORDER DERIVATIVES OF OROGRAPHY FOR NH.

  IF(LNHDYN) THEN

!       * orogll, oroglm.
    CALL REESPE(1,1,ZSWORK,ZU)
    CALL SPEUV(ZDIV,ZSWORK,ZU,ZREEL,1,1,2)

    DO JROF =1,NGPTOT
      OROGLL(JROF) = ZU(JROF)
      OROGLM(JROF) = ZREEL(JROF)
    ENDDO

!       * orogmm:
    CALL REESPE(1,1,ZSWORK,ZV)
    CALL SPEUV(ZDIV,ZSWORK,ZU,ZV,1,1,2)
    DO JROF =1,NGPTOT
      OROGMM(JROF) = ZV(JROF)
    ENDDO
  ENDIF

!*       1.2.5   DEALLOCATIONS.

  DEALLOCATE(ZREEL)
  DEALLOCATE(ZU)
  DEALLOCATE(ZV)
  DEALLOCATE(ZDIV)
  DEALLOCATE(ZSWORK)
ENDIF
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUOROG',1,ZHOOK_HANDLE)
END SUBROUTINE SUOROG

