SUBROUTINE SUPONG

! SUPONG - initialize sponge

! Purpose
! -------
!   Initialize YOMPONG, module that controls sponge for vertical plane 
!   2D model.

! Interface
! ---------
!   none 

! Method
! ------
!   Sponge is used in 2D academical experiments as absorbing
!   layer to prevent wave reflections from the top of domain.
!   Use in 3D experiments is restricted by the fact that in
!   current implementation background state is horizontaly
!   homogeneous.

!   Sponge determines background profiles using mean values
!   on the levels, taken from the initial state. Heigths of levels
!   are computed using these average profiles. Sponged quantity
!   w is correctly diagnosed for variables d3, d4.

!   Technical limitations of the sponge:

!     - one processor (needed for correct sponge setup)

!     - time constant LBC (background profiles do not evolve in time)

!     - NH model: 2D experiment with NPROMA = NDGL
!       (needed by spectral transforms called below level 
!       of CPG to compute grad(vd))

! Externals
! ---------

! Reference
! ---------

! Author
! ------
!   13-Feb-1996 R. Bubnova

! Modifications
! -------------
!   11-Oct-2002 J. Masek   addition of the new sponge, comments, cleaning
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   08-Jun-2004 J. Masek   NH cleaning (NPDVAR), removal of old sponge
!   25-Nov-2004 Y. Seity   Add new variables for GPRCP
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
! End Modifications
!------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMCST   , ONLY : RPI      ,RG
USE YOMCT0   , ONLY : LNHDYN   ,LTWOTL   ,NPROC
USE YOMDIM   , ONLY : NPROMA   ,NDGLG    ,NFLEVG   ,NMSMAX   ,LSPT
USE YOMDYNA  , ONLY : NPDVAR
USE YOMDYN   , ONLY : TSTEP
USE YOMGEM   , ONLY : VAH      ,VBH      ,VDELB    ,VC
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMLUN   , ONLY : NULNAM   ,NULOUT
USE YOMPONG  , ONLY : NSPONGE  ,REPONBT  ,REPONTP  ,REPONTAU ,REPONGF  , &
 &                    REPONGH  ,RT_SPNG  ,RU_SPNG  ,RV_SPNG
USE YOMSP    , ONLY : SPOR     ,SPSP     ,SPT      ,SPQ      ,SPL      , &
 &                    SPI      ,SPSPD    ,SPUB     ,SPVB  
USE YOM_YGFL , ONLY : YQ       ,YL       ,YI

! -----------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JLEV

REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZARG, ZPIS, ZTSTEP

REAL(KIND=JPRB) :: ZQ     (NFLEVG)  ! specific humidity q
REAL(KIND=JPRB) :: ZL     (NFLEVG)  ! specific mass of liquid water qw
REAL(KIND=JPRB) :: ZI     (NFLEVG)  ! specific mass of ice qi
REAL(KIND=JPRB) :: ZRR     (NFLEVG)  ! specific mass of rain qr
REAL(KIND=JPRB) :: ZS     (NFLEVG)  ! specific mass of snow qs
REAL(KIND=JPRB) :: ZG     (NFLEVG)  ! specific mass of graupel qg
REAL(KIND=JPRB) :: ZPIF   (NFLEVG)  ! pi at full levels
REAL(KIND=JPRB) :: ZALPH  (NFLEVG)  ! alpha
REAL(KIND=JPRB) :: ZLNPR  (NFLEVG)  ! delta(pi)/pi
REAL(KIND=JPRB) :: ZDPI   (NFLEVG)  ! delta(pi)
REAL(KIND=JPRB) :: ZRDPI  (NFLEVG)  ! 1/delta(pi)
REAL(KIND=JPRB) :: ZRPIF  (NFLEVG)  ! 1/pi at full levels
REAL(KIND=JPRB) :: ZRPP   (NFLEVG)  ! 1/pi**2 at full levels
REAL(KIND=JPRB) :: ZRTGR  (NFLEVG)  ! 1/delta(pi)*(VDELB + VC/pi)
REAL(KIND=JPRB) :: ZPD    (NFLEVG)  ! NH pressure departure (depends on NPDVAR)
REAL(KIND=JPRB) :: ZPPI   (NFLEVG)  ! p/pi
REAL(KIND=JPRB) :: ZR     (NFLEVG)  ! R, resp. R*(pi/p) for NH
REAL(KIND=JPRB) :: ZCP    (NFLEVG)  ! cp
REAL(KIND=JPRB) :: ZKAPPA (NFLEVG)  ! R/cp
REAL(KIND=JPRB) :: ZALTF  (NFLEVG)  ! full level altitude / geopotential

REAL(KIND=JPRB) :: ZPIH (0:NFLEVG)  ! hydrostatic pressure pi on half levels
REAL(KIND=JPRB) :: ZALTH(0:NFLEVG)  ! half level altitude / geopotential

CHARACTER(LEN = 80) :: CLFMT

! -----------------------------------------------------------------------------

#include "nampong.h"

#include "abor1.intfb.h"
#include "gpgeo.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gpxyb.intfb.h"
#include "posnam.intfb.h"

!---------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUPONG',0,ZHOOK_HANDLE)

! -----------------------------------------------------------------------------

! -----
! set default values
! -----

IF ( LTWOTL ) THEN
  ZTSTEP = TSTEP
ELSE
  ZTSTEP = 2.0_JPRB*TSTEP
ENDIF

NSPONGE  = 0

REPONBT  = 10000.0_JPRB
REPONTP  = 20000.0_JPRB
REPONTAU =     5.0_JPRB*ZTSTEP

! -----
! read namelist
! -----

! position file pointer to NAMPONG group
CALL POSNAM(NULNAM, 'NAMPONG')

! read NAMPONG
READ(NULNAM, NAMPONG)

! write diagnostics
WRITE(NULOUT, '(A, I10)'   ) 'NSPONGE  = ', NSPONGE

IF ( NSPONGE == 0 ) THEN

  ! write diagnostics
  WRITE(NULOUT, '(A)') 'SPONGE IS OFF'

ELSEIF ( NSPONGE == 2 ) THEN

  ! write diagnostics
  WRITE(NULOUT, '(A)') 'SPONGE IS ON'
  WRITE(NULOUT, '(A, F10.2)') 'REPONTAU = ', REPONTAU
  WRITE(NULOUT, '(A, F10.2)') 'REPONTP  = ', REPONTP
  WRITE(NULOUT, '(A, F10.2)') 'REPONBT  = ', REPONBT

  ! -----
  ! allocate arrays
  ! -----

  ALLOCATE(REPONGH(0:NFLEVG))

  ALLOCATE(REPONGF(NFLEVG))
  ALLOCATE(RT_SPNG(NFLEVG))
  ALLOCATE(RU_SPNG(NFLEVG))
  ALLOCATE(RV_SPNG(NFLEVG))

  CLFMT = '(1X,''ARRAY '',A10,'' ALLOCATED '',8I8)'

  WRITE(NULOUT, CLFMT) 'REPONGH', SIZE(REPONGH), SHAPE(REPONGH)
  WRITE(NULOUT, CLFMT) 'REPONGF', SIZE(REPONGF), SHAPE(REPONGF)
  WRITE(NULOUT, CLFMT) 'RT_SPNG', SIZE(RT_SPNG), SHAPE(RT_SPNG)
  WRITE(NULOUT, CLFMT) 'RU_SPNG', SIZE(RU_SPNG), SHAPE(RU_SPNG)
  WRITE(NULOUT, CLFMT) 'RV_SPNG', SIZE(RV_SPNG), SHAPE(RV_SPNG)

  ! -----
  ! safety checks
  ! -----

  ! check number of processors
  IF ( NPROC /= 1 ) THEN
    WRITE(NULOUT, *) 'New sponge is implemented only for 1 processor!'
    CALL ABOR1('SUPONG: ABOR1 CALLED')
  ENDIF

  ! check if temperature field is available
  IF ( .NOT.LSPT ) THEN
    WRITE(NULOUT, *) 'New sponge cannot be used with LSPT = .F.!'
    CALL ABOR1('SUPONG: ABOR1 CALLED')
  ENDIF

  ! NH specific checks
  IF ( LNHDYN ) THEN

    ! check NMSMAX
    IF ( NMSMAX /= 0 ) THEN
      WRITE(NULOUT, *) &
       & 'Sponge in NH regime is not implemented for 3D model!'  
      CALL ABOR1('SUPONG: ABOR1 CALLED')
    ENDIF

    ! check NPROMA (negative value of NPROMA in namelist means that 
    ! it will not be optimized)
    IF ( NPROMA /= NDGLG ) THEN
      WRITE(NULOUT, *) &
       & 'Sponge activated in NH regime: Please set your NPROMA = -NDGL!'  
      CALL ABOR1('SUPONG: ABOR1 CALLED')
    ENDIF

  ENDIF

  ! -----
  ! get horizontal averages from init file (here we suppose that SP* arrays
  ! contain all spectral coefficients, i.e. that only 1 processor is used)
  ! -----

  ! surface pressure
  ZPIS = EXP(SPSP(1))

  DO JLEV = 1, NFLEVG

    ! temperature
    RT_SPNG(JLEV) = SPT(JLEV, 1)

    ! U-wind
    RU_SPNG(JLEV) = SPUB(JLEV)

    ! V-wind
    RV_SPNG(JLEV) = SPVB(JLEV)

    ! specific humidity
    IF ( YQ%LSP ) THEN
      ZQ(JLEV) = SPQ(JLEV, 1)
    ELSE
      ZQ(JLEV) = 0.0_JPRB
    ENDIF

    ! liquid water
    IF ( YL%LSP ) THEN
      ZL(JLEV) = SPL(JLEV, 1)
    ELSE
      ZL(JLEV) = 0.0_JPRB
    ENDIF

    ! ice
    IF ( YI%LSP ) THEN
      ZI(JLEV) = SPI(JLEV, 1)
    ELSE
      ZI(JLEV) = 0.0_JPRB
    ENDIF

    ! No sponge for rain, snow and graupel
    ! rain
    ZRR(1:NFLEVG) = 0.0_JPRB

    ! snow
    ZS(1:NFLEVG) = 0.0_JPRB

    ! graupel
    ZG(1:NFLEVG) = 0.0_JPRB


    ! NH pressure departure
    IF ( LNHDYN ) THEN
      ZPD(JLEV) = SPSPD(JLEV, 1)
    ENDIF

  ENDDO

  ! -----
  ! determine mean geopotential of levels
  ! -----

  ! compute half level pressure
  ZPIH(NFLEVG) = ZPIS
  CALL GPPREH( &
   & 1        ,1        ,1        ,NFLEVG   ,VAH      , &
   & VBH      ,ZPIH                                     &
   & )  

  ! compute auxiliary quantities
  CALL GPXYB( &
   & 1        ,1        ,1        ,NFLEVG   ,VDELB    , &
   & VC       ,ZPIH     ,ZDPI     ,ZRDPI    ,ZLNPR    , &
   & ZALPH    ,ZRTGR    ,ZRPIF    ,ZRPP                 &
   & )  

  ! compute full level pressure
  CALL GPPREF( &
   & 1        ,1        ,1        ,NFLEVG   , &
   & ZALPH    ,ZPIH     ,ZPIF                 &
   & )  

  ! compute R, cp, kappa
  CALL GPRCP( &
   & 1        ,1        ,1        ,NFLEVG   ,PQ=ZQ    , &
   & PQI=ZI , PQL=ZL , PQR=ZRR , PQS=ZS , PQG=ZG , &
   & PCP=ZCP, PR=ZR , PKAP=ZKAPPA   &
   & )  


  IF ( LNHDYN ) THEN

    ! compute p/pi
    ! -------------------------------------------------------
    ! ky: should be replaced later by a call to GNHPRE
    IF ( NPDVAR == 2 ) THEN
      DO JLEV = 1, NFLEVG
        ZPPI(JLEV) = EXP(ZPD(JLEV))
      ENDDO
    ELSEIF ( NPDVAR == 3 .OR. NPDVAR == 4 ) THEN
      CALL ABOR1('SUPONG: Phi-gw NH model not yet coded for NSPONGE=2')
    ENDIF
    ! -------------------------------------------------------

    ! compute reduced R, i.e. R*(pi/p)
    DO JLEV = 1, NFLEVG
      ZR(JLEV) = ZR(JLEV)/ZPPI(JLEV)
    ENDDO

  ENDIF

  ! -------------------------------------------------------
  ! ky: this is OK for NPDVAR=2, not for NPDVAR=3 or 4
  ! compute geopotential
  IF ( NPDVAR == 2 ) THEN
    ZALTH(NFLEVG) = SPOR(1)
    CALL GPGEO( &
     & 1        ,1        ,1        ,NFLEVG   ,ZALTH    , &
     & ZALTF    ,RT_SPNG  ,ZR       ,ZLNPR    ,ZALPH      &
     & )  

    ! convert geopotential to altitude
    ZALTH(0) = ZALTH(0)/RG
    DO JLEV = 1, NFLEVG
      ZALTH(JLEV) = ZALTH(JLEV)/RG
      ZALTF(JLEV) = ZALTF(JLEV)/RG
    ENDDO
  ELSEIF ( NPDVAR == 3 .OR. NPDVAR == 4 ) THEN
    CALL ABOR1('SUPONG: Phi-gw NH model not yet coded for NSPONGE=2')
  ENDIF
  ! -------------------------------------------------------

  ! -----
  ! compute sponge function
  ! -----

  ! sponge function on full levels
  DO JLEV = 1, NFLEVG

    IF ( ZALTF(JLEV) >= REPONTP ) THEN
      REPONGF(JLEV) = 0.0_JPRB
    ELSEIF ( ZALTF(JLEV) <= REPONBT ) THEN
      REPONGF(JLEV) = 1.0_JPRB
    ELSE
      ZARG          = RPI*(REPONTP - ZALTF(JLEV))/ &
       &              (2.0_JPRB*(REPONTP - REPONBT))
      REPONGF(JLEV) = REPONTAU*TAN(ZARG)/(ZTSTEP + REPONTAU*TAN(ZARG))
    ENDIF

  ENDDO

  ! sponge function on half levels
  DO JLEV = 0, NFLEVG

    IF( ZALTH(JLEV) >= REPONTP ) THEN
      REPONGH(JLEV) = 0.0_JPRB
    ELSEIF( ZALTH(JLEV) <= REPONBT ) THEN
      REPONGH(JLEV) = 1.0_JPRB
    ELSE
      ZARG          = RPI*(REPONTP - ZALTH(JLEV))/ &
       &              (2.0_JPRB*(REPONTP - REPONBT))
      REPONGH(JLEV) = REPONTAU*TAN(ZARG)/(ZTSTEP + REPONTAU*TAN(ZARG))
    ENDIF

  ENDDO

  ! -----
  ! write sponge function and background profiles
  ! -----

  WRITE(NULOUT, '(A)') 'ABSORBANT COEFFICIENT ON HALF LEVELS:'
  DO JLEV = 0, NFLEVG
    WRITE(NULOUT, '(I3, 1X, F9.2, 1X, F9.6)') &
     & JLEV, ZALTH(JLEV), 1.0_JPRB - REPONGH(JLEV)
  ENDDO
  WRITE(NULOUT, *)

  WRITE(NULOUT, '(A)') 'ABSORBANT COEFFICIENT ON FULL LEVELS:'
  DO JLEV = 1, NFLEVG
    WRITE(NULOUT, '(I3, 1X, F9.2, 1X, F9.6)') &
     & JLEV, ZALTF(JLEV), 1.0_JPRB - REPONGF(JLEV)
  ENDDO
  WRITE(NULOUT, *)

  WRITE(NULOUT, '(A)') 'SPONGE TEMPERATURE ON FULL LEVELS:'
  DO JLEV = 1, NFLEVG
    WRITE(NULOUT, '(I3, 1X, F9.2, 1X, F9.2)') &
     & JLEV, ZALTF(JLEV), RT_SPNG(JLEV)
  ENDDO
  WRITE(NULOUT, *)

  WRITE(NULOUT, '(A)') 'SPONGE U-WIND ON FULL LEVELS:'
  DO JLEV = 1, NFLEVG
    WRITE(NULOUT, '(I3, 1X, F9.2, 1X, F9.2)') &
     & JLEV, ZALTF(JLEV), RU_SPNG(JLEV)
  ENDDO
  WRITE(NULOUT, *)

  WRITE(NULOUT, '(A)') 'SPONGE V-WIND ON FULL LEVELS:'
  DO JLEV = 1, NFLEVG
    WRITE(NULOUT, '(I3, 1X, F9.2, 1X, F9.2)') &
     & JLEV, ZALTF(JLEV), RV_SPNG(JLEV)
  ENDDO
  WRITE(NULOUT, *)

ELSE

  WRITE(NULOUT, *) 'Please set your NSPONGE to one of the values:'
  WRITE(NULOUT, *) '  0 - sponge off'
  WRITE(NULOUT, *) '  2 - sponge on'
  CALL ABOR1('SUPONG: ABOR1 CALLED')

ENDIF

! -----------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUPONG',1,ZHOOK_HANDLE)

END SUBROUTINE SUPONG

