SUBROUTINE SUSLB

!   ------------------------------------------------------------

!**** *SUSLB   - Setup relative pointers for semi-lagrangian buffers

!     Purpose.
!     --------
!           Initialize PTRSLB1 and PTRSLB2, modules that contain
!           the relative pointers for the semi-lagrangian buffers.
!           These are used under GP_MODEL to compute the absolute
!           pointers required. Also computes the parities needed for
!           the extension of semi-lagrangian buffer 1 over the poles.
!           Compute total number of fields in semi-lagrangian buffers.

!**   Interface.
!     ----------
!        *CALL* *SUSLB*

!        Explicit arguments : none
!        --------------------

!        Implicit arguments :
!        --------------------
!        modules PTRSLB1, PTRSLB2

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!      None
!      Is called by SUSC2B.

!     Reference.
!     ----------

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

! Modifications
! -------------
!   Original : 92-08-01
!   Modifications:
!   K. Yessad: 18/12/1992. Non interpolating vertical scheme.
!   M.Hamrud : 93-06-01   ( Supress some fields from non-interpolating
!                          vertical scheme)
!   Modified 93-12-14 by K. YESSAD: interpolations at medium point for
!                                 semi-lagrangian temperature equation.
!   C.Temperton: 93-12-20 interpolation at mid-point for
!                           semi-Lagrangian moisture equation
!   M.Hortal: 94-03-23 interpolation at mid-point for
!                           semi-Lagrangian momentum equations
!   C.Temperton: 94-05-17 spectral RT option
!   Modified 93-11-10 by R. EL Khatib (Non hydrostatic variables)
!   M. Tolstykh: 94-10-17 CUD-5 interpolation
!   Modified JAN 1995 by K. YESSAD:
!       - corrections for computation of NBFLDA.
!       - remove useless quantities NBFLDO and NBFLDM.
!       - definitions of local pointers for CUD-5 derivatives.
!       - correction of computation of RPARSL1 for CUD-5 derivatives,
!         especially for new variables (liquid water, ice, cloudiness),
!         and configurations NVLAG=3, NWLAG=3, NTLAG=3.
!       - Simplification of treatment of NVLAG=2 formulation of
!         2D shallow-water continuity equation.
!   K. YESSAD (OCTOBER 1995):
!       - Add pointers for accurate 2TL SL scheme and second-order
!         decentering for 2TL SL scheme.
!       - Rewriting according to F90 norms.
!   A. Untch: 96-03 grid-point prognostic ozone 
!   K. YESSAD and R. BUBNOVA (MAY 1996): non-hydrostatic SL scheme.
!   C.Temperton: 97-05-14: NVLAG=3 and NWLAG=3 for SWE model
!   E.Holm: 98-01-08 spectral ozone option included
!   C.Temperton: 98-01-19: 2TL SL scheme for SWE model
!   C.Temperton: 98-03-04: Repair 3TL semi-Lagrangian scheme
!   Modified 98-09-01 by N. Wedi: split physics in semi-lagrangian mode
!   K. YESSAD: 98-03-20: bug corrections.
!   K. YESSAD: 98-05-07: adapt L2TLFF to LAGPHY=.F. + cleaning.
!   K. YESSAD: July 1998: some cleanings
!       - removal of SL2TL options requiring an anterior point.
!       - correction of false comments.
!       - transfer of 2D model items in new SUSLB2. 
!   C.Temperton: 99-08-23: Pointers for TL/AD of semi-Lagrangian;
!                          allow LSPO3+LSLAG.
!   K. YESSAD: 98-12-04: New option 'lradvtoro'.
!   Modified 99-03-31 by P. SMOLIKOVA: option LRDAORG (double averaging).
!   C.Temperton: 00-02-07: MSLB1SP0 (& SP05) removed if NVLAG=3        
!   C.Temperton: 00-02-08: MSLB1C0 included for SL TL/AD               
!    (for improved vectorization in the "update loop" in LARCINAD)
!   C.Temperton: 00-02-14: LSLAG and NCONF=601       
!   C.Temperton: 00-10-10: LSLAG and NCONF=801       
!   C.Temperton: 01-01-29: TL/AD of the case LSPRT=.T.
!   Modified 01-06-26 by K. YESSAD:
!    - cleanings (remove useless variables),
!      update or correct some comments, especially in the context of the
!      introduction of thin layer hypothesis relaxation ("(a/rs)*winds"
!      are interpolated and no longer winds if lvercor=true).
!    - relaxation of thin layer hypothesis.
!   Modified 01-08-30 by K. YESSAD: pruning and some other cleanings.
!   D.Salmond (15-10-01) FULLIMP mods
!   J.Vivoda (03-2002) PC scheme for NH dynamics
!   P.Smolikova 02-09-30 : buffer SLBUF2AUX + pointers for d4 in NH
!   Modified 08-2002 C. Smith : use "w" as prognostic variable in the
!    semi-lag advection of vertical divergence in the NH model.
!   Modified 08-2003 by F. Vana: option LSLHD.
!   Modified 08-2003: option LRDBBC.
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   01-Dec-2003 M. Hortal Introduction of LSVTSM, stratospheric vertical velocity smoothing
!   10-Jun-2004 J. Masek   NH cleaning (LPC_NOTR)
!   04-Nov-2004 F. Vana : changing buffers for SLHD
!   20-Feb-2005 J. Vivoda GWADV scheme for PC_FULL with d3 and d4
!   18-Jul-2005 R. Brozkova cleaning old VDAUX structure => "X" term       
!   K. Yessad 07-02-2007: Splitting alti/surf for (gw) in NH+LGWADV.
!   K. Yessad 07-03-2007: Remove useless (gw)_surf interpolations in NH+LGWADV.
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   F. Vana  13-Jan-2009 : MSLB2KAPPA5 for TL/AD of SLHD
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMHOOK  , ONLY : LHOOK    ,DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY :    NFLEVG   ,NUNDEFLD
USE YOMSLPHY , ONLY : LSLPHY
USE YOMCT0   , ONLY : NPRINTLEV,LSLAG    ,LSPRT    ,&
 & LNHDYN   ,LALLOPR  ,NCONF    ,LVERCOR   ,&
 & LPC_OLD
USE YOMDYNA  , ONLY : LSLHD    ,LRDBBC   ,NVDVAR   ,LNH_PDVD ,LGWADV   ,&
 & LRWSDLW  ,LRWSDLR
USE YOMDYN   , ONLY : NVLAG    ,NWLAG    ,NTLAG    ,NSPDLAG  ,&
 & NSVDLAG  ,L2TLFF   ,LADVF    ,LADVFW   ,LSLINLC1 ,LSLINLC2 ,LSLINL
USE YOEPHY   , ONLY : LEPHYS   ,LAGPHY
USE YOMPHY   , ONLY :    LMPHYS
USE YOMSC2   , ONLY : NFLDSLB1 ,NFLDSLB2,NFLDSLB15
USE YOMCVER  , ONLY : LSVTSM   ,LVERTFE  ,LVFE_GW
USE PTRSLB1  , ONLY : RPARSL1  ,MSLBUF1  ,MSLB1U9  ,MSLB1V9  ,&
 & MSLB1T9  ,MSLB1GFL9, MSLB1GFLSP9  ,MSLB1PD9 ,MSLB1VD9 ,MSLB1NHX9 ,&
 & MSLB1Y9  ,MSLB1UR0 ,MSLB1VR0 ,MSLB1WR0 ,MSLB1WRA ,MSLB1UR9 ,&
 & MSLB1VR9 ,MSLB1U0  ,MSLB1V0  ,&
 & MSLB1T0  ,MSLB1PD0 ,MSLB1VD0 ,MSLB1C9  ,&
 & MSLB1SP9 ,MSLB1C0  ,&
 & MSLB1UP9 ,MSLB1VP9 ,MSLB1TP9 ,MSLB1GFLP9 ,&
 & MSLB1DBBC9, MSLB1DPHI9, MSLB1GWS9 ,&
 & MSLB1U9_SI,MSLB1V9_SI,MSLB1T9_SI,MSLB1PD9_SI,MSLB1VD9_SI,MSLB1C9_SI,&
 & MSLB1RSSA9  
USE PTRSLB15 , ONLY : RPARSL15  ,MSLBUF15,          &
 & MSLB1U95 ,MSLB1V95 ,MSLB1T95 ,MSLB1GFL95 ,&
 & MSLB1C95 ,MSLB1U05 ,&
 & MSLB1V05 ,MSLB1T05 ,MSLB1C05 ,MSLB1SP95,&
 & MSLB1UR05,MSLB1VR05,MSLB1WR05,MSLB1UR95,&
 & MSLB1VR95  
USE PTRSLB2  , ONLY : MSLBUF2  ,&
 & MSLB2Y1  ,&
 & MSLB2DBBC1, MSLB2DPHI1, &
 & MSLB2USI ,MSLB2VSI ,MSLB2TSI ,MSLB2PDSI,&
 & MSLB2VDSI,MSLB2SPSI,MSLB2VVEL,MSLB2URL,MSLB2VRL,&
 & MSLB2WRL ,MSLB2U15 ,MSLB2V15 ,MSLB2T15 ,MSLB2Q15 , &
 & MSLB2URL5,MSLB2VRL5,MSLB2WRL5,MSLB2VCVWEI,MSLB2KAPPA, &
 & MSLB2GWF ,MSLB2GDW ,MSLB2GWS ,MSLB2KAPPA5
USE YOM_YGFL , ONLY : YGFL
USE YOMMASK , ONLY : MFIELD_TYPE1, MFIELD_TYPE15, LNEW_MASK_SL

!   ------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IGFL9
INTEGER(KIND=JPIM) :: IGFL9_SPL
INTEGER(KIND=JPIM) :: IGFLP9
INTEGER(KIND=JPIM) :: IC0
INTEGER(KIND=JPIM) :: IC9,IC9_SI
INTEGER(KIND=JPIM) :: IFLDSLB2
INTEGER(KIND=JPIM) :: ILVP2
INTEGER(KIND=JPIM) :: IPD0
INTEGER(KIND=JPIM) :: IPD9,IPD9_SI
INTEGER(KIND=JPIM) :: IPTR2
INTEGER(KIND=JPIM) :: IPTX
INTEGER(KIND=JPIM) :: IRSSA9
INTEGER(KIND=JPIM) :: ISP9
INTEGER(KIND=JPIM) :: IT0
INTEGER(KIND=JPIM) :: IT9,IT9_SI
INTEGER(KIND=JPIM) :: ITP9
INTEGER(KIND=JPIM) :: IU
INTEGER(KIND=JPIM) :: IU0
INTEGER(KIND=JPIM) :: IU9,IU9_SI
INTEGER(KIND=JPIM) :: IUP9
INTEGER(KIND=JPIM) :: IUR0
INTEGER(KIND=JPIM) :: IUR9
INTEGER(KIND=JPIM) :: IV0
INTEGER(KIND=JPIM) :: IV9,IV9_SI
INTEGER(KIND=JPIM) :: IVD0
INTEGER(KIND=JPIM) :: IVD9,IVD9_SI
INTEGER(KIND=JPIM) :: INHX9
INTEGER(KIND=JPIM) :: IVP9
INTEGER(KIND=JPIM) :: IVR0
INTEGER(KIND=JPIM) :: IVR9
INTEGER(KIND=JPIM) :: IWR0
INTEGER(KIND=JPIM) :: IWRA
INTEGER(KIND=JPIM) :: IY9
INTEGER(KIND=JPIM) :: JF
INTEGER(KIND=JPIM) :: IU95
INTEGER(KIND=JPIM) :: IV95
INTEGER(KIND=JPIM) :: IT95
INTEGER(KIND=JPIM) :: IGFL95
INTEGER(KIND=JPIM) :: IC95
INTEGER(KIND=JPIM) :: IU05
INTEGER(KIND=JPIM) :: IV05
INTEGER(KIND=JPIM) :: IT05
INTEGER(KIND=JPIM) :: IC05
INTEGER(KIND=JPIM) :: ISP95
INTEGER(KIND=JPIM) :: IUR05
INTEGER(KIND=JPIM) :: IVR05
INTEGER(KIND=JPIM) :: IWR05
INTEGER(KIND=JPIM) :: IUR95
INTEGER(KIND=JPIM) :: IVR95
INTEGER(KIND=JPIM) :: INUMGFL
INTEGER(KIND=JPIM) :: INUMGFL_SPL
INTEGER(KIND=JPIM) :: INUMGFL_SL1
INTEGER(KIND=JPIM) :: JGFL,IGFL
INTEGER(KIND=JPIM) :: IDBBC9
INTEGER(KIND=JPIM) :: IDPHI9
INTEGER(KIND=JPIM) :: IGWS9

LOGICAL :: LL2TLFF1
LOGICAL :: LLP
LOGICAL :: LLSLAD

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSLB',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*    1.   COMPUTE RELATIVE POINTERS FOR SEMILAGRANGIAN BUFFERS.
!          -----------------------------------------------------

LL2TLFF1=L2TLFF.AND.(LMPHYS.OR.LEPHYS).AND.(.NOT.LAGPHY)
LLP = NPRINTLEV >= 1.OR. LALLOPR
LLP = .TRUE.
IU = NULOUT
IF (LLP) WRITE (IU,*)
IF (LLP) WRITE (IU,'(A,/)') ' SUSLB PRINTOUT : POINTERS,NFIELDS'
INUMGFL=YGFL%NUMFLDS
INUMGFL_SL1=YGFL%NUMFLDS_SL1
INUMGFL_SPL=YGFL%NUMFLDS_SPL

!*       1.1  COMPUTE POINTERS FOR SLBUF2

!*       1.1.1 Preliminary set pointers to value for undefined/unused fields

! * MSLBUF2
MSLBUF2  =NUNDEFLD
! * MSLB2[X]1
MSLB2DBBC1=NUNDEFLD
MSLB2DPHI1=NUNDEFLD
MSLB2Y1  =NUNDEFLD
! * MSLB2[X]SI
MSLB2USI =NUNDEFLD
MSLB2VSI =NUNDEFLD
MSLB2TSI =NUNDEFLD
MSLB2PDSI=NUNDEFLD
MSLB2VDSI=NUNDEFLD
MSLB2SPSI=NUNDEFLD
! * MSLB2VVEL
MSLB2VVEL=NUNDEFLD
! * MSLB2[X]RL
MSLB2URL =NUNDEFLD
MSLB2VRL =NUNDEFLD
MSLB2WRL =NUNDEFLD
! * MSLB2[X]15
MSLB2U15 =NUNDEFLD
MSLB2V15 =NUNDEFLD
MSLB2T15 =NUNDEFLD
MSLB2Q15 =NUNDEFLD
! * MSLB2[X]RL5
MSLB2URL5=NUNDEFLD
MSLB2VRL5=NUNDEFLD
MSLB2WRL5=NUNDEFLD
! * MSLB2VCVWEI
MSLB2VCVWEI=NUNDEFLD
! * MSLB2KAPPA
MSLB2KAPPA=NUNDEFLD
! * MSLB2KAPPA5
MSLB2KAPPA5=NUNDEFLD
! * MSLB2GWF, MSLB2GDW, MSLB2GWS
MSLB2GWF=NUNDEFLD
MSLB2GDW=NUNDEFLD
MSLB2GWS=NUNDEFLD

!*       1.1.2 Definite computation of pointers.

IF (LLP) WRITE (IU,'(A)') ' 1) BUFFER SLBUF2:'
IPTX    = 1

! * MSLBUF2

MSLBUF2 = IPTX

! * MSLB2DBBC1 and MSLB2DPHI1

IF(LRDBBC) THEN
  MSLB2DBBC1= IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2DBBC1= ',IPTX,1
  IPTX  = IPTX + 1
  MSLB2DPHI1= IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2DPHI1= ',IPTX,1
  IPTX  = IPTX + 1
ENDIF

! * MSLB2Y1

IF( LPC_OLD.AND.LNHDYN )THEN
  MSLB2Y1   = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2Y1   = ',IPTX,NFLEVG
  IPTX      = IPTX + NFLEVG
ENDIF

! * MSLB2[X]SI

MSLB2USI  = IPTX
IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2USI  = ',IPTX,NFLEVG
IPTX      = IPTX + NFLEVG
MSLB2VSI  = IPTX
IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2VSI  = ',IPTX,NFLEVG
IPTX      = IPTX + NFLEVG
MSLB2TSI  = IPTX
IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2TSI  = ',IPTX,NFLEVG
IPTX      = IPTX + NFLEVG
IF(LNHDYN) THEN
  MSLB2PDSI = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2PDSI = ',IPTX,NFLEVG
  IPTX      = IPTX + NFLEVG
  MSLB2VDSI = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2VDSI = ',IPTX,NFLEVG
  IPTX      = IPTX + NFLEVG
ENDIF
MSLB2SPSI = IPTX
IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2SPSI = ',IPTX,1
IPTX      = IPTX + 1

! * MSLB2VVEL

MSLB2VVEL = IPTX
IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2VVEL = ',IPTX,NFLEVG
IPTX      = IPTX + NFLEVG

! * MSLB2[X]RL

MSLB2URL = IPTX
IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2URL  = ',IPTX,NFLEVG
IPTX    = IPTX + NFLEVG
MSLB2VRL = IPTX
IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2VRL  = ',IPTX,NFLEVG
IPTX    = IPTX + NFLEVG
MSLB2WRL = IPTX
IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2WRL  = ',IPTX,NFLEVG
IPTX    = IPTX + NFLEVG

IPTR2     = IPTX

LLSLAD=LSLAG.AND.(NCONF == 131.OR.NCONF == 401.OR.NCONF == 501&
 & .OR.NCONF == 601.OR.NCONF == 801)  

! * MSLB2[X]15 then MSLB2[X]RL5

IF (LLSLAD) THEN
  IPTX    = IPTR2
  IF (LADVF.AND.L2TLFF) THEN
    MSLB2U15 = IPTX
    IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2U15  = ',IPTX,NFLEVG
    IPTX    = IPTX + NFLEVG
    MSLB2V15 = IPTX
    IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2V15  = ',IPTX,NFLEVG
    IPTX    = IPTX + NFLEVG
  ENDIF
  MSLB2URL5 = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2URL5 = ',IPTX,NFLEVG
  IPTX    = IPTX + NFLEVG
  MSLB2VRL5 = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2VRL5 = ',IPTX,NFLEVG
  IPTX    = IPTX + NFLEVG
  MSLB2WRL5 = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2WRL5 = ',IPTX,NFLEVG
  IPTX    = IPTX + NFLEVG
  IPTR2   = IPTX

ENDIF

IF (LSPRT.AND.(NCONF == 131.OR.NCONF == 401.&
 &OR.NCONF == 501.OR.NCONF == 601.OR.NCONF == 801)) THEN
  IPTX    = IPTR2
  MSLB2T15 = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2T15  = ',IPTX,NFLEVG
  IPTX    = IPTX + NFLEVG
  MSLB2Q15 = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2Q15  = ',IPTX,NFLEVG
  IPTX    = IPTX + NFLEVG
  IPTR2   = IPTX
ENDIF

! * MSLB2VCVWEI

IF (LVERCOR.AND.LSLAG) THEN
  MSLB2VCVWEI = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2VCVWEI=',IPTX,NFLEVG
  IPTX    = IPTX + NFLEVG
  IPTR2    = MAX(IPTX,IPTR2)
ENDIF

! * MSLB2KAPPA(5)

IF (LSLHD) THEN
  MSLB2KAPPA = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2KAPPA= ',IPTX,NFLEVG
  IPTX    = IPTX + NFLEVG
  IPTR2    = MAX(IPTX,IPTR2)
  IF (LLSLAD) THEN
    MSLB2KAPPA5 = IPTX
    IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2KAPPA5= ',IPTX,NFLEVG
    IPTX    = IPTX + NFLEVG
    IPTR2    = MAX(IPTX,IPTR2)
  ENDIF
ENDIF

! * MSLB2GWF, MSLB2GDW, MSLB2GWS

IF (LNHDYN.AND.LNH_PDVD.AND.LGWADV.AND.LVERTFE.AND.LVFE_GW) THEN
  MSLB2GWF = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2GWF  = ',IPTX,NFLEVG
  IPTX    = IPTX + NFLEVG
  MSLB2GDW = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2GDW  = ',IPTX,NFLEVG
  IPTX    = IPTX + NFLEVG
  MSLB2GWS = IPTX
  IF (LLP) WRITE(IU,'(1X,A12,2I4)') 'MSLB2GWS  = ',IPTX,1
  IPTX    = IPTX + 1
  IPTR2   = MAX(IPTX,IPTR2)
ENDIF

IFLDSLB2 = IPTR2-1
NFLDSLB2 = IFLDSLB2

!*       1.2  COMPUTE POINTERS FOR SLBUF1

!*       1.2.1 Preliminary set pointers to value for undefined/unused fields

! * MSLBUF1
MSLBUF1  =NUNDEFLD

! * MSLB1[X]9 and MSLB1RSSA9 (for high order interpolations)
MSLB1UR9 =NUNDEFLD
MSLB1VR9 =NUNDEFLD
MSLB1U9  =NUNDEFLD
MSLB1V9  =NUNDEFLD
MSLB1T9  =NUNDEFLD
MSLB1GFL9=NUNDEFLD
MSLB1GFLSP9 =NUNDEFLD
MSLB1SP9 =NUNDEFLD
MSLB1C9  =NUNDEFLD
MSLB1VD9 =NUNDEFLD
MSLB1NHX9=NUNDEFLD
MSLB1PD9 =NUNDEFLD
MSLB1RSSA9=NUNDEFLD

! * MSLB1[X]9_SI (additional separate linear terms)
MSLB1U9_SI=NUNDEFLD
MSLB1V9_SI=NUNDEFLD
MSLB1T9_SI=NUNDEFLD
MSLB1C9_SI=NUNDEFLD
MSLB1VD9_SI=NUNDEFLD
MSLB1PD9_SI=NUNDEFLD

! * MSLB1Y9
MSLB1Y9  =NUNDEFLD

! * Diagnostic BBC: MSLB1DBBC9, MSLB1DPHI9 and MSLB1GWS9.
MSLB1DBBC9=NUNDEFLD
MSLB1DPHI9=NUNDEFLD
MSLB1GWS9 =NUNDEFLD

! * MSLB1[X]0 (for linear interpolations)
MSLB1UR0 =NUNDEFLD
MSLB1VR0 =NUNDEFLD
MSLB1WR0 =NUNDEFLD
MSLB1WRA =NUNDEFLD
MSLB1U0  =NUNDEFLD
MSLB1V0  =NUNDEFLD
MSLB1T0  =NUNDEFLD
MSLB1C0  =NUNDEFLD
MSLB1VD0 =NUNDEFLD
MSLB1PD0 =NUNDEFLD

! * MSLB1[X]P9
MSLB1UP9 =NUNDEFLD
MSLB1VP9 =NUNDEFLD
MSLB1TP9 =NUNDEFLD
MSLB1GFLP9 =NUNDEFLD

! * MSLB1[X]95 and MSLB1[X]05
MSLBUF15 =NUNDEFLD
MSLB1U95 =NUNDEFLD
MSLB1V95 =NUNDEFLD
MSLB1T95 =NUNDEFLD
MSLB1GFL95 =NUNDEFLD
MSLB1U05 =NUNDEFLD
MSLB1V05 =NUNDEFLD
MSLB1T05 =NUNDEFLD
MSLB1C95 =NUNDEFLD
MSLB1C05 =NUNDEFLD
MSLB1SP95=NUNDEFLD
MSLB1UR05=NUNDEFLD
MSLB1VR05=NUNDEFLD
MSLB1WR05=NUNDEFLD
MSLB1UR95=NUNDEFLD
MSLB1VR95=NUNDEFLD

!*       1.2.2 Definite computation of pointers.

IF(LSLAG) THEN

  IF (LLP) WRITE (IU,*)
  IF (LLP) WRITE (IU,'(A)') ' 2) BUFFER SLBUF1:'

  ILVP2   = NFLEVG+2
  IPTX    = 1
  MSLBUF1 = IPTX

!        * Buffers for high order interpolations:
!          MSLB1[X]9 and MSLB1RSSA9

  IF (LL2TLFF1) THEN
    MSLB1UR9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1UR9    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1VR9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1VR9    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  MSLB1U9  = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1U9     = ',IPTX,ILVP2
  IPTX     = IPTX + ILVP2
  MSLB1V9  = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1V9     = ',IPTX,ILVP2
  IPTX     = IPTX + ILVP2
  MSLB1T9  = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1T9     = ',IPTX,ILVP2
  IPTX     = IPTX + ILVP2
  IF(INUMGFL_SL1> 0) THEN
    MSLB1GFL9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1GFL9   = ',IPTX,ILVP2*INUMGFL_SL1
    IPTX     = IPTX + ILVP2*INUMGFL_SL1
    IF(INUMGFL_SPL > 0) THEN
      MSLB1GFLSP9 = IPTX
      IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1GFLSP9     = ' &
       & ,IPTX,ILVP2*INUMGFL_SPL
      IPTX    = IPTX + ILVP2*INUMGFL_SPL
    ENDIF
  ENDIF
  MSLB1SP9 = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1SP9    = ',IPTX,1
  IPTX     = IPTX + 1
  IF (NVLAG == 2) THEN
    MSLB1C9  = IPTX
    IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1C9     = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  IF (LNHDYN) THEN
    MSLB1VD9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1VD9    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    IF (NVDVAR == 4) THEN
      MSLB1NHX9 = IPTX
      IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1NHX9   = ',IPTX,ILVP2
      IPTX   = IPTX + ILVP2
    ENDIF
    MSLB1PD9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1PD9    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  IF ((LVERCOR.OR.(LRWSDLW.AND.LRWSDLR)).AND.LADVFW) THEN
    MSLB1RSSA9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1RSSA9  = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF

!        * Buffers for high order interpolations,
!          for terms specific to predictor-corrector scheme
!          (case LPC_OLD).
!          MSLB1Y9

  IF(LPC_OLD.AND.LNHDYN)THEN
    MSLB1Y9  = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1Y9     = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF

!        * Buffers for high order interpolations,
!          for terms specific to diagnostic BBC in NH

  IF (LRDBBC .AND. (NSVDLAG == 2)) THEN
    MSLB1DBBC9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1DBBC9   = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1DPHI9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1DPHI9   = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  IF (LRDBBC) THEN
    MSLB1GWS9  = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1GWS9    = ',IPTX,1
    IPTX     = IPTX + 1
  ENDIF

!        * Buffers for separate linear terms
!          (linear interpolations if N(X)LAG=3). 

  IF (LSLINL .AND. NWLAG==3) THEN
    MSLB1U9_SI = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1U9_SI  = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1V9_SI = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1V9_SI  = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  IF (LSLINL .AND. NTLAG==3) THEN
    MSLB1T9_SI = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1T9_SI  = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  IF (LNHDYN .AND. LSLINL .AND. NSVDLAG==3) THEN
    MSLB1VD9_SI = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1VD9_SI = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  IF (LNHDYN .AND. LSLINL .AND. NSPDLAG==3) THEN
    MSLB1PD9_SI = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1PD9_SI = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF

!        * Buffers for linear interpolations,
!          for terms specific to diagnostic BBC in NH

  IF (LRDBBC .AND. (NSVDLAG == 3)) THEN
    MSLB1DBBC9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1DBBC9   = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1DPHI9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1DPHI9   = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF

!        * Buffers for linear interpolations,
!          for terms other than split physics.
!          MSLB1[X]0 and MSLB1C9

  MSLB1UR0 = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1UR0    = ',IPTX,ILVP2
  IPTX     = IPTX + ILVP2
  MSLB1VR0 = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1VR0    = ',IPTX,ILVP2
  IPTX     = IPTX + ILVP2
  IF(LSVTSM) THEN
    MSLB1WRA = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1WRA    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  MSLB1WR0 = IPTX
  IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1WR0    = ',IPTX,ILVP2
  IPTX     = IPTX + ILVP2
  IF (NWLAG == 3) THEN
    MSLB1U0  = IPTX
    IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1U0     = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1V0  = IPTX
    IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1V0     = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  IF (NTLAG == 3) THEN
    MSLB1T0  = IPTX
    IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1T0     = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  IF (NVLAG == 3) THEN
    MSLB1C9  = IPTX
    IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1C9     = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  IF (NVLAG == 3 .AND. (LSLINLC1.OR.LSLINLC2)) THEN
    MSLB1C9_SI= IPTX
    IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1C9_SI  = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  ! For improved vectorization in SL TL/AD applications:
  IF (LLSLAD) THEN
    MSLB1C0  = IPTX
    IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1C0     = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
  ENDIF
  IF (LNHDYN) THEN
    IF (NSVDLAG == 3) THEN
      MSLB1VD0  = IPTX
      IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1VD0    = ',IPTX,ILVP2
      IPTX     = IPTX + ILVP2
    ENDIF
    IF (NSPDLAG == 3) THEN
      MSLB1PD0  = IPTX
      IF (LLP)WRITE(IU,'(1X,A14,2I4)') 'MSLB1PD0    = ',IPTX,ILVP2
      IPTX     = IPTX + ILVP2
    ENDIF
  ENDIF

!        * Buffers for linear interpolations,
!          for terms used in split physics.
!          MSLB1[X]P9

  IF( LSLPHY ) THEN
    MSLB1UP9  = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1UP9     = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1VP9  = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1VP9     = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1TP9  = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1TP9     = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1GFLP9 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,3I4)') 'MSLB1GFLP9   = ',IPTX,ILVP2,INUMGFL
    DO JGFL=1,INUMGFL
      ! Numbering of fields, and allocation in 'SLBUF1', should be consistent
      !  with what is done in GPADDSLPHY and LARCINB.
      ! For the time being all GFL fields are counted and allocated, even those 
      !  which are useless (i.e. contain 0 or not advected).
      ! Memory optimisation when some GFL are passive ones or not advected,
      !  must be studied later. 
      IPTX    = IPTX + ILVP2
    ENDDO
  ENDIF

  NFLDSLB1 = IPTX - 1

!        * Buffers used in tangent-linear/adjoint of semi-Lagrangian.
!          MSLB1[X]95 and MSLB1[X]05

  IF (LLSLAD) THEN
    IF (LLP) WRITE (IU,*)
    IF (LLP) WRITE (IU,'(A)') ' 2) BUFFER SLBUF15:'
    IPTX    = 1
    MSLBUF15 = IPTX

    MSLB1U95 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1U95    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1V95 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1V95    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1T95 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1T95    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    IF(INUMGFL_SL1 > 0) THEN
      MSLB1GFL95 = IPTX
      IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1GFL95    = ',IPTX,ILVP2*INUMGFL_SL1
      IPTX     = IPTX + ILVP2*INUMGFL_SL1
    ENDIF
    MSLB1U05 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1U05    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1V05 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1V05    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1T05 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1T05    = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    IF (NVLAG == 2.OR.NVLAG == 3) THEN
      MSLB1C95 = IPTX
      IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1C95    = ',IPTX,ILVP2
      IPTX     = IPTX + ILVP2
      MSLB1C05 = IPTX
      IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1C05    = ',IPTX,ILVP2
      IPTX     = IPTX + ILVP2
    ENDIF
    MSLB1SP95 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1SP95   = ',IPTX,1
    IPTX     = IPTX + 1
    MSLB1UR05 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1UR05   = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1VR05 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1VR05   = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    MSLB1WR05 = IPTX
    IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1WR05   = ',IPTX,ILVP2
    IPTX     = IPTX + ILVP2
    IF (LL2TLFF1) THEN
      MSLB1UR95 = IPTX
      IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1UR95   = ',IPTX,ILVP2
      IPTX     = IPTX + ILVP2
      MSLB1VR95 = IPTX
      IF (LLP) WRITE(IU,'(1X,A14,2I4)') 'MSLB1VR95   = ',IPTX,ILVP2
      IPTX     = IPTX + ILVP2
    ENDIF

    NFLDSLB15 = IPTX - 1
  ELSE
    NFLDSLB15 = 1
  ENDIF
   
ELSE

  NFLDSLB1 = 1
  NFLDSLB15 = 1

ENDIF

IF (LLP) WRITE (IU,*)

!     ------------------------------------------------------------------

!*    2.   COMPUTE PARITIES FOR EXTENSION OVER POLE.
!          -----------------------------------------

IF (LSLAG) THEN
  IF(ALLOCATED(RPARSL1)) DEALLOCATE(RPARSL1) ! To make SUSLB recallable
  ALLOCATE(RPARSL1(NFLDSLB1))
  IF(LLP)WRITE(IU,9) 'RPARSL1  ',SIZE(RPARSL1),SHAPE(RPARSL1)
  IF(ALLOCATED(RPARSL15)) DEALLOCATE(RPARSL15) ! To make SUSLB recallable
  ALLOCATE(RPARSL15(NFLDSLB15))
  IF(LLP)WRITE(IU,9) 'RPARSL15 ',SIZE(RPARSL15),SHAPE(RPARSL15)

  IF(ALLOCATED(MFIELD_TYPE1))DEALLOCATE(MFIELD_TYPE1) !To make SUSLB recallable
  ALLOCATE(MFIELD_TYPE1(NFLDSLB1))
  MFIELD_TYPE1(:)=1
  IF(ALLOCATED(MFIELD_TYPE15))DEALLOCATE(MFIELD_TYPE15) !To make SUSLB recallab
  ALLOCATE(MFIELD_TYPE15(NFLDSLB15))
  MFIELD_TYPE15(:)=1
  LNEW_MASK_SL=.FALSE.
  IF(LLP)WRITE(IU,9) 'MFIELD_TYPE1  ',SIZE(MFIELD_TYPE1),SHAPE(MFIELD_TYPE1)
  IF(LLP)WRITE(IU,9) 'MFIELD_TYPE15 ',SIZE(MFIELD_TYPE15),SHAPE(MFIELD_TYPE15)
ENDIF
9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)

IF (LSLAG) THEN

!        2.11 Computation of RPARSL1
!             for U,V,T,gfl,dcha,Pcha equations terms.

  DO JF=1,ILVP2

    ! * MSLB1[X]9
    IF (LL2TLFF1) THEN
      IUR9=MSLB1UR9-1
      RPARSL1(JF+IUR9)=-1.0_JPRB
      IVR9=MSLB1VR9-1
      RPARSL1(JF+IVR9)=-1.0_JPRB
    ENDIF
    IU9=MSLB1U9-1
    RPARSL1(JF+IU9)=-1.0_JPRB
    IV9=MSLB1V9-1
    RPARSL1(JF+IV9)=-1.0_JPRB
    IT9=MSLB1T9-1
    RPARSL1(JF+IT9)=1.0_JPRB
    DO JGFL=1,INUMGFL_SL1
      IGFL9=MSLB1GFL9+(JGFL-1)*ILVP2-1
      RPARSL1(JF+IGFL9)=1.0_JPRB
    ENDDO
    DO JGFL=1,INUMGFL_SPL
      IGFL9_SPL=MSLB1GFLSP9+(JGFL-1)*ILVP2-1
      RPARSL1(JF+IGFL9_SPL)=1.0_JPRB
    ENDDO
    IF (LNHDYN) THEN
      IVD9=MSLB1VD9-1
      RPARSL1(JF+IVD9)=1.0_JPRB
      IF (NVDVAR == 4) THEN
        INHX9=MSLB1NHX9-1
        RPARSL1(JF+INHX9)=1.0_JPRB
      ENDIF
      IPD9=MSLB1PD9-1
      RPARSL1(JF+IPD9)=1.0_JPRB
    ENDIF
    IF ((LVERCOR.OR.(LRWSDLW.AND.LRWSDLR)).AND.LADVFW) THEN
      IRSSA9=MSLB1RSSA9-1
      RPARSL1(JF+IRSSA9)=1.0_JPRB
    ENDIF

    ! * MSLB1Y9
    IF(LPC_OLD.AND.LNHDYN)THEN
      IY9=MSLB1Y9-1
      RPARSL1(JF+IY9)=1.0_JPRB
    ENDIF

    ! * Diagnostic BBC: MSLB1DBBC9, MSLB1DPHI9 and MSLB1GWS9.
    IF (LRDBBC) THEN
      IDBBC9=MSLB1DBBC9-1
      RPARSL1(JF+IDBBC9)=1.0_JPRB
      IDPHI9=MSLB1DPHI9-1
      RPARSL1(JF+IDPHI9)=1.0_JPRB
      IGWS9 =MSLB1GWS9-1
      RPARSL1(JF+IGWS9)=1.0_JPRB
    ENDIF

    ! * MSLB1[X]9_SI
    IF (LSLINL .AND. NWLAG==3) THEN
      IU9_SI=MSLB1U9_SI-1
      RPARSL1(JF+IU9_SI)=-1.0_JPRB
      IV9_SI=MSLB1V9_SI-1
      RPARSL1(JF+IV9_SI)=-1.0_JPRB
    ENDIF
    IF (LSLINL .AND. NTLAG==3) THEN
      IT9_SI=MSLB1T9_SI-1
      RPARSL1(JF+IT9_SI)=-1.0_JPRB
    ENDIF
    IF (LNHDYN .AND. LSLINL .AND. NSVDLAG==3) THEN
      IVD9_SI=MSLB1VD9_SI-1
      RPARSL1(JF+IVD9_SI)=-1.0_JPRB
    ENDIF
    IF (LNHDYN .AND. LSLINL .AND. NSPDLAG==3) THEN
      IPD9_SI=MSLB1PD9_SI-1
      RPARSL1(JF+IPD9_SI)=-1.0_JPRB
    ENDIF

    ! * MSLB1[X]0
    IUR0=MSLB1UR0-1
    RPARSL1(JF+IUR0)=-1.0_JPRB
    IVR0=MSLB1VR0-1
    RPARSL1(JF+IVR0)=-1.0_JPRB
    IF(LSVTSM) THEN
      IWRA=MSLB1WRA-1
      RPARSL1(JF+IWRA)=1.0_JPRB
    ENDIF
    IWR0=MSLB1WR0-1
    RPARSL1(JF+IWR0)=1.0_JPRB
    IF (NWLAG == 3) THEN
      IU0=MSLB1U0-1
      RPARSL1(JF+IU0)=-1.0_JPRB
      MFIELD_TYPE1(JF+IU0)=2
      IV0=MSLB1V0-1
      RPARSL1(JF+IV0)=-1.0_JPRB
      MFIELD_TYPE1(JF+IV0)=2
    ENDIF
    IF (NTLAG == 3) THEN
      IT0=MSLB1T0-1
      RPARSL1(JF+IT0)=1.0_JPRB
      MFIELD_TYPE1(JF+IT0)=2
    ENDIF
    IF (LNHDYN) THEN
      IF (NSVDLAG == 3) THEN
        IVD0=MSLB1VD0-1
        RPARSL1(JF+IVD0)=1.0_JPRB
      ENDIF
      IF (NSPDLAG == 3) THEN
        IPD0=MSLB1PD0-1
        RPARSL1(JF+IPD0)=1.0_JPRB
      ENDIF
    ENDIF

    ! * MSLB1[X]P9
    IF( LSLPHY ) THEN
      IUP9=MSLB1UP9-1
      RPARSL1(JF+IUP9)=-1.0_JPRB
      MFIELD_TYPE1(JF+IUP9)=2
      IVP9=MSLB1VP9-1
      RPARSL1(JF+IVP9)=-1.0_JPRB
      MFIELD_TYPE1(JF+IVP9)=2
      ITP9=MSLB1TP9-1
      RPARSL1(JF+ITP9)=1.0_JPRB
      MFIELD_TYPE1(JF+ITP9)=2
      ! Same remark as above (see MSLB1GFLP9 setting).
      IGFL=0
      DO JGFL=1,INUMGFL
        IGFL=IGFL+1
        IGFLP9=MSLB1GFLP9+(IGFL-1)*ILVP2-1
        RPARSL1(JF+IGFLP9)=1.0_JPRB
      ENDDO
    ENDIF
  ENDDO

!        2.12 Computation of RPARSL1 for 3D continuity equation terms.

  ! * MSLB1C9 and MSLB1C0
  DO JF=1,ILVP2
    IF (NVLAG == 2.OR.NVLAG == 3) THEN
      IC9=MSLB1C9-1
      RPARSL1(JF+IC9)=1.0_JPRB
    ENDIF
    IF (NVLAG == 3.AND.(LSLINLC1.OR.LSLINLC2)) THEN
      IC9_SI=MSLB1C9_SI-1
      RPARSL1(JF+IC9_SI)=1.0_JPRB
    ENDIF
    IF (LLSLAD) THEN
      IC0=MSLB1C0-1
      RPARSL1(JF+IC0)=1.0_JPRB
    ENDIF
  ENDDO

!        2.13 Computation of RPARSL1 for 2D continuity equation terms.

  ! * MSLB1SP9
  ISP9=MSLB1SP9-1
  RPARSL1(1+ISP9)=1.0_JPRB

!        2.14 Computation of RPARSL1 for TL/AD of semi-Lagrangian.

  ! * MSLB1[X]95 and MSLB1[X]05
  IF (LLSLAD) THEN

    DO JF=1,ILVP2
      IU95=MSLB1U95-1
      RPARSL15(JF+IU95)=-1.0_JPRB
      IV95=MSLB1V95-1
      RPARSL15(JF+IV95)=-1.0_JPRB
      IT95=MSLB1T95-1
      RPARSL15(JF+IT95)=1.0_JPRB
      DO JGFL=1,INUMGFL_SL1
        IGFL95=MSLB1GFL95+(JGFL-1)*ILVP2-1
        RPARSL15(JF+IGFL95)=1.0_JPRB
      ENDDO
      IU05=MSLB1U05-1
      RPARSL15(JF+IU05)=-1.0_JPRB
      MFIELD_TYPE15(JF+IU05)=2
      IV05=MSLB1V05-1
      RPARSL15(JF+IV05)=-1.0_JPRB
      MFIELD_TYPE15(JF+IV05)=2
      IT05=MSLB1T05-1
      RPARSL15(JF+IT05)=1.0_JPRB
      MFIELD_TYPE15(JF+IT05)=2
      IF (NVLAG == 2.OR.NVLAG == 3) THEN
        IC05=MSLB1C05-1
        RPARSL15(JF+IC05)=1.0_JPRB
        IC95=MSLB1C95-1
        RPARSL15(JF+IC95)=1.0_JPRB
      ENDIF
      IUR05=MSLB1UR05-1
      RPARSL15(JF+IUR05)=-1.0_JPRB
      MFIELD_TYPE15(JF+IUR05)=2
      IVR05=MSLB1VR05-1
      RPARSL15(JF+IVR05)=-1.0_JPRB
      MFIELD_TYPE15(JF+IVR05)=2
      IWR05=MSLB1WR05-1
      RPARSL15(JF+IWR05)=1.0_JPRB
      MFIELD_TYPE15(JF+IWR05)=2
      IF (LL2TLFF1) THEN
        IUR95=MSLB1UR95-1
        RPARSL15(JF+IUR95)=-1.0_JPRB
        IVR95=MSLB1VR95-1
        RPARSL15(JF+IVR95)=-1.0_JPRB
      ENDIF
    ENDDO

    ISP95=MSLB1SP95-1
    RPARSL15(1+ISP95)=1.0_JPRB
  ENDIF

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSLB',1,ZHOOK_HANDLE)
END SUBROUTINE SUSLB
