SUBROUTINE SUSPECA(KFILE,LDATA,PSPVOR,PSPDIV,PSPT,PSPGFL,PSPSP,PSPOR,PSPSPD, &
 & PSPSVD,PSPUB,PSPVB,LDINOR)  

!**** *SUSPECA*  - Initialize the spectral fields from FA (Arpege or Aladin)

!     Purpose.
!     --------
!         Initialize the spectral fields of the model from ARPEGE ALADIN file.

!**   Interface.
!     ----------
!        *CALL* *SUSPECA(.....)*

!        Explicit arguments :
!        --------------------
!        KFILE : an indicator for which spectral file is to be read  (I)
!        LDATA : .TRUE. if data is returned from file                (O)
!        PSPVOR etc. - spectral fields                               (O)
!        LDINOR - switch for initializing the orography              (I)

!        Implicit arguments :
!        --------------------
!        See modules above.

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------

!        see calls below.

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!        Note de travail ARPEGE Nr 12 et 17

!     Author.
!     -------
!        Ryad El Khatib *METEO-FRANCE*

!     Modifications.
!     --------------
!      Original     : 99-04-23 (replacement of suspecasm/dm)
!      R. El Khatib : 00-02-17 Prints for blending
!      M. SZCZECH   : 00-04-18 spectral thermodynamic ozon.
!      M. SZCZECH   : 00-10-04 additional test for ozon 
!      R. El Khatib : 00-08-18 derived type
!      J. Masek     : 01-06-25 rescaling of new NH variables
!      R. El Khatib : 01-05-10 Harmonization with wrspeca; rm blending (dead)
!      R. El Khatib : 01-08-07 Pruning options
!      C. Fischer   : 02-03-08 Removed test on n3dini when calling gnhpdvdconv
!      P. Smolikova : 02-09-30 rescaling of d4 if NH
!      K. YESSAD    : 02-11-14 some cleanings
!      R. El Khatib : 03-08-05 gfl
!      R. El Khatib : 03-04-17 Cleanups
!      R. El Khatib : 03-05-14 New interfaces euvgeovd
!      G. Hello     : supress the call to gnhpdvdconv at this level
!      M.Hamrud     : 01-Oct-2003 CY28 Cleaning
!      M.Hamrud     : 10-Jan-2004 CY28R1 Cleaning
!      D. Paradis & R. El Khatib : 04-07-22 GRIBEX
!      R. Brozkova  :  05-07-23 Handle the divergence "X" term not to be read
!      R. El Ouaraini & R. El Khatib : 05-07-28 : New fields for the monitoring
!                     of update frequency of the coupling files for Aladin
!      Y. Bouteloup : 05-10-18 YCVGQ not read even it is spectral
!      K. Yessad    : 28 Feb 2006: cleanings (useless dummies, indentations).
!      Y. Seity     : 23-06 2006: remove YCVGQ specificities and replace by more
!                     general tests on NREQIN
!     -------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMAFN   , ONLY : TFP_FIS
USE YOMLUN   , ONLY : NULOUT
USE YOMDIM   , ONLY : NFLEVG   ,NFLSUR   ,NSPEC2G  ,NSPEC2   ,&
 & NSEFRE   ,NF3D  
USE YOMDYNA  , ONLY : LNHX
USE YOMLAP   , ONLY : NVALUE   ,RLAPDI
USE YOMCT0   , ONLY : N3DINI   ,NPROC    ,LNHDYN   ,LELAM    ,NPRTRV   ,&
 & NPRTRW     
USE YOMMP    , ONLY : MYPROC   ,MYSETV   ,NSTRIN   ,NPTRLL   ,NBSETLEV ,&
 & NBSETSP  ,MYSETW  
USE YOMMPG   , ONLY : NPOSSP
USE YOMFA    , ONLY :   YFAPSI   ,YFAKHI   ,YFAT     ,YFAPD    ,YFAVD    , &
 & YFASP    ,YFAUGEO  ,YFAVGEO   ,YFAFSP1  ,YFAFSP2  ,YFAFSP3  ,YFAFSP4  , &
 & YFAFSP5  
USE YOM_YGFL , ONLY : YGFL     ,YGFLC
USE YOMMCUF  ,ONLY  : NCUFNR   ,LREACUF   ,RMCUFFP

!     -------------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFILE 
LOGICAL           ,INTENT(OUT)   :: LDATA 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPVOR(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPDIV(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPT(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPGFL(:,:,:) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPSP(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPOR(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPSPD(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPSVD(:,:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPUB(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPVB(:) 
LOGICAL           ,INTENT(IN)    :: LDINOR 

!     -------------------------------------------------------------------------

!     IFLDSPG: total number of fields to read
!     IFLDSCH: number of fields in each chunk
!     ICHUNKS: number of chunks of fields
!     ICHOFF : chunk offset for fields
!     INFD   : number of fields in the current chunk for each processor
!     IFLDOFF: fields offset after distribution among NSTRIN procs.
!     CLPREF : prefix of fields to read 
!     CLSUFF : suffix of fields to read
!     ILEVG  : global level of fields to read
!     ILEVL  : local level of fields to send/receive
!     IVSET  : vertical set each field belongs (for 2nd level of distribution)
!     IVDIS  : vertical set corresponding to each chunk
!     IFLDG  : field index in global arrays
!     ZDATA  : a spectral field read from file
!     ZSPBUF : a chunk of of spectral buffer fields to distribute ; 
!              NB : while sending, ZSPBUF contains global spectra ; 
!                   while receiving, it contain local spectra
!     ISPOFF : offset of spectral coefficient
!     JACTION: action : 0 to read and send ; 1 to receive.

INTEGER(KIND=JPIM) :: JCH, JV, JGFL, JSP, JFLD, JACTION, JROC, JLEV
INTEGER(KIND=JPIM) :: IFLDSPG, ICHUNKS, ICHOFF, IPT, ISPOFF, I3DINI
INTEGER(KIND=JPIM) :: IFLDG, ISEND, IRECV, IDEB, IFIN, IINDEX
INTEGER(KIND=JPIM) :: INFD(NPROC), IFLDOFF(NPROC), IREP, IUNTIN
INTEGER(KIND=JPIM) :: INGRIB, INBITS, ISTRON, IPUILA
INTEGER(KIND=JPIM) :: IDATEF(11)
LOGICAL   :: LLCOSP = .TRUE.
LOGICAL   :: LLFILE_TO_MODEL = .TRUE.
LOGICAL   :: LLINOR
LOGICAL   :: LLEXIST

INTEGER(KIND=JPIM), ALLOCATABLE :: ILEVG (:), IVSET (:), ILEVL (:), IVDIS(:) 
INTEGER(KIND=JPIM), ALLOCATABLE :: ILEVG2(:), IVSET2(:), ILEVL2(:), IFLDSCH(:)
CHARACTER (LEN = 4), ALLOCATABLE :: CLPREF(:), CLPREF2(:)
CHARACTER (LEN = 12), ALLOCATABLE :: CLSUFF(:), CLSUFF2(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPBUF(:,:), ZSPU(:,:), ZSPV(:,:), ZDATA(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSP(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     -------------------------------------------------------------------------

#include "euvgeovd.intfb.h"

#include "abor1.intfb.h"
#include "disspec0.intfb.h"
#include "openfa.intfb.h"
#include "spreord.intfb.h"
#include "sumpioh.intfb.h"
#include "suspsp.intfb.h"

!     -------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSPECA',0,ZHOOK_HANDLE)

!     -------------------------------------------------------------------------

!*       0. PRELIMINAR INITIALISATIONS
!           --------------------------

I3DINI=N3DINI
LLINOR=LDINOR

!     -------------------------------------------------------------------------

!*       1. FIND NUMBER OF FIELDS TO READ FROM FILE
!           ---------------------------------------

!        1.1 Open file

IF (KFILE == 4) THEN
  CALL ABOR1('SUSPECA : BLENDING REMOVED')
ELSE
  LDATA = .TRUE.
  IF (MYPROC <= NSTRIN) THEN
    CALL OPENFA(KFILE,IUNTIN,IDEB,IFIN,IINDEX,IDATEF)
    ALLOCATE(ZDATA(NSEFRE))
  ENDIF
ENDIF

!        1.2 Conditional allocations

IF (LELAM) THEN
  ALLOCATE(ZSPU(NFLSUR,NSPEC2))
  ALLOCATE(ZSPV(NFLSUR,NSPEC2))
ENDIF

!        1.3 Count fields

IFLDSPG=0
! Orography :
IF (LLINOR) THEN
  IFLDSPG=IFLDSPG+1
ENDIF

! Ps and 3D fields :
IF (I3DINI == 0)  THEN
  IF (LREACUF) THEN
    IFLDSPG = IFLDSPG + 1 + NF3D*NFLEVG + NCUFNR
  ELSE
    IFLDSPG = IFLDSPG + 1 + NF3D*NFLEVG 
  ENDIF
ENDIF
! Remove "X" divergence term from reading when set
IF (LNHX) THEN
  IFLDSPG = IFLDSPG - NFLEVG
ENDIF
! Remore Spectral GFL not Required in input
DO JGFL=1, YGFL%NUMFLDS
  IF (YGFLC(JGFL)%LSP) THEN
    IF (YGFLC(JGFL)%NREQIN/= 1 ) THEN
      IFLDSPG=IFLDSPG-NFLEVG
    ENDIF   
  ENDIF
ENDDO

!     -------------------------------------------------------------------------

!*       2. INITIALIZE FROM FILE
!           --------------------

IF (IFLDSPG > 0) THEN

!        2.1 Setup various local variables related to file

  ALLOCATE(CLPREF(IFLDSPG))
  ALLOCATE(CLSUFF(IFLDSPG))
  ALLOCATE(ILEVG (IFLDSPG))
  ALLOCATE(ILEVL (IFLDSPG))
  ALLOCATE(IVSET (IFLDSPG))
  IPT=0

  ! Orography :
  IF (LLINOR) THEN
    IPT=IPT+1
    CLPREF(IPT)=TFP_FIS%CLNAME(1:4)
    ILEVG (IPT)=0
    ILEVL (IPT)=0
    CLSUFF(IPT)=TFP_FIS%CLNAME(5:)
    IVSET (IPT)=NBSETSP
  ENDIF

  ! Ps and 3D fields :
  IF (I3DINI == 0) THEN
    IPT=IPT+1
    CLPREF(IPT)=YFASP%CLNAME(1:4)
    ILEVG (IPT)=0
    ILEVL (IPT)=0
    CLSUFF(IPT)=YFASP%CLNAME(5:)
    IVSET (IPT)=NBSETSP
    IF (LREACUF)   THEN
      IF (NCUFNR>=1) THEN
        IPT=IPT+1
        CLPREF(IPT)=YFAFSP1%CLNAME(1:4)
        ILEVG (IPT)=0
        ILEVL (IPT)=0
        CLSUFF(IPT)=YFAFSP1%CLNAME(5:)
        IVSET (IPT)=NBSETSP
      ENDIF
      IF (NCUFNR>=2) THEN
        IPT=IPT+1
        CLPREF(IPT)=YFAFSP2%CLNAME(1:4)
        ILEVG (IPT)=0
        ILEVL (IPT)=0
        CLSUFF(IPT)=YFAFSP2%CLNAME(5:)
        IVSET (IPT)=NBSETSP
      ENDIF
      IF (NCUFNR>=3) THEN
        IPT=IPT+1
        CLPREF(IPT)=YFAFSP3%CLNAME(1:4)
        ILEVG (IPT)=0
        ILEVL (IPT)=0
        CLSUFF(IPT)=YFAFSP3%CLNAME(5:)
        IVSET (IPT)=NBSETSP
      ENDIF
      IF (NCUFNR>=4) THEN
        IPT=IPT+1
        CLPREF(IPT)=YFAFSP4%CLNAME(1:4)
        ILEVG (IPT)=0
        ILEVL (IPT)=0
        CLSUFF(IPT)=YFAFSP4%CLNAME(5:)
        IVSET (IPT)=NBSETSP
      ENDIF
      IF (NCUFNR>=5) THEN
        IPT=IPT+1
        CLPREF(IPT)=YFAFSP5%CLNAME(1:4)
        ILEVG (IPT)=0
        ILEVL (IPT)=0
        CLSUFF(IPT)=YFAFSP5%CLNAME(5:)
        IVSET (IPT)=NBSETSP
      ENDIF
    ENDIF
    IF (NF3D > 0) THEN
      CLPREF(IPT+1:)='S   '
      DO JFLD=IPT+1, IFLDSPG
        ILEVG (JFLD) = MOD(JFLD-(IPT+1),NFLEVG) + 1
        IVSET (JFLD) = NBSETLEV(ILEVG(JFLD))
        ILEVL (JFLD) = ILEVG(JFLD)-NPTRLL(IVSET(JFLD))+1 
      ENDDO 
      ! * Psi & Khi or U & V :
      IF (LELAM) THEN
        CLSUFF(IPT+1:IPT+NFLEVG)=YFAUGEO%CLNAME(1:12)
        CLSUFF(IPT+NFLEVG+1:IPT+2*NFLEVG)=YFAVGEO%CLNAME(1:12)
      ELSE
        CLSUFF(IPT+1:IPT+NFLEVG)=YFAPSI%CLNAME(1:12)
        CLSUFF(IPT+NFLEVG+1:IPT+2*NFLEVG)=YFAKHI%CLNAME(1:12)
      ENDIF
      IPT=IPT+2*NFLEVG
      ! * Temperature :
      CLSUFF(IPT+1:IPT+NFLEVG)=YFAT%CLNAME(1:12)
      IPT=IPT+NFLEVG
      ! * Pressure departure & Vertical divergence:
      IF (LNHDYN) THEN
        CLSUFF(IPT+1:IPT+NFLEVG)=YFAPD%CLNAME(1:12)
        CLSUFF(IPT+NFLEVG+1:IPT+2*NFLEVG)=YFAVD%CLNAME(1:12)
        IPT=IPT+2*NFLEVG
      ENDIF
      ! * other prognostic variables :
      DO JGFL=1, YGFL%NUMFLDS
        IF (YGFLC(JGFL)%LSP) THEN
          IF (YGFLC(JGFL)%NREQIN==1 ) THEN
            CLSUFF(IPT+1:IPT+NFLEVG)=YGFLC(JGFL)%CNAME(1:12)
            IPT=IPT+NFLEVG
          ENDIF   
        ENDIF
      ENDDO
    ENDIF
  ENDIF

  WRITE(NULOUT,'(1X,I5,1X,I5)') IPT,IFLDSPG

  IF (IPT /= IFLDSPG) THEN
    CALL ABOR1('SUSPECA : FIELDS DESCRIPTORS ARE UNCOMPLETE !')
  ENDIF 

!        2.2 Setup chunks of fields

  ! Within a chunk, all fields must belong to the same V-set
  IF (NPROC > 1) THEN
    ! * Spare communications
    ICHUNKS=NPRTRV
    ALLOCATE(IFLDSCH(NPRTRV))
    ALLOCATE(IVDIS(NPRTRV))
    IF (NPRTRV > 1) THEN
      ! * Reorder descriptors array and count the number of fields per chunk
      ALLOCATE(CLPREF2(IFLDSPG)) ; CLPREF2(:)=CLPREF(:)
      ALLOCATE(CLSUFF2(IFLDSPG)) ; CLSUFF2(:)=CLSUFF(:)
      ALLOCATE(ILEVG2 (IFLDSPG)) ; ILEVG2 (:)=ILEVG (:)
      ALLOCATE(ILEVL2 (IFLDSPG)) ; ILEVL2 (:)=ILEVL (:)
      ALLOCATE(IVSET2 (IFLDSPG)) ; IVSET2 (:)=IVSET (:)
      IFLDG=0
      DO JV=1, NPRTRV
        IVDIS(JV)=JV
        IFLDSCH(JV)=0
        DO JFLD=1, IFLDSPG
          IF (IVSET2(JFLD) == JV) THEN
            IFLDG=IFLDG+1
            CLPREF(IFLDG)=CLPREF2(JFLD)
            CLSUFF(IFLDG)=CLSUFF2(JFLD)
            ILEVG (IFLDG)=ILEVG2 (JFLD)
            ILEVL (IFLDG)=ILEVL2 (JFLD)
            IVSET (IFLDG)=IVSET2 (JFLD)
            IFLDSCH(JV)=IFLDSCH(JV)+1
          ENDIF
        ENDDO
      ENDDO
      DEALLOCATE(CLPREF2)
      DEALLOCATE(CLSUFF2)
      DEALLOCATE(ILEVG2 )
      DEALLOCATE(ILEVL2 )
      DEALLOCATE(IVSET2 )
    ELSE
      IFLDSCH(:)=IFLDSPG
      IVDIS(:)=1
    ENDIF
    ISEND=0
    IRECV=1
  ELSE
    ! * Spare memory
    ICHUNKS=IFLDSPG
    ALLOCATE(IFLDSCH(ICHUNKS))
    IFLDSCH(:)=1
    ALLOCATE(IVDIS(ICHUNKS))
    IVDIS(:)=1
    ISEND=1
    IRECV=1
  ENDIF

!        2.3 Read and distribute data

  DO JACTION=ISEND,IRECV

    ICHOFF=0

    DO JCH=1, ICHUNKS

      !*   2.3.1 Setup distribution among I/O processors

      CALL SUMPIOH(IFLDSCH(JCH),NPROC,NSTRIN,INFD,IFLDOFF)

      !*   2.3.2 I/O processors read and send fields to set-V processors

      IF (JACTION == ISEND .AND. INFD(MYPROC) > 0) THEN     

        ALLOCATE(ZSPBUF(NSPEC2G,INFD(MYPROC)))
        DO JFLD=1,INFD(MYPROC) 
          IFLDG=ICHOFF+IFLDOFF(MYPROC)+JFLD
          CALL FANION(IREP,IUNTIN,CLPREF(IFLDG),ILEVG(IFLDG),CLSUFF(IFLDG),&
           & LLEXIST,LLCOSP,INGRIB,INBITS,ISTRON,IPUILA)
          IF (.NOT.(INGRIB==-1 .OR. INGRIB==3)) THEN
            CALL FACILE(IREP,IUNTIN,CLPREF(IFLDG),ILEVG(IFLDG),CLSUFF(IFLDG),&
             & ZDATA,LLCOSP)
            CALL SPREORD(1,ZDATA,ZSPBUF(1,JFLD),LLFILE_TO_MODEL)
          ELSE
            CALL FACILE(IREP,IUNTIN,CLPREF(IFLDG),ILEVG(IFLDG),CLSUFF(IFLDG),&
             & ZSPBUF(1,JFLD),LLCOSP)
          ENDIF
        ENDDO
        IF (NPROC > 1) THEN
          CALL DISSPEC0(MYPROC,IVDIS(JCH),JCH,ISEND,NSPEC2G,INFD(MYPROC),&
           & NPOSSP,NPRTRW,ZSPBUF)  
        ENDIF
        IF (IVDIS(JCH) /= MYSETV) THEN
          ! Release array because we are not the same V-set (nothing more to do)
          DEALLOCATE(ZSPBUF)
        ENDIF

      ENDIF

      !*   2.3.3 set-V procs (possibly receive and) split the spectral buffer

      IF (IVDIS(JCH) == MYSETV) THEN

        ! Action because we are the proper V-set (actual reception)
        DO JROC=1,NSTRIN
          IF (JACTION == ISEND .AND. JROC == MYPROC) THEN
            ! I fill my own buffer :
            ISPOFF=NPOSSP(MYSETW)-1
          ELSEIF(JACTION == IRECV .AND. JROC /= MYPROC .AND. INFD(JROC)>0) THEN
            ! I receive from the others :
            ISPOFF=0
            ALLOCATE(ZSPBUF(NSPEC2,INFD(JROC)))
            CALL DISSPEC0(JROC,MYSETV,JCH,IRECV,NSPEC2,INFD(JROC),NPOSSP,&
             & NPRTRW,ZSPBUF)  
          ELSE
            CYCLE
          ENDIF
          DO JFLD=1,INFD(JROC)
            IFLDG=ICHOFF+IFLDOFF(JROC)+JFLD
            IF (CLSUFF(IFLDG) == TFP_FIS%CLNAME(5:)) THEN
              ! Orography :
              DO JSP=1,NSPEC2
                PSPOR(JSP)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFASP%CLNAME(5:)) THEN
              ! Surface pressure :
              DO JSP=1,NSPEC2
                PSPSP(JSP)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAFSP1%CLNAME(5:) &
               & .AND. NCUFNR >=1 .AND. LREACUF) THEN
              ! Filtered ln Ps nr 1 for monitoring the coupling updates
              DO JSP=1,NSPEC2
                RMCUFFP(JSP,0,1)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAFSP2%CLNAME(5:) &
               & .AND. NCUFNR >=2 .AND. LREACUF) THEN
              ! Filtered ln Ps nr 2 for monitoring the coupling updates
              DO JSP=1,NSPEC2
                RMCUFFP(JSP,0,2)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAFSP3%CLNAME(5:) &
               & .AND. NCUFNR >=3 .AND. LREACUF) THEN
              ! Filtered ln Ps nr 3 for monitoring the coupling updates
              DO JSP=1,NSPEC2
                RMCUFFP(JSP,0,3)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAFSP4%CLNAME(5:) &
               & .AND. NCUFNR >=4 .AND. LREACUF) THEN
              ! Filtered ln Ps nr 4 for monitoring the coupling updates
              DO JSP=1,NSPEC2
                RMCUFFP(JSP,0,4)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAFSP5%CLNAME(5:) &
               & .AND. NCUFNR >=5 .AND. LREACUF) THEN
              ! Filtered ln Ps nr 5 for monitoring the coupling updates
              DO JSP=1,NSPEC2
                RMCUFFP(JSP,0,5)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAUGEO%CLNAME(1:12)) THEN
              ! Wind/x momentum
              DO JSP=1,NSPEC2
                ZSPU(ILEVL(IFLDG),JSP)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAVGEO%CLNAME(1:12)) THEN
              ! Wind/y momentum :
              DO JSP=1,NSPEC2
                ZSPV(ILEVL(IFLDG),JSP)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAPSI%CLNAME(1:12)) THEN
              ! Psi : 
              DO JSP=1,NSPEC2
                PSPDIV(ILEVL(IFLDG),JSP)=ZSPBUF(JSP+ISPOFF,JFLD)*&
                 & RLAPDI(NVALUE(JSP))  
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAKHI%CLNAME(1:12)) THEN
              ! Khi :
              DO JSP=1,NSPEC2
                PSPVOR(ILEVL(IFLDG),JSP)=ZSPBUF(JSP+ISPOFF,JFLD)*&
                 & RLAPDI(NVALUE(JSP))  
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAT%CLNAME(1:12)) THEN
              ! Temperature :
              DO JSP=1,NSPEC2
                PSPT(ILEVL(IFLDG),JSP)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAPD%CLNAME(1:12)) THEN
              ! NH Pressure departure :
              DO JSP=1,NSPEC2
                PSPSPD(ILEVL(IFLDG),JSP)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSEIF (CLSUFF(IFLDG) == YFAVD%CLNAME(1:12)) THEN
              ! NH vertical divergence :
              DO JSP=1,NSPEC2
                PSPSVD(ILEVL(IFLDG),JSP)=ZSPBUF(JSP+ISPOFF,JFLD)
              ENDDO
            ELSE
              ! Other prognostic variables :
              DO JGFL=1, YGFL%NUMFLDS
                IF (YGFLC(JGFL)%LSP) THEN
                  IF (CLSUFF(IFLDG) == YGFLC(JGFL)%CNAME(1:12).AND.&
                     & YGFLC(JGFL)%NREQIN==1) THEN
                    DO JSP=1,NSPEC2
                      PSPGFL(ILEVL(IFLDG),JSP,YGFLC(JGFL)%MPSP) = &
                       & ZSPBUF(JSP+ISPOFF,JFLD)  
                    ENDDO
                  ENDIF  
                ENDIF
              ENDDO
            ENDIF
          ENDDO
          
          IF( (INFD(MYPROC) > 0 .AND. JACTION == ISEND) .OR. &
             & (JACTION == IRECV .AND. INFD(JROC) > 0) ) THEN
            DEALLOCATE(ZSPBUF)
          ENDIF
          IF (JACTION == ISEND .AND. JROC == MYPROC) THEN
            ! My waves are filled by myself :
            EXIT
          ENDIF
        ENDDO

      ENDIF

      ICHOFF=ICHOFF+IFLDSCH(JCH)

    ENDDO ! JCH

  ENDDO ! JACTION
  
  !special treatment fof Spectral GFL not read in initial file
  DO JGFL=1, YGFL%NUMFLDS
    IF (YGFLC(JGFL)%LSP) THEN
      IF ( YGFLC(JGFL)%NREQIN== -1) THEN
        DO JLEV=1,NFLEVG
          DO JSP=2,NSPEC2
            PSPGFL(JLEV,JSP,YGFLC(JGFL)%MPSP) = 0._JPRB
          ENDDO  
          PSPGFL(JLEV,1,YGFLC(JGFL)%MPSP)=YGFLC(JGFL)%REFVALI
        ENDDO  
      ENDIF  
    ENDIF
  ENDDO
  DO JGFL=1, YGFL%NUMFLDS
    IF (YGFLC(JGFL)%LSP) THEN
      IF ( YGFLC(JGFL)%NREQIN== 0) THEN
        DO JLEV=1,NFLEVG
          DO JSP=1,NSPEC2
            PSPGFL(ILEVL(IFLDG),JSP,YGFLC(JGFL)%MPSP) = 0._JPRB
          ENDDO 
        ENDDO  
      ENDIF  
    ENDIF
  ENDDO

!       2.4 Release descriptors

  DEALLOCATE(CLPREF)
  DEALLOCATE(CLSUFF)
  DEALLOCATE(ILEVG )
  DEALLOCATE(ILEVL )
  DEALLOCATE(IVSET )
  DEALLOCATE(IFLDSCH)
  DEALLOCATE(IVDIS)

!       2.5 Ensure the pronostic variable is Ln(Ps)

  ! unless KFILE=3 (file contains increments)
  IF (KFILE /= 3) THEN
    IF (I3DINI == 0) THEN
      ! Copy argument to ZSP to avoid an explicit interface 
      ALLOCATE(ZSP(NSPEC2))
      ZSP(:)=PSPSP(:)
      CALL SUSPSP(ZSP)
      PSPSP(:)=ZSP(:)
      DEALLOCATE(ZSP)
    ENDIF
  ENDIF

!       2.6 Compute Vorticity, Divergence & mean wind from geographical wind

  IF (LELAM) THEN
    WRITE (NULOUT,*)' SUSPECA : U.GEO,V.GEO --> VOR,DIV + MEAN U,V '
    CALL EUVGEOVD(ZSPU,ZSPV,PSPVOR,PSPDIV,PSPUB,PSPVB)
  ENDIF

ENDIF ! (IFLDSPG > 0)

!       2.7 Release conditionnally allocated arrays

IF (LELAM) THEN
  DEALLOCATE(ZSPU)
  DEALLOCATE(ZSPV)
ENDIF

!       2.8 Close file and release descriptors

IF (MYPROC <= NSTRIN) THEN
  CALL FAIRME(IREP,IUNTIN,'UNKNOWN')
  DEALLOCATE(ZDATA)
ENDIF

!     -------------------------------------------------------------------------

!*       3. INITIALIZE WITH ARTIFICIAL DATA
!           -------------------------------

! This can never happen, suspeca is only called if N3DINI == 0 /MH
!!$IF (I3DINI /= 0) THEN
!!$  LLINOR=.FALSE.
!!$  IF (LELAM .OR. LNHDYN) THEN
!!$    CALL ABOR1('SUSPECA : MISSING CODE IN SUSPECG1')
!!$  ENDIF
!!$  CALL SUSPECG1(PSPVOR,PSPDIV,PSPT,PSPGFL,PSPSP,PSPOR,&
!!$   & LLINOR,LLSPOR)  
!!$ENDIF

!     -------------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSPECA',1,ZHOOK_HANDLE)
END SUBROUTINE SUSPECA
