SUBROUTINE LCNORAVAR (KSTART,KPROF,PGELAM,PGELAT,PGM,&
 & PUT0,PVT0,PTT0,PQT0,PSPT0,&
 & PUPP,PVPP,PTPP,PQPP,PSPPP)  

!**** *LCNORAVAR* - apply mask to the avarc correction

!     PURPOSE.
!     --------
!      To multiply the input field by a weight function in gridpoint space.

!**   INTERFACE.
!     ----------
!        *CALL* *LCNORAVAR(..)

!        EXPLICIT ARGUMENTS :
!        --------------------
!        KSTART    : first point of computations               (input)
!        KPROF     : last  point of computations               (input)
!        PGELAM    : longitude on the real earth               (input)
!        PGELAT    : latitude on the real earthh               (input)
!        PUT0      : zonal wind time t                         (input)
!        PVT0      : meridional wind time t                    (input)
!        PTT0      : temperature time t                        (input)
!        PQT0      : humidity time t                           (input)
!        PSPT0     : surface pressure variable time t          (input)

!        PUPP      : zonal wind postprocessed                  (output)
!        PVPP      : meridional wind postprocessed             (output)
!        PTPP      : temperature postprocessed                 (output)
!        PQPP      : humidity postprocessed                    (output)
!        PSPPP     : surface pressure variable postprocessed   (output)

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!     The mask is defined :
!     horizontally - by the south (AVAS) and the north (AVAN) latitude
!                  - by the west  (AVAW) and the east  (AVAE) longitude
!     (Greenwich has lon=0.0, and the long goes from 0.0 to 2.*RPI
!        Be aware that only gridpoints within the mask will be selected, so
!     don't try to use it for small areas.

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------
!        ECMWF Research Department Technical Memos nos.196 and 197

!     AUTHOR.
!     -------
!        Radmila Brozkova                   *LACE*
!        (after Buizza)

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2004-08-07
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPROMA, NDLON
USE YOMCST   , ONLY : RPI
USE YOMHCP   , ONLY : AVASL, AVANL, AVAWL, AVAEL
IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPT0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUPP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVPP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTPP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQPP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PSPPP(NPROMA) 
REAL(KIND=JPRB) :: ZMASK(NPROMA)

INTEGER(KIND=JPIM) :: JL, JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZAVASS, ZAVANN, ZAVAWW, ZAVAEE, ZDEL, ZDELA, ZDELANG

!     ------------------------------------------------------------------

!*    1. Define the gg mask.
!     ----------------------

IF (LHOOK) CALL DR_HOOK('LCNORAVAR',0,ZHOOK_HANDLE)

! set-up the interval to smooth the mask at the length of 10 grid-points
  ZDELA=20._JPRB*RPI/(FLOAT(NDLON))

  ZAVASS = MAX(AVASL - ZDELA,-0.5_JPRB*RPI)
  ZAVANN = MIN(AVANL + ZDELA,0.5_JPRB*RPI)
  ZAVAWW = AVAWL - ZDELA
  IF (ZAVAWW < 0._JPRB) THEN
    ZAVAWW = ZAVAWW + 2._JPRB*RPI
  ENDIF
  ZAVAEE = AVAEL + ZDELA
  IF (ZAVAEE > 2._JPRB*RPI) THEN
    ZAVAEE = ZAVAEE - 2._JPRB*RPI
  ENDIF
  

ZMASK(:)=0.0_JPRB

DO JROF=KSTART,KPROF

!inner part: mask is 1. and north-south intermediate parts (0.< mask < 1.)
  IF(AVAWL< AVAEL.AND.(PGELAM(JROF) >= AVAWL.AND.PGELAM(JROF) <= AVAEL)) THEN
    IF ( PGELAT(JROF) >= AVASL.AND.PGELAT(JROF) <= AVANL ) THEN
      ZMASK(JROF)=1.0_JPRB
! south intermediate
    ELSEIF ( PGELAT(JROF) >= ZAVASS.AND.PGELAT(JROF) <= AVASL ) THEN
      ZDELANG= 1._JPRB - (AVASL-PGELAT(JROF))/ZDELA
  ZMASK(JROF)=MAX(0._JPRB,MIN(1.0_JPRB,3._JPRB*ZDELANG**2-2._JPRB*ZDELANG**3))
! north intermediate
    ELSEIF ( PGELAT(JROF) >= AVANL.AND.PGELAT(JROF) <= ZAVANN ) THEN
      ZDELANG= 1._JPRB - (PGELAT(JROF)-AVANL)/ZDELA
  ZMASK(JROF)=MAX(0._JPRB,MIN(1.0_JPRB,3._JPRB*ZDELANG**2-2._JPRB*ZDELANG**3))
    ENDIF
  ENDIF
  IF(AVAWL > AVAEL.AND.((PGELAM(JROF) >= AVAWL.AND.&
    &                  PGELAM(JROF) <= 2.0_JPRB*RPI).OR.&
    & (PGELAM(JROF) >= 0.0_JPRB  .AND.PGELAM(JROF) <= AVAEL)) ) THEN  
    IF ( PGELAT(JROF) >= AVASL.AND.PGELAT(JROF) <= AVANL ) THEN
      ZMASK(JROF)=1.0_JPRB
! south intermediate
    ELSEIF ( PGELAT(JROF) >= ZAVASS.AND.PGELAT(JROF) <= AVASL ) THEN
      ZDELANG= 1._JPRB - (AVASL-PGELAT(JROF))/ZDELA
  ZMASK(JROF)=MAX(0._JPRB,MIN(1.0_JPRB,3._JPRB*ZDELANG**2-2._JPRB*ZDELANG**3))
! north intermediate
    ELSEIF ( PGELAT(JROF) >= AVANL.AND.PGELAT(JROF) <= ZAVANN ) THEN
      ZDELANG= 1._JPRB - (PGELAT(JROF)-AVANL)/ZDELA
  ZMASK(JROF)=MAX(0._JPRB,MIN(1.0_JPRB,3._JPRB*ZDELANG**2-2._JPRB*ZDELANG**3))
    ENDIF
  ENDIF
!west-band intermediate part: mask is between 0. and 1.
  IF(ZAVAWW < AVAWL) THEN
    IF ( PGELAT(JROF) >= ZAVASS.AND.PGELAT(JROF) <= ZAVANN.AND.&
       & (PGELAM(JROF) >= ZAVAWW.AND.PGELAM(JROF) <= AVAWL) ) THEN  
      ZDELANG= 1._JPRB - (AVAWL-PGELAM(JROF))/ZDELA
  ZMASK(JROF)=MAX(0._JPRB,MIN(1.0_JPRB,3._JPRB*ZDELANG**2-2._JPRB*ZDELANG**3))
    ENDIF
  ELSE
    IF ( PGELAT(JROF) >= ZAVASS.AND.PGELAT(JROF) <= ZAVANN.AND.&
       & ((PGELAM(JROF) >= ZAVAWW.AND.PGELAM(JROF) <= 2.0_JPRB*RPI).OR.&
       & (PGELAM(JROF) >= 0.0_JPRB  .AND.PGELAM(JROF) <= AVAWL)) ) THEN  
      ZDEL = AVAWL-PGELAM(JROF)
      IF (ZDEL < 0.0_JPRB) ZDEL = ZDEL + 2.0_JPRB*RPI
      ZDELANG= 1._JPRB - ZDEL/ZDELA
  ZMASK(JROF)=MAX(0._JPRB,MIN(1.0_JPRB,3._JPRB*ZDELANG**2-2._JPRB*ZDELANG**3))
    ENDIF
  ENDIF
!east-band intermediate part: mask is between 0. and 1.
  IF(AVAEL < ZAVAEE) THEN
    IF ( PGELAT(JROF) >= ZAVASS.AND.PGELAT(JROF) <= ZAVANN.AND.&
       & (PGELAM(JROF) >= AVAEL.AND.PGELAM(JROF) <= ZAVAEE) ) THEN  
      ZDELANG= 1._JPRB - (PGELAM(JROF)-AVAEL)/ZDELA
  ZMASK(JROF)=MAX(0._JPRB,MIN(1.0_JPRB,3._JPRB*ZDELANG**2-2._JPRB*ZDELANG**3))
    ENDIF
  ELSE
    IF ( PGELAT(JROF) >= ZAVASS.AND.PGELAT(JROF) <= ZAVANN.AND.&
       & ((PGELAM(JROF) >= AVAEL.AND.PGELAM(JROF) <= 2.0_JPRB*RPI).OR.&
       & (PGELAM(JROF) >= 0.0_JPRB  .AND.PGELAM(JROF) <= ZAVAEE)) ) THEN  
      ZDEL = PGELAM(JROF)-AVAEL
      IF (ZDEL < 0.0_JPRB) ZDEL = ZDEL + 2.0_JPRB*RPI
      ZDELANG= 1._JPRB - ZDEL/ZDELA
  ZMASK(JROF)=MAX(0._JPRB,MIN(1.0_JPRB,3._JPRB*ZDELANG**2-2._JPRB*ZDELANG**3))
    ENDIF
  ENDIF
ENDDO


!*    2. Multiply the input field selectively by the mask.
!     ----------------------------------------------------

!      line to select surface pressure
DO  JROF=KSTART,KPROF
    PSPPP(JROF)=ZMASK(JROF)*PSPT0(JROF)
ENDDO

!* initialize output vector

DO JLEV=1,NFLEVG
  DO JL=1,NPROMA
    PUPP(JL,JLEV)=0.0_JPRB
    PVPP(JL,JLEV)=0.0_JPRB
    PTPP(JL,JLEV)=0.0_JPRB
    PQPP(JL,JLEV)=0.0_JPRB
  ENDDO
ENDDO

DO JLEV=1,NFLEVG
  DO JL=KSTART,KPROF
!        lines to select other parameters
    PUPP(JL,JLEV)=ZMASK(JL)*PUT0(JL,JLEV)
    PVPP(JL,JLEV)=ZMASK(JL)*PVT0(JL,JLEV)
    PTPP(JL,JLEV)=ZMASK(JL)*PTT0(JL,JLEV)
    PQPP(JL,JLEV)=ZMASK(JL)*PQT0(JL,JLEV)
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LCNORAVAR',1,ZHOOK_HANDLE)
END SUBROUTINE LCNORAVAR
