SUBROUTINE LCNORGGAD (KSTART,KPROF,PGELAM,PGELAT,PGM,&
 & PUT0,PVT0,PTT0,PQT0,PSPT0,&
 & PUPP,PVPP,PTPP,PQPP,PSPPP)  

!**** *LCNORGGAD* - field localization

!     PURPOSE.
!     --------
!      To multiply the input field by a weight function in gridpoint space.

!**   INTERFACE.
!     ----------
!        *CALL* *LCNORGGAD(..)

!        EXPLICIT ARGUMENTS :
!        --------------------
!        KSTART    : first point of computations               (input)
!        KPROF     : last  point of computations               (input)
!        PGELAM    : longitude on the real earth               (input)
!        PGELAT    : latitude on the real earthh               (input)
!        PUT0      : zonal wind time t                         (input)
!        PVT0      : meridional wind time t                    (input)
!        PTT0      : temperature time t                        (input)
!        PQT0      : humidity time t                           (input)
!        PSPT0     : surface pressure variable time t          (input)

!        PUPP      : zonal wind postprocessed                  (output)
!        PVPP      : meridional wind postprocessed             (output)
!        PTPP      : temperature postprocessed                 (output)
!        PQPP      : humidity postprocessed                    (output)
!        PSPPP     : surface pressure variable postprocessed   (output)

!        IMPLICIT ARGUMENTS
!        --------------------

!     METHOD.
!     -------
!     The mask is defined :
!     - vertically: by the min (NLEVMIN) and max (NMAXLEV) level number
!       (the top of the atm has lev=1, the surface level lev=NFLEVG);
!     - horizontally: it is defined by the NW (point 1) and the SE
!       (point 3) corners.
!     So we have: - by the south (ALAT3) and the north (ALAT1) latitude
!                 - by the west (ALON1) and the east (ALON3) longitude
!     (Greenwich has lon=0.0, and the long goes from 0.0 to 2.*RPI
!     - by variable type : LSELU,LSELV,LSELT,LSELQ,LSELSP for U,V,T,q,Ps
!        Be aware that only gridpoints within the mask will be selected, so
!     don't try to use it for small areas.

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------
!        ECMWF Research Department Technical Memos nos.196 and 197

!     AUTHOR.
!     -------
!        Roberto Buizza                     *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 93-02-03
!        P.Courtier 93-06-18 angles in radian
!        R. Buizza: 93-10-25
!        F. Bouttier: 94-02-08 add variable selection in DEFMASK
!        M.Hamrud   : 94-10-31 Cleaning
!        L.Isaksen  : 96-02-02 Call to DEFMASK avoided, DM option added
!        G.Hello    : 97-08-04 map factor for linear cost function
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMLCZ   , ONLY : NLEVMIN  ,NLEVMAX  ,ALAT1    ,ALON1    ,&
 & ALAT3    ,ALON3    ,LSELU    ,LSELV    ,LSELT    ,&
 & LSELQ    ,LSELSP  ,LRAIN    ,RLRAIN  
USE YOMCST   , ONLY : RPI
USE YOMSENS  , ONLY : NJROPT

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAT(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PUT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PQT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPT0(NPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PUPP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PVPP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTPP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PQPP(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPPP(NPROMA) 
REAL(KIND=JPRB) :: ZMASK(NPROMA)

INTEGER(KIND=JPIM) :: JL, JLEV, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!*    1. Define the gg mask.
!     ----------------------

IF (LHOOK) CALL DR_HOOK('LCNORGGAD',0,ZHOOK_HANDLE)
DO JROF=KSTART,KPROF
  IF(ALON1 < ALON3) THEN
    IF ( PGELAT(JROF) >= ALAT3.AND.PGELAT(JROF) <= ALAT1.AND.&
       & (PGELAM(JROF) >= ALON1.AND.PGELAM(JROF) <= ALON3) ) THEN  
      ZMASK(JROF)=1.0_JPRB
    ELSE
      ZMASK(JROF)=0.0_JPRB
    ENDIF
  ELSE
    IF ( PGELAT(JROF) >= ALAT3.AND.PGELAT(JROF) <= ALAT1.AND.&
       & ((PGELAM(JROF) >= ALON1.AND.PGELAM(JROF) <= 2.0_JPRB*RPI).OR.&
       & (PGELAM(JROF) >= 0.0_JPRB  .AND.PGELAM(JROF) <= ALON3)) ) THEN  
      ZMASK(JROF)=1.0_JPRB
    ELSE
      ZMASK(JROF)=0.0_JPRB
    ENDIF
  ENDIF
ENDDO
IF(NJROPT == 2) THEN
  DO JROF=KSTART,KPROF
    ZMASK(JROF)=ZMASK(JROF)/PGM(JROF)**2
  ENDDO
ENDIF

!*    2. Multiply the input field selectively by the mask.
!     ----------------------------------------------------

!      line to select surface pressure
DO  JL=KSTART,KPROF
  IF(LSELSP) THEN
    PSPT0(JL)=PSPT0(JL)+ZMASK(JL)*PSPPP(JL)
  ELSE
    PSPT0(JL)=0.0_JPRB
    IF(LRAIN)  RLRAIN(JL)=ZMASK(JL)*RLRAIN(JL)
  ENDIF
  PSPPP(JL)=0.0_JPRB
ENDDO

!* initialize output vector

DO JLEV=1,NFLEVG
  DO JL=1,NPROMA
    PUT0(JL,JLEV)=0.0_JPRB
    PVT0(JL,JLEV)=0.0_JPRB
    PTT0(JL,JLEV)=0.0_JPRB
    PQT0(JL,JLEV)=0.0_JPRB
  ENDDO
ENDDO

DO JLEV=MAX(NLEVMIN,1),MIN(NLEVMAX,NFLEVG)
  DO JL=KSTART,KPROF
!        lines to select other parameters
    IF(LSELU) PUT0(JL,JLEV)=PUT0(JL,JLEV)+ZMASK(JL)*PUPP(JL,JLEV)
    IF(LSELV) PVT0(JL,JLEV)=PVT0(JL,JLEV)+ZMASK(JL)*PVPP(JL,JLEV)
    IF(LSELT) PTT0(JL,JLEV)=PTT0(JL,JLEV)+ZMASK(JL)*PTPP(JL,JLEV)
    IF(LSELQ) PQT0(JL,JLEV)=PQT0(JL,JLEV)+ZMASK(JL)*PQPP(JL,JLEV)

    PUPP(JL,JLEV)=0.0_JPRB
    PVPP(JL,JLEV)=0.0_JPRB
    PTPP(JL,JLEV)=0.0_JPRB
    PQPP(JL,JLEV)=0.0_JPRB
  ENDDO
ENDDO

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('LCNORGGAD',1,ZHOOK_HANDLE)
END SUBROUTINE LCNORGGAD
