SUBROUTINE LCZTOALD(KTYPE,YD_PVIN,YD_PVOUT)

!**** *LCZTOALD*

!     Purpose.
!     --------
!           Coordinate transformation from Lanczos space to Aladin space.
!           KTYPE=1: from Lanczos to Aladin space
!           KTYPE=2: inverse
!           KTYPE=3: square of KTYPE=1
!           KTYPE=4: square of KTYPE=2

!**   Interface.
!     ----------
!        *CALL* *LCZTOALD*

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation
!cf      Not implemented for distributed memory for the time being

!     Reference.
!     ----------
!        ECMWF research department documentation of the
!     "in core model"
!        Aladin Technical notes (C. Fischer or G. Radnoti)

!     Author.
!     -------
!        Claude Fischer :                   *Meteo-France*
!         after routine LCZTOIFS (CY18T3)

!     Modifications.
!     --------------
!        Original  : 98-05-12
!        C. Fischer  99-11-05 correction of ZFACTOR (transpose of spectral
!                             array reordering)
!        C. Fischer  03-03-31 Use control_vectors.
!        M.Leutbecher: 03-04-23 Bug fix
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMLAP   , ONLY : MYMS
USE YOMCVA   , ONLY : NGRBVAR
USE SPECTRAL_FIELDS_MOD
USE CONTROL_VECTORS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPE 
TYPE(CONTROL_VECTOR),INTENT(IN)    :: YD_PVIN 
TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YD_PVOUT 
TYPE(spectral_field) :: YL_ZSPECIN,YL_ZSPECOUT
INTEGER(KIND=JPIM) :: III,JM,JNM,IN,IM,JF
REAL(KIND=JPRB) :: ZZ
REAL(KIND=JPRB), ALLOCATABLE :: ZFACTOR(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('LCZTOALD',0,ZHOOK_HANDLE)
IF(KTYPE < 1.OR.KTYPE > 4) THEN
  WRITE(NULOUT,'('' >> ERROR IN LCZTOALD BAD KTYPE '')')
  CALL ABOR1('LCZTOALD: ABOR1 CALLED')
ENDIF

CALL allocate_spec(YL_ZSPECIN ,YD_PVIN%nsmax,YD_PVIN%nmsmax,YD_PVIN%ns3d,YD_PVIN%ns2d,YD_PVIN%ns1d,NGRBVAR)
CALL allocate_spec(YL_ZSPECOUT,YD_PVIN%nsmax,YD_PVIN%nmsmax,YD_PVIN%ns3d,YD_PVIN%ns2d,YD_PVIN%ns1d,NGRBVAR)

IF (.NOT.YD_PVIN%linitcv) CALL ABOR1('LCZTOALD: No initial condition')
YL_ZSPECIN=YD_PVIN%initcv(:)

!     1. Calculate the weighting factors
!     ----------------------------------

III=0
DO JM=1,YL_ZSPECIN%nump
  im=MYMS(jm)
  DO JNM=1,YL_ZSPECIN%ncpl4m(im)
    IN=(JNM-1)/4
    III=III+1
    ZZ=REAL(2**MIN(1,IM),JPRB)*REAL(2**MIN(1,IN),JPRB)
    IF(KTYPE == 1) THEN
      ZFACTOR(III)=SQRT( 1.0_JPRB/ZZ)
    ELSEIF(KTYPE == 2) THEN
      ZFACTOR(III)=SQRT(ZZ)
    ELSEIF(KTYPE == 3) THEN
      ZFACTOR(III)=1.0_JPRB/ZZ
    ELSEIF(KTYPE == 4) THEN
      ZFACTOR(III)=ZZ
    ENDIF
  ENDDO
ENDDO

!     2. Multiply by weighting factors
!     --------------------------------

DO jf=1,YL_ZSPECIN%ns3d
  DO jnm=1,YL_ZSPECIN%nspec2
    YL_ZSPECOUT%sp3d(:,jnm,jf) = ZFACTOR(jnm) * YL_ZSPECIN%sp3d(:,jnm,jf)
  ENDDO
ENDDO

DO jf=1,YL_ZSPECIN%ns2d
  DO jnm=1,YL_ZSPECIN%nspec2
    YL_ZSPECOUT%sp2d(jnm,jf) = ZFACTOR(jnm) * YL_ZSPECIN%sp2d(jnm,jf)
  ENDDO
ENDDO

DEALLOCATE(ZFACTOR)

YD_PVOUT%initcv(:)=YL_ZSPECOUT
IF (YD_PVIN%nmoderr>0) YD_PVOUT%moderr(:) = YD_PVIN%moderr(:)
IF (YD_PVIN%ltovscv  ) YD_PVOUT%tovscv(:) = YD_PVIN%tovscv(:)
IF (YD_PVIN%lam1d    ) YD_PVOUT%lamcv (:) = YD_PVIN%lamcv (:)
IF (YD_PVIN%npara>0  ) YD_PVOUT%params(:) = YD_PVIN%params(:)

CALL DEALLOCATE_SPEC(YL_ZSPECIN)
CALL DEALLOCATE_SPEC(YL_ZSPECOUT)

IF (LHOOK) CALL DR_HOOK('LCZTOALD',1,ZHOOK_HANDLE)
END SUBROUTINE LCZTOALD
