SUBROUTINE LCZTOIFS(KTYPE,YD_PVIN,YD_PVOUT)

!**** *LCZTOIFS*

!     Purpose.
!     --------
!           Coordinate transformation from Lanczos space to IFS space.
!           KTYPE=1: from Lanczos to IFS space
!           KTYPE=2: inverse
!           KTYPE=3: square of KTYPE=1
!           KTYPE=4: square of KTYPE=2

!**   Interface.
!     ----------
!        *CALL* *LCZTOIFS*

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        See documentation

!     Reference.
!     ----------
!        ECMWF research department documentation of the
!     "in core model"

!     Author.
!     -------
!        Roberto Buizza :                   *ECMWF*

!     Modifications.
!     --------------
!        Original  : 93-11-12
!        L.Isaksen : 95-06-06 Reordering of spectral arrays
!        J.Barkmeijer/L.Isaksen : 96-01-28 Bug fix and rewrite
!        M.Fisher  : 99-04-15 TOVS control variable
!        Y.Tremolet: 02-08-13 Use control_vectors.
!        M.Leutbecher: 03-04-23 Bug fix
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN, ONLY: NULERR
USE YOMMP , ONLY: MYSETW, NPROCM
USE YOMCVA, ONLY: NGRBVAR
USE SPECTRAL_FIELDS_MOD
USE CONTROL_VECTORS

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPE 
TYPE(CONTROL_VECTOR),INTENT(IN)    :: YD_PVIN 
TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YD_PVOUT 
TYPE(spectral_field) :: YL_ZSPECIN,YL_ZSPECOUT
REAL(KIND=JPRB) :: ZZ, ZFACTOR
INTEGER(KIND=JPIM) :: jf, jir, js, jn
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('LCZTOIFS',0,ZHOOK_HANDLE)
IF(KTYPE < 1.OR.KTYPE > 4) THEN
  WRITE(NULERR,'('' >> ERROR IN LCZTOIFS: BAD KTYPE'')')
  CALL ABOR1('LCZTOIFS: ABOR1 CALLED')
ENDIF

! Case m/=0
ZZ=2.0_JPRB
IF (KTYPE == 1) THEN
  ZFACTOR=SQRT(1.0_JPRB/ZZ)
ELSEIF (KTYPE == 2) THEN
  ZFACTOR=SQRT(ZZ)
ELSEIF (KTYPE == 3) THEN
  ZFACTOR=1.0_JPRB/ZZ
ELSEIF (KTYPE == 4) THEN
  ZFACTOR=ZZ
ENDIF

CALL allocate_spec(YL_ZSPECIN ,YD_PVIN%nsmax,YD_PVIN%nmsmax,YD_PVIN%ns3d,YD_PVIN%ns2d,YD_PVIN%ns1d,NGRBVAR)
CALL allocate_spec(YL_ZSPECOUT,YD_PVIN%nsmax,YD_PVIN%nmsmax,YD_PVIN%ns3d,YD_PVIN%ns2d,YD_PVIN%ns1d,NGRBVAR)

! This limitation could be removed to allow for model error SVs
! or combined SVs. YT.
IF (.NOT.YD_PVIN%linitcv) CALL ABOR1('LCZTOIFS: No initial condition')

YL_ZSPECIN=YD_PVIN%initcv(:)

YL_ZSPECOUT%sp3d = ZFACTOR * YL_ZSPECIN%sp3d
YL_ZSPECOUT%sp2d = ZFACTOR * YL_ZSPECIN%sp2d

! Special case for M==0
! ---------------------
IF (MYSETW == NPROCM(0)) THEN
  ZZ=1.0_JPRB
  IF (KTYPE == 1) THEN
    ZFACTOR=SQRT(1.0_JPRB/ZZ)
  ELSEIF (KTYPE == 2) THEN
    ZFACTOR=SQRT(ZZ)
  ELSEIF (KTYPE == 3) THEN
    ZFACTOR=1.0_JPRB/ZZ
  ELSEIF (KTYPE == 4) THEN
    ZFACTOR=ZZ
  ENDIF

  DO jf=1,YL_ZSPECIN%ns3d
    DO jn=0,YL_ZSPECIN%nsmax
      DO jir=0,1
        js=YL_ZSPECIN%NASM0(0)+jn*2+jir
        YL_ZSPECOUT%sp3d(:,js,jf) = zfactor * YL_ZSPECIN%sp3d(:,js,jf)
      ENDDO
    ENDDO
  ENDDO

  DO jf=1,YL_ZSPECIN%ns2d
    DO jn=0,YL_ZSPECIN%nsmax
      DO jir=0,1
        js=YL_ZSPECIN%NASM0(0)+jn*2+jir
        YL_ZSPECOUT%sp2d(js,jf) = zfactor * YL_ZSPECIN%sp2d(js,jf)
      ENDDO
    ENDDO
  ENDDO
ENDIF

IF (YD_PVIN%linitcv  ) YD_PVOUT%initcv(:) = YL_ZSPECOUT
IF (YD_PVIN%nmoderr>0) YD_PVOUT%moderr(:) = YD_PVIN%moderr(:)
IF (YD_PVIN%ltovscv  ) YD_PVOUT%tovscv(:) = YD_PVIN%tovscv(:)
IF (YD_PVIN%lam1d    ) YD_PVOUT%lamcv (:) = YD_PVIN%lamcv (:)
IF (YD_PVIN%npara>0  ) YD_PVOUT%params(:) = YD_PVIN%params(:)

CALL DEALLOCATE_SPEC(YL_ZSPECIN)
CALL DEALLOCATE_SPEC(YL_ZSPECOUT)

IF (LHOOK) CALL DR_HOOK('LCZTOIFS',1,ZHOOK_HANDLE)
END SUBROUTINE LCZTOIFS
