SUBROUTINE SPEUV(PETPSI,PDIKHI,PU,PV,KFLSUR,KFIELD,KINP)

!**** *SPEUV - Transform from spectral fields to winds in physical space - DM

!     Purpose.
!     --------

!     Compute wind from vorticity/divergence or streamfunction/
!     velocity potential.

!**   Interface.
!     ----------
!        *CALL* *SPEUV(PETPSI,PDIKHI,PU,PV,KFLSUR,KFIELD,KINP)

!        Explicit arguments :
!        --------------------
!           PETPSI                     : vorticity/stream function (input)
!           PDIKHI                     : divergence/vel.pot.       (input)
!           PU                         : u velocity                (output)
!           PV                         : v velocity                (output)
!           KFLSUR                     : first dimension of spectral arrays
!           KFIELD                     : Number of fields
!           KINP                       : KINP=1 - input vor/div
!                                      : KINP=2 - input psi/kai

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals. INV_TRANS - inverse transform (TRANS lib)
!     ----------

!     Reference.
!     ----------
!        Documentation ARPEGE

!     Author.
!     -------
!        MPP Group *ECMWF*

!     Modifications.
!     --------------
!        Original : 95-10-01 Based on SPEUV 
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!         - removal of dummy argument LDPOLE, and code under LDPOLE.
!        =99-03-29= Deborah Salmond  JUMP in FTINV changed to 1
!        Modified : 99-05-28  D.Salmond - Optimise copies in trltom/trmtol. 
!        Modified : 04/06/99 D.Salmond : change order of AOA1 and SOA1
!        Modified : 25/10/00 M.Hamrud  : TRANS package
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Modified : 00-01-25 P. Marquet : Dasux compliance.
!                                        (ZETPSI+ZDIKHI for IM=0)
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSMAX    ,NSPEC2   ,&
 & NUMP  
USE YOMLAP   , ONLY : NASM0    ,MYMS     ,RLAPDI
USE YOMLUN   , ONLY : NULOUT
USE YOMCT0   , ONLY :    NPRTRV
USE YOMGEM   , ONLY : NGPTOT
USE YOMMP    , ONLY : MYSETV

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFLSUR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELD 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PETPSI(KFLSUR,NSPEC2) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIKHI(KFLSUR,NSPEC2) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PU(NGPTOT,KFIELD) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PV(NGPTOT,KFIELD) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KINP 
REAL(KIND=JPRB) :: ZETPSI(KFLSUR,NSPEC2)         , ZDIKHI(KFLSUR,NSPEC2)
REAL(KIND=JPRB) :: ZUV(NGPTOT,2*KFIELD,1)

INTEGER(KIND=JPIM) :: ILF,ISTG,IFLDPP,IREST,IGF,IM,ISP
INTEGER(KIND=JPIM) :: INUMLF(NPRTRV),IVSETUV(KFIELD)
INTEGER(KIND=JPIM) :: JROCB,JMLOC,JLEV,JN
REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "inv_trans.h"
END INTERFACE

#include "abor1.intfb.h"

!  INITIALISATION

IF (LHOOK) CALL DR_HOOK('SPEUV',0,ZHOOK_HANDLE)
ILF = 0
IF( KFIELD == 1 )THEN
  IVSETUV(1) = NPRTRV
  IF(NPRTRV == MYSETV) THEN
    ILF = 1
  ENDIF
  IGF = 1
ELSE
  IFLDPP=KFIELD/NPRTRV
  IREST=KFIELD-IFLDPP*NPRTRV
  DO JROCB=1,NPRTRV
    IF( JROCB <= IREST )THEN
      INUMLF(JROCB)=IFLDPP+1
    ELSE
      INUMLF(JROCB)=IFLDPP
    ENDIF
  ENDDO
  ILF = INUMLF(MYSETV)
  ISTG = 1
  DO JROCB=1,NPRTRV
    IVSETUV(ISTG:ISTG+INUMLF(JROCB)-1) = JROCB
    ISTG = ISTG+INUMLF(JROCB)
  ENDDO
  IGF = ISTG-1
ENDIF

!     PREPARE THE FIELDS AND RESOLVE EQ OF POISSON

IF (KINP == 1) THEN

!    INPUT VORTICITY/DIVERGENCE.

  DO JMLOC=1,NUMP
    IM = MYMS(JMLOC)
    IF( IM == 0 )THEN
      DO JLEV=1,ILF
        DO JN=IM,NSMAX
          ISP=NASM0(IM)+(JN-IM)*2
          ZETPSI(JLEV,ISP  ) = PETPSI(JLEV,ISP)
          ZETPSI(JLEV,ISP+1) = 0.0_JPRB
          ZDIKHI(JLEV,ISP  ) = PDIKHI(JLEV,ISP)
          ZDIKHI(JLEV,ISP+1) = 0.0_JPRB
        ENDDO
      ENDDO
    ELSE
      DO JLEV=1,ILF
        DO JN=IM,NSMAX
          ISP=NASM0(IM)+(JN-IM)*2
          ZETPSI(JLEV,ISP  ) = PETPSI(JLEV,ISP  )
          ZETPSI(JLEV,ISP+1) = PETPSI(JLEV,ISP+1)
          ZDIKHI(JLEV,ISP  ) = PDIKHI(JLEV,ISP  )
          ZDIKHI(JLEV,ISP+1) = PDIKHI(JLEV,ISP+1)
        ENDDO
      ENDDO
    ENDIF
  ENDDO
ELSEIF(KINP == 2) THEN

!    INPUT STREAMFUNCTION/VELOCITY POTENTIAL.

  DO JMLOC=1,NUMP
    IM = MYMS(JMLOC)
    IF( IM == 0 )THEN
      DO JLEV=1,ILF
        DO JN=IM,NSMAX
          ISP=NASM0(IM)+(JN-IM)*2
          ZETPSI(JLEV,ISP  ) = PETPSI(JLEV,ISP)*RLAPDI(JN)
          ZETPSI(JLEV,ISP+1) = 0.0_JPRB
          ZDIKHI(JLEV,ISP  ) = PDIKHI(JLEV,ISP)*RLAPDI(JN)
          ZDIKHI(JLEV,ISP+1) = 0.0_JPRB
        ENDDO
      ENDDO
    ELSE
      DO JLEV=1,ILF
        DO JN=IM,NSMAX
          ISP=NASM0(IM)+(JN-IM)*2
          ZETPSI(JLEV,ISP  ) = PETPSI(JLEV,ISP  )*RLAPDI(JN)
          ZETPSI(JLEV,ISP+1) = PETPSI(JLEV,ISP+1)*RLAPDI(JN)
          ZDIKHI(JLEV,ISP  ) = PDIKHI(JLEV,ISP  )*RLAPDI(JN)
          ZDIKHI(JLEV,ISP+1) = PDIKHI(JLEV,ISP+1)*RLAPDI(JN)
        ENDDO
      ENDDO
    ENDIF
  ENDDO

ELSE
  WRITE(NULOUT,*) ' PB KINP IN SPEUV ',KINP
  CALL ABOR1(' SPEUV : INVALID KINP ')
ENDIF

CALL INV_TRANS(PSPVOR=ZETPSI(1:ILF,:),PSPDIV=ZDIKHI(1:ILF,:),&
 & KVSETUV=IVSETUV(1:IGF),KPROMA=NGPTOT,PGP=ZUV)  

PU(:,:) = ZUV(:,1:KFIELD,1)
PV(:,:) = ZUV(:,KFIELD+1:2*KFIELD,1)

IF (LHOOK) CALL DR_HOOK('SPEUV',1,ZHOOK_HANDLE)
END SUBROUTINE SPEUV
