SUBROUTINE TRAGEOAD(KULROTC,KULDICO,KULOUT,&
 & KFIELD,KFLDSUR,KNSMAX,KASM0,KSPEC2,KUMTP,KNCMAX,&
 & KACM0,KCPEC2,KUMCP,KMYMS,KSPORTS,KTYP,KDIR,&
 & PRLOCEN,PRLOREC,PSPECS,PSPECC)  

!***  *TRAGEOAD* - Stretching and rotations in spectral space.
!                  Adjoint.
!                  Transforms spectral fields of the computational sphere (lower
!                    truncation KNSMAX) into spectral fields of the
!                    geographical sphere (upper truncation KNCMAX) if KDIR=-1.
!                  Transforms spectral fields of the geographical sphere (upper
!                    truncation KNCMAX) into spectral fields of the
!                    computational sphere (lower truncation KNSMAX) if KDIR=1.
!                  Reading on dilatation/rotation matrices is made on a LFI
!                    file.

!     Purpose.
!     --------

!**   Interface.
!     ----------
!         *CALL* TRAGEOAD(...)

!        Explicit arguments :
!        --------------------

!      KULROTC     - Logical unit number for rotation.                (INPUT)
!      KULDICO     - Logical unit number for:                         (INPUT)
!                      * dilatation if KDIR=1
!                      * contraction if KDIR=-1
!      KULOUT      - Output logical unit.                             (INPUT)
!      KFIELD      - Number of spectral fields to be transformed.     (INPUT)
!      KFLDSUR     - Odd number above KFIELD for dimensioning.        (INPUT)
!      KNSMAX      - Smaller truncation (computational sphere).       (INPUT)
!      KASM0       - KASM0(JM): index of the coefficient              (INPUT)
!                    corresponding to the wave number (m=JM,n=JM)
!                    for the smaller truncation.
!      KSPEC2      - Dimension for spectral array PSPECS.             (INPUT)
!      KUMTP       - Number of spectral waves handled by this         (INPUT)
!                    processor for KNSMAX
!      KNCMAX      - Larger truncation (geographical sphere).         (INPUT)
!      KACM0       - KACM0(JM): index of the coefficient              (INPUT)
!                    corresponding to the wave number (m=JM,n=JM)
!                    for the larger truncation.
!      KCPEC2      - Dimension for spectral array PSPECC.             (INPUT)
!      KUMCP       - Number of spectral waves handled by this         (INPUT)
!                    processor for KNCMAX
!      KMYMS       - actual wave numbers handled by this processor    (INPUT)
!      KSPORTS     - KSPORTS=1: with rotation.                        (INPUT)
!                    KSPORTS=0: without rotation.
!      KTYP        - KTYP=1: KFIELD is the first dimension of         (INPUT)
!                     the spectral arrays PSPECS and PSPECC.
!                    KTYP=2: KFIELD is the second dimension of
!                     the spectral arrays PSPECS and PSPECC.
!      KDIR        - KDIR=1 : Adjoint of dilatation.                  (INPUT)
!                    KDIR=-1: Adjoint of contraction.
!      PRLOCEN     - Geographical longitude of the stretching pole.   (INPUT)
!      PRLOREC     - Reference longitude on geographical sphere.      (INPUT)

!      IF KDIR = 1:

!      PSPECC      - Fields before adjoint of dilatation.             (INPUT)
!      PSPECS      - Fields after adjoint of dilatation.              (OUTPUT)

!      IF KDIR = -1:

!      PSPECS      - Fields before adjoint of contraction.            (INPUT)
!      PSPECC      - Fields after adjoint of contraction.             (OUTPUT)

!        Implicit arguments :
!        --------------------

!     Method.
!     -------

!     Externals.
!     ----------
!       Calls SPORTSAD,SPOLTS,SPODTSAD.

!     Reference.
!     ----------
!        ARPEGE/IFS note number 19 (in French).
!        ARPEGE/IFS documentation about conf 911, 912, TRAGEO.

!     Author.
!     -------
!        K. YESSAD. (METEO-FRANCE/CNRM/GMAP)

!     Modifications.
!     --------------
!        Original : APRIL 1993
!        Modification (opening/closing files or not) by D. Giard : July 1994
!        Modification (Reordering of spectral arrays) by L. Isaksen : 95-06-06
!        Modification (setup of loacl arrays) by D. Giard : 95-09-06
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        K.YESSAD     : Jan 2004 : debug parallel version.
!        M.Hamrud      01-May-2006 Generalized IO scheme
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMMP     ,ONLY : MYSETW
USE YOMCT0    ,ONLY : NPRTRW

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELD 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KNSMAX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSPEC2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNCMAX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KCPEC2 
INTEGER(KIND=JPIM),INTENT(IN)    :: KUMCP 
INTEGER(KIND=JPIM)               :: KULROTC ! Argument NOT used
INTEGER(KIND=JPIM)               :: KULDICO ! Argument NOT used
INTEGER(KIND=JPIM),INTENT(IN)    :: KULOUT 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDSUR 
INTEGER(KIND=JPIM),INTENT(IN)    :: KASM0(0:KNSMAX) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KUMTP
INTEGER(KIND=JPIM),INTENT(IN)    :: KACM0(0:KNCMAX) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMYMS(1:KUMCP) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSPORTS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTYP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KDIR 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRLOCEN 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRLOREC 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPECS(KFIELD*KSPEC2) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPECC(KFIELD*KCPEC2) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZSPECS(KFLDSUR,KSPEC2),ZSPECC(KFLDSUR,KCPEC2)
REAL(KIND=JPRB) :: ZSPEKC(KFLDSUR,KCPEC2)
INTEGER(KIND=JPIM) :: INPROCM(0:KNCMAX)
INTEGER(KIND=JPIM) :: INACM0(0:KNCMAX)
INTEGER(KIND=JPIM) :: INUMCPP(NPRTRW)
INTEGER(KIND=JPIM) :: INCPEC
INTEGER(KIND=JPIM) :: INCPEC2
INTEGER(KIND=JPIM) :: INCPEC2MX
INTEGER(KIND=JPIM) :: IPOSSP(NPRTRW+1)
INTEGER(KIND=JPIM) :: IMYMS(KNCMAX+1)
INTEGER(KIND=JPIM) :: ITYP2, JFIELD, JSE
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "ini_spec_dist.h"
END INTERFACE

#include "abor1.intfb.h"
#include "spodtsad.intfb.h"
#include "spolts.intfb.h"
#include "sportsad.intfb.h"
#include "trmton.intfb.h"
#include "trntom.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRAGEOAD',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!        0.    DUMMY ARGUMENTS CHECKING.
!        -------------------------------

! * KFLDSUR:
IF (KFLDSUR < KFIELD) THEN
  WRITE (KULOUT,'(A)')&
   & ' FATAL  : KFLDSUR MUST BE GREATER OR EQUAL THAN KFIELD.'  
  CALL ABOR1('TRAGEOAD: ABOR1 CALLED')
ELSEIF (MOD(KFLDSUR,2) == 0) THEN
  WRITE (KULOUT,'(A)')&
   & ' WARNING: KFLDSUR=ODD NUMBER WOULD BE BETTER FOR OPTIMIZATION.'  
ENDIF
! * KNSMAX,KNCMAX:
IF (KNCMAX < KNSMAX) THEN
  WRITE (KULOUT,'(A)')&
   & ' FATAL  : KNCMAX MUST BE GREATER OR EQUAL THAN KNSMAX.'  
  CALL ABOR1('TRAGEOAD: ABOR1 CALLED')
ENDIF
! * KDIR:
IF (IABS(KDIR) /= 1) THEN
  WRITE (KULOUT,'(A)')' FATAL  : KDIR MUST BE EQUAL TO 1 OR -1.'
  CALL ABOR1('TRAGEOAD: ABOR1 CALLED')
ENDIF
! * KTYP:
IF (KTYP /= 1.AND.KTYP /= 2) THEN
  WRITE (KULOUT,'(A)')' FATAL  : KTYP MUST BE EQUAL TO 1 OR 2.'
  CALL ABOR1('TRAGEOAD: ABOR1 CALLED')
ENDIF
! * KSPORTS:
IF (KSPORTS /= 0.AND.KSPORTS /= 1) THEN
  WRITE (KULOUT,'(A)')' FATAL  : KSPORTS MUST BE EQUAL TO 0 OR 1.'
  CALL ABOR1('TRAGEOAD: ABOR1 CALLED')
ENDIF

!     ------------------------------------------------------------------

!        1.    PRELIMINARY CALCULATIONS.
!        -------------------------------

CALL INI_SPEC_DIST(KNCMAX,KNCMAX,NPRTRW,MYSETW,&
 & KASM0=INACM0,KPROCM=INPROCM,KUMPP=INUMCPP, &
 & KSPEC=INCPEC,KSPEC2=INCPEC2,KSPEC2MX=INCPEC2MX,&
 & KPOSSP=IPOSSP,KMYMS=IMYMS)

!     ------------------------------------------------------------------

!        5.    INITIAL MEMORY TRANSFER
!              AND OVERDIMENSIONING MANAGEMENT.
!        --------------------------------------

DO JFIELD=1,KFIELD
  DO JSE=1,KSPEC2
    ZSPECS(JFIELD,JSE)= 0.0_JPRB
  ENDDO
ENDDO
DO JFIELD=1,KFIELD
  DO JSE=1,KCPEC2
    ZSPECC(JFIELD,JSE)= 0.0_JPRB
    ZSPEKC(JFIELD,JSE)= 0.0_JPRB
  ENDDO
ENDDO

IF (KDIR == -1) THEN
  IF (KTYP == 1) THEN
    DO JFIELD=1,KFIELD
      DO JSE=1,KSPEC2
        ZSPECS(JFIELD,JSE)=PSPECS((JSE-1)*KFIELD+JFIELD)
      ENDDO
    ENDDO
  ELSEIF (KTYP == 2) THEN
    DO JFIELD=1,KFIELD
      DO JSE=1,KSPEC2
        ZSPECS(JFIELD,JSE)=PSPECS((JFIELD-1)*KSPEC2+JSE)
      ENDDO
    ENDDO
  ENDIF
ELSEIF (KDIR == 1) THEN
  IF (KTYP == 1) THEN
    DO JFIELD=1,KFIELD
      DO JSE=1,KCPEC2
        ZSPECC(JFIELD,JSE)=PSPECC((JSE-1)*KFIELD+JFIELD)
      ENDDO
    ENDDO
  ELSEIF (KTYP == 2) THEN
    DO JFIELD=1,KFIELD
      DO JSE=1,KCPEC2
        ZSPECC(JFIELD,JSE)=PSPECC((JFIELD-1)*KCPEC2+JSE)
      ENDDO
    ENDDO
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!        3. and 4.   ROTATION AND DILATATION.
!        ------------------------------------

IF (KDIR == 1) THEN

!      Geographical sphere (upper truncation KNCMAX)
!      ------> Computational sphere (lower truncation KNSMAX).

  IF (KSPORTS == 1) THEN

!      --- DILATATION+ROTATION. ---

    CALL TRMTON(KCPEC2,KNCMAX,KFLDSUR,INACM0,INPROCM,ZSPECC)
    IF (ABS(PRLOCEN-PRLOREC) >= 1.E-14_JPRB) THEN
      CALL SPOLTS(KULOUT,KFIELD,KFLDSUR,KNCMAX,KACM0,KCPEC2,&
       & KUMCP,KMYMS,PRLOCEN-PRLOREC,ZSPECC,ZSPEKC)  
    ELSE
      DO JFIELD=1,KFIELD
        DO JSE=1,KCPEC2
          ZSPEKC(JFIELD,JSE)=ZSPECC(JFIELD,JSE)
        ENDDO
      ENDDO
    ENDIF

    IF(KNSMAX == KNCMAX) THEN
      ITYP2=0
    ELSE
      ITYP2=1
    ENDIF
    CALL SPORTSAD(KULROTC,KULOUT,KFIELD,KFLDSUR,&
     & KNSMAX,KNCMAX,KACM0,KCPEC2,KUMCP,KMYMS,&
     & ITYP2,'MATROTC',ZSPECC,ZSPEKC)  
    CALL TRNTOM(KCPEC2,KNCMAX,KFLDSUR,INACM0,INPROCM,ZSPECC)

    IF (KNSMAX == KNCMAX) THEN
      DO JFIELD=1,KFIELD
        DO JSE=1,KSPEC2
          ZSPECS(JFIELD,JSE)=ZSPECC(JFIELD,JSE)
        ENDDO
      ENDDO
    ELSE
      CALL SPODTSAD(KULDICO,KULOUT,KFIELD,KFLDSUR,&
       & KNSMAX,KASM0,KSPEC2,KUMTP,KNCMAX,KACM0,KCPEC2,&
       & KMYMS,1,ZSPECS,ZSPECC)  
    ENDIF

  ELSEIF (KSPORTS == 0) THEN

!       --- DILATATION ---

    IF (KNSMAX == KNCMAX) THEN
      DO JFIELD=1,KFIELD
        DO JSE=1,KSPEC2
          ZSPECS(JFIELD,JSE)=ZSPECC(JFIELD,JSE)
        ENDDO
      ENDDO
    ELSE
      CALL SPODTSAD(KULDICO,KULOUT,KFIELD,KFLDSUR,&
       & KNSMAX,KASM0,KSPEC2,KUMTP,KNCMAX,KACM0,KCPEC2,&
       & KMYMS,1,ZSPECS,ZSPECC)  
    ENDIF

  ENDIF

ELSEIF (KDIR == -1) THEN

!      Computational sphere (lower truncation KNSMAX)
!      ------> Geographical sphere (upper truncation KNCMAX).

  IF (KSPORTS == 1) THEN

!      --- ROTATION+CONTRACTION. ---

    IF (KNSMAX == KNCMAX) THEN
      DO JFIELD=1,KFIELD
        DO JSE=1,KSPEC2
          ZSPECC(JFIELD,JSE)=ZSPECS(JFIELD,JSE)
        ENDDO
      ENDDO
    ELSE
      CALL SPODTSAD(KULDICO,KULOUT,KFIELD,KFLDSUR,&
       & KNSMAX,KASM0,KSPEC2,KUMTP,KNCMAX,KACM0,KCPEC2,&
       & KMYMS,-1,ZSPECS,ZSPECC)  
    ENDIF

    IF(KNSMAX == KNCMAX) THEN
      ITYP2=0
    ELSE
      ITYP2=-1
    ENDIF
    CALL TRMTON(KCPEC2,KNCMAX,KFLDSUR,INACM0,INPROCM,ZSPECC)
    CALL SPORTSAD(KULROTC,KULOUT,KFIELD,KFLDSUR,&
     & KNSMAX,KNCMAX,KACM0,KCPEC2,KUMCP,KMYMS,&
     & ITYP2,'MATROTC',ZSPEKC,ZSPECC)  

    IF (ABS(PRLOCEN-PRLOREC) >= 1.E-14_JPRB) THEN
      CALL SPOLTS(KULOUT,KFIELD,KFLDSUR,KNCMAX,KACM0,KCPEC2,&
       & KUMCP,KMYMS,PRLOREC-PRLOCEN,ZSPEKC,ZSPECC)  
    ELSE
      DO JFIELD=1,KFIELD
        DO JSE=1,KCPEC2
          ZSPECC(JFIELD,JSE)=ZSPEKC(JFIELD,JSE)
        ENDDO
      ENDDO
    ENDIF
    CALL TRNTOM(KCPEC2,KNCMAX,KFLDSUR,INACM0,INPROCM,ZSPECC)

  ELSEIF (KSPORTS == 0) THEN

!       --- CONTRACTION ---

    IF (KNSMAX == KNCMAX) THEN
      DO JFIELD=1,KFIELD
        DO JSE=1,KSPEC2
          ZSPECC(JFIELD,JSE)=ZSPECS(JFIELD,JSE)
        ENDDO
      ENDDO
    ELSE
      CALL SPODTSAD(KULDICO,KULOUT,KFIELD,KFLDSUR,&
       & KNSMAX,KASM0,KSPEC2,KUMTP,KNCMAX,KACM0,KCPEC2,&
       & KMYMS,-1,ZSPECS,ZSPECC)  
    ENDIF

  ENDIF

ENDIF

!     ------------------------------------------------------------------

!        2.    FINAL MEMORY TRANSFER
!              AND OVERDIMENSIONING MANAGEMENT.
!        --------------------------------------

IF (KDIR == 1) THEN
  IF (KTYP == 1) THEN
    DO JFIELD=1,KFIELD
      DO JSE=1,KSPEC2
        PSPECS((JSE-1)*KFIELD+JFIELD)=ZSPECS(JFIELD,JSE)
      ENDDO
    ENDDO
  ELSEIF (KTYP == 2) THEN
    DO JFIELD=1,KFIELD
      DO JSE=1,KSPEC2
        PSPECS((JFIELD-1)*KSPEC2+JSE)=ZSPECS(JFIELD,JSE)
      ENDDO
    ENDDO
  ENDIF
ELSEIF (KDIR == -1) THEN
  IF (KTYP == 1) THEN
    DO JFIELD=1,KFIELD
      DO JSE=1,KCPEC2
        PSPECC((JSE-1)*KFIELD+JFIELD)=ZSPECC(JFIELD,JSE)
      ENDDO
    ENDDO
  ELSEIF (KTYP == 2) THEN
    DO JFIELD=1,KFIELD
      DO JSE=1,KCPEC2
        PSPECC((JFIELD-1)*KCPEC2+JSE)=ZSPECC(JFIELD,JSE)
      ENDDO
    ENDDO
  ENDIF
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRAGEOAD',1,ZHOOK_HANDLE)
END SUBROUTINE TRAGEOAD

