SUBROUTINE TRANSINV_MDL(PSPVOR,PSPDIV,PSPSP,PSPHV,PSPGFL,&
 & PGMV,PGMVS,PGFL,LDERR,LDFSCOMP)  

!**** *TRANSINV_MDL * - Inverse transforms for model

!     Purpose.  Perform inverse transform (spectral to gridpoint)
!     --------

!**   Interface.  CALL TRANSINV_MDL(...)
!     ---------- 

!     Explicit arguments : 
!     --------------------
!        PSPVOR - spectral vorticity
!        PSPDIV - spectral divergence
!        PSPSP  - spectral surface pressure
!        PSPHV  - spectral thermodynamic variables
!        PSPGFL - spectral GFL variables
!        PGMV   - GMV gridpoint
!        PGMVS  - GMV single level gridpoint
!        PGFL   - GFL gridpoint
!        LDERR  - TRUE if derivatives required
!        LDFSCOMP - true if "scientific" computations in fourier space req.

!        Called by TRANSINVH

!     Externals.
!     ----------
!     INV_TRANS - inverse transform (TRANS library)

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 00-10-25
!        R. El Khatib 02-09-05 : Fix for passive scalars.
!        Modified : 03-08-01 M.Hamrud - GFL introduction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        C.Temperton 04-01-29: Fix for SWE (is there a cleaner way?)
!        R. El Khatib 18-Apr-2005 NPROMATR
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NSPEC2   ,LVOR     ,LUVDER   ,NPROMA   ,NGPBLKS  ,&
          & NFLEVG   ,NFLEVL
USE YOMMP    , ONLY : NBSETLEV ,NBSETSP  ,MYSETV
USE YOM_YGFL , ONLY : YGFL
USE YOMGMV   , ONLY : YT0
USE YOMTRANS , ONLY : NPROMATR
USE YOMGEM   , ONLY : NGPTOT

IMPLICIT NONE

LOGICAL,         INTENT(IN)  :: LDERR
LOGICAL,         INTENT(IN)  :: LDFSCOMP
REAL(KIND=JPRB), INTENT(IN)  :: PSPSP(:)
REAL(KIND=JPRB), INTENT(IN)  :: PSPVOR(:,:)
REAL(KIND=JPRB), INTENT(IN)  :: PSPDIV(:,:)
REAL(KIND=JPRB), INTENT(IN)  :: PSPHV(:,:,:)
REAL(KIND=JPRB), INTENT(IN)  :: PSPGFL(:,:,:)
REAL(KIND=JPRB), INTENT(OUT) :: PGMVS(:,:,:)
REAL(KIND=JPRB), INTENT(OUT) :: PGMV(:,:,:,:)
REAL(KIND=JPRB), INTENT(OUT) :: PGFL(:,:,:,:)

REAL(KIND=JPRB) :: ZSP(1,NSPEC2)
REAL(KIND=JPRB), ALLOCATABLE :: ZGP(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPEC(:,:)
INTEGER(KIND=JPIM) :: IST,IOFF,IDIM3,ISTGMV,IENGMV,IDIMGMVS,ISTUV,IENUV
INTEGER(KIND=JPIM) :: IBL, IFLD, IMAXGP, ISPF, IST1, IEND1, ICHUNKS, IMAXLEV
INTEGER(KIND=JPIM) :: IENDLEV, JROF, JFLD, JSP, JKGLO, JSTLEV, IMAXSP, ILEVS
INTEGER(KIND=JPIM) :: JLEV
INTEGER(KIND=JPIM) :: IVSETSC(1)
INTEGER(KIND=JPIM), ALLOCATABLE :: IVSET(:)
LOGICAL :: LLUVDER,LLVOR,LLSCDERS,LLDIVGP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

EXTERNAL FSPGLH

INTERFACE
#include "inv_trans.h"
END INTERFACE

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSINV_MDL',0,ZHOOK_HANDLE)
LLUVDER  = LUVDER .AND. LDERR
LLVOR    = LVOR   .AND. LDERR
LLSCDERS = LDERR
LLDIVGP  = LDERR

IVSETSC(1) = NBSETSP

CALL GSTATS(1894,0)
IF(NBSETSP == MYSETV) THEN
  IST = 1
  ZSP(IST,:) = PSPSP(:)
ELSE
  IST = 0
ENDIF
CALL GSTATS(1894,1)
ISTGMV = YT0%NDIMUV+1
IENGMV = YT0%NDIM
IDIMGMVS = YT0%NDIMS
ISTUV = 1
IENUV = YT0%NDIMUV
IF(.NOT. LLSCDERS) THEN
  IENGMV = MAX(ISTGMV,ISTGMV-1+(IENGMV-ISTGMV+1)/3)
  IDIMGMVS = MAX(1,IDIMGMVS/3)
  ISTUV = YT0%MU
  IENUV = YT0%MV
ENDIF
IF(YGFL%NUMSPFLDS>0 .AND. NPROMATR<=0) THEN
  IOFF = YGFL%NUMFLDS-YGFL%NUMSPFLDS
  IDIM3 = YGFL%NUMSPFLDS
  IF (LLSCDERS) IDIM3 = IDIM3*3
  IF (LDFSCOMP) THEN
    CALL INV_TRANS(&
     & PSPVOR=PSPVOR,PSPDIV=PSPDIV,PSPSC2=ZSP(1:IST,:),PSPSC3A=PSPHV,&
     & PSPSC3B=PSPGFL, &
     & LDSCDERS=LLSCDERS,LDVORGP=LLVOR,LDDIVGP=LLDIVGP,LDUVDER=LLUVDER,&
     & KPROMA=NPROMA,KVSETUV=NBSETLEV,KVSETSC2=IVSETSC(1:1),&
     & KVSETSC3A=NBSETLEV,KVSETSC3B=NBSETLEV,&
     & PGPUV=PGMV(:,:,ISTUV:IENUV,:),PGP2=PGMVS(:,1:IDIMGMVS,:),&
     & PGP3A=PGMV(:,:,ISTGMV:IENGMV,:),&
     & PGP3B=PGFL(:,:,IOFF+1:IOFF+IDIM3,:),&
     & FSPGL_PROC=FSPGLH)  
  ELSE
    CALL INV_TRANS(&
     & PSPVOR=PSPVOR,PSPDIV=PSPDIV,PSPSC2=ZSP(1:IST,:),PSPSC3A=PSPHV,&
     & PSPSC3B=PSPGFL, &
     & LDSCDERS=LLSCDERS,LDVORGP=LLVOR,LDDIVGP=LLDIVGP,LDUVDER=LLUVDER,&
     & KPROMA=NPROMA,KVSETUV=NBSETLEV,KVSETSC2=IVSETSC(1:1),&
     & KVSETSC3A=NBSETLEV,KVSETSC3B=NBSETLEV,&
     & PGPUV=PGMV(:,:,ISTUV:IENUV,:),PGP2=PGMVS(:,1:IDIMGMVS,:),&
     & PGP3A=PGMV(:,:,ISTGMV:IENGMV,:),&
     & PGP3B=PGFL(:,:,IOFF+1:IOFF+IDIM3,:))  
  ENDIF
ELSEIF (ISTGMV>IENGMV .AND. NPROMATR<=0) THEN
  IF (LDFSCOMP) THEN
    CALL INV_TRANS(PSPVOR=PSPVOR,PSPDIV=PSPDIV,PSPSC2=ZSP(1:1,:),&
     & LDSCDERS=LLSCDERS,LDVORGP=LLVOR,LDDIVGP=LLDIVGP,LDUVDER=LLUVDER,&
     & KPROMA=NPROMA,KVSETUV=NBSETLEV,KVSETSC2=IVSETSC(1:1),&
     & PGPUV=PGMV(:,:,ISTUV:IENUV,:),PGP2=PGMVS(:,1:IDIMGMVS,:),&
     & FSPGL_PROC=FSPGLH)  
  ELSE
    CALL INV_TRANS(PSPVOR=PSPVOR,PSPDIV=PSPDIV,PSPSC2=ZSP(1:1,:),&
     & LDSCDERS=LLSCDERS,LDVORGP=LLVOR,LDDIVGP=LLDIVGP,LDUVDER=LLUVDER,&
     & KPROMA=NPROMA,KVSETUV=NBSETLEV,KVSETSC2=IVSETSC(1:1),&
     & PGPUV=PGMV(:,:,ISTUV:IENUV,:),PGP2=PGMVS(:,1:IDIMGMVS,:)) 
  ENDIF
ELSEIF(NPROMATR<=0) THEN
  IF (LDFSCOMP) THEN
    CALL INV_TRANS(PSPVOR=PSPVOR,PSPDIV=PSPDIV,PSPSC2=ZSP(1:1,:),&
     & PSPSC3A=PSPHV,&
     & LDSCDERS=LLSCDERS,LDVORGP=LLVOR,LDDIVGP=LLDIVGP,LDUVDER=LLUVDER,&
     & KPROMA=NPROMA,KVSETUV=NBSETLEV,KVSETSC2=IVSETSC(1:1),&
     & KVSETSC3A=NBSETLEV,&
     & PGPUV=PGMV(:,:,ISTUV:IENUV,:),PGP2=PGMVS(:,1:IDIMGMVS,:),&
     & PGP3A=PGMV(:,:,ISTGMV:IENGMV,:), &
     & FSPGL_PROC=FSPGLH)  
  ELSE
    CALL INV_TRANS(PSPVOR=PSPVOR,PSPDIV=PSPDIV,PSPSC2=ZSP(1:1,:),&
     & PSPSC3A=PSPHV,&
     & LDSCDERS=LLSCDERS,LDVORGP=LLVOR,LDDIVGP=LLDIVGP,LDUVDER=LLUVDER,&
     & KPROMA=NPROMA,KVSETUV=NBSETLEV,KVSETSC2=IVSETSC(1:1),&
     & KVSETSC3A=NBSETLEV,&
     & PGPUV=PGMV(:,:,ISTUV:IENUV,:),PGP2=PGMVS(:,1:IDIMGMVS,:),&
     & PGP3A=PGMV(:,:,ISTGMV:IENGMV,:))
  ENDIF
ELSE
! Slower but less memory-consuming transforms
  IST1=1
! 1/vector fields:
! Number of level chunks : deriving from NPROMATR, but at least 1 whole level :
  IF (NFLEVG==NFLEVL) THEN
    ICHUNKS=MIN((((IENUV-ISTUV+1)*NFLEVG-1)/NPROMATR+1),NFLEVG)
  ELSE
!   Vertical level distribution not coded
    ICHUNKS=1
  ENDIF
  IMAXLEV=(NFLEVG-1)/ICHUNKS+1
  DO JSTLEV=1,NFLEVG,IMAXLEV
    IENDLEV=JSTLEV-1+MIN(IMAXLEV,NFLEVG-JSTLEV+1)
    ILEVS=IENDLEV-JSTLEV+1
    IMAXGP=(IENUV-ISTUV+1)*ILEVS
    ALLOCATE(ZGP(NPROMA,IMAXGP,NGPBLKS))
    IF (LDFSCOMP) THEN
      CALL INV_TRANS(PSPVOR=PSPVOR(JSTLEV:IENDLEV,:),&
       & PSPDIV=PSPDIV(JSTLEV:IENDLEV,:),LDVORGP=LLVOR,LDDIVGP=LLDIVGP,&
       & LDUVDER=LLUVDER,KPROMA=NPROMA,KVSETUV=NBSETLEV(JSTLEV:IENDLEV),&
       & PGP=ZGP(:,:,:),FSPGL_PROC=FSPGLH)
    ELSE
      CALL INV_TRANS(PSPVOR=PSPVOR(JSTLEV:IENDLEV,:),&
       & PSPDIV=PSPDIV(JSTLEV:IENDLEV,:),LDVORGP=LLVOR,LDDIVGP=LLDIVGP,&
       & LDUVDER=LLUVDER,KPROMA=NPROMA,KVSETUV=NBSETLEV(JSTLEV:IENDLEV),&
       & PGP=ZGP(:,:,:))
    ENDIF
!$OMP PARALLEL PRIVATE(JKGLO,IBL,IEND1,JFLD,JLEV,IFLD,JROF)
!$OMP DO
    DO JKGLO=1,NGPTOT,NPROMA
      IBL=(JKGLO-1)/NPROMA+1
      IEND1=MIN(NPROMA,NGPTOT-JKGLO+1)
      DO JFLD=ISTUV,IENUV
        DO JLEV=JSTLEV,IENDLEV
          IFLD=JLEV-JSTLEV+1+(JFLD-ISTUV)*ILEVS
          DO JROF=IST1,IEND1
            PGMV(JROF,JLEV,JFLD,IBL)=ZGP(JROF,IFLD,IBL)
          ENDDO
        ENDDO
      ENDDO
    ENDDO
!$OMP END DO
!$OMP END PARALLEL
    DEALLOCATE(ZGP)
  ENDDO
! 2/scalar fields - gmvs
  IF (LDFSCOMP) THEN
    CALL INV_TRANS(PSPSCALAR=ZSP(:,:),LDSCDERS=LLSCDERS,KPROMA=NPROMA,&
     & KVSETSC=IVSETSC(:),PGP=PGMVS(:,1:IDIMGMVS,:),FSPGL_PROC=FSPGLH)
  ELSE
    CALL INV_TRANS(PSPSCALAR=ZSP(:,:),LDSCDERS=LLSCDERS,KPROMA=NPROMA,&
     & KVSETSC=IVSETSC(:),PGP=PGMVS(:,1:IDIMGMVS,:))
  ENDIF
! 3/scalar fields - gmv
  IF (NFLEVG==NFLEVL) THEN
    ICHUNKS=MIN((((IENGMV-ISTGMV+1)*NFLEVG-1)/NPROMATR+1),NFLEVG)
  ELSE
!   Vertical level distribution not coded
    ICHUNKS=1
  ENDIF
  ISPF=SIZE(PSPHV,DIM=3)
  IMAXLEV=(NFLEVG-1)/ICHUNKS+1
  DO JSTLEV=1,NFLEVG,IMAXLEV
    IENDLEV=JSTLEV-1+MIN(IMAXLEV,NFLEVG-JSTLEV+1)
    ILEVS=IENDLEV-JSTLEV+1
    IMAXGP=(IENGMV-ISTGMV+1)*ILEVS
    IMAXSP=ISPF*ILEVS
    ALLOCATE(IVSET(IMAXSP))
    ALLOCATE(ZSPEC(IMAXSP,NSPEC2))
    DO JFLD=1,ISPF
      DO JLEV=JSTLEV,IENDLEV
        IFLD=JLEV-JSTLEV+1+(JFLD-1)*ILEVS
        IVSET(IFLD)=NBSETLEV(JLEV)
        DO JSP=1,NSPEC2
          ZSPEC(IFLD,JSP)=PSPHV(JLEV,JSP,JFLD)
        ENDDO
      ENDDO
    ENDDO
    ALLOCATE(ZGP(NPROMA,IMAXGP,NGPBLKS))
    IF (LDFSCOMP) THEN
      CALL INV_TRANS(PSPSCALAR=ZSPEC(:,:),LDSCDERS=LLSCDERS,KPROMA=NPROMA,&
       & KVSETSC=IVSET(:),PGP=ZGP(:,:,:),FSPGL_PROC=FSPGLH)
    ELSE
      CALL INV_TRANS(PSPSCALAR=ZSPEC(:,:),LDSCDERS=LLSCDERS,KPROMA=NPROMA,&
       & KVSETSC=IVSET(:),PGP=ZGP(:,:,:))
    ENDIF
    DEALLOCATE(ZSPEC)
    DEALLOCATE(IVSET)
!$OMP PARALLEL PRIVATE(JKGLO,IBL,IEND1,JFLD,JLEV,IFLD,JROF)
!$OMP DO
    DO JKGLO=1,NGPTOT,NPROMA
      IBL=(JKGLO-1)/NPROMA+1
      IEND1=MIN(NPROMA,NGPTOT-JKGLO+1)
      DO JFLD=ISTGMV,IENGMV
        DO JLEV=JSTLEV,IENDLEV
          IFLD=JLEV-JSTLEV+1+(JFLD-ISTGMV)*ILEVS
          DO JROF=IST1,IEND1
            PGMV(JROF,JLEV,JFLD,IBL)=ZGP(JROF,IFLD,IBL)
          ENDDO
        ENDDO
      ENDDO
    ENDDO
!$OMP END DO
!$OMP END PARALLEL
    DEALLOCATE(ZGP)
  ENDDO
! 4/scalar fields - gfl
  IDIM3 = YGFL%NUMSPFLDS
  IF (LLSCDERS) IDIM3 = IDIM3*3
  IF (NFLEVG==NFLEVL) THEN
    ICHUNKS=MIN(((IDIM3*NFLEVG-1)/NPROMATR+1),NFLEVG)
  ELSE
!   Vertical level distribution not coded
    ICHUNKS=1
  ENDIF
  ISPF=SIZE(PSPGFL,DIM=3)
  IMAXLEV=(NFLEVG-1)/ICHUNKS+1
  DO JSTLEV=1,NFLEVG,IMAXLEV
    IENDLEV=JSTLEV-1+MIN(IMAXLEV,NFLEVG-JSTLEV+1)
    ILEVS=IENDLEV-JSTLEV+1
    IMAXGP=IDIM3*ILEVS
    IMAXSP=ISPF*ILEVS
    ALLOCATE(IVSET(IMAXSP))
    ALLOCATE(ZSPEC(IMAXSP,NSPEC2))
    DO JFLD=1,ISPF
      DO JLEV=JSTLEV,IENDLEV
        IFLD=JLEV-JSTLEV+1+(JFLD-1)*ILEVS
        IVSET(IFLD)=NBSETLEV(JLEV)
        DO JSP=1,NSPEC2
          ZSPEC(IFLD,JSP)=PSPGFL(JLEV,JSP,JFLD)
        ENDDO
      ENDDO
    ENDDO
    ALLOCATE(ZGP(NPROMA,IMAXGP,NGPBLKS))
    IF (LDFSCOMP) THEN
      CALL INV_TRANS(PSPSCALAR=ZSPEC(:,:),LDSCDERS=LLSCDERS,KPROMA=NPROMA,&
       & KVSETSC=IVSET(:),PGP=ZGP(:,:,:),FSPGL_PROC=FSPGLH)
    ELSE
      CALL INV_TRANS(PSPSCALAR=ZSPEC(:,:),LDSCDERS=LLSCDERS,KPROMA=NPROMA,&
       & KVSETSC=IVSET(:),PGP=ZGP(:,:,:))
    ENDIF
    DEALLOCATE(ZSPEC)
    DEALLOCATE(IVSET)
    IF(YGFL%NUMSPFLDS>0) THEN
      IOFF = YGFL%NUMFLDS-YGFL%NUMSPFLDS
    ELSE
      IOFF = 0
    ENDIF
!$OMP PARALLEL PRIVATE(JKGLO,IBL,IEND1,JFLD,JLEV,IFLD,JROF)
!$OMP DO
    DO JKGLO=1,NGPTOT,NPROMA
      IBL=(JKGLO-1)/NPROMA+1
      IEND1=MIN(NPROMA,NGPTOT-JKGLO+1)
      DO JFLD=1,IDIM3
        DO JLEV=JSTLEV,IENDLEV
          IFLD=JLEV-JSTLEV+1+(JFLD-1)*ILEVS
          DO JROF=IST1,IEND1
            PGFL(JROF,JLEV,JFLD+IOFF,IBL)=ZGP(JROF,IFLD,IBL)
          ENDDO
        ENDDO
      ENDDO
    ENDDO
!$OMP END DO
!$OMP END PARALLEL
    DEALLOCATE(ZGP)
  ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('TRANSINV_MDL',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

END SUBROUTINE TRANSINV_MDL
