SUBROUTINE TRANSINVH(CDCONF)

!**** *TRANSINVH * - Inverse transforms

!     Purpose.  Perform inverse transform (spectral to gridpoint)
!     --------

!**   Interface.  CALL TRANSINVH(CDCONF)
!     ---------- 

!     Explicit arguments : CDCONF - configuration of work
!     --------------------
!        Called by STEPO/STEPOTL.

!     Externals.
!     ----------
!     TRANSINV_MDL - transforms for thr model   
!     TRANSINV_FP  - transforms for Full-Pos   

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud *ECMWF*

!     Modifications.
!     --------------
!        Original : 00-10-25
!        R. El Khatib : 01-03-07 Fix
!        P.Smolikova  : 02-09-30 Interface to INI2SCAN2M for d4 in NH
!        R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!        Modified : 03-08-01 M.Hamrud - GFL introduction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        K.Yessad (Dec 2003): cleaning in horizontal diffusion.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : NCONF    ,LSLAG    ,LRFRIC
USE YOMSP    , ONLY : SPVOR    ,SPDIV    ,SP5VOR   ,SP5DIV   ,&
 & SPHV     ,SP5HV    , SPSP     ,SP5SP, SPGFL, SP5GFL
USE YOMDIM     , ONLY : LADER   ,NPROMA   ,NGPBLKS
USE YOMOMPDIST , ONLY : GPP
USE YOMFPDIM , ONLY : NFPGT0B
USE YOMGMV   , ONLY : GMV,GMVS,GMV5,GMV5S
USE YOMGFL   , ONLY : GFL,GFL5

!     ------------------------------------------------------------------

IMPLICIT NONE

!  CDCONF(2:2) Definition for INVERSE LEGENDRE TRANSFORM 

!                      Spectral data 
!                  SPA3    SPA5    SPA7
!                 ------------------------
!   Derivatives  |Yes No |Yes No |Yes No | 
! ----------------------------------------
!                |       |       |       |
!    F       T0  | A  G  | B  H  | C  I  |
!    o D         |       |       |       |
!    u a         |-----------------------|
!    r t         |       |       |       |
!    i a     T5  | D  J  | E  K  | F  L  |
!    e           |       |       |       |
!    r           |-----------------------|

!     ------------------------------------------------------------------

CHARACTER(LEN=9)  ,INTENT(IN)    :: CDCONF 

!     ------------------------------------------------------------------

LOGICAL :: LLDERR,LLADGJ,LLBEHK,LLCFIL,LLP,LLFSCOMP
LOGICAL :: LLHFP
INTEGER(KIND=JPIM) :: IMAXFLDS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "transinv_fp.intfb.h"
#include "transinv_mdl.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSINVH',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

LLDERR = LLE(CDCONF(2:2),'F') .AND. LADER 
LLADGJ = CDCONF(2:2) == 'A'.OR.CDCONF(2:2) == 'D'.OR.CDCONF(2:2) == 'G'.OR.&
 & CDCONF(2:2) == 'J'.OR.CDCONF(2:2) == 'M'  
LLBEHK = CDCONF(2:2) == 'B'.OR.CDCONF(2:2) == 'E'.OR.CDCONF(2:2) == 'H'.OR.&
 & CDCONF(2:2) == 'K'  
LLCFIL = CDCONF(2:2) == 'C'.OR.CDCONF(2:2) == 'F'.OR.CDCONF(2:2) == 'I'.OR.&
 & CDCONF(2:2) == 'L'  
LLP    = CDCONF(2:2) == 'P'
LLFSCOMP = LLDERR .AND. (LRFRIC.AND.LSLAG.AND.(NCONF == 1.OR.NCONF == 302))

LLHFP =   (CDCONF(5:5) == 'A').OR.(CDCONF(5:5) == 'Z')&
 & .OR.(CDCONF(5:5) == 'P').OR.(CDCONF(5:5) == 'C')&
 & .OR.(CDCONF(5:5) == 'X').OR.(CDCONF(5:5) == 'E')  

IF (CDCONF(3:3) == 'P') THEN
! Fullpos specific dimensionning
  IMAXFLDS=NFPGT0B
  IF (ALLOCATED(GPP)) THEN
    IF (SIZE(GPP) /= NPROMA*IMAXFLDS*NGPBLKS) DEALLOCATE(GPP)
  ENDIF
  IF (.NOT.ALLOCATED(GPP)) THEN
    ALLOCATE (GPP(NPROMA,IMAXFLDS,NGPBLKS))
  ENDIF
ENDIF

IF(LLADGJ) THEN
  IF (CDCONF(3:3) == 'A'.OR.CDCONF(3:3) == 'B'.OR.CDCONF(3:3) == 'P'.OR.&
     & CDCONF(3:3) == '1') THEN  
    CALL TRANSINV_MDL(SPVOR,SPDIV,SPSP,SPHV,SPGFL,&
     & GMV,GMVS,GFL,LLDERR,LLFSCOMP)  
  ELSEIF (CDCONF(3:3) == 'C' .OR. CDCONF(3:3) == 'D') THEN
    CALL ABOR1(' TRANSINVH - CONFIGURATION NOT SUPPORTED 0')
  ELSE
    CALL ABOR1(' TRANSINVH - CONFIGURATION NOT SUPPORTED 1')
  ENDIF
ELSEIF(LLBEHK) THEN
  CALL TRANSINV_MDL(SP5VOR,SP5DIV,SP5SP,SP5HV,SP5GFL,&
   & GMV5,GMV5S,GFL5,LLDERR,LLFSCOMP)  
ELSEIF(LLCFIL)THEN
    CALL ABOR1(' TRANSINVH - CONFIGURATION NO LONGER SUPPORTED 0')
ELSEIF(LLP) THEN
  ! Full-Pos
  CALL TRANSINV_FP(GPP)
ELSE
  CALL ABOR1(' TRANSINVH - CONFIGURATION NOT SUPPORTED 4')
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TRANSINVH',1,ZHOOK_HANDLE)
END SUBROUTINE TRANSINVH
