SUBROUTINE GRID_FROM_GRIB(YD_GRIB_HEAD,KLATS,KLONS,PLATS,KGP,KMAXLAT)

!    Purpose.
!    --------
!    Return a description of the grid from the header of a GRIB file.

!    Explicit arguments:
!    -------------------

!    Input:
!      GRIB_HEAD - GRIB header
!      KMAXLAT   - Max numer of latitude rows

!    Outputs:
!      KLATS     - number of latitude rows
!      KLONS     - number of longitudes for each row of the grid
!      PLATS     - latitude (radians) of each row of the grid
!      KGP       - number of points of the grid

!    Author.
!    -------
!      Y.Tremolet      Based on INQGRIB

!    Modifications.
!    --------------
!      Original: 18/01/01
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        Y.Tremolet    27-Apr-2005 Fix latitudes
!        K.Yessad (Nov 2008): rename arp/SUGAW into arp/SUGAWA.
!-----------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE GRIB_HEADER_MIX

!-----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KMAXLAT 
TYPE(GRIB_HEADER) ,INTENT(IN)    :: YD_GRIB_HEAD 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLATS 
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLONS(KMAXLAT) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLATS(KMAXLAT) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KGP 

!-----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: jlat
REAL(KIND=JPRB), ALLOCATABLE :: zpl(:),zwe(:)
REAL(KIND=JPRB), ALLOCATABLE :: zdd(:)

! -- ky: for call to SUGAWA --------------
INTEGER(KIND=JPIM) :: IDGLG,ISTART,IK,IODD,JN
REAL(KIND=JPRB),ALLOCATABLE :: ZFN(:,:), ZFNLAT(:)
REAL(KIND=JPRB) :: ZFNN
! ----------------------------------------

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----------------------------------------------------------------------

#include "sugawa.intfb.h"
#include "abor1.intfb.h"

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GRID_FROM_GRIB',0,ZHOOK_HANDLE)
!-----------------------------------------------------------------------

! Number of latitudes
! -------------------

IF ( (YD_GRIB_HEAD%nsec2gg(1)==0) .OR. (YD_GRIB_HEAD%nsec2gg(1)==4) ) THEN
  klats  = YD_GRIB_HEAD%nsec2gg(3)
  IF (klats > kmaxlat) CALL ABOR1('GRID_FROM_GRIB: kmaxlat TOO SMALL')
ELSE
  CALL ABOR1('GRID_FROM_GRIB: GRID TYPE NOT RECOGNIZED')
ENDIF

! Full grid
! ---------
IF (YD_GRIB_HEAD%nsec2gg(1) == 0) THEN
  klons(:)=YD_GRIB_HEAD%nsec2gg(2)
ENDIF

! Reduced grid
! ------------
IF (YD_GRIB_HEAD%nsec2gg(1) == 4) THEN
  IF (YD_GRIB_HEAD%nsec2gg(17) == 0) THEN
    klons(:) = YD_GRIB_HEAD%nsec2gg(2)
  ELSE
    DO jlat=1,klats
      klons(jlat) = YD_GRIB_HEAD%nsec2gg(22+jlat)
    ENDDO
  ENDIF
ENDIF

! Number of points
! ----------------
kgp=0
DO jlat=1,klats
  kgp=kgp+klons(jlat)
ENDDO

! Compute latitudes
! -----------------
ALLOCATE(zpl(klats))
ALLOCATE(zdd(klats))
ALLOCATE(zwe(klats))

! ky: Calls a version of SUGAW which should be the mirror of the
!     current version of tfl/module/sugaw_mod.F90.
!     We can probably call tfl/module/sugaw_mod.F90 there
!     but this must be checked (modification planned after CY36).

! * preliminary calculations to compute input quantity ZFNLAT
!   (k.y.: coded after what I found in tfl/module/suleg_mod.F90).
IDGLG=KLATS
ISTART=1
ALLOCATE(ZFN(0:IDGLG,0:IDGLG))
ALLOCATE(ZFNLAT(0:IDGLG/2))
! Belousov, Swarztrauber use ZFN(0,0)=SQRT(2._JPRB)
! IFS normalisation chosen to be 0.5*Integral(Pnm**2) = 1
ZFN(0,0)=2._JPRB
DO JN=ISTART,IDGLG
  ZFNN=ZFN(0,0)
  DO JLAT=1,JN
    ZFNN=ZFNN*SQRT(1._JPRB-0.25_JPRB/REAL(JLAT**2,JPRB))
  ENDDO
  IODD=MOD(JN,2)
  ZFN(JN,JN)=ZFNN
  DO JLAT=2,JN-IODD,2
    ZFN(JN,JN-JLAT)=ZFN(JN,JN-JLAT+2) &
     & *REAL((JLAT-1)*(2*JN-JLAT+2),JPRB)/REAL(JLAT*(2*JN-JLAT+1),JPRB)
  ENDDO
ENDDO
IODD=MOD(IDGLG,2)
IK=IODD
DO JLAT=IODD,IDGLG,2
  ZFNLAT(IK)=ZFN(IDGLG,JLAT)
  IK=IK+1
ENDDO

! * call to SUGAWA:
CALL SUGAWA(klats,ZFNLAT,zpl,zdd,zwe)

! * deallocations:
DEALLOCATE(ZFN)
DEALLOCATE(ZFNLAT)

DO jlat=1,klats
  plats(jlat)=ASIN(zdd(jlat))
ENDDO

DEALLOCATE(zpl)
DEALLOCATE(zdd)
DEALLOCATE(zwe)

!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GRID_FROM_GRIB',1,ZHOOK_HANDLE)
END SUBROUTINE GRID_FROM_GRIB
