SUBROUTINE INTERP_GP(KINTERP,KPARAM,KLEVEL,KDGLG_I,KLOENG_I,PLATIG_I,&
 & KDGLG_O,KLOENG_O,PLATIG_O,PGLFLDI,PGLFLDO)

!**** *INTERP_GP* - Interpolate grid-point fields

!     Purpose.
!     --------
!     Wrapper for different interpolations in grid-point space applied
!     to non-distributed fields

!**   Interface.
!     ----------
!        *CALL* *INTERP_GP*(...)

! Explicit arguments : KINTERP - interpolation type
! -------------------- KPARAM  - GRIB code of field to be interpolated
!                      KLEVEL  - level of field
!                      KDGLG_I - number of Gaussian lats, input field
!                      KLOENG_I - number of long. points, input field
!                      PLATIG_I - Gaussian latitudes, input field
!                      KDGLG_O - number of Gaussian lats, output field
!                      KLOENG_O - number of long. points, output field
!                      PLATIG_O - Gaussian latitudes, output field
!                      PGLFLDI  - input field (full field)
!                      PGLFLDO  - output field (full field)
!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS
!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!       Original : 15-Dec-2005
!       E.Holm    04-Jan-2008 Conserving interpolation interface
!                             changed to use surface pressure
!     ------------------------------------------------------------------

USE PARKIND1 , ONLY : JPIM     ,JPRB
USE YOMGRB   , ONLY : NGRBV,NGRBU
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOM_GRID_BICONSERV, ONLY : RGPPRS_HR, RGPPRS_LR

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KINTERP
INTEGER(KIND=JPIM),INTENT(IN) :: KPARAM
INTEGER(KIND=JPIM),INTENT(IN) :: KLEVEL
INTEGER(KIND=JPIM),INTENT(IN) :: KDGLG_I
INTEGER(KIND=JPIM),INTENT(IN) :: KLOENG_I(:)
REAL(KIND=JPRB)   ,INTENT(IN) :: PLATIG_I(:)
INTEGER(KIND=JPIM),INTENT(IN) :: KDGLG_O
INTEGER(KIND=JPIM),INTENT(IN) :: KLOENG_O(:)
REAL(KIND=JPRB)   ,INTENT(IN) :: PLATIG_O(:)
REAL(KIND=JPRB)   ,INTENT(IN) :: PGLFLDI(:)
REAL(KIND=JPRB)   ,INTENT(OUT):: PGLFLDO(:)

INTEGER(KIND=JPIM) :: IGPTOTG_I,IGPTOTG_O
LOGICAL :: LLPARITY
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "grid_bicubic.intfb.h"
#include "grid_bilinear.intfb.h"
#include "grid_biconserv.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('INTERP_GP',0,ZHOOK_HANDLE)

IGPTOTG_I = SUM(KLOENG_I(1:KDGLG_I))
IGPTOTG_O = SUM(KLOENG_O(1:KDGLG_O))

SELECT CASE (KINTERP)
CASE(1)
  CALL GRID_BILINEAR(KDGLG_I,KLOENG_I,PLATIG_I,IGPTOTG_I,&
   & KDGLG_O,KLOENG_O(1:KDGLG_O),PLATIG_O(1:KDGLG_O),IGPTOTG_O,&
   & PGLFLDI,PGLFLDO)
CASE(2)
  CALL GRID_BICUBIC(KDGLG_I,KLOENG_I,PLATIG_I,IGPTOTG_I,&
   & KDGLG_O,KLOENG_O(1:KDGLG_O),PLATIG_O(1:KDGLG_O),IGPTOTG_O,&
   & PGLFLDI,PGLFLDO)
CASE(3)
  IF (KLEVEL==0) THEN
    CALL GRID_BICONSERV(KDGLG_I,KLOENG_I,PLATIG_I,IGPTOTG_I,&
     & KDGLG_O,KLOENG_O(1:KDGLG_O),PLATIG_O(1:KDGLG_O),IGPTOTG_O,&
     & PGLFLDI,PGLFLDO)
  ELSEIF (KLEVEL>0) THEN
    IF (.NOT.ALLOCATED(RGPPRS_LR)) THEN 
      IF (.NOT.ALLOCATED(RGPPRS_HR)) THEN
        CALL ABOR1('INTER_GP:RGPPRS_HR NOT ALLOCATED')
      ENDIF
      IF (IGPTOTG_I>IGPTOTG_O) THEN
        ALLOCATE (RGPPRS_LR(IGPTOTG_O))
        CALL GRID_BICONSERV(KDGLG_I,KLOENG_I,PLATIG_I,IGPTOTG_I,&
                          & KDGLG_O,KLOENG_O(1:KDGLG_O),PLATIG_O(1:KDGLG_O),&
                          & IGPTOTG_O,&
                          & RGPPRS_HR,RGPPRS_LR)
      ELSEIF (IGPTOTG_I<=IGPTOTG_O) THEN 
        ALLOCATE (RGPPRS_LR(IGPTOTG_I))
        CALL GRID_BICONSERV(KDGLG_O,KLOENG_O(1:KDGLG_O),PLATIG_O(1:KDGLG_O),&
                          & IGPTOTG_O,&
                          & KDGLG_I,KLOENG_I,PLATIG_I,IGPTOTG_I,&
                          & RGPPRS_HR,RGPPRS_LR)
      ENDIF 
    ENDIF
    LLPARITY=.NOT.(KPARAM==NGRBV.OR.KPARAM==NGRBU )
    IF (IGPTOTG_I>IGPTOTG_O) THEN
      CALL GRID_BICONSERV(KDGLG_I,KLOENG_I,PLATIG_I,IGPTOTG_I,&
       & KDGLG_O,KLOENG_O(1:KDGLG_O),PLATIG_O(1:KDGLG_O),IGPTOTG_O,&
       & PGLFLDI,PGLFLDO,LLPARITY,KLEVEL,RGPPRS_HR,RGPPRS_LR)
    ELSEIF (IGPTOTG_I<=IGPTOTG_O) THEN 
      CALL GRID_BICONSERV(KDGLG_I,KLOENG_I,PLATIG_I,IGPTOTG_I,&
       & KDGLG_O,KLOENG_O(1:KDGLG_O),PLATIG_O(1:KDGLG_O),IGPTOTG_O,&
       & PGLFLDI,PGLFLDO,LLPARITY,KLEVEL,RGPPRS_LR,RGPPRS_HR)
    ENDIF
  ENDIF 
END SELECT
IF (LHOOK) CALL DR_HOOK('INTERP_GP',1,ZHOOK_HANDLE)
!     ------------------------------------------------------------------
END SUBROUTINE INTERP_GP
