SUBROUTINE PKSPECA(PSPEC,KFIELDG)

!**** *PKSPECA*  - PACK SPECTRAL DATA THROUGH ARPEGE FILE

!     PURPOSE.
!     --------
!        To pack model spectral fields by writing to, then reading from a
!         file ARPEGE
!        In SM version : loop on all fields to write & read back
!        In DM version : Fields are distributed between the processors ; 
!        Each processor performs its own loop on fields to write & read back
!        on a local file.

!**   INTERFACE.
!     ----------
!       *CALL* *PKSPECA*

!        EXPLICIT ARGUMENTS
!        --------------------
!        PSPEC   : (Local) spectral fields array
!        KFIELDG : Number of fields in PSPEC

!        IMPLICIT ARGUMENTS
!        --------------------
!        None

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 94-04-08
!        Modified : 95-06-06 L.Isaksen   Reordering of spectral arrays
!        Modified : 98-02-11 R. El Khatib Restructuration & DM version
!        Modified : 98-12-31 R. El Khatib Additive arguments to NEWFA
!        Modified : 99-07-21 R. El Khatib less arguments to NEWFA
!        Modified : 00-04-19 J.Vivoda do not call routine if LELAM and NGRSP=0
!    R. El Khatib : 01-03-16 No I/O in packing ; (LELAM and NGRSP) moved to PRESPFPOS
!        Modified : 01-03-20 S. Ivatek-Sahdan call DIWRSPE -> call DIWRSPE0
!                                           & call DISSPEC -> call DISSPEC0
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!      D. Paradis & R. El Khatib : 04-07-22 GRIBEX
!        R. El Khatib & O.Spaniel : B_level + fullpos inline, correction
!        R. El Khatib, thanks to Alex Deckmin 12-Jan-2009 Bugfix for large
!                      number of mpi tasks
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMLUN   , ONLY : NSCRTCH  ,NULOUT
USE YOMDIM   , ONLY : NSPEC2   ,NSPEC2G  ,NSEFRE
USE YOMOP    , ONLY : CNMCA
USE YOMCT0   , ONLY : LOUTPUT  ,NPRTRW
USE YOMMPG   , ONLY : NPOSSP
USE YOMMP    , ONLY : MYPROC, MYSETW, MYSETV

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KFIELDG 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPEC(NSPEC2,KFIELDG) 
INTEGER(KIND=JPIM), PARAMETER :: JPFAOVRSZ=2

INTEGER(KIND=JPIM) :: IASPIO(KFIELDG)
REAL(KIND=JPRB) :: ZSPFILE(NSEFRE), ZVALCO(NSEFRE+JPFAOVRSZ)

REAL(KIND=JPRB), ALLOCATABLE :: ZSPECBUF(:)

INTEGER(KIND=JPIM) :: IFIELDS, IREP, ISPN, ISPX, JF, JS, INBARI, INOMA, ILONGA
INTEGER(KIND=JPIM) :: INGRIB, INBPDG, INBCSP, ISTRON, IPUILA, IDMOPL, ISPWIO

CHARACTER(LEN=16) :: CLSCRTCH, CLNOMA
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "disspec0.intfb.h"
#include "diwrspe0.intfb.h"
#include "spreord.intfb.h"
#include "set2pe.intfb.h"

!     IASPIO(j): Processor number which packs fields #j
!     IFIELDS  : Number of fields packed by MYPROC
!     ZSPECBUF : Spectral BUFFER array (NOT global but sized as global)

!     ------------------------------------------------------------------

!*       1.1 DISTRIBUTE FIELDS
!            -----------------

IF (LHOOK) CALL DR_HOOK('PKSPECA',0,ZHOOK_HANDLE)
ISPN=NPOSSP(MYSETW)
ISPX=NPOSSP(MYSETW+1)-1
DO JF=1,KFIELDG
 ISPWIO=MOD(JF-1,NPRTRW)+1
 CALL SET2PE(IASPIO(JF),0,0,ISPWIO,MYSETV)
ENDDO
IFIELDS=0
DO JF=1,KFIELDG
  IF (IASPIO(JF) == MYPROC) THEN
    IFIELDS=IFIELDS+1
  ENDIF
ENDDO
IF (LOUTPUT) THEN
  WRITE(NULOUT,*) 'PKSPECA : IFIELDS = ',IFIELDS, ' IASPIO = '
  WRITE(NULOUT,'(15(1X,I4))') IASPIO
ENDIF

!*       1.2 MAKE AN ARPEGE FILE ENVIRONMENT
!            -------------------------------

IF (IFIELDS > 0) THEN
  CALL FANOUV(IREP,NSCRTCH,.FALSE.,CLSCRTCH,'NEW',.TRUE.,.TRUE.,1,1,INBARI,&
   & CNMCA)  
ENDIF

!*       1.3 PACK DATA THROUGH WRITE AND READ
!            --------------------------------

ALLOCATE(ZSPECBUF(NSPEC2G))

!*       1.3.1 Send fields I do not pack
DO JF=1,KFIELDG
  IF (IASPIO(JF) /= MYPROC) THEN
    CALL DIWRSPE0(IASPIO(JF),MYSETV,JF,0,NSPEC2,1,NPOSSP,NPRTRW,PSPEC(1,JF))
  ENDIF
ENDDO

!*       1.3.2 For my fields : receive, pack and send them back

IF (IFIELDS > 0) THEN
  CALL FAVEUR(IREP,NSCRTCH,INGRIB,INBPDG,INBCSP,ISTRON,IPUILA,IDMOPL)
ENDIF
DO JF=1,KFIELDG
  IF (IASPIO(JF) == MYPROC) THEN
    DO JS=ISPN,ISPX
      ZSPECBUF(JS)=PSPEC(JS-ISPN+1,JF)
    ENDDO
    CALL DIWRSPE0(IASPIO(JF),MYSETV,JF,1,NSPEC2G,1,NPOSSP,NPRTRW,ZSPECBUF)
    IF (.NOT.(INGRIB==-1 .OR. INGRIB==3)) THEN
      CALL SPREORD(1,ZSPFILE,ZSPECBUF,.FALSE.)
      CALL FACOND(IREP,NSCRTCH,'S',0,'FIELD',ZSPFILE,.TRUE.,CLNOMA,INOMA,&
       & ZVALCO,ILONGA)  
      CALL FADECO(IREP,NSCRTCH,'S',0,'FIELD',.TRUE.,CLNOMA,INOMA,ZVALCO,&
       & ILONGA,ZSPFILE)  
      CALL SPREORD(1,ZSPFILE,ZSPECBUF,.TRUE.)
    ELSE
      CALL FACOND(IREP,NSCRTCH,'S',0,'FIELD',ZSPECBUF,.TRUE.,CLNOMA,INOMA,&
       & ZVALCO,ILONGA)
      CALL FADECO(IREP,NSCRTCH,'S',0,'FIELD',.TRUE.,CLNOMA,INOMA,ZVALCO,&
       & ILONGA,ZSPECBUF)
    ENDIF
    CALL DISSPEC0(IASPIO(JF),MYSETV,JF,0,NSPEC2G,1,NPOSSP,NPRTRW,ZSPECBUF)
    DO JS=ISPN,ISPX
      PSPEC(JS-ISPN+1,JF)=ZSPECBUF(JS)
    ENDDO
  ENDIF
ENDDO

!*       1.3.3 Receive fields I have not packed
DO JF=1,KFIELDG
  IF (IASPIO(JF) /= MYPROC) THEN
    CALL DISSPEC0(IASPIO(JF),MYSETV,JF,1,NSPEC2,1,NPOSSP,NPRTRW,PSPEC(1,JF))
  ENDIF
ENDDO

DEALLOCATE(ZSPECBUF)

!*       1.4 REMOVE THE ARPEGE FILE ENVIRONMENT
!            ----------------------------------

IF (IFIELDS > 0) THEN
  CALL FAIRNO(IREP,NSCRTCH,'DELETE')
ENDIF

IF (LHOOK) CALL DR_HOOK('PKSPECA',1,ZHOOK_HANDLE)
END SUBROUTINE PKSPECA
