SUBROUTINE PRE_GRID_BICONSERV(KGPOUT,KLATS,KLONS,PLATS, &
                            & PSPIN,PSPOUT,PSPSP,PGPSP,PGPSPG)

!     Purpose.
!     --------
!       Compute pressure fields needed for conserving interpolation.

!     Arguments.
!     ----------
!       KGPOUT  : Number of points at output resolution
!       KLATS   : 
!       KLONS   : 
!       PLATS   : 
!       PSPSP   : LnPs spectral field            (PSPSP or PGPSP or PGPSPG required)
!       PGPSP   : LnPs GP field in NPROMA blocks (PSPSP or PGPSP or PGPSPG required)
!       PGPSPG  : LnPs GP global field NGPTOTG size (PSPSP or PGPSP or PGPSPG required)
!       PSPIN : surface pressure at input  res. (for conserving 3D interp)
!       PSPOUT: surface pressure at output res. (for conserving 3D interp)

!     Author.
!     -------
!       Y. Tremolet (from E.Holm and L.Isaksen code)

!     Modifications.
!     --------------
!       Original : 26-Jan-2005
!       080204 E.Holm    : Use only surface pressure
! ----------------------------------------------------------------------

USE PARKIND1,ONLY: JPIM, JPRB
USE YOMHOOK ,ONLY: LHOOK, DR_HOOK
USE YOMGEM, ONLY : NLOENG, NGPTOTG, NGPTOT
USE YOMDIM, ONLY : NDGLG, NPROMA, NGPBLKS, NSPEC2
USE YOMLEG, ONLY : RLATIG

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN) :: KGPOUT
INTEGER(KIND=JPIM), INTENT(IN) :: KLATS
INTEGER(KIND=JPIM), INTENT(IN) :: KLONS(KLATS)
REAL(KIND=JPRB), INTENT(IN)    :: PLATS(KLATS)
REAL(KIND=JPRB), INTENT(OUT)   :: PSPIN(NGPTOTG)
REAL(KIND=JPRB), INTENT(OUT)   :: PSPOUT(KGPOUT)
REAL(KIND=JPRB), OPTIONAL, INTENT(IN) :: PSPSP(NSPEC2)
REAL(KIND=JPRB), OPTIONAL, INTENT(IN) :: PGPSP(NPROMA,NGPBLKS)
REAL(KIND=JPRB), OPTIONAL, INTENT(IN) :: PGPSPG(NGPTOTG)

INTEGER(KIND=JPIM) :: jj,jg,jb,iend
REAL(KIND=JPRB) :: ZPRESSIN(NGPTOTG,1),ZPRESSLOC(NGPTOT)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "gathergpf.intfb.h"
#include "speree.intfb.h"
#include "grid_biconserv.intfb.h"
! ----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('PRE_GRID_BICONSERV',0,ZHOOK_HANDLE)

IF (.NOT. (PRESENT(PSPSP).OR.PRESENT(PGPSP).OR.PRESENT(PGPSPG))) &
  & CALL ABOR1('PRE_GRID_BICONSERV: PSPSP, PGPSP or PGPSPG required')

! Surface pressure at both input (size=NGPTOTG) and output (size=KGPOUT)
! resolution is required for conserving interpolation method, 
! so these fields are distributed from the io processor
! to all other processors.
! It is assumed that input and output has the same vertical level
! definitions, i.e. VAH and VBH are the same in the two GPPREH calls.

! HR pressure in grid point space required for conserving interpolation of
! 3D fields.


IF (PRESENT(PGPSPG)) THEN
  ZPRESSIN(:,1)=EXP(PGPSPG(:))
ELSE
  ! Transform HR lnPs to local gridpoint space
  ! and convert from ln(ps) to ps.
  IF (PRESENT(PSPSP)) THEN
    CALL SPEREE(1,1,PSPSP,ZPRESSLOC)
    ZPRESSLOC(:)=EXP(ZPRESSLOC(:))
  ELSEIF (PRESENT(PGPSP)) THEN
    jg=0
    DO jb=1,NGPBLKS
      iend=MIN(NPROMA,NGPTOT-NPROMA*(jb-1))
      DO jj=1,iend
        jg=jg+1
        ZPRESSLOC(jg)=EXP(PGPSP(jj,jb))
      ENDDO
    ENDDO
  ENDIF
  ! Gather local grid point sections to a global field
  ! and distribute to all processors
  CALL GATHERGPF(ZPRESSLOC,ZPRESSIN,1,-1)
ENDIF
PSPIN(:)=ZPRESSIN(:,1)

CALL GRID_BICONSERV(NDGLG,NLOENG(1:NDGLG),RLATIG(1:NDGLG),&
       & NGPTOTG,KLATS,KLONS,PLATS,KGPOUT,PSPIN,PSPOUT)

IF (LHOOK) CALL DR_HOOK('PRE_GRID_BICONSERV',1,ZHOOK_HANDLE)
END SUBROUTINE PRE_GRID_BICONSERV
