SUBROUTINE READ_SURFGRID_TRAJ_FROMFA(KSTEP,PTRAJ_SRFC)

!     ----------------------------------------------------------------------
!     Purpose.
!     --------
!       Read surface grid-point fields from ARPEGE files, one file per range.
!       The ARPEGE files contains the trajectory coming from
!       a "high resolution" run, transformed at the current
!       resolution by an ad-hoc procedure (for example FULL-POS).

!     * INPUT:
!       KSTEP:     step numbering for the trajectory

!     * OUTPUT:
!       PTRAJ_SRFC: buffer to store the trajectory

!     Author.
!     -------
!       K. Yessad, after routine READ_GRID_TRAJ of Yannick Tremolet *ECMWF*
!        and routine SUGRIDF.

!     Modifications.
!     --------------
!       Original  : August 2006
!       K. Yessad (04-Apr-2007): adapt to new data flux.
!       R. El Khatib  26-Nov-2008  Cleaning
!     ----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK
USE YOMCT0    ,ONLY : CNMEXP   ,NPRINTLEV
USE YOMLUN    ,ONLY : NINMSH   ,NULTRAJHR,NULOUT
USE YOMDIM    ,ONLY : NPROMA   ,NGPBLKS
USE YOMTRAJ   ,ONLY : NGP5
USE YOMGEM    ,ONLY : NGPTOT
USE YOMOP     ,ONLY : CFNTRAJHR,CFNGSH
USE SURFACE_FIELDS_MIX, ONLY : GPOPER

!     ----------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSTEP
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PTRAJ_SRFC(NPROMA,NGP5,NGPBLKS) 

!     ----------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IFILE
INTEGER(KIND=JPIM) :: IBL, ICEND, JKGLO
INTEGER(KIND=JPIM) :: IULTRAJHR

CHARACTER(LEN=16)  :: CLFNTRAJHR_SAVE
CHARACTER(LEN=4)   :: CLSTEP

REAL(KIND=JPRB)    :: ZTEMP(NPROMA,NGP5)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ----------------------------------------------------------------------

#include "abor1.intfb.h"
#include "sugrida.intfb.h"

!     ----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('READ_SURFGRID_TRAJ_FROMFA',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    READ IN FIELDS FROM ARPEGE FILE AND SAVE IN PTRAJ_SRFC
!              ------------------------------------------------------

IF (KSTEP>0) THEN

  !        1.1   Read in fields from ARPEGE file.


  ! * save CFNTRAJHR and NULTRAJHR
  CLFNTRAJHR_SAVE=CFNTRAJHR
  IF (KSTEP == 1) IULTRAJHR=NULTRAJHR

  ! * fill the surface buffers with the trajectory read on an ARPEGE file.
  !   This ARPEGE file is assumed to contain data at truncation NSMAX.
  !   The part of SUGRIDA which is used is the same as for reading the
  !   initial files but the surface buffers filled should be not the ones
  !   containing the increments and probably only the prognostic surface
  !   fields have to be read here (adaptations of SUGRIDA to read only
  !   the prognostic surface fields have to be coded later).

  IFILE=50
  IF (KSTEP>1) THEN
    WRITE(CLSTEP,'(I4.4)') KSTEP-1
    CFNTRAJHR='TRAJ'//CNMEXP(1:4)//CLSTEP//'    '
  ELSE
    ! * Trajectory file at initial instant should not be read,
    !   minimisation initial file must be read instead.
    !   This reading is necessary because SCAN2M gets trajectory
    !   information before calling GP_MODEL even at the first timestep.
    CFNTRAJHR=CFNGSH
    NULTRAJHR=NINMSH
  ENDIF

  IF (NPRINTLEV >= 2) THEN
    WRITE(NULOUT,'(A,A16)') ' READ_SURFGRID_TRAJ_FROMFA: read file ',CFNTRAJHR
  ENDIF

  IF (CFNTRAJHR == 'TRAJ'//CNMEXP(1:4)//'0000    ') THEN
    WRITE(NULOUT,'(A,A16,A)') ' File ',CFNTRAJHR,' should not be read!'
    CALL ABOR1(' READ_SURFGRID_TRAJ_FROMFA: ABOR1 nr 1a called ')
  ENDIF

  ! k.y.: for the time being, this call to SUGRIDA reads all the grid-point
  !       surface fields, but actually only those with attribute 
  !       Y[group]_[var]%Y[var]%ITRAJ equal to 1 are necessary.
  CALL SUGRIDA(IFILE)

  ! * Restore the initial content of CFNTRAJHR and NULTRAJHR
  CFNTRAJHR=CLFNTRAJHR_SAVE
  IF (KSTEP == 1) NULTRAJHR=IULTRAJHR

  !        1.2   Transfer useful surface fields from the surface buffers
  !              to PTRAJ_SRFC.

  ! k.y.: this memory transfer should be the reverse of the one done by
  !       FORCE_SFC_TRAJ under SCAN2M (cf. what is done in STORE_TRAJ_SFC).
  !       But are this memory transfer and the call to FORCE_SFC_TRAJ under SCAN2M
  !       necessary in our case (need to fill TRAJ_SRFC?)?
  DO JKGLO=1,NGPTOT,NPROMA
    IBL=(JKGLO-1)/NPROMA+1
    ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    CALL GPOPER('TRAJSTORE',KBL=IBL,PFIELD=ZTEMP)
    PTRAJ_SRFC(1:ICEND,1:NGP5,IBL)=ZTEMP(1:ICEND,1:NGP5)
  ENDDO

ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('READ_SURFGRID_TRAJ_FROMFA',1,ZHOOK_HANDLE)
END SUBROUTINE READ_SURFGRID_TRAJ_FROMFA
