SUBROUTINE VERDER(KPROMA,KSTART,KPROF,KFLEV,PIN,POUT,LDBC)

!**** *VERDER*   Vertical derivative for VFE

!     Purpose.
!     --------
!          This subroutine computes the vertical derivative (with respect
!          to eta) of a function given at full model
!          levels using a general scheme developed by means of finite-elements

!**   Interface.
!     ----------
!        *CALL* *VERDER(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension.
!          KSTART  - first element of work.
!          KPROF   - depth of work.
!          KFLEV   - number of model layers.
!          PIN     - Input field

!        OUTPUT:
!          POUT    : d(PIN)/d(eta)   at each model level

!        OPTIONAL INPUT:
!          LDBC    : T: use matrix RDERB (takes account of upper and lower BC)
!                    F: use matrix RDERI (no external BC provided)
!                    Not present=F

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by

!     Reference.

!     Author.
!     -------
!         M. Hortal (ECMWF) (after VERINT)

!     Modifications.
!     --------------
!        Original : OCT 2004
!    N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCVER  , ONLY : LVERTFE  ,RDERI    ,RDERB
USE YOMDIM   , ONLY : NFLEVG 

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PIN(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: POUT(KPROMA,KFLEV) 
LOGICAL,OPTIONAL  ,INTENT(IN)    :: LDBC

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: IDB
LOGICAL            :: LLBC
REAL(KIND=JPRB) :: ZDER(KFLEV,KFLEV)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('VERDER',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!      * 1/  Checkings

IF(.NOT.LVERTFE) CALL ABOR1(' ! LVERTFE=.F. IN VERDER !!!')

IF (PRESENT(LDBC)) THEN
  LLBC=LDBC
ELSE
  LLBC=.FALSE.
ENDIF

!      * 2/  Matrix product 

IF (LLBC) THEN
  ! In this case KFLEV should be equal to NFLEVG+2
  IF (KFLEV /= NFLEVG+2) CALL ABOR1('VERDER 2a: wrong value of KFLEV')
  ZDER(2:KFLEV-1,1:KFLEV)=RDERB(1:KFLEV-2,1:KFLEV)
  ZDER(1,1:KFLEV)=0.0_JPRB
  ZDER(KFLEV,1:KFLEV)=0.0_JPRB
ELSE
  ! In this case KFLEV should be equal to NFLEVG
  IF (KFLEV /= NFLEVG) CALL ABOR1('VERDER 2b: wrong value of KFLEV')
  ZDER(1:KFLEV,1:KFLEV)=RDERI(1:KFLEV,1:KFLEV)
ENDIF
IDB=SIZE(ZDER,DIM=1)
IF (KPROF >= KSTART) THEN
  CALL DGEMM('N','T',KPROF-KSTART+1,KFLEV,KFLEV, &
   & 1.0_JPRB,PIN,KPROMA,ZDER,IDB,0.0_JPRB,POUT,KPROMA)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('VERDER',1,ZHOOK_HANDLE)
END SUBROUTINE VERDER
