SUBROUTINE VERINTAD(KPROMA,KSTART,KPROF,KFLEV,PIN,POUT,KTYPE)

!**** *VERINT*   Adjoint of Vertical Integral

!     Purpose.
!     --------
!          This subroutine computes the adjoint of the vertical integral 
!          (with respect to eta) of a function given at full model
!          levels using a general scheme
!          The integral is given either from the top (KTYPE=0) down
!          (or from the bottom (KTYPE=1) up) to each full model level

!**   Interface.
!     ----------
!        *CALL* *VERINT(..)

!        Explicit arguments :
!        --------------------

!        INPUT:
!          KPROMA  - horizontal dimension.
!          KSTART  - first element of work.
!          KPROF   - depth of work.
!          KFLEV   - number of model layers.
!          PIN     - Input field
!          KTYPE   - starting point of the integral (0=top, 1=bottom)

!        OUTPUT:
!                    eta
!                     _
!                    |
!          POUT :    | PIN deta   at each full model level
!                   _|
!                 KTYPE

!        Implicit arguments :
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!           none
!           Called by

!     Reference.

!     Author.
!     -------
!         A. Untch (ECMWF)

!     Modifications.
!     --------------
!        Original : Dec. 2000
!        D.SALMOND : APRIL 2002 FORTRAN Matrix multiply replace by BLAS routine
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCVER  , ONLY : LVERTFE  ,RINTE
USE YOMLUN   , ONLY : NULERR

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PIN(KPROMA,KFLEV) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: POUT(KPROMA,KFLEV+1) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPE 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) ::  JLEV, JROF, IDB
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('VERINTAD',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

IF(.NOT.LVERTFE) CALL ABOR1(' ! LVERTFE=.F. IN VERINTAD !!!')
IF(KTYPE /= 0.AND.KTYPE /= 1) THEN
  WRITE(NULERR,*) ' INVALID KTYPE IN VERINTAD =',KTYPE
  CALL ABOR1(' VERINTAD: ABOR1 CALLED')
ENDIF

IF(KTYPE == 1) THEN
  POUT(:,KFLEV+1)=0.0_JPRB
  DO JLEV=1,KFLEV
    DO JROF=KSTART,KPROF
      POUT(JROF,KFLEV+1)=POUT(JROF,KFLEV+1)-POUT(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

!MATRIX MULTIPLY REPLACED BY BLAS ROUTINE
 
!PIN(KSTART:KPROF,1:KFLEV)=_ZERO_
!DO JLEV=1,KFLEV
!  DO JLE=1,KFLEV+1
!    DO JROF=KSTART,KPROF
!      PIN(JROF,JLEV)=PIN(JROF,JLEV)+RINTE(JLE,JLEV)*POUT(JROF,JLE)
!    ENDDO
!  ENDDO
!ENDDO

IDB=SIZE(RINTE,DIM=1)
IF (KPROF >= KSTART) THEN 
  CALL DGEMM('N','N',KPROF-KSTART+1,KFLEV,KFLEV+1, &
   & 1.0_JPRB,POUT,KPROMA,RINTE,IDB,0.0_JPRB,PIN,KPROMA)  
ENDIF

POUT(:,:)=0.0_JPRB

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('VERINTAD',1,ZHOOK_HANDLE)
END SUBROUTINE VERINTAD
