SUBROUTINE WRITE_GRID_TRAJ(CDFILE,KGRIB_HD,KTYPE3D,KTYPE2D,K3D,K2D,&
                        & PGRID,PSPSP,KTYPE_INTERP,LDNEW,LDCLOSE)


!     Purpose.
!     --------
!     Gather grid point fields and write them out to GRIB file.

!     Arguments (all input)
!     ---------------------
!       CDFILE       : Filename
!       KGRIB_HD     : GRIB header for output
!       KTYPE3D      : List of 3D field variable GRIB definitions
!       KTYPE2D      : List of 2D field variable GRIB definitions
!       K3D          : Number of 3D field variables
!       K2D          : Number of 2D field variables
!       PGRID        : Grid point fields to write
!       PSPSP        : LnPs spectral field (required only if KTYPE_INTERP=3)
!       KTYPE_INTERP : Interpolation type
!                      1 - bilinear
!                      2 - bicubic
!                      3 - conserving bilinear/bicubic
!       LDNEW        : true : write to new file
!                      false: append to existing file
!       LDCLOSE      : close files

!     Author.
!     -------
!       L. Isaksen and Y. Tremolet

!     Modifications.
!     --------------
!       Original : 09-Jun-2004
!        L.Isaksen     18-Jan-2005 Introduce conserving interpolation
!        Y.Tremolet    18-Feb-2005 Split pre_grid_biconserv
!        Y.Tremolet    30-Mar-2005 PREGRBENC -> PRESET_GRIB_HEAD
!        E.Holm        05-Feb-2008 Allocate surface pressure for biconserv
! ----------------------------------------------------------------------

USE PARKIND1,ONLY: JPIM, JPRB
USE PARDIM  ,ONLY: JPMXLE
USE YOMHOOK ,ONLY: LHOOK, DR_HOOK
USE YOMCT0, ONLY : NPROC
USE YOMGEM, ONLY : NGPTOTG
USE YOMLUN, ONLY : NULOUT
USE YOMMP , ONLY : MYPROC
USE YOMDIM, ONLY : NFLEVG, NPROMA, NGPBLKS, NSPEC2
USE GRIB_HEADER_MIX
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_PUT, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST, &
 & CLOSE_IOREQUEST,INQUIRE_GRIB
USE YOM_GRID_BICONSERV , ONLY : RGPPRS_HR, RGPPRS_LR

IMPLICIT NONE

CHARACTER(LEN=*)  ,INTENT(IN)    :: CDFILE 
INTEGER(KIND=JPIM), INTENT(IN)   :: KGRIB_HD
INTEGER(KIND=JPIM),INTENT(IN)    :: K3D, K2D
INTEGER(KIND=JPIM),INTENT(IN)    :: KTYPE3D(K3D),KTYPE2D(K2D)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGRID(NPROMA,K3D*NFLEVG+K2D,NGPBLKS)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPSP(:)
INTEGER(KIND=JPIM), INTENT(IN)   :: KTYPE_INTERP
LOGICAL, INTENT(IN)              :: LDNEW
LOGICAL, INTENT(IN)              :: LDCLOSE

INTEGER(KIND=JPIM) :: JF,ILEV,JLEV,IOPROC,JBLK,IOFF,IOFFG
INTEGER(KIND=JPIM) :: IGPOUT
INTEGER(KIND=JPIM) :: IOPROCS(NPROC),ILEVLIST(NFLEVG)
INTEGER(KIND=JPIM) :: IGRIB2D(NFLEVG*K3D),ILEVS2D(NFLEVG*K3D),IPROC2D(NFLEVG*K3D)
INTEGER(KIND=JPIM) :: IONPROCS,ICHUNKS,JROC,JCH,ISTLEV,IENLEV,ILEVS
REAL(KIND=JPRB),ALLOCATABLE :: ZGRID(:,:,:)
LOGICAL :: LLINTERP
LOGICAL :: LLFIRST=.TRUE.
CHARACTER(LEN=128),SAVE :: CLFILELEV(JPMXLE)
CHARACTER(LEN=128),SAVE :: CLFILESURF
CHARACTER(LEN=128) :: CLFILE
CHARACTER(LEN=3) :: CLEV
CHARACTER(LEN=1) :: CLMODE
REAL(KIND=JPRB) :: ZHOOK_HANDLE
TYPE(TYPE_IOSTREAM),SAVE :: YL_IOSTREAM_SURF
TYPE(TYPE_IOSTREAM),SAVE :: YL_IOSTREAM(JPMXLE)
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST

#include "abor1.intfb.h"
#include "grid_psglobal.intfb.h"
! ----------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('WRITE_GRID_TRAJ',0,ZHOOK_HANDLE)

IF(LLFIRST) THEN
  CLFILESURF='CLOSED'
  CLFILELEV(:)='CLOSED'
  LLFIRST=.FALSE.
ENDIF
IF (LDNEW) THEN
  CLMODE='w'
ELSE
  CLMODE='a'
ENDIF

CALL INQUIRE_GRIB(KGRIB_HD,KPTS=IGPOUT)

LLINTERP=(NGPTOTG /= igpout)
IF (LLINTERP) THEN
  SELECT CASE (ktype_interp)
  CASE(1)
    write(nulout,*)'WRITE_GRID_TRAJ: doing linear interpolation'
  CASE(2)
    write(nulout,*)'WRITE_GRID_TRAJ: doing cubic interpolation'
  CASE(3)
    write(nulout,*)'WRITE_GRID_TRAJ: doing conserving interpolation'
  CASE DEFAULT
    CALL ABOR1('WRITE_GRID_TRAJ: unkown interpolation type')
  END SELECT
ENDIF

!-----------------------------------------------------------!
! To be cleaner, the definition of the grid should be       !
! passed as an argument, not a grid header.              YT !
!-----------------------------------------------------------!
! There should be a test in case the two grids are the same !
! set linterp to false in that case.                        !
! This test is a little crude...                         YT !
!-----------------------------------------------------------!

! Processor NPROC handles the surface fields

IF (K2D > 0) THEN
  IOPROC = NPROC
  IF (K3D>0) THEN
    ILEV = 0
    WRITE(CLEV,'(I3.3)')ILEV
    CLFILE = CDFILE//'L'//CLEV
  ELSE
    CLFILE = CDFILE
  ENDIF
  IF(LDNEW .OR. CLFILE /= CLFILESURF ) THEN
    IF(TRIM(CLFILESURF) /= 'CLOSED') THEN
      CALL CLOSE_IOSTREAM(YL_IOSTREAM_SURF)
      CLFILESURF='CLOSED'
  ENDIF
    CLFILESURF=CLFILE
    CALL SETUP_IOSTREAM(YL_IOSTREAM_SURF,'CIO',TRIM(CLFILESURF),CDMODE=CLMODE,&
   & KIOMASTER=ioproc)
  ENDIF
  CALL SETUP_IOREQUEST(YL_IOREQUEST,'GRIDPOINT_FIELDS',LDGRIB=.TRUE.,&
   & KGRIB2D=KTYPE2D(1:K2D),KPROMA=NPROMA,&
   & LDINTERP=LLINTERP,KTYPE_INTERP=KTYPE_INTERP,KGRIB_HANDLE=KGRIB_HD,&
   & KCHUNKSIZE=K2D)

  CALL IO_PUT(Yl_IOSTREAM_SURF,YL_IOREQUEST,&
   & PR3=PGRID(:,K3D*NFLEVG+1:K3D*NFLEVG+K2D,:))
  CALL CLOSE_IOREQUEST(YL_IOREQUEST)
  IF(LDCLOSE) THEN
    CALL CLOSE_IOSTREAM(YL_IOSTREAM_SURF)
    CLFILESURF='CLOSED'
  ENDIF
ENDIF

IF (K3D>0) THEN
  IF (KTYPE_INTERP==3) THEN
!   Create global gridpoint surface pressure on each processor
    IF (.NOT.ALLOCATED(RGPPRS_HR)) ALLOCATE(RGPPRS_HR(NGPTOTG))
    CALL GRID_PSGLOBAL(PSPSP)
  ENDIF
  IONPROCS = MIN(NPROC,NFLEVG)
  ICHUNKS  = (NFLEVG-1)/IONPROCS+1
  DO JROC=1,IONPROCS
    IOPROCS(JROC) = JROC
  ENDDO
  DO JROC=IONPROCS+1,NPROC
    CLFILE='XXXXXXX'
  ENDDO
  DO JCH=1,ICHUNKS
    ISTLEV = (JCH-1)*IONPROCS+1
    IENLEV = MIN(NFLEVG,JCH*IONPROCS)
    ILEVS  = IENLEV-ISTLEV+1
    DO JLEV=ISTLEV,IENLEV
      
      IOPROC = JLEV-ISTLEV+1
      ILEVLIST(IOPROC) = JLEV

      IF (IOPROC == MYPROC) THEN
        WRITE(CLEV,'(I3.3)')JLEV
        CLFILE=CDFILE//'L'//CLEV
      ENDIF
    ENDDO
    IF(LDNEW .OR. CLFILE /= CLFILELEV(JCH)) THEN
      IF(TRIM(CLFILELEV(JCH)) /= 'CLOSED') THEN
        CALL CLOSE_IOSTREAM(YL_IOSTREAM(JCH))
        CLFILELEV(JCH)='CLOSED'
      ENDIF
      CLFILELEV(JCH)=CLFILE   
      CALL SETUP_IOSTREAM(YL_IOSTREAM(JCH),'CIO',TRIM(CLFILELEV(JCH)),CDMODE=CLMODE,&
     & KIOPROCS=IOPROCS(1:ILEVS))
    ENDIF
    ALLOCATE(ZGRID(NPROMA,K3D*ILEVS,NGPBLKS))
    DO JF=1,K3D
      IOFF = (JF-1)*ILEVS
      IGRIB2D(IOFF+1:IOFF+ILEVS) = KTYPE3D(JF)
      ILEVS2D(IOFF+1:IOFF+ILEVS) = ILEVLIST(1:ILEVS)
      IPROC2D(IOFF+1:IOFF+ILEVS) = IOPROCS(1:ILEVS)
    ENDDO
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(JBLK,JF,IOFF,IOFFG)
    DO JBLK=1,NGPBLKS
      DO JF=1,K3D
        IOFF = (JF-1)*ILEVS
        IOFFG = (JF-1)*NFLEVG
        ZGRID(:,IOFF+1:IOFF+ILEVS,JBLK)=  PGRID(:,IOFFG+ISTLEV:IOFFG+IENLEV,JBLK)
      ENDDO
    ENDDO
!$OMP END PARALLEL DO

    CALL SETUP_IOREQUEST(YL_IOREQUEST,'GRIDPOINT_FIELDS',LDGRIB=.TRUE.,&
     & KGRIB2D=IGRIB2D(1:K3D*ILEVS),KLEVS2D=ILEVS2D(1:K3D*ILEVS),CDLEVTYPE='ML',&
     & KPROMA=NPROMA,LDINTERP=LLINTERP,KTYPE_INTERP=KTYPE_INTERP,KGRIB_HANDLE=KGRIB_HD)
    CALL IO_PUT(Yl_IOSTREAM(jch),YL_IOREQUEST,&
     & PR3=ZGRID,KFLDPROC2D=IPROC2D(1:K3D*ILEVS))
    CALL CLOSE_IOREQUEST(YL_IOREQUEST)
    IF(LDCLOSE) THEN
      CALL CLOSE_IOSTREAM(YL_IOSTREAM(JCH))
      CLFILELEV(JCH)='CLOSED'
    ENDIF
    DEALLOCATE(ZGRID)
  ENDDO
ENDIF

IF (ALLOCATED(RGPPRS_HR)) DEALLOCATE(RGPPRS_HR)
IF (ALLOCATED(RGPPRS_LR)) DEALLOCATE(RGPPRS_LR)

IF (LHOOK) CALL DR_HOOK('WRITE_GRID_TRAJ',1,ZHOOK_HANDLE)
END SUBROUTINE WRITE_GRID_TRAJ
