subroutine bgvecs

!*** *BGVECS* - Random backgound vectors.

!   Purpose.
!   --------

!      To estimate variances of background error.
!      If NBGVECS.gt.0 then a random sample of vectors is generated
!      drawn from the same distribution as the background errors. The
!      variances of the sample are estimates of variances of the
!      population. The estimates are filtered to reduce the noise.

!      If NBGVECS.eq.0 then the variances for variables which are
!      explicit are simply copied from the background error buffer FCEBUF,
!      and the variances for variables which are implicit are estimated
!      by applying some (fairly crude) scaling to the explicit ones.

!   Interface.
!   ----------

!      CALL BGVECS

!   Externals.
!   ----------
!              CHAVARIN
!              SETRAN
!              RANDOM_NUMBERS_MIX
!              WRMLPP
!              PPFLUSH
!              PPCLOSE
!              STEPO
!              SC2RDG
!              SC2WRG
!              SCALEAE
!              WRITESD

!   Reference.
!   ----------
!      None yet!

!   Author.
!   -------
!      Mike Fisher *ECMWF*

!   Modifications.
!   --------------
!        Original  94-12-02
!     L.Isaksen    95-06-06 : Reordering of spectral arrays
!     M.Fisher     95-09-20 : Output vectors in GRIB format
!     L.Isaksen    96-05-05 : Adapted for Message Passing
!     E.Andersson  96-08-01 : Precalculated SQRT(SCALP)
!     E.Andersson  98-02-04 : Sigma_b and BgQC for radiances
!     Y.Tremolet   00-08-15 : module TRAJECTORY
!     M.Fisher     02-09-30 : RANDOM_NUMBERS_MIX module
!     G.Desroziers 02-09-16 : also treat Aladin mean wind trajectory
!     Y.Tremolet   02-08-05 : Use control_vectors.
!     C.Cardinali  03-12-03 : Analysis Sensitivity to Observations
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NFD2D    ,NS3D     ,NS2D     ,NSPEC2   ,NFLSUR, NGPBLKS
USE YOMCT0   , ONLY : LFDBOP   ,LOBS
USE YOMCT3   , ONLY : NSTEP
USE YOMCVA   , ONLY : YRSCALPSQRT
USE YOMVAR   , ONLY : NBGVECS  ,MBGVEC   ,NOANEF    ,LTWBGV   ,LWRIBVEC ,&
 & LBGTRUNC ,LBGM     ,NSSBGV_COUNT, LANOBS  , LMODERR
USE YOMJG    , ONLY : LSUBBG, LCTLFG, NOFCEF, N_SPJB_VARS, SPJB_VARS_INFO
USE YOMGEM   , ONLY : NGPTOT
USE YOMFCEB  , ONLY : FCEBUF
USE YOMANEB  , ONLY : NANEBQ   ,NANEBVO  ,ANEBUF
USE YOMMP    , ONLY : NOUTTYPE ,NPSP
USE YOMIO    , ONLY : LPPTSF
USE YOMVRTL  , ONLY : L131TL   ,LTLINT
USE YOMSP    , ONLY : SP5A3    ,SP5A2    ,SPA3     ,SPA2
USE YOMGFL   , ONLY : GFL
USE YOM_YGFL , ONLY : YGFL
USE YOMLUN   , ONLY : NULOUT
USE YOMGRB   , ONLY : NGRBVO, NGRBQ
USE TRAJECTORY_MOD, ONLY: ltrajgp, get_traj_spec
USE YOMRANDOM_STREAMS, ONLY : YD_RANDOM_STREAM_SCAN2MTL
USE YOMMODEL_ERROR, ONLY: NTYPE_MODERR, MSTEPERR, SPMODERR, GPMODERR
USE RANDOM_NUMBERS_MIX
USE CONTROL_VECTORS
USE SPECTRAL_FIELDS_MOD

IMPLICIT NONE

!    DUMMY REAL
REAL(KIND=JPRB) :: ZDUM(1,1)
REAL(KIND=JPRB) :: zane(NPROMA,NOANEF),zfce(NPROMA,NOFCEF)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPA3(:,:,:),ZSPA2(:,:), zgfl(:,:,:,:)

type(control_vector) :: YL_ZCHI_DV

INTEGER(KIND=JPIM) :: iendc, IOUTTYPE, istc, j, jk, jkglo, jlev, jn, &
                    & ISEED, JFIELD, IOFF, iweak

LOGICAL :: LL131TL, llfdbop, LLOBS, LLPPTSF

REAL(KIND=JPRB) :: zrnbg
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "chavarin.intfb.h"
#include "cnt3tl.intfb.h"
#include "estsiga.intfb.h"
#include "fltbgerr.intfb.h"
#include "ppclose.intfb.h"
#include "ppflush.intfb.h"
#include "random_ctlvec.intfb.h"
#include "scaleae.intfb.h"
#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"
#include "setran.intfb.h"
#include "stepo.intfb.h"
#include "stepotl.intfb.h"
#include "supp.intfb.h"
#include "wrmlpp.intfb.h"
#include "sualldfi.intfb.h"
#include "dealldfi.intfb.h"
#include "abor1.intfb.h"
#include "weak_constraint_tl.intfb.h"
#include "add_moderr_tl.intfb.h"

!     ------------------------------------------------------------------

!              1. Randomization Estimate
!              -------------------------

! Preset anebuf to 0

IF (LHOOK) CALL DR_HOOK('BGVECS',0,ZHOOK_HANDLE)
CALL GSTATS(1444,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jkglo,istc,iendc)
do jkglo=1,NGPTOT,NPROMA
  istc=1
  iendc=min(NPROMA,NGPTOT-jkglo+1)
  zane(istc:iendc,:) = 0.0_JPRB

  call SC2WRG (ANEBUF,jkglo,istc,iendc,zane)
enddo
!$OMP END PARALLEL DO
CALL GSTATS(1444,1)

IF (NBGVECS > 0) THEN

  if (LWRIBVEC) then
    LTWBGV = .true.
    llfdbop = LFDBOP
    LFDBOP = .false.
    IOUTTYPE=NOUTTYPE
    NOUTTYPE=1
    LLPPTSF = LPPTSF
    LPPTSF = .TRUE.
    CALL SUPP (' ')
  endif

!--- Take Bg to Fourier space.
  IF(.NOT.(LBGM .OR. LTRAJGP)) THEN
    CALL GSTATS(15,0)
    CALL get_traj_spec(SP5A3,SP5A2,ZDUM)
    CALL GSTATS(15,1)
    call STEPO('0E0000000')
  ENDIF

  call allocate_ctlvec(YL_ZCHI_DV)

  LL131TL=L131TL
  LLOBS=LOBS
  IF(.NOT.LANOBS) THEN
    L131TL=.FALSE.
    LOBS=.FALSE.
  ENDIF
  IF(LANOBS .OR. LBGM) CALL SUALLDFI(NULOUT)

!--- generate a date-dependent seed
  CALL SETRAN (96753,KOUTSEED=ISEED)

!--- loop over random vectors
  do jn=1,NBGVECS
    MBGVEC = jn

!--- generate random vector in control space (and update ISEED)

    CALL GSTATS(1958,0)
    CALL RANDOM_CTLVEC (YL_ZCHI_DV,.TRUE.,.true.,.TRUE.,.TRUE.,.FALSE.,.FALSE.,ISEED)
    CALL GSTATS(1958,1)
    CALL GSTATS(1444,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(j)
    DO j=1,SIZE(YL_ZCHI_DV%data)
      YL_ZCHI_DV%data(j) = YL_ZCHI_DV%data(j) * YRSCALPSQRT%data(j)
    ENDDO
!$OMP END PARALLEL DO
    CALL GSTATS(1444,1)

!--- transform to model variable space

    LSUBBG = .false.
    call CHAVARIN (YL_ZCHI_DV)
    LSUBBG = .NOT.LCTLFG

    IF (LMODERR) THEN
      iweak=MSTEPERR(0)
      IF (NTYPE_MODERR==1.AND.iweak>0) THEN
        CALL ABOR1('BGVECS: not ready for weak constraint')
        CALL WEAK_CONSTRAINT_TL
      ENDIF
      IF (NTYPE_MODERR==4) THEN
        CALL ADD_MODERR_TL(SPMODERR,GPMODERR,0)
      ENDIF
    ENDIF

!--- Integrate TL model, and call obs. operators.
    IF(LBGM) THEN
      LTLINT=.TRUE.
      NSSBGV_COUNT=jn
      call CNT3TL
      LTLINT=.FALSE.
    ELSEIF(LANOBS) THEN
      ALLOCATE(ZSPA3(NFLSUR,NSPEC2,NS3D))
      ALLOCATE(ZSPA2(NSPEC2,NS2D))
      ALLOCATE(ZGFL(NPROMA,NFLEVG,YGFL%NDIM,NGPBLKS))
      CALL GSTATS(1444,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(j)
      DO j=1,NSPEC2
        ZSPA3(:,j,:)=SPA3(:,j,:)
      ENDDO
!$OMP END PARALLEL DO
      CALL GSTATS(1444,1)
      IF(NPSP == 1) ZSPA2(:,1:NFD2D) = SPA2(:,1:NFD2D)
      call gstats(1244,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(j)
      do j=1,NGPBLKS
           zgfl(:,:,:,j)=gfl(:,:,:,j)
      enddo
!$OMP END PARALLEL DO
      call gstats(1244,1)
      LTLINT=.TRUE.
      NSSBGV_COUNT=jn
      call CNT3TL
      LTLINT=.FALSE.
      CALL GSTATS(1444,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(j)
      DO j=1,NSPEC2
        SPA3(:,j,:)=ZSPA3(:,j,:)
      ENDDO
!$OMP END PARALLEL DO
      CALL GSTATS(1444,1)
      IF(NPSP == 1) SPA2(:,1:NFD2D) = ZSPA2(:,1:NFD2D)
      call gstats(1244,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(j)
      do j=1,NGPBLKS
           gfl(:,:,:,j)=zgfl(:,:,:,j)
      enddo
!$OMP END PARALLEL DO
      call gstats(1244,1)
      DEALLOCATE(ZSPA3)
      DEALLOCATE(ZSPA2)
      deallocate(zgfl)
    ENDIF

!--- transform to grid space and accumulate variances

!--- radiance computation needs trajectory, so use TL routines.

    CALL INITIALIZE_RANDOM_NUMBERS (ISEED,yd_random_stream_scan2mtl)
    ISEED=ISEED+1
    IF (.NOT. LBGM) NSTEP=0
    call STEPOTL('0AA00A000')

!--- write out vector

    if (LWRIBVEC) then
      call wrmlpp ('?')
      call PPFLUSH
      call PPCLOSE
    endif
  enddo

  call deallocate_ctlvec(YL_ZCHI_DV)
  IF(LANOBS) CALL DEALLDFI(NULOUT)
  L131TL=LL131TL
  LOBS=LLOBS
  NSSBGV_COUNT=0

  if (LWRIBVEC) then
    LTWBGV = .false.
    LFDBOP = llfdbop
    NOUTTYPE=IOUTTYPE
    LPPTSF = LLPPTSF
    CALL SUPP (' ')
  endif

!--- Divide sum of squares by the number of vectors

  zrnbg = 1.0_JPRB/real(NBGVECS,JPRB)
  call scaleae (zrnbg)
ENDIF

!-----------------------------------------------------------------------

!              2. Copy Explicit Variances from FCEBUF
!              --------------------------------------

IF(NBGVECS == 0) THEN

  DO JFIELD=1,N_SPJB_VARS
    IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE_FCE==NGRBVO) THEN
      ioff=SPJB_VARS_INFO(JFIELD)%IPTFCE-1
      CALL GSTATS(1444,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jkglo,istc,iendc,jk,jlev)
      do jkglo=1,NGPTOT,NPROMA
        istc=1
        iendc=min(NPROMA,NGPTOT-jkglo+1)
        call SC2RDG (FCEBUF,jkglo,istc,iendc,zfce)

        zane(istc:iendc,:) = 0.0_JPRB

        do jk=istc,iendc
          do jlev=1,NFLEVG
            zane(jk,jlev+NANEBVO) = zfce(jk,jlev+ioff)**2
          enddo
        enddo

        call SC2WRG (ANEBUF,jkglo,istc,iendc,zane)
      enddo
!$OMP END PARALLEL DO
      CALL GSTATS(1444,1)
    ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE_FCE==NGRBQ) THEN
      ioff=SPJB_VARS_INFO(JFIELD)%IPTFCE-1
      CALL GSTATS(1444,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jkglo,istc,iendc,jk,jlev)
      do jkglo=1,NGPTOT,NPROMA
        istc=1
        iendc=min(NPROMA,NGPTOT-jkglo+1)
        call SC2RDG (FCEBUF,jkglo,istc,iendc,zfce)

        zane(istc:iendc,:) = 0.0_JPRB

        do jk=istc,iendc
          do jlev=1,NFLEVG
            zane(jk,jlev+NANEBQ ) = zfce(jk,jlev+ioff)**2
          enddo
        enddo

        call SC2WRG (ANEBUF,jkglo,istc,iendc,zane)
      enddo
!$OMP END PARALLEL DO
      CALL GSTATS(1444,1)
    ENDIF
  ENDDO

!--- Estimate variances for variables implicit in Jb

  call ESTSIGA ('bg')
ENDIF

!--- filter to reduce the random noise.

if (LBGTRUNC) call FLTBGERR

IF (LHOOK) CALL DR_HOOK('BGVECS',1,ZHOOK_HANDLE)
end subroutine bgvecs
