SUBROUTINE CVAR2AD (YDVAZG)

!     CVAR2AD : Transformation of model fields, adjoint

!     Purpose: Transform from model variables
!     --------           to Jb-normalized departures w.r.t. background, adjoint

!     Externals: CVARU3AD CVARU2AD BALVERTIAD BALSTATAD
!     ----------

!     Author.     W. HECKLEY    *ECMWF*
!     -------

!     Modifications.
!     --------------
!        W. HECKLEY    91:09:30  Change in call to STEPO
!        Bill Heckley  06:02:92 - Provision for use of mass variable P
!        Bill Heckley  21:02:92 - Use of eigenvectors of vertical correlations
!        Bill Heckley  04:03:93 - Horizontal Jg correlation computation now a
!                                 part of the control variable.
!        Bill Heckley  07:06:93   separation of code into lower level routines
!        Mike Fisher   95-01-20   LSUBFG and CALL MLTEPSAD
!        Mike Fisher   95-09-20   LMVCHV switch
!        Mats Hamrud   95-12-15   Remove NFGM .ne.1 ,cleaning
!        Mike Fisher   96-05-30   Reverse order of CVARU2AD and MLTEPSAD
!        F. Bouttier   96-07-17   Clean comments
!        John Derber   96-07-17   No call to MLTEPS if LSTABAL
!        F. Bouttier   96-12-13   Implement LSTABAL option
!        Mike Fisher   97-02-21   Simplified Kalman Filter
!        M. Siroka     98-06-16   LSTABAL & LELAM option
!        Mike Fisher 2000-10-26   Jb vertical coordinate
!        Mike Fisher 2001-12-03   Omega-equation balance operator
!        Y.Tremolet  2002-08-05   Use control_vectors.
!        D. Dee      2003-12-12   Variational bias correction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Fisher    17-Mar-2004  Wavelet Jb
!        Y.Tremolet    18-Mar-2004 Model error change of variable
!        Y.Tremolet    10-Jun-2004 Initial condition optional in control vector
!        Y.Tremolet    27-Jul-2004 Improve model error
!        P.Burton :  29-09-04 LVARBC cleanup
!        C. Fischer    21-May-2005 Really loop on lam1d cv component 
!                                  and scale by its mean wind std dev
!        Elias Holm    08-10-20  Interface cvar3ad includes gridpoint
!                                fields.
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCVA   , ONLY : YSCALPSQRT, YRSCALPSQRT, NVATOVV, NVA1D, NVA2D, NVA3D
USE YOMGEM   , ONLY : NGPTOT
USE YOMJG    , ONLY : TMEANUVER, LSPFCE
USE YOMSKF   , ONLY : LSKF     ,NSKFVECS ,YSKFROT   ,YSKFMAT   ,SKFSUBMAT
USE YOMANCS  , ONLY : RMDI
USE YOMVAR   , ONLY : LTOVSCV, LVARBC, LTSCV
USE YOMTVRAD , ONLY : TOVSCVX, TOVSCVER,NVATOVLEN
USE YOMSPJB  , ONLY : SPJB
USE YOMDIMO  , ONLY : NOBNTV
USE YOMDIM   , ONLY : NFLEVL, NFLEVG, NPROMA
USE VARBC_SETUP, ONLY : YVARBC, NOBGROUP, NOBPARAM
USE YOMMODEL_ERROR, ONLY : NDIM_MODERR, SPMODERR, GPMODERR, SPCTLMODERR, SPGPMODERR
USE YOMJQ,     ONLY : LQEQB, LJQBAL
USE CONTROL_VECTORS_COMM_MOD, ONLY : ALLGATHER_CVSECTION, SET_CVSECTION
USE CONTROL_VECTORS
USE SPECTRAL_FIELDS_MOD
USE GRIDPOINT_FIELDS_MIX
USE YOMTS, ONLY: TSCVX, TSCVER
USE YOMMP,  ONLY: MYLEVS

IMPLICIT NONE

TYPE(CONTROL_VECTOR),INTENT(INOUT) :: YDVAZG 

TYPE(CONTROL_VECTOR) :: YLCV
INTEGER(KIND=JPIM) :: ii,ji,jj,jf,js,il,jg,ibeg,iend,INGPBLKS
REAL(KIND=JPRB) :: ZWORK1(NSKFVECS), ZWORK2(NSKFVECS)
REAL(KIND=JPRB) :: ZPS
REAL(KIND=JPRB), ALLOCATABLE :: ZGP3(:,:,:), ZGP2(:,:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE
LOGICAL  :: LLSPFCE

#include "abor1.intfb.h"
#include "cvarbcad.intfb.h"
#include "cvar3ad.intfb.h"
#include "grid2specad.intfb.h"
#include "spec_split.intfb.h"

!     ------------------------------------------------------------------

!     4.    Simplified Kalman Filter
!           ------------------------

IF (LHOOK) CALL DR_HOOK('CVAR2AD',0,ZHOOK_HANDLE)
IF (LSKF) THEN
  YDVAZG%data = YDVAZG%data * YSCALPSQRT%data

!*     Apply the inverse transpose of the rotation matrix

  DO JJ=1,NSKFVECS
    ZPS = DOT_PRODUCT(YDVAZG,YSKFROT(JJ))
    YDVAZG%data = YDVAZG%data - ZPS*YSKFROT(JJ)%data
  ENDDO

!*     Apply the square root inverse covariance matrix

  CALL ALLGATHER_CVSECTION(YDVAZG,ZWORK2,1,NSKFVECS)

  ZWORK1=ZWORK2  ! Otherwise needs adding inside SET_CVSECTION

  DO ji=1,NSKFVECS
    DO jj=ji,NSKFVECS
      ZWORK1(jj) = ZWORK1(jj) + SKFSUBMAT(ji,jj)*ZWORK2(ji)
    ENDDO
  ENDDO

  DO jj=1,NSKFVECS
    ZWORK1(jj) = ZWORK1(jj) + DOT_PRODUCT(YSKFMAT(jj),YDVAZG)
  ENDDO

  CALL SET_CVSECTION(ZWORK1,YDVAZG,1,NSKFVECS)

!*     Apply the transpose of the rotation matrix

  DO JJ=NSKFVECS,1,-1
    ZPS = DOT_PRODUCT(YDVAZG,YSKFROT(JJ))
    YDVAZG%data = YDVAZG%data - ZPS*YSKFROT(JJ)%data
  ENDDO

  YDVAZG%data = YDVAZG%data * YRSCALPSQRT%data
ENDIF

!     3.    Square root of correlation operator
!           -----------------------------------

! Initial condition

INGPBLKS = (NGPTOT+NPROMA-1)/NPROMA

ALLOCATE (ZGP3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
ALLOCATE (ZGP2(NPROMA,NVA2D,INGPBLKS))

IF (YDVAZG%LINITCV) THEN
  CALL CVAR3AD(SPJB,YDVAZG,ZGP3,ZGP2,0)
ELSE
  SPJB=0.0_JPRB
  ZGP3=0.0_JPRB
  ZGP2=0.0_JPRB
ENDIF

! Model Error
IF (YDVAZG%nmoderr>0) THEN
  IF (LQEQB) THEN
    il=SPCTLMODERR%nspsizel
    CALL ALLOCATE_CTLVEC(YLCV,LDINIT=.true.,KMERR=0)
    IF (SIZE(YLCV%INITCV)/=il) CALL ABOR1('CVAR2AD: Error LQEQB')
    DO jj=1,NDIM_MODERR
      ii=(jj-1)*il
      YLCV%INITCV(:)=YDVAZG%MODERR(ii+1:ii+il)
      LLSPFCE=LSPFCE
      LSPFCE=.TRUE.
      CALL CVAR3AD(SPCTLMODERR,YLCV,ZGP3,ZGP2,0)
      LSPFCE=LLSPFCE
      GPMODERR(jj)=0.0_JPRB
      CALL SPEC_SPLIT(SPCTLMODERR,SPMODERR(jj),SPGPMODERR)
      CALL GRID2SPECAD(GPMODERR(jj),SPGPMODERR)
    ENDDO
    CALL DEALLOCATE_CTLVEC(YLCV)
  ELSE
    DO jj=1,NDIM_MODERR
      CALL CVAR3AD(SPCTLMODERR,YDVAZG,ZGP3,ZGP2,jj,ldjbbal=LJQBAL)
      GPMODERR(jj)=0.0_JPRB
      CALL SPEC_SPLIT(SPCTLMODERR,SPMODERR(jj),SPGPMODERR)
      CALL GRID2SPECAD(GPMODERR(jj),SPGPMODERR)
    ENDDO
  ENDIF
ENDIF

DEALLOCATE (ZGP3)
DEALLOCATE (ZGP2)

! Parameters to be estimated
IF (LVARBC) THEN ! Obs bias parameters
  IF (NOBPARAM/=SIZE(YDVAZG%params)) &
   & CALL ABOR1('CVAR2AD: Parameter section of control variable wrong length')
  ibeg = 1
  DO jg = 1, NOBGROUP
    iend = ibeg + YVARBC(jg)%NPARAM - 1
    YVARBC(jg)%APARAMS = YDVAZG%params(ibeg:iend)
    ibeg = iend + 1
  ENDDO
  CALL CVARBCAD
ENDIF

! MEAN WIND (LAM)

IF (YDVAZG%lam1d) THEN
  ii=0
  DO jf=1,NVA1D
    DO js=1,NFLEVL
      il=MYLEVS(js)
      ii=ii+1
      IF (TMEANUVER(il,jf)/=0.) THEN
        SPJB%SP1D(js,jf)=SPJB%SP1D(js,jf)+YDVAZG%lamcv(ii)/TMEANUVER(il,jf)
      ENDIF
    ENDDO
  ENDDO
ENDIF

! TOVS control variable

IF (YDVAZG%ltovscv) THEN
  ii=0
  DO jf=1,NVATOVV
    DO js=1,NVATOVLEN
      ii=ii+1
      IF (TOVSCVER(js,jf)/=RMDI) THEN
        TOVSCVX(js,jf) = TOVSCVX(js,jf) + YDVAZG%tovscv(ii) / TOVSCVER(js,jf)
      ENDIF
    ENDDO
  ENDDO
ENDIF

! TS control variable

IF (YDVAZG%ltscv) THEN
  ii=0
    DO js=1,NOBNTV
      ii=ii+1
      IF (TSCVER/=RMDI) THEN
        TSCVX(js) = TSCVX(js) + YDVAZG%tscv(ii) / TSCVER
      ENDIF
    ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('CVAR2AD',1,ZHOOK_HANDLE)
END SUBROUTINE CVAR2AD
