SUBROUTINE GETSATID(KLEN,KBUFID,KBUFTYP)

! ROUTINE.
! --------
!     GETSATID

! PURPOSE.
! --------
!   ROUTINE CALLED BY IFS OBSERVATION SCREENING MODULE SURAD TO RELATE
!   BUFR SATELLITE IDS TO THE RTTOV SERIES/PLATFORM AND SATELLITE ID.
!   NAMELIST INPUT FROM NAMSATS CAN BE USED TO OVERWRITE THE IMPLEMENTED
!   LOOK-UP TABLE FOR SATELLITE IDS, OR TO ALLOW FOR NEW SATELLITES.
!   NOTE THAT THIS ROUTINE ALSO SETS SOME SWITCHES REGARDING THE EMISSIVITY
!   MODEL AND CLOUD CALCULTIONS.

! INPUT.
! ------
!   KLEN    - Length of input arrays
!   KBUFID  - Array of BUFR sat ids in CMA
!   KBUFTYP - Codetype

! VARIABLES DECLARED IN YOMTVRAD - OUTPUT
! ---------------------------------------

!   MSATDIM     - MAX. NUMBER OF SATID'S FOUND PER SERIES
!   MSUBDIM     - MAX. NUMBER OF SUBTYPE'S FOUND PER SERIES
!   MNSERIES    - NUMBER OF SATELLITE SERIES
!   MNSATID     - NUMBER OF SATELLITES TO PROCESS FOR EACH SERIES
!   MNSUBTYPE   - NUMBER OF SUBTYPES FOR EACH SERIES
!   MKSERIES    - ARRAY OF SERIES NUMBERS
!   MKSATID     - ARRAY OF SAT IDs (E.G. 18 FOR NOAA-18) FOR EACH SERIES
!   MKSUBTYPE   - ARRAY OF SUBTYPE (E.G. 1  FOR TOVS) FOR EACH SERIES
!   MBSATID     - ARRAY OF BUFR SAT IDs (E.G. 209 FOR NOAA-18) FOR EACH SERIES
!   MBSUBTYPE   - ARRAY OF CODETYPES FOR EACH SERIES

! AUTHOR.
! -------
! Roger Saunders (Sep 96) 
! Rose Munro (Nov 97) Modified to accomodate multiple satellite series,
!                     id's and subtypes
! Rose Munro (Jan 98) DOCTORIZE'd
! Elisabeth Gerard (Nov 1998) All the DMSP satids F7-->F14 for ERA40
! Tony Mc Jan 2001 add NASA series and sensor AIRS
! Christina Koepken (Oct 2001) Update goes defaults
! Matt Szyndel (Mar 2005) Add MTSAT and FY-2
! Niels Bormann (Feb 2006)  Rewrite, add METOP
! Vincent GUidard (Feb 2009) Add LCO2_DIAG_AIRS
! William Bell (Feb 2009) Add Coriolis-Windsat.
! Qifeng Lu     (Mar 2009)  add FY3A
! Qifeng Lu     (May 2010)  add FY3B
! ----------------------------------------------------------------------
! Modules
! -------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMANCS  , ONLY : NMDI
USE YOMLUN   , ONLY : NULOUT    ,NULNAM
USE YOMTVRAD , ONLY : MSATDIM   ,MSUBDIM   ,MEMIS_SW  ,&
 & MNSERIES  ,MNSATID   ,MNSUBTYPE ,&
 & MKSERIES  ,MKSATID   ,MKSUBTYPE ,&
 & MBSATID   ,MBSUBTYPE ,&
 & LRTCALC_HRETR,LCLD_RTCALC_SCREEN,LCLD_RTCALC_ASSIM,&
 & MSERIES_NOAA,MSERIES_DMSP,MSERIES_EOS,MSERIES_METOP,   &
 & MSERIES_METEOSAT,MSERIES_MTSAT, MSERIES_FY2,MSERIES_TRMM,&
 & MSERIES_GOES,MSERIES_GMS, MSERIES_FY3, MSERIES_CORIOLIS, &
 & MXSERIES, CHSERIES, LRTTOV_INTERPOL,LABORT_RTTOV_FAILURE,&
 & LCO2_DIAG_AIRS
USE YOMCOCTP , ONLY : NGTHR1    ,NGTHR2    ,NGTHRB    ,NSSMI

IMPLICIT NONE

! --------------------------

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBUFID(KLEN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KBUFTYP(KLEN) 

! Parameter statements
! --------------------------
INTEGER(KIND=JPIM), PARAMETER :: JPMXSATIDS=100
INTEGER(KIND=JPIM), PARAMETER :: JPMXSUBTS=50

! Type declarations
! -----------------
TYPE SATID_TABLE_T
  INTEGER(KIND=JPIM)  :: ibufrid       ! WMO satellite ID
  INTEGER(KIND=JPIM)  :: irtseries     ! Platform/series used by RTTOV
  INTEGER(KIND=JPIM)  :: irtid         ! Satellite ID used by RTTOV
END TYPE SATID_TABLE_T

TYPE SUBT_TABLE_T
  INTEGER(KIND=JPIM)  :: icodetype     
  INTEGER(KIND=JPIM)  :: irtseries     ! Platform/series used by RTTOV
  INTEGER(KIND=JPIM)  :: irtsubtype    ! Subtype, for potential use
END TYPE SUBT_TABLE_T

! Local Variables
! ---------------
TYPE(SATID_TABLE_T), DIMENSION(JPMXSATIDS)  :: YLSATID_TABLE
TYPE(SATID_TABLE_T), DIMENSION(JPMXSATIDS)  :: YLSATID_FOUND

TYPE(SUBT_TABLE_T), DIMENSION(JPMXSUBTS)    :: YLSUBT_TABLE
TYPE(SUBT_TABLE_T), DIMENSION(JPMXSUBTS)    :: YLSUBT_FOUND

INTEGER(KIND=JPIM), DIMENSION(JPMXSATIDS,2) :: ISATIDS
INTEGER(KIND=JPIM), DIMENSION(JPMXSUBTS,2)  :: ISUBTS
INTEGER(KIND=JPIM), DIMENSION(MXSERIES)     :: ITMPSERIES_FOUND
INTEGER(KIND=JPIM) :: JS, JK, JL
INTEGER(KIND=JPIM) :: INSAT    ! Number of all satellites known to this routine
INTEGER(KIND=JPIM) :: INSUBT   ! Number of all codetypes known to this routine
INTEGER(KIND=JPIM) :: INSATF
INTEGER(KIND=JPIM) :: INSUBTF
INTEGER(KIND=JPIM) :: INSERIESF
INTEGER(KIND=JPIM) :: ISERIES
INTEGER(KIND=JPIM) :: ISTORE
LOGICAL            :: LLFOUND

CHARACTER(LEN=80)  :: CLOUTPUT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! Namelists
! ---------

#include "namsats.h"

#include "abor1.intfb.h"
#include "posnam.intfb.h"

!-----------------------------------------------------------------------
!* 1.    Read SATID_TABLE from namelist 
!        for new satellites or to overwrite predefined ones
!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('GETSATID',0,ZHOOK_HANDLE)

INSAT=0
INSUBT=0

YLSATID_TABLE(:)%ibufrid   = NMDI
YLSATID_TABLE(:)%irtseries = NMDI
YLSATID_TABLE(:)%irtid     = NMDI

YLSUBT_TABLE(:)%irtseries  = NMDI
YLSUBT_TABLE(:)%icodetype  = NMDI
YLSUBT_TABLE(:)%irtsubtype = NMDI

! Use new surface emissivity model for microwaves (default).
MEMIS_SW=1

CALL POSNAM(NULNAM,'NAMSATS')
READ(NULNAM,NAMSATS)

WRITE(NULOUT, *) 'LCO2_DIAG_AIRS = ',LCO2_DIAG_AIRS

WRITE(NULOUT, '("MEMIS_SW = ",I2)') MEMIS_SW

DO JS = 1, JPMXSATIDS
  IF( COUNT( (/YLSATID_TABLE(JS)%ibufrid,YLSATID_TABLE(JS)%irtseries,&
   &           YLSATID_TABLE(JS)%irtid/) == NMDI) > 0 .AND.      &
   &  ANY(   (/YLSATID_TABLE(JS)%ibufrid,YLSATID_TABLE(JS)%irtseries,&
   &           YLSATID_TABLE(JS)%irtid/) /= NMDI) )THEN
    WRITE(CLOUTPUT,&
     & '("GETSATID: Missing values in satellite triplet ",3(I3,1x))')&
     & YLSATID_TABLE(JS)%ibufrid,YLSATID_TABLE(JS)%irtseries,YLSATID_TABLE(JS)%irtid
    CALL ABOR1(CLOUTPUT)
  ENDIF
ENDDO
INSAT = COUNT(YLSATID_TABLE(:)%ibufrid /= NMDI)
IF(INSAT > 0)THEN
  WRITE(NULOUT,'(A)') 'NAMSATS namelist satid specifications:'
  WRITE(NULOUT,'(A)') 'BUFR-satid RTTOV-series RTTOV-satid'
  DO JS = 1, INSAT
    WRITE(NULOUT,'(7x,I3,11x,I2,10x,I2)') YLSATID_TABLE(JS)%ibufrid,&
     &      YLSATID_TABLE(JS)%irtseries, YLSATID_TABLE(JS)%irtid 
  ENDDO
ELSE
  WRITE(NULOUT,*) 'No NAMSATS namelist satid specifications.'
ENDIF

DO JS = 1, JPMXSUBTS
  IF( COUNT((/YLSUBT_TABLE(JS)%icodetype,YLSUBT_TABLE(JS)%irtseries,&
   &          YLSUBT_TABLE(JS)%irtsubtype/) == NMDI) > 0 .AND.   &
   &  ANY(  (/YLSUBT_TABLE(JS)%icodetype,YLSUBT_TABLE(JS)%irtseries,&
   &          YLSUBT_TABLE(JS)%irtsubtype/) /= NMDI) )THEN
    WRITE(CLOUTPUT,&
     & '("GETSATID: Missing values in codetype triplet ",3(I3,1x))') &
     & YLSUBT_TABLE(JS)%icodetype,YLSUBT_TABLE(JS)%irtseries,YLSUBT_TABLE(JS)%irtsubtype
    CALL ABOR1(CLOUTPUT)
  ENDIF
ENDDO
INSUBT = COUNT(YLSUBT_TABLE(:)%icodetype /= NMDI)
IF(INSUBT > 0)THEN
  WRITE(NULOUT,'(A)') 'NAMSATS namelist subtye specifications:'
  WRITE(NULOUT,'(A)') 'Subtype RTTOV-series RT-subtye'
  DO JS = 1, INSUBT
    WRITE(NULOUT,'(4x,I3,11x,I2,8x,I2)') YLSUBT_TABLE(JS)%icodetype,&
     &       YLSUBT_TABLE(JS)%irtseries, YLSUBT_TABLE(JS)%irtsubtype 
  ENDDO
ELSE
  WRITE(NULOUT,'(A)') 'No NAMSATS namelist subtye specifications.'
ENDIF



!* 2.    Define all known satellite IDs and provide RTTOV 
!        series/platform & satellite equivalent
!-----------------------------------------------------------------------

DO JS = 1, MXSERIES

  ISATIDS = NMDI
  ISUBTS  = NMDI

  SELECT CASE (CHSERIES(JS))
    CASE ('NOAA')
       ISATIDS( 1,1:2) = (/ 901, 5 /)
       ISATIDS( 2,1:2) = (/ 902, 6 /)
       ISATIDS( 3,1:2) = (/ 903, 7 /)
       ISATIDS( 4,1:2) = (/ 904, 8 /)
       ISATIDS( 5,1:2) = (/ 905, 9 /)
       ISATIDS( 6,1:2) = (/ 906,10 /)
       ISATIDS( 7,1:2) = (/ 907,11 /)
       ISATIDS( 8,1:2) = (/ 908,12 /)
       ISATIDS( 9,1:2) = (/ 909,13 /)
       ISATIDS(10,1:2) = (/ 910,14 /)
       ISATIDS(11,1:2) = (/ 911,15 /)
       ISATIDS(12,1:2) = (/ 201,11 /)
       ISATIDS(13,1:2) = (/ 202,12 /)
       ISATIDS(14,1:2) = (/ 205,14 /)
       ISATIDS(15,1:2) = (/ 206,15 /)
       ISATIDS(16,1:2) = (/ 207,16 /)
       ISATIDS(17,1:2) = (/ 208,17 /)
       ISATIDS(18,1:2) = (/ 209,18 /)
       ISATIDS(19,1:2) = (/ 223,19 /)
       ISATIDS(20,1:2) = (/ 222,20 /) 

       ISUBTS(1,1:2) = (/ NGTHR2,1 /)
       ISUBTS(2,1:2) = (/ NGTHR1,2 /)
       ISUBTS(3,1:2) = (/ NGTHRB,3 /)
      
       ISERIES = MSERIES_NOAA

    CASE ('DMSP')
       ISATIDS( 1,1:2) = (/ 240,   7 /)
       ISATIDS( 2,1:2) = (/ 241,   8 /)
       ISATIDS( 3,1:2) = (/ 242,   9 /)
       ISATIDS( 4,1:2) = (/ 243,  10 /)
       ISATIDS( 5,1:2) = (/ 244,  11 /)
       ISATIDS( 6,1:2) = (/ 245,  12 /)
       ISATIDS( 7,1:2) = (/ 246,  13 /)
       ISATIDS( 8,1:2) = (/ 247,  14 /)
       ISATIDS( 9,1:2) = (/ 248,  15 /)
       ISATIDS(10,1:2) = (/ 249,  16 /)   
       ISATIDS(11,1:2) = (/ 285,  17 /)   
       ISATIDS(12,1:2) = (/ 286,  18 /)  

       ISUBTS(1,1:2) = (/ NSSMI,1 /)
       ISUBTS(2,1:2) = (/ NGTHRB,1 /)

       ISERIES = MSERIES_DMSP

    CASE ('METEOSAT')
       ISATIDS( 1,1:2) = (/ 59, 2 /)   ! METEOSAT-2 ... METEOSAT-7
       ISATIDS( 2,1:2) = (/ 50, 3 /)
       ISATIDS( 3,1:2) = (/ 51, 4 /)
       ISATIDS( 4,1:2) = (/ 52, 5 /)
       ISATIDS( 5,1:2) = (/ 53, 6 /)
       ISATIDS( 6,1:2) = (/ 54, 7 /)
       ISATIDS( 7,1:2) = (/ 71, 8 /)   ! MSG-1 (pre operational status)
       ISATIDS( 8,1:2) = (/ 55, 8 /)   ! MSG-1 (operational status)
       ISATIDS( 9,1:2) = (/ 72, 9 /)   ! MSG-2 (pre operational status)
       ISATIDS(10,1:2) = (/ 56, 9 /)   ! MSG-2 (operational status)
       ISATIDS(11,1:2) = (/ 73, 10 /)   ! MSG-3 (pre operational status)
       ISATIDS(12,1:2) = (/ 57, 10 /)   ! MSG-3 (operational status)

       ISUBTS(1,1:2) = (/ NGTHRB,1 /)

       ISERIES = MSERIES_METEOSAT

    CASE ('GOES')
       ISATIDS( 1,1:2) = (/ 252, 8 /)
       ISATIDS( 2,1:2) = (/ 253, 9 /)
       ISATIDS( 3,1:2) = (/ 254,10 /)
       ISATIDS( 4,1:2) = (/ 255,11 /)
       ISATIDS( 5,1:2) = (/ 256,12 /)
       ISATIDS( 6,1:2) = (/ 257,13 /)
       ISATIDS( 7,1:2) = (/ 258,14 /)
       ISATIDS( 8,1:2) = (/ 259,15 /)

       ISUBTS(1,1:2) = (/ NGTHRB,1 /)

       ISERIES = MSERIES_GOES

    CASE ('GMS')
       ISATIDS( 1,1:2) = (/ 105, 1 /)
       ISATIDS( 2,1:2) = (/ 106, 2 /)

       ISUBTS(1,1:2) = (/ NGTHRB,1 /)

       ISERIES = MSERIES_GMS

    CASE ('FY-2')
       ISATIDS( 1,1:2) = (/ 513, 3 /) 

       ISUBTS(1,1:2) = (/ NGTHRB,1 /)

       ISERIES = MSERIES_FY2

    CASE ('FY-3')
       ISATIDS( 1,1:2) = (/ 520, 1 /)
       ISATIDS( 2,1:2) = (/ 521, 2 /)

       ISUBTS(1,1:2) = (/ NSSMI,1 /)
       ISUBTS(2,1:2) = (/ NGTHRB,1 /)

       ISERIES = MSERIES_FY3

    CASE ('TRMM')
       ISATIDS( 1,1:2) = (/ 282, 1 /) 

       ISUBTS(1,1:2) = (/ NGTHRB,1 /)
       ISUBTS(2,1:2) = (/ NSSMI,1 /)

       ISERIES = MSERIES_TRMM
       
    CASE ('ERS')
    CASE ('EOS')
       ISATIDS( 1,1:2) = (/783, 1 /)   ! Terra
       ISATIDS( 1,1:2) = (/784, 2 /)   ! Aqua

       ISUBTS(1,1:2) = (/ NGTHRB,1 /)
       ISUBTS(2,1:2) = (/ NSSMI,1 /)

       ISERIES = MSERIES_EOS

    CASE ('METOP')
       ISATIDS( 1,1:2) = (/  3, 1 /)  !!  METOP-B  
       ISATIDS( 2,1:2) = (/  4, 2 /)  !!  METOP-A (thanks to EUMETSAT's wisdom)
       ISATIDS( 3,1:2) = (/  5, 3 /)  !!  METOP-C

       ISUBTS(1,1:2) = (/ NGTHRB,3 /) ! Consistent with NOAA

       ISERIES = MSERIES_METOP
  
    CASE ('ENVISAT')
!       ISATIDS (1,1:2) = (/ 60, 1   /)  !!  MERIS
!       ISUBTS  (1,1:2) = (/ NSSMI,1 /)  !!  TCWV
       
    CASE ('FY-1')
    CASE ('ADEOS')
    CASE ('MTSAT')
       ISATIDS( 1,1:2) = (/ 171, 1 /)
       ISATIDS( 2,1:2) = (/ 172, 2 /)

       ISUBTS(1,1:2) = (/ NGTHRB,1 /)

       ISERIES = MSERIES_MTSAT

    CASE ('CORIOLIS')
    
       ISATIDS( 1,1:2) = (/ 283,   1 /)
       
       ISUBTS(1,1:2) = (/ NSSMI,1 /)
       ISUBTS(2,1:2) = (/ NGTHRB,1 /)
       
       ISERIES = MSERIES_CORIOLIS
    
    CASE ('NPOESS  ')
    CASE ('GIFTS   ')
    CASE ('AURA    ')

  END SELECT

  DO JK = 1, JPMXSATIDS
    IF( ISATIDS(JK,1) == NMDI ) CYCLE
    IF( INSAT > 0 )THEN
      IF(ANY(YLSATID_TABLE(1:INSAT)%ibufrid    == ISATIDS(JK,1) .AND. &
       &     YLSATID_TABLE(1:INSAT)%irtseries == ISERIES )) CYCLE
    ENDIF
    INSAT = INSAT + 1
    IF(INSAT > JPMXSATIDS) CALL ABOR1('JPMXSATIDS too small')
    YLSATID_TABLE(INSAT)%irtseries = ISERIES
    YLSATID_TABLE(INSAT)%ibufrid   = ISATIDS(JK,1)
    YLSATID_TABLE(INSAT)%irtid     = ISATIDS(JK,2)
  ENDDO

  DO JK = 1, JPMXSUBTS
    IF( ISUBTS(JK,1) == NMDI ) CYCLE
    IF( INSUBT > 0 )THEN
      IF(ANY(YLSUBT_TABLE(1:INSUBT)%icodetype == ISUBTS(JK,1) .AND. &
       &     YLSUBT_TABLE(1:INSUBT)%irtseries == ISERIES )) CYCLE
    ENDIF
    INSUBT = INSUBT + 1
    IF(INSUBT > JPMXSUBTS) CALL ABOR1('JPMXSUBTS too small')
    YLSUBT_TABLE(INSUBT)%irtseries  = ISERIES
    YLSUBT_TABLE(INSUBT)%icodetype  = ISUBTS(JK,1)
    YLSUBT_TABLE(INSUBT)%irtsubtype = ISUBTS(JK,2)
  ENDDO

ENDDO

WRITE(NULOUT,'(A)') 'GETSATID: Known satellites with RT:'
WRITE(NULOUT,'(A)') 'BUFR-satid RTTOV-series Name    RTTOV-satid'
DO JS = 1, INSAT
  WRITE(NULOUT,'(7x,I3,11x,I2,1x,A,9x,I2)') YLSATID_TABLE(JS)%ibufrid,    &
   &      YLSATID_TABLE(JS)%irtseries, CHSERIES(YLSATID_TABLE(JS)%irtseries), &
   &      YLSATID_TABLE(JS)%irtid 
ENDDO


!* 3. Build table of recognised satellite IDs and codetypes 
!-----------------------------------------------------------------------

INSATF  = 0
INSUBTF = 0
DO JK = 1, KLEN

  IF(KBUFID(JK) == -9) CYCLE

  DO JS = 1, INSAT
    IF(KBUFID(JK) == YLSATID_TABLE(JS)%ibufrid)THEN
      ! Recognised this satellite ID, 
      ! see whether we know the codetype for this series
      LLFOUND = .FALSE.
      DO JL = 1, INSUBT
        IF( YLSATID_TABLE(JS)%irtseries == YLSUBT_TABLE(JL)%irtseries .AND.  &
         &  KBUFTYP(JK)              == YLSUBT_TABLE(JL)%icodetype )THEN
          ! Codetype recognised, so add to the list
          LLFOUND = .TRUE.
          IF(INSUBTF > 0)THEN
            IF(ANY(YLSUBT_FOUND(1:INSUBTF)%icodetype==KBUFTYP(JK) .AND.          &
             &     YLSUBT_FOUND(1:INSUBTF)%irtseries==YLSATID_TABLE(JS)%irtseries)) &
             & EXIT
          ENDIF
          INSUBTF = INSUBTF + 1
          YLSUBT_FOUND(INSUBTF)%icodetype  = YLSUBT_TABLE(JL)%icodetype
          YLSUBT_FOUND(INSUBTF)%irtseries  = YLSUBT_TABLE(JL)%irtseries
          YLSUBT_FOUND(INSUBTF)%irtsubtype = YLSUBT_TABLE(JL)%irtsubtype
          EXIT
        ENDIF
      ENDDO

      ! Ignore this match if codetype has not been recognised
      IF(.NOT. LLFOUND)CYCLE

      ! Otherwise, add satellite ID to list
      IF(INSATF > 0)THEN
        IF(ANY(YLSATID_FOUND(1:INSATF)%ibufrid == KBUFID(JK))) EXIT
      ENDIF
      INSATF = INSATF + 1
      YLSATID_FOUND(INSATF)%ibufrid   = YLSATID_TABLE(JS)%ibufrid
      YLSATID_FOUND(INSATF)%irtseries = YLSATID_TABLE(JS)%irtseries
      YLSATID_FOUND(INSATF)%irtid     = YLSATID_TABLE(JS)%irtid
      EXIT
    ENDIF
  ENDDO
ENDDO

!* 4. Build list of series found
!-----------------------------------------------------------------------

INSERIESF = 0
DO JS = 1, INSATF
  IF(INSERIESF > 0)THEN
    IF( ANY(ITMPSERIES_FOUND(1:INSERIESF) == YLSATID_FOUND(JS)%irtseries) )CYCLE
  ENDIF
  INSERIESF = INSERIESF + 1
  ITMPSERIES_FOUND(INSERIESF) = YLSATID_FOUND(JS)%irtseries
  IF(INSERIESF > 1)THEN         ! Sorting of series
    DO JK = INSERIESF, 2, -1
      IF(ITMPSERIES_FOUND(JK) < ITMPSERIES_FOUND(JK-1))THEN
        ISTORE                 = ITMPSERIES_FOUND(JK-1)
        ITMPSERIES_FOUND(JK-1) = ITMPSERIES_FOUND(JK)
        ITMPSERIES_FOUND(JK)   = ISTORE 
      ELSE
        EXIT
      ENDIF
    ENDDO
  ENDIF
ENDDO

!* 5. Set output in YOMTVRAD
!-----------------------------------------------------------------------

MNSERIES = INSERIESF

IF (MNSERIES /= 0) THEN
  ALLOCATE(MKSERIES(MNSERIES))
  ALLOCATE(MNSATID(MNSERIES))
  ALLOCATE(MNSUBTYPE(MNSERIES))

  DO JS = 1, MNSERIES
    MKSERIES(JS)  = ITMPSERIES_FOUND(JS)
    MNSATID(JS)   = COUNT( YLSATID_FOUND(1:INSATF)%irtseries == MKSERIES(JS) )
    MNSUBTYPE(JS) = COUNT( YLSUBT_FOUND(1:INSUBTF)%irtseries == MKSERIES(JS) )
  ENDDO

  MSATDIM = MAXVAL(MNSATID)
  MSUBDIM = MAXVAL(MNSUBTYPE)

  ALLOCATE(MKSATID(MNSERIES,MSATDIM))
  ALLOCATE(MBSATID(MNSERIES,MSATDIM))
  ALLOCATE(MKSUBTYPE(MNSERIES,MSUBDIM))
  ALLOCATE(MBSUBTYPE(MNSERIES,MSUBDIM))

  DO JS = 1, MNSERIES
    MKSATID(JS,1:MNSATID(JS)) = PACK(YLSATID_FOUND(1:INSATF)%irtid, &
     &      MASK= YLSATID_FOUND(1:INSATF)%irtseries == MKSERIES(JS))
    MBSATID(JS,1:MNSATID(JS)) = PACK(YLSATID_FOUND(1:INSATF)%ibufrid, &
     &      MASK= YLSATID_FOUND(1:INSATF)%irtseries == MKSERIES(JS))
    ! Sort by RT-satellite ID
    DO JK = 2, MNSATID(JS)
      DO JL = MNSATID(JS), JK, -1
        IF(MKSATID(JS,JL) < MKSATID(JS,JL-1))THEN
          ISTORE           = MKSATID(JS,JL-1)
          MKSATID(JS,JL-1) = MKSATID(JS,JL)
          MKSATID(JS,JL)   = ISTORE
          ISTORE           = MBSATID(JS,JL-1)
          MBSATID(JS,JL-1) = MBSATID(JS,JL)
          MBSATID(JS,JL)   = ISTORE
        ENDIF
      ENDDO
    ENDDO

    MKSUBTYPE(JS,1:MNSUBTYPE(JS)) = PACK(YLSUBT_FOUND(1:INSUBTF)%irtsubtype, &
     &      MASK= YLSUBT_FOUND(1:INSUBTF)%irtseries == MKSERIES(JS))
    MBSUBTYPE(JS,1:MNSUBTYPE(JS)) = PACK(YLSUBT_FOUND(1:INSUBTF)%icodetype, &
     &      MASK= YLSUBT_FOUND(1:INSUBTF)%irtseries  == MKSERIES(JS))

    DO JK = 1, MNSATID(JS)
      WRITE(NULOUT, '(" GETSATID: ",A8," ID FOUND, ",I3,"= ",A8,"-",I2)') &
       & CHSERIES(MKSERIES(JS)), MBSATID(JS,JK), TRIM(CHSERIES(MKSERIES(JS))),&
       & MKSATID(JS,JK)
    ENDDO
    DO JK = 1, MNSUBTYPE(JS)
      WRITE(NULOUT,'(" GETSATID: ",A8," SUBTYPE FOUND, ",I3,"= SUBTYPE-",I2)')&
       & CHSERIES(MKSERIES(JS)), MBSUBTYPE(JS,JK),MKSUBTYPE(JS,JK)
    ENDDO

  ENDDO

ENDIF

IF (LHOOK) CALL DR_HOOK('GETSATID',1,ZHOOK_HANDLE)

END SUBROUTINE GETSATID
