SUBROUTINE GP_NEAREST(PLATOBS,PLONOBS,KSATOBS,KSEQNO,KLEN,KSTREAM1D)

!**** *GP_NEAREST* - RAIN ASSIMILATION - FIND NEAREST GRID-POINT TO SSMI OBS
!     1D-Var in clouds/precipitation for SSM/I radiances

!     P. Bauer, P. Lopez, E. Moreau, D. Salmond   ECMWF    May 2004

!     PURPOSE.
!     -------

!     For a given satellite sensor, this routine finds pairs of observations
!     and grid points satisfying the conditions that:

!      (i) They are separated by no more than the "search radius" rlimit_sd
!      (ii) Only the closest observation to each grid point is kept

!     The grid-point-to-observation mappings are stored in module variables
!     OB_JOBS, OB_FROM, OB_JROF_GP, GP_COUNT and OB_COUNT.

!**   INTERFACE.
!**   ---------
!**   *CALL* * GP_NEAREST()*  (FROM GP_SSMI)

!        Explicit arguments :


!     AUTHOR: Deborah Salmond
!     DATE  : 02-04-2004

!     Modifications :
!     D.Salmond     : 1-Jul-2005   Bit reproducible OBS/GP matching
!     J.Hague       : 12-Jul-2005  Mass library calls added
!     A.Geer        : 26-Feb-2007  Multiple sensors
!     A.Geer        : 12-Nov-2007  Minor code tidying
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMMP    , ONLY : MYPROC, NFRSTLAT,NLSTLAT,NPTRFRSTLAT,&
               &  NSTA,NONL,NGPSET2PE
USE YOMGEM   , ONLY : NLOENG, NGPTOT
USE YOMCT0   , ONLY : NPROC, N_REGIONS, N_REGIONS_NS
USE YOMLEG   , ONLY : RLATIG
USE YOMLUN   , ONLY : NULOUT
USE YOMCST   , ONLY : RPI, RA
USE MPL_MODULE, ONLY: MPL_SEND, MPL_RECV, MPL_BARRIER, MPL_BROADCAST
USE YOMONEDVAR, ONLY : RLIMIT_SD_1DV, STREAM1D_NUM
USE YOM_SSMI , ONLY : OB_JOBS, OB_FROM, OB_JROF_GP, &
                    & GP_COUNT,OB_COUNT
USE YOMJFH   , ONLY : N_VMASS

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KLEN     ! Number of observations
INTEGER(KIND=JPIM),INTENT(IN) :: KSTREAM1D  ! 1DVar stream
REAL(KIND=JPRB),INTENT(IN) :: PLATOBS(KLEN)
REAL(KIND=JPRB),INTENT(IN) :: PLONOBS(KLEN)
INTEGER(KIND=JPIM),INTENT(IN) :: KSATOBS(KLEN)
INTEGER(KIND=JPIM),INTENT(IN) :: KSEQNO(KLEN)

REAL(KIND=JPRB) :: ZLAT,ZLON,ZD,ZDPISNL
INTEGER(KIND=JPIM) :: IGPTOT(NPROC)
REAL(KIND=JPRB) :: ZCOS_GP_LAT,ZSIN_GP_LAT,ZCOS_GP_LON,ZSIN_GP_LON 
REAL(KIND=JPRB) :: ZCOS_OBS_LAT,ZSIN_OBS_LAT, &
                            &   ZCOS_OBS_LON,ZSIN_OBS_LON
REAL(KIND=JPRB), ALLOCATABLE :: ZOB_DIST(:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IOB_TO(:,:),IOB_JROF(:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IOB_SEQ(:,:)

INTEGER(KIND=JPIM), ALLOCATABLE :: IOB_FROM(:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IOB_JOBS(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZOB_DIST_MIN(:,:)
INTEGER(KIND=JPIM), ALLOCATABLE :: IOB_SEQ_MIN(:,:)
REAL(KIND=JPRB) :: ZLIMIT_DEG

INTEGER(KIND=JPIM) :: JA, JB, IGLOFF, IPE
INTEGER(KIND=JPIM) :: JLAT, JLON, JROF, JLATL, JSAT, JSTREAMOBS
INTEGER(KIND=JPIM) :: IPROC, JPROC, ITAG, JOBS
INTEGER(KIND=JPIM) :: IOBTOTG,IOB_COUNT_DIM,IGPTOTG
INTEGER(KIND=JPIM) :: J1,J2,J3,JJ,JLEN,JBLK=120
REAL(KIND=JPRB), ALLOCATABLE :: ZTMP1(:),ZTMP2(:),ZTMP3(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "onedvar_find_satsens.intfb.h"

IF (LHOOK) CALL DR_HOOK('GP_NEAREST',0,ZHOOK_HANDLE)

IF (KSTREAM1D == 1) THEN

  ALLOCATE(OB_COUNT(NPROC))
  ALLOCATE(GP_COUNT(STREAM1D_NUM,NPROC))
  ALLOCATE(OB_FROM(NGPTOT,STREAM1D_NUM,NPROC))
  ALLOCATE(OB_JOBS(NGPTOT,STREAM1D_NUM,NPROC))
  ALLOCATE(OB_JROF_GP(NGPTOT,STREAM1D_NUM,NPROC))

  OB_COUNT=0
  GP_COUNT=0
  OB_FROM=0
  OB_JOBS=0
  OB_JROF_GP=0
  
  OB_COUNT(MYPROC)=KLEN
    
ENDIF

IGPTOT(MYPROC)=NGPTOT

ITAG=1

! Send to MYPROC = 1

! Gather observation counts from each processor
IF(MYPROC /= 1) THEN

  CALL MPL_SEND(OB_COUNT(MYPROC),&
   & KDEST=1,KTAG=ITAG,CDSTRING='GP_NEAREST' )
  CALL MPL_SEND(IGPTOT(MYPROC),&
   & KDEST=1,KTAG=ITAG+1,CDSTRING='GP_NEAREST' )

ELSE

  DO IPROC=2,NPROC
    CALL MPL_RECV(OB_COUNT(IPROC),&
     & KSOURCE=IPROC,KTAG=ITAG,CDSTRING='GP_NEAREST' )
    CALL MPL_RECV(IGPTOT(IPROC),&
     & KSOURCE=IPROC,KTAG=ITAG+1,CDSTRING='GP_NEAREST' )
  ENDDO

ENDIF


IF(MYPROC == 1)THEN
  IOB_COUNT_DIM=MAXVAL(OB_COUNT(:))
ELSE
  IOB_COUNT_DIM=OB_COUNT(MYPROC)
ENDIF
ALLOCATE(ZOB_DIST(IOB_COUNT_DIM,NPROC))
ALLOCATE(IOB_TO(IOB_COUNT_DIM,NPROC))
ALLOCATE(IOB_JROF(IOB_COUNT_DIM,NPROC))
ALLOCATE(IOB_SEQ(IOB_COUNT_DIM,NPROC))

IOB_TO=0
IOB_JROF=0
ZOB_DIST=9999999.0_JPRB
ZLIMIT_DEG=RLIMIT_SD_1DV*180.0_JPRB/(RPI*RA)


CALL GSTATS(1440,0)
!$OMP PARALLEL& 
!$OMP& PRIVATE(JOBS,JSTREAMOBS,JSAT,JA,JB,IGLOFF,IPE,JROF,&
!$OMP&JLAT,ZLAT,JLATL,ZCOS_GP_LAT,ZSIN_GP_LAT,&
!$OMP&ZDPISNL,JLON,ZCOS_GP_LON,ZSIN_GP_LON,ZD,&
!$OMP&J1,J2,J3,JJ,JLEN,ZTMP1,ZTMP2,ZTMP3,&
!$OMP&ZCOS_OBS_LAT,ZSIN_OBS_LAT,ZCOS_OBS_LON,ZSIN_OBS_LON)

IF(N_VMASS /= 0 ) THEN
  ALLOCATE(ZTMP1(JBLK+N_VMASS))
  ALLOCATE(ZTMP2(JBLK+N_VMASS))
  ALLOCATE(ZTMP3(JBLK+N_VMASS))
  ZTMP1=0.5_JPRB
ENDIF

!$OMP DO SCHEDULE(DYNAMIC,1)

! For each observation on this processor
DO JOBS = 1,OB_COUNT(MYPROC)

  ! Ignore if it is not from the stream we are interested in, or if it is
  ! an unrecognised sensor (which will be flagged later in onedvar_raintb)
  CALL ONEDVAR_FIND_SATSENS( KSATOBS(JOBS), JSAT, JSTREAMOBS)    

  IF ( JSAT == 0 .OR. JSTREAMOBS /= KSTREAM1D) CYCLE 

  ! Calculate the distance from this observation to each model grid point  
  ZCOS_OBS_LAT = COS(PLATOBS(JOBS))
  ZSIN_OBS_LAT = SIN(PLATOBS(JOBS))
  ZCOS_OBS_LON = COS(PLONOBS(JOBS))
  ZSIN_OBS_LON = SIN(PLONOBS(JOBS))

  ! Loop through each distributed memory region of the globe
  DO JA=1,N_REGIONS_NS
    IF(RLATIG(NLSTLAT(JA)) <= (PLATOBS(JOBS)+ZLIMIT_DEG)  &
    &            .AND. (PLATOBS(JOBS)-ZLIMIT_DEG) <=RLATIG(NFRSTLAT(JA))) THEN
      IGLOFF=NPTRFRSTLAT(JA)-1 
      DO JB=1,N_REGIONS(JA)
        IPE=NGPSET2PE(JA,JB)
        JROF = 0

        ! Latitude loop
        DO JLATL = 1,NLSTLAT(JA)-NFRSTLAT(JA)+1
          JLAT=JLATL+NFRSTLAT(JA)-1
          ZLAT = RLATIG(JLAT)
          ZCOS_GP_LAT = COS(ZLAT)
          ZSIN_GP_LAT = SIN(ZLAT)
          ZDPISNL = 2.0_JPRB*RPI/REAL(NLOENG(JLAT),JPRB)

          ! Longitude loop
          J1=NSTA(IGLOFF+JLATL,JB)
          J2=NSTA(IGLOFF+JLATL,JB)+NONL(IGLOFF+JLATL,JB)-1
          IF(N_VMASS == 0) THEN
            DO JLON = J1,J2
              JROF = JROF + 1
              ZLON = ZDPISNL*REAL(JLON-1,JPRB)
              ZCOS_GP_LON = COS(ZLON)
              ZSIN_GP_LON = SIN(ZLON)
              ZD=RA*ACOS(ZCOS_OBS_LAT*ZCOS_GP_LAT* &
             &  (ZCOS_OBS_LON*ZCOS_GP_LON + ZSIN_OBS_LON*ZSIN_GP_LON )  &
             &   + ZSIN_OBS_LAT*ZSIN_GP_LAT)
              IF(ZD < ZOB_DIST(JOBS,MYPROC) .AND. ZD < RLIMIT_SD_1DV) THEN
                
                ! Grid point is within the search radius. This is the 
                ! closest grid point found so far
                ZOB_DIST(JOBS,MYPROC)=ZD
                IOB_TO(JOBS,MYPROC)=IPE  
                IOB_JROF(JOBS,MYPROC)=JROF
                IOB_SEQ(JOBS,MYPROC)=KSEQNO(JOBS)

              ENDIF
            ENDDO !JLON
          ELSE
            DO JJ=J1,J2,JBLK
              J3=MIN(JJ+JBLK-1,J2)
              JLEN=J3-JJ+N_VMASS-MOD(J3-JJ,N_VMASS)
              DO JLON=JJ,J3
                ZTMP1(JLON-JJ+1) = ZDPISNL*REAL(JLON-1,JPRB)
              ENDDO
              CALL VCOS(ZTMP2,ZTMP1,JLEN)
              CALL VSIN(ZTMP3,ZTMP1,JLEN)
              DO JLON=JJ,J3
                ZCOS_GP_LON = ZTMP2(JLON-JJ+1)
                ZSIN_GP_LON = ZTMP3(JLON-JJ+1)
                ZTMP1(JLON-JJ+1)=(ZCOS_OBS_LAT*ZCOS_GP_LAT)* &
               &  (ZCOS_OBS_LON*ZCOS_GP_LON + ZSIN_OBS_LON*ZSIN_GP_LON )  &
               &   + (ZSIN_OBS_LAT*ZSIN_GP_LAT)
              ENDDO
              CALL VACOS(ZTMP2,ZTMP1,JLEN)
              DO JLON = JJ,J3
                JROF = JROF + 1
                ZD=RA*ZTMP2(JLON-JJ+1)
                IF(ZD < ZOB_DIST(JOBS,MYPROC) .AND. ZD < RLIMIT_SD_1DV) THEN

                  ! Grid point is within the search radius. This is the 
                  ! closest grid point found so far             
                  ZOB_DIST(JOBS,MYPROC)=ZD
                  IOB_TO(JOBS,MYPROC)=IPE
                  IOB_JROF(JOBS,MYPROC)=JROF
                  IOB_SEQ(JOBS,MYPROC)=KSEQNO(JOBS)
                  
                ENDIF
              ENDDO 
            ENDDO ! JJ
          ENDIF
        ENDDO !JLAT
      ENDDO !JB
    ENDIF
  ENDDO !JA
ENDDO !JOBS

!$OMP END DO
IF(N_VMASS /= 0 ) THEN
  DEALLOCATE(ZTMP1,ZTMP2,ZTMP3)
ENDIF
!$OMP END PARALLEL
CALL GSTATS(1440,1)


CALL GSTATS(697,0)
! Nearest grid point info is gathered together by processor 1 
IF(MYPROC /= 1) THEN
  IF(OB_COUNT(MYPROC) /=0)THEN
    CALL MPL_SEND(ZOB_DIST(1:OB_COUNT(MYPROC),MYPROC),&
     & KDEST=1,KTAG=ITAG,CDSTRING='GP_NEAREST' )
    CALL MPL_SEND(IOB_TO(1:OB_COUNT(MYPROC),MYPROC),&
     & KDEST=1,KTAG=ITAG+1,CDSTRING='GP_NEAREST' )
    CALL MPL_SEND(IOB_JROF(1:OB_COUNT(MYPROC),MYPROC),&
     & KDEST=1,KTAG=ITAG+2,CDSTRING='GP_NEAREST' )
    CALL MPL_SEND(IOB_SEQ(1:OB_COUNT(MYPROC),MYPROC),&
     & KDEST=1,KTAG=ITAG+3,CDSTRING='GP_NEAREST' )
  ENDIF

ELSE

  DO IPROC=2,NPROC
    IF(OB_COUNT(IPROC) /=0)THEN
      CALL MPL_RECV(ZOB_DIST(1:OB_COUNT(IPROC),IPROC),&
       & KSOURCE=IPROC,KTAG=ITAG,CDSTRING='GP_NEAREST' )
      CALL MPL_RECV(IOB_TO(1:OB_COUNT(IPROC),IPROC),&
       & KSOURCE=IPROC,KTAG=ITAG+1,CDSTRING='GP_NEAREST' )
      CALL MPL_RECV(IOB_JROF(1:OB_COUNT(IPROC),IPROC),&
       & KSOURCE=IPROC,KTAG=ITAG+2,CDSTRING='GP_NEAREST' )
      CALL MPL_RECV(IOB_SEQ(1:OB_COUNT(IPROC),IPROC),&
       & KSOURCE=IPROC,KTAG=ITAG+3,CDSTRING='GP_NEAREST' )
    ENDIF
  ENDDO

ENDIF

CALL MPL_BARRIER(CDSTRING='GP_NEAREST')

ALLOCATE(ZOB_DIST_MIN(IGPTOT(MYPROC),NPROC))
ALLOCATE(IOB_SEQ_MIN(IGPTOT(MYPROC),NPROC))
ALLOCATE(IOB_FROM(IGPTOT(MYPROC),NPROC))
ALLOCATE(IOB_JOBS(IGPTOT(MYPROC),NPROC))

IF (MYPROC == 1) THEN
  IOBTOTG=SUM(OB_COUNT(:))
  WRITE(NULOUT,*)'1D-VAR OB_COUNT:', IOBTOTG, OB_COUNT(:)
ENDIF

CALL MPL_BROADCAST(IOBTOTG,KROOT=1,KTAG=ITAG,CDSTRING='GP_NEAREST')

IF(MYPROC == 1)THEN

  ZOB_DIST_MIN=9999999.0_JPRB
 
  ! Loop over all processors and all observations on each processor 
  DO JPROC=1,NPROC
    DO JOBS=1,OB_COUNT(JPROC)
       
      IF(ZOB_DIST(JOBS,JPROC) /= 9999999.0_JPRB)THEN

        ! This observation was found to be within the search radius of a grid point      
        JROF=IOB_JROF(JOBS,JPROC)
        IPROC=IOB_TO(JOBS,JPROC)
        IF(ZOB_DIST(JOBS,JPROC) < ZOB_DIST_MIN(JROF,IPROC))THEN
        
          ! This observation is the closest to the grid point yet found
          ZOB_DIST_MIN(JROF,IPROC)=ZOB_DIST(JOBS,JPROC)
          IOB_SEQ_MIN(JROF,IPROC)=IOB_SEQ(JOBS,JPROC)
          IOB_FROM(JROF,IPROC)=JPROC
          IOB_JOBS(JROF,IPROC)=JOBS
          
        ELSEIF(ZOB_DIST(JOBS,JPROC) == ZOB_DIST_MIN(JROF,IPROC) .AND. &
            &    IOB_SEQ(JOBS,JPROC) < IOB_SEQ_MIN(JROF,IPROC)) THEN
            
          ! When distances are equal, take the observation with the lowest
          ! sequence number. This preserves bit reproducibility            
          IOB_SEQ_MIN(JROF,IPROC)=IOB_SEQ(JOBS,JPROC)
          IOB_FROM(JROF,IPROC)=JPROC
          IOB_JOBS(JROF,IPROC)=JOBS
          
        ENDIF
        
      ENDIF
    ENDDO
  ENDDO

  ! Loop over all processors and all grid points on each processor   
  DO IPROC=1,NPROC
    DO JROF=1,IGPTOT(IPROC)
      IF(ZOB_DIST_MIN(JROF,IPROC) /= 9999999.0_JPRB)THEN
      
        ! This grid point has a valid observation
        
        ! Total grid points with valid observations (from the
        ! sensor of interest) on this processor
        GP_COUNT(KSTREAM1D,IPROC)=GP_COUNT(KSTREAM1D,IPROC)+1
        
        ! The processor this observation belongs to
        OB_FROM(GP_COUNT(KSTREAM1D,IPROC),KSTREAM1D,IPROC)=IOB_FROM(JROF,IPROC)
        
        ! Its index in the observation array on that processor
        OB_JOBS(GP_COUNT(KSTREAM1D,IPROC),KSTREAM1D,IPROC)=IOB_JOBS(JROF,IPROC)
        
        ! The index of this grid point in this processor's grid point array
        OB_JROF_GP(GP_COUNT(KSTREAM1D,IPROC),KSTREAM1D,IPROC)=JROF
        
      ENDIF
    ENDDO
  ENDDO

  DO IPROC=2,NPROC
    CALL MPL_SEND(GP_COUNT(KSTREAM1D,IPROC),&
     & KDEST=IPROC,KTAG=ITAG,CDSTRING='GP_NEAREST' )
  ENDDO

  IGPTOTG=SUM(GP_COUNT(KSTREAM1D,:))

  WRITE(NULOUT,*)'1D-VAR: STREAM: ',KSTREAM1D,', GP_COUNT:',IGPTOTG,GP_COUNT(KSTREAM1D,:)
  
  ! Share this information with all processors
  DO IPROC=2,NPROC
    IF(GP_COUNT(KSTREAM1D,IPROC) /= 0)THEN
      CALL MPL_SEND(OB_FROM(1:GP_COUNT(KSTREAM1D,IPROC),KSTREAM1D,IPROC),&
       & KDEST=IPROC,KTAG=ITAG+1,CDSTRING='GP_NEAREST' )
      CALL MPL_SEND(OB_JOBS(1:GP_COUNT(KSTREAM1D,IPROC),KSTREAM1D,IPROC),&
       & KDEST=IPROC,KTAG=ITAG+2,CDSTRING='GP_NEAREST' )
      CALL MPL_SEND(OB_JROF_GP(1:GP_COUNT(KSTREAM1D,IPROC),KSTREAM1D,IPROC),&
       & KDEST=IPROC,KTAG=ITAG+3,CDSTRING='GP_NEAREST' )
    ENDIF
  ENDDO

ELSE

  CALL MPL_RECV(GP_COUNT(KSTREAM1D,MYPROC),&
   & KSOURCE=1,KTAG=ITAG,CDSTRING='GP_NEAREST' )

  IF(GP_COUNT(KSTREAM1D,MYPROC) /= 0)THEN
    CALL MPL_RECV(OB_FROM(1:GP_COUNT(KSTREAM1D,MYPROC),KSTREAM1D,MYPROC),&
     & KSOURCE=1,KTAG=ITAG+1,CDSTRING='GP_NEAREST' )
    CALL MPL_RECV(OB_JOBS(1:GP_COUNT(KSTREAM1D,MYPROC),KSTREAM1D,MYPROC),&
     & KSOURCE=1,KTAG=ITAG+2,CDSTRING='GP_NEAREST' )
    CALL MPL_RECV(OB_JROF_GP(1:GP_COUNT(KSTREAM1D,MYPROC),KSTREAM1D,MYPROC),&
     & KSOURCE=1,KTAG=ITAG+3,CDSTRING='GP_NEAREST' )
  ENDIF

ENDIF

CALL MPL_BARRIER(CDSTRING='GP_NEAREST')
CALL GSTATS(697,1)

DEALLOCATE(ZOB_DIST)
DEALLOCATE(IOB_TO)
DEALLOCATE(IOB_JROF)
DEALLOCATE(ZOB_DIST_MIN)
DEALLOCATE(IOB_SEQ_MIN)
DEALLOCATE(IOB_FROM)
DEALLOCATE(IOB_JOBS)
DEALLOCATE(IOB_SEQ)

IF (LHOOK) CALL DR_HOOK('GP_NEAREST',1,ZHOOK_HANDLE)

END SUBROUTINE GP_NEAREST
