SUBROUTINE GP_SSMI_IGP2OBS(KFIELD_GP,KFIELD,KSTREAM1D)        

!**** *GP_SSMI_IGP2OBS* - RAIN ASSIMILATION
!     1D-Var in clouds/precipitation for SSM/I radiances

!     P. Bauer, P. Lopez, E. Moreau, D. Salmond   ECMWF    May 2004

!     PURPOSE.
!     -------
!     MAP SSMI OBS FROM GRID-POINT SPACE TO OBS PROCESSORS

!     Copies input integer array (KFIELD_GP) in grid-point
!     space to output integer array (KFIELD) in obs space.
!     Because there may be more than one 1D-Var stream in
!     grid_point space, this function can be called multiple 
!     times to build up data in obs space.

!     GP_SSMI_GP2OBS does similar but for arrays of reals. 

!**   INTERFACE.
!**   ---------
!**   *CALL* * GP_SSMI_IGP2OBS()*  (FROM GP_SSMI)

!        Explicit arguments :


!     AUTHOR: Deborah Salmond
!     DATE  : 02-04-2004

!     Modifications :
!     D.Salmond     :  1-Jul-2005 Bit reproducible OBS/GP matching
!     A.Geer        : 30-Mar-2006 Changed 1D-Var failure codes
!     A.Geer        : 23-Feb-2007 Multiple sensors
!     ------------------------------------------------------------------


USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMMP    , ONLY : MYPROC
USE YOMCT0   , ONLY : NPROC
USE YOMGEM   , ONLY : NGPTOT
USE YOM_SSMI , ONLY : OB_JOBS, OB_FROM, OB_JROF_GP, &
                    & GP_COUNT,OB_COUNT
USE MPL_MODULE, ONLY : MPL_SEND, MPL_RECV, MPL_BARRIER

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN) :: KFIELD_GP(NGPTOT), KSTREAM1D
INTEGER(KIND=JPIM),INTENT(INOUT) :: KFIELD(OB_COUNT(MYPROC))

INTEGER(KIND=JPIM) :: ITAG,IDIM_GP,IDIM_OB,II,IPROC
INTEGER(KIND=JPIM),ALLOCATABLE :: IBUF_GP(:,:)
INTEGER(KIND=JPIM),ALLOCATABLE :: IBUF_OBS(:,:)
INTEGER(KIND=JPIM),ALLOCATABLE :: IBUF(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('GP_SSMI_IGP2OBS',0,ZHOOK_HANDLE)

CALL GSTATS(699,0)
ITAG=1

IF(MYPROC /= 1)THEN

  ! Send current contents of obs space to processor 1
  IF(OB_COUNT(MYPROC) /= 0)THEN
    CALL MPL_SEND(KFIELD(1:OB_COUNT(MYPROC)),&
     & KDEST=1,KTAG=ITAG,CDSTRING='GP_SSMI_IGP2OBS' )
  ENDIF
  
ELSE

  ! A central buffer of all obs and grid space data in processor 1
  IDIM_GP=MAXVAL(GP_COUNT(KSTREAM1D,:))
  IDIM_OB=MAXVAL(OB_COUNT)
  ALLOCATE(IBUF_GP(IDIM_GP,NPROC))
  ALLOCATE(IBUF_OBS(IDIM_OB,NPROC))
  IBUF_GP=-1
  
  ! Gather the current obs space contents
  DO IPROC=2,NPROC
    IF(OB_COUNT(IPROC) /= 0)THEN
      CALL MPL_RECV(IBUF_OBS(1:OB_COUNT(IPROC),IPROC),&
         & KSOURCE=IPROC,KTAG=ITAG,CDSTRING='GP_SSMI_IGP2OBS' )
    ENDIF
  ENDDO

  ! And from this processor (1)
  IBUF_OBS(1:OB_COUNT(1),1) = KFIELD(:) 

ENDIF

ALLOCATE(IBUF(GP_COUNT(KSTREAM1D,MYPROC)))
IBUF=-1

DO II=1,GP_COUNT(KSTREAM1D,MYPROC)
  IBUF(II)=KFIELD_GP(OB_JROF_GP(II,KSTREAM1D,MYPROC))
ENDDO

! Send back to MYPROC=1 and copy back to OBS addressing
IF(MYPROC /=1)THEN
  IF(GP_COUNT(KSTREAM1D,MYPROC) /= 0)THEN
    CALL MPL_SEND(IBUF(1:GP_COUNT(KSTREAM1D,MYPROC)),&
      & KDEST=1,KTAG=ITAG,CDSTRING='GP_SSMI_IGP2OBS' )
  ENDIF
  
ELSE
  
  IBUF_GP(1:GP_COUNT(KSTREAM1D,1),1)= &
            &      IBUF(1:GP_COUNT(KSTREAM1D,1))
  
  DO IPROC=2,NPROC
    IF(GP_COUNT(KSTREAM1D,IPROC) /= 0)THEN
      CALL MPL_RECV(IBUF_GP(1:GP_COUNT(KSTREAM1D,IPROC),IPROC),&
         & KSOURCE=IPROC,KTAG=ITAG,CDSTRING='GP_SSMI_IGP2OBS' )
    ENDIF
  ENDDO
  
  DO IPROC=1,NPROC
    DO II=1,GP_COUNT(KSTREAM1D,IPROC)
      IBUF_OBS(OB_JOBS(II,KSTREAM1D,IPROC),OB_FROM(II,KSTREAM1D,IPROC))= &
           &                 IBUF_GP(II,IPROC)
    ENDDO
  ENDDO
  
ENDIF
   
! Send back to PE that owns OB  
IF(MYPROC == 1)THEN
  
  DO IPROC=2,NPROC
    IF(OB_COUNT(IPROC) /= 0)THEN
      CALL MPL_SEND(IBUF_OBS(1:OB_COUNT(IPROC),IPROC),&
        & KDEST=IPROC,KTAG=ITAG,CDSTRING='GP_SSMI_IGP2OBS' )
    ENDIF
  ENDDO
  
  KFIELD(1:OB_COUNT(MYPROC)) = IBUF_OBS(1:OB_COUNT(MYPROC),MYPROC)
  
ELSE
  
  IF(OB_COUNT(MYPROC) /= 0)THEN
    CALL MPL_RECV(KFIELD(1:OB_COUNT(MYPROC)),&
     & KSOURCE=1,KTAG=ITAG,CDSTRING='GP_SSMI_IGP2OBS' )
  ENDIF
   
ENDIF
    
IF(MYPROC == 1)THEN
  DEALLOCATE(IBUF_GP)
  DEALLOCATE(IBUF_OBS)
ENDIF

DEALLOCATE(IBUF)

CALL MPL_BARRIER(CDSTRING='GP_SSMI_IGP2OBS')
CALL GSTATS(699,1)

IF (LHOOK) CALL DR_HOOK('GP_SSMI_IGP2OBS',1,ZHOOK_HANDLE)

END SUBROUTINE GP_SSMI_IGP2OBS
