SUBROUTINE JBMATINTERP (PLON0G,KLON_OFFSET,PDLONE, &
 & PLATG,KLAT_OFFSET,KLATG,KLATE,KLONE,&
 & PLONM,PLATM,KNPTS,KNDIM1,KINDEX)
!     PURPOSE.
!     --------

!        To return a list of indices to the nearest vertical covariance matrix
!        at each gridpoint for the wavelet Jb.

!**   INTERFACE.
!     ----------
!        INPUT:
!           *PLON0G* - ARRAY CONTAINING LONGITUDE (RADIANS) OF THE FIRST
!                      POINT OF EACH ROW OF THE GLOBAL INPUT GRID
!           *KLON_OFFSET* - OFFSET TO THIS PEs ZEROTH LONGITUDES
!           *PDLONE* - ARRAY CONTAINING LONGITUDINAL GRID SPACING (ASSUMED
!                      CONSTANT FOR EACH ROW)
!           *PLATG*  - ARRAY CONTAINING LATITUDES OF THE GLOBAL INPUT GRID
!           *KLAT_OFFSET* - OFFSET TO THIS PEs ZEROTH LATITUDE
!           *KLATG*  - NUMBER OF LATITUDE ROWS ON THE GLOBAL INPUT GRID
!           *KLATE*  - NUMBER OF LATITUDE ROWS ON THE INPUT GRID ON THIS PE
!           *KLONE*  - ARRAY CONTAINING NUMBER OF LONGITUDE POINTS
!                      ALONG EACH LATITUDE ROW OF THE INPUT GRID
!           *PLONM*  - ARRAY CONTAINING LONGITUDES (RADIANS) OF THE
!                      INTERPOLATION POINTS
!           *PLATM*  - ARRAY CONTAINING LATITUDES (RADIANS) OF THE
!                      INTERPOLATION POINTS
!           *KNPTS*  - NUMBER OF INTERPOLATION POINTS
!           *KNDIM1* - NUMBER OF INPUT GRIDPOINTS

!        OUTPUT:
!           *KINDEX*  - ARRAY OF INDICES TO MATRICES

!     METHOD.
!     -------
!          Find the nearest row, then find the nearest point in that row.

!     EXTERNALS.
!     -----------

!     REFERENCE.
!     ----------

!        NONE.

!     AUTHOR.
!     -------

!        M. Fisher  *ECMWF* 2000-09-22 (based on SUHIFCE)

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Fisher      17-Mar-2004 Interpolation too expensive. Return indices to nearest matrices.

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCST   , ONLY : RPI
USE YOMLUN   , ONLY : NULERR
USE YOMMP    , ONLY : MYPROC

IMPLICIT NONE

!     ARGUMENTS
INTEGER(KIND=JPIM),INTENT(IN)    :: KLATG 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLATE 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNPTS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNDIM1 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLON0G(KLATG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON_OFFSET(KLATE) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDLONE(KLATE) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLATG(KLATG) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLAT_OFFSET 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLONE(KLATE) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLONM(KNPTS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLATM(KNPTS) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KINDEX(KNPTS) 
INTEGER(KIND=JPIM) :: I, ILATE, ILATN, ILATS, ILONE, ILONNE, ILONNW,&
 & ILONSE, ILONSW, IPT  

REAL(KIND=JPRB) :: ZDLAT, ZDLONN, ZDLONS, ZLATM, ZLONMN, ZLONMS, ZLONNW, ZLONSW, ZEPS

INTEGER(KIND=JPIM), ALLOCATABLE :: INDER(:,:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"

IF (LHOOK) CALL DR_HOOK('JBMATINTERP',0,ZHOOK_HANDLE)
ALLOCATE (INDER(MAXVAL(KLONE(1:KLATE)),KLATE))

IF (PLATG(KLAT_OFFSET+KLATE) > PLATG(KLAT_OFFSET+1)) THEN
  WRITE (NULERR,*)'INTERP2 EXPECTS LATITUDE ROWS IN DECREASING ORDER'
  CALL ABOR1 ('INTERP2: LATITUDE ROWS NOT IN DECREASING ORDER')
ENDIF

!--- GENERATE A MAP FROM 2D GRIDPOINT INDEXING TO 1D ARRAY INDEXING

I=0
DO ILATE=1,KLATE
  DO ILONE=1,KLONE(ILATE)
    I=I+1
    INDER(ILONE,ILATE) = I
  ENDDO
ENDDO

!--- FOR EACH INTERPOLATION POINT, FIND NEAREST ROW OF THE INPUT GRID

ZEPS=0.5*SQRT(EPSILON(ZEPS))

DO IPT=1,KNPTS
  ZLATM = PLATM(IPT)
  IF (ZLATM <= PLATG(KLATE+KLAT_OFFSET)) THEN
    ILATS = KLATE
    ILATN = KLATE
  ELSE
    ILATS = 1
    DO WHILE (ZLATM < PLATG(ILATS+KLAT_OFFSET))
      ILATS = ILATS+1
    ENDDO
    ILATN = MAX(1,ILATS-1)
  ENDIF

  IF (ILATN == ILATS) THEN
    ZDLAT = 0._JPRB
  ELSE
    ZDLAT = (ZLATM-PLATG(ILATN+KLAT_OFFSET)) &
     & /(PLATG(ILATS+KLAT_OFFSET)-PLATG(ILATN+KLAT_OFFSET))  
  ENDIF

!--- FIND THE NEAREST POINT IN THE ROW

  IF (ZDLAT<0.5) THEN
    ZLONMN = PLONM(IPT)
    DO WHILE (ZLONMN < PLON0G(ILATN+KLAT_OFFSET))
      ZLONMN=ZLONMN+2.0_JPRB*RPI
    ENDDO

    ILONNW = 1 + INT( (ZLONMN+ZEPS-PLON0G(ILATN+KLAT_OFFSET))/PDLONE(ILATN) )&
     & - KLON_OFFSET(ILATN)  

    IF (ILONNW > KLONE(ILATN)) THEN
      WRITE (NULERR,*) 'jbmatinterp: ILONNW out of bounds on PE',MYPROC
      CALL ABOR1 ('jbmatinterp: ILONNW out of bounds')
    ENDIF

    ILONNE = 1 + MOD( ILONNW , KLONE(ILATN) )

    ZLONNW = PLON0G(ILATN+KLAT_OFFSET) &
     & +PDLONE(ILATN)*REAL(ILONNW-1+KLON_OFFSET(ILATN),JPRB)  

    ZDLONN = (ZLONMN-ZLONNW)/PDLONE(ILATN)

    IF (ZDLONN<0.5) THEN
      KINDEX(IPT) = INDER(ILONNW,ILATN)
    ELSE
      KINDEX(IPT) = INDER(ILONNE,ILATN)
    ENDIF

  ELSE

    ZLONMS = PLONM(IPT)
    DO WHILE (ZLONMS < PLON0G(ILATS+KLAT_OFFSET))
      ZLONMS=ZLONMS+2.0_JPRB*RPI
    ENDDO

    ILONSW = 1 + INT( (ZLONMS+ZEPS-PLON0G(ILATS+KLAT_OFFSET))/PDLONE(ILATS) )&
     & - KLON_OFFSET(ILATS)  

    IF (ILONSW > KLONE(ILATS)) THEN
      WRITE (NULERR,*) 'jbmatinterp: ILONSW out of bounds on PE',MYPROC
      CALL ABOR1 ('jbmatinterp: ILONSW out of bounds')
    ENDIF

    ILONSE = 1 + MOD( ILONSW , KLONE(ILATS) )

    ZLONSW = PLON0G(ILATS+KLAT_OFFSET) &
     & +PDLONE(ILATS)*REAL(ILONSW-1+KLON_OFFSET(ILATS),JPRB)  

    ZDLONS = (ZLONMS-ZLONSW)/PDLONE(ILATS)

    IF (ZDLONS<0.5) THEN
      KINDEX(IPT) = INDER(ILONSW,ILATS)
    ELSE
      KINDEX(IPT) = INDER(ILONSE,ILATS)
    ENDIF
  ENDIF

ENDDO

DEALLOCATE (INDER)

IF (LHOOK) CALL DR_HOOK('JBMATINTERP',1,ZHOOK_HANDLE)
END SUBROUTINE JBMATINTERP
