SUBROUTINE JBVCOORD_INTERPOLATE (CDCONF,KNUMB,KSTGLO,PGP3JB,PGP2JB)

!      JBVCOORD_INTERPOLATE - Interpolation to and from Jb vertical levels

!     Arguments :
!     -----------
!     Input:
!       CDCONF     - M=> FROM Jb vertical levels TO model levels
!                    N=> FROM model levels TO Jb vertical levels
!       KNUMB - number of elements for which computations
!               are performed (used in message passing version)
!       KSTGLO - global offset

!     Input/output:
!       PGP3JB - 3d gridpoint fields (=transform of SPA3JB)
!       PGP2JB - 2d gridpoint fields (=transform of SPA2JB)

!     Author:  M. Fisher    *ECMWF*
!     -------

!     Modifications:
!     --------------
!     ORIGINAL 2000-03-09.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        E.Holm        20-Oct-2005 Simplify call using same variables in/out
!                                  and introduce PGP3JB and PGP2JB
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMFCEB  , ONLY : GPBUF_JBVCOORD
USE YOMCVA  , ONLY : NVA3D, NVA2D

!     ------------------------------------------------------------------

IMPLICIT NONE

CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGP3JB(NPROMA,NFLEVG,NVA3D)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PGP2JB(NPROMA,NVA2D)

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JFLD, IST, ISTC, IEND, IENDC, IDIRECTION
REAL(KIND=JPRB) :: ZINTER(NPROMA,NFLEVG), Z_LEVEL(NPROMA,NFLEVG,2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "sc2rdg.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('JBVCOORD_INTERPOLATE',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF (CDCONF=='M') THEN
  IDIRECTION=1
ELSEIF (CDCONF=='N') THEN
  IDIRECTION=2
ELSE
  CALL ABOR1 ('BAD CDCONF IN JBVCOORD_INTERPOLATE')
ENDIF

IST   = 1
IEND  = KNUMB
ISTC  = 1
IENDC = KNUMB

!       Read the fractional level numbers from the buffer

CALL SC2RDG(GPBUF_JBVCOORD,KSTGLO,ISTC,IENDC,Z_LEVEL)

!       Interpolate all fields in PGP3JB

DO JFLD = 1, NVA3D
  CALL TE_INTERP (PGP3JB(:,:,JFLD),ZINTER)
  PGP3JB(:,:,JFLD)=ZINTER(:,:)
ENDDO

IF (LHOOK) CALL DR_HOOK('JBVCOORD_INTERPOLATE',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

CONTAINS
SUBROUTINE TE_INTERP (PIN,POUT)

REAL(KIND=JPRB), INTENT(IN)  :: PIN (NPROMA,NFLEVG)
REAL(KIND=JPRB), INTENT(OUT) :: POUT(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZALPHA
INTEGER(KIND=JPIM) :: JROF, JLEV, IABOVE, IBELOW
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('JBVCOORD_INTERPOLATE:TE_INTERP',0,ZHOOK_HANDLE)
DO JLEV=1,NFLEVG
  DO JROF=IST,IEND
    IABOVE = MAX(INT(Z_LEVEL(JROF,JLEV,IDIRECTION)),1)
    IBELOW = MIN(IABOVE+1,NFLEVG)
    ZALPHA = Z_LEVEL(JROF,JLEV,IDIRECTION) - REAL(IABOVE,JPRB)

    IF (IABOVE == 1 .OR. IBELOW == NFLEVG) THEN
!---        linear interpolation
      POUT(JROF,JLEV) =    ZALPHA     *PIN(JROF,IBELOW) &
       & +(1.0_JPRB-ZALPHA)*PIN(JROF,IABOVE)  
    ELSE
!---        fit a cubic to the nearest 4 points
      POUT(JROF,JLEV) = &
       & - ZALPHA * (ZALPHA-1.0_JPRB) * (ZALPHA-2.0_JPRB) &
       & * PIN(JROF,IABOVE-1) / 6.0_JPRB &
       & + (ZALPHA+1.0_JPRB) * (ZALPHA-1.0_JPRB) * (ZALPHA-2.0_JPRB) &
       & * PIN(JROF,IABOVE) * 0.5_JPRB   &
       & - (ZALPHA+1.0_JPRB) * ZALPHA * (ZALPHA-2.0_JPRB) &
       & * PIN(JROF,IBELOW) * 0.5_JPRB &
       & + (ZALPHA+1.0_JPRB) * ZALPHA * (ZALPHA-1.0_JPRB) &
       & * PIN(JROF,IBELOW+1) / 6.0_JPRB  
    ENDIF
  ENDDO
ENDDO

IF (LHOOK) CALL DR_HOOK('JBVCOORD_INTERPOLATE:TE_INTERP',1,ZHOOK_HANDLE)
END SUBROUTINE TE_INTERP

!     ------------------------------------------------------------------
END SUBROUTINE JBVCOORD_INTERPOLATE
