#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE JBVCOR_WAVELETIN (KSCALE,YD_PCV,PSPA3,PSPA2)

!**** *JBVCOR_WAVELETIN*    - "Wrapper" for wavelet Jb grid space computations

!     PURPOSE.
!     --------

!        Multiple-resolution grid space computations for wavelet Jb.
!        Adds to PSPA3 and PSPA2 the contribution from scale KSCALE.

!**   INTERFACE.
!     ----------

!        *CALL* *JBVCOR_WAVELETIN* (KSCALE,PCV,PSPA3,PSPA2)

!     KSCALE    :  Wavelet scale (or 0 to indicate full model resolution)
!     PCV       :  Input control vector
!     PSPA3   :  Contribution from scale KSACALE is added to PSPA3 on return
!     PSPA2   :  Contribution from scale KSACALE is added to PSPA2 on return

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------
!             *TRANSDIR_WAVELET* - inverse spherical transform
!             *JBVCORG*          - apply sqrt vertical covariance matrices

!     REFERENCE.
!     ----------

!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------

!        M. Fisher     *ECMWF*

!     MODIFICATIONS.
!     --------------
!        ORIGINAL 2001-08-23
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCVA   , ONLY : NVA3D    ,NVA2D
USE YOMDIM   , ONLY : NPROMA   ,NFLEVG, NFLSUR, NSPEC2
USE YOMGEM   , ONLY : NGPTOT
USE YOMMP    , ONLY : NPSP
USE YOMWAVELET , ONLY :  GRID_DEFINITION
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KSCALE 
TYPE(CONTROL_VECTOR),INTENT(IN)    :: YD_PCV 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPA3(NFLSUR,NSPEC2,NVA3D) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSPA2(NSPEC2,NVA2D) 

REAL(KIND=JPRB), ALLOCATABLE :: ZGP3(:,:,:), ZGP2(:,:,:), &
                              & ZGT3(:,:), ZGT2(:,:), &
                              & ZSPA3(:,:,:), ZSPA2(:,:)

INTEGER(KIND=JPIM) :: JKGLO, ICEND, IBL, IOFF, &
 & IGPTOT, INGPBLKS, ISTART, JFLD, IEND, J
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "jbvcorg.intfb.h"
#include "transdir_wavelet.intfb.h"
#include "wavxform.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('JBVCOR_WAVELETIN',0,ZHOOK_HANDLE)
IGPTOT = GRID_DEFINITION(KSCALE)%NGPTOT

INGPBLKS = (IGPTOT+NPROMA-1)/NPROMA

ALLOCATE (ZGP3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
ALLOCATE (ZGP2(NPROMA,NVA2D,INGPBLKS))

CALL GSTATS(1441,0)
!$OMP PARALLEL PRIVATE(JKGLO,ICEND,IBL,JFLD,ISTART,IEND,IOFF,ZGT3,ZGT2)
ALLOCATE (ZGT3(NPROMA,NFLEVG*NVA3D))
ALLOCATE (ZGT2(NPROMA,NVA2D))

!$OMP DO SCHEDULE(STATIC)
DO JKGLO=1,IGPTOT,NPROMA
  ICEND=MIN(NPROMA,IGPTOT-JKGLO+1)
  IBL=(JKGLO-1)/NPROMA+1
  DO JFLD=1,NFLEVG*NVA3D
    ISTART = YD_PCV%ibgn_scale(KSCALE) + (JFLD-1)*NPROMA &
            & + (IBL-1)*NPROMA*(NFLEVG*NVA3D+NVA2D)
    IEND  =ISTART-1+ICEND

    ZGP3(1:ICEND,JFLD,IBL) = YD_PCV%initcv(ISTART:IEND)
  ENDDO
  DO JFLD=1,NVA2D
    ISTART = YD_PCV%ibgn_scale(KSCALE) + (JFLD-1+NFLEVG*NVA3D)*NPROMA &
            & + (IBL-1)*NPROMA*(NFLEVG*NVA3D+NVA2D)
    IEND  =ISTART-1+ICEND

    ZGP2(1:ICEND,JFLD,IBL) = YD_PCV%initcv(ISTART:IEND)
  ENDDO

  IOFF=JKGLO

!    Apply square-roots of vertical covariance
!    matrices in scale-dependent grid space for wavelet Jb.

  CALL JBVCORG ('Q',KSCALE,ICEND,&
   & GRID_DEFINITION(KSCALE)%GELAM(IOFF:),&
   & GRID_DEFINITION(KSCALE)%GELAT(IOFF:),&
   & ZGP3(1,1,IBL) ,ZGP2(1,1,IBL),&
   & ZGT3(1,1)     ,ZGT2(1,1))

  ZGP3(1:ICEND,:,IBL) = ZGT3(1:ICEND,:)
  ZGP2(1:ICEND,:,IBL) = ZGT2(1:ICEND,:)

ENDDO
!$OMP END DO

DEALLOCATE (ZGT2)
DEALLOCATE (ZGT3)

!$OMP END PARALLEL
CALL GSTATS(1441,1)

!    Transform to spectral space, perform the convolution and add in to PSPA3 and PSPA2

ALLOCATE (ZSPA3(NFLSUR,NSPEC2,NVA3D))
ALLOCATE (ZSPA2(NSPEC2,NVA2D))

CALL TRANSDIR_WAVELET (KSCALE, ZSPA3, ZSPA2, ZGP3, ZGP2)
CALL WAVXFORM (KSCALE,ZSPA3,ZSPA2)

CALL GSTATS(1441,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
DO J=1,NSPEC2
  PSPA3(:,J,:) = PSPA3(:,J,:) + ZSPA3(:,J,:)
ENDDO
!$OMP END PARALLEL DO
CALL GSTATS(1441,1)
IF (NPSP == 1) PSPA2(:,:)   = PSPA2(:,:)   + ZSPA2(:,:)

DEALLOCATE (ZSPA3)
DEALLOCATE (ZSPA2)

DEALLOCATE (ZGP2)
DEALLOCATE (ZGP3)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('JBVCOR_WAVELETIN',1,ZHOOK_HANDLE)
END SUBROUTINE JBVCOR_WAVELETIN
