SUBROUTINE JGCORI(LDTOUQ,KSPSTA,KSPEND,YDCV,YDSPEC)

!**** *JGCORI*    - INTERFACE TO MULTIPLICATION/DIVISION
!*                  BY SQRT OF INVERSE CORRELATION MATRIX

!     PURPOSE.
!     --------

!        MULTIPLICATION/DIVISION OF NORMALISED
!        BACKGROUND ERRORS IN SPECTRAL SPACE BY
!        SQRT OF INVERSE CORRELATION MATRIX

!**   INTERFACE.
!     ----------

!        *CALL* *JGCORI*

!        INPUT:
!                LDTOUQ : .TRUE. The Q field may be modified
!                         .FALSE. The Q field is not touched
!                KSPSTA : First spectral coefficients handled
!                KSPEND : Last spectral columns handled

!     METHOD.
!     -------
!        For each zonal wave number M, divide by square root of
!              1) horizontal correlations
!              2) vertical correlations

!     EXTERNALS.
!     ----------
!        JGHCORI, JGVCOR, JGHCOSI

!     REFERENCE.
!     ----------

!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------

!        F. Bouttier   *CNRM/GMAP*

!     MODIFICATIONS.
!     --------------

!        ORIGINAL     94-10-07
!        L. Isaksen   95-06-06 Reordering of spectral arrays
!        Mats Hamrud  95-12-15 Remove NFGM .ne. 1, cleaning
!        L. Isaksen   96-05-05 Adapted for Message Passing
!        E.Holm       97-08-20 Ozone included
!        S. Issara    97-05-20 3DVAR ALADIN
!        M. Siroka    98-05-04 phasing new Jb + ALADIN
!        Y.Tremolet   02-08-05  Use control_vectors.
!        M.Hamrud     03-08-01 GFL introduction
!        C.Fischer    03-09-30 GFL introduction for Aladin
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        M.Fisher      17-Mar-2004 Wavelet Jb
!        B.Chapnik    03-Mar-2006 Phasing CY30R2, merge ejgvcor and jgvcor
!        A. Deckmyn   01-Oct-2008 LAM wavelet Jb
!     -----------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVL   ,NFLEVG   ,NSMAX    ,NUMP, NSPEC2 , &
                    & NGPBLKS  ,NPROMA
USE YOMLAP   , ONLY : NASM0    ,NVALUE   ,MYMS
USE YOMJG    , ONLY : NGEOCOR  ,SPJB_VARS_INFO
USE YOMCT0   , ONLY : NCONF     ,NPRTRV   ,LELAM
USE YOMMP    , ONLY : NSPEC2V, NPSP
USE YOMWAVELET,ONLY : LJBWAVELET, N_WAVELET_SCALES
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR
USE YOMGRB   , ONLY : NGRBQ, NGRBO3
USE YOMCVA   , ONLY : NVA3D, NVA2D
USE SPECTRAL_FIELDS_MOD

!     ------------------------------------------------------------------

IMPLICIT NONE

LOGICAL           ,INTENT(IN)    :: LDTOUQ 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSPSTA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSPEND 
TYPE(CONTROL_VECTOR),INTENT(IN)    :: YDCV 
TYPE(SPECTRAL_FIELD),INTENT(INOUT) :: YDSPEC

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZSP3DG(NFLEVG,NSPEC2V,NVA3D)
REAL(KIND=JPRB) :: ZSP2DG(NSPEC2V,NVA2D)
REAL(KIND=JPRB) :: ZSP(NSPEC2,NVA2D)
REAL(KIND=JPRB), ALLOCATABLE :: ZGP2(:,:,:),ZGP3(:,:,:)

INTEGER(KIND=JPIM) :: JMLOC, IDIM, IM, ISP2END, ISP2STA, ISURDIM, JSCALE
LOGICAL :: LLSELECT3D(NVA3D), LLSELECT2D(NVA2D)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "ejghcori.intfb.h"
#include "jbvcor_waveletin.intfb.h"
#include "jghcori.intfb.h"
#include "jgvcor.intfb.h"
#include "jghcosi.intfb.h"
#include "trmtos.intfb.h"
#include "trstom.intfb.h"
#include "ejbwav_cv2wav.intfb.h"
#include "ejbwav_v2h.intfb.h"
#include "ejbwav_vcori.intfb.h"
#include "ejbwav_hcori.intfb.h"
#include "ejbwav_wav2gp.intfb.h"
#include "etransdir_jb.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('JGCORI',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

IF (LJBWAVELET .AND. NGEOCOR == 1) THEN
  IF (.NOT. LELAM) THEN
    YDSPEC=0.0_JPRB

    DO JSCALE=1,N_WAVELET_SCALES+1
      CALL JBVCOR_WAVELETIN (JSCALE,YDCV,YDSPEC%SP3D,YDSPEC%SP2D)
    ENDDO
  ELSE ! LAM wavelets
! inject control vector into LAM-wavelet array
    CALL EJBWAV_CV2WAV(YDCV%initcv,SIZE(YDCV%initcv))
! apply vertical B-blocks
    CALL EJBWAV_VCORI
! go to horizontal stratification
    CALL EJBWAV_V2H
! apply horizontal STD
    CALL EJBWAV_HCORI
! transform to spectral (via G.P.)
    ALLOCATE(ZGP3(NPROMA,NFLEVG*NVA3D,NGPBLKS))
    ALLOCATE(ZGP2(NPROMA,NVA2D,NGPBLKS))
    CALL EJBWAV_WAV2GP(ZGP2,ZGP3)
! You're in GP space at this point. So continue with spectral transform.
! Bi-periodicisation has been done in EJBWAV_WAV2GP
    CALL ETRANSDIR_JB (0,YDSPEC%SP3D,YDSPEC%SP2D,YDSPEC%SP1D,ZGP3,ZGP2)
    YDSPEC%SP1D=0.0_JPRB
    DEALLOCATE(ZGP2,ZGP3)
  ENDIF

!*    1.    HOMOGENEOUS ISOTROPIC CORRELATIONS ON WORK SPHERE
!           -------------------------------------------------

ELSEIF(NGEOCOR == 1)THEN

  YDSPEC=YDCV%initcv

  IF (LELAM .AND. NCONF/100 == 1) YDSPEC%SP1D(:,:) = 0.0_JPRB

  IF (NPRTRV > 1) THEN

!         1.1  Transpose spectral data to vertical columns

    LLSELECT3D(:)=.TRUE.
    LLSELECT2D(:)=.TRUE.
    IF (NPSP==1) ZSP=YDSPEC%SP2D
    CALL TRMTOS (PSPSEL3D=YDSPEC%SP3D,PSPSEL2D=ZSP, &
               & PSPSEL3DG=ZSP3DG,PSPSEL2DG=ZSP2DG, &
               & LDSELECT3D=LLSELECT3D,LDSELECT2D=LLSELECT2D)

!         1.2  Multiply by vertical correlations

    ! No more specific call to ejgvcori for ALADIN
    CALL JGVCOR(ZSP3DG,ZSP2DG,1,NSPEC2V,.FALSE.) 

!         1.3  Transpose spectral data back to level structure

    CALL TRSTOM (PSPSEL3DG=ZSP3DG,PSPSEL2DG=ZSP2DG, &
               & PSPSEL3D=YDSPEC%SP3D,PSPSEL2D=ZSP, &
               & LDSELECT3D=LLSELECT3D,LDSELECT2D=LLSELECT2D)
    IF (NPSP==1) YDSPEC%SP2D=ZSP

  ELSE
      CALL JGVCOR(YDSPEC%SP3D,YDSPEC%SP2D,KSPSTA,KSPEND,.FALSE.)
  ENDIF

!         1.4  Multiply by horizontal correlations

  IF (LELAM) THEN
    CALL EJGHCORI(LDTOUQ,YDSPEC%SP3D,YDSPEC%SP2D,KSPSTA,KSPEND,NVALUE)
  ELSE
    CALL JGHCORI(LDTOUQ,YDSPEC%SP3D,YDSPEC%SP2D,KSPSTA,KSPEND,NVALUE)
  ENDIF
!*    2.    HOMOGENEOUS ISOTROPIC CORRELS ON DESTRETCHED SPHERE
!           ---------------------------------------------------

ELSEIF(NGEOCOR == 2)THEN

  YDSPEC=YDCV%initcv

!       2.1 Work array dimensioning and overdimensioning

  IDIM=3*NFLEVL
  IF(LDTOUQ .AND. ANY(SPJB_VARS_INFO%IGRIBCODE==NGRBQ)) IDIM=IDIM+NFLEVL
  IF(ANY(SPJB_VARS_INFO%IGRIBCODE==NGRBO3)) IDIM=IDIM+NFLEVL
  ISURDIM=1-MOD(IDIM,2)
  IDIM=IDIM+ISURDIM

!       2.2  Multitasking loop on M

  DO JMLOC=1,NUMP
    IM=MYMS(JMLOC)
    ISP2STA=NASM0(IM)
    ISP2END=NASM0(IM)+2*(NSMAX+1-IM)-1

!         2.3  Vertical correls on work sphere (not for nonsep case)

    CALL JGVCOR(YDSPEC%SP3D,YDSPEC%SP2D,ISP2STA,ISP2END,.FALSE.)

!         2.4  Horizontal correls on destretched sphere

    CALL JGHCOSI(LDTOUQ,IDIM,IM,ISP2STA,ISP2END,YDSPEC)

  ENDDO

ELSE
  CALL ABOR1('UNAUTHORIZED NGEOCOR IN JGCOR')
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('JGCORI',1,ZHOOK_HANDLE)
END SUBROUTINE JGCORI
