SUBROUTINE PREDICT_RUNTIME (KITER,KMAXIT,PREDUC,PREQD)

!-----------------------------------------------------------------------
!   PREDICT_RUNTIME - Predict remaining runtime (wallclock) for the minimization

!   Purpose. Predict remaining runtime (wallclock) for the minimization.
!   -------- 

!   Interface.
!   ----------
!      CALL PREDICT_RUNTIME (PTIMER,KITER,KMAXIT)

!      Explicit arguments:

!      Inputs: KITER        -- Iteration counter
!              KMAXIT       -- Maximum allowed iterations
!              PREDUC       -- Gradient reduction achieved so far
!              PREQD        -- Required gradient reduction

!   Externals.   USER_CLOCK
!   ----------   

!   Reference.
!   ----------
!       None yet!

!   Method.
!   -------
!              Linear regression of the NMEM_GUESS_RUNTIME most recent values
!              of LOG(PREDUC) is extrapolated to estimate at which iteration
!              PREDUC will equal PREQD. The remaining wallclock time is estimated
!              from the estimated number of iterations and the mean wallclock
!              time per iteration.

!   Author.
!   -------
!       Mike Fisher  *ECMWF*

!   Modifications.
!   --------------
!       Original    2004/02/16
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMLUN   , ONLY : NULERR
USE YOMVAR   , ONLY : L_GUESS_RUNTIME, NITER_GUESS_RUNTIME, NFREQ_GUESS_RUNTIME, &
                   &  NMEM_GUESS_RUNTIME
USE YOMCT0,  ONLY: NPROC
USE YOMMP ,  ONLY: MYPROC
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN) :: KITER,KMAXIT
REAL(KIND=JPRB)   , INTENT(IN) :: PREDUC,PREQD

INTEGER(KIND=JPIM),SAVE       :: INMEM, IOLDEST
INTEGER(KIND=JPIM) :: I_GUESS_NITERS
INTEGER :: ICLOCK(8) ! NB: DATE_AND_TIME expects a default INTEGER, not necessarily a JPIM.

REAL(KIND=JPRB), SAVE, ALLOCATABLE, DIMENSION(:) :: ZX, ZY, ZXX, ZXY
REAL(KIND=JPRB), SAVE :: Z_START_TIME
REAL(KIND=JPRB) :: Z_DELTA_TIME, Z_TIME_PER_ITER, Z_GUESS_TIME_LEFT, Z_MAX_TIME_LEFT, &
               &   Z_GUESS_RUNTIME, Z_MAX_RUNTIME, ZSLOPE, ZINTERCEPT, &
               &   ZAVGX, ZAVGY, ZAVGXX, ZAVGXY

REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "user_clock.h"
END INTERFACE

IF (LHOOK) CALL DR_HOOK('PREDICT_RUNTIME',0,ZHOOK_HANDLE)

IF (L_GUESS_RUNTIME .AND. (NPROC==1 .OR. MYPROC==1)) THEN

  IF (KITER==0) THEN

!--- Initialize the timer, etc.

    CALL USER_CLOCK(PELAPSED_TIME=Z_START_TIME)

    ALLOCATE (ZX (NMEM_GUESS_RUNTIME)) ! NB: these arrays are never deallocated!
    ALLOCATE (ZY (NMEM_GUESS_RUNTIME))
    ALLOCATE (ZXX(NMEM_GUESS_RUNTIME))
    ALLOCATE (ZXY(NMEM_GUESS_RUNTIME))

    ZX (1) = 0.0_JPRB
    ZY (1) = 0.0_JPRB
    ZXX(1) = 0.0_JPRB
    ZXY(1) = 0.0_JPRB

    INMEM  = 1
    IOLDEST =2

  ELSEIF (KITER>=NITER_GUESS_RUNTIME .AND. &
        & MOD(KITER-NITER_GUESS_RUNTIME,NFREQ_GUESS_RUNTIME)==0) THEN

!---- Update the averages in the regresion formula

    ZX (IOLDEST) = REAL(KITER,JPRB)

    IF (PREDUC<0.0_JPRB .OR. PREDUC>1.0_JPRB) THEN
      ZY(IOLDEST) = 0.0_JPRB
    ELSE
      ZY(IOLDEST) = LOG(PREDUC)
    ENDIF

    ZXX(IOLDEST) = ZX(IOLDEST)*ZX(IOLDEST)
    ZXY(IOLDEST) = REAL(KITER,JPRB)*ZY(IOLDEST)

    INMEM=MIN(INMEM+1,NMEM_GUESS_RUNTIME)
    IOLDEST=1+MOD(IOLDEST,NMEM_GUESS_RUNTIME)

    ZAVGX  = SUM(ZX (1:INMEM))/REAL(INMEM,JPRB)
    ZAVGY  = SUM(ZY (1:INMEM))/REAL(INMEM,JPRB)
    ZAVGXX = SUM(ZXX(1:INMEM))/REAL(INMEM,JPRB)
    ZAVGXY = SUM(ZXY(1:INMEM))/REAL(INMEM,JPRB)

!---- Fit a straight line to LOG(PREDUC) as a function of KITER.

    ZSLOPE     = (ZAVGXY - ZAVGX*ZAVGY)/(ZAVGXX - ZAVGX*ZAVGX)
    ZINTERCEPT = ZAVGY - ZSLOPE*ZAVGX

!---- Estimate how many iterations will be required to reduce the gradient norm by PREQD.

    IF (ZSLOPE>=0.0) THEN
      I_GUESS_NITERS    = KMAXIT
    ELSE
      I_GUESS_NITERS    = NINT( (LOG(PREQD)-ZINTERCEPT)/ZSLOPE )
    ENDIF

    IF (I_GUESS_NITERS<1)      I_GUESS_NITERS = KMAXIT  ! Gradient norm is increasing.
    IF (I_GUESS_NITERS<=KITER) I_GUESS_NITERS = KITER+1 ! Predict at least one more iteration.
    IF (I_GUESS_NITERS>KMAXIT) I_GUESS_NITERS = KMAXIT  ! Predict iteration limit will be hit.

!---- Calculate the mean time per iteration.

    Z_DELTA_TIME = Z_START_TIME
    CALL USER_CLOCK(PELAPSED_TIME_SINCE=Z_DELTA_TIME)

    Z_TIME_PER_ITER   = Z_DELTA_TIME / REAL(KITER,JPRB)

!---- Estimate the time to completion, etc.

    Z_GUESS_TIME_LEFT = Z_TIME_PER_ITER * REAL(I_GUESS_NITERS-KITER,JPRB)
    Z_MAX_TIME_LEFT   = Z_TIME_PER_ITER * REAL(KMAXIT-KITER,JPRB)
    Z_GUESS_RUNTIME   = Z_TIME_PER_ITER * REAL(I_GUESS_NITERS,JPRB)
    Z_MAX_RUNTIME     = Z_TIME_PER_ITER * REAL(KMAXIT,JPRB)

    CALL DATE_AND_TIME (VALUES=ICLOCK)

    WRITE (NULERR,'(/)')
    WRITE (NULERR,'(''Time now is: '',I2.2,'':'',I2.2,'':'',I2.2)') ICLOCK(5:7)
    WRITE (NULERR,*) 'Completed iteration ',KITER
    WRITE (NULERR,*) 'Mean time per iteration=',Z_TIME_PER_ITER
    WRITE (NULERR,*) 'Estimated total number of iterations=',I_GUESS_NITERS
    WRITE (NULERR,*) 'Estimated time to completion=',Z_GUESS_TIME_LEFT,' seconds'
    WRITE (NULERR,*) 'Estimated total runtime=',Z_GUESS_RUNTIME,' seconds'
    WRITE (NULERR,*) 'Maximum possible number of iterations (iteration limit)=',KMAXIT
    WRITE (NULERR,*) 'Estimated time until iteration limit is hit=',Z_MAX_TIME_LEFT,' seconds'
    WRITE (NULERR,*) 'Estimated maximum possible runtime=',Z_MAX_RUNTIME,' seconds'
    WRITE (NULERR,'(/)')
  ENDIF

ENDIF

IF (LHOOK) CALL DR_HOOK('PREDICT_RUNTIME',1,ZHOOK_HANDLE)

END SUBROUTINE PREDICT_RUNTIME
