SUBROUTINE rdvarbc(cdfname,ydvarbc,kngroup,knparam,kmxngroup,kmpredxhst)
!-----------------------------------------------------------------------
!   RDVARBC - Read file with variational bias parameter information

!   Purpose.
!   --------
!   Cycling of variational bias parameter information 

!   Interface.
!   ----------
!   CALL rdvarbc(cdfname,ydvarbc,kngroup,knparam,kmxngroup)

!   Input arguments:
!      cdfname   - file name
!      kmxngroup - maximum number of parameter groups

!   Output arguments:
!      ydvarbc   - variational bias parameter data table
!      kngroup   - number of parameter groups
!      knparam   - number of bias parameters
!      kmpredxhst- histograms

!   Implicit input:
!      MPREDXCNT  - predictor sample counts
!      APREDMEAN  - predictor means
!      APREDXCOV  - predictor covariances

!   Author.
!   -------
!   Dick Dee (ECMWF)

!   Modifications.
!   --------------
!   Original    2004/03/19
!        P.Burton :  29-09-04 LVARBC cleanup
!        D. Dee      22-10-04   Read expid, date, time from formatted file
!        V.Guidard:  10-03-08 hists are output arg. rather than implicit outuput

!-----------------------------------------------------------------------

USE parkind1, ONLY : JPIM, JPRB
USE yomct0  , ONLY : nproc
USE yomhook , ONLY : lhook, dr_hook
USE yomvarbc, ONLY : TYPE_VARBC, MPREDXCNT, APREDMEAN, APREDXCOV, NBINS
USE yomlun  , ONLY : nulout, nultmp
USE mpl_module  , ONLY : MPL_BARRIER

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: kmxngroup 
CHARACTER(*)      ,INTENT(IN)    :: cdfname 
TYPE(TYPE_VARBC)  ,INTENT(INOUT) :: ydvarbc(kmxngroup) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: kngroup 
INTEGER(KIND=JPIM),INTENT(OUT)   :: knparam 
INTEGER(KIND=JPIM),INTENT(OUT)   :: kmpredxhst(kmxngroup,NBINS)

INTEGER(KIND=JPIM) :: ios, jg, inpred, idate, itime,iversion
INTEGER(KIND=JPIM) :: IPRED
CHARACTER(LEN=4)   :: clexpid
CHARACTER(LEN=80)  :: clheader
LOGICAL            :: llexist
REAL(KIND=JPRB)    :: zhook_handle

#include "abor1.intfb.h"

IF (lhook) CALL dr_hook('RDVARBC',0,zhook_handle)

WRITE(nulout,'(/,a)') &
 & 'RDVARBC: Reading bias information from file '//trim(cdfname)  

INQUIRE(file=trim(cdfname),exist=llexist)

IF (.NOT. llexist) THEN

  WRITE(nulout,'(a)') 'RDVARBC: No such file '//trim(cdfname)
  kngroup = 0
  knparam = 0

ELSE ! Read on all processors

  OPEN(nultmp,file=trim(cdfname),form='formatted',status='old',iostat=ios)
  IF (ios/=0) CALL ABOR1('RDVARBC: Cannot open file '//trim(cdfname))

  READ(nultmp,'(A)') clheader
  IF(clheader(1:11) == 'VARBC_cycle') THEN
    READ(clheader,'(19x,i3)') iversion
    READ(nultmp,*) clexpid, idate, itime
  ELSE
    iversion = 0
    READ(clheader,*) clexpid, idate, itime
  ENDIF
  WRITE(nulout,'(/,a,a4,a,i8,a,i6,a,i3)') & 
 & 'RDVARBC: expid = ', clexpid(1:4), &
 &          ' date = ', idate, &
 &          ' time = ', itime,&
 &          ' file version ',iversion
 
  READ(nultmp,*) kngroup, knparam
  IF (kngroup>kmxngroup) CALL ABOR1('RDVARBC: Insufficient allocation')
  DO jg = 1, kngroup
    IF(iversion == 0) then
      READ(nultmp,*) YDVARBC(jg)%NBUFRID, &
       &   YDVARBC(jg)%NSENSOR, &
       &   YDVARBC(jg)%NCHANNL, &
       &   YDVARBC(jg)%NPARAM, &
       &   YDVARBC(jg)%NCOUNT  
      YDVARBC(jg)%NCODETYPE = 210
    ELSE
      READ(nultmp,*) YDVARBC(jg)%NBUFRID, &
       &   YDVARBC(jg)%NSENSOR, &
       &   YDVARBC(jg)%NCODETYPE, &
       &   YDVARBC(jg)%NCHANNL, &
       &   YDVARBC(jg)%NPARAM, &
       &   YDVARBC(jg)%NCOUNT  
    ENDIF
    inpred = YDVARBC(jg)%NPARAM
    ALLOCATE(YDVARBC(jg)%NPREDCS(inpred))
    ALLOCATE(YDVARBC(jg)%APARAMS(inpred))
    ALLOCATE(YDVARBC(jg)%APARAM0(inpred))
    ALLOCATE(YDVARBC(jg)%APARAM5(inpred))
    READ(nultmp,*) YDVARBC(jg)%NPREDCS, &
               &   YDVARBC(jg)%APARAMS, &
               &   YDVARBC(jg)%APARAM0, &
               &   YDVARBC(jg)%APARAM5
  ENDDO

! Predictor statistics

IF (iversion==0) THEN
  IPRED = 10
ELSEIF (iversion==1) THEN
  IPRED = 11
ELSEIF (iversion==2) THEN  
  READ(nultmp,*) IPRED
ENDIF
READ(nultmp,*) MPREDXCNT(0:IPRED,0:IPRED), APREDMEAN(0:IPRED), APREDXCOV(0:IPRED,0:IPRED)

! FG departures histograms

  DO jg = 1, kngroup
    IF(iversion == 0) then
      READ(nultmp,*) ydvarbc(jg)%nbufrid, &
                  & ydvarbc(jg)%nsensor, &
                  & ydvarbc(jg)%nchannl, &
                  & ydvarbc(jg)%nparam, &
                  & ydvarbc(jg)%ncount  
      YDVARBC(jg)%NCODETYPE = 210
    ELSE
      READ(nultmp,*) ydvarbc(jg)%nbufrid, &
                  & ydvarbc(jg)%nsensor, &
                  & ydvarbc(jg)%ncodetype, &
                  & ydvarbc(jg)%nchannl, &
                  & ydvarbc(jg)%nparam, &
                  & ydvarbc(jg)%ncount  
    ENDIF
    READ(nultmp,*) kmpredxhst(jg,:)
  ENDDO

  CLOSE(nultmp)

  IF (nproc>1) CALL MPL_BARRIER(CDSTRING='RDVARBC')

ENDIF

WRITE(nulout,'(a,i5,a,i5,a)') &
 & 'RDVARBC: Read ', kngroup, ' groups with ', knparam, ' parameters'  
WRITE(nulout,'(a)') &
 & 'RDVARBC: Read predictor statistics'  

IF (lhook) CALL dr_hook('RDVARBC',1,zhook_handle)

END SUBROUTINE rdvarbc

