SUBROUTINE RTSETUP

!**** *RTSETUP* - SET UPS FOR TOVS RAD. TRANSFER CODE

!     PURPOSE.
!     -------
!     SET UP CONSTANTS FOR TOVS RADIATIVE TRANSFER CODE
!     1) READS IN REGRESSION COEFFICIENTS FOR EXTRAPOLATION TO 000.1 mb
!     2) RETURNS F00.M00. PRESSURE LEVEL INFORMATION
!     3) SETS UP DUMMY CLOUD AND EMISSIVITY PARAMETERS

!**   INTERFACE.
!**   ---------
!**   *CALL* * RTSETUP*  (FROM SURAD)

!**   EXTERNALS
!     ---------
!        RTTVI   - READ TOVS COEFFICIENTS (RT-CODE)

!     AUTHOR: TONY MCNALLY
!     DATE  : 91-11-30

!     MODIFICATIONS
!     -------------
!     Erik Andersson : 920304 - DOCTORize and multi-task.
!     Erik Andersson : 920818 - New version of RT-code
!     Erik Andersson : 921216 - Remove regression extrapolation
!     Erik Andersson : 930226 - Put regression extrapolation back as an option
!     Roger Saunders : 961021 - Modified call to RTTVI                         
!     Erik Andersson : 970220 - F90, simplify.
!     Erik Andersson : 971125 - Introduce OZONE climatology.
!     Roger Saunders : 981019 - Changed RTTVi interface
!     F. Chevallier  : 000425 - Allow HIRS channels 16-19 computation
!     G Kelly        : 031123 - Allow for rttov 8                               
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     N Bormann      : 060217 - METOP (except IASI)
!     A Collard      : 061109 - Add IASI.  Allow setup of a subset of channels
!     C O'Dell       : 150107 - Multiple Sensors for 1D-Var
!     A Geer         : 140408 - Multiple sensors for 4D-Var
!     N Bormann      : 070808 - RTTOV9
!     V Guidard      : 070223 - Add IASI (after A Collard)
!     N Bormann      : 081002 - Attempt to remove awful hardcoding of 
!                               max channel numbers
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMVAR   , ONLY : LCO2
USE YOMCT0   , ONLY : NPRINTLEV,LALLOPR  ,NPROC    ,LSCREEN
USE YOMDIMO  , ONLY : NLSAT    ,NMXTCH   ,NPNAV   ,&
 & NPTNAV   ,NPNSAV  ,NPNSSV   ,NPNCV    ,NMXTCHU      
USE YOMLUN   , ONLY : NULERR   ,NULOUT   ,NULRAD
USE YOMTVRAD , ONLY : NJPPF    ,NJPNSAT , O3CLRT   ,NVALCHAN, RADPRE  ,&
 & SATGRP_TABLE  , CO2BGE_S, CO2CLRT_S, CO2BGE_T, CO2CLRT_T, &
 & MSENSOR_AIRS, MSENSOR_IASI, M__NumAIRSChans, M__NumIASIChans, &
 & M__MaxIASIChans, LRTTOV_INTERPOL
USE YOEPHY   , ONLY : LERAIN , LEMWAVE
USE YOMMP    , ONLY : MYPROC
USE YOM_YGFL, ONLY : JPGHG, NGHG

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM), PARAMETER :: JPL100=43
REAL(KIND=JPRB), DIMENSION(JPL100) :: ZLEVS
INTEGER(KIND=JPIM) :: I,IPPF,IPNSAT,IPLEV,IPCH,IPCHUS,IPNAV,IPNSSV,IPNCV,IPNSAV
INTEGER(KIND=JPIM), ALLOCATABLE :: INUMCHS(:)
INTEGER(KIND=JPIM) :: IU,IIU1,ISATGRP_DIM,IERROR
INTEGER(KIND=JPIM) :: IERR(SIZE(SATGRP_TABLE(:)))
LOGICAL, ALLOCATABLE :: LLCLOUD(:),LLAEROSOL(:)
LOGICAL :: LLP
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "mwave_setup.intfb.h"
#include "onedvar_setup.intfb.h"
! #include "sunne.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('RTSETUP',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*     1.  LOAD RT COEFFICIENTS

IERR = 0
IIU1=NULRAD

ISATGRP_DIM=SIZE(SATGRP_TABLE(:))
ALLOCATE(LLCLOUD(ISATGRP_DIM))
LLCLOUD=.False.
ALLOCATE(LLAEROSOL(ISATGRP_DIM))
LLAEROSOL=.False.


NPNAV=4
IPNAV=NPNAV
IF (NGHG > 0) IPNAV=4+JPGHG
NPTNAV=IPNAV

! Restrict initialised channels for kilochannel sounders:
ALLOCATE(INUMCHS(ISATGRP_DIM))
INUMCHS(:)=0
DO I=1,ISATGRP_DIM
  SELECT CASE (SATGRP_TABLE(I)%sensor)
   CASE (MSENSOR_AIRS)
     INUMCHS(I) = M__NumAIRSChans
   CASE (MSENSOR_IASI)
     INUMCHS(I) = M__NumIASIChans
  END SELECT
ENDDO

CALL RTTVI(NPROC,MYPROC,IERR,IPPF,IPNSAT,IPLEV,IPCH,IPCHUS, &
 & IPNAV,IPNSAV,IPNSSV,IPNCV, &
 & ISATGRP_DIM,LRTTOV_INTERPOL,JPL100,NMXTCH,  & 
 & SATGRP_TABLE(:)%rtcoef_pos,SATGRP_TABLE(:)%rtseries,                     &
 & SATGRP_TABLE(:)%rtid,SATGRP_TABLE(:)%sensor, INUMCHS,                    &
 & NVALCHAN,                                                                &
 & ZLEVS,IIU1,LLAEROSOL,&
 & (SATGRP_TABLE(:)%cld_rtcalc_screen .AND. LSCREEN) &
 &  .OR.SATGRP_TABLE(:)%cld_rtcalc_assim)  

IPCH=MAX(M__MaxIASIChans,MAXVAL(INUMCHS(:)))
NMXTCHU=IPCH 

IF (ANY(IERR /= 0)) THEN
  DO I=1,ISATGRP_DIM
    IF(IERR(I) /= 0)THEN
      WRITE(NULERR,*) ' RTTVI error for satid sensor:',&
       & SATGRP_TABLE(I)%bufrid,SATGRP_TABLE(I)%sensor 
      WRITE(NULERR,*) ' RTTVI IERR=',IERR(I)
    ENDIF
  ENDDO
  CALL ABOR1('RTSETUP: RTTVI PROBLEM')
ENDIF
IF(IPLEV > JPL100) THEN
  WRITE(NULERR,*)' RTTVI IPLEV > JPL100: ',IPLEV,JPL100
  CALL ABOR1('RTSETUP: RTTVI IPLEV PROBLEM')
ENDIF

IF (LERAIN) THEN
   CALL ONEDVAR_SETUP (NPROC,MYPROC,IERROR,IIU1, &
               & ISATGRP_DIM, &
               & SATGRP_TABLE (:) % RTSERIES   , &
               & SATGRP_TABLE (:) % BUFRID     , &
               & SATGRP_TABLE (:) % RTID       , &
               & SATGRP_TABLE (:) % SENSOR     , &
               &                        INUMCHS, &
               & SATGRP_TABLE (:) % RTCOEF_POS)
   IERROR = 0
   IF (IERROR  /= 0) CALL ABOR1 ('RTSETUP: ONEDVAR_SETUP PROBLEM')
ENDIF

IF (LEMWAVE) THEN
   CALL MWAVE_SETUP (NPROC,MYPROC,IERROR,IIU1,ISATGRP_DIM,INUMCHS)
   IERROR = 0
   IF (IERROR  /= 0) CALL ABOR1 ('RTSETUP: MWAVE_SETUP PROBLEM')
   WRITE (NULOUT,*) 'RTSETUP: Done MWAVE_SETUP'
ENDIF

DEALLOCATE(INUMCHS)

!*     1.  Set number of levels and channels for ALLOCATE

NLSAT=IPLEV

IF(ALLOCATED(RADPRE) .AND. LHOOK) CALL DR_HOOK('RTSETUP',1,ZHOOK_HANDLE)
IF(ALLOCATED(RADPRE)) RETURN
LLP = NPRINTLEV >= 1.OR. LALLOPR
IU = NULOUT
IF(LLP) WRITE(NULOUT,'('' RTSETUP ALLOCATION PRINTOUTS '')')

ALLOCATE(RADPRE(NLSAT))
IF(LLP)WRITE(IU,9) 'RADPRE   ',SIZE(RADPRE   ),SHAPE(RADPRE   )
ALLOCATE(O3CLRT(NLSAT))
IF(LLP)WRITE(IU,9) 'O3CLRT   ',SIZE(O3CLRT   ),SHAPE(O3CLRT   )

!*     2.  ASSIGN STATEMENTS USING OUTPUT FROM RTTVI.F

RADPRE(1:NLSAT)=ZLEVS(1:NLSAT)

NJPNSAT = IPNSAT
NJPPF = IPPF
NPNSAV = IPNSAV
NPNSSV = IPNSSV
NPNCV = IPNCV
WRITE(NULOUT,*)' RTSETUP ', &
 & IPCH,IPNSAT,IPPF,IPNAV,IPNSAV,IPNSSV,IPNCV   
CALL FLUSH(NULOUT)

!*     3.  OTHER ASSIGN STATEMENTS *NOT* USING OUTPUT FROM RTTVI.F

! -- OZONE climatology for RT-model. (R.Saunders).
O3CLRT=(/&
 & 0.969339E-05_JPRB, 0.100043E-04_JPRB, 0.101194E-04_JPRB, 0.101751E-04_JPRB,&
 & 0.102181E-04_JPRB, 0.102463E-04_JPRB, 0.102127E-04_JPRB, 0.102454E-04_JPRB,&
 & 0.101497E-04_JPRB, 0.935746E-05_JPRB, 0.809846E-05_JPRB, 0.672055E-05_JPRB,&
 & 0.519183E-05_JPRB, 0.372400E-05_JPRB, 0.258175E-05_JPRB, 0.172204E-05_JPRB,&
 & 0.119041E-05_JPRB, 0.845131E-06_JPRB, 0.649756E-06_JPRB, 0.526723E-06_JPRB,&
 & 0.412770E-06_JPRB, 0.303262E-06_JPRB, 0.210869E-06_JPRB, 0.156281E-06_JPRB,&
 & 0.123483E-06_JPRB, 0.107569E-06_JPRB, 0.100696E-06_JPRB, 0.959905E-07_JPRB,&
 & 0.916880E-07_JPRB, 0.891324E-07_JPRB, 0.846609E-07_JPRB, 0.811542E-07_JPRB,&
 & 0.778113E-07_JPRB, 0.757306E-07_JPRB, 0.711752E-07_JPRB, 0.663465E-07_JPRB,&
 & 0.616040E-07_JPRB, 0.567638E-07_JPRB, 0.521222E-07_JPRB, 0.479273E-07_JPRB,&
 & 0.443969E-07_JPRB, 0.419678E-07_JPRB, 0.409984E-07_JPRB /)  

! -- Setup for Neural Network Bias Correction (AIRS)
! CALL SUNNE

! -- CO2 climatology for RT-model. (R.Engelen).
IF(LCO2) THEN
  CO2BGE_S  = 30.0_JPRB             
  CO2BGE_T  = 30.0_JPRB             
  CO2CLRT_S=375.0_JPRB
  CO2CLRT_T=378.0_JPRB
ENDIF

9 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('RTSETUP',1,ZHOOK_HANDLE)
END SUBROUTINE RTSETUP
