SUBROUTINE SUECGES

USE PARKIND1  ,ONLY : JPRB, JPIM
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLSUR, NSPEC2, NFTHER, NFLEVL, NFD2D, NGPBLKS, &
                    & NPROMA, NFLEVG, NS3D, NS2D
USE YOMMP    , ONLY : NPSP
USE YOMLUN   , ONLY : NULOUT, NINMSH, NULTRAJHR
USE YOMJG    , ONLY : L_JBVCOORD, N_SPJB_VARS, SPJB_VARS_INFO
USE YOMCT3   , ONLY : NSTEP
USE YOMSPJB  , ONLY : SP7A3, SP7A2, SP7A1, GP7A3
USE YOMSP    , ONLY : SPOR
USE YOMGEM   , ONLY : NGPTOT
USE YOMFCEB  , ONLY : BLTOP
USE YOMCT0   , ONLY : NCONF, LELAM, LSPRT, NPRINTLEV, LARPEGEF_TRAJHR, &
                    & LARPEGEF_RDGP_INIT
USE TRAJECTORY_MOD, ONLY: READ_TRAJECTORY, GET_TRAJ_SPEC, GET_TRAJ_GRID, &
                    & LREADGPTRAJ, LPRTTRAJ, LTRAJHR, LTRAJHR_ALTI
USE YOMGMV , ONLY : YT5
USE YOM_YGFL , ONLY : YGFL,  YGFLC
USE YOMGFL   , ONLY : GFL
USE YOMGRB   , ONLY : NGRBQ,  NGRBCLWC,  NGRBCIWC
USE YOMOP    , ONLY : CFNTRAJHR,CFNGSH

!**** *SUECGES*  - Initializes the processing for the first guess constaint.
!****            - ECMWF version.

!     Purpose.
!     --------
!           Initializes the processing for the first guess constraint.
!           - ECMWF version.

!**   Interface.
!     ----------
!        *CALL* *SUECGES

!        Explicit arguments :
!        --------------------
!        None

!        Implicit arguments :
!        --------------------
!        None

!     Method.
!     -------
!        The first guess field is read in. The forecast error variances are
!        read in. The forecast errors are interpolated in the horizontal to
!        the model grid. Surface pressure is found by transforming the first
!        guess to grid points. This information is then used to interpolate
!        the forecast errors in the vertical to the model levels. Errors
!        for specific humidity are calculated assuming a simple error in
!        relative humidity.

!     Fields, as input, are assumed to occur in the following order:
!     1 u field errors,
!     2 v field errors,
!     3 h field errors.
!     The same ordering is maintained for the interpolated fields.
!     Additional derived fields are added:
!     4 Ps field errors,
!     5 q field errors.

!     Externals.
!     ----------   SUSPEC   - Read in the first guess
!                  SPEREE - Direct spectral to grid-point transform

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Bill Heckley                       *ECMWF*

!     Modifications.
!     --------------
!        Original : 90-03-09
!        Modified : 91-09-25 Bill Heckley ECMWF - Preparation for IO scheme
!                                                 Filtering of sigma fields
!        Modified : 91-10-15 Bill Heckley and Mats Hamrud ECMWF
!                            Use of MIOLIB for IO on forecast error files
!        Modified : 92-10-05 Bill Heckley - more 'realistic' specification
!                                           of relative humidity errors.
!        Modified : 93-10-27 Roberto Buizza - for conf 601
!        M. Hamrud  :  95-10-01 - Interface block for SUSPEC
!        Modified : 95-11-30 Per Unden - Get surface fields for hum error
!        M. Hamrud  :  95-10-01 - Interface block for SUSPEC
!        Modified : 95-12-22 M.Hamrud    Change of ZFIELD layout
!        M. Hamrud  :  95-10-30 - Removed CLCONF
!        M. Hamrud  :  95-12-15 - Removed call to SUFLFCE
!        M. Fisher  :  96-04-17 - Moved lots of code into SUINFCE
!        J.N.Thepaut :  96-11-12 - Read SP7OR and fill SPOR afterwards.
!        E. Holm    :  97-11-28 - Call to SUSPEC changed (incl. ozone) 
!        L. Dragulanescu :  97 MAY - ALADIN 3DVAR
!        R. El Khatib : 98-04-02 - Cleanup ALADIN 3DVAR
!        Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of argument .FALSE. in 'CALL SPEREE'.
!        R. El Khatib 99-02-05 ESPNORMSM => SPNORM
!        M. Siroka  :  99-06-16 - debug ALADIN 3D-VAR 
!        M. Fisher  :  00-10-26 - Removed setup of variances to SU0YOMB
!        M. Fisher  :  00-10-26 - setup for Jb vertical coordinate
!        P.Smolikova:  02-09-30 - Interface to SUSPEC for d4 in NH
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        K.Yessad      June 2006   Possibility to read purely gp ARPEGE files
!        K.Yessad      09-Aug-2006 Bugfixes for LTRAJHR
!        R. El Khatib  26-Nov-2008  Cleaning
!     ------------------------------------------------------------------

IMPLICIT NONE

REAL(KIND=JPRB), ALLOCATABLE :: ZGMVBG(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMVBGS(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGFLBG(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPBGA3(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPBGA2(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPBGA1(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZSPA3(:,:,:),ZSPA2(:,:),ZSP7Q(:,:),ZSP7L(:,:),ZSP7I(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGP7Q(:,:,:),ZGP7I(:,:,:),ZGP7L(:,:,:)
INTEGER(KIND=JPIM) :: ISTEP, IPT, IPTBG, JFIELD, IOFF
LOGICAL :: LLINOR, LLSPOR, LLASTRAJ
CHARACTER(LEN=16)  :: CL_FNTRAJHR
INTEGER(KIND=JPIM) :: IULTRAJHR
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "inv_trans.h"
END INTERFACE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "stepo.intfb.h"
#include "sugridf.intfb.h"
#include "sugridu.intfb.h"
#include "suspec.intfb.h"
#include "specrt.intfb.h"
#include "especrt.intfb.h"
!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUECGES',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

NSTEP=-999

!*       1.    Initialize the first guess.
!              --------------------------------

!*    1.1   Read in the first guess


!--- SUSPEC should not be necessary for NCONF==131 (for ECMWF purposes),
!    but is. Mike F.
!--- The orography is still read in by SUSPEC. Lars I.

LLINOR=.TRUE.
ALLOCATE(ZSPA3(NFLSUR,NSPEC2,NS3D))
ALLOCATE(ZSPA2(NSPEC2,NS2D))
CALL SUSPEC(ZSPA3,ZSPA2,SP7A1,KFILE=2,LDRDGRIDSP=LARPEGEF_RDGP_INIT,LDINOR=LLINOR)
IF (NPRINTLEV >=2) THEN
  WRITE(NULOUT,*) ' SUECGES: SUSPEC(KFILE=2) called '
  CALL FLUSH(NULOUT)
ENDIF

! ky: this piece of code acts on upper air fields so I assume that it must
!     be executed for LTRAJHR_ALTI=.F., even if LTRAJHR_SURF=.T.
IF (.NOT. LTRAJHR_ALTI ) THEN
  IF(YGFL%NUMGPFLDS>0) THEN
    CALL SUGRIDU(1)
  ENDIF
  DO JFIELD=1,N_SPJB_VARS
    IPTBG = SPJB_VARS_INFO(JFIELD)%IPTBG
    IPT   = SPJB_VARS_INFO(JFIELD)%IPT
    IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
      IF (NPSP>0) THEN
        SP7A2(:,IPTBG) = ZSPA2(:,IPT)
      ENDIF
    ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA3) THEN
      SP7A3(:,:,IPTBG) = ZSPA3(:,:,IPT)
    ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_SPGFL) THEN
      IOFF=2+NFTHER
      SP7A3(:,:,IPTBG) = ZSPA3(:,:,IPT+IOFF)
    ENDIF
  ENDDO
  DO JFIELD=1,N_SPJB_VARS
    IPTBG = SPJB_VARS_INFO(JFIELD)%IPTBG
    IPT   = SPJB_VARS_INFO(JFIELD)%IPT
    IF (SPJB_VARS_INFO(JFIELD)%L_IN_GPGFL) THEN
      GP7A3(:,:,IPTBG,:) = GFL(:,:,IPT,:)
    ENDIF
  ENDDO
  IF (LSPRT) THEN
    ALLOCATE(ZSP7Q(1,1))
    ALLOCATE(ZSP7I(1,1))
    ALLOCATE(ZSP7L(1,1))
    ALLOCATE(ZGP7Q(1,1,1))
    ALLOCATE(ZGP7I(1,1,1))
    ALLOCATE(ZGP7L(1,1,1))
    DO JFIELD=1,YGFL%NUMFLDS
      IF (YGFLC(JFIELD)%LSP) THEN
        IPT=2+NFTHER+YGFLC(JFIELD)%MPSP
        IF (YGFLC(JFIELD)%IGRBCODE == NGRBQ) THEN
          DEALLOCATE(ZSP7Q)
          ALLOCATE(ZSP7Q(NFLSUR,NSPEC2))
          ZSP7Q(:,:)=ZSPA3(:,:,IPT)
        ELSEIF (YGFLC(JFIELD)%IGRBCODE == NGRBCLWC) THEN
          DEALLOCATE(ZSP7L)
          ALLOCATE(ZSP7L(NFLSUR,NSPEC2))
          ZSP7L(:,:)=ZSPA3(:,:,IPT)
        ELSEIF (YGFLC(JFIELD)%IGRBCODE == NGRBCIWC) THEN
          DEALLOCATE(ZSP7I)
          ALLOCATE(ZSP7I(NFLSUR,NSPEC2))
          ZSP7I(:,:)=ZSPA3(:,:,IPT)
        ENDIF
      ENDIF
      IF (YGFLC(JFIELD)%LGP) THEN
        IPT=YGFLC(JFIELD)%MP
        IF (YGFLC(JFIELD)%IGRBCODE == NGRBQ) THEN
          DEALLOCATE(ZGP7Q)
          ALLOCATE(ZGP7Q(NPROMA,NFLEVG,NGPBLKS))
          ZGP7Q(:,:,:)=GFL(:,:,IPT,:)
        ELSEIF (YGFLC(JFIELD)%IGRBCODE == NGRBCLWC) THEN
          DEALLOCATE(ZGP7L)
          ALLOCATE(ZGP7L(NPROMA,NFLEVG,NGPBLKS))
          ZGP7L(:,:,:)=GFL(:,:,IPT,:)
        ELSEIF (YGFLC(JFIELD)%IGRBCODE == NGRBCIWC) THEN
          DEALLOCATE(ZGP7I)
          ALLOCATE(ZGP7I(NPROMA,NFLEVG,NGPBLKS))
          ZGP7I(:,:,:)=GFL(:,:,IPT,:)
        ENDIF
      ENDIF
    ENDDO
    IF (LELAM) THEN
        CALL ESPECRT(KDIR=0,LDBCKGR=.TRUE.,PSP7Q=ZSP7Q, &
         & PSP7I=ZSP7I,PSP7L=ZSP7L,PGP7Q=ZGP7Q,PGP7I=ZGP7I,PGP7L=ZGP7L)
    ELSE
        CALL SPECRT(KDIR=0,LDBCKGR=.TRUE.,PSP7Q=ZSP7Q, &
         & PSP7I=ZSP7I,PSP7L=ZSP7L,PGP7Q=ZGP7Q,PGP7I=ZGP7I,PGP7L=ZGP7L)
    ENDIF
     DEALLOCATE(ZSP7Q)
     DEALLOCATE(ZSP7I)
     DEALLOCATE(ZSP7L)
     DEALLOCATE(ZGP7Q)
     DEALLOCATE(ZGP7I)
     DEALLOCATE(ZGP7L)
  ENDIF
ENDIF
DEALLOCATE (ZSPA3)
DEALLOCATE (ZSPA2)

IF (NCONF==131 .AND. LTRAJHR ) THEN

  ! ky: this piece of code acts on both upper air and surface fields.
  !     => called when LTRAJHR=T
  LPRTTRAJ=.TRUE.
  ISTEP    = 0
  LLASTRAJ = .FALSE.

  ! Read Background
  ! For MF applications, this call must read the same "background" file
  !  as a call to suspec(kfile=2).
  CALL READ_TRAJECTORY(ISTEP,LLASTRAJ,LREADGPTRAJ,LDBACKGR=.TRUE.)

  ! Read trajectory step 0 (needed for linearization in change of variable)
  ! For MF applications, this call must read the "initial" file defined
  ! by (CFNGSH,NINMSH) (cf. call suspec(kfile=1)),
  ! so values of (CFNTRAJHR,NULTRAJHR) are provisionally
  ! modified, then restored after the call of READ_TRAJECTORY.
  IF (LARPEGEF_TRAJHR) THEN
    CL_FNTRAJHR=CFNTRAJHR
    IULTRAJHR=NULTRAJHR
    CFNTRAJHR=CFNGSH
    NULTRAJHR=NINMSH
  ENDIF
  CALL READ_TRAJECTORY(ISTEP,LLASTRAJ,LREADGPTRAJ)
  IF (LARPEGEF_TRAJHR) THEN
    CFNTRAJHR=CL_FNTRAJHR
    NULTRAJHR=IULTRAJHR
  ENDIF

  ! ky: this piece of code seems to act only on upper air fields,
  !     so it has been put under LTRAJHR_ALTI=T.



  IF (LTRAJHR_ALTI) THEN

    IF (UBOUND(SP7A3,3)+UBOUND(SP7A2,2)>0) THEN
      ALLOCATE (ZSPBGA3(NFLSUR,NSPEC2,NS3D))
      ALLOCATE (ZSPBGA2(NSPEC2,NFD2D))
      ALLOCATE (ZSPBGA1(NFLEVL,2))
      CALL GET_TRAJ_SPEC (ZSPBGA3,ZSPBGA2,ZSPBGA1,LDBACKGR=.TRUE.)
    ENDIF

    IF (UBOUND(GP7A3,3)>0) THEN
      ALLOCATE (ZGMVBG(NPROMA,NFLEVG,YT5%NDIM,NGPBLKS))
      ALLOCATE (ZGMVBGS(NPROMA,YT5%NDIMS,NGPBLKS))
      ALLOCATE (ZGFLBG(NPROMA,NFLEVG,YGFL%NDIM5,NGPBLKS))
      ZGMVBG(:,:,:,NGPBLKS)=0.0_JPRB
      ZGMVBGS(:,:,NGPBLKS)=0.0_JPRB
      ZGFLBG(:,:,:,NGPBLKS)=0.0_JPRB
      ISTEP=0
      CALL GET_TRAJ_GRID(ZGMVBG,ZGMVBGS,ZGFLBG,KSTEP=ISTEP,LDBACKGR=.TRUE.)
    ENDIF
    LPRTTRAJ=.FALSE.

    DO JFIELD=1,N_SPJB_VARS
      IPTBG = SPJB_VARS_INFO(JFIELD)%IPTBG
      IPT   = SPJB_VARS_INFO(JFIELD)%IPT

      IF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA2) THEN
        IF (NPSP>0) THEN
          SP7A2(:,IPTBG) = ZSPBGA2(:,IPT)
        ENDIF
      ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_SPA3) THEN
        SP7A3(:,:,IPTBG) = ZSPBGA3(:,:,IPT)
      ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_SPGFL) THEN
        IOFF=2+NFTHER
        SP7A3(:,:,IPTBG) = ZSPBGA3(:,:,IPT+IOFF)
      ELSEIF (SPJB_VARS_INFO(JFIELD)%L_IN_GPGFL) THEN
        GP7A3(:,:,IPTBG,:) = ZGFLBG(:,:,IPT,:)
      ELSE
        WRITE (NULOUT,*) 'DON''T KNOW WHERE TO COPY BG FIELD FROM : ',JFIELD
        CALL ABOR1 ('DON''T KNOW WHERE TO COPY BG FIELD FROM')
      ENDIF
    ENDDO

    ! =========================================================================
    ! ky: option LTRAJHR is not yet coded in ALADIN, work has to be done later
    !     (especially here).
    IF (LELAM) THEN
      SP7A1(:,:) = ZSPBGA1(:,:)
    ENDIF
    IF (LELAM) THEN
      WRITE (NULOUT,*) ' SUECGES: LTRAJHR not yet coded for LELAM.'
      WRITE (NULOUT,*) ' SUECGES: COPY OF SP7A1 FROM TRAJ NOT YET CODED.'
      CALL ABOR1 (' SUECGES: ABOR1 CALLED')
    ENDIF
    ! =========================================================================

    IF (ALLOCATED(ZSPBGA3)) DEALLOCATE(ZSPBGA3)
    IF (ALLOCATED(ZSPBGA2)) DEALLOCATE(ZSPBGA2)
    IF (ALLOCATED(ZSPBGA1)) DEALLOCATE(ZSPBGA1)
    IF (ALLOCATED(ZGMVBG )) DEALLOCATE(ZGMVBG )
    IF (ALLOCATED(ZGMVBGS)) DEALLOCATE(ZGMVBGS)
    IF (ALLOCATED(ZGFLBG )) DEALLOCATE(ZGFLBG )

  ENDIF

ENDIF

!*    2.2   Initialise surface fields and calculate pressure on Jb vertical
!           levels

IF (L_JBVCOORD) THEN
  ALLOCATE (BLTOP(NGPTOT))
  CALL SUGRIDF(1,SPOR,LLSPOR,BLTOP)
  WRITE(NULOUT,'(/'' Initializing Jb Vertical Coordinate''/)')
  CALL STEPO('0IB00H000')
  DEALLOCATE (BLTOP)
ELSE
  CALL SUGRIDF(1,SPOR,LLSPOR)
ENDIF

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUECGES',1,ZHOOK_HANDLE)
END SUBROUTINE SUECGES
