SUBROUTINE SUINFCE

!**** *SUINFCE*  - Initializes the first guess error.
!****            - ECMWF version.

!     Purpose.
!     --------
!           Initializes the first guess error.
!           - ECMWF version.

!**   Interface.
!     ----------
!        CALL SUINFCE

!     Method.
!     -------
!        Errors are read, interpolated and modified.
!        Errors for specific humidity are calculated assuming a
!        simple error in relative humidity.

!     Externals.
!     ----------   
!                  INQGRIB -Determine forecast error grid parameters
!                  READGRIB-Read the forecast errors
!                  SUHICEE -interpolate the errors in the horizontal
!                  SUVIFCE -interpolate the errors to model levels
!                  SUMDFCE -Calculate horizontal pattern of bg error
!                  SUPRFFCE-Calculate global mean profile of bg error

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mike Fisher ECMWF

!     Modifications.
!     --------------
!        Original   : 96-04-17
!        L.Isaksen  : 96-05-05  Adapted for Message Passing
!        Mike Fisher: 96-08-01  Introduced LRDQERR and L3DBGERR
!        E. Holm    : 97-11-28  Included ozone
!        E.Andersson: 99-08-18  Use yomgrb grib codes
!        Mike Fisher: 00-10-26  Removed PFIELD argument.
!        Mike Fisher: 00-10-26  Allocation of FCEBUF removed to SUALGES.
!        R. El Khatib : 01-08-07 Pruning options
!        M.Hamrud   : 03-08-01  GFL introduction
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        M.Hamrud      01-Dec-2005 Generalized IO scheme
!        B. Strajnar: 06-05-2008 Add Aladin transforms (bi-Fourier), introduce LDLATLONG
!        R. El Ouaraini & L.Berre: 12-Feb-08 Multiply the sigmab(q) read in the errgrib file by:
!                                 REDNMC if Arpege and REDNMC*REDQGLOLAM if Aladin
!        L. Berre      06-Mar-2008 Use LSBLATLONG from YOMJG
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA, NGPBLKS
USE YOMCT0   , ONLY : NPROC,  LELAM
USE YOMLUN   , ONLY : NULOUT
USE YOMJG    , ONLY : L3DBGERR ,LCFCE    ,LRDQERR  ,LSBLATLONG  ,LJB_UNIVARIATE, &
                    & FCEMN3D, FCEMN2D, FCEMN_DERIVED    ,NOFCEF  ,&
                    & N_SPJB_VARS, SPJB_VARS_INFO, FGMWNE, REDNMC, REDQGLOLAM
USE YOMCT3   , ONLY : NSTEP
USE YOMGEM   , ONLY :    VBH      ,VAH      ,NGPTOT
USE YOMFCEB  , ONLY : FCEBUF
USE YOMCST   , ONLY : RPI
USE YOMGC    , ONLY : GELAM    ,GELAT, GM
USE YOMMP    , ONLY : NBSETLEV
USE YOMGRB   , ONLY : NGRBVO   ,NGRBU    ,NGRBQ    , &
                    & NGRBUCTP   ,NGRBUCLN   ,NGRBUCDV   , NGRBO3  , &
                    & NGRBD    ,NGRBT    ,NGRBLNSP
USE YOMSPJB  , ONLY : SP7T, SP7Q, SP7SP, GP7A3
USE YOM_YGFL , ONLY : YQ, YL, YI, YGFL, YGHG, NGHG, YGRG, NGRG, YAERO, NAERO
USE YOMGMV , ONLY : YT0, YT5
USE IOSTREAM_MIX , ONLY : SETUP_IOSTREAM, SETUP_IOREQUEST, IO_GET, &
 & CLOSE_IOSTREAM, TYPE_IOSTREAM , TYPE_IOREQUEST, IO_INQUIRE, &
 & CLOSE_IOREQUEST
USE TRAJECTORY_MOD, ONLY: GET_TRAJ_GRID, LTRAJHR, LTRAJHR_ALTI

IMPLICIT NONE

INTEGER(KIND=JPIM) :: JPMAXLAT
INTEGER(KIND=JPIM) :: JPMAXLEV
INTEGER(KIND=JPIM) :: JPMAXVAR

! ky: not consistent with PARDIM!! Why don't use JPMXGL and JPMXLE?
PARAMETER (JPMAXLAT=500,JPMAXLEV=100,JPMAXVAR=20)

INTEGER(KIND=JPIM) :: INROWS, INVARS,  II,&
                    & IPTJBVO, IPTJBD, IPTJBTU, IPTJBO3, IPTJBPSU, &
                    & IPTJBGHG(NGHG), IPTJBGRG(NGRG), IPTJBAERO(NAERO)
INTEGER(KIND=JPIM) :: ILONE(JPMAXLAT)
INTEGER(KIND=JPIM) :: IFIELDS(JPMAXVAR), INLEVS(JPMAXVAR)
INTEGER(KIND=JPIM) :: ILEVS(JPMAXLEV,JPMAXVAR)
INTEGER(KIND=JPIM) :: IGRIB2D(JPMAXLEV*JPMAXVAR),ILEVS2D(JPMAXLEV*JPMAXVAR)

REAL(KIND=JPRB) :: ZLATE(JPMAXLAT), ZLON0E(JPMAXLAT), ZDLONE(JPMAXLAT)
REAL(KIND=JPRB) :: ZA(JPMAXLEV,JPMAXVAR), ZB(JPMAXLEV,JPMAXVAR)

REAL(KIND=JPRB), ALLOCATABLE :: ZEGRID(:,:,:), ZIGRID(:,:), ZPRSFI(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVALH(:,:), ZVALM(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZPRSHO(:,:), ZPRSFO(:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZLATM(:), ZLONM(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV7(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV7S(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGFL7(:,:,:,:)

REAL(KIND=JPRB) :: ZFCE(NPROMA,NOFCEF)

real(kind=JPRB), allocatable :: zdae(:)

CHARACTER (7) :: CLFILE='errgrib'

INTEGER(KIND=JPIM) :: IENDC, IFLDOFF, &
 & INFLDS, INPTE, INPTM, IOFF, IOMASTER, &
 & ISTC, J, JK, JKGLO, JLEV, JVAR  , IFCEBVO, IFCEBD, &
 & IFCEBTU, IFCEBQ, IFCEBO3, IFCEBSPU, &
 & IFCEBGHG(NGHG), IFCEBGRG(NGRG), IFCEBAERO(NAERO), &
 & IVAR, JFIELD, IVARD, IVARTU, &
 & IVARQ, IVARO3, IVARSPU, & 
 & IVARGHG(NGHG), IVARGRG(NGRG), IVARAERO(NAERO), &
 & ISTEP0, ICEND, IBL, IGHG, IGRG, IAERO,IPTBG

LOGICAL :: LLDIV, LLFIRST, LLLNSPU, LLO3,&
 & LLQ, LLTU, LLU, LLVORT, LLFILLED, LLCLWC, LLCIWC, &
 & LLGHG(NGHG), LLGRG(NGRG), LLAERO(NAERO)  
LOGICAL :: LLSET(N_SPJB_VARS)
TYPE(TYPE_IOSTREAM) :: YL_IOSTREAM
TYPE(TYPE_IOREQUEST) :: YL_IOREQUEST

LOGICAL :: LLAER_POLE_CORR
LOGICAL :: LLAER_REL_ERR
REAL(KIND=JPRB) :: ZAER_REL_ERR, ZAEROMAX
REAL(KIND=JPRB) :: ZAERO(NPROMA,NFLEVG)

REAL(KIND=JPRB) :: ZHOOK_HANDLE

INTERFACE
#include "inv_trans.h"
#include "einv_trans.h"
END INTERFACE

#include "abor1.intfb.h"
#include "commfce2.intfb.h"
#include "gppre.intfb.h"
#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"
#include "speree.intfb.h"
#include "esperee.intfb.h"
#include "suhifce.intfb.h"
#include "sumdfce.intfb.h"
#include "suprffce.intfb.h"
#include "susepfce.intfb.h"
#include "sushfce.intfb.h"
#include "suvifce.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUINFCE',0,ZHOOK_HANDLE)

IPTJBVO  = HUGE(IPTJBVO)
IPTJBD   = HUGE(IPTJBD)
IPTJBTU  = HUGE(IPTJBTU)
IPTJBO3  = HUGE(IPTJBO3)
IPTJBPSU = HUGE(IPTJBPSU)
DO IGHG=1,NGHG
 IPTJBGHG(IGHG) = HUGE(IPTJBGHG(IGHG))
ENDDO
DO IGRG=1,NGRG
 IPTJBGRG(IGRG) = HUGE(IPTJBGRG(IGRG))
ENDDO
DO IAERO=1,NAERO
 IPTJBAERO(IAERO) = HUGE(IPTJBAERO(IAERO))
ENDDO
DO JFIELD=1,N_SPJB_VARS
  IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBVO) THEN
    IPTJBVO = SPJB_VARS_INFO(JFIELD)%IPTJB
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBD) THEN
    IPTJBD  = SPJB_VARS_INFO(JFIELD)%IPTJB
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBT) THEN
    IPTJBTU = SPJB_VARS_INFO(JFIELD)%IPTJB
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBO3) THEN
    IPTJBO3 = SPJB_VARS_INFO(JFIELD)%IPTJB
  ELSEIF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE==NGRBLNSP) THEN
    IPTJBPSU = SPJB_VARS_INFO(JFIELD)%IPTJB
  ELSEIF (NGHG > 0 .OR. NGRG > 0 .OR. NAERO > 0) THEN
    DO IGHG=1,NGHG
      IF(SPJB_VARS_INFO(JFIELD)%IGRIBCODE==YGHG(IGHG)%IGRBCODE) &
       & IPTJBGHG(IGHG) = SPJB_VARS_INFO(JFIELD)%IPTJB
    ENDDO
    DO IGRG=1,NGRG
      IF(SPJB_VARS_INFO(JFIELD)%IGRIBCODE==YGRG(IGRG)%IGRBCODE) &
       & IPTJBGRG(IGRG) = SPJB_VARS_INFO(JFIELD)%IPTJB
    ENDDO
    DO IAERO=1,NAERO
      IF(SPJB_VARS_INFO(JFIELD)%IGRIBCODE==YAERO(IAERO)%IGRBCODE) &
       & IPTJBAERO(IAERO) = SPJB_VARS_INFO(JFIELD)%IPTJB
    ENDDO
  ENDIF
ENDDO

!       1.    Only IOMASTER READS the files, the other PE's get the
!             data via message passing

IOMASTER=1
CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFILE),CDMODE='r',&
 & KIOMASTER=IOMASTER)
CALL SETUP_IOREQUEST(YL_IOREQUEST,'GRIDPOINT_FIELDS',LDGRIB=.TRUE.,LDINTONLY=.TRUE.,&
 &LDLATLONG=LSBLATLONG,KCHUNKSIZE=1)
CALL IO_INQUIRE(YL_IOSTREAM,YL_IOREQUEST,&
 & KLATS=INROWS,KLONS=ILONE,PLATS=ZLATE,&
 & KNVARS=INVARS,KPARAMS=IFIELDS,KNLEVS=INLEVS,&
 & PA=ZA,PB=ZB,KLEVS=ILEVS)
CALL CLOSE_IOREQUEST(YL_IOREQUEST)
CALL CLOSE_IOSTREAM(YL_IOSTREAM)
ZLON0E(:) = 0.0_JPRB

ZDLONE(1:INROWS) = 2.0_JPRB * RPI / ILONE(1:INROWS)

!       1.5   Test if some special-case fields are in the file

LLVORT = ANY(IFIELDS(1:INVARS) == NGRBVO)
LLU    = ANY(IFIELDS(1:INVARS) == NGRBU)
LLQ    = ANY(IFIELDS(1:INVARS) == NGRBQ)
LLTU   = ANY(IFIELDS(1:INVARS) == NGRBUCTP)
LLLNSPU= ANY(IFIELDS(1:INVARS) == NGRBUCLN)
LLDIV  = ANY(IFIELDS(1:INVARS) == NGRBUCDV)
LLO3   = ANY(IFIELDS(1:INVARS) == NGRBO3)
DO IGHG=1,NGHG
  LLGHG(IGHG) = ANY(IFIELDS(1:INVARS) == YGHG(IGHG)%IGRBCODE)
ENDDO
DO IGRG=1,NGRG
  LLGRG(IGRG) = ANY(IFIELDS(1:INVARS) == YGRG(IGRG)%IGRBCODE)
ENDDO
DO IAERO=1,NAERO
  LLAERO(IAERO) = ANY(IFIELDS(1:INVARS) == YAERO(IAERO)%IGRBCODE)
ENDDO

IF (.NOT.(LLU.OR.LLVORT )) THEN
  WRITE (NULOUT,&
   & '('' BACKGROUND ERROR FILE MUST CONTAIN '''    //&
   & ''' WIND OR VORTICITY'')')  
  CALL ABOR1 ('MISSING BACKGROUND ERROR FIELDS')
ENDIF

INPTE  = SUM(ILONE (1:INROWS))
INFLDS = SUM(INLEVS(1:INVARS))

II = 0
DO JVAR=1,INVARS
  DO JLEV=1,INLEVS(JVAR)
    II = II+1
    IGRIB2D(II)  = IFIELDS(JVAR)
    ILEVS2D (II) = ILEVS(JLEV,JVAR)
  ENDDO
ENDDO
ALLOCATE (ZEGRID(INPTE,INFLDS,1))

!       2.    Read the forecast errors

CALL SETUP_IOSTREAM(YL_IOSTREAM,'CIO',TRIM(CLFILE),CDMODE='r',KIOMASTER=1)
CALL SETUP_IOREQUEST(YL_IOREQUEST,'GRIDPOINT_FIELDS',LDGRIB=.TRUE.,&
 & KGRIB2D=IGRIB2D(1:INFLDS),KLEVS2D=ILEVS2D(1:INFLDS),&
 & KPROMA=INPTE,LDISTRIBUTE=.FALSE.,LDLATLONG=LSBLATLONG,KCHUNKSIZE=1)
CALL IO_GET(YL_IOSTREAM,YL_IOREQUEST,PR3=ZEGRID)
CALL CLOSE_IOREQUEST(YL_IOREQUEST)
CALL CLOSE_IOSTREAM(YL_IOSTREAM)

IF (NPROC>1) CALL COMMFCE2(ZEGRID,INPTE,INFLDS)

!     3.    Interpolate in the horizontal

INPTM = NGPTOT

ALLOCATE (ZLATM(INPTM),ZLONM(INPTM),ZIGRID(INPTM,INFLDS))

DO J=1,NGPTOT
  ZLONM(J)=GELAM(J)
  ZLATM(J)=GELAT(J)
ENDDO

CALL SUHIFCE(ZLON0E,ZDLONE,ZLATE,INROWS,ILONE,ZLONM,ZLATM,INPTM,&
 & INPTE,INFLDS,ZEGRID,ZIGRID)  

DEALLOCATE (ZEGRID,ZLONM,ZLATM)

ALLOCATE (ZVALM (NPROMA,NFLEVG))
ALLOCATE (ZPRSFO(NPROMA,NFLEVG))
ALLOCATE (ZPRSHO(NPROMA,0:NFLEVG))

!     3.5   Read (upper air) background state from trajectory style files
!           if LTRAJHR_ALTI=.t. 
!           otherwise obtain fields from an inverse transform of SP7 fields

ALLOCATE (ZGMV7(NPROMA,NFLEVG,YT5%NDIM,NGPBLKS))
ALLOCATE (ZGMV7S(NPROMA,YT5%NDIMS,NGPBLKS))
ALLOCATE (ZGFL7(NPROMA,NFLEVG,YGFL%NDIM5,NGPBLKS))

IF (LTRAJHR.AND.LTRAJHR_ALTI) THEN
  ISTEP0=0
  CALL GET_TRAJ_GRID(ZGMV7,ZGMV7S,ZGFL7,KSTEP=ISTEP0,LDBACKGR=.TRUE.)
ELSE
  IF (LELAM) THEN
    CALL EINV_TRANS(PSPSCALAR=SP7T,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
      & PGP=ZGMV7(:,:,YT0%MT,:))
  ELSE
    CALL INV_TRANS(PSPSCALAR=SP7T,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
      & PGP=ZGMV7(:,:,YT0%MT,:))
  ENDIF

  IF (ANY(SPJB_VARS_INFO(:)%L_IN_GPGFL)) THEN
    ! Really dirty to assume that q is the first GFL field
    ! Should be rewritten. Lars I.
    DO JFIELD=1,N_SPJB_VARS
      IF (SPJB_VARS_INFO(JFIELD)%L_IN_GPGFL) THEN
        IPTBG = SPJB_VARS_INFO(JFIELD)%IPTBG
        ZGFL7(:,:,YQ%MP,:)=GP7A3(:,:,IPTBG,:)
      ENDIF
    ENDDO

  ELSE
    IF (LELAM) THEN
      CALL EINV_TRANS(PSPSCALAR=SP7Q,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
        & PGP=ZGFL7(:,:,YQ%MP,:))
    ELSE
      CALL INV_TRANS(PSPSCALAR=SP7Q,KVSETSC=NBSETLEV,KPROMA=NPROMA,&
        & PGP=ZGFL7(:,:,YQ%MP,:))
    ENDIF
  ENDIF
  IF(LELAM) THEN
    CALL ESPEREE(1,1,SP7SP,ZGMV7S(:,YT0%MSP,:))
  ELSE
    CALL SPEREE(1,1,SP7SP,ZGMV7S(:,YT0%MSP,:))
  ENDIF
ENDIF

!     3.7   Work out what is needed, and where to put it.

CALL LOOKUP(NGRBVO,IFCEBVO)
CALL LOOKUP(NGRBQ, IFCEBQ , IVARQ)
CALL LOOKUP(NGRBO3,IFCEBO3, IVARO3)
DO IGHG=1,NGHG
  CALL LOOKUP(YGHG(IGHG)%IGRBCODE,IFCEBGHG(IGHG), IVARGHG(IGHG))
ENDDO
DO IGRG=1,NGRG
  CALL LOOKUP(YGRG(IGRG)%IGRBCODE,IFCEBGRG(IGRG), IVARGRG(IGRG))
ENDDO
DO IAERO=1,NAERO
  CALL LOOKUP(YAERO(IAERO)%IGRBCODE,IFCEBAERO(IAERO), IVARAERO(IAERO))
ENDDO

IF (LJB_UNIVARIATE) THEN
  CALL LOOKUP(NGRBT   , IFCEBTU , IVARTU)
  CALL LOOKUP(NGRBLNSP, IFCEBSPU, IVARSPU)
  CALL LOOKUP(NGRBD   , IFCEBD  , IVARD)
ELSE
  CALL LOOKUP(NGRBUCTP,   IFCEBTU , IVARTU )
  CALL LOOKUP(NGRBUCLN,   IFCEBSPU, IVARSPU)
  CALL LOOKUP(NGRBUCDV,   IFCEBD  , IVARD)
ENDIF

!*    4.     Interpolate forecast error variances in the vertical and
!            store in the forecast error buffer

LLFIRST = .TRUE.
LLSET(:)= .FALSE.

VARIABLE_LOOP : DO JVAR=1,INVARS
  IFLDOFF = SUM(INLEVS(1:JVAR-1))

!     ------------------------------------------------------------------

!*    5.    Identify field
!           --------------

  IF (.NOT.LLVORT .AND. IFIELDS(JVAR) == NGRBU) THEN
!--- If vorticity is required, but is not in the input file, then
!--- they are calculated from wind errors.
    CALL LOOKUP(NGRBVO,IOFF,IVAR)
    IF (IOFF==HUGE(IOFF)) CYCLE VARIABLE_LOOP

  ELSEIF (IFIELDS(JVAR) == NGRBQ) THEN
!--- Special case for .NOT.LRDQERR.
    IF (.NOT.LRDQERR) CYCLE VARIABLE_LOOP
    CALL LOOKUP(NGRBQ,IOFF,IVAR)
    IF (IOFF==HUGE(IOFF)) CYCLE VARIABLE_LOOP

  ELSE
!--- The default case. NB: LOOKUP returns HUGE if the grib code is not
!--- found in SPJB_VARS_INFO.
    CALL LOOKUP(IFIELDS(JVAR),IOFF,IVAR)
    IF (IOFF==HUGE(IOFF)) CYCLE VARIABLE_LOOP
  ENDIF

  ALLOCATE (ZPRSFI(NPROMA,  INLEVS(JVAR)))
  ALLOCATE (ZVALH (NPROMA,0:INLEVS(JVAR)))

  LLFILLED=FCEBUF%LFILLED

  MESSP_LOOP : DO JKGLO=1,NGPTOT,NPROMA
    ISTC=1
    IENDC=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1

    IF(.NOT.LLFIRST) THEN
      IF(LLFILLED) THEN
        CALL SC2RDG(FCEBUF,JKGLO,ISTC,IENDC,ZFCE)
      ENDIF
    ELSE
      ZFCE(:,:) = 0.0_JPRB
    ENDIF

!     5.2   Interpolate in the vertical  (Message Passing)

    IF (INLEVS(JVAR) /= 1) THEN

      
      
      DO JK=ISTC,IENDC
        ZPRSHO(JK,NFLEVG)= EXP(ZGMV7S(JK,YT0%MSP,IBL))
      ENDDO

      CALL GPPRE(NPROMA,ISTC,IENDC,NFLEVG,VAH,VBH,ZPRSHO,ZPRSFO)
      
      DO JLEV=1,INLEVS(JVAR)
        DO JK=ISTC,IENDC
          ZPRSFI(JK,JLEV) =  ZA(JLEV,JVAR)+ ZPRSHO(JK,NFLEVG)*ZB(JLEV,JVAR)
          ZVALH (JK,JLEV) = ZIGRID(JKGLO-1+JK,JLEV+IFLDOFF)
        ENDDO
      ENDDO

      ZVALH(ISTC:IENDC,0) = ZVALH(ISTC:IENDC,1)

      IF (ISTC /= 1) THEN
        WRITE (NULOUT,*) 'SUINFCE calls SUVIFCE with ISTC /= 1'
        CALL ABOR1 ('SUINFCE calls SUVIFCE with ISTC /= 1')
      ENDIF
      
      CALL SUVIFCE (ZVALH,ZPRSFI,NPROMA,IENDC-ISTC+1,&
       & INLEVS(JVAR),ZVALM,ZPRSFO,NFLEVG)  
      
      DO JLEV=1,NFLEVG
        DO JK=ISTC,IENDC
          ZFCE(JK,IOFF+JLEV) = ZVALM(JK,JLEV)
        ENDDO
      ENDDO

!     5.3   Don't interpolate single level data

    ELSE
      DO JK=ISTC,IENDC
        ZFCE(JK,IOFF+1) = ZIGRID(JKGLO-1+JK,IFLDOFF+1)
      ENDDO
    ENDIF

    LLSET(IVAR) = .TRUE.

!     IN THE CASE OF WIND, CALCULATE APPROPRIATE VORTICITY ERRORS

    IF (IOFF == IFCEBVO .AND. .NOT.LLVORT) THEN
      IF (IPTJBVO/=HUGE(IPTJBVO)) THEN
        DO JLEV=1,NFLEVG
          DO JK=ISTC,IENDC
            ZFCE(JK,IOFF+JLEV) = ZFCE(JK,IOFF+JLEV)&
             & *FCEMN3D(JLEV,IPTJBVO)/FCEMN_DERIVED%U(JLEV)  
          ENDDO
        ENDDO
      ELSE
        CALL ABOR1 ('BEHAVIOUR NOT DEFINED FOR VORTICITY NOT IN JB')
      ENDIF

      LLSET(IVAR) = .TRUE.
    ENDIF

!     CALCULATE APPROPRIATE DIVERGENCE ERRORS IF REQUIRED, BUT NOT AVAILABLE

    IF (LLFIRST .AND. .NOT.LLDIV .AND. IFCEBD/=HUGE(IFCEBD)) THEN
      IF (IPTJBD/=HUGE(IPTJBD)) THEN
        DO JLEV=1,NFLEVG
          DO JK=ISTC,IENDC
            ZFCE(JK,IFCEBD+JLEV) = FCEMN3D(JLEV,IPTJBD)
          ENDDO
        ENDDO
      ELSE
        CALL ABOR1 ('BEHAVIOUR NOT DEFINED FOR DIVERGENCE NOT IN JB')
      ENDIF

      LLSET(IVARD) = .TRUE.
    ENDIF

!     MULTIPLY HUMIDITY FORECAST ERROR BY REDNMC IF GLOBAL MODEL, BY REDNMC*REDQGLOLAM IF LAM

    IF (LLFIRST .AND. LLQ .AND. LRDQERR) THEN
        DO JLEV=1,NFLEVG
          DO JK=ISTC,IENDC
            IF (.NOT. LELAM) THEN
              ZFCE(JK,IFCEBQ+JLEV) = REDNMC * ZFCE(JK,IFCEBQ+JLEV)
            ELSE
              ZFCE(JK,IFCEBQ+JLEV) = REDNMC * REDQGLOLAM * ZFCE(JK,IFCEBQ+JLEV)
            ENDIF
          ENDDO
        ENDDO
    ENDIF

!     CALCULATE UNBALANCED TEMPERATURE IF REQUIRED, BUT NOT AVAILABLE

    IF (LLFIRST .AND. .NOT.LLTU .AND. IFCEBTU/=HUGE(IFCEBTU)) THEN
      IF (IPTJBTU/=HUGE(IPTJBTU)) THEN
        DO JLEV=1,NFLEVG
          DO JK=ISTC,IENDC
            ZFCE(JK,IFCEBTU+JLEV) = FCEMN3D(JLEV,IPTJBTU)
          ENDDO
        ENDDO
      ELSE
        CALL ABOR1 ('BEHAVIOUR NOT DEFINED FOR TEMPERATURE NOT IN JB')
      ENDIF

      LLSET(IVARTU) = .TRUE.
    ENDIF

!     CALCULATE UNBALANCED OZONE IF REQUIRED, BUT NOT AVAILABLE

    IF (LLFIRST .AND. .NOT.LLO3 .AND. IFCEBO3/=HUGE(IFCEBO3)) THEN
      IF (IPTJBO3/=HUGE(IPTJBO3)) THEN
        DO JLEV=1,NFLEVG
          DO JK=ISTC,IENDC
            ZFCE(JK,IFCEBO3+JLEV) = FCEMN3D(JLEV,IPTJBO3)
          ENDDO
        ENDDO
      ELSE
        CALL ABOR1 ('BEHAVIOUR NOT DEFINED FOR OZONE NOT IN JB')
      ENDIF

      LLSET(IVARO3) = .TRUE.
    ENDIF

!     CALCULATE UNBALANCED LNSP IF REQUIRED, BUT NOT AVAILABLE
    
    IF (LLFIRST .AND. .NOT.LLLNSPU .AND. IFCEBSPU/=HUGE(IFCEBSPU)) THEN
      IF (IPTJBPSU/=HUGE(IPTJBPSU)) THEN
        DO JK=ISTC,IENDC
          ZFCE(JK,IFCEBSPU+1) = FCEMN2D(IPTJBPSU)
        ENDDO
      ELSE
        CALL ABOR1 ('BEHAVIOUR NOT DEFINED FOR LNSP NOT IN JB')
      ENDIF

      LLSET(IVARSPU) = .TRUE.
    ENDIF
!     ------------------------------------------------------------------
!     CALCULATE GHG variables IF REQUIRED, BUT NOT AVAILABLE

     DO IGHG=1,NGHG
       IF (LLFIRST .AND. .NOT.LLGHG(IGHG) .AND. &
        & IFCEBGHG(IGHG)/=HUGE(IFCEBGHG(IGHG))) THEN
         IF (IPTJBGHG(IGHG)/=HUGE(IPTJBGHG(IGHG))) THEN
           DO JLEV=1,NFLEVG
             DO JK=ISTC,IENDC
               ZFCE(JK,IFCEBGHG(IGHG)+JLEV) = FCEMN3D(JLEV,IPTJBGHG(IGHG))
             ENDDO
           ENDDO
         ELSE
           CALL ABOR1 ('BEHAVIOUR NOT DEFINED FOR GHG NOT IN JB')
         ENDIF

         LLSET(IVARGHG(IGHG)) = .TRUE.
       ENDIF
     ENDDO

!     CALCULATE GRG variables IF REQUIRED, BUT NOT AVAILABLE

     DO IGRG=1,NGRG
       IF (LLFIRST .AND. .NOT.LLGRG(IGRG) .AND. &
        & IFCEBGRG(IGRG)/=HUGE(IFCEBGRG(IGRG))) THEN
         IF (IPTJBGRG(IGRG)/=HUGE(IPTJBGRG(IGRG))) THEN
           DO JLEV=1,NFLEVG
             DO JK=ISTC,IENDC
               ZFCE(JK,IFCEBGRG(IGRG)+JLEV) = FCEMN3D(JLEV,IPTJBGRG(IGRG))
             ENDDO
           ENDDO
         ELSE
           CALL ABOR1 ('BEHAVIOUR NOT DEFINED FOR GRG NOT IN JB')
         ENDIF

         LLSET(IVARGRG(IGRG)) = .TRUE.
       ENDIF
     ENDDO

!     CALCULATE AERO variables IF REQUIRED, BUT NOT AVAILABLE

!    20070809 J.Kaiser
!    Damping factor for the Aerosol Error at high latitudes:
!    1 between 60N and 60S, decaying to 0 at the poles - commented out. Now step function.
     allocate (zdae(IENDC))
!     do jk = 1, size (ZLATM)
!       write (*,*) jk, zlatm(j)
!     enddo

     zdae(:) = 1._JPRB

!     WRITE (*,*) "J.Kaiser: JKGLO, IENDC, NPROMA = ", JKGLO, IENDC, NPROMA

     LLAER_POLE_CORR = .FALSE. 

     IF(LLAER_POLE_CORR) THEN

      do JK = ISTC, IENDC
        J = JKGLO - 1 + JK
        if (abs (ZLATM(J)) > 1.048_JPRB) then
! linear damping
!         zdae(JK) = (abs (ZLATM(J)) - 1.5708_JPRB) * 0.9_JPRB /  &
!      &                         (1.0472_JPRB - 1.5708_JPRB) + 0.1_JPRB
! step-wise damping
          zdae(jk) = 0.001_JPRB
        endif
        WRITE (*,*) "JK, J, ZLATM(J), zdae(JK)", JK, J, ZLATM(J), zdae(JK)
      enddo

     ENDIF

! BEN - parameters for definition of standard deviation of aerosol fields as percentage of 
!       aerosol background values.
LLAER_REL_ERR = .FALSE.
ZAER_REL_ERR=0.2_JPRB ! 20% of aerosol background field

     DO IAERO=1,NAERO

! Copy background aerosol field from trajectory to local array ZAERO
       ZAERO(:,:) = 0.0_JPRB
      DO JK=ISTC,IENDC
        ZAERO(JK,:)= ZGFL7(JK,:,YAERO(IAERO)%MP,IBL) 
      ENDDO

       IF (LLFIRST .AND. .NOT.LLAERO(IAERO) .AND. &
        & IFCEBAERO(IAERO)/=HUGE(IFCEBAERO(IAERO))) THEN
         IF (IPTJBAERO(IAERO)/=HUGE(IPTJBAERO(IAERO))) THEN
           DO JLEV=1,NFLEVG
             DO JK=ISTC,IENDC
              IF(LLAER_REL_ERR) THEN
               ZAEROMAX = MAX(0.0_JPRB, ZAERO(JK,JLEV))
               ZFCE(JK,IFCEBAERO(IAERO)+JLEV) =ZAEROMAX *ZAER_REL_ERR  * zdae(JK)              
              ELSE
               ZFCE(JK,IFCEBAERO(IAERO)+JLEV) = FCEMN3D(JLEV,IPTJBAERO(IAERO)) * zdae(JK)
              ENDIF
             ENDDO
           ENDDO
         ELSE
           CALL ABOR1 ('BEHAVIOUR NOT DEFINED FOR AEROSOL NOT IN JB')
         ENDIF

         LLSET(IVARAERO(IAERO)) = .TRUE.
       ENDIF
     ENDDO

     deallocate (zdae)


!*    7.    SAVE THE INTERPOLATED VALUES IN FCEBUF  (Message Passing)
!           ---------------------------------------------------------

    CALL SC2WRG(FCEBUF,JKGLO,ISTC,IENDC,ZFCE)
    
  ENDDO MESSP_LOOP

  DEALLOCATE (ZVALH)
  DEALLOCATE (ZPRSFI)

  LLFIRST = .FALSE.
ENDDO VARIABLE_LOOP

!     ------------------------------------------------------------------

DEALLOCATE (ZIGRID)
DEALLOCATE (ZVALM)
DEALLOCATE (ZPRSFO)
DEALLOCATE (ZPRSHO)

!*    7.     Generate Q forecast errors

IF (IFCEBQ/=HUGE(IVARQ) .AND. .NOT.(LLQ.AND.LRDQERR)) THEN
  IF (LRDQERR) THEN
    WRITE(NULOUT,&
     & '(/'' WARNING: LRDQERR=.TRUE. BUT FC ERROR FILE DOES NOT '//&
     & 'CONTAIN Q ERRORS''/'//&
     & '''          BACKGROUND ERRORS FOR Q WILL BE '//&
     & 'CALCULATED FROM Q FIELD''/)')  
    LRDQERR = .FALSE.
  ENDIF
  NSTEP=-999
  LLCLWC=YL%LACTIVE
  LLCIWC=YI%LACTIVE
  WRITE(NULOUT,'(/'' SUINFCE WARNING: CLWC and CIWC not considered in ' //&
    & ' SPECRT convertion from T to Tv''/)')

  DO JKGLO=1,NGPTOT,NPROMA
    ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
    IBL=(JKGLO-1)/NPROMA+1
    CALL SUSHFCE (ICEND,JKGLO, &
                & ZGMV7(1,1,YT0%MT,IBL),ZGFL7(1,1,YQ%MP,IBL),&
                & ZGMV7S(1,YT0%MSP,IBL)) 
  ENDDO

  YL%LACTIVE=LLCLWC
  YI%LACTIVE=LLCIWC
  LLSET(IVARQ)=.TRUE.
ENDIF

IF (.NOT.ALL(LLSET(:))) THEN
  WRITE (NULOUT,*) 'ERROR (SUINFCE): SOME BG ERROR FIELDS WERE NOT INITIALIZED'
  WRITE (NULOUT,*) 'VAR  GRIB_FCE  SET?'
  WRITE (NULOUT,*) '---  --------  ----'
  DO JFIELD=1,N_SPJB_VARS
    WRITE (NULOUT,'(I3,6X,4X,L1)') JFIELD, &
         & SPJB_VARS_INFO(JFIELD)%IGRIBCODE_FCE, LLSET(JFIELD)
  ENDDO
  CALL ABOR1 ('SOME BACKGROUND ERROR FIELDS WERE NOT INITIALIZED')
ENDIF

!*    8.  Calculate Horizontal Pattern or Vertical Profile of bg error

IF (LCFCE .OR. .NOT.L3DBGERR) THEN
  CALL SUMDFCE
ELSE
  CALL SUPRFFCE
ENDIF

!*    9.  Apply map factor to U and V and do separation

DO JKGLO=1,NGPTOT,NPROMA
  ICEND=MIN(NPROMA,NGPTOT-JKGLO+1)
  CALL SUSEPFCE (ICEND,JKGLO,GM(JKGLO),FGMWNE(JKGLO))
ENDDO

DEALLOCATE(ZGMV7)
DEALLOCATE(ZGMV7S)
DEALLOCATE(ZGFL7)

IF (LHOOK) CALL DR_HOOK('SUINFCE',1,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

CONTAINS

  SUBROUTINE LOOKUP (KGRIBCODE, KOFF_FCE, KVAR)
  !
  !--- looks up the grib code KGRIBCODE in SPJB_VARS_INFO, and
  !--- returns the corresponding value of SPJB_VARS_INFO(JF)%IPTFCE-1.
  !--- Optionally returns JF. Returns HUGE if the grib code is not found.

  INTEGER(KIND=JPIM), INTENT(IN)  :: KGRIBCODE
  INTEGER(KIND=JPIM), INTENT(OUT) :: KOFF_FCE
  INTEGER(KIND=JPIM), INTENT(OUT), OPTIONAL :: KVAR

  INTEGER(KIND=JPIM) :: JF
  REAL(KIND=JPRB) :: ZHOOK_HANDLE

  IF (LHOOK) CALL DR_HOOK('SUINFCE:LOOKUP',0,ZHOOK_HANDLE)

  KOFF_FCE = HUGE(KOFF_FCE)
  IF (PRESENT(KVAR)) KVAR = HUGE(KVAR)

  DO JF=1,N_SPJB_VARS
    IF (SPJB_VARS_INFO(JF)%IGRIBCODE_FCE == KGRIBCODE) THEN
      KOFF_FCE = SPJB_VARS_INFO(JF)%IPTFCE-1
      IF (PRESENT(KVAR)) KVAR = JF
      EXIT
    ENDIF
  ENDDO

  IF (LHOOK) CALL DR_HOOK('SUINFCE:LOOKUP',1,ZHOOK_HANDLE)
  END SUBROUTINE LOOKUP

END SUBROUTINE SUINFCE
