SUBROUTINE SUJBCOSU (KPRT,PCOVTPS,PCOVP,PCOVD,PCOVQ,PCOVO3,&
 & KLEV,KNUMB)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NDGLG     ,NDGNH    ,NSMAX    ,NTMAX
USE YOMCT0   , ONLY : NPRTRW
USE YOMLEG   , ONLY : RW       ,RMU
USE YOMLUN   , ONLY : NULOUT   ,NULERR
USE YOMCST   , ONLY : RPI      ,RA
USE YOMMP    , ONLY : NPRCIDS  ,MYSETW   ,NPROCM   ,&
 & MYSETV  
USE YOMTAG   , ONLY : MTAGFCE
USE MPL_MODULE
USE YOMJG , ONLY : SPJB_VARS_INFO ,LCORCOSU_VCOR
USE YOMGRB , ONLY : NGRBO3

!**** *SUJBCOSU* - Enforce compactly supported correlations

!     Purpose. Smooth out ghost increments and spectral sampling noise.
!     --------

!     ----------
!     Interface. CALL SUJBCOSU(...)
!     ----------

!     Explicit arguments : In     : KPRT     = print level
!     -------------------- In/out : PCOVx    = covariances for parameter x
!                          In     : KLEV     = number of levels in PCOVx
!                          In     : KNUMB    = number of spectral matrices in PCOVx

!     Method.
!     -------
!        Use the Legendre transforms to go between power spectra of 
!     horizontal correlations and polar isotropic covariance structures.
!        Take the square root of the input correlation for the convolution
!     operator in physical space (i.e. for the multiplication in spectral
!     space.
!        Multiply the associated gridpoint structure by a gridpoint mask
!     enforcing compactly supported horizontal correlations. This is
!     equivalent to a convolution in spectral space, i.e. a spectral smoothing.
!        Convolve the resulting function by itself, i.e. square it in spectral
!     space. This guarantees the positive definiteness and compacity of the
!     result, as well as the fidelity to the original correlations inside
!     the mask.
!       (it is not possible to mask the correlation function itself because 
!        this may yield non-positive correlation matrices)

!     The above only applies for compact support level by level. To extend
!     the code to the whole covariance matrix, compact support is needed for 
!     the correlation between all pairs of levels. In other words, instead
!     of applying compact support to the diagonal of the covariance matrices only,
!     apply it to the whole matrix. The approach is to form the symmetric square
!     root of the covariance matrix, then apply the compact support to all
!     pairs of levels of the symmetric square root following the approach above, 
!     and then come back to the covariance matrix. Manipulating the symmetric
!     square root only guarantees positive definite covariance matrices.

!     Due to the symmetry, only the diagonal elements and below need to be
!     calculated explicitly, reducing the calculations by a factor of two.

!     Reference. Gaspari & Cohn 1996 (Math Geology), Courtier & al part I 1997
!     ---------  plus some experimentation at AES (P. Gauthier 1996)

!     Author : Francois Bouttier *ECMWF*
!     --------

!     Modifications :
!     ---------------
!     97-02-06 F.Bouttier : write original
!     Modified 98-08-10 by K. YESSAD: removal of LRPOLE option.
!          - removal of argument .FALSE. in 'CALL SPEREE'.
!     01-04-02 E. Holm    : Extend code to 3D compact support. Call moved
!                           from SUJBCOR to SUJBDAT.
!     01-11-08 E.Holm     : Fix for compact support
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!     K.Yessad   : Nov 2006  Check that the matrix entering EIGENMD is positive
!       definite.
!        L.Berre       05-Jan-2007 LCORCOSU_VCOR option to apply compact support on
!        vertical correlations (instead of covariances): this can avoid spurious
!        negative eigenvalues in specific vertically extrapolated fc diffces
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPRT 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOVTPS(KLEV+1,KLEV+1,0:KNUMB-1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOVP(KLEV,KLEV,0:KNUMB-1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOVD(KLEV,KLEV,0:KNUMB-1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOVQ(KLEV,KLEV,0:KNUMB-1) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PCOVO3(KLEV,KLEV,0:KNUMB-1) 

!     ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZPNM0(NDGNH,2*NTMAX),ZPNM0_TP(2*NTMAX,NDGNH)
REAL(KIND=JPRB) :: ZCORSP(0:NSMAX),ZCORGP(NDGLG),ZDIF(0:NSMAX)

REAL(KIND=JPRB), ALLOCATABLE :: ZCOV(:,:,:), ZMAT(:,:), ZTVAR(:)
REAL(KIND=JPRB), ALLOCATABLE :: ZVEC(:,:),ZVEC_TP(:,:), ZVAL(:), ZVALSQRT(:)
  
INTEGER(KIND=JPIM) :: IOF, JGL, JFIL, JLEV1, JLEV2, JJ, JK, JL, JN, IVARMAX, IDIM
INTEGER(KIND=JPIM) :: JROC,ISENDSETA,IBFLEN,IMSGLEN,IRECV,ISEND,ITAG
REAL(KIND=JPRB) :: ZASY, ZD1, ZD2, ZFACT(NDGLG), ZMU1, ZMU2, ZSYM

LOGICAL :: LLEVI,LLEVN,LLEVZ

INTEGER(KIND=JPIM) :: MYSENDSET
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

INTERFACE
#include "trans_inq.h"
END INTERFACE

#include "abor1.intfb.h"
#include "eigenmd.intfb.h"
#include "set2pe.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUJBCOSU',0,ZHOOK_HANDLE)

!     -----------------------------------------------------------------
!     Note the links with the notations of Courtier et al 97, part I :
!         zspec is the power spectrum = (2n+1)bn
!         bn = zspec/(2n+1) is the modal spectrum
!         fn = sqrt(2n+1)bn = zspec/sqrt(2n+1) is the representation
!                      with speree(fn)=gridpoint structure
!         fgscnm=sqrt( (2n+1)/zspec ) is the inverse square root
!         hn = sqrt(sqrt(2n+1)fn) = sqrt(zspec) is the repr of the convol sqrt
!         hn * hn = sqrt(2n+1)fn = zspec
!     here we want to mask speree(hn), rebuild zspec = hn^2 then fgscnm
!     -----------------------------------------------------------------
!        0. Initializations

!        0.1 Masking coefficients for grid point structure

ZD1=4000._JPRB
ZD2=ZD1*1.5_JPRB
IF ((ZD1<0.0_JPRB).OR.(ZD2>20000._JPRB).OR.(ZD2<0.0_JPRB).OR.(ZD2>20000._JPRB)&
 & .OR.(ZD1>ZD2)) CALL ABOR1(' SUJBCOSU : ILLEGAL ZD1 OR ZD2')  
IF(NDGNH/=NDGLG/2)CALL ABOR1(' SUJBCOSU : ODD NDGLG IS NOT SUPPORTED')
ZMU1=COS(ZD1*1000._JPRB/RA)
ZMU2=COS(ZD2*1000._JPRB/RA)
DO JGL=1,NDGLG
  IF     (RMU(JGL)>ZMU1) THEN
    ZFACT(JGL)=1.0_JPRB
  ELSEIF (RMU(JGL)<ZMU2) THEN
    ZFACT(JGL)=0.0_JPRB
  ELSE
    ZFACT(JGL)=0.5_JPRB+COS( RPI*(ZMU1-RMU(JGL))/(ZMU1-ZMU2) )/2.0_JPRB
  ENDIF
ENDDO

!        0.2 Diffusion coefficients for spectrum to minimize ripples
  
ZDIF(:)=1.0_JPRB
ZDIF(NSMAX  )=0.1_JPRB
ZDIF(NSMAX-1)=0.2_JPRB
ZDIF(NSMAX-2)=0.5_JPRB
ZDIF(NSMAX-3)=0.8_JPRB
ZDIF(NSMAX-4)=0.9_JPRB

!        0.3 Prepare the Legendre polynomials

IBFLEN=NDGNH*(2*NTMAX)
ITAG=MTAGFCE+4
IF (MYSETW==NPROCM(0)) THEN
  ZPNM0(:,:)=0.0_JPRB
  CALL TRANS_INQ(PRPNM=ZPNM0(1:NDGNH,1:2*NTMAX))
  DO JROC=1,NPRTRW-1
    ISENDSETA=MYSENDSET(NPRTRW,MYSETW,JROC)
    CALL SET2PE(ISEND,0,0,ISENDSETA,MYSETV)
    CALL MPL_SEND(ZPNM0(:,:),KDEST=NPRCIDS(ISEND),KTAG=ITAG, &
     & CDSTRING='SUJBCOSU:')  
  ENDDO
ELSE
  CALL SET2PE(IRECV,0,0,NPROCM(0),MYSETV)
  CALL MPL_RECV(ZPNM0(:,:),KSOURCE=NPRCIDS(IRECV),KTAG=ITAG, &
   & KOUNT=IMSGLEN,CDSTRING='SUJBCOSU:')  
  IF (IMSGLEN/=IBFLEN) CALL ABOR1('SUJBCOSU : RECEIVED'//&
   & ' MESSAGE LENGTH OF WRONG SIZE')  
ENDIF
ZPNM0_TP(:,:)=TRANSPOSE(ZPNM0(:,:))

!     Loop on variables :

WRITE(NULOUT,*) ' Variables for Jb compact support :'
WRITE(NULOUT,*) '     1 : unbalanced temperature and surface pressure'
WRITE(NULOUT,*) '     2 : vorticity'
WRITE(NULOUT,*) '     3 : unbalanced divergence'
WRITE(NULOUT,*) '     4 : specific humidity'
IVARMAX=4
IF(ANY(SPJB_VARS_INFO(:)%IGRIBCODE==NGRBO3))THEN
  WRITE(NULOUT,*) '     5 : unbalanced ozone '
  IVARMAX=5
ENDIF

! Only humidity and ozone
CALL GSTATS(1991,0)
VAR_LOOP : DO JFIL=4,IVARMAX

  IF (JFIL==1) THEN
    IDIM = KLEV + 1
  ELSE
    IDIM = KLEV
  ENDIF
  IF (KPRT>=1) WRITE(NULOUT,*) 'SUJBCOSU - masking ',IDIM,'by',IDIM,&
   & 'matrix - JFIL=',JFIL,' d1,d2(km)=',INT(ZD1),INT(ZD2)  

  ALLOCATE (ZCOV(IDIM,IDIM,0:KNUMB-1))
  ALLOCATE (ZMAT(IDIM,IDIM))
  ALLOCATE (ZTVAR(IDIM))
  ALLOCATE (ZVEC(IDIM,IDIM))
  ALLOCATE (ZVEC_TP(IDIM,IDIM))
  ALLOCATE (ZVAL(IDIM))
  ALLOCATE (ZVALSQRT(IDIM))
  IF (JFIL==1) ZCOV(:,:,:)=PCOVTPS(:,:,:)
  IF (JFIL==2) ZCOV(:,:,:)=PCOVP  (:,:,:)
  IF (JFIL==3) ZCOV(:,:,:)=PCOVD  (:,:,:)
  IF (JFIL==4) ZCOV(:,:,:)=PCOVQ  (:,:,:)
  IF (JFIL==5) ZCOV(:,:,:)=PCOVO3 (:,:,:)

!           0.  Calculate total variances for LJBCOSU_VCOR

  DO JJ=1,IDIM
    ZTVAR(JJ)=SUM(ZCOV(JJ,JJ,:))
  ENDDO

!           1.  Calculate symmetric square root
      
  DO JN=0,NSMAX

    IF (.NOT.LCORCOSU_VCOR) THEN
      ZMAT(:,:)=ZCOV(:,:,JN)
    ELSE
      DO JJ=1,IDIM
        DO JK=1,IDIM
          ZMAT(JJ,JK)=ZCOV(JJ,JK,JN)/SQRT(ZTVAR(JJ)*ZTVAR(JK))
        ENDDO
      ENDDO
    ENDIF

    IF (KPRT>=2) WRITE(NULOUT,*) 'SUJBCOSU - eigenvalues matrix',JN
    CALL EIGENMD(ZMAT,ZVEC,ZVAL,IDIM,LLEVI,LLEVN,LLEVZ)

    IF (LLEVI) THEN
      WRITE(NULERR,*) ' SUJBCOSU: ZMAT has at least one not real eigenvalue.'
      WRITE(NULERR,*) ' for total wavenumber =',JN
    ENDIF
    IF (LLEVN) THEN
      WRITE(NULERR,*) ' SUJBCOSU: ZMAT has at least one real eigenvalue < 0.'
      WRITE(NULERR,*) ' for total wavenumber =',JN
    ENDIF
    IF (LLEVI.OR.LLEVN) CALL FLUSH(NULERR)
    IF (LLEVI.OR.LLEVN) CALL ABOR1(' SUJBCOSU: ABOR1 nr 1a')

    ZVALSQRT(:)=SQRT(ZVAL(:))
    ZVEC_TP(:,:)=TRANSPOSE(ZVEC(:,:))
    DO JJ=1,IDIM
      DO JK=1,IDIM
        ZCOV(JK,JJ,JN)=0.0_JPRB
        DO JL=1,IDIM
          ZCOV(JK,JJ,JN)=ZCOV(JK,JJ,JN)+ &
           & ZVALSQRT(JL)*ZVEC_TP(JL,JJ)*ZVEC_TP(JL,JK)
        ENDDO
      ENDDO
    ENDDO

  ENDDO

!           2.  Make compact support for all pairs of levels

  LEV1_LOOP : DO JLEV1=1,IDIM
    LEV2_LOOP : DO JLEV2=1,JLEV1

      IF (KPRT>=2) WRITE(NULOUT,*) 'SUJBCOSU - jfil,jlev1,jlev2=',JFIL,JLEV1,JLEV2

!           2.1 Apply diffusion to spectrum of one pair of levels

      ZCORSP(:)=ZDIF(:)*ZCOV(JLEV2,JLEV1,:)

!           2.2  Convert into gridpoint structure with inverse Legendre transform

      IOF=NTMAX+2
      DO JGL=1,NDGNH
        ZSYM=0._JPRB
        ZASY=0._JPRB
        DO JN=0,NSMAX,2
          ZSYM=ZSYM+ZPNM0_TP(IOF-JN,JGL)*ZCORSP(JN)
        ENDDO
        DO JN=1,NSMAX,2
          ZASY=ZASY+ZPNM0_TP(IOF-JN,JGL)*ZCORSP(JN)
        ENDDO
        ZCORGP(JGL)=ZSYM+ZASY
        ZCORGP(NDGLG+1-JGL)=ZSYM-ZASY
      ENDDO

!          2.3  Mask the gridpoint structure

      ZCORGP(:)=ZCORGP(:)*ZFACT(:)

!          2.4  Convert back to spectral repres. with direct Legendre transform

      IOF=NTMAX+2
      ZCORSP(:)=0.0_JPRB
      DO JGL=1,NDGLG/2
        ZSYM=ZCORGP(JGL)+ZCORGP(NDGLG+1-JGL)
        ZASY=ZCORGP(JGL)-ZCORGP(NDGLG+1-JGL)
        DO JN=0,NSMAX,2
          ZCORSP(JN)=ZCORSP(JN)+RW(JGL)*ZPNM0_TP(IOF-JN,JGL)*ZSYM
        ENDDO
        DO JN=1,NSMAX,2
          ZCORSP(JN)=ZCORSP(JN)+RW(JGL)*ZPNM0_TP(IOF-JN,JGL)*ZASY
        ENDDO
      ENDDO

!          2.4  Update symmetric square root (now compactly supported)

      ZCOV(JLEV2,JLEV1,:)=ZCORSP(:)

    ENDDO LEV2_LOOP
  ENDDO LEV1_LOOP

!          3.  Update symmetric square root upper diagonal

  DO JN=0,NSMAX
    DO JJ=1,IDIM
      DO JK=JJ+1,IDIM
        ZCOV(JK,JJ,JN)=ZCOV(JJ,JK,JN)
      ENDDO
    ENDDO
  ENDDO

!          3.5  Convert back from sqrt of vcor to sqrt of vcov

!          (Note that SQRT(ZTVAR) is applied to the JK part
!          to be consistent with the way step 4 (below) is coded.)

  IF (LCORCOSU_VCOR) THEN
    DO JN=0,NSMAX
      DO JJ=1,IDIM
        DO JK=1,IDIM
          ZCOV(JJ,JK,JN)=ZCOV(JJ,JK,JN)*SQRT(ZTVAR(JK))
        ENDDO
      ENDDO
    ENDDO
  ENDIF

!           4.  Go back to covariances, squaring symmetric sqrt

  IF (JFIL==1) THEN 
    DO JN=0,NSMAX
      DO JJ=1,IDIM
        DO JK=1,IDIM
          PCOVTPS(JJ,JK,JN)=0.0_JPRB
          DO JL=1,IDIM
            PCOVTPS(JJ,JK,JN)=PCOVTPS(JJ,JK,JN)+ZCOV(JL,JJ,JN)*ZCOV(JL,JK,JN)
          ENDDO
        ENDDO
      ENDDO
    ENDDO
  ELSEIF (JFIL==2) THEN
    DO JN=0,NSMAX
      DO JJ=1,IDIM
        DO JK=1,IDIM
          PCOVP(JJ,JK,JN)=0.0_JPRB
          DO JL=1,IDIM
            PCOVP(JJ,JK,JN)=PCOVP(JJ,JK,JN)+ZCOV(JL,JJ,JN)*ZCOV(JL,JK,JN)
          ENDDO
        ENDDO
      ENDDO
    ENDDO
  ELSEIF (JFIL==3) THEN
    DO JN=0,NSMAX
      DO JJ=1,IDIM
        DO JK=1,IDIM
          PCOVD(JJ,JK,JN)=0.0_JPRB
          DO JL=1,IDIM
            PCOVD(JJ,JK,JN)=PCOVD(JJ,JK,JN)+ZCOV(JL,JJ,JN)*ZCOV(JL,JK,JN)
          ENDDO
        ENDDO
      ENDDO
    ENDDO
  ELSEIF (JFIL==4) THEN
    DO JN=0,NSMAX
      DO JJ=1,IDIM
        DO JK=1,IDIM
          PCOVQ(JJ,JK,JN)=0.0_JPRB
          DO JL=1,IDIM
            PCOVQ(JJ,JK,JN)=PCOVQ(JJ,JK,JN)+ZCOV(JL,JJ,JN)*ZCOV(JL,JK,JN)
          ENDDO
        ENDDO
      ENDDO
    ENDDO
  ELSEIF (JFIL==5) THEN
    DO JN=0,NSMAX
      DO JJ=1,IDIM
        DO JK=1,IDIM
          PCOVO3(JJ,JK,JN)=0.0_JPRB
          DO JL=1,IDIM
            PCOVO3(JJ,JK,JN)=PCOVO3(JJ,JK,JN)+ZCOV(JL,JJ,JN)*ZCOV(JL,JK,JN)
          ENDDO
        ENDDO
      ENDDO
    ENDDO
  ENDIF

  DEALLOCATE (ZCOV,ZMAT,ZTVAR,ZVEC,ZVEC_TP,ZVAL,ZVALSQRT)

ENDDO VAR_LOOP
CALL GSTATS(1991,1)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUJBCOSU',1,ZHOOK_HANDLE)
END SUBROUTINE SUJBCOSU

