SUBROUTINE SUJQCOR(klev,ksmax,PCOV,PVCORIN,PVCOR,PHCOR,cdvar)

!     Purpose.
!     --------
!       Compute vertical and horizontal model error covariances
!       from input data.
!       Adapted from SUJBCOR.

!     Author.
!     -------
!       Yannick Tremolet

!     Modifications.
!     --------------
!       Original  : 12-May-2004
!     K.Yessad   : Nov 2006  Check that the matrix entering EIGENMD is positive
!       definite.
! ------------------------------------------------------------------

USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK , ONLY: LHOOK, DR_HOOK
USE YOMLAP  , ONLY: RCAPDI
USE YOMLUN  , ONLY: NULOUT   ,NULERR

! ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN)  :: klev, ksmax
REAL(KIND=JPRB)   , INTENT(IN)  :: PCOV   (klev,klev,0:ksmax)
REAL(KIND=JPRB)   , INTENT(OUT) :: PVCORIN(klev,klev,0:ksmax)
REAL(KIND=JPRB)   , INTENT(OUT) :: PVCOR  (klev,klev,0:ksmax)
REAL(KIND=JPRB)   , INTENT(OUT) :: PHCOR(klev,0:ksmax)
CHARACTER(LEN=*), INTENT(IN), OPTIONAL :: cdvar

! ------------------------------------------------------------------

REAL(KIND=JPRB) :: ZCOR(klev,klev), ZTCOV(klev,klev)
REAL(KIND=JPRB) :: ZEIGVECS(klev,klev), ZEIGVALS(klev)
REAL(KIND=JPRB) :: ZTCOR(klev,klev), ZSPLSC(klev)

INTEGER(KIND=JPIM) :: jl, jk, jn
REAL(KIND=JPRB) :: zz, zn
LOGICAL :: LLEVI,LLEVN,LLEVZ
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "eigenmd.intfb.h"
#include "multvdv.h"

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUJQCOR',0,ZHOOK_HANDLE)

! ------------------------------------------------------------------

! 1. Compute the vertical correlation matrices

! This could be distributed in the same way as spectral wavenumbers
! if it becomes expensive. YT.

CALL GSTATS(1456,0)

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jn,jk,jl,ZCOR,ZEIGVECS,ZEIGVALS,LLEVI,LLEVN,LLEVZ)
DO jn=0,ksmax
! Normalize covariances
  DO jk=1,klev
    DO jl=1,klev
      ZCOR(jl,jk) = PCOV(jl,jk,jn)/SQRT(PCOV(jl,jl,jn)*PCOV(jk,jk,jn))
    ENDDO
  ENDDO

! Compute eigenvalues and eigenvectors of inverse correlation matrices
  CALL EIGENMD(ZCOR,ZEIGVECS,ZEIGVALS,klev,LLEVI,LLEVN,LLEVZ)
  IF (LLEVI) THEN
    WRITE(NULERR,*) ' SUJQCOR: ZCOR has at least one not real eigenvalue.'
    WRITE(NULERR,*) ' for total wavenumber =',JN
  ENDIF
  IF (LLEVN) THEN
    WRITE(NULERR,*) ' SUJQCOR: ZCOR has at least one real eigenvalue < 0.'
    WRITE(NULERR,*) ' for total wavenumber =',JN
  ENDIF
  IF (LLEVZ) THEN
    WRITE(NULERR,*) ' SUJQCOR: ZCOR has at least one real eigenvalue = 0.'
    WRITE(NULERR,*) ' for total wavenumber =',JN
  ENDIF
  IF (LLEVI.OR.LLEVN.OR.LLEVZ) CALL FLUSH(NULERR)
  IF (LLEVI.OR.LLEVN.OR.LLEVZ) CALL ABOR1(' SUJQCOR: ABOR1 nr 1a')
  ZEIGVALS(:)=SQRT(ZEIGVALS(:))

! Form inverse square root of correlation matrices and inverse
  CALL MULTVDV(ZEIGVECS,ZEIGVALS,PVCORIN(:,:,jn))
  ZEIGVALS(:)=1.0_JPRB/ZEIGVALS(:)
  CALL MULTVDV(ZEIGVECS,ZEIGVALS,PVCOR  (:,:,jn))
ENDDO
!$OMP END PARALLEL DO

! ------------------------------------------------------------------

! 2. Compute the horizontal correlation matrices

! Extract the horizontal variance spectra, level by level

ZTCOV(:,:)=PCOV(:,:,0)
DO jn=1,ksmax
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jk,jl)
  DO jk=1,klev
    DO jl=1,klev
      ZTCOV(jl,jk) = ZTCOV(jl,jk) + PCOV(jl,jk,jn)
    ENDDO
  ENDDO
!$OMP END PARALLEL DO
ENDDO

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jn,jl)
DO jn=0,ksmax
  DO jl=1,klev
    PHCOR(jl,jn) = PCOV(jl,jl,jn) / ZTCOV(jl,jl)
  ENDDO
ENDDO
!$OMP END PARALLEL DO

! Form inverse square root of the horizontal correlation
! spectra - remember the 2n+1 factor !

!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jn,zn,jl)
DO jn=0,ksmax
  zn=SQRT(REAL(2*jn+1,JPRB))
  DO jl=1,klev
    PHCOR(jl,jn) = zn / SQRT(PHCOR(jl,jn))
  ENDDO
ENDDO
!$OMP END PARALLEL DO

! ------------------------------------------------------------------
! 3. Diagnostics

IF (PRESENT(cdvar)) THEN

! Print implied vertical correlations
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jk,jl)
  DO jk=1,klev
    DO jl=1,klev
      ZTCOR(jl,jk)=ZTCOV(jl,jk)/SQRT(ZTCOV(jl,jl)*ZTCOV(jk,jk))
    ENDDO
  ENDDO
!$OMP END PARALLEL DO

  WRITE(NULOUT,*)' Diagnostics of the average vertical correlations:'
  WRITE(NULOUT,*)cdvar,' correlations (*100)'
  WRITE(NULOUT,832)(jl, jl=1,klev)
  DO jk=1,klev
    WRITE(NULOUT,831)(NINT(100._JPRB*ZTCOR(jl,jk)),jl=1,klev)
  ENDDO
  831 FORMAT(1X,100I3)
  832 FORMAT(/1X,100I3)

! Print implied horizontal correlation structures
!   Warning: this will make sense only if FGSCNM contains Ch^-1/2, not Ch
!   Print actual length scale of spectrally fitted
!   correlation function. L**2=-2*F(0)/Laplace(F)(0)
!   (The '2' should be there in the 2D-case (Daley p.110)

  WRITE(NULOUT,*)' Diagnostics of the horizontal correlations :'
  ZSPLSC(:)=0.0
  DO jn=0,ksmax
    zz=RCAPDI(jn)*REAL(2*jn+1,JPRB)
    DO jl=1,klev
      ZSPLSC(jl)=ZSPLSC(jl)+zz/(PHCOR(jl,jn)**2)  
    ENDDO
  ENDDO
  ZSPLSC(:)=SQRT( -2.0_JPRB/ ZSPLSC(:) )
  WRITE(NULOUT,*)cdvar,' correlation length scales (km):'
  WRITE(NULOUT,FMT='(1X,100F8.1)')ZSPLSC(1:klev)/1000.0_JPRB
ENDIF
CALL GSTATS(1456,1)

! ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUJQCOR',1,ZHOOK_HANDLE)
END SUBROUTINE SUJQCOR
