SUBROUTINE SUSHFCE(KNUMB,KSTGLO,PTT0,PQT0,PSPT0)

!**** *SUSHFCE*  - ROUTINE FOR GENERATING BACKGROUND ERRORS OF
!****              HUMIDITY.

!     PURPOSE.
!     --------

!        GENERATE BACKGROUND ERRORS OF SPECIFIC HUMIDITY.

!**   INTERFACE.
!     ----------

!        *CALL* *SUSHFCE(..)*

!        INPUT:
!           *PSPT0*   - FIRST GUESS SURFACE PRESSURE
!           *PTT0*    - FIRST GUESS TEMPERATURE
!           *PQT0*    - FIRST GUESS SPECIFIC HUMIDITY

!        IMPLICIT OUTPUT:
!                      BACKGROUND ERRORS FOR SPECIFIC HUMIDITY ARE
!                      WRITTEN TO THE BACKGROUND ERROR BUFFER.

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------

!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------

!        W. HECKLEY    *ECMWF*

!     MODIFICATIONS.
!     --------------

!        W. HECKLEY    *ECMWF*  91:09:25 CHANGED TO RUN BELOW STEPO
!        Modified : 91-10-15 Bill Heckley and Mats Hamrud ECMWF
!                            Use of MIOLIB for IO on forecast error files
!        W. HECKLEY 04:02:92 - Change of record order on MIO file
!        Bill Heckley 30:03:92 - IO scheme
!        Per Unden 95-10-05 - Vary humidity errors depending on RH and T
!        Per Unden 95-11-28 - Vary humidity errors with p and L/S mask
!        Per Unden 96-02-29 - Set stratospheric value very low
!        M.Hamrud  96-03-12 - Preparation for message passing
!        L.Isaksen 96-07-07 - Fix for message passing
!        M.Fisher  96-12-13 - Removed SUSHFCE from JGNR.
!        A.Simmons 96-11-04 - Remove setting of low stratospheric errors
!        Per Unden 97-03-04 - Use Tetens formula wirh new constants
!        A.Simmons 99-10-25 - Set low errors above lapse-rate tropopause
!        R. El Khatib : 01-08-07 Pruning options
!        E. Holm   03-04-14 - New humidity analysis, removal of humidity
!                             error manipulations if LJB_NONLINEAR_CVHUM
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA
USE YOMCST   , ONLY : RV       ,RCPV     ,RETV     ,RCW      ,&
 & RCS      ,RLVTT    ,RLSTT    ,RTT      ,RALPW    ,&
 & RBETW    ,RGAMW    ,RALPS    ,RBETS    ,RGAMS    ,&
 & RALPD    ,RBETD    ,RGAMD    ,RD       ,RG    ,RKAPPA  

USE YOETHF   , ONLY : R2ES     ,R3LES    ,R3IES    ,R4LES    ,&
 & R4IES    ,R5LES    ,R5IES    ,R5ALVCP  ,R5ALSCP  ,&
 & RALVDCP  ,RALSDCP  ,RTWAT    ,RTICE  ,RTICECU    ,&
 & RTWAT_RTICE_R      ,RTWAT_RTICECU_R  
USE YOMCT0B  , ONLY : LECMWF
USE YOMGEM   , ONLY : VBH      ,VAH
USE YOMJG    , ONLY : LRDQERR  ,LJB_NONLINEAR_CVHUM,NOFCEF, &
                    & N_SPJB_VARS, SPJB_VARS_INFO
USE YOMFCEB  , ONLY : FCEBUF
USE SURFACE_FIELDS_MIX, ONLY : SD_VF, YSD_VF
USE YOMGRB   , ONLY : NGRBQ

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KNUMB 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQT0(NPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPT0(NPROMA) 
REAL(KIND=JPRB),TARGET  :: ZFCE   (NPROMA,NOFCEF)
REAL(KIND=JPRB),POINTER :: ZFCEQ  (:,:)

INTEGER(KIND=JPIM) :: ILTROP(NPROMA)
INTEGER(KIND=JPIM) :: ILEVM2

REAL(KIND=JPRB) :: ZPRSHO(NPROMA,0:NFLEVG)
REAL(KIND=JPRB) :: ZPRSFO(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZSTAB

INTEGER(KIND=JPIM) :: IEND, IENDC, IST, ISTC, JLEV, JROF, &
                    & JFIELD, IOFF, IBLK

REAL(KIND=JPRB) :: ZA, ZALSM, ZB, ZC, ZEPESRP, ZP0, ZQT0, ZRHERR, ZRHFG
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcttrm.h"
#include "fcttre.h"

#include "abor1.intfb.h"
#include "gppre.intfb.h"
#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"

IF (LHOOK) CALL DR_HOOK('SUSHFCE',0,ZHOOK_HANDLE)
IST   = 1
IEND  = KNUMB
ISTC  = 1
IENDC = KNUMB

IBLK = (KSTGLO-1)/NPROMA+1

!     ------------------------------------------------------------------

!*    0.1   SET-UP
!           ------


!*    0.3   READ BACKGROUND ERRORS
!           ----------------------

CALL SC2RDG (FCEBUF,KSTGLO,ISTC,IENDC,ZFCE)

NULLIFY(ZFCEQ)

DO JFIELD=1,N_SPJB_VARS
  IF (SPJB_VARS_INFO(JFIELD)%IGRIBCODE == NGRBQ) THEN
    IOFF = SPJB_VARS_INFO(JFIELD)%IPTFCE-1    
    ZFCEQ   => ZFCE(:,IOFF+1:IOFF+NFLEVG)
    EXIT
  ENDIF
ENDDO

IF (.NOT.ASSOCIATED(ZFCEQ)) THEN
  CALL ABOR1('FAILED TO ASSOCIATE ZFCEQ')
ENDIF

!     ------------------------------------------------------------------

!*    1.    FIND SURFACE PRESSURE
!           ---------------------

DO JROF=IST,IEND
  ZPRSHO(JROF,NFLEVG)= EXP(PSPT0(JROF))
ENDDO

!     ------------------------------------------------------------------

!*    2.    COMPUTE PRESSURES OF ETA LEVELS
!           -------------------------------

CALL GPPRE(NPROMA,IST,IEND,NFLEVG,VAH,VBH,ZPRSHO,ZPRSFO)

!*    3.    COMPUTE THE BACKGROUND RELATIVE HUMIDITY
!*          AND ASSIGN RELATIVE HUMIDITY ERROR
!*          COMPUTE THE SPECIFIC HUMIDITY ERROR
!           -----------------------------------

IF(LJB_NONLINEAR_CVHUM) THEN
  ZFCEQ(:,:)=1.0_JPRB
ELSE
  IF(LECMWF) THEN
    DO JLEV=1,NFLEVG
      DO JROF=IST,IEND
        ZQT0 = MAX(0.0_JPRB, PQT0(JROF,JLEV))
        ZRHFG=(ZPRSFO(JROF,JLEV)*ZQT0)&
         & /((1.0_JPRB+RETV*ZQT0)*FOEEWM(PTT0(JROF,JLEV)))  
        ZRHERR=-0.002_JPRB*PTT0(JROF,JLEV)&
         & -0.0033_JPRB*ABS(PTT0(JROF,JLEV)-273._JPRB)&
         & +0.25_JPRB*ZRHFG - 0.35_JPRB*ABS(ZRHFG-0.4_JPRB) + 0.70_JPRB  
        ZRHERR=MIN(0.18_JPRB,MAX(0.06_JPRB,ZRHERR))
        ZEPESRP=FOEEWM(PTT0(JROF,JLEV))/ZPRSFO(JROF,JLEV)
        ZFCEQ(JROF,JLEV) = ZRHERR*ZEPESRP  / ( 1.0_JPRB - ZRHFG*RETV*ZEPESRP )**2
      ENDDO
    ENDDO
  ELSE
    DO JLEV=1,NFLEVG
      DO JROF=IST,IEND
        ZQT0 = MAX(0.0_JPRB, PQT0(JROF,JLEV))
        ZRHFG=(ZPRSFO(JROF,JLEV)*ZQT0*(RETV+1.0_JPRB))&
         & /((1.0_JPRB+RETV*ZQT0)*ES(PTT0(JROF,JLEV)))  
        ZRHERR=-0.002_JPRB*PTT0(JROF,JLEV)&
         & -0.0033_JPRB*ABS(PTT0(JROF,JLEV)-273._JPRB)&
         & +0.25_JPRB*ZRHFG - 0.35_JPRB*ABS(ZRHFG-0.4_JPRB) + 0.70_JPRB  
        ZRHERR=MIN(0.18_JPRB,MAX(0.06_JPRB,ZRHERR))
        ZFCEQ(JROF,JLEV) = ZRHERR*&
         & ES(PTT0(JROF,JLEV)) * (RETV+1.0_JPRB)*ZPRSFO(JROF,JLEV) /&
         & ( (RETV+1.0_JPRB)*ZPRSFO(JROF,JLEV) - ZRHFG*RETV*&
         & ES(PTT0(JROF,JLEV)) )**2  
      ENDDO
    ENDDO
  ENDIF

  !     LIMIT ERRORS IN Q ABOVE 850MB to 125% OF Q
  !     NOTE IF Q = 0 KG/KG or NEG THEN ZERO ERROR 
  !     IS RESET TO 2.5E-06
  
  IF (.NOT.(LRDQERR)) THEN
    DO JLEV=1,NFLEVG
      DO JROF=IST,IEND
        ZQT0 = MAX(0.0_JPRB, PQT0(JROF,JLEV))
        IF (ZPRSFO(JROF,JLEV) < 850.0E02_JPRB) THEN
          ZFCEQ(JROF,JLEV) = &
           & MIN(ZFCEQ(JROF,JLEV),1.25_JPRB*ZQT0)  
        ENDIF
        ZFCEQ(JROF,JLEV) = MAX(ZFCEQ(JROF,JLEV),2.5E-06_JPRB)
      ENDDO
    ENDDO
  ENDIF

  !     REDUCE THE ERRORS IN THE STRATOSPHERE TO TAKE INTO ACCOUNT
  !     TRAPPING OF WATER VAPOUR BY THE TROPOPAUSE.
  !     CHOSE A VERY LOW VALUE TO INHIBIT DRIFT

  !     USE LAPSE_RATE DEFINITION TO DETERMINE TROPOPAUSE
  
  ILEVM2=NFLEVG-2
  DO JROF=IST,IEND
    ILTROP(JROF)=ILEVM2
  ENDDO

  DO JLEV=1,ILEVM2
    DO JROF=IST,IEND
      IF (ZPRSFO(JROF,JLEV) >= 7.E3_JPRB .AND.&
         & ZPRSFO(JROF,JLEV) <= 5.E4_JPRB) THEN  
        ZSTAB=ZPRSFO(JROF,JLEV+1)*(PTT0(JROF,JLEV+2)-PTT0(JROF,JLEV))/&
         & (PTT0(JROF,JLEV+1)*(ZPRSFO(JROF,JLEV+2)-ZPRSFO(JROF,JLEV)))+&
         & RD*PTT0(JROF,JLEV+1)*2.5E-4_JPRB/(RG*RG)  
        IF (ILTROP(JROF) == ILEVM2 .AND. ZSTAB > RKAPPA) THEN
          ILTROP(JROF)=JLEV
        ENDIF
      ELSEIF (ZPRSFO(JROF,JLEV) > 5.E4_JPRB .AND. ILTROP(JROF) == ILEVM2)THEN
        ILTROP(JROF)=JLEV-1
      ENDIF
    ENDDO
  ENDDO

  DO JLEV=1,NFLEVG
    DO JROF=IST,IEND
      IF (JLEV <= ILTROP(JROF)) THEN
        ZFCEQ(JROF,JLEV) = 1.E-8_JPRB
      ENDIF
      ZFCEQ(JROF,JLEV) = MAX(ZFCEQ(JROF,JLEV),1.E-8_JPRB)
    ENDDO
  ENDDO

  !     REDUCE ERRORS AT LOW LEVELS OVER SEA

  ZA=0.5_JPRB
  ZB=125.E02_JPRB
  ZC=2.0_JPRB
  ZP0=800.E02_JPRB
  DO JLEV=1,NFLEVG
    DO JROF=IST,IEND
      IF(ZPRSFO(JROF,JLEV) > ZP0) THEN
        ZALSM=ZA*(1.0_JPRB-SD_VF(JROF,YSD_VF%YLSM%MP,IBLK))
        ZFCEQ(JROF,JLEV) =&
         & ZFCEQ(JROF,JLEV) *&
         & ( 1.0_JPRB-ZALSM + ZALSM*EXP( -( (ZPRSFO(JROF,JLEV)-ZP0)/ZB )**ZC ) )  
      ENDIF
    ENDDO
  ENDDO

ENDIF

!*       4.   WRITE ERRORS TO BACKGROUND ERROR BUFFER

CALL SC2WRG (FCEBUF,KSTGLO,ISTC,IENDC,ZFCE)

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUSHFCE',1,ZHOOK_HANDLE)
END SUBROUTINE SUSHFCE
