SUBROUTINE SUVAZX

!**** *SUVAZX*  - Initialize the control variable

!     Purpose.
!     --------
!           Initialize the control variable (first step of the descent).

!**   Interface.
!     ----------
!        *CALL* *SUVAZX

!     Method.
!     -------
!        We need to set up :
!       * all the model fields to define the trajectory, 
!       * and YVAZX as initial control variable for the minimizer.

!        We can reconstruct the model fields from a YVAZX file, or the
!        other way around, or read both.

!        There is no longer a simple correspondence between YVAZX (which may 
!        contain more than model fields) and the model fields (which may
!        require surface fields, clouds, etc, that are not in the control
!        variable).

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original   : 87-10-15
!        L. Isaksen : 95-06-06 - Reordering of spectral arrays
!        M. Hamrud  : 95-10-01 - Interface block for SUSPEC
!        L. Isaksen : 96-02-02 - Message passing option added
!        F. Rabier  : 96-09-25 - Read orography for L801TL
!        R. El Khatib : 98-04-02 - Cleanup ALADIN
!        E. Holm    : 97-11-28 - Ozone included
!        M.Hamrud   : 98-03-01 - Attempt at making understandable
!        E.Andersson: 98-07-06 - TOVS control variable
!        M. Torki   : 99-04-16 - Option to use Multi-incremental
!        A. Horanyi : 99-07-02 - Setting the mean wind to zero for ALADIN
!        F. Bouttier: 00-05-15 - Rewrite, cloud fields and use model fields
!        M. Fisher  : 01-02-07 - Combine GETMINC and GETMINI
!        Y.Tremolet   02-08-06 - Use control_vectors.
!        P.Smolikova: 02-09-30 - Interface to SUSPEC for d4 in NH
!        M.Janiskova: 03-05    - call for sugridu when LENCLD2
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!        Y.Tremolet    18-Mar-2004 Model error
!        G. Radnoti    08-Dec-2006 Model error, 4d-state-vector==> YVAZX%MODERR=0
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIM   , ONLY : NFD2D, NFLEVG, NPROMA
USE YOMSP    , ONLY : SPA3     ,SPA2 ,SPA1
USE YOMCT0   , ONLY : NCONF    ,LSIMOB   ,LELAM
USE YOMCT0B  , ONLY : LECMWF
USE YOMVAR   , ONLY : NUPTRA   ,LTOVSCV  ,LMODERR  ,LTSCV
USE YOMCVA   , ONLY : NVAPARAM, YVAZX, YVAZ0, NVA2D, NVA3D
USE YOMJG    , ONLY : LCTLFG
USE YOMGEM   , ONLY : NGPTOT
USE YOMLUN   , ONLY : NULOUT
USE YOMMP    , ONLY : NPSP
USE YOMIOMI  , ONLY : LREADSP
USE YOMTVRAD , ONLY : TOVSCVX
USE YOMMODEL_ERROR, ONLY: NDIM_MODERR, SPCTLMODERR, SPFGMODERR, GPFGMODERR, &
                     & SPGPMODERR, LFGMODERR, NTYPE_MODERR, LBGMODERR
USE YOMJQ    , ONLY : LQEQB, LJQBAL
USE SPECTRAL_FIELDS_MOD
USE GRIDPOINT_FIELDS_MIX
USE CONTROL_VECTORS
USE YOMTS    , ONLY : TSCVX

IMPLICIT NONE

TYPE(control_vector) :: YLCV
REAL(KIND=JPRB), ALLOCATABLE :: ZGP3(:,:,:), ZGP2(:,:,:)
LOGICAL :: llbal
INTEGER(KIND=JPIM) :: il,jj,ii,INGPBLKS
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "abor1.intfb.h"
#include "chavar.intfb.h"
#include "chavarin.intfb.h"
#include "getmini.intfb.h"
#include "grid2spec.intfb.h"
#include "spec2grid.intfb.h"
#include "cvar3.intfb.h"
#include "cvar3in.intfb.h"
#include "suinif.intfb.h"
#include "spec_concat.intfb.h"
#include "spec_split.intfb.h"

!      ----------------------------------------------------------------

!*       1.    Preinitializations.
!              -------------------

IF (LHOOK) CALL DR_HOOK('SUVAZX',0,ZHOOK_HANDLE)

! Spectral arrays are preset to zero.
IF(NPSP == 1) THEN       ! I have surface pressure on my processor
  SPA2(:,1:NFD2D)=0.0_JPRB
ELSE
  SPA2(:,:)=-99999._JPRB
ENDIF
SPA3(:,:,:)=0.0_JPRB
IF (LELAM) SPA1(:,:)=0.0_JPRB
IF (LTOVSCV) TOVSCVX(:,:)=0.0_JPRB
IF (LTSCV) TSCVX(:)=0.0_JPRB

! The control variable is preset to zero. (constant VARBC keeps YVAZX%PARAMS)
YVAZX=0.0_JPRB

!      ----------------------------------------------------------------
IF (.NOT.LECMWF .OR. (NCONF /= 131) ) THEN
! The non-analysed fields are read as in suinif, from ICMGGVAxxIMIN/IMIUA files.
! They usually contain the background, but they could be an analysis.
  WRITE(NULOUT,*) 'MODEL FIELDS ARE INITIALIZED BY SUVAZX'
  CALL SUINIF(1)
ENDIF

!*       2.    Setup of non-minimization jobs.

IF (NCONF /= 131) THEN

  WRITE(NULOUT,*) 'SUVAZX: call CHAVAR to fill YVAZX using spa3'
  CALL CHAVAR (YVAZX)

!      ----------------------------------------------------------------

!*       3.    Setup of cold minimization jobs.

ELSE
  IF (LSIMOB) CALL ABOR1('SUVAZX: LSIMOB no longer supported in conf 131.')

  IF (NUPTRA==0) THEN
    IF (LMODERR .AND. NTYPE_MODERR /= 1) THEN
      IF (LFGMODERR.AND..NOT.LBGMODERR) THEN
        IF (LJQBAL) write(NULOUT,*)'SUVAZX: Ignoring Jq balance operators'
        llbal=.false.
        INGPBLKS = (NGPTOT+NPROMA-1)/NPROMA
        IF (LQEQB) THEN
          ii=0
          il=SPCTLMODERR%nspsizel
          CALL ALLOCATE_CTLVEC(YLCV,LDINIT=.true.,KMERR=0)
          IF (SIZE(YLCV%INITCV)/=il) CALL ABOR1('SUVAZX: Error LQEQB 1')
          ALLOCATE(ZGP3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
          ALLOCATE(ZGP2(NPROMA,NVA2D,INGPBLKS))
        ELSE
          ALLOCATE (ZGP3(NPROMA,NFLEVG*NVA3D,INGPBLKS))
          ALLOCATE (ZGP2(NPROMA,NVA2D,INGPBLKS))
          ZGP3(:,:,:)=0.0_JPRB
          ZGP2(:,:,:)=0.0_JPRB 
        ENDIF

!       Set model error first guess from control vector.
!       To get zero perturbation in first TL run, need to convert 
!       control vector back to model error and over-write model error
!       first guess to avoid rounding error from cvar3+cvar3in. YT
        DO jj=1,NDIM_MODERR
          CALL GRID2SPEC(GPFGMODERR(jj),SPGPMODERR)
          CALL SPEC_CONCAT(SPFGMODERR(jj),SPGPMODERR,SPCTLMODERR)

          IF (LQEQB) THEN
            CALL CVAR3(SPCTLMODERR,YLCV,ZGP3,ZGP2,0,llbal)
            YVAZX%MODERR(ii+1:ii+il)=YLCV%INITCV(:)
            ii=ii+il
            SPCTLMODERR=0.0_JPRB
            CALL CVAR3IN(YVAZX,SPCTLMODERR,ZGP3,ZGP2,0,llbal)
          ELSE
            CALL CVAR3(SPCTLMODERR,YVAZX,ZGP3,ZGP2,jj,llbal)
            SPCTLMODERR=0.0_JPRB
            CALL CVAR3IN(YVAZX,SPCTLMODERR,ZGP3,ZGP2,jj,llbal)
          ENDIF

          WRITE(NULOUT,*)'SUVAZX: Set MODERR part of CTLVEC from FGMODERR'
          CALL SPECTRAL_NORM(SPCTLMODERR,'SUVAZX FGMODERR')
          CALL SPEC_SPLIT(SPCTLMODERR,SPFGMODERR(jj),SPGPMODERR)
          CALL SPEC2GRID(SPGPMODERR,GPFGMODERR(jj))
        ENDDO

        IF (LQEQB) THEN
          IF (SIZE(YVAZX%MODERR)/=ii) CALL ABOR1('SUVAZX: Error LQEQB 2')
          CALL DEALLOCATE_CTLVEC(YLCV)
        ENDIF
        DEALLOCATE(ZGP3)
        DEALLOCATE(ZGP2)
      ELSE
        WRITE(NULOUT,*)'SUVAZX: Set MODERR part of CTLVEC to ZERO'
      ENDIF
    ENDIF
    CALL FLUSH(NULOUT)

!      ----------------------------------------------------------------

!*       4.    Setup of warm minimization jobs.

  ELSE

    WRITE(NULOUT,*) 'SUVAZX: set up YVAZX from binary files'
    CALL GETMINI(YVAZX)
    IF (.NOT.LREADSP) THEN
      WRITE(NULOUT,*)'SUVAZX: set up spa3 from YVAZX using CHAVARIN'
      CALL CHAVARIN(YVAZX)
    ENDIF
  ENDIF

! Parameter increments initially set to zero unless constant VARBC is used

  IF (NVAPARAM>0) THEN
    CALL GSTATS(1936,0)
    YVAZX%PARAMS(:) = 0.0_JPRB
    WRITE(NULOUT,*) 'SUVAZX: Parameter section of cv set to zero'
    CALL GSTATS(1936,1)
  ENDIF

  YVAZ0=YVAZX
  IF (LCTLFG) YVAZX=0.0_JPRB 
  IF (LMODERR) YVAZX%moderr(:)=0.0_JPRB 
ENDIF

IF (LHOOK) CALL DR_HOOK('SUVAZX',1,ZHOOK_HANDLE)
END SUBROUTINE SUVAZX
