SUBROUTINE SUVIFCE(PVALIN,PRSIN,KDIM1,KLONS,KLEVSIN,PVALOUT,PRSOUT,KLEVSOUT)

!**** *SUVIFCE*  - ROUTINE FOR VERTICAL INTERPOLATION OF FORECAST
!****              ERRORS FROM THE STANDARD ERROR LEVELS.

!     PURPOSE.
!     --------

!        TO PRODUCE FIELDS OF FORECAST ERRORS ON THE MODEL GRID.

!**   INTERFACE.
!     ----------

!        *CALL* *SUVIFCE(PVALIN,PRSIN,KDIM1,KLONS,KLEVSIN,
!    +                   PVALOUT,PRSOUT,KLEVSOUT)

!        INPUT:
!           *PVALIN*   - FORECAST ERRORS ON ERROR LEVELS
!           *PRSIN*    - PRESSURE ON ERROR LEVELS
!           *KDIM1*    - FIRST DIMENSION OF ARRAYS
!           *KLONS*    - NUMBER OF POINTS IN THE HORIZONTAL
!           *KLEVSIN*  - NUMBER OF ERROR LEVELS
!           *PRSOUT*   - PRESSURE OF OUTPUT LEVELS
!           *KLEVSOUT* - NUMBER OF OUTPUT LEVELS

!        OUTPUT:
!           *PVALOUT*  - FORECAST ERROR ON OUTPUT LEVELS

!     METHOD.
!     -------

!        LINEAR INTERPOLATION.

!     EXTERNALS.
!     ----------

!        *PPINIT*   - SET UP HELP ARRAYS FOR VERTICAL INTERPOLATION
!        *PPFLEV*   - LOCATES OUTPUT LEVELS WRT INPUT LEVELS
!        *PPINTP*   - LINEAR INTERPOLATION

!     REFERENCE.
!     ----------

!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------

!        W. HECKLEY    *ECMWF*         19:12:89

!     MODIFICATIONS.
!     --------------
!        M.Hamrud      01-Oct-2003 CY28 Cleaning

!        Modified : 91-10-15 Bill Heckley and Mats Hamrud ECMWF
!                            Use of MIOLIB for IO on forecast error files
!        Bill Heckley 14:01:92 - change in intermediate file format NFCERH
!                                to agree with new subroutine *SUGRBFE*.
!        Bill Heckley  4:02:92 - Change record order on MIO file and add
!                                provision for both height and temperature
!                                (both filled with height for the moment).
!        Mike Fisher 96/04/19  - Code specific to forecast errors moved
!                                out into SUINFCE.

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMDIM   , ONLY : NPPM

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KDIM1 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVSIN 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLEVSOUT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVALIN(KDIM1,0:KLEVSIN) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSIN(KDIM1,KLEVSIN) 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLONS 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PVALOUT(KDIM1,KLEVSOUT) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSOUT(KDIM1,KLEVSOUT) 
REAL(KIND=JPRB) :: ZPXP(KDIM1,0:KLEVSIN,NPPM), ZPXPD(KDIM1,0:KLEVSIN,NPPM)
REAL(KIND=JPRB) :: ZPRSHI(KDIM1,0:KLEVSIN)

INTEGER(KIND=JPIM) :: ILEVB(KDIM1,KLEVSOUT,NPPM)

LOGICAL :: LLBELO(KDIM1,KLEVSOUT),   LLBLOW(KLEVSOUT)
LOGICAL :: LLBELS(KDIM1,KLEVSOUT),   LLBLES(KLEVSOUT)

INTEGER(KIND=JPIM) :: ISLCT, JLEV, JLON
REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "ppflev.intfb.h"
#include "ppinit.intfb.h"
#include "ppintp.intfb.h"

!*    1.    SET UP HELP ARRAYS FOR VERTICAL INTERPOLATION
!           ---------------------------------------------

IF (LHOOK) CALL DR_HOOK('SUVIFCE',0,ZHOOK_HANDLE)
DO JLEV=1,KLEVSIN-1
  DO JLON=1,KLONS
    ZPRSHI(JLON,JLEV) = 0.5_JPRB*(PRSIN(JLON,JLEV)+PRSIN(JLON,JLEV+1))
  ENDDO
ENDDO

DO JLON=1,KLONS
  ZPRSHI(JLON,0)       = 0.9_JPRB*PRSIN(JLON,1)
  ZPRSHI(JLON,KLEVSIN) = 1.1_JPRB*PRSIN(JLON,KLEVSIN)
ENDDO

CALL PPINIT(KDIM1,1,KLONS,KLEVSIN,ZPRSHI,PRSIN,ZPXP,ZPXPD)

!     ------------------------------------------------------------------

!*    2.    FIND PRESSURE LEVEL UNDER SPECIFIC PRESSURES
!           --------------------------------------------

CALL PPFLEV(KDIM1,1,KLONS,KLEVSIN,KLEVSOUT,PRSOUT,ZPRSHI,&
 & PRSIN,ILEVB,LLBELO,LLBELS,LLBLOW,LLBLES)  

!     ------------------------------------------------------------------

!*    6.    CARRY OUT THE VERTICAL INTERPOLATION
!           ------------------------------------

ISLCT = 2
CALL PPINTP(KDIM1,1,KLONS,KLEVSIN,KLEVSOUT,1,ILEVB,ISLCT,&
 & LLBELO,LLBLOW,PRSOUT,ZPXP,ZPXPD,PVALIN,PVALOUT)  

DO JLEV=1,KLEVSOUT
  IF(LLBLOW(JLEV)) THEN
    DO JLON=1,KLONS
      IF (LLBELO(JLON,JLEV)) THEN
        PVALOUT(JLON,JLEV) = PVALIN(JLON,KLEVSIN)
      ENDIF
    ENDDO
  ENDIF
ENDDO

IF (LHOOK) CALL DR_HOOK('SUVIFCE',1,ZHOOK_HANDLE)
END SUBROUTINE SUVIFCE
