!OPTION! -pvctl noouterunroll -pvctl noouterstrip
SUBROUTINE TASKOB(CDOBJOB)
!****-------------------------------------------------------------------
!**** *TASKOB* - Task treating the first set of non correlated obs
!**** which is available and repeating the action until no set left.
!****
!***  Purpose : Control the execution of one task treating one set of
!***  -------   observations which are not correlated in the horizontal.
!***
!**   Interface : *CALL* * TASKOB(CDOBJOB)*
!**   ---------   OR *CALL TSKSTART(.,TASKOB,CDOBJOB)
!**   ---------   DEPENDING ON MULTITASKING IMPLEMENTATION
!**         CDOBJOB: switch to select direct/TL/AD Jo computations
!**         The common YOMOBSET is an implicit input of TASKOB

!     EXTERNALS.  SURFACEO - JO FOR SURFACE OBS
!     ----------  SURFACAD - JO FOR SURFACE OBS - ADJOINT
!                 UPPERAIR - JO FOR UPPER AIR OBS
!                 UPPERAAD - JO FOR UPPER AIR OBS - ADJOINT
!                 SATEM    - JO FOR SATEM
!                 SATEMAD  - JO FOR SATEM - ADJOINT
!                 TOVCLR   - JO FOR TOVS CLOUD CLEARED RADIANCES
!                 TOVCLRAD - JO FOR TOVS RADIANCES - ADJOINT
!                 HRETR    - RETRIEVAL, IF SCREENING
!                 HOP      - OBSERVATION OPERATOR, ALL TYPES
!                 AEOLUS_L2BP_UNSETUP_EC - UN-SET-UP ROUTINE FOR AEOLUS

!     Author: Jean Pailleux
!     Date  : 90-02-05

!     MODIFICATIONS
!     -------------
!     Erik Andersson:  92-07-01 MAKE TOVS SETS LOOK LIKE ANY OTHER SETS
!     Erik Andersson:  94-03-28 Make Jo accumulation reproducible
!     Mike Fisher   :  95-01-04 Calls to SURFACAD, UPPERAAD, etc.
!     F. Bouttier   :  95-12-28 Introduce CDOBJOB control to manage TL/AD
!     Erik Andersson:  96-01-08 Prepare for RTOVS
!     Erik Andersson:  99-04-01 Merging obs-operators into one routine
!     M.Hamrud      :  01-Oct-2003 CY28 Cleaning
!     Paul Burton   :  10-Sep-2004 Reproducible sat. bias parameters
!     Sean Healy    :  14-03-2005 Changes for 2d operators
!     Gabor Radnoti :  21-02-2006 Double scan for correlated obs error model
!     D. Salmond    :  31-Aug-2007 Array syntax expressions must be conformant
!     S. Saarinen   :  26-Sep-2007 Obs. group based Dr.Hook'ing in KSET-loop
!     S. Saarinen   :  16-Nov-2007 Added ZTIMER to dynamically load balance the KSET-loop
!     D. Tan        :  05-Mar-2008 New OMP loop for Aeolus Level-2B processing
!     D. Tan        :  09-Sep-2008 Unsetup after Aeolus Level-2B processing
!     D. Dee        10-Oct-2008 generalise VarBC
!     D. Li         04-Dec-2008 VarBC histogram calc fix.
!     E. Sevault 18-May-2009 : SX9 compiler workaround : optimisations
!            "outerunroll" and "outerstrip" manually switched off to prevent
!            unexpected numerical variations. Note that this workaround may
!            be irrelevent when the source code of this subroutine is deeply 
!            re-written.
!     G. Mozdzynski  03-Jun-2009 VarBC reproducibility rework
!     A. Geer        07-Sep-2009 VarBC predictor covariances moved to varbc_pred
!     T. Wilhelmsson 26-Feb-2010 More VarBC reproducibility rework

!****-------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMDIMO   ,ONLY : L_TASKOB_USE_SORTED_TIMES, NOBTOT
USE YOMCOCTP  ,ONLY : NPILOT
USE YOMCT0   , ONLY : LSCREEN  ,LCANARI
USE YOMLUN    ,ONLY : NULOUT, NULERR
USE YOMOBSET  ,ONLY : MTYPOB   ,MLNSET   ,MMXBDY   ,NSETOT
USE OML_MOD   ,ONLY : OML_INIT, OML_INIT_LOCK, OML_MAX_THREADS
USE VARBC_SETUP, ONLY : YVARBC, &
                    & NOBGROUP, JPMXNPARAM, NHSTBINS, &
                    & APARAMS_COMP, NPARAMS_COMP, NHSTFGDEP_COMP
USE YOMVAR   , ONLY : LVARBC, LCO2ANER, LAEOLUSL2BP, LREPRO4DVAR
USE YOMECTAB  ,ONLY : NOBSPROFS
USE MPL_MODULE
USE YOMCOSJO  ,ONLY : LOBSCOR, OBSCOR_AUX, NBCORBLK
USE YOMTVRAD  ,ONLY : SATGRP_TABLE  ,CHSENSOR, MXSENSOR, &
                    & SATOBGRP_TABLE,CHMETHOD, MXMETHOD
USE ECSORT_MIX,ONLY : KEYSORT
USE YOMMP     ,ONLY : MYPROC
IMPLICIT NONE

CHARACTER(LEN=2)  ,INTENT(IN)    :: CDOBJOB 
LOGICAL :: LLOBDI,LLOBTL,LLOBAD

INTEGER(KIND=JPIM) :: ICSET,ITYPOB,ILEN,IDLEN,IMXBDY,JSET,JBGRP,IHORIZ
INTEGER(KIND=JPIM) :: IGRP,ISENSOR,IMETHOD,ICODETYPE,ISATID
INTEGER(KIND=JPIM) :: J,JBIN,ITHRMAX,IRET
INTEGER(KIND=JPIM) :: ISCAN
INTEGER(KIND=JPIM) :: I_ERROR_FLAG
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL(KIND=JPRB) :: ZHOOK_KSET_LOOP
REAL(KIND=JPRB) :: ZHOOK_JSET
REAL(KIND=JPRB) :: TIMEF, ZT
REAL(KIND=JPRB), ALLOCATABLE, SAVE :: ZTIMER(:,:) ! Note : never released, but a small array
CHARACTER(LEN=40) :: CLOBSGRP

#include "abor1.intfb.h"
#include "hop.intfb.h"
#include "hopad.intfb.h"
#include "hoptl.intfb.h"
#include "hretr.intfb.h"
#include "obscor_sumup_scalp.intfb.h"
!****-------------------------------------------------------------------
!        0.        Initialize job configuration
!                  ----------------------------
IF (LHOOK) CALL DR_HOOK('TASKOB',0,ZHOOK_HANDLE)
ITHRMAX=OML_MAX_THREADS()
LLOBDI = .FALSE.
LLOBTL = .FALSE.
LLOBAD = .FALSE.
IF ((CDOBJOB == 'DI').OR.(CDOBJOB == 'DA')) LLOBDI = .TRUE.
IF  (CDOBJOB == 'TA')                       LLOBTL = .TRUE.
IF ((CDOBJOB == 'DA').OR.(CDOBJOB == 'TA')) LLOBAD = .TRUE.
IF(.NOT.(LLOBDI.OR.LLOBTL.OR.LLOBAD)) THEN
  WRITE(NULERR,*) 'ERROR TASKOB : UNKNOWN JO CONF. ',CDOBJOB
  CALL ABOR1('ERROR TASKOB : UNKNOWN JO CONFIGURATION')
ENDIF

!****-------------------------------------------------------------------

! Allocate space to the component parts of the satellite bias
! parameters (PARAMS_COMP) which will be summed in a 
! reproducible manner after the JSET loop

IF (LVARBC .AND. LLOBAD) THEN
  IF (.NOT.LREPRO4DVAR) THEN
    IF (NSETOT > 0) ALLOCATE(APARAMS_COMP(NSETOT,JPMXNPARAM,NOBGROUP))
  ELSE
    ALLOCATE(APARAMS_COMP(NOBTOT,JPMXNPARAM,NOBGROUP))
    ALLOCATE(NPARAMS_COMP(JPMXNPARAM,NOBGROUP))

    NPARAMS_COMP(:,:)=0
  ENDIF
ENDIF
IF (LVARBC) THEN
  ALLOCATE(NHSTFGDEP_COMP(NOBGROUP,NHSTBINS,ITHRMAX))
  CALL GSTATS(1068,0)
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(J)
  DO J=1,ITHRMAX
    NHSTFGDEP_COMP(:,:,J)=0
  ENDDO
!$OMP END PARALLEL DO
  CALL GSTATS(1068,1)
ENDIF

!*       1.       CALL THE AD HOC OBS ROUTINE, DEPENDING ON OBS TYPE
!                 --------------------------------------------------

!   GET NEXT OBSERVATION SET NUMBER (LOOP START)
!   COUNTING BACKWARDS IN ORDER TO DO TOVS FIRST

IF (allocated(ZTIMER)) THEN
  IF (ITHRMAX > 1 .AND. L_TASKOB_USE_SORTED_TIMES) THEN
    CALL KEYSORT(IRET, ZTIMER, NSETOT, KEY=2) ! "2" := smallest time at the top
    IF(MYPROC==1)THEN
      DO JSET=1,NSETOT
        ICSET=ZTIMER(JSET,1)
        WRITE(NULOUT,'("TASKOB  : SET=",I6," TIME=",F10.3)')ICSET,ZTIMER(JSET,2)
      ENDDO
    ENDIF
  ENDIF
ELSE
!$ CALL OML_INIT_LOCK()
  ZT = TIMEF() ! Make sure the timer has been initialized outside the OpenMP-region
  ALLOCATE(ZTIMER(NSETOT,2))
  DO JSET=1,NSETOT
    ZTIMER(JSET,1) = JSET
    ZTIMER(JSET,2) = 0.0_JPRB
  ENDDO
ENDIF

CALL GSTATS(1060,0)
IF (LOBSCOR .AND. LLOBTL) THEN
  ISCAN=2
  ALLOCATE(OBSCOR_AUX(NSETOT,NBCORBLK))
ELSE
  ISCAN=1
ENDIF

! OMP loop for Aeolus Level-2B processing
!$OMP PARALLEL &
!$OMP&PRIVATE(JSET,ICSET,ITYPOB,ILEN,IDLEN,IMXBDY,IHORIZ,&
!$OMP&        ZHOOK_KSET_LOOP,IGRP)
!$OMP DO SCHEDULE(DYNAMIC,1)
AEOLUS_KSET_LOOP: DO JSET=NSETOT,1,-1
  IF (LHOOK) CALL DR_HOOK('TASKOB>AEOLUS_KSET_LOOP',0,ZHOOK_KSET_LOOP)
  ICSET = JSET

!*       1.1a  OBSERVATION TYPE, AND FIRST DIMENSION (SHOULD BE ODD)


!        1.1a.0 Check for Aeolus observations
  ITYPOB = MOD(MTYPOB(ICSET),100)
  IGRP   = MOD(MTYPOB(ICSET)/100,100) ! Consider MTYPOB(ICSET)/100
  ! LL_AEOLUS_SET = ( ITYPOB == NPILOT ) .AND. ( IGRP == 2 )
  !IF ( .NOT. LL_AEOLUS_SET ) THEN
  IF ( .NOT. ( ( ITYPOB == NPILOT ) .AND. ( IGRP == 2 ) ) ) THEN
    IF (LHOOK) CALL DR_HOOK('TASKOB>AEOLUS_KSET_LOOP',1,ZHOOK_KSET_LOOP)
    CYCLE AEOLUS_KSET_LOOP
  ENDIF

!        1.1a.1 Set string CLOBSGRP for Dr_Hook
  ! Redundant: WRITE(CLOBSGRP,*) 'TASKOB>OBSGRP=Aeolus'
  ! Redundant: IF (LHOOK) CALL DR_HOOK(CLOBSGRP,0,ZHOOK_JSET)

!        1.1a.2 Set arguments needed by HRETR
  ILEN   = MLNSET(ICSET)
  IF(MOD(ILEN,2) == 0) THEN
    IDLEN  =ILEN+1
  ELSE
    IDLEN  =ILEN
  ENDIF
!            SECOND DIMENSION, COULD BE ZERO.
  IMXBDY = MAX(MMXBDY(ICSET),1)

! How many vertical profiles are required per observation

  IHORIZ = NOBSPROFS(ITYPOB) 

!*       1.1a.3      CALL THE APPROPRIATE OBS OPERATOR

  IF(LSCREEN .OR. LCANARI) THEN
    !IF (LVARBC) THEN
    !   VarBC code not needed
    !ENDIF
    CALL HRETR(IDLEN,IMXBDY,ICSET,IHORIZ,LDFLAG_DO_AEOLUS_L2BP=.true.)
  ENDIF
  !IF (LVARBC) THEN
  !   VarBC code not needed
  !ENDIF
  !IF(MMXBDY(ICSET) /= 0) THEN
  !   HOP/HOPTL/HOPAD calls not needed
  !ENDIF
  !
  ! Redundant: IF (LHOOK) CALL DR_HOOK(CLOBSGRP,1,ZHOOK_JSET)
  IF (LHOOK) CALL DR_HOOK('TASKOB>AEOLUS_KSET_LOOP',1,ZHOOK_KSET_LOOP)
ENDDO AEOLUS_KSET_LOOP
!$OMP END DO
!$OMP END PARALLEL

! Delete Aeolus datastructures after OMP loop for Aeolus Level-2B processing
IF (LAEOLUSL2BP) THEN
  CALL AEOLUS_L2BP_UNSETUP_EC(I_ERROR_FLAG)
  IF (I_ERROR_FLAG /= 0) THEN
    WRITE(NULERR,*) 'aeolus_l2bp_unsetup_ec returned an error',I_ERROR_FLAG
  ELSE
    WRITE(NULERR,*) 'aeolus_l2bp_unsetup_ec completed OK'
  ENDIF
ENDIF

1000 FORMAT(A,A,":",I5.5,".",I2.2,".",I3.3,".",I2.2)

!$OMP PARALLEL DO ORDERED SCHEDULE(DYNAMIC,1) &
!$OMP&PRIVATE(JSET,ICSET,ITYPOB,ILEN,IDLEN,IMXBDY,IHORIZ,JBGRP,ZT,&
!$OMP&        ZHOOK_KSET_LOOP,ZHOOK_JSET,CLOBSGRP,IGRP,ISENSOR,IMETHOD,ICODETYPE,ISATID)
DO JSET=NSETOT,1,-1
  ZT = TIMEF()
  IF (LHOOK) CALL DR_HOOK('TASKOB>KSET_LOOP',0,ZHOOK_KSET_LOOP)
  ICSET = ZTIMER(JSET,1)

!*       1.1  OBSERVATION TYPE, AND FIRST DIMENSION (SHOULD BE ODD)

  ITYPOB = MOD(MTYPOB(ICSET),100)

  IF (ITYPOB == 7) THEN ! ATOVS
    IGRP = MTYPOB(ICSET)/100
    ISENSOR = SATGRP_TABLE(IGRP)%SENSOR
    ICODETYPE = SATGRP_TABLE(IGRP)%CODETYPE
    ISATID = SATGRP_TABLE(IGRP)%BUFRID
    IF (ISENSOR >= 0 .AND. ISENSOR <= MXSENSOR) THEN
      WRITE(CLOBSGRP,1000) &
           & 'TASKOB>OBSGRP=',TRIM(CHSENSOR(ISENSOR)),ISATID,ISENSOR,ICODETYPE,ITYPOB
    ELSE
      WRITE(CLOBSGRP,1000) &
           & 'TASKOB>OBSGRP=','<UNDEF>',ISATID,ISENSOR,ICODETYPE,ITYPOB
    ENDIF
  ELSEIF (ITYPOB == 3) THEN ! SATOB
    IGRP = MTYPOB(ICSET)/100
    IMETHOD = SATOBGRP_TABLE(IGRP)%METHOD
    ICODETYPE = SATOBGRP_TABLE(IGRP)%CODETYPE
    ISATID = SATOBGRP_TABLE(IGRP)%BUFRID
    IF (IMETHOD >= 0 .AND. IMETHOD <= MXMETHOD) THEN
      WRITE(CLOBSGRP,1000) &
           & 'TASKOB>OBSGRP=',TRIM(CHMETHOD(IMETHOD)),ISATID,IMETHOD,ICODETYPE,ITYPOB
    ELSE
      WRITE(CLOBSGRP,1000) &
           & 'TASKOB>OBSGRP=','<UNDEF>',ISATID,IMETHOD,ICODETYPE,ITYPOB
    ENDIF
  ELSE ! ANY OTHER OBS.TYPE
    WRITE(CLOBSGRP,'(A,I2.2)') 'TASKOB>OBSGRP=',ITYPOB
  ENDIF
  IF (LHOOK) CALL DR_HOOK(CLOBSGRP,0,ZHOOK_JSET)

  ILEN   = MLNSET(ICSET)
  IF(MOD(ILEN,2) == 0) THEN
    IDLEN  =ILEN+1
  ELSE
    IDLEN  =ILEN
  ENDIF
!            SECOND DIMENSION, COULD BE ZERO.
  IMXBDY = MAX(MMXBDY(ICSET),1)

! How many vertical profiles are required per observation

  IHORIZ = NOBSPROFS(ITYPOB) 

!*       1.3      CALL THE APPROPRIATE OBS OPERATOR

  IF(LSCREEN .OR. LCANARI) THEN
    CALL HRETR(IDLEN,IMXBDY,ICSET,IHORIZ)
  ENDIF
  IF (LVARBC) THEN
    IF (LLOBAD) THEN
      APARAMS_COMP(ICSET,:,:)=0.0_JPRB
    ENDIF
  ENDIF
  IF(MMXBDY(ICSET) /= 0) THEN
    IF(LLOBDI) CALL HOP  (IDLEN,IMXBDY,ICSET,IHORIZ)
    IF(LLOBTL) CALL HOPTL(IDLEN,IMXBDY,ICSET,IHORIZ,1)
    IF(LLOBAD .AND. ISCAN/=2) CALL HOPAD(IDLEN,IMXBDY,ICSET,IHORIZ)
  ENDIF
  IF (LHOOK) CALL DR_HOOK(CLOBSGRP,1,ZHOOK_JSET)
  IF (LHOOK) CALL DR_HOOK('TASKOB>KSET_LOOP',1,ZHOOK_KSET_LOOP)
  IF (ITHRMAX > 1 .AND. L_TASKOB_USE_SORTED_TIMES) THEN
    ZTIMER(JSET,2) = TIMEF() - ZT
  ENDIF
ENDDO
!$OMP END PARALLEL DO
IF (ISCAN==2) THEN
!   OBSCOR_AUX  ------->  OBSCOR_SCALPR ! (DM-LOCAL, POINT-WISE PRODUCTS--> INNER PRODUCT)
    CALL OBSCOR_SUMUP_SCALP
    DEALLOCATE(OBSCOR_AUX)
! second scan...

!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1) &
!$OMP&PRIVATE(JSET,ICSET,ITYPOB,ILEN,IDLEN,IMXBDY,IHORIZ,JBGRP,&
!$OMP&        ZHOOK_KSET_LOOP,ZHOOK_JSET,CLOBSGRP,IGRP,ISENSOR,IMETHOD,ICODETYPE,ISATID)
  DO JSET=NSETOT,1,-1
    IF (LHOOK) CALL DR_HOOK('TASKOB>KSET_LOOP_2',0,ZHOOK_KSET_LOOP)
    ICSET = JSET

!*       1.1/2  OBSERVATION TYPE, AND FIRST DIMENSION (SHOULD BE ODD)

    ITYPOB = MOD(MTYPOB(ICSET),100)

    IF (ITYPOB == 7) THEN ! ATOVS
      IGRP = MTYPOB(ICSET)/100
      ISENSOR = SATGRP_TABLE(IGRP)%SENSOR
      ICODETYPE = SATGRP_TABLE(IGRP)%CODETYPE
      ISATID = SATGRP_TABLE(IGRP)%BUFRID
      IF (ISENSOR >= 0 .AND. ISENSOR <= MXSENSOR) THEN
        WRITE(CLOBSGRP,1000) &
             & 'TASKOB>OBSG_2=',TRIM(CHSENSOR(ISENSOR)),ISATID,ISENSOR,ICODETYPE,ITYPOB
      ELSE
        WRITE(CLOBSGRP,1000) &
             & 'TASKOB>OBSG_2=','<UNDEF>',ISATID,ISENSOR,ICODETYPE,ITYPOB
      ENDIF
    ELSEIF (ITYPOB == 3) THEN ! SATOB
      IGRP = MTYPOB(ICSET)/100
      IMETHOD = SATOBGRP_TABLE(IGRP)%METHOD
      ICODETYPE = SATOBGRP_TABLE(IGRP)%CODETYPE
      ISATID = SATOBGRP_TABLE(IGRP)%BUFRID
      IF (IMETHOD >= 0 .AND. IMETHOD <= MXMETHOD) THEN
        WRITE(CLOBSGRP,1000) &
             & 'TASKOB>OBSG_2=',TRIM(CHMETHOD(IMETHOD)),ISATID,IMETHOD,ICODETYPE,ITYPOB
      ELSE
        WRITE(CLOBSGRP,1000) &
             & 'TASKOB>OBSG_2=','<UNDEF>',ISATID,IMETHOD,ICODETYPE,ITYPOB
      ENDIF
    ELSE ! ANY OTHER OBS.TYPE
      WRITE(CLOBSGRP,'(A,I2.2)') 'TASKOB>OBSG_2=',ITYPOB
    ENDIF
    IF (LHOOK) CALL DR_HOOK(CLOBSGRP,0,ZHOOK_JSET)

    ILEN   = MLNSET(ICSET)
    IF(MOD(ILEN,2) == 0) THEN
      IDLEN  =ILEN+1
    ELSE
      IDLEN  =ILEN
    ENDIF
!              SECOND DIMENSION, COULD BE ZERO.
    IMXBDY = MAX(MMXBDY(ICSET),1)

! How many vertical profiles are required per observation

    IHORIZ = NOBSPROFS(ITYPOB)

!*       1.3/2      CALL THE APPROPRIATE OBS OPERATOR

    IF(MMXBDY(ICSET) /= 0) THEN
      CALL HOPTL(IDLEN,IMXBDY,ICSET,IHORIZ,2)
      IF(LLOBAD) CALL HOPAD(IDLEN,IMXBDY,ICSET,IHORIZ)

    ENDIF
    IF (LHOOK) CALL DR_HOOK(CLOBSGRP,1,ZHOOK_JSET)
    IF (LHOOK) CALL DR_HOOK('TASKOB>KSET_LOOP_2',1,ZHOOK_KSET_LOOP)
  ENDDO
!$OMP END PARALLEL DO
ENDIF ! iscan ==2
LCO2ANER=.FALSE.

CALL GSTATS(1060,1)
CALL GSTATS_BARRIER(754)

IF (LVARBC) THEN

  IF (LSCREEN) THEN
!$OMP PARALLEL DO SCHEDULE(DYNAMIC,1)&
!$OMP&PRIVATE(JSET,JBIN,JBGRP) 
    DO JBIN=1,NHSTBINS
      DO JSET=1,ITHRMAX
        DO JBGRP=1,NOBGROUP
          YVARBC(JBGRP)%NHSTFGDEP(JBIN)= &
               & YVARBC(JBGRP)%NHSTFGDEP(JBIN)+NHSTFGDEP_COMP(JBGRP,JBIN,JSET)
        ENDDO
      ENDDO
    ENDDO
!$OMP END PARALLEL DO
  ENDIF

  DEALLOCATE(NHSTFGDEP_COMP) 
ENDIF

IF (LVARBC .AND. LLOBAD .AND. .NOT.LREPRO4DVAR) THEN
  ! Sum the components of the bias parameters
  DO JBGRP=1,NOBGROUP
    IF (YVARBC(JBGRP)%NPARAM > 0) THEN
      DO JSET=1,NSETOT
        YVARBC(JBGRP)%APARAMS(1:YVARBC(JBGRP)%NPARAM)= &
         & YVARBC(JBGRP)%APARAMS(1:YVARBC(JBGRP)%NPARAM) &
         & +APARAMS_COMP(JSET,1:YVARBC(JBGRP)%NPARAM,JBGRP)
      ENDDO
    ENDIF
  ENDDO
                         
  IF (NSETOT > 0) DEALLOCATE(APARAMS_COMP)
ENDIF


!-----------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('TASKOB',1,ZHOOK_HANDLE)
END SUBROUTINE TASKOB
