SUBROUTINE TROPLEV (KPROMA,KSTART,KPROF,KFLEV,PT,PQ,PRESF,KLTROP)

!      TROPLEV - Calculate model level closest to the tropopause.
!                Original code by Adrian Simmons.

!     Arguments :
!     -----------
!     Input:
!       KPROMA              - horizontal dimension
!       KSTART              - start of work
!       KPROF               - depth of work
!       KFLEV               - number of model levels
!       PT(KPROMA,KFLEV)    - temperature on full model levels
!       PQ(KPROMA,KFLEV)    - humidity on full model levels
!       PRESF(KPROMA,KFLEV) - model full level pressures

!     Input/Output:
!       KLTROP(KPROMA)      - tropopause model levels

!     Author:  Adrian Simmons   *ECMWF*
!     -------

!     Modifications:
!     --------------
!     E.Holm       05-10-20   Moved code here from symtransin and jgnr/i/ad.
!     E.Holm       07-12-14   Alternative humidity tropopause definition
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK    ,DR_HOOK

USE YOMCST    ,ONLY : RD       ,RG       ,RKAPPA
USE YOMJG    , ONLY : LJB_SUPSAT
IMPLICIT NONE


INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTART
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROF
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEV
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PQ(KPROMA,KFLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRESF(KPROMA,KFLEV)
INTEGER(KIND=JPIM),INTENT(INOUT) :: KLTROP(KPROMA)

INTEGER(KIND=JPIM) :: ILEVM2, JROF, JLEV
REAL(KIND=JPRB) :: ZSTAB
REAL(KIND=JPRB) :: ZHOOK_HANDLE


!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TROPLEV',0,ZHOOK_HANDLE)

!*         1. COMPUTE TROPOPAUSE HEIGHT FROM LAPSE RATE DEFINITION
!*            OR HUMIDITY DEFINITION
!             ----------------------------------------------------

ILEVM2=KFLEV-2
DO JROF=KSTART,KPROF
  KLTROP(JROF)=ILEVM2
ENDDO

DO JLEV=1,ILEVM2
  DO JROF=KSTART,KPROF
    IF (PRESF(JROF,JLEV)>=7.E3_JPRB .AND. PRESF(JROF,JLEV)<=5.E4_JPRB) THEN
      IF (LJB_SUPSAT) THEN
!*         1.1 Humidity tropopause
        IF (KLTROP(JROF)==ILEVM2 .AND. PQ(JROF,JLEV)  >3.E-6_JPRB &
           &                     .AND. PQ(JROF,JLEV+2)>5.E-6_JPRB) THEN
          KLTROP(JROF)=JLEV
        ENDIF
      ELSE
!*         1.2 Lapse rate tropopause
         ZSTAB =  PRESF(JROF,JLEV+1)*(   PT(JROF,JLEV+2)-   PT(JROF,JLEV))&
               &  /(PT(JROF,JLEV+1)*(PRESF(JROF,JLEV+2)-PRESF(JROF,JLEV)))&
               &+ RD*PT(JROF,JLEV+1)*2.5E-4_JPRB/(RG*RG)
         IF (KLTROP(JROF)==ILEVM2 .AND. ZSTAB>RKAPPA) THEN
           KLTROP(JROF)=JLEV
         ENDIF
      ENDIF
    ELSEIF (PRESF(JROF,JLEV)>5.E4_JPRB .AND. KLTROP(JROF)==ILEVM2)THEN
      KLTROP(JROF)=JLEV-1
    ENDIF
  ENDDO
ENDDO


!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TROPLEV',1,ZHOOK_HANDLE)
END SUBROUTINE TROPLEV

