subroutine wrevecs (knits,kmaxit,pevals,pbnds,pblim,pmineval,&
 & pv,YD_PLANCV,kngood,ptheta)  

!-----------------------------------------------------------------------
!   WREVECS - Called from CONGRAD. Computes and saves eigenvectors.

!   Purpose.
!   --------

!   Interface.
!   ----------
!      CALL WREVECS (knits,kmaxit,pevals,pbnds,pblim,pmineval,pv,plancv,&
!                   &kngood,ptheta)

!      Explicit arguments: 

!      Inputs: knits   -- Number of Lanczos vectors.
!              kmaxit  -- First dimension of 'pv'.
!              pevals  -- Approximate eigenvalues (Ritz values).
!              pbnds   -- Error bounds on eigenvectors.
!              pblim   -- Error bound below which eigenvector is 'good'.
!              pmineval-- Lower bound for acceptable eigenvalues
!              pv      -- Eigenvector matrix of tridiagonal problem.
!              plancv  -- Lanczos vectors
!      Outputs: kngood -- Number of eigenvectors writen.
!               ptheta -- Eigenvalues .

!      Implicit output: eigenvalues and eigenvectors are saved in YOMVCGL
!   Externals.   
!   ----------

!   Reference.
!   ----------
!       None yet!

!   Author.
!   -------
!       Mike Fisher  *ECMWF*

!   Modifications.
!   --------------
!       Original 20/5/94
!       M. Fisher 01-01-96 : Optionally save vectors in memory
!       M. Fisher 26-11-96 : USE DISTRIBUTED_VECTORS_MIX
!       M. Fisher 31-03-99 : Removed the last vestiges of the MIO package
!       Y.Tremolet 02-08-06: Use control_vectors.
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        M.Hamrud      01-Dec-2003 CY28R1 Cleaning
!       J.Hague       24-Feb-2004 CY29R1 Tuning
!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMLUN   , ONLY : NULOUT
USE YOMVCGL  , ONLY : YVCGLEV   ,RCGLEV   ,NVCGLEV
USE CONTROL_VECTORS, ONLY : CONTROL_VECTOR, ALLOCATE_CTLVEC, DEALLOCATE_CTLVEC, &
                          & DOT_PRODUCT, ASSIGNMENT(=)
USE OML_MOD, ONLY : OML_MAX_THREADS, OML_MY_THREAD

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KNITS 
INTEGER(KIND=JPIM),INTENT(IN)    :: KMAXIT 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEVALS(KNITS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBNDS(KNITS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PBLIM 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PMINEVAL 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KMAXIT+1,KNITS) 
TYPE(CONTROL_VECTOR),INTENT(IN)    :: YD_PLANCV(KMAXIT+1) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KNGOOD 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PTHETA(KMAXIT) 
REAL(KIND=JPRB) :: Z_DLA

INTEGER(KIND=JPIM) :: i, ij, jk, jm, jj, isize, iblock, ibeg, iend
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!--- Count and collect converged eigenvalues

IF (LHOOK) CALL DR_HOOK('WREVECS',0,ZHOOK_HANDLE)
kngood = 0
do jk=knits,1,-1
  if (pbnds(jk) <= pblim .AND. pevals(jk) > pmineval) then
    kngood=kngood+1
    ptheta(kngood) = pevals(jk)
  endif
ENDDO

write (NULOUT,'(/)')
write (NULOUT,*)'Calculating eigenvectors for the following eigenvalues:'
write (NULOUT,*) ptheta(1:kngood)
write (NULOUT,'(/)')

if (ALLOCATED(RCGLEV)) DEALLOCATE(RCGLEV)
ALLOCATE (RCGLEV(kngood))
WRITE(NULOUT,9990) 'RCGLEV   ',SIZE(RCGLEV),SHAPE(RCGLEV)

RCGLEV(:) = ptheta(1:kngood)

NVCGLEV = kngood

if (ALLOCATED(YVCGLEV)) THEN
  DO i=1,SIZE(YVCGLEV)
    CALL deallocate_ctlvec(YVCGLEV(i))
  ENDDO
  DEALLOCATE(YVCGLEV)
endif

ALLOCATE (YVCGLEV(kngood))
DO i=1,kngood
  CALL allocate_ctlvec(YVCGLEV(i))
ENDDO

!--- Calculate converged eigenvectors

ij=0
do jm=knits,1,-1

  if (pbnds(jm) <= pblim .AND. pevals(jm) > pmineval) then
    ij = ij+1
    call gstats(1239,0)
    YVCGLEV(ij) = 0.0_JPRB
    isize=size(YVCGLEV(ij)%data)
    iblock=(isize-1)/oml_max_threads()+1
    if(isize > 0)then
!$OMP PARALLEL PRIVATE(ibeg,iend,jk)
      ibeg=(oml_my_thread()-1)*iblock+1
      iend=min(isize,oml_my_thread()*iblock)
      do jk=1,knits-1,2
        YVCGLEV(ij)%data(ibeg:iend) = YVCGLEV(ij)%data(ibeg:iend) &
           & + YD_PLANCV(jk  )%data(ibeg:iend) * pv(jk  ,jm) &
           & + YD_PLANCV(jk+1)%data(ibeg:iend) * pv(jk+1,jm)
      enddo
      if(mod(knits,2) /= 0) then
        YVCGLEV(ij)%data(ibeg:iend) = YVCGLEV(ij)%data(ibeg:iend) &
           & + YD_PLANCV(knits)%data(ibeg:iend) * pv(knits,jm) 
      endif
!$OMP END PARALLEL
    endif
    call gstats(1239,1)

    do jk=1,ij-1
      Z_DLA=DOT_PRODUCT (YVCGLEV(jk),YVCGLEV(ij))
      call gstats(1239,0)
      if(isize > 0)then
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jj)
        do jj = 1,isize
          YVCGLEV(ij)%data(jj) = YVCGLEV(ij)%data(jj) - Z_DLA*YVCGLEV(jk)%data(jj)
        enddo
!$OMP END PARALLEL DO
      endif
      call gstats(1239,1)
    enddo

    Z_DLA=DOT_PRODUCT (YVCGLEV(ij),YVCGLEV(ij))
    call gstats(1239,0)
    if(isize > 0)then
!$OMP PARALLEL DO SCHEDULE(STATIC) PRIVATE(jj)
      do jj = 1,isize
        YVCGLEV(ij)%data(jj) = YVCGLEV(ij)%data(jj) / sqrt(Z_DLA)
      enddo
!$OMP END PARALLEL DO
    endif
    call gstats(1239,1)
  endif
ENDDO

9990 FORMAT(1X,'ARRAY ',A10,' ALLOCATED ',8I8)
IF (LHOOK) CALL DR_HOOK('WREVECS',1,ZHOOK_HANDLE)
end subroutine wrevecs
