SUBROUTINE SURFBC    (KIDIA,KFDIA,KLON,KTILES,&
 & PTVL   ,PTVH   ,PSOTY  , PSOALUVP, PSOALUVD, PSOALNIP, PSOALNID,&
 & PSDOR, PCVLC  ,PCVHC,&
 & PLAILC  ,PLAIHC,&
 & PLSM   ,PCI    ,PCLAKE ,PHLICE,&
 & PGEMU  ,PSNM1M ,PWLM1M ,PRSNM1M,&  
 & LDLAND ,LDSICE ,LDLAKE ,LDNH, LDOCN_KPP,&
 & KTVL   ,KTVH   ,KSOTY,&
 & PCVL   ,PCVH, PLAIL, PLAIH,  PWLMX  ,PFRTI,&
 & PALBF  ,PALUVP ,PALUVD ,PALNIP ,PALNID)    

!     ------------------------------------------------------------------
!**   *SURFBC* - CREATES BOUNDARY CONDITIONS CHARACTERIZING THE SURFACE

!     PURPOSE
!     -------
!     CREATES AUXILIARY FIELDS NEEDED BY VDFMAIN, SURFTSTP, AND SURFRAD

!     INTERFACE
!     ---------
!     *SURFBC* IS CALLED BY *CALLPAR* AND *RADPAR*

!     INPUT PARAMETERS (INTEGER):
!     *KIDIA*        START POINT
!     *KFDIA*        END POINT
!     *KLON*         NUMBER OF GRID POINTS PER PACKET
!     *KTILES*       TILE INDEX
!     *PTVL*         LOW VEGETATION TYPE (REAL)
!     *PTVH*         HIGH VEGETATION TYPE (REAL)
!     *PSOTY*        SOIL TYPE (REAL)                               (1-7)
!     *PSOALUVP*     SOIL ALBEDO UV/VIS DIRECT                      (0-1)
!     *PSOALUVD*     SOIL ALBEDO UV/VIS DIFFUSE                     (0-1)
!     *PSOALNIP*     SOIL ALBEDO NIR DIRECT                         (0-1)
!     *PSOALNID*     SOIL ALBEDO NIR DIFFUSE                        (0-1)
!     *PSDOR*        STANDARD DEV. OF OROGRAPHY (REAL)              (m)

!     INPUT PARAMETERS (REAL):
!     *PCVLC*        LOW VEGETATION COVER  (CLIMATE)                (0-1)
!     *PCVHC*        HIGH VEGETATION COVER (CLIMATE)                (0-1)
!     *PLAILC*        LOW LAI (CLIMATE)                             m2/m2
!     *PLAIHC*        HIGH LAI (CLIMATE)                            m2/m2
!     *PLSM*         LAND-SEA MASK                                  (0-1)
!     *PCI*          SEA-ICE FRACTION                               (0-1)
!     *PCLAKE*       LAKE FRACTION                                  (0-1)
!     *PHLICE*       LAKE ICE THICKNESS                               m 
!     *PGEMU*        COSINE OF LATITUDE
!     *PSNM1M*       SNOW MASS (per unit area)                      kg/m**2
!     *PWLM1M*       INTERCEPTION RESERVOIR CONTENTS                kg/m**2
!     *PRSNM1M*      SNOW DENSITY                                   kg/m**3

!     OUTPUT PARAMETERS (LOGICAL):
!     *LDLAND*       LAND INDICATOR
!     *LDSICE*       SEA-ICE INDICATOR
!     *LDLAKE*       LAKE INDICATOR
!     *LDNH*         NORTHERN HEMISPHERE INDICATOR
!     *LDOCN_KPP*    KPP MODEL INDICATOR 

!     OUTPUT PARAMETERS (REAL):
!     *PCVL*         LOW VEGETATION COVER  (CORRECTED)              (0-1)
!     *PCVH*         HIGH VEGETATION COVER (CORRECTED)              (0-1)
!     *PLAIL*        LOW LAI (REAL)                                 m2/m2
!     *PLAIH*        HIGH LAI (REAL)                                m2/m2
!     *PWLMX*        MAXIMUM SKIN RESERVOIR CAPACITY                kg/m**2
!     *PFRTI*        TILE FRACTIONS                                 (0-1)
!            1 : WATER                  5 : SNOW ON LOW-VEG+BARE-SOIL
!            2 : ICE                    6 : DRY SNOW-FREE HIGH-VEG
!            3 : WET SKIN               7 : SNOW UNDER HIGH-VEG
!            4 : DRY SNOW-FREE LOW-VEG  8 : BARE SOIL

!     OUTPUT PARAMETERS (INTEGER):
!     *KTVL*         LOW VEGETATION TYPE
!     *KTVH*         HIGH VEGETATION TYPE
!     *KSOTY*        SOIL TYPE                                      (1-7)

!     METHOD
!     ------
!     IT IS NOT ROCKET SCIENCE, BUT CHECK DOCUMENTATION

!     Modifications
!     P. VITERBO       E.C.M.W.F.         18-02-99
!     J.F. Estrade *ECMWF* 03-10-01 move in surf vob
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     P. Viterbo       24-05-2004       Change surface units         
!     G. Balsamo       15-01-2007       Add soil type
!     E. Dutra/G. Balsamo 01-05-2008    Add lake tile
!     Y. Takaya        07-10-2008       Add flag for ocean mixed layer model
!     S. Boussetta/G.Balsamo May 2009 Add lai
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

!ifndef INTERFACE

USE ABORT_SURF_MOD
USE SURFBC_CTL_MOD

!endif INTERFACE

IMPLICIT NONE

! Declaration of arguments

INTEGER(KIND=JPIM),INTENT(IN)    :: KIDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFDIA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KLON 
INTEGER(KIND=JPIM),INTENT(IN)    :: KTILES 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTVL(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTVH(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOTY(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALUVP(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALUVD(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALNIP(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSOALNID(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSDOR(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVLC(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCVHC(:)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAILC(:)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLAIHC(:)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PLSM(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCI(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSNM1M(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWLM1M(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRSNM1M(:)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PCLAKE(:) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PHLICE(:)
LOGICAL           ,INTENT(OUT)   :: LDLAND(:) 
LOGICAL           ,INTENT(OUT)   :: LDSICE(:) 
LOGICAL           ,INTENT(OUT)   :: LDNH(:) 
LOGICAL           ,INTENT(OUT)   :: LDLAKE(:)  
LOGICAL           ,INTENT(OUT)   :: LDOCN_KPP(:)  
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTVL(:) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KTVH(:) 
INTEGER(KIND=JPIM),INTENT(OUT)   :: KSOTY(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCVL(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PCVH(:)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLAIL(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PLAIH(:)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWLMX(:) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PFRTI(:,:) 
REAL(KIND=JPRB),    INTENT(OUT)  :: PALBF(:)
REAL(KIND=JPRB),    INTENT(OUT)  :: PALUVP(:)
REAL(KIND=JPRB),    INTENT(OUT)  :: PALUVD(:)
REAL(KIND=JPRB),    INTENT(OUT)  :: PALNIP(:)
REAL(KIND=JPRB),    INTENT(OUT)  :: PALNID(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!ifndef INTERFACE

IF (LHOOK) CALL DR_HOOK('SURFBC',0,ZHOOK_HANDLE)
IF(UBOUND(PTVL,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PTVL TOO SHORT!')
ENDIF

IF(UBOUND(PTVH,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PTVH TOO SHORT!')
ENDIF

IF(UBOUND(PSOTY,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PSOTY TOO SHORT!')
ENDIF

IF(UBOUND(PSOALUVP,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PSOALUVP TOO SHORT!')
ENDIF

IF(UBOUND(PSOALUVD,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PSOALUVD TOO SHORT!')
ENDIF

IF(UBOUND(PSOALNIP,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PSOALNIP TOO SHORT!')
ENDIF

IF(UBOUND(PSOALNID,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PSOALNID TOO SHORT!')
ENDIF

IF(UBOUND(PSDOR,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PSDOR TOO SHORT!')
ENDIF

IF(UBOUND(PCVLC,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PCVLC TOO SHORT!')
ENDIF

IF(UBOUND(PCVHC,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PCVHC TOO SHORT!')
ENDIF

IF(UBOUND(PLAILC,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PLAILC TOO SHORT!')
ENDIF

IF(UBOUND(PLAIHC,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PLAIHC TOO SHORT!')
ENDIF

IF(UBOUND(PLSM,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PLSM TOO SHORT!')
ENDIF

IF(UBOUND(PCI,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PCI TOO SHORT!')
ENDIF

IF(UBOUND(PCLAKE,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PCLAKE TOO SHORT!')
ENDIF

IF(UBOUND(PHLICE,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PHLICE TOO SHORT!')
ENDIF

IF(UBOUND(LDLAKE,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: LDLAKE TOO SHORT!')
ENDIF

IF(UBOUND(PGEMU,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PGEMU TOO SHORT!')
ENDIF

IF(UBOUND(PSNM1M,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PSNM1M TOO SHORT!')
ENDIF

IF(UBOUND(PWLM1M,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PWLM1M TOO SHORT!')
ENDIF

IF(UBOUND(PRSNM1M,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PRSNM1M TOO SHORT!')
ENDIF

IF(UBOUND(LDLAND,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: LDLAND TOO SHORT!')
ENDIF

IF(UBOUND(LDSICE,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: LDSICE TOO SHORT!')
ENDIF

IF(UBOUND(LDOCN_KPP,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: LDOCN_KPP TOO SHORT!')
ENDIF

IF(UBOUND(LDNH,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: LDNH TOO SHORT!')
ENDIF

IF(UBOUND(KTVL,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: KTVL TOO SHORT!')
ENDIF

IF(UBOUND(KTVH,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: KTVH TOO SHORT!')
ENDIF

IF(UBOUND(KSOTY,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: KSOTY TOO SHORT!')
ENDIF

IF(UBOUND(PCVL,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PCVL TOO SHORT!')
ENDIF

IF(UBOUND(PCVH,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PCVH TOO SHORT!')
ENDIF

IF(UBOUND(PLAIL,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PLAIL TOO SHORT!')
ENDIF

IF(UBOUND(PLAIH,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PLAIH TOO SHORT!')
ENDIF

IF(UBOUND(PWLMX,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PWLMX TOO SHORT!')
ENDIF

IF(UBOUND(PFRTI,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: FIRST DIMENSION OF PFRTI TOO SHORT!')
ENDIF

IF(UBOUND(PFRTI,2) < KTILES) THEN
  CALL ABORT_SURF('SURFBC: SECOND DIMENSION OF PFRTI TOO SHORT!')
ENDIF

IF(UBOUND(PALBF,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PALBF TOO SHORT!')
ENDIF

IF(UBOUND(PALUVP,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PALUVP TOO SHORT!')
ENDIF

IF(UBOUND(PALUVD,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PALUVD TOO SHORT!')
ENDIF

IF(UBOUND(PALNIP,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PALNIP TOO SHORT!')
ENDIF

IF(UBOUND(PALNID,1) < KLON) THEN
  CALL ABORT_SURF('SURFBC: PALNID TOO SHORT!')
ENDIF

CALL SURFBC_CTL   (KIDIA,KFDIA,KLON,KTILES,&
 & PTVL   ,PTVH   ,PSOTY  , PSOALUVP, PSOALUVD, PSOALNIP, PSOALNID,&
 & PSDOR  ,PCVLC  ,PCVHC,&
 & PLAILC  ,PLAIHC,&
 & PLSM   ,PCI    ,PCLAKE ,PHLICE, PGEMU  ,PSNM1M ,PWLM1M,PRSNM1M,&  
 & LDLAND ,LDSICE ,LDLAKE ,LDNH   ,LDOCN_KPP,&
 & KTVL   ,KTVH   ,KSOTY,&
 & PCVL   ,PCVH   ,PLAIL, PLAIH,  PWLMX  ,PFRTI,&
 & PALBF  ,PALUVP ,PALUVD ,PALNIP ,PALNID)  

IF (LHOOK) CALL DR_HOOK('SURFBC',1,ZHOOK_HANDLE)

!endif INTERFACE

!     ------------------------------------------------------------------

END SUBROUTINE SURFBC
