MODULE SRFWNG_MOD
CONTAINS
SUBROUTINE SRFWNG(KIDIA,KFDIA,KLON,PTMST,KSOTY,&
 & PWL,PWLMX,PWSA,&
 & PROS,PROD,PWFSD,&
 & LDLAND,PDHWLS)  
 
USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOS_THF   , ONLY : RHOH2O
USE YOS_SOIL  , ONLY : RWSAT    ,RWSATM   ,RDAW &
 &                    ,LEVGEN

#ifdef DOC
!**** *SRFWNG* -  COMPUTES CORRECTIONS TO AVOID NEGATIVE SOIL MOISTURE.

!     PURPOSE.
!     --------
!          THIS ROUTINE COMPUTES CORRECTIONS IN THE SOIL MOISTURE OF
!     THE SKIN, SURFACE AND DEEP RESERVOIR AND RUN-OFF TO AVOID NEGATIVE
!     SOIL MOISTURE VALUES.

!**   INTERFACE.
!     ----------
!          *SRFWNG* IS CALLED FROM *SURF*.

!     PARAMETER   DESCRIPTION                                    UNITS
!     ---------   -----------                                    -----

!     INPUT PARAMETERS (INTEGER):
!    *KIDIA*      START POINT
!    *KFDIA*      END POINT
!    *KLON*       NUMBER OF GRID POINTS PER PACKET
!    *KSOTY*      SOIL TYPE                                     (1-7)

!     INPUT PARAMETERS (REAL):
!    *PTMST*      TIME STEP FOR THE PHYSICS                       S

!     INPUT PARAMETERS (LOGICAL):
!    *LDLAND*     LAND/SEA MASK (TRUE/FALSE)

!     INPUT PARAMETERS AT T-1 (ACCUMULATED,REAL):

!     INPUT/OUTPUT PARAMETERS AT T+1 (UNFILTERED,REAL):
!    *PWL*        SKIN RESERVOIR WATER CONTENT                   kg/m**2
!    *PWLMX*      MAXIMUM SKIN RESERVOIR CONTENT                 kg/m**2
!    *PWSA*       SOIL MOISTURE                                M**3/M**3

!     INPUT/OUTPUT PARAMETERS (REAL):
!    *PROS*       RUN-OFF FOR THE SURFACE LAYER                 kg/m**2
!    *PROD*       RUN-OFF FOR DEEPER LAYERS (EXCEPT DRAINAGE)   kg/m**2
!    *PWFSD*      WATER FLUX BETWEEN SURFACE AND DEEP LAYER     kg/m**2/s

!     OUTPUT PARAMETERS (DIAGNOSTIC):
!    *PDHWLS*     Diagnostic array for soil water (see module yomcdh)

!     METHOD.
!     -------
!          STRAIGHTFORWARD ONCE THE DEFINITION OF THE CONSTANTS IS
!     UNDERSTOOD. FOR THIS REFER TO DOCUMENTATION.

!     EXTERNALS.
!     ----------
!          NONE.

!     REFERENCE.
!     ----------
!          SEE SOIL PROCESSES' PART OF THE MODEL'S DOCUMENTATION FOR
!     DETAILS ABOUT THE MATHEMATICS OF THIS ROUTINE.

!     Original    P.VITERBO      E.C.M.W.F.     16/01/89
!     Modified    P.VITERBO    99-03-26     Tiling of the land surface
!     Modified    J.F. Estrade *ECMWF* 03-10-01 move in surf vob
!     P. Viterbo    24-05-2004      Change surface units
!     G. Balsamo    03-07-2004      Add soil type
!     ------------------------------------------------------------------
#endif

IMPLICIT NONE

! Declaration of arguments

INTEGER(KIND=JPIM), INTENT(IN)   :: KIDIA
INTEGER(KIND=JPIM), INTENT(IN)   :: KFDIA
INTEGER(KIND=JPIM), INTENT(IN)   :: KLON
INTEGER(KIND=JPIM), INTENT(IN)   :: KSOTY(:)

REAL(KIND=JPRB),    INTENT(IN)   :: PTMST
REAL(KIND=JPRB),    INTENT(IN)   :: PWLMX(:)

LOGICAL,   INTENT(IN)   :: LDLAND(:)

REAL(KIND=JPRB),    INTENT(INOUT):: PWL(:)
REAL(KIND=JPRB),    INTENT(INOUT):: PROS(:)
REAL(KIND=JPRB),    INTENT(INOUT):: PWFSD(:)
REAL(KIND=JPRB),    INTENT(INOUT):: PWSA(:,:)
REAL(KIND=JPRB),    INTENT(INOUT):: PDHWLS(:,:,:)

REAL(KIND=JPRB),    INTENT(OUT)  :: PROD(:)

!*         0.2    DECLARATION OF LOCAL VARIABLES.
!                 ----------- -- ----- ----------

INTEGER(KIND=JPIM) :: JL, JS

REAL(KIND=JPRB) :: ZD1, ZD2, ZD3, ZD4, ZDWC1, ZDWC2, ZDWD, ZDZWL,&
 & ZROG, ZRRR2, ZRRR3, ZRRR4, ZWSAT,ZTMST,ZDWS  
REAL(KIND=JPRB) :: ZID1,ZID2,ZID3,ZID4,ZHOH2O
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------
!*         1.    SET UP SOME CONSTANTS.
!                --- -- ---- ----------

!*               PHYSICAL CONSTANTS.
!                -------- ----------

IF (LHOOK) CALL DR_HOOK('SRFWNG_MOD:SRFWNG',0,ZHOOK_HANDLE)
ZTMST=1.0_JPRB/PTMST
ZD1=RDAW(1)
ZID1=1.0_JPRB/ZD1
ZD2=RDAW(2)
ZID2=1.0_JPRB/ZD2
ZD3=RDAW(3)
ZID3=1.0_JPRB/ZD3
ZD4=RDAW(4)
ZID4=1.0_JPRB/ZD4
ZHOH2O=1./RHOH2O

!     ------------------------------------------------------------------
!*         2. WATER CORRECTIONS AND VEGETATION RATIO UPDATE.
!             ----- ----------- --- ---------- ----- -------
DO JL=KIDIA,KFDIA

  IF (LDLAND(JL)) THEN

    IF (LEVGEN) THEN
      JS=KSOTY(JL)
      ZWSAT=RWSATM(JS)
    ELSE
      ZWSAT=RWSAT
    ENDIF
!          LIMIT PWL        PWL < ZWLMX
    ZDZWL=MAX(0.0_JPRB,PWL(JL)-PWLMX(JL))
    PWL(JL)=PWL(JL)-ZDZWL
    PWSA(JL,1)=PWSA(JL,1)+ZDZWL*ZHOH2O*ZID1

!          LIMIT PW1       0. < PWS < WSAT
    ZDWS=MIN(PWSA(JL,1),0.0_JPRB)
    PWSA(JL,1)=PWSA(JL,1)-ZDWS
    PWSA(JL,2)=PWSA(JL,2)+ZDWS*ZD1*ZID2
    ZROG=MAX(0.0_JPRB,PWSA(JL,1)-ZWSAT)
    PROS(JL)=PROS(JL)+ZROG*RHOH2O*ZD1
    PWSA(JL,1)=PWSA(JL,1)-ZROG
    PWFSD(JL)=PWFSD(JL)+ZDWS*RHOH2O*ZD1*ZTMST

! DDH diagnostics (runoff)
!    (negative values mean water lost by the layer)
    PDHWLS(JL,1,6)=PDHWLS(JL,1,6)-(RHOH2O*ZTMST)*ZD1*(ZDWS+ZROG)
    PDHWLS(JL,2,6)=PDHWLS(JL,2,6)+(RHOH2O*ZTMST)*ZD1*ZDWS

!          LIMIT PW2       0. < PWD < WSAT

    ZDWD=MIN(PWSA(JL,2),0.0_JPRB)
    PWSA(JL,2)=PWSA(JL,2)-ZDWD
    PWSA(JL,3)=PWSA(JL,3)+ZDWD*ZD2*ZID3
    ZRRR2=MAX(0.0_JPRB,PWSA(JL,2)-ZWSAT)
    PROD(JL)=ZD2*RHOH2O*ZRRR2
    PWSA(JL,2)=PWSA(JL,2)-ZRRR2
! DDH diagnostics (runoff)
!    (negative values mean water lost by the layer)
    PDHWLS(JL,2,6)=PDHWLS(JL,2,6)-(RHOH2O*ZTMST)*ZD2*(ZDWD+ZRRR2)
    PDHWLS(JL,3,6)=PDHWLS(JL,3,6)+(RHOH2O*ZTMST)*ZD2*(ZDWD)

!          LIMIT PW3      0. < PWC1 < WSAT

    ZDWC1=MIN(PWSA(JL,3),0.0_JPRB)
    PWSA(JL,3)=PWSA(JL,3)-ZDWC1
    PWSA(JL,4)=PWSA(JL,4)+ZDWC1*ZD3*ZID4
    ZRRR3=MAX(0.0_JPRB,PWSA(JL,3)-ZWSAT)
    PWSA(JL,3)=PWSA(JL,3)-ZRRR3
    PROD(JL)=PROD(JL)+ZD3*RHOH2O*ZRRR3
! DDH diagnostics (runoff)
!    (negative values mean water lost by the layer)
    PDHWLS(JL,3,6)=PDHWLS(JL,3,6)-(RHOH2O*ZTMST)*ZD3*(ZDWC1+ZRRR3)
    PDHWLS(JL,4,6)=PDHWLS(JL,4,6)+(RHOH2O*ZTMST)*ZD3*(ZDWC1)

!          LIMIT PW4      0. < PWC2 < WSAT

    ZDWC2=MIN(PWSA(JL,4),0.0_JPRB)
    PWSA(JL,4)=PWSA(JL,4)-ZDWC2
    ZRRR4=MAX(0.0_JPRB,PWSA(JL,4)-ZWSAT)
    PWSA(JL,4)=PWSA(JL,4)-ZRRR4
    PROD(JL)=PROD(JL)+ZD4*RHOH2O*ZRRR4
! DDH diagnostics (runoff)
!    (negative values mean water lost by the layer)
    PDHWLS(JL,4,6)=PDHWLS(JL,4,6)-(RHOH2O*ZTMST)*ZD4*(ZDWC2+ZRRR4)
  ELSE
!          SEA POINTS
    PROD(JL)=0.0_JPRB
    PWFSD(JL)=0.0_JPRB
  ENDIF
ENDDO
IF (LHOOK) CALL DR_HOOK('SRFWNG_MOD:SRFWNG',1,ZHOOK_HANDLE)

END SUBROUTINE SRFWNG
END MODULE SRFWNG_MOD
