MODULE SURFBC_CTL_MOD
CONTAINS
SUBROUTINE SURFBC_CTL(KIDIA,KFDIA,KLON,KTILES,&
 & PTVL   ,PTVH   ,PSOTY  , PSOALUVP, PSOALUVD, PSOALNIP, PSOALNID,&
 & PSDOR  ,PCVLC  ,PCVHC,&
 & PLAILC ,PLAIHC ,&
 & PLSM   ,PCI    ,PCLAKE ,PHLICE,&
 & PGEMU  ,PSNM1M ,PWLM1M ,PRSNM1M,&  
 & LDLAND ,LDSICE ,LDLAKE ,LDNH, LDOCN_KPP,&      
 & KTVL   ,KTVH   ,KSOTY,&
 & PCVL   ,PCVH ,PLAIL, PLAIH ,PWLMX  ,PFRTI,&
 & PALBF  ,PALUVP ,PALUVD ,PALNIP ,PALNID)  

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOS_SOIL  , ONLY : RQSNCR   ,RWLMAX   ,RCIMIN ,RTHRFRTI, LESN09
USE YOS_VEG   , ONLY : RVCOV    ,RVLAI  ,LELAIV ,KCVL ,KCVH ,ALCVL ,ALCVH, RVVEGALB
USE YOS_FLAKE , ONLY : LEFLAKE, RH_ICE_MIN_FLK
USE YOS_OCEAN_ML , ONLY : LEOCML
USE ABORT_SURF_MOD  

USE YOMECEARTH , ONLY : LLPJGFB, LRDALB, N_COMPUTE_EFF_VEG_FRACTION

#ifdef DOC
!     ------------------------------------------------------------------
!**   *SURFBC* - CREATES BOUNDARY CONDITIONS CHARACTERIZING THE SURFACE

!     PURPOSE
!     -------
!     CREATES AUXILIARY FIELDS NEEDED BY SURFEXCDRIVER, SURFTSTP, AND SURFRAD

!     INTERFACE
!     ---------
!     *SURFBC* IS CALLED BY *CALLPAR* AND *RADPAR*

!     INPUT PARAMETERS (INTEGER):
!     *KIDIA*        START POINT
!     *KFDIA*        END POINT
!     *KLON*         NUMBER OF GRID POINTS PER PACKET
!     *KTILES*       TILE INDEX
!     *PTVL*         LOW VEGETATION TYPE (REAL)
!     *PTVH*         HIGH VEGETATION TYPE (REAL)
!     *PSOTY*        SOIL TYPE (REAL)                               (1-7)
!     *PSOALUVP*      SOIL ALBEDO UV/VIS DIRECT                     (0-1)
!     *PSOALUVD*      SOIL ALBEDO UV/VIS DIFFUSE                    (0-1)
!     *PSOALNIP*      SOIL ALBEDO NIR    DIRECT                     (0-1)
!     *PSOALNID*      SOIL ALBEDO NIR    DIFFUSE                    (0-1)
!     *PSDOR*        STANDARD DEV. OF OROGRAPHY                     (m)

!     INPUT PARAMETERS (REAL):
!     *PCVLC*        LOW VEGETATION COVER  (CLIMATE)                (0-1)
!     *PCVHC*        HIGH VEGETATION COVER (CLIMATE)                (0-1)
!     *PLAILC*        LOW LAI (FROM CLIMATE FILE)                   m2/m2
!     *PLAIHC*        HIGH LAI (FROM CLIMATE FILE)                  m2/m2
!     *PLSM*         LAND-SEA MASK                                  (0-1)
!     *PCI*          SEA-ICE FRACTION                               (0-1)
!     *PCLAKE*       LAKE FRACTION                                  (0-1)
!     *PHLICE*       LAKE ICE THICKNESS                               m
!     *PGEMU*        COSINE OF LATITUDE
!     *PSNM1M*       SNOW MASS (per unit area)                      kg/m**2
!     *PWLM1M*       INTERCEPTION RESERVOIR CONTENTS                kg/m**2
!     *PRSNM1M*      SNOW DENSITY                                   kg/m**3

!     OUTPUT PARAMETERS (LOGICAL):
!     *LDLAND*       LAND INDICATOR
!     *LDSICE*       SEA-ICE INDICATOR
!     *LDLAKE*       LAKE INDICATOR
!     *LDNH*         NORTHERN HEMISPHERE INDICATOR
!     *LDOCM_KPP*    OCEAN INDICATOR

!     OUTPUT PARAMETERS (REAL):
!     *PCVL*         LOW VEGETATION COVER  (CORRECTED)              (0-1)
!     *PCVH*         HIGH VEGETATION COVER (CORRECTED)              (0-1)
!     *PLAIL*         LOW LAI  (CORRECTED)                         (m2/m2)
!     *PLAIH*         HIGH LAI (CORRECTED)                         (m2/m2)
!     *PWLMX*        MAXIMUM SKIN RESERVOIR CAPACITY                kg/m**2
!     *PFRTI*        TILE FRACTIONS                                 (0-1)
!            1 : WATER                  5 : SNOW ON LOW-VEG+BARE-SOIL
!            2 : ICE                    6 : DRY SNOW-FREE HIGH-VEG
!            3 : WET SKIN               7 : SNOW UNDER HIGH-VEG
!            4 : DRY SNOW-FREE LOW-VEG  8 : BARE SOIL
!            9 : LAKE

!     OUTPUT PARAMETERS (INTEGER):
!     *KTVL*         LOW VEGETATION COVER
!     *KTVH*         HIGH VEGETATION COVER
!     *KSOTY*        SOIL TYPE                                      (1-7)

!     METHOD
!     ------
!     IT IS NOT ROCKET SCIENCE, BUT CHECK DOCUMENTATION

!     Modifications
!     P. VITERBO       E.C.M.W.F.         18-02-99
!     J.F. Estrade *ECMWF* 03-10-01 move in surf vob
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     P. Viterbo    24-05-2004      Change surface units
!     P. Viterbo   ECMWF   03-12-2004  Include user-defined RTHRFRTI
!     G. Balsamo   ECMWF   15-01-2007  Add soil type
!     E. Dutra/G. Balsamo  01-05-2008  Add lake tile / Allow sub-grid lake tile 
!     Y. Takaya    ECMWF   07-10-2008  Add ocean mixed layer grids
!     E. Dutra             21/11/2008  Couple snow deck with lake ice (recalculation of tile fractions )
!     S. Boussetta/G.Balsamo May 2009 Add lai
!     S. Boussetta/G.Balsamo 04-06-2009 2009 Added exp function for crops RCOV
!     E. Dutta             16-11-2009  snow 2009 cleaning
!     F. Catalano A. Alessandri ENEA    01/2016     Vegetation cover dependence on LAI
!     F. Catalano A. Alessandri ENEA    05/2016     Desert albedo estimated from MODIS
!     F. Catalano A. Alessandri ENEA    10/2017     Four components soil albedo estimated from MODIS
!     ------------------------------------------------------------------
#endif

IMPLICIT NONE

! Declaration of arguments

INTEGER(KIND=JPIM), INTENT(IN)  :: KIDIA
INTEGER(KIND=JPIM), INTENT(IN)  :: KFDIA
INTEGER(KIND=JPIM), INTENT(IN)  :: KLON
INTEGER(KIND=JPIM), INTENT(IN)  :: KTILES

REAL(KIND=JPRB),    INTENT(IN)  :: PTVL(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PTVH(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PSOTY(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PSOALUVP(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PSOALUVD(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PSOALNIP(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PSOALNID(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PSDOR(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PCVLC(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PCVHC(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PLAILC(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PLAIHC(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PLSM(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PCI(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PGEMU(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PSNM1M(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PWLM1M(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PRSNM1M(:)
REAL(KIND=JPRB),    INTENT(IN)  :: PCLAKE(:) 
REAL(KIND=JPRB),    INTENT(IN)  :: PHLICE(:)

LOGICAL,   INTENT(OUT) :: LDLAND(:)
LOGICAL,   INTENT(OUT) :: LDSICE(:)
LOGICAL,   INTENT(OUT) :: LDNH(:)
LOGICAL,   INTENT(OUT) :: LDLAKE(:)  
LOGICAL,   INTENT(OUT) :: LDOCN_KPP(:) 

INTEGER(KIND=JPIM), INTENT(OUT) :: KTVL(:)
INTEGER(KIND=JPIM), INTENT(OUT) :: KTVH(:)
INTEGER(KIND=JPIM), INTENT(OUT) :: KSOTY(:)

REAL(KIND=JPRB),    INTENT(INOUT) :: PCVL(:)
REAL(KIND=JPRB),    INTENT(INOUT) :: PCVH(:)
REAL(KIND=JPRB),    INTENT(INOUT) :: PLAIL(:)
REAL(KIND=JPRB),    INTENT(INOUT) :: PLAIH(:)
REAL(KIND=JPRB),    INTENT(OUT) :: PWLMX(:)
REAL(KIND=JPRB),    INTENT(OUT) :: PFRTI(:,:)

REAL(KIND=JPRB),    INTENT(OUT)  :: PALBF(:)
REAL(KIND=JPRB),    INTENT(OUT)  :: PALUVP(KLON),PALUVD(KLON)
REAL(KIND=JPRB),    INTENT(OUT)  :: PALNIP(KLON),PALNID(KLON)

REAL(KIND=JPRB) ::    ZCVW(KLON)   ,ZCVS(KLON) ,ZFRAUX(KLON)
REAL(KIND=JPRB) ::    ZLICE(KLON), ZPCVL,ZPCVH,ZPCVB,ZPCVLK

INTEGER(KIND=JPIM) :: JL,JT
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!         Convert types to integer

IF (LHOOK) CALL DR_HOOK('SURFBC_CTL_MOD:SURFBC_CTL',0,ZHOOK_HANDLE)
DO JL=KIDIA,KFDIA

  ! Note: PTVH & PTVL will hold the vegetation types from LPJ-GUESS, if LLPJGFB .TRUE.
  KTVL(JL)=NINT(PTVL(JL))
  KTVH(JL)=NINT(PTVH(JL))

  KSOTY(JL)=NINT(PSOTY(JL))
ENDDO

!*      FIND LAKE POINTS WITH ICE COVER
DO JL=KIDIA,KFDIA
  ZLICE(JL)=0.0_JPRB !*--> DECOUPLE SNOW DECK FROM LAKE ICE   
ENDDO
 
!         Overwrite LAI fields with look-up table values if LELAIV = .F.


! Use neither MODIS LAI fields nor Table values when LPJ-GUESS LAI is read  
! PLAIL and PLAIH have already been updated by LPJ-GUESS in updclie.F90
IF(.NOT.LLPJGFB)THEN
  IF(LELAIV)THEN
    DO JL=KIDIA,KFDIA
      PLAIL(JL)=PLAILC(JL)
      PLAIH(JL)=PLAIHC(JL)
    ENDDO
  ELSE
    DO JL=KIDIA,KFDIA
      PLAIL(JL)=RVLAI(KTVL(JL))
      PLAIH(JL)=RVLAI(KTVH(JL))
    ENDDO
  ENDIF
ELSE
  ! is this needed?
  DO JL=KIDIA,KFDIA
    PLAIL(JL)=PLAILC(JL)
    PLAIH(JL)=PLAIHC(JL)
  ENDDO
ENDIF


!         Correct fractions dependent on type

SELECT CASE (N_COMPUTE_EFF_VEG_FRACTION)
CASE (1)
  ! Use cover fractions from file, adjusted with Table values
  DO JL=KIDIA,KFDIA
    PCVL(JL)=PCVLC(JL)*RVCOV(KTVL(JL))
    PCVH(JL)=PCVHC(JL)*RVCOV(KTVH(JL))
  ENDDO
CASE (2)
  ! Vegetation fractional cover dependence on LAI
  DO JL=KIDIA,KFDIA
    PCVL(JL)=PCVLC(JL)*((1.0-EXP(-KCVL*PLAIL(JL)))**ALCVL)
    PCVH(JL)=PCVHC(JL)*((1.0-EXP(-KCVH*PLAIH(JL)))**ALCVH)
  ENDDO
CASE DEFAULT
  DO JL=KIDIA,KFDIA
    PCVH(JL)=MIN(1.0_JPRB,PCVHC(JL))
    PCVL(JL)=MIN(1.0_JPRB,PCVLC(JL))
  ENDDO
ENDSELECT

!         Miscellaneous fields needed elsewhere
DO JL=KIDIA,KFDIA
  LDLAND(JL)=PLSM(JL) > 0.5_JPRB
  LDLAKE(JL)= LEFLAKE .AND. PCLAKE(JL) > 0.5_JPRB*(1.0_JPRB - PLSM(JL) ) 
  LDSICE(JL)=( .NOT.LDLAND(JL) .AND. .NOT.LDLAKE(JL) ) .AND.PCI(JL) > RCIMIN   
  LDNH(JL)=PGEMU(JL) > 0.0_JPRB
  IF (.NOT.LESN09) THEN 
    ZCVS(JL)=MAX(0._JPRB,MIN(1.0_JPRB,PSNM1M(JL)*RQSNCR))
  ELSE
    ZCVS(JL)=MAX(0._JPRB,MIN(1.0_JPRB,(100.0_JPRB*(PSNM1M(JL)/PRSNM1M(JL))*RQSNCR)))
  ENDIF
  PWLMX(JL)=RWLMAX*(1.0_JPRB-PCVL(JL)-PCVH(JL)&
   & +PCVL(JL)*PLAIL(JL)&
   & +PCVH(JL)*PLAIH(JL))
  ZCVW(JL)=MAX(0._JPRB,MIN(1.0_JPRB,PWLM1M(JL)/PWLMX(JL)))
  LDOCN_KPP(JL) = LEOCML .AND. ( .NOT. LDLAND(JL) .AND. .NOT. LDLAKE(JL) ) 
ENDDO

!        Define surface fractions
DO JT=1,KTILES
  DO JL=KIDIA,KFDIA
    PFRTI(JL,JT)=0.0_JPRB
  ENDDO
ENDDO



IF (LEFLAKE) THEN

 DO JL=KIDIA,KFDIA
    IF (.NOT. LDLAND(JL) .AND. .NOT. LDLAKE(JL) ) THEN 
      IF (LDSICE(JL)) THEN
        PFRTI(JL,2)=MAX(RCIMIN,PCI(JL))
        PFRTI(JL,1)=1.0_JPRB-PFRTI(JL,2)
      ELSE
        PFRTI(JL,1)=1.0_JPRB
      ENDIF
    ELSE
      IF ( LDLAKE(JL) .AND. .NOT. LDLAND(JL) ) THEN
        ZPCVLK=1.0_JPRB
      ELSEIF (LDLAKE(JL)) THEN
        ZPCVLK=PCLAKE(JL)
      ELSE
        ZPCVLK=0.0_JPRB
      ENDIF
      
      ZPCVL=PCVL(JL)*(1.0_JPRB-ZPCVLK)
      ZPCVH=PCVH(JL)*(1.0_JPRB-ZPCVLK)
      ZPCVB=1.0_JPRB-ZPCVL-ZPCVH-ZPCVLK

      PFRTI(JL,3)=ZCVW(JL)*(1.0_JPRB-ZCVS(JL))*(ZPCVL+ZPCVH+ZPCVB)
      PFRTI(JL,4)=ZPCVL*(1.0_JPRB-ZCVS(JL))*(1.0_JPRB-ZCVW(JL))
      PFRTI(JL,5)=ZCVS(JL)*(ZPCVB+ZPCVL+ZPCVLK*ZLICE(JL))
      PFRTI(JL,6)=ZPCVH*(1.0_JPRB-ZCVS(JL))*(1.0_JPRB-ZCVW(JL))
      PFRTI(JL,7)=ZCVS(JL)*ZPCVH
      PFRTI(JL,8)=ZPCVB*(1.0_JPRB-ZCVS(JL))*(1.0_JPRB-ZCVW(JL))
      PFRTI(JL,9)=ZPCVLK*(1.0_JPRB-ZCVS(JL)*ZLICE(JL))

  ENDIF
ENDDO
ELSE
! For bit-reproducibility the code needs to be duplicated
 DO JL=KIDIA,KFDIA
  IF (.NOT. LDLAND(JL)) THEN
    IF (LDSICE(JL)) THEN
      PFRTI(JL,2)=MAX(RCIMIN,PCI(JL))
      PFRTI(JL,1)=1.0_JPRB-PFRTI(JL,2)
    ELSE
      PFRTI(JL,1)=1.0_JPRB
    ENDIF
  ELSE
    PFRTI(JL,5)=ZCVS(JL)*(1.0_JPRB-PCVH(JL))
    PFRTI(JL,7)=ZCVS(JL)*PCVH(JL)
    PFRTI(JL,3)=(1.0_JPRB-ZCVS(JL))*ZCVW(JL)
    PFRTI(JL,4)=PCVL(JL)*(1.0_JPRB-ZCVS(JL))*(1.0_JPRB-ZCVW(JL))
    PFRTI(JL,6)=PCVH(JL)*(1.0_JPRB-ZCVS(JL))*(1.0_JPRB-ZCVW(JL))
    PFRTI(JL,8)=(1.0_JPRB-PCVL(JL)-PCVH(JL))*(1.0_JPRB-ZCVS(JL))*(1.0_JPRB-ZCVW(JL))
  ENDIF
 ENDDO
ENDIF


! FRACTIONS OF TILES ABOVE A THRESHOLD, NORMALIZED TO SUM 1

IF (RTHRFRTI > 0._JPRB) THEN
  ZFRAUX(KIDIA:KFDIA)=0._JPRB

  DO JT=1,KTILES
    DO JL=KIDIA,KFDIA
      IF (PFRTI(JL,JT) < RTHRFRTI) THEN
        PFRTI(JL,JT)=0._JPRB
      ENDIF
      ZFRAUX(JL)=ZFRAUX(JL)+PFRTI(JL,JT)
    ENDDO
  ENDDO
  DO JT=1,KTILES
    DO JL=KIDIA,KFDIA
      PFRTI(JL,JT)=PFRTI(JL,JT)/ZFRAUX(JL)
    ENDDO
  ENDDO
ENDIF

IF (.NOT.LRDALB) THEN
! update albedo 
    DO JL=KIDIA,KFDIA
      IF (LDLAND(JL)) THEN
       ZPCVL=PCVL(JL)
       ZPCVH=PCVH(JL)
       ZPCVB=1.0_JPRB-ZPCVL-ZPCVH
       PALUVP(JL)=RVVEGALB(KTVL(JL),1)*ZPCVL+&
         & RVVEGALB(KTVH(JL),1)*ZPCVH+&
         & PSOALUVP(JL)*ZPCVB
       PALUVD(JL)=RVVEGALB(KTVL(JL),2)*ZPCVL+&
         & RVVEGALB(KTVH(JL),2)*ZPCVH+&
         & PSOALUVD(JL)*ZPCVB
       PALNIP(JL)=RVVEGALB(KTVL(JL),3)*ZPCVL+&
         & RVVEGALB(KTVH(JL),3)*ZPCVH+&
         & PSOALNIP(JL)*ZPCVB
       PALNID(JL)=RVVEGALB(KTVL(JL),4)*ZPCVL+&
         & RVVEGALB(KTVH(JL),4)*ZPCVH+&
         & PSOALNID(JL)*ZPCVB
       PALBF(JL)=0.459760*PALUVD(JL)+0.540240*PALNID(JL)
      ENDIF
    ENDDO
ENDIF

IF (LHOOK) CALL DR_HOOK('SURFBC_CTL_MOD:SURFBC_CTL',1,ZHOOK_HANDLE)

END SUBROUTINE SURFBC_CTL
END MODULE SURFBC_CTL_MOD
