#include <cassert>
#include "node.h"
#include "timer.hpp"
#include "circle.hpp"
#include "meshutil.h"
#include "polyg.h"
#include "grid.h"
#include "intersect.h"
#include "errhandle.h"
#include "misc.h"

#include "parallel_tree.hpp"


static void packElement(Elt *ptElement, char *buffer, int& index)
{
	if (buffer == NULL) index += packedPolygonSize(*ptElement);
	else packPolygon(*ptElement, buffer, index);
}

static void unpackElement(Elt *ptElement, char *buffer, int& index)
{
	unpackPolygon(*ptElement, buffer, index);
}

void CParallelTree::buildLocalTree(vector<Node>& node)
{
	vector<int> route(node.size());
	for (int i = 0; i < node.size(); i++) route[i] = node[i].route;

	CMPIRouting MPIRoute(MPI_COMM_WORLD);
	MPI_Barrier(MPI_COMM_WORLD);
	CTimer::get("buildLocalTree(initRoute)").resume();
	MPIRoute.init(&route[0], route.size());
	CTimer::get("buildLocalTree(initRoute)").suspend();
	CTimer::get("buildLocalTree(initRoute)").print();

	nbLocalElements = MPIRoute.getTotalSourceElement();
	localElements = new Elt[nbLocalElements];
	localTree.leafs.resize(nbLocalElements);

	vector<Elt*> ptElement(node.size());
	for (int i = 0; i < node.size(); i++)
		ptElement[i] = (Elt *) (node[i].data);

	vector<Elt*> ptLocalElement(nbLocalElements);
	for (int i = 0; i < nbLocalElements; i++)
		ptLocalElement[i] = &localElements[i];

	CTimer::get("buildLocalTree(transfert)").resume();
	MPIRoute.transferToTarget(&ptElement[0], &ptLocalElement[0], packElement, unpackElement);
	CTimer::get("buildLocalTree(transfert)").suspend();
	CTimer::get("buildLocalTree(transfert)").print();


	CTimer::get("buildLocalTree(local)").resume();

	for (int i = 0; i < nbLocalElements; i++) localElements[i].id.ind = i;

	initCircles(localElements, localTree.leafs); /* FIXME pass vector or better replace completely */
	assignId(localTree.leafs); /* FIXME this actually assigns IDs to Elements! make clear and/or assign somewhere else */

	localTree.build(localTree.leafs); /* FIXME pass vector */

	cptAllEltsGeom(localElements, nbLocalElements, srcGrid.pole);
	CTimer::get("buildLocalTree(local)").suspend();
	CTimer::get("buildLocalTree(local)").print();
}

void CParallelTree::buildParallelTree(vector<Node>& node, vector<Node>& node2)
{

	int assignLevel = 2;
	int nbSampleNodes = ipow(MAX_NODE_SZ + 1, assignLevel);

	// make multiple of two
	nbSampleNodes /= 2;
	nbSampleNodes *= 2;

	assert(node.size() > nbSampleNodes);
	assert(node2.size() > nbSampleNodes);
	vector<Node> sampleNodes; sampleNodes.reserve(nbSampleNodes);

	vector<int> randomArray1(node.size());
	randomizeArray(randomArray1);
	vector<int> randomArray2(node2.size());
	randomizeArray(randomArray2);

	for (int i = 0; i < nbSampleNodes/2; i++)
	{
		sampleNodes.push_back(Node(node[randomArray1[i]].centre,  node[randomArray1[i]].radius, NULL));
		sampleNodes.push_back(Node(node2[randomArray2[i]].centre, node2[randomArray2[i]].radius, NULL));
	}

	//build parallel sample tree
	CTimer::get("buildParallelSampleTree").resume();
	sampleTree.buildParallelSampleTree(sampleNodes);
	CTimer::get("buildParallelSampleTree").suspend();
	CTimer::get("buildParallelSampleTree").print();

	//route source mesh
	CTimer::get("parallelRouteNode").resume();
	sampleTree.parallelRouteNode(node);
	CTimer::get("parallelRouteNode").suspend();
	CTimer::get("parallelRouteNode").print();

	CTimer::get("buildLocalTree").resume();
	buildLocalTree(node);
	CTimer::get("buildLocalTree").suspend();
	CTimer::get("buildLocalTree").print();

	CTimer::get("buildRouteTree").resume();
	buildRouteTree();
	CTimer::get("buildRouteTree").suspend();
	CTimer::get("buildRouteTree").print();
}

void CParallelTree::assignId(vector<Node>& node)
{
	int mpiRank;
	MPI_Comm_rank(MPI_COMM_WORLD, &mpiRank);

	for (int j = 0; j < node.size(); j++)
	{
		Elt *elt = (Elt *) (node[j].data);
		elt->id.ind = j;
		elt->id.rank = mpiRank;
	}

}

/** Fills array neighbourElements which is used to compute gradients */
void CParallelTree::getNeighbours()
{
	int mpiSize, mpiRank;
	MPI_Comm_size(MPI_COMM_WORLD, &mpiSize);
	MPI_Comm_rank(MPI_COMM_WORLD, &mpiRank);

	Elt *elt;

	vector<Node> *routingList = new vector<Node>[mpiSize];

	for (int j = 0; j < localTree.leafs.size(); j++)
		routeTree.root->routingIntersecting(routingList, &(localTree.leafs[j]));
	routingList[mpiRank].clear();

	int *nbSendNode = new int[mpiSize];
	int *nbRecvNode = new int[mpiSize];
	int *sendMessageSize = new int[mpiSize];
	int *recvMessageSize = new int[mpiSize];

	for (int rank = 0; rank < mpiSize; rank++)
	{
		nbSendNode[rank] = routingList[rank].size();
		sendMessageSize[rank] = 0;
		for (size_t j = 0; j < routingList[rank].size(); j++)
		{
			elt = (Elt *) (routingList[rank][j].data);
			sendMessageSize[rank] += packedPolygonSize(*elt);
		}
	}

	MPI_Alltoall(nbSendNode, 1, MPI_INT, nbRecvNode, 1, MPI_INT, MPI_COMM_WORLD);
	MPI_Alltoall(sendMessageSize, 1, MPI_INT, recvMessageSize, 1, MPI_INT, MPI_COMM_WORLD);

	char **sendBuffer = new char*[mpiSize];
	char **recvBuffer = new char*[mpiSize];
	int *pos = new int[mpiSize];

	for (int rank = 0; rank < mpiSize; rank++)
	{
		if (nbSendNode[rank] > 0) sendBuffer[rank] = new char[sendMessageSize[rank]];
		if (nbRecvNode[rank] > 0) recvBuffer[rank] = new char[recvMessageSize[rank]];
	}

	for (int rank = 0; rank < mpiSize; rank++)
	{
		pos[rank] = 0;
		for (size_t j = 0; j < routingList[rank].size(); j++)
		{
			elt = (Elt *) (routingList[rank][j].data);
			packPolygon(*elt, sendBuffer[rank], pos[rank]);
		}
	}
	delete [] routingList;


	int nbSendRequest = 0;
	int nbRecvRequest = 0;
	MPI_Request *sendRequest = new MPI_Request[mpiSize];
	MPI_Request *recvRequest = new MPI_Request[mpiSize];
	MPI_Status  *status      = new MPI_Status[mpiSize];

	for (int rank = 0; rank < mpiSize; rank++)
	{
		if (nbSendNode[rank] > 0)
		{
			MPI_Issend(sendBuffer[rank], sendMessageSize[rank], MPI_CHAR, rank, 0, MPI_COMM_WORLD, &sendRequest[nbSendRequest]);
			nbSendRequest++;
		}
		if (nbRecvNode[rank] > 0)
		{
			MPI_Irecv(recvBuffer[rank], recvMessageSize[rank], MPI_CHAR, rank, 0, MPI_COMM_WORLD, &recvRequest[nbRecvRequest]);
			nbRecvRequest++;
		}
	}

	MPI_Waitall(nbRecvRequest, recvRequest, status);
	MPI_Waitall(nbSendRequest, sendRequest, status);

	for (int rank = 0; rank < mpiSize; rank++)
		if (nbSendNode[rank] > 0) delete [] sendBuffer[rank];
	delete [] sendBuffer;

	char **sendBuffer2 = new char*[mpiSize];
	char **recvBuffer2 = new char*[mpiSize];

	Node recvNode;
	Elt recvElt;

	for (int rank = 0; rank < mpiSize; rank++)
	{
		nbSendNode[rank] = 0;
		sendMessageSize[rank] = 0;

		if (nbRecvNode[rank] > 0)
		{
			set<NodePtr> neighbourList;
			pos[rank] = 0;
			for (int j = 0; j < nbRecvNode[rank]; j++)
			{
				unpackPolygon(recvElt, recvBuffer[rank], pos[rank]);
				initCircle(recvElt, recvNode);
				findNeighbour(localTree.root, &recvNode, neighbourList);
			}
			nbSendNode[rank] = neighbourList.size();
			for (set<NodePtr>::iterator it = neighbourList.begin(); it != neighbourList.end(); it++)
			{
				Elt *elt = (Elt *) ((*it)->data);
				sendMessageSize[rank] += packedPolygonSize(*elt);
			}

			sendBuffer2[rank] = new char[sendMessageSize[rank]];
			pos[rank] = 0;

			for (set<NodePtr>::iterator it = neighbourList.begin(); it != neighbourList.end(); it++)
			{
				Elt *elt = (Elt *) ((*it)->data);
				packPolygon(*elt, sendBuffer2[rank], pos[rank]);
			}
		}
	}
	for (int rank = 0; rank < mpiSize; rank++)
		if (nbRecvNode[rank] > 0) delete [] recvBuffer[rank];
	delete [] recvBuffer;


	MPI_Barrier(MPI_COMM_WORLD);
	MPI_Alltoall(nbSendNode, 1, MPI_INT, nbRecvNode, 1, MPI_INT, MPI_COMM_WORLD);
	MPI_Alltoall(sendMessageSize, 1, MPI_INT, recvMessageSize, 1, MPI_INT, MPI_COMM_WORLD);

	for (int rank = 0; rank < mpiSize; rank++)
		if (nbRecvNode[rank] > 0) recvBuffer2[rank] = new char[recvMessageSize[rank]];

	nbSendRequest = 0;
	nbRecvRequest = 0;

	for (int rank = 0; rank < mpiSize; rank++)
	{
		if (nbSendNode[rank] > 0)
		{
			MPI_Issend(sendBuffer2[rank], sendMessageSize[rank], MPI_CHAR, rank, 0, MPI_COMM_WORLD, &sendRequest[nbSendRequest]);
			nbSendRequest++;
		}
		if (nbRecvNode[rank] > 0)
		{
			MPI_Irecv(recvBuffer2[rank], recvMessageSize[rank], MPI_CHAR, rank, 0, MPI_COMM_WORLD, &recvRequest[nbRecvRequest]);
			nbRecvRequest++;
		}
	}

	MPI_Waitall(nbRecvRequest, recvRequest, status);
	MPI_Waitall(nbSendRequest, sendRequest, status);

	int nbNeighbourNodes = 0;
	for (int rank = 0; rank < mpiSize; rank++)
		nbNeighbourNodes += nbRecvNode[rank];

	neighbourElements = new Elt[nbNeighbourNodes];
	neighbourNodes.resize(nbNeighbourNodes);
	nbNeighbourElements = nbNeighbourNodes;

	int index = 0;
	for (int rank = 0; rank < mpiSize; rank++)
	{
		pos[rank] = 0;
		for (int j = 0; j < nbRecvNode[rank]; j++)
		{
			unpackPolygon(neighbourElements[index], recvBuffer2[rank], pos[rank]);
			neighbourElements[index].id.ind = localTree.leafs.size() + index;
			index++;
		}
	}
	for (int rank = 0; rank < mpiSize; rank++)
	{
		if (nbRecvNode[rank] > 0) delete [] recvBuffer2[rank];
		if (nbRecvNode[rank] > 0) delete [] sendBuffer2[rank];
	}
	delete [] recvBuffer2;
	delete [] sendBuffer2;
	delete [] sendMessageSize;
	delete [] recvMessageSize;
	delete [] nbSendNode;
	delete [] nbRecvNode;
	delete [] sendRequest;
	delete [] recvRequest;
	delete [] status;
	delete [] pos;

	/* re-compute on received elements to avoid having to send this information */
	initCircles(neighbourElements, neighbourNodes);
	cptAllEltsGeom(neighbourElements, nbNeighbourNodes, srcGrid.pole);

	/* the local SS tree must include nodes from other cpus if they are potential 
           intersector of a local node */
	localTree.insertNodes(neighbourNodes); /* FIXME EVIL! how does this get deallocated??*/

	/* for every local element,
           use the SS-tree to find all elements (including neighbourElements) 
           who are potential neighbours because their circles intersect,
	   then check all canditates for common edges to build up connectivity information
	*/
	assert(localTree.leafs.size() == localTree.levelSize[0]);
	for (int j = 0; j < localTree.leafs.size(); j++)
	{
		Node& node = localTree.leafs[j];

		/* find all leafs whoes circles that intersect node's circle and save into node->intersectors */
		node.search(localTree.root);

		Elt *elt = (Elt *)(node.data);

		/* FIXME: move this initialization into Elt constructor */
		for (int i = 0; i < elt->n; i++) elt->neighbour[i] = NOT_FOUND;

		/* for element `elt` loop through all nodes in the SS-tree
                   whoes circles intersect with the circle around `elt` (the SS intersectors)
                   and check if they are neighbours in the sense that the two elements share an edge.
                   If they do, save this information for elt (but not for elt2
                   FIXME: which means that twice as many neighbour checks as necessary take place */
		for (list<NodePtr>::iterator it = (node.intersectors).begin(); it != (node.intersectors).end(); ++it)
		{
			Elt *elt2 = (Elt *)((*it)->data);
			set_neighbour(*elt, *elt2);
		}

		for (int i = 0; i < elt->n; i++)
		{
			if (elt->neighbour[i] == NOT_FOUND)
				error_exit("neighbour not found");
		}
	}
}

void CParallelTree::buildRouteTree()
{
	int mpiSize;
	MPI_Comm_size(MPI_COMM_WORLD, &mpiSize);

	double *newRoutingRadius = new double[mpiSize];
	Coord *newRoutingCenter = new Coord[mpiSize];

	routeTree.leafs.resize(mpiSize);
	MPI_Allgather(&(localTree.root->radius), 1, MPI_DOUBLE, newRoutingRadius, 1, MPI_DOUBLE, MPI_COMM_WORLD);
	MPI_Allgather(&(localTree.root->centre), 3, MPI_DOUBLE, newRoutingCenter, 3, MPI_DOUBLE, MPI_COMM_WORLD);
	for (int i = 0; i < mpiSize; i++)
	{
		routeTree.leafs[i].init(newRoutingCenter[i], newRoutingRadius[i]);
		routeTree.leafs[i].route = i;
	}
	routeTree.build(routeTree.leafs);

	delete [] newRoutingRadius;
	delete [] newRoutingCenter;
}

CParallelTree::~CParallelTree()
{
	delete [] localElements;
	delete [] neighbourElements;
}
