#include <cassert>
#include "node.h"
#include "misc.h"
#include "timer.hpp"

#include "sample_tree.hpp"

CSampleTree::CSampleTree(MPI_Comm comm) : communicator(comm), sampleTree(NULL)
{
	MPI_Comm_size(communicator, &mpiSize);
	MPI_Comm_rank(communicator, &mpiRank);
}


// only the circle is packed, rest of node will be initialized on unpacking
static void packNode(Node& node, char *buffer, int& index)
{
	if (buffer == NULL) // compute size only
		index += 4 * sizeof(double);
	else
	{
		*(Coord *)(&buffer[index]) = node.centre;
		index += sizeof(Coord);

		*(double *)(&buffer[index]) = node.radius;
		index += sizeof(double);
	}
}

static void unpackNode(Node& node, char *buffer, int& index)
{
	Coord centre;
	double r;

	centre = *(Coord *)(&buffer[index]);
	index += sizeof(Coord);

	r = *(double *)(&buffer[index]);
	index += sizeof(double);

	node.init(centre, r);
}

static void transfertRoutedNodes(CMPIRouting& MPIRoute, vector<Node> nodeSend, int *route, vector<Node>& nodeRecv)
{
	MPIRoute.init(route, nodeSend.size());
	int nRecv = MPIRoute.getTotalSourceElement();
	nodeRecv.resize(nRecv);
	MPIRoute.transferToTarget(&nodeSend[0], &nodeRecv[0], packNode, unpackNode);
}


/* find number of levels of cpu-cores 
   so that we can build a hierarchy of cpu-cores to guarantee good parallel scaling */
static int cptKeepNodes(bool& hasChildTree, int mpiSize)
{
	int keepNodes = ((2 * MAX_NODE_SZ + MIN_NODE_SZ) / 3)
                      * ((2 * MAX_NODE_SZ + MIN_NODE_SZ) / 3); // nodes to keep by (2/3 M + 1/3 m)**2 = (desired number of children)**2
	int levels = 0;
	int size = keepNodes;
	while (size <= mpiSize)
	{
		levels++;
		size *= keepNodes;
	} // => keepNodes**levels = size > mpiSize
	keepNodes = pow(mpiSize, 1./(levels + 1));
	hasChildTree = (levels > 0); // levels == 0 if initial size=keepNodes was smaller than mpiSize, in that case keepNodes = mpiSize**1
	return keepNodes;
}

static void initRouting(int rank, int size, int keepNodes, int *routeRanks)
{
	int start = (rank / keepNodes) * keepNodes;

	for (int i = 0; i < keepNodes; i++)
	{
		routeRanks[i] = start;
		start++;
		if (start > size) start -= keepNodes;
	}
}

void CSampleTree::buildParallelSampleTree(vector<Node>& sampleNodes)
{
	// keepNodes is the desired number of cpu-cores per ?group?
	int keepNodes = cptKeepNodes(hasChildTree, mpiSize); // sets hasChildTree

	buildSampleTree(keepNodes, sampleNodes, communicator);

	// assign route
	int *routeRank = new int[keepNodes];
	initRouting(mpiRank, mpiSize, keepNodes, routeRank);
	const int assignLevel = 2;
	assignRoute(routeRank, assignLevel);
	delete [] routeRank;

	if (hasChildTree)
	{
		int color = mpiRank % keepNodes; // which group are we?
		MPI_Comm intraComm;
		MPI_Comm_split(communicator, color, mpiRank, &intraComm);
		sampleTree = new CSampleTree(intraComm);

		routeNodes(sampleNodes, assignLevel);
		vector<int> route(sampleNodes.size());
		for (int i = 0; i < sampleNodes.size(); i++) route[i] = sampleNodes[i].route;

		vector<Node> routedNodes;
		CMPIRouting MPIRoute(communicator);
		transfertRoutedNodes(MPIRoute, sampleNodes, &route[0], routedNodes);
		sampleTree->buildParallelSampleTree(routedNodes);
	}
}

/**
	@param keepNodes is the number of cpu-cores in one group
*/
void CSampleTree::buildSampleTree(int keepNodes, vector<Node>& node, MPI_Comm communicator)
/*
	In the beginning all the sample elements are distributed
	-> communicate to make available at each rank
	   so that each rank can build the same sample tree
*/
{
	int assignLevel = 2;
	int n = node.size(); // number of samples on this rank (local)

	int blocSize = keepNodes * ipow(MAX_NODE_SZ, assignLevel);

	int nrecv; // global number of samples
	MPI_Allreduce(&n, &nrecv, 1, MPI_INT, MPI_SUM, communicator);
	double ratio = blocSize / (1.0 * nrecv);
	int nsend = ratio * n + 1; // nsend = n_local_samples / n_global_samples * blocksize + 1 = blocksize/mpiSize
	if (nsend > n) nsend = n;

	int *counts = new int[mpiSize];
	MPI_Allgather(&nsend, 1, MPI_INT, counts, 1, MPI_INT, communicator);

	nrecv = 0;
	int *displs = new int[mpiSize];
	for (int i = 0; i < mpiSize; i++)
	{
		displs[i] = 4 * nrecv;
		nrecv += counts[i];
		counts[i] = 4 * counts[i];
	}

	/* pack circles around sample elements */
	double *sendBuffer = new double[nsend*4];
	int index = 0;
	vector<int> randomArray(n);
	randomizeArray(randomArray);
	for (int i = 0; i < nsend; i++)
	{
		Node& no = node[randomArray[i]];
		*((Coord *) (sendBuffer + index)) = no.centre;
		index += sizeof(Coord)/sizeof(*sendBuffer);
		sendBuffer[index++] = no.radius;
	}

	/* each process needs the sample elements from all processes */
	double *recvBuffer = new double[nrecv*4];
	MPI_Allgatherv(sendBuffer, 4 * nsend, MPI_DOUBLE, recvBuffer, counts, displs, MPI_DOUBLE, communicator);
	delete[] sendBuffer;
	delete[] counts;
	delete[] displs;

	/* unpack */
	randomArray.resize(nrecv);
	randomizeArray(randomArray);
	leafs.resize(nrecv);
	index = 0;
	for (int i = 0; i < nrecv; i++)
	{
		Coord x = *(Coord *)(&recvBuffer[index]);
		index += sizeof(Coord)/sizeof(*recvBuffer);
		double radius = recvBuffer[index++];
		leafs[randomArray[i]].init(x, radius);

	}

	delete [] recvBuffer;

	CTimer::get("buildSampleTree(local)").resume();
	build(leafs, true, assignLevel, keepNodes);
	CTimer::get("buildSampleTree(local)").suspend();
	CTimer::get("buildSampleTree(local)").print();
	assert(root->incluCheck() == 0);
}

void CSampleTree::parallelRouteNode(vector<Node>& node)
{
	int assignLevel = 2;

	routeNodes(node, assignLevel);

	vector<int> route(node.size()); // temporary, get out of scope outside
	for (int i = 0; i < node.size(); i++)
		route[i] = node[i].route;

	if (hasChildTree)
	{
		vector<Node> routedNodes;

		CMPIRouting MPIRoute(communicator);
		transfertRoutedNodes(MPIRoute, node, &route[0], routedNodes);
		sampleTree->parallelRouteNode(routedNodes);

		vector<int> globalRank(routedNodes.size());
		for (int i = 0; i < globalRank.size(); i++)
			globalRank[i] = routedNodes[i].route;
		MPIRoute.transferFromSource(&route[0], &globalRank[0]);
		for (int i = 0; i < node.size(); i++)
			node[i].route = route[i];
	}
	else
	{
		int rank;
		MPI_Comm_rank(MPI_COMM_WORLD, &rank);

		CMPIRouting MPIRoute(communicator);
		MPIRoute.init(&route[0], route.size());

		int nbRecvNode = MPIRoute.getTotalSourceElement();
		vector<int> globalRank(nbRecvNode);
		for (int i = 0; i < globalRank.size(); i++)
			globalRank[i] = rank;
		MPIRoute.transferFromSource(&route[0], &globalRank[0]);
		for (int i = 0; i < node.size(); i++)
			node[i].route = route[i];
	}
}

void CSampleTree::assignRoute(int *rank, int assignLevel)  // newroot <- root
{
	root->assignRoute(rank, assignLevel);
}

