#include <cassert>
#include "tree.hpp"
#include "node.h"
#include "elt.h"
#include <iostream>
#include <mpi.h>
#include <stdlib.h>     /* srand, rand */
#include <time.h>       /* time */
#include <vector>
#include <set>

#include "timer.hpp"
#include "mpi_routing.hpp"

static const int MAX_LEVEL_SIZE = 100;

double cputime();
using namespace std;

CBasicTree::CBasicTree() : ri(0), levelSize(MAX_LEVEL_SIZE)
{
}

void CBasicTree::routeNodes(vector<Node>& nodes, int assignLevel)
{
	for (int i = 0; i < nodes.size(); i++)
		root->routeNode(&nodes[i], assignLevel);
}

void CBasicTree::build(vector<Node>& nodes, bool isSampleTree, int assignLevel, int keepNodes)
{
	int level;
	newRoot(1);
	insertNodes(nodes, isSampleTree, assignLevel, keepNodes);

	if (!isSampleTree)
	{
		for (int nbIt = 0; nbIt < 1; nbIt++)
		{
			for (level = root->level - 1; level > 0; level--)
			{
				slim2(root, level);
				ri = 0;
				emptyPool();
			}


			for (level = 2; level < root->level; level++)
			{
				slim2(root, level);
				ri = 0;
				emptyPool();
			}
		}
	}
}

/** FIXME this function installs pointes on elements of array `node` and causes havoc,
   should caller free the array `node` */
void CBasicTree::insertNodes(vector<Node>& nodes, bool isSampleTree, int assignLevel, int keepNodes)
{
	NodePtr q;
	int level;
	int stepSlim;

	bool first = true;
	double ratio;

	if (isSampleTree)
	{
		ratio = 1 + (0.5 * (1 - (keepNodes - 1) / (1.0 * keepNodes)));
		stepSlim = MAX_NODE_SZ * MAX_NODE_SZ * keepNodes / 2;
	}
	else
	{
		ratio = 1.5;
		stepSlim = MAX_NODE_SZ * MAX_NODE_SZ * 2;
	}

	for (int i = 0; i < nodes.size(); i++)
	{

		NodePtr node = &nodes[i];
		node->tree = this;
		increaseLevelSize(0);
		push_back(node);

		while (pool.size())
		{
			q = pool.front();
			pool.pop_front();
			q = insert(q, root);
			if (ri)
			{
				delete q;
				ri = 0;
			}
		}

		if (root->leafCount > stepSlim) // time for slim down
		{
			for (level = root->level - 1; level > 0; level--)
			{
				slim2(root, level);
				ri = 0;
				emptyPool();
			}


			for (level = 2; level < root->level; level++)
			{
				slim2(root, level);
				ri = 0;
				emptyPool();
			}

			stepSlim = stepSlim * ratio;
		}

		if (isSampleTree && levelSize[assignLevel] >= keepNodes && first)
		{
			for (int nbIt = 0; nbIt < 3; nbIt++)
			{
				for (level = root->level - 1; level > 0; level--)
				{
					slim2(root, level);
					ri = 0;
					emptyPool();
				}
				for (level = 2; level < root->level; level++)
				{
					slim2(root, level);
					ri = 0;
					emptyPool();
				}
			}

			first = false;
		}

		if (isSampleTree && levelSize[assignLevel] >= keepNodes)
		{
			if (levelSize[assignLevel] == keepNodes)
			{
				cout << "SampleTree build : nb Node inserted : " << i << endl;
				break;
			}
			else if (levelSize[assignLevel] > keepNodes)
			{
				cout << " PROBLEM !! : node assign > keepNodes : " << levelSize[assignLevel] << " > " << keepNodes << endl;
				abort();
			}
		}
	}
	if (isSampleTree) cout << "SampleTree build : assign Level " << assignLevel << " nb Nodes : " << levelSize[assignLevel] << endl;

}

void CBasicTree::emptyPool(void)
{
	while (pool.size())
	{
		NodePtr q = pool.front();
		pool.pop_front();
		q = insert(q, root);
		if (ri)
		{
			delete q;
			ri = 0;
		}
	}
}

void CBasicTree::push_back(NodePtr node)
{
#ifdef DEBUG
	node->parent = NULL; // will be overridden later, but for mem leak check better trigger leak as soon as possible
#endif
	pool.push_back(node);
}

void CBasicTree::push_front(NodePtr node)
{
#ifdef DEBUG
	node->parent = NULL; // will be overridden later, but for mem leak check better trigger leak as soon as possible
#endif
	pool.push_front(node);
}

void CBasicTree::increaseLevelSize(int level)
{
	levelSize[level]++;
}

void CBasicTree::decreaseLevelSize(int level)
{
	levelSize[level]--;
}

void CBasicTree::newRoot(int level)  // newroot <- root
{
	root = new Node;
	if (level > 1) cout << " " << level << " ";
	root->level = level;
	root->parent = 0;
	root->leafCount = 0;
	root->centre = ORIGIN;
	root->radius = 0.;
	root->reinserted = false;
	root->updateCount = 0;
	root->tree = this;
	levelSize[level]++;
}

CBasicTree::~CBasicTree()
{
	root->free_descendants(); // recursively deletes all nodes in the tree
	delete root;
}
