SUBROUTINE SU_AEROP

!**** *SU_AEROP*   - INITIALIZE MODULES YOEAEROP

!     PURPOSE.
!     --------
!           INITIALIZE YOEAEROP THE MODULE THAT CONTAINS THE
!           OPTICAL PROPERTIES FOR THE PROGNOSTIC AEROSOLS

!**   INTERFACE.
!     ----------
!        *CALL* *SU_AEROP

!        EXPLICIT ARGUMENTS :
!        --------------------
!        NONE

!        IMPLICIT ARGUMENTS :
!        --------------------
!        YOEAEROP

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------

!     REFERENCE.
!     ----------
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE IFS

!     AUTHOR.
!     -------
!        JEAN-JACQUES MORCRETTE *ECMWF*
!        from O.BOUCHER's stand-alone codes (LOA, 2005-03) 

!     MODIFICATIONS.
!     --------------
!        ORIGINAL : 2005-03-25
!        JJMorcrette 20061101 9 bins for SS and DU
!        JJMorcrette 20090312 19 wavelengths (add 2 for potential lidar simulations)
!        JJMorcrette 20090702 aerosol lidar simulator
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOEAEROP  , ONLY : ALF_SU, ASY_SU, OMG_SU, ALF_OM, ASY_OM, OMG_OM, &
  & ALF_DD, ASY_DD, OMG_DD, ALF_SS, ASY_SS, OMG_SS, ALF_BC, ASY_BC, OMG_BC, &
  & ALF_FA, ASY_FA, OMG_FA, &
  & RALI_BC,RALI_DD,RALI_FA,RALI_OM,RALI_SU,RALI_SS

USE YOEAERSRC, ONLY : NTYPAER

USE YOEAERLID, ONLY : JWLID, NWLID, RWLID, RLICLS, RLICO2, RLIO2, RLINO2, RLIO3, RLIDELT, RLINS, &
  & RLIPREF, RLITREF, RLIT0

IMPLICIT NONE

REAL(KIND=JPRB) :: ZALFDD3(3,19), ZASYDD3(3,19), ZOMGDD3(3,19), ZRALID3(3,19)
REAL(KIND=JPRB) :: ZALFDD9(9,19), ZASYDD9(9,19), ZOMGDD9(9,19), ZRALID9(9,19)

REAL(KIND=JPRB) :: ZALFSS3(12,19,3), ZASYSS3(12,19,3), ZOMGSS3(12,19,3), ZRALIS3(12,19,3)
REAL(KIND=JPRB) :: ZALFSS9(12,19,9), ZASYSS9(12,19,9), ZOMGSS9(12,19,9), ZRALIS9(12,19,9)

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!     ----------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SU_AEROP',0,ZHOOK_HANDLE)


!*      1.    PARAMETERS RELATED TO OPTICAL PROPERTIES
!             ----------------------------------------

! All arrays are given in three blocks: 
! first, for 19 wavelengths relevant to observations 
! 340, 355, 380, 400, 440, 469, 500, 532, 555, 645, 670, 800, 858, 865, 1020, 1064, 1240, 1640, 2130 

!(  not done yet, depends whether SW6 or RRTM_SW is used, to be computed for RRTM_LW)
! second, for the spectral intervals of the SW radiation scheme
! third, for the spectral intervals of the LW radiation scheme

! For sulfates (_SU) and organic matters (_OM), values are given as function 
! of RH (12 values) for the 19 wavelengths
! For sea-salt (_SS), 3 bins are considered      (0.03, 0.50, 5.0, 20.)
! For desert dust (_DD), 3 bins are considered   (0.03, 0.55, 0.9, 20.)
! For testing, a 9-bin representation of SS and DD is also available with bin limits at
! 0.03 - 0.05 - 0.10 - 0.20 - 0.50 - 1.00 - 2.00 - 5.00 - 10.00 - 20.00 um

! IF BIN LIMITS ARE CHANGED, MAKE SURE THAT THE RELEVANT SEDIMENTATION SPEEDS ARE 
! RECOMPUTED ACCORDINGLY

! RH  0.         10.         20.         30.         40.         50.
!    60.         70.         80.         85.         90.         95.  



ALF_SS(:,:,:) =0._JPRB
ASY_SS(:,:,:) =0._JPRB
OMG_SS(:,:,:) =1._JPRB
RALI_SS(:,:,:)=0._JPRB

ALF_DD(:,:) =0._JPRB
ASY_DD(:,:) =0._JPRB
OMG_DD(:,:) =1._JPRB
RALI_DD(:,:)=0._JPRB

ALF_OM(:,:) =0._JPRB
ASY_OM(:,:) =0._JPRB
OMG_OM(:,:) =1._JPRB
RALI_OM(:,:)=0._JPRB

ALF_BC(:) =0.0_JPRB
ASY_BC(:) =0.0_JPRB
OMG_BC(:) =1.0_JPRB
RALI_BC(:)=0.0_JPRB

ALF_SU(:,:) =0._JPRB
ASY_SU(:,:) =0._JPRB
OMG_SU(:,:) =1._JPRB
RALI_SU(:,:)=0._JPRB

ALF_FA(:) =0.0_JPRB
ASY_FA(:) =0.0_JPRB
OMG_FA(:) =1.0_JPRB
RALI_FA(:)=0.0_JPRB

!------------------------------------ BLACK CARBON ---------------------------------------------
!-- from O.B.'s black carbon Mie code 20090312	(black_carbon/mie_bc_l_work)

! rmin_dry  rmax_dry   r_0_dry    sigma    rho
! 0.005E-6  0.50E-6   0.0118E-6	    2	 1.000E+3

! flat refractive index: Real 1.750  Imaginary 0.45

! alpha_aer_bc 
ALF_BC( :) = (/ &
  & 0.173328E+02_JPRB, 0.164703E+02_JPRB, 0.151691E+02_JPRB, 0.142372E+02_JPRB, 0.126195E+02_JPRB,&
  & 0.116197E+02_JPRB, 0.106841E+02_JPRB, 0.984008E+01_JPRB, 0.941229E+01_JPRB, 0.759338E+01_JPRB,&
  & 0.721234E+01_JPRB, 0.567656E+01_JPRB, 0.516905E+01_JPRB, 0.511340E+01_JPRB, 0.411645E+01_JPRB,&
  & 0.389768E+01_JPRB, 0.320991E+01_JPRB, 0.228850E+01_JPRB, 0.169767E+01_JPRB /)

! cg_aer_bc
ASY_BC( :) = (/ &
  & 0.431423E+00_JPRB, 0.422625E+00_JPRB, 0.408779E+00_JPRB, 0.398369E+00_JPRB, 0.379103E+00_JPRB,&
  & 0.366273E+00_JPRB, 0.353479E+00_JPRB, 0.341155E+00_JPRB, 0.334579E+00_JPRB, 0.303481E+00_JPRB,&
  & 0.296164E+00_JPRB, 0.262659E+00_JPRB, 0.249735E+00_JPRB, 0.248246E+00_JPRB, 0.218583E+00_JPRB,&
  & 0.211147E+00_JPRB, 0.184765E+00_JPRB, 0.138729E+00_JPRB, 0.985352E-01_JPRB /)

! om_aer_bc
OMG_BC( :) = (/ &
  & 0.300077E+00_JPRB, 0.292289E+00_JPRB, 0.279690E+00_JPRB, 0.269951E+00_JPRB, 0.251376E+00_JPRB,&
  & 0.238648E+00_JPRB, 0.225713E+00_JPRB, 0.213069E+00_JPRB, 0.206263E+00_JPRB, 0.173801E+00_JPRB,&
  & 0.166165E+00_JPRB, 0.131817E+00_JPRB, 0.119060E+00_JPRB, 0.117616E+00_JPRB, 0.901928E-01_JPRB,&
  & 0.837982E-01_JPRB, 0.629933E-01_JPRB, 0.347307E-01_JPRB, 0.183768E-01_JPRB /)

! lidar ratio
RALI_BC( :) = (/ &
  & 0.976533E+02_JPRB, 0.968641E+02_JPRB, 0.959826E+02_JPRB, 0.956180E+02_JPRB, 0.956522E+02_JPRB,&
  & 0.962204E+02_JPRB, 0.972598E+02_JPRB, 0.987489E+02_JPRB, 0.997567E+02_JPRB, 0.106868E+03_JPRB,&
  & 0.109199E+03_JPRB, 0.124116E+03_JPRB, 0.132228E+03_JPRB, 0.133268E+03_JPRB, 0.159612E+03_JPRB,&
  & 0.168265E+03_JPRB, 0.208397E+03_JPRB, 0.336082E+03_JPRB, 0.575476E+03_JPRB /)

!------------------------------------ SULPHATE -------------------------------------------------
! Sulphate optical properties from revised Olivier Boucher's code
!
! "new_ammonium_sulfate" for the real and imaginary parts of the refractive index
! "mie_sul_l_wv_work" for the actual Mie computations

! rmin_dry  rmax_dry   r_0_dry    sigma    rho
! 0.002E-6  20.0E-6   0.0355E-6    2        1.769E+3

! alpha_aer_sulfate
!-- Wavelength=0.3400E-06
ALF_SU( :, 1) = (/ &
 & 0.638406E+01_JPRB,0.638406E+01_JPRB,0.638406E+01_JPRB,0.638406E+01_JPRB,0.821194E+01_JPRB,0.915584E+01_JPRB &
 &,0.103892E+02_JPRB,0.121456E+02_JPRB,0.151294E+02_JPRB,0.177846E+02_JPRB,0.225188E+02_JPRB,0.366296E+02_JPRB /)
!-- Wavelength=0.3550E-06
ALF_SU( :, 2) = (/ &
 & 0.783236E+01_JPRB,0.783236E+01_JPRB,0.783236E+01_JPRB,0.783236E+01_JPRB,0.107086E+02_JPRB,0.119733E+02_JPRB &
 &,0.136231E+02_JPRB,0.159663E+02_JPRB,0.199159E+02_JPRB,0.233918E+02_JPRB,0.295101E+02_JPRB,0.470735E+02_JPRB /)
!-- Wavelength=0.3800E-06
ALF_SU( :, 3) = (/ &
 & 0.723461E+01_JPRB,0.723461E+01_JPRB,0.723461E+01_JPRB,0.723461E+01_JPRB,0.991514E+01_JPRB,0.111105E+02_JPRB &
 &,0.126738E+02_JPRB,0.149009E+02_JPRB,0.186714E+02_JPRB,0.220043E+02_JPRB,0.278991E+02_JPRB,0.449836E+02_JPRB /)
!-- Wavelength=0.4000E-06
ALF_SU( :, 4) = (/ &
 & 0.678337E+01_JPRB,0.678337E+01_JPRB,0.678337E+01_JPRB,0.678337E+01_JPRB,0.931322E+01_JPRB,0.104534E+02_JPRB &
 &,0.119476E+02_JPRB,0.140811E+02_JPRB,0.177052E+02_JPRB,0.209199E+02_JPRB,0.266264E+02_JPRB,0.432887E+02_JPRB /)
!-- Wavelength=0.4400E-06
ALF_SU( :, 5) = (/ &
 & 0.597721E+01_JPRB,0.597721E+01_JPRB,0.597721E+01_JPRB,0.597721E+01_JPRB,0.823880E+01_JPRB,0.927649E+01_JPRB &
 &,0.106412E+02_JPRB,0.125985E+02_JPRB,0.159436E+02_JPRB,0.189296E+02_JPRB,0.242662E+02_JPRB,0.400639E+02_JPRB /)
!-- Wavelength=0.4690E-06
ALF_SU( :, 6) = (/ &
 & 0.544760E+01_JPRB,0.544760E+01_JPRB,0.544760E+01_JPRB,0.544760E+01_JPRB,0.752774E+01_JPRB,0.849378E+01_JPRB &
 &,0.976737E+01_JPRB,0.115994E+02_JPRB,0.147437E+02_JPRB,0.175626E+02_JPRB,0.226241E+02_JPRB,0.377505E+02_JPRB /)
!-- Wavelength=0.5000E-06
ALF_SU( :, 7) = (/ &
 & 0.494247E+01_JPRB,0.494247E+01_JPRB,0.494247E+01_JPRB,0.494247E+01_JPRB,0.684925E+01_JPRB,0.774471E+01_JPRB &
 &,0.892815E+01_JPRB,0.106356E+02_JPRB,0.135782E+02_JPRB,0.162277E+02_JPRB,0.210072E+02_JPRB,0.354261E+02_JPRB /)
!-- Wavelength=0.5320E-06
ALF_SU( :, 8) = (/ &
 & 0.447513E+01_JPRB,0.447513E+01_JPRB,0.447513E+01_JPRB,0.447513E+01_JPRB,0.621964E+01_JPRB,0.704739E+01_JPRB &
 &,0.814394E+01_JPRB,0.973062E+01_JPRB,0.124762E+02_JPRB,0.149585E+02_JPRB,0.194567E+02_JPRB,0.331518E+02_JPRB /)
!-- Wavelength=0.5550E-06
ALF_SU( :, 9) = (/ &
 & 0.416698E+01_JPRB,0.416698E+01_JPRB,0.416698E+01_JPRB,0.416698E+01_JPRB,0.580253E+01_JPRB,0.658407E+01_JPRB &
 &,0.762108E+01_JPRB,0.912458E+01_JPRB,0.117334E+02_JPRB,0.140988E+02_JPRB,0.183985E+02_JPRB,0.315726E+02_JPRB /)
!-- Wavelength=0.6450E-06
ALF_SU( :,10) = (/ &
 & 0.320195E+01_JPRB,0.320195E+01_JPRB,0.320195E+01_JPRB,0.320195E+01_JPRB,0.449887E+01_JPRB,0.513098E+01_JPRB &
 &,0.597454E+01_JPRB,0.720608E+01_JPRB,0.936364E+01_JPRB,0.113393E+02_JPRB,0.149688E+02_JPRB,0.263322E+02_JPRB /)
!-- Wavelength=0.6700E-06
ALF_SU( :,11) = (/ &
 & 0.296104E+01_JPRB,0.296104E+01_JPRB,0.296104E+01_JPRB,0.296104E+01_JPRB,0.416473E+01_JPRB,0.475570E+01_JPRB &
 &,0.554550E+01_JPRB,0.670060E+01_JPRB,0.872933E+01_JPRB,0.105919E+02_JPRB,0.140237E+02_JPRB,0.248335E+02_JPRB /)
!-- Wavelength=0.8000E-06
ALF_SU( :,12) = (/ &
 & 0.208047E+01_JPRB,0.208047E+01_JPRB,0.208047E+01_JPRB,0.208047E+01_JPRB,0.296309E+01_JPRB,0.340425E+01_JPRB &
 &,0.399782E+01_JPRB,0.487304E+01_JPRB,0.642749E+01_JPRB,0.787095E+01_JPRB,0.105635E+02_JPRB,0.192584E+02_JPRB /)
!-- Wavelength=0.8580E-06
ALF_SU( :,13) = (/ &
 & 0.177154E+01_JPRB,0.177154E+01_JPRB,0.177154E+01_JPRB,0.177154E+01_JPRB,0.253109E+01_JPRB,0.291449E+01_JPRB &
 &,0.343168E+01_JPRB,0.419667E+01_JPRB,0.556134E+01_JPRB,0.683437E+01_JPRB,0.922091E+01_JPRB,0.170083E+02_JPRB /)
!-- Wavelength=0.8650E-06
ALF_SU( :,14) = (/ &
 & 0.173086E+01_JPRB,0.173086E+01_JPRB,0.173086E+01_JPRB,0.173086E+01_JPRB,0.247183E+01_JPRB,0.284682E+01_JPRB &
 &,0.335279E+01_JPRB,0.410143E+01_JPRB,0.543763E+01_JPRB,0.668478E+01_JPRB,0.902430E+01_JPRB,0.166694E+02_JPRB /)
!-- Wavelength=0.1020E-05
ALF_SU( :,15) = (/ &
 & 0.119144E+01_JPRB,0.119144E+01_JPRB,0.119144E+01_JPRB,0.119144E+01_JPRB,0.172572E+01_JPRB,0.199893E+01_JPRB &
 &,0.236988E+01_JPRB,0.292288E+01_JPRB,0.391995E+01_JPRB,0.486014E+01_JPRB,0.664352E+01_JPRB,0.126013E+02_JPRB /)
!-- Wavelength=0.1064E-05
ALF_SU( :,16) = (/ &
 & 0.106066E+01_JPRB,0.106066E+01_JPRB,0.106066E+01_JPRB,0.106066E+01_JPRB,0.153734E+01_JPRB,0.178285E+01_JPRB &
 &,0.211666E+01_JPRB,0.261514E+01_JPRB,0.351615E+01_JPRB,0.436797E+01_JPRB,0.598840E+01_JPRB,0.114359E+02_JPRB /)
!-- Wavelength=0.1240E-05
ALF_SU( :,17) = (/ &
 & 0.722259E+00_JPRB,0.722259E+00_JPRB,0.722259E+00_JPRB,0.722259E+00_JPRB,0.106152E+01_JPRB,0.123753E+01_JPRB &
 &,0.147820E+01_JPRB,0.184006E+01_JPRB,0.250021E+01_JPRB,0.313010E+01_JPRB,0.434047E+01_JPRB,0.849122E+01_JPRB /)
!-- Wavelength=0.1640E-05
ALF_SU( :,18) = (/ &
 & 0.333007E+00_JPRB,0.333007E+00_JPRB,0.333007E+00_JPRB,0.333007E+00_JPRB,0.502306E+00_JPRB,0.591092E+00_JPRB &
 &,0.713733E+00_JPRB,0.900402E+00_JPRB,0.124659E+01_JPRB,0.158237E+01_JPRB,0.223919E+01_JPRB,0.457242E+01_JPRB /)
!-- Wavelength=0.2130E-05
ALF_SU( :,19) = (/ &
 & 0.146781E+00_JPRB,0.146781E+00_JPRB,0.146781E+00_JPRB,0.146781E+00_JPRB,0.226020E+00_JPRB,0.268072E+00_JPRB &
 &,0.326667E+00_JPRB,0.416794E+00_JPRB,0.586333E+00_JPRB,0.753127E+00_JPRB,0.108451E+01_JPRB,0.229883E+01_JPRB /)

! cg_aer_sulfate
!-- Wavelength=0.3400E-06
ASY_SU( :, 1) = (/ &
 & 0.706805E+00_JPRB,0.706805E+00_JPRB,0.706805E+00_JPRB,0.706805E+00_JPRB,0.762611E+00_JPRB,0.770722E+00_JPRB &
 &,0.779776E+00_JPRB,0.790437E+00_JPRB,0.804209E+00_JPRB,0.813369E+00_JPRB,0.825514E+00_JPRB,0.844983E+00_JPRB /)
!-- Wavelength=0.3550E-06
ASY_SU( :, 2) = (/ &
 & 0.648755E+00_JPRB,0.648755E+00_JPRB,0.648755E+00_JPRB,0.648755E+00_JPRB,0.700445E+00_JPRB,0.707685E+00_JPRB &
 &,0.715766E+00_JPRB,0.725280E+00_JPRB,0.737537E+00_JPRB,0.745633E+00_JPRB,0.756301E+00_JPRB,0.772843E+00_JPRB /)
!-- Wavelength=0.3800E-06
ASY_SU( :, 3) = (/ &
 & 0.644307E+00_JPRB,0.644307E+00_JPRB,0.644307E+00_JPRB,0.644307E+00_JPRB,0.695926E+00_JPRB,0.703365E+00_JPRB &
 &,0.711681E+00_JPRB,0.721496E+00_JPRB,0.734182E+00_JPRB,0.742607E+00_JPRB,0.753747E+00_JPRB,0.771258E+00_JPRB /)
!-- Wavelength=0.4000E-06
ASY_SU( :, 4) = (/ &
 & 0.640801E+00_JPRB,0.640801E+00_JPRB,0.640801E+00_JPRB,0.640801E+00_JPRB,0.692373E+00_JPRB,0.699969E+00_JPRB &
 &,0.708473E+00_JPRB,0.718525E+00_JPRB,0.731556E+00_JPRB,0.740235E+00_JPRB,0.751752E+00_JPRB,0.770043E+00_JPRB /)
!-- Wavelength=0.4400E-06
ASY_SU( :, 5) = (/ &
 & 0.633130E+00_JPRB,0.633130E+00_JPRB,0.633130E+00_JPRB,0.633130E+00_JPRB,0.684531E+00_JPRB,0.692414E+00_JPRB &
 &,0.701264E+00_JPRB,0.711756E+00_JPRB,0.725427E+00_JPRB,0.734593E+00_JPRB,0.746825E+00_JPRB,0.766598E+00_JPRB /)
!-- Wavelength=0.4690E-06
ASY_SU( :, 6) = (/ &
 & 0.627739E+00_JPRB,0.627739E+00_JPRB,0.627739E+00_JPRB,0.627739E+00_JPRB,0.679031E+00_JPRB,0.687118E+00_JPRB &
 &,0.696214E+00_JPRB,0.707020E+00_JPRB,0.721149E+00_JPRB,0.730662E+00_JPRB,0.743407E+00_JPRB,0.764253E+00_JPRB /)
!-- Wavelength=0.5000E-06
ASY_SU( :, 7) = (/ &
 & 0.621689E+00_JPRB,0.621689E+00_JPRB,0.621689E+00_JPRB,0.621689E+00_JPRB,0.672827E+00_JPRB,0.681118E+00_JPRB &
 &,0.690459E+00_JPRB,0.701580E+00_JPRB,0.716171E+00_JPRB,0.726036E+00_JPRB,0.739303E+00_JPRB,0.761248E+00_JPRB /)
!-- Wavelength=0.5320E-06
ASY_SU( :, 8) = (/ &
 & 0.615371E+00_JPRB,0.615371E+00_JPRB,0.615371E+00_JPRB,0.615371E+00_JPRB,0.666338E+00_JPRB,0.674828E+00_JPRB &
 &,0.684411E+00_JPRB,0.695842E+00_JPRB,0.710891E+00_JPRB,0.721106E+00_JPRB,0.734892E+00_JPRB,0.757940E+00_JPRB /)
!-- Wavelength=0.5550E-06
ASY_SU( :, 9) = (/ &
 & 0.610906E+00_JPRB,0.610906E+00_JPRB,0.610906E+00_JPRB,0.610906E+00_JPRB,0.661751E+00_JPRB,0.670382E+00_JPRB &
 &,0.680134E+00_JPRB,0.691784E+00_JPRB,0.707156E+00_JPRB,0.717617E+00_JPRB,0.731775E+00_JPRB,0.755607E+00_JPRB /)
!-- Wavelength=0.6450E-06
ASY_SU( :,10) = (/ &
 & 0.592225E+00_JPRB,0.592225E+00_JPRB,0.592225E+00_JPRB,0.592225E+00_JPRB,0.642407E+00_JPRB,0.651500E+00_JPRB &
 &,0.661819E+00_JPRB,0.674208E+00_JPRB,0.690686E+00_JPRB,0.702004E+00_JPRB,0.717453E+00_JPRB,0.744063E+00_JPRB /)
!-- Wavelength=0.6700E-06
ASY_SU( :,11) = (/ &
 & 0.587865E+00_JPRB,0.587865E+00_JPRB,0.587865E+00_JPRB,0.587865E+00_JPRB,0.637956E+00_JPRB,0.647195E+00_JPRB &
 &,0.657691E+00_JPRB,0.670310E+00_JPRB,0.687128E+00_JPRB,0.698707E+00_JPRB,0.714550E+00_JPRB,0.742003E+00_JPRB /)
!-- Wavelength=0.8000E-06
ASY_SU( :,12) = (/ &
 & 0.561265E+00_JPRB,0.561265E+00_JPRB,0.561265E+00_JPRB,0.561265E+00_JPRB,0.610246E+00_JPRB,0.620003E+00_JPRB &
 &,0.631145E+00_JPRB,0.644620E+00_JPRB,0.662749E+00_JPRB,0.675363E+00_JPRB,0.692793E+00_JPRB,0.723747E+00_JPRB /)
!-- Wavelength=0.8580E-06
ASY_SU( :,13) = (/ &
 & 0.550584E+00_JPRB,0.550584E+00_JPRB,0.550584E+00_JPRB,0.550584E+00_JPRB,0.599185E+00_JPRB,0.609181E+00_JPRB &
 &,0.620618E+00_JPRB,0.634483E+00_JPRB,0.653209E+00_JPRB,0.666293E+00_JPRB,0.684446E+00_JPRB,0.717000E+00_JPRB /)
!-- Wavelength=0.8650E-06
ASY_SU( :,14) = (/ &
 & 0.549603E+00_JPRB,0.549603E+00_JPRB,0.549603E+00_JPRB,0.549603E+00_JPRB,0.598207E+00_JPRB,0.608245E+00_JPRB &
 &,0.619735E+00_JPRB,0.633668E+00_JPRB,0.652495E+00_JPRB,0.665657E+00_JPRB,0.683927E+00_JPRB,0.716734E+00_JPRB /)
!-- Wavelength=0.1020E-05
ASY_SU( :,15) = (/ &
 & 0.519605E+00_JPRB,0.519605E+00_JPRB,0.519605E+00_JPRB,0.519605E+00_JPRB,0.566735E+00_JPRB,0.577175E+00_JPRB &
 &,0.589181E+00_JPRB,0.603822E+00_JPRB,0.623776E+00_JPRB,0.637860E+00_JPRB,0.657587E+00_JPRB,0.693755E+00_JPRB /)
!-- Wavelength=0.1064E-05
ASY_SU( :,16) = (/ &
 & 0.512218E+00_JPRB,0.512218E+00_JPRB,0.512218E+00_JPRB,0.512218E+00_JPRB,0.559092E+00_JPRB,0.569680E+00_JPRB &
 &,0.581873E+00_JPRB,0.596765E+00_JPRB,0.617108E+00_JPRB,0.631505E+00_JPRB,0.651720E+00_JPRB,0.688999E+00_JPRB /)
!-- Wavelength=0.1240E-05
ASY_SU( :,17) = (/ &
 & 0.481000E+00_JPRB,0.481000E+00_JPRB,0.481000E+00_JPRB,0.481000E+00_JPRB,0.526214E+00_JPRB,0.537092E+00_JPRB &
 &,0.549674E+00_JPRB,0.565123E+00_JPRB,0.586396E+00_JPRB,0.601582E+00_JPRB,0.623086E+00_JPRB,0.663487E+00_JPRB /)
!-- Wavelength=0.1640E-05
ASY_SU( :,18) = (/ &
 & 0.418656E+00_JPRB,0.418656E+00_JPRB,0.418656E+00_JPRB,0.418656E+00_JPRB,0.460329E+00_JPRB,0.471505E+00_JPRB &
 &,0.484534E+00_JPRB,0.500686E+00_JPRB,0.523251E+00_JPRB,0.539614E+00_JPRB,0.563144E+00_JPRB,0.608848E+00_JPRB /)
!-- Wavelength=0.2130E-05
ASY_SU( :,19) = (/ &
 & 0.355723E+00_JPRB,0.355723E+00_JPRB,0.355723E+00_JPRB,0.355723E+00_JPRB,0.393623E+00_JPRB,0.404744E+00_JPRB &
 &,0.417807E+00_JPRB,0.434145E+00_JPRB,0.457277E+00_JPRB,0.474297E+00_JPRB,0.499131E+00_JPRB,0.548882E+00_JPRB /)

! om_aer_sulfate
!-- Wavelength=0.3400E-06
OMG_SU( :, 1) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.3550E-06
OMG_SU( :, 2) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.3800E-06
OMG_SU( :, 3) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.4000E-06
OMG_SU( :, 4) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.4400E-06
OMG_SU( :, 5) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.4690E-06
OMG_SU( :, 6) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.5000E-06
OMG_SU( :, 7) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.5320E-06
OMG_SU( :, 8) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.5550E-06
OMG_SU( :, 9) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.6450E-06
OMG_SU( :,10) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.6700E-06
OMG_SU( :,11) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.8000E-06
OMG_SU( :,12) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.8580E-06
OMG_SU( :,13) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.8650E-06
OMG_SU( :,14) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.1020E-05
OMG_SU( :,15) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.1064E-05
OMG_SU( :,16) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.1240E-05
OMG_SU( :,17) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.1640E-05
OMG_SU( :,18) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.2130E-05
OMG_SU( :,19) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)

!-- lidar ratio
!-- Wavelength=0.3400E-06
RALI_SU( :, 1) = (/ &
 & 0.622997E+02_JPRB,0.622997E+02_JPRB,0.622997E+02_JPRB,0.622997E+02_JPRB,0.886078E+02_JPRB,0.927770E+02_JPRB &
 &,0.978915E+02_JPRB,0.104542E+03_JPRB,0.114219E+03_JPRB,0.121370E+03_JPRB,0.131554E+03_JPRB,0.146565E+03_JPRB /)
!-- Wavelength=0.3550E-06
RALI_SU( :, 2) = (/ &
 & 0.354344E+02_JPRB,0.354344E+02_JPRB,0.354344E+02_JPRB,0.354344E+02_JPRB,0.533061E+02_JPRB,0.553709E+02_JPRB &
 &,0.576784E+02_JPRB,0.604571E+02_JPRB,0.639367E+02_JPRB,0.659329E+02_JPRB,0.682744E+02_JPRB,0.693696E+02_JPRB /)
!-- Wavelength=0.3800E-06
RALI_SU( :, 3) = (/ &
 & 0.369790E+02_JPRB,0.369790E+02_JPRB,0.369790E+02_JPRB,0.369790E+02_JPRB,0.542279E+02_JPRB,0.562635E+02_JPRB &
 &,0.585856E+02_JPRB,0.613916E+02_JPRB,0.649584E+02_JPRB,0.672007E+02_JPRB,0.698104E+02_JPRB,0.717276E+02_JPRB /)
!-- Wavelength=0.4000E-06
RALI_SU( :, 4) = (/ &
 & 0.381159E+02_JPRB,0.381159E+02_JPRB,0.381159E+02_JPRB,0.381159E+02_JPRB,0.548636E+02_JPRB,0.569103E+02_JPRB &
 &,0.592240E+02_JPRB,0.620698E+02_JPRB,0.657728E+02_JPRB,0.680469E+02_JPRB,0.709144E+02_JPRB,0.735290E+02_JPRB /)
!-- Wavelength=0.4400E-06
RALI_SU( :, 5) = (/ &
 & 0.398338E+02_JPRB,0.398338E+02_JPRB,0.398338E+02_JPRB,0.398338E+02_JPRB,0.554635E+02_JPRB,0.574654E+02_JPRB &
 &,0.597852E+02_JPRB,0.626474E+02_JPRB,0.664498E+02_JPRB,0.689812E+02_JPRB,0.722212E+02_JPRB,0.759710E+02_JPRB /)
!-- Wavelength=0.4690E-06
RALI_SU( :, 6) = (/ &
 & 0.408767E+02_JPRB,0.408767E+02_JPRB,0.408767E+02_JPRB,0.408767E+02_JPRB,0.557368E+02_JPRB,0.577302E+02_JPRB &
 &,0.600544E+02_JPRB,0.629397E+02_JPRB,0.668408E+02_JPRB,0.694915E+02_JPRB,0.729889E+02_JPRB,0.775535E+02_JPRB /)
!-- Wavelength=0.5000E-06
RALI_SU( :, 7) = (/ &
 & 0.416408E+02_JPRB,0.416408E+02_JPRB,0.416408E+02_JPRB,0.416408E+02_JPRB,0.556616E+02_JPRB,0.576417E+02_JPRB &
 &,0.599628E+02_JPRB,0.628625E+02_JPRB,0.668407E+02_JPRB,0.695919E+02_JPRB,0.733178E+02_JPRB,0.786365E+02_JPRB /)
!-- Wavelength=0.5320E-06
RALI_SU( :, 8) = (/ &
 & 0.421514E+02_JPRB,0.421514E+02_JPRB,0.421514E+02_JPRB,0.421514E+02_JPRB,0.553461E+02_JPRB,0.573086E+02_JPRB &
 &,0.596408E+02_JPRB,0.625599E+02_JPRB,0.665895E+02_JPRB,0.694305E+02_JPRB,0.733278E+02_JPRB,0.793377E+02_JPRB /)
!-- Wavelength=0.5550E-06
RALI_SU( :, 9) = (/ &
 & 0.424161E+02_JPRB,0.424161E+02_JPRB,0.424161E+02_JPRB,0.424161E+02_JPRB,0.550478E+02_JPRB,0.570035E+02_JPRB &
 &,0.593210E+02_JPRB,0.622442E+02_JPRB,0.663280E+02_JPRB,0.692364E+02_JPRB,0.732866E+02_JPRB,0.797541E+02_JPRB /)
!-- Wavelength=0.6450E-06
RALI_SU( :,10) = (/ &
 & 0.421030E+02_JPRB,0.421030E+02_JPRB,0.421030E+02_JPRB,0.421030E+02_JPRB,0.527065E+02_JPRB,0.545962E+02_JPRB &
 &,0.568633E+02_JPRB,0.597509E+02_JPRB,0.638696E+02_JPRB,0.668864E+02_JPRB,0.712363E+02_JPRB,0.789747E+02_JPRB /)
!-- Wavelength=0.6700E-06
RALI_SU( :,11) = (/ &
 & 0.420356E+02_JPRB,0.420356E+02_JPRB,0.420356E+02_JPRB,0.420356E+02_JPRB,0.522145E+02_JPRB,0.541113E+02_JPRB &
 &,0.563821E+02_JPRB,0.592932E+02_JPRB,0.634790E+02_JPRB,0.665551E+02_JPRB,0.710520E+02_JPRB,0.792508E+02_JPRB /)
!-- Wavelength=0.8000E-06
RALI_SU( :,12) = (/ &
 & 0.397222E+02_JPRB,0.397222E+02_JPRB,0.397222E+02_JPRB,0.397222E+02_JPRB,0.478247E+02_JPRB,0.496008E+02_JPRB &
 &,0.517479E+02_JPRB,0.545253E+02_JPRB,0.585979E+02_JPRB,0.616753E+02_JPRB,0.662793E+02_JPRB,0.753600E+02_JPRB /)
!-- Wavelength=0.8580E-06
RALI_SU( :,13) = (/ &
 & 0.386180E+02_JPRB,0.386180E+02_JPRB,0.386180E+02_JPRB,0.386180E+02_JPRB,0.460561E+02_JPRB,0.477976E+02_JPRB &
 &,0.499112E+02_JPRB,0.526581E+02_JPRB,0.567178E+02_JPRB,0.598153E+02_JPRB,0.645036E+02_JPRB,0.740364E+02_JPRB /)
!-- Wavelength=0.8650E-06
RALI_SU( :,14) = (/ &
 & 0.385239E+02_JPRB,0.385239E+02_JPRB,0.385239E+02_JPRB,0.385239E+02_JPRB,0.459054E+02_JPRB,0.476505E+02_JPRB &
 &,0.497698E+02_JPRB,0.525266E+02_JPRB,0.566073E+02_JPRB,0.597267E+02_JPRB,0.644580E+02_JPRB,0.741291E+02_JPRB /)
!-- Wavelength=0.1020E-05
RALI_SU( :,15) = (/ &
 & 0.350181E+02_JPRB,0.350181E+02_JPRB,0.350181E+02_JPRB,0.350181E+02_JPRB,0.409224E+02_JPRB,0.424930E+02_JPRB &
 &,0.444104E+02_JPRB,0.469239E+02_JPRB,0.506939E+02_JPRB,0.536202E+02_JPRB,0.581345E+02_JPRB,0.677975E+02_JPRB /)
!-- Wavelength=0.1064E-05
RALI_SU( :,16) = (/ &
 & 0.341386E+02_JPRB,0.341386E+02_JPRB,0.341386E+02_JPRB,0.341386E+02_JPRB,0.397530E+02_JPRB,0.412942E+02_JPRB &
 &,0.431796E+02_JPRB,0.456584E+02_JPRB,0.493946E+02_JPRB,0.523112E+02_JPRB,0.568395E+02_JPRB,0.666857E+02_JPRB /)
!-- Wavelength=0.1240E-05
RALI_SU( :,17) = (/ &
 & 0.306609E+02_JPRB,0.306609E+02_JPRB,0.306609E+02_JPRB,0.306609E+02_JPRB,0.352140E+02_JPRB,0.365695E+02_JPRB &
 &,0.382349E+02_JPRB,0.404359E+02_JPRB,0.437845E+02_JPRB,0.464291E+02_JPRB,0.505857E+02_JPRB,0.599343E+02_JPRB /)
!-- Wavelength=0.1640E-05
RALI_SU( :,18) = (/ &
 & 0.247953E+02_JPRB,0.247953E+02_JPRB,0.247953E+02_JPRB,0.247953E+02_JPRB,0.278953E+02_JPRB,0.289182E+02_JPRB &
 &,0.301815E+02_JPRB,0.318628E+02_JPRB,0.344526E+02_JPRB,0.365300E+02_JPRB,0.398533E+02_JPRB,0.476967E+02_JPRB /)
!-- Wavelength=0.2130E-05
RALI_SU( :,19) = (/ &
 & 0.202831E+02_JPRB,0.202831E+02_JPRB,0.202831E+02_JPRB,0.202831E+02_JPRB,0.224503E+02_JPRB,0.232058E+02_JPRB &
 &,0.241412E+02_JPRB,0.253903E+02_JPRB,0.273275E+02_JPRB,0.288955E+02_JPRB,0.314312E+02_JPRB,0.376053E+02_JPRB /)

!------------------------------------ ORGANIC MATTER --------------------------------------
!-- from O.B.'s oranic matter Mie code 20090312

! NB: "new_ammonium_sulfate" for the real and imaginary parts of the refractive index
! "mie_oc_l_wv_work" for the actual Mie computations

! NB: the particle size distribution parameters also follow the sulphate parameters
! rmin_dry  rmax_dry   r_0_dry    sigma    rho
! 0.002E-6  20.0E-6   0.0355E-6	    2	 1.769E+3

! alpha_aer_om
!-- Wavelength=0.3400E-06
ALF_OM( :, 1) = (/ &
 & 0.596995E+01_JPRB,0.596995E+01_JPRB,0.596995E+01_JPRB,0.596995E+01_JPRB,0.814987E+01_JPRB,0.909989E+01_JPRB &
 &,0.103372E+02_JPRB,0.120911E+02_JPRB,0.150397E+02_JPRB,0.176270E+02_JPRB,0.221687E+02_JPRB,0.351291E+02_JPRB /)
!-- Wavelength=0.3550E-06
ALF_OM( :, 2) = (/ &
 & 0.569133E+01_JPRB,0.569133E+01_JPRB,0.569133E+01_JPRB,0.569133E+01_JPRB,0.778002E+01_JPRB,0.869889E+01_JPRB &
 &,0.989760E+01_JPRB,0.116000E+02_JPRB,0.144699E+02_JPRB,0.169957E+02_JPRB,0.214419E+02_JPRB,0.342072E+02_JPRB /)
!-- Wavelength=0.3800E-06
ALF_OM( :, 3) = (/ &
 & 0.525349E+01_JPRB,0.525349E+01_JPRB,0.525349E+01_JPRB,0.525349E+01_JPRB,0.719788E+01_JPRB,0.806566E+01_JPRB &
 &,0.920060E+01_JPRB,0.108175E+02_JPRB,0.135551E+02_JPRB,0.159753E+02_JPRB,0.202561E+02_JPRB,0.326664E+02_JPRB /)
!-- Wavelength=0.4000E-06
ALF_OM( :, 4) = (/ &
 & 0.492706E+01_JPRB,0.492706E+01_JPRB,0.492706E+01_JPRB,0.492706E+01_JPRB,0.676296E+01_JPRB,0.759095E+01_JPRB &
 &,0.867595E+01_JPRB,0.102253E+02_JPRB,0.128573E+02_JPRB,0.151921E+02_JPRB,0.193371E+02_JPRB,0.314425E+02_JPRB /)
!-- Wavelength=0.4400E-06
ALF_OM( :, 5) = (/ &
 & 0.433517E+01_JPRB,0.433517E+01_JPRB,0.433517E+01_JPRB,0.433517E+01_JPRB,0.597233E+01_JPRB,0.672450E+01_JPRB &
 &,0.771373E+01_JPRB,0.913260E+01_JPRB,0.115578E+02_JPRB,0.137229E+02_JPRB,0.175931E+02_JPRB,0.290544E+02_JPRB /)
!-- Wavelength=0.4690E-06
ALF_OM( :, 6) = (/ &
 & 0.395367E+01_JPRB,0.395367E+01_JPRB,0.395367E+01_JPRB,0.395367E+01_JPRB,0.546118E+01_JPRB,0.616196E+01_JPRB &
 &,0.708587E+01_JPRB,0.841496E+01_JPRB,0.106962E+02_JPRB,0.127415E+02_JPRB,0.164144E+02_JPRB,0.273942E+02_JPRB /)
!-- Wavelength=0.5000E-06
ALF_OM( :, 7) = (/ &
 & 0.358622E+01_JPRB,0.358622E+01_JPRB,0.358622E+01_JPRB,0.358622E+01_JPRB,0.496754E+01_JPRB,0.561692E+01_JPRB &
 &,0.647515E+01_JPRB,0.771343E+01_JPRB,0.984764E+01_JPRB,0.117694E+02_JPRB,0.152366E+02_JPRB,0.256994E+02_JPRB /)
!-- Wavelength=0.5320E-06
ALF_OM( :, 8) = (/ &
 & 0.324656E+01_JPRB,0.324656E+01_JPRB,0.324656E+01_JPRB,0.324656E+01_JPRB,0.451000E+01_JPRB,0.511014E+01_JPRB &
 &,0.590517E+01_JPRB,0.705558E+01_JPRB,0.904636E+01_JPRB,0.108464E+02_JPRB,0.141087E+02_JPRB,0.240439E+02_JPRB /)
!-- Wavelength=0.5500E-06
ALF_OM( :, 9) = (/ &
 & 0.307043E+01_JPRB,0.307043E+01_JPRB,0.307043E+01_JPRB,0.307043E+01_JPRB,0.427185E+01_JPRB,0.484567E+01_JPRB &
 &,0.560681E+01_JPRB,0.670989E+01_JPRB,0.862290E+01_JPRB,0.103565E+02_JPRB,0.135060E+02_JPRB,0.231454E+02_JPRB /)
!-- Wavelength=0.6450E-06
ALF_OM( :,10) = (/ &
 & 0.231077E+01_JPRB,0.231077E+01_JPRB,0.231077E+01_JPRB,0.231077E+01_JPRB,0.324186E+01_JPRB,0.369706E+01_JPRB &
 &,0.430452E+01_JPRB,0.519141E+01_JPRB,0.674533E+01_JPRB,0.816851E+01_JPRB,0.107837E+02_JPRB,0.189774E+02_JPRB /)
!-- Wavelength=0.6700E-06
ALF_OM( :,11) = (/ &
 & 0.214877E+01_JPRB,0.214877E+01_JPRB,0.214877E+01_JPRB,0.214877E+01_JPRB,0.302097E+01_JPRB,0.344956E+01_JPRB &
 &,0.402234E+01_JPRB,0.486005E+01_JPRB,0.633139E+01_JPRB,0.768231E+01_JPRB,0.101715E+02_JPRB,0.180137E+02_JPRB /)
!-- Wavelength=0.8000E-06
ALF_OM( :,12) = (/ &
 & 0.149465E+01_JPRB,0.149465E+01_JPRB,0.149465E+01_JPRB,0.149465E+01_JPRB,0.212362E+01_JPRB,0.243938E+01_JPRB &
 &,0.286419E+01_JPRB,0.349055E+01_JPRB,0.460306E+01_JPRB,0.563632E+01_JPRB,0.756412E+01_JPRB,0.137950E+02_JPRB /)
!-- Wavelength=0.8580E-06
ALF_OM( :,13) = (/ &
 & 0.128116E+01_JPRB,0.128116E+01_JPRB,0.128116E+01_JPRB,0.128116E+01_JPRB,0.182842E+01_JPRB,0.210520E+01_JPRB &
 &,0.247855E+01_JPRB,0.303077E+01_JPRB,0.401587E+01_JPRB,0.493488E+01_JPRB,0.665786E+01_JPRB,0.122821E+02_JPRB /)
!-- Wavelength=0.8650E-06
ALF_OM( :,14) = (/ &
 & 0.125795E+01_JPRB,0.125795E+01_JPRB,0.125795E+01_JPRB,0.125795E+01_JPRB,0.179623E+01_JPRB,0.206870E+01_JPRB &
 &,0.243634E+01_JPRB,0.298032E+01_JPRB,0.395121E+01_JPRB,0.485742E+01_JPRB,0.655737E+01_JPRB,0.121127E+02_JPRB /)
!-- Wavelength=0.1020E-05
ALF_OM( :,15) = (/ &
 & 0.852843E+00_JPRB,0.852843E+00_JPRB,0.852843E+00_JPRB,0.852843E+00_JPRB,0.123135E+01_JPRB,0.142590E+01_JPRB &
 &,0.169000E+01_JPRB,0.208363E+01_JPRB,0.279330E+01_JPRB,0.346250E+01_JPRB,0.473198E+01_JPRB,0.897621E+01_JPRB /)
!-- Wavelength=0.1064E-05
ALF_OM( :,16) = (/ &
 & 0.767856E+00_JPRB,0.767856E+00_JPRB,0.767856E+00_JPRB,0.767856E+00_JPRB,0.111191E+01_JPRB,0.128938E+01_JPRB &
 &,0.153065E+01_JPRB,0.189093E+01_JPRB,0.254211E+01_JPRB,0.315773E+01_JPRB,0.432887E+01_JPRB,0.826677E+01_JPRB /)
!-- Wavelength=0.1240E-05
ALF_OM( :,17) = (/ &
 & 0.515367E+00_JPRB,0.515367E+00_JPRB,0.515367E+00_JPRB,0.515367E+00_JPRB,0.754493E+00_JPRB,0.879259E+00_JPRB &
 &,0.104981E+01_JPRB,0.130616E+01_JPRB,0.177370E+01_JPRB,0.221976E+01_JPRB,0.307684E+01_JPRB,0.601772E+01_JPRB /)
!-- Wavelength=0.1640E-05
ALF_OM( :,18) = (/ &
 & 0.231350E+00_JPRB,0.231350E+00_JPRB,0.231350E+00_JPRB,0.231350E+00_JPRB,0.345609E+00_JPRB,0.406265E+00_JPRB &
 &,0.489970E+00_JPRB,0.617239E+00_JPRB,0.853012E+00_JPRB,0.108153E+01_JPRB,0.152826E+01_JPRB,0.311569E+01_JPRB /)
!-- Wavelength=0.2130E-05
ALF_OM( :,19) = (/ &
 & 0.100662E+00_JPRB,0.100662E+00_JPRB,0.100662E+00_JPRB,0.100662E+00_JPRB,0.153010E+00_JPRB,0.181198E+00_JPRB &
 &,0.220416E+00_JPRB,0.280636E+00_JPRB,0.393703E+00_JPRB,0.504781E+00_JPRB,0.725155E+00_JPRB,0.153213E+01_JPRB /)

! cg_aer_om
!-- Wavelength=0.3400E-06
ASY_OM( :, 1) = (/ &
 & 0.651465E+00_JPRB,0.651465E+00_JPRB,0.651465E+00_JPRB,0.651465E+00_JPRB,0.703196E+00_JPRB,0.710315E+00_JPRB &
 &,0.718253E+00_JPRB,0.727584E+00_JPRB,0.739575E+00_JPRB,0.747488E+00_JPRB,0.757860E+00_JPRB,0.773817E+00_JPRB /)
!-- Wavelength=0.3550E-06
ASY_OM( :, 2) = (/ &
 & 0.648974E+00_JPRB,0.648974E+00_JPRB,0.648974E+00_JPRB,0.648974E+00_JPRB,0.700684E+00_JPRB,0.707930E+00_JPRB &
 &,0.716015E+00_JPRB,0.725536E+00_JPRB,0.737802E+00_JPRB,0.745903E+00_JPRB,0.756577E+00_JPRB,0.773137E+00_JPRB /)
!-- Wavelength=0.3800E-06
ASY_OM( :, 3) = (/ &
 & 0.644660E+00_JPRB,0.644660E+00_JPRB,0.644660E+00_JPRB,0.644660E+00_JPRB,0.696314E+00_JPRB,0.703761E+00_JPRB &
 &,0.712087E+00_JPRB,0.721911E+00_JPRB,0.734613E+00_JPRB,0.743045E+00_JPRB,0.754201E+00_JPRB,0.771738E+00_JPRB /)
!-- Wavelength=0.4000E-06
ASY_OM( :, 4) = (/ &
 & 0.641077E+00_JPRB,0.641077E+00_JPRB,0.641077E+00_JPRB,0.641077E+00_JPRB,0.692677E+00_JPRB,0.700278E+00_JPRB &
 &,0.708790E+00_JPRB,0.718851E+00_JPRB,0.731893E+00_JPRB,0.740580E+00_JPRB,0.752107E+00_JPRB,0.770419E+00_JPRB /)
!-- Wavelength=0.4400E-06
ASY_OM( :, 5) = (/ &
 & 0.633653E+00_JPRB,0.633653E+00_JPRB,0.633653E+00_JPRB,0.633653E+00_JPRB,0.685109E+00_JPRB,0.693005E+00_JPRB &
 &,0.701870E+00_JPRB,0.712379E+00_JPRB,0.726075E+00_JPRB,0.735256E+00_JPRB,0.747512E+00_JPRB,0.767332E+00_JPRB /)
!-- Wavelength=0.4690E-06
ASY_OM( :, 6) = (/ &
 & 0.628105E+00_JPRB,0.628105E+00_JPRB,0.628105E+00_JPRB,0.628105E+00_JPRB,0.679436E+00_JPRB,0.687533E+00_JPRB &
 &,0.696639E+00_JPRB,0.707458E+00_JPRB,0.721606E+00_JPRB,0.731130E+00_JPRB,0.743892E+00_JPRB,0.764773E+00_JPRB /)
!-- Wavelength=0.5000E-06
ASY_OM( :, 7) = (/ &
 & 0.622064E+00_JPRB,0.622064E+00_JPRB,0.622064E+00_JPRB,0.622064E+00_JPRB,0.673244E+00_JPRB,0.681545E+00_JPRB &
 &,0.690898E+00_JPRB,0.702033E+00_JPRB,0.716645E+00_JPRB,0.726522E+00_JPRB,0.739808E+00_JPRB,0.761791E+00_JPRB /)
!-- Wavelength=0.5320E-06
ASY_OM( :, 8) = (/ &
 & 0.615744E+00_JPRB,0.615744E+00_JPRB,0.615744E+00_JPRB,0.615744E+00_JPRB,0.666752E+00_JPRB,0.675253E+00_JPRB &
 &,0.684848E+00_JPRB,0.696294E+00_JPRB,0.711363E+00_JPRB,0.721593E+00_JPRB,0.735400E+00_JPRB,0.758487E+00_JPRB /)
!-- Wavelength=0.5500E-06
ASY_OM( :, 9) = (/ &
 & 0.612219E+00_JPRB,0.612219E+00_JPRB,0.612219E+00_JPRB,0.612219E+00_JPRB,0.663130E+00_JPRB,0.671741E+00_JPRB &
 &,0.681469E+00_JPRB,0.693086E+00_JPRB,0.708408E+00_JPRB,0.718830E+00_JPRB,0.732928E+00_JPRB,0.756628E+00_JPRB /)
!-- Wavelength=0.6450E-06
ASY_OM( :,10) = (/ &
 & 0.593120E+00_JPRB,0.593120E+00_JPRB,0.593120E+00_JPRB,0.593120E+00_JPRB,0.643417E+00_JPRB,0.652543E+00_JPRB &
 &,0.662899E+00_JPRB,0.675332E+00_JPRB,0.691872E+00_JPRB,0.703234E+00_JPRB,0.718746E+00_JPRB,0.745477E+00_JPRB /)
!-- Wavelength=0.6700E-06
ASY_OM( :,11) = (/ &
 & 0.588105E+00_JPRB,0.588105E+00_JPRB,0.588105E+00_JPRB,0.588105E+00_JPRB,0.638228E+00_JPRB,0.647476E+00_JPRB &
 &,0.657982E+00_JPRB,0.670613E+00_JPRB,0.687449E+00_JPRB,0.699041E+00_JPRB,0.714901E+00_JPRB,0.742388E+00_JPRB /)
!-- Wavelength=0.8000E-06
ASY_OM( :,12) = (/ &
 & 0.562294E+00_JPRB,0.562294E+00_JPRB,0.562294E+00_JPRB,0.562294E+00_JPRB,0.611433E+00_JPRB,0.621240E+00_JPRB &
 &,0.632438E+00_JPRB,0.645982E+00_JPRB,0.664207E+00_JPRB,0.676889E+00_JPRB,0.694417E+00_JPRB,0.725563E+00_JPRB /)
!-- Wavelength=0.8580E-06
ASY_OM( :,13) = (/ &
 & 0.551004E+00_JPRB,0.551004E+00_JPRB,0.551004E+00_JPRB,0.551004E+00_JPRB,0.599673E+00_JPRB,0.609691E+00_JPRB &
 &,0.621154E+00_JPRB,0.635050E+00_JPRB,0.653819E+00_JPRB,0.666935E+00_JPRB,0.685132E+00_JPRB,0.717775E+00_JPRB /)
!-- Wavelength=0.8650E-06
ASY_OM( :,14) = (/ &
 & 0.549653E+00_JPRB,0.549653E+00_JPRB,0.549653E+00_JPRB,0.549653E+00_JPRB,0.598265E+00_JPRB,0.608306E+00_JPRB &
 &,0.619799E+00_JPRB,0.633736E+00_JPRB,0.652568E+00_JPRB,0.665734E+00_JPRB,0.684009E+00_JPRB,0.716827E+00_JPRB /)
!-- Wavelength=0.1020E-05
ASY_OM( :,15) = (/ &
 & 0.520460E+00_JPRB,0.520460E+00_JPRB,0.520460E+00_JPRB,0.520460E+00_JPRB,0.567761E+00_JPRB,0.578260E+00_JPRB &
 &,0.590334E+00_JPRB,0.605061E+00_JPRB,0.625134E+00_JPRB,0.639305E+00_JPRB,0.659158E+00_JPRB,0.695578E+00_JPRB /)
!-- Wavelength=0.1064E-05
ASY_OM( :,16) = (/ &
 & 0.512442E+00_JPRB,0.512442E+00_JPRB,0.512442E+00_JPRB,0.512442E+00_JPRB,0.559362E+00_JPRB,0.569967E+00_JPRB &
 &,0.582180E+00_JPRB,0.597096E+00_JPRB,0.617474E+00_JPRB,0.631895E+00_JPRB,0.652146E+00_JPRB,0.689498E+00_JPRB /)
!-- Wavelength=0.1240E-05
ASY_OM( :,17) = (/ &
 & 0.481631E+00_JPRB,0.481631E+00_JPRB,0.481631E+00_JPRB,0.481631E+00_JPRB,0.527016E+00_JPRB,0.537958E+00_JPRB &
 &,0.550616E+00_JPRB,0.566160E+00_JPRB,0.587568E+00_JPRB,0.602855E+00_JPRB,0.624506E+00_JPRB,0.665211E+00_JPRB /)
!-- Wavelength=0.1640E-05
ASY_OM( :,18) = (/ &
 & 0.419073E+00_JPRB,0.419073E+00_JPRB,0.419073E+00_JPRB,0.419073E+00_JPRB,0.461040E+00_JPRB,0.472341E+00_JPRB &
 &,0.485520E+00_JPRB,0.501864E+00_JPRB,0.524710E+00_JPRB,0.541289E+00_JPRB,0.565144E+00_JPRB,0.611550E+00_JPRB /)
!-- Wavelength=0.2130E-05
ASY_OM( :,19) = (/ &
 & 0.355290E+00_JPRB,0.355290E+00_JPRB,0.355290E+00_JPRB,0.355290E+00_JPRB,0.393424E+00_JPRB,0.404658E+00_JPRB &
 &,0.417859E+00_JPRB,0.434380E+00_JPRB,0.457789E+00_JPRB,0.475028E+00_JPRB,0.500205E+00_JPRB,0.550743E+00_JPRB /)

! om_aer_om
!-- Wavelength=0.3400E-06
OMG_OM( :, 1) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.3550E-06
OMG_OM( :, 2) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.3800E-06
OMG_OM( :, 3) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.4000E-06
OMG_OM( :, 4) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.4400E-06
OMG_OM( :, 5) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.4690E-06
OMG_OM( :, 6) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.5000E-06
OMG_OM( :, 7) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.5320E-06
OMG_OM( :, 8) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.5500E-06
OMG_OM( :, 9) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.6450E-06
OMG_OM( :,10) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.6700E-06
OMG_OM( :,11) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.8000E-06
OMG_OM( :,12) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.8580E-06
OMG_OM( :,13) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.8650E-06
OMG_OM( :,14) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.1020E-05
OMG_OM( :,15) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.1064E-05
OMG_OM( :,16) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.1240E-05
OMG_OM( :,17) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.1640E-05
OMG_OM( :,18) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)
!-- Wavelength=0.2130E-05
OMG_OM( :,19) = (/ &
 & 0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB &
 &,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB,0.100000E+01_JPRB /)

!-- lidar ratio
!-- Wavelength=0.3400E-06
RALI_OM( :, 1) = (/ &
 & 0.344614E+02_JPRB,0.344614E+02_JPRB,0.344614E+02_JPRB,0.344614E+02_JPRB,0.527073E+02_JPRB,0.547469E+02_JPRB &
 &,0.571360E+02_JPRB,0.598658E+02_JPRB,0.631599E+02_JPRB,0.652079E+02_JPRB,0.672225E+02_JPRB,0.678101E+02_JPRB /)
!-- Wavelength=0.3550E-06
RALI_OM( :, 2) = (/ &
 & 0.355209E+02_JPRB,0.355209E+02_JPRB,0.355209E+02_JPRB,0.355209E+02_JPRB,0.534357E+02_JPRB,0.555168E+02_JPRB &
 &,0.578200E+02_JPRB,0.606066E+02_JPRB,0.641128E+02_JPRB,0.660861E+02_JPRB,0.684384E+02_JPRB,0.695509E+02_JPRB /)
!-- Wavelength=0.3800E-06
RALI_OM( :, 3) = (/ &
 & 0.371189E+02_JPRB,0.371189E+02_JPRB,0.371189E+02_JPRB,0.371189E+02_JPRB,0.544444E+02_JPRB,0.564974E+02_JPRB &
 &,0.588270E+02_JPRB,0.616291E+02_JPRB,0.652329E+02_JPRB,0.674382E+02_JPRB,0.701127E+02_JPRB,0.720953E+02_JPRB /)
!-- Wavelength=0.4000E-06
RALI_OM( :, 4) = (/ &
 & 0.382272E+02_JPRB,0.382272E+02_JPRB,0.382272E+02_JPRB,0.382272E+02_JPRB,0.550289E+02_JPRB,0.570707E+02_JPRB &
 &,0.594089E+02_JPRB,0.622724E+02_JPRB,0.659740E+02_JPRB,0.682579E+02_JPRB,0.711187E+02_JPRB,0.737457E+02_JPRB /)
!-- Wavelength=0.4400E-06
RALI_OM( :, 5) = (/ &
 & 0.400348E+02_JPRB,0.400348E+02_JPRB,0.400348E+02_JPRB,0.400348E+02_JPRB,0.557507E+02_JPRB,0.577754E+02_JPRB &
 &,0.601046E+02_JPRB,0.629924E+02_JPRB,0.668445E+02_JPRB,0.693494E+02_JPRB,0.726350E+02_JPRB,0.764620E+02_JPRB /)
!-- Wavelength=0.4690E-06
RALI_OM( :, 6) = (/ &
 & 0.410139E+02_JPRB,0.410139E+02_JPRB,0.410139E+02_JPRB,0.410139E+02_JPRB,0.559271E+02_JPRB,0.579347E+02_JPRB &
 &,0.602665E+02_JPRB,0.631730E+02_JPRB,0.671072E+02_JPRB,0.697425E+02_JPRB,0.732699E+02_JPRB,0.778840E+02_JPRB /)
!-- Wavelength=0.5000E-06
RALI_OM( :, 7) = (/ &
 & 0.417756E+02_JPRB,0.417756E+02_JPRB,0.417756E+02_JPRB,0.417756E+02_JPRB,0.558454E+02_JPRB,0.578398E+02_JPRB &
 &,0.601688E+02_JPRB,0.630908E+02_JPRB,0.671040E+02_JPRB,0.698466E+02_JPRB,0.736099E+02_JPRB,0.789851E+02_JPRB /)
!-- Wavelength=0.5320E-06
RALI_OM( :, 8) = (/ &
 & 0.422839E+02_JPRB,0.422839E+02_JPRB,0.422839E+02_JPRB,0.422839E+02_JPRB,0.555210E+02_JPRB,0.574912E+02_JPRB &
 &,0.598357E+02_JPRB,0.627662E+02_JPRB,0.668206E+02_JPRB,0.696957E+02_JPRB,0.736281E+02_JPRB,0.796995E+02_JPRB /)
!-- Wavelength=0.5500E-06
RALI_OM( :, 9) = (/ &
 & 0.424842E+02_JPRB,0.424842E+02_JPRB,0.424842E+02_JPRB,0.424842E+02_JPRB,0.552735E+02_JPRB,0.572434E+02_JPRB &
 &,0.595695E+02_JPRB,0.625070E+02_JPRB,0.666107E+02_JPRB,0.695132E+02_JPRB,0.735746E+02_JPRB,0.800234E+02_JPRB /)
!-- Wavelength=0.6450E-06
RALI_OM( :,10) = (/ &
 & 0.423528E+02_JPRB,0.423528E+02_JPRB,0.423528E+02_JPRB,0.423528E+02_JPRB,0.530287E+02_JPRB,0.549474E+02_JPRB &
 &,0.572398E+02_JPRB,0.601690E+02_JPRB,0.643630E+02_JPRB,0.674348E+02_JPRB,0.718858E+02_JPRB,0.798441E+02_JPRB /)
!-- Wavelength=0.6700E-06
RALI_OM( :,11) = (/ &
 & 0.420976E+02_JPRB,0.420976E+02_JPRB,0.420976E+02_JPRB,0.420976E+02_JPRB,0.522955E+02_JPRB,0.541985E+02_JPRB &
 &,0.564798E+02_JPRB,0.594044E+02_JPRB,0.636094E+02_JPRB,0.666986E+02_JPRB,0.712190E+02_JPRB,0.794764E+02_JPRB /)
!-- Wavelength=0.8000E-06
RALI_OM( :,12) = (/ &
 & 0.399140E+02_JPRB,0.399140E+02_JPRB,0.399140E+02_JPRB,0.399140E+02_JPRB,0.480812E+02_JPRB,0.498865E+02_JPRB &
 &,0.520705E+02_JPRB,0.549004E+02_JPRB,0.590572E+02_JPRB,0.622003E+02_JPRB,0.669223E+02_JPRB,0.763011E+02_JPRB /)
!-- Wavelength=0.8580E-06
RALI_OM( :,13) = (/ &
 & 0.386851E+02_JPRB,0.386851E+02_JPRB,0.386851E+02_JPRB,0.386851E+02_JPRB,0.461478E+02_JPRB,0.479006E+02_JPRB &
 &,0.500284E+02_JPRB,0.527950E+02_JPRB,0.568890E+02_JPRB,0.600168E+02_JPRB,0.647547E+02_JPRB,0.744144E+02_JPRB /)
!-- Wavelength=0.8650E-06
RALI_OM( :,14) = (/ &
 & 0.385316E+02_JPRB,0.385316E+02_JPRB,0.385316E+02_JPRB,0.385316E+02_JPRB,0.459162E+02_JPRB,0.476625E+02_JPRB &
 &,0.497836E+02_JPRB,0.525428E+02_JPRB,0.566275E+02_JPRB,0.597506E+02_JPRB,0.644878E+02_JPRB,0.741743E+02_JPRB /)
!-- Wavelength=0.1020E-05
RALI_OM( :,15) = (/ &
 & 0.351016E+02_JPRB,0.351016E+02_JPRB,0.351016E+02_JPRB,0.351016E+02_JPRB,0.410514E+02_JPRB,0.426433E+02_JPRB &
 &,0.445882E+02_JPRB,0.471412E+02_JPRB,0.509786E+02_JPRB,0.539642E+02_JPRB,0.585825E+02_JPRB,0.685250E+02_JPRB /)
!-- Wavelength=0.1064E-05
RALI_OM( :,16) = (/ &
 & 0.341572E+02_JPRB,0.341572E+02_JPRB,0.341572E+02_JPRB,0.341572E+02_JPRB,0.397834E+02_JPRB,0.413300E+02_JPRB &
 &,0.432228E+02_JPRB,0.457120E+02_JPRB,0.494658E+02_JPRB,0.523980E+02_JPRB,0.569533E+02_JPRB,0.668751E+02_JPRB /)
!-- Wavelength=0.1240E-05
RALI_OM( :,17) = (/ &
 & 0.306839E+02_JPRB,0.306839E+02_JPRB,0.306839E+02_JPRB,0.306839E+02_JPRB,0.352700E+02_JPRB,0.366401E+02_JPRB &
 &,0.383250E+02_JPRB,0.405546E+02_JPRB,0.439531E+02_JPRB,0.466424E+02_JPRB,0.508797E+02_JPRB,0.604595E+02_JPRB /)
!-- Wavelength=0.1640E-05
RALI_OM( :,18) = (/ &
 & 0.247506E+02_JPRB,0.247506E+02_JPRB,0.247506E+02_JPRB,0.247506E+02_JPRB,0.278862E+02_JPRB,0.289227E+02_JPRB &
 &,0.302049E+02_JPRB,0.319146E+02_JPRB,0.345559E+02_JPRB,0.366815E+02_JPRB,0.400949E+02_JPRB,0.482171E+02_JPRB /)
!-- Wavelength=0.2130E-05
RALI_OM( :,19) = (/ &
 & 0.202053E+02_JPRB,0.202053E+02_JPRB,0.202053E+02_JPRB,0.202053E+02_JPRB,0.223908E+02_JPRB,0.231519E+02_JPRB &
 &,0.240954E+02_JPRB,0.253576E+02_JPRB,0.273196E+02_JPRB,0.289120E+02_JPRB,0.314953E+02_JPRB,0.378293E+02_JPRB /)

!------------------------------------ DUST   3 bins ---------------------------------------------
! O.B.'s ddust Mie code 20090312 gems_dust_mie_ecmwf_work.f

! real part of the refractive index is 1.48
! imaginary part of the refractive index
!     + 0.00250 ,0.00250 ,0.00250 ,0.00250 ,0.00250 ,0.00228 ,0.00203 ,
!     + 0.00178 ,0.00164 ,0.00090 ,0.00070 ,0.00064 ,0.00061 ,0.00060 ,
!     + 0.00060, 0.00060 ,0.00060 ,0.00060 ,0.00060 

! NB: the particle size distribution parameters also follow the sulphate parameters
! rmin      rmax        r_0       sigma    rho
! 0.03E-6  20.0E-6    0.29E-6	    2	 2.610E+3

!           bin1,         bin2,         bin3

! optical thickness               
!-- WVL=0.3400E-06
ZALFDD3( :, 1) = (/ &
  & 2.07208_JPRB, 0.98177_JPRB, 0.40267_JPRB /)
!-- WVL=0.3550E-06
ZALFDD3( :, 2) = (/ &
  & 2.08899_JPRB, 0.99287_JPRB, 0.40370_JPRB /)
!-- WVL=0.3800E-06
ZALFDD3( :, 3) = (/ &
  & 2.14550_JPRB, 1.00764_JPRB, 0.40116_JPRB /)
!-- WVL=0.4000E-06
ZALFDD3( :, 4) = (/ &
  & 2.20965_JPRB, 1.01082_JPRB, 0.39807_JPRB /)
!-- WVL=0.4400E-06
ZALFDD3( :, 5) = (/ &
  & 2.35666_JPRB, 0.98479_JPRB, 0.39769_JPRB /)
!-- WVL=0.4690E-06
ZALFDD3( :, 6) = (/ &
  & 2.45347_JPRB, 0.94886_JPRB, 0.40381_JPRB /)
!-- WVL=0.5000E-06
ZALFDD3( :, 7) = (/ &
  & 2.54938_JPRB, 0.90146_JPRB, 0.41358_JPRB /)
!-- WVL=0.5320E-06
ZALFDD3( :, 8) = (/ &
  & 2.60461_JPRB, 0.87672_JPRB, 0.42333_JPRB /)
!-- WVL=0.5500E-06
ZALFDD3( :, 9) = (/ &
  & 2.63210_JPRB, 0.86791_JPRB, 0.42744_JPRB /)
!-- WVL=0.6450E-06
ZALFDD3( :,10) = (/ &
  & 2.60080_JPRB, 0.97691_JPRB, 0.42834_JPRB /)
!-- WVL=0.6700E-06
ZALFDD3( :,11) = (/ &
  & 2.55476_JPRB, 1.03854_JPRB, 0.42387_JPRB /)
!-- WVL=0.8000E-06
ZALFDD3( :,12) = (/ &
  & 2.22656_JPRB, 1.36000_JPRB, 0.40381_JPRB /)
!-- WVL=0.8580E-06
ZALFDD3( :,13) = (/ &
  & 2.06056_JPRB, 1.46369_JPRB, 0.40458_JPRB /)
!-- WVL=0.8650E-06
ZALFDD3( :,14) = (/ &
  & 2.04136_JPRB, 1.47390_JPRB, 0.40497_JPRB /)
!-- WVL=0.1020E-05
ZALFDD3( :,15) = (/ &
  & 1.60262_JPRB, 1.61028_JPRB, 0.43238_JPRB /)
!-- WVL=0.1064E-05
ZALFDD3( :,16) = (/ &
  & 1.49693_JPRB, 1.61080_JPRB, 0.44487_JPRB /)
!-- WVL=0.1240E-05
ZALFDD3( :,17) = (/ &
  & 1.11695_JPRB, 1.52352_JPRB, 0.50246_JPRB /)
!-- WVL=0.1640E-05
ZALFDD3( :,18) = (/ &
  & 0.58264_JPRB, 1.11666_JPRB, 0.59640_JPRB /)
!-- WVL=0.2130E-05
ZALFDD3( :,19) = (/ &
  & 0.26349_JPRB, 0.70536_JPRB, 0.60658_JPRB /)

! assymetry factor                
!-- WVL=0.3400E-06
ZASYDD3( :, 1) = (/ &
  & 0.66567_JPRB, 0.77215_JPRB, 0.81279_JPRB /)
!-- WVL=0.3550E-06
ZASYDD3( :, 2) = (/ &
  & 0.66359_JPRB, 0.76717_JPRB, 0.81003_JPRB /)
!-- WVL=0.3800E-06
ZASYDD3( :, 3) = (/ &
  & 0.66580_JPRB, 0.75738_JPRB, 0.80426_JPRB /)
!-- WVL=0.4000E-06
ZASYDD3( :, 4) = (/ &
  & 0.67305_JPRB, 0.74518_JPRB, 0.79915_JPRB /)
!-- WVL=0.4400E-06
ZASYDD3( :, 5) = (/ &
  & 0.69108_JPRB, 0.71172_JPRB, 0.79219_JPRB /)
!-- WVL=0.4690E-06
ZASYDD3( :, 6) = (/ &
  & 0.70382_JPRB, 0.67755_JPRB, 0.78870_JPRB /)
!-- WVL=0.5000E-06
ZASYDD3( :, 7) = (/ &
  & 0.71512_JPRB, 0.63977_JPRB, 0.78593_JPRB /)
!-- WVL=0.5320E-06
ZASYDD3( :, 8) = (/ &
  & 0.72563_JPRB, 0.60489_JPRB, 0.78205_JPRB /)
!-- WVL=0.5500E-06
ZASYDD3( :, 9) = (/ &
  & 0.72998_JPRB, 0.59123_JPRB, 0.77883_JPRB /)
!-- WVL=0.6450E-06
ZASYDD3( :,10) = (/ &
  & 0.74016_JPRB, 0.60153_JPRB, 0.74813_JPRB /)
!-- WVL=0.6700E-06
ZASYDD3( :,11) = (/ &
  & 0.73980_JPRB, 0.61728_JPRB, 0.73819_JPRB /)
!-- WVL=0.8000E-06
ZASYDD3( :,12) = (/ &
  & 0.73152_JPRB, 0.69785_JPRB, 0.68671_JPRB /)
!-- WVL=0.8580E-06
ZASYDD3( :,13) = (/ &
  & 0.72447_JPRB, 0.71930_JPRB, 0.67116_JPRB /)
!-- WVL=0.8650E-06
ZASYDD3( :,14) = (/ &
  & 0.72341_JPRB, 0.72198_JPRB, 0.66911_JPRB /)
!-- WVL=0.1020E-05
ZASYDD3( :,15) = (/ &
  & 0.69416_JPRB, 0.75196_JPRB, 0.65876_JPRB /)
!-- WVL=0.1064E-05
ZASYDD3( :,16) = (/ &
  & 0.68651_JPRB, 0.75389_JPRB, 0.66210_JPRB /)
!-- WVL=0.1240E-05
ZASYDD3( :,17) = (/ &
  & 0.64509_JPRB, 0.75486_JPRB, 0.68195_JPRB /)
!-- WVL=0.1640E-05
ZASYDD3( :,18) = (/ &
  & 0.55619_JPRB, 0.71559_JPRB, 0.71782_JPRB /)
!-- WVL=0.2130E-05
ZASYDD3( :,19) = (/ &
  & 0.39170_JPRB, 0.65174_JPRB, 0.72985_JPRB /)

! single scattering albedo        
!-- WVL=0.3400E-06
ZOMGDD3( :, 1) = (/ &
  & 0.96687_JPRB, 0.93452_JPRB, 0.88428_JPRB /)
!-- WVL=0.3550E-06
ZOMGDD3( :, 2) = (/ &
  & 0.96854_JPRB, 0.93719_JPRB, 0.88754_JPRB /)
!-- WVL=0.3800E-06
ZOMGDD3( :, 3) = (/ &
  & 0.97137_JPRB, 0.94125_JPRB, 0.89141_JPRB /)
!-- WVL=0.4000E-06
ZOMGDD3( :, 4) = (/ &
  & 0.97371_JPRB, 0.94371_JPRB, 0.89392_JPRB /)
!-- WVL=0.4400E-06
ZOMGDD3( :, 5) = (/ &
  & 0.97771_JPRB, 0.94655_JPRB, 0.89997_JPRB /)
!-- WVL=0.4690E-06
ZOMGDD3( :, 6) = (/ &
  & 0.98176_JPRB, 0.95182_JPRB, 0.91243_JPRB /)
!-- WVL=0.5000E-06
ZOMGDD3( :, 7) = (/ &
  & 0.98529_JPRB, 0.95739_JPRB, 0.92575_JPRB /)
!-- WVL=0.5320E-06
ZOMGDD3( :, 8) = (/ &
  & 0.98828_JPRB, 0.96345_JPRB, 0.93798_JPRB /)
!-- WVL=0.5500E-06
ZOMGDD3( :, 9) = (/ &
  & 0.98963_JPRB, 0.96724_JPRB, 0.94410_JPRB /)
!-- WVL=0.6450E-06
ZOMGDD3( :,10) = (/ &
  & 0.99514_JPRB, 0.98632_JPRB, 0.97139_JPRB /)
!-- WVL=0.6700E-06
ZOMGDD3( :,11) = (/ &
  & 0.99632_JPRB, 0.99026_JPRB, 0.97792_JPRB /)
!-- WVL=0.8000E-06
ZOMGDD3( :,12) = (/ &
  & 0.99688_JPRB, 0.99440_JPRB, 0.98168_JPRB /)
!-- WVL=0.8580E-06
ZOMGDD3( :,13) = (/ &
  & 0.99707_JPRB, 0.99541_JPRB, 0.98357_JPRB /)
!-- WVL=0.8650E-06
ZOMGDD3( :,14) = (/ &
  & 0.99711_JPRB, 0.99556_JPRB, 0.98395_JPRB /)
!-- WVL=0.1020E-05
ZOMGDD3( :,15) = (/ &
  & 0.99701_JPRB, 0.99660_JPRB, 0.98711_JPRB /)
!-- WVL=0.1064E-05
ZOMGDD3( :,16) = (/ &
  & 0.99697_JPRB, 0.99674_JPRB, 0.98799_JPRB /)
!-- WVL=0.1240E-05
ZOMGDD3( :,17) = (/ &
  & 0.99667_JPRB, 0.99712_JPRB, 0.99083_JPRB /)
!-- WVL=0.1640E-05
ZOMGDD3( :,18) = (/ &
  & 0.99568_JPRB, 0.99722_JPRB, 0.99418_JPRB /)
!-- WVL=0.2130E-05
ZOMGDD3( :,19) = (/ &
  & 0.99384_JPRB, 0.99682_JPRB, 0.99567_JPRB /)

! lidar ratio
!-- WVL=0.3400E-06
ZRALID3( :, 1) = (/ &
  &15.97238_JPRB,21.81725_JPRB,28.15672_JPRB /)
!-- WVL=0.3550E-06
ZRALID3( :, 2) = (/ &
  &16.59345_JPRB,19.22195_JPRB,29.30591_JPRB /)
!-- WVL=0.3800E-06
ZRALID3( :, 3) = (/ &
  &18.30039_JPRB,16.10490_JPRB,30.69097_JPRB /)
!-- WVL=0.4000E-06
ZRALID3( :, 4) = (/ &
  &20.28968_JPRB,14.40636_JPRB,30.82416_JPRB /)
!-- WVL=0.4400E-06
ZRALID3( :, 5) = (/ &
  &25.31147_JPRB,11.89081_JPRB,30.74137_JPRB /)
!-- WVL=0.4690E-06
ZRALID3( :, 6) = (/ &
  &29.16604_JPRB,10.77554_JPRB,27.84054_JPRB /)
!-- WVL=0.5000E-06
ZRALID3( :, 7) = (/ &
  &33.17186_JPRB, 9.98954_JPRB,25.08423_JPRB /)
!-- WVL=0.5320E-06
ZRALID3( :, 8) = (/ &
  &38.05093_JPRB, 9.48866_JPRB,22.95379_JPRB /)
!-- WVL=0.5500E-06
ZRALID3( :, 9) = (/ &
  &40.83049_JPRB, 9.86909_JPRB,20.37811_JPRB /)
!-- WVL=0.6450E-06
ZRALID3( :,10) = (/ &
  &51.89459_JPRB,12.75400_JPRB,14.96606_JPRB /)
!-- WVL=0.6700E-06
ZRALID3( :,11) = (/ &
  &56.64481_JPRB,13.28562_JPRB,14.45715_JPRB /)
!-- WVL=0.8000E-06
ZRALID3( :,12) = (/ &
  &67.15795_JPRB,24.91616_JPRB,11.82221_JPRB /)
!-- WVL=0.8580E-06
ZRALID3( :,13) = (/ &
  &76.65081_JPRB,29.06463_JPRB,11.74073_JPRB /)
!-- WVL=0.8650E-06
ZRALID3( :,14) = (/ &
  &76.85737_JPRB,29.95559_JPRB,11.69326_JPRB /)
!-- WVL=0.1020E-05
ZRALID3( :,15) = (/ &
  &73.51520_JPRB,45.95062_JPRB,12.80883_JPRB /)
!-- WVL=0.1064E-05
ZRALID3( :,16) = (/ &
  &78.55350_JPRB,48.63884_JPRB,13.39597_JPRB /)
!-- WVL=0.1240E-05
ZRALID3( :,17) = (/ &
  &74.02811_JPRB,60.89471_JPRB,16.59841_JPRB /)
!-- WVL=0.1640E-05
ZRALID3( :,18) = (/ &
  &64.26582_JPRB,72.00567_JPRB,25.48445_JPRB /)
!-- WVL=0.2130E-05
ZRALID3( :,19) = (/ &
  &30.35826_JPRB,79.08888_JPRB,36.13403_JPRB /)


!------------------------------------ DUST  9 bins ---------------------------------------------
! optical thickness    (from gems_dust_mie_ecmwf_work.f)           
!  bin    1           2           3           4           5
!         6           7           8           9
! optical thickness
!-- WVL=0.3400E-06
ZALFDD9( :, 1) = (/ &
  &  0.66217_JPRB, 2.85882_JPRB, 5.57273_JPRB, 2.11460_JPRB, 0.97209_JPRB &
  &, 0.46609_JPRB, 0.22808_JPRB, 0.09761_JPRB, 0.04988_JPRB /)
!-- WVL=0.3550E-06
ZALFDD9( :, 2) = (/ &
  &  0.56514_JPRB, 2.50285_JPRB, 5.25327_JPRB, 2.18259_JPRB, 0.96335_JPRB &
  &, 0.47279_JPRB, 0.22871_JPRB, 0.09791_JPRB, 0.04988_JPRB /)
!-- WVL=0.3800E-06
ZALFDD9( :, 3) = (/ &
  &  0.43948_JPRB, 2.04120_JPRB, 4.78079_JPRB, 2.32321_JPRB, 0.94311_JPRB &
  &, 0.48202_JPRB, 0.22718_JPRB, 0.09797_JPRB, 0.05003_JPRB /)
!-- WVL=0.4000E-06
ZALFDD9( :, 4) = (/ &
  &  0.36338_JPRB, 1.75853_JPRB, 4.45178_JPRB, 2.44173_JPRB, 0.93143_JPRB &
  &, 0.48176_JPRB, 0.22886_JPRB, 0.09833_JPRB, 0.05009_JPRB /)
!-- WVL=0.4400E-06
ZALFDD9( :, 5) = (/ &
  &  0.25534_JPRB, 1.33393_JPRB, 3.77645_JPRB, 2.65577_JPRB, 0.92767_JPRB &
  &, 0.47136_JPRB, 0.23138_JPRB, 0.09840_JPRB, 0.05008_JPRB /)
!-- WVL=0.4690E-06
ZALFDD9( :, 6) = (/ &
  &  0.19992_JPRB, 1.09905_JPRB, 3.31363_JPRB, 2.77654_JPRB, 0.93037_JPRB &
  &, 0.46913_JPRB, 0.22921_JPRB, 0.09885_JPRB, 0.05013_JPRB /)
!-- WVL=0.5000E-06
ZALFDD9( :, 7) = (/ &
  &  0.15599_JPRB, 0.89661_JPRB, 2.91326_JPRB, 2.86705_JPRB, 0.93166_JPRB &
  &, 0.47255_JPRB, 0.23100_JPRB, 0.09897_JPRB, 0.05038_JPRB /)
!-- WVL=0.5320E-06
ZALFDD9( :, 8) = (/ &
  &  0.12236_JPRB, 0.72911_JPRB, 2.57638_JPRB, 2.90489_JPRB, 0.93727_JPRB &
  &, 0.48251_JPRB, 0.23550_JPRB, 0.09906_JPRB, 0.05046_JPRB /)
!-- WVL=0.5500E-06
ZALFDD9( :, 9) = (/ &
  &  0.10727_JPRB, 0.65011_JPRB, 2.40264_JPRB, 2.90760_JPRB, 0.94260_JPRB &
  &, 0.49078_JPRB, 0.23656_JPRB, 0.09912_JPRB, 0.05039_JPRB /)
!-- WVL=0.6450E-06
ZALFDD9( :,10) = (/ &
  &  0.05580_JPRB, 0.36352_JPRB, 1.57751_JPRB, 2.77956_JPRB, 1.02148_JPRB &
  &, 0.52539_JPRB, 0.23162_JPRB, 0.10038_JPRB, 0.05075_JPRB /)
!-- WVL=0.6700E-06
ZALFDD9( :,11) = (/ &
  &  0.04728_JPRB, 0.31421_JPRB, 1.40208_JPRB, 2.72157_JPRB, 1.05649_JPRB &
  &, 0.52389_JPRB, 0.23403_JPRB, 0.09982_JPRB, 0.05064_JPRB /)
!-- WVL=0.8000E-06
ZALFDD9( :,12) = (/ &
  &  0.02430_JPRB, 0.15959_JPRB, 0.81962_JPRB, 2.29027_JPRB, 1.28583_JPRB &
  &, 0.48697_JPRB, 0.24462_JPRB, 0.10011_JPRB, 0.05099_JPRB /)
!-- WVL=0.8580E-06
ZALFDD9( :,13) = (/ &
  &  0.01875_JPRB, 0.12160_JPRB, 0.66638_JPRB, 2.08570_JPRB, 1.37386_JPRB &
  &, 0.47843_JPRB, 0.23905_JPRB, 0.10210_JPRB, 0.05090_JPRB /)
!-- WVL=0.8650E-06
ZALFDD9( :,14) = (/ &
  &  0.01817_JPRB, 0.11778_JPRB, 0.65032_JPRB, 2.06024_JPRB, 1.38422_JPRB &
  &, 0.47728_JPRB, 0.23849_JPRB, 0.10221_JPRB, 0.05090_JPRB /)
!-- WVL=0.1020E-05
ZALFDD9( :,15) = (/ &
  &  0.01024_JPRB, 0.06212_JPRB, 0.38664_JPRB, 1.58268_JPRB, 1.50979_JPRB &
  &, 0.48101_JPRB, 0.23938_JPRB, 0.10268_JPRB, 0.05156_JPRB /)
!-- WVL=0.1064E-05
ZALFDD9( :,16) = (/ &
  &  0.00890_JPRB, 0.05275_JPRB, 0.33522_JPRB, 1.46263_JPRB, 1.51458_JPRB &
  &, 0.49199_JPRB, 0.24535_JPRB, 0.10384_JPRB, 0.05133_JPRB /)
!-- WVL=0.1240E-05
ZALFDD9( :,17) = (/ &
  &  0.00548_JPRB, 0.02929_JPRB, 0.19434_JPRB, 1.05572_JPRB, 1.45793_JPRB &
  &, 0.55598_JPRB, 0.26655_JPRB, 0.10240_JPRB, 0.05172_JPRB /)
!-- WVL=0.1640E-05
ZALFDD9( :,18) = (/ &
  &  0.00255_JPRB, 0.01034_JPRB, 0.06719_JPRB, 0.51511_JPRB, 1.11691_JPRB &
  &, 0.73972_JPRB, 0.23824_JPRB, 0.10302_JPRB, 0.05214_JPRB /)
!-- WVL=0.2130E-05
ZALFDD9( :,19) = (/ &
  &  0.00146_JPRB, 0.00420_JPRB, 0.02440_JPRB, 0.23184_JPRB, 0.72035_JPRB &
  &, 0.79604_JPRB, 0.25041_JPRB, 0.10607_JPRB, 0.05407_JPRB /)

! assymetry factor                
!-- WVL=0.3400E-06
ZASYDD9( :, 1) = (/ &
  &  0.13844_JPRB, 0.55806_JPRB, 0.73658_JPRB, 0.65074_JPRB, 0.76836_JPRB &
  &, 0.80427_JPRB, 0.85082_JPRB, 0.85892_JPRB, 0.86890_JPRB /)
!-- WVL=0.3550E-06
ZASYDD9( :, 2) = (/ &
  &  0.12676_JPRB, 0.52755_JPRB, 0.72904_JPRB, 0.65829_JPRB, 0.75756_JPRB &
  &, 0.80349_JPRB, 0.84872_JPRB, 0.85872_JPRB, 0.86780_JPRB /)
!-- WVL=0.3800E-06
ZASYDD9( :, 3) = (/ &
  &  0.11044_JPRB, 0.46977_JPRB, 0.71876_JPRB, 0.67221_JPRB, 0.73927_JPRB &
  &, 0.80251_JPRB, 0.84289_JPRB, 0.85774_JPRB, 0.86671_JPRB /)
!-- WVL=0.4000E-06
ZASYDD9( :, 4) = (/ &
  &  0.09961_JPRB, 0.42364_JPRB, 0.71105_JPRB, 0.68623_JPRB, 0.72357_JPRB &
  &, 0.79867_JPRB, 0.84061_JPRB, 0.85766_JPRB, 0.86610_JPRB /)
!-- WVL=0.4400E-06
ZASYDD9( :, 5) = (/ &
  &  0.08232_JPRB, 0.34360_JPRB, 0.68685_JPRB, 0.71118_JPRB, 0.69694_JPRB &
  &, 0.78736_JPRB, 0.83611_JPRB, 0.85660_JPRB, 0.86470_JPRB /)
!-- WVL=0.4690E-06
ZASYDD9( :, 6) = (/ &
  &  0.07250_JPRB, 0.29758_JPRB, 0.66237_JPRB, 0.72318_JPRB, 0.68029_JPRB &
  &, 0.77955_JPRB, 0.82710_JPRB, 0.85389_JPRB, 0.86050_JPRB /)
!-- WVL=0.5000E-06
ZASYDD9( :, 7) = (/ &
  &  0.06384_JPRB, 0.25794_JPRB, 0.63820_JPRB, 0.73515_JPRB, 0.66024_JPRB &
  &, 0.77422_JPRB, 0.82108_JPRB, 0.85060_JPRB, 0.85616_JPRB /)
!-- WVL=0.5320E-06
ZASYDD9( :, 8) = (/ &
  &  0.05645_JPRB, 0.22511_JPRB, 0.62055_JPRB, 0.74052_JPRB, 0.64550_JPRB &
  &, 0.77145_JPRB, 0.81702_JPRB, 0.84729_JPRB, 0.85196_JPRB /)
!-- WVL=0.5500E-06
ZASYDD9( :, 9) = (/ &
  &  0.05284_JPRB, 0.20948_JPRB, 0.61325_JPRB, 0.74396_JPRB, 0.63719_JPRB &
  &, 0.77084_JPRB, 0.81341_JPRB, 0.84524_JPRB, 0.84929_JPRB /)
!-- WVL=0.6450E-06
ZASYDD9( :,10) = (/ &
  &  0.03853_JPRB, 0.14982_JPRB, 0.56884_JPRB, 0.74593_JPRB, 0.63099_JPRB &
  &, 0.76053_JPRB, 0.78467_JPRB, 0.83053_JPRB, 0.83584_JPRB /)
!-- WVL=0.6700E-06
ZASYDD9( :,11) = (/ &
  &  0.03573_JPRB, 0.13855_JPRB, 0.54829_JPRB, 0.74441_JPRB, 0.63751_JPRB &
  &, 0.75210_JPRB, 0.78106_JPRB, 0.82325_JPRB, 0.83186_JPRB /)
!-- WVL=0.8000E-06
ZASYDD9( :,12) = (/ &
  &  0.02513_JPRB, 0.09680_JPRB, 0.40588_JPRB, 0.72896_JPRB, 0.69046_JPRB &
  &, 0.69228_JPRB, 0.77826_JPRB, 0.81435_JPRB, 0.82842_JPRB /)
!-- WVL=0.8580E-06
ZASYDD9( :,13) = (/ &
  &  0.02187_JPRB, 0.08417_JPRB, 0.34861_JPRB, 0.71703_JPRB, 0.71095_JPRB &
  &, 0.66575_JPRB, 0.76900_JPRB, 0.81400_JPRB, 0.82561_JPRB /)
!-- WVL=0.8650E-06
ZASYDD9( :,14) = (/ &
  &  0.02152_JPRB, 0.08281_JPRB, 0.34238_JPRB, 0.71548_JPRB, 0.71323_JPRB &
  &, 0.66274_JPRB, 0.76668_JPRB, 0.81378_JPRB, 0.82531_JPRB /)
!-- WVL=0.1020E-05
ZASYDD9( :,15) = (/ &
  &  0.01551_JPRB, 0.05968_JPRB, 0.23767_JPRB, 0.68378_JPRB, 0.74095_JPRB &
  &, 0.62327_JPRB, 0.75868_JPRB, 0.80519_JPRB, 0.82527_JPRB /)
!-- WVL=0.1064E-05
ZASYDD9( :,16) = (/ &
  &  0.01426_JPRB, 0.05489_JPRB, 0.21690_JPRB, 0.67246_JPRB, 0.74448_JPRB &
  &, 0.62196_JPRB, 0.76075_JPRB, 0.80464_JPRB, 0.82410_JPRB /)
!-- WVL=0.1240E-05
ZASYDD9( :,17) = (/ &
  &  0.01051_JPRB, 0.04052_JPRB, 0.15707_JPRB, 0.61734_JPRB, 0.75063_JPRB &
  &, 0.63742_JPRB, 0.76280_JPRB, 0.79383_JPRB, 0.82297_JPRB /)
!-- WVL=0.1640E-05
ZASYDD9( :,18) = (/ &
  &  0.00602_JPRB, 0.02327_JPRB, 0.08916_JPRB, 0.52069_JPRB, 0.71975_JPRB &
  &, 0.71517_JPRB, 0.66896_JPRB, 0.77152_JPRB, 0.81179_JPRB /)
!-- WVL=0.2130E-05
ZASYDD9( :,19) = (/ &
  &  0.00357_JPRB, 0.01384_JPRB, 0.05303_JPRB, 0.32519_JPRB, 0.66733_JPRB &
  &, 0.74759_JPRB, 0.62101_JPRB, 0.76411_JPRB, 0.80368_JPRB /)

! single scattering albedo        
!-- WVL=0.3400E-06
ZOMGDD9( :, 1) = (/ &
  &  0.94661_JPRB, 0.98193_JPRB, 0.98853_JPRB, 0.96736_JPRB, 0.93494_JPRB &
  &, 0.89124_JPRB, 0.82579_JPRB, 0.82673_JPRB, 0.79183_JPRB /)
!-- WVL=0.3550E-06
ZOMGDD9( :, 2) = (/ &
  &  0.94108_JPRB, 0.98098_JPRB, 0.98850_JPRB, 0.96996_JPRB, 0.93607_JPRB &
  &, 0.89598_JPRB, 0.82985_JPRB, 0.82800_JPRB, 0.79476_JPRB /)
!-- WVL=0.3800E-06
ZOMGDD9( :, 3) = (/ &
  &  0.93097_JPRB, 0.97948_JPRB, 0.98847_JPRB, 0.97350_JPRB, 0.93790_JPRB &
  &, 0.90267_JPRB, 0.83472_JPRB, 0.82906_JPRB, 0.79978_JPRB /)
!-- WVL=0.4000E-06
ZOMGDD9( :, 4) = (/ &
  &  0.92210_JPRB, 0.97826_JPRB, 0.98829_JPRB, 0.97617_JPRB, 0.93948_JPRB &
  &, 0.90582_JPRB, 0.84059_JPRB, 0.82985_JPRB, 0.80341_JPRB /)
!-- WVL=0.4400E-06
ZOMGDD9( :, 5) = (/ &
  &  0.90228_JPRB, 0.97549_JPRB, 0.98766_JPRB, 0.98025_JPRB, 0.94370_JPRB &
  &, 0.90951_JPRB, 0.85104_JPRB, 0.82899_JPRB, 0.80911_JPRB /)
!-- WVL=0.4690E-06
ZOMGDD9( :, 6) = (/ &
  &  0.89519_JPRB, 0.97538_JPRB, 0.98834_JPRB, 0.98381_JPRB, 0.95145_JPRB &
  &, 0.91909_JPRB, 0.86569_JPRB, 0.83926_JPRB, 0.82504_JPRB /)
!-- WVL=0.5000E-06
ZOMGDD9( :, 7) = (/ &
  &  0.88974_JPRB, 0.97554_JPRB, 0.98924_JPRB, 0.98695_JPRB, 0.95899_JPRB &
  &, 0.93033_JPRB, 0.88395_JPRB, 0.84958_JPRB, 0.84355_JPRB /)
!-- WVL=0.5320E-06
ZOMGDD9( :, 8) = (/ &
  &  0.88591_JPRB, 0.97588_JPRB, 0.99012_JPRB, 0.98944_JPRB, 0.96611_JPRB &
  &, 0.94164_JPRB, 0.90256_JPRB, 0.85913_JPRB, 0.86184_JPRB /)
!-- WVL=0.5500E-06
ZOMGDD9( :, 9) = (/ &
  &  0.88491_JPRB, 0.97623_JPRB, 0.99059_JPRB, 0.99069_JPRB, 0.96974_JPRB &
  &, 0.94791_JPRB, 0.91187_JPRB, 0.86469_JPRB, 0.87168_JPRB /)
!-- WVL=0.6450E-06
ZOMGDD9( :,10) = (/ &
  &  0.89968_JPRB, 0.98136_JPRB, 0.99366_JPRB, 0.99552_JPRB, 0.98679_JPRB &
  &, 0.97502_JPRB, 0.95362_JPRB, 0.91429_JPRB, 0.92652_JPRB /)
!-- WVL=0.6700E-06
ZOMGDD9( :,11) = (/ &
  &  0.91191_JPRB, 0.98409_JPRB, 0.99481_JPRB, 0.99657_JPRB, 0.99044_JPRB &
  &, 0.98087_JPRB, 0.96457_JPRB, 0.93172_JPRB, 0.94178_JPRB /)
!-- WVL=0.8000E-06
ZOMGDD9( :,12) = (/ &
  &  0.87199_JPRB, 0.97750_JPRB, 0.99412_JPRB, 0.99698_JPRB, 0.99406_JPRB &
  &, 0.98392_JPRB, 0.97240_JPRB, 0.94349_JPRB, 0.94727_JPRB /)
!-- WVL=0.8580E-06
ZOMGDD9( :,13) = (/ &
  &  0.85372_JPRB, 0.97434_JPRB, 0.99386_JPRB, 0.99709_JPRB, 0.99512_JPRB &
  &, 0.98539_JPRB, 0.97421_JPRB, 0.94964_JPRB, 0.94908_JPRB /)
!-- WVL=0.8650E-06
ZOMGDD9( :,14) = (/ &
  &  0.85284_JPRB, 0.97421_JPRB, 0.99389_JPRB, 0.99713_JPRB, 0.99527_JPRB &
  &, 0.98573_JPRB, 0.97459_JPRB, 0.95082_JPRB, 0.94983_JPRB /)
!-- WVL=0.1020E-05
ZOMGDD9( :,15) = (/ &
  &  0.78181_JPRB, 0.96035_JPRB, 0.99196_JPRB, 0.99698_JPRB, 0.99636_JPRB &
  &, 0.98807_JPRB, 0.97758_JPRB, 0.95711_JPRB, 0.94651_JPRB /)
!-- WVL=0.1064E-05
ZOMGDD9( :,16) = (/ &
  &  0.75995_JPRB, 0.95567_JPRB, 0.99127_JPRB, 0.99690_JPRB, 0.99656_JPRB &
  &, 0.98882_JPRB, 0.97879_JPRB, 0.95895_JPRB, 0.94417_JPRB /)
!-- WVL=0.1240E-05
ZOMGDD9( :,17) = (/ &
  &  0.66858_JPRB, 0.93357_JPRB, 0.98786_JPRB, 0.99656_JPRB, 0.99699_JPRB &
  &, 0.99155_JPRB, 0.98281_JPRB, 0.96339_JPRB, 0.93854_JPRB /)
!-- WVL=0.1640E-05
ZOMGDD9( :,18) = (/ &
  &  0.46838_JPRB, 0.86319_JPRB, 0.97598_JPRB, 0.99527_JPRB, 0.99719_JPRB &
  &, 0.99531_JPRB, 0.98508_JPRB, 0.97091_JPRB, 0.94913_JPRB /)
!-- WVL=0.2130E-05
ZOMGDD9( :,19) = (/ &
  &  0.28797_JPRB, 0.74625_JPRB, 0.95244_JPRB, 0.99326_JPRB, 0.99685_JPRB &
  &, 0.99676_JPRB, 0.98903_JPRB, 0.97616_JPRB, 0.96044_JPRB /)

! lidar ratio
!-- WVL=0.3400E-06
ZRALID9( :, 1) = (/ &
  & 12.71086_JPRB,98.97321_JPRB,73.69738_JPRB,15.47308_JPRB,22.04511_JPRB &
  &,22.09013_JPRB,43.21375_JPRB,30.45018_JPRB,21.20038_JPRB /)
!-- WVL=0.3550E-06
ZRALID9( :, 2) = (/ &
  & 12.36526_JPRB,78.77721_JPRB,71.11851_JPRB,17.66363_JPRB,18.83320_JPRB &
  &,23.16846_JPRB,45.50775_JPRB,27.37975_JPRB,19.28775_JPRB /)
!-- WVL=0.3800E-06
ZRALID9( :, 3) = (/ &
  & 11.94132_JPRB,51.25376_JPRB,80.86835_JPRB,19.89788_JPRB,16.22179_JPRB &
  &,25.38117_JPRB,47.95069_JPRB,32.26424_JPRB,22.13168_JPRB /)
!-- WVL=0.4000E-06
ZRALID9( :, 4) = (/ &
  & 11.70365_JPRB,38.35840_JPRB,93.68872_JPRB,22.97060_JPRB,14.39161_JPRB &
  &,26.99193_JPRB,48.09772_JPRB,35.78511_JPRB,23.11306_JPRB /)
!-- WVL=0.4400E-06
ZRALID9( :, 5) = (/ &
  & 11.41824_JPRB,25.57187_JPRB,93.06123_JPRB,30.13552_JPRB,12.42413_JPRB &
  &,29.11519_JPRB,42.87694_JPRB,35.33555_JPRB,27.69741_JPRB /)
!-- WVL=0.4690E-06
ZRALID9( :, 6) = (/ &
  & 11.21460_JPRB,21.05433_JPRB,77.15617_JPRB,33.44013_JPRB,11.91442_JPRB &
  &,28.54418_JPRB,32.92681_JPRB,30.33715_JPRB,21.48635_JPRB /)
!-- WVL=0.5000E-06
ZRALID9( :, 7) = (/ &
  & 11.03218_JPRB,18.11273_JPRB,68.65958_JPRB,39.73633_JPRB,11.30428_JPRB &
  &,27.92285_JPRB,25.30770_JPRB,29.71380_JPRB,24.68257_JPRB /)
!-- WVL=0.5320E-06
ZRALID9( :, 8) = (/ &
  & 10.87103_JPRB,16.14366_JPRB,69.86060_JPRB,45.22785_JPRB,11.18433_JPRB &
  &,26.05700_JPRB,20.75048_JPRB,31.37545_JPRB,27.57026_JPRB /)
!-- WVL=0.5500E-06
ZRALID9( :, 9) = (/ &
  & 10.78369_JPRB,15.32120_JPRB,73.54238_JPRB,47.63467_JPRB,11.10534_JPRB &
  &,26.20325_JPRB,18.81441_JPRB,33.07014_JPRB,24.59806_JPRB /)
!-- WVL=0.6450E-06
ZRALID9( :,10) = (/ &
  & 10.23034_JPRB,12.66954_JPRB,91.69456_JPRB,61.94198_JPRB,13.08815_JPRB &
  &,17.96592_JPRB,15.02372_JPRB,23.50856_JPRB,21.35328_JPRB /)
!-- WVL=0.6700E-06
ZRALID9( :,11) = (/ &
  & 10.02288_JPRB,12.22686_JPRB,83.27264_JPRB,61.40295_JPRB,14.18845_JPRB &
  &,16.18191_JPRB,15.16450_JPRB,19.02248_JPRB,19.22158_JPRB /)
!-- WVL=0.8000E-06
ZRALID9( :,12) = (/ &
  & 10.21033_JPRB,10.95501_JPRB,33.73722_JPRB,78.66150_JPRB,22.49598_JPRB &
  &,11.15684_JPRB,21.55258_JPRB,19.44088_JPRB,20.12921_JPRB /)
!-- WVL=0.8580E-06
ZRALID9( :,13) = (/ &
  & 10.34567_JPRB,10.62460_JPRB,25.51583_JPRB,73.73571_JPRB,28.16175_JPRB &
  &,10.18025_JPRB,23.10707_JPRB,21.02596_JPRB,18.01932_JPRB /)
!-- WVL=0.8650E-06
ZRALID9( :,14) = (/ &
  & 10.34750_JPRB,10.58788_JPRB,24.81738_JPRB,73.31236_JPRB,29.03691_JPRB &
  &,10.06797_JPRB,22.82287_JPRB,21.54644_JPRB,18.17011_JPRB /)
!-- WVL=0.1020E-05
ZRALID9( :,15) = (/ &
  & 11.12323_JPRB,10.11153_JPRB,16.55293_JPRB,88.21932_JPRB,41.20959_JPRB &
  &,10.16291_JPRB,21.81913_JPRB,25.78627_JPRB,17.53583_JPRB /)
!-- WVL=0.1064E-05
ZRALID9( :,16) = (/ &
  & 11.40851_JPRB,10.03699_JPRB,15.44372_JPRB,87.07449_JPRB,45.27345_JPRB &
  &,10.64050_JPRB,19.70546_JPRB,24.56415_JPRB,19.67665_JPRB /)
!-- WVL=0.1240E-05
ZRALID9( :,17) = (/ &
  & 12.85060_JPRB, 9.90804_JPRB,12.85268_JPRB,65.66467_JPRB,62.27370_JPRB &
  &,13.60343_JPRB,16.40448_JPRB,14.77298_JPRB,22.64285_JPRB /)
!-- WVL=0.1640E-05
ZRALID9( :,18) = (/ &
  & 18.14597_JPRB,10.26723_JPRB,10.74738_JPRB,60.47050_JPRB,74.92571_JPRB &
  &,27.85699_JPRB,10.87566_JPRB,13.77573_JPRB,19.84617_JPRB /)
!-- WVL=0.2130E-05
ZRALID9( :,19) = (/ &
  & 29.34171_JPRB,11.60604_JPRB,10.02310_JPRB,22.78763_JPRB,90.94428_JPRB &
  &,49.63645_JPRB,11.29840_JPRB,27.22783_JPRB,24.76432_JPRB /)


IF (NTYPAER(2) /= 9 ) THEN
  ALF_DD(1:3,1:19) = ZALFDD3(1:3,1:19)
  ASY_DD(1:3,1:19) = ZASYDD3(1:3,1:19)
  OMG_DD(1:3,1:19) = ZOMGDD3(1:3,1:19)
  RALI_DD(1:3,1:19)= ZRALID3(1:3,1:19)
ELSE
  ALF_DD(1:9,1:19) = ZALFDD9(1:9,1:19)
  ASY_DD(1:9,1:19) = ZASYDD9(1:9,1:19)
  OMG_DD(1:9,1:19) = ZOMGDD9(1:9,1:19)
  RALI_DD(1:9,1:19)= ZRALID9(1:9,1:19)
ENDIF

!------------------------------------ SEA SALT  3 bins ---------------------------------------------

!-- For sea-salt, XX( RH, wavelength, binNo)  from gems_ss_mie_ecmwf_work.f

! real and imaginary parta of the refractive index given by ss_ri_table
!  produced by gems_ss.f

! The particle size distribution parameters is defined by two log-normal dsitributions

! r_0_80RH  = 0.19921, 1.992   ! um
! sigma_g   = 1.9    , 2.0
! Ntot      = 70.    , 3.0
! rh_growth = 0.503, 0.503, 0.503, 0.503, 0.724, 0.782, 0.838, 0.905, 1.000, 1.072, 1.188, 1.447
! rmin      = 0.03E-6
! rmax      = 20.0E-6
! rho       = 1.183E+3

  
 ! ** BIN No.            1   3.0000000000000000E-002   0.5000000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS3( :, 1, 1) = (/ &
  & 1.53511_JPRB, 1.53511_JPRB, 1.53511_JPRB, 1.53511_JPRB, 3.45281_JPRB, 3.90752_JPRB, &
  & 4.35269_JPRB, 4.90314_JPRB, 5.79278_JPRB, 6.58049_JPRB, 7.98171_JPRB,11.61997_JPRB /)
!-- WVL=0.3550E-06
ZALFSS3( :, 2, 1) = (/ &
  & 1.48093_JPRB, 1.48093_JPRB, 1.48093_JPRB, 1.48093_JPRB, 3.48296_JPRB, 3.98264_JPRB, &
  & 4.44157_JPRB, 4.99203_JPRB, 5.85967_JPRB, 6.61663_JPRB, 8.05283_JPRB,11.66881_JPRB /)
!-- WVL=0.3800E-06
ZALFSS3( :, 3, 1) = (/ &
  & 1.37713_JPRB, 1.37713_JPRB, 1.37713_JPRB, 1.37713_JPRB, 3.51520_JPRB, 4.05625_JPRB, &
  & 4.57141_JPRB, 5.15995_JPRB, 6.01391_JPRB, 6.73067_JPRB, 8.23299_JPRB,11.72169_JPRB /)
!-- WVL=0.4000E-06
ZALFSS3( :, 4, 1) = (/ &
  & 1.29931_JPRB, 1.29931_JPRB, 1.29931_JPRB, 1.29931_JPRB, 3.49222_JPRB, 4.09548_JPRB, &
  & 4.64331_JPRB, 5.27099_JPRB, 6.15169_JPRB, 6.86098_JPRB, 8.42299_JPRB,11.77559_JPRB /)
!-- WVL=0.4400E-06
ZALFSS3( :, 5, 1) = (/ &
  & 1.14169_JPRB, 1.14169_JPRB, 1.14169_JPRB, 1.14169_JPRB, 3.39968_JPRB, 4.06158_JPRB, &
  & 4.70932_JPRB, 5.43313_JPRB, 6.42236_JPRB, 7.16447_JPRB, 8.80339_JPRB,12.04866_JPRB /)
!-- WVL=0.4690E-06
ZALFSS3( :, 6, 1) = (/ &
  & 1.02888_JPRB, 1.02888_JPRB, 1.02888_JPRB, 1.02888_JPRB, 3.27173_JPRB, 3.99065_JPRB, &
  & 4.66828_JPRB, 5.49116_JPRB, 6.58478_JPRB, 7.36188_JPRB, 9.06345_JPRB,12.34178_JPRB /)
!-- WVL=0.5000E-06
ZALFSS3( :, 7, 1) = (/ &
  & 0.92536_JPRB, 0.92536_JPRB, 0.92536_JPRB, 0.92536_JPRB, 3.12766_JPRB, 3.84836_JPRB, &
  & 4.59247_JPRB, 5.45659_JPRB, 6.67235_JPRB, 7.55519_JPRB, 9.20811_JPRB,12.69834_JPRB /)
!-- WVL=0.5320E-06
ZALFSS3( :, 8, 1) = (/ &
  & 0.82776_JPRB, 0.82776_JPRB, 0.82776_JPRB, 0.82776_JPRB, 2.94311_JPRB, 3.69264_JPRB, &
  & 4.43920_JPRB, 5.38504_JPRB, 6.70167_JPRB, 7.65848_JPRB, 9.28143_JPRB,13.08190_JPRB /)
!-- WVL=0.5500E-06
ZALFSS3( :, 9, 1) = (/ &
  & 0.77456_JPRB, 0.77456_JPRB, 0.77456_JPRB, 0.77456_JPRB, 2.83535_JPRB, 3.59098_JPRB, &
  & 4.34701_JPRB, 5.31201_JPRB, 6.66770_JPRB, 7.58968_JPRB, 9.25394_JPRB,13.25057_JPRB /)
!-- WVL=0.6450E-06
ZALFSS3( :,10, 1) = (/ &
  & 0.54344_JPRB, 0.54344_JPRB, 0.54344_JPRB, 0.54344_JPRB, 2.29746_JPRB, 3.00251_JPRB, &
  & 3.74303_JPRB, 4.74889_JPRB, 6.25859_JPRB, 6.73082_JPRB, 8.78837_JPRB,13.76983_JPRB /)
!-- WVL=0.6700E-06
ZALFSS3( :,11, 1) = (/ &
  & 0.49846_JPRB, 0.49846_JPRB, 0.49846_JPRB, 0.49846_JPRB, 2.15826_JPRB, 2.84806_JPRB, &
  & 3.58355_JPRB, 4.56051_JPRB, 6.10579_JPRB, 6.50467_JPRB, 8.64527_JPRB,13.72849_JPRB /)
!-- WVL=0.8000E-06
ZALFSS3( :,12, 1) = (/ &
  & 0.31182_JPRB, 0.31182_JPRB, 0.31182_JPRB, 0.31182_JPRB, 1.56988_JPRB, 2.12781_JPRB, &
  & 2.74694_JPRB, 3.63500_JPRB, 5.09939_JPRB, 5.29071_JPRB, 7.41251_JPRB,12.89521_JPRB /)
!-- WVL=0.8580E-06
ZALFSS3( :,13, 1) = (/ &
  & 0.25047_JPRB, 0.25047_JPRB, 0.25047_JPRB, 0.25047_JPRB, 1.34491_JPRB, 1.85886_JPRB, &
  & 2.43986_JPRB, 3.24046_JPRB, 4.64749_JPRB, 4.78939_JPRB, 6.78814_JPRB,12.25120_JPRB /)
!-- WVL=0.8650E-06
ZALFSS3( :,14, 1) = (/ &
  & 0.24406_JPRB, 0.24406_JPRB, 0.24406_JPRB, 0.24406_JPRB, 1.32040_JPRB, 1.82740_JPRB, &
  & 2.40441_JPRB, 3.19742_JPRB, 4.59113_JPRB, 4.72965_JPRB, 6.71526_JPRB,12.16272_JPRB /)
!-- WVL=0.1020E-05
ZALFSS3( :,15, 1) = (/ &
  & 0.14502_JPRB, 0.14502_JPRB, 0.14502_JPRB, 0.14502_JPRB, 0.90350_JPRB, 1.26391_JPRB, &
  & 1.69608_JPRB, 2.35075_JPRB, 3.49784_JPRB, 3.74555_JPRB, 5.23875_JPRB,10.19511_JPRB /)
!-- WVL=0.1064E-05
ZALFSS3( :,16, 1) = (/ &
  & 0.12700_JPRB, 0.12700_JPRB, 0.12700_JPRB, 0.12700_JPRB, 0.81028_JPRB, 1.14664_JPRB, &
  & 1.54228_JPRB, 2.13956_JPRB, 3.23402_JPRB, 3.87779_JPRB, 4.86243_JPRB, 9.63245_JPRB /)
!-- WVL=0.1240E-05
ZALFSS3( :,17, 1) = (/ &
  & 0.07693_JPRB, 0.07693_JPRB, 0.07693_JPRB, 0.07693_JPRB, 0.51224_JPRB, 0.76059_JPRB, &
  & 1.06569_JPRB, 1.51048_JPRB, 2.31620_JPRB, 2.68063_JPRB, 3.63491_JPRB, 7.60703_JPRB /)
!-- WVL=0.1640E-05
ZALFSS3( :,18, 1) = (/ &
  & 0.02778_JPRB, 0.02778_JPRB, 0.02778_JPRB, 0.02778_JPRB, 0.21130_JPRB, 0.31540_JPRB, &
  & 0.44985_JPRB, 0.66850_JPRB, 0.81592_JPRB, 1.15745_JPRB, 1.89517_JPRB, 4.35832_JPRB /)
!-- WVL=0.2130E-05
ZALFSS3( :,19, 1) = (/ &
  & 0.01003_JPRB, 0.01003_JPRB, 0.01003_JPRB, 0.01003_JPRB, 0.08477_JPRB, 0.13107_JPRB, &
  & 0.19219_JPRB, 0.29110_JPRB, 0.36724_JPRB, 0.52288_JPRB, 0.87678_JPRB, 2.28350_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS3( :, 1, 1) = (/ &
  & 0.71041_JPRB, 0.71041_JPRB, 0.71041_JPRB, 0.71041_JPRB, 0.68746_JPRB, 0.67545_JPRB, &
  & 0.66356_JPRB, 0.65241_JPRB, 0.64666_JPRB, 0.64878_JPRB, 0.69249_JPRB, 0.70737_JPRB /)
!-- WVL=0.3550E-06
ZASYSS3( :, 2, 1) = (/ &
  & 0.70770_JPRB, 0.70770_JPRB, 0.70770_JPRB, 0.70770_JPRB, 0.69491_JPRB, 0.68277_JPRB, &
  & 0.67027_JPRB, 0.65816_JPRB, 0.64749_JPRB, 0.64715_JPRB, 0.69379_JPRB, 0.70420_JPRB /)
!-- WVL=0.3800E-06
ZASYSS3( :, 3, 1) = (/ &
  & 0.70223_JPRB, 0.70223_JPRB, 0.70223_JPRB, 0.70223_JPRB, 0.70247_JPRB, 0.69325_JPRB, &
  & 0.68262_JPRB, 0.66876_JPRB, 0.65372_JPRB, 0.64742_JPRB, 0.69871_JPRB, 0.69940_JPRB /)
!-- WVL=0.4000E-06
ZASYSS3( :, 4, 1) = (/ &
  & 0.69723_JPRB, 0.69723_JPRB, 0.69723_JPRB, 0.69723_JPRB, 0.70703_JPRB, 0.70036_JPRB, &
  & 0.68998_JPRB, 0.67876_JPRB, 0.66158_JPRB, 0.65176_JPRB, 0.70410_JPRB, 0.69777_JPRB /)
!-- WVL=0.4400E-06
ZASYSS3( :, 5, 1) = (/ &
  & 0.68178_JPRB, 0.68178_JPRB, 0.68178_JPRB, 0.68178_JPRB, 0.71139_JPRB, 0.70878_JPRB, &
  & 0.70250_JPRB, 0.69335_JPRB, 0.67793_JPRB, 0.66517_JPRB, 0.71879_JPRB, 0.69970_JPRB /)
!-- WVL=0.4690E-06
ZASYSS3( :, 6, 1) = (/ &
  & 0.66966_JPRB, 0.66966_JPRB, 0.66966_JPRB, 0.66966_JPRB, 0.71154_JPRB, 0.71133_JPRB, &
  & 0.70831_JPRB, 0.70148_JPRB, 0.68727_JPRB, 0.67680_JPRB, 0.73428_JPRB, 0.70661_JPRB /)
!-- WVL=0.5000E-06
ZASYSS3( :, 7, 1) = (/ &
  & 0.65812_JPRB, 0.65812_JPRB, 0.65812_JPRB, 0.65812_JPRB, 0.70921_JPRB, 0.71154_JPRB, &
  & 0.71128_JPRB, 0.70703_JPRB, 0.69789_JPRB, 0.68648_JPRB, 0.74497_JPRB, 0.71543_JPRB /)
!-- WVL=0.5320E-06
ZASYSS3( :, 8, 1) = (/ &
  & 0.64449_JPRB, 0.64449_JPRB, 0.64449_JPRB, 0.64449_JPRB, 0.70448_JPRB, 0.71009_JPRB, &
  & 0.71150_JPRB, 0.71102_JPRB, 0.70349_JPRB, 0.71135_JPRB, 0.75093_JPRB, 0.72468_JPRB /)
!-- WVL=0.5500E-06
ZASYSS3( :, 9, 1) = (/ &
  & 0.63517_JPRB, 0.63517_JPRB, 0.63517_JPRB, 0.63517_JPRB, 0.70177_JPRB, 0.70795_JPRB, &
  & 0.71140_JPRB, 0.71139_JPRB, 0.70656_JPRB, 0.73368_JPRB, 0.75378_JPRB, 0.73058_JPRB /)
!-- WVL=0.6450E-06
ZASYSS3( :,10, 1) = (/ &
  & 0.57965_JPRB, 0.57965_JPRB, 0.57965_JPRB, 0.57965_JPRB, 0.67826_JPRB, 0.69216_JPRB, &
  & 0.70064_JPRB, 0.70694_JPRB, 0.71156_JPRB, 0.75222_JPRB, 0.75896_JPRB, 0.75058_JPRB /)
!-- WVL=0.6700E-06
ZASYSS3( :,11, 1) = (/ &
  & 0.56807_JPRB, 0.56807_JPRB, 0.56807_JPRB, 0.56807_JPRB, 0.67104_JPRB, 0.68567_JPRB, &
  & 0.69656_JPRB, 0.70387_JPRB, 0.71080_JPRB, 0.75001_JPRB, 0.75663_JPRB, 0.75379_JPRB /)
!-- WVL=0.8000E-06
ZASYSS3( :,12, 1) = (/ &
  & 0.51110_JPRB, 0.51110_JPRB, 0.51110_JPRB, 0.51110_JPRB, 0.63192_JPRB, 0.65233_JPRB, &
  & 0.66544_JPRB, 0.67977_JPRB, 0.69649_JPRB, 0.73135_JPRB, 0.74390_JPRB, 0.75936_JPRB /)
!-- WVL=0.8580E-06
ZASYSS3( :,13, 1) = (/ &
  & 0.47329_JPRB, 0.47329_JPRB, 0.47329_JPRB, 0.47329_JPRB, 0.60737_JPRB, 0.63413_JPRB, &
  & 0.65215_JPRB, 0.66667_JPRB, 0.68541_JPRB, 0.71995_JPRB, 0.73575_JPRB, 0.75661_JPRB /)
!-- WVL=0.8650E-06
ZASYSS3( :,14, 1) = (/ &
  & 0.46811_JPRB, 0.46811_JPRB, 0.46811_JPRB, 0.46811_JPRB, 0.60436_JPRB, 0.63159_JPRB, &
  & 0.65035_JPRB, 0.66523_JPRB, 0.68391_JPRB, 0.71844_JPRB, 0.73469_JPRB, 0.75612_JPRB /)
!-- WVL=0.1020E-05
ZASYSS3( :,15, 1) = (/ &
  & 0.34720_JPRB, 0.34720_JPRB, 0.34720_JPRB, 0.34720_JPRB, 0.55238_JPRB, 0.57431_JPRB, &
  & 0.59755_JPRB, 0.62541_JPRB, 0.65296_JPRB, 0.68068_JPRB, 0.70586_JPRB, 0.74144_JPRB /)
!-- WVL=0.1064E-05
ZASYSS3( :,16, 1) = (/ &
  & 0.31739_JPRB, 0.31739_JPRB, 0.31739_JPRB, 0.31739_JPRB, 0.54021_JPRB, 0.56202_JPRB, &
  & 0.58287_JPRB, 0.61031_JPRB, 0.64292_JPRB, 0.66451_JPRB, 0.69688_JPRB, 0.73584_JPRB /)
!-- WVL=0.1240E-05
ZASYSS3( :,17, 1) = (/ &
  & 0.22798_JPRB, 0.22798_JPRB, 0.22798_JPRB, 0.22798_JPRB, 0.47072_JPRB, 0.51245_JPRB, &
  & 0.53809_JPRB, 0.56007_JPRB, 0.59092_JPRB, 0.62030_JPRB, 0.65972_JPRB, 0.70808_JPRB /)
!-- WVL=0.1640E-05
ZASYSS3( :,18, 1) = (/ &
  & 0.12740_JPRB, 0.12740_JPRB, 0.12740_JPRB, 0.12740_JPRB, 0.27369_JPRB, 0.32330_JPRB, &
  & 0.37358_JPRB, 0.43164_JPRB, 0.48402_JPRB, 0.52316_JPRB, 0.56361_JPRB, 0.63153_JPRB /)
!-- WVL=0.2130E-05
ZASYSS3( :,19, 1) = (/ &
  & 0.07546_JPRB, 0.07546_JPRB, 0.07546_JPRB, 0.07546_JPRB, 0.15720_JPRB, 0.18451_JPRB, &
  & 0.21355_JPRB, 0.25190_JPRB, 0.29853_JPRB, 0.34521_JPRB, 0.42119_JPRB, 0.54010_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS3( :, 1, 1) = (/ &
  & 0.99895_JPRB, 0.99895_JPRB, 0.99895_JPRB, 0.99895_JPRB, 0.99851_JPRB, 0.99834_JPRB, &
  & 0.99814_JPRB, 0.99790_JPRB, 0.99758_JPRB, 0.99736_JPRB, 0.99901_JPRB, 0.99898_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS3( :, 2, 1) = (/ &
  & 0.99896_JPRB, 0.99896_JPRB, 0.99896_JPRB, 0.99896_JPRB, 0.99861_JPRB, 0.99844_JPRB, &
  & 0.99826_JPRB, 0.99804_JPRB, 0.99771_JPRB, 0.99750_JPRB, 0.99907_JPRB, 0.99903_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS3( :, 3, 1) = (/ &
  & 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99871_JPRB, 0.99859_JPRB, &
  & 0.99844_JPRB, 0.99823_JPRB, 0.99793_JPRB, 0.99771_JPRB, 0.99915_JPRB, 0.99910_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS3( :, 4, 1) = (/ &
  & 0.99899_JPRB, 0.99899_JPRB, 0.99899_JPRB, 0.99899_JPRB, 0.99879_JPRB, 0.99868_JPRB, &
  & 0.99855_JPRB, 0.99838_JPRB, 0.99810_JPRB, 0.99788_JPRB, 0.99921_JPRB, 0.99916_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS3( :, 5, 1) = (/ &
  & 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99888_JPRB, 0.99881_JPRB, &
  & 0.99871_JPRB, 0.99859_JPRB, 0.99837_JPRB, 0.99817_JPRB, 0.99932_JPRB, 0.99925_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS3( :, 6, 1) = (/ &
  & 0.99896_JPRB, 0.99896_JPRB, 0.99896_JPRB, 0.99896_JPRB, 0.99893_JPRB, 0.99887_JPRB, &
  & 0.99881_JPRB, 0.99869_JPRB, 0.99851_JPRB, 0.99835_JPRB, 0.99947_JPRB, 0.99932_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS3( :, 7, 1) = (/ &
  & 0.99894_JPRB, 0.99894_JPRB, 0.99894_JPRB, 0.99894_JPRB, 0.99896_JPRB, 0.99892_JPRB, &
  & 0.99886_JPRB, 0.99879_JPRB, 0.99864_JPRB, 0.99850_JPRB, 0.99954_JPRB, 0.99938_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS3( :, 8, 1) = (/ &
  & 0.99891_JPRB, 0.99891_JPRB, 0.99891_JPRB, 0.99891_JPRB, 0.99898_JPRB, 0.99896_JPRB, &
  & 0.99892_JPRB, 0.99885_JPRB, 0.99873_JPRB, 0.99886_JPRB, 0.99957_JPRB, 0.99944_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS3( :, 9, 1) = (/ &
  & 0.99888_JPRB, 0.99888_JPRB, 0.99888_JPRB, 0.99888_JPRB, 0.99899_JPRB, 0.99896_JPRB, &
  & 0.99894_JPRB, 0.99888_JPRB, 0.99878_JPRB, 0.99921_JPRB, 0.99959_JPRB, 0.99947_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS3( :,10, 1) = (/ &
  & 0.99875_JPRB, 0.99875_JPRB, 0.99875_JPRB, 0.99875_JPRB, 0.99897_JPRB, 0.99899_JPRB, &
  & 0.99899_JPRB, 0.99897_JPRB, 0.99893_JPRB, 0.99959_JPRB, 0.99963_JPRB, 0.99957_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS3( :,11, 1) = (/ &
  & 0.99871_JPRB, 0.99871_JPRB, 0.99871_JPRB, 0.99871_JPRB, 0.99897_JPRB, 0.99898_JPRB, &
  & 0.99899_JPRB, 0.99898_JPRB, 0.99895_JPRB, 0.99959_JPRB, 0.99960_JPRB, 0.99959_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS3( :,12, 1) = (/ &
  & 0.99840_JPRB, 0.99840_JPRB, 0.99840_JPRB, 0.99840_JPRB, 0.99888_JPRB, 0.99893_JPRB, &
  & 0.99896_JPRB, 0.99898_JPRB, 0.99899_JPRB, 0.99960_JPRB, 0.99960_JPRB, 0.99964_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS3( :,13, 1) = (/ &
  & 0.99824_JPRB, 0.99824_JPRB, 0.99824_JPRB, 0.99824_JPRB, 0.99882_JPRB, 0.99888_JPRB, &
  & 0.99893_JPRB, 0.99896_JPRB, 0.99898_JPRB, 0.99959_JPRB, 0.99960_JPRB, 0.99965_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS3( :,14, 1) = (/ &
  & 0.99822_JPRB, 0.99822_JPRB, 0.99822_JPRB, 0.99822_JPRB, 0.99881_JPRB, 0.99888_JPRB, &
  & 0.99892_JPRB, 0.99896_JPRB, 0.99898_JPRB, 0.99959_JPRB, 0.99960_JPRB, 0.99965_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS3( :,15, 1) = (/ &
  & 0.99775_JPRB, 0.99775_JPRB, 0.99775_JPRB, 0.99775_JPRB, 0.99863_JPRB, 0.99873_JPRB, &
  & 0.99880_JPRB, 0.99886_JPRB, 0.99893_JPRB, 0.99942_JPRB, 0.99958_JPRB, 0.99966_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS3( :,16, 1) = (/ &
  & 0.99760_JPRB, 0.99760_JPRB, 0.99760_JPRB, 0.99760_JPRB, 0.99856_JPRB, 0.99868_JPRB, &
  & 0.99876_JPRB, 0.99883_JPRB, 0.99890_JPRB, 0.99914_JPRB, 0.99957_JPRB, 0.99966_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS3( :,17, 1) = (/ &
  & 0.99685_JPRB, 0.99685_JPRB, 0.99685_JPRB, 0.99685_JPRB, 0.99823_JPRB, 0.99841_JPRB, &
  & 0.99855_JPRB, 0.99867_JPRB, 0.99878_JPRB, 0.99917_JPRB, 0.99953_JPRB, 0.99964_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS3( :,18, 1) = (/ &
  & 0.99409_JPRB, 0.99409_JPRB, 0.99409_JPRB, 0.99409_JPRB, 0.99731_JPRB, 0.99763_JPRB, &
  & 0.99787_JPRB, 0.99809_JPRB, 0.99922_JPRB, 0.99930_JPRB, 0.99939_JPRB, 0.99957_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS3( :,19, 1) = (/ &
  & 0.98836_JPRB, 0.98836_JPRB, 0.98835_JPRB, 0.98835_JPRB, 0.99539_JPRB, 0.99612_JPRB, &
  & 0.99665_JPRB, 0.99712_JPRB, 0.99885_JPRB, 0.99897_JPRB, 0.99912_JPRB, 0.99943_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS3( :, 1, 1) = (/ &
  & 56.0557_JPRB, 56.0557_JPRB, 56.0557_JPRB, 56.0557_JPRB, 25.6490_JPRB, 21.6896_JPRB, &
  & 18.0394_JPRB, 14.5273_JPRB, 12.1561_JPRB, 10.8644_JPRB, 24.9996_JPRB, 24.1211_JPRB /)
!-- WVL=0.3550E-06
ZRALIS3( :, 2, 1) = (/ &
  & 56.8930_JPRB, 56.8930_JPRB, 56.8930_JPRB, 56.8930_JPRB, 29.4168_JPRB, 24.1928_JPRB, &
  & 19.3507_JPRB, 16.7825_JPRB, 13.1500_JPRB, 11.7087_JPRB, 27.0075_JPRB, 25.0079_JPRB /)
!-- WVL=0.3800E-06
ZRALIS3( :, 3, 1) = (/ &
  & 64.8275_JPRB, 64.8275_JPRB, 64.8275_JPRB, 64.8275_JPRB, 34.4119_JPRB, 28.5291_JPRB, &
  & 24.1523_JPRB, 18.8881_JPRB, 14.8463_JPRB, 13.0928_JPRB, 29.2025_JPRB, 26.0949_JPRB /)
!-- WVL=0.4000E-06
ZRALIS3( :, 4, 1) = (/ &
  & 70.5650_JPRB, 70.5650_JPRB, 70.5650_JPRB, 70.5650_JPRB, 39.3353_JPRB, 32.5222_JPRB, &
  & 26.7900_JPRB, 23.0137_JPRB, 17.7492_JPRB, 14.4136_JPRB, 30.5850_JPRB, 27.9106_JPRB /)
!-- WVL=0.4400E-06
ZRALIS3( :, 5, 1) = (/ &
  & 67.9159_JPRB, 67.9159_JPRB, 67.9159_JPRB, 67.9159_JPRB, 46.2929_JPRB, 40.6755_JPRB, &
  & 34.4431_JPRB, 28.5826_JPRB, 22.7195_JPRB, 18.2375_JPRB, 35.3683_JPRB, 29.9439_JPRB /)
!-- WVL=0.4690E-06
ZRALIS3( :, 6, 1) = (/ &
  & 68.8229_JPRB, 68.8229_JPRB, 68.8229_JPRB, 68.8229_JPRB, 54.7149_JPRB, 44.8499_JPRB, &
  & 40.3271_JPRB, 33.4269_JPRB, 25.5756_JPRB, 22.2747_JPRB, 42.4309_JPRB, 32.8609_JPRB /)
!-- WVL=0.5000E-06
ZRALIS3( :, 7, 1) = (/ &
  & 74.2507_JPRB, 74.2507_JPRB, 74.2507_JPRB, 74.2507_JPRB, 56.2785_JPRB, 53.4932_JPRB, &
  & 44.3735_JPRB, 39.3353_JPRB, 30.9989_JPRB, 25.2878_JPRB, 48.3665_JPRB, 36.4462_JPRB /)
!-- WVL=0.5320E-06
ZRALIS3( :, 8, 1) = (/ &
  & 76.0154_JPRB, 76.0154_JPRB, 76.0154_JPRB, 76.0154_JPRB, 60.5012_JPRB, 56.1104_JPRB, &
  & 52.6128_JPRB, 43.2216_JPRB, 35.6094_JPRB, 35.5132_JPRB, 51.5800_JPRB, 38.3123_JPRB /)
!-- WVL=0.5500E-06
ZRALIS3( :, 9, 1) = (/ &
  & 74.0725_JPRB, 74.0725_JPRB, 74.0725_JPRB, 74.0725_JPRB, 65.7130_JPRB, 56.7494_JPRB, &
  & 55.4556_JPRB, 46.2929_JPRB, 38.9333_JPRB, 46.8852_JPRB, 53.3678_JPRB, 42.2389_JPRB /)
!-- WVL=0.6450E-06
ZRALIS3( :,10, 1) = (/ &
  & 58.1996_JPRB, 58.1996_JPRB, 58.1996_JPRB, 58.1996_JPRB, 67.4616_JPRB, 70.6565_JPRB, &
  & 67.6304_JPRB, 57.4223_JPRB, 54.3685_JPRB, 68.5241_JPRB, 63.3678_JPRB, 51.4392_JPRB /)
!-- WVL=0.6700E-06
ZRALIS3( :,11, 1) = (/ &
  & 57.1752_JPRB, 57.1752_JPRB, 57.1752_JPRB, 57.1752_JPRB, 68.3509_JPRB, 68.8715_JPRB, &
  & 70.7667_JPRB, 61.5839_JPRB, 55.9630_JPRB, 71.1398_JPRB, 63.6450_JPRB, 53.3796_JPRB /)
!-- WVL=0.8000E-06
ZRALIS3( :,12, 1) = (/ &
  & 52.2732_JPRB, 52.2732_JPRB, 52.2732_JPRB, 52.2732_JPRB, 73.0790_JPRB, 75.8730_JPRB, &
  & 70.7231_JPRB, 67.5959_JPRB, 70.7844_JPRB, 78.4116_JPRB, 70.0844_JPRB, 64.1805_JPRB /)
!-- WVL=0.8580E-06
ZRALIS3( :,13, 1) = (/ &
  & 44.1022_JPRB, 44.1022_JPRB, 44.1022_JPRB, 44.1022_JPRB, 64.5294_JPRB, 73.7658_JPRB, &
  & 75.9012_JPRB, 70.1165_JPRB, 68.7976_JPRB, 77.7779_JPRB, 74.9681_JPRB, 69.2674_JPRB /)
!-- WVL=0.8650E-06
ZRALIS3( :,14, 1) = (/ &
  & 42.9915_JPRB, 42.9915_JPRB, 42.9915_JPRB, 42.9915_JPRB, 63.5949_JPRB, 72.9708_JPRB, &
  & 76.1139_JPRB, 70.8308_JPRB, 68.3938_JPRB, 77.1582_JPRB, 75.9389_JPRB, 70.1364_JPRB /)
!-- WVL=0.1020E-05
ZRALIS3( :,15, 1) = (/ &
  & 24.4931_JPRB, 24.4931_JPRB, 24.4931_JPRB, 24.4931_JPRB, 56.7779_JPRB, 57.6168_JPRB, &
  & 61.6771_JPRB, 70.8470_JPRB, 75.7631_JPRB, 75.9988_JPRB, 73.0439_JPRB, 73.8518_JPRB /)
!-- WVL=0.1064E-05
ZRALIS3( :,16, 1) = (/ &
  & 21.7284_JPRB, 21.7284_JPRB, 21.7284_JPRB, 21.7284_JPRB, 56.3323_JPRB, 56.9466_JPRB, &
  & 58.6502_JPRB, 65.4903_JPRB, 75.8165_JPRB, 76.0978_JPRB, 73.3724_JPRB, 78.4961_JPRB /)
!-- WVL=0.1240E-05
ZRALIS3( :,17, 1) = (/ &
  & 15.8004_JPRB, 15.8004_JPRB, 15.8004_JPRB, 15.8004_JPRB, 43.5486_JPRB, 52.5357_JPRB, &
  & 56.1836_JPRB, 56.9021_JPRB, 60.1154_JPRB, 65.5556_JPRB, 77.2797_JPRB, 73.6170_JPRB /)
!-- WVL=0.1640E-05
ZRALIS3( :,18, 1) = (/ &
  & 11.6921_JPRB, 11.6921_JPRB, 11.6921_JPRB, 11.6921_JPRB, 18.4660_JPRB, 22.2373_JPRB, &
  & 27.4096_JPRB, 35.8985_JPRB, 45.8813_JPRB, 53.7586_JPRB, 57.6028_JPRB, 66.7418_JPRB /)
!-- WVL=0.2130E-05
ZRALIS3( :,19, 1) = (/ &
  & 10.2242_JPRB, 10.2242_JPRB, 10.2243_JPRB, 10.2243_JPRB, 12.7135_JPRB, 13.7823_JPRB, &
  & 15.0821_JPRB, 17.1150_JPRB, 20.2759_JPRB, 24.3508_JPRB, 34.0495_JPRB, 56.2329_JPRB /)
  
 ! ** BIN No.            2   0.5000000000000000         5.000000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS3( :, 1, 2) = (/ &
  & 0.13219_JPRB, 0.13219_JPRB, 0.13219_JPRB, 0.13219_JPRB, 0.26222_JPRB, 0.30569_JPRB, &
  & 0.35157_JPRB, 0.41000_JPRB, 0.49863_JPRB, 0.56960_JPRB, 0.69761_JPRB, 1.01645_JPRB /)
!-- WVL=0.3550E-06
ZALFSS3( :, 2, 2) = (/ &
  & 0.13339_JPRB, 0.13339_JPRB, 0.13339_JPRB, 0.13339_JPRB, 0.26234_JPRB, 0.30578_JPRB, &
  & 0.35127_JPRB, 0.40988_JPRB, 0.49981_JPRB, 0.57226_JPRB, 0.69924_JPRB, 1.02127_JPRB /)
!-- WVL=0.3800E-06
ZALFSS3( :, 3, 2) = (/ &
  & 0.13568_JPRB, 0.13568_JPRB, 0.13568_JPRB, 0.13568_JPRB, 0.26358_JPRB, 0.30597_JPRB, &
  & 0.35114_JPRB, 0.40978_JPRB, 0.50052_JPRB, 0.57428_JPRB, 0.70160_JPRB, 1.02834_JPRB /)
!-- WVL=0.4000E-06
ZALFSS3( :, 4, 2) = (/ &
  & 0.13687_JPRB, 0.13687_JPRB, 0.13687_JPRB, 0.13687_JPRB, 0.26472_JPRB, 0.30685_JPRB, &
  & 0.35130_JPRB, 0.40940_JPRB, 0.50061_JPRB, 0.57533_JPRB, 0.70114_JPRB, 1.03182_JPRB /)
!-- WVL=0.4400E-06
ZALFSS3( :, 5, 2) = (/ &
  & 0.13993_JPRB, 0.13993_JPRB, 0.13993_JPRB, 0.13993_JPRB, 0.26823_JPRB, 0.30959_JPRB, &
  & 0.35313_JPRB, 0.40981_JPRB, 0.49989_JPRB, 0.57532_JPRB, 0.70061_JPRB, 1.03835_JPRB /)
!-- WVL=0.4690E-06
ZALFSS3( :, 6, 2) = (/ &
  & 0.14120_JPRB, 0.14120_JPRB, 0.14120_JPRB, 0.14120_JPRB, 0.27202_JPRB, 0.31241_JPRB, &
  & 0.35523_JPRB, 0.41144_JPRB, 0.50022_JPRB, 0.57449_JPRB, 0.69969_JPRB, 1.03991_JPRB /)
!-- WVL=0.5000E-06
ZALFSS3( :, 7, 2) = (/ &
  & 0.14295_JPRB, 0.14295_JPRB, 0.14295_JPRB, 0.14295_JPRB, 0.27488_JPRB, 0.31641_JPRB, &
  & 0.35850_JPRB, 0.41362_JPRB, 0.50089_JPRB, 0.57487_JPRB, 0.70012_JPRB, 1.03919_JPRB /)
!-- WVL=0.5320E-06
ZALFSS3( :, 8, 2) = (/ &
  & 0.14417_JPRB, 0.14417_JPRB, 0.14417_JPRB, 0.14417_JPRB, 0.27945_JPRB, 0.31979_JPRB, &
  & 0.36274_JPRB, 0.41739_JPRB, 0.50334_JPRB, 0.57585_JPRB, 0.70317_JPRB, 1.03818_JPRB /)
!-- WVL=0.5500E-06
ZALFSS3( :, 9, 2) = (/ &
  & 0.14441_JPRB, 0.14441_JPRB, 0.14441_JPRB, 0.14441_JPRB, 0.28137_JPRB, 0.32209_JPRB, &
  & 0.36532_JPRB, 0.41911_JPRB, 0.50476_JPRB, 0.57762_JPRB, 0.70474_JPRB, 1.03716_JPRB /)
!-- WVL=0.6450E-06
ZALFSS3( :,10, 2) = (/ &
  & 0.14682_JPRB, 0.14682_JPRB, 0.14682_JPRB, 0.14682_JPRB, 0.29069_JPRB, 0.33368_JPRB, &
  & 0.37772_JPRB, 0.43298_JPRB, 0.51851_JPRB, 0.59752_JPRB, 0.72149_JPRB, 1.04289_JPRB /)
!-- WVL=0.6700E-06
ZALFSS3( :,11, 2) = (/ &
  & 0.14682_JPRB, 0.14682_JPRB, 0.14682_JPRB, 0.14682_JPRB, 0.29218_JPRB, 0.33686_JPRB, &
  & 0.38025_JPRB, 0.43745_JPRB, 0.52178_JPRB, 0.60339_JPRB, 0.72585_JPRB, 1.04552_JPRB /)
!-- WVL=0.8000E-06
ZALFSS3( :,12, 2) = (/ &
  & 0.14679_JPRB, 0.14679_JPRB, 0.14679_JPRB, 0.14679_JPRB, 0.29930_JPRB, 0.34726_JPRB, &
  & 0.39350_JPRB, 0.45372_JPRB, 0.54157_JPRB, 0.62634_JPRB, 0.75105_JPRB, 1.07440_JPRB /)
!-- WVL=0.8580E-06
ZALFSS3( :,13, 2) = (/ &
  & 0.14764_JPRB, 0.14764_JPRB, 0.14764_JPRB, 0.14764_JPRB, 0.30125_JPRB, 0.34909_JPRB, &
  & 0.39879_JPRB, 0.45834_JPRB, 0.55106_JPRB, 0.63181_JPRB, 0.76364_JPRB, 1.08742_JPRB /)
!-- WVL=0.8650E-06
ZALFSS3( :,14, 2) = (/ &
  & 0.14764_JPRB, 0.14764_JPRB, 0.14764_JPRB, 0.14764_JPRB, 0.30162_JPRB, 0.34918_JPRB, &
  & 0.39922_JPRB, 0.45904_JPRB, 0.55193_JPRB, 0.63247_JPRB, 0.76530_JPRB, 1.08983_JPRB /)
!-- WVL=0.1020E-05
ZALFSS3( :,15, 2) = (/ &
  & 0.14504_JPRB, 0.14504_JPRB, 0.14504_JPRB, 0.14504_JPRB, 0.30299_JPRB, 0.35507_JPRB, &
  & 0.40522_JPRB, 0.46818_JPRB, 0.56759_JPRB, 0.64846_JPRB, 0.78662_JPRB, 1.12874_JPRB /)
!-- WVL=0.1064E-05
ZALFSS3( :,16, 2) = (/ &
  & 0.14494_JPRB, 0.14494_JPRB, 0.14494_JPRB, 0.14494_JPRB, 0.30239_JPRB, 0.35444_JPRB, &
  & 0.40727_JPRB, 0.47015_JPRB, 0.57006_JPRB, 0.65121_JPRB, 0.79242_JPRB, 1.13988_JPRB /)
!-- WVL=0.1240E-05
ZALFSS3( :,17, 2) = (/ &
  & 0.14809_JPRB, 0.14809_JPRB, 0.14809_JPRB, 0.14809_JPRB, 0.30587_JPRB, 0.35470_JPRB, &
  & 0.40515_JPRB, 0.47442_JPRB, 0.57854_JPRB, 0.66040_JPRB, 0.80150_JPRB, 1.16854_JPRB /)
!-- WVL=0.1640E-05
ZALFSS3( :,18, 2) = (/ &
  & 0.14605_JPRB, 0.14605_JPRB, 0.14605_JPRB, 0.14605_JPRB, 0.30266_JPRB, 0.35010_JPRB, &
  & 0.40390_JPRB, 0.47615_JPRB, 0.56990_JPRB, 0.66009_JPRB, 0.80751_JPRB, 1.19712_JPRB /)
!-- WVL=0.2130E-05
ZALFSS3( :,19, 2) = (/ &
  & 0.15525_JPRB, 0.15525_JPRB, 0.15525_JPRB, 0.15525_JPRB, 0.30590_JPRB, 0.35973_JPRB, &
  & 0.41264_JPRB, 0.47607_JPRB, 0.57766_JPRB, 0.65641_JPRB, 0.79778_JPRB, 1.20195_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS3( :, 1, 2) = (/ &
  & 0.74556_JPRB, 0.74556_JPRB, 0.74556_JPRB, 0.74556_JPRB, 0.76662_JPRB, 0.77208_JPRB, &
  & 0.77767_JPRB, 0.78287_JPRB, 0.78765_JPRB, 0.79007_JPRB, 0.81040_JPRB, 0.81724_JPRB /)
!-- WVL=0.3550E-06
ZASYSS3( :, 2, 2) = (/ &
  & 0.74432_JPRB, 0.74432_JPRB, 0.74432_JPRB, 0.74432_JPRB, 0.76270_JPRB, 0.76895_JPRB, &
  & 0.77446_JPRB, 0.77985_JPRB, 0.78581_JPRB, 0.78866_JPRB, 0.80889_JPRB, 0.81622_JPRB /)
!-- WVL=0.3800E-06
ZASYSS3( :, 3, 2) = (/ &
  & 0.74266_JPRB, 0.74266_JPRB, 0.74266_JPRB, 0.74266_JPRB, 0.75855_JPRB, 0.76352_JPRB, &
  & 0.76905_JPRB, 0.77526_JPRB, 0.78209_JPRB, 0.78590_JPRB, 0.80582_JPRB, 0.81475_JPRB /)
!-- WVL=0.4000E-06
ZASYSS3( :, 4, 2) = (/ &
  & 0.74064_JPRB, 0.74064_JPRB, 0.74064_JPRB, 0.74064_JPRB, 0.75481_JPRB, 0.76002_JPRB, &
  & 0.76525_JPRB, 0.77070_JPRB, 0.77851_JPRB, 0.78326_JPRB, 0.80298_JPRB, 0.81328_JPRB /)
!-- WVL=0.4400E-06
ZASYSS3( :, 5, 2) = (/ &
  & 0.73914_JPRB, 0.73914_JPRB, 0.73914_JPRB, 0.73914_JPRB, 0.74986_JPRB, 0.75371_JPRB, &
  & 0.75852_JPRB, 0.76347_JPRB, 0.77102_JPRB, 0.77696_JPRB, 0.79703_JPRB, 0.80993_JPRB /)
!-- WVL=0.4690E-06
ZASYSS3( :, 6, 2) = (/ &
  & 0.73628_JPRB, 0.73628_JPRB, 0.73628_JPRB, 0.73628_JPRB, 0.74744_JPRB, 0.75054_JPRB, &
  & 0.75398_JPRB, 0.75928_JPRB, 0.76673_JPRB, 0.77149_JPRB, 0.79371_JPRB, 0.80652_JPRB /)
!-- WVL=0.5000E-06
ZASYSS3( :, 7, 2) = (/ &
  & 0.73443_JPRB, 0.73443_JPRB, 0.73443_JPRB, 0.73443_JPRB, 0.74478_JPRB, 0.74792_JPRB, &
  & 0.75086_JPRB, 0.75481_JPRB, 0.76125_JPRB, 0.76717_JPRB, 0.78958_JPRB, 0.80277_JPRB /)
!-- WVL=0.5320E-06
ZASYSS3( :, 8, 2) = (/ &
  & 0.73217_JPRB, 0.73217_JPRB, 0.73217_JPRB, 0.73217_JPRB, 0.74362_JPRB, 0.74530_JPRB, &
  & 0.74814_JPRB, 0.75167_JPRB, 0.75763_JPRB, 0.76562_JPRB, 0.78654_JPRB, 0.79875_JPRB /)
!-- WVL=0.5500E-06
ZASYSS3( :, 9, 2) = (/ &
  & 0.73010_JPRB, 0.73010_JPRB, 0.73010_JPRB, 0.73010_JPRB, 0.74242_JPRB, 0.74437_JPRB, &
  & 0.74690_JPRB, 0.74986_JPRB, 0.75511_JPRB, 0.76983_JPRB, 0.78449_JPRB, 0.79630_JPRB /)
!-- WVL=0.6450E-06
ZASYSS3( :,10, 2) = (/ &
  & 0.72282_JPRB, 0.72282_JPRB, 0.72282_JPRB, 0.72282_JPRB, 0.73843_JPRB, 0.74016_JPRB, &
  & 0.74182_JPRB, 0.74414_JPRB, 0.74761_JPRB, 0.77301_JPRB, 0.77805_JPRB, 0.78677_JPRB /)
!-- WVL=0.6700E-06
ZASYSS3( :,11, 2) = (/ &
  & 0.71995_JPRB, 0.71995_JPRB, 0.71995_JPRB, 0.71995_JPRB, 0.73656_JPRB, 0.73974_JPRB, &
  & 0.74055_JPRB, 0.74341_JPRB, 0.74597_JPRB, 0.77257_JPRB, 0.77583_JPRB, 0.78449_JPRB /)
!-- WVL=0.8000E-06
ZASYSS3( :,12, 2) = (/ &
  & 0.70490_JPRB, 0.70490_JPRB, 0.70490_JPRB, 0.70490_JPRB, 0.72940_JPRB, 0.73363_JPRB, &
  & 0.73546_JPRB, 0.73868_JPRB, 0.74054_JPRB, 0.76915_JPRB, 0.77071_JPRB, 0.77780_JPRB /)
!-- WVL=0.8580E-06
ZASYSS3( :,13, 2) = (/ &
  & 0.70108_JPRB, 0.70108_JPRB, 0.70108_JPRB, 0.70108_JPRB, 0.72614_JPRB, 0.72985_JPRB, &
  & 0.73364_JPRB, 0.73565_JPRB, 0.73968_JPRB, 0.76679_JPRB, 0.76972_JPRB, 0.77579_JPRB /)
!-- WVL=0.8650E-06
ZASYSS3( :,14, 2) = (/ &
  & 0.70049_JPRB, 0.70049_JPRB, 0.70049_JPRB, 0.70049_JPRB, 0.72581_JPRB, 0.72935_JPRB, &
  & 0.73335_JPRB, 0.73543_JPRB, 0.73947_JPRB, 0.76658_JPRB, 0.76962_JPRB, 0.77566_JPRB /)
!-- WVL=0.1020E-05
ZASYSS3( :,15, 2) = (/ &
  & 0.68281_JPRB, 0.68281_JPRB, 0.68281_JPRB, 0.68281_JPRB, 0.71381_JPRB, 0.72168_JPRB, &
  & 0.72532_JPRB, 0.72826_JPRB, 0.73375_JPRB, 0.75673_JPRB, 0.76558_JPRB, 0.77270_JPRB /)
!-- WVL=0.1064E-05
ZASYSS3( :,16, 2) = (/ &
  & 0.67910_JPRB, 0.67910_JPRB, 0.67910_JPRB, 0.67910_JPRB, 0.70978_JPRB, 0.71760_JPRB, &
  & 0.72344_JPRB, 0.72640_JPRB, 0.73179_JPRB, 0.74393_JPRB, 0.76456_JPRB, 0.77210_JPRB /)
!-- WVL=0.1240E-05
ZASYSS3( :,17, 2) = (/ &
  & 0.67154_JPRB, 0.67154_JPRB, 0.67154_JPRB, 0.67154_JPRB, 0.70079_JPRB, 0.70498_JPRB, &
  & 0.70924_JPRB, 0.71697_JPRB, 0.72449_JPRB, 0.74182_JPRB, 0.75733_JPRB, 0.76843_JPRB /)
!-- WVL=0.1640E-05
ZASYSS3( :,18, 2) = (/ &
  & 0.64213_JPRB, 0.64213_JPRB, 0.64213_JPRB, 0.64213_JPRB, 0.67512_JPRB, 0.67978_JPRB, &
  & 0.68654_JPRB, 0.69589_JPRB, 0.72666_JPRB, 0.73391_JPRB, 0.74032_JPRB, 0.75599_JPRB /)
!-- WVL=0.2130E-05
ZASYSS3( :,19, 2) = (/ &
  & 0.66215_JPRB, 0.66215_JPRB, 0.66215_JPRB, 0.66215_JPRB, 0.65291_JPRB, 0.66285_JPRB, &
  & 0.66953_JPRB, 0.67369_JPRB, 0.71032_JPRB, 0.71249_JPRB, 0.71783_JPRB, 0.73945_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS3( :, 1, 2) = (/ &
  & 0.98851_JPRB, 0.98851_JPRB, 0.98851_JPRB, 0.98851_JPRB, 0.98457_JPRB, 0.98430_JPRB, &
  & 0.98417_JPRB, 0.98403_JPRB, 0.98384_JPRB, 0.98372_JPRB, 0.99297_JPRB, 0.99390_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS3( :, 2, 2) = (/ &
  & 0.98907_JPRB, 0.98907_JPRB, 0.98907_JPRB, 0.98907_JPRB, 0.98475_JPRB, 0.98443_JPRB, &
  & 0.98423_JPRB, 0.98410_JPRB, 0.98395_JPRB, 0.98381_JPRB, 0.99307_JPRB, 0.99388_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS3( :, 3, 2) = (/ &
  & 0.98988_JPRB, 0.98988_JPRB, 0.98988_JPRB, 0.98988_JPRB, 0.98524_JPRB, 0.98472_JPRB, &
  & 0.98443_JPRB, 0.98424_JPRB, 0.98404_JPRB, 0.98394_JPRB, 0.99313_JPRB, 0.99397_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS3( :, 4, 2) = (/ &
  & 0.99034_JPRB, 0.99034_JPRB, 0.99034_JPRB, 0.99034_JPRB, 0.98570_JPRB, 0.98503_JPRB, &
  & 0.98462_JPRB, 0.98433_JPRB, 0.98413_JPRB, 0.98402_JPRB, 0.99316_JPRB, 0.99404_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS3( :, 5, 2) = (/ &
  & 0.99126_JPRB, 0.99126_JPRB, 0.99126_JPRB, 0.99126_JPRB, 0.98699_JPRB, 0.98592_JPRB, &
  & 0.98525_JPRB, 0.98471_JPRB, 0.98432_JPRB, 0.98419_JPRB, 0.99325_JPRB, 0.99415_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS3( :, 6, 2) = (/ &
  & 0.99173_JPRB, 0.99173_JPRB, 0.99173_JPRB, 0.99173_JPRB, 0.98794_JPRB, 0.98675_JPRB, &
  & 0.98585_JPRB, 0.98514_JPRB, 0.98457_JPRB, 0.98430_JPRB, 0.99413_JPRB, 0.99416_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS3( :, 7, 2) = (/ &
  & 0.99224_JPRB, 0.99224_JPRB, 0.99224_JPRB, 0.99224_JPRB, 0.98882_JPRB, 0.98778_JPRB, &
  & 0.98671_JPRB, 0.98570_JPRB, 0.98488_JPRB, 0.98454_JPRB, 0.99455_JPRB, 0.99426_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS3( :, 8, 2) = (/ &
  & 0.99268_JPRB, 0.99268_JPRB, 0.99268_JPRB, 0.99268_JPRB, 0.98957_JPRB, 0.98863_JPRB, &
  & 0.98765_JPRB, 0.98650_JPRB, 0.98535_JPRB, 0.98693_JPRB, 0.99473_JPRB, 0.99431_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS3( :, 9, 2) = (/ &
  & 0.99291_JPRB, 0.99291_JPRB, 0.99291_JPRB, 0.99291_JPRB, 0.98990_JPRB, 0.98902_JPRB, &
  & 0.98812_JPRB, 0.98698_JPRB, 0.98564_JPRB, 0.99052_JPRB, 0.99486_JPRB, 0.99437_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS3( :,10, 2) = (/ &
  & 0.99383_JPRB, 0.99383_JPRB, 0.99383_JPRB, 0.99383_JPRB, 0.99143_JPRB, 0.99073_JPRB, &
  & 0.99007_JPRB, 0.98920_JPRB, 0.98786_JPRB, 0.99541_JPRB, 0.99559_JPRB, 0.99471_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS3( :,11, 2) = (/ &
  & 0.99401_JPRB, 0.99401_JPRB, 0.99401_JPRB, 0.99401_JPRB, 0.99173_JPRB, 0.99109_JPRB, &
  & 0.99044_JPRB, 0.98964_JPRB, 0.98840_JPRB, 0.99559_JPRB, 0.99537_JPRB, 0.99486_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS3( :,12, 2) = (/ &
  & 0.99482_JPRB, 0.99482_JPRB, 0.99482_JPRB, 0.99482_JPRB, 0.99294_JPRB, 0.99245_JPRB, &
  & 0.99195_JPRB, 0.99133_JPRB, 0.99040_JPRB, 0.99634_JPRB, 0.99596_JPRB, 0.99579_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS3( :,13, 2) = (/ &
  & 0.99513_JPRB, 0.99513_JPRB, 0.99513_JPRB, 0.99513_JPRB, 0.99340_JPRB, 0.99293_JPRB, &
  & 0.99250_JPRB, 0.99190_JPRB, 0.99107_JPRB, 0.99661_JPRB, 0.99622_JPRB, 0.99611_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS3( :,14, 2) = (/ &
  & 0.99515_JPRB, 0.99515_JPRB, 0.99515_JPRB, 0.99515_JPRB, 0.99345_JPRB, 0.99297_JPRB, &
  & 0.99255_JPRB, 0.99195_JPRB, 0.99114_JPRB, 0.99663_JPRB, 0.99625_JPRB, 0.99612_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS3( :,15, 2) = (/ &
  & 0.99580_JPRB, 0.99580_JPRB, 0.99580_JPRB, 0.99580_JPRB, 0.99422_JPRB, 0.99391_JPRB, &
  & 0.99353_JPRB, 0.99306_JPRB, 0.99245_JPRB, 0.99610_JPRB, 0.99684_JPRB, 0.99678_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS3( :,16, 2) = (/ &
  & 0.99598_JPRB, 0.99598_JPRB, 0.99598_JPRB, 0.99598_JPRB, 0.99443_JPRB, 0.99413_JPRB, &
  & 0.99380_JPRB, 0.99334_JPRB, 0.99272_JPRB, 0.99404_JPRB, 0.99693_JPRB, 0.99692_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS3( :,17, 2) = (/ &
  & 0.99666_JPRB, 0.99666_JPRB, 0.99666_JPRB, 0.99666_JPRB, 0.99516_JPRB, 0.99480_JPRB, &
  & 0.99446_JPRB, 0.99414_JPRB, 0.99364_JPRB, 0.99569_JPRB, 0.99733_JPRB, 0.99737_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS3( :,18, 2) = (/ &
  & 0.99748_JPRB, 0.99748_JPRB, 0.99748_JPRB, 0.99748_JPRB, 0.99630_JPRB, 0.99596_JPRB, &
  & 0.99568_JPRB, 0.99540_JPRB, 0.99825_JPRB, 0.99814_JPRB, 0.99792_JPRB, 0.99793_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS3( :,19, 2) = (/ &
  & 0.99823_JPRB, 0.99823_JPRB, 0.99823_JPRB, 0.99823_JPRB, 0.99722_JPRB, 0.99700_JPRB, &
  & 0.99678_JPRB, 0.99645_JPRB, 0.99869_JPRB, 0.99857_JPRB, 0.99839_JPRB, 0.99841_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS3( :, 1, 2) = (/ &
  & 13.4510_JPRB, 13.4510_JPRB, 13.4510_JPRB, 13.4510_JPRB, 13.1611_JPRB, 13.9120_JPRB, &
  & 14.2096_JPRB, 14.4700_JPRB, 15.0864_JPRB, 16.2016_JPRB, 10.1325_JPRB,  8.5022_JPRB /)
!-- WVL=0.3550E-06
ZRALIS3( :, 2, 2) = (/ &
  & 13.4744_JPRB, 13.4744_JPRB, 13.4744_JPRB, 13.4744_JPRB, 12.8099_JPRB, 13.4462_JPRB, &
  & 14.2363_JPRB, 14.1323_JPRB, 14.5027_JPRB, 15.4349_JPRB, 10.4529_JPRB,  8.2213_JPRB /)
!-- WVL=0.3800E-06
ZRALIS3( :, 3, 2) = (/ &
  & 13.2152_JPRB, 13.2152_JPRB, 13.2152_JPRB, 13.2152_JPRB, 12.7487_JPRB, 12.8781_JPRB, &
  & 13.4580_JPRB, 14.3245_JPRB, 14.3493_JPRB, 14.5925_JPRB, 10.5397_JPRB,  8.7003_JPRB /)
!-- WVL=0.4000E-06
ZRALIS3( :, 4, 2) = (/ &
  & 12.8790_JPRB, 12.8790_JPRB, 12.8790_JPRB, 12.8790_JPRB, 12.8911_JPRB, 12.7376_JPRB, &
  & 13.0527_JPRB, 13.7093_JPRB, 14.1758_JPRB, 14.4568_JPRB, 11.0703_JPRB,  9.7162_JPRB /)
!-- WVL=0.4400E-06
ZRALIS3( :, 5, 2) = (/ &
  & 13.0716_JPRB, 13.0716_JPRB, 13.0716_JPRB, 13.0715_JPRB, 13.5905_JPRB, 13.0600_JPRB, &
  & 12.7519_JPRB, 12.8785_JPRB, 13.7555_JPRB, 14.2910_JPRB, 11.0329_JPRB,  9.4902_JPRB /)
!-- WVL=0.4690E-06
ZRALIS3( :, 6, 2) = (/ &
  & 13.4759_JPRB, 13.4759_JPRB, 13.4759_JPRB, 13.4759_JPRB, 13.5170_JPRB, 13.5036_JPRB, &
  & 13.0177_JPRB, 12.7442_JPRB, 13.1868_JPRB, 13.8198_JPRB, 11.4968_JPRB, 10.2995_JPRB /)
!-- WVL=0.5000E-06
ZRALIS3( :, 7, 2) = (/ &
  & 14.0465_JPRB, 14.0465_JPRB, 14.0465_JPRB, 14.0465_JPRB, 13.4790_JPRB, 13.5995_JPRB, &
  & 13.5372_JPRB, 12.8912_JPRB, 12.7449_JPRB, 13.2303_JPRB, 11.8356_JPRB, 10.6710_JPRB /)
!-- WVL=0.5320E-06
ZRALIS3( :, 8, 2) = (/ &
  & 14.6239_JPRB, 14.6239_JPRB, 14.6239_JPRB, 14.6239_JPRB, 13.3621_JPRB, 13.4971_JPRB, &
  & 13.6410_JPRB, 13.4924_JPRB, 12.8112_JPRB, 14.1622_JPRB, 12.0813_JPRB, 10.6715_JPRB /)
!-- WVL=0.5500E-06
ZRALIS3( :, 9, 2) = (/ &
  & 14.7125_JPRB, 14.7124_JPRB, 14.7124_JPRB, 14.7124_JPRB, 13.1470_JPRB, 13.4552_JPRB, &
  & 13.5417_JPRB, 13.5906_JPRB, 12.8955_JPRB, 14.6342_JPRB, 12.2502_JPRB, 11.1099_JPRB /)
!-- WVL=0.6450E-06
ZRALIS3( :,10, 2) = (/ &
  & 14.6044_JPRB, 14.6044_JPRB, 14.6043_JPRB, 14.6044_JPRB, 13.1696_JPRB, 12.9010_JPRB, &
  & 13.0809_JPRB, 13.3568_JPRB, 13.5504_JPRB, 14.6314_JPRB, 13.5144_JPRB, 12.0526_JPRB /)
!-- WVL=0.6700E-06
ZRALIS3( :,11, 2) = (/ &
  & 14.5170_JPRB, 14.5170_JPRB, 14.5169_JPRB, 14.5170_JPRB, 13.4563_JPRB, 12.9872_JPRB, &
  & 12.9216_JPRB, 13.2346_JPRB, 13.4807_JPRB, 14.7502_JPRB, 14.0753_JPRB, 12.2241_JPRB /)
!-- WVL=0.8000E-06
ZRALIS3( :,12, 2) = (/ &
  & 13.1938_JPRB, 13.1938_JPRB, 13.1939_JPRB, 13.1939_JPRB, 14.6791_JPRB, 14.3684_JPRB, &
  & 13.7476_JPRB, 13.0900_JPRB, 12.8906_JPRB, 16.0327_JPRB, 14.9114_JPRB, 13.7229_JPRB /)
!-- WVL=0.8580E-06
ZRALIS3( :,13, 2) = (/ &
  & 12.3963_JPRB, 12.3963_JPRB, 12.3963_JPRB, 12.3963_JPRB, 14.7850_JPRB, 14.6945_JPRB, &
  & 14.4395_JPRB, 13.6170_JPRB, 12.9476_JPRB, 17.0909_JPRB, 15.6369_JPRB, 14.5676_JPRB /)
!-- WVL=0.8650E-06
ZRALIS3( :,14, 2) = (/ &
  & 12.2960_JPRB, 12.2960_JPRB, 12.2960_JPRB, 12.2960_JPRB, 14.7798_JPRB, 14.6855_JPRB, &
  & 14.4878_JPRB, 13.7901_JPRB, 12.9917_JPRB, 17.4184_JPRB, 15.6063_JPRB, 14.5494_JPRB /)
!-- WVL=0.1020E-05
ZRALIS3( :,15, 2) = (/ &
  & 10.3829_JPRB, 10.3829_JPRB, 10.3829_JPRB, 10.3829_JPRB, 14.0605_JPRB, 14.6535_JPRB, &
  & 14.7936_JPRB, 14.7420_JPRB, 14.3313_JPRB, 17.4577_JPRB, 18.2759_JPRB, 15.3353_JPRB /)
!-- WVL=0.1064E-05
ZRALIS3( :,16, 2) = (/ &
  & 10.1023_JPRB, 10.1023_JPRB, 10.1023_JPRB, 10.1023_JPRB, 13.7809_JPRB, 14.3385_JPRB, &
  & 14.7480_JPRB, 14.7443_JPRB, 14.6123_JPRB, 15.1343_JPRB, 18.6968_JPRB, 15.6780_JPRB /)
!-- WVL=0.1240E-05
ZRALIS3( :,17, 2) = (/ &
  &  9.8368_JPRB,  9.8368_JPRB,  9.8368_JPRB,  9.8368_JPRB, 12.3548_JPRB, 13.1873_JPRB, &
  & 13.7744_JPRB, 14.3049_JPRB, 14.6793_JPRB, 16.3227_JPRB, 18.2617_JPRB, 18.3285_JPRB /)
!-- WVL=0.1640E-05
ZRALIS3( :,18, 2) = (/ &
  & 11.5775_JPRB, 11.5775_JPRB, 11.5775_JPRB, 11.5775_JPRB,  9.8794_JPRB, 10.1548_JPRB, &
  & 10.7444_JPRB, 11.6659_JPRB, 21.9492_JPRB, 21.7029_JPRB, 20.6073_JPRB, 18.9291_JPRB /)
!-- WVL=0.2130E-05
ZRALIS3( :,19, 2) = (/ &
  & 19.0679_JPRB, 19.0679_JPRB, 19.0679_JPRB, 19.0679_JPRB, 10.5246_JPRB, 10.1710_JPRB, &
  &  9.9750_JPRB,  9.8048_JPRB, 23.1870_JPRB, 22.6639_JPRB, 22.3590_JPRB, 21.2407_JPRB /)
  
 ! ** BIN No.            3    5.000000000000000         20.00000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS3( :, 1, 3) = (/ &
  & 0.03811_JPRB, 0.03811_JPRB, 0.03811_JPRB, 0.03811_JPRB, 0.07796_JPRB, 0.09089_JPRB, &
  & 0.10405_JPRB, 0.12117_JPRB, 0.14764_JPRB, 0.16944_JPRB, 0.20763_JPRB, 0.30691_JPRB /)
!-- WVL=0.3550E-06
ZALFSS3( :, 2, 3) = (/ &
  & 0.03817_JPRB, 0.03817_JPRB, 0.03817_JPRB, 0.03817_JPRB, 0.07818_JPRB, 0.09088_JPRB, &
  & 0.10422_JPRB, 0.12139_JPRB, 0.14777_JPRB, 0.16947_JPRB, 0.20785_JPRB, 0.30689_JPRB /)
!-- WVL=0.3800E-06
ZALFSS3( :, 3, 3) = (/ &
  & 0.03821_JPRB, 0.03821_JPRB, 0.03821_JPRB, 0.03821_JPRB, 0.07819_JPRB, 0.09117_JPRB, &
  & 0.10436_JPRB, 0.12148_JPRB, 0.14804_JPRB, 0.16982_JPRB, 0.20781_JPRB, 0.30717_JPRB /)
!-- WVL=0.4000E-06
ZALFSS3( :, 4, 3) = (/ &
  & 0.03839_JPRB, 0.03839_JPRB, 0.03839_JPRB, 0.03839_JPRB, 0.07846_JPRB, 0.09118_JPRB, &
  & 0.10456_JPRB, 0.12176_JPRB, 0.14819_JPRB, 0.16994_JPRB, 0.20831_JPRB, 0.30805_JPRB /)
!-- WVL=0.4400E-06
ZALFSS3( :, 5, 3) = (/ &
  & 0.03840_JPRB, 0.03840_JPRB, 0.03840_JPRB, 0.03840_JPRB, 0.07867_JPRB, 0.09153_JPRB, &
  & 0.10475_JPRB, 0.12210_JPRB, 0.14866_JPRB, 0.17029_JPRB, 0.20872_JPRB, 0.30830_JPRB /)
!-- WVL=0.4690E-06
ZALFSS3( :, 6, 3) = (/ &
  & 0.03867_JPRB, 0.03867_JPRB, 0.03867_JPRB, 0.03867_JPRB, 0.07871_JPRB, 0.09168_JPRB, &
  & 0.10513_JPRB, 0.12213_JPRB, 0.14872_JPRB, 0.17082_JPRB, 0.20905_JPRB, 0.30869_JPRB /)
!-- WVL=0.5000E-06
ZALFSS3( :, 7, 3) = (/ &
  & 0.03863_JPRB, 0.03863_JPRB, 0.03863_JPRB, 0.03863_JPRB, 0.07907_JPRB, 0.09182_JPRB, &
  & 0.10524_JPRB, 0.12259_JPRB, 0.14917_JPRB, 0.17086_JPRB, 0.20962_JPRB, 0.30933_JPRB /)
!-- WVL=0.5320E-06
ZALFSS3( :, 8, 3) = (/ &
  & 0.03871_JPRB, 0.03871_JPRB, 0.03871_JPRB, 0.03871_JPRB, 0.07903_JPRB, 0.09216_JPRB, &
  & 0.10545_JPRB, 0.12260_JPRB, 0.14929_JPRB, 0.17119_JPRB, 0.20956_JPRB, 0.30972_JPRB /)
!-- WVL=0.5500E-06
ZALFSS3( :, 9, 3) = (/ &
  & 0.03889_JPRB, 0.03889_JPRB, 0.03889_JPRB, 0.03889_JPRB, 0.07921_JPRB, 0.09226_JPRB, &
  & 0.10549_JPRB, 0.12293_JPRB, 0.14965_JPRB, 0.17160_JPRB, 0.21005_JPRB, 0.31026_JPRB /)
!-- WVL=0.6450E-06
ZALFSS3( :,10, 3) = (/ &
  & 0.03898_JPRB, 0.03898_JPRB, 0.03898_JPRB, 0.03898_JPRB, 0.07971_JPRB, 0.09273_JPRB, &
  & 0.10627_JPRB, 0.12351_JPRB, 0.15013_JPRB, 0.17256_JPRB, 0.21085_JPRB, 0.31091_JPRB /)
!-- WVL=0.6700E-06
ZALFSS3( :,11, 3) = (/ &
  & 0.03912_JPRB, 0.03912_JPRB, 0.03912_JPRB, 0.03912_JPRB, 0.08007_JPRB, 0.09269_JPRB, &
  & 0.10657_JPRB, 0.12351_JPRB, 0.15050_JPRB, 0.17245_JPRB, 0.21107_JPRB, 0.31167_JPRB /)
!-- WVL=0.8000E-06
ZALFSS3( :,12, 3) = (/ &
  & 0.03962_JPRB, 0.03962_JPRB, 0.03962_JPRB, 0.03962_JPRB, 0.08069_JPRB, 0.09333_JPRB, &
  & 0.10738_JPRB, 0.12443_JPRB, 0.15176_JPRB, 0.17329_JPRB, 0.21278_JPRB, 0.31286_JPRB /)
!-- WVL=0.8580E-06
ZALFSS3( :,13, 3) = (/ &
  & 0.03949_JPRB, 0.03949_JPRB, 0.03949_JPRB, 0.03949_JPRB, 0.08100_JPRB, 0.09403_JPRB, &
  & 0.10717_JPRB, 0.12524_JPRB, 0.15157_JPRB, 0.17454_JPRB, 0.21278_JPRB, 0.31428_JPRB /)
!-- WVL=0.8650E-06
ZALFSS3( :,14, 3) = (/ &
  & 0.03951_JPRB, 0.03951_JPRB, 0.03951_JPRB, 0.03951_JPRB, 0.08097_JPRB, 0.09416_JPRB, &
  & 0.10721_JPRB, 0.12525_JPRB, 0.15164_JPRB, 0.17465_JPRB, 0.21270_JPRB, 0.31423_JPRB /)
!-- WVL=0.1020E-05
ZALFSS3( :,15, 3) = (/ &
  & 0.04075_JPRB, 0.04075_JPRB, 0.04075_JPRB, 0.04075_JPRB, 0.08183_JPRB, 0.09431_JPRB, &
  & 0.10835_JPRB, 0.12636_JPRB, 0.15260_JPRB, 0.17514_JPRB, 0.21429_JPRB, 0.31568_JPRB /)
!-- WVL=0.1064E-05
ZALFSS3( :,16, 3) = (/ &
  & 0.04089_JPRB, 0.04089_JPRB, 0.04089_JPRB, 0.04089_JPRB, 0.08229_JPRB, 0.09487_JPRB, &
  & 0.10816_JPRB, 0.12660_JPRB, 0.15308_JPRB, 0.17520_JPRB, 0.21429_JPRB, 0.31561_JPRB /)
!-- WVL=0.1240E-05
ZALFSS3( :,17, 3) = (/ &
  & 0.04032_JPRB, 0.04032_JPRB, 0.04032_JPRB, 0.04032_JPRB, 0.08183_JPRB, 0.09578_JPRB, &
  & 0.11028_JPRB, 0.12724_JPRB, 0.15412_JPRB, 0.17640_JPRB, 0.21646_JPRB, 0.31749_JPRB /)
!-- WVL=0.1640E-05
ZALFSS3( :,18, 3) = (/ &
  & 0.04309_JPRB, 0.04309_JPRB, 0.04309_JPRB, 0.04309_JPRB, 0.08428_JPRB, 0.09886_JPRB, &
  & 0.11243_JPRB, 0.12888_JPRB, 0.15760_JPRB, 0.17859_JPRB, 0.21926_JPRB, 0.32121_JPRB /)
!-- WVL=0.2130E-05
ZALFSS3( :,19, 3) = (/ &
  & 0.04249_JPRB, 0.04249_JPRB, 0.04249_JPRB, 0.04249_JPRB, 0.08629_JPRB, 0.09824_JPRB, &
  & 0.11182_JPRB, 0.13106_JPRB, 0.15778_JPRB, 0.18272_JPRB, 0.22583_JPRB, 0.32438_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS3( :, 1, 3) = (/ &
  & 0.80625_JPRB, 0.80625_JPRB, 0.80625_JPRB, 0.80625_JPRB, 0.81200_JPRB, 0.81336_JPRB, &
  & 0.81411_JPRB, 0.81520_JPRB, 0.81656_JPRB, 0.81731_JPRB, 0.83622_JPRB, 0.83970_JPRB /)
!-- WVL=0.3550E-06
ZASYSS3( :, 2, 3) = (/ &
  & 0.80542_JPRB, 0.80542_JPRB, 0.80542_JPRB, 0.80542_JPRB, 0.81155_JPRB, 0.81258_JPRB, &
  & 0.81359_JPRB, 0.81473_JPRB, 0.81604_JPRB, 0.81672_JPRB, 0.83563_JPRB, 0.83941_JPRB /)
!-- WVL=0.3800E-06
ZASYSS3( :, 3, 3) = (/ &
  & 0.80374_JPRB, 0.80374_JPRB, 0.80374_JPRB, 0.80374_JPRB, 0.81010_JPRB, 0.81170_JPRB, &
  & 0.81255_JPRB, 0.81362_JPRB, 0.81508_JPRB, 0.81607_JPRB, 0.83479_JPRB, 0.83893_JPRB /)
!-- WVL=0.4000E-06
ZASYSS3( :, 4, 3) = (/ &
  & 0.80308_JPRB, 0.80308_JPRB, 0.80308_JPRB, 0.80308_JPRB, 0.80966_JPRB, 0.81059_JPRB, &
  & 0.81180_JPRB, 0.81306_JPRB, 0.81434_JPRB, 0.81524_JPRB, 0.83435_JPRB, 0.83845_JPRB /)
!-- WVL=0.4400E-06
ZASYSS3( :, 5, 3) = (/ &
  & 0.79996_JPRB, 0.79996_JPRB, 0.79996_JPRB, 0.79996_JPRB, 0.80816_JPRB, 0.80928_JPRB, &
  & 0.81015_JPRB, 0.81168_JPRB, 0.81319_JPRB, 0.81388_JPRB, 0.83298_JPRB, 0.83746_JPRB /)
!-- WVL=0.4690E-06
ZASYSS3( :, 6, 3) = (/ &
  & 0.79915_JPRB, 0.79915_JPRB, 0.79915_JPRB, 0.79915_JPRB, 0.80671_JPRB, 0.80824_JPRB, &
  & 0.80944_JPRB, 0.81033_JPRB, 0.81203_JPRB, 0.81326_JPRB, 0.83352_JPRB, 0.83670_JPRB /)
!-- WVL=0.5000E-06
ZASYSS3( :, 7, 3) = (/ &
  & 0.79663_JPRB, 0.79663_JPRB, 0.79663_JPRB, 0.79663_JPRB, 0.80600_JPRB, 0.80702_JPRB, &
  & 0.80818_JPRB, 0.80966_JPRB, 0.81113_JPRB, 0.81208_JPRB, 0.83340_JPRB, 0.83589_JPRB /)
!-- WVL=0.5320E-06
ZASYSS3( :, 8, 3) = (/ &
  & 0.79433_JPRB, 0.79433_JPRB, 0.79433_JPRB, 0.79433_JPRB, 0.80422_JPRB, 0.80622_JPRB, &
  & 0.80723_JPRB, 0.80836_JPRB, 0.81004_JPRB, 0.81481_JPRB, 0.83221_JPRB, 0.83527_JPRB /)
!-- WVL=0.5500E-06
ZASYSS3( :, 9, 3) = (/ &
  & 0.79395_JPRB, 0.79395_JPRB, 0.79395_JPRB, 0.79395_JPRB, 0.80368_JPRB, 0.80553_JPRB, &
  & 0.80655_JPRB, 0.80816_JPRB, 0.80980_JPRB, 0.82100_JPRB, 0.83197_JPRB, 0.83479_JPRB /)
!-- WVL=0.6450E-06
ZASYSS3( :,10, 3) = (/ &
  & 0.78737_JPRB, 0.78737_JPRB, 0.78737_JPRB, 0.78737_JPRB, 0.79971_JPRB, 0.80179_JPRB, &
  & 0.80356_JPRB, 0.80514_JPRB, 0.80680_JPRB, 0.82581_JPRB, 0.82911_JPRB, 0.83225_JPRB /)
!-- WVL=0.6700E-06
ZASYSS3( :,11, 3) = (/ &
  & 0.78625_JPRB, 0.78625_JPRB, 0.78625_JPRB, 0.78625_JPRB, 0.79937_JPRB, 0.80040_JPRB, &
  & 0.80287_JPRB, 0.80401_JPRB, 0.80630_JPRB, 0.82464_JPRB, 0.82739_JPRB, 0.83192_JPRB /)
!-- WVL=0.8000E-06
ZASYSS3( :,12, 3) = (/ &
  & 0.77975_JPRB, 0.77975_JPRB, 0.77975_JPRB, 0.77975_JPRB, 0.79381_JPRB, 0.79531_JPRB, &
  & 0.79840_JPRB, 0.79985_JPRB, 0.80285_JPRB, 0.82071_JPRB, 0.82354_JPRB, 0.82887_JPRB /)
!-- WVL=0.8580E-06
ZASYSS3( :,13, 3) = (/ &
  & 0.77458_JPRB, 0.77458_JPRB, 0.77458_JPRB, 0.77458_JPRB, 0.79145_JPRB, 0.79397_JPRB, &
  & 0.79527_JPRB, 0.79868_JPRB, 0.80038_JPRB, 0.81993_JPRB, 0.82165_JPRB, 0.82794_JPRB /)
!-- WVL=0.8650E-06
ZASYSS3( :,14, 3) = (/ &
  & 0.77409_JPRB, 0.77409_JPRB, 0.77409_JPRB, 0.77409_JPRB, 0.79111_JPRB, 0.79381_JPRB, &
  & 0.79498_JPRB, 0.79836_JPRB, 0.80022_JPRB, 0.81973_JPRB, 0.82138_JPRB, 0.82771_JPRB /)
!-- WVL=0.1020E-05
ZASYSS3( :,15, 3) = (/ &
  & 0.76976_JPRB, 0.76976_JPRB, 0.76976_JPRB, 0.76976_JPRB, 0.78600_JPRB, 0.78675_JPRB, &
  & 0.78989_JPRB, 0.79348_JPRB, 0.79548_JPRB, 0.81115_JPRB, 0.81736_JPRB, 0.82416_JPRB /)
!-- WVL=0.1064E-05
ZASYSS3( :,16, 3) = (/ &
  & 0.76763_JPRB, 0.76763_JPRB, 0.76763_JPRB, 0.76763_JPRB, 0.78485_JPRB, 0.78627_JPRB, &
  & 0.78778_JPRB, 0.79183_JPRB, 0.79434_JPRB, 0.80212_JPRB, 0.81584_JPRB, 0.82296_JPRB /)
!-- WVL=0.1240E-05
ZASYSS3( :,17, 3) = (/ &
  & 0.75482_JPRB, 0.75482_JPRB, 0.75482_JPRB, 0.75482_JPRB, 0.77435_JPRB, 0.78003_JPRB, &
  & 0.78455_JPRB, 0.78600_JPRB, 0.78901_JPRB, 0.80049_JPRB, 0.81216_JPRB, 0.81904_JPRB /)
!-- WVL=0.1640E-05
ZASYSS3( :,18, 3) = (/ &
  & 0.75084_JPRB, 0.75084_JPRB, 0.75084_JPRB, 0.75084_JPRB, 0.76246_JPRB, 0.76807_JPRB, &
  & 0.77094_JPRB, 0.77185_JPRB, 0.79566_JPRB, 0.79644_JPRB, 0.80210_JPRB, 0.81099_JPRB /)
!-- WVL=0.2130E-05
ZASYSS3( :,19, 3) = (/ &
  & 0.70829_JPRB, 0.70829_JPRB, 0.70829_JPRB, 0.70829_JPRB, 0.75207_JPRB, 0.75126_JPRB, &
  & 0.75310_JPRB, 0.75884_JPRB, 0.78212_JPRB, 0.78778_JPRB, 0.79391_JPRB, 0.79952_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS3( :, 1, 3) = (/ &
  & 0.98059_JPRB, 0.98059_JPRB, 0.98059_JPRB, 0.98059_JPRB, 0.98155_JPRB, 0.98130_JPRB, &
  & 0.98095_JPRB, 0.98050_JPRB, 0.97981_JPRB, 0.97924_JPRB, 0.99079_JPRB, 0.99147_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS3( :, 2, 3) = (/ &
  & 0.98050_JPRB, 0.98050_JPRB, 0.98050_JPRB, 0.98050_JPRB, 0.98167_JPRB, 0.98145_JPRB, &
  & 0.98115_JPRB, 0.98077_JPRB, 0.98012_JPRB, 0.97959_JPRB, 0.99096_JPRB, 0.99165_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS3( :, 3, 3) = (/ &
  & 0.98044_JPRB, 0.98044_JPRB, 0.98044_JPRB, 0.98044_JPRB, 0.98170_JPRB, 0.98165_JPRB, &
  & 0.98144_JPRB, 0.98111_JPRB, 0.98057_JPRB, 0.98011_JPRB, 0.99120_JPRB, 0.99192_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS3( :, 4, 3) = (/ &
  & 0.98054_JPRB, 0.98054_JPRB, 0.98054_JPRB, 0.98054_JPRB, 0.98166_JPRB, 0.98169_JPRB, &
  & 0.98160_JPRB, 0.98135_JPRB, 0.98087_JPRB, 0.98044_JPRB, 0.99136_JPRB, 0.99211_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS3( :, 5, 3) = (/ &
  & 0.98071_JPRB, 0.98071_JPRB, 0.98071_JPRB, 0.98071_JPRB, 0.98121_JPRB, 0.98161_JPRB, &
  & 0.98171_JPRB, 0.98166_JPRB, 0.98134_JPRB, 0.98101_JPRB, 0.99160_JPRB, 0.99243_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS3( :, 6, 3) = (/ &
  & 0.98108_JPRB, 0.98108_JPRB, 0.98108_JPRB, 0.98108_JPRB, 0.98077_JPRB, 0.98129_JPRB, &
  & 0.98162_JPRB, 0.98169_JPRB, 0.98155_JPRB, 0.98133_JPRB, 0.99263_JPRB, 0.99260_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS3( :, 7, 3) = (/ &
  & 0.98136_JPRB, 0.98136_JPRB, 0.98136_JPRB, 0.98136_JPRB, 0.98055_JPRB, 0.98084_JPRB, &
  & 0.98129_JPRB, 0.98166_JPRB, 0.98169_JPRB, 0.98152_JPRB, 0.99306_JPRB, 0.99274_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS3( :, 8, 3) = (/ &
  & 0.98167_JPRB, 0.98167_JPRB, 0.98167_JPRB, 0.98167_JPRB, 0.98043_JPRB, 0.98059_JPRB, &
  & 0.98090_JPRB, 0.98139_JPRB, 0.98171_JPRB, 0.98378_JPRB, 0.99304_JPRB, 0.99289_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS3( :, 9, 3) = (/ &
  & 0.98194_JPRB, 0.98194_JPRB, 0.98194_JPRB, 0.98194_JPRB, 0.98045_JPRB, 0.98052_JPRB, &
  & 0.98071_JPRB, 0.98121_JPRB, 0.98169_JPRB, 0.98717_JPRB, 0.99304_JPRB, 0.99294_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS3( :,10, 3) = (/ &
  & 0.98307_JPRB, 0.98307_JPRB, 0.98307_JPRB, 0.98307_JPRB, 0.98084_JPRB, 0.98058_JPRB, &
  & 0.98048_JPRB, 0.98048_JPRB, 0.98079_JPRB, 0.99143_JPRB, 0.99265_JPRB, 0.99306_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS3( :,11, 3) = (/ &
  & 0.98343_JPRB, 0.98343_JPRB, 0.98344_JPRB, 0.98343_JPRB, 0.98106_JPRB, 0.98064_JPRB, &
  & 0.98055_JPRB, 0.98042_JPRB, 0.98062_JPRB, 0.99140_JPRB, 0.99196_JPRB, 0.99304_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS3( :,12, 3) = (/ &
  & 0.98515_JPRB, 0.98515_JPRB, 0.98515_JPRB, 0.98515_JPRB, 0.98205_JPRB, 0.98144_JPRB, &
  & 0.98119_JPRB, 0.98078_JPRB, 0.98054_JPRB, 0.99163_JPRB, 0.99149_JPRB, 0.99273_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS3( :,13, 3) = (/ &
  & 0.98569_JPRB, 0.98569_JPRB, 0.98568_JPRB, 0.98569_JPRB, 0.98257_JPRB, 0.98198_JPRB, &
  & 0.98148_JPRB, 0.98116_JPRB, 0.98065_JPRB, 0.99181_JPRB, 0.99158_JPRB, 0.99270_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS3( :,14, 3) = (/ &
  & 0.98577_JPRB, 0.98577_JPRB, 0.98577_JPRB, 0.98577_JPRB, 0.98259_JPRB, 0.98206_JPRB, &
  & 0.98151_JPRB, 0.98117_JPRB, 0.98067_JPRB, 0.99185_JPRB, 0.99161_JPRB, 0.99271_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS3( :,15, 3) = (/ &
  & 0.98755_JPRB, 0.98755_JPRB, 0.98755_JPRB, 0.98755_JPRB, 0.98406_JPRB, 0.98324_JPRB, &
  & 0.98273_JPRB, 0.98222_JPRB, 0.98146_JPRB, 0.98983_JPRB, 0.99196_JPRB, 0.99284_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS3( :,16, 3) = (/ &
  & 0.98793_JPRB, 0.98793_JPRB, 0.98793_JPRB, 0.98793_JPRB, 0.98450_JPRB, 0.98369_JPRB, &
  & 0.98300_JPRB, 0.98252_JPRB, 0.98173_JPRB, 0.98498_JPRB, 0.99206_JPRB, 0.99289_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS3( :,17, 3) = (/ &
  & 0.98912_JPRB, 0.98912_JPRB, 0.98912_JPRB, 0.98912_JPRB, 0.98573_JPRB, 0.98512_JPRB, &
  & 0.98458_JPRB, 0.98377_JPRB, 0.98286_JPRB, 0.98799_JPRB, 0.99262_JPRB, 0.99322_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS3( :,18, 3) = (/ &
  & 0.99187_JPRB, 0.99187_JPRB, 0.99187_JPRB, 0.99187_JPRB, 0.98846_JPRB, 0.98785_JPRB, &
  & 0.98717_JPRB, 0.98630_JPRB, 0.99446_JPRB, 0.99415_JPRB, 0.99373_JPRB, 0.99405_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS3( :,19, 3) = (/ &
  & 0.99338_JPRB, 0.99338_JPRB, 0.99338_JPRB, 0.99338_JPRB, 0.99088_JPRB, 0.99005_JPRB, &
  & 0.98937_JPRB, 0.98875_JPRB, 0.99551_JPRB, 0.99529_JPRB, 0.99490_JPRB, 0.99492_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS3( :, 1, 3) = (/ &
  & 16.8849_JPRB, 16.8848_JPRB, 16.8847_JPRB, 16.8848_JPRB, 17.5353_JPRB, 16.4736_JPRB, &
  & 16.6036_JPRB, 17.2215_JPRB, 16.9801_JPRB, 15.7166_JPRB,  5.3978_JPRB,  3.8840_JPRB /)
!-- WVL=0.3550E-06
ZRALIS3( :, 2, 3) = (/ &
  & 17.2194_JPRB, 17.2193_JPRB, 17.2193_JPRB, 17.2193_JPRB, 18.0609_JPRB, 17.0478_JPRB, &
  & 16.2224_JPRB, 16.8591_JPRB, 17.2947_JPRB, 16.4386_JPRB,  5.5758_JPRB,  4.0970_JPRB /)
!-- WVL=0.3800E-06
ZRALIS3( :, 3, 3) = (/ &
  & 17.9917_JPRB, 17.9916_JPRB, 17.9916_JPRB, 17.9916_JPRB, 18.1049_JPRB, 17.9916_JPRB, &
  & 16.9154_JPRB, 16.1693_JPRB, 17.0283_JPRB, 17.3307_JPRB,  5.8539_JPRB,  4.3909_JPRB /)
!-- WVL=0.4000E-06
ZRALIS3( :, 4, 3) = (/ &
  & 18.7642_JPRB, 18.7641_JPRB, 18.7641_JPRB, 18.7641_JPRB, 17.6657_JPRB, 18.1680_JPRB, &
  & 17.6224_JPRB, 16.5682_JPRB, 16.6095_JPRB, 17.2608_JPRB,  5.8332_JPRB,  4.3943_JPRB /)
!-- WVL=0.4400E-06
ZRALIS3( :, 5, 3) = (/ &
  & 18.5918_JPRB, 18.5917_JPRB, 18.5918_JPRB, 18.5917_JPRB, 16.4622_JPRB, 17.2541_JPRB, &
  & 18.1579_JPRB, 17.9858_JPRB, 16.6429_JPRB, 16.4291_JPRB,  6.2282_JPRB,  4.8489_JPRB /)
!-- WVL=0.4690E-06
ZRALIS3( :, 6, 3) = (/ &
  & 17.6507_JPRB, 17.6506_JPRB, 17.6507_JPRB, 17.6506_JPRB, 16.4874_JPRB, 16.5089_JPRB, &
  & 17.4422_JPRB, 18.1463_JPRB, 17.4669_JPRB, 16.5346_JPRB,  5.4822_JPRB,  4.7929_JPRB /)
!-- WVL=0.5000E-06
ZRALIS3( :, 7, 3) = (/ &
  & 16.4986_JPRB, 16.4985_JPRB, 16.4985_JPRB, 16.4985_JPRB, 17.1251_JPRB, 16.4102_JPRB, &
  & 16.4162_JPRB, 17.6658_JPRB, 18.0643_JPRB, 17.3716_JPRB,  5.5431_JPRB,  4.9098_JPRB /)
!-- WVL=0.5320E-06
ZRALIS3( :, 8, 3) = (/ &
  & 15.3030_JPRB, 15.3029_JPRB, 15.3030_JPRB, 15.3029_JPRB, 17.5961_JPRB, 17.0010_JPRB, &
  & 16.3596_JPRB, 16.6020_JPRB, 18.0809_JPRB, 16.2254_JPRB,  6.0010_JPRB,  5.1208_JPRB /)
!-- WVL=0.5500E-06
ZRALIS3( :, 9, 3) = (/ &
  & 15.0341_JPRB, 15.0339_JPRB, 15.0340_JPRB, 15.0339_JPRB, 18.0710_JPRB, 17.2273_JPRB, &
  & 16.6330_JPRB, 16.4622_JPRB, 17.7243_JPRB, 12.2329_JPRB,  6.2160_JPRB,  5.1526_JPRB /)
!-- WVL=0.6450E-06
ZRALIS3( :,10, 3) = (/ &
  & 14.4718_JPRB, 14.4718_JPRB, 14.4717_JPRB, 14.4718_JPRB, 18.5162_JPRB, 18.9629_JPRB, &
  & 18.2471_JPRB, 17.3557_JPRB, 16.4453_JPRB,  7.7635_JPRB,  6.7055_JPRB,  5.9191_JPRB /)
!-- WVL=0.6700E-06
ZRALIS3( :,11, 3) = (/ &
  & 14.2486_JPRB, 14.2486_JPRB, 14.2485_JPRB, 14.2486_JPRB, 17.8535_JPRB, 18.8282_JPRB, &
  & 18.7978_JPRB, 17.6934_JPRB, 16.8009_JPRB,  7.8794_JPRB,  7.0976_JPRB,  6.1846_JPRB /)
!-- WVL=0.8000E-06
ZRALIS3( :,12, 3) = (/ &
  & 14.5230_JPRB, 14.5230_JPRB, 14.5231_JPRB, 14.5230_JPRB, 15.0372_JPRB, 15.7883_JPRB, &
  & 17.0923_JPRB, 18.5022_JPRB, 18.7825_JPRB,  8.9799_JPRB,  8.5921_JPRB,  6.6416_JPRB /)
!-- WVL=0.8580E-06
ZRALIS3( :,13, 3) = (/ &
  & 15.1843_JPRB, 15.1843_JPRB, 15.1844_JPRB, 15.1843_JPRB, 14.7909_JPRB, 15.0567_JPRB, &
  & 15.8011_JPRB, 17.4459_JPRB, 18.7862_JPRB,  9.0638_JPRB,  9.0623_JPRB,  6.9300_JPRB /)
!-- WVL=0.8650E-06
ZRALIS3( :,14, 3) = (/ &
  & 15.2529_JPRB, 15.2529_JPRB, 15.2530_JPRB, 15.2529_JPRB, 14.7665_JPRB, 15.0507_JPRB, &
  & 15.6949_JPRB, 17.0265_JPRB, 18.7683_JPRB,  9.0845_JPRB,  9.1611_JPRB,  6.9142_JPRB /)
!-- WVL=0.1020E-05
ZRALIS3( :,15, 3) = (/ &
  & 17.8068_JPRB, 17.8069_JPRB, 17.8069_JPRB, 17.8069_JPRB, 14.3330_JPRB, 14.2577_JPRB, &
  & 14.6342_JPRB, 14.9263_JPRB, 15.9521_JPRB, 11.9362_JPRB,  9.2166_JPRB,  8.1451_JPRB /)
!-- WVL=0.1064E-05
ZRALIS3( :,16, 3) = (/ &
  & 18.2162_JPRB, 18.2163_JPRB, 18.2163_JPRB, 18.2163_JPRB, 14.3399_JPRB, 14.3044_JPRB, &
  & 14.4325_JPRB, 14.8442_JPRB, 15.3336_JPRB, 17.2092_JPRB,  9.5161_JPRB,  8.2696_JPRB /)
!-- WVL=0.1240E-05
ZRALIS3( :,17, 3) = (/ &
  & 17.6348_JPRB, 17.6348_JPRB, 17.6349_JPRB, 17.6349_JPRB, 15.2180_JPRB, 14.5174_JPRB, &
  & 14.2756_JPRB, 14.2494_JPRB, 14.6621_JPRB, 15.6265_JPRB, 10.8550_JPRB,  8.8493_JPRB /)
!-- WVL=0.1640E-05
ZRALIS3( :,18, 3) = (/ &
  & 12.1962_JPRB, 12.1962_JPRB, 12.1963_JPRB, 12.1963_JPRB, 18.3182_JPRB, 18.1232_JPRB, &
  & 17.1902_JPRB, 15.9137_JPRB, 13.0019_JPRB, 12.4159_JPRB, 12.2056_JPRB, 10.9663_JPRB /)
!-- WVL=0.2130E-05
ZRALIS3( :,19, 3) = (/ &
  &  8.7663_JPRB,  8.7663_JPRB,  8.7663_JPRB,  8.7663_JPRB, 14.1792_JPRB, 15.5410_JPRB, &
  & 16.7911_JPRB, 18.2503_JPRB, 14.4878_JPRB, 14.2335_JPRB, 13.5088_JPRB, 12.1490_JPRB /)


!------------------------------------ SEA SALT 9 bins ---------------------------------------------

!-- For sea-salt, XX(RH, wavelength, binNo)

  
 ! ** BIN No.            1   3.0000000000000000E-002   5.0000000000000000E-002
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS9( :, 1, 1) = (/ &
  & 0.02685_JPRB, 0.02685_JPRB, 0.02685_JPRB, 0.02685_JPRB, 0.23533_JPRB, 0.37231_JPRB, &
  & 0.56123_JPRB, 0.88373_JPRB, 1.58358_JPRB, 2.36335_JPRB, 3.16105_JPRB, 8.73123_JPRB /)
!-- WVL=0.3550E-06
ZALFSS9( :, 2, 1) = (/ &
  & 0.02266_JPRB, 0.02266_JPRB, 0.02266_JPRB, 0.02266_JPRB, 0.19833_JPRB, 0.31395_JPRB, &
  & 0.47366_JPRB, 0.74698_JPRB, 1.34283_JPRB, 2.01070_JPRB, 2.71092_JPRB, 7.62934_JPRB /)
!-- WVL=0.3800E-06
ZALFSS9( :, 3, 1) = (/ &
  & 0.01735_JPRB, 0.01735_JPRB, 0.01735_JPRB, 0.01735_JPRB, 0.15142_JPRB, 0.23980_JPRB, &
  & 0.36211_JPRB, 0.57204_JPRB, 1.03224_JPRB, 1.55190_JPRB, 2.11510_JPRB, 6.11179_JPRB /)
!-- WVL=0.4000E-06
ZALFSS9( :, 4, 1) = (/ &
  & 0.01420_JPRB, 0.01420_JPRB, 0.01420_JPRB, 0.01420_JPRB, 0.12355_JPRB, 0.19567_JPRB, &
  & 0.29558_JPRB, 0.46732_JPRB, 0.84498_JPRB, 1.27325_JPRB, 1.74717_JPRB, 5.13493_JPRB /)
!-- WVL=0.4400E-06
ZALFSS9( :, 5, 1) = (/ &
  & 0.00981_JPRB, 0.00981_JPRB, 0.00981_JPRB, 0.00981_JPRB, 0.08469_JPRB, 0.13408_JPRB, &
  & 0.20256_JPRB, 0.32050_JPRB, 0.58083_JPRB, 0.87767_JPRB, 1.21642_JPRB, 3.66474_JPRB /)
!-- WVL=0.4690E-06
ZALFSS9( :, 6, 1) = (/ &
  & 0.00768_JPRB, 0.00768_JPRB, 0.00768_JPRB, 0.00768_JPRB, 0.06580_JPRB, 0.10412_JPRB, &
  & 0.15726_JPRB, 0.24883_JPRB, 0.45128_JPRB, 0.68267_JPRB, 0.93097_JPRB, 2.90016_JPRB /)
!-- WVL=0.5000E-06
ZALFSS9( :, 7, 1) = (/ &
  & 0.00602_JPRB, 0.00602_JPRB, 0.00602_JPRB, 0.00602_JPRB, 0.05113_JPRB, 0.08083_JPRB, &
  & 0.12203_JPRB, 0.19304_JPRB, 0.35019_JPRB, 0.53007_JPRB, 0.71994_JPRB, 2.28211_JPRB /)
!-- WVL=0.5320E-06
ZALFSS9( :, 8, 1) = (/ &
  & 0.00476_JPRB, 0.00476_JPRB, 0.00476_JPRB, 0.00476_JPRB, 0.04007_JPRB, 0.06328_JPRB, &
  & 0.09547_JPRB, 0.15096_JPRB, 0.27380_JPRB, 0.39001_JPRB, 0.56470_JPRB, 1.80232_JPRB /)
!-- WVL=0.5500E-06
ZALFSS9( :, 9, 1) = (/ &
  & 0.00420_JPRB, 0.00420_JPRB, 0.00420_JPRB, 0.00420_JPRB, 0.03517_JPRB, 0.05552_JPRB, &
  & 0.08372_JPRB, 0.13233_JPRB, 0.23995_JPRB, 0.31049_JPRB, 0.49549_JPRB, 1.58608_JPRB /)
!-- WVL=0.6450E-06
ZALFSS9( :,10, 1) = (/ &
  & 0.00235_JPRB, 0.00235_JPRB, 0.00235_JPRB, 0.00235_JPRB, 0.01895_JPRB, 0.02978_JPRB, &
  & 0.04477_JPRB, 0.07058_JPRB, 0.12769_JPRB, 0.14734_JPRB, 0.26490_JPRB, 0.85306_JPRB /)
!-- WVL=0.6700E-06
ZALFSS9( :,11, 1) = (/ &
  & 0.00205_JPRB, 0.00205_JPRB, 0.00205_JPRB, 0.00205_JPRB, 0.01637_JPRB, 0.02570_JPRB, &
  & 0.03860_JPRB, 0.06080_JPRB, 0.10990_JPRB, 0.12673_JPRB, 0.23118_JPRB, 0.73468_JPRB /)
!-- WVL=0.8000E-06
ZALFSS9( :,12, 1) = (/ &
  & 0.00112_JPRB, 0.00112_JPRB, 0.00112_JPRB, 0.00112_JPRB, 0.00837_JPRB, 0.01304_JPRB, &
  & 0.01947_JPRB, 0.03050_JPRB, 0.05484_JPRB, 0.06282_JPRB, 0.11583_JPRB, 0.36470_JPRB /)
!-- WVL=0.8580E-06
ZALFSS9( :,13, 1) = (/ &
  & 0.00089_JPRB, 0.00089_JPRB, 0.00089_JPRB, 0.00089_JPRB, 0.00647_JPRB, 0.01003_JPRB, &
  & 0.01493_JPRB, 0.02332_JPRB, 0.04179_JPRB, 0.04766_JPRB, 0.08782_JPRB, 0.27637_JPRB /)
!-- WVL=0.8650E-06
ZALFSS9( :,14, 1) = (/ &
  & 0.00087_JPRB, 0.00087_JPRB, 0.00087_JPRB, 0.00087_JPRB, 0.00628_JPRB, 0.00973_JPRB, &
  & 0.01448_JPRB, 0.02261_JPRB, 0.04050_JPRB, 0.04616_JPRB, 0.08504_JPRB, 0.26762_JPRB /)
!-- WVL=0.1020E-05
ZALFSS9( :,15, 1) = (/ &
  & 0.00053_JPRB, 0.00053_JPRB, 0.00053_JPRB, 0.00053_JPRB, 0.00349_JPRB, 0.00534_JPRB, &
  & 0.00786_JPRB, 0.01216_JPRB, 0.02156_JPRB, 0.02576_JPRB, 0.04441_JPRB, 0.13927_JPRB /)
!-- WVL=0.1064E-05
ZALFSS9( :,16, 1) = (/ &
  & 0.00047_JPRB, 0.00047_JPRB, 0.00047_JPRB, 0.00047_JPRB, 0.00302_JPRB, 0.00460_JPRB, &
  & 0.00675_JPRB, 0.01041_JPRB, 0.01840_JPRB, 0.02493_JPRB, 0.03763_JPRB, 0.11783_JPRB /)
!-- WVL=0.1240E-05
ZALFSS9( :,17, 1) = (/ &
  & 0.00032_JPRB, 0.00032_JPRB, 0.00032_JPRB, 0.00032_JPRB, 0.00183_JPRB, 0.00273_JPRB, &
  & 0.00395_JPRB, 0.00601_JPRB, 0.01047_JPRB, 0.01318_JPRB, 0.02072_JPRB, 0.06439_JPRB /)
!-- WVL=0.1640E-05
ZALFSS9( :,18, 1) = (/ &
  & 0.00018_JPRB, 0.00018_JPRB, 0.00018_JPRB, 0.00018_JPRB, 0.00082_JPRB, 0.00117_JPRB, &
  & 0.00164_JPRB, 0.00240_JPRB, 0.00270_JPRB, 0.00398_JPRB, 0.00714_JPRB, 0.02161_JPRB /)
!-- WVL=0.2130E-05
ZALFSS9( :,19, 1) = (/ &
  & 0.00012_JPRB, 0.00012_JPRB, 0.00012_JPRB, 0.00012_JPRB, 0.00045_JPRB, 0.00062_JPRB, &
  & 0.00083_JPRB, 0.00116_JPRB, 0.00111_JPRB, 0.00160_JPRB, 0.00279_JPRB, 0.00801_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS9( :, 1, 1) = (/ &
  & 0.03555_JPRB, 0.03555_JPRB, 0.03555_JPRB, 0.03555_JPRB, 0.07319_JPRB, 0.08532_JPRB, &
  & 0.09797_JPRB, 0.11436_JPRB, 0.14012_JPRB, 0.16179_JPRB, 0.19419_JPRB, 0.29743_JPRB /)
!-- WVL=0.3550E-06
ZASYSS9( :, 2, 1) = (/ &
  & 0.03264_JPRB, 0.03264_JPRB, 0.03264_JPRB, 0.03264_JPRB, 0.06717_JPRB, 0.07829_JPRB, &
  & 0.08986_JPRB, 0.10483_JPRB, 0.12829_JPRB, 0.14794_JPRB, 0.17729_JPRB, 0.27022_JPRB /)
!-- WVL=0.3800E-06
ZASYSS9( :, 3, 1) = (/ &
  & 0.02852_JPRB, 0.02852_JPRB, 0.02852_JPRB, 0.02852_JPRB, 0.05869_JPRB, 0.06838_JPRB, &
  & 0.07846_JPRB, 0.09147_JPRB, 0.11176_JPRB, 0.12867_JPRB, 0.15388_JPRB, 0.23277_JPRB /)
!-- WVL=0.4000E-06
ZASYSS9( :, 4, 1) = (/ &
  & 0.02576_JPRB, 0.02576_JPRB, 0.02576_JPRB, 0.02576_JPRB, 0.05302_JPRB, 0.06177_JPRB, &
  & 0.07085_JPRB, 0.08257_JPRB, 0.10080_JPRB, 0.11595_JPRB, 0.13849_JPRB, 0.20841_JPRB /)
!-- WVL=0.4400E-06
ZASYSS9( :, 5, 1) = (/ &
  & 0.02132_JPRB, 0.02132_JPRB, 0.02132_JPRB, 0.02132_JPRB, 0.04390_JPRB, 0.05114_JPRB, &
  & 0.05865_JPRB, 0.06831_JPRB, 0.08331_JPRB, 0.09572_JPRB, 0.11411_JPRB, 0.17042_JPRB /)
!-- WVL=0.4690E-06
ZASYSS9( :, 6, 1) = (/ &
  & 0.01878_JPRB, 0.01878_JPRB, 0.01878_JPRB, 0.01878_JPRB, 0.03868_JPRB, 0.04506_JPRB, &
  & 0.05168_JPRB, 0.06019_JPRB, 0.07338_JPRB, 0.08426_JPRB, 0.10015_JPRB, 0.14930_JPRB /)
!-- WVL=0.5000E-06
ZASYSS9( :, 7, 1) = (/ &
  & 0.01653_JPRB, 0.01653_JPRB, 0.01653_JPRB, 0.01653_JPRB, 0.03407_JPRB, 0.03970_JPRB, &
  & 0.04553_JPRB, 0.05302_JPRB, 0.06462_JPRB, 0.07419_JPRB, 0.08803_JPRB, 0.13097_JPRB /)
!-- WVL=0.5320E-06
ZASYSS9( :, 8, 1) = (/ &
  & 0.01461_JPRB, 0.01461_JPRB, 0.01461_JPRB, 0.01461_JPRB, 0.03013_JPRB, 0.03510_JPRB, &
  & 0.04026_JPRB, 0.04689_JPRB, 0.05714_JPRB, 0.06509_JPRB, 0.07779_JPRB, 0.11549_JPRB /)
!-- WVL=0.5500E-06
ZASYSS9( :, 9, 1) = (/ &
  & 0.01368_JPRB, 0.01368_JPRB, 0.01368_JPRB, 0.01368_JPRB, 0.02821_JPRB, 0.03286_JPRB, &
  & 0.03769_JPRB, 0.04390_JPRB, 0.05349_JPRB, 0.06026_JPRB, 0.07280_JPRB, 0.10799_JPRB /)
!-- WVL=0.6450E-06
ZASYSS9( :,10, 1) = (/ &
  & 0.00996_JPRB, 0.00996_JPRB, 0.00996_JPRB, 0.00996_JPRB, 0.02056_JPRB, 0.02396_JPRB, &
  & 0.02748_JPRB, 0.03201_JPRB, 0.03901_JPRB, 0.04340_JPRB, 0.05307_JPRB, 0.07850_JPRB /)
!-- WVL=0.6700E-06
ZASYSS9( :,11, 1) = (/ &
  & 0.00923_JPRB, 0.00923_JPRB, 0.00923_JPRB, 0.00923_JPRB, 0.01906_JPRB, 0.02221_JPRB, &
  & 0.02548_JPRB, 0.02969_JPRB, 0.03618_JPRB, 0.04024_JPRB, 0.04928_JPRB, 0.07278_JPRB /)
!-- WVL=0.8000E-06
ZASYSS9( :,12, 1) = (/ &
  & 0.00648_JPRB, 0.00648_JPRB, 0.00648_JPRB, 0.00648_JPRB, 0.01340_JPRB, 0.01562_JPRB, &
  & 0.01792_JPRB, 0.02088_JPRB, 0.02545_JPRB, 0.02830_JPRB, 0.03471_JPRB, 0.05117_JPRB /)
!-- WVL=0.8580E-06
ZASYSS9( :,13, 1) = (/ &
  & 0.00564_JPRB, 0.00564_JPRB, 0.00564_JPRB, 0.00564_JPRB, 0.01165_JPRB, 0.01359_JPRB, &
  & 0.01559_JPRB, 0.01817_JPRB, 0.02215_JPRB, 0.02463_JPRB, 0.03021_JPRB, 0.04454_JPRB /)
!-- WVL=0.8650E-06
ZASYSS9( :,14, 1) = (/ &
  & 0.00555_JPRB, 0.00555_JPRB, 0.00555_JPRB, 0.00555_JPRB, 0.01147_JPRB, 0.01337_JPRB, &
  & 0.01534_JPRB, 0.01788_JPRB, 0.02180_JPRB, 0.02424_JPRB, 0.02972_JPRB, 0.04383_JPRB /)
!-- WVL=0.1020E-05
ZASYSS9( :,15, 1) = (/ &
  & 0.00399_JPRB, 0.00399_JPRB, 0.00399_JPRB, 0.00399_JPRB, 0.00826_JPRB, 0.00963_JPRB, &
  & 0.01105_JPRB, 0.01288_JPRB, 0.01571_JPRB, 0.01758_JPRB, 0.02142_JPRB, 0.03160_JPRB /)
!-- WVL=0.1064E-05
ZASYSS9( :,16, 1) = (/ &
  & 0.00367_JPRB, 0.00367_JPRB, 0.00367_JPRB, 0.00367_JPRB, 0.00759_JPRB, 0.00885_JPRB, &
  & 0.01016_JPRB, 0.01184_JPRB, 0.01444_JPRB, 0.01639_JPRB, 0.01970_JPRB, 0.02906_JPRB /)
!-- WVL=0.1240E-05
ZASYSS9( :,17, 1) = (/ &
  & 0.00270_JPRB, 0.00270_JPRB, 0.00270_JPRB, 0.00270_JPRB, 0.00559_JPRB, 0.00652_JPRB, &
  & 0.00749_JPRB, 0.00873_JPRB, 0.01065_JPRB, 0.01201_JPRB, 0.01453_JPRB, 0.02144_JPRB /)
!-- WVL=0.1640E-05
ZASYSS9( :,18, 1) = (/ &
  & 0.00155_JPRB, 0.00155_JPRB, 0.00155_JPRB, 0.00155_JPRB, 0.00320_JPRB, 0.00373_JPRB, &
  & 0.00428_JPRB, 0.00500_JPRB, 0.00590_JPRB, 0.00678_JPRB, 0.00832_JPRB, 0.01229_JPRB /)
!-- WVL=0.2130E-05
ZASYSS9( :,19, 1) = (/ &
  & 0.00092_JPRB, 0.00092_JPRB, 0.00092_JPRB, 0.00092_JPRB, 0.00190_JPRB, 0.00221_JPRB, &
  & 0.00254_JPRB, 0.00296_JPRB, 0.00350_JPRB, 0.00402_JPRB, 0.00494_JPRB, 0.00730_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS9( :, 1, 1) = (/ &
  & 0.97421_JPRB, 0.97421_JPRB, 0.97421_JPRB, 0.97421_JPRB, 0.99045_JPRB, 0.99220_JPRB, &
  & 0.99348_JPRB, 0.99463_JPRB, 0.99579_JPRB, 0.99642_JPRB, 0.99862_JPRB, 0.99919_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS9( :, 2, 1) = (/ &
  & 0.97094_JPRB, 0.97094_JPRB, 0.97094_JPRB, 0.97094_JPRB, 0.98928_JPRB, 0.99127_JPRB, &
  & 0.99271_JPRB, 0.99401_JPRB, 0.99533_JPRB, 0.99604_JPRB, 0.99849_JPRB, 0.99913_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS9( :, 3, 1) = (/ &
  & 0.96490_JPRB, 0.96490_JPRB, 0.96490_JPRB, 0.96490_JPRB, 0.98712_JPRB, 0.98954_JPRB, &
  & 0.99129_JPRB, 0.99288_JPRB, 0.99447_JPRB, 0.99534_JPRB, 0.99824_JPRB, 0.99901_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS9( :, 4, 1) = (/ &
  & 0.95954_JPRB, 0.95954_JPRB, 0.95954_JPRB, 0.95954_JPRB, 0.98520_JPRB, 0.98799_JPRB, &
  & 0.99002_JPRB, 0.99186_JPRB, 0.99371_JPRB, 0.99472_JPRB, 0.99801_JPRB, 0.99891_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS9( :, 5, 1) = (/ &
  & 0.94736_JPRB, 0.94737_JPRB, 0.94737_JPRB, 0.94737_JPRB, 0.98078_JPRB, 0.98444_JPRB, &
  & 0.98711_JPRB, 0.98952_JPRB, 0.99195_JPRB, 0.99328_JPRB, 0.99749_JPRB, 0.99866_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS9( :, 6, 1) = (/ &
  & 0.93730_JPRB, 0.93730_JPRB, 0.93730_JPRB, 0.93730_JPRB, 0.97708_JPRB, 0.98146_JPRB, &
  & 0.98467_JPRB, 0.98756_JPRB, 0.99048_JPRB, 0.99207_JPRB, 0.99735_JPRB, 0.99844_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS9( :, 7, 1) = (/ &
  & 0.92537_JPRB, 0.92537_JPRB, 0.92537_JPRB, 0.92537_JPRB, 0.97263_JPRB, 0.97789_JPRB, &
  & 0.98173_JPRB, 0.98520_JPRB, 0.98870_JPRB, 0.99062_JPRB, 0.99702_JPRB, 0.99819_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS9( :, 8, 1) = (/ &
  & 0.91181_JPRB, 0.91182_JPRB, 0.91181_JPRB, 0.91182_JPRB, 0.96750_JPRB, 0.97375_JPRB, &
  & 0.97832_JPRB, 0.98246_JPRB, 0.98664_JPRB, 0.98996_JPRB, 0.99649_JPRB, 0.99789_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS9( :, 9, 1) = (/ &
  & 0.90363_JPRB, 0.90364_JPRB, 0.90363_JPRB, 0.90364_JPRB, 0.96436_JPRB, 0.97121_JPRB, &
  & 0.97623_JPRB, 0.98078_JPRB, 0.98538_JPRB, 0.99110_JPRB, 0.99616_JPRB, 0.99771_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS9( :,10, 1) = (/ &
  & 0.85430_JPRB, 0.85430_JPRB, 0.85431_JPRB, 0.85430_JPRB, 0.94466_JPRB, 0.95524_JPRB, &
  & 0.96304_JPRB, 0.97014_JPRB, 0.97735_JPRB, 0.99083_JPRB, 0.99399_JPRB, 0.99653_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS9( :,11, 1) = (/ &
  & 0.83975_JPRB, 0.83974_JPRB, 0.83976_JPRB, 0.83974_JPRB, 0.93858_JPRB, 0.95029_JPRB, &
  & 0.95893_JPRB, 0.96682_JPRB, 0.97484_JPRB, 0.98981_JPRB, 0.99277_JPRB, 0.99616_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS9( :,12, 1) = (/ &
  & 0.75596_JPRB, 0.75596_JPRB, 0.75595_JPRB, 0.75595_JPRB, 0.90093_JPRB, 0.91936_JPRB, &
  & 0.93313_JPRB, 0.94583_JPRB, 0.95887_JPRB, 0.98322_JPRB, 0.98744_JPRB, 0.99379_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS9( :,13, 1) = (/ &
  & 0.71558_JPRB, 0.71557_JPRB, 0.71557_JPRB, 0.71557_JPRB, 0.88095_JPRB, 0.90277_JPRB, &
  & 0.91917_JPRB, 0.93438_JPRB, 0.95009_JPRB, 0.97955_JPRB, 0.98470_JPRB, 0.99246_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS9( :,14, 1) = (/ &
  & 0.71063_JPRB, 0.71063_JPRB, 0.71062_JPRB, 0.71063_JPRB, 0.87841_JPRB, 0.90065_JPRB, &
  & 0.91738_JPRB, 0.93291_JPRB, 0.94896_JPRB, 0.97907_JPRB, 0.98435_JPRB, 0.99228_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS9( :,15, 1) = (/ &
  & 0.60051_JPRB, 0.60050_JPRB, 0.60049_JPRB, 0.60050_JPRB, 0.81613_JPRB, 0.84792_JPRB, &
  & 0.87243_JPRB, 0.89560_JPRB, 0.91999_JPRB, 0.95796_JPRB, 0.97505_JPRB, 0.98775_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS9( :,16, 1) = (/ &
  & 0.56994_JPRB, 0.56994_JPRB, 0.56993_JPRB, 0.56993_JPRB, 0.79660_JPRB, 0.83110_JPRB, &
  & 0.85790_JPRB, 0.88339_JPRB, 0.91039_JPRB, 0.93588_JPRB, 0.97189_JPRB, 0.98619_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS9( :,17, 1) = (/ &
  & 0.45627_JPRB, 0.45627_JPRB, 0.45626_JPRB, 0.45626_JPRB, 0.71311_JPRB, 0.75760_JPRB, &
  & 0.79330_JPRB, 0.82819_JPRB, 0.86622_JPRB, 0.91297_JPRB, 0.95671_JPRB, 0.97868_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS9( :,18, 1) = (/ &
  & 0.26670_JPRB, 0.26670_JPRB, 0.26669_JPRB, 0.26669_JPRB, 0.51931_JPRB, 0.57622_JPRB, &
  & 0.62564_JPRB, 0.67757_JPRB, 0.85318_JPRB, 0.87724_JPRB, 0.90645_JPRB, 0.95297_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS9( :,19, 1) = (/ &
  & 0.14256_JPRB, 0.14256_JPRB, 0.14255_JPRB, 0.14254_JPRB, 0.33093_JPRB, 0.38379_JPRB, &
  & 0.43373_JPRB, 0.49074_JPRB, 0.72732_JPRB, 0.76650_JPRB, 0.81676_JPRB, 0.90344_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS9( :, 1, 1) = (/ &
  &  9.3751_JPRB,  9.3751_JPRB,  9.3751_JPRB,  9.3751_JPRB, 10.1483_JPRB, 10.4590_JPRB, &
  & 10.8057_JPRB, 11.2885_JPRB, 12.1276_JPRB, 12.9174_JPRB, 14.2903_JPRB, 20.6552_JPRB /)
!-- WVL=0.3550E-06
ZRALIS9( :, 2, 1) = (/ &
  &  9.3390_JPRB,  9.3390_JPRB,  9.3390_JPRB,  9.3390_JPRB, 10.0027_JPRB, 10.2763_JPRB, &
  & 10.5809_JPRB, 11.0032_JPRB, 11.7293_JPRB, 12.4029_JPRB, 13.5522_JPRB, 18.5859_JPRB /)
!-- WVL=0.3800E-06
ZRALIS9( :, 3, 1) = (/ &
  &  9.3024_JPRB,  9.3024_JPRB,  9.3024_JPRB,  9.3024_JPRB,  9.8081_JPRB, 10.0315_JPRB, &
  & 10.2807_JPRB, 10.6247_JPRB, 11.2093_JPRB, 11.7420_JPRB, 12.6264_JPRB, 16.2436_JPRB /)
!-- WVL=0.4000E-06
ZRALIS9( :, 4, 1) = (/ &
  &  9.2911_JPRB,  9.2911_JPRB,  9.2911_JPRB,  9.2911_JPRB,  9.6862_JPRB,  9.8771_JPRB, &
  & 10.0911_JPRB, 10.3867_JPRB, 10.8864_JPRB, 11.3374_JPRB, 12.0713_JPRB, 14.9653_JPRB /)
!-- WVL=0.4400E-06
ZRALIS9( :, 5, 1) = (/ &
  &  9.3089_JPRB,  9.3088_JPRB,  9.3088_JPRB,  9.3088_JPRB,  9.5083_JPRB,  9.6475_JPRB, &
  &  9.8071_JPRB, 10.0299_JPRB, 10.4062_JPRB, 10.7425_JPRB, 11.2693_JPRB, 13.2700_JPRB /)
!-- WVL=0.4690E-06
ZRALIS9( :, 6, 1) = (/ &
  &  9.3508_JPRB,  9.3508_JPRB,  9.3508_JPRB,  9.3508_JPRB,  9.4207_JPRB,  9.5295_JPRB, &
  &  9.6585_JPRB,  9.8415_JPRB, 10.1530_JPRB, 10.4312_JPRB, 10.8462_JPRB, 12.4568_JPRB /)
!-- WVL=0.5000E-06
ZRALIS9( :, 7, 1) = (/ &
  &  9.4199_JPRB,  9.4198_JPRB,  9.4199_JPRB,  9.4198_JPRB,  9.3559_JPRB,  9.4367_JPRB, &
  &  9.5381_JPRB,  9.6862_JPRB,  9.9427_JPRB, 10.1730_JPRB, 10.5003_JPRB, 11.8132_JPRB /)
!-- WVL=0.5320E-06
ZRALIS9( :, 8, 1) = (/ &
  &  9.5156_JPRB,  9.5156_JPRB,  9.5156_JPRB,  9.5156_JPRB,  9.3143_JPRB,  9.3691_JPRB, &
  &  9.4458_JPRB,  9.5637_JPRB,  9.7741_JPRB,  9.9447_JPRB, 10.2229_JPRB, 11.3102_JPRB /)
!-- WVL=0.5500E-06
ZRALIS9( :, 9, 1) = (/ &
  &  9.5800_JPRB,  9.5799_JPRB,  9.5800_JPRB,  9.5799_JPRB,  9.3004_JPRB,  9.3416_JPRB, &
  &  9.4056_JPRB,  9.5083_JPRB,  9.6961_JPRB,  9.8136_JPRB, 10.0928_JPRB, 11.0791_JPRB /)
!-- WVL=0.6450E-06
ZRALIS9( :,10, 1) = (/ &
  & 10.0427_JPRB, 10.0428_JPRB, 10.0426_JPRB, 10.0428_JPRB,  9.3182_JPRB,  9.2918_JPRB, &
  &  9.2966_JPRB,  9.3322_JPRB,  9.4259_JPRB,  9.4064_JPRB,  9.6104_JPRB, 10.2420_JPRB /)
!-- WVL=0.6700E-06
ZRALIS9( :,11, 1) = (/ &
  & 10.1990_JPRB, 10.1990_JPRB, 10.1989_JPRB, 10.1990_JPRB,  9.3445_JPRB,  9.3006_JPRB, &
  &  9.2907_JPRB,  9.3107_JPRB,  9.3838_JPRB,  9.3417_JPRB,  9.5294_JPRB, 10.0923_JPRB /)
!-- WVL=0.8000E-06
ZRALIS9( :,12, 1) = (/ &
  & 11.2548_JPRB, 11.2549_JPRB, 11.2549_JPRB, 11.2549_JPRB,  9.6022_JPRB,  9.4604_JPRB, &
  &  9.3730_JPRB,  9.3140_JPRB,  9.2907_JPRB,  9.1282_JPRB,  9.2351_JPRB,  9.5661_JPRB /)
!-- WVL=0.8580E-06
ZRALIS9( :,13, 1) = (/ &
  & 11.8659_JPRB, 11.8660_JPRB, 11.8661_JPRB, 11.8661_JPRB,  9.7788_JPRB,  9.5870_JPRB, &
  &  9.4617_JPRB,  9.3661_JPRB,  9.3010_JPRB,  9.0797_JPRB,  9.1576_JPRB,  9.4189_JPRB /)
!-- WVL=0.8650E-06
ZRALIS9( :,14, 1) = (/ &
  & 11.9459_JPRB, 11.9460_JPRB, 11.9461_JPRB, 11.9460_JPRB,  9.8026_JPRB,  9.6045_JPRB, &
  &  9.4744_JPRB,  9.3742_JPRB,  9.3040_JPRB,  9.0753_JPRB,  9.1499_JPRB,  9.4036_JPRB /)
!-- WVL=0.1020E-05
ZRALIS9( :,15, 1) = (/ &
  & 14.0842_JPRB, 14.0843_JPRB, 14.0845_JPRB, 14.0845_JPRB, 10.4696_JPRB, 10.1102_JPRB, &
  &  9.8599_JPRB,  9.6473_JPRB,  9.4560_JPRB,  9.1244_JPRB,  9.0499_JPRB,  9.1615_JPRB /)
!-- WVL=0.1064E-05
ZRALIS9( :,16, 1) = (/ &
  & 14.8280_JPRB, 14.8281_JPRB, 14.8283_JPRB, 14.8283_JPRB, 10.7091_JPRB, 10.2956_JPRB, &
  & 10.0053_JPRB,  9.7561_JPRB,  9.5265_JPRB,  9.3117_JPRB,  9.0410_JPRB,  9.1182_JPRB /)
!-- WVL=0.1240E-05
ZRALIS9( :,17, 1) = (/ &
  & 18.4794_JPRB, 18.4797_JPRB, 18.4801_JPRB, 18.4801_JPRB, 11.9058_JPRB, 11.2315_JPRB, &
  & 10.7510_JPRB, 10.3287_JPRB,  9.9210_JPRB,  9.4448_JPRB,  9.0691_JPRB,  9.0169_JPRB /)
!-- WVL=0.1640E-05
ZRALIS9( :,18, 1) = (/ &
  & 31.5272_JPRB, 31.5272_JPRB, 31.5290_JPRB, 31.5293_JPRB, 16.2555_JPRB, 14.6688_JPRB, &
  & 13.5279_JPRB, 12.5122_JPRB,  9.9593_JPRB,  9.7068_JPRB,  9.4290_JPRB,  9.0555_JPRB /)
!-- WVL=0.2130E-05
ZRALIS9( :,19, 1) = (/ &
  & 58.8938_JPRB, 58.8938_JPRB, 58.8992_JPRB, 58.9001_JPRB, 25.4299_JPRB, 21.9441_JPRB, &
  & 19.4326_JPRB, 17.1922_JPRB, 11.6156_JPRB, 11.0356_JPRB, 10.3793_JPRB,  9.4371_JPRB /)
  
 ! ** BIN No.            2   5.0000000000000000E-002   0.1000000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS9( :, 1, 2) = (/ &
  & 0.18820_JPRB, 0.18820_JPRB, 0.18820_JPRB, 0.18820_JPRB, 1.40284_JPRB, 2.07244_JPRB, &
  & 2.92546_JPRB, 4.30678_JPRB, 7.21491_JPRB,10.26634_JPRB,12.08667_JPRB,26.41733_JPRB /)
!-- WVL=0.3550E-06
ZALFSS9( :, 2, 2) = (/ &
  & 0.15954_JPRB, 0.15954_JPRB, 0.15954_JPRB, 0.15954_JPRB, 1.22270_JPRB, 1.81799_JPRB, &
  & 2.57240_JPRB, 3.77508_JPRB, 6.28452_JPRB, 9.01072_JPRB,10.78307_JPRB,24.23724_JPRB /)
!-- WVL=0.3800E-06
ZALFSS9( :, 3, 2) = (/ &
  & 0.12260_JPRB, 0.12260_JPRB, 0.12260_JPRB, 0.12260_JPRB, 0.97595_JPRB, 1.46846_JPRB, &
  & 2.09481_JPRB, 3.08051_JPRB, 5.07683_JPRB, 7.25611_JPRB, 8.84076_JPRB,21.14804_JPRB /)
!-- WVL=0.4000E-06
ZALFSS9( :, 4, 2) = (/ &
  & 0.10034_JPRB, 0.10034_JPRB, 0.10034_JPRB, 0.10034_JPRB, 0.81778_JPRB, 1.24161_JPRB, &
  & 1.78528_JPRB, 2.64145_JPRB, 4.34665_JPRB, 6.17214_JPRB, 7.56002_JPRB,18.89177_JPRB /)
!-- WVL=0.4400E-06
ZALFSS9( :, 5, 2) = (/ &
  & 0.06896_JPRB, 0.06896_JPRB, 0.06896_JPRB, 0.06896_JPRB, 0.58071_JPRB, 0.89469_JPRB, &
  & 1.30583_JPRB, 1.96344_JPRB, 3.26938_JPRB, 4.62515_JPRB, 5.68419_JPRB,14.72501_JPRB /)
!-- WVL=0.4690E-06
ZALFSS9( :, 6, 2) = (/ &
  & 0.05358_JPRB, 0.05358_JPRB, 0.05358_JPRB, 0.05358_JPRB, 0.45801_JPRB, 0.71113_JPRB, &
  & 1.04697_JPRB, 1.59180_JPRB, 2.68726_JPRB, 3.82224_JPRB, 4.62219_JPRB,12.17907_JPRB /)
!-- WVL=0.5000E-06
ZALFSS9( :, 7, 2) = (/ &
  & 0.04158_JPRB, 0.04158_JPRB, 0.04158_JPRB, 0.04158_JPRB, 0.35923_JPRB, 0.56116_JPRB, &
  & 0.83214_JPRB, 1.27778_JPRB, 2.18894_JPRB, 3.14193_JPRB, 3.81132_JPRB,10.02818_JPRB /)
!-- WVL=0.5320E-06
ZALFSS9( :, 8, 2) = (/ &
  & 0.03251_JPRB, 0.03251_JPRB, 0.03251_JPRB, 0.03251_JPRB, 0.28287_JPRB, 0.44386_JPRB, &
  & 0.66182_JPRB, 1.02442_JPRB, 1.77816_JPRB, 2.41465_JPRB, 3.17202_JPRB, 8.34554_JPRB /)
!-- WVL=0.5500E-06
ZALFSS9( :, 9, 2) = (/ &
  & 0.02850_JPRB, 0.02850_JPRB, 0.02850_JPRB, 0.02850_JPRB, 0.24857_JPRB, 0.39077_JPRB, &
  & 0.58407_JPRB, 0.90735_JPRB, 1.58492_JPRB, 1.95276_JPRB, 2.86564_JPRB, 7.57949_JPRB /)
!-- WVL=0.6450E-06
ZALFSS9( :,10, 2) = (/ &
  & 0.01517_JPRB, 0.01517_JPRB, 0.01517_JPRB, 0.01517_JPRB, 0.13293_JPRB, 0.21014_JPRB, &
  & 0.31642_JPRB, 0.49735_JPRB, 0.88806_JPRB, 0.99217_JPRB, 1.70669_JPRB, 4.77167_JPRB /)
!-- WVL=0.6700E-06
ZALFSS9( :,11, 2) = (/ &
  & 0.01306_JPRB, 0.01306_JPRB, 0.01306_JPRB, 0.01306_JPRB, 0.11437_JPRB, 0.18092_JPRB, &
  & 0.27269_JPRB, 0.42933_JPRB, 0.76924_JPRB, 0.86333_JPRB, 1.51766_JPRB, 4.24950_JPRB /)
!-- WVL=0.8000E-06
ZALFSS9( :,12, 2) = (/ &
  & 0.00652_JPRB, 0.00652_JPRB, 0.00652_JPRB, 0.00652_JPRB, 0.05663_JPRB, 0.08968_JPRB, &
  & 0.13546_JPRB, 0.21418_JPRB, 0.38740_JPRB, 0.44178_JPRB, 0.80285_JPRB, 2.37050_JPRB /)
!-- WVL=0.8580E-06
ZALFSS9( :,13, 2) = (/ &
  & 0.00498_JPRB, 0.00498_JPRB, 0.00498_JPRB, 0.00498_JPRB, 0.04292_JPRB, 0.06795_JPRB, &
  & 0.10264_JPRB, 0.16238_JPRB, 0.29418_JPRB, 0.33689_JPRB, 0.61530_JPRB, 1.84949_JPRB /)
!-- WVL=0.8650E-06
ZALFSS9( :,14, 2) = (/ &
  & 0.00482_JPRB, 0.00482_JPRB, 0.00482_JPRB, 0.00482_JPRB, 0.04157_JPRB, 0.06580_JPRB, &
  & 0.09939_JPRB, 0.15724_JPRB, 0.28491_JPRB, 0.32640_JPRB, 0.59643_JPRB, 1.79600_JPRB /)
!-- WVL=0.1020E-05
ZALFSS9( :,15, 2) = (/ &
  & 0.00257_JPRB, 0.00257_JPRB, 0.00257_JPRB, 0.00257_JPRB, 0.02171_JPRB, 0.03429_JPRB, &
  & 0.05174_JPRB, 0.08181_JPRB, 0.14838_JPRB, 0.18130_JPRB, 0.31453_JPRB, 0.97236_JPRB /)
!-- WVL=0.1064E-05
ZALFSS9( :,16, 2) = (/ &
  & 0.00220_JPRB, 0.00220_JPRB, 0.00220_JPRB, 0.00220_JPRB, 0.01840_JPRB, 0.02904_JPRB, &
  & 0.04379_JPRB, 0.06921_JPRB, 0.12550_JPRB, 0.17322_JPRB, 0.26650_JPRB, 0.82734_JPRB /)
!-- WVL=0.1240E-05
ZALFSS9( :,17, 2) = (/ &
  & 0.00125_JPRB, 0.00125_JPRB, 0.00125_JPRB, 0.00125_JPRB, 0.01015_JPRB, 0.01596_JPRB, &
  & 0.02399_JPRB, 0.03784_JPRB, 0.06846_JPRB, 0.08966_JPRB, 0.14569_JPRB, 0.45669_JPRB /)
!-- WVL=0.1640E-05
ZALFSS9( :,18, 2) = (/ &
  & 0.00048_JPRB, 0.00048_JPRB, 0.00048_JPRB, 0.00048_JPRB, 0.00353_JPRB, 0.00548_JPRB, &
  & 0.00817_JPRB, 0.01276_JPRB, 0.01730_JPRB, 0.02614_JPRB, 0.04816_JPRB, 0.15157_JPRB /)
!-- WVL=0.2130E-05
ZALFSS9( :,19, 2) = (/ &
  & 0.00023_JPRB, 0.00023_JPRB, 0.00023_JPRB, 0.00023_JPRB, 0.00140_JPRB, 0.00213_JPRB, &
  & 0.00312_JPRB, 0.00480_JPRB, 0.00624_JPRB, 0.00939_JPRB, 0.01721_JPRB, 0.05382_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS9( :, 1, 2) = (/ &
  & 0.13692_JPRB, 0.13692_JPRB, 0.13692_JPRB, 0.13692_JPRB, 0.29955_JPRB, 0.35678_JPRB, &
  & 0.41571_JPRB, 0.48429_JPRB, 0.55733_JPRB, 0.58577_JPRB, 0.61830_JPRB, 0.67098_JPRB /)
!-- WVL=0.3550E-06
ZASYSS9( :, 2, 2) = (/ &
  & 0.12538_JPRB, 0.12538_JPRB, 0.12538_JPRB, 0.12538_JPRB, 0.27176_JPRB, 0.32351_JPRB, &
  & 0.37822_JPRB, 0.44597_JPRB, 0.52975_JPRB, 0.57040_JPRB, 0.60819_JPRB, 0.65522_JPRB /)
!-- WVL=0.3800E-06
ZASYSS9( :, 3, 2) = (/ &
  & 0.10926_JPRB, 0.10926_JPRB, 0.10926_JPRB, 0.10926_JPRB, 0.23365_JPRB, 0.27732_JPRB, &
  & 0.32433_JPRB, 0.38574_JPRB, 0.47433_JPRB, 0.53080_JPRB, 0.58252_JPRB, 0.63656_JPRB /)
!-- WVL=0.4000E-06
ZASYSS9( :, 4, 2) = (/ &
  & 0.09856_JPRB, 0.09856_JPRB, 0.09856_JPRB, 0.09856_JPRB, 0.20897_JPRB, 0.24731_JPRB, &
  & 0.28871_JPRB, 0.34375_JPRB, 0.42841_JPRB, 0.49016_JPRB, 0.55237_JPRB, 0.62626_JPRB /)
!-- WVL=0.4400E-06
ZASYSS9( :, 5, 2) = (/ &
  & 0.08147_JPRB, 0.08147_JPRB, 0.08147_JPRB, 0.08147_JPRB, 0.17065_JPRB, 0.20093_JPRB, &
  & 0.23346_JPRB, 0.27699_JPRB, 0.34729_JPRB, 0.40548_JPRB, 0.47606_JPRB, 0.60332_JPRB /)
!-- WVL=0.4690E-06
ZASYSS9( :, 6, 2) = (/ &
  & 0.07176_JPRB, 0.07176_JPRB, 0.07176_JPRB, 0.07176_JPRB, 0.14943_JPRB, 0.17546_JPRB, &
  & 0.20324_JPRB, 0.24028_JPRB, 0.30044_JPRB, 0.35179_JPRB, 0.41824_JPRB, 0.57495_JPRB /)
!-- WVL=0.5000E-06
ZASYSS9( :, 7, 2) = (/ &
  & 0.06320_JPRB, 0.06320_JPRB, 0.06320_JPRB, 0.06320_JPRB, 0.13105_JPRB, 0.15352_JPRB, &
  & 0.17737_JPRB, 0.20897_JPRB, 0.26015_JPRB, 0.30423_JPRB, 0.36410_JPRB, 0.53210_JPRB /)
!-- WVL=0.5320E-06
ZASYSS9( :, 8, 2) = (/ &
  & 0.05589_JPRB, 0.05589_JPRB, 0.05589_JPRB, 0.05589_JPRB, 0.11554_JPRB, 0.13513_JPRB, &
  & 0.15581_JPRB, 0.18304_JPRB, 0.22684_JPRB, 0.26189_JPRB, 0.31749_JPRB, 0.48054_JPRB /)
!-- WVL=0.5500E-06
ZASYSS9( :, 9, 2) = (/ &
  & 0.05232_JPRB, 0.05232_JPRB, 0.05232_JPRB, 0.05232_JPRB, 0.10804_JPRB, 0.12627_JPRB, &
  & 0.14546_JPRB, 0.17065_JPRB, 0.21102_JPRB, 0.23991_JPRB, 0.29494_JPRB, 0.45114_JPRB /)
!-- WVL=0.6450E-06
ZASYSS9( :,10, 2) = (/ &
  & 0.03816_JPRB, 0.03816_JPRB, 0.03816_JPRB, 0.03816_JPRB, 0.07856_JPRB, 0.09161_JPRB, &
  & 0.10523_JPRB, 0.12291_JPRB, 0.15077_JPRB, 0.16850_JPRB, 0.20860_JPRB, 0.32076_JPRB /)
!-- WVL=0.6700E-06
ZASYSS9( :,11, 2) = (/ &
  & 0.03539_JPRB, 0.03539_JPRB, 0.03539_JPRB, 0.03539_JPRB, 0.07284_JPRB, 0.08492_JPRB, &
  & 0.09750_JPRB, 0.11381_JPRB, 0.13942_JPRB, 0.15573_JPRB, 0.19274_JPRB, 0.29485_JPRB /)
!-- WVL=0.8000E-06
ZASYSS9( :,12, 2) = (/ &
  & 0.02489_JPRB, 0.02489_JPRB, 0.02489_JPRB, 0.02489_JPRB, 0.05124_JPRB, 0.05970_JPRB, &
  & 0.06848_JPRB, 0.07979_JPRB, 0.09738_JPRB, 0.10855_JPRB, 0.13364_JPRB, 0.20064_JPRB /)
!-- WVL=0.8580E-06
ZASYSS9( :,13, 2) = (/ &
  & 0.02166_JPRB, 0.02166_JPRB, 0.02166_JPRB, 0.02166_JPRB, 0.04461_JPRB, 0.05197_JPRB, &
  & 0.05960_JPRB, 0.06943_JPRB, 0.08467_JPRB, 0.09432_JPRB, 0.11596_JPRB, 0.17317_JPRB /)
!-- WVL=0.8650E-06
ZASYSS9( :,14, 2) = (/ &
  & 0.02132_JPRB, 0.02132_JPRB, 0.02132_JPRB, 0.02132_JPRB, 0.04390_JPRB, 0.05114_JPRB, &
  & 0.05865_JPRB, 0.06831_JPRB, 0.08331_JPRB, 0.09280_JPRB, 0.11407_JPRB, 0.17026_JPRB /)
!-- WVL=0.1020E-05
ZASYSS9( :,15, 2) = (/ &
  & 0.01536_JPRB, 0.01536_JPRB, 0.01536_JPRB, 0.01536_JPRB, 0.03166_JPRB, 0.03689_JPRB, &
  & 0.04231_JPRB, 0.04927_JPRB, 0.06005_JPRB, 0.06723_JPRB, 0.08198_JPRB, 0.12143_JPRB /)
!-- WVL=0.1064E-05
ZASYSS9( :,16, 2) = (/ &
  & 0.01412_JPRB, 0.01412_JPRB, 0.01412_JPRB, 0.01412_JPRB, 0.02912_JPRB, 0.03393_JPRB, &
  & 0.03891_JPRB, 0.04532_JPRB, 0.05523_JPRB, 0.06267_JPRB, 0.07537_JPRB, 0.11150_JPRB /)
!-- WVL=0.1240E-05
ZASYSS9( :,17, 2) = (/ &
  & 0.01041_JPRB, 0.01041_JPRB, 0.01041_JPRB, 0.01041_JPRB, 0.02149_JPRB, 0.02504_JPRB, &
  & 0.02873_JPRB, 0.03346_JPRB, 0.04078_JPRB, 0.04599_JPRB, 0.05561_JPRB, 0.08206_JPRB /)
!-- WVL=0.1640E-05
ZASYSS9( :,18, 2) = (/ &
  & 0.00596_JPRB, 0.00596_JPRB, 0.00596_JPRB, 0.00596_JPRB, 0.01232_JPRB, 0.01437_JPRB, &
  & 0.01649_JPRB, 0.01921_JPRB, 0.02268_JPRB, 0.02604_JPRB, 0.03194_JPRB, 0.04709_JPRB /)
!-- WVL=0.2130E-05
ZASYSS9( :,19, 2) = (/ &
  & 0.00354_JPRB, 0.00354_JPRB, 0.00354_JPRB, 0.00354_JPRB, 0.00732_JPRB, 0.00854_JPRB, &
  & 0.00980_JPRB, 0.01142_JPRB, 0.01348_JPRB, 0.01549_JPRB, 0.01900_JPRB, 0.02803_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS9( :, 1, 2) = (/ &
  & 0.99554_JPRB, 0.99554_JPRB, 0.99554_JPRB, 0.99554_JPRB, 0.99788_JPRB, 0.99811_JPRB, &
  & 0.99826_JPRB, 0.99841_JPRB, 0.99858_JPRB, 0.99871_JPRB, 0.99948_JPRB, 0.99963_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS9( :, 2, 2) = (/ &
  & 0.99505_JPRB, 0.99505_JPRB, 0.99505_JPRB, 0.99505_JPRB, 0.99773_JPRB, 0.99799_JPRB, &
  & 0.99817_JPRB, 0.99833_JPRB, 0.99850_JPRB, 0.99863_JPRB, 0.99945_JPRB, 0.99962_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS9( :, 3, 2) = (/ &
  & 0.99414_JPRB, 0.99414_JPRB, 0.99414_JPRB, 0.99414_JPRB, 0.99743_JPRB, 0.99776_JPRB, &
  & 0.99799_JPRB, 0.99819_JPRB, 0.99839_JPRB, 0.99851_JPRB, 0.99940_JPRB, 0.99960_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS9( :, 4, 2) = (/ &
  & 0.99333_JPRB, 0.99333_JPRB, 0.99333_JPRB, 0.99333_JPRB, 0.99716_JPRB, 0.99755_JPRB, &
  & 0.99783_JPRB, 0.99807_JPRB, 0.99829_JPRB, 0.99842_JPRB, 0.99936_JPRB, 0.99958_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS9( :, 5, 2) = (/ &
  & 0.99145_JPRB, 0.99145_JPRB, 0.99145_JPRB, 0.99145_JPRB, 0.99652_JPRB, 0.99705_JPRB, &
  & 0.99743_JPRB, 0.99776_JPRB, 0.99808_JPRB, 0.99824_JPRB, 0.99928_JPRB, 0.99952_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS9( :, 6, 2) = (/ &
  & 0.98988_JPRB, 0.98988_JPRB, 0.98988_JPRB, 0.98988_JPRB, 0.99597_JPRB, 0.99662_JPRB, &
  & 0.99708_JPRB, 0.99749_JPRB, 0.99789_JPRB, 0.99809_JPRB, 0.99930_JPRB, 0.99948_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS9( :, 7, 2) = (/ &
  & 0.98798_JPRB, 0.98798_JPRB, 0.98798_JPRB, 0.98798_JPRB, 0.99531_JPRB, 0.99609_JPRB, &
  & 0.99665_JPRB, 0.99716_JPRB, 0.99765_JPRB, 0.99791_JPRB, 0.99928_JPRB, 0.99943_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS9( :, 8, 2) = (/ &
  & 0.98578_JPRB, 0.98578_JPRB, 0.98578_JPRB, 0.98578_JPRB, 0.99454_JPRB, 0.99547_JPRB, &
  & 0.99615_JPRB, 0.99676_JPRB, 0.99736_JPRB, 0.99791_JPRB, 0.99921_JPRB, 0.99939_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS9( :, 9, 2) = (/ &
  & 0.98443_JPRB, 0.98443_JPRB, 0.98443_JPRB, 0.98443_JPRB, 0.99406_JPRB, 0.99509_JPRB, &
  & 0.99584_JPRB, 0.99652_JPRB, 0.99718_JPRB, 0.99821_JPRB, 0.99917_JPRB, 0.99937_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS9( :,10, 2) = (/ &
  & 0.97586_JPRB, 0.97586_JPRB, 0.97586_JPRB, 0.97586_JPRB, 0.99103_JPRB, 0.99267_JPRB, &
  & 0.99386_JPRB, 0.99493_JPRB, 0.99601_JPRB, 0.99836_JPRB, 0.99886_JPRB, 0.99922_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS9( :,11, 2) = (/ &
  & 0.97318_JPRB, 0.97317_JPRB, 0.97318_JPRB, 0.97317_JPRB, 0.99008_JPRB, 0.99191_JPRB, &
  & 0.99323_JPRB, 0.99443_JPRB, 0.99564_JPRB, 0.99821_JPRB, 0.99866_JPRB, 0.99917_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS9( :,12, 2) = (/ &
  & 0.95615_JPRB, 0.95615_JPRB, 0.95615_JPRB, 0.95615_JPRB, 0.98398_JPRB, 0.98701_JPRB, &
  & 0.98922_JPRB, 0.99121_JPRB, 0.99322_JPRB, 0.99724_JPRB, 0.99786_JPRB, 0.99883_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS9( :,13, 2) = (/ &
  & 0.94681_JPRB, 0.94681_JPRB, 0.94681_JPRB, 0.94681_JPRB, 0.98058_JPRB, 0.98429_JPRB, &
  & 0.98698_JPRB, 0.98942_JPRB, 0.99188_JPRB, 0.99670_JPRB, 0.99746_JPRB, 0.99864_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS9( :,14, 2) = (/ &
  & 0.94561_JPRB, 0.94561_JPRB, 0.94561_JPRB, 0.94561_JPRB, 0.98014_JPRB, 0.98393_JPRB, &
  & 0.98669_JPRB, 0.98918_JPRB, 0.99170_JPRB, 0.99663_JPRB, 0.99741_JPRB, 0.99862_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS9( :,15, 2) = (/ &
  & 0.91484_JPRB, 0.91484_JPRB, 0.91484_JPRB, 0.91484_JPRB, 0.96866_JPRB, 0.97469_JPRB, &
  & 0.97910_JPRB, 0.98308_JPRB, 0.98711_JPRB, 0.99338_JPRB, 0.99604_JPRB, 0.99795_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS9( :,16, 2) = (/ &
  & 0.90468_JPRB, 0.90467_JPRB, 0.90467_JPRB, 0.90467_JPRB, 0.96477_JPRB, 0.97155_JPRB, &
  & 0.97651_JPRB, 0.98100_JPRB, 0.98555_JPRB, 0.98978_JPRB, 0.99557_JPRB, 0.99773_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS9( :,17, 2) = (/ &
  & 0.85806_JPRB, 0.85806_JPRB, 0.85805_JPRB, 0.85805_JPRB, 0.94621_JPRB, 0.95651_JPRB, &
  & 0.96410_JPRB, 0.97099_JPRB, 0.97800_JPRB, 0.98619_JPRB, 0.99329_JPRB, 0.99662_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS9( :,18, 2) = (/ &
  & 0.72521_JPRB, 0.72521_JPRB, 0.72519_JPRB, 0.72519_JPRB, 0.88583_JPRB, 0.90683_JPRB, &
  & 0.92262_JPRB, 0.93721_JPRB, 0.97621_JPRB, 0.98047_JPRB, 0.98538_JPRB, 0.99279_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS9( :,19, 2) = (/ &
  & 0.54775_JPRB, 0.54775_JPRB, 0.54773_JPRB, 0.54772_JPRB, 0.78171_JPRB, 0.81820_JPRB, &
  & 0.84670_JPRB, 0.87391_JPRB, 0.95029_JPRB, 0.95910_JPRB, 0.96939_JPRB, 0.98496_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS9( :, 1, 2) = (/ &
  & 12.0148_JPRB, 12.0148_JPRB, 12.0148_JPRB, 12.0148_JPRB, 20.6697_JPRB, 26.4272_JPRB, &
  & 35.7134_JPRB, 55.2637_JPRB, 94.5000_JPRB, 99.3343_JPRB, 84.8094_JPRB, 71.3247_JPRB /)
!-- WVL=0.3550E-06
ZRALIS9( :, 2, 2) = (/ &
  & 11.6332_JPRB, 11.6332_JPRB, 11.6332_JPRB, 11.6332_JPRB, 18.5772_JPRB, 22.8024_JPRB, &
  & 29.2947_JPRB, 42.7398_JPRB, 77.8415_JPRB,100.2667_JPRB, 96.7041_JPRB, 66.5796_JPRB /)
!-- WVL=0.3800E-06
ZRALIS9( :, 3, 2) = (/ &
  & 11.1336_JPRB, 11.1336_JPRB, 11.1336_JPRB, 11.1336_JPRB, 16.2218_JPRB, 18.9670_JPRB, &
  & 22.8815_JPRB, 30.4242_JPRB, 51.5222_JPRB, 78.4737_JPRB,104.2978_JPRB, 68.8057_JPRB /)
!-- WVL=0.4000E-06
ZRALIS9( :, 4, 2) = (/ &
  & 10.8227_JPRB, 10.8227_JPRB, 10.8227_JPRB, 10.8227_JPRB, 14.9436_JPRB, 17.0073_JPRB, &
  & 19.8098_JPRB, 24.9023_JPRB, 38.4201_JPRB, 57.6559_JPRB, 89.8927_JPRB, 77.3402_JPRB /)
!-- WVL=0.4400E-06
ZRALIS9( :, 5, 2) = (/ &
  & 10.3598_JPRB, 10.3598_JPRB, 10.3598_JPRB, 10.3598_JPRB, 13.2574_JPRB, 14.5619_JPRB, &
  & 16.2111_JPRB, 18.9438_JPRB, 25.3011_JPRB, 33.7500_JPRB, 51.8961_JPRB,102.0012_JPRB /)
!-- WVL=0.4690E-06
ZRALIS9( :, 6, 2) = (/ &
  & 10.1156_JPRB, 10.1156_JPRB, 10.1156_JPRB, 10.1156_JPRB, 12.4527_JPRB, 13.4517_JPRB, &
  & 14.6701_JPRB, 16.5953_JPRB, 20.7429_JPRB, 25.8257_JPRB, 36.3112_JPRB,101.9998_JPRB /)
!-- WVL=0.5000E-06
ZRALIS9( :, 7, 2) = (/ &
  &  9.9130_JPRB,  9.9130_JPRB,  9.9130_JPRB,  9.9130_JPRB, 11.8180_JPRB, 12.6015_JPRB, &
  & 13.5304_JPRB, 14.9436_JPRB, 17.8031_JPRB, 21.0593_JPRB, 27.4722_JPRB, 77.7395_JPRB /)
!-- WVL=0.5320E-06
ZRALIS9( :, 8, 2) = (/ &
  &  9.7509_JPRB,  9.7509_JPRB,  9.7509_JPRB,  9.7509_JPRB, 11.3239_JPRB, 11.9542_JPRB, &
  & 12.6859_JPRB, 13.7673_JPRB, 15.8518_JPRB, 17.9299_JPRB, 22.3311_JPRB, 53.5021_JPRB /)
!-- WVL=0.5500E-06
ZRALIS9( :, 9, 2) = (/ &
  &  9.6762_JPRB,  9.6761_JPRB,  9.6762_JPRB,  9.6761_JPRB, 11.0975_JPRB, 11.6618_JPRB, &
  & 12.3106_JPRB, 13.2574_JPRB, 15.0432_JPRB, 16.6016_JPRB, 20.3788_JPRB, 44.1257_JPRB /)
!-- WVL=0.6450E-06
ZRALIS9( :,10, 2) = (/ &
  &  9.4199_JPRB,  9.4199_JPRB,  9.4199_JPRB,  9.4199_JPRB, 10.2851_JPRB, 10.6295_JPRB, &
  & 11.0148_JPRB, 11.5543_JPRB, 12.5010_JPRB, 13.1843_JPRB, 14.9575_JPRB, 22.6385_JPRB /)
!-- WVL=0.6700E-06
ZRALIS9( :,11, 2) = (/ &
  &  9.3811_JPRB,  9.3811_JPRB,  9.3810_JPRB,  9.3811_JPRB, 10.1420_JPRB, 10.4497_JPRB, &
  & 10.7929_JPRB, 11.2708_JPRB, 12.1002_JPRB, 12.6896_JPRB, 14.2126_JPRB, 20.3715_JPRB /)
!-- WVL=0.8000E-06
ZRALIS9( :,12, 2) = (/ &
  &  9.3043_JPRB,  9.3043_JPRB,  9.3043_JPRB,  9.3043_JPRB,  9.6542_JPRB,  9.8340_JPRB, &
  & 10.0364_JPRB, 10.3164_JPRB, 10.7896_JPRB, 11.0969_JPRB, 11.9012_JPRB, 14.5766_JPRB /)
!-- WVL=0.8580E-06
ZRALIS9( :,13, 2) = (/ &
  &  9.3221_JPRB,  9.3221_JPRB,  9.3221_JPRB,  9.3221_JPRB,  9.5270_JPRB,  9.6689_JPRB, &
  &  9.8318_JPRB, 10.0591_JPRB, 10.4434_JPRB, 10.6799_JPRB, 11.3249_JPRB, 13.3736_JPRB /)
!-- WVL=0.8650E-06
ZRALIS9( :,14, 2) = (/ &
  &  9.3261_JPRB,  9.3261_JPRB,  9.3261_JPRB,  9.3261_JPRB,  9.5142_JPRB,  9.6521_JPRB, &
  &  9.8108_JPRB, 10.0325_JPRB, 10.4077_JPRB, 10.6369_JPRB, 11.2661_JPRB, 13.2555_JPRB /)
!-- WVL=0.1020E-05
ZRALIS9( :,15, 2) = (/ &
  &  9.5012_JPRB,  9.5012_JPRB,  9.5012_JPRB,  9.5012_JPRB,  9.3383_JPRB,  9.4015_JPRB, &
  &  9.4866_JPRB,  9.6150_JPRB,  9.8418_JPRB,  9.9707_JPRB, 10.3395_JPRB, 11.4966_JPRB /)
!-- WVL=0.1064E-05
ZRALIS9( :,16, 2) = (/ &
  &  9.5792_JPRB,  9.5792_JPRB,  9.5793_JPRB,  9.5793_JPRB,  9.3173_JPRB,  9.3628_JPRB, &
  &  9.4313_JPRB,  9.5398_JPRB,  9.7368_JPRB,  9.8851_JPRB, 10.1651_JPRB, 11.1846_JPRB /)
!-- WVL=0.1240E-05
ZRALIS9( :,17, 2) = (/ &
  & 10.0096_JPRB, 10.0096_JPRB, 10.0097_JPRB, 10.0097_JPRB,  9.3241_JPRB,  9.3041_JPRB, &
  &  9.3148_JPRB,  9.3573_JPRB,  9.4610_JPRB,  9.5094_JPRB,  9.6788_JPRB, 10.3369_JPRB /)
!-- WVL=0.1640E-05
ZRALIS9( :,18, 2) = (/ &
  & 11.7175_JPRB, 11.7175_JPRB, 11.7178_JPRB, 11.7178_JPRB,  9.7406_JPRB,  9.5621_JPRB, &
  &  9.4469_JPRB,  9.3615_JPRB,  9.0671_JPRB,  9.1028_JPRB,  9.1906_JPRB,  9.4764_JPRB /)
!-- WVL=0.2130E-05
ZRALIS9( :,19, 2) = (/ &
  & 15.4240_JPRB, 15.4240_JPRB, 15.4247_JPRB, 15.4248_JPRB, 10.9059_JPRB, 10.4501_JPRB, &
  & 10.1289_JPRB,  9.8520_JPRB,  9.1073_JPRB,  9.0677_JPRB,  9.0488_JPRB,  9.1062_JPRB /)
  
 ! ** BIN No.            3   0.1000000000000000        0.2000000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS9( :, 1, 3) = (/ &
  & 0.86296_JPRB, 0.86296_JPRB, 0.86296_JPRB, 0.86296_JPRB, 4.26903_JPRB, 5.80672_JPRB, &
  & 7.44698_JPRB, 9.62028_JPRB,13.38761_JPRB,16.34210_JPRB,18.99175_JPRB,31.72890_JPRB /)
!-- WVL=0.3550E-06
ZALFSS9( :, 2, 3) = (/ &
  & 0.75236_JPRB, 0.75236_JPRB, 0.75236_JPRB, 0.75236_JPRB, 3.90393_JPRB, 5.34228_JPRB, &
  & 6.97457_JPRB, 9.09004_JPRB,12.69385_JPRB,15.79589_JPRB,18.19324_JPRB,31.06768_JPRB /)
!-- WVL=0.3800E-06
ZALFSS9( :, 3, 3) = (/ &
  & 0.60784_JPRB, 0.60784_JPRB, 0.60784_JPRB, 0.60784_JPRB, 3.41043_JPRB, 4.63917_JPRB, &
  & 6.14864_JPRB, 8.25186_JPRB,11.56927_JPRB,14.61642_JPRB,16.91602_JPRB,29.94401_JPRB /)
!-- WVL=0.4000E-06
ZALFSS9( :, 4, 3) = (/ &
  & 0.51976_JPRB, 0.51976_JPRB, 0.51976_JPRB, 0.51976_JPRB, 3.07415_JPRB, 4.18555_JPRB, &
  & 5.52803_JPRB, 7.54274_JPRB,10.81437_JPRB,13.62303_JPRB,15.78999_JPRB,28.71112_JPRB /)
!-- WVL=0.4400E-06
ZALFSS9( :, 5, 3) = (/ &
  & 0.38930_JPRB, 0.38930_JPRB, 0.38930_JPRB, 0.38930_JPRB, 2.45198_JPRB, 3.44862_JPRB, &
  & 4.56560_JPRB, 6.20665_JPRB, 9.28850_JPRB,11.98635_JPRB,13.66578_JPRB,26.05570_JPRB /)
!-- WVL=0.4690E-06
ZALFSS9( :, 6, 3) = (/ &
  & 0.31899_JPRB, 0.31899_JPRB, 0.31899_JPRB, 0.31899_JPRB, 2.04094_JPRB, 2.96211_JPRB, &
  & 4.00651_JPRB, 5.46371_JPRB, 8.16654_JPRB,10.78832_JPRB,12.13696_JPRB,24.18134_JPRB /)
!-- WVL=0.5000E-06
ZALFSS9( :, 7, 3) = (/ &
  & 0.25910_JPRB, 0.25910_JPRB, 0.25910_JPRB, 0.25910_JPRB, 1.67013_JPRB, 2.47660_JPRB, &
  & 3.44698_JPRB, 4.80336_JPRB, 7.16016_JPRB, 9.49370_JPRB,10.70552_JPRB,22.02704_JPRB /)
!-- WVL=0.5320E-06
ZALFSS9( :, 8, 3) = (/ &
  & 0.21000_JPRB, 0.21000_JPRB, 0.21000_JPRB, 0.21000_JPRB, 1.37256_JPRB, 2.04371_JPRB, &
  & 2.90475_JPRB, 4.17291_JPRB, 6.33464_JPRB, 7.86003_JPRB, 9.41008_JPRB,20.02123_JPRB /)
!-- WVL=0.5500E-06
ZALFSS9( :, 9, 3) = (/ &
  & 0.18699_JPRB, 0.18699_JPRB, 0.18699_JPRB, 0.18699_JPRB, 1.23798_JPRB, 1.83858_JPRB, &
  & 2.62664_JPRB, 3.83122_JPRB, 5.92043_JPRB, 6.67102_JPRB, 8.76849_JPRB,19.01800_JPRB /)
!-- WVL=0.6450E-06
ZALFSS9( :,10, 3) = (/ &
  & 0.10443_JPRB, 0.10443_JPRB, 0.10443_JPRB, 0.10443_JPRB, 0.76237_JPRB, 1.12500_JPRB, &
  & 1.59319_JPRB, 2.36106_JPRB, 3.94492_JPRB, 4.01446_JPRB, 6.25213_JPRB,14.09780_JPRB /)
!-- WVL=0.6700E-06
ZALFSS9( :,11, 3) = (/ &
  & 0.09041_JPRB, 0.09041_JPRB, 0.09041_JPRB, 0.09041_JPRB, 0.67684_JPRB, 1.00257_JPRB, &
  & 1.41930_JPRB, 2.09506_JPRB, 3.50716_JPRB, 3.59378_JPRB, 5.79067_JPRB,13.00470_JPRB /)
!-- WVL=0.8000E-06
ZALFSS9( :,12, 3) = (/ &
  & 0.04548_JPRB, 0.04548_JPRB, 0.04548_JPRB, 0.04548_JPRB, 0.37330_JPRB, 0.56876_JPRB, &
  & 0.82093_JPRB, 1.22019_JPRB, 2.01745_JPRB, 2.11036_JPRB, 3.52231_JPRB, 8.85363_JPRB /)
!-- WVL=0.8580E-06
ZALFSS9( :,13, 3) = (/ &
  & 0.03453_JPRB, 0.03453_JPRB, 0.03453_JPRB, 0.03453_JPRB, 0.29006_JPRB, 0.44653_JPRB, &
  & 0.65131_JPRB, 0.97905_JPRB, 1.63236_JPRB, 1.72040_JPRB, 2.85436_JPRB, 7.37146_JPRB /)
!-- WVL=0.8650E-06
ZALFSS9( :,14, 3) = (/ &
  & 0.03344_JPRB, 0.03344_JPRB, 0.03344_JPRB, 0.03344_JPRB, 0.28154_JPRB, 0.43388_JPRB, &
  & 0.63360_JPRB, 0.95374_JPRB, 1.59240_JPRB, 1.67999_JPRB, 2.78655_JPRB, 7.20515_JPRB /)
!-- WVL=0.1020E-05
ZALFSS9( :,15, 3) = (/ &
  & 0.01742_JPRB, 0.01742_JPRB, 0.01742_JPRB, 0.01742_JPRB, 0.15108_JPRB, 0.23666_JPRB, &
  & 0.35217_JPRB, 0.54358_JPRB, 0.93940_JPRB, 1.08119_JPRB, 1.71689_JPRB, 4.40713_JPRB /)
!-- WVL=0.1064E-05
ZALFSS9( :,16, 3) = (/ &
  & 0.01473_JPRB, 0.01473_JPRB, 0.01473_JPRB, 0.01473_JPRB, 0.12829_JPRB, 0.20150_JPRB, &
  & 0.30083_JPRB, 0.46660_JPRB, 0.81301_JPRB, 1.07147_JPRB, 1.51004_JPRB, 3.89059_JPRB /)
!-- WVL=0.1240E-05
ZALFSS9( :,17, 3) = (/ &
  & 0.00804_JPRB, 0.00804_JPRB, 0.00804_JPRB, 0.00804_JPRB, 0.07040_JPRB, 0.11123_JPRB, &
  & 0.16735_JPRB, 0.26273_JPRB, 0.46806_JPRB, 0.59589_JPRB, 0.91884_JPRB, 2.48494_JPRB /)
!-- WVL=0.1640E-05
ZALFSS9( :,18, 3) = (/ &
  & 0.00270_JPRB, 0.00270_JPRB, 0.00270_JPRB, 0.00270_JPRB, 0.02328_JPRB, 0.03684_JPRB, &
  & 0.05565_JPRB, 0.08801_JPRB, 0.12095_JPRB, 0.18219_JPRB, 0.33216_JPRB, 0.99223_JPRB /)
!-- WVL=0.2130E-05
ZALFSS9( :,19, 3) = (/ &
  & 0.00100_JPRB, 0.00100_JPRB, 0.00100_JPRB, 0.00100_JPRB, 0.00832_JPRB, 0.01313_JPRB, &
  & 0.01979_JPRB, 0.03126_JPRB, 0.04319_JPRB, 0.06534_JPRB, 0.12033_JPRB, 0.37415_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS9( :, 1, 3) = (/ &
  & 0.53761_JPRB, 0.53761_JPRB, 0.53761_JPRB, 0.53761_JPRB, 0.65149_JPRB, 0.67813_JPRB, &
  & 0.69291_JPRB, 0.70290_JPRB, 0.71878_JPRB, 0.72746_JPRB, 0.76630_JPRB, 0.77785_JPRB /)
!-- WVL=0.3550E-06
ZASYSS9( :, 2, 3) = (/ &
  & 0.50942_JPRB, 0.50942_JPRB, 0.50942_JPRB, 0.50942_JPRB, 0.63464_JPRB, 0.66430_JPRB, &
  & 0.68472_JPRB, 0.69756_JPRB, 0.71152_JPRB, 0.72272_JPRB, 0.76214_JPRB, 0.77759_JPRB /)
!-- WVL=0.3800E-06
ZASYSS9( :, 3, 3) = (/ &
  & 0.45509_JPRB, 0.45509_JPRB, 0.45509_JPRB, 0.45509_JPRB, 0.61221_JPRB, 0.63808_JPRB, &
  & 0.66470_JPRB, 0.68666_JPRB, 0.70145_JPRB, 0.71177_JPRB, 0.75299_JPRB, 0.77575_JPRB /)
!-- WVL=0.4000E-06
ZASYSS9( :, 4, 3) = (/ &
  & 0.41103_JPRB, 0.41103_JPRB, 0.41103_JPRB, 0.41103_JPRB, 0.59995_JPRB, 0.61984_JPRB, &
  & 0.64507_JPRB, 0.67327_JPRB, 0.69500_JPRB, 0.70381_JPRB, 0.74419_JPRB, 0.77229_JPRB /)
!-- WVL=0.4400E-06
ZASYSS9( :, 5, 3) = (/ &
  & 0.33389_JPRB, 0.33389_JPRB, 0.33389_JPRB, 0.33389_JPRB, 0.57996_JPRB, 0.59622_JPRB, &
  & 0.61211_JPRB, 0.63788_JPRB, 0.67466_JPRB, 0.69101_JPRB, 0.72759_JPRB, 0.76264_JPRB /)
!-- WVL=0.4690E-06
ZASYSS9( :, 6, 3) = (/ &
  & 0.28939_JPRB, 0.28939_JPRB, 0.28939_JPRB, 0.28939_JPRB, 0.55893_JPRB, 0.58314_JPRB, &
  & 0.59729_JPRB, 0.61584_JPRB, 0.65200_JPRB, 0.67635_JPRB, 0.71734_JPRB, 0.75303_JPRB /)
!-- WVL=0.5000E-06
ZASYSS9( :, 7, 3) = (/ &
  & 0.25101_JPRB, 0.25101_JPRB, 0.25101_JPRB, 0.25101_JPRB, 0.52438_JPRB, 0.56419_JPRB, &
  & 0.58430_JPRB, 0.59995_JPRB, 0.62748_JPRB, 0.65415_JPRB, 0.70115_JPRB, 0.74157_JPRB /)
!-- WVL=0.5320E-06
ZASYSS9( :, 8, 3) = (/ &
  & 0.21919_JPRB, 0.21919_JPRB, 0.21919_JPRB, 0.21919_JPRB, 0.47824_JPRB, 0.53382_JPRB, &
  & 0.56683_JPRB, 0.58750_JPRB, 0.60864_JPRB, 0.63266_JPRB, 0.67937_JPRB, 0.73078_JPRB /)
!-- WVL=0.5500E-06
ZASYSS9( :, 9, 3) = (/ &
  & 0.20403_JPRB, 0.20403_JPRB, 0.20403_JPRB, 0.20403_JPRB, 0.45036_JPRB, 0.51190_JPRB, &
  & 0.55307_JPRB, 0.57996_JPRB, 0.60090_JPRB, 0.62610_JPRB, 0.66655_JPRB, 0.72510_JPRB /)
!-- WVL=0.6450E-06
ZASYSS9( :,10, 3) = (/ &
  & 0.14609_JPRB, 0.14609_JPRB, 0.14609_JPRB, 0.14609_JPRB, 0.32050_JPRB, 0.38025_JPRB, &
  & 0.43912_JPRB, 0.50221_JPRB, 0.56073_JPRB, 0.58766_JPRB, 0.61750_JPRB, 0.68113_JPRB /)
!-- WVL=0.6700E-06
ZASYSS9( :,11, 3) = (/ &
  & 0.13514_JPRB, 0.13514_JPRB, 0.13514_JPRB, 0.13514_JPRB, 0.29430_JPRB, 0.34972_JPRB, &
  & 0.40643_JPRB, 0.47209_JPRB, 0.54258_JPRB, 0.57173_JPRB, 0.60755_JPRB, 0.66650_JPRB /)
!-- WVL=0.8000E-06
ZASYSS9( :,12, 3) = (/ &
  & 0.09447_JPRB, 0.09447_JPRB, 0.09447_JPRB, 0.09447_JPRB, 0.19950_JPRB, 0.23566_JPRB, &
  & 0.27454_JPRB, 0.32614_JPRB, 0.40592_JPRB, 0.44526_JPRB, 0.52751_JPRB, 0.61314_JPRB /)
!-- WVL=0.8580E-06
ZASYSS9( :,13, 3) = (/ &
  & 0.08216_JPRB, 0.08216_JPRB, 0.08216_JPRB, 0.08216_JPRB, 0.17206_JPRB, 0.20254_JPRB, &
  & 0.23523_JPRB, 0.27881_JPRB, 0.34861_JPRB, 0.38645_JPRB, 0.47294_JPRB, 0.59309_JPRB /)
!-- WVL=0.8650E-06
ZASYSS9( :,14, 3) = (/ &
  & 0.08084_JPRB, 0.08084_JPRB, 0.08084_JPRB, 0.08084_JPRB, 0.16916_JPRB, 0.19906_JPRB, &
  & 0.23109_JPRB, 0.27381_JPRB, 0.34236_JPRB, 0.37986_JPRB, 0.46613_JPRB, 0.59021_JPRB /)
!-- WVL=0.1020E-05
ZASYSS9( :,15, 3) = (/ &
  & 0.05827_JPRB, 0.05827_JPRB, 0.05827_JPRB, 0.05827_JPRB, 0.12057_JPRB, 0.14107_JPRB, &
  & 0.16272_JPRB, 0.19127_JPRB, 0.23718_JPRB, 0.26738_JPRB, 0.33216_JPRB, 0.49105_JPRB /)
!-- WVL=0.1064E-05
ZASYSS9( :,16, 3) = (/ &
  & 0.05360_JPRB, 0.05360_JPRB, 0.05360_JPRB, 0.05360_JPRB, 0.11071_JPRB, 0.12941_JPRB, &
  & 0.14910_JPRB, 0.17495_JPRB, 0.21637_JPRB, 0.24815_JPRB, 0.30280_JPRB, 0.45592_JPRB /)
!-- WVL=0.1240E-05
ZASYSS9( :,17, 3) = (/ &
  & 0.03958_JPRB, 0.03958_JPRB, 0.03958_JPRB, 0.03958_JPRB, 0.08150_JPRB, 0.09504_JPRB, &
  & 0.10920_JPRB, 0.12758_JPRB, 0.15657_JPRB, 0.17753_JPRB, 0.21720_JPRB, 0.33213_JPRB /)
!-- WVL=0.1640E-05
ZASYSS9( :,18, 3) = (/ &
  & 0.02273_JPRB, 0.02273_JPRB, 0.02273_JPRB, 0.02273_JPRB, 0.04680_JPRB, 0.05452_JPRB, &
  & 0.06253_JPRB, 0.07284_JPRB, 0.08613_JPRB, 0.09897_JPRB, 0.12170_JPRB, 0.18190_JPRB /)
!-- WVL=0.2130E-05
ZASYSS9( :,19, 3) = (/ &
  & 0.01352_JPRB, 0.01352_JPRB, 0.01352_JPRB, 0.01352_JPRB, 0.02787_JPRB, 0.03248_JPRB, &
  & 0.03725_JPRB, 0.04338_JPRB, 0.05123_JPRB, 0.05881_JPRB, 0.07212_JPRB, 0.10661_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS9( :, 1, 3) = (/ &
  & 0.99854_JPRB, 0.99854_JPRB, 0.99854_JPRB, 0.99854_JPRB, 0.99897_JPRB, 0.99901_JPRB, &
  & 0.99904_JPRB, 0.99905_JPRB, 0.99905_JPRB, 0.99904_JPRB, 0.99962_JPRB, 0.99965_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS9( :, 2, 3) = (/ &
  & 0.99845_JPRB, 0.99845_JPRB, 0.99845_JPRB, 0.99845_JPRB, 0.99895_JPRB, 0.99898_JPRB, &
  & 0.99902_JPRB, 0.99905_JPRB, 0.99905_JPRB, 0.99905_JPRB, 0.99962_JPRB, 0.99966_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS9( :, 3, 3) = (/ &
  & 0.99833_JPRB, 0.99833_JPRB, 0.99833_JPRB, 0.99833_JPRB, 0.99891_JPRB, 0.99895_JPRB, &
  & 0.99898_JPRB, 0.99903_JPRB, 0.99905_JPRB, 0.99905_JPRB, 0.99962_JPRB, 0.99967_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS9( :, 4, 3) = (/ &
  & 0.99822_JPRB, 0.99822_JPRB, 0.99822_JPRB, 0.99822_JPRB, 0.99886_JPRB, 0.99892_JPRB, &
  & 0.99896_JPRB, 0.99900_JPRB, 0.99904_JPRB, 0.99905_JPRB, 0.99962_JPRB, 0.99968_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS9( :, 5, 3) = (/ &
  & 0.99798_JPRB, 0.99798_JPRB, 0.99798_JPRB, 0.99798_JPRB, 0.99873_JPRB, 0.99884_JPRB, &
  & 0.99890_JPRB, 0.99895_JPRB, 0.99900_JPRB, 0.99904_JPRB, 0.99961_JPRB, 0.99968_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS9( :, 6, 3) = (/ &
  & 0.99777_JPRB, 0.99777_JPRB, 0.99777_JPRB, 0.99777_JPRB, 0.99862_JPRB, 0.99875_JPRB, &
  & 0.99884_JPRB, 0.99891_JPRB, 0.99897_JPRB, 0.99900_JPRB, 0.99964_JPRB, 0.99968_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS9( :, 7, 3) = (/ &
  & 0.99751_JPRB, 0.99751_JPRB, 0.99751_JPRB, 0.99751_JPRB, 0.99850_JPRB, 0.99864_JPRB, &
  & 0.99876_JPRB, 0.99886_JPRB, 0.99894_JPRB, 0.99897_JPRB, 0.99965_JPRB, 0.99968_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS9( :, 8, 3) = (/ &
  & 0.99720_JPRB, 0.99720_JPRB, 0.99720_JPRB, 0.99720_JPRB, 0.99838_JPRB, 0.99852_JPRB, &
  & 0.99865_JPRB, 0.99878_JPRB, 0.99889_JPRB, 0.99906_JPRB, 0.99963_JPRB, 0.99967_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS9( :, 9, 3) = (/ &
  & 0.99701_JPRB, 0.99701_JPRB, 0.99701_JPRB, 0.99701_JPRB, 0.99831_JPRB, 0.99846_JPRB, &
  & 0.99859_JPRB, 0.99873_JPRB, 0.99886_JPRB, 0.99925_JPRB, 0.99962_JPRB, 0.99967_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS9( :,10, 3) = (/ &
  & 0.99574_JPRB, 0.99574_JPRB, 0.99574_JPRB, 0.99574_JPRB, 0.99793_JPRB, 0.99814_JPRB, &
  & 0.99829_JPRB, 0.99844_JPRB, 0.99862_JPRB, 0.99942_JPRB, 0.99956_JPRB, 0.99963_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS9( :,11, 3) = (/ &
  & 0.99534_JPRB, 0.99534_JPRB, 0.99534_JPRB, 0.99534_JPRB, 0.99780_JPRB, 0.99804_JPRB, &
  & 0.99821_JPRB, 0.99836_JPRB, 0.99855_JPRB, 0.99939_JPRB, 0.99950_JPRB, 0.99962_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS9( :,12, 3) = (/ &
  & 0.99273_JPRB, 0.99273_JPRB, 0.99273_JPRB, 0.99273_JPRB, 0.99694_JPRB, 0.99738_JPRB, &
  & 0.99769_JPRB, 0.99795_JPRB, 0.99821_JPRB, 0.99924_JPRB, 0.99933_JPRB, 0.99956_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS9( :,13, 3) = (/ &
  & 0.99128_JPRB, 0.99128_JPRB, 0.99128_JPRB, 0.99128_JPRB, 0.99644_JPRB, 0.99699_JPRB, &
  & 0.99737_JPRB, 0.99771_JPRB, 0.99804_JPRB, 0.99917_JPRB, 0.99927_JPRB, 0.99952_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS9( :,14, 3) = (/ &
  & 0.99109_JPRB, 0.99109_JPRB, 0.99109_JPRB, 0.99109_JPRB, 0.99638_JPRB, 0.99694_JPRB, &
  & 0.99733_JPRB, 0.99768_JPRB, 0.99801_JPRB, 0.99916_JPRB, 0.99926_JPRB, 0.99951_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS9( :,15, 3) = (/ &
  & 0.98614_JPRB, 0.98614_JPRB, 0.98614_JPRB, 0.98614_JPRB, 0.99465_JPRB, 0.99557_JPRB, &
  & 0.99622_JPRB, 0.99681_JPRB, 0.99739_JPRB, 0.99860_JPRB, 0.99908_JPRB, 0.99939_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS9( :,16, 3) = (/ &
  & 0.98444_JPRB, 0.98444_JPRB, 0.98444_JPRB, 0.98444_JPRB, 0.99406_JPRB, 0.99509_JPRB, &
  & 0.99584_JPRB, 0.99651_JPRB, 0.99717_JPRB, 0.99790_JPRB, 0.99901_JPRB, 0.99936_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS9( :,17, 3) = (/ &
  & 0.97629_JPRB, 0.97629_JPRB, 0.97629_JPRB, 0.97629_JPRB, 0.99118_JPRB, 0.99279_JPRB, &
  & 0.99396_JPRB, 0.99501_JPRB, 0.99606_JPRB, 0.99746_JPRB, 0.99869_JPRB, 0.99921_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS9( :,18, 3) = (/ &
  & 0.94858_JPRB, 0.94858_JPRB, 0.94858_JPRB, 0.94858_JPRB, 0.98123_JPRB, 0.98481_JPRB, &
  & 0.98741_JPRB, 0.98976_JPRB, 0.99617_JPRB, 0.99680_JPRB, 0.99753_JPRB, 0.99867_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS9( :,19, 3) = (/ &
  & 0.89570_JPRB, 0.89570_JPRB, 0.89569_JPRB, 0.89569_JPRB, 0.96129_JPRB, 0.96874_JPRB, &
  & 0.97420_JPRB, 0.97914_JPRB, 0.99224_JPRB, 0.99359_JPRB, 0.99515_JPRB, 0.99752_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS9( :, 1, 3) = (/ &
  & 75.2504_JPRB, 75.2504_JPRB, 75.2504_JPRB, 75.2504_JPRB, 78.1091_JPRB, 91.5744_JPRB, &
  & 86.3934_JPRB, 69.8257_JPRB, 68.2882_JPRB, 69.7803_JPRB, 71.4989_JPRB, 63.1681_JPRB /)
!-- WVL=0.3550E-06
ZRALIS9( :, 2, 3) = (/ &
  & 62.7635_JPRB, 62.7635_JPRB, 62.7635_JPRB, 62.7635_JPRB, 70.9406_JPRB, 85.1905_JPRB, &
  & 91.9548_JPRB, 78.8898_JPRB, 65.6076_JPRB, 70.2832_JPRB, 80.1910_JPRB, 70.0668_JPRB /)
!-- WVL=0.3800E-06
ZRALIS9( :, 3, 3) = (/ &
  & 44.0668_JPRB, 44.0668_JPRB, 44.0668_JPRB, 44.0668_JPRB, 67.8190_JPRB, 72.1255_JPRB, &
  & 85.4134_JPRB, 91.3910_JPRB, 71.8985_JPRB, 65.6381_JPRB, 80.2886_JPRB, 68.3903_JPRB /)
!-- WVL=0.4000E-06
ZRALIS9( :, 4, 3) = (/ &
  & 34.2463_JPRB, 34.2463_JPRB, 34.2463_JPRB, 34.2463_JPRB, 71.0520_JPRB, 67.8493_JPRB, &
  & 74.9932_JPRB, 89.8358_JPRB, 83.3900_JPRB, 68.7535_JPRB, 74.1398_JPRB, 68.5086_JPRB /)
!-- WVL=0.4400E-06
ZRALIS9( :, 5, 3) = (/ &
  & 23.6895_JPRB, 23.6895_JPRB, 23.6895_JPRB, 23.6895_JPRB, 82.5088_JPRB, 73.0064_JPRB, &
  & 67.8278_JPRB, 72.0517_JPRB, 90.4161_JPRB, 88.5064_JPRB, 79.8954_JPRB, 83.7036_JPRB /)
!-- WVL=0.4690E-06
ZRALIS9( :, 6, 3) = (/ &
  & 19.7873_JPRB, 19.7873_JPRB, 19.7873_JPRB, 19.7873_JPRB, 83.1243_JPRB, 81.0791_JPRB, &
  & 72.4016_JPRB, 67.6743_JPRB, 78.3746_JPRB, 91.0407_JPRB, 93.8128_JPRB, 79.1135_JPRB /)
!-- WVL=0.5000E-06
ZRALIS9( :, 7, 3) = (/ &
  & 17.1909_JPRB, 17.1909_JPRB, 17.1909_JPRB, 17.1909_JPRB, 69.3131_JPRB, 84.1490_JPRB, &
  & 80.4632_JPRB, 71.0520_JPRB, 69.0119_JPRB, 79.5141_JPRB, 94.4577_JPRB, 73.5172_JPRB /)
!-- WVL=0.5320E-06
ZRALIS9( :, 8, 3) = (/ &
  & 15.4293_JPRB, 15.4293_JPRB, 15.4293_JPRB, 15.4293_JPRB, 50.9992_JPRB, 73.5645_JPRB, &
  & 84.4242_JPRB, 78.5748_JPRB, 68.2856_JPRB, 68.9417_JPRB, 81.6508_JPRB, 79.8344_JPRB /)
!-- WVL=0.5500E-06
ZRALIS9( :, 9, 3) = (/ &
  & 14.6899_JPRB, 14.6899_JPRB, 14.6899_JPRB, 14.6899_JPRB, 42.8201_JPRB, 63.8196_JPRB, &
  & 81.4080_JPRB, 82.5088_JPRB, 70.6232_JPRB, 67.9027_JPRB, 75.0184_JPRB, 86.8620_JPRB /)
!-- WVL=0.6450E-06
ZRALIS9( :,10, 3) = (/ &
  & 12.3283_JPRB, 12.3283_JPRB, 12.3283_JPRB, 12.3283_JPRB, 22.3927_JPRB, 29.2690_JPRB, &
  & 40.0595_JPRB, 59.7926_JPRB, 83.5390_JPRB, 86.8316_JPRB, 74.3580_JPRB, 82.6891_JPRB /)
!-- WVL=0.6700E-06
ZRALIS9( :,11, 3) = (/ &
  & 11.9509_JPRB, 11.9509_JPRB, 11.9509_JPRB, 11.9509_JPRB, 20.1655_JPRB, 25.3858_JPRB, &
  & 33.4200_JPRB, 49.0143_JPRB, 77.3911_JPRB, 86.2895_JPRB, 79.3755_JPRB, 74.9950_JPRB /)
!-- WVL=0.8000E-06
ZRALIS9( :,12, 3) = (/ &
  & 10.7087_JPRB, 10.7087_JPRB, 10.7087_JPRB, 10.7087_JPRB, 14.4800_JPRB, 16.3027_JPRB, &
  & 18.7125_JPRB, 22.9245_JPRB, 33.3305_JPRB, 41.4864_JPRB, 69.5873_JPRB, 76.9048_JPRB /)
!-- WVL=0.8580E-06
ZRALIS9( :,13, 3) = (/ &
  & 10.3789_JPRB, 10.3789_JPRB, 10.3789_JPRB, 10.3789_JPRB, 13.3048_JPRB, 14.6204_JPRB, &
  & 16.2791_JPRB, 19.0120_JPRB, 25.2608_JPRB, 30.2661_JPRB, 49.0241_JPRB, 86.3741_JPRB /)
!-- WVL=0.8650E-06
ZRALIS9( :,14, 3) = (/ &
  & 10.3449_JPRB, 10.3449_JPRB, 10.3449_JPRB, 10.3449_JPRB, 13.1896_JPRB, 14.4595_JPRB, &
  & 16.0529_JPRB, 18.6619_JPRB, 24.5733_JPRB, 29.3076_JPRB, 47.0021_JPRB, 87.0122_JPRB /)
!-- WVL=0.1020E-05
ZRALIS9( :,15, 3) = (/ &
  &  9.8063_JPRB,  9.8063_JPRB,  9.8063_JPRB,  9.8063_JPRB, 11.4782_JPRB, 12.1528_JPRB, &
  & 12.9397_JPRB, 14.1103_JPRB, 16.3878_JPRB, 18.2733_JPRB, 23.6050_JPRB, 54.9792_JPRB /)
!-- WVL=0.1064E-05
ZRALIS9( :,16, 3) = (/ &
  &  9.7068_JPRB,  9.7068_JPRB,  9.7069_JPRB,  9.7069_JPRB, 11.1761_JPRB, 11.7611_JPRB, &
  & 12.4352_JPRB, 13.4214_JPRB, 15.2874_JPRB, 17.0371_JPRB, 20.9231_JPRB, 44.1843_JPRB /)
!-- WVL=0.1240E-05
ZRALIS9( :,17, 3) = (/ &
  &  9.4488_JPRB,  9.4488_JPRB,  9.4488_JPRB,  9.4488_JPRB, 10.3618_JPRB, 10.7245_JPRB, &
  & 11.1309_JPRB, 11.7016_JPRB, 12.7078_JPRB, 13.5363_JPRB, 15.3614_JPRB, 23.6112_JPRB /)
!-- WVL=0.1640E-05
ZRALIS9( :,18, 3) = (/ &
  &  9.3289_JPRB,  9.3289_JPRB,  9.3289_JPRB,  9.3289_JPRB,  9.5728_JPRB,  9.7260_JPRB, &
  &  9.9009_JPRB, 10.1442_JPRB, 10.4512_JPRB, 10.8118_JPRB, 11.5039_JPRB, 13.7259_JPRB /)
!-- WVL=0.2130E-05
ZRALIS9( :,19, 3) = (/ &
  &  9.6610_JPRB,  9.6610_JPRB,  9.6611_JPRB,  9.6611_JPRB,  9.3222_JPRB,  9.3561_JPRB, &
  &  9.4144_JPRB,  9.5113_JPRB,  9.5808_JPRB,  9.7550_JPRB, 10.0819_JPRB, 11.0339_JPRB /)
  
 ! ** BIN No.            4   0.2000000000000000        0.5000000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS9( :, 1, 4) = (/ &
  & 1.60328_JPRB, 1.60328_JPRB, 1.60328_JPRB, 1.60328_JPRB, 3.39003_JPRB, 3.74511_JPRB, &
  & 4.07947_JPRB, 4.47759_JPRB, 5.09460_JPRB, 5.67315_JPRB, 6.95934_JPRB, 9.71628_JPRB /)
!-- WVL=0.3550E-06
ZALFSS9( :, 2, 4) = (/ &
  & 1.55397_JPRB, 1.55397_JPRB, 1.55397_JPRB, 1.55397_JPRB, 3.45726_JPRB, 3.87127_JPRB, &
  & 4.22173_JPRB, 4.62610_JPRB, 5.23607_JPRB, 5.76891_JPRB, 7.11709_JPRB, 9.84072_JPRB /)
!-- WVL=0.3800E-06
ZALFSS9( :, 3, 4) = (/ &
  & 1.45345_JPRB, 1.45345_JPRB, 1.45345_JPRB, 1.45345_JPRB, 3.53856_JPRB, 4.01757_JPRB, &
  & 4.44203_JPRB, 4.89049_JPRB, 5.51406_JPRB, 6.01096_JPRB, 7.44110_JPRB,10.01594_JPRB /)
!-- WVL=0.4000E-06
ZALFSS9( :, 4, 4) = (/ &
  & 1.37629_JPRB, 1.37629_JPRB, 1.37629_JPRB, 1.37629_JPRB, 3.54463_JPRB, 4.10305_JPRB, &
  & 4.57866_JPRB, 5.07940_JPRB, 5.73807_JPRB, 6.25010_JPRB, 7.75885_JPRB,10.19911_JPRB /)
!-- WVL=0.4400E-06
ZALFSS9( :, 5, 4) = (/ &
  & 1.21537_JPRB, 1.21537_JPRB, 1.21537_JPRB, 1.21537_JPRB, 3.50073_JPRB, 4.13444_JPRB, &
  & 4.74093_JPRB, 5.38200_JPRB, 6.17965_JPRB, 6.74037_JPRB, 8.37922_JPRB,10.76197_JPRB /)
!-- WVL=0.4690E-06
ZALFSS9( :, 6, 4) = (/ &
  & 1.09819_JPRB, 1.09819_JPRB, 1.09819_JPRB, 1.09819_JPRB, 3.39844_JPRB, 4.10158_JPRB, &
  & 4.74802_JPRB, 5.51504_JPRB, 6.46271_JPRB, 7.07030_JPRB, 8.80933_JPRB,11.26804_JPRB /)
!-- WVL=0.5000E-06
ZALFSS9( :, 7, 4) = (/ &
  & 0.99026_JPRB, 0.99026_JPRB, 0.99026_JPRB, 0.99026_JPRB, 3.27457_JPRB, 3.99035_JPRB, &
  & 4.71663_JPRB, 5.53848_JPRB, 6.65297_JPRB, 7.40362_JPRB, 9.10227_JPRB,11.86644_JPRB /)
!-- WVL=0.5320E-06
ZALFSS9( :, 8, 4) = (/ &
  & 0.88774_JPRB, 0.88774_JPRB, 0.88774_JPRB, 0.88774_JPRB, 3.09942_JPRB, 3.85955_JPRB, &
  & 4.59851_JPRB, 5.51860_JPRB, 6.76167_JPRB, 7.66926_JPRB, 9.30329_JPRB,12.47825_JPRB /)
!-- WVL=0.5500E-06
ZALFSS9( :, 9, 4) = (/ &
  & 0.83151_JPRB, 0.83151_JPRB, 0.83151_JPRB, 0.83151_JPRB, 2.99374_JPRB, 3.76681_JPRB, &
  & 4.52325_JPRB, 5.46989_JPRB, 6.76296_JPRB, 7.70360_JPRB, 9.33279_JPRB,12.75885_JPRB /)
!-- WVL=0.6450E-06
ZALFSS9( :,10, 4) = (/ &
  & 0.58583_JPRB, 0.58583_JPRB, 0.58583_JPRB, 0.58583_JPRB, 2.44761_JPRB, 3.18726_JPRB, &
  & 3.95594_JPRB, 4.98780_JPRB, 6.49720_JPRB, 7.00729_JPRB, 9.05639_JPRB,13.78957_JPRB /)
!-- WVL=0.6700E-06
ZALFSS9( :,11, 4) = (/ &
  & 0.53783_JPRB, 0.53783_JPRB, 0.53783_JPRB, 0.53783_JPRB, 2.30293_JPRB, 3.02905_JPRB, &
  & 3.79715_JPRB, 4.80564_JPRB, 6.37009_JPRB, 6.79839_JPRB, 8.94219_JPRB,13.84581_JPRB /)
!-- WVL=0.8000E-06
ZALFSS9( :,12, 4) = (/ &
  & 0.33736_JPRB, 0.33736_JPRB, 0.33736_JPRB, 0.33736_JPRB, 1.68579_JPRB, 2.27946_JPRB, &
  & 2.93485_JPRB, 3.87130_JPRB, 5.40354_JPRB, 5.60409_JPRB, 7.79969_JPRB,13.31811_JPRB /)
!-- WVL=0.8580E-06
ZALFSS9( :,13, 4) = (/ &
  & 0.27114_JPRB, 0.27114_JPRB, 0.27114_JPRB, 0.27114_JPRB, 1.44686_JPRB, 1.99572_JPRB, &
  & 2.61382_JPRB, 3.46115_JPRB, 4.94316_JPRB, 5.09034_JPRB, 7.17662_JPRB,12.74831_JPRB /)
!-- WVL=0.8650E-06
ZALFSS9( :,14, 4) = (/ &
  & 0.26422_JPRB, 0.26422_JPRB, 0.26422_JPRB, 0.26422_JPRB, 1.42080_JPRB, 1.96238_JPRB, &
  & 2.57657_JPRB, 3.41632_JPRB, 4.88501_JPRB, 5.02854_JPRB, 7.10302_JPRB,12.66667_JPRB /)
!-- WVL=0.1020E-05
ZALFSS9( :,15, 4) = (/ &
  & 0.15723_JPRB, 0.15723_JPRB, 0.15723_JPRB, 0.15723_JPRB, 0.97595_JPRB, 1.36305_JPRB, &
  & 1.82593_JPRB, 2.52566_JPRB, 3.74673_JPRB, 4.00504_JPRB, 5.58274_JPRB,10.76735_JPRB /)
!-- WVL=0.1064E-05
ZALFSS9( :,16, 4) = (/ &
  & 0.13774_JPRB, 0.13774_JPRB, 0.13774_JPRB, 0.13774_JPRB, 0.87584_JPRB, 1.23775_JPRB, &
  & 1.66216_JPRB, 2.30128_JPRB, 3.46900_JPRB, 4.15091_JPRB, 5.18941_JPRB,10.19840_JPRB /)
!-- WVL=0.1240E-05
ZALFSS9( :,17, 4) = (/ &
  & 0.08351_JPRB, 0.08351_JPRB, 0.08351_JPRB, 0.08351_JPRB, 0.55453_JPRB, 0.82284_JPRB, &
  & 1.15202_JPRB, 1.63074_JPRB, 2.49472_JPRB, 2.88218_JPRB, 3.89822_JPRB, 8.10722_JPRB /)
!-- WVL=0.1640E-05
ZALFSS9( :,18, 4) = (/ &
  & 0.03016_JPRB, 0.03016_JPRB, 0.03016_JPRB, 0.03016_JPRB, 0.22927_JPRB, 0.34205_JPRB, &
  & 0.48758_JPRB, 0.72406_JPRB, 0.88253_JPRB, 1.25102_JPRB, 2.04561_JPRB, 4.68382_JPRB /)
!-- WVL=0.2130E-05
ZALFSS9( :,19, 4) = (/ &
  & 0.01088_JPRB, 0.01088_JPRB, 0.01088_JPRB, 0.01088_JPRB, 0.09206_JPRB, 0.14232_JPRB, &
  & 0.20865_JPRB, 0.31592_JPRB, 0.39825_JPRB, 0.56670_JPRB, 0.94926_JPRB, 2.46688_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS9( :, 1, 4) = (/ &
  & 0.71922_JPRB, 0.71922_JPRB, 0.71922_JPRB, 0.71922_JPRB, 0.69243_JPRB, 0.67601_JPRB, &
  & 0.65955_JPRB, 0.64329_JPRB, 0.62987_JPRB, 0.62850_JPRB, 0.67466_JPRB, 0.68677_JPRB /)
!-- WVL=0.3550E-06
ZASYSS9( :, 2, 4) = (/ &
  & 0.71673_JPRB, 0.71673_JPRB, 0.71673_JPRB, 0.71673_JPRB, 0.70194_JPRB, 0.68596_JPRB, &
  & 0.66901_JPRB, 0.65197_JPRB, 0.63395_JPRB, 0.62876_JPRB, 0.67841_JPRB, 0.68350_JPRB /)
!-- WVL=0.3800E-06
ZASYSS9( :, 3, 4) = (/ &
  & 0.71189_JPRB, 0.71189_JPRB, 0.71189_JPRB, 0.71189_JPRB, 0.71107_JPRB, 0.69989_JPRB, &
  & 0.68575_JPRB, 0.66691_JPRB, 0.64537_JPRB, 0.63376_JPRB, 0.68814_JPRB, 0.67908_JPRB /)
!-- WVL=0.4000E-06
ZASYSS9( :, 4, 4) = (/ &
  & 0.70727_JPRB, 0.70727_JPRB, 0.70727_JPRB, 0.70727_JPRB, 0.71612_JPRB, 0.70856_JPRB, &
  & 0.69577_JPRB, 0.68041_JPRB, 0.65671_JPRB, 0.64215_JPRB, 0.69740_JPRB, 0.67918_JPRB /)
!-- WVL=0.4400E-06
ZASYSS9( :, 5, 4) = (/ &
  & 0.69205_JPRB, 0.69205_JPRB, 0.69205_JPRB, 0.69205_JPRB, 0.72023_JPRB, 0.71793_JPRB, &
  & 0.71110_JPRB, 0.70002_JPRB, 0.67930_JPRB, 0.66187_JPRB, 0.71833_JPRB, 0.68640_JPRB /)
!-- WVL=0.4690E-06
ZASYSS9( :, 6, 4) = (/ &
  & 0.67981_JPRB, 0.67981_JPRB, 0.67981_JPRB, 0.67981_JPRB, 0.72029_JPRB, 0.72024_JPRB, &
  & 0.71745_JPRB, 0.70991_JPRB, 0.69218_JPRB, 0.67779_JPRB, 0.73726_JPRB, 0.69822_JPRB /)
!-- WVL=0.5000E-06
ZASYSS9( :, 7, 4) = (/ &
  & 0.66789_JPRB, 0.66789_JPRB, 0.66789_JPRB, 0.66789_JPRB, 0.71811_JPRB, 0.72028_JPRB, &
  & 0.72022_JPRB, 0.71612_JPRB, 0.70556_JPRB, 0.69111_JPRB, 0.75051_JPRB, 0.71182_JPRB /)
!-- WVL=0.5320E-06
ZASYSS9( :, 8, 4) = (/ &
  & 0.65366_JPRB, 0.65366_JPRB, 0.65366_JPRB, 0.65366_JPRB, 0.71386_JPRB, 0.71892_JPRB, &
  & 0.72024_JPRB, 0.72004_JPRB, 0.71224_JPRB, 0.71945_JPRB, 0.75828_JPRB, 0.72463_JPRB /)
!-- WVL=0.5500E-06
ZASYSS9( :, 9, 4) = (/ &
  & 0.64399_JPRB, 0.64399_JPRB, 0.64399_JPRB, 0.64399_JPRB, 0.71148_JPRB, 0.71696_JPRB, &
  & 0.72015_JPRB, 0.72023_JPRB, 0.71561_JPRB, 0.74281_JPRB, 0.76199_JPRB, 0.73219_JPRB /)
!-- WVL=0.6450E-06
ZASYSS9( :,10, 4) = (/ &
  & 0.58662_JPRB, 0.58662_JPRB, 0.58662_JPRB, 0.58662_JPRB, 0.68852_JPRB, 0.70234_JPRB, &
  & 0.71046_JPRB, 0.71605_JPRB, 0.72030_JPRB, 0.76121_JPRB, 0.76839_JPRB, 0.75782_JPRB /)
!-- WVL=0.6700E-06
ZASYSS9( :,11, 4) = (/ &
  & 0.57464_JPRB, 0.57464_JPRB, 0.57464_JPRB, 0.57464_JPRB, 0.68122_JPRB, 0.69593_JPRB, &
  & 0.70663_JPRB, 0.71333_JPRB, 0.71958_JPRB, 0.75896_JPRB, 0.76591_JPRB, 0.76201_JPRB /)
!-- WVL=0.8000E-06
ZASYSS9( :,12, 4) = (/ &
  & 0.51607_JPRB, 0.51607_JPRB, 0.51607_JPRB, 0.51607_JPRB, 0.64062_JPRB, 0.66184_JPRB, &
  & 0.67548_JPRB, 0.69003_JPRB, 0.70656_JPRB, 0.74138_JPRB, 0.75310_JPRB, 0.76871_JPRB /)
!-- WVL=0.8580E-06
ZASYSS9( :,13, 4) = (/ &
  & 0.47758_JPRB, 0.47758_JPRB, 0.47758_JPRB, 0.47758_JPRB, 0.61525_JPRB, 0.64290_JPRB, &
  & 0.66165_JPRB, 0.67675_JPRB, 0.69567_JPRB, 0.73038_JPRB, 0.74551_JPRB, 0.76564_JPRB /)
!-- WVL=0.8650E-06
ZASYSS9( :,14, 4) = (/ &
  & 0.47231_JPRB, 0.47231_JPRB, 0.47231_JPRB, 0.47231_JPRB, 0.61215_JPRB, 0.64027_JPRB, &
  & 0.65978_JPRB, 0.67527_JPRB, 0.69417_JPRB, 0.72889_JPRB, 0.74452_JPRB, 0.76512_JPRB /)
!-- WVL=0.1020E-05
ZASYSS9( :,15, 4) = (/ &
  & 0.34979_JPRB, 0.34979_JPRB, 0.34979_JPRB, 0.34979_JPRB, 0.55841_JPRB, 0.58110_JPRB, &
  & 0.60511_JPRB, 0.63388_JPRB, 0.66250_JPRB, 0.69092_JPRB, 0.71645_JPRB, 0.75103_JPRB /)
!-- WVL=0.1064E-05
ZASYSS9( :,16, 4) = (/ &
  & 0.31965_JPRB, 0.31965_JPRB, 0.31965_JPRB, 0.31965_JPRB, 0.54586_JPRB, 0.56838_JPRB, &
  & 0.58995_JPRB, 0.61828_JPRB, 0.65204_JPRB, 0.67436_JPRB, 0.70743_JPRB, 0.74580_JPRB /)
!-- WVL=0.1240E-05
ZASYSS9( :,17, 4) = (/ &
  & 0.22940_JPRB, 0.22940_JPRB, 0.22940_JPRB, 0.22940_JPRB, 0.47497_JPRB, 0.51744_JPRB, &
  & 0.54368_JPRB, 0.56637_JPRB, 0.59827_JPRB, 0.62858_JPRB, 0.66923_JPRB, 0.71869_JPRB /)
!-- WVL=0.1640E-05
ZASYSS9( :,18, 4) = (/ &
  & 0.12811_JPRB, 0.12811_JPRB, 0.12811_JPRB, 0.12811_JPRB, 0.27551_JPRB, 0.32562_JPRB, &
  & 0.37648_JPRB, 0.43531_JPRB, 0.48876_JPRB, 0.52864_JPRB, 0.57008_JPRB, 0.64016_JPRB /)
!-- WVL=0.2130E-05
ZASYSS9( :,19, 4) = (/ &
  & 0.07586_JPRB, 0.07586_JPRB, 0.07586_JPRB, 0.07586_JPRB, 0.15809_JPRB, 0.18560_JPRB, &
  & 0.21486_JPRB, 0.25353_JPRB, 0.30071_JPRB, 0.34792_JPRB, 0.42493_JPRB, 0.54598_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS9( :, 1, 4) = (/ &
  & 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99846_JPRB, 0.99824_JPRB, &
  & 0.99799_JPRB, 0.99767_JPRB, 0.99722_JPRB, 0.99690_JPRB, 0.99886_JPRB, 0.99877_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS9( :, 2, 4) = (/ &
  & 0.99899_JPRB, 0.99899_JPRB, 0.99899_JPRB, 0.99899_JPRB, 0.99857_JPRB, 0.99837_JPRB, &
  & 0.99814_JPRB, 0.99786_JPRB, 0.99742_JPRB, 0.99711_JPRB, 0.99894_JPRB, 0.99884_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS9( :, 3, 4) = (/ &
  & 0.99901_JPRB, 0.99901_JPRB, 0.99901_JPRB, 0.99901_JPRB, 0.99870_JPRB, 0.99855_JPRB, &
  & 0.99837_JPRB, 0.99811_JPRB, 0.99772_JPRB, 0.99741_JPRB, 0.99906_JPRB, 0.99894_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS9( :, 4, 4) = (/ &
  & 0.99902_JPRB, 0.99902_JPRB, 0.99902_JPRB, 0.99902_JPRB, 0.99879_JPRB, 0.99866_JPRB, &
  & 0.99850_JPRB, 0.99830_JPRB, 0.99794_JPRB, 0.99764_JPRB, 0.99914_JPRB, 0.99902_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS9( :, 5, 4) = (/ &
  & 0.99901_JPRB, 0.99901_JPRB, 0.99901_JPRB, 0.99901_JPRB, 0.99889_JPRB, 0.99881_JPRB, &
  & 0.99870_JPRB, 0.99855_JPRB, 0.99828_JPRB, 0.99803_JPRB, 0.99928_JPRB, 0.99916_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS9( :, 6, 4) = (/ &
  & 0.99900_JPRB, 0.99900_JPRB, 0.99900_JPRB, 0.99900_JPRB, 0.99895_JPRB, 0.99888_JPRB, &
  & 0.99881_JPRB, 0.99868_JPRB, 0.99846_JPRB, 0.99826_JPRB, 0.99945_JPRB, 0.99925_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS9( :, 7, 4) = (/ &
  & 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99894_JPRB, &
  & 0.99887_JPRB, 0.99879_JPRB, 0.99862_JPRB, 0.99844_JPRB, 0.99953_JPRB, 0.99934_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS9( :, 8, 4) = (/ &
  & 0.99895_JPRB, 0.99895_JPRB, 0.99895_JPRB, 0.99895_JPRB, 0.99900_JPRB, 0.99898_JPRB, &
  & 0.99893_JPRB, 0.99886_JPRB, 0.99872_JPRB, 0.99884_JPRB, 0.99957_JPRB, 0.99941_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS9( :, 9, 4) = (/ &
  & 0.99892_JPRB, 0.99892_JPRB, 0.99892_JPRB, 0.99892_JPRB, 0.99901_JPRB, 0.99899_JPRB, &
  & 0.99896_JPRB, 0.99889_JPRB, 0.99878_JPRB, 0.99921_JPRB, 0.99959_JPRB, 0.99944_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS9( :,10, 4) = (/ &
  & 0.99880_JPRB, 0.99880_JPRB, 0.99880_JPRB, 0.99880_JPRB, 0.99901_JPRB, 0.99902_JPRB, &
  & 0.99902_JPRB, 0.99899_JPRB, 0.99895_JPRB, 0.99960_JPRB, 0.99964_JPRB, 0.99957_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS9( :,11, 4) = (/ &
  & 0.99876_JPRB, 0.99876_JPRB, 0.99876_JPRB, 0.99876_JPRB, 0.99900_JPRB, 0.99901_JPRB, &
  & 0.99902_JPRB, 0.99900_JPRB, 0.99897_JPRB, 0.99960_JPRB, 0.99961_JPRB, 0.99959_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS9( :,12, 4) = (/ &
  & 0.99848_JPRB, 0.99848_JPRB, 0.99848_JPRB, 0.99848_JPRB, 0.99892_JPRB, 0.99897_JPRB, &
  & 0.99899_JPRB, 0.99901_JPRB, 0.99902_JPRB, 0.99961_JPRB, 0.99961_JPRB, 0.99965_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS9( :,13, 4) = (/ &
  & 0.99832_JPRB, 0.99832_JPRB, 0.99832_JPRB, 0.99832_JPRB, 0.99887_JPRB, 0.99892_JPRB, &
  & 0.99897_JPRB, 0.99899_JPRB, 0.99901_JPRB, 0.99960_JPRB, 0.99961_JPRB, 0.99966_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS9( :,14, 4) = (/ &
  & 0.99831_JPRB, 0.99831_JPRB, 0.99831_JPRB, 0.99831_JPRB, 0.99886_JPRB, 0.99892_JPRB, &
  & 0.99896_JPRB, 0.99899_JPRB, 0.99901_JPRB, 0.99960_JPRB, 0.99961_JPRB, 0.99966_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS9( :,15, 4) = (/ &
  & 0.99787_JPRB, 0.99787_JPRB, 0.99787_JPRB, 0.99787_JPRB, 0.99869_JPRB, 0.99879_JPRB, &
  & 0.99885_JPRB, 0.99890_JPRB, 0.99897_JPRB, 0.99944_JPRB, 0.99959_JPRB, 0.99967_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS9( :,16, 4) = (/ &
  & 0.99773_JPRB, 0.99773_JPRB, 0.99773_JPRB, 0.99773_JPRB, 0.99863_JPRB, 0.99874_JPRB, &
  & 0.99881_JPRB, 0.99887_JPRB, 0.99894_JPRB, 0.99917_JPRB, 0.99959_JPRB, 0.99967_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS9( :,17, 4) = (/ &
  & 0.99704_JPRB, 0.99704_JPRB, 0.99704_JPRB, 0.99704_JPRB, 0.99832_JPRB, 0.99848_JPRB, &
  & 0.99862_JPRB, 0.99873_JPRB, 0.99883_JPRB, 0.99920_JPRB, 0.99955_JPRB, 0.99966_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS9( :,18, 4) = (/ &
  & 0.99448_JPRB, 0.99448_JPRB, 0.99448_JPRB, 0.99448_JPRB, 0.99747_JPRB, 0.99776_JPRB, &
  & 0.99798_JPRB, 0.99818_JPRB, 0.99926_JPRB, 0.99933_JPRB, 0.99941_JPRB, 0.99959_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS9( :,19, 4) = (/ &
  & 0.98916_JPRB, 0.98916_JPRB, 0.98916_JPRB, 0.98916_JPRB, 0.99568_JPRB, 0.99636_JPRB, &
  & 0.99685_JPRB, 0.99729_JPRB, 0.99891_JPRB, 0.99903_JPRB, 0.99916_JPRB, 0.99946_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS9( :, 1, 4) = (/ &
  & 55.4743_JPRB, 55.4743_JPRB, 55.4742_JPRB, 55.4743_JPRB, 23.8407_JPRB, 19.6041_JPRB, &
  & 15.9146_JPRB, 12.5403_JPRB, 10.1029_JPRB,  8.8321_JPRB, 21.4009_JPRB, 20.2310_JPRB /)
!-- WVL=0.3550E-06
ZRALIS9( :, 2, 4) = (/ &
  & 56.7865_JPRB, 56.7865_JPRB, 56.7865_JPRB, 56.7865_JPRB, 27.8046_JPRB, 22.2036_JPRB, &
  & 17.2967_JPRB, 14.7011_JPRB, 11.1400_JPRB,  9.6494_JPRB, 23.2812_JPRB, 20.9534_JPRB /)
!-- WVL=0.3800E-06
ZRALIS9( :, 3, 4) = (/ &
  & 66.2323_JPRB, 66.2323_JPRB, 66.2323_JPRB, 66.2323_JPRB, 33.0599_JPRB, 26.8826_JPRB, &
  & 22.1575_JPRB, 16.8266_JPRB, 12.8625_JPRB, 11.0835_JPRB, 25.7229_JPRB, 22.1941_JPRB /)
!-- WVL=0.4000E-06
ZRALIS9( :, 4, 4) = (/ &
  & 73.4216_JPRB, 73.4215_JPRB, 73.4215_JPRB, 73.4215_JPRB, 38.0901_JPRB, 31.0806_JPRB, &
  & 25.0654_JPRB, 20.9218_JPRB, 15.6099_JPRB, 12.4173_JPRB, 27.5213_JPRB, 24.0973_JPRB /)
!-- WVL=0.4400E-06
ZRALIS9( :, 5, 4) = (/ &
  & 71.9412_JPRB, 71.9412_JPRB, 71.9412_JPRB, 71.9412_JPRB, 45.1651_JPRB, 39.4394_JPRB, &
  & 33.0923_JPRB, 26.9382_JPRB, 20.6221_JPRB, 16.1350_JPRB, 32.6617_JPRB, 26.1107_JPRB /)
!-- WVL=0.4690E-06
ZRALIS9( :, 6, 4) = (/ &
  & 73.8190_JPRB, 73.8189_JPRB, 73.8189_JPRB, 73.8189_JPRB, 53.8527_JPRB, 43.6855_JPRB, &
  & 39.0887_JPRB, 32.0330_JPRB, 23.7603_JPRB, 20.1784_JPRB, 39.7074_JPRB, 29.3920_JPRB /)
!-- WVL=0.5000E-06
ZRALIS9( :, 7, 4) = (/ &
  & 80.8169_JPRB, 80.8169_JPRB, 80.8169_JPRB, 80.8169_JPRB, 55.9036_JPRB, 52.5705_JPRB, &
  & 43.1976_JPRB, 38.0901_JPRB, 29.4655_JPRB, 23.4429_JPRB, 46.0723_JPRB, 33.5354_JPRB /)
!-- WVL=0.5320E-06
ZRALIS9( :, 8, 4) = (/ &
  & 83.1688_JPRB, 83.1688_JPRB, 83.1688_JPRB, 83.1688_JPRB, 61.1468_JPRB, 55.5857_JPRB, &
  & 51.6592_JPRB, 42.0199_JPRB, 34.2971_JPRB, 34.0480_JPRB, 49.9964_JPRB, 35.5985_JPRB /)
!-- WVL=0.5500E-06
ZRALIS9( :, 9, 4) = (/ &
  & 80.8391_JPRB, 80.8391_JPRB, 80.8391_JPRB, 80.8391_JPRB, 67.2853_JPRB, 56.5951_JPRB, &
  & 54.6638_JPRB, 45.1651_JPRB, 37.6844_JPRB, 45.8948_JPRB, 52.2339_JPRB, 39.5027_JPRB /)
!-- WVL=0.6450E-06
ZRALIS9( :,10, 4) = (/ &
  & 61.9808_JPRB, 61.9808_JPRB, 61.9808_JPRB, 61.9808_JPRB, 71.6696_JPRB, 74.1607_JPRB, &
  & 69.6010_JPRB, 57.4629_JPRB, 53.4854_JPRB, 68.0123_JPRB, 62.9687_JPRB, 49.7963_JPRB /)
!-- WVL=0.6700E-06
ZRALIS9( :,11, 4) = (/ &
  & 60.7410_JPRB, 60.7410_JPRB, 60.7410_JPRB, 60.7410_JPRB, 73.1816_JPRB, 72.7344_JPRB, &
  & 73.7433_JPRB, 62.4168_JPRB, 55.3102_JPRB, 70.7422_JPRB, 63.0990_JPRB, 52.2477_JPRB /)
!-- WVL=0.8000E-06
ZRALIS9( :,12, 4) = (/ &
  & 54.8885_JPRB, 54.8885_JPRB, 54.8885_JPRB, 54.8885_JPRB, 79.6400_JPRB, 82.9126_JPRB, &
  & 76.2982_JPRB, 71.7190_JPRB, 73.7738_JPRB, 81.1270_JPRB, 70.2747_JPRB, 63.7319_JPRB /)
!-- WVL=0.8580E-06
ZRALIS9( :,13, 4) = (/ &
  & 45.7911_JPRB, 45.7911_JPRB, 45.7911_JPRB, 45.7911_JPRB, 69.4286_JPRB, 80.4688_JPRB, &
  & 82.9498_JPRB, 75.5147_JPRB, 72.6712_JPRB, 81.8807_JPRB, 76.6843_JPRB, 68.7921_JPRB /)
!-- WVL=0.8650E-06
ZRALIS9( :,14, 4) = (/ &
  & 44.5738_JPRB, 44.5738_JPRB, 44.5738_JPRB, 44.5738_JPRB, 68.3259_JPRB, 79.5095_JPRB, &
  & 83.2345_JPRB, 76.4368_JPRB, 72.3288_JPRB, 81.3376_JPRB, 77.9016_JPRB, 69.6805_JPRB /)
!-- WVL=0.1020E-05
ZRALIS9( :,15, 4) = (/ &
  & 24.8398_JPRB, 24.8398_JPRB, 24.8398_JPRB, 24.8398_JPRB, 60.1761_JPRB, 61.2841_JPRB, &
  & 66.0699_JPRB, 76.9529_JPRB, 82.7684_JPRB, 82.6021_JPRB, 77.7371_JPRB, 75.0430_JPRB /)
!-- WVL=0.1064E-05
ZRALIS9( :,16, 4) = (/ &
  & 21.9707_JPRB, 21.9707_JPRB, 21.9708_JPRB, 21.9708_JPRB, 59.5852_JPRB, 60.4389_JPRB, &
  & 62.5139_JPRB, 70.5649_JPRB, 82.9352_JPRB, 83.0369_JPRB, 78.7727_JPRB, 80.9305_JPRB /)
!-- WVL=0.1240E-05
ZRALIS9( :,17, 4) = (/ &
  & 15.8837_JPRB, 15.8837_JPRB, 15.8837_JPRB, 15.8837_JPRB, 45.1838_JPRB, 55.1859_JPRB, &
  & 59.4036_JPRB, 60.3742_JPRB, 64.2368_JPRB, 70.6926_JPRB, 84.7030_JPRB, 78.3877_JPRB /)
!-- WVL=0.1640E-05
ZRALIS9( :,18, 4) = (/ &
  & 11.7126_JPRB, 11.7126_JPRB, 11.7127_JPRB, 11.7127_JPRB, 18.6102_JPRB, 22.4974_JPRB, &
  & 27.8880_JPRB, 36.8913_JPRB, 47.8758_JPRB, 56.7568_JPRB, 61.2903_JPRB, 72.1438_JPRB /)
!-- WVL=0.2130E-05
ZRALIS9( :,19, 4) = (/ &
  & 10.2273_JPRB, 10.2273_JPRB, 10.2273_JPRB, 10.2273_JPRB, 12.7466_JPRB, 13.8306_JPRB, &
  & 15.1519_JPRB, 17.2262_JPRB, 20.4830_JPRB, 24.7122_JPRB, 34.9684_JPRB, 59.6386_JPRB /)
  
 ! ** BIN No.            5   0.5000000000000000         1.000000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS9( :, 1, 5) = (/ &
  & 0.59821_JPRB, 0.59821_JPRB, 0.59821_JPRB, 0.59821_JPRB, 1.08588_JPRB, 1.28519_JPRB, &
  & 1.50921_JPRB, 1.81838_JPRB, 2.24308_JPRB, 2.50754_JPRB, 3.09801_JPRB, 4.23479_JPRB /)
!-- WVL=0.3550E-06
ZALFSS9( :, 2, 5) = (/ &
  & 0.62282_JPRB, 0.62282_JPRB, 0.62282_JPRB, 0.62282_JPRB, 1.09079_JPRB, 1.27031_JPRB, &
  & 1.48346_JPRB, 1.78999_JPRB, 2.25145_JPRB, 2.56090_JPRB, 3.13305_JPRB, 4.28933_JPRB /)
!-- WVL=0.3800E-06
ZALFSS9( :, 3, 5) = (/ &
  & 0.67403_JPRB, 0.67403_JPRB, 0.67403_JPRB, 0.67403_JPRB, 1.09388_JPRB, 1.27278_JPRB, &
  & 1.45907_JPRB, 1.73411_JPRB, 2.21153_JPRB, 2.58753_JPRB, 3.11040_JPRB, 4.43298_JPRB /)
!-- WVL=0.4000E-06
ZALFSS9( :, 4, 5) = (/ &
  & 0.71029_JPRB, 0.71029_JPRB, 0.71029_JPRB, 0.71029_JPRB, 1.11207_JPRB, 1.27307_JPRB, &
  & 1.45737_JPRB, 1.71281_JPRB, 2.16468_JPRB, 2.55779_JPRB, 3.04566_JPRB, 4.55825_JPRB /)
!-- WVL=0.4400E-06
ZALFSS9( :, 5, 5) = (/ &
  & 0.78093_JPRB, 0.78093_JPRB, 0.78093_JPRB, 0.78093_JPRB, 1.14757_JPRB, 1.30287_JPRB, &
  & 1.46563_JPRB, 1.70473_JPRB, 2.09399_JPRB, 2.45545_JPRB, 2.93147_JPRB, 4.63361_JPRB /)
!-- WVL=0.4690E-06
ZALFSS9( :, 6, 5) = (/ &
  & 0.82620_JPRB, 0.82620_JPRB, 0.82620_JPRB, 0.82620_JPRB, 1.20060_JPRB, 1.32770_JPRB, &
  & 1.49394_JPRB, 1.70642_JPRB, 2.07131_JPRB, 2.41143_JPRB, 2.88248_JPRB, 4.55874_JPRB /)
!-- WVL=0.5000E-06
ZALFSS9( :, 7, 5) = (/ &
  & 0.85299_JPRB, 0.85299_JPRB, 0.85299_JPRB, 0.85299_JPRB, 1.26261_JPRB, 1.38778_JPRB, &
  & 1.52062_JPRB, 1.73761_JPRB, 2.08200_JPRB, 2.37935_JPRB, 2.89181_JPRB, 4.43916_JPRB /)
!-- WVL=0.5320E-06
ZALFSS9( :, 8, 5) = (/ &
  & 0.86998_JPRB, 0.86998_JPRB, 0.86998_JPRB, 0.86998_JPRB, 1.35182_JPRB, 1.45393_JPRB, &
  & 1.58617_JPRB, 1.76450_JPRB, 2.09585_JPRB, 2.39128_JPRB, 2.90062_JPRB, 4.33123_JPRB /)
!-- WVL=0.5500E-06
ZALFSS9( :, 9, 5) = (/ &
  & 0.87320_JPRB, 0.87320_JPRB, 0.87320_JPRB, 0.87320_JPRB, 1.40489_JPRB, 1.49969_JPRB, &
  & 1.62403_JPRB, 1.79307_JPRB, 2.11946_JPRB, 2.40874_JPRB, 2.92929_JPRB, 4.30862_JPRB /)
!-- WVL=0.6450E-06
ZALFSS9( :,10, 5) = (/ &
  & 0.84510_JPRB, 0.84510_JPRB, 0.84510_JPRB, 0.84510_JPRB, 1.64758_JPRB, 1.78318_JPRB, &
  & 1.90875_JPRB, 2.03780_JPRB, 2.28315_JPRB, 2.74246_JPRB, 3.12422_JPRB, 4.29874_JPRB /)
!-- WVL=0.6700E-06
ZALFSS9( :,11, 5) = (/ &
  & 0.82635_JPRB, 0.82635_JPRB, 0.82635_JPRB, 0.82635_JPRB, 1.70270_JPRB, 1.85102_JPRB, &
  & 1.98265_JPRB, 2.13185_JPRB, 2.34667_JPRB, 2.85202_JPRB, 3.18875_JPRB, 4.34596_JPRB /)
!-- WVL=0.8000E-06
ZALFSS9( :,12, 5) = (/ &
  & 0.70227_JPRB, 0.70227_JPRB, 0.70227_JPRB, 0.70227_JPRB, 1.81039_JPRB, 2.08493_JPRB, &
  & 2.32487_JPRB, 2.55450_JPRB, 2.82509_JPRB, 3.43123_JPRB, 3.78423_JPRB, 4.68830_JPRB /)
!-- WVL=0.8580E-06
ZALFSS9( :,13, 5) = (/ &
  & 0.64593_JPRB, 0.64593_JPRB, 0.64593_JPRB, 0.64593_JPRB, 1.80986_JPRB, 2.11101_JPRB, &
  & 2.39488_JPRB, 2.70178_JPRB, 3.03094_JPRB, 3.62958_JPRB, 4.08824_JPRB, 4.98131_JPRB /)
!-- WVL=0.8650E-06
ZALFSS9( :,14, 5) = (/ &
  & 0.63872_JPRB, 0.63872_JPRB, 0.63872_JPRB, 0.63872_JPRB, 1.80679_JPRB, 2.11224_JPRB, &
  & 2.40090_JPRB, 2.71307_JPRB, 3.05386_JPRB, 3.64984_JPRB, 4.12156_JPRB, 5.02647_JPRB /)
!-- WVL=0.1020E-05
ZALFSS9( :,15, 5) = (/ &
  & 0.48926_JPRB, 0.48926_JPRB, 0.48926_JPRB, 0.48926_JPRB, 1.64172_JPRB, 2.02287_JPRB, &
  & 2.40699_JPRB, 2.83305_JPRB, 3.40599_JPRB, 3.88956_JPRB, 4.65069_JPRB, 5.98616_JPRB /)
!-- WVL=0.1064E-05
ZALFSS9( :,16, 5) = (/ &
  & 0.45215_JPRB, 0.45215_JPRB, 0.45215_JPRB, 0.45215_JPRB, 1.57724_JPRB, 1.96887_JPRB, &
  & 2.35859_JPRB, 2.83140_JPRB, 3.44195_JPRB, 3.90989_JPRB, 4.73540_JPRB, 6.24348_JPRB /)
!-- WVL=0.1240E-05
ZALFSS9( :,17, 5) = (/ &
  & 0.32307_JPRB, 0.32307_JPRB, 0.32307_JPRB, 0.32307_JPRB, 1.33079_JPRB, 1.70278_JPRB, &
  & 2.09844_JPRB, 2.62349_JPRB, 3.40052_JPRB, 3.85429_JPRB, 4.72440_JPRB, 6.92378_JPRB /)
!-- WVL=0.1640E-05
ZALFSS9( :,18, 5) = (/ &
  & 0.15635_JPRB, 0.15635_JPRB, 0.15635_JPRB, 0.15635_JPRB, 0.81143_JPRB, 1.11123_JPRB, &
  & 1.44409_JPRB, 1.91020_JPRB, 2.17908_JPRB, 2.76360_JPRB, 3.85628_JPRB, 6.72868_JPRB /)
!-- WVL=0.2130E-05
ZALFSS9( :,19, 5) = (/ &
  & 0.06851_JPRB, 0.06851_JPRB, 0.06851_JPRB, 0.06851_JPRB, 0.43697_JPRB, 0.61984_JPRB, &
  & 0.83617_JPRB, 1.16285_JPRB, 1.33988_JPRB, 1.78172_JPRB, 2.64665_JPRB, 5.18193_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS9( :, 1, 5) = (/ &
  & 0.60845_JPRB, 0.60845_JPRB, 0.60845_JPRB, 0.60845_JPRB, 0.65385_JPRB, 0.67600_JPRB, &
  & 0.70089_JPRB, 0.72601_JPRB, 0.74360_JPRB, 0.74520_JPRB, 0.77113_JPRB, 0.77207_JPRB /)
!-- WVL=0.3550E-06
ZASYSS9( :, 2, 5) = (/ &
  & 0.61759_JPRB, 0.61759_JPRB, 0.61759_JPRB, 0.61759_JPRB, 0.63933_JPRB, 0.66225_JPRB, &
  & 0.68680_JPRB, 0.71230_JPRB, 0.73894_JPRB, 0.74441_JPRB, 0.76698_JPRB, 0.77147_JPRB /)
!-- WVL=0.3800E-06
ZASYSS9( :, 3, 5) = (/ &
  & 0.63680_JPRB, 0.63680_JPRB, 0.63680_JPRB, 0.63680_JPRB, 0.62362_JPRB, 0.64216_JPRB, &
  & 0.66256_JPRB, 0.68996_JPRB, 0.72304_JPRB, 0.73920_JPRB, 0.75509_JPRB, 0.77310_JPRB /)
!-- WVL=0.4000E-06
ZASYSS9( :, 4, 5) = (/ &
  & 0.65224_JPRB, 0.65224_JPRB, 0.65224_JPRB, 0.65224_JPRB, 0.61274_JPRB, 0.62804_JPRB, &
  & 0.64897_JPRB, 0.67006_JPRB, 0.70542_JPRB, 0.72765_JPRB, 0.74314_JPRB, 0.77313_JPRB /)
!-- WVL=0.4400E-06
ZASYSS9( :, 5, 5) = (/ &
  & 0.68204_JPRB, 0.68204_JPRB, 0.68204_JPRB, 0.68204_JPRB, 0.60112_JPRB, 0.60878_JPRB, &
  & 0.62356_JPRB, 0.64196_JPRB, 0.67157_JPRB, 0.69720_JPRB, 0.71325_JPRB, 0.76592_JPRB /)
!-- WVL=0.4690E-06
ZASYSS9( :, 6, 5) = (/ &
  & 0.69687_JPRB, 0.69687_JPRB, 0.69687_JPRB, 0.69687_JPRB, 0.60297_JPRB, 0.60144_JPRB, &
  & 0.61006_JPRB, 0.62537_JPRB, 0.65422_JPRB, 0.67337_JPRB, 0.69312_JPRB, 0.75074_JPRB /)
!-- WVL=0.5000E-06
ZASYSS9( :, 7, 5) = (/ &
  & 0.70914_JPRB, 0.70914_JPRB, 0.70914_JPRB, 0.70914_JPRB, 0.61229_JPRB, 0.60213_JPRB, &
  & 0.60151_JPRB, 0.61274_JPRB, 0.63338_JPRB, 0.65573_JPRB, 0.67781_JPRB, 0.73185_JPRB /)
!-- WVL=0.5320E-06
ZASYSS9( :, 8, 5) = (/ &
  & 0.71633_JPRB, 0.71633_JPRB, 0.71633_JPRB, 0.71633_JPRB, 0.62950_JPRB, 0.60941_JPRB, &
  & 0.60174_JPRB, 0.60186_JPRB, 0.62101_JPRB, 0.63586_JPRB, 0.66719_JPRB, 0.71405_JPRB /)
!-- WVL=0.5500E-06
ZASYSS9( :, 9, 5) = (/ &
  & 0.72023_JPRB, 0.72023_JPRB, 0.72023_JPRB, 0.72023_JPRB, 0.63833_JPRB, 0.61674_JPRB, &
  & 0.60391_JPRB, 0.60112_JPRB, 0.61375_JPRB, 0.63569_JPRB, 0.66145_JPRB, 0.70283_JPRB /)
!-- WVL=0.6450E-06
ZASYSS9( :,10, 5) = (/ &
  & 0.72884_JPRB, 0.72884_JPRB, 0.72884_JPRB, 0.72884_JPRB, 0.68728_JPRB, 0.66301_JPRB, &
  & 0.64192_JPRB, 0.62021_JPRB, 0.60271_JPRB, 0.66397_JPRB, 0.65340_JPRB, 0.66776_JPRB /)
!-- WVL=0.6700E-06
ZASYSS9( :,11, 5) = (/ &
  & 0.72738_JPRB, 0.72738_JPRB, 0.72738_JPRB, 0.72738_JPRB, 0.69544_JPRB, 0.67497_JPRB, &
  & 0.65389_JPRB, 0.63144_JPRB, 0.60723_JPRB, 0.67268_JPRB, 0.65363_JPRB, 0.66142_JPRB /)
!-- WVL=0.8000E-06
ZASYSS9( :,12, 5) = (/ &
  & 0.70639_JPRB, 0.70639_JPRB, 0.70639_JPRB, 0.70639_JPRB, 0.72167_JPRB, 0.71288_JPRB, &
  & 0.70130_JPRB, 0.68521_JPRB, 0.65407_JPRB, 0.72185_JPRB, 0.69301_JPRB, 0.65587_JPRB /)
!-- WVL=0.8580E-06
ZASYSS9( :,13, 5) = (/ &
  & 0.69532_JPRB, 0.69532_JPRB, 0.69532_JPRB, 0.69532_JPRB, 0.72790_JPRB, 0.72070_JPRB, &
  & 0.71297_JPRB, 0.69997_JPRB, 0.67544_JPRB, 0.73874_JPRB, 0.71364_JPRB, 0.66899_JPRB /)
!-- WVL=0.8650E-06
ZASYSS9( :,14, 5) = (/ &
  & 0.69395_JPRB, 0.69395_JPRB, 0.69395_JPRB, 0.69395_JPRB, 0.72820_JPRB, 0.72181_JPRB, &
  & 0.71384_JPRB, 0.70154_JPRB, 0.67822_JPRB, 0.74037_JPRB, 0.71564_JPRB, 0.67087_JPRB /)
!-- WVL=0.1020E-05
ZASYSS9( :,15, 5) = (/ &
  & 0.66646_JPRB, 0.66646_JPRB, 0.66646_JPRB, 0.66646_JPRB, 0.72196_JPRB, 0.72845_JPRB, &
  & 0.72864_JPRB, 0.72416_JPRB, 0.71255_JPRB, 0.75263_JPRB, 0.75281_JPRB, 0.71617_JPRB /)
!-- WVL=0.1064E-05
ZASYSS9( :,16, 5) = (/ &
  & 0.65689_JPRB, 0.65689_JPRB, 0.65689_JPRB, 0.65689_JPRB, 0.71665_JPRB, 0.72569_JPRB, &
  & 0.72891_JPRB, 0.72753_JPRB, 0.71695_JPRB, 0.73125_JPRB, 0.75790_JPRB, 0.72648_JPRB /)
!-- WVL=0.1240E-05
ZASYSS9( :,17, 5) = (/ &
  & 0.60278_JPRB, 0.60278_JPRB, 0.60278_JPRB, 0.60278_JPRB, 0.69463_JPRB, 0.70682_JPRB, &
  & 0.71578_JPRB, 0.72502_JPRB, 0.72884_JPRB, 0.75181_JPRB, 0.76811_JPRB, 0.75897_JPRB /)
!-- WVL=0.1640E-05
ZASYSS9( :,18, 5) = (/ &
  & 0.49120_JPRB, 0.49120_JPRB, 0.49120_JPRB, 0.49120_JPRB, 0.63569_JPRB, 0.65904_JPRB, &
  & 0.67288_JPRB, 0.68513_JPRB, 0.72449_JPRB, 0.73691_JPRB, 0.75356_JPRB, 0.77395_JPRB /)
!-- WVL=0.2130E-05
ZASYSS9( :,19, 5) = (/ &
  & 0.30800_JPRB, 0.30800_JPRB, 0.30800_JPRB, 0.30800_JPRB, 0.53832_JPRB, 0.56579_JPRB, &
  & 0.59089_JPRB, 0.62121_JPRB, 0.66596_JPRB, 0.68644_JPRB, 0.70779_JPRB, 0.74569_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS9( :, 1, 5) = (/ &
  & 0.99698_JPRB, 0.99698_JPRB, 0.99698_JPRB, 0.99698_JPRB, 0.99495_JPRB, 0.99459_JPRB, &
  & 0.99432_JPRB, 0.99412_JPRB, 0.99360_JPRB, 0.99302_JPRB, 0.99738_JPRB, 0.99718_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS9( :, 2, 5) = (/ &
  & 0.99723_JPRB, 0.99723_JPRB, 0.99723_JPRB, 0.99723_JPRB, 0.99518_JPRB, 0.99479_JPRB, &
  & 0.99450_JPRB, 0.99425_JPRB, 0.99387_JPRB, 0.99338_JPRB, 0.99751_JPRB, 0.99731_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS9( :, 3, 5) = (/ &
  & 0.99762_JPRB, 0.99762_JPRB, 0.99762_JPRB, 0.99762_JPRB, 0.99554_JPRB, 0.99512_JPRB, &
  & 0.99478_JPRB, 0.99448_JPRB, 0.99415_JPRB, 0.99386_JPRB, 0.99766_JPRB, 0.99756_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS9( :, 4, 5) = (/ &
  & 0.99788_JPRB, 0.99788_JPRB, 0.99788_JPRB, 0.99788_JPRB, 0.99581_JPRB, 0.99540_JPRB, &
  & 0.99505_JPRB, 0.99466_JPRB, 0.99431_JPRB, 0.99406_JPRB, 0.99774_JPRB, 0.99774_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS9( :, 5, 5) = (/ &
  & 0.99827_JPRB, 0.99827_JPRB, 0.99827_JPRB, 0.99827_JPRB, 0.99635_JPRB, 0.99591_JPRB, &
  & 0.99554_JPRB, 0.99512_JPRB, 0.99465_JPRB, 0.99439_JPRB, 0.99786_JPRB, 0.99798_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS9( :, 6, 5) = (/ &
  & 0.99846_JPRB, 0.99846_JPRB, 0.99846_JPRB, 0.99846_JPRB, 0.99672_JPRB, 0.99627_JPRB, &
  & 0.99588_JPRB, 0.99546_JPRB, 0.99494_JPRB, 0.99461_JPRB, 0.99823_JPRB, 0.99807_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS9( :, 7, 5) = (/ &
  & 0.99862_JPRB, 0.99862_JPRB, 0.99862_JPRB, 0.99862_JPRB, 0.99710_JPRB, 0.99665_JPRB, &
  & 0.99624_JPRB, 0.99581_JPRB, 0.99526_JPRB, 0.99492_JPRB, 0.99843_JPRB, 0.99815_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS9( :, 8, 5) = (/ &
  & 0.99873_JPRB, 0.99873_JPRB, 0.99873_JPRB, 0.99873_JPRB, 0.99748_JPRB, 0.99702_JPRB, &
  & 0.99661_JPRB, 0.99615_JPRB, 0.99560_JPRB, 0.99603_JPRB, 0.99854_JPRB, 0.99822_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS9( :, 9, 5) = (/ &
  & 0.99879_JPRB, 0.99879_JPRB, 0.99879_JPRB, 0.99879_JPRB, 0.99764_JPRB, 0.99721_JPRB, &
  & 0.99680_JPRB, 0.99635_JPRB, 0.99579_JPRB, 0.99733_JPRB, 0.99860_JPRB, 0.99826_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS9( :,10, 5) = (/ &
  & 0.99896_JPRB, 0.99896_JPRB, 0.99896_JPRB, 0.99896_JPRB, 0.99833_JPRB, 0.99803_JPRB, &
  & 0.99771_JPRB, 0.99730_JPRB, 0.99670_JPRB, 0.99888_JPRB, 0.99887_JPRB, 0.99853_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS9( :,11, 5) = (/ &
  & 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99844_JPRB, 0.99818_JPRB, &
  & 0.99790_JPRB, 0.99752_JPRB, 0.99693_JPRB, 0.99896_JPRB, 0.99882_JPRB, 0.99860_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS9( :,12, 5) = (/ &
  & 0.99904_JPRB, 0.99904_JPRB, 0.99904_JPRB, 0.99904_JPRB, 0.99881_JPRB, 0.99867_JPRB, &
  & 0.99852_JPRB, 0.99830_JPRB, 0.99790_JPRB, 0.99929_JPRB, 0.99912_JPRB, 0.99893_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS9( :,13, 5) = (/ &
  & 0.99903_JPRB, 0.99903_JPRB, 0.99903_JPRB, 0.99903_JPRB, 0.99889_JPRB, 0.99880_JPRB, &
  & 0.99868_JPRB, 0.99850_JPRB, 0.99819_JPRB, 0.99938_JPRB, 0.99924_JPRB, 0.99907_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS9( :,14, 5) = (/ &
  & 0.99903_JPRB, 0.99903_JPRB, 0.99903_JPRB, 0.99903_JPRB, 0.99890_JPRB, 0.99881_JPRB, &
  & 0.99869_JPRB, 0.99852_JPRB, 0.99822_JPRB, 0.99939_JPRB, 0.99925_JPRB, 0.99908_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS9( :,15, 5) = (/ &
  & 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99898_JPRB, 0.99900_JPRB, 0.99897_JPRB, &
  & 0.99891_JPRB, 0.99884_JPRB, 0.99867_JPRB, 0.99934_JPRB, 0.99945_JPRB, 0.99935_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS9( :,16, 5) = (/ &
  & 0.99895_JPRB, 0.99895_JPRB, 0.99895_JPRB, 0.99895_JPRB, 0.99902_JPRB, 0.99899_JPRB, &
  & 0.99895_JPRB, 0.99888_JPRB, 0.99874_JPRB, 0.99898_JPRB, 0.99948_JPRB, 0.99940_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS9( :,17, 5) = (/ &
  & 0.99884_JPRB, 0.99884_JPRB, 0.99883_JPRB, 0.99883_JPRB, 0.99903_JPRB, 0.99904_JPRB, &
  & 0.99902_JPRB, 0.99900_JPRB, 0.99893_JPRB, 0.99928_JPRB, 0.99957_JPRB, 0.99955_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS9( :,18, 5) = (/ &
  & 0.99843_JPRB, 0.99843_JPRB, 0.99843_JPRB, 0.99843_JPRB, 0.99890_JPRB, 0.99896_JPRB, &
  & 0.99899_JPRB, 0.99902_JPRB, 0.99961_JPRB, 0.99961_JPRB, 0.99961_JPRB, 0.99966_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS9( :,19, 5) = (/ &
  & 0.99772_JPRB, 0.99772_JPRB, 0.99772_JPRB, 0.99772_JPRB, 0.99862_JPRB, 0.99874_JPRB, &
  & 0.99881_JPRB, 0.99887_JPRB, 0.99954_JPRB, 0.99956_JPRB, 0.99959_JPRB, 0.99968_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS9( :, 1, 5) = (/ &
  &  9.1919_JPRB,  9.1919_JPRB,  9.1919_JPRB,  9.1919_JPRB,  6.3233_JPRB,  6.5496_JPRB, &
  &  7.2324_JPRB,  8.6807_JPRB, 10.5762_JPRB, 12.4075_JPRB, 18.5750_JPRB, 15.5089_JPRB /)
!-- WVL=0.3550E-06
ZRALIS9( :, 2, 5) = (/ &
  & 10.2734_JPRB, 10.2734_JPRB, 10.2734_JPRB, 10.2734_JPRB,  6.1976_JPRB,  6.3543_JPRB, &
  &  6.9735_JPRB,  7.6163_JPRB,  9.6170_JPRB, 11.0524_JPRB, 17.9590_JPRB, 15.5061_JPRB /)
!-- WVL=0.3800E-06
ZRALIS9( :, 3, 5) = (/ &
  & 12.1495_JPRB, 12.1495_JPRB, 12.1495_JPRB, 12.1495_JPRB,  6.3335_JPRB,  6.2366_JPRB, &
  &  6.3564_JPRB,  7.0757_JPRB,  8.4845_JPRB,  9.6696_JPRB, 18.0407_JPRB, 16.3822_JPRB /)
!-- WVL=0.4000E-06
ZRALIS9( :, 4, 5) = (/ &
  & 14.8288_JPRB, 14.8287_JPRB, 14.8288_JPRB, 14.8287_JPRB,  6.4762_JPRB,  6.2743_JPRB, &
  &  6.3283_JPRB,  6.4054_JPRB,  7.3105_JPRB,  8.7430_JPRB, 18.0178_JPRB, 16.7221_JPRB /)
!-- WVL=0.4400E-06
ZRALIS9( :, 5, 5) = (/ &
  & 20.4358_JPRB, 20.4358_JPRB, 20.4358_JPRB, 20.4358_JPRB,  7.3431_JPRB,  6.6524_JPRB, &
  &  6.3352_JPRB,  6.2328_JPRB,  6.4174_JPRB,  7.2035_JPRB, 17.0193_JPRB, 17.8386_JPRB /)
!-- WVL=0.4690E-06
ZRALIS9( :, 6, 5) = (/ &
  & 24.0491_JPRB, 24.0491_JPRB, 24.0491_JPRB, 24.0491_JPRB,  8.1381_JPRB,  7.1676_JPRB, &
  &  6.5971_JPRB,  6.2594_JPRB,  6.3238_JPRB,  6.4601_JPRB, 16.9056_JPRB, 17.5754_JPRB /)
!-- WVL=0.5000E-06
ZRALIS9( :, 7, 5) = (/ &
  & 29.0684_JPRB, 29.0684_JPRB, 29.0684_JPRB, 29.0684_JPRB,  9.5817_JPRB,  7.9256_JPRB, &
  &  7.1047_JPRB,  6.4762_JPRB,  6.1940_JPRB,  6.3200_JPRB, 17.0087_JPRB, 17.0330_JPRB /)
!-- WVL=0.5320E-06
ZRALIS9( :, 8, 5) = (/ &
  & 33.6335_JPRB, 33.6335_JPRB, 33.6335_JPRB, 33.6335_JPRB, 11.6256_JPRB,  9.2963_JPRB, &
  &  7.8542_JPRB,  7.0136_JPRB,  6.3697_JPRB,  7.6083_JPRB, 17.7465_JPRB, 17.4486_JPRB /)
!-- WVL=0.5500E-06
ZRALIS9( :, 9, 5) = (/ &
  & 37.2279_JPRB, 37.2278_JPRB, 37.2278_JPRB, 37.2278_JPRB, 12.3237_JPRB, 10.1485_JPRB, &
  &  8.4179_JPRB,  7.3431_JPRB,  6.4720_JPRB, 11.0363_JPRB, 18.1964_JPRB, 16.5524_JPRB /)
!-- WVL=0.6450E-06
ZRALIS9( :,10, 5) = (/ &
  & 56.2867_JPRB, 56.2867_JPRB, 56.2867_JPRB, 56.2867_JPRB, 21.8052_JPRB, 15.9931_JPRB, &
  & 12.8879_JPRB, 10.6634_JPRB,  8.0613_JPRB, 21.6191_JPRB, 20.5844_JPRB, 17.6612_JPRB /)
!-- WVL=0.6700E-06
ZRALIS9( :,11, 5) = (/ &
  & 58.3163_JPRB, 58.3163_JPRB, 58.3163_JPRB, 58.3164_JPRB, 23.6738_JPRB, 18.2676_JPRB, &
  & 15.0313_JPRB, 11.7470_JPRB,  9.0321_JPRB, 22.6087_JPRB, 20.5608_JPRB, 18.1944_JPRB /)
!-- WVL=0.8000E-06
ZRALIS9( :,12, 5) = (/ &
  & 70.2268_JPRB, 70.2268_JPRB, 70.2268_JPRB, 70.2268_JPRB, 38.3667_JPRB, 30.9870_JPRB, &
  & 25.5715_JPRB, 21.2972_JPRB, 15.0519_JPRB, 32.5851_JPRB, 26.9244_JPRB, 21.9823_JPRB /)
!-- WVL=0.8580E-06
ZRALIS9( :,13, 5) = (/ &
  & 70.6407_JPRB, 70.6407_JPRB, 70.6407_JPRB, 70.6407_JPRB, 44.7165_JPRB, 37.6086_JPRB, &
  & 31.0432_JPRB, 25.0487_JPRB, 18.4072_JPRB, 37.5985_JPRB, 30.6721_JPRB, 24.1031_JPRB /)
!-- WVL=0.8650E-06
ZRALIS9( :,14, 5) = (/ &
  & 70.5826_JPRB, 70.5826_JPRB, 70.5826_JPRB, 70.5826_JPRB, 45.6262_JPRB, 38.4758_JPRB, &
  & 31.6133_JPRB, 25.6698_JPRB, 19.2685_JPRB, 38.0984_JPRB, 30.8671_JPRB, 24.2593_JPRB /)
!-- WVL=0.1020E-05
ZRALIS9( :,15, 5) = (/ &
  & 88.3158_JPRB, 88.3158_JPRB, 88.3158_JPRB, 88.3158_JPRB, 58.5715_JPRB, 57.6843_JPRB, &
  & 48.1007_JPRB, 40.2404_JPRB, 30.7888_JPRB, 45.1430_JPRB, 45.8037_JPRB, 33.6836_JPRB /)
!-- WVL=0.1064E-05
ZRALIS9( :,16, 5) = (/ &
  & 91.3608_JPRB, 91.3608_JPRB, 91.3608_JPRB, 91.3608_JPRB, 61.4886_JPRB, 58.3509_JPRB, &
  & 55.0244_JPRB, 43.9064_JPRB, 34.2194_JPRB, 36.7300_JPRB, 47.8780_JPRB, 34.7408_JPRB /)
!-- WVL=0.1240E-05
ZRALIS9( :,17, 5) = (/ &
  & 72.2317_JPRB, 72.2317_JPRB, 72.2317_JPRB, 72.2317_JPRB, 70.6072_JPRB, 70.0731_JPRB, &
  & 62.3215_JPRB, 58.3310_JPRB, 51.1217_JPRB, 55.2090_JPRB, 57.9928_JPRB, 49.3558_JPRB /)
!-- WVL=0.1640E-05
ZRALIS9( :,18, 5) = (/ &
  & 48.4956_JPRB, 48.4956_JPRB, 48.4956_JPRB, 48.4956_JPRB, 85.7799_JPRB, 91.1354_JPRB, &
  & 82.9673_JPRB, 72.1376_JPRB, 76.6853_JPRB, 78.4774_JPRB, 72.3316_JPRB, 67.1717_JPRB /)
!-- WVL=0.2130E-05
ZRALIS9( :,19, 5) = (/ &
  & 20.9475_JPRB, 20.9475_JPRB, 20.9475_JPRB, 20.9475_JPRB, 59.2048_JPRB, 63.8132_JPRB, &
  & 68.7806_JPRB, 79.3271_JPRB, 91.5940_JPRB, 93.1209_JPRB, 79.1904_JPRB, 77.9983_JPRB /)
  
 ! ** BIN No.            6    1.000000000000000         2.000000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS9( :, 1, 6) = (/ &
  & 0.25716_JPRB, 0.25716_JPRB, 0.25716_JPRB, 0.25716_JPRB, 0.48924_JPRB, 0.57766_JPRB, &
  & 0.66482_JPRB, 0.75673_JPRB, 0.91863_JPRB, 1.04993_JPRB, 1.28178_JPRB, 1.88453_JPRB /)
!-- WVL=0.3550E-06
ZALFSS9( :, 2, 6) = (/ &
  & 0.25876_JPRB, 0.25876_JPRB, 0.25876_JPRB, 0.25876_JPRB, 0.49251_JPRB, 0.57199_JPRB, &
  & 0.66656_JPRB, 0.76710_JPRB, 0.92026_JPRB, 1.05214_JPRB, 1.28729_JPRB, 1.90362_JPRB /)
!-- WVL=0.3800E-06
ZALFSS9( :, 3, 6) = (/ &
  & 0.25421_JPRB, 0.25421_JPRB, 0.25421_JPRB, 0.25421_JPRB, 0.49720_JPRB, 0.57295_JPRB, &
  & 0.65700_JPRB, 0.77831_JPRB, 0.92667_JPRB, 1.05759_JPRB, 1.28965_JPRB, 1.90345_JPRB /)
!-- WVL=0.4000E-06
ZALFSS9( :, 4, 6) = (/ &
  & 0.24919_JPRB, 0.24919_JPRB, 0.24919_JPRB, 0.24919_JPRB, 0.49674_JPRB, 0.57839_JPRB, &
  & 0.65639_JPRB, 0.77091_JPRB, 0.94342_JPRB, 1.06016_JPRB, 1.30598_JPRB, 1.89588_JPRB /)
!-- WVL=0.4400E-06
ZALFSS9( :, 5, 6) = (/ &
  & 0.24575_JPRB, 0.24575_JPRB, 0.24575_JPRB, 0.24575_JPRB, 0.50559_JPRB, 0.57945_JPRB, &
  & 0.66610_JPRB, 0.76748_JPRB, 0.94268_JPRB, 1.09034_JPRB, 1.32431_JPRB, 1.90877_JPRB /)
!-- WVL=0.4690E-06
ZALFSS9( :, 6, 6) = (/ &
  & 0.24885_JPRB, 0.24885_JPRB, 0.24885_JPRB, 0.24885_JPRB, 0.52354_JPRB, 0.58694_JPRB, &
  & 0.66555_JPRB, 0.77611_JPRB, 0.93336_JPRB, 1.08435_JPRB, 1.30419_JPRB, 1.92178_JPRB /)
!-- WVL=0.5000E-06
ZALFSS9( :, 7, 6) = (/ &
  & 0.25500_JPRB, 0.25500_JPRB, 0.25500_JPRB, 0.25500_JPRB, 0.53518_JPRB, 0.60623_JPRB, &
  & 0.67263_JPRB, 0.77616_JPRB, 0.94201_JPRB, 1.07301_JPRB, 1.31197_JPRB, 1.96225_JPRB /)
!-- WVL=0.5320E-06
ZALFSS9( :, 8, 6) = (/ &
  & 0.25776_JPRB, 0.25776_JPRB, 0.25776_JPRB, 0.25776_JPRB, 0.53224_JPRB, 0.62268_JPRB, &
  & 0.69337_JPRB, 0.78025_JPRB, 0.94776_JPRB, 1.08445_JPRB, 1.32289_JPRB, 1.96004_JPRB /)
!-- WVL=0.5500E-06
ZALFSS9( :, 9, 6) = (/ &
  & 0.25880_JPRB, 0.25880_JPRB, 0.25880_JPRB, 0.25880_JPRB, 0.52561_JPRB, 0.62541_JPRB, &
  & 0.70578_JPRB, 0.78998_JPRB, 0.94719_JPRB, 1.08181_JPRB, 1.32202_JPRB, 1.94406_JPRB /)
!-- WVL=0.6450E-06
ZALFSS9( :,10, 6) = (/ &
  & 0.25454_JPRB, 0.25454_JPRB, 0.25454_JPRB, 0.25454_JPRB, 0.51020_JPRB, 0.59476_JPRB, &
  & 0.70078_JPRB, 0.83728_JPRB, 0.99639_JPRB, 1.15922_JPRB, 1.37697_JPRB, 1.96179_JPRB /)
!-- WVL=0.6700E-06
ZALFSS9( :,11, 6) = (/ &
  & 0.25532_JPRB, 0.25532_JPRB, 0.25532_JPRB, 0.25532_JPRB, 0.51439_JPRB, 0.59331_JPRB, &
  & 0.68986_JPRB, 0.82971_JPRB, 1.01357_JPRB, 1.15605_JPRB, 1.39901_JPRB, 1.96132_JPRB /)
!-- WVL=0.8000E-06
ZALFSS9( :,12, 6) = (/ &
  & 0.29122_JPRB, 0.29122_JPRB, 0.29122_JPRB, 0.29122_JPRB, 0.53572_JPRB, 0.62015_JPRB, &
  & 0.69815_JPRB, 0.79574_JPRB, 0.98217_JPRB, 1.09991_JPRB, 1.38206_JPRB, 2.06419_JPRB /)
!-- WVL=0.8580E-06
ZALFSS9( :,13, 6) = (/ &
  & 0.31693_JPRB, 0.31693_JPRB, 0.31693_JPRB, 0.31693_JPRB, 0.52920_JPRB, 0.62557_JPRB, &
  & 0.71227_JPRB, 0.81237_JPRB, 0.97018_JPRB, 1.11333_JPRB, 1.35097_JPRB, 2.10955_JPRB /)
!-- WVL=0.8650E-06
ZALFSS9( :,14, 6) = (/ &
  & 0.31982_JPRB, 0.31982_JPRB, 0.31982_JPRB, 0.31982_JPRB, 0.52809_JPRB, 0.62482_JPRB, &
  & 0.71317_JPRB, 0.81462_JPRB, 0.97045_JPRB, 1.11573_JPRB, 1.35013_JPRB, 2.11020_JPRB /)
!-- WVL=0.1020E-05
ZALFSS9( :,15, 6) = (/ &
  & 0.37999_JPRB, 0.37999_JPRB, 0.37999_JPRB, 0.37999_JPRB, 0.54073_JPRB, 0.61640_JPRB, &
  & 0.70755_JPRB, 0.83546_JPRB, 1.01347_JPRB, 1.15930_JPRB, 1.40111_JPRB, 2.00775_JPRB /)
!-- WVL=0.1064E-05
ZALFSS9( :,16, 6) = (/ &
  & 0.39125_JPRB, 0.39125_JPRB, 0.39125_JPRB, 0.39125_JPRB, 0.55877_JPRB, 0.62160_JPRB, &
  & 0.70722_JPRB, 0.82906_JPRB, 1.02036_JPRB, 1.16455_JPRB, 1.41115_JPRB, 1.99926_JPRB /)
!-- WVL=0.1240E-05
ZALFSS9( :,17, 6) = (/ &
  & 0.40845_JPRB, 0.40845_JPRB, 0.40845_JPRB, 0.40845_JPRB, 0.65963_JPRB, 0.70143_JPRB, &
  & 0.75383_JPRB, 0.83445_JPRB, 1.00590_JPRB, 1.14375_JPRB, 1.40133_JPRB, 2.08279_JPRB /)
!-- WVL=0.1640E-05
ZALFSS9( :,18, 6) = (/ &
  & 0.34857_JPRB, 0.34857_JPRB, 0.34857_JPRB, 0.34857_JPRB, 0.83432_JPRB, 0.94104_JPRB, &
  & 1.02175_JPRB, 1.10622_JPRB, 1.38114_JPRB, 1.46937_JPRB, 1.59771_JPRB, 2.07405_JPRB /)
!-- WVL=0.2130E-05
ZALFSS9( :,19, 6) = (/ &
  & 0.24267_JPRB, 0.24267_JPRB, 0.24267_JPRB, 0.24267_JPRB, 0.78547_JPRB, 0.96197_JPRB, &
  & 1.12536_JPRB, 1.32116_JPRB, 1.57156_JPRB, 1.79853_JPRB, 2.10907_JPRB, 2.60706_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS9( :, 1, 6) = (/ &
  & 0.74540_JPRB, 0.74540_JPRB, 0.74539_JPRB, 0.74540_JPRB, 0.75341_JPRB, 0.76186_JPRB, &
  & 0.76644_JPRB, 0.76635_JPRB, 0.77244_JPRB, 0.77576_JPRB, 0.79701_JPRB, 0.80739_JPRB /)
!-- WVL=0.3550E-06
ZASYSS9( :, 2, 6) = (/ &
  & 0.74300_JPRB, 0.74300_JPRB, 0.74300_JPRB, 0.74300_JPRB, 0.75188_JPRB, 0.75628_JPRB, &
  & 0.76429_JPRB, 0.76668_JPRB, 0.76972_JPRB, 0.77338_JPRB, 0.79545_JPRB, 0.80745_JPRB /)
!-- WVL=0.3800E-06
ZASYSS9( :, 3, 6) = (/ &
  & 0.73043_JPRB, 0.73043_JPRB, 0.73043_JPRB, 0.73043_JPRB, 0.74910_JPRB, 0.75290_JPRB, &
  & 0.75637_JPRB, 0.76488_JPRB, 0.76600_JPRB, 0.76976_JPRB, 0.79140_JPRB, 0.80390_JPRB /)
!-- WVL=0.4000E-06
ZASYSS9( :, 4, 6) = (/ &
  & 0.71642_JPRB, 0.71642_JPRB, 0.71642_JPRB, 0.71642_JPRB, 0.74539_JPRB, 0.75075_JPRB, &
  & 0.75263_JPRB, 0.75915_JPRB, 0.76639_JPRB, 0.76666_JPRB, 0.79128_JPRB, 0.80010_JPRB /)
!-- WVL=0.4400E-06
ZASYSS9( :, 5, 6) = (/ &
  & 0.69419_JPRB, 0.69419_JPRB, 0.69419_JPRB, 0.69419_JPRB, 0.74158_JPRB, 0.74309_JPRB, &
  & 0.74907_JPRB, 0.75287_JPRB, 0.75967_JPRB, 0.76648_JPRB, 0.78919_JPRB, 0.79600_JPRB /)
!-- WVL=0.4690E-06
ZASYSS9( :, 6, 6) = (/ &
  & 0.68712_JPRB, 0.68712_JPRB, 0.68712_JPRB, 0.68712_JPRB, 0.74490_JPRB, 0.74118_JPRB, &
  & 0.74364_JPRB, 0.75054_JPRB, 0.75342_JPRB, 0.76086_JPRB, 0.78382_JPRB, 0.79346_JPRB /)
!-- WVL=0.5000E-06
ZASYSS9( :, 7, 6) = (/ &
  & 0.67625_JPRB, 0.67625_JPRB, 0.67625_JPRB, 0.67625_JPRB, 0.74472_JPRB, 0.74464_JPRB, &
  & 0.74141_JPRB, 0.74539_JPRB, 0.75187_JPRB, 0.75360_JPRB, 0.78183_JPRB, 0.79441_JPRB /)
!-- WVL=0.5320E-06
ZASYSS9( :, 8, 6) = (/ &
  & 0.66487_JPRB, 0.66487_JPRB, 0.66487_JPRB, 0.66487_JPRB, 0.73627_JPRB, 0.74518_JPRB, &
  & 0.74411_JPRB, 0.74170_JPRB, 0.74882_JPRB, 0.75741_JPRB, 0.77908_JPRB, 0.79085_JPRB /)
!-- WVL=0.5500E-06
ZASYSS9( :, 9, 6) = (/ &
  & 0.65567_JPRB, 0.65567_JPRB, 0.65567_JPRB, 0.65567_JPRB, 0.72932_JPRB, 0.74347_JPRB, &
  & 0.74518_JPRB, 0.74158_JPRB, 0.74602_JPRB, 0.76047_JPRB, 0.77707_JPRB, 0.78751_JPRB /)
!-- WVL=0.6450E-06
ZASYSS9( :,10, 6) = (/ &
  & 0.60499_JPRB, 0.60499_JPRB, 0.60499_JPRB, 0.60499_JPRB, 0.69176_JPRB, 0.70750_JPRB, &
  & 0.72619_JPRB, 0.74146_JPRB, 0.74493_JPRB, 0.77076_JPRB, 0.77400_JPRB, 0.77948_JPRB /)
!-- WVL=0.6700E-06
ZASYSS9( :,11, 6) = (/ &
  & 0.59813_JPRB, 0.59813_JPRB, 0.59813_JPRB, 0.59813_JPRB, 0.68728_JPRB, 0.69936_JPRB, &
  & 0.71559_JPRB, 0.73460_JPRB, 0.74550_JPRB, 0.76667_JPRB, 0.77359_JPRB, 0.77705_JPRB /)
!-- WVL=0.8000E-06
ZASYSS9( :,12, 6) = (/ &
  & 0.61655_JPRB, 0.61655_JPRB, 0.61655_JPRB, 0.61655_JPRB, 0.65229_JPRB, 0.67090_JPRB, &
  & 0.68366_JPRB, 0.69312_JPRB, 0.71545_JPRB, 0.72729_JPRB, 0.75142_JPRB, 0.77479_JPRB /)
!-- WVL=0.8580E-06
ZASYSS9( :,13, 6) = (/ &
  & 0.63941_JPRB, 0.63941_JPRB, 0.63941_JPRB, 0.63941_JPRB, 0.62857_JPRB, 0.65442_JPRB, &
  & 0.67069_JPRB, 0.68444_JPRB, 0.69905_JPRB, 0.71669_JPRB, 0.73368_JPRB, 0.77326_JPRB /)
!-- WVL=0.8650E-06
ZASYSS9( :,14, 6) = (/ &
  & 0.64281_JPRB, 0.64281_JPRB, 0.64281_JPRB, 0.64281_JPRB, 0.62667_JPRB, 0.65205_JPRB, &
  & 0.66881_JPRB, 0.68349_JPRB, 0.69705_JPRB, 0.71676_JPRB, 0.73215_JPRB, 0.77257_JPRB /)
!-- WVL=0.1020E-05
ZASYSS9( :,15, 6) = (/ &
  & 0.69618_JPRB, 0.69618_JPRB, 0.69618_JPRB, 0.69618_JPRB, 0.59283_JPRB, 0.60082_JPRB, &
  & 0.62049_JPRB, 0.64608_JPRB, 0.67163_JPRB, 0.68822_JPRB, 0.70762_JPRB, 0.73832_JPRB /)
!-- WVL=0.1064E-05
ZASYSS9( :,16, 6) = (/ &
  & 0.70356_JPRB, 0.70356_JPRB, 0.70356_JPRB, 0.70356_JPRB, 0.59822_JPRB, 0.59451_JPRB, &
  & 0.60730_JPRB, 0.63080_JPRB, 0.66323_JPRB, 0.67797_JPRB, 0.70208_JPRB, 0.73090_JPRB /)
!-- WVL=0.1240E-05
ZASYSS9( :,17, 6) = (/ &
  & 0.72645_JPRB, 0.72645_JPRB, 0.72645_JPRB, 0.72645_JPRB, 0.64106_JPRB, 0.61551_JPRB, &
  & 0.59894_JPRB, 0.59353_JPRB, 0.61581_JPRB, 0.63410_JPRB, 0.66488_JPRB, 0.70879_JPRB /)
!-- WVL=0.1640E-05
ZASYSS9( :,18, 6) = (/ &
  & 0.71729_JPRB, 0.71729_JPRB, 0.71729_JPRB, 0.71729_JPRB, 0.71786_JPRB, 0.70215_JPRB, &
  & 0.68652_JPRB, 0.66304_JPRB, 0.72181_JPRB, 0.70152_JPRB, 0.66724_JPRB, 0.64809_JPRB /)
!-- WVL=0.2130E-05
ZASYSS9( :,19, 6) = (/ &
  & 0.67644_JPRB, 0.67644_JPRB, 0.67644_JPRB, 0.67644_JPRB, 0.72531_JPRB, 0.72985_JPRB, &
  & 0.72847_JPRB, 0.72311_JPRB, 0.76711_JPRB, 0.76130_JPRB, 0.74411_JPRB, 0.69716_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS9( :, 1, 6) = (/ &
  & 0.99298_JPRB, 0.99298_JPRB, 0.99298_JPRB, 0.99298_JPRB, 0.99006_JPRB, 0.98962_JPRB, &
  & 0.98909_JPRB, 0.98815_JPRB, 0.98717_JPRB, 0.98638_JPRB, 0.99453_JPRB, 0.99457_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS9( :, 2, 6) = (/ &
  & 0.99327_JPRB, 0.99327_JPRB, 0.99327_JPRB, 0.99327_JPRB, 0.99042_JPRB, 0.98983_JPRB, &
  & 0.98945_JPRB, 0.98867_JPRB, 0.98761_JPRB, 0.98689_JPRB, 0.99471_JPRB, 0.99483_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS9( :, 3, 6) = (/ &
  & 0.99354_JPRB, 0.99354_JPRB, 0.99354_JPRB, 0.99354_JPRB, 0.99096_JPRB, 0.99036_JPRB, &
  & 0.98983_JPRB, 0.98938_JPRB, 0.98828_JPRB, 0.98759_JPRB, 0.99502_JPRB, 0.99510_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS9( :, 4, 6) = (/ &
  & 0.99369_JPRB, 0.99369_JPRB, 0.99369_JPRB, 0.99369_JPRB, 0.99126_JPRB, 0.99077_JPRB, &
  & 0.99020_JPRB, 0.98969_JPRB, 0.98890_JPRB, 0.98808_JPRB, 0.99528_JPRB, 0.99527_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS9( :, 5, 6) = (/ &
  & 0.99416_JPRB, 0.99417_JPRB, 0.99416_JPRB, 0.99417_JPRB, 0.99198_JPRB, 0.99137_JPRB, &
  & 0.99096_JPRB, 0.99036_JPRB, 0.98964_JPRB, 0.98919_JPRB, 0.99568_JPRB, 0.99566_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS9( :, 6, 6) = (/ &
  & 0.99465_JPRB, 0.99465_JPRB, 0.99465_JPRB, 0.99465_JPRB, 0.99258_JPRB, 0.99185_JPRB, &
  & 0.99135_JPRB, 0.99088_JPRB, 0.99005_JPRB, 0.98965_JPRB, 0.99631_JPRB, 0.99590_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS9( :, 7, 6) = (/ &
  & 0.99507_JPRB, 0.99507_JPRB, 0.99507_JPRB, 0.99507_JPRB, 0.99314_JPRB, 0.99245_JPRB, &
  & 0.99180_JPRB, 0.99126_JPRB, 0.99061_JPRB, 0.99002_JPRB, 0.99669_JPRB, 0.99619_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS9( :, 8, 6) = (/ &
  & 0.99543_JPRB, 0.99543_JPRB, 0.99543_JPRB, 0.99543_JPRB, 0.99345_JPRB, 0.99301_JPRB, &
  & 0.99238_JPRB, 0.99168_JPRB, 0.99106_JPRB, 0.99197_JPRB, 0.99685_JPRB, 0.99636_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS9( :, 9, 6) = (/ &
  & 0.99560_JPRB, 0.99560_JPRB, 0.99560_JPRB, 0.99560_JPRB, 0.99355_JPRB, 0.99326_JPRB, &
  & 0.99271_JPRB, 0.99198_JPRB, 0.99124_JPRB, 0.99424_JPRB, 0.99694_JPRB, 0.99642_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS9( :,10, 6) = (/ &
  & 0.99621_JPRB, 0.99621_JPRB, 0.99621_JPRB, 0.99621_JPRB, 0.99429_JPRB, 0.99386_JPRB, &
  & 0.99361_JPRB, 0.99328_JPRB, 0.99255_JPRB, 0.99729_JPRB, 0.99739_JPRB, 0.99686_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS9( :,11, 6) = (/ &
  & 0.99641_JPRB, 0.99641_JPRB, 0.99641_JPRB, 0.99641_JPRB, 0.99458_JPRB, 0.99402_JPRB, &
  & 0.99376_JPRB, 0.99345_JPRB, 0.99288_JPRB, 0.99738_JPRB, 0.99726_JPRB, 0.99693_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS9( :,12, 6) = (/ &
  & 0.99740_JPRB, 0.99740_JPRB, 0.99740_JPRB, 0.99740_JPRB, 0.99564_JPRB, 0.99526_JPRB, &
  & 0.99482_JPRB, 0.99427_JPRB, 0.99376_JPRB, 0.99768_JPRB, 0.99751_JPRB, 0.99751_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS9( :,13, 6) = (/ &
  & 0.99776_JPRB, 0.99776_JPRB, 0.99776_JPRB, 0.99776_JPRB, 0.99591_JPRB, 0.99561_JPRB, &
  & 0.99526_JPRB, 0.99472_JPRB, 0.99404_JPRB, 0.99788_JPRB, 0.99762_JPRB, 0.99772_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS9( :,14, 6) = (/ &
  & 0.99781_JPRB, 0.99781_JPRB, 0.99781_JPRB, 0.99781_JPRB, 0.99594_JPRB, 0.99565_JPRB, &
  & 0.99529_JPRB, 0.99484_JPRB, 0.99413_JPRB, 0.99791_JPRB, 0.99763_JPRB, 0.99775_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS9( :,15, 6) = (/ &
  & 0.99847_JPRB, 0.99847_JPRB, 0.99847_JPRB, 0.99847_JPRB, 0.99666_JPRB, 0.99628_JPRB, &
  & 0.99600_JPRB, 0.99571_JPRB, 0.99525_JPRB, 0.99765_JPRB, 0.99807_JPRB, 0.99799_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS9( :,16, 6) = (/ &
  & 0.99857_JPRB, 0.99857_JPRB, 0.99857_JPRB, 0.99857_JPRB, 0.99694_JPRB, 0.99649_JPRB, &
  & 0.99618_JPRB, 0.99588_JPRB, 0.99545_JPRB, 0.99632_JPRB, 0.99817_JPRB, 0.99807_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS9( :,17, 6) = (/ &
  & 0.99885_JPRB, 0.99885_JPRB, 0.99885_JPRB, 0.99885_JPRB, 0.99779_JPRB, 0.99738_JPRB, &
  & 0.99697_JPRB, 0.99652_JPRB, 0.99612_JPRB, 0.99738_JPRB, 0.99843_JPRB, 0.99841_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS9( :,18, 6) = (/ &
  & 0.99904_JPRB, 0.99904_JPRB, 0.99904_JPRB, 0.99904_JPRB, 0.99874_JPRB, 0.99855_JPRB, &
  & 0.99836_JPRB, 0.99807_JPRB, 0.99930_JPRB, 0.99919_JPRB, 0.99898_JPRB, 0.99881_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS9( :,19, 6) = (/ &
  & 0.99900_JPRB, 0.99900_JPRB, 0.99900_JPRB, 0.99900_JPRB, 0.99900_JPRB, 0.99896_JPRB, &
  & 0.99889_JPRB, 0.99879_JPRB, 0.99954_JPRB, 0.99950_JPRB, 0.99941_JPRB, 0.99928_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS9( :, 1, 6) = (/ &
  & 13.6391_JPRB, 13.6391_JPRB, 13.6391_JPRB, 13.6391_JPRB, 19.0355_JPRB, 18.4301_JPRB, &
  & 16.2796_JPRB, 13.6739_JPRB, 12.2311_JPRB, 12.6341_JPRB, 12.0777_JPRB, 12.7411_JPRB /)
!-- WVL=0.3550E-06
ZRALIS9( :, 2, 6) = (/ &
  & 12.3585_JPRB, 12.3585_JPRB, 12.3585_JPRB, 12.3585_JPRB, 19.6545_JPRB, 18.7334_JPRB, &
  & 17.7060_JPRB, 15.2374_JPRB, 12.3832_JPRB, 12.2847_JPRB, 12.9658_JPRB, 13.0144_JPRB /)
!-- WVL=0.3800E-06
ZRALIS9( :, 3, 6) = (/ &
  & 10.2682_JPRB, 10.2682_JPRB, 10.2682_JPRB, 10.2682_JPRB, 19.1949_JPRB, 19.5564_JPRB, &
  & 18.7250_JPRB, 17.4449_JPRB, 13.7610_JPRB, 12.4065_JPRB, 13.9326_JPRB, 12.4272_JPRB /)
!-- WVL=0.4000E-06
ZRALIS9( :, 4, 6) = (/ &
  &  8.5884_JPRB,  8.5884_JPRB,  8.5884_JPRB,  8.5884_JPRB, 19.3052_JPRB, 19.0371_JPRB, &
  & 19.0294_JPRB, 18.3651_JPRB, 15.7136_JPRB, 13.3830_JPRB, 15.3193_JPRB, 12.0300_JPRB /)
!-- WVL=0.4400E-06
ZRALIS9( :, 5, 6) = (/ &
  &  7.1598_JPRB,  7.1598_JPRB,  7.1598_JPRB,  7.1598_JPRB, 16.8671_JPRB, 18.1646_JPRB, &
  & 19.1749_JPRB, 19.5880_JPRB, 18.4363_JPRB, 16.8264_JPRB, 15.9358_JPRB, 13.1022_JPRB /)
!-- WVL=0.4690E-06
ZRALIS9( :, 6, 6) = (/ &
  &  6.8279_JPRB,  6.8279_JPRB,  6.8279_JPRB,  6.8279_JPRB, 15.2869_JPRB, 17.5264_JPRB, &
  & 18.6217_JPRB, 19.5358_JPRB, 19.0588_JPRB, 18.5272_JPRB, 15.2312_JPRB, 14.1182_JPRB /)
!-- WVL=0.5000E-06
ZRALIS9( :, 7, 6) = (/ &
  &  6.5921_JPRB,  6.5921_JPRB,  6.5921_JPRB,  6.5921_JPRB, 13.2180_JPRB, 15.8125_JPRB, &
  & 17.8781_JPRB, 19.3053_JPRB, 19.5646_JPRB, 19.1119_JPRB, 15.1646_JPRB, 15.5457_JPRB /)
!-- WVL=0.5320E-06
ZRALIS9( :, 8, 6) = (/ &
  &  6.5598_JPRB,  6.5597_JPRB,  6.5597_JPRB,  6.5597_JPRB, 10.9204_JPRB, 13.4828_JPRB, &
  & 15.8045_JPRB, 17.5253_JPRB, 19.2294_JPRB, 17.1914_JPRB, 14.7441_JPRB, 15.8247_JPRB /)
!-- WVL=0.5500E-06
ZRALIS9( :, 9, 6) = (/ &
  &  6.5188_JPRB,  6.5188_JPRB,  6.5188_JPRB,  6.5188_JPRB, 10.0953_JPRB, 12.5275_JPRB, &
  & 14.6640_JPRB, 16.8671_JPRB, 19.1357_JPRB, 15.9545_JPRB, 14.5629_JPRB, 15.8972_JPRB /)
!-- WVL=0.6450E-06
ZRALIS9( :,10, 6) = (/ &
  &  6.8169_JPRB,  6.8169_JPRB,  6.8169_JPRB,  6.8169_JPRB,  7.0672_JPRB,  8.1282_JPRB, &
  &  9.6896_JPRB, 11.8682_JPRB, 15.4964_JPRB, 17.8308_JPRB, 15.9312_JPRB, 14.8602_JPRB /)
!-- WVL=0.6700E-06
ZRALIS9( :,11, 6) = (/ &
  &  7.4410_JPRB,  7.4410_JPRB,  7.4410_JPRB,  7.4410_JPRB,  6.8014_JPRB,  7.5196_JPRB, &
  &  8.5212_JPRB, 10.7493_JPRB, 13.8522_JPRB, 18.4333_JPRB, 16.8990_JPRB, 14.5619_JPRB /)
!-- WVL=0.8000E-06
ZRALIS9( :,12, 6) = (/ &
  & 11.4679_JPRB, 11.4679_JPRB, 11.4679_JPRB, 11.4679_JPRB,  6.4137_JPRB,  6.6168_JPRB, &
  &  6.8354_JPRB,  7.1105_JPRB,  8.5127_JPRB, 17.2094_JPRB, 18.1700_JPRB, 16.1818_JPRB /)
!-- WVL=0.8580E-06
ZRALIS9( :,13, 6) = (/ &
  & 13.2417_JPRB, 13.2417_JPRB, 13.2417_JPRB, 13.2417_JPRB,  6.4965_JPRB,  6.4814_JPRB, &
  &  6.6109_JPRB,  6.7810_JPRB,  7.4953_JPRB, 16.9381_JPRB, 17.7287_JPRB, 16.8818_JPRB /)
!-- WVL=0.8650E-06
ZRALIS9( :,14, 6) = (/ &
  & 13.9211_JPRB, 13.9211_JPRB, 13.9211_JPRB, 13.9211_JPRB,  6.5098_JPRB,  6.4098_JPRB, &
  &  6.5369_JPRB,  6.8426_JPRB,  7.3785_JPRB, 17.3476_JPRB, 17.4854_JPRB, 16.7806_JPRB /)
!-- WVL=0.1020E-05
ZRALIS9( :,15, 6) = (/ &
  & 24.4311_JPRB, 24.4311_JPRB, 24.4311_JPRB, 24.4311_JPRB,  7.9758_JPRB,  6.9592_JPRB, &
  &  6.6508_JPRB,  6.5342_JPRB,  6.6349_JPRB, 14.2524_JPRB, 16.8896_JPRB, 16.8841_JPRB /)
!-- WVL=0.1064E-05
ZRALIS9( :,16, 6) = (/ &
  & 27.4447_JPRB, 27.4447_JPRB, 27.4447_JPRB, 27.4447_JPRB,  8.9608_JPRB,  7.5425_JPRB, &
  &  6.8165_JPRB,  6.5668_JPRB,  6.5391_JPRB,  9.0640_JPRB, 17.1272_JPRB, 17.2917_JPRB /)
!-- WVL=0.1240E-05
ZRALIS9( :,17, 6) = (/ &
  & 42.3020_JPRB, 42.3020_JPRB, 42.3020_JPRB, 42.3020_JPRB, 13.5587_JPRB, 11.3699_JPRB, &
  &  8.9754_JPRB,  7.5468_JPRB,  6.9779_JPRB, 10.6953_JPRB, 17.3616_JPRB, 17.7005_JPRB /)
!-- WVL=0.1640E-05
ZRALIS9( :,18, 6) = (/ &
  & 70.7952_JPRB, 70.7952_JPRB, 70.7952_JPRB, 70.7952_JPRB, 33.9639_JPRB, 26.7024_JPRB, &
  & 21.8948_JPRB, 17.3449_JPRB, 32.6654_JPRB, 30.1178_JPRB, 24.0900_JPRB, 19.4796_JPRB /)
!-- WVL=0.2130E-05
ZRALIS9( :,19, 6) = (/ &
  & 92.2893_JPRB, 92.2893_JPRB, 92.2893_JPRB, 92.2893_JPRB, 55.9528_JPRB, 51.6111_JPRB, &
  & 48.1981_JPRB, 36.4348_JPRB, 54.7342_JPRB, 49.0530_JPRB, 40.9951_JPRB, 28.7801_JPRB /)
  
 ! ** BIN No.            7    2.000000000000000         5.000000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS9( :, 1, 7) = (/ &
  & 0.10092_JPRB, 0.10092_JPRB, 0.10092_JPRB, 0.10092_JPRB, 0.20634_JPRB, 0.23905_JPRB, &
  & 0.27368_JPRB, 0.31883_JPRB, 0.38665_JPRB, 0.44382_JPRB, 0.54274_JPRB, 0.80030_JPRB /)
!-- WVL=0.3550E-06
ZALFSS9( :, 2, 7) = (/ &
  & 0.10118_JPRB, 0.10118_JPRB, 0.10118_JPRB, 0.10118_JPRB, 0.20593_JPRB, 0.24033_JPRB, &
  & 0.27411_JPRB, 0.31873_JPRB, 0.38758_JPRB, 0.44452_JPRB, 0.54287_JPRB, 0.80137_JPRB /)
!-- WVL=0.3800E-06
ZALFSS9( :, 3, 7) = (/ &
  & 0.10230_JPRB, 0.10230_JPRB, 0.10230_JPRB, 0.10230_JPRB, 0.20676_JPRB, 0.24033_JPRB, &
  & 0.27595_JPRB, 0.31961_JPRB, 0.38923_JPRB, 0.44530_JPRB, 0.54636_JPRB, 0.80412_JPRB /)
!-- WVL=0.4000E-06
ZALFSS9( :, 4, 7) = (/ &
  & 0.10277_JPRB, 0.10277_JPRB, 0.10277_JPRB, 0.10277_JPRB, 0.20739_JPRB, 0.24076_JPRB, &
  & 0.27623_JPRB, 0.32074_JPRB, 0.38942_JPRB, 0.44735_JPRB, 0.54627_JPRB, 0.80387_JPRB /)
!-- WVL=0.4400E-06
ZALFSS9( :, 5, 7) = (/ &
  & 0.10390_JPRB, 0.10390_JPRB, 0.10390_JPRB, 0.10390_JPRB, 0.20911_JPRB, 0.24263_JPRB, &
  & 0.27701_JPRB, 0.32188_JPRB, 0.39136_JPRB, 0.44814_JPRB, 0.54832_JPRB, 0.80720_JPRB /)
!-- WVL=0.4690E-06
ZALFSS9( :, 6, 7) = (/ &
  & 0.10326_JPRB, 0.10326_JPRB, 0.10326_JPRB, 0.10326_JPRB, 0.20947_JPRB, 0.24407_JPRB, &
  & 0.27837_JPRB, 0.32280_JPRB, 0.39366_JPRB, 0.44947_JPRB, 0.55132_JPRB, 0.81047_JPRB /)
!-- WVL=0.5000E-06
ZALFSS9( :, 7, 7) = (/ &
  & 0.10359_JPRB, 0.10359_JPRB, 0.10359_JPRB, 0.10359_JPRB, 0.20913_JPRB, 0.24431_JPRB, &
  & 0.28032_JPRB, 0.32405_JPRB, 0.39305_JPRB, 0.45237_JPRB, 0.55060_JPRB, 0.81001_JPRB /)
!-- WVL=0.5320E-06
ZALFSS9( :, 8, 7) = (/ &
  & 0.10403_JPRB, 0.10403_JPRB, 0.10403_JPRB, 0.10403_JPRB, 0.21119_JPRB, 0.24386_JPRB, &
  & 0.28044_JPRB, 0.32689_JPRB, 0.39472_JPRB, 0.45185_JPRB, 0.55259_JPRB, 0.81333_JPRB /)
!-- WVL=0.5500E-06
ZALFSS9( :, 9, 7) = (/ &
  & 0.10408_JPRB, 0.10408_JPRB, 0.10408_JPRB, 0.10408_JPRB, 0.21202_JPRB, 0.24443_JPRB, &
  & 0.28058_JPRB, 0.32673_JPRB, 0.39549_JPRB, 0.45352_JPRB, 0.55336_JPRB, 0.81478_JPRB /)
!-- WVL=0.6450E-06
ZALFSS9( :,10, 7) = (/ &
  & 0.10840_JPRB, 0.10840_JPRB, 0.10840_JPRB, 0.10840_JPRB, 0.21489_JPRB, 0.24993_JPRB, &
  & 0.28426_JPRB, 0.32810_JPRB, 0.39964_JPRB, 0.45509_JPRB, 0.55916_JPRB, 0.81967_JPRB /)
!-- WVL=0.6700E-06
ZALFSS9( :,11, 7) = (/ &
  & 0.10904_JPRB, 0.10904_JPRB, 0.10904_JPRB, 0.10904_JPRB, 0.21403_JPRB, 0.25107_JPRB, &
  & 0.28545_JPRB, 0.33036_JPRB, 0.39911_JPRB, 0.45788_JPRB, 0.55929_JPRB, 0.82093_JPRB /)
!-- WVL=0.8000E-06
ZALFSS9( :,12, 7) = (/ &
  & 0.11004_JPRB, 0.11004_JPRB, 0.11004_JPRB, 0.11004_JPRB, 0.21575_JPRB, 0.25106_JPRB, &
  & 0.28645_JPRB, 0.33612_JPRB, 0.40651_JPRB, 0.46755_JPRB, 0.56687_JPRB, 0.82972_JPRB /)
!-- WVL=0.8580E-06
ZALFSS9( :,13, 7) = (/ &
  & 0.11047_JPRB, 0.11047_JPRB, 0.11047_JPRB, 0.11047_JPRB, 0.21874_JPRB, 0.25159_JPRB, &
  & 0.28831_JPRB, 0.33397_JPRB, 0.41063_JPRB, 0.46476_JPRB, 0.57275_JPRB, 0.82847_JPRB /)
!-- WVL=0.8650E-06
ZALFSS9( :,14, 7) = (/ &
  & 0.11044_JPRB, 0.11044_JPRB, 0.11044_JPRB, 0.11044_JPRB, 0.21939_JPRB, 0.25172_JPRB, &
  & 0.28848_JPRB, 0.33409_JPRB, 0.41069_JPRB, 0.46444_JPRB, 0.57343_JPRB, 0.82943_JPRB /)
!-- WVL=0.1020E-05
ZALFSS9( :,15, 7) = (/ &
  & 0.10692_JPRB, 0.10692_JPRB, 0.10692_JPRB, 0.10692_JPRB, 0.22599_JPRB, 0.26284_JPRB, &
  & 0.29573_JPRB, 0.33772_JPRB, 0.41048_JPRB, 0.46892_JPRB, 0.57195_JPRB, 0.84761_JPRB /)
!-- WVL=0.1064E-05
ZALFSS9( :,16, 7) = (/ &
  & 0.10707_JPRB, 0.10707_JPRB, 0.10707_JPRB, 0.10707_JPRB, 0.22593_JPRB, 0.26368_JPRB, &
  & 0.30002_JPRB, 0.34074_JPRB, 0.41116_JPRB, 0.47065_JPRB, 0.57418_JPRB, 0.85124_JPRB /)
!-- WVL=0.1240E-05
ZALFSS9( :,17, 7) = (/ &
  & 0.11389_JPRB, 0.11389_JPRB, 0.11389_JPRB, 0.11389_JPRB, 0.22885_JPRB, 0.26589_JPRB, &
  & 0.30274_JPRB, 0.35312_JPRB, 0.42407_JPRB, 0.48568_JPRB, 0.58618_JPRB, 0.84890_JPRB /)
!-- WVL=0.1640E-05
ZALFSS9( :,18, 7) = (/ &
  & 0.12433_JPRB, 0.12433_JPRB, 0.12433_JPRB, 0.12433_JPRB, 0.22692_JPRB, 0.25830_JPRB, &
  & 0.29840_JPRB, 0.35410_JPRB, 0.42196_JPRB, 0.49319_JPRB, 0.60603_JPRB, 0.89009_JPRB /)
!-- WVL=0.2130E-05
ZALFSS9( :,19, 7) = (/ &
  & 0.14947_JPRB, 0.14947_JPRB, 0.14947_JPRB, 0.14947_JPRB, 0.25030_JPRB, 0.28620_JPRB, &
  & 0.32115_JPRB, 0.36042_JPRB, 0.44342_JPRB, 0.49246_JPRB, 0.58783_JPRB, 0.89947_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS9( :, 1, 7) = (/ &
  & 0.77752_JPRB, 0.77752_JPRB, 0.77752_JPRB, 0.77752_JPRB, 0.79333_JPRB, 0.79509_JPRB, &
  & 0.79726_JPRB, 0.79978_JPRB, 0.80156_JPRB, 0.80361_JPRB, 0.82258_JPRB, 0.82894_JPRB /)
!-- WVL=0.3550E-06
ZASYSS9( :, 2, 7) = (/ &
  & 0.77528_JPRB, 0.77528_JPRB, 0.77528_JPRB, 0.77528_JPRB, 0.79124_JPRB, 0.79439_JPRB, &
  & 0.79583_JPRB, 0.79817_JPRB, 0.80060_JPRB, 0.80257_JPRB, 0.82168_JPRB, 0.82785_JPRB /)
!-- WVL=0.3800E-06
ZASYSS9( :, 3, 7) = (/ &
  & 0.77322_JPRB, 0.77322_JPRB, 0.77322_JPRB, 0.77322_JPRB, 0.78909_JPRB, 0.79153_JPRB, &
  & 0.79446_JPRB, 0.79619_JPRB, 0.79936_JPRB, 0.80061_JPRB, 0.82066_JPRB, 0.82643_JPRB /)
!-- WVL=0.4000E-06
ZASYSS9( :, 4, 7) = (/ &
  & 0.77084_JPRB, 0.77084_JPRB, 0.77084_JPRB, 0.77084_JPRB, 0.78722_JPRB, 0.78987_JPRB, &
  & 0.79260_JPRB, 0.79488_JPRB, 0.79763_JPRB, 0.79996_JPRB, 0.81906_JPRB, 0.82539_JPRB /)
!-- WVL=0.4400E-06
ZASYSS9( :, 5, 7) = (/ &
  & 0.76720_JPRB, 0.76720_JPRB, 0.76720_JPRB, 0.76720_JPRB, 0.78407_JPRB, 0.78706_JPRB, &
  & 0.78907_JPRB, 0.79152_JPRB, 0.79496_JPRB, 0.79686_JPRB, 0.81659_JPRB, 0.82323_JPRB /)
!-- WVL=0.4690E-06
ZASYSS9( :, 6, 7) = (/ &
  & 0.76118_JPRB, 0.76118_JPRB, 0.76118_JPRB, 0.76118_JPRB, 0.78065_JPRB, 0.78486_JPRB, &
  & 0.78704_JPRB, 0.78941_JPRB, 0.79338_JPRB, 0.79504_JPRB, 0.81673_JPRB, 0.82209_JPRB /)
!-- WVL=0.5000E-06
ZASYSS9( :, 7, 7) = (/ &
  & 0.75780_JPRB, 0.75780_JPRB, 0.75780_JPRB, 0.75780_JPRB, 0.77617_JPRB, 0.78129_JPRB, &
  & 0.78513_JPRB, 0.78722_JPRB, 0.79040_JPRB, 0.79366_JPRB, 0.81446_JPRB, 0.82008_JPRB /)
!-- WVL=0.5320E-06
ZASYSS9( :, 8, 7) = (/ &
  & 0.75506_JPRB, 0.75506_JPRB, 0.75506_JPRB, 0.75506_JPRB, 0.77426_JPRB, 0.77712_JPRB, &
  & 0.78173_JPRB, 0.78600_JPRB, 0.78846_JPRB, 0.79471_JPRB, 0.81289_JPRB, 0.81840_JPRB /)
!-- WVL=0.5500E-06
ZASYSS9( :, 9, 7) = (/ &
  & 0.75298_JPRB, 0.75298_JPRB, 0.75298_JPRB, 0.75298_JPRB, 0.77290_JPRB, 0.77535_JPRB, &
  & 0.77986_JPRB, 0.78407_JPRB, 0.78729_JPRB, 0.80011_JPRB, 0.81181_JPRB, 0.81781_JPRB /)
!-- WVL=0.6450E-06
ZASYSS9( :,10, 7) = (/ &
  & 0.75032_JPRB, 0.75032_JPRB, 0.75032_JPRB, 0.75032_JPRB, 0.76551_JPRB, 0.76998_JPRB, &
  & 0.77222_JPRB, 0.77508_JPRB, 0.78084_JPRB, 0.79922_JPRB, 0.80628_JPRB, 0.81300_JPRB /)
!-- WVL=0.6700E-06
ZASYSS9( :,11, 7) = (/ &
  & 0.74795_JPRB, 0.74795_JPRB, 0.74795_JPRB, 0.74795_JPRB, 0.76188_JPRB, 0.76855_JPRB, &
  & 0.77055_JPRB, 0.77411_JPRB, 0.77813_JPRB, 0.79842_JPRB, 0.80361_JPRB, 0.81182_JPRB /)
!-- WVL=0.8000E-06
ZASYSS9( :,12, 7) = (/ &
  & 0.72927_JPRB, 0.72927_JPRB, 0.72927_JPRB, 0.72927_JPRB, 0.75227_JPRB, 0.75686_JPRB, &
  & 0.75969_JPRB, 0.76603_JPRB, 0.77054_JPRB, 0.79310_JPRB, 0.79586_JPRB, 0.80550_JPRB /)
!-- WVL=0.8580E-06
ZASYSS9( :,13, 7) = (/ &
  & 0.72113_JPRB, 0.72113_JPRB, 0.72113_JPRB, 0.72113_JPRB, 0.75062_JPRB, 0.75276_JPRB, &
  & 0.75685_JPRB, 0.76016_JPRB, 0.76843_JPRB, 0.78793_JPRB, 0.79428_JPRB, 0.80154_JPRB /)
!-- WVL=0.8650E-06
ZASYSS9( :,14, 7) = (/ &
  & 0.71963_JPRB, 0.71963_JPRB, 0.71963_JPRB, 0.71963_JPRB, 0.75038_JPRB, 0.75219_JPRB, &
  & 0.75661_JPRB, 0.75964_JPRB, 0.76794_JPRB, 0.78723_JPRB, 0.79404_JPRB, 0.80127_JPRB /)
!-- WVL=0.1020E-05
ZASYSS9( :,15, 7) = (/ &
  & 0.68062_JPRB, 0.68062_JPRB, 0.68062_JPRB, 0.68062_JPRB, 0.74221_JPRB, 0.74972_JPRB, &
  & 0.75091_JPRB, 0.75118_JPRB, 0.75689_JPRB, 0.77591_JPRB, 0.78463_JPRB, 0.79684_JPRB /)
!-- WVL=0.1064E-05
ZASYSS9( :,16, 7) = (/ &
  & 0.67324_JPRB, 0.67324_JPRB, 0.67324_JPRB, 0.67324_JPRB, 0.73717_JPRB, 0.74601_JPRB, &
  & 0.75078_JPRB, 0.75076_JPRB, 0.75470_JPRB, 0.76534_JPRB, 0.78294_JPRB, 0.79536_JPRB /)
!-- WVL=0.1240E-05
ZASYSS9( :,17, 7) = (/ &
  & 0.65827_JPRB, 0.65827_JPRB, 0.65827_JPRB, 0.65827_JPRB, 0.72039_JPRB, 0.72954_JPRB, &
  & 0.73658_JPRB, 0.74560_JPRB, 0.75051_JPRB, 0.76555_JPRB, 0.77733_JPRB, 0.78690_JPRB /)
!-- WVL=0.1640E-05
ZASYSS9( :,18, 7) = (/ &
  & 0.62720_JPRB, 0.62720_JPRB, 0.62720_JPRB, 0.62720_JPRB, 0.66395_JPRB, 0.67459_JPRB, &
  & 0.68904_JPRB, 0.70898_JPRB, 0.72869_JPRB, 0.74340_JPRB, 0.75739_JPRB, 0.77731_JPRB /)
!-- WVL=0.2130E-05
ZASYSS9( :,19, 7) = (/ &
  & 0.66607_JPRB, 0.66607_JPRB, 0.66607_JPRB, 0.66607_JPRB, 0.63660_JPRB, 0.64715_JPRB, &
  & 0.65559_JPRB, 0.66109_JPRB, 0.69424_JPRB, 0.69730_JPRB, 0.70960_JPRB, 0.75099_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS9( :, 1, 7) = (/ &
  & 0.98538_JPRB, 0.98538_JPRB, 0.98538_JPRB, 0.98538_JPRB, 0.98107_JPRB, 0.98080_JPRB, &
  & 0.98071_JPRB, 0.98077_JPRB, 0.98083_JPRB, 0.98101_JPRB, 0.99166_JPRB, 0.99302_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS9( :, 2, 7) = (/ &
  & 0.98599_JPRB, 0.98600_JPRB, 0.98600_JPRB, 0.98600_JPRB, 0.98119_JPRB, 0.98095_JPRB, &
  & 0.98075_JPRB, 0.98071_JPRB, 0.98078_JPRB, 0.98091_JPRB, 0.99163_JPRB, 0.99297_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS9( :, 3, 7) = (/ &
  & 0.98691_JPRB, 0.98691_JPRB, 0.98691_JPRB, 0.98691_JPRB, 0.98167_JPRB, 0.98116_JPRB, &
  & 0.98094_JPRB, 0.98074_JPRB, 0.98075_JPRB, 0.98078_JPRB, 0.99164_JPRB, 0.99294_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS9( :, 4, 7) = (/ &
  & 0.98749_JPRB, 0.98749_JPRB, 0.98749_JPRB, 0.98749_JPRB, 0.98220_JPRB, 0.98146_JPRB, &
  & 0.98110_JPRB, 0.98085_JPRB, 0.98071_JPRB, 0.98079_JPRB, 0.99163_JPRB, 0.99291_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS9( :, 5, 7) = (/ &
  & 0.98848_JPRB, 0.98848_JPRB, 0.98848_JPRB, 0.98848_JPRB, 0.98368_JPRB, 0.98245_JPRB, &
  & 0.98167_JPRB, 0.98116_JPRB, 0.98084_JPRB, 0.98072_JPRB, 0.99170_JPRB, 0.99288_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS9( :, 6, 7) = (/ &
  & 0.98894_JPRB, 0.98894_JPRB, 0.98894_JPRB, 0.98894_JPRB, 0.98478_JPRB, 0.98347_JPRB, &
  & 0.98237_JPRB, 0.98156_JPRB, 0.98106_JPRB, 0.98082_JPRB, 0.99277_JPRB, 0.99290_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS9( :, 7, 7) = (/ &
  & 0.98949_JPRB, 0.98949_JPRB, 0.98949_JPRB, 0.98949_JPRB, 0.98567_JPRB, 0.98455_JPRB, &
  & 0.98338_JPRB, 0.98220_JPRB, 0.98130_JPRB, 0.98105_JPRB, 0.99321_JPRB, 0.99292_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS9( :, 8, 7) = (/ &
  & 0.99003_JPRB, 0.99003_JPRB, 0.99003_JPRB, 0.99003_JPRB, 0.98658_JPRB, 0.98547_JPRB, &
  & 0.98444_JPRB, 0.98316_JPRB, 0.98179_JPRB, 0.98377_JPRB, 0.99345_JPRB, 0.99295_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS9( :, 9, 7) = (/ &
  & 0.99026_JPRB, 0.99026_JPRB, 0.99026_JPRB, 0.99026_JPRB, 0.98698_JPRB, 0.98592_JPRB, &
  & 0.98497_JPRB, 0.98368_JPRB, 0.98214_JPRB, 0.98812_JPRB, 0.99358_JPRB, 0.99303_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS9( :,10, 7) = (/ &
  & 0.99169_JPRB, 0.99169_JPRB, 0.99170_JPRB, 0.99169_JPRB, 0.98862_JPRB, 0.98791_JPRB, &
  & 0.98713_JPRB, 0.98614_JPRB, 0.98470_JPRB, 0.99409_JPRB, 0.99445_JPRB, 0.99343_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS9( :,11, 7) = (/ &
  & 0.99197_JPRB, 0.99197_JPRB, 0.99197_JPRB, 0.99197_JPRB, 0.98889_JPRB, 0.98832_JPRB, &
  & 0.98755_JPRB, 0.98666_JPRB, 0.98527_JPRB, 0.99432_JPRB, 0.99414_JPRB, 0.99360_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS9( :,12, 7) = (/ &
  & 0.99302_JPRB, 0.99302_JPRB, 0.99302_JPRB, 0.99302_JPRB, 0.99035_JPRB, 0.98975_JPRB, &
  & 0.98912_JPRB, 0.98855_JPRB, 0.98755_JPRB, 0.99518_JPRB, 0.99469_JPRB, 0.99468_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS9( :,13, 7) = (/ &
  & 0.99346_JPRB, 0.99346_JPRB, 0.99346_JPRB, 0.99346_JPRB, 0.99097_JPRB, 0.99030_JPRB, &
  & 0.98977_JPRB, 0.98909_JPRB, 0.98832_JPRB, 0.99539_JPRB, 0.99503_JPRB, 0.99502_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS9( :,14, 7) = (/ &
  & 0.99349_JPRB, 0.99349_JPRB, 0.99349_JPRB, 0.99349_JPRB, 0.99107_JPRB, 0.99037_JPRB, &
  & 0.98983_JPRB, 0.98915_JPRB, 0.98840_JPRB, 0.99542_JPRB, 0.99507_JPRB, 0.99504_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS9( :,15, 7) = (/ &
  & 0.99424_JPRB, 0.99424_JPRB, 0.99424_JPRB, 0.99424_JPRB, 0.99230_JPRB, 0.99183_JPRB, &
  & 0.99123_JPRB, 0.99050_JPRB, 0.98971_JPRB, 0.99463_JPRB, 0.99567_JPRB, 0.99576_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS9( :,16, 7) = (/ &
  & 0.99450_JPRB, 0.99450_JPRB, 0.99450_JPRB, 0.99450_JPRB, 0.99254_JPRB, 0.99210_JPRB, &
  & 0.99163_JPRB, 0.99090_JPRB, 0.99008_JPRB, 0.99189_JPRB, 0.99584_JPRB, 0.99592_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS9( :,17, 7) = (/ &
  & 0.99559_JPRB, 0.99559_JPRB, 0.99559_JPRB, 0.99559_JPRB, 0.99347_JPRB, 0.99301_JPRB, &
  & 0.99257_JPRB, 0.99215_JPRB, 0.99141_JPRB, 0.99413_JPRB, 0.99637_JPRB, 0.99640_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS9( :,18, 7) = (/ &
  & 0.99697_JPRB, 0.99697_JPRB, 0.99697_JPRB, 0.99697_JPRB, 0.99499_JPRB, 0.99447_JPRB, &
  & 0.99409_JPRB, 0.99375_JPRB, 0.99761_JPRB, 0.99748_JPRB, 0.99722_JPRB, 0.99724_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS9( :,19, 7) = (/ &
  & 0.99811_JPRB, 0.99811_JPRB, 0.99811_JPRB, 0.99811_JPRB, 0.99653_JPRB, 0.99617_JPRB, &
  & 0.99579_JPRB, 0.99524_JPRB, 0.99827_JPRB, 0.99807_JPRB, 0.99780_JPRB, 0.99786_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS9( :, 1, 7) = (/ &
  & 15.0493_JPRB, 15.0492_JPRB, 15.0492_JPRB, 15.0492_JPRB, 15.3823_JPRB, 16.8445_JPRB, &
  & 17.2672_JPRB, 17.2023_JPRB, 17.7825_JPRB, 18.8355_JPRB,  9.0117_JPRB,  7.1739_JPRB /)
!-- WVL=0.3550E-06
ZRALIS9( :, 2, 7) = (/ &
  & 14.8754_JPRB, 14.8753_JPRB, 14.8753_JPRB, 14.8753_JPRB, 14.7656_JPRB, 15.9453_JPRB, &
  & 17.1934_JPRB, 17.0106_JPRB, 17.3602_JPRB, 18.2291_JPRB,  9.0608_JPRB,  7.0028_JPRB /)
!-- WVL=0.3800E-06
ZRALIS9( :, 3, 7) = (/ &
  & 14.5440_JPRB, 14.5440_JPRB, 14.5440_JPRB, 14.5440_JPRB, 14.5614_JPRB, 14.8565_JPRB, &
  & 15.9850_JPRB, 17.2781_JPRB, 17.2617_JPRB, 17.3507_JPRB,  9.0243_JPRB,  7.3922_JPRB /)
!-- WVL=0.4000E-06
ZRALIS9( :, 4, 7) = (/ &
  & 14.2705_JPRB, 14.2705_JPRB, 14.2705_JPRB, 14.2705_JPRB, 14.6657_JPRB, 14.5895_JPRB, &
  & 15.1512_JPRB, 16.5682_JPRB, 17.2473_JPRB, 17.2029_JPRB,  9.7012_JPRB,  8.4731_JPRB /)
!-- WVL=0.4400E-06
ZRALIS9( :, 5, 7) = (/ &
  & 14.5769_JPRB, 14.5769_JPRB, 14.5769_JPRB, 14.5769_JPRB, 15.5970_JPRB, 15.0210_JPRB, &
  & 14.5636_JPRB, 14.8583_JPRB, 16.6493_JPRB, 17.2461_JPRB,  9.6371_JPRB,  8.0082_JPRB /)
!-- WVL=0.4690E-06
ZRALIS9( :, 6, 7) = (/ &
  & 15.1983_JPRB, 15.1983_JPRB, 15.1983_JPRB, 15.1983_JPRB, 15.3637_JPRB, 15.5335_JPRB, &
  & 14.8397_JPRB, 14.5358_JPRB, 15.3945_JPRB, 16.7647_JPRB, 10.2689_JPRB,  8.9815_JPRB /)
!-- WVL=0.5000E-06
ZRALIS9( :, 7, 7) = (/ &
  & 16.1805_JPRB, 16.1804_JPRB, 16.1805_JPRB, 16.1804_JPRB, 14.8923_JPRB, 15.4144_JPRB, &
  & 15.5163_JPRB, 14.6657_JPRB, 14.6562_JPRB, 15.5015_JPRB, 10.6220_JPRB,  9.1943_JPRB /)
!-- WVL=0.5320E-06
ZRALIS9( :, 8, 7) = (/ &
  & 16.9737_JPRB, 16.9736_JPRB, 16.9737_JPRB, 16.9736_JPRB, 14.7247_JPRB, 14.9856_JPRB, &
  & 15.5137_JPRB, 15.5669_JPRB, 14.5722_JPRB, 16.1406_JPRB, 10.9260_JPRB,  9.1622_JPRB /)
!-- WVL=0.5500E-06
ZRALIS9( :, 9, 7) = (/ &
  & 16.9714_JPRB, 16.9713_JPRB, 16.9713_JPRB, 16.9713_JPRB, 14.5165_JPRB, 14.8400_JPRB, &
  & 15.2907_JPRB, 15.5971_JPRB, 14.7074_JPRB, 15.2221_JPRB, 11.0516_JPRB,  9.6942_JPRB /)
!-- WVL=0.6450E-06
ZRALIS9( :,10, 7) = (/ &
  & 15.5325_JPRB, 15.5325_JPRB, 15.5325_JPRB, 15.5325_JPRB, 14.5546_JPRB, 14.2466_JPRB, &
  & 14.4263_JPRB, 14.8104_JPRB, 15.3864_JPRB, 12.9544_JPRB, 12.1328_JPRB, 10.8964_JPRB /)
!-- WVL=0.6700E-06
ZRALIS9( :,11, 7) = (/ &
  & 14.7262_JPRB, 14.7263_JPRB, 14.7262_JPRB, 14.7263_JPRB, 15.1374_JPRB, 14.4307_JPRB, &
  & 14.2355_JPRB, 14.6594_JPRB, 15.0811_JPRB, 12.9226_JPRB, 12.7090_JPRB, 11.0753_JPRB /)
!-- WVL=0.8000E-06
ZRALIS9( :,12, 7) = (/ &
  & 11.3563_JPRB, 11.3563_JPRB, 11.3563_JPRB, 11.3563_JPRB, 17.0390_JPRB, 16.6376_JPRB, &
  & 15.5111_JPRB, 14.5636_JPRB, 14.2232_JPRB, 13.8301_JPRB, 12.8486_JPRB, 12.3626_JPRB /)
!-- WVL=0.8580E-06
ZRALIS9( :,13, 7) = (/ &
  & 10.2749_JPRB, 10.2749_JPRB, 10.2750_JPRB, 10.2749_JPRB, 16.6213_JPRB, 16.9910_JPRB, &
  & 16.6725_JPRB, 15.3587_JPRB, 14.4208_JPRB, 14.7162_JPRB, 13.3406_JPRB, 12.9716_JPRB /)
!-- WVL=0.8650E-06
ZRALIS9( :,14, 7) = (/ &
  & 10.0676_JPRB, 10.0676_JPRB, 10.0676_JPRB, 10.0676_JPRB, 16.5414_JPRB, 17.0503_JPRB, &
  & 16.8376_JPRB, 15.6103_JPRB, 14.4587_JPRB, 15.0104_JPRB, 13.2846_JPRB, 12.9209_JPRB /)
!-- WVL=0.1020E-05
ZRALIS9( :,15, 7) = (/ &
  &  7.5608_JPRB,  7.5608_JPRB,  7.5608_JPRB,  7.5608_JPRB, 13.7777_JPRB, 15.4572_JPRB, &
  & 16.2802_JPRB, 16.7853_JPRB, 16.4716_JPRB, 15.2106_JPRB, 15.6660_JPRB, 13.1074_JPRB /)
!-- WVL=0.1064E-05
ZRALIS9( :,16, 7) = (/ &
  &  7.2673_JPRB,  7.2673_JPRB,  7.2673_JPRB,  7.2673_JPRB, 12.8823_JPRB, 14.4005_JPRB, &
  & 15.7634_JPRB, 16.5136_JPRB, 16.9616_JPRB, 14.9057_JPRB, 15.8640_JPRB, 13.3585_JPRB /)
!-- WVL=0.1240E-05
ZRALIS9( :,17, 7) = (/ &
  &  7.0996_JPRB,  7.0996_JPRB,  7.0996_JPRB,  7.0996_JPRB, 10.1680_JPRB, 11.3906_JPRB, &
  & 12.8840_JPRB, 14.3964_JPRB, 15.5492_JPRB, 15.0320_JPRB, 15.1170_JPRB, 15.4260_JPRB /)
!-- WVL=0.1640E-05
ZRALIS9( :,18, 7) = (/ &
  &  9.0169_JPRB,  9.0169_JPRB,  9.0169_JPRB,  9.0169_JPRB,  7.0662_JPRB,  7.3221_JPRB, &
  &  7.9689_JPRB,  9.0844_JPRB, 17.4485_JPRB, 17.4099_JPRB, 16.9250_JPRB, 15.6653_JPRB /)
!-- WVL=0.2130E-05
ZRALIS9( :,19, 7) = (/ &
  & 16.7800_JPRB, 16.7800_JPRB, 16.7800_JPRB, 16.7800_JPRB,  7.9431_JPRB,  7.5023_JPRB, &
  &  7.2297_JPRB,  7.0407_JPRB, 17.7942_JPRB, 17.2465_JPRB, 17.2474_JPRB, 17.1285_JPRB /)
  
 ! ** BIN No.            8    5.000000000000000         10.00000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS9( :, 1, 8) = (/ &
  & 0.04849_JPRB, 0.04849_JPRB, 0.04849_JPRB, 0.04849_JPRB, 0.09902_JPRB, 0.11546_JPRB, &
  & 0.13208_JPRB, 0.15383_JPRB, 0.18737_JPRB, 0.21503_JPRB, 0.26334_JPRB, 0.38875_JPRB /)
!-- WVL=0.3550E-06
ZALFSS9( :, 2, 8) = (/ &
  & 0.04855_JPRB, 0.04855_JPRB, 0.04855_JPRB, 0.04855_JPRB, 0.09938_JPRB, 0.11543_JPRB, &
  & 0.13237_JPRB, 0.15421_JPRB, 0.18757_JPRB, 0.21503_JPRB, 0.26404_JPRB, 0.38879_JPRB /)
!-- WVL=0.3800E-06
ZALFSS9( :, 3, 8) = (/ &
  & 0.04861_JPRB, 0.04861_JPRB, 0.04861_JPRB, 0.04861_JPRB, 0.09929_JPRB, 0.11586_JPRB, &
  & 0.13256_JPRB, 0.15425_JPRB, 0.18797_JPRB, 0.21557_JPRB, 0.26398_JPRB, 0.38961_JPRB /)
!-- WVL=0.4000E-06
ZALFSS9( :, 4, 8) = (/ &
  & 0.04888_JPRB, 0.04888_JPRB, 0.04888_JPRB, 0.04888_JPRB, 0.09977_JPRB, 0.11584_JPRB, &
  & 0.13283_JPRB, 0.15465_JPRB, 0.18821_JPRB, 0.21575_JPRB, 0.26463_JPRB, 0.39146_JPRB /)
!-- WVL=0.4400E-06
ZALFSS9( :, 5, 8) = (/ &
  & 0.04882_JPRB, 0.04882_JPRB, 0.04882_JPRB, 0.04882_JPRB, 0.10009_JPRB, 0.11634_JPRB, &
  & 0.13303_JPRB, 0.15518_JPRB, 0.18883_JPRB, 0.21613_JPRB, 0.26458_JPRB, 0.39145_JPRB /)
!-- WVL=0.4690E-06
ZALFSS9( :, 6, 8) = (/ &
  & 0.04933_JPRB, 0.04933_JPRB, 0.04933_JPRB, 0.04933_JPRB, 0.10006_JPRB, 0.11661_JPRB, &
  & 0.13365_JPRB, 0.15509_JPRB, 0.18890_JPRB, 0.21699_JPRB, 0.26554_JPRB, 0.39142_JPRB /)
!-- WVL=0.5000E-06
ZALFSS9( :, 7, 8) = (/ &
  & 0.04916_JPRB, 0.04916_JPRB, 0.04916_JPRB, 0.04916_JPRB, 0.10059_JPRB, 0.11671_JPRB, &
  & 0.13384_JPRB, 0.15590_JPRB, 0.18965_JPRB, 0.21703_JPRB, 0.26625_JPRB, 0.39219_JPRB /)
!-- WVL=0.5320E-06
ZALFSS9( :, 8, 8) = (/ &
  & 0.04931_JPRB, 0.04931_JPRB, 0.04931_JPRB, 0.04931_JPRB, 0.10055_JPRB, 0.11726_JPRB, &
  & 0.13405_JPRB, 0.15584_JPRB, 0.18967_JPRB, 0.21745_JPRB, 0.26592_JPRB, 0.39310_JPRB /)
!-- WVL=0.5500E-06
ZALFSS9( :, 9, 8) = (/ &
  & 0.04955_JPRB, 0.04955_JPRB, 0.04955_JPRB, 0.04955_JPRB, 0.10076_JPRB, 0.11737_JPRB, &
  & 0.13416_JPRB, 0.15639_JPRB, 0.19032_JPRB, 0.21817_JPRB, 0.26673_JPRB, 0.39435_JPRB /)
!-- WVL=0.6450E-06
ZALFSS9( :,10, 8) = (/ &
  & 0.04967_JPRB, 0.04967_JPRB, 0.04967_JPRB, 0.04967_JPRB, 0.10146_JPRB, 0.11812_JPRB, &
  & 0.13521_JPRB, 0.15715_JPRB, 0.19085_JPRB, 0.21933_JPRB, 0.26779_JPRB, 0.39450_JPRB /)
!-- WVL=0.6700E-06
ZALFSS9( :,11, 8) = (/ &
  & 0.04987_JPRB, 0.04987_JPRB, 0.04987_JPRB, 0.04987_JPRB, 0.10215_JPRB, 0.11785_JPRB, &
  & 0.13573_JPRB, 0.15712_JPRB, 0.19147_JPRB, 0.21914_JPRB, 0.26805_JPRB, 0.39572_JPRB /)
!-- WVL=0.8000E-06
ZALFSS9( :,12, 8) = (/ &
  & 0.05053_JPRB, 0.05053_JPRB, 0.05053_JPRB, 0.05053_JPRB, 0.10286_JPRB, 0.11883_JPRB, &
  & 0.13690_JPRB, 0.15830_JPRB, 0.19329_JPRB, 0.22036_JPRB, 0.27070_JPRB, 0.39707_JPRB /)
!-- WVL=0.8580E-06
ZALFSS9( :,13, 8) = (/ &
  & 0.05039_JPRB, 0.05039_JPRB, 0.05039_JPRB, 0.05039_JPRB, 0.10344_JPRB, 0.11982_JPRB, &
  & 0.13647_JPRB, 0.15973_JPRB, 0.19271_JPRB, 0.22246_JPRB, 0.27083_JPRB, 0.39980_JPRB /)
!-- WVL=0.8650E-06
ZALFSS9( :,14, 8) = (/ &
  & 0.05043_JPRB, 0.05043_JPRB, 0.05043_JPRB, 0.05043_JPRB, 0.10333_JPRB, 0.12002_JPRB, &
  & 0.13655_JPRB, 0.15966_JPRB, 0.19279_JPRB, 0.22269_JPRB, 0.27069_JPRB, 0.39962_JPRB /)
!-- WVL=0.1020E-05
ZALFSS9( :,15, 8) = (/ &
  & 0.05251_JPRB, 0.05251_JPRB, 0.05251_JPRB, 0.05251_JPRB, 0.10455_JPRB, 0.12023_JPRB, &
  & 0.13808_JPRB, 0.16126_JPRB, 0.19428_JPRB, 0.22303_JPRB, 0.27255_JPRB, 0.40188_JPRB /)
!-- WVL=0.1064E-05
ZALFSS9( :,16, 8) = (/ &
  & 0.05267_JPRB, 0.05267_JPRB, 0.05267_JPRB, 0.05267_JPRB, 0.10547_JPRB, 0.12093_JPRB, &
  & 0.13778_JPRB, 0.16176_JPRB, 0.19504_JPRB, 0.22313_JPRB, 0.27271_JPRB, 0.40125_JPRB /)
!-- WVL=0.1240E-05
ZALFSS9( :,17, 8) = (/ &
  & 0.05150_JPRB, 0.05150_JPRB, 0.05150_JPRB, 0.05150_JPRB, 0.10444_JPRB, 0.12217_JPRB, &
  & 0.14136_JPRB, 0.16223_JPRB, 0.19624_JPRB, 0.22452_JPRB, 0.27575_JPRB, 0.40374_JPRB /)
!-- WVL=0.1640E-05
ZALFSS9( :,18, 8) = (/ &
  & 0.05620_JPRB, 0.05620_JPRB, 0.05620_JPRB, 0.05620_JPRB, 0.10769_JPRB, 0.12745_JPRB, &
  & 0.14430_JPRB, 0.16434_JPRB, 0.20122_JPRB, 0.22768_JPRB, 0.27942_JPRB, 0.40910_JPRB /)
!-- WVL=0.2130E-05
ZALFSS9( :,19, 8) = (/ &
  & 0.05393_JPRB, 0.05393_JPRB, 0.05393_JPRB, 0.05393_JPRB, 0.11028_JPRB, 0.12539_JPRB, &
  & 0.14290_JPRB, 0.16705_JPRB, 0.20129_JPRB, 0.23299_JPRB, 0.29087_JPRB, 0.41370_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS9( :, 1, 8) = (/ &
  & 0.80316_JPRB, 0.80315_JPRB, 0.80315_JPRB, 0.80315_JPRB, 0.80940_JPRB, 0.81110_JPRB, &
  & 0.81179_JPRB, 0.81293_JPRB, 0.81446_JPRB, 0.81528_JPRB, 0.83459_JPRB, 0.83804_JPRB /)
!-- WVL=0.3550E-06
ZASYSS9( :, 2, 8) = (/ &
  & 0.80213_JPRB, 0.80213_JPRB, 0.80213_JPRB, 0.80213_JPRB, 0.80903_JPRB, 0.81008_JPRB, &
  & 0.81136_JPRB, 0.81256_JPRB, 0.81390_JPRB, 0.81463_JPRB, 0.83381_JPRB, 0.83793_JPRB /)
!-- WVL=0.3800E-06
ZASYSS9( :, 3, 8) = (/ &
  & 0.80022_JPRB, 0.80022_JPRB, 0.80022_JPRB, 0.80022_JPRB, 0.80730_JPRB, 0.80917_JPRB, &
  & 0.81010_JPRB, 0.81136_JPRB, 0.81287_JPRB, 0.81393_JPRB, 0.83269_JPRB, 0.83757_JPRB /)
!-- WVL=0.4000E-06
ZASYSS9( :, 4, 8) = (/ &
  & 0.79960_JPRB, 0.79960_JPRB, 0.79960_JPRB, 0.79960_JPRB, 0.80696_JPRB, 0.80789_JPRB, &
  & 0.80921_JPRB, 0.81069_JPRB, 0.81213_JPRB, 0.81300_JPRB, 0.83215_JPRB, 0.83689_JPRB /)
!-- WVL=0.4400E-06
ZASYSS9( :, 5, 8) = (/ &
  & 0.79576_JPRB, 0.79576_JPRB, 0.79576_JPRB, 0.79576_JPRB, 0.80534_JPRB, 0.80642_JPRB, &
  & 0.80730_JPRB, 0.80914_JPRB, 0.81083_JPRB, 0.81156_JPRB, 0.83073_JPRB, 0.83540_JPRB /)
!-- WVL=0.4690E-06
ZASYSS9( :, 6, 8) = (/ &
  & 0.79536_JPRB, 0.79536_JPRB, 0.79536_JPRB, 0.79536_JPRB, 0.80355_JPRB, 0.80542_JPRB, &
  & 0.80660_JPRB, 0.80753_JPRB, 0.80943_JPRB, 0.81096_JPRB, 0.83118_JPRB, 0.83446_JPRB /)
!-- WVL=0.5000E-06
ZASYSS9( :, 7, 8) = (/ &
  & 0.79203_JPRB, 0.79203_JPRB, 0.79203_JPRB, 0.79203_JPRB, 0.80280_JPRB, 0.80391_JPRB, &
  & 0.80538_JPRB, 0.80696_JPRB, 0.80864_JPRB, 0.80950_JPRB, 0.83094_JPRB, 0.83384_JPRB /)
!-- WVL=0.5320E-06
ZASYSS9( :, 8, 8) = (/ &
  & 0.78949_JPRB, 0.78949_JPRB, 0.78949_JPRB, 0.78949_JPRB, 0.80081_JPRB, 0.80309_JPRB, &
  & 0.80413_JPRB, 0.80537_JPRB, 0.80723_JPRB, 0.81190_JPRB, 0.82974_JPRB, 0.83337_JPRB /)
!-- WVL=0.5500E-06
ZASYSS9( :, 9, 8) = (/ &
  & 0.78894_JPRB, 0.78894_JPRB, 0.78894_JPRB, 0.78894_JPRB, 0.80016_JPRB, 0.80224_JPRB, &
  & 0.80342_JPRB, 0.80534_JPRB, 0.80708_JPRB, 0.81821_JPRB, 0.82961_JPRB, 0.83281_JPRB /)
!-- WVL=0.6450E-06
ZASYSS9( :,10, 8) = (/ &
  & 0.78114_JPRB, 0.78114_JPRB, 0.78114_JPRB, 0.78114_JPRB, 0.79557_JPRB, 0.79821_JPRB, &
  & 0.80000_JPRB, 0.80182_JPRB, 0.80367_JPRB, 0.82290_JPRB, 0.82605_JPRB, 0.82980_JPRB /)
!-- WVL=0.6700E-06
ZASYSS9( :,11, 8) = (/ &
  & 0.77975_JPRB, 0.77975_JPRB, 0.77975_JPRB, 0.77975_JPRB, 0.79557_JPRB, 0.79629_JPRB, &
  & 0.79943_JPRB, 0.80062_JPRB, 0.80321_JPRB, 0.82146_JPRB, 0.82428_JPRB, 0.82958_JPRB /)
!-- WVL=0.8000E-06
ZASYSS9( :,12, 8) = (/ &
  & 0.77215_JPRB, 0.77215_JPRB, 0.77215_JPRB, 0.77215_JPRB, 0.78882_JPRB, 0.79062_JPRB, &
  & 0.79438_JPRB, 0.79573_JPRB, 0.79942_JPRB, 0.81694_JPRB, 0.82023_JPRB, 0.82582_JPRB /)
!-- WVL=0.8580E-06
ZASYSS9( :,13, 8) = (/ &
  & 0.76653_JPRB, 0.76653_JPRB, 0.76653_JPRB, 0.76653_JPRB, 0.78627_JPRB, 0.78896_JPRB, &
  & 0.79059_JPRB, 0.79475_JPRB, 0.79625_JPRB, 0.81646_JPRB, 0.81823_JPRB, 0.82506_JPRB /)
!-- WVL=0.8650E-06
ZASYSS9( :,14, 8) = (/ &
  & 0.76596_JPRB, 0.76596_JPRB, 0.76596_JPRB, 0.76596_JPRB, 0.78575_JPRB, 0.78879_JPRB, &
  & 0.79026_JPRB, 0.79429_JPRB, 0.79600_JPRB, 0.81623_JPRB, 0.81788_JPRB, 0.82478_JPRB /)
!-- WVL=0.1020E-05
ZASYSS9( :,15, 8) = (/ &
  & 0.76238_JPRB, 0.76238_JPRB, 0.76238_JPRB, 0.76238_JPRB, 0.77982_JPRB, 0.78043_JPRB, &
  & 0.78395_JPRB, 0.78862_JPRB, 0.79079_JPRB, 0.80649_JPRB, 0.81307_JPRB, 0.82067_JPRB /)
!-- WVL=0.1064E-05
ZASYSS9( :,16, 8) = (/ &
  & 0.76006_JPRB, 0.76006_JPRB, 0.76006_JPRB, 0.76006_JPRB, 0.77907_JPRB, 0.77978_JPRB, &
  & 0.78151_JPRB, 0.78682_JPRB, 0.78947_JPRB, 0.79748_JPRB, 0.81147_JPRB, 0.81899_JPRB /)
!-- WVL=0.1240E-05
ZASYSS9( :,17, 8) = (/ &
  & 0.74439_JPRB, 0.74439_JPRB, 0.74439_JPRB, 0.74439_JPRB, 0.76626_JPRB, 0.77251_JPRB, &
  & 0.77873_JPRB, 0.77944_JPRB, 0.78284_JPRB, 0.79467_JPRB, 0.80715_JPRB, 0.81454_JPRB /)
!-- WVL=0.1640E-05
ZASYSS9( :,18, 8) = (/ &
  & 0.74499_JPRB, 0.74499_JPRB, 0.74499_JPRB, 0.74499_JPRB, 0.75262_JPRB, 0.76058_JPRB, &
  & 0.76296_JPRB, 0.76275_JPRB, 0.78799_JPRB, 0.78887_JPRB, 0.79533_JPRB, 0.80547_JPRB /)
!-- WVL=0.2130E-05
ZASYSS9( :,19, 8) = (/ &
  & 0.68786_JPRB, 0.68786_JPRB, 0.68786_JPRB, 0.68786_JPRB, 0.74173_JPRB, 0.74050_JPRB, &
  & 0.74286_JPRB, 0.74835_JPRB, 0.77331_JPRB, 0.77936_JPRB, 0.78792_JPRB, 0.79230_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS9( :, 1, 8) = (/ &
  & 0.98012_JPRB, 0.98012_JPRB, 0.98012_JPRB, 0.98012_JPRB, 0.98250_JPRB, 0.98243_JPRB, &
  & 0.98224_JPRB, 0.98198_JPRB, 0.98153_JPRB, 0.98111_JPRB, 0.99166_JPRB, 0.99241_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS9( :, 2, 8) = (/ &
  & 0.97988_JPRB, 0.97988_JPRB, 0.97988_JPRB, 0.97988_JPRB, 0.98253_JPRB, 0.98248_JPRB, &
  & 0.98236_JPRB, 0.98217_JPRB, 0.98175_JPRB, 0.98136_JPRB, 0.99176_JPRB, 0.99255_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS9( :, 3, 8) = (/ &
  & 0.97969_JPRB, 0.97969_JPRB, 0.97969_JPRB, 0.97969_JPRB, 0.98232_JPRB, 0.98254_JPRB, &
  & 0.98248_JPRB, 0.98233_JPRB, 0.98204_JPRB, 0.98174_JPRB, 0.99194_JPRB, 0.99275_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS9( :, 4, 8) = (/ &
  & 0.97977_JPRB, 0.97977_JPRB, 0.97977_JPRB, 0.97977_JPRB, 0.98212_JPRB, 0.98242_JPRB, &
  & 0.98251_JPRB, 0.98244_JPRB, 0.98220_JPRB, 0.98195_JPRB, 0.99204_JPRB, 0.99288_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS9( :, 5, 8) = (/ &
  & 0.97998_JPRB, 0.97998_JPRB, 0.97998_JPRB, 0.97998_JPRB, 0.98121_JPRB, 0.98196_JPRB, &
  & 0.98232_JPRB, 0.98253_JPRB, 0.98244_JPRB, 0.98227_JPRB, 0.99219_JPRB, 0.99310_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS9( :, 6, 8) = (/ &
  & 0.98051_JPRB, 0.98051_JPRB, 0.98051_JPRB, 0.98051_JPRB, 0.98044_JPRB, 0.98136_JPRB, &
  & 0.98201_JPRB, 0.98237_JPRB, 0.98250_JPRB, 0.98244_JPRB, 0.99306_JPRB, 0.99320_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS9( :, 7, 8) = (/ &
  & 0.98087_JPRB, 0.98087_JPRB, 0.98087_JPRB, 0.98087_JPRB, 0.98000_JPRB, 0.98058_JPRB, &
  & 0.98140_JPRB, 0.98212_JPRB, 0.98249_JPRB, 0.98249_JPRB, 0.99339_JPRB, 0.99330_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS9( :, 8, 8) = (/ &
  & 0.98149_JPRB, 0.98149_JPRB, 0.98149_JPRB, 0.98149_JPRB, 0.97971_JPRB, 0.98008_JPRB, &
  & 0.98065_JPRB, 0.98154_JPRB, 0.98229_JPRB, 0.98442_JPRB, 0.99331_JPRB, 0.99337_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS9( :, 9, 8) = (/ &
  & 0.98195_JPRB, 0.98195_JPRB, 0.98195_JPRB, 0.98195_JPRB, 0.97969_JPRB, 0.97991_JPRB, &
  & 0.98032_JPRB, 0.98121_JPRB, 0.98215_JPRB, 0.98749_JPRB, 0.99325_JPRB, 0.99339_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS9( :,10, 8) = (/ &
  & 0.98441_JPRB, 0.98441_JPRB, 0.98442_JPRB, 0.98441_JPRB, 0.98011_JPRB, 0.97983_JPRB, &
  & 0.97970_JPRB, 0.97983_JPRB, 0.98048_JPRB, 0.99122_JPRB, 0.99256_JPRB, 0.99333_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS9( :,11, 8) = (/ &
  & 0.98506_JPRB, 0.98506_JPRB, 0.98506_JPRB, 0.98506_JPRB, 0.98044_JPRB, 0.97988_JPRB, &
  & 0.97979_JPRB, 0.97971_JPRB, 0.98017_JPRB, 0.99116_JPRB, 0.99183_JPRB, 0.99325_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS9( :,12, 8) = (/ &
  & 0.98737_JPRB, 0.98737_JPRB, 0.98737_JPRB, 0.98737_JPRB, 0.98211_JPRB, 0.98110_JPRB, &
  & 0.98065_JPRB, 0.98006_JPRB, 0.97980_JPRB, 0.99139_JPRB, 0.99122_JPRB, 0.99262_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS9( :,13, 8) = (/ &
  & 0.98799_JPRB, 0.98799_JPRB, 0.98799_JPRB, 0.98799_JPRB, 0.98320_JPRB, 0.98201_JPRB, &
  & 0.98112_JPRB, 0.98060_JPRB, 0.97988_JPRB, 0.99164_JPRB, 0.99132_JPRB, 0.99252_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS9( :,14, 8) = (/ &
  & 0.98809_JPRB, 0.98809_JPRB, 0.98809_JPRB, 0.98809_JPRB, 0.98330_JPRB, 0.98213_JPRB, &
  & 0.98121_JPRB, 0.98064_JPRB, 0.97992_JPRB, 0.99167_JPRB, 0.99134_JPRB, 0.99254_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS9( :,15, 8) = (/ &
  & 0.98981_JPRB, 0.98981_JPRB, 0.98981_JPRB, 0.98981_JPRB, 0.98599_JPRB, 0.98468_JPRB, &
  & 0.98358_JPRB, 0.98245_JPRB, 0.98109_JPRB, 0.98987_JPRB, 0.99187_JPRB, 0.99262_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS9( :,16, 8) = (/ &
  & 0.99012_JPRB, 0.99012_JPRB, 0.99012_JPRB, 0.99012_JPRB, 0.98662_JPRB, 0.98541_JPRB, &
  & 0.98423_JPRB, 0.98310_JPRB, 0.98156_JPRB, 0.98484_JPRB, 0.99207_JPRB, 0.99268_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS9( :,17, 8) = (/ &
  & 0.99093_JPRB, 0.99093_JPRB, 0.99093_JPRB, 0.99093_JPRB, 0.98804_JPRB, 0.98735_JPRB, &
  & 0.98670_JPRB, 0.98553_JPRB, 0.98385_JPRB, 0.98891_JPRB, 0.99315_JPRB, 0.99316_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS9( :,18, 8) = (/ &
  & 0.99331_JPRB, 0.99331_JPRB, 0.99331_JPRB, 0.99331_JPRB, 0.99046_JPRB, 0.99005_JPRB, &
  & 0.98942_JPRB, 0.98858_JPRB, 0.99545_JPRB, 0.99515_JPRB, 0.99472_JPRB, 0.99478_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS9( :,19, 8) = (/ &
  & 0.99455_JPRB, 0.99455_JPRB, 0.99455_JPRB, 0.99454_JPRB, 0.99231_JPRB, 0.99163_JPRB, &
  & 0.99112_JPRB, 0.99067_JPRB, 0.99628_JPRB, 0.99611_JPRB, 0.99585_JPRB, 0.99580_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS9( :, 1, 8) = (/ &
  & 16.4593_JPRB, 16.4592_JPRB, 16.4591_JPRB, 16.4592_JPRB, 17.6700_JPRB, 16.3637_JPRB, &
  & 17.1152_JPRB, 17.8147_JPRB, 18.2562_JPRB, 16.1130_JPRB,  5.6829_JPRB,  4.8619_JPRB /)
!-- WVL=0.3550E-06
ZRALIS9( :, 2, 8) = (/ &
  & 16.5836_JPRB, 16.5835_JPRB, 16.5835_JPRB, 16.5835_JPRB, 18.9749_JPRB, 16.9654_JPRB, &
  & 16.4006_JPRB, 17.4793_JPRB, 18.5680_JPRB, 17.2406_JPRB,  5.5834_JPRB,  5.0102_JPRB /)
!-- WVL=0.3800E-06
ZRALIS9( :, 3, 8) = (/ &
  & 17.9711_JPRB, 17.9710_JPRB, 17.9710_JPRB, 17.9710_JPRB, 18.0895_JPRB, 18.7855_JPRB, &
  & 16.9255_JPRB, 16.4811_JPRB, 17.6713_JPRB, 18.6487_JPRB,  6.0174_JPRB,  4.8175_JPRB /)
!-- WVL=0.4000E-06
ZRALIS9( :, 4, 8) = (/ &
  & 19.5309_JPRB, 19.5308_JPRB, 19.5308_JPRB, 19.5308_JPRB, 17.1225_JPRB, 18.7561_JPRB, &
  & 18.1312_JPRB, 16.5135_JPRB, 17.2690_JPRB, 18.0202_JPRB,  6.1738_JPRB,  4.4529_JPRB /)
!-- WVL=0.4400E-06
ZRALIS9( :, 5, 8) = (/ &
  & 19.8617_JPRB, 19.8616_JPRB, 19.8616_JPRB, 19.8615_JPRB, 16.7665_JPRB, 16.7607_JPRB, &
  & 18.0481_JPRB, 18.7157_JPRB, 16.4245_JPRB, 16.9524_JPRB,  7.2031_JPRB,  5.2027_JPRB /)
!-- WVL=0.4690E-06
ZRALIS9( :, 6, 8) = (/ &
  & 17.5766_JPRB, 17.5765_JPRB, 17.5765_JPRB, 17.5765_JPRB, 16.4212_JPRB, 16.7681_JPRB, &
  & 16.8753_JPRB, 18.3987_JPRB, 17.6513_JPRB, 16.3975_JPRB,  5.9738_JPRB,  5.7149_JPRB /)
!-- WVL=0.5000E-06
ZRALIS9( :, 7, 8) = (/ &
  & 15.4554_JPRB, 15.4553_JPRB, 15.4554_JPRB, 15.4553_JPRB, 16.5501_JPRB, 16.5248_JPRB, &
  & 16.7697_JPRB, 17.1226_JPRB, 19.0223_JPRB, 17.5109_JPRB,  6.4570_JPRB,  5.9543_JPRB /)
!-- WVL=0.5320E-06
ZRALIS9( :, 8, 8) = (/ &
  & 14.3369_JPRB, 14.3368_JPRB, 14.3369_JPRB, 14.3368_JPRB, 17.2480_JPRB, 16.5068_JPRB, &
  & 16.5149_JPRB, 16.6105_JPRB, 17.7380_JPRB, 16.7271_JPRB,  7.1238_JPRB,  5.9753_JPRB /)
!-- WVL=0.5500E-06
ZRALIS9( :, 9, 8) = (/ &
  & 14.2420_JPRB, 14.2419_JPRB, 14.2420_JPRB, 14.2419_JPRB, 18.1515_JPRB, 16.6104_JPRB, &
  & 16.4623_JPRB, 16.7666_JPRB, 17.1552_JPRB, 12.0880_JPRB,  7.1994_JPRB,  5.7843_JPRB /)
!-- WVL=0.6450E-06
ZRALIS9( :,10, 8) = (/ &
  & 13.9982_JPRB, 13.9982_JPRB, 13.9981_JPRB, 13.9982_JPRB, 19.3375_JPRB, 20.4553_JPRB, &
  & 18.3798_JPRB, 16.7653_JPRB, 16.4556_JPRB,  8.8723_JPRB,  8.0033_JPRB,  7.0388_JPRB /)
!-- WVL=0.6700E-06
ZRALIS9( :,11, 8) = (/ &
  & 13.5917_JPRB, 13.5917_JPRB, 13.5916_JPRB, 13.5917_JPRB, 17.8640_JPRB, 20.4513_JPRB, &
  & 19.7281_JPRB, 17.4205_JPRB, 16.4312_JPRB,  8.8466_JPRB,  8.3174_JPRB,  7.1854_JPRB /)
!-- WVL=0.8000E-06
ZRALIS9( :,12, 8) = (/ &
  & 12.9263_JPRB, 12.9263_JPRB, 12.9263_JPRB, 12.9263_JPRB, 14.2446_JPRB, 14.7157_JPRB, &
  & 16.5982_JPRB, 19.5598_JPRB, 19.7583_JPRB,  9.9718_JPRB, 10.0844_JPRB,  8.0687_JPRB /)
!-- WVL=0.8580E-06
ZRALIS9( :,13, 8) = (/ &
  & 13.5883_JPRB, 13.5883_JPRB, 13.5883_JPRB, 13.5883_JPRB, 14.3813_JPRB, 14.2845_JPRB, &
  & 14.7244_JPRB, 17.0848_JPRB, 20.4107_JPRB, 10.5636_JPRB, 10.2026_JPRB,  7.8721_JPRB /)
!-- WVL=0.8650E-06
ZRALIS9( :,14, 8) = (/ &
  & 13.7759_JPRB, 13.7759_JPRB, 13.7760_JPRB, 13.7760_JPRB, 14.3704_JPRB, 14.2466_JPRB, &
  & 14.5800_JPRB, 16.4159_JPRB, 20.2019_JPRB, 10.4092_JPRB, 10.2105_JPRB,  7.8584_JPRB /)
!-- WVL=0.1020E-05
ZRALIS9( :,15, 8) = (/ &
  & 19.7838_JPRB, 19.7838_JPRB, 19.7838_JPRB, 19.7838_JPRB, 13.4744_JPRB, 13.6698_JPRB, &
  & 14.2085_JPRB, 14.3384_JPRB, 14.8644_JPRB, 12.8549_JPRB, 10.4537_JPRB,  9.4701_JPRB /)
!-- WVL=0.1064E-05
ZRALIS9( :,16, 8) = (/ &
  & 21.1960_JPRB, 21.1960_JPRB, 21.1960_JPRB, 21.1960_JPRB, 13.2879_JPRB, 13.5545_JPRB, &
  & 13.9615_JPRB, 14.4754_JPRB, 14.3681_JPRB, 17.8287_JPRB, 10.5778_JPRB,  9.5622_JPRB /)
!-- WVL=0.1240E-05
ZRALIS9( :,17, 8) = (/ &
  & 20.4545_JPRB, 20.4545_JPRB, 20.4546_JPRB, 20.4546_JPRB, 13.6741_JPRB, 12.9605_JPRB, &
  & 13.1538_JPRB, 13.4893_JPRB, 14.2708_JPRB, 15.7072_JPRB, 12.6288_JPRB, 10.1165_JPRB /)
!-- WVL=0.1640E-05
ZRALIS9( :,18, 8) = (/ &
  & 11.6702_JPRB, 11.6702_JPRB, 11.6703_JPRB, 11.6703_JPRB, 21.6176_JPRB, 20.8517_JPRB, &
  & 18.1472_JPRB, 14.9175_JPRB, 14.4229_JPRB, 13.4408_JPRB, 12.8427_JPRB, 12.8767_JPRB /)
!-- WVL=0.2130E-05
ZRALIS9( :,19, 8) = (/ &
  &  6.9397_JPRB,  6.9397_JPRB,  6.9397_JPRB,  6.9398_JPRB, 14.5588_JPRB, 16.7978_JPRB, &
  & 19.0484_JPRB, 21.5228_JPRB, 14.9473_JPRB, 15.1071_JPRB, 15.0988_JPRB, 13.2696_JPRB /)
  
 ! ** BIN No.            9    10.00000000000000         20.00000000000000     
 !-- optical thickness
!-- WVL=0.3400E-06
ZALFSS9( :, 1, 9) = (/ &
  & 0.02506_JPRB, 0.02506_JPRB, 0.02506_JPRB, 0.02506_JPRB, 0.05148_JPRB, 0.05999_JPRB, &
  & 0.06880_JPRB, 0.08009_JPRB, 0.09767_JPRB, 0.11212_JPRB, 0.13757_JPRB, 0.20403_JPRB /)
!-- WVL=0.3550E-06
ZALFSS9( :, 2, 9) = (/ &
  & 0.02511_JPRB, 0.02511_JPRB, 0.02511_JPRB, 0.02511_JPRB, 0.05152_JPRB, 0.06001_JPRB, &
  & 0.06882_JPRB, 0.08013_JPRB, 0.09773_JPRB, 0.11218_JPRB, 0.13725_JPRB, 0.20391_JPRB /)
!-- WVL=0.3800E-06
ZALFSS9( :, 3, 9) = (/ &
  & 0.02514_JPRB, 0.02514_JPRB, 0.02514_JPRB, 0.02514_JPRB, 0.05165_JPRB, 0.06011_JPRB, &
  & 0.06891_JPRB, 0.08028_JPRB, 0.09783_JPRB, 0.11231_JPRB, 0.13723_JPRB, 0.20344_JPRB /)
!-- WVL=0.4000E-06
ZALFSS9( :, 4, 9) = (/ &
  & 0.02519_JPRB, 0.02519_JPRB, 0.02519_JPRB, 0.02519_JPRB, 0.05166_JPRB, 0.06018_JPRB, &
  & 0.06903_JPRB, 0.08040_JPRB, 0.09788_JPRB, 0.11234_JPRB, 0.13755_JPRB, 0.20313_JPRB /)
!-- WVL=0.4400E-06
ZALFSS9( :, 5, 9) = (/ &
  & 0.02529_JPRB, 0.02529_JPRB, 0.02529_JPRB, 0.02529_JPRB, 0.05176_JPRB, 0.06035_JPRB, &
  & 0.06920_JPRB, 0.08051_JPRB, 0.09816_JPRB, 0.11265_JPRB, 0.13842_JPRB, 0.20377_JPRB /)
!-- WVL=0.4690E-06
ZALFSS9( :, 6, 9) = (/ &
  & 0.02526_JPRB, 0.02526_JPRB, 0.02526_JPRB, 0.02526_JPRB, 0.05186_JPRB, 0.06034_JPRB, &
  & 0.06927_JPRB, 0.08068_JPRB, 0.09819_JPRB, 0.11278_JPRB, 0.13803_JPRB, 0.20467_JPRB /)
!-- WVL=0.5000E-06
ZALFSS9( :, 7, 9) = (/ &
  & 0.02540_JPRB, 0.02540_JPRB, 0.02540_JPRB, 0.02540_JPRB, 0.05201_JPRB, 0.06053_JPRB, &
  & 0.06928_JPRB, 0.08072_JPRB, 0.09828_JPRB, 0.11281_JPRB, 0.13839_JPRB, 0.20518_JPRB /)
!-- WVL=0.5320E-06
ZALFSS9( :, 8, 9) = (/ &
  & 0.02536_JPRB, 0.02536_JPRB, 0.02536_JPRB, 0.02536_JPRB, 0.05197_JPRB, 0.06061_JPRB, &
  & 0.06951_JPRB, 0.08082_JPRB, 0.09852_JPRB, 0.11304_JPRB, 0.13874_JPRB, 0.20486_JPRB /)
!-- WVL=0.5500E-06
ZALFSS9( :, 9, 9) = (/ &
  & 0.02548_JPRB, 0.02548_JPRB, 0.02548_JPRB, 0.02548_JPRB, 0.05212_JPRB, 0.06069_JPRB, &
  & 0.06946_JPRB, 0.08087_JPRB, 0.09852_JPRB, 0.11304_JPRB, 0.13881_JPRB, 0.20452_JPRB /)
!-- WVL=0.6450E-06
ZALFSS9( :,10, 9) = (/ &
  & 0.02554_JPRB, 0.02554_JPRB, 0.02554_JPRB, 0.02554_JPRB, 0.05236_JPRB, 0.06081_JPRB, &
  & 0.06990_JPRB, 0.08124_JPRB, 0.09895_JPRB, 0.11376_JPRB, 0.13933_JPRB, 0.20573_JPRB /)
!-- WVL=0.6700E-06
ZALFSS9( :,11, 9) = (/ &
  & 0.02561_JPRB, 0.02561_JPRB, 0.02561_JPRB, 0.02561_JPRB, 0.05231_JPRB, 0.06105_JPRB, &
  & 0.06990_JPRB, 0.08124_JPRB, 0.09898_JPRB, 0.11372_JPRB, 0.13941_JPRB, 0.20593_JPRB /)
!-- WVL=0.8000E-06
ZALFSS9( :,12, 9) = (/ &
  & 0.02591_JPRB, 0.02591_JPRB, 0.02591_JPRB, 0.02591_JPRB, 0.05282_JPRB, 0.06126_JPRB, &
  & 0.07028_JPRB, 0.08185_JPRB, 0.09953_JPRB, 0.11408_JPRB, 0.13995_JPRB, 0.20704_JPRB /)
!-- WVL=0.8580E-06
ZALFSS9( :,13, 9) = (/ &
  & 0.02578_JPRB, 0.02578_JPRB, 0.02578_JPRB, 0.02578_JPRB, 0.05278_JPRB, 0.06161_JPRB, &
  & 0.07034_JPRB, 0.08189_JPRB, 0.09984_JPRB, 0.11427_JPRB, 0.13982_JPRB, 0.20682_JPRB /)
!-- WVL=0.8650E-06
ZALFSS9( :,14, 9) = (/ &
  & 0.02578_JPRB, 0.02578_JPRB, 0.02578_JPRB, 0.02578_JPRB, 0.05285_JPRB, 0.06163_JPRB, &
  & 0.07031_JPRB, 0.08198_JPRB, 0.09990_JPRB, 0.11429_JPRB, 0.13982_JPRB, 0.20690_JPRB /)
!-- WVL=0.1020E-05
ZALFSS9( :,15, 9) = (/ &
  & 0.02597_JPRB, 0.02597_JPRB, 0.02597_JPRB, 0.02597_JPRB, 0.05327_JPRB, 0.06173_JPRB, &
  & 0.07099_JPRB, 0.08247_JPRB, 0.10019_JPRB, 0.11494_JPRB, 0.14104_JPRB, 0.20722_JPRB /)
!-- WVL=0.1064E-05
ZALFSS9( :,16, 9) = (/ &
  & 0.02607_JPRB, 0.02607_JPRB, 0.02607_JPRB, 0.02607_JPRB, 0.05314_JPRB, 0.06211_JPRB, &
  & 0.07093_JPRB, 0.08240_JPRB, 0.10032_JPRB, 0.11494_JPRB, 0.14078_JPRB, 0.20790_JPRB /)
!-- WVL=0.1240E-05
ZALFSS9( :,17, 9) = (/ &
  & 0.02627_JPRB, 0.02627_JPRB, 0.02627_JPRB, 0.02627_JPRB, 0.05340_JPRB, 0.06261_JPRB, &
  & 0.07120_JPRB, 0.08324_JPRB, 0.10117_JPRB, 0.11590_JPRB, 0.14191_JPRB, 0.20906_JPRB /)
!-- WVL=0.1640E-05
ZALFSS9( :,18, 9) = (/ &
  & 0.02661_JPRB, 0.02661_JPRB, 0.02661_JPRB, 0.02661_JPRB, 0.05486_JPRB, 0.06292_JPRB, &
  & 0.07236_JPRB, 0.08429_JPRB, 0.10275_JPRB, 0.11686_JPRB, 0.14364_JPRB, 0.21066_JPRB /)
!-- WVL=0.2130E-05
ZALFSS9( :,19, 9) = (/ &
  & 0.02809_JPRB, 0.02809_JPRB, 0.02809_JPRB, 0.02809_JPRB, 0.05611_JPRB, 0.06411_JPRB, &
  & 0.07274_JPRB, 0.08581_JPRB, 0.10307_JPRB, 0.11956_JPRB, 0.14406_JPRB, 0.21203_JPRB /)
  
 !-- assymetry factor
!-- WVL=0.3400E-06
ZASYSS9( :, 1, 9) = (/ &
  & 0.81387_JPRB, 0.81387_JPRB, 0.81387_JPRB, 0.81387_JPRB, 0.81825_JPRB, 0.81885_JPRB, &
  & 0.81969_JPRB, 0.82061_JPRB, 0.82156_JPRB, 0.82223_JPRB, 0.84016_JPRB, 0.84378_JPRB /)
!-- WVL=0.3550E-06
ZASYSS9( :, 2, 9) = (/ &
  & 0.81346_JPRB, 0.81346_JPRB, 0.81346_JPRB, 0.81346_JPRB, 0.81762_JPRB, 0.81854_JPRB, &
  & 0.81901_JPRB, 0.82001_JPRB, 0.82126_JPRB, 0.82175_JPRB, 0.83987_JPRB, 0.84301_JPRB /)
!-- WVL=0.3800E-06
ZASYSS9( :, 3, 9) = (/ &
  & 0.81220_JPRB, 0.81220_JPRB, 0.81220_JPRB, 0.81220_JPRB, 0.81695_JPRB, 0.81782_JPRB, &
  & 0.81857_JPRB, 0.81917_JPRB, 0.82052_JPRB, 0.82127_JPRB, 0.83982_JPRB, 0.84219_JPRB /)
!-- WVL=0.4000E-06
ZASYSS9( :, 4, 9) = (/ &
  & 0.81149_JPRB, 0.81149_JPRB, 0.81149_JPRB, 0.81149_JPRB, 0.81634_JPRB, 0.81711_JPRB, &
  & 0.81816_JPRB, 0.81886_JPRB, 0.81978_JPRB, 0.82066_JPRB, 0.83961_JPRB, 0.84219_JPRB /)
!-- WVL=0.4400E-06
ZASYSS9( :, 5, 9) = (/ &
  & 0.81020_JPRB, 0.81020_JPRB, 0.81020_JPRB, 0.81020_JPRB, 0.81491_JPRB, 0.81627_JPRB, &
  & 0.81695_JPRB, 0.81781_JPRB, 0.81886_JPRB, 0.81956_JPRB, 0.83850_JPRB, 0.84249_JPRB /)
!-- WVL=0.4690E-06
ZASYSS9( :, 6, 9) = (/ &
  & 0.80852_JPRB, 0.80852_JPRB, 0.80852_JPRB, 0.80852_JPRB, 0.81428_JPRB, 0.81501_JPRB, &
  & 0.81630_JPRB, 0.81705_JPRB, 0.81824_JPRB, 0.81885_JPRB, 0.83917_JPRB, 0.84211_JPRB /)
!-- WVL=0.5000E-06
ZASYSS9( :, 7, 9) = (/ &
  & 0.80781_JPRB, 0.80781_JPRB, 0.80781_JPRB, 0.80781_JPRB, 0.81386_JPRB, 0.81452_JPRB, &
  & 0.81511_JPRB, 0.81634_JPRB, 0.81726_JPRB, 0.81826_JPRB, 0.83939_JPRB, 0.84080_JPRB /)
!-- WVL=0.5320E-06
ZASYSS9( :, 8, 9) = (/ &
  & 0.80619_JPRB, 0.80619_JPRB, 0.80619_JPRB, 0.80619_JPRB, 0.81240_JPRB, 0.81385_JPRB, &
  & 0.81465_JPRB, 0.81548_JPRB, 0.81676_JPRB, 0.82176_JPRB, 0.83812_JPRB, 0.83984_JPRB /)
!-- WVL=0.5500E-06
ZASYSS9( :, 9, 9) = (/ &
  & 0.80623_JPRB, 0.80622_JPRB, 0.80622_JPRB, 0.80622_JPRB, 0.81216_JPRB, 0.81356_JPRB, &
  & 0.81402_JPRB, 0.81491_JPRB, 0.81633_JPRB, 0.82777_JPRB, 0.83761_JPRB, 0.83961_JPRB /)
!-- WVL=0.6450E-06
ZASYSS9( :,10, 9) = (/ &
  & 0.80270_JPRB, 0.80270_JPRB, 0.80270_JPRB, 0.80270_JPRB, 0.80975_JPRB, 0.81056_JPRB, &
  & 0.81218_JPRB, 0.81318_JPRB, 0.81435_JPRB, 0.83290_JPRB, 0.83651_JPRB, 0.83820_JPRB /)
!-- WVL=0.6700E-06
ZASYSS9( :,11, 9) = (/ &
  & 0.80223_JPRB, 0.80223_JPRB, 0.80223_JPRB, 0.80223_JPRB, 0.80871_JPRB, 0.81048_JPRB, &
  & 0.81132_JPRB, 0.81229_JPRB, 0.81386_JPRB, 0.83247_JPRB, 0.83503_JPRB, 0.83761_JPRB /)
!-- WVL=0.8000E-06
ZASYSS9( :,12, 9) = (/ &
  & 0.79855_JPRB, 0.79855_JPRB, 0.79855_JPRB, 0.79855_JPRB, 0.80611_JPRB, 0.80678_JPRB, &
  & 0.80832_JPRB, 0.80992_JPRB, 0.81130_JPRB, 0.82995_JPRB, 0.83152_JPRB, 0.83624_JPRB /)
!-- WVL=0.8580E-06
ZASYSS9( :,13, 9) = (/ &
  & 0.79452_JPRB, 0.79452_JPRB, 0.79452_JPRB, 0.79452_JPRB, 0.80429_JPRB, 0.80623_JPRB, &
  & 0.80676_JPRB, 0.80840_JPRB, 0.81048_JPRB, 0.82845_JPRB, 0.82992_JPRB, 0.83491_JPRB /)
!-- WVL=0.8650E-06
ZASYSS9( :,14, 9) = (/ &
  & 0.79431_JPRB, 0.79431_JPRB, 0.79431_JPRB, 0.79431_JPRB, 0.80431_JPRB, 0.80606_JPRB, &
  & 0.80649_JPRB, 0.80834_JPRB, 0.81041_JPRB, 0.82825_JPRB, 0.82987_JPRB, 0.83492_JPRB /)
!-- WVL=0.1020E-05
ZASYSS9( :,15, 9) = (/ &
  & 0.78868_JPRB, 0.78868_JPRB, 0.78868_JPRB, 0.78868_JPRB, 0.80130_JPRB, 0.80229_JPRB, &
  & 0.80443_JPRB, 0.80540_JPRB, 0.80688_JPRB, 0.82243_JPRB, 0.82792_JPRB, 0.83272_JPRB /)
!-- WVL=0.1064E-05
ZASYSS9( :,16, 9) = (/ &
  & 0.78703_JPRB, 0.78703_JPRB, 0.78703_JPRB, 0.78703_JPRB, 0.79940_JPRB, 0.80227_JPRB, &
  & 0.80312_JPRB, 0.80423_JPRB, 0.80618_JPRB, 0.81344_JPRB, 0.82661_JPRB, 0.83264_JPRB /)
!-- WVL=0.1240E-05
ZASYSS9( :,17, 9) = (/ &
  & 0.78067_JPRB, 0.78067_JPRB, 0.78067_JPRB, 0.78067_JPRB, 0.79446_JPRB, 0.79867_JPRB, &
  & 0.79921_JPRB, 0.80218_JPRB, 0.80407_JPRB, 0.81471_JPRB, 0.82455_JPRB, 0.82984_JPRB /)
!-- WVL=0.1640E-05
ZASYSS9( :,18, 9) = (/ &
  & 0.76646_JPRB, 0.76646_JPRB, 0.76646_JPRB, 0.76646_JPRB, 0.78689_JPRB, 0.78730_JPRB, &
  & 0.79101_JPRB, 0.79429_JPRB, 0.81451_JPRB, 0.81509_JPRB, 0.81879_JPRB, 0.82462_JPRB /)
!-- WVL=0.2130E-05
ZASYSS9( :,19, 9) = (/ &
  & 0.75797_JPRB, 0.75797_JPRB, 0.75797_JPRB, 0.75797_JPRB, 0.77768_JPRB, 0.77782_JPRB, &
  & 0.77850_JPRB, 0.78462_JPRB, 0.80385_JPRB, 0.80840_JPRB, 0.80920_JPRB, 0.81740_JPRB /)
  
 !-- single scattering albedo
!-- WVL=0.3400E-06
ZOMGSS9( :, 1, 9) = (/ &
  & 0.98178_JPRB, 0.98179_JPRB, 0.98179_JPRB, 0.98179_JPRB, 0.97925_JPRB, 0.97854_JPRB, &
  & 0.97782_JPRB, 0.97690_JPRB, 0.97565_JPRB, 0.97474_JPRB, 0.98870_JPRB, 0.98922_JPRB /)
!-- WVL=0.3550E-06
ZOMGSS9( :, 2, 9) = (/ &
  & 0.98198_JPRB, 0.98198_JPRB, 0.98198_JPRB, 0.98198_JPRB, 0.97962_JPRB, 0.97896_JPRB, &
  & 0.97826_JPRB, 0.97742_JPRB, 0.97620_JPRB, 0.97530_JPRB, 0.98898_JPRB, 0.98950_JPRB /)
!-- WVL=0.3800E-06
ZOMGSS9( :, 3, 9) = (/ &
  & 0.98223_JPRB, 0.98223_JPRB, 0.98223_JPRB, 0.98223_JPRB, 0.98021_JPRB, 0.97955_JPRB, &
  & 0.97894_JPRB, 0.97818_JPRB, 0.97705_JPRB, 0.97618_JPRB, 0.98939_JPRB, 0.98994_JPRB /)
!-- WVL=0.4000E-06
ZOMGSS9( :, 4, 9) = (/ &
  & 0.98239_JPRB, 0.98239_JPRB, 0.98239_JPRB, 0.98239_JPRB, 0.98058_JPRB, 0.97999_JPRB, &
  & 0.97941_JPRB, 0.97872_JPRB, 0.97766_JPRB, 0.97681_JPRB, 0.98970_JPRB, 0.99027_JPRB /)
!-- WVL=0.4400E-06
ZOMGSS9( :, 5, 9) = (/ &
  & 0.98255_JPRB, 0.98255_JPRB, 0.98255_JPRB, 0.98255_JPRB, 0.98116_JPRB, 0.98072_JPRB, &
  & 0.98021_JPRB, 0.97956_JPRB, 0.97867_JPRB, 0.97795_JPRB, 0.99019_JPRB, 0.99080_JPRB /)
!-- WVL=0.4690E-06
ZOMGSS9( :, 6, 9) = (/ &
  & 0.98252_JPRB, 0.98252_JPRB, 0.98252_JPRB, 0.98252_JPRB, 0.98156_JPRB, 0.98107_JPRB, &
  & 0.98068_JPRB, 0.98010_JPRB, 0.97924_JPRB, 0.97861_JPRB, 0.99157_JPRB, 0.99114_JPRB /)
!-- WVL=0.5000E-06
ZOMGSS9( :, 7, 9) = (/ &
  & 0.98248_JPRB, 0.98248_JPRB, 0.98248_JPRB, 0.98248_JPRB, 0.98191_JPRB, 0.98149_JPRB, &
  & 0.98105_JPRB, 0.98058_JPRB, 0.97979_JPRB, 0.97918_JPRB, 0.99222_JPRB, 0.99141_JPRB /)
!-- WVL=0.5320E-06
ZOMGSS9( :, 8, 9) = (/ &
  & 0.98216_JPRB, 0.98216_JPRB, 0.98216_JPRB, 0.98216_JPRB, 0.98210_JPRB, 0.98182_JPRB, &
  & 0.98146_JPRB, 0.98098_JPRB, 0.98030_JPRB, 0.98222_JPRB, 0.99242_JPRB, 0.99168_JPRB /)
!-- WVL=0.5500E-06
ZOMGSS9( :, 9, 9) = (/ &
  & 0.98198_JPRB, 0.98198_JPRB, 0.98198_JPRB, 0.98198_JPRB, 0.98226_JPRB, 0.98198_JPRB, &
  & 0.98161_JPRB, 0.98116_JPRB, 0.98054_JPRB, 0.98647_JPRB, 0.99254_JPRB, 0.99183_JPRB /)
!-- WVL=0.6450E-06
ZOMGSS9( :,10, 9) = (/ &
  & 0.97984_JPRB, 0.97984_JPRB, 0.97984_JPRB, 0.97984_JPRB, 0.98255_JPRB, 0.98243_JPRB, &
  & 0.98232_JPRB, 0.98201_JPRB, 0.98153_JPRB, 0.99192_JPRB, 0.99284_JPRB, 0.99240_JPRB /)
!-- WVL=0.6700E-06
ZOMGSS9( :,11, 9) = (/ &
  & 0.97950_JPRB, 0.97950_JPRB, 0.97950_JPRB, 0.97950_JPRB, 0.98254_JPRB, 0.98253_JPRB, &
  & 0.98239_JPRB, 0.98213_JPRB, 0.98172_JPRB, 0.99202_JPRB, 0.99228_JPRB, 0.99253_JPRB /)
!-- WVL=0.8000E-06
ZOMGSS9( :,12, 9) = (/ &
  & 0.97964_JPRB, 0.97964_JPRB, 0.97963_JPRB, 0.97964_JPRB, 0.98190_JPRB, 0.98234_JPRB, &
  & 0.98254_JPRB, 0.98255_JPRB, 0.98238_JPRB, 0.99227_JPRB, 0.99215_JPRB, 0.99301_JPRB /)
!-- WVL=0.8580E-06
ZOMGSS9( :,13, 9) = (/ &
  & 0.97999_JPRB, 0.97999_JPRB, 0.97998_JPRB, 0.97998_JPRB, 0.98098_JPRB, 0.98195_JPRB, &
  & 0.98234_JPRB, 0.98254_JPRB, 0.98253_JPRB, 0.99225_JPRB, 0.99222_JPRB, 0.99316_JPRB /)
!-- WVL=0.8650E-06
ZOMGSS9( :,14, 9) = (/ &
  & 0.98005_JPRB, 0.98005_JPRB, 0.98005_JPRB, 0.98005_JPRB, 0.98089_JPRB, 0.98189_JPRB, &
  & 0.98228_JPRB, 0.98254_JPRB, 0.98255_JPRB, 0.99225_JPRB, 0.99223_JPRB, 0.99317_JPRB /)
!-- WVL=0.1020E-05
ZOMGSS9( :,15, 9) = (/ &
  & 0.98181_JPRB, 0.98181_JPRB, 0.98181_JPRB, 0.98181_JPRB, 0.97930_JPRB, 0.97965_JPRB, &
  & 0.98064_JPRB, 0.98166_JPRB, 0.98236_JPRB, 0.98967_JPRB, 0.99223_JPRB, 0.99339_JPRB /)
!-- WVL=0.1064E-05
ZOMGSS9( :,16, 9) = (/ &
  & 0.98239_JPRB, 0.98239_JPRB, 0.98239_JPRB, 0.98239_JPRB, 0.97919_JPRB, 0.97941_JPRB, &
  & 0.97998_JPRB, 0.98109_JPRB, 0.98214_JPRB, 0.98532_JPRB, 0.99211_JPRB, 0.99343_JPRB /)
!-- WVL=0.1240E-05
ZOMGSS9( :,17, 9) = (/ &
  & 0.98461_JPRB, 0.98461_JPRB, 0.98461_JPRB, 0.98461_JPRB, 0.98003_JPRB, 0.97964_JPRB, &
  & 0.97922_JPRB, 0.97938_JPRB, 0.98036_JPRB, 0.98574_JPRB, 0.99139_JPRB, 0.99337_JPRB /)
!-- WVL=0.1640E-05
ZOMGSS9( :,18, 9) = (/ &
  & 0.98808_JPRB, 0.98808_JPRB, 0.98808_JPRB, 0.98808_JPRB, 0.98352_JPRB, 0.98224_JPRB, &
  & 0.98150_JPRB, 0.98071_JPRB, 0.99206_JPRB, 0.99166_JPRB, 0.99130_JPRB, 0.99236_JPRB /)
!-- WVL=0.2130E-05
ZOMGSS9( :,19, 9) = (/ &
  & 0.99058_JPRB, 0.99058_JPRB, 0.99058_JPRB, 0.99058_JPRB, 0.98731_JPRB, 0.98616_JPRB, &
  & 0.98503_JPRB, 0.98406_JPRB, 0.99364_JPRB, 0.99324_JPRB, 0.99246_JPRB, 0.99284_JPRB /)
  
 !-- lidar ratio
!-- WVL=0.3400E-06
ZRALIS9( :, 1, 9) = (/ &
  & 18.1044_JPRB, 18.1044_JPRB, 18.1043_JPRB, 18.1044_JPRB, 17.0093_JPRB, 16.6500_JPRB, &
  & 15.3048_JPRB, 15.8109_JPRB, 14.4806_JPRB, 14.7178_JPRB,  4.6784_JPRB,  2.6318_JPRB /)
!-- WVL=0.3550E-06
ZRALIS9( :, 2, 9) = (/ &
  & 19.0292_JPRB, 19.0291_JPRB, 19.0291_JPRB, 19.0291_JPRB, 16.1290_JPRB, 17.0383_JPRB, &
  & 15.9527_JPRB, 15.6500_JPRB, 14.9817_JPRB, 14.6921_JPRB,  5.4413_JPRB,  2.8717_JPRB /)
!-- WVL=0.3800E-06
ZRALIS9( :, 3, 9) = (/ &
  & 17.9580_JPRB, 17.9579_JPRB, 17.9579_JPRB, 17.9579_JPRB, 18.2353_JPRB, 16.2947_JPRB, &
  & 17.1199_JPRB, 15.6735_JPRB, 15.8842_JPRB, 14.9238_JPRB,  5.5824_JPRB,  3.6356_JPRB /)
!-- WVL=0.4000E-06
ZRALIS9( :, 4, 9) = (/ &
  & 17.0158_JPRB, 17.0158_JPRB, 17.0158_JPRB, 17.0158_JPRB, 19.2854_JPRB, 16.8798_JPRB, &
  & 16.6359_JPRB, 16.9040_JPRB, 15.3799_JPRB, 15.7007_JPRB,  5.2133_JPRB,  4.2757_JPRB /)
!-- WVL=0.4400E-06
ZRALIS9( :, 5, 9) = (/ &
  & 16.1756_JPRB, 16.1756_JPRB, 16.1756_JPRB, 16.1755_JPRB, 15.6307_JPRB, 18.6115_JPRB, &
  & 18.2701_JPRB, 16.2926_JPRB, 16.9206_JPRB, 15.2635_JPRB,  4.6720_JPRB,  4.1700_JPRB /)
!-- WVL=0.4690E-06
ZRALIS9( :, 6, 9) = (/ &
  & 17.9939_JPRB, 17.9937_JPRB, 17.9938_JPRB, 17.9937_JPRB, 16.5444_JPRB, 15.7924_JPRB, &
  & 18.8627_JPRB, 17.4809_JPRB, 16.8694_JPRB, 16.7698_JPRB,  4.5462_JPRB,  3.4542_JPRB /)
!-- WVL=0.5000E-06
ZRALIS9( :, 7, 9) = (/ &
  & 19.6769_JPRB, 19.6767_JPRB, 19.6768_JPRB, 19.6767_JPRB, 18.7847_JPRB, 16.1131_JPRB, &
  & 15.7704_JPRB, 19.2855_JPRB, 16.1624_JPRB, 16.8747_JPRB,  4.1665_JPRB,  3.4419_JPRB /)
!-- WVL=0.5320E-06
ZRALIS9( :, 8, 9) = (/ &
  & 18.3758_JPRB, 18.3755_JPRB, 18.3757_JPRB, 18.3755_JPRB, 18.4000_JPRB, 18.4016_JPRB, &
  & 15.9049_JPRB, 16.4049_JPRB, 18.8367_JPRB, 14.8826_JPRB,  4.2973_JPRB,  3.8253_JPRB /)
!-- WVL=0.5500E-06
ZRALIS9( :, 9, 9) = (/ &
  & 17.4281_JPRB, 17.4279_JPRB, 17.4280_JPRB, 17.4279_JPRB, 17.7901_JPRB, 18.9968_JPRB, &
  & 16.9496_JPRB, 15.6308_JPRB, 19.1472_JPRB, 12.7320_JPRB,  4.6046_JPRB,  4.1047_JPRB /)
!-- WVL=0.6450E-06
ZRALIS9( :,10, 9) = (/ &
  & 15.7858_JPRB, 15.7859_JPRB, 15.7857_JPRB, 15.7859_JPRB, 16.7655_JPRB, 16.1445_JPRB, &
  & 17.8525_JPRB, 18.8818_JPRB, 16.3591_JPRB,  5.8950_JPRB,  4.7028_JPRB,  4.3000_JPRB /)
!-- WVL=0.6700E-06
ZRALIS9( :,11, 9) = (/ &
  & 16.1677_JPRB, 16.1678_JPRB, 16.1676_JPRB, 16.1678_JPRB, 17.8060_JPRB, 15.8728_JPRB, &
  & 16.9029_JPRB, 18.3806_JPRB, 17.8431_JPRB,  6.3297_JPRB,  5.2335_JPRB,  4.6057_JPRB /)
!-- WVL=0.8000E-06
ZRALIS9( :,12, 9) = (/ &
  & 20.7441_JPRB, 20.7442_JPRB, 20.7442_JPRB, 20.7442_JPRB, 17.4571_JPRB, 19.3037_JPRB, &
  & 18.6826_JPRB, 16.4418_JPRB, 16.8515_JPRB,  7.3797_JPRB,  6.3331_JPRB,  4.6096_JPRB /)
!-- WVL=0.8580E-06
ZRALIS9( :,13, 9) = (/ &
  & 21.2290_JPRB, 21.2290_JPRB, 21.2291_JPRB, 21.2291_JPRB, 15.9107_JPRB, 17.3829_JPRB, &
  & 19.3057_JPRB, 18.5380_JPRB, 15.8790_JPRB,  6.8062_JPRB,  7.1127_JPRB,  5.3011_JPRB /)
!-- WVL=0.8650E-06
ZRALIS9( :,14, 9) = (/ &
  & 20.7617_JPRB, 20.7617_JPRB, 20.7618_JPRB, 20.7618_JPRB, 15.8010_JPRB, 17.3628_JPRB, &
  & 19.2973_JPRB, 18.8336_JPRB, 16.0394_JPRB,  6.8401_JPRB,  7.2011_JPRB,  5.3997_JPRB /)
!-- WVL=0.1020E-05
ZRALIS9( :,15, 9) = (/ &
  & 14.2151_JPRB, 14.2152_JPRB, 14.2152_JPRB, 14.2152_JPRB, 16.9608_JPRB, 15.8953_JPRB, &
  & 15.7415_JPRB, 16.5740_JPRB, 19.3891_JPRB, 10.0006_JPRB,  7.1969_JPRB,  6.1634_JPRB /)
!-- WVL=0.1064E-05
ZRALIS9( :,16, 9) = (/ &
  & 13.4323_JPRB, 13.4324_JPRB, 13.4324_JPRB, 13.4324_JPRB, 17.8398_JPRB, 16.5289_JPRB, &
  & 15.7175_JPRB, 15.8364_JPRB, 18.2578_JPRB, 15.7621_JPRB,  7.7502_JPRB,  6.2843_JPRB /)
!-- WVL=0.1240E-05
ZRALIS9( :,17, 9) = (/ &
  & 13.1431_JPRB, 13.1431_JPRB, 13.1432_JPRB, 13.1432_JPRB, 21.0713_JPRB, 20.6264_JPRB, &
  & 18.0748_JPRB, 16.5048_JPRB, 15.6552_JPRB, 15.6292_JPRB,  8.2037_JPRB,  6.7394_JPRB /)
!-- WVL=0.1640E-05
ZRALIS9( :,18, 9) = (/ &
  & 13.9062_JPRB, 13.9062_JPRB, 13.9063_JPRB, 13.9063_JPRB, 13.2768_JPRB, 13.5605_JPRB, &
  & 15.0867_JPRB, 18.9720_JPRB, 10.4129_JPRB, 10.4321_JPRB, 10.9283_JPRB,  8.1066_JPRB /)
!-- WVL=0.2130E-05
ZRALIS9( :,19, 9) = (/ &
  & 24.1593_JPRB, 24.1593_JPRB, 24.1595_JPRB, 24.1596_JPRB, 13.2860_JPRB, 13.0951_JPRB, &
  & 12.9878_JPRB, 13.2810_JPRB, 13.5070_JPRB, 12.3613_JPRB, 10.6386_JPRB, 10.1305_JPRB /)

IF (NTYPAER(1) /= 9) THEN
  ALF_SS(1:12,1:19,1:3) = ZALFSS3(1:12,1:19,1:3)
  ASY_SS(1:12,1:19,1:3) = ZASYSS3(1:12,1:19,1:3)
  OMG_SS(1:12,1:19,1:3) = ZOMGSS3(1:12,1:19,1:3)
  RALI_SS(1:12,1:19,1:3)= ZRALIS3(1:12,1:19,1:3)
ELSE
  ALF_SS(1:12,1:19,1:9) = ZALFSS9(1:12,1:19,1:9)
  ASY_SS(1:12,1:19,1:9) = ZASYSS9(1:12,1:19,1:9)
  OMG_SS(1:12,1:19,1:9) = ZOMGSS9(1:12,1:19,1:9)
  RALI_SS(1:12,1:19,1:9)= ZRALIS9(1:12,1:19,1:9)
ENDIF

!     ----------------------------------------------------------------  

!*      1.    PARAMETERS SPECIFIC TO THE LIDAR SIMULATOR FOR AEROSOLS
!             -------------------------------------------------------


! Rayleigh |  355nm / 28169cm-1 |  532nm / 18797cm-1 | 1064nm / 9398.5cm-1 |
! CO2      |         no         |         no         |        yes          |
! O2       |         no         |         no         |        yes          |
! O3       |        yes         |        yes         |        yes          |
! NO2      |        yes         |        yes         |	      yes          |


!- Lidar wavelengths to be simulated
NWLID = 3
RWLID = (/ 0.355E-06, 0.532E-06, 1.064E-06 /)
JWLID = (/    2     ,    8     ,   16      /) 

!- Absorption coefficients (for Goody/Malkmus statistical models of transmission function)
!-- CO2 and O2 at 1064 nm
RLICO2 = (/ &
  & 0.21829E-03_JPRB, 0.27491E+00_JPRB,-0.34678E-02_JPRB, 0.82722E-06_JPRB,-0.64858E-02_JPRB, 0.70242E-05_JPRB /)
RLIO2  = (/ &
  & 0.16816E-05_JPRB, 0.13899E+00_JPRB, 0.35668E-02_JPRB,-0.12886E-04_JPRB, 0.15543E-02_JPRB,-0.94945E-05_JPRB /)

!-- NO2 absorption cross-sections (cm2/molec) K. Bogumil, J. Orphal, and J. P. Burrows
!   at 203, 223, 243, 273 and 293K    for 335.0041, 532.0856 nm 
RLINO2(:,1) = (/ &   
  & 3.910908E-19_JPRB, 3.780920E-19_JPRB, 3.639214E-19_JPRB, 3.703919E-19_JPRB, 3.704435E-19_JPRB /)
RLINO2(:,2) = (/ &
  & 1.454774E-19_JPRB, 1.454790E-19_JPRB, 1.494360E-19_JPRB, 1.521450E-19_JPRB, 1.554439E-19_JPRB /)
RLINO2(:,3) = (/ &
  & 0.0_JPRB         , 0.0_JPRB         , 0.0_JPRB         , 0.0_JPRB         , 0.0_JPRB          /) 

!-- O3 absorption cross-sections (cm2/molec) K. Bogumil, J. Orphal, and J. P. Burrows
!   at 203, 223, 243, 273 and 293K    for 335.0041, 532.0856, and 1063.9788 nm
RLIO3(:,1) = (/ &
  & 1.27033E-21_JPRB, 1.41847E-21_JPRB, 6.43084E-23_JPRB, 2.11856E-21_JPRB, 1.18642E-22_JPRB /)   
RLIO3(:,2) = (/ &
  & 2.84646E-21_JPRB, 2.80979E-21_JPRB, 2.81856E-21_JPRB, 2.85269E-21_JPRB, 2.82891E-21_JPRB /)
RLIO3(:,3) = (/ &
  & 1.15032E-24_JPRB, 7.57913E-25_JPRB, 4.00344E-25_JPRB, 8.67150E-25_JPRB, 1.44082E-24_JPRB /) 

RLICLS= 1.0_JPRB   
RLIDELT=0.0279_JPRB
!-- RLINS: molecular density of a gas at 288.15K and 101325 Pa  (molecules cm-3)
RLINS = 2.54743E+19_JPRB
RLIPREF=101325._JPRB
RLITREF=288.15_JPRB
RLIT0 = 250._JPRB

!     ----------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SU_AEROP',1,ZHOOK_HANDLE)
END SUBROUTINE SU_AEROP
